#!/bin/csh
#
# FUNCTION:	adapsbench.csh
#
# PURPOSE:	Generate a temporary PDF that will allow a user to run a LAS
#		function as a cron job.  The temporary PDF will be created in
#		the specified directory and the LAS process will be run from
#		that directory as well.


#--------------------
# Check the arguments
#--------------------
if (($#argv != 1) && ($#argv != 2)) then
    echo 'usage: adapsbench.csh workingdir [rundir]'
    echo ''
    echo '       Create a temporary PDF that will allow a user to run the ADAPS'
    echo '       benchmark as a cron job.  The temporary PDF will be created in'
    echo '       workingdir and the LAS process will be run from that directory'
    echo '       as well.'
    echo ''
    echo '       If [rundir] is specified, a new environment pointing to that'
    echo '       run directory will be created.  If [rundir] is not specified,'
    echo '       the environment will not be altered prior to executing the'
    echo '       benchmark.'
    echo ''
    echo '       The temporary PDF will be removed as soon as the LAS process'
    echo '       begins.'
    exit 1
endif

#------------------------------------
# Change to the working directory
#------------------------------------
cd $1


#-------------------------------------------------------------------------
# Set up the production LAS environment,
# if is hasn't already been setup.
#-------------------------------------------------------------------------
if ((! $?EDCSOFT) && (-r /etc/login)) then
    source /etc/login
    eval `$masterlog response comfile las72`
    unset masterlog
    unset response
    unset comfile
endif


#----------------------------------------------------------------------------
# If the user has specified an optional environment to run from, reconfigure
# the production LAS environment to point to the new environment.
#----------------------------------------------------------------------------
if ($#argv == 2) then

    # Remember the following production settings
    set tmplogdir = $ADAPSLOG
    set tmptabdir = $ADAPSTABLES
    set tmpddpdir = $LASDDP

    # Reconfigure LAS to point to the new environment
    setenv EDCSOFT $2
    source $2/env/las/assign/lasenv.csh $2

    # Reassign the following production settings
    setenv ADAPSLOG $tmplogdir
    setenv ADAPSTABLES $tmptabdir
    setenv LASDDP $tmpddpdir

    # Hardwire the production log directory
    setenv ADAPSLOG "/sgs20/km1/adaps/log"

endif


#------------------------------
# Create a unique log file name
#------------------------------
set tstamp = `date '+%H%M%S'`


#-------------------------
# Generate a temporary PDF
#-------------------------
cat << EOF > bmark_$tstamp.pdf
procedure bmark_$tstamp

body
!-----------------------------------------------------------------
! Submit into batch queue.  The first "batch" means a batch job.
! The second occurance indicates the name of the queue to be used.
!-----------------------------------------------------------------
adapsbench  |runtype=(batch,ageneral) stdout="bmark_$tstamp.log"| +
    testdir="/sgs20/km1/adaps/benchmark/" +
    srcdir="/sgs20/km1/adaps/benchmark/adaps/"
exit
end-proc
EOF


#--------------------------------------------
# Start LAS and submit the PDF as a batch job
#--------------------------------------------
cat << EOJ | tae >& bmark_$tstamp.stdout
bmark_$tstamp
EOJ


#-------------------------
# Remove the temporary PDF
#-------------------------
rm -f bmark_$tstamp.pdf

exit 0
