#!/bin/csh -f
#
# FUNCTION:  cklsu.csh
#
# PURPOSE:   Check the images sent to us from lsu
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# J.Fenno          05/92  Development
# J.Fenno          10/92  Rewritten to check the LSU images and move them right
#                         into the ADAPSACQ directory.  This will skip the
#                         process of copying the images to tape.
# J.Fenno          10/92  Changed wrtlog key for operators
# J.Fenno          11/92  Changed the wrtlog message for cklsu-sum.
#                         Added code that will find the paths for the four
#                         commands checkspace, lsfiles, get_num.csh, and
#                         write_log in the pdf and then pass them as arguments.
# J.Fenno          12/92  Combined into one pdf called cktrans this then calls
#                         ckbaa.csh or cklsu.csh.  Modified so the directory
#                         is not hard coded but passed as an argument.
# J.Fenno          01/93  Added check so that multiple images are not renamed
#                         to the same acquire image name.
# J.Fenno          03/94  Added code to create a temporary pdf which will
#                         return the number of images checked.
# 
#
#


#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
onintr -
set dstdir = $ADAPSACQ
set nbytes = 500000000
set printflag = "0"
set noglob
unalias cd
unalias df
unalias du
unalias ls
unalias rm


#-----------------------------------------------------------------------------
# Ensure there are five arguments.  These are the directory where the images
# are located and the paths for the commands checkspace, lsfiles, get_num.csh,
# and write_log respectively.
#-----------------------------------------------------------------------------
if ($#argv != 6) then
    echo "[cktrans-args] Must have six arguments for cklsu.csh"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#-----------------------------------------------------------------------------
# The calling pdf expects a temporary pdf called $tmppdf that will return the
# number of images that were checked.  Create this pdf so that it returns zero
# in the event of an error.
#-----------------------------------------------------------------------------
set tmppdf = `pwd`/nlsuchecked$6.pdf

echo "procedure"                >  $tmppdf
echo "parm num type=name"   >> $tmppdf
echo "body"                     >> $tmppdf
echo "let num = 0"          >> $tmppdf
echo "ush rm -f $tmppdf"        >> $tmppdf
echo "end-proc"                 >> $tmppdf


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we will be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------
set srcdir = $1
if ("$2" == ".") then
    alias ckspace `pwd`/checkspace
else
    alias ckspace $2/checkspace
endif
if ("$3" == ".") then
    alias lsfiles `pwd`/lsfiles
else
    alias lsfiles $3/lsfiles
endif
if ("$4" == ".") then
    alias get_num.csh `pwd`/get_num.csh
else
    alias get_num.csh $4/get_num.csh
endif
if ("$5" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $5/write_log
endif

#- $6 is the unique session number -#

#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSACQ) || (! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[cktrans-env] ADAPSACQ, ADAPSLOG, or ADAPSERR is not set"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#--------------------------------------------------------
# Create a log file.  This must be a LAS name for wrtlog.
#--------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]cktrans" | tr / . | sed  's/\[\./\[/'`


#------------------------------------------------------------------------------
# Check to see if there are any images pushed from LSU.  These images will be
# named like n11.921006.2023.acq.  If none exist then issue a message and exit.
#------------------------------------------------------------------------------
cd $srcdir
set images = `lsfiles '^n.*\.acq\.gz$'`
if ("$images" == "") then
    #--------------------------------------------------------------------
    # if there aren't any files we don't want any messages to the logfile
    # so we don't fill up the log file
    #--------------------------------------------------------------------
    tty -s
    if ($status == 0) then
	echo "[informational] No LSU images"
    endif
    goto NO_FILES
else
    set text = "Starting to check LSU images"
    echo "[informational] $text"

    #-------------------------------------------------------------------------
    # Recreate the temporary pdf that will return the number of images checked
    #-------------------------------------------------------------------------
    echo "procedure"                >  $tmppdf
    echo "parm num type=name"   >> $tmppdf
    echo "body"                     >> $tmppdf
    echo "let num = $#images"   >> $tmppdf
    echo "ush rm -f $tmppdf"        >> $tmppdf
    echo "end-proc"                 >> $tmppdf
endif


#------------------------------------------------------------------------------
# For each image pushed from LSU check to see if there is a corresponding check
# sum file.  If there is process the file, otherwise, issue an error message.
#------------------------------------------------------------------------------
foreach image ($images)
    onintr  ERROUT
    set filenum = `get_num.csh $srcdir`
    set key     = "netlsu-$filenum-lsu"
    set cksum   = `echo $image | sed -e 's/acq.gz$/sum/'`
    if ((! -e $cksum) || (-z $cksum)) then
	#seeing if the file is over a day old
	# (not have the check sum file and over
	# a day old means that the file needs to
	# be resent)
	#if $test = 0 the file is not over a day old
	#if $test = 1 the file is over a day old
	set test = 0
	set test = `find $srcdir -name $image -mtime +1 -exec "echo" 1 \;`
	if ($test) then
	    mv $srcdir/$image $ADAPSERR/$image
	    set text = "No check sum file for $image - moved to ADAPSERR"
	    wrtlog $logfile $key cklsu-error $text
	    echo "[cklsu-sum] $text"
	endif
	continue
    endif


    #--------------------------------------------------------------------------
    # Get the remote and local check sums.  Just use the first 5 characters of
    # the string returned by the sum command.  When working with large files
    # the check sum value and the number of blocks merge into a 10 digit number
    # and we are only interested in the first 5 digits.  (Block sizes are
    # different on different machines)
    #
    # LSU is now using "sum" instead of "sum -r"
    #--------------------------------------------------------------------------
    set remote_sum = `cat $cksum`
    set remote_sum = `echo $remote_sum | awk '{printf "%5.5s",$1}'`
    set local_sum  = `sum $image`
    set local_sum  = `echo $local_sum  | awk '{printf "%5.5s",$1}'`


    #--------------------------------------
    # Determine if the check sums are equal
    #--------------------------------------
    onintr -
    if ($local_sum != $remote_sum) then
	mv $image $ADAPSERR/$image
	mv $cksum $ADAPSERR/$cksum
	set text = "Check sums do not agree for $image - Moved to ADAPSERR"
	wrtlog $logfile $key cklsu-error $text
	echo "[cklsu-unequal] $text"
	set text = "Local sum = $local_sum, remote sum = $remote_sum"
	wrtlog $logfile $key cklsu-error $text
	echo "[cklsu-unequal] $text"
    else
	#----------------------------------------------------
	# Uncompress and rename the image using the gunzip
	# routine.  Strip the .gz extension from the image
	# variable.  Save compressed image name for uncompress
	# check.
	#----------------------------------------------------
	set cimg = $image
	set image = `echo $image | awk '{printf "%-19.19s",$1}'`
	gunzip  $cimg
	if ( $status ) then
	    #----------------------------------------------------------------
	    # Occasionally the LSU files will not unzip properly returning an
	    # unexpected end of file error.  If the normal unzip fails unzip
	    # it directly to a file.
	    #----------------------------------------------------------------
	    set text = "Gunzip status returned error for $cimg"
	    echo "[cklsu-uncompress] $text"
	    wrtlog $logfile $key [cklsu-uncompress] $text
	    gunzip -c $cimg > $image

	    # this is a temporary image copy for diagnostics
	    #-----------------------------------------------
	    cp $cimg /sgs4/km1/test/error/$cimg
	endif

	#----------------------------------------------------
	# Make sure the file was uncompressed.  If not, move
	# it to $ADAPSERR.
	#----------------------------------------------------
	if ( ! -e $image ) then
	    set text = "Error uncompressing $image"
	    echo "[cklsu-uncompress] $text"
	    wrtlog $logfile $key [cklsu-uncompress] $text
	    mv $cimg $ADAPSERR/$cimg
	    mv $cksum $ADAPSERR/$cksum
	    continue
	endif

	#---------------------------------------------------------
	# Get the year from the file name.  LSU will sometimes use
	# underscores in their file names and sometimes dots.
	#---------------------------------------------------------
	set year = `echo $image | awk '{printf "%-6.6s",$1}'`
	if (`echo $year | awk -F\. '{print NF}'` == 2) then
	    set year = `echo $year | awk -F\. '{print $2}'`
	else if (`echo $year | awk -F_ '{print NF}'` == 2) then
	    set year = `echo $year | awk -F_ '{print $2}'`
	else
	    set year = `date '+%y'`
	    echo "[cklsu-year] Unknown year.  Setting it to $year"
	endif

	#---------------------------------------
	# Create a new name and rename the image
	#---------------------------------------
	set newimage = "akm_lsu_"$year"_fn$filenum.lsu"
	if (-e $newimage) then
	    set text = "Image $image could not be renamed to $newimage"
	    wrtlog $logfile $key cktrans-error $text
	    echo "[cktrans-exists] $text"
	else
	    mv $image $newimage
	    rm -f $cksum
	    set text = "$image renamed to $newimage"
	    wrtlog $logfile $key cklsu-rename $text
	    echo "[informational] $text"
	endif
    endif
end


#----------------------------------------------------------------------------
# Move all akm_lsu_??_fn???.lsu images to $ADAPSACQ/ if there is enough room.
#----------------------------------------------------------------------------
set images = `lsfiles '^akm_lsu_[0-9][0-9]_fn[0-9][0-9][0-9]\.lsu$'`
foreach image ($images)
    #-----------------------------------------------------------------------
    # Move the image to ADAPSACQ.  Just in case the current directory is on
    # another disk pack, move the file in such a way that the automatic
    # ingester doesn't grab it before we are done getting it there.  Move it
    # there using a temporary name and then rename the file once its there.
    #-----------------------------------------------------------------------
    onintr REMOVE_IMG
    if ( ! { ckspace "" $nbytes $ADAPSACQ } ) then
	set text = "Not enough room in ADAPSACQ ($nbytes bytes)"
	wrtlog $logfile cktrans cktrans-disk $text
	echo "[cktrans-disk] $text"
	break
    else if ( { mv $image $ADAPSACQ/_tmp_$image } ) then
	onintr -
	mv $ADAPSACQ/_tmp_$image $ADAPSACQ/$image
	set text = "$image moved to ADAPSACQ"
	wrtlog $logfile cktrans cklsu-move $text
	echo "[informational] $text"
        set printflag = "1"
    else
	onintr -
	rm -f $ADAPSACQ/_tmp_$image
	set text = "Error moving $image to ADAPSACQ"
	wrtlog $logfile cktrans cklsu-error $text
	echo "[cktrans-move] $text"
    endif
end


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
NO_FILES:
set text = "CKLSU completed successfully"
#-------------------------------------------------------------------
#if there aren't any files we don't want any messages to the logfile
#-------------------------------------------------------------------
if ("$printflag" != "0") then
   wrtlog $logfile cktrans cklsu-success $text
endif
echo "[informational] $text"
exit 0


REMOVE_IMG:
    onintr -
    rm -f $ADAPSACQ/$image

ERROUT:
    onintr -
    set text = "User aborted CKLSU"
    wrtlog $logfile cktrans cklsu-abort $text
    echo "[cktrans-abort] $text"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
