#!/bin/csh -f
#
# FUNCTION:  ckpas.csh
#
# PURPOSE:   Check the images sent to us from Prince Albert
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# D Lloyd          02/00  Taken from ckopf.csh and modified for pas


#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrupt happens
#	With the - argument, the shell ignores all interrupts.  
#	With a label argument, the shell executes a goto label
onintr -
set nbytes = 500000000
set printflag = "0"
set noglob
unalias cd
unalias df
unalias du
unalias ls
unalias rm


#-----------------------------------------------------------------------------
# Ensure there are six arguments.  These are the directory where the images
# are located and the paths for the commands checkspace, lsfiles, get_num.csh,
# write_log and then the session number respectively.
#-----------------------------------------------------------------------------
if ($#argv != 6) then
    echo "[cktrans-args] Must have six arguments for ckpas.csh"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#-----------------------------------------------------------------------------
# The calling pdf expects a temporary pdf called $tmppdf that will return the
# number of images that were checked.  Create this pdf so that it returns zero
# in the event of an error.
#-----------------------------------------------------------------------------
set tmppdf = `pwd`/paschecked$6.pdf

echo "procedure"                >  $tmppdf
echo "parm num type=name"   >> $tmppdf
echo "body"                     >> $tmppdf
echo "let num = 0"          >> $tmppdf
echo "ush rm -f $tmppdf"        >> $tmppdf
echo "end-proc"                 >> $tmppdf


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we will be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------

#the dir where the Prince Albert are placed
set srcdir = $1

if ("$2" == ".") then
    alias get_num.csh `pwd`/get_num.csh
else
    alias get_num.csh $2/get_num.csh
endif
if ("$3" == ".") then
    alias ckspace `pwd`/checkspace
else
    alias ckspace $3/checkspace
endif
if ("$4" == ".") then
    alias lsfiles `pwd`/lsfiles
else
    alias lsfiles $4/lsfiles
endif
if ("$5" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $5/write_log
endif

#- $6 is the unique session number -#

#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSACQ) || (! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[cktrans-env] ADAPSACQ, ADAPSLOG, or ADAPSERR are not set"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]cktrans" | tr / . | sed  's/\[\./\[/'`


#------------------------------------------------------------------------------
#?????????
# Check for the existance of a .done file that is older than 15 minutes that
# doesn't have a corresponding image file.  O. Germany requested that we leave
# the .done file hanging around for a while so that they can insure the file
# transfer was complete.
#------------------------------------------------------------------------------
cd $srcdir
set donefiles = `lsfiles '.*done$'`

if ( "$donefiles" != "" ) then
    foreach done ($donefiles)
        # check for the existance of the image file
        set root = `echo $done | awk -F. '{print $1"."$2"."$3}'`
        if (! -e $root ) then
            # image file doesn't exist so see if .done file is at
            # least 15 minutes old and delete it if so
            set hour = `date +%H`
            set min = `date +%M`
            set filet = `ls -l $done | awk '{print $8}'`
            set fileh = `echo $filet | awk -F: '{print $1}'`
            set filem = `echo $filet | awk -F: '{print $2}'`
            if ( $hour != $fileh ) then
                @ min = $min + 60
            endif
            @ filem = $filem + 15
            if ( $min > $filem ) then
                rm -f $done
            endif
        endif
    end
endif
 

#------------------------------------------------------------------------------
# Check to see if there are any images pushed from Prince Albert
# These images will be named like noaa_14_20000222_1841_26531.zip
# If none exist then issue a message and exit.
#------------------------------------------------------------------------------
cd $srcdir
set images = `lsfiles '^noaa.*\.zip$'`

#if there weren't any files
if ("$images" == "") then
    #-------------------------------------------------------------------
    #if there aren't any files we don't want any messages to the logfile
    # so we don't fill up the log file
    #---------------------------------------------------
    set text = "No PAS images"
    tty -s
    if ($status == 0) then
	echo "[informational] $text"
    endif
    goto NO_FILES
else
    set text = "Starting to check PAS images"
    echo "[informational] $text"

    #-------------------------------------------------------------------------
    # Recreate the temporary pdf that will return the number of images checked
    #-------------------------------------------------------------------------
    echo "procedure"                >  $tmppdf
    echo "parm num type=name"   >> $tmppdf
    echo "body"                     >> $tmppdf
    echo "let num = $#images"   >> $tmppdf
    echo "ush rm -f $tmppdf"        >> $tmppdf
    echo "end-proc"                 >> $tmppdf
endif


#------------------------------------------------------------------------------
# For each image pushed from PAS check to see if there is a corresponding
# file with a .done extension.  If there is then Prince Albert done
# sending us the file.  So we can process the file.  If there isn't a .done
# file then ignore the file.
#------------------------------------------------------------------------------
foreach img ($images)
    onintr  ERROUT

    #creating the corresponding name for the file with a .done extension
    #if the .done file doesn't exist then Prince Albert still transferring the
    # file to us (The .done file is the last to be sent)
    set done = `echo $img | awk -F. '{printf "%s.done", $1}'`
    if (! -e $done) then
	#seeing if the file is over a day old
        # (not have the check sum file and over
        # a day old means that the file needs to
        # be resent)
        #if $test = 0 the file is not over a day old
        #if $test = 1 the file is over a day old
        set test = 0
        set test = `find $srcdir -name $img -mtime +1 -exec "echo" 1 \;`
        if ($test) then
            mv $srcdir/$img $ADAPSERR/$img
            set filenum = `get_num.csh $srcdir`
	    set key     = "netpas-$filenum-pas"
            set text = "No check sum file for $img - moved to ADAPSERR"
            wrtlog $logfile $key ckpas-error $text
            echo "[ckpas-sum] $text"
        endif
	continue
    endif

    #--------------------------------
    # seeing if the filesizes differ
    #--------------------------------
    #the size that Prince Albert it was
    set remote = `head -1 $done | awk '{printf("%d", $1)}'`
    # the 5th column in ls -l is the size of the file we received
    set local = `/bin/ls -l $img | awk '{print $5}'`
    # if the size we received is not = to what Prince Albert it was then
    #  put the image file and the .done file in $ADAPSERR
    if ("$remote" != "$local") then
	set text1 = "File sizes differ for $img."
	set text2 = "Local size = $local, remote size = $remote."
	echo "[ckpas-unequal] $text1"
	echo "[ckpas-unequal] $text2"
	wrtlog $logfile ckpas.csh [ckpas-unequal] $text1
	wrtlog $logfile ckpas.csh [ckpas-unequal] $text2
	mv $img $ADAPSERR/$img
	mv $done $ADAPSERR/$done
        continue
    endif

    #---------------------------------------------------------
    # Uncompress and rename the image.  Use the unzip routine. 
    # Several files will be extracted from the zipped file
    # these include jpg, img, led, nul, trl and vol files.
    # We are interested in the header file (led) and image file (img).
    #---------------------------------------------------------
    unzip -j $img
    set filenum = `get_num.csh $srcdir`
    set cimg = $img
    set img = `echo $cimg | awk -F. '{printf "%s.img", $1}'`
    set header = `echo $cimg | awk -F. '{printf "%s.led", $1}'`
 
    #----------------------------------------------------
    # Make sure the file was uncompressed.  If not, move
    # it to $ADAPSERR.
    #----------------------------------------------------
    if ( ! -e $img  || ! -e $header ) then
	set text1 = "Error uncompressing $cimg"
	echo "[ckpas-uncompress] $text1"
	wrtlog $logfile ckpas.csh [ckpas-uncompress] $text1
	mv $cimg $ADAPSERR/$cimg
	mv $done $ADAPSERR/$done
	continue
    endif

    set year = `echo $img | awk -F. '{print substr($1, 11, 2)}'`
    set station = pas
    set newimage = "akm_pas_"$year"_fn$filenum.pas"
    if (-e $newimage) then
        set text = "Image $img could not be renamed to $newimage"
        wrtlog $logfile cktrans cktrans-error $text
        echo "[informational] $text"
    else
        cat $header $img > $newimage
        set text = "$header and $img renamed to $newimage"
        wrtlog $logfile cktrans ckpas-rename $text
        echo "[informational] $text"
    endif

    #-----------------------------------------------------------------------
    # Move the image to ADAPSACQ.  Just in case the current directory is on
    # another disk pack, move the file in such a way that the automatic
    # ingester doesn't grab it before we are done getting it there.  Move it
    # there using a temporary name and then rename the file once its there.
    #-----------------------------------------------------------------------
    onintr REMOVE_IMG
    # if there isn't enough space....
    if ( ! { ckspace "" $nbytes $ADAPSACQ } ) then
	set text = "Not enough room in ADAPSACQ ($nbytes bytes)"
	wrtlog $logfile cktrans cktrans-disk $text
	echo "[cktrans-disk] $text"
	break
    #try to do the move to a temporary name and then rename it to original name
    else if ( { mv $newimage $ADAPSACQ/_tmp_$newimage } ) then
	onintr -
	mv $ADAPSACQ/_tmp_$newimage $ADAPSACQ/$newimage
	set text = "$newimage moved to ADAPSACQ"
	wrtlog $logfile cktrans ckpas-move $text
	echo "[informational] $text"
        set printflag = "1"
    else
	onintr -
	rm -f $ADAPSACQ/_tmp_$newimage
	set text = "Error moving $img to ADAPSACQ"
	wrtlog $logfile cktrans ckpas-error $text
	echo "[cktrans-move] $text"
    endif

    #---------------------------------------------
    # remove the zip file and associated files
    #---------------------------------------------
    rm -f $cimg $img $header 
    unset noglob
    rm -f *.jpg *.nul *.trl *.vol
    set noglob
end


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
NO_FILES:
set text = "CKPAS completed successfully"
#if there aren't any files we don't want any messages to the logfile
if ("$printflag" != "0") then
   wrtlog $logfile cktrans ckpas-success $text
endif
echo "[informational] $text"
exit 0


REMOVE_IMG:
    onintr -
    rm -f $ADAPSACQ/$img

ERROUT:
    onintr -
    set text = "User aborted CKPAS"
    wrtlog $logfile cktrans cktrans-abort $text
    echo "[cktrans-abort] $text"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
