#!/bin/csh -f
#
# FUNCTION:  ckutokyo.csh
#
# PURPOSE:   Check the images sent to us from the University of Tokyo
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# J.Fenno          05/92  Development
# A Vander Vorste  01/96  Modified to behave like and have
#			  same saftey checks as cklsu.csh and ckbaa.csh


#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrup happens
#	With the - argument, the shell ignores all interrupts.  
#	With a label argument, the shell executes a goto label
onintr -
set nbytes = 500000000
set printflag = "0"
set noglob
unalias cd
unalias df
unalias du
unalias ls
unalias rm


#-----------------------------------------------------------------------------
# Ensure there are six arguments.  These are the directory where the images
# are located and the paths for the commands checkspace, lsfiles, get_num.csh,
# write_log and then the session number respectively.
#-----------------------------------------------------------------------------
if ($#argv != 6) then
    echo "[cktrans-args] Must have six arguments for ckutokyo.csh"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#-----------------------------------------------------------------------------
# The calling pdf expects a temporary pdf called $tmppdf that will return the
# number of images that were checked.  Create this pdf so that it returns zero
# in the event of an error.
#-----------------------------------------------------------------------------
set tmppdf = `pwd`/ntk2checked$6.pdf

echo "procedure"                >  $tmppdf
echo "parm num type=name"   >> $tmppdf
echo "body"                     >> $tmppdf
echo "let num = 0"          >> $tmppdf
echo "ush rm -f $tmppdf"        >> $tmppdf
echo "end-proc"                 >> $tmppdf


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we will be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------

#the dir where the U. of Tokyo images are placed
set srcdir = $1

if ("$2" == ".") then
    alias get_num.csh `pwd`/get_num.csh
else
    alias get_num.csh $2/get_num.csh
endif
if ("$3" == ".") then
    alias ckspace `pwd`/checkspace
else
    alias ckspace $3/checkspace
endif
if ("$4" == ".") then
    alias lsfiles `pwd`/lsfiles
else
    alias lsfiles $4/lsfiles
endif
if ("$5" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $5/write_log
endif

#- $6 is the unique session number -#

#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSACQ) || (! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[cktrans-env] ADAPSACQ, ADAPSLOG, or ADAPSERR are not set"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]cktrans" | tr / . | sed  's/\[\./\[/'`


#------------------------------------------------------------------------------
# Check to see if there are any images pushed from U. of Tokyo.  
# These images will be named like ah14012596042353.tk2.  If none exist 
# then issue a message and exit.
#------------------------------------------------------------------------------
cd $srcdir
set images = `lsfiles '^a[hlg]..............\.tk2$'`

#if there weren't any files
if ("$images" == "") then
    #-------------------------------------------------------------------
    #if there aren't any files we don't want any messages to the logfile
    # so we don't fill up the log file
    #---------------------------------------------------
    set text = "No TK2 images"
    tty -s
    if ($status == 0) then
	echo "[informational] $text"
    endif
    goto NO_FILES
else
    set text = "Starting to check TK2 images"
    echo "[informational] $text"

    #-------------------------------------------------------------------------
    # Recreate the temporary pdf that will return the number of images checked
    #-------------------------------------------------------------------------
    echo "procedure"                >  $tmppdf
    echo "parm num type=name"   >> $tmppdf
    echo "body"                     >> $tmppdf
    echo "let num = $#images"   >> $tmppdf
    echo "ush rm -f $tmppdf"        >> $tmppdf
    echo "end-proc"                 >> $tmppdf
endif


#------------------------------------------------------------------------------
# For each image pushed from TK2 check to see if there is a corresponding
# file with a .done extension.  If there is then Tokyo is done sending us the
# file.  So we can process the file.  If there's not a .done file then
# ignore the file
#------------------------------------------------------------------------------
foreach img ($images)
    onintr  ERROUT

    #find the beginning of the image name (image name without .tk2 extension)
    set root = `echo $img | awk -F. '{print $1}'`
    #creating the corresponding name for the file with a .done extension
    set done = ${root}.done
    #if the .done file doesn't exist then Tokyo is still transfering the
    # file to us (The .done file is the last to be sent)
    if (! -e $done) then
	#seeing if the file is over a day old
        # (not have the check sum file and over
        # a day old means that the file needs to
        # be resent)
        #if $test = 0 the file is not over a day old
        #if $test = 1 the file is over a day old
        set test = 0
        set test = `find $srcdir -name $img -mtime +1 -exec "echo" 1 \;`
        if ($test) then
            mv $srcdir/$img $ADAPSERR/$img
	    set filenum = `get_num.csh $srcdir`
	    set key     = "nettk2-$filenum-tk2"
            set text = "No check sum file for $img - moved to ADAPSERR"
            wrtlog $logfile $key cktk2-error $text
            echo "[cktk2-sum] $text"
        endif
	continue
    endif

    #--------------------------------
    # seeing if the filesizes differ
    #--------------------------------
    #the size that tokyo thought it was
    set remote = `cat $done`
    # the 5th column in ls -l is the size of the file we recieved
    set local = `/bin/ls -l $img | awk '{print $5}'`
    # if the size we received is not = to what tokyo thought it was then
    #  put the image file and the .done file in $ADAPSERR
    if ("$remote" != "$local") then
	set text1 = "File sizes differ for $img."
	set text2 = "Local size = $local, remote size = $remote."
	echo "[ckutokyo-unequal] $text1"
	echo "[ckutokyo-unequal] $text2"
	wrtlog $logfile ckutokyo.csh [ckutokyo-unequal] $text1
	wrtlog $logfile ckutokyo.csh [ckutokyo-unequal] $text2
	mv $img $ADAPSERR/$img
	mv $done $ADAPSERR/$done
        continue
    endif

    #-----------------------------------------------------------------------
    # Move the image to ADAPSACQ.  Just in case the current directory is on
    # another disk pack, move the file in such a way that the automatic
    # ingester doesn't grab it before we are done getting it there.  Move it
    # there using a temporary name and then rename the file once its there.
    #-----------------------------------------------------------------------
    onintr REMOVE_IMG
    # if there isn't enough space....
    if ( ! { ckspace "" $nbytes $ADAPSACQ } ) then
	set text = "Not enough room in ADAPSACQ ($nbytes bytes)"
	wrtlog $logfile cktrans cktrans-disk $text
	echo "[cktrans-disk] $text"
	break
    #try to do the move to a temporary name and then rename it to original name
    else if ( { mv $img $ADAPSACQ/_tmp_$img } ) then
	onintr -
	mv $ADAPSACQ/_tmp_$img $ADAPSACQ/$img
    	rm -f $done
	set text = "$img moved to ADAPSACQ"
	wrtlog $logfile cktrans ckutokyo-move $text
	echo "[informational] $text"
        set printflag = "1"
    else
	onintr -
	rm -f $ADAPSACQ/_tmp_$img
	set text = "Error moving $img to ADAPSACQ"
	wrtlog $logfile cktrans ckutokyo-error $text
	echo "[cktrans-move] $text"
    endif
end


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
NO_FILES:
set text = "CKUTOKYO completed successfully"
#if there aren't any files we don't want any messages to the logfile
if ("$printflag" != "0") then
   wrtlog $logfile cktrans ckutokyo-success $text
endif
echo "[informational] $text"
exit 0


REMOVE_IMG:
    onintr -
    rm -f $ADAPSACQ/$img

ERROUT:
    onintr -
    set text = "User aborted CKUTOKYO"
    wrtlog $logfile cktrans cktrans-abort $text
    echo "[cktrans-abort] $text"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
