#!/bin/csh -f
#
# FUNCTION:  ckykt.csh
#
# PURPOSE:   Check the images sent to us from Yakutsk Russia
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# D.Lloyd          11/02  Taken from ckopf.csh and modified for YKT


#-------------------------------------------------------------------
# Set up the shell. 
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrupt happens
#	With the - argument, the shell ignores all interrupts.  
#	With a label argument, the shell executes a goto label
onintr -
set nbytes = 500000000
set printflag = "0"
unalias cd
unalias df
unalias du
unalias ls
unalias rm
# For testing set the working directory to something else
set workdir = /sgs4/km1/xfer/ykt


#-----------------------------------------------------------------------------
# Ensure there are six arguments.  These are the directory where the images
# are located and the paths for the commands checkspace, lsfiles, get_num.csh,
# write_log and then the session number respectively.
#-----------------------------------------------------------------------------
if ($#argv != 6) then
    echo "[cktrans-args] Must have six arguments for ckykt.csh"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#-----------------------------------------------------------------------------
# The calling pdf expects a temporary pdf called $tmppdf that will return the
# number of images that were checked.  Create this pdf so that it returns zero
# in the event of an error.
#-----------------------------------------------------------------------------
set tmppdf = `pwd`/yktchecked$6.pdf

echo "procedure"                >  $tmppdf
echo "parm num type=name"   >> $tmppdf
echo "body"                     >> $tmppdf
echo "let num = 0"          >> $tmppdf
echo "ush rm -f $tmppdf"        >> $tmppdf
echo "end-proc"                 >> $tmppdf


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we will be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------

#the dir where the Yakutsk images are placed
set srcdir = $1

if ("$2" == ".") then
    alias get_num.csh `pwd`/get_num.csh
else
    alias get_num.csh $2/get_num.csh
endif
if ("$3" == ".") then
    alias ckspace `pwd`/checkspace
else
    alias ckspace $3/checkspace
endif
if ("$4" == ".") then
    alias lsfiles `pwd`/lsfiles
else
    alias lsfiles $4/lsfiles
endif
if ("$5" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $5/write_log
endif

#- $6 is the unique session number -#

#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSACQ) || (! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[cktrans-env] ADAPSACQ, ADAPSLOG, or ADAPSERR are not set"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "$ADAPSLOG/cktrans"`

#----------------------------------------------------------
# Check to see if there are any images pushed from Yakutsk.
# These images will look like FR23MARB.ZIP
# If none exist then issue a message and exit.
#----------------------------------------------------------
cd $srcdir
set images = `ls *.ZIP`

#if there weren't any files
if ("$images" == "") then
    #-------------------------------------------------------------------
    #if there aren't any files we don't want any messages to the logfile
    # so we don't fill up the log file
    #---------------------------------------------------
    set text = "No YKT images"
    tty -s
    if ($status == 0) then
	echo "[informational] $text"
    endif
    goto NO_FILES
else
    set text = "Starting to check YKT images"
    echo "[informational] $text"

    #-------------------------------------------------------------------------
    # Recreate the temporary pdf that will return the number of images checked
    #-------------------------------------------------------------------------
    echo "procedure"                >  $tmppdf
    echo "parm num type=name"   >> $tmppdf
    echo "body"                     >> $tmppdf
    echo "let num = $#images"   >> $tmppdf
    echo "ush rm -f $tmppdf"        >> $tmppdf
    echo "end-proc"                 >> $tmppdf
endif

#------------------------------------------------------------------------------
# For each image pushed from YKT check to see if there is a corresponding
# file with a .DONE extension.  If there is then Yakutsk is done
# sending us the file.  So we can process the file.  If there isn't a .DONE
# file then ignore the file.
#------------------------------------------------------------------------------
foreach img ($images)
    onintr  ERROUT
########### The following is done in the source directory #############
    cd $srcdir

    #creating the corresponding name for the file with a .done extension
    set done = `echo ${img} | awk -F. '{print $1}'`
    set done = "${done}.DONE"
    #if the .DONE file doesn't exist then Yakutsk is still transferring the
    # file to us (The .DONE file is the last to be sent)
    if (! -e "$done") then
#=Temporary code to accomodate no .done files are marked this way ==========
#=============== From here =================================================
	# Temporarily if the .done file is not around go ahead and
	# copy the file if it's 6 hours old.  This is to give plenty
	# of time to let them finish the transmission
	if ( 1 ) then
	    # Create a reference file that is 6 hours old
	    set x1 = `date +%m%d`
	    set hr = `date +%H`
	    set x2 = `date +%M`
	    @ hr = $hr - 6
	    # to prevent a whole bunch of worrying about going into yesterday
	    # which may go into last month or last year, if the hour is less
	    # than 6 skip these tests until it's at least 6.
	    if ( $hr < 0 ) then
		continue
	    else if ( $hr < 10 ) then
		set hr = "0$hr"
	    endif
	    touch -t $x1$hr$x2 Timecompfile

	    # Check that the file is older than the reference file
	    set st = `find $srcdir/$img -newer Timecompfile`
	    rm -f Timecompfile
	    if ( "$st" == "" ) then
		set text = "Creating done file for $img"
		set key = "nodone"
		wrtlog $logfile $key ckykt-sixhr $text
		set local = `/bin/ls -l $img | awk '{print $5}'`
		echo $local > $done
	    else
		continue
	    endif
	else
#=============== to here =================================================
	#seeing if the file is over a day old
        # (not have the check sum file and over
        # a day old means that the file needs to
        # be resent)
        #if $test = 0 the file is not over a day old
        #if $test = 1 the file is over a day old
        set test = 0
        set test = `find $srcdir -name $img -mtime +1 -exec "echo" 1 \;`
        if ($test) then
            mv $img $ADAPSERR/$img
            set filenum = `get_num.csh $workdir`
	    set key     = "netykt-$filenum-ykt"
            set text = "No check sum file for $img - moved to ADAPSERR"
            wrtlog $logfile $key ckykt-error $text
            echo "[ckykt-sum] $text"
        endif
	continue
#=============== and here ================================================
	#???temporary
	endif
#=============== to here =================================================
    endif

########### The rest is done in the working directory #############
    cd $workdir
    # move the files to the working directory
    mv $srcdir/$done $done
    mv $srcdir/$img $img

    #--------------------------------
    # seeing if the filesizes differ
    #--------------------------------
    #the size that Yakutsk thought it was
    set remote = `cat $done`
    # the 5th column in ls -l is the size of the file we received
    set local = `/bin/ls -l $img | awk '{print $5}'`
    # if the size we received is not = to what Yakutsk thought it was then
    #  put the image file and the .DONE file in $ADAPSERR
    if ("$remote" != "$local") then
	set text1 = "File sizes differ for $img."
	set text2 = "Local size = $local, remote size = $remote."
	echo "[ckykt-unequal] $text1"
	echo "[ckykt-unequal] $text2"
	wrtlog $logfile ckykt.csh "[ckykt-unequal]" $text1
	wrtlog $logfile ckykt.csh "[ckykt-unequal]" $text2
	mv $img $ADAPSERR/$img
	mv $done $ADAPSERR/$done
        continue
    endif

    #----------------------------------------------------
    # Uncompress and rename the image.
    #----------------------------------------------------
    set cimg = $img
    set img = `echo $cimg | awk -F. '{printf "%s.ykt", $1}'`
    gunzip -cS .zip $cimg > $img
 
    #----------------------------------------------------
    # Make sure the file was uncompressed.  If not, move
    # it to $ADAPSERR.
    #----------------------------------------------------
    #if ( $status == 1 ) then
	#set text1 = "Error uncompressing $img"
	#echo "[ckykt-uncompress] $text1"
	#wrtlog $logfile ckykt.csh "[ckykt-uncompress]" $text1
	#mv $cimg $ADAPSERR/$cimg
	#mv $done $ADAPSERR/$done
	#rm $img
	#continue
    #else
        rm -f $cimg
        rm -f $done
    #endif

    set filenum = `get_num.csh .`
    # get the real year from the file
    set year = `od -j 58 -N 3 -t c $img | awk '{print $2$4}'`

    set newimage = "akm_ykt_${year}_fn${filenum}.ykt"
    if (-e $newimage) then
        set text = "Image $img could not be renamed to $newimage"
        wrtlog $logfile cktrans cktrans-error $text
        echo "[informational] $text"
    else
        mv $img $newimage
        set text = "$img renamed to $newimage"
        wrtlog $logfile cktrans ckykt-rename $text
        echo "[informational] $text"
    endif
    set img = $newimage

    #-----------------------------------------------------------------------
    # Move the image to ADAPSACQ.  Just in case the current directory is on
    # another disk pack, move the file in such a way that the automatic
    # ingester doesn't grab it before we are done getting it there.  Move it
    # there using a temporary name and then rename the file once its there.
    #-----------------------------------------------------------------------
    onintr REMOVE_IMG
    # if there isn't enough space....
    if ( ! { ckspace "" $nbytes $ADAPSACQ } ) then
	set text = "Not enough room in ADAPSACQ ($nbytes bytes)"
	wrtlog $logfile cktrans cktrans-disk $text
	echo "[cktrans-disk] $text"
	break
    #try to do the move to a temporary name and then rename it to original name
    else if ( { mv $img $ADAPSACQ/_tmp_$img } ) then
	onintr -
	mv $ADAPSACQ/_tmp_$img $ADAPSACQ/$img
	set text = "$img moved to ADAPSACQ"
	wrtlog $logfile cktrans ckykt-move $text
	echo "[informational] $text"
        set printflag = "1"
    else
	onintr -
	rm -f $ADAPSACQ/_tmp_$img
	set text = "Error moving $img to ADAPSACQ"
	wrtlog $logfile cktrans ckykt-error $text
	echo "[cktrans-move] $text"
    endif
end


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
NO_FILES:
set text = "CKYKT completed successfully"
#if there aren't any files we don't want any messages to the logfile
if ("$printflag" != "0") then
   wrtlog $logfile cktrans ckykt-success $text
endif
echo "[informational] $text"
exit 0


REMOVE_IMG:
    onintr -
    rm -f $ADAPSACQ/$img

ERROUT:
    onintr -
    set text = "User aborted CKYKT"
    wrtlog $logfile cktrans cktrans-abort $text
    echo "[cktrans-abort] $text"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
