#!/bin/csh
#
# conflict.csh
# 
# Purpose: To generate live acquisition add.sch file in a temporary environment.
# This is to allow scheduled conflicting scenes to be removed from the add.sch
# file before adding the scenes to the master schedule.
# 
# Reasoning: When two scheduled scenes are in conflict, one must be removed from
# the schedule and later scheduled for ftp pull from NOAA.  Not all cases will
# be resolved by NOAAFTP and this scheduling guarantees a match can be made from
# the NOAA recorder schedule.  Also this will reduce the number of conflicts
# that are printed on the conflict report and, is therefore, less confusing to
# operations personnel.
# 
# Usage: conflict.csh MM DD YY [mm dd yy]
#        where MM DD YY is the start month day and year for the schedule period
#              mm dd yy is the optional stop month day and year, if omitted
#	      autogen will generate a stop date 7 days after the start date
# 
#     The date strings also allow, and correct, dates such as:
#         mm/dd/yy, mm/dd/yyyy or mm dd yyyy
#*******************************************************************************

echo $PRINTER
# Check for command line arguments must have either 3 or 6
#---------------------------------------------------------
if ( $#argv == 3 || $#argv == 6 ) then
    set startdate = "$1 $2 $3"
    if ( $#argv == 6) then
        set stopdate = "$4 $5 $6"
    endif

# Otherwise prompt for the arguments and either accept them or exit
#------------------------------------------------------------------
else
    echo ''
    echo 'Usage: conflict.csh MM DD YY [mm dd yy]'
    echo 'Where MM DD YY is the start date'
    echo '      mm dd yy is the optional stop date'
    echo ''

    # prompt the user for the dates
    echo -n 'Would you like to enter the dates? (yes/no) '
    set userinput = $<
    if ( $userinput == "yes" || $userinput == "y") then
	# get the start date
        echo -n 'Enter start date: '
	set startdate = $<

	# Now get the stop date
        echo -n 'Enter stop date: [optional] '
	set stopdate = $<
	# if none was entered unset the variable
	if ( "$stopdate" == "" ) then
	    unset stopdate
	endif
    else
	exit 1
    endif
endif

# Do some error checking on the input arguments.  Autogen expects
# a date in the form MM DD YY.  Check and correct MM/DD/YY and
# MM/DD/YYYY.  Other errors will be detected by autogen.
#----------------------------------------------------------------
set startdate = `echo $startdate | tr "/" " "`
if ( $#startdate != 3 ) then
    echo "Incorrect start date: $startdate, can't continue"
    exit 1
endif
if( $startdate[3] > 100 ) then
    @ startdate[3] %= 100
endif
if ( ${?stopdate} ) then
    set stopdate = `echo $stopdate | tr "/" " "`
    if ( $#stopdate != 3 ) then
	echo "Incorrect stop date: $stopdate, can't continue"
	exit 1
    endif
    if( $stopdate[3] > 100 ) then
	@ stopdate[3] %= 100
    endif
endif

# Save the current directory
# and change to a temp dir
#---------------------------
set olddir = `pwd`
set stamp = `date +%m%d%y%H%M%S`
set tmpdir = "schedtemp_$stamp"
while ( -e $tmpdir )
    set tmpdir = '${tmpdir}0'
end
mkdir $tmpdir
cd $tmpdir

# Save the original adaps tables environment and
# set it to a new temporary location.  The temp
# location must have copies or links to the actual
# satellite.att, satellite.ids and *.dat files
#-------------------------------------------------
setenv ORIGTABLES  $ADAPSTABLES
setenv ADAPSTABLES `pwd`
ln -s $ORIGTABLES/*.dat .
ln -s $ORIGTABLES/satellite.att .
ln -s $ORIGTABLES/satellite.ids .

# Run xplot: user schedules live scenes
# make a temporary copy of the file generated by
# xplot and add these scenes to a new master schedule
#----------------------------------------------------
$ADAPSBASE/xplot
cp add.sch temp_add.sch
echo "0" > $ADAPSTABLES/master.sch
$ADAPSBASE/schedit -af add.sch

# Create a temporary pdf to run autogen
#--------------------------------------
if ( ${?stopdate} ) then
    set cmdstr = "autogen stadate=($startdate) stodate=($stopdate)"
else
    set cmdstr = "autogen stadate=($startdate)"
endif
cat << EOF > autogen_temp.pdf
procedure autogen_temp
body
$cmdstr
exit
end-proc
EOF

# Execute autogen
#----------------
cat << EOJ | tae >& autogen_temp.stdout
autogen_temp
EOJ
# ??? A good status from autogen seems to be 1
if ( $status != 1 ) then
    echo "Autogen error status: $status"
    echo "          start date: $startdate"
    echo "           stop date: $stopdate"
    exit 1
endif

# Create a conflict report from the
# output of autogen.
#----------------------------------
cat autogen17.prt autogen18.prt autogen19.prt > autogen.prt
rm -f autogen12.prt autogen14.prt autogen15.prt autogen16.prt \
autogen17.prt autogen18.prt autogen19.prt
$ADAPSBASE/schedcnflt autogen.prt conflict.report $ADAPSTABLES/master.sch
lp -d$PRINTER conflict.report

# Build a file of conflicting scene IDs
#--------------------------------------
grep -i conflict conflict.report | grep -v Automatic | cut -c 1-16 > delete.sid

# Using the file of scenes to delete, make a new
# add.sch file excluding the scenes we want to delete
# This must be done prior to the next step since the
# delete.sid file will be deleted by schedit.
#------------------------------------------------
grep -vif delete.sid temp_add.sch > new_add.sch

# Delete the scenes from the schedule
#------------------------------------
echo "These are the scenes you need to schedule via ftp:\n" > onlydeletethese
grep -if delete.sid $ADAPSTABLES/master.sch | cut -c 1-16 >> onlydeletethese
lp -d$PRINTER onlydeletethese
$ADAPSBASE/schedit -ds delete.sid

# Execute autogen again
#----------------------
cat << EOJ | tae >& autogen_temp.stdout
autogen_temp
EOJ
# A good status from autogen seems to be 1
if ( $status != 1 ) then
    echo "Autogen error status: $status"
    echo "          start date: $startdate"
    echo "           stop date: $stopdate"
    exit 1
endif

# Create a new conflict report from the
# output of autogen.
#----------------------------------
cat autogen17.prt autogen18.prt autogen19.prt > autogen.prt
rm -f autogen12.prt autogen14.prt autogen15.prt autogen16.prt \
autogen17.prt autogen18.prt autogen19.prt
$ADAPSBASE/schedcnflt autogen.prt newconflict.report $ADAPSTABLES/master.sch
lp -d$PRINTER newconflict.report
#$ADAPSBASE/schedit -rb newschedule
#lpland $PRINTER newschedule

# Decide if all is well
#----------------------
echo -n "Do you want to continue or quit without scheduling? (yes/quit) "
set userinput = $<
if ( $userinput == "yes" || $userinput == "y") then
    cp new_add.sch $olddir/add.sch
    cd $olddir
    # ??? rm -r "schedtemp_$stamp"
    exit 0
else
    cd $olddir
    # ??? rm -r "schedtemp_$stamp"
    exit 1
endif

# If you got here it had to be by mistake
# so exit with an error status
#----------------------------------------
exit 1
