#!/bin/csh
#
# FUNCTION:  adapstables_backup.csh
#
# PURPOSE:   Back-up files in $ADAPSTABLES which change daily;
#            namely ephemeris data tables and NBS time correction tables.
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# T.Baltzer        09/98    Development
#
#

#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrup happens
#       With the - argument, the shell ignores all interrupts.
#       With a label argument, the shell executes a goto label
onintr ERROUT
set noglob

#-----------------------------------------------------------------------------
# Ensure the correct number of arguments have been specified.
#-----------------------------------------------------------------------------
if ($#argv != 1) then
   echo ""
   echo "Usage: adapstables_backup.csh wrtlog_path"
   echo ""
   echo "       <wrtlog_path> is the path of the write_log executable."
   echo ""
   echo "[adapstables-fatal] Fatal error encountered"
   exit 1
endif

#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if (! $?ADAPSLOG) then
    echo "[adapstables-env] ADAPSLOG is not set"
    echo "[adapstables-fatal] Fatal error encountered"
    exit 1
endif

#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]tablesbckup" | tr / . | sed  's/\[\./\[/'`

#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we may be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------
if ("$1" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $1/write_log
endif

#  ------------------------------------------------------
#  Define the archive directory on the silo.
#  ------------------------------------------------------
set ARCHDIR = "/dmf/edc/avhrr/adapstables_backup"

#  ------------------------------------------------------
#  Define the number of days old an archive file has to
#  be before it gets deleted.
#  ------------------------------------------------------
set DAYS = 30

#  ------------------------------------------------------
#  Get the system date, use it to make a timestamp
#  suffix for the file names in the archive directory.
#  ------------------------------------------------------
set sysdate = `date '+%d%H%M%S'`

#  ------------------------------------------------------
#  Save the original directory, will want to return here.
#  ------------------------------------------------------
set origdir = `pwd`

#  ------------------------------------------------------
#  Go to the ADAPSTABLES directory.
#  ------------------------------------------------------
cd $ADAPSTABLES

#  ------------------------------------------------------
#  Copy each of the NBS time correction files and each of
#  the ephemeris data files to the archive directory.
#  Append a time stamp to each filename to make it unique.
#  ------------------------------------------------------
foreach file (`ls -c1 | grep '^noaa.*.timcor$'`)
    set newfile = ${file}_$sysdate
    cp $file $newfile
    umput -d $ARCHDIR $newfile
    rm $newfile
end
foreach file (`ls -c1 | grep '^[0-9].*.dat$'`)
    set newfile = ${file}_$sysdate
    cp $file $newfile
    umput -d $ARCHDIR $newfile
    rm $newfile
end

#  ------------------------------------------------------
#  Return to the original directory.
#  ------------------------------------------------------
cd $origdir

#  ------------------------------------------------------
#  Delete the files in ARCHDIR which are more than DAYS
#  days old.
#  ------------------------------------------------------
$ADAPSLOCAL/cleansilo $DAYS $ARCHDIR "*"

#--------------------------------------
# Issue a successful completion message
#--------------------------------------
set text = "Successfully backed up ADAPSTABLES files"
wrtlog $logfile adapstables_backup adapstables_backup $text
echo "[informational]" $text
exit 0

#------------------------------------------------------------------
# If an error occurs, write an error message to the log and screen.
#------------------------------------------------------------------
ERROUT:
    onintr -

    set text = "Error backing up ADAPSTABLES files"
    wrtlog $logfile adapstables_backup adapstables_backup $text
    echo "[informational]" $text
    echo "[adapstables-fatal] Fatal error encountered"
    exit 1

