#!/bin/csh -f
#
# FUNCTION:  esameta.csh
#
# PURPOSE:   Integrate the metadata sent from ESA for the ESA stitched orbits
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# T.Baltzer        04/96  Development
# 
#
#  Parameters:
#  - The first parameter is the directory in which write_log resides.
#  - The second parameter must be characters 3 thru 16 of the scene id in
#    question.  Example:  scene id    = ao11121996042354
#                         parameter 2 =   11121996042354
#
#
#  The ESA metadata files exist in ADAPSORB/esameta.  Search these files
#  for the scene specified in parameter $2.
#
#  If a metadata file is found:
#    1) copy it to ADAPSDB/mmddyyhhmmss.orb (where mmddyyhhmmss is the
#       system date and time.  This syntax is consistent with EDC stitching.
#    2) Move the file from ADAPSORB/esameta to "ADAPSDBTMP/mmddyyhhmmss.esa".
#       This syntax is consistent with EDC stitching except that the file
#       would be called "ADAPSDBTMP/mmddyyhhmmss.done".
#
#  If the metadata file isn't found in ADAPSORB/esameta:
#    1) Generate a file ADAPSDB/aossmmddyyhhmmss.required where 
#       "aossmmddyyhhmmss" is the orbital scene id.
#
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# Unalias UNIX utilities being used
#-------------------------------------------------------------------
unalias cd
unalias ls

set noglob


#-------------------------------------------------------------------
# Make sure there are two parameters
#-------------------------------------------------------------------
if ($#argv != 2) then
    echo "[esameta-args] Must have two argument for esameta.csh"
    echo "[esameta-fatal] Fatal error encountered"
    exit 1
endif


#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSDB) || (! $?ADAPSLOG) || (! $?ADAPSORB) || (! $?ADAPSDBTMP)) then
    echo "[esameta-env] Required environment variables are not set"
    echo "[esameta-fatal] Fatal error encountered"
    exit 1
endif


#-------------------------------------------------------------
# Get the location of the write log code
#-------------------------------------------------------------
if ("$1" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $1/write_log
endif


#--------------------------------------------------------
# Create a log file.  This must be a LAS name for wrtlog.
#--------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]esaorb" | tr / . | sed  's/\[\./\[/'`


#-------------------------------------------------------------------
# Determine the .required file name.  If it doesn't exist, generate
# one.  If a subsequent error occurs, a .required file will at least
# exist.  This .required file will be deleted below, if the
# appropriate metadata is found and moved successfully.
#-------------------------------------------------------------------
set outdbreq = $ADAPSDB/ao$2.required
if (! -e $outdbreq) then
    echo "Metadata from ESA required for ao$2" >> $outdbreq
    if ($status != 0) then
	set text = "Cannot create file ao$2.required"
	wrtlog $logfile ao$2 esameta-cre $text
	echo "[esameta-cre] $text"
	echo "[esameta-fatal] Fatal error encountered"
	exit 1
    endif
endif


#-------------------------------------------------------------------
# Determine the first 12 numeral char of the sceneid.  For example,
# for sceneid AO11031296133245
# extract       110312961332
# (The HHMM part of HHMMSS is unique enough, seconds may vary).
#-------------------------------------------------------------------
set searchstr = $2
set searchstr = `echo $searchstr | awk '{printf "%12.12s",$1}'`


#-------------------------------------------------------------------
# Search for the metadata file(s) and respond accordingly.
# More than one metadata file may exist if the orbit was reprocessed.
# If more than one exists for a given orbit, move all of them, sleep
# 1 second to ensure a unique file name.
#-------------------------------------------------------------------
set found_it = false
set esametadir = $ADAPSORB/esameta
cd $esametadir
foreach metafile (`ls`)
    grep $searchstr $metafile >> /dev/null
    if ($status == 0) then
	sleep 1
	set datevar = `date '+%m%d%y%H%M%S'`
	set outdbname = $ADAPSDB/$datevar.orb
	set outdbtmpname = $ADAPSDBTMP/$datevar.esa
	cp $metafile $outdbtmpname
	mv $metafile $outdbname
	set text = "Moved ESA metadata $metafile to $datevar.orb"
	wrtlog $logfile ao$2 esameta-move $text
	echo "[informational] $text"
        set found_it = true
    endif
end


#-------------------------------------------------------------------
# If ESA metadata has not been found for this orbit, leave the file
# flagging the unsuccessful search.  If the metadata has been found,
# delete the file.
#-------------------------------------------------------------------
if ($found_it == true) then
    rm -f $outdbreq
    set text = "ESA metadata processed successfully for ao$2"
    wrtlog $logfile ao$2 esameta-succ $text
    echo "[informational] $text"
else
    set text = "ESA metadata not found, ao$2.required created"
    wrtlog $logfile ao$2 esameta-move $text
    echo "[informational] $text"
endif


exit 0

