#!/bin/csh
#
# FUNCTION:     build_software.csh
#
# PURPOSE:      Build the software in a build directory, identify any obsolete
#               files in the release by comparing to the build directory,
#               and release the new build to the release directory (relative
#               to $EDCSOFT).
#
# Usage:
#   $1  -  Tag to be used to check out the source from the CVS repository.
#          When no Installation directory is specified:
#              Specifying a tag will build the TAE/LAS/ADAPS software in the
#              "test_build" directory with the debug options when compiled
#              and will NOT distribute the software to other systems.
#
#              Specifying no tag, will build the TAE/LAS/ADAPS software in the
#              "release_build" directory and WILL distribute the software to
#              other associated systems.
#
#              To specify no tag use an empty string (double quotes "") or
#              leave the parameter off the command line.
#
#   $2  -  Installation directory.  This is an optional parameter that
#          specifies the location of the the software build.  The source
#          code will be checked out, with the specified tag, to this
#          directory and built.  The software will NOT be distributed to
#          other systems when this parameter is specified.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development
# T. Beckmann      07/00  Automatically start the LAS benchmark after the build
#


#-----------------------------------
# Retreive input arguments specified
#-----------------------------------
if ( $#argv < 1 ) then
    set tag = ""
    set install_dir = ""
else if ( $#argv == 1 ) then
    set tag = $1
    set install_dir = ""
else if ( $#argv == 2 ) then
    set tag = $1
    set install_dir = $2
else
    goto USAGE
endif

#-----------------------
# Set up shell variables
#-----------------------
unalias cd
set halfline = "----------------------------------------"
set stamp   = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`

#-------------------------------------------------
# By default do not distribute or build with debug
#-------------------------------------------------
set do_release = 0
set debug_flag = 0

#---------------------------------------------------
# Determine the directory location for the new build
#---------------------------------------------------
set system = `uname -n`
if ($install_dir == "") then
    if ($system == "edcsgs20") then
        if ($tag == "") then
            #--------------------------------------------
            # Allow no tag to mean release build, for now
            #--------------------------------------------
            set install_dir = "/sgs20/csb/release/las/release_build"
            set do_release = 1       # Distribute the software in this case!
        else
            set install_dir = "/sgs20/csb/release/las/${tag}_build"
            set debug_flag = 1
        endif
    else if ($system == "edcsns1") then
        if ($tag == "") then
            #--------------------------------------------
            # Allow no tag to mean release build, for now
            #--------------------------------------------
            set install_dir = "/edcsns1/csb2/release/release_build"
            set do_release = 1
        else
            set install_dir = "/edcsns1/csb2/release/${tag}_build"
            set debug_flag = 1
        endif
    else
        echo "Invalid build system $system"
        echo "Installation directory must be specified"
        goto USAGE
    endif
endif

# set the public defines needed for includes
if ($system == "edcsgs20") then
    set public_dir = "/sgs20/csb/software/build/public"
    set hdf_dir = "/sgs20/csb/software/build/hdf"
    set odl_dir = "/sgs20/csb/software/build/odl"
else if ($system == "edcsns1") then
    set public_dir = "/edcsns1/pkg/software/build/public"
    set hdf_dir = "/edcsns1/pkg/software/build/hdf"
    set odl_dir = "/edcsns1/pkg/software/build/odl"
else
    echo "Invalid build system $system"
    echo "Installation directory must be specified"
    goto USAGE
endif

#------------------------------------------------------------------
# Make sure the installation directory exists and if not, create it
#------------------------------------------------------------------
if (! (-e $install_dir)) then
   echo "Creating install directory $install_dir"
   mkdir $install_dir
endif

#--------------------------------------------
# Clean up the build installation directories
#--------------------------------------------
clean_install.csh $install_dir
if ($status) then
    echo "ERROR: Cleaning the installation directory $install_dir"
    exit 1
endif

#--------------------------------------
# Create a link to the public directory
#--------------------------------------
if (! -d $public_dir) then
    echo "The public library directory $public_dir does not exist"
    exit 1
endif
ln -s $public_dir $install_dir/build/public

#-----------------------------------
# Create a link to the hdf directory
#-----------------------------------
if (! -d $hdf_dir) then
    echo "The hdf library directory $hdf_dir does not exist"
    exit 1
endif
ln -s $hdf_dir $install_dir/build/hdf

#-----------------------------------
# Create a link to the odl directory
#-----------------------------------
if (! -d $odl_dir) then
    echo "The odl library directory $odl_dir does not exist"
    exit 1
endif
ln -s $odl_dir $install_dir/build/odl

#------------------------
# Spacing in the log file
#------------------------
echo ""

#-------------------------------------------------------
# Checkout TAE, LAS, and ADAPS, using the specified tag,
# from the CVS repository.
#-------------------------------------------------------
checkout_software.csh $install_dir $tag
if ($status) then
    echo "ERROR: Checking out TAE/LAS/ADAPS"
    exit 1
endif

#---------------------------------------------------------------
# Copy the enet include directory to the correct build directory
#---------------------------------------------------------------
cp $install_dir/src/enet/include/*.h $install_dir/build/enet/include

#------------------------
# Spacing in the log file
#------------------------
echo ""

#--------------------
# Build TAE/LAS/ADAPS
#--------------------
taelas_build.csh $install_dir $debug_flag
if ($status) then
    echo "ERROR: Building TAE/LAS/ADAPS"
    exit 1
endif

#------------------------
# Spacing in the log file
#------------------------
echo ""

#-------------------------------------
# Only do the following when releasing
#-------------------------------------
if ($do_release == 1) then
    #-------------------------------------------------
    # Identify obsolete files in the release directory
    # * The release directory is relative to $EDCSOFT
    #-------------------------------------------------
    check_obsolete.csh $install_dir
    if ($status) then
        echo "WARNING: Error checking for obsolete files in $install_dir"
    endif

    #------------------------
    # Spacing in the log file
    #------------------------
    echo ""

    #------------------------------------------------------
    # Copy new files to the release directory after copying
    # the release files to the old directory
    # * The release directory is relative to $EDCSOFT
    #------------------------------------------------------
    release_software.csh $install_dir -report_only
    if ($status) then
        echo "ERROR: Releasing software from $install_dir"
        echo "ERROR: May be a partial release -- Review logs"
        exit 1
    endif
endif

#--------------------------
# Start the LAS benchmark 
#--------------------------
# set the user to be mailed with the results of the benchmark
if (! $?LASBENCH_MAIL) then
    setenv LASBENCH_MAIL "tbeckman@usgs.gov"
endif
echo ""
echo "Starting LAS benchmark on $system at `date`"
$install_dir/src/las/base/app/lasbench/lasbench.csh $install_dir $install_dir

#---------------------------
# Run the automated testing.
#---------------------------
echo "Testing LAS/ADAPS in $install_dir/src"
# make sure EDCSOFT is set for the directory currently being built
setenv EDCSOFT $install_dir
cd $install_dir/src
Build.las -test >& test.log
if ($status) then
    echo "ERROR: Testing LAS/ADAPS"
    exit 1
endif
echo "LAS/ADAPS testing completed successfully.  (Does not refer to the"
echo "     standalone benchmarks.)"

exit 0

USAGE:
cat << EOM
usage: build_software.csh <tag> <install_dir>

    Check out TAE, LAS, and ADAPS software, with the specified <tag>, from the
    CVS repository. A release or test build of the software will be done on the
    main build systems (edcsgs20 and edcsns1) based on <tag>.  The "TEST" <tag>
    indicates that a test build is desired.  No <tag> specifies a release build
    is being done and obsolete files will be identified and logged and new
    files will be released to the release directory (relative to $EDCSOFT).

    <tag> - The tag to be used when checking out the software from CVS.
            Tag to be used to check out the source from the CVS repository.
            When no Installation directory is specified:
                The tag will build the TAE/LAS/ADAPS software in a
                "<tag>_build" directory with the debug options when compiled
                and will NOT distribute the software to other systems.

                Specifying no tag,  will build the TAE/LAS/ADAPS software in
                the "release_build" directory and WILL distribute the software
                to other associated systems.

            To specify no tag use an empty string (double quotes "") or leave
            the parameter off the command line.

    <install_dir> - Installation directory.  This is an optional parameter
            that specifies the location of the the software build.  The source
            code will be checked out, with the specified tag, to this
            directory and built.  The software will NOT be distributed to
            other systems when this parameter is specified.
EOM
exit 1
