#!/bin/csh
#
# FUNCTION:     checkout_software.csh
#
# PURPOSE:      Checkout TAE, LAS, and ADAPS software from CVS, using the
#               specified tag, into a specified directory.
#
# Usage:
#       -  Option to only checkout -tae or -las or -all
#   $1  -  Directory to checkout the software to
#   $2  -  Tag to be used to check out the source from the CVS repository
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development
# T. Beckmann      07/00  Added support to checkout a tag from CVS

#------------------------------------------------------
# Verify the correct number of arguments were specified
#------------------------------------------------------
if ( $#argv < 1 ) then
    goto USAGE
endif

#------------------------------------------------------
# Save the input arguments to meaningful variable names
#------------------------------------------------------
set checkout_dir = $1
set source_dir = $checkout_dir/src      # for LAS/ADAPS
set tag = $2

#------------------------------------------
# Make sure the checkout directories exists
#------------------------------------------
if (! (-d $checkout_dir)) then
   echo "Checkout directory $checkout_dir does not exist"
   echo ""
   goto USAGE
endif

if (! (-d $source_dir)) then
   echo "The LAS/ADAPS checkout directory $source_dir does not exist"
   echo ""
   goto USAGE
endif

echo "Checking out TAE from the CVS repository"
#-------------------------------------------------------------------------
# Check out the TAE source from CVS for building.
# The tag is not used # at this time since TAE does not change very often.
#-------------------------------------------------------------------------
cd $checkout_dir
cvs co tae >& checkout.log
if ($status) then
    echo "ERROR: Checking out TAE"
    exit 1
endif
if ($tag != "") then
    echo "-------------------------------------"
    echo "Checking out tag $tag" >>& checkout.log
    echo "-------------------------------------"
    checkout_tag.pl tae $tag >>& checkout.log
    if ($status) then
        echo "ERROR: Checking out tae with tag $tag"
        exit 1
    endif
endif


echo "Checking out LAS/ADAPS from the CVS repository"
#-------------------------------------------------------------
# Check out LAS/ADAPS source, starting with the trunk version
# then updating with the specified tag, from CVS
#-------------------------------------------------------------
cd $source_dir
cvs co adaps bin las enet >& checkout.log
if ($status) then
    echo "ERROR: Checking out LAS/ADAPS"
    exit 1
endif

if ($tag != "") then
    echo "-------------------------------------"
    echo "Checking out tag $tag" >>& checkout.log
    echo "-------------------------------------"
    foreach dir (adaps bin las enet)
        checkout_tag.pl $dir $tag >>& checkout.log
        if ($status) then
            echo "ERROR: Checking out $dir with tag $tag"
            exit 1
        endif
    end
endif

echo "TAE/LAS/ADAPS checkout completed successfully"
exit 0

USAGE:
cat << EOM
usage: checkout_softare.csh <checkout_dir> [<tag>]

       Checkout TAE, LAS, and ADAPS software from the CVS repository, with the
       specified <tag>, into <checkout_dir>.  The output from the CVS checkout
       will be captured in the "checkout.log" file in the <checkout_dir>/tae
       directory for TAE, and in the <checkout_dir>/src directory for LAS and
       ADAPS.

       <checkout_dir> - Directory to checkout source from the CVS repository.
                        This directory must exist and contain the directories
                        needed for the checkout/installation.
                        (<checkout_dir>/tae and <checkout_dir>/src).

                        Run "lasdir_setup.csh" in the <checkout_dir> to create
                        the proper directories for checkout/installation.

       [<tag>] - Tag to be used when checking out source from the CVS
                 repository.  If <tag> is not specified, the main trunk
                 is checked out.
EOM
exit 1
