#!/bin/csh
#
# FUNCTION:     dist_one_system.csh
#
# PURPOSE:      Distribute the released binaries, relative to $EDCSOFT, to a
#               specified system.
#
#         NOTE: This function assumes the release directories are relative to 
#               $EDCSOFT on both the current and destination systems.
#
# Usage:
#  $1 - System to distribute released binaries to.
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

unalias cd
#-----------------------
# Set up shell variables
#-----------------------
set sys_err    = 0
set usr        = release
set twoline  = "=================================="

#--------------------
# Verify the argument
#--------------------
if ($#argv < 1) then
    cat << EOM - 
    Usage: dist_one_system.csh <system>
        Distribute the binaries in the release directory (relative to \$EDCSOFT)
        to the specified destination <system>.
EOM
    exit 1
endif

set system = $1
set current = `pwd`

#---------------------------------
# Check to see if the system is up
#---------------------------------
ssh -q $system -l $usr -n date > /dev/null
if ($status) then
    echo "ERROR: $system is down"
    echo "       Please distribute to this system when it is up."
    exit 1
endif

echo "Distributing files to $system"
echo "$twoline$twoline"

#------------------------------------------------
# Distribute the include, libraries, and binaries
#------------------------------------------------
foreach dir ( \
    build/las/include \
    build/adaps/include \
    build/las/lib \
    build/adaps/lib \
    run/adaps/base \
    run/adaps/contrib \
    run/adaps/local \
    run/las/base \
    run/las/contrib \
    run/las/ditto \
    run/las/local \
    run/bin \
    env/las/tables \
    env/adaps/tables \
    env/las/assign \
    )
    #---------------------------------------------------------
    # Change to the directory to distribute the binaries from
    #---------------------------------------------------------
    cd $EDCSOFT/$dir
    echo "    Copying files from $EDCSOFT/$dir"

    #---------------------------------------
    # Remove the temp copy file if it exists
    #---------------------------------------
    if (-e copy.files) then
        /bin/rm copy.files
    endif

    #------------------------------------------------------------------------
    # for the adaps tables directories, do not copy the .dat, .timcor or
    # .lock files since sgs20's release directories are the master on those 
    # files
    #------------------------------------------------------------------------
    if ($dir == "env/adaps/tables") then
        foreach file (*)
            if ((! -l $file) && (! -d $file) && ($file != "copy.files")) then
                echo $file | awk 'n=0; /\.dat$/ {n = 1}; /\.timcor$/ {n=1}; \
                           /\.lock$/ {n=1}; /\.stop$/ {n=1}; \
                           n<1 {print $0};' >> copy.files
            endif
        end
    else
        #-----------------------------------
        # Weed out the links and directories
        #-----------------------------------
        foreach file (*)
            if ((! -l $file) && (! -d $file) && ($file != "copy.files")) then
                echo $file >> copy.files
            endif
        end
    endif

    #-----------------------------------------------------
    # Copy the binaries from the current directory to the
    # same directory on the associated system.
    #-----------------------------------------------------
    scp -qp `cat copy.files` ${user}@${system}:'$EDCSOFT'/$dir
    if ($status) then
        echo "ERROR: Copy of files from $EDCSOFT/$dir did not complete"
        set sys_err = 1
    else
        echo "    Copying files completed successfully"
        /bin/rm copy.files
    endif

    #------------------------
    # Spacing in the log file
    #------------------------
    echo ""
end

#--------------------------------------
# Log an error or successful completion
#--------------------------------------
cd $current
if ($sys_err == 1) then
    echo "All or part of the distribution to $system did not succeed"
    echo ""
    exit 1
else
    echo "Distribution to $system completed successfully"
    echo ""
endif

exit 0
