#!/bin/csh
#
# FUNCTION: dlt_avhrr_stage.csh
#
# PURPOSE:  Distribute all of the AVHRR images in the DLT stage directory to
#	    the appropriate AVHRR image directory.  The directory scheme is
#	    based upon the scene ID.
#
#    edcsgs7: /archive/daac/avhrr/orbits/SN/YEAR/MN/DY
#
#	where   SN: NOAA Satellite Number (11, 12, 14, etc.)
#	      YEAR: Four character year of acquisition
#	        MN: Two character month of acquisition
#	        DY: Two character day of acquisition
#
#
#	    The directory structure as seen from edcsgs4 (NFS mount) is
#	    different from the actual directory structure on the DLT system.
#
#    edcsgs4: /daac/avhrr/...
#
#           The NFS mount from edcsgs4 may be used except for actual data
#           transmission.  The LAN should be used for actual data transfer
#           ("rcp") because it is faster than going through an NFS mount.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# J.Fenno          05/95  Development
# A.Vander Vorste  02/96  Modified to also distribute level1b images
# T.Baltzer        04/97  Modified so that it will use "rcp" from edcsgs4 to
#			  edcsgs7 rather using an NFS mounted disk to transfer
#			  the data.
# T.Baltzer        09/97  Modified so that Level-1B orbital segments are
#			  written to a "tmp" directory and copied by IMS from
#			  there to the DAAC DLT.
#


#-----------------
# Set up the shell
#-----------------
unalias cd
set linesize = 13796
set DAACDLTHOST = "edcsgs7"


#---------------------------------------------------------------------
# Determine the hostname.  Only edcsgs4 is a valid option.
#---------------------------------------------------------------------
set host = `uname -n`
if ("$host" != "edcsgs4") then
    echo "dlt_avhrr_stage.csh: error -- invalid system $host"
    goto USAGE
endif


#------------------------------------------------------
# Only certain users have permission to run this script
#------------------------------------------------------
set user = `whoami`
if (("$user" != "ops") && ("$user" != "opsavhrr") && ("$user" != "dtrans")) then
    echo "dlt_avhrr_stage.csh: error -- invalid user $user"
    goto USAGE
endif


#--------------------
# Check the arguments
#--------------------
if ($#argv != 0) goto USAGE


#-----------------------------------------
# Set up variables for the directory paths
#-----------------------------------------
set rootdirsgs4 = "/daac/avhrr"
set rootdirsgs7 = "/archive/daac/avhrr"


#----------------------------
# Move to the stage directory
#----------------------------
set stagedir = "/sgs4/km1/adaps/dltstage"
cd $stagedir


#----------------------------------------------------------
# Determine if there are any archive files in the directory
#----------------------------------------------------------
set count = `/bin/ls | egrep '^a.*\.arch$' | wc -l`
if ($count == 0) then
    goto level_1b
endif


#-----------------------------------------
# Examine all files in the stage directory
#-----------------------------------------
foreach arch (a[ghlo]*.arch)
    #---------------------------
    # Build the header file name
    #---------------------------
    set ahdr = `echo $arch | awk -F\. '{printf "%s.ahdr",$1}'`


    #-------------------------------------------------------------
    # If the associated header file does not exist, skip the image
    #-------------------------------------------------------------
    if (! -e $ahdr) continue


    #------------------------------------
    # Retrieve the size of the image file
    #------------------------------------
    set actual_size = `/bin/ls -l $arch | awk '{print $5}'`


    #--------------------------------------------------------------------
    # Determine what the size of the image should be.  This is done by
    # extracting the number of lines from the header file and multiplying
    # by the bytes per line.
    #--------------------------------------------------------------------
    set nlines = `dd if=$ahdr bs=80 skip=7 count=1 |& awk '(NR == 1){print $7}'`
    @ expected_size = $nlines * $linesize


    #-----------------------------------------------------------------
    # If the image isn't the expected size then skip the image for now
    #-----------------------------------------------------------------
    if ($actual_size != $expected_size) continue


    #---------------------------------------------------------
    # Determine the directory where the image should be placed
    #---------------------------------------------------------
    set sat  = `echo $arch | cut -c3-4 -`
    set mon  = `echo $arch | cut -c5-6 -`
    set day  = `echo $arch | cut -c7-8 -`
    set year = `echo $arch | cut -c9-10 -`
    if (($year > 74) && ($year < 100)) then
	set year = "19$year"
    else
	set year = "20$year"
    endif
    if ("$arch" =~ ao*) then
	set dstdirsgs4 = $rootdirsgs4/orbits/$sat/$year/$mon/$day
	set dstdirsgs7 = $rootdirsgs7/orbits/$sat/$year/$mon/$day
    else
	set dstdirsgs4 = $rootdirsgs4/scenes/$sat/$year/$mon/$day
	set dstdirsgs7 = $rootdirsgs7/scenes/$sat/$year/$mon/$day
    endif


    #-----------------------------------------------------
    # If the destination directory does not exist, make it
    #-----------------------------------------------------
    if (! -d $dstdirsgs4) then
	mkdir -p $dstdirsgs4
	if ($status) then
	    echo "dlt_avhrr_stage.csh: error creating $dstdirsgs4"
	    continue
	endif
	chgrp avhrr $dstdirsgs4
	chmod 775 $dstdirsgs4
	chmod g+s $dstdirsgs4
    endif


    #------------------------------------------------------
    # Remote copy the files to the destination directory.
    # Use "rcp" because it is faster than going through an NFS mount.
    # If it is not successfully transferred, leave it to try later.
    # Remove the local version once successfully copied.
    #------------------------------------------------------
    if (! -w $dstdirsgs4) then
	echo "dlt_avhrr_stage.csh: cannot write to $dstdirsgs4"
	continue
    endif
    rcp $arch $DAACDLTHOST":"$dstdirsgs7
    if ($status) then
	echo "dlt_avhrr_stage.csh: error transfering $arch"
    else
	rm -f $arch
    endif
    rcp $ahdr $DAACDLTHOST":"$dstdirsgs7
    if ($status) then
	echo "dlt_avhrr_stage.csh: error transfering $ahdr"
    else
	rm -f $ahdr
    endif
end




#----------------------------------------------------------
# Determine if there are any level1b files in the directory
# (Orbital segments)
#----------------------------------------------------------
level_1b:
set count = `/bin/ls | egrep '^ao.*\.done$' | wc -l`
if ($count == 0) then
    goto level_1b2
endif

set linesize = 14800
#-----------------------------------------
# Examine all Level-1B orbital segments in the stage directory
#-----------------------------------------
foreach level1b (ao*.l1b)
    #---------------------------
    # Build the header file name
    #---------------------------
    set done = `echo $level1b | awk -F\. '{printf "%s.done",$1}'`


    #-------------------------------------------------------------
    # If the associated done file does not exist, skip the image
    # because it hasn't been completely processed
    #-------------------------------------------------------------
    if (! -e $done) then 
	continue
    endif

    #---------------------------------------------------------
    # Determine the directory where the image should be placed.
    # Level-1B orbital segments are written to a temporary
    # space where IMS finds them and moves them onto the DAAC
    # DLT once they match them with their database info.
    #---------------------------------------------------------
    set dstdirsgs7 = /sgs7/disk3/orb_seg/tmp


    #------------------------------------------------------
    # Remote copy the file to the destination directory.
    # Use "rcp" because it is faster than going through an NFS mount.
    # If it is not successfully transferred, leave it to try later.
    # Remove the local version once successfully copied.
    #------------------------------------------------------
    rcp $level1b $DAACDLTHOST":"$dstdirsgs7
    if ($status) then
	echo "dlt_avhrr_stage.csh: error transfering $level1b"
    else
	rm -f $level1b
	rm -f $done
    endif

end


#----------------------------------------------------------
# Determine if there are any level1b files in the directory
# (Individual acquisitions)
#----------------------------------------------------------
level_1b2:
set count = `/bin/ls | egrep '^a.*\.done$' | wc -l`
if ($count == 0) then
    exit 0
endif



#-----------------------------------------
# Examine all individual acquisitions in the stage directory
#-----------------------------------------
foreach level1b (a[ghl]*.l1b)
    #---------------------------
    # Build the header file name
    #---------------------------
    set done = `echo $level1b | awk -F\. '{printf "%s.done",$1}'`


    #-------------------------------------------------------------
    # If the associated done file does not exist, skip the image
    # because it hasn't been completely processed
    #-------------------------------------------------------------
    if (! -e $done) then 
	continue
    endif

    #---------------------------------------------------------
    # Determine the directory where the image should be placed
    #---------------------------------------------------------
    set sat  = `echo $level1b | cut -c3-4 -`
    set mon  = `echo $level1b | cut -c5-6 -`
    set day  = `echo $level1b | cut -c7-8 -`
    set year = `echo $level1b | cut -c9-10 -`
    if (($year > 74) && ($year < 100)) then
        set year = "19$year"
    else
        set year = "20$year"
    endif
    set dstdirsgs4 = $rootdirsgs4/scenes/$sat/$year/$mon/$day
    set dstdirsgs7 = $rootdirsgs7/scenes/$sat/$year/$mon/$day


    #-----------------------------------------------------
    # If the destination directory does not exist, make it
    #-----------------------------------------------------
    if (! -d $dstdirsgs4) then
        mkdir -p $dstdirsgs4
        if ($status) then
            echo "dlt_avhrr_stage.csh: error creating $dstdirsgs4"
            continue
        endif
        chgrp avhrr $dstdirsgs4
        chmod 775 $dstdirsgs4
        chmod g+s $dstdirsgs4
    endif


    #------------------------------------------------------
    # Remote copy the file to the destination directory.
    # Use "rcp" because it is faster than going through an NFS mount.
    # If it is not successfully transferred, leave it to try later.
    # Remove the local version once successfully copied.
    #------------------------------------------------------
    if (! -w $dstdirsgs4) then
        echo "dlt_avhrr_stage.csh: cannot write to $dstdirsgs4"
        continue
    endif
    rcp $level1b $DAACDLTHOST":"$dstdirsgs7
    if ($status) then
	echo "dlt_avhrr_stage.csh: error transfering $level1b"
    else
	rm -f $level1b
	rm -f $done
    endif

end




#------------------
# Normal completion
#------------------
exit 0



USAGE:
    onintr -
    echo 'usage: dlt_avhrr_stage.csh'
    echo ''
    echo '       Move all of the AVHRR archive images from the stage directory'
    echo '       on the DLT system to the appropriate AVHRR image directory.'
    echo '       This function should be run as a cron job to ensure all AVHRR'
    echo '       images in the stage directory are distributed for user access'
    echo '       on a timely basis.  This function can only be run on edcsgs4'
    echo '       as ops, opsavhrr, or dtrans.  To run on another'
    echo '       system or as another user, please contact software'
    echo '       engineering.'
    exit 1
