#!/bin/csh
#
# FUNCTION: las_compile.csh
#
# PURPOSE:  Compile a LAS application on all supported systems.
#           This function will fail if any one of the target systems is down
#           or if any of the makes fail.  To determine if a make fails,
#           standard output is searched for one of three phrases, "ERROR",
#           "Error code", or "Fatal error".  If any one of these phrases
#           exist in the output, it is assumed that the make failed.
#
#    NOTE:  It is up to the user to remove the necessary files from the
#           systems on which the make command succeeded before the failure.
#
#    NOTE:  This function assumes the current working directory contains
#           all of the necessary source code.  The function
#           compile_one_system.csh is called to actually move the source
#           code to the target system and compile it.
#
#
# Programmer      Date  Modifications
# --------------- ----- --------------------------------------------------------
# J.Fenno         04/94 Development
# J.Fenno         06/94 Changed the name of the function and modified so the
#                       first argument is the destination directory.
# J.Fenno         08/94 Added code to compile on all systems at the same time
#                       and to accept a command line argument indicating
#                       which systems to build on.
# J.Fenno         12/94 Added the documentation directory as an additional
#                       argument.  This will allow us to distribute all of the
#                       application documentation to each system.
# J.Fenno         03/95 Added sgs6 as a system with it's own version of LAS.
# J.Fenno         06/95 Added code for aircraft software
# J.Fenno         08/95 Modified to allow programmers to commit on Thursday
#                       through Sunday but the applications will not be
#                       pre-released until the second Monday afterwards.
# J.Fenno         10/95 Modified so LAS and ADAPS documentation is only built
#                       on $doc_sys.  This refers to the documentation in
#                       src/las/base/doc and src/adaps/base/doc.  The
#                       documentation for individual applications is only
#                       built on $doc_sys but this is taken care of in
#                       compile_one_system.csh.
# J.Fenno         11/95 Having router problems.  Changed sgs4 and sgs6 to
#                       edcsgs4 and edcsgs6.
# J.Fenno         11/95 Added the capability to define the environment
#                       variable CVSNODISTRIBUTE so this script will not ask
#                       the programmer if the source should be distributed to
#                       all systems.  It will only be checked into the
#                       repository if this is defined.
# D.Etrheim       07/96 Modified to build to "release/func_name" to allow for 
#                       "forced" distributions
# L. Johnson      04/98 Added check so that DESIGN subdirectories will be
#                       ignored (not compiled).
# J. Willems      06/98 Added edcsgs18
# T. Beckmann     04/00 Removed edcsgs6 since it was removed from service
# D. Hollaren     04/00 Removed distributing code checked in after Thursday
#                       to the nextweek directory.
# T. Beckmann     06/13 Added check to ignore test directory

#-------------------------------------------------------------------------------
# Constants
#    copyonly:   File containing the name of each file that must be
#                copied to the remote systems for compilation.  This is
#                only used when compiling support libraries.
#    hline:      Half a line of equal characters.  Used as a visual
#                separator.
#    target_sys: List of target systems to compile source code on.
#    usr:        Name of the account under which the source code should
#                be compiled.
#    assoc_?:    Names of associated systems where the compiled code
#                should be copied to.  This eliminates the need to
#                compile on some systems.
#-------------------------------------------------------------------------------
set copyonly   = copy.these.supports.only
set hline      = "========================================"
set doc_sys    = edcsgs20
set target_sys = (edcsns1 edcsgs20)
set usr        = release
set assoc_sun  = ()
set assoc_sgs20  = ()


#-------------------------------------------------
# Ensure there are the correct number of arguments
#-------------------------------------------------
if ((("$1" =~ -*) && ($#argv < 4)) || ($#argv < 3)) then
    cat << EOM -
usage: las_compile.csh [opt] <dstdir> <docdir> <repository> [file...]

       Compile one LAS or ADAPS application.  This funciton was designed to be
       called by the CVS functions when an application is committed to the
       repository, however, it can be called interactively to compile an
       application on all systems.

       Valid options are -none, -all, -edcsgs20, and -edcsns1.  If one or more
       of these options are specified, the user will not be prompted for the
       target systems.  If the environment variable \$CVSNODISTRIBUTE is
       defined, the source code will be checked into the repository and will
       not be distributed to any system.  \$CVSNODISTRIBUTE will override any
       command line arguments.

       <dstdir> must be the destination directory relative to \$EDCSOFT.  The
       release subdirectory is appended to the specified directory.

       <docdir> must be the documentation directory relative to \$EDCSOFT.

       <repository> is location of the source code.  This is automatically
       supplied by CVS but is ignored by this command.
EOM
    exit 1
endif

#---------------------------------------------
# Check for the definition of $CVSNODISTRIBUTE
#---------------------------------------------
if ($?CVSNODISTRIBUTE) then
    cat << EOM -
\$CVSNODISTRIBUTE is defined.  Source will not be distributed to any system.
EOM
    exit 0
else
    # Check to see if the client is local or remote.
    tty -s
    if ($status == 1) then        # remote access
        if (-e ~/.cvsremotedata) then
            # Extract remote information from file.
            #--------------------------------------

            # Get the repository.
            # Verify that the repository agrees with that given in
            # the remote data file.
            set remoterepository = \
                      `grep Repository: ~/.cvsremotedata | awk '{print $2}'`
            set repository = $3
            if ("$remoterepository" != "$repository") then
                cat << EOM -
ERROR: The .cvsremotedata file is incompatible with this commit process.
       The remote repository does not agree with the present repository.
       Removing .cvsremotedata.
EOM
                rm ~/.cvsremotedata
                exit 1
            endif


            # Check the age of the file to make sure that it isn't an old
            # file from a previous run. (< 15 minutes old)
            fileage.pl ~/.cvsremotedata 15
            if ($status) then
                echo "ERROR: .cvsremotedata file more than 15 minutes old."
                exit 1
            endif

            # Get systems to distribute to.  This is a future enhancement
            # and is commented out for now.
#            set systems = ""
#            set insystems = `grep "Build platforms:" ~/.cvsremotedata | awk '{print $3 " " $4 " " $5 " " $6}'`
#            foreach sys ($insystems)
#                if ($sys == "none") then
#                    echo "Source code will not be distributed to any system."
#                    exit 0
#                else if ($sys == "all") then
#                    set systems = ($target_sys)
#                    break
#                else
#                    foreach tsys ($target_sys)
#                        if ($sys == $tsys) then
#                            set systems = ($systems $sys)
#                            set good
#                        endif
#                    end
#
#                    if ($?good) then
#                        unset good
#                    else
#                        echo "ERROR: Invalid system specified ($sys)"
#                        exit 1
#                    endif
#                endif
#            end

            exit 0

        else  # ~/.cvsremotedata does not exist.
            echo "Use remotecommit for remote checkins."
            exit 1
        endif
    endif     # end of remote access block
endif

#----------------------------------------------------
# Unset noclobber and create an alias that will issue
# a message to standard output and log it in a file.
#----------------------------------------------------
unset noclobber
alias msg 'echo \!*; echo \!* >> $logfile'

#-----------------------------------------------------------------------------
# Create a unique log file name and a temporary file name.  The temporary file
# name will be used to catch the standard output from the compile command on
# each target system.
#-----------------------------------------------------------------------------
set stamp = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set logfile = "compilelog.$stamp"

#---------------------------------------------------
# Determine if the user has specified system options
#---------------------------------------------------
if ("$1" =~ -*) then
    set systems = ""
    while ("$1" =~ -*)
	    set sys = `echo $1 | awk -F- '{print $2}'`
        if ($sys == "none") then
	        echo "Source code will not be distributed to any system."
	        exit 0
        else if ($sys == "all") then
            set systems = ($target_sys)
            shift
            break
        else
	        foreach tsys ($target_sys)
	            if ($sys == $tsys) then
                    set systems = ($systems $sys)
                    set good
	            endif
            end

            if ($?good) then
                unset good
                shift
            else
                echo "ERROR: Invalid system specified ($sys)"
                exit 1
            endif
        endif
    end
endif

#------------------------------------------------------------------------------
# The first argument is the destination directory relative to the environment
# variable $EDCSOFT.  The second argument is the documentation directory
# relative to $EDCSOFT, and the third argument is the repository path that is
# being updated.  All three of these arguments should be shifted from the
# argument list once they have been retrieved.  The remaining arguments will be
# the names of the modified files.
#------------------------------------------------------------------------------
set dstdir = "$1"
set docdir = "$2"
set repository = "$3"
shift
shift
shift

#------------------------------------------------------------------------------
# Change the user to ddps for those apps DDP is releaseing
#------------------------------------------------------------------------------
if ("$dstdir" == run/las/ddp) then
   set usr = ddps
endif

#------------------------------------------------------------------------------
# The documentation for LAS and ADAPS should only be built on $doc_sys.  This
# system is the WWW server for the online documentation.  The documentation for
# individual applications is built only on $doc_sys as well, but this is taken
# care of in compile_one_system.csh.
#------------------------------------------------------------------------------
if (("$dstdir" == "run/adaps/doc") || ("$dstdir" == "run/las/doc") || \
    ("$dstdir" == "../release/doc")) then
    #------------------------------------------------------------------------
    # Issue a message to the user.  The commitinfo file should contain a
    # command line option to force compilation on sgs20 and therefore, the user
    # doesn't get any feedback until the documentation is installed.
    #------------------------------------------------------------------------
    msg 'Installing documentation in $EDCSOFT'"/$dstdir on $doc_sys"

    #-----------------------------------------
    # Distribute the documentation on $doc_sys
    #-----------------------------------------
    ssh -q $doc_sys -l $usr \
        distribute_doc.csh `uname -n` `pwd` $dstdir >& $logfile

    #----------------------------------------------------------------
    # Check for one of the valid error messages in the temporary file
    #----------------------------------------------------------------
    egrep -s "ERROR|Error code|Fatal error|fatal error" $logfile
    if ($status) then
	    msg "Documentation installed successfully on $doc_sys."
    else
	    msg "Error installing documentation on $doc_sys."
	    goto FATAL
    endif

    #----------------------
    # Complete successfully
    #----------------------
    exit 0
endif

#-----------------------------------------------------------------------
# Certain subdirectories should never be compiled on all systems.  It is
# assumed that these are subdirectories for an application and have been
# distributed with the main application directory.
#-----------------------------------------------------------------------
if (("$repository" =~ *las/base/app/tiepts/*) || \
    ("$repository" =~ *adaps/base/app/xplot/*) || \
    ("$repository" =~ *las/base/app/xid/*) || \
    ("$repository" =~ *las/base/app/arc2las/*) || \
    ("$repository" =~ *las/local/app/qlprinter/*) || \
    ("$repository" =~ *mosaic/*)) then
    echo "las_compile.csh: Subdir not to be compiled by itself."
    exit 0
endif

#-----------------------------------------------------------------------
# The DESIGN and DOC(S) subdirectories never needs to be compiled. 
# Therefore, exit if $repository is the DESIGN or DOC(S) directory.
#-----------------------------------------------------------------------
if (("$repository" =~ *DESIGN*) || ("$repository" =~ *DOC*) || \
    ("$repository" =~ */test)) then
    exit 0
endif

#---------------------------------------------------------------------------
# If the user did not specify the systems on the command line, interactively
# determine which systems to distribute the source code to.
#---------------------------------------------------------------------------
if (! $?systems) then
    set tmp = ""
    while (("$tmp" !~ y*) && ("$tmp" !~ n*))
	    echo -n 'Distribute to ALL systems [y/n/none]: '
	    set tmp = $<
    end

    if ("$tmp" == "none") then
	    echo "Source code will not be distributed to any system."
	    exit 0
    else if ("$tmp" =~ y*) then
	    set systems = ($target_sys)
    else
	    set systems = ""
	    while ($#target_sys)
	        set tmp = ""
	        while (("$tmp" !~ y*) && ("$tmp" !~ n*))
		        echo -n "Distribute to $target_sys[1] "'[y/n]: '
		        set tmp = $<
	        end

	        if ("$tmp" =~ y*) then
		        set systems = ($systems $target_sys[1])
	        endif

	        shift target_sys
	    end

	    if ("$systems" == "") then
	        echo "Source code will not be distributed to any system."
	        exit 0
	    endif
    endif
endif

#--------------------------------
# Check the status of each system
#--------------------------------
onintr ABORT
msg "Checking status of systems"
foreach sys ($systems)
    #--------------------------------------------------------------
    # If a system is down, force the user to commit at a later time
    #--------------------------------------------------------------
    ssh -q $sys -l $usr date > /dev/null
    if ($status) then
	    goto SYSDOWN
    endif
    msg "    $sys is up"

    #-------------------------------------------------------------------
    # If running as ddps, make sure associated systems are up since they 
    # distibute to all sytems at this time
    #-------------------------------------------------------------------
    if ($usr == "ddps") then
        if ($sys == "edcsgs20") then
            set assoc = ($assoc_sgs20)
        else if ($sys == "edcsns1") then
            set assoc = ($assoc_sun)
        else
            set assoc = ""
        endif
        foreach sys ($assoc)
            ssh -q $sys -l $usr date > /dev/null
            if ($status) then
                goto SYSDOWN
            endif
            msg "    $sys is up"
        end
    endif
end

msg "All systems are up"
msg " "

#---------------------------------------------------------------------------
# If we are going to be compiling a support save the names of the files that
# must be copied to the target systems in the $copyonly file.  This will
# prevent us from recompiling source code that hasn't changed.
# This file is created in the user's directory that the "cvs commit" was run.
#---------------------------------------------------------------------------
if (("$dstdir" == "build/adaps/lib") || \
    ("$dstdir" == "build/aircraft/lib") || \
    ("$dstdir" == "build/elips/lib") || \
    ("$dstdir" == "build/las/lib")) then
    /bin/ls | egrep 'Makefile|\.h$' > $copyonly
    foreach file ( $* )
 	    echo $file >> $copyonly
    end
    sort -u $copyonly -o $copyonly
endif

#----------------------------------------------------------
# Add the release subdirectory to the destination directory
# for BASE, CONTRIB, DITTO, and LOCAL applications.
#----------------------------------------------------------
if (("$dstdir" == run/adaps/base)       || ("$dstdir" == run/adaps/contrib)  || \
    ("$dstdir" == run/adaps/ditto)      || ("$dstdir" == run/adaps/local)    || \
    ("$dstdir" == run/aircraft/base)    || ("$dstdir" == env/adaps/tables)   || \
    ("$dstdir" == run/aircraft/contrib) || \
    ("$dstdir" == run/aircraft/ditto)   || \
    ("$dstdir" == run/aircraft/local)   || \
    ("$dstdir" == run/elips/base)       || ("$dstdir" == run/elips/contrib)  || \
    ("$dstdir" == run/elips/ditto)      || ("$dstdir" == run/elips/local)    || \
    ("$dstdir" == run/las/base)         || ("$dstdir" == run/las/contrib)    || \
    ("$dstdir" == run/las/ditto)        || ("$dstdir" == run/las/local)) then
    set dstdir = "$dstdir/release"

    #---------------------------
    # Append the function name
    #---------------------------
    set ttt=`basename $repository`
    set dstdir = "$dstdir/$ttt"
    unset ttt
endif

#------------------------------------------------------------
# Compile the application on all systems.  Place each command
# in the background to speed compilation as much as possible.
#------------------------------------------------------------
foreach sys ($systems)
    #-------------------------------------------------
    # Determine the associated systems to copy code to 
    # when ddps code is being updated
    #-------------------------------------------------
    set assoc = ""
    if ($usr == "ddps") then
        if ($sys == "edcsgs20") then
            set assoc = ($assoc_sgs20)
        else if ($sys == "un1") then
            set assoc = ($assoc_sun)
        endif
    endif

    #-------------------------------------------------------------
    # Compile on target system and copy code to associated systems
    #-------------------------------------------------------------
    echo " "
    echo "Compiling on $sys"
    ssh -q $sys -l $usr compile_one_system.csh `uname -n` `pwd` \
        $dstdir $docdir $assoc >& run.$sys.$stamp ; mv run.$sys.$stamp \
        log.$sys.$stamp &
end

#----------------------------------
# Watch for each log file to appear
#----------------------------------
@ i = 0
while ($i < $#systems)
    @ i = 0
    sleep 5
    foreach sys ($systems)
        if (-e log.$sys.$stamp) then
            @ i += 1
        endif
    end
end

#-----------------------------------------------------------
# Now that all systems have completed, combine the log files
#-----------------------------------------------------------
foreach sys ($systems)
    echo "Compilation from $sys" >> $logfile
    echo "$hline$hline" >> $logfile
    cat log.$sys.$stamp >> $logfile
    /bin/rm -f log.$sys.$stamp
    echo "" >> $logfile
    echo "" >> $logfile
end

#----------------------------------------------------------------
# Check for one of the valid error messages in the temporary file
#----------------------------------------------------------------
egrep -s "ERROR|Error code|Fatal error|fatal error" $logfile
if ($status) then
    msg "Compile completed successfully on all system(s)."
else
    msg "Compile failed on at least one system."
    msg "Ensure files are cleaned up on all systems."
    goto FATAL
endif

#----------------------------------------------------
# Remove the file containing the names of each support
# file that must be copied to each system.
# This is the file in the user's directory.
#-----------------------------------------------------
/bin/rm -f $copyonly

#--------------
# End of script
#--------------
exit 0

#---------------
# System is down
#---------------
SYSDOWN:
    onintr -
    msg "ERROR: $sys is down"
    msg "       Please commit changes when the system is up."

#----------------------
# Fatal error condition
#----------------------
FATAL:
    onintr -
    /bin/rm -f $copyonly run.*.$stamp log.*.$stamp
    echo "Created log file $logfile"
    echo -n "[7mHit RETURN to continue:[m"
    set tmp = $<
    exit 1

#----------------
# Abort condition
#----------------
ABORT:
    onintr -
    /bin/rm -f $copyonly run.*.$stamp log.*.$stamp
    msg "Compile aborted by user"
    echo "Created log file $logfile"
    echo -n "[7mHit RETURN to continue:[m"
    set tmp = $<
    exit 1
