#!/bin/csh
#
# queuestat.csh
#
# Usage:    queuestat.csh pattern
#
# pattern:  Specify a pattern to be used in the search.  Follow the rules for
#           grep in specifying the pattern.
#
# Output:   All queue entries that contain the pattern
#
# Returns:  0 - at least one instance of the pattern was found
#           1 - no instances of the pattern were found
#
# This script checks each individual queue for the pattern.  If the pattern is
# found in a queue, the exit status flag is set to 0.  If the pattern is not
# found in any queue the exit status is 1.  All entries containing the pattern
# are sent to stdout.
#
# This script was introduced as a result of a change in batch queue processes.
# The new qstat command only lists the processes it contains when the individual
# batch queue is specified eg. qstat aingest.  To minimize changes in existing
# code, this script can be called instead of multiple qstat commands.  Note that
# this script will also work with the old batch queue system.  See CR 5595.
#-----------------------------------------------------------------------------

# Check for one command argument.  If it isn't there
# exit with an error condition and usage message.
#---------------------------------------------------
if ($#argv != 1) then
    echo "\n\tUsage: queuestat.csh pattern\n"
    exit 1
endif

# Get a list of all the queues.  Each of the named queues begin with the lower
# case letters a through z.  Ignore all others.
# Notes: The "grep -v empty" will suppress lines that report the queue is empty
# preventing duplicate entries in the list.  The "sub(/:/,"",$1)" command
# removes the colon from the queue names.  These commands are required if this
# script is used on systems that have the old batch queues implemented.
#------------------------------------------------------------------------------
set quelist = `qstat | grep -v empty | awk '/^[a-z]/ {sub(/:/,"",$1);print $1}'`

# set the exit flag to 1 (not found)
#-----------------------------------
@ exitflag = 1

# Check all queues and, if a queue contains the
# pattern, set the exit flag to 0 (found).  The
# loop will continue until all queues are checked
# so that each queue entry that matches the
# pattern is sent to standard output.
#------------------------------------------------
foreach queue ($quelist)
    qstat $queue | grep $1
    if ( $status == 0 ) then
	@ exitflag = 0
    endif
end

# Return the exit flag
#---------------------
exit $exitflag
