#!/bin/csh
#
# FUNCTION:     release_copy.csh
#
# PURPOSE:      Copy those files that are different between the specified new
#               build directories and the current release directories (relative
#               to $EDCSOFT) into an old directory.  After the copy to the old
#               directory the new file is copied to the release directory.  The
#               released file name is then written to a log file.
#
#         NOTE: This function assumes that the release directories are relative
#               to $EDCSOFT and that the release directory structure exists for
#               both $EDCSOFT and the specified new build directory.
#
# Usage:
#   $1  -  Directory to release files from.
#   $2  -  Directory to release files to
#   $3  -  Report only option
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

if ($#argv < 2) then
    cat << EOM - 
    Usage: release_copy.csh <new_dir> <release_dir> [-report_only]
        This function compares files in the <new_dir> directory with files in
        the <release_dir> directory.  When a difference is found the released
        file is copied to an old directory, under the <release_dir> directory,
        and the new file is copied to the <release_dir> release directory.

        The [-report_only] flag overrides the copying of files.  The files
        that are different or new are reported but NOT copied.
EOM
    exit 1
endif

#---------------------------
# Set up the shell variables
#---------------------------
unalias cd
set halfline = "-------------------------------------"

#--------------------------------------
# Setup directory paths to the software
#--------------------------------------
set new_path = $1               # Location of the new binaries to be released
set rel_path = $2               # Location to copy new binaries to
set old_path = $rel_path/old    # Location to copy old release binaries to
set use_old = 1                 # Default to moving previous versions to old dir

#----------------------------------------------------
# do not use an old directory for the doc directories
#----------------------------------------------------
if ("$new_path" =~ *run/las/doc*) then
    set use_old = 0
endif

#----------------------------
# Check for the report option
#----------------------------
set report_only = 0
if (($#argv == 3) && ("$3" == "-report_only")) then
    set report_only = 1
endif

#-----------------------------
# Verify directory paths exist
#-----------------------------
if (! -d $new_path) then
    echo "The new release directory $new_path does not exist"
    exit 1
else if (! -d $rel_path) then
    echo "The release directory $rel_path does not exist"
    exit 1
else if (($use_old == 1) && (! -d $old_path)) then
    echo "The old release directory $old_path does not exist"
    exit 1
else
    #--------------------------------------------------------------------
    # Replace binaries in the release directory that have changed with
    # the new build binaries.
    #--------------------------------------------------------------------
    cd $new_path
    echo "$halfline$halfline"
    echo "Releasing files from $new_path"
    echo "$halfline$halfline"

    #-------------------------------------------------
    # When there are files in the directory, copy them
    #-------------------------------------------------
    set nfiles = `/bin/ls | /bin/wc -l`
    if ($nfiles != 0) then
        foreach file (*)
            #--------------------------------------
            # Make sure the file is not a directory
            #--------------------------------------
            if (-d $file) then
                continue
            endif

            #----------------------------------------------------------------
            # When the new file doesn't exist in the release directory, copy
            # the new file to the release directory.  Otherwise, check to see
            # if the new and released files are different.
            #----------------------------------------------------------------
            if (-e $rel_path/$file) then
                diff $file $rel_path >& /dev/null
                if ($status) then
                    #-------------------------------------------------------
                    # File was different so copy the release file to the
                    # old directory and then copy the new file to release.
                    #-------------------------------------------------------
                    if ($report_only == 0) then
                        if ($use_old == 1) then
                            cp -p $rel_path/$file $old_path
                        else
                            set status = 0
                        endif
                        if ($status) then
                            echo "ERROR: Copying $rel_path/$file to $old_path"
                        else
                            cp -p $new_path/$file $rel_path
                            if ($status) then
                               echo "ERROR: Copying $new_path/$file to $rel_path"
                            else
                                echo "Released $file"
                            endif
                        endif
                    else
                        echo "Will release $file"
                    endif
                endif
            else
                #----------------------------------------------
                # Copy the new file into the release directory. 
                #----------------------------------------------
                if ($report_only == 0) then
                    cp -p $new_path/$file $rel_path
                    if ($status) then
                        echo "ERROR: Copying new $new_path/$file to $rel_path"
                    else
                        echo "Released new $file"
                    endif
                else
                    echo "Will release new $file"
                endif
            endif
        end
    else
        #----------------------------------------
        # Log a message if no files were released
        #----------------------------------------
        echo "No files to copy in $new_path"
        exit 0
    endif

endif

#-----------------------------------------------------
# Log that an error occured or a successful completion
#-----------------------------------------------------
echo "Copying from $new_path completed successfully"

exit 0
