#!/bin/csh
#
# FUNCTION:     release_software.csh
#
# PURPOSE:      Release files from a new build.
#               Copy those files that are different between the specified new
#               build directories and the current release directories (relative
#               to $EDCSOFT) into an old directory.  After the copy to the old
#               directory the new file is moved to the release directory.  The
#               released file name is then written to a log file.
#
#         NOTE: This function assumes that the release directories are relative
#               to $EDCSOFT and that the release directory structure exists for
#               both $EDCSOFT and the specified new build directory.
#
# Usage:
#   $1  -  Directory to release files from
#   $2  -  Report only option
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

if ($#argv < 1) then
    cat << EOM - 
    Usage: release_software.csh <newdir> [-report_only]
        Release files from the specified <newdir> directory to the release
        directories.  This function compares files in the directories under
        <newdir> with files in directories under the release directory
        (relative to \$EDCSOFT).  When a difference is found the released
        file is copied to an old directory and the new file is copied to the
        release directory.

        The [-report_only] flag overrides the copying of files.  The files
        that are different or new are reported but NOT copied.

        The files moved are logged in the $HOME/bldlog/release_tstamp.log file
EOM
    exit 1
endif

#---------------------------
# Set up the shell variables
#---------------------------
unalias cd
set twoline = "====================================="
set stamp   = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set system = `uname -n`
set log_file = "$HOME/bldlog/release_$stamp.log"
set rel_type = "Release"
set report_flag = ""

#----------------------------------------
# Verify the new release directory exists
#----------------------------------------
set new_release = $1
if (! -d $new_release) then
   echo "The specified release directory $new_release does not exist"
   exit 1
endif

#-------------------------------
# Check for the report only flag
#-------------------------------
if (($#argv == 2) && ("$2" == "-report_only")) then
    set report_flag = "$2"
    set log_file = "$HOME/bldlog/release_$stamp.rpt"
    set rel_type = "Report"
endif

echo "$rel_type build files on $system"
echo $twoline$twoline
echo "$rel_type build files on $system" >> $log_file
echo $twoline$twoline >> $log_file

#-----------------------------------------------------------------------
# Check for differences between the new files and the released files 
#   Note: the doc directories do not appear in the dist_one_system.csh
#         script since edcsgs20 is the only one maintained for documentation
#-----------------------------------------------------------------------
foreach dir ( \
	build/las/include \
	build/adaps/include \
	build/enet/include \
	build/las/lib \
	build/adaps/lib \
	run/adaps/base \
	run/adaps/contrib \
	run/adaps/local \
	run/las/base \
	run/las/contrib \
	run/las/ditto \
	run/las/local \
    run/bin \
    env/las/tables \
    env/adaps/tables \
    env/las/assign \
    run/las/doc/user \
    run/las/doc/user/local \
    run/las/doc/adaps/user \
    run/las/doc/adaps/user/local \
    )
    #--------------------------------------
    # Setup directory paths to the software
    #--------------------------------------
    set new_path = $new_release/$dir
    set rel_path = $EDCSOFT/$dir

    #----------------------------------------------------------------
    # Do a difference and copy the new files to the release directory
    #----------------------------------------------------------------
    release_copy.csh $new_path $rel_path $report_flag >> $log_file
    if ($status) then
        echo "ERROR: Release failed"
        echo "ERROR: Release failed" >> $log_file
        exit 1
    endif

    #------------------------
    # Spacing in the log file
    #------------------------
    echo "" >> $log_file
end

#----------------------------
# Log a successful completion
#----------------------------
echo "Release completed successfully"
echo "Release completed successfully"  >> $log_file
exit 0
