
#ifndef ARCHIO_H
#define ARCHIO_H

#include "model.h"
#include "dropped.h"

/*******************************************************************************
* NOTES:                                                                       *
*                                                                              *
*    1. ARCH_BLK should be as large as possible and never decreased.           *
*       bufread() uses this to calculate the maximum number of bytes to be     *
*       read from an archive tape.  If this decreases in size then there will  *
*       be tapes in the archive that bufread() will be unable to handle.       *
*                                                                              *
*    2. If ARCH_GAC or ARC_LAC change, the open() statements in c_arcopnrd()   *
*       and c_arcopnwrt() must also be changed.                                *
*******************************************************************************/
#define ARCH_BLK         2 /* Number of lines per tape block in the archive  */
#define ARCH_MNR       140 /* Number of bytes of Minor Frame data in archive */
#define ARCH_GAC      2728 /* Number of bytes of image data in GAC archive   */
#define ARCH_LAC     13656 /* Number of bytes of image data in LAC archive   */
#define MNR_WORDS      103 /* Number of 10 bit words of Minor Frame data     */
#define GAC_WORDS     2045 /* Number of 10 bit words of GAC image data       */
#define LAC_WORDS    10240 /* Number of 10 bit words of LAC image data       */

#define GAC_BYTES (ARCH_MNR + ARCH_GAC)  /* Bytes/archive line of GAC data   */
#define LAC_BYTES (ARCH_MNR + ARCH_LAC)  /* Bytes/archive line of LAC data   */
#define GAC_BLOCK (ARCH_BLK * GAC_BYTES) /* Bytes/block of GAC data          */
#define LAC_BLOCK (ARCH_BLK * LAC_BYTES) /* Bytes/block of LAC data          */


/*
  Define the structure used for AVHRR archive image I/O
*/
#define ARCBUFLINES (ARCH_BLK * 175)/* # of image lines in the I/O buffer    */
struct ARCDESC
    {
    int   fdesc;                /* File descriptor for images on disk        */
    long  access;               /* Type of access the image is opened for    */
    long  arcbytes;             /* Number of bytes/line in the archive image */
    long  current;              /* Next line number to return to the user    */
    long  first;                /* Number of the line in unpck_buf[0]        */
    long  last;                 /* Number of the last line in unpck_buf      */
    long  tpunit;               /* Tape descriptor for images on tape        */
    long  window[4];            /* Window used to open the archive image     */
    long  words_img;            /* Number of 10 bit words in the image data  */
    long  words_mnr;            /* Number of 10 bit words in the minor frame */
    short ascending;            /* Flag indicating ascending/descending pass */
    short disk;                 /* Flag indicating image is on disk or tape  */
    short *mnr;                 /* Unpacked minor frame (interface with user)*/
    short *samples;             /* Unpacked samples     (interface with user)*/
    short satview;              /* Flag for processing ascending passes      */
    short savebuf;              /* Flag indicating buffer needs to be saved  */
    short saveline;             /* Flag indicating current line needs saving */
    unsigned char *packed;      /* Packed buffer        (interface to file)  */
    short *swpbuf;      /* Temporary buffer for swapping             */
    };


/*
  Define the headings for the archive header
*/
#define CEOS_START   "/*CEOS_IEF"             /* CEOS_IEF header flag        */
#define CEOS_END     "/*END_IEF"              /* CEOS_IEF header end flag    */
#define SFL_START    "SFL_ARCH_HEAD_START"    /* SFL archive header flag     */
#define SFL_START_V2 "SFL_ARCH_HEAD_START_V2" /* SFL archive header flag     */
#define SFL_END      "SFL_ARCH_HEAD_END"      /* SFL archive header end flag */

#define META_LINES  2


/* Prototype functions to be called by any ADAPS support routine or module    */

int c_arclose
(
    char *image,	    /* IN  - Host file name or scene ID of image      */
    long *arcdesc           /* IN - Archive descriptor record                 */
);

int c_arcopnrd
(
    char *image,            /* IN     - Host file name or scene ID of image   */
    long *access,           /* IN     - IREAD or IUPDATE                      */
    long window[4],         /* IN/OUT - Window for opening the image          */
    struct MODEL *model,    /* IN     - AVHRR model structure                 */
    long *tpunit,           /* IN     - Tape unit.  NULL if image on disk     */
    long *arcdesc           /* OUT    - Archive descriptor record             */
);

int c_arcopnwrt
(
    char *image,            /* IN  - Host file name or scene ID of image      */
    long *dtype,            /* IN  - Data type of the image (GAC,HRPT,LAC)    */
    long *nlines,           /* IN  - Number of lines in the image	      */
    long *arcdesc           /* OUT - Archive descriptor record                */
);

int c_arcread
(
    long *arcdesc,      /* IN  - Archive descriptor record                    */
    short **mnr,        /* OUT - Buffer of I*2 minor frame data               */
    short **samples	/* OUT - Buffer of I*2 sample values                  */
);

int c_arcseek
(
    long *arcdesc,     /* IN - Archive descriptor record                     */
    long line          /* IN - Next line to be read by c_arcread()           */
);

int c_arcwrite
(
    long *arcdesc,     /* IN  - Archive descriptor record                    */
    short mnr[],       /* OUT - Buffer of I*2 minor frame data               */
    short samples[]    /* OUT - Buffer of I*2 sample values                  */
);

int c_getaddr
(
    char *host_img,		  /* Host input image name		     */
    struct MODEL *model		  /* Model and orbital information	     */
);

int c_getahdr
(
    char *arch_name,            /* IN  - Name of the archive image            */
    struct MODEL *model,        /* OUT - Model and orbital information        */
    struct DROPPED *drop	/* OUT - Archive dropped line information     */
);

int c_hdrclose
(
    int *fd		/* File descriptor to the associated archive header  */
);

int c_hdropen
(
int	*fd,		/* File descriptor to acquire image	*/
char	*file,		/* Acquisition file name		*/
long	*access 	/* Type of file access			*/
);

int c_hdrread
(
    char buffer[][HDR_SIZ+1], /* IN  - Buffer containing archive header     */
    struct MODEL *model,      /* OUT - Model and orbital information        */
    struct DROPPED *drop      /* OUT - Dropped line index information       */
);

int c_hdrwrite
(
int    *out_fd,         /* IN - File descriptor to archive header           */
struct MODEL   *model,	/* IN - Model and orbital information               */
struct DROPPED *drop 	/* IN - Dropped line index information              */
);

int c_putaddr
(
    char *host_img,		  /* Host input image name		     */
    struct MODEL *model		  /* Model and orbital information	     */
);

int c_putahdr
(
    char *arch_name,	  /* Name of the archive image		     */
    struct MODEL *model,  /* Model and orbital information	     */
    struct DROPPED *drop  /* Archive dropped line information        */
);

int c_putdrpl
(
    char *image,		/* Host file name of output image             */
    struct DROPPED *drop	/* Dropped line data structure                */
);

void c_setatt
(
    char buffer[][HDR_SIZ+1], /* IN  - Buffer containing ASCII header info   */
    struct MODEL *model	      /* OUT - Model and orbital information	     */
);

int c_setdrop
(
    char buffer[][HDR_SIZ+1],	/* IN  - Buffer holding dropped line info     */
    struct DROPPED *drop	/* OUT - Dropped line structure               */
);

void c_sethead
(
    char buffer[][HDR_SIZ+1], /* IN  - Buffer containing ASCII header info  */
    struct MODEL *model,      /* OUT - Model and orbital information	    */
    long version	      /* Version of the header			    */
);

void c_toatt
(
    struct MODEL *model,	/* IN  - Model and orbital information	     */
    char buffer[][HDR_SIZ+1]	/* OUT - Buffer ASCII header information     */
);

void c_todrop
(
    struct DROPPED *drop,       /* IN  - Dropped line index information       */
    char buffer[][DRP_SIZ+1],   /* OUT - Buffer containing dropped line info  */
    long *nbuf		        /* OUT - Number of dropped line buffers       */
);

void c_toephem
(
    struct MODEL *model,	/* Model and orbital information	     */
    char buffer[]		/* Asci buffer for ephemeris information     */
);

void c_tohead
(
    struct MODEL *model,	/* IN  - Model and orbital information	     */
    char buffer[][HDR_SIZ+1]	/* OUT - Buffer ASCII header information     */
);

void c_tometa
(
struct MODEL *model,                       /* IN  - Model and orbital info   */
char   meta_data[META_LINES][HDR_SIZ + 1]  /* OUT - Buffer for CEOS IEF      */
);

int c_tpgetahdr
(
    char *sceneid,          /* IN  - Scene ID of the archive image to read    */
    char *comment,          /* IN  - Tape mounting comment                    */
    long *tpunit,           /* OUT - Tape descriptor record                   */
    struct MODEL *model,    /* OUT - Model structure                          */
    struct DROPPED *drop    /* OUT - Dropped line structure                   */
);

#endif

