/******************************************************************************
FUNCTION:	GETALIAS

PURPOSE:	Searches the input string for a "$" (indicating an alias name),
		and returns the string with the alias text, if any, substituted
		for the alias name.
		Because the alias name has no termination character, it is
		assumed to be terminated by the end of the input string or
		by the occurrence of any non-alphanumeric character.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      10/86    C. Greenhagen	New Development
  1.1      12/87    B. Ailts		Change include dir. spec.
  5.0      04/88    D. Hollaren		changed include spec to las
					changed cerrmsg to c_errmsg and
					passed status code by address
					SUCCESS/FAIL to E_SUCC/E_FAIL
  5.1      09/88    B. Ailts		Expanded code to recognize the $$ as
					not an alias name
  5.2      03/89    B. Davis		commented out printf left from debugging
  5.3	   10/90    T. Sturman 		Added error checking for length of 
					output string before returning
  5.4      11/90    D. Etrheim		Corrected for alias of length 9.  Need
					ASZ+2 to handle lf and NULL chars.
					Changed declaration of addstring.
  5.1      12/90    D. Etrheim		Change nonfatal to 1.
  5.2      1/91     B. Davis		Standardized errmsg calls and status
					returns according to EDC guidelines.
  6.0      6/95     T. Wolbrink		Added string length checks prior
					to string concatenations to fix PR 33.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 

PROJECT			LAS

ALGORITHM 
Search for a "$" in the input string.
Copy any text preceeding the "$" into the output string.
Return if there were no aliases in the input string.
Get the user's ALIAS LIST and open it.
Parse the alias name from the input string
    Search the ALIAS LIST for a match
	substitute the alias text into the output string
    Copy any non-alias text into the output string
    Search for the next alias in the input string
Repeating the same steps for each alias in the input string.
Return the output string with all aliases substituted with there associated
alias text.

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <math.h>

#ifdef vms
#include <file.h>
#else
#include <sys/types.h>
#include <sys/file.h>
#endif
#ifdef sun
#include <fcntl.h>
#endif

#include "las.h"
#include "alias.h"

/**
 * c_getalias:
 * @instring:  I: input string
 * @outstring:  O: output string
 *
 * Searches the input string for a "$" (indicating an alias name),
 * and returns the string with the alias text, if any, substituted
 * for the alias name.
 * Because the alias name has no termination character, it is
 * assumed to be terminated by the end of the input string or
 * by the occurrence of any non-alphanumeric character.
 *
 * Returns: E_SUCC:  Successful completion.
 *          E_FAIL:  Unsuccessful completion.
 */
int c_getalias
(
    char *instring,			/* input string			*/
    char *outstring			/* output string		*/
)
{
#define ASZ	9			/* max length of alias name	*/
#define TSZ	CMLEN-1			/* max length of alias text	*/

static char
    alphanumeric[] =			/* alphanumeric characters	*/
    {"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"};

char
    aliasfile[CMLEN],			/* name of user's alias file	*/
    aliasname[ASZ+2],			/* alias name			*/
    aliastext[TSZ+1],			/* alias text			*/
    addstring[CMLEN],			/* string to accumulate text    */
    *tempstring,			/* temporary work string	*/
    *inptr;				/* pointer to input string	*/

int
    fd,					/* file descriptor		*/
    len,				/* string length 		*/
    morealias,				/* more aliases to search for	*/
    status;				/* cmlock status		*/

long nonfatal = 1;			/* nonfatal code for error message */

char msg[ERRLEN+1];			/* Error message string		*/

FILE *aliasf = NULL;			/* alias file			*/

len = strcspn (instring,"$");			/* length of string	*/
						/* preceding "$"	*/
if ((strlen (instring)) == len)  		/* no alias used	*/ 
    {
     if (len <  CMLEN) 
        {
          strcpy (outstring,instring);
          return (E_SUCC);
        }
     else
        {
          c_errmsg ("Text is too long","getalias-text",&nonfatal);
          return(E_FAIL);
        }
   }

if (strlen(instring) >= CMLEN) 
        {
        c_errmsg("Expanded alias text too long", "getalias-text", &nonfatal);
        return(E_FAIL);
        }
strncpy (addstring,instring,len);		/* copy 1st part of string */
addstring[len] = 0;

bldalias (aliasfile);			/* build alias file name*/
status = cmlock(aliasfile,&fd,O_RDONLY,60);	/* open alias file	*/
if (status == E_SUCC)
   aliasf = fdopen (fd,"r");

if ((status != E_SUCC) || (aliasf == NULL))
{
   sprintf(msg,"Error opening file %s",aliasfile);
   c_errmsg(msg,"getalias-open",&nonfatal);
   strcpy(msg,"Error returned from cmlock");
   c_errmsg(msg,"getalias-call",&nonfatal);
   return(E_FAIL);
}

morealias = 1;
inptr = instring;
tempstring = strchr (inptr,'$');		/* get alias name,	*/
if (tempstring == 0)
   morealias = 0;

while (morealias)
   {
      len = strspn(tempstring,"$");
/*
      printf ("tempstring = %s\n",tempstring);
      printf ("len = %d\n",len);
*/
      if (len == 2)
         {
            tempstring ++;
            len = strspn ((tempstring),"$"); /* name length	*/
	    if ( (strlen(addstring) + len) >= CMLEN )
	        {
        	c_errmsg("Expanded alias text too long", "getalias-text", &nonfatal);
        	return(E_FAIL);
        	}
            strncat(addstring,tempstring,len);
            inptr = tempstring + len;
         }
      else
         {
            len = strspn ((tempstring+1),alphanumeric)+1;/* name length	*/
            if (len > ASZ)
               {
                  c_errmsg ("Alias name too long","getalias-name",&nonfatal);
                  return(E_FAIL);
               }

            strncpy (aliasname,tempstring,len);
            aliasname[len] = 0;
            inptr = tempstring + len;

            if (findalias(aliasf,aliasname,aliastext) == E_SUCC)
		{
		if ( (strlen(addstring) + strlen(aliastext)) >= CMLEN)
        		{
        		c_errmsg("Expanded alias text too long", "getalias-text", &nonfatal);
        		return(E_FAIL);
        		}
               	strcat(addstring,aliastext);
		}
            else
               {
               c_errmsg("Specified alias not found","getalias-found",&nonfatal);
               return(E_FAIL);
               }
          }

      len = strcspn (inptr,"$");		/* copy non-alias text	*/
      strncat (addstring,inptr,len);		/* to outstring		*/

      tempstring = strchr (inptr,'$');	/* get next alias name 	*/
      if (tempstring == 0)
         morealias = 0;
      else
         rewind (aliasf);
      }

/*move addstring to outstring before returning */
strcpy (outstring,addstring);
fclose (aliasf);

return (E_SUCC);
}
