#!/bin/sh
# ******************************************************************************
# NAME        	    	            ANUDEM	
# 
# PURPOSE  XCORRECT performs along-line systematic corrections on a full-sized,
#	   non-geometrically corrected Landsat MSS CCT-X scene.  The image data
#	   is corrected for the following systematic errors:
#               1)  Line length adjustment
#	        2)  Detector-to-detector sampling delay
#          
# 	   After the systematic corrections are applied, the data is 
#          resampled along-line using the specified algorithm.
#
# This script is called by TAE with two parameters.
#	$1	$exec_name	The executable to use
#	$2	$tempfile	The temporary file redirected as input 
#				to exec_name
#           
# PROGRAMMER	  DATE		REASON
# ----------	  ----		------
# R. Kruse          1/93	PRF# 6124 New development digital elevation data
# 
# *****************************************************************************/

exec_name=$1
tempfile=$2
stdout_file=anudem.out$$
stderr_file=anudem.err$$
errno=0

touch $stdout_file $stderr_file
errno=$?
if [ ! $errno -eq 0 ]; then
	echo "[anudem-files]!Unable to create temporary files"
	exit -1
fi

# *****************************************************************************/
# Temporary variables for testing
#
# LASLOCAL="/sg2/csb/kruse/las/anudem"
#
# *****************************************************************************/

if [ -r "$tempfile" -a -x $LASLOCAL/$exec_name ]; then
	$LASLOCAL/$exec_name < $tempfile > $stdout_file 2> $stderr_file
	errno=$?
fi

if [ $errno -eq 0 ]; then
	# This occurs when the fortran program exits with success or failure
	# The C part of the program will return an errno.  The other instance
	# where errno is set, is when the program is not loaded successfully.
	# ie. not enough memory to run the program.
	if [ -s $stderr_file ]; then	# file exists an is not zero bytes
		# if this file exist and is not zero bytes then there where
		# errors in the input parameters.  The stderr will be tailed so
		# the user has some type of error message
		tail -11 $stdout_file
		cat $stderr_file
		exit 1
	else
		# The program executed successfully
		tail -10 $stdout_file | egrep "ANUDEM - SUCCESSFUL COMPLETION"
		errno=$?
		if [ $errno -eq 0 ]; then
		     rm $stderr_file $stdout_file
		     exit 0
		else
			tail -11 $stdout_file
		fi
	fi
else
	# This occurs under two conditions. 1) the program failed to load
	#				    2) LAS supports returned and errno
	# These can be separated by looking at the stdout file.
	# The conditions are as follows:
	#	LAS error messages will be printed at the tail end of the
	#	   stdout file.  Fatal errors will be searched for in the 
	#	   last 10 lines of the stdout file.
	#	If no errors exist then other LAS messages will be printed by
	#	searching for the "{" in the first column.

	# The next statment will print any error messages found which are 
	# of LAS origin.
        tail -10 $stdout_file | egrep "ANUDEM - SUCCESSFUL COMPLETION" > /dev/null
	errno=$?
	if [ $errno -eq 0 ]; then
		# Successful completion
		# printout informational messages
		tail $stdout_file | egrep -i "\[*\-*\]"
		exit 0
	elif [ $errno -eq 139 ]; then
		# not enough resources to run the program
		echo "[anudem-resources]!Not enough resources to run program"
		exit 1
	else
		tail -11 $stdout_file 
		cat $stderr_file
		exit 1
	fi

fi
