/****************************************************************************
FUNCTION:	acu_command

PURPOSE:	This is a simple interface to send commands to the ACU and
		get the ACU response.  Both command and response are printed
		on stdout.

USAGE:		acu_command [port command]

		If no command is issued the following is used:
		port 6100 command 'SYST:TIME,?'


****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>

static int execute_command( int, const char *, char * );
int pos( int, char * );
char *parse_string( const char *, const char *, int );
int change_access( int, int );
int change_axismode( int, char *, int );
int validate_response( const char *, const char *, char * );
float fparse_string( const char *, const char *, int );
int iparse_string( const char *, const char *, int );
char *get_err_msgs( int, int );
int stop_track( int );
int track_mode( int );
int stopfile( int );
void write_log( int, const char *, const char *, const char * );
int stow_elevation( int );

#define BUFLEN 1024
#define TRUE  1
#define FALSE !TRUE
#define AZ    "AXS1"
#define EL    "AXS2"
#define ADMIN  0
#define OPER   1
#define OBS    2
#define BADVAL 9999
#define CREATE 111
#define REMOVE 222

enum CMDS { SMOD, AMOD, APOS, STOW, ACCS0, ACCS1, ACCS2, POIN };

int main( int argc, char *argv[] )
{
    char IPaddress[25];		/* ACU IP address			*/
    char command[BUFLEN];	/* Command string			*/
    char err_msg[BUFLEN];	/* Command string			*/
    char portnum[10];		/* ACU port number			*/
    char acu_msg[BUFLEN];	/* Message returned by ACU		*/
    char msgs[9][51] = {
             { "SYST:SMOD,?" },
             { "OPNS:AMOD,?" },
             { "OPNS:APOS,?" },
             { "SYST:STOW,1,@" },
             { "CCFG:ACCS,0,Admin3880,@" },
             { "CCFG:ACCS,1,operator,@" },
             { "CCFG:ACCS,2,@" },
             { "MSSN:POIN:STOP,@" },
             { "" }};
    int n = 1;	/* Number of commands we want to run from msgs above	*/
    int  sk;			/* Socket descriptor			*/
    struct hostent     *hp;	/* host name of address			*/
    struct sockaddr_in skaddr; 	/* socket information structure		*/
    int i;
    int mode;
    char *s;
    int done, tries;
    int access_level = OBS;
    int axis_mode;
    char wm[BUFLEN];
    int status;


    /* Set up default IP address and port strings
    ---------------------------------------------*/
    strcpy( IPaddress, "192.168.10.3" );
    strcpy( portnum, "6100" );


    /* Create a socket
    ------------------*/
    if (( sk = socket( PF_INET, SOCK_STREAM, 0 )) < 0 )
    {
        write_log(TRUE,"stow","Error","Problem creating socket\n");
	return 1;
    }

    /* Set the client name and port
    -------------------------------*/
    skaddr.sin_family = AF_INET;
    if (( hp = gethostbyname( IPaddress )) != 0 )
        memcpy( &skaddr.sin_addr.s_addr, hp->h_addr, hp->h_length );
    else
    {
        if ( inet_aton( IPaddress, &skaddr.sin_addr ) == 0 )
	{
            sprintf( wm, "Invalid IP address: %s",IPaddress);
	    write_log(TRUE,"stow","Error",wm);
	    return 1;
	}
    }
    skaddr.sin_port = htons(atoi(portnum));


    /* establish the connection
    ---------------------------*/
    if (( connect( sk, (struct sockaddr *) &skaddr, sizeof( skaddr ))) < 0  )
    {
        write_log(TRUE,"stow","Error","Problem connecting socket");
	return 1;
    }

    stopfile( CREATE );
    /* Must be in operator or administrator access level */
    if ( change_access( sk, ADMIN ))
    {
        if ( change_access( sk, OPER ))
        {
            write_log(TRUE,"stow","Error",
               "Unable to enter proper access level!" );
            close( sk );
            stopfile( REMOVE );
            return 1;
        }
        else
        {
            write_log(FALSE,"stow","Access", "In operator access level" );
            access_level = OPER;
        }
    }
    else
    {
        write_log(FALSE,"stow","Access", "In administrator access level" );
        access_level = ADMIN;
    }
            
    /* Check to see if the antenna is still tracking.  If it is tell
    it to stop. */
    if ( stop_track( sk ))
    {
        write_log(FALSE,"stow","Error", "Unable to stop track" );
        /*
        change_access( sk, OBS );
        close( sk );
        return 1;
        */
        /* Give it a rest and try again */
        sleep(5);
        if ( stop_track( sk ))
        {
            write_log(FALSE,"stow","Error",
                "Still couldn't stop track, sleeping for 30" );
            sleep(30);
        }
    }


    /* Stow the elevation: put the antenna at 90 degrees */
    if ( pos( sk, EL ) != 90 )
    {
        if ( stow_elevation( sk ))
        {
            sleep( 2 );
            write_log(FALSE,"stow","Retry", "Trying to stow elevation again");
            if ( stow_elevation( sk ))
            {
                write_log(FALSE,"stow","Error", "Unable to stow in elevation");
            }
        }
#if 0
        /* Put the elevation in manual */
        if ( change_axismode( sk, EL, 1 ))
        {
            write_log(FALSE,"stow","Error",
                "Unable to change to manual elevation mode" );
        }
        else
        {
            tries = 0;
            if ( pos( sk, EL ) != 90 )
            {
                strcpy( command, "OPNS:POSN:AXS2,90.0,@" );
                execute_command( sk, command, acu_msg );
                if ( validate_response( command, acu_msg, err_msg ))
                {
                    sprintf( wm, "Elevation stow returned: %s", acu_msg );
                    write_log(FALSE,"stow","Information", wm );
                    tries = 15;
                }
            }
            while ( tries < 15 && pos( sk, EL ) != 90 )
            {
                sprintf( wm, "Elevation at: %d", pos( sk, EL ) );
                write_log(FALSE,"stow","Information", wm );
                sleep(2);
                tries++;
            }
            if ( tries == 15 )
            {
                sprintf( wm, "Unable to stow elevation.  Position is: %d",
                    pos( sk, EL ));
                write_log(FALSE,"stow","Error", wm );
            }

            /* Let it settle a bit */
            /* It would probably be better to wait for the rates to settle */
            /* but that will have to wait for the telemetry function */
            sleep(4);

            /* Put it in standby */
            if ( change_axismode( sk, EL, 0 ))
            {
                write_log(FALSE,"stow","Error",
                    "Unable to change to standby elevation mode" );
            }
        }
#endif
    }
    else
    {
        write_log(FALSE,"stow","Information",
            "Elevation already in stow position" );
    }
    sprintf( wm, "Elevation at: %d", pos( sk, EL ));
    write_log(FALSE,"stow","Information", wm );
    /* Put it in standby */
    if ( change_axismode( sk, EL, 0 ))
    {
        write_log(FALSE,"stow","Error",
            "Unable to change to standby elevation mode" );
    }
    else
    {
        write_log(FALSE,"stow","Information",
            "Elevation is in standby" );
    }


    /* Stow the azimuth: put the antenna at 0 degrees */
    /* Must be in AMDIN level to execute this command */
    if ( access_level != ADMIN )
    {
        write_log(FALSE,"stow","Error",
            "Can't change to admin access level, azimuth not stowed");
    }
    else
    {
        /* Put the azimuth in mancual mode */
        if ( change_axismode( sk, AZ, 1 ))
        {
            write_log(FALSE,"stow","Error",
                "Unable to change to manual azimuth mode" );
        }
        else
        {
            tries = 0;
            /* We may execute this command */
            strcpy( command, "OPNS:XPOS:AXS1,0.0,@" );
            execute_command( sk, command, acu_msg );
            if ( validate_response( command, acu_msg, err_msg ))
            {
                sprintf( wm, "ACU responded to stow azimuth with: %s",
                    acu_msg );
                write_log(FALSE,"stow","Error", wm );
                tries = 30;
            }
            while ( tries < 30 && pos( sk, AZ )%360 != 0 )
            {
                sprintf( wm, "Azimuth at: %d", pos( sk, AZ ) );
                write_log(FALSE,"stow","Information", wm );
                sleep(2);
            }
            if ( tries == 30 )
            {
                sprintf( wm, "Unable to stow azimuth.  Position is: %d",
                    pos( sk, AZ ));
                write_log(FALSE,"stow","Error", wm );
            }

            /* Let it settle a bit */
            /* It would probably be better to wait for the rates to settle */
            /* but that will have to wait for the telemetry function */
            sleep(4);

            /* Put it in standby */
            if ( change_axismode( sk, AZ, 0 ))
            {
                write_log(FALSE,"stow","Error",
                    "Unable to change to standby azimuth mode" );
            }
        }
    }
    sprintf( wm, "Azimuth at: %d", pos( sk, AZ ));
    write_log(FALSE,"stow","Information", wm );
    /* Put it in standby */
    if ( change_axismode( sk, AZ, 0 ))
    {
        write_log(FALSE,"stow","Error",
            "Unable to change to standby azimuth mode" );
    }
    else
    {
        write_log(FALSE,"stow","Information", "Azimuth is in standby" );
    }





    /* Downgrade to observer level */
    if ( change_access( sk, ADMIN ))
    {
        write_log(FALSE,"stow","Error",
            "Unable to downgrade to observer access" );
    }
    else
    {
        write_log(FALSE,"stow","Access", "Changed to observer access" );
    }
    stopfile( REMOVE );
    close( sk );
    return 0;
}  /* END main */



int execute_command( int socket, const char *command_str, char *response )
{
    char message[BUFLEN+2];
    int  mlen;
    int  done;
    int  n, i;
    char wm[BUFLEN];

    /* Copy the command into message upper casing it as we go
    for ( n = 0; n < strlen( command_str ); n++ )
        message[n] = command_str[n];
    ---------------------------------------------------------*/
    strcpy( message, command_str );
    n = strlen( message );
    /* Add the CR LF and null terminator */
    message[n++] = 0x0d;
    message[n++] = 0x0a;
    message[n]   = '\0';
    mlen = n;

    /* Write message to the socket
    ------------------------------*/
    n = write( socket, message, mlen );
    if ( n != mlen )
    {
        sprintf( wm, "Error sending message: %s", message);
        write_log(FALSE,"stow","Error",wm);
	return 1;
    }

    /* Get the response
    -------------------*/
    done = i = 0;
    do
    {
        n = read( socket, &response[i], 1 );
	if ( n != 1 )
	    done = 1;
	else if ( response[i] == 0x0a && response[i-1] == 0x0d )
	    done = 1;
	i++;
    } while ( !done );

    /* If we got here properly there will be CR LF at the end of response
    ---------------------------------------------------------------------*/
    if ( response[i-2] == 0x0d && response[i-1] == 0x0a )
        response[i-2] = '\0';
    else
        response[i] = '\0';

    return 0;

}  /* END execute_command */




int pos( int sk, char *axis )
{
    char  resp[BUFLEN];
    char err_msg[BUFLEN];
    float position;
    char  cmd[BUFLEN];
    char *s;
    char wm[BUFLEN];

    sprintf( cmd, "OPNS:POSN:%s,?", axis );
    execute_command( sk, cmd, resp );
    if ( validate_response( cmd, resp, err_msg ))
    {
        sprintf( wm, "Bad response: %s", resp );
        write_log(FALSE,"stow","Error", wm );
        return BADVAL;
    }
    s = parse_string( resp, ",", 4 );
    sscanf( s, "%f", &position );

    return (int)( position + 0.5 );
}  /* END pos */

float fparse_string( const char *str, const char *dlim, int n )
{
    float x;
    char *s;

    s = parse_string( str, dlim, n );
    sscanf( s, "%f", &x );
    return x;
}

int iparse_string( const char *str, const char *dlim, int n )
{
    int i;
    char *s;

    s = parse_string( str, dlim, n );
    sscanf( s, "%d", &i );
    return i;
}


char *parse_string( const char *str, const char *dlim, int n )
{
    int i;
    char *s;
    char *t;

    s = malloc( sizeof( char ) * ( 1+ strlen( str )));
    if ( !s )
        return NULL;
    strcpy( s, str );

    t = strtok( s, dlim );
    for ( i = 1; i < n; i++ )
        t = strtok( NULL, dlim );

    return t;
}

int change_access( int sk, int level )
{
    char *s;
    char resp[BUFLEN];
    char cmds[3][25] = {
        { "CCFG:ACCS,0,Admin3880,@" },
        { "CCFG:ACCS,1,operator,@" },
        { "CCFG:ACCS,2,@" }};
    int acu_level;
    char wm[BUFLEN];

    execute_command( sk, "CCFG:ACCS,?", resp );
    s = parse_string( resp, ",", 4 );
    sscanf( s, "%d", &acu_level );

    if ( acu_level != level )
    {
        execute_command( sk, cmds[level], resp );
        if ( strcmp( resp, "ccfg:accs,0,0" ) != 0 )
        {
            sprintf( wm, "Unable to enter access level: %d", level );
            write_log(FALSE,"stow","Error", wm );
            return 1;
        }
    }

    return 0;
}


int change_axismode( int sk, char *axis, int mode )
{
    char *s;
    char resp[BUFLEN];
    char err_msg[BUFLEN];
    int  axis_mode;
    char cmd[BUFLEN];
    int  tries;

    /* What is the current mode */
    sprintf( cmd, "OPNS:MODE:%s,?", axis );
    execute_command( sk, cmd, resp );
    if ( validate_response( cmd, resp, err_msg ))
        return 1;
    axis_mode = iparse_string( resp, ",", 4 );
    if ( axis_mode == mode )
        return 0;

    sprintf( cmd, "OPNS:MODE:%s,%d,@", axis, mode );
    execute_command( sk, cmd, resp );
    if ( validate_response( cmd, resp, err_msg ))
        return 1;

    /* Give it a bit to get there */
    tries = 0;
    while ( tries < 5 )
    {
        sprintf( cmd, "OPNS:MODE:%s,?", axis );
        execute_command( sk, cmd, resp );
        if ( validate_response( cmd, resp, err_msg ))
            return 1;
        axis_mode = iparse_string( resp, ",", 4 );
        if ( axis_mode != mode )
            sleep(1);
        tries++;
    }
    if ( axis_mode != mode )
        return 1;

    return 0;
}

/* Validate the response from the ACU */
int validate_response( const char *cmd, const char *resp, char *err )
{
    int return_status = 0;
    char *c, *r, *e;
    int i, status;

    strcpy( err, "" );

    /* First see if the stuff up to the last colon match.  Will have to lower
    case the command portion. */
    c = parse_string( cmd, ",", 1 );
    r = parse_string( resp, ",", 1 );
    if ( strcasecmp( c, r ) != 0 )
        return_status = 1;



    /* Now get the first argument -- it should be zero */
    /* If it isn't find out what the error is */
    /* and tack on the error string to err */ 
    /* set the return_status to 1 */
    status = iparse_string( resp, ",", 2 );
    if ( status != 0 )
    {
        e = get_err_msgs( 0, status );
        strcpy( err, e );
        return_status = 1;
    }


    /* The second argument should also be zero */
    /* If it isn't find out what the error is */
    /* and tack on the error string to err -- add a new line first */ 
    /* set the return_status to 1 */
    status = iparse_string( resp, ",", 3 );
    if ( status != 0 )
    {
        e = get_err_msgs( 0, status );
        strcat( err, "\n" );
        strcat( err, e );
        return_status = 1;
    }

    return return_status;
}


static char *ACUStatus[8] = {
    "ACU is OK",
    "SERVO not communicating",
    "SERVO Non-operational",
    "ACU local lockout engaged",
    "Pedestal disabled",
    "External commands blocked",
    "Controller non-operational",
    "Another client already operator or administrator control level" };

static int ACUIndex[48] = { 3, 4, 5, 6, 7, 100, 101, 102, 200, 201, 202,
    203, 204, 205, 206, 207, 208, 209, 400, 401, 402,
    403, 404, 405, 406, 500, 501, 502, 503, 504, 505,
    507, 509, 510, 511, 512, 513, 514, 600, 601, 602,
    603, 604, 605, 606, 607, 608, 609 };

static char *ACUMessage[48] = {
    "ACU local lockout engaged",
    "Pedestal disabled",
    "External commands blocked",
    "Controller non-operational",
    "Another client already operator or administrator con",
    "Disk full",
    "Beacon receiver interface non-operational",
    "NTS off line",
    "Command attempted from user not in control",
    "Command malformed",
    "Command parameter out of range",
    "Command parameter garbled",
    "Command failed in access",
    "Command in wrong access control level",
    "ACU was in the wrong mode to accept command",
    "Command Obsolete",
    "Command failed due to setup or configuration",
    "Command failed to instrument communication link",
    "File not found",
    "File corrupt",
    "File not writable",
    "FTP download failed",
    "Database access error",
    "Database Key error",
    "Key or string searched for is not found in file",
    "Servo software error",
    "Can't find key value in config3880.ini file",
    "Axis not defined in Servo System",
    "Can't send to ACU Controller on SVR Socket",
    "Servo Processor internal communications error",
    "The command is invalid",
    "Walkbox active, can't implement command",
    "Functionality not available yet",
    "The message data field is invalid",
    "Axis is stowed, cannot execute command",
    "Upgrade Failed",
    "Invalid Command Action (@,$,#,? Was invalid for command)",
    "Wrong Mode Error. The ACU was in an incorrect mode for the command.",
    "Propagator not loaded.",
    "Propagator could not find next visibility.",
    "Propagator end of data points for pointing angles",
    "Mission is active, command ignored",
    "Mission is not active, command ignored.",
    "Telemetry Logger is active, command ignored",
    "Telemetry Logger is not active, command ignored",
    "Test is active, command ignored",
    "Test is not active, command ignored.",
    "Test is not available in this system" };


char *get_err_msgs( int type, int err )
{
    static char msg[BUFLEN];
    int i;
    int index;

    /* Somehow return the pointer to the static error messages
    based on the type of error and the error number itself */
    switch ( type )
    {
        case 0 :
            strcpy( msg, ACUStatus[err] );
            break;
        case 1 :
            for ( i = 0, index = 48; i < 48; i++ )
            {
                if ( ACUIndex[i] == err )
                {
                    index = i;
                    break;
                }
            }
            if ( index == 48 )
                strcpy( msg, "Unknown error code" );
            else
                strcpy( msg, ACUMessage[index] );
            break;
        default:
            strcpy( msg, "Unknown message type" );
    }

    return msg;
}


int stop_track( int sk )
{
    char *s;
    char resp[BUFLEN];
    char err_msg[BUFLEN];
    char cmd[BUFLEN];
    int  tries;
    int  mode;
    
    /* What is the current mode */
    mode = track_mode( sk );
    if ( mode == 0 || mode == -1 )
        return 0;

    tries = 0;
    while ( tries < 5 )
    {
        sprintf( cmd, "MSSN:POIN:STOP,@" );
        execute_command( sk, cmd, resp );
        if ( validate_response( cmd, resp, err_msg ))
            return 1;
        sleep(1);
        mode = track_mode( sk );
        if ( mode == 0 || mode == -1 )
            tries = 5;
    }

    return 0;
}

int track_mode( int sk )
{
    char *s;
    char resp[BUFLEN];
    char err_msg[BUFLEN];
    char cmd[BUFLEN];
    int  mode;

    /* What is the current mode */
    sprintf( cmd, "MSSN:POIN,?" );
    execute_command( sk, cmd, resp );
    if ( validate_response( cmd, resp, err_msg ))
        return BADVAL;
    mode = iparse_string( resp, ",", 5 );

    return mode;
}


int stopfile( int mode )
{
    char file[] = {"/avhrr/software/las/build/env/adaps/tables/stowing"};
    int  status;
    
    switch ( mode )
    {
        case CREATE :
            status = open( file, O_CREAT );
            if ( status == -1 )
                write_log(FALSE,"stow","Error", "Couldn't create a stop file" );
            else
                write_log(FALSE,"stow","Information", "Created stop file" );
            break;

        case REMOVE :
            status = remove( file );
            if ( status )
                write_log(FALSE,"stow","Error", "Couldn't remove stop file" );
            else
                write_log(FALSE,"stow","Information", "Removed stop file" );
            break;
    }

    return status;
}

int stow_elevation( int sk )
{
    int tries;
    char command[BUFLEN];
    char acu_msg[BUFLEN];
    char err_msg[BUFLEN];
    char wm[BUFLEN];

    /* Put the elevation in manual */
    if ( change_axismode( sk, EL, 1 ))
    {
        write_log(FALSE,"stow","Error",
            "Unable to change to manual elevation mode" );
        return 1;
    }
    else
    {
        tries = 0;
        if ( pos( sk, EL ) != 90 )
        {
            strcpy( command, "OPNS:POSN:AXS2,90.0,@" );
            execute_command( sk, command, acu_msg );
            if ( validate_response( command, acu_msg, err_msg ))
            {
                sprintf( wm, "Elevation stow returned: %s", acu_msg );
                write_log(FALSE,"stow","Information", wm );
                tries = 15;
            }
        }
        while ( tries < 15 && pos( sk, EL ) != 90 )
        {
            sprintf( wm, "Elevation at: %d", pos( sk, EL ) );
            write_log(FALSE,"stow","Information", wm );
            sleep(2);
            tries++;
        }
        if ( tries == 15 )
        {
            sprintf( wm, "Antenna not stowed in elevation.  Position is: %d",
                pos( sk, EL ));
            write_log(FALSE,"stow","Information", wm );
            return 1;
        }

        /* Let it settle a bit */
        /* It would probably be better to wait for the rates to settle */
        /* but that will have to wait for the telemetry function */
        sleep(4);

        /* Put it in standby */
        if ( change_axismode( sk, EL, 0 ))
        {
            write_log(FALSE,"stow","Error",
                "Unable to change to standby elevation mode" );
            return 1;
        }
    }
    
    return 0;
}
