
/*!
    \file ProcessExecution.cpp
    \brief Provides the implementation for ProcessExecution.

    See ProcessExecution.
*/

/* System Includes */

/* Qt Includes */
#include "qapplication.h"
#include "qcursor.h"
#include "qregexp.h"

/* Local Includes */
#include "ProcessExecution.h"

ProcessExecution::ProcessExecution(QWidget * parent,
                                   const char * name,
                                   WFlags f)
: QWidget(parent, name, f),
  myProcess(NULL)
{
    myProcess = new QProcess(this);

    // Connect the outputs so we can read them.
    connect(myProcess, SIGNAL(readyReadStdout()),
            this, SLOT(readFromStdoutSlot()));
    connect(myProcess, SIGNAL(readyReadStderr()),
            this, SLOT(readFromStderrSlot()));
}

ProcessExecution::~ProcessExecution()
{
    delete(myProcess);
}

QString ProcessExecution::unEnvCmd(QString & cmd)
{
    QString inputCmd = cmd;
    QString outputCmd("");

    QStringList splitCmd = QStringList::split(QRegExp("\\/"), inputCmd);

    if(splitCmd[0].isEmpty())
    {
        outputCmd += "/";
    }

    QStringList::iterator itr;
    for(itr = splitCmd.begin(); itr != splitCmd.end(); ++itr)
    {
        QString part = (*itr);

        if(part.isEmpty())
        {
            // Skip empty slashes "/////" that are in the path.
            continue;
        }

        if(part.contains(QRegExp("\\$")))
        {
            part.replace('$', ""); // Remove the $ so getenv works.
            part = getenv(part);

            if(part.isEmpty())
            {
                return "";
            }
        }

        outputCmd += "/" + part;
    }

    return outputCmd;
}

bool ProcessExecution::execute(QString & cmd,
                               bool wait)
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

    QStringList list = QStringList::split(QRegExp("\\s+"), cmd);

    // Remove environment variables from the path string.
    list[0] = unEnvCmd(list[0]);

    myStdout.clear();
    myStderr.clear();

    if(list[0].isEmpty())
    {
        myStderr.append("Invalid envoronment variable in path.");
        QApplication::restoreOverrideCursor();
        return false;
    }

    myProcess->clearArguments();
    myProcess->setArguments(list);

    if(!myProcess->start())
    {
        QApplication::restoreOverrideCursor();
        return false;
    }

    // We need to wait for the process to finish so that we can
    // get the results.
    while(myProcess->isRunning() && wait)
    {
        qApp->processEvents(); // Process other events if there are any.
    }

    if(!myStderr.isEmpty())
    {
        QApplication::restoreOverrideCursor();
        return false;
    }

    QApplication::restoreOverrideCursor();
    return true;
}

QStringList ProcessExecution::getStdout()
{
    return myStdout;
}

QStringList ProcessExecution::getStderr()
{
    return myStderr;
}

void ProcessExecution::readFromStdoutSlot()
{
    // Create a list of the results.
    while(myProcess->canReadLineStdout())
    {
        myStdout.append(myProcess->readLineStdout());
    }
}

void ProcessExecution::readFromStderrSlot()
{
    // Create a list of the results.
    while(myProcess->canReadLineStderr())
    {
        myStderr.append(myProcess->readLineStderr());
    }
}

