/*************************************************************************
FUNCTION:	alert_user

PURPOSE:	Popup a warning dialog box to alert the user that some
		error has occurred: file not updated, process not running,
		etc.  The warning dialog also allows the user to disable
		future warnings for specific items.

INPUT:		linked list of alerts

OUTPUT:		none

RETURNS:	none

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00	D.Lloyd         Initial development

PROJECT:        ADAPS
****************************************************************************/
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/PanedW.h>
#include "adapsmonitor.h"

/* Global variables referenced
------------------------------*/
extern Widget toplevel;


void alert_user
    (
    struct ITEM *alarmlist	/* IN - list of alarms to display	*/
    )
    {
    Display *display = XtDisplay( toplevel ); /* for event loop trap	*/
    Widget alert_window;	/* the main alert shell 		*/
    Widget alertform;		/* sole child of the alert shell window	*/
    Widget headerform;		/* child of the alert form window	*/
    Widget header;		/* the header information form		*/
    Widget listform;		/* child of the alert form window	*/
    Widget scrollform;          /* window that contains working widgets */
    Widget buttonform;		/* child of the alert form window	*/
    Widget applybutton;		/* contains the apply button & label	*/
    Widget ackbutton;		/* contains acknowledge button & label	*/
    Window win;			/* the alert_window widgets window	*/
    XEvent event;		/* event structure 			*/
    struct ITEM *oldalarmlist;	/* previous alarm list already in use	*/


    /* If the dialog shell already exists, destroy it before
    trying to recreate it.  There is also an alarm list
    associated with the window that needs to be freed.
    -------------------------------------------------------*/
    alert_window = XtNameToWidget( toplevel, WnALERTWINDOW );
    if ( alert_window )
        {
        XtDestroyWidget( alert_window );
	oldalarmlist = get_alarm_address();
	free_itemlist( &oldalarmlist );
	}

    /* Replace the alarm list value with the current one.
    ----------------------------------------------------*/
    store_alarm_address( alarmlist );

    /* Create the popup dialog shell.  Set the delete response
    to DESTROY since we don't want to reuse this widget again.
    We will recreate this widget every time since the contents
    will be different with each instance.
    ----------------------------------------------------------*/
    alert_window = XtVaCreatePopupShell( WnALERTWINDOW,
        xmDialogShellWidgetClass, toplevel,
	XmNdeleteResponse, XmDESTROY,
	NULL );

    /* Create the sole child of the alert_window
    --------------------------------------------*/
    alertform = XtVaCreateWidget( WnALERTFORM,
        xmFormWidgetClass, alert_window,
	XmNwidth, 600,
	XmNheight, 447,
	XmNx, 20,
	XmNy, 20,
	NULL );
    XtManageChild( alertform );
    
    /* Create the static header for the window
    ------------------------------------------*/
    headerform = XtVaCreateWidget( WnALERTHEADERFORM, xmFormWidgetClass,
        alertform,
	XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNtopAttachment, XmATTACH_FORM,
	XmNtopOffset, 2,
	XmNheight, 32,
	XmNresizable, False,
	XmNx, 2,
	XmNwidth, 596,
	NULL );
    XtManageChild( headerform );

    /* Create the label for the header.  The actual string
    is defined in Adapsmonitor.ad and fallbacks.h 
    -----------------------------------------------------*/
    header = XtVaCreateWidget( WnALERTHEADER, xmLabelWidgetClass, headerform,
	XmNx, 7,
	XmNy, 8,
	XmNwidth, 582,
	XmNheight, 23,
	NULL );
    XtManageChild( header );

    /* Create the display form
    --------------------------*/
    listform = XtVaCreateWidget( WnALERTLISTFORM, xmFormWidgetClass,
        alertform,
	XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNtopAttachment, XmATTACH_WIDGET,
	XmNtopOffset, 36,
	XmNbottomAttachment, XmATTACH_WIDGET,
	XmNbottomOffset, 2,
	XmNx, 2,
	XmNy, 36,
	XmNwidth, 596,
	XmNheight, 365,
	NULL );
    XtManageChild( listform );

    /* Create the button form
    -------------------------*/
    buttonform = XtVaCreateWidget( WnALERTBUTTONFORM, xmFormWidgetClass,
        alertform,
	XmNheight, 40,
	XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNbottomOffset, 2,
	XmNresizable, False,
	XmNfractionBase, 5,
	XmNy, 403,
	NULL );
    XtManageChild( buttonform );

    /* listform needs a scrolled window in case the list of alerts
    is large.  Within the scrolled window is a 2-column widget:
    the left column contains buttons to disable individual alerts
    and the right the name of the item that caused the alert.
    Since the number of widgets in this form will be dynamic,
    these widget names are not contained in widget.h
    -------------------------------------------------------------*/
    scrollform = XtVaCreateManagedWidget( WnALERTSCROLL, 
        xmScrolledWindowWidgetClass, listform,
	XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNtopAttachment, XmATTACH_FORM,
	XmNtopOffset, 2,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNbottomOffset, 2,
	XmNscrollingPolicy, XmAUTOMATIC,
	XmNscrollBarDisplayPolicy, XmAS_NEEDED,
	XmNvisualPolicy, XmCONSTANT,
	NULL );
    XtManageChild( scrollform );
    

    /* Create the row col of buttons and
    labels, the widget value is not needed.
    ---------------------------------------*/
    create_toggle_array( scrollform, alarmlist, WnALERTWINDOW );


    /* Create the button bar buttons the labels
    are defined in Adapsmonitor.ad and fallbacks.h
    ----------------------------------------------*/
    ackbutton = XtVaCreateWidget( WnALERTACK, xmPushButtonWidgetClass,
        buttonform,
	XmNheight, 30,
	XmNwidth, 100,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNbottomOffset, 5,
	XmNrightAttachment, XmATTACH_POSITION,
	XmNrightPosition, 5,
	XmNrightOffset, 20,
	XmNresizable, False,
	NULL );
    XtManageChild( ackbutton );
    XtAddCallback( ackbutton, XmNactivateCallback, destroy_alert, alarmlist );

    applybutton = XtVaCreateWidget( WnALERTAPPLY, xmPushButtonWidgetClass,
        buttonform,
	XmNheight, 30,
	XmNwidth, 100,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNbottomOffset, 5,
	XmNleftAttachment, XmATTACH_POSITION,
	XmNleftPosition, 0,
	XmNleftOffset, 20,
	XmNresizable, False,
	NULL );
    XtManageChild( applybutton );
    XtAddCallback( applybutton, XmNactivateCallback, apply_alert_changes,
        alarmlist );

    /* Attach the scroll form to the button form
    --------------------------------------------*/
    XtVaSetValues( listform, XmNbottomWidget, buttonform, NULL );

    /* realize the alert window.  Must map the display in case it has been
    iconified, otherwise the alert window will remain iconified as well.
    Mapping the display also pops up the alert window so there is no
    reason to call XtPopup.
    ----------------------------------------------------------------------*/
    XMapWindow( XtDisplay( toplevel ), XtWindow( toplevel ));
    XBell( display, 100 );

    /* We need to enter an event loop here because we don't want the
    user to be able to do anything with the main window or modify 
    window -- i.e. we want to exclude the user from messing with the
    item list since the alert window depends on the list being the
    same as it was when this was called.
    ----------------------------------------------------------------*/
    win = XtWindow( alert_window );
    while ( XCheckMaskEvent( display, ButtonPressMask | ButtonReleaseMask |
        ButtonMotionMask | PointerMotionMask | KeyPressMask, &event ))
	{
	if ( event.xany.window == win )
	    XtDispatchEvent( &event );
	else
	    /* throw the event away */;
	}

    }  /* END alert_user */
