/****************************************************************************
FUNCTION:	check_line

PURPOSE:	Check a line of data from an initialization file for
  		consistency.

INPUT:	        buffer of character data read from initialization file

OUTPUT:         none

RETURNS:	int SUCCESS or FAIL

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "adapsmonitor.h"

int check_line
    (
    char inbuf[]		/* IN - input buffer to check		*/
    )
    {
    char   item_name[CMLEN];	/* name of the item			*/
    char   item_type[CMLEN];	/* item type from buffer		*/
    char   logfile[CMLEN];	/* name of log file			*/
    char   monitor_state[CMLEN];/* initial monitor state		*/
    char   msg[CMLEN];		/* error message text			*/
    char   sys_name[CMLEN];	/* name of system			*/
    char   userid[CMLEN];	/* user ID				*/

    int    monitor_interval;	/* monitor interval			*/
    int    num_batch;		/* number of batch jobs			*/
    int    num_scanned;		/* number of elements scanned from inbuf*/
    int    rwast_interval;	/* monitor interval for RWAST state	*/
    int    status;		/* return status			*/

    ItemType type;		/* enumerated item type			*/



    /* scan the nine items into local variables
    -------------------------------------------*/
    num_scanned = sscanf( inbuf, "%s %s %s %d %d %s %d %s %s", item_name,
        sys_name, item_type, &monitor_interval, &rwast_interval, monitor_state,
	&num_batch, userid, logfile );
    if ( num_scanned != 9 )
        {
	/* write an error message to log */
	write_log( FALSE, "check_line", "parse", inbuf );
	sprintf( msg, "%d of 9 parsed from input file", num_scanned );
	write_log( TRUE, "check_line", "parse", msg );
	return FAIL;
	}
    
    /* check that monitor intervals are within reason.  These values
    should not be less than 1, except for -1 which is used to indicate
    there is no value.  Values should not be greater than MAXINTERVAL.
    -------------------------------------------------------------------*/
    status = SUCCESS;
    if ( monitor_interval > MAXINTERVAL ||
         monitor_interval < -1          ||
	 monitor_interval == 0 )
        {
	/* write error message to log */
	sprintf( msg, "Monitor interval invalid: %d", monitor_interval );
	write_log( TRUE, "check_line", "interval", msg );
	status = FAIL;
	}
    if ( rwast_interval > MAXINTERVAL ||
         rwast_interval < -1          ||
	 rwast_interval == 0 )
        {
	/* write error message to log */
	sprintf( msg, "Monitor interval invalid: %d", monitor_interval );
	write_log( TRUE, "check_line", "interval", msg );
	status = FAIL;
	}
    
    /* Check that the number of batch jobs is within reason.  These values
    should not be less than 1, except for -1 which is used to indicate
    there is no value.  Values should not be greater than MAXBATCHJOB.
    ----------------------------------------------------------------------*/
    if ( num_batch > MAXBATCHJOB || num_batch < -1 || num_batch == 0 )
        {
	/* write error message to log */
	sprintf( msg, "Number of batch jobs invalid: %d ", num_batch );
	write_log( TRUE, "check_line", "numbatch", msg );
	status = FAIL;
	}

    /* Check that the item type is one of the accepted values.
    ----------------------------------------------------------*/
    type = get_item_type( item_type );
    if ( type == UNDEFINED )
        {
	/* write error message to log */
	sprintf( msg, "Item type is undefined: %s", item_type );
	write_log( TRUE, "check_line", "item-type", msg );
	status = FAIL;
	}

    /* Check that batch jobs have number of jobs, not -1,  and
    user ID, not NONE, specified.
    ----------------------------------------------------------*/
    if ( type == BATCH )
        {
	if ( num_batch < 1 )
	    {
	    /* write error message to log */
	    sprintf( msg, "Number of batch jobs invalid: %d ", num_batch );
	    write_log( TRUE, "check_line", "numbatch", msg );
	    status = FAIL;
	    }
	if ( strcmp( userid, "NONE" ) == 0 )
	    {
	    /* write error message to log */
	    write_log( TRUE, "check_line", "numbatch",
	        "No user ID specified for batch job" );
	    status = FAIL;
	    }
	}
    
    /* Make sure vax processes have a RWAST interval specified.
    -----------------------------------------------------------*/
    if ( type == VAXPROCESS )
        {
	if ( rwast_interval < 1 )
	    {
	    /* write error message to log */
	    sprintf( msg, "RWAST monitor interval invalid: %d", rwast_interval);
	    write_log( TRUE, "check_line", "interval", msg );
	    status = FAIL;
	    }
	}
    
    /* Batch jobs and processes must have a log file specified
    ----------------------------------------------------------*/
    if ( type == BATCH || type == PROCESS )
        {
	if ( strcmp( logfile, "NONE" ) == 0 )
	    {
	    /* write error message to log */
	    write_log( TRUE, "check_line", "nolog", 
	        "Batch jobs must have a log file specified" );
	    status = FAIL;
	    }
	}

    /* Check for a valid monitor state
    ----------------------------------*/
    if ( get_monitor_state( monitor_state ) == UNINITIALIZED )
        {
	sprintf( msg, "Monitor state %s undefined", monitor_state );
	write_log( TRUE, "initialize", "monitor_state", msg );
	status = FAIL;
	}

    return status;
    }  /* END check_line */
