/******************************************************************************
FUNCTION:	checkstatus_callback

PURPOSE:	Go through the item list and call check_item for each
  		item in the list that hasn't been checked for X seconds,
		where X is the item monitor interval.  If any alarms
		are set, issue an alert callback.

INPUT:		widget information
		client data: pointer to int -- monitor interval

OUTPUT:		none

RETURNS:	none

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "adapsmonitor.h"

extern XtAppContext app;	/* main application context             */
extern Widget toplevel;         /* top level shell widget               */

/* These bitmaps were taken from <X11/bitmaps/xm_hour16>
and <X11/bitmaps/xm_hour16m> since they don't exist on
edcsns1.  They are recreated here so that the monitor can
compile on the sun.
---------------------------------------------------------*/
#define hour16_x_hot 7
#define hour16_y_hot 7
#define hour16_width 16
#define hour16_height 16
static char hour16_bits[] = {
    0x00, 0x00, 0xfe, 0x7f, 0x14, 0x28, 0x14, 0x28, 0x14, 0x28, 0x24, 0x24,
    0x44, 0x22, 0x84, 0x21, 0x84, 0x21, 0x44, 0x22, 0x24, 0x24, 0x14, 0x28,
    0x94, 0x29, 0xd4, 0x2b, 0xfe, 0x7f, 0x00, 0x00};
#define hour16m_width 16
#define hour16m_height 16
static char hour16m_bits[] = {
    0xff, 0xff, 0xff, 0xff, 0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x7f,
    0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x7f,
    0xfe, 0x7f, 0xfe, 0x7f, 0xff, 0xff, 0xff, 0xff};


/* local function to change cursor	*/
void change_cursor( short );

void checkstatus_callback
    (
    XtPointer    client_data,	/* IN - client data, timer interval	*/
    XtIntervalId *id		/* IN - ID of the timer callback	*/
    )
    {
    Widget display_list;	/* the main display list widget		*/
    Widget alert_window;        /* the main alert shell                 */

    char   alertstr[CMLEN];     /* alert string for write_log           */
    char   wstring[CMLEN];	/* widget string			*/

    int    *iptr = (int *)client_data; /* pointer to client data	*/
    int    timerinterval;	/* monitor interval for next timer	*/

    struct ITEM *alarm = NULL;	/* list of items with alarms		*/
    struct ITEM *curr;		/* list of items with alarms		*/
    struct ITEM *item;		/* the item list to check		*/
    struct ITEM *last;		/* list of items with alarms		*/

    time_t now;			/* time in seconds since beginning of	*/
    				/* unix time				*/


change_cursor( TRUE );

    /* Get the most recent item list to check
    -----------------------------------------*/
    item = get_list_address();

    /* Get the current local time
    -----------------------------*/
    now = time( 0 );

    /* loop through each item in the list
    -------------------------------------*/
    while ( item != NULL )
        {
	/* See if we need to check the status of this item
	--------------------------------------------------*/
	if ((( item->monitor_state == ENABLED ) &&
	     (now - item->last_check > item->mon_interval )) ||
	     item->alarm == TRUE )
	    {
	    /* Get the item's current state
	    -------------------------------*/
	    item->current_state = check_item( item, now );

	    /* Set the status and time checked display fields
	    to current values and recreate the display strings
	    --------------------------------------------------*/
	    item->fieldflg[3] = INVALID;
	    /* item->last_check = now; */
	    item->fieldflg[4] = INVALID;
	    create_displist( item );

	    /* Add current item to alarm list if not okay
	    ---------------------------------------------*/
	    if ( item->current_state != RUNNING &&
	         item->current_state != RESPONDING &&
	         item->current_state != UPTODATE )
	        {
		/* Set the alarm flag so that during the next monitor
		interval we check this item again and redisplay the alert
		---------------------------------------------------------*/
	        item->alarm = TRUE;

		/* Allocate and copy this item for the alarm list
		-------------------------------------------------*/
		curr = allocate_item();
		if ( !curr )
		    {
		    write_log( FALSE, "checkstatus_callback", "alloc",
                        "Unable to allocate memory" );
                    /* free previously allocated memory */
                    if ( alarm )
                        free_itemlist( &alarm );
		    /* Re-register the timer for this callback if
		    the interval specified is greater than zero.
		    ---------------------------------------------*/
		    timerinterval = *iptr;
		    if ( timerinterval > 0 )
			XtAppAddTimeOut(app, timerinterval,
			    checkstatus_callback, client_data);
		    return;
		    }
		*curr = *item;

		/* Add this new alarm to the alarm list
		---------------------------------------*/
		if ( alarm == NULL )
		    alarm = last = curr;
		else
		    last->next = curr;
		curr->next = NULL;
		last = curr;
		}
	    else
		/* Reset the alarm flag.  This may have been previously
		set and now the item's state is okay.  Resetting the flag
		will prevent it from being checked or displayed as an
		alert the next time.
		-------------------------------------------------------*/
	        item->alarm = FALSE;

	    }  /* END if */

	/* Go to next item in list
	--------------------------*/
	item = item->next;
	}  /* END while item not null */

    /* Get the main window scrolled list widget and
    update it with the modified items in the list.
    -----------------------------------------------*/
    sprintf( wstring, "%s.%s.scrolledlistSW.%s", WnMAINW, WnMWFORM2,
        WnSCROLLEDLIST );
    display_list = XtNameToWidget( toplevel, wstring );
    update_displist( display_list, get_list_address());

    /* If any alarms were registered notify the user
    -----------------------------------------------*/
    if ( alarm )
        {
	/* write an alert to the log
	----------------------------*/
        curr = alarm;
        while ( curr != NULL )
            {
            sprintf( alertstr, "%s: %s", curr->name_field, curr->status_field );
            write_log( FALSE, "checkstatus_callback", "alert", alertstr );
            curr = curr->next;
            }

	alert_user( alarm );
	}  /* END if alarm */
    else
        {
	/* There are no alarms so if there is an alert window
	destroy it and free the alarm list associated with it
	-----------------------------------------------------*/
	alert_window = XtNameToWidget( toplevel, WnALERTWINDOW );
	if ( alert_window )
	    {
	    XtDestroyWidget( alert_window );
	    alarm = get_alarm_address();
	    free_itemlist( &alarm );
	    }
	}  /* END else */
    
change_cursor( FALSE );
    /* Re-register the timer for this callback if
    the interval specified is greater than zero.
    ---------------------------------------------*/
    timerinterval = *iptr;
    if ( timerinterval > 0 )
	XtAppAddTimeOut(app, timerinterval, checkstatus_callback, client_data);

    }  /* END checkstatus_callback */



void change_cursor
    (
    short on			/* IN - flag T = turn on hourglass	*/
    )				/*           F = return original cursor	*/
    {
    Colormap colormap;		/* the current colormap			*/
    Display  *dpy = XtDisplay( toplevel ); /* the display		*/
    Pixmap   mask;		/* the hourglass mask			*/
    Pixmap   shape;		/* the hourglass pixmap			*/
    XColor   black;		/* black value for hourglass color	*/
    XColor   white;		/* white value for maks color		*/
    XSetWindowAttributes attrs;	/* display window attributes structure	*/
    static   Cursor cursor;	/* the hourglass cursor			*/
    static   int locked;	/* flag: value > 0 means cursor is changed */
    char     wstring[CMLEN];	/* widget string			*/
    Widget   w;


    /* Keep track of how many times the cursor has been
    requested to change to hourglass.  Value greater
    than zero means the cursor is an hourglass.  Value
    of zero means we have original cursor.  This allows
    for multiple calls.
    ---------------------------------------------------*/
    if ( on )
        locked++;
    else
        locked--;
    
    /* If the cursor is still locked return
    ---------------------------------------*/
    if (( locked > 1 || locked == 1 ) && on == 0 )
        return;
    
    /* If the hourglass cursor hasn't been
    created yet, create it.
    ---------------------------------------*/
    if ( !cursor )
	{
	shape = XCreatePixmapFromBitmapData( dpy, XtWindow( toplevel ),
	    hour16_bits, hour16_width, hour16_height, 1, 0, 1 );
	mask = XCreatePixmapFromBitmapData( dpy, XtWindow( toplevel ),
	    hour16m_bits, hour16m_width, hour16m_height, 1, 0, 1 );
	colormap = DefaultColormap( dpy, 0 );
	XParseColor( dpy, colormap, "black", &black );
	XParseColor( dpy, colormap, "white", &white );
	XFreeColormap( dpy, colormap );

	cursor = XCreatePixmapCursor( dpy, shape, mask, &black, &white,
	    hour16_x_hot, hour16_y_hot );
	}

    /* Set the attributes of the cursor.  If the cursor is an
    hourglass return to the original cursor by setting the
    attributes to None, otherwise set it to the hourglass cursor
    -----------------------------------------------------------*/
    attrs.cursor = on ? cursor : None;
    XChangeWindowAttributes( dpy, XtWindow( toplevel ), CWCursor, &attrs );
    sprintf( wstring, "%s.%s", WnMODIFYWIND, WnMODIFY );
    w = XtNameToWidget( toplevel, wstring );
    if ( XtWindow( w ) != NULL )
	XChangeWindowAttributes( dpy, XtWindow( w ),
	    CWCursor, &attrs );
    sprintf( wstring, "%s", WnFILESELECTPOPUP );
    w = XtNameToWidget( toplevel, wstring );
    if ( XtWindow( w ) != NULL )
	XChangeWindowAttributes( dpy, XtWindow( w ),
	    CWCursor, &attrs );
    sprintf( wstring, "%s", "dialog_popup" );
    w = XtNameToWidget( toplevel, wstring );
    if ( w != NULL )
	if ( XtWindow( w ) != NULL )
	    XChangeWindowAttributes( dpy, XtWindow( w ),
		CWCursor, &attrs );
    sprintf( wstring, "%s", "dialog2_popup" );
    w = XtNameToWidget( toplevel, wstring );
    if ( w != NULL )
	if ( XtWindow( w ) != NULL )
	    XChangeWindowAttributes( dpy, XtWindow( w ),
		CWCursor, &attrs );
    sprintf( wstring, "%s", WnALERTWINDOW );
    w = XtNameToWidget( toplevel, wstring );
    if ( w != NULL )
	if ( XtWindow( w ) != NULL )
	    XChangeWindowAttributes( dpy, XtWindow( w ),
		CWCursor, &attrs );
    XFlush( dpy );
    }  /* END change_cursor */
