/*****************************************************************************
FUNCTION:	create_displist

PURPOSE:	Create a display string for the given item.

The display string is set up as four fields:
  1)  monitor - this field starts at position 1 and is MONITORLEN characters
  		in length, the values are Enabled and Disabled
  2)     name - Starts at 11 and is NAMELEN characters long
  3)   status - starts at 63 and is STATUSLEN characters long
  4)    as of - starts at 81 and is ASOFLEN characters long

There are two spaces between each field for a total of 98 characters.

INPUT:	        Pointer to the item list

OUTPUT:         Display strings for main window are modified in item list

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "adapsmonitor.h"


void create_displist
    (
    struct ITEM *item		/* IN/OUT - current item		*/
    )
    {
    char   *ptr;		/* pointer to a string			*/
    char   tempstr[CMLEN];	/* temporary string for the item name	*/

    int    i;			/* loop control				*/
    int    len;			/* string length			*/
    int    system_len;		/* length of the system name		*/

    struct tm *loc;		/* local time structure			*/


    /* Decide which if any of the fields are invalid and correct them
    -----------------------------------------------------------------*/
    /* Check the monitor state
    --------------------------*/
    if ( item->fieldflg[1] == INVALID )
	{
        if ( item->monitor_state == ENABLED )
	    strncpy( item->monitor_field, "Enabled ", MONITORLEN );
	else
	    strncpy( item->monitor_field, "Disabled", MONITORLEN );
	item->monitor_field[MONITORLEN] = '\0';
	item->fieldflg[0] = INVALID;	/* indicate that a field changed */
	item->fieldflg[1] = VALID;
	}
    
    /* Check the name field
    -----------------------*/
    if ( item->fieldflg[2] == INVALID )
        {
	/* If the item is a standard process or data file,
	prepend the name field with the system name
	-------------------------------------------------*/
	if ( item->itemtype == PROCESS || item->itemtype == BATCH ||
	     item->itemtype == DATAFILE )
	    {
	    system_len = strlen( item->machine ) + 2;
	    sprintf( tempstr, "%s: ", item->machine );
	    }
	else
	    {
	    system_len = 0;
	    tempstr[0] = '\0';
	    }

	/* The name field cannot be larger than NAMELEN characters.
	If it is, point to the last NAMELEN characters of the name.
	Otherwise point to the first character.
	------------------------------------------------------*/
	len = strlen( item->itemname );
	if ( len + system_len > NAMELEN )
	    {
	    ptr = &(item->itemname)[len+system_len-NAMELEN];
	    len = NAMELEN - system_len;
	    }
	else
	    ptr = item->itemname;
	
	/* copy the name and pad it out to NAMELEN characters if needed
	---------------------------------------------------------------*/
	sprintf( item->name_field, "%s%s", tempstr, ptr );
	for ( i = len + system_len; i < NAMELEN; i++ )
	    item->name_field[i] = ' ';
	item->name_field[NAMELEN] = '\0';

	/* If the length was greater than NAMELEN place ellipsis
	at the beginning to indicate that the name was truncated.
	-------------------------------------------------------*/
	if ( strlen( item->itemname ) + system_len > NAMELEN )
	    item->name_field[system_len] = item->name_field[system_len+1] =
	    item->name_field[system_len+2] = '.';
	item->fieldflg[0] = INVALID;	/* indicate that a field changed */
	item->fieldflg[2] = VALID;
	}  /* END if ( item->fieldflg[2] == INVALID ) */

    /* Check the current state
    --------------------------*/
    if ( item->fieldflg[3] == INVALID )
        {
	item->status_field[0] = '\0';
	switch ( item->current_state )
	    {
	    case UNCHECKED :
	        strncat( item->status_field, "Unchecked", STATUSLEN );
		break;
	    case RUNNING :
	        strncat( item->status_field, "Running", STATUSLEN );
		break;
	    case UPTODATE :
	        strncat( item->status_field, "Up to date", STATUSLEN );
		break;
	    case OUTOFDATE :
	        strncat( item->status_field, "Not up to date", STATUSLEN );
		break;
	    case NOTINQUEUE :
	        strncat( item->status_field, "Not in queue", STATUSLEN );
		break;
	    case RWAST :
	        strncat( item->status_field, "RWAST State", STATUSLEN );
		break;
	    case SUSPENDED :
	        strncat( item->status_field, "Suspended", STATUSLEN );
		break;
	    case HUNG :
	        strncat( item->status_field, "Hung", STATUSLEN );
		break;
	    case RESPONDING :
	        strncat( item->status_field, "Responding", STATUSLEN );
		break;
	    case NOTRESPONDING :
	        strncat( item->status_field, "Not responding", STATUSLEN );
		break;
	    case NOTRUNNING :
	        strncat( item->status_field, "Not running", STATUSLEN );
		break;
	    default :
	        strncat( item->status_field, "Error", STATUSLEN );
	    }
	/* Pad it out with zeros
	------------------------*/
	for ( i = strlen( item->status_field ); i < STATUSLEN; i++ )
	    item->status_field[i] = ' ';
	item->status_field[STATUSLEN] = '\0';
	item->fieldflg[0] = INVALID;	/* indicate that a field changed */
	item->fieldflg[3] = VALID;
	}  /* END if ( item->fieldflg[3] == INVALID ) */

    /* Check the last check time
    ----------------------------*/
    if ( item->fieldflg[4] == INVALID )
        {
	/* If there is a time, convert it to a readable string
	------------------------------------------------------*/
	if ( item->last_check != 0 )
	    {
	    loc = localtime( &(item->last_check) );
	    sprintf( item->asoftime_field, "%02d/%02d/%02d %02d:%02d:%02d",
		loc->tm_mon + 1, loc->tm_mday, loc->tm_year%100, 
		loc->tm_hour, loc->tm_min, loc->tm_sec );
	    }
	/* Otherwise mark it as unchecked
	---------------------------------*/
	else
	    {
	    strncpy( item->asoftime_field, "Unchecked         ", ASOFLEN );
	    item->asoftime_field[ASOFLEN] = '\0';
	    }
	item->fieldflg[0] = INVALID;	/* indicate that a field changed */
	item->fieldflg[4] = VALID;
	}  /* END if ( item->fieldflg[4] == INVALID ) */
    
    /* If any of the previous fields were changed the
    big display string needs to be changed too.
    -------------------------------------------------*/
    if ( item->fieldflg[0] == INVALID )
        {
	sprintf( item->display, "%s  %s  %s  %s", item->monitor_field,
            item->name_field, item->status_field, item->asoftime_field );
	item->fieldflg[0] = REDISPLAY;
	}
    }  /* END create_displist */
