/*****************************************************************************
FUNCTION:       load_newfile_callback
		fileok_callback
		filecancel_callback

PURPOSE:        Call back routines for the file selection dialog  window.

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include "adapsmonitor.h"



/****************************************************************************
FUNCTION:       load_newfile_callback

PURPOSE:        Register the callback for the file dialog window and
                realize the file dialog.

INPUTS:         calling widget data
		client data - pointer to ITEM list

OUTPUTS:        none

RETURNS:        none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void load_newfile_callback
    (
    Widget w,			/* IN - widget associated with callback	*/
    XtPointer client_data,	/* IN - client data: ITEM struct pointer*/
    XtPointer call_data		/* IN - call data			*/
    )
    {
    struct ITEM *item = (struct ITEM *)client_data;	/* the item list*/
    Widget fileselectwindow;	/* the file select window		*/

    fileselectwindow = get_fileselectwindow( w );

    /* Register the Okay button callback
    ------------------------------------*/
    XtAddCallback( fileselectwindow, XmNokCallback, fileok_callback, item );

    /* Register the cancel button callback
    --------------------------------------*/
    XtAddCallback( fileselectwindow, XmNcancelCallback,
        filecancel_callback, item );

    XtManageChild( fileselectwindow );
    }  /* END load_newfile_callback */




/****************************************************************************
FUNCTION:       fileok_callback

PURPOSE:        To retrieve the file name that the user selected, open
		it and reinitialize the ITEM structure.  The file selection
		widget is also unmanaged.

INPUTS:         calling widget data
		client data is a pointer to an ITEM structure
		call data is a pointer to a file selection callback structure

OUTPUTS:        reinitialized ITEM structure

RETURNS:        none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void fileok_callback
    (
    Widget w,			/* IN - widget associated with callback	*/
    XtPointer client_data,	/* IN/OUT - pointer to ITEM structure	*/
    XtPointer call_data		/* IN - file selection callback struct	*/
    )
    {
    Widget fileselectwindow;	/* the file select window		*/
    Widget rowcol;		/* widget where the list is displayed	*/

    XmFileSelectionBoxCallbackStruct *cbs = /* the file selection call	*/
        (XmFileSelectionBoxCallbackStruct *)call_data; /* back structure*/

    char *filename;		/* file name returned in CB struct	*/
    char message[CMLEN];	/* error message string			*/

    struct ITEM *item = (struct ITEM *)client_data; /* the item list	*/
    struct ITEM *newlist;	/* newly initialized item structure	*/


    /* get the file selection widget value
    --------------------------------------*/
    fileselectwindow = get_fileselectwindow( w );

    /* Remove the associated callbacks.  This is done because
    the pointer to the item structure will be a different
    value each time these callbacks are registered.
    ----------------------------------------------------------*/
    XtRemoveCallback( fileselectwindow, XmNokCallback,
        fileok_callback, item );
    XtRemoveCallback( fileselectwindow, XmNcancelCallback,
        filecancel_callback, item );

    /* remove the file selection widget
    -----------------------------------*/
    XtUnmanageChild( fileselectwindow );

    /* Get the file name from the call back structure
    -------------------------------------------------*/
    if ( !XmStringGetLtoR( cbs->value, XmFONTLIST_DEFAULT_TAG, &filename ))
        {
	post_dialog( w, "Error retrieving file name", "Acknowledge" );
	return;
	}
    
    /* Initialize a new ITEM structure using the filename
    -----------------------------------------------------*/
    newlist = initstruct( filename, FALSE );
    if ( newlist == NULL )
	{
	/* give the user an error dialog and return
	-------------------------------------------*/
	sprintf( message, "Error initializing from file: %s", filename );
	write_log( FALSE, "fileok_callback", "init", message );
	post_dialog( w, message, "Acknowledge" );
        return;
	}

    /* The pointer returned by initstruct is for a new list of
    items.  Since other callbacks are registered with the original
    item list we need to replace the old list with the new list.
    But in doing so we cannot lose the pointer to the first item
    in the list since that's the address the callbacks have.

    This is accomplished by copying the first item values of the
    new list into the first item of the old list.  The remainder of
    the old list must also be freed.
    Before:
        origitem1->origitem1-> ... origitemN->NULL
	newitem1->newitem2-> ... newitemM->NULL
    After:
        copy values from newitem1 to origitem1
        origitem1->newitem2-> ... newitemM->NULL
	newitem1 is freed
	origitem2 to origitemN are freed
    --------------------------------------------------------------*/
    free_itemlist( &item->next );	/* free remainder of old list	      */
    *item = *newlist;			/* copy the new first item to old one */
    newlist->next = NULL;		/* only want to free the first item   */
    free_itemlist( &newlist );		/* that we just copied		      */
    if ( item->next != NULL )		/* if more than one item in list      */
	item->next->prev = item;	/* fix the prev link of the 2nd item  */

    /* Destroy the current rowcol window
    ------------------------------------*/
    XtDestroyWidget( get_rowcol_widget());

    /* Create a new rowcol window with the new list
    -----------------------------------------------*/
    rowcol = create_toggle_array( scroll_window_child( w, WnSCROLLEDWINDOW1 ),
        item, WnMODIFY );
    store_rowcol_widget( rowcol );
    XtManageChild( rowcol );
    }  /* END fileok_callback */



/****************************************************************************
FUNCTION:       filecancel_callback

PURPOSE:        Remove the file selection box.

INPUTS:         calling widget data
                the item list

OUTPUTS:        none

RETURNS:        none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void filecancel_callback
    (
    Widget w,			/* IN - widget associated with callback	*/
    XtPointer client_data,	/* IN - client data, the item list	*/
    XtPointer call_data		/* IN - call data			*/
    )
    {
    struct ITEM *item = (struct ITEM *)client_data; /* the item list	*/
    Widget fileselectwindow;	/* the file select window		*/


    /* get the value of the file select widget
    ------------------------------------------*/
    fileselectwindow = get_fileselectwindow( w );

    /* Remove the associated callbacks.  This is done because
    the pointer to the item structure will be a different
    value each time these callbacks are registered.
    ----------------------------------------------------------*/
    XtRemoveCallback( fileselectwindow, XmNokCallback,
        fileok_callback, item );
    XtRemoveCallback( fileselectwindow, XmNcancelCallback,
        filecancel_callback, item );

    /* Unmanage the file dialog
    ---------------------------*/
    XtUnmanageChild( fileselectwindow );
    }  /* END filecancel_callback */
