/****************************************************************************
FUNCTION:	get_user_response

PURPOSE:	A general purpose routine to popup a message dialog and get
  		the user response.  This is a full application modal function
		so that the system continues to respond to other events.
		But it doesn't allow the user to do anything else until he
		or she responds to the dialog.

INPUT:	        widget of the caller		
		message to be displayed		
		the no answer button label	
		the yes answer button label	
		default answer to use YES or NO

OUTPUT:         None

RETURNS:	integer YES/NO: the user's response

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include "adapsmonitor.h"

/* Global variables
-------------------*/
extern XtAppContext app;	/* The applications context		*/
extern Widget toplevel;


int get_user_response
    (
    Widget   parent,		/* IN - widget of the caller		*/
    char     *question,		/* IN - message to be displayed		*/
    char     *ans1,		/* IN - the no answer button label	*/
    char     *ans2,		/* IN - the yes answer button label	*/
    int      default_answer	/* IN - default answer to use		*/
    )
    {
    XmString no;		/* text of the no label			*/
    XmString text;		/* text of the message			*/
    XmString yes;		/* text of the yes label		*/

    static   Widget dialog;	/* dialog widget static to avoid dupes	*/

    int      answer;		/* answer from response			*/
    


    /* Create the dialog box if it isn't already created
    ----------------------------------------------------*/
    if ( !dialog )
	{
	dialog = XmCreateMessageDialog( toplevel, "dialog", NULL, 0 );
	XtVaSetValues( dialog,
	    XmNdialogStyle,	XmDIALOG_FULL_APPLICATION_MODAL,
	    NULL );
	/* get rid of the help button */
	XtUnmanageChild( XmMessageBoxGetChild( dialog, XmDIALOG_HELP_BUTTON ));
	XtAddCallback( dialog, XmNokCallback, response, &answer );
	XtAddCallback( dialog, XmNcancelCallback, response, &answer );
	}

    /* Set up the text strings
    --------------------------*/
    text = XmStringCreateLocalized( question );
    yes = XmStringCreateLocalized( ans1 );
    no = XmStringCreateLocalized( ans2 );
    XtVaSetValues( dialog,
        XmNmessageString,	text,
	XmNokLabelString,	yes,
	XmNcancelLabelString,	no,
	XmNdefaultButtonType,	default_answer == YES ?
	    XmDIALOG_OK_BUTTON : XmDIALOG_CANCEL_BUTTON,
	NULL );
    XmStringFree( text );
    XmStringFree( yes );
    XmStringFree( no );

    /* Realize the dialog
    ---------------------*/
    XtManageChild( dialog );
    XtPopup( XtParent( dialog), XtGrabNone );


    /* duplicate the main processing loop while
    waiting for a response from the user.
    -------------------------------------------*/
    answer = FALSE;
    while ( answer == FALSE )
        XtAppProcessEvent( app, XtIMAll );

    
    /* Remove the dialog from the screen
    ------------------------------------*/
    XtPopdown( XtParent( dialog ));
    XmUpdateDisplay( parent );

    return answer;
    }  /* END get_user_response */


/****************************************************************************
FUNCTION:	response

PURPOSE:	Call back registered to get_user_response that gets and
		returns the user's response to the dialog -- which button
		the user pushes.

INPUT:	        widget of the caller		
		client data, pointer to an int, the answer
		call data, pointer to CB structure, the user response
		
OUTPUT: 	client data: user's response YES or NO

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void response
    (
    Widget    widget,		/* IN - widget associated with callback */
    XtPointer client_data,	/* IN/OUT - pointer to int, the answer	*/
    XtPointer call_data		/* IN - call back struct for response	*/
    )
    {
    /* cast the client and call data into appropriate values
    --------------------------------------------------------*/
    int *answer = (int *)client_data;
    XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct *)call_data;


    /* check and return the response
    --------------------------------*/
    if ( cbs->reason == XmCR_OK )
        *answer = YES;
    else if ( cbs->reason == XmCR_CANCEL )
        *answer = NO;
    }  /* END response */
