/*****************************************************************************
FUNCTION:	initstruct

PURPOSE:	Initialize the itemlist data structure.

INPUT:		file name of the initialization file
		flag TRUE/FALSE: use default file on open failure

OUTPUT;		None

RETURNS:	struct ITEM *, filled item list

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "adapsmonitor.h"


struct ITEM *initstruct
    ( 
    char    *infile,		/* IN - input file name			*/
    short   defaultflg		/* IN - TRUE/FALSE: use default file	*/
    				/* if infile is not present or corrupt	*/
    )
    {
    char    *adapstables;	/* pointer to environment variable	*/
    char    defaultfile[CMLEN];	/* default file name			*/
    char    inbuf[CMLEN];	/* input buffer				*/
    char    msg[CMLEN];		/* message text for write_log		*/

    FILE    *inf;		/* input file descriptor		*/

    int     linecount;		/* line number of initialization file	*/

    struct  ITEM *curr;		/* current item being added to list	*/
    struct  ITEM *itemlist;	/* pointer to head of itemlist		*/


    /* Attempt to open the file passed as parameter.  Note: we
    can't pop up a dialog box for any of the errors that follow
    since the main app may not have been realized yet.
    -----------------------------------------------------------*/
    inf = fopen( infile, "r" );
    if ( !inf )
        {
	if ( !defaultflg )
	    {
	    /* Write an error to the log file
	    ---------------------------------*/
	    sprintf( msg, "Could not open file: %s", infile );
	    write_log( FALSE, "initstruct", "fopen", msg );
	    return NULL;
	    }
	else
	    {
	    adapstables = getenv( "ADAPSTABLES" );
	    if ( adapstables == NULL )
	        {
		write_log( FALSE, "initstruct", "environment",
		    "ADAPSTABLES not defined" );
		return NULL;
		}
	    sprintf( defaultfile, "%s/%s", adapstables, ADAPSMONITORDEFAULT );

	    /* Write an error to the log file
	    ---------------------------------*/
	    sprintf( msg, "Could not open file: %s trying %s", infile,
	        defaultfile );
	    write_log( FALSE, "initstruct", "fopen", msg );

	    /* Attempt to open default file
	    -------------------------------*/
	    inf = fopen( defaultfile, "r" );
	    if ( !inf )
	        {
		/* Write an error to the log file
		---------------------------------*/
		sprintf( msg, "Could not open file: %s", defaultfile );
		write_log( FALSE, "initstruct", "fopen", msg );
		return NULL;
		}
	    }  /* END else */
	}  /* END if ( !inf ) */

    /* Read all lines in the input file and fill the itemlist
    ---------------------------------------------------------*/
    itemlist = curr = NULL;
    linecount = 0;
    while ( fgets( inbuf, CMLEN, inf ) != NULL )
        {
	/* Skip comments
	----------------*/
	if ( inbuf[0] == '#' )
	    continue;

	/* parse the line and add a new item if it's okay
	-------------------------------------------------*/
	linecount++;
	if ( check_line( inbuf ) == SUCCESS )
	    {
	    /* get a new item record
	    ------------------------*/
	    curr = allocate_item();
	    if ( curr == NULL )
	        {
	        free_itemlist( &itemlist );
		return NULL;
		}

	    /* fill the new item
	    --------------------*/
	    if ( fill_item( inbuf, curr ) == FAIL )
	        {
		free_itemlist( &itemlist );
		return NULL;
		}
	    curr->monitor_state = curr->initial_state;

	    /* Update the main window display strings
	    -----------------------------------------*/
	    create_displist( curr );

	    /* Add the new item to the list
	    -------------------------------*/
	    append_item( &itemlist, curr );

	    }  /* END if ( check_line... */

	/* Otherwise free up the list and return an error
	-------------------------------------------------*/
	else
	    {
	    /* free up the item list
	    ------------------------*/
	    free_itemlist( &itemlist );

	    /* write an error to log
	    ------------------------*/
	    sprintf( msg, "Error initializing item list on line %d", linecount);
	    write_log( FALSE, "initstruct", "inititem", msg );

	    return NULL;
	    }  /* END else */
	}  /* END while */

    /* Return the complete list
    ---------------------------*/
    return itemlist;
    }  /* END initstruct */
