/*****************************************************************************
FUNCTION:	allocate_item
  		free_itemlist
		append_item
		fill_item
		store_list_address
		get_list_address
		store_alarm_address
		get_alarm_address
		copy_itemlist
		duplicate_item

PURPOSE:	A collection of routines to allocate, free, fill, copy,
		and print item structures.  There is also a routine that
		stores and returns the address of the main window item
		structure.

NOTE:		None of these routines popup a message dialog.  The
		first time they are called the application windows
		have not been built yet.

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "adapsmonitor.h"


/*****************************************************************************
FUNCTION:	allocate_item

PURPOSE:	Allocate an ITEM structure and clear all fields.

INPUT:          None

OUTPUT:         None

RETURNS:	struct ITEM *, pointer to a single zeroed ITEM structure

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
struct ITEM *allocate_item( void )
    {
    int    i;			/* loop control				*/
    struct ITEM *item;		/* pointer to new item			*/

    /* Allocate memory for the item
    -------------------------------*/
    item = (struct ITEM *)malloc( sizeof( struct ITEM ));
    if ( !item )
        {
	/* Write error message to log
	-----------------------------*/
        write_log( FALSE, "allocate_item", "alloc", "Error allocating memory" );
	return NULL;
	}

    /* Set initial state and zero out stuff
    ---------------------------------------*/
    item->next = NULL;
    item->prev = NULL;
    item->itemname[0] = '\0';
    item->machine[0] = '\0';
    item->itemtype = UNDEFINED;
    item->mon_interval = 0;
    item->rwast_interval = 0;
    item->initial_state = UNINITIALIZED;
    item->num_batch = 0;
    item->userid[0] = '\0';
    item->log_name[0] = '\0';
    item->monitor_state = UNINITIALIZED;
    item->current_state = UNCHECKED;
    item->last_check = 0;
    item->alarm = FALSE;
    item->rwast_flg = 0;
    item->rwast_time = 0;
    item->job_number = 0;
    item->display[0] = '\0';
    item->monitor_field[0] = '\0';
    item->name_field[0] = '\0';
    item->status_field[0] = '\0';
    item->asoftime_field[0] = '\0';
    item->item_index = 0;
    for ( i = 0; i < 5; i++ )
        item->fieldflg[i] = INVALID;
    item->mon_state_mod = FALSE;
    item->normalbgcolor[0] = '\0';
    item->changebgcolor[0] = '\0';
    return item;
    }  /* END allocate_item */



/*****************************************************************************
FUNCTION:	free_itemlist

PURPOSE:	Free an entire ITEM list structure

INPUT:          pointer to item list to be freed

OUTPUT:         Nulled pointer

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
void free_itemlist
    (
    struct ITEM **itemlist	/* IN/OUT - list structure to free	*/
    )
    {
    struct ITEM *curr;		/* current item	 in the list		*/
    struct ITEM *next;		/* next item in the list		*/

    curr = *itemlist;

    /* while we are not at the end of the list,
    free up the current item.
    -------------------------------------------*/
    while ( curr != NULL )
        {
	next = curr->next;
	free( curr );
	curr = next;
	}

    /* Set the list to NULL to indicate it's been freed
    ---------------------------------------------------*/
    *itemlist = NULL;
    
    }  /* END free_itemlist */


/*****************************************************************************
FUNCTION:	append_item

PURPOSE:	Append a single ITEM structure to the end of the linked list
		of ITEM structures.

INPUT:          pointer to linked item list
		pointer to the item to be added

OUTPUT:         linked list with item added

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
void append_item
    (
    struct ITEM **itemlist,	/* IN/OUT - Item list to append to	*/
    struct ITEM *item		/* IN - Item to append to list		*/
    )
    {
    struct ITEM *curr;		/* pointer to current item in itemlist	*/
    struct ITEM *last;		/* pointer to last item visited in list	*/


    /* If the itemlist is new, just return item
    -------------------------------------------*/
    if ( *itemlist == NULL )
        *itemlist = item;

    /* otherwise add the item to the end of itemlist
    ------------------------------------------------*/
    else
        {
	/* search for the end of the list
	---------------------------------*/
	curr = *itemlist;
	do
	    {
	    last = curr;
	    curr = last->next;
	    }
	while ( curr != NULL );
	
	/* last now points to the last item in
	the list so append the new item to it
	--------------------------------------*/
	last->next = item;
	item->prev = last;
	}
    }  /* END append_item */



/*****************************************************************************
FUNCTION:	fill_item

PURPOSE:	Fill an item structure with values read from the
		initialization file.

INPUT:          buffer of input values read from initialization file
		pointer to the item structure to be filled

OUTPUT:         filled item structure

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
int fill_item
    (
    char   *buf,                /* IN - buffer of input values            */
    struct ITEM *item           /* IN/OUT - pointer to structure to fill  */
    )
    {
    char   item_type[CMLEN];    /* item type from buffer                */
    char   monitor_state[CMLEN];/* initial monitor state                */


    /* scan the nine items in the buffer into appropriate
    items.  Buffer has already been checked for the presense
    of nine values.  No need to check for sscanf errors.
    --------------------------------------------------------*/
    if ( 9 != sscanf( buf, "%s %s %s %d %d %s %d %s %s", item->itemname,
        item->machine, item_type, &item->mon_interval, &item->rwast_interval,
	monitor_state, &item->num_batch, item->userid, item->log_name ))
	return FAIL;

    /* Convert monitor intervals to seconds
    ---------------------------------------*/
    item->mon_interval *= 60;
    item->rwast_interval *= 60;

    /* Convert item type string to an enumerated type
    -------------------------------------------------*/
    item->itemtype = get_item_type( item_type );

    /* Convert initial state to an enumerated type
    ----------------------------------------------*/
    if ( strcmp( monitor_state, "enabled" ) == 0 )
        item->initial_state = ENABLED;
    else
        item->initial_state = DISABLED;
    
    return SUCCESS;
    }  /* END fill_item */


static struct ITEM *current_itemlist_address = NULL;
/*****************************************************************************
FUNCTION:	store_list_address

PURPOSE:	To store the address of the list structure.
		Allows access to callback routines that are unable to
		access it otherwise.

INPUT:		The address of the list to save for future calls

OUTPUT:         None

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	09/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
void store_list_address
    (
    struct ITEM *listaddress	/* IN - address to store or NULL	*/
    )
    {
    current_itemlist_address = listaddress;
    }  /* END store_list_address */


/*****************************************************************************
FUNCTION:	get_list_address

PURPOSE:	To retrieve the address of the list structure.
		Allows access to callback routines that are unable to
		access it otherwise.

INPUT:		none

OUTPUT:         None

RETURNS:	struct ITEM *, The stored address or NULL if not previously set

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	09/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
struct ITEM *get_list_address
    (
    void
    )
    {
    return current_itemlist_address;
    }  /* END get_list_address */


static struct ITEM *current_alarm_address = NULL;
/*****************************************************************************
FUNCTION:	get_alarm_address

PURPOSE:	To retrieve the address of the alarm list structure.
		This allows access to callback routines that are unable to
		access it otherwise.

INPUT:		None

OUTPUT:         None

RETURNS:	struct ITEM *, The stored address or NULL if not previously set

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	09/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
struct ITEM *get_alarm_address
    (
    void
    )
    {
    return current_alarm_address;
    }  /* END get_alarm_address */


/*****************************************************************************
FUNCTION:	store_alarm_address

PURPOSE:	To store the address of the alarm list structure.
		Allows access to callback routines that are unable to
		access it otherwise.

INPUT:		The address of the list to save for future calls

OUTPUT:         None

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	08/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
void store_alarm_address
    (
    struct ITEM *listaddress	/* IN - address to store or NULL	*/
    )
    {
    current_alarm_address = listaddress;
    }  /* END store_alarm_address */


/*****************************************************************************
FUNCTION:	copy_itemlist

PURPOSE:	Make a complete copy of an item list

INPUT:          pointer to an ITEM structure

OUTPUT:         None

RETURNS:	struct ITEM *, pointer to a ITEM structure

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
struct ITEM *copy_itemlist
    (
    struct ITEM *inlist		/* IN - list to be copied		*/
    )
    {
    struct ITEM *curr;		/* pointer to current item in outlist	*/
    struct ITEM *last;		/* the last item added to outlist	*/
    struct ITEM *next;		/* the next item in inlist to copy	*/
    struct ITEM *outlist = NULL;/* copy of input list			*/


    next = inlist;
    while ( next != NULL )
        {
	/* Allocate and make a copy of the current item
	-----------------------------------------------*/
	curr = allocate_item();
	if ( curr == NULL )
	    return NULL;
	*curr = *next;

	/* This is the first item in the list so
	point all the pointers to the new item.
	----------------------------------------*/
	if ( outlist == NULL )
	    outlist = last = curr;

	/* Otherwise it's a second or subsequent item
	so set up the pointers appropriately.
	---------------------------------------------*/
	else
	    {
	    last->next = curr;
	    curr->prev = last;
	    last = curr;
	    }

	/* Go to the next item in the list
	----------------------------------*/
	next = next->next;
	}  /* END while */

    return outlist;
    }  /* END copy_itemlist */



#if DEBUG
/***************************************************************************
FUNCTION:	print_list

PURPOSE:	Diagnostic routine to print the contents of a list to a file

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void print_list
    (
    struct ITEM *item,		/* IN - item list to print		*/
    char *filename		/* IN - file name to print to		*/
    )
    {
    FILE *ouf;			/* output file descriptor		*/
    struct ITEM *curr;		/* pointer into the item list		*/
    int i;

    /* open the output file
    -----------------------*/
    ouf = fopen( filename, "w" );
    if ( !ouf )
        return;
    
    curr = item;
    while ( curr != NULL )
        {
	fprintf( ouf, "next: %x\nprev: %x\n", curr->next, curr->prev );
	fprintf( ouf, "itemname: %s\nmachine: %s\n", curr->itemname,
	    curr->machine );
	switch ( curr->itemtype )
	    {
	    case UNDEFINED :
	        fprintf( ouf, "itemtype: UNDEFINED\n" ); break;
	    case BATCH :
	        fprintf( ouf, "itemtype: BATCH\n" ); break;
	    case PROCESS :
	        fprintf( ouf, "itemtype: PROCESS\n" ); break;
	    case VAXPROCESS :
	        fprintf( ouf, "itemtype: VAXPROCESS\n" ); break;
	    case SYSTEM :
	        fprintf( ouf, "itemtype: SYSTEM\n" ); break;
	    case DATAFILE :
	        fprintf( ouf, "itemtype: DATAFILE\n" ); break;
	    default :
	        fprintf( ouf, "itemtype: Unknown\n" );
	    }
	fprintf( ouf, "mon_interval: %d\nrwast_interval: %d\n",
	    curr->mon_interval, curr->rwast_interval );
	switch ( curr->initial_state )
	    {
	    case UNINITIALIZED :
	        fprintf( ouf, "initial_state: UNINITIALIZED\n" ); break;
	    case ENABLED :
	        fprintf( ouf, "initial_state: ENABLED\n" ); break;
	    case DISABLED :
	        fprintf( ouf, "initial_state: DISABLED\n" ); break;
	    default :
	        fprintf( ouf, "initial_state: Unknown\n" );
	    }
	fprintf( ouf, "num_batch: %d\nuserid: %s\nlog_name: %s\n",
	    curr->num_batch, curr->userid, curr->log_name );
	switch ( curr->monitor_state )
	    {
	    case UNINITIALIZED :
	        fprintf( ouf, "monitor_state: UNINITIALIZED\n" ); break;
	    case ENABLED :
	        fprintf( ouf, "monitor_state: ENABLED\n" ); break;
	    case DISABLED :
	        fprintf( ouf, "monitor_state: DISABLED\n" ); break;
	    default :
	        fprintf( ouf, "monitor_state: Unknown\n" );
	    }
	switch ( curr->current_state )
	    {
	    case UNCHECKED :
	        fprintf( ouf, "current_state: UNCHECKED\n" ); break;
	    case RUNNING :
	        fprintf( ouf, "current_state: RUNNING\n" ); break;
	    case UPTODATE :
	        fprintf( ouf, "current_state: UPTODATE\n" ); break;
	    case NOTINQUEUE :
	        fprintf( ouf, "current_state: NOTINQUEUE\n" ); break;
	    case RWAST :
	        fprintf( ouf, "current_state: RWAST\n" ); break;
	    case SUSPENDED :
	        fprintf( ouf, "current_state: SUSPENDED\n" ); break;
	    case HUNG :
	        fprintf( ouf, "current_state: HUNG\n" ); break;
	    case NOTRESPONDING :
	        fprintf( ouf, "current_state: NOTRESPONDING\n" ); break;
	    default :
	        fprintf( ouf, "current_state: Unknown\n" );
	    }
	fprintf( ouf, "last_check: %d\nrwast_flg: %d\nrwast_time: %d\n",
	    (int)curr->last_check, curr->rwast_flg, (int)curr->rwast_time );
	fprintf( ouf, "job_number: %d\ndisplay: %s\n", curr->job_number,
	    curr->display );
	for ( i = 0; i < 5; i++ )
	    fprintf( ouf, "fieldflg[%d]: %d\n", i, curr->fieldflg[i] );

	fprintf( ouf, "monitor_field: %s\n", curr->monitor_field );
	fprintf( ouf, "name_field: %s\n", curr->name_field );
	fprintf( ouf, "status_field: %s\n", curr->status_field );
	fprintf( ouf, "asoftime_field: %s\n", curr->asoftime_field );
	fprintf( ouf, "item_index: %d\n\n", curr->item_index );

        curr = curr->next;
	}

    fclose( ouf );
    }  /* END print_list */
#endif
