/******************************************************************************
FUNCTION:	adapsmonitor (main)

PURPOSE:	The adapsmonitor keeps watch on critical ADAPS processes,
  		systems and files.  The user is kept informed through
		display windows and alerted when a monitored item doesn't
		appear to be working or updated properly.

		The main function gets things started by initializing the
		data structures and window widgets then entering an event
		driven loop.

INPUT:          optional command line switches
		    -f <initialization file name>
		    -interval <monitor interval in seconds>
		optional resource file
		    Adapsmonitor.ad

OUTPUT:         messages are logged in $ADAPSLOG/adapsmonitor.log
		changes to the monitor's state are stored in $ADAPSTABLES/
		    adapsmonitor.state
		display windows

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
******************************************************************************/
#include <stdlib.h>
#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <Xm/FileSB.h>
#include <X11/Shell.h>
#include "adapsmonitor.h"

/* Global variables
-------------------*/
XtAppContext app;		/* main application context		*/
Widget	toplevel;		/* top level application shell		*/


void main
    (
    int  argc,			/* IN - count of arguments passed in	*/
    char *argv[]		/* IN - argument array			*/
    )
    {
    Widget   fileselectwindow;	/* widget for file selection		*/
    Widget   modifyshell;	/* shell widget for modify window	*/

    struct   ITEM *itemlist;	/* list of items to monitor		*/

    unsigned long timer_interval;/* check status interval in millisec	*/

    #include "fallbacks.h"	/* resource strings to fall back on if	*/
    				/* user doesn't specify them.		*/


    /* Write a startup message to log
    ---------------------------------*/
    write_log( FALSE, "main", "startup", "ADAPSMONITOR started" );
    
    /* Check for a user specified timer interval,
    if none specified use the default interval.
    --------------------------------------------*/
    timer_interval = get_timer_interval( argv );
    if ( timer_interval == 0 )
        timer_interval = DEFAULTINTERVAL;

    /* Set up the item list with the initialization file or the default.
    -------------------------------------------------------------------*/
    itemlist = initialize( argv );
    if ( itemlist == NULL )
        {
	/*write an error message */
	write_log( TRUE, "main", "init", "Initialization error encountered" );
	exit(1);
	}
    /* Save the address for access by others
    ----------------------------------------*/
    store_list_address( itemlist );


    /* Set up the user's locale and get a top level shell
    -----------------------------------------------------*/
    XtSetLanguageProc( NULL, NULL, NULL );
    toplevel = XtVaAppInitialize( &app, WnADAPSMONITOR, NULL, 0,
        &argc, argv, fallbacks, NULL );

    
    /* Create a file selection window
    ---------------------------------*/
    fileselectwindow = XmCreateFileSelectionDialog( toplevel, WnFILESELECT,
        NULL, 0 );
    XtUnmanageChild( XmFileSelectionBoxGetChild( fileselectwindow, 
        XmDIALOG_HELP_BUTTON ));


    /* Create the main display window
    ---------------------------------*/
    create_main_display( toplevel, itemlist );
    

    /* Create the Modify window.  Don't realize the window since
    that's the job of the main window modify button call back.
    ------------------------------------------------------------*/
    modifyshell = XtVaCreatePopupShell( WnMODIFYWIND,
        xmDialogShellWidgetClass,
        toplevel,
	NULL );
    create_modify( modifyshell, itemlist );


    /* Issue a short duration timer event so that the check_status
    function will be run to initialize the status of items.
    --------------------------------------------------------------*/
    XtAppAddTimeOut( app, 1000, checkstatus_callback, &timer_interval );

    
    /* Make the main window visible and enter event loop
    ----------------------------------------------------*/
    XtRealizeWidget( toplevel );
    XtAppMainLoop( app );
    }  /* END main */
