/*****************************************************************************
FUNCTION:	modify_callback
		cancel_modify_callback
		apply_modify_callback

LOCAL FUNCTION:	remove_modify_callbacks
		get_modify_childbutton

PURPOSE:	Call back routines for the modify window.

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <Xm/List.h>
#include "adapsmonitor.h"

/* Global variables
-------------------*/
extern XtAppContext app;        /* main application context             */


/* Local function definitions
-----------------------------*/
static void remove_modify_callbacks
    (
    Widget w,			/* IN - calling widget value		*/
    struct ITEM *item		/* IN - pointer to item structure       */
    );
static Widget get_modify_childbutton
    (
    Widget w,			/* IN - calling widget value		*/
    char widget_name[]		/* IN - name of widget to get           */
    );


/****************************************************************************
FUNCTION:	modify_callback

PURPOSE:	Realize the modify window, create a copy of the item list
		and register callbacks for the Cancel and Apply buttons
		with the item list.  This function also creates the row
		column child of the modify window's scrolled window widget.

INPUTS:		calling widget data

OUTPUTS:	none

RETURNS:	none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void modify_callback
    (
    Widget    w,		/* IN - widget associated with callback	*/
    XtPointer client_data,	/* IN - client data			*/
    XtPointer call_data		/* IN - call data			*/
    )
    {
    char   wstring[CMLEN];	/* representation of widget tree	*/

    struct ITEM *itemcopy;	/* copy of the item list		*/
    struct ITEM *itemlist;	/* current item list			*/

    Widget rowcol;		/* child of the scrolled window		*/

    
    /* Make a copy of the item list
    -------------------------------*/
    itemlist = get_list_address();
    itemcopy = copy_itemlist( itemlist );

    /* Create the contents of the scrolled window of the
    modify window.  The contents are created here because
    they have callbacks associated with the item list copy. 
    ------------------------------------------------------*/
    rowcol = create_toggle_array( scroll_window_child( w, WnSCROLLEDWINDOW1 ),
        itemcopy, WnMODIFY );
    store_rowcol_widget( rowcol );

    /* register callbacks to Cancel, Load and Apply buttons
    ------------------------------------------------------*/
    XtAddCallback( get_modify_childbutton( w, WnAPPLYBUTTON ),
        XmNactivateCallback, apply_modify_callback, itemcopy );
    XtAddCallback( get_modify_childbutton( w, WnCANCELBUTTON ),
        XmNactivateCallback, cancel_modify_callback, itemcopy );
    XtAddCallback( get_modify_childbutton( w, WnLOADBUTTON ),
        XmNactivateCallback, load_newfile_callback, itemcopy );

    /* Create the resource name of the
    modify window and manage the widget.
    ------------------------------------*/
    sprintf( wstring, "%s.%s", WnMODIFYWIND, WnMODIFY );
    XtManageChild( XtNameToWidget( get_toplevel_shell( w ), wstring ));
    } /* END modify_callback */


/****************************************************************************
FUNCTION:	cancel_modify_callback

PURPOSE:	Remove the modify window without applying any changes

INPUTS:		calling widget data
		pointer to item structure in client_data

OUTPUTS:	none

RETURNS:	none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void cancel_modify_callback
    (
    Widget w, 			/* IN - widget associated with callback */
    XtPointer client_data, 	/* IN - pointer to item structure	*/
    XtPointer call_data 	/* IN - call data                       */
    )
    {
    char   wstring[CMLEN];	/* representation of widget tree	*/

    struct ITEM *itemlist = (struct ITEM *)client_data; /* the item list*/


    /* Remove the callback procedures for the modify window
    -------------------------------------------------------*/
    remove_modify_callbacks( w, itemlist );

    /* Free the item list
    ---------------------*/
    free_itemlist( &itemlist );

    /* destroy the rowcol window
    ----------------------------*/
    XtDestroyWidget( get_rowcol_widget());

    /* remove the modify window
    ---------------------------*/
    sprintf( wstring, "%s.%s", WnMODIFYWIND, WnMODIFY );
    XtUnmanageChild( XtNameToWidget( get_toplevel_shell( w ), wstring ));
    } /* END cancel_modify_callback */


/****************************************************************************
FUNCTION:	apply_modify_callback

PURPOSE:	Apply changes to the list structure by replacing the 
		current list with a new one.  Also updates the main
		window display with changes.

INPUTS:		calling widget data
		client data - pointer to item list

OUTPUTS:	Stores a pointer to the new list

RETURNS:	none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
void apply_modify_callback
    (
    Widget w,			/* IN - widget associated with callback */
    XtPointer client_data,	/* IN - pointer to item structure	*/
    XtPointer call_data		/* IN - call data                       */
    )
    {
    Widget scrolledlist;	/* the window containing the list	*/

    char   wstring[CMLEN];	/* representation of widget tree	*/

    struct ITEM *curr;		/* pointer into item list		*/
    struct ITEM *newitemlist = (struct ITEM *)client_data; /* new list  */
    struct ITEM *olditemlist = get_list_address(); /* the current list	*/

    static unsigned long interval = 0;	/* interval of zero so that the	*/
    				/* timer issued later doesn't spawn	*/
				/* another timer.  Needs to be static	*/
				/* since we pass its address to the CB	*/

    /* Replace the old data structure with the new one
    and free memory associated with the old one.
    --------------------------------------------------*/
    store_list_address( newitemlist );
    free_itemlist( &olditemlist );

    /* Update the main window.  If the list is new because of a
    Load, the current display list must be destroyed before updating
    with new information.  If current items are not deleted the new
    data will be incorrectly appended to the old data.
    ----------------------------------------------------------------*/
    /* get the scrolled list widget */
    sprintf( wstring, "%s.%s.scrolledlistSW.%s", WnMAINW, WnMWFORM2,
	WnSCROLLEDLIST );
    scrolledlist = XtNameToWidget( get_toplevel_shell( w ), wstring );
    if ( newitemlist->item_index == 0 )
	{
	XmListDeleteAllItems( scrolledlist );

	/* Also since this is a new list, issue a short
	duration timer event to initialize the status of the new items
	----------------------------------------------------------*/
	XtAppAddTimeOut( app, 10, checkstatus_callback, &interval );
	}
    /* update the display string for each item */
    curr = newitemlist;
    while ( curr != NULL )
	{
	create_displist( curr );
	curr = curr->next;
	}
    update_displist( scrolledlist, newitemlist );
    /* The current state of the monitor
    has changed so save the state.
    -----------------------------------*/
    save_state( newitemlist );

    /* Remove the callback procedures for the modify window
    -------------------------------------------------------*/
    remove_modify_callbacks( w, newitemlist );

    /* destroy the rowcol window
    ----------------------------*/
    XtDestroyWidget( get_rowcol_widget());

    /* remove the modify window
    ---------------------------*/
    sprintf( wstring, "%s.%s", WnMODIFYWIND, WnMODIFY );
    XtUnmanageChild( XtNameToWidget( get_toplevel_shell( w ), wstring ));
    }  /* END apply_modify_callback */



/**************************************************************************
FUNCTION:	remove_modify_callbacks

PURPOSE:	To remove the callback functions registered to the
		Apply and Cancel buttons of the modify window.

INPUTS:		calling widget value
                pointer to item structure

OUTPUTS:	none

RETURNS:	none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
***************************************************************************/
void remove_modify_callbacks
    (
    Widget w,			/* IN - calling widget value		*/
    struct ITEM *item		/* IN - pointer to item structure	*/
    )
    {
    /* remove the apply button callback
    -----------------------------------*/
    XtRemoveCallback( get_modify_childbutton( w, WnAPPLYBUTTON ),
        XmNactivateCallback, apply_modify_callback, item );

    /* remove the cancel button callback
    ------------------------------------*/
    XtRemoveCallback( get_modify_childbutton( w, WnCANCELBUTTON ),
        XmNactivateCallback, cancel_modify_callback, item );

    /* remove the load button callback
    ------------------------------------*/
    XtRemoveCallback( get_modify_childbutton( w, WnLOADBUTTON ),
        XmNactivateCallback, load_newfile_callback, item );

    }  /* END remove_modify_callbacks */


/***************************************************************************
FUNCTION:	get_modify_childbutton

PURPOSE:	Retrieve the widget of a button on the modify window.

INPUTS:		calling widget value
                string: widget name of button

OUTPUTS:	none

RETURNS:	button widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
***************************************************************************/
Widget get_modify_childbutton
    (
    Widget w,			/* IN - calling widget value		*/
    char widget_name[]		/* IN - name of widget to get		*/
    )
    {
    Widget buttonform;		/* the modify button form widget	*/

    char   wstring[CMLEN];	/* representation of widget tree	*/


    /* Get the button form of the modify window
    -------------------------------------------*/
    sprintf( wstring, "%s.%s.%s", WnMODIFYWIND, WnMODIFY, WnBUTTONFORM );
    buttonform = XtNameToWidget( get_toplevel_shell( w ), wstring );

    /* Now get and return the appropriate child widget
    -------------------------------------------------*/
    return XtNameToWidget( buttonform, widget_name );
    }  /* END get_modify_childbutton */
