#include <stdlib.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qcolor.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include "monitorcontrol.h"
#include "ColoredTableItem.h"
#include "ProcessExecution.h"


static QColor unk_color   = QColor( 250, 230, 110 );
static QColor grn_color   = QColor( 175, 255, 160 );
static QColor ltgrn_color = QColor( 216, 228, 255 );
static QColor blue_color  = QColor( 178, 199, 255 );
static QColor stop_color  = QColor( 255, 80,   80 );
static QColor black_color = QColor(   0,   0,   0 );
static QColor white_color = QColor( 255, 255, 255 );
static QColor hang_color  = QColor( 255, 109,  68 );

// Constructor
MonitorControl::MonitorControl( QWidget* parent, const char* name,
    WFlags fl )
    : MonitorControlBase( parent, name, fl )
{
    // Get environment variables
    ADAPSTABLES = getenv( "ADAPSTABLES" );
    ADAPSBASE   = getenv( "ADAPSBASE" );
    ADAPSLOG    = getenv( "ADAPSLOG" );

    // QProcess passes on | with no special meaning it also doesn't expand *,
    // so the following shell script does the ls piped into grep for us like
    // this:
    //( "/bin/ls -gG --time-style=long-iso | grep -E '(dat|timcor)'" );
    // Hard-coded to my directory now and should be placed in ADAPSBASE
    file_cmd = ADAPSBASE + "/file_check";
    // This script runs this command
    // /bin/ps -u opsavhrr -o pid,ppid,cmd | grep ADAPS
    proc_cmd = ADAPSBASE + "/daemon_check";
    // This contains the commands to execute a daemon process
    spawn_cmd = ADAPSBASE + "/spawn_ADAPS_daemon";

    // Set column headings
    QStringList labels;
    labels.append( "Status" );
    labels.append( "Process Name" );
    labels.append( "Control" );
    proc_table->setColumnLabels( labels );
    labels.clear();
    labels.append( "Status" );
    labels.append( "File Name" );
    labels.append( "Control" );
    file_table->setColumnLabels( labels );

    // Set up a timer
    timex = new QTimer( this );

    // Set up the label tags and their colors
    // NOTE: there must be a one-to-one correspondence with these.
    tags[RUNNING]    = "Running";     colors[RUNNING]    = grn_color;
    tags[IGNORE]     = "Ignore";      colors[IGNORE]     = ltgrn_color;
    tags[START]      = "Start";       colors[START]      = blue_color;
    tags[STOP]       = "Stop";        colors[STOP]       = stop_color;
    tags[STOPPEND]   = "Stop Pending";colors[STOPPEND]   = blue_color;
    tags[STOPPED]    = "Stopped";     colors[STOPPED]    = stop_color;
    tags[RESTART]    = "Restart";     colors[RESTART]    = stop_color;
    tags[KILL]       = "Kill";        colors[KILL]       = black_color;
    tags[UPTODATE]   = "Up to date";  colors[UPTODATE]   = grn_color;  
    tags[STOWED]     = "Stowed";      colors[STOWED]     = grn_color;
    tags[REMOTE]     = "Remote";      colors[REMOTE]     = grn_color;
    tags[NOTRUNNING] = "Not Running"; colors[NOTRUNNING] = stop_color; 
    tags[KILLED]     = "Killed";      colors[KILLED]     = black_color;
    tags[NOTUPTODATE]= "Out of date"; colors[NOTUPTODATE]= stop_color; 
    tags[NOTSTOWED]  = "Not stowed";  colors[NOTSTOWED]  = stop_color; 
    tags[LOCAL]      = "Local";       colors[LOCAL]      = stop_color;
    tags[HANG]       = "Process hang";colors[HANG]       = hang_color;

    // Connections
    connect( exit_button, SIGNAL( clicked() ), this, SLOT( close() ) );
    connect( proc_table, SIGNAL( valueChanged( int, int )), 
             this, SLOT( proc_change( int, int )));
    connect( file_table, SIGNAL( valueChanged( int, int )), 
             this, SLOT( file_change( int, int )));
    connect( check_button, SIGNAL( clicked() ), this, SLOT( testem() ) );
    // connect the timer to a slot
    connect( timex, SIGNAL( timeout() ), this, SLOT( testem() ) );
    // connect the start and stop all buttons
    connect( pushButton5, SIGNAL( clicked() ), this, SLOT( all_start() ));
    connect( pushButton6, SIGNAL( clicked() ), this, SLOT( all_stop() ));

    // Initialize the monitor structure and the tables
    if ( init() )
    {
        QMessageBox::critical(this, "AMC - Error", "Cannot initialize" );
        this->close();
    }

    // Test the daemons, files and ACU the first time.  A single-shot timer
    // is set at the end of the method.
    testem();

} // END constructor

MonitorControl::~MonitorControl()
{
} // END destructor


bool MonitorControl::init()
{
    QStringList procs;
    QStringList files;
    QStringList acu;
    // Read from a configuration file to fill the tables
    if ( read_resource() )
        return true;
    else
    {
        check();
        QStringList line;
        QStringList::iterator it;
        int i = 0;
        for( it = monitor_defaults.begin();
             it != monitor_defaults.end(); it++, i++ )
        {
            //qDebug( *it );
            line = QStringList::split( '|', *it );
            //qDebug( line[1] );
            monitor_struct[i].type = line[0];
            monitor_struct[i].name = line[1];
            monitor_struct[i].default_interval = line[2].toInt();
            monitor_struct[i].interval = monitor_struct[i].default_interval;
            monitor_struct[i].file = line[3];
            monitor_struct[i].dsp_flg = 0;
            monitor_struct[i].pid = -1;
            monitor_struct[i].ppid = -1;
            monitor_struct[i].stop_pending = false;
            monitor_struct[i].kill_pending = false;
            if ( line[0] == "daemon" )
            {
                procs.append( line[1] );
                monitor_struct[i].itype = 0;
                monitor_struct[i].row = procs.count() - 1;
                if ( process_running( &monitor_struct[i], true ))
                {
                    ; // do nothing
//qDebug( monitor_struct[i].name + " is running " );
//qDebug( "pid = %d", monitor_struct[i].pid );
                }
            }
            else if ( line[0] == "file" )
            {
                files.append( line[1] );
                monitor_struct[i].itype = 1;
                monitor_struct[i].row = files.count() - 1;
            }
            else if ( line[0] == "acu" )
            {
                acu.append( line[1] );
                monitor_struct[i].itype = 2;
                monitor_struct[i].row = acu.count() - 1;
            }
        }
        the_count = i;
    } 

    int numrows = procs.count();
    int numrows2 = files.count();
    int numrows3 = acu.count();

    unsigned int len = 0;
    for ( int r = 0; r < numrows; r++ )
        if ( procs[r].length() > len )
            len = procs[r].length();
    for ( int r = 0; r < numrows2; r++ )
        if ( files[r].length() > len )
            len = files[r].length();
    for ( int r = 0; r < numrows3; r++ )
        if ( acu[r].length() > len )
            len = acu[r].length();
    proc_table->setColumnWidth( 1, len*8 );
    file_table->setColumnWidth( 1, len*8 );
    acu_table->setColumnWidth( 1, len*8 );

    QStringList combo;
    combo.append( "Monitor" );
    combo.append( "Ignore" );
    combo.append( "Start" );
    combo.append( "Stop" );
    combo.append( "Restart" );
    combo.append( "Kill" );
    QStringList combo2;
    combo2.append( "Monitor" );
    combo2.append( "Ignore" );

    proc_table->setNumRows( numrows );
    for ( int r = 0; r < numrows; r++ )
    {

        ColoredTableItem *c0 = new ColoredTableItem( proc_table,
            QTableItem::Never, "Unknown" );
        proc_table->setItem( r, 0, (QTableItem *)c0 );
        c0->setBackgroundColor( unk_color );
        ColoredTableItem *c1 = new ColoredTableItem( proc_table,
            QTableItem::Never, procs[r] );
        proc_table->setItem( r, 1, (QTableItem *)c1 );
        c1->setBackgroundColor( unk_color );
        QComboTableItem *c2 = new QComboTableItem( proc_table, combo );
        proc_table->setItem( r, 2, (QTableItem *)c2 );
    }


    file_table->setNumRows( numrows2 );
    for ( int r = 0; r < numrows2; r++ )
    {
        ColoredTableItem *c0 = new ColoredTableItem( file_table,
            QTableItem::Never, "Up to Date" );
        file_table->setItem( r, 0, (QTableItem *)c0 );
        c0->setBackgroundColor( grn_color );
        ColoredTableItem *c1 = new ColoredTableItem( file_table,
            QTableItem::Never, files[r] );
        file_table->setItem( r, 1, (QTableItem *)c1 );
        c1->setBackgroundColor( grn_color );
        QComboTableItem *c2 = new QComboTableItem( file_table, combo2 );
        file_table->setItem( r, 2, (QTableItem *)c2 );
    }


    acu_table->setNumRows( numrows3 );
    QStringList aculist;
    aculist.append( "Up to Date" );
    aculist.append( "Stowed" );
    aculist.append( "Remote" );
    for ( int r = 0; r < numrows3; r++ )
    {
        ColoredTableItem *c0 = new ColoredTableItem( acu_table,
            QTableItem::Never, aculist[r] );
        acu_table->setItem( r, 0, (QTableItem *)c0 );
        c0->setBackgroundColor( grn_color );
        ColoredTableItem *c1 = new ColoredTableItem( acu_table,
            QTableItem::Never, acu[r] );
        acu_table->setItem( r, 1, (QTableItem *)c1 );
        c1->setBackgroundColor( grn_color );
        //QComboTableItem *c2 = new QComboTableItem( acu_table, combo2 );
        //acu_table->setItem( r, 2, (QTableItem *)c2 );
    }

    return false;
}

void MonitorControl::proc_change( int row, int col )
{
    response( row, col, A_DAEMON );
}

void MonitorControl::file_change( int row, int col )
{
    response( row, col, A_FILE );
}

void MonitorControl::acu_change( int row, int col )
{
    response( row, col, AN_ACU );
}

void MonitorControl::response( int row, int col, OBJ_TYPE obj_type )
{
    QColor color;
    QColor fore;
    QStringList combo;
    combo.append( "Monitor" );
    combo.append( "Ignore" );
    combo.append( "Start" );
    combo.append( "Stop" );
    combo.append( "Restart" );
    combo.append( "Kill" );
    QString str;
    int index = -1;
    int action = -1;
    bool update_dsp = true;

    if ( col != 2 )
        return;

    // Get the monitor structure
    for ( int i = 0; i < the_count; i++ )
    {
        if ( monitor_struct[i].type == "daemon" && 
             monitor_struct[i].row == row && obj_type == A_DAEMON )
        {
            index = i;
            action = 0;
        }
        else if ( monitor_struct[i].type == "file" &&
                  monitor_struct[i].row == row && obj_type == A_FILE )
        {
            index = i;
            action = 0;
        }
        else if ( monitor_struct[i].type == "acu" &&
                  monitor_struct[i].row == row && obj_type == AN_ACU )
        {
            index = i;
            action = 20;
        }
    }
    if ( index < 0 )
    {
qDebug( "This shouldn't happen but it did! the_count %d", the_count );
        return;  // This shouldn't happen
    }
//qDebug( "index = %d", index );

    // Get the table item
    QComboTableItem *c;
    ColoredTableItem *c0;
    ColoredTableItem *c1;
    switch ( obj_type )
    {
        case A_DAEMON :
            c = (QComboTableItem *)proc_table->item( row, col );
            c0 = (ColoredTableItem *)proc_table->item( row, 0 );
            c1 = (ColoredTableItem *)proc_table->item( row, 1 );
            break;
        case A_FILE :
            c = (QComboTableItem *)file_table->item( row, col );
            c0 = (ColoredTableItem *)file_table->item( row, 0 );
            c1 = (ColoredTableItem *)file_table->item( row, 1 );
            break;
        case AN_ACU :
            c = (QComboTableItem *)acu_table->item( row, col );
            c0 = (ColoredTableItem *)acu_table->item( row, 0 );
            c1 = (ColoredTableItem *)acu_table->item( row, 1 );
            break;
        case UNK :
        default  :
            return; // this shouldn't happen
    }
    QString s = c->currentText();
    action += c->currentItem();
//printf( "%d\n", action );
    switch ( action )
    {
        case 0 : // monitor
            color = unk_color;
            fore = black_color;
            str = combo[0];
            monitor_struct[index].interval =
                monitor_struct[index].default_interval;
            break;
        case 1 : // ignore
            color = ltgrn_color;
            fore = black_color;
            str = combo[1];
            monitor_struct[index].interval = -1;
            break;
        case 2 : // start
            color = blue_color;
            fore = black_color;
            str = combo[2];
            monitor_struct[index].interval =
                monitor_struct[index].default_interval;
//qDebug( "calling start_daemon with: " + monitor_struct[index].name );
            if ( start_daemon( &monitor_struct[index] ))
            {
                color = grn_color;
//qDebug( "start_daemon returned true" );
            }
            break;
        case 3 : // stop
            color = blue_color;
            fore = black_color;
            //str = combo[3];
            str = "Stop Pending";
            monitor_struct[index].interval =
                monitor_struct[index].default_interval;
            if ( stop_daemon( &monitor_struct[index] ))
            {
                color = grn_color;
//qDebug( "stop_daemon returned true" );
            }
            break;
        case 4 : // restart
            if ( restart_daemon( &monitor_struct[index] ))
            {
                update_dsp = false;
            }
            else
            {
                color = stop_color;
                fore = black_color;
                str = combo[4];
                monitor_struct[index].interval =
                    monitor_struct[index].default_interval;
            }
            break;
        case 5 : // kill
            color = black_color;
            fore = white_color;
            str = "Kill Pending";
            monitor_struct[index].interval =
                monitor_struct[index].default_interval;
            if ( kill_daemon( &monitor_struct[index] ) == 1 )
                update_dsp = false;
            else
            break;
        case 20 :
        case 21 :
        default :
            break;
    }

    if ( update_dsp )
    {
        //ColoredTableItem *c0 = (ColoredTableItem *)proc_table->item( row, 0 );
        switch ( obj_type )
        {
            case A_DAEMON :
                proc_table->setText( row, 0, str );
                // Don't want to mess with the painter so
                // set the text to force a paint
                str = proc_table->text( row, 1 );
                proc_table->setText( row, 1, str );
                break;
            case A_FILE :
                file_table->setText( row, 0, str );
                str = file_table->text( row, 1 );
                file_table->setText( row, 1, str );
                break;
            case AN_ACU :
                acu_table->setText( row, 0, str );
                str = acu_table->text( row, 1 );
                acu_table->setText( row, 1, str );
                break;
            case UNK:
            default :
                ;// shouldn't happen
        }
        c0->setBackgroundColor( color );
        c0->setForegroundColor( fore );
        c1->setBackgroundColor( color );
        c1->setForegroundColor( fore );
    }

//qDebug( s );
}

void MonitorControl::check( void )
{
    process_list = exec_cmd( QString( proc_cmd ));
    //QStringList::iterator it;
    //for( it = process_list.begin(); it != process_list.end(); ++it )
        //qDebug( *it );

    file_list = exec_cmd( QString( file_cmd ));
    //for( it = file_list.begin(); it != file_list.end(); ++it )
        //qDebug( *it );
}


QStringList MonitorControl::exec_cmd( QString cmd )
{
    QStringList stdout_list;
    ProcessExecution pe( this );

    if( !pe.execute(cmd) )
    {
        // Grab the errors
        QStringList::iterator it;
        QStringList errorList = pe.getStderr();
        QString errors( "" );
        for( it = errorList.begin(); it != errorList.end(); ++it )
        {
            errors += (*it) + "\n";
        }
        QString msg = tr("Failed to execute command:\n") +
                      cmd + "\n" + errors;
        QMessageBox::critical(this, "OUI - " + tr("Error"), msg);
        return "error";
    }
    else
    {
        stdout_list = pe.getStdout();
    }

    return stdout_list;

}  // END get_pids


bool MonitorControl::read_resource( void )
{
    QString filename =
        QString( ADAPSTABLES + "/moncontrolrc" );
    QFile resource( filename );
    if ( !resource.open( IO_ReadOnly ))
    {
        QString msg = "Cannot open resource file";
        QMessageBox::critical(this, "AMC - Error", msg );
        return true;
    }

    QTextStream stream( &resource );
    QString line;
    while ( !stream.atEnd() )
    {
        line = stream.readLine();
        // Add those lines that aren't comments
        if ( line[0] != '#' )
            monitor_defaults.append( line );
    }

    resource.close();
    return false;
}

void MonitorControl::testem( void )
{
//qDebug( "Checking system now" );
    check();
//process_list file_list
    //process_list = exec_cmd( QString( proc_cmd ));


    for ( int i = 0; i < the_count; i++ )
    {
        QString type = monitor_struct[i].type;
        if ( type == "daemon" )
        {
//if (monitor_struct[i].name == "bwtrans") qDebug("***** bwtrans start *****");
            if ( monitor_struct[i].interval < 0 )
//qDebug("in the empty if")
                ;  // Do nothing
            else if ( monitor_struct[i].kill_pending )
            {
//qDebug("0");
                set_tag( monitor_struct[i], KILLED );
                monitor_struct[i].kill_pending = false;
                monitor_struct[i].interval = -1;
            }
            else if ( process_running( &monitor_struct[i], false ))
            {
//qDebug("1");
                // Make sure it's actually doing something
                if ( ! file_up2date( monitor_struct[i] ))
                {
//qDebug("2");
//qDebug( monitor_struct[i].file + " isn't up to date" );
                    if ( monitor_struct[i].dsp_flg != 2 )
                        set_tag( monitor_struct[i], HANG );
                    if ( monitor_struct[i].stop_pending )
                        remove_stop( &monitor_struct[i] );
                }
                else
                {
//qDebug("3");
//qDebug( monitor_struct[i].file + " -- Up to date" );
                    if ( monitor_struct[i].stop_pending )
                        set_tag( monitor_struct[i], STOPPEND );
                    else if ( monitor_struct[i].dsp_flg != 1 )
                        set_tag( monitor_struct[i], RUNNING );
                }
            }
            else
            {
//qDebug("4");
//qDebug( monitor_struct[i].name + " isn't running" );
                if ( monitor_struct[i].dsp_flg != 2 )
                    set_tag( monitor_struct[i], NOTRUNNING );
                // flag that is isn't running
                if ( monitor_struct[i].stop_pending )
                    remove_stop( &monitor_struct[i] );
                monitor_struct[i].pid = 0;
                monitor_struct[i].ppid = 0;
            }
//if ( monitor_struct[i].name == "bwtrans" ) qDebug("***** bwtrans stop *****");
        }
        else if ( type == "file" )
        {
            if ( monitor_struct[i].interval < 0 )
                ;  // Do nothing
            else if ( ! file_up2date( monitor_struct[i] ))
            {
//qDebug( monitor_struct[i].file + " isn't up to date" );
                set_tag( monitor_struct[i], NOTUPTODATE );
            }
            else
            {
//qDebug( monitor_struct[i].file + " -- Up to date" );
                set_tag( monitor_struct[i], UPTODATE );
            }
        }
        else if ( type == "acu" )
        {
            //if ( monitor_struct[i].name == "Ephemeris" )
            TAGS goodtag = UPTODATE;
            TAGS badtag = NOTUPTODATE;
            if ( monitor_struct[i].name == "Antenna" )
            {
                goodtag = STOWED;
                badtag = NOTSTOWED;
            }
            else if ( monitor_struct[i].name == "Control mode" )
            {
                goodtag = REMOTE;
                badtag = LOCAL;
            }
            if ( acu_query( monitor_struct[i] ))
                set_tag( monitor_struct[i], goodtag );
            else
                set_tag( monitor_struct[i], badtag );
        }

    }

    // Set a single shot timer to execute me again in 60 seconds
    // There is no other way to exit this method prior to this point
    // so it should be good to go here.
    timex->start( 60*1000, true );

}


bool MonitorControl::process_running( MONITOR_STRUCT *mon, bool setpid )
{
    bool found = false;
    int  count = 0;
    int  pid;
    QStringList::iterator it;
    QStringList::iterator lit;
    QStringList line;
    QStringList namelist;
    QString thename;

// TODO: I don't like this here, see if there is a better place such as
//       doing the check at the beginning of a method that calls this
//check();
    for( it = process_list.begin(); it != process_list.end(); ++it )
    {
        line = QStringList::split( '|', *it );
        lit = line.end();
        lit--;
//qDebug( "..: " + (*lit).remove( "_ADAPS"));
//qDebug( "**: " + *lit );
        pid = line[0].toInt();

        // Split the name
        namelist = QStringList::split( '/', *lit );
        lit = namelist.end();
        lit--;
        thename = (*lit).remove( "_ADAPS");
//qDebug( "..: " + (*lit).remove( "_ADAPS"));
//qDebug( "**: " + *lit );

        // Is this the one we're looking for?
        if ( mon->name == thename )
        {
            if ( setpid )
            {
//qDebug( "setting: %d", pid );
                mon->pid = pid;
            }
            found = true;
            count++;
            if ( mon->pid != pid )
            {
                ;// running with a different pid
qDebug( "pid %d != %d", mon->pid, pid );
            }
        }
    }

    if ( count > 1 )
qDebug( "Multiple " + thename + " processes running" );

    return found;
}


bool MonitorControl::file_up2date( MONITOR_STRUCT mon )
{
    bool up2date = false;
    QStringList::iterator it;
    QStringList line;

    for( it = file_list.begin(); it != file_list.end(); ++it )
    {
        line = QStringList::split( '|', *it );
        // Is this the one we're looking for?
        if ( mon.file == line[5] )
        {
//qDebug( mon.file + " " + line[5] );
            // Pull out the date
            QString modtime = line[3] + "T" + line[4];
//qDebug( modtime );
            QDateTime mod = QDateTime::fromString( modtime, Qt::ISODate );
//qDebug( mod.toString( "yyyy-MM-dd hh:mm:ss" ));
            QDateTime cur = QDateTime::currentDateTime( Qt::LocalTime );
//int i = mod.secsTo( cur );
//QString s;
//s.sprintf( "%d", i );
//qDebug( cur.toString( "yyyy-MM-dd hh:mm:ss" ) + " seconds to " + s );
            if ( mod.secsTo( cur ) <= ( mon.interval * 60 ))
                up2date = true;
        }
    }

    return up2date;
}


bool MonitorControl::acu_query( MONITOR_STRUCT mon )
{
    bool response = false;

    // Temporary stub
    if ( mon.name == "Ephemeris" )
        response = true;
    if ( mon.name == "Antenna" )
        response = true;
    if ( mon.name == "Control mode" )
        response = false;

    return response;
}



void MonitorControl::set_tag( MONITOR_STRUCT mon, TAGS tag )
{
    // Get the table item
    ColoredTableItem *c0;
    ColoredTableItem *c1;
    QString label = tags[tag];
    QColor color = colors[tag];
    QString str;
    switch ( mon.itype )
    {
        case 0 :
            c0 = (ColoredTableItem *)proc_table->item( mon.row, 0 );
            c1 = (ColoredTableItem *)proc_table->item( mon.row, 1 );
            proc_table->setText( mon.row, 0, label );
            // Don't want to mess with the painter so set the text to 
            // force a paint
            str = proc_table->text( mon.row, 1 );
            proc_table->setText( mon.row, 1, str );
            break;
        case 1 :
            c0 = (ColoredTableItem *)file_table->item( mon.row, 0 );
            c1 = (ColoredTableItem *)file_table->item( mon.row, 1 );
            file_table->setText( mon.row, 0, label );
            str = file_table->text( mon.row, 1 );
            file_table->setText( mon.row, 1, str );
            break;
        case 2 :
            c0 = (ColoredTableItem *)acu_table->item( mon.row, 0 );
            c1 = (ColoredTableItem *)acu_table->item( mon.row, 1 );
            acu_table->setText( mon.row, 0, label );
            str = acu_table->text( mon.row, 1 );
            acu_table->setText( mon.row, 1, str );
            break;
        default:
            return;
    }

    c0->setBackgroundColor( color );
    c1->setBackgroundColor( color );
    if ( color == black )
    {
        c0->setForegroundColor( white_color );
        c1->setForegroundColor( white_color );
    }
    else
    {
        c0->setForegroundColor( black_color );
        c1->setForegroundColor( black_color );
    }
    mon.dsp_flg = 1;
}

bool MonitorControl::stop_daemon( MONITOR_STRUCT *mon )
{
    bool return_status = false;
    QString msg;

    // See if it's actually running
    check();
    if ( !process_running( mon, false ))
    {
        msg = mon->name + " is not running";
        QMessageBox::critical(this, "AMC - Error", msg );
        return return_status;
    }

    if ( create_stop( mon ) == true )
        mon->stop_pending = true;

    return return_status;
}

bool MonitorControl::create_stop( MONITOR_STRUCT *mon )
{
    QString msg;

    // Create a stop file
    QString stopfile = ADAPSTABLES + "/" + mon->name + ".stop";
    QFile file( stopfile );
    if ( file.open( IO_WriteOnly ))
    {
        QTextStream stream( &file );
        stream << "stop";
        file.close();
    }
    else
    {
        msg = "Couldn't create:\n" + stopfile;
        QMessageBox::critical(this, "AMC - Error", msg );
        return false;
    }
    return true;
}

bool MonitorControl::remove_stop( MONITOR_STRUCT *mon )
{
    QString stopfile = ADAPSTABLES + "/" + mon->name + ".stop";
    QFile file( stopfile );
    if ( file.exists() )
    {
        if ( !file.remove() )
        {
            QString msg = "Couldn't remove:\n" + stopfile;
            QMessageBox::critical(this, "AMC - Error", msg );
            return true;
        }
        mon->stop_pending = false;
    }

    return false;
}

int MonitorControl::kill_daemon( MONITOR_STRUCT *mon )
{
    QString msg;
    QStringList standard_out;
    QString proc_cmd;

    // See if it's actually running
    check();
    if ( !process_running( mon, false ) || mon->pid <= 0 )
    {
qDebug( "mon->pid = %d", mon->pid );
        msg = mon->name + " is not running";
        QMessageBox::critical(this, "AMC - Error", msg );
        return 1;
    }

    proc_cmd.sprintf( "/bin/kill -9 %d", mon->pid );
//qDebug( "NOT executing kill command: " + proc_cmd );
    standard_out = exec_cmd( QString( proc_cmd ));
    if ( standard_out.join( "" ) == "error" )
        return 1;
qDebug( "Executed kill command: " + proc_cmd );

    if ( remove_stop( mon ))
        mon->stop_pending = false;
    mon->kill_pending = true;
    // start a single-shot timer
    timex->start( 6*1000, true );

    return 0;
}

bool MonitorControl::start_daemon( MONITOR_STRUCT *mon )
{
    bool return_status = false;
    QStringList standard_out;
    QString proc_cmd;
    QString msg;
    int count;

    // See if it's already running
    check();
    if ( process_running( mon, false ))
    {
        msg = mon->name + " is already running";
        QMessageBox::critical(this, "AMC - Error", msg );
        return true;
    }

    count = spawn_daemon( mon );

    // More than one line returned?
    if ( count == 0 )
    {
        QString msg = "Houston we have a problem -- no stdout";
        QMessageBox::critical(this, "AMC - Error", msg );
        return_status = true;
    }
    else if ( count > 1 )
    {
        QString msg = "Houston we have a problem -- had more than one";
        QMessageBox::critical(this, "AMC - Error", msg );
        return_status = true;
    }

    timex->start( 6*1000, true );
    return return_status;
}

bool MonitorControl::restart_daemon( MONITOR_STRUCT *mon )
{
    QString msg;
    msg = "Restart is not implemented at this time\n";
    msg += "Called with: " + mon->name;
    QMessageBox::critical(this, "AMC - Information", msg );
    return true;
}

void MonitorControl::all_start( void )
{
    //QString msg;
    //msg = "Start All is not implemented at this time";
    //QMessageBox::critical(this, "AMC - Information", msg );

    check();
    for ( int i = 0; i < the_count; i++ )
    {
        if ( monitor_struct[i].type == "daemon" )
        {
            if ( !process_running( &monitor_struct[i], false ))
            {
                spawn_daemon( &monitor_struct[i] );
            }
        }
    }
    timex->start( 6*1000, true );
}


void MonitorControl::all_stop( void )
{
    //QString msg;
    //msg = "Stop All is not implemented at this time";
    //QMessageBox::critical(this, "AMC - Information", msg );
    //return;

    check();
    for ( int i = 0; i < the_count; i++ )
    {
        if ( monitor_struct[i].type == "daemon" )
        {
            if ( process_running( &monitor_struct[i], false ))
            {
                if ( create_stop( &monitor_struct[i] ) == true )
                {
                    monitor_struct[i].stop_pending = true;
                }
            }
        }
    }  // END for i
}

int MonitorControl::spawn_daemon( MONITOR_STRUCT *mon )
{
    QStringList standard_out;
    QString proc_cmd;
    QStringList::iterator it;
    QStringList line;
    int count = 0;

    mon->pid = mon->ppid = 0;
    // Attempt to start the daemon
    if ( mon->name == "refaid_daemon" )
        proc_cmd = spawn_cmd + " " + mon->name + " SFL";
    else if ( mon->name == "prodgen_daemon" )
        proc_cmd = spawn_cmd + " " + mon->name + " OPS";
#if 0
    else if ( mon->name == "acquman_daemon" )
    {
        QString msg;
        msg  = "Acquman cannot be started from the monitor.\n";
        msg += "In a terminal window execute: start_acquman.csh";
        QMessageBox::critical(this, "AMC - Information", msg );
        return 0;
    }
#endif
    else
        proc_cmd = spawn_cmd + " " + mon->name;
    standard_out = exec_cmd( QString( proc_cmd ));
qDebug( "'" + proc_cmd + "'" );

    // Parse standard out and get the process IDs
    for( it = standard_out.begin(); it != standard_out.end(); ++it )
    {
        // Only process lines that begin with a pipe symbol
//qDebug( "This is *it: " + *it );
        QString header = (*it).left( 7 );
        if ( header == "|sPaWn|" )
        {
            (*it).remove( 0,7 );
//qDebug("Result after remove: " + *it );
            line = QStringList::split( '|', *it );
            if ( line.size() != 3 )
qDebug("Problem with line: "+ *it );
            else
            {
                mon->pid = line[0].toInt();
                mon->ppid = line[1].toInt();
            }
            count++;
        }
    }

    return count;
}
