/*******************************************************************************
FUNCTION:	ANGINTERP

PURPOSE:	Perform interpolation of the viewing angles based on grids
		created by RECTIFY.  Based on a projected input image, produce
		an output image that may have the following bands:

			Satellite zenith angles
			Sun zenith angles
			Relative azimuths

		These are the only angles supplied by RECTIFY.  The satellite 
		azimuth and sun azimuth are not supplied and cannot be created.
		The grid must be the same size as the input image (must start
		at 1,1 and have same number of lines & samples).

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    03/93  T.Rockvam        Development.  Partially taken from SUNSAT &
                                 ATMOCOR.  Special thanks to Tim Baltzer for
				 development of the interpolation routine &
				 development of ATMOCOR.
  1.1    10/93  J.Fenno          Finishing touches for release.
  1.2    12/94  T.Baltzer        Revamp scaling to conform to new standard.
  1.3    02/95  J.Fenno		 Remove the 10bit scaling option.  This was
				 decided in the AVHRR processing summit meetings
				 held.  Users felt the 16bit and 10bit options
				 allowed for global scaling was too confusing.
				 If odtype=i*2 and scalopt=global, use the 10bit
				 scaling values.

PROJECT:	ADAPS

ALGORITHM REFERENCES
    Kidwell, Katherine B. "NOAA Polar Orbiter Data Users Guide"
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "grid.h"
#include "adapsgeo.h"
#include "anginterp.h"

/*
  The following variables are global for image I/O
*/
struct FDESC *fdout;		/* image file descriptors used for image I/O */

int main (void)
{
extern long curent;
extern long curcat;

struct  PARBLK  vblock;        /* TAE variable block                   */
struct GRID angle_grid;
struct DDR ddr;
struct BDDR bddr[3];
struct SCOFF scoff[N_ANGLES];	/* Scale factors, offsets for scaling angles */

char	angle_gridfile[CMLEN],	/* angles grid file name		*/
	hosin[1][CMLEN],	/* input image host file name		*/
	hosout[3][CMLEN],	/* host file names of output files      */
	minmax[10],		/* minmax calculation flag		*/
	s[CMLEN],		/* temporary string			*/
	taeout[3][CMLEN];	/* las file names of output files       */

long	bands[1][MAXBND+1],	/* band numbers for input image		*/
	b_no,			/* band number				*/
	cflag = NOCOMB,		/* ddr update flag			*/
	i,			/* looping variable			*/
	msgtim,			/* processing message time		*/
	inbands[1][4],		/* band numbers from input image	*/
	nbands[1],		/* number of bands from input image	*/
	nmrimg   = 1,		/* number of images or files            */
	nonfatal = 1,		/* nonfatal error code			*/
	odtype,			/* output image data type		*/
	outcount = 1,		/* number of output files		*/
	scaling,		/* scaling option specified             */
	status,			/* function return status		*/
	totbnd,			/* total number of bands to create 	*/
	window[1][4];		/* window of input image		*/

double	maxval[8],		/* maximum pixel value for each output band*/
	minval[8],		/* minimum pixel value for each output band*/
	one = 1.0;		/* line_inc and samp_inc		*/

static char ang_name[3][7] = {"SatZen","SolZen","RelAz"}; /* Angle names	*/


/*
  Get parameters
*/
getpar(&vblock,hosin,window,bands,&totbnd,&ddr,hosout,taeout,angle_gridfile,
       &angle_grid,&odtype,&scaling,&msgtim,minmax);

/*
  Set up the input and output images
*/
set_up_images(hosout[0],window[0],totbnd,odtype);

/*
  Use the view_angles grid to generate the angle bands
  Band 1: Zenith angle to the satellite from each image pixel (satellite zenith)
  Band 2: Zenith angle to the sun from each image pixel (sun zenith)
  Band 3: Relative azimuth (absolute difference of satellite & solar azimuths)
*/
status = process(&angle_grid,window[0],bands[0],totbnd,odtype,scaling,scoff,msgtim);
c_freegrid(&angle_grid);
c_eclose(&fdout);
if (status != E_SUCC)
    c_clean(&vblock,"anginterp",(const char (*)[256])hosout[0],&outcount);


/*
  Use input ddr to update ddr for output image
*/
nbands[0] = 0;
inbands[0][0] = 1;
inbands[0][1] = 2;
inbands[0][2] = 3;
inbands[0][3] = 0;
c_upddr(&vblock,(const char (*)[256])hosin[0],hosout[0],&nmrimg,
    (const long (*)[4])window[0],(const long (*)[257])inbands,nbands,
            &one,&one,&totbnd,&cflag);

/*
  Get the new output ddr
  Invalidate, then update minmax
*/
if (c_getddr(hosout[0], &ddr) != E_SUCC)
    c_errmsg("Nonfatal error encountered","anginterp-warn",&nonfatal);
if (strcmp(minmax,"YES") == 0)
    {
    nbands[0] = totbnd;
    c_svmnmx(hosout[0],&totbnd,minval,maxval,"NO");
    c_minmax((const char (*)[256])hosout,&nmrimg,window,(long (*)[257])inbands,nbands,minval,maxval);
    }

/*
  Update the ddr and band records 
*/
for (i = 0; i < totbnd; i++)
    {
    /*
      Get band specific DDR info
    */
    b_no = i + 1;
    if (c_getbdr(hosout[0], &bddr[i], &b_no) != E_SUCC)
        c_errmsg("Nonfatal error encountered","anginterp-warn",&nonfatal);

    /*
      Fill in appropriate data fields
    */
    bddr[i].bandno = b_no;

    /*
      If the data was scaled, store the offset and scaling factor
      as a comment (ascii string).
    */
    if ((scaling == US_SCALE) || (scaling == GLOBAL_SCALE))
        sprintf(s, "%s deg=(pix-%.3f)/%.3f", ang_name[bands[0][i]-1],
    	        scoff[bands[0][i]-1].offset, scoff[bands[0][i]-1].scale);
    else
	sprintf(s, "%s in degrees", ang_name[bands[0][i]-1]);


    /*
      copy s into the source field
    */
    strcpy(bddr[i].source, s);
    }


/*
  Put DDR
*/
if (c_putddr(hosout[0], &ddr) != E_SUCC)
    c_errmsg("Nonfatal error encountered","anginterp-warn",&nonfatal);
for (i = 0; i < totbnd; i++)
    if (c_putbdr(hosout[0], &bddr[i]) != E_SUCC)
        c_errmsg("Nonfatal error encountered","anginterp-warn",&nonfatal);

/*
  Update history
*/
if (c_hstout(&vblock,hosout[0],(const char (*)[256])hosin,&nmrimg," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","anginterp-warn",&nonfatal);

/*
  Issue completion message
*/
c_complt("ANGINTERP",(char (*)[256])taeout[0],&outcount);

/*
  Put a return here to make the compiler happy
*/
return(SUCCESS);
}
