/*******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	initialize TAE and get parameters
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "adaps.h"
#include "grid.h"
#include "adapsgeo.h"
#include "anginterp.h"

int getpar
(
    struct PARBLK *vblock,	/* IN  - TAE variable block                   */
    char hosin[][CMLEN],	/* OUT - Input host file name                 */
    long window[][4],		/* OUT - Specified window of the image        */
    long bands[][MAXBND+1],	/* OUT - Specified bands of the image         */
    long *totbnd,		/* OUT - Total number of bands in image       */
    struct DDR *ddr,		/* OUT - Input image ddr                      */
    char hosout[][CMLEN],	/* OUT - Output host file name                */
    char taeout[][CMLEN],	/* OUT - Output tae file name                 */
    char *angle_gridfile,	/* OUT - Angles grid file name                */
    struct GRID *angle_grid,	/* OUT - Angles grid                          */
    long *odtype,		/* OUT - Input image data type                */
    long *scaling,		/* OUT - Scaling method requested by the user */
    long *msgtim,		/* OUT - Processing message time              */
    char *minmax		/* OUT - Minmax calculating flag              */
)
{
struct  VARIABLE *var;

char    taenam[1][CMLEN]; /* tae file name                       	*/

long	i;		/* looping index				*/
long	idtype;		/* input image data type			*/
long	stdlun;		/* standard output 				*/
long	status;		/* function return status			*/
long	zero = 0;	/* value of 0					*/
long 	nmrimg;		/* number of images specified			*/
long	nbands[1];	/* number of bands specified			*/
long 	imglimit = 1;	/* only one image may be specified (no +)	*/
long	same_window;	/* window size flag for multiple images (not used)*/
long	qflag[1][2];	/* windowing and banding flags			*/
long	nonfatal = 1;	/* nonfatal error flag				*/
long	fatal = -1;	/* fatal error flag				*/
long	access = IREAD;	/* file access flag				*/

/*
  Initialize TAE
*/
c_inlas(vblock,&stdlun);

/*
  Get the input image specification
*/
status = c_getin(vblock,"IN",&zero,taenam,hosin,&nmrimg,window,bands,nbands,
	 totbnd,&imglimit,&same_window,qflag);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);

/*
  Get the data type
*/
c_getddr(hosin[0],ddr);
idtype = ddr->dtype;
window[0][0] = 1;
window[0][1] = 1;
window[0][2] = ddr->nl;
window[0][3] = ddr->ns;

/*
  Get the output image specification
*/
status = c_getout(vblock,"OUT",taeout,hosout);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);
c_lsmknm(hosout[0], "ddr", hosout[1]);
c_lsmknm(hosout[0], "his", hosout[2]);
strcpy(taeout[1], hosout[1]);
strcpy(taeout[2], hosout[2]);

/*
  Get the output bands desired
*/
var = p_find(vblock,"BANDS");
*totbnd = (var->v_count);
for (i = 0; i < *totbnd; i++)
    bands[0][i] = IVAL(*var,i);
for (i = *totbnd; i < N_ANGLES; i++)
    bands[0][i] = 0;

/*
  Get the output data type
*/
var = p_find(vblock,"ODTYPE");
if (strcmp(SVAL(*var,0),"BYTE") == 0)
   *odtype = EBYTE;
else if (strcmp(SVAL(*var,0),"I*2") == 0)
   *odtype = EWORD;
else if (strcmp(SVAL(*var,0),"I*4") == 0)
   *odtype = ELONG;
else if (strcmp(SVAL(*var,0),"R*4") == 0)
   *odtype = EREAL;
else
   *odtype = idtype;

/*
  Get the scaling option
*/
var = p_find(vblock,"SCALOPT");
if (strcmp(SVAL(*var,0),"US") == 0)
   *scaling = US_SCALE;
else if (strcmp(SVAL(*var,0),"GLOBAL") == 0)
   *scaling = GLOBAL_SCALE;
else
   *scaling = NO_SCALE;


/*
  Because byte data cannot represent enough values to store all of the angle
  values without scaling, do not allow an output data type of byte with no
  scaling.
*/
if ((*odtype == EBYTE) && (*scaling == NO_SCALE))
    {
    c_errmsg("Invalid data type for NONE scaling",
	"anginterp-dtype", &nonfatal);
    c_errmsg("Fatal error encountered", "anginterp-fatal", &fatal);
    }

/*
  Get the angles grid file name
*/
var = p_find(vblock,"ANGGRID");
if (var->v_count == 1)
    {
    status = c_getnam(vblock,"ANGGRID",&zero,angle_gridfile,"ang",&access);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);
    }
else	/* default NULL grid name; use IN as root file name */
    {
    c_lsmknm((const char *)hosin,"ang",angle_gridfile);
    status = c_lsstat(angle_gridfile,&access);
    if (status != E_SUCC)
	{
        c_errmsg("Angle grid file does not exist", "anginterp-exist",&nonfatal);
        c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);
	}
    }

/*
  Read the angle grid and make sure it matches the image
*/
if (c_readgrid(angle_gridfile,angle_grid) != E_SUCC)
    {
    c_errmsg("Error reading angle grid", "anginterp-read",&nonfatal);
    c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);
    }

if ((strcmp(ddr->proj_units,angle_grid->units) != 0) ||
    (ddr->proj_code  != angle_grid->code) ||
    (ddr->zone_code  != angle_grid->zone) ||
    (ddr->datum_code != angle_grid->datum) ||
    (ddr->pdist_y    != angle_grid->pdist[0]) ||
    (ddr->pdist_x    != angle_grid->pdist[1]) ||
    (ddr->upleft[0]  != angle_grid->corners[0]) ||
    (ddr->upleft[1]  != angle_grid->corners[1]) ||
    (ddr->loleft[0]  != angle_grid->corners[2]) ||
    (ddr->loleft[1]  != angle_grid->corners[3]) ||
    (ddr->upright[0] != angle_grid->corners[4]) ||
    (ddr->upright[1] != angle_grid->corners[5]) ||
    (ddr->loright[0] != angle_grid->corners[6]) ||
    (ddr->loright[1] != angle_grid->corners[7]) ||
    (ddr->nl 	     != angle_grid->lines) ||
    (ddr->ns 	     != angle_grid->samples))
    {
    c_freegrid(angle_grid);
    c_errmsg("Angle grid does not match image","anginterp-match",&nonfatal);
    c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);
    }

if ((*(angle_grid->out_lines) != 1) || (*(angle_grid->out_samps) != 1))
    {
    c_freegrid(angle_grid);
    c_errmsg("Angle grid does not start at (1,1)","anginterp-start",&nonfatal);
    c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);
    }

if (angle_grid->dtype != EREAL)
    {
    c_freegrid(angle_grid);
    c_errmsg("Grid does not contain expected data type",
	"anginterp-grid", &nonfatal);
    c_errmsg("Fatal error encountered","anginterp-fatal",&fatal);
    }

/* 
  Get the processing message time
*/
var = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*var,0);

/* 
  Get the processing message time
*/
var = p_find(vblock,"$MINMAX");
strcpy(minmax,SVAL(*var,0));

return(E_SUCC);
}
