/*******************************************************************************
FUNCTION:	process

PURPOSE:	Compute the requested angle values for every pixel in the
		output image by interpolating these values from the grid points
		contained in the angles grid.

		The diagram below shows how a grid is organized for an image.
		The satellite zenith, solar zenith, and relative azimuth angles
		are stored in the grid structure for each of the grid points,
		marked by an X.  The following algorithm is then used:
		
		for each requested angle
		    {
		    for each grid row
			{
			for each grid column
			    {
			    Interpolate the pixel values for every pixel in the
			    current grid cell using the values at the four
			    surrounding grid points.

			    Copy the interpolated values into the correct
			    buffer that will be sent to the LAS I/O routines.
			    }

			Write all of the image lines contained in the current
			grid row to the output image (LAS I/O routines).
			}
		    }


		Col 1       Col 2       Col 3       Col 4       Col 4
            X-----------X-----------X-----------X-----------X-----------X
     Row 1  |           |           |           |           |           |
            |  Cell 1   |  Cell 2   |  Cell 3   |  Cell 4   |  Cell 5   |
            |           |           |           |           |           |
            X-----------X-----------X-----------X-----------X-----------X
     Row 2  |           |           |           |           |           |
            |  Cell 6   |  Cell 7   |  Cell 8   |  Cell 9   |  Cell 10  |
            |           |           |           |           |           |
            X-----------X-----------X-----------X-----------X-----------X
     Row 3  |           |           |           |           |           |
            |  Cell 11  |  Cell 12  |  Cell 13  |  Cell 14  |  Cell 15  |
            |           |           |           |           |           |
            X-----------X-----------X-----------X-----------X-----------X


*******************************************************************************/
#include "las.h"
#include "util.h"
#include "anginterp.h"

extern struct FDESC *fdout;	/* File descriptor used for image I/O         */
extern long curent;		/* Current line                               */
extern long curcat;		/* Current band                               */




int process
(
    struct GRID *grid,	/* IN - Angles grid generated by RECTIFY              */
    long window[],	/* IN - Window for the input image                    */
    long bands[],	/* IN - Bands to be created for the output image      */
    long nbands,	/* IN - Number of the bands to be placed in output    */
    long dtype,		/* IN - Data type of the output image                 */
    long scaling,	/* IN - Scaling option specified by the user          */
    struct SCOFF scoff[N_ANGLES], /* Scale factors, offsets for scaling       */
    long msgtim		/* IN - Number of seconds between processing messages */
)
{
char str[CMLEN];	/* Temporary string variable                          */

double tmpdbl;		/* Temporary double value for calling ceil()          */

long band;		/* Zero relative requested band number                */
long band_index;	/* Loop control - zero relative band number           */
long grid_col;		/* Current column in the grid                         */
long grid_row;		/* Current row in the grid                            */
long i;			/* Index                                              */
long j;			/* Index                                              */
long line;		/* Loop control                                       */
long lines_per_grid;	/* Number of lines in all grid cells                  */
long msgfmt   = ONECAT;	/* Processing message format flag                     */
long nlines;		/* # of lines from grid cell to put in output image   */
long nonfatal = 1;	/* Nonfatal error code                                */
long nsamps;		/* # of samples from grid cell to put in output image */
long sample;		/* Loop control                                       */
long samps_per_grid;	/* Number of samples in all grid cells                */

float bot_lf;		/* Angle value at bottom left corner of a grid cell   */
float bot_rt;		/* Angle value at bottom right corner of a grid cell  */
float *float_ptr;	/* Ptr to floating point values                       */
float top_lf;		/* Angle value at top left corner of a grid cell      */
float top_rt;		/* Angle value at top right corner of a grid cell     */

short nnflag;		/* Nearest Neighbor flag                              */


/* Ptrs for all memory allocation */
float *gridcell  = NULL;/* Interpolated angles for one cell                   */
float *float_buf = NULL;/* I/O buffer for R*4 images                          */
char  *char_buf  = NULL;/* I/O buffer for BYTE images                         */
long  *long_buf  = NULL;/* I/O buffer for I*4 images                          */
short *short_buf = NULL;/* I/O buffer for I*2 images                          */




/* Determine the size of a grid cell
------------------------------------*/
tmpdbl = (double)grid->lines / ((double)grid->nrows - 1.0);
lines_per_grid = (long)(ceil(tmpdbl));
tmpdbl = (double)grid->samples / ((double)grid->ncols - 1.0);
samps_per_grid = (long)(ceil(tmpdbl));
sprintf(str,"Cell size: %d lines by %d samples",lines_per_grid,samps_per_grid);
c_errmsg(str, "informational", &nonfatal);


/* The maximum number of lines to be written to the output image is MAXLINES.
   If the number of lines per grid is greater than this then error out.
-----------------------------------------------------------------------------*/
if (lines_per_grid > MAXLINES)
    {
    sprintf(str, "Lines per grid cell cannot exceed %d", MAXLINES);
    c_errmsg(str, "anginterp-nlines", &nonfatal);
    return(E_FAIL);
    }


/* Allocate a buffer large enough to contain lines_per_grid lines of the output
   image.  This buffer will be filled as each grid cell is interpolated across
   a single grid row.  When filled, all lines_per_grid lines will be written to
   the output image at one time.
------------------------------------------------------------------------------*/
switch (dtype)
    {
    case EBYTE:
	char_buf = (char *)malloc(lines_per_grid * window[NS] * sizeof(char));
	break;

    case EWORD:
	short_buf = (short *)malloc(lines_per_grid * window[NS] *sizeof(short));
	break;

    case ELONG:
	long_buf = (long *)malloc(lines_per_grid * window[NS] * sizeof(long));
	break;

    case EREAL:
	float_buf = (float *)malloc(lines_per_grid * window[NS] *sizeof(float));
	break;
    }
if ((char_buf == NULL) && (short_buf == NULL) &&
    (long_buf == NULL) && (float_buf == NULL))
    {
    c_errmsg("Error allocating memory", "anginterp-alloc", &nonfatal);
    return(E_FAIL);
    }


/* Set the scaling and offset values
------------------------------------*/
if (set_scale_offset(dtype, scaling, scoff) != E_SUCC)
    {
    free(float_buf);free(char_buf);free(long_buf);free(short_buf);
    return(E_FAIL);
    }


/* Allocate a floating point buffer for the nearest neighbor and interpolation
   routines.  This buffer will contain the angle values for one grid cell.
------------------------------------------------------------------------------*/
gridcell = (float *)malloc(lines_per_grid * samps_per_grid * sizeof(float));
if (gridcell == NULL)
    {
    free(float_buf);free(char_buf);free(long_buf);free(short_buf);
    c_errmsg("Error allocating memory", "anginterp-alloc", &nonfatal);
    return(E_FAIL);
    }


/* Initialize and start the processing message
----------------------------------------------*/
curcat = 1;
curent = 1;
c_inittm("ANGINTERP", "interpolated", "line", "band", "",
    &window[NL], &nbands, &msgtim, &msgfmt);
c_startm();


/* Process each band of the output image
----------------------------------------*/
for (band_index = 0; band_index < nbands; band_index++, curcat++)
    {
    band = bands[band_index] - 1; /* zero relative number of current band */
    curent = 1;                   /* current line number processed */


    /* For each row of grid cells in the output image
    -------------------------------------------------*/
    for (grid_row = 0; grid_row < grid->nrows-1; grid_row++)
	{


	/* For each column of grid cells in the current grid row
	--------------------------------------------------------*/
	for (grid_col = 0; grid_col < grid->ncols-1; grid_col++)
	    {
	    /* Set a flag indicating if the nearest neighbor or interpolation
	       routine should be used to compute all of the angle values in the
	       current grid cell.
	    ------------------------------------------------------------------*/
            nnflag = FALSE;


	    /* Set a pointer into the grid values array and extract the top
	       left and top right angle values for the current grid cell.
	    ---------------------------------------------------------------*/
	    i = grid_row * grid->ncols + grid_col;
	    float_ptr = (float *)grid->values[band] + i;
	    top_lf = *(float_ptr++);
	    top_rt = *(float_ptr++);


	    /* Set a pointer into the grid values array and extract the bottom
	       left and bottom right angle values for the current grid cell.
	    ------------------------------------------------------------------*/
	    i = (grid_row + 1) * grid->ncols + grid_col;
	    float_ptr = (float *)grid->values[band] + i;
	    bot_lf = *(float_ptr++);
	    bot_rt = *(float_ptr++);


	    /* The AVHRR model returns angles in radians.  Convert to degrees.
	    ------------------------------------------------------------------*/
	    top_lf *= DEG_PER_RAD;
	    top_rt *= DEG_PER_RAD;
	    bot_lf *= DEG_PER_RAD;
	    bot_rt *= DEG_PER_RAD;


	    /* The AVHRR model returns relative azimuth in the range 0 to 360.
	       Convert relative azimuth to the range 0 to 180 for US scaling
	       and -180 to 180 for NO scaling.
	       For GLOBAL and NO scaling we must be careful when interpolating
	       from 180 degrees to -180 degrees.  Since atmospheric correction
	       algorithms are concerned with the cosine of this angle, we do
	       not want to interpolate from 180 to -180.  This would result in
	       a grid cell containing the full range of possible values.  In
	       reality, these cells should only contain values of 180 and -180.
	       Through experimentation, we have determined that whenever one
	       grid point is greater than zero and another is less than zero we
	       should use a nearest neighbor algorithm to determine the angle
	       values in that grid cell.
	    ------------------------------------------------------------------*/
	    if (band == RELAZ)
		{
		/* Convert from the range 0 to 360 into the range -180 to 180
		-------------------------------------------------------------*/
		top_lf -= 180.0;
		top_rt -= 180.0;
		bot_lf -= 180.0;
		bot_rt -= 180.0;


		/* Convert US scaling to the range 0 to 180.  For GLOBAL and NO
		   scaling, check to see if the four grid points lie on both
		   sides of zero.  If so, we must use nearest neighbor instead
		   of interpolation.
		--------------------------------------------------------------*/
		if (scaling == US_SCALE)
		    {
		    top_lf = fabs(top_lf);
		    top_rt = fabs(top_rt);
		    bot_lf = fabs(bot_lf);
		    bot_rt = fabs(bot_rt);
		    }
		else if ((((top_lf > 0.0) || (top_rt > 0.0) ||
			  (bot_lf > 0.0) || (bot_rt > 0.0)) &&
			 ((top_lf < 0.0) || (top_rt < 0.0) ||
			  (bot_lf < 0.0) || (bot_rt < 0.0))) &&
			  (fabs(top_lf) > 90.0))
		    nnflag = TRUE;
		}


	    /* Scale the grid points.  The bounds of the grid points 
	       will be checked at this time.
	    -------------------------------------------------------*/
	    top_lf = SCALE(top_lf,scoff[band].minval,scoff[band].maxval,
			   scoff[band].scale,scoff[band].offset);
	    top_rt = SCALE(top_rt,scoff[band].minval,scoff[band].maxval,
			   scoff[band].scale,scoff[band].offset);
	    bot_lf = SCALE(bot_lf,scoff[band].minval,scoff[band].maxval,
			   scoff[band].scale,scoff[band].offset);
	    bot_rt = SCALE(bot_rt,scoff[band].minval,scoff[band].maxval,
			   scoff[band].scale,scoff[band].offset);


	    /* Compute the angle value for every pixel in the current grid cell
	    ------------------------------------------------------------------*/
	    if (nnflag)
		nearest_neighbor(top_lf, top_rt, bot_lf, bot_rt,
		    samps_per_grid, lines_per_grid, gridcell);
	    else
		c_interp(top_lf, top_rt, bot_lf, bot_rt,
		    samps_per_grid, lines_per_grid, gridcell);
	    

	    /* Determine the number of lines and samples to be copied to the
	       output image.  Because every grid cell must contain the same
	       number of lines and samples (a LAS geom package requirement),
	       the last row of grid cells may contain more lines than are
	       required for the output image.  Likewise, the last column of
	       grid cells may contain more samples than are required for the
	       output image.  Only copy those lines and samples that are
	       needed.
	    ------------------------------------------------------------------*/
	    if (grid->out_lines[grid_row+1] > window[NL])
		nlines = window[NL] - grid->out_lines[grid_row] + 1;
	    else
		nlines = lines_per_grid;
	    if (grid->out_samps[grid_col+1] > window[NS])
		nsamps = window[NS] - grid->out_samps[grid_col] + 1;
	    else
		nsamps = samps_per_grid;


	    /* Copy the angle values to the output buffer
	    ---------------------------------------------*/
	    if (dtype == EBYTE)
		{
		for (line = 0; line < nlines; line++)
		    {
		    for (sample = 0; sample < nsamps; sample++)
			{
			/* index for the current sample in the grid cell */
			i = line * samps_per_grid + sample;

			/* index for the current sample in the output buffer */
			j = line * window[NS] + grid_col *
			    samps_per_grid + sample;

			char_buf[j] = (unsigned char)gridcell[i];
			}
		    }
		}
	    else if (dtype == EWORD)
		{
		for (line = 0; line < nlines; line++)
		    {
		    for (sample = 0; sample < nsamps; sample++)
			{
			/* index for the current sample in the grid cell */
			i = line * samps_per_grid + sample;

			/* index for the current sample in the output buffer */
			j = line * window[NS] + grid_col *
			    samps_per_grid + sample;

			short_buf[j] = (short)gridcell[i];
			}
		    }
		}
	    else if (dtype == ELONG)
		{
		for (line = 0; line < nlines; line++)
		    {
		    for (sample = 0; sample < nsamps; sample++)
			{
			/* index for the current sample in the grid cell */
			i = line * samps_per_grid + sample;

			/* index for the current sample in the output buffer */
			j = line * window[NS] + grid_col *
			    samps_per_grid + sample;

			long_buf[j] = (long)gridcell[i];
			}
		    }
		}
	    else if (dtype == EREAL)
		{
		for (line = 0; line < nlines; line++)
		    {
		    for (sample = 0; sample < nsamps; sample++)
			{
			/* index for the current sample in the grid cell */
			i = line * samps_per_grid + sample;

			/* index for the current sample in the output buffer */
			j = line * window[NS] + grid_col *
			    samps_per_grid + sample;

			float_buf[j] = gridcell[i];
			}
		    }
		}
	    }


	/* Write the lines that have been interpolated from each of the grid
	   cells in the current grid row.  The variable "i" is set to be the
	   actual band number in the output image (not the requested band
	   number).
	----------------------------------------------------------------------*/
	i = band_index + 1;                /* band number in output image */
	line = grid->out_lines[grid_row];  /* line number in output image */
	switch (dtype)
	    {
	    case EBYTE:
		j = c_ewrite(&fdout, &i, &line, (unsigned char *)char_buf, &nlines);
		break;

	    case EWORD:
		j = c_ewrite(&fdout, &i, &line, (unsigned char *)short_buf, &nlines);
		break;

	    case ELONG:
		j = c_ewrite(&fdout, &i, &line, (unsigned char *)long_buf, &nlines);
		break;

	    case EREAL:
		j = c_ewrite(&fdout, &i, &line, (unsigned char *)float_buf, &nlines);
		break;
	    }
	if (j != (nlines * window[NS]))
	    {
	    c_errmsg("Error writing to output image",
		"anginterp-write", &nonfatal);
	    free(gridcell);
	    free(float_buf);free(char_buf);free(long_buf);free(short_buf);
	    return(E_FAIL);
	    }


	/* Increment the processing message line counter
	------------------------------------------------*/
	curent += nlines;
	}
    }


/* Stop the processing message
------------------------------*/
curcat = nbands;
curent = window[NL];
c_stoptm();


/* Free all memory
------------------*/
free(gridcell);
free(float_buf);free(char_buf);free(long_buf);free(short_buf);
return(E_SUCC);
}
