/*******************************************************************************
FUNCTION:	ansistack_get

PURPOSE:	Determine which files should be stacked to tape and move them
		into the specified temporary directory.  ANSISTACK creates a
		file which contains the tape name of every file that has been
		stacked to the tape in its temporary stacking directory.  The
		file names are read from this file and stored in a queue of
		strings.  This function then checks the tape name for every
		file in the groups that it finds in the input directory against
		the list of file names already on the tape.  If any of the
		names are the same, the file group is ignored.  If the names
		are all unique, the new file group is added to a queue of
		eligible file groups.

		After the function has examined all of the available file
		groups in the input directory, it checks to see which files
		will fit on the tape.  Those that will fit are moved into the
		temporary stacking directory.

		If a fatal error is encountered do not issue the ansistack-fatal
		message.  This will be taken care of in the pdf.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "queue.h"
#include "ansistack_get.h"


#define SLEEP_TIME	120	/* Seconds to sleep between searches	*/


main()
{
char aftype[NASSOC][CMLEN];/* Associated file types (list of extensions)      */
char command[SMALLSTR];	/* Command being run (gen, avhrr, mssx)               */
char errdir[CMLEN];	/* Directory to move file groups into on errors       */
char infile[CMLEN];	/* Input file specification                           */
char stackdir[CMLEN];	/* Directory where file should be placed for stacking */
char stopfile[CMLEN];	/* Host file name of the stop file                    */
char str[CMLEN];	/* Temporary string                                   */
char tapelist[CMLEN];	/* File containing tape name of files copied to tape  */
char ttype[SMALLSTR];	/* Tape type                                          */

long first_call;	/* Flag indicating first call to c_getfiles()         */
long max_mbytes;	/* Max megabytes to stack to tape                     */
long mbytes;		/* Number of mbytes already stacked to the tape       */
long min_mbytes;	/* Min megabytes to stack to tape                     */
long nassoc;		/* Number of associated files                         */
long nfiles;		/* Number of files returned by c_getfiles()           */
long q_eligible;	/* Queue of eligible file groups                      */
long q_stack;		/* Queue of file groups that should be stacked to tape*/
long q_tapenames;	/* Queue of file names already written to tape        */
long status;		/* Function return status                             */
long std_out;		/* Standard output file                               */

struct PARBLK vblock;		/* TAE parameter block                        */
struct FILEGROUP *group_ptr;	/* Ptr to list of files names for 1 file group*/

int forever = TRUE;	/* to eliminate warning	*/


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the parameters
--------------------------*/
status = getpar(&vblock, command, infile, &min_mbytes, &max_mbytes, &mbytes,
    ttype, &nassoc, aftype, stopfile, stackdir, errdir, tapelist);
if (status != E_SUCC)
    z_exit(-1, "ANSISTACK_GET failed");


/* If we are working with a 4mm or 8mm tape then we must read the names of
   every file that has been stacked to the tape.  This allows us to ensure
   no files are written to the tape that have the same name on tape.  This
   queue can allow duplicates so that queue operations are faster.
--------------------------------------------------------------------------*/
q_tapenames = c_qcreate("ascending", "dups");
read_tape_names(tapelist, q_tapenames);


/* Create a queue that will contain all of the file groups that are to be
   stacked to tape.  This queue may allow duplicates because the queue of
   eligible file groups does not allow duplicates.
-------------------------------------------------------------------------*/
q_stack = c_qcreate("fifo", "dups");


/* Keep looking for files to stack to tape
------------------------------------------*/
while (c_qsize(q_stack) == 0)
    {
    /* Check for the existence of the stop file.  If this file exists then stop
       looking for files, clean up all allocated memory, and exit successfully.
    --------------------------------------------------------------------------*/
    first_call = TRUE;
    c_getfiles(stopfile, (char (*)[256])str, (int *)&nfiles, 1, first_call, FALSE);
    if (nfiles)
	{
	c_qdestroy(q_tapenames);
	c_qdestroy(q_stack);
	z_exit(1, "ANSISTACK_GET completed successfully");
	}


    /* Create a new queue to hold all of the eligible file groups
    -------------------------------------------------------------*/
    q_eligible = c_qcreate("fifo", "nodups");


    /* Place all of the eligible file groups in the queue
    -----------------------------------------------------*/
    while ( forever )
	{
	/* Retrieve one file that matches infile
	----------------------------------------*/
	c_getfiles(infile, (char (*)[256])str, (int *)&nfiles, 1, first_call, FALSE);
	first_call = FALSE;


	/* If there are no files that match infile then exit the loop
	-------------------------------------------------------------*/
	if (nfiles == 0) break;


	/* Create a file group for the current file
	-------------------------------------------*/
	status = create_group(str, aftype, nassoc, command, ttype, &group_ptr);
	if (status != E_SUCC)
	    {
	    c_qdestroy(q_tapenames);
	    c_qdestroy(q_stack);
	    c_qdestroy(q_eligible);
	    z_exit(-1, "ANSISTACK_GET failed");
	    }
	if (group_ptr == NULL)
	    continue;
	

	/* Ensure the tape name of every file in the
	   group does not already appear on the tape.
	---------------------------------------------*/
	check_tape_names(&group_ptr, q_tapenames);
	if (group_ptr == NULL)
	    continue;


	/* Insert the file group into the queue of eligible groups
	----------------------------------------------------------*/
	c_qinsert(q_eligible, group_ptr, compare_tape_names);
	}
    

    /* Determine which file groups should be stacked to the tape
    ------------------------------------------------------------*/
    select_groups(ttype, min_mbytes, max_mbytes, mbytes, q_eligible, q_stack);


    /* If the tape is a high capacity 4mm or 8mm tape and there are eligible
       file groups for stacking but none were selected, then assume the tape is
       full and break out of this loop.
    --------------------------------------------------------------------------*/
    if (((strcmp(ttype, "4mm") == 0) ||
	 (strcmp(ttype, "8lo") == 0) ||
	 (strcmp(ttype, "8hi") == 0) ||
         (strcmp(ttype, "dlt4000") == 0) ||
         (strcmp(ttype, "dlt7000") == 0) ||
         (strcmp(ttype, "d3") == 0)) &&
	(c_qsize(q_eligible) > 0) && (c_qsize(q_stack) == 0))
	{
	c_qdestroy(q_eligible);
	break;
	}


    /* Destroy the queue of eligible file groups
    --------------------------------------------*/
    c_qdestroy(q_eligible);


    /* If there are no file groups that should be stacked to the tape then
       sleep for SLEEP_TIME seconds before checking for additional files.
    ----------------------------------------------------------------------*/
    if (c_qsize(q_stack) == 0)
	{
	/* possibly issue a message to the user */
	sleep(SLEEP_TIME);
	}
    }


/* Free the memory used for the queue of tape names
---------------------------------------------------*/
c_qdestroy(q_tapenames);


/* Move all of the file groups into the temporary directory.
-----------------------------------------------------------*/
move_queue(q_stack, stackdir, errdir);


/* Free the memory used for the queue of file groups to be stacked
------------------------------------------------------------------*/
c_qdestroy(q_stack);


/* Issue a successful completion message
----------------------------------------*/
z_exit(1, "ANSISTACK_GET completed successfully");

/* Put a return here to make the compiler happy
------------------------------------------------*/
return E_SUCC;

}




/*############################################################################*/
long getpar
(
    struct PARBLK *vblock,	/* IN  - TAE parameter block                  */
    char *command,		/* OUT - Command being run (gen,avhrr,mssx)   */
    char *infile,		/* OUT - Input file specification             */
    long *min_mbytes,		/* OUT - Min megabytes to stack to tape       */
    long *max_mbytes,		/* OUT - Max megabytes to stack to tape       */
    long *mbytes,		/* OUT - Megabytes already stacked to tape    */
    char *ttype,		/* OUT - Tape type (4mm, 8mm, 3480, 9-track..)*/
    long *nassoc,		/* OUT - Number of associated files in aftype */
    char aftype[][CMLEN],	/* OUT - Extensions for each associated file  */
    char *stopfile,		/* OUT - Host file name of the stop file      */
    char *stackdir,		/* OUT - Host file name of the stack directory*/
    char *errdir,		/* OUT - Error directory                      */
    char *tapelist 		/* OUT - Host file name of tmp tape list file */
)

{
char str[CMLEN];		/* Temporary string variable                  */

long access;			/* File access                                */
long i;				/* Loop control                               */
long element;			/* Element of TAE parameter to retrieve       */
long status;			/* Function return status                     */


struct VARIABLE *parm;		/* Ptr to a TAE variable                      */



/* Get the command being executed
---------------------------------*/
parm = p_find(vblock, "CMD");
strcpy(command, SVAL(*parm, 0));
i = (long) strlen(command);
c_up2low(command, &i);


/* Get the input file specification
-----------------------------------*/
access = IUPDATE;
element = 0;
if (strcmp(command, "gen") == 0)
    status = c_getnam(vblock, "INFILE", &element, infile, "img", &access);
else if (strcmp(command, "avhrr") == 0)
    status = c_getnam(vblock, "INFILE", &element, infile, "ahdr", &access);
else if (strcmp(command, "mssx") == 0)
    status = c_getnam(vblock, "INFILE", &element, infile, "ahdr", &access);
if (status != E_SUCC)
    return(E_FAIL);


/* Get the minimum and maximum number of megabytes to stack to tape and
   the number of megabytes that have already been stacked to the tape.
-----------------------------------------------------------------------*/
parm = p_find(vblock, "MINBYTES");
*min_mbytes = IVAL(*parm,0);
parm = p_find(vblock, "MAXBYTES");
*max_mbytes = IVAL(*parm,0);
parm = p_find(vblock, "MBYTES");
*mbytes = IVAL(*parm,0);


/* Get the type of tape being stacked
-------------------------------------*/
parm = p_find(vblock, "TTYPE");
strcpy(ttype, SVAL(*parm, 0));
i = (long) strlen(ttype);
c_up2low(ttype, &i);


/* Get the associated file types
--------------------------------*/
parm = p_find(vblock, "AFTYPE");
*nassoc = parm->v_count;
for (i = 0; i < *nassoc; i++)
    strcpy(aftype[i], SVAL(*parm, i));


/* Get the stop file
--------------------*/
access = IUPDATE;
element = 0;
status = c_getnam(vblock, "STOPFILE", &element, stopfile, "stop", &access);
if (status != E_SUCC)
    return(E_FAIL);


/* Get the temporary directory
------------------------------*/
parm = p_find(vblock, "STACKDIR");
strcpy(str, SVAL(*parm, 0));
c_cmhost(str, stackdir);
strcat(stackdir,"/");


/* Get the error directory
--------------------------*/
parm = p_find(vblock, "ERRDIR");
strcpy(str, SVAL(*parm, 0));
c_cmhost(str, errdir);
strcat(errdir,"/");


/* Get the name of the file that will contain the
   tape name of every file stacked to tape so far
-------------------------------------------------*/
access = IUPDATE;
element = 0;
status = c_getnam(vblock, "TAPELIST", &element, tapelist, "list", &access);
if (status != E_SUCC)
    return(E_FAIL);


return(E_SUCC);
}
