/*******************************************************************************
FUNCTION:	ansistack_wrt

PURPOSE:	Write all of the files found in the stacking directory to an
		ANSI labeled tape.

		If an error is encountered move the offending file group to the
		specified error directory, move all of the other files back to
		the original directory, and issue an error message.  However,
		this function should not issue the ansistack-fatal message.
		This will be taken care of in the pdf.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "queue.h"
#include "ansistack_wrt.h"


main()
{
char aftype[NASSOC][CMLEN];/* Associated file types (list of extensions)      */
char command[SMALLSTR];	/* Command being run (gen, avhrr, mssx)               */
char drive[SMALLSTR];	/* Tape drive                                         */
char dstdir[CMLEN];	/* Destination directory for files after stacking     */
char errdir[CMLEN];	/* Destination directory for files after an error     */
char infile[CMLEN];	/* Input file specification                           */
char logfile[CMLEN];	/* Log file for standard ADAPS logging                */
char rundir[CMLEN];	/* Directory from which to get stackone.pdf and .csh  */
char srcdir[CMLEN];	/* Source directory where ANSISTACK_GET found them    */
char stackdir[CMLEN];	/* Directory where file should be placed for stacking */
char str[CMLEN];	/* Temporary string                                   */
char sumfile[CMLEN];	/* Name of the tape summary file                      */
char tape_buf[TPFIELDS][SMALLSTR];/* Buffer for creating a tape label         */
char tapeid[SMALLSTR];	/* Identification number of tape being stacked        */
char tapelist[CMLEN];	/* File containing tape name of all files stacked     */
char ttype[SMALLSTR];	/* Tape type (4mm, 8lo, 8hi, 3480, 3490, 9-track)     */
char acctnum[10];	/* Account number				      */
char ordernum[14];	/* Order number					      */
char prodsum[4];	/* Product summary flag (yes or no)		      */
char unitnum[5];	/* Unit number					      */


long block_size[GROUPSIZE];/* Block size for each file in a group             */
long first_call;	/* Flag indicating first call to c_getfiles()         */
long i;			/* Loop control                                       */
long nassoc;		/* Number of associated files                         */
int nfiles;		/* Number of files stacked to tape                    */
int ngroups;		/* Number of file grops stacked to tape               */
long nmonths;		/* Number of months to retain the tape                */
long nonfatal     = 1;	/* Nonfatal error flag                                */
long print_flag;	/* Flag for printing tape labels or not               */
long outform;		/* Output format, 0=Level 0, 1=Level 1B		      */
long q_stacked;		/* Queue of file groups successfully stacked to tape  */
long q_tostack;		/* Queue of file groups to be stacked to tape         */
long record_size[GROUPSIZE];/* Record size for each file in a group           */
long status;		/* Function return status                             */
long std_out;		/* Standard output file                               */
long tmp_nfiles   = 0;	/* # of files stacked by this call to ANSISTACK_WRT   */
long tmp_ngroups  = 0;	/* # of groups stacked by this call to ANSISTACK_WRT  */

int mbytes;		/* Number of megabytes that have been stacked to tape */

short delete_flag;	/* Flag to delete/move files after stacking to tape   */
short stop_flag;	/* Flag indicating no files are present to stack      */

struct PARBLK vblock;		/* TAE parameter block                        */
struct FILEGROUP *group_ptr;	/* Ptr to list of files names for 1 file group*/

int forever = TRUE;	/* to eliminate warning	*/


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the parameters
--------------------------*/
status = getpar(&vblock, command, infile, srcdir, stackdir, dstdir, errdir,
    &nassoc, aftype, record_size, block_size, &delete_flag, &mbytes, drive,
    ttype, tapeid, sumfile, &stop_flag, tapelist, logfile, &ngroups, &nfiles,
    &nmonths, tape_buf, &print_flag, acctnum, ordernum, unitnum,
    &outform, rundir, prodsum);
if (status != E_SUCC)
    z_exit(-1, "ANSISTACK_WRT failed");


/* If a manual stop has been encountered then all we need to do is print a tape
   label for those tapes that have had at least one file group stacked to them.
   Remember to return a value of 2 to indicate successful completion and that
   we should not continue stacking files to the tape.
------------------------------------------------------------------------------*/
if (stop_flag)
    {
    if (ngroups > 0)
	tape_label(tape_buf, nmonths, (long)ngroups, print_flag);
    z_exit(2, "ANSISTACK_WRT completed successfully");
    }


/* Create a queue for all file groups to be stacked and
   one for all file groups that have been stacked.
-------------------------------------------------------*/
q_tostack = c_qcreate("fifo", "dups");
q_stacked = c_qcreate("fifo", "dups");


/* Place all of the file groups in the queue
--------------------------------------------*/
first_call = TRUE;
while ( forever )
    {
    /* Retrieve one file that matches infile
    ----------------------------------------*/
    c_getfiles(infile, (char (*)[256])str, (int *)&i, 1, first_call, FALSE);
    first_call = FALSE;


    /* If there are no files that match infile then exit the loop
    -------------------------------------------------------------*/
    if (i == 0) break;


    /* Create a file group for the current file
    -------------------------------------------*/
    status = create_group(str, aftype, nassoc, command, ttype, &group_ptr);
    if (status != E_SUCC)
	{
	c_qdestroy(q_tostack);
	c_qdestroy(q_stacked);
	z_exit(-1, "ANSISTACK_WRT failed");
	}
    if (group_ptr == NULL)
	{
	sprintf(str,"Error locating file groups in %s", stackdir);
	c_errmsg(str, "ansistack-group", &nonfatal);
	c_qdestroy(q_tostack);
	c_qdestroy(q_stacked);
	z_exit(-1, "ANSISTACK_WRT failed");
	}
    

    /* Insert the file group into the queue of eligible groups
    ----------------------------------------------------------*/
    c_qinsert(q_tostack, group_ptr, compare_tape_names);
    }


/* Rename all of the file groups from their disk names to their tape names
--------------------------------------------------------------------------*/
if (rename_queue(q_tostack, errdir, outform) != E_SUCC)
    {
    move_queue(q_tostack, srcdir, errdir);
    c_qdestroy(q_tostack);
    c_qdestroy(q_stacked);
    z_exit(-1, "ANSISTACK_WRT failed");
    }


/* Stack all of the file groups to the tape
-------------------------------------------*/
while (c_qsize(q_tostack))
    {
    /* Remove the first element from the tostack queue for stacking
    ---------------------------------------------------------------*/
    c_qremove(q_tostack, 1, (void **)&group_ptr);


    /* Stack the files for this group to tape
    -----------------------------------------*/
    status = stack_group(group_ptr, record_size, block_size, drive,
	ttype, tapeid, tapelist, logfile, (nfiles + tmp_nfiles),
	acctnum, ordernum, unitnum, outform, rundir, prodsum);
    if (outform == 0)
	rename_group(group_ptr);
    if (status == E_SUCC)
	{
	/* If the file group was successfully stacked to tape then we need to
	   increment the number of files and groups written to the tape,
	   increment the total number of mbytes that have been written to the
	   tape, add the file group to the tape summary file, and insert it
	   into the queue of stacked file groups.
	---------------------------------------------------------------------*/
	tmp_nfiles += group_ptr->nfiles;
	tmp_ngroups++;
	mbytes += group_ptr->mbytes;
	tape_summary(group_ptr, tapeid, ttype, drive, sumfile,
	    (ngroups + tmp_ngroups), outform);
	c_qinsert(q_stacked, group_ptr, compare_tape_names);
	}
    else
	{
	/* If the file group was not successfully stacked to tape then we need
	   to move the file group into the error directory, free the memory
	   allocated for the group, rename all of the groups left in the
	   tostack queue, move all of the files in both queues back to the
	   source directory, destory both queues, and reset the number of files
	   and groups that this call to ansistack_wrt() has written to tape.
	----------------------------------------------------------------------*/
	move_group(group_ptr, errdir);
	free(group_ptr);
	rename_queue(q_tostack, errdir, outform);
	move_queue(q_tostack, srcdir, errdir);
	move_queue(q_stacked, srcdir, errdir);
	c_qdestroy(q_tostack);
	c_qdestroy(q_stacked);
	tmp_nfiles  = 0;
	tmp_ngroups = 0;
	break;
	}
    }


/* If no file groups were successfully stacked to tape during this call then
   that means we must have encountered an error.  If we have already stacked
   some file groups to the tape then this is a nonfatal error, otherwise, it
   is a fatal error.
----------------------------------------------------------------------------*/
if (tmp_ngroups == 0)
    {
    /* If no file groups have been stacked to the tape, issue a fatal error
    -----------------------------------------------------------------------*/
    if (ngroups < 1)
	{
	c_errmsg("Error stacking files to tape", "ansistack-fstack", &nonfatal);
	z_exit(-1, "ANSISTACK_WRT failed");
	}
    

    /* If some file groups have been stacked to the tape, issue a nonfatal error
    --------------------------------------------------------------------------*/
    c_errmsg("Error stacking files to tape", "ansistack-nstack", &nonfatal);
    }
else
    {
    /* Increment the total number of files and file groups
       on the tape and return the new values to TAE.
    ------------------------------------------------------*/
    nfiles  += tmp_nfiles;
    ngroups += tmp_ngroups;
    if ((q_intg(&vblock, "FILES",  1, &nfiles,  P_UPDATE) != SUCCESS) ||
        (q_intg(&vblock, "GROUPS", 1, &ngroups, P_UPDATE) != SUCCESS) ||
        (q_intg(&vblock, "MBYTES", 1, &mbytes,  P_UPDATE) != SUCCESS) ||
	(q_out(&vblock) != SUCCESS))
	{
	c_errmsg("Error sending parameter to TAE", "ansistack-tae", &nonfatal);
	move_queue(q_stacked, srcdir, errdir);
	c_qdestroy(q_tostack);
	c_qdestroy(q_stacked);
	z_exit(-1, "ANSISTACK_WRT failed");
	}


    /* Move the file groups to another directory or delete them
    -----------------------------------------------------------*/
    if (delete_flag)
	delete_queue(q_stacked);
    else
	move_queue(q_stacked, dstdir, errdir);


    /* Free all memory used by the queues
    -------------------------------------*/
    c_qdestroy(q_tostack);
    c_qdestroy(q_stacked);
    }


/* Issue a successful completion message.  If stacking a high capacity 4mm,
   8hi, or 8lo type tape and we stacked a file group to tape then exit with a
   status of one.  This indicates successful completion but keep stacking.
   Otherwise, print a tape label and exit with a status of two which indicates
   successful completion and the tape is full.
------------------------------------------------------------------------------*/
if ((tmp_ngroups > 0) &&
    ((strcmp(ttype,"4mm") == 0) ||
     (strcmp(ttype,"8hi") == 0) ||
     (strcmp(ttype,"8lo") == 0) ||
     (strcmp(ttype,"dlt4000") == 0) ||
     (strcmp(ttype,"dlt7000") == 0) ||
     (strcmp(ttype,"d3") == 0)))
    z_exit(1, "ANSISTACK_WRT completed successfully");


tape_label(tape_buf, nmonths, (long)ngroups, print_flag);
z_exit(2, "ANSISTACK_WRT completed successfully");


/* Put a return here to make the compiler happy
------------------------------------------------*/
return E_SUCC;

}




/*############################################################################*/
long getpar
    (
    struct PARBLK *vblock,	/* IN  - TAE parameter block                  */
    char *command,		/* OUT - Command be run (gen, avhrr, mssx)    */
    char *infile,		/* OUT - File specification for input files   */
    char *srcdir,		/* OUT - Source dir where file where located  */
    char *stackdir,		/* OUT - Temporary stacking directory         */
    char *dstdir,		/* OUT - Destination directory for all files  */
    char *errdir,		/* OUT - Dir where files go after errors      */
    long *nassoc,		/* OUT - Number of associated files           */
    char aftype[][CMLEN],	/* OUT - Associated file extensions           */
    long record_size[],		/* OUT - Record size for each file            */
    long block_size[],		/* OUT - Block size for each file             */
    short *delete_flag,		/* OUT - Flag for deleting/moving files       */
    int *mbytes,		/* OUT - Mbytes already stacked to tape       */
    char *drive,		/* OUT - Tape drive where tape is mounted     */
    char *ttype,		/* OUT - Type of tape that is mounted         */
    char *tapeid,		/* OUT - ID number of the mounted tape        */
    char *sumfile,		/* OUT - Name of the tape summary file        */
    short *stop_flag,		/* OUT - Flag indicating no files to stack    */
    char *tapelist,		/* OUT - File to hold tape name of all files  */
    char *logfile,		/* OUT - Log file                             */
    int *ngroups,		/* OUT - Num of groups already written to tape*/
    int *nfiles,		/* OUT - Num of files already written to tape */
    long *nmonths,		/* OUT - Num of months to retain the tape     */
    char tape_buf[][SMALLSTR],	/* OUT - Info for creating tape labels        */
    long *print_flag,		/* OUT - Flag to print tape labels or not     */
    char acctnum[],		/* OUT - Account number			      */
    char ordernum[],		/* OUT - Order number			      */
    char unitnum[],		/* OUT - Unit number			      */
    long *outform,		/* OUT - Output format			      */
    char rundir[],		/* OUT - Run directory			      */
    char prodsum[] 		/* OUT - Product summary flag		      */
    )

{
char ext[CMLEN];		/* Extension portion of a file name           */
char root[CMLEN];		/* Root portion of a file name                */
char str[CMLEN];		/* Temporary string variable                  */
char outformstr[8];		/* Output format in char format		      */

long access;			/* File access                                */
long element;			/* Element of TAE parameter to retrieve       */
long i;				/* Loop control                               */
long status;			/* Function return status                     */

struct VARIABLE *parm;		/* Ptr to a TAE variable                      */




/* Get the command being executed
---------------------------------*/
parm = p_find(vblock, "CMD");
strcpy(command, SVAL(*parm, 0));
i = (long)strlen(command);
c_up2low(command, &i);


/* Get the stacking directory
-----------------------------*/
parm = p_find(vblock, "STACKDIR");
strcpy(str, SVAL(*parm, 0));
c_cmhost(str, stackdir);
strcat(stackdir,"/");


/* Get the input file specification and exchange the specified directory for
   the stacking directory.  We do this because ANSISTACK_GET has already moved
   the files that should be stacked from the input directory into the stacking
   directory.
------------------------------------------------------------------------------*/
access = IUPDATE;
element = 0;
if (strcmp(command, "gen") == 0)
    status = c_getnam(vblock, "INFILE", &element, infile, "img", &access);
else if (strcmp(command, "avhrr") == 0)
    status = c_getnam(vblock, "INFILE", &element, infile, "ahdr", &access);
else if (strcmp(command, "mssx") == 0)
    status = c_getnam(vblock, "INFILE", &element, infile, "ahdr", &access);
if (status != E_SUCC)
    return(E_FAIL);
c_splitname(infile, srcdir, root, ext);
sprintf(infile, "%s%s.%s", stackdir, root, ext);


/* Because it is possible for the stop file to be created between the time
   that ANSISTACK_GET moves files into the stacking directory and the time
   this code executes, we do not want to check for the existence of the stop
   file.  Instead, we will assume that if there are no files to be stacked
   that a manual stop has been encountered.  If ANSISTACK_GET has already
   moved some files into the stacking directory then we will want to stack
   these files to tape.
----------------------------------------------------------------------------*/
c_getfiles(infile, (char (*)[256])str, (int *)&i, 1, TRUE, FALSE);
if (i > 0)
    *stop_flag = FALSE;
else
    *stop_flag = TRUE;


/* Get the directory where the files are to be placed after stacking
--------------------------------------------------------------------*/
parm = p_find(vblock, "OUTDIR");
if (parm->v_count == 0)
    strcpy(dstdir, "");
else
    {
    strcpy(str, SVAL(*parm, 0));
    c_cmhost(str, dstdir);
    strcat(dstdir,"/");
    }


/* Determine where files should be moved when an error occurs
-------------------------------------------------------------*/
parm = p_find(vblock, "ERRDIR");
strcpy(str, SVAL(*parm, 0));
c_cmhost(str, errdir);
strcat(errdir,"/");


/* Get the associated file types
--------------------------------*/
parm = p_find(vblock, "AFTYPE");
*nassoc = parm->v_count;
for (i = 0; i < *nassoc; i++)
    strcpy(aftype[i], SVAL(*parm, i));


/* Retrieve the record size for each file
-----------------------------------------*/
parm = p_find(vblock, "RECSIZ");
for (i = 0; i < (*nassoc + 1); i++)
    record_size[i] = IVAL(*parm, i);


/* Retrieve the blocking factor and convert it to a block size
--------------------------------------------------------------*/
parm = p_find(vblock, "BLKSIZ");
for (i = 0; i < (*nassoc + 1); i++)
    block_size[i] = IVAL(*parm, i) * record_size[i];


/* Retrieve the delete/move option
----------------------------------*/
parm = p_find(vblock, "FILEOPT");
strcpy(str, SVAL(*parm, 0));
if (strcmp(str, "DELETE") == 0)
    *delete_flag = TRUE;
else
    *delete_flag = FALSE;


/* Get the total mbytes that have already been stacked to the tape
------------------------------------------------------------------*/
parm = p_find(vblock, "MBYTES");
*mbytes = IVAL(*parm,0);


/* Retrieve the tape drive
--------------------------*/
parm = p_find(vblock, "DRIVE");
strcpy(drive, SVAL(*parm, 0));
i = (long)strlen(drive);
c_up2low(drive, &i);


/* Get the type of tape being stacked
-------------------------------------*/
parm = p_find(vblock, "TTYPE");
strcpy(ttype, SVAL(*parm, 0));
i = (long)strlen(ttype);
c_up2low(ttype, &i);


/* Retrieve the tape ID
-----------------------*/
parm = p_find(vblock, "TAPEID");
strcpy(tapeid, SVAL(*parm, 0));
i = (long)strlen(tapeid);
c_up2low(tapeid, &i);


/* Retrieve the name of the tape summary file
---------------------------------------------*/
parm = p_find(vblock, "SUMFILE");
if (parm->v_count == 0)
    strcpy(sumfile, "");
else
    {
    access = IUPDATE;
    element = 0;
    status = c_getnam(vblock, "SUMFILE", &element, sumfile, "list", &access);
    if (status != E_SUCC)
	return(E_FAIL);
    }


/* Get the name of the file that will contain the
   tape name of every file stacked to tape so far
-------------------------------------------------*/
access = IUPDATE;
element = 0;
status = c_getnam(vblock, "TAPELIST", &element, tapelist, "list", &access);
if (status != E_SUCC)
    return(E_FAIL);


/* Retrieve the name of the log file
------------------------------------*/
parm = p_find(vblock, "LOGFILE");
if (parm->v_count == 0)
    strcpy(logfile, "");
else
    {
    access = IUPDATE;
    element = 0;
    status = c_getnam(vblock, "LOGFILE", &element, logfile, "log", &access);
    if (status != E_SUCC)
	return(E_FAIL);
    }


/* Retrieve the number of file groups already written to the tape
-----------------------------------------------------------------*/
parm = p_find(vblock, "GROUPS");
*ngroups = IVAL(*parm,0);


/* Retrieve the number of files already written to the tape
-----------------------------------------------------------*/
parm = p_find(vblock, "FILES");
*nfiles = IVAL(*parm,0);


/* Retrieve all of the variables used to create a tape label
------------------------------------------------------------*/
parm = p_find(vblock, "NAME");
if (parm->v_count == 0)
    strcpy(tape_buf[5], "");
else
    strcpy(tape_buf[5], SVAL(*parm, 0));

parm = p_find(vblock, "CODE");
if (parm->v_count == 0)
    strcpy(tape_buf[0], "");
else
    strcpy(tape_buf[0], SVAL(*parm, 0));

parm = p_find(vblock, "PROJECT");
if (parm->v_count == 0)
    strcpy(tape_buf[4], "");
else
    strcpy(tape_buf[4], SVAL(*parm, 0));

parm = p_find(vblock, "DESCRPT");
if (parm->v_count == 0)
    strcpy(tape_buf[2], "ANSI TAPE: ? file groups");
else
    strcpy(tape_buf[2], SVAL(*parm, 0));

parm = p_find(vblock, "NMONTHS");
*nmonths = IVAL(*parm, 0);

strcpy(tape_buf[1], "");
strcpy(tape_buf[3], drive);
strcpy(tape_buf[6], "1/1");
strcpy(tape_buf[7], tapeid);
strcpy(tape_buf[8], "");
strcpy(tape_buf[9], "");
strcpy(tape_buf[10], "");


parm = p_find(vblock, "$FNAME");
if (strcmp(SVAL(*parm,0), "EROS Data Center") == 0)
    *print_flag = TRUE;
else
    *print_flag = FALSE;


/* Retrieve the account number
-------------------------------*/
parm = p_find(vblock, "ACCTNUM");
if (parm->v_count == 0)
    strcpy(acctnum, "");
else
    strcpy(acctnum, SVAL(*parm, 0));


/* Retrieve the order number
-------------------------------*/
parm = p_find(vblock, "ORDERNUM");
if (parm->v_count == 0)
    strcpy(ordernum, "");
else
    strcpy(ordernum, SVAL(*parm, 0));


/* Retrieve the unit number
-------------------------------*/
parm = p_find(vblock, "UNITNUM");
if (parm->v_count == 0)
    strcpy(unitnum, "");
else
    strcpy(unitnum, SVAL(*parm, 0));


/* Retrieve the output format on tape
--------------------------------------*/
parm = p_find(vblock, "OUTFORM");
strcpy(outformstr, SVAL(*parm, 0));
i = (long)strlen(outformstr);
c_up2low(outformstr, &i);
if (strcmp(outformstr,"level0") == 0)
    *outform = 0;
else
    *outform = 1;


/* Retrieve the running directory for stackone.pdf stackone.csh
----------------------------------------------------------------*/
parm = p_find(vblock, "RUNDIR");
strcpy(rundir, SVAL(*parm, 0));


/* Get the product summary flag
-------------------------------------------*/
parm = p_find(vblock, "PRODSUM");
if (parm->v_count == 0)
    strcpy(prodsum,"no");
else
    {
    strcpy(prodsum, SVAL(*parm, 0));
    i = (long)strlen(prodsum);
    c_up2low(prodsum, &i);
    }


return(E_SUCC);
}
