/*******************************************************************************
FUNCTION:	create_group

PURPOSE:	Given the host name of an input file, a list of associated
		file extensions, and the command type that is being run, create
		a FILEGROUP structure that contains the disk name and the tape
		name for each file in the file group.  If any of the files in
		the group do not exist then a NULL ptr should be returned.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adapssup.h"
#include "util.h"
#include "labserv.h"
#include "ansistack.h"


int create_group
(
    char *filename,		/* IN  - Host file name of input file         */
    char aftype[][CMLEN],	/* IN  - List of file extensions              */
    long nassoc,		/* IN  - Number of file extensions            */
    char *command,		/* IN  - Command being run                    */
    char *ttype,		/* IN  - Type of tape being stacked to        */
    struct FILEGROUP **ptr 	/* OUT - Ptr to a new malloc'd structure      */
)

{
char dir[CMLEN];	/* Directory portion of a file name                   */
char ext[CMLEN];	/* Extension portion of a file name                   */
char root[CMLEN];	/* Root portion of a file name                        */
char fnames[GROUPSIZE][CMLEN];/* Temporary copy of disknames with directory   */

double sizes[GROUPSIZE];/* Size of each file in the group                     */
double mbytes;		/* Temporary double for computing megabytes           */

long access;		/* File access flag                                   */
long nonfatal = 1;	/* Nonfatal error flag                                */

unsigned int i;		/* Loop control                                       */

struct FILEGROUP *tmpptr;/* Temporary pointer to the new structure            */


/* Initialize ptr
-----------------*/
*ptr = NULL;


/* Allocate a new structure
---------------------------*/
tmpptr = (struct FILEGROUP *)malloc(sizeof(struct FILEGROUP));
if (tmpptr == NULL)
    {
    c_errmsg("Error allocating memory", "ansistack-alloc", &nonfatal);
    return(E_FAIL);
    }


/* Determine the directory where the file group is located and set the flag
   indicating which file name is currently be used (ie. disk or tape names).
----------------------------------------------------------------------------*/
c_splitname(filename, dir, root, ext);
strcpy(tmpptr->dir, dir);
tmpptr->nameflag = DISKNAME;


/* Determine the name without the directory specification
---------------------------------------------------------*/
if (strlen(ext))
    sprintf(tmpptr->disknames[0], "%s.%s", root, ext);
else
    strcpy(tmpptr->disknames[0], root);


/* Add the name of each associated file to the group
----------------------------------------------------*/
if (strcmp(command, "mssx") == 0)
    {
    i = strlen(root) - 1;   /* index of last character in file name */
    root[i] = '1';
    c_lsmknm(root, aftype[0], tmpptr->disknames[1]);
    root[i] = '2';
    c_lsmknm(root, aftype[1], tmpptr->disknames[2]);
    root[i] = '3';
    c_lsmknm(root, aftype[2], tmpptr->disknames[3]);
    root[i] = '4';
    c_lsmknm(root, aftype[3], tmpptr->disknames[4]);
    root[i] = 'c';
    c_lsmknm(root, aftype[4], tmpptr->disknames[5]);
    root[i] = 'o';
    c_lsmknm(root, aftype[5], tmpptr->disknames[6]);
    }
else
    {
    for (i = 0; i < nassoc; i++)
	c_lsmknm(tmpptr->disknames[0], aftype[i], tmpptr->disknames[i+1]);
    }
tmpptr->nfiles = nassoc + 1;


/* Ensure each file of the group exists
---------------------------------------*/
access = 0; /* existence flag */
for (i = 0; i < tmpptr->nfiles; i++)
    {
    /* If one of the files does not exist then deallocate the structure and
       return successfully.  This indicates the entire file group does not
       exist but this is not an error.
    -----------------------------------------------------------------------*/
    sprintf(fnames[i], "%s%s", tmpptr->dir, tmpptr->disknames[i]);
    if (c_lsstat(fnames[i], &access) != E_SUCC)
	{
	free(tmpptr);
	return(E_SUCC);
	}
    }


/* Create the tape name for each file in the group
--------------------------------------------------*/
tape_names(tmpptr, command);


/* Determine the total size of all the files in the group.  If any of the
   file sizes are less than zero then there was an error determining the
   size of that file and this group of files should be ignored.  Convert
   the total size to mbytes.
-------------------------------------------------------------------------*/
c_getsizes(fnames, tmpptr->nfiles, sizes);
mbytes = 0.0;
for (i = 0; i < tmpptr->nfiles; i++)
    {
    if (sizes[i] < 0)
	{
	free(tmpptr);
	return(E_SUCC);
	}
    mbytes += sizes[i];
    }
mbytes /= 1000000.0;


/* Add the number of mbytes that will be used by tape marks to the total size
   NOTES:
       1. The constants representing the number of mbytes used per tape
	  mark should be real values so no precision is lost during the
	  computations.
       2. There are 3 tape marks for each file in the file group (ansi
	  header file, file from the group, ansi trailer file).
--------------------------------------------------------------------------*/
if (strcmp(ttype, "9-track") == 0)
    mbytes += (TMARK_9TRACK * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "3480") == 0)
    mbytes += (TMARK_3480 * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "3490") == 0)
    mbytes += (TMARK_3490 * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "4mm") == 0)
    mbytes += (TMARK_4MM * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "8lo") == 0)
    mbytes += (TMARK_8LO * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "8hi") == 0)
    mbytes += (TMARK_8HI * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "dlt4000") == 0)
    mbytes += (TMARK_8HI * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "dlt7000") == 0)
    mbytes += (TMARK_8HI * 3.0 * (double)tmpptr->nfiles);
else if (strcmp(ttype, "d3") == 0)
    mbytes += (TMARK_8HI * 3.0 * (double)tmpptr->nfiles);



/* Save the total size in the group structure
---------------------------------------------*/
mbytes = ceil(mbytes);
tmpptr->mbytes = (long)mbytes;


/* Return a pointer to the new structure
----------------------------------------*/
*ptr = tmpptr;
return(E_SUCC);
}
