/*******************************************************************************
FUNCTION:	select_groups

PURPOSE:	Given the type of tape that is being stacked, the minimum and
		maximum number of megabytes to be stacked to the tape, and a
		queue of eligible file groups, determine which file groups
		should be stacked to the tape.  All groups that are to be
		stacked to the tape should be removed from the eligible queue
		and placed in the stack queue.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "queue.h"
#include "ansistack.h"



int select_groups
(
    char *ttype,		/* IN  - Type of tape (4mm,8mm,3480,9-track..)*/
    long min_mbytes,		/* IN  - Min megabytes to stack to the tape   */
    long max_mbytes,		/* IN  - Max megabytes to stack to the tape   */
    long mbytes,		/* IN  - Mbytes already stacked to tape       */
    long q_eligible,		/* IN  - Queue of eligible file groups        */
    long q_stack 		/* OUT - Queue of file groups to be stacked   */
)

{

long first_group;		/* Position of first group to be stacked in q */
long pos;			/* Position within a queue                    */
long total_mbytes;		/* Total megabytes used by the selected groups*/

short found;			/* Indicates we have found groups to stack    */

struct FILEGROUP *group;	/* Ptr to a file group                        */



/* If queue of eligible groups is empty, simply return
------------------------------------------------------*/
if (c_qsize(q_eligible) == 0)
    return(E_SUCC);


/* Get the first eligible file group in the queue
-------------------------------------------------*/
c_qfirst(q_eligible, (void **)&group);


/* For high capacity 4mm and 8mm tapes select the first file that will fit on
   the tape.  For low capacity 3480 cartridges, 3490, and 9-track tapes select
   a set of files that will best fit on the tape.  This will increase the tape
   utilization.
-----------------------------------------------------------------------------*/
found = FALSE;
if ((strcmp(ttype, "4mm") == 0) ||
    (strcmp(ttype, "8hi") == 0) ||
    (strcmp(ttype, "8lo") == 0))
    {
    /* Select the first file group that will fit on the tape
    --------------------------------------------------------*/
    pos = 0;
    while (!found && (group != NULL))
	{
	pos++;
	total_mbytes = group->mbytes + mbytes;
	if (total_mbytes <= max_mbytes)
	    {
	    found = TRUE;
	    c_qremove(q_eligible, pos, (void **)&group);
	    c_qinsert(q_stack, group, compare_tape_names);
	    }
	else
	    c_qnext(q_eligible, (void **)&group);
	}
    }
else
    {
    /* Determine if there is a set of file groups that will fit on the tape.
       By just looking for a set of groups first, there is no need to transfer
       each group from the eligible queue to the stack queue and then back
       again if nothing will fill the tape.
    --------------------------------------------------------------------------*/
    first_group = 0;
    while (!found && (first_group < c_qsize(q_eligible)))
	{
	first_group++;
	total_mbytes = mbytes;
	c_qposition(q_eligible, first_group, (void **)&group);
	do
	    {
	    if ((total_mbytes + group->mbytes) <= max_mbytes)
		total_mbytes += group->mbytes;
	    c_qnext(q_eligible, (void **)&group);
	    } while (group != NULL);


	/* Did we find a set of groups that will fill the tape
	------------------------------------------------------*/
	if ((total_mbytes >= min_mbytes) && (total_mbytes <= max_mbytes))
	    found = TRUE;
	}
    

    /* If we have found a set of groups that will fill the tape, starting
       with first_group, transfer each of these groups from the eligible
       queue to the stack queue.
    ---------------------------------------------------------------------*/
    if (found)
	{
	pos = first_group;
	total_mbytes = mbytes;
	while (pos <= c_qsize(q_eligible))
	    {
	    c_qposition(q_eligible, pos, (void **)&group);
	    if ((total_mbytes + group->mbytes) <= max_mbytes)
		{
		total_mbytes += group->mbytes;
		c_qremove(q_eligible, pos, (void **)&group);
		c_qinsert(q_stack, group, compare_tape_names);
		}
	    else
		pos++;
	    }
	}
    }


return(E_SUCC);
}
