/*******************************************************************************
FUNCTION:	stack_group

PURPOSE:	Stack one file group to tape.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "adapsutil.h"
#include "ansistack_wrt.h"



/* Define a location for the terminal output from ANSIOUT on UNIX machines.
   If you would like to have the ANSIOUT messages written to the terminal
   then define OUTPUT to be a space.  Note that the system function executes
   commands in the Bourne Shell so use the appropriate type of redirection.

   OUTPUT = ">> /dev/null 2>> /dev/null"   Throw output away
   OUTPUT = " "                            Display output on terminal
------------------------------------------------------------------------------*/
#define OUTPUT ">> /dev/null 2>> /dev/null"


int stack_group
(
    struct FILEGROUP *ptr,/* IN - Ptr to a file group                        */
    long record_size[],	/* IN  - Record size for each file in the group       */
    long block_size[],	/* IN  - Block size for each file in the group        */
    char drive[],	/* IN  - Tape drive                                   */
    char ttype[],	/* IN  - Tape type (4mm, 8lo, 8hi, 3480, 3490,9-track)*/
    char tapeid[],	/* IN  - Tape identification number                   */
    char tapelist[],	/* IN  - Name of file containing file names           */
    char logfile[],	/* IN  - Name of file containing file names           */
    long nfiles,	/* IN  - Number of files already stacked to tape      */
    char acctnum[],	/* IN  - Account number				      */
    char ordernum[],	/* IN  - Order number				      */
    char unitnum[],	/* IN  - Unit number				      */
    long outform,	/* IN  - Output format 0=Level 0, 1=Level 1B	      */
    char rundir[],	/* IN  - Run directory				      */
    char prodsum[] 	/* IN  - Product summary flag			      */
)

{
char cmd[CMLEN*20];	/* Command sent to system to write a file to tape     */
char key[CMLEN];	/* Key for logging processing messages to logfile     */
char root[CMLEN];	/* Root portion of a file name                        */
char str[CMLEN];	/* Temporary string                                   */
char tapedens[CMLEN];   /* Tape density (LO, HI, or 0)		   	      */
char l1bname[CMLEN];	/* Level 1B name without the suffix		      */

long l1blen;		/* Length of l1bname				      */
long i;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */
long tplen    = 2400;	/* Length of tape in feet                             */

FILE *fp      = NULL;	/* File ptr for file containing list of tape names    */


/* Change into the directory where the file group is located.  This is done so
   that the directory path does not have to be specified to ansiout.  Each file
   is only allowed TPNAMELEN characters for their file name and we don't want
   to waste any of these on directory paths.
------------------------------------------------------------------------------*/
chdir(ptr->dir);


/* Open the file that should contain all of the tape names for each file that
   has been stacked to tape.  This file should only be created for 4mm, 8hi,
   and 8lo type tapes.
-----------------------------------------------------------------------------*/
if ((strcmp(ttype,"4mm") == 0) ||
    (strcmp(ttype,"8hi") == 0) ||
    (strcmp(ttype,"8lo") == 0))
    {
    fp = fopen(tapelist, "a");
    if (fp == NULL)
	{
	c_errmsg("Error opening temporary file", "ansistack-open", &nonfatal);
	return(E_FAIL);
	}
    }


/* If the user supplied a log file
----------------------------------*/
if (strlen(logfile))
    {
    c_splitname(ptr->disknames[0], str, root, str);
    sprintf(key, "drive_%s", drive);
    sprintf(str, "Writing %s to %s", root, tapeid);
    c_wrtlog(logfile, key, "stack-write", str);
    }


/* Set tapedens if working with 8mm
------------------------------------*/
if (!strcmp(ttype,"8hi"))
    strcpy(tapedens,"HI");
else if (!strcmp(ttype,"8lo"))
    strcpy(tapedens,"LO");
else
    strcpy(tapedens,"0");
		

/* Stack each of the files in the group to tape
-----------------------------------------------*/
for (i = 0; i < ptr->nfiles; )
    {
    nfiles++;

    /* If generating a Level 1B product, get the disk name without the suffix */
    if (outform == 1)
	{
	l1blen = (long) strlen(ptr->disknames[i]) - 5;
	strncpy(l1bname,ptr->disknames[i],l1blen);
	l1bname[l1blen] = '\0';
	}


    /* Create the command that will write each file to tape.  Note that the
       first file that is written to tape should not use the -A (append) flag
       for ansiout.  All subsequent files should use this flag.  The options
       for ansiout are case sensitive, however, VMS systems will uppercase all
       of the options.  Therefore, the options must be enclosed in double
       quotes for VMS systems which will have no effect on UNIX machines.

       A = Append file to tape from current tape position (flag)
       a = append to existing ANSI labeled tape
       T = Tape drive                                     value required
       v = Volume id                                      value required
       F = Fixed length records                           (flag)
       b = Block size                                     value required
       d = Density                                        value required
       r = Record size                                    value required
       f = # of this file on the tape                     value required
       t = Tape length                                    value required
       R = No rewind                                      (flag)
    --------------------------------------------------------------------------*/
    if (outform == 0)
	{
	if (nfiles == 1)
	    sprintf(cmd,"ansiout \"-TvFrbdftR\" %s %s %1d %1d %s %1d %1d %s %s",
		    drive, tapeid, record_size[i], block_size[i], tapedens,
		    nfiles, tplen, ptr->tapenames[i], OUTPUT);
	else
	    sprintf(cmd,"ansiout \"-ATvFrbdftR\" %s %s %1d %1d %s %1d %1d %s %s"
		    ,drive, tapeid, record_size[i], block_size[i], tapedens,
		    nfiles, tplen, ptr->tapenames[i], OUTPUT);
	if (strcmp(prodsum,"yes") == 0)
	    wrtprdsum(acctnum,ttype,ptr->disknames[i],ordernum,unitnum);
	}
    else
	{
	/* Build the command line string for system call to mklevel1b
	--------------------------------------------------------------*/
	sprintf( cmd, "%s/mklevel1b -line 'Please mount tape' ", rundir );
	sprintf( str, "'%s' %s %s %s ", l1bname, acctnum, ordernum, unitnum );
	strcat( cmd, str );
	sprintf( str, "%s %s ", drive, ttype );
	strcat( cmd, str );
	strcat( cmd, "mvolflg=no " );
	}

    /* Call the operating system to write the file
    ----------------------------------------------*/
    if (system(cmd) != SYS_SUCC)
	{
	/* Issue an error message to the terminal
	-----------------------------------------*/
	if (outform == 0)
	    sprintf(str, "Error writing %s", ptr->disknames[i]);
	else
	    sprintf(str, "Error writing %s", l1bname);
	c_errmsg(str, "ansistack-wrt", &nonfatal);


	/* If the user supplied a log file
	----------------------------------*/
	if (strlen(logfile))
	    {
	    if (outform == 0)
		sprintf(str, "Error writing %s to %s", ptr->disknames[i],
			tapeid);
	    else
		sprintf(str, "Error writing %s to %s", l1bname, tapeid);
	    c_wrtlog(logfile, key, "stack-error", str);
	    }


	/* Remove the ansiotmp
	----------------------*/
	sprintf(str, "%sansiotmp", ptr->dir);
	remove(str);


	if (fp != NULL) fclose(fp);
	return(E_FAIL);
	}


    /* Informational message after writing the file
    -----------------------------------------------*/
    if (outform == 0)
	sprintf(str, "Successfully wrote %s", ptr->disknames[i]);
    else
	sprintf(str, "Successfully wrote %s", l1bname);
    c_errmsg(str, " ", &nonfatal);
    

    /* Add the tape name of the current file to the tapelist file
    -------------------------------------------------------------*/
    if (outform == 0)
	if (fp != NULL) fprintf(fp, "%s\n", ptr->tapenames[i]);
    else
	if (fp != NULL) fprintf(fp, "%s\n", l1bname);


    /* For Level 0 products, write each file out, for Level 1B products,
       write out 1 file for every 2 (.ahdr and .arch => .l1b)
    -------------------------------------------------------------*/
    if (outform == 0)
	i++;
    else
	i+=2;

    }


/* If the user supplied a log file
----------------------------------*/
if (strlen(logfile))
    {
    sprintf(str, "Wrote   %s to %s", root, tapeid);
    c_wrtlog(logfile, key, "stack-wrote", str);
    }


/* Close the file if open
-------------------------*/
if (fp != NULL) fclose(fp);


/* Remove the ansiotmp file
---------------------------*/
sprintf(str, "%sansiotmp", ptr->dir);
remove(str);


return(E_SUCC);
}
