/*************************************************************************
FUNCTION:	WRTPRDSUM

PURPOSE:	

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       09/96    T. Baltzer          Original development stole from MK1B.
 1.1	   07/98    D.Lloyd		Modified for Y2K compliance

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
                NOAA Polar Orbiter Data User's Guide
                  K. Kidwell, NOAA
                  Section 2 Level 1B Data Base
                  Section 3 AVHRR Level 1B Data Base
                NOAA Technical Memorandum NESS 107
                  Section 5.1 AVHRR Calibration, Appendix A, Appendix B

*************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "archio.h"
#include "ansistack_wrt.h"

#ifndef DEG
#define DEG(p)   p * 180 / PIE
#endif
#ifndef MIN
#define MIN(a,b)  ((a) < (b) ? (a) : (b))
#endif
#ifndef MIN4
#define MIN4(a,b,c,d) (MIN(a,b) < MIN(c,d) ? MIN(a,b) : MIN(c,d))
#endif
#ifndef MAX4
#define MAX4(a,b,c,d) (LASMAX(a,b) > LASMAX(c,d) ? LASMAX(a,b) : LASMAX(c,d))
#endif


void wrtprdsum
(
char	acctnum[10],     	/* account number                       */
char	ttype[5],		/* tape type				*/
char	archnam[CMLEN],		/* input archive image host file name   */
char	ordernum[14],   	/* order number                         */
char	unitnum[5]      	/* unit number                          */
)

{

struct MODEL	model;		/* Model and orbital information        */
struct DROPPED	drop;		/* dropped lines			*/

long	fatal = -1,		/* fatal error status			*/
	status;			/* general return status		*/

float	rgwin[4];		/* lat/long of original window 		*/

char	density[5],       	/* tape density                         */
	directory[CMLEN],	/* directory portion of path/file	*/
	extension[CMLEN],	/* suffix portion of path/file		*/
	out[2][CMLEN],		/* output file names			*/
        sceneid[30];    	/* scene id                       	*/

/*
  Determine the desired density
*/
if (!strcasecmp(ttype,"8hi"))
    strcpy(density,"HI");
else if (!strcasecmp(ttype,"8lo"))
    strcpy(density,"LO");
else
    strcpy(density,"0");

/*
  Get the scene id
*/
c_splitname(archnam,directory,sceneid,extension);

/*
  Fill the model and drpl structures with info from the header file
*/
status = c_getahdr(archnam,&model,&drop);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ansistack-fatal",&fatal);

/*
  Determine the original lat/long of image
*/
prepimg(rgwin, &model);

/*
  Create product summary
*/
suminfo(out[0],acctnum,ordernum,unitnum,sceneid,"LEVEL-0",
	rgwin,density,&model);

}

/******************************************************************************
FUNCTION:	suminfo

PURPOSE:	Create a product summary

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0	   09/96    T. Baltzer 		Stole from MK1B

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/

#define EL 2
#define ES 3

void suminfo
(
char	out[CMLEN],	/* output file names			*/
char	acctnum[10],    /* account number                       */
char    ordernum[14],   /* order number                         */
char	unitnum[5],     /* unit number                          */
char	sceneid[30],    /* input scene id                       */
char	prodesc[18],    /* product description                  */
float	rgwin[4],	/* original window in lat/long form	*/
char	density[5],	/* tape density				*/
struct MODEL *model     /* Model and orbital information	*/
)


{
double  elev = 0.0,     /* elevation of data                    */
        lat1,long1,     /* lat/long upper left corner           */
        lat2,long2,     /* lat/long upper right corner          */
        lat3,long3,     /* lat/long lower left corner           */
        lat4,long4,     /* lat/long lower right corner          */
        *view_angles = NULL;    /* view angles flag             */

long	gday,		/* gregorian day			*/
	month,		/* month				*/
	nl,ns;		/* # lines, samples in output file    	*/

char	acqdat[9],	/* formatted acquisition date		*/
	acqtim[9],	/* formateed acquisition time		*/
	curdate[9],	/* current date and time		*/
	satid[8],	/* satellite id				*/
        source[5];      /* Data source                          */

float   beglat,         /* beginning lat                        */
        beglong,        /* beginning long                       */
        endlat,         /* ending lat                           */
        endlong,        /* ending long                          */
	inwin[4];       /* input window                         */

long	now;		/* time now				*/
struct tm *curtime;	/* current time				*/

/*
  Set up satellite ID name
*/
strcpy(satid,"NOAA-");
sprintf(&satid[5],"%02d",model->satnum);

/*
  Determine data source from image title
*/
strcpy(source,"HRPT");

/*
  Determine current date and time
*/
now = time(NULL);
curtime = localtime(&now);
/***********************************************
DLLOYD Y2K
  tm_year will be 100 in 2000, force to 2-digit
************************************************/
sprintf(curdate,"%02d/%02d/%02d",
    (curtime->tm_mon + 1),curtime->tm_mday,curtime->tm_year%100);

/*
  Determine the gregorian acquisition date ???? use model->month etc.
*/
c_julgre(&(model->year),&(model->jday),&month,&gday);

/*
  Generate a formatted acquisition date
*/
sprintf(acqdat,"%02d/%02d/%02d",month,gday,model->year);

/*
  Generate a formatted acquisition time
*/
sprintf(acqtim,"%02d:%02d:%02d",model->shour,model->smin,model->ssec);

/*
  Save the output area of the entire image
*/
inwin[SL] = 1;
inwin[SS] = 1;
inwin[EL] = (float)model->nlines;
inwin[ES] = (float)model->nsamples;
c_ls2ll(&inwin[SL],&inwin[SS],&elev,model,&lat1,&long1,view_angles);
c_ls2ll(&inwin[SL],&inwin[ES],&elev,model,&lat2,&long2,view_angles);
c_ls2ll(&inwin[EL],&inwin[SS],&elev,model,&lat3,&long3,view_angles);
c_ls2ll(&inwin[EL],&inwin[ES],&elev,model,&lat4,&long4,view_angles);

beglat  = (float)DEG(MAX4(lat1,lat2,lat3,lat4));
endlat  = (float)DEG(MIN4(lat1,lat2,lat3,lat4));
beglong = (float)DEG(MAX4(long1,long2,long3,long4));
endlong = (float)DEG(MIN4(long1,long2,long3,long4));

/*
  Set number of lines and samples in output image
*/
nl = model->nlines;
ns = model->nsamples;

/*
  Do the product summary
*/
dosummary(out,acctnum,ordernum,unitnum,curdate,sceneid,model->sorbit,acqdat,
	  acqtim,prodesc,satid,source,rgwin[0],rgwin[1],rgwin[2],rgwin[3],
	  beglat,endlat,beglong,endlong,nl,ns,density);

return;
}

/******************************************************************************
FUNCTION:	PREPIMG

PURPOSE:	Determine the original window in lat/long specification.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       09/96    T. Baltzer          Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/

int prepimg
(
float	rgwin[4],		/* original window			*/
struct MODEL *model 		/* Model and orbital information	*/
)

{

double	elev = 0.0,		/* elevation of data			*/
	lat,lng,		/* latitude/longitude			*/
	*view_angles = NULL;	/* calculated view angles flag		*/

float	temp1,			/* Temporary floating pt value		*/
	temp2;			/* Temporary floating pt value		*/


temp1 = 1;				/* Starting line of image */
temp2 = 1;				/* Starting sample of image */
c_ls2ll(&temp1,&temp2,&elev,model,&lat,&lng,view_angles);
rgwin[0] = (float)(DEG(lat));
rgwin[1] = (float)(DEG(lng));

temp1 = (float)model->nlines;		/* Number of lines in image */
temp2 = (float)model->nsamples;		/* Number of samples in image */
c_ls2ll(&temp1,&temp2,&elev,model,&lat,&lng,view_angles);
rgwin[2] = (float)(DEG(lat));
rgwin[3] = (float)(DEG(lng));


return(E_SUCC);
}

/*************************************************************************
FUNCTION:	DOSUMMARY

PURPOSE:	Generate the product summary report

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       09/96    T. Baltzer          Stole from MK1B

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*************************************************************************/
#define PWIDE 80		/* width of page			*/

void dosummary
(
    char    *out,		/* output file name			*/
    char    *acctnum,           /* account number                       */
    char    *ordernum,          /* order number                         */
    char    *unitnum,           /* unit number                          */
    char    *cdate,             /* current date                         */
    char    *sceneid,           /* scene identifier                     */
    long    orbit,              /* orbit number                         */
    char    *acqdate,           /* acquisition date                     */
    char    *acqtime,           /* acquisition time                     */
    char    *prodesc,           /* product description                  */
    char    *satid,             /* satellite name                       */
    char    *source,            /* sensor data type (HRPT)              */
    float   uplat,              /* upper left latitude                  */
    float   uplong,             /* upper left longitude                 */
    float   lolat,              /* lower right latitude                 */
    float   lolong,             /* lower right longitude                */
    float   beglat,             /* beginning latitude of output file    */
    float   endlat,             /* ending latitude of output file       */
    float   beglong,            /* beginning longitude of output file   */
    float   endlong,            /* ending longitude of output file      */
    long    nl,                 /* number of lines in output file       */
    long    ns,                 /* number of samples in output file     */
    char    *density            /* tape density (6250, 1600)            */
)


{
FILE	*fp;			/* pointer to the output file		*/

long	i;			/* loop counter				*/

char	buffer[PWIDE],		/* output buffer			*/
	blanks[PWIDE],		/* 80 characters of blanks		*/
	fill[PWIDE];		/* string of blanks for filler		*/

/*
  Open the product summary file
*/
sprintf(out,"%s.prs",ordernum);
fp = fopen(out,"w");

/*
  Generate a string of blanks
*/
for (i = 0; i < PWIDE; i++)
    blanks[i] = ' ';
blanks[PWIDE - 1] = '\0';

/*
  Print the header for the product summary
*/

/* skip two lines from the top of the page */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* generate the title line */
strcpy(fill,&blanks[PWIDE-26]);
sprintf(buffer,"%sAVHRR PRODUCT SUMMARY REPORT",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the underline for the title */
sprintf(buffer,"%s----------------------------",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* write out the account and date */
strcpy(fill,&blanks[PWIDE-17]);
sprintf(buffer,"%sAccount: %s",fill,acctnum);
strcat(buffer,&blanks[PWIDE-57+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Date: %s",cdate);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* write out the order and unit numbers */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sOrder-units #: %s-%s",fill,ordernum,unitnum);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/*
  Print the identification information for the product summary
*/

/* generate the string buffer for the scene id and orbit number */
strcpy(fill,&blanks[PWIDE-7]);
sprintf(buffer,"%sScene Identification: %s",fill,sceneid);
strcat(buffer,&blanks[PWIDE-56+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Orbit: %d",orbit);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the acquistion date and time */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sAcquisition Date: %s",fill,acqdate);
strcat(buffer,&blanks[PWIDE-57+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Time: %s",acqtime);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the product description and satellite */
strcpy(fill,&blanks[PWIDE-8]);
sprintf(buffer,"%sProduct Description: %s",fill,prodesc);
strcat(buffer,&blanks[PWIDE-52+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Satellite: %s",satid);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the sensor data type */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sSensor Data Type: %s",fill,source);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);
	
/*
  Print the user specified area for the product summary
*/

/* generate the title line and write it to the output file */
strcpy(fill,&blanks[PWIDE-31]);
sprintf(buffer,"%sUser Specified Area",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the underline for the title and write it to the output file */
sprintf(buffer,"%s-------------------",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* generate the string buffer for the upper left lat and beginning lat */
strcpy(fill,&blanks[PWIDE-8]);
sprintf(buffer,"%sUpper Left Latitude: %f",fill,uplat);
strcat(buffer,&blanks[PWIDE-43+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Beginning Latitude: %f",uplat);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the upper left long and ending lat */
strcpy(fill,&blanks[PWIDE-7]);
sprintf(buffer,"%sUpper Left Longitude: %f",fill,uplong);
strcat(buffer,&blanks[PWIDE-46+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Ending Latitude: %f",lolat);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the lower right latitude area */
sprintf(buffer,"%sLower Right Latitude: %f",fill,lolat);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the lower left latitude area */
strcpy(fill,&blanks[PWIDE-6]);
sprintf(buffer,"%sLower Right Longitude: %f",fill,lolong);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);
	
/*
  Print delivered product information for the product summary 
*/

/* generate the title line and write it to the output file */
strcpy(fill,&blanks[PWIDE-32]);
sprintf(buffer,"%sDelivered Product",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the underline for the title and write it to the output file */
sprintf(buffer,"%s-----------------",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* generate the string buffer for the beginning latitude and density */
strcpy(fill,&blanks[PWIDE-9]);
sprintf(buffer,"%sBeginning Latitude: %f",fill,beglat);
strcat(buffer,&blanks[PWIDE-54+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Density: %s",density);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the ending latitude and number of lines */
strcpy(fill,&blanks[PWIDE-12]);
sprintf(buffer,"%sEnding Latitude: %f",fill,endlat);
strcat(buffer,&blanks[PWIDE-49+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Number Lines: %d",nl);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the beginning long and number of samples */
strcpy(fill,&blanks[PWIDE-8]);
sprintf(buffer,"%sBeginning Longitude: %f",fill,beglong);
strcat(buffer,&blanks[PWIDE-47+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Number Samples: %d",ns);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the ending longitude */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sEnding Longitude: %f",fill,endlong);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);
	
/*
  Close the product summary file
*/
fclose (fp);

return;
}
