#!/bin/csh -f
#
# FUNCTION:  aobrw_list.csh
#
# PURPOSE:   Compile a list of browse images that will fit on a tape
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# T.Baltzer        02/97    Development
#
#


#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrup happens
#	With the - argument, the shell ignores all interrupts.  
#	With a label argument, the shell executes a goto label
onintr ERROUT
set noglob
unalias du


#-----------------------------------------------------------------------------
# Ensure the correct number of arguments have been specified.
#-----------------------------------------------------------------------------
if ($#argv != 3) then
    echo ""
    echo "Usage:  aobrw_list.csh input_file tape_size wrtlog_path"
    echo ""
    echo "     <input_file> is the name of the browse image list."
    echo "     <tape_size> is the maximum size of the tape."
    echo "     <wrtlog_path> is the path of the write_log executable."
    echo ""
    echo "[aobrw_list-fatal] Fatal error encountered"
    exit 1
endif


#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[aobrw_list-env] ADAPSLOG or ADAPSERR are not set"
    echo "[aobrw_list-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]archbrw" | tr / . | sed  's/\[\./\[/'`


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we may be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------
if ("$3" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $3/write_log
endif


#-----------------------------
# Set up variable names
#-----------------------------
set input_file = $1
set tapesize = $2


#---------------------------------------------------
# Make sure the list (file) does not already exist
#---------------------------------------------------
if (-e $input_file) then
    set text = "List ${input_file} already exists"
    wrtlog $logfile archbrw aobrw_list $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1
endif


#---------------------------------------------------------------------------
# Loop thru all of the browse images until no more will fit on the specified
# tape size.  Generate a list of the ones that will fit.
#
# Set the maximum number of browse images that can be put on one tape.
# This limit is necessary because of the unix size constraint
# ("Arguments too long" problem).
#---------------------------------------------------------------------------
set img_count = 0
set totalsize = 0
set images = `/bin/ls -tr | grep '^ao.*\.brw$' | awk ' (NR <= 350) {print}'`
foreach img ($images)
    set blocksize = `du -sk $img | awk '{print $1}'`

    #----------------------------------------
    # Account for tape marks between files -
    # .038 Meg per file or about 38 blocks
    #----------------------------------------
    @ blocksize += 38


    #----------------------------------------
    # Increment accumulative block size
    #----------------------------------------
    @ totalsize += $blocksize

    #----------------------------------------------------------
    # If accumulative block size exceeds tape size then quit
    #----------------------------------------------------------
    if ($totalsize > $tapesize) then
	break
    endif

    echo $img >> $input_file
    @ img_count += 1

end


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
set text = "Generated list ${input_file} with ${img_count} entries"
wrtlog $logfile archbrw aobrw_list $text
echo "[informational]" $text
exit 0


#------------------------------------------------------------------
# If an error occurs, delete the list and write an error message
# to the log and screen.
#------------------------------------------------------------------
ERROUT:
    onintr -
    rm -f $input_file

    set text = "Error generating list ${input_file}"
    wrtlog $logfile archbrw aobrw_list $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1

