#!/bin/csh -f
#
# FUNCTION:  aobrw_summ.csh
#
# PURPOSE:   Generate a summary listing of browse images that went on a tape.
#	     Delete the browse images from disk, delete the temporary listing.
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# T.Baltzer        02/97    Development
#
#


#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrup happens
#	With the - argument, the shell ignores all interrupts.  
#	With a label argument, the shell executes a goto label
onintr ERROUT
set noglob


#-----------------------------------------------------------------------------
# Ensure the correct number of arguments have been specified.
#-----------------------------------------------------------------------------
if ($#argv != 6) then
   echo ""
   echo "Usage:  aobrw_summ.csh wrtlog_path input_file tape_id tape_type "
   echo "                       output_system drive"
   echo ""
   echo "       <wrtlog_path> is the path of the write_log executable."
   echo "       <input_file> is the list of files to be transferred."
   echo "       <tape_id> is the tape ID of tape being processed."
   echo "       <tape_type> is the type of tape being processed."
   echo "       <output_system> is the system the tape is being generated on."
   echo "       <drive> is the tape drive used to generate the tape."
   echo ""
   echo "[aobrw_summ-fatal] Fatal error encountered"
   exit 1
endif


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we may be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------
if ("$1" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $1/write_log
endif


#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSLOG) || (! $?ADAPSBRW)) then
    echo "[aobrw_summ-env] ADAPSLOG or ADAPSBRW are not set"
    echo "[aobrw_summ-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]archbrw" | tr / . | sed  's/\[\./\[/'`


#--------------------------------------------
# Assign the arguments to local variables
#--------------------------------------------
set input_file = $2
set tapeid = $3
set tapetype = $4
set output_system = $5
set drive = $6


#---------------------------------------------------
# Make sure the input list (file) exists
#---------------------------------------------------
if (! -e $input_file) then
    set text = "List ${input_file} doesn't exist"
    wrtlog $logfile archbrw aobrw_summ $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the summary listing.
#----------------------------------------------------------------------
set summlist = $ADAPSBRW"/archive/summary"$tapeid".list"


#--------------------------------------------
# Generate the header of the summary listing
#--------------------------------------------
set nfiles = `wc $input_file | awk '{print $1}'`
set datetime = `date '+Created: %m/%d/%y %H:%M:%S'`

echo "TapeID:  $tapeid        TapeType $tapetype           $datetime" >> $summlist
echo "System:  $output_system       Drive:    $drive" >> $summlist
echo "Nfiles:  $nfiles" >> $summlist
echo "" >> $summlist
echo "------------------------------" >> $summlist


#-----------------------------------------------------
# Write all of the file names to the summary listing
#-----------------------------------------------------
cat $input_file >> $summlist


#----------------------------------------------------------------------------
# Since everything has ran to this point without error, we will now delete
# all of the browse images listed in the temporary browse list, and then
# delete the temporary browse list itself.
#----------------------------------------------------------------------------
unset noglob
set files = `cat $input_file`
set noglob
foreach file ($files)
    rm -f $file
    if (! $status) then
	set text = "Successfully deleted image $file"
	wrtlog $logfile aobrw_summ archbrw-success $text
    else
	set text = "ERROR deleting image $file"
	wrtlog $logfile aobrw_summ archbrw-del $text
	echo "[informational]" $text
	exit 1
    endif
end

rm -f $input_file
if (! $status) then
    set text = "Successfully deleted file $input_file"
    wrtlog $logfile aobrw_summ archbrw-success $text
else
    set text = "ERROR deleting file $input_file"
    wrtlog $logfile aobrw_summ archbrw-del $text
    echo "[informational]" $text
    exit 1
endif


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
set text = "Generated summary${tapeid}.list with ${nfiles} entries"
wrtlog $logfile archbrw aobrw_summ $text
echo "[informational]" $text
exit 0


#------------------------------------------------------------------
# If an error occurs, write an error message to the log and screen.
#------------------------------------------------------------------
ERROUT:
    onintr -

    set text = "Error generating summary for ${tapeid}"
    wrtlog $logfile archbrw aobrw_summ $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1

