#!/bin/csh -f
#
# FUNCTION:  aobrw_tape.csh
#
# PURPOSE:   Generate a tape of browse images based on an input list of browse
#            file names.
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# T.Baltzer        02/97    Development
#
#

#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrup happens
#	With the - argument, the shell ignores all interrupts.  
#	With a label argument, the shell executes a goto label
onintr ERROUT
set noglob


#-----------------------------------------------------------------------------
# Ensure the correct number of arguments have been specified.
#-----------------------------------------------------------------------------
if ($#argv != 4) then
   echo ""
   echo "Usage: aobrw_tape.csh input_file output_system device_name wrtlog_path"
   echo ""
   echo "       <input_file> is the list of files to be processed."
   echo "       <output_system> is the system the tape is being generated on."
   echo "       <device_name> is the name of the tape device."
   echo "       <wrtlog_path> is the path of the write_log executable."
   echo ""
   echo "[aobrw_tape-fatal] Fatal error encountered"
   exit 1
endif


#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[aobrw_tape-env] ADAPSLOG or ADAPSERR are not set"
    echo "[aobrw_tape-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]archbrw" | tr / . | sed  's/\[\./\[/'`


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we may be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------
if ("$4" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $4/write_log
endif


#-----------------------------
# Set up variable names
#-----------------------------
set input_file=$1
set system=$2
set device=$3


#---------------------------------------------------
# Make sure the list (file) already exists
#---------------------------------------------------
if (! -e $input_file) then
    set text = "Input list ${input_file} does not exist"
    wrtlog $logfile archbrw aobrw_tape $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Read the input list of browse images and pipe it to tar.  The output
# of tar is piped to dd which will actually write it onto tape.
#----------------------------------------------------------------------
cat $input_file | tar -cf - - | rsh $system dd obs=10240 of=$device

if ($status != 0) then
    set text = "Error generating tape for ${input_file}"
    wrtlog $logfile archbrw aobrw_tape $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1
endif


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
set text = "Generated a tape for list ${input_file}"
wrtlog $logfile archbrw aobrw_tape $text
echo "[informational]" $text
exit 0


#------------------------------------------------------------------
# If an error occurs, write an error message to the log and screen.
#------------------------------------------------------------------
ERROUT:
    onintr -

    set text = "Error generating tape for ${input_file}"
    wrtlog $logfile archbrw aobrw_tape $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1

