/*******************************************************************************
FUNCTION:	archdr

PURPOSE:	Read the header file from an EAF formatted AVHRR archive tape
		to determine the tape ID and how the image(s) from the tape
		should be read.

		This program assumes that the tape has already been mounted.
		The tape will be rewound to BOT when this function exits
		successfully.

Version  Date   Programmer       Modification
-------  -----  ---------------  -----------------------------------------------
  1.1    06/93  J.Fenno          Development
  1.2    08/93  J.Fenno          Added code for the 15 character julian IDs
        
PROJECT:	ADAPS
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "adaps.h"


#define ANSIBYTES    80		/* Number of bytes in an ANSI header record   */
#define CEOSBYTES    80		/* Number of bytes in a CEOS header record    */
#define K32       32768		/* Number of bytes in a 32k tape record       */
#define K64       65024		/* Number of bytes in a 64k tape record       */
#define MAXBYTES  65024		/* Maximum number of bytes in a tape record   */
#define NAMELEN      17		/* Length of a file name in an ANSI record    */


int main()
{
char buffer[MAXBYTES];		/* Buffer for reading a record from tape      */
char drive[CMLEN];		/* Tape drive specification (2, 4, sg1@4, etc)*/
char sceneid[CMLEN];		/* Scene ID of the first image stored on tape */
char str[CMLEN];		/* Temporary string variable                  */
char *string1[1];		/* Ptr to a string for calling q_string()     */
char *string2[1];		/* Ptr to a string for calling q_string()     */
char tapeid[7];			/* Tape identification number                 */
char tapetype[CMLEN];		/* Type of tape mounted on drive              */

long access    = TREAD;		/* Type of access to the tape drive           */
long eofs      = 0;		/* Number of EOF marks to write to the tape   */
long err_mode  = HANDLE;	/* Error handling mode for tapeio functions   */
long fatal     = -1;		/* Fatal error flag                           */
long i;				/* Loop control                               */
long j;				/* Loop control                               */
long nbytes;			/* Number of bytes read from tape             */
long nonfatal  = 1;		/* Nonfatal error flag                        */
long rewind    = REWIND;	/* Rewind option for tapeio functions         */
long size;			/* Size of the buffer sent to tapeio routines */
long skip      = 0;		/* # of files to skip when a tape is opened   */
long stdlun;			/* Standard output used with LAS              */
long tpunit;			/* Tape descriptor returned by c_tpreop()     */

struct PARBLK vblock;	   	/* Block with pdf variables                   */
struct VARIABLE *var;		/* Pointer to a TAE variable in vblock        */


/* Initialize LAS 
-----------------*/ 
c_inlas(&vblock, &stdlun);


/* Get the tape drive from the variable block
---------------------------------------------*/
var = p_find(&vblock, "DRIVE");
strcpy(drive, SVAL(*var, 0));


/* Reopen the tape drive (assume the tape already has been mounted)
-------------------------------------------------------------------*/
nbytes = c_tpreop(&tpunit, &skip, &err_mode, &rewind, "0", &access, drive);
if (nbytes != TAPE_SUCC)
    c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);


/* Read the first record of the tape.  This must be an ANSI record of ANSIBYTES
   bytes.  It will contain the six character tape ID in bytes five through ten.
------------------------------------------------------------------------------*/
size = MAXBYTES;
nbytes = c_tpread(&tpunit, (unsigned char *)buffer, &size, &err_mode);
if (nbytes != ANSIBYTES)
    {
    sprintf(str, "Tape on drive %s is not labelled", drive);
    c_errmsg(str, "archdr-label", &nonfatal);
    c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
    }
i = 4;
j = 0;
while ((buffer[i] != ' ') && (j < 6))
    {
    tapeid[j++] = buffer[i++];
    tapeid[j] = '\0';
    }


/* Read the first file identification record in the ANSI header.  If there is a
   file name stored in bytes five through 21 then the tape can be read using
   ansiin.  If not, then tpacquire will have to be used.
------------------------------------------------------------------------------*/
size = MAXBYTES;
nbytes = c_tpread(&tpunit, (unsigned char *)buffer, &size, &err_mode);
if (nbytes != ANSIBYTES)
    {
    sprintf(str, "Error reading label on tape %s", tapeid);
    c_errmsg(str, "archdr-hdr", &nonfatal);
    c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
    }
i = 4;
j = 0;
while ((buffer[i] != ' ') && (buffer[i] != '.') && (j < NAMELEN))
    sceneid[j++] = buffer[i++];
sceneid[j] = '\0';


/* Skip to the next file on the tape and read the first record
   to determine how big the records are on this tape.
--------------------------------------------------------------*/
skip = 1;
nbytes = c_tpfsf(&tpunit, &skip, &err_mode);
if (nbytes != TAPE_SUCC)
    c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
size = MAXBYTES;
nbytes = c_tpread(&tpunit, (unsigned char *)buffer, &size, &err_mode);


/* Determine the tape type
--------------------------*/
if ((sceneid[0] == 'A') || (sceneid[0] == 'a'))
    {
    /* Old ANSI labelled version.  Must be blocked at K32 or K64 */
    strcpy(tapetype, "ANSIOLD");
    if ((nbytes != K32) && (nbytes != K64))
	{
	sprintf(str, "Invalid record size %d on tape %s", nbytes, tapeid);
	c_errmsg(str, "archdr-recsize", &nonfatal);
	c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
	}
    
    /* Check to see if this is a 15 character scene ID */
    if (strlen(sceneid) == 15)
	strcpy(tapetype, "JULIAN");
    }
else if ((sceneid[0] == 'H') || (sceneid[0] == 'h'))
    {
    /* Newest ANSI labelled version.  Header file must be blocked at 80 bytes */
    strcpy(tapetype, "ANSINEW");
    if (nbytes != CEOSBYTES)
	{
	sprintf(str, "Invalid record size %d on tape %s", nbytes, tapeid);
	c_errmsg(str, "archdr-recsize", &nonfatal);
	c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
	}
    }
else if (strcmp(sceneid, ""))
    {
    /* ANSI labelled but this cannot be an AVHRR archive tape */
    sprintf(str, "Invalid file name %s on tape %s", sceneid, tapeid);
    c_errmsg(str, "archdr-fname", &nonfatal);
    c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
    }
else if (nbytes == K64)
    strcpy(tapetype, "EA1");
else if (nbytes == K32)
    strcpy(tapetype, "EA2");
else
    {
    sprintf(str, "Invalid record size %d on tape %s", nbytes, tapeid);
    c_errmsg(str, "archdr-recsize", &nonfatal);
    c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
    }


/* Close and rewind the tape
----------------------------*/
c_tpclos(&tpunit, &err_mode, &rewind, &eofs);


/* Return the tape ID  and tape type to the parameter block
-----------------------------------------------------------*/
string1[0] = tapeid;
string2[0] = tapetype;
if ((q_string(&vblock, "TAPEID",   1, (const char **)string1, P_UPDATE) != SUCCESS) ||
    (q_string(&vblock, "TAPETYPE", 1, (const char **)string2, P_UPDATE) != SUCCESS) ||
    (q_out(&vblock) != SUCCESS))
    {
    sprintf(str, "Error sending parameter to variable block");
    c_errmsg(str, "archdr-par", &nonfatal);
    c_errmsg("Fatal error encountered", "archdr-fatal", &fatal);
    }


/* Complete successfully without logging a completion message
-------------------------------------------------------------*/
z_exit(1, "ARCHDR completed successfully");


/* Put a return here to make the compiler happy
------------------------------------------------*/
return(SUCCESS);

}
