#!/bin/csh
# Process:	archive_avhrr.csh
#
# Input:	$ADAPSARC/sfl/sfl/archive/current_tar
#		scenes in $ADAPSARC/sfl/sfl/
#		processed files lists in $ADAPSARC/sfl/sfl
#
# Output:	$ADAPSLOG/archive_avhrr.log
#		archive/tarfile_YYMMDDhhmmss
#		$ADAPSARC/sfl/sfl/archive/current_tar
#		scenes placed in $ADAPSERR/archive on error
#		scenes placed in /dmf/pdsc/avhrr/satSS/monthMM/dayDD on	success
#
# Returns:	0 on success
#		-1 on failure
#
# Usage:	archive_avhrr.csh
#
# General:
#
#   This process is designed to run from cron, although it may be run at any
#   time.  Processed files lists contain a storage location, or the word error
#   on the first one or two lines followed by a list of files that were
#   archived on the tape.  AVHRR scenes consist of two or three files:
#   sceneid.arch, sceneid.ahdr and sceneid.cksum.  The checksum file will be
#   created by this process and is only created for archive purposes.  The
#   scene ID is: a[hlo]SSMMDDYYhhmmss_hhmmss where:
#
#	SS is satellite number
#	MM is month of acquisition
#	DD is day of acquisition
#	YY is year of acquisition
#	hh is hour of acquisition and time stamp hour
#	mm is minute of acquisition and time stamp minute
#	ss is second of acquisition and time stamp second
#
#   When run this process looks for processed files lists and scenes in the
#   $ADAPSARC/sfl/sfl/ directory.  Metadata will be generated for processed
#   files lists.  Scenes will have a checksum file created and the scene will
#   be placed on nearline storage.  Additionally the scene will be added to a
#   list of files to be archived to tape (tarfiles list).  A running count of
#   the size of the files in the tarfiles list is maintained and, when the LTO
#   limit of 90% of 200Gb is reached, a workorder is created; the tarfile and
#   workorder are copied to the archive system.
#
# Algorithm:
#
#   setup
#   change to ADAPSARC/sfl/sfl directory
#   if archive/current_tar exists
#       get current tarfile and size from archive/current_tar
#   else
#       create it and begin new tarfile
#   endif
#
#   for each processed file in ADAPSARC/sfl/sfl
#       if error is on first or second line of file
#           put processed file in $ADAPSERR/archive
#           print error message
#       else
#           extract tape ID
#           extract list of scenes
#           create pipe delimited metadata containing:
#               |tapeid|sceneid
#           push metadata to production database system
#       endif
#   endfor
#
#   for each scene in ADAPSARC/sfl/sfl
#       create a checksum file
#       copy scene to nearline storage
#       if an error ocurred in transfer
#           delete any partial transferred files on nearline
#           move scene, without cksum, to $ADAPSERR/archive
#           print error message
#       else
#           add scene files to tarfiles list
#           add scene size to current size
#           remove scene
#       endif
#
#       if current size is greater than limit
#           create a workorder
#           copy tarfiles list and workorder to archive system
#           zero current size
#           create new tarfiles list
#       endif
#       save tarfiles list name and current size in archive/current_tar
#   endfor
#
#   exit with success
#
#=============================================================================
#
#-----------------------------------------------------------------------------
# Set up the operating environment.  Since this is designed to be run from
# cron, the LAS environment needs to be explicitly set up.
#-----------------------------------------------------------------------------
if ((! $?EDCSOFT) && (-r /etc/login)) then
    source /etc/login
    eval `$masterlog response comfile las74`
    unset masterlog
    unset response
    unset comfile
endif

#---------------------------
# Set up miscellaneous stuff
#---------------------------
set logfile      = $ADAPSLOG/archive_avhrr.log
set backupdir    = $ADAPSLOG/old/afterloggers/lto_archive
set error        = $ADAPSERR/archive
set currentfile  = $ADAPSARC/sfl/sfl/archive/current_tar
set metasystem   = edclxs53
set metauser     = ops
set metapath     = "/inv/inv/avhrr/data/storloc"
set tarsystem    = edcsns7
#set tardirectory = "/dmf/edc/csb/ops/archive/archive"
#set siloroot     = "/dmf/pdsc/avhrr"
set tardirectory = "/hsm/home/lta/ops/archive/archive"
set siloroot     = "/hsm/lta1/avhrr"

# Maximum size of a cshell integer variable is 2 Gb so tape
# capacity must be in Kb.
# LTO 4 has 800Gb capacity: 90% of 800 Gb = 754974720 Kb
# LTO 3 has 400Gb capacity: 90% of 400 Gb = 377487360 Kb
# LTO 2 has 200Gb capacity: 90% of 200 Gb = 188743680 Kb
# LTO 1 has 100Gb capacity: 90% of 100 Gb =  94371840 Kb
#----------------------------------------------------------
set tapecapacity = 754974720

#----------------------------
# Change to working directory
#----------------------------
if ( ! -e $ADAPSARC/sfl/sfl ) then
    echo -n `date +'%D  %T'` >> $logfile
    echo " Error: $ADAPSARC/sfl/sfl does not exist" >> $logfile
    exit 1
else
    cd $ADAPSARC/sfl/sfl
endif
if ( ! -e archive ) then
    mkdir archive
endif

#-----------------------------------------------------------------------
# Get the current tarfile and size.  If the file does not exist, print a
# warning message that we are starting a new file.
#-----------------------------------------------------------------------
if ( ! -e $currentfile ) then
    echo -n `date +'%D  %T'` >> $logfile
    echo "  Warning: $currentfile does not exist, beginning new tar" >> $logfile
    set datestr = `date +%Y%m%d%H%M%S`
    set tarfile = "archive/tarfile_$datestr"
    touch $tarfile
    set tarsize = 0
    echo "$tarfile-$tarsize" > $currentfile
    echo -n `date +'%D %T'` >> $logfile
    echo "  New tarfile $tarfile started" >> $logfile
else
    set tarfile = `cat $currentfile | awk -F'-' '{print $1}'`
    set tarsize = `cat $currentfile | awk -F'-' '{print $2}'`
    if ( ! -e $tarfile ) then
        echo -n `date +'%D %T'` >> $logfile
	echo "  Fatal Error: Sorry $tarfile does not exist" >> $logfile
	exit 1
    endif
endif

#----------------------------------------------------------------------------
# Process any processed files lists found.  Setting a variable with wildcards
# can easily blow up: argument too long.  It is very unlikely that more than
# one processed files list will ever be present but just in case use a
# temporary file to hold the list.  It's more important in the next loop.
# Note: it is essential that once a file is either processed successfully or
# an error occurs that the file be moved away otherwise an infinite loop will
# occur.  If the file can't be moved away, Terminate!
#----------------------------------------------------------------------------
set procfiles = "true"
while ( "$procfiles" != "" )
    /bin/ls -1 | grep '^processed' > ProcListing
    set procfiles = `head -10 ProcListing`
    rm ProcListing
    foreach procfile ( $procfiles )
        set datestr  = `echo $procfile | awk -F'_' '{print $2}'`
	set metadata = "avhrrmetadata.$datestr"

        # The offsite processed file contains a numeric tape ID and the first
        # three digits are the dateset_id.  Get the dateset_id and test the
        # tape ID
        #--------------------------------------------------------------------
        set tapeID = `head -1 $procfile`
        set dateset_id = `grep DATESET_ID $procfile | awk -F'|' '{print $2}'`
        set first3 = `expr $tapeID : '\(...\)'`
        if ( "$first3" != "$dateset_id" ) then
            echo -n `date +'%D %T'` >> $logfile
            echo "  ERROR $file dataset ID mismatch $first3 vs. $dateset_id" \
                >> $logfile
	    mv $procfile $error/.
            if ( $status ) then
                echo -n `date +'%D %T'` >> $logfile
                echo "  Fatal error, unable to move $procfile away" >> $logfile
                exit -1
            endif
	    continue
	endif
        # Check that the remainder of the characters are numeric
        expr "$tapeID" : "...[0-9][0-9][0-9][0-9][0-9][0-9]" > /dev/null
        if ( $status ) then
            # It's an error condition
	    echo -n `date +'%D %T'` >> $logfile
            echo "  ERROR $procfile contains incorrect tape ID: $tapeID"
                >> $logfile
	    mv $procfile $error/.
            if ( $status ) then
                echo -n `date +'%D %T'` >> $logfile
                echo "  Fatal error, unable to move $procfile away" >> $logfile
                exit -1
            endif
	    continue
	endif
        # Now make sure the tape ID is nine, and only nine, characters long
        set tapeidlen = `expr $tapeID : '.*'`
        if ( $tapeidlen != 9 ) then
            # It's an error condition
            echo -n `date +'%D %T'` >> $logfile
            echo \
                "  ERROR $file contains tape ID of length $tapeidlen: $tapeID" \
                >> $logfile
	    mv $procfile $error/.
            if ( $status ) then
                echo -n `date +'%D %T'` >> $logfile
                echo "  Fatal error, unable to move $procfile away" >> $logfile
                exit -1
            endif
	    continue
	endif

	# Final error check.  See if there is error on the
	# second line of the file.
	#-------------------------------------------------
	set errorstat = `head -n 2 $procfile | tr '[:upper:]' '[:lower:]'`
	set errorstat = `echo $errorstat | awk '{print $2}'`
	if ( "$errorstat" == "error" ) then
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Error detected in $procfile" >> $logfile
	    mv $procfile $error/.
            if ( $status ) then
                echo -n `date +'%D %T'` >> $logfile
                echo "  Fatal error, unable to move $procfile away" >> $logfile
                exit -1
            endif
	    continue
	endif

        # The metadata file needs to have the following:
        # eid|tstamp|mediaid|mediatype|dsid|mdesc|project|pdesccd|pdesc
        # AH11032293101923|3646|002000134|LTO2|002|2259 ACQ SFL|AVHRR|018|AVHRR
        # DATA
        # Each of these values are in the processed file
        #----------------------------------------------------------------------
        set codes = "MEDIA_TYPE DATESET_ID MEDIA_DESCRIPTION 1PROJECT 2PROJECT_DESC_CODE 3PROJECT_DESCRIPTION"
        set metastring = "$tapeID"
        foreach code ( $codes )
            set item = `grep $code $procfile | awk -F'|' '{print $2}'`
            set metastring = "${metastring}|${item}"
        end


	# There was no error and the tape ID was present.  Create the metadata
	# file by stripping the 16-digit sceneid and timestamp from the file
	# and add the Tape ID to each line.  We only need one of the file
	# names of the three comprising a scene -- here we use the arch file.
	#---------------------------------------------------------------------
	awk /arch/'{ \
	    N = split($1,array,"/") \
	    L = substr(toupper(array[N]),1,16) \
	    M = substr(array[N],18,6) \
	    printf "%s|%s|%s\n", L,M,META \
	    }' META="$metastring" $procfile > $metadata
	
	# Copy the metadata to production database system
	#------------------------------------------------
	scp $metadata ${metauser}@${metasystem}:$metapath/$metadata
	if ( $status ) then
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Error transferring $metadata to $metasystem" >>$logfile
	    mv $metadata $error/.
            if ( $status ) then
                echo -n `date +'%D %T'` >> $logfile
                echo "  Fatal error, unable to move $procfile away" >> $logfile
                exit -1
            endif
	else
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Successfully transferred $metadata" >> $logfile
	    ssh ${metauser}@${metasystem} chmod 666 $metapath/$metadata
	    if ( $status ) then
	        echo -n `date +'%D %T'` >> $logfile
		echo "  Error running chmod for $metadata on $metasystem" \
		    >> $logfile
	    endif
	endif

	# Move the processed files and metadata files to the ADAPSLOG
	# directory for temporary safekeeping.  These will be cleaned up
	# eventually by ops.
	#---------------------------------------------------------------
	mv $procfile $backupdir/.
	if ( $status ) then
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Fatal error, unable to move $procfile away" >> $logfile
	    exit -1
	endif
	# Checking the status of this isn't important since failure won't
	# cause problems with the looping structure.
	#----------------------------------------------------------------
	mv $metadata $backupdir/.

	# Ops and RDM wanted a mail message so...
	#----------------------------------------
	echo -n `date +'%D %T'` > arch_mailfile
	echo " Created AVHRR archive tape number: $tapeID" >> arch_mailfile
	Mail -s "Created AVHRR LTO archive" goodale@usgs.gov < arch_mailfile
	rm arch_mailfile

    end # foreach
end # while

#---------------------------------------------------------------------------
# Process any scenes found.  In this case there can easily be 50 scenes or
# more to process.  The technique for doing a subset of the scenes will also
# be done here.
#---------------------------------------------------------------------------
set archfiles = "true"
while ( "$archfiles" != "" )
    /bin/ls -1 | grep '^a.*.arch$' > ArchListing
    set archfiles = `head -10 ArchListing`
    rm ArchListing
    foreach archfile ( $archfiles )
	# Set up the associated file names
	#---------------------------------
        set sceneid   = "$archfile:r"
	set ahdrfile  = "$sceneid.ahdr"
	set cksumfile = "$sceneid.cksum"

	# Create checksum.  Checksum consists of two lines which are the
	# output of cksum.  First line is for the .arch, second the .ahdr
	#----------------------------------------------------------------
	cksum $archfile > $cksumfile
	if ( $status ) then
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Error creating check sum for $sceneid" >> $logfile
	    rm -f $cksumfile
	    mv $sceneid* $error/.
	    if ( $status ) then
	        echo -n `date +'%D %T'` >> $logfile
		echo "  Fatal error: could not move $sceneid away" >> $logfile
		exit -1
	    endif
	    continue
	endif
	cksum $ahdrfile >> $cksumfile
	if ( $status ) then
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Error creating check sum for $sceneid" >> $logfile
	    rm -f $cksumfile
	    mv $sceneid* $error/.
	    if ( $status ) then
	        echo -n `date +'%D %T'` >> $logfile
		echo "  Fatal error: could not move $sceneid away" >> $logfile
		exit -1
	    endif
	    continue
	endif

	# Transfer scene to near line storage.  The path is
	# /dmf/pdsc/avhrr/satSS/monthMM/dayDD
	#--------------------------------------------------
	set sat      = `echo $sceneid | awk '{print substr($1,3,2)}'`
	set month    = `echo $sceneid | awk '{print substr($1,5,2)}'`
	set day      = `echo $sceneid | awk '{print substr($1,7,2)}'`
	set datapath = "$siloroot/sat$sat/month$month/day$day"

	# At this point the migration effort checked for existance of the file
	# on silo.  What do we want to do here?
	#---------------------------------------------------------------------

	# Transfer the scene.  If one file does not transfer the scene
	# transfer is not complete and it is moved to error.  We don't
	# want partial scenes on the silo so remove the files that
	# successfully transferred.
	#-------------------------------------------------------------
	scp $archfile edcsns7:$datapath/.
	scp $ahdrfile edcsns7:$datapath/.
	scp $cksumfile edcsns7:$datapath/.

	# Check to see if the files actually transferred by comparing
	# the local and remote sizes.  Any differences indicate a
	# questionable transfer and will be flagged as an error.
	#------------------------------------------------------------
	set temp = `ssh edcsns7 /bin/ls -l $datapath/$archfile`
	set siloarchsize = `echo $temp | awk -F' ' '{print $5}'`
	set temp = `ssh edcsns7 /bin/ls -l $datapath/$ahdrfile`
	set siloahdrsize = `echo $temp | awk -F' ' '{print $5}'`
	set temp = `ssh edcsns7 /bin/ls -l $datapath/$cksumfile`
	set silocksumsize = `echo $temp | awk -F' ' '{print $5}'`
	set temp = `/bin/ls -l $archfile`
	set localarchsize = `echo $temp | awk -F' ' '{print $5}'`
	set temp = `/bin/ls -l $ahdrfile`
	set localahdrsize = `echo $temp | awk -F' ' '{print $5}'`
	set temp = `/bin/ls -l $cksumfile`
	set localcksumsize = `echo $temp | awk -F' ' '{print $5}'`
	if ( $siloarchsize != $localarchsize || \
	     $siloahdrsize != $localahdrsize || \
	     $silocksumsize != $localcksumsize ) then
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Error transferring scene: $sceneid moved to error" \
		>> $logfile
	    rm $cksumfile
            mv $sceneid* $error/.
	    if ( $status ) then
	        echo -n `date +'%D %T'` >> $logfile
		echo "  Fatal error: could not move $sceneid away" >> $logfile
		exit -1
	    endif

	    # Delete the scene from the silo
	    #-------------------------------
	    ssh opsavhrr@edcsns7 rm -f "$datapath/${sceneid}*"
	    continue
	endif

	# The scene can now be safely removed from the archive directory
	#---------------------------------------------------------------
	rm -f $sceneid.*
	if ( $status ) then
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Fatal error: could not move $sceneid away" >> $logfile
	    exit -1
	endif

	# Issue a completion message for the scene
	#-----------------------------------------
	echo -n `date +'%D %T'` >> $logfile
	echo "  $sceneid processed" >> $logfile

	# Add the scene to the tarfile.  If we got here all the
	# preceding operations were successful and the complete scene
	# resides on the silo.  Reduce the datapath to the avhrr
	# directory.  This will be the root directory of the tar.
	#------------------------------------------------------------
	set datapath = "sat$sat/month$month/day$day"
	echo "$datapath/$archfile" >> $tarfile
	echo "$datapath/$ahdrfile" >> $tarfile
	echo "$datapath/$cksumfile" >> $tarfile

	# And add the scene size to tarsize
	#----------------------------------
	@ filesize = $localarchsize
	@ filesize += $localahdrsize
	@ filesize += $localcksumsize
	# Tape capacity is in Kb since it is greater than 2Gb
	@ tarsize += $filesize / 1024
	if ( $filesize % 1024 >= 512 ) then
	    @ tarsize += 1
	endif

	# See if it's time to tar.  If it is, copy
	# the tarlist to the tar system and execute
	# the tar process using tarlist as input
	#------------------------------------------
	if ( $tarsize > $tapecapacity ) then

            echo -n `date +'%D %T'` >> $logfile
	    echo "  Starting tar process with $tarfile" >> $logfile

	    # Create a workorder -- an attempt to give some priority to tar
	    # lists that have been in the works for some time, give the work
	    # order the same time stamp as the tar list.
	    #---------------------------------------------------------------
	    set tarfilename = "$tarfile:t"
	    set tstamp = `echo $tarfilename | awk -F'_' '{print $2}'`
	    set wo = "workorder_$tstamp"
	    #echo "edclxs1|$ADAPSARC/sfl/sfl|avhrr_offsite|$siloroot|$tarfilename" > $wo
	    echo "edclxs53|/home/ops/archive/avhrr|avhrr_offsite|$siloroot|$tarfilename" > $wo

	    # Copy the tarfile to the tarsystem
	    #----------------------------------
	    scp $tarfile ops@${tarsystem}:$tardirectory/.
	    if ( $status ) then
	        echo -n `date +'%D %T'` >> $logfile
		echo "  Error copying $tarfile to $tarsystem" >> $logfile
		echo "  scp $tarfile ${tarsystem}:$tardirectory/." \
		    >> $logfile
		mv $wo $tarfile $error/.
	    else
	        # If transfer was successful copy workorder to the tarsystem
		#-----------------------------------------------------------
		scp $wo ops@${tarsystem}:$tardirectory/.
		if ( $status ) then
		    echo -n `date +'%D %T'` >> $logfile
		    echo "  Error copying $wo to $tarsystem" >> $logfile
		    mv $wo $tarfile $error/.
		else
		    mv $wo $tarfile $backupdir/.
		endif
	    endif

	    # Start a new tarfile and zero tarsize
	    #-------------------------------------
	    set tarsize = 0
	    set datestr = `date +%Y%m%d%H%M%S`
	    set tarfile = "archive/tarfile_$datestr"
	    touch $tarfile
	    echo -n `date +'%D %T'` >> $logfile
	    echo "  Starting new file: $tarfile" >> $logfile
	endif

	# Save the current tarfile and new tarsize
	#-----------------------------------------
	cp $currentfile $currentfile.bak
	echo "$tarfile-$tarsize" > $currentfile

    end # foreach
end # while

exit 0
