/*******************************************************************************
NAME:		ATMOCOR

PURPOSE:	Copies an image into another image after having applied
		atmospheric corrections.  It allows the input image to be
		subwindowed and subbanded.  The input image may be any data
		type.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/92  T.Baltzer        Development
  2.0    04/94  D.Hollaren       Convert ADAPS 2.0/LAS 6.0
  3.0    07/94  J.Fenno          Completed conversion and released.
  4.0    12/94  T.Baltzer        Modified to conform to new scaling.
				 During the scaling mods it was realized
				 that the ability to correct only one band 
				 is only partially implemented.  For now
				 require 2 bands to be input, assuming these
				 are AVHRR band 1 and band 2 in that order!
				 correct() is the worst offender, assuming
				 2 bands are always being corrected.
  4.1    02/95  J.Fenno          Remove the global 16bit scaling option.  The
				 users felt this was confusing.  We the global
				 scaling option is specified and the data type
				 is i*2, we will hard code the scaling options
				 to AVHRR_10BIT.
  4.2    01/97  T.Baltzer        Modified to be able to process NOAA-14 data.
  5.0    11/00  D.Lloyd		 Modified so that the input file specification
  				 can be "image1 + image2" for AVHRR input bands
				 and angles bands.  The AVHRR bands may be
				 input in any order and an input parameter
				 BANDSEQ specifies the order.  CR 3384
				 Rewrote the image I/O to use standard LAS
				 sequential I/O.  Corrected an error where the
				 output DDR was not being updated with the
				 data source properly.  Cleaned up compile
				 time warnings.
  5.2    01/01  T.Baltzer        Modified to be able to process NOAA-16 data.
  5.3    09/02  D.Lloyd          Modified to be able to process NOAA-17 data.
 

PROJECT:	ADAPS

ALGORITHM REFERENCES:
    Tanre, D., Deroo, C., Duhaut, P., et al. (1986), Simulation
    of the Satellite Signal in the Solar Spectrum (5S), User's
    Guide. Laboratoire d'Optique Atmospherique, Universite des
    Sciences et Techniques de Lille, 58655 Villeneuve d'Ascq
    Cedex, France, 343 pp.

    Teillet, P.M. (1992) "An Algorithm for the Radiometric and
    Atmospheric Correction of AVHRR Data in the Solar Reflective
    Channels", Remote Sensing of the Environment. 41:185-195.

    Kidwell, Katherine B. "NOAA Polar Orbiter Data Users Guide"

*******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "adaps.h"
#include "atmocor.h"	


/* The following variables have been declared global for image I/O routines
-----------------------------------------------------------------------------*/
extern long curcat;             /* current processing band                   */
extern long curent;             /* current processing line                   */



void main( void )
{
struct FDESC *fdesc[NUM_IMGS] = {NULL};	/* File information		*/
struct GDESC *gdesc = NULL;	/* Greoup file descriptor		*/
struct PARBLK vblock;	/* pointer to TAE parameter block		*/
struct SCOFF scoff[5];	/* Scale and offset values for:			*/
			/*   1: Satellite zenith angles			*/
			/*   2: Solar zenith angles			*/
			/*   3: Relative azimuth angles			*/
			/*   4: Input image reflectance or radiance	*/
			/*   5: Output image reflectance		*/

char angle_scaling[CMLEN];	/* angle scaling type			*/
char hosin[NUM_IMGS][CMLEN] = {0};/* Host input image file name		*/
char hosout[1][CMLEN];		/* Host output image file name		*/
char input_scaling[CMLEN];	/* input scaling type			*/
char output_scaling[CMLEN];	/* output scaling type			*/
char taeout[1][CMLEN];		/* TAE output image file name		*/

double dsquared;		/* Sun-earth distance			*/
double maxval[MAXBANDS];	/* max pixel value array		*/
double minval[MAXBANDS];	/* min pixel value array		*/

int  band_sequence[MAXVISBAND] = {0}; /* sequence of AVHRR channels	*/

long  nbands[MAXBANDS];		/* number of bands specified in each image */
long bands[MAXBANDS][MAXVISBAND+1] = {0};/* image band specifications	*/
long maxbnds = MAXBANDS;	/* Maximum number of bands		*/
long mmflag;			/* Calculate min/max flag		*/
long msg_flag	= ALLCAT;	/* flag for porcessing message		*/
long msgtim;			/* elapsed time between display messages*/
long odtype;			/* output image data type		*/
long offset[MAXBANDS*MAXVISBAND] = {0};/* offsets into thebuf		*/
long one	= 1;		/* value of one				*/
long radopt;			/* Input radiance option (RADIANCE or ALBEDO)*/
long satnum;			/* Satellite number			*/
long std_out;			/* LUN to standard output    		*/
long total_bands;		/* Total bands				*/
long window[NUM_IMGS][4];	/* input image window specification	*/

unsigned char *thebuf = NULL;	/* I/O data buffer			*/


/* Initialize LAS
-----------------*/
c_inlas(&vblock,&std_out);


/* Get parameters and globals
-----------------------------*/
if ( getpar(&vblock, hosin, taeout, hosout, window, bands, nbands, &satnum,
        &radopt, &dsquared, &msgtim, &mmflag, input_scaling, angle_scaling,
	output_scaling, &odtype, band_sequence) == E_FAIL )
    c_lasmsg(TRUE,"atmocor-fatal","Fatal error encountered");


/* Load the A, B, and S look-up tables
--------------------------------------*/
if (loadcoef(satnum, radopt) != E_SUCC)
    c_clean(&vblock, "atmocor", (const char (*)[CMLEN])hosout, &one);


/* Open the images
------------------*/
if ( open_las( &gdesc, fdesc, hosin, hosout, odtype, bands, nbands, offset,
	&thebuf, window[0] ) != E_SUCC )
    c_clean(&vblock, "atmocor", (const char (*)[CMLEN])hosout, &one);


/* Get the input and output scaling and load structure
------------------------------------------------------*/
if ( get_scale( hosin, radopt, input_scaling, angle_scaling, output_scaling,
	odtype, scoff ) != E_SUCC )
    c_clean(&vblock, "atmocor", (const char (*)[CMLEN])hosout, &one);


/* Initialize min/max arrays.  The call to c_inmnmx() can be made
since unsigned data types are no longer supported in ATMOCOR.
-----------------------------------------------------------------*/
if ( mmflag )
   c_inmnmx( &odtype, &maxbnds, minval, maxval );


/* Setup for processing message utility
---------------------------------------*/
c_inittm( "ATMOCOR", "Processed", "line", "bands", " ", &window[0][NL],
	 &nbands[IMG_OUTPUT], &msgtim, &msg_flag );


/* Process the images
---------------------*/
if ( process( gdesc, thebuf, offset, window[0], minval, maxval, mmflag,
#ifdef QUICKCOR_COMPARE
	radopt,
#endif
	dsquared, scoff, band_sequence) != E_SUCC )
    c_clean( &vblock, "atmocor", (const char (*)[CMLEN])hosout, &one );


/* Update the DDR
-----------------*/
#ifdef DEBUG_IMG
    total_bands = MAXBANDS;
#else
    total_bands = 2;
#endif
update_ddr( &vblock, hosin, hosout[0], window, &total_bands );


/* Save the min/max values
--------------------------*/
if ( mmflag )
    c_svmnmx( hosout[0], &nbands[IMG_OUTPUT], minval, maxval, "YES" );


/* Update history file 
----------------------*/
if ( c_hstout( &vblock, hosout[0], (const char (*)[CMLEN])hosin, &one, " " )
        != E_SUCC )
   c_lasmsg(FALSE,"atmocor-nonfatal","Nonfatal error encountered");


/* Close the image files
------------------------*/
c_egclse( &gdesc );


/* Output completion message 
----------------------------*/
c_complt( "ATMOCOR", taeout, &one );
} 
