/* Debug constants.

    - If DEBUG_IMG is defined the output image will contain 16 bands of
      information: input ch1, input ch2, Acoef ch1, Acoef ch2, Bcoef ch1, Bcoef
      ch2, Scoef ch1, Scoef ch2, elevation, satellite zenith, solar zenith,
      relative azimuth, ozone, water vapor, corrected ch1, and corrected ch2.
      EXTREMELY much disk space required!!!
   
    - If QUICKCOR_COMPARE is defined a file called atmocor_table.PID will be
      opened for writing (no check is done for existence).  This will print
      the input radiance values, all atmospheric coefficients, view angles,
      quickcor corrected reflectance values, and atmocor corrected reflectance
      values.  Quickcor does not use the sun-to-earth distance in its
      computations.  This will result in a minor difference, which increases as
      the input radiance increases.  When this option is enabled, you must run
      in the directory where the quickcor function and its look up tables are
      located.  EXTREMELY much slow!!!
define QUICKCOR_COMPARE
define DEBUG_IMG
------------------------------------------------------------------------------*/


/* Image constants
Rearranged these defines and added MAXVISBAND CR 3384
------------------------------------------------------------------------------*/
#define MAXVISBAND      2         /* maximum number of visual bands allowable */
#define IMG_DEM		0         /* Digital Elevation Model image            */
#define IMG_OZONE	1         /* Ozone image                              */
#define IMG_WATER	2         /* Water vapor image                        */
#define IMG_ANGLES	3         /* View angles image, offset to image one   */
				  /* there can be three images specified      */
#define IMG_INPUT	6         /* Offset to input image one, there can be  */
				  /* a total of MAXVISBAND input images       */
#define IMG_OUTPUT	9         /* Output image                             */
#define NUM_IMGS	10        /* Total number of possible images          */

#define NBANDS		2         /* Number of output bands                   */
#define MAXBANDS       19         /* Number of output bands in debug image    */

#define NCOEFF		9         /* Number of coefficients for atmocor       */

#define OFF_DEM		0
#define OFF_OZONE	1
#define OFF_WATER	2
#define OFF_SATZEN	3
#define OFF_SOLZEN	4
#define OFF_RELAZ	5
#define OFF_INCHAN	6

/* Structure containing the necessary information for the input/ouput images
struct FDESC
    {
    int  fd;
    long access;
    long dtype;
    long nlines;
    long nsamples;
    long nbands;
    long bands[MAXBND];
    long scaling;
    char host[CMLEN];
    unsigned char  *ptr1;
    short	   *ptr2;
    long 	   *ptr3;
    float	   *ptr4;
    };
------------------------------------------------------------------------------*/


#define IN_REFLECT 3	/* Index to input image reflectance 		*/
#define OUT_REFLECT 4	/* Index to output image reflectance 		*/


/* Structure used for storing scaling, offset quantities */
/*-------------------------------------------------------*/
struct SCOFF
   {
   long   scaling;
   long   odtype;
   double minval;               /* Minimum value of appropriate quantity */
   double maxval;               /* Maximum value of appropriate quantity */
   double scale;                /* Scale value to apply                  */
   double offset;               /* Offset value to apply                 */
   double unscale;              /* Value to apply to unscale             */
   };

/*****************************************************************
	FUNCTION prototypes
*****************************************************************/
#ifndef ATMOCOR_H
#define ATMOCOR_H
/* alloc_las.c */
int alloc_las
(
    struct FDESC  *fdesc[]
);

/* close_las.c */
int close_las
(
    struct FDESC *fdesc[]
);

/* correct.c */
int loadcoef 
(
    long satnum, 
    long radiance
);
int correct 
(
    double chans[], 
    double angs[], 
    double elev, 
    double *ozone, 
    double water, 
    double sun_earth, 
    long   validflg[], 
    double corrected[], 
    double outcoef[]
#ifdef QUICKCOR_COMPARE
    , double qcor[2]
#endif
);

/* free_las.c */
int free_las
(
    unsigned char *in_buffer, 
    unsigned char *in_sbuf, 
    unsigned char *ang_buffer, 
    unsigned char *ang_sbuf, 
    unsigned char *dem_buffer, 
    unsigned char *oz_buffer, 
    unsigned char *wv_buffer, 
    unsigned char *out_buffer, 
    long	ozone_img, 
    long	water_img
);

/* get_scale.c */
int get_scale
(
    char hosin[][CMLEN],
    long radopt, 
    char input_scaling[],
    char angle_scaling[],
    char output_scaling[],
    long odtype,
    struct SCOFF scoff[]
);

/* getpar.c */
int getpar 
    (
    struct PARBLK *vblock,      /* IN pointer to parameter block        */
    char   hosin[][CMLEN],      /* OUT host input image file names      */
    char   taeout[][CMLEN],     /* OUT tae output file name             */
    char   hosout[][CMLEN],     /* OUT host output file name            */
    long   window[][WINDOW],    /* OUT input window specification       */
    long   bands[][MAXVISBAND+1], /* OUT image band specifications      */
    long   nbands[],            /* OUT number of bands in each image    */
    long   *satnum,             /* OUT satelite number                  */
    long   *radopt,             /* OUT flag (T/F) input data radiance   */
    double *dsquared,           /* OUT sun-earth distance squared       */
    long   *msgtim,             /* OUT time between display messages    */
    long   *mmflag,             /* OUT flag (T/F) calculate min/max     */
    char   *input_scaling,      /* OUT input scaling                    */
    char   *angle_scaling,      /* OUT angles scaling                   */
    char   *output_scaling,     /* OUT output scaling                   */
    long   *odtype,             /* OUT output data type                 */
    int    *band_sequence       /* OUT band sequence of input image(s)  */
    );

/* line_las.c */
int line_las 
(
    struct FDESC fdesc[], 
    long line
);

/* load_bnds.c */
int load_bnds 
(
    struct FDESC fdesc[], 
    long sample, 
    double chans[], 
    double angs[], 
    double *dem, 
    double *ozone, 
    double *water,
    int    *band_sequence
);

/* open_las.c */
int open_las 
(
    struct GDESC  **gdesc,
    struct FDESC  *fdesc[],
    char        hosin[][CMLEN], 
    char	hosout[][CMLEN], 
    long	odtype,
    long        bands[][MAXVISBAND+1],
    long	nbands[],
    long        offset[],
    unsigned char **thebuf,
    long	window[]
);

/* process.c */
int process 
(
    struct GDESC  *gdesc, 
    unsigned char *thebuf,
    long offseg[],
    long window[], 
    double minval[], 
    double maxval[], 
    long mmflag, 
#ifdef QUICKCOR_COMPARE
    long radopt, 
#endif
    double dsquared,
    struct SCOFF scoff[],
    int  *band_sequence
);

/* quickcor.c */
int quickcor 
(
    double chans[], 
    double angs[], 
    double elev, 
    double ozone, 
    double sun_earth, 
    double qcor[]
);

/* scale.c */
int scale 
(
    long totalbands,
    double atmo[], 
    long validflg[], 
    struct SCOFF scoff[], 
    long mmflag, 
    double minval[], 
    double maxval[]
);

/* update_ddr.c */
int update_ddr 
(
    struct PARBLK *vblock, 
    char hosin[][CMLEN], 
    char hosout[], 
    long windows[][4], 
    long *total_bands
);
#endif
