/*******************************************************************************
FUNCTIONS:	loadcoef() correct()

PURPOSE:	These algorithms are based upon the article "Radiometric and
		Atmospheric Correction Procedures for AVHRR Preprocessing in
		the Solar Reflective Channels" by:
		
			P.M. Teillet
			Canada Centre for Remote Sensing
			2464 Sheffield Road
			Ottawa, Ontario, Canada K1A 0Y7

		loadcoef()

			This function loads a set of static arrays from Look
			Up Tables (LUT) in ADAPSTABLES.  These arrays are used
			by correct() to retrieve the appropriate atmospheric
			correction coefficients for a specified pixel.
		    
		correct()

			This function will retrieve the necessary coefficients
			from the static arrays and apply them to AVHRR channels
			1 and 2 to correct these values for atmospheric effects.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    07/94  D.Meyer/J.Fenno  Implementation
  2.0    14/99  D.Lloyd		 Modified for water vapor correction and
  				 new ozone LUT layout.  Replaced hard-coded
				 Kvals with function call.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "adapssup.h"
#include "atmocor.h"


#define NUM_CHANS   2	/* # of AVHRR channels to atmospherically correct     */
#define SOLZEN_PTS 12	/* # of solar zenith angles in the LUTs               */
#define SATZEN_PTS 12	/* # of satellite zenith angles in the LUTs           */
#define RELAZ_PTS  18	/* # of relative azimuth angles in the LUTs           */
#define ELEV_PTS    5	/* # of elevation points in the LUTs                  */
#define OZONE_ANG  16	/* # of ozone angles (solzen + satzen) / 10           */
#define OZONE_CON  46	/* # of ozone concentration indices                   */
#define WATVAP_CON 60	/* # of water vapor concentration indices	      */

#define AVG_OZONE 0.319	/* Average ozone over the earth                       */



/* Look Up Tables for A, B, S, O, and K coefficients.  The tables are loaded
   from files in ADAPSTABLES or the static array of constants, k_values.
------------------------------------------------------------------------------*/
static short luts_not_loaded = TRUE;
static double a_lut[NUM_CHANS][SOLZEN_PTS][SATZEN_PTS][ELEV_PTS];
static double b_lut[NUM_CHANS][SOLZEN_PTS][SATZEN_PTS][RELAZ_PTS][ELEV_PTS];
static double s_lut[NUM_CHANS][ELEV_PTS];
static double o_lut[OZONE_CON][SOLZEN_PTS][SATZEN_PTS];
static double w_lut[NUM_CHANS][WATVAP_CON][SOLZEN_PTS][SATZEN_PTS];
static double k_lut[2];




/* The following static arrays are the grid points that are stored in the
   A and B coefficient LUTs in ADAPSTABLES.  The view angles and elevation
   of each individual pixel are looked up in these arrays to determine the
   appropriate indices in the LUTs.
------------------------------------------------------------------------------*/
static double solzen_gridpts[SOLZEN_PTS] =
    {0.0, 15.0, 30.0, 40.0, 50.0, 58.0, 64.0, 69.0, 73.0, 76.0, 78.0, 80.0};

static double satzen_gridpts[SATZEN_PTS] =
    {0.0, 10.0, 20.0, 30.0, 40.0, 46.0, 51.0, 56.0, 60.0, 64.0, 67.0, 70.0};

static double relaz_gridpts[RELAZ_PTS] =
    {  0.0,  11.0,  23.0,  37.0,  53.0,  67.0,  79.0,  90.0, 100.0,
     110.0, 120.0, 130.0, 140.0, 150.0, 160.0, 168.0, 174.0, 180.0};

static double elev_gridpts[ELEV_PTS] =
    {0.0, 0.5, 1.1, 1.8, 2.6};

static double water_grid[3] = { 0.1, 6.0, 0.1 };
static double ozone_grid[3] = { 0.05, 0.5, 0.01 };



/* The following static array is the Exoatmospheric Solar Flux, filtered
   through AVHRR channels 1 and 2 (NOAA-11).  Units are W/m^2/band.  This is
   not used currently, but may be of use in the future.
static double exo_solar_flux[NUM_CHANS] =
    {1629.2, 1052.8};
------------------------------------------------------------------------------*/





/*############################################################################*/
int loadcoef
(
    long satnum,	/* IN - NOAA satellite number (6-11, 14, 16, 17)      */
    long radiance	/* IN - Pixels in radiance (nonzero) or albedo (zero) */
)
{
char adapstables[CMLEN];/* The ADAPSTABLES directory                          */
char hostname[CMLEN];	/* Host name of the Look Up Tables                    */
char *ptr;		/* Ptr to a character string                          */
char str[CMLEN];	/* Temporary working string                           */

long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long k;			/* Loop control                                       */
long l;			/* Loop control                                       */
long m;			/* Loop control                                       */
long nonfatal	= 1;	/* Nonfatal error flag                                */

FILE *fp;		/* File ptr for opening the LUTs                      */



/* Ensure the satellite number is valid
---------------------------------------*/
if ((satnum < 6) || ((satnum > 11) && (satnum != 14) &&
    (satnum != 16) && (satnum != 17) && (satnum != 18)))
    {
    c_lasmsg(FALSE,"atmocor-sat","No atmospheric correction for NOAA %d",
	     satnum);
    return(E_FAIL);
    }


/* Retrieve the directory for ADAPSTABLES
-----------------------------------------*/
ptr = getenv("ADAPSTABLES");
if (ptr == NULL)
    {
    c_errmsg("Error retrieving ADAPSTABLES", "atmocor-env", &nonfatal);
    return(E_FAIL);
    }
strcpy(adapstables, ptr);


/* Create the name of the A coefficient LUT file, open it,
   skip the header portion of the file, and read the LUT.
----------------------------------------------------------*/
sprintf(hostname, "%s/acoef_n%02d.lut", adapstables, satnum);
fp = fopen(hostname, "r");
if (fp == NULL)
    {
    c_errmsg("Error opening A LUT file", "atmocor-open", &nonfatal);
    return(E_FAIL);
    }
fgets(str, CMLEN, fp);
sscanf(str, "%d", &i);
for (; i > 1; i--)
    fgets(str, CMLEN, fp);
for (i = 0; i < NUM_CHANS; i++)
    for (j = 0; j < SOLZEN_PTS; j++)
	for (k = 0; k < SATZEN_PTS; k++)
	    for (l = 0; l < ELEV_PTS; l++)
		if (fscanf(fp, "%lf", &a_lut[i][j][k][l]) != 1)
		    {
		    fclose(fp);
		    c_errmsg("Error reading A LUT", "atmocor-read", &nonfatal);
		    return(E_FAIL);
		    }
fclose(fp);


/* Create the name of the B coefficient LUT file, open it,
   skip the header portion of the file, and read the LUT.
----------------------------------------------------------*/
sprintf(hostname, "%s/bcoef_n%02d.lut", adapstables, satnum);
fp = fopen(hostname, "r");
if (fp == NULL)
    {
    c_errmsg("Error opening B LUT file", "atmocor-open", &nonfatal);
    return(E_FAIL);
    }
fgets(str, CMLEN, fp);
sscanf(str, "%d", &i);
for (; i > 1; i--)
    fgets(str, CMLEN, fp);
for (i = 0; i < NUM_CHANS; i++)
    for (j = 0; j < SOLZEN_PTS; j++)
	for (k = 0; k < SATZEN_PTS; k++)
	    for (l = 0; l < RELAZ_PTS; l++)
		for (m = 0; m < ELEV_PTS; m++)
		    if (fscanf(fp, "%lf", &b_lut[i][j][k][l][m]) != 1)
			{
			fclose(fp);
			c_errmsg("Error reading B LUT", "atmocor-read",
			    &nonfatal);
			return(E_FAIL);
			}
fclose(fp);


/* Create the name of the S coefficient LUT file, open it,
   skip the header portion of the file, and read the LUT.
----------------------------------------------------------*/
sprintf(hostname, "%s/scoef_n%02d.lut", adapstables, satnum);
fp = fopen(hostname, "r");
if (fp == NULL)
    {
    c_errmsg("Error opening S LUT file", "atmocor-open", &nonfatal);
    return(E_FAIL);
    }
fgets(str, CMLEN, fp);
sscanf(str, "%d", &i);
for (; i > 1; i--)
    fgets(str, CMLEN, fp);
for (i = 0; i < NUM_CHANS; i++)
    for (j = 0; j < ELEV_PTS; j++)
	if (fscanf(fp, "%lf", &s_lut[i][j]) != 1)
	    {
	    fclose(fp);
	    c_errmsg("Error reading S LUT", "atmocor-read", &nonfatal);
	    return(E_FAIL);
	    }
fclose(fp);


/* Create the name of the ozone LUT file, open it, skip
   the header portion of the file, and read the LUT.
-------------------------------------------------------*/
sprintf(hostname, "%s/ozone_n%02d.lut", adapstables, satnum);
fp = fopen(hostname, "r");
if (fp == NULL)
    {
    c_errmsg("Error opening ozone LUT file", "atmocor-open", &nonfatal);
    return(E_FAIL);
    }
fgets(str, CMLEN, fp);
sscanf(str, "%d", &i);
for (; i > 1; i--)
    fgets(str, CMLEN, fp);
for (i = 0; i < OZONE_CON; i++)
    for (j = 0; j < SOLZEN_PTS; j++)
	for (k = 0; k < SATZEN_PTS; k++)
	    if (fscanf(fp, "%lf", &o_lut[i][j][k]) != 1)
		{
		fclose(fp);
		c_errmsg("Error reading ozone LUT", "atmocor-read",
			&nonfatal);
		return(E_FAIL);
		}
fclose(fp);


/* Create the name of the water vapor LUT file, open it, skip
   the header portion of the file, and read the LUT.
-------------------------------------------------------------*/
sprintf(hostname, "%s/watvap_n%02d.lut", adapstables, satnum);
fp = fopen(hostname, "r");
if (fp == NULL)
    {
    c_errmsg("Error opening water vapor LUT file", "atmocor-open", &nonfatal);
    return(E_FAIL);
    }
fgets(str, CMLEN, fp);
sscanf(str, "%d", &i);
for (; i > 1; i--)
    fgets(str, CMLEN, fp);
for (i = 0; i < NUM_CHANS; i++)
    for (j = 0; j < WATVAP_CON; j++)
	for (k = 0; k < SOLZEN_PTS; k++)
	    for (l = 0; l < SATZEN_PTS; l++)
		if (fscanf(fp, "%lf", &w_lut[i][j][k][l]) != 1)
		    {
		    fclose(fp);
		    c_errmsg("Error reading water vapor LUT", "atmocor-read",
			    &nonfatal);
		    return(E_FAIL);
		    }
fclose(fp);


/* Load the correct constants for converting radiance to albedo
---------------------------------------------------------------*/
if (radiance)
    {
    k_lut[0] = 1.0;   /* No conversion necessary */
    k_lut[1] = 1.0;
    }
else
    {
    c_getkvals( satnum, "prelaunch", k_lut );
    k_lut[0] = 1.0 / k_lut[0];
    k_lut[1] = 1.0 / k_lut[1];
    }


/* Reset the flag to indicate the LUTs have been loaded
-------------------------------------------------------*/
luts_not_loaded = FALSE;
return(E_SUCC);
}





/*############################################################################*/
int correct
(
    double chans[],		/* IN  - AVHRR channel 1 and 2 values         */
    double angs[],		/* IN  - View angles for this pixel           */
    double elev,		/* IN  - Elevation (kilometers)               */
    double *ozone,		/* IN  - Ozone value                          */
    double water,		/* IN  - Water vapor                          */
    double sun_earth,		/* IN  - (Sun-Earth dist)^2 astronomical units*/
    long   validflg[],		/* IN  - Valid data flag (NOT mask value)     */
    double corrected[],		/* OUT - Atmospherically corrected chans[]    */
    double outcoef[]		/* OUT - Coefficients used to correct input   */
#ifdef QUICKCOR_COMPARE
    , double qcor[2]		/* OUT - Quickcor results (returned only if   */
				/*       QUICKCOR_COMPARE is defined          */
#endif
)
{
static double Tg;		/* Ozone gaseous transmittance                */
static double Tw[NUM_CHANS];	/* Water vapor transmittance		      */
static double Y;		/* Temporary results for correction           */
static double abs_angs[3];	/* Absolute angle values                      */
static double acoef[NUM_CHANS];	/* A coefficients for AVHRR channels 1 and 2  */
static double bcoef[NUM_CHANS];	/* B coefficients for AVHRR channels 1 and 2  */
static double elev_wf;		/* Elevation weighting factor                 */
static double relaz_wf;		/* Elevation weighting factor                 */
static double satzen_wf;	/* Elevation weighting factor                 */
static double water_wf;		/* water vapor weighting factor               */
static double ozone_wf;		/* ozone weighting factor		      */
static double scoef[NUM_CHANS];	/* S coefficients for AVHRR channels 1 and 2  */
static double solzen_wf;	/* Elevation weighting factor                 */
static double tmp0;		/* Temporary variable for interpolation       */
static double tmp1;		/* Temporary variable for interpolation       */
static double tmp00;		/* Temporary variable for interpolation       */
static double tmp01;		/* Temporary variable for interpolation       */
static double tmp10;		/* Temporary variable for interpolation       */
static double tmp11;		/* Temporary variable for interpolation       */
static double tmp000;		/* Temporary variable for interpolation       */
static double tmp001;		/* Temporary variable for interpolation       */
static double tmp010;		/* Temporary variable for interpolation       */
static double tmp011;		/* Temporary variable for interpolation       */
static double tmp100;		/* Temporary variable for interpolation       */
static double tmp101;		/* Temporary variable for interpolation       */
static double tmp110;		/* Temporary variable for interpolation       */
static double tmp111;		/* Temporary variable for interpolation       */

static long chan;		/* Loop control for the current AVHRR channel */
static long elev_hi;		/* Elevation index in the LUTs                */
static long elev_lo;		/* Elevation index in the LUTs                */
static long i;			/* Loop control                               */
static long relaz_hi;		/* Relative azimuth index in the LUTs         */
static long relaz_lo;		/* Relative azimuth index in the LUTs         */
static long satzen_hi;		/* Satellite zenith index in the LUTs         */
static long satzen_lo;		/* Satellite zenith index in the LUTs         */
static long solzen_hi;		/* Solar zenith index in the LUTs             */
static long solzen_lo;		/* Solar zenith index in the LUTs             */
static long water_lo;		/* water vapor index in the LUTs              */
static long water_hi;		/* water vapor index in the LUTs              */
static long ozone_lo;		/* ozone index in the LUTs		      */
static long ozone_hi;		/* ozone index in the LUTs		      */



/* If the LUTs have not been loaded, error out
----------------------------------------------*/
if (luts_not_loaded)
    return(E_FAIL);


/* Convert the angles to positive values
----------------------------------------*/
abs_angs[SATZEN] = fabs( angs[SATZEN] );
abs_angs[SOLZEN] = fabs( angs[SOLZEN] );
abs_angs[RELAZ]  = fabs( angs[RELAZ] );


/* Determine where the elevation falls within elev_gridpts
----------------------------------------------------------*/
if (elev <= elev_gridpts[0])
    {
    elev_lo = elev_hi = 0;
    elev_wf = 0.0;
    }
else if (elev >= elev_gridpts[ELEV_PTS-1])
    {
    elev_lo = elev_hi = ELEV_PTS-1;
    elev_wf = 1.0;
    }
else
    {
    for (i = 1; (i < ELEV_PTS) && (elev > elev_gridpts[i]); i++);
    elev_lo = i - 1;
    elev_hi = i;
    elev_wf = (elev - elev_gridpts[elev_lo]) /
	(elev_gridpts[elev_hi] - elev_gridpts[elev_lo]);
    }


/* Determine where the relative azimuth falls within relaz_gridpts
------------------------------------------------------------------*/
if (abs_angs[RELAZ] <= relaz_gridpts[0])
    {
    relaz_lo = relaz_hi = 0;
    relaz_wf = 0.0;
    }
else if (abs_angs[RELAZ] >= relaz_gridpts[RELAZ_PTS-1])
    {
    relaz_lo = relaz_hi = RELAZ_PTS-1;
    relaz_wf = 1.0;
    }
else
    {
    for (i = 1; (i < RELAZ_PTS) && (abs_angs[RELAZ] > relaz_gridpts[i]); i++);
    relaz_lo = i - 1;
    relaz_hi = i;
    relaz_wf = (abs_angs[RELAZ] - relaz_gridpts[relaz_lo]) /
	(relaz_gridpts[relaz_hi] - relaz_gridpts[relaz_lo]);
    }


/* Determine where the satellite zenith falls within satzen_gridpts
------------------------------------------------------------------*/
if (abs_angs[SATZEN] <= satzen_gridpts[0])
    {
    satzen_lo = satzen_hi = 0;
    satzen_wf = 0.0;
    }
else if (abs_angs[SATZEN] >= satzen_gridpts[SATZEN_PTS-1])
    {
    satzen_lo = satzen_hi = SATZEN_PTS-1;
    satzen_wf = 1.0;
    }
else
    {
    for (i = 1; (i < SATZEN_PTS) && (abs_angs[SATZEN] > satzen_gridpts[i]);i++);
    satzen_lo = i - 1;
    satzen_hi = i;
    satzen_wf = (abs_angs[SATZEN] - satzen_gridpts[satzen_lo]) /
	(satzen_gridpts[satzen_hi] - satzen_gridpts[satzen_lo]);
    }


/* Determine where the solar zenith falls within solzen_gridpts
---------------------------------------------------------------*/
if (abs_angs[SOLZEN] <= solzen_gridpts[0])
    {
    solzen_lo = solzen_hi = 0;
    solzen_wf = 0.0;
    }
else if (abs_angs[SOLZEN] >= solzen_gridpts[SOLZEN_PTS-1])
    {
    solzen_lo = solzen_hi = SOLZEN_PTS-1;
    solzen_wf = 1.0;
    }
else
    {
    for (i = 1; (i < SOLZEN_PTS) && (abs_angs[SOLZEN] > solzen_gridpts[i]);i++);
    solzen_lo = i - 1;
    solzen_hi = i;
    solzen_wf = (abs_angs[SOLZEN] - solzen_gridpts[solzen_lo]) /
	(solzen_gridpts[solzen_hi] - solzen_gridpts[solzen_lo]);
    }


/* Determine where the water vapor transmittance falls 
------------------------------------------------------*/
if ( water <= water_grid[0] )
    {
    water_lo = water_hi = 0;
    water_wf = 0.0;
    }
else if ( water >= water_grid[1] )
    {
    water_lo = water_hi = WATVAP_CON-1;
    water_wf = 1.0;
    }
else
    {
    for ( i = 0; i < WATVAP_CON; i++ )
        if ((i*water_grid[2])+water_grid[0] > water )
	    {
	    water_hi = i;
	    break;
	    }
    water_lo = water_hi - 1;
    water_wf = (water - (water_lo*water_grid[2]+water_grid[0])) / water_grid[2];
    }


/* Determine where the ozone transmittance falls 
------------------------------------------------*/
if ( *ozone <= ozone_grid[0] )
    {
    ozone_lo = ozone_hi = 0;
    ozone_wf = 0.0;
    }
else if ( *ozone >= ozone_grid[1] )
    {
    ozone_lo = ozone_hi = OZONE_CON-1;
    ozone_wf = 1.0;
    }
else
    {
    for ( i = 0; i < OZONE_CON; i++ )
        if ((i*ozone_grid[2])+ozone_grid[0] > *ozone )
	    {
	    ozone_hi = i;
	    break;
	    }
    ozone_lo = ozone_hi - 1;
    ozone_wf = (*ozone - (ozone_lo*ozone_grid[2]+ozone_grid[0])) /ozone_grid[2];
    }


/* Interpolate the coefficients for each channel
------------------------------------------------*/
for (chan = 0; chan < NUM_CHANS; chan++)
    {
    /* Interpolate the A coefficient
    --------------------------------*/
    tmp00 =
     a_lut[chan][solzen_lo][satzen_lo][elev_lo] +
    (a_lut[chan][solzen_lo][satzen_lo][elev_hi] -
     a_lut[chan][solzen_lo][satzen_lo][elev_lo]) * elev_wf;

    tmp01 =
     a_lut[chan][solzen_hi][satzen_lo][elev_lo] +
    (a_lut[chan][solzen_hi][satzen_lo][elev_hi] -
     a_lut[chan][solzen_hi][satzen_lo][elev_lo]) * elev_wf;

    tmp10 =
     a_lut[chan][solzen_lo][satzen_hi][elev_lo] +
    (a_lut[chan][solzen_lo][satzen_hi][elev_hi] -
     a_lut[chan][solzen_lo][satzen_hi][elev_lo]) * elev_wf;

    tmp11 =
     a_lut[chan][solzen_hi][satzen_hi][elev_lo] +
    (a_lut[chan][solzen_hi][satzen_hi][elev_hi] -
     a_lut[chan][solzen_hi][satzen_hi][elev_lo]) * elev_wf;
    
    tmp0 = tmp00 + (tmp10 - tmp00) * satzen_wf;
    tmp1 = tmp01 + (tmp11 - tmp01) * satzen_wf;

    acoef[chan] = tmp0 + (tmp1 - tmp0) * solzen_wf;



    /* Interpolate the B coefficient
    --------------------------------*/
    tmp000 =
     b_lut[chan][solzen_lo][satzen_lo][relaz_lo][elev_lo] +
    (b_lut[chan][solzen_lo][satzen_lo][relaz_lo][elev_hi] -
     b_lut[chan][solzen_lo][satzen_lo][relaz_lo][elev_lo]) * elev_wf;

    tmp001 =
     b_lut[chan][solzen_hi][satzen_lo][relaz_lo][elev_lo] +
    (b_lut[chan][solzen_hi][satzen_lo][relaz_lo][elev_hi] -
     b_lut[chan][solzen_hi][satzen_lo][relaz_lo][elev_lo]) * elev_wf;

    tmp010 =
     b_lut[chan][solzen_lo][satzen_hi][relaz_lo][elev_lo] +
    (b_lut[chan][solzen_lo][satzen_hi][relaz_lo][elev_hi] -
     b_lut[chan][solzen_lo][satzen_hi][relaz_lo][elev_lo]) * elev_wf;

    tmp011 =
     b_lut[chan][solzen_hi][satzen_hi][relaz_lo][elev_lo] +
    (b_lut[chan][solzen_hi][satzen_hi][relaz_lo][elev_hi] -
     b_lut[chan][solzen_hi][satzen_hi][relaz_lo][elev_lo]) * elev_wf;

    tmp100 =
     b_lut[chan][solzen_lo][satzen_lo][relaz_hi][elev_lo] +
    (b_lut[chan][solzen_lo][satzen_lo][relaz_hi][elev_hi] -
     b_lut[chan][solzen_lo][satzen_lo][relaz_hi][elev_lo]) * elev_wf;

    tmp101 =
     b_lut[chan][solzen_hi][satzen_lo][relaz_hi][elev_lo] +
    (b_lut[chan][solzen_hi][satzen_lo][relaz_hi][elev_hi] -
     b_lut[chan][solzen_hi][satzen_lo][relaz_hi][elev_lo]) * elev_wf;

    tmp110 =
     b_lut[chan][solzen_lo][satzen_hi][relaz_hi][elev_lo] +
    (b_lut[chan][solzen_lo][satzen_hi][relaz_hi][elev_hi] -
     b_lut[chan][solzen_lo][satzen_hi][relaz_hi][elev_lo]) * elev_wf;

    tmp111 =
     b_lut[chan][solzen_hi][satzen_hi][relaz_hi][elev_lo] +
    (b_lut[chan][solzen_hi][satzen_hi][relaz_hi][elev_hi] -
     b_lut[chan][solzen_hi][satzen_hi][relaz_hi][elev_lo]) * elev_wf;

    tmp00 = tmp000 + (tmp100 - tmp000) * relaz_wf;
    tmp01 = tmp001 + (tmp101 - tmp001) * relaz_wf;
    tmp10 = tmp010 + (tmp110 - tmp010) * relaz_wf;
    tmp11 = tmp011 + (tmp111 - tmp011) * relaz_wf;

    tmp0 = tmp00 + (tmp10 - tmp00) * satzen_wf;
    tmp1 = tmp01 + (tmp11 - tmp01) * satzen_wf;

    bcoef[chan] = tmp0 + (tmp1 - tmp0) * solzen_wf;


    /* Interpolate the S coefficient
    --------------------------------*/
    scoef[chan] = s_lut[chan][elev_lo] +
	(s_lut[chan][elev_hi] - s_lut[chan][elev_lo]) * elev_wf;
    
    /* Interpolate the water vapor transmittance
    --------------------------------------------*/
    tmp00 = 
        w_lut[chan][water_lo][solzen_lo][satzen_lo] +
       (w_lut[chan][water_lo][solzen_lo][satzen_hi] -
        w_lut[chan][water_lo][solzen_lo][satzen_lo]) * satzen_wf;
    tmp01 = 
        w_lut[chan][water_hi][solzen_lo][satzen_lo] +
       (w_lut[chan][water_hi][solzen_lo][satzen_hi] -
        w_lut[chan][water_hi][solzen_lo][satzen_lo]) * satzen_wf;
    tmp10 = 
        w_lut[chan][water_lo][solzen_hi][satzen_lo] +
       (w_lut[chan][water_lo][solzen_hi][satzen_hi] -
        w_lut[chan][water_lo][solzen_hi][satzen_lo]) * satzen_wf;
    tmp11 =
        w_lut[chan][water_hi][solzen_hi][satzen_lo] +
       (w_lut[chan][water_hi][solzen_hi][satzen_hi] -
        w_lut[chan][water_hi][solzen_hi][satzen_lo]) * satzen_wf;
    

    tmp0 = tmp00 + (tmp10 - tmp00) * solzen_wf;
    tmp1 = tmp01 + (tmp11 - tmp01) * solzen_wf;

    Tw[chan] = tmp0 + (tmp1 - tmp0) * water_wf;
    }


/* Look up the Ozone gaseous transmittance.  If the ozone value is zero or
   less, set it to the standard ozone value for the earth.  Note this has
   no effect on AVHRR channel 2.
--------------------------------------------------------------------------*/
if (*ozone <= 0.0)
    *ozone = AVG_OZONE;


/* Interpolate the O3 transmittance
-----------------------------------*/
tmp00 = o_lut[ozone_lo][solzen_lo][satzen_lo] +
       (o_lut[ozone_lo][solzen_lo][satzen_hi] -
        o_lut[ozone_lo][solzen_lo][satzen_lo]) * satzen_wf;
tmp01 = o_lut[ozone_hi][solzen_lo][satzen_lo] +
       (o_lut[ozone_hi][solzen_lo][satzen_hi] -
        o_lut[ozone_hi][solzen_lo][satzen_lo]) * satzen_wf;
tmp10 = o_lut[ozone_lo][solzen_hi][satzen_lo] +
       (o_lut[ozone_lo][solzen_hi][satzen_hi] -
        o_lut[ozone_lo][solzen_hi][satzen_lo]) * satzen_wf;
tmp11 = o_lut[ozone_hi][solzen_hi][satzen_lo] +
       (o_lut[ozone_hi][solzen_hi][satzen_hi] -
        o_lut[ozone_hi][solzen_hi][satzen_lo]) * satzen_wf;
tmp0 = tmp00 + (tmp10 - tmp00) * solzen_wf;
tmp1 = tmp01 + (tmp11 - tmp01) * solzen_wf;
Tg = tmp0 + (tmp1 - tmp0) * ozone_wf;


/* Atmospherically correct the data which is valid. For invalid
   data (mask data), set the output to the input.
---------------------------------------------------------------*/
for (chan = 0; chan < NUM_CHANS; chan++)
    {
    if (validflg[chan])
	{
	if (chan == 0)
	    Y = acoef[chan] / (Tg * Tw[chan]) * sun_earth * chans[chan] *
	   	 k_lut[chan] - bcoef[chan];
	else
	    Y = acoef[chan] / Tw[chan] * sun_earth * chans[chan] *
	   	 k_lut[chan] - bcoef[chan];
    
	corrected[chan] = 100.0 * Y / (100.0 + Y * scoef[chan]);
	}
    else
	corrected[chan] = chans[chan];
    }

outcoef[0] = acoef[0];
outcoef[1] = acoef[1];
outcoef[2] = bcoef[0];
outcoef[3] = bcoef[1];
outcoef[4] = scoef[0];
outcoef[5] = scoef[1];
outcoef[6] = Tg;
outcoef[7] = Tw[0];
outcoef[8] = Tw[1];


#ifdef QUICKCOR_COMPARE
quickcor(chans, abs_angs, elev, *ozone, sun_earth, qcor);
#endif

return(E_SUCC);
}
