/*******************************************************************************
FUNCTION:	get_scale

PURPOSE:	Set the scaling factors for the reflectance, radiance,
	 	and angle bands based on the scaling option and data
		type.

*******************************************************************************/
#include <stdio.h>	/* Standard I/O include			*/
#include "las.h"	/* LAS include file			*/
#include "util.h"
#include "adaps.h"	/* ADAPS include file			*/
#include "adapssup.h"	/* ADAPS include file			*/
#include "atmocor.h"

/* Local function that returns the appropriate
data type for the input scaling value.
----------------------------------------------*/
long get_dtype
(
char scale[CMLEN],
long dtype
);

int get_scale
(
    char hosin[][CMLEN],	/* IN host input file names		*/
    long radopt,		/* IN Input radiance values flag	*/
    char input_scaling[],	/* IN input scaling parameter		*/
    char angle_scaling[],	/* IN angle scaling parameter		*/
    char output_scaling[],	/* IN output scaling parameter		*/
    long odtype,		/* IN output data type			*/
    struct SCOFF scoff[]	/* OUT Scale factors, offsets for scaling*/
)
{
char quantity[12];		/* Radiance or Reflectance		*/
long status;			/* Return status			*/
long one = 1;			/* constant of one			*/
long dtype;			/* data type value			*/
long dtypescale;		/* scaling value adjusted for data type	*/


/* Get the scaling factors, offsets for satellite zenith angles
---------------------------------------------------------------*/
if ( c_gettyp( (const char (*)[CMLEN])&hosin[IMG_ANGLES], &one, &dtype )
       != E_SUCC )
    return E_FAIL;
dtypescale = get_dtype( angle_scaling, dtype );
if ( dtypescale == E_FAIL )
    return E_FAIL;
status = c_getscale( dtypescale, "satzenith",
    &scoff[SATZEN].minval, &scoff[SATZEN].maxval, &scoff[SATZEN].scale,
    &scoff[SATZEN].unscale, &scoff[SATZEN].offset);
if (status != E_SUCC)
    return(E_FAIL);
scoff[SATZEN].scaling = dtypescale;


/* Get the scaling factors, offsets for solar zenith angles
-----------------------------------------------------------*/
if ( c_gettyp( (const char (*)[CMLEN])&hosin[IMG_ANGLES], &one, &dtype )
       != E_SUCC )
    return E_FAIL;
dtypescale = get_dtype( angle_scaling, dtype );
if ( dtypescale == E_FAIL )
    return E_FAIL;
status = c_getscale( dtypescale, "solzenith",
    &scoff[SOLZEN].minval, &scoff[SOLZEN].maxval, &scoff[SOLZEN].scale,
    &scoff[SOLZEN].unscale, &scoff[SOLZEN].offset);
if (status != E_SUCC)
    return(E_FAIL);
scoff[SOLZEN].scaling = dtypescale;


/* Get the scaling factors, offsets for relative azimuth angles
---------------------------------------------------------------*/
if ( c_gettyp( (const char (*)[CMLEN])&hosin[IMG_ANGLES], &one, &dtype )
       != E_SUCC )
    return E_FAIL;
dtypescale = get_dtype( angle_scaling, dtype );
if ( dtypescale == E_FAIL )
    return E_FAIL;
status = c_getscale( dtypescale, "relazimuth",
    &scoff[RELAZ].minval, &scoff[RELAZ].maxval, &scoff[RELAZ].scale,
    &scoff[RELAZ].unscale, &scoff[RELAZ].offset);
if (status != E_SUCC)
    return(E_FAIL);
scoff[RELAZ].scaling = dtypescale;


/* Get the scaling factors, offsets for input image visible bands
-----------------------------------------------------------------*/
if ( c_gettyp( (const char (*)[CMLEN])&hosin[IMG_INPUT], &one, &dtype )
       != E_SUCC )
    return E_FAIL;
dtypescale = get_dtype( input_scaling, dtype );
scoff[IMG_INPUT].scaling = dtypescale;
if ( dtypescale == E_FAIL )
    return E_FAIL;
if (radopt)
   strcpy(quantity,"radiance");
else
   strcpy(quantity,"reflectance");
status = c_getscale( dtypescale, quantity,
    &scoff[IN_REFLECT].minval, &scoff[IN_REFLECT].maxval,
    &scoff[IN_REFLECT].scale, &scoff[IN_REFLECT].unscale,
    &scoff[IN_REFLECT].offset);
if (status != E_SUCC)
    return(E_FAIL);
scoff[IN_REFLECT].scaling = dtypescale;


/* Get the scaling factors, offsets for output image visible bands.
   The units are always relectance or albedo, as opposed to radiance.
---------------------------------------------------------------------*/
dtypescale = get_dtype( output_scaling, odtype );
if ( dtypescale == E_FAIL )
    return E_FAIL;
status = c_getscale( dtypescale, "reflectance",
    &scoff[OUT_REFLECT].minval, &scoff[OUT_REFLECT].maxval,
    &scoff[OUT_REFLECT].scale, &scoff[OUT_REFLECT].unscale,
    &scoff[OUT_REFLECT].offset);
if (status != E_SUCC)
    return(E_FAIL);
scoff[OUT_REFLECT].scaling = dtypescale;
scoff[OUT_REFLECT].odtype = odtype;


return(E_SUCC);
}



/* Return the appropriate data type
for the type of scaling input.
-----------------------------------*/
long get_dtype
(
char scale[CMLEN],	/* IN scaling type			*/
long dtype		/* IN data type				*/
)
{
long dtypescale;	/* scaling value adjusted for data type	*/


if ( strcmp( scale, "GLOBAL" ) == 0 )
    {
    switch ( dtype )
        {
	case EBYTE: dtypescale = AVHRR_BYTE; break;
	case EWORD: dtypescale = AVHRR_10BIT; break;
	case ELONG: dtypescale = AVHRR_I4; break;
	case EREAL: dtypescale = AVHRR_REAL; break;
	default   : return E_FAIL;
	}
    }
else if ( strcmp( scale, "US" ) == 0 )
    {
    switch ( dtype )
	{
	case EBYTE: dtypescale = US_BYTE; break;
	case EWORD: dtypescale = US_I2; break;
	case ELONG: dtypescale = US_I4; break;
	case EREAL: dtypescale = US_REAL; break;
	default   : return E_FAIL;
	}
    }
else
    {
    dtypescale = NO_SCALING;
    }

return dtypescale;
}
