/*******************************************************************************
NAME:		PROCESS

PURPOSE:	Apply the atmospheric correction to the input image.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "atmocor.h"


#define MINPIXEL 10.0		/* Min valid pixel for global scaling	*/

extern long curent;		/* current processing line		*/


int process
(
    struct GDESC  *gdesc,	/* IN group file descriptor for images	*/
    unsigned char *thebuf,	/* IN the input output buffer		*/
    long offset[],		/* IN offset array into thebuf		*/
    long window[],		/* IN Input window to process		*/
    double minval[],		/* min pixel value array		*/
    double maxval[],		/* max pixel value array		*/
    long mmflag,		/* min/max flag				*/
#ifdef QUICKCOR_COMPARE
    long radopt,		/* Radiance flag (RADIANCE or ALBEDO)	*/
#endif
    double dsquared,		/* Sun-earth distance squared		*/
    struct SCOFF scoff[],	/* IN scaling coefficients		*/
    int *band_sequence		/* input band sequence array		*/
)
{
double coeffs[NCOEFF];	/* Atmospheric correction coefficients		*/
double angs[3];		/* Unscaled view angles				*/
double atmo[MAXVISBAND];/* Atmospherically corrected pixels		*/
double chans[MAXVISBAND];/* Unscaled input pixel values			*/
double dem;		/* Digital Elevation Model for current pixel	*/
double ozone;		/* Ozone value for current pixel		*/
double water;		/* Water vapor value for current pixel		*/

long band;		/* loop index counting the bands		*/
long sample;		/* sample counter				*/
long validflg[MAXVISBAND];/* Valid data (TRUE of FALSE) for each chan	*/

short global_in;	/* Flag indication global scaling input image	*/

float *water_ptr;	/* pointer into thebuf for water vapor		*/
float *dem_ptr;		/* pointer into thebuf for elevation image	*/
float *ozone_ptr;	/* pointer into thebuf for ozone image		*/
float *angle_ptr[3];	/* pointer into thebuf for angles bands		*/
float *chan_ptr[3];	/* pointer into thebuf for avhrr channels	*/

int totalbands = 2;	/* total number of input avhrr bands		*/
int outndx;		/* index into thebuf for the first output band	*/

unsigned char *ptr1[3];	/* pointers into thebuf for the output bands	*/
short         *ptr2[3];	/* pointers into thebuf for the output bands	*/
long          *ptr3[3];	/* pointers into thebuf for the output bands	*/
float         *ptr4[3];	/* pointers into thebuf for the output bands	*/

/* Variables for quick cor
--------------------------*/
#ifdef QUICKCOR_COMPARE
double qcor[MAXVISBAND];/* Atmospherically corrected pixels returned by Dave  */
			/* Meyer's quickcor program                           */
int    i;		/* Loop control                              */
char str[CMLEN];	/* String for error messages                 */
FILE *fp;		/* File ptr for printing debug info          */
#endif

/* Variables for DEBUG
----------------------*/
#ifdef DEBUG_IMG
float *debug_ptr[MAXBANDS];
#endif


/* If we are comparing ATMOCOR to quickcor, open a file and print a header
--------------------------------------------------------------------------*/
#ifdef QUICKCOR_COMPARE
c_lasmsg(FALSE," ","Comparing ATMOCOR to QUICKCOR");
c_lasmsg(FALSE," ","Will be extremely slow...");

sprintf(str, "atmocor_table.%d", getpid());
fp = fopen(str, "w");
if (fp == NULL)
    {
    c_lasmsg(FALSE,"atmocor-open","Error opening debug file");
    return E_FAIL;
    }

fprintf(fp, "ATMOCOR output:\n");
fprintf(fp, "\n");
if (radopt)
    fprintf(fp, "Chan1 and Chan2 are in radiance.\n");
else
    fprintf(fp, "Chan1 and Chan2 are in reflectance.\n");
fprintf(fp, "Distance to sun squared (astronomical units): %6.3lf\n", dsquared);
fprintf(fp, "\n");

fprintf(fp, "Chan1    Chan2    ");
fprintf(fp, "A(1)    A(2)    B(1)    B(2)    S(1)    S(2)    Tg      ");
fprintf(fp, "Elev    ");
fprintf(fp, "SatZen   SolZen   RelAz    ");
fprintf(fp, "Ozone   Water   Qcor(1)  Qcor(2)  Atmo(1)  Atmo(2)\n");

fprintf(fp, "-------- -------- ");
fprintf(fp, "------- ------- ------- ------- ------- ------- ------- ");
fprintf(fp, "------- ");
fprintf(fp, "-------- -------- -------- ");
fprintf(fp, "------- ------- -------- -------- -------- --------\n");
#endif



/* Determine if the input image is globally scaled
--------------------------------------------------*/
if ((scoff[IMG_INPUT].scaling == AVHRR_BYTE)  ||
    (scoff[IMG_INPUT].scaling == AVHRR_10BIT) ||
    (scoff[IMG_INPUT].scaling == AVHRR_I2)    ||
    (scoff[IMG_INPUT].scaling == AVHRR_I4)    ||
    (scoff[IMG_INPUT].scaling == AVHRR_REAL))
    global_in = TRUE;
else
    global_in = FALSE;


/* Start the processing message
-------------------------------*/
curent = 1;
c_startm();


/* Determine the index into the offset array for the output image
-----------------------------------------------------------------*/
outndx = OFF_INCHAN + totalbands;


/* Main processing loop
-----------------------*/
for (curent = 1; curent <= window[NL]; curent++)
    {
    /* Get the next line of the images
    ----------------------------------*/
    if ( c_estep( &gdesc ) != E_SUCC )
	return E_FAIL;

    /* Set up the pointers into thebuf, the inputs were all opened
    as R*4 but the output could be anything so we must check for
    the output type and set the appropriate pointer.
    --------------------------------------------------------------*/
    water_ptr    = (float *)( thebuf + ( offset[OFF_WATER] * sizeof(float)));
    ozone_ptr    = (float *)( thebuf + ( offset[OFF_OZONE] * sizeof(float)));
    dem_ptr      = (float *)( thebuf + ( offset[OFF_DEM] * sizeof(float)));
    angle_ptr[0] = (float *)( thebuf + ( offset[OFF_SATZEN] * sizeof(float)));
    angle_ptr[1] = (float *)( thebuf + ( offset[OFF_SOLZEN] * sizeof(float)));
    angle_ptr[2] = (float *)( thebuf + ( offset[OFF_RELAZ] * sizeof(float)));
    for ( band = 0; band < totalbands; band++ )
	{
	/* The input AVHRR channel bands may be in any order so we must
	use the band sequence array to determine which pointer to set.
	---------------------------------------------------------------*/
	chan_ptr[band_sequence[band]-1] = (float *)( thebuf
				+ ( offset[OFF_INCHAN+band] * sizeof(float)));

	/* The output bands will always be in order (1,2 3)
	--------------------------------------------------*/
	switch ( scoff[OUT_REFLECT].odtype )
	    {
	    case EBYTE :
		ptr1[band] = (unsigned char *)( thebuf + ( offset[outndx+band]
			    * sizeof( unsigned char )));
		break;
	    case EWORD :
		ptr2[band] = (short *)( thebuf + ( offset[outndx+band]
			    * sizeof(short)));
		break;
	    case ELONG :
		ptr3[band] = (long *)( thebuf + ( offset[outndx+band]
			    * sizeof(long)));
		break;
	    case EREAL :
		ptr4[band] = (float *)( thebuf + ( offset[outndx+band]
			    * sizeof(float)));
		break;
	    }
	}  /* END for band loop */
#ifdef DEBUG_IMG
    for ( band = 0; band < MAXBANDS; band++ )
        {
	debug_ptr[band] = (float *)( thebuf + ( offset[outndx+band]
			 * sizeof(float)));
	}
#endif



    /* Process the line
    -------------------*/
    for (sample = 0; sample < window[NS]; sample++)
	{
	/* Begin by assuming that the input pixels are valid
	----------------------------------------------------*/
	for (band = 0; band < totalbands; band++)
	    validflg[band] = TRUE;

	/* Read the bands of data for a given sample
	and increment the pointers to the next sample
	--------------------------------------------*/
	water = *water_ptr++;
	ozone = *ozone_ptr++;
	dem   = *dem_ptr++;
	angs[0] = *angle_ptr[0]++;
	angs[1] = *angle_ptr[1]++;
	angs[2] = *angle_ptr[2]++;
	for ( band = 0; band < totalbands; band++ )
	    chans[band] = *chan_ptr[band]++;
	
	/* Unscale the input pixel values.
	----------------------------------*/
	if (global_in)
	    {
	    /* Do not atmospherically correct any of the mask values
	    --------------------------------------------------------*/
	    for (band = 0; band < totalbands; band++)
		if (chans[band] < MINPIXEL)
		    validflg[band] = FALSE;
		else
		    chans[band] = UNSCALE(chans[band],
			scoff[IN_REFLECT].unscale, scoff[IN_REFLECT].offset);
	    }
	else
	    {
	    for (band = 0; band < totalbands; band++)
		{
		chans[band] = UNSCALE(chans[band],
		    scoff[IN_REFLECT].unscale, scoff[IN_REFLECT].offset);
		}
	    }
	
	/* Unscale the angles.  Should check for mask values in the angles
	as well.  At this point we have decided that the angles are
	computed and should be valid all the time.  This will save some
	compution time.
	------------------------------------------------------------------*/
	for (band = 0; band < 3; band++)
	    {
	    angs[band] = UNSCALE(angs[band], scoff[band].unscale,
		scoff[band].offset);
	    }
	

	/* Atmospherically correct the AVHRR channels
	---------------------------------------------*/
	dem = dem / 1000.0;
	correct(chans, angs, dem, &ozone, water, dsquared,
	    validflg, atmo, coeffs
#ifdef QUICKCOR_COMPARE
	    , qcor
#endif
	    );


#ifdef QUICKCOR_COMPARE
        /*
	  Print the input channels
	*/
	fprintf(fp, "%+08.3lf %+08.3lf ", chans[0], chans[1]);

	/*
	  Print the coefficients
	*/
	for (i = 0; i < NCOEFF; i++)
	    fprintf(fp, "%+07.3lf ", coeffs[i]);

	/*
	  Print the elevation and angle values
	*/
	fprintf(fp, "%+07.3lf %+08.3lf %+08.3lf %+08.3lf ", dem,
	    angs[SATZEN], angs[SOLZEN], angs[RELAZ]);

	/*
	  Print the ozone, water vapor, quickcor results, and ATMOCOR's results
	*/
	fprintf(fp, "%+07.3lf %+07.3lf %+08.3lf %+08.3lf %+08.3lf %+08.3lf\n",
	    ozone, water, qcor[0], qcor[1], atmo[0], atmo[1]);

	/*
	  Flush the buffer
	*/
	fflush(fp);
#endif


#ifndef DEBUG_IMG
	/* Scale the output values and store them in the output buffer.
	--------------------------------------------------------------*/
	scale(totalbands, atmo, validflg, scoff, mmflag, minval, maxval);

	/* Store the scaled values in the output buffer
	and increment the pointers to the next sample
	-----------------------------------------------*/
	for ( band = 0; band < totalbands; band++ )
	    {
	    switch ( scoff[OUT_REFLECT].odtype )
		{
		case EBYTE :
		    *ptr1[band] = (unsigned char)atmo[band];
		    ptr1[band]++;
		    break;
		case EWORD :
		    *ptr2[band] = (short)atmo[band];
		    ptr2[band]++;
		    break;
		case ELONG :
		    *ptr3[band] = (long)atmo[band];
		    ptr3[band]++;
		    break;
		case EREAL :
		    *ptr4[band] = (float)atmo[band];
		    ptr4[band]++;
		    break;
		}
	    }
#else
	/* Save the debug information.
	------------------------------*/
	*debug_ptr[0]  = (float)chans[0];
	*debug_ptr[1]  = (float)chans[1];
	*debug_ptr[2]  = (float)coeffs[0];
	*debug_ptr[3]  = (float)coeffs[1];
	*debug_ptr[4]  = (float)coeffs[2];
	*debug_ptr[5]  = (float)coeffs[3];
	*debug_ptr[6]  = (float)coeffs[4];
	*debug_ptr[7]  = (float)coeffs[5];
	*debug_ptr[8]  = (float)dem;
	*debug_ptr[9]  = (float)angs[SATZEN];
	*debug_ptr[10] = (float)angs[SOLZEN];
	*debug_ptr[11] = (float)angs[RELAZ];
	*debug_ptr[12] = (float)ozone;
	*debug_ptr[13] = (float)water;
	*debug_ptr[14] = (float)atmo[0];
	*debug_ptr[15] = (float)atmo[1];
	*debug_ptr[16] = (float)coeffs[6];
	*debug_ptr[17] = (float)coeffs[7];
	*debug_ptr[18] = (float)coeffs[8];

	/* Adjust the min/max arrays if needed
	--------------------------------------*/
	if ( mmflag )
	    {
	    for ( band = 0; band < 19; band++ )
		{
	        if ( *debug_ptr[band] > maxval[band] )
		    maxval[band] = *debug_ptr[band];
		if ( *debug_ptr[band] < minval[band] )
		    minval[band] = *debug_ptr[band];
		}
	    }

	/* Increment the pointers to the next output pixel
	--------------------------------------------------*/
	for ( band = 0; band < 19; band++ )
	    debug_ptr[band]++;
#endif

	}  /* END of for sample loop (process the line) */
    }  /* END main processing loop */

/* Final write
--------------*/
if ( c_estep( &gdesc ) == E_FAIL )
    return E_FAIL;

/* Stop the processing messages
-------------------------------*/
c_stoptm();


/* Close the table if necessary
-------------------------------*/
#ifdef QUICKCOR_COMPARE
fclose(fp);
#endif

return(E_SUCC);
}
