/*******************************************************************************
NAME:		AVCOORD

PURPOSE:	Convert line/sample pairs to latitude/longitude pairs or
		convert latitude/longitude pairs to line/sample pairs using
		the AVHRR orbital model.
	   

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   J. Matzner 	   5/93		 EDC/CSB	Original Development
							for ADAPS
  2.0      D. Hames       11/99          EDC/CSB        Remove 50000 limits from
						        avcoord.pdf. Add error
							checks to getpar to 
							ensure that line and
							sample are positive 
							values. Add prototypes.
							SRF #893
PROJECT:			ADAPS

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "model.h"
#include "constants.h"
#include "util.h"
#include "archio.h"
#include "protos.h"

#define NC	50		/* Number of coordinates		     */
#define DMS	0.0

void main
(void)
{
struct PARBLK vblock;		/* TAE parameter block			     */
struct MODEL  model;  		/* Model and orbital information	     */

FILE	*prt_fd;		/* Print file descriptor		     */

long	totpairs,		/* Total number of pairs to convert	     */
	status,			/* Return status variable		     */
	std_out,		/* Standard output			     */
	i,
	fatal = -1,		/* Fatal error code			     */
	nonfatal = 1,		/* Non-fatal error code			     */
	ocount = 0;		/* Number of output files created	     */

char	hosin[CMLEN],           /* Host names of input image		     */
	subcmd[CMLEN],          /* Subcommand the routine was run with	     */
	prtfile[CMLEN],		/* Print file				     */
	ofiles[1][CMLEN],	/* Output files created			     */
	slats[NC][11],		/* Latitudes in DMS format		     */
	slongs[NC][11],		/* Longitudes in DMS format		     */
	*sptr[NC];		/* Array of ptrs for variable block output   */

long	lines[NC],		/* Line values				     */
	samps[NC];		/* Sample values			     */

double	lats[NC],		/* Latitudes in DEG or RAD format	     */
	longs[NC],		/* Longtitude in DEG or RAD format	     */
	unit;		        /* unit scale factor			     */

/*
  Initialization LAS
*/
c_inlas(&vblock,&std_out);

/*
  Get the AVHRR LAS image and retreive the pairs of lat/lng or line/samp
*/
status = getpar(&vblock,hosin,subcmd,&totpairs,slats,slongs,lats,longs,
		lines,samps,&unit,&prt_fd,prtfile,(char *)ofiles,&ocount);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);

/*
  Get the header and ephemeris information from the ADDR
*/
status = c_getaddr(hosin,&model);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);

/*
  Main processing loop, returns lat/lng or line/samp
*/
if (strcmp(subcmd,"LINESAMP") == 0)
    {
    if (unit == DMS)
	{
	/*
	  Convert the input latitude/longitude DMS pairs to line/sample pairs
	*/
	dmsll(&model,totpairs,slats,slongs,lines,samps,prt_fd);
	}
    else
	{
	/*
	  Convert the input latitude/longitude degree or radian pairs to
	  line/sample pairs
	*/
	degll(&model,unit,totpairs,lats,longs,lines,samps,prt_fd);
	}

    /*
      Write the lines to the variable block
    */
    status = q_intg(&vblock,"LINE",totpairs,(int *)lines,P_UPDATE);
    if (status != SUCCESS)
	c_errmsg("Error updating variable block","avconv-vblk", &nonfatal);

    /*
      Write the samples to the variable block
    */
    status = q_intg(&vblock,"SAMPLE",totpairs, (int *)samps,P_UPDATE);
    if (status != SUCCESS)
	c_errmsg("Error updating variable block","avconv-vblk", &nonfatal);
    }
else
    {
    if (unit == DMS)
	{
	/*
	  Convert the input line/sample pairs to latitude/longitude DMS pairs
	*/
	dmsls(&model,totpairs,lines,samps,slats,slongs,prt_fd);

	/*
	  Setup an array pointer to the latitudes
	*/
	for (i = 0; i < totpairs; i++)
	    sptr[i] = slats[i];

	/*
	  Write the latitudes to the variable block
	*/
	status = q_string(&vblock,"LAT",totpairs,(const char **)sptr,P_UPDATE);
	if (status != SUCCESS)
	    c_errmsg("Error updating variable block","avconv-vblk", &nonfatal);

	/*
	  Setup an array pointer to the longitudes
	*/
	for (i = 0; i < totpairs; i++)
	    sptr[i] = slongs[i];

	/*
	  Write the longitudes to the variable block
	*/
	status = q_string(&vblock,"LONG",totpairs,(const char **)sptr,P_UPDATE);
	if (status != SUCCESS)
	    c_errmsg("Error updating variable block","avconv-vblk", &nonfatal);
	}
    else
	{
	/*
	  Convert the input line/sample pairs to latitude/longitude degree,
          or radian pairs
	*/
	degls(&model,unit,totpairs,lines,samps,lats,longs,prt_fd);

	/*
	  Write the latitudes to the variable block
	*/
	status = q_real(&vblock,"LAT",totpairs,lats,P_UPDATE);
	if (status != SUCCESS)
	    c_errmsg("Error updating variable block","avconv-vblk", &nonfatal);

	/*
	  Write the longitudes to the variable block
	*/
	status = q_real(&vblock,"LONG",totpairs,longs,P_UPDATE);
	if (status != SUCCESS)
	    c_errmsg("Error updating variable block","avconv-vblk", &nonfatal);
	}
    }

/*
  Send the variable block to TAE
*/
status = q_out(&vblock);
if (status != SUCCESS)
    c_errmsg("Error updating variable block","avcoord-vblk", &nonfatal);

/*
  Close the print file
*/
if (prt_fd != NULL)
    fclose(prt_fd);

/*
  Display a completion message
*/
c_complt("AVCOORD",ofiles,&ocount);
}
