/*******************************************************************************
FUNCTION:	formatit

PURPOSE:	To format slightly massaged input cut from the level 1b
		notices and transform it to a format suitable for adding to
		the $ADAPSTABLES/noaa.cal file.

INPUT:		File with data that looks like this:

_NOAA-16 AVHRR_
1    0.05398 -2.081 
0.1576 -53.55 498.96
2    0.05822 -2.199 
0.1715 -58.87 500.16
_NOAA-17 AVHRR_
1    0.05484 -2.195 
0.1610 -55.39 501.12
2    0.06660 -2.615 
0.1983 -68.64 500.73
3A   0.03169 -1.336 
0.2222 -97.00 501.37
_NOAA-18 AVHRR_
1    0.05045 -1.990 
0.1504 -51.73 500.54
2    0.05485 -2.161 
0.1646 -57.35 500.40 

		Start date - YYYYJJJ
		End date - YYYYJJJ

OUTPUT:		Standard out that looks like this:

satellite:   16                                                               |
date range:  2005261 2005365                                                  |
1    0.05398  -2.081  0.1576   -53.55      498                                |
2    0.05822  -2.199  0.1715   -58.87      500                                |
3A   0.0262   -1.010  0.1920   -84.20      502                                |
satellite:   17                                                               |
date range:  2005261 2005365                                                  |
1    0.05484  -2.195  0.1610   -55.39      501                                |
2    0.06660  -2.615  0.1983   -68.64      500                                |
3A   0.03169  -1.336  0.2222   -97.00      501                                |
satellite:   18                                                               |
date range:  2005261 2005365                                                  |
1    0.05045  -1.990  0.1504   -51.73      500                                |
2    0.05485  -2.161  0.1646   -57.35      500                                |
3A   0.02618  -0.982  0.18492  -80.88      500                                |

*******************************************************************************/
#include <stdio.h>
#include <string.h>

int main( int argc, char *argv[] )
{
FILE *inf;
char  buf1[100];
char  buf2[100];
int   i,j,count;
char *p;
float s1,s2,i1,i2,x;
char  band[3];
int   year;
int   jday;


/* Check for proper arguments
-----------------------------*/
if ( argc != 4 )
    {
    fprintf( stderr, "\nUsage: %s <infile> <sdate> <edate>\n\n", argv[0] );
    return 1;
    }

/* Attempt to open the input file
---------------------------------*/
inf = fopen( argv[1], "r" );
if ( !inf )
    {
    fprintf( stderr, "\nUnable to open file: %s\n\n", argv[1] );
    return 1;
    }

/* Extract the year and julian day of the start date
----------------------------------------------------*/
sscanf( argv[2], "%04d%03d", &year, &jday );
jday--;
if ( jday == 0 )
    {
    if ( (year-1) % 4 == 0 )
        jday = 366;
    else
        jday = 365;
    }


/* Loop through each of the active satellites
---------------------------------------------*/
for ( i = 16; i <= 18; i++ )
    {
    /* get and throw away the header line */
    fgets( buf1, 100, inf );
    /* set the number of data lines, N17 has 3 the rest only 2 */
    if ( i == 17 )
	count = 3;
    else
	count = 2;

    /* output the satellite number and date range lines */
    printf( "satellite:   %2d          ", i );
    printf( "                                                     |\n" );
    printf( "date range:  %s %s", argv[2], argv[3] );
    printf( "                                                  |\n" );

    /* for each data line */
    for ( j = 0; j < count; j++ )
	{
	/* data lines are actually split in two so read each line */
	fgets( buf1, 100, inf );
	fgets( buf2, 100, inf );

	/* look for the return character and if found replace it with ' ' */
	p = strchr( buf1, '\n' );
	if ( p )
	    *p = ' ';

	/* extract band number; two slope and intercepts and the pivot */
	sscanf( buf1, "%2s %f %f", band, &s1, &i1 );
	sscanf( buf2, "%f %f %f", &s2, &i2, &x );

	/* if the band isn't two characters add a ' ' */
	if ( strlen( band ) != 2 )
	    strcat( band, " " );

	/* print the line in proper format */
	printf( "%s   %7.5f  %6.3f  %6.4f   %6.2f      %3d",
	    band, s1, i1, s2, i2, (int)x);
	printf( "                                |\n" );
	}  /* END for each data line loop */

    /* for those satellites that don't vary channel 3A, print the info */
    if ( i == 16 )
        {
	printf( "3A   0.0262   -1.010  0.1920   -84.20      502" );
	printf( "                                |\n" );
	}
    else if ( i == 18 )
        {
	printf( "3A   0.02618  -0.982  0.18492  -80.88      500" );
	printf( "                                |\n" );
	}
    }  /* END for satellite loop */

/* Now give a little instruction to stderr to make sure things don't fall
through the cracks.
-------------------------------------------------------------------------*/
fprintf( stderr,
    "\n\t*****************************************************\n" );
fprintf( stderr, "\t* Concatenate the output onto $ADAPSTABLES/noaa.cal *\n" );
fprintf( stderr, "\t* and make sure the end date for previous group is  *\n" );
fprintf( stderr, "\t* changed to: %04d%03d                               *\n",
    year, jday );
fprintf( stderr,
    "\t*****************************************************\n\n" );


fclose( inf );
}  /* END main */

