/******************************************************************************
FUNCTION:	AVHRRCAL

PURPOSE:	Read a LAS AVHRR image, calculate its calibration coefficients
		and apply them creating a new LAS image of reflectance and
		temperature values.  


PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
1.0	   02/99    D.Lloyd		Developed from HRPTCAL

ALGORITHM:


ALGORITHM REFERENCES:
  Katherine B. Kidwell, NOAA Polar Orbiter Data Users Guide
      National Oceanic and Atmospheric Administration, Washington, D.C.
      July, 1991.
  NOAA KLM User's Guide, Draft April 1998.

*******************************************************************************/
#include "las.h"
#include "util.h"
#include "avhrrcal.h"


main()
{
char	hosin[MAXIMG][CMLEN];	/* Input host file names		*/
char	hosmnr[CMLEN];		/* host minor frame label services name	*/
char	hosout[2][CMLEN];	/* Output host file names		*/
char	minmax[4];		/* Min/max flag				*/
char	taein[MAXIMG][CMLEN];	/* Input LAS file names			*/
char	taeout[2][CMLEN];	/* Output LAS file names		*/

double	fone = 1.0;		/* Double constant of one		*/
double	omax[NCHANS];		/* Maximum image value			*/
double	omin[NCHANS];		/* Minimum image value			*/

FILE	*fptr;			/* Pointer to report file		*/

float	clipact[NCHANS][2];	/* Clipping limits (actual)		*/
float	cliplim[NCHANS][2];	/* Clipping limits (scaled)		*/

int	fdesc[MAXIMG+1];	/* Image file descriptors for image I/O	*/
int	gdesc;			/* Group descriptor for image I/O	*/

long	bands[MAXIMG][MAXBND+1];/* list of input bands			*/
long	cflag = NOCOMB;		/* Images not combined			*/
long	channel[NCHANS];	/* Channels needed			*/
long	clipcnt[NCHANS][2];	/* Clipping statistics			*/
long	mline;			/* Image master line number		*/
long	maxchan;		/* Number of channels needed		*/
long	msgtim;			/* Time between processing messages	*/
long	nbands[MAXIMG];		/* number of bands in each input image	*/
long	nimgs;			/* Number of input images		*/
long	nout;			/* Number of output files		*/
long	offset1[MAXIMG*NCHANS];	/* Offsets into "buffer" for input bands*/
long	offset2[NCHANS];	/* Offsets into "buffer" for output bands*/
long	status;			/* Status variable			*/
long	std_out;		/* Standard output			*/
long	totbnds;		/* Total input bands			*/
long	window[MAXIMG][4];	/* input window specification		*/

struct	CALSTR	calstr;		/* Calibration parameter structure	*/
struct	MODEL	model;		/* Model and orbital information        */
struct	PARBLK	vblock;		/* TAE paramter block			*/

struct COEFFSTR sumsqr;		/* Sum of squared values for each coeff */
struct COEFFSTR wtmean;		/* Weighted mean for each coeff         */
struct COEFFSTR stdev;		/* Std dev for each coeff               */

unsigned char	*buffer;	/* Image I/O buffer			*/


/* Initialize las
-----------------*/
c_inlas( &vblock, &std_out );


/* Get the input parameters
---------------------------*/
getpar( &vblock, taein, hosin, hosmnr, window, bands, nbands, &nimgs,
	&totbnds, taeout, hosout, channel, &maxchan, &msgtim, minmax,
	&model, &nout, &calstr, &mline );


/* Open the input image and create the output image
---------------------------------------------------*/
open_las( &vblock, hosin, &nimgs, hosout[0], window, bands, nbands, &totbnds,
	 &calstr.odtype, &model, fdesc, &gdesc, &nout, offset1, offset2,
	 &buffer );


/* Open the statistics and clipping statistics report file
-----------------------------------------------------------*/
report_open(hosout, &fptr, &sumsqr, &wtmean, &stdev);


/* Process the data
-------------------*/
process( &gdesc, &vblock, window[0], mline, channel, maxchan, minmax,
	omin, omax, &msgtim, hosmnr, hosout, &nout, clipact, cliplim,
	clipcnt, offset1, offset2, buffer, &model, &calstr, fptr,
	&sumsqr, &wtmean, &stdev );


/* Close the input and output files
-----------------------------------*/
status = c_egclse( (struct GDESC **)&gdesc );
if ( status != E_SUCC )
    c_lasmsg( FALSE, "avhrrcal-warn", "Nonfatal error encountered" );


/* Update the DDR information
-----------------------------*/
status = c_upddr( &vblock, (const char (*)[CMLEN])hosin, hosout[0], &nimgs,
	(const long (*)[4])window, (const long (*)[257])bands, nbands,
	&fone, &fone, &maxchan, &cflag );
if ( status != E_SUCC )
    c_lasmsg( FALSE, "avhrrcal-warn", "Nonfatal error encountered" );


/* Update the minimum and maximum data values in the DDR
--------------------------------------------------------*/
status = c_svmnmx( hosout[0], &totbnds, omin, omax, minmax );
if ( status != E_SUCC )
    c_lasmsg( FALSE, "avhrrcal-warn", "Nonfatal error encountered" );


/* Update the output history information
----------------------------------------*/
status = c_hstout(&vblock, hosout[0], (const char (*)[CMLEN])hosin, &nimgs," ");
if ( status != E_SUCC )
    c_lasmsg( FALSE, "avhrrcal-warn", "Nonfatal error encountered" );


/* Generate a clipping statistics report
----------------------------------------*/
clip_writ( fptr, &calstr, clipact, cliplim, clipcnt, channel,
	   maxchan, window[0][NL]*window[0][NS] );


/* Display a standard completion message
----------------------------------------*/
c_complt( "avhrrcal", taeout, &nout );


/* Put a return here to appease the compiler
----------------------------------------*/
return E_SUCC;

}
