/*
   Program to convert Rao coefficients to our EDC table format:

   Sample input file:
   ------------------

2001 13 Jun C1: Slope = 0.1428 Int = -5.8550 C2: Slope = 0.1653 Int = -6.7787
2001  8 May C1: Slope = 0.1423 Int = -5.8357 C2: Slope = 0.1649 Int = -6.7596
2001 10 Apr C1: Slope = 0.1420 Int = -5.8202 C2: Slope = 0.1645 Int = -6.7444

   Sample output:
   --------------

satellite:   14                                                               |
launch date: 94364                                                            |
date range: 01100 01127                                                       |
A   0.0        0.0                                                            |
B   0.0        0.0                                                            |
C   1.35887    1.85459                                                        |
D   0.0        0.0                                                            |
E   0.0        0.0                                                            |
F  40.98732   40.99939                                                        |
satellite:   14                                                               |
launch date: 94364                                                            |
date range: 01128 01163                                                       |
A   0.0        0.0                                                            |
B   0.0        0.0                                                            |
C   1.35601    1.85009                                                        |
D   0.0        0.0                                                            |
E   0.0        0.0                                                            |
F  41.00984   40.99212                                                        |
satellite:   14                                                               |
launch date: 94364                                                            |
date range: 01164 00000                                                       |
A   0.0        0.0                                                            |
B   0.0        0.0                                                            |
C   1.35126    1.84561                                                        |
D   0.0        0.0                                                            |
E   0.0        0.0                                                            |
F  41.00140   41.00847                                                        |

   Note that the ending date in the last date range must be manually changed
   from 00000 to whatever it should be.

*******************************************************************************


Explaination as to how we get from Rao published slope/intercept coefficients
to coefficients found in Rao calibration table for use at EDC:

K = 100 PI w / F  (equation for K value to convert radiance <-> albedo)

The use of the A B C D E F coefficients (relate to TIME):

a = 1 / (B * dsl + C)    dsl = days since launch
			 Note that the A term is not used.

a = 1 / C                Since Rao already takes into account dsl, make the
			 B term zero.

Co = E * dsl + F         Note that the D term is not used.

Co = F                   Since Rao already takes into account dsl, make the
			 E term zero.

Rao slope = K a

Rao int   = K (-Co) a

Consequently, using the above equations:

F = (-) Rao int / Rao slope

C = K / Rao slope

*******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define K1 0.19296
#define K2 0.30508

int convert( int year, int day, char *month );
int subone( int x );

int main( int argc, char *argv[] )
{
FILE *inf, *ouf;
float alpha, C0;
float c1, f1;
float c2, f2;
float slope1, intercept1;
float slope2, intercept2;
int start, end;
char str[120];
char buf[120];
int year, day, mon;
char month[4];
int len;

if ( argc != 3 )
    {
    printf( "\nUsage %s <infile> <outfile>\n\n", argv[0] );
    exit(1);
    }

inf = fopen( argv[1], "r" );
if ( !inf )
    {
    printf( "\nUnable to open file: %s\n\n", argv[1] );
    exit(1);
    }

ouf = fopen( argv[2], "w" );
if ( !ouf )
    {
    printf( "\nUnable to open file: %s\n\n", argv[2] );
    exit(1);
    }

end = 0;
while ( fgets( str, 120, inf ) != NULL )
    {
    sscanf( str, "%d %d %s %*s %*s %*s %f %*s %*s %f %*s %*s %*s %f %*s %*s %f",
        &year, &day, month, &slope1, &intercept1, &slope2, &intercept2 );
    
    start = convert( year, day, month );
    c1 = K1 / slope1;
    f1 = -intercept1 / slope1;
    c2 = K2 / slope2;
    f2 = -intercept2 / slope2;
    fprintf( ouf, "satellite:   14                                                               |\n" );
    fprintf( ouf, "launch date: 94364                                                            |\n" );
    fprintf( ouf, "date range: %05d %05d                                                       |\n", start, end );
    fprintf( ouf, "A   0.0        0.0                                                            |\n" );
    fprintf( ouf, "B   0.0        0.0                                                            |\n" );
    sprintf( buf, "C   %7.5f    %7.5f", c1, c2 );
    strncat( buf, "                                                                               ", 79 );
    buf[78] = '|';
    buf[79] = 0;
    fprintf( ouf, "%s\n", buf );
    fprintf( ouf, "D   0.0        0.0                                                            |\n" );
    fprintf( ouf, "E   0.0        0.0                                                            |\n" );

    sprintf( buf, "F  %7.5f   %7.5f", f1, f2 );
    strncat( buf, "                                                                               ", 79 );
    buf[78] = '|';
    buf[79] = 0;
    fprintf( ouf, "%s\n", buf );




    end = subone( start );
    }
return 0;
}

int convert( int year, int day, char *month )
{
int yr, mon, leap;
int dateval;

if ( year % 4 == 0 )
    leap = 1;
else
    leap = 0;

dateval = ( year % 100 ) * 1000;

if ( strcmp( month, "Jan" ) == 0 )
    mon = 1;
else if ( strcmp( month, "Feb" ) == 0 )
    mon = 2;
else if ( strcmp( month, "Mar" ) == 0 )
    mon = 3;
else if ( strcmp( month, "Apr" ) == 0 )
    mon = 4;
else if ( strcmp( month, "May" ) == 0 )
    mon = 5;
else if ( strcmp( month, "Jun" ) == 0 )
    mon = 6;
else if ( strcmp( month, "Jul" ) == 0 )
    mon = 7;
else if ( strcmp( month, "Aug" ) == 0 )
    mon = 8;
else if ( strcmp( month, "Sep" ) == 0 )
    mon = 9;
else if ( strcmp( month, "Oct" ) == 0 )
    mon = 10;
else if ( strcmp( month, "Nov" ) == 0 )
    mon = 11;
else if ( strcmp( month, "Dec" ) == 0 )
    mon = 12;

if ( mon == 1 )
    dateval += day;
else if ( mon == 2 )
    dateval += 31 + day;
else
    {
    if ( leap )
        dateval++;
    switch ( mon )
	{
	case  3 : dateval += 59 + day; break;
	case  4 : dateval += 90 + day; break;
	case  5 : dateval += 120 + day; break;
	case  6 : dateval += 151 + day; break;
	case  7 : dateval += 181 + day; break;
	case  8 : dateval += 212 + day; break;
	case  9 : dateval += 243 + day; break;
	case 10 : dateval += 273 + day; break;
	case 11 : dateval += 304 + day; break;
	case 12 : dateval += 334 + day; break;
	}
    }

return dateval;
}

int subone( int x )
{
int year, day;

year = x / 1000;
day = x % 1000;
day--;

if ( day == 0 )
    {
    year--;
    if ( year < 0 )
        year = 99;
    if ( year % 4 == 0 )
        day = 366;
    else
        day = 365;
    }

return year * 1000 + day;
}
