/*******************************************************************************
FUNCTION:	PROCESS

PURPOSE:	Process the input data and write it to the output file

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
1.0	02/99	D.Lloyd		Developed from HRPTCAL and MAKENCAL

ALGORITHM:

ALGORITHM REFERENCES
*******************************************************************************/
#include "dltf.h"
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "adapssup.h"
#include "avhrrcal.h"


void process
(
int	*gdesc,			/* Group descriptor used for image I/O	*/
struct	PARBLK *vblock,		/* TAE paramter block			*/
long	window[],		/* Window to process			*/
long	mline,			/* Image master line number		*/
long	channel[],		/* Channels to process			*/
long	maxchan,		/* Maximum number of channels to process*/
char	*minmax,		/* Min/max flag				*/
double	omin[NCHANS],		/* min pixel value array		*/
double	omax[NCHANS],		/* max pixel value array		*/
long	*msgtim,		/* Time between processing messages	*/
char	*hosmnr,		/* Input minor frame label services name*/
char	hosout[][CMLEN],       	/* Output host file names		*/
long	*nout,			/* Number of output files		*/
float	clipact[NCHANS][2],	/* Clipping limits (actual)		*/
float	cliplim[NCHANS][2],	/* Clipping limits (scaled)		*/
long	clipcnt[NCHANS][2],	/* Clipping statistics			*/
long	offset1[MAXIMG*NCHANS],	/* Offsets into "buffer" for input bands*/
long	offset2[NCHANS],	/* offsets into "buffer" output bands	*/
unsigned char	*buffer,	/* image I/O buffer			*/
struct	MODEL	*model,		/* Model				*/
struct	CALSTR	*calstr,	/* Calibration parameters structure	*/
FILE	*fptr,			/* Pointer to report file               */
struct COEFFSTR *sumsqr,	/* Sum of squared values for each coeff */
struct COEFFSTR *wtmean,	/* Weighted mean for each coeff		*/
struct COEFFSTR *stdev 		/* Std dev for each coeff		*/
)

{
char	char_buf[1];		/* buffer for label services calls	*/
char	dtype[DTYPL];		/* Data type for label services calls	*/
char	linekey[KEYL];		/* line number key for minor frame file	*/

extern	long	curent;		/* Current processing line		*/

long	minfd;			/* minor frame file descriptor		*/

long	lastline;		/* Last line when coeff were updated	*/
long	start_line;		/* Starting line for this calibration	*/
long	flag = ALLCAT;		/* Processing message format flag	*/
long	one = 1;		/* Constant 				*/
long	line;			/* Line counter				*/
long	minor_nbytes = MNR_WORDS*sizeof(short);	/* num bytes in min frm	*/
long	status;			/* Return status variable		*/
long	zero = 0;		/* value of zero			*/

short	min_frm[MNR_WORDS];	/* minor frame buffer			*/

struct	COEFFSTR coeff;		/* Calibration coefficient structure	*/



/* Open the minor frame file
----------------------------*/
status = c_lsopen( &minfd, hosmnr, &zero );
if ( status != E_SUCC )
    {
    c_lasmsg( FALSE, "avhrrcal-open", "Error opening minor frame file" );
    c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
    }


/* Initailize and start the processing messages
-----------------------------------------------*/
curent = 1;
#if NODEBUG
c_inittm( "AVHRRCAL", "Processed", "line", "bands", " ", &window[NL],
	&maxchan, msgtim, &flag );
c_startm();
#endif


/* Initialize the clipping report structures
--------------------------------------------*/
status = clip_init(calstr,clipact,cliplim,clipcnt);
if ( status != E_SUCC )
    {
    c_lsclos( &minfd, hosmnr, &zero );
    c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
    }


/* Process the input lines
--------------------------*/
start_line = lastline = 1;
for ( line = 1; line <= window[NL]; line++, curent++ )
    {
    /* Get a line of data
    ---------------------*/
    if ( c_estep( (struct GDESC **)gdesc ) != E_SUCC )
	{
	c_lsclos( &minfd, hosmnr, &zero );
	c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
	}

    /* Get calibration coefficients for this line
    ---------------------------------------------*/
    sprintf( linekey, "%d", line + mline );
    if ( c_lsread( &minfd, linekey, &one, &minor_nbytes, char_buf,
    		(unsigned char *)min_frm, dtype ) != E_SUCC )
	{
	c_lsclos( &minfd, hosmnr, &zero );
	c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
	}
    status = c_calcoeff( line, mline, min_frm, minfd, model, &coeff, calstr );
    if ( status != E_SUCC )
	{
	c_lsclos( &minfd, hosmnr, &zero );
        c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
	}

    /*
      If the sensor configuration has changed, finish the current statistics
      table, write it to the statistics report, and reinitialize the
      parameters for a new statistics table.  Ignore the "reconfiguration"
      occuring on the first line processed.
    */
    if ((calstr->reconfig == TRUE) && (line != 1))
        {
        report_fini(line,lastline,window[NL],calstr,&coeff,sumsqr,
                    wtmean,stdev);
        report_writ(fptr,wtmean,stdev,calstr,start_line,line-1,
                    (long)model->satnum);
        start_line = line;
        report_init(sumsqr,wtmean,stdev);
        }


    /*
      If the return status from ccupdt() is TRUE, update the report
      statistics
    */
    if (calstr->nprdflg == TRUE)
        {
        report_updt(line,&lastline,calstr,&coeff,sumsqr,wtmean);
        }


    /* Apply the coefficients to the line
    -------------------------------------*/
    status = c_appcoeff( model, calstr, window, channel, maxchan, minmax, 
    	omin, omax, buffer, offset1, offset2, &coeff );
    if ( status != E_SUCC )
	{
	c_lsclos( &minfd, hosmnr, &zero );
	c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
	}

    /* Update the clipping statistics
    ---------------------------------*/
    clip_loop(calstr, window, channel, maxchan, cliplim, clipcnt,
	      offset2, buffer);
    }


/* Close the minor frame
------------------------*/
c_lsclos( &minfd, hosmnr, &zero );

/* Write out the last line to the output image
----------------------------------------------*/
if ( c_estep( (struct GDESC **)gdesc ) != E_SUCC )
   c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );


/*
  Finish up stats of any remaining lines for stats report.
  Write the stat info to the report.
*/
report_fini(line,lastline,window[NL],calstr,&coeff,sumsqr,wtmean,stdev);
report_writ(fptr,wtmean,stdev,calstr,start_line,line-1,(long)model->satnum);


/*
  Write final statistical info to the report.
*/
report_clos(fptr,calstr);


/* Stop the processing messages
-------------------------------*/
#if NODEBUG
c_stoptm();
#endif

return;
}
