/******************************************************************************
FUNCTION:       REPORT_OPEN

PURPOSE:        Open the calibration report and initialize variables

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      01/99    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

ALGORITHM REFERENCES:   NONE

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "calcoef.h"
#include "avhrrcal.h"

#define REPORT 1

void report_open
(
char ohfiles[][CMLEN],	/* Calibration host file name	*/
FILE **fptr,		/* File pointer to report file	*/
struct COEFFSTR *sumsqr,/* Sum of squared values for each coeff	*/
struct COEFFSTR *wtmean,/* Weighted mean for each coeff		*/
struct COEFFSTR *stdev 	/* Std dev for each coeff		*/
)

{

/*
  Open the report file
*/
*fptr = fopen(ohfiles[REPORT],"w");
if (*fptr == NULL)
    c_lasmsg(TRUE, "avhrrcal-open", "Error opening report file");

fprintf(*fptr,"\n                           AVHRR calibration coefficient ");
fprintf(*fptr,"statistics     ");
fprintf(*fptr,"\n                          -------------------------------");
fprintf(*fptr,"-----------\n\n");
fprintf(*fptr,"\n  This AVHRR image segment was calibrated with ");
fprintf(*fptr,"the following coefficient\n");
fprintf(*fptr,"  statistics (for the purpose of calibration, a ");
fprintf(*fptr,"satellite sensor configuration \n");
fprintf(*fptr,"  change will define a new image segment) \n\n");

/*
  Initialize summary info
*/
report_init(sumsqr,wtmean,stdev);


return;
}

/******************************************************************************
FUNCTION:       REPORT_INIT

PURPOSE:        Initialize the calibration report

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      01/99    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

ALGORITHM REFERENCES:   NONE

*******************************************************************************/

void report_init
(
struct COEFFSTR *sumsqr,	/* Sum of squared values for each coeff	*/
struct COEFFSTR *wtmean,	/* Weighted mean for each coeff		*/
struct COEFFSTR *stdev 		/* Std dev for each coeff		*/
)

{
long i,k;		/* Counters	*/


/*
  Initialize summary info
*/
for (i = 0; i < MAXOPTICBND; i++)
    {
    for (k = 0; k < 2; k++)
	{
	wtmean->ocoeff.order1[i][k] = wtmean->ocoeff.order0[i][k] = 0.0;
	sumsqr->ocoeff.order1[i][k] = sumsqr->ocoeff.order0[i][k] = 0.0;
	stdev->ocoeff.order1[i][k]  = stdev->ocoeff.order0[i][k]  = 0.0;
	}
    }
for (i = 0; i < MAXTHERMBND; i++)
    {
    wtmean->tcoeff.order2[i] =
    wtmean->tcoeff.order1[i] =
    wtmean->tcoeff.order0[i] = 0.0;
    sumsqr->tcoeff.order2[i] =
    sumsqr->tcoeff.order1[i] =
    sumsqr->tcoeff.order0[i] = 0.0;
    stdev->tcoeff.order2[i] =
    stdev->tcoeff.order1[i] =
    stdev->tcoeff.order0[i] = 0.0;
    }

return;
}

/******************************************************************************
FUNCTION:       REPORT_UPDT

PURPOSE:        Update the calibration report.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      01/99    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

Optical channels:
	  Get the scaled coefficients and convert to gain and offset
	  values.  The scaled values are:
  		"gain": K*alpha
  		"offset": -K*alpha*C0
	  the values we want to report are:
  		Gain:   1/alpha = K / "gain"
  		Offset: C0      = ("offset" * Gain) / (-K)
	  where K = (PI / Fs) * 100 = (PI * W / F) * 100.
Thermal channels:
	  Get the coefficients and convert to gain and offset
	  values.  The values are:
  		"gain":   slope
  		"offset": intercept
*******************************************************************************/


void report_updt
(
long	line,			/* Line in image being processed	*/
long	*lastline,		/* Last line when coeff were updated	*/
struct CALSTR	*calstr,	/* Calibration parameters structure	*/
struct COEFFSTR	*coeff,		/* All coefficients (optical, thermal)	*/
struct COEFFSTR *sumsqr,	/* Sum of squared values for each coeff	*/
struct COEFFSTR *wtmean 	/* Weighted mean for each coeff		*/
)

{

long i,k;			/* Counters */

double	gain;			/* True gain value for summary report	*/
double	offset;			/* True offset value for summary report */

/*
  Optical channels
*/

for (i = 0; i < calstr->nopticbnds; i++)
    {
    for (k = 0; k < 2; k++)
	{
	gain   = calstr->Kvals[i] / coeff->ocoeff.order1[i][k];
	offset = coeff->ocoeff.order0[i][k] * gain / (-calstr->Kvals[i]);
	wtmean->ocoeff.order1[i][k] += (double)(line-(*lastline)) * gain;
	sumsqr->ocoeff.order1[i][k] += (double)(line-(*lastline)) * gain * gain;
	wtmean->ocoeff.order0[i][k] += (double)(line-(*lastline)) * offset;
	sumsqr->ocoeff.order0[i][k] += (double)(line-(*lastline)) * offset * offset;
	}
    }

/*
  Thermal channels
*/

for (i = calstr->nopticbnds; i < NCHANS; i++)
    {
    wtmean->tcoeff.order2[i-MINOPTICBND] += (double)(line-(*lastline)) *
					    coeff->tcoeff.order2[i-MINOPTICBND];
    sumsqr->tcoeff.order2[i-MINOPTICBND] += (double)(line-(*lastline)) *
					   coeff->tcoeff.order2[i-MINOPTICBND] *
					   coeff->tcoeff.order2[i-MINOPTICBND];
    wtmean->tcoeff.order1[i-MINOPTICBND] += (double)(line-(*lastline)) *
					    coeff->tcoeff.order1[i-MINOPTICBND];
    sumsqr->tcoeff.order1[i-MINOPTICBND] += (double)(line-(*lastline)) *
					   coeff->tcoeff.order1[i-MINOPTICBND] *
					   coeff->tcoeff.order1[i-MINOPTICBND];
    wtmean->tcoeff.order0[i-MINOPTICBND] += (double)(line-(*lastline)) *
					   coeff->tcoeff.order0[i-MINOPTICBND];
    sumsqr->tcoeff.order0[i-MINOPTICBND] += (double)(line-(*lastline)) *
					   coeff->tcoeff.order0[i-MINOPTICBND] *
					   coeff->tcoeff.order0[i-MINOPTICBND];
    }
*lastline = line;

return;
}

/******************************************************************************
FUNCTION:       REPORT_FINI

PURPOSE:        Finish up statistics of any remaining lines for
		the calibration report.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      01/99    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

*******************************************************************************/


void report_fini
(
long	line,			/* Line in image being processed	*/
long	lastline,		/* Last line when coeff were updated	*/
long	nl,			/* Number of lines in image to process	*/
struct CALSTR	*calstr,	/* Calibration parameters structure	*/
struct COEFFSTR	*coeff,		/* All coefficients (optical, thermal)	*/
struct COEFFSTR *sumsqr,	/* Sum of squared values for each coeff	*/
struct COEFFSTR *wtmean,	/* Weighted mean for each coeff		*/
struct COEFFSTR *stdev 		/* Std dev for each coeff		*/
)

{

long i,k;			/* Counters */

double	gain;			/* True gain value for summary report	*/
double	offset;			/* True offset value for summary report */

/*
  Finish up stats for any remaining lines
*/
for (i = 0; i < calstr->nopticbnds; i++)
    {
    for (k = 0; k < 2; k++)
	{
	gain   = calstr->Kvals[i] / coeff->ocoeff.order1[i][k];
	offset = coeff->ocoeff.order0[i][k] * gain / (-calstr->Kvals[i]);
	wtmean->ocoeff.order1[i][k] += (double)(line-lastline) * gain;
	sumsqr->ocoeff.order1[i][k] += (double)(line-lastline) * gain * gain;
	wtmean->ocoeff.order0[i][k] += (double)(line-lastline) * offset;
	sumsqr->ocoeff.order0[i][k] += (double)(line-lastline) * offset *
						offset;
	}
    }
for (i = calstr->nopticbnds; i < NCHANS; i++)
    {
    wtmean->tcoeff.order2[i-MINOPTICBND] += (double)(line-lastline) *
					   coeff->tcoeff.order2[i-MINOPTICBND];
    sumsqr->tcoeff.order2[i-MINOPTICBND] += (double)(line-lastline) *
					   coeff->tcoeff.order2[i-MINOPTICBND] *
					   coeff->tcoeff.order2[i-MINOPTICBND];
    wtmean->tcoeff.order1[i-MINOPTICBND] += (double)(line-lastline) *
					   coeff->tcoeff.order1[i-MINOPTICBND];
    sumsqr->tcoeff.order1[i-MINOPTICBND] += (double)(line-lastline) *
					   coeff->tcoeff.order1[i-MINOPTICBND] *
					   coeff->tcoeff.order1[i-MINOPTICBND];
    wtmean->tcoeff.order0[i-MINOPTICBND] += (double)(line-lastline) *
					   coeff->tcoeff.order0[i-MINOPTICBND];
    sumsqr->tcoeff.order0[i-MINOPTICBND] += (double)(line-lastline) *
					   coeff->tcoeff.order0[i-MINOPTICBND] *
					   coeff->tcoeff.order0[i-MINOPTICBND];
    }

/*
  Update the summary report
*/
for (i = 0; i < calstr->nopticbnds; i++)
    {
    for (k = 0; k < 2; k++)
	{
	wtmean->ocoeff.order1[i][k] /= nl;
	wtmean->ocoeff.order0[i][k] /= nl;
	stdev->ocoeff.order1[i][k] = (double)((sumsqr->ocoeff.order1[i][k]-nl
				      *wtmean->ocoeff.order1[i][k]
				      *wtmean->ocoeff.order1[i][k])/nl);
	stdev->ocoeff.order1[i][k] = sqrt(fabs(stdev->ocoeff.order1[i][k]));
	stdev->ocoeff.order0[i][k] = (double)((sumsqr->ocoeff.order0[i][k]-nl
			    *wtmean->ocoeff.order0[i][k]
			    *wtmean->ocoeff.order0[i][k])/nl);
	stdev->ocoeff.order0[i][k] = sqrt(fabs(stdev->ocoeff.order0[i][k]));
	}
    }

for (     ; i < NCHANS; i++)
    {
    wtmean->tcoeff.order2[i-MINOPTICBND] /= nl;
    wtmean->tcoeff.order1[i-MINOPTICBND] /= nl;
    wtmean->tcoeff.order0[i-MINOPTICBND] /= nl;
    stdev->tcoeff.order2[i-MINOPTICBND] = (double)
				  ((sumsqr->tcoeff.order2[i-MINOPTICBND]-nl
				    *wtmean->tcoeff.order2[i-MINOPTICBND]
				    *wtmean->tcoeff.order2[i-MINOPTICBND])/nl);
    stdev->tcoeff.order2[i-MINOPTICBND] =
				sqrt(fabs(stdev->tcoeff.order2[i-MINOPTICBND]));
    stdev->tcoeff.order1[i-MINOPTICBND] = (double)
				  ((sumsqr->tcoeff.order1[i-MINOPTICBND]-nl
				    *wtmean->tcoeff.order1[i-MINOPTICBND]
				    *wtmean->tcoeff.order1[i-MINOPTICBND])/nl);
    stdev->tcoeff.order1[i-MINOPTICBND] =
				sqrt(fabs(stdev->tcoeff.order1[i-MINOPTICBND]));
    stdev->tcoeff.order0[i-MINOPTICBND] =
			(double)((sumsqr->tcoeff.order0[i-MINOPTICBND]-nl
			    *wtmean->tcoeff.order0[i-MINOPTICBND]
			    *wtmean->tcoeff.order0[i-MINOPTICBND])/nl);
    stdev->tcoeff.order0[i-MINOPTICBND] =
				sqrt(fabs(stdev->tcoeff.order0[i-MINOPTICBND]));
    }

return;
}

/******************************************************************************
FUNCTION:       REPORT_WRIT

PURPOSE:        Write statistics to the calibration report.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      01/99    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

*******************************************************************************/

void report_writ
(
FILE 	*fptr,			/* file descriptor for report file	*/
struct	COEFFSTR *wtmean,	/* Weighted mean for each coeff		*/
struct	COEFFSTR *stdev,	/* Std dev for each coeff		*/
struct CALSTR	 *calstr,	/* Calibration parameters structure	*/
long	start_line,		/* Starting line of the cal period	*/
long	end_line,		/* Ending line of the cal period	*/
long	satnum 			/* Satellite number (14 = noaa14)	*/
)

{
long 	i,k;			/* loop counter				*/
long	numoptic;		/* Number of optical bands in this cal	*/
long	numtherm;		/* Number of thermal bands in this cal	*/

/*
  Determine the number of optical/thermal bands in this calibration period.
  If this routine is being called because the sensors were just reconfigured,
  we need to swap the numbers stored in the calibration structure.  If this
  routine is being called because the image is ending, the calibration
  structure is reflecting the current sensor configuration, so don't swap
  the numbers in the structure.
*/
if (calstr->reconfig == TRUE)
    {
    numoptic = calstr->nthermbnds;
    numtherm = calstr->nopticbnds;
    }
else
    {
    numoptic = calstr->nopticbnds;
    numtherm = calstr->nthermbnds;
    }

/* Write the preliminary information
   ----------------------------------*/
fprintf(fptr,"\n\n");
fprintf(fptr,"************************************************");
fprintf(fptr,"*******************************\n\n");
fprintf(fptr,"   Starting line: %5d       Number of optical channels: %d \n",
	start_line,numoptic);
fprintf(fptr,"   Ending line  : %5d       Number of thermal channels: %d \n",
	end_line,numtherm);

fprintf(fptr,"\n\n");

/*
  If we have a low and high albedo coefficient values, print the heading.
  Otherwise, print the general "optical" heading.
*/
if (satnum > 14)
    {
    fprintf(fptr,"                 LOW albedo range statistics");
    fprintf(fptr," (optical channels)\n\n");
    }
else
    fprintf(fptr,"                           Optical channel statistics\n\n");

/*
  Print the optical channel statistics
*/
fprintf(fptr,"                       Gain                            Offset\n");
fprintf(fptr,
"Channel        Mean          Std Dev              Mean          Std Dev\n");
k = 0;
for (i = 0; i < numoptic; i++)
    fprintf(fptr,"   %1d   % 1.9e % 1.9e   % 1.9e % 1.9e \n",
		i+1,wtmean->ocoeff.order1[i][k],stdev->ocoeff.order1[i][k],
		wtmean->ocoeff.order0[i][k],stdev->ocoeff.order0[i][k]);

/*
  If we have a low and high albedo coefficient values, print the high values
  for the optical channels.
*/
if (satnum > 14)
    {
    fprintf(fptr,"\n\n");
    fprintf(fptr,"                 HIGH albedo range statistics");
    fprintf(fptr," (optical channels)\n\n");
    fprintf(fptr,
	    "                       Gain                            Offset\n");
    fprintf(fptr,"Channel        Mean          Std Dev");
    fprintf(fptr,"              Mean          Std Dev\n");
    k = 1;
    for (i = 0; i < numoptic; i++)
	fprintf(fptr,"   %1d   % 1.9e % 1.9e   % 1.9e % 1.9e \n",
		i+1,wtmean->ocoeff.order1[i][k],stdev->ocoeff.order1[i][k],
		wtmean->ocoeff.order0[i][k],stdev->ocoeff.order0[i][k]);
    }

/*
  Finally, print out the thermal channel statistics
*/
fprintf(fptr,"\n\n");
fprintf(fptr,"                           Thermal channel statistics\n\n");
fprintf(fptr,"                     Offset (order 0 coefficients)\n");
fprintf(fptr,"Channel        Mean          Std Dev\n");
k = 0;
for (i = numoptic; i < NCHANS; i++)
    fprintf(fptr,"   %1d   % 1.9e % 1.9e\n",
		i+1,wtmean->tcoeff.order0[i-MINOPTICBND],
		stdev->tcoeff.order0[i-MINOPTICBND]);

fprintf(fptr,"\n\n");
fprintf(fptr,"                      Gain  (order 1 coefficients)\n");
fprintf(fptr,"Channel        Mean          Std Dev\n");
k = 0;
for (i = numoptic; i < NCHANS; i++)
    fprintf(fptr,"   %1d   % 1.9e % 1.9e\n",
		i+1,wtmean->tcoeff.order1[i-MINOPTICBND],
		stdev->tcoeff.order1[i-MINOPTICBND]);

fprintf(fptr,"\n\n");
fprintf(fptr,"                    Quadratic (order 2 coefficients)\n");
fprintf(fptr,"Channel        Mean          Std Dev\n");
k = 0;
for (i = numoptic; i < NCHANS; i++)
    fprintf(fptr,"   %1d   % 1.9e % 1.9e\n",
		i+1,wtmean->tcoeff.order2[i-MINOPTICBND],
		stdev->tcoeff.order2[i-MINOPTICBND]);


fprintf(fptr,"\n\n");

return;
}

/******************************************************************************
FUNCTION:       REPORT_CLOS

PURPOSE:        Write final statistical information.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      01/99    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

*******************************************************************************/

void report_clos
(
FILE 	*fptr,			/* file descriptor for report file	*/
struct CALSTR *calstr		/* Calibration parameters structure	*/
)

{

/*
  Print suffix report information
*/
fprintf(fptr,"\n\n");
fprintf(fptr,
	" The Gain and Offset values for optical channels were calculated\n"); 
if (strcmp(calstr->calmethod,"teillet") == 0)
    {
    fprintf(fptr,
	" using a linear function based on days since launch using \n");
    fprintf(fptr," Teillet-Holben recommended values. \n\n\n");
    }
else if (strcmp(calstr->calmethod,"vermote") == 0)
    {
    fprintf(fptr,
	" using a linear function based on days since launch using \n");
    fprintf(fptr," Vermote recommended values. \n\n\n");
    }
else if (strcmp(calstr->calmethod,"rao") == 0)
    {
    fprintf(fptr,
	" using a linear function based on days since launch using \n");
    fprintf(fptr," Rao recommended values. \n\n\n");
    }
else if (strcmp(calstr->calmethod,"prelaunch") == 0)
    {
    fprintf(fptr, " from NOAA's prelaunch values. \n\n\n");
    }
else
    {
    fprintf(fptr, " using Kaufman-Holben desert values.\n\n\n");
    }
fprintf(fptr,"************************************************");
fprintf(fptr,"*******************************\n\n");

return;
}

