/*******************************************************************************
NAME:    AVHRRDB

PURPOSE: A data base file is created by extracting information from the LAS
	 associated ADDR file.  The data base file name is the scene id which
	 is also created from the ADDR information.  Pipes (broken bars) are
	 used as field separators, standard separator for the data base.

         NOTE: If ANY change is made in the order of the fields or adding new
	       field to the data base file, that change MUST be made in the
	       production data base spec file also!  

PROGRAM HISTORY:

Version  Programmer       Date   Modification
-------  ---------------  -----  -----------------------------------------------
  1.0    B.Rieger         02/87  Original development for ADAPS
  1.1    E.Cihak          11/91  Updated to LAS--5.0 standards and modified to
                                 read the update information from the header.
  1.2    J.Fenno          09/92  Change the orbit field in the header structure
				 to sorbit.
  1.3    T.Baltzer        08/93  Added format "9" for stitched images
  1.3a   T.Baltzer        10/93  Take format "9" out again!  A scene becomes
				 format "9" when it has been stitched into
				 a half orbit.
  1.4    K.Lowell	  06/96  Added 'c','n' as valid first position name
				 characters, and '_' as a valid second position
				 name character in support of AGRHYMET.
  1.5	 D.Lloyd	  07/98  Modified for Y2K compliance
        
PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "util.h"
#include "adapssup.h"
#include "avhrrdb.h"

main()
{
FILE 	*fp;			/* File pointer to the data base file */

struct PARBLK vblock;	   	/* Block with pdf variables */
struct tm *lctm;	   	/* Local time structure */
struct MODEL	model;		/* Model and orbital information */

char 	hosin[1][CMLEN],        /* Input image name */  
	dbnam[CMLEN],		/* Database output file name */
	dir[CMLEN],		/* Directory part of hosin */
	ext[CMLEN],		/* File name extension part of hosin */
	filename[CMLEN],	/* File name part of hosin */
	tapeid[7],		/* Tape id  */
	sceneid[25],		/* Scene id */
	tstamp[7],		/* Time that the browse was created	     */
	mgrcode[3],		/* Management code based on data type */
        chans[6],               /* Channel numbers in image */
	curr_date[9];		/* Current date (mm/dd/yy) */

short	valid;			/* Loop control flag */

long	stdlun,			/* Logical unit number */
	zero = 0,		/* Constant zero */
	one = 1,		/* Constant one */
	len,			/* Length of the sceneid */
	time_val,		/* Time variable */
        degs,mins,secs,		/* Degrees, minutes, and seconds */
	dbflag,			/* Data base temp or daily file flag */
        i;                      /* Loop counter */

double  temp;			/* Temporary for coordinate conversion */

/* Initialization 
-----------------*/ 
c_inlas(&vblock,&stdlun);

/* Get the parameters from the variable block
---------------------------------------------*/
getpar(&vblock,hosin[0],&model,tstamp,tapeid,&dbflag);
 
/* Build the sceneid:
   If the user specified file name contains a valid scene ID then extract the
   16 character scene ID, otherwise create the scene ID from the AVHRR model
   structure.  Valid scene IDs must be 16 or 23 characters long, the first
   character must be an 'a','n', or 'c'
   the second must be a 'g', 'h', 'l', '_', or 'o', and
   characters 3 through 16 must be digits.
------------------------------------------------------------------------------*/
c_splitname((const char *)hosin, dir, filename, ext);
i = (long)strlen(filename);
valid = (((i == 16) || (i == 23)) &&
    ((filename[0] == 'a') || (filename[0]=='n') || (filename[0]=='c')) &&
    ((filename[1] == 'g') || (filename[1] == 'h') || (filename[1]=='_') ||
     (filename[1] == 'l') || (filename[1] == 'o')));
for (i = 2; (i < 16) && valid; i++)
    {
    if ((filename[i] < '0') || (filename[i] > '9'))
        valid = FALSE;
    }
if (valid)
    {
    strcpy(sceneid, filename);
    sceneid[16] = '\0';
    }
else
    {
    c_getscene(&model, sceneid);
    /* check for a stitched orbit (station id = "KM*") */
    if (strncmp(model.station,"KM",2) == 0)
       sceneid[1] = 'o';
    }


/* Set the manager code
-----------------------*/
if (model.dtype == GAC)
    strcpy(mgrcode,"52");
else if (model.dtype == LAC)
    strcpy(mgrcode,"51");
else if (model.dtype == HRPT)
    strcpy(mgrcode,"32");
else
    strcpy(mgrcode,"99");


/* Open the data base file
--------------------------*/
opendb(&fp,sceneid,tstamp,&dbflag,dbnam);

/* Upcase the sceneid for writting to the records
-------------------------------------------------*/
len = (long)strlen(sceneid);
c_low2up(sceneid,&len);

/*
  START THE AVHRR DATABASE SOURCE RECORD
*/

/* Start with the scene id
--------------------------*/
fprintf(fp,"%16s|",sceneid);

/* Add the ending time of acquisition
-------------------------------------*/ 
fprintf(fp,"%02d%02d%02d|",model.ehour,model.emin,model.esec);

/* Get the current date.  Add one to month because it's zero relative 
---------------------------------------------------------------------*/
time (&time_val);
lctm = localtime (&time_val);
/**************************************************
DLLOYD Y2K
  lctm->tm_year = 100 in year 2000 so MOD with 100
***************************************************/
sprintf(curr_date,"%02d/%02d/%02d",lctm->tm_mon+1,lctm->tm_mday,
		lctm->tm_year % 100 );

/* Date entered and last date updated are the same
--------------------------------------------------*/
fprintf(fp,"%8s|%8s|",curr_date,curr_date);

/* Day/Night flag
-----------------*/
if (model.day_night == DAY)
    fprintf(fp,"D|");
else 
    fprintf(fp,"N|");

/* Ascending/Descending flag
----------------------------*/
if (model.spass == ASCENDING)
    fprintf(fp,"A|");
else
    fprintf(fp,"D|");

/* Equitorial Crossing (deg, mins)-seconds are not transfered
------------------------------------------------------------*/
temp = model.eqcrs * DEG_PER_RAD;
c_cvtdms(&temp,&degs,&mins,&secs);
fprintf(fp,"%04d%02d|",degs,mins);

/* Add the management code
--------------------------*/
fprintf(fp,"%2s|",mgrcode);

/* Orbit number
---------------*/
fprintf(fp,"%5d|",model.sorbit);

/* Pass duration   
----------------*/
mins = 60 * (model.ehour + (model.ehour < model.shour ? 24 : 0) -
	     model.shour);
if (model.emin > model.smin)
    mins += (model.emin - model.smin);
else
    mins -= (model.smin - model.emin);
fprintf(fp,"%3d|",mins);

/* Record status.  This is a Master record
------------------------------------------*/
fprintf(fp,"M|");

/* Number of lines in the image
-------------------------------*/
fprintf(fp,"%5d|",model.nlines);

/* Number of times ordered
--------------------------*/
fprintf(fp,"%4d",zero);

/* Fill the record out to 80 characters.
   Fixed length records are required for tape transfer.
-------------------------------------------------------*/
fprintf(fp," \n");

/*
  START THE AVHRR ARCHIVE DATABASE RECORD
*/

/* Start with the sceneid
-------------------------*/
fprintf(fp,"%16s|",sceneid);

/* Contributor of the input data
fprintf(fp,"%2d|",model.contrib);
*/

/* Write out constants. Tape storage location (May not be known yet),
   File number, Format, and the Archive status.
--------------------------------------------------------------------*/
fprintf(fp,"%6s|01|08|G|",tapeid);

/* Band number in the input image
---------------------------------*/
for (i = 0; i < NCHANS; i++)
    sprintf(&chans[i],"%d",model.chans[i]); 

fprintf (fp,"%5s|",chans);

/* Current add date and last date updated are the same
------------------------------------------------------*/
fprintf(fp,"%8s|%8s|",curr_date,curr_date);

/* ???
------*/
fprintf(fp,"%3d|",one);

/* Set the station code
-----------------------*/
fprintf(fp,"%3s|",model.station);

/* Write out the browse time stamp
----------------------------------*/
fprintf(fp,"%6s",tstamp);

/* Fill the record out to 80 characters.
   Fixed length records are required for tape transfer.
-------------------------------------------------------*/
fprintf(fp,"         \n");


/*
  START THE AVHRR DATABASE GEO-LOCATION RECORDS
*/

/* North West coordinate
------------------------*/
fprintf(fp,"%16s|NW|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.nw[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.nw[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* North Nadir coordinate
------------------------*/
fprintf(fp,"%16s|NN|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.nn[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.nn[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* North East coordinate
------------------------*/
fprintf(fp,"%16s|NE|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.ne[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.ne[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* Center West coordinate
------------------------*/
fprintf(fp,"%16s|CW|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.cw[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.cw[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* Center Nadir coordinate
--------------------------*/
fprintf(fp,"%16s|CN|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.cn[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.cn[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* Center East coordinate
------------------------*/
fprintf(fp,"%16s|CE|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.ce[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.ce[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* South West coordinate
------------------------*/
fprintf(fp,"%16s|SW|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.sw[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.sw[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* South Nadir coordinate
-------------------------*/
fprintf(fp,"%16s|SN|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.sn[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.sn[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* South East coordinate
------------------------*/
fprintf(fp,"%16s|SE|%8s|%8s|",sceneid,curr_date,curr_date);

/* Latitude
-----------*/
temp = model.se[0] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%3d%02d|",degs,mins);

/* Longitude
------------*/
temp = model.se[1] * DEG_PER_RAD; 
c_cvtdms(&temp,&degs,&mins,&secs); 
fprintf(fp,"%4d%02d|",degs,mins);

/* Fill to 80
-------------*/
fprintf(fp,"                            \n");

/* Close the file  
-----------------*/
fclose(fp);

/* Output completion message
----------------------------*/
c_complt("AVHRRDB",(char (*)[256])dbnam,&one);

/* Put a return here to make the compiler happy
------------------------------------------------*/
return(SUCCESS);

}
