/*******************************************************************************


NAME:		AVHRRIN

PURPOSE:   	Reads and converts an AVHRR archive image to a LAS image.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    04/91  D.VanderZee      Development
  5.0    10/91  E.Cihak          Finished conversion, debugging, and testing
  5.1    02/92  D.Hollaren       Updated windowing by latitude range
  5.2    04/93  J.Fenno          Modified to read archive images directly from
				 tape.  Also made changes to incorporate the
				 new archio support routines.
  5.3    03/95  T.Baltzer        Modified to reinitialize the model if the image
				 is windowed:
				 1) The model is updated to reflect only the
				    lines within the windowed portion of the
				    image.  This model is saved in the output
				    addr file.
				 2) The master line is always set to zero, the
				    master sample is set appropriately.
				 3) The line counter in the minor frame data
				    starts at "1".
				 4) Also made mod so that OFFSET parameter for
				    -latrange subcmd correctly calculates
				    window[SS], window[NS].
  5.4    09/97  T.Baltzer        Check return status from c_acqtmcor() call.
  6.0    10/97  T.Baltzer        Modified drastically to call c_windmod().  Do
				 this to fix bug wherein the start/end times
				 were not being adjusted due to a new NBS offset


PROJECT:	ADAPS 
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "tapeio.h"
#include "adaps.h"
#include "archio.h"
#include "avhrrin.h"


/* The following variables have been declared global for image I/O routines
------------------------------------------------------------------------------*/
struct FDESC *fdesc[2];	      /* image file descriptors used for image I/O    */
struct GDESC *gdesc;	      /* group descriptor used for image I/O          */
unsigned char *imgbuf;	      /* buffer for image I/O                         */
long out_ptr[NCHANS];  	      /* pointer into "imgbuf" for output bands       */


int main (void)
{
extern long curent;           /* Current processing line                      */

char	comment[CMLEN],       /* Tape mounting comment                        */
        hosin[1][CMLEN],      /* Host names of input images                   */
	hosout[CMLEN],        /* Host name of output image                    */
	outfiles[6][CMLEN],   /* List of output files for cleanup             */
	taeout[CMLEN];        /* TAE name of output image                     */

double	maxval[5],            /* Max values for output image                  */
	minval[5];            /* Min values for output image                  */

long 	access,               /* File access flag                             */
	adesc,                /* Archive image descriptor record              */
	b_no,                 /* Nominal band number                          */
	bands[MAXBND+1],      /* Bands to be processed - all images           */
	dtype       = EWORD,  /* Image data type                              */
	i,                    /* Loop control                                 */
	mmflg,                /* Compute min max flag                         */
	mode        = HANDLE, /* Error handling flag for tape I/O routines    */
	msg_frmt    = 0,      /* Format flag for processing message           */
	msgtim,               /* Elapsed time between display msg             */
	n_outfiles  = 0,      /* Number of output files                       */
	nbands,               /* Number of bands per input image              */
	rewoffl     = REWOFFL,/* Rewind option for tape I/O routines          */
	status,               /* Return status                                */
	std_out,              /* LUN to standard output                       */
	tpunit      = NULL,   /* Tape unit for images being read from tape    */
	window[4],            /* Window for each input image                  */
	zero        = 0;      /* Number zero for passing by address           */

struct BDDR    bddr[NCHANS];  /* LAS band ddr's                               */
struct DDR     ddr;           /* LAS image ddr                                */
struct MODEL   model;         /* AVHRR model information                      */
struct PARBLK  vblock;        /* TAE parameter block                          */


/* Initialization
-----------------*/
c_inlas(&vblock, &std_out);


/* Set up for processing 
------------------------*/
status = setup(&vblock, &model, comment, hosin[0], hosout, outfiles, taeout,
    bands, &mmflg, &msgtim, &n_outfiles, &nbands, &tpunit, window);
if (status != E_SUCC)
    {
    if (tpunit != NULL) c_tpclos(&tpunit, &mode, &rewoffl, &zero);
    c_clean (&vblock, "avhrrin", (const char (*)[256])outfiles, &n_outfiles);
    }
if (mmflg)
    c_inmnmx(&dtype, &nbands, minval, maxval);


/* Initialize processing message utility 
----------------------------------------*/
c_inittm("AVHRRIN", "Processed", "line", "bands", " ",
    &window[2], &nbands, &msgtim, &msg_frmt);


/* Open the AVHRR archive file
------------------------------*/
access = IREAD;
status = c_arcopnrd(hosin[0], &access, window, &model, &tpunit, &adesc);
if (status != E_SUCC)
    c_clean (&vblock, "avhrrin", (const char (*)[256])outfiles, &n_outfiles);


/* Copy and convert the AVHRR archive image to LAS format.  All ascending
   images located on tape must be processed as the satellite acquired them.
   This implies that we must use random LAS I/O to create the LAS image which
   is much slower than sequential I/O.  All other images can be processed in a
   "North-up" order allowing us to use sequential I/O.
------------------------------------------------------------------------------*/
if ((tpunit != NULL) && (model.cpass == ASCENDING))
    status = convert_rev(hosout, outfiles, &adesc, bands, &mmflg,
	&n_outfiles, &nbands, window, maxval, minval);
else
    status = convert_seq(hosout, outfiles, &adesc, bands, &mmflg,
	&n_outfiles, &nbands, window, maxval, minval);
c_arclose(hosin[0],&adesc);
if (status != E_SUCC)
    c_clean(&vblock, "avhrrin", (const char (*)[256])outfiles, &n_outfiles);


/* Update DDR for output image
------------------------------*/
c_lsmknm(hosout, "ddr", outfiles[n_outfiles]);
n_outfiles++;
if (c_getddr(hosout, &ddr) != E_SUCC)
    c_clean(&vblock, "avhrrin", (const char (*)[256])outfiles, &n_outfiles);


/* Set the master line and sample values.  For ascending passes that
   are being processed in satellite view we must flip the image.
   Always set the master line to 0 (if the image is being windowed,
   the new model saved in the .addr is relative to the new specified
   starting line number).
--------------------------------------------------------------------*/
if ((model.cpass == ASCENDING) && model.sat_view)
    ddr.master_sample = model.nsamples - (window[SS] + window[NS] - 1);
else
    ddr.master_sample = window[SS] - 1;
ddr.master_line = 0;


/* Set the line and sample increment values
-------------------------------------------*/
if (model.dtype == GAC)
    {
    ddr.line_inc = 4.0;
    ddr.sample_inc = 5.0;
    }
else
    {
    ddr.line_inc = 1.0;
    ddr.sample_inc = 1.0;
    }
ddr.valid[7] = VALID;  		/* line_inc and samp_inc validity flag */

for (i = 0; i < nbands; i++)
    {
    /* Get band specific DDR info
    -----------------------------*/
    b_no = i + 1;
    if (c_getbdr(hosout, &bddr[i], &b_no) != E_SUCC)
	c_clean(&vblock, "avhrrin", (const char (*)[256])outfiles, &n_outfiles);


    /* Fill in appropriate data fields
    ----------------------------------*/ 
    bddr[i].bandno = b_no;
    if (mmflg)
	{
	bddr[i].minval = minval[i];
	bddr[i].maxval = maxval[i];
	bddr[i].valid = VALID;
	}

    sprintf (bddr[i].source, "AVHRR Channel %d", bands[i]);
    if (model.cpass == ASCENDING)
	sprintf (bddr[i].direction, "Ascending");
    else 
	sprintf (bddr[i].direction, "Descending");
    }


/* Put DDR 
----------*/
if (c_putddr(hosout, &ddr) != E_SUCC)
    c_clean(&vblock, "avhrrin", (const char (*)[256])outfiles, &n_outfiles);
for (i = 0; i < nbands; i++)
    if (c_putbdr(hosout, &bddr[i]) != E_SUCC)
	c_clean(&vblock, "avhrrin", (const char (*)[256])outfiles, &n_outfiles);


/* Update history file 
----------------------*/
c_lsmknm(hosout, "his", outfiles[n_outfiles]);
n_outfiles++;
c_hstout(&vblock, hosout, (const char (*)[256])hosin[0], &zero, " ");


/* Output completion message 
----------------------------*/
c_complt("AVHRRIN", outfiles, &n_outfiles);


/* Return a value here to make the compiler happy
-------------------------------------------------*/
return(SUCCESS);

}
