/*******************************************************************************
NAME:		convert_rev

PURPOSE:	Convert an AVHRR archive image to a LAS image.  This "convert
		in reverse order" routine is used to process ascending images
		that are being read from tape.  These images must be processed
		in "satellite view" so that we do not have to back skip records
		on a tape device.  This implies that the LAS image must be
		written backwards using random LAS I/O.

		Return E_SUCC if successful, E_FAIL otherwise.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "vll.h"
#include "adaps.h"
#include "archio.h"
#include "adapsutil.h"


/* The following variables have been declared global for image I/O routines
------------------------------------------------------------------------------*/
extern long curent;             /* Current processing line                    */


int convert_rev
(
    char *hosout,              /* Host file name of output image             */
    char (*outfiles)[CMLEN],   /* Array of output file names                 */
    long *adesc,               /* AVHRR archive descriptor                   */
    long bands[],              /* Band specifications for each input image   */
    long *mmflg,               /* Min max tracking flag                      */
    long *n_outfiles,          /* Number of output files created             */
    long *nbands,              /* Number of bands per input image            */
    long window[],             /* Window specifications for each input image */
    double maxval[],           /* Max pixel value array                      */
    double minval[]            /* Min pixel value array                      */
)
{
long	access       = IWRITE, 	/* I/O open access flag                       */
	dtype        = EWORD,	/* Data type of the LAS image                 */
	i, j, k,		/* Loop control                               */
	line,                   /* Number of the current line being written   */
	mnr_fd,			/* HRPT minor frame data file descriptor      */
        mnr_nbytes,             /* Number of bytes in an HRPT minor frame     */
	nonfatal     = 1,	/* Nonfatal error flag                        */
	npixels,                /* Number of pixels in an archive line        */
	one          = 1,	/* Number of bytes to move file pointer       */
	open_opt[3],		/* Options for opening a LAS image            */
	status,			/* Return status variable                     */
	zero         = 0;	/* Zero constant                              */

struct FDESC *fdesc[MAXIMG+1];	/* File descriptor for the LAS image	      */

short	*arcbuf,		/* Ptr to buffer of pixel data from AVHRR     */
	*imgbuf[NCHANS],        /* Buffer for writing the LAS image           */
	*mnrbuf,		/* Buffer containing minor frame data         */
	*mnrptr1,		/* Ptr to Minor Frame buffer from c_arcread() */
	*mnrptr2;		/* Ptr into local Minor Frame buffer          */

char	char_buf[1],		/* Character buffer for label services calls  */
	linekey[KEYL],		/* Line number key for minor frame data file  */
	mnr_fname[CMLEN];	/* HRPT minor frame data file name            */



/* Allocate space to store all of the minor frame data in memory.  Because we
   are processing the archive image backwards and the minor frame data must be
   stored in the same order that they appear in the LAS image, we must store
   all of this data in memory and write it when the conversion is finished.
------------------------------------------------------------------------------*/
mnr_nbytes = MNR_WORDS * sizeof(short);
mnrbuf = (short *)malloc(window[NL] * mnr_nbytes);
if (mnrbuf == NULL)
    {
    c_errmsg("Error allocating memory", "avhrrin-alloc", &nonfatal);
    return(E_FAIL);
    }


/* Open the LAS image for random I/O.  NOTE: because we are writing the LAS
   image backwards, we cannot create a compressed image.  To create a
   compressed image we would have to write the lines in correct order.
---------------------------------------------------------------------------*/
open_opt[0] = open_opt[1] = open_opt[2] = 0;
status = c_eopenr(fdesc, hosout, nbands, &one, &one, &window[NL],
    &window[NS], &access, &dtype, open_opt, &one);
if (status != E_SUCC)
    {
    free(mnrbuf);
    return(E_FAIL);
    }


/* Keep track of output files for cleanup
-----------------------------------------*/
strcpy (outfiles[*n_outfiles], hosout);
*n_outfiles += 1;


/* Allocate memory for image I/O 
--------------------------------*/
for (i = 0; i < *nbands; i++)
    {
    imgbuf[i] = (short *)malloc(window[NS] * sizeof(short));
    if (imgbuf[i] == NULL)
	{
	free(mnrbuf);
	c_errmsg("Error allocating memory", "avhrrin-alloc", &nonfatal);
	c_eclose(fdesc);
	return(E_FAIL);
	}
    }


/* Calculate the number of BIP pixels that will be returned by c_arcread()
--------------------------------------------------------------------------*/
npixels = window[NS] * NCHANS;


/* Start processing messages
----------------------------*/
curent = 1;
c_startm();


/* Read and convert AVHRR archive data to LAS image data
--------------------------------------------------------*/
for (i = 0, line = window[NL]; i < window[NL]; i++, line--, curent++)
    {
    /* Read a line of image data from the AVHRR archive file
    --------------------------------------------------------*/
    status = c_arcread(adesc, &mnrptr1, &arcbuf);
    if (status == E_EOF)
	c_errmsg("Unexpected EOF encountered", "avhrrin-eof", &nonfatal);
    if (status != E_SUCC)
	{
	free(mnrbuf);
	for (k = 0; k < *nbands; k++) free(imgbuf[k]);
	c_eclose(fdesc);
	return(E_FAIL);
	}

    /* All images processed in this function are ascending passes read from
       tape.  These passes need to have their pixels flipped to be in the
       correct order for the LAS image.
    -----------------------------------------------------------------------*/
    c_pixflip(arcbuf, npixels);


    /* Copy the Minor Frame data from the buffer returned by c_arcread() to our
       own buffer.  This is necessary because the internal buffer used by
       c_arcread() will be overwritten by the next call.
    --------------------------------------------------------------------------*/
    mnrptr2 = mnrbuf + MNR_WORDS * i;
    for (j = 0; j < MNR_WORDS; j++)
	*(mnrptr2++) = *(mnrptr1++);


    /* Convert the BIP archive format to the BSQ format of LAS images
    -----------------------------------------------------------------*/
    for (j = 0; j < window[NS]; j++)
	for (k = 0; k < *nbands; k++)
	    {
	    imgbuf[k][j] = arcbuf[(NCHANS * j) + (bands[k] - 1)];
	    if (*mmflg)
		{
		if (imgbuf[k][j] > maxval[k]) maxval[k] = imgbuf[k][j];
		if (imgbuf[k][j] < minval[k]) minval[k] = imgbuf[k][j];
		}
	    }


    /* Write each of the image buffers to the LAS image
    ---------------------------------------------------*/
    for (k = 1; k <= *nbands; k++)
	{
	status = c_ewrite(fdesc, &k, &line, (unsigned char *)imgbuf[k-1],&one);
	if (status == E_FAIL)
	    {
	    c_errmsg("Error writing to LAS image", "avhrrin-write", &nonfatal);
	    free(mnrbuf);
	    for (k = 0; k < *nbands; k++) free(imgbuf[k]);
	    c_eclose(fdesc);
	    return(E_FAIL);
	    }
	}
    }


/* Free the image buffer and close the LAS image
------------------------------------------------*/
for (k = 0; k < *nbands; k++) free(imgbuf[k]);
if (c_eclose(fdesc) != E_SUCC)
    {
    free(mnrbuf);
    return(E_FAIL);
    }


/* Open HRPT minor frame file
-----------------------------*/
c_lsmknm(hosout, "mnr", mnr_fname);
status = c_lsopen(&mnr_fd, mnr_fname, &access);
if (status != E_SUCC)
    {
    free(mnrbuf);
    c_errmsg("Error opening HRPT minor frame file", "avhrrin-open", &nonfatal);
    return(E_FAIL);
    }


/* Keep track of output files for cleanup
-----------------------------------------*/
strcpy (outfiles[*n_outfiles], mnr_fname);
*n_outfiles += 1;


/* Store all of the minor frame data to the file
------------------------------------------------*/
for (i = 0; i < window[NL]; i++)
    {
    /* Write Minor Frame data to label services file.  The line number key
       will never have a master line offset (if the entire image is being
       processed there is no offset, if a window is being processed, the
       model is being initialized to reflect this window only).
    -----------------------------------------------------------------------*/
    sprintf(linekey, "%d", i + 1);
    mnrptr1 = mnrbuf + MNR_WORDS * (window[NL] - i - 1);
    status = c_lswrit(&mnr_fd, linekey, &zero, &mnr_nbytes, char_buf,
	(char *)mnrptr1, "I2");
    if (status != E_SUCC)
	{
	free(mnrbuf);
	c_errmsg("Error writing to minor frame file","avhrrin-write",&nonfatal);
	return(E_FAIL);
	}
    }
free(mnrbuf);


/* Close the minor frame file and stop the processing messages
--------------------------------------------------------------*/
c_lsclos(&mnr_fd, mnr_fname, &zero);
c_stoptm ();
return (E_SUCC);
}
