/*******************************************************************************
NAME:		convert_seq

PURPOSE:    	Convert an AVHRR archive image to a LAS image.  This version of
		the convert routine will work will all descending passes and
		ascending passes that exist on disk.  It will read the archive
		images in a "North-up" order and it uses sequential LAS I/O
		to create the LAS image which should increase processing speed.

		Return E_SUCC if successful, E_FAIL otherwise.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    04/91  D.VanderZee      Development
  2.1    04/93  J.Fenno          Modified to read archive images directly from
				 tape.  Also made changes to incorporate the
				 new archio support routines.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "vll.h"
#include "adaps.h"
#include "archio.h"


/* The following variables have been declared global for image I/O routines
------------------------------------------------------------------------------*/
extern struct FDESC *fdesc[2];  /* Image file descriptors used for image I/O  */
extern struct GDESC *gdesc;     /* Group descriptor used for image I/O        */
extern unsigned char *imgbuf;   /* Buffer for image I/O                       */
extern long out_ptr[NCHANS];    /* Pointer into "imgbuf" for output bands     */
extern long curent;             /* Current processing line                    */


int convert_seq
(
    char *hosout,              /* Host file name of output image             */
    char (*outfiles)[CMLEN],   /* Array of output file names                 */
    long *adesc,	       /* AVHRR archive descriptor                   */
    long bands[],              /* Band specifications for each input image   */
    long *mmflg,               /* Min max tracking flag                      */
    long *n_outfiles,          /* Number of output files created             */
    long *nbands,              /* Number of bands per input image            */
    long window[],             /* Window specifications for each input image */
    double maxval[],           /* Max pixel value array                      */
    double minval[]            /* Min pixel value array                      */
)
{
char	chrbuf[1],		/* Character buffer for label services calls  */
	linekey[KEYL],		/* Line number key for minor frame data file  */
	mnr_fname[CMLEN];	/* HRPT minor frame data file name            */

long	access       = IWRITE, 	/* I/O open access flag                       */
	bufsiz,			/* Number of bytes in the image I/O buffer    */
	dtype        = EWORD,	/* Data type of the LAS image                 */
	i, j, k,		/* Loop control                               */
	index,                  /* Index for arcbuf                           */
	mnr_fd,			/* HRPT minor frame data file descriptor      */
        mnr_nbytes,             /* Number of bytes in an HRPT minor frame     */
	nonfatal     = 1,	/* Nonfatal error flag                        */
	one          = 1,	/* Number of bytes to move file pointer       */
	open_opt[3],		/* Options for opening a LAS image            */
	status,			/* Return status variable                     */
	zero         = 0;	/* Zero constant                              */

short	*arcbuf,		/* Ptr to buffer of pixel data from AVHRR     */
	*mnrbuf,		/* Ptr to buffer of Minor Frame data          */
	*outline[NCHANS];	/* Pointers to output line                    */


/* Open the LAS image for sequential I/O 
----------------------------------------*/
open_opt[0] = open_opt[1] = open_opt[2] = 0;
status = c_eopens(&fdesc[0], hosout, nbands, &one, &one, &window[NL],
    &window[NS], out_ptr, &access, &dtype, &zero, open_opt);
if (status != E_SUCC)
    return(E_FAIL);
fdesc[1] = 0;


/* Keep track of output files for cleanup
-----------------------------------------*/
strcpy (outfiles[*n_outfiles], hosout);
*n_outfiles += 1;


/* Allocate memory for image I/O 
--------------------------------*/
if ((imgbuf = (unsigned char *)c_ealloc(fdesc, &bufsiz)) == NULL)
    {
    c_errmsg("Error allocating memory", "avhrrin-alloc", &nonfatal);
    return(E_FAIL);
    }


/* Group the image for I/O
--------------------------*/
if (c_egroup(fdesc, &gdesc, imgbuf, &bufsiz) != E_SUCC)
    return(E_FAIL);


/* Make the associated minor frame format file name and open it
---------------------------------------------------------------*/
c_lsmknm(hosout, "mnr", mnr_fname);
mnr_nbytes = MNR_WORDS * sizeof(short);
status = c_lsopen(&mnr_fd, mnr_fname, &access);
if (status != E_SUCC)
    {
    c_errmsg("Error opening HRPT minor frame file", "avhrrin-open", &nonfatal);
    return(E_FAIL);
    }


/* Keep track of output files for cleanup
-----------------------------------------*/
strcpy (outfiles[*n_outfiles], mnr_fname);
*n_outfiles += 1;


/* Set up for processing
------------------------*/
status = c_estep(&gdesc);
if (status != E_SUCC)
    return (E_FAIL);
for (i = 0; i < *nbands; i++)
    outline[i] = (short *) (imgbuf + out_ptr[i] * sizeof(short));


/* Start processing messages
----------------------------*/
curent = 1;
c_startm();


/* Read and convert AVHRR archive data to LAS image data
--------------------------------------------------------*/
for (i = 0; i < window[NL]; i++, curent++)
    {
    /* Read a line of image data from the AVHRR archive file.  All images
       processed in this function are being returned from c_arcread() in a
       "North-up" order so it will take care of any pixel flipping that is
       necessary for ascending passes.
    ----------------------------------------------------------------------*/
    status = c_arcread(adesc, &mnrbuf, &arcbuf);
    if (status == E_EOF)
	c_errmsg("Unexpected EOF encountered", "avhrrin-eof", &nonfatal);
    if (status != E_SUCC)
	{
	c_egclse(&gdesc);
	return(E_FAIL);
	}


    /* Convert the BIP archive format to the BSQ format of LAS images
    -----------------------------------------------------------------*/
    for (j = 0; j < window[NS]; j++)
	for (k = 0; k < *nbands; k++)
	    {
	    index = NCHANS * j + bands[k] - 1;
	    outline[k][j] = arcbuf[index];
	    if (*mmflg)
		{
		if (outline[k][j] > maxval[k]) maxval[k] = outline[k][j];
		if (outline[k][j] < minval[k]) minval[k] = outline[k][j];
		}
	    }


    /* Write Minor Frame data to label services file.  The line number key
       will never have a master line offset (if the entire image is being
       processed there is no offset, if a window is being processed, the
       model is being initialized to reflect this window only).
    -----------------------------------------------------------------------*/
    sprintf(linekey, "%d", i + 1);
    status = c_lswrit(&mnr_fd, linekey, &zero, &mnr_nbytes, chrbuf,
	(char *)mnrbuf, "I2");
    if (status != E_SUCC)
	{
	c_errmsg("Error writing to minor frame file","avhrrin-write",&nonfatal);
	c_egclse(&gdesc);
	return(E_FAIL);
	}


    /* Write output line using image I/O 
    ------------------------------------*/
    status = c_estep(&gdesc);
    if (status != E_SUCC)
	{
	c_errmsg("Error writing to minor frame file","avhrrin-write",&nonfatal);
	c_egclse(&gdesc);
	return (E_FAIL);
	}


    /* Set up pointers to next line 
    -------------------------------*/
    for (j = 0; j < *nbands; j++)
	outline[j] = (short *) (imgbuf + out_ptr[j] * sizeof(short));
    }


/* Close images in I/O group and free the I/O buffer
----------------------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
    return(E_FAIL);
free(imgbuf);


/* Close the minor frame file and stop the processing messages
--------------------------------------------------------------*/
c_lsclos(&mnr_fd, mnr_fname, &zero);
c_stoptm ();
return (E_SUCC);
}
