/*******************************************************************************
NAME:		SETUP	

PURPOSE:    	Retrieves the parameters from the TAE parameter block and sets
		up for processing.
			 
Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  5.0    04/91  D.VanderZee      Development
  5.1    04/93  J.Fenno          Modified to read archive images directly from
				 tape.  Also made changes to incorporate the
				 new archio support routines.
				
PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "typlim.h"
#include "adaps.h"
#include "adapssup.h"
#include "archio.h"
#include "avhrrin.h"

#ifndef min4
#define min4(a,b,c,d) (min(a,b) < min(c,d) ? min(a,b) : min(c,d))
#endif

#ifndef max4
#define max4(a,b,c,d) (max(a,b) > max(c,d) ? max(a,b) : max(c,d))
#endif

#define FIRST	0
#define SECOND	1

int setup
(
    struct PARBLK *vblock,	/* Pointer to TAE parameter block             */
    struct MODEL *model,	/* AVHRR model structure                      */
    char *comment,		/* Tape mounting comment                      */
    char *hosin, 		/* Host file names of input image             */
    char *hosout,		/* Host file name of output image             */
    char outfiles[][CMLEN],	/* List of output files                       */
    char *taeout,		/* TAE file name of output image              */
    long bands[],		/* Band specifications for each input image   */
    long *mmflg,		/* Min/Max flag for LAS images                */
    long *msgtim,		/* Elapsed time between display messages      */
    long *n_outfiles,		/* Number of output files                     */
    long *nbands,		/* Number of bands per input image            */
    long *tpunit,		/* Tape unit for reading images from tape     */
    long window[]		/* Window specifications for each input image */
)
{
char 	msg[ERRLEN+1],		/* Message buffer			     */
        subcmd[10];		/* Subcommand string			     */

double 	slat,			/* Start latitude			     */
        slong,			/* Start longitude			     */
        elat,			/* End latitude				     */ 
        elev,			/* Elevation of the point                    */
        elong;			/* End longitude			     */

float 	lines[4],		/* Lines after conversion from lat/long      */
        samps[4];		/* Samples after conversion from lat/long    */

long	access    = IREAD,	/* Access flag				     */
        areaopt,		/* Areas to be selected			     */
        el,			/* End line				     */
        element   = 0,		/* Parameter element number		     */
        es,			/* End sample				     */
        i,			/* Loop counter				     */
        nonfatal  = 1,		/* Nonfatal error flag			     */
        numfound,		/* Number of areas found		     */
        off_nadir,		/* # of pixels to save each side of nadir    */
        quadnum,		/* Quadrant that the latitude range is in    */
	satview,                /* Temporary copy of the model->sat_view flag*/
        sl,			/* Starting line 			     */
        ss,			/* Starting sample			     */
        startln[2],		/* Line corresponding to first lat	     */
        status,			/* Return status variable		     */
        stopln[2];		/* Line corresponding to second lat	     */

struct DROPPED  drop;		/* dropped line data structure 		     */
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */


/* Retrieve the tape mounting comment
-------------------------------------*/
val = p_find(vblock, "COMMENT");
if (val->v_count == 0)
    strcpy(comment, "");
else
    strcpy(comment, SVAL(*val, 0));


/* Depending upon the location of the archive image (disk or tape)
   retrieve the name of the image and read its header file.
------------------------------------------------------------------*/
if (strlen(comment) == 0)
    {
    /* Retrieve archive image name
    ------------------------------*/
    status = c_getnam(vblock, "IN", &element, hosin, "arch", &access);
    if (status != E_SUCC)
	return(E_FAIL);


    /* Get the AVHRR header information
    -----------------------------------*/
    status = c_getahdr(hosin, model, &drop);
    if (status != E_SUCC)
	return(E_FAIL);
    }
else
    {
    /* Retrieve the scene ID
    ------------------------*/
    val = p_find(vblock, "IN");
    strcpy(hosin, SVAL(*val, 0));


    /* Get the AVHRR header information.  NOTE: this routine mounts the tape,
       reads the header file, and positions the tape at the start of the image
       file.  If a non-success status is returned, NULL the tape unit number
       so that avhrrin.c doesn't try to close the tape again (c_tpgetahdr()
       already closed the tape in the event of non-success).
    --------------------------------------------------------------------------*/
    status = c_tpgetahdr(hosin, comment, tpunit, model, &drop);
    if (status != E_SUCC)
	{
	*tpunit = NULL;
	return(E_FAIL);
	}
    }


/* Process all archive images in a "North-up" order except for ascending
   images located on tape.  These must be processed in satellite view to
   avoid skipping the tape forward and backward.
------------------------------------------------------------------------*/
if ((model->cpass == ASCENDING) && strlen(comment))
    model->sat_view = TRUE;
else
    model->sat_view = FALSE;


/* Retrieve the output image name
---------------------------------*/
status = c_getout(vblock, "OUT", (char (*)[256])taeout, (char (*)[256])hosout);
if (status != E_SUCC)
    return(E_FAIL);


/* Retrieve band numbers to read and convert
--------------------------------------------*/
val = p_find(vblock, "BANDS");
if (val->v_count == 0)
    {
    for (*nbands = 0, i = 0; i < NCHANS; i++)
	if (model->chans[i] != 0)
	    {
	    bands[*nbands] = model->chans[i];
	    *nbands += 1;
	    }
    }
else 
    {
    for (i = 0; i < val->v_count; i++)
	{
	bands[i] = IVAL(*val, i);
	if (model->chans[bands[i] - 1] == 0)
	    {
	    sprintf(msg, "Band %1d not present in archive image", bands[i]);
	    c_errmsg(msg, "avhrrin-band", &nonfatal);
	    return(E_FAIL);
	    }
	}
    *nbands = val->v_count;	
    }


/* Ensure the rest of the band elements are set to zero
-------------------------------------------------------*/
for (i = *nbands; i < MAXBND+1; i++)
    bands[i] = 0;


/* Retrieve subcommand specification
------------------------------------*/ 
val = p_find(vblock, "_SUBCMD");
strcpy(subcmd, SVAL(*val, 0));


/* Retrieve parameters specific to each subcommand
--------------------------------------------------*/
if (strcmp (subcmd, "LINESAMP") == 0)
    {
    val = p_find(vblock, "SL");
    if (val->v_count != 0)
	window[SL] = IVAL(*val, 0);
    else 
	window[SL] = 1;

    val = p_find(vblock, "SS");
    if (val->v_count != 0)
	window[SS] = IVAL(*val, 0);
    else 
	window[SS] = 1;

    val = p_find(vblock, "NL");
    if (val->v_count != 0)
	window[NL] = IVAL(*val, 0);
    else 
	window[NL] = model->nlines;

    val = p_find(vblock, "NS");
    if (val->v_count != 0)
	window[NS] = IVAL(*val, 0);
    else
	window[NS] = model->nsamples;


    /* If this is an ascending pass that is being read from tape (ie. it is
       being processed in satellite view), we must flip the line and sample
       numbers so that we process the same portion of the image that we
       would process if it were on disk.
    -----------------------------------------------------------------------*/
    if ((model->cpass == ASCENDING) && model->sat_view)
	{
	window[SL] = model->nlines - (window[SL] + window[NL] - 1) + 1;
	window[SS] = model->nsamples - (window[SS] + window[NS] - 1) + 1;
	}
    }
else if (strcmp (subcmd, "LATLONG") == 0)
    {
    val = p_find(vblock, "SLAT");
    slat = RVAL(*val, 0);

    val = p_find(vblock, "SLONG");
    slong = RVAL(*val, 0);

    val = p_find(vblock, "ELAT");
    elat = RVAL(*val, 0);

    val = p_find(vblock, "ELONG");
    elong = RVAL(*val, 0);

    /* Convert lat and long coordinates to line sample 
    --------------------------------------------------*/
    slat  *= RAD_PER_DEG;
    slong *= RAD_PER_DEG;
    elat  *= RAD_PER_DEG;
    elong *= RAD_PER_DEG;


    /* Calculate the line/sample locations to be extracted.  The model will
       return the line number in the correct order depending upon the sat_view
       field of the model structure.
    --------------------------------------------------------------------------*/
    elev = 0;
    c_ll2ls(&slat, &slong, &elev, model, &lines[0], &samps[0], NULL);
    c_ll2ls(&elat, &elong, &elev, model, &lines[1], &samps[1], NULL);
    c_ll2ls(&slat, &elong, &elev, model, &lines[2], &samps[2], NULL);
    c_ll2ls(&elat, &slong, &elev, model, &lines[3], &samps[3], NULL);

    sl = (long)min4(lines[0], lines[1], lines[2], lines[3]);
    ss = (long)min4(samps[0], samps[1], samps[2], samps[3]);
    el = (long)max4(lines[0], lines[1], lines[2], lines[3]);
    es = (long)max4(samps[0], samps[1], samps[2], samps[3]);

    if (sl < 0)
	window[SL] = 1;
    else
	window[SL] = sl;

    if (ss < 0)
	window[SS] = 1;
    else
	window[SS] = ss;

    if (el > model->nlines)
	window[NL] = model->nlines - window[SL] + 1;
    else
	window[NL] = el - window[SL] + 1;

    if (es > model->nsamples)
	window[NS] = model->nsamples - window[SS] + 1;
    else
	window[NS] = es - window[SS] + 1;
    } 
else if (strcmp (subcmd, "LATRANGE") == 0)
    {
    /* Retrieve the starting latitude range
    ---------------------------------------*/
    val = p_find(vblock, "SLAT");
    slat = RVAL(*val, 0);

    /* Retrieve the ending latitude range
    -------------------------------------*/
    val = p_find(vblock, "ELAT");
    elat = RVAL(*val, 0);

    /* Retrieve quadrant number that the specified latitude range is wanted for
    --------------------------------------------------------------------------*/
    val = p_find(vblock, "QUADNUM");
    quadnum = IVAL(*val, 0);

    /* Determine the number of samples to be selected
    -------------------------------------------------*/
    val = p_find(vblock, "OFFSET");
    if ((val->v_count == 0) && (model->dtype == GAC))
	{
	window[SS] = 1;
	window[NS] = GAC_SAMPS;
	}
    else if (val->v_count == 0)
	{
	window[SS] = 1;
	window[NS] = LAC_SAMPS;
	}
    else
	{
	/* Get the number of samples each side of nadir
	-----------------------------------------------*/
	off_nadir = IVAL(*val, 0);

	/* Set sample window for GAC size images
	----------------------------------------*/
	if (model->dtype == GAC)
	    {
	    /* Check bounds for GAC data and set the window to the specified
	       number of samples selected from each side of the image.
	    ----------------------------------------------------------------*/
	    if (off_nadir >= GAC_NADIR)
		{
		c_errmsg("Offest too large for GAC reset to maximum",
		    "avhrrin-offset", &nonfatal);
		window[SS] = 1;
		window[NS] = GAC_SAMPS;
		}
	    else
		{
		window[SS] = (long)GAC_NADIR - off_nadir;
		window[NS] = 2 * off_nadir + 1;
		}
	    }
	else
	    {
	    /* Set the number of samples to be taken from each side of nadir
	    ----------------------------------------------------------------*/
	    window[SS] = (long)LAC_NADIR - off_nadir + 1; 
	    window[NS] = 2 * off_nadir;
	    }
	}

    /* Retrieve the area option to determine which area is to be processed.
    -----------------------------------------------------------------------*/
    val = p_find(vblock, "AREAOPT");
    if (strcmp(SVAL(*val, 0), "FIRST") == 0)
	areaopt = FIRST;
    else
	areaopt = SECOND;


    /* Findrange() processes all passes as if they are in satellite view.
       Therefore, we must temporarily set the model->sat_view flag, call
       findrange, reset the model->sat_view flag, and then flip the window
       if necessary.
    -----------------------------------------------------------------------*/
    satview = model->sat_view;
    model->sat_view = TRUE;


    /* Find the starting and ending lines for the specified latitude range.
       NOTE: areaopt is FIRST or SECOND, not BOTH, so zero or one set of
       lines will be sent back, never two.
    ----------------------------------------------------------------------*/
    slat *= RAD_PER_DEG;
    elat *= RAD_PER_DEG;
    status =findrange(slat,elat,startln,stopln,quadnum,areaopt,&numfound,model);
    if (status != E_SUCC)
	return(E_FAIL);
    

    /* Reset the model->sat_view flag
    ---------------------------------*/
    model->sat_view = satview;
    if ((model->cpass == ASCENDING) && !model->sat_view)
	window[SL] = model->nlines - stopln[0] + 1;
    else
	window[SL] = startln[0];
    window[NL] = stopln[0] - startln[0] + 1;
    }


/* Check the starting line and sample
-------------------------------------*/
if (window[SL] > model->nlines)
    {
    sprintf(msg, "Starting line must be less than %d", model->nlines);
    c_errmsg (msg, "avhrrin-start", &nonfatal);
    return(E_FAIL);
    }
if (window[SS] > model->nsamples)
    {
    sprintf(msg, "Starting sample must be less than %d", model->nsamples);
    c_errmsg (msg, "avhrrin-start", &nonfatal);
    return(E_FAIL);
    }


/* Check the number of lines
----------------------------*/
if (window[NL] > 0)
    {
    if ((window[SL] + window[NL] - 1) > model->nlines)
	{
	sprintf(msg, "Number of lines reset from %1d to %1d",
	    window[NL], model->nlines - window[SL] + 1);
	c_errmsg (msg, "avhrrin-size", &nonfatal);
	window[NL] = model->nlines - window[SL] + 1;
	}
    }
else
    {
    sprintf (msg, "The number of lines %d must be positive", window[NL]);
    c_errmsg (msg, "avhrrin-positive", &nonfatal);
    return(E_FAIL);
    }


/* Check the number of samples
------------------------------*/
if (window[NS] > 0)
    {
    if ((window[SS] + window[NS] - 1) > model->nsamples)
	{
	sprintf(msg, "Number of samples reset from %1d to %1d",
	    window[NS], model->nsamples - window[SS] + 1);
	c_errmsg (msg, "avhrrin-size", &nonfatal);
	window[NS] = model->nsamples - window[SS] + 1;
	}
    }
else
    {
    sprintf (msg, "The number of samples %d must be positive", window[NS]);
    c_errmsg (msg, "avhrrin-positive", &nonfatal);
    return(E_FAIL);
    }


/* Update the LAS associated ADDR and Dropped line files
--------------------------------------------------------*/
status = upassoc(hosout, outfiles, n_outfiles, window, &drop, model,
	 	 bands);
if (status != E_SUCC)
    return(E_FAIL);


/* Retrieve globals
-------------------*/
val = p_find(vblock, "$MSGTIM");
*msgtim = IVAL(*val, 0);
val = p_find(vblock, "$MINMAX");
*mmflg = (strcmp("YES", SVAL(*val, 0)) == 0);

return(E_SUCC);
}
