/*******************************************************************************
FUNCTION:	print_form

PURPOSE:	Print a shipping form for the global variable tape_id


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"
#include "queue.h"
#include "avhrrstack_.h"


#define MAXIMGS     60	/* Max number of images per page for a shipping form  */
#define NCOLS        3	/* Number of columns of image names per page          */
#define NCOPIES      2	/* Number of copies that should be printed            */

#ifdef vms
#define SYS_SUCC     1	/* VMS  successful exit status                        */
#else
#define SYS_SUCC     0	/* UNIX successful exit status                        */
#endif



int print_form
(
    long queue,			/* IN  - Queue of AVHRR image names           */
    char *adapsarc,		/* IN  - The ADAPSARC directory               */
    char *tapeid,		/* IN  - Identification number of the tape    */
    char *printer,		/* IN  - Printer to print the forms on        */
    char *logfile,		/* IN  - Log file for processing messages     */
    long print,			/* IN  - Flag for printing files or not       */
    long maxfiles,		/* IN  - Max number of output files           */
    char instruct[9][30],	/* IN  - Special instructions to print	      */
    char address[5][27],	/* IN  - Address to ship to		      */
    char fnames[][CMLEN],	/* OUT - LAS names of files if not printed    */
    long *nfiles 		/* OUT - Number of file names in fnames[]     */
)

{
char filename[CMLEN];	/* Name of file that contains the current page        */
char images[MAXIMGS][CMLEN];/* Name of every image on the current page        */
char line[CMLEN];	/* One line of the form                               */
char lprint[CMLEN];	/* Command for printing the shipping form file        */
char *ptr;		/* Ptr to an image name in the queue                  */
char str[CMLEN];	/* Temporary string                                   */



long col;		/* Loop control                                       */
long i;			/* Loop control                                       */
long image_num;		/* Number of the current image (1 to total on tape)   */
long j;			/* Loop control                                       */
long nimages;		/* Number of images on the current page               */
long nonfatal     = 1;	/* Nonfatal error flag                                */
long nrows;		/* Number of scenes in the longest column             */
long page;		/* Current page number of the shipping form           */

FILE *fp;		/* Output file                                        */



/* Issue an informational message
---------------------------------*/
if (print)
    {
    sprintf(str, "Printing %1d shipping form(s) to %s", NCOPIES, printer);
    c_errmsg(str, " ", &nonfatal);
    c_wrtlog(logfile, tapeid, "shipform-print", str);
    }


/* Locate the first image name in the queue
-------------------------------------------*/
c_qfirst(queue, (void **)&ptr);


/* Create the shipping form
---------------------------*/
page = 0;
image_num = 0;
while (ptr != NULL)
    {
    page++;   /* increment page number */


    /* Place up to MAXIMGS image names from the queue into an array.  When each
       name is placed in the array preceed it with the appropriate number.
    --------------------------------------------------------------------------*/
    nimages = 0;
    while ((ptr != NULL) && (nimages < MAXIMGS))
	{
	sprintf(images[nimages++], "   %2d. %-16.16s", ++image_num, ptr);
	c_qnext(queue, (void **)&ptr);
	}


    /* Open an output file
    ----------------------*/
    sprintf(filename, "%s/forms/%s_page%d.prt", adapsarc, tapeid, page);
    fp = fopen(filename, "w");
    if (fp == NULL)
	{
	strcpy(str, "Error opening shipping form file");
	c_errmsg(str, "shipform-open", &nonfatal);
	c_wrtlog(logfile, tapeid, "shipform-error", str);
	return(E_FAIL);
	}


    /* Print the appropriate header for the form
    --------------------------------------------*/
    print_header(fp, tapeid, address);


    /*=======================================================================
    = Print the names of each scene in NCOLS columns.  The following is the =
    = way that 11 scenes should be printed in 3 columns.  The number of the =
    = images have already been placed in the images strings.                =
    =                                                                       =
    =    1. XXXXXXXXXXXXXXXX    5. XXXXXXXXXXXXXXXX    9. XXXXXXXXXXXXXXXX  =
    =    2. XXXXXXXXXXXXXXXX    6. XXXXXXXXXXXXXXXX   10. XXXXXXXXXXXXXXXX  =
    =    3. XXXXXXXXXXXXXXXX    7. XXXXXXXXXXXXXXXX   11. XXXXXXXXXXXXXXXX  =
    =    4. XXXXXXXXXXXXXXXX    8. XXXXXXXXXXXXXXXX                         =
    =======================================================================*/


    /* For every row
    ----------------*/
    nrows = (long)ceil((double)nimages / NCOLS);
    for (i = 0; i < nrows; i++)
	{
	/* For every column in the current row
	--------------------------------------*/
	strcpy(line, "    ");                        /* leading spaces */
	for (col = 0; col < NCOLS; col++)
	    {
	    j = i + (long)(col * nrows);
	    if (j < nimages)
		strcat(line, images[j]);
	    }
	fprintf(fp, "%s\n", line);
	}


    /* Print a continue statement if there is another image in the queue
    --------------------------------------------------------------------*/
    if (ptr != NULL)
	{
	fprintf(fp, "\n");
	fprintf(fp, "       (continued on next page...)\n");
	}


    /* Print the footer of the shipping form
    ----------------------------------------*/
    print_footer(fp, instruct);
    fclose(fp);


    /* Print the shipping form and delete the file or save the output file name
    --------------------------------------------------------------------------*/
    if (!print)
	{
	if (*nfiles < maxfiles)
            strcpy(fnames[(*nfiles)++], filename);
	}
    else
	{
	sprintf(lprint, "lp -c -s -d%s %s", printer, filename);
	for (i = 0; i < NCOPIES; i++)
	    {
	    if (system(lprint) != SYS_SUCC)
		{
		sprintf(str, "Error printing shipping form for %s", tapeid);
		c_errmsg(str, "shipform-lprint", &nonfatal);
		c_wrtlog(logfile, tapeid, "shipform-error", str);
		}
	    }
	remove(filename);
	}
    }


/* Print a message stating that a form has been created
-------------------------------------------------------*/
sprintf(str, "Shipping form created for tape %s", tapeid);
c_errmsg(str, " ", &nonfatal);
c_wrtlog(logfile, tapeid, "shipform-form", str);
return(E_SUCC);
}




void print_header
(
    FILE *fp,		/* IN - Output file                                   */
    char *tapeid,	/* IN - ID of tape the form is being created for      */
    char address[5][27] /* IN - Shipping address			      */
)

{
char id[CMLEN];		/* Uppercase copy of tapeid                           */
long len;		/* Length of tapeid                                   */

strcpy(id, tapeid);
len = (long)strlen(id);
c_low2up(id, &len);
fprintf(fp, "%55s\n", "U.S. DEPARTMENT OF THE INTERIOR");
fprintf(fp, "%48s\n", "GEOLOGICAL SURVEY");
fprintf(fp, "\n");
fprintf(fp, "%44s\n", "SHIPPING");
fprintf(fp, "%45s\n", "MEMORANDUM");
fprintf(fp, "\n");
fprintf(fp, "\n");
fprintf(fp, "   TO:%62s\n", "Date Shipped:___________");
fprintf(fp, "       %-26s\n", address[0]);
fprintf(fp, "       %-26s\n", address[1]);
fprintf(fp, "       %-26s\n", address[2]);
fprintf(fp, "       %-26s\n", address[3]);
fprintf(fp, "       %-26s\n", address[4]);
fprintf(fp, "\n");
fprintf(fp, "\n");
fprintf(fp, "       The enclosed tape, identification number ");
fprintf(fp, "%s, contains the\n", id);
fprintf(fp, "       following images:\n");
fprintf(fp, "\n");
}




void print_footer
(
    FILE *fp,			/* IN - Output file                           */
    char instruct[9][30] 	/* IN  - Special instructions to print	      */
)

{
fprintf(fp, "\n");
fprintf(fp, "\n");
fprintf(fp, "       If there are any problems with this tape please include ");
fprintf(fp, "the tape\n");
fprintf(fp, "       identification number and a description of the problem.\n");
fprintf(fp, "\n");
fprintf(fp, "       ________________________________________________");
fprintf(fp, "__________________\n");
fprintf(fp, "                                          |\n");
fprintf(fp, "       %-29s      |    Shipment ", instruct[0]);
fprintf(fp, "received by:\n");
fprintf(fp, "       %-29s      |\n", instruct[1]);
fprintf(fp, "       %-29s      |    ________", instruct[2]);
fprintf(fp, "______________\n");
fprintf(fp, "       %-29s      |\n", instruct[3]);
fprintf(fp, "       %-29s      |\n", instruct[4]);
fprintf(fp, "       %-29s      |    Date ", instruct[5]);
fprintf(fp, "Received:\n");
fprintf(fp, "       %-29s      |\n", instruct[6]);
fprintf(fp, "       %-29s      |    ________", instruct[7]);
fprintf(fp, "______________\n");
fprintf(fp, "       %-29s      |\n", instruct[8]);
fprintf(fp, "       ___________________________________|____________");
fprintf(fp, "___________________\n");
}
