/*******************************************************************************
FUNCTION:	read_summary

PURPOSE:	Read the contents of the tape summary file and return the tape
		ID along with a queue containing the name of each file group
		that was stacked to the tape.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    01/94  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "queue.h"



int read_summary
(
    char *function,	/* IN  - Name of the calling function                 */
    char *sumfile,	/* IN  - Host file name of the tape summary file      */
    char *tapeid,	/* OUT - ID of the tape in the summary file           */
    char *system,	/* OUT - System on which the tape was created         */
    char *drive,	/* OUT - Tape drive on which the tape was created     */
    char *groups,	/* OUT - The number of file groups stored on the tape */
    long *queue 	/* OUT - Queue of file groups on the tape             */
)

{
char diskname[CMLEN];	/* Name of the file as stored on disk                 */
char key[CMLEN];	/* Key for error messages                             */
char *ptr;		/* Ptr to a dynamically allocated string              */
char root[CMLEN];	/* Root portion of a file name                        */
char str[CMLEN];	/* Temporary string                                   */
char tapename[CMLEN];	/* Name of the file as stored on tape                 */

long files_per_group;	/* Number of files per file group (nfiles/ngroups)    */
long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long nfiles;		/* Number of files stored on the tape                 */
long ngroups;		/* Number of file groups stored on the tape           */
long nonfatal =  1;	/* Nonfatal error flag                                */

FILE *fp;		/* File pointer for the tape summary file             */


/* Open the summary file for reading
------------------------------------*/
fp = fopen(sumfile, "r");
if (fp == NULL)
    {
    sprintf(key, "%s-open", function);
    c_errmsg("Error opening tape summary file", key, &nonfatal);
    return(E_FAIL);
    }


/* Read the first line from the file and extract the tape ID, read the second
   line and extract the system name and the tape drive where the tape was
   created, extract the number of groups and the number of files from the third
   line, and then skip the next three lines, leaving the file pointer at the
   beginning of the file names.
------------------------------------------------------------------------------*/
fgets(str, CMLEN, fp);
sscanf(str, "%*s %s", tapeid);
fgets(str, CMLEN, fp);
sscanf(str, "%*s %s %*s %s", system, drive);
fgets(str, CMLEN, fp);
sscanf(str, "%*s %d %*s %d", &ngroups, &nfiles);
/* Repeat to get groups as a string */
sscanf(str, "%*s %s %*s %d", groups, &nfiles);
fgets(str, CMLEN, fp);
fgets(str, CMLEN, fp);
fgets(str, CMLEN, fp);


/* Lowercase the tape ID
------------------------*/
i = (long)strlen(tapeid);
c_up2low(tapeid, &i);


/* Read all of the files that have been stacked to tape and
   store the root file name of every file group in a queue.
-----------------------------------------------------------*/
*queue = c_qcreate("fifo", "dups");
files_per_group = nfiles / ngroups;
for (i = 0; i < ngroups; i++)
    {
    /* Read the first file of the group
    -----------------------------------*/
    if (fscanf(fp, "%s %s", tapename, diskname) != 2)
	{
	fclose(fp);
	c_qdestroy(*queue);
	sprintf(key, "%s-read", function);
	c_errmsg("Error reading summary file", key, &nonfatal);
	return(E_FAIL);
	}
    

    /* Allocate a string to store the root file name.  When placing data
       in a queue it must be stored in dynamically allocated memeory.
    --------------------------------------------------------------------*/
    c_splitname(diskname, str, root, str);
    ptr = (char *)malloc(strlen(root) + 1);
    if (ptr == NULL)
	{
	fclose(fp);
	c_qdestroy(*queue);
	sprintf(key, "%s-alloc", function);
	c_errmsg("Error allocating memory", key, &nonfatal);
	return(E_FAIL);
	}
    strcpy(ptr, root);
    c_qinsert(*queue, (void *)ptr, (long (*)())strcmp);

    
    /* Read the rest of the files in the file group
    -----------------------------------------------*/
    for (j = 1; j < files_per_group; j++)
	if (fscanf(fp, "%s %s", tapename, diskname) != 2)
	    {
	    fclose(fp);
	    c_qdestroy(*queue);
	    sprintf(key, "%s-read", function);
	    c_errmsg("Error reading summary file", key, &nonfatal);
	    return(E_FAIL);
	    }
    }


fclose(fp);
return(E_SUCC);
}
