/*******************************************************************************
FUNCTION:	shipform

PURPOSE:	Print a shipping form for the given tape ID and optionally a
		tape label.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development
  1.1    10/92  J.Fenno          Added code to optionally print a tape label
  1.2    01/94  J.Fenno          Rewrite of AUTOSTACK into ANSISTACK and
				 AVHRRSTACK.  Modified to work with code from
				 AVHRRSTACK.
  1.3    11/94  J.Fenno          Modified getpar_sh() so it does not change the
				 printer from ops to ops-n.  This is no longer
				 a valid printer for the lp command.
  2.0    04/97  R.Dilley         Modified getpar_sh() to accept a new parameter.
                                 Restructured shipform() to be able to print
                                 both archive and free format labels.
  2.1    04/98  D.Lloyd          Initialized address and instruction strings
				 to NULL to prevent garbage from printing.

PROJECT: ADAPS
*******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "tae.h"
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "queue.h"
#include "avhrrstack_.h"

#define MAXOUT  10	/* Maximum number of output files                     */
#define NMONTHS  0	/* Number of months to retain the tape (0=forever)    */
#define FREE     0	/* Free format tape label                             */
#define LIBARY   1	/* EROS Data Center's tape library label              */


main()
{
char adapsarc[CMLEN];	/* The ADAPSARC directory                             */
char address[5][27];	/* Shipping address				      */
char procopt[CMLEN];	/* Which format to print                              */
char date_time[CMLEN];	/* String version of current date and time            */
char drive[CMLEN];	/* Tape drive on which the tape was created           */
char fnames[MAXOUT][CMLEN]; /* For call to c_complt                           */
char ngroups[CMLEN];	/* Number of file groups stored on the tape           */
char instruct[9][30];	/* Special instruction to print on shipping form      */
char logfile[CMLEN];	/* File for logging all processing messages           */
char printer[CMLEN];	/* Printer to print the shipping form to              */
char ordernum[14];	/* Order number					      */
char outform[9];	/* Output format				      */
char str[CMLEN];	/* Temporary working string                           */
char sumfile[CMLEN];	/* Full path specification of file containing IDs     */
char system[CMLEN];	/* System on which the tape was created               */
char tape_buf[11][50];	/* Buffer used for printing tape labels               */
char *tape_ptr[11];	/* Array of ptrs for calling c_tapelabel()            */
char tapeid[CMLEN];	/* ID of the tape to print a shipping form for        */
char *ptr;		/* Pointer to a string name from the queue            */
char sfl_km1[CMLEN];	/* A value of KM1 or SFL for the archive description  */

long fatal    = -1;	/* Fatal error flag                                   */
long i;			/* Loop control                                       */
long nfiles   =  0;	/* Number of output files                             */
long nonfatal =  1;	/* Nonfatal error flag                                */
long now;		/* Current date and time encoded as an integer        */
long print;		/* Flag for printing output                           */
long queue;		/* Queue of AVHRR images stacked to the tape          */
long status;		/* Return status of a function                        */
long std_out;		/* Standard output file                               */
long tplabel;		/* Flag indicating if tape label should be created    */

struct PARBLK vblock;	/* TAE parameter block                                */
struct tm *today;	/* Structure containing current date and time         */



/* Initialize address and instruction strings to NULL
------------------------------------------------------*/
for ( i = 0; i < 6; i++ )
    address[i][0] = 0;
for ( i = 0; i < 10; i++ )
    instruct[i][0] = 0;


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Determine the date and time
------------------------------*/
now = time(NULL);
today = localtime(&now);
sprintf(date_time, "%02d/%02d/%02d %02d:%02d:%02d", today->tm_mon + 1,
    today->tm_mday, today->tm_year, today->tm_hour, today->tm_min,
    today->tm_sec % 60);


/* Get all of the parameters from the variable block
----------------------------------------------------*/
getpar_sh(&vblock, tapeid, printer, adapsarc, logfile, sumfile,
       &tplabel, &print, outform, address, ordernum, instruct, procopt);

/* Read the tape summary file
-----------------------------*/
status = read_summary("shipform", sumfile, tapeid, system,
                      drive, ngroups, &queue);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered", "shipform-fatal", &fatal);

if (strcmp(procopt,"FREE") == 0)
{
    /* Create the tape label 
    -------------------------------------------*/
    sprintf(tape_buf[0], " ");
    sprintf(tape_buf[1], " %s", address[0]);
    sprintf(tape_buf[2], " %s", address[1]);
    sprintf(tape_buf[3], " %s", address[2]);
    sprintf(tape_buf[4], " %s", address[3]);
    sprintf(tape_buf[5], " %s", address[4]);
    sprintf(tape_buf[6], " Order #: %s", ordernum);
    sprintf(tape_buf[7], " AVHRR %s Images", outform);
    sprintf(tape_buf[8], " Tape: %s  %03d Images", tapeid, c_qsize(queue));
    sprintf(tape_buf[9], " Created:EROS Data Center");
    sprintf(tape_buf[10], " Dr:%.4s %s", drive, date_time);


    /* If a tape label was requested print it according to the print flag
    ---------------------------------------------------------------------*/
    if (tplabel && !print)
        c_errmsg("Tape label not printed", " ", &nonfatal);
    else if (tplabel)
        {
        /* Set up the buffer used for calling c_tapelabel()
        ---------------------------------------------------*/
        for (i = 0; i < 11; i++)
            tape_ptr[i] = tape_buf[i];
        status = c_tapelabel(tape_ptr, NMONTHS, print, FREE, str);
        if (status != E_SUCC)
            c_errmsg("Error printing tape label", "shipform-lbl", &nonfatal);
        c_errmsg("Tape label printed", " ", &nonfatal);
        }


    /* Print the shipping form
    --------------------------*/
    if (print_form(queue, adapsarc, tapeid, printer,
        logfile, print, MAXOUT, instruct, address, fnames, &nfiles) != E_SUCC)
        {
        c_qdestroy(queue);
        c_errmsg("Fatal error encountered", "shipform-fatal", &fatal);
        }
}
else
{
    /* Determine if it is an SFL or KM1 tape
    ------------------------------------------*/
    c_qfirst(queue, (void **)&ptr);
    strcpy(sfl_km1,"SFL");
    if (ptr[1] == 'o')
        strcpy(sfl_km1,"KM1");

    /* Set up the Label information
    ---------------------------------------*/
    sprintf(tape_buf[0], "018");
    sprintf(tape_buf[1], " ");
    sprintf(tape_buf[2], "%s acq %s", ngroups, sfl_km1);
    sprintf(tape_buf[3], "%s", drive);
    sprintf(tape_buf[4], "avhrr");
    sprintf(tape_buf[5], "ops");
    sprintf(tape_buf[6], "1/1");
    sprintf(tape_buf[7], "%s", tapeid);
    sprintf(tape_buf[8], " ");
    sprintf(tape_buf[9], " ");
    sprintf(tape_buf[10], " ");

    /* Set up the buffer used for calling c_tapelabel()
    ---------------------------------------------------*/
    for (i = 0; i < 11; i++)
        tape_ptr[i] = tape_buf[i];
    status = c_tapelabel(tape_ptr, NMONTHS, print, LIBARY, str);
    if (status != E_SUCC)
        c_errmsg("Error printing tape label", "shipform-lbl", &nonfatal);
    c_errmsg("Tape label printed", " ", &nonfatal);
}

/* Free all memory used by the queue of image names
---------------------------------------------------*/
c_qdestroy(queue);


/* Issue a completion message
-----------------------------*/
c_complt("SHIPFORM", fnames, &nfiles);

/* Put a return here to make the compiler happy
------------------------------------------------*/
return(1);

}




long getpar_sh
(
    struct PARBLK *vblock,	/* IN  - TAE parameter block                  */
    char *tapeid,		/* OUT - ID of tape for which to print a form */
    char *printer,		/* OUT - Printer where forms should be sent   */
    char *adapsarc,		/* OUT - ADAPSARC environment variable        */
    char *logfile,		/* OUT - Host name of the log file            */
    char *sumfile,		/* OUT - Host name of the tape summary file   */
    long *tplabel,		/* OUT - Flag indicating to print tape label  */
    long *print,		/* OUT - Flag to print form and tape label    */
    char *outform,		/* OUT - Output format			      */
    char address[5][27],	/* OUT - Output shipping address	      */
    char ordernum[],		/* OUT - Order number			      */
    char instruct[9][30],	/* OUT - Special instructions		      */
    char *procopt 		/* OUT - Which format to print the tape label */
)

{
char *ptr;			/* Ptr to a string                            */

long access;			/* Access flag for a file                     */
long fatal    = -1;		/* Fatal error flag                           */
long i;				/* Counter				      */
long len;			/* Length of a string                         */
long nonfatal =  1;		/* Nonfatal error flag                        */
long zero     =  0;		/* Zero constant                              */

struct VARIABLE *parm;		/* Ptr to a TAE variable                      */



/* Get TAPEID from the variable block (must be all lowercase)
-------------------------------------------------------------*/
parm = p_find(vblock, "TAPEID");
strcpy(tapeid, SVAL(*parm,0));
len = (long)strlen(tapeid);
c_up2low(tapeid, &len);


/* Get PRINTER from the variable block
--------------------------------------*/
parm = p_find(vblock, "PRINTER");
strcpy(printer, SVAL(*parm,0));
len = (long)strlen(printer);
c_up2low(printer, &len);


/* Get the tape label creation flag
-----------------------------------*/
parm = p_find(vblock, "TPLBLFLG");
if (strcmp(SVAL(*parm,0), "YES") == 0)
    *tplabel = TRUE;
else
    *tplabel = FALSE;


/* Retrieve the log file or create the default name
---------------------------------------------------*/
parm = p_find(vblock, "LOGFILE");
if (parm->v_count == 1)
    {
    access = IUPDATE;
    if (c_getnam(vblock, "LOGFILE", &zero, logfile, "log", &access) != E_SUCC)
	c_errmsg("Fatal error encountered", "shipform-fatal", &fatal);
    }
else
    {
    ptr = (char *)getenv("ADAPSLOG");
    if (ptr == NULL)
	{
	c_errmsg("Error retrieving ADAPSLOG", "shipform-env", &nonfatal);
	c_errmsg("Fatal error encountered", "shipform-fatal", &fatal);
	}
    sprintf(logfile, "%s/shipform.log", ptr);
    }


/* We used to print the shipping form if the TAE variable FNAME was set
   to "EROS Data Center", otherwise the file was just left on disk.  This
   logic was implemented because the local utility "nprint" was used.
   With the change to use the UNIX command "lp", the shipping form will
   always be printed.
-------------------------------------------------------------------------*/
*print = TRUE;


/* Create the name for the tape summary file
--------------------------------------------*/
ptr = (char *)getenv("ADAPSARC");
if (ptr == NULL)
    {
    c_errmsg("Error retrieving ADAPSARC", "shipform-env", &nonfatal);
    c_errmsg("Fatal error encountered", "shipform-fatal", &fatal);
    }
strcpy(adapsarc, ptr);
sprintf(sumfile, "%s/forms/%s.list", adapsarc, tapeid);


/* Get the tape output format
-----------------------------------*/
parm = p_find(vblock, "OUTFORM");
strcpy(outform, SVAL(*parm,0));
len = (long)strlen(outform);
c_up2low(outform, &len);
if (strcmp(outform, "level0") == 0)
    strcpy(outform,"Level-0");
else
    strcpy(outform,"Level-1B");


/* Get the shipping address
------------------------------------*/
parm = p_find(vblock, "ADDRESS");
for (i = 0; i < parm->v_count; i++)
    {
    strcpy(address[i], SVAL(*parm,i));
    }


/* Get the order number
------------------------------------*/
parm = p_find(vblock, "ORDERNUM");
if (parm->v_count == 0)
    strcpy(ordernum,"");
else
    strcpy(ordernum,SVAL(*parm,0));


/* Get the special instructions
------------------------------------*/
parm = p_find(vblock, "INSTRUCT");
for (i = 0; i < parm->v_count; i++)
    {
    strcpy(instruct[i], SVAL(*parm,i));
    }

/* Get the PROCOPT to decide which format to print
---------------------------------------------------*/
parm = p_find(vblock, "PROCOPT");
strcpy(procopt, SVAL(*parm,0));
len = (long)strlen(procopt);
c_low2up(procopt, &len);

return(E_SUCC);
}
