/*******************************************************************************
FUNCTION:	update_dbase

PURPOSE:	Update the data base record for each file group that was
		stacked to an SFL AVHRR archive tape.  The data base records
		are stored in the ADAPSDBTMP directory.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    01/94  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "avhrrstack_.h"
#include "queue.h"
#include "adapsutil.h"


#define IDLEN 16	/* Length of a AVHRR scene ID                         */

#define MIN(a,b) ((a) < (b) ? (a) : (b))


int update_dbase
(
    long *queue,	/* IN - Queue of file groups on the tape              */
    char *tapeid,	/* IN - ID of tape the file groups were stacked to    */
    char *logfile 	/* IN - Host file name of the log file                */
)

{
char adapsdb[CMLEN];	/* ADAPSDB directory                                  */
char adapsdbtmp[CMLEN];	/* ADAPSDBTMP directory                               */
char *ptr;		/* Ptr to a dynamically allocated string              */
char scene_id[IDLEN+1];	/* Scene ID of the current file group                 */
char str[CMLEN];	/* Temporary string                                   */

long nonfatal =  1;	/* Nonfatal error flag                                */



/* Retrieve the value of the environment variable ADAPSDBTMP
------------------------------------------------------------*/
ptr = getenv("ADAPSDBTMP");
if (ptr == NULL)
    {
    c_errmsg("Error retrieving ADAPSDBTMP", "updatedb-env", &nonfatal);
    return(E_FAIL);
    }
sprintf(adapsdbtmp, "%s", ptr);


/* Retrieve the value of the environment variable ADAPSDB
---------------------------------------------------------*/
ptr = getenv("ADAPSDB");
if (ptr == NULL)
    {
    c_errmsg("Error retrieving ADAPSDB", "updatedb-env", &nonfatal);
    return(E_FAIL);
    }
sprintf(adapsdb, "%s", ptr);


/* Update the data base record for each file group in the queue
---------------------------------------------------------------*/
c_qfirst(*queue, (void **)&ptr);
while (ptr != NULL)
    {
    /* Shorten the group name to a IDLEN character scene ID
    -------------------------------------------------------*/
    strncpy(scene_id, ptr, IDLEN);
    scene_id[IDLEN] = '\0';


    /* Update the data base record for this one file group
    ------------------------------------------------------*/
    if (update_single(adapsdbtmp, adapsdb, tapeid, ptr) != E_SUCC)
	{
	sprintf(str, "Error updating database record for %s", ptr);
	c_errmsg(str, "updatedb-update", &nonfatal);
	sprintf(str, "Error updating database record");
	c_wrtlog(logfile, scene_id, "updatedb-error", str);
	}
    else
	{
	sprintf(str, "Updated data base for %s", ptr);
	c_errmsg(str, " ", &nonfatal);
	c_wrtlog(logfile, scene_id, "updatedb-update", str);
	}


    /* Get the next file group from the queue
    -----------------------------------------*/
    c_qnext(*queue, (void **)&ptr);
    }


return(E_SUCC);
}




long update_single
(
    char *adapsdbtmp,	/* IN - ADAPSDBTMP directory                          */
    char *adapsdb,	/* IN - ADAPSDB directory                             */
    char *tapeid,	/* IN - ID of the tape the image was stacked to       */
    char *root 		/* IN - Root file name of the image                   */
)

{
char buffer[CMLEN];		/* Buffer for reading data base record        */
char dbtmp_orig[CMLEN];		/* Name of the temporary data base record     */
char dbtmp_final[CMLEN];	/* Name of the temporary data base record     */
char dbrec_updated[CMLEN];	/* Name of the updated data base record       */
char dbrec_final[CMLEN];	/* Name of the updated data base record       */
char str[CMLEN];		/* Temporary string                           */

long nonfatal =  1;		/* Nonfatal error flag                        */

FILE *fp_new;			/* File ptr to the new data base record file  */
FILE *fp_old;			/* File ptr to the old data base record file  */



/* Create the names of the necessary files.
   dbtmp_orig   = Original data base record w/o tapeid in ADAPSDBTMP.  These
		  files have the .db extension.
   dbtmp_final  = After copying the original data base record and updating the
		  copy with the tapeid, the original record is left in
		  ADAPSDBTMP with the .done extension.
   dbrec_update = Copy of the original data base record with the tapeid.  This
		  file is temporarily located in the ADAPSDBTMP directory so
		  that a partial record never exists in the ADAPSDB directory.
   dbrec_final  = Once the update is completed the dbrec_update file is renamed
		  into the ADAPSDB directory.
------------------------------------------------------------------------------*/
sprintf(dbtmp_orig,    "%s/%s.db",   adapsdbtmp, root);
sprintf(dbtmp_final,   "%s/%s.done", adapsdbtmp, root);
sprintf(dbrec_updated, "%s/%s.dat",  adapsdbtmp, root);
sprintf(dbrec_final,   "%s/%s.dat",  adapsdb, root);


/* Open the files
-----------------*/
fp_old = fopen(dbtmp_orig, "r");
if (fp_old == NULL)
    {
    sprintf(str, "Error opening %s.db file", root);
    c_errmsg(str, "updatedb-open", &nonfatal);
    return(E_FAIL);
    }
fp_new = fopen(dbrec_updated, "w");
if (fp_new == NULL)
    {
    sprintf(str, "Error opening %s.dat file", root);
    c_errmsg(str, "updatedb-open", &nonfatal);
    fclose(fp_old);
    return(E_FAIL);
    }


/* Copy the SOURCE record
-------------------------*/
fgets(buffer, CMLEN, fp_old);
fputs(buffer, fp_new);


/* Read the ARCHIVE record and update the tape storage location
---------------------------------------------------------------*/
fgets(buffer, CMLEN, fp_old);
strncpy(&buffer[IDLEN+1], tapeid, MIN(strlen(tapeid),6));
fputs(buffer, fp_new);


/* Read the GEO-LOCATION records
--------------------------------*/
while (fgets(buffer, CMLEN, fp_old) != NULL)
    fputs(buffer, fp_new);


/* Close the files
------------------*/
fclose(fp_old);
fclose(fp_new);


/* Rename the data base records
-------------------------------*/
rename(dbtmp_orig, dbtmp_final);
if (rename(dbrec_updated, dbrec_final))
    return(E_FAIL);

return(E_SUCC);
}
