/*******************************************************************************
FUNCTION:	OUTBR

PURPOSE:	Read a LAS image and write a browse file to disk

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    08/90  D.Hollaren       Modified version of VMSOUT
  5.0    03/92  E.Cihak/         LAS-5.0 version for ADAPS
                T.Rockvam
  5.1    09/93  J.Fenno          Correct indexing problem.  Last line was not
				 being written out when NL - 1 was divisible by
				 four.
  5.2    12/96  T.Baltzer        Call c_pxcopy() instead of strncpy() to load
				 outbuf.  strncpy() stops at "\0" creating
				 partially zeroed lines.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include "las.h"	
#include "util.h"	
#include "pixman.h"	
#include "browse.h"	

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC *fdesc[1];  /* Image file descriptors used for image I/O */
extern long curent;		/* Current processing line		     */
extern long curcat;		/* Current processing band		     */

void outbr
(
    struct PARBLK *vblock,	/* TAE parameter block			*/
    char *hosin,		/* host name of input image		*/
    char oname[][CMLEN],	/* output file names			*/
    long *nmrout,		/* number of output files		*/
    long window[],		/* window specifier			*/
    long outlines,		/* number of lines after subsampling	*/
    long outsamps,		/* number of samples after subsampling	*/
    long bands[],		/* band numbers specified		*/
    long nbands,		/* number of bands			*/
    long *idtype,		/* input data type 			*/
    long *msgtim,		/* message time counter			*/
    long *lskip,		/* number of lines to skip to		*/
    long *sskip,		/* number samples to skip to subsample	*/
    struct LINE *brw_line	/* header record for output file	*/
)

{
int	outfd;			/* output file descriptor	     	*/

long 	acc=IREAD,		/* image file open access	     	*/
	line,			/* loop counter for i/o blocks       	*/
	fatal = -1,		/* fatal error flag		     	*/
	flag = 1,		/* display message flag		     	*/
	i,j,			/* loop counter                      	*/
	lut[1024],              /* look-up table for byte data       	*/
	nbnd = 0,		/* open and read the number of bands 	*/
	nonfatal = 1,		/* nonfatal error flag		     	*/
	odtype = EBYTE,		/* output data type		     	*/
	one = 1,                /* constant one                      	*/
	opt = 0,		/* image file open option	     	*/
	size,    		/* number of bytes/pixel	     	*/
	status;			/* status indicator		     	*/

unsigned short  recsize = 408;	/* record size of browse image file	*/
unsigned short  hdrsize = 114;	/* record size of browse image file	*/

double  maxm,			/* maximum pixel value from pxmnmx   	*/
	minm;			/* minimum pixel value from pxmnmx   	*/

char	cmin[4],		/* new minimum pixel value string	*/
	cmax[4];		/* new maximum pixel value string	*/

unsigned char	*thebuf;	/* input buffer			     	*/
unsigned char   *outbuf;        /* output buffer with subsampled data	*/

/*
  Determine bytes/pixel of the input file
*/
switch(*idtype)
    {
    case EBYTE:  
	size = 1; 
	break;
    case EWORD:  
	size = 2; 
	/* scale from 0 - 1023 to 0 - 255 because AVHRR is 10-bit data */
	for (i = 0; i < 1024; i++)
	    lut[i] = (long)(255.0 * (i / 1023.0));
	break;
    }

/*
  Open the image file for read access	
*/
status = c_eopenr(fdesc,hosin,&nbnd,&window[SL],&window[SS],&window[NL],  
		   &window[NS],&acc,idtype,&opt,&one);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","browse-fatal",&fatal);

/*
  Allocate the input buffer
*/
if ((thebuf = (unsigned char *) malloc(window[NS] * size)) == NULL) 
    {
    c_eclose(fdesc);
    c_errmsg("Error allocating dynamic memory","browse-alloc",&nonfatal);
    c_errmsg("Fatal error encountered","browse-fatal",&fatal);
    }

/*
  Allocate the output buffer
*/
if ((outbuf = (unsigned char *) calloc(recsize,1)) == NULL) 
    {
    c_eclose(fdesc);
    c_errmsg("Error allocating dynamic memory","browse-alloc", &nonfatal);
    c_errmsg("Fatal error encountered","browse-fatal",&fatal);
    }

/*
  Initialize the processing message
*/
curcat=1; 	/* bands */
curent=1;	/* lines */
c_inittm("BROWSE","Processed","line","band"," ",&outlines,
         &nbands,msgtim,&flag);
c_startm();

/*
  Do for each band
*/
for (curcat = 1; curcat <= nbands; curcat++)
   {
   /*
     If each band goes to separate output file
     or this is the first band, open a new file
     and write out the header record
   */
   if ((*nmrout == nbands) || (curcat == 1))
	{
	outfd = open(oname[curcat-1],O_CREAT | O_WRONLY | O_TRUNC,0644);
	if (outfd < 0)
	    { 
            c_eclose(fdesc);
	    close(outfd);
	    c_errmsg("Error opening output image","browse-outopen",&nonfatal);
            c_clean(vblock,"browse",(const char (*)[256])oname,nmrout);
	    }

	/*
  	  Write header record
	*/
	status = write(outfd,(char *)brw_line,hdrsize);
	if (status < 0)
	   {
           c_eclose(fdesc);
	   close(outfd);
	   c_errmsg("Error writing header record","browse-write",&nonfatal);
           c_clean(vblock,"browse",(const char (*)[256])oname,nmrout); 
	   }
        }

    /*
      Read lines from the LAS image file
      and write out in browse format
    */
    maxm = 0.0;
    minm = 255.0;
    curent = 1;
    for (line = 1; line <= window[NL]; line += *lskip, curent++)
	{
	status = c_eread(fdesc,&bands[curcat-1],&line,thebuf,&one); 
	if (status != window[NS])
                {
                c_eclose(fdesc);
	        close(outfd);
                c_clean(vblock,"browse",(const char (*)[256])oname,nmrout);
                }
	if (*lskip != 1)
	    {
	    /*
  	      Subsample the input buffer
	    */
	    c_pxsamp(&window[NS],idtype,sskip,thebuf);
	    }

	if (*idtype == 1)
	    /*
  	      Input data is byte, copy into outbuf
	    */
	    c_pxcopy(thebuf, outbuf, idtype, &outsamps);
	else 
	    {
	    /*
  	      Input data is I*2, scale down into outbuf
	    */
	    for (j = 0; j < outsamps; j++) 
		{
		if (*((short *)thebuf+j) < 0)
                    outbuf[j] = (unsigned char)lut[0]; 
		else if (*((short *)thebuf+j) > 1023)
                    outbuf[j] = (unsigned char)lut[1023]; 
		else
                    outbuf[j] = (unsigned char)lut[*((short *)thebuf+j)]; 
		}
            }

	/*
  	  Update min/max values
	*/
	if ((c_pxmnmx(outbuf,&outsamps,&minm,&maxm,&odtype)) != E_SUCC)
	    {
            c_eclose(fdesc);
	    close(outfd);
	    c_clean(vblock,"browse",(const char (*)[256])oname,nmrout);
	    }

	/*
  	  Write input data to output browse file
	*/
        status = write(outfd,outbuf,recsize);
        if (status < 0)
            {
            c_eclose(fdesc);
	    close(outfd);
	    c_errmsg("Error writing output image","browse-write",
	        &nonfatal);
            c_clean(vblock,"browse",(const char (*)[256])oname,nmrout);
            }
	}

    /*
      Finish up
    */
    if ((*nmrout == nbands)||(curcat == nbands))
	{
	/*
	  Set up new max/min
	*/
	sprintf(cmin,"%3d",(long)minm);
	sprintf(cmax,"%3d",(long)maxm);
	strncpy(&brw_line->header[30],cmin,3);
	strncpy(&brw_line->header[33],cmax,3);

	/*
	  Move back to beginning of browse file and rewrite header 
	  with new min/max of the browse
	*/
	if (lseek(outfd,0,0) < 0)
    	    {
            c_eclose(fdesc);
            close(outfd);
    	    c_errmsg("Error resetting the browse file","browse-reset",
		&nonfatal);
            c_clean(vblock,"browse",(const char (*)[256])oname,nmrout);
    	    }
        status = write(outfd,(char *)brw_line,hdrsize);
        if (status < 0)
            {
            c_eclose(fdesc);
            close(outfd);
            c_errmsg("Error writing output image","browse-write",
	        &nonfatal);
            c_clean(vblock,"browse",(const char (*)[256])oname,nmrout);
            }
	/*
	  Close output file 
	*/
        close(outfd);
	}

    } /* END OF curcat <= nbands FOR LOOP */

/*
  Stop the processing message
*/
c_stoptm();

/*
  Free the allocated buffer space
*/
free (outbuf);
free (thebuf);

/*
  Close the input files
*/
c_eclose(fdesc);

return;
}
