#!/bin/csh -f
#
# FUNCTION: bwtrans.csh
#
# PURPOSE:  Perform automatic transfer of AVHRR browse images to GLIS
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# T.Rockvam        ??/??  Development
# J.Fenno          01/93  Modified to use rcp instead of ncp.
#                         Converted from sh to csh
# T.Baltzer	   05/94  Add stop file check
# D.Lloyd	   02/99  Changed the primary browse system to edcsgs12
#			  but also left the transfer to glis as well per
#			  Dave J. as an interim safety factor
# D.Lloyd	   03/99  Removed the transfer to glis
#
#


#------------------------
# Set up global variables
#------------------------
set noglob
set brwsys  = edcsns17
set brwdir  = /edcsns17/browse1/avhrr/avhrr.in
set logfile = `echo "[$ADAPSLOG]bwtrans" | tr / . | sed  's/\[\./\[/'`
set hostlogfile = "$ADAPSLOG/bwtrans.log"
set ntries  = 3
set opsuser = "ops"


#-----------------------------------------------------------------
# Set up csh.  If there is a Berkley version of rcp use it instead
#-----------------------------------------------------------------
unalias cd
unalias rm
unalias rcp
if (-e /usr/bsd/rcp) alias rcp /usr/bsd/rcp


#---------------------------------------------------------
# Ensure the path to write_log was supplied as an argument
#---------------------------------------------------------
if ($#argv != 2) then
   echo ""
   echo "Usage: bwtrans <lpath> <wpath>"
   echo ""
   echo "       <lpath> is the directory where the lsfiles command is located."
   echo "       <wpath> is the directory where the write_log command is"
   echo "       located."
   echo ""
   exit 1
   endif


#------------------------------------------------------------------------------
# Set up an alias for the lsfiles and write_log commands.  The paths could be
# the current directory in which case we must set up an alias before we change
# directories.
#------------------------------------------------------------------------------
if ("$1" == ".") then
   alias lsfiles `pwd`/lsfiles
else
   alias lsfiles $1/lsfiles
   endif
if ("$2" == ".") then
   alias wrtlog `pwd`/write_log
else
   alias wrtlog $2/write_log
   endif


#----------------------------
# cd to the correct directory
#----------------------------
cd $ADAPSBRW

set stopfile = $ADAPSTABLES/bwtrans_daemon.stop


#-----------------------------------
# Keep looking for files to transfer
#-----------------------------------
# Only going to run once and let the daemon call this at intervals
#while (1)
   # -------------------------------------------
   # Touch the logfile so that the adapsmonitor
   # knows that bwtrans is still running
   #-------------------------------------------
   touch $hostlogfile

   #--------------------------------------------
   # Transfer all files in the current directory
   # NOTE:
   # The permanent file matching scheme:
   #    set files = `lsfiles '\.brw$'`
   # has been altered to be:
   #    set files = `lsfiles '^a[ghl].*\.brw$'`
   # This is to discriminate against stitched
   # orbits (ao....) which are not to be
   # treated like other images at this time.
   #--------------------------------------------
   unset noglob
   set files = `lsfiles '^a[ghl].*\.brw$'`
   set noglob
   foreach file ($files)
      if (-z $file) continue

      #----------------------------------------------
      # Retrieve just the scene ID from the file name
      #----------------------------------------------
      set sceneid = `echo $file | awk -F\_ '{print $1}'`

      #-----------------------------------------------------
      # Strip off the extension for the file name on $brwsys
      #-----------------------------------------------------
      set newfile = `echo $file | awk -F\. '{print $1}' | tr "[A-Z]" "[a-z]"`
      
      #--------------------------------
      # Issue a message to the log file
      #--------------------------------
      set text = "Starting the browse transfer"
      wrtlog $logfile $sceneid bwtrans-start $text

      # Convert to jpeg
      $ADAPSCONTRIB/gdal_translate -of JPEG -outsize 20% 20%  $file $newfile
      if ( $status ) then
          # Error
	  set text = "Error creating jpeg"
	  wrtlog $logfile $sceneid bwtrans-error $text ""
	  mv $file $ADAPSERR/.
      else
	  #-------------------
	  # Transfer the image
	  #-------------------
	  @ i = 0
	  while ($i < $ntries)
	     @ i += 1
	     rcp -p $newfile $opsuser"@"$brwsys":"$brwdir/$newfile
	     if (! $status) then
		#rsh $opsuser"@"$brwsys chmod 664 $brwdir/$newfile
                rsh -l $opsuser $brwsys chmod 664 $brwdir/$newfile
		if ( $status ) then
		    set text = "ERROR changing modes on "
		    wrtlog $logfile $sceneid bwtrans-error $text $brwsys
		endif
		rm -f $file $newfile
		set text = "Completed the browse transfer to "
		wrtlog $logfile $sceneid bwtrans-success $text $brwsys
		break
	     else if ($i < $ntries) then
		sleep 60
	     else
		mv $file $ADAPSERR/.
		rm -f $newfile
		set text = "ERROR encountered in the browse transfer to "
		wrtlog $logfile $sceneid bwtrans-error $text $brwsys
	     endif
	  end
      endif

      #----------------------------------------------------------
      # Check to see if the user is trying to stop this process
      #----------------------------------------------------------
      if (-e $stopfile) then
         exit 0
      endif

   end

#end

exit 0
