#!/bin/csh -f
#
# FUNCTION:  ckbaa.csh
#
# PURPOSE:   Check the images sent to us from Buenos Aires, Argentina.  The
#            images are in the Miami format.  If the header is in the correct
#            format then remove the header from each image.
#
# Programmer       Date   Modification
# ---------------  -----  ------------------------------------------------------
# J.Fenno          05/92  Development
# J.Fenno          12/92  Combined into one pdf called cktrans this then calls
#                         ckbaa.csh or cklsu.csh.  Modified so the directory
#                         is not hard coded but passed as an argument.
# J.Fenno          01/93  Added check so that multiple images are not renamed
#                         to the same acquire image name.
#                         Deleted the code to remove the Miami header from the
#                         image and then extract the correct number of scan
#                         lines.  This will now be done in INGEST.
# J.Fenno          03/94  Added code to create a temporary pdf which will
#                         return the number of images checked.
# J.Fenno          04/95  Shipment of tapes with a new record size.  Word three
#                         of the header is now c439.  These still process ok.
#
#


#---------------------------------------------------------------------------
# Set up the shell.  Set noglob so the error messages are printed correctly.
#---------------------------------------------------------------------------
onintr -
set dstdir = $ADAPSACQ
set nbytes = 500000000
set printflag = "0"
set noglob
unalias cd
unalias df
unalias du
unalias ls
unalias rm


#-----------------------------------------------------------------------------
# Ensure there are five arguments.  These are the directory where the images
# are located and the paths for the commands get_num.csh, checkspace, lsfiles,
# and write_log respectively.
#-----------------------------------------------------------------------------
if ($#argv != 6) then
    echo "[cktrans-args] Must have six arguments for ckbaa.csh"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#-----------------------------------------------------------------------------
# The calling pdf expects a temporary pdf called $tmppdf that will return the
# number of images that were checked.  Create this pdf so that it returns zero
# in the event of an error.
#-----------------------------------------------------------------------------
set tmppdf = `pwd`/nbaachecked$6.pdf

echo "procedure"		>  $tmppdf
echo "parm num type=name"	>> $tmppdf
echo "body"			>> $tmppdf
echo "let num = 0"		>> $tmppdf
echo "ush rm -f $tmppdf"	>> $tmppdf
echo "end-proc"			>> $tmppdf


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we will be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------
set srcdir = $1
if ("$2" == ".") then
    alias get_num.csh `pwd`/get_num.csh
else
    alias get_num.csh $2/get_num.csh
endif
if ("$3" == ".") then
    alias ckspace `pwd`/checkspace
else
    alias ckspace $3/checkspace
endif
if ("$4" == ".") then
    alias lsfiles `pwd`/lsfiles
else
    alias lsfiles $4/lsfiles
endif
if ("$5" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $5/write_log
endif

#- $6 is the unique session number -#


#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSACQ) || (! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[cktrans-env] ADAPSACQ, ADAPSLOG, or ADAPSERR is not set"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#--------------------------------------------------------
# Name of log file.  This must be a LAS name for wrtlog.
#--------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]cktrans" | tr / . | sed  's/\[\./\[/'`


#----------------------------------------------------------------------------
# Check to see if there are any images pulled from BAA.  These images will be
# named like 92092063917.n11.  If none exist then issue a message and exit.
#----------------------------------------------------------------------------
cd $srcdir
set images = `lsfiles '^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]\.n1[0-9]$'`
if ("$images" == "") then
    #-------------------------------------------------------------------
    #if there aren't any files we don't want any messages to the logfile
    # so we don't fill up the log file
    #---------------------------------------------------
    set text = "No BAA images"
    tty -s
    if ($status == 0) then
	echo "[informational] $text"
    endif
    goto NO_FILES
else
    set text = "Starting to check BAA images"
    echo "[informational] $text"

    #-------------------------------------------------------------------------
    # Recreate the temporary pdf that will return the number of images checked
    #-------------------------------------------------------------------------
    echo "procedure"        >  $tmppdf
    echo "parm num type=name"   >> $tmppdf
    echo "body"         >> $tmppdf
    echo "let num = $#images"   >> $tmppdf
    echo "ush rm -f $tmppdf"    >> $tmppdf
    echo "end-proc"         >> $tmppdf
endif


#-------------------------------------------------------------------------------
# All Miami images must have the following hex values for words 3, 5, 11 and 12
# in the first 512 byte block.  If any of these differ then we must manually
# inspect the image and most likely have a new image sent in the correct format.
#
# In addition word 7 of the header specifies the number of scan lines in the
# image.  This will be used to dd just the raw AVHRR data portion of the Miami
# images.
#
#    Word  3 = e639 | c439   Record length in bytes.
#    Word  5 = 0400          Offset to image data in 512 byte blocks
#    Word  7 = ????          Number of scan lines contained in the image
#    Word 11 = 0100          Data compression flag (0100 = compressed)
#    Word 12 = 0100          Byte order flag (0100 = VAX byte order)
#-------------------------------------------------------------------------------
set word03_1 = "e639"
set word03_2 = "c439"
set word03_3 = "003e"
set word05   = "0400"
set word11   = "0100"
set word12   = "0100"
set word1 = "aaaa"
set word2 = "aaaa"
set word4 = "0400"
set word5 = "2400"



#--------------------------------
# Create two temporary file names
#--------------------------------
set tmp_file = `date '+ckbaa_hdr_%Y%b%d_%H%M%S' | tr "[A-Z]" "[a-z]"`
set hex_file = $tmp_file.hex


#-----------------------------------------------------------------------
# For each image pulled from BAA check the Miami header within the image
#-----------------------------------------------------------------------
foreach image ($images)
    if (-e $srcdir/$image.done) then
	onintr ERROUT
	set filenum = `get_num.csh $srcdir`
	set key     = "netbaa-$filenum-baa"
	rm -f $tmp_file $hex_file

	if (! -d $image) then
	    #-------------------------------------------------------------------
	    # Pull off first 512 bytes of the image and create a hex dump of it
	    #-------------------------------------------------------------------
	    dd if=$image of=$tmp_file bs=512 count=1 >& /dev/null
	    od -x $tmp_file >! $hex_file
	    rm -f $tmp_file


	    #------------------------------------
	    # Retrieve the five words of interest
	    #------------------------------------
	    set head03 = `cat $hex_file | awk '(NR == 1) {print $4}'`
	    set head05 = `cat $hex_file | awk '(NR == 1) {print $6}'`
	    set head07 = `cat $hex_file | awk '(NR == 1) {print $8}'`
	    set head11 = `cat $hex_file | awk '(NR == 2) {print $4}'`
	    set head12 = `cat $hex_file | awk '(NR == 2) {print $5}'`
	    set head1  = `cat $hex_file | awk '(NR == 1) {print $2}'`
	    set head2  = `cat $hex_file | awk '(NR == 1) {print $3}'`
	    set head4  = `cat $hex_file | awk '(NR == 1) {print $5}'`
	    set head5  = `cat $hex_file | awk '(NR == 1) {print $6}'`
	    rm -f $hex_file


	    # First check for a level 1b header
	    #----------------------------------
	    if (("$head1" == "$word1") && ("$head2" == "$word2") && \
	        ("$head4" == "$word4") && ("$head5" == "$word5")) then
		# Make sure this is a KLM scene
		if ( "$head03" == "$word03_3" ) then
		    set extension = "ba2"

		# If in future they send a different
		# record size in level 1b format one
		# could check that size here and set
		# a different extension.

		# Otherwise it's an error for now.
		#-----------------------------------
		else
		    set text = "Format is not KLM data size"
		    wrtlog $logfile $key ckbaa-error $text
		    echo "[ckbaa-hdr] $text"
		    mv $image $ADAPSERR/.
		    mv ${image}.done $ADAPSERR/.
		    continue
		endif

	    # Now check for the older header
	    #-------------------------------
	    else if ((("$head03" == "$word03_1") || \
		("$head03" == "$word03_2")) && \
		("$head05" == "$word05") && \
		("$head11" == "$word11") && \
		("$head12" == "$word12")) then
		set extension = "baa"

	    else
		# If neither one it's an error
		#-----------------------------
		set text = "Invalid header found in $image"
		wrtlog $logfile $key ckbaa-error $text
		echo "[ckbaa-hdr] $text"
		mv $image $ADAPSERR/.
		mv ${image}.done $ADAPSERR/.
		continue
	    endif


	    #---------------------------------------
	    # Move the image to the new acquire name
	    #---------------------------------------
	    set year = `echo $image | awk '{printf "%2.2s",$1}'`
	    set newimage = "akm_baa_"$year"_fn$filenum.$extension"
	    if (-e $newimage) then
		set text = "Image $image could not be renamed to $newimage"
		wrtlog $logfile $key cktrans-error $text
		echo "[cktrans-exists] $text"
	    else
		mv $image $newimage
		set text = "$image renamed to $newimage"
		wrtlog $logfile $key ckbaa-rename $text
		echo "[informational] $text"
	    endif

	# Remove the done file
	#---------------------
	rm -f $srcdir/$image.done
	endif
    else
        #seeing if the file is over a day old
        # (not have the check sum file and over
        # a day old means that the file needs to
        # be resent)
        #if $test = 0 the file is not over a day old
        #if $test = 1 the file is over a day old
        set test = 0
        set test = `find $srcdir -name $image -mtime +1 -exec "echo" 1 \;`
        if ($test) then
            mv $srcdir/$image $ADAPSERR/$image
            set filenum = `get_num.csh $srcdir`
            set key     = "netbaa-$filenum-baa"
            set text = "No check sum file for $image - moved to ADAPSERR"
            wrtlog $logfile $key ckbaa-error $text
            echo "[ckbaa-sum] $text"
        endif
        continue
    endif

end

#----------------------------------------------------------------
# Move all of the files matching akm_baa_??_fn???.baa to ADAPSACQ
#----------------------------------------------------------------
set images = `lsfiles '^akm_baa_[0-9][0-9]_fn[0-9][0-9][0-9]\.ba[2a]$'`
foreach image ($images)
    #---------------------------------------------------------------------------
    # Move the image to ADAPSACQ.  Just in case this directory is on another
    # disk pack, move the file in such a way that the automatic ingester doesn't
    # grab it before we are done getting it there.  Move it there using a
    # temporary name and then rename the file once its there.
    #---------------------------------------------------------------------------
    if ( ! { ckspace "" $nbytes $ADAPSACQ } ) then
	set text = "Not enough room in ADAPSACQ ($nbytes bytes)"
	wrtlog $logfile cktrans ckbaa-disk $text
	echo "[cktrans-disk] $text"
	break
    else if ( { mv $image $ADAPSACQ/_tmp_$image } ) then
	onintr -
	mv $ADAPSACQ/_tmp_$image $ADAPSACQ/$image
	set text = "$image moved to ADAPSACQ"
	wrtlog $logfile cktrans ckbaa-move $text
	echo "[informational] $text"
        set printflag = "1"
    else
	onintr -
	rm -f $ADAPSACQ/_tmp_$image
	set text = "Error moving $image to ADAPSACQ"
	wrtlog $logfile cktrans ckbaa-error $text
	echo "[cktrans-move] $text"
    endif
end

#--------------------------------------
# Issue a successful completion message
#--------------------------------------
NO_FILES:
set text = "CKBAA completed successfully"
#-------------------------------------------------------------------
#if there aren't any files we don't want any messages to the logfile
#-------------------------------------------------------------------
if ("$printflag" != "0") then
   wrtlog $logfile cktrans ckbaa-success $text
endif
echo "[informational] $text"
exit 0



ERROUT:
    onintr -
    set text = "User aborted CKBAA"
    wrtlog $logfile $key ckbaa-error $text
    echo "[cktrans-abort] $text"
    echo "[cktrans-fatal] Fatal error encountered"

    if ($?exchange) then
	rm -f $image
	rm -f $hex_file
	mv $tmp_file $image
    else
	rm -f $tmp_file
	rm -f $hex_file
    endif
    exit 1
