/*******************************************************************************
FUNCTION:	cknsidc

PURPOSE:	Check all of the images transferred across the network from
		the Nation Snow and Ice Data Center (NSIDC).  NSIDC is using
		our ADAPS system to process their data so minimal checks are
		necessary to ensure the image and header files are correct.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    11/93  J.Fenno          Development
  1.1    03/94  J.Fenno          Added code to return the number of images that
				 where checked by the function.
  1.2    06/96  T.Baltzer        Look for ahdr files instead of arch files so
				 that checking is not done until the entire
				 image is transfered (assumes ahdr sent last).
  1.3    11/02  D.Lloyd          Cleaned up warnings and added includes


PROJECT: ADAPS

$Log: cknsidc.c,v $
Revision 3.8  2002/11/20 14:49:27  dlloyd
Fixed two warning on the sns1 make.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util2.h"
#include "labserv.h"
#include "cm.h"
#include "adaps.h"
#include "adapsutil.h"
#include "adapssup.h"
#include "archio.h"

#define MXFILES 20		/* Max # of files returned by c_getfiles()    */


int main()
{
char adapsing[CMLEN];		/* ADAPSING directory                         */
char adapserr[CMLEN];		/* ADAPSERR directory                         */
char ahdr[CMLEN];		/* Name of the AVHRR header file              */
char arch[CMLEN];		/* Name of the AVHRR image file               */
char dir[CMLEN];		/* Directory where images are located         */
char err_ahdr[CMLEN];		/* Name for the ahdr image if an error found  */
char err_arch[CMLEN];		/* Name for the arch image if an error found  */
char files[MXFILES][CMLEN];	/* Host names of images to be checked         */
char logfile[CMLEN];		/* Name of the log file                       */
char text[CMLEN];		/* Text string for error and log messages     */
char tmp1[CMLEN];		/* Temporary name                             */
char tmp2[CMLEN];		/* Temporary name                             */

double sizes[MXFILES];		/* Size of each file in files                 */

long fatal      = -1;		/* Fatal error flag                           */
long first_call;		/* Flag for first call to c_getfiles()        */
long i;				/* Loop control                               */
int nfiles;			/* Number of files to be checked              */
int nimages;			/* Number of images checked by the function   */
long nonfatal   =  1;		/* Nonfatal error flag                        */
long now;			/* Current date and time encoded as a long    */
long status;			/* Function return status                     */
long std_out;			/* Standard output file                       */

int printflag;			/* Flag to tell if a log message is needed    */

struct DROPPED drop;		/* AVHRR dropped line structure               */
struct MODEL model;		/* AVHRR model structure                      */
struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE *parm;		/* Ptr to a TAE variable                      */
struct tm *today;		/* Structure containing current date and time */



/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the directory containing the images
-----------------------------------------------*/
parm = p_find(&vblock, "INDIR");
strcpy(dir, SVAL(*parm, 0));


/* Change into the specified directory
--------------------------------------*/
if (chdir(dir))
    {
    c_errmsg("Error accessing INDIR", "cktrans-dir", &nonfatal);
    c_errmsg("Fatal error encountered", "cktrans-fatal", &fatal);
    }


/* Get the required environment variables (assumed we are on unix)
------------------------------------------------------------------*/
if ((getenv("ADAPSING") == NULL) || (getenv("ADAPSLOG") == NULL) ||
	(getenv("ADAPSERR") == NULL))
    {
    c_errmsg("ADAPSING, ADAPSLOG, or ADAPSERR is not set", "cktrans-env", &nonfatal);
    c_errmsg("Fatal error encountered", "cktrans-fatal", &fatal);
    }
sprintf(adapsing, "%s/", getenv("ADAPSING"));
sprintf(logfile, "%s/cktrans.log", getenv("ADAPSLOG"));
sprintf(adapserr, "%s/", getenv("ADAPSERR"));

/* Check all of the images matching a*.ahdr, assume there is a corresponding
   a*.arch available for each a*.ahdr.
----------------------------------------------------------------------------*/
first_call = TRUE;
nfiles = MXFILES;
nimages = 0;
printflag = 0;
while (nfiles == MXFILES)
    {
    /* Get a list of files
    ----------------------*/
    status = c_getfiles("a*_*_*_*.ahdr",files,&nfiles,MXFILES,first_call,TRUE);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered", "cktrans-fatal", &fatal);
    
    /* Rename the files from "ahdr" to "arch" files
    ------------------------------------------------*/
    for (i = 0; i < nfiles; i++)
	strncpy(&files[i][strlen(files[i])-4],"arch\0",5);
	
    /* If there are no images then issue a message and quit
    -------------------------------------------------------*/
    if (first_call && (nfiles == 0))
	{

	/* Don't issue a message for this to the logfile 
	   because this message tends to fill up the logfile
	----------------------------------------------------*/
	sprintf(text, "No NSIDC images");
	c_errmsg(text, " ", &nonfatal);
	break;
	}


    /* Issue a message to the log file
    ----------------------------------*/
    sprintf(text, "Starting to check NSIDC images");
    c_wrtlog(logfile, "cktrans", "cknsidc-start", text);
    c_errmsg(text, " ", &nonfatal);


    /* Set the flag indicating we have already called c_getfiles()
    --------------------------------------------------------------*/
    first_call = FALSE;


    /* Get the sizes of each of the files
    -------------------------------------*/
    c_getsizes(files, nfiles, sizes);

    /* Increment the number of images that have been found
    ------------------------------------------------------*/
    nimages += nfiles;


    /* Check each of the files found
    --------------------------------*/
    for (i = 0; i < nfiles; i++)
	{
	/* Create the error name for the image
	--------------------------------------*/
	sprintf(err_arch, "%s/%s", adapserr,files[i]);
	sprintf(err_ahdr, "%s/%s", adapserr,files[i]);
	strncpy(&err_ahdr[strlen(err_ahdr)-4],"ahdr\0",5);


	/* Read the header for the current image
	----------------------------------------*/
	status = c_getahdr(files[i], &model, &drop);
	if (status != E_SUCC)
	    {
	    sprintf(text, "Error reading header for %s", files[i]);
	    c_wrtlog(logfile, "netnsidc", "cknsidc-hdr", text);
	    c_errmsg(text, "cknsidc-hdr", &nonfatal);
	    rename(files[i], err_arch);
	    strncpy(&files[i][strlen(files[i])-4],"ahdr\0",5);
            rename(files[i], err_ahdr);
	    continue;
	    }
	

	/* Check the size of the image and the number of lines
	------------------------------------------------------*/
	if (sizes[i] != (model.nlines * LAC_BYTES))
	    {
	    sprintf(text, "File size not multiple of %d for %s",
		LAC_BYTES, files[i]);
	    c_wrtlog(logfile, "netnsidc", "cknsidc-size", text);
	    c_errmsg(text, "cknsidc-size", &nonfatal);
	    rename(files[i], err_arch);
	    strncpy(&files[i][strlen(files[i])-4],"ahdr\0",5);
            rename(files[i], err_ahdr);
	    continue;
	    }
	

	/* Check the CEOS ID of the image.  NSIDC does not guarantee that
	   their IDs match the internationally approved ones.  If they are
	   not one of the expected IDs then error out.
	------------------------------------------------------------------*/
	if (strcmp(model.station, "MCM") == 0)
	    strcpy(model.station, "MMR");
	else if (strcmp(model.station, "PAL") == 0)
	    strcpy(model.station, "PLM");
	else
	    {
	    sprintf(text, "Unexpected CEOS ID found: %s", model.station);
	    c_wrtlog(logfile, "netnsidc", "cknsidc-ceos", text);
	    c_errmsg(text, "cknsidc-ceos", &nonfatal);
	    rename(files[i], err_arch);
	    strncpy(&files[i][strlen(files[i])-4],"ahdr\0",5);
            rename(files[i], err_ahdr);
	    continue;
	    }


	/* Create the EDC scene ID and determine the current date and time
	------------------------------------------------------------------*/
	c_getscene(&model, tmp1);
	now = time(NULL);
	today = localtime(&now);
	sprintf(tmp2, "%s_%02d%02d%02d", tmp1,
	    today->tm_hour, today->tm_min, today->tm_sec % 60);


	/* Save the name for the header and image files
	-----------------------------------------------*/
	sprintf(ahdr, "%s.ahdr", tmp2);
	sprintf(arch, "%s.arch", tmp2);


	/* Save the new header, rename the image file, and delete old header
	--------------------------------------------------------------------*/
	status = c_putahdr(arch, &model, &drop);
	if (status != E_SUCC)
	    {
	    sprintf(text, "Error saving header file");
	    c_wrtlog(logfile, "netnsidc", "cknsidc-save", text);
	    c_errmsg(text, "cknsidc-save", &nonfatal);
	    rename(files[i], err_arch);
	    strncpy(&files[i][strlen(files[i])-4],"ahdr\0",5);
            rename(files[i], err_ahdr);
	    continue;
	    }
	rename(files[i], arch);
	c_lsmknm(files[i], "ahdr", tmp1);
	remove(tmp1);


	/* Move the image to the ADAPSING directory for processing.  Because
	   we may be moving the image from one disk to another (ie. the files
	   must first be copied), care must be taken so that the image is not
	   processed before it is entirely moved into the appropriate
	   directory.  Therefore, a temporary name must be used.  In addition,
	   the C function rename() cannot be used across disk packs.
	----------------------------------------------------------------------*/
	sprintf(tmp1, "mv %s %s/tmp_%s", ahdr, adapsing, ahdr);
	system(tmp1);
	sprintf(tmp1, "mv %s/tmp_%s %s/%s", adapsing, ahdr, adapsing, ahdr);
	system(tmp1);

	sprintf(tmp1, "mv %s %s/tmp_%s", arch, adapsing, arch);
	system(tmp1);
	sprintf(tmp1, "mv %s/tmp_%s %s/%s", adapsing, arch, adapsing, arch);
	system(tmp1);

	/* Set the printflag */
	printflag = 1;

	/* Issue a message to the log file
	----------------------------------*/
	sprintf(text, "%s renamed to %s", files[i], arch);
	c_wrtlog(logfile, "netnsidc", "cknsidc-rename", text);
	c_errmsg(text, " ", &nonfatal);
	}
    }


/* Return the number of images found to TAE
-------------------------------------------*/
if ((q_intg(&vblock, "NIMAGES", 1, &nimages, P_UPDATE) != SUCCESS) ||
    (q_out(&vblock) != SUCCESS))
    {
    c_errmsg("Error sending parameter to TAE", "cknsidc-tae", &nonfatal);
    }


/* Issue a completion message
-----------------------------*/
sprintf(text, "CKNSIDC completed successfully");
/* if there were no files then don't print out the message
   so we don't fill up the log file
----------------------------------------------------------*/
if (printflag != 0)
    c_wrtlog(logfile, "netnsidc", "cknsidc-success", text);
c_errmsg(text, " ", &nonfatal);

z_exit(1, "CKNSIDC completed successfully");
return 0;
}
