#!/bin/csh -f
#
# FUNCTION:  ckopf.csh
#
# PURPOSE:   Check the images sent to us from Oberpfaffenhofen Germany
#
#
# Programmer       Date     Modification
# ---------------  -----  ------------------------------------------------------
# D Lloyd          06/97  Taken from ckutokyo.csh and modified for opf


#-------------------------------------------------------------------
# Set up the shell.  Set noglob so the errors are printed correctly.
#-------------------------------------------------------------------
#onintr -> tells the csh what to do when an interrupt happens
#	With the - argument, the shell ignores all interrupts.  
#	With a label argument, the shell executes a goto label
onintr -
set nbytes = 500000000
set printflag = "0"
set noglob
unalias cd
unalias df
unalias du
unalias ls
unalias rm


#-----------------------------------------------------------------------------
# Ensure there are six arguments.  These are the directory where the images
# are located and the paths for the commands checkspace, lsfiles, get_num.csh,
# write_log and then the session number respectively.
#-----------------------------------------------------------------------------
if ($#argv != 6) then
    echo "[cktrans-args] Must have six arguments for ckopf.csh"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#-----------------------------------------------------------------------------
# The calling pdf expects a temporary pdf called $tmppdf that will return the
# number of images that were checked.  Create this pdf so that it returns zero
# in the event of an error.
#-----------------------------------------------------------------------------
set tmppdf = `pwd`/opfchecked$6.pdf

echo "procedure"                >  $tmppdf
echo "parm num type=name"   >> $tmppdf
echo "body"                     >> $tmppdf
echo "let num = 0"          >> $tmppdf
echo "ush rm -f $tmppdf"        >> $tmppdf
echo "end-proc"                 >> $tmppdf


#----------------------------------------------------------------------------
# Set up an alias for all of the commands that have had their path sent as an
# argument.  Because we will be changing directories, change paths of "." to
# the current working directory.
#----------------------------------------------------------------------------

#the dir where the Oberpfaffenhofen images are placed
set srcdir = $1

if ("$2" == ".") then
    alias get_num.csh `pwd`/get_num.csh
else
    alias get_num.csh $2/get_num.csh
endif
if ("$3" == ".") then
    alias ckspace `pwd`/checkspace
else
    alias ckspace $3/checkspace
endif
if ("$4" == ".") then
    alias lsfiles `pwd`/lsfiles
else
    alias lsfiles $4/lsfiles
endif
if ("$5" == ".") then
    alias wrtlog `pwd`/write_log
else
    alias wrtlog $5/write_log
endif

#- $6 is the unique session number -#

#-------------------------------------------------------------
# Test for the existence of the required environment variables
#-------------------------------------------------------------
if ((! $?ADAPSACQ) || (! $?ADAPSLOG) || (! $?ADAPSERR)) then
    echo "[cktrans-env] ADAPSACQ, ADAPSLOG, or ADAPSERR are not set"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
endif


#----------------------------------------------------------------------
# Create the name of the log file.  This must be a LAS name for wrtlog.
#----------------------------------------------------------------------
set logfile = `echo "[$ADAPSLOG]cktrans" | tr / . | sed  's/\[\./\[/'`


#------------------------------------------------------------------------------
# Check for the existance of a .done file that is older than 15 minutes that
# doesn't have a corresponding image file.  O. Germany requested that we leave
# the .done file hanging around for a while so that they can insure the file
# transfer was complete.
#------------------------------------------------------------------------------
cd $srcdir
set donefiles = `lsfiles '.*done$'`

if ( "$donefiles" != "" ) then
    foreach done ($donefiles)
        # check for the existance of the image file
        set root = `echo $done | awk -F. '{print $1"."$2"."$3}'`
        if (! -e $root ) then
            # image file doesn't exist so see if .done file is at
            # least 15 minutes old and delete it if so
            set hour = `date +%H`
            set min = `date +%M`
            set filet = `ls -l $done | awk '{print $8}'`
            set fileh = `echo $filet | awk -F: '{print $1}'`
            set filem = `echo $filet | awk -F: '{print $2}'`
            if ( $hour != $fileh ) then
                @ min = $min + 60
            endif
            @ filem = $filem + 15
            if ( $min > $filem ) then
                rm -f $done
            endif
        endif
    end
endif
 

#------------------------------------------------------------------------------
# Check to see if there are any images pushed from Oberpfaffenhofen.
# These images will be named like xxN149705121055.hrpt.Z or .gz where xx will
# be OP for Oberpfaffenhofen or TF for O'Higgins.  If none exist
# then issue a message and exit.
#------------------------------------------------------------------------------
cd $srcdir
set images = `lsfiles '^[OT][PF]N............\.hrpt\..*[zZ]$'`

#if there weren't any files
if ("$images" == "") then
    #-------------------------------------------------------------------
    #if there aren't any files we don't want any messages to the logfile
    # so we don't fill up the log file
    #---------------------------------------------------
    set text = "No OPF images"
    tty -s
    if ($status == 0) then
	echo "[informational] $text"
    endif
    goto NO_FILES
else
    set text = "Starting to check OPF images"
    echo "[informational] $text"

    #-------------------------------------------------------------------------
    # Recreate the temporary pdf that will return the number of images checked
    #-------------------------------------------------------------------------
    echo "procedure"                >  $tmppdf
    echo "parm num type=name"   >> $tmppdf
    echo "body"                     >> $tmppdf
    echo "let num = $#images"   >> $tmppdf
    echo "ush rm -f $tmppdf"        >> $tmppdf
    echo "end-proc"                 >> $tmppdf
endif


#------------------------------------------------------------------------------
# For each image pushed from OPF check to see if there is a corresponding
# file with a .done extension.  If there is then Oberpfaffenhofen is done
# sending us the file.  So we can process the file.  If there isn't a .done
# file then ignore the file.
#------------------------------------------------------------------------------
foreach img ($images)
    onintr  ERROUT

    #creating the corresponding name for the file with a .done extension
    set done = ${img}.done
    #if the .done file doesn't exist then Oberpfaffenhofen is still transferring the
    # file to us (The .done file is the last to be sent)
    if (! -e $done) then
	#seeing if the file is over a day old
        # (not have the check sum file and over
        # a day old means that the file needs to
        # be resent)
        #if $test = 0 the file is not over a day old
        #if $test = 1 the file is over a day old
        set test = 0
        set test = `find $srcdir -name $img -mtime +1 -exec "echo" 1 \;`
        if ($test) then
            mv $srcdir/$img $ADAPSERR/$img
            set filenum = `get_num.csh $srcdir`
	    set key     = "netopf-$filenum-opf"
            set text = "No check sum file for $img - moved to ADAPSERR"
            wrtlog $logfile $key ckopf-error $text
            echo "[ckopf-sum] $text"
        endif
	continue
    endif

    #--------------------------------
    # seeing if the filesizes differ
    #--------------------------------
    #the size that Oberpfaffenhofen thought it was
    set remote = `cat $done`
    # the 5th column in ls -l is the size of the file we received
    set local = `/bin/ls -l $img | awk '{print $5}'`
    # if the size we received is not = to what Oberpfaffenhofen thought it was then
    #  put the image file and the .done file in $ADAPSERR
    if ("$remote" != "$local") then
	set text1 = "File sizes differ for $img."
	set text2 = "Local size = $local, remote size = $remote."
	echo "[ckopf-unequal] $text1"
	echo "[ckopf-unequal] $text2"
	wrtlog $logfile ckopf.csh [ckopf-unequal] $text1
	wrtlog $logfile ckopf.csh [ckopf-unequal] $text2
	mv $img $ADAPSERR/$img
	mv $done $ADAPSERR/$done
        continue
    endif

    #----------------------------------------------------
    # Uncompress and rename the image.  Uses the gunzip
    # routine since file can be compressed with either
    # compress (.Z) or gzip (.gz).  Strip the .Z or .gz
    # extension from the variable img.
    #----------------------------------------------------
    gunzip $img
    set filenum = `get_num.csh $srcdir`
    set cimg = $img
    set img = `echo $img | awk -F. '{printf "%s.%s", $1, $2}'`
 
    #----------------------------------------------------
    # Make sure the file was uncompressed.  If not, move
    # it to $ADAPSERR.
    #----------------------------------------------------
    if ( ! -e $img) then
	set text1 = "Error uncompressing $img"
	echo "[ckopf-uncompress] $text1"
	wrtlog $logfile ckopf.csh [ckopf-uncompress] $text1
	mv $cimg $ADAPSERR/$cimg
	mv $done $ADAPSERR/$done
	continue
    endif

    set year = `echo $img | awk -F. '{print substr($1, 6, 2)}'`
    set station = `echo $img | awk -F. '{printf "%2.2s", $1}'`
    if ( $station == "OP" ) then 
        set newimage = "akm_opf_"$year"_fn$filenum.opf"
    else 
        set newimage = "akm_ohg_"$year"_fn$filenum.ohg"
    endif
    if (-e $newimage) then
        set text = "Image $img could not be renamed to $newimage"
        wrtlog $logfile cktrans cktrans-error $text
        echo "[informational] $text"
    else
        mv $img $newimage
        set text = "$img renamed to $newimage"
        wrtlog $logfile cktrans ckopf-rename $text
        echo "[informational] $text"
    endif
    set img = $newimage

    #-----------------------------------------------------------------------
    # Move the image to ADAPSACQ.  Just in case the current directory is on
    # another disk pack, move the file in such a way that the automatic
    # ingester doesn't grab it before we are done getting it there.  Move it
    # there using a temporary name and then rename the file once its there.
    #-----------------------------------------------------------------------
    onintr REMOVE_IMG
    # if there isn't enough space....
    if ( ! { ckspace "" $nbytes $ADAPSACQ } ) then
	set text = "Not enough room in ADAPSACQ ($nbytes bytes)"
	wrtlog $logfile cktrans cktrans-disk $text
	echo "[cktrans-disk] $text"
	break
    #try to do the move to a temporary name and then rename it to original name
    else if ( { mv $img $ADAPSACQ/_tmp_$img } ) then
	onintr -
	mv $ADAPSACQ/_tmp_$img $ADAPSACQ/$img
	set text = "$img moved to ADAPSACQ"
	wrtlog $logfile cktrans ckopf-move $text
	echo "[informational] $text"
        set printflag = "1"
    else
	onintr -
	rm -f $ADAPSACQ/_tmp_$img
	set text = "Error moving $img to ADAPSACQ"
	wrtlog $logfile cktrans ckopf-error $text
	echo "[cktrans-move] $text"
    endif
end


#--------------------------------------
# Issue a successful completion message
#--------------------------------------
NO_FILES:
set text = "CKOPF completed successfully"
#if there aren't any files we don't want any messages to the logfile
if ("$printflag" != "0") then
   wrtlog $logfile cktrans ckopf-success $text
endif
echo "[informational] $text"
exit 0


REMOVE_IMG:
    onintr -
    rm -f $ADAPSACQ/$img

ERROUT:
    onintr -
    set text = "User aborted CKOPF"
    wrtlog $logfile cktrans cktrans-abort $text
    echo "[cktrans-abort] $text"
    echo "[cktrans-fatal] Fatal error encountered"
    exit 1
