/***************************************************************************
FUNCTION:	CREATMF

PURPOSE:	To create a monthly mean climatology (fill image) from
		a list of daily mean climatologies.  Take 31 daily fill
		images, +-15 days from current day and average them into
		a simgle image.  If any holes remain fill them with the
		mean of the eight nearest neighbors.

Version	Date	Programmer	Modification
-------	-----	------------	--------------------------------
  1.0	06/99	D.Lloyd		Initial Development

PROJECT:	ADAPS


***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "creatmf.h"



void main( void )
{
char	host[CMLEN];		/* host file name for images		*/
char	infile[32][CMLEN];	/* Input file name			*/
char	outfile[CMLEN];		/* Output file name			*/

float	buf[NLINES][NSAMPS];	/* I/O buffer				*/
float	count[NLINES][NSAMPS] = {0.0};	/* count array			*/
float	mean[NLINES][NSAMPS];	/* mean value array			*/
float	sum[NLINES][NSAMPS] = {0.0};	/* Summation array		*/

int	fdesc;			/* image I/O file descriptor		*/

long	band;			/* image band number			*/
long	dtype = EREAL;		/* image data type			*/
long	filecount;		/* number of files in infile array	*/
long	fill = NOFIL;		/* fill technique specification flag	*/
long	holecount;		/* count of holes in mean image		*/
long	i;			/* loop control variable		*/
long	j;			/* loop control variable		*/
long	k;			/* loop control variable		*/
long	nextflg;		/* call type: CREAT INIT CURNT NEXT	*/
long	nlav;			/* number of valid lines in buffer	*/
long	nlb = 0;		/* number of image boundary lines	*/
long	nlbuf = NLINES;		/* total number of lines in buffer	*/
long	nli = NLINES;		/* number of lines in image window	*/
long	nsav;			/* number of valid samples in buffer	*/
long	nsb = 0;		/* number of image boundary samples	*/
long	nsbuf = NSAMPS;		/* total number of samples in buffer	*/
long	nsi = NSAMPS;		/* number of samples in image window	*/
long	outcnt = 1;		/* count of output files		*/
long	slcb = 1;		/* starting line of data within image	*/
long	sli = 1;		/* starting line of image window	*/
long	sscb = 1;		/* starting sample of data within image	*/
long	ssi = 1;		/* starting sample of image window	*/
long	status;			/* Function return status		*/
long	stdlun;			/* Standard output			*/
long	two = 2;		/* value of two for put_buf() call	*/
long	wtype = EREAL;		/* output image conversion data type	*/

struct	PARBLK	vblock;		/* Parameter block			*/

unsigned char	*buffer;	/* pointer to input buffer		*/


/* Initialize LAS
-----------------*/
c_inlas( &vblock, &stdlun );


/* Get the parameters
---------------------*/
status = getpar( &vblock, infile, outfile, &filecount );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "creatmf-fatal", "Fatal error occured" );


/* point the image I/O pointer to the floating point I/O buf
------------------------------------------------------------*/
buffer = (unsigned char *)buf;


/* for each file in the infile list, add
the data to the sum and count arrays
----------------------------------------*/
for ( k = 0; k < filecount; k++ )
    {
    /* get the ith file host name
    -----------------------------*/
    c_cmhost( infile[k], host );

    /* read band one of the image which contains the H20 water vapor
    mean values.  This first get_buf() call also opens the image.
    ----------------------------------------------------------------*/
    nextflg = INIT;
    band = 1;
    status = get_buf( buffer, &nlbuf, &nsbuf, &nlav, &nsav, &sli, &ssi,
    		      &nli, &nsi, &nlb, &nsb, &slcb, &sscb, &nextflg,
		      &band, &fill, (struct FDESC **)&fdesc, host, &dtype);
    if ( status != E_SUCC )
	c_lasmsg(TRUE,"creatmf-gtbuf","Error reading from image %s", host);

    /* Save the mean values in the mean array
    don't save the negative values.
    -----------------------------------------*/
    for ( i = 0; i < NLINES; i++ )
        for ( j = 0; j < NSAMPS; j++ )
	    if ( buf[i][j] >= 0.0 )
		mean[i][j] = buf[i][j];
	    else
	        mean[i][j] = 0.0;

    /* read band two of the image which contains counts
    ---------------------------------------------------*/
    nextflg = CURNT;
    band = 2;
    status = get_buf( buffer, &nlbuf, &nsbuf, &nlav, &nsav, &sli, &ssi,
    		      &nli, &nsi, &nlb, &nsb, &slcb, &sscb, &nextflg,
		      &band, &fill, (struct FDESC **)&fdesc, host, &dtype);
    if ( status != E_SUCC )
	c_lasmsg(TRUE,"creatmf-gtbuf","Error reading from image %s", host);

    /* Add the counts to the count array, expand the
    mean to sums and add the sums to the sum array.
    ------------------------------------------------*/
    for ( i = 0; i < NLINES; i++ )
        for ( j = 0; j < NSAMPS; j++ )
	    {
	    count[i][j] += buf[i][j];
	    sum[i][j] += mean[i][j] * buf[i][j];
	    }

    /* close this image file
    ------------------------*/
    status = c_eclose( (struct FDESC **)&fdesc );
    if ( status != E_SUCC )
        c_lasmsg(TRUE,"creatmf-clos","Error closing file %s", host);
    }


/* create the means by dividing sums by counts
----------------------------------------------*/
holecount = 0;
for ( i = 0; i < NLINES; i++ )
    for ( j = 0; j < NSAMPS; j++ )
        {
	/* don't divide by zero, a zero count indicates a hole */
        if ( count[i][j] > 0 )
	    mean[i][j] = sum[i][j] / count[i][j];
	/* mark holes with -1.0 and count them */
	else
	    {
	    holecount++;
	    mean[i][j] = -1.0;
	    }
	}


/* Okay, let's fill any pesky holes that remain
-----------------------------------------------*/
if ( holecount > 0 )
    {
    c_lasmsg(FALSE,"creatmf-fill","Filling %d holes",holecount);
    fillem( mean );
    }


/* write band one -- the mean H2O water vapor values.
The first call to put_buf() creates the image and ddr.
------------------------------------------------------*/
nextflg = CREAT;		/* we need to create this file		*/
band = 1;
wtype = SAME;
buffer = (unsigned char *)mean;	/* point to the buffer we want write	*/
status = put_buf( buffer, &nlbuf, &nsbuf, &nlbuf, &nsbuf, &sli, &ssi,
		  &nlbuf, &nsbuf, &nextflg, &band, &two,
		  (struct FDESC **)&fdesc, outfile, &dtype, &wtype );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"creatmf-ptbuf","Error writing to output file");


/* write band two -- the counts of values in the means
------------------------------------------------------*/
nextflg = CURNT;			/* write to the same file	*/
band = 2;
wtype = SAME;
buffer = (unsigned char *)count;	/* point to the correct buffer	*/
status = put_buf( buffer, &nlbuf, &nsbuf, &nlbuf, &nsbuf, &sli, &ssi,
		  &nlbuf, &nsbuf, &nextflg, &band, &two, (struct FDESC **)&fdesc, outfile,
		  &dtype, &wtype );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"creatmf-ptbuf","Error writing to output file");


/* close the output image file
------------------------------*/
status = c_eclose( (struct FDESC **)&fdesc );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"creatmf-clos","Error closing output file");


/* We need to modify the ddr for the fill file
----------------------------------------------*/
status = upddr( outfile );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"creatmf-upddr","Error updating the output ddr");


/* Display a standard completion message
----------------------------------------*/
c_complt( "CREATMF", (char (*)[256])outfile, &outcnt );
} 
