/*****************************************************************************
FUNCTION:	FILLEM

PURPOSE:	Go through the array of mean values looking for holes which
		are indicated by a value of -1.0.  When a hole is encountered,
		take a mean of its eight neighbors (excluding holes) and fill
		the pixel with that value.  Left and right edge pixels are
		adjacent so treat them as neighbors.  Top and bottom edge
		pixels are not adjacent so mirror the next/previous line. 
		If holes remain, do the loop again.


Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    06/99	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "las.h"
#include "creatmf.h"


void  fillem
(
float   means[NLINES][NSAMPS]	/* means array that needs to be filled	*/
)

{
float   x[8];			/* eight neighbor values		*/

long    count;			/* count of neighbor values		*/
long    holes;			/* count of holes that still exist	*/
long    i0, i1, i2;		/* line indices				*/
long    i;			/* index into x[]			*/
long    j0, j1, j2;		/* sample indices			*/
long    line;			/* line loop control variable		*/
long    pass = 0;		/* count of passes through loop		*/
long    samp;			/* sample loop control variable		*/


/* while holes remain in the image fill them.
set holes to 1 initially to get into the loop
limit the loop to five passes.
---------------------------------------------*/
holes = 1;
while ( holes && ( pass < 5 ))
    {
    pass++;     /* increment the pass counter   */
    holes = 0;  /* reset the number of holes    */

    /* for each line and sample in the means array
    ----------------------------------------------*/
    for ( line = 0; line < NLINES; line++ )
        for ( samp = 0; samp < NSAMPS; samp++ )
            /* if the current value is a hole try to
            fill it with the eight-neighbor mean
            ----------------------------------------*/
            if ( means[line][samp] == -1.0 )
                {
                /* set up line indices -- if we are at top or bottom of
                image mirror the next/previous line.
                -------------------------------------------------------*/
                i0 = ( line == 0 )        ? line + 1 : line - 1;
                i1 = line;
                i2 = ( line == NLINES-1 ) ? line - 1 : line + 1;

                /* set up the sample indices -- the left and right sides
                of the image wrap around, sample 1 and sample NSAMPS are
                adjacent so use them for the 8-neighbor mean.
                --------------------------------------------------------*/
                j0 = ( samp == 0 ) ? NSAMPS-1 : samp - 1;
                j1 = samp;
                j2 = ( samp == NSAMPS-1 ) ? 0 : samp + 1;

                /* get the neighbors, ignoring -1.0 values
                ------------------------------------------*/
                count = 0;
                if ( means[i0][j0] != -1.0 ) x[count++] = means[i0][j0];
                if ( means[i0][j1] != -1.0 ) x[count++] = means[i0][j1];
                if ( means[i0][j2] != -1.0 ) x[count++] = means[i0][j2];
                if ( means[i1][j0] != -1.0 ) x[count++] = means[i1][j0];
                if ( means[i1][j2] != -1.0 ) x[count++] = means[i1][j2];
                if ( means[i2][j0] != -1.0 ) x[count++] = means[i2][j0];
                if ( means[i2][j1] != -1.0 ) x[count++] = means[i2][j1];
                if ( means[i2][j2] != -1.0 ) x[count++] = means[i2][j2];

                /* if there were non-hole neighbors
                -----------------------------------*/
                if ( count > 0 )
                    {
                    /* sum the neighbors
                    --------------------*/
                    for ( i = 1; i < count; i++ )
                        x[0] += x[i];

                    /* calculate the mean and store
                    it in the current pixel
                    -------------------------------*/
                    means[line][samp] = x[0] / (float)count;
                    }

                /* otherwise count it as a hole to be filled later
                --------------------------------------------------*/
                else
                    holes++;
                }

    /* diagnostic message 
    ---------------------*/
    if ( holes > 0 )
        c_lasmsg(FALSE,"","Pass %d Still %d holes", pass, holes);
    }
}
