/*******************************************************************************
FUNCTION:	DBSETUP

PURPOSE:	Determines the AVHRR archive header name, output archive
		location, band to process, solar zenith angle at the center
		of the pass, receiving station, scene area, and generate
		a scene ID.  All these values are returned to the TAE
		parameter block and used in the REFAID procedure.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  5.0    03/92  D.Hollaren       Original development
  5.1    11/92  J.Fenno          Added the FAXOPT parameter to allow the user
				 to specify if a facsimile should or should not
				 be generated.
				 Updated the User's Guide.
  5.2    07/93  J.Fenno          Modified so the scene ID is extracted from the
				 filename if it is valid.
  5.3    01/95  T.Baltzer        Modified so that NOAA-14 scenes are sent to ESA
				 as well as NOAA-11 scenes.
  5.4    08/00  T.Baltzer        Delete 'fax', 'microfiche', 'select' coding.

PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "labserv.h"
#include "adaps.h"
#include "adapssup.h"
#include "util.h"
#include "archio.h"

int main()
{
struct PARBLK	vblock;		/* TAE parameter block			     */
struct MODEL 	model;		/* Archive header model info		     */	
struct DROPPED  drop;		/* Dropped line data structure 		     */
struct tm 	*lctm;	   	/* Local time structure			     */

short   valid;			/* Loop control flag                         */

int	band,			/* Band to process			     */
	i;			/* Looping variable			     */

long	std_out,		/* LUN to standard output		     */
	status,			/* Return status			     */
	time_val,		/* Time variable			     */
	len,			/* Length of a string			     */
	count     = 0,		/* Number of areas image covers		     */
	zero      = 0,		/* Constant of zero			     */
	nonfatal  = 1,		/* Nonfatal error flag			     */
	fatal     = -1,		/* Fatal error flag			     */
	access    = IREAD;	/* Access flag				     */

TAEFLOAT sunzen[1];		/* Sun zenith				     */

char	
	arch_hdr[1][CMLEN],	/* Input header file name		     */
	area[5][8],		/* Area of the image for WDBII files         */
	dir[CMLEN],		/* directory part of hosin		     */
	ext[CMLEN],		/* file name extension part of hosin	     */
	filename[CMLEN],	/* file name part of hosin		     */
	hosin[CMLEN],		/* Input host file name			     */
	sceneid[CMLEN],		/* Scene ID				     */
	*strrchr(),		/* Function				     */
	*svec[5],		/* Array of text pointers to send params     */
	tmphos[CMLEN],		/* Temporary host file name		     */
	tstamp[CMLEN];		/* Current time				     */

struct VARIABLE *p_find();      /* Function for locating a TAE parameter     */


/*
  Crude Latitude/longitude minimum and maximum areas
  (North America, South America, Asia, Europe/Africa)
*/
static double regions[4][4] = {84.0 * RAD_PER_DEG,  -12.0 * RAD_PER_DEG,
			       24.0 * RAD_PER_DEG, -169.0 * RAD_PER_DEG,
			       33.0 * RAD_PER_DEG,  -28.0 * RAD_PER_DEG,
			      -85.0 * RAD_PER_DEG, -122.0 * RAD_PER_DEG,
			       82.0 * RAD_PER_DEG,  180.0 * RAD_PER_DEG,
			      -54.0 * RAD_PER_DEG, -180.0 * RAD_PER_DEG,
			       72.0 * RAD_PER_DEG,   77.0 * RAD_PER_DEG,
			      -54.0 * RAD_PER_DEG,  -32.0 * RAD_PER_DEG};

/*
  Initialization
*/
c_inlas(&vblock,&std_out);


/*
  Retrieve input avhrr archive file name
*/
access = IREAD;
status = c_getnam(&vblock, "IN", &zero, hosin, "arch", &access);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);


/*
  Get the AVHRR header information
*/
status = c_getahdr(hosin,&model,&drop);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered", "dbsetup-fatal", &fatal);


/*
  Create the avhrr header file name
*/
c_lsmknm(hosin, "ahdr", tmphos);
strcpy(arch_hdr[0], tmphos);


/*
  Determine which band to process
*/
if (model.day_night == NIGHT)
    band = 4;
else
    band = 2;


/*
  Check for North America
*/
if (((model.nw[0] < regions[0][0]) && (model.nw[0] > regions[0][2]) &&
     (model.nw[1] < regions[0][1]) && (model.nw[0] > regions[0][3]))||
    ((model.sw[0] < regions[0][0]) && (model.sw[0] > regions[0][2]) &&
     (model.sw[1] < regions[0][1]) && (model.sw[0] > regions[0][3]))||
    ((model.se[0] < regions[0][0]) && (model.se[0] > regions[0][2]) &&
     (model.se[1] < regions[0][1]) && (model.se[0] > regions[0][3]))||
    ((model.ne[0] < regions[0][0]) && (model.ne[0] > regions[0][2]) &&
     (model.ne[1] < regions[0][1]) && (model.ne[0] > regions[0][3])))
    {
    strcpy(area[count],"NA");
    count++;
    }


/*
  Check for South America
*/
if (((model.nw[0] < regions[1][0]) && (model.nw[0] > regions[1][2]) &&
     (model.nw[1] < regions[1][1]) && (model.nw[0] > regions[1][3]))||
    ((model.sw[0] < regions[1][0]) && (model.sw[0] > regions[1][2]) &&
     (model.sw[1] < regions[1][1]) && (model.sw[0] > regions[1][3]))||
    ((model.se[0] < regions[1][0]) && (model.se[0] > regions[1][2]) &&
     (model.se[1] < regions[1][1]) && (model.se[0] > regions[1][3]))||
    ((model.ne[0] < regions[1][0]) && (model.ne[0] > regions[1][2]) &&
     (model.ne[1] < regions[1][1]) && (model.ne[0] > regions[1][3])))
    {
    strcpy(area[count],"SA");
    count++;
    }


/*
  Check for Asia
*/
if (((model.nw[0] < regions[2][0]) && (model.nw[0] > regions[2][2]) &&
     (model.nw[1] < regions[2][1]) && (model.nw[0] > regions[2][3]))||
    ((model.sw[0] < regions[2][0]) && (model.sw[0] > regions[2][2]) &&
     (model.sw[1] < regions[2][1]) && (model.sw[0] > regions[2][3]))||
    ((model.se[0] < regions[2][0]) && (model.se[0] > regions[2][2]) &&
     (model.se[1] < regions[2][1]) && (model.se[0] > regions[2][3]))||
    ((model.ne[0] < regions[2][0]) && (model.ne[0] > regions[2][2]) &&
     (model.ne[1] < regions[2][1]) && (model.ne[0] > regions[2][3])))
    {
    strcpy(area[count],"AS");
    count++;
    }


/*
  Check for Europe and Africa
*/
if (((model.nw[0] < regions[3][0]) && (model.nw[0] > regions[3][2]) &&
     (model.nw[1] < regions[3][1]) && (model.nw[0] > regions[3][3]))||
    ((model.sw[0] < regions[3][0]) && (model.sw[0] > regions[3][2]) &&
     (model.sw[1] < regions[3][1]) && (model.sw[0] > regions[3][3]))||
    ((model.se[0] < regions[3][0]) && (model.se[0] > regions[3][2]) &&
     (model.se[1] < regions[3][1]) && (model.se[0] > regions[3][3]))||
    ((model.ne[0] < regions[3][0]) && (model.ne[0] > regions[3][2]) &&
     (model.ne[1] < regions[3][1]) && (model.ne[0] > regions[3][3])))
    {
    strcpy(area[count],"EU");
    count++;
    }


/*
  If it is undetermined where the image is then cover the world!
*/
if (count == 0)
    {
    strcpy(area[0],"NA");
    strcpy(area[1],"SA");
    strcpy(area[2],"AS");
    strcpy(area[3],"EU");
    count = 4;
    }


/*
  Split up the directory, file name, extension
*/
c_splitname(hosin, dir, filename, ext);


/*
  If the filename contains a valid scene ID then extract the 16 character
  scene ID, otherwise create the scene ID from the AVHRR model structure.
  Valid scene IDs must be 16 or 23 characters long, the first character must
  be an 'a', the second must be a 'g', 'h', 'l', or 'o', and characters 3
  through 16 must be digits.
*/
i = strlen(filename);
valid = (((i == 16) || (i == 23)) &&
    (filename[0] == 'a') &&
    ((filename[1] == 'g') || (filename[1] == 'h') ||
     (filename[1] == 'l') || (filename[1] == 'o')));
for (i = 2; (i < 16) && valid; i++)
    {
    if ((filename[i] < '0') || (filename[i] > '9'))
	valid = FALSE;
    }
if (valid)
    {
    strcpy(sceneid, filename);
    sceneid[16] = '\0';
    }
else
    {
    c_getscene(&model, sceneid);
    }


/*
  Get the processing time stamp or create a new one
*/
if (strlen(filename) == 23)
    {
    strcpy(tstamp,&filename[17]);
    }
else
    {
    time (&time_val);
    lctm = localtime (&time_val);
    sprintf(tstamp,"%02d%02d%02d",lctm->tm_hour,lctm->tm_min,lctm->tm_sec);
    }


/*
  Write the name of the input associated header file to the parameter block
*/
svec[0] = arch_hdr[0];
status = q_string(&vblock,"INHDR",1,(const char **)svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending INHDR to TAE", "dbsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Write the scene-id to the parameter block
*/
svec[0] = sceneid;
status = q_string(&vblock,"SCENEID",1,(const char **)svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending SCENEID to TAE", "dbsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Lower case the the ceos-id and write it to the parameter block
*/
len = (long)strlen(model.station);
c_up2low(model.station,&len);
svec[0] = model.station;
status = q_string(&vblock,"CEOSID",1,(const char **)svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending CEOSID to TAE", "dbsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Write the band to process to the parameter block
*/
status = q_intg(&vblock,"BAND",1,&band,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending BAND to TAE", "dbsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Write the area of the image for WDBII
*/
for (i = 0; i < count; i++)
    svec[i] = area[i];
status = q_string(&vblock,"AREA",count,(const char **)svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending AREA to TAE", "dbsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Write the current time to the parameter block
*/
svec[0] = tstamp;
status = q_string(&vblock,"TSTAMP",1,(const char **)svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending TSTAMP to TAE", "dbsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Write the sun zenith to the parameter block
*/
sunzen[0] = (TAEFLOAT)(model.sunzen * DEG_PER_RAD);
status = q_real(&vblock,"SUNZEN",1,sunzen,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending SUNZEN to TAE", "dbsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Write the parameter block to TAE
*/
status = q_out(&vblock);
if (status != SUCCESS)
    {
    c_errmsg("Error sending variable block to TAE", "dbsetup-qout", &nonfatal);
    c_errmsg("Fatal error encountered","dbsetup-fatal",&fatal);
    }


/*
  Write a completion message 
*/
c_complt ("DBSETUP",arch_hdr,&zero);


/*
  Put a return here to make the compiler happy!
*/
return(1);
}
