#!/bin/csh
#
# FUNCTION: dbtrans.csh
#
# PURPOSE:  Cat all of the appropriate data base records into big files
#	    and transfer them to the data base manager host system.
#	    Update the GLIS data base on the host system using these
#	    concatinated files.  This procedure will run as a cron job
#	    and calls the "transferdb.csh" shell script to do the bulk
#	    of the work.
#
#
#
# Version  Programmer       Date   Modifications
# -------  ---------------  -----  ---------------------------------------------
#   1.0    T. Baltzer       07/94  Initial development
#


#------------------------
# Assign constant values
#------------------------
set DBSYS = "edclxs53"
set DBUSR = "ops"


#-------------------------------------------
# Make certain the environment has been set
#-------------------------------------------
if (! $?EDCSOFT) then
    setenv EDCSOFT /avhrr/software/las/build
    source $EDCSOFT/env/las/assign/lasenv.csh $EDCSOFT
endif


#----------------------------
# Alias appropriate commands
#----------------------------
alias wrtlog $ADAPSBASE/write_log


#--------------------------------------------------------
# Create a log file.  This must be a LAS name for wrtlog.
#--------------------------------------------------------
set noglob
set logfile = `echo "[$ADAPSLOG]dbtrans" | tr / . | sed  's/\[\./\[/'`


#-------------------------------------
# Check the arguments (should be none)
#-------------------------------------
if ($#argv != 0) then
    wrtlog $logfile "" "dbtrans-parm" "Invalid usage: dbtrans.csh"
    exit 1
endif


#------------------------------
# Unalias appropriate commands
#------------------------------
unalias stat
unalias rm
unalias cat


#--------------------------------------------------------------
# Verify that the data base manager host system is operational
# This ssh used to be a ping.  But there have been instances when the host was
# responding to the ping but not actually operational.  Hopefully if the
# problem happens again the date command will fail.  We'll have to wait and
# see.
#--------------------------------------------------------------
ssh ops@edclxs53 date >& /dev/null
if ($status) then
    wrtlog $logfile $DBSYS "dbtrans-error" "The dbm host is not responding"
    exit 1
else
    wrtlog $logfile $DBSYS "informational" "The dbm host is operational"
endif


#------------------------------------------------------
# See if a lock file exists indicating that another
# process like this one is already running.  If there
# is, terminate; otherwise create a lock file.
#------------------------------------------------------
if (-e $ADAPSTABLES/dbtrans.lock) then
    # Wait before quitting check to see how old the lock file is.  If it's
    # more than an hour old somthing is amiss.  If it's four hours or older
    # somethings very wrong.  Use the time of the logfile for the comparison
    # since it was just updated with the informational message about the host
    # being operationsl.
    #------------------------------------------------------------------------
    set logtime = `stat -qm $ADAPSLOG/dbtrans.log`
    set locktime = `stat -qm $ADAPSTABLES/dbtrans.lock`
    @ diff = ( $logtime - $locktime ) / 3600
    # If the file is four hours or older send the appropriate messages about
    # checking for hung processes and that we are continuing.  We'll let the
    # process clean up the lock file later.
    #-----------------------------------------------------------------------
    if ( "$diff" > 4 ) then
        wrtlog $logfile "ERROR" "dbrans-lock" \
	                "Olde lock file check for hung process"
	wrtlog $logfile $DBSYS "Informational" "Continuing"
    # Otherwise if it's more than an hour old just send an ERROR message and
    # quit.  This should alert ops to check into it.  It might not though that
    # is why the four hour check will run anyway.
    #-------------------------------------------------------------------------
    else if ( "$diff" > 1 ) then
	wrtlog $logfile "ERROR" "dbtrans-mult" \
	                "A lock file older than 1 hour is present"
	exit 1
    # Less than an hour old just send the standard message.
    #------------------------------------------------------
    else
	wrtlog $logfile "" "dbtrans-mult" "DBTRANS procedure already running"
	exit 1
    endif
else
    echo "locked" >> $ADAPSTABLES/dbtrans.lock
endif


# Echo the date to standard output just for reference
#-----------------------------------------------------
echo " "
echo " "
date
echo "-----------------------------------------------------------------------"

#--------------------------------------------------
# Transfer the data base records to the dbm host:
# (1) Records to update indicating AVHRR scene has
#     now become a part of a stitched half orbit.
# (2) New AVHRR archive scenes (including orbital
#     stitched images).
#
#--------------------------------------------------
$ADAPSBASE/transferdb.csh $ADAPSDB orbit
$ADAPSBASE/transferdb.csh $ADAPSDB new
$ADAPSBASE/transferdb.csh $ADAPSDB archive


#--------------------------------------------------------
# Remove the lock file indicating that this process is no
# longer running.
#--------------------------------------------------------
rm -f $ADAPSTABLES/dbtrans.lock


#-----------------------
# Successful completion
#-----------------------
exit 0
