#!/bin/csh
#
# FUNCTION: transferdb.csh
#
# PURPOSE:  Cat all of the appropriate data base records into one big file
#	    and transfer that big file to the data base manager host system.
#	    This file will be used later to update the GLIS data base.  See
#	    in-line documentation for more details.
#
#
# Version  Programmer       Date   Modifications
# -------  ---------------  -----  ---------------------------------------------
#   1.0    T. Baltzer       07/94  Initial development
#


#------------------------
# Assign constant values
#------------------------
set DBSYS = "edclxs53"
set DBUSR = "ops"
set DBDIR = "/inv/inv/avhrr/data/metadata"


#----------------------------
# Alias appropriate commands
#----------------------------
alias lsfiles $ADAPSBASE/lsfiles
alias wrtlog $ADAPSBASE/write_log


#--------------------------------------------------------
# Create a log file.  This must be a LAS name for wrtlog.
#--------------------------------------------------------
set noglob
set logfile = `echo "[$ADAPSLOG]dbtrans" | tr / . | sed  's/\[\./\[/'`


#--------------------
# Check the arguments
#--------------------
if ($#argv != 2) then
    wrtlog $logfile "" "dbtrans-parm" "Usage: transferdb.csh <dir> <filetype>"
    exit 1
endif
set dir = $1
set filetype = $2


#------------------------------
# Unalias appropriate commands
#------------------------------
unalias cat
unalias cd
unalias mv


#---------------------------------------------------------------------
# cd to the specified directory and ensure the old subdirectory exists.
#---------------------------------------------------------------------
cd $dir
if (! -d old) then
   wrtlog $logfile "" "dbtrans-old" "The old subdirectory does not exist"
   exit 1
endif


#--------------------------------------------------------
# Create a file name to hold all of the data base records
#--------------------------------------------------------
set sysdate = `date '+%d%H%M%S'`
if ("$filetype" == "orbit") then
    set outfile = ORBIT$sysdate
    set rptfile = $outfile.pr
    set outfile = $outfile.orbit
else
    set outfile = NEW$sysdate
    set rptfile = $outfile.pr
    set outfile = $outfile.new
endif


#------------------------------------------------
# Generate a list of all of the files to process
#------------------------------------------------
switch ($filetype)
    case orbit:
	set files = `lsfiles '^[01].*\.orb$'`
	breaksw
    case new:
	set files = `lsfiles '^a[ghlo].*\.dat$'`
	breaksw
    case archive:
	set files = `lsfiles '^a[ghlo].*\.db$'`
	breaksw
endsw

#----------------------------------------
# Make sure there are files to process
#----------------------------------------
if ($#files < 1) then 
    exit 1
else
    switch ($filetype)
	case orbit:
	    wrtlog $logfile "" "dbtrans-nof" "Processing orbital database files"
	    breaksw
	case new:
	    wrtlog $logfile "" "dbtrans-nof" "Processing new database files"
	    breaksw
	case archive:
	    wrtlog $logfile "" "dbtrans-nof" "Processing db database files"
	    breaksw
    endsw
endif


#-------------------------------------------------------------
# Concatinate all of the data base records into a single file.
# Move them into the 'old' directory as a back-up.
# Ignore empty data base record files.
#-------------------------------------------------------------
foreach file ($files)
   if (-z $file) continue

   cat $file >>! $outfile
   mv $file old/.
end

#-------------------------------------------------------------
# If the output file does not exist or it exists but it is 
# empty then don't transfer anything.  Otherwise, transfer
# the concatenated data base records.
#-------------------------------------------------------------
if ((! -e $outfile) || (-z $outfile)) then
    rm -f $outfile
    exit 0
else
    switch ($filetype)
	case orbit:
	    set ofile = "$DBDIR/avhrr.orbit.$sysdate"
	    breaksw
	case new:
	    set ofile = "$DBDIR/avhrr.new.$sysdate"
	    breaksw
	case archive:
	    set ofile = "$DBDIR/avhrr.new.$sysdate"
	    breaksw
    endsw
    set outdest = ":"
    set outdest = "$DBUSR@$DBSYS$outdest$ofile"
    # may have to change file modes too
    if (! { scp -p $outfile $outdest } ) then
	wrtlog $logfile $outfile "dbtrans-trans" \
	    "Error copying to $DBUSR@$DBSYS$outdest"
	goto ERROUT
    endif
    if (! { ssh $DBUSR@$DBSYS chmod 666 $ofile } ) then
        wrtlog $logfile $outfile "dbtrans-chmod" \
	    "Error changing mode of $ofile"
	goto ERROUT
    endif
endif


#---------------------------------------------------
# Move the data base file into the /old subdirectory
#---------------------------------------------------
mv $outfile old/.


#----------------------------------------------
# Remove the individual record files from the
# "old" backup directory.
#----------------------------------------------
cd old
foreach file ($files)
    if (! -e $file) continue
    rm $file
end


#---------------------------
# Successful return status.
#---------------------------
exit 0


#-------------------
# Error handling
#-------------------
ERROUT:

rm -f $outfile
cd old
foreach file ($files)
    if (! -e $file) continue
    mv $file ../.
end
if ($status) then
    wrtlog $logfile "" "dbtrans-move" "You MUST move files from /old directory"
endif
exit 1
