/*******************************************************************************
NAME:		check_proj_pole

PURPOSE:	Determine if the projected image crosses a pole.

		Use the Polar Stereographic (sphere) projection centered at 
		each pole to transform the lat/long corner 
		coordinates into projection coordinates.

		given:
 		  R, radius of earth:			1.0
 		  clat,clng, center of the projection:	90.0 N lat, 0.0 long
							90.0 S lat, 0.0 long
 		  k0, central scale factor:		1.0
 		  lat, lng of pole:			90.0 N lat, 0.0 long
 		  plat, plng, points of interest for each of the 4 corners

		Find x,y for each of the points of interest in each projection:
		(North:)
		  x = R * k * cos(plat) * sin(plng)
		  y = R * k * (-cos(plat) * cos(plng))
  		where
		  k=2.0*k0/(1.0 + sin(plat))

		(South:)
		  x = R * k * cos(plat) * sin(plng)
		  y = R * k * (cos(plat) * cos(plng))
  		where
		  k=2.0*k0/(1.0 - sin(plat))

		So the points surrounding a pole will look something like:
			|
  		ul------+------ur
  		|       |       |
  	       -+------Pole-----+-   Pole = (0,0)
  		|       |       |
  		ll------+------lr
			|
		So to check if we cross the pole, the coords along at least
		one side will be different sign.
		However, the projection used in the image may be rotated
		(e.g., central meridian != 0).

ALGORITHM REFERENCES:
	For further information, see Map Projections--A Working Manual by 
	J.P.Snyder, pp. 154-160 and pp. 312-313, Polar Stereographic projection.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "adaps.h"
#include "dcw2ras.h"
#define RADIUS 1.0

int check_proj_pole
(
struct DDR *ddr,        /* image DDR information                        */
long 	*ddr_units,	/* input units					*/
long    *geo_code,      /* geographic projection code           	*/
long    *geo_zone,      /* geographic projection zone           	*/
long    *geo_units,     /* geographic projection units          	*/
double  geo_parms[]     /* geographic projection parameters             */
)
{
double	ullon,		/* upper left longitude				*/
	ullat,		/* upper left latitude				*/
	urlon,		/* upper right longitude			*/
	urlat,		/* upper right latitude				*/
	lllon,		/* lower left longitude				*/
	lllat,		/* lower left latitude				*/
	lrlon,		/* lower right longitude			*/
	lrlat,		/* lower right latitude				*/
	xul,xll,	/* upper & lower left x				*/
	xur,xlr,	/* upper & lower right x			*/
	yul,yll,	/* upper & lower left y				*/
	yur,ylr,	/* upper & lower right y			*/
	k;		/* conversion constant				*/
long	inx,iny,	/* pole is within in x and y directions		*/
	prtprm[2];	/* print parameters				*/
char	fname = '\0';   /* file name                                 */

/*
  First convert the cornerst to geographic degrees
*/
prtprm[GEO_TERM] = prtprm[GEO_FILE] = FALSE;

if (c_transinit((long *)&ddr->proj_code,ddr_units,(long *)&ddr->zone_code,(long *)&(ddr->datum_code),
       ddr->proj_coef,geo_code,geo_units,geo_zone,(long *)&(ddr->datum_code),
       geo_parms,prtprm,&fname) != E_SUCC)
       return(NONE);

if (c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->upleft[1],&ddr->upleft[0],&ullon,&ullat) != E_SUCC)
       return(NONE);
ullon *= RAD_PER_DEG;
ullat *= RAD_PER_DEG;

if (c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->loleft[1],&ddr->loleft[0],&lllon,&lllat) != E_SUCC)
       return(NONE);
lllon *= RAD_PER_DEG;
lllat *= RAD_PER_DEG;

if (c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->upright[1],&ddr->upright[0],&urlon,&urlat) != E_SUCC)
       return(NONE);
urlon *= RAD_PER_DEG;
urlat *= RAD_PER_DEG;

if (c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->loright[1],&ddr->loright[0],&lrlon,&lrlat) != E_SUCC)
       return(NONE);
lrlon *= RAD_PER_DEG;
lrlat *= RAD_PER_DEG;


/* Check the North Pole
   --------------------*/
/* find the corners */
if (ullat >= 0.0)
    {
    k = 2.0*(1.0/(1.0+sin(ullat)));
    xul = RADIUS * k * cos(ullat) * sin(ullon);
    yul = RADIUS * k * (-cos(ullat)*cos(ullon));
    }
else
   xul = yul = 0.0;

if (lllat >= 0.0)
    {
    k = 2.0*(1.0/(1.0+sin(lllat)));
    xll = RADIUS * k * cos(lllat) * sin(lllon);
    yll = RADIUS * k * (-cos(lllat)*cos(lllon));
    }
else
   xll = yll = 0.0;

if (urlat >= 0.0)
    {
    k = 2.0*(1.0/(1.0+sin(urlat)));
    xur = RADIUS * k * cos(urlat) * sin(urlon);
    yur = RADIUS * k * (-cos(urlat)*cos(urlon));
    }
else
   xur = yur = 0.0;

if (lrlat >= 0.0)
    {
    k = 2.0*(1.0/(1.0+sin(lrlat)));
    xlr = RADIUS * k * cos(lrlat) * sin(lrlon);
    ylr = RADIUS * k * (-cos(lrlat)*cos(lrlon));
    }
else
   xlr = ylr = 0.0;

inx = iny = FALSE;
if (((xul * xur) < 0.0) || ((xll * xlr) < 0.0) ||
    ((xul * xll) < 0.0) || ((xur * xlr) < 0.0))
    inx = TRUE;

if (((yul * yll) < 0.0) || ((yur * ylr) < 0.0) ||
    ((yul * yur) < 0.0) || ((yll * ylr) < 0.0))
    iny = TRUE;

if (inx && iny)
    return(NORTH);


/* Check the South Pole
   --------------------*/
/* find the corners */
if (ullat <= 0.0)
    {
    k = 2.0*(1.0/(1.0-sin(ullat)));
    xul = RADIUS * k * cos(ullat) * sin(ullon);
    yul = RADIUS * k * (cos(ullat)*cos(ullon));
    }
else
    xul = yul = 0.0;

if (lllat <= 0.0)
    {
    k = 2.0*(1.0/(1.0-sin(lllat)));
    xll = RADIUS * k * cos(lllat) * sin(lllon);
    yll = RADIUS * k * (cos(lllat)*cos(lllon));
    }
else
    xll = yll = 0.0;

if (urlat <= 0.0)
    {
    k = 2.0*(1.0/(1.0-sin(urlat)));
    xur = RADIUS * k * cos(urlat) * sin(urlon);
    yur = RADIUS * k * (cos(urlat)*cos(urlon));
    }
else
    xur = yur = 0.0;

if (lrlat <= 0.0)
    {
    k = 2.0*(1.0/(1.0-sin(lrlat)));
    xlr = RADIUS * k * cos(lrlat) * sin(lrlon);
    ylr = RADIUS * k * (cos(lrlat)*cos(lrlon));
    }
else
    xlr = ylr = 0.0;

inx = iny = FALSE;
if (((xul * xur) < 0.0) || ((xll * xlr) < 0.0) ||
    ((xul * xll) < 0.0) || ((xur * xlr) < 0.0))
    inx = TRUE;

if (((yul * yll) < 0.0) || ((yur * ylr) < 0.0) ||
    ((yul * yur) < 0.0) || ((yll * ylr) < 0.0))
    iny = TRUE;

if (inx && iny)
    return(SOUTH);

return(NONE);
}
