/******************************************************************************
FUNCTION:	find_path

PURPOSE:	Find which entry in the datapath environment variable will be 
		sufficient for the file.  The directory whose disk space 
		available that would be best utilized for the creation of the 
		file is used.

ALGORITHM REFERENCES
  The DATAPATH contains directories seperated by a colon ":"
  ie) "/software/test:/production/job:/production/batch/proj1"
*******************************************************************************/
#include	<stdio.h>
#include 	<string.h>
#include	"las.h"
#include	"diskio.h"
#include "dcw2ras.h"

#ifdef UNIX_DISK
#include	<ctype.h>
#include	<errno.h>
#include	<pwd.h>
#include	<sys/file.h>
#include	<sys/types.h>
#include	<sys/stat.h>

#include	"typlim.h"
#ifdef sgi
#include	<sys/statfs.h>
#else
#ifdef DGUX
#include	<sys/statfs.h>
#else
#ifdef sun
#include	<sys/statvfs.h>
#else
#include	<sys/vfs.h>
#endif
#endif
#endif
#ifdef sun
#include	<string.h>
#else
#include	<strings.h>
#endif

char *strrchr();
char *strchr();
char *strtok();

#define	DATAPATH	"DATAPATH"
#define	PATHNAMESZ	1024

/*
 * System & Library Functions
 */
long			atol();			/* ascii to long int */
char			*getwd();		/* get working directory */
char			*getenv();		/* get environment */
char			*mktemp();		/* make a unique file name */

/*
 * External Globals
 */
extern int errno;			/* error number */

/*
 * Private Functions
 */
static char	**createpaths(void);		/* create search paths */
static double	freespace(char *);		/* free space in path */
static char	**getpaths(char **);		/* get search paths */
static char	*search(register char **);	/* search paths for space */

/*
 * Private Globals
 */
static char		*cwd;			/* current working directory */
static char		*datapath;		/* DATAPATH environment */
static struct stat	*statp;			/* file status structure */
#ifdef sun
static struct statvfs	*statfsp;		/* file system statistics */
#else
static struct statfs	*statfsp;		/* file system statistics */
#endif

long find_path
(
char	directory[] 		/* array of directories to search for space */
				/* wait = 0, do not wait for space	*/
)
{
char	**paths;	/* array of pointers to path names	*/
char	*dirs[MAXPATH]; /* data paths				*/
long	i;		/* Looping variable			*/
long	nonfatal = 1;	/* Non-fatal error code			*/
long	status = E_SUCC;/* Set successful status code		*/
char	*tempdir;	/* temporary directory			*/


/*
  Allocate space for the data path name
*/
if ((datapath = malloc(PATHNAMESZ)) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","dkcre-alloc",
		&nonfatal);
    return(E_FAIL);
    }

/*
  Allocate space for the file status structure
*/
if ((statp = (struct stat *)malloc(sizeof(struct stat))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","dkcre-alloc",
		&nonfatal);
    return(E_FAIL);
    }

/*
  Allocate space for the file systems statistics
*/
#ifdef sun
if ((statfsp = (struct statvfs *)malloc(sizeof(struct statvfs))) == NULL)
#else
if ((statfsp = (struct statfs *)malloc(sizeof(struct statfs))) == NULL)
#endif
    {
    c_errmsg("Error allocating dynamic memory","dkcre-alloc",
		&nonfatal);
    return(E_FAIL);
    }

/*
  Set an array of pointers to the specified directory paths
*/
for (i = 0; i < MAXPATH; i++)
    dirs[i] = NULL;

/*
  Get the directories to search
*/
paths = getpaths(dirs);

tempdir = search(paths);
strcpy(directory,tempdir);

/*
  Free up allocated space
*/
free(cwd);
free(datapath);
free(statp);
free(statfsp);

if (!dirs)
    free(paths);

return(status);
}

/******************************************************************************
FUNCTION:	createpaths

PURPOSE:	Get the directory paths from the external global DATAPATH

ALGORITHM REFERENCES
  DATAPATH has directory paths seperated by a colon between
  each directory path.  ie) /xxx/xxx/xxx:/sss/sss/sss:/nnn/nnn
*******************************************************************************/
static char **createpaths(void)
{
char	*pathname;	/* pathname pointer			*/
char	**path;		/* path array pointer			*/
char	**paths;	/* array of pathnames			*/
long	nonfatal = 1;	/* Non-fatal erro code			*/
long	npaths;		/* number of pathnames			*/

/*
  Determine the number of path names specified
  Path names are seperated by colons ":"
*/
for (pathname = datapath, npaths = 2;
     pathname = strchr(pathname,':'); 
     npaths++, pathname++)
    ;

/*
  Allocate space for the path directory names
*/
if ((paths = (char **)calloc(npaths,sizeof(char *))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","dkcre-alloc",
		&nonfatal);
    return(paths);
    }

/*
  Parse the path names
*/
path = paths;
*path = strtok(datapath,":");
for (path++; *path = strtok((char *)0,":"); path++)
	;

return(paths);
}

/******************************************************************************
FUNCTION:	freespace

PURPOSE:	Determine the amount of free space available on a file system

*******************************************************************************/
static double freespace
(
char	*path			/* pathname to determine free space	*/
)
{
double	avail;		/* free space that is available		*/
double	savail;		/* software space reserved		*/
double  dbltmp1;
double  dbltmp2;
long	nonfatal = 1;	/* non-fatal error code			*/

/*
  Abort if this pathname is not a directory
*/
if (stat(path,statp) == -1)
    {
    c_errmsg("Error getting path status information","dkcre-stat",
		&nonfatal);
    return(E_FAIL);
    }

/*
  Error if the pathname is not a directory
*/
if (!(statp->st_mode & S_IFDIR))
    {
    errno = ENOTDIR;
    c_errmsg("Invalid pathname","dkcre-path",&nonfatal);
    return(E_FAIL);
    }

/*
  Get the file status information to determine the space avaliable
*/
#ifdef sgi
  if (statfs(path,statfsp,sizeof(struct statfs),0) == -1)
#else
#ifdef DGUX
  if (statfs(path,statfsp,sizeof(struct statfs),0) == -1)
#else
#ifdef sun
  if (statvfs(path,statfsp) == -1)
#else
  if (statfs(path,statfsp) == -1)
#endif
#endif
#endif
    {
    c_errmsg("Error getting file system status information",
		"dkcre-statfs",&nonfatal);
    return(E_FAIL);
    }

/*
  Determine the amount of available space
  bavail seems more acurate then bfree on DG
*/ 
#if defined(sgi) /* || defined(DGUX) */
        dbltmp1 = (double)(statfsp->f_bfree);
        dbltmp2 = (double)(statfsp->f_bsize);
        avail = dbltmp1 * dbltmp2;
#else
        avail = (double)(statfsp->f_bavail) * (double)(statfsp->f_bsize);
#endif

/*
  Subtract the amount of space that is software reserved
*/
if ((savail = (double)(c_dkrspac(path))) == E_FAIL)
   return(E_FAIL);
avail = avail - savail;

return(avail);
}

/******************************************************************************
FUNCTION:	getpaths

PURPOSE:	Get the path names that are used to determine where the
		file is to be created.

ALGORITHM REFERENCES
  DATAPATH has directory paths seperated by a colon between
  each directory path.  ie) /xxx/xxx/xxx:/sss/sss/sss:/nnn/nnn
*******************************************************************************/
static char **getpaths
(
char	**pathnames	/* array of pathnames	*/
)
{
char	*datapathp;

/*
  Return if the path names were specified
*/
if (*pathnames)
    return(pathnames);

/*
  Get the path names from the DATAPATH or use the current working directory
*/
if ((datapathp = getenv(DATAPATH)))
    strcpy(datapath,datapathp);
else
    strcpy(datapath,cwd); 

return(createpaths());
}

/******************************************************************************
FUNCTION:	search

PURPOSE:	Search all paths to determine with file system will be best
		for the creation of the output file.

*******************************************************************************/
static char *search
(
register char	**paths	/* array of directories to search for space */
)
{
double		avail = 0;		/* best-fit for space */
char		*searchp = (char *)0;	/* pathname where space exists*/
register double	space;			/* space in pathname */
long		i;

/*
  Search each path for space available to determine the first/best path
  location to place the file into
*/
for (i = 0; *paths != NULL; paths++,i++)
    {
    if ((space = freespace(*paths)) > avail)
	{
	avail = space;
	searchp = *paths;
	}
    }

return(searchp);
}

#else
long find_path
(
char    directory[]             /* array of directories to search for space */
                                /* wait = 0, do not wait for space      */
)
{
return(E_SUCC);
}
#endif
