/*******************************************************************************
NAME:		prominmax

PURPOSE:	Determine the minimum and maximum latitude and longitudes
		of the scene in another projection.
		This subroutine makes one major assumption about the input 
		image.  If this does not hold true for a particular image, 
		the routine may calculate incorrect values.  this routine 
		assumes:
	         The input image has only one high or low point along its
	         edge (ie. change is smooth, increasing then decreasing.  not
	         increasing, decreasing, increasing).  For example,  along 
	         the top of the image, the latitude of the image only peaks 
	         once.
	         By the same token, the latitude along the bottom of the
	         image reaches only one low point.

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "dcw2ras.h"

#ifndef MIN3
#define MIN3(a,b,c) (min(a,b) < c) ? min(a,b) : (c)
#endif
#ifndef MAX3
#define MAX3(a,b,c) (max(a,b) > c) ? max(a,b) : (c)
#endif

int prominmax
(
struct DDR *ddr,        /* image DDR information                        */
long 	*ddr_units,	/* input units					*/
long    *geo_code,      /* geographic projection code           	*/
long    *geo_zone,      /* geographic projection zone           	*/
long    *geo_units,     /* geographic projection units          	*/
double  geo_parms[],    /* geographic projection parameters             */
long    window[4],	/* user's input window				*/
double	*minlat,	/* minimum latitude of the pass			*/
double  *minlon,	/* minimum longitude of the pass		*/
double  *maxlat,	/* maximum latitude of the pass			*/
double  *maxlon 	/* maximum longitude of the pass		*/
)
{
long	nonfatal = 1,	/* nonfatal error flag				*/
	rtx,		/* right side sample number			*/
	ltx,		/* left side sample number			*/
	midsamp,	/* midpoint sample for comparing		*/
	midline,	/* midpoint line for comparing			*/
	topline,	/* top line for comparing			*/
	botline,	/* bottom line for comparing			*/
        prtprm[2];      /* print parameters                             */
char    fname = '\0';   /* file name                                 */

double	ullon,		/* upper left longitude				*/
	ullat,		/* upper left latitude				*/
	urlon,		/* upper right longitude			*/
	urlat,		/* upper right latitude				*/
	lllon,		/* lower left longitude				*/
	lllat,		/* lower left latitude				*/
	lrlon,		/* lower right longitude			*/
	lrlat,		/* lower right latitude				*/
	clatr,		/* right lat for comparing			*/
	clatl,		/* left lat for comparing			*/
	clont,		/* top long for comparing			*/
	clonb,		/* bottom long for comparing			*/
     	llon,		/* longitude of pixel to the left of midpoint	*/
     	llat,		/* latitude of pixel to the left of midpoint	*/
     	rlon,		/* longitude of pixel to the right of midpoint	*/
     	rlat,		/* latitude of pixel to the right of midpoint	*/
     	tlon,		/* longitude of pixel just above midpoint	*/
     	tlat,		/* latitude of pixel just above midpoint	*/
     	blon,		/* longitude of pixel just below midpoint	*/
     	blat,		/* latitude of pixel just below midpoint	*/
     	zlont,		/* z longitude top				*/
     	zlonb,		/* z longitude bottom				*/
     	zlon,		/* z longitude 					*/
	lon,		/* temporary longitude value			*/
	lat,		/* temporary latitude value			*/
	coordy,		/* y-coordinate (line number)			*/
	coordx,		/* x-coordinate (sample number)			*/
	tmpcoord,	/* coords on either side of our point		*/
	max3,		/* maximum of 3 values				*/
	min3;		/* minimum of 3 values				*/

/*
  Find the latitude and longitude of the corners of the image window
    coordx = ddr->upleft[1] + (samp - 1) * ddr->pdist_x;
    coordy = ddr->upleft[0] - (line - 1) * ddr->pdist_y;
*/
prtprm[GEO_TERM] = prtprm[GEO_FILE] = FALSE;

if (c_transinit((long *)&ddr->proj_code,ddr_units,(long *)&ddr->zone_code,(long *)&(ddr->datum_code),
       ddr->proj_coef,geo_code,geo_units,geo_zone,(long *)&(ddr->datum_code),
       geo_parms,prtprm,&fname) != E_SUCC)
       return(NONE);

coordx = ddr->upleft[1] + (window[SS] - 1) * ddr->pdist_x;
coordy = ddr->upleft[0] - (window[SL] - 1) * ddr->pdist_y;
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&ullon,&ullat);

coordy = ddr->upleft[0] - (window[SL] + window[NL] - 2) * ddr->pdist_y;
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&lllon,&lllat);

coordx = ddr->upleft[1] + (window[SS] + window[NS] - 2) * ddr->pdist_x;
coordy = ddr->upleft[0] - (window[SL] - 1) * ddr->pdist_y;
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&urlon,&urlat);

coordy = ddr->upleft[0] - (window[SL] + window[NL] - 2) * ddr->pdist_y;
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&lrlon,&lrlat);

/*****************************************************************************
  FIND MAXLAT
  To find maximum latitude, vary the sample (coordx) across the top line 
  of the image (at ddr->upleft[0])
*****************************************************************************/
/*
  Set up the initial values for comparing
midsamp =  ddr->ns / 2;
ltx = 1;
rtx = ddr->ns;
*/
midsamp =  window[SS] + window[NS] / 2;
coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
coordy = ddr->upleft[0] - (window[SL] - 1) * ddr->pdist_y;
clatl = ullat;
clatr = urlat;
ltx = window[SS];
rtx = window[SS] + window[NS] - 1;

/*
  Convert the midpoint to a longitude,latitude coordinate
*/
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&lon,&lat);

if ((lat < clatl) && (lat < clatr))
    {
    /*
      The curve is decreasing, so the corner points are the greatest
    */
    *maxlat = max(ullat,urlat);
    }
else if ((lat == clatl) && (lat == clatr))
    {
    /*
      The latitude is constant across the top of the image
    */
    *maxlat = ullat;
    }
else
    {
    /*
      While haven't reached the maximum latitude value
    */
    while ((abs(midsamp - rtx) > 1) && (abs(midsamp - ltx) > 1))
	{
        if ((lat < clatl) && (lat < clatr))
            {
            /*
              Error out if this routine has made an invalid assumption
            */
            c_errmsg ("Cannot calculate maximum latitude","dcw2ras-calc",
		&nonfatal);
	    return(E_FAIL);
            }
        else if ((lat == clatl) && (lat == clatr))
            {
            /*
              If the midpoint is equal to the 2 end points, max has been found
	      Just break; maxlat is correctly set after the while loop
            */
            break;
            }
        else if ((lat >= clatl) && (lat <= clatr))
            {
            /*
              The maximum is to the right of the current midpoint
            */
            clatl = lat;
            ltx = midsamp;
            midsamp = (midsamp + rtx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
            }
        else if ((lat <= clatl) && (lat >= clatr))
            {
            /*
              The maximum is to the left of the current midpoint
            */
            clatr = lat;
            rtx = midsamp;
            midsamp = (midsamp + ltx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
            }
        else if ((lat > clatl) && (lat > clatr))
	    {
            /*
              Move out one pixel at a time to find the direction of the curve
            */
            tmpcoord = ddr->upleft[1] + ((midsamp-1) - 1) * ddr->pdist_x;
            c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
		    &tmpcoord,&coordy,&llon,&llat);
    
            tmpcoord = ddr->upleft[1] + ((midsamp+1) - 1) * ddr->pdist_x;
            c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
			&tmpcoord,&coordy,&rlon,&rlat);
	    max3 = MAX3(rlat,llat,lat);
            if (max3 == lat)
	        {
                /*
		  Found high point;
		  maxlat is set after while loop from clatr & clatl (use clatr)
		*/
		clatr = lat;
		clatl = lat;
		break;
	        }
            else if (rlat > lat)
	        {
	        /* Curve rises to the right of the midpoint */
	        clatl = lat;
	        ltx = midsamp;
	        midsamp = (midsamp+1+rtx) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
	        }
            else if (llat > lat)
	        {
	        /* curve rises to the left of the midpoint */
	        clatr = lat;
	        rtx = midsamp;
	        midsamp = (ltx+midsamp+1) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
	        }
	    }
    
        /*
          convert new midpoint to longitude,latitude
        */
        c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       	       &coordx,&coordy,&lon,&lat);
	} /* while */
    /*
      no more pixels in between, pick the maximum 
    */
    *maxlat = max(clatr,clatl);
    }

/*****************************************************************************
  FIND MINLAT
  To find minimum latitude, vary the sample (coordx) across the bottom line 
  of the image (at ddr->loright[0])
*****************************************************************************/
/*
  Set up the initial values for comparing
midsamp = ddr->ns / 2;
ltx = 1;
rtx = ddr->ns;
*/
midsamp =  window[SS] + window[NS] / 2;
coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
coordy = ddr->upleft[0] - (window[SL] + window[NL] - 2) * ddr->pdist_y;
clatl = lllat;
clatr = lrlat;
ltx = window[SS];
rtx = window[SS] + window[NS] - 1;

/*
  Convert the midpoint to a longitude,latitude coordinate
*/
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&lon,&lat);
if ((lat > clatl) && (lat > clatr))
    {
    /*
      the curve is increasing, so the corner points are the minimum
    */
    *minlat = min(lllat,lrlat);
    }
else if ((lat == clatl) && (lat == clatr))
    {
    /*
      the latitude is constant across the bottom of the image
    */
    *minlat = lllat;
    }
else
    {
    while ((abs(midsamp - rtx) > 1) && (abs(midsamp - ltx) > 1))
	{
	/*
  	  while haven't reached the minimum latitude value
	*/
	if ((lat > clatl) && (lat > clatr))
    	    {
    	    /*
      	      error out if this routine has made an invalid assumption
    	    */
            c_errmsg ("Cannot calculate minimum latitude","dcw2ras-calc",
		&nonfatal);
	    return(E_FAIL);
    	    }
	else if ((lat == clatl) && (lat == clatr))
    	    {
    	    /*
      	      assume if the midpoint = the 2 end points, the min has been found
    	    */
    	    break;
    	    }
	else if ((lat <= clatl) && (lat >= clatr))
    	    {
    	    /*
      	    The minimum is to the right of the current midpoint
    	    */
    	    clatl = lat;
    	    ltx = midsamp;
    	    midsamp = (midsamp + rtx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
    	    }
	else if ((lat >= clatl) && (lat <= clatr))
    	    {
    	    /*
      	    The minimum is to the left of the current midpoint
    	    */
    	    clatr = lat;
    	    rtx = midsamp;
    	    midsamp = (midsamp + ltx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
    	    }
	else if ((lat < clatl) && (lat < clatr))
    	    {
	    /*
      	      move out one pixel at a time to find the direction of the curve
    	    */
            tmpcoord = ddr->upleft[1] + ((midsamp-1) - 1) * ddr->pdist_x;
	    c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       		   &tmpcoord,&coordy,&llon,&llat);
    
            tmpcoord = ddr->upleft[1] + ((midsamp+1) - 1) * ddr->pdist_x;
	    c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       	  	   &tmpcoord,&coordy,&rlon,&rlat);
	    min3 = MIN3(rlat,llat,lat);
    	    if (min3 == lat)
		{
		/* found low point */
    		clatr = lat;
		clatl = lat;
		break;
		}
    	    else if (rlat < lat)
		{
		/* curve falls to the right of the midpoint */
		clatl = lat;
		ltx = midsamp;
		midsamp = (midsamp+1+rtx) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
		}
    	    else if (llat < lat)
		{
		/* curve falls to the left of the midpoint */
		clatr = lat;
		rtx = midsamp;
		midsamp = (ltx+midsamp+1) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
		}
   	    }
	/*
          convert new midpoint to longitude,latitude
	*/
	c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       	       &coordx,&coordy,&lon,&lat);
	} /* while */

    *minlat = min(clatr,clatl);
    }

/*****************************************************************************
  FIND MAXLON
  To find maximum longitude, vary the line (coordy) across the right side 
  of the image (at ddr->loright[1])
*****************************************************************************/
/*
  Set up the initial values for comparing
midline = ddr->nl / 2;
topline = 1;
botline = ddr->nl;
*/
midline = window[SL] + window[NL] / 2;
coordx = ddr->upleft[1] + (window[SS] + window[NS] - 2) * ddr->pdist_x;
coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
clont = urlon;
clonb = lrlon;
topline = window[SL];
botline = window[SL] + window[NL] - 1;

/*
  Convert the midpoint to a longitude,latitude coordinate
*/
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&lon,&lat);

/*
  Adjust the longitudes if they cross 0 or 180
*/
if (clont < 0)
    zlont = clont + 360.0;
else
    zlont = clont;

if (clonb < 0)
    zlonb = clonb + 360.0;
else
    zlonb = clonb;

if (lon < 0)
    zlon = lon + 360.0;
else 
    zlon = lon;

if ((zlon < zlont) && (zlon < zlonb))
    /*
      the curve is concave, so the corner points are the greatest
    */
    *maxlon = max(urlon,lrlon);
else if ((zlon == zlont) && (zlon == zlonb))
    /*
      the longitude is constant along the right edge of the image
    */
    *maxlon = urlon;
else
    {
    /*
      while haven't reached the maximum longitude value
    */
    while ((abs(midline - topline) > 1) && (abs(midline - botline) > 1))
	{
	if ((zlon < zlont) && (zlon < zlonb))
      	    {
    	    if ((lon < clont) && (lon < clonb))
		{
		/*
  	  	  error out if this routine has made an invalid assumption
		*/
                c_errmsg ("Cannot calculate maximum longitude","dcw2ras-calc",
		    &nonfatal);
		return(E_FAIL);
		}
    	    else
		{
        	/*
          	the maximum is below the current midpoint
        	*/
        	clont = lon;
        	zlont = zlon;
        	topline = midline;
        	midline = (midline + botline) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
		}
    	    }
	else if ((zlon == zlont) && (zlon == zlonb))
    	    {
    	    /*
      	      assume if the midpoint = the 2 end points, the max has been found
    	    */
	    clont = lon;
	    clonb = lon;
    	    break;
    	    }
	else if ((zlon >= zlont) && (zlon <= zlonb))
    	    {
    	    /*
      	    the maximum is below the current midpoint
    	    */
    	    clont = lon;
    	    zlont = zlon;
    	    topline = midline;
    	    midline = (midline + botline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
    	    }
	else if ((zlon <= zlont) && (zlon >= zlonb))
    	    {
    	    /*
      	      the maximum is above the current midpoint
    	    */
    	    clonb = lon;
    	    zlonb = zlon;
    	    botline = midline;
    	    midline = (midline + topline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
    	    }
	else if ((zlon > zlont) && (zlon > zlonb))
    	    {
    	    /*
      	      move out one pixel at a time to find the direction of the curve
    	    */
    	    tmpcoord = ddr->upleft[0] - (midline-1 - 1) * ddr->pdist_y;
            c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
			&coordx,&tmpcoord,&tlon,&tlat);
    
    	    tmpcoord = ddr->upleft[0] - (midline+1 - 1) * ddr->pdist_y;
            c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
			&coordx,&tmpcoord,&blon,&blat);
	    max3 = MAX3(tlon,blon,lon);
    	    if (max3 == lon)
		{
  		/* found high point */
		clont = lon;
	        clonb = lon;
		break;
		}
    	    else if (blon > lon)
		{
		/* curve rises below the midpoint */
		clont = lon;
		zlont = zlon;
		topline = midline;
		midline = (midline+1+botline) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
		}
    	    else if (tlon > lon)
		{
		/* curve rises above the midpoint */
		clonb = lon;
		zlonb = zlon;
		botline = midline;
		midline = (topline+midline+1) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
		}
    	    }

    	/*
      	  convert new midpoint to longitude,latitude
    	*/
	c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
        	&coordx,&coordy,&lon,&lat);

    	/*
      	  adjust the longitudes if they cross 0 or 180
    	*/
    	if (lon < 0)
	    zlon = lon + 360.0;
    	else 
	    zlon = lon;
    	} /* while */
    /*
      no more pixels in between, pick the maximum 
    */
    *maxlon = max(clont,clonb);
    }

/*****************************************************************************
  FIND MINLON
  To find minimum longitude, vary the line (coordy) across the left side 
  of the image (at ddr->upleft[1])
*****************************************************************************/
/*
  Set up the initial values for comparing
midline = ddr->nl / 2;
topline = 1;
botline = ddr->nl;
*/
midline = window[SL] + window[NL] / 2;
coordx = ddr->upleft[1] + (window[SS] - 1) * ddr->pdist_x;
coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
clont = ullon;
clonb = lllon;
topline = window[SL];
botline = window[SL] + window[NL] - 1;

/*
  convert the midpoint to a longitude,latitude coordinate
*/
c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&coordy,&lon,&lat);
/*
  adjust the longitudes if they cross 0 or 180
*/
if (clont < 0)
    zlont = clont + 360.0;
else
    zlont = clont;

if (clonb < 0)
    zlonb = clonb + 360.0;
else
    zlonb = clonb;

if (lon < 0)
    zlon = lon + 360.0;
else 
    zlon = lon;


if ((zlon > zlont) && (zlon > zlonb))
    /*
      the curve is convex, so the corner points are the minimum
    */
    *minlon = min(ullon,lllon);
else if ((zlon == zlont) && (zlon == zlonb))
    /*
      the longitude is constant along the left side of the image
    */
    *minlon = ullon;
else
    {
    while ((abs(midline - topline) > 1) && (abs(midline - botline) > 1))
	{
        /*
          while haven't reached the minimum longitude value
        */
        if ((zlon > zlont) && (zlon > zlonb))
	    {
            if ((lon > clont) && (lon > clonb))
  	        {
	        /*
  	          error out if this routine has made an invalid assumption
	        */
                c_errmsg ("Cannot calculate minimum longitude","dcw2ras-calc",
	            &nonfatal);
	        return(E_FAIL);
	        }
            else
	        {
                /*
                  the minimum is above the current midpoint
                */
                clonb = lon;
                zlonb = zlon;
                botline = midline;
                midline = (midline + topline) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
	        }
	    }
        else if ((zlon == zlont) && (zlon == zlonb))
            {
            /*
              assume if the midpoint = the 2 end points, the min has been found
            */
            clont = lon;
	    clonb = lon;
            break;
            }
        else if ((zlon <= zlont) && (zlon >= zlonb))
            {
            /*
              the minimum is below the current midpoint
            */
            clont = lon;
            zlont = zlon;
            topline = midline;
            midline = (midline + botline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
            }
        else if ((zlon >= zlont) && (zlon <= zlonb))
            {
            /*
              the minimum is above the current midpoint
            */
            clonb = lon;
            zlonb = zlon;
            botline = midline;
            midline = (midline + topline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
            }
        else if ((zlon < zlont) && (zlon < zlonb))
            {
            /*
              move out one pixel at a time to find the direction of the curve
            */
    	    tmpcoord = ddr->upleft[0] - (midline-1 - 1) * ddr->pdist_y;
            c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
			&coordx,&tmpcoord,&tlon,&tlat);
    
    	    tmpcoord = ddr->upleft[0] - (midline+1 - 1) * ddr->pdist_y;
            c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
			&coordx,&tmpcoord,&blon,&blat);

	    min3 = MIN3(tlon,blon,lon);
            if (min3 == lon)
	        {
	        /* found low point */
		clont = lon;
		clonb = lon;
		break;
	        }
            else if (blon < lon)
	        {
	        /* curve falls below the midpoint */
	        clont = lon;
	        zlont = zlon;
	        topline = midline;
	        midline = (midline+1+botline) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
	        }
            else if (tlon < lon)
	        {
	        /* curve falls above the midpoint */
	        clonb = lon;
	        zlonb = zlon;
	        botline = midline;
	        midline = (topline+midline+1) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
	        }

	    }
        /*
          convert new midpoint to longitude,latitude
        */
	c_trans((long *)&ddr->proj_code,ddr_units, geo_code,geo_units,
       		&coordx,&coordy,&lon,&lat);

        /*
          adjust the longitudes if they cross 0 or 180
        */
        if (lon < 0)
	    zlon = lon + 360.0;
        else 
	    zlon = lon;

	} /* while */

    *minlon = min(clont,clonb);
    }

return(E_SUCC);
}
