/******************************************************************************
NAME:		WRITE_REPORT

PURPOSE:	Write a formatted output report of the AVHRR model header
		and dropped line information.

PROGRAM HISTORY:

VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 D. Hollaren   03/92	Original Development
  5.1    J. Fenno      09/92    Changed the orbit field in the archive header
				structure from orbit to sorbit.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:

NOTE:  This routine is used in DSPADDR and DSPAHDR

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "model.h"
#include "archio.h"
#include "avhrr.h"
#include "adapsutil.h"
#include "adapssup.h"
#include "constants.h"

#define EHDR	9

void write_report
(
char	*hosin,		   /* Input host file name			     */
struct MODEL	*model,	   /* Model and orbital information		     */
struct DROPPED	*drop,	   /* Dropped line information			     */
FILE 	*fp 		   /* File pointer to print file		     */
)
{
double	temp;		   /* Input location in total degrees	             */

char	spass[5],	   /* Direction of the pass at start	             */
	cpass[5],	   /* Direction of the pass at center	             */
	epass[5],	   /* Direction of the pass at center	             */
	dtype[5],	   /* Data type of the pass (1)LAC (2)GAC (3)HRPT    */
	day_night[6],	   /* Day/Night indicator (1)DAY (-1)NIGHT	     */
	chans[6],	   /* Channel numbers			             */
	ebuf[HDR_SIZ];	   /* Ephemeris buffer				     */

char	satnam[8],	   /* Satellite name (NOAA-11)			     */
	satno[4],	   /* Satellite number (N11)			     */
	satid[6],	   /* Satellite id (19531)			     */
	satcode[2];	   /* Satellite code (I)			     */

long	deg,		   /* Degrees					     */
	mins,		   /* Minutes					     */
	sec,		   /* Seconds					     */
	year,		   /* Four digit year				     */
	i;		   /* Looping variable				     */

/*
  Write out the name of the input file
*/
fprintf(fp,"\n\nFILE: %s\n\n", hosin);

/*
  Conver the archive data type to ascii
*/
if (model->dtype == LAC)
    strcpy(dtype,"LAC");
else if (model->dtype == GAC)
    strcpy(dtype,"GAC");
else
    strcpy(dtype,"HRPT");

/*
  Get the satellite name
*/
c_getsat("SATNUM",&model->satnum,satnam,satno,satid,satcode);

fprintf(fp,"SATELLITE INFORMATION\n");
fprintf(fp,"    Satellite: %s  Type: %4s\n",satnam,dtype);

/*
  Make the year a four digit year
*/
if (model->year < 70)
   year = model->year + 2000;
else if (model->year < 100)
   year = model->year + 1900;

fprintf(fp,"    Date: %02d/%02d/%04d   Julian Date: %02d%03d\n",
	model->month,model->day,year,model->year,model->jday);

fprintf(fp,"    Start Time:  %02d:%02d:%02d.%03d\n",
	model->shour, model->smin, model->ssec, model->sfrac);

fprintf(fp,"    End   Time:  %02d:%02d:%02d.%03d\n",
	model->ehour, model->emin, model->esec, model->efrac);

/*
  Set a string for the direction of the pass at the start of the pass
*/
if (model->spass == DESCENDING)
    strcpy(spass,"Desc");
else
    strcpy(spass,"Asc ");

/*
  Set a string for the direction of the pass at the center of the pass
*/
if (model->cpass == DESCENDING)
    strcpy(cpass,"Desc");
else
    strcpy(cpass,"Asc ");

/*
  Set a string for the direction of the pass at the end of the pass
*/
if (model->epass == DESCENDING)
    strcpy(epass,"Desc");
else
    strcpy(epass,"Asc ");

/*
  Convert the day/night indicator to ascii
*/
if (model->day_night == DAY)
    strcpy(day_night,"DAY");
else
    strcpy(day_night,"NIGHT");

fprintf(fp,"    Orbit: %05d  D/N: %5s  Direction: %s %4s %s %4s %s %4s\n",
	model->sorbit,day_night,"(Start)",spass,"(Center)",cpass,"(End)",epass);

/*
  Convert/print the sun zenith angle
*/
temp = model->sunzen * DEG_PER_RAD;
fprintf(fp,"    Sun Zenith: %8.3lf\n",temp);

/*
  Convert/print the Equitorial crossing
*/
temp = model->eqcrs * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    Equitorial crossing: %3dD %02dM %02dS\n\n",deg,mins,sec);

/*
  Set a string that represents the channel numbers in the image
*/
sprintf(chans,"%1d%1d%1d%1d%1d",model->chans[0], model->chans[1],
	model->chans[2], model->chans[3], model->chans[4]);
	    
/*
  Convert the image information
*/
fprintf(fp,"IMAGE INFORMATION\n");
fprintf(fp,"    # Lines: %5d  # Samples: %4d  # Missed:  %4d\n",
	model->nlines, model->nsamples, model->totmiss);

fprintf(fp,"    # Bands: %1d  Channels: %s  Station: %4s\n\n",
	model->nchans, chans,model->station);

fprintf(fp,"ATTITUDE CORRECTIONS\n");
fprintf(fp,"    Clock error: %5d  Delta time: %lf\n\n",
	model->clock_err,model->delta_time);

fprintf(fp,"    Roll coefficients:      %lf %lf %lf %lf %lf\n",
	model->roll_coefs[0], model->roll_coefs[1], model->roll_coefs[2],
	model->roll_coefs[3], model->roll_coefs[4]);

fprintf(fp,"    Pitch coefficients:     %lf %lf %lf %lf %lf\n",
	model->pitch_coefs[0], model->pitch_coefs[1], model->pitch_coefs[2],
	model->pitch_coefs[3], model->pitch_coefs[4]);

fprintf(fp,"    Yaw coefficients:       %lf %lf %lf %lf %lf\n",
	model->yaw_coefs[0], model->yaw_coefs[1], model->yaw_coefs[2],
	model->yaw_coefs[3], model->yaw_coefs[4]);

fprintf(fp,"    Altitude coefficients:  %lf %lf %lf %lf %lf\n\n",
	model->alt_coefs[0], model->alt_coefs[1], model->alt_coefs[2],
	model->alt_coefs[3], model->alt_coefs[4]);

fprintf(fp,"COORDINATE POINTS\n");
/*
  Convert the North West point
*/
temp = model->nw[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    NW: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->nw[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the North Nadir point
*/
temp = model->nn[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    NN: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->nn[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the North East point
*/
temp = model->ne[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    NE: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->ne[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the Center West point
*/
temp = model->cw[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    CW: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->cw[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the Center Nadir point
*/
temp = model->cn[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    CN: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->cn[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the Center East point
*/
temp = model->ce[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    CE: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->ce[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the South West point
*/
temp = model->sw[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    SW: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->sw[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the South Nadir point
*/
temp = model->sn[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    SN: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->sn[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n",deg,mins,sec);

/*
  Convert the South East point
*/
temp = model->se[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"    SE: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->se[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(fp,"%4dD %02dM %02dS\n\n",deg,mins,sec);

/*
  Write the ephemris information
*/
fprintf(fp,"EPHEMERIS INFORMATION\n");
c_toephem(model,ebuf);
ebuf[EHDR + EPHEM_SIZ - 1] = '\0';
fprintf(fp,"    %s\n\n",&ebuf[EHDR]);

/*
  Write the dropped line information
*/
fprintf(fp,"DROPPED LINE INFORMATION\n");

/*
  Convert the gap counter
*/
fprintf(fp,"    Gaps: %5d  Total Missed:  %4d\n",drop->ngaps, model->totmiss);

/*
  Convert the location and number of the dropped lines
*/
for (i = 0; i < drop->ngaps; i++)
    {
    fprintf(fp,"    Line: %5d  # Missed: %5d\n",drop->lmissed[i],
	    drop->nmissed[i]);
    }

return;
}
