/***************************************************************************
FUNCTION:	PROCESS

PURPOSE:	Open and read the input files into buffers.  Combine the
		buffers into one and create the QC buffer.

Version	Date	Programmer	Modification
-------	-----	------------	--------------------------------
  1.0	10/99	D.Lloyd		Initial Development

PROJECT:	ADAPS


***************************************************************************/
#include "las.h"
#include "util.h"
#include "filltovs.h"


int process
( 
char	infile[3][CMLEN],	/* Input file name			*/
float	wvbuf[NLINES][NSAMPS],	/* daily observation H2O buffer		*/
float	qcbuf[NLINES][NSAMPS]	/* quality control buffer	*/
)
{
float	dfval[NLINES][NSAMPS];	/* daily fill value buffer		*/
float	mfval[NLINES][NSAMPS];	/* monthly fill value buffer		*/
float	mfcnt[NLINES][NSAMPS];	/* monthly fill count buffer		*/

struct	FDESC	*fdesc;		/* image I/O file descriptor		*/

long	band;			/* image band number			*/
long	dtype = EREAL;		/* image data type			*/
long	fill = NOFIL;		/* fill technique specification flag	*/
long	i;			/* loop control variable		*/
long	j;			/* loop control variable		*/
long	k;			/* loop control variable		*/
long	nextflg;		/* call type: CREAT INIT CURNT NEXT	*/
long	nlav;			/* number of valid lines in buffer	*/
long	nlb = 0;		/* number of image boundary lines	*/
long	nlbuf = NLINES;		/* total number of lines in buffer	*/
long	nli = NLINES;		/* number of lines in image window	*/
long	nsav;			/* number of valid samples in buffer	*/
long	nsb = 0;		/* number of image boundary samples	*/
long	nsbuf = NSAMPS;		/* total number of samples in buffer	*/
long	nsi = NSAMPS;		/* number of samples in image window	*/
long	slcb = 1;		/* starting line of data within image	*/
long	sli = 1;		/* starting line of image window	*/
long	sscb = 1;		/* starting sample of data within image	*/
long	ssi = 1;		/* starting sample of image window	*/
long	status;			/* Function return status		*/

unsigned char	*buffer;	/* pointer to input buffer		*/


/* Read the to-be-filled water vapor image
The call to get_buf opens the image.
------------------------------------------*/
buffer = (unsigned char *)wvbuf;
nextflg = INIT;
band = 1;
status = get_buf( buffer, &nlbuf, &nsbuf, &nlav, &nsav, &sli, &ssi, &nli,
		  &nsi, &nlb, &nsb, &slcb, &sscb, &nextflg, &band, &fill,
		  &fdesc, infile[0], &dtype);
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"filltovs-gtbuf","Error reading from image %s", infile[0]);
    c_eclose( &fdesc );
    return E_FAIL;
    }

/* Close the water vapor image
------------------------------*/
status = c_eclose( &fdesc );
if ( status != E_SUCC )
    c_lasmsg(FALSE,"filltovs-clos","Error closing file %s", infile[0]);


/* for each fill file read the image and count data
---------------------------------------------------*/
for ( k = 1; k < 3; k++ )
    {
    /* Read band one of the fill image.  This
    first get_buf() call also opens the image.
    -----------------------------------------*/
    if ( k == 1 )
	buffer = (unsigned char *)dfval;
    else
	buffer = (unsigned char *)mfval;
    nextflg = INIT;
    band = 1;
    status = get_buf( buffer, &nlbuf, &nsbuf, &nlav, &nsav, &sli, &ssi, &nli,
   		      &nsi, &nlb, &nsb, &slcb, &sscb, &nextflg, &band, &fill,
		      &fdesc, infile[k], &dtype);
    if ( status != E_SUCC )
	{
	c_lasmsg(FALSE,"filltovs-gtbuf","Error reading from image %s",
	         infile[k]);
	c_eclose( &fdesc );
	return E_FAIL;
	}

    /* read band two of the monthly fill image which contains the
    counts of the data.  These values will be used for the QC band
    to determine if the value in the monthly climatology was
    derived from daily climatology or an 8-neighbor mean.  If the
    count value is zero it is an 8-neighbor mean.
    --------------------------------------------------------------*/
    if ( k == 2 )
	{
	buffer = (unsigned char *)mfcnt;
	nextflg = CURNT;
	band = 2;
	status = get_buf( buffer, &nlbuf, &nsbuf, &nlav, &nsav, &sli, &ssi,
			  &nli, &nsi, &nlb, &nsb, &slcb, &sscb, &nextflg,
			  &band, &fill, &fdesc, infile[k],
			  &dtype);
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"filltovs-gtbuf","Error reading from image %s",
	             infile[k]);
	    c_eclose( &fdesc );
	    return E_FAIL;
	    }
	}

    /* close this image file
    ------------------------*/
    status = c_eclose( &fdesc );
    if ( status != E_SUCC )
        c_lasmsg(FALSE,"filltovs-clos","Error closing file %s", infile[k]);
    }

/* Now fill the water vapor buffer with values from the
fill buffers and create the QC buffer.
-------------------------------------------------------*/
for ( i = 0; i < NLINES; i++ )
    {
    for ( j = 0; j < NSAMPS; j++ )
        {
	/* If the H2O value is negative it's a hole so fill it */
	if ( wvbuf[i][j] < 0.0 )
	    {
	    /* Use the daily clim. value if it's valid */
	    if ( dfval[i][j] >= 0.0 )
	        {
		wvbuf[i][j] = dfval[i][j];
		qcbuf[i][j] = 1.0; /* mark QC as a daily clim value */
		}
	    /* Otherwise use the monthly clim. value, it is assumed valid */
	    else
	        {
		wvbuf[i][j] = mfval[i][j];
		/* If there is a count it was derived from daily clim. values */
		if ( mfcnt[i][j] > 0.0 )
		    qcbuf[i][j] = 2.0;
		/* Otherwise it was an 8-neighbor mean */
		else
		    qcbuf[i][j] = 3.0;
		}
	    }
	}
    }

return E_SUCC;
} 
