/*******************************************************************************
FUNCTION:	c_getlui

PURPOSE:	Given a pointer to a Look Up Image (LUI) and a line, sample
		pair, return the value of that pixel.  The line, sample values
		are relative to the window used to open the LUI.  For example,
		if a LUI is opened with window (100 100 1000 1000) then
		requesting pixel 1, 1 will return the value for pixel 100, 100
		within the original image.  If the requested pixel lies outside
		the LUI then zero will be returned.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development


PROJECT:  ADAPS
*******************************************************************************/
#include "getatm.h"


/* Reference the array of LUI structures
----------------------------------------*/
extern long firstlui;     /* Flag indicating no LUIs have been loaded yet     */
extern long lui_access[]; /* Number of pixels accessed outside of each LUI    */
extern struct LUI lui[];  /* Array of MAXLUIS Look Up Images                  */


int c_getlui
(
    long i,              /* IN  - Ptr returned by c_loadlui                   */
    long line,           /* IN  - Line number of the desired pixel            */
    long sample,         /* IN  - Sample number of the desired pixel          */
    unsigned char *value /* OUT - Ptr to var of type specified in c_loadlui   */
)
{

long index;              /* Index into the LUI buffer                         */
long nonfatal = 1;       /* Nonfatal error status                             */
long one      = 1;       /* Value of one                                      */
long status;             /* Function return status                            */

unsigned char *ptr;      /* Ptr to the LUI value in the buffer                */



/* Ensure i points to a valid LUI
---------------------------------*/
if (firstlui || (i < 0) || (i >= MAXLUIS))
    {
    c_errmsg("Invalid look up image specified", "c_getlui-lui", &nonfatal);
    return(E_FAIL);
    }


/* Ensure the specified LUI is opened
-------------------------------------*/
if ((lui[i].fdesc == 0) || (lui[i].buffer == NULL))
    {
    c_errmsg("Specified look up image is not open", "c_getlui-open", &nonfatal);
    return(E_FAIL);
    }


/* Determine the number of bytes needed to store a single
   line of the LUI and set the output value to 0
---------------------------------------------------------*/
switch (lui[i].dtype)
    {
    case EBYTE:
	*value = 0;
	/*
	nbytes = lui[i].window[NS] * sizeof(unsigned char);
	*/
	break;
    case EWORD:
	*(short *)value = 0;
	/*
	nbytes = lui[i].window[NS] * sizeof(short);
	*/
	break;
    case ELONG:
	*(long *)value = 0;
	/*
	nbytes = lui[i].window[NS] * sizeof(long);
	*/
	break;
    case EREAL:
	*(float *)value = 0.0;
	/*
	nbytes = lui[i].window[NS] * sizeof(float);
	*/
	break;
    }


/* Determine if the pixel is not within the LUI image
-----------------------------------------------------*/
if ((sample < 1) || (sample > lui[i].window[NS]) ||
    (line   < 1) || (line   > lui[i].window[NL]))
    {
    lui_access[i]++;
    return(E_SUCC);
    }


/* If the requested line is not in the buffer then fill the
   buffer so that the line is in the middle of the buffer
-----------------------------------------------------------*/
if ((line < lui[i].first) || (line > lui[i].last))
    {
    /* Determine the first line to be placed in the buffer
    ------------------------------------------------------*/
    lui[i].first = line - lui[i].bufsize / 2;
    if (lui[i].first < 1) lui[i].first = 1;


    /* Determine the last line to be placed in the buffer.  If the last line in
       the buffer is greater than the number of lines in the image then reset it
       to the last line in the image.  When recalculating the first line in the
       buffer we do not have to worry about lui[i].first being less than 1
       because the buffer size will be at most the number of lines in the image.
    --------------------------------------------------------------------------*/
    lui[i].last = lui[i].first + lui[i].bufsize - 1;
    if (lui[i].last > lui[i].window[NL])
	{
	lui[i].last  = lui[i].window[NL];
	lui[i].first = lui[i].last - lui[i].bufsize + 1;
	}


    /* Load the buffer with lui[i].bufsize lines starting with line lui[i].first
    --------------------------------------------------------------------------*/
    status = c_eread((struct FDESC **)&lui[i].fdesc, &one, &lui[i].first,
        lui[i].buffer, &lui[i].bufsize);
    if (status == E_FAIL)
	{
	c_closelui(i);
	return(E_FAIL);
	}
    }


/* Get the value for the specified pixel
----------------------------------------*/
index = (line - lui[i].first) * lui[i].window[NS] + sample - 1;
switch (lui[i].dtype)
    {
    case EBYTE:
	index *= sizeof(unsigned char);
	ptr = &lui[i].buffer[index];
	*value = *ptr;
	break;
    case EWORD:
	index *= sizeof(short);
	ptr = &lui[i].buffer[index];
	*(short *)value = *(short *)ptr;
	break;
    case ELONG:
	index *= sizeof(long);
	ptr = &lui[i].buffer[index];
	*(long *)value = *(long *)ptr;
	break;
    case EREAL:
	index *= sizeof(float);
	ptr = &lui[i].buffer[index];
	*(float *)value = *(float *)ptr;
	break;
    }

return(E_SUCC);
}
