/*******************************************************************************
FUNCTION:	c_loadlui

PURPOSE:	Given the name of a Look Up Image (LUI) open the image and load
		as much of the image into memory as possible.  The user may
		specify a window for the LUI.  If either window[SL] or
		window[SS] are less than one the entire image will be loaded.
		If either window[NL] or window[NS] are less than zero then the
		remaining portion of the image from pixel window[SL],
		window[SS] will be loaded into memory.  Window will be updated
		to reflect the portion of the image that was opened.  If nlines
		is greater than zero its value will be used as the maximum
		number of lines to load into memory instead of window[NL].


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    03/93  J.Fenno          Development
  3.0    03/94  D. Hollaren	 Added more stringent window checking
  3.1    01/97  T. Baltzer	 Modify to read one line of data at a time.
				 Do this to avoid memory errors in c_eopenr().


PROJECT:  ADAPS
*******************************************************************************/
#include "getatm.h"


/* Declare global data structures
---------------------------------*/
long firstlui = TRUE;     /* Flag indicating no LUIs have been loaded yet     */
long lui_access[MAXLUIS]; /* Number of pixels accessed outside of each LUI    */
struct LUI lui[MAXLUIS];  /* Array of MAXLUIS Look Up Images                  */


int c_loadlui
(
    char  *hostname,     /* IN     - Hostname of the LUI image                */
    long  window[4],     /* IN/OUT - Window for the LUI image                 */
    long  dtype,         /* IN     - Data type to be returned by c_getlui     */
    long  *ptr,          /* OUT    - Pointer to the LUI opened                */
    long  *nlines        /* IN/OUT - On input this is the maximum number of   */
                         /*          lines that the user would like to have   */
                         /*          read into the buffer.  If zero this      */
                         /*          function will attempt to read the whole  */
                         /*          image into the buffer.  On output, this  */
                         /*          is the number of lines held in the       */
                         /*          buffer.                                  */
)
{
char str[CMLEN];         /* Temporary string                                  */
unsigned char* buffer_pntr; /* Buffer pointer				      */

long access   = IREAD;   /* Access mode for the LUI image                     */
long i;                  /* Index into the LUI array                          */
long jj;		 /* Index used to read in buffer of data	      */
long nbands   = 0;       /* Number of bands in the LUI image                  */
long nonfatal = 1;       /* Nonfatal error status                             */
long one      = 1;       /* Value of one                                      */
long status;             /* Function return status                            */
long option[3];		 /* Open options				      */

struct DDR ddr;          /* DDR structure for the LUI                         */

unsigned long nbytes;    /* Number of bytes used for one line of the image    */


/* If no Look Up Images have been opened yet then initialize the array
----------------------------------------------------------------------*/
if (firstlui)
    {
    firstlui = FALSE;
    for (i = 0; i < MAXLUIS; i++)
	{
	lui[i].fdesc  = 0;
	lui[i].buffer = NULL;
	lui_access[i] = 0;
	}
    }


/* Determine which structure of the array can be used
-----------------------------------------------------*/
*ptr = -1;
i = 0;
while ((i < MAXLUIS) && (lui[i].fdesc != 0))
    i++;
if (i == MAXLUIS)
    {
    *nlines = 0;
    sprintf(str, "Maximum of %1d look up images already opened", MAXLUIS);
    c_errmsg(str, "c_loadlui-max", &nonfatal);
    return(E_FAIL);
    }


/* Read the DDR for the image and ensure the required fields are valid
----------------------------------------------------------------------*/
if (c_getddr(hostname, &ddr) != E_SUCC)
    return(E_FAIL);

/* Determine/Check the window for the LUI
-----------------------------------------*/
if ((window[SL] > ddr.nl) || (window[SS] > ddr.ns))
    {
    c_errmsg("Invalid window specified", "c_loadlui-window", &nonfatal);
    return(E_FAIL);
    }

if (window[SL] <= 0)
    window[SL] = 1;

if (window[SS] <= 0)
    window[SS] = 1;

if ((window[NL] <= 0) || (window[NL] > ddr.nl) ||
    ((window[SL] + window[NL] - 1) > ddr.nl))
    window[NL] = ddr.nl - window[SL] + 1;

if ((window[NS] <= 0) || (window[NS] > ddr.ns) ||
    ((window[SS] + window[NS] - 1) > ddr.ns))
    window[NS] = ddr.ns - window[SS] + 1;

/* Determine the maximum number of lines to be stored in the buffer
-------------------------------------------------------------------*/
if (*nlines > 0)
    lui[i].bufsize = *nlines;
else
    lui[i].bufsize = window[NL];
*nlines = 0;


/* Determine the number of bytes used for one line of the buffer
----------------------------------------------------------------*/
switch (dtype)
    {
    case EBYTE:
	nbytes = (long)window[NS] * sizeof(unsigned char);
	break;
    case EWORD:
	nbytes = (long)window[NS] * sizeof(short);
	break;
    case ELONG:
	nbytes = (long)window[NS] * sizeof(long);
	break;
    case EREAL:
	nbytes = (long)window[NS] * sizeof(float);
	break;
    default:
	strcpy(str, "Invalid data type.  Valid types: EBYTE EWORD ELONG EREAL");
	c_errmsg(str, "c_loadlui-dtype", &nonfatal);
	return(E_FAIL);
    }


/* Attempt to allocate as large a buffer as possible for the LUI
----------------------------------------------------------------*/
while ((lui[i].buffer == NULL) && (lui[i].bufsize > 0))
    {
    lui[i].buffer = (unsigned char *)malloc(lui[i].bufsize * nbytes);
    if (lui[i].buffer == NULL) lui[i].bufsize -= LUIDECREASE;
    }
if (lui[i].buffer == NULL)
    {
    c_errmsg("Error allocating memory", "c_loadlui-alloc", &nonfatal);
    return(E_FAIL);
    }

/* Open the image for random access.  Open it to read only one line at a
   time, will call c_eread() multiple times to get all of the needed lines.
   Do this to avoid memory limit errors in c_eopenr().
-----------------------------------------------------------------------------*/
option[0] = option[1] = option[2] = 0;
status = c_eopenr((struct FDESC **)&lui[i].fdesc, hostname, &nbands,
    &window[SL], &window[SS], &window[NL], &window[NS], &access, &dtype,
    option, &one);
if (status != E_SUCC)
    {
    c_closelui(i);
    return(E_FAIL);
    }


/* Ensure the LUI contains only one band
----------------------------------------*/
if (nbands != 1)
    {
    c_closelui(i);
    strcpy(str, "Look up image must contain only one band");
    c_errmsg(str, "c_loadlui-band", &nonfatal);
    return(E_FAIL);
    }


/* Load the buffer with lui[i].bufsize lines starting with line window[SL]
--------------------------------------------------------------------------*/
buffer_pntr = lui[i].buffer;
for (jj = 1; jj <= lui[i].bufsize; jj++)
    {
    status = c_eread((struct FDESC **)&lui[i].fdesc, &one, &jj, buffer_pntr,
        &one);
    if (status == E_FAIL)
	{
	c_closelui(i);
    	return(E_FAIL);
	}
    buffer_pntr += nbytes;
    }

/* Set the first, last, dtype, and window fields of lui[i]
----------------------------------------------------------*/
lui[i].first      = 1;
lui[i].last       = lui[i].bufsize;
lui[i].dtype      = dtype;
lui[i].window[SL] = window[SL];
lui[i].window[SS] = window[SS];
lui[i].window[NL] = window[NL];
lui[i].window[NS] = window[NS];
*ptr = i;
*nlines = lui[i].bufsize;


/* Reset the counter that keeps track of how many pixels
   have been accessed that do not lie within the LUI.
--------------------------------------------------------*/
lui_access[i] = 0;

return(E_SUCC);
}
