/*****************************************************************************
FUNCTION:	GETATM

PURPOSE:	Create an image containing atmospheric correction values
		for elevation, ozone, water vapor, angles and a QC/QA image.
		The correction values are sub-sampled at grid points from
		the imput images and filled using either a nearest neighbor
		or a bi-linear method.  The QC/QA image indicates solar
		zenith angles greater than 80 degrees and the source of
		TOVS water vapor values.
		

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    03/99	D.Lloyd		Initial development
  1.1    09/99  D.Lloyd		Modified to accept an input image with valid
				projection information in the DDR, also ability
				to create two additional bands, line and sample
				of geometric TOVS/ozone image.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "getatm.h"


int main( void )
{
char	hosin[CMLEN];		/* output host file name		*/
char	hosout[CMLEN];		/* output host file name		*/
char	infiles[4][CMLEN];	/* input file names			*/
char	*thebuf;		/* image output buffer			*/

int	compflg;		/* image compression flag		*/
int	fdesc[2];		/* output file descriptor		*/
int	gdesc;			/* group descriptor			*/

long	angflg;			/* generate angles bands flag		*/
long	interpflg;		/* interpolation flag, NN or BL		*/
long	linegrid;		/* number lines for grid cell		*/
long	lsbflg;			/* line/sample band flag		*/
long	msgtim;			/* time between processing message	*/
long	offset[MAXBNDS];	/* offsets into the output buffer	*/
long	out = 1;		/* number of output files		*/
long	sampgrid;		/* number samples in grid cell		*/
long	scaling = NO_SCALE;	/* angles scaling flag			*/
long	status;			/* function return status		*/
long	stdlun;			/* standard output			*/
long	window[4];		/* input image window values		*/

struct	DDR	*ddr = NULL;	/* input image data descriptor struct	*/
struct	FRAME	frame;		/* output framing parameters		*/
struct	MODEL	*model = NULL;	/* AVHRR model structure		*/
struct	PARBLK  vblock;		/* TAE variable block			*/



/* Initialize LAS
-----------------*/
c_inlas( &vblock, &stdlun );


/* Get the input parameters
---------------------------*/
status = getpar( &vblock, hosin, hosout, infiles, &angflg, &linegrid,
		&sampgrid, &frame, &model, &msgtim, &interpflg,
		&ddr, &lsbflg, window, &scaling, &compflg );
if ( status != E_SUCC )
    return E_FAIL;

/* Determine the window for the elevation, ozone and water vapor images
-----------------------------------------------------------------------*/
status = getwindows( &frame, model );
if ( status != E_SUCC )
    return E_FAIL;

/* Open the output image
------------------------*/
status = openlas( hosout, &frame, window, linegrid, sampgrid, angflg,
		lsbflg, fdesc, &gdesc, &thebuf, offset, compflg );
if ( status != E_SUCC )
    return E_FAIL;

/* Create the output image
--------------------------*/
status = process( &gdesc, offset, thebuf, &frame, model, angflg, linegrid,
		sampgrid, &msgtim, interpflg, lsbflg, ddr, window, scaling,
		hosin );
if ( status != E_SUCC )
    {
    c_egclse((struct GDESC **)&gdesc );
    return E_FAIL;
    }

/* Close images
---------------*/
status = c_egclse((struct GDESC **)&gdesc );
if ( status != E_SUCC )
    return E_FAIL;

/* Update the output DDR with band information
----------------------------------------------*/
update_ddr( hosout, ddr, &frame, angflg, lsbflg, scaling );

/* Generate standard completion message
---------------------------------------*/
c_complt( "GETATM", hosout, &out );
return E_SUCC;
}
