#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "ltable.h"
#include "proj.h"
#include "adaps.h"
#include "grid.h"
#include "util.h"

#define	NN	54	/* nearest neighbor interpolation		*/
#define	BL	45	/* bi-linear interpolation			*/


/* Defines used for Look Up Images (LUI).  A LUI is an image that contains
   values that must be accessed quickly.  Therefore, an attempt is made to load
   the entire image into memory.  This group of definitions should some day be
   removed and made into a support along with the functions c_closelui,
   c_getlui, and c_loadlui.
==============================================================================*/
#define LUIDECREASE   100 /* # of lines to decrease a LUI buffer by           */
#define MAXLUIS        10 /* Max number of LUI's opended at the same time     */
struct LUI
   {
   int  fdesc;            /* File descriptor for the LUI image                */
   long bufsize;          /* Number of lines that can be held in the buffer   */
   long dtype;            /* Data type that the user would like returned      */
   long first;            /* First line that is currently in the buffer       */
   long last;             /* Last line that is currently in the buffer        */
   long window[4];        /* Window of the LUI that has been opened           */
   unsigned char *buffer; /* Buffer for the LUI data                          */
   };


/* Defines used in the atmospheric correction routines
==============================================================================*/
#define ALINES        720 /* Lines per A coefficient LUT                      */
#define BLINES      12960 /* Lines per B coefficient LUT                      */
#define MXBND           2 /* maximum number of bands to process               */
#define NSATK           7 /* # of NOAA satellites (6-12) with known K values  */
#define NSATS           6 /* # of NOAA satellites (6-11) with known S values  */
#define PRCNT          18 /* Number of relative azimuth increments in LUT     */
#define SECNT          12 /* Number of satellite zenith increments in LUT     */
#define SUCNT          12 /* Number of solar zenith increments in LUT         */
#define ZICNT           5 /* Number of elevation increments in LUT            */


/* Other misc defines
==============================================================================*/
#define BUFFER_DEG      5 /* Number of degrees to enlarge LUI images with     */
#define LAT             0 /* Latitude index for each corner in frame.latlong  */
#define LON             1 /* Longitude index for each corner in frame.latlong */
#define OUTPUT_XY     105 /*                                                  */
#define OUTPUT_LS     106 /*                                                  */
#define INPUT         107 /*                                                  */
#define MINBOX        108 /*                                                  */
#define NO_RECTIFY    -99 /*                                                  */
#define NO_SCALE        0 /* Angle values are not scaled                      */
#define US_SCALE        1 /* Angle values are scaled for US composites        */
#define GLOBAL_SCALE    2 /* Angle values are scaled for Global composites    */
#define QCBAND3A       10 /* Value to indicate channel 3A in the QC band      */
#define QCBAND3B       20 /* Value to indicate channel 3B in the QC band      */

#define MAXBNDS         9 /* Max number of output bands			      */

/* Misc macros
==============================================================================*/
#define DMOD(A,B)  (A) - ((long)((A) / (B)) * (B))


/* Structure to hold information about the elevation, ozone,
   and water vapor LUIs.  This structure is specific to rectify.
==============================================================================*/
struct LUIIMAGE
   {
   char   hostname[CMLEN];/* Host file name of the image                      */
   double ullat;          /* Latitude of the upper left corner                */
   double ullon;          /* Longitude of the upper left corner               */
   double lrlat;          /* Latitude of the lower right corner               */
   double lrlon;          /* Longitude of the lower right corner              */
   double pdistx;         /* Pixel distance in the x (sample) direction       */
   double pdisty;         /* Pixel distance in the y (line) direction         */
   long   nl;             /* Number of lines in the image                     */
   long   ns;             /* Number of samples in the image                   */
   long   window[4];      /* Window for opening the image                     */
   };


/* Structure containing framing parameters & related gridding
   information.  These are all user-entered parameters.
==============================================================================*/
struct FRAME 
   {
   char   geogrid[CMLEN]; /* Name of the geometric mapping grid               */
   char   anggrid[CMLEN]; /* Name of the angles grid                          */
   char   atmgrid[CMLEN]; /* Name of the atmospheric correction grid          */
   double corners[8];     /* Resulting grid projection corners                */
   double latlong[8][2];  /* Lat/Lon of eight points around the output space  */
   double o_dist[2];      /* Projection distance in y & x; input space        */
   double tol;            /* Tolerance for linear approximation-grid reduce   */
   long   ngline;         /* Number of lines in a grid cell                   */
   long   ngsamp;         /* Number of samples in a grid cell                 */
   long   window[4];      /* Window parameters for output space               */
   struct LUIIMAGE elev;  /* Structure containing info about elevation image  */
   struct LUIIMAGE ozone; /* Structure containing info about ozone image      */
   struct LUIIMAGE water; /* Structure containing info about water vapor image*/
   struct LUIIMAGE qc;    /* Structure containing info about water vapor qc   */
   FILE   *fptr;          /* File pointer to print file                       */
   };


/* Structure holding projection parameters
==============================================================================*/
struct PROJ
   {
   long proj_code;        /* Projection code number                           */
   long units;            /* Projection units code number                     */
   long zone_code;        /* Projection zone code (UTM or State Plane)        */
   long datum_code;       /* Projection datum code                            */
   double proj_coef[15];  /* 15 projection coefficients                       */
   };

/* Structure used for storing scaling, offset quantities
--------------------------------------------------------*/
struct SCOFF
    {
    double minval;	/* Minimum value of appropriate quantity	*/
    double maxval;	/* Maximum value of appropriate quantity	*/
    double scale;	/* Scale value to apply				*/
    double offset;	/* Offset value to apply			*/
    double unscale;	/* Value to apply to unscale			*/
    };

/***************************************************************
	FUNCTION prototypes
***************************************************************/
#ifndef GETATM_H
#define GETATM_H

/* c_closelui.c */
int c_closelui 
(
    long  i
);

/* c_getlui.c */
int c_getlui 
(
    long i, 
    long line, 
    long sample, 
    unsigned char *value
);

/* c_loadlui.c */
int c_loadlui 
(
    char  *hostname, 
    long  window[4], 
    long  dtype, 
    long  *ptr, 
    long  *nlines
);

/* closelui.c */
void closelui
(
    long	elevlui, 
    long	ozonelui, 
    long	qclui, 
    long	waterlui
);

/* getgrid.c */
int getgrid
(
    long	luiptr, 
    struct	LUIIMAGE *luiimage, 
    long	numgrids, 
    float	*latlon[], 
    long	dtype, 
    float	gridbuf[]
);

/* getpar.c */
int getpar
(
    struct	PARBLK	*vblock, 
    char	*hosin, 
    char	*hosout, 
    char	infiles[][CMLEN], 
    long	*angflg, 
    long	*linegrid, 
    long	*sampgrid, 
    struct	FRAME	*frame, 
    struct	MODEL	**model, 
    long	*msgtim, 
    long	*interpflg,
    struct	DDR	**imgddr,
    long	*lsbflg,
    long	window[],
    long	*scaling,
    int		*compflg
);

/* getwindows.c */
int getwindows 
(
    struct FRAME  *frame, 
    struct MODEL  *model
);

/* interp.c */
float interp
(
    long	y0, 
    long	y1, 
    long	y2, 
    long	x0, 
    long	x1, 
    long	x2, 
    float	v1, 
    float	v2, 
    float	v3, 
    float	v4, 
    float	*v0
);
float nn
(
    long	y0, 
    long	y1, 
    long	y2, 
    long	x0, 
    long	x1, 
    long	x2, 
    float	v1, 
    float	v2, 
    float	v3, 
    float	v4, 
    float	*v0
);
float nvn
(
long    l0,             /* IN - interpolate point line location         */
long    l1,             /* IN - upper left line location                */
long    l2,             /* IN - lower right line location               */
long    s0,             /* IN - interpolate point sample location       */
long    s1,             /* IN - upper left sample location              */
long    s2,             /* IN - lower right sample location             */
float   v1,             /* IN - Q1 interpolation value                  */
float   v2,             /* IN - Q2 interpolation value                  */
float   v3,             /* IN - Q3 interpolation value                  */
float   v4              /* IN - Q4 interpolation value                  */
);


/* loadll.c */
void loadll
(
    long	curline, 
    long	sampgrid, 
    long	angflg, 
    struct	MODEL	*model, 
    struct	DDR	*ddr,
    long	proj_units,
    long	window[],
    long	scaling,
    struct	SCOFF	scoff[],
    float	*latbuf, 
    float	*lonbuf, 
    float	*satzebuf, 
    float	*solzebuf, 
    float	*relazbuf,
    double	rot_coeffs[]
);

/* openlas.c */
int openlas
(
    char	*hosout, 
    struct	FRAME	*frame, 
    long	window[],
    long	linegrid, 
    long	sampgrid, 
    long	angflg, 
    long	lsbflg,
    int	fdesc[], 
    int	*gdesc, 
    char	**thebuf, 
    long	offset[],
    int		compflg
);

/* process.c */
int process
(
    int	*gdesc, 
    long	offset[], 
    char	*thebuf, 
    struct	FRAME	*frame, 
    struct	MODEL	*model, 
    long	angflg, 
    long	linegrid, 
    long	sampgrid, 
    long	*msgtim, 
    long	interpflg,
    long	lsbflg,
    struct	DDR	*ddr,
    long	window[],
    long	scaling,
    char	*hosin
);

/* update_ddr.c */
int update_ddr
(
    char	*hosname, 
    struct	DDR	*input_ddr,
    struct	FRAME	*frame, 
    long	angflg,
    long	lsflg,
    long	scaling
);
#endif
