/*****************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the input parameters from TAE

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    03/99	D.Lloyd		Initial development
  1.1    12/00  D.Lloyd		When using an ADDR for model information, use
  				the DDR window so that the samples are correct
				and the output image is windowed properly.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "getatm.h"
#include "archio.h"
#include "labserv.h"


int getpar
(
struct	PARBLK	*vblock,	/* IN - TAE variable block		*/
char	*hosin,			/* OUT - output host file name		*/
char	*hosout,		/* OUT - output host file name		*/
char	infiles[][CMLEN],	/* OUT - water vapor host file name	*/
long	*angflg,		/* OUT - generate angles bands flag	*/
long	*linegrid,		/* OUT - number lines for grid cell	*/
long	*sampgrid,		/* OUT - number samples in grid cell	*/
struct	FRAME	*frame,		/* OUT - output framing parameters	*/
struct	MODEL	**imgmodel,	/* OUT - AVHRR model structure		*/
long	*msgtim,		/* OUT - time between processing message*/
long	*interpflg,		/* OUT - interpolation flag, NN or BL	*/
struct	DDR	**imgddr,	/* OUT - image data descriptor struct	*/
long	*lsbflg,		/* OUT - line sample bands flag		*/
long	window[],		/* OUT - input image window values	*/
long	*scaling,		/* OUT - angles scaling flag		*/
int	*compflg		/* OUT - image compression flag		*/
)
{
char	dir[CMLEN];		/* host directory path name		*/
char	ext[CMLEN];		/* host file name extension		*/
char	hnames[2][CMLEN];	/* host file name specified in IN	*/
char	root[CMLEN];		/* host root file name			*/
char	temphos[CMLEN];		/* temporary host file name		*/
char	tnames[2][CMLEN];	/* tae file name specified in IN	*/

long	access;			/* file access indicator		*/
long	bands[2][MAXBND+1];	/* bands specified in IN		*/
long	element = 0;		/* element number for getnam call	*/
long	i = 0;			/* index into infile array		*/
long	imglimit = 2;		/* image limit for getin call		*/
long	mode = 0;		/* mode for lsstat, check for existence	*/
long	nbands[2];		/* number of bands specified in IN	*/
long	numimages;		/* number of images specified in IN	*/
long	qflag[2][2];		/* window band flags for getin call	*/
long	same_window;		/* same window flag for getin call	*/
long	status;			/* function return status		*/
long	totband;		/* total bands from IN specification	*/
long	wind[2][4];		/* window specification in IN		*/

struct	DDR	 ddr;		/* local data descriptor record		*/
struct	MODEL	 *model;	/* local AVHRR model structure		*/
struct	VARIABLE *var;		/* pointer to a TAE PARBLK		*/


/* get the input image
----------------------*/
status = c_getin( vblock, "IN", &element, tnames, hnames, &numimages,
	wind, bands, nbands, &totband, &imglimit, &same_window, qflag );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "getatm-fatal", "Fatal error encountered" );
strcpy( hosin, hnames[0] );

/* get the water vapor image
----------------------------*/
var = p_find( vblock, "WATER" );
if ( var->v_count == 0 )
    strcpy( frame->water.hostname, "" );
else
    {
    status = c_getnam( vblock, "WATER", &element, infiles[i], "img", &access );
    if ( status != E_SUCC )
	c_lasmsg( TRUE, "getatm-fatal", "Fatal error encountered" );
    strcpy( frame->water.hostname, infiles[i] );
    status = c_getddr( infiles[i], &ddr );
    if ( status != E_SUCC )
        return E_FAIL;
    if ( ddr.proj_code != 0 )
	{
        c_lasmsg( FALSE, "getatm-proj",
		"Water vapor projection code must be geographic" );
	return E_FAIL;
	}
    if ( strcmp( ddr.proj_units, "degrees" ))
        {
	c_lasmsg( FALSE, "getatm-units",
		"Water vapor projection units must be degrees" );
	return E_FAIL;
	}
    /* the floor() and ceil() functions are used here because the water
    vapor images are 1 degree resolution and we want the upper left
    and lower right lat/lon of the pixel not some other point within
    the pixel.
    -------------------------------------------------------------------*/
    frame->water.ullat  = ceil(ddr.upleft[0]);
    frame->water.ullon  = floor(ddr.upleft[1]);
    frame->water.lrlat  = floor(ddr.loright[0]);
    frame->water.lrlon  = ceil(ddr.loright[1]);
    frame->water.pdistx = ddr.pdist_x;
    frame->water.pdisty = ddr.pdist_y;
    frame->water.nl     = ddr.nl;
    frame->water.ns     = ddr.ns;
    }

/* get the ozone image
----------------------*/
var = p_find( vblock, "OZONE" );
if ( var->v_count == 0 )
    strcpy( frame->ozone.hostname, "" );
else
    {
    i++;
    status = c_getnam( vblock, "OZONE", &element, infiles[i], "img", &access );
    if ( status != E_SUCC )
	c_lasmsg( TRUE, "getatm-fatal", "Fatal error encountered" );
    strcpy( frame->ozone.hostname, infiles[i] );
    status = c_getddr( infiles[i], &ddr );
    if ( status != E_SUCC )
        return E_FAIL;
    if ( ddr.proj_code != 0 )
	{
        c_lasmsg( FALSE, "getatm-proj",
		"Ozone projection code must be geographic" );
	return E_FAIL;
	}
    if ( strcmp( ddr.proj_units, "degrees" ))
        {
	c_lasmsg( FALSE, "getatm-units",
		"Ozone projection units must be degrees" );
	return E_FAIL;
	}
    /* the floor() and ceil() functions are used here because the ozone
    images are 1 degree resolution and we want the upper left
    and lower right lat/lon of the pixel not some other point within
    the pixel.
    -------------------------------------------------------------------*/
    frame->ozone.ullat  = ceil(ddr.upleft[0]);
    frame->ozone.ullon  = floor(ddr.upleft[1]);
    frame->ozone.lrlat  = floor(ddr.loright[0]);
    frame->ozone.lrlon  = ceil(ddr.loright[1]);
    frame->ozone.pdistx = ddr.pdist_x;
    frame->ozone.pdisty = ddr.pdist_y;
    frame->ozone.nl     = ddr.nl;
    frame->ozone.ns     = ddr.ns;
    }

/* get the elevation image
--------------------------*/
var = p_find( vblock, "ELEV" );
if ( var->v_count == 0 )
    strcpy( frame->elev.hostname, "" );
else
    {
    i++;
    status = c_getnam( vblock, "ELEV", &element, infiles[i], "img", &access );
    if ( status != E_SUCC )
	c_lasmsg( TRUE, "getatm-fatal", "Fatal error encountered" );
    strcpy( frame->elev.hostname, infiles[i] );
    status = c_getddr( frame->elev.hostname, &ddr );
    if ( status != E_SUCC )
        return E_FAIL;
    if ( ddr.proj_code != 0 )
	{
        c_lasmsg( FALSE, "getatm-proj",
		"Elevation projection code must be geographic" );
	return E_FAIL;
	}
    if ( strcmp( ddr.proj_units, "degrees" ))
        {
	c_lasmsg( FALSE, "getatm-units",
		"Elevation projection units must be degrees" );
	return E_FAIL;
	}
    frame->elev.ullat  = ddr.upleft[0];
    frame->elev.ullon  = ddr.upleft[1];
    frame->elev.lrlat  = ddr.loright[0];
    frame->elev.lrlon  = ddr.loright[1];
    frame->elev.pdistx = ddr.pdist_x;
    frame->elev.pdisty = ddr.pdist_y;
    frame->elev.nl     = ddr.nl;
    frame->elev.ns     = ddr.ns;
    }

/* get the QC image
-------------------*/
var = p_find( vblock, "QC" );
if ( var->v_count == 0 )
    strcpy( frame->qc.hostname, "" );
else
    {
    i++;
    status = c_getnam( vblock, "QC", &element, infiles[i], "img", &access );
    if ( status != E_SUCC )
	c_lasmsg( TRUE, "getatm-fatal", "Fatal error encountered" );
    strcpy( frame->qc.hostname, infiles[i] );
    status = c_getddr( infiles[i], &ddr );
    if ( ddr.proj_code != 0 )
	{
        c_lasmsg( FALSE, "getatm-proj",
		"Water vapor qc projection code must be geographic" );
	return E_FAIL;
	}
    if ( status != E_SUCC )
        return E_FAIL;
    if ( strcmp( ddr.proj_units, "degrees" ))
        {
	c_lasmsg( FALSE, "getatm-units",
		"Water vapor qc projection units must be degrees" );
	return E_FAIL;
	}
    /* the floor() and ceil() functions are used here because the water
    vapor qc images are 1 degree resolution and we want the upper left
    and lower right lat/lon of the pixel not some other point within
    the pixel.
    -------------------------------------------------------------------*/
    frame->qc.ullat  = ceil(ddr.upleft[0]);
    frame->qc.ullon  = floor(ddr.upleft[1]);
    frame->qc.lrlat  = floor(ddr.loright[0]);
    frame->qc.lrlon  = ceil(ddr.loright[1]);
    frame->qc.pdistx = ddr.pdist_x;
    frame->qc.pdisty = ddr.pdist_y;
    frame->qc.nl     = ddr.nl;
    frame->qc.ns     = ddr.ns;
    }

/* NULL terminate the infiles list
----------------------------------*/
infiles[++i][0] = 0;


/* get the output image
-----------------------*/
access = IWRITE;
status = c_getnam( vblock, "OUT", &element, hosout, "img", &access );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "getatm-fatal", "Fatal error encountered" );

/* get the angles bands flag
----------------------------*/
var = p_find( vblock, "ANGFLG" );
if ( strcmp( SVAL( *var, 0 ), "YES" ) == 0 )
    *angflg = TRUE;
else
    *angflg = FALSE;

/* get the grid size
--------------------*/
var = p_find( vblock, "LINEGRID" );
*linegrid = IVAL( *var, 0 );
var = p_find( vblock, "SAMPGRID" );
*sampgrid = IVAL( *var, 0 );

/* retrieve the AVHRR model information from the associated ADDR file
or, in the absence of the ADDR, get the DDR and check its validity.
---------------------------------------------------------------------*/
model = (struct MODEL *)malloc( sizeof(struct MODEL));
if ( model == NULL )
    {
    c_lasmsg(FALSE,"getatm-alloc","Error allocating memory");
    return E_FAIL;
    }
c_splitname( hosin, dir, root, ext );
strcpy( temphos, dir );
strcat( temphos, root );
strcat( temphos, ".addr");
if ( c_lsstat( temphos, &mode ) == E_SUCC )
    {
    if ( c_getaddr( hosin, model ) != E_SUCC )
        {
	c_lasmsg(FALSE,"getatm-getaddr","Error reading ADDR");
	free( model );
	return E_FAIL;
	}
    *imgmodel = model;

    /* Note: The following lines that contain '???' will need to
    be modified, '???' removed, and comments removed once it is
    determined what the output ML and MS of AVHRRIN will be.
    That is: will they be 0 or 1 relative.
    -----------------------------------------------------------*/
    /* Since AVHRRIN can window the image, use the DDR instead of
    the model to set the window.  The ML must be 0 (or 1 ???) or
    else an error occurred in AVHRRIN windmod() so error out.
    Set the starting sample to the DDR MS + 1 ( or no +1 ??? )
    and the number samples to DDR NS.
    ------------------------------------------------------------*/
    status = c_getddr( hosin, &ddr );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"getatm-getddr","Error reading image DDR");
	return E_FAIL;
	}
    if ( ddr.master_line != 0 ) /*???*/
        {
	c_lasmsg(FALSE,"getatm-mline","DDR master line not equal to 0");/*???*/
	return E_FAIL;
	}
    window[SL] = 1;
    window[SS] = ddr.master_sample + 1;	/*??? or no plus one */
    window[NL] = ddr.nl;
    window[NS] = ddr.ns;
    }
else
    {
    c_lasmsg(FALSE,"","ADDR does not exist, using DDR");
    free( model );
    model = NULL;
    /* try to get a DDR */
    *imgddr = (struct DDR *)malloc( sizeof(struct DDR));
    if ( *imgddr == NULL )
	{
	c_lasmsg(FALSE,"getatm-alloc","Error allocating memory");
	return E_FAIL;
	}
    status = c_getddr( hosin, *imgddr );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"getatm-getddr","Error reading image DDR");
	return E_FAIL;
	}
    /* make sure we have a good DDR here
    Check for projection parameters etc */
    if (( (*imgddr)->valid[0] == INVAL ) ||	/* projection code	*/
        ( (*imgddr)->valid[3] == INVAL ) ||	/* projection coeffs	*/
        ( (*imgddr)->valid[4] == INVAL ))	/* projection units	*/
	{
	c_lasmsg(FALSE,"getatm-noproj",
		"Projection information not available from DDR");
	return E_FAIL;
	}
    
    window[SL] = window[SS] = 1;
    window[NL] = (*imgddr)->nl;
    window[NS] = (*imgddr)->ns;
    }

/* issue a warning and reset the angles flag if
there isn't a model and angles are desired.
-----------------------------------------------*/
if ( *angflg && !*imgmodel )
    {
    *angflg = FALSE;
    c_lasmsg(FALSE,"getatm-angles",
        "Can't generate angles without a MODEL resetting ANGFLG to NO");
    }

/* get the message time
-----------------------*/
var = p_find( vblock, "$MSGTIM" );
*msgtim = IVAL( *var, 0 );

/* get the interpolation flag
-----------------------------*/
var = p_find( vblock, "INTERPFLG" );
if (strcmp( SVAL( *var, 0 ), "BL" ) == 0 )
    *interpflg = BL;
else
    *interpflg = NN;

/* get the line/sample band flag
--------------------------------*/
var = p_find( vblock, "LSBFLG" );
if (strcmp( SVAL( *var, 0 ), "YES" ) == 0 )
    *lsbflg = TRUE;
else
    *lsbflg = FALSE;

/* get the angles scaling flag
------------------------------*/
var = p_find( vblock, "ANGSCALE" );
if (strcmp( SVAL( *var, 0 ), "GLOBAL" ) == 0 )
    *scaling = GLOBAL_SCALE;
else if (strcmp( SVAL( *var, 0 ), "US" ) == 0 )
    *scaling = US_SCALE;
else
    *scaling = NO_SCALING;

/* set the compression flag
---------------------------*/
var = p_find( vblock, "$COMPFLG" );
if (strcmp( SVAL( *var, 0 ), "ON" ) == 0 )
    *compflg = TRUE;
else
    *compflg = FALSE;

return E_SUCC;
}

