/*****************************************************************************
FUNCTION:	INTERP

PURPOSE:	Do a bi-linear interpolation between four grid points.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    05/99	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS

REFERENCE:	Press, Teukolsky, Vettering, and Flannery, 
		"Numerical Recipes in C", 2nd Ed. 1992, pp 123-125
*****************************************************************************/
#include "getatm.h"



float interp
(
    long        y0,	/* IN - location in the line direction          */
    long        y1,	/* IN - location in the line direction          */
    long        y2,	/* IN - location in the line direction          */
    long        x0,	/* IN - location in the samples direction       */
    long        x1,	/* IN - location in the samples direction       */
    long        x2,	/* IN - location in the samples direction       */
    float       v1,	/* IN - interpolation values                    */
    float       v2,	/* IN - interpolation values                    */
    float       v3,	/* IN - interpolation values                    */
    float       v4,	/* IN - interpolation values                    */
    float       *v0	/* OUT - interpolated value                     */
)
{
float	t, u;		/* intermediate results				*/
float	v;		/* interpolated value				*/

t = (float)( x0 - x1 ) / (float)( x2 - x1 );
u = (float)( y0 - y1 ) / (float)( y2 - y1 );

v = ( 1.0 - t ) * ( 1.0 - u ) * v1;
v += t * ( 1.0 - u ) * v2;
v += ( 1.0 - t ) * u * v3;
v += t * u * v4;

*v0 = v;
return v;
}


/***************************************************************************
FUNCTION:	NN

PURPOSE:	Do a nearest neighbor interpolation of four grid points
****************************************************************************/
float nn
(
    long        y0,	/* IN - location in the line direction          */
    long        y1,	/* IN - location in the line direction          */
    long        y2,	/* IN - location in the line direction          */
    long        x0,	/* IN - location in the samples direction       */
    long        x1,	/* IN - location in the samples direction       */
    long        x2,	/* IN - location in the samples direction       */
    float       v1,	/* IN - interpolation values                    */
    float       v2,	/* IN - interpolation values                    */
    float       v3,	/* IN - interpolation values                    */
    float       v4,	/* IN - interpolation values                    */
    float       *v0	/* OUT - interpolated value                     */
)

{
float	x,y,v;

if ( x0 - x1 < x2 - x0 )
    {
    x = v1;
    y = v3;
    }
else
    {
    x = v2;
    y = v4;
    }

if ( y0 - y1 < y2 - y0 )
    *v0 = x;
else
    *v0 = y;
v = *v0;
return v;
}

/****************************************************************************
FUNCTION:	NVN

PURPOSE:	Do a nearest valid neighbor interpolation of four grid points.
		Invalid values, less than or equal to zero, are not used.  If
		there are no valid values a negative one is returned.

AUTHOR:		Dave Lloyd, Sep 15, 1999

ALGORITHM:

    Determine which quadrant the pixel is in
    Set up the value array depending on the quadrant as follows:
        Q1 val = { V1, V2, V3, V4 }
	Q2 val = { V2, V4, V1, V3 }
	Q3 val = { V3, V4, V1, V2 }
	Q4 val = { V4, V2, V3, V1 }
    if val[0] is valid return it
    return the first valid value in the array unless both val[1]
        and val[2] are valid
    if val[1] and val[2] are valid
        Determine distance to correct diagonal.  For Q1 and Q4 the
	    diagonal is from upper left to lower right and for
	    Q2 and Q3 it is from upper right to lower left
	if distance to pixel is greater than distance to diagonal
	    return val[1]
	else
	    return val[2]
    return an invalid value


   V1                   V2
   -----------------------
   |\         |   *      /|
   |  \       |        /  |
   |   Q1     |     Q2    |
   |     \    |    /      |
   |       \  |  /     @  |
   |         \|/          |
   -----------------------
   |         /|\          |
   |       /  |  \        |
   |     /    |    \      |
   |   Q3     |     Q4    |
   |  /       |       \   |
   |/         |         \ |
   -----------------------
   V3                   V4

Example:  Pixel, *, above in quadrant 2 (Q2) and the value, V2,
for Q2 is invalid.

Case 1: If V1 is valid and V4 is invalid return V1
Case 2: If V1 is invalid and V4 is valid return V4
Case 3: If V1 and V4 are invalid and V3 is valid return V3
Case 4: If V1 and V4 are valid return V1 since pixel is to the left
        of the Q2 diagonal and is in the space adjacent to V1.
        Note: if the pixel was @, V4 would be returned.
Case 5: All quadrants invalid return invalid value
****************************************************************************/
float nvn
(
long	l0,		/* IN - interpolate point line location		*/
long	l1,		/* IN - upper left line location		*/
long	l2,		/* IN - lower right line location		*/
long	s0,		/* IN - interpolate point sample location	*/
long	s1,		/* IN - upper left sample location		*/
long	s2,		/* IN - lower right sample location		*/
float	v1,		/* IN - Q1 interpolation value			*/
float	v2,		/* IN - Q2 interpolation value			*/
float	v3,		/* IN - Q3 interpolation value			*/
float	v4		/* IN - Q4 interpolation value			*/
)
{
float	val[4];		/* array of ordered interpolation values	*/
int	quadrant;	/* quadrant number				*/
int	del_l;		/* delta line					*/
int	del_s;		/* delta sample					*/



/* Determine the quadrant number for the pixel.
----------------------------------------------*/
if ( s0 - s1 < s2 - s0 )	/* The pixel is on the left side	*/
    quadrant = 1;		/* indicate that Q1 is the primary	*/
else				/* the pixel is on the right side	*/
    quadrant = 2;		/* indicate that Q2 is the primary	*/

if ( l0 - l1 >= l2 - l0 )	/* the pixel is in the bottom half	*/
    quadrant += 2;		/* increment Q1 to Q3 or Q2 to Q4	*/
/* otherwise */			/* the pixel is in the bottom half and	*/
				/* the quadrant is properly numbered	*/


/* Set up the value array.  This array is set up so that the
first "valid" value in the array is the correct one to return
with the exception of Case 4. (see comment and diagram above)
-------------------------------------------------------------*/
switch ( quadrant )
    {
    case 1 :
        val[0] = v1;
	val[1] = v2;
	val[2] = v3;
	val[3] = v4;
	break;
    case 2 :
        val[0] = v2;
	val[1] = v4;
	val[2] = v1;
	val[3] = v3;
	break;
    case 3 :
        val[0] = v3;
	val[1] = v4;
	val[2] = v1;
	val[3] = v2;
	break;
    case 4 :
        val[0] = v4;
	val[1] = v2;
	val[2] = v3;
	val[3] = v1;
	break;
    }


/* If the value chosen is valid, return it.
-------------------------------------------*/
if ( val[0] > 0.0 )
    return val[0];


/* Otherwise decide which, if any, of the remaining
values are valid and return that value. See diagram
and comments above.

Case 1 and 2: If either of the adjacent quadrants
are valid but not both return the valid value.
----------------------------------------------------*/
if ( val[1] > 0.0 && val[2] <= 0.0 )
    return val[1];
if ( val[2] > 0.0 && val[1] <= 0.0 )
    return val[2];


/* Case 3: Otherwise if both adjacent quadrants are invalid
and the opposite quadrant is valid return the valid value
-----------------------------------------------------------*/
if ( val[1] <= 0.0 && val[2] <= 0.0 && val[3] > 0.0 )
    return val[3];


/* Case 4: If both adjacent quadrants are valid then we must decide on what
side of the diagonal the pixel lies and return the value for the quadrant
nearest the pixel.  To do this measure the distance from the upper left
corner to the pixel and to the diagonal in the pixels quadrant.  If the
pixel distance is greater the pixel is to the right of the quadrant diagonal
so we return val[1] otherwise return val[2].  Note: the distances are simple
linear measures, line + sample.
---------------------------------------------------------------------------*/
if ( val[1] > 0.0 && val[2] > 0.0 )
    {
    /* determine the change in line and sample, they may be different */
    del_s = s2 - s1;
    del_l = l2 - l1;
    switch ( quadrant )
        {
	case 1 :
	case 4 :
	    if (( l0 + s0 ) > 		/* compare distances */
	       (l1 + s1 + (int)((l0 - l1) * ( 1.0 + (float)del_s / del_l ))))
	        return val[1];
	    else
	        return val[2];

	case 2 :
	case 3 :
            if (( l0 + s0 ) >		/* compare distances */
	       (l1 + s1 + del_s +
	       (int)((l0 - l1) * ( 1.0 - (float)del_s / del_l ))))
	        return val[1];
	    else
	        return val[2];
	}
    }


/* Case 5: If nothing else matched above all four points
contain invalid values so return an invalid value any one
will work.
--------------------------------------------------------*/
return val[0];
}
