/*****************************************************************************
FUNCTION:       LOADLL

PURPOSE:        Load the lat long buffer and, if specified the viewing angles.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    04/99  D.Lloyd         Initial development
  1.1    12/00  D.Lloyd		Modified to use the window samples values so
  				that the output image is the same as the
				input image.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS
*****************************************************************************/
#include "getatm.h"


void loadll
(
long	curline,	/* IN - current line number			*/
long	sampgrid,	/* IN - sample grid size			*/
long	angflg,		/* IN - generate the angles bands		*/
struct	MODEL	*model,	/* IN - AVHRR model structure			*/
struct	DDR	*ddr,	/* IN - input image ddr structure		*/
long	proj_units,	/* IN - projection units value			*/
long	window[],	/* IN - input image window			*/
long	scaling,	/* IN - scale flag: US_SCALE GLOBAL_SCALE NONE	*/
struct	SCOFF scoff[],	/* IN - scaling coefficient structure		*/
float	*latbuf,	/* OUT - latitude values			*/
float	*lonbuf,	/* OUT - longitude values			*/
float	*satzebuf,	/* OUT - satellite zenith angles		*/
float	*solzebuf,	/* OUT - solar zenith angles			*/
float	*relazbuf,	/* OUT - relative azimuth angles		*/
double	rot_coeffs[]	/* IN - rotation coefficient array		*/
)

{
double	lat;		/* latitude for one pixel			*/
double	lon;		/* longitude for one pixel			*/
double	*ptr = NULL;	/* pointer to angles array for ls2ll()		*/
double	angles[3];	/* angles array					*/
double	elev = 0.0;	/* elevation for ls2ll() call			*/
double	proj_x;		/* projection coordinate of a grid point	*/
double	proj_y;		/* projection coordinate of a grid point	*/

float	line;		/* current line number				*/
float	samp;		/* current sample number			*/

long	degrees = 4;	/* units code for geographic projection(degrees)*/
long	geo = 0;	/* geographic projection flag			*/
long	i;		/* loop index					*/
long	nadir;		/* nadir pixel index				*/
long	numgrids;	/* number of sample grids			*/


/* Determine how many grid points there are
-------------------------------------------*/
numgrids = window[NS] / sampgrid;

/* If we need to calculate angles, give ptr something
to point to so c_ls2ll() will calculate the angles.
-----------------------------------------------------*/
if ( angflg )
    {
    ptr = angles;
    nadir = numgrids / 2;
    }

/* On the current line, for each of the grid points,
get the latitude, longitude and angles
----------------------------------------------------*/
line = (float)curline;
for ( i = 0; i < numgrids + 1; i++ )
    {
    /* compute a sample index */
    samp = (float)( i * sampgrid + window[SS] );

    /* make sure we don't go beyond the end of the line which can
    happen if the number of samples don't divide the line evenly
    -------------------------------------------------------------*/
    if ( samp > (float)( window[SS] + window[NS] - 1 ))
        samp = (float)( window[SS] + window[NS] - 1 );
    
    /* get the lat/lon and angles */
    if ( model )
	{
	c_ls2ll( &line, &samp, &elev, model, &lat, &lon, ptr );
	/* store the lat/lon */
	latbuf[i] = DEG_PER_RAD * lat;
	lonbuf[i] = DEG_PER_RAD * lon;
	}
    else
        {
	/* Determine the projection coordinates taking
	into account any rotation of the image.
	-----------------------------------------------*/
	proj_y = ( rot_coeffs[0] * (double)line ) +
	         ( rot_coeffs[1] * (double)samp ) + rot_coeffs[2];
	proj_x = ( rot_coeffs[3] * (double)line ) +
	         ( rot_coeffs[4] * (double)samp ) + rot_coeffs[5];

	/* Compute the latitude and longitude of the location. 
	If c_trans does not return E_SUCC we assume we are
	in a invalid region such as an interrupted Goode region
	so set the lat lon to invalid values.
	-------------------------------------------------------*/
	if ( c_trans( &ddr->proj_code, &proj_units, &geo, &degrees,
	              &proj_x, &proj_y, &lon, &lat ) != E_SUCC )
	    lat = lon = -999.0;
	latbuf[i] = lat;
	lonbuf[i] = lon;
	}

    /* store the angles */
    if ( angflg )
        {
	/* Satellite zenith angles are negative from the first
	scanned pixel to nadir and positive after nadir.
	Note: Added for clarification 09/14/2001 DEL.  The above
	statement is correct as well as the logic below.  Since
	the image is not in satellite view, all line sample values
	are in image space not in satellite space.  The
	variable i below is an index into a grid varying from
	0 to numgrids and moving from left to right in the LAS
	image.  For an ASCENDING pass the image was actually
	scanned right to left which is why ( i > nadir ) is
	correct.  For a DESCENDING pass the image was scanned
	in the same direction as i is moving and therefore
	( i <= nadir ) is correct.
	------------------------------------------------------*/
	satzebuf[i] = angles[0] * DEG_PER_RAD;
	if ((( model->cpass == ASCENDING )  && ( i >  nadir )) ||
	    (( model->cpass == DESCENDING ) && ( i <= nadir )))
	    satzebuf[i] *= -1.0;

	/* Solar zenith is okay as is.
	------------------------------*/
	solzebuf[i] = angles[1] * DEG_PER_RAD;

	/* relative azimuth is defined as -180 to +180
	so make sure it's in that range.
	-----------------------------------------------*/
	relazbuf[i] = angles[2] * DEG_PER_RAD - 180.0;
	
	/* Scale the angles if specified.  For US scaling
	relative azimuth is defined as 0 to 180.
	-------------------------------------------------*/
	switch ( scaling )
	    {
	    case US_SCALE :
		relazbuf[i] = fabs( relazbuf[i] );
	    case GLOBAL_SCALE :
		satzebuf[i] = SCALE( satzebuf[i], scoff[SATZEN].minval,
		    scoff[SATZEN].maxval, scoff[SATZEN].scale,
		    scoff[SATZEN].offset );
		solzebuf[i] = SCALE( solzebuf[i], scoff[SOLZEN].minval,
		    scoff[SOLZEN].maxval, scoff[SOLZEN].scale,
		    scoff[SOLZEN].offset );
		relazbuf[i] = SCALE( relazbuf[i], scoff[RELAZ].minval,
		    scoff[RELAZ].maxval, scoff[RELAZ].scale,
		    scoff[RELAZ].offset );
	    case NO_SCALE :
	        break;
	    }
	}
    }

/* Duplicate the last values for interpolation routine
------------------------------------------------------*/
latbuf[numgrids+1] = latbuf[numgrids];
lonbuf[numgrids+1] = lonbuf[numgrids];
if ( angflg )
    {
    satzebuf[numgrids+1] = satzebuf[numgrids];
    solzebuf[numgrids+1] = solzebuf[numgrids];
    relazbuf[numgrids+1] = relazbuf[numgrids];
    }
}
