/******************************************************************************
FUNCTION:	GETREGIONS

PURPOSE:	Find the region numbers which lie within a given image
		and return them to TAE land.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       12/93    T. Baltzer		initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include "las.h"		/* LAS include			       */
#include "util.h"		/* LAS include			       */
#include "adaps.h"		/* ADAPS include		       */
#include "getregions.h"

main()
{
struct PARBLK   vblock;		/* TAE parameter block			*/

char hosin[1][CMLEN];		/* Host name of input image		*/
char *sptr[1];                  /* pointer array for updating vblock    */
char regions[28];		/* Regions which lie within image	*/
char tmpstr[3];			/* Temporary string			*/

long linenum;			/* Line number under consideration	*/
long i;				/* Counter				*/
long nlines;			/* Number of lines between evaluations	*/
long nonfatal = 1;              /* Non-fatal error status               */
long regflg[12];		/* Region flag: 0=Not found, 1=Found	*/
long sampnum;			/* Sample number under consideration	*/
long status;			/* Return status			*/
long std_out;			/* standard output device 		*/

float flt_line;			/* Floating point version of line 	*/
float flt_samp;			/* Floating point version of sample	*/

double lat,			/* Latitude of line/sample		*/
       lng,			/* Longitude of line/sample		*/
       elev = 0.0;		/* Elevation				*/

struct MODEL model;		/* AVHRR model information		*/

/* The 12 regions are defined as follows (degrees are easier to read):

region number   min latitude   max latitude   min longitude   max longitude
1		40.73333	90.		-180.		-40.
2		40.73333	90.		-40.		180.
3		0. 		40.73333	-180.		-40.
4		0.		40.73333	-40.		180.
5		-40.73333	0.		-180.		-100.
6		-40.73333	0.		-100.		-20.
7		-40.73333	0.		-20.		80.
8		-40.73333	0.		80.		180.
9		-90.		-40.73333	-180.		-100.
10		-90.		-40.73333	-100.		-20.
11		-90.		-40.73333	-20.		80.
12		-90.		-40.73333	80.		180.
*/
static double reg[12][4] =
	{
                0.7109880,      1.570796,       -3.141593,      -0.7109880,
                0.7109880,      1.570796,       -0.7109880,     3.141593,
                0.0,            0.7109880,      -3.141593,      -0.7109880,
                0.0,            0.7109880,      -0.7109880,     3.141593,
                -0.7109880,     0.0,            -3.141593,      -1.745329,
                -0.7109880,     0.0,            -1.745329,      -0.349066,
                -0.7109880,     0.0,            -0.349066,      1.396263,
                -0.7109880,     0.0,            1.396263,       3.141593,
                -1.570796,      -0.7109880,     -3.141593,      -1.745329,
                -1.570796,      -0.7109880,     -1.745329,      -0.349066,
                -1.570796,      -0.7109880,     -0.349066,      1.396263,
                -1.570796,      -0.7109880,     1.396263,       3.141593
	};

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);


/* Get the input parameters
----------------------------*/
status = getpar(&vblock,hosin,&model,&nlines);
if (status != E_SUCC)
   z_exit(-1,"");


/* Initialize the regions to all "Not found"
---------------------------------------------*/
for (i = 0; i < 12; i++)
   regflg[i] = FALSE;


/* To determine the regions which lie within the image,
   select pixels at sample 1, sample n/2, and sample n
   where n is the number of samples in the image.  These
   columns represent leftmost, nadir, and rightmost values.
   The pixel line numbers will start at 1 and increment
   at the user specified value.  The final line number
   will be used even if it does not fall on the increment.
     Given these pixel values, evaluate their location to
   determine which of the 12 regions on earth that they
   fall into.
*/
for (i = 0; i < 3; i++)
   {
   switch (i)
      {
      case 0: sampnum = 1;
	      break;
      case 1: sampnum = model.nsamples / 2;
	      break;
      case 2: sampnum = model.nsamples;
	      break;
      }
   for (linenum = 1; linenum <= model.nlines; linenum += nlines)
      {
      flt_line = (float)linenum;
      flt_samp = (float)sampnum;
      c_ls2ll(&flt_line,&flt_samp,&elev,&model,&lat,&lng,NULL);
      evalreg(reg,lat,lng,regflg);
      }
   if (linenum - nlines != model.nlines)
      {
      linenum = model.nlines;
      flt_line = (float)linenum;
      flt_samp = (float)sampnum;
      c_ls2ll(&flt_line,&flt_samp,&elev,&model,&lat,&lng,NULL);
      evalreg(reg,lat,lng,regflg);
      }
   }


/*
  Load string variable of regions to return to tae land
*/
regions[0] = '\0';
for (i = 0; i < 12; i++)
   {
   if (regflg[i] == TRUE)
      {
      sprintf(tmpstr,"%d ",i+1);
      strcat(regions,tmpstr);
      }
   }


/*
  Get rid of the last blank space in string
*/
if (strcmp(regions,"") != 0)
   regions[strlen(regions)-1] = '\0';


/*
  Output the values to the variable block
*/
sptr[0] = regions;
status = q_string(&vblock,"REGIONS",1,(const char **)sptr,P_UPDATE);
if (status != SUCCESS)
   {
   c_errmsg("Error sending variable to TAE", "getregions-dtae",&nonfatal);
   c_errmsg("Fatal error encountered","getregions-fatal",&nonfatal);
   z_exit(-1,"");
   }
status = q_out(&vblock);
if (status != SUCCESS)
   {
   c_errmsg("Error sending variable to TAE", "getregions-dtae",&nonfatal);
   c_errmsg("Fatal error encountered","getregions-fatal",&nonfatal);
   z_exit(-1,"");
   }


/*
  Exit with a success status without displaying a completion message
*/
z_exit(1,"");
return E_SUCC;
}
