/*******************************************************************************
FUNCTION:	check_time

PURPOSE:	Check the validity of the time stamp code from the current
		line compared to the old time code.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/93  J.Fenno          Modified calls to the new archio supports
  2.2    12/99  D.Lloyd          Corrected error where drop lines were
				 allowed to end a scene
  2.3    08/00  D.Lloyd          Removed code that caused an infinite loop
  				 when a zero time stamp occurred after a group
				 of out of sequence time stamps. CR 4020


ALGORITHM REFERENCE:
  See "NOAA Technical Memorandum NESS 107" for a detailed description
  of the HRPT Minor Frame Format.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "ingest.h"
#include "worgen.h"
#include "avhrr.h"

extern int acqskip( long *, long);

int check_time
(
long	*acq_fd,	      /* File descriptor to the acquire file	     */
long	*dtype,		      /* Type of data (LAC, GAC, HRPT)		     */
long	*milsec,	      /* Time in milliseconds of current line	     */
long	*milold,	      /* Time in milliseconds of previous line	     */
long	*missing,	      /* Number of missing lines in a gap	     */
long	*decline	      /* Number of lines to decrease input by	     */
)
{
float   min_time;             /* 1/2 a line time interval                    */
long	rdstat,		      /* Status of the acquire read		     */
	milnew,		      /* Time in milliseconds of next line	     */
	misnew,		      /* # of missing lines between old and new line */
	nrecs,		      /* Number of records read ahead		     */
	j,		      /* Looping variables			     */
	same_time = FALSE,    /* Same time stamp code flag		     */
	zero_line = FALSE,    /* Zero dropped line indicator		     */
	new_time = FALSE,     /* New time needed flag			     */
	missav[CONSEC],	      /* Save the missing line count between lines   */
	count = 1;	      /* Consecutive non-missed line counter         */

static short mnrbuf[MNR_SIZ], /* HRPT minor frame buffer		     */
	     tipbuf[TIP_SIZ], /* Tip data buffer			     */
	     imgbuf[LAC_SIZ]; /* Image data buffer			     */

/* Zero this out */
for (j = 0; j < CONSEC; j++)
     missav[j] = 0;
/*
  Calculate the number of missed lines, and 1/2 a line time interval
*/
if (*dtype == GAC)
    {
    min_time = MIL_PER_SEC / GAC_PER_SEC / 2.0;
    *missing = (long)((*milsec - *milold)/(MIL_PER_SEC/GAC_PER_SEC) + .5 - 1);
    }
else
    {
    min_time = MIL_PER_SEC / LAC_PER_SEC / 2.0;
    *missing = (long)((*milsec - *milold)/(MIL_PER_SEC/LAC_PER_SEC) + .5 - 1);
    }

/*
  Determine if a duplicate record was found
*/
if (*milsec == *milold)
    {
    same_time = TRUE;
    *missing = 1;
    }

/*
  Determine if the current line is a zero filled dropped line
*/
if (*milsec == 0)
    {
    zero_line = TRUE;
    *missing = 1;
    }

/*
  Negative number of missing lines is impossible so the time stamp is bad
  Reset the number of missing lines to one
*/
if (*missing < 0)
    *missing = 1;

/* DEL added for CR 1554
If the difference between milsec and milold is small or negative,
we don't want to return E_SUCC automatically.  Mark the line as a
candidate for fixing the time stamp and check the next three lines.
This can easily happen if there is a one or two bit error in the 
time code.  If the next three lines are good and fixing the time
stamp on this line would put it in order we can fix it.
Setting missing to 1 forces us into the if statement following the
next one.
-------------------------------------------------------------------*/
if (( *milsec != 0 ) && (float)( *milsec - *milold ) < min_time )
    {
    new_time++;
    *missing = 1;
    }

/*
  When missing or duplicate lines were found try to determine if it was due
  to a bad time stamp or is there were really bad lines.
*/
if ((*missing != 0) || (same_time))
    {
    /*
      Save the initial number of lines missed
    */
    missav[0] = *missing;

    /*
      Loop on the Maximum number of bad lines allowed to determine if
      there are dropped lines or just something wrong with the time stamp
    */
    for (nrecs = 1;;nrecs++)
	{
	/*
	  Read the next line from the acqusition file
	*/
        rdstat = c_acqread(acq_fd,mnrbuf,tipbuf,imgbuf);
	if ((rdstat == E_EOF) || (rdstat == E_BOF) || (rdstat == E_SMAL))
	    {
	    if (nrecs > 1)
		c_lasmsg(FALSE, "ingest-warn", "EOF encountered before a %s",
		    "bad time stamp could be resolved");

	    return(E_EOF);
	    }
	else if (rdstat != E_SUCC)
	    return(E_FAIL);

	/*
	  Get the next time stamp code
	*/
	milnew = mnrbuf[9] & 0x7f;
        milnew = (milnew << 10) | (mnrbuf[10] & 0x3ff);
        milnew = (milnew << 10) | (mnrbuf[11] & 0x3ff);
	/*
	  Calculate the number of missed lines between the oldest and
	  the newest lines
	*/
	if (*dtype == GAC)
	    misnew = (long)((milnew - *milold) /
	                    (MIL_PER_SEC / GAC_PER_SEC) + 0.5 - 1.0);
	else
	    misnew = (long)((milnew - *milold) /
	                    (MIL_PER_SEC / LAC_PER_SEC) + 0.5 - 1.0);

	/*
	  Check for duplicate records and treat them as a missing line.
	*/
	if (milnew == *milold)
	    {
	    *missing += 1;
	    continue;
	    }

	/*
	  Set a flag indicating that the next time stamp looks good.
	  The bad time stamp needs to be corrected before a zero drop
          line is processed.
	*/ 
	if (misnew == nrecs)
	    new_time++;

	/* DEL added for CR 1555
	If we get a time stamp that is not zero after looping through
	one or more zero time stamps, reset the zero_line flag so we can
	look for three consecutive valid time stamps.  This is not
	important unless there were more drop lines between the current
	line and the last good line.  If this is the case, the next N
	lines will be dropped (where N is the difference between the
	actual drop lines and the number that should have been there).
	If there are less than N consecutive good lines in the image,
	the archive image will contain drop lines to the end of the
	image.
	----------------------------------------------------------------*/
	if ( milnew != 0 && zero_line )
	    zero_line = FALSE;


	/*
	  Duplicate records were found and there were no missing lines
	  between the old line and the newest good line.
	*/
	if (((same_time) && (misnew == *missing)) || (zero_line))
	    {

	    /*
	      Skip back one record so that this good record will be re-read
	    */
	    acqskip(acq_fd,-1);


	    if ((zero_line) && (misnew == 0))
		{
		*decline = *missing;
		return(NONE_MISSED);
		}

	    /*
	      Duplicate data is invalid so treat them as missing lines and
	      fill the gap with zeros
	    */
	    return(MISSING_TIME);
	    }

	/*
	  Check if there were problems with just the time codes
	*/
	if ((misnew < 0) || (misnew > 5400))
	    {
	    /*
	      Reset the consecutive line counter back to one because
	      a bad time code was found
	    */
	    count = 1;

	    continue;
	    }
	else	
	    {
	    /*
	      Move the number of lines missed up in the missing line array
	    */
	    for (j = CONSEC; j > 1; j--)
		missav[j-1] = missav[j-2];

	    /*
	      Save the most current number of missing lines
	    */
	    missav[0] = misnew;

	    /*
	      Keep track of the number of consecutive lines that look to
	      have a time stamp code that has come back into order.  The
	      first one is always in sequence so count is set to one.
	    */
	    if ((missav[0] - missav[1]) == 1)
		count++;
	    else
		count = 1;

	    /*
	      Check to determine if MAX_BAD consecutive lines had good
	      time stamps
	    */
	    if (count == CONSEC)
		{
		/*
		  The time codes must be back into order and there must
		  be missing lines.  Reset the number of missing lines
		  to the start of the good time stamps
		*/
		*missing = missav[CONSEC-1];

		if (*missing == (nrecs - CONSEC) + 1)
		    {
		    /*
		      Reset the file pointer back an extra record so that the
		      current line is re-read and the time stamp is fixed
		    */
		    nrecs = -(nrecs + 1);
		    acqskip(acq_fd,nrecs);

		    /*
		      Return a status that will indicate the there are lines
		      with bad time stamps but the data is probably ok.  The
		      time stamps should be corrected and the data written to
		      the output archive file.
		    */
		    return(NEW_TIME);
		    }
		else
		    {
		    /*
		      Reset the file pointer back four records so that the
		      current line is re-read and the dropped lines are added.
		    */
		    acqskip(acq_fd,-CONSEC);
                    /*
                      Check for the case where extra lines have been found
                      that don't need to be there according to the time
                      stamp code.  Also, lines may need to be added based
		      on the time stamp (decline will be negative).
                    */
                    *decline = nrecs - CONSEC - *missing + 1;

		    /*
		      Return a status that will indicate the there are lines
		      that were missed and should be written to the output
		      archive file as zero lines.
		    */
		    return(MISSING_TIME);
		    }
		}
	    }
	}
    }

return(E_SUCC);
}
