/*******************************************************************************
FUNCTION:	first_rec

PURPOSE:	Use the julian date from the records to determine the 
		first valid record within the acquired image.  Check
		at least four lines of data before a valid date/time
		is returned.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    05/91  D.Hollaren       Conversion to LAS5.0
  2.1    07/92  T.Baltzer        Check time difference between two lines
  2.2    04/93  J.Fenno          Modified calls to the new archio supports
  2.3    03/98  D.Lloyd          Added check and error message for julian
                                 day of 366 on a non-leap year
  2.4	 07/98	D.Lloyd		 Added a check for NOAA 15 satellite id
  2.5    10/00  D.Lloyd		 Added a check for NOAA 16 satellite id
  2.6    04/02  D.Lloyd		 Added a check for NOAA 17 satellite id
  2.7    11/04  D.Lloyd		 Added a check for NOAA 18 satellite id


ALGORITHM REFERENCE:
  See "NOAA Technical Memorandum NESS 107" for a detailed description
  of the HRPT Minor Frame Format.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "ingest.h"
#include "util.h"
#include "worgen.h"
#include "avhrr.h"
#include "adapsutil.h"

extern int acqskip( long *, long);

int first_rec
(
long	*acq_fd,		/* File descriptor to the acquire file	     */
struct	SATPASS *pass		/* Model and orbital information	     */
)
{
struct tm       *lctm;          /* Local time structure                      */

long	all_days,		/* total of all the julian dates for ave.    */
	ave_days,		/* Average of all the julian dates	     */
	cday,			/* Current julian date			     */
	lday,			/* Julian date of the current line	     */
	first_line,		/* First valid line found flag		     */
	i,			/* Looping variable			     */
	jday[MAX_BAD],		/* Julian dates for each line		     */
	milsec[MAX_BAD],	/* Time code (milliseconds) of each line     */
	milfst,			/* Time code (milliseconds) for EAF fist line*/
	satidx,			/* Acquisition satellite id		     */
	satidx_buf[MAX_BAD],	/* Buffer containing the last MAX_BAD sat ids*/
	satindx = 0,		/* Circular index pointer to satidx_buf	     */
        eyear,                  /* Current ephemeris record year             */
        emonth,                 /* Current ephemeris record month            */
        eday,                   /* Current ephemeris record day              */
        ejdate,                 /* Current ephemeris record julian date      */
	status,			/* Return status			     */
	nonfatal = 1,		/* Non-fatal error code			     */
	nrecs = 0,		/* Number of records read in		     */
	tdiff,			/* Time difference between two lines	     */
        time_val;               /* Time variable                             */

char    eline[EPHEM_SIZ],       /* Ascii ephemris line information           */
	msg[ERRLEN];		/* Informational error message string	     */

for (i=0; i< MAX_BAD; i++)
    {
    jday[i] = 0;
    milsec[i] = 0;
    }

/*
  Get the current julian date to verify the date of the
  current line
*/
time(&time_val);
lctm = localtime(&time_val);
cday = c_4digityr(lctm->tm_year) * 1000 + lctm->tm_yday+1;

/*
  Cann't be past the current year
*/
if (c_4digityr(pass->year) > c_4digityr(lctm->tm_year))
    {
    sprintf(msg,"Invalid year specified.  Year reset from %02d to %02d",
	    pass->year,(c_4digityr(pass->year)-1)%100);
    c_errmsg(msg,"ingest-invyr",&nonfatal);
    pass->year = c_4digityr(pass->year)-1;
    }

/*
  Read in the first (MAX_BAD - 1) records and load up the julian date and
  millisecond arrays.
*/
for (i = 1; i < MAX_BAD; i++)
    {
    if (get_next_rec(acq_fd,jday,milsec,&satidx) == E_FAIL)
       return(E_FAIL);
    satidx_buf[satindx++] = satidx;
    if (satindx >= MAX_BAD)  satindx = 0;
    nrecs++;
    }

/*
  Use the julian date and time to determine the first valid line of data
*/
for (; ;)
    {
    /*
      Read the next line from the acqusition file
    */
    if (get_next_rec(acq_fd,jday,milsec,&satidx) == E_FAIL)
       return(E_FAIL);
    satidx_buf[satindx++] = satidx;
    if (satindx >= MAX_BAD)  satindx = 0;
    nrecs++;

    /*
      Initialize the first valid line flag assuming we have found it
    */
    first_line = TRUE;

    /*
      Make sure the julian date and time code are within valid ranges
    */
    for (i = 0; i < MAX_BAD; i++)
        {
        if ( (jday[i] <= 0) || (jday[i] > MAX_JDAY) ||
             (milsec[i] <= 0) || (milsec[i] > MIL_PER_DAY) )
	    first_line = FALSE;
        }
    
    all_days = jday[0];

    /*
      Make sure the time difference between any two consecutive lines
      does not exceed the maximum
    */
    for (i = 1; i < MAX_BAD; i++)
	{
        if (pass->dtype == GAC)
            tdiff = (long)((milsec[i-1] - milsec[i]) / (1000 /
                     LAC_PER_SEC) + .5 - 1);
        else
            tdiff = (long)((milsec[i-1] - milsec[i]) / (MIL_PER_SEC /
                     LAC_PER_SEC) + .5 - 1);

        if (tdiff != 0)
            first_line = FALSE;
	all_days += jday[i];
	}

    /*
      Check to determine if the first good line has a valid julian date
    */
    ave_days = (long)(((float)all_days / MAX_BAD) + .5);
    if (jday[MAX_BAD-1] != ave_days)
	 first_line = FALSE;

    /*
      A valid line is found and used as the first line of the output image
    */
    if (first_line)
	{
	/*
	  Verify that the time stamp is really valid.
	*/
	if ((milsec[0] < 0) || (milsec[0] > MIL_PER_DAY))
	    continue;

	/*
	  Verify that the julian date is valid.  If the date of the
	  current line is after the current date there are problems.
	  NOTE: adding 1 to current day because when processing on the adaps
		side we are 6 hours behind GMT.  Thus an image can be acquired
		on Jan 2 GMT, but processed here on Jan 1 CST.
	*/
	lday = (c_4digityr(pass->year) * 1000) + jday[0];
	if (lday > (cday + 1))
           {
           sprintf(msg,"Invalid year specified.  Year reset from %02d to %02d",
	           pass->year,(c_4digityr(pass->year)-1)%100);
           c_errmsg(msg,"ingest-invyr",&nonfatal);
           pass->year = (c_4digityr(pass->year)-1)%100;
           lday = (c_4digityr(pass->year) * 1000) + jday[0];
           }

	nrecs -= (MAX_BAD);
	sprintf(msg,"Found %d invalid records at the start of the pass",nrecs);
	c_errmsg(msg," ",&nonfatal);
	if (pass->nlines != 0)
	    {
	    /* if we already know the number of lines, adjust 
	    for the number of invalid records already skipped 
	    & not being read */
	    pass->nlines -= nrecs;
	    }

	/*
	  Position the pointer to first valid record
	*/
	acqskip(acq_fd,-MAX_BAD);

	break;
	}
    }	/* end of infinite 'for' construct */

/*
  Convert the julian date to gregorian for readability
*/
pass->jday = jday[MAX_BAD-1];
c_julgre(&pass->year,&pass->jday,&pass->month,&pass->day);
if ( !c_isleap( &pass->year ) && pass->jday == MAX_JDAY )
    {
    c_lasmsg(FALSE,"","Julian day of first data line is 366");
    sprintf(msg,"Specified year, %d, is not a leap year",pass->year);
    c_lasmsg(TRUE,"ingest-nleap",msg);
    }

/*
  Convert the start time in milliseconds to a readable format
*/
c_cvtmil(&milsec[MAX_BAD-1],&pass->shour,&pass->smin,&pass->ssec,
	 &pass->sfrac);

/*
  From the circular satellite id buffer containing the most recently read
  MAX_BAD satellite id's, find the most commonly occuring satellite id.
*/
find_satid(satidx_buf,&satidx);

/*
  Get the satellite id and number
*/
switch (satidx)
    {
    case 1:
	pass->satnum = 11;
	break;
    case 3:
        /* NOAA16 has a code of 3 also, check
	   date to determine which one it is.
	--------------------------------------*/
	if ( pass->year > 79 && pass->year < 86 )
	    pass->satnum = 6;
	else
	    pass->satnum = 16;
	break;
    case 5:
	pass->satnum = 14;
	break;
    case 7:
        /* NOAA15 has a code of 7 also, check
	   date to determine which one it is.
	--------------------------------------*/
	if ( pass->year > 80 && pass->year < 87 )
	    pass->satnum = 7;
	else
	    pass->satnum = 15;
	break;
    case 9:
	pass->satnum = 12;
	break;
    case 11:
        /* NOAA17 has a code of 9 also, check
	   date to determine which one it is.
	--------------------------------------*/
	if ( pass->year >= 85 && pass->year <= 88 )
	    pass->satnum = 9;
	else
	    pass->satnum = 17;
	break;
    case 13:
        /* NOAA18 has a code of 13 also, check
	   date to determine which one it is.
	--------------------------------------*/
	if ( pass->year >= 05 )
	    pass->satnum = 18;
	else
	    pass->satnum = 8;
	break;
    case 15:
         /* NOAA19 has a code of 15 also, check
           date to determine which one it is.
        --------------------------------------*/
	if (pass->year >= 86  && pass->year <= 94)
            pass->satnum = 10;
        else
            pass->satnum = 19;
	break;
    default:
	if ( pass->satnum == 0 )
	    {
	    sprintf(msg,"Invalid satellite id: %d",satidx);
	    c_errmsg(msg,"ingest-invid",&nonfatal);
	    return(E_FAIL);
	    }
	else
	    {
	    sprintf(msg,"Invalid satellite id: %d using %d from header",
	    	satidx, pass->satnum );
	    c_errmsg(msg,"ingest-invid",&nonfatal);
	    }
    }

/*
  Get the NBS offset
*/
if ((strcmp(pass->station,"EAF") == 0) ||
    (strcmp(pass->station,"NG1") == 0) ||  /* Archive conversion */
    (strcmp(pass->station,"NO1") == 0) ||  /* Archive conversion */
    (strcmp(pass->station,"NM2") == 0))
    {
    /*
      Calculated the corrected start time.  The time from the EAF/NM2
      format is based off the 15th line unless more than 15 lines of bad
      time codes were found.

      **********************> ! N O T E ! <*********************************

      THE NBS OFFSET CONTAINS THE FIRST SCAN TIME OF THE EAF FORMAT
      (EAF format will go away when converted to the new format!)

      ***********************> ! N O T E ! <********************************
    */
    if (nrecs <= 15)
	{
	if (pass->dtype == GAC)
	    milfst = (long)(pass->clock_err - (14 * MIL_PER_SEC / GAC_PER_SEC));
	else
	    milfst = (long)(pass->clock_err - (14 * MIL_PER_SEC / LAC_PER_SEC));
	}
    else
	{
	if (pass->dtype == GAC)
	    milfst = (long)(pass->clock_err - ((nrecs-1) * MIL_PER_SEC / GAC_PER_SEC));
	else
	    milfst = (long)(pass->clock_err - ((nrecs-1) * MIL_PER_SEC / LAC_PER_SEC));
	}

    /*
      Calculate the offset needed to correct the time of first scan to
      the corrected time pulled from the EAF/NM2 header.
    */
    pass->clock_err = milsec[MAX_BAD-1] - milfst;
    }
else if (!(((strcmp(pass->station,"SFL") == 0) ||
	 (strcmp(pass->station,"NOA") == 0) ||
	 (strcmp(pass->station,"NGC") == 0) ||
	 (strcmp(pass->station,"PAS") == 0)) &&
	 (pass->clock_err != 0)))
    {
    /*
      If this is a SFL NOA NGC or PAS scene then we already got the NBS
      offset from the header.  If the value from the header is 0, or this
      scene is from a recieving station other than these 4, get the NBS
      offset offset from the appropriate time corrections table.
    */
    status = c_acqtmcor(&pass->satnum,&pass->month,&pass->day,&pass->year,
			&pass->shour,&pass->smin,&pass->ssec,&pass->sfrac,
			&pass->clock_err);
    if (status != E_SUCC)
	return(E_FAIL);
    }

/*
  This is an attempt to help processing when starting into the new year.  This
  can be best explained by an example!
  Here it goes:
  An image from 1992 is being processed and the user specified the year
  parameter as 1993.  The user did this because they were not sure what
  year the data was acquired and it isn't available in the data.  This
  is also the only way that we can help the user.  We try to help by reading
  the ephemeris information for the specified julian date and year.  The
  julian data is extracted from the data.  Because this is a new year
  hopefully the ephemeris doesn't exist yet (i.e. julian date 93336).  When
  the date of the ephemeris returned is more than 60 days before the julian
  date subtract one from the specified year to create a new julian date
  (i.e. julian date 92366).
*/
status = c_readephem(&pass->satnum,&pass->year,&pass->month,&pass->day,eline);
if (status != E_SUCC)
    return(E_FAIL);

/*
  Extract the year month and day from the ephemeris
*/
sscanf(&eline[63],"%02d%02d%02d",&eyear,&emonth,&eday);

/*
  Create a julian date from the gregorian date
*/
c_grejul(&eyear,&emonth,&eday,&ejdate);
ejdate = (c_4digityr(eyear) * 1000) + ejdate;

/*
  An invalid year must have been specified because ephemeris data doesn't
  even exist within 30 days of the acquisition date.  Subtract one from the
  year.
*/
if ((lday - ejdate) > 30)
    {
    if ((lday > (c_4digityr(pass->year) * 1000) + 5) &&
       (ejdate < (eyear * 1000) + 360))
	{
	sprintf(msg,"Invalid year specified.  Year reset from %d to %d",
		pass->year,(c_4digityr(pass->year)-1)%100);
	c_errmsg(msg,"ingest-invyr",&nonfatal);
	pass->year = (c_4digityr(pass->year)-1)%100;
	}
    }

return(E_SUCC);
}
/****************************************************************************
NAME:		GET_NEXT_REC

PURPOSE:	Get the next record of data from the acquired image.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		7/92	T. Baltzer 	   CSB      Initial development

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*****************************************************************************/
#include "ingest.h"

int get_next_rec
(
long	*acq_fd,	/* File descriptor to the acquire file	            */
long	jday[],		/* Julian dates for each line		            */
long	milsec[],	/* Time code (milliseconds) of each line            */
long	*satidx		/* Acquisition satellite id		            */
)

{
long	i;			/* Looping variable			     */
long	nonfatal = 1;		/* Non-fatal error code			     */
long	rdstat;			/* Status of the acquire read		     */

static short mnrbuf[MNR_SIZ],	/* HRPT minor frame buffer		     */
	     tipbuf[TIP_SIZ],	/* Tip data buffer			     */
	     imgbuf[LAC_SIZ];	/* Image data buffer			     */

/*
  Push all of the values 'up' one element in the array
  so that the new one can go into the 0'th element.
*/
for (i = MAX_BAD - 1 ; i > 0; i--)
   {
   jday[i] = jday[i - 1];
   milsec[i] = milsec[i - 1];
   }

/*
  Read the next line from the acqusition file
*/
rdstat = c_acqread(acq_fd,mnrbuf,tipbuf,imgbuf);
if (rdstat == E_EOF)
    {
    c_errmsg("No valid data found in acquired image","ingest-noval",&nonfatal);
    return(E_FAIL);
    }
else if (rdstat != E_SUCC)
    {
    c_errmsg("Error reading from the acquisition file","ingest-read",&nonfatal);
    return(E_FAIL);
    }

/*
  Get the julian date and time of the current line
*/
jday[0] = (mnrbuf[8] >> 1) & 0x3ff;
milsec[0] = mnrbuf[9] & 0x7f;
milsec[0] = (milsec[0] << 10) | (mnrbuf[10] & 0x3ff);
milsec[0] = (milsec[0] << 10) | (mnrbuf[11] & 0x3ff);
/*
  Get the acquisition satellite id
*/
*satidx = (mnrbuf[6] >> 3) & 0xf;

return(E_SUCC);
}
