/*******************************************************************************
FUNCTION:	fix_lines

PURPOSE:	Fix the specified number of lines by correcting the time
		stamp and writing the line to archive file.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    07/91  D.Hollaren       Development
  2.1    04/93  J.Fenno          Modified calls to the new archio supports


ALGORITHM REFERENCE:
  See "NOAA Technical Memorandum NESS 107" for a detailed description
  of the HRPT Minor Frame Format.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "ingest.h"
#include "avhrr.h"
#include "archio.h"

int fix_lines
(
long	*acq_fd,		/* File descriptor to the acquire file	     */
int	tip_fd,			/* Tip output file descriptor		     */
long	*arc_fd,		/* File descriptor to the archive file       */
long	*dtype,			/* Type of data (LAC, GAC, HRPT)	     */
long	*missing,		/* Number of lines to be fixed		     */
double	*milold			/* Time in milliseconds of previous line     */
)
{
extern long	curent;		/* Current line being processed		     */

long	nonfatal = 1,		/* Non-fatal error code			     */
	milseclong,		/* Long version of milsec		     */
	rdstat,			/* Status of the acquire read		     */
	status,			/* Return status variable		     */
	i;			/* Looping variable			     */
long	j;			/* loopp control variable		     */
long	zero_count;		/* count of zero points in minor frame       */

double	milsec;			/* Time in milliseconds of current line	     */

static short mnrbuf[MNR_SIZ],	/* HRPT minor frame buffer		     */
	     tipbuf[TIP_SIZ+1],	/* Tip data buffer			     */
	     imgbuf[LAC_SIZ];	/* Image data buffer			     */

/*
  Found a section of bad time stamp codes that can be corrected
*/
for (i = 0; i < *missing; i++)
    {
    /*
      Read a line from the acqusition file
    */
    rdstat = c_acqread(acq_fd,mnrbuf,tipbuf,imgbuf);
    if (rdstat == E_EOF)
	return(E_EOF);
    else if (rdstat != E_SUCC)
	return(E_FAIL);

    /*
      Strip the TIP data from the line if the user wants it.  The file
      descriptor will not be valid if the data didn't exist when opened.
    */
    if (tip_fd)
	{
	/*
	  Write the tip data to the output file
	*/
	status = write(tip_fd,tipbuf,TIP_UNP);
	if (status <0)
	    {
	    c_errmsg("Error writing TIP data to an output file","ingest-tip",
		     &nonfatal);
	    return(E_FAIL);
	    }
	}

    /* Determine what the correct milsec should be
    ----------------------------------------------*/
    if (*dtype == GAC)
	milsec = *milold + (MIL_PER_SEC / GAC_PER_SEC);
    else
	milsec = *milold + (MIL_PER_SEC / LAC_PER_SEC);
    milseclong = (long)(milsec + 0.5);

    /* DEL added for CR 1556
    Instead of just fixing the time stamp first check a few
    other points in the line.  If they are all zero we assume
    the line is a drop line and we don't want to fix the time
    stamp.  Note: can't rely on the first 10 words of the
    minor frame because some acquisition software packages zero
    out the frame sync and parts of others.
    The locations 17 through 101 of the minor frame contain
    PRT counts, calibration target view and space data.
    Location 22 is undefined so don't check it.
    18 thru 98 at 5 gives 17 check points
    ----------------------------------------------------------*/
    zero_count = 0;
    for ( j = 18; j <= 98; j += 5 )
        if ( mnrbuf[j] == 0 )
            zero_count++;
    /* END of test fix */

    /* Check with some appropriate value less than 17.  Note: It depends on
    how strict we want to be.  Checking for == 0 means that all the points
    have to be non-zero before we fix the time stamp. Checking for < 8 means
    at least 10 points must be non-zero and for < 17 means that only one
    non-zero point will allow us to fix the time stamp.
    If zero count is okay reset the time stamp code to what was expected.
    ------------------------------------------------------------------------*/
    if ( zero_count < 7 )
        {
	/*
	  The time stamp code was bad so correct them for the archive
	*/
	mnrbuf[11] = (short)(milseclong & 0x3ff);
	mnrbuf[10] = (short)((milseclong >> 10) & 0x3ff);
        mnrbuf[9] &= 0xff80;
	mnrbuf[9] += (short)((milseclong >> 20) & 0x7f);
	}

    /*
      Write a line to the output image
    */
    status = c_arcwrite(arc_fd,mnrbuf,imgbuf);
    if (status != E_SUCC)
	return(E_FAIL);

    /*
      Save the time code
    */
    *milold = milsec;

    /*
      Increment the line counter
    */
    curent++;
    }

/*
  Reset the number of missing lines
*/
*missing = 0;

return(E_SUCC);
}
