/*******************************************************************************
FUNCTION:	ingest



PURPOSE:	Ingest raw image data acquired from the satellite.  The image
		data is converted to BIP and the time codes are checked to
		detect and fill any dropped lines.  A header file associated
		with the output image is also filled with calculated
		information about the image.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/93  J.Fenno          Modified calls to the new archio supports
  2.2    07/96  T.Baltzer        Modified to implement DATAPATH
  2.3    08/97  D.Lloyd		 Modified to increase precision of calulated
				 timestamps within a segment of noise.  Fixes
				 the bug wherein an extra drop line is inserted
				 because of accumlative error in the calculated
				 timestamps.
  2.4    03/98	D.Lloyd		 Added logic to change station id OP1 to OPF.
  2.5    12/99  D.Lloyd		 Prototyped module and modified first_rec to
  				 use header satellite number when it is not
				 in the data SRF 3134
  2.6    02/00  D.Lloyd		 Removed logic that changes the station ID.
  				 This is now accomplished in acqio functions.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include <string.h>

#ifdef VMS
#include <file.h>
#else
#include <sys/file.h>
#endif


#include "ingest.h"
#include "util.h"
#include "labserv.h"
#include "archio.h"
#include "adapssup.h"
#include "acqio.h"

int main( void )
{
struct tm 	*lctm;	   	/* Local time structure			     */

long	acq_fd,			/* File descriptor to the acquire file	*/
	arc_fd;			/* File descriptor to the archive file	*/

int	tip_fd;			/* File descriptor to the TIP file	*/

long	ocount = 0,		/* Number of output files created	*/
	expected,		/* Number of expected lines             */
	nonfatal = 1,		/* Non-Fatal error code			*/
	fatal = -1,		/* Fatal error code			*/
	stdlun,			/* Standard output			*/
	tip_flg,		/* TIP data existance flag		*/
	time_val,		/* Time variable			*/
	access,			/* Access type to archive files		*/
	genfil,			/* Flag to generate a output file name  */
	year,			/* Year of the pass			*/
	dtype,			/* Type of data (HRPT, LAC, or GAC)	*/
	msgtim,			/* Time between processing messages	*/
	status;			/* Status variable			*/

char	hosin[CMLEN],		/* Input host name (raw acquired image) */
	hosout[CMLEN],		/* Output host file name		*/
	tmpfil[CMLEN],		/* Temporary output file name		*/
	assout[CMLEN],		/* Output associated file		*/
	tmptip[CMLEN],		/* Temporary (optional) tip file name   */
	tipfil[CMLEN],		/* File name of the TIP file		*/
	fname[CMLEN],		/* Temporary file name construction	*/
	out[3][CMLEN],		/* Output file names created		*/
	sceneid[17],		/* Scene ID				*/
	tstamp[7],		/* Time stamp code			*/
	station[4],		/* Station identifier			*/
	*svec[1];		/* Array of pointers for output to TAE	*/

struct PARBLK	vblock;		/* TAE parameter block			*/
struct MODEL	model;		/* Model and orbital information	*/
struct SATPASS	pass;		/* Satellite pass information		*/
struct DROPPED	drop;		/* Archive dropped line information	*/

/*
  Initalize LAS
*/
c_inlas(&vblock,&stdlun);

/*
  Get the input parameters
*/
getpar(&vblock, hosin, hosout, tmpfil, tmptip,
    &genfil, &year, station, &dtype, &msgtim);

/*
  Open the acquired image.  The station id will be used to read the
  STATION.IDS table to determine how to read the data.
*/
access = IREAD;
status = c_acqopen(&acq_fd,hosin,&access,station,&dtype,&tip_flg,&year,&pass);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ingest-fatal",&fatal);

/*
  Issue a message on the expected size of the acquired image
*/
size(hosin, &expected, &acq_fd);

/*
  Open the output image
*/
access = IWRITE;
status = c_arcopnwrt(tmpfil, &dtype, &expected, &arc_fd);
if (status != E_SUCC)
    {
    c_acqclose(&acq_fd);
    c_errmsg("Fatal error encountered","ingest-fatal",&fatal);
    }

/*
  Save the temporary archive name for clean up
*/
strcpy(out[ocount],tmpfil);
ocount++;

/*
  Open the output tip file if it is to be saved
*/
if ((tip_flg) && (strlen(tmptip) > 0))
    {
    access = IWRITE;
    status = c_tipopen(&tip_fd,tmptip,&access);
    if (status != E_SUCC)
	{
	c_acqclose(&acq_fd);
	c_arclose(tmpfil,&arc_fd);
	c_clean(&vblock,"ingest",(const char (*)[CMLEN])out,&ocount);
	}

    /*
      Save the temporary tip file name for clean up
    */
    strcpy(out[ocount],tmptip);
    ocount++;
    }
else
    tip_fd = 0;

/*
  Process the GAC, LAC or HRPT data
*/
status = process(&acq_fd,&arc_fd,tip_fd,&pass,&model,&drop,&msgtim);
if (status != E_SUCC)
    {
    c_acqclose(&acq_fd);
    c_arclose(tmpfil,&arc_fd);
    if (tip_fd)
	close(tip_fd);
    c_clean(&vblock,"ingest",(const char (*)[CMLEN])out,&ocount);
    }

/*
  Close the acquire, archive, and associated AHDR files
*/
c_acqclose(&acq_fd);
c_arclose(tmpfil,&arc_fd);

/*
  Close the TIP file
*/
if (tip_fd)
    close(tip_fd);

/*
  Generate a scene id
*/
c_getscene(&model,sceneid);

/*
  Send the sceneid to the variable block
*/
svec[0] = sceneid;
status = q_string(&vblock,"SCENEID",1,(const char **)svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error placing the scene ID in the parameter block",
	     "ingest-sceneid",&nonfatal);
    c_errmsg("Fatal error encountered","ingest-fatal",&fatal);
    }

/*
  Generate a time stamp code from the current time
*/
time(&time_val);
lctm = localtime(&time_val);
sprintf(tstamp,"%02d%02d%02d",lctm->tm_hour,lctm->tm_min,lctm->tm_sec);

/*
  Write the time stamp to the parameter block
*/
svec[0] = tstamp;
status = q_string(&vblock,"TSTAMP",1,(const char **)svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error placing the time stamp in the parameter block",
	     "ingest-tstamp",&nonfatal);
    c_errmsg("Fatal error encountered","ingest-fatal",&fatal);
    }

/*
  Send the number of expected lines to the variable block
*/
status = q_intg(&vblock,"EXPECTED",1,(int *)&expected,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg(
	"Error placing the expected number of lines in the parameter block",
	"ingest-expected",&nonfatal);
    c_errmsg("Fatal error encountered","ingest-fatal",&fatal);
    }

/*
  Send the total number of missed lines to the variable block
*/
status = q_intg(&vblock,"NMISSED",1,(int *)&model.totmiss,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg(
	"Error placing the total number of missed lines in the parameter block",
	"ingest-nmissed",&nonfatal);
    c_errmsg("Fatal error encountered","ingest-fatal",&fatal);
    }

/*
  Write the parameter block to TAE
*/
status = q_out(&vblock);
if (status != SUCCESS)
    {
    c_errmsg("Error sending the parameter block","ingest-vblock",&nonfatal);
    c_errmsg("Fatal error encountered","ingest-fatal",&fatal);
    }
/*
  Generate a file name using the header information when no output 
  file name was specified by the user.
*/
if (genfil)
    {
    sprintf(fname,"%s_%s.arch",sceneid,tstamp);
    strcat(hosout,fname);
    }

/*
  Rename the output file to the sceneid or the user specified name
*/
rename(tmpfil,hosout);

/*
  Reset the output file counter and save the output file name
*/
ocount = 0;
strcpy(out[ocount], hosout);
ocount++;

/*
  Create the name for the archive associated header file
*/
c_lsmknm(hosout,"ahdr",assout);
strcpy(out[ocount], assout);
ocount++;

/*
  Update the model station ID
*/
strcpy(model.station,pass.station);

/*
  Write the header information to the associated header file
*/
status = c_putahdr(hosout,&model,&drop);
if (status != E_SUCC)
    c_clean(&vblock,"ingest",(const char (*)[CMLEN])out,&ocount);

/*
  Check to see if the TIP file was processed
*/
if (strlen(tmptip) > 0)
    {
    c_lsmknm(hosout,"tip",tipfil);
    rename(tmptip,tipfil);

    /*
      Add the TIP file to the list of output files created
    */
    strcpy(out[ocount], tipfil);
    ocount++;
    }

/*
  Display a completion message
*/
c_complt("INGEST",out,&ocount);
return E_SUCC;
}
