/*******************************************************************************
FUNCTION:	process

PURPOSE:	Read HRPT, LAC, or GAC data from the acqusition file, check
		the time stamp codes for gaps (fill in gaps with zero lines),
		and write the data to the output image file for archive.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/93  J.Fenno          Modified calls to the new archio supports


ALGORITHM REFERENCE:
  See "NOAA Technical Memorandum NESS 107" for a detailed description
  of the HRPT Minor Frame Format.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "ingest.h"
#include "util.h"
#include "pixman.h"
#include "avhrr.h"
#include "archio.h"
#include "adapsutil.h"

int process
(
long	*acq_fd,		/* File descriptor to the acquire file	     */
long	*arc_fd,		/* File descriptor to the archive file       */
int	tip_fd,			/* Tip output file descriptor		     */
struct	SATPASS	*pass,		/* Satellite pass information		     */
struct	MODEL	*model,		/* Model and orbital			     */
struct	DROPPED	*drop,		/* Dropped line index			     */
long	*msgtim			/* Time between processing messages	     */
)
{
extern long	curent;		/* Current line being processed		     */

long    
	dtype = EWORD,		/* Type of data being processed		     */
	flag = 2,		/* Processing message format flag	     */
	max_entity = MAX_LINES,	/* Maximum number of lines		     */
	maxsam = MAX_ACQ,	/* Maximum number of samples in aquire       */
	miloldlong,		/* Long version of milold		     */
	nonfatal = 1,		/* Non-fatal error code			     */
	zero = 0;		/* Constant zero			     */

long	missing,		/* Number of missing lines in a gap	     */
	decline = 0,		/* Number of lines to decrease input by	     */
	totmiss = 0,		/* Total number of missing lines	     */
	i,			/* Looping variables			     */
	rdstat,			/* Status of the acquire read		     */
	status;			/* Status variable			     */

long	jday,			/* Julian date of current line		     */
	jold = -1,		/* Julian date of previous line		     */
	milsec,			/* Time in milliseconds of current line	     */
	first_mil,		/* Time in milliseconds of first line        */
	last_mil;		/* Time in milliseconds of last line	     */

double	milold;			/* Time in milliseconds of previous line     */

static short mnrbuf[MNR_SIZ],	/* HRPT minor frame buffer		     */
	     tipbuf[TIP_SIZ+1],	/* Tip data buffer			     */
	     imgbuf[LAC_SIZ],	/* Image data buffer			     */
	     zerobuf[MAX_ACQ];	/* Fill for missing lines		     */

char	satid[9],		/* Satellite id				     */
	msg[ERRLEN];		/* Error messages			     */

/*
  Fill the zero buffer with zeros to signal a dropped line.
*/
c_pxfill((unsigned char *)zerobuf,&dtype,&maxsam,(unsigned char *)&zero);
drop->ngaps=0;

/*
  Find and place the file descriptor at the first valid record.  The
  julian date, first time stamp, satellite id, number of samples,
  number of bands, and the valid channels are also set in the archive
  header.
*/
rdstat = first_rec(acq_fd,pass);
if (rdstat != E_SUCC)
    return(E_FAIL); 

/*
  Read the first line from the acqusition file
*/
rdstat = c_acqread(acq_fd,mnrbuf,tipbuf,imgbuf);
if (rdstat != E_SUCC)
    return(E_FAIL);

/*
  Get the time stamp code
*/
jday = (mnrbuf[8] >> 1) & 0x3ff;
milsec = mnrbuf[9] & 0x7f;
milsec = (milsec << 10) | (mnrbuf[10] & 0x3ff);
milsec = (milsec << 10) | (mnrbuf[11] & 0x3ff);
/*
  Reset the first time stamp code with the NBS offset
*/
first_mil = milsec - pass->clock_err;
c_cvtmil(&first_mil,&pass->shour,&pass->smin,&pass->ssec,&pass->sfrac);

/*
  Tell the user what is being processed
*/
sprintf(satid,"NOAA-%02d",pass->satnum);
sprintf(msg,"Satellite:  %s  NBS offset:  %d",satid,pass->clock_err);
c_errmsg(msg," ",&nonfatal);

sprintf(msg,"%s  %02d/%02d/%02d  %s  %02d:%02d:%02d.%03d",
	"Start date:",pass->month,pass->day,pass->year,
	"Start time:",pass->shour,pass->smin,pass->ssec,pass->sfrac);
c_errmsg(msg," ",&nonfatal);

/*
  Start the processing message
*/
curent = 1;
c_inittm("INGEST","Processed","line"," "," ",&max_entity,&zero,msgtim,
	 &flag);
c_startm();

/*
  Read the input acquired image, optionally save the TIP data
  to a TIP file, write the HRPT minor frame and image data, and
  update the associated AHDR file.
*/
while ((rdstat != E_EOF) && (rdstat != E_BOF) && (rdstat != E_SMAL))
    {
    if (jold > -1)
	{
	/*
	  Read a line from the acqusition file
	*/
	rdstat = c_acqread(acq_fd,mnrbuf,tipbuf,imgbuf);
	if (rdstat == E_EOF)
	    continue;
	else if (rdstat == E_SMAL)
	    {
	    c_lasmsg(FALSE, "ingest-ignore", "Ignoring incomplete line at EOF");
	    continue;
	    }
	else if (rdstat == E_BOF)
	    /*
	      If we hit Beginning of File (for flipped data),
	      complete this iteration, then stop
	    */
	    ;
	else if (rdstat != E_SUCC)
	   return(E_FAIL);


	/*
	  Get the time stamp code
	*/
	jday = (mnrbuf[8] >> 1) & 0x3ff;
	milsec = mnrbuf[9] & 0x7f;
	milsec = (milsec << 10) | (mnrbuf[10] & 0x3ff);
	milsec = (milsec << 10) | (mnrbuf[11] & 0x3ff);
        /*
	  Check to determine if the current julian date is good
	  compared to the old julian date.
	*/
	status = check_date(&jday,&jold,&milsec,&milold);
	if (status == BAD_DAY)
	    {
	    /*
	      Try to correct the bad julian date by using the previous
	      julian date.
	    */
	    mnrbuf[8] = (short)((jold & 0x3ff) << 1);
	    }

	/*
	  Check the time stamp code
	*/
	miloldlong = (long)milold;
	status = check_time(acq_fd,&pass->dtype,&milsec,&miloldlong,&missing,
			    &decline);
        switch (status)
            {
            case NEW_TIME:
	    /*
	      Fix a group of lines that have bad time stamps.  The number of
	      lines to be fixed is passed back in the missing parameter.
	    */
	    status = fix_lines(acq_fd,tip_fd,arc_fd,&pass->dtype,&missing,
				&milold);
            if (status == E_EOF)
                {
                rdstat = E_EOF;
                continue;
                }
            else if (status != E_SUCC)
		return(E_FAIL);

	    /*
	      The group of lines have all been fixed and written to the
	      archive file so go get the next line
	    */
	    continue;

            case MISSING_TIME:
	    /*
	      Record and fill any missing lines
	    */
	    status = fill_missing(arc_fd,drop,&missing,zerobuf);
	    if (status != E_SUCC)
		return(E_FAIL);

	    /*
	      Keep track of the total number of missing lines
	    */
	    totmiss += missing;
	    sprintf(msg,
		    "%d lines missed at line %d totaling %d lines missed",
		    missing,drop->lmissed[drop->ngaps-1],totmiss);
	    c_errmsg(msg," ",&nonfatal);

	    /*
              ************ Temporary for evaluations *******************
	      Check the total missing lines limit
	    if (totmiss > MAX_MISSING)
	        {
	        sprintf(msg,"%d total lines missed exceeds the limit %d",
			totmiss,MAX_MISSING);
	        c_errmsg(msg,"ingest-totmiss",&nonfatal);
	        return(E_FAIL);
	        }
              ************ Temporary for evaluations *******************
	    */

	    /*
	      Reset the time in milliseconds of the previous line to take
	      into account the missing lines.
	    */
	    if (pass->dtype == GAC)
		milold = milold + ((float)missing / GAC_PER_SEC) * MIL_PER_SEC;
	    else
		milold = milold + ((float)missing / LAC_PER_SEC) * MIL_PER_SEC;

	    /*
	      The group of lines have all been zero filled and written to the
	      archive file so go get the next line.  But first drop into the
	      next case to decrement the input number of lines, if necessary.
	    */

            case NONE_MISSED:
	    /*
	      Some of the previous archive tapes have zero filled lines
	      that don't need to be there.  Ignore these dropped lines
	      and decrement the number of lines for the input image.
	      Decrement the number of lines only if it is known and
	      being used (non-zero).
	    */
	    if (pass->nlines != 0)
               pass->nlines -= decline;

	    /*
	      Reset the number of missing line for a gap.
	      Reset the number of lines to decrement the input image by.
	    */
	    missing = 0;
	    decline = 0;
	    continue;

            case E_EOF:
	    rdstat = E_EOF;
	    continue;

            case E_SUCC:
	    break;

	    default:
	    return(E_FAIL);
            }
	}

    /*
      Save the old julian date and time in milliseconds so we can look for
      gaps in the data.
    */
    jold = jday;
    milold = (double)milsec;

    /*
      Strip the TIP data from the line if the user wants it.  The file
      descriptor will not be valid if the data didn't exist when opened.
    */
    if (tip_fd)
        {
        /*
          Write the tip data to the output file
        */
        status = write(tip_fd,tipbuf,TIP_UNP);
        if (status <0)
            {
            c_errmsg("Error writing TIP data to an output file","ingest-tip",
                     &nonfatal);
            return(E_FAIL);
            }
        }

    /*
      Write a line to the output image
    */
    status = c_arcwrite(arc_fd,mnrbuf,imgbuf);
    if (status != E_SUCC)
	return(E_FAIL);

    /*
      Increment the line counter
    */
    curent++;

    /*
      Stop processing when the number of lines are known
    */
    if ((pass->nlines > 0) && (curent > pass->nlines))
	rdstat = E_EOF;
    }

/*
  One too many lines counted, and stop the processing message
*/
curent--;
c_stoptm();

/*
  Save the number of lines processed and number of missing lines
*/
sprintf(msg,"Processed %d lines with a total of %d missing lines",
	curent,totmiss);
c_errmsg(msg," ",&nonfatal);
model->totmiss = totmiss;
model->clock_err = pass->clock_err;

/*
  Initialize the model and orbital information
*/
c_initmod(&pass->satnum,&pass->year,&pass->month,&pass->day,&pass->shour,
	  &pass->smin,&pass->ssec,&pass->sfrac,&pass->dtype,&curent,model);

/*
  Calculate the time at the last line from the length of the pass.
  If a day was spanned, subtract the time of one day.
*/
if (model->dtype == GAC)
    last_mil = (long)(first_mil +
	       ((float)model->nlines / GAC_PER_SEC) * MIL_PER_SEC + 0.5);
else
    last_mil = (long)(first_mil +
	       ((float)model->nlines / LAC_PER_SEC) * MIL_PER_SEC + 0.5);
if (last_mil >= MIL_PER_DAY)
   last_mil -= MIL_PER_DAY;

/*
  Reset the end time in the model structure
*/
c_cvtmil(&last_mil,&model->ehour,&model->emin,&model->esec,&model->efrac);

/*
  Set the number of channels and channel numbers
*/
model->nchans = NCHANS;
for (i = 0; i < NCHANS; i++)
    model->chans[i] = i + 1;

return(E_SUCC);
}
