/******************************************************************************
FUNCTION:       INGEST1B


PURPOSE:        Ingest a Level 1b file.  See the NOAA Polor Orbital User's
                Guide for a description of the Level 1b file format.  Create
		a generic AVHRR output archive image.

		NOTES:	INGEST1B will handle multiple volume tapes.

Version	Date	Programmer	Modifications
-------	-----	---------------	----------------------------------------------
  1.0	01/92	T.Baltzer	Development
  1.1	05/93	J.Fenno		Modified to use the new AVHRR model and the
				new archio supports.
  1.2	09/93	T.Rockvam	Check start time of first line of image data
				to make sure it matches the headers;
				fix problem writing out correct mnr frame
				for GAC lines
  1.3	10/94	T.Baltzer	Use line counter found in the data set header
				record for verification only.  Upon
				encountering an EOT, only ask for the next
				tape if the user has specified a next tape.
  1.4	07/96	T.Baltzer	Modify calls to archio so that it follows
				DATAPATH.
  2.0	11/97	D.Lloyd		Modified to allow input from disk file,
				to make calls to Level 1b I/O supports and
				eliminate most global variables.  Major
				rewrite of complete module.
  2.1	12/98	D.Lloyd		Modified for NOAA KLM series sats.


ALGORITHM:
	The first record of a Level 1b file is the Terabit memory (TBM) header
	record or Archive Retrieval System (ARS) header for NOAA KLM series.
	The TBM/ARS header contains information as to the type of data,
	beginning latitude, ending latitude, beginning longitude, ending
	longitude, start hour, start minute, number of minutes, appended data
	selection, and the bands in the file.  This information is output to a
	user-specified destination.

	The second and third records of a LAC/HRPT file define the Data Set
	Header.  The second record contains the Data Set Header and the third
	record is meaningless to the user.  The second record of a GAC file
	define the Data Set Header.  For NOAA KLM satellites the second record
	contains the DSH -- however any number of subsequent records may be
	included see NOAA KLM Users Guide.  The Data Set Header record
	describes the data set and is stored in binary.

	The header records are followed by image data records.  Data records
	may be packed or unpacked, LAC/HRPT or GAC data, and may contain 1-5
	bands of data.  The data is band interleaved by pixel.  For LAC/HRPT
	data, two physical records are combined to form one line of image data
	for all bands.  For GAC data, one physical record contains two lines of
	image data for all bands in the file.  The first 448 bytes of the
	record (odd records only for LAC data) consist of information about the
	image line (including the calibration coefficients).  For packed LAC
	data, the last 696 bytes of the second record are nonimage data.
	The NOAA KLM data records all have one line per record; the minor
	frame telemetry and other information is unpacked (first 1264 bytes);
	the video data may be packed or unpacked.

	Forward skip records on file until the starting line is reached.  For
	each line to be copied, the following steps are taken.  Read two
	records (LAC) or one record (GAC and KLM) from the file (one line of
	image data) into a buffer.  Unpack the buffer and store the image
	sample values in a INTEGER*2 buffer.  Output this INTEGER*2 buffer.


ALGORITHM REFERENCES:
	1. NOAA Polor Orbital User's Guide for a description of the Level 1b
	   file format.
	2. NOAA KLM Users Guide for description of level 1b format for KLM
	   series satellites.

PROJECT:	ADAPS
******************************************************************************/
#include "ingest1b.h"
#include "util2.h"
#include "adapsutil.h"



int main(void)
{
char	hosout[1][CMLEN] = {{0}};/* Output host image names		*/
char	infile[CMLEN] = {0};	/* Input file name			*/
char	msg[CMLEN] = {0};	/* message string			*/
char	out[2][CMLEN] = {{0},{0}};/* names of output files created	*/
char	tmpass[CMLEN] = {0};	/* Temp associated output file name	*/
char	tmpfil[CMLEN] = {0};	/* Temporary output file name		*/

long	genfil;			/* Flag to generate a output file name	*/
long	goodlines;		/* Number of good (non-dropped) lines	*/
long    handle = HANDLE;        /* handle tape errors                   */
long	inchan[MOSTBND];	/* Flags for the input bands to process	*/
long	msgtim;			/* Time between processing messages	*/
long	nchans;			/* Number of bands to process		*/
long	numlines;		/* approx number of lines in output	*/
long    one = 1;                /* value of one                         */
long	oufopn = FALSE;		/* output file open flag		*/
long	outcnt = 0;		/* Number of output files		*/
long	out_fd;			/* File descriptor to output image	*/
long	recsiz;			/* Record size of data read		*/
long	seqchan[MOSTBND];	/* array of sequential band numbers	*/
long	skip;			/* number of files to skip on tape	*/
long	status;			/* function return status		*/
long	std_out;		/* Standard output			*/
long	window[4];		/* Input window to process		*/

struct	FDESC	in_fd;		/* input file descriptor		*/
struct	MODEL	model;		/* AVHRR model information		*/
struct	PARBLK	vblock;		/* Variable block			*/

/*
  Initialize LAS
----------------*/
c_inlas( &vblock, &std_out );


/*
  Get the input parameters
--------------------------*/
status = getpar( &vblock, &skip, hosout[0], tmpass, &genfil, &nchans, 
	inchan, &msgtim, tmpfil, infile, &in_fd );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"ingest1b-fatal","Fatal error encountered");
    return E_FAIL;
    }

/*
  For tape input skip to the proper file.
-----------------------------------------*/
if ( in_fd.ftype == L1BTAPE && skip > 0 )
    {
    status = c_l1bskipt(in_fd.drive, &skip );
    if ( status != E_SUCC )
	c_lasmsg(TRUE,"ingest1b-skip","Error skipping to file on tape");
    }


/*
  Open the input file 
---------------------*/
status = open_input( infile, window, &in_fd, &model, &recsiz, &goodlines );
if ( status != E_SUCC )
    {
    sprintf( msg, "Error opening %s", infile );
    c_lasmsg(TRUE,"ingest1b-open",msg);
    }


/*
  Get the window parameters and compute the window if user specified
  lat/long or lat/range
--------------------------------------------------------------------*/
status = c_windpar( &vblock, &model, window );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"ingest1b-windp","Error converting window parameters");


/*
  Check the channels in the file and the ones the user wants.
  On error: skip to the end of file (tape only), close the input
  file, and return.  Note: return E_SUCC since there may be more
  input files to process.
----------------------------------------------------------------*/
status = ck_chans( &model, inchan, &nchans, seqchan );
if ( status != E_SUCC )
    {
    if ( in_fd.ftype == L1BTAPE && in_fd.tape == TRUE )
	{
	c_tpfsf(&in_fd.tun, &one, &handle );
	status = c_l1bclost( &in_fd.tun, &in_fd.tape );
	}
    else if ( in_fd.ftype == L1BDISK )
	status = c_l1bclosd( in_fd.fd );
    if ( status != E_SUCC )
	{
	sprintf( msg, "Error closing %s", infile );
	c_lasmsg(FALSE,"ingest1b-close",msg);
	}
    return E_SUCC;
    }


/*
  Open the output image and signal that it's open on success.
-------------------------------------------------------------*/
det_size( &model, window, &numlines );
if ( c_arcopnwrt( tmpfil, &model.dtype, &numlines, &out_fd ) != E_SUCC )
    {
    oufopn = FALSE;
    sprintf( msg, "Error opening %s", tmpfil );
    c_lasmsg(FALSE,"ingest1b-open",msg);
    }
else
    oufopn = TRUE;


/*
  Process the image only if the output file is open.  Continue and
  close files even on failure.  If the return status indicates that
  an end of volume or end of tape was reached we want to return this
  status to the pdf so that processing can terminate.  For all other
  cases we will return E_SUCC.
---------------------------------------------------------------------*/
if ( oufopn )
    {
    status = process( &in_fd, &out_fd, tmpfil, window, inchan, seqchan,
		&model, &recsiz, &goodlines, &msgtim );

    /*
      Note: it is not sufficient to simply check for TAPE_EOV in the following
      if statement since E_EOF is defined as the same value.
    -------------------------------------------------------------------------*/
    if (in_fd.ftype == L1BTAPE && (status == TAPE_EOV || status == TAPE_EOT))
	ret_val( &vblock, status );
    else
	ret_val( &vblock, E_SUCC );
    }

/*
  Close the output image files 
------------------------------*/
if ( oufopn )
    {
    status = close_output(&out_fd,hosout[0],tmpfil,tmpass,genfil,out);
    if ( status != E_SUCC )
	{
	sprintf( msg, "Error closing %s", tmpfil );
	c_lasmsg(FALSE,"ingest1b-close",msg);
	}
    }


/*
  Close input file 
------------------*/
if ( in_fd.ftype == L1BTAPE && in_fd.tape == TRUE )
    status = c_l1bclost( &in_fd.tun, &in_fd.tape );
else if ( in_fd.ftype == L1BDISK )
    status = c_l1bclosd( in_fd.fd );
if ( status != E_SUCC )
    {
    sprintf( msg, "Error closing %s", infile );
    c_lasmsg(FALSE,"ingest1b-close",msg);
    }


/*
  Display a standard completion message
---------------------------------------*/
if ( out[1][0] != 0 )
    outcnt = 2;
else if ( out[0][0] != 0 )
    outcnt = 1;
c_complt( "INGEST1B", out, &outcnt );
return E_SUCC;
}
