/*****************************************************************************

FUNCTION:	PROCESS

PURPOSE:	Read data from the Level 1b input file and copy the image
		into an AVHRR archive (Level 0) file.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    11/97	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "ingest1b.h"
#include "util2.h"

#define	NODEBUG	TRUE		/* set to FALSE when debugging using	*/
				/* dbx TRUE otherwise			*/

int process
(
struct FDESC	*in_fd,		/* IN: input file descriptor		*/
long		*out_fd,	/* IN: output file descriptor		*/
char		tmpfil[],	/* IN: temp file name for archive file	*/
long		window[],	/* IN: window to process		*/
long		inchan[],	/* IN: input channels to process	*/
long		seqchan[],	/* IN: sequential band numbers to proc.	*/
struct MODEL	*model,		/* IN: AVHRR model structure		*/
long		*recsiz,	/* IN: input record size		*/
long		*goodlines,	/* IN: number of lines in input file	*/
long		*msgtim		/* IN: time between processing messages */
)
{
char		msg[CMLEN];	/* message string			*/

long		dum1;		/* dummy variable			*/
long		dum2;		/* dummy variable			*/
long		flag = GEN1;	/* processing message format flag	*/
long		i;		/* loop control variable		*/
long		nchans = 0;	/* number of channels to process	*/
long		outbytes;	/* number of bytes in output buffer	*/
long		status;		/* function return status		*/

short		*outbuf;	/* output buffer			*/

struct DROPPED	drop;		/* dropped line structure		*/
struct DROPPED	win_drop;	/* windowed dropped line structure	*/
struct MODEL	win_model;	/* windowed AVHRR model structure	*/

unsigned char	*bscan;		/* Level 1b scan line			*/


/*
  Initialize the dropped line structure to zero
-----------------------------------------------*/
drop.ngaps = 0;
for ( i = 0; i < 2164; i++ )
    drop.lmissed[i] = drop.nmissed[i] = 0;

/*
  Determine number of channels to process
-----------------------------------------*/
for ( i = 0; i < 5; i++ )
    if ( inchan[i] )
	nchans++;

/*
  Modify window[NL] to reflect window ending line number when
  user specified all lines by entering -1.
-------------------------------------------------------------*/
if ( window[NL] == -1 )
    window[NL] = model->nlines - window[SL] + 1;

/*
  Initialize the processing message
-----------------------------------*/
sprintf( msg, "%s %ld %s",
	"INGEST1B processing all lines concurrently for",nchans,"bands");
c_lasmsg(FALSE," ",msg);
#if NODEBUG
c_inittm("INGEST1B","Processed","line"," "," ",&dum1,&dum2,msgtim,&flag);
#endif

/*
  Allocate memory for the Level 1b scan line.  Dynamic allocation
  ensures that the buffer is word aligned.
-----------------------------------------------------------------*/
if (( bscan = (unsigned char *)malloc(*recsiz * 2)) == NULL )
    {
    c_lasmsg(TRUE,"ingest1b-alloc","Error allocation dynamic memory");
    return E_FAIL;
    }

/*
  Determine size of output buffer and allocate.  The output buffer is a
  fixed length depending on whether it is LAC or GAC data.  Sub-banding or
  sub-sampling has no bearing on the buffer size.
--------------------------------------------------------------------------*/
if ( model->dtype == GAC )
    outbytes = (MNR_WORDS + GAC_WORDS) * sizeof(short);
else
    outbytes = (MNR_WORDS + LAC_WORDS) * sizeof(short);
if (( outbuf = (short *) malloc(outbytes)) == NULL)
    {
    c_lasmsg(FALSE,"ingest1b-alloc","Error allocating dynamic memory");
    return E_FAIL;
    }

/*
  Convert the data file to Level 0 and write to archive file
------------------------------------------------------------*/
status = read_file(in_fd,out_fd,window,inchan,seqchan,model->nchans,*goodlines,
		    recsiz,bscan,outbuf,model,&drop);

free( bscan );
free( outbuf );
#if NODEBUG
c_stoptm();
#endif

/*
  Create a windowed model based on the user input window.  The
  window must be flipped for the call to c_windmod() if the pass
  is ASCENDING and in satellite view.  Also the dropped line
  structure just built needs to be relative to the entire scene.
----------------------------------------------------------------*/
if ( model->cpass == ASCENDING && model->sat_view == TRUE )
    window[SL] = model->nlines - window[SL] - window[NL] + 2;

for ( i = 0; i < drop.ngaps; i++ )
    drop.lmissed[i] += window[SL] - 1;
 
/* Recompute the model based on the actual number of lines in
the image.
-------------------------------------------------------------*/
model->ehour = model->emin = model->esec = model->efrac = 0;
status = c_initmod( &model->satnum, &model->year, &model->month, &model->day,
	&model->shour, &model->smin, &model->ssec, &model->sfrac,
	&model->dtype, &model->nlines, model );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"ingest1b-init","Error initializing model");
    return E_FAIL;
    }

/* Create the windowed model
----------------------------*/
if (c_windmod(model,&drop,&win_model,&win_drop,window,inchan,TRUE) != E_SUCC)
    {
    c_lasmsg(FALSE,"ingest1b-wind","Error creating windowed model");
    return E_FAIL;
    }

/*
  Put the model and dropped line information in the archive header file
-----------------------------------------------------------------------*/
if (c_putahdr(tmpfil,&win_model,&win_drop) != E_SUCC)
    {
    c_lasmsg(FALSE,"ingest1b-warn","Error writing archive header file");
    return E_FAIL;
    }

/*
  Return the status of the call to read_file() this is necessary
  to allow the calling pdf to detect TAPE_EOV or TAPE_EOT when
  using tape input.
----------------------------------------------------------------*/
return status;

}
