/******************************************************************************
FUNCTION:	READ_FILE

PURPOSE:	Convert the data from the input Level 1b image into the
		archive image format.

Version	Date	Programmer	Modifications
-------	-----	---------------	-----------------------------------------------
  1.0	01/92	T.Baltzer	Development
  1.1	05/93	J.Fenno		Modified to use the new AVHRR model and the new
				archio supports.
  2.0	11/97	D.Lloyd		Modified to make Level 1b support calls,
				eliminated most globals, combined with read_gac
				and renamed read_file.  Part of major INGEST1B
				rewrite.
  2.1	12/98	D.Lloyd		Modifications for NOAA KLM series sats.

PROJECT:	ADAPS
******************************************************************************/
#include "ingest1b.h"
#include "util2.h"
#include "adapsutil.h"

#define NODEBUG TRUE            /* set to FALSE when debugging using    */
                                /* dbx TRUE otherwise                   */

extern long curent;		/* current processing line		*/


int read_file
(
struct FDESC	*in_fd,		/* IN: input file descriptor		*/
long		*out_fd,	/* IN: output file descriptor		*/
long		window[],	/* IN: Input window		 	*/
long		inchan[],	/* IN: Flags for input bands	 	*/
long		seqchan[],      /* IN: sequential band numbers to proc.	*/
long		totchan,	/* IN: total number bands in the input	*/
long		goodlines,	/* IN: total number good lines in input	*/
long		*rec_size,	/* IN: Record size of data read		*/
unsigned char	*bscan,		/* IN: Pointer to Level 1b scan line	*/
short		*outbuf,	/* IN: Output buffer			*/
struct MODEL	*model,		/* IN: AVHRR model structure		*/
struct DROPPED	*drop		/* OUT: dropped line structure		*/
)
{
char    msg[ERRLEN];            /* Error message buffer                 */

long	drop_line = 0;		/* dropped lines before 1st line	*/
long	handle = HANDLE;	/* handle tape errors			*/
long	last_line;		/* last line to be read from input	*/
long	num_0_lines;		/* Number of dropped lines to zero fill	*/
long	nwords;			/* Number of 32 bit words to unpack	*/
long	one = 1;		/* value of one				*/
long	status;			/* function return status		*/
long	swap = FALSE;		/* swap bytes flag			*/
long	tot_0_lines = 0;	/* total number of dropped lines	*/
long	tot_lns_read = 0;	/* total number of lines read so far	*/

short	inbuf[LAC_SAMPS*MOSTBND];   /* unpacked input buffer		*/
short   pscanline;		    /* Previous scan line number	*/
short	*scanline = (short *)bscan; /* Scan line num from data header	*/

unsigned char   *dataaddr;      /* Data address, pointer to data        */


/*
  Point to the video data in the scan line
------------------------------------------*/
if ( model->satnum <= 14 )
    dataaddr = bscan + PREVIDEO;
else
    dataaddr = bscan + KLMPREVIDEO;


/*
  Calculate the number of I*2 words in the scan line
----------------------------------------------------*/
if ( model->dtype == GAC )
    nwords = GAC_SAMPS * totchan;
else
    nwords = LAC_SAMPS * totchan;


/*
  Skip to the proper location in the input file for the desired window
----------------------------------------------------------------------*/
if ( in_fd->ftype == L1BDISK )
    status = c_l1bseekd( in_fd->fd, window, rec_size, model, &drop_line );
else
    status = c_l1bseekt( &in_fd->tun, window, rec_size, model, in_fd->comment,
			 &in_fd->volnum, &in_fd->tape, &drop_line );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"ingest1b-seek","Error locating first line");
    return status;
    }


/*
  Read the first line
---------------------*/
if ( in_fd->ftype == L1BDISK )
    status = c_l1breadd( in_fd->fd, rec_size, bscan );
else
    status = c_l1breadt( &in_fd->tun, rec_size, in_fd->comment, &in_fd->volnum,
			&in_fd->tape, bscan);
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"ingest1b-read","Error reading first line");
    return status;
    }
tot_lns_read++;


/*
  Calculate one of the stopping conditions
------------------------------------------*/
last_line = *scanline + window[NL] - drop_line - 1;


/*
  Main processing loop
----------------------*/
curent = 1;     /* prepare for first processing message	*/
#if NODEBUG
c_startm();     /* start processing message		*/
#endif
curent = 0;     /* no lines really processed yet	*/
while ( TRUE ) 
    {
    /*
      PACKED DATA:
      The data samples read are in a packed format in which three 10-bit data
      samples are stored in a 4-byte word.
      UNPACKED DATA:
      The data samples read are in an unpacked format in which one 10-bit data
      sample is stored in a 2-byte word.

      The appropriate subroutine is called to byte swap and unpack the data
      samples into the i*2 buffer of inbuf.  Inbuf contains the data samples
      of the image line for all bands.  The data samples are interleaved,
      which means the first element belongs to band #1, the second element
      belongs to band #2, and so on.
    ------------------------------------------------------------------------*/
    if ( *rec_size == PCK_LAC_5 || *rec_size == PCK_GAC_5 ||
         *rec_size == KLM_PCK_LAC_5 || *rec_size == KLM_PCK_GAC_5 )
	c_unpck32( dataaddr, inbuf, &nwords, &swap );	/* packed data	 */
    else
	unpk_break( dataaddr, inbuf, &nwords);		/* unpacked data */

    /*
     Copy the unpacked input image data to the AVHRR archive file
    -------------------------------------------------------------*/
    status =  put_line( out_fd, bscan, inbuf, totchan, inchan, seqchan, model,
		   outbuf );
    if ( status != E_SUCC)
	{
	c_lasmsg(FALSE,"ingest1b-write","Error writing archive");
	break;
	}

    /*
      Read the next line
    --------------------*/
    pscanline = *scanline;
    if ( in_fd->ftype == L1BDISK )
	status = c_l1breadd( in_fd->fd, rec_size, bscan );
    else
	status = c_l1breadt( &in_fd->tun, rec_size, in_fd->comment,
			 &in_fd->volnum, &in_fd->tape, bscan);;

    /*
      See if we are done.  The stopping conditions are: the end of file
      is reached, end of tape or the ending line has been processed.
    -------------------------------------------------------------------*/
    if ( status == TAPE_EOV || status == TAPE_EOT )
	break;
    if ( status == E_EOF || status == TAPE_EOF )
	break;
    if ( *scanline > last_line )
	break;
    if ( status == E_FAIL )
        break;
    tot_lns_read++;

    /*
     Write dropped line(s) before writing data line
    -----------------------------------------------*/
    if ((num_0_lines = *scanline - pscanline - 1) > 0)
        {
	tot_0_lines += num_0_lines;
	zero_fill( out_fd, model, drop, num_0_lines, outbuf );
	}

    }

/*
  If the entire image was to be read, but the number of lines read differs
  from the number of lines stored in the data set header record, print a
  warning error message and adjust the model and window.
--------------------------------------------------------------------------*/
if (( window[NL] == model->nlines ) && (tot_lns_read != goodlines))
    {
    sprintf(msg,"%ld %s %ld %s",tot_lns_read,"lines read from tape,",goodlines,
	   "lines expected according to header");
    c_lasmsg(FALSE," ",msg);
    model->nlines = tot_lns_read + tot_0_lines;
    window[NL] = tot_lns_read + tot_0_lines;
    }

/*
  If using tape as input, insure that it is positioned
  at the end of the current file.
------------------------------------------------------*/
if ( in_fd->ftype == L1BTAPE && status != TAPE_EOF &&
	  status != TAPE_EOV && status != TAPE_EOT )
    status = c_tpfsf( &in_fd->tun, &one, &handle );

/*
  Return the status from either the c_tpfsf() the c_l1breadd() or the
  c_l1breadt() call.  This status is returned so that an end of tape or
  end of volume can be detected by the caller.
-----------------------------------------------------------------------*/
return status;

}
