/***************************************************************************
NAME      		  COMBINE_SEG

PURPOSE: Combine bad data segments based on user defined threshold percentage.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	Oct. 1991	Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS

PROJECT   	         ADAPS		                

LANGUAGE		  C	

ALGORITHM
  Assume the line below represents a line of data within the image.  There
are two bad segments of data in the line, a segment of length 'a', and a
segment of length 'b'.

xxxxxxxxxxxxxxxbbbbbbbbbbbbbbbbbbbbbxxxxxxxxxxxxxxxbbbbbbbbbbbbbbbbbxxxxxxxx
               <------ a ---------->               <------ b ------>
               <------------------------- c ----------------------->
<---------------------------------- d ------------------------------------->


   We will combine bad data segments based on the user defined threshold
 percentage 'THRPERC'.  Two criteria are used to combine bad segments:

(1)  If  (a + b) / c >= THRPERC  then combine a and b into one of length c.
     Check all ajoining pairs on the line, and combine if appropriate. 
     Then:

(2)  If  (a + b) / d >= THRPERC  then consider the entire line to be a bad
     data segment.  (a + b) here represents the sum of all bad data segments
     on the line, and d represents all of the pixels on the entire line.

Note that for criteria (1) nothing is done about the ends ( the good pixels
to the left of the leftmost bad pixel segment are left unchecked, the good
pixels to the right of the rightmost bad pixel segment are left unchecked).

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "linedrop.h"


FUNCTION combine_seg(ns, segment_count, thrperc)


   long		ns;		 /* Number of samples per line to be processed*/
   long 	*segment_count;  /* Number of bad segments in a line          */
   float         thrperc;        /* Threshold percentage                      */

{

float percent;		/* percentage of bad pixels */

long totbad;		/* total bad pixels in the line */

struct BAD_SEG *ptr;    /* pointer to a bad segment structure */


if (alph_seg)		/* not a null list, there are bad segments */
   {
   ptr = alph_seg;
   while (ptr->next)	/* while there is another bad segment to the right */
      {
      percent = (ptr->length + ptr->next->length) /
                (ptr->next->end_samp - ptr->start_samp + 1.0);
      if (percent >= thrperc)
         {
         ptr->end_samp = ptr->next->end_samp;		/* combine segments */
         ptr->length = ptr->end_samp - ptr->start_samp + 1;
         if ( delete_seg(ptr->next) == E_FAIL )
            return(E_FAIL);
         (*segment_count)--;
         }
      else
         ptr = ptr->next;	/* don't combine, step right one segment */
      }

   /* now check the entire line to see if it is bad */

   ptr = alph_seg;
   totbad = ptr->length;
   while (ptr->next)	/* while there is another bad segment to the right */
      {
      totbad += ptr->next->length;
      ptr = ptr->next;
      }
   percent = (float)totbad / (float)ns;
   if (percent >= thrperc)
      {
      while (alph_seg->next)	/* delete all but the first segment */
         if ( delete_seg(alph_seg->next) == E_FAIL )
            return(E_FAIL);
      alph_seg->start_samp = 1;
      alph_seg->length = alph_seg->end_samp = ns;
      *segment_count = 1;
      }

   }	/* end 'if' not a null list, there are bad segments */

return(E_SUCC);
}
