/*******************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the LINEDROP function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
K. Kimball	Aug. 1991	Updated to 5.0 standards
T. Baltzer	Oct. 1991	OUT parameter no longer needed
S. Nelson	Jun. 1992	Added option to fix dropped lines

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	       LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

NOTE: More than one input image is checked for here although this senario
      should never occur.  This coding was in existence, and is left as is
      in the event that more than one input image would be allowed some day.

*******************************************************************************/


#include <stdio.h>
#include "las.h"	
#include "linedrop.h"



FUNCTION getpar(vblock, fname, hosin, minmax, taein, thrperc, bands,
                kerndim, msgtim, nmrimg, nbands, npixels, thrval,
		totbnd, window, print, prtfile, drfile, fixlast, maxlfix)

struct  PARBLK    *vblock;      /* pointer to TAE block	 	              */

char     *fname;		/* output file name			      */
char     hosin[][CMLEN];  	/* host file names of input images	      */
char     *minmax;		/* min max flag buffer		              */
char     taein[][CMLEN];	/* TAE file names of input images	      */
char	 drfile[CMLEN];		/* Dropline input file name		      */

float    *thrperc;              /* Threshold percent indicating the percent of
                                   pixels in a line that must be good for the
                                   line to be valid.                          */

long     (*bands)[MAXBND+1];    /* band specification 		              */
long	 *fixlast;		/* Flag to fix the line after dropline	      */
long     *kerndim;              /* Length of a moving one dimensional window
                                   through the input image                    */
long	 *maxlfix;		/* Maximum number of droplines to fix	      */
long     *msgtim;		/* elapsed time between display messages      */
long     *nmrimg; 		/* number of input images		      */
long     *nbands;		/* number of bands per input image            */
long     *npixels;              /* Number of pixels around a pixel that are
                                   made bad when a bad pixel is found         */
long     *thrval;               /* Threshold value used to set a threshold 
                                   for the sum of the squared differences     */
long	 *totbnd;		/* total number of bands		      */
long     (*window)[4];	        /* window specification		              */
long 	 print[3];		/* output destination			      */
char 	 *prtfile;		/* print file name			      */


{
struct  VARIABLE  *p_find();     /* Function to find a TAE variable           */
struct  VARIABLE  *val;	      	 /* Pointer to TAE variable within v-block    */

long	 access;		 /* File i/o value			      */
long     element = 0;		 /* Parameter element number		      */
long     fatal = -1;		 /* Fatal error flag			      */
long     imglim = MAXIMG;	 /* Max number of images to be processed      */
long     nonfatal = 1;		 /* Nonfatal error flag			      */
long	 one = 1;		 /* The value of one	     		      */
long     qflag[MAXIMG][2];  	 /* Flag indicating if subwindows specified   */
long     same;		    	 /* Flag indicating if images have same window*/
long     status;	   	 /* Function return status code		      */
long	 zero = 0;		 /* Contains the value of zero		      */

char	 dropflg[4];		/* Flag to fix the line after the dropline    */


/* Retrieve the TAE global variables from the parameter block and store them
   in their respective program variables. 
-----------------------------------------------------------------------------*/

status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       			nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);

/* (don't check 'same' flag, only one input image is valid) */

if (*nmrimg != 1)
   {
   c_errmsg("Invalid number of input images specified","linedrop-nmrimg",
             							&nonfatal);
   c_errmsg("Fatal Error encountered","linedrop-fatal", &fatal);
   }

/* Get the parameter for dropline file
   ----------------------------------*/
access = IREAD;
status = c_getnam(vblock,"INFILE", &zero, drfile, "drpl", &access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);

/* Check for both windowing and non-null INFILE
   -------------------------------------------*/
if ((drfile[0] != NULL) && (qflag[0][0] == one))
   {
   c_errmsg("Improper use of windowing","linedrop-window",&nonfatal);
   c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
   }

/* Get the parameters
   ------------------*/

val = p_find(vblock, "thrperc");
*thrperc = RVAL(*val, 0);

/* Turn it into a fraction
   -----------------------*/

*thrperc = *thrperc / 100.0;


val = p_find(vblock, "thrval");
*thrval = IVAL(*val, 0);

val = p_find(vblock, "npixels");
*npixels = IVAL(*val, 0);

val = p_find(vblock, "kerndim");
*kerndim = IVAL(*val, 0);

if ((*npixels % 2)  !=  1  ||  (*kerndim % 2)  !=  1)
   {
   c_errmsg("KERNDIM, NPIXELS values must both be odd","lindrop-odd",&nonfatal);
   c_errmsg("Fatal Error encountered","linedrop-fatal", &fatal);
   }
else if((window[0][3] < *kerndim) || (window[0][3] < *npixels))
   {
   c_errmsg ("KERNDIM or NPIXELS larger than image or window","linedrop-large",
								&nonfatal);
   c_errmsg("Fatal Error encountered","linedrop-fatal", &fatal);
   }

/* Retrieve the TAE global variable for MAXLFIX
   -------------------------------------------*/
val = p_find(vblock, "MAXLFIX");
*maxlfix = IVAL(*val, 0);

/* Retrieve the TAE global variable for DROPFLAG and set new variable to
   one for yes and zero for no--this value will be used in FINDBAD
   --------------------------------------------------------------------*/
val = p_find(vblock, "DROPFLG");
strcpy(dropflg, SVAL(*val, 0));
if (strcmp(dropflg,"YES") == 0)
   *fixlast = one;
else
   *fixlast = zero;

/* get print option
-------------------*/
strcpy(prtfile,"linedrop");
status = c_setprt(vblock,&print[TERM],&print[PFILE],&print[LP],prtfile,
                  fname);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","dspddr-fatal",&fatal);


/* Retrieve the TAE global variables from the parameter block and store them
   in their respective program variables. 
-----------------------------------------------------------------------------*/

val = p_find(vblock, "$msgtim");
*msgtim = IVAL(*val, 0);

val = p_find(vblock, "$minmax");
strcpy(minmax, SVAL(*val,0));

/* Load structure values needed to fix the bad data blocks
   ------------------------------------------------------- */
fix.vblock = vblock;
strcpy(fix.minmax,minmax);
fix.msgtim = *msgtim;

return;
}
