/*****************************************************************************
NAME:   		       PROCES

PURPOSE    Identify and fix bad rectangular areas within an image. 
	   The actual heart of LINEDROP is performed by this code.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.1     K. Kimball       8/91           EDC           Modularizing LINEDROP
							update to 5.0 standards 
  5.2     T. Baltzer      10/91           EDC           Major mods to include
							updating an existing
							image, etc.
  5.3	  S. Nelson	   6/92		  EDC		Added option to fix
							dropped lines.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCES must be run under TAE.

PROJECT			LAS

ALGORITHM:


ALGORITHM REFERENCES		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "linedrop.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[];	        /* Image file descriptors used for image I/O */
extern int  gdesc;		/* Group descriptor used for image I/O       */

extern long curcat;		/* Current processing band		     */
extern long curent;		/* Current processing line		     */


FUNCTION proces(thrperc, bands, kerndim, nbands, nmrimg, npixels,
		dtype, thrval, window, hosin, lines, nblks, nbrln, fixlast)


   float        thrperc;        /* Threshold percent indicating the percent of
                                    pixels in a line that must be good for the
                                    line to be valid.                        */

   long		(*bands)[MAXBND+1];
                                /* Bands to be processed for all images      */
   long		fixlast;	/* Flag to fix the line after dropline	     */
   long     	*kerndim;       /* Length of a one line kernel moving 	     */
                                /* through the input image                   */
   long		*lines;		/* List of line numbers that start each
				    block of drop lines			     */
   long         *nbands;	/* Number of bands per input image           */
   long		nblks;		/* Number of blocks with drop lines          */
   long		*nbrln;		/* List of number of lines in each block
				   of drop lines			     */
   long 	*nmrimg;	/* Number of output images		     */
   long         *npixels;       /* Number of pixels that are considered bad  */
				/* when a bad pixel is found        	     */
   long 	dtype[];	/* Output data type			     */
   long         *thrval;        /* Threshold value used to set a threshold   */
                                /* for the sum of the squared differences    */
   long		window[MAXIMG][4]; /* Window specification for input images  */
   char 	hosin[][CMLEN];    /* Input image host name                  */

{

long    bndnum;			/* band number counter 			     */
long    fatal = -1;		/* Fatal error flag                          */
long 	image;			/* Iterates through all output images        */
long	nonfatal = 1;		/* Nonfatal error flag			     */
long	nl;			/* number of lines in current image	     */
long 	ns;  		        /* Number of samples in current image        */
long    one = 1;		/* Address containing value of one           */
long    segment_count;          /* Counts the number of bad segments in a 
                                   line                                      */
long 	size;		        /* Number of bytes per pixel                 */
long	status;			/* Function return status code               */

unsigned char *buffer1;    	/* Image input buffer                        */
unsigned char *buffer2;         /* Image input buffer                        */
unsigned char *thebuf;      	/* Image input buffer                        */

/*
   Process the image windows to be fixed.  Allocate buffers per image since
   the image or image window sizes may vary.
---------------------------------------------------------------------------*/

for (image = 0; image < *nmrimg; image++)
   {
   nl = window[image][NL];
   ns = window[image][NS];
   switch(dtype[image])
      {
      case EBYTE:       size = 1; break;
      case EWORD:       size = 2; break;
      case ELONG:       size = 4; break;
      case EREAL:       size = 4; break;
      }
   fix.dtype = dtype[image];	/* update structure used to fix bad data */
   fix.size = size;
   fix.window[0][SL] = window[image][SL];
   fix.window[0][SS] = window[image][SS];
   fix.window[0][NL] = window[image][NL];
   fix.window[0][NS] = window[image][NS];
   strcpy(fix.hostname[0],hosin[image]);


   if ((thebuf = (unsigned char *) malloc(ns * size)) == NULL)
      {
      closefile(*nmrimg);
      c_errmsg("Error allocating dynamic memory","linedrop-alloc",&nonfatal);
      c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
      }
   if ((buffer1 = (unsigned char *) malloc(ns * size)) == NULL)
      {
      closefile(*nmrimg);
      c_errmsg("Error allocating dynamic memory","linedrop-alloc",&nonfatal);
      c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
      }

   if ((buffer2 = (unsigned char *) malloc(ns * size)) == NULL)
      {
      closefile(*nmrimg);
      c_errmsg("Error allocating dynamic memory","linedrop-alloc",&nonfatal);
      c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
      }

   /* Set the number of bands for this image
      --------------------------------------*/

   for (bndnum = 0; bndnum < nbands[image]; bndnum++)
      {
      curcat = bands[image][bndnum];
      fix.bands[0][0] = bands[image][bndnum];

      /* Loop through reading a line at a time, find bad segments,
         and combine bad line segments into block of bad pixels
         ------------------------------------------------------*/

      for (curent = 1; curent <= nl; curent++)
         {
	 status = c_eread(&fdesc[image], &bands[image][bndnum], &curent,
                          				thebuf, &one);
	 if (status == E_FAIL)
	    {
	    closefile(image);
	    c_errmsg("Fatal error encountered", "linedrop-fatal", &fatal);
	    }

	 status = findbad(*kerndim, *npixels, ns, dtype[image], size,
                 &segment_count, *thrval, thebuf, lines, nblks, nbrln,fixlast);
	 if (status == E_FAIL)
	    {
	    closefile(image);
	    c_errmsg("Fatal error encountered", "linedrop-fatal", &fatal);
	    }

         /* If any bad segments were found then investigate combining them.
            Combine the bad line segments into bad data blocks.
	    Repair the bad data blocks that are found.
            --------------------------------------------------------------*/

         if (segment_count > 0) 
	    {
	    if ( combine_seg(ns,&segment_count,thrperc) == E_FAIL )
               {
               closefile(*nmrimg);
               c_errmsg("Fatal error encountered", "linedrop-fatal", &fatal);
               }
	    if (  blk_em_up(curent) == E_FAIL )
               {
               closefile(*nmrimg);
               c_errmsg("Fatal error encountered", "linedrop-fatal", &fatal);
               }
	    }
         }      /* end of line loop      */

      if ( clean_blk() == E_FAIL )	/* cleanup any outstanding data blocks */
         {
         closefile(*nmrimg);
         c_errmsg("Fatal error encountered", "linedrop-fatal", &fatal);
         }

      }     /* end of bndnum loop    */

   free(thebuf);
   free(buffer1);
   free(buffer2);

   }   /* end of image loop */

return;
}
