/*****************************************************************************
NAME:                            TERMINATE

PURPOSE   To update the ddr, close the image files, etc. 

PROGRAM HISTORY:
VERSION  PROGRAMMER  DATE   CODE/CONT  REASON
=======  ==========  =====  =========  =========================================
 5.1     J Fenno     08/91  EDC        Creation.
 5.2    T. Baltzer   11/91  EDC        Make modes to use in LINEDROP application
                                        (came from SEGMREPR application).


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TERMINATE must be run as part of SEGMREPR.

PROJECT:              LAS

ALGORITHM:

   NOTE:
   =====
      Due to the capability of updating input images, this code must ensure
      that min/max values are calculated correctly and that they are valid.

      When NOT using the update mode a new image is created and min/max values
      are not set.  Therefore, if the user requests that they be calculated you
      simply call c_minmax to do the calculations.

      When using update mode, one or more of the images may already have the
      min/max values calculated.  If the user does not request that the values
      be calculated we must insure that none of the images keep their old 
      values.  To do this simply call SVMNMX with the m_flag set to "NO" for
      each image.
      
      When the user requests that the min/max values be calculated in update
      mode we cannot just call c_minmax to calculate the new values.  In this
      case c_minmax simply retrieves the values that exist and these may be
      incorrect.  So in this case we also need to call SVMNMX with the m_flag
      set to "NO" for each input image to "erase" the current values and then
      call c_minmax to recalculate them.  The call to c_minmax will
      automatically update the ddr.


ALGORITHM REFERENCES:
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "segmrepr.h"
#include "linedrop.h"

/* The following variables must be declared global for image i/o routines
------------------------------------------------------------------------------*/
extern int  fdesc[];          /* Image file descriptors used for image I/O    */
extern long curcat;           /* Current processing band                      */
extern long curent;           /* Current processing line                      */

FUNCTION terminate(vblock, hostname, minmax, maxval, minval,
                   bands, nbands, nmrimg, update, window,
                   out_fd, taeout, print, fname)

struct PARBLK *vblock;           /* TAE parameter block                       */
char    hostname[][CMLEN];       /* Host names of images                      */
char    *minmax;                 /* Min max flag                              */

double  maxval[];                /* Max pixel value array                     */
double  minval[];                /* Min pixel value array                     */

long    bands[MAXIMG][MAXBND+1]; /* Bands to be processed for all images      */
long    nbands[MAXIMG];          /* # of bands per input image                */
long    nmrimg;                  /* # of input images                         */
long    update;                  /* flag to modify the input images           */
long    window[][NUM_SPECS];     /* Window specification of input images      */
FILE    *out_fd; 	         /* Output file descriptor used to record
                                    the blocks being fixed                    */
char 	taeout[][CMLEN];	 /* LAS name of output image                  */
long    print[3];                /* output destination                        */
char    *fname;       		 /* Output file name                          */



{
long    image;                   /* loop control                              */
long    nonfatal = 1;            /* Nonfatal error flag                       */
long    status;                  /* Function return status code               */
long    zero = 0;                /* address containing the value of zero      */
long    one = 1;	 	 /* Address containing value of one           */
long    two = 2;	 	 /* Address containing value of two           */
char 	filegen[2][CMLEN];	 /* output files generated                    */

/* Turn off display messages
----------------------------*/
c_stoptm();

if (update)
   for (image = 0; image < nmrimg; image++)
      {
      status = c_svmnmx(hostname[image], &nbands[image], minval, maxval, "NO");
      if (status != E_SUCC)
         c_errmsg("Nonfatal error encountered","linedrop-warn",&nonfatal);
      }

/* Calculate the new min/max values for each of the images
----------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   status = c_minmax(hostname, &nmrimg, window, bands, nbands, minval, maxval);
   if (status != E_SUCC)
      c_errmsg("Nonfatal error encountered","linedrop-warn",&nonfatal);
   }

/* Update associated history files
----------------------------------*/
for (image = 0; image < nmrimg; image++)
   if (c_hstout(vblock, hostname[image], "", &zero, "") != E_SUCC)
      c_errmsg("Nonfatal error encountered", "linedrop-warn",&nonfatal);

/* Close the print file
------------------------*/

if (fclose(out_fd) != 0)
   {
   c_errmsg("Unable to close output file", "linedrop-close",&nonfatal);
   c_errmsg("Nonfatal error encountered", "linedrop-warn", &nonfatal);
   }


/* Close all of the image files
-------------------------------*/
closefile(nmrimg);


/* Inform user of successful completion
   ------------------------------------*/

strcpy(&filegen[0][0],taeout[0]);
if (print[PFILE])
   {
   strcpy(&filegen[1][0],fname);
   c_complt("LINEDROP", filegen, &two);
   }
else
   c_complt("LINEDROP", filegen, &one);

return;
}
