/*******************************************************************************
FUNCTION:	check_pixels

PURPOSE:   	Check the pixel values on the current line.  The following five
		checks are made for each pixel.

		1. If the difference of the pixels lying above and below the
		   current pixel is "reasonable" continue with check #2
		   otherwise, assume the pixel is good.

		2. If the difference between the current pixel and the pixel
		   lying above it is "reasonable" assume the current pixel is
		   good.  Otherwise, the pixel is bad.

		3. If too many adjacent bad pixels have been found then the
		   whole line is bad.

		4. If a certain percentage of the pixels within a kernel window
		   are bad then the whole line is bad.

		5. If a certain percentage of the pixels on the entire line are
		   are bad then the whole line is bad.

*******************************************************************************/
#include "lndetect.h"

/* Global variables for controlling the line detection algorithm
----------------------------------------------------------------*/
extern double ADJACENT_PER;
extern double DIFF_PER[];
extern long   KERNEL_WIDTH[];
extern double KERNEL_PER[];
extern double LINE_PER;
extern double MEAN_PER[];
extern double SPOT_PER[];

extern long bands_det[]; /* Bands used for bad line detection                 */
extern long drop_adj[NCHANS];
extern long drop_badkernel[NCHANS];
extern long drop_total[NCHANS];


FUNCTION check_pixels (line, r, buffer)

    long line;			/* IN - Current line in the buffer            */
    long r;			/* IN - Number to add/subtract from line      */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
{
char str[CMLEN];	/* Temporary string                                   */

double diff_above_below;	/* Difference of pixels above and below       */
double diff_above_current;	/* Difference of pixel above and current      */
double mean;			/* Mean of all the pixel values on a line     */
double mean_above_below;	/* Mean of pixels above and below current     */
double var;			/* Variance of all pixel values on a line     */

long bad_adjacent;		/* # of adjacent bad pixels found on the line */
long bad_total;			/* Total number of bad pixels in current line */
long i;				/* Loop control                               */
long j;				/* Loop control                               */
long kernel_value;		/* Current value of the kernel                */
long max_bad_adjacent;		/* Max # of adjacent bad pixels               */
long max_bad_kernel;		/* Max # of bad pixels allowed in the kernel  */
long max_bad_line;		/* Max # of bad pixels allowed on a line      */
long nonfatal = 1;		/* Nonfatal error flag                        */

short *mnr;			/* Ptr to Minor Frame data                    */
short *pixel;			/* Ptr to the line of image data              */
short *px_above;		/* Ptr to line of pixels above current line   */
short *px_below;		/* Ptr to line of pixels below current line   */


/* Look at all of the bands to be used in the line detection algorithm
----------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    if (bands_det[i])
	{
	/* Compute the maximum number of bad adjacent pixels, the maximum
	   number of bad pixels allowed in the kernel, and the maximum number
	   of bad pixels in the current line.
	---------------------------------------------------------------------*/
	max_bad_adjacent = buffer->ns   * ADJACENT_PER;
	max_bad_kernel   = KERNEL_WIDTH[i] * KERNEL_PER[i];
	max_bad_line     = buffer->ns   * LINE_PER;


	/* Retrieve a pointer to the pixel values of the current line, the line
	   above the current line, and the line below the current line.
	----------------------------------------------------------------------*/
	get_line(line - r, bands_det[i], buffer, &mnr, &px_above, &mean, &var);
	get_line(line,     bands_det[i], buffer, &mnr, &pixel,    &mean, &var);
	get_line(line + r, bands_det[i], buffer, &mnr, &px_below, &mean, &var);


	/* Initialize the counters for the current line
	-----------------------------------------------*/
	bad_adjacent = 0;
	bad_total    = 0;
	kernel_value = 0;


	/* Iterate through each of the pixels for the current line
	----------------------------------------------------------*/
	for (j = 0; j < buffer->ns; j++, px_above++, pixel++, px_below++)
	    {
	    diff_above_below   = abs(*px_above - *px_below);
	    diff_above_current = abs(*px_above - *pixel);
	    mean_above_below   = (*px_above + *px_below) / 2.0;


	    /* If the pixel is zero then count it as bad data
	    -------------------------------------------------*/
	    if (*pixel == 0)
		{
		bad_adjacent += 1;
		bad_total    += 1;
		buffer->drop_pixel[j] = BAD;
		}
	    else if (diff_above_below > (MEAN_PER[i] * mean_above_below))
		{
		/* The large difference implies that one or both of the pixels
		   lying above and below the current pixel are bad.  Therefore,
		   the mean value of these two pixels is invalid and we cannot
		   rely upon their values to make a decision about the current
		   pixel.  Assume it is good.
		--------------------------------------------------------------*/
		bad_adjacent = 0;
		buffer->drop_pixel[j] = GOOD;
		}
	    else if (diff_above_current <= (DIFF_PER[i] * mean_above_below))
		{
		/* The difference between the current pixel and the pixel lying
		   above it is small enough to consider it a good value.
		--------------------------------------------------------------*/
		bad_adjacent = 0;
		buffer->drop_pixel[j] = GOOD;
		}
	    else
		{
		bad_adjacent += 1;
		bad_total    += 1;
		buffer->drop_pixel[j] = BAD;
		}


	    /* If too many bad pixels have been found adjacent to
	       one another then consider the current line bad.
	    -----------------------------------------------------*/
	    if (bad_adjacent >= max_bad_adjacent)
		{
		drop_adj[i] += 1;
		return(BAD);
		}


	    /* A kernel of KERNEL_WIDTH pixels is kept in the variable
	       kernel_value.  Since the bad pixels are marked with a one and
	       good pixels are marked with a zero, the sum of these values
	       tells us how many bad pixels are currently in the kernel.  By
	       subtracting the status of the KERNEL_WIDTH previous pixel the
	       kernel value is kept up to date.
	    ----------------------------------------------------------------*/
	    if (j >= KERNEL_WIDTH[i])
		kernel_value -= buffer->drop_pixel[j - KERNEL_WIDTH[i]];
	    kernel_value += buffer->drop_pixel[j];
	    if (kernel_value >= max_bad_kernel)
		{
		drop_badkernel[i] += 1;
		return(BAD);
		}


	    /* If the total number of bad pixels exceeds the maximum
	       allowed per line then return a bad status.
	    --------------------------------------------------------*/
	    if (bad_total >= max_bad_line)
		{
		drop_total[i] += 1;
		return(BAD);
		}
	    }
	}


/* The line has made it through all tests.  It is a good line.
--------------------------------------------------------------*/
return(GOOD);
}
