/*******************************************************************************
FUNCTION:	check_reset

PURPOSE:   	Once the entire image has been run through the line detection
		algorithm look at the dropped line flags to determine if a
		forced reset occurred.  A forced reset occurs when all of the
		previous lines currently held in the buffer are bad.  To reset
		the logic, the line detection algorithm considers the current
		line good and then resumes checking.  Therefore, if there are
		HALFLINES lines of bad data followed by a good line, followed
		by a bad line then the good line must be re-inspected.

*******************************************************************************/
#include "lndetect.h"

#define NLINES (7 * NBUFLINES)	/* Number of lines in the window              */
#define PERCENTAGE 0.25		/* Percent of variance allowed for the means  */

extern long curent;
extern long bands_det[];
extern long drop_reset[NCHANS];
extern char outfile[];		/* IN - Hostname of the output image          */

static int reset_condition();


FUNCTION check_reset (msgtim, buffer, model)

    long msgtim;		/* IN - Seconds between processing messages   */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
    struct MODEL *model;	/* IN - Model for the output image            */
{
char str[CMLEN];	/* Temporary working string                           */

double hi;		/* Highest acceptable mean value                      */
double lo;		/* Lowest acceptable mean value                       */
double mean;		/* Mean value of the line means                       */
double sum[NCHANS];	/* Sum of the mean values for NLINES lines            */

long access   = IUPDATE;/* Access code for the output image                   */
long adesc;		/* AVHRR archive descriptor record                    */
long band;		/* Loop control                                       */
long count;		/* Number of lines currently in the mean              */
long line;		/* Loop control                                       */
long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long msgfmt   = 3;	/* Processing message format                          */
long nonfatal = 1;	/* Nonfatal error flag                                */
long status;		/* Function return status                             */
long tpunit   = NULL;	/* Tape descriptor unit                               */
long window[4];		/* Window for the output image                        */
long zero     = 0;	/* Constant of zero                                   */

short *img;		/* Pointer to image data                              */
short *mnr;		/* Pointer to minor frame data                        */


/* Compute the sum of the line means for the first NLINES lines.  The sum must
   be computed for each band that is being used for line detection.  Do not
   include the means from lines that are considered bad.
------------------------------------------------------------------------------*/
count = 0;
for (band = 0; band < NCHANS; band++)
    sum[band] = 0;
for (line = 1; line <= NLINES; line++)
    {
    if (buffer->drop_line[line] == GOOD)
	{
	for (band = 0; band < NCHANS; band++)
	    if (bands_det[band])
		sum[band] += buffer->mean[band][line];
	count++;
	}
    }


/* Open the entire output image for updating
--------------------------------------------*/
window[SL] = 0;
status = c_arcopnrd(outfile, &access, window, model, &tpunit, &adesc);


/* Initialize and start a processing message
--------------------------------------------*/
curent = 1;
c_inittm("LNDETECT", "Reprocessed", "line", "", "for pass two", &buffer->nl,
    &zero, &msgtim, &msgfmt);
c_startm();


/* The lines that could be affected by a forced reset are
   HALFLINES + 1 through buffer->nl - HALFLINES.
---------------------------------------------------------*/
for (line = HALFLINES + 1; line <= (buffer->nl - HALFLINES); line++, curent++)
    {
    /* Compute 1/2 of the number of lines used in the window.  The number of
       lines in the window of mean values is odd because NBUFLINES must be odd.
       If the line number is greater than (NLINES/2 + 1) and less than
       (buffer->nl - NLINES/2) then update the sum of the mean values for each
       band.
    --------------------------------------------------------------------------*/
    i = NLINES / 2;
    if ((line > (i + 1)) && (line < (buffer->nl - i)))
	{
	/* Remove the first line from the window
	----------------------------------------*/
	if (buffer->drop_line[line - i] == GOOD)
	    {
	    for (band = 0; band < NCHANS; band++)
		if (bands_det[band])
		    sum[band] -= buffer->mean[band][line - i];
	    count--;
	    }
	

	/* Add the next line to the window
	----------------------------------*/
	if (buffer->drop_line[line + i] == GOOD)
	    {
	    for (band = 0; band < NCHANS; band++)
		if (bands_det[band])
		    sum[band] += buffer->mean[band][line + i];
	    count++;
	    }
	}


    /* Determine if current line was considered to be good from a forced reset
    --------------------------------------------------------------------------*/
    if (reset_condition(line, buffer))
	{
	for (band=0; (band < NCHANS)&&(buffer->drop_line[line] == GOOD); band++)
	    {
	    if (bands_det[band])
		{
		/* Compute the mean of the means for the window
		-----------------------------------------------*/
		mean = sum[band] / count;


		/* Compute the highest and lowest acceptable mean values
		--------------------------------------------------------*/
		hi = mean * (1.0 + PERCENTAGE);
		lo = mean * (1.0 - PERCENTAGE);


		/* If the mean for the current line is not within the
		   computed range then zero this line of data.
		-----------------------------------------------------*/
		if ((buffer->mean[band][line] > hi) ||
		    (buffer->mean[band][line] < lo))
		    {
		    /* Seek to the correct line in the output image
		    -----------------------------------------------*/
		    status = c_arcseek(&adesc, line);
		    if (status != E_SUCC)
			{
			c_arclose(outfile,&adesc);
			errout();
			}
		    

		    /* Read the specified line of data from the output image
		       and then zero the buffer that is returned from the
		       c_arcread() call.  The next call to this function or
		       c_arclose() will store the data back into the image.
		    --------------------------------------------------------*/
		    status = c_arcread(&adesc, &mnr, &img);
		    if (status != E_SUCC)
			{
			c_arclose(outfile,&adesc);
			errout();
			}
		    for (j = 0; j < (buffer->ns * NCHANS); j++)
			*(img++) = 0;
		    buffer->drop_line[line] = BAD;
		    drop_reset[band] += 1;
		    }
		}
	    }
	}
    }


/* Close the output image
------------------------------------*/
status = c_arclose(outfile,&adesc);
if (status != E_SUCC)
    errout();


/* Stop the processing message
------------------------------*/
curent = buffer->nl;
c_stoptm();


return(E_SUCC);
}




static int reset_condition (line, buffer)

    long line;			/* IN - Current line number                   */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
{
long i;				/* Loop control                               */


/* The current line could not be considered good by a forced reset condition
   if it is one of the first HALFLINES lines or the last HALFLINES lines in
   the image.
----------------------------------------------------------------------------*/
if ((line <= HALFLINES) || (line >= (buffer->nl - HALFLINES)))
    return(FALSE);


/* If current line is bad, it wasn't determined to be good by a forced reset
----------------------------------------------------------------------------*/
if (buffer->drop_line[line] == BAD)
    return(FALSE);


/* If any of the previous HALFLINES lines were good then the current
   line could not have been considered good by a forced reset.
--------------------------------------------------------------------*/
for (i = line - HALFLINES; i < line; i++)
    if (buffer->drop_line[i] == GOOD)
	return(FALSE);


/* If the next line is good, the current line wasn't
   determined to be good by a forced reset.
if (buffer->drop_line[line + 1] == GOOD)
    return(FALSE);
----------------------------------------------------*/


/* The current line was determined to be good by a forced reset
---------------------------------------------------------------*/
return(TRUE);
}
