/*******************************************************************************
FUNCTION:	compute_stats

PURPOSE:   	This function will compute the mean and variance for a line in
		each of the specified bands.

		While computing these values it is also necessary to check for
		zeroed pixels.  In some images a line may not have been dropped
		by the ingest procedure but yet is still contains a majority of
		bad or zeroed pixels.  If too many adjacent bad pixels have
		been found or, if a certain percentage of the pixels within a
		kernel window are bad, or if a certain percentage of the pixels
		on the entire line are are bad then the whole line is bad.

*******************************************************************************/
#include "lndetect.h"


extern double ADJACENT_PER;
extern long   KERNEL_WIDTH[];
extern double KERNEL_PER[];
extern double LINE_PER;

extern long bands_det[]; /* Bands used for bad line detection                 */
extern long drop_adj[NCHANS];
extern long drop_zerokernel[NCHANS];
extern long drop_total[NCHANS];




FUNCTION compute_stats (line, buffer)

    long line;			/* IN - Current line under examination        */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
{
double diff;			/* Difference between pixel value and mean    */
double mean;			/* Mean value for the line                    */
double total;			/* Temporary total value                      */
double variance;		/* Variance value for the line                */

long bad_adjacent;		/* # of adjacent bad pixels found on the line */
long bad_total;			/* Total number of bad pixels in current line */
long i;				/* Loop control                               */
long j;				/* Loop control                               */
long kernel_value;		/* Current value of the kernel                */
long max_bad_adjacent;		/* Max # of adjacent bad pixels               */
long max_bad_kernel;		/* Max # of bad pixels allowed in the kernel  */
long max_bad_line;		/* Max # of bad pixels allowed on a line      */

short lindex;			/* Line index                                 */
short *pixel;			/* Ptr to the pixels in a line of image data  */



/* Determine the index for the requested line
---------------------------------------------*/
lindex = (line - 1) % NBUFLINES;


/* For each of the bands to be used in line detection
-----------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    {
    if ((bands_det[i]) && (buffer->drop_line[line] == BAD))
	{
	buffer->mean[i][line] = 0.0;
	buffer->variance[i][line] = 0.0;
	}
    else if (bands_det[i])
	{
	/* Compute the maximum number of bad adjacent pixels, the maximum
	   number of bad pixels allowed in the kernel, and the maximum number
	   of bad pixels in the current line.
	---------------------------------------------------------------------*/
	max_bad_adjacent = buffer->ns   * ADJACENT_PER;
	max_bad_kernel   = KERNEL_WIDTH[i] * KERNEL_PER[i];
	max_bad_line     = buffer->ns   * LINE_PER;


	/* Initialize the counters for the current line
	-----------------------------------------------*/
	bad_adjacent = 0;
	bad_total    = 0;
	kernel_value = 0;
	total        = 0.0;


	/* Examine every pixel on the current line, keeping track of the sum
	   of all of the pixels and checking for too many zero values.
	--------------------------------------------------------------------*/
	pixel = buffer->img[lindex] + (i * buffer->ns);
	for (j = 0; j < buffer->ns; j++, pixel++)
	    {
	    /* If the pixel is zero then count it as bad data
	    -------------------------------------------------*/
	    if (*pixel == 0)
		{
		bad_adjacent += 1;
		bad_total    += 1;
		buffer->drop_pixel[j] = BAD;
		}
	    else
		{
		bad_adjacent = 0;
		buffer->drop_pixel[j] = GOOD;
		}


	    /* If too many bad pixels have been found adjacent to
	       one another then consider the current line bad.
	    -----------------------------------------------------*/
	    if (bad_adjacent >= max_bad_adjacent)
		{
		drop_adj[i] += 1;
		buffer->drop_line[line] = BAD;
		for (i = 0; i < NCHANS; i++)
		    {
		    if (bands_det[i])
			{
			buffer->mean[i][line] = 0.0;
			buffer->variance[i][line] = 0.0;
			}
		    }
		return(E_SUCC);
		}


	    /* A kernel of KERNEL_WIDTH pixels is kept in the variable
	       kernel_value.  Since the bad pixels are marked with a one and
	       good pixels are marked with a zero, the sum of these values
	       tells us how many bad pixels are currently in the kernel.  By
	       subtracting the status of the KERNEL_WIDTH previous pixel the
	       kernel value is kept up to date.
	    ----------------------------------------------------------------*/
	    if (j >= KERNEL_WIDTH[i])
		kernel_value -= buffer->drop_pixel[j - KERNEL_WIDTH[i]];
	    kernel_value += buffer->drop_pixel[j];
	    if (kernel_value >= max_bad_kernel)
		{
		drop_zerokernel[i] += 1;
		buffer->drop_line[line] = BAD;
		for (i = 0; i < NCHANS; i++)
		    {
		    if (bands_det[i])
			{
			buffer->mean[i][line] = 0.0;
			buffer->variance[i][line] = 0.0;
			}
		    }
		return(E_SUCC);
		}


	    /* If the total number of bad pixels exceeds the maximum
	       allowed per line then return a bad status.
	    --------------------------------------------------------*/
	    if (bad_total >= max_bad_line)
		{
		drop_total[i] += 1;
		buffer->drop_line[line] = BAD;
		for (i = 0; i < NCHANS; i++)
		    {
		    if (bands_det[i])
			{
			buffer->mean[i][line] = 0.0;
			buffer->variance[i][line] = 0.0;
			}
		    }
		return(E_SUCC);
		}


	    /* Add the pixel's value to the total
	    -------------------------------------*/
	    total += *pixel;
	    }


	/* Compute the mean for the line
	--------------------------------*/
	mean = total / (double)buffer->ns;


	/* Compute the variance for this line
	-------------------------------------*/
	total = 0.0;
	pixel = buffer->img[lindex] + (i * buffer->ns);
	for (j = 0; j < buffer->ns; j++)
	    {
	    diff = *(pixel++) - mean;
	    total += diff * diff;
	    }
	variance = total / (double)buffer->ns;

	buffer->mean[i][line] = mean;
	buffer->variance[i][line] = variance;
	}
    }

return(E_SUCC);
}
