/*******************************************************************************
FUNCTION:	getpar

PURPOSE:   	Retrieve the parameters from the variable block.

*******************************************************************************/
#include "lndetect.h"


/* Global variables used to control the line detection alogirthm
------------------------------------------------------------------------------*/
extern double ADJACENT_PER;
extern double DIFF_PER[NCHANS];
extern double THRESHOLD_MIN[];
extern double KERNEL_PER[NCHANS];
extern double LINE_PER;
extern double MEAN_PER[NCHANS];
extern double SPOT_PER[NCHANS];
extern long   KERNEL_WIDTH[NCHANS];


/* Global variables designating which bands are used in line detection
------------------------------------------------------------------------------*/
extern long bands_all[]; /* All bands in an AVHRR archive image               */
extern long bands_det[]; /* Bands used for bad line detection                 */
extern long bands_spt[]; /* Bands that should have spot noise zeroed          */
extern char infile[];		/* OUT - Host name of input archive image     */
extern char outfile[];		/* OUT - Host name for output image           */


FUNCTION getpar (vblock, tapeid, msgtim)

    struct PARBLK *vblock;	/* IN  - TAE parameter block                  */
    char *tapeid;		/* OUT - ID number of tape input image is on  */
    long *msgtim;		/* OUT - Seconds between processing messages  */
{
char str[CMLEN];	/* Temporary working string                           */
char dir[CMLEN];	/* Directory portion of a file name                   */
char ext[CMLEN];	/* Extension portion of a file name                   */
char root[CMLEN];	/* Root portion of a file name                        */

long access   = IREAD;	/* File access mode for the input image               */
long fatal    = -1;	/* Fatal error flag                                   */
long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long nonfatal =  1;	/* Nonfatal error flag                                */
long zero     =  0;	/* Constant of zero                                   */

struct VARIABLE *p_find(); /* Function to retrieve a TAE variable             */
struct VARIABLE *parm;     /* Pointer to TAE parameter within vblock          */


/* Determine if the input image is to be read from an AVHRR archive tape
------------------------------------------------------------------------*/
parm = p_find(vblock, "TAPEID");
if (parm->v_count == 0)
    strcpy(tapeid, "");
else
    strcpy(tapeid, SVAL(*parm,0));


/* Get the input image name.  If there is a tape ID then IN must be a 16
   character scene ID which specifies an image on an AVHRR archive tape.
------------------------------------------------------------------------*/
if (strlen(tapeid))
    {
    parm = p_find(vblock, "IN");
    strcpy(infile, SVAL(*parm,0));
    if ((strlen(infile) != 16) || (infile[0] != 'a') ||
	((infile[1] != 'g') && (infile[1] != 'h') &&
	 (infile[1] != 'l') && (infile[1] != 'o')))
	{
	c_errmsg("Invalid scene ID", "lndetect-sceneid", &nonfatal);
	c_errmsg("Fatal error encountered", "lndetect-fatal", &fatal);
	}
    }
else if (c_getnam(vblock, "IN", &zero, infile, "arch", &access) != E_SUCC)
    c_errmsg("Fatal error encountered", "lndetect-fatal", &fatal);


/* Get the output image name.  If the user didn't specify OUT then store the
   current working directory in outfile and open_images() will create the
   output file name.  If there is a name then check to see if it is just a
   directory.
----------------------------------------------------------------------------*/
parm = p_find(vblock, "OUT");
if (parm->v_count == 0)
    c_cmhost("", outfile);
else
    {
    c_cmhost(SVAL(*parm,0), str);
    c_splitname(str, dir, root, ext);
    if (strlen(root) == 0)
	strcpy(outfile, dir);
    else
	sprintf(outfile, "%s%s.arch", dir, root);
    }


/* Get the bands that are to be used in the line detection algorithm
--------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    bands_det[i] = 0;
parm = p_find(vblock, "BANDS");
for (i = 0; i < parm->v_count; i++)
    bands_det[IVAL(*parm,i) - 1] = IVAL(*parm,i);


/* Get the bands that should have spot noise zeroed
---------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    bands_spt[i] = 0;
parm = p_find(vblock, "SPOTBANDS");
for (i = 0; i < parm->v_count; i++)
    bands_spt[IVAL(*parm,i) - 1] = IVAL(*parm,i);


/* Set up an array that indicates those bands not being used in the line
   detection algorithm and one that indicates all NCHANS channels in an
   AVHRR archive image.
------------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    bands_all[i] = i + 1;


/* Retrieve the parameters used to control the line detection algorithm
-----------------------------------------------------------------------*/
parm = p_find(vblock, "ADJPERC");
ADJACENT_PER = RVAL(*parm,0) / 100.0;

parm = p_find(vblock, "LINEPERC");
LINE_PER = RVAL(*parm,0) / 100.0;

parm = p_find(vblock, "MEANPERC");
for (i = 0, j = 0; i < NCHANS; i++)
    {
    if (bands_det[i])
	{
	MEAN_PER[i] = RVAL(*parm,j) / 100.0;
	j++;
	}
    else
	MEAN_PER[i] = 0.0;
    }

parm = p_find(vblock, "DIFPERC");
for (i = 0, j = 0; i < NCHANS; i++)
    {
    if (bands_det[i])
	{
	DIFF_PER[i] = RVAL(*parm,j) / 100.0;
	j++;
	}
    else
	DIFF_PER[i] = 0.0;
    }

parm = p_find(vblock, "THRSMIN");
for (i = 0, j = 0; i < NCHANS; i++)
    {
    if (bands_det[i])
	{
	THRESHOLD_MIN[i] = RVAL(*parm,j);
	j++;
	}
    else
	THRESHOLD_MIN[i] = 0.0;
    }

parm = p_find(vblock, "SPOTPERC");
for (i = 0, j = 0; i < NCHANS; i++)
    {
    if (bands_spt[i])
	{
	SPOT_PER[i] = RVAL(*parm,j) / 100.0;
	j++;
	}
    else
	SPOT_PER[i] = 0.0;
    }

parm = p_find(vblock, "KERNPERC");
for (i = 0, j = 0; i < NCHANS; i++)
    {
    if (bands_det[i])
	{
	KERNEL_PER[i] = RVAL(*parm,j) / 100.0;
	j++;
	}
    else
	KERNEL_PER[i] = 0.0;
    }

parm = p_find(vblock, "KERNWDTH");
for (i = 0, j = 0; i < NCHANS; i++)
    {
    if (bands_det[i])
	{
	KERNEL_WIDTH[i] = IVAL(*parm,j);
	j++;
	}
    else
	KERNEL_WIDTH[i] = 0;
    }


/* Get the number of seconds between processing messages
--------------------------------------------------------*/
parm = p_find(vblock, "$MSGTIM");
*msgtim = IVAL(*parm,0);

return(E_SUCC);
}
