/*******************************************************************************
FUNCTION:	init_buffer

PURPOSE:   	Allocate the dynamic memory required for the line detection
		algorithm and load the buffer with the first NBUFLINES of image
		data.
		
		The first HALFLINES must be considered good data because there
		is no previous data to compare it to.  For this reason these
		lines are directly copied to the output image and when this
		function finishes the HALFLINES + 1 line from the input image
		will be considered the current line in the algorithm.

*******************************************************************************/
#include "lndetect.h"

extern long bands_all[]; /* All bands in an AVHRR archive image               */
extern long bands_det[]; /* Bands used for bad line detection                 */


FUNCTION init_buffer (buffer, drop, model)

    struct BUFFER *buffer;	/* IN/OUT - Buffer for line detection         */
    struct DROPPED *drop;	/* IN     - AVHRR dropped line structure      */
    struct MODEL *model;	/* IN     - AVHRR model structure             */
{
char str[CMLEN];	/* Temporary working string                           */

long dtype    = EWORD;	/* Data type of the AVHRR image data                  */
long fill     = TRUE;	/* Fill flag for c_bip2bil()                          */
long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long k;			/* Loop control                                       */
long nonfatal =  1;	/* Nonfatal error flag                                */
long status;		/* Function return status                             */

short *img;		/* Ptr to buffer of image data                        */
short *mnr;		/* Ptr to buffer of Minor Frame data                  */


/* Initialize all of the memory pointers to NULL.  This is done so the errout()
   function can determine which memory elements need to be freed if an error
   occurs.
------------------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    {
    buffer->mean[i] = NULL;
    buffer->variance[i] = NULL;
    }
buffer->bip = NULL;
buffer->drop_line = NULL;
buffer->drop_pixel = NULL;
for (i = 0; i < NBUFLINES; i++)
    {
    buffer->img[i] = NULL;
    buffer->mnr[i] = NULL;
    }
buffer->zero = NULL;


/* Store the number of lines and samples in the image
-----------------------------------------------------*/
buffer->nl = model->nlines;
buffer->ns = model->nsamples;


/* Allocate the buffers needed to store the mean and variance for each
   line of the image within the bands to be used for line detection.
----------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    {
    if (bands_det[i])
	{
	buffer->mean[i] = (double *)calloc(buffer->nl + 1, sizeof(double));
	if (buffer->mean[i] == NULL)
	    {
	    c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
	    errout();
	    }

	buffer->variance[i] = (double *)calloc(buffer->nl + 1, sizeof(double));
	if (buffer->variance[i] == NULL)
	    {
	    c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
	    errout();
	    }
	}
    }


/* Allocate one buffer to be used for converting the line detection buffer
   from BIL to BIP.  This will eliminate the necessity to allocate and
   deallocate memory each time through the loop when this conversion is
   performed.
-------------------------------------------------------------------------*/
buffer->bip = (short *)calloc(buffer->ns * NCHANS, sizeof(short));
if (buffer->bip == NULL)
    {
    c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
    errout();
    }


/* Initialize an array with one element for each line of the input image.  This
   array will be used to indicate if a line is dropped or bad.  A value of zero
   indicates a good line while a value of 1 indicates a dropped line.  Calloc()
   will set all elements to zero.
------------------------------------------------------------------------------*/
buffer->drop_line = (char *)calloc(buffer->nl + 1, sizeof(char));
if (buffer->drop_line == NULL)
    {
    c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
    errout();
    }
for (i = 0; i < drop->ngaps; i++)
    {
    j = drop->lmissed[i] + drop->nmissed[i] - 1;
    for (k = drop->lmissed[i]; k <= j; k++)
	buffer->drop_line[k] = BAD;
    }


/* Initialize an array with one element for each pixel of the input image.
   This array will be used to indicate if a pixel within the current line is
   bad.  A value of zero indicates a good pixel while a value of 1 indicates
   a bad pixel.  Calloc() will set all elements to zero.
------------------------------------------------------------------------------*/
buffer->drop_pixel = (char *)calloc(buffer->ns, sizeof(char));
if (buffer->drop_pixel == NULL)
    {
    c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
    errout();
    }


/* Allocate one Minor Frame and one BIL buffer
   for each line of the line detection buffer
----------------------------------------------*/
for (i = 0; i < NBUFLINES; i++)
    {
    buffer->mnr[i] = (short *)calloc(MNR_WORDS, sizeof(short));
    if (buffer->mnr[i] == NULL)
	{
	c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
	errout();
	}
    buffer->img[i] = (short *)calloc(buffer->ns * NCHANS, sizeof(short));
    if (buffer->img[i] == NULL)
	{
	c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
	errout();
	}
    }


/* Load the line detection buffer with the first NBUFLINES.  The first
   HALFLINES must be considered good data because there is no previous data to
   compare it to.  For this reason these lines are directly copied to the
   output image.
------------------------------------------------------------------------------*/
for (i = 0; i < NBUFLINES; i++)
    {
    /* Read the image data from the input file
    ------------------------------------------*/
    status = c_arcread(&buffer->indesc, &mnr, &img);
    if (status != E_SUCC)
	errout();


    /* Copy the Minor Frame data into the line detection buffer
    -----------------------------------------------------------*/
    k = MNR_WORDS;
    c_pxcopy(mnr, buffer->mnr[i], &dtype, &k);


    /* Convert the image data from BIP to BIL and copy it
       into the line detection buffer at the same time.
    -----------------------------------------------------*/
    status = c_bip2bil(img, buffer->img[i], &buffer->ns, bands_all, &fill);
    if (status != E_SUCC)
	errout();
    

    /* If this is one of the first HALFLINES lines then compute the necessary
       statistics and copy it directly to the output image.  We are not running
       the spot detection function on this first set of lines because the input
       buffer must contain the line prior to the line that is being spot
       detected.  In addition, the function advance_buf() is written such that
       it assumes the line detection buffer is full.
    --------------------------------------------------------------------------*/
    if (i < HALFLINES)
	{
	compute_stats (i+1, buffer);
	status = c_arcwrite(&buffer->outdesc, mnr, img);
	if (status != E_SUCC)
	    errout();
	}
    }


/* Allocate a buffer of zero'd BIP data.  This buffer will be used so that the
   actual image data does not have to be zero'd and it can be used in the
   algorithm when checking the next lines of the image.
------------------------------------------------------------------------------*/
buffer->zero = (short *)calloc(buffer->ns * NCHANS, sizeof(short));
if (buffer->zero == NULL)
    {
    c_errmsg("Error allocating memory", "lndetect-alloc", &nonfatal);
    errout();
    }


return(E_SUCC);
}
