/*******************************************************************************
FUNCTION:	lndetect

PURPOSE:   	Detect bad pixel segments in an AVHRR archive image and replace
		the lines with dropped lines.  The idea is not to fix bad data
		by "inventing" new data but to throw out the bad data so that
		it does not corrupt the products that are generated.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  0.1    08/93  J.Dewald         Algorithm development
  1.0    08/93  J.Fenno          Coding of algorithm for ADAPS
  1.1    05/94  J.Fenno          Removed the printing of the statistics to a
				 file.  The statistics are now only printed to
				 standard output.
  1.2    04/95  J.Fenno          Modified so the user specifies which bands will
				 have spot noise zeroed.
  1.3    07/96  T. Baltzer       Modified to implement DATAPATH solely in the
				 supports c_arcopnwrt() and c_arclose().
  1.4    10/96  T. Baltzer       Modified to zero out minor frame data for a
				 drop line.

PROJECT:	ADAPS 
*******************************************************************************/
#include "lndetect.h"


/* Number of adjacent, bad pixels that will trigger a bad line, expressed as a
   percentage of the total number of pixels in a line.
------------------------------------------------------------------------------*/
double ADJACENT_PER;


/* If the difference between the current pixel and the pixel lying above it is
   more than DIFF_PER percent of the mean value of the pixels lying above and
   below the current pixel, the current pixel is assumed to be bad.  There is
   a different value for each band of AVHRR data.
------------------------------------------------------------------------------*/
double DIFF_PER[NCHANS];


/* Threshold minimum values used in check_good() and check_bad()
----------------------------------------------------------------*/
double THRESHOLD_MIN[NCHANS];


/* Values for the number of pixels to be contained in a kernel and the
   percentage of these pixels that will trigger a bad line.  If more than
   KERNEL_PER percent of the pixels in a kernel are bad, the entire line is
   assumed to be bad.  There is a different value for each band of AVHRR data.
------------------------------------------------------------------------------*/
long   KERNEL_WIDTH[NCHANS];
double KERNEL_PER[NCHANS];


/* Number of bad pixels found within a line that will trigger a bad line,
   expressed as a percentage of the total number of pixels in a line.
------------------------------------------------------------------------------*/
double LINE_PER;


/* If the difference of the pixels lying above and below the current pixel is
   more than MEAN_PER percent of their mean value, it is assumed that one or
   both of the pixels are bad.  Therefore, no comparison will be made between
   the current pixel and these pixels.  There is a different value for each
   band of AVHRR data.
------------------------------------------------------------------------------*/
double MEAN_PER[NCHANS];


/* If the value of a pixel varies by more than SPOT_PER percent from the mean
   of the pixels lying to its left and right, the pixel will be zeroed as spot
   noise.  There is a different value for each band of AVHRR data.
------------------------------------------------------------------------------*/
double SPOT_PER[NCHANS];




/* Global variables to allow easy calling of the error handling routine
------------------------------------------------------------------------------*/
char infile[CMLEN];	/* Host name of the input image                       */
char outfile[CMLEN];	/* Host name of the output image                      */
char outhost[2][CMLEN];	/* Host name of the files created by this function    */
char outlas[2][CMLEN];	/* Host name of the files created by this function    */

long bands_all[NCHANS];	/* All bands in an AVHRR archive image                */
long bands_det[NCHANS];	/* Bands used for bad line detection                  */
long bands_spt[NCHANS];	/* Bands that should have spot noise zeroed           */
long drop_good[NCHANS];	/* Lines dropped by the check_good function           */
long drop_bad[NCHANS];	/* Lines dropped by the check_bad function            */
long drop_adj[NCHANS];	/* Lines dropped by the adjacent test                 */
long drop_badkernel[NCHANS];/* Lines dropped by bad pixels in the kernel      */
long drop_zerokernel[NCHANS];/* Lines dropped by zero pixels in the kernel    */
long drop_total[NCHANS];/* Lines dropped by the total test                    */
long drop_reset[NCHANS];/* Lines dropped by the forced reset condition        */
long drop_pixels[NCHANS];/* Number of pixels this function zero'd             */
long nfiles = 0;	/* Number of files created by this function           */

struct DROPPED drop;	/* AVHRR dropped line structure                       */
struct BUFFER bbuffer;   /* Buffer for input image                             */
struct MODEL model;	/* AVHRR model structure                              */
struct PARBLK vblock;	/* TAE parameter block                                */


main()
{
char str[CMLEN];	/* Temporary working string                           */
char tapeid[7];		/* Tape ID number if the input image is on tape       */

long fatal    = -1;	/* Fatal error flag                                   */
long i;			/* Loop control                                       */
long msgfmt   =  3;	/* Format of the processing message                   */
long msgtim;		/* Elapsed time between processing messages           */
long nonfatal =  1;	/* Nonfatal error flag                                */
long nskip;		/* Number of lines to be skipped                      */
long status;		/* Function return status                             */
long std_out;		/* LUN to standard output                             */
long total;		/* Total number of lines dropped                      */
long tpunit;            /* Tape drive unit if the image is on tape            */
long zero     =  0;	/* Value of zero for passing to functions             */

extern long curent;	/* Current processing line                            */

struct ARCDESC x;


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* The following two checks are to ensure programmers have the defines set up
   appropriately.  These error messages will not show up in the user's guide.
-----------------------------------------------------------------------------*/
if ((NBUFLINES % 2) == 0)
    {
    c_errmsg("Number of buffer lines is even", "lndetect-even", &nonfatal);
    c_errmsg("Fatal error encountered", "lndetect-fatal", &fatal);
    }
if ((NBUFLINES / 2) != HALFLINES)
    {
    sprintf(str, "HALFLINES must be %d", (NBUFLINES / 2));
    c_errmsg(str, "lndetect-half", &nonfatal);
    c_errmsg("Fatal error encountered", "lndetect-fatal", &fatal);
    }


/* Zero the counters used for keeping track of the dropped lines and pixels
---------------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    {
    drop_good[i] = 0;
    drop_bad[i] = 0;
    drop_adj[i] = 0;
    drop_badkernel[i] = 0;
    drop_zerokernel[i] = 0;
    drop_total[i] = 0;
    drop_reset[i] = 0;
    drop_pixels[i] = 0;
    }


/* Get the parameters
---------------------*/
getpar(&vblock, tapeid, &msgtim);


/* Open the input and output images
-----------------------------------*/
open_images(tapeid, outhost, outlas, &nfiles, &bbuffer, &drop, &model);


/* Allocate and initialize the line detection buffer
----------------------------------------------------*/
init_buffer(&bbuffer, &drop, &model);


/* Initialize processing message utility 
----------------------------------------*/
curent = 1;
c_inittm("LNDETECT", "Processed", "line", "", "for pass one", &model.nlines,
    &zero, &msgtim, &msgfmt);
c_startm();


/* Run the bad line detection functions on each line of the input image.  The
   way the buffering is set up, the detection functions are not run on the
   first HALFLINES or last HALFLINES of the image.
-----------------------------------------------------------------------------*/
for (curent = HALFLINES + 1; curent <= model.nlines - HALFLINES; curent++)
    {
    /* If the current line is not a bad line (dropped lines from the ingest
       routine are treated just like bad lines) then run the spot detection
       function on each of the bands specified by the user.
    -----------------------------------------------------------------------*/
    if (bbuffer.drop_line[curent] == GOOD)
	spot_detect(curent, bands_spt, &bbuffer);


    /* Compute the necessary statistics for this line
    -------------------------------------------------*/
    compute_stats(curent, &bbuffer);


    /* Examine the current line with respect to the other lines in the line
       detection buffer.  If the line detection algorithm needs to reset
       itself, examine_line() will return the number of lines that should be
       skipped.
    -------------------------------------------------------------------------*/
    nskip = examine_line(curent, &bbuffer);
    if (nskip)
	{
	/* The current line has been dropped and the line detection algorithm
	   is resetting itself by skipping 'nskip' lines.  For each of these
	   lines, if the line is not bad then the spot detection algorithm must
	   be run on the line.  The statistics must be computed for every
	   line.
	----------------------------------------------------------------------*/
	for (i = 0; i < nskip; i++)
	    {
	    advance_buf(curent, &bbuffer);
	    curent++;
	    if (bbuffer.drop_line[curent] == GOOD)
		spot_detect(curent, bands_spt, &bbuffer);
	    compute_stats(curent, &bbuffer);
	    }
	}


    /* Advance the line detection buffer forward one line
    -----------------------------------------------------*/
    advance_buf(curent, &bbuffer);
    }


/* Flush the line detection buffer
----------------------------------*/
flush_buf(curent, &bbuffer);
curent = model.nlines;


/* Close both the input and output images
-----------------------------------------*/
status = c_arclose(infile,&bbuffer.indesc);
if (status != E_SUCC)
    errout();
bbuffer.indesc = NULL;
status = c_arclose(outfile,&bbuffer.outdesc);
if (status != E_SUCC)
    errout();
bbuffer.outdesc = NULL;


/* Stop the processing message
------------------------------*/
c_stoptm();


/* Check for lines that were marked as good by a forced reset of the algorithm
------------------------------------------------------------------------------*/
check_reset(msgtim, &bbuffer, &model);


/* Save the header file for the output image
--------------------------------------------*/
save_ahdr(&bbuffer, &model, &drop);


/* Free all of the dynamically allocated memory
-----------------------------------------------*/
free_buf();


/* Issue messages about the number of pixels and lines dropped
--------------------------------------------------------------*/
print_stats();


/* Output completion message 
----------------------------*/
c_complt("LNDETECT", outlas, &nfiles);
}
