/*******************************************************************************
FUNCTION:	spot_detect

PURPOSE:   	This function will look for local pixel variations that exceed
		SPOT_PER percent of the surrounding pixels.  Pixels that exceed
		this variation will be zeroed.

		This algorithm works well for isolated noise; however, in
		extreme clustering situations the surrounding bad data may
		corrupt valid data.

		This function should be called prior to the other line
		detection functions.  By zeroing suspicious pixels we are
		increasing the line to line variance.  If enough bad pixels are
		found on a line the other line detection routines will drop the
		entire line.

*******************************************************************************/
#include "lndetect.h"

extern double SPOT_PER[];
extern long drop_pixels[NCHANS]; /* Number of pixels this function zero'd     */


FUNCTION spot_detect (line, spot_bands, buffer)

    long line;			/* IN - Current line under examination        */
    long spot_bands[];		/* IN - Bands to run through spot detection   */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
{
char str[CMLEN];	/* Temporary working string                           */

double hi_percent;	/* Value of 1.0 + SPOT_PER                            */
double lo_percent;	/* Value of 1.0 - SPOT_PER                            */
double mean_line;	/* Mean pixel value for a line                        */
double variance;	/* Variance in pixel values for a line                */

long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long nonfatal =  1;	/* Nonfatal error flag                                */

short hi_above;		/* Upper limit of a valid pixel value                 */
short hi_across;	/* Upper limit of a valid pixel value                 */
short last_pixel;	/* Value of pixel to the left of the current pixel    */
short lo_above;		/* Lower limit of a valid pixel value                 */
short lo_across;	/* Lower limit of a valid pixel value                 */
short mean_pixel;	/* Mean value of pixels on either side of current pix */
short *mnr;		/* Ptr to the Minor Frame data of a line              */
short *pixel;		/* Pixel value in the current line                    */
short *pixel_above;	/* Pixel value from line before the current line      */



/* For each of the specified bands to be run through spot detection
-------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    {
    if (spot_bands[i])
	{
	/* Set up the percentages for upper and lower pixel values
	----------------------------------------------------------*/
	hi_percent = 1.0 + SPOT_PER[i];
	lo_percent = 1.0 - SPOT_PER[i];


	/* Set a ptr to the first pixel in the current line and the line above
	----------------------------------------------------------------------*/
	get_line(line-1, i+1, buffer, &mnr, &pixel_above, &mean_line,&variance);
	get_line(line, i+1, buffer, &mnr, &pixel, &mean_line, &variance);


	/* The spot detection algorithm makes two tests for each pixel to
	   determine if it is bad or not.  The first comparison is to see if
	   the current pixel is SPOT_PER percent above or below the mean of the
	   pixels immediately to its left and right.  If the pixel is good then
	   it is compared to the pixel lying immediately above.  If the pixel
	   is determined to be bad then it is zeroed.  We do not want to use
	   the zero in the comparison for the next pixel because a zero will
	   bias the mean too heavily.  Therefore, store the pixel's original
	   value in the variable last_pixel and use this in the comparison.
	   Note that the leftmost and rightmost pixels on a line can only be
	   compared to the pixels lying above them.
	----------------------------------------------------------------------*/
	last_pixel = *pixel;


	/* If the pixel above the first pixel is not zero
	   then check the first pixel against the it.
	-------------------------------------------------*/
	if (*pixel_above)
	    {
	    hi_above = *pixel_above * hi_percent;
	    lo_above = *pixel_above * lo_percent;
	    if ((*pixel > hi_above) || (*pixel < lo_above))
		{
		drop_pixels[i]++;
		*pixel = 0;
		}
	    }


	/* Advance the pixel pointers
	-----------------------------*/
	pixel++;
	pixel_above++;


	/* Check the second through the next to the last pixel values
	-------------------------------------------------------------*/
	for (j = 1; j < (buffer->ns - 1); j++, pixel++, pixel_above++)
	    {
	    /* Compute the acceptable high and low values for
	       the current pixel with respect to the mean of
	       the pixels lying on the left and right.
	    -------------------------------------------------*/
	    mean_pixel = (last_pixel + *(pixel + 1)) / 2;
	    hi_across = mean_pixel * hi_percent;
	    lo_across = mean_pixel * lo_percent;


	    /* Store the current pixel's original value
	    -------------------------------------------*/
	    last_pixel = *pixel;


	    /* If the pixel lying above the current pixel is non-zero then
	       include it in the comparison otherwise, exclude it.
	    --------------------------------------------------------------*/
	    if (*pixel_above)
		{
		/* Compute the acceptable high and low values for the
		   current pixel with respect to the pixel lying above
		------------------------------------------------------*/
		hi_above  = *pixel_above * hi_percent;
		lo_above  = *pixel_above * lo_percent;

		/* Do the comparisons and zero the current pixel if necessary
		-------------------------------------------------------------*/
		if (((*pixel > hi_above) && (*pixel > hi_across)) ||
		    ((*pixel < lo_above) && (*pixel < lo_across)))
		    {
		    drop_pixels[i]++;
		    *pixel = 0;
		    }
		}
	    else
		{
		/* Do the comparisons and zero the current pixel if necessary
		-------------------------------------------------------------*/
		if ((*pixel > hi_across) || (*pixel < lo_across))
		    {
		    drop_pixels[i]++;
		    *pixel = 0;
		    }
		}
	    }


	/* If the pixel above the last pixel is not zero
	   then check the last pixel against the it.
	------------------------------------------------*/
	if (*pixel_above)
	    {
	    hi_above = *pixel_above * hi_percent;
	    lo_above = *pixel_above * lo_percent;
	    if ((*pixel > hi_above) || (*pixel < lo_above))
		{
		drop_pixels[i]++;
		*pixel = 0;
		}
	    }
	}
    }


return(E_SUCC);
}
