/*******************************************************************************
NAME:           add_scene

PURPOSE:        Add the input scene to the output half-orbit scene
		terminology:
		 "gap" means space in the output image that is not filled by 
			the input image
		 "overlap" means space already in output image that could be 
			filled in by input image

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    ------          ------
  2.0           8/92    T.Rockvam	Original Development

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "acqio.h"
#include "makorb.h"
#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

FUNCTION add_scene(in_fd,out_fd,in_pass,out_pass)

long 	*in_fd,			/* input image file descriptor		*/
	*out_fd;		/* output image file descriptor		*/

struct ORBIT	*in_pass,	/* input image information		*/
		*out_pass;	/* output image information		*/
{
extern long      curent;        /* current line being processed         */

long	in_overlap_start,	/* input line of overlap start		*/
	in_overlap_end,		/* input line of overlap end		*/
	out_overlap_start,	/* output line of overlap start		*/
	out_overlap_end,	/* output line of overlap end		*/
	in_line,		/* line in input image			*/
	out_line,		/* line in output image			*/
	line_offset,		/* line # offset from input to output image */
	dtype = EWORD,          /* type of data being processed		*/
	maxsam = MAX_ACQ,       /* maximum number of samples       	*/
	status,			/* function return status		*/
	zero = 0;               /* constant zero			*/

static short *mnrbuf,		/* minor frame data			*/
	*imgbuf,		/* image data				*/
	zerobuf[MAX_ACQ];  	/* fill for missing lines		*/

long 	inpri,
	outpri;
/*
  Get the priorities of the orbit & new pass
*/
getpri(in_pass,&inpri);
getpri(out_pass,&outpri);

/*
  Fill a dropped line
*/
c_pxfill(zerobuf,&dtype,&maxsam,&zero);

/*
  Find the line numbers in the output scene of the overlap
  (line in output where input image starts and end of output image)
  and the line numbers in the input scene of the overlap
  (start of input image and line in input where output ends)
*/
in_overlap_start = 1;
out_overlap_start = time_line(&out_pass->start_mil,in_pass->start_mil);
if (out_pass->end_mil == -1)
    {
    /* set out_pass end time to same as in_pass end time */
    out_overlap_end = time_line(&out_pass->end_mil,in_pass->end_mil);
    out_pass->model.nlines = 0;
    in_overlap_end = in_overlap_start - 1;
    }
else
    {
    /* use (end_mil - 166) to represent the STARTING time of the last line */
    out_overlap_end = time_line(&out_pass->start_mil,(in_pass->end_mil-166));
    if (out_overlap_end > out_pass->model.nlines)
        out_overlap_end = out_pass->model.nlines;
    in_overlap_end = time_line(&in_pass->start_mil,(out_pass->end_mil-166));
    }

if (in_overlap_end > in_pass->model.nlines)
    in_overlap_end = in_pass->model.nlines;

/*
  Calculate difference in input/output lines
*/
line_offset = out_overlap_start - in_overlap_start;

/*
  Fill in any gaps (if input image starts after end of existing output image)
  ** out_overlap_start is the line in the output image where the 
  input image starts, even if it is beyond the number of lines 
  in the output image
  Update nl of output image and output dropped lines info
*/
if (out_overlap_start > out_overlap_end)
    {
    c_arcseek(out_fd,out_pass->model.nlines+1);
    status = fill_missing(out_fd,&out_pass->drop,out_overlap_end+1,
	out_overlap_start-out_overlap_end-1,zerobuf);
    if (status != E_SUCC)
	return(E_FAIL);
    out_pass->model.nlines += out_overlap_start-out_overlap_end-1;
    curent += out_overlap_start-out_overlap_end-1;
    }

/*
  Now extend output end time; 
  it will not be used until next scene is added
*/
if (in_pass->end_mil > out_pass->end_mil)
        out_pass->end_mil = in_pass->end_mil;

/*
  Coming in, the output file pointer may be beyond the start 
  of the overlap area; must reset back to the beginning 
  of the overlap area
*/
c_arcseek(out_fd,out_overlap_start);
curent = out_overlap_start;

/*
  For the OVERLAP area we must check the priorities:
    if outpri > inpri  we only copy new lines over if the old one was dropped
    if inpri >= outpri we copy the new one over the old unless new is dropped
*/
if (outpri > inpri) /* only replace if dropped */
    {
    for (in_line = in_overlap_start; in_line <= in_overlap_end; in_line++) 
    	{
    	status = c_arcread(in_fd,&mnrbuf,&imgbuf);
    	if (status == E_EOF)
	    break;
        else if (status != E_SUCC)
 	    return(E_FAIL);
        out_line = in_line + line_offset;
	/* check if old line is dropped */
        if (is_dropped(out_line,&out_pass->drop))
	    {
	    /* check if input line is also dropped */
	    if (is_dropped(in_line,&in_pass->drop))
		{
	    	/* no good replacement - go on to next output line */
	    	c_arcseek(out_fd,out_line + 1);
		}
	    else
		{
	    	/* replace it if input line isn't dropped */
	        status = c_arcwrite(out_fd,mnrbuf,imgbuf);
	        if (status == E_FAIL)
	            return(E_FAIL);
	    	/* subtract this line number from the dropped line list */
	        drop_del(out_line,&out_pass->drop);
		}
	    }
        else
	    {
	    /* old input lines is fine; */
	    /* do not replace it & go to next output line 	*/
	    c_arcseek(out_fd,out_line + 1);
	    }
        curent++;
        }
    }
else /* inpri >= outpri; usually the default; replace old lines with new */
    {
    for (in_line = in_overlap_start; in_line <= in_overlap_end; in_line++) 
    	{
    	status = c_arcread(in_fd,&mnrbuf,&imgbuf);
    	if (status == E_EOF)
	    break;
        else if (status != E_SUCC)
 	    return(E_FAIL);
        out_line = in_line + line_offset;
        if (is_dropped(in_line,&in_pass->drop))
	    {
	    /* new input lines is dropped; */
	    /* do not replace it & go to next output line 	*/
	    c_arcseek(out_fd,out_line + 1);
	    }
        else
	    {
	    /* replace input line into output image */
	    status = c_arcwrite(out_fd,mnrbuf,imgbuf);
	    if (status == E_FAIL)
	        return(E_FAIL);
	    /* subtract this line number from the dropped line list */
	    if (is_dropped(out_line,&out_pass->drop))
	        drop_del(out_line,&out_pass->drop);
	    }
        curent++;
        }
    }

/*
  Copy remainder of the input image to the output image,
  dropped lines and all
*/
if (in_overlap_end < 0)
    in_overlap_end = 0;
for (in_line = in_overlap_end+1; in_line <= in_pass->model.nlines; in_line++)
    {
    status = c_arcread(in_fd,&mnrbuf,&imgbuf);
    if (status == E_EOF)
	break;
    else if (status != E_SUCC)
 	return(E_FAIL);
    status = c_arcwrite(out_fd,mnrbuf,imgbuf);
    if (status == E_FAIL)
 	return(E_FAIL);
    out_pass->model.nlines++;
    curent++;
    }
if (in_overlap_end < in_pass->model.nlines)
    {
    /* add dropped line info from input file to output file */
    drop_add(&in_pass->drop,&out_pass->drop,in_overlap_end+1,line_offset);
    }

/*
  Make sure the output dropped line structure is in order
*/
drop_sort(&out_pass->drop);

/*
  Update the orbit with the last input CEOS id
*/
strcpy(out_pass->model.station,in_pass->model.station);

return(E_SUCC);
}
