/*******************************************************************************
NAME:		GETPAR

PURPOSE:	Get the user specified parameters.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		REASON
-------		----	------		------
  2.0		8/92	D. Hollaren	Original Development
  2.1		7/94	T. Baltzer	Add spanning date logic

*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "adaps.h"
#include "makorb.h"

FUNCTION getpar(vblock,in_pass,nfiles,hosout,exists,out_model,msgtim)

struct PARBLK	*vblock;	/* TAE parameter block			      */
struct ORBIT	in_pass[];	/* start and stop times of the passes         */
struct MODEL	*out_model;	/* the output model and orbital pass info     */

char		*hosout;	/* name of output host name		      */

long		*nfiles,	/* number of files to process		      */
		*exists,	/* indicates if the orbital pass exists       */
		*msgtim;	/* time between processing messages	      */
{
struct VARIABLE *p_find(),	/* function to retrieve a TAE variable	      */
		*val;		/* pointer to TAE variable with vblock	      */

struct tm	 *curtime,      /* current time				      */
		 *localtime();	/* get current time			      */

long		atype,		/* access type needed for the specified file  */
		day1,		/* First day of acquisition within half orbit */
		day2,		/* Second day of acquisition within half orbit*/
		element = 0,	/* parameter element			      */
		i,		/* looping variable			      */
		len,		/* Length of string			      */
		nbs = 0,	/* average NBS offset			      */
		now,		/* current system time			      */
		fatal = -1,	/* fatal error flag			      */
		status;		/* return error status			      */

double		roll_coefs = 0.0,	/* average roll correction coeff      */
		pitch_coefs = 0.0,	/* average pitch correction coeff     */
		yaw_coefs = 0.0,	/* average yaw correction coeff       */
		delta_time = 0.0;	/* average delta time correction      */

int		key_compare();  /* function to compare starting time for sort */

char		directory[CMLEN];	/* Directory part of OUTFILE	      */
char		root[CMLEN];		/* Root part of OUTFILE		      */
char		extension[CMLEN];	/* Extension part of OUTFILE	      */
char		tmpname[24];		/* Temporary file name		      */
char		station[4];		/* Station or CEOS id		      */

/*
  Retrieve the CEOS ID
*/
val = p_find(vblock,"CEOSID");
strcpy(station,SVAL(*val,0));
len = strlen(station);
c_low2up(station,&len);
if (strncmp(station,"KM",2) != 0)
    c_lasmsg(TRUE,"makorb-ceos","Invalid CEOS ID specified: %s",station);

/*
  Retrieve the input archive file names
*/
val = p_find(vblock,"INFILES");
*nfiles = val->v_count;
atype = IREAD;
for (i = 0; i < *nfiles; i++)
    {
    /*
      Create the input pass names
    */
    c_mknam(SVAL(*val,i),in_pass[i].filename,"arch",&atype);

    /*
      Retrieve the archive header for the pass
    */
    status = c_getahdr(in_pass[i].filename,&in_pass[i].model,&in_pass[i].drop);
    if (status != E_SUCC)
       c_errmsg("Fatal error encountered","makorb-fatal",&fatal);

    /*
      Save the satellite offsets.  The average of the offsets will be used
      as the offsets over for the orbital pass.  Average only the first
      element of the coefficients since this is the mathematically correct.
      To average coefficients other than the stand-alone constant would be
      mathematically incorrect.
    */
    nbs += in_pass[i].model.clock_err;
    roll_coefs += in_pass[i].model.roll_coefs[0];
    pitch_coefs += in_pass[i].model.pitch_coefs[0];
    yaw_coefs += in_pass[i].model.yaw_coefs[0];
    delta_time += in_pass[i].model.delta_time;


    /*
      Get the start time of the pass
      Subtract the nbs offset to get true satellite time
    */
    in_pass[i].start_mil = (in_pass[i].model.shour * 3600 + 
			    in_pass[i].model.smin * 60 +
			    in_pass[i].model.ssec) * 1000 + 
			    in_pass[i].model.sfrac + 
			    in_pass[i].model.clock_err;

    /*
      Get the end time of the pass
      Subtract the nbs offset to get true satellite time
    */
    in_pass[i].end_mil = (in_pass[i].model.ehour * 3600 + 
			  in_pass[i].model.emin * 60 +
			  in_pass[i].model.esec) * 1000 + 
			  in_pass[i].model.efrac + 
			  in_pass[i].model.clock_err;

    }


/*
  See if a day is spanned between scenes within the half
  orbit:  Determine if there are two julian days within
  the half orbit.  If there are, add 1 days worth of
  milliseconds to the start,end time for each scene in
  the second day.
*/
day1 = day2 = in_pass[0].model.jday;
for (i = 1; i < *nfiles; i++)
    {
    if (in_pass[i].model.jday != day1)
	{
	if (in_pass[i].model.jday > day1)
	    day2 = in_pass[i].model.jday;
	else
	    day1 = in_pass[i].model.jday;
	break;
	}
    }

if (day1 != day2)
    {
    for (i = 0; i < *nfiles; i++)
	{
	if (in_pass[i].model.jday == day2)
	    {
	    in_pass[i].start_mil += 86400000;
	    in_pass[i].end_mil += 86400000;
	    }
	}
    }

/*
  Now check to see if an individual scene within
  the half orbit spans midnight.  If it does,
  add 1 days worth of milliseconds to the end time.
*/
for (i = 0; i < *nfiles; i++)
    {
    if (in_pass[i].end_mil < in_pass[i].start_mil)
	in_pass[i].end_mil += 86400000;
    }

/*
  Retrieve the output orbit name
*/
atype = IUPDATE;
c_getnam(vblock,"OUTFILE",&element,hosout,"arch",&atype);

/*
  Check to see if the output orbital pass exists
*/
*exists = FALSE;
if (access(hosout,0) == 0)
    {
    /*
      The output orbital pass exists
    */
    *exists = TRUE;

    /*
      Retrieve the output orbital pass archive header and place the
      information in the last element of the input pass structure
    */
    status = c_getahdr(hosout,out_model,&in_pass[*nfiles].drop);
    if (status != E_SUCC)
       c_errmsg("Fatal error encountered","makorb-fatal",&fatal);

    /*
      Save the NBS offset of the orbital pass.  The average NBS will
      be used as the offset over the whole orbital pass
    */
    nbs += out_model->clock_err;
    roll_coefs += out_model->roll_coefs[0];
    pitch_coefs += out_model->pitch_coefs[0];
    yaw_coefs += out_model->yaw_coefs[0];
    delta_time += out_model->delta_time;
    
    /*
      Save the julian date of the orbital pass
    */
    in_pass[*nfiles].model.jday = out_model->jday = day1;

    /*
      Calculate the orbital pass start time
      Subtract the nbs offset to get true satellite time
    */
    in_pass[*nfiles].start_mil = (out_model->shour * 3600 + out_model->smin*60 +
				  out_model->ssec) * 1000 + out_model->sfrac + 
				  out_model->clock_err;

    /*
      Calculate the orbital pass end time
      Subtract the nbs offset to get true satellite time
    */
    in_pass[*nfiles].end_mil = (out_model->ehour * 3600 + out_model->emin * 60 +
			        out_model->esec) * 1000 + out_model->efrac + 
				out_model->clock_err;

    /*
      Save the number of lines in the orbital pass
    */
    in_pass[*nfiles].model.nlines = out_model->nlines;

    /*
      Save the name of the orbital pass
    */
    strcpy(in_pass[*nfiles].filename,hosout);

    /*
      If any of the input passes are before the orbital pass
      the orbital pass is treated as an input file.  This will result
      in copying the orbital pass instead of trying to extend and move
      the data in the output file.
    */
    for (i = 0; i < *nfiles; i++)
	{
	if (in_pass[i].start_mil < in_pass[*nfiles].start_mil)
	    {
	    /*
	      Found a pass before the orbital pass; make the
	      output orbital pass part of the input pass structure
	    */
	    *nfiles += 1;
	    out_model->clock_err = nbs / *nfiles;
	    *exists = FALSE;
	    break;
	    }
	}
    }

/*
  Sort the input files by their start times
*/
qsort(in_pass,*nfiles,sizeof(struct ORBIT),key_compare);

/*
  When the output orbital pass doesn't exist create a temporary file
  name and initialize the output parameters
*/
if (!(*exists))
    {
    /*
      Create a temporary file name in the same directory
      as OUTFILE.
    */
    strcpy(tmpname,"orbpass");
    now = time(NULL);
    curtime = localtime(&now);
    sprintf(&tmpname[7],"%02d",curtime->tm_mon + 1);
    sprintf(&tmpname[9],"%02d",curtime->tm_mday);
    sprintf(&tmpname[11],"%02d",curtime->tm_year);
    sprintf(&tmpname[13],"%02d",curtime->tm_hour);
    sprintf(&tmpname[15],"%02d",curtime->tm_min);
    sprintf(&tmpname[17],"%02d",curtime->tm_sec);
    strcat(tmpname,".tmp");
    c_splitname(hosout,directory,root,extension);
    strcpy(in_pass[*nfiles].filename,directory);
    strcat(in_pass[*nfiles].filename,tmpname);
    out_model->clock_err = nbs / *nfiles;
    out_model->roll_coefs[0] = roll_coefs / *nfiles;
    out_model->pitch_coefs[0] = pitch_coefs / *nfiles;
    out_model->yaw_coefs[0] = yaw_coefs / *nfiles;
    out_model->delta_time = delta_time / *nfiles;
    
    out_model->satnum = in_pass[*nfiles-1].model.satnum;
    out_model->sorbit = in_pass[*nfiles-1].model.sorbit;
    out_model->eorbit = in_pass[*nfiles-1].model.eorbit;
    out_model->eqcrs = in_pass[*nfiles-1].model.eqcrs;

    for (i = 0; i < *nfiles; i++)
	{
	if (in_pass[i].model.jday == day1)
	    {
	    out_model->year = in_pass[i].model.year;
	    out_model->jday = in_pass[i].model.jday;
	    break;
	    }
	}
    c_julgre(&out_model->year,&out_model->jday,&out_model->month,
	     &out_model->day);

    out_model->dtype = HRPT;
    strcpy(out_model->station,station);

    out_model->nsamples = LAC_SAMPS;
    out_model->nchans = NCHANS;
    for (i = 0; i < NCHANS; i++)
	out_model->chans[i] = i + 1;

    in_pass[*nfiles].model.jday = day1;
    in_pass[*nfiles].start_mil = -1;
    in_pass[*nfiles].end_mil   = -1;
    in_pass[*nfiles].model.nlines = 0;
    strcpy(in_pass[*nfiles].model.station,station);
    }

/*
  Get the time between processing messages
*/
val = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);

return;
}
