/******************************************************************************
FUNCTION:	MAKORB

PURPOSE:	Generate a half orbit by stitching together the specified
		archive files.  The stitching is done by checking the
		time stamp codes for each line.  When scenes overlap the
		scene that contains the least amount of dropped lines in
		the overlap area is used for the stitching.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  2.0      08/92    D. Hollaren		Original Development
  2.1	   06/93    T. Baltzer		Switch to new model; add priority of
					beginning of new scene over end of
					previous
  2.2	   08/93    T. Rockvam		Added station priorities
  2.3	   12/94    T. Baltzer		Made in_pass 11 instead of 10 so that
					there is room for the output pass.
  2.4	   07/96    T. Baltzer		Modified so that it would follow
					DATAPATH for output archive file.
  2.5	   01/98    D. Lloyd		Made in_pass MAX_PASS + 1 to allow for
					orbits with more than 10 input scenes.
					Included stitch.h


PROJECT		1KM
*******************************************************************************/
#include <stdio.h>

#ifdef VMS
#include <file.h>
#else
#include <sys/file.h>
#endif

#include "las.h"
#include "util.h"
#include "adaps.h"
#include "acqio.h"
#include "makorb.h"
#include "stitch.h"

#define SAT_PERIOD 104		/* make the period long to ensure all scenes */
				/* can be included 			     */

main()
{
extern long curent;	     /* current output line number		     */

long	in_fd,		     /* file descriptor to the input archive files   */
	out_fd;		     /* file descriptor to the output orbit file     */

long	atype,		     /* access type to archive files		     */
	etime,		     /* ending time of entire orbit in milliseconds  */
 	exist,		     /* output scene already exists		     */
 	i,		     /* loop counter				     */
	img_typ = HRPT,      /* type of data (HRPT, LAC, or GAC)	     */
 	nfiles,		     /* number of input files 			     */
	nonfatal = 1,	     /* non-Fatal error code			     */
	fatal = -1,	     /* fatal error code			     */
	flag = 2,	     /* processing message flag			     */
	stdlun,		     /* standard output				     */
	max_entity, 	     /* maximum number of lines			     */
	msgtim,		     /* time between processing messages	     */
	numlines,	     /* approximate number of lines in entire orbit  */
 	ocount = 0,	     /* number of output files created		     */
	status,		     /* status variable				     */
	stime,		     /* starting time of entire orbit in milliseconds*/
	tpunit = NULL,	     /* tape unit.  NULL since image must be on disk */
	window[4],	     /* window to open				     */
	zero = 0;

char	hosout[CMLEN];	     /* output file host name		     	     */
char	tae_name[CMLEN];     /* tae file name				     */
char	msg[ERRLEN];	     /* error message				     */

struct PARBLK	vblock;	     /* TAE parameter block			     */
static struct ORBIT in_pass[MAX_PASS+1]; /* start and stop times of the passes       */
struct MODEL	out_model;   /* Model and orbital information		     */

/*
  Initialize LAS
*/
c_inlas(&vblock,&stdlun);

/*
  Get the input parameters
*/
getpar(&vblock,in_pass,&nfiles,hosout,&exist,&out_model,&msgtim);
if (!exist)
    /* if this is a new output scene, we will delete it on fatal error */
    ocount = 1;

/*
  Determine the approximate number of lines in the output image.  This will be
  used when opening the output image.
*/
stime = in_pass[0].start_mil;
etime = in_pass[0].end_mil;
for (i = 0; i < nfiles; i++)
     {
     if (in_pass[i].start_mil < stime) stime = in_pass[i].start_mil;
     if (in_pass[i].end_mil   > etime) etime = in_pass[i].end_mil;
     }
if (img_typ == GAC)
    numlines = (etime - stime) / 1000.0 * GAC_PER_SEC;
else
    numlines = (etime - stime) / 1000.0 * LAC_PER_SEC;

/*
  Open the output orbital pass archive file
*/
status = c_arcopnwrt(in_pass[nfiles].filename,&img_typ,&numlines,&out_fd);
if (status != E_SUCC)
    {
    c_errmsg("Error opening the output orbit file","makorb-open",&nonfatal);
    c_errmsg("Fatal error encountered","makorb-fatal",&fatal);
    }
    
/*
  Set up the processing message
*/
max_entity = (long)((SAT_PERIOD/2.0 * 60.0) * LAC_PER_SEC);
curent = 1;
c_inittm("MAKORB","Processed","line"," "," ",&max_entity,&zero,&msgtim,
         &flag);
c_startm();

/*
  Process the images
*/
atype = IREAD;
for (i = 0; i < nfiles; i++)
    {
    /*
      Open the input scene to be added to the orbit.
      Always window the entire image.
      Make sure the image is read in satellite view.
    */
    window[SL] = window[SS] = -1;
    in_pass[i].model.sat_view = TRUE;
    status = c_arcopnrd(in_pass[i].filename,&atype,window,&in_pass[i].model,
			&tpunit,&in_fd);
    if (status != E_SUCC)
	{
        strcpy(tae_name, in_pass[i].filename);
	sprintf(msg,"Error opening the input file %s",tae_name);
	c_errmsg(msg,"makorb-iopen",&nonfatal);
	/* go on to the next input image */
	continue;
	}

    /*
      Add the scene's data to the output half orbit file
    */
    status = add_scene(&in_fd,&out_fd,&in_pass[i],&in_pass[nfiles]);
    if (status != E_SUCC)
	c_clean(&vblock,"MAKORB",in_pass[nfiles].filename,&ocount);
	
    /*
      Close the input scene
    */
    c_arclose(in_pass[i].filename,&in_fd);
    }

/*
  Stop the processing message
*/
curent--;
c_stoptm();

/*
  Add back in the average nbs offset
  The current start times are satellite clock times
*/
in_pass[nfiles].start_mil -= out_model.clock_err;
in_pass[nfiles].end_mil   -= out_model.clock_err;
c_cvtmil(&in_pass[nfiles].start_mil,&out_model.shour,&out_model.smin,
         &out_model.ssec,&out_model.sfrac);
c_cvtmil(&in_pass[nfiles].end_mil,&out_model.ehour,&out_model.emin,
         &out_model.esec,&out_model.efrac);
/*
  If the start, end time were incremented by 1 day because a GMT day was
  spanned within the half orbit, subtract that time back out.
*/
if (out_model.shour > 23)
    out_model.shour -= 24;
if (out_model.ehour > 23)
    out_model.ehour -= 24;

out_model.nlines = in_pass[nfiles].model.nlines;
out_model.totmiss = 0;
for (i = 0; i < in_pass[nfiles].drop.ngaps; i++)
    out_model.totmiss += in_pass[nfiles].drop.nmissed[i];

/*
  Update the header information for the orbital pass
*/
status = c_initmod(&out_model.satnum,&out_model.year,&out_model.month,&out_model.day,
		   &out_model.shour,&out_model.smin,&out_model.ssec,&out_model.sfrac,
		   &out_model.dtype,&out_model.nlines,&out_model);
if (status != E_SUCC)
    c_clean(&vblock,"MAKORB",in_pass[nfiles].filename,&ocount);

/*
  Close the output orbital pass archive file
*/
c_arclose(in_pass[nfiles].filename,&out_fd);

/*
  Rename the temporary output orbital file to the user specified
  file name
*/
if (!exist)
    {
    status = rename(in_pass[nfiles].filename,hosout);
    if (status != E_SUCC)
	{
        strcpy(tae_name, in_pass[nfiles].filename);
	sprintf(msg,"Error renaming the file %s",tae_name);
	c_errmsg(msg,"makorb-renam",&nonfatal);
	c_clean(&vblock,"MAKORB",in_pass[nfiles].filename,&ocount);
	}
    }

/*
  Write the header information to the associated header file
*/
status = c_putahdr(hosout,&out_model,&in_pass[nfiles].drop);
if (status != E_SUCC)
    c_clean(&vblock,"MAKORB",hosout,&ocount);

/*
  Display a completion message
*/
sprintf(msg,"Output scene was %d lines",out_model.nlines);
c_errmsg(msg,"informational",&nonfatal);
c_complt("MAKORB",hosout,&ocount);
}
