/*******************************************************************************
FUNCTION:	micro_get

PURPOSE:	Retrieve the names of 24 images for creation of a microfiche.

Version  Programmer       Date   Modification
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          11/93  Development
        
PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "util.h"


#define MAXFILES 100		/* Max # of files returned by c_getfiles      */
#define MAXIMGS   24		/* Max # of image names to return to TAE      */


main()
{
char dir[CMLEN];		/* Directory where the images are located     */
char files[MAXFILES][CMLEN];	/* File names returned by c_getfiles()        */
char filespec[CMLEN];		/* File specification for microfiche images   */
char images[MAXIMGS][CMLEN];	/* Images to place in a microfiche image      */
char *str_ptrs[MAXIMGS];	/* Ptrs used for the call to q_string()       */
char str1[CMLEN];		/* Temporary string                           */
char str2[CMLEN];		/* Temporary string                           */

long fatal    = -1;		/* Fatal error flag                           */
long first;			/* Flag for first call to c_getfiles()        */
long i;				/* Loop control                               */
long nfiles;			/* Number of files returned by c_getfiles()   */
long nimages;			/* Number of images to be returned to TAE     */
long nonfatal =  1;		/* Nonfatal error flag                        */
long status;			/* Function return status                     */
long stdlun;			/* Standard output variable used by LAS       */

short acqflag;			/* Flag indicating where image was acquired   */
short america;			/* Flag indicating microfiche type            */
short save;			/* Flag indicating if image will be in micro  */

struct MODEL model;		/* AVHRR model and orbital information        */
struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE *var;		/* Ptr to a variable in the parameter block   */


/* Initialization 
-----------------*/ 
c_inlas(&vblock, &stdlun);


/* Get the directory where the images are located.  Convert the LAS directory
   path into a host path and then ensure there are no file names included.
-----------------------------------------------------------------------------*/
var = p_find(&vblock, "INDIR");
c_cmhost(SVAL(*var,0), str1);
c_splitname(str1, dir, str2, str2);


/* Get the type of microfiche to be created (AMER or WORLD) and
   create the file specification to be sent to c_getfiles().
---------------------------------------------------------------*/
var = p_find(&vblock, "IMGTYPE");
if (strcmp(SVAL(*var,0), "AMER") == 0)
    {
    america = TRUE;
    strcpy(filespec, "ah*.img");
    }
else
    {
    america = FALSE;
    strcpy(filespec, "a*.img");
    }


/* The support c_getfiles() opens a pipe to the system command 'echo filespec'
   to retrieve the names of the files matching the specification.  The echo
   command has a system limit on the number of characters that be specified on
   the command line.  Therefore, to reduce the overall number of characters, we
   will cd into the specified directory and leave the directory path out of our
   file specification.  We must then add it back before we return the list of
   files.  NOTE: VMS systems always return the directory path.
------------------------------------------------------------------------------*/
if (chdir(dir))
    {
    sprintf(str1, "%s: permission denied", dir);
    c_errmsg(str1, "micro-dir", &nonfatal);
    c_errmsg("Fatal error encountered", "micro-fatal", &fatal);
    }


/* Loop until there are no more images that match the file specification.  Even
   if we have found MAXIMGS images, ensure that c_getfiles() is called enough
   times such that it returns all file names matching our specification.  This
   ensures that the pipe command is closed and no defunct processes are left.
------------------------------------------------------------------------------*/
first   = TRUE;       /* First call to c_getfiles()? */
nfiles  = MAXFILES;
nimages = 0;
while (nfiles == MAXFILES)
    {
    /* Get a list of files matching the file specification
    ------------------------------------------------------*/
    status = c_getfiles(filespec, files, &nfiles, MAXFILES, first, TRUE);
    first = FALSE;


    /* For each of the files, determine if they are of the correct type
    -------------------------------------------------------------------*/
    for (i = 0; (i < nfiles) && (nimages < MAXIMGS); i++)
	{
	/* Read the addr file for the image
	-----------------------------------*/
	status = c_getaddr(files[i], &model);
	if (status != E_SUCC)
	    {
	    sprintf(str1, "Error reading ADDR for %s", files[i]);
	    c_errmsg(str1, "micro-addr", &nonfatal);
	    continue;
	    }


	/* Determine if the image was acquired by NGC, NOA, or SFL
	----------------------------------------------------------*/
	if ((strcmp(model.station, "NGC") == 0) ||
	    (strcmp(model.station, "NOA") == 0) ||
	    (strcmp(model.station, "SFL") == 0))
	    acqflag = TRUE;
	else
	    acqflag = FALSE;


	/* All HRPT images acquired by one of the above stations should only
	   be included in AMER type microfiche.  All other images should be
	   included in WORLD microfiche.
	--------------------------------------------------------------------*/
	if (america && (model.dtype == HRPT) && acqflag)
	    save = TRUE;
	else if (!america && (model.dtype == HRPT) && !acqflag)
	    save = TRUE;
	else if (!america && (model.dtype != HRPT))
	    save = TRUE;
	else
	    save = FALSE;
	

	/* If the image should be processed for this type of microfiche, add
	   it to the list of images.  The directory will not be included in
	   the files[] array on UNIX systems, while it will be on VMS systems.
	----------------------------------------------------------------------*/
	if (save)
	    {
#ifdef vms
	    sprintf(str1, "%s", files[i]);
#else
	    sprintf(str1, "%s%s", dir, files[i]);
#endif
            strcpy(images[nimages++], str1);
	    }
	}
    }


/* Return the list of images back to TAE
----------------------------------------*/
for (i = 0; i < nimages; i++)
    str_ptrs[i] = images[i];
if ((q_string(&vblock, "IMAGES", nimages, str_ptrs, P_UPDATE) != SUCCESS) ||
    (q_out(&vblock) != SUCCESS))
    {
    c_errmsg("Error passing variables to TAE", "micro-tae", &nonfatal);
    c_errmsg("Fatal error encountered", "micro-fatal", &fatal);
    }


/* Output completion message
----------------------------*/
z_exit(1, "MICRO_GET completed successfully");
}
