/*************************************************************************
FUNCTION:	MKLEVEL1B


PURPOSE:	

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0				Original development
 5.0	12/91	T. Rockvam	Conversion to LAS 5.0
 5.1	2/93	T. Rockvam	Changed to OSS calibration
 5.2	5/94	T. Rockvam	Added output to disk option and hooks 
				for NOAA phase II & III format changes
 5.3	3/95	J. Fenno	Updated for NOAA-14.  Added a shell
				script for converting an EDC AVHRR
				archive image to level 1b on the fly.
 5.4	7/96	T. Baltzer	Modified so that Level1b image follows
				DATAPATH.
 6.0	02/98	D.Lloyd		Rewritten to include calls to Level 1b and
				other new support routines.  GAC and LAC
				processing combined.  Provisions added for
				direct call from unix.  
 6.1	5/98	T. Baltzer	Modified to call the support c_l1bsumm(),
				consequently get rid of suminfo.c and
				dosummary.c
 6.2	10/98	D.Lloyd		Modifications for NOAA-15 ARS header

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
 	NOAA Polar Orbiter Data User's Guide
 	  K. Kidwell, NOAA
 	  Section 2 Level 1b Data Base
 	  Section 3 AVHRR Level 1b Data Base
 	NOAA Technical Memorandum NESS 107
 	  Section 5.1 AVHRR Calibration, Appendix A, Appendix B

*************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "archio.h"
#include "tapeio.h"
#include "model.h"
#include "level1bio.h"
#include "mklevel1b.h"
#include "adapsutil.h"


int main
(
   int argc,
   char **argv
)
{
char	acctnum[10];		/* account number			*/
char	archnam[CMLEN];		/* input archive image host file name	*/
char	comment[CMLEN];		/* tape mount comment			*/
char	density[5];		/* tape density				*/
char	drive[CMLEN];		/* name of tape drive			*/
char    hosdir[CMLEN];          /* host directory name                  */
char    hosext[CMLEN];          /* host directory name                  */
char	hosname[CMLEN];		/* output host file name		*/
char	hosout[CMLEN];		/* output host file name		*/
char    hosroot[CMLEN];         /* host root name                       */
char	msg[CMLEN];		/* message string			*/
char	ordernum[14];		/* order number				*/
char	out[2][CMLEN];		/* output file names			*/
char	prodesc[18];		/* product description			*/
char	tmpfil[CMLEN];		/* temporary output file name		*/
char	ttype[5];		/* tape type				*/
char	unitnum[5];		/* unit number				*/

FILE	*fdout = NULL;		/* output file descriptor		*/

long	archfd = 0;		/* file descriptor for image arch file	*/
long	bands[NCHANS] = {0};	/* bands to be processed - all images	*/
long	count = 1;		/* number of strings in out[]		*/
long	genfil;			/* flag: generate file name from sceneid*/
long	i;			/* loop index variable			*/
long	msgtim = 10;		/* time between processing messages	*/
long	mvol;			/* multiple volume flag			*/
long	nbands;			/* number of bands chosen		*/
long	kbyteswritten;		/* number bytes written to current tape */
long	nvols;			/* number of volumes used for this scene*/
long	nwin[4];		/* new window for windowed model	*/
long	pack;			/* packing flag				*/
long	recsiz;			/* Level 1b record size			*/
long	status;			/* general return status		*/
long	stdlun;			/* standard output			*/
long	tae;			/* flag; TRUE if called from taeland	*/
long	tape = FALSE;		/* tape open flag			*/
long	tplun = FALSE;		/* tape logical unit number		*/
long	window[4];		/* input window, SL,SS,NL,NS		*/
long    maxkbytes;              /* maximum Kbytes to write to tape	*/

struct	DROPPED	drop;		/* dropped lines			*/
struct	DROPPED	win_drop;	/* windowed dropped lines		*/
struct	MODEL	model;		/* Model and orbital information	*/
struct	MODEL	win_mod;	/* windowed model and orbital info	*/
struct	PARBLK	vblock;		/* parameter block			*/

unsigned char dsh[MAXLINSIZ];	/* Data set header record buffer	*/
unsigned char buf[ARS_HDR_SIZ];	/* TBM/ARS header buffer		*/


/*
  Decide who called this function.  We will assume that if there
  is more than one argument in argv[] the function was called from
  a unix system() call.  Otherwise we assume it was called from
  TAE.  Note: originally the return status from p_inim() was used
  indicating the presense of a vblock, but this proved to be
  unstable when the system call to mklevel1b originated in c-code
  that was called from a pdf.
------------------------------------------------------------------*/
if ( argc > 1 )
    tae = FALSE;
else
    tae = TRUE;
if ( tae )
    {
    /* Get parameters from the variable block */
    c_inlas(&vblock, &stdlun );
    getpar( &vblock, archnam, bands, &nbands, comment, hosout, acctnum,
	ordernum, unitnum, prodesc, drive, &pack, &mvol, &nvols, density,
	ttype, &msgtim, out[0], tmpfil, &genfil, &maxkbytes, &kbyteswritten );
    }
else
    /* Get parameters from argv[] */
    getargv( argc, argv, comment, archnam, hosout, acctnum, ordernum,
	unitnum, prodesc, drive, ttype, &nvols, &mvol, density,
	window, bands, &nbands, &pack, out[0], &maxkbytes, tmpfil,
	&genfil );


/*
  Fill the model and drpl structures with info from the header file
--------------------------------------------------------------------*/
status = c_getahdr( archnam, &model, &drop );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"mklevel1b-fatal","Fatal error encountered");


/*
  Get the window parameters
-----------------------------*/
if ( tae )
    {
    status = c_windpar( &vblock, &model, window );
    if ( status != E_SUCC )
	{
	c_lasmsg(FALSE,"mklevel1b-parm","Error determining window parameters");
	return E_FAIL;
	}
    }
else
    {
    window[NS] = model.nsamples;
    if ( window[NL] == -1 )
	window[NL] = model.nlines - window[SL] + 1;
    sprintf(msg,"Window specified for image is (%ld %ld %ld %ld)", window[SL],
	window[SS], window[NL], window[NS] );
    c_lasmsg(FALSE,"",msg);
    }


/*
  Open input image
-------------------*/
status = prepimg( archnam, &archfd, &model );
if ( status != E_SUCC )
    {
    finish( &vblock, drive, &nvols, tplun, fdout, archnam, &archfd,
	    hosout, &win_mod, tmpfil, genfil, out[1], &kbyteswritten, tae );
    if ( fdout )
        c_delfil( hosout );
    c_lasmsg(TRUE,"mklevel1b-fatal","Fatal error encountered");
    }


/*
  Create the Level 1b headers.  Flip the window coordinates for ASCENDING
  pass since that is what c_windmod() expects.
--------------------------------------------------------------------------*/
if ( model.cpass == ASCENDING )
    {
    nwin[SL] = model.nlines - window[SL] - window[NL] + 2;
    nwin[SS] = window[SS];
    nwin[NL] = window[NL];
    nwin[NS] = window[NS];
    }
else
    for ( i = 0; i < 4; i++ )
	nwin[i] = window[i];

/* First create a windowed model */
status = c_windmod( &model, &drop, &win_mod, &win_drop, nwin, bands, FALSE );
if (status != E_SUCC)
    c_lasmsg(TRUE,"mklevel1b-fatal","Fatal error encountered");

/* Next get the record size */
if ( model.satnum >= 15 )
    c_l1brecsiz( model.dtype, pack, TRUE, nbands, &recsiz );
else
    c_l1brecsiz( model.dtype, pack, FALSE, nbands, &recsiz );

/* Now set nwin to include the entire new windowed model */
nwin[SL]=1;

/* Finally make the headers */
status = c_l1bmkhdr( &win_mod, &win_drop, nwin, bands, &recsiz, buf, dsh );
if (status != E_SUCC)
    c_lasmsg(TRUE,"mklevel1b-fatal","Fatal error encountered");

/*
  Open output file/tape for image and write headers
----------------------------------------------------*/
status = prep1b( &tplun, &fdout, hosout, density, drive, &recsiz, 
		  &win_mod, buf, dsh, &tape );
if (status != E_SUCC)
    c_lasmsg(TRUE,"mklevel1b-fatal","Fatal error encountered");

/* update the number Kbytes written */
kbyteswritten += (long)(TBM_HDR_SIZ / 1000.0 + 0.5 );
if ( model.dtype == GAC )
    kbyteswritten += (long)( recsiz / 1000.0 + 0.5 );
else
    kbyteswritten += (long)((recsiz * 2) / 1000.0 + 0.5 );


/*
  Write image data to tape
---------------------------*/
if ( model.satnum < 15 )
    status = process( &tplun, fdout, &archfd, window, bands, nbands, pack,
		recsiz, msgtim, &model, maxkbytes,
		&kbyteswritten, &tape );
else
    status = process_klm( &tplun, fdout, &archfd, window, bands, nbands, pack,
		recsiz, msgtim, &model, maxkbytes, &kbyteswritten, &tape );
if ( status != E_SUCC )
    {
    finish( &vblock, drive, &nvols, tplun, fdout, archnam, &archfd,
	hosout, &win_mod, tmpfil, genfil, out[1], &kbyteswritten, tae );
    if ( fdout )
        c_delfil( hosout );
    c_lasmsg(FALSE,"mklevel1b-fatal","Fatal error encountered");
    return E_SUCC;
    }
else if ( tplun && kbyteswritten > maxkbytes )
    {
    genfil = FALSE;
    finish( &vblock, drive, &nvols, tplun, fdout, archnam, &archfd,
        hosout, &win_mod, tmpfil, genfil, out[1], &kbyteswritten, tae );
    if ( fdout )
        c_delfil( hosout );
    c_lasmsg(FALSE,"mklevel1b-ltend","Logical tape end reached");
    return E_SUCC;
    }


/*
  Finish with input and output and images
------------------------------------------*/
finish( &vblock, drive, &nvols, tplun, fdout, archnam, &archfd, hosout,
	&win_mod, tmpfil, genfil, out[1], &kbyteswritten, tae );


/*
  Determine what to call the root name of the product summary file.
---------------------------------------------------------------------*/
c_cmhost(out[1],hosname);
c_splitname( hosname, hosdir, hosroot, hosext );


/*
  Create the Level 1b product summary file.  Do not check the return
  status, even if an error generating the summary file occurs, continue
  processing (do not consider this a fatal error).
------------------------------------------------------------------------*/
c_l1bsumm( hosroot, acctnum, ordernum, unitnum, prodesc,
	density, &win_mod, out[0] );


/*
  Generate completion message
------------------------------*/
if ( hosout[0] ) 
    count++;
if ( tae )
    c_complt( "MKLEVEL1B", out, &count );
else
    {
    c_lasmsg(FALSE," ","Program MKLEVEL1B completed successfully.");
    c_lasmsg(FALSE," ","Created output:");
    c_lasmsg(FALSE," ","   %s",out[0]);
    c_lasmsg(FALSE," ","   %s",out[1]);
    }

return E_SUCC;
}
