/******************************************************************************
FUNCTION:       pck_zen_dec

PURPOSE:        Pack the extra decimal places for the sun zenith angles
		into 19 bytes + 1 bit (153 bits total)

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 5.0       09/92    T. Rockvam          Original development: P6174

PROJECT         ADAPS

ALGORITHM:
    Input buffer: 51 bytes containing the integer code in the range 0-4
		  representing the first decimal place of each sun zenith angle
  +--------+
  |00000aaa|
  +--------+
  +--------+
  |00000bbb|
  +--------+
  +--------+
  |00000ccc|
  +--------+
  etc.

    Format of output buffer:
  +--------+--------+--------+--------+--------+--------+--------+--------+
  |aaabbbcc|cdddeeef|ffggghhh|        |        |        |        |        |
  +--------+--------+--------+--------+--------+--------+--------+--------+
  etc.
  +--------+--------+
  |xxxyyyzz|z0000000|
  +--------+--------+

Place 8 input chars into 3 bytes at a time:
  This is 6 sets of 8 (48) input chars and 3 (18) output bytes,
  leaving the remaining 3 input chars to go into 2 bytes


ALGORITHM REFERENCES:
                NOAA Polar Orbiter Data User's Guide
                  K. Kidwell, NOAA
                  Section 3 AVHRR Level 1B Data Base
		MEMORANDUM: Current Polar Satellite Level 1B Data Enhancements,
		  Helen M. Wood, Director, Office of Satellite Data Processing 
		  and Distribution, 5/4/1992
*****************************************************************************/

#include <stdio.h>
#include "worgen.h"

#define MASK	0X0007

void pck_zen_dec
(
    unsigned char dec[51], /* Bytes representing decimal place for each s.z.a */
    unsigned char outbuf[20]    /* 20-byte output buffer		      */
)
{
long	in,		/* Input 3-bit counter				*/
	out;		/* Output byte counter				*/

/*
  Pack sets of 8 three-bit chars into the long
*/
for (in = 0,out= 0; in < 48; in +=8, out+=3)
    {
    outbuf[out]   = ((dec[in]   & MASK) << 5) + 
		    ((dec[in+1] & MASK) << 2) + 
		    ((dec[in+2] & MASK) >> 1);
    outbuf[out+1] = ((dec[in+2] & MASK) << 7) + 
		    ((dec[in+3] & MASK) << 4) + 
		    ((dec[in+4] & MASK) << 1) +
		    ((dec[in+5] & MASK) >> 2);
    outbuf[out+2] = ((dec[in+5] & MASK) << 6) + 
		    ((dec[in+6] & MASK) << 3) + 
		     (dec[in+7] & MASK);
    }

/*
  Handle the bits which are left over
*/
outbuf[18] = ((dec[48] & MASK) << 5) + 
	     ((dec[49] & MASK) << 2) + 
	     ((dec[50] & MASK) >> 1);
outbuf[19] =  (dec[50] & MASK) << 7;

return;
}
