/******************************************************************************
FUNCTION:	PROCESS

PURPOSE:	Process the block of output lines by adding calibration,
		earth location and zenith angles; write to output


Version	Date	Programmer	Modifications
-------	-----	--------------	-----------------------------------------------
 ???	??/??	???		Development
	12/91  	T.Rockvam	Conversion to LAS 5.0.  Taken from wrdtrc.c;
				this function is for LAC/HRPT data only.
	06/95	J.Fenno		Changed osscals() to teilletcals().
	07/95	J.Fenno		Updated to use Vermote calibration coefficients
				for NOAA-14.
 6.0	02/98	D.Lloyd		Combined with wrtgac() and modified to write
				one line of data utilizing Level 1b support
				routines.
 6.1	01/99	D.Lloyd		Modified to call new support c_calcoeff()
 				instead of c_ccprim() and c_ccupdt().
 6.2    02/02   D.Lloyd		CR 5813, when c_calcoef() returns failure,
 				set the quality flag for no calibration and
				zero the calibration coefficients.


PROJECT:	ADAPS

ALGORITHM REFERENCES:
		NOAA Polar Orbiter Data User's Guide
		    K. Kidwell, NOAA
		    Section 3 AVHRR Level 1B Data Base
		NOAA Technical Memorandum NESS 107
		    Section 5.1 AVHRR Calibration, Appendix A, Appendix B

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "adaps.h"
#include "adapsutil.h"
#include "adapssup.h"
#include "archio.h"
#include "mklevel1b.h"
#include "calcoef.h"
#include "level1bio.h"

#define NODEBUG   FALSE		/* Set to FALSE when debugging with dbx	*/

extern	long	curent;		/* current processing line		*/


int process
(
    long *tplun,		/* IN: output tape lun			*/
    FILE *fdout,		/* IN: output file descriptor		*/
    long *archfd,		/* IN: archive image file descriptor	*/
    long window[4],		/* IN: input window			*/
    long *bands,		/* IN: band numbers to write to tape	*/
    long nbands,		/* IN: number of bands to write		*/
    long pack,			/* IN: packing flag			*/
    long recsiz,		/* IN: Level 1b record size		*/
    long msgtim,		/* IN: time for processing message	*/
    struct MODEL *model,	/* IN: Model and orbital information	*/
    long max_bytes,		/* IN: max bytes to write to tape	*/
    long *kbyteswritten,	/* IN/OUT: bytes written to tape	*/
    long *tape 			/* IN/OUT: tape open flag		*/
)
{
char	binyr;			/* binary representation of year	*/
char	outbuf[MAXLINSIZ];	/* output data buffer			*/
char	zen_dec[51];		/* decimal portion of zenith angle	*/

double	elev = 0.0;		/* Elevation of data			*/
double	lat,lng;		/* lat/long of a point			*/
double	tsec;			/* double value of total seconde	*/
double	*view_angles = NULL;	/* view angles flag			*/
double	zenith;			/* solar zenith angle at a point	*/

float	fbyteswritten;		/* floating point value of kbyteswritten*/
float	line;			/* floating point value of line		*/
float	pix_start;		/* zenith starting pixel		*/
float	pix_step;		/* zenith sampling step size		*/
float	sample;			/* floating point value of sample	*/

long	dayn;			/* day/night indicator			*/
long	el;			/* ending line				*/
long	flag = ALLCAT;		/* all categories (bands) proc. at once	*/
long	i,j;			/* loop counters			*/
long	itsec;			/* total milliseconds			*/
long	mline = 0;		/* master line				*/
long	mnr_words = MNR_WORDS;	/* value of 103 - # minor frame words	*/
long	nbytes;			/* number of bytes per line		*/
long	nl;			/* number of lines			*/
long	nsamples;		/* number of samples per line		*/
long	*outlng;		/* long reference to output buffer	*/
long	pixcnt;			/* counter to fill buffer with pixels	*/
long	pixrec;			/* number of pixels in rec		*/
long	record;			/* current record (line) being written	*/
long	sl;			/* starting line			*/
long	status;			/* general return status		*/
long	zen_start;		/* starting byte of decimal zenith angle*/

short	*outsh;			/* short reference to output buffer	*/
short	pixels[MAXRECSIZ];	/* pixel values in bip format		*/
short	zsh;			/* short zenith angle			*/

struct  LINE   imgline;		/* one image line                       */
struct	COEFFSTR allcoeffs;	/* all coefficients (optical, thermal)	*/
struct	CALSTR	calstr;		/* calibration parameters structure	*/

int     no_calibration;		/* flag to indicate no calibration info	*/


/*
  Set up pointers to buffers
-----------------------------*/
outlng = (long *) outbuf;
outsh  = (short *) outbuf;

/*
  Set up data type dependant variables
---------------------------------------*/
if ( model->dtype == GAC )
    {
    nbytes = recsiz / 2;	/* GAC has 2 lines per record		    */
    nsamples = GAC_SAMPS;
    pix_start = 5.0;		/* calibration sampled starting at pixel 5  */
    pix_step = 8.0;		/* and every 8 pixels thereafter            */
    pixrec = GAC_SAMPS * nbands;
    zen_start = 3176;		/* PREVIDEO + 2728 bytes of packed video    */
    }
else
    {
    nbytes = recsiz * 2;	/* LAC/HRPT has 2 records per line	    */
    nsamples = LAC_SAMPS;
    pix_start = 25.0;		/* calibration sampled starting at pixel 25 */
    pix_step = 40.0;		/* and every 40 pixels thereafter           */
    pixrec = LAC_SAMPS * nbands;
    zen_start = 14104;		/* PREVIDEO + 13656 bytes of packed video   */
    }

/* Set up the calibration structure
-----------------------------------*/
calstr.calper = CALPER;
calstr.prtper = PRTPER;
calstr.file_type = ARCIMAGE;
calstr.illum = FALSE;
if ( model->satnum == 14 )
    strcpy( calstr.calmethod, "vermote" );
else
    strcpy( calstr.calmethod, "teillet" );


/*
  Determine the line numbers relative to the archive image:
  starting line must be flipped for ascending pass.
-----------------------------------------------------------*/
nl = window[NL];
if ( model->cpass == ASCENDING )
    sl = model->nlines - window[SL] - nl + 2;
else
    sl = window[SL];
el = sl + nl - 1;

/*
  Set up for processing
------------------------*/
status = c_arcseek( archfd, sl );
if ( status == E_FAIL )
    return E_FAIL;
binyr = (char)( model->year );

/*
  Start processing message
---------------------------*/
if ( NODEBUG )
    {
    curent = 1;
    c_inittm("mklevel1b","wrote","line","bands"," ",&nl,&nbands,&msgtim,&flag);
    c_startm();
    }

/*
  Process image lines
------------------------*/
fbyteswritten = (float)(*kbyteswritten);
for ( record = sl; record <= el; record++ )
    {
    /*
      Read image line 
    --------------------*/
    status = c_arcread( archfd, &imgline.min_frm, (short **)&imgline.data );
    if ( status != E_SUCC )
        return E_FAIL;

    /*
      Get calibration coefficients
    --------------------------------*/
    status = c_calcoeff( record, mline, &(imgline.min_frm[0]), *archfd, model,
    			&allcoeffs, &calstr );
    /* CR 5813: If the function is unsuccessful for whatever reason,
    set the no_calibration flag to TRUE and continue.  Otherwise set
    it to FALSE.
    ----------------------------------------------------------------*/
    if ( status != E_SUCC )
        no_calibration = TRUE;
    else
        no_calibration = FALSE;

    /*
      Set up records and write to tape only if not dropped line
      Check the frame sync and time stamp code
    ------------------------------------------------------------*/
    if (( imgline.min_frm[0] != 0 ) || ( imgline.min_frm[9] != 0 ) ||
        ( imgline.min_frm[10] != 0 ) || ( imgline.min_frm[11] != 0 ))
	{
        /*
          Record number field (in binary)
	  take last two bytes of record
        ------------------------------------*/
	outsh[0] = (short)record;

	/*
          Time code
        ------------*/
	mk1btm( &(imgline.min_frm[8]), (unsigned char *)&binyr,
		(unsigned char *)&outbuf[2] );
	/* save milliseconds for use in calculating zenith angles */
	itsec = *( (long *)&outbuf[4] );

        /*
          Reset quality indicators
	  Note: this also sets the SBBC indicators to "no correction" 
	   for phase II in outbuf[9]
	  CR5813: if there is no calibration indicate this in byte 8.
	--------------------------------------------------------------*/
	if ( no_calibration )
	    outbuf[8] = 0x08;
	else
	    outbuf[8]  = 0;
        outbuf[9]  = 0;
        outbuf[10] = 0;
        outbuf[11] = 0;

	/*
	  Calibration coefficients.  CR 5813 If the no calibration
	  flag is set, zero all the coefficients.  This is done
	  because there is no way to indicate in the quality flags
	  which set of coefficients are valid.
	  Note: when c_calcoef() is made smarter, as a minimum, the
	  visible coefficients could be set regardless.
	----------------------------------------------------------*/
	if ( no_calibration )
	    {
	    for ( i = 0; i < 2; i++ )
		{
		outlng[2*i+3] = 0;
		outlng[2*i+4] = 0;
		}
	    for ( i = 0; i < 3; i++ )
		{
		outlng[2*(i+2)+3] = 0;
		outlng[2*(i+2)+4] = 0;
		}
	    }
	else
	    {
	    for ( i = 0; i < 2; i++ )
		{
		outlng[2*i+3] = (long)(allcoeffs.ocoeff.order1[i][0] * TWO30);
		outlng[2*i+4] = (long)(allcoeffs.ocoeff.order0[i][0] * TWO22);
		}
	    for ( i = 0; i < 3; i++ )
		{
		outlng[2*(i+2)+3] = (long)(allcoeffs.tcoeff.order1[i] * TWO30);
		outlng[2*(i+2)+4] = (long)(allcoeffs.tcoeff.order0[i] * TWO22);
		}
	    }


	/*
          Number of zenith angles and earth location points
        ----------------------------------------------------*/
	outbuf[52] = 51;

	/*
	  Create zenith angles and earth locations
	--------------------------------------------*/
	for ( i = 0; i < 51; i++ )
	    {
	    line = (float)record;
    	    sample = pix_start + ( (float)i * pix_step );
    	    c_ls2ll( &line, &sample, &elev, model, &lat, &lng, view_angles);
	    outsh[52 + i*2] = (short)( DEG( lat ) * 128.0 );
	    outsh[53 + i*2] = (short)( DEG( lng ) * 128.0 );
	    tsec = (double)( itsec / 1000.0 );
	    c_daynite( &(model->jday), &(model->year), &tsec, &lat,
			&lng, &zenith, &dayn );

	    zsh = (short)( DEG( zenith ) * 2.0 );
	    outbuf[53+i] = (char)zsh;
	    zen_dec[i] = 0;
	    }

	/*
	  Pack telemetry data (103-word header)
	----------------------------------------*/
	c_pck32( &(imgline.min_frm[0]), (unsigned char *)&outbuf[TELOFF],
		&mnr_words );

 	/*
	  Set up buffer of pixels in bip format
	----------------------------------------*/
	pixcnt = 0;
	for ( j = 0; j < nsamples; j++ )
	    {
    	    for ( i = 0; i < nbands; i++ )
                {
                pixels[pixcnt] = imgline.data[j][bands[i]-1];
	  	pixcnt++;
          	}
	    }

	/*
	  Put video (and zenith for packed format) into output buffer
	--------------------------------------------------------------*/
	if (( nbands == NCHANS ) && ( pack ))
	    {
	    /*
	      Packed format: pack 3 10-bit pixels into a 32-bit word
	      set up Level 1b video data
	    ---------------------------------------------------------*/
 	    c_pck32( &pixels[0], (unsigned char *)&outbuf[PREVIDEO], &pixrec );

            /*
              Save the decimal portions of the zenith angles
            ------------------------------------------------*/
            pck_zen_dec( (unsigned char *)zen_dec,
			(unsigned char *)&outbuf[zen_start] );

	    /*
	      Set clock drift delta to 0 to indicate no adjustments 
	      in time codes (phase II)
	    --------------------------------------------------------*/
	    outbuf[zen_start+20] = 0;

	    /* zero out remaining bytes */
	    for ( i = zen_start+21; i < nbytes; i++ )
		outbuf[i] = 0;

	    }
	else
	    {
            /*
	      unpacked format: put 2 10-bit pixels into a 32-bit word 
	      for selective copy, we pack the output buffer with shorts (and
	      byteswap 4 bytes at a time if necessary), effectively loading 
	      2 shorts into a long. Also insure the last 2 bytes of output 
	      buffer are zero.  The offset into the buffer is PREVIDEO / 2
	      since we are referencing the buffer as shorts.
	    ----------------------------------------------------------------*/
	    j = PREVIDEO / 2;
	    outsh[j + pixrec] = 0;
	    for ( i = 0; i < pixrec; i++ )
     	        outsh[j + i] = pixels[i];
	    }

	/*
	  Check that we aren't going to write beyond the logical tape end
	  indicated by max_bytes.
	------------------------------------------------------------------*/
 	if ( *tplun && ( (fbyteswritten + (nbytes / 1000.0)) > max_bytes ))
	    {
	    /*
	      I tried to back space to the beginning of the partial file
	      but was unsuccessful.  After using c_tpbsf() of c_tpfsf(),
	      any subsequent tape operations returned tape errors.  If the
	      tape I/O errors ever get fixed the following code may be
	      uncommented so that partial files aren't left on the tape.
	    --------------------------------------------------------------*/
	    /* Backspace to the beginning of the previous scene		*/
	    /* status = c_tpbsf( tplun, &count, &mode ); */
	    /* Forward space to the beginning of the partial scene	*/
	    /* This puts us at the end of the last full scene written	*/
	    /* status = c_tpfsf( tplun, &count, &mode ); */
	    /* c_tpclos( tplun, &mode, &rewopt, &eofopt ); */
	    /* *tape = FALSE; */
	    /* *tplun = 0; */
	    *kbyteswritten = max_bytes + 1;
	    return E_SUCC;
	    }

	/* write one Level 1b line */
        if ( *tplun )
	    status = c_l1bwritet( tplun, (unsigned char *)outbuf, &recsiz,
				  tape );
        else
            status = c_l1bwrited( fdout, (unsigned char *)outbuf, &recsiz );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"mklevel1b-bdimagwr","Error writing image line");
	    return E_FAIL;
	    }
	fbyteswritten += (float)nbytes / 1000.0;
	}
    curent++;
    }

/*
  Return number of bytes written
---------------------------------*/
*kbyteswritten = (long)( fbyteswritten + 0.5 );

/*
  Stop processing message
--------------------------*/
if ( NODEBUG )
    c_stoptm();


return E_SUCC;
}
