/******************************************************************************
FUNCTION:	PROCESS_KLM

PURPOSE:	Process the block of output lines by adding calibration,
		earth location and zenith angles; write to output


Version	Date	Programmer	Modifications
-------	-----	--------------	-----------------------------------------------
 1.0	11/98	D.Lloyd		Stolen from process and modified for new NOAA
 				KLM data record format.
 1.1    02/02   D.Lloyd		CR 5813, when c_calcoef() returns failure,
 				set the quality flags for no calibration and
				zero the calibration coefficients.


PROJECT:	ADAPS

ALGORITHM REFERENCES:
		NOAA KLM User's Guide
		    K. Kidwell, NOAA
		    Section 8 AVHRR Level 1B Data Base


*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "adaps.h"
#include "adapsutil.h"
#include "adapssup.h"
#include "archio.h"
#include "mklevel1b.h"
#include "calcoef.h"
#include "level1bio.h"

#define NODEBUG   TRUE		/* Set to FALSE when debugging with dbx	*/

extern	long	curent;		/* current processing line		*/


int process_klm
(
    long *tplun,		/* IN: output tape lun			*/
    FILE *fdout,		/* IN: output file descriptor		*/
    long *archfd,		/* IN: archive image file descriptor	*/
    long window[4],		/* IN: input window			*/
    long *bands,		/* IN: band numbers to write to tape	*/
    long nbands,		/* IN: number of bands to write		*/
    long pack,			/* IN: packing flag			*/
    long recsiz,		/* IN: Level 1b record size		*/
    long msgtim,		/* IN: time for processing message	*/
    struct MODEL *model,	/* IN: Model and orbital information	*/
    long max_bytes,		/* IN: max bytes to write to tape	*/
    long *kbyteswritten,	/* IN/OUT: bytes written to tape	*/
    long *tape 			/* IN/OUT: tape open flag		*/
)
{
char	outbuf[MAXLINSIZ];	/* output data buffer			*/

double	ddummy;			/* dummy variable			*/
double	dmilsec;		/* milliseconds value for current line	*/
double	elev = 0.0;		/* Elevation of data			*/
double	lat,lng;		/* lat/long of a point			*/
double	view_angles[3];		/* view angles array			*/

float	fbyteswritten;		/* floating point value of kbyteswritten*/
float	line;			/* floating point value of line		*/
float	pix_start;		/* zenith starting pixel		*/
float	pix_step;		/* zenith sampling step size		*/
float	sample;			/* floating point value of sample	*/

long	el;			/* ending line				*/
long	flag = ALLCAT;		/* all categories (bands) proc. at once	*/
long	i,j,k;			/* loop counters			*/
long	ldummy;			/* dummy variable			*/
long	milsec;			/* milliseconds of current line		*/
long	mline = 0;		/* master line				*/
long	nbytes;			/* number of bytes per line		*/
long	nl;			/* number of lines			*/
long	nsamples;		/* number of samples per line		*/
long	*outlng;		/* long reference to output buffer	*/
long	passdir;		/* pass direction flag			*/
long	pixcnt;			/* counter to fill buffer with pixels	*/
long	pixrec;			/* number of pixels in rec		*/
long	record;			/* current record (line) being written	*/
long	sl;			/* starting line			*/
long	status;			/* general return status		*/

short	*outsh;			/* short reference to output buffer	*/
short	pixels[MAXRECSIZ];	/* pixel values in bip format		*/
short	p_jday;			/* julian day of previous line		*/
short	c_jday;			/* julian day of current line		*/
short	year;			/* 4-digit year value			*/
short	sync_code[3] = { 0x0284, 0x016f, 0x035c };

struct  LINE   imgline;		/* one image line			*/
struct	COEFFSTR allcoeffs;	/* all coefficients (optical, thermal)	*/
struct	CALSTR	calstr;		/* calibration parameters structure	*/


int     no_calibration;		/* flag to indicate no calibration info	*/

/* Set up pointers to buffers
-----------------------------*/
outlng = (long *) outbuf;
outsh  = (short *) outbuf;

/* Set up data type dependant variables
---------------------------------------*/
if ( model->dtype == GAC )
    {
    nbytes = recsiz;		/* KLM GAC has 1 line per record	    */
    nsamples = GAC_SAMPS;
    pix_start = 5.0;		/* calibration sampled starting at pixel 5  */
    pix_step = 8.0;		/* and every 8 pixels thereafter            */
    pixrec = GAC_SAMPS * nbands;
    }
else
    {
    nbytes = recsiz;		/* KLM LAC/HRPT has 1 record per line	    */
    nsamples = LAC_SAMPS;
    pix_start = 25.0;		/* calibration sampled starting at pixel 25 */
    pix_step = 40.0;		/* and every 40 pixels thereafter           */
    pixrec = LAC_SAMPS * nbands;
    }


/* Set up the calibration structure
-----------------------------------*/
calstr.calper = CALPER;
calstr.prtper = PRTPER;
calstr.file_type = ARCIMAGE;
calstr.illum = FALSE;
strcpy( calstr.calmethod, "prelaunch" );


/* Determine the line numbers relative to the archive image:
starting line must be flipped for ascending pass.
-----------------------------------------------------------*/
nl = window[NL];
if ( model->cpass == ASCENDING )
    sl = model->nlines - window[SL] - nl + 2;
else
    sl = window[SL];
el = sl + nl - 1;


/* Set up for processing
------------------------*/
status = c_arcseek( archfd, sl );
if ( status == E_FAIL )
    return E_FAIL;
p_jday = 0;
year = (short)c_4digityr( model->year );

/* Start processing message
---------------------------*/
if ( NODEBUG )
    {
    curent = 1;
    c_inittm("mklevel1b","wrote","line","bands"," ",&nl,&nbands,&msgtim,&flag);
    c_startm();
    }

/* Process image lines
------------------------*/
fbyteswritten = (float)(*kbyteswritten);
for ( record = sl; record <= el; record++ )
    {
    /* Read image line 
    --------------------*/
    status = c_arcread( archfd, &imgline.min_frm, (short **)&imgline.data );
    if ( status != E_SUCC )
        return E_FAIL;

    /* Get calibration coefficients
    --------------------------------*/
    status = c_calcoeff( record, mline, &(imgline.min_frm[0]), *archfd, model,
    			&allcoeffs, &calstr );
    /* CR 5813: If the function is unsuccessful for whatever reason,
    set the no_calibration flag to TRUE and continue.  Otherwise set
    it to FALSE.
    ----------------------------------------------------------------*/
    if ( status != E_SUCC )
        no_calibration = TRUE;
    else
        no_calibration = FALSE;

    /* Set up records and write to tape only if not dropped line
    Check the frame sync and time stamp code
    ------------------------------------------------------------*/
    if (( imgline.min_frm[0] != 0 ) || ( imgline.min_frm[9] != 0 ) ||
        ( imgline.min_frm[10] != 0 ) || ( imgline.min_frm[11] != 0 ))
	{
        /* Record number field (in binary)
	take last two bytes of record
        ------------------------------------*/
	outsh[0] = (short)record;

	/* Date/Time code
        -----------------*/
	c_jday = (imgline.min_frm[8] & 01776) >> 1;
	milsec = imgline.min_frm[9] & 0177;
	milsec = (milsec << 10) + (imgline.min_frm[10] & 01777);
	milsec = (milsec << 10) + (imgline.min_frm[11] & 01777);
	if ( c_jday < p_jday )	/* spanned a year	*/
	    year++;
	outsh[1] = year;
	outsh[2] = c_jday;
	outsh[3] = (short)model->clock_err;
	outlng[2] = milsec;
	p_jday = c_jday;
	
	/* determine direction of pass for this scan line
	-------------------------------------------------*/
	dmilsec = (double)milsec;
	c_getorb( &dmilsec, model, &ldummy, &passdir, &ddummy );
	
	/* clear scan line bit field and set direction flag
	---------------------------------------------------*/
	outsh[6] = 0;
	if ( passdir == 1 )
	    outsh[6] |= 0x80;
	
	/* determine and set the channel 3 flag
	---------------------------------------*/
	if ( imgline.min_frm[6] & 0x01 )
	    outsh[6] |= 0x01;

	/* zero fill
	------------*/
	for ( i = 7; i < 24; i++ )
	    outsh[i] = 0;

	/* Count and save number of bit errors in frame sync
	----------------------------------------------------*/
	outsh[19] = 0;
	for ( i = 0; i < 3; i++ )
	    {
	    k = imgline.min_frm[i] ^ sync_code[i];
	    for ( j = 0; j < 10; j++ )
	        {
		if ( k & 0x01 )
		    outsh[13]++;
		k = k >> 1;
		}
	    }

	/* CR 5813: If no calibration information is available set the
	flag: octet 25-28 bit 28 or byte 24 bit 4.  Zero out the
	calibration coefficients.
	Note: When c_calcoef() is made smarter, only set the appopriate
	flags and only zero those coefficients.
	--------------------------------------------------------------*/
	if ( no_calibration )
	    {
	    outbuf[24] |= 0x10;
	    outbuf[33] |= 0x80;
	    outbuf[35] |= 0x80;
	    outbuf[37] |= 0x80;
	    for ( i = 12; i < 75; i++ )
	        outlng[i] = 0;
	    }
	else
	    {

	    /* Calibration coefficients -- visible coeffs words 12-56
	    thermal coeffs words 57-74
	    ---------------------------------------------------------*/
	    /* First do the optical channels 1 and 2, and 3A if present
	    Note: there are 3 sets of coefficients for each optical
	    channel.  Currently we are just repeating the coefficients
	    for all 3 sets as NOAA is doing in theirs. (hence the for
	    k loop)
	    -----------------------------------------------------------*/
	    for ( i = 0, j = 12; i < calstr.nopticbnds; i++ )
		for ( k = 0; k < 3; k++ )
		    {
		    outlng[j++] =
		        (long)RND(allcoeffs.ocoeff.order1[i][0]*10000000);
		    outlng[j++] =
		        (long)RND(allcoeffs.ocoeff.order0[i][0]*1000000);
		    outlng[j++] =
		        (long)RND(allcoeffs.ocoeff.order1[i][1]*10000000);
		    outlng[j++] =
		        (long)RND(allcoeffs.ocoeff.order0[i][1]*1000000);
		    outlng[j++] = allcoeffs.ocoeff.count[i];
		    }

	    /* Next if channel 3B (thermal) is present store its coeffs
	    Note: there are 2 sets of coefficients for each thermal
	    channel.  Currently  we are just repeating the coefficients
	    for both sets as NOAA is doing in theirs. (hence the for k
	    loop)
	    if ( calstr.nopticbnds == 2 )
	    -----------------------------------------------------------*/
	    if ( calstr.nthermbnds == 3 )
		{
		j = 57;
		for ( i = 42; i < 57; i++ )
		    outlng[i] = 0;
		for ( k = 0; k < 2; k++ )
		    {
		    outlng[j++] =
		        (long)RND(allcoeffs.tcoeff.order0[0] * 1000000);
		    outlng[j++] =
		        (long)RND(allcoeffs.tcoeff.order1[0] * 1000000);
		    outlng[j++] = 0;
		    }
		}

	    /* otherwise zero out 3B
	    ------------------------*/
	    else
		{
		j = 63;
		for ( i = 57; i < 63; i++ )
		    outlng[i] = 0;
		}

	    /* Finally store the thermal coeffs for 4 and 5
	    Note: there are 2 sets of coefficients for each thermal
	    channel.  Currently  we are just repeating the coefficients
	    for both sets as NOAA is doing in theirs. (hence the for k
	    loop)
	    -----------------------------------------------------------*/
	    for ( i = 1; i < 3; i++ )
		for ( k = 0; k < 2; k++ )
		    {
		    outlng[j++] =
		        (long)(RND(allcoeffs.tcoeff.order0[i]*1000000.0));
		    outlng[j++] =
		        (long)(RND(allcoeffs.tcoeff.order1[i]*1000000.0));
		    outlng[j++] =
		        (long)(RND(allcoeffs.tcoeff.order2[i]*1000000.0));
		    }
	    }


	/* zero fill
	------------*/
	for ( i = 300; i < 328; i++ )
	    outbuf[i] = 0;

	/* Create zenith angles and earth locations
	--------------------------------------------*/
	for ( i = 0; i < 51; i++ )
	    {
	    line = (float)record;
    	    sample = pix_start + ( (float)i * pix_step );
    	    c_ls2ll( &line, &sample, &elev, model, &lat, &lng, view_angles);
	    /* store scaled solar, satellite and rel az angles	*/
	    outsh[164 + i*3] = (short)( DEG( view_angles[1] ) * 100 );
	    outsh[165 + i*3] = (short)( DEG( view_angles[0] ) * 100 );
	    outsh[166 + i*3] = (short)((DEG( view_angles[2] ) - 180.0) * 100 );

	    /* store scaled lat long */
	    outlng[160 + i*2] = (long)( DEG( lat ) * 10000 );
	    outlng[161 + i*2] = (long)( DEG( lng ) * 10000 );
	    }

	/* zero fill
	------------*/
	for ( i = 634; i < 640; i++ )
	    outbuf[i] = 0;
	for ( i = 1048; i < 1056; i++ )
	    outbuf[i] = 0;

	/* Copy (don't pack) telemetry data (103-word header)
	-----------------------------------------------------*/
	for ( i = 0; i < MNR_WORDS; i++ )
	    outsh[528+i] = imgline.min_frm[i];

	/* zero fill
	------------*/
	for ( i = 1262; i < 1264; i++ )
	    outbuf[i] = 0;

 	/* Set up buffer of pixels in bip format
	----------------------------------------*/
	pixcnt = 0;
	for ( j = 0; j < nsamples; j++ )
	    {
    	    for ( i = 0; i < nbands; i++ )
                {
                pixels[pixcnt] = imgline.data[j][bands[i]-1];
	  	pixcnt++;
          	}
	    }

	/* Put video (and zenith for packed format) into output buffer
	--------------------------------------------------------------*/
	if (( nbands == NCHANS ) && ( pack ))
	    {
	    /* Packed format: pack 3 10-bit pixels into a 32-bit word
	    set up Level 1b video data
	    ---------------------------------------------------------*/
 	    c_pck32( &pixels[0], (unsigned char *)&outbuf[1264], &pixrec );


	    /* Zero out remaining bytes in the buffer, this
	    includes TIP and CLAVR that we don't process.
	    -----------------------------------------------*/
	    for ( i = 14920; i < nbytes; i++ )
		outbuf[i] = 0;

	    }
	else
	    {
            /* unpacked format: put 2 10-bit pixels into a 32-bit word 
	    for selective copy, we pack the output buffer with shorts (and
	    byteswap 4 bytes at a time if necessary), effectively loading 
	    2 shorts into a long. Also insure the last 2 bytes of output 
	    buffer are zero.  The offset into the buffer is PREVIDEO / 2
	    since we are referencing the buffer as shorts.
	    --------------------------------------------------------------*/
	    j = KLMPREVIDEO / 2;
	    outsh[j + pixrec] = 0;
	    for ( i = 0; i < pixrec; i++ )
     	        outsh[j + i] = pixels[i];

	    /* Zero out remaining bytes in the buffer, this
	    includes TIP and CLAVR that we don't process.
	    -----------------------------------------------*/
	    for ( i = pixrec * 2 + KLMPREVIDEO; i < nbytes; i++ )
		outbuf[i] = 0;

	    }

	/* Check that we aren't going to write beyond the logical tape end
	indicated by max_bytes.
	------------------------------------------------------------------*/
 	if ( *tplun && ( (fbyteswritten + (nbytes / 1000.0)) > max_bytes ))
	    {
	    /* I tried to back space to the beginning of the partial file
	    but was unsuccessful.  After using c_tpbsf() of c_tpfsf(),
	    any subsequent tape operations returned tape errors.  If the
	    tape I/O errors ever get fixed the following code may be
	    uncommented so that partial files aren't left on the tape.
	    --------------------------------------------------------------*/
	    /* Backspace to the beginning of the previous scene		*/
	    /* status = c_tpbsf( tplun, &count, &mode );		*/
	    /* Forward space to the beginning of the partial scene	*/
	    /* This puts us at the end of the last full scene written	*/
	    /* status = c_tpfsf( tplun, &count, &mode );		*/
	    /* c_tpclos( tplun, &mode, &rewopt, &eofopt );		*/
	    /* *tape = FALSE;						*/
	    /* *tplun = 0;						*/
	    /* ---------------------------------------------------------*/
	    /* Here are the local variable declarations needed if the code
	    above is ever uncommented:
	    long rewopt = REWOFFL;
	    long eofopt = 1;
	    long mode = HANDLE;
	    long count = 1;
	    ------------------------------------------------------------*/
	    *kbyteswritten = max_bytes + 1;
	    return E_SUCC;
	    }

	/* write one Level 1b line 
	--------------------------*/
        if ( *tplun )
	    status = c_l1bwritet( tplun, (unsigned char *)outbuf, &recsiz,
				  tape );
        else
            status = c_l1bwrited( fdout, (unsigned char *)outbuf, &recsiz );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"mklevel1b-bdimagwr","Error writing image line");
	    return E_FAIL;
	    }
	fbyteswritten += (float)nbytes / 1000.0;
	}
    curent++;
    }

/* Return number of bytes written
---------------------------------*/
*kbyteswritten = (long)( fbyteswritten + 0.5 );

/* Stop processing message
--------------------------*/
if ( NODEBUG )
    c_stoptm();


return E_SUCC;
}
