/*******************************************************************************
FUNCTION:	MODLUT

PURPOSE:	Generate an enhacement lookup table (LUT) for the facsimile
		recorder.

		The user enters a set of calibration grey levels, calibration
		densities (a representation of the actual response of the film
		recorder), control grey levels, control densities (a
		representation of a desired response of the film recorder),
		and enhancement FROM/TO values (create a contrast enhancement
		of the image data) to create a pretty image on the facsimile
		recorder.

		This function was also adapted to list the grey levels
		and densities.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    D. Zokaties      ?????  Original algorithm development
  5.0    J.Matzner        08/92  Conversion to LAS5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	MODLUT must be run under TAE.
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "las.h"
#ifdef unix
#include "sys/file.h"
#else
#include <file.h>
#endif
#include "modlut.h"

main ()
{
struct PARBLK vblock;	   /* Parameter block				     */

FILE	*fp,		   /* File pointer				     */
	*fopen();          /* Function for opening files		     */

float	caldens[MAXPT],    /* Calibration density values                     */
	condens[MAXPT],    /* Control density values                         */
	to[MAXPT],         /* Enhancement to grey level values               */
	enlut[LUTSIZ],     /* Enhancement LUT                                */
	callut[LUTSIZ],	   /* Calibration LUT                                */
	conlut[LUTSIZ];    /* Control LUT                                    */

long	calgrey[MAXPT],    /* Calibration grey level values                  */
	congrey[MAXPT],    /* Control grey level values                      */
	from[MAXPT],       /* Enhancement from grey level values             */
	linear[LUTSIZ],	   /* Linear LUT for fill			     */
	comlut[LUTSIZ],	   /* Combined LUTs				     */
	calcount,          /* Number of calibration pairs                    */
	concount,          /* Number of control pairs                        */
	encount,           /* Number of from-to pairs                        */
	fopt,		   /* Output file list option                        */
	i, 		   /* Looping index                                  */
	last,		   /* Flag indicating last LUT table                 */
	std_out,	   /* Standard output device                         */
	nonfatal = 1,	   /* Nonfatal error flag                            */
	fatal = -1,	   /* Fatal error flag                               */
	ocount = 0,	   /* Number of output files created		     */
	calmax = 0;	   /* Maximum value in CALGREY array		     */

char 	disk[CMLEN],	   /* Disk location of the saved LUTs		     */
	subcmd[CMLEN],	   /* Specified subcommand                           */
        prtfil[CMLEN],     /* Temporary print file name for fileutil	     */
	temp[CMLEN],	   /* Temporary output LUT file name                 */
	lutname[CMLEN],	   /* Output LUT name				     */
        outfil[1][CMLEN];  /* Output files created                           */

/* 
  Initialize TAE and get the file descriptor for standard output
*/
c_inlas(&vblock,&std_out);

/*
  Get user specified parameters
*/
getpar(&vblock,calgrey,caldens,&calcount,congrey,condens,&concount,from,to,
       &encount,lutname,subcmd,&last,disk,&fopt,prtfil,outfil[ocount]);

/*
  Create an output print file for the LUTs.  The LUTs will be appended to
  the output file so that only one report is generated.
*/
if (strcmp(subcmd,"LIST") == 0)
    {
    fp = fopen(prtfil,"a");
    if (fp == NULL)
	{
	c_errmsg("Error opening the output print file","modlut-opnerr",
		  &nonfatal);
	c_errmsg("Fatal error encountered","modlut-fatal",&fatal);
	}

    fprintf(fp, "\n");
    fprintf(fp, "			      %s Parameters\n",lutname);
    fprintf(fp, "\n");
    fprintf(fp, "       Calibration 	            Control\n");
    fprintf(fp,
      "          Grey       Calibration     Grey       Control\n");
    fprintf(fp, "          Level        Density       Level      Density");
    fprintf(fp, "    From    To\n");
    fprintf(fp, "       -----------   -----------    -------     -------");
    fprintf(fp, "    ----    --\n");

    for (i = 0; i < calcount; i++)
	{
	if (i <= encount-1)
	    {
	    fprintf(fp, "  %2d       %3d         %5.4f         %3d",
		    i+1, calgrey[i], caldens[i], congrey[i]);

	    fprintf(fp, "       %5.4f      %3d   %3d\n",
		    condens[i], from[i], (long)to[i]);
	    }
	else
	    {
	    fprintf(fp, "  %2d       %3d         %5.4f         ",
		    i+1, calgrey[i], caldens[i]);

	    fprintf(fp, "%3d       %5.4f\n",
		    congrey[i], condens[i]);
	    }
	}

    if (last == 0)
	{
	fprintf(fp,"\f");
	}
    else
	{
	if (fopt)
	    ocount = 1;
	}


    }
else
    {
    /*
      Set up linear table for filler
    */
    for (i = 0; i < LUTSIZ; i++)
	linear[i] = i;

    /*
      Generate calibration LUT from input grey level and density values
    */
    filler(calgrey,caldens,linear,calcount,LUTSIZ,callut);

    /*
      Find the maximum calibration grey level value in the array
    */
    for (i = 1; i < calcount; i++)
	{
	if (calgrey[i] > calmax)
	    calmax = calgrey[i];
	}

    /*
      Modify the calibration LUT based on the maximum grey level value
    */
    filler(calgrey,caldens,linear,calcount,calmax,callut);

    /*
      Generate control LUT from input grey level and density values
    */
    filler(congrey,condens,linear,concount,LUTSIZ,conlut);

    /*
      Generate an enhancement piece-wise linear LUT from input from and to
      values
    */
    piece(from,to,encount,enlut);

    /* 
      Combine the three above LUTs
    */
    punt(callut,conlut,enlut,comlut);

    /*
      Add the LUT name to the output file list
    */
    strcpy(outfil[ocount],lutname);
    ocount++;

    /*
      Create a file with the LUT values in it
    */
#ifdef unix
    sprintf(temp,"%s/%s.dat",disk,lutname);
#else
    sprintf(temp,"%s%s.dat",disk,lutname);
#endif

    /*
      Open the specified LUT
    */
    fp = fopen(temp,"w");
    if (fp == NULL)
	{
	c_errmsg("Error opening LUT","modlut-opnerr",&nonfatal);
	c_errmsg("Fatal error encountered","modlut-fatal",&fatal);
	}

    /*
      Copy the combined LUT to the output file
    */
    for (i = 0; i < LUTSIZ; i++)
	fprintf (fp,"%d\n",comlut[i]);

    /*
      Close the file
    */
    fclose(fp);
    }

c_complt("MODLUT",outfil,&ocount);
}
