/*******************************************************************************
FUNCTION: getpar


PURPOSE:  Retrieve all of the parameters for PRODEDIT

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          09/92  Development
  1.1    J.Fenno          10/92  Separated the functions prodgen and prodedit.
				 Created object library for product generation.
  1.2    J.Fenno          02/93  Added an age field to the prodgen table
  3.1    A.Vander Vorste  01/96  Added windowing and station field to the 
				 prodgen table.
  3.2    A. Vander Vorste 03/96  Added output directory field prodgen.table

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Run under TAE

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util2.h"
#include "adaps.h"
#include "products.h"
#include "prodedit.h"

/* External variables
---------------------*/
extern char function[10];      /* Name of the function for error messages     */
extern char table[CMLEN];      /* Host file name of the prodgen;table         */


int getpar
(
   struct PARBLK *vblock,  /* IN  - TAE parameter block                       */
   char   *order,          /* OUT - Standing order string                     */
   char   *subcommand      /* OUT - Subcommand being executed                 */
)
{
char bands[CMLEN];         /* Bands being ordered. Example: 12345  00305      */
char day_nite[CMLEN];      /* Day night indicator (DAY, NIGHT)                */
char description[CMLEN];   /* Description entered by the user                 */
char output[CMLEN];	   /* Output directory or media			      */
char dtype[CMLEN];         /* Data type being ordered (GAC, HRPT)             */
char eastlong[15];         /* Easternmost longitude of the area of interest   */
char northlat[15];         /* Northernmost latitude of the area of interest   */
char southlat[15];         /* Southernmost latitude of the area of interest   */
char pole[CMLEN];          /* Pole crossing flag (NORTH, SOUTH)               */
char *ptr;                 /* String pointer                                  */
char satnum[4];            /* Satellite number                                */
char station[4];	   /* Station ID that acquired data		      */
char string[CMLEN];        /* Temporary working string                        */
char westlong[15];         /* Westernmost longitude of the area of interest   */
char window[2];		   /* Windowing flag - if set, only keep lines of 
				interest */ 
long account;              /* Customers account number                        */
long age;                  /* Max # of days old that the data should be       */
long band;                 /* Individual band number                          */
long edate;                /* End date in integer form (YYYYMMDD)             */
long fatal     = -1;       /* Fatal error flag                                */
long i;                    /* Loop control                                    */
long minlines;             /* Minimum number of lines a product should contain*/
long ordernum;             /* The standing order number being deleted         */
long product;              /* Product code that is being ordered              */
long sdate;                /* Start date in integer form (YYYYMMDD)           */

struct VARIABLE *parm;     /* Ptr to a TAE parameter                          */



/* Get the parameters
---------------------*/
parm = p_find(vblock, "_SUBCMD");
strcpy(subcommand, SVAL(*parm, 0));
if (strcmp(subcommand, "DEL") == 0)
   {
   parm = p_find(vblock, "ACCOUNT");
   account = IVAL(*parm,0);

   parm = p_find(vblock, "ORDER");
   ordernum = IVAL(*parm,0);

   /* Create a partial standing order string
   -----------------------------------------*/
   sprintf(order, "%09d %03d", account, ordernum);
   }
else
   {
   /* Retrieve the parameters that are the same for subcommands ADD and POLE
   -------------------------------------------------------------------------*/
   parm = p_find(vblock, "ACCOUNT");
   account = IVAL(*parm,0);

   parm = p_find(vblock, "SDATE");
   strcpy(string, SVAL(*parm,0));
   get_date(string, &sdate);

   parm = p_find(vblock, "EDATE");
   strcpy(string, SVAL(*parm,0));
   get_date(string, &edate);

   parm = p_find(vblock, "AGE");
   age = IVAL(*parm,0);

   parm = p_find(vblock, "STATION");
   strcpy(station, SVAL(*parm,0));

   parm = p_find(vblock, "SATNUM");
   strcpy(satnum, SVAL(*parm,0));

   parm = p_find(vblock, "DAYOPT");
   strcpy(day_nite, SVAL(*parm,0));
   i = (long)strlen(day_nite);
   c_up2low(day_nite, &i);

   parm = p_find(vblock, "DTYPE");
   strcpy(dtype, SVAL(*parm,0));

   parm = p_find(vblock, "MINLINES");
   minlines = IVAL(*parm,0);

   parm = p_find(vblock, "WINDOW");
   strcpy(window, SVAL(*parm,0));

   parm = p_find(vblock, "PRODUCT");
   product = IVAL(*parm,0);

   parm = p_find(vblock, "DESCRPT");
   strcpy(description, SVAL(*parm,0));

   parm = p_find(vblock, "OUTPUT");
   strcpy(output, SVAL(*parm,0));

   strcpy(bands, "00000");
   parm = p_find(vblock, "BANDS");
   for (i = 0; i < parm->v_count; i++)
      {
      band = IVAL(*parm, i);
      bands[band - 1] = (char)((int)'0' + band);
      }


   /* Retrieve the parameters that are different for subcommands ADD and POLE
   --------------------------------------------------------------------------*/
   if (strcmp(subcommand, "ADD") == 0)
      {
      parm = p_find(vblock, "NORTHLAT");
      strcpy(string, SVAL(*parm,0));
      if (lat_long(string, 6, northlat) == E_FAIL)
	 c_errmsg("Invalid specification of NORTHLAT", "prodedit-dms", &fatal);

      parm = p_find(vblock, "SOUTHLAT");
      strcpy(string, SVAL(*parm,0));
      if (lat_long(string, 6, southlat) == E_FAIL)
	 c_errmsg("Invalid specification of SOUTHLAT", "prodedit-dms", &fatal);

      parm = p_find(vblock, "WESTLONG");
      strcpy(string, SVAL(*parm,0));
      if (lat_long(string, 7, westlong) == E_FAIL)
	 c_errmsg("Invalid specification of WESTLONG", "prodedit-dms", &fatal);

      parm = p_find(vblock, "EASTLONG");
      strcpy(string, SVAL(*parm,0));
      if (lat_long(string, 7, eastlong) == E_FAIL)
	 c_errmsg("Invalid specification of EASTLONG", "prodedit-dms", &fatal);
      }
   else
      {
      parm = p_find(vblock, "POLE");
      strcpy(pole, SVAL(*parm,0));

      parm = p_find(vblock, "LAT");
      strcpy(string, SVAL(*parm,0));
      if (lat_long(string, 6, string) == E_FAIL)
	 c_errmsg("Invalid specification of LAT", "prodedit-dms", &fatal);
      if (strcmp(pole, "NORTH") == 0)
	 {
	 strcpy(northlat, string);
	 strcpy(southlat, "+000000");
	 strcpy(westlong, "+0000000");
	 strcpy(eastlong, "+0000000");
	 }
      else
	 {
	 strcpy(northlat, "+000000");
	 strcpy(southlat, string);
	 strcpy(westlong, "+0000000");
	 strcpy(eastlong, "+0000000");
	 }
      }


   /* Ensure the ending date is greater than the starting date
   -----------------------------------------------------------*/
   if (sdate > edate)
      c_errmsg("Starting date after ending date", "prodedit-dates", &fatal);


   /* Take all of the parameters and create a new standing order string
   --------------------------------------------------------------------*/
   sprintf(order,
      "%09d 000 %8d %8d %04d %3s %2s %-5s %-4s %s %s %s %s %05d %s %03d %s %-17s %s",
      account, sdate, edate, age, station, satnum, day_nite, dtype, northlat, 
      southlat, westlong, eastlong, minlines, window, product, bands, description,
      output);
   }


/* Get the name of the prodgen;table file in ADAPSTABLES
--------------------------------------------------------*/
ptr = (char *)getenv("ADAPSTABLES");
if (ptr == NULL)
   c_errmsg("Error retrieving ADAPSTABLES", "prodedit-env", &fatal);
strcpy(table, ptr);
#ifdef unix
strcat(table, "/");
#endif
strcat(table, TABLE);


/* Initialize the function name for error messages
--------------------------------------------------*/
strcpy(function, "prodedit");

return(E_SUCC);
}



/*==============================================================================
  Given:  A string representing a latitude or longitude in Degrees, Minutes,
	  Seconds (DMS) form and the number of digits to be in the output string
  Task:   Convert the given string to a DMS string with the specified number of
	  digits
  Return: The new DMS string with a leading sign
==============================================================================*/
long lat_long
(
   char *str,          /* IN  - DMS string                                    */
   long ndigits,       /* IN  - Number of digits in the output string         */
   char dms[CMLEN]     /* OUT - DMS string that contains ndigits digits       */
)
{
char tmp[CMLEN];       /* Local copy of the input str                         */

long i;                /* Loop control                                        */
long len;              /* Length of a string                                  */


/* Copy the input string so it is not destroyed
-----------------------------------------------*/
strcpy(tmp, str);


/* Copy the leading sign to the dms string
------------------------------------------*/
if (tmp[0] == '-')
   {
   strcpy(dms, "-");
   strcpy(tmp, &tmp[1]);
   }
else if (tmp[0] == '+')
   {
   strcpy(dms, "+");
   strcpy(tmp, &tmp[1]);
   }
else
   strcpy(dms, "+");


/* Get the length of the string.  Ensure its not too long
---------------------------------------------------------*/
len = (long)strlen(tmp);
if (len > ndigits)
   return(E_FAIL);


/* Ensure all chars are digits
------------------------------*/
for (i = 0; i < len; i++)
   if ((tmp[i] != '0') && (tmp[i] != '1') && (tmp[i] != '2') &&
       (tmp[i] != '3') && (tmp[i] != '4') && (tmp[i] != '5') &&
       (tmp[i] != '6') && (tmp[i] != '7') && (tmp[i] != '8') &&
       (tmp[i] != '9'))
      return(E_FAIL);


/* Add necessary leading zeros to pad dms to 7 characters
---------------------------------------------------------*/
for (i = len; i < ndigits; i++)
   strcat(dms, "0");


/* Add the tmp string to the dms string
-----------------------------------------*/
strcat(dms, tmp);
return E_SUCC;
}



/*==============================================================================
  Given:  A string in the form of MM/DD/YYYY
  Task:   Convert the given string into an integer date (YYYYMMDD)
  Return: The integer date
==============================================================================*/
void get_date
(
   char *date_str,     /* IN  - String date of the form MM/DD/YYYY            */
   long *date          /* OUT - Integer value of the form YYYYMMDD            */
)
{
char str[CMLEN];       /* Local copy of the date string                       */
char *ptr;             /* Character pointer for manipulating the string       */

long day;              /* Integer value of the day                            */
long fatal     = -1;   /* Fatal error flag                                    */
long month;            /* Integer value of the month                          */
long year;             /* Integer value of the year                           */


/* Copy the input string so it is not destroyed
-----------------------------------------------*/
strcpy(str, date_str);


/* Retrieve the year
--------------------*/
ptr = strrchr(str, '/');
if (ptr == NULL)
   c_errmsg("Invalid date format", "prodedit-date", &fatal);
*(ptr++) = '\0';
if (strlen(ptr) != 4)
   c_errmsg("Invalid date format", "prodedit-date",  &fatal);
if (sscanf(ptr, "%d", &year) != 1)
   c_errmsg("Invalid date format", "prodedit-date", &fatal);


/* Retrieve the day
-------------------*/
ptr = strrchr(str, '/');
if (ptr == NULL)
   c_errmsg("Invalid date format", "prodedit-date", &fatal);
*(ptr++) = '\0';
if (sscanf(ptr, "%d", &day) != 1)
   c_errmsg("Invalid date format", "prodedit-date", &fatal);


/* Retrieve the month
---------------------*/
if (sscanf(str, "%d", &month) != 1)
   c_errmsg("Invalid date format", "prodedit-date", &fatal);


/* Put the year, month, and day into an integer
-----------------------------------------------*/
*date = (year * 10000) + (month * 100) + day;
}
