/*******************************************************************************
FUNCTION: prodedit

PURPOSE:  Allow the user to add and delete standing orders from the
          prodgen;table file in ADAPSTABLES.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          09/92  Development
  1.1    J.Fenno          10/92  Separated prodgen and prodedit.  Created an
				 object library for product generation.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Run under TAE

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util2.h"
#include "adaps.h"
#include "adapsutil.h"
#include "queue.h"
#include "products.h"
#include "prodedit.h"


/* Global variables
-------------------*/
char   function[10];      /* Name of the function for error messages          */
char   table[CMLEN];      /* Name of the prodgen;table file in ADAPSTABLES    */
long   std_out;           /* Standard output for TAE                          */
struct PARBLK vblock;     /* TAE parameter block                              */


int main(void)
{
char filenames[1][CMLEN]; /* Array of filenames. Only used to call complt     */
char order[CMLEN];        /* String containing a new standing order           */
char subcommand[CMLEN];   /* Sub command being executed                       */

long fatal    = -1;       /* Fatal error flag                                 */
long i;                   /* Loop control                                     */
long lock     = LOCK;     /* Flag for c_lockfil to lock a file                */
long nonfatal =  1;       /* Nonfatal error flag                              */
long orders_q;            /* Q of standing order strings read from the table  */
long position;            /* Position of an element in the queue              */
long unlock = UNLOCK;     /* Flag for c_lockfil to unlock a file              */

void *ptr1;               /* Void ptr for calls to queue functions            */
void *ptr2;               /* Void ptr for calls to queue functions            */

struct DESCRIPT descript; /* Struct containing description lines from table   */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Get the parameters
---------------------*/
getpar(&vblock, order, subcommand);


/* Lock the prodgen table
-------------------------*/
i = 0;
while (c_lockfil(table, &lock) != E_SUCC)
   {
   ++i;
   if (i > 10)
      c_errmsg("Unable to lock table", "prodedit-nolock", &fatal);
   sleep(5);
   }


/* Read all of the standing orders from the table
-------------------------------------------------*/
if (c_pgread(&descript, &orders_q))
   c_errmsg("Error reading the standing order table", "prodedit-read", &fatal);


/* Add or delete the specified order from the queue of standing orders
----------------------------------------------------------------------*/
ptr1 = (void *)order;
if (strcmp(subcommand, "DEL") == 0)
   {
   /* Find the position of the standing order in the queue of orders
   -----------------------------------------------------------------*/
   if (c_qfind(orders_q, ptr1, &ptr2, delete_cmp, &position))
      {
      c_lockfil(table, &unlock);
      c_qdestroy(orders_q);
      c_errmsg("Error accessing queue", "prodedit-queue", &fatal);
      }
   

   /* If the returned position is not zero then delete that position
   -----------------------------------------------------------------*/
   if (position == 0)
      c_errmsg("Standing order not found", "prodedit-none", &nonfatal);
   else if (c_qdelete(orders_q, position))
      {
      c_lockfil(table, &unlock);
      c_qdestroy(orders_q);
      c_errmsg("Error accessing queue", "prodedit-queue", &fatal);
      }
   }
else if (c_qinsert(orders_q, ptr1, add_cmp))
   {
   c_lockfil(table, &unlock);
   c_qdestroy(orders_q);
   c_errmsg("Error accessing queue", "prodedit-queue", &fatal);
   }


/* Write the new set of orders to the prodgen table and unlock it
-----------------------------------------------------------------*/
if (c_pgwrite(&descript, orders_q))
   c_errmsg("Error writing the standing order table", "prodedit-write", &fatal);
if (c_lockfil(table, &unlock) != E_SUCC)
   c_errmsg("Error unlocking table", "prodedit-unlock", &nonfatal);


/* Destroy the queue of orders
------------------------------*/
c_qdestroy(orders_q);


/* Issue a completion message
-----------------------------*/
i = 0;
c_complt("PRODEDIT", filenames, &i);
return E_SUCC;
}



/*==============================================================================
  Given:  Two pointers to void objects
  Task:   Compare the two strings that are stored in a queue.  When adding a
	  standing order to the queue we want to ignore the order numbers.  A
	  new order number will be assigned when the table is written.
  Return: A long value less than 0 if ptr1 is less than ptr2; 0 if ptr1 == ptr2;
	  a long value greater than 0 if ptr1 is greater than ptr2.
==============================================================================*/
long add_cmp
(
   void *ptr1,
   void *ptr2 
)
{
char *p1;
char *p2;
long value;

p1 = (char *)ptr1;
p2 = (char *)ptr2;
value = strncmp(p1, p2, 9);
if (value == 0)
   value = strcmp(&p1[14], &p2[14]);

return(value);
}



/*==============================================================================
  Given:  Two pointers to void objects
  Task:   Compare the two strings that are stored in a queue.  When deleting
	  a standing order we only key off the first 13 characters.
  Return: A long value less than 0 if ptr1 is less than ptr2; 0 if ptr1 == ptr2;
	  a long value greater than 0 if ptr1 is greater than ptr2.
==============================================================================*/
long delete_cmp
(
   void *ptr1,
   void *ptr2 
)
{
char *p1;
char *p2;

p1 = (char *)ptr1;
p2 = (char *)ptr2;

return(strncmp(p1, p2, 13));
}
