#!/bin/csh
#
# FUNCTION: daac_avhrr_stage.csh
#
# PURPOSE:  Distribute all of the AVHRR images in the DAAC stage directory to
#	    the appropriate AVHRR image directory.  The directory scheme is
#	    based upon the scene ID.
#
#    silo: /dmf/pds/daac/avhrr/orbits/SN/YEAR/MN/DY
#
#	where   SN: NOAA Satellite Number (11, 12, 14, etc.)
#	      YEAR: Four character year of acquisition
#	        MN: Two character month of acquisition
#	        DY: Two character day of acquisition
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------


#-----------------
# Set up the shell
#-----------------
unalias cd
set linesize = 13796


#---------------------------------------------------------------------
# Determine the hostname.  Only edcsgs20 is a valid option.
#---------------------------------------------------------------------
set host = `uname -n`
if ("$host" != "edcsgs20") then
    echo "daac_avhrr_stage.csh: error -- invalid system $host"
    goto USAGE
endif


#------------------------------------------------------
# Only certain users have permission to run this script
#------------------------------------------------------
set user = `whoami`
if (("$user" != "ops") && ("$user" != "opsavhrr") && ("$user" != "dtrans")) then
    echo "daac_avhrr_stage.csh: error -- invalid user $user"
    goto USAGE
endif


#--------------------
# Check the arguments
#--------------------
if ($#argv != 0) goto USAGE


#-----------------------------------------
# Set up variables for the directory paths
#-----------------------------------------
set rootdirsgs20 = "/daac/avhrr"
set rootdirsgs3 = "/dmf/pds/daac/avhrr"


#----------------------------
# Move to the stage directory
#----------------------------
set stagedir = "/sgs20/km1/adaps/daacstage"
cd $stagedir


#----------------------------------------------------------
# Determine if there are any archive files in the directory
#----------------------------------------------------------
set count = `/bin/ls | egrep '^a.*\.arch$' | wc -l`
if ($count == 0) then
    goto level_1b
endif


#-----------------------------------------
# Examine all files in the stage directory
#-----------------------------------------
foreach arch (a[ghlo]*.arch)
    #---------------------------
    # Build the header file name
    #---------------------------
    set ahdr = `echo $arch | awk -F\. '{printf "%s.ahdr",$1}'`


    #-------------------------------------------------------------
    # If the associated header file does not exist, skip the image
    #-------------------------------------------------------------
    if (! -e $ahdr) continue


    #------------------------------------
    # Retrieve the size of the image file
    #------------------------------------
    set actual_size = `/bin/ls -l $arch | awk '{print $5}'`


    #--------------------------------------------------------------------
    # Determine what the size of the image should be.  This is done by
    # extracting the number of lines from the header file and multiplying
    # by the bytes per line.
    #--------------------------------------------------------------------
    set nlines = `dd if=$ahdr bs=80 skip=7 count=1 |& awk '(NR == 1){print $7}'`
    @ expected_size = $nlines * $linesize


    #-----------------------------------------------------------------
    # If the image isn't the expected size then skip the image for now
    #-----------------------------------------------------------------
    if ($actual_size != $expected_size) continue


    #---------------------------------------------------------
    # Determine the directory where the image should be placed
    #---------------------------------------------------------
    set sat  = `echo $arch | cut -c3-4 -`
    set mon  = `echo $arch | cut -c5-6 -`
    set day  = `echo $arch | cut -c7-8 -`
    set year = `echo $arch | cut -c9-10 -`
    if (($year > 74) && ($year < 100)) then
	set year = "19$year"
    else
	set year = "20$year"
    endif
    if ("$arch" =~ ao*) then
	set dstdir = $rootdirsgs3/orbits/$sat/$year/$mon/$day
    else
	set dstdir = $rootdirsgs3/scenes/$sat/$year/$mon/$day
    endif


    #---------------------------------------------------------------
    # UMPUT the scene to silo, can't rely on an NFS mount.
    # Use the -m switch to create the directory if it doesn't exist.
    # If it is not successfully transferred, leave it to try later.
    # Remove the local version once successfully copied.
    #---------------------------------------------------------------
    umput -m -d $dstdir $arch
    if ($status) then
	echo "daac_avhrr_stage.csh: error transfering $arch"
    else
	rm -f $arch
    endif
    umput -m -d $dstdir $ahdr
    if ($status) then
	echo "daac_avhrr_stage.csh: error transfering $ahdr"
    else
	rm -f $ahdr
    endif
end




#----------------------------------------------------------
# Determine if there are any level1b files in the directory
# (Orbital segments)
#----------------------------------------------------------
level_1b:
set count = `/bin/ls | egrep '^ao.*\.done$' | wc -l`
if ($count == 0) then
    goto level_1b2
endif

set linesize = 14800
#-----------------------------------------
# Examine all Level-1B orbital segments in the stage directory
#-----------------------------------------
foreach level1b (ao*.l1b)
    #---------------------------
    # Build the header file name
    #---------------------------
    set done = `echo $level1b | awk -F\. '{printf "%s.done",$1}'`


    #-------------------------------------------------------------
    # If the associated done file does not exist, skip the image
    # because it hasn't been completely processed
    #-------------------------------------------------------------
    if (! -e $done) then 
	continue
    endif

    #---------------------------------------------------------
    # Determine the directory where the image should be placed.
    # Level-1B orbital segments are written to a temporary
    # space where IMS finds them and moves them onto the DAAC
    # stage area once they match them with their database info.
    #---------------------------------------------------------
    set dstdir = /dmf/pds/daac/orb_seg/tmp


    #---------------------------------------------------------------
    # Open the file access wide open so that IT can have full access
    # to the files once on silo.
    # UMPUT the scene to silo, can't rely on an NFS mount.
    # Use the -m switch to create the directory if it doesn't exist.
    # If it is not successfully transferred, leave it to try later.
    # Remove the local version once successfully copied.
    #---------------------------------------------------------------
    chmod 777 $level1b
    umput -m -d $dstdir $level1b
    if ($status) then
	echo "daac_avhrr_stage.csh: error transfering $level1b"
    else
	rm -f $level1b
	rm -f $done
    endif

end


#----------------------------------------------------------
# Determine if there are any level1b files in the directory
# (Individual acquisitions)
#----------------------------------------------------------
level_1b2:
set count = `/bin/ls | egrep '^a.*\.done$' | wc -l`
if ($count == 0) then
    exit 0
endif



#-----------------------------------------
# Examine all individual acquisitions in the stage directory
#-----------------------------------------
foreach level1b (a[ghl]*.l1b)
    #---------------------------
    # Build the header file name
    #---------------------------
    set done = `echo $level1b | awk -F\. '{printf "%s.done",$1}'`


    #-------------------------------------------------------------
    # If the associated done file does not exist, skip the image
    # because it hasn't been completely processed
    #-------------------------------------------------------------
    if (! -e $done) then 
	continue
    endif

    #---------------------------------------------------------
    # Determine the directory where the image should be placed
    #---------------------------------------------------------
    set sat  = `echo $level1b | cut -c3-4 -`
    set mon  = `echo $level1b | cut -c5-6 -`
    set day  = `echo $level1b | cut -c7-8 -`
    set year = `echo $level1b | cut -c9-10 -`
    if (($year > 74) && ($year < 100)) then
        set year = "19$year"
    else
        set year = "20$year"
    endif
    set dstdir = $rootdirsgs3/scenes/$sat/$year/$mon/$day


    #---------------------------------------------------------------
    # UMPUT the scene to silo, can't rely on an NFS mount.
    # Use the -m switch to create the directory if it doesn't exist.
    # If it is not successfully transferred, leave it to try later.
    # Remove the local version once successfully copied.
    #---------------------------------------------------------------
    umput -m -d $dstdir $level1b
    if ($status) then
	echo "daac_avhrr_stage.csh: error transfering $level1b"
    else
	rm -f $level1b
	rm -f $done
    endif

end




#------------------
# Normal completion
#------------------
exit 0



USAGE:
    onintr -
    echo 'usage: daac_avhrr_stage.csh'
    echo ''
    echo '       Move all of the AVHRR archive images from the stage directory'
    echo '       to the appropriate AVHRR image directory on the DAAC stage area.'
    echo '       This function should be run as a cron job to ensure all AVHRR'
    echo '       images in the stage directory are distributed for user access'
    echo '       on a timely basis.  This function can only be run on edcsgs20'
    echo '       as ops, opsavhrr, or dtrans.  To run on another'
    echo '       system or as another user, please contact software'
    echo '       engineering.'
    exit 1
