/*******************************************************************************
FUNCTION:	prodgen_get

PURPOSE:	Given an archive image, determine the products that need to be
		created for the standing orders.  All standing orders are
		contained in the prodgen.table file in the ADAPSTABLES
		directory.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    08/92  J.Fenno          Development
  1.1    05/93  J.Fenno          Modified to use the new model.
  1.2    08/94  T.Baltzer        Add HOLD, RM option.
  3.3    03/96  A. Vander Vorste Added output dir to prodgen.table and products 
				 300 and 301 to put level0 and level1 files to 
				 a specified output directory.


PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "util.h"
#include "archio.h"
#include "adapsutil.h"
#include "queue.h"
#include "products.h"


/* Global variables
-------------------*/
long   std_out;           /* Standard output for TAE                          */
struct PARBLK vblock;     /* TAE parameter block                              */


main()
{
char hostname[CMLEN];     /* Host file name of the archive image              */
char key[CMLEN];          /* Key to be used for all calls to c_wrtlog         */
char logfile[CMLEN];      /* Host file specification of the log file          */
char *ch;                 /* String pointer                                   */
char string[CMLEN];       /* Temporary string                                 */
char table[CMLEN];        /* Name of the standing order table in ADAPSTABLES  */
char *bands[MXPRODS];     /* Band specification for products to be produced   */
char *output[MXPRODS];    /* Output media/dir specification                   */

long access;              /* File access flag                                 */
int accounts[MXPRODS];   /* Account numbers requesting the products          */
long fatal    = -1;       /* Fatal error flag                                 */
long i;                   /* Loop control                                     */
long lock     = LOCK;     /* Flag for c_lockfil to lock a file                */
long nerrors;             /* # of errors that occur wile processing orders    */
int nl[MXPRODS];         /* # of lines required for each product             */
long nlines;              /* # of lines to be included in a single product    */
long nonfatal =  1;       /* Nonfatal error flag                              */
long norders;             /* # of standing orders to fill                     */
long npixels;             /* # pixels to skip between points along image edge */
int ns[MXPRODS];         /* # of samples required for each product           */
int orders[MXPRODS];     /* Order numbers associated with each account number*/
long orders_q;            /* Q of standing orders read from the table         */
long points_q;            /* Q of points surrounding the edge of the image    */
int products[MXPRODS];   /* Product code for each product to be produced     */
int sl[MXPRODS];         /* Starting line for each product to be produced    */
long sline;               /* Starting line for a single product               */
int ss[MXPRODS];         /* Starting sample required for each product        */
long unlock   = UNLOCK;   /* Flag for c_lockfil to unlock a file              */
long zero     = 0;        /* Value of 0                                       */

void *ptr;                /* Void ptr for calls to c_qfirst()                 */

struct DESCRIPT descript; /* Struct containing description lines from table   */
struct DROPPED  dropped;  /* Struct to contain the dropped line information   */
struct MODEL    model;    /* Struct to contain the SFL archive header         */
struct ORDER    order;    /* Struct for a single standing order               */
struct VARIABLE *parm;    /* Ptr to a TAE parameter                           */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Get the name of the archive image
------------------------------------*/
access = IREAD;
if (c_getnam(&vblock, "IN", &zero, hostname, "arch", &access) != E_SUCC)
    exit(1);
i = (long)strlen(hostname);


/* Get the number of pixels to skip around the edge of the image
----------------------------------------------------------------*/
parm = p_find(&vblock, "NPIXELS");
npixels = IVAL(*parm, 0);


/* Get the key used in the log file
-----------------------------------*/
parm = p_find(&vblock, "KEY");
strcpy(key, SVAL(*parm, 0));


/* Get the name of the log file
-------------------------------*/
access = IUPDATE;
if (c_getnam(&vblock, "LOGFILE", &zero, logfile, "log", &access) != E_SUCC)
    exit(1);


/* Get the name of the prodgen;table file in ADAPSTABLES
--------------------------------------------------------*/
ch = (char *)getenv("ADAPSTABLES");
if (ch == NULL)
    {
    sprintf(string, "Error retrieving ADAPSTABLES");
    c_wrtlog(logfile, key, "prodgen-error", string);
    c_errmsg(string, "prodgen-env", &fatal);
    }
strcpy(table, ch);
#ifdef unix
strcat(table, "/");
#endif
strcat(table, TABLE);


/* Read the archive header file
-------------------------------*/
if (c_getahdr(hostname, &model, &dropped) != E_SUCC)
    {
    sprintf(string, "Error reading header for %s", hostname);
    c_wrtlog(logfile, key, "prodgen-error", string);
    c_errmsg(string, "prodgen-header", &fatal);
    }


/* Lock the prodgen table
-------------------------*/
i = 0;
while (c_lockfil(table, &lock) != E_SUCC)
    {
    i++;
    if (i > 10)
	{
	sprintf(string, "Unable to lock table");
	c_wrtlog(logfile, key, "prodgen-error", string);
	c_errmsg(string, "prodgen-nolock", &fatal);
	}
    sleep(5);
    }


/* Read all of the standing orders from the table
-------------------------------------------------*/
if (c_pgread(&descript, &orders_q))
    {
    sprintf(string, "Error reading the standing order table");
    c_wrtlog(logfile, key, "prodgen-error", string);
    c_errmsg(string, "prodgen-read", &fatal);
    }


/* Free the character buffer used to store the description
   portion of the table and unlock the table
----------------------------------------------------------*/
free(descript.buffer);
if (c_lockfil(table, &unlock) != E_SUCC)
    {
    sprintf(string, "Error unlocking table");
    c_wrtlog(logfile, key, "prodgen-error", string);
    c_errmsg(string, "prodgen-unlock", &nonfatal);
    }


/* Create a list of points that surround the edge of the image
--------------------------------------------------------------*/
if (c_pgpoints(&model, npixels, &points_q))
    {
    c_qdestroy(orders_q);
    sprintf(string, "Error selecting points");
    c_wrtlog(logfile, key, "prodgen-error", string);
    c_errmsg(string, "prodgen-pts", &fatal);
    }


/* Determine if the image intersects the area
   of interest for any of the standing orders
---------------------------------------------*/
nerrors = 0;
norders = 0;
while (!c_qempty(orders_q))
    {
    /* Retrieve the first standing order string from the
       queue and convert it to a standing order structure
    -----------------------------------------------------*/
    c_qfirst(orders_q, &ptr);
    c_pg2struct((char *)ptr, &order);


    /* Determine if the image should be processed for the current order
    -------------------------------------------------------------------*/
    switch (c_pgtest(&model, &order, points_q, &sline, &nlines))
	{
	case PROCESS:
	    if (norders == MXPRODS)
		{
		/* Issue a warning message because we cannot pass more than
		   MXPRODS product orders back through the TAE variable block
		-------------------------------------------------------------*/
		sprintf(string, "Account %09d, order %03d was not processed",
		    order.account, order.ordernum);
		c_wrtlog(logfile, key, "prodgen-error", string);
		c_errmsg(string, "prodgen-max", &nonfatal);
		}
	    else
		{
		/* Store the account number, product code, starting line,
		   starting sample, number of lines, and number of samples
		----------------------------------------------------------*/
		accounts[norders] = order.account;
		orders[norders]   = order.ordernum;
		products[norders] = order.product_code;
		sl[norders]       = sline;
		ss[norders]       = 1;
		nl[norders]       = nlines;
		ns[norders]       = model.nsamples;


		/* Allocate space for the band specification and
		   then create the band specification in a string
		-------------------------------------------------*/
		bands[norders] = (char *)malloc(CMLEN);
		if (bands[norders] == NULL)
		    {
		    c_qdestroy(orders_q);
		    c_qdestroy(points_q);
		    sprintf(string, "Error allocating memory");
		    c_wrtlog(logfile, key, "prodgen-error", string);
		    c_errmsg("Error allocating memory","prodgen-alloc",&fatal);
		    }
		strcpy(bands[norders], "");
		for (i = 0; i < 5; i++)
		    if ((order.bands[i]) && (strlen(bands[norders]) == 0))
			sprintf(bands[norders], "%c", ('1' + i));
		    else if (order.bands[i])
			{
			sprintf(string, " %c", ('1' + i));
			strcat(bands[norders], string);
			}

                /* Allocate space for the band specification and
                   then create the band specification in a string
                -------------------------------------------------*/
		output[norders] = (char *)malloc(CMLEN);
                strcpy(output[norders], order.output);
                if (output[norders] == NULL)
                    {
                    c_qdestroy(orders_q);
                    c_qdestroy(points_q);
                    sprintf(string, "Error getting OUTPUT from order");
                    c_wrtlog(logfile, key, "prodgen-error", string);
                    c_errmsg(string,"prodgen-error",&fatal);
                    }
                norders++;
                }

	    break;

	case NOPROCESS:
	    break;

	default:
	    sprintf(string, "Error processing account %09d, order %03d",
		order.account, order.ordernum);
	    c_wrtlog(logfile, key, "prodgen-error", string);
	    c_errmsg(string, "prodgen-order", &nonfatal);
	    nerrors++;
	    break;
	}


    /* Delete the first order from the queue
    ----------------------------------------*/
    c_qdelete(orders_q, 1);
    }


/* Destroy the queue of orders and points around the image's edge
-----------------------------------------------------------------*/
c_qdestroy(orders_q);
c_qdestroy(points_q);


/* If all standing orders error'd then return a bad status
----------------------------------------------------------*/
if ((norders == 0) && (nerrors > 0))
    exit(-1);


/* Send the account numbers, product codes, starting lines, starting samples,
   number of lines, number of samples, and the bands specifications to TAE.
   If norders is 0 we must still return these parameters so their counts are
   set to zero.
-----------------------------------------------------------------------------*/
if (q_intg(&vblock,   "ACCOUNTS", norders, accounts, P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_intg(&vblock,   "ORDERS",   norders, orders,   P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_intg(&vblock,   "PRODUCTS", norders, products, P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_intg(&vblock,   "SL",       norders, sl,       P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_intg(&vblock,   "SS",       norders, ss,       P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_intg(&vblock,   "NL",       norders, nl,       P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_intg(&vblock,   "NS",       norders, ns,       P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_string(&vblock, "BANDS",    norders, bands,    P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_string(&vblock, "OUTPUT",   norders, output,   P_UPDATE) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);
if (q_out(&vblock) != SUCCESS)
    c_errmsg("Error sending variables to TAE", "prodgen-pass", &fatal);

z_exit(1, "PRODGEN_GET completed successfully");
/* to make compiler happy */
return 0;
}
