#!/bin/csh -f
#
# FUNCTION: readarch.csh
#
# PURPOSE:  Acquire images from tape to disk.
#
#
# Version  Programmer       Date   Change Request
# -------  ---------------  -----  ---------------------------------------------
#   1.2    J.Fenno          07/93  Stolen from readeaf
#
#


#---------------------------
# On interrupt go to CLEANUP
#---------------------------
onintr CLEANUP
set noglob


#------------------------------
# Check the number of arguments
#------------------------------
if ($#argv != 7) goto USAGE


#---------------------
# Set up the variables
#---------------------
set job      = $1
set drive_no = $2
set tape_id  = `echo $3 | tr "[A-Z]" "[a-z]"`
set destdir  = $4
set ttype    = `echo $5 | tr "[A-Z]" "[a-z]"`


#---------------------------------------------------------
# Set up aliases for checkspace, get_device, and write_log
#---------------------------------------------------------
if ("$6" == ".") then
    alias checkspace `pwd`/checkspace
else
    alias checkspace $6/checkspace
endif
if ("$7" == ".") then
    alias get_device `pwd`/get_device
else
    alias get_device $7/get_device
endif


#---------------------------------------------------------------------------
# Unalias all commands used in this shell script to get rid of user aliases.
# If there is a /usr/bsd/rsh then use this command instead of the /bin/rsh
# because the Berkley version actually works with remote systems while the
# /bin/rsh version does not.  If there isn't a /usr/bsd/rsh then /bin/rsh
# usually works.
#---------------------------------------------------------------------------
unalias cd
unalias dd
unalias mt
unalias rm
unalias rsh
if (-e /usr/bsd/rsh) alias rsh /usr/bsd/rsh


#--------------------------------------------------------------------
# Set up the processing variables.  $logfile must be a LAS file name.
#--------------------------------------------------------------------
set disk_space    = "NO"
set eov_reached   = "NO"
set free_bytes    = 150000000
set remote_system = "NO"


#------------------------------------------------------------------------------
# Get the device name of the drive and set the TAPE environment variable for mt
#------------------------------------------------------------------------------
set drive_name = `get_device $drive_no | awk '{print $1}'`
setenv TAPE $drive_name


#-------------------------------------------------------------
# Determine if we are using a drive on a remote system.  This
# is indicated by having a drive number of the form edcsgs4@4.
#-------------------------------------------------------------
set system_name = `uname -n`
set drive_num = `echo $drive_no | awk -F@ '{print $2}'`
if ($drive_num != "") then
    set system_name = `echo $drive_no | awk -F@ '{print $1}'`
    set remote_system = "YES"


    #--------------------------------------------------------------
    # Check to see if the user can do rshells to this remote system
    #--------------------------------------------------------------
    if ( ! { rsh $system_name "date >> /dev/null" } ) then
	echo "[readarch-rsh] Permission denied to remote system $system_name"
	exit 1
    endif
endif


#------------------------------------------------------------------------------
# Create an alias for the dd and mt commands if using a remote system.
#
# NOTE: Silicon Graphics systems use a -t option for the mt command while all
#       other systems us a -f option.
#------------------------------------------------------------------------------
if ($remote_system == "YES") then
    #-------------
    # Alias for mt
    #-------------
    if (("$system_name" =~ sg*) || ("$system_name" =~ edcsg*)) then
	alias mt rsh $system_name mt -t $drive_name
    else
	alias mt rsh $system_name mt -f $drive_name
    endif

    #-------------
    # Alias for dd
    #-------------
    alias dd rsh $system_name dd
endif


#--------------------------
# Set up a unique file name
#--------------------------
set tmpname = $destdir${job}_${tape_id}.eaf


#---------------------------
# Set up the tape block size
#---------------------------
switch ("$ttype")
    case ea1:
	set blksz = 65024
	breaksw
    case ea2:
	set blksz = 32768
	breaksw
    default:
	echo "[readarch-ttype] Unknown tape type $ttype"
	exit 1
	breaksw
    endsw


#--------------
# Log the start
#--------------
echo "[informational] Starting the READARCH_ACQ procedure"


#---------------------
# Check for disk space
#---------------------
set i = 0
while (($disk_space == "NO") && ($i < 6))
    @ i += 1
    if ({ checkspace "" $free_bytes $destdir }) then
	set disk_space = "YES"
    else
	#-------------------------------------------
	# If not enough space, sleep 5 min & recheck
	#-------------------------------------------
	echo "[informational] READARCH_ACQ waiting for disk space"
	sleep 300
    endif
end
      
#----------------------------------------------------
# Issue an error message if no space became available
#----------------------------------------------------
if ($disk_space == "NO") then
    echo "[readarch-disk] Disk space not available for 30 minutes"
    exit 1
else if ($i > 1) then
    echo "[informational] Disk space available.  Continuing"
endif


#---------------------------------
# Copy the image from dist to tape
#---------------------------------
if (! { mt fsf >>& /dev/null }) goto FSFERR
if (! { (dd if=$drive_name bs=$blksz > $tmpname) }) goto READERR


#----------------------------
# Check the size of the image
#----------------------------
if (-z $tmpname) then
    echo "[readarch-read] Error reading image(s) from tape"
    rm -f $tmpname
    exit 1
endif


#-----------------------------------------------------------
# Issue a message stating how many files were read from tape
#-----------------------------------------------------------
echo "[informational] Read 1 image from drive $drive_no"
exit 0



#---------------
# User interrupt
#---------------
CLEANUP:
    onintr -
    echo "[readarch-user] User interrupt"
    rm -f $tmpname
    exit 1


#---------------------------
# Error forward spacing file
#---------------------------
FSFERR:
    echo "[readarch-fsf] Error forward skipping file"
    rm -f $tmpname
    exit 1


#--------------------
# Error reading image
#--------------------
READERR:
    set text = "Error reading image $file_no from tape"
    echo "[readarch-read] Error reading image(s) from tape"
    rm -f $tmpname
    exit 1


#--------------------
# Usage error message
#--------------------
USAGE:
    echo ''
    echo 'USAGE: readarch job drive tape dir type cpath gpath'
    echo ''
    echo 'WHERE:'
    echo '   job     = Process ID for unique naming of files'
    echo '   drive   = Tape drive number'
    echo '   tape    = 6 character tape ID'
    echo '   dir     = Host destination directory'
    echo '   type    = Type of tape being read (EA1, EA2)'
    echo '   cpath   = Path for the command checkspace'
    echo '   gpath   = Path for the command get_device'
    echo ''
    exit 1
