/*******************************************************************************
FUNCTION:	readarch_new

PURPOSE:	Read AVHRR archive images from an ANSINEW archive tape.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    10/92  J.Fenno          Development
  2.1    03/93  T.Baltzer        Include orbit stitched images and lower case
                                 scene id's
  2.2    07/93  J.Fenno          Stolen from READARCH


PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "adaps.h"


/* System return status
=======================*/
#ifdef vms
#define SYS_SUCC      1    /* VMS  successful exit status                     */
#else
#define SYS_SUCC      0    /* UNIX successful exit status                     */
#endif


main()
{
struct PARBLK   vblock;    /* TAE parameter block                             */
struct VARIABLE *p_find(); /* Function to retrieve a TAE variable             */
struct VARIABLE *parm;     /* Pointer to TAE variable within vblock           */

long i;			   /* Counter					      */
long len;		   /* Length of the scene id			      */
long nimages  =  0;        /* Number of archive images read to disk           */
long nscenes;		   /* Number of scenes specified in scene_ids	      */
long fatal    = -1;        /* Fatal error code                                */
long stdlun;               /* Standard output                                 */

char *images[20];          /* Scenes actually copied to disk by ansiin        */
char drive[CMLEN];         /* Tape drive the tape is mounted on               */
char outdir[CMLEN];        /* Disk location where the images should be placed */
char scene_ids[20][20];	   /* Scene IDs requested by the user                 */
char str[CMLEN];           /* Temporary string                                */
char sys_command[4*CMLEN]; /* Command sent to the operating system            */
char tapeid[CMLEN];        /* Tape ID                                         */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &stdlun);


/* Get the tape drive that the tape is mounted on
-------------------------------------------------*/
parm = p_find(&vblock, "DRIVE");
strcpy(drive, SVAL(*parm, 0));


/* Get the tape ID
------------------*/
parm = p_find(&vblock, "TAPEID");
strcpy(tapeid, SVAL(*parm, 0));


/* Get the scene ID of the images to be read.  Lower case the scene ID and
   add a '.' since all files written to an archive tape end in a '.'
--------------------------------------------------------------------------*/
parm = p_find(&vblock, "SCENEID");
nscenes = parm->v_count;
for (i = 0; i < nscenes; i++)
    {
    strcpy(scene_ids[i], SVAL(*parm, i));
    len = strlen(scene_ids[i]);
    c_up2low(scene_ids[i], &len);
    strcat(scene_ids[i], ".");
    }


/* Get the directory where the image should be placed and then chdir to that
   directory.  After calling c_cmhost the last character for UNIX directories
   should be a '/' and for VMS it should be ']'.
-----------------------------------------------------------------------------*/
parm = p_find(&vblock, "OUTDIR");
if (parm->v_count != 0)
    {
    strcpy(str, SVAL(*parm, 0));
    c_cmhost(str, outdir);
    strcat( outdir, "/" );
    if (outdir[strlen(outdir) - 1] != HOST_DIR_END)
	c_errmsg("Invalid directory specification","readarch-outdir",&fatal);
    if (chdir(outdir))
	c_errmsg("Error moving to specified directory","readarch-chdir",&fatal);
    }


#ifdef vms

    /* Set up the ansiin command that will read the scene(s) from tape.  VMS
       systems must have double quotes around the argument string so that all
       lowercase letters are not converted to uppercase.
    -------------------------------------------------------------------------*/
    sprintf(sys_command, "ansiin \"-Tvl\" %s %s", drive, tapeid);
    if (strcmp(scene_ids[0], "*.") != 0)
	for (i = 0; i < nscenes; i++)
	    {
	    /* Add a specification for the header file(s)
	    ---------------------------------------------*/
	    strcat(sys_command, " ");
	    scene_ids[i][0] = 'h';
	    strcat(sys_command, scene_ids[i]);

	    /* Add a specification for the image file(s)
	    --------------------------------------------*/
	    strcat(sys_command, " ");
	    scene_ids[i][0] = 'i';
	    strcat(sys_command, scene_ids[i]);
	    }

#else

    /* Set up the ansiin command that will read the scene(s) from tape.  UNIX
       systems must have single quotes around the scene IDs to protect any
       asterisks from shell expansion.
    -------------------------------------------------------------------------*/
    sprintf(sys_command, "ansiin -Tvl %s %s", drive, tapeid);
    if (strcmp(scene_ids[0], "*.") != 0)
	for (i = 0; i < nscenes; i++)
	    {
	    /* Add a specification for the header file(s)
	    ---------------------------------------------*/
	    strcat(sys_command, " '");
	    scene_ids[i][0] = 'h';
	    strcat(sys_command, scene_ids[i]);
	    strcat(sys_command, "' '");

	    /* Add a specification for the image file(s)
	    --------------------------------------------*/
	    scene_ids[i][0] = 'i';
	    strcat(sys_command, scene_ids[i]);
	    strcat(sys_command, "'");
	    }

#endif


/* Make the system call to ansiin
---------------------------------*/
if (system(sys_command) != SYS_SUCC)
    c_errmsg("Error reading image(s) from tape", "readarch-read", &fatal);


/* Rename all of the files read from tape
-----------------------------------------*/
if (strcmp(scene_ids[0], "*.") == 0)
    {
    /* May have GAC, HRPT, LAC or stitched images intermixed on a single tape
    -------------------------------------------------------------------------*/
    rename_files("hg*.", "ahdr", images, &nimages);
    rename_files("hh*.", "ahdr", images, &nimages);
    rename_files("hl*.", "ahdr", images, &nimages);
    rename_files("ho*.", "ahdr", images, &nimages);
    rename_files("ig*.", "arch", images, &nimages);
    rename_files("ih*.", "arch", images, &nimages);
    rename_files("il*.", "arch", images, &nimages);
    rename_files("io*.", "arch", images, &nimages);
    }
else
    {
    /* Only header and image files matching scene_ids will have been read from
       tape.  Therefore, only rename the header and image files that match the
       scene_ids.
    --------------------------------------------------------------------------*/
    for (i = 0; i < nscenes; i++)
	{
	scene_ids[i][0] = 'h';
	rename_files(scene_ids[i], "ahdr", images, &nimages);
	scene_ids[i][0] = 'i';
	rename_files(scene_ids[i], "arch", images, &nimages);
	}
    }


/* Return the LAS names of each archive image read to disk
----------------------------------------------------------*/
q_string(&vblock, "SCENES", nimages, images, P_UPDATE);
if (q_out(&vblock) != SUCCESS)
    {
    strcpy(str, "Error sending parameters to TAE");
    c_errmsg(str, "readarch-parm", &fatal);
    }
for (i = 0; i < nimages; i++)
    free(images[i]);

z_exit(1, "READARCH_NEW completed successfully");
}
