/*******************************************************************************
FUNCTION:	readarch_sid

PURPOSE:	Check the scene IDs for a specified AVHRR archive tape type.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    07/93  J.Fenno          Development

PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "las.h"
#include "adaps.h"

main()
{
struct PARBLK   vblock;		/* TAE parameter block                        */
struct VARIABLE *p_find();      /* Function for locating a TAE parameter      */
struct VARIABLE *parm;          /* Ptr to a TAE parameter                     */

char *scene_ids[20];		/* Scene IDs specified by the user            */
char str[CMLEN];		/* Temporary string variable                  */
char *strptr[1];                /* Array for passing strings to TAE           */
char ttype[CMLEN];		/* Type of AVHRR archive tape                 */
char year[3];			/* Two digit year extracted from the scene ID */

long fatal    = -1;             /* Fatal error flag                           */
long i;				/* Loop control                               */
long j;				/* Loop control                               */
long length;			/* Length of the scene ID                     */
long nscenes;			/* Number of scene IDs specified by the user  */
long std_out;			/* Standard output used by LAS                */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the user specified scene IDs and lowercase them
-----------------------------------------------------------*/
parm = p_find(&vblock, "SCENEID");
nscenes = parm->v_count;
for (i = 0; i < nscenes; i++)
    {
    scene_ids[i] = (char *)malloc(17);
    if (scene_ids[i] == NULL)
	c_errmsg("Error allocating memory", "readarch-alloc", &fatal);
    strcpy(scene_ids[i], SVAL(*parm, i));
    length = strlen(scene_ids[i]);
    c_up2low(scene_ids[i], &length);
    }


/* Retrieve the type of tape
----------------------------*/
parm = p_find(&vblock, "TAPETYPE");
strcpy(ttype, SVAL(*parm, 0));


/* Depending upon the type of tape, inspect the scene IDs
---------------------------------------------------------*/
if ((strcmp(ttype, "ANSINEW") == 0) || (strcmp(ttype, "ANSIOLD") == 0))
    {
    for (i = 0; i < nscenes; i++)
	{
	/* If the scene ID is less than 16 characters and the last character
	   isn't an asterisk then add an implied one.  Leave length equal to
	   the index position of the last character in the scene ID.
	--------------------------------------------------------------------*/
	length = strlen(scene_ids[i]);
	if ((length < 16) && (scene_ids[i][length - 1] != '*'))
	    strcat(scene_ids[i], "*");
	else
	    length--;


	/* This loop has some inefficiencies however, it has the
	   fewest nested if statements to get the job done.
	--------------------------------------------------------*/
	for (j = 0; j <= length; j++)
	    {
	    /* Allow scene IDs ending with an asterisk
	    ------------------------------------------*/
	    if ((scene_ids[i][j] == '*') && (j == length))
		break;

	    /* Ensure the scene ID starts with an 'a'
	    -----------------------------------------*/
	    if (scene_ids[i][0] != 'a')
		{
		sprintf(str, "Character 1 of scene ID %1d is invalid", i + 1);
		c_errmsg(str, "readarch-sid", &fatal);
		}
	    
	    /* Ensure the second character is a 'g', 'h', 'l', or 'o'
	    ---------------------------------------------------------*/
	    if ((j == 1) &&
		(scene_ids[i][j] != 'g') && (scene_ids[i][j] != 'h') &&
		(scene_ids[i][j] != 'l') && (scene_ids[i][j] != 'o'))
		{
		sprintf(str, "Character 2 of scene ID %1d is invalid", i + 1);
		c_errmsg(str, "readarch-sid", &fatal);
		}
	    
	    /* Ensure all characters beyond the second, are digits
	    ------------------------------------------------------*/
	    if ((j > 1) && !isdigit(scene_ids[i][j]))
		{
		sprintf(str, "Character %1d of scene ID %1d is invalid",
		    j + 1, i + 1);
		c_errmsg(str, "readarch-sid", &fatal);
		}
	    }
	}


    /* Return a bogus year because it will be contained
       in the scene ID located on the tape for each image
    -----------------------------------------------------*/
    strcpy(year, "xx");
    }
else
    {
    /* There is only one image stored on every EA1, EA2, and JULIAN tape
    --------------------------------------------------------------------*/
    if (nscenes > 1)
	{
	sprintf(str, "Only one scene stored on an %s tape", ttype);
	c_errmsg(str, "readarch-nscenes", &fatal);
	}


    /* Scene IDs must be 16 characters long for these tapes because there
       are no scene IDs stored on the tapes from which we can extract an
       acquisition year.
    ---------------------------------------------------------------------*/
    if (strlen(scene_ids[0]) < 16)
	{
	strcpy(str, "Scene ID must be 16 characters long");
	c_errmsg(str, "readarch-sidlen", &fatal);
	}


    /* Ensure the scene ID starts with an 'a'
    -----------------------------------------*/
    if (scene_ids[0][0] != 'a')
	{
	strcpy(str, "Character 1 of scene ID 1 is invalid");
	c_errmsg(str, "readarch-sid", &fatal);
	}

    
    /* Ensure the second character is a 'g', 'h', 'l', or 'o'
    ---------------------------------------------------------*/
    if ((scene_ids[0][1] != 'g') && (scene_ids[0][1] != 'h') &&
	(scene_ids[0][1] != 'l') && (scene_ids[0][1] != 'o'))
	{
	strcpy(str, "Character 2 of scene ID 1 is invalid");
	c_errmsg(str, "readarch-sid", &fatal);
	}


    /* Ensure the remaining characters are digits
    ---------------------------------------------*/
    for (j = 2; j < 16; j++)
	{
	if (!isdigit(scene_ids[0][j]))
	    {
	    sprintf(str, "Character %1d of scene ID 1 is invalid", j + 1);
	    c_errmsg(str, "readarch-sid", &fatal);
	    }
	}
    

    /* Extract the year from the scene ID
    -------------------------------------*/
    year[0] = scene_ids[0][8];
    year[1] = scene_ids[0][9];
    year[3] = '\0';
    }


/* Return the scene ID and year to the TAE parameter block
----------------------------------------------------------*/
q_string(&vblock, "SCENEID", nscenes, scene_ids, P_UPDATE);

strptr[0] = year;
q_string(&vblock, "YEAR", 1, strptr, P_UPDATE);
if (q_out(&vblock) != SUCCESS)
    {
    strcpy(str, "Error sending parameters to TAE");
    c_errmsg(str, "readarch-parm", &fatal);
    }


/* Free all of the allocated memory
-----------------------------------*/
for (i = 0; i < nscenes; i++)
    free(scene_ids[i]);

z_exit(1, "READARCH_SID completed successfully");
}
