/*******************************************************************************
FUNCTION:	ang_elev_geo

PURPOSE:	Create the angles, elevation, and geometric mapping grids.  An
		angles grid contains the satellite zenith, solar zenith, and
		the absolute difference of the satellite and solar azimuth
		angles for each grid point.  The elevation grid contains the
		elevation for each grid point.  The geometric mapping grid
		contains the line and sample location for each grid point in
		the input image.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    ??/??  ?.???            Development
  1.1    03/93  J.Fenno          Added code to create the angles and elevation
                                 grids.
  1.2    09/93  J.Fenno          Modified so angles match the AVHRR model
  1.3	 02/97  D.Etrheim	 Modified to request only read one line from					 elevation image.  This was necessary after
				 switching to gtopo30 as the elevation image.
  1.4    02/99  D.Lloyd          Moved call to c_transinit() outside of main
                                 processing loop

PROJECT:  ADAPS
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "rectify.h"


extern long curent;        /* Current grid point for processing messages      */


int ang_elev_geo
(
struct GRID   *anggrid, /* IN/OUT - The partially complete angles grid    */
struct GRID   *elevgrid,/* IN/OUT - The partially complete elevation grid */
struct GRID   *geogrid, /* IN/OUT - The partially complete geo. mapping   */
struct FRAME  *frame,   /* IN  - Output framing parameters                */
struct MODEL  *model,   /* IN  - AVHRR model data structure               */
struct PROJ   *outp,    /* IN  - Output projection parameters             */
long   mline,           /* IN  - Master line for the input image          */
long   msample,         /* IN  - Master sample for the input image        */
long   msgtime          /* IN  - # of seconds between processing messages */
)
{
double all_zero[15];       /* Zero projection parameters                      */
double angles[N_ANGLES];   /* Satellite/Solar Zenith and relative azimuth     */
double *elev;              /* Elevation of a grid point                       */
double outx;               /* Projection coordinate of x after Xform          */
double outy;               /* Projection coordinate of y after Xform          */
double *geoptrs[2];        /* Ptrs used to walk down geogrid->values[0-1]     */
double proj_x;             /* Projection coordinate of a grid point           */
double proj_y;             /* Projection coordinate of a grid point           */

float  *angptrs[N_ANGLES]; /* Ptrs used to walk down anggrid->values[]        */
float  in_line;            /* Line # of a grid point in the input image       */
float  in_samp;            /* Sample # of a grid point in the input image     */
float  nadir;              /* Nadir sample of the AVHRR image                 */

long   *cur_col;           /* Ptr to the columns in the grid (sample numbers) */
long   *cur_row;           /* Ptr to the rows in the grid (line numbers)      */
long   degrees   =  4;     /* Units code for geographic projection (degrees)  */
long   elevlui;            /* Ptr to the elevation Look Up Image (LUI)        */
long   geo       =  0;     /* Geographic projection flag                      */
long   i;                  /* Loop control                                    */
long   j;                  /* Loop control                                    */
long   line;               /* Line number in the elevation image              */
long   msgtype   = 3;      /* Flag for the processing message type            */
long   npts;               /* Number of grid points (nrows * ncols)           */
long   one       = 1;      /* Constant value one                              */
long   sample;             /* Sample number in the elevation image            */
long   status;             /* Return status from a function                   */
long   prtprm[2];          /* Print flags for c_transinit                     */
long   utm       = 62;     /* UTM projection code                             */

short  elevflag;           /* Flag indicating a user specified elevation image*/
short  wordval;            /* I*2 variable for retrieving the elevation       */



/* Calculate the number of grid points
--------------------------------------*/
npts = anggrid->nrows * anggrid->ncols;


/* Allocate memory for the angles returned by the AVHRR orbital model
---------------------------------------------------------------------*/
anggrid->nvalues = 0;
for (i = 0; i < N_ANGLES; i++)
    {
    anggrid->values[i] = (void *)malloc(npts * sizeof(float));
    if (anggrid->values[i] == NULL)
	return(NO_SPACE);
    anggrid->nvalues++;
    }


/* Allocate memory for the elevations to be stored in the elevation grid
------------------------------------------------------------------------*/
elevgrid->values[0] = (void *)malloc(npts * sizeof(double));
if (elevgrid->values[0] == NULL)
    return(NO_SPACE);
elevgrid->nvalues = 1;


/* Allocate memory for the line and sample values in the geometric grid
-----------------------------------------------------------------------*/
geogrid->nvalues = 0;
for (i = 0; i < 2; i++)
    {
    geogrid->values[i] = (void *)malloc(npts * sizeof(double));
    if (geogrid->values[i] == NULL)
	return(NO_SPACE);
    geogrid->nvalues++;
    }


/* Determine if the user specified an elevation image
-----------------------------------------------------*/
if (strlen(frame->elev.hostname) == 0)
    elevflag = FALSE;
else
    {
    elevflag = TRUE;

    /* Load the Look Up Image (LUI) using the calculated window
    -----------------------------------------------------------*/
    i = 1; /* Number of lines to load into memory */
    status = c_loadlui(frame->elev.hostname, frame->elev.window, EWORD,
	&elevlui, &i);
    if (status != E_SUCC)
	return(E_FAIL);
    }


/* Determine the nadir sample of the input image
------------------------------------------------*/
if (model->dtype == GAC)
    nadir = GAC_NADIR;
else
    nadir = LAC_NADIR;


/* Set the pointers used to walk through the values fields of each grid
-----------------------------------------------------------------------*/
angptrs[SATZEN] = (float *)anggrid->values[SATZEN];
angptrs[SOLZEN] = (float *)anggrid->values[SOLZEN];
angptrs[RELAZ]  = (float *)anggrid->values[RELAZ];
elev       = (double *)elevgrid->values[0];
geoptrs[0] = (double *)geogrid->values[0];
geoptrs[1] = (double *)geogrid->values[1];


/* Start the processing message
-------------------------------*/
curent = 1;
c_inittm("RECTIFY", "Generated", "grid point", "", "for geometric mapping",
    &npts, &one, &msgtime, &msgtype);
c_startm();


/* Initialize parameters for projection and datum transformations
-----------------------------------------------------------------*/
for (i = 0; i < 15; i++)
    all_zero[i] = 0.0;
status = c_transinit(&outp->proj_code, &outp->units, &outp->zone_code,
    &outp->datum_code, outp->proj_coef, &geo, &degrees, &utm, 
    &outp->datum_code, all_zero, prtprm, "\0");
if (status != E_SUCC)
    {
    c_closelui(elevlui);
    return(NO_RECTIFY);
    }


/* Calculate the 3 angles, elevation, and the line
   and sample location for each of the grid points.
---------------------------------------------------*/
for (i = 0, cur_row = anggrid->out_lines; i < anggrid->nrows; i++, cur_row++)
    {
    /* Compute the Y or vertical projection coordinate for the current grid row
    --------------------------------------------------------------------------*/
    proj_y = frame->corners[0] - ((*cur_row - 1) * frame->o_dist[0]);

    for (j = 0, cur_col = anggrid->out_samps; j < anggrid->ncols; j++,cur_col++)
	{
	/* Compute the X or horizontal projection
	   coordinate for the current grid point
	-----------------------------------------*/
	proj_x = frame->corners[1] + ((*cur_col - 1) * frame->o_dist[1]);


	/* Calculate the latitude and longitude of the current grid point
	-----------------------------------------------------------------*/
	status = c_trans(&outp->proj_code, &outp->units, &geo, &degrees, 
            &proj_x, &proj_y, &outx, &outy);
	if (status != E_SUCC)
	    {
	    c_closelui(elevlui);
	    return(NO_RECTIFY);
	    }


	/* Get the elevation for the current grid point
	-----------------------------------------------*/
	if (elevflag)
	    {
	    /* Convert the latitude and longitude of the grid
	       point to a line sample for the elevation image
	    -------------------------------------------------*/
	    c_geo2ls(&frame->elev.ullat, &frame->elev.ullon,
		&frame->elev.pdistx, &frame->elev.pdisty, &outy,
		&outx, &line, &sample);
	    line += 1;     /* line   is the offset from pixel 1,1 */
	    sample += 1;   /* sample is the offset from pixel 1,1 */


	    /* Get the elevation for this grid point
	    ----------------------------------------*/
	    status = c_getlui(elevlui, line, sample, (unsigned char *)&wordval);
	    if (status != E_SUCC)
		{
		c_closelui(elevlui);
		return(NO_RECTIFY);
		}
	 

	    /* Convert the I*2 value to a double
	    ------------------------------------*/
	    *elev = (double)wordval;
	    }
	else
	    *elev = 0.0;


	/* Convert the latitude/longitude to radians
	--------------------------------------------*/
	outy *= RAD_PER_DEG;
	outx *= RAD_PER_DEG;


	/* Given the latitude, longitude, and the elevation of the current grid
	   point get the line, sample, absolute difference of satellite and
	   solar azimuth angles, satellite zenith angle, and the solar zenith
	   angle for this point in the input image.
	----------------------------------------------------------------------*/
	c_ll2ls(&outy, &outx, elev, model, &in_line, &in_samp, angles);


	/* Satellite zenith angles are negative from the first
	   scanned pixel to nadir and positive after nadir.
	Note: Added for clarification 09/14/2001 DEL.  The above
        statement is correct as well as the logic below.  Since
        the image is not in satellite view, all line sample values
        are in image space not satellite space.  The variable
	in_samp below is the sample value from ll2ls() which has
	been modified to be north up ( sample 1 is the leftmost
	pixel in the image, sample 2048 the rightmost).
        For an ASCENDING pass the image was actually scanned
        right to left which is why (in_samp > nadir) is
        correct.  For a DESCENDING pass the image was scanned
        from left to right (as viewed north up) which is why
        (in_samp <= nadir) is correct.
	------------------------------------------------------*/
	if (((model->cpass == ASCENDING)  && (in_samp > nadir)) ||
	    ((model->cpass == DESCENDING) && (in_samp <= nadir)))
	    angles[SATZEN] *= -1;


	*(angptrs[SATZEN]++) = (float)angles[SATZEN];
	*(angptrs[SOLZEN]++) = (float)angles[SOLZEN];
	*(angptrs[RELAZ]++)  = (float)angles[RELAZ];
	*(geoptrs[0]++) = (double)(in_line - (float)mline);     /* Line   */
	*(geoptrs[1]++) = (double)(in_samp - (float)msample);   /* Sample */


	/* Advance the pointer to elevgrid->values[0] and add one to curent
	-------------------------------------------------------------------*/
	elev++;
	curent++;
	}
    }


/* Stop the processing message and close the elevation image
------------------------------------------------------------*/
c_stoptm();
if (elevflag)
    c_closelui(elevlui);


/* Update the angles grid to reflect the values added to it
-----------------------------------------------------------*/
strcpy(anggrid->gridname,  frame->anggrid);
strcpy(anggrid->labels[SATZEN], "satellite zenith");
strcpy(anggrid->labels[SOLZEN], "solar zenith");
strcpy(anggrid->labels[RELAZ],  "relative azimuth");
anggrid->dtype = EREAL;


/* Update the elevation grid to reflect the values added to it
--------------------------------------------------------------*/
strcpy(elevgrid->gridname,  "tmp_elev_grid.elev");
strcpy(elevgrid->labels[0], "elevation");
elevgrid->dtype = EDOUBLE;


/* Update the geometric mapping grid to reflect the values added to it
----------------------------------------------------------------------*/
strcpy(geogrid->gridname,  frame->geogrid);
strcpy(geogrid->labels[0], "line number");
strcpy(geogrid->labels[1], "sample number");
geogrid->dtype = EDOUBLE;

return(E_SUCC);
}
