/*******************************************************************************
FUNCTION:	getpar

PURPOSE:	Get user entered parameters from the TAE parameter block

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    ??/??  ???              ???
  1.1    03/93  J.Fenno          Updated to add the option of creating an angles
				 grid and/or an atmospheric corrections grid.

*******************************************************************************/
#include "rectify.h"
#include "util.h"
#include "archio.h"


int getpar
(
    struct PARBLK *vblk,  /* IN  - TAE variable block                         */
    struct FRAME  *frame, /* OUT - Output framing parameters                  */
    struct PROJ   *outp,  /* OUT - Output projection parameters               */
    struct MODEL  *model, /* OUT - AVHRR model data structure                 */
    char  *pname,         /* OUT - Output print file name                     */
    long  *master_line,   /* OUT - Line relative to the original pass         */
    long  *master_sample, /* OUT - Sample relative to the original pass       */
    long  *msgtim         /* OUT - Time between processing messages           */
)
{
struct VARIABLE *var;     /* pointer to a TAE variable in vblk                */
struct DDR ddr;           /* Data Descriptor Record                           */

double ul_corner[2];      /* Upper left corner coordinate                     */
double lr_corner[2];      /* Lower right corner coordinate                    */

long   access;            /* File access indicatori                           */
long   file;              /* Print option flags                               */
long   ln_pr;             /* Print option flags                               */
long   lscoors[2];        /* Line,sample coordinates of ul coord              */
long   mode;              /* Framing mode                                     */
long   nl;                /* The number of lines in output                    */
long   nonfatal = 1;      /* Nonfatal error flag                              */
long   ns;                /* The number of samples in output                  */
long   out_def;           /* Default flags for projection parameters          */
long   status;            /* Status variable                                  */
long   term;              /* Print option flags                               */
long   unit_code;         /* Unit code of corner coordinates entered          */
long   zero = 0;          /* Constant of zero                                 */

char   filename[CMLEN];   /* Temporary filename                               */
char   host_img[CMLEN];   /* Host input image name                            */
char   proj_name[CMLEN];  /* Projection parameter file name                   */
char   outkey[CMLEN];     /* Output projection key                            */
char   units[12];         /* Units of corner coordinates entered              */


/* Get the input image
----------------------*/
access = IREAD;
if (c_getnam(vblk, "IN", &zero, host_img, "img", &access) != E_SUCC)
    return(E_FAIL);


/* Get the DDR information for the input image
----------------------------------------------*/
if (c_getddr(host_img, &ddr) != E_SUCC)
    return(E_FAIL);


/* Get the line and sample relative to the original image
---------------------------------------------------------*/
*master_line   = ddr.master_line;
*master_sample = ddr.master_sample;


/* Retreive the AVHRR model information from the associated ADDR file
---------------------------------------------------------------------*/
if (c_getaddr(host_img, model) != E_SUCC)
    return(E_FAIL);


/* Get the name of the output geometric mapping grid file
---------------------------------------------------------*/
access = IWRITE;
if (c_getnam(vblk, "GEOGRID", &zero, frame->geogrid, "grid", &access) != E_SUCC)
    return(E_FAIL);


/* Get the name of the angles grid file
---------------------------------------*/
var = p_find(vblk, "ANGGRID");
if (var->v_count == 0)
    strcpy(frame->anggrid, "");
else if (c_getnam(vblk,"ANGGRID",&zero,frame->anggrid,"ang",&access) != E_SUCC)
    return(E_FAIL);


/* Retrieve the name of the elevation image from the parameter block.  Ensure
   the projection units are degrees and that the data type is EWORD.  Store the
   necessary fields from the DDR in the frame structure.
------------------------------------------------------------------------------*/
access = IREAD;
var = p_find(vblk, "ELEV");
if (var->v_count == 0)
    strcpy(frame->elev.hostname, "");
else
    {
    if (c_getnam(vblk,"ELEV",&zero,frame->elev.hostname,"img",&access) !=E_SUCC)
	return(E_FAIL);
    if (c_getddr(frame->elev.hostname, &ddr) != E_SUCC)
	return(E_FAIL);
    if (strcmp(ddr.proj_units, "degrees"))
	{
	c_errmsg("Elevation projection units must be degrees",
	    "rectify-units", &nonfatal);
	return(E_FAIL);
	}
    if (ddr.dtype != EWORD)
	c_errmsg("Elevation image will be read as I*2",
	    "rectify-dtype", &nonfatal);
    frame->elev.ullat  = ddr.upleft[0];
    frame->elev.ullon  = ddr.upleft[1];
    frame->elev.lrlat  = ddr.loright[0];
    frame->elev.lrlon  = ddr.loright[1];
    frame->elev.pdistx = ddr.pdist_x;
    frame->elev.pdisty = ddr.pdist_y;
    frame->elev.nl     = ddr.nl;
    frame->elev.ns     = ddr.ns;
    }


/* Retrieve the name of the ozone image from the parameter block.  Ensure the
   projection units are degrees and that the data type is EWORD.  Store the
   necessary fields from the DDR in the frame structure.
-----------------------------------------------------------------------------*/
access = IREAD;
var = p_find(vblk, "OZONE");
if (var->v_count == 0)
    strcpy(frame->ozone.hostname, "");
else
    {
    status = c_getnam(vblk,"OZONE",&zero,frame->ozone.hostname,"img",&access);
    if (status != E_SUCC)
	return(E_FAIL);
    if (c_getddr(frame->ozone.hostname, &ddr) != E_SUCC)
	return(E_FAIL);
    if (strcmp(ddr.proj_units, "degrees"))
	{
	c_errmsg("Ozone projection units must be degrees",
	    "rectify-units", &nonfatal);
	return(E_FAIL);
	}
    if (ddr.dtype != EWORD)
	c_errmsg("Ozone image will be read as I*2","rectify-dtype",&nonfatal);
    frame->ozone.ullat  = ddr.upleft[0];
    frame->ozone.ullon  = ddr.upleft[1];
    frame->ozone.lrlat  = ddr.loright[0];
    frame->ozone.lrlon  = ddr.loright[1];
    frame->ozone.pdistx = ddr.pdist_x;
    frame->ozone.pdisty = ddr.pdist_y;
    frame->ozone.nl     = ddr.nl;
    frame->ozone.ns     = ddr.ns;
    }


/* Retrieve the name of the water vapor image from the parameter block.  Ensure
   the projection units are degrees and that the data type is EWORD.  Store the
   necessary fields from the DDR in the frame structure.
------------------------------------------------------------------------------*/
access = IREAD;
var = p_find(vblk, "WATER");
if (var->v_count == 0)
    strcpy(frame->water.hostname, "");
else
    {
    status = c_getnam(vblk,"WATER",&zero,frame->water.hostname,"img",&access);
    if (status != E_SUCC)
	return(E_FAIL);
    if (c_getddr(frame->water.hostname, &ddr) != E_SUCC)
	return(E_FAIL);
    if (strcmp(ddr.proj_units, "degrees"))
	{
	c_errmsg("Water vapor projection units must be degrees",
	    "rectify-units", &nonfatal);
	return(E_FAIL);
	}
    if (ddr.dtype != EWORD)
	c_errmsg("Water vapor image will be read as I*2",
	    "rectify-dtype", &nonfatal);
    frame->water.ullat  = ddr.upleft[0];
    frame->water.ullon  = ddr.upleft[1];
    frame->water.lrlat  = ddr.loright[0];
    frame->water.lrlon  = ddr.loright[1];
    frame->water.pdistx = ddr.pdist_x;
    frame->water.pdisty = ddr.pdist_y;
    frame->water.nl     = ddr.nl;
    frame->water.ns     = ddr.ns;
    }


/* Get the name of the file containing projection information
-------------------------------------------------------------*/
access = IREAD;
if (c_getnam(vblk, "INPROJ", &zero, proj_name, "proj", &access) != E_SUCC)
    return(E_FAIL);


/* Get the output projection information
----------------------------------------*/
var = p_find(vblk, "OPROJKEY");
out_def = TRUE;
if (var->v_count > 0)
    {
    out_def = FALSE;
    strcpy(outkey, SVAL(*var, 0));
    }
if (get_proj_prm(proj_name, outkey, out_def, outp) != E_SUCC)
    return(E_FAIL);


/* Get the pixel size of the output
-----------------------------------*/
var = p_find(vblk, "PIXSIZ");
frame->o_dist[0] = frame->o_dist[1] = RVAL(*var, 0);


/* Starting line/sample output coordinates
------------------------------------------*/
var = p_find(vblk, "LSCOORS");
lscoors[0] = IVAL(*var,0);
lscoors[1] = IVAL(*var,1);


/* Retreive the number of lines and samples in the output area
--------------------------------------------------------------*/
var = p_find(vblk, "NL");
nl = IVAL(*var,0);
var = p_find(vblk, "NS");
ns = IVAL(*var,0);


/* Get the units that the coordinate are in
-------------------------------------------*/
var = p_find(vblk, "COORUNIT");
strcpy(units, SVAL(*var, 0));
if(strcmp(units, "PRO") == 0) unit_code = 0;
if(strcmp(units, "RAD") == 0) unit_code = 0;
if(strcmp(units, "SEC") == 0) unit_code = 3;
if(strcmp(units, "DEG") == 0) unit_code = 4;
if(strcmp(units, "DMS") == 0) unit_code = 5;


/* Get the output projection coordinates
----------------------------------------*/
var = p_find(vblk, "COORS");
sscanf(SVAL(*var, 0), "%lf", &(ul_corner[0]));
sscanf(SVAL(*var, 1), "%lf", &(ul_corner[1]));
if (var->v_count == 2)
    mode = OUTPUT_LS;
else if (strcmp(units, "PRO") == 0)
    mode = OUTPUT_XY;
else
    mode = INPUT;
if ((mode == INPUT) || (mode == OUTPUT_XY))
    {
    sscanf(SVAL(*var, 2), "%lf", &(lr_corner[0]));
    sscanf(SVAL(*var, 3), "%lf", &(lr_corner[1]));
    }


/* Get the number of lines and samples desired in each grid cell
----------------------------------------------------------------*/
var = p_find(vblk, "NLGRID");
frame->ngline = IVAL(*var, 0);
var = p_find(vblk, "NSGRID");
frame->ngsamp = IVAL(*var, 0);


/* Get the message timer
------------------------*/
var = p_find(vblk, "$msgtim");
*msgtim = IVAL(*var, 0);


/* Get the print option.  Make a file name (if
   neccessary) and open the temporary file.
----------------------------------------------*/
strcpy(host_img, "rectify");
c_setprt(vblk, &term, &file, &ln_pr, host_img, filename);
access = IUPDATE;
if (file == 1)
    c_mknam(filename, pname, "prt", &access);
else 
    strcpy(pname, "");


/* Open the temporary print file and print the names of the grid files
----------------------------------------------------------------------*/
frame->fptr = fopen(host_img, "w");
fprintf(frame->fptr, "Geometric Mapping Grid name:  %s\n", frame->geogrid);
if (strlen(frame->anggrid) > 0)
    fprintf(frame->fptr, "Angles Grid name:             %s\n",
            frame->anggrid);


/* Compute transformation mapping from output image
   coordinates to output space projection coordinates
-----------------------------------------------------*/
status = out_frame(frame, outp, mode, ul_corner, lr_corner, lscoors, nl, ns,
    unit_code, host_img);
if (status != E_SUCC)
    return(E_FAIL); 


/* Zero out the grid reduction tolerance value.  This is no longer used.
------------------------------------------------------------------------*/
frame->tol = 0;
return(E_SUCC);
}


#include "ltable.h"

/*******************************************************************************
NAME:    get_proj_prm

PURPOSE: Retrieves the projection parameters from the projection parameter file
*******************************************************************************/
int get_proj_prm
(
    char *proj_name,            /* Projection parameter file name             */
    char *outkey,               /* Output projection parameter key            */
    long out_def,               /* Boolean flags--default given by user?      */
    struct PROJ *outp           /* Output projection parameters               */
)
{
struct TAB_DEF tab;	        /* Tabular structure of label table           */
struct VECTOR *keyptr;	        /* Pointer to key field                       */
struct VECTOR *codeptr;	        /* Pointer to projection code field           */
struct VECTOR *zoneptr;	        /* Pointer to zone code field                 */
struct VECTOR *unitptr;	        /* Pointer to unit of measure field           */
struct VECTOR *sphptr;	        /* Pointer to datum (spheroid) code field     */
struct VECTOR *parmptr;	        /* Pointer to projection coefficients field   */
long status;		        /* Function return status                     */
long i;			        /* Loop counter                               */


/* Open the labeled table file & retrieve the label
---------------------------------------------------*/
open_tab(&tab, proj_name, IREAD);
get_vector(&tab);


/* Determine pointers to fields in the file
-------------------------------------------*/
keyptr = get_field_ptr(&tab, "PROJKEY");
if (keyptr == NULL) return(E_FAIL);
codeptr = get_field_ptr(&tab, "PROJTYPE");
if (codeptr == NULL) return(E_FAIL);
zoneptr = get_field_ptr(&tab, "PROJZONE");
if (zoneptr == NULL) return(E_FAIL);
unitptr = get_field_ptr(&tab, "PROJUNITS");
if (unitptr == NULL) return(E_FAIL);
sphptr = get_field_ptr(&tab, "PROJSPH");
if (sphptr == NULL) return(E_FAIL);
parmptr = get_field_ptr(&tab, "PROJPARMS");
if (parmptr == NULL) return(E_FAIL);


/* Loop until key is found or EOF
---------------------------------*/
for (;;)
    {
    status = get_record(&tab);
    if (status == E_FAIL) return(E_FAIL);
    if (status == E_EOF) 
    {
    close_tab(&tab);
    c_lasmsg(TRUE,"rectify-oprojkey","%s key not found in %s",outkey,proj_name);
    }
    if((strcmp(keyptr->ptr.c, outkey) == 0) || out_def) break;
    }


outp->proj_code = *codeptr->ptr.i4;
outp->zone_code = *zoneptr->ptr.i4;
outp->units = *unitptr->ptr.i4;
outp->datum_code = *sphptr->ptr.i4;
for (i = 0; i < 15; i++)
    outp->proj_coef[i] = parmptr->ptr.r8[i];


/* Close the file & return
--------------------------*/
close_tab(&tab);
return(E_SUCC);
}
