/*******************************************************************************
FUNCTION:	getwindows

PURPOSE:	Compute a window for opening the elevation, ozone, and water
		vapor images.  It is assumed that each of these images is in
		the geographic projection with the upper left corner at 90N,
		180W and the projection units are degrees.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    03/93  J.Fenno          Development

PROJECT:  ADAPS
*******************************************************************************/
#include "adaps.h"
#include "rectify.h"


int getwindows
(
    struct FRAME  *frame,  /* IN/OUT - Output framing parameters              */
    struct MODEL  *model   /* IN     - AVHRR model structure                  */
)
{
char   imgname[CMLEN];     /* Name of the image we are currently working on   */

double maxlat;             /* Maximum latitude for each image                 */
double maxlon;             /* Maximum longitude for each image                */
double minlat;             /* Minimum latitude for each image                 */
double minlon;             /* Minimum longitude for each image                */

long   i;                  /* Loop control                                    */
long   j;                  /* Loop control                                    */
long   line;               /* Line number for the given latitude              */
long   sample;             /* Sample number in the elevation image            */

struct LUIIMAGE *image;    /* Ptr to a LUIIMAGE structure                     */


/* For each of the images
-------------------------*/
for (i = 0; i < 3; i++)
    {
    /* Set up a pointer to the LUI image structure in frame
    -------------------------------------------------------*/
    switch (i)
	{
	case 0:
	    image = &frame->elev;
	    strcpy(imgname, "Elevation");
	    break;
	case 1:
	    image = &frame->ozone;
	    strcpy(imgname, "Ozone");
	    break;
	case 2:
	    image = &frame->water;
	    strcpy(imgname, "Water vaopor");
	    break;
	}


    /* Initialize the window
    ------------------------*/
    image->window[SL] = 0;
    image->window[SS] = 0;
    image->window[NL] = 0;
    image->window[NS] = 0;

    /* If the user did not specify the current image then continue
    --------------------------------------------------------------*/
    if (strlen(image->hostname) == 0)
	continue;


    /* Determine the minimum and maximum latitude and longitude for the 8
       points around the output space.  We must ensure that every pixel we
       access is within the window that we are calculating.  Therefore, we
       are going to add BUFFER_DEG degrees to both latitudes and longitudes.
    ------------------------------------------------------------------------*/
    maxlat = frame->latlong[0][LAT];
    minlat = frame->latlong[0][LAT];
    minlon = frame->latlong[0][LON];
    maxlon = frame->latlong[0][LON];
    for (j = 1; j < 8; j++)
	{
	maxlat = maxlat > frame->latlong[j][LAT] ?maxlat:frame->latlong[j][LAT];
	minlat = minlat < frame->latlong[j][LAT] ?minlat:frame->latlong[j][LAT];
	minlon = minlon < frame->latlong[j][LON] ?minlon:frame->latlong[j][LON];
	maxlon = maxlon > frame->latlong[j][LON] ?maxlon:frame->latlong[j][LON];
	}
    maxlat += BUFFER_DEG;
    minlat -= BUFFER_DEG;
    minlon -= BUFFER_DEG;
    maxlon += BUFFER_DEG;
    if (maxlat > image->ullat) maxlat = image->ullat;
    if (minlat < image->lrlat) minlat = image->lrlat;
    if (minlon < image->ullon) minlon = image->ullon;
    if (maxlon > image->lrlon) maxlon = image->lrlon;

    /* Determine if the output space crosses longitude 180.
       If it does, we must read entire lines from the image.
    --------------------------------------------------------*/
    if ((frame->latlong[0][LON] > frame->latlong[2][LON]) ||
        (frame->latlong[3][LON] > frame->latlong[4][LON]) ||
        (frame->latlong[5][LON] > frame->latlong[7][LON]))
	{
	image->window[SS] = 1;
	image->window[NS] = image->ns;
	}


    /* Determine if the input image crosses either pole.  If it does, we must
       read every line from the pole to the farthest latitude and every sample
       for these lines.
    -------------------------------------------------------------------------*/
    if ((model->spass != model->cpass) || (model->cpass != model->epass))
	{
	image->window[SS] = 1;
	image->window[NS] = image->ns;
	if (model->spass == ASCENDING) /* If ascending and we cross a pole it */
	    maxlat = image->ullat;     /* must be the North pole.  Otherwise  */
	else                           /* it is a decending pass and we have  */
	    minlat = image->lrlat;     /* crossed the South pole.             */
	}


    /* Compute the starting line and sample for the window
    ------------------------------------------------------*/
    c_geo2ls(&image->ullat, &image->ullon, &image->pdistx, &image->pdisty,
	&maxlat, &minlon, &line, &sample);
    image->window[SL] = line + 1;
    if (image->window[SS] == 0)
	image->window[SS] = sample + 1;

    /* Compute the number of lines and samples for the window
    ---------------------------------------------------------*/
    c_geo2ls(&image->ullat, &image->ullon, &image->pdistx, &image->pdisty,
	&minlat, &maxlon, &line, &sample);
    line++;    /* line is an offset from the upper left corner   */
    sample++;  /* sample is an offset from the upper left corner */
    image->window[NL] = line - image->window[SL] + 1;
    if (image->window[NS] == 0)
	image->window[NS] = sample - image->window[SS] + 1;


    /* Reset the upper left latitude and longitude to correspond
       the the upper left pixel in the window.
    ------------------------------------------------------------*/
    image->ullat = maxlat;
    image->ullon = minlon;
    }

return(E_SUCC);
}
