/*******************************************************************************
FUNCTION:	out_frame

PURPOSE:	Calculates a set of coefficients mapping from output space
		image coordinates to output space projection coordinates

*******************************************************************************/
#include "rectify.h"


int out_frame
(
    struct FRAME *frame,/* Framing parameters                                 */
    struct PROJ  *outp, /* Output projection parameters                       */
    long mode,          /* Type of framing                                    */
    double *ul,         /* Upper left corner coordinates                      */
    double *lr,         /* Lower right corner coordinates                     */
    long *ls,           /* Line/sample coordinates of ul                      */
    long nl,            /* Number of lines in output space                    */
    long ns,            /* Number of samples in output space                  */
    long units,         /* Units code of coordinates entered                  */
    char *prtfile       /* Output print file name                             */
)
{
double zero_parms[15];  /* Zeroed projectection coeffients                    */
double pcoords[8][2];   /* Projection coordinates of eight points             */
double pxhalf;          /* Half of the distance between pxmin and pxmax       */
double pxmax;           /* Maximum X projection coordinate                    */
double pxmin;           /* Minimum X projection coordinate                    */
double pyhalf;          /* Half of the distance between pymin and pymax       */
double pymax;           /* Maximum Y projection coordinate                    */
double pymin;           /* Minimum Y projection coordinate                    */
double temp_mod;        /* Results from DMOD operation...                     */

long degrees   =  4;    /* Unit code                                          */
long geo       =  0;    /* Geographic projection system flag                  */
long i;                 /* Loop control                                       */
long nonfatal  =  1;    /* Nonfatal error flag                                */
long null_zone = 62;    /* Null zone code for projection xforms               */
long prtprm[2];         /* Print flags for c_transinit                        */
long status;            /* Function return status                             */

for (i = 0; i < 15; i++)
    zero_parms[i] = 0.0;

/* Determine the mode
---------------------*/
if (mode == INPUT)
    {
    /* Determine output projection space minimums and maximums
    ----------------------------------------------------------*/
    status = c_mapedg(&(ul[1]), &(lr[1]), &(ul[0]), &(lr[0]), &units,
	&(outp->proj_code), &(outp->zone_code), outp->proj_coef, &(outp->units),
	&pxmin, &pxmax, &pymin, &pymax, &(outp->datum_code));
    if (status != E_SUCC) 
	return(E_FAIL);


    /* Adjust to even pixel increment
    ---------------------------------*/
    temp_mod = DMOD(pymax, frame->o_dist[0]);
    if (temp_mod != 0.0) pymax += (frame->o_dist[0] - temp_mod);
    pymin -= DMOD(pymin, frame->o_dist[0]);
    pxmin -= DMOD(pxmin, frame->o_dist[1]);
    temp_mod = DMOD(pxmax, frame->o_dist[1]);
    if (temp_mod != 0.0) pxmax += (frame->o_dist[1] - temp_mod);
    }
else if (mode == OUTPUT_XY)
    {
    /* If output frame was given in projection coordinates, for two points
    ----------------------------------------------------------------------*/
    pymax = ul[0] + ((ls[0] - 1) * frame->o_dist[0]);
    pxmin = ul[1] - ((ls[1] - 1) * frame->o_dist[1]);
    pymin = lr[0];
    pxmax = lr[1];
    }
else if (mode == OUTPUT_LS)
    {
    /* If output frame was given in projection coordinates, 1 pt & nl, ns
    ---------------------------------------------------------------------*/
    pymax = ul[0] + ((ls[0] - 1) * frame->o_dist[0]);
    pxmin = ul[1] - ((ls[1] - 1) * frame->o_dist[1]);
    pymin = pymax - ((nl - 1) * frame->o_dist[0]);
    pxmax = pxmin + ((ns - 1) * frame->o_dist[1]);
    }


/* Report to the user the projection minimums and maximums of the frame
-----------------------------------------------------------------------*/
fprintf(frame->fptr, "   Min/max Y projection coordinates:  %lf %lf\n",
    pymin, pymax);
fprintf(frame->fptr, "   Min/max X projection coordinates:  %lf %lf\n",
    pxmin, pxmax);


/* Report to the user the pixel size in line & sample directions
----------------------------------------------------------------*/
fprintf(frame->fptr,"   Pixel size in line direction:    %f\n", 
    frame->o_dist[0]);
fprintf(frame->fptr,"   Pixel size in sample direction:  %f\n",
    frame->o_dist[1]);


/* Report to the user the number of lines and samples in the output space
-------------------------------------------------------------------------*/
frame->window[SL] = frame->window[SS] = 1;
frame->window[NL] = (long)((pymax - pymin) / frame->o_dist[0] + 1.5);
frame->window[NS] = (long)((pxmax - pxmin) / frame->o_dist[1] + 1.5);
fprintf(frame->fptr,"   Number of lines in output space:    %d\n",
    frame->window[NL]);
fprintf(frame->fptr,"   Number of samples in output space:  %d\n\n",
    frame->window[NS]);


/* Pack the corner projection coordinates in the frame structure
----------------------------------------------------------------*/
frame->corners[1] = frame->corners[3] = pxmin;
frame->corners[0] = frame->corners[4] = pymax;
frame->corners[6] = frame->corners[2] = pymin;
frame->corners[5] = frame->corners[7] = pxmax;


/* To save time and reduce our memory requirements we want to compute the
   minimum and maximum latitude and longitude of the output space.  This allows
   us to compute an appropriate window for the elevation, ozone, and water
   vapor images.  Because we are working in projection space we will need to
   compute the latitude and longitude for eight points around the image.
------------------------------------------------------------------------------*/
pyhalf = (pymax - pymin) / 2.0;
pxhalf = (pxmax - pxmin) / 2.0;
pcoords[0][LAT] = pymax;        /* Upper Left */
pcoords[0][LON] = pxmin;
pcoords[1][LAT] = pymax;        /* Upper Center */
pcoords[1][LON] = pxhalf;
pcoords[2][LAT] = pymax;        /* Upper Right */
pcoords[2][LON] = pxmax;
pcoords[3][LAT] = pyhalf;       /* Center Left */
pcoords[3][LON] = pxmin;
pcoords[4][LAT] = pyhalf;       /* Center Right */
pcoords[4][LON] = pxmax;
pcoords[5][LAT] = pymin;        /* Lower Left */
pcoords[5][LON] = pxmin;
pcoords[6][LAT] = pymin;        /* Lower Center */
pcoords[6][LON] = pxhalf;
pcoords[7][LAT] = pymin;        /* Lower Right */
pcoords[7][LON] = pxmax;

status = c_transinit(&outp->proj_code, &outp->units, &outp->zone_code, 
	 &outp->datum_code, outp->proj_coef, &geo, &degrees, &null_zone, 
	 &outp->datum_code, zero_parms, prtprm, prtfile);
if (status != E_SUCC)
   {
   c_errmsg("Error initializing transformation","rectify-trans",&nonfatal);
   return(E_FAIL);
   }

for (i = 0; i < 8; i++)
    {
    status = c_trans(&outp->proj_code, &outp->units, &geo, &degrees, 
	&pcoords[i][LON], &pcoords[i][LAT], &(frame->latlong[i][LON]),
	&(frame->latlong[i][LAT]));
    if (status != E_SUCC)
	{
	c_errmsg("Error calculating lat/long", "rectify-ll", &nonfatal);
	return(E_FAIL);
	}
    }

return(E_SUCC);
}
