/*******************************************************************************
FUNCTION:	rectify

PURPOSE:	Creates a geometric mapping grid file which specifies an image
		projection change from one of the projections supported by the
		projection transformation package to another.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    01/90  D.Steinwand      Development
  2.0    03/91  D.Hollaren       Modified to use new model
  3.0    03/93  J.Fenno          Restructured to use a new gridding structure
				 and optionally create an angles and/or an
				 atmospheric grid.
  4.0    06/01  D.Lloyd          Removed the atmospheric grid code.  CR221
                                 Prototyped and cleaned up compiler warnings.
  4.1    10/01  D.Lloyd          Removed calls to fatal_err() after getpar()
                                 and getwindows().  The fatal_err() call tries
				 to free unallocated memory.  Replaced with
				 c_clean() calls.  See CR 5213.

PROJECT:  ADAPS

ALGORITHM REFERENCES:
    This version of RECTIFY is a total re-write from the LAS 4.0 version.
    LAS 5.0 RECTIFY allows any of the projection transformation package
    projections,  flexable framing parameter options, and an option to
    define the input image if the DDR does not contain valid info.

        1.  PROCHG, a LAS (pre 4.0) local EDC function which changed 
            projections of an image to any of 21 projections.  Written
            by J. Thormodsgard & J. Peterson, EDC, 1984.

        2.  RECTIFY, the LAS 4.0 projection change routine by A. DasGupta 
            (7/83, CSC) and S. Kondal (4/85, SAR)
*******************************************************************************/
#include "rectify.h"
#include "util.h"


/* Local function for issuing a fatal error message
---------------------------------------------------*/
static void fatal_err
(
    char   hnames[][CMLEN],/* IN - Host names of the file created so far      */
    long   nnames,         /* IN - The number of file names in hnames         */
    struct GRID   *g1,     /* IN - A grid to be freed                         */
    struct GRID   *g2,     /* IN - A grid to be freed                         */
    struct GRID   *g3,     /* IN - A grid to be freed                         */
    struct PARBLK *vblk    /* IN - TAE parameter block                        */
)
{
c_freegrid(g1);
c_freegrid(g2);
c_freegrid(g3);
c_clean(vblk, "rectify", hnames, &nnames);/* Issue fatal message delete files */
}



int main( void )
{
struct FRAME   frame;    /* Framing & gridding parameters                     */
struct GEOGRID oldgeo;   /* An old geometric mapping grid                     */
struct GRID    anggrid;  /* Angles grid                                       */
struct GRID    elevgrid; /* Elevation grid                                    */
struct GRID    geogrid;  /* Geometric mapping grid                            */
struct MODEL   model;    /* AVHRR model data structure                        */
struct PARBLK  vblk;     /* TAE parameter block                               */
struct PROJ    outp;     /* Output space projection parameters                */

long num      =  0;      /* Number of output files                            */
long nonfatal =  1;      /* Nonfatal error flag                               */
long mline;              /* Line relative to the original pass                */
long msample;            /* Sample relative to the original pass              */
long msgtime;            /* Time between processing messages                  */
long std_out;            /* Dummy variable to standard output                 */
long status;             /* Return status                                     */

char pname[CMLEN];       /* Printer file name                                 */
char hnames[4][CMLEN];   /* Host name of output files for calling c_clean     */


/* Initalize las
----------------*/
c_inlas(&vblk,&std_out);


/* Get the parameters
---------------------*/
status =getpar(&vblk, &frame, &outp, &model, pname, &mline, &msample, &msgtime);
if (status != E_SUCC)
    c_clean(&vblk, "rectify", hnames, &num);


/* Determine windows for the elevation, ozone, and water vapor images
---------------------------------------------------------------------*/
if (getwindows(&frame, &model) != E_SUCC)
    c_clean(&vblk, "rectify", hnames, &num);


/* Zero all of the fields in each grid structure
------------------------------------------------*/
c_zerogrid(&geogrid);
c_zerogrid(&anggrid);
c_zerogrid(&elevgrid);


/* Initailize the geometric mapping grid with valid values.
-----------------------------------------------------------*/
if (initgrid(&geogrid, &frame, &outp, model.satnum) != E_SUCC)
    {
    c_errmsg("Error allocating memory", "rectify-alloc", &nonfatal);
    fatal_err(hnames, num, &anggrid, &elevgrid, &geogrid, &vblk);
    }


/* Copy the geometric mapping grid to the other three grids
-----------------------------------------------------------*/
if ((c_copygrid(&geogrid, &anggrid)  != E_SUCC) ||
    (c_copygrid(&geogrid, &elevgrid) != E_SUCC))
    {
    c_errmsg("Error creating grid", "rectify-grid", &nonfatal);
    fatal_err(hnames, num, &anggrid, &elevgrid, &geogrid, &vblk);
    }


/* Create the angles, elevation, and geometric grids
----------------------------------------------------*/
status = ang_elev_geo(&anggrid, &elevgrid, &geogrid,
    &frame, &model, &outp, mline, msample, msgtime);
if (status == NO_RECTIFY)
    c_errmsg("Unable to rectify to selected output frame",
	"rectify-nomap", &nonfatal);
else if (status == NO_SPACE)
    c_errmsg("Error allocating memory", "rectify-alloc", &nonfatal);
if (status != E_SUCC)
    fatal_err(hnames, num, &anggrid, &elevgrid, &geogrid, &vblk);


/* Currently the GEOM package and c_grdred use an older version of the
   geometric mapping grid.  Convert the new style grid to the old grid, reduce
   it if necessary, save it to a file, and free all of the memory within the
   new the GRID structure.
------------------------------------------------------------------------------*/
strcpy(hnames[num++], geogrid.gridname);
convertgrid(&geogrid, &oldgeo);
if (c_putgrd(geogrid.gridname, &oldgeo) != E_SUCC)
    fatal_err(hnames, num, &anggrid, &elevgrid, &geogrid, &vblk);
c_freegrid(&geogrid);


/* Save the angles grid if it was requested by the user
-------------------------------------------------------*/
if (strlen(anggrid.gridname) > 0)
    {
    strcpy(hnames[num++], anggrid.gridname);
    if (c_wrtgrid(&anggrid) != E_SUCC)
	fatal_err(hnames, num, &anggrid, &elevgrid, &geogrid, &vblk);
    }


/* Free all of the grid structures
----------------------------------*/
c_freegrid(&anggrid);
c_freegrid(&elevgrid);


/* Generate a LAS name for the temporary print file
---------------------------------------------------*/
if (strlen(pname) > 0)
   strcpy(hnames[num++], pname);


/* Display a standard completion message
----------------------------------------*/
c_complt("RECTIFY", hnames, &num);
return 0;
}
