#ifndef RECTIFY_H
#define RECTIFY_H
#include "geompak.h"
#include "grid.h"
#include "model.h"
#include "adapsgeo.h"

/* Defines used for Look Up Images (LUI).  A LUI is an image that contains
   values that must be accessed quickly.  Therefore, an attempt is made to load
   the entire image into memory.  This group of definitions should some day be
   removed and made into a support along with the functions c_closelui,
   c_getlui, and c_loadlui.
==============================================================================*/
#define LUIDECREASE   100 /* # of lines to decrease a LUI buffer by           */
#define MAXLUIS        10 /* Max number of LUI's opended at the same time     */
struct LUI
   {
   struct FDESC *  fdesc; /* File descriptor for the LUI image                */
   long bufsize;          /* Number of lines that can be held in the buffer   */
   long dtype;            /* Data type that the user would like returned      */
   long first;            /* First line that is currently in the buffer       */
   long last;             /* Last line that is currently in the buffer        */
   long window[4];        /* Window of the LUI that has been opened           */
   unsigned char *buffer; /* Buffer for the LUI data                          */
   };


/* Other misc defines
==============================================================================*/
#define BUFFER_DEG      5 /* Number of degrees to enlarge LUI images with     */
#define LAT             0 /* Latitude index for each corner in frame.latlong  */
#define LON             1 /* Longitude index for each corner in frame.latlong */
#define OUTPUT_XY     105 /*                                                  */
#define OUTPUT_LS     106 /*                                                  */
#define INPUT         107 /*                                                  */
#define MINBOX        108 /*                                                  */
#define NO_RECTIFY    -99 /*                                                  */


/* Misc macros
==============================================================================*/
#define DMOD(A,B)  (A) - ((long)((A) / (B)) * (B))


/* Structure to hold information about the elevation, ozone,
   and water vapor LUIs.  This structure is specific to rectify.
==============================================================================*/
struct LUIIMAGE
   {
   char   hostname[CMLEN];/* Host file name of the image                      */
   double ullat;          /* Latitude of the upper left corner                */
   double ullon;          /* Longitude of the upper left corner               */
   double lrlat;          /* Latitude of the lower right corner               */
   double lrlon;          /* Longitude of the lower right corner              */
   double pdistx;         /* Pixel distance in the x (sample) direction       */
   double pdisty;         /* Pixel distance in the y (line) direction         */
   long   nl;             /* Number of lines in the image                     */
   long   ns;             /* Number of samples in the image                   */
   long   window[4];      /* Window for opening the image                     */
   };


/* Structure containing framing parameters & related gridding
   information.  These are all user-entered parameters.
==============================================================================*/
struct FRAME 
   {
   char   geogrid[CMLEN]; /* Name of the geometric mapping grid               */
   char   anggrid[CMLEN]; /* Name of the angles grid                          */
   double corners[8];     /* Resulting grid projection corners                */
   double latlong[8][2];  /* Lat/Lon of eight points around the output space  */
   double o_dist[2];      /* Projection distance in y & x; input space        */
   double tol;            /* Tolerance for linear approximation-grid reduce   */
   long   ngline;         /* Number of lines in a grid cell                   */
   long   ngsamp;         /* Number of samples in a grid cell                 */
   long   window[4];      /* Window parameters for output space               */
   struct LUIIMAGE elev;  /* Structure containing info about elevation image  */
   struct LUIIMAGE ozone; /* Structure containing info about ozone image      */
   struct LUIIMAGE water; /* Structure containing info about water vapor image*/
   FILE   *fptr;          /* File pointer to print file                       */
   };


/* Structure holding projection parameters
==============================================================================*/
struct PROJ
   {
   long proj_code;        /* Projection code number                           */
   long units;            /* Projection units code number                     */
   long zone_code;        /* Projection zone code (UTM or State Plane)        */
   long datum_code;       /* Projection datum code                            */
   double proj_coef[15];  /* 15 projection coefficients                       */
   };

/* Funtion prototypes
=====================*/

/* ang_elev_geo.c */
int ang_elev_geo (
	struct GRID *anggrid, 
	struct GRID *elevgrid, 
	struct GRID *geogrid, 
	struct FRAME *frame, 
	struct MODEL *model, 
	struct PROJ *outp, 
	long mline, 
	long msample, 
	long msgtime
);

/* c_closelui.c */
int c_closelui (
	long i
);

/* c_getlui.c */
int c_getlui (
	long i, 
	long line, 
	long sample, 
	unsigned char *value
);

/* c_loadlui.c */
int c_loadlui (
	char *hostname, 
	long window[4], 
	long dtype, 
	long *ptr, 
	long *nlines
);

/* convertgrid.c */
int convertgrid (
	struct GRID *new, 
	struct GEOGRID *old
);

/* getpar.c */
int getpar (
	struct PARBLK *vblk, 
	struct FRAME *frame, 
	struct PROJ *outp, 
	struct MODEL *model, 
	char *pname, 
	long *master_line, 
	long *master_sample, 
	long *msgtim
);
int get_proj_prm(
	char *proj_name, 
	char *outkey, 
	long out_def, 
	struct PROJ *outp
);

/* getwindows.c */
int getwindows (
	struct FRAME *frame, 
	struct MODEL *model
);

/* initgrid.c */
int initgrid (
	struct GRID *grid, 
	struct FRAME *frame, 
	struct PROJ *outp, 
	long satnum
);

/* out_frame.c */
int out_frame(
	struct FRAME *frame, 
	struct PROJ *outp, 
	long mode, 
	double *ul, 
	double *lr, 
	long *ls, 
	long nl, 
	long ns, 
	long units, 
	char *prtfile
);

#endif
