/*******************************************************************************
FUNCTION:	refaid_get

PURPOSE:	Retrieve the name of an AVHRR archive image to be refaid'd.
		The images should be prioritized by receiving station.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    11/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "util.h"

#define MXFILES 20		/* Max # of files returned by c_getfiles()    */
#define MXPRIOR 10		/* Max # of priority station IDs              */


main()
{
char files[MXFILES][CMLEN];	/* Host names of images to be checked         */
char filespec[CMLEN];		/* File specification for AVHRR archive images*/
char lasname[CMLEN];		/* LAS name of the highest priority image     */
char priority_ids[MXPRIOR][4];	/* CEOS IDs of priority stations              */
char *priority_img[1];		/* Pointer to the priority image name         */
char str1[CMLEN];		/* Temporary string                           */
char str2[CMLEN];		/* Temporary string                           */

long current_priority;		/* Priority of the current image              */
long fatal      = -1;		/* Fatal error flag                           */
long first_call;		/* Flag for first call to c_getfiles()        */
long i;				/* Loop control                               */
long len;			/* Length of a string                         */
long priority;			/* Loop control                               */
long nfiles;			/* Number of files to be checked              */
long nonfatal   =  1;		/* Nonfatal error flag                        */
long npriorities;		/* Number of priority stations                */
long std_out;			/* Standard output file                       */

short done;			/* Indicates highest priority image found     */

struct DROPPED drop;		/* AVHRR dropped line structure               */
struct MODEL model;		/* AVHRR model structure                      */
struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE *parm;		/* Ptr to a TAE variable                      */



/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the recieving station priorities
--------------------------------------------*/
parm = p_find(&vblock, "PRIORITY");
npriorities = parm->v_count;
if (npriorities > MXPRIOR)
    {
    sprintf(str1,"Using first %d priorities only", MXPRIOR);
    c_errmsg(str1, "refaid_get-reset", &nonfatal);
    npriorities = MXPRIOR;
    }
for (i = 0; i < npriorities; i++)
    {
    strcpy(priority_ids[i], SVAL(*parm, i));
    len = (long)strlen(priority_ids[i]);
    if (len != 3)
	{
	c_errmsg("Station IDs must be 3 characters","refaid_get-ids",&nonfatal);
	c_errmsg("Fatal error encountered", "refaid_get-fatal", &fatal);
	}
    c_low2up(priority_ids[i], &len);
    }


/* Get ADAPSING
---------------*/
if (getenv("ADAPSING") == NULL)
    {
    c_errmsg("Error retrieving ADAPSING", "refaid_get-env", &nonfatal);
    c_errmsg("Fatal error encountered", "refaid_get-fatal", &fatal);
    }
#ifdef vms
sprintf(filespec, "%sa*.arch", getenv("ADAPSING"));
#else
sprintf(filespec, "%s/a*.arch", getenv("ADAPSING"));
#endif


/* Check all of the images matching filespec
--------------------------------------------*/
done = FALSE;
first_call = TRUE;
nfiles = MXFILES;
while (!done && (nfiles == MXFILES))
    {
    /* Get a list of files
    ----------------------*/
    if (c_getfiles(filespec,files,&nfiles,MXFILES,first_call,TRUE) != E_SUCC)
	c_errmsg("Fatal error encountered", "refaid_get-fatal", &fatal);
    

    /* If this is the first call to c_getfiles() and we
       found no files at all then exit successfully
    ---------------------------------------------------*/
    if (first_call && (nfiles == 0))
	{
	if ((q_string(&vblock,"OUTFILE",0,priority_img,P_UPDATE) != SUCCESS) ||
	    (q_out(&vblock) != SUCCESS))
	    {
	    c_errmsg("Error returning image", "refaid_get-parm", &nonfatal);
	    c_errmsg("Fatal error encountered", "refaid_get-fatal", &fatal);
	    }
	z_exit(1, "REFAID_GET completed successfully");
	}
    

    /* If this is the first call to c_getfiles() then the first image that we
       have found must be saved.  If we don't find any priority images then we
       always return the first one.
    --------------------------------------------------------------------------*/
    if (first_call)
	{
        strcpy(lasname, files[0]);
	current_priority = npriorities;
	}
    

    /* If no priority stations were specified then just return the first image
    --------------------------------------------------------------------------*/
    if (npriorities == 0)
	done = TRUE;


    /* Set the flag indicating we have already called c_getfiles()
    --------------------------------------------------------------*/
    first_call = FALSE;


    /* Check each of the files found
    --------------------------------*/
    for (i = 0; !done && (i < nfiles); i++)
	{
	/* Read the header for the current image
	----------------------------------------*/
	if (c_getahdr(files[i], &model, &drop) != E_SUCC)
	    {
            strcpy(str1, files[i]);
	    sprintf(str2, "Error reading header for %s", str1);
	    c_errmsg(str2, "refaid_get-hdr", &nonfatal);
	    continue;
	    }


	/* Check the image's receiving station against all higher priority IDs
	----------------------------------------------------------------------*/
	for (priority = 0; !done && (priority < current_priority); priority++)
	    {
	    if (strcmp(model.station, priority_ids[priority]) == 0)
		{
                strcpy(lasname, files[i]);
		current_priority = priority;
		if (current_priority == 0)
		    done = TRUE;
		}
	    }
	}
    }


/* To ensure that we do not leave any defunct processes around keep calling
   c_getfiles() until it does not return the maximum number of MXFILES files.
-----------------------------------------------------------------------------*/
while (nfiles == MXFILES)
    if (c_getfiles(filespec,files,&nfiles,MXFILES,first_call,TRUE) != E_SUCC)
	nfiles = 0;


/* Return the LAS name of the image
-----------------------------------*/
priority_img[0] = lasname;
if ((q_string(&vblock,"OUTFILE",1,priority_img,P_UPDATE) != SUCCESS) ||
    (q_out(&vblock) != SUCCESS))
    {
    c_errmsg("Error returning image", "refaid_get-parm", &nonfatal);
    c_errmsg("Fatal error encountered", "refaid_get-fatal", &fatal);
    }


z_exit(1, "REFAID_GET completed successfully");


/* Put a return here to make the compiler happy
-----------------------------------------------*/
return(E_SUCC);
}
