/****************************************************************************
NAME:		GETPAR

PURPOSE:	Get the input parameters

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "typlim.h"
#include "rescale.h"


FUNCTION getpar(vblock, hosin, hosout, minmax, taein, taeout, bands, idtype,
                imask, msgtim, nbands, nmasks, nmrimg, odtype, omask, rndopt,
		totbnd, window, iscale, ioffset, oscale, ooffset, omax, omin)

    struct PARBLK *vblock;	/* Pointer to TAE parameter block            */
    char
	    hosin[][CMLEN],	/* host file names of input images 	     */
	    hosout[],		/* host file name of output image	     */
	    minmax[],		/* min max flag buffer                       */
	    taein[][CMLEN],	/* TAE file names of input images  	     */
	    taeout[];		/* TAE file name of output image	     */
    long
	    bands[][MAXBND+1],	/* band specifications for each input image  */
	    *idtype,		/* input data type                           */
	    imask[],		/* Input mask values			     */
	    *msgtim,		/* elapsed time between display messages     */
	    nbands[],		/* number of bands per input image           */
	    *nmasks,		/* Number of mask values                     */
	    *nmrimg,		/* number of input images                    */
	    *odtype,		/* output data type                          */
	    omask[],		/* Output mask values			     */
	    *rndopt,		/* Rounding option (TRUE, FALSE)	     */
	    *totbnd,		/* total number of bands to be processed     */
	    window[][4];	/* window specifications for each input image*/
    double
	    iscale[],		/* Input scale factors			     */
	    ioffset[],		/* Input offset factors			     */
	    oscale[],		/* Output scale factors			     */
	    ooffset[],		/* Output offset factors		     */
	    omax[],		/* Maximum output pixel values               */
	    omin[];		/* Minimum output pixel values               */
{
struct VARIABLE	*p_find();	/* function to find a TAE variable           */
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

double  minimum,		/* Minimum value for the output data type    */
	maximum,		/* Maximum value for the output data type    */
	shift[MAXBND+1];	/* Data shift values for each band	     */

long	imglim = MAXIMG,	/* maximum number of images to be processed  */
	fatal = -1,		/* fatal error flag			     */
	nonfatal = 1,		/* nonfatal error flag			     */
	element = 0,		/* parameter element number		     */
	qflag[MAXIMG][2],	/* flag indicating if subwindows specified   */
	same,			/* flag indicating if images have same window*/
	i, 			/* Looping variable 			     */
	status;			/* function return status code               */


/*
  Retrieve and parse input image specification 
*/
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,nbands,
		totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","rescale-fatal",&fatal);


/*
  To get this function working as quickly as possible, do not allow image
  addition.  There is a problem with sequential imageio.  It will not work
  correctly with images that have an odd number of lines and/or samples.  Limit
  the number of images to one and use random imageio.
*/
if (*nmrimg > 1)
    {
    c_errmsg("Cannot specify multiple images", "rescale-mult", &nonfatal);
    c_errmsg("Fatal error encountered", "rescale-fatal", &fatal);
    }


/*
  Ensure all images have the same window size.  Currently this will never
  happen due to the fact that multiple images are not allowed.  When this is
  corrected, add this error message in the FATALS sections of the PDF.
*/
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes",
       "rescale-window", &nonfatal);
   c_errmsg("Fatal error encountered", "rescale-fatal", &fatal);
   }

/*
  Retrieve the data type of the input image
*/
status = c_gettyp(hosin,nmrimg,idtype);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);

/*
  Retrieve input data type
*/
val = p_find(vblock,"IDTYPE");	
if (val->v_count)
    {
    if (strcmp(SVAL(*val,0),"U*2") == 0)
	*idtype = EUWORD;
    else
	*idtype = EULONG;
    }

/*
  Retrieve output image name
*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","rescale-fatal",&fatal);

/*
  Retrieve output data type
*/
val = p_find(vblock,"ODTYPE");	
if (val->v_count == 0)
    *odtype = *idtype;
else if (strcmp(SVAL(*val,0),"BYTE") == 0)
    *odtype = EBYTE;
else if (strcmp(SVAL(*val,0),"I*2") == 0)
    *odtype = EWORD;
else if (strcmp(SVAL(*val,0),"I*4") == 0)
    *odtype = ELONG;
else if (strcmp(SVAL(*val,0),"R*4") == 0)
    *odtype = EREAL;
else if (strcmp(SVAL(*val,0),"U*2") == 0)
    *odtype = EUWORD;
else if (strcmp(SVAL(*val,0),"U*4") == 0)
    *odtype = EULONG;
else
    {
    c_errmsg("Invalid data type", "rescale-type", &nonfatal);
    c_errmsg("Fatal error encountered", "rescale-fatal", &fatal);
    }

/*
  Initialize the scale and offset values to the default
*/ 
for (i = 0; i < *totbnd; i++)
    {
    iscale[i]  = 1.0;
    ioffset[i] = 0.0;
    oscale[i]  = 1.0;
    ooffset[i] = 0.0;
    }

/*
  Get the input scale factors
*/
val = p_find(vblock,"ISCALE");
if (val->v_count == 1)
    {
    for (i = 0; i < *totbnd; i++)
	iscale[i] = RVAL(*val,0);
    }
else if (val->v_count == *totbnd)
    {
    for (i = 0; i < *totbnd; i++)
	iscale[i] = RVAL(*val,i);
    }
else
    {
    c_errmsg("Invalid number of values for ISCALE","rescale-num",&nonfatal);
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);
    }

/*
  Get the input offset values
*/
val = p_find(vblock,"IOFFSET");
if (val->v_count == 1)
    {
    for (i = 0; i < *totbnd; i++)
	ioffset[i] = RVAL(*val,0);
    }
else if (val->v_count == *totbnd)
    {
    for (i = 0; i < *totbnd; i++)
	ioffset[i] = RVAL(*val,i);
    }
else
    {
    c_errmsg("Invalid number of values for IOFFSET","rescale-num",&nonfatal);
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);
    }

/*
  Get the output scale factors
*/
val = p_find(vblock,"OSCALE");
if (val->v_count == 1)
    {
    for (i = 0; i < *totbnd; i++)
	oscale[i] = RVAL(*val,0);
    }
else if (val->v_count == *totbnd)
    {
    for (i = 0; i < *totbnd; i++)
	oscale[i] = RVAL(*val,i);
    }
else
    {
    c_errmsg("Invalid number of values for OSCALE","rescale-num",&nonfatal);
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);
    }

/*
  Get the output offset values
*/
val = p_find(vblock,"OOFFSET");
if (val->v_count == 1)
    {
    for (i = 0; i < *totbnd; i++)
	ooffset[i] = RVAL(*val,0);
    }
else if (val->v_count == *totbnd)
    {
    for (i = 0; i < *totbnd; i++)
	ooffset[i] = RVAL(*val,i);
    }
else
    {
    c_errmsg("Invalid number of values for OOFFSET","rescale-num",&nonfatal);
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);
    }


/*
  Initialize the input and output mask values
*/
for (i = 0; i < MAXMASKS; i++)
    {
    imask[i] = 0;
    omask[i] = 0;
    }


/*
  Retrieve the input mask values
*/
val = p_find(vblock,"IMASK");
*nmasks = val->v_count;
for (i = 0; i < *nmasks; i++)
    imask[i] = IVAL(*val,i);


/*
  Retrieve the output mask values.  If IMASK or OMASK are
  NULL, the output masks have already been set to zero.
*/
val = p_find(vblock,"OMASK");
if (*nmasks && val->v_count)
    {
    if (val->v_count == 1)
	{
	/* all output mask values are set to the same value */
	for (i = 0; i < *nmasks; i++)
	    omask[i] = IVAL(*val,0);
	}
    else if (val->v_count == *nmasks)
	{
	/* retrieve the matching output mask values for each input mask value */
	for (i = 0; i < *nmasks; i++)
	    omask[i] = IVAL(*val,i);
	}
    else if (val->v_count != *nmasks)
	{
	c_errmsg("IMASK and OMASK do not contain same number of values",
	    "rescale-masks", &nonfatal);
	c_errmsg("Fatal error encountered", "rescale-fatal", &fatal);
	}
    }


/*
  Initialize the data shift values
*/
for (i = 0; i < *totbnd; i++)
    shift[i] = 0.0;


/*
  Retrieve the data shift values
*/
val = p_find(vblock,"SHIFT");
if (val->v_count == 1)
    {
    for (i = 0; i < *totbnd; i++)
	shift[i] = RVAL(*val,0);
    }
else if (val->v_count == *totbnd)
    {
    for (i = 0; i < *totbnd; i++)
	shift[i] = RVAL(*val,i);
    }
else if (val->v_count != 0)
    {
    c_errmsg("Invalid number of values for SHIFT","rescale-num",&nonfatal);
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);
    }


/*
  Convert the shift values so they can be applied at the same time the offset
  is applied.  This will save one subtraction per pixel.  The equation shown
  in the scaling documents is:

  scaled = (actual + shift) * scale + offset
	 = actual * scale + shift * scale + offset
	 = actual * scale + (shift * scale + offset)
	 = actual * scale + newoffset
*/
for (i = 0; i < *totbnd; i++)
    ioffset[i] += (shift[i] * iscale[i]);


/*
  Convert iscale to be the inverse of what the user specifies.  This will allow
  us to use a multiplication in the rescaling computation instead of a division.
  This can only be done after we have recomputed the shift values and added them
  to the ioffset.
*/
for (i = 0; i < *totbnd; i++)
    if (iscale[i] != 0.0)
	iscale[i] = 1.0 / iscale[i];


/*
  Determine the default min/max values for the output pixels
*/
switch (*odtype)
    {
    case EBYTE:
	minimum = MINUTINY;
	maximum = MAXUTINY;
	break;
    case EWORD:
	minimum = MINWORD;
	maximum = MAXWORD;
	break;
    case ELONG:
	minimum = MINLONG;
	maximum = MAXLONG;
	break;
    case EREAL:
	minimum = MINREAL;
	maximum = MAXREAL;
	break;
    case EUWORD:
	minimum = MINUWORD;
	maximum = MAXUWORD;
	break;
    case EULONG:
	minimum = MINULONG;
	maximum = MAXULONG;
	break;
    default:
	c_errmsg("Unknown output data type", "rescale-unkwn", &nonfatal);
	c_errmsg("Fatal error encountered", "rescale-fatal", &fatal);
	break;
    }


/*
  Initialize the output max/min values for the output data type
*/
for (i = 0; i < *totbnd; i++)
    {
    omax[i] = maximum;
    omin[i] = minimum;
    }


/*
  Retrieve the output max values
*/
val = p_find(vblock,"OMAX");
if (val->v_count == 1)
    {
    for (i = 0; i < *totbnd; i++)
	omax[i] = RVAL(*val,0);
    }
else if (val->v_count == *totbnd)
    {
    for (i = 0; i < *totbnd; i++)
	omax[i] = RVAL(*val,i);
    }
else if (val->v_count)
    {
    c_errmsg("Invalid number of values for OMAX","rescale-num",&nonfatal);
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);
    }


/*
  Retrieve the output min values
*/
val = p_find(vblock,"OMIN");
if (val->v_count == 1)
    {
    for (i = 0; i < *totbnd; i++)
	omin[i] = RVAL(*val,0);
    }
else if (val->v_count == *totbnd)
    {
    for (i = 0; i < *totbnd; i++)
	omin[i] = RVAL(*val,i);
    }
else if (val->v_count)
    {
    c_errmsg("Invalid number of values for OMIN","rescale-num",&nonfatal);
    c_errmsg("Fatal error encountered","rescale-fatal",&fatal);
    }


/*
  Ensure the output mask vaules and the output min/max
  values are in the data range for the output data type
*/
for (i = 0; i < *nmasks; i++)
    if ((omask[i] < minimum) || (omask[i] > maximum))
	{
	c_errmsg("OMASK out of range", "rescale-range", &nonfatal); 
	c_errmsg("Fatal error encountered", "rescale-fatal", &fatal); 
	}
for (i = 0; i < *totbnd; i++)
    {
    if (omax[i] > maximum)
	{
	c_errmsg("OMAX out of range", "rescale-range", &nonfatal); 
	c_errmsg("Fatal error encountered", "rescale-fatal", &fatal); 
	}
    if (omin[i] < minimum)
	{
	c_errmsg("OMIN out of range", "rescale-range", &nonfatal); 
	c_errmsg("Fatal error encountered", "rescale-fatal", &fatal); 
	}
    }


/*
  Retrieve the rounding option
*/
val = p_find(vblock,"ROUNDOPT");
if (strcmp(SVAL(*val,0),"ROUND") == 0)
    *rndopt = TRUE;
else
    *rndopt = FALSE;

/*
  Retrieve globals
*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);

val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return(E_SUCC);
}
