/*******************************************************************************
NAME:		PROCESS

PURPOSE:	Open the files, and rescale the input buffer for the
		specified data types.
  
*******************************************************************************/
#include <stdio.h>
#include "las.h"


/* The following variables have been declared global for image I/O routines
-----------------------------------------------------------------------------*/
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


FUNCTION process (vblock, hosin, hosout, minmax, bands, idtype, imask, msgtim,
                  nbands, nmasks, nmrimg, odtype, omask, rndopt, totbnd,
                  window, iscale, ioffset, oscale, ooffset, omax, omin)

struct PARBLK *vblock;		/*  TAE parameter block			     */

char	
	hosin[][CMLEN],		/* host names of input images		     */
	hosout[],		/* host name of output image		     */
	minmax[];		/* min max flag                              */

long	
	bands[][MAXBND+1],	/* bands to be processed                     */
	idtype,			/* data type of input data                   */
	imask[],		/* Input image mask values                   */
	msgtim,			/* elapsed time between display messages     */
	nbands[],		/* number of bands per input image           */
	nmasks,			/* Number of input image mask values         */
	nmrimg,			/* number of input images                    */
	odtype,			/* data type of output data                  */
	omask[],		/* Output image mask values                  */
	rndopt,			/* Rounding option (TRUE, FALSE)	     */
	totbnd,			/* total number of bands to be processed     */
	window[][4];		/* window specification of input images      */

double	
	iscale[],		/* Input scale factors			     */
	ioffset[],		/* Input offset factors			     */
	oscale[],		/* Output scale factors			     */
	ooffset[],		/* Output offset factors		     */
	omax[],			/* Maximum output pixel values               */
	omin[];			/* Minimum output pixel values               */
{
double  minval[MAXBND];		/* min pixel value array                     */
double  maxval[MAXBND];		/* max pixel value array                     */
double  line_inc;		/* line increment 			     */
double  samp_inc;		/* sample increment			     */

long
	access,			/* image I/O access type                     */
	band,			/* loop index counting the bands             */
	isize,			/* number of bytes per pixel in input 	     */
	mmflag,			/* Min/Max calculation flag		     */
	msg_flag = ALLCAT,	/* flag for processing message               */
	nonfatal = 1,		/* value of one                              */
	one = 1,		/* one					     */
	opt[3], 		/* conversion options			     */
	osize,			/* number of bytes per pixel in output 	     */
	status,			/* function return status code               */
	tmp_idtype,		/* Temporary input data type                 */
	tmp_odtype,		/* Temporary output data type                */
	upddr_flag;	        /* flag for update DDR			     */

int indesc;			/* Input file descriptor                     */
int outdesc;			/* Output file descriptor                    */
unsigned char *inbuf;		/* Input buffer                              */
unsigned char *outbuf;		/* Output buffer                             */


/*
  LAS does not currently support unsigned data types.  Convert the EUWORD and
  EULONG data types for passing to the LAS routines.  We will be "tricking"
  them into thinking they are working with EWORD and ELONG, respectively.
*/
switch (idtype)
    {
    case EUWORD:
	tmp_idtype = EWORD;
	break;
    case EULONG:
	tmp_idtype = ELONG;
	break;
    default:
	tmp_idtype = idtype;
	break;
    }
switch (odtype)
    {
    case EUWORD:
	tmp_odtype = EWORD;
	break;
    case EULONG:
	tmp_odtype = ELONG;
	break;
    default:
	tmp_odtype = odtype;
	break;
    }


/*
  Setup for input data type
*/
switch(idtype)
   {
   case EBYTE:  isize = 1; break;
   case EWORD:  isize = 2; break;
   case ELONG:	isize = 4; break;
   case EREAL:  isize = 4; break;
   case EUWORD: isize = 2; break;
   case EULONG: isize = 4; break;
   }

/*
  Setup for output data type
*/
switch(odtype)
   {
   case EBYTE:  osize = 1; break;
   case EWORD:  osize = 2; break;
   case ELONG:	osize = 4; break;
   case EREAL:  osize = 4; break;
   case EUWORD: osize = 2; break;
   case EULONG: osize = 4; break;
   }


/*
  Set the options
  (default data type conversion, default compression, array end)
*/
opt[0] = opt[1] = opt[2] = 0;


/*
  Temporary fix.  LAS sequentail imageio does not work with an odd number of
  lines and/or samples.  Use random imageio and assume only one input image.
*/
band = 0;       /* open image with all bands but only process requested ones */
access = IREAD;
status = c_eopenr(&indesc, hosin[0], &band, &window[0][SL], &window[0][SS],
    &window[0][NL], &window[0][NS], &access, &tmp_idtype, opt, &one);
if (status != E_SUCC)
    c_clean(vblock, "rescale", hosout, &one);


/*
  Open output image
*/
access = IWRITE;	
status = c_eopenr(&outdesc, hosout, &totbnd, &one, &one, &window[0][NL],
    &window[0][NS], &access, &tmp_odtype, opt, &one);
if (status != E_SUCC)
    {
    c_eclose(&indesc);
    c_clean(vblock, "rescale", hosout, &one);
    }


/*
  Allocate I/O buffers
*/
isize *= window[0][NS];
osize *= window[0][NS];
inbuf  = (unsigned char *)malloc(isize);
outbuf = (unsigned char *)malloc(osize);
if ((inbuf == NULL) || (outbuf == NULL))
    {
    c_errmsg("Error allocating dynamic memory","rescale-alloc",&nonfatal);
    free(inbuf);
    free(outbuf);
    c_eclose(&indesc);
    c_eclose(&outdesc);
    c_clean(vblock,"rescale",hosout,&one);
    }
strcpy(inbuf, "?");
strcpy(outbuf, "?");


/*
  Initialize the min/max arrays.  mmflag is set to be used within the
  processing loop because it is quicker than strcmp().  Since LAS does not
  support unsigned data types, the min/max calculation is a waste of time
  because it will not mean anything.
*/
mmflag = FALSE;
if ((strcmp(minmax,"YES") == 0) && ((odtype == EUWORD) || (odtype == EULONG)))
    {
    c_errmsg("Not computing min/max", "rescale-mmflag", &nonfatal);
    strcpy(minmax, "NO");
    }
else if (strcmp(minmax,"YES") == 0)
    {
    c_inmnmx(&odtype,&totbnd,minval,maxval);
    mmflag = TRUE;
    }


/*
  Setup for processing message utility
*/
c_inittm("RESCALE", "Processed", "line", "bands", " ", &window[0][NL], &totbnd,
    &msgtim, &msg_flag);
curent = 1;    
c_startm();


/*
  Main processing loop
*/
for (curent = 1; curent <= window[0][NL]; curent++)
    {
    /*
      Process each band
    */
    for (band = 1; band <= totbnd; band++)
	{
	/*
	  Get a line
	*/
	status = c_eread(&indesc, &bands[0][band-1], &curent, inbuf, &one);
	if (status != window[0][NS])
	    {
	    c_errmsg("Error reading image", "rescale-read", &nonfatal);
	    free(inbuf);
	    free(outbuf);
	    c_eclose(&indesc);
	    c_eclose(&outdesc);
	    c_clean(vblock, "rescale", hosout, &one);
	    }

	/*
	  Rescale the input
	*/
	c_pxscale(inbuf, idtype, window[0][NS], iscale[band-1], ioffset[band-1],
	    imask, nmasks, outbuf, odtype, oscale[band-1], ooffset[band-1],
	    omask, omax[band-1], omin[band-1], rndopt);

	/*
	  Calculate the min/max if specified
	*/
	if (mmflag)
	    c_pxmnmx(outbuf, &window[0][NS], &minval[band-1], &maxval[band-1],
		&odtype); 
	

	/*
	  Write the line
	*/
	status = c_ewrite(&outdesc, &band, &curent, outbuf, &one);
	if (status != window[0][NS])
	    {
	    c_errmsg("Error writing image", "rescale-write", &nonfatal);
	    free(inbuf);
	    free(outbuf);
	    c_eclose(&indesc);
	    c_eclose(&outdesc);
	    c_clean(vblock, "rescale", hosout, &one);
	    }
	}
    }


/*
  Turn off display messages
*/
c_stoptm();


/*
  Display the number of pixels too large/small
*/
c_pxscale_report();


/*
  Update the DDR
*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock, hosin[0], hosout, &nmrimg, window, bands, nbands,
    &line_inc, &samp_inc, &totbnd, &upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"rescale",hosout,&one);

/*
  Save the min/max values
*/
if (c_svmnmx(hosout,&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","rescale-warn",&one);

/*
  Close all image files
*/
c_eclose(&indesc);
c_eclose(&outdesc);

/*
  Free buffer
*/
free(inbuf);
free(outbuf);

return(E_SUCC);
}
