/****************************************************************************
NAME:	CALC_NAVSP

PURPOSE:  Calculate the navspasur values
		(1) equatorial crossing
		(2) time of ascending node
		(3) orbit number
	  for each of the orbits per day.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      03/92    T. Baltzer	        Original Development

ALGORITHM REFERENCES:   none
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "worgen.h"
#include "util2.h"
#include "adaps.h"
#include "adapsutil.h"
#include "navspasur.h"

#define ORBITPER96  56		/* app. number of orbits per 96 hour period  */

#define SPO  6120		/* approx. seconds per orbit (102 minutes) */

int calc_navsp
(
char	line1[],	/* Navspasur line of data to process		*/
FILE	**print_file	/* File descriptor of the output print file	*/
)

{
double	anlng,		/* Longitude of ascending node                  */
	deglong,	/* longitude in degrees not radians		*/
        inclin,		/* Inclination angle                            */
	lasta1,		/* Last true anomaly past perigee		*/
        lastas,		/* Last true anomaly past equator               */
	lastime,	/* Last acumulative time of flight since epoch	*/
        lastlong,	/* Last longitude of satellite			*/
	rsat,		/* Distance of satellite from earth center      */
        slat,		/* Geocenteric latitude of satellite            */
        slong,		/* Longitude of satellite                       */
	temptime,	/* Temporary variable to hold time of flight	*/
        thetas,		/* True anomaly past equator                    */
	tof = 0.0,	/* Time of flight, init to 0 seconds past epoch */
	truea1,		/* True anomaly past perigee                    */
	truea2,		/* True anomally of next point			*/
	ttof = 0.0;	/* Acumulative time of flight since epoch	*/

long	day,		/* day of acquisition				*/
        eorbit,		/* Orbit counter from equator                   */
	hour = 0,	/* hardwire a value, doesn't matter here	*/
	prevhr = 0,	/* hour value on previous iteration		*/
	jdate,		/* julian date					*/
	minut = 0,	/* hardwire a value, doesn't matter here	*/
	month,		/* month of acquisition				*/
	norbits,	/* orbit number					*/
        porbit,		/* Orbit counter from perigee			*/
	satnum,		/* satellite number				*/
	sec = 0,	/* hardwire a value, doesn't matter here	*/
	status,		/* return status				*/
	year,		/* year of acquisition				*/
	zdot;		/* Ascending(1) / descending(-1) flag           */

char	satnam[8],	/* satellite name (NOAA-11)			*/
	satno[4],	/* satellite number (N11)			*/
	satcode[2],	/* satellite code (I)				*/
	satid[6];	/* satellite id (19531 for example)		*/

struct MODEL    model;	/* Model and orbital information		*/


strncpy(satid,line1,5);
satid[5] = '\0';
if (c_getsat("SATID",&satnum,satnam,satno,satid,satcode) != E_SUCC)
   return(E_FAIL);

sscanf(&line1[63],"%2d",&year);
sscanf(&line1[65],"%2d",&month);
sscanf(&line1[67],"%2d",&day);

c_grejul(&year,&month,&day,&jdate);

/*
  Load the model data structure
*/
status = c_setephem(line1,&model);
if (status != E_SUCC)
   return(E_FAIL);

/*
  Print the header
*/
fprintf (*print_file, "  Ascending Equatorial Crossings For Satellite %s \n",
	 satnam);
fprintf (*print_file, "           Epoch Date: %02d/%02d/%02d   (YY/MM/DD)\n\n",
         year, month, day);
fprintf (*print_file, "         Orbit    Node Time (GMT)  Node Crossing \n");
fprintf (*print_file, "         Number   (DDD:HH:MM:SS)   Longitude     \n");
fprintf (*print_file, "         --------------------------------------- \n");

for (norbits = 1; norbits <= ORBITPER96; norbits++)
   {
   /*
     Call brouwer() to retrieve initial truea1, thetas for this orbit.
   */
   brouwer(&ttof,&model,&truea1,&rsat,&thetas,&anlng,&inclin,&zdot,
   	   &eorbit,&porbit,&slat,&slong);

   /* 
     Calculate the first truea2
   */
   truea2 = truea1 - thetas;

   for ( ; ; )
      {
      /*
        Get the time of flight
      */
      timeoflight(&truea1,&truea2,&model,&tof);

      /*
	Check if converged within satisfactory tollerances.  If so,
        call brouwer() one last time to make sure the correct orbit
	number has been found (round-off errors might effect the
	orbit number coming out of brouwer() when near the equator).
	Increment in time slightly and use this orbit number (eorbit). 
      */
      if (fabs(tof) < 1.0)
         {
         lastime = ttof + 150.0;
         brouwer(&lastime,&model,&lasta1,&rsat,&lastas,&anlng,&inclin,
		 &zdot,&eorbit,&porbit,&slat,&lastlong);
         break;
         }

      /*
        Call brouwer() with the most recent time of flight
      */
      ttof += tof;
      brouwer(&ttof,&model,&truea1,&rsat,&thetas,&anlng,&inclin,&zdot,
      	      &eorbit,&porbit,&slat,&slong);

      /*
        Make sure that thetas is the distance from the closest
        equatorial crossing.  If the equator has not yet been crossed,
        set the orbit number to the next orbit.
      */
      if (thetas > PIE)
         {
         thetas -= PIE2;
         eorbit++;
         }

      /* 
        Adjust true anomoly and reiterate
      */
      truea2 = truea1 - thetas;
      }

      /*
        The first orbit found may be the last one from the previous
	day (the total accumulated time is negative).  If so, skip
	over this orbit and keep going.  If this really is an orbit
	from today (the total accumulated time is positive), print
	it out.
      */
      if (ttof > 0.0)
         {
         temptime = ttof;
         day = (long)temptime / SEC_PER_DAY;
         temptime -= (day * SEC_PER_DAY);
         hour = (long)temptime / 3600;
         temptime -= (hour * 3600);
         minut = (long)temptime / 60;
         temptime -= (minut * 60);
         sec = (long)temptime;
	 /* convert longitude from radians East to degrees West */
	 deglong = (PIE2 - slong) * 180.0 / PIE;
	 if (hour < prevhr) /* new day */
	    jdate++;
         fprintf (*print_file, "          %5d   %3d:%02d:%02d:%02d      %7.3f W\n",
                  eorbit,jdate,hour,minut,sec,deglong);
	 prevhr = hour;
         }
      else
         norbits--;

   /*
      Prepare to compute the next orbit
   */
   ttof += SPO;

   }
   fprintf (*print_file, "\n\n");

return(E_SUCC);
}
