/****************************************************************************
NAME:	PUT_NORAD

PURPOSE:  Take the norad data and print it to the specified file, also
	  calculate information to be entered into the Antenna Control
	  Unit(ACU).

ALGORITHM:  See in-line documentation

ALGORITHM REFERENCES:   none
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "worgen.h"
#include "util2.h"
#include "constants.h"
#include "norad.h"

void put_norad
(
char   satid[],            /* Satellite id number */
long   epyr,               /* Epoch year */
long   epdy,               /* Epoch day */
double epfd,               /* Epoch fractional day */
double inclin,             /* Inclination angle */
double rtasc,              /* Right ascension of the ascending node */
double ecc,                /* Eccentricity */
double argper,             /* Argument of perigee */
double meanan,             /* Mean anomaly */ 
double meanm,              /* Mean motion */
FILE   **print_file        /* File descriptor of the output print file */
)

{

char   acu[39];            /* Compressed line for ACU(no blanks or periods) */ 

long   dtime,              /* Delta time in real format */
       nodtim[3],          /* Nodal time (HH:MM:SS) */
       satnum;             /* Satellite number (1 - NO9, 2 - N10, 3 - N11,  */
			   /* 4 - N12, 5 - N14, 6 - N15, 7 - N16, etc )     */

double a1, a2,             /* Variables for calculations */
       a3, a4,             /* Variables for calculations */
       a5, a6,             /* Variables for calculations */
       a7, a8,             /* Variables for calculations */
       orbrds,             /* Orbit radius */
       nodprd,             /* Nodal Period */
       asdnod,             /* Ascending Node */
       delta,              /* Delta time in real format */
       dsec,               /* Delta time in decimal format */
       rsum,               /* Sum of perigee and mean anomaly */
       dec;                /* Decimal representation of nodal time */

/* Set a satellite number according to the satid
This number refers to the ACU satellite number.
Reset the series beginning with NOAA 10 as 1.
------------------------------------------------*/
     if (strcmp (satid, "16969") == 0)
        satnum = 1;
     else if (strcmp (satid, "29499") == 0)
        satnum = 2;
     else if (strcmp (satid, "21263") == 0)
        satnum = 3;
     else if (strcmp (satid, "23455") == 0)
        satnum = 4;
     else if (strcmp (satid, "25338") == 0)
        satnum = 5;
     else if (strcmp (satid, "26536") == 0)
        satnum = 6;
     else if (strcmp (satid, "27453") == 0)
        satnum = 7;
     else if (strcmp (satid, "28654") == 0)
        satnum = 8;
     else if (strcmp (satid, "33591") == 0)
        satnum = 9;     


     a1 = pow ((double)(11467.87423 / pow ((double)(meanm * PIE2),2.0)),
          (1.0 / 3.0));
     a2 = (a1 * a1) * pow ((double)(1.0 - (ecc * ecc)),(3.0 / 2.0));

     a3 = 1.0 - (pow (sin(inclin * RAD_PER_DEG),2.0) * 1.5);
     a2 = (-1.5 * 0.001082549 * a3) / a2;

     a3 = (((1.0 / 3.0) * a2) + 1.0) * a1;
     a2 = a3 - ((a2 * a2) / 3.0) + 0.0002;

/* Derive the nodal period and the orbital radius from above information
------------------------------------------------------------------------*/  
     nodprd = (pow(a2,(3.0 /2.0)) * 1440.0 * 0.058673) + 0.0005;
     orbrds = (a2 * 6378.17) + 0.0005;

     /*
       DLLOYD Y2K -- do year calculations with 4-digit years
    ---------------------------------------------------------*/
     a5 = (c_4digityr(epyr) - 1952.0) * 365.0;
     a5 += epdy + (long)(((c_4digityr(epyr) - 1952.0) + 3.0) / 4.0);
     a6 = (a5 + 18991.50) / 36525.0;
     a3 = (a6 * a6) * 0.0929;

     a4 = ((8640184.542 * a6) + 23925.836 + a3) / 240.0;
     a3 = fmod(a4,360.0) * RAD_PER_DEG;

     a7 = ((0.00007292115 * 86400.0 * epfd) + a3) * DEG_PER_RAD;
     if (a7 >= rtasc)
        a8 = rtasc + 360.0;
     else 
        a8 = rtasc;

/* Find the ascending node from A7 and A8
-----------------------------------------*/
     asdnod = 360.0 - (a8 - a7) + 0.0005;
     if (asdnod >= 360.0)
        asdnod -= 360.0;

/* Calculate the Nodal Time from the epoch fractional day
---------------------------------------------------------*/
     rsum = argper + meanan;
     if (rsum >= 360.0)
        rsum -= 360.0;
 
     delta = ((-rsum * nodprd) / 360.0) * 60.0;
     if (delta < 0.0)
        dtime = (long)(delta - 0.5);
     else
        dtime = (long)(delta + 0.5);

     dsec = dtime / 3600.0;
  
     dec = 24 * epfd;
     dec += dsec;

     nodtim[0] = (long)(dec);
     nodtim[1] = (long)((dec - nodtim[0]) * 60);
     nodtim[2] = (long)((dec - nodtim[0] - nodtim[1] / 60.0) * 3600);

/* Generate a string for the ACU that contains no decimal points or blanks.
   Multiply doubles by 1000 to save up the the third value past the decimal
   point.  Zero fill blanks
---------------------------------------------------------------------------*/
     sprintf (acu, "%1d%07d%07d%07d%07d%03d%02d%02d%02d", satnum,
              (long)((orbrds * 1000) + .5), (long)((inclin * 1000) + .5),
              (long)((nodprd * 1000) + .5), (long)((asdnod * 1000) + .5),
              epdy, nodtim[0], nodtim[1], nodtim[2]);


/* Write the information to a file for output to the device specified
---------------------------------------------------------------------*/
     fprintf (*print_file, "         Input Norad Parameters \n");
     fprintf (*print_file, " ______________________________________\n");
     fprintf (*print_file, "\n");
     fprintf (*print_file, " Satellite ID          %s\n",satid);
     fprintf (*print_file, " Epoch Year            %d\n",epyr);
     fprintf (*print_file, " Epoch Day             %d\n",epdy);
     fprintf (*print_file, " Epoch Fractional Day   %11.8lf\n",epfd);
     fprintf (*print_file, " Inclination            %8.5lf\n",inclin);
     fprintf (*print_file, " Right Ascension Node  %8.4lf\n",rtasc);
     fprintf (*print_file, " Eccentricity            %9.7lf\n",ecc);
     fprintf (*print_file, " Argument of Perigee   %8.4lf\n",argper);
     fprintf (*print_file, " Mean Anomaly          %8.4lf\n",meanan);
     fprintf (*print_file, " Mean Motion            %11.8lf\n",meanm);
     fprintf (*print_file, "\n");
     fprintf (*print_file, "\n");
     fprintf (*print_file, "        Output Orbital Parameters\n");
     fprintf (*print_file, " _______________________________________\n");
     fprintf (*print_file, "\n");
     fprintf (*print_file, " Satellite Number      %d\n",satnum);
     fprintf (*print_file, " Satellite ID          %s\n",satid);
     fprintf (*print_file, " Orbital Radius        %8.3lf KM\n",orbrds);
     fprintf (*print_file, " Orbital Inclination     %8.5lf\n",inclin);
     fprintf (*print_file, " Nodal Period          %8.3lf\n",nodprd);
     fprintf (*print_file, " Ascending Node        %8.3lf W\n",asdnod);
     fprintf (*print_file, " Nodal Time (DDD:HH:MM:SS) %3d:%02d:%02d:%02d\n", 
              epdy,nodtim[0],nodtim[1],nodtim[2]);
     fprintf (*print_file, " Nodal Time Correction     %3d Seconds\n",dtime);
     fprintf (*print_file, "\n");
     fprintf (*print_file, "\n");
     fprintf (*print_file, "       Antenna Control Unit String\n");
     fprintf (*print_file, " _______________________________________\n");
     fprintf (*print_file, "\n");
     fprintf (*print_file, " %s\n",acu);
     fprintf (*print_file, "\n");
     fprintf (*print_file, "\n");
     fprintf (*print_file, "\n");
     fprintf (*print_file, "\n");

/***********
  Somewhere in here the ACU string should be sent to the ACU whenever
  The calls are done  ??????????
***********/

}
