/*****************************************************************************
NAME:			SATRACK

PURPOSE:  Read one-line navspasur data and two-line norad data from a
          single input file and write them to the appropriate update file 


PROGRAM HISTORY:
Version     Date     Author             Description
-------     ----     ------             --------------
 5.0        10/91    E. Cihak		Original Development
 5.1        02/92    T. Baltzer		Complete original development
 5.2        02/92    T. Baltzer		Make mods so it will converge
					to a solution (timeoflight()).
 5.3        01/95    J. Fenno		Added code for NOAA-14.
 5.5	    06/98    D. Lloyd		Added code for NOAA-15
 5.6	    09/00    D. Lloyd		Added code for NOAA-16
 5.7	    04/02    D. Lloyd		Added code for NOAA-17
 5.8	    11/04    D. Lloyd		Added code for NOAA-18
 5.9        05/07    D. Lloyd           Added code for METOP-2
 5.10       07/09    K.Vanumamalai	Added code for NOAA-19

PROJECT:       ADAPS

ALGORITHM:     See in-line documentation

ALGORITHM REFERENCES:      none
******************************************************************************/
#include <stdio.h>
#include <sys/file.h>
#include "las.h"
#include "util.h"
#include "util2.h"
#include "labserv.h"
#include "model.h"
#include "navspasur.h"
#include "norad.h"
#include "satrack.h"

#define   LINESIZ     80

int main( void )
{
struct PARBLK     vblock;           /* TAE parameter block */

long   access = 4,	/* File access(0=exist 1=execute 2=write 4=read)*/
       stdlun,          /* Logical unit number */
       n_out,           /* Number of output files */
       navspid,	        /* NAVSPASUR Satellite id */
       odecay,		/* Orbital decay */
       nonfatal = 1,    /* Nonfatal error code */
       one = 1,		/* constant value */
       len;             /* Length of ptr1 string */  

                        /* Satellite numbers to print out */
static long numsat[TOTSAT] ={0,0,0,0,0,0,0,0,0,0};

char   filnam[CMLEN],		 /* Name of input file */
       fileutil_file[CMLEN],     /* Temporary file for fileutil */
       out[1][CMLEN],            /* List of output files */
       line1[EPHEM_SIZ+1],	 /* Line read from input file */
       line2[LINESIZ],           /* Line read from input file */
       navid[TOTSAT],		 /* NAVSPASUR Satellite id */
       norid[TOTSAT],		 /* Norad satellite id */
       ptr1[LINESIZ],            /* String for lines from input file */
       *pointer1,                /* Pointer to line in input file */ 
       *pointer2;                /* Pointer to line in input file */ 

FILE   *fin,                     /* File descriptor for input file */
       *print_file;              /* Name of output file */ 

/* Initialize LAS
-----------------*/
c_inlas (&vblock, &stdlun);

/* Get the input file name and call fileutil to set print options
-----------------------------------------------------------------*/
get_trk_par (&vblock, filnam, &print_file, out, &n_out, numsat, fileutil_file); 

/* Make sure the file exists with read permission, then open the
   input file
-----------------------------------------------------------------*/
if ( c_lsstat(filnam,&access) != E_SUCC )
   {
   c_errmsg ("Error locating file","satrack-locf",&nonfatal);
   fclose (print_file);
   c_clean(&vblock,"SATRACK",(const char (*)[CMLEN])fileutil_file,&one);
   }

fin = fopen (filnam, "r");
if (fin == NULL)
    {
    c_errmsg ("Error opening input file", "satrack-inopn", &nonfatal);
    fclose (print_file);
    c_clean(&vblock,"SATRACK",(const char (*)[CMLEN])fileutil_file,&one);
    }

/* Read input file line by line and establish name of update file.
   The infinite loop is broken when no more input data is available.
-----------------------------------------------------------------*/
for ( ; ; )
    {
    /*
      Read a line from the file
    */
    pointer1 = fgets (ptr1, LINESIZ, fin);
    if (pointer1 == NULL)
	break;

    len = (long)strlen (ptr1);
    /* CR 8122  It doesn't appear that the input files have changed so perhaps
    we've just been lucky all these years.  If a blank line is encountered, or
    one with only 1 character, data from the previous read is still in the
    line1 string.  If the previous line was a valid 2-line element line and
    the blank line follows, norid will be extracted and all following output
    will be garbage.  Several options such as clearing line1 or testing for
    less than 8 characters could be used to prevent this from happening.
    However, any valid ephemeris line must contain a minimum of 69 characters.
    If it does not continue on to the next line.
    -------------------------------------------------------------------------*/
    if (len < 69)
        continue;
    if (len > 70)
	{
        strncpy (&line1[0], &ptr1[0], 69);
        line1[69] = '\0';
        }
    else
        strcpy (line1, ptr1);
  
    strncpy (navid, line1, 5);
    navid[5] = '\0';
    strncpy (norid, &line1[2], 6);  
    norid[6] = '\0';

    /*
      Check if the line is a navspasur line
    */
    if ((strcmp(navid,"15427") == 0) || (strcmp(navid,"16969") == 0) ||
        (strcmp(navid,"19531") == 0) || (strcmp(navid,"21263") == 0) ||
	(strcmp(navid,"23455") == 0) || (strcmp(navid,"25338") == 0) ||
	(strcmp(navid,"26536") == 0) || (strcmp(navid,"27453") == 0) ||
	(strcmp(navid,"28654") == 0) || (strcmp(navid,"29499") == 0) ||
        (strcmp(navid,"33591") == 0))
        {
	/*
	  Some times the orbital decay is way off.  It needs to be reset
          to zero when it is.
	*/
	sscanf(&line1[26],"%5d",&odecay);
	if (odecay > 90000)
	    strncpy(&line1[26],"00000",5);

        if (put_navsp(line1,&print_file,numsat) != E_SUCC)
            {
	    fclose (print_file);
    	    c_clean(&vblock,"SATRACK",(const char (*)[CMLEN])fileutil_file,
	        &one);
	    }
        }
    else
        {
	/*
	  Not a navspasur line, it may be a norad line
	*/

	if ((strcmp(norid, "15427U") == 0) || (strcmp(norid, "16969U") == 0) ||
	    (strcmp(norid, "19531U") == 0) || (strcmp(norid, "21263U") == 0) ||
	    (strcmp(norid, "23455U") == 0) || (strcmp(norid, "25338U") == 0) ||
	    (strcmp(norid, "26536U") == 0) || (strcmp(norid, "27453U") == 0) ||
	    (strcmp(norid, "28654U") == 0) || (strcmp(norid, "29499U") == 0) ||
            (strcmp(norid, "33591U") == 0)) 
	   {
	   /* CR 8122  As a further confidence check, the first character of
	   line one of 2-line ephemeris must be a 1.  Check for this and
	   continue if it is not.
	   ----------------------------------------------------------------*/
	   if ( line1[0] != '1' )
	       {
	       c_errmsg("First 2-line element in error","satrack-inread",
	                &nonfatal);
	       continue;
	       }
	   pointer2 = fgets (line2, LINESIZ, fin);  
	   /* CR 8122  Finally line 2 must have a 2 as first character
	   -----------------------------------------------------------*/
	   if ( line2[0] != '2' )
	       {
	       c_errmsg("Second 2-line element in error","satrack-inread",
	                &nonfatal);
	       continue;
	       }
	   if (pointer2 == NULL)
	      {
	      c_errmsg ("Error reading input file", "satrack-inread", 
			&nonfatal);
	      fclose (print_file);
    	      c_clean(&vblock,"SATRACK",(const char (*)[CMLEN])fileutil_file,
	          &one);
	      }

	   sscanf (norid, "%5d", &navspid);
           if ( gen_rpt(navspid,numsat) == E_SUCC)
	      sep_norad (line1, line2, &print_file ); 
	   }
        }
    }		/* end of infinite loop construct */

/* Close the input file and the output file  
-------------------------------------------*/
fclose (fin);
fclose (print_file);

/* Write a completion message
-----------------------------*/
c_complt ("SATRACK", out, &n_out); 
return E_SUCC;
}
