/****************************************************************************
NAME:		TBUS

PURPOSE:	Convert TBUS information into a comparable ephemeris
		information.  The calculated information is the used
		to update the specified satellite ephemeris files.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  2.0	 J. Matzner    10/92	Original development
  2.0	 D. Hollaren   07/93	Modified to read the input file better
				and add the TBUS to the correct position
				in the output file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        TBUS must be run under TAE.

PROJECT                         ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adapsutil.h"
#include "util2.h"
#include "model.h"
#include "constants.h"
#include "navspasur.h"
#include "tbus.h"

int main( void )
{
/* Satellite numbers to print out*/
static long numsat[TOTSAT] = {9,10,11,12,14,15,16,17,18,19};

FILE    *in_fd,		/* File which will contain parm list		*/
	*prt_fd;	/* File descriptor to the output print file	*/

char	prtfile[CMLEN], /* Temporary print file				*/
	ofiles[1][CMLEN],/* Array for output file names			*/
	tbus[CMLEN],	/* Input file name				*/
	disk[CMLEN],	/* Full pathname for output file		*/
	msg[CMLEN],	/* Error message string				*/
	satnam[8],	/* Satellite name   (NOAA-11)			*/
	satno[4],	/* Satellite number (N11)			*/
	satid[6],	/* Satellite id     (19531)			*/
	satcode[2],	/* Satellite code   (I)				*/
	satname[4][4],	/* Satellite names to be processed		*/
	line[81];	/* String for reading line of TBUS file		*/

char	outline[EPHEM_SIZ+1];	/* String for outputting ephemeris line	*/

long	satcnt,		/* Number of satellites to be processed		*/
	satnum,		/* Current satellite number from TBUS file	*/
	orbit,		/* Orbit number					*/
	epoch_date,	/* Epoch Date					*/
	epoch_hr,	/* Epoch Hour					*/
	epoch_min,	/* Epoch Minute					*/
	epoch_sec,	/* Epoch Seconds				*/
	anom_per,	/* Anomalistic Period				*/
	arg_per,	/* Argument of perigee         			*/
	ecc,		/* Eccentricity					*/
	inclin,		/* Inclination					*/
	mean_anomaly,	/* Mean anomaly					*/
	mean_motion,	/* Mean motion					*/
	smajor_axis,	/* Semi-major axis				*/
	decay,		/* Orbital decay (negligable)			*/
	lon_anode,	/* Longitude of ascending node			*/
	tsec,		/* Time in seconds of epoch pass		*/
	porbit,		/* Perigee orbits				*/
	eorbit,		/* Equator orbits				*/
	std_out,	/* Standard output				*/
	one = 1,	/* Constant of one				*/
	ocount = 0,	/* Counter for number of output files		*/
	pcount = 0,	/* Satellite processing counter			*/
	nonfatal = 1,	/* Non-fatal error code				*/
	process = FALSE,/* Process flag					*/
	i;		/* Looping variable				*/

double	gha,		/* Greenwich Hour Angle				*/
	right_anode,	/* Right angle of ascending node		*/
	dt,		/* Total HERGS from Epoch			*/
	we,		/* Rotation rate of earth in rad/hergs  	*/
	adj_lon_anode,	/* Adjusted longitude of ascending node 	*/
	adj_mean_anomaly,/* Adjusted mean anomaly			*/
	adj_arg_per,	/* Adjusted argument of perigee			*/
	thetas0,	/* True anomaly past equator at epoch		*/
	temp;


struct 	MODEL  model;	/* Model and orbital information		*/

struct  PARBLK vblock;	/* Parameter block                      	*/

/* Initialize TAE and get the file descriptor for standard output
   -------------------------------------------------------------- */
c_inlas(&vblock,&std_out);

/*
  Get user-specified parameters   
*/
get_tbus_par(&vblock,tbus,satname,&satcnt,&prt_fd,prtfile,ofiles[ocount],
	     &ocount);

/*
  Get output file name in ADAPSTABLES directory 
*/
if (getenv("ADAPSTABLES") == NULL)
    {
    c_errmsg("Error getting environment variable ADAPSTABLES",
	"tbus-getenv", &nonfatal);
    c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
    }
strcpy(disk, getenv("ADAPSTABLES"));

/*
  Open the TBUS input file
*/
in_fd = fopen(tbus,"r");
if (in_fd == NULL)
    {
    sprintf(msg,"Error opening TBUS file %s",tbus);
    c_errmsg(msg,"tbus-open",&nonfatal);
    c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
    }

/*
  Read the TBUS file until the End Of File (EOF) is found
*/
while (fgets(line,80,in_fd) != NULL)
    {
    /*
      Find the start of the satellite information
    */
    while (strstr(line,"APT PREDICT") == NULL)
	{
	if (fgets(line,80,in_fd) == NULL)
	    {
	    if (feof(in_fd))
		break;
	    else
		{
		c_errmsg("Error reading the TBUS file","tbus-read1",&nonfatal);
		fclose(in_fd);
		fclose(prt_fd);
		c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
		}
	    }
	}

    if (feof(in_fd))
	break;

    /*
      Extract the satellite number from the next line
    */
    if (fgets(line,80,in_fd) == NULL)
	{
	c_errmsg("Error reading the TBUS file","tbus-read2",&nonfatal);
	fclose(in_fd);
	fclose(prt_fd);
	c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
	}
    sscanf(line,"%*s %*s %2d",&satnum);

    /*
      Get the other IDs for the satellite
    */
    c_getsat("SATNUM",&satnum,satnam,satno,satid,satcode);

    /*
      Determine if the TBUS for this satellite is to be processed
    */
    process = FALSE;
    for (i = 0; i < satcnt; i++)
	{
	if (strcmp(satname[i],satno) == 0)
	    process = TRUE;
	}

    /*
      Process the TBUS information for this satellite
    */
    if (process)
	{
	pcount++;
	/*
          Find the start of the satellite information needed to calculate
          the ephemeris information
	*/
	while (strstr(line,"PART IV") == NULL)
	    {
	    if (fgets(line,80,in_fd) == NULL)
		{
		c_errmsg("Error reading the TBUS file","tbus-read3",&nonfatal);
		fclose(in_fd);
		fclose(prt_fd);
		c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
		}
	    }
	
	/*
	  Get Orbit ID, Epoch Date and GHA from next line   
	*/
	if (fgets(line,80,in_fd) == NULL)
	    {
	    c_errmsg("Error reading the TBUS file","tbus-read4",&nonfatal);
	    fclose(in_fd);
	    fclose(prt_fd);
	    c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
	    }

	sscanf(line,"%*s %*s %5d %*s %6d%2d%2d%2d%*3d %lf",
		&orbit,&epoch_date,&epoch_hr,&epoch_min,&epoch_sec,&gha);
	gha *= 0.0001;

	/*
	  Get Anomalistic Period, Eccentricity, Arg. of Perigee, 
	  Right Angle of Ascending Node and Inclination    
	*/
	if (fgets(line,80,in_fd) == NULL)
	    {
	    c_errmsg("Error reading the TBUS file","tbus-read5",&nonfatal);
	    fclose(in_fd);
	    fclose(prt_fd);
	    c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
	    }

	sscanf(line,"%d %*d %d %d %lf %d",
		&anom_per,&ecc,&arg_per,&right_anode,&inclin);
	right_anode *= 0.00001;

	/*
	  Calculate the mean motion in HERGS
	*/
	mean_motion = (long)((PIE2 / ((anom_per * 0.0001) * 60.0) * HERG) *
	    100000000);

	/*
	  Convert the argument of perigee, inclination angle, and longitude
	  of the ascending node to REVS
	*/
	arg_per = (long)(((arg_per * 0.00001) / 360.0) * 100000000);
	inclin  = (long)(((inclin  * 0.00001) / 360.0) * 100000000);
	temp = 360 - gha + right_anode;
	if (temp > 360)
	    lon_anode = (long)(((temp - 360.0) / 360.0) *  100000000);
	else
	    lon_anode = (long)((temp / 360.0) * 100000000);

	/*
	  Get Mean Anomaly and Semi Major Axis from next line   
	*/
	if (fgets(line,80,in_fd) == NULL)
	    {
	    c_errmsg("Error reading the TBUS file","tbus-read6",&nonfatal);
	    fclose(in_fd);
	    fclose(prt_fd);
	    c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
	    }

	sscanf(line,"%d %d",&mean_anomaly,&smajor_axis);

	/*
	  Convert the mean anomaly to REVS
	*/
	mean_anomaly = (long)(((mean_anomaly * .00001) / 360.0) * 100000000);

	/*
	  Set the orbital decay to zero because it is negligible   
	*/
	decay = 0;

	/*
	  Create the output line
	*/
	sprintf(outline,"%5s%05d%08ld%08ld%05ld%08ld%08ld%08ld%08ld%06ld",
		satid,orbit,mean_anomaly,mean_motion,decay,ecc,arg_per,
		lon_anode,inclin,epoch_date);

	/*
	  Load the ephemeris information into the model structure
	*/
	if (c_setephem(outline,&model) != E_SUCC)
	    {
            c_errmsg("Error setting ephemeris structure","tbus-setephem",
		     &nonfatal);
	    fclose(in_fd);
	    fclose(prt_fd);
	    c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
	    } 
		    
	/*
	  Convert Mean Anomaly, Argument of Perigee, and Long. of Ascending
	  Node for Epoch Time so is relative to midnight GMT which is
	  required for ephemeris.
	*/

	/*
	  Adjust Mean Anomaly
	*/
	tsec = (epoch_hr * 3600) + (epoch_min * 60) + epoch_sec;
	dt = -tsec / HERG;
	adj_mean_anomaly = model.mean_anomaly + model.mean_motion * dt +
			   model.decay * dt * dt;

	/*	
	  Adjust Argument of Perigee
	*/
	adj_arg_per = model.arg_per + model.der_argper * 
		      (adj_mean_anomaly - model.mean_anomaly);


	/*
	  Adjust Longitude of Ascending Node
	*/
	we = WE * HERG;
	adj_lon_anode = model.lon_anode + model.der_anode *
			(adj_mean_anomaly - model.mean_anomaly) - we * dt;

	/*
	  Store old orbital elements
	*/
	eorbit = orbit;

	/*
	  Count the perigee orbits
	*/
	porbit = (long)(adj_mean_anomaly / PIE2);

	/*
	  Count the equ. orbits
	*/
	orbit = (long)((adj_mean_anomaly + adj_arg_per) / PIE2);
	eorbit += orbit;
	thetas0 = model.arg_per + model.mean_anomaly;
	if (thetas0 > PIE2)
	    eorbit--;

	/*
	  Make sure mean anomaly is less than PIE2.  Make sure the mean
	  anomaly is not less than zero.
	*/
	adj_mean_anomaly = (adj_mean_anomaly / PIE2 - porbit) * PIE2;
	if (adj_mean_anomaly < 0.0)
	    adj_mean_anomaly += PIE2;

	/*
	  Convert back to REVS
	*/
	mean_anomaly = (long)((adj_mean_anomaly * 100000000.0) / PIE2);

	/*
	  Make argument of perigee less than PIE2.  Make sure arg. of per.
	  is not less than zero.
	*/
	adj_arg_per = (adj_arg_per / PIE2 - (long)(adj_arg_per / PIE2)) * PIE2;
	if (adj_arg_per < 0.0)
	    adj_arg_per += PIE2;

	/*
	  Convert back to REVS
	*/
	arg_per = (long)((adj_arg_per * 100000000) / PIE2);

	/*
	  Make Long. of Asc. Node less than PIE2.  Make sure arg. of per.
	  is not less than zero.
	*/
	adj_lon_anode = (adj_lon_anode / PIE2 - (long)(adj_lon_anode / PIE2)) *
			PIE2;
	if (adj_lon_anode < 0.0)
	    adj_lon_anode += PIE2;
	        
	/*
	  Convert back to REVS
	*/
	lon_anode = (long)((adj_lon_anode * 100000000) / PIE2);

	/*
	  Create adjusted output line
	*/
	sprintf(outline,"%5s%05ld%08ld%08ld%05ld%08ld%08ld%08ld%08ld%06ld",
		satid,eorbit,mean_anomaly,mean_motion,decay,ecc,
		arg_per,lon_anode,inclin,epoch_date);

        /*
          Write the information to the ephemeris file
	*/
	if (put_navsp(outline,&prt_fd,numsat) != E_SUCC)
	    {
	    fclose(in_fd);
	    fclose(prt_fd);
	    c_clean(&vblock,"TBUS",(const char (*)[CMLEN])prtfile,&one);
	    }

	sprintf(msg,"Processed TBUS for satellite %s",satnam);
	c_errmsg(msg,"tbus-proc",&nonfatal);
	}
    }

/*
  Close the input file and the print file
*/
fclose(in_fd);
fclose(prt_fd);

if (!pcount)
    c_errmsg("No TBUS information found","tbus-none",&nonfatal);

/*
  Write a completion message
*/
c_complt("TBUS",ofiles,&ocount);
return E_SUCC;
}
