/************************************************************************
FUNCTION:	SET_LOCAL

PURPOSE:	Adjust integer date/time ( YYYYJJJ and SSSSSS ) from
		UTC to local time.


PROGRAM HISTORY:
Version Date    Author          Modification
_______ _____   _____________   ___________________
  1.0   10/98   D.Lloyd         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under Unix

PROJECT         ADAPS

$Log: set_local.c,v $
Revision 1.5  2002/08/02 18:03:13  dlloyd
Modified so that altzone is no longer used to determine seconds from UTC
since it is not available on Linux systems.  Added one to tm_yday since
it's 0-based.

Revision 1.4  2002/08/02 14:17:42  dlloyd
Modified the header file to only include headers for its prototypes.
Added necessary include in each .c file.
Moved external declaration from .h to .c file.
Added the Log directive.

*************************************************************************/
#include <time.h>
#include "worgen.h"
#include "util.h"
#include "schedcnflt.h"

int set_local(

    long	*date,		/* IN/OUT: integer date as YYYYJJJ	*/
    long	*ltime		/* IN/OUT: seconds since midnight	*/
)
{
long	i;			/* loop control variable		*/
long	jday;			/* julian day from date parameter	*/
long	year;			/* current year				*/

int     local_hour;		/* local hour for UTC offset calculation*/
int     dst_flag;		/* flag indicating local time is DST	*/

static	long	ch_day = 0;	/* julian day that time zone changes	*/
static	long	ch_flag = FALSE;/* flag: change in time zones		*/
static	long	first = TRUE;	/* flag: first call to function		*/
static	long	gmoffset[2];	/* difference(s) from UTC to local time	*/

struct	tm	*tmstruct;	/* time structure			*/

time_t	t;			/* seconds since beginning of unix time	*/


/* The first time this function is called we need to calculate the
difference between local time and UTC.  Also check ahead for 2 weeks
to see if we change to/from Daylight Savings time and calculate
that time difference as well.
--------------------------------------------------------------------*/
if ( first )
    {
    first = FALSE;

    /* Get the local time hour and set the daylight savings flag
    ------------------------------------------------------------*/
    t = time( NULL );
    tmstruct = localtime( &t );
    dst_flag = tmstruct->tm_isdst;
    local_hour = tmstruct->tm_hour;

    /* Now get UTC time and calculate the offset
    --------------------------------------------*/
    tmstruct = gmtime( &t );
    gmoffset[0] = ( local_hour - tmstruct->tm_hour ) * SEC_PER_HR;

    /* Determine if there will be a change over
    the next two weeks to or from DST.
    -------------------------------------------*/
    for ( i = 0; i < 14; i++ )
        {
	/* Add a day of seconds to t */
	t += SEC_PER_DAY;

	/* Get the time associated with t */
	tmstruct = localtime( & t );
	 /* if there is a change in time zones */
	if ( tmstruct->tm_isdst != dst_flag )
	    {
	    /* set the change flag and save the chageover day */
	    ch_flag = TRUE;
	    ch_day = tmstruct->tm_yday + 1;
	    /* Do the offset calculation */
	    local_hour = tmstruct->tm_hour;
	    tmstruct = gmtime( &t );
	    gmoffset[1] = ( local_hour - tmstruct->tm_hour ) * SEC_PER_HR;
	    break;
	    }
	}  /* END for i */
    }  /* END if first */

/* Extract the year and julian date from parameter date
-------------------------------------------------------*/
year = *date / 1000;
jday = *date % 1000;

/* Apply the appropriate offset to the time
-------------------------------------------*/
if ( ch_flag && ( jday == ch_day ))
    *ltime += gmoffset[1];
else
    *ltime += gmoffset[0];

/* Correct the time for spanning day/month/year
-----------------------------------------------*/
if ( *ltime <= 0 )
    {
    *ltime += SEC_PER_DAY;
    jday--;
    if ( jday <= 0 )
        {
	year--;
	if ( c_isleap( &year ))
	    jday = 366;
	else
	    jday = 365;
	}
    }

/* Recombine the year and jday into date
----------------------------------------*/
*date = ( year * 1000 ) + jday;

return E_SUCC;
}
