/******************************************************************************
FUNCTION:	INSERT_REC

PURPOSE:	Inserts a record at the correct position in the schedule
		in memory.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0
  5.1      09/98     D. Lloyd		If duplicate records have status
  					values of None and Pend, change the
					None to Pend.
  5.2      05/01     D.Lloyd		If duplicate records have status
  					values of None and Pend, also change
					the operation code to that of the new
					record.  CR 3121

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "schedit.h"

int insert_rec
(
struct	SCHED	*schrec			/* Record information to add	*/
)
{
extern long	nrecs;			/* Number of master records	*/
extern struct SCHED *master[MAXRECS];	/* Master schedule addresses	*/

struct SCHED	*recptr;		/* Pointer to master schedule	*/

long	rec;				/* Record counter		*/
long	status;				/* Function return status	*/

/*
  See if there is room to add more records
*/
if ((nrecs + 1) > MAXRECS)
    {
    printf("[schedit-maxrec]  The maximum number of master schedule records has been reached\n");
    return(E_FAIL);
    }

/*
  Inserting a new record by moving the last entries down until the
  location for the new entry is found
*/
for (rec = nrecs - 1; rec >= 0; rec--)
    {
    /*
      The new record is to be inserted after the entry having a start time
      less than the start time of the inserted record.
    */
    recptr = *(master + rec);
    if (recptr->lstart < schrec->lstart)
	break;

    /*
      The location for the new record wasn't found yet, so move the current
      entry down and look at the next entry
    */
    *(master + rec + 1) = *(master + rec);
    }

/*
  The correct position for the new record has been found.  Make sure the new
  record does not have a duplicate start time. Otherwise insert the record.
  Just because the start time is the same doesn't mean it's the same record.
  Scenes scheduled from the polar recorder schedule may have the same start
  time although the start time for these is the time we expect them to be
  downloaded from the satellite.  So if the scene ID is different don't count
  it as a duplicate.
*/
recptr = *(master + rec + 2);
if ((rec < nrecs-1) && (recptr->lstart == schrec->lstart) &&
     (strcmp( recptr->scene_id, schrec->scene_id ) == 0 ))
    {
    /* If the new record has a status of Pend and the old record has
    a status of None we will update the old status to Pend.
    CR 3121 Also change the operation code to that of the new record
    -----------------------------------------------------------------*/
    if ( strcmp( recptr->stat, "N" ) == 0 &&
         strcmp( schrec->stat, "P" ) == 0 )
	{
	recptr->stat[0] = 'P';
	recptr->opmode = schrec->opmode;
	printf("[schedit-dupch] Duplicate record changed from None to Pend\n");
	}
    else
	printf("[schedit-duprec] Duplicate record found, %s not entered\n",
	    schrec->scene_id);

    /*
      Duplicate record was found.  Have to move the records that were moved
      down to make room for the new record back up.
    */
    for ( ; rec < nrecs; rec++)
	*(master + rec + 1) = *(master + rec + 2);
    }
else
    {
    /*
      Set the Day/Night flag
    */
    status = setdn(schrec->satid,schrec->rstart,schrec->rstop,schrec->dnflag,
		   schrec->atype);
    if (status == E_FAIL)
	{
	printf("[schedit-dayn] Error occurred setting day/night indicator for %d acquisition\n",schrec->rstart);
	printf("[schedit-dayn] Setting indicator to DAY\n");
	strcpy(schrec->dnflag,"D");
	}

    /*
      Insert new record into the master
    */
    *(master + rec + 1) = schrec;

    /*
      Increment the number of master records
    */
    nrecs++;
    }

return(E_SUCC);
}
