/******************************************************************************
FUNCTION:	SCHEDIT

PURPOSE:	Edit the AVHRR pass acquisition schedule.  Add, modify, delete,
		and list records from the master acquisition schedule.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren	Conversion to LAS-5.0
  5.1      07/92     T. Rockvam		Conversion to LAS-5.0;
					deletes the input file if schedule is
					successfully updated

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
			None.

PROJECT			ADAPS/DOMSAT

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "worgen.h"
#include "schedit.h"

long	nrecs;			/* Number of master records		*/
struct SCHED *master[MAXRECS];	/* Master schedule addresses		*/

void main 
(
long	argc,			/* Number of input arguments		*/
char	*argv[]			/* Argument list		   	*/
)
{
struct SCHED	*addrec,	/* Pointer to record being added	*/
		currec;		/* record being processed         	*/

FILE    *mfp;                  	/* File pointer for master schedule file*/
FILE	*fp;			/* File pointer to the temp update file	*/

long	afile,			/* Flag for acquisition input file	*/
	orignrecs,		/* Original number of records in schedule*/
	exstat = 0,		/* Program exit status			*/
	sform,			/* Report long or short format list	*/
	scode,			/* Return status code			*/
	status = 0;		/* Status flag				*/

char	filename[100],		/* File name of temp schedule		*/
	operation[5],		/* Operation to be done in schedule	*/
	station_key[2];		/* Stations to update for MODBSTA subcmd*/

#ifdef vms
/*
  On VMS systems, make sure the first message starts on a new line
*/
printf("\n");
#endif

/*
  Check the number of arguments
*/
if (argc < 3)
    {
    fprintf(stderr,"%s\n",USAGE_ERROR);
    fprintf(stderr,"[schedit-argc] Invalid number of arguments\n");
    fprintf(stderr,"[schedit-fatal] Fatal error encountered\n");
    exit(1);
    }

/*
  Get the operation to be done
*/
strcpy(operation,argv[1]);
strcpy(filename,argv[2]);

/*
  Load the master schedule into memory
  Keep the master schedule open so no one else can modify it while we work
*/
scode = get_sched(&mfp,operation);
if (scode != E_SUCC)
    strcpy(operation,"-er");
orignrecs = nrecs;

/*
  Open the acquisition input file which contains information used to add,
  modify or delete from the master schedule
*/
if ((strcmp(operation,"-rb") == 0) || (strcmp(operation,"-rl") == 0) ||
    (strcmp(operation,"-er") == 0))
    afile = 0;
else 
    {
    if ((strlen(filename) > 0) && (strcmp(filename,"--") != 0))
	{
	fp = fopen(filename,"r");
	if (fp == NULL)
	    {
	    printf("[schedit-aopen] Error opening acquisition input file\n");
	    strcpy(operation,"-er");
	    }
	else
	    afile = 1;
	}
    else
	fp = stdin;
    }

/*
  Add, Delete, Modify or List the schedule
*/
if (strcmp(operation,"-af") == 0)		/* Add from a file    */
    {
    addrec = (struct SCHED *) malloc(sizeof(struct SCHED));

    while (status != EOF)
	{
	status = fscanf(fp,INSERT_FORMAT,addrec->scene_id, addrec->satid,
			&addrec->lstart, &addrec->lstop, &addrec->rstart,
			&addrec->rstop, &addrec->opmode, addrec->stat,
			&addrec->tfile, addrec->pflag, addrec->station,
			addrec->atype,	addrec->orbit);

	if (status != EOF)
	    {
	    scode = insert_rec(addrec);
	    if (scode != E_SUCC)
		{
		printf("[schedit-fatal] Fatal error encountered\n");
		if (afile)
		    fclose(fp);
		afile = 0;
		exstat = 1;
		break;
		}

	    /*
	      Allocate space for another record
	    */
	    addrec = (struct SCHED *) malloc(sizeof(struct SCHED));
	    }
	}
    printf("[informational] %d records were added\n",nrecs - orignrecs);
    }
else if (strcmp(operation,"-df") == 0)		/* Delete from a file */
    {
    while (status != EOF)
	{
	status = fscanf(fp,DELETE_FORMAT,&currec.lstart,&currec.lstop);
	if (status != EOF)
	    {
	    delete_rec(&currec);
	    }
	}
    }
else if (strcmp(operation,"-ds") == 0)		/* Delete by scene-id */
    {
    while (status != EOF)
	{
	status = fscanf(fp,DELSID_FORMAT,currec.scene_id);
	if (status != EOF)
	    {
	    delete_sid(&currec);
	    }
	}
    }
else if (strcmp(operation,"-mf") == 0)		/* Modify from file   */
    {
    while (status != EOF)
	{
	status = fscanf(fp, MODIFY_FORMAT, &currec.lstart, &currec.lstop,
			currec.satid, &currec.opmode, currec.stat, 
			&currec.tfile, currec.pflag, currec.station,
			currec.atype,currec.orbit);

	if (status != EOF)
	    {
	    modify_rec(&currec);
	    }
	}
    }
else if (strcmp(operation,"-ma") == 0)		/* Modify station   */
    {
    while (status != EOF)
	{
	status = fscanf(fp, MODSTA_FORMAT, &currec.lstart, &currec.lstop,
			station_key, currec.satid, &currec.opmode, currec.stat, 
			&currec.tfile, currec.pflag, currec.station,
			currec.atype,currec.orbit);

	if (status != EOF)
	    {
	    modify_sta(&currec,station_key);
	    }
	}
    }
else if (strcmp(operation,"-ms") == 0)		/* Modify by scene-id */
    {
    while (status != EOF)
	{
	status = fscanf(fp,MODSID_FORMAT, currec.scene_id, currec.satid,
			&currec.opmode, currec.stat, &currec.tfile, 
			currec.pflag, currec.station, currec.atype, 
			currec.orbit);

	if (status != EOF)
	    {
	    modify_sid(&currec);
	    }
	}
    }
else if ((strcmp(operation,"-rl") == 0) || (strcmp(operation,"-rb") == 0))
    {
    if (operation[2] == 'l')
	sform = FALSE;
    else
	sform = TRUE;

    scode = list_sched(filename,sform);	/* List the schedule      */
    if (scode != E_SUCC)
	{
	printf("[schedit-fatal] Fatal error encountered\n");
	if (afile)
	    fclose(fp);
	afile = 0;
	exstat = 1;
	}
    }
else if (strcmp(operation,"-er") == 0)		/* Fatal error	      */
    {
    printf("[schedit-fatal] Fatal error encountered\n");
    if (afile)
	fclose(fp);
    afile = 0;
    exstat = 1;
    }
else						/* Error in request   */
    {
    fprintf(stderr,"%s\n",USAGE_ERROR);
    }

/*
  Update the master schedule and close the update information file
*/
if (afile)
    {
    scode = put_sched(mfp);
    if (scode != E_SUCC)
	{
	printf("[schedit-noup] Master schedule was not updated\n");
	printf("[schedit-fatal] Fatal error encountered\n");
        exit(1);
	}
    fclose(fp);
    /*
      Delete the input file
    */
    c_delfil(filename);
    }
else
    {
    fclose(mfp);
    }

exit(exstat);
}
