/******************************************************************************
FUNCTION:	SETDN

PURPOSE:	Set the day/night flag to the appropriate value based on
		sun zenith angle

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0
  5.1      07/92     T. Rockvam         Use new model, sun zenith angle to
					figure day/night

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "adapsutil.h"
#include "schedit.h"

int setdn
(
char    *satid,			/* Satellite id (e.g. N11)              */
long    start, 			/* Start time in total seconds          */
long    stop, 			/* Stop time in total seconds           */
char    *dnflag,		/* Day/Night flag                       */
char    *atype			/* Acquisition data type                */
)
{
struct	MODEL	model;		/* model information structure		*/

char	satname[8],		/* full satellite name	       (NOAA-11)*/
	satnum[6],		/* satellite code number       (19531)  */
	satcode[2];		/* satellite code letter       (I)	*/

long	syear,			/* start of pass year			*/
	year2,			/* 2-digit year				*/
	sjday,			/* start of pass julian day		*/
	smonth,			/* start of pass month			*/
	sday,			/* start of pass day			*/
	shour,			/* start of pass hour			*/
	smin,			/* start of pass minute			*/
	ssec,			/* start of pass seconds		*/
	sfsec = 0;		/* start of pass fractional seconds	*/

long	satno,			/* integer number of satellite (11)	*/
	dtype,			/* data type code			*/
	nlines,			/* number of lines in scene		*/
	status;			/* function return status		*/

float	midline,midsamp;	/* line,sample at middle nadir of the pass*/

double	zero = 0.0,		/* latitude at equatorial crossing	*/
	lat,lng;		/* lat,long at middle nadir of the pass */
double	view_angles[3];		/* sat & sun zenith, rel azimuth	*/

/*
  Get the satellite number
*/
c_getsat("SATNO",&satno,satname,satid,satnum,satcode);

/*
  Start is total seconds from 1986
  Convert to year, month, day, time
*/
consec(&start,&syear,&sjday,&smonth,&sday,&shour,&smin,&ssec);
year2 = syear % 100;

/*
  Set up the data type and line,sample 
  coordinates at middle nadir of the pass
*/
if (strcmp(atype,"HRPT") == 0)
    {
    dtype = HRPT;
    midline = (float)(stop - start) / 2.0 * LAC_PER_SEC;
    midsamp = LAC_NADIR;
    nlines = (long)((stop - start) * LAC_PER_SEC);
    }
else if (strcmp(atype,"LAC_") == 0)
    {
    dtype = LAC;
    midline = (float)(stop - start) / 2.0 * LAC_PER_SEC;
    midsamp = LAC_NADIR;
    nlines = (long)((stop - start) * LAC_PER_SEC);
    }
else
    {
    dtype = GAC;
    midline = (float)(stop - start) / 2.0 * GAC_PER_SEC;
    midsamp = GAC_NADIR;
    nlines = (long)((stop - start) * GAC_PER_SEC);
    }

/* 
  Get orbital parameters to find sun zenith angle
*/
model.ehour = model.emin = model.esec = model.efrac = 0;
status = c_initmod(&satno,&year2,&smonth,&sday,&shour,&smin,&ssec,
    		    &sfsec,&dtype,&nlines,&model);
if (status != E_SUCC)
    return(E_FAIL);

/*
  Calculate lat/long at middle of scene
*/
c_ls2ll(&midline,&midsamp,&zero,&model,&lat,&lng,view_angles);

/* 
  Check if day or night pass, depending on view_angles
*/
if (view_angles[1] >= HALFPIE)
    strcpy(dnflag,"N");
else
    strcpy(dnflag,"D");

return(E_SUCC);
}
