/******************************************************************************
FUNCTION:	WRITE_LINE

PURPOSE:	Writes a single schedule record to a report file.  The total
		seconds since 1986 is converted to the actual month, day, year,
		and time of acquisition. 

		It also breaks the report into pages.  Each page starts with
		a header.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0
  5.1      07/92     T. Rockvam         Split into separate file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "schedit.h"

void write_line
(
FILE    *report,		/* Report file pointer                  */
long    *line,			/* Line number counter                  */
struct SCHED *schrec,		/* Pointer to schedule record           */
long    *loffset,		/* Total seconds from local to GMT      */
long    *otype			/* Header type (tape or other)          */
)
{
long	tyear,			/* Year calculated from total seconds	*/
	tjday,			/* Julian day calculated from total seconds*/
	tmonth,			/* Month calculated from total seconds	*/
	tday,			/* Day calculated from total seconds	*/
	thour,			/* Hours calculated from total seconds	*/
	tmin,			/* Minutes calculated from total seconds*/
	tsec,			/* Seconds calculated from total seconds*/
	local;			/* Local time in seconds from 1986	*/

char	sdate[9],		/* Formatted start date			*/
	stime[9],		/* Formatted start time			*/
	etime[9],		/* Formatted stop time			*/
	ldate[9],		/* Formatted local date			*/
	ltime[9],		/* Formatted local time			*/
	station[10],		/* Formatted station id			*/
	dtype[5],		/* Formatted data type			*/
	sflag[5],		/* Formatted status flag		*/
	tfile[5];		/* Formatted Tape file number		*/

/*
  New page.  Write the header information
*/
if (*line == 0)
    write_head(report,line,otype);

/*
  Convert the stop time in total seconds from 1986 to a reportable format
*/
consec(&schrec->lstop,&tyear,&tjday,&tmonth,&tday,&thour,&tmin,&tsec);
sprintf(etime,"%02d:%02d:%02d",thour,tmin,tsec);

/*
  Convert the start time in total seconds from 1986 to a reportable format
*/
consec(&schrec->lstart,&tyear,&tjday,&tmonth,&tday,&thour,&tmin,&tsec);
sprintf(sdate,"%02d/%02d/%02d",tmonth,tday,tyear%100);
sprintf(stime,"%02d:%02d:%02d",thour,tmin,tsec);

/*
  Find the local time in total seconds from 1986 and convert to a reportable
  format.
*/
local = schrec->lstart - *loffset;
consec(&local,&tyear,&tjday,&tmonth,&tday,&thour,&tmin,&tsec);
sprintf(ldate,"%02d/%02d/%02d",tmonth,tday,tyear%100);
sprintf(ltime,"%02d:%02d:%02d",thour,tmin,tsec);

/*
  Define the full station location name
*/
if (*schrec->station == 'F')
    strcpy(station,"Fairbanks");
/*  strcpy(station,"Gilmore"); */
else if (*schrec->station == 'W')
    strcpy(station,"Wallops  ");
else
    strcpy(station,"Eros     ");

/*
  Skip the file number if image was not on tape (99)
*/
if (schrec->tfile == 99)
    strcpy(tfile,"    ");
else
    sprintf(tfile," %02d ",schrec->tfile);

strcpy(dtype,schrec->atype);
if (dtype[3] == '_')
    dtype[3] = ' ';

if (schrec->stat[0] == BOTH)
    strcpy(sflag,"Both");
else if (schrec->stat[0] == DISK)
    strcpy(sflag,"Disk");
else if (schrec->stat[0] == TAPE)
    strcpy(sflag,"Tape");
else if (schrec->stat[0] == PENDING)
    strcpy(sflag,"Pend");
else if (schrec->stat[0] == MISSED)
    strcpy(sflag,"Miss");
else
    strcpy(sflag,"None");

/*
  Write the line to the report file
*/
fprintf(report,LINE_FORMAT,TAB,schrec->scene_id,schrec->satid,schrec->orbit,
	ldate,ltime,sdate,stime,etime,station,dtype,schrec->opmode,sflag,
	tfile,schrec->pflag,schrec->dnflag);

/*
  Increment the line number
*/
(*line)++;

/*
  Check to determine if this should be the last line on the page
*/
if (*line > PAGE_SIZE)
    {
    write_foot(report,line);
    fprintf(report,"\f");
    *line = 0;
    }

return;
}
