/****************************************************************************** 
NAME: SCRUNCH

PURPOSE: To reduce an image in size by iteratively skipping lines
         and/or samples.  The reduced image is then used by the facsimile
         recorder or some other printer.

PROGRAM HISTORY:
Version		Date	Author		Code/Conr	Request
-------		----	------		--------	-------
  1.0		5/92	T. Baltzer	   CSB     	Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        SCRUNCH must be run under TAE.

PROJECT: LAS		

ALGORITHM REFERENCES:	none

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "scrunch.h"

/* The following variables must be declared
   global for image I/O routines
-------------------------------------------*/
int fdesc1[MAXIMG+1];           /* Image file descriptors used for image I/O */
int fdesc2[2];                  /* Output image file descriptors */
long offset1[MAXIMG*MAXBND];    /* Offsets into "thebuf" for input bands */


void main
(
  void
)
{
  struct PARBLK vblock;     /* TAE parameter block */

  long kernl = 1;           /* Kernel line to be used by SYSSAMP */
  long kerns = 1;           /* Kernel sample to be used by SYSSAMP */
  long maxnl;               /* Maximum number of lines in output image */
  long maxns;               /* Maximum num of samples in output image */
  long nl;                  /* Number of lines in input image */
  long ns;                  /* Number of samples in input image */
  long nlout;               /* Number of lines in output image	*/
  long nsout;               /* Number of samples in output image */
  long skipl;               /* Skip line factor */
  long skips;               /* Skip sample factor */
  long status;              /* Function return status */
  long std_out;             /* Standard output */
  long window[MAXIMG][4];   /* Window specification */
  long zero = 0;            /* Constant value 0 */

  char errbuf[ERRLEN];      /* Error text buffer */


  /* Initialize TAE 
  -----------------*/
  c_inlas(&vblock,&std_out);

  /* Get user specified parameters from
     TAE and checks window validity 
  -------------------------------------*/
  status = getpar(&vblock,window,&maxnl,&maxns);
  if (status != E_SUCC)
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");

  /* Calculate the skip factor in the lines direction, basis:

  IF HALF OR MORE OF THE LINES ARE TO BE RETAINED:
     original lines less skipped lines <= maxnl
     nl - (nl / skipl) <= maxnl
     skipl <= nl / (nl - maxnl)   (let it truncate)
  IF MORE THAN HALF OF THE LINES ARE TO BE DISCARDED:
     lines retained <= maxnl
     ((nl - 1) / skipl) + 1 <= maxnl
     skipl >= (nl - 1) / (maxnl - 1)

  ---(likewise in the samples direction)---

  examples:
  if kernl = 1, skipl = 1   simple copy with no lines skipped
  if kernl = 1, skipl = 5   keep 1 line, then skip four
  if kernl = 4, skipl = 5   keep 4 lines, then skip one
  -------------------------------------------------------------*/
  ns = window[0][NS];
  nl = window[0][NL];

  if (nl <= maxnl)
  {
    skipl = 1;
    sprintf(errbuf,"None of the lines in the input image will be skipped");
    nlout = nl;
  }
  else
  {
    skipl = nl / (nl - maxnl);
    if (skipl < 2)
    {
      skipl = (nl - 1.0) / (maxnl - 1.0);
      nlout = (((nl - kernl) / skipl) + 1) * kernl;
      if (nlout > maxnl)
      {
        skipl++;
        nlout = (((nl - kernl) / skipl) + 1) * kernl;
      }
      sprintf(errbuf,
              "One in every %d lines of the input image will be retained",
              skipl);
    }
    else
    {
      kernl = skipl - 1;
      nlout = (((nl - kernl) / skipl) + 1) * kernl;
      sprintf(errbuf,"One in every %d lines of the input image will be skipped",
              skipl);
    }
  }

  c_lasmsg(FALSE," ",errbuf);

  if (ns <= maxns)
  {
    skips = 1;
    sprintf(errbuf,"None of the samples in the input image will be skipped");
    nsout = ns;
  }
  else
  {
    skips = ns / (ns - maxns);
    if (skips < 2)
    {
      skips = (ns - 1.0) / (maxns - 1.0);
      nsout = (((ns - kerns) / skips) + 1) * kerns;
      if (nsout > maxns)
      {
        skips++;
        nsout = (((ns - kerns) / skips) + 1) * kerns;
      }
      sprintf(errbuf,
              "One in every %d samples of the input image will be retained",
              skips);
    }
    else
    {
      kerns = skips - 1;
      nsout = (((ns - kerns) / skips) + 1) * kerns;
      sprintf(errbuf,
              "One in every %d samples of the input image will be skipped",
              skips);
    }
  }

  c_lasmsg(FALSE," ",errbuf);

  /* Send the kernel line, sample back to TAE land
     send the skip line, sample back to TAE land
  ------------------------------------------------*/
  if (q_intg(&vblock, "SKIPL", 1, &skipl, P_UPDATE) != SUCCESS)
  {
    c_lasmsg(FALSE,"scrunch-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");
  }
  if (q_intg(&vblock, "SKIPS", 1, &skips, P_UPDATE) != SUCCESS)
  {
    c_lasmsg(FALSE,"scrunch-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");
  }
  if (q_intg(&vblock, "KERNL", 1, &kernl, P_UPDATE) != SUCCESS)
  {
    c_lasmsg(FALSE,"scrunch-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");
  }
  if (q_intg(&vblock, "KERNS", 1, &kerns, P_UPDATE) != SUCCESS)
  {
    c_lasmsg(FALSE,"scrunch-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");
  }

  /* Send the number of lines and samples
     in the output image back to TAE land
  ---------------------------------------*/
  if (q_intg(&vblock, "NL", 1, &nlout, P_UPDATE) != SUCCESS)
  {
    c_lasmsg(FALSE,"scrunch-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");
  }
  if (q_intg(&vblock, "NS", 1, &nsout, P_UPDATE) != SUCCESS)
  {
    c_lasmsg(FALSE,"scrunch-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");
  }
  if (q_out(&vblock) != SUCCESS)
  {
    c_lasmsg(FALSE,"scrunch-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"scrunch-fatal","Fatal error encountered");
  }

  /* Output completion message
  ----------------------------*/
  c_complt("SCRUNCH","",&zero);
}
