/*******************************************************************************
NAME:		SETANNOT

PURPOSE:	Build a standard sized and formatted annotation block for use 
                in facsimile prints.  The two-line block may contain a
                maximum of 97 characters.  All fields are compressed.
		The format is:

	Line 1: [ 1:19]  sceneid
		[22:34]  acquisition date
		[37:49]  equatorial crossing point (longitude Deg and Min)
		[52:62]  orbit number
		[64:67]  data type (LAC, HRPT, GAC)
		[70:73]  direction indicator (Ascending, Descending)
		[75:80]  day/night indicator (Day, Night)
		[83:89]  browse timestamp (HHMMSS) 

     	Line 2: [ 1:29]  upper left coordinates (Lat/Long in Deg, Min, Sec)
		[32:60]  lower right coordinates (Lat/Long in Deg, Min, Sec)
		[63:68]  pass duration (Min)
		[71:80]  bands in the image
		[83:88]  LUT number
		[90:93]  band processed (Day - Band 2, Night - Band 4)

PROGRAM HISTORY

Version  Programmer       Date   Modification
-------  ---------------  -----  -----------------------------------------------
  1.0    J.LaVergne       12/88  Original development
  5.0    E.Cihak          07/91  Conversion to LAS--5.0
  5.1    J.Fenno          09/92  Changed the orbit field in ther archive header
				 structure to sorbit.

PROJECT:	ADAPS

ALGORITHM REFERENCES:	None
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"

main()
{
struct PARBLK vblock;		/* TAE parameter block			     */
struct MODEL  addr_model;	/* AVHRR model information		     */

char	adflag[5],		/* Direction of pass			     */
	annot1[86], 		/* First annotation line		     */
	annot2[86],		/* Second annotation line 		     */
	bnd[2],			/* Band processed(day-2, night-4) 	     */
	chans[6],		/* Bands in the image		 	     */
	dnflag[6],		/* Day/night indicator			     */
	dtype[5],		/* Data type				     */
	lutnum[3],		/* LUT number				     */
	out[1][CMLEN],		/* List of output files created		     */
	sceneid[17],	 	/* Sceneid number			     */
	tstamp[7],		/* Time that the browse was created	     */
	*vec[1];           	/* Array of pointers for q_string	     */
 
long	degs, mins, secs,	/* Coordinate degrees, minutes, and seconds  */
	passdur,		/* Pass duration			     */
	std_out,		/* Logical unit number			     */
	zero = 0,		/* Number of output files		     */
	fatal = -1,		/* Fatal error code			     */
	nonfatal = 1,		/* Nonfatal error code			     */
	status,			/* Status return variable		     */
	len,			/* Length of the scene ID		     */
	i;			/* Loping counter			     */

double  temp;			/* Temporary to convert coordinates	     */

/* Initialize LAS
-----------------*/ 
c_inlas(&vblock,&std_out);

/* Get the parameters from the variable block
---------------------------------------------*/
getpar(&vblock,&addr_model,lutnum,tstamp);

/* Build the sceneid
--------------------*/
c_getscene(&addr_model,sceneid);
len = strlen(sceneid);
c_low2up(sceneid,&len);

/* Convert equitorial crossing to degrees, minutes, seconds
-----------------------------------------------------------*/
temp = addr_model.eqcrs * DEG_PER_RAD;
c_cvtdms(&temp,&degs,&mins,&secs);

/* Create the first annotation line
-----------------------------------*/
/*
  DLLOYD Y2K insure year is only 2-digit
-----------------------------------------*/
sprintf(annot1,"ID:%16s Date:%02d/%02d/%02d EqCr:%4dD%02dM Orbit:%05d ",
        sceneid,addr_model.month,addr_model.day,addr_model.year%100,degs,mins,
	addr_model.sorbit);   

/*
  Get the type of data
----------------------*/
if (addr_model.dtype == GAC)
    strcpy(dtype,"GAC ");
else if (addr_model.dtype == LAC)
    strcpy(dtype,"LAC ");
else
    strcpy(dtype,"HRPT");

/*
  Get the direction of the pass
-------------------------------*/
if (addr_model.spass == ASCENDING)
    strcpy(adflag,"Asc ");
else
    strcpy(adflag,"Desc");

/*
  Get the day/night flag
------------------------*/
if (addr_model.day_night == DAY)
    {
    strcpy(dnflag,"Day  ");
    strcpy(bnd,"2");
    }
else
    {
    strcpy(dnflag,"Night");
    strcpy(bnd,"4");
    }


/* Continuation of the first annotation line
--------------------------------------------*/
sprintf(&annot1[60],"%-4s %-4s %-5s %6s",
        dtype,adflag,dnflag,tstamp); 

/* Create the second annotation line
------------------------------------*/

/*
  Add the upper left latitude coordinate
----------------------------------------*/
temp = addr_model.nw[0] * DEG_PER_RAD;
c_cvtdms(&temp,&degs,&mins,&secs);
sprintf(annot2,"UpLft:%3dD%02dM%02dS ",degs,mins,secs);

/*
  Add the upper left longitude coordinate
------------------------------------------*/
temp = addr_model.nw[1] * DEG_PER_RAD;
c_cvtdms(&temp,&degs,&mins,&secs);
sprintf(&annot2[17],"%4dD%02dM%02dS ",degs,mins,secs);

/*
  Add the lower right latitude coordinate
------------------------------------------*/
temp = addr_model.se[0] * DEG_PER_RAD;
c_cvtdms(&temp,&degs,&mins,&secs);
sprintf(&annot2[29],"LoRgt:%3dD%02dM%02dS ",degs,mins,secs);

/*
  Add the lower right longitude coordinate
------------------------------------------*/
temp = addr_model.se[1] * DEG_PER_RAD;
c_cvtdms(&temp,&degs,&mins,&secs);
sprintf(&annot2[46],"%4dD%02dM%02dS ",degs,mins,secs);

/*
  Calculate the pass duration in minutes
----------------------------------------*/
if (addr_model.emin < addr_model.smin)
    passdur = (60 + addr_model.emin) - addr_model.smin;
else
    passdur = addr_model.emin - addr_model.smin;

/*
  Get the band numbers in the image
-----------------------------------*/
for (i = 0; i < NCHANS; i++)           
    sprintf(&chans[i],"%d",addr_model.chans[i]);

/*
  Continuation of annotation string two
---------------------------------------*/
/*
sprintf(&annot2[58],"Dur:%-2d Bnds:%-5s LUT:%-2s Bnd:%s",
	passdur, chans, lutnum, bnd);
*/

sprintf(&annot2[58],"Dur:%-2d Bnds:%-5s LUT:%-2s B%s",
	passdur, chans, lutnum, bnd);

/*
  Write the Set the annot1 and annot2 text in the variable block
------------------------------------------------------*/
vec[0] = annot1;
status = q_string(&vblock,"ANNOT1",1,vec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error placing the annotation in the parameter block",
	     "setannot-annot",&nonfatal);
    c_errmsg("Fatal error encountered","setannot-fatal",&fatal);
    }

vec[0] = annot2;
status = q_string(&vblock,"ANNOT2",1,vec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error placing the annotation in the parameter block",
	     "setannot-annot",&nonfatal);
    c_errmsg("Fatal error encountered","setannot-fatal",&fatal);
    }

status = q_out(&vblock);
if (status != SUCCESS)
    {
    c_errmsg("Error sending the parameter block","setannot-vblock",&nonfatal);
    c_errmsg("Fatal error encountered","setannot-fatal",&fatal);
    }

/*
  Write a completion message 
----------------------------*/
c_complt ("SETANNOT", out, &zero);
}
