/******************************************************************************
FUNCTION:	SUBREGIONS

PURPOSE:	Generate an orbital stitch sub-list from a full list by
		limiting the sub-list to specified regions.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       02/94    T. Baltzer		initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES

The 12 regions are defined by the GOODES projection as follows (degrees):

region number   min latitude   max latitude   min longitude   max longitude
1		40.73333	90.		-180.		-40.
2		40.73333	90.		-40.		180.
3		0. 		40.73333	-180.		-40.
4		0.		40.73333	-40.		180.
5		-40.73333	0.		-180.		-100.
6		-40.73333	0.		-100.		-20.
7		-40.73333	0.		-20.		80.
8		-40.73333	0.		80.		180.
9		-90.		-40.73333	-180.		-100.
10		-90.		-40.73333	-100.		-20.
11		-90.		-40.73333	-20.		80.
12		-90.		-40.73333	80.		180.

*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include "las.h"		/* LAS include			       */
#include "util.h"
#include "adaps.h"		/* ADAPS include		       */
#include "stitch.h"             /* Orbital stitch include file         */


main()
{
struct PARBLK   vblock;		/* TAE parameter block			*/

char infile[CMLEN];		/* Input file name			*/
char linebuf[CMLEN];		/* Buffer containing line of data	*/
char orbbuf[MAX_PASS][CMLEN];	/* Half orbit line buffer		*/
char outfile[CMLEN];		/* Output file name			*/
char msg[CMLEN];		/* Error message string			*/

FILE *infp;			/* Input file pointer			*/

struct ONELN currline;		/* Structure with info from current line*/
struct ONELN prevline;		/* Struct with info from previous line	*/

long break_flg = FALSE;		/* Break in half orbit (TRUE or FALSE)	*/
long count = 0;			/* File counter				*/
long fatal = -1;		/* Fatal error status			*/
long i = 0;			/* Counter				*/
long orbflg;			/* Flag to print single orbits or not	*/
long status;			/* Return status			*/
long std_out;			/* standard output device 		*/
long usrreg[12];		/* User specified regions		*/


/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);


/* Get the input parameters
----------------------------*/
getpar(&vblock,infile,outfile,usrreg,&orbflg);


/* Get the line of header data
-------------------------------*/
readheader(infile,linebuf,&infp);


/* Write the header to the output file
---------------------------------------*/
status = wrtbuf(outfile,linebuf);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","subregions-fatal",&fatal);


/* Process data in input file.  Gather all of the lines associated
   with a half orbit and then print them to the sub-list if a region
   of interest lies within that half orbit.
---------------------------------------------------------------------*/
status = line_info(infp,linebuf);
if (status == E_SUCC)
   {
   initvals(linebuf,&currline);
   for ( ; ; )
      {
      i = 0;
      strcpy(&orbbuf[i++][0],linebuf);
      for ( ; ; )
         {
	 break_flg = FALSE;
         status = line_info(infp,linebuf);
         if (status != E_SUCC)
            break;
         prevline = currline;
         initvals(linebuf,&currline);
         status = break_orb(orbbuf[i-1],linebuf,&prevline,&currline,&break_flg);
         if (status != E_SUCC)
            c_errmsg("Fatal error encountered","subregions-fatal",&fatal);
         if (break_flg)  break;
         strcpy(&orbbuf[i++][0],linebuf);

	 /*
	   Check for array bounds overflow, signal user and quit
	 --------------------------------------------------------*/
	 if ( i == MAX_PASS )
	    {
	    sprintf(msg,
	       "Too many scenes for half orbit, currently limited to %d",
	       MAX_PASS);
	    c_lasmsg(TRUE,"subregions-numscene",msg);
	    }
         }

      wrthalf(outfile,orbbuf,i,usrreg,orbflg);
      if (!break_flg)  break;
      }

   }

/*
   Close the input file
------------------------
*/
fclose(infp);


count = 1;
c_complt("SUBREGIONS",outfile,&count);
}
