/*******************************************************************************
FUNCTION:	calc_angles	

PURPOSE:	Compute the satellite zenith, solar zenith, and the absolute
		difference of the azimuths for every pixel of each line in the
		image.

		The AVHRR model always returns a positive satellite zenith
		angle.  This angle should be negative for every pixel on a line
		from the first pixel scanned through nadir.  Every pixel
		scanned after nadir must have a positive satellite zenith
		angle.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/93  J.Fenno          Rewrite of the original SUNSAT.
  1.1    03/94  J.Fenno          Modified so the angle values are converted from
				 radians to degrees before interpolation.  Also
				 made relative azimuth angles positive to be
				 compatible with the past versions of SUNSAT.


PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"



FUNCTION calc_angles (ss, es, step, nadir, line, odtype, model, angles)

    long  ss;		/* IN - Starting sample to compute angles for         */
    long  es;		/* IN - Ending sample to compute angles for           */
    long  step;		/* IN - Interval for computing angles                 */
    long  nadir;	/* IN - Nadir pixel                                   */
    long  line;		/* IN - Current line number                           */
    long  odtype;	/* IN - Output data type (image I/O converts to this) */
    struct MODEL *model;/* IN - AVHRR model structure for the input image     */
    double angles[][N_ANGLES];/* OUT - Angles for every pixel on a line       */
{
double elev = 0.0;	/* Set elevation for each pixel to zero               */
double lat;		/* Latitude of a pixel                                */
double lng;		/* Longitude of a pixel                               */

long i;			/* Loop control                                       */
long next;		/* Next pixel to compute the angles for               */
long prev;		/* Previous pixel the angles were computed for        */

float f_line;		/* Floating point value of the current line           */
float f_sample;		/* Floating point value of the current sample         */


/* Compute the angles at the starting sample
--------------------------------------------*/
f_line = (float)line;
f_sample = (float)ss;
c_ls2ll(&f_line, &f_sample, &elev, model, &lat, &lng, angles[ss]);

/* Satellite zenith angles are negative from the first
scanned pixel to nadir and positive after nadir.
Note: Added for clarification 09/14/2001 DEL.  The above
statement is correct as well as the logic below.  Since
the image is not in satellite view, all line sample values
are in image space not satellite space.  The variable
ss below is the sample value in north up oriented image
space ( sample 1 is the leftmost pixel in the image,
sample 2048 the rightmost).
For an ASCENDING pass the image was actually scanned
right to left which is why (ss > nadir) is
correct.  For a DESCENDING pass the image was scanned
from left to right (as viewed north up) which is why
(ss <= nadir) is correct.
------------------------------------------------------*/
if (((model->cpass == ASCENDING)  && (ss > nadir)) ||
    ((model->cpass == DESCENDING) && (ss < nadir)))
    angles[ss][SATZEN] *= -1;


/* Convert the radian values returned by the model to degrees and convert the
   relative azimuth angle from the range of 0-360 into -180-180, then take its
   absolute value.
------------------------------------------------------------------------------*/
for (i = 0; i < N_ANGLES; i++)
    angles[ss][i] *= DEG_PER_RAD;
angles[ss][RELAZ] = fabs(angles[ss][RELAZ] - 180.0);


/* Compute the angles for each pixel from ss to es inclusive, using
   interpolation to compute most values.  This will save time due to
   the extensive computations that are involved in the model.
--------------------------------------------------------------------*/
prev = ss;
next = prev + step;
while (next < es)
    {
    f_sample = (float)next;
    c_ls2ll(&f_line, &f_sample, &elev, model, &lat, &lng, angles[next]);
    /* See note added 09/14/2001 above for clarification
    ----------------------------------------------------*/
    if (((model->cpass == ASCENDING)  && (next > nadir)) ||
	((model->cpass == DESCENDING) && (next < nadir)))
	angles[next][SATZEN] *= -1;


    /* Convert the radian values returned by the model to degrees and convert
       the relative azimuth angle from the range of 0-360 into -180-180, then
       take its absolute value.
    -------------------------------------------------------------------------*/
    for (i = 0; i < N_ANGLES; i++)
	angles[next][i] *= DEG_PER_RAD;
    angles[next][RELAZ] = fabs(angles[next][RELAZ] - 180.0);


    /* Interpolate all angle values from prev to next exclusive
    -----------------------------------------------------------*/
    interpolate(angles, prev, next);
    prev = next;
    next = prev + step;
    }


/* If the starting sample and the ending sample are not the same then compute
   the angles at the ending sample and interpolate the values between prev and
   es.
------------------------------------------------------------------------------*/
if (es > ss)
    {
    f_sample = (float)es;
    c_ls2ll(&f_line, &f_sample, &elev, model, &lat, &lng, angles[es]);
    /* See note added 09/14/2001 above for clarification
    ----------------------------------------------------*/
    if (((model->cpass == ASCENDING)  && (es > nadir)) ||
	((model->cpass == DESCENDING) && (es < nadir)))
	angles[es][SATZEN] *= -1;


    /* Convert the radian values returned by the model to degrees and convert
       the relative azimuth angle from the range of 0-360 into -180-180, then
       take its absolute value.
    -------------------------------------------------------------------------*/
    for (i = 0; i < N_ANGLES; i++)
	angles[es][i] *= DEG_PER_RAD;
    angles[es][RELAZ] = fabs(angles[es][RELAZ] - 180.0);


    interpolate(angles, prev, es);
    }



/* Scale all of the output angle values according to the output data type
-------------------------------------------------------------------------*/
scale_angles(angles, ss, es, odtype);


return(E_SUCC);
}
