/*******************************************************************************
FUNCTION:	compute_angles	

PURPOSE:	Compute the satellite zenith, solar zenith, and the absolute
		difference of the azimuths for every pixel of each line in the
		image.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/93  J.Fenno          Rewrite of the original SUNSAT.


PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"


/* Global variables */
extern long curent;

FUNCTION compute_angles (model, bands, nbands, offset, odtype, window,
			 gdesc, iobuf, npixels, minval, maxval, minmax)

    struct MODEL *model;	/* IN - Model and orbital information         */
    long   bands[N_ANGLES];	/* IN - Bands to be output                    */
    long   nbands;		/* IN - Number of bands to be output          */
    long   offset[N_ANGLES];	/* IN - Pointers for each band                */
    long   odtype;		/* IN - Output data type                      */
    long   window[];		/* IN - Window for the input image            */
    int    *gdesc;		/* IN - Group descriptor for image I/O        */
    float  *iobuf;		/* IN - I/O buffer                            */
    long   npixels;		/* IN - Interval for computing angles         */
    double minval[];		/* IN/OUT - Minimum value for bands           */
    double maxval[];		/* IN/OUT - Maximum value for bands           */
    char   *minmax;		/* IN - Min/max flag                          */
{
double angles[LAC_SAMPS+1][N_ANGLES];/* Array to hold all angles for a line   */

long dtype = EREAL;		/* Data type of the computed angles           */
long es;			/* Ending sample on the line                  */
long i;				/* Loop control                               */
long j;				/* Loop control                               */
long line;			/* Current line number in output image        */
long nadir;			/* Nadir pixel for the image                  */
long sample;			/* Current sample number                      */
long ss;			/* Starting sample on the line                */


/* Determine which pixel is the Nadir pixel for the
   image and compute the starting and ending samples.
-----------------------------------------------------*/
if (model->dtype == GAC)
    nadir = GAC_NADIR;
else
    nadir = LAC_NADIR;
ss = window[SS];
es = window[SS] + window[NS] - 1;


/* For each line of the input image compute the requested angles
----------------------------------------------------------------*/
for (curent = 0, line = window[SL]; curent < window[NL]; curent++, line++)
    {
    /* If the nadir pixel is contained on the line then calculate the angles
       for this line without interpolating across nadir.  If nadir is not
       contained in the line then simply calculate the angles by interpolating
       from the starting sample to the ending sample.
    -------------------------------------------------------------------------*/
    if ((ss < nadir) && (es > nadir))
	{
	calc_angles(ss, nadir-1, npixels, nadir, line, odtype, model, angles);
	calc_angles(nadir, nadir, npixels, nadir, line, odtype, model, angles);
	calc_angles(nadir+1, es, npixels, nadir, line, odtype, model, angles);
	}
    else
	calc_angles(ss, es, npixels, nadir, line, odtype, model, angles);


    /* Copy the angles from the array into the I/O buffer
    -----------------------------------------------------*/
    for (i = 0, sample = ss; i < window[NS]; i++, sample++)
	{
	for (j = 0; j < nbands; j++)
	    iobuf[offset[j] + i] = (float)angles[sample][bands[j] - 1];
	}


    /* Compute the Min/Max values for this line if necessary
    --------------------------------------------------------*/
    if (strcmp(minmax, "YES") == 0)
	{
	for (i = 0; i < nbands; i++)
	    c_pxmnmx(&iobuf[offset[i]], &window[NS], &minval[i],
		&maxval[i], &dtype);
	}


    /* Write the current line to the output image
    ---------------------------------------------*/
    if (c_estep(gdesc) == E_FAIL)
	return(E_FAIL);
    }


return(E_SUCC);
}
