/*******************************************************************************
FUNCTION:	sunsat

PURPOSE:   	SUNSAT Generates zenith view angle bands for an avhrr image.
		The bands that can be created are:

    		   Band 1:  Zenith angle to satellite from each image pixel
    		   Band 2:  Zenith angle to sun from each image pixel
    		   Band 3:  Absolute difference of the azimuths

    		An output image is created containing these new bands.

			 
Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    08/87  D.Hollaren       Originally called viewsunbnd
  2.0    10/87  B.Rieger         Added sataz and getangles to calculate sat az.
  2.1    10/87  B.Rieger         Added the interpolation routine.
  3.0    02/90  C.Wivell         New interpolation.  New angle calc routine.
				 Great circle approx.  New sun position routine.
				 Gac handling.  Byte data type and scaling.
  3.1    03/90  C.Wivell         Add band for abs. diff of azimuths.
  5.0    10/90  D.VanderZee      Re-retrocode (All new C version!)
  5.1    12/91  T.Baltzer        Completed Re-retrocode began by VanderZee.
  5.2    09/93  J.Fenno          Complete rewrite.
  5.3    03/94  J.Fenno          Due to a change in the model, modified so the
				 scaling is the same as the old version.
				

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"



/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
int fdesc[2];			/* image file descriptors used for image I/O  */
int gdesc;			/* group descriptor used for image I/O        */
long bnd_ptr[N_ANGLES];		/* pointer into iobuf for output bands        */


main()
{
char *c_ealloc();		/* function to allocate iobuf                 */
char hosin[1][CMLEN];		/* host names of input images                 */
char hosout[CMLEN];		/* host name of output image                  */
char minmax[4];			/* compute min and max flag                   */
char str[CMLEN];		/* Temporary string                           */
char taeout[CMLEN];		/* TAE name of output image                   */

double ln_inc       = 1;	/* line increment for ddr                     */
double maxval[N_ANGLES];	/* max values for output image                */
double minval[N_ANGLES];	/* min values for output image                */
double smp_inc      = 1;	/* sample increment for ddr                   */

float *iobuf;			/* buffer for image i/o                       */

long access        = IWRITE;	/* write access flag                          */
long bands[1][N_ANGLES];	/* bands to be output                         */
long bufsiz;			/* image i/o buffer size                      */
long cflag         = NOCOMB;	/* flag for updating ddr                      */
long fatal         = -1;	/* fatal error flag                           */
long fltype        = EREAL;	/* float (real) data type                     */
long ml;			/* master line                                */
long ms;			/* master sample                              */
long msg_frmt      = 0;		/* format flag for processing message         */
long msgtim;			/* elapsed time between display msg           */
long nbands;			/* number of bands to be created              */
long nmrimg;			/* number of input images                     */
long nonfatal      = 1;		/* nonfatal error flag                        */
long npixels;			/* interpolated sample interval               */
long odtype;			/* output data type                           */
long one           = 1;		/* number one for passing by address          */
long open_opt[3];		/* optional open parameter                    */
long status;			/* Function return status                     */
long std_out;			/* LUN to standard output                     */
long totbnd;			/* total number of bands                      */
long window[1][4];		/* window for each input image                */
long zero          = 0;		/* number zero for passing by address         */

extern long curent;		/* current processing line                    */

struct PARBLK vblock;		/* TAE parameter block                        */
struct MODEL model;		/* Model and orbital information              */


/* Initialize LAS
-----------------*/
c_inlas(&vblock,&std_out);


/* Get parameters and globals
-----------------------------*/
getpar(&vblock, hosin, taeout, hosout, &nmrimg, window, bands,
    &nbands, &totbnd, &odtype, &npixels, &msgtim, minmax);


/* Initialize min/max arrays
----------------------------- */
if (strcmp(minmax,"YES") == 0)
    {
    if (c_inmnmx(&fltype,&nbands,minval,maxval) != E_SUCC)
	c_errmsg("Fatal error encountered","sunsat-fatal",&fatal);
    }


/* Initialize model and master line/sample 
------------------------------------------*/
init_avhrr(hosin[0], &model, &ml, &ms);


/* We are not actually reading the input image but are just calculating the
   requested angles for each pixel.  In order to calculate the correct angles
   we must adjust the window of the input image to correspond to the actual
   AVHRR image.  Therefore, we need to update the starting line and sample for
   the specified window of the input image.
------------------------------------------------------------------------------*/
window[0][SL] += ml;
window[0][SS] += ms;


/* Open output image file
-------------------------*/
if (odtype == EREAL)
    open_opt[0] = open_opt[1] = open_opt[2] =0;
else
    {
    open_opt[0] = odtype;
    open_opt[1] = open_opt[2] =0;
    }
status = c_eopens(&fdesc[0], hosout, &nbands, &one, &one, &window[0][NL],
    &window[0][NS], bnd_ptr, &access, &fltype, &zero, open_opt);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","sunsat-fatal",&fatal);

 
/* Allocate memory for image I/O, group the output image for sequential
   I/O, and step the image to initialize all of the I/O pointers
-----------------------------------------------------------------------*/
if ((iobuf = (float *)c_ealloc(fdesc,&bufsiz)) == NULL)
    c_clean(&vblock,"sunsat",hosout,&one);
if (c_egroup(fdesc,&gdesc,iobuf,&bufsiz) != E_SUCC)
   c_clean(&vblock,"sunsat",hosout,&one); 
if (c_estep(&gdesc) == E_FAIL)
   c_clean(&vblock,"sunsat",hosout,&one); 


/* Initialize processing message utility 
----------------------------------------*/
curent = 1;
c_inittm("SUNSAT", "Processed", "line", "bands", " ",
    &window[0][NL], &nbands, &msgtim, &msg_frmt);
c_startm();


/* Compute zenith and/or azimuth angles for specified bands 
-----------------------------------------------------------*/
status = compute_angles(&model, bands[0], nbands, bnd_ptr, odtype, window,
    &gdesc, iobuf, npixels, minval, maxval, minmax);
if (status != E_SUCC)
    c_clean(&vblock,"sunsat",hosout,&one);


/* Stop processing messages
---------------------------*/
c_stoptm();


/* Close the output image and free the I/O buffer
-------------------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
    c_clean(&vblock,"sunsat",hosout,&one);
free(iobuf);


/* Update DDR for output image
------------------------------*/
bands[0][0] = 0;
nbands = 0;
status = c_upddr(&vblock, hosin[0], hosout, &nmrimg, window[0],
    bands, &nbands, &ln_inc, &smp_inc, &totbnd, &cflag);
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered","sunsat-warn",&nonfatal);


/* if minmax to be tracked, then store them
-------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
    {
    status = c_svmnmx(hosout,&totbnd,minval,maxval,"YES");
    if (status != E_SUCC)
	c_errmsg("Nonfatal error encountered","sunsat-warn",&nonfatal);
    }

 
/* Update history file 
----------------------*/
status = c_hstout(&vblock, hosout, hosin[0], &nmrimg, " ");
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered","sunsat-warn",&nonfatal);


/* Output completion message 
----------------------------*/
c_complt("SUNSAT",taeout,&one);
}
