/******************************************************************************

FUNCTION:	PROCESS

PURPOSE:	Process the lines of the image.


Version	Date	Programmer	Modifications
-------	-----	--------------	-----------------------------------------------
1.0 	04/98	T. Baltzer	Development
1.1	01/99	D.Lloyd		Modifications for NOAA KLM series sats.


PROJECT:	ADAPS

ALGORITHM REFERENCES:
		NOAA Polar Orbiter Data User's Guide
		    K. Kidwell, NOAA
		    Section 3 AVHRR Level 1B Data Base
		NOAA Technical Memorandum NESS 107
		    Section 5.1 AVHRR Calibration, Appendix A, Appendix B
		NOAA KLM Users Guide, April 1998
		    Goodrum, Kidwell, Winston, NOAA
		    Section 8, Level 1B Data Base

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "level1bio.h"
#include "tapel1b.h"

#define NODEBUG   TRUE		/* Set to FALSE when debugging with dbx	*/

extern	long	curent;		/* current processing line		*/


int process
(
long	*tplun,			/* IN: output tape lun			*/
FILE	*in_fd,			/* IN: input image file descriptor	*/
long	recsiz,			/* IN: Level 1b record size		*/
long	msgtim,			/* IN: time for processing message	*/
struct	MODEL	*model,		/* IN: Model and orbital information	*/
long	max_bytes,		/* IN: max bytes to write to tape	*/
long	*kbyteswritten,		/* IN/OUT: bytes written to tape	*/
long	*tape			/* IN/OUT: tape open flag		*/
)
{

unsigned char l1bbuf[MAXLINSIZ];/* Buffer of Level-1b data to read/write*/

float	fbyteswritten;		/* floating point value of kbyteswritten*/

short *shortptr;		/* Pointer to short, for scan line count*/

long	curscanline;		/* Current scan line count		*/
long	first_line = TRUE;	/* First line in image (flag)		*/
long	flag = ALLCAT;		/* all categories (bands) proc. at once	*/
long	klmflg;			/* flag for KLM series sats		*/
long	nbytes;			/* number of bytes per line		*/
long	prescanline;		/* Previous scan line count		*/
long	record;			/* current record (line) being written	*/
long	scanlinedif;		/* Scan line difference			*/
long	status;			/* general return status		*/


/*
  Set up data type dependant variables
---------------------------------------*/
if ( model->satnum >= 15 )
    klmflg = TRUE;
else
    klmflg = FALSE;
if ( model->dtype == GAC && !klmflg )
    nbytes = recsiz / 2;        /* GAC has 2 lines per record               */
else
    nbytes = recsiz * 2;        /* LAC/HRPT has 2 records per line          */

/*
  The total number of drop lines hasn't been defined in the model, make
  sure it is initialized to zero.
------------------------------------------------------------------------*/
model->totmiss = 0;

/*
  Start processing message
---------------------------*/
#if NODEBUG
curent = 1;
c_inittm("TAPEL1B","wrote","line","bands"," ",&model->nlines,&model->nchans,
	    &msgtim,&flag);
c_startm();
#endif

/*
  Process image lines
------------------------*/
fbyteswritten = (float)*kbyteswritten;
for ( record = 1; record <= (model->nlines - model->totmiss); record++ )
    {

    /* read one Level 1b line */
    status = c_l1breadd( in_fd, &recsiz, l1bbuf );
    if ( status != E_SUCC )
	{
	c_lasmsg(FALSE,"tapel1b-iread","Error reading image line");
	return E_FAIL;
	}

    /*
      Check that we aren't going to write beyond the logical tape end
      indicated by max_bytes.
      ------------------------------------------------------------------*/
    if ( *tplun && ( (fbyteswritten + (nbytes / 1000.0)) > max_bytes ))
	{
	/*
	  I tried to back space to the beginning of the partial file
	  but was unsuccessful.  After using c_tpbsf() of c_tpfsf(),
	  any subsequent tape operations returned tape errors.  If the
	  tape I/O errors ever get fixed the following code may be
	  uncommented so that partial files aren't left on the tape.
	  --------------------------------------------------------------*/
	/* Backspace to the beginning of the previous scene		*/
	/* status = c_tpbsf( tplun, &count, &mode ); */
	/* Forward space to the beginning of the partial scene	*/
	/* This puts us at the end of the last full scene written	*/
	/* status = c_tpfsf( tplun, &count, &mode ); */
	/* c_tpclos( tplun, &mode, &rewopt, &eofopt ); */
	/* *tape = FALSE; */
	/* *tplun = 0; */

	*kbyteswritten = max_bytes + 1;
	return E_SUCC;
	}

    /* write one Level 1b line */
    status = c_l1bwritet( tplun, l1bbuf, &recsiz, tape );
    if ( status != E_SUCC )
	{
	c_lasmsg(FALSE,"tapel1b-bdimagwr","Error writing image line");
	return E_FAIL;
	}

    /* increment bytes written */
    fbyteswritten += (float)nbytes / 1000.0;

    /* Time to check if drop lines occurred here in the image:
       If the scan line count skips lines, we know that drop lines go into
       the image here.  Update the "totmiss" field of the model, so that
       we know exactly how many lines of data to copy.
    -------------------------------------------------------------------*/
    shortptr = (short *)l1bbuf;
    if (first_line)
        {
        prescanline = (long)(*shortptr) - 1;
        first_line = FALSE;
        }
    else
        prescanline = curscanline;
    curscanline = (long)(*shortptr);
    scanlinedif = curscanline - prescanline;
    if (scanlinedif > 1)
	model->totmiss += scanlinedif - 1;

    curent++;
    }

/*
  Return number of bytes written
---------------------------------*/
*kbyteswritten = (long)( fbyteswritten + 0.5 );


/*
  Stop processing message
--------------------------*/
curent = record - 1;
#if NODEBUG
    c_stoptm();
#endif

return E_SUCC;
}
