/*************************************************************************
FUNCTION:	TAPEL1B


PURPOSE:	

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0    04/98	T. Baltzer	Original development (borrowed from MKLEVEL1B)
 1.1	01/99	D.Lloyd		Modifications for NOAA KLM series sats


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
 	NOAA Polar Orbiter Data User's Guide
 	  K. Kidwell, NOAA
 	  Section 2 Level 1b Data Base
 	  Section 3 AVHRR Level 1b Data Base
 	NOAA Technical Memorandum NESS 107
 	  Section 5.1 AVHRR Calibration, Appendix A, Appendix B
        NOAA KLM Users Guide, April 1998
          Goodrum, Kidwell, Winston, NOAA
          Section 8, Level 1B Data Base
*************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapssup.h"
#include "archio.h"
#include "tapeio.h"
#include "model.h"
#include "level1bio.h"
#include "tapel1b.h"

int main( int argc, char **argv )
{
char	acctnum[10];		/* account number			*/
char	infile[CMLEN];		/* input archive image host file name	*/
char	comment[CMLEN];		/* tape mount comment			*/
char	density[5];		/* tape density				*/
char	drive[CMLEN];		/* name of tape drive			*/
char	hosdir[CMLEN];		/* host directory name			*/
char	hosext[CMLEN];		/* host directory name			*/
char	hosroot[CMLEN];		/* host root name			*/
char	ordernum[14];		/* order number				*/
char	out[1][CMLEN];		/* output file names			*/
char	prodesc[18];		/* product description			*/
char	sceneid[CMLEN];		/* Scene id				*/
char	ttype[5];		/* tape type				*/
char	unitnum[5];		/* unit number				*/

FILE	*in_fd;			/* file descriptor for input file	*/

long	count = 1;		/* number of strings in out[]		*/
long	msgtim = 10;		/* time between processing messages	*/
long	mvol;			/* multiple volume flag			*/
long	kbyteswritten;		/* number bytes written to current tape */
long	nvols;			/* number of volumes used for this scene*/
long	recsiz;			/* Level 1b record size			*/
long	status;			/* general return status		*/
long	stdlun;			/* standard output			*/
long	tae;			/* flag; TRUE if called from tae land	*/
long	tape = FALSE;		/* tape open flag			*/
long	tplun = 0;		/* tape logical unit number		*/
long    maxkbytes;              /* maximum Kbytes to write to tape	*/

struct	MODEL	model;		/* Model and orbital information	*/
struct	PARBLK	vblock;		/* parameter block			*/

unsigned char dsh[MAXLINSIZ];	/* Data set header record buffer	*/
unsigned char buf[ARS_HDR_SIZ];	/* TBM/ARS header buffer		*/


/*
  Decide who called this function.  We will assume that if there
  is more than one argument in argv[] the function was called from
  a unix system() call.  Otherwise we assume it was called from
  TAE.  Note: originally the return status from p_inim() was used
  indicating the presense of a vblock, but this proved to be
  unstable when the system call to tapel1b originated in c-code
  that was called from a pdf.
------------------------------------------------------------------*/
if ( argc > 1 )
    tae = FALSE;
else
    tae = TRUE;
if ( tae )
    {
    /* Initialize TAE */
    c_inlas(&vblock, &stdlun );

    /* Get parameters from the variable block */
    getpar( &vblock, infile, comment, acctnum, ordernum, unitnum, prodesc,
	    drive, &mvol, &nvols, density, ttype, &msgtim, 
	    &maxkbytes, &kbyteswritten );
    }
else
    /* Get parameters from argv[] */
    getargv( argc, argv, comment, infile, acctnum, ordernum,
	     unitnum, prodesc, drive, ttype, &nvols, &mvol, density,
	     &maxkbytes );


/*
  Open input image
-------------------*/
status = c_l1bopndr( infile, &in_fd, &recsiz, buf, dsh, &model);
if ( status != E_SUCC )
    {
    strcpy(sceneid,"");
    finish(&vblock, drive, &nvols, tplun, in_fd, &kbyteswritten, sceneid, "",
	   tae);
    c_lasmsg(TRUE,"tapel1b-fatal","Fatal error encountered");
    }


/*
  Determine the 16 char scene ID from the model.  If this scene id less the
  "seconds" designation corresponds to the root name, and the root name is
  23 char's long, assume that the root file name consists of
  "sceneid_timestamp".  Use this 23 char sceneid for the sceneid instead of
  just the 16 char sceneid.
----------------------------------------------------------------------------*/
c_getscene(&model,sceneid);
c_splitname( infile, hosdir, hosroot, hosext );
if ((strlen(hosroot) == 23) && (strncmp(sceneid,hosroot,14) == 0))
    strcpy(sceneid,hosroot);


/*
  Reopen the tape (opened by GETDRIVE) and write headers
---------------------------------------------------------------*/
status = c_l1bopntw( drive, density, &tplun, buf, dsh, &recsiz, &tape );
if ( status != TAPE_SUCC )
    c_lasmsg(TRUE,"tapel1b-fatal","Fatal error encountered");


/*
  Update the number Kbytes written 
-----------------------------------------------------*/
if ( model.satnum >= 15 )
    kbyteswritten += (long)(ARS_HDR_SIZ / 1000.0 + 0.5 );
else
    kbyteswritten += (long)(TBM_HDR_SIZ / 1000.0 + 0.5 );
if ( model.dtype == GAC && model.satnum < 15 )
    kbyteswritten += (long)( recsiz / 1000.0 + 0.5 );
else
    kbyteswritten += (long)((recsiz * 2) / 1000.0 + 0.5 );


/*
  Write image data to tape
---------------------------*/
status = process( &tplun, in_fd, recsiz, msgtim,
		  &model, maxkbytes, &kbyteswritten, &tape );
if ( status != E_SUCC )
    {
    finish(&vblock, drive, &nvols, tplun, in_fd, &kbyteswritten, sceneid, "",
	   tae);
    c_lasmsg(FALSE,"tapel1b-fatal","Fatal error encountered");
    return E_SUCC;
    }
else if ( kbyteswritten > maxkbytes )
    {
    finish(&vblock, drive, &nvols, tplun, in_fd, &kbyteswritten, sceneid, "",
	   tae);
    c_lasmsg(FALSE,"tapel1b-ltend","Logical tape end reached");
    return E_SUCC;
    }


/*
  Create the Level 1b product summary file.  Do not check the return
  status, even if an error generating the summary file occurs, continue
  processing (do not consider this a fatal error).
------------------------------------------------------------------------*/
c_l1bsumm( sceneid, acctnum, ordernum, unitnum, prodesc, density,
	   &model, out[0] );


/*
  Finish with input and output and images
------------------------------------------*/
finish(&vblock, drive, &nvols, tplun, in_fd, &kbyteswritten, sceneid, out[0],
	tae);


/*
  Generate completion message
------------------------------*/
if ( tae )
    c_complt( "TAPEL1B", out, &count );
else
    {
    c_lasmsg(FALSE," ","Program TAPEL1B completed successfully.");
    c_lasmsg(FALSE," ","Created output:");
    c_lasmsg(FALSE," ","   %s",out[0]);
    }

return E_SUCC;
}
