/***************************************************************************
FUNCTION:	TOV_INGEST

PURPOSE:	To create a .img file from a TOVS data file.  The F77 codes
		are stripped from the records and the -999.9 values are
		replaced by -1.0.  The image is flipped when written so it
		is North up.  The image created can be thought of as a bsq
		file.  Each band contains 180 lines by 360 samples of values
		for a single days observation, either AM or PM depending on
		input file.

Version	Date	Programmer	Modification
-------	-----	------------	--------------------------------
  1.0	02/99	D.Lloyd		Initial Development

PROJECT:	ADAPS


***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tovsin.h"


#define	INRECSIZ	64802
#define	ORECSIZ		64800
#define NLINES          180
#define NSAMPS          360

void main( void )
{
char	infile[CMLEN];		/* Input file name			*/
char	outfile[CMLEN];		/* Output file name			*/

FILE	*inf;			/* input file descriptor		*/
FILE	*ouf;			/* output file descriptor		*/

float	*ptr[NLINES];		/* pointers into buf for each image line*/
float	buf[INRECSIZ];		/* I/O buffer				*/

long	bytes_read;		/* num bytes read from input file	*/
long	i;			/* loop variable			*/
long	j;			/* loop variable			*/
long	nbands[1];		/* number of bands converted		*/
long	outcnt = 1;		/* count of output files		*/
long	status;			/* Function return status		*/
long	stdlun;			/* Standard output			*/

struct	PARBLK	vblock;		/* Parameter block			*/


/* Initialize LAS
-----------------*/
c_inlas( &vblock, &stdlun );


/* Get the parameters
---------------------*/
status = getpar( &vblock, infile, outfile );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "tovsin-fatal", "Fatal error occured" );


/* open the files
-----------------*/
inf = fopen( infile, "r" );
if ( !inf )
    c_lasmsg( TRUE, "tovsin-open", "Unable to open: %s", infile );
ouf = fopen( outfile, "w" );
if ( !ouf )
    c_lasmsg( TRUE, "tovsin-open", "Unable to open: %s", outfile );


/* Set up pointers into buffer, point to each line of the image.  The
pointers are set up so that the image will be flipped when written.
The +1 offset in [i*NSAMPS+1] is to bypass the F77 record header.
---------------------------------------------------------------------*/
for ( i = NLINES - 1, j = 0; i >= 0; i--, j++ )
    ptr[j] = &buf[i*NSAMPS+1];


/* for each day of the month
----------------------------*/
nbands[0] = 0;
for ( i = 0; i < 31; i++ )
    {
    /* read a line of data
    ----------------------*/
    bytes_read = fread( buf, sizeof(float), INRECSIZ, inf );

    /* break from the loop at end of file, for months
    less than 31 days or for single day files.
    -------------------------------------------------*/
    if ( bytes_read != INRECSIZ )
        break;
    
    /* for visualization purposes, especially XID,
    convert the -999.9 values to -1.0
    ----------------------------------------------*/
    for ( j = 1; j < INRECSIZ; j++ )
        if ( (long)buf[j] == -999 )
	    buf[j] = -1.0;

    /* write the new image w/o F77 codes
    ------------------------------------*/
    for ( j = 0; j < NLINES; j++ )
        fwrite( ptr[j], sizeof(float), NSAMPS, ouf );
    
    nbands[0]++;  /* count the band */
    }


/* close the files
------------------*/
fclose( inf );
fclose( ouf );


/* Return the number of bands to tae
------------------------------------*/
status = q_intg( &vblock, "NBANDS", 1, &nbands, P_UPDATE );
if ( status != E_SUCC )
    c_lasmsg(FALSE,"tovsin-update","Error updating variable block");
status = q_out( &vblock );
if ( status != E_SUCC )
    c_lasmsg(FALSE,"tovsin-update","Error updating variable block");


/* Display a standard completion message
----------------------------------------*/
c_complt( "TOVSIN", outfile, &outcnt );
} 
