#!/bin/csh
#--------------------------------------------------------------------
# trantovs.csh
#
# The purpose of this shell script is to download tovs data from the
# Goddard ftp site and, once downloaded, unzip the files, rename
# them to a more LAS-friendly name and move them to ADAPSACQ/tovs.
#
# An effort was made to prevent transferring the same file more than
# once.  A list of transferred files is maintained in $ADAPSTABLES
# as tovstranlist.  The format of this file is:
#
#    9999999
#    2000018 daily.pptasurf.b991230.e991230.am.men.nj.eenso97q.gz
#    ...
#    1999365 daily.pptasurf.b991229.e991229.pm.men.nj.eenso97q.gz
#
# where 2000018 is the date of tranfer as 4-digit year and julian date
# and daily.b9...97q.gz is the file name.  The line with 9999999 and a
# blank file name is there in case of ftp errors.  Occasionally a ftp
# session will error and blank lines will end up in the tovs listing.
# Maintaining the blank file name 'forever' in tovstranlist will prevent
# the comm command from identifying a blank line as a file to transfer. 
# To prevent the transfer list from getting too large, it is pruned
# to the latest $1 entries or the default 60 before being saved.
#--------------------------------------------------------------------

#-------------------------------------------------------
# If an argument was supplied set numfiles to that value
# otherwise set it to 60.
#-------------------------------------------------------
if ( $#argv != 1 ) then
    set numfiles = "60"
else
    set numfiles = $1
endif

#-----------------------------------------------------------
# Set directories and logfile
# The logfile needs to be in LAS syntax for wrtlog so noglob
# must be set to prevent the []'s from being interpreted
# as a pattern matching construct.
#-----------------------------------------------------------
set noglob
set logfile = `echo "[$ADAPSLOG]trantovs" | tr / . | sed  's/\[\./\[/'`
alias wrtlog $ADAPSBASE/write_log

#--------------------------------------------------
# Directory where tovsin_daemon looks for new files
#--------------------------------------------------
set tovsacq =  $ADAPSACQ/tovs/

#------------------------------
# Get the current date and time
#------------------------------
set datetime = `date +%b%d%H%M%S`
set juldate = `date +%Y%j`

#-----------------------------------------------------
# The working directory should not be in ADAPSACQ to
# prevent the tovsin_daemon from picking up the files
# before they are in the proper form.
#-----------------------------------------------------
set workdir = $ADAPSPROD/tovs$datetime
if ( ! -e $workdir ) then
    mkdir $workdir
endif
cd $workdir

#------------------------
# Print a startup message
#------------------------
set key = "daily_tovs_file"
set text = "Starting the TRANTOVS procedure"
wrtlog $logfile $key trantovs-start $text

#----------------------------------------------------
# Get a listing of the files at Goddard. Redirect the
# output of the list to the file dailyfasterlist.
#----------------------------------------------------
set ftpsite = "faster.gsfc.nasa.gov"
set address = $user@edcmail.cr.usgs.gov
(ftp -n $ftpsite > dailyfasterlist) >& stderror << "END of stdin"
user ftp $address
cd pub/data/meyer
ls daily*
bye
"END of stdin"

#-----------------------------------------------------------------------
# Determine the files we need to transfer.  We don't want to transfer
# the same file more than once.  The tovstranlist file contains
# a list of files already transferred.  Using the comm command ftpfiles
# will contain the files that are on the Goddard server that we have not
# yet transferred.
#-----------------------------------------------------------------------
cp $ADAPSTABLES/tovstranlist .
cat tovstranlist | awk '{printf "%s\n", $2}' | sort > tovstranfiles
cat dailyfasterlist | grep pptasurf | awk '{printf "%s\n", $9}' | \
    sort > dailyfasterfiles
set ftpfiles = `comm -13  tovstranfiles dailyfasterfiles`

#----------------------------------------------------------
# Transfer each file in ftpfiles, unzip the file, rename it
# to a LAS friendly file name and move it to ADAPSACQ/tovs
#----------------------------------------------------------
if ( "$ftpfiles" != "" ) then
    foreach cfile ($ftpfiles)
	#--------------------------------------------------------------
	# Create a temporary script to transfer the file and execute it
	# The temp script is needed so we can expand the variable cfile
	#--------------------------------------------------------------
	echo "(ftp -n $ftpsite > dailyfasterlist) >& stderror << ENDOFSTDIN" > ftp1file.csh
	echo user ftp $address >> ftp1file.csh
	echo cd pub/data/meyer >> ftp1file.csh
	echo binary >> ftp1file.csh
	echo prompt >> ftp1file.csh
	echo get $cfile >> ftp1file.csh
	echo bye >> ftp1file.csh
	echo ENDOFSTDIN >> ftp1file.csh
	csh ftp1file.csh

	#-------------------------------------------------
	# Check that the file actually got transferred and
	# break out of this iteration of foreach if not.
	#-------------------------------------------------
	if ( ! -e $cfile ) then
	    set text = "Error transferring $cfile"
	    wrtlog $logfile $key trantovs-error $text
	    break
	endif

	#---------------------------------------
	# Uncompress the daily file, we expect
	# the files to be compressed using gzip.
	#---------------------------------------
	gunzip $cfile

	#-----------------------------------------------------
	# Make sure the file unzipped, if not move it to error
	#-----------------------------------------------------
	set file = `echo $cfile | awk -F. '{printf "%s.%s.%s.%s.%s.%s.%s.%s", $1, $2, $3, $4, $5, $6, $7, $8}'`
	if ( ! -e $file ) then
	    set text = "Error uncompressing $cfile"
	    wrtlog $logfile $key trantovs-error $text
	    mv $cfile $ADAPSERR/$cfile
	    break
	endif
	set text = "Uncompressed: $cfile"
	wrtlog $logfile $key trantovs-uncomp $text

	#-----------------------------------------------------------
	# Extract the information we need from the file name create,
	# a new file name and move it to the acquire directory.
	# Make sure the name is lower case and replace '.' with '_'.
	# The output file name will look like this:
	#     tovsb991001e991031amnj.bin
	# where b991001 is the beginning date 10/01/99, e991031 is
	# the ending date 10/31/99, am is the time of day and nj
	# is the satellite id nj = NOAA-14.
	#-----------------------------------------------------------
	set ofile = `echo $file | awk -F. '{printf "tovs%s%s%s%s",$3,$4,$5,$7}'`
	set ofile = `echo $ofile | tr "A-Z" "a-z" | tr "." "_"`
	mv $file $tovsacq$ofile.bin
	set text = "Created: $ofile.bin"
	wrtlog $logfile $key trantovs-create $text

	#-------------------------------------------------
	# Update the tovstranlist by appending a date code
	# and transferred file name to the transfer list
	#-------------------------------------------------
	echo "$juldate $cfile" >> tovstranlist
    end
endif

#---------------------------------------------------------------------
# Sort the transfer list in reverse numerical order and only keep the
# top XX entries.  If XX is 60, aproximately one months worth of files
# will be kept.  Copy the completed list to ADAPSTABLES.
#---------------------------------------------------------------------
sort -nr tovstranlist > newtovstranlist
head -$numfiles newtovstranlist > tovstranlist
cp tovstranlist $ADAPSTABLES/tovstranlist

#-----------------------------
# remove the working directory
#-----------------------------
# ??? uncomment next line and remove this comment
#rm -r $workdir

#--------------------------------------
# Print a successful completion message
#--------------------------------------
set text = "TRANTOVS completed successfully"
wrtlog $logfile $key trantovs-success $text

exit 0
