/*****************************************************************************

FUNCTION:       CREATE_ACQUIRE

PURPOSE:        Takes the six files extracted from the VMS backup tape and
		concatenates them into a CPA acquired image suitable for
		processing with INGEST.

*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "las.h"
#include "adapsutil.h"
#include "readcpa.h"

int create_acquire
(
    int setnum,		/* Number of current saveset	*/
    char *tapeid,	/* ID of tape being processed 	*/
    char *dstdir	/* Destination directory	*/
)
{
    FILE *fp[6];	/* Six file pointers for associated data files	*/
    FILE *ofp;		/* Output acquire image file			*/
    char filename[255];	/* Storage space to build filenames		*/
    char year[3];	/* Year of acquisition				*/
    char *tmp;		/* Temporary pointer for getenv()		*/
    char key[15];	/* Key field for c_wrtlog()			*/
    char logfile[255];	/* Name of log file name			*/
    char text[255];	/* Text for c_wrtlog() error message		*/

    unsigned char unpacked[1536];	/* Data buffer for minor frame	*/
    unsigned char unbuff[4096];		/* Data buffer for video data	*/

    int yr;		/* Integer year value				*/
    int i;		/* Loop counter					*/
    int done = 0;	/* Flag for finished processing			*/
    int bytes_read;	/* Number of bytes read				*/

    tmp = getenv("ADAPSLOG");
    sprintf(logfile, "%s/tpacquire.log", tmp);

    /* Open the six data files extracted from the VMS BACKUP tape and
       an output file.
       -------------------------------------------------------------*/
    for (i = 0; i < 6; ++i)
	{
	sprintf(filename, "%s/z%d_%d.%d", dstdir, (int) getpid(), setnum, i);
	fp[i] = fopen(filename, "rb");

	/* If we are unable to open an unexpected input file, then
	   something went wrong in the saveset (most likely, we hit an
	   EOF before we had read all of the associated channel data.)
	   -----------------------------------------------------------*/
	if (!fp[i])
	    {
	    close_tape();
	    sprintf(key, "%s-%d-cpa", tapeid, setnum);
	    sprintf(text, "Unexpected EOF in saveset %d", setnum);
	    c_wrtlog(logfile, key, "tpacq-error", text);
	    c_lasmsg(TRUE, "readcpa-eof", "Unexpected EOF in saveset %d",
			setnum);
	    }
	}

    /* Generate an output filename of the format:

       z<tapeid>_<year>_fn<image number>.cpa

       The year can be found in ASCII format in the first two bytes of
       the image auxiliary file, i.e. "951021".  The "z" prefix is so
       that the INGEST daemon does not mistakenly find the image while
       we are in the middle of building it.
       ---------------------------------------------------------------*/
    year[0] = fgetc(fp[0]);
    year[1] = fgetc(fp[0]);
    year[2] = '\0';
    yr = atoi( year );
    sprintf(filename, "%s/z%s_%02d_fn%03d.cpa", dstdir, tapeid, yr, setnum);
    ofp = fopen(filename, "wb");
    if (!ofp)
	{
	close_tape();
	c_lasmsg(TRUE, "readcpa-diskwrite", "Error writing output file %s",
		 filename);
	}

    /* In the input auxiliary data file, skip past the initial header 
       block (512 bytes), the initial navigation block (512 bytes),
       and the 3 words (6 bytes) of data that are apparently garbage
       at the front of the HRPT minor frame.
       --------------------------------------------------------------*/
    if (fseek(fp[0], 1030, SEEK_SET))
	{
	close_tape();
	c_lasmsg(TRUE, "readcpa-diskread", "Error reading input file %s",
		 filename);
	}

    /* In the input data files for each band, skip the initial header
       block and navigation block (1024 bytes.)
       --------------------------------------------------------------*/
    for (i = 1; i < 6; ++i)
	if (fseek(fp[i], 1024, SEEK_SET))
	    {
	    close_tape();
	    c_lasmsg(TRUE, "readcpa-diskread", "Error reading input file %s",
		     filename);
	    }

    /* Main loop.  Construct the acquisition image by writing:

       (Line 1) minor frame, ch1 data, ch2 data, ..., ch5 data
       (Line 2) minor frame, ch1 data, ch2 data, ..., ch5 data

       etc. until we reach the end of the input files.
       -------------------------------------------------------*/
    while (!done)
	{
	for (i = 0; i < 6; ++i)
	    {

	    /* For the auxiliary file, read 1536 bytes at a time.  The
	       last line will be six bytes shorter due to the three 
	       words of garbage at the front of the minor frames.
	       -------------------------------------------------------*/
	    if (i == 0)
		{
		bytes_read = fread(unpacked, 1, 1536, fp[0]);
		if ((bytes_read != 1536) && (bytes_read != 1530))
		    {
		    done = 1;
		    continue;
		    }
		if (fwrite(unpacked, 1, 1536, ofp) != 1536)
		    {
		    close_tape();
		    c_lasmsg(TRUE, "readcpa-diskwrite",
			     "Error writing output file %s",
			     filename);
		    }
		}

	    /* For the image data (.1 - .5), read 4096 bytes at a time.
	       --------------------------------------------------------*/
	    else
		{
		if (fread(unbuff, 1, 4096, fp[i]) != 4096)
		    {
		    done = 1;
		    continue;
		    }
		if (fwrite(unbuff, 1, 4096, ofp) != 4096)
		    {
		    close_tape();
                    c_lasmsg(TRUE, "readcpa-diskwrite",
			     "Error writing output file %s",
			     filename);
		    }
		}
	    }
	}

    /* Close all of the associated files.
       ----------------------------------*/
    for (i = 0; i < 6; ++i)
	fclose(fp[i]);
    fclose(ofp);

    /* Remove all of the temporary files from disk.
       --------------------------------------------*/
    sprintf(filename, "rm %s/z%d_%d.[0-5]", dstdir, (int) getpid(), setnum);
    system(filename);

    /* Rename the created acquisition image to use an "a" prefix so
       that the INGEST daemon will pick it up.
       ------------------------------------------------------------*/
    sprintf(filename, "mv %s/z%s_%02d_fn%03d.cpa %s/a%s_%02d_fn%03d.cpa",
	    dstdir, tapeid, yr, setnum, dstdir, tapeid, yr, setnum);
    system(filename);
    sprintf(key, "%s-%d-cpa", tapeid, setnum);
    c_wrtlog(logfile, key, "tpacq-success", "Completed successful acquire");

    return 0;
}
