/*****************************************************************************

FUNCTION:	PROCESS_BLOCK

PURPOSE:	This function reads data in from VMS backup blocks.  Blocks
		are read from the tape one at a time and sent to this
		function, where the records within them are interpreted and
		the proper information is written to disk files.

ALGORITHM REFERENCES

	Source code for "vmsbackup" by John D. Carey and Sven-Ove Westberg

*****************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include "las.h"
#include "readcpa.h"

int process_block
(
	unsigned char *buffer,
	int blocksize,
	int setnum,
	char *dstdir
)
{
    int16 recsize;	/* Size of a record within the block		*/
    int16 rectype;	/* Type of a record within the block		*/
    int16 appcode;	/* Application code				*/
    int filetype;	/* Type of the file currently being read	*/
    int always = 1;	/* for compiler warning				*/

    static int writeflg = 0;	/* Should we extract this file?		*/
    static char filename[255];	/* Name of file currently being read	*/

    char *ptr = (char *)buffer;	/* Position within backup block			*/
    int bytes_read = 0;	/* Bytes read from current backup block		*/

    /* Check the application code, found in bytes 6-7 of the block header.
       For reasons unknown at this time, it appears that only blocks with
       an application code of 1 contain valid data.  Any other codes
       appear to hold only junk.  This should be investigated further.
       ------------------------------------------------------------------*/
    appcode = to_int16(ptr, 6);
    if (appcode != 1)
	return 0;

    bytes_read += 256;
    ptr += 256;

    while (always) /* to eliminate compiler warning */
    {
	/* At this point, ptr should be positioned on a record header.
           -----------------------------------------------------------*/
	recsize = to_int16(ptr, 0);
	rectype = to_int16(ptr, 2);

	/* Advance ptr past record header to the actual record itself.
	   -----------------------------------------------------------*/
	ptr += 16;

	/* Now that we've determined the record type, we need to process it
	   accordingly.  We only need to be concerned with Type 1 (saveset
	   summaries), Type 3 (file entries), and Type 4 (Virtual Block
	   Records, which contain the actual file data).
	   ----------------------------------------------------------------*/
	switch (rectype)
	{
	    case REC_NULL:

 		/* Null records contain no data and are usually found right
		   at the end of the block; so we'll simply ignore it. */

		break;

	    case REC_SUMMARY:
		
		/* Summary records contain information about the date, the
		   tape, and the subcommands that were given to the VMS
		   BACKUP command; at this point they don't appear to
		   include any useful information to our specific purpose,
		   so ignore them as we did the null records. */

		break;

	    case REC_FILE:

		/* If this is a file descriptor record, obtain the type of
		   data that it contains.  If it's an image data file,
		   create a unique temporary name to write it to.  If the
		   file contains no relevant data, don't write it. */

		filetype = process_file((unsigned char *)ptr);
		if (filetype < 0)
		    writeflg = 0;
		else
		    {
		    writeflg = 1;
		    sprintf(filename, "%s/z%d_%d.%d", dstdir, 
			    (int) getpid(), setnum, filetype); 
		    if (filetype == 0)
			printf("[informational] Extracting auxiliary information from saveset %d\n", setnum);
		    else
		        printf("[informational] Extracting band %d from saveset %d\n",
			   filetype, setnum);
		    }
		break;

	    case REC_VBN:

		/* If this is a virtual block record (contains file data),
		   then dump that data to an output file if it contains
		   image data; ignore it if it does not. */

		if (writeflg)
		    process_vbn((unsigned char *)ptr, filename, recsize);
		break;

	    default:

		/* If we encounter an unexpected record type, issue a 
		   warning message and continue processing. */

		c_lasmsg(FALSE, "readcpa-rectype",
			 "Unknown record type %d", rectype);
		break;
	}

	/* After processing the current record, increment the pointer into
	   the buffer to point to the next record.  If we're done processing
	   the entire block, ride off into the sunset and read the next one.
	   -----------------------------------------------------------------*/
	ptr += recsize;
	bytes_read = bytes_read + 16 + recsize;
	if (bytes_read >= blocksize) break;
    }
    return 0;
}
