/*****************************************************************************

PROGRAM:	READCPA

PURPOSE:	Produce AVHRR acquire images from magnetic tapes recieved
	        in the VMS BACKUP format.

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      1/97   C. Engebretson   Original development

ALGORITHM REFERENCES:

	Source code for "vmsbackup" by John D. Carey and Sven-Ove Westberg

*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util2.h"
#include "adapsutil.h"
#include "readcpa.h"
#include "tapeio.h"

int main()
{
    long blocksize;		/* Size of blocks in saveset		*/
    int blocks_read;		/* Number of blocks read in current set */
    int setnum;			/* Number of current saveset		*/
    int image[MAX_IMG];		/* Savesets to process on tape		*/
    unsigned char *buffer = 0;	/* Buffer for reading blocks from tape  */
    long mode = HANDLE;		/* Tape read mode			*/
    long unit;			/* Unit number of input tape device	*/
    long std_out;		/* Standard output			*/
    long to_skip;		/* Number of scenes to skip		*/
    int numimg;			/* Number of images to process		*/
    int curimg = 0;		/* Image number being processed		*/
    int curpos = 0;		/* Current physical position on tape	*/
    char tapeid[CMLEN];		/* Tape ID				*/
    char dest[CMLEN];		/* Environment variable to destination	*/
    char dstdir[CMLEN];		/* Destination directory		*/
    char *tmp;			/* Temporary pointer			*/
    int status;			/* Return code of library functions	*/
    struct PARBLK vblock;	/* TAE parameter block			*/
    struct TAPEDESC *drive;	/* Tape drive descriptor		*/
    long zero = 0;		/* Constant of zero			*/
    char key[15];		/* Key field for c_wrtlog()		*/
    char filename[CMLEN];	/* Name of log file			*/
    char text[CMLEN];		/* Text field for c_wrtlog()		*/
    int always = 1;		/* to eliminate compiler warning	*/
    char dummy[1][CMLEN] = {""};/* for c_complt parameter type match	*/

    /* Initialize LAS and retrieve parameters from TAE.
       -----------------------------------------------*/
    c_inlas(&vblock, &std_out);
    getpar(&vblock, tapeid, image, &numimg, dest);
    tmp = getenv(dest);
    if (!tmp)
	c_lasmsg(TRUE, "readcpa-env", "Unable to retrieve value of $%s\n",
		dest);
    strcpy(dstdir, tmp);

    if (mount_tape(tapeid, &unit) != E_SUCC)
	c_lasmsg(TRUE, "readcpa-fatal", "Fatal error encountered");
    drive = (struct TAPEDESC *) unit;

    /* Note start of procedure in the TPACQUIRE log file.
       --------------------------------------------------*/
    tmp = getenv("ADAPSLOG");
    if (!tmp)
	c_lasmsg(TRUE, "readcpa-env", "Unable to retrieve value of $ADAPSLOG");
    sprintf(filename, "%s/tpacquire.log", tmp);
    sprintf(key, "%s-0-cpa", tapeid);
    c_wrtlog(filename, key, "tpacq-start", "Starting the TPACQUIRE procedure");

    while(always)	/* for compiler warning */
	{
	/* If need be, fast forward to the desired image.
	   ----------------------------------------------*/
	if ((image[curimg] != curpos) && (numimg != 0))
	    {
	    to_skip = image[curimg] - curpos;
	    sprintf(text, "Skipping %d savesets on tape", to_skip);
	    c_wrtlog(filename, key, "tpacq-skip", text);
	    printf("[informational] Skipping %d savesets on tape\n", to_skip);
	    curpos += to_skip;
	    to_skip *= 3;
	    status = c_tpfsf(&unit, &to_skip, &mode);
	    }

	/* Main loop.  Read the ANSI header record to determine the block
	   size and the sequential number of the current saveset.
	   --------------------------------------------------------------*/
	if (read_header(unit, &blocksize, &setnum) == -1)
	    break;

	if (blocksize == -1)
	    {
	    free(buffer);
	    close_tape();
	    c_wrtlog(filename, key, "tpacq-error",
			"Unable to determine block size");
	    c_lasmsg(TRUE, "readcpa-blocksize",
		     "Unable to determine block size");
	    }

	buffer = realloc(buffer, blocksize);
	if (!buffer)
	    {
	    close_tape();
	    c_wrtlog(filename, key, "tpacq-error",
			"Error allocating dynamic memory");
	    c_lasmsg(TRUE, "readcpa-alloc",
		     "Error allocating dynamic memory");
	    }

	blocks_read = 0;

	while(always)	/* for compiler warning */
	    {

	    /* Read data from tape.  If we've hit an EOF, then we're at
	       the end of a saveset and need to combine every file that
	       we've produced into a single acquisition image.  If not,
	       process the current block.
	       --------------------------------------------------------*/
	    status = c_tpread(&unit, buffer, &blocksize, &mode);
	    if (status == TPREAD_READ)
		{
		free(buffer);
		close_tape();
		c_wrtlog(filename, key, "tpacq-error",
			"Error reading from tape");
		c_lasmsg(TRUE, "readcpa-read", "Error reading from tape");
		}

	    if (status == TAPE_EOF)
		{
		if (blocks_read == 0)
		    break;
		printf("[informational] Creating aquisition image\n");
		create_acquire(setnum, tapeid, dstdir);
		break;
		}
	    if (status != blocksize)
		printf("expected %d, read %d\n", blocksize, status);

	    process_block(buffer, blocksize, setnum, dstdir);
	    blocks_read++;
	    }

	/* Read the ANSI trailer file and continue through the loop,
	   reading the header file for the next saveset.
	   ---------------------------------------------------------*/
	if (blocks_read == 0)
	    break;
	read_trailer(unit, setnum);

	/* If we've read all images, then exit the loop.
	   ---------------------------------------------*/
	curimg++;
	curpos++;
	if ((curimg == numimg) && (numimg != 0))
	    break;
	}

    /* At this point, the end-of-tape has been encountered (read_header()
       returns a value of -1 to notate this situation.)  Clean up and exit.
       ------------------------------------------------------------------*/
    free(buffer);
    close_tape();
    sprintf(text, "Read %d images from drive %s",
	    (numimg ? numimg : curimg), drive->name);
    c_wrtlog(filename, key, "tpacq-drive", text);

    c_complt("READCPA", dummy, &zero);
    return 0;
}
