#!/bin/csh -f
#
# FUNCTION: tpacquire
#
# PURPOSE:  Acquire images from tape to disk.
#
#
# Version  Programmer       Date   Change Request
# -------  ---------------  -----  ---------------------------------------------
#   1.0    T.Rockvam        06/92  Original development
#   1.1    J.Fenno          11/92  Broke the large switch statement into if
#                                  statements.  Apparently the switch statement
#                                  can get too large in shell programming.
#                                  Added error checking for all fsf and dd
#                                  commands.
#                                  Added parameters that identify where the
#                                  commands checkspace, get_device, and
#                                  write_log are located.
#                                  Combined tpacquire and tpselect.
#   1.2    J.Fenno          09/93  Added code for MN2.
#   1.3    J.Fenno          12/94  By request from Jeff Powell, removed the 30
#				   minute limit when waiting for disk space.
#				   The function should wait indefinitely.
#   1.4    T.Baltzer        11/96  Modified to add station CFQ, same as TSV.
#
#
# COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
#   cshell script called from TPACQUIRE.PDF or from command line if drive is
#   allocated.
#
#
# NOTES:
#   1. When using remotely mounted tapes the user must have an account on the
#      remote machine and they must have a .rhosts file on that machine.
#
#   2. Shell metacharacters which are not quoted are interpreted on local
#      machine, while quoted metacharacters are interpreted on the remote
#      machine.  Thus the command
#
#         rsh otherhost cat remotefile >> localfile
#
#      appends the remote file remotefile to the localfile localfile, while
#
#         rsh otherhost cat remotefile ">>" otherremotefile
#
#      appends remotefile to otherremotefile.
#
#   3. To redirect standard output and standard error to separate files, use
#      (cmd > file1) >& file2.  On the DG, there is a bug in the csh which does
#      not allow this double redirection.  There is also a problem when trying
#      to redirect the output from the dd commands on certain systems.
#      Therefore, all of the dd commands have a set of ()'s around the command
#      which clears up the problem?!
#
#   4. The dd, bsf, fsf junk at the end is to try to check for end of volume.
#      We didn't just capture the status of the dd that reads the image because
#      this may indicate a problem with the tape or drive within the image
#      file.  Each tape/system/station combination has special cases:
#
#      DG: Use fsr to check for EOV.  Use fsr instead of dd because dd gets
#          hung up at EOV.
#
#      Other systems: Use dd to check for EOV
#
#      8mm/9-track: When at EOV a dd creates a 0-size file.  We must do an
#          extra fsf to get to the correct side of the FMK for further dd's to
#          fail.
#
#      3480: fsf, dd.  dd fails at EOV for 3480 
#
#   5. The Japan tapes (tk2) are composed of tar files.  A tar archive is a
#      stream of 512-byte header structures which may be followed by file data
#      in 512-byte blocks.  They used a blocking factor of 20, so we set the
#      block size to 10240, so when checking for EOV, we fsr or dd ahead 1
#      record, which will be a 10240-byte header in the tar file.
#
#   6. The variable free_bytes designates how many free bytes of disk space
#      must be available before tpacquire will read an image(s) from tape.
#      This value is currently much higher than the typical 70-110 Mbytes for
#      an AVHRR image because we cannot fill up a disk with acquired images,
#      leaving us no disk space for processing the images.
#
#
#
#


#---------------------------
# On interrupt go to CLEANUP
#---------------------------
onintr CLEANUP
set noglob


#------------------------------
# Check the number of arguments
#------------------------------
if ($#argv < 10) goto USAGE


#---------------------
# Set up the variables
#---------------------
set tape_id  = `echo $1 | awk '{printf "%-6s",$1}' | tr "[A-Z]" "[a-z]"`
set drive_no = $2
set ceos_id  = `echo $3 | tr "[A-Z]" "[a-z]"`
set year     = $4
set destdir  = $5


#---------------------------------------------------------
# Set up aliases for checkspace, get_device, and write_log
#---------------------------------------------------------
if ("$6" == ".") then
   alias checkspace `pwd`/checkspace
else
   alias checkspace $6/checkspace
   endif
if ("$7" == ".") then
   alias get_device `pwd`/get_device
else
   alias get_device $7/get_device
   endif
if ("$8" == ".") then
   alias wrtlog `pwd`/write_log
else
   alias wrtlog $8/write_log
   endif


#--------------------------------------------------------------------------
# Determine if the user is specifying a range or list of image numbers.  In
# either case, set up the variable $images which is a list of all the image
# numbers to be read from tape.
#--------------------------------------------------------------------------
set cmd = `echo $9 | tr "[A-Z]" "[a-z]"`
if ("$cmd" == "range") then
   set i = ($10)
   if (($#argv != 11) || ($#i != 1)) goto USAGE
   set images = ""
   while ($i <= "$11")
      set images = ($images $i)
      @ i += 1
      end
else if ("$cmd" == "select") then
   set i = 0
   while ($i < 9)
      @ i += 1
      shift
      end
   set images = ($*)
else
   goto USAGE
   endif
unset cmd


#---------------------------------------------------------------
# Check the length of the CEOS ID and the Year.  Note that wc -c
# reports one more character than the length of a string.
#---------------------------------------------------------------
@ len = `echo $ceos_id | wc -c` - 1
if ($len != 3) then
   echo "[tpacquire-len] CEOSID must be three characters"
   exit 1
   endif
@ len = `echo $year | tr "[A-Z]" "[a-z]" | wc -c` - 1
if ($len != 2) then
   echo "[tpacquire-len] YEAR must be two characters"
   exit 1
   endif


#---------------------------------------------------------------------------
# Unalias all commands used in this shell script to get rid of user aliases.
# If there is a /usr/bsd/rsh then use this command instead of the /bin/rsh
# because the Berkley version actually works with remote systems while the
# /bin/rsh version does not.  If there isn't a /usr/bsd/rsh then /bin/rsh
# usually works.
#---------------------------------------------------------------------------
unalias cd
unalias dd
unalias mt
unalias rm
unalias rsh
if (-e /usr/bsd/rsh) alias rsh /usr/bsd/rsh


#--------------------------------------------------------------------
# Set up the processing variables.  $logfile must be a LAS file name.
#--------------------------------------------------------------------
set disk_space    = "NO"
set eov_reached   = "NO"
set free_bytes    = 500000000
set logfile       = `echo "[$ADAPSLOG]tpacquire" | tr / . | sed  's/\[\./\[/'`
set remote_system = "NO"


#------------------------------------------------------------------------------
# Get the device name of the drive and set the TAPE environment variable for mt
#------------------------------------------------------------------------------
set drive_name = `get_device $drive_no | awk '{print $1}'`
setenv TAPE $drive_name


#-------------------------------------------------------------
# Determine if we are using a drive on a remote system.  This
# is indicated by having a drive number of the form edcsgs4@4.
#-------------------------------------------------------------
set system_name = `uname -n`
set drive_num = `echo $drive_no | awk -F@ '{print $2}'`
if ($drive_num != "") then
   set system_name = `echo $drive_no | awk -F@ '{print $1}'`
   set remote_system = "YES"
   endif


#------------------------------------------------------------------------------
# Create an alias for the dd and mt commands if using a remote system.
#
# NOTE: Silicon Graphics systems use a -t option for the mt command while all
#       other systems us a -f option.
#------------------------------------------------------------------------------
if ($remote_system == "YES") then
   #-------------
   # Alias for mt
   #-------------
   if (("$system_name" =~ sg*) || ("$system_name" =~ edcsg*)) then
      alias mt rsh $system_name mt -t $drive_name
   else
      alias mt rsh $system_name mt -f $drive_name
      endif

   #-------------
   # Alias for dd
   #-------------
   alias dd rsh $system_name dd
   endif


#----------------------------------------------------
# Set up a temporary file name so the ADAPS ingester
# doesn't grab an image while we are still reading it
#----------------------------------------------------
set tmpname = $destdir/z${tape_id}_${year}_${ceos_id}.tmp


#-----------------------------------------------------------------------------
# Set up the tape block size.  Ensure that if a CEOS ID is added to this case
# statement then it is added to one of the if statements within the processing
# loop.
#-----------------------------------------------------------------------------
switch ("$ceos_id")
   case baa:
      set blksz = 29696
      breaksw
   case bio:
   case dvw:
   case emt:
   case pas:
      set hdrsz = 2520
      set blksz = 21872

      #-------------------------------------
      #	Skip over the first tape header file
      #-------------------------------------
      if (! { mt fsf >>& /dev/null }) then
	 set text = "Error forward skipping file"
	 wrtlog $logfile ${tape_id}-0-${ceos_id} tpacq-error "$text"
	 echo "[tpacquire-fsf] $text"
	 exit 1
	 endif
      breaksw
   case bjg:
      set blksz = 14800
      breaksw
   case dxl:
   case gzc:
   case uqc:
      set blksz = 7400
      breaksw
   case jsa:
      set blksz = 10464
      breaksw
   case cpa:
      set hdrsz = 122
      set blksz = 7400
      breaksw
   case ea1:
      set blksz = 65024
      breaksw
   case ea2:
   case eaf:
   case nm2:
      set blksz = 32768
      breaksw
   case hb3:
      set blksz = 5596
      breaksw
   case adl:
   case als:
   case asp:
   case cfq:
   case csy:
   case dar:
   case mn2:
   case pth:
   case sdn:
   case hbt:
   case tsv:
      set blksz = 14788
      breaksw
   case lsu:
      set blksz = 22180
      breaksw
   case cai:
   case klm:
   case mnl:
   case mps:
   case nmy:
   case nrb:
   case op1:
   case run:
   case scz:
   case tms:
   case tnb:
      set blksz = 21980
      breaksw
   case nm1:
      set blksz = 11008
      breaksw
   case dsa:
   case hln:
   case ssp:
      set blksz = 1024
      breaksw
   case hbk:
      set blksz = 63488
      breaksw
   case tk2:
      #---------------------------------------------------
      # cd to the destination directory because you cannot
      # specify the destination for the output of tar.
      #---------------------------------------------------
      cd $destdir
      set blksz = 10240

      #----------------------------------------------------------------------
      #	If on remote system, reset name of the tape drive to use.  tar allows
      # remote tape drives by using the syntax: [user@]system:/dev/....
      #----------------------------------------------------------------------
      if (($remote_system == "YES") && ($system_name !~ edcsg*) && \
	  ($system_name !~ sg*)) then
	 set text = "Remote system not allowed for tk2"
	 wrtlog $logfile ${tape_id}-0-${ceos_id} tpacq-error "$text"
	 echo "[tpacquire-remote] $text"
	 exit 1
      else if ($remote_system == "YES") then
	 set drive_name = "${system_name}:$drive_name"
	 endif

      breaksw
   default:
      set text = "Unknown CEOS ID $ceos_id"
      wrtlog $logfile ${tape_id}-0-${ceos_id} tpacq-error "$text"
      echo "[tpacquire-ceos] $text"
      exit 1
      breaksw
   endsw


#---------------------------
# Log the start of tpacquire
#---------------------------
set text = "Starting the TPACQUIRE procedure"
wrtlog $logfile ${tape_id}-0-${ceos_id} tpacq-start "$text"
echo "[informational] $text"


#--------------------------
# Loop until EOV is reached
#--------------------------
set file_no     = 1
set images_read = 0

while (($eov_reached == "NO") && ($#images > 0))

   #----------------------------------------
   # Get the first image number from $images
   #----------------------------------------
   set image = $images[1]


   #------------------------------------------------
   # If we want to read this image, check disk space
   #------------------------------------------------
   if ($file_no == $image) then
      set i = 0
      while ($disk_space == "NO")
	 @ i += 1
	 if ({ checkspace "" $free_bytes $destdir }) then
	    set disk_space = "YES"
	 else if ($i < 61) then
	    sleep 60
	 else
	    set i = 1
	    echo "[informational] TPACQUIRE waiting for disk space"
	    sleep 60
	    endif
	 end
      
      #-------------------------------
      # Issue an informational message
      #-------------------------------
      if ($i > 1) then
	 echo "[informational] Disk space available.  Continuing"
	 endif
      endif


   #-----------------------------
   # Process Australia
   # Process the new South Africa
   #-----------------------------
   if (("$ceos_id" == "adl") || ("$ceos_id" == "als") || \
       ("$ceos_id" == "asp") || ("$ceos_id" == "cfq") || \
       ("$ceos_id" == "csy") || ("$ceos_id" == "dar") || \
       ("$ceos_id" == "pth") || ("$ceos_id" == "sdn") || \
       ("$ceos_id" == "hbt") || ("$ceos_id" == "tsv") || \
       ("$ceos_id" == "hbk") || ("$ceos_id" == "mn2")) then
      #---------------------------------------------------------
      # If the current file number is equal to the desired image
      # number then read the image file.  Otherwise, skip it.
      #---------------------------------------------------------
      if ($file_no == $image) then
	 if (! { (dd if=$drive_name bs=$blksz > $tmpname) }) goto READERR
      else if (! { mt fsf >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif
       
       
   #-----------------------------
   # Process China
   # Process Jiddah, Saudi Arabia
   #-----------------------------
   if (("$ceos_id" == "bjg") || ("$ceos_id" == "dxl") || \
       ("$ceos_id" == "gzc") || ("$ceos_id" == "uqc") || \
       ("$ceos_id" == "jsa")) then
      #---------------------------------------------------------------------
      # If the current file number is equal to the desired image number then
      # read the image file, skipping the first record.  Otherwise, skip it.
      #---------------------------------------------------------------------
      if ($file_no == $image) then
	 if (! { (dd if=$drive_name bs=$blksz skip=1 > $tmpname) }) goto READERR
      else if (! { mt fsf >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif

   #-----------------------------
   # Process Brazil
   #-----------------------------
   if ("$ceos_id" == "cpa") then
      #---------------------------------------------------------------------
      # If the current file number is equal to the desired image number then
      # read the image file, skipping the first record.  Otherwise, skip it.
      #---------------------------------------------------------------------
      if ($file_no == $image) then
	 mt fsr 1 >>& /dev/null
	 if (! { (dd if=$drive_name bs=$blksz > $tmpname) }) goto READERR
      else if (! { mt fsf >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif

   
   #------------------------------------------------------------
   # Process Sioux Falls EDC archive format 1 (4/1/87 - 5/31/88)
   # Process Sioux Falls EDC archive format 2 (6/1/88 - 9/4/90)
   #------------------------------------------------------------
   if (("$ceos_id" == "ea1") || ("$ceos_id" == "ea2")) then
      #-------------------------------------------------------------------
      # If the current file number is equal to the desired image number
      # then skip the header file and read the data file.  Otherwise, skip
      # both files.
      #-------------------------------------------------------------------
      if ($file_no == $image) then
	 if (! { mt fsf >>& /dev/null }) goto FSFERR
	 if (! { (dd if=$drive_name bs=$blksz > $tmpname) }) goto READERR
      else if (! { mt fsf 2 >>& /dev/null }) then
	 goto FSFERR
	 endif

      #---------------------------------------------------
      # There is only one image on these old archive tapes
      #---------------------------------------------------
      set eov_reached = "YES"
      endif


   #-------------------------------------------
   # Process all ESA stations
   # Process Niamey, Niger version for Agrhymet
   # Process Sea Space station
   #-------------------------------------------
   if (("$ceos_id" == "mps") || ("$ceos_id" == "nmy") || \
       ("$ceos_id" == "nrb") || ("$ceos_id" == "op1") || \
       ("$ceos_id" == "tms") || ("$ceos_id" == "nm1") || \
       ("$ceos_id" == "run") || ("$ceos_id" == "mnl") || \
       ("$ceos_id" == "scz") || ("$ceos_id" == "ssp") || \
       ("$ceos_id" == "tnb") || ("$ceos_id" == "cai") || \
       ("$ceos_id" == "klm")) then
      #-------------------------------------------------------------------
      # If the current file number is equal to the desired image number
      # then skip the header file and read the data file.  Otherwise, skip
      # both files.
      #-------------------------------------------------------------------
      if ($file_no == $image) then
	  if (! { mt fsf >>& /dev/null }) goto FSFERR
	  if (! { (dd if=$drive_name bs=$blksz > $tmpname) }) goto READERR
      else if (! { mt fsf 2 >>& /dev/null }) then
	  goto FSFERR
	 endif
    endif


   #----------------------------------------------------------
   # Process Buenos Aires, Argentina (same as Miami)
   # Process Sioux Falls EDC archive format (9/5/90 - present)
   # Process Louisiana State University
   #----------------------------------------------------------
   if (("$ceos_id" == "baa") || ("$ceos_id" == "eaf") || \
       ("$ceos_id" == "lsu") || ("$ceos_id" == "nm2")) then
      #--------------------------------------------------------------------
      # If the current file number is equal to the desired image number
      # then skip the header file, read the data file, and skip the trailer
      # file.  Otherwise, skip all three files.
      #--------------------------------------------------------------------
      if ($file_no == $image) then
	 if (! { mt fsf >>& /dev/null }) goto FSFERR
	 if (! { (dd if=$drive_name bs=$blksz > $tmpname) }) goto READERR
	 if (! { mt fsf >>& /dev/null }) goto FSFERR
      else if (! { mt fsf 3 >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif


   #---------------
   # Process Canada
   #---------------
   if (("$ceos_id" == "bio") || ("$ceos_id" == "dvw") || \
       ("$ceos_id" == "emt") || ("$ceos_id" == "pas")) then
      #-------------------------------------------------------------------
      # If the current file number is equal to the desired image number
      # then skip two files, read the data file, and then skip two trailer
      # files.  Otherwise, skip all five files.
      #-------------------------------------------------------------------
      if ($file_no == $image) then
	 if (! { mt fsf 1 >>& /dev/null }) goto FSFERR
	 if (! { (dd if=$drive_name bs=$hdrsz > $tmpname) }) goto READERR
	 if (! { (dd if=$drive_name bs=$blksz >> $tmpname) }) goto READERR
	 if (! { mt fsf 2 >>& /dev/null }) goto FSFERR
      else if (! { mt fsf 5 >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif
   

   #------------------------------
   # Process Dhahran, Saudi Arabia
   # Process Snug Harbor, Hawaii
   #------------------------------
   if (("$ceos_id" == "dsa") || ("$ceos_id" == "hln")) then
      #-----------------------------------------------------------------
      # If the current file number is equal to the desired image number
      # then skip the header file and read the data file Otherwise, skip
      # both files.
      #-----------------------------------------------------------------
      if ($file_no == $image) then
	 if (! { mt fsf >>& /dev/null }) goto FSFERR
	 if (! { (dd if=$drive_name ibs=1024 obs=22528 > $tmpname) }) then
	   goto READERR
	   endif
      else if (! { mt fsf 2 >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif


   #---------------------
   # Process South Africa
   #---------------------
   if ("$ceos_id" == "hb3") then
      #--------------------------------------------------------------------
      # If the current file number is equal to the desired image number
      # then skip the header file, read all five data files, and skip the
      # trailer file.  Otherwise, skip all seven files.
      #--------------------------------------------------------------------
      if ($file_no == $image) then
	 if (! { mt fsf >>& /dev/null }) goto FSFERR
	 
	 @ i = 0
	 while ($i < 5)
	    @ i += 1
	    if (! { (dd if=$drive_name bs=$blksz skip=1 > $tmpname$i) }) then
	       set text = "Error reading image $file_no from tape"
	       wrtlog $logfile ${tape_id}-${file_no}-$ceos_id tpacq-error $text
	       echo "[tpacquire-read] $text"
	       rm -f ${tmpname}1
	       rm -f ${tmpname}2
	       rm -f ${tmpname}3
	       rm -f ${tmpname}4
	       rm -f ${tmpname}5
	       exit 1
	       endif
	    end
	 if (! { mt fsf >>& /dev/null }) goto FSFERR

	 #------------------------------
	 # cat all of the files together
	 #------------------------------
	 mv  ${tmpname}1 $tmpname
	 cat ${tmpname}2 >> $tmpname
	 rm -f ${tmpname}2
	 cat ${tmpname}3 >> $tmpname
	 rm -f ${tmpname}3
	 cat ${tmpname}4 >> $tmpname
	 rm -f ${tmpname}4
	 cat ${tmpname}5 >> $tmpname
	 rm -f ${tmpname}5
      else if (! { mt fsf 7 >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif


   #--------------
   # Process Japan
   #--------------
   if ("$ceos_id" == "tk2") then
      if ($file_no == $image) then
	 #--------------------------------------------------------------------
	 # Capture the output of tar to get the name of the image file.  On an
	 # interrupt continue on to the next tar file.  An alternate tar
	 # command for remotely accessing a drive on a Sun
	 # `rsh -n $system_name dd if=/dev/rmt0 bs=20b | tar xvBfb - 20`
	 #--------------------------------------------------------------------
	 onintr  CONTIN
	 set tar_stat = `tar -xvf $drive_name |& awk '{print}'`

	 onintr  CLEANUP
	 if ($tar_stat[1] != "x") goto READERR

	 set tar_name=`echo $tar_stat | awk '{print $2}' | awk -F, '{print $1}'`
	 if (! { mt fsf >>& /dev/null }) goto FSFERR
	 mv $tar_name $tmpname
	 goto CONTIN2

	 #-------------------------------------------
	 # If tar asks for another tape the user must
	 # interrupt the shell script with <ctrl-c>.
	 #-------------------------------------------
	 CONTIN:
	 onintr CLEANUP
	 echo ""
	 echo -n "Interrupt <ctrl-c> within 10 seconds to abort"
	 sleep 10
	 set text = "Partial tar file was file $file_no"
	 wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} tpacq-error $text
	 echo "[tpacquire-partial] $text"
	 set disk_space = "NO"
	 @ file_no     += 1
	 @ images_read += 1
	 shift images
	 if ($#images == 0) break
	 continue

	 CONTIN2:
      else if (! { mt fsf >>& /dev/null }) then
	 goto FSFERR
	 endif
      endif
 

   #-----------------------------------------------------------------
   # Check to see if the output file has anything in it.  A file size
   # of zero indicates that we hit another EOF which implies EOV.
   #-----------------------------------------------------------------
   if ($file_no == $image) then
      if (! -e $tmpname) break
      if (-z $tmpname) then
	 rm -f $tmpname
	 break
	 endif

      #----------------------------------------------------
      # Create a new name for the output file and rename it
      #----------------------------------------------------
      set t = "a.$tape_id.$year.fn.$file_no.$ceos_id"
      set t = \
        `echo $t | awk -F\. '{printf "%s%6s_%2s_%s%03d.%3s",$1,$2,$3,$4,$5,$6}'`
      set t = `echo $t | tr " " "_"`
      mv $tmpname $destdir/$t

      #--------------------------------
      # Issue a message to the log file
      #--------------------------------
      set text = "Completed successful acquire"
      wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} tpacq-success "$text"
      echo "[informational] $text"


      #-----------------------------------
      # Shift the image numbers in $images
      #-----------------------------------
      @ images_read += 1
      shift images
      endif


   #--------------------------------
   # Update the processing variables
   #--------------------------------
   set disk_space = "NO"
   @ file_no += 1
   if ($#images == 0) break


   #-----------------------------------------
   # Read ahead to see if we have reached EOV
   #-----------------------------------------
   if ($system_name =~ dg*) then
      #-----------------------------------------------------------
      # Hand csh the command so we don't see the ugly error at EOV
      #-----------------------------------------------------------
      if ({ csh -c mt fsr >>& /dev/null }) then
	 #------------------------------------------------
	 # If ok then back up to the beginning of the file
	 #------------------------------------------------
	 mt bsr 1 >>& /dev/null
      else
	 #----------------------------
	 # dg fsr returns error at EOV
	 #----------------------------
	 set eov_reached = "YES"
	 endif
   else
      if ({ (dd if=$drive_name bs=$blksz count=1 > _temp1) >>& \
	 /dev/null }) then
	 #-----------------------------------------------------------
	 # If check was successful, see if we actually read something
	 #-----------------------------------------------------------
	 if (-e _temp1) then
	    if (-z _temp1) then
	       set eov_reached = "YES"
	       endif
	    rm -f _temp1
	 else
	    set eov_reached = "YES"
	    endif

         #--------------------------------------------------------------------
	 # If all is ok, back up to the beginning of the file.  We must bsf
	 # once to back up before the filemark and then fsf to position at the
	 # beginning of the file we dd'd.
         #--------------------------------------------------------------------
	 mt bsf >>& /dev/null
	 mt fsf >>& /dev/null
      else
	 if (-e _temp1) rm -f _temp1
	 set eov_reached = "YES"
	 endif
      endif
   end


#-----------------------------------------------------------
# Issue a message stating how many files were read from tape
#-----------------------------------------------------------
set text = "Read $images_read images from drive $drive_no"
wrtlog $logfile ${tape_id}-0-${ceos_id} tpacq-drive "$text"
echo "[informational] $text"
exit 0



#---------------
# User interrupt
#---------------
CLEANUP:
   onintr -
   set text = "Read $images_read images from drive $drive_no"
   wrtlog $logfile ${tape_id}-0-${ceos_id} tpacq-drive "$text"
   echo "[informational] $text"

   set text = "User interrupt"
   wrtlog $logfile ${tape_id}-0-${ceos_id} tpacq-usrint "$text"
   echo "[tpacquire-user] $text"
   exit 1


#---------------------------
# Error forward spacing file
#---------------------------
FSFERR:
   set text = "Error forward skipping file"
   wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} tpacq-error "$text"
   echo "[tpacquire-fsf] $text"
   rm -f $tmpname
   exit 1


#--------------------
# Error reading image
#--------------------
READERR:
   set text = "Error reading image $file_no from tape"
   wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} tpacq-error "$text"
   echo "[tpacquire-read] $text"
   rm -f $tmpname
   exit 1


#--------------------
# Usage error message
#--------------------
USAGE:
   echo ''
   echo 'USAGE: tpacquire tape drive ceos year dir cpath gpath wpath cmd ...'
   echo ''
   echo 'WHERE:'
   echo '   tape    = 6 character tape ID'
   echo '   drive   = Tape drive number'
   echo '   ceos    = 3 character CEOS ID'
   echo '   year    = 2 digit year'
   echo '   dir     = Host destination directory'
   echo '   cpath   = Path for the command checkspace'
   echo '   gpath   = Path for the command get_device'
   echo '   wpath   = Path for the command write_log'
   echo '   cmd     = Command type:'
   echo '                range  -> $10 = start image'
   echo '                          $11 = stop image'
   echo '                select -> $10, $11, $12, ... = images to read'
   echo ''
   echo ''
   echo 'EXAMPLES:'
   echo ''
   echo '   tpacquire km1000 sg2@2 lsu 92 /tmp ~/bin ~/bin ~/bin range 2 10'
   echo '      Read images 2 through 10 from tape km1000 which is mounted on'
   echo '      tape drive sg2@2.'
   echo ''
   echo '   tpacquire km1000 2 lsu 92 /tmp ~/bin ~/bin ~/bin select 2 5 9'
   echo '      Read images 2, 5, and 9 from tape km1000 which is mounted on'
   echo '      tape drive 2 of the current computer system.  It is assumed that'
   echo '      the list of images to read is sorted in ascending order, if not,'
   echo '      some of the images will not be read from tape.'
   echo ''
   exit 1
