/*****************************************************************************
FUNCTION:	TAREX

PURPOSE:	Extract tar files from tape and skip any files that have
		bad checksums in their tar header.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    06/97	R.Dilley	Initial development
  2.0	 05/98	D.Lloyd		Modified to properly skip bad checksums and
  				to comply with LAS coding standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util2.h"
#include "adapsutil.h"
#include "tapeio.h"
#include "tarex.h"


int main(void)
{
long	count = 0;	/* count of output images created		*/
long	eofopt = 0;	/* write zero EOFs to tape on close		*/
long	image[MAX_IMAGES];	/* array of image number to extract	*/
long	inout = TREAD;	/* flag: tape open in read mode			*/
long	maxbyte;	/* maximum bytes allowed for tape device	*/
long	mode = HANDLE;	/* flag: tape I/O function handle errors	*/
long	msgtim;		/* time between processing messages		*/
long	nimages;	/* the number of files to extract		*/
long	one = 1;	/* value of one					*/
long	rewopt = NOREW;	/* flag: don't rewind tape on open		*/
long	skip = 0;	/* number of files to skip in tape		*/
long	status;		/* function return status			*/
long	std_out;	/* standard output				*/
long	tape_blksize;	/* block size of the tape blocks		*/
long	tplun;		/* tape logical unit number			*/

char	ceosid[4];	/* the CEOS ID of the receiving station		*/
char	density[5];	/* tape density					*/
char	drive[CMLEN];	/* tape drive name				*/
char	dstdir[CMLEN];	/* destination directory of the images		*/
char	errdir[CMLEN];	/* error directory name				*/
char	key[16];	/* key field for c_wrtlog()			*/
char	log[CMLEN];	/* log file path and name			*/
char	msg[CMLEN];	/* message string for wrtlog and lasmsg		*/
char	out[MAX_IMAGES][CMLEN];	/* list of output images created	*/
char	*tape_buf;	/* tape input buffer				*/
char	tapeid[CMLEN];	/* ID of tape mounted				*/
char	tar_buf[RECORDS][RECORD_SIZE];	/* current tar block		*/
char	*tmpbuffer;	/* temporary string				*/


struct	PARBLK	vblock;	/* TAE parameter block				*/
struct	TAPEDESC *tp;	/* pointer to the tape descriptor		*/



enum	stats	bufstat;	/* tar buffer status			*/




/* Initialize LAS and retrieve parameters from TAE.
---------------------------------------------------*/
c_inlas( &vblock, &std_out );
getpar( &vblock, tapeid, drive, ceosid, dstdir, image, &nimages, &msgtim );


/*
  The environment variable name of the destination directory is passed in
  so get the full path name and put back in dstdir
-------------------------------------------------------------------------*/
tmpbuffer = getenv( dstdir );
if ( !tmpbuffer )
    c_lasmsg(TRUE, "tarex-env","Unable to retrieve value of $%s",dstdir);
else
    strcpy( dstdir, tmpbuffer );


/*
  Retrieve the ADAPSERR environment variable
----------------------------------------------*/
tmpbuffer = getenv( "ADAPSERR" );
if ( !tmpbuffer )
    c_lasmsg(TRUE, "tarex-env","Unable to retrieve value of $ADAPSERR");
else
    strcpy( errdir, tmpbuffer );


/*
  Retrieve the ADAPSLOG environment variable.
----------------------------------------------*/
tmpbuffer = getenv( "ADAPSLOG" );
if ( !tmpbuffer )
    c_lasmsg(TRUE, "tarex-env","Unable to retrieve value of $ADAPSLOG");
else
    {
    sprintf( log, "%s/tpacquire.log", tmpbuffer );
    sprintf( key, "%s", tapeid );
    }


/*
  Reopen the tape drive.
-------------------------*/
strcpy( density, "HI" );
status = c_tpreop( &tplun, &skip, &mode, &rewopt, density, &inout, drive );
if ( status != TAPE_SUCC )
    c_lasmsg(TRUE,"tarex-open","Unable to open tape drive");


/*
  Determine the tape block size by reading a block of MAXBYTE size
  then rewind the tape to beginning.
-------------------------------------------------------------------*/
tp = (struct TAPEDESC *)tplun;
maxbyte = tp->maxbyte;
tmpbuffer = (char *)malloc( maxbyte + 1 );
if ( tmpbuffer == NULL )
    {
    c_tpclos( &tplun, &mode, &rewopt, &eofopt );
    c_lasmsg(TRUE,"tarex-alloc","Error allocating memory for tape_buf");
    }
tape_blksize = c_tpread( &tplun, (unsigned char *)tmpbuffer, &maxbyte, &mode );
c_tprew( &tplun, &mode, &one );
free( tmpbuffer );


/*
  Now allocate the appropriate size for the tape_buf.  The amount
  needed is default tar block size (10240) plus the tape block size.
  This size is needed to account for tape block sizes that are not
  multiples of the tar block size.  We will read from tape until
  we have enough for a full tar buffer which could be more than
  the tar block size.
---------------------------------------------------------------------*/
tape_buf = (char *)malloc( DEFAULT_TAR_BLK + tape_blksize );
if ( tape_buf == NULL )
    {
    c_tpclos( &tplun, &mode, &rewopt, &eofopt );
    c_lasmsg(TRUE,"tarex-alloc","Error allocating memory for tape_buf");
    }


/*
  Initialize loop parameters.
------------------------------*/
bufstat = BUF_EMPTY;
status = fill_tar_blk( &tplun, tape_buf, (char *)tar_buf, tape_blksize );
if ( status > 0 )
    bufstat = BUF_FULL;


/*
  Display a starting message and the tape block size.
-----------------------------------------------------*/
sprintf( msg, "Starting to process tape %s", tapeid );
c_wrtlog( log, key, "tarex-start", msg );
c_lasmsg(FALSE,"tarex-start",msg);
sprintf( msg, "Tape block size: %d", tape_blksize );
c_lasmsg(FALSE," ",msg);
c_wrtlog( log, key, "informational", msg );


/*
  Process the tape
-------------------*/
process( tar_buf, msgtim, log, key, image, tplun, tape_buf, tape_blksize,
	&bufstat, dstdir, tapeid, ceosid, errdir, out, nimages, &count );


/*
  Close input
--------------*/
status = c_tpclos( &tplun, &mode, &rewopt, &eofopt );
if ( status != TAPE_SUCC )
    c_lasmsg(FALSE,"tarex-close","Error closing tape");


/*
  Generate completion message
------------------------------*/
sprintf( msg, "Extracted %d images from tape %s", count, tapeid );
c_wrtlog( log, key, "tarex-complete", msg );
c_complt( "TAREX", out, &count );
return 0;
}
