/*******************************************************************************
NAME:		FPROCPROJ

PURPOSE:	Process a line segment from the World Data Bank
		We quickly process the segment & don't bother to 
		check for duplicate segments

*******************************************************************************/
#include <stdio.h>
#include "las.h"

FUNCTION fprocproj(ddr,fp_rel,window,total_pts,pixval,linwt,pntint,fp_scn,
		   geo_code,geo_zone,geo_units,geo_parms)

struct DDR *ddr;	/* image DDR information			     */

FILE 	*fp_rel,	/* file pointer to ".REL" file		             */
	*fp_scn;	/* file pointer to the scan file	             */

long	window[],	/* window to process				     */
	*total_pts,	/* total number of points in the segment             */
	pixval,		/* pixel intensity value for rasterized pixel        */
	linwt,		/* line weight of the rasterized line		     */
	pntint,		/* point interval				     */
	*geo_code,	/* geographic projection code			     */
	*geo_zone,	/* geographic projection zone			     */
	*geo_units;	/* geographic projection units			     */

double  geo_parms[];	/* geographic projection parameters		     */
{
double	lat,		/* temporary latitude in a double		     */
	lon;		/* temporary longitude in a double		     */

float	pnts[100];	/* points in the line				     */

long 	l1, s1,		/* line/sample of first endpoint in line segment     */
	l2, s2;		/* line/sample of second endpoint in line segment    */

long	start,		/* starting line segment point			     */
	last_lat,       /* index of latitude of last point                   */
	last_lng,       /* index of longitude of last point                  */
	num_pts,	/* number of points from current segment to process  */
	pt_num = 1,     /* point number processed                            */
	pts_in_seg,     /* total number of points in segment                 */
	i,		/* looping counters				     */
	status;		/* return status variable			     */

/*
  Save the total number of points in this segment
  and figure out the indices of the last point
*/
pts_in_seg = *total_pts;
last_lat = ((pts_in_seg - 1) % 50) * 2;
last_lng = ((pts_in_seg - 1) % 50) * 2 + 1;

/*
  Read the points to be processed (Maximum of 50 at a time).

  NOTE:  The ".REL" file pointer was set to the correct location when
         reading the ".SEQ" file with the defined rank and area.
*/
status = c_wdbrel(fp_rel,total_pts,&num_pts,pnts);
if (status == E_FAIL)
     return(E_FAIL);

/*
  Set floats into doubles
*/
lat = (double)pnts[0];
lon = (double)pnts[1];

/*
  Calculate the line/sample for the starting point of the line segments
*/
cal_lin_smp(ddr,geo_code,geo_units,geo_zone,geo_parms,
                     &lat,&lon,&l2,&s2);

/*
  Set the start to indicate the start of a new line segment
*/
start = 2*pntint;

/*
  Process all the points of the line segment
*/
while (status != E_EOF)
    {
    /*
      Process the points of the line segments
    */
    for (i = start; i < (num_pts * 2); i += 2*pntint, pt_num += pntint)
	{
	/*
	  Move the end of the line segment to be the beginning of the next
	  line segment
	*/
	l1 = l2;
	s1 = s2;

	/*
	  Convert the latitude/longitude of the next point to a line/sample
	*/
	lat = (double)pnts[i];
	lon = (double)pnts[i+1];
	cal_lin_smp(ddr,geo_code,geo_units,geo_zone,geo_parms,
                     &lat,&lon,&l2,&s2);

	/*
	  Validate the line segment is at least partially in the image
	*/
	if (((l1 < window[SL]) && (l2 < window[SL])) ||
	   ((l1 > (window[SL] + window[NL]-1)) &&
	    (l2 > (window[SL] + window[NL]-1))))
	    continue;

	/*
	  Validate the line segment is at least partially in the image
	*/
	if (((s1 < window[SS]) && (s2 < window[SS])) ||
	    ((s1 > (window[SS] + window[NS] - 1)) &&
	     (s2 > (window[SS] + window[NS]-1))))
	    continue;

	/*
	  Determine the samples to be filled in for this line segment
	*/
	scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt);
	}

    /*
      Read the next set of  points
    */
    status = c_wdbrel(fp_rel,total_pts,&num_pts,pnts);
    if (status == E_FAIL)
	return(E_FAIL);

    /*
      Reset the start to indicate a continuation of the line segment
    */
    start = 0;
    }

/*
  Find out if we still need to process the last point
*/
if (pt_num < pts_in_seg)
    {
    /*
      Move the end of the line segment to be the beginning of the next
      line segment
    */
    l1 = l2;
    s1 = s2;

    /*
      Convert the latitude/longitude of the last point to a line/sample
    */
    lat = (double)pnts[last_lat];
    lon = (double)pnts[last_lng];
    cal_lin_smp(ddr,geo_code,geo_units,geo_zone,geo_parms,
                &lat,&lon,&l2,&s2);

    /*
      If the line segment is at least partially in the image,
      determine the samples to be filled in for this line segment
    */
    if (((l1 >= window[SL]) || (l2 >= window[SL])) &&
       ((l1 <= (window[SL] + window[NL] - 1)) ||
        (l2 <= (window[SL] + window[NL] - 1))))
    if (((s1 >= window[SS]) || (s2 >= window[SS])) &&
       ((s1 <= (window[SS] + window[NS] - 1)) ||
        (s2 <= (window[SS] + window[NS] - 1))))
        scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt);
    }

return(E_SUCC);
}
