/*******************************************************************************
NAME:		FPROCSAT

PURPOSE:	Process a line segment from the World Data Bank
                We quickly process the segment & don't bother to
                check for duplicate segments

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "model.h"
#include "constants.h"

FUNCTION fprocsat(fp_rel,ddr,model,window,lineinc,sampinc,total_pts,
		  pixval,linwt,pntint,fp_scn)

FILE 	*fp_rel,	/* file pointer to ".REL" file		             */
	*fp_scn;	/* file pointer to the scan file	             */

long	window[],	/* window to process				     */
	*total_pts,	/* total number of points in the segment             */
	pixval,		/* pixel intensity value for rasterized pixel        */
	linwt,		/* line weight of the rasterized line		     */
	pntint;		/* which points to skip in the segment		     */

double	lineinc,	/* subsampling factor of lines			     */
	sampinc;	/* subsampling factor of samples		     */

struct DDR     *ddr;	/* DDR information for the image		     */
struct MODEL   *model;	/* model and orbital information		     */
{
double	lat,		/* temporary latitude in a double		     */
	lon,		/* temporary longitude in a double		     */
	elev = 0.0;	/* elevation					     */

float	pnts[100],	/* points in the line                                */
	fline,		/* float value of line at given lat/long	     */
	fsamp;		/* float value of sample at given lat/long	     */

long	l1, s1,		/* line/Sample of first endpoint in line segment     */
	l2, s2;		/* line/Sample of second endpoint in line segment    */

long	start,		/* starting line segment point			     */
	last_lat,	/* index of latitude of last point		     */
	last_lng,	/* index of longitude of last point		     */
	num_pts,	/* number of points from current segment to process  */
	pt_num = 1,	/* point number processed			     */
	pts_in_seg,	/* total number of points in segment		     */
	i, 		/* looping counters				     */
	status;		/* return status variable			     */


/*
  Save the total number of points in this segment
  and figure out the indices of the last point
*/
pts_in_seg = *total_pts;
last_lat = ((pts_in_seg - 1) % 50) * 2;
last_lng = ((pts_in_seg - 1) % 50) * 2 + 1;

/*
  Read the points to be processed (Maximum of 50 at a time).

  NOTE:  The ".REL" file pointer was set to the correct location when
         reading the ".SEQ" file with the defined rank and area.
*/
status = c_wdbrel(fp_rel,total_pts,&num_pts,pnts);
if (status == E_FAIL)
    return(E_FAIL);

/*
  Set floats into doubles
*/
lat = (double)(pnts[0] * RAD_PER_DEG);
lon = (double)(pnts[1] * RAD_PER_DEG);

/*
  Calculate the line/sample for the starting point of the line segments
  c_ll2ls returns line/samp relative to the original image
*/
c_ll2ls(&lat,&lon,&elev,model,&fline,&fsamp,NULL);
/* correct for windowed and/or subsampled image */
l2 = (long)((fline + 0.5 - ddr->master_line) / lineinc);
s2 = (long)((fsamp + 0.5 - ddr->master_sample) / sampinc);

/*
  Set the start to indicate the start of a new line segment
*/
start = 2*pntint;

/*
  Process all the points of the line segment
*/
while (status != E_EOF)
    {
    /*
      Process the points of the line segments
    */
    for (i = start; i < (num_pts * 2); i += 2*pntint, pt_num += pntint)
	{
	/*
	  Move the end of the line segment to be the beginning of the next
	  line segment
	*/
	l1 = l2;
	s1 = s2;

	/*
	  Convert the latitude/longitude of the next point to a line/sample
	*/
	lat = (double)(pnts[i] * RAD_PER_DEG);
	lon = (double)(pnts[i+1] * RAD_PER_DEG);
	c_ll2ls(&lat,&lon,&elev,model,&fline,&fsamp,NULL);
	l2 = (long)((fline + 0.5 - ddr->master_line) / lineinc);
	s2 = (long)((fsamp + 0.5 - ddr->master_sample) / sampinc);

	/*
	  Validate the line segment is at least partially in the image
	*/
	if (((l1 < window[SL]) && (l2 < window[SL])) ||
	   ((l1 > (window[SL] + window[NL] - 1)) &&
	    (l2 > (window[SL] + window[NL] - 1))))
	    continue;

	if (((s1 < window[SS]) && (s2 < window[SS])) ||
	   ((s1 > (window[SS] + window[NS] - 1)) &&
	    (s2 > (window[SS] + window[NS] - 1))))
	    continue;

	/*
	  Determine the samples to be filled in for this line segment
	*/
	scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt);
	}

    /*
      Read the next set of  points
    */
    status = c_wdbrel(fp_rel,total_pts,&num_pts,pnts);
    if (status == E_FAIL)
	return(E_FAIL);

    /*
      Reset the start to indicate a continuation of the line segment
    */
    start = 0;
    }

/*
  Find out if we still need to process the last point
*/
if (pt_num < pts_in_seg)
    {
    /*
      Move the end of the line segment to be the beginning of the next
      line segment
    */
    l1 = l2;
    s1 = s2;

    /*
      Convert the latitude/longitude of the last point to a line/sample
    */
    lat = (double)(pnts[last_lat] * RAD_PER_DEG);
    lon = (double)(pnts[last_lng] * RAD_PER_DEG);
    c_ll2ls(&lat,&lon,&elev,model,&fline,&fsamp,NULL);
    l2 = (long)((fline + 0.5 - ddr->master_line) / lineinc);
    s2 = (long)((fsamp + 0.5 - ddr->master_sample) / sampinc);

    /*
      If the line segment is at least partially in the image,
      determine the samples to be filled in for this line segment
    */
    if (((l1 >= window[SL]) || (l2 >= window[SL])) &&
       ((l1 <= (window[SL] + window[NL] - 1)) ||
        (l2 <= (window[SL] + window[NL] - 1))))
    if (((s1 >= window[SS]) || (s2 >= window[SS])) &&
       ((s1 <= (window[SS] + window[NS] - 1)) ||
        (s2 <= (window[SS] + window[NS] - 1))))
            scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt);
    }

return(E_SUCC);
}
