/*******************************************************************************
NAME:		LLMINMAX

PURPOSE:	Determine the minimum and maximum latitude and longitudes
		of the pass.
  		NOTE: model->sat_view is TRUE for this routine because it
		figures min/max lat/long much like the -LATRANGE subcommand
		of AVHRRIN (using the same supports and logic)

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "wdb2ras.h"

#define SAT_PERIOD 102
#ifndef MIN4
#define MIN4(a,b,c,d) (MIN(a,b) < MIN(c,d) ? MIN(a,b) : MIN(c,d))
#endif
#ifndef MAX4
#define MAX4(a,b,c,d) (LASMAX(a,b) > LASMAX(c,d) ? LASMAX(a,b) : LASMAX(c,d))
#endif

FUNCTION llminmax(model,crossflg,minlat,minlon,maxlat,maxlon)

struct MODEL	*model;	/* model and orbital information		*/

long	*crossflg;	/* indicates crossing Internat'l Date Line	*/

float	*minlat,	/* minimum latitude of the pass			*/
	*minlon,	/* minimum longitude of the pass		*/
	*maxlat,	/* maximum latitude of the pass			*/
	*maxlon;	/* maximum longitude of the pass		*/
{
long	i,		/* loop counter					*/
	orblines,	/* number of lines in an orbit			*/
	maxline,	/* unflipped line of maximum latitude		*/
	minline;	/* unflipped line of minimum latitude		*/

double	lat[4], 	/* latitude values				*/
	lng[4], 	/* maximum longitude values			*/
	elev = 0.0;	/* elevation					*/

float	line,		/* line						*/
	samp1,samp2;	/* sample					*/
float	poslon[4],	/* positive longitudes				*/
	neglon[4];	/* negative longitudes				*/

if (model->dtype == GAC)
    orblines = (long)((SAT_PERIOD * 60.0) * GAC_PER_SEC);
else
    orblines = (long)((SAT_PERIOD * 60.0) * LAC_PER_SEC);

if (model->nlines > orblines)
    {
    *maxlat = 90.0;
    *minlat = -90.0;
    *maxlon = 180.0;
    *minlon = -180.0;
    return;
    }

/*
  Find the minimum and maximum latitude lines
*/
findpolecross(&maxline,&minline,model);
if (maxline < 1) maxline = 1;
if (minline < 1) minline = 1;
if (maxline > model->nlines) maxline = model->nlines;
if (minline > model->nlines) minline = model->nlines;

/*
  Find lat/long at maxline
*/
samp1 = 1.0;
samp2 = (float)model->nsamples;
line = (float)maxline;
c_ls2ll(&line,&samp1,&elev,model,&lat[0],&lng[0],NULL);
c_ls2ll(&line,&samp2,&elev,model,&lat[1],&lng[1],NULL);

/*
  Find lat/long at minline
*/
line = (float)minline;
c_ls2ll(&line,&samp1,&elev,model,&lat[2],&lng[2],NULL);
c_ls2ll(&line,&samp2,&elev,model,&lat[3],&lng[3],NULL);

/*
  Set up initial min/max longitudes
*/
*minlon = (float)(MIN4(lng[0],lng[1],lng[2],lng[3]) * DEG_PER_RAD);
*maxlon = (float)(MAX4(lng[0],lng[1],lng[2],lng[3]) * DEG_PER_RAD);

/*
  Find out if we are crossing International Date Line
*/
if ((*maxlon - *minlon) > 180.0)
    *crossflg = TRUE;
else
    *crossflg = FALSE;

/*
  If crossed Int'l Date Line, we see if wdbII segments are east of the maxlon 
  (the positive longitude) and west of the minlon (the negative longitude).  
  Right now minlon and maxlon are the min and max of the 4 longitudes.  We 
  reset minlon to be the easternmost (biggest) negative longitude and maxlon 
  to be the westmost (smallest) positive longitude.
  e.g.    +170   |  -170		+125   |  +150
	    +---------+           	  +---------+
	    |    |    |           	  |    |    |
	    |    |    |           	  |    |    |
	    +---------+           	  +---------+
          +165   |  -165		+120   |  -170
	maxlon 170; reset to 165	maxlon 150; reset to 120
	minlon -170; reset to -165	minlon -170; ok
*/
if (*crossflg)
    {
    for (i = 0; i < 4; i++)
	{
	if (lng[i] < 0.0)
	    {
	    neglon[i] = lng[i];
	    poslon[i] = 180.0;
	    }
	else
	    {
	    neglon[i] = -180.0;
	    poslon[i] = lng[i];
	    }
	}
    *maxlon =(float)(MIN4(poslon[0],poslon[1],poslon[2],poslon[3])*DEG_PER_RAD);
    *minlon =(float)(MAX4(neglon[0],neglon[1],neglon[2],neglon[3])*DEG_PER_RAD);
    }

/*
  Set up max latitudes
  and reset longitudes if necessary
*/
if ((maxline != 1) && (maxline != model->nlines))
    {
    /* crossed the North Pole */
    *maxlat = 90.0;
    *minlon = -180.0;
    *maxlon = 180.0;
    *crossflg = FALSE;
    }
else
    *maxlat = (float)(max(lat[0],lat[1]) * DEG_PER_RAD);

/*
  Set up max latitudes
  and reset longitudes if necessary
*/
if ((minline != 1) && (minline != model->nlines))
    {
    /* crossed the South Pole */
    *minlat = -90.0;
    *minlon = -180.0;
    *maxlon = 180.0;
    *crossflg = FALSE;
    }
else
    *minlat = (float)(min(lat[2],lat[3]) * DEG_PER_RAD);

return;
}
