/*******************************************************************************
NAME:		PROCPROJ

PURPOSE:	Process a line segment from the World Data Bank

*******************************************************************************/
#include <stdio.h>
#include "las.h"

/*
  When rasterizing WDB2 line work for AVHRR data some of the latitude/longitude
  line segments are mapped to the same line/sample locations.  To prevent
  re-processing most of these duplicate line segments we save the last
  15 line segments and compare those to the current segment.
*/
#define HOLD	15

FUNCTION procproj(ddr,fp_rel,window,total_pts,pixval,linwt,fp_scn,
		  geo_code,geo_zone,geo_units,geo_parms)

struct DDR *ddr;	/* image DDR information			     */

FILE 	*fp_rel,	/* file pointer to ".REL" file		             */
	*fp_scn;	/* file pointer to the scan file	             */

long	window[],	/* window to process				     */
	*total_pts,	/* total number of points in the segment             */
	pixval,		/* pixel intensity value for rasterized pixel        */
	linwt,		/* line weight of the rasterized line		     */
	*geo_code,	/* geographic projection code			     */
	*geo_zone,	/* geographic projection zone			     */
	*geo_units;	/* geographic projection units			     */
double  geo_parms[];	/* geographic projection parameters		     */
{
double	lat,		/* temporary latitude in a double		     */
	lon;		/* temporary longitude in a double		     */

float	pnts[100];	/* points in the line				     */

long 	l1, s1,		/* line/sample of first endpoint in line segment     */
	l2, s2,		/* line/sample of second endpoint in line segment    */
	ll1[HOLD],	/* last X line values of the first endpoints         */
	ls1[HOLD],	/* last X sample values of the first endpoints       */
	ll2[HOLD],	/* last X line values of the second endpoints        */
	ls2[HOLD];	/* last X sample values of the second endpoints      */

long	last = 0,	/* counter to save the last line segment point	     */
	start,		/* starting line segment point			     */
	done,		/* flag to indicate the line segment was already done*/
	num_pts,	/* number of points from current segment to process  */
	i, j,		/* looping counters				     */
	status;		/* return status variable			     */

/*
  Initialize the arrays containing the last X line segments
*/
for (j = 0; j < HOLD; j++)
    {
    ll1[j] = -1;
    ls1[j] = -1;
    ll2[j] = -1;
    ls2[j] = -1;
    }

/*
  Read the points to be processed (Maximum of 50 at a time).

  NOTE:  The ".REL" file pointer was set to the correct location when
         reading the ".SEQ" file with the defined rank and area.
*/
status = c_wdbrel(fp_rel,total_pts,&num_pts,pnts);
if (status == E_FAIL)
     return(E_FAIL);

/*
  Set floats into doubles
*/
lat = (double)pnts[0];
lon = (double)pnts[1];

/*
  Calculate the line/sample for the starting point of the line segments
*/
cal_lin_smp(ddr,geo_code,geo_units,geo_zone,geo_parms,
                     &lat,&lon,&l2,&s2);

/*
  Set the start to indicate the start of a new line segment
*/
start = 2;

/*
  Process all the points of the line segment
*/
while (status != E_EOF)
    {
    /*
      Process the points of the line segments
    */
    for (i = start; i < (num_pts * 2); i += 2)
	{
	/*
	  Move the end of the line segment to be the beginning of the next
	  line segment
	*/
	l1 = l2;
	s1 = s2;

	/*
	  Convert the latitude/longitude of the next point to a line/sample
	*/
	lat = (double)pnts[i];
	lon = (double)pnts[i+1];
	cal_lin_smp(ddr,geo_code,geo_units,geo_zone,geo_parms,
                     &lat,&lon,&l2,&s2);

	/*
	  Validate the line segment is at least partially in the image
	*/
	if (((l1 < window[SL]) && (l2 < window[SL])) ||
	   ((l1 > (window[SL] + window[NL]-1)) &&
	    (l2 > (window[SL] + window[NL]-1))))
	    continue;

	/*
	  Validate the line segment is at least partially in the image
	*/
	if (((s1 < window[SS]) && (s2 < window[SS])) ||
	   ((s1 > (window[SS] + window[NS] - 1)) &&
	    (s2 > (window[SS] + window[NS]-1))))
	    continue;

	/*
	  Determine if the current segment has already been processed
	*/
	done = FALSE;
	for (j = 0; j < HOLD; j++)
	    {
	    if ((l1 == ll1[j]) && (s1 == ls1[j]) &&
		(l2 == ll2[j]) && (s2 == ls2[j]))
		{
		/*
		  Found a segment that has already been processed
		*/
		done = TRUE;
		break;
		}
	    }

	/*
	  Continue to the next point
	*/
	if (done)
	    continue;

	/*
	  Determine the samples to be filled in for this line segment
	*/
	scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt);

	/*
	  Increment the counter to hold the X previous line segment points
	*/
	last++;
	last = last % HOLD;

	/*
	  Save the endpoints of the line segment
	*/
	ll1[last] = l1;
	ls1[last] = s1;
	ll2[last] = l2;
	ls2[last] = s2;
	}

    /*
      Read the next set of  points
    */
    status = c_wdbrel(fp_rel,total_pts,&num_pts,pnts);
    if (status == E_FAIL)
	return(E_FAIL);

    /*
      Reset the start to indicate a continuation of the line segment
    */
    start = 0;
    }

return(E_SUCC);
}
