/*******************************************************************************
NAME:		profindlat

PURPOSE:	Find the lat farthest from the pole
		This is effectively the minlat for North Pole images
		and the maxlat for South Pole images
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "wdb2ras.h"

#ifndef MIN3
#define MIN3(a,b,c) (min(a,b) < c) ? min(a,b) : (c)
#endif

FUNCTION profindlat(flag,ddr,ddr_units,geo_code,geo_zone,geo_units,geo_parms,
		    mylat)

struct DDR *ddr;        /* image DDR information                        */

long 	flag,		/* North (1) or South (-1) Pole			*/
	*ddr_units,	/* input units					*/
	*geo_code,      /* geographic projection code           	*/
        *geo_zone,      /* geographic projection zone           	*/
        *geo_units;     /* geographic projection units          	*/

double  geo_parms[];    /* geographic projection parameters             */

double	*mylat;		/* minimum latitude of the pass			*/
{
long	nonfatal = 1,	/* nonfatal error flag				*/
	rtx,		/* right side sample number			*/
	ltx,		/* left side sample number			*/
	midsamp,	/* midpoint sample for comparing		*/
	midline,	/* midpoint line for comparing			*/
	topline,	/* top line for comparing			*/
	botline,	/* bottom line for comparing			*/
	prtprm[2];	/* Print flags for c_transinit 			*/


double	ullon,		/* upper left longitude				*/
	ullat,		/* upper left latitude				*/
	urlon,		/* upper right longitude			*/
	urlat,		/* upper right latitude				*/
	lllon,		/* lower left longitude				*/
	lllat,		/* lower left latitude				*/
	lrlon,		/* lower right longitude			*/
	lrlat,		/* lower right latitude				*/
	clatr,		/* right lat for comparing			*/
	clatl,		/* left lat for comparing			*/
	clatt,		/* top long for comparing			*/
	clatb,		/* bottom long for comparing			*/
     	llon,		/* longitude of pixel to the left of midpoint	*/
     	llat,		/* latitude of pixel to the left of midpoint	*/
     	rlon,		/* longitude of pixel to the right of midpoint	*/
     	rlat,		/* latitude of pixel to the right of midpoint	*/
     	tlon,		/* longitude of pixel just above midpoint	*/
     	tlat,		/* latitude of pixel just above midpoint	*/
     	blon,		/* longitude of pixel just below midpoint	*/
     	blat,		/* latitude of pixel just below midpoint	*/
	lon,		/* temporary longitude value			*/
	lat,		/* temporary latitude value			*/
	lat1,lat2,lat3,lat4,
	coordy,		/* y-coordinate (line number)			*/
	coordx,		/* x-coordinate (sample number)			*/
	tmpcoord,	/* coords on either side of our point		*/
	max3;		/* maximum of 3 values				*/

/*
  Find the latitude and longitude of the corners of the image
    coordx = ddr->upleft[1] + (samp - 1) * ddr->pdist_x;
    coordy = ddr->upleft[0] - (line - 1) * ddr->pdist_y;
    call ls2ll (midline,1,lat,lon);
*/
c_transinit(&ddr->proj_code,ddr_units,&ddr->zone_code,&(ddr->datum_code),
       ddr->proj_coef,geo_code,geo_units,geo_zone,&(ddr->datum_code),
       geo_parms,prtprm,"\0");

c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->upleft[1],&ddr->upleft[0],&ullon,&ullat);
ullat *= flag;
ullon *= flag;

c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->loleft[1],&ddr->loleft[0],&lllon,&lllat);
lllat *= flag;
lllon *= flag;

c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->upright[1],&ddr->upright[0],&urlon,&urlat);
urlat *= flag;
urlon *= flag;

c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->loright[1],&ddr->loright[0],&lrlon,&lrlat);
lrlat *= flag;
lrlon *= flag;

/*****************************************************************************
  Vary the sample (coordx) across the top line 
  of the image (at ddr->upleft[0])
*****************************************************************************/
/*
  Set up the initial values for comparing
*/
midsamp =  ddr->ns / 2;
coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
clatl = ullat;
clatr = urlat;
ltx = 1;
rtx = ddr->ns;

/*
  Convert the midpoint to a longitude,latitude coordinate
*/
c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&ddr->upleft[0],&lon,&lat);
lat *= flag;
lon *= flag;

if ((lat > clatl) && (lat > clatr))
    lat1 = min(ullat,urlat);
else if ((lat == clatl) && (lat == clatr))
    lat1 = ullat;
else
    {
    /*
      While haven't reached the min/max latitude value
    */
    while ((abs(midsamp - rtx) > 1) && (abs(midsamp - ltx) > 1))
	{
        if ((lat > clatl) && (lat > clatr))
            {
            /*
              Error out if this routine has made an invalid assumption
            */
            c_errmsg ("Cannot calculate min/max latitude","wdb2ras-calc",
		&nonfatal);
	    return(E_FAIL);
            }
        else if ((lat == clatl) && (lat == clatr))
            {
            /*
              If the midpoint is equal to the 2 end points, max has been found
	      Just break; lat1 is correctly set after the while loop
            */
            break;
            }
        else if ((lat <= clatl) && (lat >= clatr))
            {
            /*
              The maximum is to the right of the current midpoint
            */
            clatl = lat;
            ltx = midsamp;
            midsamp = (midsamp + rtx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
            }
        else if ((lat >= clatl) && (lat <= clatr))
            {
            /*
              The maximum is to the left of the current midpoint
            */
            clatr = lat;
            rtx = midsamp;
            midsamp = (midsamp + ltx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
            }
        else if ((lat < clatl) && (lat < clatr))
	    {
            /*
              Move out one pixel at a time to find the direction of the curve
            */
            tmpcoord = ddr->upleft[1] + ((midsamp-1) - 1) * ddr->pdist_x;
            c_trans(&ddr->proj_code,ddr_units,geo_code,geo_units,
		    &tmpcoord,&ddr->upleft[0],&llon,&llat);
	    llat *= flag;
	    llon *= flag;
    
            tmpcoord = ddr->upleft[1] + ((midsamp+1) - 1) * ddr->pdist_x;
            c_trans(&ddr->proj_code,ddr_units,geo_code,geo_units,
			&tmpcoord,&ddr->upleft[0],&rlon,&rlat);
	    rlat *= flag;
	    rlon *= flag;

    	    max3 = MIN3(rlat,llat,lat);
            if (max3 == lat)
	        {
                /*
		  Found high point;
		  lat1 is set after while loop from clatr & clatl (use clatr)
		*/
		clatr = lat;
		clatl = lat;
		break;
	        }
            else if (rlat < lat)
	        {
	        /* Curve rises to the right of the midpoint */
	        clatl = lat;
	        ltx = midsamp;
	        midsamp = (midsamp+1+rtx) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
	        }
            else if (llat < lat)
	        {
	        /* curve rises to the left of the midpoint */
	        clatr = lat;
	        rtx = midsamp;
	        midsamp = (ltx+midsamp+1) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
	        }
	    }
    
        /*
          convert new midpoint to longitude,latitude
        */
        c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       	       &coordx,&ddr->upleft[0],&lon,&lat);
	lat *= flag;
	lon *= flag;
	} /* while */
    /*
      no more pixels in between, pick the maximum 
    */
    lat1 = min(clatr,clatl);
    }

/*****************************************************************************
  Vary the sample (coordx) across the bottom line 
  of the image (at ddr->loright[0])
*****************************************************************************/
/*
  Set up the initial values for comparing
*/
midsamp = ddr->ns / 2;
coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
clatl = lllat;
clatr = lrlat;
ltx = 1;
rtx = ddr->ns;

/*
  Convert the midpoint to a longitude,latitude coordinate
*/
c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &coordx,&ddr->loright[0],&lon,&lat);
lat *= flag;
lon *= flag;

if ((lat > clatl) && (lat > clatr))
    /*
      the curve is increasing, so the corner points are the minimum
    */
    {
    lat2 = min(lllat,lrlat);
    }
else if ((lat == clatl) && (lat == clatr))
    /*
      the latitude is constant across the bottom of the image
    */
    {
    lat2 = lllat;
    }
else
    {
    while ((abs(midsamp - rtx) > 1) && (abs(midsamp - ltx) > 1))
	{
	/*
  	  while haven't reached the minimum latitude value
	*/
	if ((lat > clatl) && (lat > clatr))
    	    {
    	    /*
      	      error out if this routine has made an invalid assumption
    	    */
            c_errmsg ("Cannot calculate minimum latitude","wdb2ras-calc",
		&nonfatal);
	    return(E_FAIL);
    	    }
	else if ((lat == clatl) && (lat == clatr))
    	    {
    	    /*
      	      assume if the midpoint = the 2 end points, the min has been found
    	    */
    	    break;
    	    }
	else if ((lat <= clatl) && (lat >= clatr))
    	    {
    	    /*
      	    The minimum is to the right of the current midpoint
    	    */
    	    clatl = lat;
    	    ltx = midsamp;
    	    midsamp = (midsamp + rtx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
    	    }
	else if ((lat >= clatl) && (lat <= clatr))
    	    {
    	    /*
      	    The minimum is to the left of the current midpoint
    	    */
    	    clatr = lat;
    	    rtx = midsamp;
    	    midsamp = (midsamp + ltx) / 2;
	    coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
    	    }
	else if ((lat < clatl) && (lat < clatr))
    	    {
	    /*
      	      move out one pixel at a time to find the direction of the curve
    	    */
            tmpcoord = ddr->upleft[1] + ((midsamp-1) - 1) * ddr->pdist_x;
	    c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       		   &tmpcoord,&ddr->loright[0],&llon,&llat);
	    llat *= flag;
	    llon *= flag;
    
            tmpcoord = ddr->upleft[1] + ((midsamp+1) - 1) * ddr->pdist_x;
	    c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       	  	   &tmpcoord,&ddr->loright[0],&rlon,&rlat);
	    rlat *= flag;
	    rlon *= flag;
    	    max3 = MIN3(rlat,llat,lat);
    	    if (max3 == lat)
		{
		/* found low point */
    		clatr = lat;
		clatl = lat;
		break;
		}
    	    else if (rlat < lat)
		{
		/* curve falls to the right of the midpoint */
		clatl = lat;
		ltx = midsamp;
		midsamp = (midsamp+1+rtx) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
		}
    	    else if (llat < lat)
		{
		/* curve falls to the left of the midpoint */
		clatr = lat;
		rtx = midsamp;
		midsamp = (ltx+midsamp+1) / 2;
		coordx = ddr->upleft[1] + (midsamp - 1) * ddr->pdist_x;
		}
   	    }
	/*
          convert new midpoint to longitude,latitude
	*/
	c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       	       &coordx,&ddr->loright[0],&lon,&lat);
	lat *= flag;
	lon *= flag;
	} /* while */

    lat2 = min(clatr,clatl);
    }

/*****************************************************************************
  Vary the line (coordy) across the right side 
  of the image (at ddr->loright[1])
*****************************************************************************/
/*
  Set up the initial values for comparing
*/
midline = ddr->nl / 2;
coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
clatt = urlat;
clatb = lrlat;
topline = 1;
botline = ddr->nl;

/*
  Convert the midpoint to a longitude,latitude coordinate
*/
c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->loright[1],&coordy,&lon,&lat);
lat *= flag;
lon *= flag;

if ((lat > clatt) && (lat > clatb))
    /*
      the curve is concave, so the corner points are the greatest
    */
    lat3 = min(urlat,lrlat);
else if ((lat == clatt) && (lat == clatb))
    /*
      the longitude is constant along the right edge of the image
    */
    lat3 = urlat;
else
    {
    /*
      while haven't reached the maximum longitude value
    */
    while ((abs(midline - topline) > 1) && (abs(midline - botline) > 1))
	{
	if ((lat > clatt) && (lat > clatb))
	    {
	    /*
  	      error out if this routine has made an invalid assumption
	    */
            c_errmsg ("Cannot calculate maximum longitude","wdb2ras-calc",
		    &nonfatal);
	    return(E_FAIL);
	    }
	else if ((lat == clatt) && (lat == clatb))
    	    {
    	    /*
      	      assume if the midpoint = the 2 end points, the max has been found
    	    */
    	    break;
    	    }
	else if ((lat <= clatt) && (lat >= clatb))
    	    {
    	    /*
      	      the maximum is below the current midpoint
    	    */
    	    clatt = lat;
    	    topline = midline;
    	    midline = (midline + botline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
    	    }
	else if ((lat >= clatt) && (lat <= clatb))
    	    {
    	    /*
      	      the maximum is above the current midpoint
    	    */
    	    clatb = lat;
    	    botline = midline;
    	    midline = (midline + topline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
    	    }
	else if ((lat < clatt) && (lat < clatb))
    	    {
    	    /*
      	      move out one pixel at a time to find the direction of the curve
    	    */
    	    tmpcoord = ddr->upleft[0] - (midline-1 - 1) * ddr->pdist_y;
            c_trans(&ddr->proj_code,ddr_units,geo_code,geo_units,
			&tmpcoord,&ddr->loright[0],&tlon,&tlat);
	    tlat *= flag;
	    tlon *= flag;
    
    	    tmpcoord = ddr->upleft[0] - (midline+1 - 1) * ddr->pdist_y;
            c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
			&tmpcoord,&ddr->loright[0],&blon,&blat);
	    blat *= flag;
	    blon *= flag;

    	    max3 = MIN3(tlat,blat,lat);
    	    if (max3 == lat)
		{
  		/* found high point */
		clatt = lat;
	        clatb = lat;
		break;
		}
    	    else if (blat < lat)
		{
		/* curve rises below the midpoint */
		clatt = lat;
		topline = midline;
		midline = (midline+1+botline) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
		}
    	    else if (tlon < lon)
		{
		/* curve rises above the midpoint */
		clatb = lat;
		botline = midline;
		midline = (topline+midline+1) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
		}
    	    }

    	/*
      	  convert new midpoint to longitude,latitude
    	*/
	c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
        	&ddr->loright[1],&coordy,&lon,&lat);
	lat *= flag;
	lon *= flag;
    	} /* while */
    /*
      no more pixels in between, pick the maximum 
    */
    lat3 = min(clatt,clatb);
    }

/*****************************************************************************
  Vary the line (coordy) across the left side 
  of the image (at ddr->upleft[1])
*****************************************************************************/
/*
  Set up the initial values for comparing
*/
midline = ddr->nl / 2;
coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
clatt = ullat;
clatb = lllat;
topline = 1;
botline = ddr->nl;

/*
  convert the midpoint to a longitude,latitude coordinate
*/
c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       &ddr->upleft[1],&coordy,&lon,&lat);
lat *= flag;
lon *= flag;

if ((lat > clatt) && (lat > clatb))
    /*
      the curve is convex, so the corner points are the minimum
      lat4 = min(ullon,lllon);
    */
    lat4 = min(ullat,lllat);
else if ((lat == clatt) && (lat == clatb))
    /*
      the longitude is constant along the left side of the image
    */
    lat4 = ullat;
else
    {
    while ((abs(midline - topline) > 1) && (abs(midline - botline) > 1))
	{
        /*
          while haven't reached the minimum longitude value
        */
        if ((lat > clatt) && (lat > clatb))
	    {
	    /*
  	      error out if this routine has made an invalid assumption
	    */
            c_errmsg ("Cannot calculate minimum longitude","wdb2ras-calc",
	        &nonfatal);
	    return(E_FAIL);
	    }
        else if ((lat == clatt) && (lat == clatb))
            {
            /*
              assume if the midpoint = the 2 end points, the min has been found
            */
            clatt = lat;
	    clatb = lat;
            break;
            }
        else if ((lat <= clatt) && (lat >= clatb))
            {
            /*
              the minimum is below the current midpoint
            */
            clatt = lat;
            topline = midline;
            midline = (midline + botline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
            }
        else if ((lat >= clatt) && (lat <= clatb))
            {
            /*
              the minimum is above the current midpoint
            */
            clatb = lat;
            botline = midline;
            midline = (midline + topline) / 2;
	    coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
            }
        else if ((lat < clatt) && (lat < clatb))
            {
            /*
              move out one pixel at a time to find the direction of the curve
            */
    	    tmpcoord = ddr->upleft[0] - (midline-1 - 1) * ddr->pdist_y;
            c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
			&tmpcoord,&ddr->upleft[0],&tlon,&tlat);
	    tlat *= flag;
	    tlon *= flag;
    
    	    tmpcoord = ddr->upleft[0] - (midline+1 - 1) * ddr->pdist_y;
            c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
			&tmpcoord,&ddr->upleft[0],&blon,&blat);
	    blat *= flag;
	    blon *= flag;

    	    max3 = MIN3(tlat,blat,lat);
            if (max3 == lat)
	        {
	        /* found low point */
		clatt = lat;
		clatb = lat;
		break;
	        }
            else if (blat < lat)
	        {
	        /* curve falls below the midpoint */
	        clatt = lat;
	        topline = midline;
	        midline = (midline+1+botline) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
	        }
            else if (tlat < lat)
	        {
	        /* curve falls above the midpoint */
	        clatb = lat;
	        botline = midline;
	        midline = (topline+midline+1) / 2;
		coordy = ddr->upleft[0] - (midline - 1) * ddr->pdist_y;
	        }

	    }
        /*
          convert new midpoint to longitude,latitude
        */
	c_trans(&ddr->proj_code,ddr_units, geo_code,geo_units,
       		&ddr->upleft[1],&coordy,&lon,&lat);
	lat *= flag;
	lon *= flag;
	} /* while */

    lat4 = min(clatt,clatb);
    }

/*
lat1 = flag * min(flag*lat1,flag*lat2);
lat2 = flag * min(flag*lat3,flag*lat4);
*mylat = flag * min(flag*lat1,flag*lat2);
*/
lat1 = min(lat1,lat2);
lat2 = min(lat3,lat4);
*mylat = flag * min(lat1,lat2);

return(E_SUCC);
}
