/*******************************************************************************
NAME:   PROJSEL

PURPOSE:        Select lines from WDBII for placement in the image which is 
		not in satellite projection (use DDR for projection information)

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "proj.h"
#include "wdb2ras.h"
 
FUNCTION projsel(vblock,ddr,area,acount,feature,fcount,ftypes,
                pixval,linwt,window,pntint,msgtim,fp_scn)

struct  PARBLK  *vblock;        /* TAE parameter block                  */
struct  DDR	*ddr;		/* DDR information			*/

FILE    *fp_scn;                /* file pointer to scan file            */

char 	area[][8];              /* area to be rasterized (NA,SA,AS,EU,AF)*/

long    *acount,                /* number of areas to rasterize		*/
        feature[][15],          /* feature classifications to rasterize */
        fcount[],               /* number of sub-levels in each feature */
        ftypes[],               /* feature types selected               */
        pixval,                 /* pixel value of rasterized line       */
        linwt,                 	/* weight of the line to be processed   */
        window[],               /* window to process                    */
        pntint,         	/* point interval                       */
        *msgtim;                /* time between processing messages     */
{

extern char tfiles[3][CMLEN];	/* output temporary host file names	     */
extern long numout;		/* number of output temp files		     */
extern long curent;		/* current "line" for processimg messages    */

/*
  Europe and Africa duplicate level flags
*/
static long	ea[LEVELS+1] = {0,0,0,0,0,0,0};
static long	ns[LEVELS+1] = {0,0,0,0,0,0,0};

/*
  Cover codes for the types of World Data Bank II areas
*/
static char cover[REGIONS][4] = {"NA","SA","AS","EU","AF"};

/*
  Root names of the World Data Bank II files
  North America, South America, Asia, Europe, and Africa
*/
static char wdbfil[REGIONS][LEVELS][4]= {{"na1","na2","na3","am1","am2","NAF"},
				   	 {"sa1","sa2","sa3","am1","am2","sa4"},
				   	 {"as1","as2","as3","as4","as5","as6"},
				   	 {"ea1","ea2","ea3","ea4","ea5","eu1"},
				   	 {"ea1","ea2","ea3","ea4","ea5","af1"}};

/*
  Portion of the processing message for the area being rasterized
*/
static char rtext[REGIONS][16] = {"of North Am. ",
				  "of South Am. ",
				  "of Asian ",
			     	  "of European ",
				  "of African " };

/*
  Portion of the processing message for the level being rasterized
*/
static char ltext[LEVELS+1][36] = {"coastlines, islands, lakes file",
				   "rivers file",
			    	   "international boundaries file",
			    	   "internal boundaries file",
				   "rail roads file",
			    	   "roads file",
				   "user specified file"}; 

FILE	*fp_rel,		/* file pointer to ".REL" WDBII file	*/
	*fp_seq;	   	/* file pointer to ".SEQ" WDBII file    */

float	minlat,		   	/* minumum latitude in the image     	*/
	minlon,		   	/* minumum longitude in the image     	*/
	maxlat,		   	/* maximum latitude in the image     	*/
	maxlon;		   	/* maximum longitude in the image     	*/

long	i, j,		   	/* looping variables		     	*/
	crossflg = FALSE,	/* indicates crossing Internat'l Date Line */
	region,		   	/* current region being processed     	*/
	level,		   	/* current level of data being rasterized*/
	ranks[20],	   	/* WDBII ranking order			*/
	total_pts,	   	/* total number of points in a line segment*/
	zero = 0,	   	/* constant of zero		     	*/
	eight = 8,		/* constant of eight			*/
	nonfatal = 1,	   	/* non-fatal error code		     	*/
	msgfmt = GEN1,	   	/* general message format one	     	*/
	max_entity = 9999, 	/* maximum entity for processing message*/
	status,stat;   	   	/* return status			*/

char	filnam[CMLEN],     	/* name of the file to be rasterized	*/
	ptext[ERRLEN];     	/* processing message text		*/

long	geo_code,		/* projection code			*/
	geo_zone,		/* projection zone			*/
	geo_units,		/* projection units			*/
	geo_datum,		/* projection datum			*/
	ddr_units,		/* projection units from image ddr	*/
	prtprm[2];         	/* print parameters - report destination*/

double 	geo_parms[15],		/* projection parameters		*/
	lon1,lon2,		/* min,max longitude  			*/
	lat1,lat2;		/* min,max latitude 			*/

char	fname = '\0';		/* no destination for report		*/

/*
  Check validity of the projection code, 
  corner projection coordinates, and projection distance
*/
if (ddr->valid[0] == INVAL)
    {
    c_errmsg("Projection code of image is invalid","wdb2ras-pcode",
                 &nonfatal);
    return(E_FAIL);
    }
if (ddr->valid[DDCCV] == INVAL)
    {
    c_errmsg("Corner projection coordinates of image are invalid",
        "wdb2ras-pcoord",&nonfatal);
    return(E_FAIL);
    }
if (ddr->valid[DDPDV] == INVAL)
    {
    c_errmsg("Projection distances per pixel of image are invalid",
        "wdb2ras-pdist",&nonfatal);
    return(E_FAIL);
    }

/*
  Set up projection information
*/
geo_code = GEO;
geo_zone = NULZONE;
geo_units = DEGREES;
geo_datum = ddr->datum_code;	/* input & output datum must be same 	*/
for (i = 0; i < 15; i++)
    geo_parms[i] = 0.0;
status = c_ckunit(ddr->proj_units,&ddr_units);
if (status != E_SUCC)
    {
    c_errmsg("Invalid projection unit","wdb2ras-punit", &nonfatal);
    return(E_FAIL);
    }
prtprm[GEO_TERM] = prtprm[GEO_FILE] = FALSE;

/* 
  Initialize the Projection Transformation Package
*/
status = c_transinit(&geo_code,&geo_units,&geo_zone,&geo_datum,geo_parms,
		  &(ddr->proj_code),&ddr_units,&(ddr->zone_code), 
		  &(ddr->datum_code),ddr->proj_coef,prtprm,&fname);
if (status != E_SUCC)
    return(E_FAIL);

/*
  Determine the min and max latitude and longitude of the image
*/
/* first check to see if it crosses a pole */
status=check_proj_pole(ddr,&ddr_units,&geo_code,&geo_zone,&geo_units,geo_parms);
if (status == NORTH)
    {
    /* if so (e.g. for North Pole) find minlat, use (minlat, 90, -180, 180) */
    profindlat(NORTH,ddr,&ddr_units,&geo_code,&geo_zone,&geo_units,geo_parms,
                   &lat1);
    minlat = (float)lat1;
    maxlat =   90.0;
    minlon = -180.0;
    maxlon =  180.0;
    }
else if (status == SOUTH)
    {
    /* if so (e.g. for South Pole) find maxlat, use (-90, maxlat, -180, 180) */
    profindlat(SOUTH,ddr,&ddr_units,&geo_code,&geo_zone,&geo_units,geo_parms,
                   &lat1);
    minlat =  -90.0;
    maxlat = (float)lat1;
    minlon = -180.0;
    maxlon =  180.0;
    }
else
    {
    /* if no pole use the usual min/max routine */
    status = prominmax(ddr,&ddr_units,&geo_code,&geo_zone,&geo_units,geo_parms,
                   &lat1,&lon1,&lat2,&lon2);
    if (status != E_SUCC)
        return(E_FAIL);

    minlat = (float)lat1;
    maxlat = (float)lat2;
    minlon = (float)lon1;
    maxlon = (float)lon2;

    if ((minlon - maxlon) > 180.0)
        crossflg = TRUE;
    }

/*
  Rasterize each area
*/
for (i = 0; i < *acount; i++)
    {
    /*
      Check to see if the area specified is from the World Data Bank II files
      or a user specified file with the same format as a WDBII file.
    */
    region = -1;
    for (j = 0; j < REGIONS; j++)
	{
	if (strcmp(area[i],cover[j]) == 0)
	    {
	    /*
	      This is a standard WDBII file
	    */
	    region = j;
	    break;
	    }
	}
    c_up2low(area[i],&eight);

    /*
      Rasterize the levels within each area
    */
    for (level = 0; level <= LEVELS; level++)
	{
	/*
	  Check to see if the feature is wanted
	*/
	if (fcount[level] == 0)
	    continue;

	/*
	  A portion of Europe and Africa and a portion of North and South
	  America are in the same file.  If the feature has already been
	  processed using the other area no use waisting time doing it again.
	*/
        if (((strncmp(area[i],"af",2) == 0) || (strncmp(area[i],"eu",2) == 0))
           && (ea[level]))
            continue;
        if (((strncmp(area[i],"na",2) == 0) || (strncmp(area[i],"sa",2) == 0))
           && (ns[level]))
            continue;

	/*
	  Generate the input file name for the area and a processing message
	*/
	if ((region < 0) && (level == LEVELS))
	    {
	    strcpy(filnam,area[i]);
	    sprintf(ptext,"of %s",area[i]);
	    }
	else if (region < 0)
	    {
	    sprintf(filnam,"%s%d",area[i],ftypes[level]);
	    sprintf(ptext,"of %s level %d",area[i],ftypes[level]);
	    }
	else
	    {
	    strcpy(filnam,wdbfil[region][level]);
	    sprintf(ptext,"%s%s",rtext[region],ltext[ftypes[level]-1]);
	    }

	if (strcmp(filnam,"NAF") == 0)
	    {
	    sprintf(ptext,"%s %s does not exist",ltext[ftypes[level]-1],
		    rtext[region]);
	    c_errmsg(ptext,"wdb2ras-nofile",&nonfatal);
	    break;
	    }

	/*
	  Initailize and start the processing message
	*/
	curent = 1;
	c_inittm("WDB2RAS","Processed","record"," ",ptext,&max_entity,&zero,
		 msgtim,&msgfmt);
	c_startm();

	/*
	  Sort the features into WDB2 ranks
	*/
	status = c_wdbrank(ftypes[level],fcount[level],feature[level],ranks);
	if (status != E_SUCC)
	    {
	    fclose(fp_scn);
	    c_clean(vblock,"wdb2ras",tfiles,&numout);
	    }

	/*
	  Open the WDB2 files
	*/
	status = c_wdbopen(filnam,&fp_seq,&fp_rel);
	if (status != E_SUCC)
	    {
	    fclose(fp_scn);
	    c_clean(vblock,"wdb2ras",tfiles,&numout);
	    }

	/*
	  Process all the specified features in the current area
	*/
	status = 1;
	while (status != 0)
	    {
	    /*
	      For the specified rank and a defined latitude/longitude area
	      position the ".REL" file pointer to the starting record
	      location that contains the line segments to be processed.
	    */
	    status = c_wdbseq(fp_seq,ranks,crossflg,&minlat,&minlon,
			      &maxlat,&maxlon,&total_pts,fp_rel);
	    if (status == 0)
		continue;

	    /*
	      Process the segments
	    */
	    if (pntint == 1)
	    	stat = procproj(ddr,fp_rel,window,&total_pts,pixval,linwt,
				fp_scn,&geo_code,&geo_zone,&geo_units,
				geo_parms);
	    else
	    	stat = fprocproj(ddr,fp_rel,window,&total_pts,pixval,linwt,
				 pntint,fp_scn,&geo_code,&geo_zone,&geo_units,
				 geo_parms);
	    if (stat != E_SUCC)
	    	{
	    	fclose(fp_scn);
		c_clean(vblock,"wdb2ras",tfiles,&numout);
		}

	    /*
	      Increment the segment counter
	    */
	    curent++;
	    }

	/*
 	  Close the WDB2 files
	*/
	c_wdbclose(fp_seq,fp_rel);

	/*
	  Stop the processing message
	*/
	c_stoptm();

        /*
          For Europe/Asia CIL,RIV,BDY,PDY,RRR,RDS
          and North/South America PDY,RRR
          flag that this file is already done
        */
        if (((strncmp(area[i],"af",2) == 0) || (strncmp(area[i],"eu",2) == 0))
           && (level < LEVELS))
            ea[level] = TRUE;
        if (((strncmp(area[i],"na",2) == 0) || (strncmp(area[i],"sa",2) == 0))
           && ((level == 3) || (level == 4)))
	    ns[level] = TRUE;

	/*
 	  If no lines were found, go on to the next area
	*/
	if (curent == 1)
	    break;
	}
    }

/*
  Close the scan file
*/
fclose(fp_scn);

return(E_SUCC);
}
