/*******************************************************************************
NAME:   SATSEL

PURPOSE:        Select lines from WDBII for placement in the image

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "wdb2ras.h"

FUNCTION satsel(vblock,ddr,model,hosin,area,acount,feature,fcount,ftypes,
		pixval,linwt,window,pntint,msgtim,fp_scn)

struct  PARBLK	*vblock;	/* TAE parameter block			     */
struct  DDR     *ddr;           /* DDR information                           */
struct  MODEL	*model;	   	/* model and orbital information	     */

FILE	*fp_scn;		/* file Pointer to scan file		     */

char	*hosin,			/* input host file name			     */
	area[][8];		/* area to be rasterized (NA,SA,AS,EU,AF)    */

long	*acount,		/* number of areas to rasterize              */
	feature[][15],		/* feature classifications to rasterize	     */
	fcount[],		/* number of sub-levels in each feature	     */
	ftypes[],		/* feature types selected		     */
	pixval,			/* pixel value of rasterized line	     */
	linwt,			/* weight of the line to be processed	     */
	window[],		/* window to process			     */
	pntint,			/* point intervals 			     */
	*msgtim;		/* time between processing messages	     */
{
extern char tfiles[3][CMLEN];	/* output temporary host file names	     */
extern long numout;		/* number of output temp files		     */
extern long curent;		/* current "line" for processimg messages    */


/*
  Europe/Africa and North/South America duplicate level file flags
*/
static long	ea[LEVELS+1] = {0,0,0,0,0,0,0};
static long	ns[LEVELS+1] = {0,0,0,0,0,0,0};

/*
  Cover codes for the types of World Data Bank II areas
*/
static char cover[REGIONS][4] = {"NA","SA","AS","EU","AF"};

/*
  Root names of the World Data Bank II files
  North America, South America, Asia, Europe, and Africa
*/
static char wdbfil[REGIONS][LEVELS][4]= {{"na1","na2","na3","am1","am2","NAF"},
				   	 {"sa1","sa2","sa3","am1","am2","sa4"},
				   	 {"as1","as2","as3","as4","as5","as6"},
				   	 {"ea1","ea2","ea3","ea4","ea5","eu1"},
				   	 {"ea1","ea2","ea3","ea4","ea5","af1"}};

/*
  Portion of the processing message for the area being rasterized
*/
static char rtext[REGIONS][16] = {"of North Am. ",
				  "of South Am. ",
				  "of Asian ",
			     	  "of European ",
				  "of African " };

/*
  Portion of the processing message for the level being rasterized
*/
static char ltext[LEVELS+1][36] = {"coastlines, islands, lakes file",
				   "rivers file",
			    	   "international boundaries file",
			    	   "internal boundaries file",
				   "rail roads file",
			    	   "roads file",
				   "user specified file"}; 

FILE	*fp_rel,	   /* File Pointer to ".REL" WDBII file		     */
	*fp_seq;	   /* File Pointer to ".SEQ" WDBII file		     */

float	minlat,		   /* Minumum latitude in the image		     */
	minlon,		   /* Minumum longitude in the image		     */
	maxlat,		   /* Maximum latitude in the image		     */
	maxlon;		   /* Maximum longitude in the image		     */

long	i, j,		   /* Looping variables				     */
        crossflg,          /* indicates crossing Internat'l Date Line        */
	region,		   /* Current region being processed		     */
	level,		   /* Current level of data being rasterized	     */
	ranks[20],	   /* WDBII ranking order			     */
	total_pts,	   /* Total number of points in a line segment       */
	zero = 0,	   /* Constant of zero				     */
	eight = 8,	   /* Constant of eight				     */
	nonfatal = 1,	   /* Non-fatal error code			     */
	msgfmt = GEN1,	   /* General message format one		     */
	max_entity = 9999, /* Maximum entity for processing message	     */
	status,stat;   	   /* Return status				     */

char	filnam[CMLEN],     /* Name of the file to be rasterized		     */
	ptext[ERRLEN];     /* Processing message text			     */

double	lineinc,sampinc;

/*
  Retreive the AVHRR DDR when the input image is in satellite prespective
*/
status = c_getaddr(hosin,model);
if (status != E_SUCC)
    {
    fclose(fp_scn);
    c_clean(vblock,"wdb2ras",tfiles,&numout);
    }

/*
  Find out if this image was subsampled
  other than the normal
*/
if (model->dtype == GAC)
    {
    if (ddr->line_inc != 4.0)
        /* divide by the original line increment */
        lineinc = ddr->line_inc / 4.0;
    else
        lineinc = 1.0;
    if (ddr->sample_inc != 5.0)
        /* divide by the original sample increment */
        sampinc = ddr->sample_inc / 5.0;
    else
        sampinc = 1.0;
    }
else
    {
    lineinc = ddr->line_inc;
    sampinc = ddr->sample_inc;
    }

/*
  Determine the min and max latitude and longitude of the image
*/
model->sat_view = TRUE;
llminmax(model,&crossflg,&minlat,&minlon,&maxlat,&maxlon);
model->sat_view = FALSE;

/*
  Set up line/samp of corners relative to the original image
*/

/*
  Rasterize each area
*/
for (i = 0; i < *acount; i++)
    {
    /*
      Check to see if the area specified is from the World Data Bank II files
      or a user specified file with the same format as a WDBII file.
    */
    region = -1;
    for (j = 0; j < REGIONS; j++)
	{
	if (strcmp(area[i],cover[j]) == 0)
	    {
	    /*
	      This is a standard WDBII file
	    */
	    region = j;
	    break;
	    }
	}
    c_up2low(area[i],&eight);

    /*
      Rasterize the levels within each area
    */
    for (level = 0; level <= LEVELS; level++)
	{
	/*
	  Check to see if the feature is wanted
	*/
	if (fcount[level] == 0)
	    continue;

	/*
	  A portion of Europe and Africa and a portion of North and South
	  America are in the same file.  If the feature has already been
	  processed using the other area no use wasting time doing it again.
	*/
	if (((strncmp(area[i],"af",2) == 0) || (strncmp(area[i],"eu",2) == 0)) 
	   && (ea[level]))
	    continue;
	if (((strncmp(area[i],"na",2) == 0) || (strncmp(area[i],"sa",2) == 0)) 
	   && (ns[level]))
	    continue;

	/*
	  Generate the input file name for the area and a processing message
	*/
	if ((region < 0) && (level == LEVELS))
	    {
	    /* user-specified area */
	    strcpy(filnam,area[i]);
	    sprintf(ptext,"of %s",area[i]);
	    }
	else if (region < 0)
	    {
	    /* user-specified area plus feature type */
	    sprintf(filnam,"%s%d",area[i],ftypes[level]);
	    sprintf(ptext,"of %s level %d",area[i],ftypes[level]);
	    }
	else if ((region >= 0) && (level == LEVELS))
	    {
	    /* matched a coverage code but no features entered */
	    sprintf(ptext,"No features were specified for area %s",area[i]);
	    c_errmsg(ptext,"wdb2ras-nofeat",&nonfatal);
	    break;
	    }
	else
	    {
	    /* matched a coverage code and a feature */
	    strcpy(filnam,wdbfil[region][level]);
	    sprintf(ptext,"%s%s",rtext[region],ltext[ftypes[level]-1]);
	    }

	if (strcmp(filnam,"NAF") == 0)
	    /* no file available for NA RDS: see wdbfil[0][5] */
	    {
	    sprintf(ptext,"%s %s does not exist",ltext[ftypes[level]-1],
		    rtext[region]);
	    c_errmsg(ptext,"wdb2ras-nofile",&nonfatal);
	    break;
	    }

	/*
	  Initialize and start the processing message
	*/
	curent = 1;
	c_inittm("WDB2RAS","Processed","record"," ",ptext,&max_entity,&zero,
		 msgtim,&msgfmt);
	c_startm();

	/*
	  Sort the features into WDB2 ranks
	*/
	status = c_wdbrank(ftypes[level],fcount[level],feature[level],ranks);
	if (status != E_SUCC)
	    {
	    fclose(fp_scn);
	    c_clean(vblock,"wdb2ras",tfiles,&numout);
	    }

	/*
	  Open the WDB2 files
	*/
	status = c_wdbopen(filnam,&fp_seq,&fp_rel);
	if (status != E_SUCC)
	    {
	    fclose(fp_scn);
	    c_clean(vblock,"wdb2ras",tfiles,&numout);
	    }

	/*
	  Process all the specified features in the current area
	*/
	status = 1;
	while (status != 0)
	    {
	    /*
	      For the specified rank and a defined latitude/longitude area
	      position the ".REL" file pointer to the starting record
	      location that contains the line segments to be processed.
	    */
	    status = c_wdbseq(fp_seq,ranks,crossflg,&minlat,&minlon,
			      &maxlat,&maxlon,&total_pts,fp_rel);
	    if (status == 0)
		continue;

	    /*
	      Process the segments
	    */
	    if (pntint != 1)
	    	stat = fprocsat(fp_rel,ddr,model,window,lineinc,sampinc,
				&total_pts,pixval,linwt,pntint,fp_scn);
	    else
	    	stat = procsat(fp_rel,ddr,model,window,lineinc,sampinc,
			       &total_pts,pixval,linwt,fp_scn);
	    if (stat != E_SUCC)
	    	{
	    	fclose(fp_scn);
		c_clean(vblock,"wdb2ras",tfiles,&numout);
		}

	    /*
	      Increment the segment counter
	    */
	    curent++;
	    }

	/*
 	  Close the WDB2 files
	*/
	c_wdbclose(fp_seq,fp_rel);

	/*
	  Stop the processing message
	*/
	c_stoptm();

	/*
	  For Europe/Asia CIL,RIV,BDY,PDY,RRR,RDS
	  and North/South America PDY,RRR
	  flag that this file is already done
	*/
	if (((strncmp(area[i],"af",2) == 0) || (strncmp(area[i],"eu",2) == 0)) 
	   && (level < LEVELS))
	    ea[level] = TRUE;
	if (((strncmp(area[i],"na",2) == 0) || (strncmp(area[i],"sa",2) == 0)) 
	   && ((level == 3) || (level == 4)))
	    ns[level] = TRUE;

        /*
          If no lines were found, go on to the next area
        */
        if (curent == 1)
            break;
	}
    }

/*
  Close the scan file
*/
fclose(fp_scn);

return;
}
