/*******************************************************************************
NAME:		SCANI

PURPOSE:	Find the line/sample coordinates to create a segment between
		the two given endpoints

*******************************************************************************/
#include <stdio.h>
#include "las.h"

FUNCTION scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt)

FILE	*fp_scn;	/* file pointer for scan file			*/

long	l1,		/* first y coordinate				*/
	s1,		/* first x coordinate				*/
	l2,		/* second y coordinate				*/
	s2;		/* second x coordinate				*/

long	window[],	/* window to process				*/
	pixval,		/* pixel value for the line			*/ 
	linwt;		/* weight of the line				*/
{
float	slope;		/* inverse of the slope of the line segment	*/
float	xrasti;		/* current sample				*/

long	yhi,		/* high point line value of line segment	*/
	ylo,		/* low point line value of the line segment	*/
	xhi,		/* high point sample value of line segment	*/
	xlo;		/* low point sample value of line segment	*/

long	rastmax,	/* maximum sample value				*/
	rastmin,	/* minimum sample value				*/
	scanmax,	/* maximum scan line value			*/
	scanmin,	/* minimum line value				*/
	xstart,		/* first pixel to turn on			*/
	xstop,		/* Last pixel to turn on			*/
	yscani,		/* current line					*/
	lwidth,		/* width of each line				*/
	temp;		/* temp variable				*/

long	eflag = FALSE,  /* even line weight flag			*/
	prior = 1;	/* priority of the line (all the same)		*/

/*
  Determine the orientation of the line segment
  a smaller line number is further north (higher)
*/
if (l1 < l2)
    {
    /* the line falls */
    yhi = l1;
    ylo = l2;
    xhi = s1;
    xlo = s2;
    }
else
    {
    /* the line rises */
    ylo = l1;
    yhi = l2;
    xlo = s1;
    xhi = s2;
    }

/*
  Calculate line weight
*/
if (linwt % 2 == 0)
    {
    lwidth = linwt / 2;
    eflag = TRUE;
    }
else
    lwidth = (linwt - 1) / 2;

/*
  Calculate the inverse slope of the line segment
*/
if (yhi != ylo)
    slope = (float)( xhi - xlo ) / (float)( yhi - ylo);
else
    slope = 0.0;

/*
  Set start/stop points acording to the line weight
*/
if (eflag == TRUE)
    { 
    scanmin = yhi - lwidth;
    scanmax = ylo + lwidth - 1;

    if (xhi > xlo)
	{
	rastmax = xhi + lwidth - 1;
	rastmin = xlo - lwidth;
	}
    else
	{
	rastmax = xlo + lwidth - 1;
	rastmin = xhi - lwidth;
	}
    }
else
    {
    scanmin = yhi - lwidth;
    scanmax = ylo + lwidth;

    if (xhi > xlo)
	{
	rastmax = xhi + lwidth;
	rastmin = xlo - lwidth;
	}
    else
	{
	rastmax = xlo + lwidth;
	rastmin = xhi - lwidth;
	}
    }

/*
  Validate the range of scan lines to be scanned.  Line segments
  that lie entirely above or below the image are ignored.  Line
  segments that lie partially above or below the image are included.
*/
if ((scanmax < window[SL]) || (scanmin > (window[SL] + window[NL] -1)))
    return;

/*
  Validate the range of pixels to be scanned.  Line segments
  that lie entirely to the left or right of the image are
  ignored.  Line segments that lie partially to the left or
  right of the image are included.
*/
if (((rastmax < window[SS]) && (rastmin < window[SS])) ||
    ((rastmax > (window[SS] + window[NS] - 1)) &&
     (rastmin > (window[SS] + window[NS] - 1))))
    return;

/*
  Scan the segments that intersect only 1 scan line
*/
if (slope == 0.0)
    {
    xstart = rastmin;
    xstop  = rastmax;

    /*
      Check the bounds
    */
    if (scanmin < window[SL])
	scanmin = window[SL];

    if (scanmax > (window[SL] + window[NL] - 1))
	scanmax =  window[SL] + window[NL] - 1;

    if (xstart < window[SS])
	xstart = window[SS];

    if (xstop > (window[SS] + window[NS] - 1))
	xstop =  window[SS] + window[NS] - 1;

    /*
      Write the scan line segment to the scan file
    */
    for (yscani = scanmin; yscani <= scanmax; yscani++)
	{
	fprintf(fp_scn,"%10d %10d %10d %10d %11d\n",
		yscani,prior,xstart,xstop,pixval);
	}
    }
else
    {
    /*
      Set the initial line and sample for the line segment
    */
    yscani = scanmin;
    xrasti = (float)xhi;

    /*
      Find each scan line intersected by this line segment
    */
    do
	{
	/*
	  Make sure the scan line is within the window
	*/
	if (yscani <= (window[SL] + window[NL] - 1))
	    {
	    /*
	      Adjust for even line weight
	    */
	    if (eflag)
		{
		/*
		  Even line weight
		  Determine the scan line segment limits with negative slope
		*/
		if (slope < 0.0)
		    {
		    if (slope < -9.0)
			{
			xstart = (xrasti + slope * lwidth) - lwidth;
			xstop  = (xrasti - slope * lwidth) + lwidth + 1;
			}
		    else
			{
			xstart = (xrasti + slope) - lwidth;
			xstop  = xrasti + lwidth - 1;
			}

		    /*
		      Check the bounds
		    */
		    if (xstart < rastmin)
			xstart = rastmin;
		    if (xstop > rastmax)
			xstop = rastmax;
		    if (xstop < rastmin)
			xstop = rastmin;
		    if (xstart > rastmax)
			xstart = rastmax;
		    }
		else
		    {
		    /*
		      Even line weight
		      Determine the scan line segment limits with positive
		      slopes
		    */
		    if (slope > 9.0)
			{
			xstart = (xrasti - slope * lwidth) - lwidth;
			xstop  = (xrasti + slope * lwidth) + lwidth - 1;
			}
		    else
			{
			xstart = xrasti - lwidth;
			xstop  = (xrasti + slope) + lwidth - 1;
			}

		    /*
		      Check the bounds
		    */
		    if (xstop > rastmax)
			xstop = rastmax;
		    if (xstart < rastmin)
			xstart = rastmin;
		    if (xstart > rastmax)
			xstart = rastmax;
		    if (xstop < rastmin)
			xstop = rastmin;
		    }
		}
	    else
		{
	  	/*
		  Odd line weight
		  Determine the scan line segment limits with negative slope
		*/
		if (slope < 0.0)
		    {
		    if (slope < -9.0)
			{
			xstart = (xrasti + slope * lwidth * 0.6 + slope)-lwidth;
			xstop  = (xrasti - slope * lwidth * 1.6);
			}
		    else
			{
			xstart = xrasti + slope - lwidth;
			xstop  = xrasti + lwidth;
			}

		    /*
		      Check the bounds
		    */
		    if (xstart < rastmin)
			xstart = rastmin;
		    if (xstop > rastmax)
			xstop = rastmax;
		    if (xstop < rastmin)
			xstop = rastmin;
		    if (xstart > rastmax)
			xstart = rastmax;
		    }
		else
		    {
		    /*
		      Odd line weight
		      Determine the scan line segment limits with positive
		      slope
		    */
		    if (slope > 9.0)
			{
			xstart = (xrasti - slope * lwidth * 1.6) - lwidth;
			xstop  = (xrasti + slope * lwidth * 0.6 + slope)+lwidth;
			}
		    else
			{
			xstart = xrasti - lwidth;
			xstop  = xrasti + slope + lwidth;
			}

		    /*
		      Check the bounds
		    */
		    if (xstop > rastmax)
			xstop = rastmax;
		    if (xstart < rastmin)
			xstart = rastmin;
		    if (xstart > rastmax)
			xstart = rastmax;
		    if (xstop < rastmin)
			xstop = rastmin;
		    }
		}

	    /*
	      Determine if the scan line segment intersects
	      the image window
	    */
	    if (yscani >= window[SL])
		{
		/*
		  Order the samples
		*/
		if (xstart > xstop)
		    {
		    temp = xstart;
		    xstart = xstop;
		    xstop = temp;
		    }

		if ((xstart <= (window[SS] + window[NS] - 1)) &&
		    (xstop > 0))
		    {
		    /*
		      Check the bounds
		    */
		    if (xstart < window[SS])
		        xstart = window[SS];

		    if (xstop > (window[SS] + window[NS] - 1))
		        xstop = window[SS] + window[NS] - 1;

		    /*
      		      Write the scan line segment to the scan file
		    */
		    fprintf(fp_scn,"%10d %10d %10d %10d %11d\n",
		    	yscani,prior,xstart,xstop,pixval);
		    }
		}

	    /*
	      Next sample
	    */
	    xrasti += slope;
	    }

	/*
	  Next line
	*/
	yscani++;
	}
    while (yscani <= scanmax);
    }

return;
}
