/*******************************************************************************
NAME:	WDB2RAS

PURPOSE:	Rasterize data from the World Data Bank 2 (WDBII)
		and either create a seperate image that can be used
		as an overlay for the input image or write the line
		work directly into the image.  The input image must 
		also have an associated AVHRR Data Descriptor Record 
		(ADDR) if using the satellite projection, or must have 
		a valid DDR if using another projection.

PROGRAM HISTORY:
Version      Date    Programmer		Request
-------      ----    ----------         -------
  1.0        03/87   L. Huewe		Original development
  2.0        10/91   D. Hollaren/	Conversion to LAS5.0
  	     05/92   T. Rockvam		Added ability to process other 
					projections and subsampled images
					Added "quick" processing of every
					PNTINT'th point
					Added check for International Dat Line
  2.1        05/92   T. Rockvam         Added "catch" for ending point in the
                                        segment; added ability to copy 
					background from IN into OUT, plus lines
  2.2	     02/93   T. Rockvam		Added DATAPATH to system sort for unix

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	Must be run under TAE

PROJECT:	ADAPS

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "model.h"

int fdesc[3];		/* Image file descriptors used for image I/O	*/
int gdesc;		/* Group descriptor used for image I/O		*/
char *thebuf;		/* Image buffer					*/
long ooffset[2];	/* Offset into "thebuf" for output bands	*/
long ioffset[2];	/* Offset into "thebuf" for input bands		*/

char tfiles[3][CMLEN];	/* Temporary host file names			*/
long numout;		/* Number of temp host files			*/

main()
{
extern long curent;	/* Current processing line			*/
struct	PARBLK	vblock;	/* TAE parameter block				*/
struct  DDR     ddr;    /* DDR information                              */
struct  MODEL	model;	/* model and orbital information		*/

FILE	*fp_scn,	/* file pointer to scan file			*/
	*fp_sort;	/* file pointer to sorted scan file		*/

long	acount,		/* number of areas to rasterize			*/
	backgrnd,	/* background value for rasterized image	*/
	imgback = FALSE, /* image data in background flag		*/
	pixval,		/* intensity value for rasterized pixels	*/
	linwt,		/* weight of the rasterized line		*/
	overlay,	/* overlay line work on input image flag	*/
	odtype,		/* output data type				*/
	window[1][4], 	/* window specification for input/output image  */
	bands[1][MAXBND+1],	/* band number to be processed		*/
	nbands[1],	/* number of bands per input image		*/
	std_out,	/* standard output				*/
	pntint,		/* point interval 				*/
	msgtim,		/* time between processing messages		*/
	status,		/* return status variable			*/
	fatal = -1;	/* fatal error code				*/

long	fcount[7],	/* number of options for each feature		*/
	ftypes[7],	/* feature type (CIL,RIV,BDY,PDY,RDS,RRR)	*/
	feature[7][15];	/* the options selected to be rasterized for	*/
			/* each major feature 				*/

char	hosin[1][CMLEN],  /* input host file				*/
	taein[1][CMLEN],  /* input TAE image name			*/
	hosout[1][CMLEN], /* output hos file image name		*/
	taeout[1][CMLEN]; /* output TAE image name			*/

char	proj[6],	/* projection type (satellite or reprojected)	*/
	hisflag[4],	/* history update flag				*/
	mnmx[4],	/* min/max calculation flag			*/
	area[5][8];	/* area to be rasterized (NA - North America,	*/
			/* SA - South America, EU - Europe, AF - Africa	*/
			/* AS - Asia)					*/

/*
  Initialize the TAE parameter block
*/
c_inlas(&vblock,&std_out);

/*
  Initialize the number of output files
*/
numout = 0;

/*
  Get the input parameters
*/
getpar(&vblock,&ddr,area,&acount,feature,fcount,ftypes,taein,hosin,
	taeout,hosout,window,bands,nbands,&odtype,&pixval,&backgrnd,
	&imgback,&linwt,&overlay,proj,&pntint,&msgtim,mnmx,hisflag);

/*
  Create/Open the temporary scan file
*/
open_scan(&fp_scn);

/*
  Scan the lines from the WDB2 for placement in the image
*/
if (strcmp(proj,"SAT") == 0)
    satsel(&vblock,&ddr,&model,hosin[0],area,&acount,feature,fcount,
	   ftypes,pixval,linwt,window[0],pntint,&msgtim,fp_scn);
else
    {
    status = projsel(&vblock,&ddr,area,&acount,feature,fcount,ftypes,
		     pixval,linwt,window[0],pntint,&msgtim,fp_scn);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
    }

/*
  Sort the scan output to line and start sample sequence, with a descending
  stop sample sequence within start sample
*/
sortscan(&vblock);

/*
  Initialize the image I/O interface for the new raster image
*/
inirst(&vblock,hosin[0],hosout[0],window[0],bands[0],&odtype,overlay,imgback,
       &fp_sort);

/*
  Generate the raster image using the sorted scan output or write the
  lines work into the image image
*/
genrst(&vblock,window[0],&odtype,overlay,&backgrnd,imgback,&msgtim,fp_sort);

/*
  Finish up (update associated files)
*/
finish(hosin,hosout,taein,taeout,&vblock,&model,window,bands,nbands,
	proj,overlay,mnmx);

}
