/*******************************************************************************
PURPOSE:        Window a Level 1b file.  See the NOAA Polor Orbital User's
                Guide or the NOAA KLM User's Guide for descriptions of the
		Level 1b formats. 



Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    11/97  T.Baltzer	 Development
  1.1    12/98  D.Lloyd		 Modifications for NOAA KLM series sats.


ALGORITHM:


ALGORITHM/DATA REFERENCES:
	1. NOAA Polor Orbital User's Guide (July 1991) for a description of the 
	   Level 1b format.
	2. NOAA KLM User's Guide for description of KLM series level 1b format


PROJECT:	ADAPS
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "diskio.h"
#include "tapeio.h"
#include "adaps.h"
#include "archio.h"
#include "level1bio.h"
#include "adapsutil.h"
#include "labserv.h"
#include "adapssup.h"
#include "localprotos.h"

#define MAX_VOL       3     /* Max number of tapes to read	*/
#define DISK          1     /* Flag for disk media		*/
#define TAPE         -1     /* Flag for tape media		*/


extern long curent;	    /* Current processing line		*/


int main(void)
{

short *shortptr;	/* Pointer to short, used for scan line count	*/

long long approxfs;	/* Approximate output file size			*/

long curentline,	/* Current processing line			*/
     curscanline,	/* Current scan line count			*/
     dum1 = 0,		/* Dummy variable				*/
     dum2 = 0,		/* Dummy variable				*/
     fflag = GEN1,	/* Processing  message format flag		*/
     first_line = TRUE,	/* First line in window (flag)			*/
     genfil=0,		/* Flag to generate a output file name		*/
     handle = HANDLE,	/* Handle tape errors				*/
     hsize,		/* TBM/ARS header size				*/
     i,			/* Counter					*/
     ichan[NCHANS],	/* Input channels available			*/ 
     in_disk_tape_flg,	/* Flag to tell if reading from tape or disk	*/
     irecsiz,		/* Record size of input data			*/
     itape,		/* Flag for input tape open			*/
     itun = 0,		/* Input tape unit number			*/
     ivolnum,		/* Input tape volume number			*/
     klmflg = FALSE,	/* flag: indicates KLM series spacecraft	*/
     lineswritten = 0,	/* Lines written to the output image		*/
     msgtim=0,		/* Time between processing messages		*/
     one = 1,		/* Value of one					*/
     orecsiz,		/* Record size of output data			*/
     outcnt = 0,	/* Number of output files			*/
     prescanline,	/* Previous scan line count			*/
     scanlinedif,	/* Scan line difference				*/
     skip,		/* Number of files to skip before processing	*/
     skip_drop,		/* Number of drop lines	skipped			*/
     status,		/* Return status of function 			*/
     rdstatus=0,	/* Return read status of function		*/
     std_out=0,		/* Standard output				*/
     uchan[NCHANS],	/* Flags for the user selected channels		*/
     unchans=0,		/* Number of bands user requested be processed  */
     pack_flg,		/* Flag to tell if user asked for packed data	*/
     window[4];		/* Image window					*/

char comment[MAX_VOL][CMLEN],	/* Description of tape			*/
     hosout[CMLEN],		/* Output host image names		*/
     idrive[CMLEN],		/* Input tape drive name		*/
     infile[CMLEN], 		/* Input file				*/
     outnam[2][CMLEN],		/* Output files (2 - 1 output, 1 - temp)*/
     sceneid[CMLEN];		/* Scene id				*/

unsigned char dsh[MAXLINSIZ],	/* DSH header buffer			*/
              ilineptr[MAXLINSIZ],/* Input Level 1b scan line		*/
              olineptr[MAXLINSIZ],/* Ouput Level 1b scan line		*/
              buf[ARS_HDR_SIZ];	/* TBM/ARS header buffer		*/

struct DROPPED in_drop;		/* Dropped line structure		*/
struct MODEL in_model;		/* AVHRR model info (input scene)	*/
struct MODEL out_model;		/* AVHRR model info (new output scene)	*/
struct PARBLK vblock;		/* TAE variable block			*/

FILE *in_fd = NULL;		/* File descriptor to the input file	*/
FILE *out_fd = NULL;		/* File descriptor to output file	*/


/* Initialize LAS
------------------------------------------------------------------------*/
c_inlas(&vblock,&std_out);


/*
  Signals to clean_up function that the image file and tape have not
  been opened yet.
------------------------------------------------------------------------*/
itape = FALSE;


/*
  Get the input parameters
------------------------------------------------------------------------*/
outnam[0][0] = outnam[1][0] = '\0';
status = getpar(&vblock,comment,infile,&skip,hosout,&genfil,&unchans,uchan,
		&msgtim,&pack_flg,outnam,&outcnt,idrive);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,outcnt);


/*
  If infile equals NULL, then read from tape  (COMMENT and IN cannot both
  be NULL - checked in WINDOW1B.PDF).
  Set the flag to tell if reading from tape or disk.
------------------------------------------------------------------------*/
status = strcmp(infile,"\0" );
if (status == 0)
    in_disk_tape_flg = TAPE;
else 
    in_disk_tape_flg = DISK;


/*
  See if the output file already exists - if it does - exit
  (if we already know the output name).  If the output file name must
  be generated, there is no way to precheck its existance.
------------------------------------------------------------------------*/
if (genfil == FALSE ) /* if we already know the output name */
    {
    if (strlen(hosout) < 1)
        {
        status = c_lsstat(hosout,0);
        if (status != E_SUCC)
	    {
            c_lasmsg(FALSE,"window1b-exist","Output file already exists");
	    clean_up(&vblock,in_fd,out_fd,itape,itun,
	             (const char (*)[256])outnam,outcnt);
	    }
        }
    } 


/* If reading from tape, check if files need to be skipped before
   processing begins.
------------------------------------------------------------------------*/
if (skip > 0)
    {
    status = c_l1bskipt(idrive,&skip);
    if (status != E_SUCC)
	clean_up(&vblock,in_fd,out_fd,itape,itun,
	         (const char (*)[256])outnam,outcnt);
    }


/* Open the input Level 1b file, load the AVHRR model, load the headers
   (This model and headers are relative to the entire image).
------------------------------------------------------------------------*/
if (in_disk_tape_flg == DISK)
    status = c_l1bopndr(infile,&in_fd,&irecsiz,buf,dsh,&in_model);
else
    status = c_l1bopntr(idrive,&itun,buf,dsh,&irecsiz,&in_model,&itape);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);
if ( in_model.satnum >= 15 )
    klmflg = TRUE;


/* Determine what the window should be based on what the user specified.
   We are doing this here because we now have the model information
   available to be able to window the image.
------------------------------------------------------------------------*/
status = c_windpar(&vblock,&in_model,window);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);


/* Go to the first record to read within the input Level 1b file
------------------------------------------------------------------------*/
if (in_disk_tape_flg == DISK)
    status = c_l1bseekd(in_fd, window, &irecsiz, &in_model, &skip_drop);
else
    status = c_l1bseekt(&itun, window, &irecsiz, &in_model, comment,
			&ivolnum, &itape, &skip_drop);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);


/* Drop line structure maintainence time:
   If the start of the specified window was bounded by drop lines, then
   update the drop line structure to reflect such.  First initialize the
   drop line structure.
------------------------------------------------------------------------*/
curentline = 0;
in_drop.ngaps = 0;
if (skip_drop > 0)
    {
    in_drop.ngaps = 1;
    in_drop.nmissed[0] = skip_drop;
    if (in_model.cpass == ASCENDING)
	in_drop.lmissed[0] =
	in_model.nlines - (window[SL] + window[NL] - 1) + 1;
    else
	in_drop.lmissed[0] = window[SL];
    curentline = skip_drop;
    }


/* Determine what the output record size should be.
------------------------------------------------------------------------*/
c_l1brecsiz(in_model.dtype,pack_flg,klmflg,unchans,&orecsiz);


/* Allocate disk space for the output file.
   Approximate the output file size.
------------------------------------------------------------------------*/
if (in_model.dtype == GAC && !klmflg)
    approxfs = ((long long)orecsiz / 2) * (window[NL] + 1);
else
    approxfs = (long long)orecsiz * (window[NL] + 1);

status = c_dkcre((char (*)[256])NULL,outnam[0],approxfs,0);
if (status != E_SUCC)
    {
    c_lasmsg(FALSE,"window1b-openout","Error opening output file");
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);
    }


/* Open the output Level 1b file, write NULL headers
   (will always go to disk, never to tape).
------------------------------------------------------------------------*/
if ( klmflg )
    hsize = ARS_HDR_SIZ;
else
    hsize = TBM_HDR_SIZ;
status = c_l1bopndw(outnam[0], &orecsiz, buf, dsh, &hsize, &out_fd);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);


/* Initialize the proccessing message, and start it up
------------------------------------------------------------------------*/
curent = 1;
c_lasmsg(FALSE," ","WINDOW1B processing all lines concurrently for all %d bands"
	 ,unchans);
c_inittm("WINDOW1B","Processed","line"," "," ",&dum1,&dum2,&msgtim,&fflag);
c_startm();
curent = curentline;


/* Loop for each line within the window to be processed
------------------------------------------------------------------------*/
while ( curentline < window[NL] )
    {

    /* Read a line of Level 1b data from the input file
    ------------------------------------------------------------------------*/
    if (in_disk_tape_flg == DISK)
	{
        status = c_l1breadd(in_fd, &irecsiz, ilineptr);
	if (status != E_SUCC)
	    clean_up(&vblock,in_fd,out_fd,itape,itun,
	             (const char (*)[256])outnam,outcnt);
	}
    else
	{
        rdstatus = c_l1breadt(&itun, &irecsiz, comment, &ivolnum, &itape,
			    ilineptr);
	if (rdstatus != TAPE_EOF && rdstatus != TAPE_EOV &&
	    rdstatus != TAPE_EOT && rdstatus != E_SUCC)
	    clean_up(&vblock,in_fd,out_fd,itape,itun,
	             (const char (*)[256])outnam,outcnt);
	}


    /* Drop line structure maintainence time:
       If the scan line count skips lines, we know that drop lines go into
       the image here.  Update the drop line structure to reflect this.
       Update the process line counter also.  If this gap puts us outside
       the specified window, break out of the processing loop.
    ------------------------------------------------------------------------*/
    shortptr = (short *)ilineptr;
    if (first_line)
	{
	prescanline = (long)(*shortptr) - 1;
	first_line = FALSE;
	}
    else
	prescanline = curscanline;
    curscanline = (long)(*shortptr);
    scanlinedif = curscanline - prescanline;
    if (scanlinedif > 1)
	{
	in_drop.nmissed[in_drop.ngaps] = scanlinedif - 1;
	if (in_model.cpass == ASCENDING)
	    in_drop.lmissed[in_drop.ngaps] =
	    in_model.nlines - (window[SL] + window[NL] - (curentline + 1)) + 1;
	else
	    in_drop.lmissed[in_drop.ngaps] = window[SL] + curentline;
        in_drop.ngaps += 1;
	curentline += (scanlinedif - 1);
	curent = curentline;
	if (curentline >= window[NL])  break;
	}


    /* Convert the input line of Level 1b data to a output line of Level 1b data
    ------------------------------------------------------------------------*/
    for (i = 0; i < NCHANS; i++)
	ichan[i] = in_model.chans[i];	
    c_l1b2l1b(irecsiz, ichan, ilineptr, orecsiz, uchan, olineptr);


    /* Write the line of Level 1b data to the output file
       (write to disk only, never to tape).
    ------------------------------------------------------------------------*/
    status = c_l1bwrited(out_fd, olineptr, &orecsiz);
    if (status != E_SUCC)
	clean_up(&vblock,in_fd,out_fd,itape,itun,
	         (const char (*)[256])outnam,outcnt);

    lineswritten++;
    curentline++;
    curent = curentline;

    }  /* end while loop */


/* If we were reading from input tape, we will need to make certain that
   we forward skip to the end of the Level 1b file (if we were windowing
   the file).  If the entire Level 1b file was read, then we don't have
   to do anything.
------------------------------------------------------------------------*/
if (in_disk_tape_flg != DISK)
    {
    if (rdstatus != TAPE_EOF && rdstatus != TAPE_EOV && rdstatus != TAPE_EOT )
	{
	rdstatus = c_tpfsf( &itun, &one, &handle );
	if (rdstatus != TAPE_EOF && rdstatus != TAPE_EOV &&
	    rdstatus != TAPE_EOT && rdstatus != E_SUCC)
	    clean_up(&vblock,in_fd,out_fd,itape,itun,
	             (const char (*)[256])outnam,outcnt);
	}
    }


/* Stop the processing message
------------------------------------------------------------------------*/
c_stoptm();

c_lasmsg(FALSE," ","A total of %d lines were written to the output image.",
	 lineswritten);


/* Compute and create Level 1b headers for the window that was processed
   and write them to the output file.
   The drop line structure "in_drop" has only been defined for the window
   that was processed, not the entire input image.  However, this should
   be sufficient information to compute the headers for the windowed
   output image.
------------------------------------------------------------------------*/
c_lasmsg(FALSE," ","Updating the header information");
status = c_l1buphdr(out_fd, &in_model, &in_drop, window, uchan, &orecsiz);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);


/* Close the input Level 1b file
------------------------------------------------------------------------*/
if (in_disk_tape_flg == DISK)
    status = c_l1bclosd(in_fd);
else
    status = c_l1bclost(&itun, &itape);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);


/* Close the output Level 1b file (always on disk, never on tape).
------------------------------------------------------------------------*/
status = c_l1bclosd(out_fd);
if (status != E_SUCC)
    clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,
             outcnt);


/*
  Generate a file name using the model information when no output 
  file name was specified by the user.  The model must be relative to the
  output file just written, not the input file model (the output image
  may be a window of the input image).  Open the temporary file to load
  the model structure of the output file.
------------------------------------------------------------------------*/
if (genfil == TRUE)
    {
    status = c_l1bopndr(outnam[0],&out_fd,&orecsiz,buf,dsh,&out_model);
    if (status != E_SUCC)
	clean_up(&vblock,in_fd,out_fd,itape,itun,
	         (const char (*)[256])outnam,outcnt);

    c_getscene(&out_model,sceneid);
    strcat(sceneid,".l1b");
    strcat(hosout,sceneid);
    strcat(outnam[1],sceneid);
    }


/*
  free the reserved disk space
*/
c_dkfree(outnam[0]);


/*
  Rename the output file to the sceneid or the user specified name
*/
status = renamefile(outnam[0],outnam[1],FALSE,TRUE,FALSE); 
if ( status != E_SUCC )
   {
   c_lasmsg(FALSE,"window1b-renm","Error renaming the output file");
   clean_up(&vblock,in_fd,out_fd,itape,itun,(const char (*)[256])outnam,outcnt);
   }

/*
  Return the read status flag back to TAE so it knows if we hit
  EOT or EOV.
*/
taeland(&vblock,rdstatus);


/*
  Display a standard completion message
*/
strcpy(outnam[1], hosout);
outcnt = 1;

c_complt("WINDOW1B",(char (*)[256])outnam[1],&outcnt);
return E_SUCC;
}
