/******************************************************************************
FUNCTION:	AUTOGEN

PURPOSE:	Produces a standard list of passes to be acquired for a set of
		NOAA satellites, for a specified period of days and period
		of time for each day.  The list can also be trimmed down such
		that all passes must be over a specified duration.  A script
		file and schedule update file can optionally be generated to 
		create pass-plots for the selected passes and update the 
		master schedule.


PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne	updated to receive the output file names as 
				parameters so their version numbers are already
				known.  This eliminates problems in batch.
5.0	   10/92    T. Rockvam	Updated to LAS 5.0
				Uses new model

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
			None.

PROJECT			ADAPS/DOMSAT
*******************************************************************************/
#include <stdio.h>
#include <Xm/Xm.h>
#include "adaps.h"
#include "autogen.h"
#include "scenes.h"

float circle[NUMPNT][2];	/* target coverage circle		*/
extern void makesecs();

/*
  autogen must convert satno to satnam & satid; open the report file; convert
  strings to longs/floats; call findpasses; close the report file
*/
autogen(repname,qtemp,qkeep,satno,latstring,longstring,
	startdate,starttime,stopdate,stoptime,mintime,vartime,mxdiststr,
	opmodestr,plotflag,errmsg,procMsgText)

long	qtemp,
	qkeep;

long	satno,
	plotflag;

char	*repname,
	*latstring,
	*longstring,
	*startdate,
	*starttime,
	*stopdate,
	*stoptime,
	*mintime,
	*vartime,
	*mxdiststr,
	*opmodestr,
	*errmsg;

Widget procMsgText;
{
FILE	*report;		/* autogen report			*/
Widget  procMsgShell;		/* processing message shell		*/

long	status,			/* function return status		*/
	temp,			/* temporary				*/
	lincnt = 0,		/* report line count			*/
	minsecs,		/* minimum pass length			*/
	varsecs,		/* variance time			*/
	opmode,			/* operation mode 			*/
	sdate[3],		/* user's start date			*/
	edate[3],		/* user's end date			*/
	stime[3],		/* user's start time			*/
	etime[3];		/* user's end time			*/
long	hr,min,sec;
long	totsecs1,
	totsecs2;

char	satnam[8],		/* satellite name 	(NOAA-11)	*/
	satnum[6],		/* satellite number 	(19531)		*/
	satcode[2],		/* satellite code 	(I)		*/
	satid[4];		/* satellite id 	(N11)		*/
static char key[] = "SATNUM";   /* what type of satellite info we want  */

float	target[3],		/* target lat/long			*/
	mxdist;			/* maximum distance (meters)		*/

/*
  Set up the coverage circle for the default target
ccircle();
*/

/*
  Set key values for the satellite being processed
    c_getsat("SATNUM",11,     "NOAA-11", "N11", "19531", "I")
*/
if (c_getsat(key,     &satno, satnam   , satid, satnum,  satcode) != 0)
    {
    strcpy(errmsg,"Invalid satellite");
    return(-1);
    }

/*
  Convert the strings to numeric values
*/
status = sscanf(latstring, "%f",&target[0]);
if ((status != 1) || (target[0] > 90.0) ||  (target[0] < -90.0))
    {
    strcpy(errmsg,"Invalid target latitude");
    return(-1);
    }
target[0] *= RAD_PER_DEG;

status = sscanf(longstring,"%f",&target[1]);
if ((status != 1) || (target[1] > 180.0) ||  (target[1] < -180.0))
    {
    strcpy(errmsg,"Invalid target longitude");
    return(-1);
    }
target[1] *= RAD_PER_DEG;

status = sscanf(startdate, "%d/%d/%d",&sdate[0],&sdate[1],&sdate[2]);
/*
if ((status != 3) || (checkday(sdate) != 0))
    {
    strcpy(errmsg,"Invalid starting date");
    return(-1);
    }
*/
if (status != 3)
    {
    status = sscanf(startdate,"%d",&temp);
    if (status != 1)
	{
    	strcpy(errmsg,"Invalid starting date");
    	return(-1);
	}
    sdate[0] = temp / 10000;
    sdate[1] = (temp / 100) % 100;
    sdate[2] = temp % 100;
    }
if (checkday(sdate) != 0)
    {
    strcpy(errmsg,"Invalid starting date");
    return(-1);
    }

status = sscanf(starttime, "%d:%d:%d",&stime[0],&stime[1],&stime[2]);
/*
if ((status != 3) || (checktim(stime) != 0))
    {
    strcpy(errmsg,"Invalid starting time");
    return(-1);
    }
*/
if (status != 3)
    {
    status = sscanf(starttime,"%d",&temp);
    if (status != 1)
	{
    	strcpy(errmsg,"Invalid starting time");
    	return(-1);
	}
    stime[0] = temp / 10000;
    stime[1] = (temp / 100) % 100;
    stime[2] = temp % 100;
    }
if (checktim(stime) != 0)
    {
    strcpy(errmsg,"Invalid starting time");
    return(-1);
    }

status = sscanf(stopdate,  "%d/%d/%d",&edate[0],&edate[1],&edate[2]);
if (status != 3)
    {
    temp = 0;
    status = sscanf(stopdate,"%d",&temp);
    edate[0] = temp / 10000;
    edate[1] = (temp / 100) % 100;
    edate[2] = temp % 100;
    }
if (checkday(edate) != 0)
    {
    if (status == -1)
        {
        edate[0] = sdate[0];
        edate[1] = sdate[1];
        edate[2] = sdate[2];
        }
    else
	{
        strcpy(errmsg,"Invalid ending date");
        return(-1);
	}
    }

status = sscanf(stoptime,  "%d:%d:%d",&etime[0],&etime[1],&etime[2]);
/*
if ((status != 3) || (checktim(etime) != 0))
    {
    strcpy(errmsg,"Invalid ending time");
    return(-1);
    }
*/
if (status != 3)
    {
    status = sscanf(stoptime,"%d",&temp);
    if (status != 1)
	{
    	strcpy(errmsg,"Invalid ending time");
    	return(-1);
	}
    etime[0] = temp / 10000;
    etime[1] = (temp / 100) % 100;
    etime[2] = temp % 100;
    }
if (checktim(etime) != 0)
    {
    strcpy(errmsg,"Invalid ending time");
    return(-1);
    }

makesecs(sdate,stime,&totsecs1);
makesecs(edate,etime,&totsecs2);
if (totsecs2 < totsecs1)
    {
    strcpy(errmsg,"End time before start time");
    return(-1);
    }

hr = min = sec = 0;
status = sscanf(vartime,  "%d:%d:%d",&hr,&min,&sec);
if ((status != 3) && (status >= 0))
    {
    strcpy(errmsg,"Please enter variance time as hh:mm:ss or NULL");
    return(-1);
    }
varsecs = hr * 3600 + min * 60 + sec;

hr = min = sec = 0;
status = sscanf(mintime,  "%d:%d:%d",&hr,&min,&sec);
if ((status != 3) && (status >= 0))
    {
    strcpy(errmsg,"Please enter minimum time as hh:mm:ss or NULL");
    return(-1);
    }
minsecs = hr * 3600 + min * 60 + sec;
if ((minsecs > (2*varsecs)) && (varsecs != 0))
    {
    strcpy(errmsg,"Minimum pass duration longer than allowed variance about target");
    return(-1);
    }

status = sscanf(mxdiststr,  "%f",&mxdist);
if (status != 1)
    {
    strcpy(errmsg,"Please enter maximum distance in nautical miles");
    return(-1);
    }
mxdist *= M_PER_MILE;	/* convert to meters */

status = sscanf(opmodestr,  "%d",&opmode);
if (status != 1)
    {
    strcpy(errmsg,"Please re-enter operation mode");
    return(-1);
    }

/*
  Set up the coverage circle for the specified target
*/
set_circle(target[0]*DEG_PER_RAD,target[1]*DEG_PER_RAD);

/*
  Open the report file
*/
report = fopen(repname,"a+");
if (report == NULL)
    {
    strcpy(errmsg,"Error opening autogen report file");
    return(-1);
    }

/*
  Write the header for the output report
*/
header(report,&lincnt,satnam,satnum,sdate,stime,edate,etime,
       target[0],target[1]);

/*
  Pop up the processing message dialog
  We do this after parameters are checked in case of an error
*/
XtVaSetValues(procMsgText,XmNvalue,"0",NULL);
procMsgShell = XtParent(XtParent(procMsgText));
XtPopup(procMsgShell,XtGrabNone);
forcedialog(procMsgShell);
/*XFlush(display);*/

/*
  Use the starting and ending times to find the passes we want
*/
status = findpasses(report,qtemp,qkeep,satno,satnam,
	 satnum,target[0],target[1],sdate,stime,edate,etime,minsecs,varsecs,
	 mxdist,opmode,plotflag,procMsgText);
if (status != 0)
    {
    strcpy(errmsg,"Error generating passes");
    return(-1);
    }

/*
  Close the appropriate files
*/
fclose(report);

return(0);
}
