/******************************************************************************
FUNCTION:	calcdist

PURPOSE:	Given the latitudes and longitudes of two points on the globe 
		this function computes the angular distance and distance in 
		meters between the two points.  One point worth mentioning...
		the oblate earth model is approximated by converting latitudes 
		from geodetic to geocentric using:
           		geocentric=atan(((f-1)/f)**2 * tan(geodetic))
     		where f=298.3; (f-1)**2 = RPSQR and (f)**2 = RESQR
     		This simplifies to:
           		geocentric=atan(0.993307*tan(geodetic))
		   or	geocentric=atan(RPSQR / RESQR * tan(geodetic))
     		The formulas used to derive bearing and distance can be found 
		in the 'Reference Data for Radio Engineers' published by 
		Howard W. Sams And Co.  They are commonly referred to as 
		Napier's Analogies.  Formulas for special cases 
		(e.g.  stlat=endlat and stlat=-endlat) were derived from 
		spherical triangle relations found in any good math handbook.

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne
5.0  	   10/92    T.Rockvam	Translated bearing.for to C; fixed an error in 
				the formulas for temp1 and temp2; brought in 
				the time_in_view calculation from pcap.for


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
			None.

PROJECT			ADAPS

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "adaps.h"
#include "ellipsoid.h"
#define GEOCENTRIC(x) atan(RPSQR / RESQR * tan(x))

void calcdist(lat1,lon1,lat2,lon2,mm,angdist,distance,viewtime)

double	lat1, lon1,		/* lat/long of target point (radians)	*/
	lat2, lon2,		/* lat/long of satellite (radians)	*/
	*angdist,		/* angular distance between points (rad)*/
	*distance,		/* distance in meters between points (m)*/
	mm;			/* mean motion of the satellite		*/

long	*viewtime;		/* time the sat is in view (total secs)	*/
{
double	deltalon,		/* difference in longitude		*/
	locerad,		/* local radius of the earth		*/
	z,			/* angular distance			*/
	large,			/* larger latitude			*/
	small,			/* smaller latitude			*/
	temp1,temp2, 		/* temporary angles			*/
	alph,alph0,		/* used to find time in view		*/
	tview;			/* time in view				*/

double 	secs; 		/* minutes & seconds in view		*/

/*
  Find difference in longitude
*/
deltalon = lon2 - lon1;
if (((lon1 * lon2) < 0.0) &&	/* one negative, one positive	*/
    ((fabs(lon1) + fabs(lon2)) > PIE))
    deltalon = PIE2 - fabs(lon1 - lon2);

/*
  Two points along a meridian
*/
if (deltalon == 0.0)
    {
    temp1 = lat2;
    if (fabs(temp1) < HALFPIE)
	temp1 = GEOCENTRIC(temp1);
    temp2 = lat1;
    if (fabs(temp2) < HALFPIE)
	temp2 = GEOCENTRIC(temp2);
    *angdist = fabs(temp1 - temp2);
    }
/*
  Two points on the equator
*/
else if ((lat1 == 0.0) && (lat2 == 0.0))
    {
    *angdist = deltalon;
    }
/*
  Two points on same line of latitude
*/
else if (lat1 == lat2)
    {
    temp2 = lat1;
    if (fabs(temp2) < HALFPIE)
	temp2 = GEOCENTRIC(temp2);
    z = acos(1.0 - cos(temp2) * cos(temp2) * (1.0 - cos(deltalon)));
    *angdist = z;
    }
/*
  Two points on same | latitude |, opp. sides of equator
*/
else if (lat1 == -lat2)
    {
    temp2 = lat1;
    if (fabs(temp2) < HALFPIE)
	temp2 = GEOCENTRIC(temp2);
    z = acos(-1.0 + cos(temp2) * cos(temp2) * (1.0 + cos(deltalon)));
    *angdist = z;
    }

/*
  All other cases
	using Napier's analogies:
  given a, (lat1) 		
        b, (lat2) 		find c, the angular distance between A and B
  and   C  (deltalon)

  use:
    sin (A-B)/2   tan (a-b)/2
    ----------- = -----------
    sin (A+B)/2   tan (c)/2
  		                    sin (A+B)/2)
  then c = 2 * atan ( tan (a-b)/2 * ------------  )
		  		    sin (A-B)/2

  Use tan (A+B)/2    cos (a-b)/2
      ----------- =  -----------    to find (A+B)/2  (= temp1)
      cot (C)/2      cos (a+b)/2

  and tan (A-B)/2    sin (a-b)/2
      ----------- =  -----------    to find (A-B)/2  (= temp2)
      cot (C)/2      sin (a+b)/2
*/
else 
    {
    if ((lat1 > 0) || (lat2 > 0))
	if (lat1 > lat2)
	    {
    	    large = lat1;
    	    small = lat2;
	    }
	else
	    {
    	    large = lat2;
    	    small = lat1;
	    }
    else
	if (lat1 > lat2)
	    {
    	    large = lat2;
    	    small = lat1;
	    }
	else
	    {
    	    large = lat1;
    	    small = lat2;
	    }
    if (fabs(large) < HALFPIE)
        large = GEOCENTRIC(large);
    if (fabs(small) < HALFPIE)
        small = GEOCENTRIC(small);
   temp1 = atan((1.0 / tan(deltalon/2.0)) * 
		 cos((large-small)/2.0) / cos((large+small)/2.0));
    if (temp1 < 0.0)
	temp1 += PIE;
   temp2 = atan((1.0 / tan(deltalon/2.0)) * 
		 (sin((large-small)/2.0) / sin((large + small)/2.0)));
    if (temp2 < 0.0)
	temp2 += PIE;
    z = 2.0 * atan(tan((large-small)/2.0) * (sin(temp1)/sin(temp2)));
    *angdist = fabs(z);
    }

/*
  Make longitude compatible with 'bearing' and compute
  subpoint-to-target distance
	 distance=(local earth radius)*(angular distance in radians)
  where:
    	local earth rad=sqrt((a**2)(1-e**2)/(1-(e**2)(sin(latitude)**2)))
  and
	a=earth equatorial radius
	a**2 = RESQR
	e=earth eccentricity
	e**2 = F
	latitude = average latitude of the 2 points
*/
locerad = sqrt(RESQR * (1.0 - F) / 
	       (1 - (F * sin((lat1+lat2)/2.0) * sin((lat1+lat2)/2.0))));
*distance = *angdist * locerad;

/*
  Find the time the satellite is in view
  taken from version 4.0 pcap.for )
  is section figures the time from point of closest approach to the 
  target (receiving station) point, then multiplies by 2 because 
  time in view extends an equal amount of time on the other side of 
  the target point
*/
temp1 = GEOCENTRIC(lat1);
temp2 = GEOCENTRIC(lat2);
alph  = cos(temp1) * cos(temp2) * cos(lon1-lon2) + sin(temp1) * sin(temp2);
if (alph > 1.0) 
    alph = 1.0;
else if (alph < -1.0) 
    alph = -1.0;
alph = acos(alph);
alph0 = acos((REM/1000)/(REM/1000 + SAT_ALTITUDE));
tview = cos(alph0)/cos(alph);
if (tview > 1.0) 
    tview = 1.0;
else if (tview < -1.0) 
    tview = -1.0;

/* 2x to include time on both sides of the target */
tview = 2.0 * acos(tview);

/* period of satellite (secs) */
secs = PIE2 * (HERG) / mm;

/* time in view (secs) */
tview = secs * tview/PIE2;

*viewtime = (long)(tview + 0.5);
}
