/******************************************************************************
FUNCTION:       readrecorder

PURPOSE:        Read the recorder schedule and write a file of all passes 
		contained, which will be plotted

PROGRAM HISTORY:
Version    Date     Author      Change Request
-------    -----    ---------   --------------
1.0	  	    C. Wivell   Original Development
1.1	   1/93	    T. Rockvam  Switched to C for XPlot
1.2	   12/96    T. Baltzer  Get year from each line instead of header
				because year is sometimes spanned (input file).
1.3	   06/97    T. Baltzer  Get year for XFR case as well as REC
				because XFR could happen before REC.
1.4        09/98    D. Lloyd	Added an offset to the acquisition start and
				end times for XFR case where more than one
				scene has the same XFR time.
1.5	   11/99    D.Lloyd	Modified for new NOAA Polar recorder schedule
				and added the recorder schedule line that
				causes errors to be displayed in the popup
				dialog error box.
1.6        09/00    D.Lloyd	Increased NSATS to 8 for NOAA 16
1.7        11/01    D.Lloyd	Added code to ignore the SAIP case
1.8        04/02    D.Lloyd     Increased NSATS to 9 for NOAA 17
1.9        11/04    D.Lloyd     Increased NSATS to 10 for NOAA 18
1.10       07/09    K.Vanumam   Increased NSATS to 11 for NOAA 19

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT         ADAPS/DOMSAT

ALGORITHM REFERENCES    none

*******************************************************************************/
#include "xplot.h"


#define NSATS 11         /* 11 satellites (9,10,11,12,13,14,15,16,17,18,19)*/
#define NRECS 5				/* 5 recorders			*/
#define NCHNS 2				/* 2 channels			*/
#define NSCNS 5				/* 5 scenes per each		*/

#define W 0
#define F 1

FUNCTION readrecorder(filename,q,chosen_numsats,chosen_satellite,dtypes)

char    *filename;

long	q;
long	chosen_numsats;
long	chosen_satellite[];
long	dtypes;
{
FILE	*rfile;				/* recorder schedule file	*/

struct SCENEINFO this_scene;            /* information about one scene  */

char	temp[120];
char	record[89],			/* a record from the rec. sched */
	chan[2],			/* channel: A or B		*/
	action[10],			/* action: REC,STP,PBK,XFR,AOS  */
	atype[10],			/* type of action START,STOP	*/
	dtype[5],			/* data type LAC/GAC/HRPT 	*/
	cstation[2],			/* station: F or W		*/
	station[10],			/* full station name		*/
	flag[10];			/* flag for start of schedule	*/

long	orbit,				/* orbit number of scene	*/
	hrpton = 0,			/* flag if hrpt is on 		*/
	i,j,k,				/* loop indices			*/
	year,				/* year of schedule		*/
	start_jday[NSATS][NRECS][NCHNS][NSCNS],	/* recorded scene info	*/
	start_hr[NSATS][NRECS][NCHNS][NSCNS],
	start_min[NSATS][NRECS][NCHNS][NSCNS],
	start_sec[NSATS][NRECS][NCHNS][NSCNS],
	stop_jday[NSATS][NRECS][NCHNS][NSCNS],
	stop_hr[NSATS][NRECS][NCHNS][NSCNS],
	stop_min[NSATS][NRECS][NCHNS][NSCNS],
	stop_sec[NSATS][NRECS][NCHNS][NSCNS],
	pbk_start_jday[NSATS][NRECS][NCHNS][NSCNS][2],	/* played back 	*/
	pbk_start_hr[NSATS][NRECS][NCHNS][NSCNS][2],	/* scene info	*/
	pbk_start_min[NSATS][NRECS][NCHNS][NSCNS][2],
	pbk_start_sec[NSATS][NRECS][NCHNS][NSCNS][2],
	pbk_stop_jday[NSATS][NRECS][NCHNS][NSCNS][2],
	pbk_stop_hr[NSATS][NRECS][NCHNS][NSCNS][2],
	pbk_stop_min[NSATS][NRECS][NCHNS][NSCNS][2],
	pbk_stop_sec[NSATS][NRECS][NCHNS][NSCNS][2],
	pbk_nscenes[NSATS][NRECS][NCHNS][2],
	statn,						/* transfer station */
	transfer_jday[NSATS][NRECS][NCHNS],		/* time of transfer */
	transfer_hr[NSATS][NRECS][NCHNS],
	transfer_min[NSATS][NRECS][NCHNS],
	transfer_sec[NSATS][NRECS][NCHNS],
	nrecon[NSATS][NRECS][NCHNS],	/* # scenes REC			    */
	nrecoff[NSATS][NRECS][NCHNS],	/* # scenes STP			    */
	nscenes[NSATS][NRECS][NCHNS],	/* # scenes on the recorder/channel */
	nrec, nstp,
	satno, satind, savsat = -1,
	recorder, channel,
	hrpt_start_jday, hrpt_start_mo, hrpt_start_day,
	hrpt_start_hr=0, hrpt_start_min=0, hrpt_start_sec=0,
	hrpt_stop_jday, hrpt_stop_mo, hrpt_stop_day,
	hrpt_stop_hr, hrpt_stop_min, hrpt_stop_sec,
	trans_day, trans_mo,
	stop_day, stop_mo,
	start_day, start_mo,
	l_jday, l_hour, l_min, l_sec;
long	pdate[3],ptime[3];
long	keep_hrpt,keep_lac,keep_gac;
long	acq_offset;			/* offset from start of acqisition */

float	start_frac_day,
	stop_frac_day,
	fxsjday,
	trans_frac_day,
	djday, dxjday,
	fstop_hr,fstop_min,
	ftrans_hr,ftrans_min,
	maxlen = 1.0/96.0;		/*maximum pass length as part of a day*/
					/* 1/96 = 900/86400 or 15 minutes */
					/* 1/90 = 960/86400 or 16 minutes */

long comp();
void makesecs();
void plot_err();


/*
  Initialize the counter arrays
*/
for (i = 0; i < NSATS; i++)
    {
    for (j = 0; j < NRECS; j++)
	{
	nrecon[i][j][0]  = nrecon[i][j][1]  = 0;
	nrecoff[i][j][0] = nrecoff[i][j][1] = 0;
	nscenes[i][j][0] = nscenes[i][j][1] = 0;
	}
    }
station[0] = '\0';

if ((dtypes / 100) == 1) 
    keep_hrpt = True;
else
    keep_hrpt = False;
if (((dtypes / 10) % 10) == 1)
    keep_lac = True;
else
    keep_lac = False;
if ((dtypes % 10) == 1)
    keep_gac = True;
else
    keep_gac = False;

/*
  Set up constants for all scenes from the recorder schedule
*/
this_scene.where = 'D';
this_scene.opcode = 0;
this_scene.hdtrfile = 99;
this_scene.status = 'N';
this_scene.prodflag = '.';

if (isdir(filename))
    {
    plot_err("Error opening recorder schedule");
    return(-1);
    }

rfile = fopen(filename,"r");
if (rfile == NULL)
    {
    plot_err("Error opening recorder schedule");
    return(-1);
    }

/* 
  Skip to the header line beginning with "COVERING...".
  Do this because we used to extract the year from this line.  We no longer
  extract the year from this line because the file may span a year.
  Consequently, we need to extract the year for each schedule entry.
*/
fgets(record,89,rfile);
sscanf(&record[13],"%8s",flag);
while (strcmp(flag,"COVERING") != 0)
    {
    fgets(record,89,rfile);
    sscanf(&record[13],"%8s",flag);
    }

/*
  Skip 3 lines of header
*/
for (i = 0; i < 3; i++)
    fgets(record,89,rfile);

/*
  Read all other records
*/
while (TRUE)
    {
    /* break when EOF is hit */
    if (fgets(record,89,rfile) == NULL)
        break;
    atype[0] = dtype[0] = action[0] = '\0';
    if (strlen(record) < 41)
    	action[0] = '\0';
    else
        {
	/* replace commas with spaces for the sscanf */
	for ( i = 34; i < strlen(record); i++ )
	    if ( record[i] == ',' ) record[i] = ' ';
	/* get action & data type of this line */
    	sscanf(&record[34],"%s %s %s",action,atype,dtype);
	}

    /* We don't care about RECEIVE actions */
    if ( strcmp(action,"RECEIVE") == 0 )
        action[0] = '\0';
    /* We don't care about XFR,HRPT actions */
    if ( strcmp(action,"XFR") == 0 && strcmp(dtype,"HRPT") == 0 )
	atype[0] = dtype[0] = action[0] = '\0';
    /* An END action type corresponds to the old STP action */
    if ( strcmp(atype,"END") == 0 )
        strcpy( action, "STP" );
    /* reverse the action and data type for CMT actions */
    if ( strcmp(action,"CMT") == 0 )
        sscanf(&record[34],"%s %s %s",action,dtype,atype);

    sscanf(&record[23],"%2d",&satno);
    satind = satno - 9;	/* sats 9,10,11,12,13,14 are array 0,1,2,3,4,5 */

    /* on new satellites reinit stuff */
    if (satno != savsat)
        {
        savsat = satno;
        hrpton = 0;
        for (i = 0; i < NSATS; i++)
            {
            for (j = 0; j < NRECS; j++)
                {
     	        nrecon[i][j][0]  = nrecon[i][j][1]  = 0;
     	        nrecoff[i][j][0] = nrecoff[i][j][1] = 0;
     	        nscenes[i][j][0] = nscenes[i][j][1] = 0;
                for (k = 0; k < NCHNS; k++)
	            pbk_nscenes[i][j][k][0] = pbk_nscenes[i][j][k][1] = 0;
     	       }
            }
        }

    /* don't "stip" or "saip" data at this time */
    if ((strcmp(dtype,"STIP") == 0) || (strcmp(dtype,"SAIP") == 0))
	continue;

    if (strcmp(action,"REC") == 0)
	{
	/* if RECording starts, save recorder start time & keep track 
	of state of the recorder */
        if ((satind < 0) || (satind >= NSATS))
	    {
	    sprintf(temp,"Invalid satellite number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[48],"%1d",&recorder);
	recorder -= 1;	/* subtract 1 to make an array index */
	if ((recorder < 0) || (recorder >= NRECS))
	    {
	    sprintf(temp,"Invalid recorder number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[49],"%1s",chan);
	if (strcmp(chan,"A") == 0)
	    channel = 0;
	else if (strcmp(chan,"B") == 0)
	    channel = 1;
	else
	    {
	    sprintf(temp,"Invalid channel number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }

	/* increment the # of scenes recorded on this recorder & channel */
	nrecon[satind][recorder][channel] += 1;
	nrec = nrecon[satind][recorder][channel]-1;
	nscenes[satind][recorder][channel] += 1;

	/* save recorder date, start time for recorder and channel */
        sscanf( &record[0],"%4d",&year);
	year = year % 100;
        sscanf(&record[5],"%3d",&start_jday[satind][recorder][channel][nrec]);
        sscanf(&record[9],"%2d",&start_hr[satind][recorder][channel][nrec]);
        sscanf(&record[12],"%2d",&start_min[satind][recorder][channel][nrec]);
        sscanf(&record[15],"%2d",&start_sec[satind][recorder][channel][nrec]);
	sscanf(&record[26],"%1s",cstation);
	if (strcmp(cstation,"F") == 0)
    	    strcpy(station,"Fairbanks");
	else if (strcmp(cstation,"W") == 0)
    	    strcpy(station,"Wallops");
	}


    else if (strcmp(action,"STP") == 0)
	{
        if ((satind < 0) || (satind >= NSATS))
	    {
	    sprintf(temp,"Invalid satellite number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[46],"%1d",&recorder);
	recorder -= 1;	/* subtract 1 to make an array index */
	if ((recorder < 0) || (recorder >= NRECS))
	    {
	    sprintf(temp,"Invalid recorder number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[47],"%1s",chan);
	if (strcmp(chan,"A") == 0)
	    channel = 0;
	else if (strcmp(chan,"B") == 0)
	    channel = 1;
	else
	    {
	    sprintf(temp,"Invalid channel number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	if (nrecon[satind][recorder][channel] == 0)
	    continue;

	/* if the recorder is turned off before the transfer of the scene 
	   keep track of the stop times */
	nrecoff[satind][recorder][channel] += 1;
	nstp = nrecoff[satind][recorder][channel]-1;

	/* save recorder stop time for recorder and channel */
	sscanf(&record[5],"%3d",&stop_jday[satind][recorder][channel][nstp]);
	sscanf(&record[9],"%2d",&stop_hr[satind][recorder][channel][nstp]);
	sscanf(&record[12],"%2d",&stop_min[satind][recorder][channel][nstp]);
	sscanf(&record[15],"%2d",&stop_sec[satind][recorder][channel][nstp]);
	}


    else if (strcmp(action,"PBK") == 0)
	{
        if ((satind < 0) || (satind >= NSATS))
	    {
	    sprintf(temp,"Invalid satellite number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[26],"%1s",cstation);
	if (strcmp(cstation,"F") == 0)
	    {
    	    strcpy(station,"Fairbanks");
	    statn = F;
	    }
	else if (strcmp(cstation,"W") == 0)
	    {
    	    strcpy(station,"Wallops");
	    statn = W;
	    }
	else
	    continue;
	sscanf(&record[48],"%1d",&recorder);
	recorder -= 1;	/* subtract 1 to make an array index */
	if ((recorder < 0) || (recorder >= NRECS))
	    {
	    sprintf(temp,"Invalid recorder number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[49],"%1s",chan);
	if (strcmp(chan,"A") == 0)
	    channel = 0;
	else if (strcmp(chan,"B") == 0)
	    channel = 1;
	else
	    {
	    sprintf(temp,"Invalid channel number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }

	/* save all recorded scenes which have been 
	   played back to the receiving station */
	for (nrec = 0; nrec < nscenes[satind][recorder][channel]; nrec++)
	    {
	    pbk_start_jday[satind][recorder][channel][nrec][statn] = 
	        start_jday[satind][recorder][channel][nrec];
	    pbk_start_hr[satind][recorder][channel][nrec][statn] =
	        start_hr[satind][recorder][channel][nrec];
	    pbk_start_min[satind][recorder][channel][nrec][statn] =
	        start_min[satind][recorder][channel][nrec];
	    pbk_start_sec[satind][recorder][channel][nrec][statn] = 
	        start_sec[satind][recorder][channel][nrec];
	    pbk_stop_jday[satind][recorder][channel][nrec][statn] = 
	        stop_jday[satind][recorder][channel][nrec];
	    pbk_stop_hr[satind][recorder][channel][nrec][statn] = 
	        stop_hr[satind][recorder][channel][nrec];
	    pbk_stop_min[satind][recorder][channel][nrec][statn] = 
	        stop_min[satind][recorder][channel][nrec];
	    pbk_stop_sec[satind][recorder][channel][nrec][statn] = 
	        stop_sec[satind][recorder][channel][nrec];
	    }
	pbk_nscenes[satind][recorder][channel][statn] = 
	    nscenes[satind][recorder][channel];

	/* reset counts */
	nrecon[satind][recorder][channel]  = 0;
	nrecoff[satind][recorder][channel] = 0;
	nscenes[satind][recorder][channel] = 0;
	}

    else if (strcmp(action,"CMT") == 0)
	{
        if ((satind < 0) || (satind >= NSATS))
	    {
	    sprintf(temp,"Invalid satellite number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[51],"%6d",&orbit);
	sscanf(&record[26],"%1s",cstation);
	if (strcmp(cstation,"F") == 0)
	    {
    	    strcpy(station,"Fairbanks");
	    statn = F;
	    }
	else if (strcmp(cstation,"W") == 0)
	    {
    	    strcpy(station,"Wallops");
	    statn = W;
	    }

	/* Save the year */
	sscanf( &record[0],"%4d",&year);
	year = year % 100;

	/* if action is CMT and the hrpt pass is on */
	/* then this time is the stop time of the hrpt */
	if (hrpton == 1 && strncmp(atype,"END",3) == 0)
    	    {
    	    /* read hrpt stop time */
    	    sscanf(&record[5],"%3d",&hrpt_stop_jday);
    	    sscanf(&record[9],"%2d",&hrpt_stop_hr);
    	    sscanf(&record[12],"%2d",&hrpt_stop_min);
    	    sscanf(&record[15],"%2d",&hrpt_stop_sec);

    	    /* write out the scene */
    	    hrpton = 0;
    	    c_julgre(&year,&hrpt_start_jday,&hrpt_start_mo,&hrpt_start_day);
    	    c_julgre(&year,&hrpt_stop_jday,&hrpt_stop_mo,&hrpt_stop_day);

    	    /* check length */
    	    start_frac_day= hrpt_start_jday + hrpt_start_hr/24.0 + 
		hrpt_start_min/1440.0 + hrpt_start_sec/86400.0;
    	    stop_frac_day= hrpt_stop_jday + hrpt_stop_hr/24.0 + 
		hrpt_stop_min/1440.0 + hrpt_stop_sec/86400.0;
    	    djday = stop_frac_day - start_frac_day;
    	    if (djday < 0.0) djday += 1.0;
	    /* make sure acq is no more than 15 minutes */
    	    if (djday > maxlen)
		{
		djday = maxlen;
		stop_frac_day = start_frac_day + djday;
		hrpt_stop_jday = (long)stop_frac_day;
		c_julgre(&year,&hrpt_stop_jday,&hrpt_stop_mo,&hrpt_stop_day);
		fstop_hr = (stop_frac_day - hrpt_stop_jday) * 24.0000001;
		hrpt_stop_hr = (long)fstop_hr;
		fstop_min = (fstop_hr - hrpt_stop_hr) * 60.0;
		hrpt_stop_min = (long)fstop_min;
		hrpt_stop_sec = (long)((fstop_min - hrpt_stop_min) * 60.0);
		}
	    sprintf(this_scene.sceneid,"AH%02d%02d%02d%02d%02d%02d%02d",
		    satno,hrpt_start_mo,hrpt_start_day,year,hrpt_start_hr,
		    hrpt_start_min,hrpt_start_sec);
	    pdate[0]=hrpt_start_mo; pdate[1]=hrpt_start_day; pdate[2] = year;
	    ptime[0]=hrpt_start_hr;
	    ptime[1]=hrpt_start_min;
	    ptime[2]=hrpt_start_sec;
	    makesecs(pdate,ptime,&this_scene.acqstart);
	    makesecs(pdate,ptime,&this_scene.passtart);
	    pdate[0]=hrpt_stop_mo; pdate[1]=hrpt_stop_day; pdate[2] = year;
	    ptime[0]=hrpt_stop_hr; 
	    ptime[1]=hrpt_stop_min; 
	    ptime[2]=hrpt_stop_sec;
	    makesecs(pdate,ptime,&this_scene.acqstop);
	    makesecs(pdate,ptime,&this_scene.passtop);
	    this_scene.dtype = HRPT;
	    this_scene.orbit = orbit;
	    if (strcmp(station,"Fairbanks") == 0)
    	    	this_scene.station = FAIRBANKS;
	    else if (strcmp(station,"Wallops") == 0)
    	    	this_scene.station = WALLOPS;
	    for (i = 0; i < chosen_numsats; i++)
                {
            	if ((satno == chosen_satellite[i]) && (keep_hrpt))
                    {
	    	    qinsert(q,&this_scene,comp);
                    break;
                    }
            	}
	    }
	}

    else if (strcmp(action,"XFR") == 0)
	{
        if ((satind < 0) || (satind >= NSATS))
	    {
	    sprintf(temp,"Invalid satellite number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[51],"%6d",&orbit);
	sscanf(&record[26],"%1s",cstation);
	if (strcmp(cstation,"F") == 0)
	    {
    	    strcpy(station,"Fairbanks");
	    statn = F;
	    }
	else if (strcmp(cstation,"W") == 0)
	    {
    	    strcpy(station,"Wallops");
	    statn = W;
	    }
	sscanf(&record[48],"%1d",&recorder);
	recorder -= 1;	/* subtract 1 to make an array index */
	if ((recorder < 0) || (recorder >= NRECS))
	    {
	    sprintf(temp,"Invalid recorder number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }
	sscanf(&record[49],"%1s",chan);
	if (strcmp(chan,"A") == 0)
	    channel = 0;
	else if (strcmp(chan,"B") == 0)
	    channel = 1;
	else
	    {
	    sprintf(temp,"Invalid channel number\n%s",record);
	    plot_err(temp);
	    return(-1);
	    }

	/* if the recorder was not on, then we do not need transfer time */
	/* however we need to check to see if the transfer ends a HRPT scene */
	if ((pbk_nscenes[satind][recorder][channel][statn] == 0) && 
	    (hrpton == 0))
	    continue;

	/* save transfer time for recorder and channel */
	sscanf(&record[5],"%3d",&transfer_jday[satind][recorder][channel]);
	sscanf(&record[9],"%2d",&transfer_hr[satind][recorder][channel]);
	sscanf(&record[12],"%2d",&transfer_min[satind][recorder][channel]);
	sscanf(&record[15],"%2d",&transfer_sec[satind][recorder][channel]);

	/* Save the year */
	sscanf( &record[0],"%4d",&year);
	year = year % 100;

	/* if action is XFR and the hrpt pass is on */
	/* then this time is the stop time of the hrpt */
	if (hrpton == 1)
    	    {
    	    /* read hrpt stop time */
    	    sscanf(&record[5],"%3d",&hrpt_stop_jday);
    	    sscanf(&record[9],"%2d",&hrpt_stop_hr);
    	    sscanf(&record[12],"%2d",&hrpt_stop_min);
    	    sscanf(&record[15],"%2d",&hrpt_stop_sec);

    	    /* write out the scene */
    	    hrpton = 0;
    	    c_julgre(&year,&hrpt_start_jday,&hrpt_start_mo,&hrpt_start_day);
    	    c_julgre(&year,&hrpt_stop_jday,&hrpt_stop_mo,&hrpt_stop_day);

    	    /* check length */
    	    start_frac_day= hrpt_start_jday + hrpt_start_hr/24.0 + 
		hrpt_start_min/1440.0 + hrpt_start_sec/86400.0;
    	    stop_frac_day= hrpt_stop_jday + hrpt_stop_hr/24.0 + 
		hrpt_stop_min/1440.0 + hrpt_stop_sec/86400.0;
    	    djday = stop_frac_day - start_frac_day;
    	    if (djday < 0.0) djday += 1.0;
	    /* make sure acq is no more than 15 minutes */
    	    if (djday > maxlen)
		{
		djday = maxlen;
		stop_frac_day = start_frac_day + djday;
		hrpt_stop_jday = (long)stop_frac_day;
		c_julgre(&year,&hrpt_stop_jday,&hrpt_stop_mo,&hrpt_stop_day);
		fstop_hr = (stop_frac_day - hrpt_stop_jday) * 24.0000001;
		hrpt_stop_hr = (long)fstop_hr;
		fstop_min = (fstop_hr - hrpt_stop_hr) * 60.0;
		hrpt_stop_min = (long)fstop_min;
		hrpt_stop_sec = (long)((fstop_min - hrpt_stop_min) * 60.0);
		}
	    sprintf(this_scene.sceneid,"AH%02d%02d%02d%02d%02d%02d%02d",
		    satno,hrpt_start_mo,hrpt_start_day,year,hrpt_start_hr,
		    hrpt_start_min,hrpt_start_sec);
	    pdate[0]=hrpt_start_mo; pdate[1]=hrpt_start_day; pdate[2] = year;
	    ptime[0]=hrpt_start_hr;
	    ptime[1]=hrpt_start_min;
	    ptime[2]=hrpt_start_sec;
	    makesecs(pdate,ptime,&this_scene.acqstart);
	    makesecs(pdate,ptime,&this_scene.passtart);
	    pdate[0]=hrpt_stop_mo; pdate[1]=hrpt_stop_day; pdate[2] = year;
	    ptime[0]=hrpt_stop_hr; 
	    ptime[1]=hrpt_stop_min; 
	    ptime[2]=hrpt_stop_sec;
	    makesecs(pdate,ptime,&this_scene.acqstop);
	    makesecs(pdate,ptime,&this_scene.passtop);
	    this_scene.dtype = HRPT;
	    this_scene.orbit = orbit;
	    if (strcmp(station,"Fairbanks") == 0)
    	    	this_scene.station = FAIRBANKS;
	    else if (strcmp(station,"Wallops") == 0)
    	    	this_scene.station = WALLOPS;
	    for (i = 0; i < chosen_numsats; i++)
                {
            	if ((satno == chosen_satellite[i]) && (keep_hrpt))
                    {
	    	    qinsert(q,&this_scene,comp);
                    break;
                    }
            	}
	    }
	if ((strcmp(dtype,"LAC") == 0) || (strcmp(dtype,"GAC") == 0))
	    {
	    /* make sure there was at least one scene */
	    if (pbk_nscenes[satind][recorder][channel][statn] == 0)
		continue;
	    dxjday = 0.0;

	    /* Initialize the offset to zero so that first scene
	    starts at acquisition time.
	    ----------------------------------------------------*/
	    acq_offset = 0;

	    for (nrec = 0; nrec < pbk_nscenes[satind][recorder][channel][statn];
			   nrec++)
		{
		c_julgre(&year,
			&pbk_start_jday[satind][recorder][channel][nrec][statn],
		        &start_mo,&start_day);
		c_julgre(&year,
			&pbk_stop_jday[satind][recorder][channel][nrec][statn],
		        &stop_mo,&stop_day);
		c_julgre(&year,&transfer_jday[satind][recorder][channel],
		    &trans_mo,&trans_day);
		start_frac_day =
		 pbk_start_jday[satind][recorder][channel][nrec][statn] +
		 pbk_start_hr[satind][recorder][channel][nrec][statn]/24.0 +
		 pbk_start_min[satind][recorder][channel][nrec][statn]/1440.0 +
		 pbk_start_sec[satind][recorder][channel][nrec][statn]/86400.0;
		stop_frac_day =
		 pbk_stop_jday[satind][recorder][channel][nrec][statn] + 
		 pbk_stop_hr[satind][recorder][channel][nrec][statn]/24.0 +
		 pbk_stop_min[satind][recorder][channel][nrec][statn]/1440.0 +
		 pbk_stop_sec[satind][recorder][channel][nrec][statn]/86400.0;
		trans_frac_day =dxjday+transfer_jday[satind][recorder][channel]+
			 transfer_hr[satind][recorder][channel]/24.0 +
			 transfer_min[satind][recorder][channel]/1440.0 +
			 transfer_sec[satind][recorder][channel]/86400.0;
		djday = stop_frac_day - start_frac_day;
		if (djday < 0.0) djday += 1.0;
		if (strcmp(dtype,"LAC") == 0)
		    dxjday = djday / 2.0;    /* LAC transfers at 2x real rate */
		else
		    dxjday = djday / 19.0;  /* GAC transfers at 19x real rate */
		fxsjday = trans_frac_day + dxjday;
		l_jday = (long)fxsjday;
		ftrans_hr = (fxsjday - l_jday) * 24.0000001;
		l_hour = (long)ftrans_hr;
		ftrans_min = (ftrans_hr - l_hour) * 60.0;
		l_min = (long)ftrans_min;
		l_sec = (long)((ftrans_min-l_min) * 60.0 + 0.5); /* round */
	    	sprintf(this_scene.sceneid,"A%c%02d%02d%02d%02d%02d%02d%02d",
    	    		dtype[0],satno,start_mo,start_day,year,
			pbk_start_hr[satind][recorder][channel][nrec][statn],
			pbk_start_min[satind][recorder][channel][nrec][statn],
			pbk_start_sec[satind][recorder][channel][nrec][statn]);
	 	pdate[0]=trans_mo; pdate[1]=trans_day; pdate[2] = year;
	 	ptime[0]=transfer_hr[satind][recorder][channel];
	 	ptime[1]=transfer_min[satind][recorder][channel];
	 	ptime[2]=transfer_sec[satind][recorder][channel];
	    	makesecs(pdate,ptime,&this_scene.acqstart);

		/* Add the offset to the start time.  For the first scene
		this will be zero.  For subsequent scenes, with the same
		acquisition time, the offset will be the duration of the
		previous scenes.
		---------------------------------------------------------*/
		this_scene.acqstart += acq_offset;

		c_julgre(&year,&l_jday,&trans_mo,&trans_day); /* end day */
	 	pdate[0]=trans_mo; pdate[1]=trans_day; pdate[2] = year;
	 	ptime[0]=l_hour;
	 	ptime[1]=l_min;
	 	ptime[2]=l_sec;
	    	makesecs(pdate,ptime,&this_scene.acqstop);

		/* Add the offset to the acquisition stop time.
		-----------------------------------------------*/
		this_scene.acqstop += acq_offset;

		if (this_scene.acqstop < this_scene.acqstart)
		    this_scene.acqstop += 86400;

		/* Add the scene duration to the offset.
		----------------------------------------*/
		acq_offset += this_scene.acqstop - this_scene.acqstart;

	 	pdate[0]=start_mo; pdate[1]=start_day; pdate[2] = year;
	 	ptime[0]=pbk_start_hr[satind][recorder][channel][nrec][statn];
	 	ptime[1]=pbk_start_min[satind][recorder][channel][nrec][statn];
	 	ptime[2]=pbk_start_sec[satind][recorder][channel][nrec][statn];
	    	makesecs(pdate,ptime,&this_scene.passtart);
	 	pdate[0]=stop_mo; pdate[1]=stop_day; pdate[2] = year;
	 	ptime[0]=pbk_stop_hr[satind][recorder][channel][nrec][statn];
	 	ptime[1]=pbk_stop_min[satind][recorder][channel][nrec][statn];
	 	ptime[2]=pbk_stop_sec[satind][recorder][channel][nrec][statn];
	    	makesecs(pdate,ptime,&this_scene.passtop);
		if (this_scene.passtop < this_scene.passtart)
		    this_scene.passtop += 86400;
	    	if (strcmp(dtype,"GAC") == 0) 
		    this_scene.dtype = GAC;
	    	else 				
		    this_scene.dtype = LAC;
	    	this_scene.orbit = orbit;
	    	if (strcmp(station,"Fairbanks") == 0)
    	    	    this_scene.station = FAIRBANKS;
	    	else
    	    	    this_scene.station = WALLOPS;
	    	for (i = 0; i < chosen_numsats; i++)
                    {
            	    if (satno == chosen_satellite[i])
                    	{
			if (((strcmp(dtype,"GAC") == 0) && (keep_gac)) ||
			    ((strcmp(dtype,"LAC") == 0) && (keep_lac)))
	    	    	    qinsert(q,&this_scene,comp);
                    	break;
                    	}
            	    }
		}
	    } /* LAC/GAC */
	}


    else if (strncmp(action,"AOS",3) == 0)	/* acquisition of signal */
	{
	hrpton = 1;
	sscanf(&record[26],"%1s",cstation);
	if (strcmp(cstation,"F") == 0)
    	    strcpy(station,"Fairbanks");
	else if (strcmp(cstation,"W") == 0)
    	    strcpy(station,"Wallops");
	else if (strcmp(cstation,"L") == 0)
    	    strcpy(station,"Lanion");
	else
	    strcpy(station,"Unknown");

	/* if action equals aos then save this time as the start time  */
	/* of the hrpt pass */
	sscanf(&record[5],"%3d",&hrpt_start_jday);
	sscanf(&record[9],"%2d",&hrpt_start_hr);
	sscanf(&record[12],"%2d",&hrpt_start_min);
	sscanf(&record[15],"%2d",&hrpt_start_sec);
	}


    else
	{
	continue;
	}
    }


fclose(rfile);

return(0);
}
