#ifndef ACQIO_H
#define ACQIO_H


#define MAX_ACQ     11091   /* Maximum AVHRR acquire data size (words)       */

#define MNR_SIZ      103    /* Size of the HRPT minor frame (words)          */
#define TIP_SIZ      520    /* Size of the TIP data (words)                  */
#define LAC_SIZ    10240    /* Size of the HRPT/LAC image data (words)       */
#define GAC_SIZ     2045    /* Size of the GAC image data (words)            */

#define MNR_PCK      140    /* Size of the packed HRPT minor frame (bytes)   */
#define TIP_PCK      696    /* Size of the packed TIP data (bytes)           */
#define LAC_PCK    13656    /* Size of the packed HRPT/LAC image data (bytes)*/
#define GAC_PCK     2728    /* Size of the packed GAC image data (bytes)     */

#define MNR_UNP      206    /* Size of the unpacked HRPT minor frame (bytes) */
#define TIP_UNP     1040    /* Size of the unpacked TIP data (bytes)         */
#define LAC_UNP    20480    /* Size of the unpacked HRPT/LAC image (bytes)   */
#define GAC_UNP     4090    /* Size of the unpacked GAC image data (bytes)   */

#define FULL_LAC_PACK  14788 /* Size of a full LAC data stream packed        */
#define FULL_GAC_PACK   3860 /* Size of a full GAC data stream packed        */
#define FULL_LAC_BPACK 14848 /* Size of full LAC data stream packed & blocked*/
#define FULL_GAC_BPACK  4096 /* Size of full GAC data stream packed & blocked*/
#define KLM_LAC_PACK 15872   /* Size of partially packed NOAA KLM LAC stream */
#define KLM_GAC_PACK 4608    /* Size of partially packed NOAA KLM GAC stream */

#define BIL 1               /* Band Interleved by Line format                */
#define BSQ 2               /* Band Sequential format                        */
#define BIP 3               /* Band Interleved by Pixel format               */

#define MISREG_LIMIT   10   /* Misregistered line limit                      */
#define MISSING_LIMIT 500   /* Missing line limit                            */

#define E_BOF          -4   /* Bottom (beginning) of file                    */

struct ACQDESC
    {
    int fd;                     /* File descriptor to aquire file            */
    char station[4];            /* Station ID                                */
    long img_typ;               /* Type of image data (HRPT, LAC, GAC)       */
    long skip_head;             /* Number of header records to skip          */
    long rec_line;              /* Number of records per line                */
    long byte_line;             /* Number of bytes per line                  */
    long mnr_ptr;               /* Pointer to the start of HRPT minor frame  */
    long mnr_siz;               /* Size of the HRPT minor frame              */
    long tip_ptr;               /* Pointer to the start of TIP data          */
    long tip_siz;               /* Size of the TIP data                      */
    long img_ptr;               /* Pointer to the image data                 */
    long img_siz;               /* Size of the image data                    */
    long img_fmt;               /* Image format (BIP, BIL, BSQ)              */
    long swap_flg;              /* Byte swap indicator                       */
    long flip_flg;              /* Flag to flip data                         */
    char sta_name[25];          /* Station ID name                           */
    };

/*
  Satellite pass information structure
*/
struct SATPASS
    {
    long   clock_err;           /* Satellite clock error (milliseconds)      */
    long   satnum;              /* Satellite number                          */
    long   dtype;               /* Data type  (1)LAC, (2)GAC, (3)HRPT        */
    char   station[4];          /* Station id                                */
    long   month;               /* Month of acquisition                      */
    long   day;                 /* Day of acquisition                        */
    long   year;                /* Year of acquisition                       */
    long   jday;                /* Julian day of acquisition                 */
    long   shour;               /* Starting hour of acquisition              */
    long   smin;                /* Starting minute of acquisition            */
    long   ssec;                /* Starting second of acquisition            */
    long   sfrac;               /* Starting fractional seconds of acq        */
    long   nlines;              /* Number of lines in image                  */
    long   nsamples;            /* Number of samples in image                */
    };


/* Prototype functions to be called by any ADAPS support routine or module   */

int c_acqclose
(
    long *fdesc         /* File descriptor to the acquire image         */
);

int c_acqopen
(
    long *fdesc,        /* File descriptor to acquire image                  */
    char *file,         /* Acquisition file name                             */
    long *access,       /* Type of file access                               */
    char *station,      /* Station the data is from                          */
    long *img_typ,      /* Type of data (HRPT, LAC, or GAC)                  */
    long *tip_flg,      /* Flag to indicate TIP data exits in acquired image */
    long *year,
    struct SATPASS *pass/* Satellite pass information                        */
);

int c_acqread
(
    long *fdesc,                /* File descriptor to the acquire file       */
    short *mnrbuf,              /* Output buffer for HRPT minor frame        */
    short *tipbuf,              /* Output buffer for TIP data                */
    short *imgbuf               /* Output buffer for acquire image data      */
);

int c_acqtmcor
(
    long *satnum,               /* Satellite number                          */
    long *f_month,              /* Month to find                             */
    long *f_day,                /* Day to find                               */
    long *f_year,               /* Year to find                              */
    long *f_hour,               /* Hour to find                              */
    long *f_min,                /* Minute to find                            */
    long *f_sec,                /* Seconds of to find                        */
    long *f_fsec,               /* Fractional seconds to find                */
    long *timcor                /* Time correction                           */
);

int c_bil2bip
(
    short *ibuf,        /* IN  - Input BIL buffer                             */
    short *obuf,        /* OUT - Output BIP buffer                            */
    long *nsamps,       /* IN  - Number of samples in a line                  */
    long bands[],       /* IN  - Selected bands to copy into obuf             */
    long *fill          /* IN  - Zero fill nonselected bands in obuf?         */
);

int c_bip2bil
(
    short *ibuf,        /* IN     - Input BIP buffer                          */
    short *obuf,        /* IN/OUT - Output BIL buffer                         */
    long *nsamps,       /* IN     - Number of samples in a line               */
    long bands[],       /* IN     - Selected bands                            */
    long *fill          /* IN     - Zero fill nonselected bands in obuf       */
);

int c_tipopen
(
    int *fd,            /* File descriptor to the TIP file      */
    char *file,         /* File name of the TIP file            */
    long *access        /* Type of access to the file           */
);

#endif

