#ifndef CALCOEF_H
#define CALCOEF_H


#define MAXLNS  50	/* Maximum number of lines                            */
#define MAXBUF 512	/* Maximum number of bytes in prefix record           */

#define MAXTHERMBND 3	/* Maximum number of thermal bands		      */
#define MAXOPTICBND 3	/* Maximum number of optical bands		      */
#define MINOPTICBND 2	/* Minimum number of optical bands		      */

#define MAXDAT  1024            /* Maximum input data value range       */

/*
  Constant used to calculate the calibration coefficients using Planck's
  function.  (NOAA Poloar Orbiter Data Users Guide - Appendix A)
#define C1	1.1910659E-5
#define C2	1.438833
*/
#define C1      1.1910427e-5
#define C2      1.4387752

/*
  Defines for ccprim() so it can differentiate between LAS and
  archive images
*/
#define LASIMAGE 88
#define ARCIMAGE 99

/*
  Define the master calibration structure
*/
struct CALSTR
    {
    /* USER SPECIFIED PARAMETERS */
    char calmethod[CMLEN];	/* Calibration method (roa,vermote, ...)      */
    long file_type;		/* File type, LASIMAGE or ARCIMAGE	      */
    long calper;		/* Calibration period			      */
    long prtper;		/* # cal periods in PRT period		      */
    long illum;			/* Illumination correction flag		      */
    long temp_range;		/* Estimated temperature range (1,2,3,4)      */
    long rad_opt;		/* reflectance/radiance option		      */
    long lin_opt;		/* non-linear correction option		      */
    long ros_opt;		/* zero radiance of space option	      */
    long odtype;		/* data type of output image		      */
    char scale_opt[CMLEN];	/* scale option (US, GLOBAL)		      */
    /* GENERAL PARAMETERS */
    long lmo;			/* Launch month for the satellite	      */
    long lyr;			/* Launch years for the satellite	      */
    /* THERMAL PARAMETERS */
    double aij[4][5];		/* PRT Calibration Factors		      */
    double bi[4];		/* PRT Weighting Factors		      */
    double nsp[MAXTHERMBND];	/* Space Radiance Correction		      */
    double nu1[MAXTHERMBND];	/* nu1 from normalized response		      */
    double dltanu[MAXTHERMBND];	/* delta nu				      */
    long n[MAXTHERMBND];	/* number of elements in phihat		      */
    double phihat[MAXTHERMBND][60]; /* Numerator of beta funct phihat*deltanu */
    double rats_vc[MAXTHERMBND][4];/*   Rad to temp coeff: central wave number*/
    double rats_A[MAXTHERMBND];	/* Radiance to temp coeff: A coefficient      */
    double rats_B[MAXTHERMBND];	/* Radiance to temp coeff: B coefficient      */
    double corr[MAXTHERMBND][3];/* non-linear correction coefficients	      */
    /* OPTICAL PARAMETERS */
    double vsgain[MAXOPTICBND][2];/* Low/high albedo gain published by NOAA   */
    double vsntcp[MAXOPTICBND][2];/* Low/high albedo intercept pub by NOAA    */
    long vscnt[MAXOPTICBND];	  /* Intersection count for low/high albedo   */
    double C0[MAXOPTICBND][2];	  /* Counts of space			      */
    double Kvals[MAXOPTICBND];    /* K values for this satellite	      */
    double rd2rf_w[MAXOPTICBND];  /* Radiance to reflectance: equivalent width*/
    double rd2rf_l[MAXOPTICBND];  /* Rad to ref: effective central wavelength */
    double rd2rf_F[MAXOPTICBND];  /* Rad to ref: in-band solar irradiance     */
    /* "handy to know" parameters */
    long reconfig;		/* Sensor reconfiguration flag, TRUE or FALSE */
    long nopticbnds;		/* Number of optic bands in sensor config     */
    long nthermbnds;		/* Number of thermal bands in sensor config   */
    long nprdflg;		/* Start of new calibration period, TRUE/FALSE*/
    };

/*
  Define the structure to contain the coefficients.
  OPTICAL CHANNELS:  Some satellites have only one albedo value which
  is applied to all raw counts. For other satellites, there is a low
  albedo coefficient to be applied to a low raw count, and a high
  albedo coefficient to be applied to a high raw count.  The raw count
  at which to switch from low to high coefficients is defined by "count".
  THERMAL CHANNELS: For some satellites, there are only order0 and order1
  coefficients (linear function).  For other satellites, there are order0,
  order1, and order2 coefficients (quadratic function).
*/
struct OCOEFFSTR
    {
    double order0[MAXOPTICBND][2]; /* 0th order coefficients (high/low albedo)*/
    double order1[MAXOPTICBND][2]; /* 1st order coefficients (high/low albedo)*/
    long count[MAXOPTICBND];	/* Raw count at which to switch from low to   */
				/* high albedo values			      */
    };
struct TCOEFFSTR
    {
    double order0[MAXTHERMBND]; /* 0th order coefficients */
    double order1[MAXTHERMBND]; /* 1st order coefficients */
    double order2[MAXTHERMBND]; /* 2nd order coefficients */
    };
struct COEFFSTR
    {
    struct OCOEFFSTR ocoeff;
    struct TCOEFFSTR tcoeff;
    };

/* Structure used for storing scaling quantities */
/*-----------------------------------------------*/
struct SCALESTR
   {
   double minval;               /* Minimum value of appropriate quantity */
   double maxval;               /* Maximum value of appropriate quantity */
   double scale;                /* Scale value to apply                  */
   double offset;               /* Offset value to apply                 */
   };


#endif
