/****************************************************************************
NAME:           ACQ_FRAME

PURPOSE:        Check for frame sync shift registration.  Some times the
                data stream gets shifted during acquisition.  If the
                acquisition procedures don't take care of the shifting
                the first 6 words of data (frame sync) should be checked
                and the input buffer adjusted.  The first two words of
                the frame sync are looked at; sloppy but a bit faster!

                NOTE:  This is currently handled by EDC acquire.

PROGRAM HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  5.0    D. Hollaren   02/92    Original Development

PROJECT:  ADAPS

ALGORITHM REFERENCES:
        See "NOAA Technical Memorandum NESS 107" for a detailed
        description of the HRPT minor frame format.
*****************************************************************************/
/* This comment added for a test of Y2K-Compliance branch tag in cvs */
#include <stdio.h>
#include "worgen.h"
#include "tae.h"
#include "acqio.h"

int acq_frame
(
    short *inbuf,       /* Buffer to check for fame sync errors         */
    long *bufsiz,       /* Size of the buffer to check                  */
    long *line,         /* Current line being processed                 */
    long *nbad          /* Number of bad frame sync error lines found   */
)
{
long    shift,          /* Shift offset found                           */
        i;              /* Looping counter                              */

long    nonfatal = 1;   /* Nonfatal error code                          */

char    msg[ERRLEN];    /* Error message                                */

/*
  Check for frame sync shift mis-registration.
*/
if ((inbuf[0] != 0x284) && (inbuf[1] != 0x16f))
    {
    /*
      Find the shift offset
    */
    shift = 0;
    for (i = 0; i < *bufsiz; i++)
        {
        if ((inbuf[0] == 0x284) && (inbuf[1] == 0x16f) && 
            (inbuf[2] == 0x35c) && (inbuf[3] == 0x19d))
            {
            shift = i;
            break;
            }
        }

    /*
      Realign the buffer
    */
    if (shift > 0)
        {
        sprintf(msg,"Frame sync error at line %ld",*line);
        c_errmsg(msg,"acq_frame-frame",&nonfatal);

        for (i = 0; i < *bufsiz - shift; i++)
            inbuf[i] = inbuf[i + shift];

        /*
          Fill the shifted words with zero's; for lack of anything better
        */
        for (i = *bufsiz - shift; i < *bufsiz; i++)
            inbuf[i] = 0;
        }
    else
        {
        sprintf(msg,"Frame sync did not match; bad line at %ld",*line);
        c_errmsg(msg,"acq_frame-frame",&nonfatal);

        /*
          Bad line!  If there aren't to many bad lines forget about this
          one and try to read the next one.
        */
        *nbad++;
        if (*nbad > MISREG_LIMIT)
            {
            sprintf(msg,"%ld lines misregistered at line %ld %s %d",
                    *nbad,*line,"exceeds the limit",MISREG_LIMIT);
            c_errmsg(msg,"acq_frame-misreg",&nonfatal);
            return(E_FAIL);
            }
        else
            return(E_SMAL);
        }
    }

return(E_SUCC);
}
