/*******************************************************************************
FUNCTION:	acqhead

PURPOSE:	Retrieve the acquisition header information.
		

Date   Programmer       Modifications
-----  ---------------  --------------------------------------------------------
02/91  D.Hollaren       Conversion to LAS5.0
06/93  J.Fenno          Added code for NG1 and NO1 that will be used for the
			archive conversion process.
09/95  J.Fenno		Added code for NO2 and NG2 that will be used for images
			ftp'd from NOAA when they are missed for some reason at
			the EDC.
06/96  T.Baltzer	Added UMX station id.
04/98  T.Baltzer	Added BJ2 station id.
12/98  D.Lloyd		Changed HBK to HB2 and added new HBK
02/00  D.Lloyd		Renamed as a local routine, added a header type param.
			Changed from if-else construct to switch on header
			type.
10/00  D.Lloyd		Added new header type for NOAA KLM series for download
			from NOAA ftp server.  These files have no ARS header.
			Also extracts data type from data header.

PROJECT:	ADAPS

$Log: acqhead.c,v $
Revision 3.9  2002/11/20 13:04:28  dlloyd
Added case 12 for BA2, a new Miami level 1b format.  CR 6817

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "worgen.h"
#include "tae.h"
#include "pixman.h"
#include "sysdef.h"
#include "avhrr.h"
#include "model.h"
#include "acqio.h"
#include "local_protos.h"
#include "lasendian.h"

#ifndef SEEK_END    /* Not defined on PowerNode */
#define SEEK_SET  0
#define SEEK_CUR  1
#define SEEK_END  2
#endif

long	satnum;

int acqhead
(
    int *fdesc,                 /* File descriptor to the acquire file       */
    long *year,                 /* 2-digit Year of the pass                  */
    char *station,              /* Station ID                                */
    long *dtype,                /* Type of data (HRPT, LAC, or GAC)          */
    struct SATPASS *pass,       /* Satellite pass information                */
    long headtype		/* Information header type		     */
)
{
struct ACQDESC *acq;		/* Pointer to the acqusition input structure */

long	nonfatal = 1,		/* Non-fatal error code			     */
	status,			/* Return status variable		     */
	nbytes,			/* Number of bytes to read		     */
	bytes_read,		/* Number of bytes read on read		     */
	ltype,			/* Type of data to be converted		     */
	shour,			/* Starting pass time hour		     */
	smin,			/* Starting pass time minute		     */
	ssec,			/* Starting pass time second		     */
	sfrac,			/* Starting pass time fractional second      */
	ns,			/* Number of samples to be converted	     */
	i;			/* Looping variable			     */
int	allzeroflg;		/* flag indicating all zeros in buffer	     */
long	zerocount;		/* count of zero values in buffer	     */
long    header_year;		/* year of pass from header                  */ 

long long lstatus;		/* Long long return status		     */
long long filesize;             /* Total number of bytes in the file         */

#ifdef vms
long	outsys = SYS_VAX_VMS;	/* Convert the data to VMS format numbers    */
#else
long	outsys = SYS_IEEE_STD;  /* Convert the data to IEEE format numbers   */
#endif

long	insys = SYS_VAX_VMS,	/* Input type conversion		     */
	flag  = SYS_ERR;	/* Flag to return errors when converting     */

float	miltim;			/* Time in milliseconds at start of pass     */

char	img_dtype[5],		/* Type of data (HRPT, LAC, GAC)	     */
	*ptr;			/* Charcter pointer to extract information   */

unsigned char header[21872];    /* Ascii header from the acquisition file    */

unsigned char under = 0;	/* Underflow value			     */
unsigned char over = 0;		/* Overflog value			     */

/*
  Save the initial input values in the satellite pass structure
*/
pass->year = *year;
pass->dtype = *dtype;
strcpy(pass->station,station);

/*
  Set the acquisition structure to the file descriptor address
*/
acq = (struct ACQDESC *) *fdesc;

switch ( headtype )
    {
    case 1 :
	/* SFL, NOA and NGC format section
	----------------------------------*/
	{
	/*
	  Read the header information from the acquired image
	*/
	nbytes = 512;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Convert the ascii header record into a structure.  This data is
	  informational only; EXCEPT THE YEAR and DATA TYPE!  All the other
	  information can and is obtained from HRPT minor frame.
	*/
	ptr = (char *) &header[256];
	sscanf(ptr,"%5ld %2ld %4s %3s %4ld %4ld %2ld/%2ld/%2ld %3ld %2ld:%2ld:%2ld.%3ld",
	       &pass->clock_err, &pass->satnum, img_dtype, pass->station,
	       &pass->nlines, &pass->nsamples, &pass->month, &pass->day,
	       &pass->year, &pass->jday, &pass->shour, &pass->smin,
	       &pass->ssec, &pass->sfrac);

	if (strcmp(img_dtype,"LAC") == 0)
	    pass->dtype = LAC;
	else if (strcmp(img_dtype,"GAC") == 0)
	    pass->dtype = GAC;
	else
	    pass->dtype = HRPT;
	break;
	}
    case 2 :
	/* EAF, NG1, NO1 and NM2 format section
	---------------------------------------*/
	{
	/*
	  Read the header information from the acquired image
	*/
	for (i = 0; i < 3; i++)
	    {
	    nbytes = 512;
	    bytes_read = read(acq->fd,header,nbytes);
	    if (bytes_read <= 0)
		{
		c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
		return(E_FAIL);
		}
	    }

	/*
	  Extract the number of lines from the header
	*/
	pass->nlines = ((header[157] << 8) + header[156]) - 1;

	/*
	  Extract the starting time of the pass and convert it
	But first swap the bytes if we're on a little endian system.
	*/
#ifdef LAS_LITTLE_ENDIAN
	{
	char swaptemp;
	
	swaptemp    = header[148];
	header[148] = header[151];
	header[151] = swaptemp;
	swaptemp    = header[149];
	header[149] = header[150];
	header[150] = swaptemp;
	}
#endif

	ns = 1;
	ltype = EREAL;
	c_pxsys(&insys,&outsys,&header[148],&ltype,&ns,&under,&over,&flag);

	/*
	  Copy the bytes to a float variable
	*/
	ns = 1;
	c_pxcopy(&header[148],(unsigned char *)&miltim,&ltype,&ns);

	/*
	  Calculate the hours, minutes, seconds, and fractional seconds
	  from the float
	*/
	shour = (long)(miltim / 10000);
	smin  = (long)((miltim - (float)(shour * 10000)) / 100);
	ssec  = (long)((miltim - (float)(shour * 10000 + smin * 100)));
	sfrac = (long)((miltim - (float)(shour * 10000 + smin * 100 + ssec))
		       * 1000);

	/****************> ! N O T E ! <**************************
	  Calculate the start time in total milliseconds.
	  THE NBS VARIABLE IS ONLY SET TEMPORARILY TO PASS THE START TIME
	  (EAF, NG1, NO1 codes will go away when converted to the new format)
	*****************> ! N O T E ! <**************************/
	pass->clock_err = (shour * 3600 + smin * 60 + ssec) * 1000 + sfrac;
	break;
	}
    case 3 :
	/* BAA format section
	---------------------*/
	{
	/*
	  Determine the size of the file
	*/
	filesize = lseek(acq->fd,0,SEEK_END);     /* determine bytes in file  */
	if (filesize < 0)
	    {
	    c_errmsg("Error determining file size","acqhead-fsiz",&nonfatal);
	    return(E_FAIL);
	    }
     
	/*
	  Calculate the number of lines in the file.  There are four 512 byte
	  header records and nine 512 byte trailer records (6656 bytes) to be
	  subtracted from the file size to determine the number of lines in
	  the file.
	*/
	pass->nlines = (long)((filesize - 6656) / acq->byte_line);

	/*
	  Reset the file pointer to the beginning of the file
	*/
	lstatus = lseek(acq->fd,0,SEEK_SET);
	if (lstatus == E_FAIL)
	    {
	    c_errmsg("Error seeking to the beginning of the image",
	             "acqhead-seek",
		    &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Read the header information from the acquired image
	*/
	nbytes = 512;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Extract the type of data from the header
	*/
	if (header[10] == 1)
	    pass->dtype = HRPT;
	else if (header[10] == 4)
	    pass->dtype = LAC;
	else
	    pass->dtype = GAC;

	/*
	  Extract the year from the header
	*/
	pass->year = ((header[17] << 8) + header[16]) % 100;

	/*
	  Skip the remaining header records
	*/
	for (i = 0; i < 3; i++)
	    {
	    nbytes = 512;
	    bytes_read = read(acq->fd,header,nbytes);
	    if (bytes_read <= 0)
		{
		c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
		return(E_FAIL);
		}
	    }
	break;
	}
    case 4 :
	/* CP2 format section
	---------------------*/
	{
	/*
	  Read the TBM header record from the acquired image
	*/
	nbytes = 122;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	/* Determine the actual pass direction and, if it's descending,
	reset the flip flag -- otherwise continue with the flip.
	---------------------------------------------------------------*/
	status = passdrctn( acq );
	if ( status == DESCENDING )
	    acq->flip_flg = 0;
	else if ( status != ASCENDING )
	    {
	    c_lasmsg(FALSE,"acqhead-dir","Unknown direction");
	    return E_FAIL;
	    }

	/*
	  Set the file pointer to the end of the file for backward processing
	  for ASCENDING data that has already been flipped.
	*/
	if (acq->flip_flg)
	    {
	    /* Set the file pointer to the end
	    of the file for backward processing.
	    ------------------------------------*/
	    lstatus = lseek(acq->fd,-(acq->byte_line),SEEK_END);
	    if (lstatus == E_FAIL)
		{
		c_errmsg("Error seeking to the end of the image","acqhead-seek",
			 &nonfatal);
		return(E_FAIL);
		}
	    }
	break;
	}
    case 5 :
	/* PAS format section
	---------------------*/
	{
	/*
	  Read the header information from the acquired image.

	  NOTE:  The header information is a seperate file on tape
		 and has been concated to the head of the image file.
	*/
	for (i = 0; i < 6; i++)
	    {
	    nbytes = 2520;
	    bytes_read = read(acq->fd,header,nbytes);
	    if (bytes_read <= 0)
		{
		c_errmsg("Error reading from the acquire header","acqhead-read",
			 &nonfatal);
		return(E_FAIL);
		}

	    /*
	      Extract the satellite number and direction of the pass from
	      the second record in the header file.
	    */
	    if (i == 1)
		{
		/* Get the satellite number from the header record.
		NOTE:  This is passed through an external variable
		to build the minor frame from the sharp format.
		---------------------------------------------------*/
		ptr = (char *) &header[313];
		sscanf(ptr,"%2ld",&satnum);
		pass->satnum = satnum;

		/*
		  From the direction of the pass flag in the header reset
		  the flip flag for DESCENDING passes
		*/
		if (header[356] == 'D')
		    acq->flip_flg = 0;

		/*
		  Get the number of lines of image data
		*/
		ptr = (char *) &header[1444];
		sscanf(ptr,"%4ld",&pass->nlines);
		}
	    }

	/*
	  Set the file pointer to the end of the file for backward processing
	  for ASCENDING data that has already been flipped.
	*/
	if (acq->flip_flg)
	    {
	    /* Set the file pointer to the end of
	    the file for backward processing.
	    -------------------------------------*/
	    lstatus = lseek(acq->fd,-acq->byte_line,SEEK_END);
	    if (lstatus == E_FAIL)
		{
		c_errmsg("Error seeking to the end of the image","acqhead-seek",
			 &nonfatal);
		return(E_FAIL);
		}
	    }
	/*
	  Set the time clock correction to zero because the times have already
	  been corrected
	*/
	pass->clock_err = 0;
	break;
	}
    case 6 :
	/* HB2 format section
	---------------------*/
	{
	/*
	  Read the header information from the acquired image
	*/
	nbytes = 14;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	ptr = (char *) &header[8];
	sscanf(ptr,"%2ld",&pass->year);
	break;
	}
    case 7 :
	/* UMX, HBK and BJ2 format section
	----------------------------------*/
	{
	/*
	  Read the TBM header record from the acquired image
	*/
	nbytes = 122;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
	    return(E_FAIL);
	    }
	/* There is purposely no break here.  We want to
	fall through to the next case and get the year.
	------------------------------------------------*/
	}
    case 8 : 
	/* BJG, NG2, NO2, JSA, as well as UMX, HBK and BJ2 format section
	-----------------------------------------------------------------*/
	/* This format is the one exception to the rule of only
	reading information headers in this function.  Read the
	data header to extract the year from the DSH.
	-------------------------------------------------------*/
	{
	/* Read data from the input until a non-zero byte is read
	---------------------------------------------------------*/
	nbytes = 7400;
	allzeroflg = TRUE;
	zerocount = 0;
	while ( allzeroflg )
	    {
	    bytes_read = read(acq->fd,header,nbytes);
	    if (bytes_read <= 0)
		{
		c_errmsg("Error reading from the acquire header","acqhead-read",
			 &nonfatal);
		return(E_FAIL);
		}
	    for ( i = 0; i < nbytes; i++ )
		if ( header[i] != 0 )	/* found a non-zero value	*/
		    {
		    allzeroflg = FALSE;
		    i--;		/* The previous zero is the	*/
		    break;		/* first byte of the data header*/
		    }
	    zerocount += i;		/* count the zero bytes		*/
	    }

	/* If there were zeros at the beginning of the file,
	reset to the beginning and read all the zeros.  This
	will put the file pointer at the beginning of the data.
	Next read the first half of the Data Set Header.
	------------------------------------------------------*/
	if ( zerocount > 0 )
	    {
	    lstatus = lseek(acq->fd,0,SEEK_SET);
	    if (lstatus == E_FAIL)
		{
		c_errmsg("Error seeking to the beginning of the image",
			 "acqhead-seek",
			&nonfatal);
		return(E_FAIL);
		}
	    bytes_read = read(acq->fd,header,zerocount);
	    if (bytes_read <= 0)
		{
		c_errmsg("Error reading from the acquire header","acqhead-read",
			 &nonfatal);
		return(E_FAIL);
		}
	    bytes_read = read(acq->fd,header,nbytes);
	    if (bytes_read <= 0)
		{
		c_errmsg("Error reading from the acquire header","acqhead-read",
			 &nonfatal);
		return(E_FAIL);
		}
	    }
	    
     
	/*
	  Extract the year from the header information
	*/
	pass->year = (header[2] & 0xfe) >> 1;
	pass->year %= 100;

	/*
	  Extract the data type from the header
	*/
	pass->dtype = (header[1] & 0xf0) >> 4;
	if ( pass->dtype > 3 && pass->dtype < 1 )
	    pass->dtype = *dtype;

	/*
	  Read the second header record from the acquired image
	*/
	nbytes = 7400;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
	    return(E_FAIL);
	    }
	break;
	}
    case 9 :
	/* LSU format section
	---------------------*/
	{
	/* 
	  Read past the first 10 bytes of garbage data
	*/
	nbytes = 10;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
		     &nonfatal);
	    return(E_FAIL);
	    }
	break;
	}
    case 12 : 
        /* This section is for level 1b from BAA extension BA2
	------------------------------------------------------*/
	{
	nbytes = 2048;
	bytes_read = read(acq->fd,header,nbytes);
	if (bytes_read <= 0)
	    {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
	             &nonfatal);
	    return(E_FAIL);
	    }

	/* Get the year of the pass from the Miami header
	-------------------------------------------------*/
	header_year = (long)header[17] << 8;
	header_year += (long)header[16];
	header_year %= 100;

	/* If the year in the header and the input parameter
	differ, issue and fatal error message.
	----------------------------------------------------*/
	if ( header_year != pass->year )
	    {
	    c_lasmsg(FALSE,"acqhead-year",
	        "Year mismatch between header and original file name");
	    return E_FAIL;
	    }

	/* Fall through to next case and following
	case to extract the ARS and data headers.
	------------------------------------------*/
	}
    case 10:
	/* This section is for the NOAA level 1b format (with ARS
	header) of KLM series satellites: N15, N16 and N17.  Use
	the default (header type 11) for level 1b without ARS
	header -- these include NOAA FTP format.
	------------------------------------------------------*/
        {
        nbytes = 512;
        bytes_read = read(acq->fd,header,nbytes);
        if (bytes_read <= 0)
            {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
	             &nonfatal);
	    return(E_FAIL);
	    }
	/* fall through to next case to extract the header
	--------------------------------------------------*/
        }
    case 11:
        {
	/* This section is for the NOAA ftp level 1b format of KLM
	series satellites: N15, N16 and N17.  This format does not
	include an ARS header.
	----------------------------------------------------------*/
        nbytes = 15872;
        bytes_read = read(acq->fd,header,nbytes);
        if (bytes_read <= 0)
            {
	    c_errmsg("Error reading from the acquire header","acqhead-read",
	             &nonfatal);
	    return(E_FAIL);
	    }
	/* check the header for the data type
	-------------------------------------*/
	if ( header[26] == 'H' )
	    pass->dtype = 3;
	else
	    pass->dtype = 1;
        break;
        }
    case  0:
    default:
	{
	/*
	  Set nbs offset, number of lines, and number of samples to unknown
	*/
	pass->clock_err = 0;
	pass->nlines = 0;
	pass->nsamples = 0;
	}
    }

return(E_SUCC);
}
