/******************************************************************************
NAME:           ACQINFO

PURPOSE:        Get information about the acquire file format based on
                the station code.

PROGRAM HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  5.0    D. Hollaren   03/92    Original development
  5.2    T.Baltzer     09/94    Rename BUFSIZ to BUFSIZE to avoid system
                                naming conflict (stdio.h).
  5.4    D.Lloyd       02/00    Modified to read New columns in station.ids
  				table: FORMAT TYPE and HEADER TYPE.  Changed
				name and moved to local function only.
  5.5    D.Lloyd       02/00    Modified to use the data type from the
  				station.ids table instead of the file name.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "tae.h"
#include "avhrr.h"
#include "acqio.h"

#define BUFSIZE 95

int acqinfo
(
    char *station,               /* Station ID                                */
    long *img_typ,               /* Type of image data (HRPT, LAC, GAC)       */
    long *skip_head,            /* Number of records to skip go get to image */
    long *rec_line,             /* Number of records per line                */
    long *byte_line,            /* Total number of bytes per line            */
    long *mnr_ptr,              /* Pointer to the start of HRPT minor frame  */
    long *mnr_siz,              /* Size of the HRPT minor frame (bytes)      */
    long *tip_ptr,              /* Pointer to the start of TIP data          */
    long *tip_siz,              /* Size of the TIP data (bytes)              */
    long *img_ptr,              /* Pointer to the image data                 */
    long *img_siz,              /* Size of the image data (bytes)            */
    long *img_fmt,              /* Image format (BIP, BIL, BIP)              */
    long *swap_flg,             /* Byte swap indicator                       */
    long *flip_flg,             /* Flip data indicator                       */
    long *headtype,             /* information header type		     */
    char *sta_name              /* Station ID name                           */
)
{
FILE    *fp;

char    disk[CMLEN],            /* Disk location of station id table         */
        sta_file[CMLEN],        /* Station id file name                      */
        buffer[BUFSIZE],        /* Buffer to read station id file with       */
        sta_key[4],             /* Station id key                            */
        ceosid[4],              /* Station id key                            */
        *gstatus;               /* Status from fgets routine                 */

long    img_key,                /* Image type key                            */
        skip,                   /* # of records to skip in station id file   */ 
        i,                      /* Looping variable                          */
        nonfatal = 1,           /* Non-fatal error code                      */
        found = FALSE,          /* Found a match flag                        */
        status = 5;             /* Read status                               */

/*
  Get the acquire station id file name
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(disk,getenv("ADAPSTABLES"));
else
    {
    c_errmsg("Error retrieving value of ADAPSTABLES","acqinfo-find",&nonfatal);
    return(E_FAIL);
    }

sprintf(sta_file,"%s/%s",disk,"station.ids");

/*
  Open the acquire station id file
*/
fp = fopen(sta_file,"r");
if (fp == NULL)
    {
    c_errmsg("Error opening the station id file","acqinfo-open",&nonfatal);
    return(E_FAIL);
    }

/*
  Read each line until a match to the station key is found
*/
status = fscanf(fp,"%3ld",&skip);
if (status < 1)
    {
    fclose(fp);
    c_errmsg("Error reading the station id file","acqinfo-read",
        &nonfatal);
    return(E_FAIL);
    }

/*
  Skip over the header records
*/
for (i = 0; i < skip; i++)
    {
    gstatus = fgets(buffer,BUFSIZE,fp);
    if (gstatus == NULL)
        {
        fclose(fp);
        c_errmsg("Error skipping the station id header","acqinfo-skip",
                 &nonfatal);
        return(E_FAIL);
        }
    }

/*
  Read each line until a match to the station key is found
*/
while (status != EOF)
    {
    gstatus = fgets(buffer,BUFSIZE,fp);
    if (gstatus == NULL)
        {
        if (feof(fp) != 0)
            {
            status = EOF;
            continue;
            }
        else
            {
            fclose(fp);
            c_errmsg("Error reading the station id file","acqinfo-read",
                     &nonfatal);
            return(E_FAIL);
            }
        }

    status = sscanf(buffer,
          "%3s %3s %ld %1ld %2ld %2ld %5ld %5ld %4ld %5ld %4ld %4ld %5ld %1ld %1ld %2ld %24s",
          sta_key, ceosid, headtype, &img_key, skip_head, rec_line, byte_line,
	  mnr_ptr, mnr_siz, tip_ptr, tip_siz, img_ptr, img_siz, img_fmt,
	  swap_flg, flip_flg, sta_name);

    /*
      Check to see if the station number matches the specified key
      Break out if they match
    */
    if (strcmp(station,sta_key) == 0)
        {
        found = TRUE;
	/* return the CEOS ID */
	strcpy( station, ceosid );
	*img_typ = img_key;
	/*
        if ((*img_typ == GAC) && (img_key != 2))
            found = FALSE;
	*/

        if (found)
            break;
        }
    }

/*
  Close the acquire station id file
*/
fclose(fp);

/*
  Error if the station code wasn't found
*/
if (!found)
    {
    c_errmsg("Specified station id and image type not found","acqinfo-find",
             &nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
