/******************************************************************************
NAME:           C_ACQOPEN

PURPOSE:        

PROGRAM HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  5.0    D. Hollaren   03/92    Original Development
  5.1    D.Lloyd       02/00    Added call to acqhead() and logic to omit
  				call to acqskip() for ascending, flipped data
				On return from this function the file pointer
				will be at the first data record.
  5.2    D.Lloyd       02/00    Modified to use the data type from the
  				station.ids table instead of the file name.
  6.0    D.Lloyd       03/2002  Modified to reverse the meaning of the swap
                                flag on little endian systems

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include "worgen.h"
#include "tae.h"
#include "acqio.h"
#include "avhrr.h"
#include "local_protos.h"
#include "lasendian.h"

#ifndef SEEK_END
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

int c_acqopen
(
    long *fdesc,        /* File descriptor to acquire image                  */
    char *file,         /* Acquisition file name                             */
    long *access,       /* Type of file access                               */
    char *station,      /* Station the data is from                          */
    long *img_typ,      /* Type of data (HRPT, LAC, or GAC)                  */
    long *tip_flg,      /* Flag to indicate TIP data exits in acquired image */
    long *year,		/* Year of acquisition				     */
    struct SATPASS *pass/* Satellite pass information                        */
)
{
struct ACQDESC *acq;    /* Pointer to the acquisition input structure        */

int     flags;          /* Access flag for open                              */

long	nlines,         /* Number of image lines                             */
        nonfatal = 1,   /* Non-fatal error code                              */
        status;         /* Return status variable                            */
long	headtype;	/* Information header type			     */

off_t	  filesize,     /* Number of bytes in the file                       */
          lstatus;	/* Long long return status			     */

/*
  Dynamically allocate space for the acquire structure
*/
acq = (struct ACQDESC *) malloc(sizeof(struct ACQDESC));
if (acq == NULL)
    {
    c_errmsg("Error allocating dynamic memory","acqopen-alloc",&nonfatal);
    return(E_FAIL);
    }

/*
  Set the file descriptor to the address of the structure
*/
*fdesc = (long)acq;

/*
  Set information supplied from user
*/
strcpy(acq->station,station);

/*
  Get information about the possible data format of the acquired
  image based on the receiving station.
*/
status = acqinfo(acq->station, &acq->img_typ, &acq->skip_head, 
                   &acq->rec_line, &acq->byte_line, &acq->mnr_ptr, 
                   &acq->mnr_siz, &acq->tip_ptr, &acq->tip_siz, 
                   &acq->img_ptr, &acq->img_siz, &acq->img_fmt, 
                   &acq->swap_flg, &acq->flip_flg, &headtype,
		   acq->sta_name);
if (status != E_SUCC)
    return(E_FAIL);
*img_typ = acq->img_typ;

/* Reverse the meaning of the swap flag
when running on little endian systems.
---------------------------------------*/
#ifdef LAS_LITTLE_ENDIAN
acq->swap_flg = !acq->swap_flg;
#endif

if (acq->tip_siz > 0)
    *tip_flg = TRUE;
else
    *tip_flg = FALSE;

/*
  Open the acquire file with the specified access
*/
if (*access == IWRITE)
    flags = (O_WRONLY | O_CREAT);
else if (*access == IREAD)
    flags = O_RDONLY;
else
    flags = (O_RDWR | O_CREAT);

/*
  Open the file
*/
acq->fd = open(file,flags,0);
if (acq->fd < 0)
    {
    c_errmsg("Error opening the acquisition image","acqopen-open",&nonfatal);
    return(E_FAIL);
    }

/* Read the information header, if present, extracting any
pertinent information.  Data returned in the pass structure.
-----------------------------------------------------------*/
status = acqhead( (int *)&acq, year, acq->station, img_typ, pass, headtype );
if (status != E_SUCC)
    return(E_FAIL);

/*
  For an image that needs to be flipped, set the
  file pointer to the first band of the last line
*/
if (acq->flip_flg)
    {
    if (acq->img_fmt == BSQ)
        {
        filesize = lseek(acq->fd,0,SEEK_END);     /* determine bytes in file  */
        if (filesize < 0)
            {
            c_errmsg("Error determining file size","acqopen-fsiz",&nonfatal);
            return(E_FAIL);
            }

        nlines = (long)(filesize / (acq->byte_line * NCHANS));

        /* set to ending line of first band */
        lstatus = lseek(acq->fd,(nlines - 1) * acq->byte_line,SEEK_SET); 
        if (lstatus == E_FAIL)
            {
            c_errmsg("Error seeking within image","acqopen-seek",&nonfatal);
            return(E_FAIL);
            }
        }
    }

/*
  Skip any data header records if data is not flipped
*/
else
    {
    status = acqskip(fdesc,acq->skip_head);
    if (status != E_SUCC)
	return(E_FAIL);
    }

return(E_SUCC);
}
