/*******************************************************************************
NAME:           C_ACQREAD

PURPOSE:        Read a line from the acquired image 

Date   Programmer       Modifications
-----  ---------------  --------------------------------------------------------
02/92  D.Hollaren       Development
07/92  T.Baltzer        Add BSQ capability
09/95  J.Fenno          Removed code that was commented out.  This will be
                        tracked by CVS.  The code tracking a shift in the frame
                        sync data.
12/96  D.Etrheim        Allocate inbuf which will make it word alligned.
02/00  D.Lloyd          Added code for NOAA KLM data format which comes in a
                        partially packed format.  Data is packed but minor
                        frame is not.
03/02  D.Lloyd          Modified to support different endian systems.  Removed
                        the two local functions and placed in their own file.
                        Rearranged the logic and removed redundant code.

PROJECT:  ADAPS
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "tae.h"
#include "acqio.h"
#include "archio.h"
#include "avhrr.h"
#include "adapsutil.h"
#include "lasendian.h"
#include "local_protos.h"

#define MNR_SHP 238

int c_acqread
(
    long  *fdesc,          /* IN:  File descriptor to the acquire file  */
    short *mnrbuf,         /* OUT: Output buffer for HRPT minor frame   */
    short *tipbuf,         /* OUT: Output buffer for TIP data           */
    short *imgbuf          /* OUT: Output buffer for acquire image data */
)
{
struct ACQDESC *acq;       /* Pointer to the acquisition input structure*/

unsigned char *inbuf;      /* Input buffer for acquire*/

static short tmpbuf[MAX_ACQ]; /* Temporary buffer for image conversions */
static long  bands[6] = {1,2,3,4,5,0}; /* Select all bands, for bil2bip */

long output_dtype = EWORD; /* data type for all output data buffers     */
long fill  = TRUE;         /* Fill in any missing band, for bil2bip     */
long input_dtype = EBYTE;  /* input data type for c_pxconv              */
long num_samples;          /* Number of samples in a band for bil2bip   */
long num_words;            /* Number of words to be copied              */
long read_status;          /* Return status from the read               */


/* Allocate the input buffer
----------------------------*/
inbuf = (unsigned char *) malloc(MAX_ACQ * sizeof( short ));

/* Set the acquisition structure to the file descriptor address
---------------------------------------------------------------*/
acq = (struct ACQDESC *) *fdesc;

/* Read a full line of image data from each band.  The line
comes back in one of several formats: BIP, BIL, packed, unpacked,
partial pack etc. depending on the source of the data.  BSQ data
is transformed to BIL data in read_bands.
-----------------------------------------------------------------*/
read_status = read_bands( acq, inbuf );
if (( read_status != E_SUCC ) && ( read_status != E_BOF ))
    {
    free ( inbuf );
    return read_status;
    }

/*************************************************************************
  Extract the minor frame from the input buffer. 

Regardless of the format of the input buffer the minor frame buffer will
contain 103 words of minor frame data, stored as short, and byte-swapped
to the local endian format.
**************************************************************************/

/* In case of full packed data unpack the entire input
buffer.  The full buffer must be unpacked since the
minor frame, TIP information and image data overlap.
The unpacked data is placed in the tmpbuf so that TIP
and image data can be extracted properly later on.
------------------------------------------------------*/
if ( acq->byte_line == FULL_LAC_PACK || acq->byte_line == FULL_LAC_BPACK ||
     acq->byte_line == FULL_GAC_PACK || acq->byte_line == FULL_GAC_BPACK )
    {
    /* First unpack the entire input buffer.
    The swap flag takes care of endian changes.
    -------------------------------------------*/
    num_words = MAX_ACQ;
    c_unpck32( inbuf, tmpbuf, &num_words, &acq->swap_flg );

    /* Now copy the minor frame
    ---------------------------*/
    num_words = MNR_SIZ;
    c_pxcopy((unsigned char *)&tmpbuf[acq->mnr_ptr], (unsigned char *)mnrbuf,
              &output_dtype, &num_words );
    }
else
    {
    num_words = MNR_SIZ;

    switch ( acq->mnr_siz )
	{
	case MNR_PCK :
	    /* Unpack the minor frame, the swap flag
	    takes care of the local endian changes.
	    ----------------------------------------*/
	    c_unpck32((unsigned char *)&inbuf[acq->mnr_ptr], mnrbuf, &num_words,
		       &acq->swap_flg );
	    break;

	case MNR_SIZ :
	    /* If the minor frame is 8-bit convert to
	    16-bit.  No endian changes needed here.
	    -----------------------------------------*/
	    c_pxconv( &input_dtype, &output_dtype, &inbuf[acq->mnr_ptr],
	            (unsigned char *)mnrbuf, &num_words );
	    break;

	case MNR_SHP :
	    /* If it's a sharp minor frame, sharp_mnr
	    takes care of endian changes.
	    -----------------------------------------*/
	    sharp_mnr( inbuf, mnrbuf );
	    break;

	case MNR_UNP :
	    /* Otherwise copy the buffer directly, the swap
	    flag takes care of the local endian changes.
	    -----------------------------------------------*/
	    if ( acq->swap_flg )
		c_pxswap( &inbuf[acq->mnr_ptr], &num_words, &output_dtype );
	    
	    c_pxcopy( &inbuf[acq->mnr_ptr], (unsigned char *)mnrbuf,
	              &output_dtype, &num_words );
	    break;

	default :
	    /* The default takes care of zero size minor frames
	    No information will be output to the minor frame buffer.
	    -------------------------------------------------------*/
	    break;
	}  /* END switch */
    }  /* END if else for minor frame */


/*************************************************************************
  Extract the tip information from the input buffer. 

Regardless of the format of the input buffer the tip information buffer
will contain 520 words of tip information data, stored as short, and
byte-swapped to the local endian format.
**************************************************************************/
num_words = TIP_SIZ;

/* In case of full packed data the entire input
buffer was unpacked earlier into the tmpbuf.  So simply
copy the information into the tipbuf.
-------------------------------------------------------*/
if ( acq->byte_line == FULL_LAC_PACK || acq->byte_line == FULL_LAC_BPACK ||
     acq->byte_line == FULL_GAC_PACK || acq->byte_line == FULL_GAC_BPACK )
    {
    c_pxcopy((unsigned char *)&tmpbuf[acq->tip_ptr],
             (unsigned char *)&tipbuf[1], &output_dtype, &num_words );
    }
else
    {
    /* Save the AVHRR ID from the HRPT minor frame to the TIP buffer
    ----------------------------------------------------------------*/
    tipbuf[0] = mnrbuf[6];

    switch ( acq->tip_siz )
	{
	case TIP_PCK :
	    /* Unpack the tip information, the swap flag
	    takes care of the local endian changes.
	    --------------------------------------------*/
	    c_unpck32((unsigned char *)&tmpbuf[acq->tip_ptr], &tipbuf[1],
		       &num_words, &acq->swap_flg );
	    break;

	case TIP_SIZ :
	    /* If the tip buffer is 8-bit, convert
	    to 16-bit.  No endian changes needed.
	    --------------------------------------*/
	    c_pxconv( &input_dtype, &output_dtype, &inbuf[acq->tip_ptr],
		    (unsigned char *)&tipbuf[1], &num_words );
	    break;

	case TIP_UNP :
	    /* Otherwise copy the buffer directly, the swap
	    flag takes care of the local endian changes.
	    -----------------------------------------------*/
	    if ( acq->swap_flg )
		c_pxswap( &inbuf[acq->tip_ptr], &num_words, &output_dtype );

	    c_pxcopy( &inbuf[acq->tip_ptr], (unsigned char *)&tipbuf[1],
		      &output_dtype, &num_words );
	    break;

	default :
	    /* The default takes care of zero size TIP buffers
	    No information will be output to the tip buffer.
	    --------------------------------------------------*/
            break;
	}  /* END switch */
    }  /* END if else for TIP information */


/***************************************************************************
  Extract the image data from the input buffer.

Regardless of the format of the input buffer, the image buffer will
contain either LAC_SIZ or GAC_SIZ words of image data, stored as short,
band interleaved by pixel and in the local endian format.
****************************************************************************/

/* Set image size based on image type.  All input images are
assumed to have five AVHRR bands and full width swaths.
------------------------------------------------------------*/
if ( acq->img_typ == GAC )
    {
    num_words = GAC_SIZ;
    num_samples = GAC_SAMPS;
    }
else
    {
    num_words = LAC_SIZ;
    num_samples = LAC_SAMPS;
    }


/* In case of full packed data the entire input buffer
was already unpacked.  If the data is BIP simply copy
the data, otherwise the data will be converted to BIP
in the next if statement.
-------------------------------------------------------*/
if ( acq->byte_line == FULL_LAC_PACK || acq->byte_line == FULL_LAC_BPACK ||
     acq->byte_line == FULL_GAC_PACK || acq->byte_line == FULL_GAC_BPACK )
    {
    if ( acq->img_fmt == BIP )
	c_pxcopy((unsigned char *)&tmpbuf[acq->img_ptr],
	         (unsigned char *)imgbuf, &output_dtype, &num_words );
    }

/* For KLM and other packed data, unpack the image
data, the swap flag takes care of endian changes.
--------------------------------------------------*/
else if ( acq->byte_line == KLM_LAC_PACK  || acq->byte_line == KLM_GAC_PACK ||
          acq->img_siz   == LAC_PCK       || acq->img_siz   == GAC_PCK )
    {
    /* Unpack directly to the image buffer for BIP */
    if ( acq->img_fmt == BIP )
        c_unpck32( &inbuf[acq->img_ptr], imgbuf, &num_words, &acq->swap_flg );
    /* Unpack to the tempbuf for BIL BSQ */
    else
        c_unpck32( &inbuf[acq->img_ptr], tmpbuf, &num_words, &acq->swap_flg );
    }

/* If the image data is 8-bit, convert
to 16-bit.  No endian changes needed.
--------------------------------------*/
else if ( acq->img_siz == LAC_SIZ || acq->img_siz == GAC_SIZ )
    {
    /* Convert directly to the image buffer for BIP */
    if ( acq->img_fmt == BIP )
	c_pxconv( &input_dtype, &output_dtype, &inbuf[acq->img_ptr],
		 (unsigned char *)imgbuf, &num_words );
    /* Convert to the tempbuf for BIL BSQ */
    else
	c_pxconv( &input_dtype, &output_dtype, &inbuf[acq->img_ptr],
		 (unsigned char *)&tmpbuf[acq->img_ptr], &num_words );
    }

/* Otherwise copy the image data directly, the swap
flag takes care of the local endian changes.
-----------------------------------------------------*/
else
    {
    if ( acq->swap_flg )
        c_pxswap( &inbuf[acq->img_ptr], &num_words, &output_dtype );

    /* Copy directly to image buffer for BIP */
    if ( acq->img_fmt == BIP )
	c_pxcopy( &inbuf[acq->img_ptr], (unsigned char *)imgbuf,
	          &output_dtype, &num_words );
    /* Copy to tempbuf for BIL BSQ */
    else
	c_pxcopy( &inbuf[acq->img_ptr], (unsigned char *)&tmpbuf[acq->img_ptr],
	          &output_dtype, &num_words );
    }

/* If the image data is either BIL or BSQ it was
unpacked, converted or copied to the temp buffer.
Endian changes were done at that time.  Convert
to BIP and copy to the image buffer.
------------------------------------------------*/
if ( acq->img_fmt != BIP )
    c_bil2bip( &tmpbuf[acq->img_ptr], imgbuf, &num_samples, bands, &fill );


free( inbuf );

/* Status is either E_SUCC or E_BOF from read_bands
---------------------------------------------------*/
return read_status;
}  /* END c_acqread */
