/******************************************************************************
FUNCTION:       c_acqtmcor

PURPOSE:        Retreive a time correction from the appropriate time
                correction table for a given date and time.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    02/98  T.Baltzer        Totally rewritten from previous version.
  1.1    03/98  T.Baltzer        Allow a default 0 value to be returned when
                                 an appropriate value isn't found in the table.


PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "util.h"
#include "avhrr.h"
#include "acqio.h"
#include "adapsutil.h"
#include "local_protos.h"

#define MAXDAYS  2      /* Maximum number of days in which to extrapolate */
                        /* or interpolate from the closest value in the   */
                        /* table.                                         */
#define MAXTMDIFF 900   /* Maximum difference in time correction values   */
                        /* between two entries in the tables (millisec).  */

int c_acqtmcor
(
    long *satnum,               /* Satellite number                          */
    long *f_month,              /* Month to find                             */
    long *f_day,                /* Day to find                               */
    long *f_year,               /* Year to find                              */
    long *f_hour,               /* Hour to find                              */
    long *f_min,                /* Minute to find                            */
    long *f_sec,                /* Seconds of to find                        */
    long *f_fsec,               /* Fractional seconds to find                */
    long *timcor                /* Time correction                           */
)
{

long    after_year,             /* Entry AFTER input time/date: year         */
        after_month,            /* Entry AFTER input time/date: month        */
        after_day,              /* Entry AFTER input time/date: day          */
        after_jday,             /* Entry AFTER input time/date: julian day   */
        after_time,             /* Entry AFTER input time/date: time         */
        after_timcor,           /* Entry AFTER input time/date: t correction */
        after_flag,             /* Entry AFTER input time/date: value found? */
        after_valid,            /* Entry AFTER input time/date: valid value? */
        before_year,            /* Entry BEFORE input time/date: year        */
        before_month,           /* Entry BEFORE input time/date: month       */
        before_day,             /* Entry BEFORE input time/date: day         */
        before_jday,            /* Entry BEFORE input time/date: julian day  */
        before_time,            /* Entry BEFORE input time/date: time        */
        before_timcor,          /* Entry BEFORE input time/date: t correction*/
        before_flag,            /* Entry BEFORE input time/date: value found?*/
        before_valid,           /* Entry BEFORE input time/date: valid value?*/
        ndays,                  /* Number of days from last time correction  */
        f_jday,                 /* Julian date to seach for                  */
        f_time,                 /* Time to search for                        */
        status;                 /* Read status                               */

/*
  Initialize the time correction to zero
-------------------------------------------------*/
*timcor = 0;


/*
  Convert the date of interest to a julian day
-------------------------------------------------*/
c_grejul(f_year,f_month,f_day,&f_jday);


/*
  Convert the time of interest to milliseconds
-------------------------------------------------*/
f_time = (*f_hour * 3600 + *f_min * 60 + *f_sec) * 1000 + *f_fsec;


/*
  Read the appropriate time correction table and determine the "before"
  and "after" entry closest to the date and time of interest.
------------------------------------------------------------------------*/
status = tmcor_read(*satnum,*f_year,f_jday,f_time,
                    &before_year,&before_month,&before_day,&before_time,
                    &before_timcor,&before_flag,
                    &after_year,&after_month,&after_day,&after_time,
                    &after_timcor,&after_flag);
if (status != E_SUCC)
    return(E_FAIL);


/*
  We have now determined the closest "before" and "after" entry in the
  appropriate time correction table (if they existed).  If values were
  found, determine if they are close enough to the time of interest to be
  used.
------------------------------------------------------------------------*/
before_valid = FALSE;
if (before_flag == TRUE)
    {
    c_daysdiff(f_month,f_day,f_year,&before_month,&before_day,&before_year,
               &ndays);
    if (ndays <= MAXDAYS)
        before_valid = TRUE;
    }

after_valid = FALSE;
if (after_flag == TRUE)
    {
    c_daysdiff(f_month,f_day,f_year,&after_month,&after_day,&after_year,
               &ndays);
    if (ndays <= MAXDAYS)
        after_valid = TRUE;
    }


/*
  If there is a big difference in the two time corrections from the table,
  an error probably exists, or perhaps NOAA reset the satellite clock.
  The best we can do in this case is assume that the satellite clock was
  reset, and that the time of interest occurred AFTER the clock was reset
  (a debateable statement, no doubt).  In this case, we will disqualify
  the "before" entry in the table.
------------------------------------------------------------------------*/
if (before_valid && after_valid)
    {
    if (abs(after_timcor - before_timcor) > MAXTMDIFF)
        before_valid = FALSE;
    }


/*
  If a "before" and "after" value are valid (available for use) then
  interpolate between the two.
  If just one of the two is available, just use that time correction.
  If neither are available, return with a default 0 time correction.
---------------------------------------------------------------------*/
if (before_valid && after_valid)
    {
    c_grejul(&before_year,&before_month,&before_day,&before_jday);
    c_grejul(&after_year,&after_month,&after_day,&after_jday);
    tmcor_interp(before_year,before_jday,before_time,before_timcor,
                 after_year,after_jday,after_time,after_timcor,
                 *f_year,f_jday,f_time,timcor);
    }
else if (before_valid)
    *timcor = before_timcor;
else if (after_valid)
    *timcor = after_timcor;
else
    {
    c_lasmsg(FALSE,"acqtmcor-notfnd",
             "%s %02d/%02d/%02d %s NOAA%02d time correction file",
             "Entry for date",*f_month,*f_day,*f_year,"not found in the",
             *satnum);
    return(E_SUCC);
    }

return(E_SUCC);
}

