/*******************************************************************************
FUNCTION:       c_bil2bip

PURPOSE:        Convert a buffer of BIL data into a buffer of AVHRR BIP data.
                If the fill flag is set, the output buffer must be large enough
                to hold all NCHANS channels of AVHRR data and the channels that
                are not specified in parameter bands[] will be zeroed.  If the
                fill flag is NOT set, the output buffer must be large enough to
                hold as many channels as specified in parameter bands[].

                This function will return E_SUCC if successful, E_FAIL
                otherwise.


                Examples:  In the following examples a '0' indicates the
                           function has zeroed the pixel value and values of
                           '1', '2', '3', '4', and '5' indicates the function
                           has copied a pixel value from the input buffer from
                           band 1, 2, 3, 4, or 5, respectively.


                    Bands = (2 4)  Fill = 1

                            -----------------------------------------------
                    obuf =  | 0 | 2 | 0 | 4 | 0 | 0 | 2 | 0 | 4 | 0 | ...
                            ----------------------------------------------



                    Bands = (2 4)  Fill = 0

                            -----------------------------------------------
                    obuf =  | 2 | 4 | 2 | 4 | 2 | 4 | 2 | 4 | 2 | 4 | ...
                            ----------------------------------------------



Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    03/91  D.Hollaren       Development
  2.1    01/94  J.Fenno          Fixed problem with indices.

PROJECT:        ADAPS
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "tae.h"
#include "adaps.h"
#include "acqio.h"


int c_bil2bip
(
    short *ibuf,        /* IN  - Input BIL buffer                             */
    short *obuf,        /* OUT - Output BIP buffer                            */
    long *nsamps,       /* IN  - Number of samples in a line                  */
    long bands[],       /* IN  - Selected bands to copy into obuf             */
    long *fill          /* IN  - Zero fill nonselected bands in obuf?         */
)
{
char str[CMLEN];        /* Temporary string                                   */

long i;                 /* Loop control                                       */
long j;                 /* Loop control                                       */
long nchans;            /* Number of channels selected                        */
long nonfatal = 1;      /* Nonfatal error flag                                */
long chans[NCHANS];     /* Selected channels in order                         */

short *iptr;            /* Pointer to the input buffer                        */
short *optr;            /* Pointer to the output buffer                       */



/* Initialize the selected channel array
----------------------------------------*/
for (j = 0; j < NCHANS; j++)
    chans[j] = 0;


/* Order the selected bands in the channel array
------------------------------------------------*/
if (*fill)
    {
    for (j = 0; (j < NCHANS) && (bands[j] > 0); j++)
        {
        if (bands[j] > NCHANS)
            {
            sprintf(str, "Element %ld of bands exceeds %d", j, NCHANS);
            c_errmsg(str, "c_bip2bil-bands", &nonfatal);
            return(E_FAIL);
            }
        chans[bands[j] - 1] = bands[j];
        }
    nchans = NCHANS;
    }
else
    {
    for (j = 0; (j < NCHANS) && (bands[j] > 0); j++)
        {
        if (bands[j] > NCHANS)
            {
            sprintf(str, "Element %ld of bands exceeds %d", j, NCHANS);
            c_errmsg(str, "c_bip2bil-bands", &nonfatal);
            return(E_FAIL);
            }
        chans[j] = bands[j];
        }
    nchans = j;
    }


/* Copy the input BIL buffer to the output BIP buffer
-----------------------------------------------------*/
for (j = 0; j < nchans; j++)
    {
    /* Set pointers to the appropriate place in the input and output buffers
    ------------------------------------------------------------------------*/
    iptr = ibuf + *nsamps * (chans[j] - 1);
    optr = obuf + j;


    /* If the specified channel is set, copy the input buffer to the output
       buffer.  If the channel is not set, zero fill the output buffer.
    -----------------------------------------------------------------------*/
    if (chans[j])
        {
        for (i = 0; i < *nsamps; i++)
            {
            *optr = *(iptr++);
            optr += nchans;
            }
        }
    else
        {
        for (i = 0; i < *nsamps; i++)
            {
            *optr = 0;
            optr += nchans;
            }
        }
    }
        
return(E_SUCC);
}
