/*******************************************************************************
FUNCTION:       c_bip2bil

PURPOSE:        Convert a buffer of AVHRR BIP data into a buffer of BIL data.
                If the fill flag is set, the output buffer must be large enough
                to hold all NCHANS channels of an AVHRR image.  If the fill
                flag is not set then the output buffer will only contain the
                requested bands.  It is assumed that the input buffer contains
                all NCHANS channels.

                This function will return E_SUCC if successful, E_FAIL
                otherwise.

                Examples:

                    Bands = (2 4)  Fill = 1

                    obuf =      -----------------------------------------
                                | Band 1 all zeroed                     |
                                -----------------------------------------
                                | Band 2 from input buffer              |
                                -----------------------------------------
                                | Band 3 all zeroed                     |
                                -----------------------------------------
                                | Band 4 from input buffer              |
                                -----------------------------------------
                                | Band 5 all zeroed                     |
                                -----------------------------------------


                    Bands = (2 4)  Fill = 0

                    obuf =      -----------------------------------------
                                | Band 2 from input buffer              |
                                -----------------------------------------
                                | Band 4 from input buffer              |
                                -----------------------------------------


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    03/91  D.Hollaren       Development
  2.1    08/93  J.Fenno          Fixed problem with indices.

PROJECT:        ADAPS
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "tae.h"
#include "adaps.h"
#include "acqio.h"


int c_bip2bil
(
    short *ibuf,        /* IN     - Input BIP buffer                          */
    short *obuf,        /* IN/OUT - Output BIL buffer                         */
    long *nsamps,       /* IN     - Number of samples in a line               */
    long bands[],       /* IN     - Selected bands                            */
    long *fill          /* IN     - Zero fill nonselected bands in obuf       */
)
{
char str[CMLEN];        /* Temporary string                                   */

long i;                 /* Loop control                                       */
long j;                 /* Loop control                                       */
long nonfatal = 1;      /* Nonfatal error message                             */

short chans[NCHANS];    /* Selected channels in order                         */
short *iptr;            /* Pointer into the input buffer                      */
short nchans;           /* Number of channels selected                        */
short *optr;            /* Pointer into the output buffer                     */


/* Initialize the selected channel array
----------------------------------------*/
for (j = 0; j < NCHANS; j++)
    chans[j] = 0;


/* Order the selected bands in the channel array
------------------------------------------------*/
if (*fill)
    {
    for (j = 0; (j < NCHANS) && (bands[j] > 0); j++)
        {
        if (bands[j] > NCHANS)
            {
            sprintf(str, "Element %ld of bands exceeds %d", j, NCHANS);
            c_errmsg(str, "c_bip2bil-bands", &nonfatal);
            return(E_FAIL);
            }
        chans[bands[j] - 1] = (short)(bands[j]);
        }
    nchans = NCHANS;
    }
else
    {
    for (j = 0; (j < NCHANS) && (bands[j] > 0); j++)
        {
        if (bands[j] > NCHANS)
            {
            sprintf(str, "Element %ld of bands exceeds %d", j, NCHANS);
            c_errmsg(str, "c_bip2bil-bands", &nonfatal);
            return(E_FAIL);
            }
        chans[j] = (short)(bands[j]);
        }
    nchans = (short)j;
    }
 

/* Copy the input BIP buffer to the output BIL buffer
-----------------------------------------------------*/
for (j = 0; j < nchans; j++)
    {
    /* Set pointers to the appropriate place in the input and output buffers
    ------------------------------------------------------------------------*/
    iptr = ibuf + chans[j] - 1;
    optr = obuf + j * *nsamps;


    /* If the specified channel is set, copy the input buffer to the output
       buffer.  If the channel is not set, zero fill the output buffer.
    -----------------------------------------------------------------------*/
    if (chans[j])
        {
        for (i = 0; i < *nsamps; i++)
            {
            *(optr++) = *iptr;
            iptr += nchans;
            }
        }
    else
        {
        for (i = 0; i < *nsamps; i++)
            *(optr++) = 0;
        }
    }
        
return(E_SUCC);
}
