/******************************************************************************
NAME:		GET_LINE

PURPOSE:	Read a line from the acquired image 

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 T. Baltzer   06/92	Original Development
  5.1    D.Lloyd      09/99     Modified to properly handle flipping packed
                                image data.  Also fixed a bug where unpacked
                                BIP data was incorrectly flipped.
  6.0    D.Lloyd      03/2002   Removed from c_acqread.c and made its own file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdlib.h>
#include <unistd.h>
#include "worgen.h"
#include "tae.h"
#include "acqio.h"
#include "archio.h"
#include "avhrr.h"
#include "adapsutil.h"
#include "lasendian.h"
#include "pixman.h"
#include "local_protos.h"


int get_line
(
    struct ACQDESC *acq,        /* Acquisition structure                     */
    unsigned char *acqbuf       /* Output buffer for acquire image data      */
)
{
long	i;    			/* Loop counter				     */
long	j;    			/* Loop counter				     */
long	k;    			/* Loop counter				     */
long    nonfatal = 1;		/* Non-fatal error code			     */
long    shortend;		/* End of short buffer			     */
long    imgptr;			/* Start of the image data in buffer         */
long    nwords = LAC_WORDS;	/* number of words to unpack		     */
long    status;			/* Return status from the read		     */

long	dtype = EWORD;		/* Type of data wanted in output	     */

char	msg[ERRLEN];		/* Error message string			     */
char    *charbuf;		/* Temporary buffer before flipping	     */

short	*shortbuf;		/* Temporary buffer of shorts before flipping*/
short   *shortacq;		/* Short version of acquire buffer	     */
short	*flipbuf;		/* flipped buffer of shorts		     */



if (!acq->flip_flg)
    {
    /* 
      No flipping is required; read a line of the acquire file 
      straight into the buffer 
    */
    status = read(acq->fd,acqbuf,acq->byte_line);

    if (status == 0)
        return(E_EOF);
    else if (status < 0)
        {
        c_errmsg("Error reading from the acquired file","acqread-read",
	        &nonfatal);
        return(E_FAIL);
        }
    else if (status != acq->byte_line)
        {
        sprintf(msg,"%s %ld %s %ld %s %s",
	       "The number of bytes read from the acquire file",status,
	       "doesn't match the expected number of bytes",acq->byte_line,
	       "for the specified station",acq->station);
        c_errmsg(msg,"acqread-nbyte",&nonfatal);
        return(E_SMAL);
        }
    }
else
    {
    /* 
      Flipping is required; read a line of the acquire file 
      into the temp buffer first
    */
    switch ( acq->img_siz )
	{
	case LAC_SIZ :	/* 8-bit LAC data	*/
	case GAC_SIZ :	/* 8-bit GAC DATA	*/
	    {
	    charbuf = (char *)malloc(acq->byte_line);

	    status = read(acq->fd,charbuf,acq->byte_line);
	    if (status == 0)
	        {
		free( charbuf );
		return(E_EOF);
		}
	    else if (status < 0)
		{
		free( charbuf );
		c_errmsg("Error reading from the acquired file","acqread-read",
			&nonfatal);
		return(E_FAIL);
		}
	    else if (status != acq->byte_line)
		{
		free( charbuf );
		sprintf(msg,"%s %ld %s %ld %s %s",
		       "The number of bytes read from the acquire file",status,
		       "doesn't match the expected number of bytes",
		       acq->byte_line, "for the specified station",
		       acq->station);
		c_errmsg(msg,"acqread-nbyte",&nonfatal);
		return(E_SMAL);
		}
	
	    /*
	      Assume minor frame and tip data aren't flipped
	      Straight copy these; flip image data only
	    */
	    for (i = 0; i < acq->img_ptr; i++)
		acqbuf[i] = charbuf[i];
	    if (acq->img_fmt == BSQ)
		{
		for (i = acq->img_ptr,j = 0; i < acq->byte_line; i++,j++)
		    acqbuf[i] = charbuf[acq->byte_line - j - 1];
		}
	    else
		{
		for (j = 0; j < NCHANS; j++)
		    {
		    for (i = 0; i < LAC_SAMPS; i++)
			{
			acqbuf[(j * LAC_SAMPS + i) + acq->img_ptr] = 
			    charbuf[acq->img_ptr + (j+1) * LAC_SAMPS - i - 1];
			}
		    }
		}

	    free( charbuf );
	    break;
	    }
	case LAC_UNP :	/* unpacked LAC data	*/
	case GAC_UNP :	/* unpacked GAC data	*/
	    {
	    shortbuf = (short *)malloc(acq->byte_line);
	    shortacq = (short *)acqbuf;

	    status = read(acq->fd,shortbuf,acq->byte_line);
	    if (status == 0)
		{
		free( shortbuf );
		return(E_EOF);
		}
	    else if (status < 0)
		{
		free( shortbuf );
		c_errmsg("Error reading from the acquired file","acqread-read",
			&nonfatal);
		return(E_FAIL);
		}
	    else if (status != acq->byte_line)
		{
		free( shortbuf );
		sprintf(msg,"%s %ld %s %ld %s %s",
		       "The number of bytes read from the acquire file",status,
		       "doesn't match the expected number of bytes",
		       acq->byte_line, "for the specified station",
		       acq->station);
		c_errmsg(msg,"acqread-nbyte",&nonfatal);
		return(E_SMAL);
		}
	
	    /*
	      Assume minor frame and tip data aren't flipped
	      Straight copy these; flip image data only
	    */
	    shortend = acq->byte_line / 2;
	    imgptr = acq->img_ptr / 2;
	    if (acq->img_fmt == BSQ)
		{
		for (i = 0; i < imgptr; i++)
		    shortacq[i] = shortbuf[i];

		for (i = imgptr,j = 0; i < shortend; i++,j++)
		    shortacq[i] = shortbuf[(shortend - 1) - j];
		}
	    else if (acq->img_fmt == BIL)
		{
		/*
		  Copy the full data line and then just flip the image data
		*/
		c_pxcopy((unsigned char *)shortbuf,(unsigned char *)shortacq,
			 &dtype,&shortend);

		for (j = 0; j < NCHANS; j++)
		    {
		    for (i = 0; i < LAC_SAMPS; i++)
			{
			shortacq[imgptr + j * LAC_SAMPS + i] = 
			      shortbuf[imgptr + ((j + 1) * LAC_SAMPS - 1) - i];
			}
		    }
		}
	    else	/* BIP data */
		{
		/* straight copy the minor frame */
		for (i = 0; i < imgptr; i++)
		    shortacq[i] = shortbuf[i];

		/* flip the image data */
		shortend = acq->img_siz / 2 + imgptr;
		for ( i = imgptr, j = shortend - NCHANS; i < shortend;
		      i += NCHANS, j -= NCHANS )
		    for ( k = 0; k < NCHANS; k++ )
			shortacq[i+k] = shortbuf[j+k];
		}

	    free( shortbuf );
	    break;
	    }
	case ARCH_GAC :		/* packed GAC data	*/
	    nwords = GAC_WORDS; /* reset nwords for GAC */
	case ARCH_LAC :		/* packed LAC data	*/
	    /* For this type data we have to unpack the image data before
	    flipping the pixels because three 10-bit values are stored in
	    one 32-bit value.  There is no 'easy' way to step through a
	    buffer of packed data.  Also since the calling routine still
	    expects the data to be packed we must pack the image data
	    once flipped -- messy but necessary.
	    -------------------------------------------------------------*/

	    /* Allocate the appropriate buffers
	    -----------------------------------*/
	    charbuf = (char *)malloc( acq->byte_line );
	    if ( !charbuf )
	        {
		c_lasmsg(FALSE,"acqread-alloc","Error allocating memory");
		return(E_FAIL);
		}
	    shortbuf = (short *)malloc( sizeof(short) * acq->byte_line );
	    if ( !shortbuf )
	        {
		free( charbuf );
		c_lasmsg(FALSE,"acqread-alloc","Error allocating memory");
		return(E_FAIL);
		}
	    flipbuf = (short *)malloc( sizeof(short) * acq->byte_line );
	    if ( !flipbuf )
	        {
		free( charbuf );
		free( shortbuf );
		c_lasmsg(FALSE,"acqread-alloc","Error allocating memory");
		return(E_FAIL);
		}

	    /* Read the line of data from the input file */
	    status = read(acq->fd,charbuf,acq->byte_line);
	    if (status == 0)
		{
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		return(E_EOF);
		}
	    else if (status < 0)
		{
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		c_errmsg("Error reading from the acquired file","acqread-read",
			&nonfatal);
		return(E_FAIL);
		}
	    else if (status != acq->byte_line)
		{
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		sprintf(msg,"%s %ld %s %ld %s %s",
		       "The number of bytes read from the acquire file",status,
		       "doesn't match the expected number of bytes",
		       acq->byte_line, "for the specified station",
		       acq->station);
		c_errmsg(msg,"acqread-nbyte",&nonfatal);
		return(E_SMAL);
		}

	    /* Assume the minor frame is not flipped
	    and copy directly to the output buffer.
	    ----------------------------------------*/
	    for ( i = 0; i < acq->img_ptr; i++ )
	        acqbuf[i] = charbuf[i];

	    /* Now flip the data by first unpacking, flipping
	    and repacking to the output buffer.
	    --------------------------------------------------*/
	    if ( acq->img_fmt == BIP )
		{
		c_unpck32((unsigned char *)&charbuf[acq->img_ptr], shortbuf,
		    &nwords, &acq->swap_flg);
		for ( i = 0, j = nwords - NCHANS; i < nwords;
		      i += NCHANS, j -= NCHANS )
		    for ( k = 0; k < NCHANS; k++ )
			flipbuf[i+k] = shortbuf[j+k];
		c_pck32( flipbuf, &acqbuf[acq->img_ptr], &nwords );
		}
	    else
	        {
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		c_lasmsg(FALSE,"acqread-fmt","Can't flip packed BSQ or BIL");
		return(E_FAIL);
		}

	    free( charbuf );
	    free( shortbuf );
	    free( flipbuf );
	    break;

	default :
	    c_errmsg("Invalid image data size","acqread-size",&nonfatal);
	    return(E_FAIL);
	}
    }

return(E_SUCC);
}
