/***************************************************************************
FUNCTION:	PASSDRCTN

PURPOSE:	Read milliseconds from lines in the scene to determine the
  		direction of the scene pass, either ascending or descending.
  
Date	Programmer		Modification
-----	----------------------	------------------------------------------
09/99	D.Lloyd			Initial development
03/2002 D.Lloyd                 Modified to support little endian systems

ALGORITHM:	Read a line from the scene and extract the millisecond
		time.  Compare the milliseconds for the last five lines.
		If the difference is either 166 or 167 count the difference,
		if difference is positive count it as descending or
		as ascending if negative.  When there are four counts
		for ascending return ASCENDING or if there are four counts
		for descending return DESCENDING.  For errors or a read
		through the entire file without determining a direction,
		return a failure value.

PROJECT:	ADAPS
***************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include "las.h"
#include "avhrr.h"
#include "acqio.h"
#include "lasendian.h"
#include "adapsendian.h"

/* Define a operation array to do endian swapping just for this routine
See acqio/DOCS/design.html for an explanation of operation arrays.
-----------------------------------------------------------------------*/
#define PASSDRCTNOPARR { 1, 4, 4, 1 }

/* We can't return E_FAIL since it matches
define of DESCENDING which is -1 also.  So
use this define of FAILURE in its place.
-------------------------------------------*/
#define FAILURE -9

int passdrctn
(
struct ACQDESC *acq	/* struct containing pass information		*/
)
{
int	asc;		/* count of ascending milsecs within -166/-167	*/
int	dsc;		/* count of descending milsecs within 166/167	*/
int	i;		/* loop counter					*/

long	milsec[5] = {0};/* array of milliseconds from the last 5 lines	*/
long	*ptr;		/* long pointer to line buffer			*/

off_t	fd_position;	/* the file position pointer on entry		*/

unsigned char *buf;	/* input line buffer				*/


/* Save the current position in the file so we can
return to it before returning from the function.
--------------------------------------------------*/
fd_position = lseek( acq->fd, 0L, SEEK_CUR );
if ( fd_position == -1 )
    {
    c_lasmsg(FALSE,"passdrctn-seek","Error seeking within image");
    return FAILURE;
    }

/* Allocate the buffer space
----------------------------*/
buf = (unsigned char *)malloc( sizeof(unsigned char) * acq->byte_line );
if ( !buf )
    {
    c_lasmsg(FALSE,"passdrctn-alloc","Error allocating memory");
    return FAILURE;
    }
ptr = (long *)&buf[0];

/* Read the DSH
---------------*/
if ( acq->byte_line != read( acq->fd, buf, acq->byte_line ))
    {
    c_lasmsg(FALSE,"passdrctn-read","Unabel to read DSH");
    free( buf );
    return FAILURE;
    }

/* Read lines from the file until EOF is encountered or we find
five consecutive lines that are 166/167 milliseconds apart.
---------------------------------------------------------------*/
while ( acq->byte_line == read( acq->fd, buf, acq->byte_line ))
    {
    /* Insure correct byte sequence
    -------------------------------*/
    import_endian_mixed_buffer( buf, PASSDRCTNOPARR );

    /* Extract the milsec from the current line putting it
    in the last position of the array.  The first 4 times
    through the while loop we are just filling the array.
    ------------------------------------------------------*/
    milsec[4] = ptr[1];

    /* count consecutive milsecs that are +/-166 or +/-167 apart */
    asc = dsc = 0;
    for ( i = 1; i < 5; i++ )
	{
	/* the line is later in time, count it as descending */
	if (( milsec[i] - milsec[i-1] == 166 ) ||
	    ( milsec[i] - milsec[i-1] == 167 ))
	    dsc++;

	/* the line is earlier in time, count it as ascending */
	else if (( milsec[i] - milsec[i-1] == -166 ) ||
		 ( milsec[i] - milsec[i-1] == -167 ))
	    asc++;
	}

    /* If we have four milsecs the proper distance apart,
    reset the file to the proper location and return
    -----------------------------------------------------*/
    if ( asc == 4 )
	{
	free( buf );
	if ( lseek( acq->fd, fd_position, SEEK_SET ) == -1 )
	    {
	    c_lasmsg(FALSE,"passdrctn-seek","Error seeking within image");
	    return FAILURE;
	    }
	return ASCENDING;
	}
    else if ( dsc == 4 )
	{
	free( buf );
	if ( lseek( acq->fd, fd_position, SEEK_SET ) == -1 )
	    {
	    c_lasmsg(FALSE,"passdrctn-seek","Error seeking within image");
	    return FAILURE;
	    }
	return DESCENDING;
	}

    /* Otherwise, shift the last four milsecs
    forward one in the array and continue.
    -----------------------------------------*/
    else
	{
	for ( i = 0; i < 4; i++ )
	    milsec[i] = milsec[i+1];
	}
    }  /* END while */

/* We read through the entire file without finding
a sequence of 4 milsecs so return a failure value.
--------------------------------------------------*/
free( buf );
return FAILURE;
}
