/******************************************************************************
NAME:		READ_BANDS

PURPOSE:	Read a line of data from each band of the acquired
		image (whether the data be BIP, BIL, or BSQ).

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-------------------------------------------
  5.0	 T. Baltzer    02/92	Original Development (modified c_acqread)
  6.0    D.Lloyd       03/2002  Removed from c_acqread.c and made its own file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <unistd.h>
#include "worgen.h"
#include "tae.h"
#include "acqio.h"
#include "pixman.h"
#include "local_protos.h"


#ifndef SEEK_END    /* Not defined on PowerNode */
#define SEEK_SET  0
#define SEEK_CUR  1
#define SEEK_END  2
#endif


#define NUM_BANDS  5	/* assume 5 bands of data for BSQ format */

int read_bands
(
    struct ACQDESC *acq,        /* Pointer to the acqusition input structure */
    unsigned char inbuf[]       /* Input buffer for acquire                  */
)
{

static unsigned char rdbuf[MAX_ACQ * sizeof(short)];	/* read buffer	     */

long	bytes_per_imgline;	/* bytes of image data per line		     */
long    dtype = EWORD;		/* type of data wanted in output	     */
long    i;			/* looping variable			     */
long    lines_per_band;		/* lines per band in file		     */
long    nonfatal = 1;		/* non-fatal error code			     */
long    status;			/* return status from the read		     */
long    words_per_imgline;	/* words of image data per line		     */
long    zero = 0;		/* constant				     */

long long filesize;		/* total number of bytes in the file	     */
long long lstatus;		/* Long long return status		     */
long long startpos = 0;		/* starting position within the first band   */

/* 
  If the data is BSQ (Band Sequential) skipping around within the file
  will be required.  Once the lines have been properly read and loaded,
  the output buffer will look like a buffer of BIL data.
*/
if (acq->img_fmt == BSQ)
    {
    startpos = lseek(acq->fd,zero,SEEK_CUR);  /* determine current location */
    if (startpos == E_FAIL)
        {
        c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
        return(E_FAIL);
        }

    filesize = lseek(acq->fd,zero,SEEK_END);	/* determine bytes in file  */
    if (filesize < 0)
        {
        c_errmsg("Error determining file size","acqread-fsiz",&nonfatal);
        return(E_FAIL);
        }

    lstatus = lseek(acq->fd,startpos,SEEK_SET);	/* return to proper location */
    if (lstatus == E_FAIL)
        {
        c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
        return(E_FAIL);
        }

    lines_per_band = (long)(filesize / (acq->byte_line * NUM_BANDS));
    bytes_per_imgline = acq->byte_line - acq->img_ptr;

    status = get_line(acq,inbuf);
    if (status != E_SUCC)
        return(status);

    words_per_imgline = bytes_per_imgline / (long)sizeof(short);

    for (i = 0; i < NUM_BANDS - 1; i++)
        {
        lstatus = lseek(acq->fd,(lines_per_band - 1) * acq->byte_line,SEEK_CUR);
        if (lstatus == E_FAIL)
            {
            c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
            return(E_FAIL);
            }

	status = get_line(acq,rdbuf);
        if (status != E_SUCC)
            return(status);

        c_pxcopy(&rdbuf[acq->img_ptr],&inbuf[acq->byte_line + 
		(bytes_per_imgline * i)],&dtype,&words_per_imgline);
        }

    /* return to first band */
    if (acq->flip_flg)
	{
	/* if going backwards, subtract one line from previous starting line*/
	startpos -= acq->byte_line;
 	if (startpos < 0)
            return(E_BOF);
	}
    else
	startpos += acq->byte_line;

    lstatus = lseek(acq->fd,startpos,SEEK_SET);
    if (lstatus == E_FAIL)
        {
        c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
        return(E_FAIL);
        }
    }
else	/* BIL or BIP format */
    {
    /*
      Determine the current position when the data needs to be flipped
    */
    if (acq->flip_flg)
	{
	startpos = lseek(acq->fd,zero,SEEK_CUR);
	if (startpos == E_FAIL)
	    {
	    c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
            return(E_FAIL);
            }
	}

    if (acq->rec_line < 0)
        {
        /*
          A full image line is contained in more than one acquire line.
          Read the acquire image until the full image line is in the buffer.
        */
        for (i = 0; i < abs(acq->rec_line); i++)
    	    {
	    /*
	      Read a line of the acquire file into the buffer
	    */
	    status = get_line(acq,&inbuf[acq->byte_line * i]);
	    if (status != E_SUCC)
	        return(status);
	    }
        }
    else
        {
        /*
          The full image line is contained within one acquire line.
          Read a line of the acquire file into the buffer.
        */
	status = get_line(acq,inbuf);
        if (status != E_SUCC)
            return(status);
        }

    /*
      When flipping the data the file descriptor must be set back a line
    */
    if (acq->flip_flg)
	{
	startpos -= (acq->byte_line * abs(acq->rec_line));
 	if (startpos < 0)
            return(E_BOF);

	lstatus = lseek(acq->fd,startpos,SEEK_SET);
	if (lstatus == E_FAIL)
	    {
	    c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
	    return(E_FAIL);
	    }
	}
    }

return(E_SUCC);
}
