/******************************************************************************
NAME:           SHARP_MNR

PURPOSE:        Generate an HRPT minor frame from the sharp format

PROGRAM HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  5.0    D. Hollaren   11/92    Original Development
  6.0    D. Lloyd      03/02    Modified to ensure the minor frame is in
  				the local endian format

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdio.h>
#include "pixman.h"
#include "worgen.h"
#include "acqio.h"
#include "avhrr.h"
#include "lasendian.h"
#include "adapsendian.h"

int sharp_mnr
(
    unsigned char *inbuf,       /* Input buffer from the sharp format        */
    short *mnrbuf               /* Output buffer for HRPT minor frame        */
)
{
long    jday;                   /* Julian date of the image                  */

long    *milsec;                /* Milliseconds of day                       */

long    dtype = EWORD,          /* Type of data wanted in output             */
        i,                      /* Looping variable                          */
        zero = 0,               /* Zero value                                */
        nwords;                 /* Number of words to copy                   */


/* Make sure the sharp minor frame is in the local endian format
----------------------------------------------------------------*/
import_endian_mixed_buffer( inbuf, SHARPMINOROPARRAY );

/*
  Return a zero filled minor frame when a dropped line is found
*/
if ((inbuf[36] == 0) && (inbuf[4132] == 0) && (inbuf[8228] == 0) &&
    (inbuf[12324] == 0) && (inbuf[16420] == 0))
    {
    nwords = 103;
    c_pxfill((unsigned char *)(&mnrbuf[0]),&dtype,&nwords,
	     (unsigned char *)(&zero));
    return(E_SUCC);
    }

/*
  Set a standard frame sync pattern
*/
mnrbuf[0] = 0x0284;
mnrbuf[1] = 0x016f;
mnrbuf[2] = 0x035c;
mnrbuf[3] = 0x019d;
mnrbuf[4] = 0x020f;
mnrbuf[5] = 0x0095;

mnrbuf[6] = *(short *)(&inbuf[20520]);
mnrbuf[7] = 0;

/*
  Convert the milliseconds to the minor frame time code
*/
milsec = (long *) &inbuf[24];
jday = (long)inbuf[20544] * 100 + (long)inbuf[20545];

mnrbuf[11] = (short)(*milsec & 0x3ff);
mnrbuf[10] = (short)((*milsec >> 10) & 0x3ff);
mnrbuf[9]  = (short)((*milsec >> 20) & 0x7f);
mnrbuf[8]  = (short)((jday & 0x3ff) << 1);

/*
  Copy the Telemetry (ramp calibration & internal target temperature)
*/
nwords = 5;
c_pxcopy(&inbuf[20652],(unsigned char *)(&mnrbuf[12]),&dtype,&nwords);
nwords = 3;
c_pxcopy(&inbuf[20666],(unsigned char *)(&mnrbuf[17]),&dtype,&nwords);

/*
  Patch temperature
*/
mnrbuf[20] = *(short *)&inbuf[20676];

/*
  Spare for telemetry
*/
mnrbuf[21] = 1;

/*
  Copy the Internal target data
  This data is not band-interleaved, it is bsq for bands 3,4,5
*/
for (i = 0; i < 10; i++)
    {
    mnrbuf[22+i*3]   = *(short *)&inbuf[20722+2*i];
    mnrbuf[22+i*3+1] = *(short *)&inbuf[20742+2*i];
    mnrbuf[22+i*3+2] = *(short *)&inbuf[20762+2*i];
    }

/*
  Copy the Space data
  This data is not band-interleaved, it is bsq
*/
for (i = 0; i < 10; i++)
    {
    mnrbuf[52+i*5]   = *(short *)&inbuf[20552+2*i];
    mnrbuf[52+i*5+1] = *(short *)&inbuf[20572+2*i];
    mnrbuf[52+i*5+2] = *(short *)&inbuf[20592+2*i];
    mnrbuf[52+i*5+3] = *(short *)&inbuf[20612+2*i];
    mnrbuf[52+i*5+4] = *(short *)&inbuf[20632+2*i];
    }

/*
  AVHRR sync
*/
mnrbuf[102] = *(short *)&inbuf[20524];

return(E_SUCC);
}
