/******************************************************************************
FUNCTION:       tmcor_read

PURPOSE:        Read the appropriate time correction table and return the
                "before" and "after" time closest to the date and time of
                interest.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    02/98  T.Baltzer        Development, stolen from old c_acqtmcor().


PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "acqio.h"

#define BUFSIZE 30

int tmcor_read
(
    long satnum,                /* Satellite number                          */
    long f_year,                /* Year to find                              */
    long f_jday,                /* Julian date to seach for                  */
    long f_time,                /* Time to search for                        */
    long *before_year,          /* Entry BEFORE input time/date: year        */
    long *before_month,         /* Entry BEFORE input time/date: month       */
    long *before_day,           /* Entry BEFORE input time/date: day         */
    long *before_time,          /* Entry BEFORE input time/date: time        */
    long *before_timcor,        /* Entry BEFORE input time/date: t correction*/
    long *before_flag,          /* Entry BEFORE input time/date: value found?*/
    long *after_year,           /* Entry AFTER input time/date: year         */
    long *after_month,          /* Entry AFTER input time/date: month        */
    long *after_day,            /* Entry AFTER input time/date: day          */
    long *after_time,           /* Entry AFTER input time/date: time         */
    long *after_timcor,         /* Entry AFTER input time/date: t correction */
    long *after_flag            /* Entry AFTER input time/date: value found? */
)

{

FILE    *fp;                    /* File pointer to the time correction table */

char    disk[CMLEN],            /* Disk location of time correction table    */
        tm_file[CMLEN],         /* Time correction table file name           */
        buffer[BUFSIZE],        /* Buffer to read the time corrections to    */
        *gstatus;               /* Status from fgets routine                 */

long    before_flg,             /* Before or after time of interest          */
        i,                      /* Looping variable                          */
        rstatus = 0,            /* Read status                               */
        sat_year,               /* Year of satellite time correction         */
        sat_month,              /* Month of satellite time correction        */
        sat_day,                /* Day of satellite time correction          */
        sat_hour,               /* Hout of satellite time correction         */
        sat_min,                /* Min of satellite time correction          */
        sat_sec,                /* Second of satellite time correction       */
        sat_fsec,               /* Fractional second of satellite time corr  */
        sat_jday,               /* Julian date of satellite time correction  */
        sat_time,               /* Satellite time                            */
        sat_timcor,             /* Satellite time correction                 */
        skip,                   /* # of records to skip in station id file   */ 
        status;                 /* Return status                             */


/*
  Initialize the flags
--------------------------*/
*before_flag = FALSE;
*after_flag = FALSE;

/*
  Make sure the year of interest is expressed in 4 digits
-----------------------------------------------------------*/
f_year = c_4digityr(f_year);


/*
  No time correction tables exist for the satellite less than NOAA-11.
  Return with a default 0 time correction.
----------------------------------------------------------------------*/
if (satnum < 11)
    {
    c_lasmsg(FALSE,"acqtmcor-nov",
             "Time correction values not available for NOAA-%d",satnum);
    return(E_SUCC);
    }

/*
  Get the time correction table file name
----------------------------------------------*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(disk,getenv("ADAPSTABLES"));
else
    {
    c_lasmsg(FALSE,"acqtmcor-find","Error retrieving value of ADAPSTABLES");
    return(E_FAIL);
    }
sprintf(tm_file,"%s/noaa%02ld.timcor",disk,satnum); 


/*
  Open the time correction file
----------------------------------*/
fp = fopen(tm_file,"r");
if (fp == NULL)
    {
    c_lasmsg(FALSE,"acqtmcor-open",
             "Error opening the NOAA%02d time correction file",satnum);
    return(E_FAIL);
    }

/*
  Get the number of header records
-------------------------------------*/
status = fscanf(fp,"%3ld",&skip);
if (status < 1)
    {
    fclose(fp);
    c_lasmsg(FALSE,"acqtmcor-read",
             "Error reading the NOAA%02d time correction file",satnum);
    return(E_FAIL);
    }

/*
  Skip over the header records
----------------------------------*/
for (i = 0; i < skip; i++)
    {
    gstatus = fgets(buffer,BUFSIZE,fp);
    if (gstatus == NULL)
        {
        fclose(fp);
        c_lasmsg(FALSE,"acqtmcor-skip",
                 "Error skipping the NOAA%02d time correction header",satnum);
        return(E_FAIL);
        }
    }


/*
  Read the time correction table to determine the closest entry
  "before" the time of interest and the closest entry "after" the
  time of interest.
-------------------------------------------------------------------*/
while (rstatus != EOF)
    {
    gstatus = fgets(buffer,BUFSIZE,fp);
    if (gstatus == NULL)
        {
        if (feof(fp) != 0)
            {
            rstatus = EOF;
            continue;
            }
        else
            {
            fclose(fp);
            c_lasmsg(FALSE,"acqtmcor-read",
                     "Error reading the NOAA%02d time correction file",satnum);
            return(E_FAIL);
            }
        }
    else if (strcmp(buffer,"\n") == 0)
        {
        continue;    /* skip blank lines */
        }

    /*
      Load the fields with the data from this table entry
    ------------------------------------------------------*/
    status = sscanf(buffer,"%2ld/%2ld/%2ld %2ld:%2ld:%2ld.%3ld %5ld",
                    &sat_month,&sat_day,&sat_year,&sat_hour,
                    &sat_min,&sat_sec, &sat_fsec, &sat_timcor);

    /*
    Make sure the year of interest is expressed in 4 digits
    -----------------------------------------------------------*/
    sat_year = c_4digityr(sat_year);

    /*
      Convert the date on this table entry to a julian date
    ----------------------------------------------------------*/
    c_grejul(&sat_year,&sat_month,&sat_day,&sat_jday);

    /*
      Convert the time on this table entry to milliseconds
    --------------------------------------------------------*/
    sat_time = (sat_hour * 3600 + sat_min * 60 + sat_sec) * 1000 + sat_fsec;


    /*
      If the time of interest is greater than or equal to the time found in
      this record of the time correction table, then set the "before" time.
      If the time of interest is less than the time found in this record of
      the time correction table, then set the "after" time, and set the flag
      to terminate the reading of the time correction table.
    ------------------------------------------------------------------------*/
    if (f_year > sat_year)
        before_flg = 1;
    else if (f_year < sat_year)
        before_flg = 0;
    else if (f_jday > sat_jday)
        before_flg = 1;
    else if (f_jday < sat_jday)
        before_flg = 0;
    else if (f_time >= sat_time)
        before_flg = 1;
    else
        before_flg = 0;

    if (before_flg)
        {
        *before_year = sat_year;
        *before_month = sat_month;
        *before_day = sat_day;
        *before_time = sat_time;
        *before_timcor = sat_timcor;
        *before_flag = TRUE;
        }
    else
        {
        *after_year = sat_year;
        *after_month = sat_month;
        *after_day = sat_day;
        *after_time = sat_time;
        *after_timcor = sat_timcor;
        *after_flag = TRUE;
        rstatus = EOF;
        }

    } /* end of 'while' construct */

/*
  Close the time corrections file
---------------------------------*/
fclose(fp);


return(E_SUCC);

}
