/*******************************************************************************
FUNCTION:	C_APPCOEFF

PURPOSE:	Apply predetermined calibration coefficients to one
		line of image data.

ALGORITHM:

ALGORITHM REFERENCES
  Katherine B. Kidwell, NOAA Polar Orbiter Data Users Guide
  National Oceanic and Atmospheric Administration, Washington, D.C.
  July, 1991.
  Geoffrey Goodrum, Katherine B. Kidwell, Wayne Winston, NOAA KLM User's Guide,
  National Oceanic and Atmospheric Administration, Washington, D.C.
  March, 1999.
*******************************************************************************/
#include <stdio.h>		/* Standard I/O include			*/
#include "las.h"		/* LAS include file			*/
#include "pixman.h"		/* LAS include file			*/
#include "adaps.h"		/* ADAPS include file			*/
#include "constants.h"		/* Constant values defined		*/
#include "calcoef.h"		/* Calibration coefficient structure	*/
#include "adapssup.h"		/* ADAPS support routines		*/
#include "local_protos.h"	/* local ADAPS support routines		*/

int c_appcoeff
(
    struct MODEL *model,	/* AVHRR model information		*/
    struct CALSTR *calstr,	/* Calibration structure		*/
    long window[],	/* Image window specification			*/
    long channel[],	/* Channels to process				*/
    long maxchan,	/* Maximum number of channels to process	*/
    char *minmax,	/* Min/max flag					*/
    double omin[NCHANS],	/* min pixel value array		*/
    double omax[NCHANS],	/* max pixel value array		*/
    unsigned char *buffer,    /* image I/O buffer                       */
    long offset1[MAXIMG*NCHANS],/* Offsets into "buffer" for input bands*/
    long offset2[NCHANS],    /* Offsets into "buffer" for output bands	*/
    struct COEFFSTR *coeff	/* Calibration coefficients		*/
)

{

static long	soption;	/* Scaling option as expected by c_getscale()*/

static long	firsttime = TRUE;	/* First time in this routine T/F    */

static long datasize[NDTYPES] = {0,1,2,4,4,8};

static struct  SCALESTR a_scale;/* Reflectance scaling structure	*/
static struct  SCALESTR r_scale;/* Radiance scaling structure		*/
static struct  SCALESTR t_scale;/* Temperature scaling structure	*/

static float cwncon2[MAXTHERMBND];/* CWN times constant two		*/
static float cwn3con1[MAXTHERMBND];/* CWN times constant one		*/


float	ovalue,			/* Temp variable for the output pixel	*/
	lut[NCHANS][MAXDAT];	/* Reflectance, temperature LUT for data*/

double	a_unscale,		/* Reflectance unscale factor		*/
	r_unscale,		/* Radiance unscale factor		*/
	t_unscale;		/* Temperature unscale factor		*/

float 	*inpix;		  	/* Pointer into the input buffer	*/
unsigned char 	*outpix1;	/* Pointer into the output buffer	*/
short 	*outpix2;		/* Pointer into the output buffer	*/
long 	*outpix3;		/* Pointer into the output buffer	*/
float 	*outpix4;		/* Pointer into the output buffer	*/

long	chan, samp,       /* Looping variables				*/
	status;		  /* Return status variable			*/


/*
  If this is the first call to this routine, initialize the appropriate
  parameters.
*/
if (firsttime == TRUE)
    {

    /*
      Calculate the CWN * constants to save computation time later.
    */
    for (chan = 0; chan < MAXTHERMBND; chan++)
	{
	cwncon2[chan] = C2 * calstr->rats_vc[chan][calstr->temp_range-1];
	cwn3con1[chan] = C1 *
	    calstr->rats_vc[chan][calstr->temp_range-1] *
	    calstr->rats_vc[chan][calstr->temp_range-1] *
	    calstr->rats_vc[chan][calstr->temp_range-1] ;
	}

    /*
      Set the scaling based on the scaling option and the quantity.
      Get the scaling for reflectance, radiance, temperature.
    */
    c_getscaleopt(calstr->scale_opt,calstr->odtype,&soption);
    status = c_getscale(soption,"reflectance",&a_scale.minval,&a_scale.maxval,
			&a_scale.scale,&a_unscale,&a_scale.offset);
    if (status != E_SUCC)
	return(status);

    status = c_getscale(soption,"radiance",&r_scale.minval,&r_scale.maxval,
			&r_scale.scale,&r_unscale,&r_scale.offset);
    if (status != E_SUCC)
	return(status);

    status = c_getscale(soption,"thermal",&t_scale.minval,&t_scale.maxval,
			&t_scale.scale,&t_unscale,&t_scale.offset);
    if (status != E_SUCC)
	return(status);

    firsttime = FALSE;
    }


/*
  Generate the reflectance and temperature look-up tables for this
  line using the gain and intercept values for this line.
  NOTE: Technically, this look-up table needs to be generated only for a
  line wherein the coefficients change from the previous line.  But
  for the sake of simplicity, we will generate it for every line.
*/
ccgen_luts(calstr,model,channel,maxchan,coeff,cwncon2,cwn3con1,
	 &a_scale,&r_scale,&t_scale,lut);

/*
  Lookup the output value for each sample of each band that is needed
*/
for (chan = 0; chan < maxchan; chan++)
    {
    /*
      Set the input and output buffer pointers to the starting addresses
    */
    inpix  = (float *)&buffer[offset1[chan] * datasize[EREAL]];
    if ((*inpix < 0) || (*inpix > 1023))
	{
	c_lasmsg(FALSE,"appcoeff-ran","Input value beyond range of 0-1023");
	return(E_FAIL);
	}

    if (calstr->odtype == EBYTE)
	outpix1 = (unsigned char *)&buffer[offset2[chan]*datasize[EBYTE]];
    else if (calstr->odtype == EWORD)
	outpix2 = (short *)&buffer[offset2[chan]*datasize[EWORD]];
    else if (calstr->odtype == ELONG)
	outpix3 = (long *)&buffer[offset2[chan]*datasize[ELONG]];
    else if (calstr->odtype == EREAL)
	outpix4 = (float *)&buffer[offset2[chan]*datasize[EREAL]];

    /*
      Process the samples from a line
    */
    for (samp = 0; samp < window[NS]; samp++)
	{
	/*
	  Set the output pixel value.
	*/
	ovalue = lut[channel[chan]-1][(long)*inpix];

	/*
	  Round the value for non-real data
	*/
	if (calstr->odtype != EREAL)
	    {
	    if (ovalue < 0)
		ovalue -= 0.5;
	    else
		ovalue += 0.5;
	    }

	/*
	  Assign the output pixel value to the value from the table
	  generated.
	*/
	if (calstr->odtype == EBYTE)
	    {
	    /*
	      Check byte limits
	    */
	    if (ovalue < 0.0)
		ovalue = 0.0;
	    else if (ovalue > 255.0)
		ovalue = 255.0;

	    *outpix1 = (unsigned char)ovalue;
		outpix1++;
	    }
	else if (calstr->odtype == EWORD)
	    {
	    *outpix2 = (short)ovalue;
	    outpix2++;
	    }
	else if (calstr->odtype == ELONG)
	    {
	    *outpix3 = (long)ovalue;
	    outpix3++;
	    }
	else if (calstr->odtype == EREAL)
	    {
	    *outpix4 = (float)ovalue;
	    outpix4++;
	    }

	/*
	  Increment the input buffer pointer
	*/
	inpix++;

	}

    /*
      Calculate and update the the minimum and maximum values from the line
    */
    if (strcmp(minmax,"YES") == 0)
	{
	status = c_pxmnmx((long *)(&buffer[offset2[chan] * datasize[EWORD]]),
			      &(window[NS]),&omin[chan],&omax[chan],
			      &(calstr->odtype));
	if (status != E_SUCC)
	    c_lasmsg(FALSE,"appcoeff-warn","Nonfatal error encountered");
	}
    }

return(E_SUCC);
}
