/******************************************************************************
FUNCTION:	C_CALCOEFF


PURPOSE:	Generates thermal calibration coefficients from PRT counts.
		Also apply illumination correction to optical coefficents
		if appropriate.
		NOAA averages the black body and space data over 1 calibration
		period and the PRT data over 10 calibration periods.  NOAA
		defines a calibration period to be 5 lines.  Like NOAA,
		C_CALCOEFF averages the black body and space data over 1
		calibration period.  The PRT data is averaged over the
		number of cal periods the user specifies.  Likewise, the
		number of lines in a cal period is also specified by the
		user.  The user also specifies how many lines to calibrate
		and where to start.

Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
T. Baltzer	 12/98	Developed as a front-end to ccprim() and ccupdt().
D.Lloyd          04/01  Moved the call to ccprim from the if ( firstcall )
			loop so that ccprim will be called any time the
			channel 3 sensor changes.  CR 4802
D.Lloyd          02/02  CR 5813 If ccprim() fails, set a force_prime flag so
			that on subsequent entry to this routine ccprim()
			will be called again.


PROJECT:	ADAPS

ALGORITHM REFERENCES:
                NOAA Technical Memorandum NESS 107
                  Section 5.1 AVHRR Calibration, Appendix A, Appendix B
                NOAA Polar Orbiter Data User's Guide
                  K. Kidwell, NOAA
                  Section 3.3 Calibration of AVHRR Data

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "calcoef.h"
#include "adapssup.h"
#include "labserv.h"
#include "local_protos.h"


int c_calcoeff
(
    long line,			/* INPUT: Line being processed		     */
    long mline,			/* INPUT: Image master line number	     */
    short min_frm[MNR_WORDS],	/* INPUT: Buffer containing minor frame data */
    int minfd,			/* INPUT: Minor frame file descriptor	     */
    struct MODEL *model,	/* INPUT: Model and orbital information	     */
    struct COEFFSTR *coeff,	/* INPUT/OUTPUT: All coeff (optical, thermal)*/
    struct CALSTR *calstr	/* OUTPUT: Calibration parameters structure  */
)

{

static long firstcall = TRUE;	/* First call to this routine flag T,F	*/
static long intsum[MAXTHERMBND];/* Sum of internal counts               */
static long spcsum[MAXTHERMBND];/* Sum of space counts                  */
static long cntsum[4] = {0};	/* Sum of prt sample counts             */
static long prtsmp[4] = {0};	/* Number of samples of prt counts      */
static long *prtcnt;		/* PRT counts				*/
static long prtndx[4];		/* Offsets for correct PRT sample values*/
static long prtpop; 		/* PRT population (CALPER*PRTPER)       */
static long nxtprt = 0;		/* Index in PRTCNT of where to put next	*/
				/* prt (when full it rotates around)	*/
static long slprim;		/* Starting line for calibration priming*/
static long sampls;		/* # of samples of target & space	*/
static long flag = FALSE;

static double bnmrp[MAXTHERMBND][60];
				/* Numerator of beta function phihat*deltanu */
static double enmr[MAXTHERMBND][60]; /* Numerator of exp function	*/

static struct COEFFSTR lastcoeff; /* Last set of cal coeff saved	*/
static struct OCOEFFSTR origocoeff; /* Original optical coefficients	*/

static int force_prime = FALSE;	/* flag to force coefficient priming	*/


long i;			/* Loop counter				   */
long status;		/* Return status			   */

if (firstcall)
    {

    /*
      Read and calculate the required parameters from the calibration
      tables (files) and load them into the calibration structure.
    */
    status = ccload_struct( model, calstr, coeff );
    if ( status == E_FAIL )
	return(status);


    /*
       Save the original value of the optical coefficients loaded from
       the tables.
    */
    origocoeff = coeff->ocoeff;


    /* 
      If this is the first call to this routine, allocate space for PRT counts.
      Note that this space will never be explicitly freed, but will be freed
      at the termination of the program (this assumption should be safe since
      this allocation should only occur ONE time per execution of "main").
    */
    prtcnt = (long *) malloc ((calstr->calper*calstr->prtper+1)*sizeof(long));
    if (prtcnt == NULL)
	{
	c_lasmsg(FALSE, "calcoeff-aloc", "Error dynamically allocating memory");
	return(E_FAIL);
	}

    /*
      Initialize the number of optical bands spec so as to force the first
      call to ccsconfig() to detect a change in configuration, thereby
      forcing a call to ccprim().  This will ensure that the thermal
      calibration algorithm is properly initialized on the first line of
      the image to be processed.
    */
    calstr->nopticbnds = 0;
    calstr->nthermbnds = 0;

    /*
      Set flag so that the first call to this routine is false.
    */
    firstcall = FALSE;
    }

/*
  Determine if the sensor recording configuration has changed.
*/
status = ccsconfig(min_frm,model,calstr);
if (status != E_SUCC)
    return(status);

/* If the sensor has changed, channel 3 must be primed if the sensor went
from a visible to a thermal.  Since it isn't known what the actual change
is, at this point, prime the channel 3 sums.
CR 5813: if the force_prime flag is TRUE a previous ccprim() call failed.
This can happen when the channel 3 sensor changes near the beginning or
end of the scene so enter as though the sensor changed.  Noisy data can
also cause this to happen near the beginning and end of a scene.
-------------------------------------------------------------------------*/
if (calstr->reconfig == TRUE || force_prime )
    {
    intsum[0] = spcsum[0] = 0;
    if ((min_frm[0] != 0)  || (min_frm[9] != 0) ||
        (min_frm[10] != 0) || (min_frm[11] != 0))
        {
	if ( sampls > 10 )
	    {
	    for ( i = 0; i < 10; i++ )
		{
		intsum[0] += min_frm[22 + (i * 3)];
		spcsum[0] += min_frm[54 + (i * 5)];
		}
	    intsum[0] *= sampls / 10;
	    spcsum[0] *= sampls / 10;
	    }
	}
    slprim = line;

    /* Reprime the coefficients on any channel 3 sensor change.  CR 4802
    (For LAS image i/o, must first rewind the minor frame file since
    we already read one line further than what ccprim is expecting).
    --------------------------------------------------------------------*/
    if (calstr->file_type == LASIMAGE)
	{
	status = c_lsrew( (long *)&minfd );
	if ( status != E_SUCC )
	return(status);
	}

    status = ccprim(calstr, slprim, mline, coeff, &lastcoeff, &origocoeff,
		    prtcnt, prtndx, &prtpop, (long *)&minfd, bnmrp, enmr,
		    model, intsum, spcsum );
    /* CR 5813: set the force_prime flag appropriately
    --------------------------------------------------*/
    if (status != E_SUCC)
	{
	force_prime = TRUE;
	return(status);
	}
    else
        force_prime = FALSE;

    flag = TRUE;
    }


/* Update the thermal calibration coefficients */
calstr->nprdflg = ccupdt(calstr,line,mline,min_frm,coeff,&lastcoeff,&origocoeff,
			 prtcnt,cntsum,prtsmp,prtndx,&nxtprt,prtpop,&sampls,
			 intsum,spcsum,slprim,bnmrp,enmr,model,flag);
flag = FALSE;


return(E_SUCC);
}

