/*******************************************************************************
FUNCTION: c_getdirs

PURPOSE:  Given a file specification find all subdirectories for this
	  specification.  As an example if the file specification is 
	  /sg2/csb/user/mydir/ * or /sg2/csb/user/mydir/ then c_getdirs
	  will return all subdirectories in /sg2/csb/user/mydir/.  If
          the file specification is /sg2/csb/user/mydir then c_getdirs
          will return all subdirectories in /sg2/csb/user/.

PROGRAM HISTORY:

Version  Author           Date   Modification
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          05/92  Revision of getdirs that will work correctly 
				 on both UNIX and VMS systems.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under LAS

PROJECT: LAS

ALGORITHM REFERENCES: LAS support getdirs
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "tae.h"
#include "cm.h"

/*****************
*  UNIX VERSION  *
*****************/
#ifdef unix

int c_getdirs
(
    char *filespec,           /* File specification for location of subdirs */
    char dir_names[][CMLEN],  /* Array of directory names to be returned    */
    long *ndirs,              /* Number of directories found                */
    long maxdirs,             /* Max number of directories to return        */
    long first_call           /* Flag for first call to this function (0,1) */
)
{
char dirspec[CMLEN];         /* Directory specification                    */
char filename[CMLEN];        /* Name of one file                           */
char *ptr;                   /* Character pointer                          */
char sys_command[CMLEN];     /* System command which produces file names   */

long i;                      /* Loop control                               */
long nonfatal = 1;           /* Nonfatal error flag                        */

static FILE *file_ptr;       /* Temp file ptr to the list of file names    */

#ifdef _IBMR2
   short c;                  /* char returned by fget */
#else
#ifdef sgi
   short c;                  /* char returned by fget */
#else
#ifdef DGUX
   short c;                  /* char returned by fget */
#else
   char c;                   /* char returned by fget */
#endif
#endif
#endif


/* Ensure that maxdirs is at least one and initialize all variables
-------------------------------------------------------------------*/
if (maxdirs < 1)
    return(E_FAIL);
*ndirs = 0;
i = 0;
strcpy(filename, "");


/* Transform the file specification
-----------------------------------*/
strcpy(dirspec, filespec);
ptr = (char *)strrchr(dirspec, '/');
if (ptr == NULL)
    strcpy(dirspec, "*");
else
    {
    *ptr = '\0';
    strcat(dirspec, "/*");
    }


/* Create a file with the list of desired file names.  If the file
   specification uses an asterisk and no files are found that match, 
   then the file will contain the given specification.  Therefore, if 
   an asterisk is present then no files were found.
---------------------------------------------------------------------------*/
if (first_call == 1)
    {	
    sprintf(sys_command, "/bin/echo %s", dirspec);
    file_ptr = popen(sys_command, "r");
    if (file_ptr == NULL)
	{
	c_errmsg("Error opening temporary file", "c_getdirs-open",
	    	 &nonfatal);
	return(E_FAIL);
	}
    }


/* Get the desired number of directory names from the temporary file
--------------------------------------------------------------------*/
do  {
    c = fgetc(file_ptr);

    if (c == '*')
	{
	*ndirs = 0;
	pclose(file_ptr);
	return(E_SUCC);
	}
    else if ((c != ' ') && (c != '\n'))
	filename[i++] = c;
    else if ((c == ' ') || (c == '\n') || ((c == EOF) && (i > 0)))
	{
	filename[i] = '\0';
	if (isdir(filename))
	    strcpy(dir_names[(*ndirs)++], filename);
	strcpy(filename, "");
	i = 0;
	}
    } while ((c != EOF) && (*ndirs < maxdirs));


/* Close the temporary file if done and return successfully
-----------------------------------------------------------*/
if (*ndirs < maxdirs) 
    pclose(file_ptr);
return (E_SUCC);
}

#endif



/****************
*  VMS VERSION  *
****************/
#ifdef vms
#include "desc.h"

static long num_dirs_reported; /* Number of directories reported so far    */


FUNCTION c_getdirs(filespec, dir_names, ndirs, maxdirs, startf)

   char *filespec;          /* File specification of desired files to find */
   char dir_names[][CMLEN]; /* Names of all the files found                */

   long  maxdirs;           /* Max number of directories to report         */
   long  *ndirs;            /* Number of directories found                 */
   long  startf;            /* Flag for first call or not (0,1)            */

{
char dirspec[CMLEN];        /* Directory specification                     */
char filerslt[CMLEN];       /* Resultant file name from genf               */
char *ptr;                  /* Ptr to strings                              */
char start;                 /* Sent to genf - is this the first call? (0,1)*/
char *strcpy();             /* function for copying strings                */
char temp[CMLEN];           /* Temp string to hold local modifications     */
char wildstr[CMLEN];        /* Holds wildcard file specification           */

long count;                 /* Loop control                                */
long i;                     /* Loop control                                */
long status;                /* Function return status                      */
long dir_found;             /* Flag indicating if genf found a directory   */

long nonfatal = 1;          /* Nonfatal error flag                         */

short int name_size;        /* Size of directory name returned by genf     */

struct DESC wild;           /* Wildcard file spec passed to genf           */
struct DESC file;           /* Filename returned by genf                   */

FILE *tmpfp, *tmpfile();    /* File ptrs                                   */


   /******************************************************************
   * This routine builds a list of file names from a "wildcard" file *
   * specification.  The genf routine (see worrmsio.c) is called to  *
   * actually generate the filenames (VMS).                          *
   ******************************************************************/

/* Ensure that maxdirs is at least one
--------------------------------------*/
if (maxdirs < 1)
    return(E_FAIL);


/* Transform the file specification
-----------------------------------*/
strcpy(dirspec, filespec);
ptr = strrchr(dirspec, ']');
if (ptr == NULL)
    strcpy(dirspec, "*");
else
    {
    *ptr = '\0';
    strcat(dirspec, "]*");
    }


/* Create a temporary file
--------------------------*/
tmpfp = tmpfile();
if (tmpfp == NULL)
    {
    c_errmsg("Error opening temporary file","c_getdirs-open",&nonfatal);
    return(E_FAIL);
    }


/* Build a descriptor pointing to the file specification
--------------------------------------------------------*/
wild.addr   = dirspec;
wild.length = strlen(wild.addr);


/* Build a descriptor pointing to the first entry in
   the filelist array to receive the first file name.
-----------------------------------------------------*/
file.addr   = filerslt;
file.length = 100;


/* Call genf (see worrmsio.c) to get filenames,
   one at a time, from the wildcard string.
-----------------------------------------------*/
start     = TRUE;	
dir_found = FALSE;
status    = genf(&wild, &file, &name_size, &start, &dir_found);
while ((status == E_SUCC) && dir_found)
    {
    filerslt[name_size] = '\0';

    /* lose the version number
    --------------------------*/
    strcpy(temp, filerslt);
    ptr = strchr(temp,';');
    *ptr = '\0';
    fprintf(tmpfp, "%s\n", temp);
    start = FALSE;  /* indicates this is not the first call to genf */
    status = genf(&wild, &file, &name_size, &start, &dir_found);
    }

/* Reset file to the beginning to read from it
----------------------------------------------*/
fseek(tmpfp,0,0);

/* Skip all of the directories that have already been reported
--------------------------------------------------------------*/
if (startf)
    num_dirs_reported = 0;
else
    {
    count = 0;
    while (count < num_dirs_reported)
	{
	status = fgets(temp, CMLEN, tmpfp);
	if (temp[0] == '\0')
	    break;

	/* Anything ending in .dir is assumed to be a directory
	-------------------------------------------------------*/
	ptr = strrchr(temp, '.');
	if ((ptr != NULL) && (strncmp(ptr, ".DIR", 4) == 0))
	    count++;
	}
    }


/* Find all directory names up to maxdirs
-----------------------------------------*/
*ndirs = 0;
while (*ndirs < maxdirs)
    {
    status = fgets(temp, CMLEN, tmpfp);
    if (temp[0] == '\0')
	break;
    i = strlen(temp);
    temp[i-1] = '\0';

    /* Anything ending in .dir is assumed to be a directory
    -------------------------------------------------------*/
    ptr = strrchr(temp, '.');
    if ((ptr != NULL) && (strncmp(ptr, ".DIR", 4) == 0))
	{
	/* Remove the .dir and make temp a directory specification
	----------------------------------------------------------*/
	ptr = strrchr(temp, ']');
	*ptr = '.';
	ptr = strrchr(temp, '.');
	*ptr = '\0';
	strcat(temp, "]");
	strcpy(dir_names[(*ndirs)++], temp);
	}
    }
fclose(tmpfp);
num_dirs_reported += *ndirs;
return (E_SUCC);
}
#endif
 
