/*******************************************************************************
FUNCTION:	C_GETKVALS


PURPOSE:	Return the desired K values for each of the NOAA AVHRR
		satellites.  These values are used to convert to/from
		calibrated radiance to reflectance.

		The equations for these conversions are:

		    reflectance = radiance * K

		    radiance = reflectance / K
		
		where

		        100 * PI * W
		    K = ------------
			      F
		
		    F = Integrated solar spectral irradiance, weighted by the
			spectral response function of the channel in watts/m*m
		    W = Equivalent width of the spectral response function in
			micro-meters


Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
TBaltzer         12/98  Development

*******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "calcoef.h"
#include "adapssup.h"
#include "local_protos.h"


int c_getkvals
(
    long satnum,	/* IN  - Satnum (0=tiros-n, 1=noaa-1, 2=noaa-2, ...)  */
    char ocalmeth[],	/* IN  - Type kaufman, prelaunch, teillet, vermote,rao*/
    double kvals[]	/* OUT - K values returned			      */
)
{
char kvalfile[CMLEN];	/* Calibration file name			      */
char buffer[MAXBUF];	/* buffer to read data into			      */

long end;		/* Ending satellite number in the table		      */
long fd;		/* file descriptor				      */
long i;			/* Loop control                                       */
long maxoptic;		/* Max number of optical channels for this satellite  */
long start;		/* Starting satellite number in the table	      */

/*
  Read the appropriate table into memory
*/
strcpy( kvalfile, getenv( "ADAPSTABLES" ));
sprintf ( kvalfile, "%s/%s%s", kvalfile, ocalmeth, ".cal" );


/*
  Open the file
*/
fd = open( kvalfile, 0 );


/* Skip down to the K values */
if (ccfindstr(fd, buffer, "OPTICAL K VALUES:") != E_SUCC)
    return(E_FAIL);

/* Skip header info */
if (ccreadline(fd,3,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the starting and ending satellite numbers */
sscanf( buffer, "%d %d", &start, &end);

/* Check to see if there is info for this satellite */
if ((satnum < start) || (satnum > end))
    {
    c_lasmsg(FALSE,"getkvals-read",
	     "K values unavailable for NOAA-%d, %s method", satnum, ocalmeth);
    return(E_FAIL);
    }

/* Skip header info */
if (ccreadline(fd,3,buffer) != E_SUCC)
    return(E_FAIL);

/* Read down into the table the appropriate number of lines */
if (ccreadline(fd,(satnum - start + 1),buffer) != E_SUCC)
    return(E_FAIL);

/* Load the K values */
sscanf(buffer, "%lf %lf %lf", &kvals[0], &kvals[1], &kvals[2]);

/* For satellites before NOAA-15 (NOAA-KLM), 2 optical channels max,
   3 thereafter */
if (satnum < 15)
    maxoptic = 2;
else
    maxoptic = 3;

/* If any of the values are zero, they are invalid */
for (i = 0; i < maxoptic; i++)
    if (kvals[i] == 0.0)
	{ 
	c_lasmsg(FALSE,"getkvals-zero","No %s K values for NOAA-%d",
		 ocalmeth, satnum);
	return(E_FAIL);
	}

/* Close the degradation table */
close(fd);

return(E_SUCC);
}
