/*******************************************************************************
FUNCTION:	c_getscale()

PURPOSE:	Given a scaling option and a data type retrieve the scaling
		factor and offset.


Version  Date   Programmer       Modification
-------  -----  ---------------  -----------------------------------------------
  1.0    10/94  J.Fenno          Development
  1.1    11/94  J.Fenno          Modified to get rid of the shift value.
				 Although a shift value is more clear for users,
				 it adds one extra addition/subtraction when
				 scaling and unscaling.


PROJECT:	ADAPS

ALGORITHM:
    In the documentation, the equations for scaling are:

	scaled = ((actual + shift) * scale) + offset
	actual = (scaled - offset) / scale - shift
    
    For efficiency, we have combined the shift and offset values so only one
    addition or subtraction is performed for each pixel.  We have also changed
    the division in the unscaling equation to a multiplication.  The true
    equations are:

	scaled = (actual * scale) + offset
	actual = (scaled - offset) * unscale
    
    where:

	unscale = 1.0 / scale
    
    See the macros SCALE and UNSCALE which are defined in
    $ADAPSINCLUDE/constants.h.

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "c_getscale.h"


/* Internal defines to prevent multiple strcmp() calls
------------------------------------------------------*/
#define SATZENITH   0
#define SOLZENITH   1
#define RELAZIMUTH  2
#define REFLECTANCE 3
#define RADIANCE    4
#define THERMAL     5


int c_getscale
(
    long option,	/* IN  - Scaling option (US_BYTE, US_I2, US_I4, ...)  */
    char value[],	/* IN  - String indicating values being scaled        */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for the specified option      */
    double *unscale,	/* OUT - Unscaling factor for the specified option    */
    double *offset	/* OUT - Offset value for the specified option        */
)
{
char str[CMLEN];	/* Temporary string                                   */

long len;		/* Length of the input string                         */
long nonfatal = 1;	/* Nonfatal error flag                                */
long units;		/* Flag indicating units                              */


/* Initialize the output values
-------------------------------*/
*minimum    = 0.0;
*maximum    = 0.0;
*scale      = 1.0;
*unscale    = 1.0;
*offset     = 0.0;


/* Copy the specified string and lowercase it
---------------------------------------------*/
strncpy(str, value, CMLEN);
len = (long)strlen(str);
c_up2low(str, &len);


/* Set a flag for the specified units
-------------------------------------*/
if (strncmp(str, "satzenith", 9) == 0)
    units = SATZENITH;
else if (strncmp(str, "solzenith", 9) == 0)
    units = SOLZENITH;
else if (strncmp(str, "relazimuth", 10) == 0)
    units = RELAZIMUTH;
else if (strncmp(str, "reflectance", 11) == 0)
    units = REFLECTANCE;
else if (strncmp(str, "radiance", 8) == 0)
    units = RADIANCE;
else if (strncmp(str, "thermal", 7) == 0)
    units = THERMAL;
else
    {
    c_errmsg("Invalid units", "c_getscale-units", &nonfatal);
    return(E_FAIL);
    }


/* Call the correct function to set the scale and offset parameters
-------------------------------------------------------------------*/
switch (option)
    {
    case NO_SCALING:
	no_scale(units, minimum, maximum, scale, offset);
	break;
    
    case US_BYTE:
	us_byte(units, minimum, maximum, scale, offset);
	break;

    case US_I2:
	us_i2(units, minimum, maximum, scale, offset);
	break;

    case US_I4:
	us_i4(units, minimum, maximum, scale, offset);
	break;

    case US_REAL:
	us_real(units, minimum, maximum, scale, offset);
	break;

    case AVHRR_BYTE:
	avhrr_byte(units, minimum, maximum, scale, offset);
	break;

    case AVHRR_10BIT:
	avhrr_10bit(units, minimum, maximum, scale, offset);
	break;

    case AVHRR_I2:
	avhrr_i2(units, minimum, maximum, scale, offset);
	break;

    case AVHRR_I4:
	avhrr_i4(units, minimum, maximum, scale, offset);
	break;

    case AVHRR_REAL:
	avhrr_real(units, minimum, maximum, scale, offset);
	break;

    default:
	c_errmsg("Invalid scaling option", "c_getscale-opt", &nonfatal);
	return(E_FAIL);
	break;
    }


/* Determine the unscaling factor.  If we use the inverse of the scaling
   factor, we can use a multiply when we unscale.  This will save a significant
   amount of cpu time.
------------------------------------------------------------------------------*/
*unscale = 1.0 / *scale;

return(E_SUCC);
}


/*############################################################################*/
static int no_scale
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    = -180.0;
	*maximum    =  180.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case SOLZENITH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case RELAZIMUTH:
	*minimum    = -180.0;
	*maximum    =  180.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case REFLECTANCE:
	*minimum    =    0.0;
	*maximum    =  100.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case RADIANCE:
	*minimum    =    0.0;
	*maximum    =  540.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case THERMAL:
	*minimum    =  160.0;
	*maximum    =  340.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int us_byte
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =  -90.000;
	*maximum    =   90.000;
	*scale      =    1.000;
	*offset     =   90.000;
        break;

    case SOLZENITH:
	*minimum    =    0.000;
	*maximum    =  180.000;
	*scale      =    1.000;
	*offset     =    0.000;
        break;

    case RELAZIMUTH:
	*minimum    =    0.000;
	*maximum    =  180.000;
	*scale      =    1.000;
	*offset     =    0.000;
        break;

    case REFLECTANCE:
	*minimum    =    0.000;
	*maximum    =   63.000;
	*scale      =    4.000;
	*offset     =    0.000;
        break;

    case RADIANCE:
	*minimum    =    0.000;
	*maximum    =  333.000;
	*scale      =    0.766;
	*offset     =    0.000;
        break;

    case THERMAL:
	*minimum    =  203.000;
	*maximum    =  330.000;
	*scale      =    2.000;
	*offset     = -405.000;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int us_i2
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =  -90.0;
	*maximum    =   90.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case SOLZENITH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case RELAZIMUTH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case REFLECTANCE:
	*minimum    =    0.0;
	*maximum    =  100.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case RADIANCE:
	*minimum    =    0.0;
	*maximum    =  540.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case THERMAL:
	*minimum    =  160.0;
	*maximum    =  340.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int us_i4
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =  -90.0;
	*maximum    =   90.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case SOLZENITH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case RELAZIMUTH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case REFLECTANCE:
	*minimum    =    0.0;
	*maximum    =  100.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case RADIANCE:
	*minimum    =    0.0;
	*maximum    =  540.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;

    case THERMAL:
	*minimum    =  160.0;
	*maximum    =  340.0;
	*scale      =   10.0;
	*offset     =    0.0;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int us_real
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =  -90.0;
	*maximum    =   90.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case SOLZENITH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case RELAZIMUTH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case REFLECTANCE:
	*minimum    =    0.0;
	*maximum    =  100.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case RADIANCE:
	*minimum    =    0.0;
	*maximum    =  540.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;

    case THERMAL:
	*minimum    =  160.0;
	*maximum    =  340.0;
	*scale      =    1.0;
	*offset     =    0.0;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int avhrr_byte
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =  -90.000;
	*maximum    =   90.000;
	*scale      =    1.000;
	*offset     =  100.000;
        break;

    case SOLZENITH:
	*minimum    =    0.000;
	*maximum    =  180.000;
	*scale      =    1.000;
	*offset     =   10.000;
        break;

    case RELAZIMUTH:
	*minimum    = -180.000;
	*maximum    =  180.000;
	*scale      =    0.500;
	*offset     =  100.000;
        break;

    case REFLECTANCE:
	*minimum    =    0.000;
	*maximum    =  100.000;
	*scale      =    1.000;
	*offset     =   10.000;
        break;

    case RADIANCE:
	*minimum    =    0.000;
	*maximum    =  540.000;
	*scale      =    0.454;
	*offset     =   10.000;
        break;

    case THERMAL:
	*minimum    =  160.000;
	*maximum    =  340.000;
	*scale      =    1.359;
	*offset     = -207.440;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int avhrr_10bit
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =  -90.000;
	*maximum    =   90.000;
	*scale      =    1.000;
	*offset     =  100.000;
        break;

    case SOLZENITH:
	*minimum    =    0.000;
	*maximum    =  180.000;
	*scale      =    1.000;
	*offset     =   10.000;
        break;

    case RELAZIMUTH:
	*minimum    = -180.000;
	*maximum    =  180.000;
	*scale      =    1.000;
	*offset     =  190.000;
        break;

    case REFLECTANCE:
	*minimum    =    0.000;
	*maximum    =  100.000;
	*scale      =   10.000;
	*offset     =   10.000;
        break;

    case RADIANCE:
	*minimum    =    0.000;
	*maximum    =  540.000;
	*scale      =    1.874;
	*offset     =   10.000;
        break;

    case THERMAL:
	*minimum    =  160.000;
	*maximum    =  340.000;
	*scale      =    5.602;
	*offset     = -886.320;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int avhrr_i2
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =   -90.0;
	*maximum    =    90.0;
	*scale      =    10.0;
	*offset     =   910.0;
        break;

    case SOLZENITH:
	*minimum    =     0.0;
	*maximum    =   180.0;
	*scale      =    10.0;
	*offset     =    10.0;
        break;

    case RELAZIMUTH:
	*minimum    =  -180.0;
	*maximum    =   180.0;
	*scale      =    10.0;
	*offset     =  1810.0;
        break;

    case REFLECTANCE:
	*minimum    =     0.0;
	*maximum    =   100.0;
	*scale      =    10.0;
	*offset     =    10.0;
        break;

    case RADIANCE:
	*minimum    =     0.0;
	*maximum    =   540.0;
	*scale      =    10.0;
	*offset     =    10.0;
        break;

    case THERMAL:
	*minimum    =   160.0;
	*maximum    =   340.0;
	*scale      =    10.0;
	*offset     = -1590.0;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int avhrr_i4
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =    -90.0;
	*maximum    =     90.0;
	*scale      =    100.0;
	*offset     =   9010.0;
        break;

    case SOLZENITH:
	*minimum    =      0.0;
	*maximum    =    180.0;
	*scale      =    100.0;
	*offset     =     10.0;
        break;

    case RELAZIMUTH:
	*minimum    =   -180.0;
	*maximum    =    180.0;
	*scale      =    100.0;
	*offset     =  18010.0;
        break;

    case REFLECTANCE:
	*minimum    =      0.0;
	*maximum    =    100.0;
	*scale      =    100.0;
	*offset     =     10.0;
        break;

    case RADIANCE:
	*minimum    =      0.0;
	*maximum    =    540.0;
	*scale      =    100.0;
	*offset     =     10.0;
        break;

    case THERMAL:
	*minimum    =    160.0;
	*maximum    =    340.0;
	*scale      =    100.0;
	*offset     = -15990.0;
        break;
    }
return(E_SUCC);
}


/*############################################################################*/
static int avhrr_real
(
    long units,		/* IN  - Units to return the scaling factor for       */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for 'units'                   */
    double *offset	/* OUT - Offset value for 'units'                     */
)
{
switch (units)
    {
    case SATZENITH:
	*minimum    =  -90.0;
	*maximum    =   90.0;
	*scale      =    1.0;
	*offset     =  100.0;
        break;

    case SOLZENITH:
	*minimum    =    0.0;
	*maximum    =  180.0;
	*scale      =    1.0;
	*offset     =   10.0;
        break;

    case RELAZIMUTH:
	*minimum    = -180.0;
	*maximum    =  180.0;
	*scale      =    1.0;
	*offset     =  190.0;
        break;

    case REFLECTANCE:
	*minimum    =    0.0;
	*maximum    =  100.0;
	*scale      =    1.0;
	*offset     =   10.0;
        break;

    case RADIANCE:
	*minimum    =    0.0;
	*maximum    =  540.0;
	*scale      =    1.0;
	*offset     =   10.0;
        break;

    case THERMAL:
	*minimum    =  160.0;
	*maximum    =  340.0;
	*scale      =    1.0;
	*offset     = -150.0;
        break;
    }
return(E_SUCC);
}
