/*******************************************************************************
Purpose:
    This routine performs a query on the WVS for the given 
    file and area of interest.

    The WVS files are split up by 10 degree latitude ranges within each
    coverage area, starting at the south (minimum latitude) and moving north.  
    The files are named "wvs[coverage index][area index].data".
    Thus, the Southern Ocean files are such:
    wvs000.data -- -90.0 to -80.0 latitude
    wvs001.data -- -80.0 to -70.0 latitude
    wvs002.data -- -70.0 to -60.0 latitude

 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    ------          ------
  1.0           06/94   T. Rockvam      Original development
  2.0           10/94   T. Rockvam      Split up WVS files to speed up reading
 
PROJECT:  LAS
*******************************************************************************/
#include <malloc.h>
#include "las.h"
#include "wvs.h"
 
int c_wvsquery
(
    char *path,				/* location of WVS files	*/
    char *wvsfile,			/* WVS file name		*/
    struct WVS_VECTOR *wvs_vector,	/* resulting vector information	*/
    double lllat,			/* bounding rect - minlat	*/
    double lllon,			/*		   minlon	*/
    double urlat,			/*		   maxlat	*/
    double urlon 			/*		   maxlon	*/
)
{
FILE	*fd;				/* wvs file			*/

char  	filename[CMLEN];                /* name of wvs file             */
long	nonfatal = 1;			/* first file in area to read	*/
short	numpts;				/* number of points per vector	*/
float	minmax[4];			/* min lat,long, max lat,long	*/
double  limit;				/* when to stop searching for lines*/

/*
  Open the file
*/
sprintf(filename,"%s/%s",path,wvsfile);
fd = fopen(filename,"r");
if (fd == NULL)
    {
    c_errmsg("Error opening WVS file\n","wvsquery-open",&nonfatal);
    return(E_FAIL);
    }

/*
  Start reading
*/
limit = urlat + 1.0;
while (fread(&numpts,sizeof(short),1,fd) == 1)
    {
    /*
      Read the lat/long limits of this line
    */
    if (fread(minmax,sizeof(float),4,fd) != 4)
	{
    	c_errmsg("Error reading WVS file\n","wvsquery-read",&nonfatal);
    	fclose(fd);
	return(E_FAIL);
	}
    if ((urlat < minmax[0]) ||
        (lllat > minmax[2]) ||
        (urlon < minmax[1]) ||
        (lllon > minmax[3]))
	{
	/*
	  If we are past our area of interest, skip
	  reading the rest of this file
	*/
    	if (minmax[0] >= limit) 
	    break;

	/*
	  Skip the points in this line & go on
	*/
	if (fseek(fd,(long)(numpts*2*sizeof(float)),1) != 0)
	    {
    	    c_errmsg("Error reading WVS file\n","wvsquery-read",&nonfatal);
    	    fclose(fd);
	    return(E_FAIL);
	    }
	}
    else
	{
	/*
	  Read the points in this line
	*/
	wvs_vector->line[wvs_vector->numlines].pts = 
	    (float*)malloc(numpts * 2 * sizeof(float));
	wvs_vector->line[wvs_vector->numlines].npts = numpts;
    	if (fread(wvs_vector->line[wvs_vector->numlines].pts,
	    sizeof(float),numpts*2,fd) != numpts*2)
	    {
    	    c_errmsg("Error reading WVS file\n","wvsquery-read",&nonfatal);
	    free(wvs_vector->line[wvs_vector->numlines].pts);
	    wvs_vector->line[wvs_vector->numlines].npts = 0;
    	    fclose(fd);
	    return(E_FAIL);
	    }
	wvs_vector->numlines++;
	if (wvs_vector->numlines == wvs_vector->maxline)
    	    {
    	    wvs_vector->maxline += LINEBLOCK;
    	    wvs_vector->line = (struct WVS_LINE *)realloc(wvs_vector->line,
		    wvs_vector->maxline * sizeof(struct WVS_LINE));
    	    }
	}
    }

fclose(fd);

return(E_SUCC);
}
