/******************************************************************************
FUNCTION:	CCCALC_C0

PURPOSE:	Calculate the counts of space from the
		values read from the appropriate table (file).

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
  1.0	09/98	T. Baltzer	Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
******************************************************************************

Calibration equations that are used :

        S = Kalpha

        I = - Kalpha * C0

   Kalpha = Kval * (St * nmo + It)

where

       St = Slope value from prelaunch.dgrd table
       It = Intercept value from prelaunch.dgrd table
      nmo = number of months since launch
     Kval = K value to convert from radiance/reflectance
   Kalpha = Kval * post-launch gain-compensation
       C0 = counts of space
        S = Slope value written to the label table, value defined
            in NOAA documentation
        I = Intercept value written to the label table, value defined
            in NOAA documentation

Now by definition: St = 0.0 (degradation over time is not known at
prelaunch), then the equation:

   Kalpha = Kval * (St * nmo + It)    simplifies to:

   Kalpha = Kval * It


How the C0 values are determined:

Remember from above that:

        S = Kalpha

        I = - Kalpha * C0

So transpose to solve for C0:

       C0 = - I / Kalpha

       C0 = - I / S

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "calcoef.h"

void cccalc_C0
(
    struct CALSTR *calstr  /* Calibration parameter structure		*/
)

{
long i,j;		/* Counters */


for (i = 0; i < MAXOPTICBND; i++)
    for (j = 0; j < 2; j++)
	{
	if (calstr->vsgain[i][j] == 0.0)
	    calstr->C0[i][j] = 0.0;
	else
	    calstr->C0[i][j] = - calstr->vsntcp[i][j] / calstr->vsgain[i][j];
	}

}


/******************************************************************************
FUNCTION:	CCCALC_DGRD

PURPOSE:	Calculate the degradation slope/intercept values for
		PRELAUNCH conditions.  Calculated from information
		supplied by NOAA.

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
  1.0	09/98	T. Baltzer	Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
******************************************************************************

Calibration equations that are used :

        S = Kalpha

        I = - Kalpha * C0

   Kalpha = Kval * (St * nmo + It)

where

       St = Slope value to be calculated by this routine
       It = Intercept value to be calculated by this routine
      nmo = number of months since launch
     Kval = K value to convert from radiance/reflectance
   Kalpha = Kval * post-launch gain-compensation
       C0 = counts of space
        S = Slope value written to the label table, value defined
            in NOAA documentation
        I = Intercept value written to the label table, value defined
            in NOAA documentation

Now by definition: St = 0.0 (degradation over time is not known at
prelaunch), then the equation:

   Kalpha = Kval * (St * nmo + It)    simplifies to:

   Kalpha = Kval * It

   It = Kalpha / Kval   (transpose)

Remember from above that:

        S = Kalpha

So to determine St/It values:

   It = S / Kval
   St = 0.0


******************************************************************************/

void cccalc_dgrd
(
    struct CALSTR *calstr,  /* Calibration parameter structure		*/
    struct COEFFSTR *coeff /* OUT - Calibration coefficient structure	*/
)

{
long i,j;		/* Counters */


for (i = 0; i < MAXOPTICBND; i++)
    for (j = 0; j < 2; j++)
	{
	coeff->ocoeff.order1[i][j] = 0.0;
	coeff->ocoeff.order0[i][j] = calstr->vsgain[i][j] / calstr->Kvals[i];
	coeff->ocoeff.count[i] = calstr->vscnt[i];
	}

return;
}

