/*******************************************************************************
FUNCTION:	CCGEN_LUTS

PURPOSE:	Generate the reflectance and temperature look-up tables for
		the specified channels.  The look-up tables contain data for
		the ten bit data range (0-1023).

ALGORITHM:

VISIABLE:
   Reflectance formula for channels 1, 2, and 3a

	A = G*D + I

RADIANCE:
   The following formula is used to convert the reflectance value to radiance:

        R = A (F / (100 * PI * W))

   Were:
        R - Radiance
        A - Reflectance
        F - Integrated solar spectral irradiance,
        W - Equivalent width of the spectral response function

   Table 3.3.2-2, on page 3-19 of the NOAA Polar Orbiter Data Users Guide,
   July, 1991, contains the values of W and F derived from Neckel and Labs
   (1984)


THERMAL:
   Temperature formula for channels 3, 4, and 5 (infared)

	E = order0 + order1*D + order2*D*D

   order2 is zero (0) for channel 3, non-zero for channels 4 and 5.

   Energy is then converted in temperature (T) by using the inverse of
   Planck's radiation function as shown in the following equation.

	            C2*CWN
             ___________________
     T =
                  C1 * CWN**3
          ln( 1 + ___________ )
                      E

   Where E is energy from the above formula, C1 is a constant equal to
   1.1910659E-5, C2 is a constant equal to 1.438833, and CWN is the
   central wave number given for each channel of each satellite for
   a specified surface temperature.  

   Then:

     T = A + (B * Te)

   Where Te is the "effective" temperature.
   EXPLAINATION:  For satellite prior to NOAA-KLM, there are 4 CWN to
   use, depending on the estimated temperature range of the surface
   being analyzed.  For these satellites, the A and B correction
   coefficients are 0.0 and 1.0 respectively (no correction).  For
   NOAA-KLM, there is only one CWN for all temperatures, but correction
   coefficients A and B to apply to the "effective" temperature coming
   out of the inverse of Planck's radiation function.

   When the temperature is less than 160 degrees Kelvin the lower
   temperature limit has been reached.  Any values below this are
   set to a constant limit value (160).  When the temperature is greater
   than 340 degrees Kelvin the upper temperature limit has been reached.
   Any values above this are set to a constant limit value (340).

ALGORITHM REFERENCES
  Katherine B. Kidwell, NOAA Polar Orbiter Data Users Guide
  National Oceanic and Atmospheric Administration, Washington, D.C.
  July, 1991.

  Table 3.3.2-2, on page 3-19 of the NOAA Polar Orbiter Data Users Guide,
  July, 1991, contains the values of W and F derived from Neckel and Labs
  (1984)

  Geoffrey Goodrum, Katherine B. Kidwell, Wayne Winston, NOAA KLM User's Guide,
  National Oceanic and Atmospheric Administration, Washington, D.C.
  March, 1999.
*******************************************************************************/
#include <stdio.h>		/* Standard I/O include			*/
#include <math.h>		/* Math library				*/
#include "las.h"		/* LAS include file			*/
#include "adaps.h"		/* ADAPS include file			*/
#include "adapssup.h"		/* ADAPS supports include file		*/
#include "calcoef.h"            /* Calibration coefficient structure    */
#include "local_protos.h"       /* local ADAPS support include file	*/

void ccgen_luts
(
    struct CALSTR *calstr,/* Calibration structure			*/
    struct MODEL *model,  /* AVHRR model structure			*/
    long channel[],	  /* Channels to process			*/
    long nchans,	  /* Number of channels to process		*/
    struct COEFFSTR *coeff,/* Calibration coefficients			*/
    float cwncon2[],	  /* CWN times constant two			*/
    float cwn3con1[],	  /* CWN times constant one			*/
    struct SCALESTR *a_scale, /* Scaling values for reflectance		*/
    struct SCALESTR *r_scale, /* Scaling values for radiance		*/
    struct SCALESTR *t_scale, /* Scaling values for temperature		*/
    float lut[][MAXDAT]  /* Temperature look-up table for the channel	*/
)

{
float	reflect,	/* Reflectance value				*/
	rad,		/* Radiance value				*/
	energy,		/* Energy					*/
	temperature;	/* Temperature Kelvin values			*/

long	chan, samp,	/* Looping variables				*/
	temp_limit[3];	/* Temperature limit value			*/


/*
  Process each channel
*/
for (chan = 0; chan < nchans; chan++)
    {
    /*
      Reflectance formula for channels 1, 2, and 3a (visible)
    */
    if (channel[chan] <= calstr->nopticbnds)
	{
	/*
	  Generate the reflectance/radiance LUT for the valid 10 bit
	  data range (0-1023).  Always make the 0'th element MASK_NO_DATA
	  since input sensor values of 0 are treated as no data.
	*/
	lut[channel[chan]-1][0] = MASK_NO_DATA;
	for (samp = 1; samp < MAXDAT; samp++)
	    {
	    /*
	      Calculate the reflectance.  Use the high albedo coefficient if:
	      - there is one for this satellite, and
	      - the raw value is high enough to use the high albedo value.
	      Otherwise, use the low albedo coefficient.  (If there is only
	      one coefficient for this satellite, it is stored as the
	      "zero'th", or low albedo value).
	    */
	    if ((coeff->ocoeff.count[channel[chan]-1] != 0) &&
		(samp >= coeff->ocoeff.count[channel[chan]-1]))
		reflect = (coeff->ocoeff.order1[channel[chan]-1][1] * samp) +
			   coeff->ocoeff.order0[channel[chan]-1][1];
	    else
		reflect = (coeff->ocoeff.order1[channel[chan]-1][0] * samp) +
			   coeff->ocoeff.order0[channel[chan]-1][0];


	    /*
	      Convert the reflectance to radiance if specified by the user
	    */
	    if (calstr->rad_opt)
	        {
		rad = reflect / calstr->Kvals[channel[chan] - 1];

	        /*
		  Scale the radiance value and put it in the look up table.
	        */
	        lut[channel[chan]-1][samp] = SCALE(rad,r_scale->minval,
					     r_scale->maxval,r_scale->scale,
					     r_scale->offset);
		}
    	    else
		{
	        /*
		  Scale the reflectance value and put it in the look up table.
	        */
	        lut[channel[chan]-1][samp] = SCALE(reflect,a_scale->minval,
						   a_scale->maxval,a_scale->scale,
						   a_scale->offset);
		}
	    }
	}
    else
	{
	/*
	  Initialize the temperature limit
	*/
	temp_limit[channel[chan]-1-MINOPTICBND] = MAXDAT;

	/*
	  Generate the temperature LUT for the valid 10 bit data range (0-1023)
	*/
	for (samp = 0; samp < MAXDAT; samp++)
	    {
	    /*
	      Calculate the energy value from the coefficients
	    */
	    energy = coeff->tcoeff.order0[channel[chan]-MAXOPTICBND] +
		(coeff->tcoeff.order1[channel[chan]-MAXOPTICBND] * samp) +
		(coeff->tcoeff.order2[channel[chan]-MAXOPTICBND] * samp * samp);

	    /*
	      Apply nonlinear correction to the energy value (NOAA-14)
	      if called for.  For other satellites, we will apply nonlinear
	      correction to temperature values later.
	    */
	    if ((calstr->lin_opt) && (model->satnum == 14))
	      ccnonlin_energy(channel[chan],&energy);

	    /*
	      Energy is then converted in temperature (T) by using
	      the inverse of Planck's radiation function.
	    */
	    if (energy > 0.0)
		{
		temperature = cwncon2[channel[chan]-1-MINOPTICBND] /
		              log(1 + (cwn3con1[channel[chan]-1-MINOPTICBND] /
			      energy));
		temperature = calstr->rats_A[channel[chan]-1-MINOPTICBND] +
		(calstr->rats_B[channel[chan]-1-MINOPTICBND] * temperature);
		}
	    else
		temperature = 0.0;

	    /*
	    When the lower temperature is reached set the remaining
            look-up table values to the lower limit.  When an upper
	    temperature limit is calculated set it to the the upper
 	    limit.  Otherwise put the temperature value into the
	    look-up table.  This clipping is done by scaling also,
	    however non-linear correction is performed prior to
	    scaling.
	    */
	    if (temperature < t_scale->minval)
		{
		temp_limit[channel[chan]-1-MINOPTICBND] = samp;
		for (samp = temp_limit[channel[chan]-1-MINOPTICBND];
							samp < MAXDAT; samp++)
		    lut[channel[chan]-1][samp] = t_scale->minval;
		break;
		}
	    else if (temperature > t_scale->maxval)
		lut[channel[chan]-1][samp] = t_scale->maxval;
	    else
		lut[channel[chan]-1][samp] = temperature;
	    }

	/*
	  Add to the temperature values the non-linear error corrections,
	  if LINOPT is yes and satellite number is 9 thru 12 and channels
	  is 4 or 5
	*/
	if ((calstr->lin_opt) && 
	    ((channel[chan] != 3) && (model->satnum > 8) &&
	     (model->satnum < 13)))
	    ccnonlin_temp(model,calstr,lut,channel[chan],
				temp_limit[channel[chan]-1-MINOPTICBND]);

	/*
        Input sensor values of 0 are thought of as no data, the output should
	be set to MASK_NO_DATA.  Otherwise scale the temperature in the
	look-up table (KELVIN).
	*/
	lut[channel[chan]-1][0] = MASK_NO_DATA;
	for (samp = 1; samp < MAXDAT; samp++)
	    {
	    lut[channel[chan]-1][samp] = SCALE(lut[channel[chan]-1][samp],
					      t_scale->minval,t_scale->maxval,
					      t_scale->scale,t_scale->offset);
	    }
	}
    }

return;
}
