
/*******************************************************************************
FUNCTION:       CCLOAD_DGRD


PURPOSE:        Get the calibration coefficients for the optical channels
		from the appropriate degradation table.


Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
T.Baltzer        10/98  Created this support to replace teilletcals(),
			raocals(), etc.


PROJECT         ADAPS

ALGORITHM REFERENCES
                NOAA Technical Memorandum NESS 107
                  Section 5.1 AVHRR Calibration, Appendix A, Appendix B
                NOAA Polar Orbiter Data User's Guide
                  K. Kidwell, NOAA
                  Section 3.3 Calibration of AVHRR Data
                OSS Working Committee Recommendations of Radiometric
                  Calibration of AVHRR Visible and Near_Infrared Bands
                  B. Holben, NASA/GSFC; P. Teillet, Canada Centre for
                  Remote Sensing; and M. Sharman, Institute for Remote
                  Sensing Applications, Italy

*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "util.h"
#include "adaps.h"
#include "calcoef.h"
#include "adapsutil.h"
#include "adapssup.h"
#include "local_protos.h"

int ccload_dgrd
(
    struct MODEL *model,	/* IN - Model and orbital information	*/
    struct CALSTR *calstr,	/* IN/OUT - Calibration parameter struct*/
    struct COEFFSTR *coeff	/* Calibration coefficients structure	*/
)

{

char hosdgrd[CMLEN];		/* Degradation table host file name	*/
long status;			/* Return status			*/


/*
  Put together calfile (degradation table) name
*/
if (getenv("ADAPSTABLES") != NULL)
    sprintf (hosdgrd,"%s/%s%s",getenv("ADAPSTABLES"),calstr->calmethod,".cal");
else
    {
    c_lasmsg(FALSE, "ccloaddg-find", "Error retrieving value of ADAPSTABLES");
    return(E_FAIL);
    }

/*
  Get the optical coefficients for the calibration
*/
if (strcmp(calstr->calmethod,"teillet") == 0)
    {
    status = ccload_dgrd2(hosdgrd, model->satnum, model->year, model->jday,
			  calstr, coeff);
    }
else if (strcmp(calstr->calmethod,"vermote") == 0)
    {
    status = ccload_dgrd2(hosdgrd, model->satnum, model->year, model->jday,
			  calstr, coeff);
    }
else if (strcmp(calstr->calmethod,"rao") == 0)
    {
    status = ccload_dgrd2(hosdgrd, model->satnum, model->year, model->jday,
			  calstr, coeff);
    }
else if (strcmp(calstr->calmethod,"prelaunch") == 0)
    {
    status = ccload_dgrd1(hosdgrd, model->satnum, model->year, model->jday,
			  calstr, coeff);
    }
else if (strcmp(calstr->calmethod,"kaufman") == 0)
    {
    status = ccload_dgrd1(hosdgrd, model->satnum, model->year, model->jday,
			  calstr, coeff);
    }
else if (strcmp(calstr->calmethod,"noaa") == 0)
    {
    int c;	/* channel counter	*/
    int r;	/* range counter	*/

    /* Read the NOAA based slope/intercept coefficients
    ---------------------------------------------------*/
    status = ccread_noaa(model->satnum, model->year, model->jday, calstr);

    /* Calculate a new C0 based on new slope/intercept
    --------------------------------------------------*/
    cccalc_C0( calstr );

    /* Apply the slope/intercept directly to to optical coefficient structure
    -------------------------------------------------------------------------*/
    for ( c = 0; c < 3; c++ )
	{
        for ( r = 0; r < 2; r++ )
	    {
	    coeff->ocoeff.order0[c][r] = calstr->vsntcp[c][r];
	    coeff->ocoeff.order1[c][r] = calstr->vsgain[c][r];
	    }
	coeff->ocoeff.count[c] = calstr->vscnt[c];
	}  /* END for r */
    }  /* END for c */
else
    {
    c_lasmsg(FALSE, "ccloaddg-invl", "Invalid calibration method specified");
    status = E_FAIL;
    }

return(status);
}

/*******************************************************************************
FUNCTION:	CCLOAD_DGRD1


PURPOSE:	Get the calibration coefficients for the visual channels from
		the degradation table

		NOTE: Reads the degradation tables of the format used by
		      the "kaufman" calibration method.  If "prelaunch"
		      calibration method is selected, the degradation
		      values are computed from prelaunch satellite data
		      already read from tables.

Programmer       Date   Modifications
---------------  -----  --------------------------------------------------------
T.Baltzer	 10/98	Developed for NOAA-15 (NOAA-KLM) modifications.


PROJECT:	ADAPS

ALGORITHM REFERENCES:
                NOAA Technical Memorandum NESS 107
                  Section 5.1 AVHRR Calibration, Appendix A, Appendix B
                NOAA Polar Orbiter Data User's Guide
                  K. Kidwell, NOAA
                  Section 3.3 Calibration of AVHRR Data
		OSS Working Committee Recommendations of Radiometric
		  Calibration of AVHRR Visible and Near_Infrared Bands
		  B. Holben, NASA/GSFC; P. Teillet, Canada Centre for
		  Remote Sensing; and M. Sharman, Institute for Remote
 		  Sensing Applications, Italy

*******************************************************************************/

int ccload_dgrd1
(
    char *hostname,	/* IN  - Calibration table host file name	*/
    long satnum,	/* IN  - Satellite number			*/
    long ayear,		/* IN  - Acquisition year			*/
    long aday,		/* IN  - Acquisition day			*/
    struct CALSTR *calstr, /* IN/OUT - Calibration parameter structure	*/
    struct COEFFSTR *coeff	/* OUT - Calibration coefficients	*/
)
{

struct COEFFSTR tmpcoeff;	/* Temporary coefficients		*/

double kalpha[MAXOPTICBND][2];	/* K * post-launch gain-compensation	*/

long
    albedo,		/* low/high albedo looping variable	*/
    ch,			/* channel looping variable		*/
    cmo,		/* month of pass			*/
    nmo,		/* number of months since launch	*/
    status; 		/* status variable			*/


/*
  Calculate approx. decimal month
*/
cmo = (long)(aday / 30.5) + 1;


/*
  Calculate number of months since launch
*/
nmo = (c_4digityr(ayear) - c_4digityr(calstr->lyr) - 1) * 12 +
      (12 - calstr->lmo + cmo);


/*
  If PRELAUNCH method has been specified, calculate the slope/intercept
  values from the prelaunch satellite info already read in.  Otherwise,
  read the slope/intercept values from the appropriate degradation table.
  NOTE: If the satellite is NOAA-6 then always use prelaunch values
	(NOAA-6 maintained nearly constant calibration over its lifespan -
	Holben, Teillet, & Sharman). 
*/
if ((strcmp(calstr->calmethod,"prelaunch") == 0) || (satnum == 6))
    cccalc_dgrd( calstr, &tmpcoeff );
else
    {
    status = ccread_dgrd1 (hostname, satnum, &tmpcoeff);
    if (status != E_SUCC)
	    return(status);
    }

/*
  Calculate visual coefficients for optical channels.  Most of the time
  there is only channels 1 and 2, but sometimes channel 3a.  Compute
  values even if they are only "place holders" in some cases.
*/
for (ch = 0; ch < MAXOPTICBND; ch++)
    {
    for (albedo = 0; albedo < 2; albedo++)
	{
	kalpha[ch][albedo] = calstr->Kvals[ch] *
			     (tmpcoeff.ocoeff.order1[ch][albedo]
				 * nmo + tmpcoeff.ocoeff.order0[ch][albedo]);
	coeff->ocoeff.order1[ch][albedo] = kalpha[ch][albedo];
	coeff->ocoeff.order0[ch][albedo] = -kalpha[ch][albedo] *
					calstr->C0[ch][albedo];
	}
    coeff->ocoeff.count[ch] = tmpcoeff.ocoeff.count[ch];
    }

return(E_SUCC);
}

/*******************************************************************************
FUNCTION:       CCLOAD_DGRD2


PURPOSE:        Get the calibration coefficients for the visual channels
		from the degradation table using a piecewise linear 
		method of calibration, based on days since launch.

		NOTE: Reads the degradation tables of the format used by
		      the "rao", "teillet", "vermote" calibration method.


Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
T.Baltzer        10/98  Created this support to replace teilletcals(),
			raocals(), etc.


PROJECT         ADAPS

ALGORITHM REFERENCES
                NOAA Technical Memorandum NESS 107
                  Section 5.1 AVHRR Calibration, Appendix A, Appendix B
                NOAA Polar Orbiter Data User's Guide
                  K. Kidwell, NOAA
                  Section 3.3 Calibration of AVHRR Data
                OSS Working Committee Recommendations of Radiometric
                  Calibration of AVHRR Visible and Near_Infrared Bands
                  B. Holben, NASA/GSFC; P. Teillet, Canada Centre for
                  Remote Sensing; and M. Sharman, Institute for Remote
                  Sensing Applications, Italy

Note: 	a and d taken from the dgrd table were not used because the desired
	formula is linear.  However, these values were left in the degradation
	table in case they are wanted later for a non-linear calculation, such
	as in Appendix A of the OSS paper.
*******************************************************************************/

int ccload_dgrd2
(
    char *hostname,	/* IN  - Calibration table host file name             */
    long satnum,	/* IN  - Satellite number                             */
    long ayear,		/* IN  - Acquisition year                             */
    long aday,		/* IN  - Julian acquisition day                       */
    struct CALSTR *calstr, /* IN/OUT - Calibration parameter structure	      */
    struct COEFFSTR *coeff  /* OUT - Calibration coefficients structure      */
)
{
double
    a[2],		/* ((counts/rad)/day)/day                             */
    b[2],		/*  (counts/rad)/day                                  */
    c[2],		/*   counts/radiance                                  */
    d[2],		/*  (counts/day)/day                                  */
    e[2],		/*   counts/day                                       */
    f[2],		/*   counts                                           */
    c0[2],		/* zero radiance offset (counts)                      */
    alpha[2];		/* post-launch gain-compensation                      */

long
    acqyr,		/* full acquisition year                              */
    adate,		/* scene acquisition date                             */
    days = 0,		/* total days since launch                            */
    i,			/* looping variable                                   */
    launch,		/* launch date as yyddd                               */
    launchyr,		/* launch year                                        */
    status; 		/* status variable                                    */

/*
  Acquisition date as yyddd
*/
adate = ayear * 1000 + aday;


/*
  Read the calibration degradation table
*/
status = ccread_dgrd2 (hostname,adate,satnum,&launch,a,b,c,d,e,f);
if (status != E_SUCC)
    return(E_FAIL);


/*
  Calculate number of days since launch
*/
acqyr = c_4digityr(ayear);
launchyr = c_4digityr(launch / 1000);
if (c_isleap(&launchyr))			/* remainder of first year */
    days += 366 - launch % 1000;
else
    days += 365 - launch % 1000;
days += aday;					/* acquisition year */
for (i = launchyr + 1; i < acqyr; i++)		/* in-between years */
    {
    if (c_isleap(&i))
        days += 366;
    else
        days += 365;
    }


/*
  Calculate visual coefficients.  Zero for high albedo since there
  are no actual split coeff available.  Note that only two optical
  bands are being supported at this time also (NOT optical channel 3a).
*/
for (i = 0; i < 2; i++)
    {
    alpha[i] = 1.0 / (b[i] * days + c[i]);
    c0[i] = e[i] * days + f[i];
    coeff->ocoeff.order1[i][0] = calstr->Kvals[i] * alpha[i];
    coeff->ocoeff.order0[i][0] = calstr->Kvals[i] * -c0[i] * alpha[i];

    coeff->ocoeff.order1[i][1] = 0.0;
    coeff->ocoeff.order0[i][1] = 0.0;
    coeff->ocoeff.count[i]     = 0;
    }

return(E_SUCC);
}
