/*******************************************************************************
FUNCTION:	CCNONLIN_ENERGY

PURPOSE:	Add the non-linear corrections for NOAA-14 data.  This
		computation is not done in ccnonlin_temp() since non-linear
		corrections there are being applied to temperatures.  For
		NOAA-14 data, the non-linear correction is to be applied to
		energy levels not temperatures, thus meriting its own
		routine.

		For NOAA-14, the nonlinearity is corrected using correction
		coefficients A, B, C applied to linear radiance RLIN as such:
		  radiance = A * RLIN + B * RLIN * RLIN + C

ALGORITHM 

The gain and intercept coefficients are derived with the following equations:                                       
G = (RT - RSP)/(CT - CSP )              (1)
I = RSP- G * CSP

These coefficients provide a linear radiance:

RLIN = G * C + I.                                  (2)

This linear calibration procedure is described in more detail in NOAA
Technical Memorandum NESS 107. The following equation provides a corrected
radiance for AVHRR channels 3, 4 and 5 which is a function only of the
linear radiance derived from eq. 2:

            RAD= A * RLIN + B * RLIN * RLIN + C     (3)

With the energy tables or by using the inverse of the Planck
function with the appropriate central wave numbers, this radiance
can be converted to temperature. No additional temperature
correction should be applied. The coefficients to apply this
equation for channels 3, 4 & 5 of FM. 204 are as follows:
                                                                            
                     Channel 3      Channel 4      Channel 5

     Coefficient A :  1.00359        0.92378        0.96194
     Coefficient B :  0              0.0003822      0.0001742
     Coefficient C :  -0.0031        3.72           2.00              

 Space Radiance RSP:  0.0069         -4.05          -2.29          

In summary, users of level 1B data need only apply equations 2 and 3  using
the slope and intercept values imbedded in the level 1B data.  Users of
direct readout data must also apply equation 1.
          

ALGORITHM REFERENCES
  NOAA Technical Memorandum NESS 107.
  Appendix B:  NOAA-XX coefficients

  Katherine B. Kidwell, NOAA Polar Orbiter Data Users Guide
  National Oceanic and Atmospheric Administration, Washington, D.C.
  July, 1991.
*******************************************************************************/
#include <stdio.h>		/* Standard I/O include			*/
#include "las.h"		/* LAS include file			*/

void ccnonlin_energy
(
    long channel,		/* Channel number			*/
    float *energy		/* Energy in, corrected energy out	*/
)

{

float Acoef;			/* A coefficient	*/
float Bcoef;			/* B coefficient	*/
float Ccoef;			/* C coefficient	*/
float nonlin_engy;		/* Energy value with nonlin correction	*/


/* Determine correct coefficients
----------------------------------*/

if (channel == 3)		/* Channel 3	 */
    {
    Acoef = 1.00359;
    Bcoef = 0.0;
    Ccoef = -0.0031;
    }
else if (channel == 4)		/* Channel 4	 */
    {
    Acoef = 0.92378;
    Bcoef = 0.0003822;
    Ccoef = 3.72;
    }
else				/* Channel 5	 */
    {
    Acoef = 0.96194;
    Bcoef = 0.0001742;
    Ccoef = 2.00;
    }


/* Apply the nonlinear correction to the energy value
------------------------------------------------------*/
nonlin_engy = (Acoef * *energy) + (Bcoef * *energy * *energy) + Ccoef;


/* Return the corrected energy value
------------------------------------*/
*energy = nonlin_engy;


return;
}

