/*******************************************************************************
FUNCTION:	CCNONLIN_TEMP

PURPOSE:	Add the non-linear corrections to the calculated temperatures.

		The non-linearity corrections are the difference between
		the actual scene target temperature and the temperature
		derived from a two-point linear calibration using a radiance
		of space of zero.  Knowing the scene temperature (from the
		linear calibration) and the internal target temperature, the
		the corrections are found by interpolating between values
		from the non-linearity correction table.  See Appendix B
                of NOAA Technical Memorandum NESS 107.

		NOTE: NOAA-14 nonlinear correction is not applied here since
		it corrects energy values instead of temperature values.  See
		ccnonlin_energy() for more details.

ALGORITHM 
  "serror" variable is taken from the Non-linearity Correction Table for
  internal target temperature of 15 deg Celcius.  The difference between
  the non-linearity terms divided by number of degrees between the terms
  are placed in the "ierr" variable.

  Example:
                  Channel 4 Non-linearity Table (NOAA-11)

         Measured             Internal Target Temperature (deg C)
  Scene Temperature (deg K)        10       15       20       25
            205                         -1.76
            215                         -2.02
             .                            . 
             .                            . 
             .                            . 
             
          term2 - term1               -2.02 - -1.76     -0.26
  delta = -------------  =  ierr[0] = ------------- = --------- = -0.026
          temp2 - temp1                 215 - 205         10

  
  Setup the correct non-linear correction values for the specified
  satellite, channel, and radiance of space.
  Add the correct non-linear correction to the specified temperature

  For NOAA-14, A, B, and C values are places in the first three elements of 
  ierr[].

ALGORITHM REFERENCES
  NOAA Technical Memorandum NESS 107.
  Appendix B:  NOAA-XX coefficients

  Katherine B. Kidwell, NOAA Polar Orbiter Data Users Guide
  National Oceanic and Atmospheric Administration, Washington, D.C.
  July, 1991.
*******************************************************************************/
#include <stdio.h>		/* Standard I/O include			*/
#include "las.h"		/* LAS include file			*/
#include "adaps.h"		/* ADAPS include file			*/
#include "calcoef.h"		/* Calibration parameter defines	*/

void ccnonlin_temp
(
    struct MODEL *model,	/* AVHRR model structure		*/
    struct CALSTR *calstr,	/* Calibration parameter structure	*/
    float lut[][MAXDAT],	/* Calculated temperature LUT		*/
    long channel,		/* Channel number			*/
    long temp_limit		/* Maximum valid temperature limit	*/
)

{
float	adderr,			/* Amount of temp to add for correction	*/
	linerr,			/* linear error correction calculated	*/
	ierr[13],		/* Difference between corection values	*/
	serror[13];		/* Nonlinearity Correction value	*/

long	i;			/* Looping variable			*/

/*
  The corrections for NOAA 9 are for the 15 degree blackbody temperature
*/
if (model->satnum == 9)
    {
    if (calstr->ros_opt)
	{
	if (channel == 4)   /* Channel 4, zero radiance of space, and NOAA 9 */
	    {
	    serror[0]= -1.5;
	    ierr[0] = 0.0;
	    serror[1] = -1.5;
	    ierr[1] = .02;
	    serror[2] = -1.3;
	    ierr[2] = -.01;
	    serror[3] = -1.4;
	    ierr[3] = .01;
	    serror[4] = -1.3;
	    ierr[4] = 0.0;
	    serror[5] = -1.3;
	    ierr[5] = .02;
	    serror[6] = -1.1;
	    ierr[6] = .04;
	    serror[7] = -0.7;
	    ierr[7] = .055;
	    serror[8] = -0.15;
	    ierr[8] = .055;
	    serror[9] = 0.4;
	    ierr[9] = 0.06;
	    serror[10] = 1.0;
	    ierr[10] = 0.08;
	    serror[11] = 1.4;
	    ierr[11] = 0.1;
	    serror[12] = 1.9;
	    ierr[12] = 0.08;
	    }
	else		/* Channel 5, zero radiance of space and NOAA 9 */
	    {
	    serror[0] = -1.6;
	    ierr[0] = .02;
	    serror[1] = -1.4;
	    ierr[1] = .04;
	    serror[2] = -1.0;
	    ierr[2] = -0.02;
	    serror[3] = -1.2;
	    ierr[3] = .04;
	    serror[4] = -0.8;
	    ierr[4] = 0.0;
	    serror[5] = -0.8;
	    ierr[5] = .02;
	    serror[6] = -0.6;
	    ierr[6] = .03;
	    serror[7] = -0.3;
	    ierr[7] = .025;
	    serror[8] = -0.05;
	    ierr[8] = 0.025;
	    serror[9] = 0.2;
	    ierr[9] = 0.02;
	    serror[10] = 0.4;
	    ierr[10] = 0.06;
	    serror[11] = 0.7;
	    ierr[11] = 0.04;
	    serror[12] = 0.9;
	    ierr[12] = 0.052;
	    }
	}
    else 	/* Channel 4, non-zero radiance of space, and NOAA 9 */
	{
	if (channel == 4)
	    {
	    serror[0] = 5.6;
	    ierr[0] = -0.17;
	    serror[1] = 3.9;
	    ierr[1] = -0.17;
	    serror[2] = 2.2;
	    ierr[2] = -0.09;
	    serror[3] = 1.3;
	    ierr[3] = -0.09;
	    serror[4] = 0.4;
	    ierr[4] = -0.03;
	    serror[5] = 0.1;
	    ierr[5] = -0.02;
	    serror[6] = -0.1;
	    ierr[6] = -0.02;
	    serror[7] = -0.3;
	    ierr[7] = .01;
	    serror[8] = -0.2;
	    ierr[8] = .03;
	    serror[9] = 0.1;
	    ierr[9] = .04;
	    serror[10] = 0.5;
	    ierr[10] = .035;
	    serror[11] = 0.85;
	    ierr[11] = 0.035;
	    serror[12] = 1.2;
	    ierr[12] = 0.0;
	    }
	else	/* Channel 5, non-zero radiance of space, and NOAA 9 */
	    {
	    serror[0] = 2.1;
	    ierr[0] = -0.065;
	    serror[1] = 1.45;
	    ierr[1] = -0.065;
	    serror[2] = 0.8;
	    ierr[2] = -0.03;
	    serror[3] = 0.5;
	    ierr[3] = -0.03;
	    serror[4] = 0.2;
	    ierr[4] = -0.01;
	    serror[5] = 0.1;
	    ierr[5] = -0.01;
	    serror[6] = 0.0;
	    ierr[6] = -0.01;
	    serror[7] = -0.1;
	    ierr[7] = 0.01;
	    serror[8] = 0.0;
	    ierr[8] = 0.01;
	    serror[9] = 0.1;
	    ierr[9] = 0.02;
	    serror[10] = 0.3;
	    ierr[10] = .03;
	    serror[11] = 0.45;
	    ierr[11] = 0.03;
	    serror[12] = 0.6;
	    ierr[12] = 0.0;
	    }
	}
    }
  		/* The error corrections below are for NOAA10 and are for    */
		/* the 15 degree celcius blackbody temperature.              */
else if (model->satnum == 10)	/* Channel 4  NOAA10				     */
    {
    serror[0]= -2.1;
    ierr[0] = 0.02;
    serror[1] = -1.9;
    ierr[1] = 0.02;
    serror[2] = -1.7;
    ierr[2] = 0.02;
    serror[3] = -1.5;
    ierr[3] = .01;
    serror[4] = -1.4;
    ierr[4] = .03;
    serror[5] = -1.1;
    ierr[5] = .02;
    serror[6] = -0.9;
    ierr[6] = .04;
    serror[7] = -0.5;
    ierr[7] = .02;
    serror[8] = -0.3;
    ierr[8] = 0.05;
    serror[9] = 0.2;
    ierr[9] = 0.05;
    serror[10] = 0.7;
    ierr[10] = 0.06;
    serror[11] = 1.0;
    ierr[11] = 0.06;
    serror[12] = 1.3;
    ierr[12] = 0.06;
    }
else if (model->satnum == 11) 	/* Channel 4 and 5  NOAA11		     */
    {
    if (channel == 4)
       {
       serror[0]= -1.76;
       ierr[0] = -0.026;
       serror[1] = -2.02;
       ierr[1] = -0.012;
       serror[2] = -2.14;
       ierr[2] = 0.04;
       serror[3] = -2.10;
       ierr[3] = 0.014;
       serror[4] = -1.96;
       ierr[4] = 0.024;
       serror[5] = -1.72;
       ierr[5] = 0.035;
       serror[6] = -1.37;
       ierr[6] = 0.058;
       serror[7] = -0.79;
       ierr[7] = 0.058;
       serror[8] = -0.21;
       ierr[8] = 0.089;
       serror[9] = 0.68;
       ierr[9] = 0.105;
       serror[10] = 1.73;
       ierr[10] = 0.12;
       serror[11] = 2.33;
       ierr[11] = 0.13;
       serror[12] = 2.98;
       ierr[12] = 0.146;
       }
    else if (channel == 5)
       {
       serror[0]= -1.27;
       ierr[0] = 0.003;
       serror[1] = -1.24;
       ierr[1] = 0.018;
       serror[2] = -1.06;
       ierr[2] = 0.004;
       serror[3] = -1.02;
       ierr[3] = 0.012;
       serror[4] = -0.90;
       ierr[4] = 0.013;
       serror[5] = -0.77;
       ierr[5] = 0.026;
       serror[6] = -0.51;
       ierr[6] = 0.017;
       serror[7] = -0.34;
       ierr[7] = 0.027;
       serror[8] = -0.07;
       ierr[8] = 0.035;
       serror[9] = 0.28;
       ierr[9] = 0.036;
       serror[10] = 0.64;
       ierr[10] = 0.04;
       serror[11] = 0.84;
       ierr[11] = 0.038;
       serror[12] = 1.03;
       ierr[12] = 0.046;
       }
    }
else if (model->satnum == 12) 	/* Channel 4 and 5  NOAA12		     */
    {
    if (channel == 4)
       {
       serror[0]= -1.80;
       ierr[0] = 0.015;
       serror[1] = -1.65;
       ierr[1] = 0.0;
       serror[2] = -1.65;
       ierr[2] = 0.006;
       serror[3] = -1.59;
       ierr[3] = 0.019;
       serror[4] = -1.40;
       ierr[4] = 0.024;
       serror[5] = -1.20;
       ierr[5] = 0.02;
       serror[6] = -0.97;
       ierr[6] = .023;
       serror[7] = -0.84;
       ierr[7] = .013;
       serror[8] = -0.23;
       ierr[8] = .076;
       serror[9] = 0.53;
       ierr[9] = 0.089;
       serror[10] = 1.42;
       ierr[10] = 0.104;
       serror[11] = 1.94;
       ierr[11] = 0.09;
       serror[12] = 2.39;
       ierr[12] = 0.098;
       }
    else if (channel == 5)
       {
       serror[0] = -1.16;
       ierr[0] = .003;
       serror[1] = -1.19;
       ierr[1] = .009;
       serror[2] = -1.10;
       ierr[2] = 0.016;
       serror[3] = -0.94;
       ierr[3] = .018;
       serror[4] = -0.76;
       ierr[4] = 0.023;
       serror[5] = -0.53;
       ierr[5] = .012;
       serror[6] = -0.41;
       ierr[6] = .01;
       serror[7] = -0.31;
       ierr[7] = .023;
       serror[8] = -0.08;
       ierr[8] = 0.026;
       serror[9] = 0.18;
       ierr[9] = 0.043;
       serror[10] = 0.61;
       ierr[10] = 0.024;
       serror[11] = 0.73;
       ierr[11] = 0.014;
       serror[12] = 0.80;
       ierr[12] = 0.0;
       }
    }

/*
  Add the correct non-linear correction to the specified temperature
*/
  for (i = 0; i < temp_limit; i++)
    {
    if ((lut[channel-1][i] >= 205) && (lut[channel-1][i] < 215))
	{
	adderr = (lut[channel-1][i] - 205) * ierr[0];
	linerr = adderr + serror[0];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 215) && (lut[channel-1][i] < 225))
	{
	adderr = (lut[channel-1][i] - 215) * ierr[1];
	linerr = adderr + serror[1];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 225) && (lut[channel-1][i] < 235))
	{
	adderr = (lut[channel-1][i] - 225) * ierr[2];
	linerr = adderr + serror[2];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 235) && (lut[channel-1][i] < 245))
	{
	adderr = (lut[channel-1][i] - 235) * ierr[3];
	linerr = adderr + serror[3];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 245) && (lut[channel-1][i] < 255))
	{
	adderr = (lut[channel-1][i] - 245) * ierr[4];
	linerr = adderr + serror[4];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 255) && (lut[channel-1][i] < 265))
	{
	adderr = (lut[channel-1][i] - 255) * ierr[5];
	linerr = adderr + serror[5];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 265) && (lut[channel-1][i] < 275))
	{
	adderr = (lut[channel-1][i] - 265) * ierr[6];
	linerr = adderr + serror[6];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 275) && (lut[channel-1][i] < 285))
	{
	adderr = (lut[channel-1][i] - 275) * ierr[7];
	linerr = adderr + serror[7];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 285) && (lut[channel-1][i] < 295))
	{
	adderr = (lut[channel-1][i] - 285) * ierr[8];
	linerr = adderr + serror[8];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 295) && (lut[channel-1][i] < 305))
	{
	adderr = (lut[channel-1][i] - 295) * ierr[9];
	linerr = adderr + serror[9];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 305) && (lut[channel-1][i] < 310))
	{
	adderr = (lut[channel-1][i] - 305) * ierr[10];
	linerr = adderr + serror[10];
	lut[channel-1][i] += linerr;
	}
    else if ((lut[channel-1][i] >= 310) && (lut[channel-1][i] < 315))
	{
	adderr = (lut[channel-1][i] - 310) * ierr[11];
	linerr = adderr + serror[11];
	lut[channel-1][i] += linerr;
	}
    else if (lut[channel-1][i] >= 315)
	{
	adderr = (lut[channel-1][i] - 315) * ierr[12];
	linerr = adderr + serror[12];
	lut[channel-1][i] += linerr;
	}
    }

return;
}
