/******************************************************************************
FUNCTION:	CCPRIM

PURPOSE:	Initializes values used to generate calibration coefficients.
		Coefficients are generated for the first CALPER*PRTPER lines.
		Returns E_SUCC for successful completion of function; E_FAIL 
		for errors encountered reading files.

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
  1.0	04/87	P. Mumford	Initial development
  1.1	01/90	C. Wivell	Optical sensor degradation and
				illumination correction
  5.0	02/90	D. Hollaren	Conversion to LAS-5.0
  5.1	10/91	T. Rockvam	Read calibration table and fixed sl 
				and el to use for priming coeff
  5.2	01/92	T. Rockvam	Added master line in case input image
				was windowed from another image
  5.3	01/98	D.Lloyd		Combined the nearly equivalent MKLEVEL1B and
				MAKENCAL functions to create a common support. 
				The original functions only differed in the
				input file data type.
  5.4	01/99	T.Baltzer	Revamp for NOAA-15 (NOAA-KLM) calibration.
  5.5   03/00   D.Lloyd		Changed the initialization of prtndx in line
  				with changes made to ccupdt.c
  5.6   04/01   D.Lloyd		Modified the reaction to a change in channel 3
  				sensor during priming.  It now breaks from the
				loop instead of a fatal error.  CR 4802

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
		NOAA Technical Memorandum NESS 107
			Section 5.1 AVHRR Calibration, Appendix A, Appendix B
		NOAA Polar Orbiter Data User's Guide
			K. Kidwell, NOAA
			Section 3.3 Calibration of AVHRR Data
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "vll.h"
#include "archio.h"
#include "model.h"
#include "avhrr.h"
#include "calcoef.h"
#include "adapssup.h"
#include "local_protos.h"

int ccprim
(
    struct CALSTR *calstr,	/* Calibration parameter structure	*/
    long slprim,		/* Line at which to start priming	*/
    long mline,			/* Image master line number		*/
    struct COEFFSTR *coeff,	/* All coefficients (optical, thermal)	*/
    struct COEFFSTR *lastcoeff,/* Last set of calibration coeff saved	*/
    struct OCOEFFSTR *origocoeff,/* Original optical coefficient struct	*/
    long *prtcnt,	/* PRT counts					*/
    long prtndx[],	/* Offsets to find the four sample prt values	*/
    long *prtpop,	/* PRT population (CALPER*PRTPER)		*/
    long *filedes,	/* file descriptor structure			*/
    double bnmrp[][60],	/* Numerator of beta function phihat*deltanu	*/
    double enmr[][60],	/* Numerator of exp function			*/
    struct MODEL *model,/* Model and orbital information		*/
    long intsum[],	/* Sum of internal counts			*/
    long spcsum[]	/* Sum of space counts				*/
)

{
char	char_buf[1];		/* buffer for label services calls	*/
char	dtype[4];		/* data file data type			*/
char	linekey[KEYL];		/* line key for minor frame data file	*/

long	cntsum[4];		/* sum of prt counts			*/
long	elprim;			/* ending line used in priming		*/
long	i,j,k;			/* loop counters			*/
long	line;			/* line index				*/
long	minor_nbytes = MNR_WORDS * sizeof(short); /* # of minor frame bytes */
long	nxtprt;			/* index in PRTCNT for the next prt	*/
long	one = 1;		/* value of one				*/
long	prtsmp[4];		/* number of samples of prt counts	*/
long	sampls;			/* # of samples of target & space counts */
long	status;			/* function return value		*/

short	*min_frm;		/* buffer to read minor frame data into	*/
short	(*samples)[NCHANS];	/* pointers to image data for c_arcread	*/

struct CALSTR tmpcalstr;	/* Temp calibration parameter structure	*/

/*
  Set the temporary calibration structure to the official calibration
  structure.  This temporary structure is necessary for priming so as
  to maintain the overall configuration status for overall processing.
*/
tmpcalstr = *calstr;

/*
  Set up: for a LAS image we need an buffer to store the minor frame,
  for an archive image we only need a pointer.
----------------------------------------------------------------------*/
if ( calstr->file_type == LASIMAGE )
    min_frm = (short *)malloc( sizeof( short ) * MNR_WORDS );


/*
  Initialize thermal coefficients
*/
for ( i = 0; i < MAXTHERMBND; i++ )  
    {
    coeff->tcoeff.order0[i] = lastcoeff->tcoeff.order0[i] = 0.0;
    coeff->tcoeff.order1[i] = lastcoeff->tcoeff.order1[i] = 0.0;
    coeff->tcoeff.order2[i] = lastcoeff->tcoeff.order2[i] = 0.0;
    }

/*
  Calculate values based on the calibration parameters
*/
for ( i = 0; i < 3; i++ )
    {
    for ( j = 0; j < calstr->n[i]; j++ )
        {
        bnmrp[i][j] = calstr->phihat[i][j] * calstr->dltanu[i] * C1 *
                      pow(( calstr->nu1[i] + j * calstr->dltanu[i] ),3.0 );
        enmr[i][j] = ( calstr->nu1[i] + j * calstr->dltanu[i] ) * C2;
        }
    }

/*
  Initialize counter variables
*/
*prtpop = calstr->calper * calstr->prtper;
sampls = 0;
nxtprt = 0;

/*
  Initialize summation variables
*/
for ( i = 0; i < 4; i++ )
    {
    cntsum[i] = 0;
    prtsmp[i] = 0;
    prtndx[i] = i + 1;
    }
for ( i = 0; i < MAXTHERMBND; i++ )
    {
    intsum[i] = 0;
    spcsum[i] = 0;
    }
for ( i = 0; i <= *prtpop; i++ )
    prtcnt[i] = 0;

/*
  Determine the ending line to be used for priming.
  We go one extra line (prtper * calper + 1) because the coefficients are
  only calculated (in ccupdt) at the start of a new calibration period.
*/
elprim = calstr->prtper * calstr->calper + slprim;
if ((elprim + mline) > model->nlines)
    {
    c_lasmsg(FALSE,"ccprim-el","Insufficient lines needed to prime coeff");
    return E_FAIL;
    }


/*
  For an archive image, make sure we read from beginning
---------------------------------------------------------*/
if ( calstr->file_type == ARCIMAGE )
    {
    status = c_arcseek( filedes, slprim );
    if ( status == E_FAIL )
	{
	c_lasmsg(FALSE,"ccprim-seek","Error seeking in archive image file");
	return E_FAIL;
	}
    }

/*
  Initialize the calibration coefficients
*/
for ( line = slprim; line <= elprim; line++ )
    {
    if ( calstr->file_type == LASIMAGE )
	{
	sprintf( linekey, "%d", line + mline );
	strcpy ( dtype, "I2" );
	status = c_lsread ( filedes, linekey, &one, &minor_nbytes, char_buf,
                	(unsigned char *)min_frm, dtype );
	}
    else
	status = c_arcread( filedes, &min_frm, (short **)&samples );
    if ( status != E_SUCC )
        {
        c_lasmsg(FALSE,"ccprim-read","Error reading minor frame file");
	return E_FAIL;
        }

    /*
      Make sure the sensor configuration has not changed.  If it does
      change during the calibration coeff priming, break out of this
      priming loop.
    */
    status = ccsconfig(min_frm,model,&tmpcalstr);
    if (status != E_SUCC)
	return(status);
    if (tmpcalstr.reconfig == TRUE)
        {
        c_lasmsg(FALSE,"ccprim-recn","Sensor reconfigured during %s",
		 "calibration priming");
	break;
        }

    status = ccupdt( calstr, line, mline, min_frm, coeff, lastcoeff, origocoeff,
		     prtcnt, cntsum, prtsmp, prtndx, &nxtprt, *prtpop,
		     &sampls, intsum, spcsum, slprim,
		     bnmrp, enmr, model, FALSE );
    }

/*
  Rewind minor frame label services file to read it again
*/
if ( calstr->file_type == LASIMAGE )
    {
    /* Rewind the label services file to the beginning.  There is no way
    of positioning a label services file arbitrarily, however, because of
    the way the lsread works, the next read will be from the proper file
    location.  Added note for CR 4802.
    --------------------------------------------------------------------*/
    status = c_lsrew( filedes );
    if ( status != E_SUCC )
	{
	c_lasmsg(FALSE,"ccprim-rewind","Error rewinding minor frame file");
	return E_FAIL;
	}
    }
else
    {
    /* Reposition the file to the line it was on on entry to this function.
       Read the line so that everything is as it was prior to calling this
       function (including the minor frame buffer).
    */
    status = c_arcseek( filedes, slprim);
    if ( status == E_FAIL )
	{
	c_lasmsg(FALSE,"ccprim-seek","Error seeking in archive image file");
	return E_FAIL;
	}
    status = c_arcread( filedes, &min_frm, (short **)&samples );
    if ( status != E_SUCC )
        {
        c_lasmsg(FALSE,"ccprim-read","Error reading minor frame file");
	return E_FAIL;
        }
    }

/*
  Make sure 1st set of lastcoeff are not 0
  If coeff are zero, set up so when coeff are applied,
  the initial count value is not changed:
   r = gC + i;  set g = 1, i = 0 => r = C
  we need to change gains only, leave intercepts as 0
  NOTE:  Thermal channels only use one set of coefficients.
*/
for ( i = 0; i < calstr->nopticbnds; i ++)
    for ( k = 0; k < 2; k ++)
	{
	if ((lastcoeff->ocoeff.order1[i][k] == 0.0) &&
	    (lastcoeff->ocoeff.order0[i][k] == 0.0))
	     lastcoeff->ocoeff.order1[i][k] = 1.0;
	}
for (      ; i < 5; i ++)
	{
	if ((lastcoeff->tcoeff.order1[i-MINOPTICBND] == 0.0) &&
	    (lastcoeff->tcoeff.order0[i-MINOPTICBND] == 0.0))
	     lastcoeff->tcoeff.order1[i-MINOPTICBND] = 1.0;
	}


return E_SUCC;
}
