/******************************************************************************
FUNCTION:	CCREAD_SAT

PURPOSE:	Read all of the necessary calibration parameters from the
		table (file) relative to the appropriate satellite.  These
		files can be found in ADAPSTABLES and are named "noaaXX.cal".

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
  1.0	09/98	T. Baltzer	Development
  1.1   01/99	D.Lloyd		Added reading of non-linear correction factors

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "calcoef.h"
#include "adapssup.h"
#include "local_protos.h"

#define CALLEN 80	/* Record length in calibration table		*/


int ccread_sat
(
    long satnum,	/* Satellite number				*/
    struct CALSTR *calstr  /* Calibration parameter structure		*/
)


{
char	buffer[MAXBUF];		/* buffer to read data into		*/
char	calfil[CMLEN];		/* calibration file			*/
char	direct[CMLEN];		/* directory of the specified file	*/
char	sat[3];			/* satellite id number			*/

long	fd;			/* file descriptor			*/
long	i,j,k;			/* loop counters			*/
long	nchan;			/* Number of channels to read in 	*/

/*
  Put together the calibration table name
*/
strcpy( direct, getenv( "ADAPSTABLES" ));
sprintf( sat, "%02d", satnum );
sprintf ( calfil, "%s/%s%s%s", direct, "noaa", sat, ".cal" );

/*
  Open the file containing the Nonlinearity Correction Information
*/
fd = open( calfil, 0 );

/* Skip down to the launch date */
if (ccfindstr(fd, buffer, "LAUNCH DATE:") != E_SUCC)
    return(E_FAIL);

/* Skip header info */
if (ccreadline(fd,4,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the launch date info */
sscanf( buffer, "%d %d", &calstr->lmo, &calstr->lyr);

/* Skip down to the slope/intercept visible coefficients */
if (ccfindstr(fd, buffer, "VISIBLE CHANNEL CALIBRATION:") != E_SUCC)
    return(E_FAIL);

/* Determine the number of channels for which values are given */
sscanf( buffer, "%*s %*s %*s %d", &nchan);

/* Skip header info */
if (ccreadline(fd,3,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the slope/intercept visible coefficients */
for (i = 0; i < nchan; i++)
    {
    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);
    sscanf( buffer, "%*s %lf %lf %lf %lf %d", &calstr->vsgain[i][0],
	    &calstr->vsgain[i][1], &calstr->vsntcp[i][0],
	    &calstr->vsntcp[i][1], &calstr->vscnt[i] );
    }

/* Skip down to the radiance to reflectance values */
if (ccfindstr(fd, buffer, "RADIANCE TO REFLECTANCE:") != E_SUCC)
    return(E_FAIL);

/* Determine the number of channels for which values are given */
sscanf( buffer, "%*s %*s %*s %d", &nchan);

/* Skip header info */
if (ccreadline(fd,6,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the values */
for (i = 0; i < nchan; i++)
    {
    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);
    sscanf( buffer, "%*s %lf %lf %lf", &calstr->rd2rf_w[i],
	    &calstr->rd2rf_l[i], &calstr->rd2rf_F[i]);
    }

/* Skip down to the PRT coefficients */
if (ccfindstr(fd, buffer, "PRT CALIBRATION FACTORS:") != E_SUCC)
    return(E_FAIL);

/* Skip header info */
if (ccreadline(fd,3,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the PRT coefficients */
for ( i = 0; i < 4; i++ )
    {
    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);
    sscanf( buffer, "%d %lf %lf %lf %lf %lf", &j,
        &calstr->aij[i][0], &calstr->aij[i][1], &calstr->aij[i][2],
	&calstr->aij[i][3], &calstr->aij[i][4] );
    }
    
/* Skip down to the PRT weighting factors */
if (ccfindstr(fd, buffer, "PRT WEIGHTING FACTORS:") != E_SUCC)
    return(E_FAIL);

/* Skip header info */
if (ccreadline(fd,4,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the PRT weighting factors */
sscanf( buffer, "%lf %lf %lf %lf", &calstr->bi[0], &calstr->bi[1],
	&calstr->bi[2], &calstr->bi[3] );

/* Skip down to the Radiance of Space values (chans 3, 4, & 5) */
if (ccfindstr(fd, buffer, "SPACE RADIANCE CORRECTION:") != E_SUCC)
    return(E_FAIL);

/* Skip header info */
if (ccreadline(fd,4,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the Radiance of Space values (chans 3, 4, & 5) */
sscanf( buffer, "%lf %lf %lf", &calstr->nsp[0], &calstr->nsp[1],
	&calstr->nsp[2] );

/* Skip down to the Radiance to Temperature coefficients */
if (ccfindstr(fd, buffer, "RADIANCE TO TEMPERATURE COEFFICIENTS:") != E_SUCC)
    return(E_FAIL);

/* Skip header info */
if (ccreadline(fd,3,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the Radiance to temperature coefficients */
for (i = 0; i < MAXTHERMBND; i++)
    {
    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);
    sscanf( buffer, "%*s %lf %lf %lf %lf %lf %lf",
	    &calstr->rats_vc[i][0],&calstr->rats_vc[i][1],
	    &calstr->rats_vc[i][2],&calstr->rats_vc[i][3],
	    &calstr->rats_A[i], &calstr->rats_B[i]);
    }

/* Skip down to the Nonlinearity Correction Table */
if (ccfindstr(fd, buffer, "AVHRR NORMALIZED RESPONSE FUNCTIONS:") != E_SUCC)
    return(E_FAIL);

/* Read the Nonlinearity Correction Table info */
for ( i = 0; i < 3; i++ )
    {
    /* Skip header info */
    if (ccreadline(fd,2,buffer) != E_SUCC)
	return(E_FAIL);

    sscanf( buffer, "%*s %*s %*s %lf %*s %lf %*s %d", &calstr->nu1[i],
	    &calstr->dltanu[i], &calstr->n[i] );

    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);

    for ( j = 0, k = 0; j < 12; j++, k+=5 ) 
	{
	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
        sscanf( buffer, "%lf %lf %lf %lf %lf", &calstr->phihat[i][k],
		&calstr->phihat[i][k+1], &calstr->phihat[i][k+2],
		&calstr->phihat[i][k+3], &calstr->phihat[i][k+4] );
	}
    }

if (satnum >= 15)
    {
    /* Skip down to the non-linear correction coefficients */
    if (ccfindstr(fd, buffer, "NON-LINEAR CORRECTION FACTORS:") != E_SUCC)
	return(E_FAIL);

    /* Skip header info */
    if (ccreadline(fd,3,buffer) != E_SUCC)
	return(E_FAIL);

    /* Read the non-linear correction coefficients */
    for ( i = 0; i < 3; i++ )
	{
	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
	sscanf( buffer, "%d %lf %lf %lf", &j,
		&calstr->corr[i][0], &calstr->corr[i][1], &calstr->corr[i][2] );
	}
    }

/*
  Close the calibration file
*/
close( fd );

return E_SUCC;
}

/*******************************************************************************
FUNCTION:	CCREAD_DGRD1


PURPOSE:	Get the calibration coefficients for the visual channels from
		the degradation table (format such as that of "kaufman").

Programmer       Date   Modifications
---------------  -----  --------------------------------------------------------
T.Baltzer	 10/98	Make into it's own routine.


PROJECT:	ADAPS

ALGORITHM REFERENCES:

*******************************************************************************/

int ccread_dgrd1
(
    char *hostname,	/* IN  - Calibration table host file name	*/
    long satnum,	/* IN  - Satellite number			*/
    struct COEFFSTR *coeff /* OUT - Calibration coefficient structure	*/
)
{

long
    nchans,		/* Number of channels in the table	*/
    stasat,		/* starting satellite number		*/
    endsat,		/* ending satellite number		*/
    i,j;		/* looping variable			*/

int fd;			/* file descriptor 			*/

char buffer[MAXBUF];	/* buffer to read navigation info into	*/



/*
  Open the degradation table
*/
fd = open(hostname,0);
if (fd < 0)
    {
    c_lasmsg(FALSE,"read_dgrd-opn","Error opening visual degradation table");
    return(E_FAIL);
    }

/* Skip down to the optical coefficients */
if (ccfindstr(fd, buffer, "OPTICAL COEFFICIENTS:") != E_SUCC)
    return(E_FAIL);

/* Skip header info */
if (ccreadline(fd,3,buffer) != E_SUCC)
    return(E_FAIL);

/* Retrieve the defined satellite range */
sscanf(buffer, "%d %d %d",&stasat,&endsat,&nchans);

/* Check to see if the values exist for specified satellite */
if ((satnum < stasat) || (satnum > endsat))
    {
    close(fd);
    c_lasmsg(FALSE,"read_dgrd-sat",
	     "Entry for NOAA-%d not found in degradation table",satnum);
    return(E_FAIL);
    }

/* Skip header info */
if (ccreadline(fd,4,buffer) != E_SUCC)
    return(E_FAIL);

/* Read the table values */
for (i = stasat; i <= satnum; i++)
    {
    for (j = 0; j < nchans; j++)
	{
	if (ccreadline(fd,1,buffer) != E_SUCC)
	     return(E_FAIL);
	if (i == satnum)
	    {
	    sscanf(buffer, "%lf %lf %lf %lf %d",
		   &coeff->ocoeff.order1[j][0],&coeff->ocoeff.order1[j][1],
		   &coeff->ocoeff.order0[j][0],&coeff->ocoeff.order0[j][1],
		   &coeff->ocoeff.count[j]);
	    }
	}
    }

/* Close the degradation table */
close(fd);

return(E_SUCC);
}

/*******************************************************************************
FUNCTION:       CCREAD_DGRD2


PURPOSE:	Read the degradation tables of the format used by
		the "rao", "teillet", "vermote" calibration method.


Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
T.Baltzer        12/98  Made into it's own function.

PROJECT         ADAPS

ALGORITHM REFERENCES

*******************************************************************************/

int ccread_dgrd2
(
    char *hostname,	/* IN  - Calibration table host file name             */
    long adate,		/* IN  - Scene acquisition date                       */
    long satnum,	/* IN  - Satellite number                             */
    long *launch,	/* OUT - launch date as yyddd                         */
    double a[2],	/* OUT - ((counts/rad)/day)/day                       */
    double b[2],	/* OUT - (counts/rad)/day                             */
    double c[2],	/* OUT - counts/radiance                              */
    double d[2],	/* OUT - (counts/day)/day                             */
    double e[2],	/* OUT - counts/day                                   */
    double f[2]	/* OUT - counts                                       */
)
{

long
    end,		/* end date for piecewise coefficients                */
    satno,		/* satellite number                                   */
    start;		/* start date for piecewise coefficients              */

char buffer[MAXBUF];	/* buffer to read navigation info into                */

int fd;			/* file descriptor                                    */


/*
  Open the calibration degradation table
*/
fd = open(hostname,0);
if (fd < 0)
    {
    c_lasmsg(FALSE,"read_dgrd-opn","Error opening visual degradation table");
    return(E_FAIL);
    }

/* Skip down to the optical coefficients */
if (ccfindstr(fd, buffer, "OPTICAL COEFFICIENTS:") != E_SUCC)
    return(E_FAIL);

/* Skip header line */
if (ccreadline(fd,1,buffer) != E_SUCC)
    return(E_FAIL);

for(;;)
    {
    /* Retrieve the satellite number */
    if (ccreadline(fd,1,buffer) != E_SUCC)
	{
	c_lasmsg(FALSE,"read_dgrd-sat",
		 "Entry for NOAA-%d not found in degradation table",satnum);
	return(E_FAIL);
	}
    sscanf(buffer, "%*s %d",&satno);

    /* Retrieve the launch date */
    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);
    sscanf(buffer, "%*s %*s %d",launch);

    /* Retrieve the start, end date */
    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);
    sscanf(buffer, "%*s %*s %d %d",&start,&end);

    /* Convert from YYJJJ to YYYYJJJ */
    start = c_4digitdt(start);
    end = c_4digitdt(end);
    adate = c_4digitdt(adate);

    /*
      Determine if this is the correct record.
    */
    if ((satno == satnum) && (start <= adate) && (adate <= end))
 	{
    	/*
      	  Retrieve a,b,c,d,e,f
    	*/
	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
    	sscanf(buffer, "%*s %lf %lf",&a[0],&a[1]);

	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
    	sscanf(buffer, "%*s %lf %lf",&b[0],&b[1]);

	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
    	sscanf(buffer, "%*s %lf %lf",&c[0],&c[1]);

	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
    	sscanf(buffer, "%*s %lf %lf",&d[0],&d[1]);

	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
    	sscanf(buffer, "%*s %lf %lf",&e[0],&e[1]);

	if (ccreadline(fd,1,buffer) != E_SUCC)
	    return(E_FAIL);
    	sscanf(buffer, "%*s %lf %lf",&f[0],&f[1]);

	break;
	}
    else
	{
	/* This was not the correct time set,*/
	/* skip 6 records to get to next set */
	if (ccreadline(fd,6,buffer) != E_SUCC)
	    return(E_FAIL);
	}
    }

/*
  Close the degradation table
*/
close(fd);

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	CCFINDSTR

PURPOSE:	Read the calibration table until a line beginning with a
		specific string pattern is found.

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
  1.0	12/98	T. Baltzer	Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
******************************************************************************/

int ccfindstr
(
    long fd,			/* file descriptor			*/
    char buffer[MAXBUF],		/* buffer to read data into		*/
    char srchstr[]		/* String to search file in file	*/
)

{

long status;			/* Return status			*/

for ( ; ; )
    {
    status = read( fd, buffer, CALLEN );
    if ( status != CALLEN )
	{
	close( fd );
	c_lasmsg(FALSE,"ccread-rd",
		"Error reading satellite calibration table");
	return(E_FAIL);
        }

    if (strncmp(srchstr,buffer,strlen(srchstr)) == 0)
	break;
    }

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	CCREADLINE

PURPOSE:	Read the calibration table for the specified number of
		lines.  Return the last line read in the buffer.

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
  1.0	12/98	T. Baltzer	Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
******************************************************************************/

int ccreadline
(
    long fd,			/* file descriptor			*/
    long nlines,			/* Number of lines to skip		*/
    char buffer[MAXBUF]		/* buffer to read data into		*/
)

{

long status;			/* Return status			*/
long i;				/* Counter				*/

for (i = 0; i < nlines; i++)
    {
    status = read( fd, buffer, CALLEN );
    if ( status != CALLEN )
	{
	close( fd );
	c_lasmsg(FALSE,"ccread-rd",
		"Error reading satellite calibration table");
	return(E_FAIL);
        }
    }

return(E_SUCC);
}

int ccread_noaa
(
long satnum,		/* IN - satellite number		*/
long acquisition_year,	/* IN - acquisition year		*/
long acquisition_jday,	/* IN - acquisition julian date		*/
struct CALSTR *calstr	/* OUT - calibration structure		*/
)
{
char    buffer[MAXBUF];         /* buffer to read data into             */
char    calfil[CMLEN];          /* calibration file                     */
char    direct[CMLEN];          /* directory of the specified file      */
long    fd;                     /* file descriptor                      */
long table_satnum;	/* satellite number from calibration table	*/
long start;		/* start date from calibration table		*/
long end;		/* end date from calibration table		*/
long adate;		/* 7-digit date built from input year and jday	*/

/* Convert the acquisition date to one that
can be easily compared with the calibration
table dates.  YYYYJJJ.
-------------------------------------------*/
adate = c_4digityr(acquisition_year) * 1000;
adate += acquisition_jday;

/* Build the calibration table name
-----------------------------------*/
strcpy( direct, getenv( "ADAPSTABLES" ));
sprintf ( calfil, "%s/%s%s", direct, "noaa", ".cal" );

/* Open the file and position the file pointer
at the first line of optical coefficient data.
----------------------------------------------*/
fd = open( calfil, 0 );
if (ccfindstr(fd, buffer, "OPTICAL COEFFICIENTS:") != E_SUCC)
    return(E_FAIL);
if (ccreadline(fd,1,buffer) != E_SUCC)
    return(E_FAIL);

for(;;)
    {
    /* retrieve the satellite number */
    if (ccreadline(fd,1,buffer) != E_SUCC)
        return E_FAIL;
    sscanf(buffer, "%*s %ld",&table_satnum);

    /* Retrieve the start, end date */
    if (ccreadline(fd,1,buffer) != E_SUCC)
	return(E_FAIL);
    sscanf(buffer, "%*s %*s %ld %ld",&start,&end);

    /* is it the correct record? */
    if ((table_satnum == satnum) && (start <= adate) && (adate <= end))
        {
	/* it is so get the three lines of data and store in the struct */
        if (ccreadline(fd,1,buffer) != E_SUCC)
	    return E_FAIL;
	sscanf(buffer, "%*s %lf %lf %lf %lf %ld", &(calstr)->vsgain[0][0],
		&(calstr)->vsntcp[0][0], &(calstr)->vsgain[0][1],
		&(calstr)->vsntcp[0][1], &(calstr)->vscnt[0] );
        if (ccreadline(fd,1,buffer) != E_SUCC)
	    return E_FAIL;
	sscanf(buffer, "%*s %lf %lf %lf %lf %ld", &(calstr)->vsgain[1][0],
		&(calstr)->vsntcp[1][0], &(calstr)->vsgain[1][1],
		&(calstr)->vsntcp[1][1], &(calstr)->vscnt[1] );
        if (ccreadline(fd,1,buffer) != E_SUCC)
	    return E_FAIL;
	sscanf(buffer, "%*s %lf %lf %lf %lf %ld", &(calstr)->vsgain[2][0],
		&(calstr)->vsntcp[2][0], &(calstr)->vsgain[2][1],
		&(calstr)->vsntcp[2][1], &(calstr)->vscnt[2] );
	break;
	}
    else
        {
	/* It wasn't so skip the three lines and go to next record */
	if (ccreadline(fd,3,buffer) != E_SUCC)
	    return E_FAIL;
	}
    }

close(fd);
return E_SUCC;
}
