/******************************************************************************
FUNCTION:	CCSCONFIG


PURPOSE:	Determine the satellite sensor recording configuration.

Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
T. Baltzer       11/98  Development
D.Lloyd          04/01	Added calls to a new local routine confirmed().  The
			routine attempts to confirm that a sensor change has
			actually occurred.  CR 4802

PROJECT:	ADAPS

ALGORITHM REFERENCES:

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "calcoef.h"

#define OPTICALBAND_3A    1
#define THERMALBAND_3B    0

#define OPTICALTHRESHOLD  0.5
#define THERMALTHRESHOLD  0.5


int confirmed( int band, short min_frm[] );


int ccsconfig
(
    short min_frm[],		/* Buffer to read minor frame data into */
    struct MODEL *model,	/* Model and orbital information	*/
    struct CALSTR *calstr	/* Calibration parameters structure 	*/
)

{

/*
  Set the reconfiguration flag to signify that no change has occurred
*/
calstr->reconfig = FALSE;

/*
  If this is a drop line, we cannot determine sensor configuration.
*/
if ((min_frm[0]  == 0) && (min_frm[9]  == 0) &&
    (min_frm[10] == 0) && (min_frm[11] == 0)) 
    return(E_SUCC);

/*
  Those NOAA satellites before NOAA-15 (NOAA-KLM) always have 2 optical
  channels and 3 thermal channels (channel 3 does not toggle).
*/
if (model->satnum < 15)
    {
    if ((calstr->nthermbnds != MAXTHERMBND) ||
	(calstr->nopticbnds != (5 - MAXTHERMBND)))
	{
	calstr->reconfig = TRUE;
	calstr->nthermbnds = MAXTHERMBND;
	calstr->nopticbnds = 5 - MAXTHERMBND;
	}
    return(E_SUCC);
    }

/*
  The least significant bit of the 7th word in the minor frame determines
  which sensor is being recorded:
    0 = Chan 3B (thermal band)
    1 = Chan 3A (optical band)
*/

if ((min_frm[6] & 1) == OPTICALBAND_3A)	/* Chan 3A */ 
    {
    if (calstr->nopticbnds != MAXOPTICBND)
        /* Try to confirm that the sensor data is visible data CR 4802
	--------------------------------------------------------------*/
        if ( confirmed( OPTICALBAND_3A, min_frm ))
	    {
	    calstr->reconfig = TRUE;
	    calstr->nopticbnds = MAXOPTICBND;
	    calstr->nthermbnds = 5 - MAXOPTICBND;
	    }
    }
else if ((min_frm[6] & 1) == THERMALBAND_3B)	/* Chan 3B */
    {
    if (calstr->nthermbnds != MAXTHERMBND)
        /* Try to confirm that the sensor data is thermal data CR 4802
	--------------------------------------------------------------*/
        if ( confirmed( THERMALBAND_3B, min_frm ))
	    {
	    calstr->reconfig = TRUE;
	    calstr->nthermbnds = MAXTHERMBND;
	    calstr->nopticbnds = 5 - MAXTHERMBND;
	    }
    }
else
    {
    c_lasmsg(FALSE, "sconfig-sens","Error determining sensor configuration");
    return(E_FAIL);
    }

return(E_SUCC);
}

/************************************************************************
FUNCTION:	confirmed

PURPOSE:	Attempt to confirm that the channel 3 sensor is the same
		as that passed in the band parameter.  CR 4802

INPUT:		band flag: OPTICALBAND_3A or THERMALBAND_3B
		minor frame

RETURNS:	TRUE or FALSE

ALGORITHM:	Within the minor frame there are counts for space data.
		The space data for the thermal channels has been shown
		to be consistant (typically in the 900 to 920 range).
		The visible channel 3 counts are much lower (30 to 40).
		This function determines the mean normalized difference
		between channel 3 space counts and channel 4 and 5 space
		counts.  A normalized difference of 0.0 would indicate
		that the channel 3 is a thermal channel, whereas a value
		of 1 would indicate a visible channel.  Therefore, if
		the band parameter and the channel data are consistent
		return a TRUE value indicating they are the same and
		FALSE otherwise.
************************************************************************/
int confirmed
(
int   band,		/* IN - OPTICALBAND_3A or THERMALBAND_3B	*/
short min_frm[]		/* IN - buffer of minor frame data		*/
)
{
int   i;		/* loop index					*/
float normalized_diff;	/* normalized difference			*/
int   total_diffs;	/* total number of differences taken		*/

normalized_diff = 0.0;
total_diffs = 0;

/* The space counts begin at 52 in the minor frame.  They are multiplexed
as AVHRR chan1, chan2, chan3, chan4 and chan5.  There are ten counts for
each channel.  The first channel 3 count is at index 54.
------------------------------------------------------------------------*/
for ( i = 54; i < 102; i+= 5 )
    {
    if ( min_frm[i+1] > 0.0 ) /* prevent division by zero */
	{
	/* compute channel 3 and 4 normalized difference */
        normalized_diff += fabs( min_frm[i] - min_frm[i+1] ) /
	                       ( min_frm[i] + min_frm[i+1] );
	total_diffs++;
	}
    if ( min_frm[i+2] > 0.0 ) /* prevent division by zero */
	{
	/* compute channel 3 and 5 normalized difference */
        normalized_diff += fabs( min_frm[i] - min_frm[i+2] ) /
	                       ( min_frm[i] + min_frm[i+2] );
	total_diffs++;
	}
    }

/* compute the mean normalized difference
-----------------------------------------*/
normalized_diff /= (float)total_diffs;

/* Check the normalized difference against the input band.  Note: An
arbitrary value of 0.5 has been chosen as the cutoff.  It may be more
correct to insure the normalized_diff values are more restricted.  In
case the data is noisy and falls in the mid range of [0..1] we can't say
with any certainty that the data is one channel or the other.  In this
case we would just return a FALSE.  Eg. greater than 0.9 for optical and
less than 0.1 for thermal.  (the values are only example values).
------------------------------------------------------------------------*/
if ( band == OPTICALBAND_3A && normalized_diff > OPTICALTHRESHOLD )
    return TRUE;
else if ( band == THERMALBAND_3B && normalized_diff < THERMALTHRESHOLD )
    return TRUE;

return FALSE;
}  /* END confirmed */
