/*******************************************************************************
FUNCTION:      CCUPDT

PURPOSE:	Updates calibration coefficients.  
		Recalculates coefficients once per calibration period.
		Returns TRUE if the coefficients have been recalculated,
			FALSE otherwise.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      04/87    P. Mumford          Initial development
  1.1      01/90    C. Wivell           Optical sensor degradation and
                                        illumination correction
  5.0      02/90    D. Hollaren         Conversion to LAS-5.0
  5.1      10/91    T. Rockvam		Using model to find satellite position;
					correctly set up prtndx as offset into
					array of prt counts
  5.2	   01/93    T. Baltzer		Modified check for drop line
  5.2	   01/93    D. Hollaren		Modified to use the terrain model
  5.3      01/98    D.Lloyd		Changed name to ccupdt()
  5.4	   01/99    T. Baltzer		Revamp for NOAA-15 (NOAA-KLM)
					calibration.
  5.5      03/00    D.Lloyd             Added code to pick the median value
  					of the three PRT counts instead of
					the second value.
  5.6      03/00    D.Lloyd		Changed the computation of the average
  					temperature to be a simple average
					without using zero PRT counts in the
					equation.
  5.7      03/00    D.Lloyd		Changed the way PRT counts are entered
  					into the prtcnt array to prevent the
					loss of synchronization with the prtndx
					array due to extra or missing lines
					in the data.
  5.8	   10/00    T. Baltzer		Abandon 60-point spline for KLM.
  5.9      12/00    D.Lloyd		Corrected a division by zero error when
 					the prt population was zero for scenes
					beginning with drop lines. CR 2797
  6.0      01/01    D.Lloyd		Added code to keep a running count of
  					the internal and space counts and to
					calculate a mean used to determine if
					the count should be used in the intsum
					and spcsum values.  CR 3483

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT         ADAPS

ALGORITHM REFERENCES:
		NOAA Technical Memorandum NESS 107
		  Section 5.1 AVHRR Calibration, Appendix A, Appendix B
		NOAA Polar Orbiter Data User's Guide
		  K. Kidwell, NOAA
		  Section 3.3 Calibration of AVHRR Data
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "adaps.h"
#include "calcoef.h"
#include "adapssup.h"
#include "local_protos.h"

#define  CALPER   5
#define  PRTREFERENCE  10
#define  SUMPOP        500  /* number of sums needed in internal and	*/
			    /* space count sums	CR 3483			*/
#define  THRESH        25   /* threshold value used to determine if an	*/
			    /* internal or space count is used CR 3483	*/

short medianof3( short, short, short );

int ccupdt
(
    struct CALSTR *calstr,	/* Calibration parameter structure	      */
    long curlin,		/* Current line                               */
    long mline,			/* Master line				      */
    short min_frm[],        	/* Buffer containing minor frame data         */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    struct COEFFSTR *lastcoeff,	/* Last set of Calibration coefficients	      */
    struct OCOEFFSTR *origocoeff,/* Original optical coefficients	      */
    long prtcnt[],		/* PRT counts                                 */
    long cntsum[],		/* Sum of prt counts                          */
    long prtsmp[],              /* Number of samples of prt counts            */
    long prtndx[],		/* Offsets to find the four sample prt values */
    long *nxtprt,		/* Index in PRTCNT of the next prt	      */
    long prtpop,		/* PRT population (CALPER*PRTPER)             */
    long *sampls,		/* Number of samps of target & space counts   */
    long intsum[],		/* Sum of internal counts                     */
    long spcsum[],		/* Sum of space counts                        */
    long slprim,                /* Starting line at which priming began	      */
    double bnmrp[][60],		/* Numerator of beta function phihat*deltanu  */
    double enmr[][60],		/* Numerator of exp function                  */
    struct MODEL *model,	/* Model and orbital information	      */
    long flag			/* Flag to force coefficient recalculation    */
)
{
int	num_prt;		/* number of prt's with valid counts	      */
long    line = 0,               /* line counter                               */
        i,j,k= 0,		/* loop counters                              */
	status = 0;             /* return status                              */

double  t = 0.0,		/* temperature for each prt                   */
        xi = 0.0,               /* average counts for each prt                */
        tmpave = 0.0;           /* average temperature                        */
         
static  int probindex = 0;	/* index into probation array		      */
static  int throwcount = 0;	/* count of PRT values thrown away	      */
static	long probation[CALPER] = {0};	/* holding array for PRT counts that  */
					/* are on probation awaiting clearence*/
					/* to be put into prtcnt array	      */
static double lasttempave;	/* Last valid average PRT temperature	      */
static long  lastintsum[MAXTHERMBND];/* Last valid sum of internal counts     */
static long  lastspcsum[MAXTHERMBND];/* Last valid sum of space counts	      */
static long  lastsampls;     /* Last valid # of samples of target, space      */

/* Added for CR 3483
--------------------*/
long        internalmean;		/* mean of internal counts            */
long        spacemean;			/* mean of space counts               */
/* These declared static to maintain their values across calls                */
static int  sumpop_flag = FALSE;	/* flag: all counts > sumpop	      */
static long internalsums[MAXTHERMBND] = {0}; /* sum of internal space counts  */
static long spacesums[MAXTHERMBND] = {0}; /* sum of space counts              */
static long internalcount[MAXTHERMBND] = {0}; /* total of internal counts     */
static long spacecount[MAXTHERMBND] = {0}; /* total of space counts           */


/*
  Start of calibration period and past PRT population; generate coefficients
  for previous period and initialize for new period
*/
if (((curlin - slprim) % calstr->calper) == 0 || flag)
    {
/*
    Compute sums for each of the 4 prt's
    Counts for the same thermometer come every 5 lines
    Go up to (prtpop-4) because prtndx[i] will read ahead up
    to 4 elements
*/
    for (line = 0; line < (prtpop - 4); line+=5)
        {
        for (i = 0; i < 4; i++)
            {
            if(prtcnt[line+prtndx[i]] > 0)
                {
                cntsum[i] = cntsum[i] + prtcnt[line+prtndx[i]];
                prtsmp[i] += 1;
                }
            }
        }

/*
    Convert prt count sums to temps using aij's and bi's:
    Ti = ai0 + (ai1)(xi) + (ai2)(xi**2) + (ai3)(xi**3) + (ai4)(xi**4)
    T = blT1 + blT2 + blT3 + blT4

Added for version 5.6 del.
    Since the bi's actually define a simple average: b1=0.25, b2=0.25,
    b3=0.25 and b4=0.25 for N14 and earlier satellites, and NOAA KLM
    series satellites do not have defined bi's we will calculate T
    as follows:

    T = ( T1 + T2 + T3 + T4 ) / 4.0

    Also, if xi = 0.0 do not set Ti = ai0.  These two changes eliminate
    the bias introduced when one or more PRT count is missing (zero value).
    See CR 3409.
*/
    tmpave = 0.0;
/*
    Care has been taken to insure that only full groups of four PRT counts
    are stored in prtcnt but, as an added precaution, count the non-zero
    PRT's that go into the sum.
*/
    num_prt = 0;
    for (k = 0; k < 4; k++)
        {           
	/* Only calculate t is there were valid PRT counts */
        if (prtsmp[k] != 0)
            {
            xi = (double)(cntsum[k]) / (double)(prtsmp[k]);
            t = calstr->aij[k][4];
            for (j = 3; j >= 0; j--)
	        t = (t * xi) +  calstr->aij[k][j];
	    tmpave += t;
	    num_prt++;
            }
        }
    if ( num_prt != 0 )
	{
	tmpave /= (double)num_prt;
	lasttempave = tmpave;
	}
/*
    If the average is near zero, use the last average
*/
    if ( tmpave <= 1.0 )
        tmpave = lasttempave;

/*
    If we don't have a valid space sum due to drop lines, use the previous
    valid one
*/
    if (*sampls == 0)
	{
	for (i = 0; i < MAXTHERMBND; i++) 
	    {
	    spcsum[i] = lastspcsum[i];
	    intsum[i] = lastintsum[i];
	    }
	*sampls = lastsampls;
	}

/*
    Convert average temperature to radiance, then compute coefficients.
*/
    if ( model->satnum <= 14 )
	cctmp2rad(calstr,coeff,sampls,intsum,spcsum,bnmrp,enmr,&tmpave);
    else
	cctmp2rad3(calstr,coeff,sampls,intsum,spcsum,&tmpave);


/*
    For optical channels, do illumination correction if asked for, or
    use values already calculated in getcals.
*/
    ccillum_corr(curlin,mline,calstr,coeff,origocoeff,model);

 
/*
    Test for zero coefficients
    and set up new set of last coefficients
    NOTE:  Thermal channels only use one set of coefficients,
           so setting the high albedo coeff's is unneccessary.
*/
    for (i = 0; i < calstr->nopticbnds; i++)
	for (k = 0; k < 1; k++)
	    {
	    if (coeff->ocoeff.order0[i][k] == 0.0) 
		coeff->ocoeff.order0[i][k] = lastcoeff->ocoeff.order0[i][k];
	    else
		lastcoeff->ocoeff.order0[i][k] = coeff->ocoeff.order0[i][k];
	    if (coeff->ocoeff.order1[i][k] == 0.0) 
		coeff->ocoeff.order1[i][k] = lastcoeff->ocoeff.order1[i][k];
	    else
		lastcoeff->ocoeff.order1[i][k] = coeff->ocoeff.order1[i][k];
	    }
    for (     ; i < 5; i++)
	{
	if (coeff->tcoeff.order0[i-MINOPTICBND] == 0.0) 
	    coeff->tcoeff.order0[i-MINOPTICBND] =
		lastcoeff->tcoeff.order0[i-MINOPTICBND];
	else
	    lastcoeff->tcoeff.order0[i-MINOPTICBND] =
		coeff->tcoeff.order0[i-MINOPTICBND];
	if (coeff->tcoeff.order1[i-MINOPTICBND] == 0.0) 
	    coeff->tcoeff.order1[i-MINOPTICBND] =
		lastcoeff->tcoeff.order1[i-MINOPTICBND];
	else
	    lastcoeff->tcoeff.order1[i-MINOPTICBND] =
		coeff->tcoeff.order1[i-MINOPTICBND];
	}

/*
    Re-initialize summation variables
    to start a new calibration period
*/
    for (i = 0; i < 4; i++)
        {
        cntsum[i] = 0;
        prtsmp[i] = 0;
        }
    for (i = 0; i < MAXTHERMBND; i++)
        {
	if (*sampls != 0)
	    {
	    lastintsum[i] = intsum[i];
	    lastspcsum[i] = spcsum[i];
	    }
        intsum[i] = 0;
        spcsum[i] = 0;
        }
    if (*sampls != 0)
	lastsampls = *sampls;
    *sampls = 0;
    status = TRUE;
    }
else
    status = FALSE;

/* Update the PRT data.  There are times when the PRT counts are corrupted
by the introduction of noise.  A complicating factor is that when these 
errors occur in lines that have zero PRT counts the error can significantly
affect the calibration coefficients.  Picking the median of the three counts
is a quick and simple way of eliminating one bad PRT count.  For example:

	 3 PRT counts    Median
Case 1	396  909  395      396
Case 2	128    0    0        0

In the examples both the 128 and 909 are caused by single bit errors.
Also note that finding two equal PRT counts is not sufficient since in
case one there are no equal values.

The probation array was introduced to keep the PRT values in the prtcnt
array synchronized with the prtndx array used to calculate the average PRT
count over prtpop lines.  Synchronization can be lost if there are added
or missing lines in the image subsequent to initializing the prtndx array.
See CR 3409 and DESIGN notes.
--------------------------------------------------------------------------*/
probation[probindex] = medianof3( min_frm[17], min_frm[18], min_frm[19] );

/* Make sure that we only put values less than PRTREFERENCE into
probation[0] location.  If it isn't throw it away.  There are
times when the reference value is non-zero (see N11 and N14
PRT counts) so we don't compare against zero.
-------------------------------------------------------------*/
if ( probindex == 0 )
    {
    if ( probation[probindex] < PRTREFERENCE )
        probindex++;
    else
        throwcount++;
    }

/* Make sure we only put values greater that PRTREFERENCE into the
rest of the probation array.  A value greater than PRTREFERENCE means
we don't have a pattern match.  In this case throw away the previous
entries and (implicitly) put the value in probation[0].
---------------------------------------------------------------------*/
else
    {
    if ( probation[probindex] >= PRTREFERENCE )
        probindex++;
    else
        {
	throwcount += probindex;
	probindex = 1;
	}
    }

/* Check for a pattern match.  If we have CALPER valid
values in probation, the index will reach CALPER.  If
so, move the values into prtcnt.
------------------------------------------------------*/
if ( probindex == CALPER )
    {
    for ( i = 0; i < CALPER; i++ )
        prtcnt[*nxtprt+i] = probation[i];
    if ( prtpop != 0 )
	*nxtprt = (*nxtprt + CALPER) % prtpop;
    else
        *nxtprt = 0;
    probindex = 0;
    }

/* Check for throwcount.  If there are CALPER or more values
that we have thrown away, put CALPER zero values into prtcnt
and decrement the count.  This will keep the PRT values in
prtcnt within CALPER lines of the image lines.
------------------------------------------------------------*/
if ( throwcount >= CALPER )
    {
    for ( i = 0; i < CALPER; i++ )
        prtcnt[*nxtprt+i] = 0;
    if ( prtpop != 0 )
	*nxtprt = (*nxtprt + CALPER) % prtpop;
    else
        *nxtprt = 0;
    throwcount -= CALPER;
    }


/* If the channel 3 sensor has changed, the internal and space sums are
no longer valid and need to be recomputed.  Zero the channel 3 values
and reset the sumpop flag so that new sums will be computed.
------------------------------------------------------------------------*/
if ( calstr->reconfig == TRUE )
    {
    sumpop_flag = FALSE;
    internalcount[0] = spacecount[0] = 0;
    internalsums[0] = spacesums[0] = 0;
    }


/*
  Update the space and blackbody data if
  the line is not a drop line

In noisy regions the internal and space counts tend to fluctuate.  This
fluctuation causes the final calibration coefficients to fluctuate as well.
The running sum and total of these counts were added so that a mean value
could be calculated.  Once a sufficient number of values have been added
into the sum, the value of SUMPOP is used, the mean is calculated.  If the
count in question is within +/-THRESH it is added into the sum, otherwise
the mean value is used.  This mechanism substantially decreases, but does
not eliminate, the coefficient fluctuation.
Added for CR 3483
---------------------------------------------------------------------------*/
if ((min_frm[0] != 0) || (min_frm[9] != 0) || (min_frm[10] != 0) || (min_frm[11] != 0))
    {
    /* Make sure that there are at least sumpop values in each of the
    counts before using them to determine the mean.
    -----------------------------------------------------------------*/
    if ( !sumpop_flag )
        {
	if ( internalcount[0] >= SUMPOP && spacecount[0]  >= SUMPOP &&
	     internalcount[1] >= SUMPOP && spacecount[1]  >= SUMPOP &&
	     internalcount[2] >= SUMPOP && spacecount[2]  >= SUMPOP )
            sumpop_flag = TRUE;
	}
    
    /* If there aren't enough values accumulated yet, unconditionally
    add values to the internal and space sums until we have enough.
    -----------------------------------------------------------------*/
    if ( !sumpop_flag )
	{
	for (i = 0; i < 10; i++) 
	    {
	    for (j = 0; j < MAXTHERMBND; j++) 
		{
		intsum[j] = intsum[j] + min_frm[21 + (i * 3) + (j + 1)];
		spcsum[j] = spcsum[j] + min_frm[53 + (i * 5) + (j + 1)];
		internalsums[j] += min_frm[21 + (i * 3) + (j + 1)];
		spacesums[j] += min_frm[53 + (i * 5) + (j + 1)];
		internalcount[j]++;
		spacecount[j]++;
		}
	    }
	}
    /* Otherwise, calculate a mean value of the internal and space counts.
    Only add the count into the sums if they are within +/-THRESH.  If not
    within the threshold add the mean value into the sums.
    ----------------------------------------------------------------------*/
    else
	{
	for (i = 0; i < 10; i++) 
	    {
	    for (j = 0; j < MAXTHERMBND; j++) 
		{
		/* calculate the means */
		internalmean = internalsums[j] / internalcount[j];
		spacemean = spacesums[j] / spacecount[j];

		/* check and add to the internal sum */
		if ((( min_frm[21+(i*3)+(j+1)]) - internalmean <  THRESH ) &&
		     ( min_frm[21+(i*3)+(j+1)]) - internalmean > -THRESH )
		    {
		    intsum[j] += min_frm[21 + (i * 3) + (j + 1)];
		    internalsums[j] += min_frm[21 + (i * 3) + (j + 1)];
		    internalcount[j]++;
		    }
		else
		    intsum[j] += internalmean;

		/* check and add to the space sum */
		if ((( min_frm[53+(i*5)+(j+1)]) - spacemean <  THRESH ) &&
		     ( min_frm[53+(i*5)+(j+1)]) - spacemean > -THRESH )
		    {
		    spcsum[j] += min_frm[53 + (i * 5) + (j + 1)];
		    spacesums[j] += min_frm[53 + (i * 5) + (j + 1)];
		    spacecount[j]++;
		    }
		else
		    spcsum[j] += spacemean;
		}  /* END for j loop */
	    }  /* END for i loop */
	}  /* END else */

    *sampls += 10;
    }  /* END if */

return(status);
}
/*******************************************************************************
FUNCTION:      CCTMP2RAD

PURPOSE:	Convert average temperature to radiance using C1, C2, n,
		and phihat:
		N(T) = sum 1 to n of beta(nui,T)phihat(nui)deltanu
		beta(nui,T) = (C1*nui**3) / ((C2*nui)/(exp(T)-1))
		Compute for thermal channels.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0	   09/98    T. Baltzer		Made into it's own function.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT         ADAPS

ALGORITHM REFERENCES:
		NOAA Technical Memorandum NESS 107
		  Section 5.1 AVHRR Calibration, Appendix A, Appendix B
		NOAA Polar Orbiter Data User's Guide
		  K. Kidwell, NOAA
		  Section 3.3 Calibration of AVHRR Data
*******************************************************************************/

void cctmp2rad
(
    struct CALSTR *calstr,		/* Calibration parameter structure	      */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    long *sampls,		/* Number of samps of target & space counts   */
    long intsum[],		/* Sum of internal counts                     */
    long spcsum[],		/* Sum of space counts                        */
    double bnmrp[][60],		/* Numerator of beta function phihat*deltanu  */
    double enmr[][60],		/* Numerator of exp function                  */
    double *tmpave		/* average temperature                        */
)

{
long	k,ichan = 0;		/* loop counters                              */

double	tplanck = 0.0,          /* temporary numerator of the Planck function */
        nt = 0.0,               /* radiance of internal target                */
        xsp = 0.0,              /* average space count                        */
        xt = 0.0,               /* average internal target count              */
        gain = 0.0;             /* temporary location for gain                */
         

for (ichan = (MAXTHERMBND - calstr->nthermbnds); ichan < MAXTHERMBND; ichan++) 
    {
    nt = 0.0;
    for (k = 0; k < calstr->n[ichan]; k++) 
	{
	if (*tmpave != 0)
	    {
	    tplanck = exp(enmr[ichan][k]/(*tmpave)) - 1.0;
	    if (tplanck != 0) nt = nt + bnmrp[ichan][k] / tplanck;
	    }
	}

    /*
      Compute gain and intercept
    */
    if (*sampls != 0)
	{
	xsp = (double)(spcsum[ichan]) / (double)(*sampls);
	xt = (double)(intsum[ichan]) / (double)(*sampls);
	if (fabs(xsp - xt) > 1)
	    gain = (calstr->nsp[ichan] - nt) / (xsp - xt);
	else
	    gain = 0.0;
	}
    else
	{ 
	gain = 0.0;
	}
    coeff->tcoeff.order1[ichan] = gain;
    coeff->tcoeff.order0[ichan] = calstr->nsp[ichan] - gain * xsp;
    }

return;
}
 
/*******************************************************************************
FUNCTION:      CCILLUM_CORR

PURPOSE:	For optical channels, do illumination correction if asked
		for, or use values already calculated in getcals.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      09/98    T. Baltzer          Made into it's own function

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT         ADAPS

ALGORITHM REFERENCES:
*******************************************************************************/

void ccillum_corr
(
    long curlin,			/* Current line                               */
    long mline,			/* Master line				      */
    struct CALSTR *calstr,		/* Calibration parameter structure	      */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    struct OCOEFFSTR *origocoeff,	/* Original optical calibration coefficients  */
    struct MODEL *model		/* Model and orbital information	      */
)
{
long    i,k = 0;		/* loop counter                               */

double  satlat,satlng = 0.0,	/* satellite lat/long                         */
	view_angles[3],		/* Satellite/Sun zenith and relative azimuth  */
	elev = 0.0,		/* Elevation				      */
	cos_zenith = 0.0;	/* Cosine of the solar zenith angle	      */

float   sample = 0.0,		/* current sample			      */
	cline = 0.0,		/* current line				      */
        aus = 0.0;		/* sun-earth distance correction factor	      */


/*
    For optical channels do illumination correction if asked for.
*/
if (calstr->illum)
    {
    /*
      Find the nadir sample based on the avhrr image type
    */
    if ((model->dtype == LAC) || (model->dtype == HRPT))
	sample = LAC_SAMPS / 2;
    else if (model->dtype == GAC)
	sample = GAC_SAMPS / 2.0 + 0.5;

    /*
    Get the satellite position (lat/long) based on line and sample
    in satellite projection
    */
    cline = (float)(curlin + mline);
    c_ls2ll(&cline,&sample,&elev,model,&satlat,&satlng,view_angles);

    /*
      Get the cosine of the solar zenith angle
    */
    cos_zenith = cos(view_angles[1]);

    if (cos_zenith < 0.1)
	cos_zenith = 0.1;

    /*
      Correct for variations in earth-sun distance, which introduce a 
      +/- 3.5% error in the computations. These errors are small at 
      equinoxes, but are significant at solstices.
    */
    sunearthdis(model->jday,&aus);
    for (i = 0; i < calstr->nopticbnds; i++)
	for (k = 0; k < 2; k++)
	    {
	    coeff->ocoeff.order0[i][k] = origocoeff->order0[i][k] /
					 cos_zenith * aus * aus;
	    coeff->ocoeff.order1[i][k] = origocoeff->order1[i][k] /
					 cos_zenith * aus * aus;
	    }
    }
 
return;
}

/*******************************************************************************
FUNCTION:	CCTMP2RAD3

PURPOSE:	Compute calibration coefficients for thermal channels.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0	   01/99    D. Lloyd		Modified CCTMP2RAD to compute the
  					quadratic coefficients for NOAA K
  1.1	   10/00    T. Baltzer		Abandon 60-point spline for KLM.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT         ADAPS

ALGORITHM REFERENCES:
		NOAA KLM User's Guide DRAFT April 98
		  Goodrum, Kidwell, and Winston, editors NOAA
		  Section 7.2 Calibration of AVHRR Data
		  and Appendix D.

ALGORITHM:

1) Convert average temperatue to radiance using:

    N(T) = (C1*nui**3) / (exp(C2*nui/((T-A)/B)) - 1)

    where
      C1 = 1.191062 x 10-5
      C2 = 1.4387863
    The values for nui, A, and B for channels 3B, 4, and 5 are unique for each
    spacecraft and can be found in Appendix D.

This equation is derived from the equations in Section 7.1.2.2 NOAA KLM, by
solving for the radiance N.

2) Compute the calibration coefficients.

    a) The channel 3B coeff are linear from:

	      nt ( xsp - Ce )
	Ne = ----------------------
	       ( xsp - xt )
	
	where Ne is radiance, Ce is count of earth data, xsp is count
	of space data, and xt count of internal target.  Note that
	a "non-zero" radiance of space is NOT used.

    b) Channel 4 and 5 coeff are quadratic from:

	    Ne = Nlin + Ncor				(1

				    ( xsp - Ce )
	    Nlin = C0 + ( nt - C0 )---------------	(2
				    ( xsp - xt )
	    
	    Ncor = C1 + C2*Nlin + C3*Nlin^2		(3

	The coeff a, b and c from:

	    Ne = a + b*Ce + c*Ce^2
	
	are obtained by substituting for Nlin and Ncor in equations
	(1 and (3 and combining the Ce terms.  (The algebra is left
	as an exercise for interested programmers) The constants C0, C1,
	C2, and C3 are found in NOAA KLM section 7.1.2.3.2 and in
	the noaa15.cal/noaa16.cal/noaa17.cal  file as Table
	NON-LINEAR CORRECTION FACTORS.
	They are stored in the calibration structure member corr.
	
*******************************************************************************/

int cctmp2rad3
(
    struct CALSTR *calstr,	/* Calibration parameter structure	      */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    long *sampls,		/* Number of samps of target & space counts   */
    long intsum[],		/* Sum of internal counts                     */
    long spcsum[],		/* Sum of space counts                        */
    double *tmpave		/* average temperature                        */
)

{
long	ichan = 0;		/* loop counters                              */

double	a,b,c,			/* conversion coefficients for ch 4 & 5       */
	alpha,beta,gamma,	/* intermediate results			      */
	denom = 0.0,		/* intermediate results denominator	      */
        nt = 0.0,               /* radiance of internal target                */
        xsp = 0.0,              /* average space count                        */
        xt = 0.0,               /* average internal target count              */
        gain = 0.0;             /* temporary location for gain                */
double  t_star;			/* effective blackbody temperature            */

for (ichan = (MAXTHERMBND - calstr->nthermbnds); ichan < MAXTHERMBND; ichan++) 
    {
    /* For NOAA KLM series satellites, the same values are used for all
    temperature ranges.  So hard code the last rats_vc subscript in case
    the value of calstr->temp_range has not been defined.
    --------------------------------------------------------------------*/
    t_star = calstr->rats_A[ichan] + (*tmpave * calstr->rats_B[ichan]);
    nt = C1 * calstr->rats_vc[ichan][0] * calstr->rats_vc[ichan][0] *
	      calstr->rats_vc[ichan][0];
    nt = nt / (exp(C2 * calstr->rats_vc[ichan][0] / t_star) - 1.0);

    if ( *sampls != 0 )
        {
	xsp = (double)(spcsum[ichan]) / (double)(*sampls);
	xt = (double)(intsum[ichan]) / (double)(*sampls);
	denom = xsp - xt;
	}

    /* Compute the linear calibration coefficients for 3B
    -----------------------------------------------------*/
    if ( calstr->nthermbnds == 3 && ichan == 0 )
        {
	if ( fabs( xsp - xt ) > 1 )
	    gain = -nt / ( xsp - xt );
	else
	    gain = 0.0;
	coeff->tcoeff.order0[ichan] = -gain * xsp;
	coeff->tcoeff.order1[ichan] = gain;
	}

    /* Compute the quadratic coefficients for 4 and 5
    -------------------------------------------------*/
    else
        {
	alpha = calstr->nsp[ichan] + calstr->corr[ichan][0] +
		(calstr->nsp[ichan] * calstr->corr[ichan][1]) +
		(calstr->nsp[ichan] * calstr->nsp[ichan] * 
		calstr->corr[ichan][2]);
	beta = ( 1.0 + calstr->corr[ichan][1] + ( 2.0 * calstr->nsp[ichan] *
		calstr->corr[ichan][2] )) * ( nt - calstr->nsp[ichan] );
	gamma = calstr->corr[ichan][2] * ( nt - calstr->nsp[ichan] ) *
		( nt - calstr->nsp[ichan] );

	if ( fabs(denom) > 0.0 )
	    {
	    a = alpha + (beta * xsp / denom) + (gamma*xsp*xsp/(denom*denom));
	    b = (-beta / denom) - (2.0 * xsp * gamma / (denom*denom));
	    c = gamma / (denom*denom);
	    }
	else
	    a = b = c = 0.0;
	coeff->tcoeff.order0[ichan] = a;
	coeff->tcoeff.order1[ichan] = b;
	coeff->tcoeff.order2[ichan] = c;
	}
    }

return E_SUCC;
}

/*******************************************************************************
FUNCTION:	MEDIANOF3

PURPOSE:	Return the median of three short values.
*******************************************************************************/
short medianof3
(
  short prt1,	/* IN: PRT value one	*/
  short prt2,	/* IN: PRT value two	*/
  short prt3	/* IN: PRT value three	*/
)
{
static short smallvalue[2];		/* the two smallest values	*/

/* Determine the two smallest of the three inputs
-------------------------------------------------*/
if ( prt1 > prt2 )
    {
    smallvalue[0] = prt2;
    if ( prt1 > prt3 )
	smallvalue[1] = prt3;
    else
	smallvalue[1] = prt1;
    }
else
    {
    smallvalue[0] = prt1;
    if ( prt2 > prt3 )
	smallvalue[1] = prt3;
    else
	smallvalue[1] = prt2;
    }

/* Return the larger of the two smallest values
-----------------------------------------------*/
if ( smallvalue[0] > smallvalue[1] )
    return smallvalue[0];
else
    return smallvalue[1];
}
