/******************************************************************************
FUNCTION:	findpolecross

PURPOSE:	Find line numbers where scene crosses poles, or maximum and
		minimum lines if it doesn't cross poles

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 5.0	   12/91    T. Rockvam		Original development for LAS 5.0
 5.1 	   1/93	    T. Rockvam		Fixed calculation of minline when 
					crossing North Pole and maxline when 
					crossing South Pole

ALGORITHM REFERENCES
	Uses Chuck Wivell's formulas for finding the line 
	corresponding to a latitude

*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "model.h"
#include "avhrr.h"
#include "adapssup.h"
#include "constants.h"
#include "local_protos.h"

void findpolecross
(
    long *maxline,		/* line number of maximum lat		*/
    long *minline,		/* line number of minimum lat		*/
    struct MODEL *model		/* Model and orbital information	*/
)
{
double	maxlat = HALFPIE;	/* maximum lat of pass = 90		*/
double	minlat = -HALFPIE;	/* minimum lat of pass = -90		*/
long	i;
long	minmul = 1;
long	maxmul = -1;
long	maxlns[3],
	minlns[3],
	quads[3];

/*
  For scenes that pass one or both poles, find line numbers at pole crossings
*/
if ((model->spass != model->cpass) || (model->spass != model->epass))
    {
    /* Find the line of maximum latitude in the image */
    if (((model->spass == ASCENDING) && (model->cpass == DESCENDING)) ||
        ((model->cpass == ASCENDING) && (model->epass == DESCENDING)))
	{
        /* find max lat & corresponding line number */
	getline(5,maxmul,maxlat,maxlns,quads,model);
	i = 0;
	while ((maxlns[i] < 0) && (i <= 3))
	    i++;
	*maxline = maxlns[i];
	}
    else
	/* we crossed the South Pole: */
	/* if cpass is DESC, most of the pass is DESC, */
	/* so the start is further north than the end */
	if (model->cpass == DESCENDING)
	    *maxline = 1;
	else
	    *maxline = model->nlines;

    /* Find the line of minimum latitude in the image */
    if (((model->spass == DESCENDING) && (model->cpass == ASCENDING)) ||
        ((model->cpass == DESCENDING) && (model->epass == ASCENDING)))
	{
        /* find min lat & corresponding line number */
	getline(5,minmul,minlat,minlns,quads,model);
	i = 0;
	while ((minlns[i] < 0) && (i <= 3))
	    i++;
	*minline = minlns[i];
	}
    else
	/* we crossed the North Pole: */
	/* if cpass is DESC, most of the pass is DESC, */
	/* so the end is further south than the start */
	if (model->cpass == DESCENDING)
	    *minline = model->nlines;
	else
	    *minline = 1;
    }
else	/* scene is all one direction */
    {
    if (model->cpass == DESCENDING)
	{
	*minline = model->nlines;
	*maxline = 1;
	}
    else
	{
	*minline = 1;
	*maxline = model->nlines;
	}
    }

return;
}
 
