/******************************************************************************
FUNCTION:	findrange

PURPOSE:	Finds the line numbers corresponding to the lat range

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 5.0	    2/92    T. Rockvam		Original development for LAS 5.0 

PROJECT		ADAPS

ALGORITHM REFERENCES
     This algorithm uses spherical trigonometry to find the location
     of the desired latitude in relation to the satellite's location
     at a given time in the scene.  This yields an approximate line
     number, which is then refined to find the exact latitude.
     Formulas used were developed by Chuck Wivell.

     The lines of lat are not parallel to the scan lines, and 
     are oriented differently depending if the pass is ascending or 
     descending.  
   EXAMPLE:
   --------           ASCENDING 			 DESCENDING 
                    long -10  long   0              long -10  long   0
                    samp  10  samp   0              samp   0  samp  10
                        +---------+                     +---------+
        line 10, lat 16 |lr     ll|      line  0 lat 16 |ul     ur|
                        |         |                     |         |
                        |         |                     |         |
        line  0, lat  6 |ur     ul|      line 10 lat  6 |ll     lr|
                        +---------+                     +---------+
  	lat increases/line # increases   lat increases/line # decreases
  	lng increases/samp # decreases   lng increases/samp # increases
  	west side is max sample          west side is min sample

*****************************************************************************/
#include <stdio.h>
#include "tae.h"
#include "worgen.h"
#include "model.h"
#include "avhrr.h"
#include "adapssup.h"
#include "local_protos.h"

#define MINMUL 1
#define MAXMUL -1

int findrange
(
    double lat1,		/* first lat in range			*/
    double lat2,		/* second lat in range			*/
    long startln[2],		/* line corresponding to first lat	*/
    long stopln[2],		/* line corresponding to second lat	*/
    long quadnum,		/* quad number				*/
    long areaopt,		/* areas to look for			*/
    long *numfound,		/* number of areas found		*/
    struct MODEL *model		/* Model and orbital information	*/
)
{
long 	firstmul,		/* multiplier for first pole passed	*/
 	firstpole,		/* line number of first pole passed	*/
 	i,			/* loop counters			*/
	maxline,		/* line number of max lat (North Pole)	*/
 	minline,		/* line number of min lat (South Pole)	*/
 	nfound = 0,		/* temporary number of matches found 	*/
 	nmatch = 0,		/* temporary number of matches found 	*/
 	nonfatal = 1,		/* nonfatal error flag			*/
 	passdir,		/* direction of found window pass	*/
 	secndmul,		/* multiplier for second pole passed	*/
 	secndpole,		/* line number of second pole passed	*/
 	templn1[3],		/* temporary line number 1 of matches found */
	templn2[3];		/* temporary line number 2 of matches found */

long	getlns1[3],
	getlns2[3],
	quads1[3],
	quads2[3];

double	hold,			/* used to swap variables 		*/
	templat1,templat2;	/* temp lats to go with each line number*/

/*
  Set up
*/
*numfound = 0;
for (i = 0; i < 3; i++)
    {
    getlns1[i] = 0;
    getlns2[i] = 0;
    }

/* 
  Find line numbers where the scene passes the poles
*/
findpolecross (&maxline,&minline,model);

/*
  Quadrants 1,2,3,4,5
*/
if (quadnum <= 5)
    {
/*
    Make sure order of lats matches center pass direction
    and set up multipliers
*/
    if (model->cpass == ASCENDING)
	{ 
	firstmul = MAXMUL;
	secndmul = MINMUL;
        if (lat1 > lat2)
	    {
	    hold = lat1; lat1 = lat2; lat2 = hold;
	    }
	}
    else
	{
	firstmul = MINMUL;
	secndmul = MAXMUL;
        if (lat1 < lat2)
	    {
	    hold = lat1; lat1 = lat2; lat2 = hold;
	    }
	}

    getline(quadnum,firstmul,lat1,templn1,quads1,model);
    getline(quadnum,secndmul,lat2,templn2,quads2,model);

    if (quadnum <= 4)
	{
        /*
          Find the sets of lines that fell in the quad
	  Give priority to areas where both ends are in the quad
        */
        for (i = 0; i < 3; i++)
	    {
	    if ((quads1[i] == quadnum) && (quads2[i] == quadnum))
	        {
	        /* both ends of area are in the right quad */
	        getlns1[nmatch] = templn1[i];
	        getlns2[nmatch] = templn2[i];
	        nmatch += 1;
	        }
	    }
        for (i = 0; i < 3; i++)
	    {
	    if (((quads1[i] == quadnum) || (quads2[i] == quadnum)) &&
	         (quads1[i] != quads2[i]))
	        {
	        /* only one end of area is in the right quad */
	        getlns1[nmatch] = templn1[i];
	        getlns2[nmatch] = templn2[i];
	        nmatch += 1;
	        }
	    }
	}
    else /* quad 5, take all lines found regardless of quad */
	{
	for (i = 0; i < 3; i++)
	    {
	    getlns1[i] = templn1[i];
	    getlns2[i] = templn2[i];
	    }
	nmatch = 3;
	}

    /*
      Process each set of lines that fell in the quad
    */
    for (i = 0; i < nmatch; i++)
	{
	if (((getlns1[i] < 1) && (getlns2[i] < 1)) ||
	    ((getlns1[i] > model->nlines) && (getlns2[i] > model->nlines)))
	    ;
	else
	    {
	    /* set up temporary latitudes to stay with each set of lines */
	    templat1 = lat1;
	    templat2 = lat2;

	    /* refine to get the exact line numbers */
            if ((quadnum == 5) && 
		(getlns1[i] < maxline) && (getlns2[i] > maxline) && 
		(maxline != 1) && (maxline != model->nlines))
		{
		/* crossed North Pole 	*/
                refinepole(templat1,templat2,&getlns1[i],&getlns2[i],1,
			   maxline,1,model);
		}
		
            else if ((quadnum == 5) &&
		     (getlns1[i] < minline) && (getlns2[i] > minline) &&
		     (minline != 1) && (minline != model->nlines))
		{
		/* crossed South Pole 	*/
                refinepole(templat1,templat2,&getlns1[i],&getlns2[i],1,
			   maxline,1,model);
		}
	    else /* not a polar crossing */
	   	{
    	        /* set up in ascending line number order to find out passdir */
    	        if (getlns1[i] > getlns2[i])
	            {
	            hold=getlns1[i];
		    getlns1[i]=getlns2[i];
		    getlns2[i]=(long)hold;
	            hold = templat1;
		    templat1 = templat2;
		    templat2 = hold;
	            }

	        /* find out the pass direction at the found lines */
	        if ((model->spass == model->cpass) &&
		    (model->spass == model->epass))
		    passdir = model->cpass;
	        else
		    {
                    if (model->cpass == DESCENDING)
            	        if (getlns2[i] > maxline)
                            if (getlns1[i] >= minline)
                    	        passdir = ASCENDING;
                            else
                    	        passdir = DESCENDING;
            	        else
                	        passdir = ASCENDING;
                    else
            	        if (getlns2[i] > minline)
                            if (getlns1[i] >= maxline)
                    	        passdir = DESCENDING;
                            else
                    	        passdir = ASCENDING;
            	        else
            	                passdir = DESCENDING;
		    }

	        /* make sure order of lats matches passdir */
    	        if ((passdir == ASCENDING) && (templat1 > templat2))
		    {
		    hold=getlns1[i];
		    getlns1[i]=getlns2[i];
		    getlns2[i]=(long)hold;
		    hold = templat1;
		    templat1 = templat2;
		    templat2 = hold;
		    }
    	            else if ((passdir == DESCENDING) && (templat1 < templat2))
		    {
		    hold=getlns1[i];
		    getlns1[i]=getlns2[i];
		    getlns2[i]=(long)hold;
		    hold = templat1;
		    templat1 = templat2;
		    templat2 = hold;
		    }

	        /* refine to get the exact line numbers */
    	        refine(templat1,templat2,&getlns1[i],&getlns2[i],
		       maxline,minline,passdir,model);
		}
    	    if (getlns1[i] < 1)
        	getlns1[i] = 1;
    	    if (getlns2[i] > model->nlines)
        	getlns2[i] = model->nlines;
	    templn1[nfound] = getlns1[i];
	    templn2[nfound] = getlns2[i];
	    nfound += 1;
	    }
	}

    if (nfound == 0)
	{
	c_errmsg("No areas fell within the specified quad","findrange-out",
	    &nonfatal);
        return(E_FAIL);
	}
    if (areaopt == 0)           /* take first area found */
        {
        startln[0] = templn1[0];
        stopln[0] = templn2[0];
        *numfound = 1;
        }
    else if (areaopt == 1)      /* take second area found */
        if (nfound < 2)
            {
            c_errmsg("No second latitude area was found","findrange-nosecond",
                      &nonfatal);
            return(E_FAIL);
            }
        else
            {
            startln[0] = templn1[1];
            stopln[0] = templn2[1];
            *numfound = 1;
            }
    else /* if (areaopt == 2) */     /* take both areas found */
        if (nfound >= 2)
            {
            startln[0] = templn1[0];
            stopln[0] = templn2[0];
            startln[1] = templn1[1];
            stopln[1] = templn2[1];
            *numfound = 2;
            }
        else /* nfound == 1 */
            {
            c_errmsg("Only one latitude area was found","findrange-onearea",
                      &nonfatal);
            startln[0] = templn1[0];
            stopln[0] = templn2[0];
            *numfound = 1;
            }
    }

/* 
  North Pole crossing
*/
else if (quadnum == 6)
    {
    if ((maxline == model->nlines) || (maxline == 1))
	{
	c_errmsg("This scene does not pass the North Pole","findrange-pole",
	    &nonfatal);
	return(E_FAIL);
	}
    /* Find line before the pole */
    getline(quadnum,MAXMUL,lat1,getlns1,quads1,model);
    startln[0] = 0;
    for (i = 2; i >= 0; i--)
	if (getlns1[i] <= maxline)
	    {
	    startln[0] = getlns1[i];
	    break;
	    }
    /* Find line after the pole */
    getline(quadnum,MAXMUL,lat2,getlns2,quads2,model);
    stopln[0] = model->nlines + 1;
    for (i = 0; i < 3; i++)
	if (getlns2[i] >= maxline)
	    {
	    stopln[0] = getlns2[i];
	    break;
	    }

    if (((startln[0] < 1) && (stopln[0] < 1)) ||
	((startln[0] > model->nlines) && (stopln[0] > model->nlines)))
	{
	c_errmsg("No areas crossed the specified pole","findrange-out",
	    &nonfatal);
        return(E_FAIL);
	}
    else
	{
    	refinepole(lat1,lat2,&startln[0],&stopln[0],1,maxline,1,model);
    	if (((startln[0] < 1) && (stopln[0] < 1)) ||
	    ((startln[0] > model->nlines) && (stopln[0] > model->nlines)))
	    {
	    c_errmsg("Both latitudes fell outside of the image","findrange-out",
	        &nonfatal);
            return(E_FAIL);
	    }
    	else 
	    {
	    if (startln[0] < 1)
                startln[0] = 1;
	    if (stopln[0] > model->nlines)
	        stopln[0] = model->nlines;
            *numfound = 1;
	    }
	}
    }

/* 
  South Pole crossing
*/
else if (quadnum == 7)
    {
    if ((minline == model->nlines) || (minline == 1))
	{
	c_errmsg("This scene does not pass the South Pole","findrange-pole",
	    &nonfatal);
	return(E_FAIL);
	}
    /* Find line before the pole */
    getline(quadnum,MINMUL,lat1,getlns1,quads1,model);
    startln[0] = 0;
    for (i = 2; i >= 0; i--)
	if (getlns1[i] <= minline)
	    {
	    startln[0] = getlns1[i];
	    break;
	    }
    /* Find line after the pole */
    getline(quadnum,MINMUL,lat2,getlns2,quads2,model);
    stopln[0] = model->nlines + 1;
    for (i = 0; i < 3; i++)
	if (getlns2[i] >= minline)
	    {
	    stopln[0] = getlns2[i];
	    break;
	    }

    if (((startln[0] < 1) && (stopln[0] < 1)) ||
	((startln[0] > model->nlines) && (stopln[0] > model->nlines)))
	{
	c_errmsg("No areas crossed the specified pole","findrange-out",
	    &nonfatal);
        return(E_FAIL);
	}
    else
	{
        refinepole(lat1,lat2,&startln[0],&stopln[0],-1,minline,
	    model->nsamples,model);
        if (((startln[0] < 1) && (stopln[0] < 1)) ||
	    ((startln[0] > model->nlines) && (stopln[0] > model->nlines)))
	    {
	    c_errmsg("Both latitudes fell outside of the image","findrange-out",
	        &nonfatal);
            return(E_FAIL);
	    }
        else 
	    {
	    if (startln[0] < 1)
                startln[0] = 1;
	    if (stopln[0] > model->nlines)
	        stopln[0] = model->nlines;
            *numfound = 1;
	    }
	}
    }

/*
  Cross both poles
*/
else if (quadnum == 8)
    {
    /* find part of pass that passes the pole and which pole we pass first*/
    if ((model->spass == ASCENDING) && 
	(model->cpass == DESCENDING) && 
	(model->epass == ASCENDING))
	{
	if (lat1 < lat2)
	    {
	    hold = lat1; lat1 = lat2; lat2 = hold;
	    }
	firstpole = maxline;
	secndpole = minline;
	firstmul = MAXMUL;
	secndmul = MINMUL;
	passdir = ASCENDING;
	}
    else if ((model->spass == DESCENDING) && 
	     (model->cpass == ASCENDING) && 
	     (model->epass == DESCENDING))
	{
	if (lat1 > lat2)
	    {
	    hold = lat1; lat1 = lat2; lat2 = hold;
	    }
	firstpole = minline;
	secndpole = maxline;
	firstmul = MINMUL;
	secndmul = MAXMUL;
	passdir = DESCENDING;
	}
    else
	{
	c_errmsg("This scene does not pass both poles","findrange-pole",
	    &nonfatal);
	return(E_FAIL);
	}
    /* Find line before the pole */
    getline(quadnum,firstmul,lat1,getlns1,quads1,model);
    startln[0] = 0;
    for (i = 2; i >= 0; i--)
	if (getlns1[i] <= firstpole)
	    {
	    startln[0] = getlns1[i];
	    break;
	    }
    /* Find line after the pole */
    getline(quadnum,secndmul,lat2,getlns2,quads2,model);
    stopln[0] = model->nlines + 1;
    for (i = 0; i < 3; i++)
	if (getlns2[i] >= secndpole)
	    {
	    stopln[0] = getlns2[i];
	    break;
	    }

    refine(lat1,lat2,&startln[0],&stopln[0],secndpole,firstpole,
	   passdir,model);
    if (startln[0] < 1)
        startln[0] = 1;
    if (stopln[0] > model->nlines)
	stopln[0] = model->nlines;
    *numfound = 1;
    }

return(E_SUCC);
}
 
