/******************************************************************************
FUNCTION:	REFINE

PURPOSE:	Refine the line number of the lat of interest; this is
		done because getline gives only an approximation

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 5.0 	   12/91    T. Rockvam		Original development for LAS 5.0 
					based on fndsln from old makencal

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "model.h"
#include "avhrr.h"

void refine
(
    double lat1,
    double lat2,
    long *line1,	/* northern-most line			*/
    long *line2,	/* southern-most line			*/
    long maxln,		/* line number of maximum latitude	*/
    long minln,		/* line number of minimum latitude      */
    long passdir,	/* direction of the user-specified pass	*/
    struct MODEL *model /* Model and orbital information	*/
)
{
double	northlat,	/* northern latitude			*/
	southlat,	/* southern latitude			*/
	maxline,	/* line of maximum latitude		*/
	minline,	/* line of minimum latitude		*/
	thislat,	/* lat of current line			*/
	thislong,	/* long of current line			*/
	elev = 0.0;	/* Elevation				*/

float	northline,	/* current line number of north lat	*/
	southline,	/* current line number of south lat	*/
	northside,	/* side of image to search for north lat*/
	southside,	/* side of image to search for south lat*/
	northstep,	/* how to move north line by line	*/
	southstep;	/* how to move south line by line	*/

/*
  Set up lines of max & min latitude
*/
maxline = (float)maxln;
minline = (float)minln;

/*
  Set up for north & south lines
*/
if (passdir == DESCENDING)
    {
    northlat = lat1;
    southlat = lat2;
    northline = (float)*line1;
    southline = (float)*line2;
    southside = 1.0;
    if (model->dtype == GAC)
        northside = (float)GAC_SAMPS;
    else
        northside = (float)LAC_SAMPS;
    northstep = -1.0;
    southstep = 1.0;
    }
else /* ASCENDING */
    {
    northlat = lat2;
    southlat = lat1;
    northline = (float)*line2;
    southline = (float)*line1;
    southside = 1.0;
    if (model->dtype == GAC)
        northside = (float)GAC_SAMPS;
    else
        northside = (float)LAC_SAMPS;
    northstep = 1.0;
    southstep = -1.0;
    }

/*
  Find the latitude at the northern line
*/
c_ls2ll(&northline,&northside,&elev,model,&thislat,&thislong,NULL);

/*
  Refine number to exactly match northern lat of interest
  if we get to the maximum image line go no further
*/
if (passdir == DESCENDING)
    {
    while ((thislat > northlat) && (northline >= maxline))
        {
        /* move south */
        northline += southstep;
	c_ls2ll(&northline,&northside,&elev,model,&thislat,&thislong,NULL);
        }
    while ((thislat < northlat) && (northline >= maxline))
        {
        /* move north */
        northline += northstep;
	c_ls2ll(&northline,&northside,&elev,model,&thislat,&thislong,NULL);
        }
    }
else
    {
    while ((thislat > northlat) && (northline <= maxline))
        {
        /* move south */
        northline += southstep;
	c_ls2ll(&northline,&northside,&elev,model,&thislat,&thislong,NULL);
        }
    while ((thislat < northlat) && (northline <= maxline))
        {
        /* move north */
        northline += northstep;
	c_ls2ll(&northline,&northside,&elev,model,&thislat,&thislong,NULL);
        }
    }

/*
  Find the latitude at the southern line
*/
c_ls2ll(&southline,&southside,&elev,model,&thislat,&thislong,NULL);

/*
  Refine line number to exactly match southern lat of interest
  if we get to the minimum image line go no further
*/
if (passdir == DESCENDING)
    {
    while ((thislat < southlat) && (southline <= minline))
        {
        /* move north */
        southline += northstep;
	c_ls2ll(&southline,&southside,&elev,model,&thislat,&thislong,NULL);
        }
    while ((thislat > southlat) && (southline <= minline))
        {
        /* move south */
        southline += southstep;
	c_ls2ll(&southline,&southside,&elev,model,&thislat,&thislong,NULL);
        }
    }
else
    {
    while ((thislat < southlat) && (southline >= minline))
        {
        /* move north */
        southline += northstep;
	c_ls2ll(&southline,&southside,&elev,model,&thislat,&thislong,NULL);
        }
    while ((thislat > southlat) && (southline >= minline))
        {
        /* move south */
        southline += southstep;
	c_ls2ll(&southline,&southside,&elev,model,&thislat,&thislong,NULL);
        }
    }

if (passdir == DESCENDING)
    {
    *line1 = (long)northline;
    *line2 = (long)southline;
    }
else
    {
    *line1 = (long)southline;
    *line2 = (long)northline;
    }

return;
}
 
