/*******************************************************************************
FUNCTION:	bufread

PURPOSE:	Given an archive descriptor record, fill the buffer with the
		next set of image lines.

		Return E_SUCC if the next buffer was successfully read, E_EOF
		if all of the windowed image has already been read, and E_FAIL
		for all errors.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#ifdef sun
#include <sys/types.h>
#include <unistd.h>
#endif

#include "las.h"
#include "tapeio.h"
#include "adaps.h"
#include "archio.h"



int bufread
(
    struct ARCDESC *arc     /* IN - Archive descriptor record                 */
)
{
char str[CMLEN];            /* Temporary string                               */

long lines_per_rec;         /* # of archive image lines in a tape record      */
long mode     = HANDLE;     /* Error handling mode for tape I/O routines      */
long nbytes;                /* Number of bytes to read/seek in archive images */
long nlines;                /* Number of lines to read from the archive image */
long nonfatal = 1;          /* Nonfatal error flag                            */
long status;                /* Return status from a function call             */

short done;                 /* Loop control                                   */

unsigned char *pck_ptr;     /* Ptr to walk through the packed archive buffer  */


/* Check the access of the archive image
----------------------------------------*/
if ((arc->access != IREAD) && (arc->access != IUPDATE))
    {
    strcpy(str, "Archive image not opened for read or update access");
    c_errmsg(str, "bufread-access", &nonfatal);
    return(E_FAIL);
    }


/* Determine the number of lines that are to be read from the archive image.
   If all of the lines have been read from the window then return E_EOF.
----------------------------------------------------------------------------*/
nlines = arc->window[SL] + arc->window[NL] - arc->current;
if (nlines > ARCBUFLINES)
    nlines = ARCBUFLINES;
if (nlines <= 0)
    return(E_EOF);


/* Read the next buffer of image data from disk or tape
-------------------------------------------------------*/
if (arc->disk)
    {
    /* Seek to the appropriate position in the archive file before reading
    ----------------------------------------------------------------------*/
    if (arc->ascending && !arc->satview)
	{
	nbytes = (arc->current + nlines - 1) * arc->arcbytes * -1;
	status = (long)lseek(arc->fdesc, nbytes, SEEK_END);
	if (status < 0)
	    {
	    strcpy(str, "Error seeking in the archive image");
	    c_errmsg(str, "bufread-seek", &nonfatal);
	    return(E_FAIL);
	    }
	}
    else
	{
	nbytes = (arc->current - 1) * arc->arcbytes;
	status = (long)lseek(arc->fdesc, nbytes, SEEK_SET);
	if (status != nbytes)
	    {
	    strcpy(str, "Error seeking in the archive image");
	    c_errmsg(str, "bufread-seek", &nonfatal);
	    return(E_FAIL);
	    }
	}


    /* Read nlines into the packed buffer.  Read all the lines at once.
    -------------------------------------------------------------------*/
    nbytes = nlines * arc->arcbytes;
    status = read(arc->fdesc, arc->packed, nbytes);
    if (status != nbytes)
	{
	strcpy(str, "Error reading the archive image");
	c_errmsg(str, "bufread-read", &nonfatal);
	return(E_FAIL);
	}


    /* Reset the first and last line currently held in the buffer
    -------------------------------------------------------------*/
    arc->first = arc->current;
    arc->last  = arc->first + nlines - 1;
    }
else
    {
    /* Read through the image until the current line is in the buffer.  Because
       the tape records will contain multiple image lines the current line may
       not be the first line within the buffer when this loop is finished.
    --------------------------------------------------------------------------*/
    nbytes = ARCH_BLK * arc->arcbytes;
    done = FALSE;
    while (!done)
	{
	/* c_tpread will return one of four status' or the number of bytes
	   read.  If an EOV, EOT, or EOF is encountered before we find a valid
	   image line then we should return E_EOF to indicate the end of the
	   archive image.  Every record read from the tape must be a multiple
	   of the number of bytes in an archive line.
	----------------------------------------------------------------------*/
	status = c_tpread(&arc->tpunit, arc->packed, &nbytes, &mode);
	if ((status == TAPE_EOV) || (status == TAPE_EOT) || (status==TAPE_EOF))
	    return(E_EOF);
	else if (status == TPREAD_READ)
	    return(E_FAIL);
	else if (status % arc->arcbytes)
	    {
	    strcpy(str,"Tape record size not a multiple of the archive record");
	    c_errmsg(str, "bufread-record", &nonfatal);
	    return(E_FAIL);
	    }


	/* Remember how many lines were contained in this tape record, update
	   the first and last line numbers stored in the buffer, and determine
	   if we are finished with the loop.
	----------------------------------------------------------------------*/
	lines_per_rec = status / arc->arcbytes;
	arc->first = arc->last + 1;
	arc->last = arc->first + lines_per_rec - 1;
	done = (arc->last >= arc->current);
	}


    /* Fill the remaining portion of the buffer with the archive image.  Since
       each record from tape will contain multiple lines it is assumed that the
       next tape record will contain the same number of lines as the last
       record.  The one exception is the last record of the image which may
       contain fewer lines.
    --------------------------------------------------------------------------*/
    pck_ptr = arc->packed;
    nlines = lines_per_rec; /* Number of lines currently in the buffer */
    while ((nlines + lines_per_rec) <= ARCBUFLINES)
	{
	/* Keep track of where the next lines should be placed in the buffer
	--------------------------------------------------------------------*/
	pck_ptr += status;

	/* Read the next tape record.  At this point we have successfully read
	   some lines into the buffer so if we encounter EOV, EOT, or EOF we
	   must continue with the rest of the function.
	----------------------------------------------------------------------*/
	status = c_tpread(&arc->tpunit, pck_ptr, &nbytes, &mode);
	if ((status == TAPE_EOV) || (status == TAPE_EOT) || (status==TAPE_EOF))
	    break;
	else if (status == TPREAD_READ)
	    return(E_FAIL);
	else if (status % arc->arcbytes)
	    {
	    strcpy(str,"Tape record size not a multiple of the archive record");
	    c_errmsg(str, "bufread-record", &nonfatal);
	    return(E_FAIL);
	    }

	/* Remember how many lines were contained in this tape record, update
	   the total number of lines read into the buffer so far, and update
	   the last line number held in the buffer
	---------------------------------------------------------------------*/
	lines_per_rec = status / arc->arcbytes;
	nlines += lines_per_rec;
	arc->last += lines_per_rec;
	}
    }

return(E_SUCC);
}
 
