/*******************************************************************************
FUNCTION:	c_arclose

PURPOSE:	Close the specified archive image.  If the image was opened for
		for IUPDATE or IWRITE access then write the image's buffer
		before closing.

		Return E_SUCC on successful completion, E_FAIL otherwise.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    05/93  J.Fenno          Added code for a temporary VMS buffer to avoid
				 byte swapping the packed buffer.
  1.2    07/96  T.Baltzer        Added call to c_dkfree() for DATAPATH
				 capability.
  2.0    03/02  D.Lloyd          Removed vms, no longer supported.  Changed
                                 vmsbuf to swpbuf for endian support.  CR 5541

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <sys/file.h>

#include "las.h"
#include "tapeio.h"
#include "diskio.h"
#include "adaps.h"
#include "archio.h"
#include "local_protos.h"


int c_arclose
(
    char *image,	    /* IN  - Host file name or scene ID of image      */
    long *arcdesc           /* IN - Archive descriptor record                 */
)
{
long eofs     = 0;          /* Number of EOF marks to write to tape           */
long mode     = HANDLE;     /* Error handling mode for tape I/O routines      */
long nonfatal = 1;          /* Nonfatal error flag                            */
long rewind   = REWOFFL;    /* Flag for unloading the tape drive unit         */
long status   = E_SUCC;     /* The return status for this function            */

struct ARCDESC *arc;        /* Archive descriptor record                      */



/* Ensure arcdesc is non-null and convert it into an address
------------------------------------------------------------*/
if (*arcdesc == 0)
    {
    c_errmsg("Invalid archive descriptor", "arclose-desc", &nonfatal);
    return(E_FAIL);
    }
arc = (struct ARCDESC *)*arcdesc;


/* If the current line needs to be packed back into the buffer then do so
-------------------------------------------------------------------------*/
if (arc->saveline)
    packline(arc);


/* If the buffer needs to be saved then write it to the image file
------------------------------------------------------------------*/
if (arc->savebuf)
    {
    status = bufwrite(arc);
    if (status != E_SUCC)
	status = E_FAIL;
    }


/* Close the disk file or release the tape drive
------------------------------------------------*/
if (arc->disk)
    {
    if (close(arc->fdesc) < 0)
	{
	c_errmsg("Error closing archive image", "arclose-close", &nonfatal);
	status = E_FAIL;
	}
    }
else
    {
    if (c_tpclos(&arc->tpunit, &mode, &rewind, &eofs) != TAPE_SUCC)
	{
	c_errmsg("Error closing archive image", "arclose-close", &nonfatal);
	status = E_FAIL;
	}
    }


/* Free reserved space on a given pack, if opened for write access.
---------------------------------------------------------------------*/
if (arc->access == IWRITE)
    c_dkfree(image);


/* Free all of the memory for this archive descriptor record
------------------------------------------------------------*/
if (arc->mnr     != NULL) free(arc->mnr);
if (arc->samples != NULL) free(arc->samples);
if (arc->packed  != NULL) free(arc->packed);
if (arc->swpbuf  != NULL) free(arc->swpbuf);
free(arc);


/* Zero out the user's address to the descriptor record
-------------------------------------------------------*/
*arcdesc = 0;


/* Return the current status
----------------------------*/
return(status);
}
 
