/*******************************************************************************
FUNCTION:	c_arcopnrd

PURPOSE:	Open an AVHRR archive image for reading or updating.  If tpunit
		is NULL, image is taken as the host file name of the archive
		image to be opened.  If it is non-null, image will be used as 
		the scene ID of the image to be opened.  The tape must be
		positioned at the beginning of the image data file.  This may
		be accomplished by calling c_tpgetahdr().  Images on tape can
		only be opened for reading.

		The sat_view flag within the model structure indicates the
		order in which ascending passes are processed.  If sat_view is
		nonzero, the lines of ascending passes will be processed in the
		same order as they were acquired by the satellite.  If sat_view
		is zero, the lines of ascending passes will be processed in a
		"North-up" order and the pixels will be flipped before they are
		returned.  If a window of (1 1 10 2048) is specified then the
		top 10 lines of the archive image will be returned if sat_view
		is nonzero and the bottom 10 lines of the archive image will be
		returned if sat_view is zero.  For descending passes satellite
		view and "North-up" are the same and therefore, the sat_view
		flag has no effect.

		If window[SL] or window[SS] are less than 1, the entire archive
		image will be opened.  If window[NL] is less than 1 the
		remaining lines from the starting line will be included in the
		window.  Likewise, if window[NS] is less than 1 the remaining
		samples from the starting sample will be included in the
		window.  If the window is set or modified by this function the
		new window will be returned.  When the window is reported to
		the user it will always be given in a "North-up" order.

		A status of E_SUCC will be returned upon successful completion,
		otherwise E_FAIL.  If the image is being read from tape and an
                error is encountered, the tape will be dismounted before E_FAIL
                is returned.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    05/93  J.Fenno          Added code for a temporary VMS buffer to avoid
				 byte swapping the packed buffer.
  2.0    03/02  D.Lloyd          Removed vms code, no longer supported.  Changed
                                 vmsbuf to swpbuf for endian support.  CR 5541

PROJECT:	ADAPS
*******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/file.h>
#include <string.h>

#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "labserv.h"
#include "adaps.h"
#include "archio.h"
#include "local_protos.h"


int c_arcopnrd
(
    char *image,            /* IN     - Host file name or scene ID of image   */
    long *access,           /* IN     - IREAD or IUPDATE                      */
    long window[4],         /* IN/OUT - Window for opening the image          */
    struct MODEL *model,    /* IN     - AVHRR model structure                 */
    long *tpunit,           /* IN     - Tape unit.  NULL if image on disk     */
    long *arcdesc           /* OUT    - Archive descriptor record             */
)
{
char dir[CMLEN];            /* Directory portion of the archive image name    */
char ext[CMLEN];            /* Extension portion of the archive image name    */
char root[CMLEN];           /* Root portion of the archive image name         */
char str[CMLEN];            /* Temporary string                               */

long eofs     = 0;          /* Number of EOF marks to write to tape           */
long exist    = 0;          /* File test (0=exist 1=execute 2=write 3=read)   */
long flags;                 /* Access flags for calling open()                */
long mode     = HANDLE;     /* Error handling mode for tape I/O routines      */
long nonfatal = 1;          /* Nonfatal error flag                            */
long rewind   = REWOFFL;    /* Flag for unloading the tape drive unit         */

struct ARCDESC *arc;        /* Archive descriptor record                      */


/* Ensure the requested access is valid
---------------------------------------*/
if ((*access != IREAD) && (*access != IUPDATE))
    {
    c_errmsg("Invalid access mode", "arcopnrd-mode", &nonfatal);
    if (*tpunit != 0) c_tpclos(tpunit, &mode, &rewind, &eofs);
    return(E_FAIL);
    }


/* Ensure images on tape are only opened with IREAD access
----------------------------------------------------------*/
if ((*tpunit != 0) && (*access != IREAD))
    {
    strcpy(str, "Archive images can only be read from tape");
    c_errmsg(str, "arcopnrd-tape", &nonfatal);
    if (*tpunit != 0) c_tpclos(tpunit, &mode, &rewind, &eofs);
    return(E_FAIL);
    }


/* Ensure sat_view is TRUE for access to ascending images on tape
-----------------------------------------------------------------*/
if ((*tpunit != 0) && (model->cpass == ASCENDING) && !model->sat_view)
    {
    strcpy(str, "Ascending passes on tape must be processed in satellite view");
    c_errmsg(str, "arcopnrd-satview", &nonfatal);
    if (*tpunit != 0) c_tpclos(tpunit, &mode, &rewind, &eofs);
    return(E_FAIL);
    }


/* Allocate a new archive descriptor record and initialize it
-------------------------------------------------------------*/
arc = (struct ARCDESC *)malloc(sizeof(struct ARCDESC));
if (arc == NULL)
    {
    c_errmsg("Error allocating memory", "arcopnrd-alloc", &nonfatal);
    if (*tpunit != 0) c_tpclos(tpunit, &mode, &rewind, &eofs);
    return(E_FAIL);
    }
arc->fdesc     = 0;
arc->access    = *access;
arc->first     = 0;
arc->last      = 0;
arc->tpunit    = *tpunit;
arc->words_mnr = MNR_WORDS;
arc->ascending = (model->cpass == ASCENDING);
arc->disk      = (*tpunit == 0);       /* TRUE or FALSE: is image on disk? */
arc->mnr       = NULL;
arc->samples   = NULL;
arc->satview   = (model->sat_view != 0);  /* TRUE or FALSE: satellite view?   */
arc->savebuf   = FALSE;
arc->saveline  = FALSE;
arc->packed    = NULL;
arc->swpbuf    = NULL;
if (model->dtype == GAC)
    {
    arc->arcbytes  = GAC_BYTES;
    arc->words_img = GAC_WORDS;
    }
else
    {
    arc->arcbytes  = LAC_BYTES;
    arc->words_img = LAC_WORDS;
    }


/* If the archive image is on disk then open it with the specified access
-------------------------------------------------------------------------*/
if (arc->disk)
    {
    /* Ensure the archive image exists
    ----------------------------------*/
    if (c_lsstat(image, &exist) == E_FAIL)
	{
	c_errmsg("Archive image does not exist", "arcopnrd-noexist", &nonfatal);
	free_desc(arc);
	return(E_FAIL);
	}


    /* Open the archive image for the specified access
    --------------------------------------------------*/
    if (arc->access == IREAD)
	flags = O_RDONLY;
    else
	flags = O_RDWR | O_CREAT;
    arc->fdesc = open(image, flags, 0664);
    if (arc->fdesc < 0)
	{
	c_errmsg("Error opening archive image", "arcopnrd-open", &nonfatal);
	free_desc(arc);
	return(E_FAIL);
	}
    }


/* Check the window specified by the user
-----------------------------------------*/
if ((window[SL] < 1) || (window[SS] < 1))
    {
    window[SL] = 1;
    window[SS] = 1;
    window[NL] = model->nlines;
    window[NS] = model->nsamples;
    }
else if (window[SL] > model->nlines)
    {
    sprintf(str, "Starting line exceeds number of lines in image");
    c_errmsg(str, "arcopnrd-nlines", &nonfatal);
    free_desc(arc);
    return(E_FAIL);
    }
else if (window[SS] > model->nsamples)
    {
    sprintf(str, "Starting sample exceeds number of samples in image");
    c_errmsg(str, "arcopnrd-nlines", &nonfatal);
    free_desc(arc);
    return(E_FAIL);
    }

if (window[NL] < 1)
    window[NL] = model->nlines - window[SL] + 1;
else if ((window[SL] + window[NL] - 1) > model->nlines)
    {
    window[NL] = model->nlines - window[SL] + 1;
    sprintf(str, "Resetting number of lines to %ld", window[NL]);
    c_errmsg(str, " ", &nonfatal);
    }

if (window[NS] < 1)
    window[NS] = model->nsamples - window[SS] + 1;
else if ((window[SS] + window[NS] - 1) > model->nsamples)
    {
    window[NS] = model->nsamples - window[SS] + 1;
    sprintf(str, "Resetting number of samples to %ld", window[NS]);
    c_errmsg(str, " ", &nonfatal);
    }


/* Report the window to the user.  For ascending passes that are being
   processed in satellite view we must report the window to the user
   as if it is being processed "North-up"
----------------------------------------------------------------------*/
c_splitname(image, dir, root, ext);
if (strlen(ext) > 0)
    {
    sprintf(dir, "%s.%s", root, ext);
    strcpy(root, dir);
    }
if (arc->ascending && arc->satview)
    sprintf(str, "%s windowed at (%ld %ld %ld %ld)", root,
	(model->nlines - (window[SL] + window[NL] - 1) + 1),
	(model->nsamples - (window[SS] + window[NS] - 1) + 1),
	window[NL], window[NS]);
else
    sprintf(str, "%s windowed at (%ld %ld %ld %ld)", root,
	window[SL], window[SS], window[NL], window[NS]);
c_errmsg(str, " ", &nonfatal);


/* Store the window in the archive descriptor record
----------------------------------------------------*/
arc->window[SL] = window[SL];
arc->window[SS] = window[SS];
arc->window[NL] = window[NL];
arc->window[NS] = window[NS];


/* Allocate the buffers needed for reading and updating an archive image
------------------------------------------------------------------------*/
arc->mnr     = (short *)malloc(arc->words_mnr * sizeof(short));
arc->samples = (short *)malloc(arc->words_img * sizeof(short));
arc->packed  = (unsigned char *)malloc(ARCBUFLINES * arc->arcbytes);
if ((arc->mnr == NULL) || (arc->samples == NULL) || (arc->packed == NULL))
    {
    c_errmsg("Error allocating memory", "arcopnrd-alloc", &nonfatal);
    free_desc(arc);
    return(E_FAIL);
    }
arc->swpbuf = (short *)malloc(((arc->words_img > arc->words_mnr) ? 
                                arc->words_img * sizeof(short) :
			        arc->words_mnr * sizeof(short)));
if ( arc->swpbuf == NULL )
    {
    c_errmsg("Error allocating memory", "arcopnrd-alloc", &nonfatal);
    free_desc(arc);
    return(E_FAIL);
    }

/* Load the buffer for the first time.  When finished, reset arc->current to
   just before the user's window.  Issue a message indicating that we are
   forward skipping the tape to the correct position.
----------------------------------------------------------------------------*/
if (!arc->disk)
   c_errmsg("Positioning tape to the specified window", " ", &nonfatal);
arc->current = arc->window[SL];
if (bufread(arc) != E_SUCC)
    {
    free_desc(arc);
    return(E_FAIL);
    }
arc->current = arc->window[SL] - 1;


/* Send back the address of the archive descriptor
   record, hiding the structure from the user.
--------------------------------------------------*/
*arcdesc = (long)arc;
return(E_SUCC);
}



/*******************************************************************************
FUNCTION:	free_desc

PURPOSE:	Free all of the allocated memory for an archive image
		descriptor and close the disk file or the tape unit.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    05/93  J.Fenno          Added code for a temporary VMS buffer to avoid
				 byte swapping the packed buffer.
  2.0     03/03 D.Lloyd          Changed vmsbuf to swpbuf for endian support.
                                 Vms no longer supported.  

PROJECT:	ADAPS
*******************************************************************************/
void free_desc
(
    struct ARCDESC *arc     /* Archive descriptor record                      */
)
{
long eofs     = 0;          /* Number of EOF marks to write to tape           */
long mode     = HANDLE;     /* Error handling mode for tape I/O routines      */
long rewind   = REWOFFL;    /* Flag for unloading the tape drive unit         */

if (arc->mnr     != NULL) free(arc->mnr);
if (arc->samples != NULL) free(arc->samples);
if (arc->packed  != NULL) free(arc->packed);
if (arc->swpbuf  != NULL) free(arc->swpbuf);
if (arc->fdesc   != 0   ) close(arc->fdesc);
if (arc->tpunit  != 0   ) c_tpclos(&arc->tpunit, &mode, &rewind, &eofs);
free(arc);
}
 
