/*******************************************************************************
FUNCTION:	c_arcread

PURPOSE:	Read the next line of data from an archive image.  Each line
		will be unpacked into a buffer of I*2 values.  All ascending
		passes being read in a non-satellite view will have their
		pixels flipped before they are returned and before they are
		written back to the image when updating.  The number of pixels
		returned in the buffer will depend upon the window that was
		specified in the call to c_arcopnrd().
		
		To increase processing speed this function uses a buffer to
		read ARCBUFLINES lines at a time.  Each time the function is
		called a new buffer address will be returned for the user to
		access.  Ascending passes will have their lines returned in the
		order specified in the call to c_arcopnrd().

		If an image is opened for updating, each successive call will
		store the previously accessed line back into the buffer and
		return the next line in the image.  When the entire buffer has
		been accessed it will be written back out to the archive image
		and then the buffer will be refilled.  It is imperative that
		c_arclose() be called to write the last buffer back out to the
		image file.

		This function will return E_SUCC upon successful completion,
		E_FAIL if an error is encountered, and E_EOF if every line of
		the window has been read.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    05/93  J.Fenno          Added code for a temporary VMS buffer to avoid
				 byte swapping the packed buffer.
  2.0    03/02  D.Lloyd		 Removed vax code, no longer supported, and
  				 added code for endian swapping CR 5541.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "adapsutil.h"
#include "archio.h"
#include "local_protos.h"
#include "lasendian.h"


int c_arcread
(
    long *arcdesc,      /* IN  - Archive descriptor record                    */
    short **mnr,        /* OUT - Buffer of I*2 minor frame data               */
    short **samples	/* OUT - Buffer of I*2 sample values                  */
)
{
long nonfatal = 1;      /* Nonfatal error flag                                */
long status;            /* Function return status                             */
#ifdef LAS_LITTLE_ENDIAN
/* CR 5541: If this is to be run on a little endian
system the meaning of the swap flag must be reversed.  
-----------------------------------------------------*/
long swap     = TRUE;   /* Flag indicating byte swapping is necessary         */
#else
long swap     = FALSE;  /* Flag indicating no byte swapping is necessary      */
#endif

struct ARCDESC *arc;    /* Archive descriptor record                          */

unsigned char *pck_ptr; /* Ptr into the packed I/O buffer                     */


/* Ensure arcdesc is non-null and convert it into an address
------------------------------------------------------------*/
if (*arcdesc == 0)
    {
    c_errmsg("Invalid archive descriptor", "arcread-desc", &nonfatal);
    return(E_FAIL);
    }
arc = (struct ARCDESC *)*arcdesc;


/* Ensure the access for this archive image is IREAD or IUPDATE
---------------------------------------------------------------*/
if ((arc->access != IREAD) && (arc->access != IUPDATE))
    {
    c_errmsg("Invalid access mode", "arcread-access", &nonfatal);
    return(E_FAIL);
    }


/* If the saveline flag is set, pack the current line
   back into the buffer to save the user's updates.
-----------------------------------------------------*/
if (arc->saveline)
    packline(arc);


/* Update the current line that should be returned to the user
--------------------------------------------------------------*/
arc->current++;


/* If the current line is not within the buffer then read a new buffer
----------------------------------------------------------------------*/
if (arc->current > arc->last)
    {
    /* If the savebuf flag is set then save the buffer before reading
    -----------------------------------------------------------------*/
    if (arc->savebuf)
	{
	status = bufwrite(arc);
	if (status != E_SUCC)
	    return(status);
	}


    /* Read the next set of image lines
    -----------------------------------*/
    status = bufread(arc);
    if (status != E_SUCC)
	return(status);
    }


/* Set up a pointer into the packed archive buffer for the current line
-----------------------------------------------------------------------*/
if (arc->ascending && !arc->satview)
    pck_ptr = arc->packed + (arc->last - arc->current) * arc->arcbytes;
else
    pck_ptr = arc->packed + (arc->current - arc->first) * arc->arcbytes;


/* Unpack the current line into the minor frame and image buffers
-----------------------------------------------------------------*/
c_unpck32(&pck_ptr[0],        arc->mnr,     &arc->words_mnr, &swap);
c_unpck32(&pck_ptr[ARCH_MNR], arc->samples, &arc->words_img, &swap);


/* Swap pixels for ascending passes being processed in a non-satellite view
--------------------------------------------------------------------------*/
if (arc->ascending && !arc->satview)
    c_pixflip(arc->samples, arc->words_img);


/* Return a pointer to the Minor Frame data and one to the appropriate place in
   the image data buffer, taking into account the the window that the image was
   opened with.
------------------------------------------------------------------------------*/
*mnr     = arc->mnr;
*samples = &arc->samples[(arc->window[SS] - 1) * NCHANS];


/* If the image is open for updating then set the savebuf and saveline flags
----------------------------------------------------------------------------*/
if (arc->access == IUPDATE)
    {
    arc->savebuf  = TRUE; /* Packed buffer needs to be written back to image */
    arc->saveline = TRUE; /* Current line needs to be packed into the buffer */
    }

return(E_SUCC);
}
 
