/*******************************************************************************
FUNCTION:	c_arcseek

PURPOSE:	Seek to the correct position in the archive file so that the
		next call to c_arcread() will return the specified line or the
		next call to c_arcwrite() will write the specified line.  For
		images that have been opened with IREAD or IUPDATE access the
		line number must be relative to the window that the image was
		opened with (ie. 1, 2, 3, ...).

		This function will return E_SUCC upon successful completion or
		E_FAIL if an error is encountered.

		NOTE: The AVHRR archive I/O routines buffer data to help
		      increase processing speed.  If seeking is done in such a
		      way that the buffer must be refilled for each line read,
		      processing speed may suffer.
		      
		      Images being read from tape will not be allowed to seek
		      backwards past the first line that is currently held
		      within the buffer.  E_FAIL will be returned in such
		      cases.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    06/93  J.Fenno          Modified to allow seeking with images opened
				 for writing.
  2.0    03/02  D.Lloyd          Removed vms code, no longer supported

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "local_protos.h"


int c_arcseek
(
    long *arcdesc,     /* IN - Archive descriptor record                     */
    long line         /* IN - Next line to be read by c_arcread()           */
)
{
char str[CMLEN];        /* Temporary string                                   */

long nonfatal = 1;      /* Nonfatal error flag                                */
long status;            /* Function return status                             */

struct ARCDESC *arc;    /* Archive descriptor record                          */


/* Ensure arcdesc is non-null and convert it into an address
------------------------------------------------------------*/
if (*arcdesc == 0)
    {
    c_errmsg("Invalid archive descriptor", "arcseek-desc", &nonfatal);
    return(E_FAIL);
    }
arc = (struct ARCDESC *)*arcdesc;


/* Ensure the access for this archive image is IREAD or IUPDATE
---------------------------------------------------------------*/
if ((arc->access == IREAD) || (arc->access == IUPDATE))
    {
    /* Ensure the requested line lies within the window that the image was
       opened with and then update the line number so it is a line number
       within the entire image, not relative to the window.
    ----------------------------------------------------------------------*/
    if ((line < 1) || (line > arc->window[NL]))
	{
	c_errmsg("Invalid line number", "arcseek-line", &nonfatal);
	return(E_FAIL);
	}
    line = line + arc->window[SL] - 1;


    /* If the image is being read from tape and the requested
       line lies before the buffer then return E_FAIL.
    ---------------------------------------------------------*/
    if (!arc->disk && (line < arc->first))
	{
	strcpy(str, "Cannot seek past beginning of buffer");
	c_errmsg(str, "arcseek-tpseek", &nonfatal);
	return(E_FAIL);
	}


    /* If the current line needs to be packed back into the buffer then do so
    -------------------------------------------------------------------------*/
    if (arc->saveline)
	packline(arc);


    /* If the requested line is not within the buffer then read a new buffer
    ------------------------------------------------------------------------*/
    if ((line < arc->first) || (line > arc->last))
	{
	/* If the buffer needs to be saved then write it to the image file
	------------------------------------------------------------------*/
	if (arc->savebuf)
	    {
	    status = bufwrite(arc);
	    if (status != E_SUCC)
		return(E_FAIL);
	    }


	/* Fill the buffer
	------------------*/
	arc->current = line;
	status = bufread(arc);
	if (status != E_SUCC)
	    return(E_FAIL);
	}
    }
else
    {
    /* Ensure the line is valid.  On UNIX systems a seek beyond the end of the
       file will cause the operating system to zero fill the blank space.
    --------------------------------------------------------------------------*/
    if (line < 1)
	{
	c_errmsg("Invalid line number", "arcseek-line", &nonfatal);
	return(E_FAIL);
	}
    

    /* If the buffer currently contains some image lines and the line that is
       being seeked to does not lie within the buffer then write the buffer to
       the archive image.
    --------------------------------------------------------------------------*/
    if (arc->savebuf && ((line < arc->first) || (line > arc->last)))
	{
	status = bufwrite(arc);
	if (status != E_SUCC)
	    return(E_FAIL);
	}


       
    /* If the buffer doesn't need to be saved then it must be empty and we can
       reset arc->first and arc->last.  If the buffer still needs to be saved
       then we must be seeking to a line currently held within the buffer and
       there is no need to update arc->first and arc->last.
    --------------------------------------------------------------------------*/
    if (!arc->savebuf)
	{
	arc->first = line;
	arc->last  = line;
	}
    }


/* Set arc->current so c_arcread() will return the requested
   line or c_arcwrite() will write the requested line.
------------------------------------------------------------*/
arc->current = line - 1;
return(E_SUCC);
}
 
