/*******************************************************************************
FUNCTION:	c_arcwrite

PURPOSE:	Write the next line of an archive image.  Each line of the
		image will be packed before it is written to the disk file.  It
		is assumed that when the first line of an archive image is
		written, we do not know if it is an ascending or descending
		pass and how many lines there will be in the image.  Therefore,
		no pixel flipping is performed and the lines are written in the
		order that they are passed to this function.  The minor frame
		buffer must be large enough to contain MNR_WORDS of I*2 values
		and the buffer of image data must be large enough to contain
		either GAC_WORDS or LAC_WORDS of I*2 values, depending upon the
		data type that was supplied to the c_arcopnwrt() call.

		To increase processing speed this function uses a buffer to
		write ARCBUFLINES lines to the archive image at a time.  When
		the buffer has been filled it will be written to the archive
		image.  It is imperative that c_arclose() be called to write
		the last buffer to the image file.

		This function will return a status of E_SUCC upon successful
		completion and E_FAIL if an error is encountered.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    06/93  J.Fenno          Modified to allow seeking with images opened
				 for writing.
  2.0    03/02  D.Lloyd		 Removed vax code -- no longer supported. Added
  				 code for endian swapping. CR 5541

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "adapsutil.h"
#include "archio.h"
#include "local_protos.h"
#include "lasendian.h"



int c_arcwrite
(
    long *arcdesc,     /* IN  - Archive descriptor record                    */
    short mnr[],        /* OUT - Buffer of I*2 minor frame data               */
    short samples[]    /* OUT - Buffer of I*2 sample values                  */
)
{
char str[CMLEN];        /* Temporary string                                   */

long nonfatal = 1;      /* Nonfatal error flag                                */
long status;            /* Function return status                             */

struct ARCDESC *arc;    /* Archive descriptor record                          */

unsigned char *pck_ptr; /* Ptr into the packed I/O buffer                     */


/* Ensure arcdesc is non-null and convert it into an address
------------------------------------------------------------*/
if (*arcdesc == 0)
    {
    c_errmsg("Invalid archive descriptor", "arcwrite-desc", &nonfatal);
    return(E_FAIL);
    }
arc = (struct ARCDESC *)*arcdesc;


/* Ensure the access for this archive image is IWRITE
-----------------------------------------------------*/
if (arc->access != IWRITE)
    {
    strcpy(str, "Archive image not opened for write access");
    c_errmsg(str, "arcwrite-access", &nonfatal);
    return(E_FAIL);
    }


/* Update the current line that is being written, set a pointer into the I/O
   buffer for this line, pack the minor frame and image data, and set the flag
   indicating that the buffer needs to be saved.
----------------------------------------------------------------------------*/
arc->current++;
pck_ptr = arc->packed + (arc->current - arc->first) * arc->arcbytes;
c_pck32(mnr, &pck_ptr[0], &arc->words_mnr);
c_pck32(samples, &pck_ptr[ARCH_MNR], &arc->words_img);

/* CR 5541: Make sure the buffer is in the proper format for the system
-----------------------------------------------------------------------*/
exportendian_buf( &pck_ptr[0], sizeof(int), arc->arcbytes/4 );
arc->savebuf = TRUE;


/* Update the last line that is stored in the buffer.  Because the user may
   have seeked backwards within the buffer and we want to keep track of every
   line that has been stored in the buffer, only update arc->last if we have
   added a line to the end of the buffer.
-----------------------------------------------------------------------------*/
if (arc->current > arc->last)
    arc->last = arc->current;


/* Keep track of the total number of lines in the image
-------------------------------------------------------*/
if (arc->current > arc->window[NL])
    arc->window[NL] = arc->current;


/* If the buffer is full then write it to the image
---------------------------------------------------*/
if ((arc->last - arc->first + 1) >= ARCBUFLINES)
    {
    status = bufwrite(arc);
    if (status != E_SUCC)
	return(E_FAIL);
    arc->first = arc->last + 1;
    }

return(E_SUCC);
}
 
