/*******************************************************************************
NAME:    C_SETHEAD

PURPOSE: Convert the ASCII header information into a header structure.
		

Version  Date   Author		 Reason
-------  -----	---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Original development
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
                                 Placed the EROS Archive Header in the free text
				 portion of the CEOS_IEF header.
  2.2    09/92  J.Fenno          Added an ending orbit to the header and changed
				 the number of dropped lines from 4 to 5 digits.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "tae.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"

void c_sethead
(
    char buffer[][HDR_SIZ+1],	/* IN  - Buffer containing ASCII header info */
    struct MODEL *model,	/* OUT - Model and orbital information	     */
    long version		/* Version of the header		     */
)
{
char err_str[ERRLEN];      /* String for printing error messages             */
char tmp_str[256];         /* Temp string                                    */

double	otime,		     /* Time offset				     */
	oroll,		     /* Roll offset				     */
	opitch,		     /* Pitch offset				     */
	oyaw,		     /* Yaw offset				     */
	dtime,		     /* Delta time correction			     */
	dalt,		     /* Delta altitude correction		     */
	droll,		     /* Delta roll correction			     */
	dpitch,		     /* Delta pitch correction			     */
	dyaw;		     /* Delta yaw correction			     */

long	i,		     /* Looping variable			     */
	nonfatal = 1;        /* Nonfatal error flag                          */


/* Read the first line of the archive header information
--------------------------------------------------------*/
sscanf(&buffer[0][3],  "%ld",&model->clock_err);
sscanf(&buffer[0][9],  "%ld",&model->satnum);
sscanf(&buffer[0][12], "%s",tmp_str);
if (strcmp(tmp_str, "GAC") == 0)
   model->dtype = GAC;
else if (strcmp(tmp_str, "LAC") == 0)
   model->dtype = LAC;
else if (strcmp(tmp_str, "HRPT") == 0)
   model->dtype = HRPT;
else
   {
   sprintf(err_str, "Unknown data type %s", tmp_str);
   c_errmsg(err_str, "sethead-unknown", &nonfatal);
   model->dtype = HRPT;
   }

sscanf(&buffer[0][17], "%s", tmp_str);
if (strlen(tmp_str) != 3)
   strcpy(model->station, "???");
else
   strcpy(model->station, tmp_str);

if (version == 2)
    {
    /*********************************************************
    DLLOYD Y2K
      year is stored as 4-digit in version 2 ahdr, this
      conversion is okay but had to add = for 2000 comparison
    **********************************************************/
    sscanf(&buffer[0][21], "%ld/%ld/%ld", &model->month,
	   &model->day, &model->year);
    if (model->year >= 2000)
	model->year -= 2000;
    else
	model->year -= 1900;

    sscanf(&buffer[0][32], "%ld", &model->jday);
    sscanf(&buffer[0][36], "%ld:%ld:%ld.%ld", &model->shour, &model->smin,
	   &model->ssec, &model->sfrac);
    sscanf(&buffer[0][49], "%ld:%ld:%ld.%ld", &model->ehour, &model->emin,
	   &model->esec, &model->efrac);
    sscanf(&buffer[0][62], "%ld", &model->sorbit);
    sscanf(&buffer[0][68], "%ld", &model->eorbit);
    }
else
    {
    /*********************************************************
    DLLOYD Y2K
      year is stored as 2-digit year in non-version 2 ahdr
      this is okay
    **********************************************************/
    sscanf(&buffer[0][24], "%ld/%ld/%ld", &model->month,
	   &model->day, &model->year);
    sscanf(&buffer[0][35], "%ld", &model->jday);
    sscanf(&buffer[0][39], "%ld:%ld:%ld.%ld", &model->shour, &model->smin,
	   &model->ssec, &model->sfrac);
    sscanf(&buffer[0][52], "%ld:%ld:%ld.%ld", &model->ehour, &model->emin,
	   &model->esec, &model->efrac);
    sscanf(&buffer[0][65], "%ld", &model->sorbit);
    sscanf(&buffer[0][71], "%ld", &model->eorbit);
    }

/* Read the second line of the archive header information
---------------------------------------------------------*/
sscanf(&buffer[1][3], "%s", tmp_str);
if (strcmp(tmp_str, "ASC") == 0)
   model->spass = ASCENDING;
else if (strcmp(tmp_str, "DESC") == 0)
   model->spass = DESCENDING;
else
   {
   sprintf(err_str, "Unknown starting pass %s", tmp_str);
   c_errmsg(err_str, "sethead-unknown", &nonfatal);
   model->spass = ASCENDING;
   }

sscanf(&buffer[1][8], "%s", tmp_str);
if (strcmp(tmp_str, "ASC") == 0)
   model->cpass = ASCENDING;
else if (strcmp(tmp_str, "DESC") == 0)
   model->cpass = DESCENDING;
else
   {
   sprintf(err_str, "Unknown center pass %s", tmp_str);
   c_errmsg(err_str, "sethead-unknown", &nonfatal);
   model->cpass = ASCENDING;
   }

sscanf(&buffer[1][13], "%s", tmp_str);
if (strcmp(tmp_str, "ASC") == 0)
   model->epass = ASCENDING;
else if (strcmp(tmp_str, "DESC") == 0)
   model->epass = DESCENDING;
else
   {
   sprintf(err_str, "Unknown ending pass %s", tmp_str);
   c_errmsg(err_str, "sethead-unknown", &nonfatal);
   model->epass = ASCENDING;
   }

sscanf(&buffer[1][18], "%ld",  &model->nchans);
sscanf(&buffer[1][20], "%1ld", &model->chans[0]);
sscanf(&buffer[1][21], "%1ld", &model->chans[1]);
sscanf(&buffer[1][22], "%1ld", &model->chans[2]);
sscanf(&buffer[1][23], "%1ld", &model->chans[3]);
sscanf(&buffer[1][24], "%1ld", &model->chans[4]);
sscanf(&buffer[1][26], "%ld",  &model->nlines);
sscanf(&buffer[1][32], "%ld",  &model->nsamples);
sscanf(&buffer[1][37], "%ld",  &model->totmiss);

/* set to 42 vs 43 for the old headers. change back after conversion */
sscanf(&buffer[1][42], "%s", tmp_str);
if (strcmp(tmp_str, "DAY") == 0)
   model->day_night = DAY;
else if (strcmp(tmp_str, "NIGHT") == 0)
   model->day_night = NIGHT;
else
   {
   sprintf(err_str, "Unknown day_night %s", tmp_str);
   c_errmsg(err_str, "sethead-unknown", &nonfatal);
   model->day_night = DAY;
   }

/* set to 58 vs 59 for the old headers. change back after conversion */
sscanf(&buffer[1][58], "%lf", &model->sunzen);
model->sunzen *= RAD_PER_DEG;

if (version == 2)
    {
    /* Read the third line of the archive header information
    ----------------------------------------------------------*/
    sscanf(&buffer[2][10], "%lf", &model->nw[0]);
    model->nw[0] *= RAD_PER_DEG;
    sscanf(&buffer[2][24], "%lf", &model->nw[1]);
    model->nw[1] *= RAD_PER_DEG;

    sscanf(&buffer[2][47], "%lf", &model->nn[0]);
    model->nn[0] *= RAD_PER_DEG;
    sscanf(&buffer[2][61], "%lf", &model->nn[1]);
    model->nn[1] *= RAD_PER_DEG;

    /* Read the fourth line of the archive header information
    ---------------------------------------------------------*/
    sscanf(&buffer[3][10], "%lf", &model->ne[0]);
    model->ne[0] *= RAD_PER_DEG;
    sscanf(&buffer[3][24], "%lf", &model->ne[1]);
    model->ne[1] *= RAD_PER_DEG;

    sscanf(&buffer[3][47], "%lf", &model->cw[0]);
    model->cw[0] *= RAD_PER_DEG;
    sscanf(&buffer[3][61], "%lf", &model->cw[1]);
    model->cw[1] *= RAD_PER_DEG;

    /* Read the fifth line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[4][10], "%lf", &model->cn[0]);
    model->cn[0] *= RAD_PER_DEG;
    sscanf(&buffer[4][24], "%lf", &model->cn[1]);
    model->cn[1] *= RAD_PER_DEG;

    sscanf(&buffer[4][47], "%lf", &model->ce[0]);
    model->ce[0] *= RAD_PER_DEG;
    sscanf(&buffer[4][61], "%lf", &model->ce[1]);
    model->ce[1] *= RAD_PER_DEG;

    /* Read the sixth line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[5][10], "%lf", &model->sw[0]);
    model->sw[0] *= RAD_PER_DEG;
    sscanf(&buffer[5][24], "%lf", &model->sw[1]);
    model->sw[1] *= RAD_PER_DEG;

    sscanf(&buffer[5][47], "%lf", &model->sn[0]);
    model->sn[0] *= RAD_PER_DEG;
    sscanf(&buffer[5][61], "%lf", &model->sn[1]);
    model->sn[1] *= RAD_PER_DEG;

    /* Read the seventh line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[6][10], "%lf", &model->se[0]);
    model->se[0] *= RAD_PER_DEG;
    sscanf(&buffer[6][24], "%lf", &model->se[1]);
    model->se[1] *= RAD_PER_DEG;
    
    sscanf(&buffer[6][47], "%lf", &model->eqcrs);
    model->eqcrs *= RAD_PER_DEG;
    
    sscanf(&buffer[6][69],  "%ld",&model->sat_view);
    }
else
    {
    /* Read the third line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[2][10], "%lf", &oroll);
    oroll *= RAD_PER_DEG;

    sscanf(&buffer[2][34], "%lf", &opitch);
    opitch *= RAD_PER_DEG;

    sscanf(&buffer[2][58], "%lf", &oyaw);
    oyaw *= RAD_PER_DEG;


    /* Read the fourth line of the archive header information
    ---------------------------------------------------------*/
    sscanf(&buffer[3][10], "%lf", &otime);

    sscanf(&buffer[3][34], "%lf", &droll);
    droll *= RAD_PER_DEG;

    sscanf(&buffer[3][58], "%lf", &dpitch);
    dpitch *= RAD_PER_DEG;


    /* Read the fifth line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[4][10], "%lf", &dyaw);
    dyaw *= RAD_PER_DEG;

    sscanf(&buffer[4][34], "%lf", &dalt);
    sscanf(&buffer[4][58], "%lf", &dtime);

    model->delta_time     = otime + dtime;
    model->roll_coefs[0]  = oroll + droll;
    model->pitch_coefs[0] = opitch + dpitch;
    model->yaw_coefs[0]   = oyaw + dyaw;
    model->delta_alt      = dalt;

    for (i = 1; i < 5; i++)
	{
	model->roll_coefs[i]  = 0;
	model->pitch_coefs[i] = 0;
	model->yaw_coefs[i]   = 0;
	model->alt_coefs[i]   = 0;
	}

    /* Read the sixth line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[5][10], "%lf", &model->eqcrs);
    model->eqcrs *= RAD_PER_DEG;

    sscanf(&buffer[5][47], "%lf", &model->nw[0]);
    model->nw[0] *= RAD_PER_DEG;
    sscanf(&buffer[5][60], "%lf", &model->nw[1]);
    model->nw[1] *= RAD_PER_DEG;


    /* Read the seventh line of the archive header information
    ----------------------------------------------------------*/
    sscanf(&buffer[6][10], "%lf", &model->nn[0]);
    model->nn[0] *= RAD_PER_DEG;
    sscanf(&buffer[6][23], "%lf", &model->nn[1]);
    model->nn[1] *= RAD_PER_DEG;

    sscanf(&buffer[6][47], "%lf", &model->ne[0]);
    model->ne[0] *= RAD_PER_DEG;
    sscanf(&buffer[6][60], "%lf", &model->ne[1]);
    model->ne[1] *= RAD_PER_DEG;


    /* Read the eighth line of the archive header information
    ---------------------------------------------------------*/
    sscanf(&buffer[7][10], "%lf", &model->cw[0]);
    model->cw[0] *= RAD_PER_DEG;
    sscanf(&buffer[7][23], "%lf", &model->cw[1]);
    model->cw[1] *= RAD_PER_DEG;

    sscanf(&buffer[7][47], "%lf", &model->cn[0]);
    model->cn[0] *= RAD_PER_DEG;
    sscanf(&buffer[7][60], "%lf", &model->cn[1]);
    model->cn[1] *= RAD_PER_DEG;

    /* Read the ninth line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[8][10], "%lf", &model->ce[0]);
    model->ce[0] *= RAD_PER_DEG;
    sscanf(&buffer[8][23], "%lf", &model->ce[1]);
    model->ce[1] *= RAD_PER_DEG;

    sscanf(&buffer[8][47], "%lf", &model->sw[0]);
    model->sw[0] *= RAD_PER_DEG;
    sscanf(&buffer[8][60], "%lf", &model->sw[1]);
    model->sw[1] *= RAD_PER_DEG;


    /* Read the tenth line of the archive header information
    --------------------------------------------------------*/
    sscanf(&buffer[9][10], "%lf", &model->sn[0]);
    model->sn[0] *= RAD_PER_DEG;
    sscanf(&buffer[9][23], "%lf", &model->sn[1]);
    model->sn[1] *= RAD_PER_DEG;

    sscanf(&buffer[9][47], "%lf", &model->se[0]);
    model->se[0] *= RAD_PER_DEG;
    sscanf(&buffer[9][60], "%lf", &model->se[1]);
    model->se[1] *= RAD_PER_DEG;
    }

return;
}
 
