/*******************************************************************************
NAME:		C_TOMETA

PURPOSE:	Convert the header structure to the CEOS_IEF meta data format
		
PROGRAM HISTORY:

Version  Date   Author           Reason
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Original development
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
                                 Placed the EROS Archive Header in the free text
				 portion of the CEOS_IEF header.
  2.2    09/92  J.Fenno          Changed the staring orbit variable name from
                                 orbit to sorbit in the header structure.

*******************************************************************************/
#include <stdio.h>
#ifdef sun
#include <fcntl.h>
#endif
#include <string.h>
#include "worgen.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "archio.h"
#include "local_protos.h"

#define FALSE       0
#define TRUE        1


void c_tometa
(
struct MODEL *model,                       /* IN  - Model and orbital info   */
char   meta_data[META_LINES][HDR_SIZ + 1]  /* OUT - Buffer for CEOS IEF      */
)
{
char tmp_str[HDR_SIZ + 1];

long i;


/* MISSION_ID:  unique code for NOAA series and NIMBUS-7 satellites
-------------------------------------------------------------------*/
if ((model->satnum < 0) || (model->satnum > 99))
   sprintf(meta_data[0], "N??");
else
   sprintf(meta_data[0], "N%02ld", model->satnum);

/* SENSOR_ID:  sensor code (AVH, TOV, AVT, CZC, SWF, xxx)
---------------------------------------------------------*/
strcat(meta_data[0], "AVH");

/* DATA_MODE:  data collection mode (LAC, GAC, HRPT)
----------------------------------------------------*/
switch (model->dtype)
   {
   case LAC:
      strcat(meta_data[0], "LAC ");
      break;
   case GAC:
      strcat(meta_data[0], "GAC ");
      break;
   case HRPT:
      strcat(meta_data[0], "HRP ");
      break;
   default:
      strcat(meta_data[0], "--- ");
   }

/* ACQ_DATE:  date at start of acquisition
------------------------------------------*/
sprintf(tmp_str, "%02ld%02ld%02ld ", model->year%100, model->month%100,
	model->day%100);
strcat(meta_data[0], tmp_str);

/* START_TIME:  GMT at start of acquisition
-------------------------------------------*/
sprintf(tmp_str, "%02ld%02ld%02ld ", model->shour%100, model->smin%100,
	model->ssec%100);
strcat(meta_data[0], tmp_str);

/* END_TIME:  GMT at end of acquisition
---------------------------------------*/
sprintf(tmp_str, "%02ld%02ld%02ld ", model->ehour%100, model->emin%100,
	model->esec%100);
strcat(meta_data[0], tmp_str);

/* ACQ_ST_ID:  unique code for receiving station
------------------------------------------------*/
sscanf(model->station, "%3s", tmp_str); /* skip any leading spaces */
strcat(meta_data[0], tmp_str);

/* ARC_CENTER:  unique code for the data center holding the data
----------------------------------------------------------------*/
strcat(meta_data[0], "SFL");

/* ORBIT:  orbit number of the satellite at the start of acquisition
--------------------------------------------------------------------*/
sprintf(tmp_str, "%05ld", model->sorbit % 100000);
strcat(meta_data[0], tmp_str);

/* ASC_DSC:  direction of travel at start of acquistion
-------------------------------------------------------*/
if (model->spass == DESCENDING)
   strcat(meta_data[0], "D ");
else if (model->spass == ASCENDING)
   strcat(meta_data[0], "A ");
else
   strcat(meta_data[0], "? ");

/* EQCRX_TIME:  GMT at the equatorial crossing
----------------------------------------------*/
strcat(meta_data[0], "??????");

/* EQCRX_LON:  longitude of the satellite nadir at the equatorial crossing
--------------------------------------------------------------------------*/
format_dbl_m(model->eqcrs * DEG_PER_RAD, 7, tmp_str);
strcat(meta_data[0], tmp_str);

/* PRO_LEVEL:  processing level of the data in the archive
----------------------------------------------------------*/
strcat(meta_data[0], " 0");

/* BAND_MODE:  the number and type of bands archived
----------------------------------------------------*/
for (i = 0; i < 5; i++)        /* EROS archive headers have at most 5 bands */
   if (model->chans[i] > 0)
      tmp_str[i] = (char)('1' + i);
   else
      tmp_str[i] = '-';
tmp_str[i] = '\0';
strcat(tmp_str, "- ");         /* add a - for the 6th band and a space */
strcat(meta_data[0], tmp_str);

/* PIX_FORMAT:  code for the # of bits stored (10 = 3 words packed in 32 bits)
------------------------------------------------------------------------------*/
strcat(meta_data[0], "10");

/* QL_FLAG:  quick look browse (Y, A, D, N, ?)
----------------------------------------------*/
strcat(meta_data[0], "D ");

/* STORE_ID:  storage media identifier (H, C, O, R, D, X)
---------------------------------------------------------*/
strcat(meta_data[0], "C        ");

/* LINE_NO:  identifies this as the first line of meta data
-----------------------------------------------------------*/
strcat(meta_data[0], "1");


/*##############################################################################
#  In the CEOS_IEF header the corners Top Left (TL) and Top Right (TR) refer   #
#  to the endpoints of the first line as it is scanned by the satellite.       #
#  Likewise, Bottom Left (BL) and Bottom Right (BR) refer to the endpoints of  #
#  the last line that is scanned.  Therefore, if the pass is ascending we      #
#  must invert the corner coordinates that are stored in the EROS Archive      #
#  Header.                                                                     #
##############################################################################*/

/* TL_LAT:  latitude of the top left corner
-------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->se[0] * DEG_PER_RAD, 6, tmp_str);
else
   format_dbl_m(model->nw[0] * DEG_PER_RAD, 6, tmp_str);
strcpy(meta_data[1], tmp_str);

/* TL_LON:  longitude of the top left corner
--------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->se[1] * DEG_PER_RAD, 7, tmp_str);
else
   format_dbl_m(model->nw[1] * DEG_PER_RAD, 7, tmp_str);
strcat(meta_data[1], tmp_str);

/* TR_LAT:  latitude of the top right corner
--------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->sw[0] * DEG_PER_RAD, 6, tmp_str);
else
   format_dbl_m(model->ne[0] * DEG_PER_RAD, 6, tmp_str);
strcat(meta_data[1], tmp_str);

/* TR_LON:  longitude of the top right corner
---------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->sw[1] * DEG_PER_RAD, 7, tmp_str);
else
   format_dbl_m(model->ne[1] * DEG_PER_RAD, 7, tmp_str);
strcat(meta_data[1], tmp_str);

/* BR_LAT:  latitude of the bottom right corner
-----------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->nw[0] * DEG_PER_RAD, 6, tmp_str);
else
   format_dbl_m(model->se[0] * DEG_PER_RAD, 6, tmp_str);
strcat(meta_data[1], tmp_str);

/* BR_LON:  longitude of the bottom right corner
------------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->nw[1] * DEG_PER_RAD, 7, tmp_str);
else
   format_dbl_m(model->se[1] * DEG_PER_RAD, 7, tmp_str);
strcat(meta_data[1], tmp_str);

/* BL_LAT:  latitude of the bottom left corner
----------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->ne[0] * DEG_PER_RAD, 6, tmp_str);
else
   format_dbl_m(model->sw[0] * DEG_PER_RAD, 6, tmp_str);
strcat(meta_data[1], tmp_str);

/* BL_LON:  longitude of the bottom left corner
-----------------------------------------------*/
if (model->cpass == ASCENDING)
   format_dbl_m(model->ne[1] * DEG_PER_RAD, 7, tmp_str);
else
   format_dbl_m(model->sw[1] * DEG_PER_RAD, 7, tmp_str);
strcat(meta_data[1], tmp_str);

/* QC_QL:  quick look available and data quality
------------------------------------------------*/
strcat(meta_data[1], "N? ");

/* QC_CC:  cloud cover
----------------------*/
strcat(meta_data[1], "?????? ");

/* QC_CLASS:  classification of land, water, ice/snow, glitter
--------------------------------------------------------------*/
strcat(meta_data[1], "???????? ");

/* QC_LINES:  number of bad lines 00-99
---------------------------------------*/
if (model->totmiss < 100)
   sprintf(tmp_str, "%02ld", model->totmiss);
else
   sprintf(tmp_str, "99" );
strcat(meta_data[1], tmp_str);

/* fill
-------*/
strcat(meta_data[1], "       ");
}


/*************************************************************************
*  Format a double such that there is a leading sign and 2 digits after  *
*  the decimal point.  The length of str should be width.  Note that     *
*  widths less than 5 will result in a string of '?' of the appropriate  *
*  length.                                                               *
*************************************************************************/
void format_dbl_m
(
   double number,  /* the number to be formatted                      */
   long   width,   /* the width of the entire field                   */
   char   str[]    /* a string representing number formatted as +0.00 */
)
{
char tmp_str[100];
long i;
long negative;


/* Determine if the number is negative
--------------------------------------*/
if (number < 0)
   {
   number   = -number;
   negative = TRUE;
   }
else
   negative = FALSE;

/* Format the number with at least a field width of 4
-----------------------------------------------------*/
sprintf(tmp_str, "%4.2lf", number);

/* Place the sign and any leading zeros in str
----------------------------------------------*/
if (negative)
   strcpy(str, "-");
else
   strcpy(str, "+");
for (i = 0; i < width - (strlen(tmp_str) + 1); i++)
   strcat(str, "0");

/* Add on the string representing the number
--------------------------------------------*/
strcat(str, tmp_str);

/* If the formatted string is not the correct length return an error string
---------------------------------------------------------------------------*/
if (strlen(str) != width)
   {
   for (i = 0; i < width; i++)
      str[i] = '?';
   str[i] = '\0';
   }
}
 
