/*******************************************************************************

FUNCTION:	c_tpgetahdr

PURPOSE:	Mount the required tape, search for the archive header file,
		read the header file, and forward skip the tape to the
		beginning of the archive image file.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    09/93  J.Fenno          Indexing problem when reading the ahdr file
                                 from tape.

PROJECT:	ADAPS
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "adaps.h"
#include "archio.h"


#define MAXHDRRECS (DRP_RECS + CEOS_RECS) /* # of records in an archive header*/


int c_tpgetahdr
(
    char *sceneid,          /* IN  - Scene ID of the archive image to read    */
    char *comment,          /* IN  - Tape mounting comment                    */
    long *tpunit,           /* OUT - Tape descriptor record                   */
    struct MODEL *model,    /* OUT - Model structure                          */
    struct DROPPED *drop    /* OUT - Dropped line structure                   */
)
{
char ahdr[18];              /* Tape name of the archive header file           */
char buffer[HDR_SIZ + 1];   /* Buffer for reading ANSI header files           */
char file[18];              /* Name of the current file on tape               */
char header[MAXHDRRECS][HDR_SIZ+1];/* Buffer for the archive header           */
char str[CMLEN];            /* Temporary string                               */
char tapeid[CMLEN];         /* ID of the tape that the operators mounted      */

long bufsize  = HDR_SIZ;    /* Number of bytes in the buffer                  */
long length;                /* Length of a string                             */
long mode     = HANDLE;     /* Error handling mode for tape routines          */
long nimages;               /* Number of archive images found on the tape     */
long nonfatal = 1;          /* Nonfatal error flag                            */
long rewind   = REWIND;     /* Rewind flag for the tape routines              */
long rewoffl  = REWOFFL;    /* Rewind flag for closing the tape on errors     */
long skip;                  /* Number of tape files/records to skip           */
long status;                /* Function return status                         */
long tpaccess = TREAD;      /* Access flag for the tape routines              */
long zero     = 0;          /* Constant value of zero                         */

short found;                /* Flag indicating the archive image was found    */
short i;                    /* Loop control                                   */
short j;                    /* Loop control                                   */



/* File names are at most 17 characters long on ANSI labeled tapes
------------------------------------------------------------------*/
if (strlen(sceneid) > 17)
    {
    c_errmsg("Scene ID too long", "tpgetahdr-long", &nonfatal);
    return(E_FAIL);
    }


/* Create the tape name of the archive header file
--------------------------------------------------*/
strcpy(ahdr, sceneid);
ahdr[0] = 'h';          /* First character for header files is 'h' */
length = (long)strlen(ahdr);
c_low2up(ahdr, &length);


/* Mount the tape containing the archive image
----------------------------------------------*/
skip = 0;
strcpy(str, comment);
status = c_tpopen(tpunit,&skip,&mode,&rewind,"0",&tpaccess,str,tapeid);
if (status != TAPE_SUCC)
    return(E_FAIL);


/* Issue a message to the user indicating that we are searching for the scene
-----------------------------------------------------------------------------*/
sprintf(str, "Searching for %s on tape %s", sceneid, tapeid);
c_errmsg(str, " ", &nonfatal);


/* Read the ANSI header files looking for the specified archive image.  Here is
   an example of the first two ANSI header files from an archive tape.  The
   first file contains an extra record that identifies the tape.  The remaining
   header files will contain two records.

VOL1035149                           OPS                                       3
HDR1HH11040193115216.03514900010001000000 93091 00000 000000EDCSGS4             
HDR2F0008000080                                   00                            

HDR1IH11040193115216.03514900010002000000 93091 00000 000000EDCSGS4             
HDR2F2759213796                                   00                            
------------------------------------------------------------------------------*/
found   = FALSE;
status  = TAPE_SUCC;
nimages = 0;
while (status == TAPE_SUCC)
    {
    /* Skip the tape volume record in the first ANSI header
    -------------------------------------------------------*/
    nimages++;
    skip = 1;
    if (nimages == 1)
	{
	status = c_tpread(tpunit, (unsigned char *) buffer, &bufsize, &mode);
	if (status != HDR_SIZ)
	    {
	    c_errmsg("Error reading ANSI header", "tpgetahdr-hdr", &nonfatal);
	    c_tpclos(tpunit, &mode, &rewoffl, &zero);
	    return(E_FAIL);
	    }
	}


    /* Read the first file identification record in the ANSI header
    ---------------------------------------------------------------*/
    status = c_tpread(tpunit, (unsigned char *) buffer, &bufsize, &mode);
    if ((status == TAPE_EOV) || (status == TAPE_EOT))
	break;
    else if (status != HDR_SIZ)
	{
	c_errmsg("Error reading ANSI header", "tpgetahdr-hdr", &nonfatal);
	c_tpclos(tpunit, &mode, &rewoffl, &zero);
	return(E_FAIL);
	}


    /* Extract the file name from the buffer
    ----------------------------------------*/
    i = 0;
    j = 4;
    while ((i < 17) && (buffer[j] != ' ') && (buffer[j] != '.'))
	file[i++] = buffer[j++];
    file[i] = '\0';


    /* Skip past the rest of the ANSI header file
    ---------------------------------------------*/
    skip = 1;
    status = c_tpfsf(tpunit, &skip, &mode);
    if (status != TAPE_SUCC)
	{
	c_errmsg("Error reading ANSI header", "tpgetahdr-hdr", &nonfatal);
	c_tpclos(tpunit, &mode, &rewoffl, &zero);
	return(E_FAIL);
	}


    /* If the file name found on the tape matches the archive header file, stop
    --------------------------------------------------------------------------*/
    length = (long)strlen(file);
    c_low2up(file, &length);
    if (strcmp(file, ahdr) == 0)
	{
	found = TRUE;
	break;
	}


    /* Skip the rest of this archive image (the archive header file, ANSI
       trailer file, ANSI header file, archive image, and the ANSI trailer
       file = 5 files).
    ----------------------------------------------------------------------*/
    skip = 5;
    status = c_tpfsf(tpunit, &skip, &mode);
    }


/* If not found then close the tape drive and return
----------------------------------------------------*/
if (!found)
    {
    sprintf(str, "%s not found on tape %s", sceneid, tapeid);
    c_errmsg(str, "tpgetahdr-exist", &nonfatal);
    c_tpclos(tpunit, &mode, &rewoffl, &zero);
    return(E_FAIL);
    }


/* Read the archive header file
-------------------------------*/
i = 0;
while (i < MAXHDRRECS)
    {
    status = c_tpread(tpunit, (unsigned char *)header[i], &bufsize, &mode);
    if (status == TAPE_EOF)
	break;
    else if (status != HDR_SIZ)
	{
	c_errmsg("Error reading CEOS header", "tpgetahdr-hdr", &nonfatal);
	c_tpclos(tpunit, &mode, &rewoffl, &zero);
	return(E_FAIL);
	}
    header[i++][HDR_SIZ] = '\0';
    }


/* Ensure the header was large enough
-------------------------------------*/
if (i < MIN_CEOS_RECS)
    {
    c_errmsg("CEOS header too small", "tpgetahdr-small", &nonfatal);
    c_tpclos(tpunit, &mode, &rewoffl, &zero);
    return(E_FAIL);
    }


/* Send the buffer to c_hdrread() to extract the information
------------------------------------------------------------*/
if (c_hdrread(header, model, drop) != E_SUCC)
    {
    c_tpclos(tpunit, &mode, &rewoffl, &zero);
    return(E_FAIL);
    }


/* Forward skip the tape to image file
--------------------------------------*/
skip = 2;
status = c_tpfsf(tpunit, &skip, &mode);
if (status != E_SUCC)
    {
    c_errmsg("Error finding archive image", "tpgetahdr-img", &nonfatal);
    c_tpclos(tpunit, &mode, &rewoffl, &zero);
    return(E_FAIL);
    }

return(E_SUCC);
}
