/*******************************************************************************
FUNCTION: c_copygrid

PURPOSE:  Copy all of the fields from one grid structure to another.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
#include "las.h"
#include "grid.h"
#include "adapsgeo.h"


int c_copygrid
(
    struct GRID *ingrid,  /* IN  - A grid structure with all fields set       */
    struct GRID *outgrid  /* OUT - A copy of the input grid                   */
)
{

long  i;                  /* Loop control                                     */
long  j;                  /* Loop control                                     */
long  nbytes;             /* # of bytes contained in each element of values   */
long  npts;               /* # of grid points in the grids                    */
long  nonfatal = 1;       /* Nonfatal error flag                              */

unsigned char *inptr;     /* Ptr used to copy the 'values' field of ingrid    */
unsigned char *outptr;    /* Ptr used to copy the 'values' field of ingrid    */


/* Free all of the space associated with outgrid
------------------------------------------------*/
c_freegrid(outgrid);


/* Determine the number of grid points and the number of
   bytes required to store one value for each point.
--------------------------------------------------------*/
npts = ingrid->nrows * ingrid->ncols;
switch (ingrid->dtype)
   {
   case EBYTE:
      nbytes = npts * (long)sizeof(unsigned char);
      break;
   case EWORD:
      nbytes = npts * (long)sizeof(int);
      break;
   case ELONG:
      nbytes = npts * (long)sizeof(long);
      break;
   case EREAL:
      nbytes = npts * (long)sizeof(float);
      break;
   case EDOUBLE:
      nbytes = npts * (long)sizeof(double);
      break;
   case ECHAR:
      nbytes = npts * (long)sizeof(char);
      break;
   default:
      c_errmsg("Invalid data type", "c_copygrid-dtype", &nonfatal);
      c_freegrid(outgrid);
      return(E_FAIL);
      break;
   }


/* Allocate the space for the coeffs field
------------------------------------------*/
if (ingrid->ncoeff == 0)
   outgrid->coeffs = NULL;
else
   {
   outgrid->coeffs = (double *)malloc(2 * ingrid->ncoeff);
   if (outgrid->coeffs == NULL)
      {
      c_errmsg("Error allocating memory", "c_copygrid-alloc", &nonfatal);
      return(E_FAIL);
      }
   }


/* Allocate space for the grid rows and columns
-----------------------------------------------*/
outgrid->out_lines = (long *)malloc(ingrid->nrows * sizeof(long));
outgrid->out_samps = (long *)malloc(ingrid->ncols * sizeof(long));
if ((outgrid->out_lines == NULL) || (outgrid->out_samps == NULL))
   {
   c_freegrid(outgrid);
   c_errmsg("Error allocating memory", "c_copygrid-alloc", &nonfatal);
   return(E_FAIL);
   }


/* Allocate enough space for all of the values associated with each grid point
------------------------------------------------------------------------------*/
for (i = 0; i < MAXGRIDVALS; i++)
   if (i < ingrid->nvalues)
      {
      outgrid->values[i] = (void *)malloc(nbytes);
      if (outgrid->values[i] == NULL)
	 {
	 c_freegrid(outgrid);
	 c_errmsg("Error allocating memory", "c_copygrid-alloc", &nonfatal);
	 return(E_FAIL);
	 }
      }
   else
      outgrid->values[i] = NULL;


/* Initialize every field of the grid
-------------------------------------*/
strcpy(outgrid->gridname, ingrid->gridname);
for (i = 0; i < MAXGRIDVALS; i++)
   if (i < ingrid->nvalues)
      strcpy(outgrid->labels[i], ingrid->labels[i]);
   else
      strcpy(outgrid->labels[i], "");
strcpy(outgrid->model, ingrid->model);
strcpy(outgrid->units, ingrid->units);
for (i = 0; i < 32; i++)
   outgrid->act_grid_err[i] = ingrid->act_grid_err[i];
for (i = 0; i < 2; i++)
   outgrid->ave_fit_err[i]  = ingrid->ave_fit_err[i];
for (i = 0; i < 2; i++)
   outgrid->ave_grid_err[i] = ingrid->ave_grid_err[i];
if (ingrid->coeffs != NULL)
   for (i = 0; i < (2 * ingrid->ncoeff); i++)
      outgrid->coeffs[i] = ingrid->coeffs[i];
for (i = 0; i < 8; i++)
   outgrid->corners[i] = ingrid->corners[i];
outgrid->grid_tol = ingrid->grid_tol;
for (i = 0; i < 2; i++)
   outgrid->max_fit_err[i] = ingrid->max_fit_err[i];
for (i = 0; i < 2; i++)
   outgrid->max_grid_err[i] = ingrid->max_grid_err[i];
for (i = 0; i < 2; i++)
   outgrid->pdist[i] = ingrid->pdist[i];
for (i = 0; i < 15; i++)
   outgrid->projprms[i] = ingrid->projprms[i];
outgrid->rms_fit_err   = ingrid->rms_fit_err;
outgrid->rms_grid_err  = ingrid->rms_grid_err;
outgrid->code          = ingrid->code;
outgrid->datum         = ingrid->datum;
outgrid->dtype         = ingrid->dtype;
outgrid->fiterr_valid  = ingrid->fiterr_valid;
outgrid->griderr_valid = ingrid->griderr_valid;
outgrid->lines         = ingrid->lines;
outgrid->ncoeff        = ingrid->ncoeff;
outgrid->ncols         = ingrid->ncols;
outgrid->nrows         = ingrid->nrows;
outgrid->nvalues       = ingrid->nvalues;
for (i = 0; i < ingrid->nrows; i++)
   outgrid->out_lines[i] = ingrid->out_lines[i];
for (i = 0; i < ingrid->ncols; i++)
   outgrid->out_samps[i] = ingrid->out_samps[i];
outgrid->proj_valid    = ingrid->proj_valid;
outgrid->samples       = ingrid->samples;
outgrid->satnum        = ingrid->satnum;
outgrid->zone          = ingrid->zone;
for (i = 0; i < ingrid->nvalues; i++)
   {
   outptr = (unsigned char *)outgrid->values[i];
   inptr  = (unsigned char *)ingrid->values[i];
   for (j = 0; j < nbytes; j++)
      *(outptr++) = *(inptr++);
   }

return(E_SUCC);
}
