/*******************************************************************************
FUNCTION: c_geo2ls

PURPOSE:  Given the latitude and longitude of a known pixel in an image, the
          size of the pixels within the image (in degrees), and the latitude
	  and longitude of a desired pixel, return the line and sample offset
	  for the desired pixel from the known pixel.
   
	  This function assumes that the image extends eastward around the
	  globe from the longitude of the known pixel and that the image does
	  not extend above the North pole or below the South pole.  Depending
	  upon the latitude and longitude of the known pixel, the line offset
	  may be either positive or negative while the sample offset will
	  always be positive.  Therefore, the known pixel can be visualized as
	  lying somewhere on longitude -135 in the following diagram.


    -135      -90      -45       0       45       90       135      180    -135
   90 Z------------------------------------------------------------------------
      |        |  Y     |        |        |        |        |        |        |
      |        |        |        |        |        |        |        |        |
   45 X------------------------------------------------------------------------
      |        |        |        |        |        |        |        |        |
      |        |        |        |        |        |        |        |        |
    0 -------------------------------------------------------------------------
      |        |        |        |        |        |        |        |        |
      |        |        |        |        |        |        |        |        |
  -45 -------------------------------------------------------------------------
      |        |        |        |        |        |        |        |        |
      |        |        |        |        |        |        |        |        |
  -90 -------------------------------------------------------------------------

	  Assuming X is the known pixel and Y is the desired pixel then this
	  function will return a negative line offset and a positive sample
	  offset.  If Z where the known pixel then a negative line offset would
	  never be returned (ie. no latitude is greater than 90 degrees north).


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/92  Development

PROJECT:  ADAPS
*******************************************************************************/
#include "las.h"


int c_geo2ls
(
    double *ullat,   /* IN - Latitude of a known pixel in the image           */
    double *ullon,   /* IN - Longitude of a known pixel in the image          */
    double *pdistx,  /* IN - Pixel distance in the sample direction (Degrees) */
    double *pdisty,  /* IN - Pixel distance in the line direction   (Degrees) */
    double *pixlat,  /* IN - Latitude of the desired pixel                    */
    double *pixlon,  /* IN - Longitude of the desired pixel                   */
    long *pixline,   /* OUT - Line offset to desired pixel from known pixel   */
    long *pixsamp    /* OUT - Sample offset to desired pixel from known pixel */
)
{
double line_offset; /* Offset in the line direction                           */
double samp_offset; /* Offset in the sample direction                         */


/* Compute the line offset.  Because latitudes decrease as line numbers
   increase, we must subtract the desired pixel's latituded from the known
   pixel's.
--------------------------------------------------------------------------*/
line_offset = (*ullat - *pixlat) / *pdisty;


/* Compute the sample offset.  Because we are assuming that the known pixel is
   on the western edge of the image, if the longitude of the desired pixel is
   less than that of the known pixel we must add 360 degrees.
------------------------------------------------------------------------------*/
if (*pixlon < *ullon)
   samp_offset = (*pixlon + 360.0 - *ullon) / *pdistx;
else
   samp_offset = (*pixlon - *ullon) / *pdistx;

*pixline = (long)line_offset;
*pixsamp = (long)samp_offset;

return(E_SUCC);
}
