/*******************************************************************************
FUNCTION: c_readgrid

PURPOSE:  Read a geometric mapping grid from a file.  See grid.h for an
	  explanation of how a grid structure is stored in a label services
	  file.  If the grid structure being passed to this function already
	  contains a grid then c_freegrid() should be called first to free all
	  its dynamic memory.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/92  Development

PROJECT:  ADAPS
*******************************************************************************/
#include "las.h"
#include "labserv.h"
#include "grid.h"


static void freespace
(
    struct GRID *grid,  /* IN - Pointer to a grid structure                   */
    char *buf1,         /* IN - Pointer to a character buffer                 */
    long *buf2,         /* IN - Pointer to a long buffer                      */
    double *buf3,       /* IN - Pointer to a double buffer                    */
    long alloc          /* IN - Flag indicating an error occurred allocating  */
);


int c_readgrid
(
    char *hostname,     /* IN  - Host file name of the grid file to read      */
    struct GRID *grid   /* OUT - Grid structure to read from the file         */
)
{
char dtype[CMLEN];      /* Data type of the information read from the file    */
char key[CMLEN];        /* Key of the record to be read from the grid file    */
char str[CMLEN];        /* Temporary string                                   */
char tmp_dtype[CMLEN];  /* Temporary copy of the dtype variable               */
char tmp_key[CMLEN];    /* Temporary copy of the key variable                 */

long clen;              /* Number of bytes in the character buffer            */
long close_action;      /* Action to be taken when the grid file is closed    */
long dlen;              /* Number of bytes in one of the data buffers         */
long fdesc;             /* File descriptor for the grid file                  */
long i;                 /* Loop control                                       */
long j;                 /* Loop control                                       */
long ndoubles;          /* Number of double elements in the grid              */
long nlongs;            /* Number of long elements in the grid                */
long nonfatal = 1;      /* Nonfatal error flag                                */
long num;               /* Number of elements in the stored in grid->values   */
long status;            /* Return status of a function call                   */

char   *buf_chr;        /* Character buffer sent to c_lswrit                  */
long   *buf_lng;        /* Long buffer sent to c_lswrit                       */
double *buf_dbl;        /* Double buffer sent to c_lswrit                     */


/* Ensure the file exists
-------------------------*/
i = 0; /* test for existence */
if (c_lsstat(hostname, &i) != E_SUCC)
   {
   c_errmsg("Grid file does not exist", "c_readgrid-exist", &nonfatal);
   return(E_FAIL);
   }


/* Open the grid file
---------------------*/
i = 0; /* open for reading */
if (c_lsopen(&fdesc, hostname, &i) != E_SUCC)
   {
   c_errmsg("Error opening grid file", "c_readgrid-open", &nonfatal);
   return(E_FAIL);
   }


/* Set up the action that should be taken when the grid file is closed.
      0 = Close the file normally
      1 = Delete all records with a key of "DELETED"
      2 = Delete the file
-----------------------------------------------------------------------*/
close_action = 0;


/* Allocate a character buffer for reading from the grid file
-------------------------------------------------------------*/
clen = CMLEN;
buf_chr = (char *)malloc(clen);
if (buf_chr == NULL)
   {
   freespace(grid, buf_chr, buf_lng, buf_dbl, TRUE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Allocate a long buffer for reading from the grid file
--------------------------------------------------------*/
dlen = 2 * (long)sizeof(long);
buf_lng = (long *)malloc(dlen);
if (buf_lng == NULL)
   {
   freespace(grid, buf_chr, buf_lng, buf_dbl, TRUE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Read the first record from the file
--------------------------------------*/
sprintf(key, "RECORD1");
status = c_lsread(&fdesc, key, &clen, &dlen, buf_chr,
   (unsigned char *)buf_lng, dtype);
if ((status != E_SUCC) || strcmp(key, "RECORD1") || strcmp(dtype, "I4"))
   {
   c_errmsg("Error reading grid file", "c_readgrid-read", &nonfatal);
   freespace(grid, buf_chr, buf_lng, buf_dbl, FALSE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Retrieve grid->model and grid->units from the character buffer.
   In addition, store the host file name in grid->gridname.
------------------------------------------------------------------*/
sscanf(&buf_chr[9],  "%s", grid->model);
sscanf(&buf_chr[24], "%s", grid->units);
strcpy(grid->gridname, hostname);


/* Retrieve the number of double and long values
   that are stored in records 1 and 2.
------------------------------------------------*/
ndoubles = buf_lng[0];
nlongs   = buf_lng[1];
free(buf_lng);


/* Allocate a double buffer for reading from the file
-----------------------------------------------------*/
clen = CMLEN;
dlen = ndoubles * (long)sizeof(double);
buf_dbl = (double *)malloc(dlen);
if (buf_dbl == NULL)
   {
   freespace(grid, buf_chr, buf_lng, buf_dbl, TRUE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Read the second record from the file
---------------------------------------*/
sprintf(key, "RECORD2");
status = c_lsread(&fdesc, key, &clen, &dlen, buf_chr,
   (unsigned char *)buf_dbl, dtype);
if ((status != E_SUCC) || strcmp(key, "RECORD2") || strcmp(dtype, "R8"))
   {
   c_errmsg("Error reading grid file", "c_readgrid-read", &nonfatal);
   freespace(grid, buf_chr, buf_lng, buf_dbl, FALSE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Allocate a long buffer for reading from the file
---------------------------------------------------*/
clen = CMLEN;
dlen = nlongs * (long)sizeof(long);
buf_lng = (long *)malloc(dlen);
if (buf_lng == NULL)
   {
   freespace(grid, buf_chr, buf_lng, buf_dbl, TRUE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Read the third record from the file
--------------------------------------*/
sprintf(key, "RECORD3");
status = c_lsread(&fdesc, key, &clen, &dlen, buf_chr,
   (unsigned char *)buf_lng, dtype);
if ((status != E_SUCC) || strcmp(key, "RECORD3") || strcmp(dtype, "I4"))
   {
   c_errmsg("Error reading grid file", "c_readgrid-read", &nonfatal);
   freespace(grid, buf_chr, buf_lng, buf_dbl, FALSE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Retrieve all of the long fields from the buffer.  We need to retrieve these
   values before the double values so that we know how many elements there are
   in grid->coeffs.
------------------------------------------------------------------------------*/
i = 0;
grid->code          = buf_lng[i++];
grid->datum         = buf_lng[i++];
grid->dtype         = buf_lng[i++];
grid->fiterr_valid  = buf_lng[i++];
grid->griderr_valid = buf_lng[i++];
grid->lines         = buf_lng[i++];
grid->ncoeff        = buf_lng[i++];
grid->ncols         = buf_lng[i++];
grid->nrows         = buf_lng[i++];
grid->nvalues       = buf_lng[i++];


/* Allocate enough space for grid->out_lines, grid->out_samps, and grid->coeffs
------------------------------------------------------------------------------*/
grid->out_lines = (long *)malloc(grid->nrows * sizeof(long));
grid->out_samps = (long *)malloc(grid->ncols * sizeof(long));
grid->coeffs    = (double *)malloc(2 * grid->ncoeff * sizeof(double));
if ((grid->out_lines == NULL) ||
    (grid->out_samps == NULL) ||
    (grid->coeffs    == NULL))
   {
   freespace(grid, buf_chr, buf_lng, buf_dbl, TRUE);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Retrieve the remaining long fields from the buffer
-----------------------------------------------------*/
for (j = 0; j < grid->nrows; j++)
   grid->out_lines[j] = buf_lng[i++];
for (j = 0; j < grid->ncols; j++)
   grid->out_samps[j] = buf_lng[i++];
grid->proj_valid    = buf_lng[i++];
grid->samples       = buf_lng[i++];
grid->satnum        = buf_lng[i++];
grid->zone          = buf_lng[i++];
free(buf_lng);


/* Retrieve all of the double fields from the buffer
----------------------------------------------------*/
i = 0;
for (j = 0; j < 32; j++)
   grid->act_grid_err[j] = buf_dbl[i++];
for (j = 0; j < 2; j++)
   grid->ave_fit_err[j] = buf_dbl[i++];
for (j = 0; j < 2; j++)
   grid->ave_grid_err[j] = buf_dbl[i++];
for (j = 0; j < (2 * grid->ncoeff); j++)
   grid->coeffs[j] = buf_dbl[i++];
for (j = 0; j < 8; j++)
   grid->corners[j] = buf_dbl[i++];
grid->grid_tol = buf_dbl[i++];
for (j = 0; j < 2; j++)
   grid->max_fit_err[j] = buf_dbl[i++];
for (j = 0; j < 2; j++)
   grid->max_grid_err[j] = buf_dbl[i++];
for (j = 0; j < 2; j++)
   grid->pdist[j] = buf_dbl[i++];
for (j = 0; j < 15; j++)
   grid->projprms[j] = buf_dbl[i++];
grid->rms_fit_err = buf_dbl[i++];
grid->rms_grid_err = buf_dbl[i++];
free(buf_dbl);
free(buf_chr);


/* Determine the number of elements stored in each grid->values array,
   determine the number of bytes in each array, and set up a string indicating
   the type of data stored in the arrays.
------------------------------------------------------------------------------*/
num = grid->nrows * grid->ncols;
switch (grid->dtype)
   {
   case EBYTE:
      dlen = num * (long)sizeof(unsigned char);
      strcpy(dtype, "B");
      break;
   case EWORD:
      dlen = num * (long)sizeof(int);
      strcpy(dtype, "I4");
      break;
   case ELONG:
      dlen = num * (long)sizeof(long);
      strcpy(dtype, "I4");
      break;
   case EREAL:
      dlen = num * (long)sizeof(float);
      strcpy(dtype, "R4");
      break;
   case EDOUBLE:
      dlen = num * (long)sizeof(double);
      strcpy(dtype, "R8");
      break;
   case ECHAR:
      dlen = num * (long)sizeof(char);
      strcpy(dtype, "B");
      break;
   default:
      strcpy(str, "Unknown data type for grid values");
      c_errmsg(str, "c_readgrid-dtype", &nonfatal);
      freespace(grid, buf_chr, buf_lng, buf_dbl, FALSE);
      c_lsclos(&fdesc, grid->gridname, &close_action);
      return(E_FAIL);
      break;
   }


/* Retrieve all of the grid->labels and grid->values arrays from the file
-------------------------------------------------------------------------*/
for (i = 0; i < grid->nvalues; i++)
   {
   /* Allocate memory for grid->values[i]
   --------------------------------------*/
   grid->values[i] = (void *)malloc(dlen);
   if (grid->values[i] == NULL)
      {
      freespace(grid, buf_chr, buf_lng, buf_dbl, TRUE);
      c_lsclos(&fdesc, grid->gridname, &close_action);
      return(E_FAIL);
      }


   /* Read the (i + 4)'th record from the file.  We must pass a temporary copy
      of the key and dtype variables to c_lsread because it will return the
      values that it finds.  We must compare the values that were sent and
      returned.
   ---------------------------------------------------------------------------*/
   sprintf(key, "RECORD%1d", i + 4);
   strcpy(tmp_key, key);
   strcpy(tmp_dtype, dtype);
   clen = CMLEN;
   status = c_lsread(&fdesc, tmp_key, &clen, &dlen, grid->labels[i],
      (unsigned char *)grid->values[i], tmp_dtype);
   if ((status != E_SUCC) || strcmp(key, tmp_key) || strcmp(dtype, tmp_dtype))
      {
      c_errmsg("Error reading grid file", "c_readgrid-read", &nonfatal);
      freespace(grid, buf_chr, buf_lng, buf_dbl, FALSE);
      c_lsclos(&fdesc, grid->gridname, &close_action);
      return(E_FAIL);
      }
   }


/* Close the grid file normally
-------------------------------*/
if (c_lsclos(&fdesc, grid->gridname, &close_action) != E_SUCC)
   {
   strcpy(str, "Error closing the grid file");
   c_errmsg(str, "c_readgrid-close", &nonfatal);
   }

return(E_SUCC);
}


/*******************************************************************************
FUNCTION: freespace

PURPOSE:  Free all of the space that may have been dynamically allocated.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.1    J.Fenno          02/93  Development

PROJECT:  ADAPS
*******************************************************************************/
static void freespace
(
    struct GRID *grid,  /* IN - Pointer to a grid structure                   */
    char *buf1,         /* IN - Pointer to a character buffer                 */
    long *buf2,         /* IN - Pointer to a long buffer                      */
    double *buf3,       /* IN - Pointer to a double buffer                    */
    long alloc          /* IN - Flag indicating an error occurred allocating  */
)
{
long i;                 /* Loop control                                       */
long nonfatal = 1;      /* Nonfatal error flag                                */

/* Free all memory that may have been allocated.  If it hasn't been
   allocated, free will just return an error that we ignore.
-------------------------------------------------------------------*/
free(grid->coeffs);
free(grid->out_lines);
free(grid->out_samps);
for (i = 0; i < MAXGRIDVALS; i++)
   free(grid->values[i]);
free(buf1);
free(buf2);
free(buf3);

if (alloc)
   c_errmsg("Error allocating memory", "c_readgrid-alloc", &nonfatal);
}
 
