/*******************************************************************************
FUNCTION: c_wrtgrid

PURPOSE:  Write a geometric mapping grid to a file.  See grid.h for an
	  explanation of how a grid structure is stored in a label services
	  file.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/92  Development

PROJECT:  ADAPS
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "grid.h"


int c_wrtgrid
(
    struct GRID *grid   /* IN - Grid structure to write to a file             */
)
{
char dtype[CMLEN];      /* Type of data being writing to the grid file        */
char str[CMLEN];        /* Temporary string                                   */

long clen;              /* Number of bytes in the character buffer            */
long close_action;      /* Action to be taken when the grid file is closed    */
long dlen;              /* Number of bytes in one of the data buffers         */
long fdesc;             /* File descriptor for the grid file                  */
long i;                 /* Loop control                                       */
long j;                 /* Loop control                                       */
long ndoubles;          /* Number of double elements in the grid              */
long nlongs;            /* Number of long elements in the grid                */
long nonfatal = 1;      /* Nonfatal error flag                                */
long num;               /* Number of elements in the stored in grid->values   */
long status;            /* Return status of a function call                   */

char   *buf_chr;        /* Character buffer sent to c_lswrit                  */
long   *buf_lng;        /* Long buffer sent to c_lswrit                       */
double *buf_dbl;        /* Double buffer sent to c_lswrit                     */


/* Ensure the file does not exist
---------------------------------*/
i = 0; /* test for existence */
if (c_lsstat(grid->gridname, &i) == E_SUCC)
   {
   sprintf(str, "Grid file already exists");
   c_errmsg(str, "c_wrtgrid-exist", &nonfatal);
   return(E_FAIL);
   }


/* Open the output geometric mapping grid file
----------------------------------------------*/
i = 1; /* open for writing */
if (c_lsopen(&fdesc, grid->gridname, &i) != E_SUCC)
   {
   sprintf(str, "Error opening grid file");
   c_errmsg(str, "c_wrtgrid-open", &nonfatal);
   return(E_FAIL);
   }


/* Set up the action that should be taken when the grid file is closed.
   Temporarily, we will indicate that the grid file should be deleted when
   closed upon encountering an error.
      0 = Close the file normally
      1 = Delete all records with a key of "DELETED"
      2 = Delete the file
--------------------------------------------------------------------------*/
close_action = 2;


/* Determine the number of double and long elements in the grid structure.
   Note that the values being added are just the dimensions of the double and
   long fields within the data structure.  These values exclude the
   grid->values array.
------------------------------------------------------------------------------*/
ndoubles = 32 + 2 + 2 + (2 * grid->ncoeff) + 8 + 1 + 2 + 2 + 2 + 15 + 1 + 1;
nlongs   = 14 + grid->nrows + grid->ncols;


/* Set up a character buffer containing grid->model and grid->units.  This
   buffer is written at the same time that the number of double and long values
   are written.  We do not store the gridname field.
------------------------------------------------------------------------------*/
buf_chr = (char *)malloc(CMLEN);
if (buf_chr == NULL)
   {
   sprintf(str, "Error allocating memory");
   c_errmsg(str, "c_wrtgrid-alloc", &nonfatal);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }
sprintf(buf_chr, "GEOGRID: %-15.15s%-12.12s", grid->model, grid->units);
clen = (long)strlen(buf_chr);


/* Allocate a long buffer large enough to
   hold the two values, ndoubles and nlongs
-------------------------------------------*/
dlen = 2 * sizeof(long);
buf_lng = (long *)malloc(dlen);
if (buf_lng == NULL)
   {
   sprintf(str, "Error allocating memory");
   c_errmsg(str, "c_wrtgrid-alloc", &nonfatal);
   free(buf_chr);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }
buf_lng[0] = ndoubles;
buf_lng[1] = nlongs;


/* Write the character buffer and the long buffer to the
   grid file.  When finished, deallocate the long buffer.
---------------------------------------------------------*/
status = c_lswrit(&fdesc, "RECORD1", &clen, &dlen, buf_chr,
   (char *)buf_lng, "I4");
free(buf_lng);
if (status != E_SUCC)
   {
   sprintf(str, "Error writing grid file");
   c_errmsg(str, "c_wrtgrid-write", &nonfatal);
   free(buf_chr);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Reset the character buffer for the rest of the records
---------------------------------------------------------*/
sprintf(buf_chr, "GEOGRID: ");
clen = (long)strlen(buf_chr);


/* Allocate enough memory to hold all of the double values in the grid
   structure and store all of these values into the double buffer.
----------------------------------------------------------------------*/
dlen = ndoubles * (long)sizeof(double);
buf_dbl = (double *)malloc(dlen);
if (buf_dbl == NULL)
   {
   sprintf(str, "Error allocating memory");
   c_errmsg(str, "c_wrtgrid-alloc", &nonfatal);
   free(buf_chr);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }
i = 0;
for (j = 0; j < 32; j++)
   buf_dbl[i++] = grid->act_grid_err[j];
for (j = 0; j < 2; j++)
   buf_dbl[i++] = grid->ave_fit_err[j];
for (j = 0; j < 2; j++)
   buf_dbl[i++] = grid->ave_grid_err[j];
for (j = 0; j < (2 * grid->ncoeff); j++)
   buf_dbl[i++] = grid->coeffs[j];
for (j = 0; j < 8; j++)
   buf_dbl[i++] = grid->corners[j];
buf_dbl[i++] = grid->grid_tol;
for (j = 0; j < 2; j++)
   buf_dbl[i++] = grid->max_fit_err[j];
for (j = 0; j < 2; j++)
   buf_dbl[i++] = grid->max_grid_err[j];
for (j = 0; j < 2; j++)
   buf_dbl[i++] = grid->pdist[j];
for (j = 0; j < 15; j++)
   buf_dbl[i++] = grid->projprms[j];
buf_dbl[i++] = grid->rms_fit_err;
buf_dbl[i++] = grid->rms_grid_err;


/* Write the character buffer and the double buffer to the
   grid file.  When finished, deallocate the double buffer.
-----------------------------------------------------------*/
status = c_lswrit(&fdesc, "RECORD2", &clen, &dlen, buf_chr,
   (char *)buf_dbl, "R8");
free(buf_dbl);
if (status != E_SUCC)
   {
   sprintf(str, "Error writing grid file");
   c_errmsg(str, "c_wrtgrid-write", &nonfatal);
   free(buf_chr);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Allocate enough memory to hold all of the long values in the grid
   structure and store all of these values into the long buffer.
--------------------------------------------------------------------*/
dlen = nlongs * (long)sizeof(long);
buf_lng = (long *)malloc(dlen);
if (buf_lng == NULL)
   {
   sprintf(str, "Error allocating memory");
   c_errmsg(str, "c_wrtgrid-alloc", &nonfatal);
   free(buf_chr);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }
i = 0;
buf_lng[i++] = grid->code;
buf_lng[i++] = grid->datum;
buf_lng[i++] = grid->dtype;
buf_lng[i++] = grid->fiterr_valid;
buf_lng[i++] = grid->griderr_valid;
buf_lng[i++] = grid->lines;
buf_lng[i++] = grid->ncoeff;
buf_lng[i++] = grid->ncols;
buf_lng[i++] = grid->nrows;
buf_lng[i++] = grid->nvalues;
for (j = 0; j < grid->nrows; j++)
   buf_lng[i++] = grid->out_lines[j];
for (j = 0; j < grid->ncols; j++)
   buf_lng[i++] = grid->out_samps[j];
buf_lng[i++] = grid->proj_valid;
buf_lng[i++] = grid->samples;
buf_lng[i++] = grid->satnum;
buf_lng[i++] = grid->zone;


/* Write the character buffer and the long buffer to the
   grid file.  When finished, deallocate the long buffer.
---------------------------------------------------------*/
status = c_lswrit(&fdesc, "RECORD3", &clen, &dlen, buf_chr,
   (char *)buf_lng, "I4");
free(buf_chr);
free(buf_lng);
if (status != E_SUCC)
   {
   sprintf(str, "Error writing grid file");
   c_errmsg(str, "c_wrtgrid-write", &nonfatal);
   c_lsclos(&fdesc, grid->gridname, &close_action);
   return(E_FAIL);
   }


/* Determine the number of elements stored in each grid->values array,
   determine the number of bytes in each array, and set up a string indicating
   the type of data stored in the arrays.
------------------------------------------------------------------------------*/
num = grid->nrows * grid->ncols;
switch (grid->dtype)
   {
   case EBYTE:
      dlen = num * (long)sizeof(unsigned char);
      strcpy(dtype, "B");
      break;
   case EWORD:
      dlen = num * (long)sizeof(int);
      strcpy(dtype, "I4");
      break;
   case ELONG:
      dlen = num * (long)sizeof(long);
      strcpy(dtype, "I4");
      break;
   case EREAL:
      dlen = num * (long)sizeof(float);
      strcpy(dtype, "R4");
      break;
   case EDOUBLE:
      dlen = num * (long)sizeof(double);
      strcpy(dtype, "R8");
      break;
   case ECHAR:
      dlen = num * (long)sizeof(char);
      strcpy(dtype, "B");
      break;
   default:
      strcpy(str, "Unknown data type for grid values");
      c_errmsg(str, "c_wrtgrid-dtype", &nonfatal);
      c_lsclos(&fdesc, grid->gridname, &close_action);
      return(E_FAIL);
      break;
   }


/* Write each of the grid->values arrays to the grid file
---------------------------------------------------------*/
for (i = 0; i < grid->nvalues; i++)
   {
   clen = (long)strlen(grid->labels[i]);
   sprintf(str, "RECORD%1d", i + 4);
   status = c_lswrit(&fdesc, str, &clen, &dlen, grid->labels[i],
      (char *)grid->values[i], dtype);
   if (status != E_SUCC)
      {
      sprintf(str, "Error writing grid file");
      c_errmsg(str, "c_wrtgrid-write", &nonfatal);
      c_lsclos(&fdesc, grid->gridname, &close_action);
      return(E_FAIL);
      }
   }


/* Close the grid file normally
-------------------------------*/
close_action = 0;
if (c_lsclos(&fdesc, grid->gridname, &close_action) != E_SUCC)
   {
   strcpy(str, "Error closing the grid file");
   c_errmsg(str, "c_wrtgrid-close", &nonfatal);
   }

return(E_SUCC);
}
 
