/*******************************************************************************
FUNCTION:       C_L1B2L1B

PURPOSE:        Given a buffer of Level-1B data, return a buffer of data in
		some other Level-1B data format.  See "level1bio.h" for a
		summary of the 24 Level-1B data formats.
		NOTE: No error checking is done for the sake of efficiency.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    11/97  T.Baltzer        Development
  2.0    12/98  D.Lloyd          Added KLM record and prevideo sizes

PROJECT:        ADAPS
*******************************************************************************/
#include "las.h"
#include "pixman.h"
#include "adaps.h"
#include "acqio.h"
#include "level1bio.h"
#include "local_protos.h"



void c_l1b2l1b
(
    long irecsiz,		/* Input buffer record size	*/
    long ichan[],		/* Input channels within inbuf	*/
    unsigned char *ibuf,	/* Input buffer			*/
    long orecsiz,		/* Output buffer record size	*/
    long ochan[],		/* Output channels within obuf	*/
    unsigned char *obuf		/* Output buffer		*/
)

{

long dtype;			/* Data type		*/
long klmflg = FALSE;		/* KLM flag				 */
long ns;			/* Size of the packed video data (bytes) */
long offset;			/* Offset into buffer			 */
long prevideo;			/* Size of pre-video data (bytes)	 */
long postvideo = 0;		/* Size of post-video data (bytes)	 */
long sparesize = 0;		/* Size of spare bytes past post-video data */
long videosize = 0;		/* Size of the video data buffer (bytes) */
unsigned char fillval;		/* Fill value				 */


/* Determine the size of the OUTPUT video data, etc. (refer to the table
   in "level1bio.h").
------------------------------------------------------------------------*/
switch (orecsiz)
    {
    case UNP_LAC_1:
	videosize = V_UNP_LAC_1; postvideo = 0; sparesize = 0; break;
    case UNP_LAC_2:
	videosize = V_UNP_LAC_2; postvideo = 0; sparesize = 0; break;
    case UNP_LAC_3:
	videosize = V_UNP_LAC_3; postvideo = 0; sparesize = 0; break;
    case UNP_LAC_4:
	videosize = V_UNP_LAC_4; postvideo = 0; sparesize = 0; break;
    case UNP_LAC_5:
	videosize = V_UNP_LAC_5; postvideo = 0; sparesize = 0; break;
    case UNP_GAC_1:
	videosize = V_UNP_GAC_1; postvideo = 0; sparesize = 0; break;
    case UNP_GAC_2:
	videosize = V_UNP_GAC_2; postvideo = 0; sparesize = 0; break;
    case UNP_GAC_3:
	videosize = V_UNP_GAC_3; postvideo = 0; sparesize = 0; break;
    case UNP_GAC_4:
	videosize = V_UNP_GAC_4; postvideo = 0; sparesize = 0; break;
    case UNP_GAC_5:
	videosize = V_UNP_GAC_5; postvideo = 0; sparesize = 0; break;
    case PCK_LAC_5:
	videosize = V_PCK_LAC_5; postvideo = 22; sparesize = 674; break;
    case PCK_GAC_5:
	videosize = V_PCK_GAC_5; postvideo = 22; sparesize = 22; break;
    case KLM_UNP_LAC_1:
	videosize = V_UNP_LAC_1; postvideo = 0; sparesize = 784; 
	klmflg = TRUE; break;
    case KLM_UNP_LAC_2:
	videosize = V_UNP_LAC_2; postvideo = 0; sparesize = 784; 
	klmflg = TRUE; break;
    case KLM_UNP_LAC_3:
	videosize = V_UNP_LAC_3; postvideo = 0; sparesize = 784; 
	klmflg = TRUE; break;
    case KLM_UNP_LAC_4:
	videosize = V_UNP_LAC_4; postvideo = 0; sparesize = 784; 
	klmflg = TRUE; break;
    case KLM_UNP_LAC_5:
	videosize = V_UNP_LAC_5; postvideo = 0; sparesize = 784; 
	klmflg = TRUE; break;
    case KLM_UNP_GAC_1:
	videosize = V_UNP_GAC_1; postvideo = 0; sparesize = 276; 
	klmflg = TRUE; break;
    case KLM_UNP_GAC_2:
	videosize = V_UNP_GAC_2; postvideo = 0; sparesize = 276; 
	klmflg = TRUE; break;
    case KLM_UNP_GAC_3:
	videosize = V_UNP_GAC_3; postvideo = 0; sparesize = 272; 
	klmflg = TRUE; break;
    case KLM_UNP_GAC_4:
	videosize = V_UNP_GAC_4; postvideo = 0; sparesize = 280; 
	klmflg = TRUE; break;
    case KLM_UNP_GAC_5:
	videosize = V_UNP_GAC_5; postvideo = 0; sparesize = 276; 
	klmflg = TRUE; break;
    case KLM_PCK_LAC_5:
	videosize = V_PCK_LAC_5; postvideo = 0; sparesize = 952; 
	klmflg = TRUE; break;
    case KLM_PCK_GAC_5:
	videosize = V_PCK_GAC_5; postvideo = 0; sparesize = 616; 
	klmflg = TRUE; break;
    }


/* Copy the pre-video data from the input buffer to the output buffer.  The
   size and contents of this data never changes, even if the Level-1B format
   does change.
------------------------------------------------------------------------*/
dtype = EBYTE;
if ( klmflg )
    prevideo = ns = KLMPREVIDEO;
else
    prevideo = ns = PREVIDEO;
c_pxcopy(ibuf,obuf,&dtype,&ns);


/* Now copy/transpose the video data to whatever it should be.
------------------------------------------------------------------------*/
l1bvideo(irecsiz, ichan, &ibuf[prevideo], orecsiz, ochan, &obuf[prevideo],
	 videosize);


/* If post-video data needs to be written, write it here.  (refer to the table
   in "level1bio.h").
   NOTE: Since we at EDC currently support only the pre Sep '92 Level-1B format,
	 we will zero fill the post-video data (Decimal portion of 51 Solar
	 Zenith Angles, and Clock drift delta).  If subseqent versions of the
	 Level-1B format are supported in the future, these fields will have
	 to be computed and packed, or gotten from the input buffer, whichever
	 is appropriate.

   NOTE: KLM only.  Sine we at EDC don't archive the TIP data and don't
         currently have the CLAVR algorithm, we fill all post-video with
	 zero's.
------------------------------------------------------------------------*/
if (postvideo != 0)
    {
    dtype = EBYTE;
    ns = postvideo;
    offset = prevideo + videosize;
    fillval = 0x00;
    c_pxfill(&obuf[offset],&dtype,&ns,&fillval);
    }


/* If spare filler data needs to be written, write it here.  Just zero fill
   these spare bytes (refer to the table in "level1bio.h").
------------------------------------------------------------------------*/
if (sparesize != 0)
    {
    dtype = EBYTE;
    ns = sparesize;
    offset = prevideo + videosize + postvideo;
    fillval = 0x00;
    c_pxfill(&obuf[offset],&dtype,&ns,&fillval);
    }

}
