/*******************************************************************************
FUNCTION:	C_L1BOPNDR

PURPOSE:	Open a Level-1B file on disk for read access.  Read the TBM/ARS
		header record and the Data Set Header record.  Load the
		AVHRR model with the information from the headers.  Verify
		the first time stamp in the image.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T. Baltzer 	 Development
  1.1    12/98  D.Lloyd		 Modifications for NOAA KLM
  1.2    05/99  T. Baltzer       Mods to give PCK_LAC_5 format precedence.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "level1bio.h"
#include "local_protos.h"


int c_l1bopndr
(
char infile[CMLEN],     /* INPUT:  Input file				*/
FILE	**in_fd,	/* OUTPUT: File descriptor to the input file	*/
long	*recsiz,	/* OUTPUT: Record size of data read		*/
unsigned char buf[],	/* OUTPUT: TBM/ARS header buffer		*/
unsigned char dsh[],	/* OUTPUT: DSH header buffer			*/
struct MODEL *model 	/* OUTPUT: AVHRR model information		*/
)

{

long	dtype,			/* Data type LAC/GAC			*/
	goodlines,		/* number of good lines in the image	*/
	linfile,		/* Number of line based on file size	*/
	nchans_tmp,		/* Number of channels from file size	*/
	status,			/* Return status variable		*/
	hdrflg;			/* TBM/ARS header flag (yes or no)	*/

unsigned char linbuf[MAXRECSIZ];        /* First line of data in file	*/

/*
  Read the information required from the file
*/
status = l1b_file_in(infile,in_fd,buf,dsh,linbuf,recsiz,&dtype,&linfile,
		     &hdrflg,&nchans_tmp);
if (status != E_SUCC)
    return(status);


/*
  Initialize the model structure using the TBM/ARS and DSH headers
*/
status = l1b_load_mod(buf,dsh,*recsiz,hdrflg,&goodlines,model);
if (status != E_SUCC)
    return(status);

/*
  If there was a disagreement between the information as determined
  by the file size and the header information, error out.  This check will
  not catch all possible error scenarios, but will hopefully catch the most
  frequently occurring ones.
*/
if ((nchans_tmp != model->nchans) ||
    (((dtype == LAC) || (dtype == HRPT)) &&
     ((model->dtype != LAC) && (model->dtype != HRPT))) ||
    ((dtype == GAC) && (model->dtype != GAC)))
    {
    c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
    return(E_FAIL);
    }


/*
  Check the first time stamp in the image
*/
l1b_ck_tstamp(linbuf,model);


/*
  Check to make sure that there is not a discrepancy between the number of
  lines in the file based on the file size and the number of lines reported
  in the DSH header.
*/
if (goodlines != linfile)
    {
    c_lasmsg(FALSE,"l1bopndr-lin","Warning: discrepancy in good line count");
    c_lasmsg(FALSE,"l1bopndr-lin","%d lines according to DSH header",goodlines);
    c_lasmsg(FALSE,"l1bopndr-lin","%d lines according to file size",linfile);
    }


return(E_SUCC);
}
