/*****************************************************************************
FUNCTION:	C_L1BOPNDW

PURPOSE:	Open a Level 1b disk file for writing.  The TBM/ARS and DSH
		headers are written and file pointer positioned and ready
		for first data record.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1    12/98	D.Lloyd		Modifications for NOAA KLM ARS header
  2.0  03/2002  D.Lloyd         Unconditionally allocate temporary buffers
                                and copy input buffers.  Byte-swap the
				buffers on little endian systems.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"


int c_l1bopndw
(
char		outfile[CMLEN],	/* IN: output file name			*/
long		*recsiz,	/* IN: data record size in bytes	*/
unsigned char	*buf,		/* IN: TBM/ARS header buffer		*/
unsigned char	*dsh,		/* IN: DSH header buffer		*/
long		*hsize,		/* IN: TBM/ARS header size		*/
FILE		**out_fd 	/* OUT: output file descriptor		*/
)

{
long	nbytes;			/* number of bytes written		*/

int     bufptr_allocated = FALSE; /* flag to denote memory was allocated*/
				/* for the TBM/ARS pointer		*/

unsigned char	*dshptr;	/* pointer to DSH header buffer		*/
unsigned char	*bufptr;	/* pointer to TBM/ARS header buffer	*/



/*
  Determine if input buffer pointers are valid.  Create zero buffers
  if not.
*/
if ( buf == NULL || dsh == NULL )
    {
    bufptr = (unsigned char *)calloc( *recsiz, sizeof( unsigned char ));
    if ( bufptr == NULL )
	{
	c_lasmsg(FALSE,"l1bopndw-alloc","Error allocating memory");
        return E_FAIL;
	}
    bufptr_allocated = TRUE;
    dshptr = (unsigned char *)calloc( *recsiz, sizeof( unsigned char ));
    if ( dshptr == NULL )
	{
	c_lasmsg(FALSE,"l1bopndw-alloc","Error allocating memory");
        return E_FAIL;
	}
    }  /* END if */

/* Allocate the buffer and copy the dsh buffer into it.  This is
done to prevent the input buffer from being byte-swapped on little
endian systems.  Also note that c_l1bopndw() is called once during
execution and the malloc and memcpy create little system overhead.
----------------------------------------------------------------------*/
else
    {
    bufptr = buf;

    dshptr = (unsigned char *)malloc( *recsiz * sizeof( unsigned char ));
    if ( dshptr == NULL )
	{
	c_lasmsg(FALSE,"l1bopen-alloc","Error allocating memory");
        return E_FAIL;
	}
    memcpy( dshptr, dsh, *recsiz );
    switch ( *recsiz )
        {
        case PCK_LAC_5 :
	case UNP_LAC_5 :
	case UNP_LAC_4 :
	case UNP_LAC_3 :
	case UNP_LAC_2 :
	case UNP_LAC_1 :
        case PCK_GAC_5 :
	case UNP_GAC_5 :
	case UNP_GAC_4 :
	case UNP_GAC_3 :
	case UNP_GAC_2 :
	case UNP_GAC_1 :
	    export_endian_mixed_buffer( dshptr, NOAADATASETHEADER );
	    break;
	case KLM_PCK_LAC_5 :
	case KLM_UNP_LAC_5 :
	case KLM_UNP_LAC_4 :
	case KLM_UNP_LAC_3 :
	case KLM_UNP_LAC_2 :
	case KLM_UNP_LAC_1 :
        case KLM_PCK_GAC_5 :
	case KLM_UNP_GAC_5 :
	case KLM_UNP_GAC_4 :
	case KLM_UNP_GAC_3 :
	case KLM_UNP_GAC_2 :
	case KLM_UNP_GAC_1 :
	    export_endian_mixed_buffer( dshptr, KLMDATASETHEADER );
	    break;
	}  /* END switch */
    }  /* END else */


/*
  Open the Level 1b file
*/
if (( *out_fd = fopen( outfile, "w" )) == NULL )
    {
    c_lasmsg(FALSE,"l1bopen-open","Error opening output file for write");
    return E_FAIL;
    }  /* END if */

/*
  Write the TBM/ARS header
*/
nbytes = (long)fwrite( bufptr, 1, *hsize, *out_fd );
if ( nbytes != *hsize )
    {
    c_lasmsg(FALSE,"l1bopen-write","Error writing to Level 1b file");
    return E_FAIL;
    }  /* END if */

/*
  Write the DSH 
*/
nbytes = (long)fwrite( dshptr, 1, *recsiz, *out_fd );
if ( nbytes != *recsiz )
    {
    c_lasmsg(FALSE,"l1bopen-write","Error writing DSH to output");
    return E_FAIL;
    }  /* END if */

/*
  In case of pre-KLM LAC data write a dummy record
*/
switch ( *recsiz )
    {
    case PCK_LAC_5 :
    case UNP_LAC_5 :
    case UNP_LAC_4 :
    case UNP_LAC_3 :
    case UNP_LAC_2 :
    case UNP_LAC_1 :
	nbytes = (long)fwrite( dshptr, 1, *recsiz, *out_fd );
	if ( nbytes != *recsiz )
	    {
	    c_lasmsg(FALSE,"l1bopen-write","Error writing dummy to output");
	    return E_FAIL;
	    }  /* END if */
    }  /* END switch */

if ( bufptr_allocated )
    free( bufptr );
free( dshptr );
return E_SUCC;

}  /* END c_l1bopndw */
