/*****************************************************************************
FUNCTION:	C_L1BOPNTW

PURPOSE:	Open a Level 1b tape for writing.  The TBM/ARS header and DSH
		records are written and file pointer positioned for write
		of first data record.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd         Modifications for NOAA KLM
  2.0   03/2002 D.Lloyd         Allocate temporary buffers and copy input
                                buffers.  Byte-swap the buffers on little
				endian systems.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "tapeio.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"


int c_l1bopntw
(
    char drive[CMLEN],	/* IN: drive name			*/
    char *density,	/* IN: tape density			*/
    long *tun,		/* OUT: tape logical unit number	*/
    unsigned char *buf,	/* IN: pointer to TBM/ARS header buffer	*/
    unsigned char *dsh,	/* IN: pointer to DSH buffer		*/
    long *recsiz,	/* IN: record size, DSH size, in bytes	*/
    long *tape		/* IN: tape open flag			*/
)

{

long	inout =  TWRITE;	/* write to tape 			*/
long	mode = HANDLE;		/* handle tape errors			*/
long	nbytes;			/* number of bytes written		*/
long	rewopt = NOREW;		/* no rewind flag			*/
long	skip = 0;		/* number of files to skip		*/
long	status;			/* function return status		*/

unsigned char   *dshptr;        /* pointer to DSH header buffer         */


/* Allocate the buffer and copy the dsh buffer into it.  This is
done to prevent the dsh buffer from being byte-swapped on little
endian systems.  Also note that c_l1bopndw() is called once during
execution and the malloc and memcpy create little system overhead.
----------------------------------------------------------------------*/
dshptr = (unsigned char *)malloc( *recsiz * sizeof( unsigned char ));
if ( dshptr == NULL )
    return E_FAIL;
memcpy( dshptr, dsh, *recsiz );
switch ( *recsiz )
    {
    case PCK_LAC_5 :
    case UNP_LAC_5 :
    case UNP_LAC_4 :
    case UNP_LAC_3 :
    case UNP_LAC_2 :
    case UNP_LAC_1 :
    case PCK_GAC_5 :
    case UNP_GAC_5 :
    case UNP_GAC_4 :
    case UNP_GAC_3 :
    case UNP_GAC_2 :
    case UNP_GAC_1 :
	export_endian_mixed_buffer( dshptr, NOAADATASETHEADER );
	break;
    case KLM_PCK_LAC_5 :
    case KLM_UNP_LAC_5 :
    case KLM_UNP_LAC_4 :
    case KLM_UNP_LAC_3 :
    case KLM_UNP_LAC_2 :
    case KLM_UNP_LAC_1 :
    case KLM_PCK_GAC_5 :
    case KLM_UNP_GAC_5 :
    case KLM_UNP_GAC_4 :
    case KLM_UNP_GAC_3 :
    case KLM_UNP_GAC_2 :
    case KLM_UNP_GAC_1 :
	export_endian_mixed_buffer( dshptr, KLMDATASETHEADER );
	break;
    }  /* END switch */


/*
  Open tape for write
*/
status = c_tpreop( tun, &skip, &mode, &rewopt, density, &inout, drive );
if ( status != TAPE_SUCC )
    return E_FAIL;
else
    *tape = TRUE;


/*
  Write TBM/ARS header
*/
switch ( *recsiz )
    {
    case KLM_UNP_LAC_1 :
    case KLM_UNP_LAC_2 :
    case KLM_UNP_LAC_3 :
    case KLM_UNP_LAC_4 :
    case KLM_UNP_LAC_5 :
    case KLM_PCK_LAC_5 :
    case KLM_UNP_GAC_1 :
    case KLM_UNP_GAC_2 :
    case KLM_UNP_GAC_3 :
    case KLM_UNP_GAC_4 :
    case KLM_UNP_GAC_5 :
    case KLM_PCK_GAC_5 :
        nbytes = ARS_HDR_SIZ;
	break;
    default :
	nbytes = TBM_HDR_SIZ;
    }
status = c_tpwrit( tun, buf, &nbytes, &mode );
if ( status != nbytes )
    return E_FAIL;


/*
  Write DSH
*/
status = c_tpwrit( tun, dshptr, recsiz, &mode );
if ( status != *recsiz )
    return E_FAIL;


/*
  For pre-KLM LAC/HRPT data write out a dummy record
*/
switch ( *recsiz )
    {
    case PCK_LAC_5 :
    case UNP_LAC_5 :
    case UNP_LAC_4 :
    case UNP_LAC_3 :
    case UNP_LAC_2 :
    case UNP_LAC_1 :
	status = c_tpwrit( tun, dshptr, recsiz, &mode );
	if ( status != *recsiz )
	    return E_FAIL;
    }  /* END switch */

free( dshptr );
return E_SUCC;

}  /* END c_l1bopntw */
