/******************************************************************************
FUNCTION:	C_L1BREADD

PURPOSE:	Get a line of Level 1b data from disk

Version  Date   Programmer       Modifications
-------  -----  ---------------  ----------------------------------------------
  1.0    01/92  A.Vander Vorste  Development
  1.1	 09/97	D.Lloyd		 Modified as a support routine: eliminated
				 reference to external model information and
				 removed code that modifies parameter recsiz.
  1.2    12/98  D.Lloyd		 Modified for NOAA KLM
  2.0  03/2002  D.Lloyd          Byte-swap the record on little endian systems

PROJECT:	ADAPS
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"


int c_l1breadd
(
FILE		*in_fd,		/* IN: file pointer to input file	*/
long		*recsiz,	/* IN: Record size of data to read	*/
unsigned char	*recptr 	/* OUT: Pointer to Level 1b scan line	*/
)

{
char	message[CMLEN];		/* message string			*/

long	bytes_read;		/* the number of bytes read		*/
long	nbytes;			/* number of bytes per line		*/
long	status;			/* Return status			*/


/*
  Determine number of bytes per line: if LAC or HRPT there are 2 records
  per line, GAC is two lines per record.  NOTE: KLM stores 1 line per
  record.
*/
switch ( *recsiz )
    {
    case  PCK_LAC_5 :
    case  UNP_LAC_5 :
    case  UNP_LAC_4 :
    case  UNP_LAC_3 :
    case  UNP_LAC_2 :
    case  UNP_LAC_1 : 
	nbytes = *recsiz * 2;
	break;
    case  PCK_GAC_5 :
    case  UNP_GAC_5 :
    case  UNP_GAC_4 :
    case  UNP_GAC_3 :
    case  UNP_GAC_2 :
    case  UNP_GAC_1 : 
	nbytes = *recsiz / 2;
	break;
    case  KLM_UNP_LAC_1 :
    case  KLM_UNP_LAC_2 :
    case  KLM_UNP_LAC_3 :
    case  KLM_UNP_LAC_4 :
    case  KLM_UNP_LAC_5 :
    case  KLM_PCK_LAC_5 :
    case  KLM_UNP_GAC_1 :
    case  KLM_UNP_GAC_2 :
    case  KLM_UNP_GAC_3 :
    case  KLM_UNP_GAC_4 :
    case  KLM_UNP_GAC_5 :
    case  KLM_PCK_GAC_5 :
        nbytes = *recsiz;
	break;
    default :
	c_lasmsg(FALSE,"l1breadd-irec","Invalid record size");
	return E_FAIL;
    }  /* END switch */

/*
  read a line of data
*/
bytes_read = (long)fread( recptr, 1, nbytes, in_fd );
if ( bytes_read <= 0 )
   {
   /* fgetc returns EOF if we are at End of File */
   status = fgetc( in_fd );

   /* If we reached End Of File */
   if ( status == EOF )
	return E_EOF;
   else
	{
	c_lasmsg(FALSE,"l1breadd-read","Error reading line from file");
	return E_FAIL;
	}  /* END else */
   }  /* END if */

/*
  Make sure we got the full line
*/
if ( bytes_read != nbytes )
   {
   sprintf(message, "Read %ld bytes of %ld in scan line",bytes_read,nbytes);
   c_lasmsg(FALSE,"l1breadd-read",message);
   return E_FAIL;
   }  /* END if */

/* On little endian system the level 1b record must be byte-swapped
-------------------------------------------------------------------*/
switch ( *recsiz )
    {
    case PCK_LAC_5 : import_endian_mixed_buffer( recptr, DATAREC_LP5 ); break;
    case UNP_LAC_5 : import_endian_mixed_buffer( recptr, DATAREC_LU5 ); break;
    case UNP_LAC_4 : import_endian_mixed_buffer( recptr, DATAREC_LU4 ); break;
    case UNP_LAC_3 : import_endian_mixed_buffer( recptr, DATAREC_LU3 ); break;
    case UNP_LAC_2 : import_endian_mixed_buffer( recptr, DATAREC_LU2 ); break;
    case UNP_LAC_1 : import_endian_mixed_buffer( recptr, DATAREC_LU1 ); break;
    case PCK_GAC_5 : import_endian_mixed_buffer( recptr, DATAREC_GP5 ); break;
    case UNP_GAC_5 : import_endian_mixed_buffer( recptr, DATAREC_GU5 ); break;
    case UNP_GAC_4 : import_endian_mixed_buffer( recptr, DATAREC_GU4 ); break;
    case UNP_GAC_3 : import_endian_mixed_buffer( recptr, DATAREC_GU3 ); break;
    case UNP_GAC_2 : import_endian_mixed_buffer( recptr, DATAREC_GU2 ); break;
    case UNP_GAC_1 : import_endian_mixed_buffer( recptr, DATAREC_GU1 ); break;
    case KLM_PCK_LAC_5 :
        import_endian_mixed_buffer( recptr, KLMDATAREC_LP5 ); break;
    case KLM_UNP_LAC_5 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_LU5 ); break;
    case KLM_UNP_LAC_4 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_LU4 ); break;
    case KLM_UNP_LAC_3 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_LU3 ); break;
    case KLM_UNP_LAC_2 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_LU2 ); break;
    case KLM_UNP_LAC_1 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_LU1 ); break;
    case KLM_PCK_GAC_5 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_GP5 ); break;
    case KLM_UNP_GAC_5 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_GU5 ); break;
    case KLM_UNP_GAC_4 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_GU4 ); break;
    case KLM_UNP_GAC_3 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_GU3 ); break;
    case KLM_UNP_GAC_2 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_GU2 ); break;
    case KLM_UNP_GAC_1 : 
        import_endian_mixed_buffer( recptr, KLMDATAREC_GU1 ); break;
    }  /* END switch */

return E_SUCC;

}  /* END c_l1breadd */
