/*****************************************************************************
FUNCTION:	C_L1BREADT

PURPOSE:	Get and return one line of Level 1b data from tape.

Version  Date	Programmer	Modifications
-------  -----	--------------	---------------------------------------------
  1.0    09/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd         Modifications for NOAA KLM

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "las.h"
#include "adaps.h"
#include "level1bio.h"
#include "local_protos.h"


int c_l1breadt
(
    long *tun,			/* IN: Tape logical unit number		*/
    long *recsiz,		/* IN: Record size in bytes		*/
    char comment[][CMLEN],	/* IN: Tape description			*/
    long *volnum,		/* IN/OUT: Volume number		*/
    long *tape,			/* IN/OUT: Tape open flag		*/
    unsigned char *recptr	/* OUT: Pointer to Level 1b scan record	*/
)

{
long	dtype;			/* Data type, GAC or LAC		*/
long	i;			/* Loop index variable			*/
long	nbytes;			/* number of bytes per scan line	*/
long	status;			/* function return status		*/

/*
  These variable are declared static to handle the GAC data type case.
  Reading one record of GAC gives two lines so the buffer must be
  maintained over separate calls.
*/
static long good_buf = FALSE;	/* Flag to show good data in GAC buffer	*/
static unsigned char buffer[MAXLINSIZ];	/* GAC line buffer		*/


/*
  Determine number of bytes per line: if LAC or HRPT there are 2 records
  per line, GAC is two lines per record.  NOTE: for KLM there is one line
  per record.
*/
switch ( *recsiz )
    {
    case  PCK_LAC_5 :
    case  UNP_LAC_5 :
    case  UNP_LAC_4 :
    case  UNP_LAC_3 :
    case  UNP_LAC_2 :
    case  UNP_LAC_1 :
	dtype = LAC;
	good_buf = FALSE;
        nbytes = *recsiz * 2;
        break;
    case  PCK_GAC_5 :
    case  UNP_GAC_5 :
    case  UNP_GAC_4 :
    case  UNP_GAC_3 :
    case  UNP_GAC_2 :
    case  UNP_GAC_1 :
	dtype = GAC;
        nbytes = *recsiz / 2;
        break;
    case  KLM_UNP_LAC_1 :
    case  KLM_UNP_LAC_2 :
    case  KLM_UNP_LAC_3 :
    case  KLM_UNP_LAC_4 :
    case  KLM_UNP_LAC_5 :
    case  KLM_PCK_LAC_5 :
	dtype = LAC;
	good_buf = FALSE;
        nbytes = *recsiz;
	break;
    case  KLM_UNP_GAC_1 :
    case  KLM_UNP_GAC_2 :
    case  KLM_UNP_GAC_3 :
    case  KLM_UNP_GAC_4 :
    case  KLM_UNP_GAC_5 :
    case  KLM_PCK_GAC_5 :
	dtype = LAC;      /* KLM GAC needs to be handled the same as LAC  */
	good_buf = FALSE; /* KLM GAC buffer doesn't need to be maintained */
        nbytes = *recsiz;
	break;
    default :
        c_lasmsg(FALSE,"l1breadt-irec","Invalid record size");
        return E_FAIL;
    }  /* END switch */

/*
  If data type is GAC and we don't need to read a new buffer, copy data
  from second half of current buffer to output and signal that the buffer
  no longer contains good data.
*/
if ( dtype == GAC && good_buf )
    {
    for ( i = 0; i < nbytes; i++ )
	recptr[i] = buffer[i+nbytes];
    good_buf = FALSE;
    }  /* END if */

/*
  Otherwise read a new record from the tape.  For GAC read into the GAC line
  buffer, for LAC read two records directly into the output buffer.
*/
else
    {
    if ( dtype == GAC )
	{
	/*
	  Read two lines of data into buffer, signal that the buffer has
	  good data, and copy first line into output buffer.
	*/
	status = get_rec_tape(tun,buffer,recsiz,volnum,comment,tape);
	if ( status != E_SUCC )
	    return status;
	good_buf = TRUE;
	for ( i = 0; i < nbytes; i++ )
	    recptr[i] = buffer[i];
	}  /* END if */
    else
	{
	/*
	  Read one record into the first half of the output buffer and
	  another into the second half of the output buffer.
	*/
	status = get_rec_tape( tun, recptr, recsiz, volnum, comment, tape );
        if ( status != E_SUCC )
            return status;
        status = get_rec_tape( tun, recptr+*recsiz, recsiz, volnum, 
				comment, tape );
        if ( status != E_SUCC )
            return status;
	}  /* END else */

    }  /* END else */


return E_SUCC;

}  /* END c_l1breadt */
