/*****************************************************************************
FUNCTION:	C_L1BSEEKD

PURPOSE:	Seek to the specified line number in the Level 1b disk file.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd         Modifications for NOAA KLM
  1.2  03/2002  D.Lloyd         Changed the way the the line number was
                                extracted for little endian compatibility.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "level1bio.h"


#define  ABS(x)  ((x) > 0 ? (x) : -(x))


int c_l1bseekd
(
FILE		*in_fd,			/* IN: file descriptor		*/
long		window[],		/* IN: line number to seek	*/
long		*recsiz,		/* IN: record size in bytes	*/
struct MODEL	*model,			/* IN: AVHRR model structure	*/
long		*drop_line 		/* OUT: number of drop lines	*/
)

{
int             true = TRUE;            /* for compiler warning         */
long		current_pos;		/* current file position	*/
long		file_line;		/* file relative line number	*/
long		first_pos;		/* first line file position	*/
long		last_step;		/* last step size in lines	*/
long		line_diff;		/* difference in line numbers	*/
long		first_line;		/* first scan line number	*/
long		line_size;		/* size of scan line		*/
long		offset;			/* byte offset to seek		*/
long		status;			/* function return status	*/

short		current_line;		/* current line number		*/

unsigned char	dummy[MAXRECSIZ];	/* dummy buffer			*/


/*
  Save the file position for later
*/
first_pos = ftell( in_fd );


/*
  Determine line size: 2 lines per GAC record, 2 records per LAC line
  For KLM 1 line per record
*/
if ( model->satnum <= 14 )
    {
    if ( model->dtype == GAC )
	line_size = *recsiz / 2;
    else
	line_size = *recsiz * 2;
    }
else
    line_size = *recsiz;


/*
  Find out what the first line number actually is.
*/
fread( dummy, sizeof(short), 1, in_fd );
current_line = (short)(( dummy[0] << 8 ) + dummy[1] );
first_line = current_line;
fseek( in_fd, -2, SEEK_CUR );


/*
  Determine file-relative line number.  If pass is ascending then
  the line number must be calculated relative to the ending line.
*/
if ( model->cpass == ASCENDING && model->sat_view == TRUE )
    file_line = model->nlines - window[0] - window[2] + first_line + 1;
else
    file_line = first_line + window[0] - 1; 


/*
  Move to the correct line or at least get close to it.
*/
last_step = file_line - first_line + 1;
while ( true )
    {
    fread( dummy, sizeof(short), 1, in_fd );
    current_line = (short)(( dummy[0] << 8 ) + dummy[1] );

    /* break out of loop if the correct line is found */
    if ( current_line == file_line )
	{
	fseek( in_fd, -2, SEEK_CUR );
	break;
	}  /* END if */

    /* Break out when the step size reaches 1 */
    if ( ABS( last_step ) == 1 )
	break;

    /*
      Cut the step size in half to keep from seeking past
      file boundaries and to narrow the search
    */
    line_diff = file_line - current_line;
    if ( ABS( line_diff ) >= ABS( last_step ) )
	{
	if ( current_line > file_line )
	    line_diff = -ABS( last_step ) / 2;
	else
	    line_diff = ABS( last_step ) / 2;
	}  /* END if */
    last_step = line_diff;

    /* seek to the next record */
    offset = line_size * line_diff - 2;
    fseek( in_fd, offset, SEEK_CUR );

    }  /* END while */


/*
  The previous loop exited either because the line was found or the
  step size was reduced to 1.  If the latter occurred, most likely
  the requested line is a dropped line.  Search for the line and,
  if it's within a dropped line sequence, return the next good line.
*/
while( current_line != file_line )
    {
    line_diff = file_line - current_line;

    /*
      When a sign change occurs, we've stepped over a drop line
      sequence without finding the correct line so break returning
      the bordering line number
     */
    if (( line_diff * last_step ) < 0 )
	{
	/* if the previous search was backwards skip forward 1 line */
	if ( last_step < 0 )
	    {
	    fseek( in_fd, line_size-2, SEEK_CUR );
	    fread( dummy, sizeof(short), 1, in_fd );
	    current_line = (short)(( dummy[0] << 8 ) + dummy[1] );
	    }  /* END if */
	fseek( in_fd, -2, SEEK_CUR );
	break;
	}  /* END if */

    /* seek the next line */
    offset = line_size * last_step - 2;
    status = fseek( in_fd, offset, SEEK_CUR );
    if ( status != 0 )
	return E_FAIL;
    fread( dummy, sizeof(short), 1, in_fd );
    current_line = (short)(( dummy[0] << 8 ) + dummy[1] );

    /* break out if we find the correct line */
    if ( current_line == file_line )
	{
	fseek( in_fd, -2, SEEK_CUR );
	break;
	}  /* END if */

    }  /* END while */


/*
  There is an additional worry with pre-KLM GAC data since there are two lines
  per record.  It must be insured that the first call to c_l1breadd by
  the user returns the proper line.  If the current line resides in the
  second half of the record, back up one line and call c_l1breadd to
  prime the static GAC buffer maintained in c_l1breadd.
*/
if ( model->dtype == GAC && model->satnum <= 14 )
    {
    current_pos = ftell( in_fd ) - first_pos;
    current_pos /= line_size;

    /* if we are not an even number of lines from the first line */
    if (( current_pos != 0 ) && (( current_pos % 2 ) != 0 ))
	{
	/* back up 1 line */
	fseek( in_fd, -line_size, SEEK_CUR );

	/* prime the GAC buffer */
	status = c_l1breadd( in_fd, recsiz, dummy );
	if ( status != E_SUCC )
	    c_lasmsg( FALSE, "Error reading line", "l1bseekd-seek" );
	}  /* END if */
    }  /* END if */


/* return the number of drop lines encountered */
*drop_line = current_line - file_line ;
return E_SUCC;

}  /* END c_l1bseekd */
