/*****************************************************************************
FUNCTION:	C_L1BWRITED

PURPOSE:	Write a Level 1b line to disk.  When used for pre-KLM GAC
		data you must call this function with a zero-filled buffer
		and a zero recsiz one time prior to closing the output file.
		This insures that any buffered data is flushed to the output
		file.  Using c_l1bclosd to close the file does this for you.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    09/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd         Modifications for NOAA KLM
  2.0  03/2002  D.Lloyd         Added an output buffer for endian swapping
                                on little endian systems.  Added export
				calls in the big switch.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "avhrr.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"


int c_l1bwrited
(
FILE		*out_fd,	/* IN: output file descriptor		*/
unsigned char	*recptr,	/* IN: pointer to Level 1b data line	*/
long		*recsiz 	/* IN: record size in bytes for output	*/
)

{
char	message[CMLEN];		/* message string			*/

long	bytes_written;		/* number of bytes written to file	*/
long	dtype;			/* data type, GAC or LAC		*/
long	i;			/* loop index variable			*/
long	nbytes;			/* number of bytes per line		*/

/*
  These variables are declared static to handle the GAC data type case.
  One record of GAC is two lines of data so the buffer must be maintained
  over separate calls.
*/
static unsigned char buffer[MAXLINSIZ];	/* GAC line buffer		*/
static long write_buf = 0;	/* flag to show buffer must be written	*/
				/* either contains 0 or record size	*/

static unsigned char *outbuf = NULL; /* temporary output buffer so that */
                                     /* input buffer is not corrupted   */

#ifdef LAS_LITTLE_ENDIAN
/* For little endian systems, allocate and copy the data record to outbuf
Only allocate once and allow the system to unallocate on exit.
-------------------------------------------------------------------------*/
if ( outbuf == NULL )
    {
    outbuf = (unsigned char *)malloc( MAXLINSIZ );
    if ( outbuf == NULL )
        return E_FAIL;
    }

switch ( *recsiz )
    {
    case PCK_LAC_5 :
    case UNP_LAC_5 :
    case UNP_LAC_4 :
    case UNP_LAC_3 :
    case UNP_LAC_2 :
    case UNP_LAC_1 :
        nbytes = *recsiz * 2;
	break;
    case PCK_GAC_5 :
    case UNP_GAC_5 :
    case UNP_GAC_4 :
    case UNP_GAC_3 :
    case UNP_GAC_2 :
    case UNP_GAC_1 :
        nbytes = *recsiz / 2;
	break;
    default :
        nbytes = *recsiz;
    }  /* END switch */
    
memcpy( outbuf, recptr, nbytes );
#else
/* Otherwise just point to the record
-------------------------------------*/
outbuf = recptr;
#endif

/*
  Determine number of bytes per line: if GAC there are two lines per
  record, if LAC or HRPT there are two records per line.  Insure the
  buffer in in the proper endian format.
*/
switch ( *recsiz )
    {
    case KLM_PCK_LAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LP5 );
        dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_LAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU5 );
        dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_LAC_4 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU4 );
        dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_LAC_3 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU3 );
        dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_LAC_2 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU2 );
        dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_LAC_1 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU1 );
        dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz;
	break;

    case KLM_PCK_GAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GP5 );
        dtype = LAC;	 /* KLM GAC handled the same as LAC */
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_GAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU5 );
        dtype = LAC;	 /* KLM GAC handled the same as LAC */
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_GAC_4 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU4 );
        dtype = LAC;	 /* KLM GAC handled the same as LAC */
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_GAC_3 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU3 );
        dtype = LAC;	 /* KLM GAC handled the same as LAC */
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_GAC_2 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU2 );
        dtype = LAC;	 /* KLM GAC handled the same as LAC */
	write_buf = 0;
	nbytes = *recsiz;
	break;
    case KLM_UNP_GAC_1 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU1 );
        dtype = LAC;	 /* KLM GAC handled the same as LAC */
	write_buf = 0;
	nbytes = *recsiz;
	break;

    case PCK_LAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_LP5 );
	dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz * 2;
	break;
    case UNP_LAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_LU5 );
	dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz * 2;
	break;
    case UNP_LAC_4 : export_endian_mixed_buffer( outbuf, DATAREC_LU4 );
	dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz * 2;
	break;
    case UNP_LAC_3 : export_endian_mixed_buffer( outbuf, DATAREC_LU3 );
	dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz * 2;
	break;
    case UNP_LAC_2 : export_endian_mixed_buffer( outbuf, DATAREC_LU2 );
	dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz * 2;
	break;
    case UNP_LAC_1 : export_endian_mixed_buffer( outbuf, DATAREC_LU1 );
	dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz * 2;
	break;

    case PCK_GAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_GP5 );
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;
    case UNP_GAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_GU5 );
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;
    case UNP_GAC_4 : export_endian_mixed_buffer( outbuf, DATAREC_GU4 );
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;
    case UNP_GAC_3 : export_endian_mixed_buffer( outbuf, DATAREC_GU3 );
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;
    case UNP_GAC_2 : export_endian_mixed_buffer( outbuf, DATAREC_GU2 );
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;
    case UNP_GAC_1 : export_endian_mixed_buffer( outbuf, DATAREC_GU1 );
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;

    /* This case is for the final buffer flush
    which has already been swapped.
    ------------------------------------------*/
    case 0 :
	*recsiz = write_buf;
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;
    
    default :
        c_lasmsg(FALSE, "l1bwrited-irec", "Invalid record size");
        return E_FAIL;
    }  /* END switch */

/*
  If write_buf is non-zero then the first half of the output buffer is
  filled.  Copy the data to the second half, write it, and reset the
  write_buf flag to zero.
*/
if ( dtype == GAC && write_buf != 0 )
    {
    for ( i = 0; i < nbytes; i++ )
	buffer[i+nbytes] = outbuf[i];
    bytes_written = (long)fwrite( buffer, 1, *recsiz, out_fd );
    if ( bytes_written < *recsiz )
	{
	sprintf( message, "Wrote %ld bytes of %ld in scan line",
		bytes_written, *recsiz );
	c_lasmsg(FALSE, "l1bwrited-write", message );
	return E_FAIL;
	}  /* END if */
    write_buf = 0;
    }  /* END if */

/*
  If the data type is GAC then we need to fill the first half of the
  buffer and set the write_buf flag to recsiz.
*/
else if ( dtype == GAC )
    {
    for ( i = 0; i < nbytes; i++ )
	buffer[i] = outbuf[i];
    write_buf = *recsiz;
    }  /* END else if */

/*
  Otherwise we have LAC or HRPT data so write the line.
*/
else
    {
    bytes_written = (long)fwrite( outbuf, 1, nbytes, out_fd );
    if ( bytes_written < *recsiz )
	{
	sprintf( message, "Wrote %ld bytes of %ld in scan line",
                bytes_written, nbytes );
        c_lasmsg(FALSE, "l1bwrited-write", message );
        return E_FAIL;
        }  /* END if */
    }  /* END else */

return E_SUCC;

}  /*  END c_l1bwrited */

