/*****************************************************************************
FUNCTION:	C_L1BWRITET

PURPOSE:	Write a line of Level 1b data to tape.  When used for pre-KLM
		GAC data you must call this function with a zero-filled buffer
		and a zero recsiz one time prior to closing the output 
		tape.  This insures that any buffered data is flushed to 
		the output tape.  Using c_l1bclost to close the tape does 
		this for you.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    09/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd		Modifications for NOAA KLM

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "las.h"
#include "avhrr.h"
#include "level1bio.h"
#include "local_protos.h"


int c_l1bwritet
(
    long *tun,			/* IN: tape logical unit number		*/
    unsigned char *recptr,	/* IN: pointer to level 1b scan record	*/
    long *recsiz,		/* IN: record size in bytes		*/
    long *tape			/* IN/OUT: tape open flag		*/
)

{
long	dtype;			/* data type, GAC or LAC		*/
long	i;			/* loop index variable			*/
long	nbytes;			/* number of bytes per scan line	*/
long	status;			/* function return status		*/

/*
  These variables are declared static to handle the GAC data type case.
  One record holds two lines so the buffer must be maintained across
  function calls.
*/
static unsigned char buffer[MAXLINSIZ];	/* GAC line buffer		*/
static long write_buf = 0;	/* flag showing buffer has data 	*/
				/* either zero or recsiz		*/


/*
  Determine the number of bytes per line.  If LAC or HRPT there are two
  records per line, GAC has two lines per record.
*/
switch ( *recsiz )
    {
    case KLM_PCK_LAC_5 :
    case KLM_UNP_LAC_5 :
    case KLM_UNP_LAC_4 :
    case KLM_UNP_LAC_3 :
    case KLM_UNP_LAC_2 :
    case KLM_UNP_LAC_1 :
        dtype = LAC;
        write_buf = 0;
        nbytes = *recsiz;
        break;

    case KLM_PCK_GAC_5 :
    case KLM_UNP_GAC_5 :
    case KLM_UNP_GAC_4 :
    case KLM_UNP_GAC_3 :
    case KLM_UNP_GAC_2 :
    case KLM_UNP_GAC_1 :
        dtype = LAC;	 /* KLM GAC handeled the same as LAC */
        write_buf = 0;
        nbytes = *recsiz;
        break;

    case PCK_LAC_5 :
    case UNP_LAC_5 :
    case UNP_LAC_4 :
    case UNP_LAC_3 :
    case UNP_LAC_2 :
    case UNP_LAC_1 :
	dtype = LAC;
	write_buf = 0;
	nbytes = *recsiz * 2;
	break;

    case 0 :
	*recsiz = write_buf;
    case PCK_GAC_5 :
    case UNP_GAC_5 :
    case UNP_GAC_4 :
    case UNP_GAC_3 :
    case UNP_GAC_2 :
    case UNP_GAC_1 :
	dtype = GAC;
	nbytes = *recsiz / 2;
        break;
    default :
        c_lasmsg(FALSE,"l1bwritet-irec","Invalid record size");
        return E_FAIL;
    }  /* END switch */


/*
  If write_buf is non-zero then the first half of the output buffer is
  filled.  Copy the data into the second half, write it, and reset the
  write_buf flag to zero.
*/
if ( dtype == GAC && write_buf != 0 )
    {
    for ( i = 0; i < nbytes; i++ )
	buffer[i+nbytes] = recptr[i];
	status = put_rec_tape( tun, buffer, recsiz, tape );
	if ( status != E_SUCC )
	    return status;
	write_buf = 0;
    }  /* END if */

/*
  If the data type is GAC then we need to fill the first half of the
  buffer and set the write_buf flag to recsiz.
*/
else if ( dtype == GAC )
    {
    for ( i = 0; i < nbytes; i++ )
	buffer[i] = recptr[i];
    write_buf = *recsiz;
    }  /* END else if */

/*
  Otherwise we have LAC or HRPT data so write the line.
*/
else
    {
    status = put_rec_tape( tun, recptr, recsiz, tape );
    if ( status != E_SUCC )
	return status;
    if ( nbytes > *recsiz )
	{
	status = put_rec_tape( tun, recptr+*recsiz, recsiz, tape );
	if ( status != E_SUCC )
	    return status;
	}
    }  /* END else */

return E_SUCC;

}  /* END c_l1bwritet */
