/******************************************************************************
FUNCTION:	GET_REC_TAPE

PURPOSE:	Get a record of Level 1b data from tape.

Version  Date   Programmer       Modifications
-------  -----  ---------------  ----------------------------------------------
  1.0    01/92  T.Baltzer        Development
  1.1    05/93  J.Fenno          Modified to use the new AVHRR model and the
				 new archio supports.
  1.2    09/97	D.Lloyd		 Modified for use as a support routine by
				 forcing externs to be passed as parameters.
  2.0  03/2002  D.Lloyd          Added support for byte-swapping on little
                                 endian systems.

PROJECT:	ADAPS
******************************************************************************/
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"


int get_rec_tape
(
    long *tun,			/* Unit number for tape drive		*/
    unsigned char *recptr,	/* Pointer to the Level 1b scan record	*/
    long *nbyte,		/* Record size of data to read		*/
    long *volnum,		/* Volume number			*/
    char comment[][CMLEN],	/* Description of tape			*/
    long *tape			/* Flag for tape open			*/
)
{
char    density[5];		/* default Tape density			*/
char	drive[CMLEN];           /* drive name                           */

long	inout = TREAD;          /* Read only from tape                  */
long	mode = HANDLE;		/* Handle tape errors			*/
long	rewopt = REWOFFL;       /* Rewind off-line option               */
long	skip = 0;		/* Number of files to skip              */
long	status;			/* Status variables			*/
long	zero = 0;               /* Value of zero                        */


status = c_tpread( tun, recptr, nbyte, &mode );
if ( status == *nbyte )
    return E_SUCC;

if (( status == TAPE_EOF ) || ( status == TAPE_EOV ))
   return status;

/* 
  End Of Tape or error reading tape:
  Close the tape.  If EOT and another tape has been specified to read,
  open it.
*/
else
   {
   if ( status == TAPE_EOT )
      {
      if ( strcmp( comment[*volnum], "\0" ) != 0 )
         {
	 status = c_tpclos( tun, &mode, &rewopt, &zero );
	 *tape = FALSE;
         strcpy( density, "0" );
	 rewopt = REWIND;
         status = c_tpopen( tun, &skip, &mode, &rewopt, density, &inout,
			comment[*volnum], drive );
         if ( status != TAPE_SUCC )
             return E_FAIL;
         *tape = TRUE;
         (*volnum)++;
         status = get_rec_tape( tun, recptr, nbyte, volnum, comment, tape );
         if ( status != E_SUCC )
            {
            c_lasmsg(FALSE,"get_rec_tape-span",
		     "Unable to read data spanning tape volumes");
            return E_FAIL;
            }
         }
      else
         {
         c_lasmsg(FALSE,"get_rec_tape-ueot","Unexpected End of Tape");
         return TAPE_EOT;
         }
     }
  else
     {
     c_lasmsg(FALSE,"get_rec_tape-reco","Recover data that has been read");
     return status;
     }
   }

/* Insure the data buffer is in the proper endian format
--------------------------------------------------------*/
switch ( *nbyte )
    {
    case KLM_PCK_LAC_5 : import_endian_mixed_buffer( recptr, KLMDATAREC_LP5 );
        break;
    case KLM_UNP_LAC_5 : import_endian_mixed_buffer( recptr, KLMDATAREC_LU5 );
        break;
    case KLM_UNP_LAC_4 : import_endian_mixed_buffer( recptr, KLMDATAREC_LU4 );
        break;
    case KLM_UNP_LAC_3 : import_endian_mixed_buffer( recptr, KLMDATAREC_LU3 );
        break;
    case KLM_UNP_LAC_2 : import_endian_mixed_buffer( recptr, KLMDATAREC_LU2 );
        break;
    case KLM_UNP_LAC_1 : import_endian_mixed_buffer( recptr, KLMDATAREC_LU1 );
        break;
    case KLM_PCK_GAC_5 : import_endian_mixed_buffer( recptr, KLMDATAREC_GP5 );
        break;
    case KLM_UNP_GAC_5 : import_endian_mixed_buffer( recptr, KLMDATAREC_GU5 );
        break;
    case KLM_UNP_GAC_4 : import_endian_mixed_buffer( recptr, KLMDATAREC_GU4 );
        break;
    case KLM_UNP_GAC_3 : import_endian_mixed_buffer( recptr, KLMDATAREC_GU3 );
        break;
    case KLM_UNP_GAC_2 : import_endian_mixed_buffer( recptr, KLMDATAREC_GU2 );
        break;
    case KLM_UNP_GAC_1 : import_endian_mixed_buffer( recptr, KLMDATAREC_GU1 );
        break;

    case PCK_LAC_5 : import_endian_mixed_buffer( recptr, DATAREC_LP5 ); break;
    case UNP_LAC_5 : import_endian_mixed_buffer( recptr, DATAREC_LU5 ); break;
    case UNP_LAC_4 : import_endian_mixed_buffer( recptr, DATAREC_LU4 ); break;
    case UNP_LAC_3 : import_endian_mixed_buffer( recptr, DATAREC_LU3 ); break;
    case UNP_LAC_2 : import_endian_mixed_buffer( recptr, DATAREC_LU2 ); break;
    case UNP_LAC_1 : import_endian_mixed_buffer( recptr, DATAREC_LU1 ); break;
    case PCK_GAC_5 : import_endian_mixed_buffer( recptr, DATAREC_GP5 ); break;
    case UNP_GAC_5 : import_endian_mixed_buffer( recptr, DATAREC_GU5 ); break;
    case UNP_GAC_4 : import_endian_mixed_buffer( recptr, DATAREC_GU4 ); break;
    case UNP_GAC_3 : import_endian_mixed_buffer( recptr, DATAREC_GU3 ); break;
    case UNP_GAC_2 : import_endian_mixed_buffer( recptr, DATAREC_GU2 ); break;
    case UNP_GAC_1 : import_endian_mixed_buffer( recptr, DATAREC_GU1 ); break;
    }  /* END switch */


return E_SUCC;

}  /* END get_rec_tape */
