/******************************************************************************
FUNCTION:	L1B_CK_TSTAMP

PURPOSE:	Check the first time stamp in the image to make sure it matches
		the information in the model (from the DSH and TBM/ARS headers).
		NOTE: This routine may update the model structure based on the
		      time stamp code in the first line of image data.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/97  T.Baltzer        Development
  1.1    12/98  D.Lloyd          Modifications for NOAA KLM series sat.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "level1bio.h"
#include "adapsutil.h"
#include "local_protos.h"



void l1b_ck_tstamp
(
    unsigned char *bscan, /* Pointer to the Level 1b scan line	*/
    struct MODEL *model	  /* AVHRR model information	        */
)

{
long stampyear;		/* Year contained in the time stamp		*/
long stampjday;		/* Julian day contained in the time stamp	*/
long stampstime;	/* Start time contained in the time stamp	*/
long stampshour;	/* Start hour contained in the time stamp	*/
long stampsmin;		/* Start minute contained in the time stamp	*/
long stampssec;		/* Start second contained in the time stamp	*/
long stampsfrac;	/* Start mil sec contained in the time stamp	*/


/*
  Extract the time, date from the buffer
*/
if ( model->satnum <= 14 )
    l1b_dsh_time(bscan+2,&stampyear,&stampjday,&stampstime);
else
    {
    stampyear = (long)(*(short *)&bscan[2]);
    stampyear %= 100;
    stampjday = (long)(*(short *)&bscan[4]);
    stampstime = *(long *)&bscan[8];
    }

/*
  Take into account the NBS offset that the model is using before we check the
  time and date in the time stamp.  Modify the time and date from the time stamp
  using the NBS offset value.
*/
c_cvtoffset(stampyear,stampjday,stampstime, - model->clock_err,
	    &stampyear,&stampjday,&stampstime);

if (stampyear != model->year)
    {
    c_lasmsg(FALSE,"l1bio-dis","Discrepancy in acquisition year between");
    c_lasmsg(FALSE,"l1bio-dis","header and first time stamp");
    c_lasmsg(FALSE,"l1bio-dis","header value: %d, time stamp value: %d",
	     model->year, stampyear);
    c_lasmsg(FALSE,"l1bio-dis","Updating value in model to %d",stampyear);
    model->year = stampyear;
    }
if (stampjday != model->jday)
    {
    c_lasmsg(FALSE,"l1bio-dis","Discrepancy in julian day between");
    c_lasmsg(FALSE,"l1bio-dis","header and first time stamp");
    c_lasmsg(FALSE,"l1bio-dis","header value: %d, time stamp value: %d",
	     model->jday, stampjday);
    c_lasmsg(FALSE,"l1bio-dis","Updating value in model to %d",stampjday);
    model->jday = stampjday;
    }

c_cvtmil(&stampstime,&stampshour,&stampsmin,&stampssec,&stampsfrac);

if (stampshour != model->shour)
    {
    c_lasmsg(FALSE,"l1bio-dis","Discrepancy in starting hour between");
    c_lasmsg(FALSE,"l1bio-dis","header and first time stamp");
    c_lasmsg(FALSE,"l1bio-dis","header value: %d, time stamp value: %d",
	     model->shour, stampshour);
    c_lasmsg(FALSE,"l1bio-dis","Updating value in model to %d",stampshour);
    model->shour = stampshour;
    }
if (stampsmin != model->smin)
    {
    c_lasmsg(FALSE,"l1bio-dis","Discrepancy in starting minute between");
    c_lasmsg(FALSE,"l1bio-dis","header and first time stamp");
    c_lasmsg(FALSE,"l1bio-dis","header value: %d, time stamp value: %d",
	     model->smin, stampsmin);
    c_lasmsg(FALSE,"l1bio-dis","Updating value in model to %d",stampsmin);
    model->smin = stampsmin;
    }
if (stampssec != model->ssec)
    {
    c_lasmsg(FALSE,"l1bio-dis","Discrepancy in starting second between");
    c_lasmsg(FALSE,"l1bio-dis","header and first time stamp");
    c_lasmsg(FALSE,"l1bio-dis","header value: %d, time stamp value: %d",
	     model->ssec, stampssec);
    c_lasmsg(FALSE,"l1bio-dis","Updating value in model to %d",stampssec);
    model->ssec = stampssec;
    }
if (stampsfrac != model->sfrac)
    {
    c_lasmsg(FALSE,"l1bio-dis","Discrepancy in starting millisecond between");
    c_lasmsg(FALSE,"l1bio-dis","header and first time stamp");
    c_lasmsg(FALSE,"l1bio-dis","header value: %d, time stamp value: %d",
	     model->sfrac, stampsfrac);
    c_lasmsg(FALSE,"l1bio-dis","Updating value in model to %d",stampsfrac);
    model->sfrac = stampsfrac;
    }

return;
}

