/*******************************************************************************
FUNCTION:	L1B_FILE_IN

PURPOSE:	Level-1B file interface routine for c_l1bopndr().


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T. Baltzer 	 Development
  1.1    12/98  D.Lloyd          Modifications for NOAA KLM
  1.2    05/99  T. Baltzer       Mods to give PCK_LAC_5 format precedence.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "adapssup.h"
#include "level1bio.h"
#include "local_protos.h"
#include "lasendian.h"
#include "adapsendian.h"

#define NOAALINEBUFTIME { 2, 2, 2, 1, 4, 4, 1 }
#define KLMLINEBUFTIME  { 2, 2, 2, 2, 8, 4, 1 }


int l1b_file_in
(
char infile[],		/* Input file                                   */
FILE	**in_fd,	/* File descriptor to the input  file   	*/
unsigned char hdrbuf[],	/* TBM/ARS header buffer			*/
unsigned char dshbuf[],	/* DSH header buffer				*/
unsigned char linbuf[],	/* Buffer containing first line of data		*/
long	*recsiz,	/* Record size of data read             	*/
long *dtype,		/* Data type LAC/GAC				*/
long *linfile,		/* Lines in file according to file size		*/
long *hdrflg,		/* TBM/ARS header flag (yes or no)		*/
long *nchans_tmp 	/* Number of channels (temporarily used)	*/
)

{
int	tbm_chans = 0;		/* number of channels from TBM header	*/
int	bytes_read; 		/* number of bytes read from file	*/
long	nbyte,			/* Number of bytes to read from file	*/
	pack_flg,		/* Pack flag				*/
	status;			/* Return status variable		*/
long	i;			/* loop counter				*/
long	klmflg = FALSE;		/* flag: NOAA KLM series satellite	*/
double	filesiz[1];		/* file size				*/
short   *sptr;			/* pointer to short			*/
int	count;			/* count of dataset headers		*/

char tmp[CMLEN];
char byte_31[4];


/*
  Initialize the TBM/ARS header flag to FALSE
*/
*hdrflg = FALSE;

/*
  Open the level1b image.
*/
if ((*in_fd = fopen(infile,"r")) == NULL)
    {
    c_lasmsg(FALSE,"l1bopen-open","Error opening the level1b image");
    }

/*
  Sometimes there isn't a TBM/ARS HEADER.
  Read in the first 122 bytes to see if it is the TBM/ARS HEADER.
  If 'NSS' appears at byte 31 then it is the TBM/ARS HEADER, otherwise
  we'll assume it's the Data Set Header (DSH)
*/
nbyte = TBM_HDR_SIZ;
bytes_read = fread(hdrbuf,1,nbyte,*in_fd);
if (bytes_read <= 0)
   {
   c_lasmsg(FALSE,"c_l1bopndr-read","Error reading from the level1b header");
   return(E_FAIL);
   }
if (bytes_read != nbyte)
   {
   c_lasmsg(FALSE,"c_l1bopndr-read","Read %d bytes of %d in TBM/ARS Header",
	    bytes_read,nbyte);
   return(E_FAIL);
   }


/* 
  seeing if 'NSS' is at byte 31 of the header
*/
sscanf((const char *)hdrbuf,"%30c %3c",tmp,byte_31);
 
if (byte_31[3] != '\0')
	byte_31[3] = '\0';

strcpy(tmp,"NSS");
if (strncmp(tmp,byte_31,3) == 0)
    {
    /*
    if 'NSS' is at byte 31 of the header, then the
    header is the TBM/ARS header
    */
    *hdrflg = TBM_PRESENT;
    /* also check byte 41 if it is a 'K', 'L' or 'M' then we
    actually have an ARS header not a TBM header.
    --------------------------------------------------------*/
    if ( hdrbuf[40] == 'K' || hdrbuf[40] == 'L' || hdrbuf[40] == 'M' )
        {
	*hdrflg = ARS_PRESENT;
	klmflg = TRUE;
	/* rewind the file and read the full ARS header */
	status = fseek( *in_fd, 0, SEEK_SET );
	if ( status != 0 )
	    {
	    c_lasmsg(FALSE,"c_l1bopndr-read",
	    	"Error returning to beginning of file");
	    return E_FAIL;
	    }
	nbyte = ARS_HDR_SIZ;
	bytes_read = fread(hdrbuf,1,nbyte,*in_fd);
	if ( bytes_read != nbyte )
	    {
	    c_lasmsg(FALSE,"c_l1bopndr-read",
	    	"Read %d bytes of %d in TBM/ARS Header",
	                bytes_read,nbyte);
	    return E_FAIL;
	    }
	}
    else
        {
	/* determine the number of channels fromt the header */
	if ( hdrbuf[74] == 'T' )
	    tbm_chans = 5;
	else
	    {
	    tbm_chans = 0;
	    for ( i = 97; i < 102; i++ )
		if ( hdrbuf[i] != 0 )
		    tbm_chans++;
	    }
	}
    }
else
    {
    /* 
      If this wasn't the TBM/ARS HEADER, go back to the beginning
      of the file to read the DSH HEADER.
    */
    *hdrflg = FALSE;
    status = fseek(*in_fd,0,SEEK_SET);
    if (status != 0)
	{
	c_lasmsg(FALSE,"c_l1bopndr-read","Error returning to beginning of file");
	return(E_FAIL);
	}
    bytes_read = fread(dshbuf,1,64,*in_fd);
    if ( bytes_read != 64 )
	{
	c_lasmsg(FALSE,"c_l1bopndr-read",
	    "Error reading the Data Set Header");
	return(E_FAIL);
	}
    if ( strncmp( tmp, (const char *)&dshbuf[22], 3 ) == 0 )
        {
	klmflg = TRUE;
	fseek(*in_fd,0,SEEK_SET);
	}
    }


/*
  Determine the number of bytes per record
*/
if ( !klmflg )
    {
    if (l1b_format(infile,*hdrflg,&pack_flg,nchans_tmp,dtype,linfile)
    		!= E_SUCC)
	{
	/* This was modified as a temporary fix.  It previously just
	returned failure.  l1b_format() was modified as well so that
	the first record size it checks is PCK_LAC_5 which sets pack_flg
	and nchans_tmp if PCK_LAC_5 matches.  If a subsequent record size
	matches l1b_format() returns failure.  We give priority to
	PCK_LAC_5 by checking the returned values here.  If PCK_LAC_5
	didn't match we return failure
	---------------------------------------------------------------*/
	if ( pack_flg && (*nchans_tmp == 5 ))
	    c_lasmsg(FALSE,"c_l1bsup-form",
	             "Precedence given to Packed-LAC-5-band");
	else
	    return E_FAIL;
	}
    c_l1brecsiz(*dtype,pack_flg,klmflg,*nchans_tmp,&nbyte);
    }
else
    {
    if (l1b_format(infile,*hdrflg,&pack_flg,nchans_tmp,dtype,linfile)
    		!= E_SUCC)
	{
	if ( *hdrflg == ARS_PRESENT )
	    {
	    c_lasmsg(FALSE,"l1b_file_in-size","Using ARS for record size");
	    sscanf( (const char *)&hdrbuf[181], "%ld %ld", &nbyte, linfile );
	    c_getsizes((char (*)[])infile,1,filesiz);
	    filesiz[0] -= (double)ARS_HDR_SIZ;
	    filesiz[0] -= (double)nbyte;
	    filesiz[0] /= (double)nbyte;
	    if ( *linfile != (long)filesiz[0] )
	        *linfile = (long)filesiz[0];
	    switch ( hdrbuf[34] )
	        {
		case 'H' :
		case 'L' : *dtype = LAC; break;
		case 'G' : *dtype = GAC; break;
		}
	    *nchans_tmp = 0;
	    for ( i = 97; i < 102; i++ )
	        if ( hdrbuf[i] == 'Y' )
		    (*nchans_tmp)++;
	    }
	else
	    {
	    c_lasmsg(FALSE,"l1b_file_in-size",
	        "Attempting to use DSH for record size");
	    bytes_read = fread(dshbuf,1,140,*in_fd);
	    if ( bytes_read != 140 )
	        {
		c_lasmsg(FALSE,"c_l1bopndr-read",
		    "Error reading the Data Set Header");
		return(E_FAIL);
		}
#ifdef LAS_LITTLE_ENDIAN
            {
	    char temp_char = dshbuf[10];
	    dshbuf[10] = dshbuf[11];
	    dshbuf[11] = temp_char;
	    temp_char = dshbuf[128];
	    dshbuf[128] = dshbuf[129];
	    dshbuf[129] = temp_char;
	    }
#endif
	    nbyte = (long)(*(short *)&dshbuf[10]);
	    *linfile = (long)(*(short *)&dshbuf[128]);
	    c_getsizes((char (*)[])infile,1,filesiz);
	    if ( (long)filesiz[0] % nbyte != 0 )
	        {
		if ( (long)filesiz[0] % (*linfile + 1) == 0 )
		    {
		    c_lasmsg(FALSE,"l1b_file_in-size",
		     "Discrepancy in DSH record size, using number of records");
		    nbyte = (long)filesiz[0] / (*linfile + 1);
		    }
		else
		    {
		    c_lasmsg(FALSE,"l1b_file_in-size",
			"Unable to determine record size");
		    return E_FAIL;
		    }
		}
	    filesiz[0] -= (double)nbyte;
	    filesiz[0] /= (double)nbyte;
	    if ( *linfile != (long)filesiz[0] )
	        *linfile = (long)filesiz[0];
	    switch ( dshbuf[26] )
	        {
		case 'H' :
		case 'L' : *dtype = LAC; break;
		case 'G' : *dtype = GAC; break;
		}
	    status = fseek(*in_fd,0,SEEK_SET);
	    if (status != 0)
		{
		c_lasmsg(FALSE,"c_l1bopndr-seek",
		    "Error seeking in input file");
		return(E_FAIL);
		}
	    }
	}
    else
	c_l1brecsiz(*dtype,pack_flg,klmflg,*nchans_tmp,&nbyte);
    }

/*
  Read in the DSH header
*/
bytes_read = fread(dshbuf,1,nbyte,*in_fd);
if (bytes_read <= 0)
   {
   c_lasmsg(FALSE,"c_l1bopndr-read","Error reading the Data Set Header");
   return(E_FAIL);
   }
if (bytes_read != nbyte)
   {
   c_lasmsg(FALSE,"c_l1bopndr-read","Read %d bytes of %d in Data Set Header",
	    bytes_read,nbyte);
   return(E_FAIL);
   }
*recsiz = bytes_read;


/* Insure the DSH is in the proper endian format
------------------------------------------------*/
if ( klmflg )
    {
    import_endian_mixed_buffer( dshbuf, KLMDATASETHEADER );
    }
else
    {
    import_endian_mixed_buffer( dshbuf, NOAADATASETHEADER );
    }


/*
  Read the second Data Set Header record - dummy record
  (pre-KLM LAC/HRPT data only)
*/
if (*dtype != GAC && !klmflg)
    {
    bytes_read = fread(linbuf,1,nbyte,*in_fd);
    if (bytes_read <= 0)
   	{
	c_lasmsg(FALSE,"c_l1bopndr-dumhdr",
		 "Error reading the dummy header record");
	return(E_FAIL);
   	}
	if (bytes_read != nbyte)
   	{
	   c_lasmsg(FALSE,"c_l1bopndr-read",
		    "Read %d bytes of %d in 2nd Data Set Header",bytes_read,
		    nbyte);
	   return(E_FAIL);
   	}

    }

/*
  Read any additional header records for KLMNN' data
*/
if ( klmflg )
    {
    sptr = (short *)&dshbuf[14];
    count = *sptr;
    if ( count > 1 )
        {
	for ( i = 1; i < count; i++ )
	    {
	    bytes_read = fread(linbuf,1,nbyte,*in_fd);
	    if (bytes_read <= 0)
		{
		c_lasmsg(FALSE,"c_l1bopndr-dumhdr",
			 "Error reading the extra header record");
		return(E_FAIL);
		}
		if (bytes_read != nbyte)
		{
		   c_lasmsg(FALSE,"c_l1bopndr-read",
			    "Read %d bytes of %d in extra Data Set Header",
			    bytes_read, nbyte);
		   return(E_FAIL);
		}
	    }
	}
    }


/* 
  Now read the first line of data (used to verify the first time stamp)
*/
bytes_read = fread(linbuf,1,nbyte,*in_fd);
if (bytes_read != nbyte)
    {
    c_lasmsg(FALSE,"c_l1bopndr-read", "Read %d bytes of %d in first line",
	     bytes_read, nbyte);
    return(E_FAIL);
    }

/* Insure the line time portion is in the proper endian format
--------------------------------------------------------------*/
if ( klmflg )
    {
    import_endian_mixed_buffer( linbuf, KLMLINEBUFTIME );
    }
else
    {
    import_endian_mixed_buffer( linbuf, NOAALINEBUFTIME );
    }


/*
  Now backspace in the file to the first line of data
*/
status = fseek(*in_fd,-bytes_read,1);
if (status != 0)
    {
    c_lasmsg(FALSE,"c_l1bopndr-seek", "Error seeking in input file");
    return(E_FAIL);
    }


return(E_SUCC);
}
