/*******************************************************************************
FUNCTION:	L1B_FORMAT

PURPOSE:	Determine the format (packed/unpacked, number of bands,
		LAC/GAC data) and number of lines of a Level-1B file
		based on the file size.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
1.0	 10/97	T. Baltzer	 Development 
1.1      12/98  D.Lloyd          Modifications for NOAA KLM series sats.
1.2	 05/99	T. Baltzer	 Mods to give PCK_LAC_5 format precedence.
1.3	 05/99	T. Baltzer	 Add rest of KLM format types,
				 check header type before format type.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "avhrr.h"
#include "adapssup.h"
#include "level1bio.h"



int l1b_format
(
    char infile[CMLEN],		/* Input file name (input)	*/
    long hdr_flg,		/* TBM header present? (input)	*/
    long *pack_flg,		/* Data packed? (output)	*/
    long *nchans,		/* Number of channels (output)	*/
    long *dtype,		/* Data type LAC/GAC (output)	*/
    long *linfile		/* Number of lines in image (output) */
)

{

double  file_size[1];   /* size of infile                               */


/* Get the size of the file on disk */
c_getsizes((char (*)[])infile,1,file_size);

/* If the tbm/ars header is present subtract the size of it */
if (hdr_flg == TBM_PRESENT)
    file_size[0] -= TBM_HDR_SIZ;
else if ( hdr_flg == ARS_PRESENT )
    file_size[0] -= ARS_HDR_SIZ;

/* Initialize the number of channels to zero to signify that	*/
/* the format has not been yet determined.			*/
*nchans = 0;

/* Determine the Level-1B format based solely on the size of the file.	*/
/* If the file size is a multiple of TWO Level-1B record sizes, error	*/
/* out since the format cannot be uniquely determined.			*/

/* First try noaa KLM format */
if ((((long)file_size[0]) % KLM_PCK_LAC_5) == 0)
    {
    *pack_flg = TRUE;
    *nchans = 5;
    *dtype = LAC;
    *linfile = (long)file_size[0] / KLM_PCK_LAC_5 - 1;
    return E_SUCC;
    }

/* Next try the pre-KLM sizes.						*/
if ( hdr_flg != ARS_PRESENT )
    {
    /* The following if statement was moved from the end of this block
    as a temporary fix to insure that PCK_LAC_5 is checked and set first
    If any other record sizes match, an error message is printed and
    we return failure.  The calling function can check to see if the
    pack_flg and nchans have been set for PCK_LAC_5 and continue
    processing.  This gives true priority to PCK_LAC_5.  When this was
    at the bottom of the block if two record sizes divide evenly the
    function would return failure even though PCK_LAC_5 was correct.
    -------------------------------------------------------------------*/
    if ((((long)file_size[0]) % PCK_LAC_5) == 0)
	{
#if 0
/* This is temporarily removed pending a proper fix.  See above comment */
	if (*nchans)
	    {
	    /* If a non-unique record size has been encountered, give	*/
	    /* PCK_LAC_5 precedence since it is the most commonly 	*/
	    /* requested format.  Even if this is an incorrect 		*/
	    /* assumption, error checking is later performed against	*/
	    /* the AVHRR model to catch such an error.			*/
	    c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    c_lasmsg(FALSE,"c_l1bsup-form",
		     "Precedence given to Packed-LAC-5-band");
	    }
#endif
	*pack_flg = TRUE;
	*nchans = 5;
	*dtype = LAC;
	*linfile = (long)file_size[0] / PCK_LAC_5 / 2 - 1;
	}
    if ((((long)file_size[0]) % UNP_LAC_1) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 1;
	*dtype = LAC;
	*linfile = (long)file_size[0] / UNP_LAC_1 / 2 - 1;
	}
    if ((((long)file_size[0]) % UNP_LAC_2) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 2;
	*dtype = LAC;
	*linfile = (long)file_size[0] / UNP_LAC_2 / 2 - 1;
	}
    if ((((long)file_size[0]) % UNP_LAC_3) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 3;
	*dtype = LAC;
	*linfile = (long)file_size[0] / UNP_LAC_3 / 2 - 1;
	}
    if ((((long)file_size[0]) % UNP_LAC_4) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
    	    }
	*pack_flg = FALSE;
	*nchans = 4;
	*dtype = LAC;
	*linfile = (long)file_size[0] / UNP_LAC_4 / 2 - 1;
	}
    if ((((long)file_size[0]) % UNP_LAC_5) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 5;
	*dtype = LAC;
	*linfile = (long)file_size[0] / UNP_LAC_5 / 2 - 1;
	}
    if ((((long)file_size[0]) % UNP_GAC_1) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 1;
	*dtype = GAC;
	*linfile = (((long)file_size[0] / UNP_GAC_1) - 1) * 2;
	}
    if ((((long)file_size[0]) % UNP_GAC_2) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 2;
	*dtype = GAC;
	*linfile = (((long)file_size[0] / UNP_GAC_2) - 1) * 2;
	}
    if ((((long)file_size[0]) % UNP_GAC_3) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 3;
	*dtype = GAC;
	*linfile = (((long)file_size[0] / UNP_GAC_3) - 1) * 2;
	}
    if ((((long)file_size[0]) % UNP_GAC_4) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 4;
	*dtype = GAC;
	*linfile = (((long)file_size[0] / UNP_GAC_4) - 1) * 2;
	}
    if ((((long)file_size[0]) % UNP_GAC_5) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 5;
	*dtype = GAC;
	*linfile = (((long)file_size[0] / UNP_GAC_5) - 1) * 2;
	}
    if ((((long)file_size[0]) % PCK_GAC_5) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = TRUE;
	*nchans = 5;
	*dtype = GAC;
	*linfile = (((long)file_size[0] / PCK_GAC_5) - 1) * 2;
	}
    }


/* Now try the KLM sizes.  */

if (hdr_flg != TBM_PRESENT)
    {
    if ((((long)file_size[0]) % KLM_UNP_LAC_1) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 1;
	*dtype = LAC;
	*linfile = (long)file_size[0] / KLM_UNP_LAC_1 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_LAC_2) == 0)
	{
	if (*nchans)
	    {
	    c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 2;
	*dtype = LAC;
	*linfile = (long)file_size[0] / KLM_UNP_LAC_2 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_LAC_3) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 3;
	*dtype = LAC;
	*linfile = (long)file_size[0] / KLM_UNP_LAC_3 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_LAC_4) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 4;
	*dtype = LAC;
	*linfile = (long)file_size[0] / KLM_UNP_LAC_4 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_LAC_5) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 5;
	*dtype = LAC;
	*linfile = (long)file_size[0] / KLM_UNP_LAC_5 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_GAC_1) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 1;
	*dtype = GAC;
	*linfile = (long)file_size[0] / KLM_UNP_GAC_1 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_GAC_2) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 2;
	*dtype = GAC;
	*linfile = (long)file_size[0] / KLM_UNP_GAC_2 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_GAC_3) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 3;
	*dtype = GAC;
	*linfile = (long)file_size[0] / KLM_UNP_GAC_3 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_GAC_4) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 4;
	*dtype = GAC;
	*linfile = (long)file_size[0] / KLM_UNP_GAC_4 - 1;
	}
    if ((((long)file_size[0]) % KLM_UNP_GAC_5) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = FALSE;
	*nchans = 5;
	*dtype = GAC;
	*linfile = (long)file_size[0] / KLM_UNP_GAC_5 - 1;
	}
    if ((((long)file_size[0]) % KLM_PCK_GAC_5) == 0)
	{
	if (*nchans)
	    {
            c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    return(E_FAIL);
	    }
	*pack_flg = TRUE;
	*nchans = 5;
	*dtype = GAC;
	*linfile = (long)file_size[0] / KLM_PCK_GAC_5 - 1;
	}
    if ((((long)file_size[0]) % KLM_PCK_LAC_5) == 0)
	{
	if (*nchans)
	    {
	    /* If a non-unique record size has been encountered, give	*/
	    /* KLM_PCK_LAC_5 precedence since it is the most commonly 	*/
	    /* requested format.  Even if this is an incorrect 		*/
	    /* assumption, error checking is later performed against	*/
	    /* the AVHRR model to catch such an error.			*/
	    c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
	    c_lasmsg(FALSE,"c_l1bsup-form",
		     "Precedence given to KLM-Packed-LAC-5-band");
	    }
	*pack_flg = TRUE;
	*nchans = 5;
	*dtype = LAC;
	*linfile = (long)file_size[0] / KLM_PCK_LAC_5 - 1;
	}
    }

/* If after all of that, it didn't match anywhere, failure! */
if (*nchans == 0)
    {
    c_lasmsg(FALSE,"c_l1bsup-form","Error determining Level-1B format");
    return(E_FAIL);
    }

return(E_SUCC);

}
