/******************************************************************************
FUNCTION:	L1B_LOAD_ARS

PURPOSE:	Load the model, etc. with information from the ARS header
		record of the Level 1b image.


Version	Date	Programmer	Modifications
-------	-----	---------------	-----------------------------------------------
  1.0	12/98	D.Lloyd		Initial development (stolen from l1b_load_tbm
  				and modified for NOAA KLM)
  1.1   10/00   D.Lloyd		Added NOAA 16 support
  1.2 04/2002   D.Lloyd		Added NOAA 17 support

ALGORITHM REFERENCES:
	1. "NOAA KLM Users Guide" for the record structure of the
	ARS header record.

PROJECT:	ADAPS
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "level1bio.h"



int l1b_load_ars
(
    unsigned char ars[],	/* ARS header record			*/
    long totchdsh,		/* Total channels from DSH header	*/
    struct MODEL *model		/* AVHRR model structure		*/
)

{
char	beg_lat[4];		/* beginning latitude			*/
char	dform[5];      		/* data format GAC LAC HRPT             */
char	end_lat[4];		/* Ending latitude			*/
char	jday[4];		/* Julian date of the pass		*/
char	jyear[3];		/* Year of the pass			*/
char	loc[3];			/* Location data generated at		*/

long	dtype_from_dsh;		/* Data type from DSH header		*/
long	i; 			/* counter				*/
long	jday_from_dsh;		/* Julian day from DSH header		*/
long	tpchan[MOSTBND];	/* channels stored in the file		*/
long	year_from_dsh;		/* Year from DSH header			*/

/* The following values are presently unused, but may be needed later
--------------------------------------------------------------------*/
char    direction[11];		/* Direction of the pass                */
char    phr[3];			/* Start hour of the pass               */
char    pmin[3];		/* Start minute of the pass             */
char    sat[2];			/* Code for space craft id              */
char    satid[8];		/* Space craft ID                       */
char    source[30];		/* Source of data                       */


/* Determine the space craft id
-------------------------------*/
strncpy(sat,(const char *)(ars+40),1);
switch (sat[0])
    {
    case 'K':	strcpy(satid,"noaa-15");	break;
    case 'L':	strcpy(satid,"noaa-16");	break;
    case 'M':	strcpy(satid,"noaa-17");	break;
    case 'N':	strcpy(satid,"noaa-18");	break;
    case 'P':   strcpy(satid,"noaa-19");        break;
    default:	strcpy(satid,"unknown");	break;
    }


/* Retrieve the year of the pass
--------------------------------*/
strncpy(jyear,(const char *)(ars+43),2);
jyear[2] = '\0';


/* Retrieve the julian date and convert it to gregorian
-------------------------------------------------------*/
strncpy(jday,(const char *)(ars+45),3);
jday[3] = '\0';


/* Save the julian year and day within the model structure
Save the gregorian month and day within the structure, too.
Check to make sure the DSH and ARS don't disagree.
-----------------------------------------------------------*/
year_from_dsh = model->year;
jday_from_dsh = model->jday;

sscanf(jyear,"%ld",&model->year);
sscanf(jday,"%ld",&model->jday);
c_julgre(&model->year, &model->jday, &model->month, &model->day);
if ((year_from_dsh != model->year) || (jday_from_dsh != model->jday))
   {
   c_lasmsg(FALSE,"c_l1bsup-hdrs","Discrepancy in ARS and DSH headers");
   return(E_FAIL);
   }


/* Retrieve the start hour of the pass
--------------------------------------*/
strncpy(phr,(const char *)(ars+50),2);
phr[2] = '\0';


/* Retrieve the start minute of the pass
----------------------------------------*/
strncpy(pmin,(const char *)(ars+52),2);
pmin[2] = '\0';


/* Determine the data creation source.  We have CEOS ids set up
for SFL, NGC and NOA; for the other valid codes, eg SO and WE,
we have no associated CEOS id.  See NOAA KLM Useres Guide 
sections 8.2 and 8.3.1.2.
-----------------------------------------------------------------*/
strncpy(loc,(const char *)(ars+70),2);
loc[2] = '\0';
sprintf(source,"Level 1b / location code: %s",loc);
if (strcmp(loc,"SF") == 0)
    strcpy(model->station,"SFL");
else if (strcmp(loc,"GC") == 0)
    strcpy(model->station,"NGC");
else if (strcmp(loc,"WI") == 0)
    strcpy(model->station,"NOA");


/* Retrieve the beginning latitude
----------------------------------*/
strncpy(beg_lat,(const char *)(ars+75),3);
beg_lat[3] = '\0';


/* Retrieve the ending latitude
-------------------------------*/
strncpy(end_lat,(const char *)(ars+78),3);
end_lat[3] = '\0';


/* Determine the direction of the pass
--------------------------------------*/
if ( ars[146] == 'A' )
    strcpy(direction,"ascending");
else if ( ars[146] == 'D' )
    strcpy(direction,"descending");
else
    strcpy(direction,"unknown");


/* Get the data type (GAC LAC HRPT)
Check to make sure the DSH and ARS don't disagree.
----------------------------------------------------*/
dtype_from_dsh = model->dtype;
strncpy(dform,(const char *)(ars+34),4);
dform[4] = '\0';
if ((strcmp(dform,"GAC ") == 0) || (strcmp(dform,"GHRR") == 0))
   model->dtype = GAC;
else if ((strcmp(dform,"LAC ") == 0) || (strcmp(dform,"LHRR") == 0))
   model->dtype = LAC;
else
   model->dtype = HRPT;
if (model->dtype != dtype_from_dsh)
    {
    c_lasmsg(FALSE,"c_l1bsup-hdrs","Discrepancy in ARS and DSH headers");
    return(E_FAIL);
    }


/* Store the number of samples per scan in the model structure
--------------------------------------------------------------*/
if (model->dtype == GAC)
   model->nsamples = GAC_SAMPS;
else
   model->nsamples = LAC_SAMPS;


/* Get the channels in file, if all channels are on the tape 'T'
(total) will be set, else 'S' (select copy) will be set
----------------------------------------------------------------*/
if (ars[74] == 'T')
   {
   for (i = 0; i < MOSTBND; i++)
      tpchan[i] = TRUE;
   }
else
   {
   for (i = 0; i < MOSTBND; i++)
      if ( ars[97+i] == 'Y' )
          tpchan[i] = 1;
      else
          tpchan[i] = 0;
   }


/* NOTE: If 'S' was set but none of the channels were set we will assume
that we cannot tell from the ARS header which channels are in the file.
-------------------------------------------------------------------------*/


/* Load the model with the channel information
----------------------------------------------*/
model->nchans = 0;
for (i = 0; i < MOSTBND; i++)
    {
    if (tpchan[i])
	{
	model->chans[i] = i + 1;
	(model->nchans)++;
	}
    else
	model->chans[i] = 0;
    }


/* If the model now shows no channels, use the # channels from the DSH
----------------------------------------------------------------------*/
if (model->nchans == 0)
    model->nchans = totchdsh;

return(E_SUCC);
}
