/*******************************************************************************
FUNCTION:	L1B_LOAD_DSH

PURPOSE:	Load the model, etc. with information from the Data Set Header
		record of the Level 1b file.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T.Baltzer        Development (stole from INGEST1B code)
  1.1    12/98  D.Lloyd          Modifications for NOAA KLM series sats.
  1.2    10/00  D.Lloyd		 Added support for NOAA 16
  1.3  04/2002  D.Lloyd		 Added support for NOAA 17

PROJECT:	ADAPS
*******************************************************************************/
#include <math.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"
#include "level1bio.h"
#include "local_protos.h"



int l1b_load_dsh
(
    unsigned char dshbuf[],	/* Input buffer from file		*/
    long bytesread,		/* number of bytes read into dshbuf	*/
    long *totchdsh,		/* Total number of bands according to DSH*/
    long *goodlines,		/* total good lines in the input image	*/
    struct MODEL *model		/* AVHRR model information		*/
)
{

long	eyear,			/* ending year of acquire		*/
	eday,			/* ending Julian day of acquire		*/
	etime,			/* ending GMT time in milliseconds	*/
	sday,			/* starting Julian day of acquire	*/
	stime,			/* starting GMT time in milliseconds	*/
	syear,			/* starting year of acquire		*/
	time_diff;		/* time difference in milliseconds	*/

/*
  Determine the data format from the DSH header.  The number of samples
  within the image can then also be determined.
*/
model->dtype = ((dshbuf[1] & 0xf0) >> 4);
if (model->dtype == LAC)
    {
    model->nsamples = LAC_SAMPS;
    }
else if (model->dtype == GAC)
    {
    model->nsamples = GAC_SAMPS;
    }
else if (model->dtype == HRPT)
    {
    model->nsamples = LAC_SAMPS;
    }
else
    {
    c_lasmsg(FALSE,"c_l1bsup-dform","Invalid data format");
    return(E_FAIL);
    }


if (model->dtype == GAC)		/* GAC data */
   {
   switch (bytesread)
      {
      case UNP_GAC_1:	*totchdsh = 1;	break;
      case UNP_GAC_2:	*totchdsh = 2;	break;
      case UNP_GAC_3:	*totchdsh = 3;	break;
      case UNP_GAC_4:	*totchdsh = 4;	break;
      case UNP_GAC_5:	*totchdsh = 5;	break;
      case PCK_GAC_5:	*totchdsh = 5;	break;
      case KLM_UNP_GAC_1:	*totchdsh = 1;	break;
      case KLM_UNP_GAC_2:	*totchdsh = 2;	break;
      case KLM_UNP_GAC_3:	*totchdsh = 3;	break;
      case KLM_UNP_GAC_4:	*totchdsh = 4;	break;
      case KLM_UNP_GAC_5:	*totchdsh = 5;	break;
      case KLM_PCK_GAC_5:	*totchdsh = 5;	break;
      default:
	 c_lasmsg(FALSE,"c_l1bsup-dshdr",
		  "Error reading the data set header record");
	 return(E_FAIL);
      }
   }
else					/* LAC data */
   {
   switch (bytesread)
      {
      case UNP_LAC_1:	*totchdsh = 1;	break;
      case UNP_LAC_2:	*totchdsh = 2;	break;
      case UNP_LAC_3:	*totchdsh = 3;	break;
      case UNP_LAC_4:	*totchdsh = 4;	break;
      case UNP_LAC_5:	*totchdsh = 5;	break;
      case PCK_LAC_5:	*totchdsh = 5;	break;
      case KLM_UNP_LAC_1:	*totchdsh = 1;	break;
      case KLM_UNP_LAC_2:	*totchdsh = 2;	break;
      case KLM_UNP_LAC_3:	*totchdsh = 3;	break;
      case KLM_UNP_LAC_4:	*totchdsh = 4;	break;
      case KLM_UNP_LAC_5:	*totchdsh = 5;	break;
      case KLM_PCK_LAC_5:	*totchdsh = 5;	break;
      default:
	 c_lasmsg(FALSE,"c_l1bsup-dshdr",
		  "Error reading the Data Set Header record");
	 return(E_FAIL);
      }
   }

/*
  Determine the number of lines in the entire input image.
  Put this line count in the model.
*/
l1b_dsh_time(dshbuf+2,&syear,&sday,&stime);	/* get start time	*/
l1b_dsh_time(dshbuf+10,&eyear,&eday,&etime);	/* get ending time	*/
if (sday == eday)
   time_diff = etime - stime;			/* don't straddle days	*/
else
   time_diff = etime + MIL_PER_DAY - stime;	/* do straddle days	*/
if (model->dtype == GAC)
   model->nlines = ((long) ceil((double)time_diff * GAC_PER_SEC / MIL_PER_SEC));
else
   model->nlines = ((long) ceil((double)time_diff * LAC_PER_SEC / MIL_PER_SEC));

/*
  Return the number of good lines (lines stored in the Level-1b file)
*/
*goodlines = (long)(*(short *)(&dshbuf[8]));

/*
  The day and year need to be determined from the DSH header.  Save the
  julian year and day within the AVHRR model structure.  Save the
  gregorian month and day within the structure, too.
*/
model->year = syear;
model->jday = sday;
c_julgre(&model->year, &model->jday, &model->month, &model->day);


/*
  Find the satellite number by reading the spacecraft id from the data
  set header record and converting it (Conversion table in NOAA Polar
  Orbiter Data Users Guide)
  NOTE:
  Satellite number 1 refers to NOAA-11 (Nov 88 - Apr 95) and
			       TIROSN  (prior to Nov 88)
  We always assume it refers to NOAA-11.
  Satellite number 2 refers to NOAA-6 and NOAA-13.  We always assume it
  refers to NOAA-6 since NOAA-13 malfunctioned at launch.
  Satellite number 4 refers to NOAA-7 prior to 98 and NOAA-15 (K) after
  Satellite number 2 refers to NOAA-6 prior to 00 and NOAA-16 (L) after
  Note: the NOAA KLM changes are not absolutely necessary since this
  function won't be called for them (see l1b_load_klmdsh.c)
*/
model->satnum = (long) dshbuf[0];

switch (model->satnum)
   {
   case 1:	model->satnum = 11;	break;
   case 2:
        if ( c_4digityr( model->year ) >= 2000 )
	    model->satnum = 16;
	else
	    model->satnum = 6;
	break;
   case 3:	model->satnum = 14;	break;
   case 4:
  	if ( c_4digityr( model->year ) >= 1998 )
	    model->satnum = 15;
	else
	    model->satnum = 7;
	break;
   case 5:	model->satnum = 12;	break;
   case 6: model->satnum = 8; break;
   case 7:
  	if ( c_4digityr( model->year ) >= 2002 )
	    model->satnum = 17;
	else
             model->satnum = 9;
        break;
   case 8:	model->satnum = 10;	break;
   default:
      c_lasmsg(FALSE,"c_l1bsup-satnm",
	       "Error retrieving satellite number");
      break;
   }
/*
  Convert the start (end) time in milliseconds to hours, minutes
  seconds and milliseconds (put in the AVHRR model structure)
*/
c_cvtmil(&stime,&model->shour,&model->smin,&model->ssec,&model->sfrac);
c_cvtmil(&etime,&model->ehour,&model->emin,&model->esec,&model->efrac);

/*
  Adjust the start (end) time in the model based on the NBS offset
*/
if (l1b_tim_cor(model) != E_SUCC)
    return(E_FAIL);

return(E_SUCC);
}
