/*******************************************************************************
FUNCTION:	L1B_LOAD_KLMDSH

PURPOSE:	Load the model, etc. with information from a KLM Data Set
		Header record of the Level 1b file.


Version	Date	Programmer	Modifications
-------	-----	---------------	-----------------------------------------------
  1.0	12/98	D.Lloyd		Development (stole from INGEST1B code)
  1.1   10/00   D.Lloyd		Added NOAA 16 support
  1.2 04/2002   D.Lloyd		Added NOAA 17 support

PROJECT:	ADAPS
*******************************************************************************/
#include <math.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"
#include "level1bio.h"
#include "local_protos.h"



int l1b_load_klmdsh
(
    unsigned char dshbuf[],	/* Input buffer from file		*/
    long bytesread,		/* number of bytes read into dshbuf	*/
    long *totchdsh,		/* Total number of bands according to DSH*/
    long *goodlines,		/* total good lines in the input image	*/
    struct MODEL *model		/* AVHRR model information		*/
)

{

long	eday;			/* ending Julian day of acquire		*/
long	etime;			/* ending GMT time in milliseconds	*/
long	sday;			/* starting Julian day of acquire	*/
long	stime;			/* starting GMT time in milliseconds	*/
long	syear;			/* starting year of acquire		*/
long	time_diff;		/* time difference in milliseconds	*/


/* Determine the data format from the DSH header.  The number of samples
within the image can then also be determined.
------------------------------------------------------------------------*/
model->dtype = (long)(*(short *)(&dshbuf[76]));
if (model->dtype == LAC)
    {
    model->nsamples = LAC_SAMPS;
    }
else if (model->dtype == GAC)
    {
    model->nsamples = GAC_SAMPS;
    }
else if (model->dtype == HRPT)
    {
    model->nsamples = LAC_SAMPS;
    }
else
    {
    c_lasmsg(FALSE,"c_l1bsup-dform","Invalid data format");
    return E_FAIL;
    }


if (model->dtype == GAC)		/* GAC data */
   {
   switch (bytesread)
      {
      case KLM_UNP_GAC_1:	*totchdsh = 1;	break;
      case KLM_UNP_GAC_2:	*totchdsh = 2;	break;
      case KLM_UNP_GAC_3:	*totchdsh = 3;	break;
      case KLM_UNP_GAC_4:	*totchdsh = 4;	break;
      case KLM_UNP_GAC_5:	*totchdsh = 5;	break;
      case KLM_PCK_GAC_5:	*totchdsh = 5;	break;
      default:
	 c_lasmsg(FALSE,"c_l1bsup-dshdr",
		  "Error reading the data set header record");
	 return E_FAIL;
      }
   }
else					/* LAC data */
   {
   switch (bytesread)
      {
      case KLM_UNP_LAC_1:	*totchdsh = 1;	break;
      case KLM_UNP_LAC_2:	*totchdsh = 2;	break;
      case KLM_UNP_LAC_3:	*totchdsh = 3;	break;
      case KLM_UNP_LAC_4:	*totchdsh = 4;	break;
      case KLM_UNP_LAC_5:	*totchdsh = 5;	break;
      case KLM_PCK_LAC_5:	*totchdsh = 5;	break;
      default:
	 c_lasmsg(FALSE,"c_l1bsup-dshdr",
		  "Error reading the Data Set Header record");
	 return E_FAIL;
      }
   }


/* Determine the number of lines in the entire input image.
Put this line count in the model.
-----------------------------------------------------------*/
syear = (long)(*(short *)(&dshbuf[84]));
sday = (long)(*(short *)(&dshbuf[86]));
stime = *(long *)(&dshbuf[88]);
eday = (long)(*(short *)(&dshbuf[98]));
etime = *(long *)(&dshbuf[100]);
if (sday == eday)
   time_diff = etime - stime;			/* don't straddle days	*/
else
   time_diff = etime + MIL_PER_DAY - stime;	/* do straddle days	*/
if (model->dtype == GAC)
   model->nlines = ((long) ceil((double)time_diff * GAC_PER_SEC / MIL_PER_SEC));
else
   model->nlines = ((long) ceil((double)time_diff * LAC_PER_SEC / MIL_PER_SEC));


/* Return the number of good lines (lines stored in the Level-1b file)
----------------------------------------------------------------------*/
*goodlines = (long)(*(short *)(&dshbuf[128]));


/* The day and year need to be determined from the DSH header.  Save the
julian year and day within the AVHRR model structure.  Save the
gregorian month and day within the structure, too.
------------------------------------------------------------------------*/
model->year = syear % 100;
model->jday = sday;
c_julgre(&model->year, &model->jday, &model->month, &model->day);


/* Find the satellite number by reading the spacecraft id from the data
set header record and converting it
NOTE: Satellite number 4 refers to NOAA-K (15)
-----------------------------------------------------------------------*/
model->satnum = (long)(*(short *)(&dshbuf[72]));

switch (model->satnum)
   {
   case 4:
	model->satnum = 15;
	break;
   case 2:
        model->satnum = 16;
	break;
   case 6:
        model->satnum = 17;
	break;
   case 7:
        model->satnum = 18;
	break;
   case 8:
        model->satnum = 19;
        break;
   default:
      c_lasmsg(FALSE,"c_l1bsup-satnm",
	       "Error retrieving satellite number");
      break;
   }


/* Convert the start (end) time in milliseconds to hours, minutes
seconds and milliseconds (put in the AVHRR model structure)
------------------------------------------------------------------*/
c_cvtmil(&stime,&model->shour,&model->smin,&model->ssec,&model->sfrac);
c_cvtmil(&etime,&model->ehour,&model->emin,&model->esec,&model->efrac);


/* Adjust the start (end) time in the model based on the NBS offset
-------------------------------------------------------------------*/
if (l1b_tim_cor(model) != E_SUCC)
    return E_FAIL;

return E_SUCC;
}
