/******************************************************************************
FUNCTION:	L1B_LOAD_TBM

PURPOSE:	Load the model, etc. with information from the TBM header
		record of the Level 1b image.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T.Baltzer        Development

ALGORITHM REFERENCES:
	1. "NOAA Polar Orbiter Data Users Guide" section 2.1.1 for the record
	   structure of the TBM header record.

PROJECT:	ADAPS
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "level1bio.h"



int l1b_load_tbm
(
    unsigned char tbmbuf[],	/* TBM header record		*/
    long totchdsh,		/* Total channels from DSH header*/
    struct MODEL *model		/* AVHRR model structure	*/
)
{
char	beg_lat[4],	/* beginning latitude			*/
	dform[5],       /* data format GAC LAC HRPT             */
	end_lat[4],	/* Ending latitude			*/
	jyear[3],	/* Year of the pass			*/
	jday[4],	/* Julian date of the pass		*/
	loc[3];		/* Location data generated at		*/

long	blat,		/* Beginning latitutde			*/
	dtype_from_dsh,	/* Data type from DSH header		*/
	year_from_dsh,	/* Year from DSH header			*/
	jday_from_dsh,	/* Julian day from DSH header		*/
	elat,		/* Ending latitutde			*/
	i, 		/* counter				*/
	off_set = 97,	/* offset to the channel specifications	*/
	tpchan[MOSTBND];/* channels stored in the file		*/
/* 
  The following values are presently unused, but may be needed later
*/
char    direction[11];  /* Direction of the pass                */
char    phr[3];         /* Start hour of the pass               */
char    pmin[3];        /* Start minute of the pass             */
char    sat[2];         /* Code for space craft id              */
char    satid[8];       /* Space craft ID                       */
char    source[30];     /* Source of data                       */

/*
  Determine the space craft id
*/
strncpy(sat,(const char *)(tbmbuf+40),1);
switch (sat[0])
    {
    case 'A':	strcpy(satid,"noaa-6");		break;
    case 'C':	strcpy(satid,"noaa-7");		break;
    case 'D':	strcpy(satid,"noaa-12");	break;
    case 'E':	strcpy(satid,"noaa-8"); 	break;
    case 'F':	strcpy(satid,"noaa-9");		break;
    case 'G':	strcpy(satid,"noaa-10");	break;
    case 'H':	strcpy(satid,"noaa-11");	break;
    case 'J':	strcpy(satid,"noaa-14");	break;
    default:	strcpy(satid,"unknown");	break;
    }

/*
  Retrieve the year of the pass
*/
strncpy(jyear,(const char *)(tbmbuf+43),2);
jyear[2] = '\0';

/*
  Retrieve the julian date and convert it to gregorian
*/
strncpy(jday,(const char *)(tbmbuf+45),3);
jday[3] = '\0';

/*
  Save the julian year and day within the model structure
  Save the gregorian month and day within the structure, too.
  Check to make sure the DSH and TBM don't disagree.
*/
year_from_dsh = model->year;
jday_from_dsh = model->jday;

sscanf(jyear,"%ld",&model->year);
sscanf(jday,"%ld",&model->jday);
c_julgre(&model->year, &model->jday, &model->month, &model->day);
if ((year_from_dsh != model->year) || (jday_from_dsh != model->jday))
   {
   c_lasmsg(FALSE,"c_l1bsup-hdrs","Discrepancy in TBM and DSH headers");
   return(E_FAIL);
   }

/*
  Retrieve the start hour of the pass
*/
strncpy(phr,(const char *)(tbmbuf+50),2);
phr[2] = '\0';

/*
  Retrieve the start minute of the pass
*/
strncpy(pmin,(const char *)(tbmbuf+52),2);
pmin[2] = '\0';

/*
  Determine the source the data was created at
  we know SFL, NGC, and NOA; for SO and WE have no associated CEOS id
  see NOAA Polar Orbiter Data Users Guide section 2.0.1
*/
strncpy(loc,(const char *)(tbmbuf+70),2);
loc[2] = '\0';
sprintf(source,"Level 1b / location code: %s",loc);
if (strcmp(loc,"SF") == 0)
    strcpy(model->station,"SFL");
else if (strcmp(loc,"GC") == 0)
    strcpy(model->station,"NGC");
else if (strcmp(loc,"WI") == 0)
    strcpy(model->station,"NOA");

/*
  Retrieve the beginning latitude
*/
strncpy(beg_lat,(const char *)(tbmbuf+75),3);
beg_lat[3] = '\0';

/*
  Retrieve the ending latitude
*/
strncpy(end_lat,(const char *)(tbmbuf+78),3);
end_lat[3] = '\0';

/*
  Determine the direction of the pass from the beginning and ending latitudes
*/
if (strcmp(beg_lat,"AL") == 0)
    strcpy(direction,"unknown");
else
    {
    sscanf(beg_lat,"%ld",&blat);
    sscanf(end_lat,"%ld",&elat);

    if (blat < elat)
	strcpy(direction,"ascending");
    else
	strcpy(direction,"descending");
    }

/*
  Get the data type (GAC LAC HRPT)
  Check to make sure the DSH and TBM don't disagree.
*/
dtype_from_dsh = model->dtype;
strncpy(dform,(const char *)(tbmbuf+34),4);
dform[4] = '\0';
if ((strcmp(dform,"GAC ") == 0) || (strcmp(dform,"GHRR") == 0))
   model->dtype = GAC;
else if ((strcmp(dform,"LAC ") == 0) || (strcmp(dform,"LHRR") == 0))
   model->dtype = LAC;
else
   model->dtype = HRPT;
if (model->dtype != dtype_from_dsh)
    {
    c_lasmsg(FALSE,"c_l1bsup-hdrs","Discrepancy in TBM and DSH headers");
    return(E_FAIL);
    }



/*
  Store the number of samples per scan in the model structure
*/
if (model->dtype == GAC)
   model->nsamples = GAC_SAMPS;
else
   model->nsamples = LAC_SAMPS;

/*
  Get the channels in file, if all channels are on the tape 'T'
  (total) will be set, else 'S' (select copy) will be set
*/
if (tbmbuf[74] == 'T')
   {
   for (i = 0; i < MOSTBND; i++)
      tpchan[i] = TRUE;
   }
else
   {
   for (i = 0; i < MOSTBND; i++)
      tpchan[i] = (long) (*(tbmbuf + off_set + i) & 0x01);
   }


/*
  NOTE:
  If 'S' was set but none of the channels were set we will assume that we
  cannot tell from the TBM header which channels are in the file.
*/


/*
  Load the model with the channel information
*/
model->nchans = 0;
for (i = 0; i < MOSTBND; i++)
    {
    if (tpchan[i])
	{
	model->chans[i] = i + 1;
	(model->nchans)++;
	}
    else
	model->chans[i] = 0;
    }


/*
  If the model now shows no channels, use the # channels from the DSH
*/
if (model->nchans == 0)
    model->nchans = totchdsh;

return(E_SUCC);
}
