/*****************************************************************************
FUNCTION:	L1B_MAKE_ARS

PURPOSE:	Create a ARS header for a Level 1b file.  See the NOOA KLM
		User's Guide for a description of the ARS format.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    12/98	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE
	This function is designed to create a ARS header from model and
	dropped lines structures.  When all information was present in the
	input file, the ARS header will be complete.  If, however, the ARS
	isn't present in the original file and less than five bands are
	present, the number of bands can be only be "guessed" at.  If five
	bands are present the ARS will reflect this.  If less than five bands
	are present, byte 75 will be set to 'S' but bytes 98-117 will be left
	as zero since it's impossible to determine which bands are actually
	present in the scene.  User's should be aware of this limitation.

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include <time.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "level1bio.h"
#include "local_protos.h"



int l1b_make_ars
(
unsigned char	*ars,		/* ARS header buffer		*/
unsigned char	*dsh,		/* DSH record			*/
struct MODEL	*model,		/* AVHRR model structure	*/
long		*recsiz 	/* data record size		*/
)

{
double	elev = 0.0;		/* elevation of data		*/
double	lat[4];			/* corner latitudes of scene	*/
double	lon[4];			/* corner longitudes of scene	*/
double	*view_angles = NULL;	/* view angles flag		*/

float	beglat;			/* beggining latitude		*/
float	beglon;			/* beginning longitude		*/
float	endlat;			/* ending latitude		*/
float	endlon;			/* ending longitude		*/
float	line;			/* line number inscene		*/
float	sample;			/* scene sample number		*/

long	gmtime;			/* acquisition GMT time		*/
long	i,j;			/* loop index variable		*/
long	jday;			/* julian day of acquisition	*/
long	year;			/* acquisition year		*/

short	*sptr;			/* pointer to a short		*/

unsigned char	temp[25];	/* temporary character buffer	*/

struct	tm	*lctm;		/* local time structure		*/
time_t	t;			/* gm seconds			*/

/* The first 30 characters of ARS are the order ID which is
specific to NOAA -- we will just put in spaces and the
creation day and year.
--------------------------------------------------------------*/
for ( i = 0; i < 30; i++ )
    ars[i] = ' ';
t = time( NULL );
lctm = localtime( &t );
sprintf( (char *)&ars[14], "%4d%03d", lctm->tm_year+1900, lctm->tm_yday+1 );
ars[21] = ' ';

/* Characters 31-71 are the data set name and can be obtained
from the dsh bytes 22-63.
--------------------------------------------------------------*/
for ( i = 30, j = 22; i <72; i++, j++ )
    ars[i] = dsh[j];

/* Determine start and end times
---------------------------------*/
l1b_dsh_time( dsh+2, &year, &jday, &gmtime );


/* Constant field for source, NOAA only defines
a few of these sources.  The only two that we
receive data from are the WI and GC stations.
For all others we will put SF.
-----------------------------------------------*/
if ( strcmp( model->station, "NOA" ) == 0 )
    strcpy( (char *)&ars[70], "WI  " );
else if ( strcmp( model->station, "NGC" ) == 0 )
    strcpy( (char *)&ars[70], "GC  " );
else
    strcpy( (char *)&ars[70], "SF  " );


/* Determine if total or selective and set selected bands
---------------------------------------------------------*/
if ( model->nchans == 5 )
    ars[74] = 'T';
else
    ars[74] = 'S';

/* Set the latitude/longitude and time selection criteria as ALL
and the appended data flag as Y.  Note: the NOAA Level 1b ARS
that I used as a guide has -180 and +180 in the longitude
extents so that is what I used here as well.
----------------------------------------------------------------*/
strcpy( (char *)&ars[75], "ALLALL-180-180ALL ALLY" );

/* Set the selected bands
-------------------------*/
for ( i = 97; i < 117; i++ )
    ars[i] = 'N';
for ( i = 0; i < 5; i++ )
    if ( model->chans[i] > 0 )
	ars[96+model->chans[i]] = 'Y';

/* This is the beginning of the Dataset summary
-----------------------------------------------*/
for ( i = 117; i < 147; i++ )
    ars[i] = ' ';

/* Set the ascend/descend flag
------------------------------*/
if ( model->spass && model->cpass && model->epass )
    ars[146] = 'A';
else if ( !model->spass && !model->cpass && !model->epass )
    ars[146] = 'D';
else
    ars[146] = 'B';

/* Get the lat/lon coordinate extents of the scene.  They go in bytes 76-96.
----------------------------------------------------------------------------*/
if ( model->cpass == ASCENDING )
    line = (float)model->nlines;
else
    line = 1;
sample = 1;
c_ls2ll( &line, &sample, &elev, model, &lat[0], &lon[0], view_angles );
sample = (float)model->nsamples;
c_ls2ll( &line, &sample, &elev, model, &lat[1], &lon[1], view_angles );
if ( model->cpass == ASCENDING )
    line = 1;
else
    line = (float)model->nlines;
sample = 1;
c_ls2ll( &line, &sample, &elev, model, &lat[2], &lon[2], view_angles );
sample = (float)model->nsamples;
c_ls2ll( &line, &sample, &elev, model, &lat[3], &lon[3], view_angles );

beglat = (float)DEG( lat[0] );
endlat = (float)DEG( lat[3] );
beglon = (float)DEG( lon[0] );
endlon = (float)DEG( lon[3] );

/* store the extents in the header
----------------------------------*/
sprintf( (char *)temp, "%+03ld%+03ld%+04ld%+04ld", (long)beglat, (long)endlat,
		(long)beglon, (long)endlon );
strcpy( (char *)&ars[147], (const char *)temp );

/* Store the data format.  the version number is in the dsh[4].
---------------------------------------------------------------*/
sptr = (short *)&dsh[4];
sprintf( (char *)temp, "NOAA Level 1b v%-2d   ", *sptr );
strcpy( (char *)&ars[161], (const char *)temp );

/* Store the record size and number of lines
--------------------------------------------*/
sprintf( (char *)temp, "%6ld%6ld", *recsiz, model->nlines );
strcpy( (char *)&ars[181], (const char *)temp );

/* Space fill the remaining bytes
---------------------------------*/
for ( i = 193; i < 512; i++ )
    ars[i] = ' ';

return E_SUCC;

}  /* END l1b_make_ars */

