/*****************************************************************************
FUNCTION:       L1B_MAKE_KLMDSH

PURPOSE:        Build a Data header record for NOAA KLM series satellite.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    12/98  D.Lloyd         Initial development
  1.1    10/00  D.Lloyd		Added NOAA 16 support
  1.2  04/2002  D.Lloyd		Added NOAA 17 support
  1.3  08/2005  D.Lloyd         Added NOAA 18 support

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"
#include "archio.h"
#include "level1bio.h"
#include "local_protos.h"



int l1b_make_klmdsh
(
    unsigned char *dsh,         /* pointer to DSH buffer                */
    struct MODEL *model,        /* AVHRR model structure                */
    struct DROPPED *drop,	/* dropped line structure		*/
    long recsiz         	/* data record size in bytes            */
)

{
char	*adapstables;		/* pointer to environment variable	*/
char	str[CMLEN];		/* string from calibration file		*/
char	tablefile[CMLEN];	/* table file name			*/

double	x1;			/* value read from calibration file	*/
double	x2;			/* value read from calibration file	*/
double	x3;			/* value read from calibration file	*/

FILE	*fd;			/* table file descriptor		*/

int     not_found = 1;          /* loop control to eliminate warning    */
long	count;			/* count of dropped lines		*/
long	eday;			/* ending day				*/
long	ejday;			/* ending jday				*/
long	emon;			/* ending month				*/
long	eyear;			/* ending year				*/
long	i;			/* loop control variable		*/
long	*lptr;			/* pointer to a long			*/
long	msec;			/* millisecond time			*/
long	sday = 1;		/* starting day				*/
long	smon = 1;		/* starting month			*/
long	syear = 1950;		/* starting year			*/
long	status;			/* function return status		*/
long	v1;			/* value read from table file		*/
long	v2;			/* value read from table file		*/
long	v3;			/* value read from table file		*/
long	v4;			/* value read from table file		*/
long	v5;			/* value read from table file		*/
long	v6;			/* value read from table file		*/

short   *sptr;                  /* pointer to a short                   */


/* Creation site
----------------*/
strcpy( (char *)&dsh[0], "NSS " );

/* Level 1b format version number, year, and day (using
the same values found in NOAA ftp scene) This will be
version 1, 6/9/97 until changes are made in the NOAA KLM
Users Guide.  At that time the version number will
increment and date will be changed.  I will hard code
this since, presumably, a change in the format version
will precipitate a change to the code in this function.
cr 10172: Level 1b format changed.  Based on information
contained in the test dataset, I made the following
changes:

*sptr = 1;	==> *sptr = 2;
*sptr = 1997;	==> *sptr = 2003;
*sptr = 159;	==> *sptr = 268;
--------------------------------------------------------*/
sptr = (short *)&dsh[4];
*sptr = 2;
sptr++;
*sptr = 2003;
sptr++;
*sptr = 268;

/* Logical record size, block size and header count
At this time we are only putting one header per
scene and block/record sizes are equal.
---------------------------------------------------*/
sptr++;
*sptr = (short)recsiz;
sptr++;
*sptr = (short)recsiz;
sptr++;
*sptr = 1;
for ( i = 16; i < 22; i++ )	/* zero fill */
    dsh[i] = 0;

/* Data set name
----------------*/
strcpy( (char *)&dsh[22], "NSS." );
switch ( model->dtype )
    {
    case 1 :
        strcpy( (char *)&dsh[26], "LHRR." );
        break;
    case 2 :
        strcpy( (char *)&dsh[26], "GHRR." );
        break;
    case 3 :
        strcpy( (char *)&dsh[26], "HRPT." );
        break;
    case 5 :
        strcpy( (char *)&dsh[26], "HIRX." );
        break;
    default :
        strcpy( (char *)&dsh[26], "    ." );
    }
switch ( model->satnum )
    {
    case 15 :
        strcpy( (char *)&dsh[31], "NK." );
        break;
    case 16 :
        strcpy( (char *)&dsh[31], "NL." );
	break;
    case 17 :
        strcpy( (char *)&dsh[31], "NM." );
	break;
    case 18 :
        strcpy( (char *)&dsh[31], "NN." );
	break;
    default :
        strcpy( (char *)&dsh[31], "  ." );
        break;
    }

/* NOTE: this may change for Y2K compliance.  In a memo from NOAA,
the hardcoded 'D' may be changed to a 'D' for scenes in 19XX and
a 'F' for scenes in 20XX.
this is easily changed as follows:
if ( c_4digityr( model->year ) >= 2000 )
    sprintf( &dsh[34], "F%02d%03d.", model->year, model->jday );
else
-----------------------------------------------------------------*/
sprintf( (char *)&dsh[34], "D%02ld%03ld.", model->year, model->jday );
sprintf( (char *)&dsh[41], "S%02ld%02ld.", model->shour, model->smin );
sprintf( (char *)&dsh[47], "E%02ld%02ld.", model->ehour, model->emin );
sprintf( (char *)&dsh[53], "B%05ld%02ld.", model->sorbit, model->sorbit % 100 );

/* NOAA only defines a few of these source codes.  We only
receive data from the WI and GC stations.  For all others
we will use SF.
----------------------------------------------------------*/
if ( strcmp( model->station, "NOA" ) == 0 )
    strcpy( (char *)&dsh[62], "WI" );
else if ( strcmp( model->station, "NGC" ) == 0 )
    strcpy( (char *)&dsh[62], "GC" );
else
    strcpy( (char *)&dsh[62], "SF" );

/* processing block id
----------------------*/
sprintf( (char *)&dsh[64], "B%05ld%02ld", model->sorbit, model->eorbit % 100 );

/* spacecraft and instrument id
data type and tip source
2 = NOAA-L
4 = NOAA-K
6 = NOAA-M
7 = NOAA-N
8 = NOAA-N'
-------------------------------*/
sptr = (short *)&dsh[72];
switch ( model->satnum ) {
    case 15 :
        *sptr = 4;
	break;
    case 16 :
        *sptr = 2;
	break;
    case 17 :
        *sptr = 6;
	break;
    case 18 :
        *sptr = 7;
	break;
    default :
    	*sptr = 0;
	break;
    }

/* Instrument ID from the NOAA KLM User's Guide
This is valid for  NOAA K but unsure of what it
will be for subsequent sensors.
301 = s/n A301 NOAA-L
302 = s/n A301 NOAA-K
303 = s/n A301 NOAA-N'
304 = s/n A301 NOAA-M
306 = s/n A301 NOAA-N
-----------------------------------------------*/
sptr++;
switch ( model->satnum ) {
    case 15 :
	*sptr = 302;	
	break;
    case 16 :
        *sptr = 301;
	break;
    case 17 :
        *sptr = 304;
	break;
    case 18 :
        *sptr = 306;
	break;
    default :
    	*sptr = 0;
	break;
    }

/* Data type and TIP source.  Since we don't
archive the TIP, set to unused.
--------------------------------------------*/
sptr++;
*sptr = (short)model->dtype;
sptr++;
*sptr = 0;

/* store number of days since Jan 1, 1950 to start day
------------------------------------------------------*/
lptr = (long *)&dsh[80];
status = c_daysdiff( &smon, &sday, &syear, &model->month, &model->day,
	&model->year, lptr );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"make_klmdsh-ndays","Error calculating days since 1/1/50");
    return E_FAIL;
    }

/* Start of Data Set: day count, year, julian
day and UTC time in miliseconds
---------------------------------------------*/
sptr = (short *)&dsh[84];
*sptr = (short)c_4digityr( model->year );
sptr++;
*sptr = (short)model->jday;

msec = model->shour * MIL_PER_HOUR;
msec += model->smin * MIL_PER_MIN;
msec += model->ssec * MIL_PER_SEC;
msec += model->sfrac;
msec += model->clock_err;
lptr = (long *)&dsh[88];
*lptr = msec;

/* determine ending date */
span( model, &emon, &eday, &eyear, &ejday );

/* store number of days since Jan 1, 1950 to end day
----------------------------------------------------*/
lptr = (long *)&dsh[92];
status = c_daysdiff( &smon, &sday, &syear, &emon, &eday, &eyear, lptr );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"make_klmdsh-ndays","Error calculating days since 1/1/50");
    return E_FAIL;
    }

sptr = (short *)&dsh[96];
*sptr = (short)c_4digityr( eyear );
sptr++;
*sptr = (short)ejday;

msec = model->ehour * MIL_PER_HOUR;
msec += model->emin * MIL_PER_MIN;
msec += model->esec * MIL_PER_SEC;
msec += model->efrac;
msec += model->clock_err;
lptr = (long *)&dsh[100];
*lptr = msec;

/* Calibration Parameter ID (CPIDS) database date
This is a dynamically updated database at NESDIS
We currently don't have access to this database
so it is really meaningless.
-------------------------------------------------*/
sptr = (short *)&dsh[104];
*sptr = 1998;	/* using date found in NOAA N15	*/
sptr++;
*sptr = 236;	/* using date found in NOAA N15	*/
for ( i = 108; i < 116; i++ )	/* zero fill */
    dsh[i] = 0;

/* Data set quality indicators
These data can apparently be found in the Digital
B telemetry which can be found in the TIP minor
frame, which we currently throw away so set to
zero.

dsh[116] (4bytes) instrument status
dsh[122] (2bytes) record number of status change
dsh[124] (4 bytes) second instrument status
------------------------------------------------*/
lptr = (long *)&dsh[116];
*lptr = 0;
sptr = (short *)&dsh[120];
*sptr = 0;	/* zero fill */
sptr++;
*sptr = 0;
lptr = (long *)&dsh[124];
*lptr = 0;

/* count of data records, count of valid lines, count
of missing lines, count of data gaps
-----------------------------------------------------*/
sptr = (short *)&dsh[128];
*sptr = (short)model->nlines;
sptr++;
*sptr = (short)model->nlines;
sptr++;
count = 0;
for ( i = 0; i < drop->ngaps; i++ )
    count += drop->nmissed[i];
*sptr = (short)count;
sptr++;
*sptr = (short)drop->ngaps;

/* data set quality indicators cont.

dsh[136] (2 bytes) count of data frames without frame sync word errors
dsh[138] (2 bytes) count of PACS detected TIP parity errors
dsh[140] (2 bytes) sum of all auxiliary sync errors detected
dsh[142] (2 bytes) time sequence error
dsh[144] (2 bytes) time sequence error code
dsh[146] (2 bytes) SOCC clock update indicator
dsh[148] (2 bytes) earth location error indicator
dsh[150] (2 bytes) earth location error code
dsh[152] (2 bytes) PACS status bit field
dsh[154] (2 bytes) PACS data source
dsh[156] (4 bytes) zero fill
dsh[160] (8 bytes) reserved for ingester -- space fill
dsh[168] (8 bytes) reserved for decommutation -- space fill
dsh[176] (10 bytes) zero fill
----------------------------------------------------------------------*/
for ( i = 136; i < 160; i++ )
    dsh[i] = 0;
for ( i = 160; i < 176; i++ )
    dsh[i] = ' ';
for ( i = 176; i < 186; i++ )
    dsh[i] = 0;

/* calibration

dsh[186] (2 bytes) ramp/auto calibration indicators bit field
dsh[188] (2 bytes) year of most recent solar channel calibration
  The previous field is for support of CLAVR
dsh[190] (2 bytes) day of most recent solar channel calibration
dsh[192] (2 bytes) primary calibration algorithm ID
dsh[194] (2 bytes) primary calibration algorithm selected options
dsh[196] (2 bytes) secondary calibration algorithm ID
dsh[198] (2 bytes) secondary calibration algorithm selected options
-------------------------------------------------------------------*/
for ( i = 186; i < 200; i++ )
    dsh[i] = 0;

/* Open the calibration table
-----------------------------*/
adapstables = getenv( "ADAPSTABLES" );
if ( adapstables == NULL )
    {
    c_lasmsg(FALSE,"make_klmdsh-env","Error retrieving ADAPSTABLES");
    return E_FAIL;
    }
sprintf( tablefile, "%s/noaa%2ld.cal", adapstables, model->satnum );
fd = fopen( tablefile, "r" );
if ( !fd )
    {
    c_lasmsg(FALSE,"make_klmdsh-open","Unable to open calibration file");
    return E_FAIL;
    }

/* IR target temperature i conversion coefficient j
These can be found in Table D.1-8, and start on line
PRT CALIBRATION FACTORS of calfile.

i = [1..4] j = [1..6]
dsh[200] to dsh[246] (2 bytes each)
---------------------------------------------------*/
while ( not_found )	/* Find table D.1-8 */
    {
    if ( fgets( str, 100, fd ) == 0 )
        {
	c_lasmsg(FALSE,"make_klmdsh-cal","Unable to find calibration coeffs");
	return E_FAIL;
	}
    if ( strncmp( str, "PRT CALIBRATION FACTORS", 23 ) == 0 )
	break;
    }
for ( i = 0; i < 3; i++ )
    fgets( str, 100, fd );
sptr = (short *)&dsh[200];
for ( i = 0; i < 4; i++ )
    {
    fgets( str, 100, fd );
    sscanf( str, "%*s %lf %lf %lf", &x1, &x2, &x3 );
    *sptr = x1 * 100 + 0.5;
    sptr++;
    *sptr = x2 * 100000 + 0.5;
    sptr++;
    *sptr = x3 * 100000000 + 0.5;
    sptr++;
    *sptr = 0;
    sptr++;
    *sptr = 0;
    sptr++;
    *sptr = 0;
    sptr++;
    }
for ( i = 248; i < 256; i++ )	/* zero fill */
    dsh[i] = 0;

/* Radiance conversion

Solar filtered irradiance (col 3) and equivalent filter width (col 1)
from table D. 1-7 in NOAA KLM doc.
dsh[256] (4 bytes) channel 1 irradiance col_3_value * 10
dsh[260] (4 bytes) channel 1 filter width col_1_value * 10^3
dsh[268] (4 bytes) channel 2 irradiance col_3_value * 10
dsh[272] (4 bytes) channel 2 filter width col_1_value * 10^3
dsh[276] (4 bytes) channel 3A irradiance col_3_value * 10
dsh[260] (4 bytes) channel 3A filter width col_1_value * 10^3
---------------------------------------------------------------------*/
rewind( fd );
while ( not_found )	/* Find table D.1-7 */
    {
    if ( fgets( str, 100, fd ) == 0 )
        {
	c_lasmsg(FALSE,"make_klmdsh-cal","Unable to find calibration coeffs");
	return E_FAIL;
	}
    if ( strncmp( str, "RADIANCE TO REFLECTANCE", 23 ) == 0 )
	break;
    }
for ( i = 0; i < 6; i++ )
    fgets( str, 100, fd );

/* Store the scaled values
--------------------------*/
lptr = (long *)&dsh[256];
for ( i = 0; i < 3; i++ )
    {
    fgets( str, 100, fd );
    sscanf( str, "%*s %lf %lf %lf", &x1, &x2, &x3 );
    *lptr = (long)(x3 * 10.0 + 0.5);
    lptr++;
    *lptr = (long)(x1 * 1000.0 + 0.5);
    lptr++;
    }

/*

central wavenumber (col 1) and constants (col 2&3)come from NOAA KLM
users doc Table D. 1-11.
dsh[280] (4 bytes) chan 3B central wavenumber col_1_val * 10^2
dsh[284] (4 bytes) chan 3B constant 1 col_2_val * 10^5
dsh[288] (4 bytes) chan 3B constant 2 col_3_val * 10^6
dsh[292] (4 bytes) chan 4 central wavenumber col_1_val * 10^3
dsh[296] (4 bytes) chan 4 constant 1 col_2_val * 10^5
dsh[300] (4 bytes) chan 4 constant 2 col_3_val * 10^6
dsh[304] (4 bytes) chan 5 central wavenumber col_1_val * 10^3
dsh[308] (4 bytes) chan 5 constant 1 col_2_val * 10^5
dsh[312] (4 bytes) chan 5 constant 2 col_3_val * 10^6
---------------------------------------------------------------------*/
rewind( fd );
while ( not_found )	/* Find table D.1-11 */
    {
    if ( fgets( str, 100, fd ) == 0 )
        {
	c_lasmsg(FALSE,"make_klmdsh-cal","Unable to find calibration coeffs");
	return E_FAIL;
	}
    if ( strncmp( str, "RADIANCE TO TEMPERATURE COEFF", 29 ) == 0 )
	break;
    }
for ( i = 0; i < 3; i++ )
    fgets( str, 100, fd );

/* Store the scaled values
--------------------------*/
lptr = (long *)&dsh[280];
for ( i = 0; i < 3; i++ )
    {
    fgets( str, 100, fd );
    sscanf( str, "%*s %lf %*f %*f%*f  %lf %lf", &x1, &x2, &x3 );
    if ( i == 0 )
	*lptr = (long)(x1 * 100 + 0.5);
    else
	*lptr = (long)(x1 * 1000 + 0.5);
    lptr++;
    *lptr = (long)(x2 * 100000 + 0.5);
    lptr++;
    *lptr = (long)(x3 * 1000000 + 0.5);
    lptr++;
    }
for ( i = 316; i < 328; i++ )	/* zero fill */
    dsh[i] = 0;

fclose( fd );


/* Navigation

dsh[328] (8 bytes character) ellipsoid model ID
dsh[336] (2 bytes) nadir earth location tolerance
dsh[338] (2 bytes) earth location bit field
dsh[340] (2 bytes) zero fill
dsh[342] (2 bytes) constant roll attitude error
dsh[344] (2 bytes) constant pitchattitude error
dsh[346] (2 bytes) constant yaw attitude error
-------------------------------------------------*/
for ( i = 328; i < 348; i++ )
    dsh[i] = 0;

/* epoch year, day and milliseconds
-----------------------------------*/
sptr = (short *)&dsh[348];
*sptr = (short)model->epoch_year;
sptr++;
*sptr = (short)model->epoch_jday;
/* dsh[352] (4 bytes) epoch UTC time of day in milliseconds */
lptr = (long *)&dsh[352];
*lptr = (long)(model->sec_epoch * 1000);

/* semi-major axis
------------------*/
lptr++;
*lptr = (long)(model->smajor_axis * 100000);

/* eccentricity
---------------*/
lptr++;
*lptr = (long)(model->ecc * 100000000);

/* inclination
--------------*/
lptr++;
*lptr = (long)(model->inclin * 100000);

/* argument of perigee
----------------------*/
lptr++;
*lptr = (long)(model->arg_per * 100000);

/* right ascension of the ascending node
not in model
----------------------------------------*/
lptr++;
*lptr = 0;

/* mean anomally
----------------*/
lptr++;
*lptr = (long)(model->mean_anomaly * 100000);

/* position vector
------------------*/
lptr++;
*lptr = (long)model->satint.target.x /** 100000*/;
lptr++;
*lptr = (long)model->satint.target.y /** 100000*/;
lptr++;
*lptr = (long)model->satint.target.z /** 100000*/;

/* velocity vector
------------------*/
lptr++;
*lptr = (long)model->satint.vel.x /** 100000000*/;
lptr++;
*lptr = (long)model->satint.vel.y /** 100000000*/;
lptr++;
*lptr = (long)model->satint.vel.z /** 100000000*/;

/* earth/sun distance ratio
dsh[404] (4 bytes) 
not in model
---------------------------*/
lptr++;
*lptr = 0;
for ( i = 407; i < 424; i++ )	/* zero fill */
    dsh[i] = 0;

/* Analog Telemetry Conversion.  NOTE: At this time I believe that
these coefficients are constant -- or at least will only be updated
periodically.  The noaaXX.tel table can be zeroed out if we don't
want to use these coeffs.  If the values periodically change, the
table will have to be modified to include a date and code added to
search for the appropriate date and and position the file pointer.
-------------------------------------------------------------------*/
sprintf( tablefile, "%s/noaa%2ld.tel", adapstables, model->satnum );
fd = fopen( tablefile, "r" );
if ( !fd )
    {
    c_lasmsg(FALSE,"make_lkmdsh-open","Unable to open calibration file");
    return E_FAIL;
    }
fgets( str, 100, fd );	/* position file pointer appropriately	*/
fgets( str, 100, fd );	/* position file pointer appropriately	*/

/* patch temperature conversion coefficients
dsh[424] - dsh[447] (4 bytes each) 6 coefs
--------------------------------------------*/
lptr = (long *)&dsh[424];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* patch temperature extended conversion coeffs
dsh[448] - dsh[471] (4 bytes each) 6 coefs
-----------------------------------------------*/
lptr = (long *)&dsh[448];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* patch power conversion coefs
dsh[472] - dsh[495] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[472];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* radiator temperature conversion coefs
dsh[496] - dsh[519] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[496];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* blackbody temperature 1 conversion coefs
dsh[520] - dsh[543] (4 bytes each) 6 coefs
-------------------------------------------*/
lptr = (long *)&dsh[520];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* blackbody temperature 2 conversion coefs
dsh[544] - dsh[567] (4 bytes each) 6 coefs
-------------------------------------------*/
lptr = (long *)&dsh[544];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* blackbody temperature 3 conversion coefs
dsh[568] - dsh[591] (4 bytes each) 6 coefs
-------------------------------------------*/
lptr = (long *)&dsh[568];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* blackbody temperature 4 conversion coefs
dsh[592] - dsh[615] (4 bytes each) 6 coefs
-------------------------------------------*/
lptr = (long *)&dsh[592];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* electronics current conversion coefs
dsh[616] - dsh[639] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[616];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* motor current conversion coefs
dsh[640] - dsh[663] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[640];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* earth shield position conversion coefs
dsh[664] - dsh[687] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[664];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* electronics temperature conversion coefs
dsh[688] - dsh[711] (4 bytes each) 6 coefs
-------------------------------------------*/
lptr = (long *)&dsh[688];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* cooler housing temperature conversion coefs
dsh[712] - dsh[735] (4 bytes each) 6 coefs
----------------------------------------------*/
lptr = (long *)&dsh[712];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* baseplate temperature conversion coefs
dsh[736] - dsh[759] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[736];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* motor housing temperature conversion coefs
dsh[760] - dsh[783] (4 bytes each) 6 coefs
---------------------------------------------*/
lptr = (long *)&dsh[760];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* A/D converter temperature conversion coefs
dsh[784] - dsh[807] (4 bytes each) 6 coefs
---------------------------------------------*/
lptr = (long *)&dsh[784];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* detector #4 bias voltage conversion coefs
dsh[808] - dsh[831] (4 bytes each) 6 coefs
--------------------------------------------*/
lptr = (long *)&dsh[808];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* detector #5 bias voltage conversion coefs
dsh[832] - dsh[855] (4 bytes each) 6 coefs
--------------------------------------------*/
lptr = (long *)&dsh[832];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* chan 3b blackbody view conversion coefs
dsh[856] - dsh[879] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[856];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* chan 4 blackbody view conversion coefs
dsh[880] - dsh[903] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[880];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* chan 5 blackbody view conversion coefs
dsh[904] - dsh[927] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[904];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

/* reference voltage conversion coefs
dsh[928] - dsh[951] (4 bytes each) 6 coefs
------------------------------------------*/
lptr = (long *)&dsh[928];
fgets( str, 100, fd );
sscanf( str, "%ld %ld %ld %ld %ld %ld", &v1, &v2, &v3, &v4, &v5, &v6 );
*lptr = v1;
lptr++;
*lptr = v2;
lptr++;
*lptr = v3;
lptr++;
*lptr = v4;
lptr++;
*lptr = v5;
lptr++;
*lptr = v6;

fclose( fd );

/* zero filled to end of record
-------------------------------*/
for ( i = 952; i < recsiz; i++ )
    dsh[i] = 0;

return E_SUCC;
}
