/*****************************************************************************
FUNCTION:	L1B_MAKE_TBM

PURPOSE:	Create a TBM header for a Level 1b file.  See the NOOA Polar
		Orbiter Data User's Guide for a description of the TBM format.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    07/97	D.Lloyd		Initial development
  1.1    10/97	D.Lloyd		Modified as a Level 1b support routine
  1.2	 07/98	D.Lloyd		Modified for Y2K compliance

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE
	This function is designed to create a TBM header from model and
	dropped lines structures.  When all information was present in the
	input file, the TBM header will be complete.  If, however, the TBM
	isn't present in the original file and less than five bands are
	present, the number of bands can be only be "guessed" at.  If five
	bands are present the TBM will reflect this.  If less than five bands
	are present, byte 75 will be set to 'S' but bytes 98-117 will be left
	as zero since it's impossible to determine which bands are actually
	present in the scene.  User's should be aware of this limitation.

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "level1bio.h"
#include "local_protos.h"



int l1b_make_tbm
(
    unsigned char *tbm,		/* TBM header buffer		*/
    unsigned char *dsh,		/* DSH record			*/
    struct MODEL *model		/* AVHRR model structure	*/
)

{
double	elev = 0.0;		/* elevation of data		*/
double	lat[4];			/* corner latitudes of scene	*/
double	lon[4];			/* corner longitudes of scene	*/
double	*view_angles = NULL;	/* view angles flag		*/

float	beglat;			/* beggining latitude		*/
float	beglon;			/* beginning longitude		*/
float	endlat;			/* ending latitude		*/
float	endlon;			/* ending longitude		*/
float	line;			/* line number inscene		*/
float	sample;			/* scene sample number		*/
float	ttime;			/* temporary time variable	*/

long	dtype_lower;		/* lower 4 bits of data type	*/
long	dtype_upper;		/* upper 4 bits of data type	*/
long	gmtime;			/* acquisition GMT time		*/
long	hour;			/* acquisition hour		*/
long	i;			/* loop index variable		*/
long	jday;			/* julian day of acquisition	*/
long	minute;			/* acquisition minute		*/
long	year;			/* acquisition year		*/

unsigned char	temp[25];	/* temporary character buffer	*/


/*
  The first 30 characters of TBM are spaces followed by "NSS."
*/
for ( i = 0; i < 30; i++ )
    tbm[i] = ' ';
strcpy( (char *)&tbm[30], "NSS." );


/*
  Characters 35-38 indicate the data type, this can be determined
  from byte 2 of the DSH.
*/
strcpy( (char *)&tbm[34], "    ." );
dtype_upper = dsh[1] >> 4;
dtype_lower = dsh[1] & 0x0f;
switch ( dtype_upper )
    {
    case  1 :
	strcpy( (char *)&tbm[34], "LHRR." );		/* LAC data	*/
	break;
    case  2 :
	strcpy( (char *)&tbm[34], "GHRR." );		/* GAC data	*/
	break;
    case  3 :
	strcpy( (char *)&tbm[34], "HRPT." );		/* HRPT data	*/
	break;
    case  5 :
	if ( dtype_lower == 1 )
	    strcpy( (char *)&tbm[34], "HIRX." );	/* HIRS/2 embedded TIP*/
	else if ( dtype_lower == 2 )
	    strcpy( (char *)&tbm[34], "HIRS." );	/* HIRS/2 stored TIP */
	break;
    case  6 :
	if ( dtype_lower == 1 )
	    strcpy( (char *)&tbm[34], "MSUX." );	/* MSU embedded TIP */
	else if ( dtype_lower == 2 )
	    strcpy( (char *)&tbm[34], "MSUS." );	/* MSU stored TIP */
	break;
    case  7 :
	if ( dtype_lower == 1 )
	    strcpy( (char *)&tbm[34], "SSUX." );	/* SSU embedded TIP */
	else if ( dtype_lower == 2 )
	    strcpy( (char *)&tbm[34], "SSUS." );	
	break;
    }  /* END switch */


/*
  Determine start and end times
*/
l1b_dsh_time( dsh+2, &year, &jday, &gmtime );


/*
  Spots 40-41 are the spacecraft type -- byte 1 of DSH
*/
strcpy( (char *)&tbm[39], "  ." );
switch ( dsh[0] )
    {
    case  1 :
	/********************************************
	DLLOYD Y2K
	  Restrict the date range for TIROS-N.
	  Although this restriction isn't absolutely
	  necessary it doesn't hurt to do so.
	*********************************************/
	if ( year > 78 && year < 88 )
	    strcpy( (char *)&tbm[39], "TN." );		/* TIROS-N	*/
	else
	    strcpy( (char *)&tbm[39], "NH." );		/* NOAA-11	*/
	break;
    case  2 :
	/**************************************
	DLLOYD Y2K
	  Restrict the date range for NOAA-6.
	***************************************/
	if ( year > 79 && year < 93 )
	    strcpy( (char *)&tbm[39], "NA." );		/* NOAA-6	*/
	else
	    strcpy( (char *)&tbm[39], "NI." );		/* NOAA-13	*/
	break;
    case  3 :
	strcpy( (char *)&tbm[39], "NJ." );		/* NOAA-14	*/
	break;
    case  4 :
	strcpy( (char *)&tbm[39], "NC." );		/* NOAA-7	*/
	break;
    case  5 : 
	strcpy( (char *)&tbm[39], "ND." );		/* NOAA-12	*/
	break;
    case  6 : 
	strcpy( (char *)&tbm[39], "NE." );		/* NOAA-8	*/
	break;
    case  7 : 
	strcpy( (char *)&tbm[39], "NF." );		/* NOAA-9	*/
	break;
    case  8 : 
	strcpy( (char *)&tbm[39], "NG." );		/* NOAA-10	*/
	break;
    }  /* END switch */


/*
  Put year and julian day in header bytes 43-48
*/
/* NOTE: this may change for Y2K compliance.  In a memo from NOAA,
the hardcoded 'D' may be changed to a 'D' for scenes in 19XX and
a 'F' for scenes in 20XX.
easily changed:
if ( c_4digityr( year ) >= 2000 )
    sprintf( temp "F%02d%03d.", year, jday );
else
-----------------------------------------------------------------*/
sprintf( (char *)temp, "D%02ld%03ld.", year, jday );
strcpy( (char *)&tbm[42], (const char *)temp );


/*
  Put start time (hours and minutes) in header bytes 50-54
*/
ttime = (float)gmtime / MIL_PER_HOUR;
hour = (long)ttime;
minute = (long)((ttime - (float)hour) * 60.0);
sprintf( (char *)temp, "S%02ld%02ld.", hour, minute );
strcpy( (char *)&tbm[49], (const char *)temp );


/*
  Put end time (hours and minutes) in header bytes 56-60
*/
l1b_dsh_time( dsh+10, &year, &jday, &gmtime );
ttime = (float)gmtime / MIL_PER_HOUR;
hour = (long)ttime;
minute = (long)((ttime - (float)hour) * 60.0);
sprintf( (char *)temp, "E%02ld%02ld.", hour, minute );
strcpy( (char *)&tbm[55], (const char *)temp );


/*
  Next comes the processing block id starting at byte 62 and is
  found in DSH bytes 17-23
*/
tbm[61] = 'B';
for ( i = 16; i < 23; i++ )
    tbm[i+46] = dsh[i];
tbm[69] = '.';


/*
  Constant field for source
*/
strcpy( (char *)&tbm[70], "SF  " );


/*
  Determine if total or selective and set selected bands
*/
if ( model->nchans == 5 )
    tbm[74] = 'T';
else
    tbm[74] = 'S';


/*
  Get the lat/lon coordinate extents of the scene.  They go in bytes 76-96.
*/
if ( model->cpass == ASCENDING )
    line = (float)model->nlines;
else
    line = 1;
sample = 1;
c_ls2ll( &line, &sample, &elev, model, &lat[0], &lon[0], view_angles );
sample = (float)model->nsamples;
c_ls2ll( &line, &sample, &elev, model, &lat[1], &lon[1], view_angles );
if ( model->cpass == ASCENDING )
    line = 1;
else
    line = (float)model->nlines;
sample = 1;
c_ls2ll( &line, &sample, &elev, model, &lat[2], &lon[2], view_angles );
sample = (float)model->nsamples;
c_ls2ll( &line, &sample, &elev, model, &lat[3], &lon[3], view_angles );

beglat = (float)DEG( lat[0] );
endlat = (float)DEG( lat[3] );
beglon = (float)DEG( lon[0] );
endlon = (float)DEG( lon[3] );


/*
  Finally set the fields indicating that ALL lat/lon and ALL times are
  contained in the data set and that earth location data is appended.
*/
sprintf( (char *)temp, "%+03ld%+03ld%+04ld%+04ldALL ALLY", (long)beglat,
         (long)endlat, (long)beglon, (long)endlon );
strcpy( (char *)&tbm[75], (char *)temp );


/*
  Set the selected bands
*/
for ( i = 97; i < 122; i++ )
    tbm[i] = 0;
if ( tbm[74] == 'S' )
    for ( i = 0; i < 5; i++ )
	if ( model->chans[i] > 0 )
	    tbm[96+model->chans[i]] = 1;


return E_SUCC;

}  /* END l1b_make_tbm */

