/*******************************************************************************
FUNCTION:	L1B_TAPE_IN

PURPOSE:	Level-1B tape interface routine for c_l1bopntr().


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T.Baltzer        Development
  1.1    11/98  D.Lloyd          Modifications for NOAA KLM

PROJECT:	ADAPS
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "tapeio.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"

#define NOAALINEBUFTIME { 2, 2, 2, 1, 4, 4, 1 }
#define KLMLINEBUFTIME  { 2, 2, 2, 2, 8, 4, 1 }



int l1b_tape_in
(
    char drive[CMLEN],		/* Tape drive name			*/
    long *tun,			/* Tape unit number			*/
    unsigned char hdrbuf[],	/* TBM/ARS header buffer		*/
    unsigned char dshbuf[],	/* DSH header buffer			*/
    unsigned char linbuf[],	/* Buffer containing first line of data	*/
    long *recsiz,		/* Record size of data read            	*/
    long *hdrflg,		/* TBM/ARS header flag (TRUE or FALSE)	*/
    long *tape			/* Flag for tape open			*/
)

{

unsigned char *hdrptr;	/* Pointer to TBM/ARS header buffer		*/
unsigned char *dshptr;	/* Pointer to DSH header buffer			*/

unsigned char *dumptr;	/* Pointer to DSH dummy header buffer		*/

unsigned char inbuf1[MAXRECSIZ]; /* Input buffer from tape		*/
unsigned char inbuf2[MAXRECSIZ]; /* Input buffer from tape		*/
unsigned char inbuf3[MAXRECSIZ]; /* Input buffer from tape		*/


long	i,			/* Counter				*/
	inout = TREAD,		/* Read only from tape			*/
 	mode = HANDLE,		/* This routine will handle errors	*/
	nbyte = MAXRECSIZ,	/* Number of bytes to read from tape	*/
	one = 1,		/* Constant value			*/
	rewopt = NOREW,		/* Rewind option flag			*/
	skip = 0,		/* Number of files to skip		*/
	status;			/* Return status variable		*/
long	hdrsiz = 0;		/* TBM/ARS header size			*/

char    density[5];		/* default Tape density			*/


/*
  Reopen the tape
*/
strcpy(density,"0");
status = c_tpreop( tun, &skip, &mode, &rewopt, density, &inout, drive );
if ( status != TAPE_SUCC )
    return E_FAIL;


/*
  Signal for clean up that the tape is open
*/
*tape = TRUE;


/*
  Read the first record on tape.  If the record is a TBM/ARS header record,
  process it and then read and process the DSH record.  Otherwise the
  first record is presumably the DSH record, process it (no TBM/ARS header
  exists).
*/
status = c_tpread(tun,inbuf1,&nbyte,&mode);
if (status == TBM_HDR_SIZ || status == ARS_HDR_SIZ)
   {
   hdrsiz = status;
   *hdrflg = TRUE;
   hdrptr = inbuf1;
   dshptr = inbuf2;
   status = c_tpread(tun,dshptr,&nbyte,&mode);
   }
else
   {
   *hdrflg = FALSE;
   hdrptr = inbuf2;
   dshptr = inbuf1;
   }
dumptr = inbuf3; 
*recsiz = status;


/*
  Load the output buffers from the local buffers
*/
if (*hdrflg)
    {
    for (i = 0; i < hdrsiz; i++)
	hdrbuf[i] = hdrptr[i];
    }

for (i = 0; i < MAXRECSIZ; i++)
    dshbuf[i] = dshptr[i];

/* Insure the DSH is in the proper endian format
------------------------------------------------*/
if ( *recsiz == ARS_HDR_SIZ )
    {
    import_endian_mixed_buffer( dshbuf, KLMDATASETHEADER );
    }
else
    {
    import_endian_mixed_buffer( dshbuf, NOAADATASETHEADER );
    }


/*
  Read the second Data Set Header record - dummy record
  (pre-KLM LAC/HRPT data only)
*/
switch ( *recsiz )
    {
    case UNP_LAC_1 :
    case UNP_LAC_2 :
    case UNP_LAC_3 :
    case UNP_LAC_4 :
    case UNP_LAC_5 :
    case PCK_LAC_5 :
       status = c_tpread(tun,dumptr,&nbyte,&mode);
       if (*recsiz != status)
	   {
	   c_lasmsg(FALSE,"c_l1bopntr-dumhdr",
	   	"Error reading dummy header record");
	   return(E_FAIL);
	   }
    }


/*
  Now read the first line of data (used to verify the first time stamp)
*/
status = c_tpread(tun,linbuf,&nbyte,&mode);
if (*recsiz != status)
    {
    c_lasmsg(FALSE,"c_l1bopntr-linbuf","Error reading first data line");
    return(E_FAIL);
    }


/* Insure the line time portion is in the proper endian format
--------------------------------------------------------------*/
if ( *recsiz == ARS_HDR_SIZ )
    {
    import_endian_mixed_buffer( linbuf, KLMLINEBUFTIME );
    }
else
    {
    import_endian_mixed_buffer( linbuf, NOAALINEBUFTIME );
    }


/*
  Now backspace in the file to the first line of data
*/
status = c_tpbsr(tun,&one,&mode);
if (status != TAPE_SUCC)
    {
    c_lasmsg(FALSE,"c_l1bopntr-bsr","Error backspacing first data line");
    return(E_FAIL);
    }


return(E_SUCC);
}
