/*******************************************************************************
FUNCTION:       L1BVIDEO

PURPOSE:        Given a buffer of L1B video data, return a buffer of L1B video
		data in some other format.
		NOTE: No error checking is done for the sake of efficiency.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    11/97  T.Baltzer        Development
  1.1    11/98  D.Lloyd          Modifications for NOAA KLM

PROJECT:        ADAPS
*******************************************************************************/
#include "las.h"
#include "adaps.h"
#include "acqio.h"
#include "adapsutil.h"
#include "level1bio.h"
#include "pixman.h"
#include "local_protos.h"



void l1bvideo
(
    long irecsiz,		/* Input buffer record size	*/
    long ichan[],		/* Input channels within inbuf	*/
    unsigned char *ibuf,	/* Input buffer			*/
    long orecsiz,		/* Output buffer record size	*/
    long ochan[],		/* Output channels within obuf	*/
    unsigned char *obuf,	/* Output buffer		*/
    long videosize		/* Output video buffer size	*/
)

{

unsigned char tbuf[MAXLINSIZ];	/* Temporary buffer	*/

unsigned char *zptr;	/* Pointer to buffer		*/

long dtype;			/* Data type		*/
long i;				/* Counter		*/
long ns;			/* Size of the packed video data (bytes) */
long nwords;			/* Number of 16-bit words */
long nbands;			/* Number of bands	*/
long swap = 0;			/* Swap flag		*/


/* In some cases, the video data has some trailing bits used to space to the
   end of a word boundary (see level1bio.h).  Any of these trailing bits not
   containing actual video data should be set to zero.  To ensure that this is
   the case, set the bits within the last 8 bytes of video data to zero.
*/
zptr =  obuf + videosize - 4;
for (i = 0; i < 8; i++, zptr++)
    *zptr = *zptr & 0x00;


/* Explaination of algorithm:
     Input data	    Output data		   Action
     ----------------------------------------------------------------
	Packed		Packed		Simple bit copy
	Unpacked	Packed		Pack data
	Packed		Unpacked	Unpack data, window channels
	Unpacked	Unpacked	Window channels
*/


switch (irecsiz)
    {
    case PCK_LAC_5:
	switch (orecsiz)
	    {
	    case PCK_LAC_5:
		/* Packed input to packed output */
		/* ------------------------------*/
		dtype = EBYTE;
		ns = V_PCK_LAC_5;
		c_pxcopy(ibuf, obuf, &dtype, &ns);
		break;
	    case UNP_LAC_1:
	    case UNP_LAC_2:
	    case UNP_LAC_3:
	    case UNP_LAC_4:
	    case UNP_LAC_5:
		/* Packed input to unpacked output */
		/* --------------------------------*/
		nbands = 5;
		nwords = LAC_SAMPS * nbands;
		c_unpck32(ibuf, (short *)tbuf, &nwords, &swap);
		copy_chan((short *)tbuf,(short *)obuf,ichan,ochan,LAC_SAMPS);
		break;
	    }
	break;
    case PCK_GAC_5:
	switch (orecsiz)
	    {
	    case PCK_GAC_5:
		/* Packed input to packed output */
		/* ------------------------------*/
		dtype = EBYTE;
		ns = V_PCK_GAC_5;
		c_pxcopy(ibuf, obuf, &dtype, &ns);
		break;
	    case UNP_GAC_1:
	    case UNP_GAC_2:
	    case UNP_GAC_3:
	    case UNP_GAC_4:
	    case UNP_GAC_5:
		/* Packed input to unpacked output */
		/* --------------------------------*/
		nbands = 5;
		nwords = GAC_SAMPS * nbands;
		c_unpck32(ibuf, (short *)tbuf, &nwords, &swap);
		copy_chan((short *)tbuf,(short *)obuf,ichan,ochan,GAC_SAMPS);
		break;
	    }
	break;
    case UNP_LAC_5:
	switch (orecsiz)
	    {
	    case PCK_LAC_5:
		/* Unpacked input to packed output */
		/* --------------------------------*/
		nwords = LAC_SAMPS * 5;
		c_pck32((short *)ibuf, obuf, &nwords);
		break;
	    case UNP_LAC_1:
	    case UNP_LAC_2:
	    case UNP_LAC_3:
	    case UNP_LAC_4:
	    case UNP_LAC_5:
		/* Unpacked input to unpacked output */
		/* ----------------------------------*/
		copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,LAC_SAMPS);
		break;
	    }
	break;
    case UNP_LAC_1:
    case UNP_LAC_2:
    case UNP_LAC_3:
    case UNP_LAC_4:
	/* Unpacked input to unpacked output */
	/* ----------------------------------*/
	copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,LAC_SAMPS);
	break;
    case UNP_GAC_5:
	switch (orecsiz)
	    {
	    case PCK_GAC_5:
		/* Unpacked input to packed output */
		/* --------------------------------*/
		nwords = GAC_SAMPS * 5;
		c_pck32((short *)ibuf, obuf, &nwords);
		break;
	    case UNP_GAC_1:
	    case UNP_GAC_2:
	    case UNP_GAC_3:
	    case UNP_GAC_4:
	    case UNP_GAC_5:
		/* Unpacked input to unpacked output */
		/* ----------------------------------*/
		copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,GAC_SAMPS);
		break;
	    }
	break;
    case UNP_GAC_1:
    case UNP_GAC_2:
    case UNP_GAC_3:
    case UNP_GAC_4:
	/* Unpacked input to unpacked output */
	/* ----------------------------------*/
	copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,GAC_SAMPS);
	break;
    
    /* NOAA KLM section
    -------------------*/
    case KLM_PCK_LAC_5:
	switch (orecsiz)
	    {
	    case KLM_PCK_LAC_5:
		/* Packed input to packed output */
		/* ------------------------------*/
		dtype = EBYTE;
		ns = V_PCK_LAC_5;
		c_pxcopy(ibuf, obuf, &dtype, &ns);
		break;
	    case KLM_UNP_LAC_1:
	    case KLM_UNP_LAC_2:
	    case KLM_UNP_LAC_3:
	    case KLM_UNP_LAC_4:
	    case KLM_UNP_LAC_5:
		/* Packed input to unpacked output */
		/* --------------------------------*/
		nbands = 5;
		nwords = LAC_SAMPS * nbands;
		c_unpck32(ibuf, (short *)tbuf, &nwords, &swap);
		copy_chan((short *)tbuf,(short *)obuf,ichan,ochan,LAC_SAMPS);
		break;
	    }
	break;
    case KLM_PCK_GAC_5:
	switch (orecsiz)
	    {
	    case KLM_PCK_GAC_5:
		/* Packed input to packed output */
		/* ------------------------------*/
		dtype = EBYTE;
		ns = V_PCK_GAC_5;
		c_pxcopy(ibuf, obuf, &dtype, &ns);
		break;
	    case KLM_UNP_GAC_1:
	    case KLM_UNP_GAC_2:
	    case KLM_UNP_GAC_3:
	    case KLM_UNP_GAC_4:
	    case KLM_UNP_GAC_5:
		/* Packed input to unpacked output */
		/* --------------------------------*/
		nbands = 5;
		nwords = GAC_SAMPS * nbands;
		c_unpck32(ibuf, (short *)tbuf, &nwords, &swap);
		copy_chan((short *)tbuf,(short *)obuf,ichan,ochan,GAC_SAMPS);
		break;
	    }
	break;
    case KLM_UNP_LAC_5:
	switch (orecsiz)
	    {
	    case KLM_PCK_LAC_5:
		/* Unpacked input to packed output */
		/* --------------------------------*/
		nwords = LAC_SAMPS * 5;
		c_pck32((short *)ibuf, obuf, &nwords);
		break;
	    case KLM_UNP_LAC_1:
	    case KLM_UNP_LAC_2:
	    case KLM_UNP_LAC_3:
	    case KLM_UNP_LAC_4:
	    case KLM_UNP_LAC_5:
		/* Unpacked input to unpacked output */
		/* ----------------------------------*/
		copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,LAC_SAMPS);
		break;
	    }
	break;
    case KLM_UNP_LAC_1:
    case KLM_UNP_LAC_2:
    case KLM_UNP_LAC_3:
    case KLM_UNP_LAC_4:
	/* Unpacked input to unpacked output */
	/* ----------------------------------*/
	copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,LAC_SAMPS);
	break;
    case KLM_UNP_GAC_5:
	switch (orecsiz)
	    {
	    case KLM_PCK_GAC_5:
		/* Unpacked input to packed output */
		/* --------------------------------*/
		nwords = GAC_SAMPS * 5;
		c_pck32((short *)ibuf, obuf, &nwords);
		break;
	    case KLM_UNP_GAC_1:
	    case KLM_UNP_GAC_2:
	    case KLM_UNP_GAC_3:
	    case KLM_UNP_GAC_4:
	    case KLM_UNP_GAC_5:
		/* Unpacked input to unpacked output */
		/* ----------------------------------*/
		copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,GAC_SAMPS);
		break;
	    }
	break;
    case KLM_UNP_GAC_1:
    case KLM_UNP_GAC_2:
    case KLM_UNP_GAC_3:
    case KLM_UNP_GAC_4:
	/* Unpacked input to unpacked output */
	/* ----------------------------------*/
	copy_chan((short *)ibuf,(short *)obuf,ichan,ochan,GAC_SAMPS);
	break;
    }

}
