/*****************************************************************************
FUNCTION:       PUT_REC_TAPE

PURPOSE:        Write a record of Level 1b data to tape.  If EOT is
		encountered a new tape is mounted and record is written
		to the new tape.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    09/97  D.Lloyd         Initial development
  1.1    10/97  D.Lloyd		Removed code to span tape.  If physical EOT,
				write tape marks, close tape and return.
  2.0  03/2002  D.Lloyd         Added support for byte-swapping on little
                                endian systems.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"


int put_rec_tape
(
    long *tun,             /* tape logical unit number             */
    unsigned char *recptr, /* pointer to level 1b scan record      */
    long *nbyte,	   /* number of bytes in one record        */
    long *tape             /* tape open flag                       */
)

{
long    mode = HANDLE;          /* handle tape errors                   */
long    rewopt = REWOFFL;       /* rewind off-line flag                 */
long    status;                 /* funtion return status                */
long    two = 2;                /* value of two                         */

static unsigned char *outbuf = NULL; /* temporary buffer so that the    */
                                     /* input buffer is not corrupted   */

#ifdef LAS_LITTLE_ENDIAN
/* For little endian systems, allocate and copy the data record to outbuf
Only allocate once and allow the system to unallocate on exit.
-------------------------------------------------------------------------*/
if ( outbuf == NULL )
    {
    outbuf = (unsigned char *)malloc( MAXLINSIZ );
    if ( outbuf == NULL )
        return E_FAIL;
    }

memcpy( outbuf, recptr, *nbyte );
#else
/* Otherwise just point to the record
-------------------------------------*/
outbuf = recptr;
#endif


/* Insure the data buffer is in the proper endian format
--------------------------------------------------------*/
switch ( *nbyte )
    {
    case KLM_PCK_LAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LP5 );
        break;
    case KLM_UNP_LAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU5 );
        break;
    case KLM_UNP_LAC_4 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU4 );
        break;
    case KLM_UNP_LAC_3 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU3 );
        break;
    case KLM_UNP_LAC_2 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU2 );
        break;
    case KLM_UNP_LAC_1 : export_endian_mixed_buffer( outbuf, KLMDATAREC_LU1 );
        break;
    case KLM_PCK_GAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GP5 );
        break;
    case KLM_UNP_GAC_5 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU5 );
        break;
    case KLM_UNP_GAC_4 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU4 );
        break;
    case KLM_UNP_GAC_3 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU3 );
        break;
    case KLM_UNP_GAC_2 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU2 );
        break;
    case KLM_UNP_GAC_1 : export_endian_mixed_buffer( outbuf, KLMDATAREC_GU1 );
        break;

    case PCK_LAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_LP5 ); break;
    case UNP_LAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_LU5 ); break;
    case UNP_LAC_4 : export_endian_mixed_buffer( outbuf, DATAREC_LU4 ); break;
    case UNP_LAC_3 : export_endian_mixed_buffer( outbuf, DATAREC_LU3 ); break;
    case UNP_LAC_2 : export_endian_mixed_buffer( outbuf, DATAREC_LU2 ); break;
    case UNP_LAC_1 : export_endian_mixed_buffer( outbuf, DATAREC_LU1 ); break;
    case PCK_GAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_GP5 ); break;
    case UNP_GAC_5 : export_endian_mixed_buffer( outbuf, DATAREC_GU5 ); break;
    case UNP_GAC_4 : export_endian_mixed_buffer( outbuf, DATAREC_GU4 ); break;
    case UNP_GAC_3 : export_endian_mixed_buffer( outbuf, DATAREC_GU3 ); break;
    case UNP_GAC_2 : export_endian_mixed_buffer( outbuf, DATAREC_GU2 ); break;
    case UNP_GAC_1 : export_endian_mixed_buffer( outbuf, DATAREC_GU1 ); break;
    }  /* END switch */


/*
  Write the record and get the write status
*/
status = c_tpwrit( tun, outbuf, nbyte, &mode );
if ( status == *nbyte )
    return E_SUCC;

/*
  An error occurred, check for end of tape or write error.
*/
else
    {
    /* If EOT, write tape mark, close tape and return error */
    if ( status == TAPE_EOT )
        {
	c_lasmsg(FALSE,"put_rec_tape-eot",
		 "EOT encountered: record not written");
        status = c_tpclos( tun, &mode, &rewopt, &two );
        *tape = FALSE;
        return status;
        }  /* END if */

    else
        {
        c_lasmsg(FALSE,"put_rec_tape-write","Tape write error");
        return status;
        }  /* END else */

    }  /* END else */


}  /* END put_rec_tape */
