/****************************************************************************
NAME:		ATTITUDE

PURPOSE:	Calculate the spacecraft attitude vectors

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <math.h>
#include "worgen.h"
#include "vector.h"
#include "constants.h"
#include "model.h"
#include "local_protos.h"

void attitude
(
    struct STATE *sat,	 	/* Satellite velocity vector		     */
    struct MVECTOR *nadvec, 	/* Vector from satellite to nadir point	     */
    double *roll,		/* Roll correction			     */
    double *pitch,		/* Pitch correction			     */
    double *yaw,		/* Yaw correction			     */
    struct ATTITUDE *att    	/* Attitude correction vectors information   */
)
{
struct MVECTOR vvec;	 /* Velocity vector				     */
struct MVECTOR xvec;	 /* Vector in the direction of geocentric nadir	     */
struct MVECTOR yvec;	 /* Vector in the direction of angular momentum	     */
struct MVECTOR zvec;	 /* Vector in the direction of circular velocity     */
struct MVECTOR nxvec;	 /* New position vector with pitch correction	     */
struct MVECTOR nyvec;	 /* New angular momentum vector pitch/yaw corrections*/
struct MVECTOR nzvec;	 /* New direction of motion vec with pitch correction*/
struct MVECTOR nadir;	 /* Satellite looking at geodetic nadir		     */

double	troll,		 /* Roll correction for geodetic nadir		     */
	tpitch,		 /* Pitch correction for geodetic nadir		     */
	tyaw,		 /* Yaw correction for geodetic nadir		     */
	sat_alt,	 /* Satellite altitude				     */
	rearth,		 /* Radius of the earth				     */
	amdot,		 /* mean motion */
	row,		 /* */
	row2,		 /* */
	cosque,		 /* */
	we,		 /* Rotation of the earth			     */
	a, b, c,	 /* Temp variables for calculations		     */
	dtime,		 /* Delta time correction			     */
	slat,		 /* Satellite latitude				     */
	slon,		 /* Satellite longitude				     */
	srad,		 /* Satellite radius				     */
	magy,		 /* Magnitude of the angular momentum vector	     */
	magz,		 /* Magnitude of the velocity vector		     */
	maxlat,		 /* Maximum satellite latitude			     */
	vmag = 1.0;	 /* Magnitude of unit velocity vector		     */

/*
  Maximum satellite latitude
*/
maxlat = 82.0 * RAD_PER_DEG;

/*
  Calculate the negative unit vector in the direction of geocentric nadir
*/
xvec.x = -sat->target.x / sat->mag;
xvec.y = -sat->target.y / sat->mag;
xvec.z = -sat->target.z / sat->mag;

/*
  Calculate the unit vector for the velocity of the satellite
*/
vvec.x = sat->vel.x / sat->vmag;
vvec.y = sat->vel.y / sat->vmag;
vvec.z = sat->vel.z / sat->vmag;

/*
  Find vector in the direction of the angular momentum
*/
cross(&vvec,&xvec,&yvec);

/*
  Find vector in the direction of the velocity
*/
cross(&xvec,&yvec,&zvec);

magy = norm(&yvec);
magz = norm(&zvec);

yvec.x = yvec.x / magy;
yvec.y = yvec.y / magy;
yvec.z = yvec.z / magy;

zvec.x = zvec.x / magz;
zvec.y = zvec.y / magz;
zvec.z = zvec.z / magz;


/*
  Determine the altitude of the satellite and the radius of the earth
*/
sat_alt = norm(nadvec);
rearth = sat->mag - sat_alt;

/*
  The update routine that calculates the pitch values causes the point on
  the earth to be imaged at a different time, which means the point on the
  ground has moved with respect to the satellite, some of the pitch value
  goes into a roll value and the original pitch values is reduced.
*/
if (fabs(*pitch) > 0.000001)
    {
    cart2sph(&sat->target,&slat,&slon,&srad);
    amdot = sqrt(AMU / (sat->mag * sat->mag * sat->mag));
    row = asin(sin(*pitch) * sat->mag / rearth) - *pitch;
    cosque = cos(maxlat) / cos(slat);
    we = WE * cos(slat);

    /*
      Time difference is t = (-b +/- sqrt(b * b - 4 * a * c)) / 2 * a
    */
    a = -2.0 * amdot * we * cosque - 2.0 * amdot * amdot;
    b = -2.0 * we * row * cosque;
    c = 2.0 * row * row;

    if (*pitch > 0.0)
        dtime = (-b - sqrt(b * b - 4.0 * a * c)) / (2.0 * a);
    else
        dtime = (-b + sqrt(b * b - 4.0 * a * c)) / (2.0 * a);

    row2 = amdot * dtime * rearth;
    tpitch = atan(row2 / sat_alt);

    if ((dtime*dtime*(amdot*amdot+we*we+2.0*amdot*we*cosque)-row*row) < 0.0) 
        troll  = *roll;
    else
	{
        troll  = sqrt(dtime * dtime* (amdot * amdot + we * we +
		    2.0 * amdot * we * cosque) - row * row) * rearth;
	troll = (atan(troll / sat_alt) *
		-fabs(sat->target.z) / sat->target.z *
		fabs(dtime) / dtime *
		fabs(sat->vel.z) / sat->vel.z) +
		*roll;
	}
    }
else
    {
    tpitch = 0.0;
    troll  = *roll;
    }

/*
  Satellite looking at geodetic nadir
*/
nadir.x = dot(nadvec,&xvec);
nadir.y = dot(nadvec,&yvec);
nadir.z = dot(nadvec,&zvec);

/*
  Find the roll, pitch and yaw values due to the satellite looking
  at geodetic nadir
*/
tpitch += atan(nadir.z / nadir.x);
troll  += atan(nadir.y / nadir.x);
tyaw   = *yaw;

/*
  Use the roll correction to calculate the new position vector
*/
newvec(&xvec,&yvec,0,&troll,&vmag,&nxvec);

/*
  Use the direction of velocity and the new position vector to calculate
  the new angular momentum vector
*/
cross(&zvec,&nxvec,&nyvec);

/*
  Use the pitch correction to calculate the attitude position vector
*/
newvec(&nxvec,&zvec,0,&tpitch,&vmag,&att->posvec);

/*
  Use the attitude position vector to find the new direction of velocity vector
*/
cross(&att->posvec,&nyvec,&nzvec);

/*
  Use the yaw correction to calculate the attitude direction of motion vector
*/
newvec(&nzvec,&nyvec,0,&tyaw,&vmag,&att->dirvec);

/*
  Use the attitude direction of motion vector and the attitude postion
  vector to calculate the attitude angular momentum vector.
*/
cross(&att->dirvec,&att->posvec,&att->angvec);

return;
}
 
