/******************************************************************************
NAME:		BROUWCORRECT

PURPOSE:	Convert mean element values into osculating elements.

PROGRAM HISTORY:
VERSION         DATE	AUTHOR		CODE/CONT	COMMENTS
-------		----	------		---------	--------
  1.0		1/70    Unknown		   ESA		Original Development
  2.0		2/91	C. Wivell	   CSB		Converted to C for LAS50

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
	'' BROUWER-LYDDANE ORBIT GENERATOR ROUTINE ''
		   (X-553W-70-223)
		BY E.A. GALBREATH 1970
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "brouwdef.h"
#include "constants.h"
#include "model.h"
#include "local_protos.h"

void brouwcorrect
(
    double dpele[],	/* Mean elements		*/
    double oselem[]	/* Oscillating elements		*/
)

{
double	adp,edp,bidp,hdp,gdp,bldp,h0,
	g0,bl0,sinhdp,coshdp,edp2,cn2,cn,gm2,
	gmp2,gm4,gmp4,theta,theta2,theta4,gm3,gmp3,gm5,
	gmp5,g3dg2,g4dg2,g5dg2,a1,a2,a3,a4,a5,a6,sini,a10,
	a7,a8p,a8,b13,b14,b15,a11,a12,a13,a14,a17,a15,
	a16,a18,a19,a21,a22,sini2,cosi2,tani2,a26,a27,
	b1,b2,b3,b4,b5,b6,b7,b8,b9,b10,b11,b12,
	a,e,bi,h,g,bl,eadp,sinde,cosde,sinfd,
	cosfd,fdp,dadr,cs2gfd,dadr2,dadr3,
	cosfd2,sn2gfd,snf2gd,csf2gd,sn2gd,cs2gd,sn3gd,
	cs3gd,sn3fgd,cs3fgd,singd,cosgd,dlt1e,blghp,edpdl,
	dlti,sindh,blgh,dlte,edpdl2,edpde2,sindh2,
	squar,sqri,sinldp,cosldp,arg1,arg2;

long	nn;

adp  = dpele[0];
edp  = dpele[1];
bidp = dpele[2];
gdp  = dpele[3];
hdp  = dpele[4];
bldp = dpele[5];

h0 = hdp;
g0 = gdp;
bl0 = bldp;
sinhdp = sin(hdp);
coshdp = cos(hdp);

/*
  Compute mean motion
*/
edp2 = edp * edp;
cn2 = 1.0 - edp2;
cn = sqrt(cn2);
gm2 = BK2 / (adp * adp);
gmp2 = gm2 / (cn2 * cn2);
gm4 = BK4 / (adp * adp * adp * adp);
gmp4 = gm4 / (cn * cn * cn * cn * cn * cn * cn * cn);
theta = cos(bidp);
theta2 = theta * theta;
theta4 = theta2 * theta2;

gm3 = BK3 / (adp * adp * adp);
gmp3 = gm3 / (cn2 * cn2 * cn2);
gm5 = BK5 / (adp * adp * adp * adp * adp);
gmp5 = gm5 / (cn2 * cn2 * cn2 * cn2 * cn2);
g3dg2 = gmp3 / gmp2;
g4dg2 = gmp4 / gmp2;
g5dg2 = gmp5 / gmp2;

/*
  Compute a1-a8
*/
a1 = (F1D8 * gmp2 * cn2) * (1.0 - 11.0 * theta2
   - ((40.0 * theta4) / (1.0 - 5.0 * theta2)));
a2 = (F5D12 * g4dg2 * cn2) * (1.0 - ((8.0 * theta4) / (1.0 - 5.0 * theta2)) 
   -3.0 * theta2);
a3 = g5dg2 * ((3.0 * edp2) + 4.0);
a4 = g5dg2 * (1.0 - (24.0 * theta4) / (1.0 - 5.0 * theta2) - 9.0 *theta2);
a5 = (g5dg2 * (3.0 * edp2 + 4.0)) * (1.0-(24.0 * theta4) / (1.0 - 5.0 * theta2)
   - 9.*theta2);
a6 = g3dg2 * F1D4;
sini = sin(bidp);
a10 = cn2 * sini;
a7 = a6* a10;
a8p = g5dg2 * edp * (1.0-(16.0 * theta4) / (1.0 - 5.0 * theta2) - 5.0 * theta2);
a8 = a8p * edp;

/*
  Compute b13-b15
*/
b13 = edp * (a1 - a2);
b14 = a7 + F5D64 * a5 * a10;
b15 = a8 * a10 * F35384;

/*
  Compute a11-a27
*/
a11 = 2.0 + edp2;
a12 = 3.0 *edp2 + 2.0;
a13 = theta2 * a12;
a14 = (5.0 * edp2 + 2.0) * (theta4 / (1.0 - 5.0 * theta2));
a15 = (edp2 * theta4 * theta2) / ((1.0 - 5.0 * theta2) * (1.0 - 5.0 * theta2));
a16 = theta2 / (1.0 - 5.0 * theta2);
a17 = theta4 / ((1.0 - 5.0 * theta2) * (1.0 - 5.0 * theta2));
a18 = edp * sini;
a19 = a18 / (1.0 + cn);
a21 = edp * theta;
a22 = edp2 * theta;
sini2 = sin(bidp / 2.0);
cosi2 = cos(bidp / 2.0);
tani2 = tan(bidp / 2.0);
a26 = 16.0 * a16 + 40.0 * a17 + 3.0;
a27 = a22 * F1D8 * (11. + 200.0 * a17 + 80.0 * a16);

/*
  Compute b1-b12
*/
b1 = cn * (a1 - a2) - ((a11 - 400.0 * a15 - 40.0 * a14 - 11.0 * a13) *
     F1D16 + (11.0 + 200.0 * a17 + 80.0 * a16) * a22 * F1D8) * gmp2 +
     ((-80.0 * a15 - 8.0 * a14 - 3.0 * a13 + a11) * F5D24 + F5D12 * a26 * a22) *
     g4dg2;

b2 = a6 * a19 * (2.0 + cn - edp2) + F5D64 * a5 * a19 * cn2 - F15D32 * a4 * a18 *
     cn*cn2 + (F5D64 * a5 + a6) * a21 * tani2 + (9.0 * edp2 + 26.0) *
     F5D64 * a4 * a18 + F15D32 * a3 * a21 * a26 * sini * (1.0 - theta);

b3 = ((80.0 * a17 + 5.0 + 32.0 * a16) * a22 * sini * (theta - 1.0) * F35576 *
     g5dg2 * edp) - ((a22 * tani2 + (2.0 * edp2 + 3.0 * (1.0 - cn2 * cn)) *
     sini) * F35D52 * a8p);

b4 = cn * edp * (a1 - a2);
b5 = ((9.0 * edp2 + 4.0) * a10 * a4 * F5D64 + a7) * cn;
b6 = F35384 * a8 * cn2 * cn * sini;

b7 = ((cn2 * a18) / (1.0 - 5.0 * theta2)) * (F1D8 * gmp2 * (1.0 - 15.0 *
     theta2) + (1.0 - 7.0 * theta2) * g4dg2 * (-F5D12));

b8 = F5D64 * (a3 * cn2 * (1.0 - 9.0 * theta2 - (24.0 * theta4  /
     (1.0 - 5.0 * theta2)))) + a6 * cn2;

b9 = a8 * F35384 * cn2;
b10 = sini * (a22 * a26 * g4dg2 * F5D12 - a27 * gmp2);
b11 = a21 * (a5 * F5D64 + a6 + a3 * a26 * F15D32 * sini * sini);
b12 = -((80.0 * a17 + 32.0 * a16 + 5.0) * (a22 * edp * sini * sini * F35576 *
      g5dg2) + (a8 * a21 * F35D52));

/*
  Compute secular terms
*/

/*
  '' mean '' mean anomaly
*/
bldp = bl0;
bldp = fmod(bldp,PIE2);
if (bldp < 0.0)
    bldp = bldp + PIE2;

/*
  Mean argument of perigee
*/
gdp = g0;
gdp = fmod(gdp,PIE2);
if (gdp < 0.0)
    gdp = gdp + PIE2;
 
/*
  Mean longitude of ascending node
*/
hdp = h0;
hdp = fmod(hdp,PIE2);
if (hdp < 0.0)
    hdp = hdp + PIE2;

for(nn = 0; nn < 6; nn++)
    oselem[nn] = dpele[nn];

a = adp;
e = edp;
bi = bidp;
h = hdp;
g = gdp;
bl = bldp;

/*
  Compute true anomaly (double primed)
*/
dkeplr(bldp,edp,&eadp);

sinde = sin(eadp);
cosde = cos(eadp);
sinfd = cn * sinde;
cosfd = cosde - edp;

fdp = atan2(sinfd,cosfd);
if (fdp < 0)
    fdp = fdp + PIE2;

dadr = 1.0 / (1.0 - edp * cosde);
sinfd = sinfd * dadr;
cosfd = cosfd * dadr;
cs2gfd = cos(2.0 * gdp + 2.0 * fdp);
dadr2 = dadr * dadr;
dadr3 = dadr2 * dadr;
cosfd2 = cosfd * cosfd;

/*
  Compute a(semi-major axis)
*/
a = adp * (1.0 + gm2 * ((3.0 * theta2 - 1.0) * (edp2 / (cn2 * cn2 * cn2)) *
    (cn + (1.0 / (1.0 + cn))) + ((3.0 * theta2 - 1.0) / (cn2 * cn2 * cn2)) *
    (edp * cosfd) * (3.0 + 3.0 * edp * cosfd + edp2 * cosfd2) + 3.0 *
    (1.0 - theta2) * dadr3 * cs2gfd));

sn2gfd = sin(2.0 * gdp + 2.0 * fdp);
snf2gd = sin(2.0 * gdp + fdp);
csf2gd = cos(2.0 * gdp + fdp);
sn2gd  = sin(2.0 * gdp);
cs2gd  = cos(2.0 * gdp);                                                       
sn3gd  = sin(3.0 * gdp);
cs3gd  = cos(3.0 * gdp);
sn3fgd = sin(3.0 * fdp+2.0 * gdp);
cs3fgd = cos(3.0 * fdp+2.0 * gdp);
singd  = sin(gdp);
cosgd  = cos(gdp);
dlt1e  = b14 * singd + b13 * cs2gd - b15 * sn3gd;

/*
  Compute (l+g+h) primed
*/
blghp = hdp + gdp + bldp + b3 * cs3gd + b1 * sn2gd + b2 * cosgd;
blghp = fmod(blghp,PIE2);
if (blghp < 0.0)
    blghp = blghp + PIE2;

edpdl = b4 * sn2gd - b5 * cosgd + b6 * cs3gd - F1D4 * cn2 * cn * gmp2 * 
	(2.0 * (3.0 * theta2 - 1.0) * (dadr2 * cn2 + dadr + 1.0) * 
	sinfd + 3.0 * (1.0 - theta2) * ((-dadr2 * cn2 - dadr + 1.0) *
	snf2gd + (dadr2 * cn2 + dadr + F1D3) * sn3fgd));

dlti  = F1D2 * theta * gmp2 * sini * (edp * cs3fgd + 3.0 * (edp * csf2gd + 
	cs2gfd)) - (a21 / cn2) * (b8 * singd + b7 * cs2gd - b9 * sn3gd);

sindh = (1.0 / cosi2) * (F1D2 * (b12 * cs3gd + b11 * cosgd + b10 * sn2gd -
	(F1D2 * gmp2 * theta * sini * (6.0 * (edp * sinfd - bldp + fdp) -
	(3.0 * (sn2gfd + edp * snf2gd) + edp * sn3fgd)))));

/*
  Compute (l+g+h)
*/
blgh  = blghp + ((1.0 / (cn + 1.0)) * F1D4 * edp * gmp2 * cn2 * (3.0 *
	(1.0 - theta2) * (sn3fgd * (F1D3 + dadr2 * cn2 + dadr) + snf2gd *
	(1.0 - (dadr2 * cn2 + dadr))) + 2.0 * sinfd * (3.0 * theta2 - 1.0) *
	(dadr2 * cn2 + dadr + 1.0))) + gmp2 * F3D2 * ((-2.0 * theta - 1.0 +
	5.0 * theta2) * (edp * sinfd + fdp - bldp)) + (3.0 + 2.0 * theta - 5.0 *
	theta2) * (gmp2 * F1D4 * (edp * sn3fgd + 3 * (sn2gfd + edp * snf2gd)));

blgh  = fmod(blgh,PIE2);
if (blgh < 0.0)
    blgh = blgh + PIE2;

dlte  = dlt1e + (F1D2 * cn2 * ((3.0 * (1.0 / (cn2 * cn2 * cn2)) * gm2 *
	(1.0 - theta2) * cs2gfd * (3.0 * edp * cosfd2 + 3.0 * cosfd + edp2 *
	cosfd * cosfd2 + edp)) - (gmp2 * (1.0 - theta2) * (3.0 * csf2gd +
	cs3fgd)) + (3.0 * theta2 - 1.0) * gm2 * (1.0 / (cn2 * cn2 * cn2)) *
	(edp * cn + (edp / (1.0 + cn)) + 3.0 * edp * cosfd2 + 3.0 * cosfd +
	edp2 * cosfd * cosfd2)));
edpdl2 = edpdl * edpdl;
edpde2 = (edp + dlte) * (edp + dlte);

/*
  Compute e(eccentricity)
*/
e = sqrt(edpdl2 + edpde2);
sindh2 = sindh * sindh;
sini2 = sin(bidp / 2.0);
cosi2 = cos(bidp / 2.0);
squar = (dlti * cosi2 * F1D2 + sini2) * (dlti * cosi2 * F1D2 + sini2);
sqri = sqrt(sindh2 + squar);

/*
  Compute bi (inclination)
*/
bi = asin(sqri);
bi = 2.0 * bi;
bi = fmod(bi,PIE2);
if (bi < 0.0)
    bi = bi + PIE2;

/*
  Check for eccentricity=0
*/
if (e != 0.0)
    {
    sinldp = sin(bldp);
    cosldp = cos(bldp);
    sinhdp = sin(hdp);
    coshdp = cos(hdp);

    /*
      Compute mean anomaly
    */
    arg1 = edpdl * cosldp + (edp + dlte) * sinldp;
    arg2 = (edp + dlte) * cosldp - (edpdl * sinldp);

    bl = atan2(arg1,arg2);
    if (e < 0)
	bl = bl + PIE;

    bl = fmod(bl,PIE2);
    if (bl < 0.0)
	bl = bl + PIE2;
    } 
else
    bl=0.0;

/*
  Check for bi(inclination)=0
  Compute h(longitude of ascending node)
*/
if (bi != 0.0)
    {
    arg1 = sindh * coshdp + sinhdp * (F1D2 * dlti * cosi2 + sini2);
    arg2 = coshdp * (F1D2 * dlti * cosi2 + sini2) - (sindh * sinhdp);
    h = atan2(arg1,arg2);
    h = fmod(h,PIE2);
    if (h < 0.)
	h = h + PIE2;
    }
else
    h = 0.0;

/*
  Compute g(argument of perigee)
*/
g = blgh - bl - h;
if (e < 0)
    g = g + PIE;

g = fmod(g,PIE2);
if (g < 0.0)
    g = g + PIE2;

oselem[0] = a;
oselem[1] = fabs(e);
oselem[2] = bi;
oselem[3] = g;
oselem[4] = h;
oselem[5] = bl;

return;
}
 
