/*****************************************************************************
NAME:		BROUWER

PURPOSE:	Propagate the orbital elements from epoch to the time of 
		interest.

PROGRAM HISTORY:
VERSION         PROGRAMMER           DATE              COMMENTS
-------         ----------           ----              --------
2.0              WIVELL              2/91              Original development

COMPUTER SOFTWARE OR HARDWARE LIMITATIONS:

PROJECT:             ADAPS

ALGORITHM DESCRIPTION:
Store or calculate old orbital elements
New orbital elements at time t
Count orbits and make m less than PIE2
Make sure m is not less than zero
Change mean elements into oscilating elements
Find new eccentric anomaly
Find the true anomaly
Find true anomaly passed the ascending node
Find orbital radius of the satellite at thetas
Is this time an ascending pass zdot=+ yes, zdot=- no
Call latlong to find satellite's lat and long

ALGORITHM REFERENCES:

******************************************************************************/
#include <math.h>
#include "worgen.h"
#include "avhrr.h"
#include "model.h"
#include "constants.h"
#include "local_protos.h"

void brouwer
(
    double *t,			/* Total seconds past epoch		*/
    struct MODEL *model,	/* Model and orbital information	*/
    double *truea,		/* True anomaly past perigee		*/
    double *rsat,		/* Distance of satellite from earth center*/
    double *thetas,		/* True anomaly past equator		*/
    double *anlng,		/* Longitude of ascending node		*/
    double *inclin,		/* Inclination angle			*/
    long *zdot,			/* Ascending(1) / descending(-1) flag	*/
    long *eorbit,		/* Orbit counter from equator 		*/
    long *porbit, 		/* Orbit counter from perigee 		*/
    double *slat,		/* Geocenteric latitude of satellite	*/
    double *slong		/* Longitude of satellite		*/
)
{
double	m,		/* Mean anomaly at time t			*/
	w,		/* argument of perigee at time t		*/
	a,		/* Semi-major axis at time t			*/
	ex,		/* Eccentricity					*/
	e,		/* Eccentric anomaly				*/
	thetas0,	/* True anomaly past equator at epoch		*/
	eg,		/* Guess of eccentric anomaly			*/
	ep,		/* Eccentric anomaly prime			*/
	dt,		/* Total hergs from epoch			*/
	we,		/* Rotation rate of earth in radians/herg	*/
	elems[6],	/* Elements to brouwer				*/
	oscil[6];	/* Elements returned from brouwer		*/

long	orbit,		/* Temporary orbital counter			*/
	morbit;		/* Temporary orbital counter			*/
      
/*
  Store old orbital elements
*/
*porbit = model->orbit;
*eorbit = model->orbit;


/*
  New orbital elements at time t
*/
dt = *t / HERG;
we = WE * HERG;
m = model->mean_anomaly + model->mean_motion * dt + model->decay * dt * dt;
w = model->arg_per + model->der_argper * (m - model->mean_anomaly);
*anlng = model->lon_anode + model->der_anode * (m - model->mean_anomaly) - 
	 we * dt;

/* 
  Count the perigee orbits
*/
morbit = (long)(m / PIE2);
*porbit += morbit;

/*
  Count equ. orbits
*/
orbit = (long)((m + w)/PIE2);
*eorbit += orbit;
thetas0 = model->arg_per + model->mean_anomaly;
if (thetas0 > PIE2)
    (*eorbit)--;

/*
  Make mean anomaly (m) less than PIE2.
  Make sure the mean anomaly (m) is not less than zero
*/
m = (m / PIE2 - morbit) * PIE2;
if (m < 0.0)
    m += PIE2;

/*
  Make w less than PIE2.
  Make sure w is not less than zero.
*/
w = (w / PIE2 - (long)(w / PIE2)) * PIE2;
if (w < 0.0)
    w += PIE2;

*anlng = (*anlng / PIE2 - (long)(*anlng / PIE2)) * PIE2;
if (*anlng < 0.0)
    *anlng += PIE2;

a = model->smajor_axis + model->adot * dt;

/*
  Load up an array with the mean elements
*/
elems[0] = a;
elems[1] = model->ecc;
elems[2] = model->inclin;
elems[3] = w;
elems[4] = *anlng;
elems[5] = m;

/*
  Change mean elements into oscilating elements
*/
brouwcorrect (elems,oscil);

/*
  Reassign the mean elements to the oscilating elements
*/
a = oscil[0];
ex = oscil[1];
*inclin = oscil[2];
w = oscil[3];
*anlng = oscil[4];
m = oscil[5];

/*
  Find new eccentric anomaly
*/
eg = m;
do
    {
    e = m + ex * sin(eg);
    ep = ex * cos(eg) - 1.0;
    eg = eg + (eg - e) / ep;
    }
while (fabs(e - eg) > 0.000001);

/*
  Find the true anomaly
*/
*truea=acos((cos(e) - ex) / (1.0 - ex * cos(e)));
if (e > PIE)
    *truea = PIE2 - *truea;

/*
  Find true anomaly passed the ascending node
*/
*thetas = w + *truea;
orbit = (long)( *thetas / PIE2);
*thetas = (*thetas / PIE2 - orbit) * PIE2;
if (*thetas < 0.0)
    *thetas += PIE2;

/*
  Find orbital radius of the satellite at thetas
*/
*rsat = REM * a * (1.0 - ex * ex) / (1.0 + ex * cos(*truea));

/*
  Set the ascending(1) / descending(-1) flag (zdot)
*/
*zdot = ASCENDING;
if ((*thetas > PIE / 2.0) && (*thetas < 1.5 * PIE))
    *zdot = DESCENDING;

/*
  Find satellite's lat and long
*/
blatlong(thetas,anlng,inclin,slat,slong);

return;
}
 
