/****************************************************************************
NAME:		C_FORWARD

PURPOSE:	Forwad avhrr model to convert a line/sample to a
		latitude/longitude

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "model.h"
#include "local_protos.h"

void c_forward
(
    double *line,	/* Input line number				     */
    double *sample,	/* Input sample number				     */
    double *tarelev,	/* Target elevation				     */
    struct MODEL *model,/* Model and orbital information		     */
    double *tarlatd,	/* Target latitude				     */
    double *tarlong,	/* Target longitude				     */
    double *view_angles	/* Calculated view angles			     */
)
{
struct ATTITUDE att;	/* Attitude correction vectors information	     */
struct STATE    sat;	/* Satellite state vector information		     */
struct STATE    tarint;	/* Initial target location information		     */
struct MVECTOR  tarvec;	/* Target vector information			     */
struct MVECTOR  nadvec;	/* Satellite to nadir vector			     */
struct MVECTOR  losv;   /* Line of sight from satellite to target vector     */

double	roll,		/* Roll angle					     */
	pitch,		/* Pitch angle					     */
	yaw,		/* Yaw angle					     */
	elev,		/* Elevation of the target			     */
	tarlatc,	/* Target geocentric latitude			     */
	tarrad,		/* Target radius				     */
	scan_angle,	/* Scan angle from satellite to target		     */
	height,		/* Height of target				     */
	tstime,		/* Total seconds from start time		     */
	midline,	/* Line number at the middle of the pass	     */
	offl = 0.0,	/* Line offset					     */
	offs = 0.0,	/* Sample offset				     */
	Time = 0.0;	/* Time from reference (center of pass)		     */

/*
  Make sure the elevation isn't to negative.  The satellite only images
  the surface of the ocean which has an elevation of zero.
*/
if (*tarelev < -200.0)
    elev = 0.0;
else
    elev = *tarelev;

/*
  Find the time of the line
*/
midline = model->nlines / 2.0;
findtime(line,&offl,&midline,&model->dtype,&model->cpass,&Time);

/*
  Find the scan angle for the sample
*/
findscanangle(sample,&offs,&model->dtype,&model->cpass,&scan_angle);

/*
  Calculate the yaw, roll, and pitch from the attitude correction coefficients
*/
calcyaw(&Time,model->yaw_coefs,&yaw);
calcroll(&Time,model->roll_coefs,&roll);
calcpitch(&Time,model->pitch_coefs,&pitch);

/*
  Move the satellite to the specified time
*/
movesat(&model->satint,model->alt_coefs,&model->mean_motion1,
	&model->mean_motion2,&Time,&sat);

/*
  Calculate the vector from the satellite to nadir
*/
nadirvec(&sat.target,&nadvec);

/*
  Calculate the attitude corrections vector information
*/
attitude(&sat,&nadvec,&roll,&pitch,&yaw,&att);

/*
  Find the line of sight vector from the satellite to the target
*/
findlos(&att,&scan_angle,&losv);

/*
  Find the target vector
*/
findtarvec(&sat.target,&losv,&tarvec,&tarlatc,tarlong,&tarrad);

/*
  Convert the geocentric target latitude to a geodetic target latitude
*/
centh2det(&tarlatc,&tarrad,tarlatd,&height);

/*
  Calculate the initial target vector location from the latitude/longitude
*/
inttar(tarlatd,tarlong,&elev,&tarint);

/*
  Adjust the target longitude by taking into account the rotation of the earth
*/
*tarlong = *tarlong - WE * Time;

/*
  Calculate the view angles for the target
*/
if (view_angles != NULL)
    {
    tstime = model->sec_epoch + model->delta_time + model->scene_ctime + Time;
    calcviews(model,&tstime,&losv,tarlatd,tarlong,&tarint.target,view_angles);
    }

/*
  Check the longitude and correct the sign.  West Greenwich mean is negative.
*/
if (*tarlong > PIE2)
    *tarlong -= PIE2;

if (*tarlong < 0.0)
    *tarlong += PIE2;

if (*tarlong > PIE)
    *tarlong -= PIE2;

return;
}
 
