/*******************************************************************************
NAME:		C_GETORB

PURPOSE:	Calculate the orbit number, equitorial crossing, and
		the direction of the pass.

                NOTE: c_getorb expects tsec to be seconds from 00:00:00 of the
                SCENE'S jday, not the ephem jday, if they are different.  This
                may occur when projecting beyond known ephemeris data, when the
                ephemeris is taken from the last line of the ephemeris table.
                For example, you may be projecting a scene for 08:40:23 of
                8/13/92, but the latest ephemeris line is from 8/10/92.  Then
                tsec should be 31223.0, which is the number of seconds from
                00:00:00 of 8/13.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		2/91	D. Hollaren	   CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <math.h>
#include "worgen.h"
#include "avhrr.h"
#include "constants.h"
#include "model.h"
#include "local_protos.h"

void c_getorb
(
    double *tsec,	/* Total seconds at interest (GMT midnight)	*/
    struct MODEL *model,/* Model and orbital information		*/
    long *orbit,	/* Oribt number 				*/
    long *passdir,	/* Pass direction				*/
    double *eqxing	/* Equitorical crossing				*/
)
{
double	epoch,		/* Total seconds since epoch			*/
	truea,		/* True anomaly					*/
	rsat,		/* Radius for the satellite			*/
	thetas,		/* True anomaly past the equator		*/
	anlng,		/* Ascending node longitude			*/
	inclin,		/* Inclination from brouwer			*/
	slat,		/* Satellite latitude				*/
	slong,		/* Satellite longitude				*/
	thetadot,	/* Mean motion					*/
	ssec,		/* Total seconds at start time (GMT midnight)	*/
	delta;		/* Difference in start an interest		*/

long	porbit;		/* Orbit at perigee				*/

ssec = model->shour * 3600 + model->smin * 60 + model->ssec;
delta = *tsec - ssec;
epoch = model->sec_epoch + delta;

/*
  Propigate the orbit to the start time of the pass and calculate the
  direction of the pass, orbit number referenced from the equator, orbit
  number referenced from perigee, and the satellite lat/long.
*/
brouwer(&epoch,model,&truea,&rsat,&thetas,&anlng,&inclin,passdir,orbit,
	&porbit,&slat,&slong);

/*
  Calculate the mean motion
*/
thetadot = sqrt(AMU / (rsat * rsat * rsat));

/*
  Calculate the equitorial crossing
*/
*eqxing = anlng - (-.004166 * RAD_PER_DEG) * thetas / thetadot;
if (*eqxing < -1.0 * PIE)
    *eqxing = PIE2 + *eqxing;

if (*eqxing > PIE)
    *eqxing = *eqxing - PIE2;

return;
}
 
