/****************************************************************************
NAME:		C_INITMOD

PURPOSE:	Initialize the avhrr model and get information needed for
		access to the model calculations

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "util.h"
#include "model.h"
#include "avhrr.h"
#include "adapsutil.h"
#include "local_protos.h"

int c_initmod
(
    long *satnum,        /* Satellite number                     	     */
    long *year,          /* Year of the pass				     */
    long *month,         /* Month of the pass				     */
    long *day,           /* Day of the pass 				     */
    long *shour,         /* Start hour of pass                   	     */
    long *smin,          /* Start minute of pass                 	     */
    long *ssec,          /* Start second of pass                 	     */
    long *sfrac,         /* Start fractional seconds of pass     	     */
    long *dtype,         /* Type of data LAC(1), GAC(2) ,HRPT(3) 	     */
    long *nlines,	 /* Number of lines in the pass			     */
    struct MODEL *model	 /* Model and orbital information		     */
)
{
long	status,		/* Return status variable			     */
	milsec,		/* Milliseconds at end time			     */
	i;		/* Looping variables				     */

char    eline[EPHEM_SIZ]; /* Ascii ephemris line information		     */

/*
  Set the specified input information into the model structure
*/
model->satnum   = *satnum;
model->year     = *year;
model->month    = *month;
model->day      = *day;
model->shour    = *shour;
model->smin     = *smin;
model->ssec     = *ssec;
model->sfrac    = *sfrac;
model->dtype    = *dtype;
model->nlines   = *nlines;
model->sat_view = TRUE;

c_grejul(&model->year,&model->month,&model->day,&model->jday);

/*
  Read the ephemeris information from the tables
*/
status=c_readephem(&model->satnum,&model->year,&model->month,&model->day,eline);
if (status != E_SUCC)
   return(E_FAIL);

/*
  Initialize and get the initial satellite attitude correction coefficients
*/
model->delta_alt = 0;
for (i = 0; i < 5; i++)
    {
    model->yaw_coefs[i] = 0.0;
    model->roll_coefs[i] = 0.0;
    model->pitch_coefs[i] = 0.0;
    }

status = satatt(&model->satnum,&model->roll_coefs[0],&model->pitch_coefs[0],
		&model->yaw_coefs[0],&model->delta_time);
if (status != E_SUCC)
   return(E_FAIL);

/*
  Set the ephemris data into the model structure
*/
c_setephem(eline,model);

/*
  Set the satellite information in to the model strucure
*/
c_setsat(model);

/*
  Calculate the end time
*/
milsec = (model->ehour * 3600 + model->emin * 60 + model->esec) * 1000 +
	  model->efrac;
if (milsec == 0.0)
    {
    if (model->dtype == GAC)
	{
	milsec = (long)(((model->shour * 3600 + model->smin * 60 + model->ssec)
			  * 1000 + model->sfrac) + (model->nlines / GAC_PER_SEC)
			  * 1000);
	}
    else
	{
	milsec = (long)(((model->shour * 3600 + model->smin * 60 + model->ssec)
			  * 1000 + model->sfrac) + (model->nlines / LAC_PER_SEC)
			  * 1000);
	}

    c_cvtmil(&milsec,&model->ehour,&model->emin,&model->esec,&model->efrac);
    }

/*
  Calculate the nine image lat/long coordinates.  We want c_coords to compute
  the coordinates for each of these coordinates in the "North-up" orientation.
  Therefore, we need to temporarily change the sat_view flag.
*/
model->sat_view = FALSE;
c_coords(model);
model->sat_view = TRUE;

return(E_SUCC);
}
 
