/******************************************************************************
NAME:		C_INVERSE

PURPOSE:	Inverse avhrr model to convert a latitude/longitude to
		a line/sample.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "model.h"
#include "constants.h"
#include "local_protos.h"

void c_inverse
(
    double *tarlatd,		/* Latitude of target			     */
    double *tarlong,		/* Longitude of target			     */
    double *tarelev,		/* Elevation of target			     */
    struct MODEL *model,	/* Model and orbital information	     */
    double *line,		/* Output line in scene			     */
    double *sample,	/* Output sample in scene			     */
    double *view_angles	/* Zenith and relative azimuth angles		     */
)
{
struct ATTITUDE att;	/* Attitude correction vectors information	     */
struct STATE    sat;	/* Satellite state vector information		     */
struct STATE    tarint;	/* Initial target location information		     */
struct STATE    tarvec;	/* Target vector information			     */
struct MVECTOR  nadvec;	/* Satellite to nadir vector			     */
struct MVECTOR  losv;	/* Line of sight from satellite to target vector     */

double	roll,		/* Roll angle					     */
	pitch,		/* Pitch angle					     */
	yaw,		/* Yaw angle					     */
	dma,		/* Delta mean anomaly				     */
	tstime,		/* Total seconds from start time		     */
	pntlng,		/* Positive longitude of the target		     */
	elev,		/* Elevation of the target			     */
	midline,	/* Line at middle of pass			     */
	Time = 0.0,	/* Time from reference				     */
	offl = 0.0,
	offs = 0.0,
	dtime = 1000.0;	/* Change in time from last guess		     */

long	iter;		/* Looping variable				     */

/*
  Make the longitude positive
*/
if (*tarlong < 0.0)
    pntlng = *tarlong + PIE2;
else
    pntlng = *tarlong;

/*
  Make sure the elevation isn't to negative.  The satellite only images
  the surface of the ocean which has an elevation of zero.
*/
if (*tarelev < -200.0)
    elev = 0.0;
else
    elev = *tarelev;

/*
  Get the initial target vector
*/
inttar(tarlatd,&pntlng,&elev,&tarint);

/*
  Find the satellite and target location for the specified latitude/longitude
*/
for (iter = 1; iter <= 10; iter++)
    {
    /*
      Calculate the yaw, roll, and pitch from the attitude correction
      coefficients
    */
    calcyaw(&Time,model->yaw_coefs,&yaw);
    calcroll(&Time,model->roll_coefs,&roll);
    calcpitch(&Time,model->pitch_coefs,&pitch);

    /*
      Move the target and the satellite to the specified time
    */
    movetar(&tarint,&Time,&tarvec);
    movesat(&model->satint,model->alt_coefs,&model->mean_motion1,
	    &model->mean_motion2,&Time,&sat);

    /*
      Calculate the nadir vector from the satellite
    */
    nadirvec(&sat.target,&nadvec);

    /*
      Calculate the attitude correction vectors information
    */
    attitude(&sat,&nadvec,&roll,&pitch,&yaw,&att);

    /*
      Calculate the line of sight from the satellite to the target
    */
    losvec(&sat.target,&tarvec.target,&losv);

    calcdmeanm(&sat,&losv,&att.dirvec,&dma);
    calcdtime(&dma,&model->mean_motion1,&model->mean_motion2,&dtime);

    /*
      Increment the time by the calculated delta time and check
      to see if under the tolerance
    */
    Time += dtime;
    if (fabs(dtime) < 0.01)
	break;
    }

/*
  Calculate the line and sample location
*/
midline = model->nlines / 2.0;
calcline(&Time,&offl,&midline,&model->dtype,&model->cpass,line);
calcsample(&losv,&att.angvec,&offs,&model->dtype,&model->cpass,sample);

/*
  Check the sample to make sure it isn't looking through the earth
*/
chksamp(&sat.target,&tarvec.target,&losv,&att.angvec,&offs,&model->dtype,
	 &model->cpass,sample);

/*
  Calculate the view angles
*/
if (view_angles != NULL)
    {
    tstime = model->sec_epoch + model->delta_time + model->scene_ctime + Time;
    calcviews(model,&tstime,&losv,tarlatd,&pntlng,&tarvec.target,view_angles); 
    }

return;
}
 
