/*******************************************************************************
NAME:		C_READEPHEM

PURPOSE:	Read the ephemeris information from the appropriate file.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		2/91	D. Hollaren	   CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "util.h"
#include "adapsutil.h"
#include "model.h"
#include "avhrr.h"


int c_readephem
(
    long *satnum,		/* Satellite number			*/
    long *year,			/* Year of ephemeris looking for	*/
    long *month,		/* Month of ephemeris looking for	*/
    long *day,			/* Day of ephemeris looking for		*/
    char *eline			/* Ascii ephemris line			*/
)
{
FILE	*fp;			/* File pointer to the ephemeris table  */

char	disk[CMLEN],		/* Disk location of ephemeris tables	*/
	satfile[CMLEN],		/* Ephemeris file			*/
	*ptr,			/* Pointer to ephemris information	*/
	satnam[8],		/* Satellite name			*/
	satno[4],		/* Short satellite name			*/
	satid[6],		/* Satellite id				*/
	satcode[2];		/* Satellite code			*/

long	found,			/* Flag for found ephemeris record	*/
	jdate,			/* Julian date searching for		*/
	eyear,			/* Current ephemeris record year	*/
	emonth,			/* Current ephemeris record month	*/
	eday,			/* Current ephemeris record day		*/
	ejdate,			/* Current ephemeris record julian date	*/
	iwant,			/* The ephemeris date sought (YYYYJJJ)	*/
	itest,			/* Date of the current record (YYYYJJJ)	*/
	upper,			/* Upper binary search limit		*/
	lower,			/* Lower binary search limit		*/
	mid,			/* Mid location for binary search	*/
	nrecs,			/* No. of records in the ephemeris file	*/
	loc,			/* Location in ephemeris file to go to	*/
	pos,			/* Current position in the ephemeris file*/
	nhead,			/* # of header records in ephemris file */
	status,			/* Return status			*/
	nonfatal = 1;		/* Non-fatal error code			*/

/*
  Convert the date to a julian date for easier comparisons.  This is the
  ephemeris information date wanted.
*/
c_grejul(year,month,day,&jdate);
iwant = c_4digitdt(*year * 1000 + jdate);

/*
  Get the ephemeris file name for the specified satellite
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(disk,getenv("ADAPSTABLES"));
else
    {
    c_errmsg("Error retrieving value of ADAPSTABLES","readephem-find",
	     &nonfatal);
    return(E_FAIL);
    }

status = c_getsat("SATNUM",satnum,satnam,satno,satid,satcode);
if (status != E_SUCC)
    return(E_FAIL);

#ifdef VMS
    sprintf(satfile,"%s%s.dat",disk,satid);
#else
    sprintf(satfile,"%s/%s.dat",disk,satid);
#endif

/*
  Open the ephemeris file 
*/
fp = fopen(satfile,"r");
if (fp == NULL)
    {
    c_errmsg("Error opening the ephemeris file","readephem-open",&nonfatal);
    return(E_FAIL);
    }

/*
  Read the first record from the ephemeris file and get the number of records
*/
ptr = fgets(eline,EPHEM_SIZ,fp);
if (ptr == NULL)
    {
    c_errmsg("Error reading the ephemeris file","readephem-read",&nonfatal);
    return(E_FAIL);
    }
sscanf(eline,"%d%d",&nrecs,&nhead);

/*
  Set flag for no ephemeris found yet
*/
found = FALSE;

/*
  Get the first ephemeris record
*/
upper = nhead;			/* Skip any header records	*/
loc = upper * EPHEM_SIZ;
pos = fseek(fp,loc,0);
if (pos != 0)
    {
    c_errmsg("Error seeking in ephemeris file","readephem-seek",&nonfatal);
    return(E_FAIL);
    }
ptr = fgets(eline,EPHEM_SIZ,fp);
if (ptr == NULL)
    {
    c_errmsg("Error reading the ephemeris file","readephem-read",&nonfatal);
    return(E_FAIL);
    }

/*
  Check for the upper bound first
*/
sscanf(&eline[63],"%02d%02d%02d",&eyear,&emonth,&eday);
c_grejul(&eyear,&emonth,&eday,&ejdate);
itest = c_4digitdt(eyear * 1000 + ejdate);
if (itest > iwant)
    {
    c_errmsg("No ephemeris data before the specified date","readephem-noephm",
	     &nonfatal);
    return(E_FAIL);
    }
else if (itest == iwant)
    found = TRUE;

/*
  Get the last ephemeris record
*/
lower = nrecs;			/* Skip any header records	*/
loc = lower * EPHEM_SIZ;
pos = fseek(fp,loc,0);
if (pos != 0)
    {
    c_errmsg("Error seeking in ephemeris file","readephem-seek",&nonfatal);
    return(E_FAIL);
    }
ptr = fgets(eline,EPHEM_SIZ,fp);
if (ptr == NULL)
    {
    c_errmsg("Error reading the ephemeris file","readephem-read",&nonfatal);
    return(E_FAIL);
    }

/*
  Check for the lower bound
*/
sscanf(&eline[63],"%02d%02d%02d",&eyear,&emonth,&eday);
c_grejul(&eyear,&emonth,&eday,&ejdate);
itest = c_4digitdt(eyear * 1000 + ejdate);
if (itest <= iwant)
    found = TRUE;

/*
  Do a binary search to locate the desired record
*/
while (!found)
    {
    /*
      Calculate the middle and read the ephemeris record
    */
    mid = upper + (lower - upper) / 2;
    loc = mid * EPHEM_SIZ;
    pos = fseek(fp,loc,0);
    if (pos != 0)
	{
	c_errmsg("Error seeking in ephemeris file","readephem-seek",&nonfatal);
	return(E_FAIL);
	}
    ptr = fgets(eline,EPHEM_SIZ,fp);
    if (ptr == NULL)
	{
	c_errmsg("Error reading the ephemeris file","readephem-read",&nonfatal);
	return(E_FAIL);
	}

    /*
      See if this is the record wanted
    */
    sscanf(&eline[63],"%02d%02d%02d",&eyear,&emonth,&eday);
    c_grejul(&eyear,&emonth,&eday,&ejdate);
    itest = c_4digitdt(eyear * 1000 + ejdate);
    if ((itest == iwant) || ((lower - upper) <= 1))
	found = TRUE;
    else if (itest < iwant)
	upper = mid;
    else if (itest > iwant)
	lower = mid;
    }

/*
  Close the ephemeris file
*/
fclose(fp);

return(E_SUCC);
}
 
