/****************************************************************************
NAME:		C_SETSAT

PURPOSE:	Set the initial satellite information needed for input to 
		the model

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "local_protos.h"

void c_setsat
(
    struct MODEL *model	/* Model and orbital information		     */
)

{
struct MVECTOR strvec;	/* The vector to the start of the pass		     */
struct MVECTOR cenvec;  /* The vector to the center of the pass		     */
struct MVECTOR endvec;  /* The vector to the end of the pass		     */

double	truea,		/* True anomaly					     */
	thetas,		/* True anomaly passed equator			     */
	anlng,		/* Longitude of ascending node			     */
	inclin,		/* Inclination angle				     */
	radii[5],	/* Satellite radii at start, 1/4, center, 3/4, & end */
	str_latc,  	/* Satellite geocentric latitude at start of pass    */
	str_long,	/* Satellite longitude at start of pass		     */
	t2_latc,	/* Satellite geocentric latitude at 1/4 of pass	     */
	t2_long,	/* Satellite longtidue at 1/4 of pass		     */
	cen_latc,	/* Satellite geocentric latitude at center of pass   */
	cen_long,	/* Satellite longitude at center of pass	     */
	t4_latc,	/* Satellite geocenteric latitude at 3/4 of pass     */
	t4_long,	/* Satellite longitude at 3/4 of pass		     */
	end_latc,	/* Satellite geocenteric latitude at end of pass     */
	end_long,	/* Satellite longtidue at end of pass		     */
	angle1,		/* Angle between the start and center vectors	     */
	angle2,		/* Angle between the end and center vectors	     */
	step_delta,	/* Delta time to add to the temp time		     */
	t1;		/* Temporary time variable			     */

double	vel_angle = VEL_ANGLE;	/* Approx. angle between velocity vector and */
				/* position vector			     */

long	zdot,		/* Direction of the pass			     */
	overhalf=FALSE,	/* Flag for over 1/2 orbit			     */
	eorbit,		/* Orbit number from equator			     */
	porbit;		/* Orbit number from perigee			     */

/*
  Calculate the time deltas for the beginning and end of the pass
*/
if (model->dtype == GAC)
    {
    model->scene_ctime = (model->nlines / 2.0) / GAC_PER_SEC;
    step_delta = (model->nlines / 4.0) / GAC_PER_SEC;
    model->nsamples = GAC_SAMPS;
    if (model->nlines > (102 * 60 * GAC_PER_SEC))
    	overhalf = TRUE;
    }
else
    {
    model->scene_ctime = (model->nlines / 2.0) / LAC_PER_SEC;
    step_delta = (model->nlines / 4.0) / LAC_PER_SEC;
    model->nsamples = LAC_SAMPS;
    if (model->nlines > (102 * 60 * LAC_PER_SEC))
    	overhalf = TRUE;
    }

/*
  Get satellite lat, long, and radius at the start of the pass
*/
t1 = model->sec_epoch + model->delta_time;
brouwer(&t1,model,&truea,&radii[0],&thetas,&anlng,&inclin,&model->spass,
	&model->sorbit,&porbit,&str_latc,&str_long);

/*
  Calculate the equitorial crossing
*/
model->eqcrs = anlng - (-.004166 * RAD_PER_DEG) * thetas /
		sqrt(AMU / (radii[0] * radii[0] * radii[0]));

if (model->eqcrs < -1.0 * PIE)
    model->eqcrs += PIE2;

if (model->eqcrs > PIE)
    model->eqcrs -= PIE2;

/*
  Get satellite lat, long, and radius at 1/4 of the pass
*/
t1 += step_delta;
brouwer(&t1,model,&truea,&radii[1],&thetas,&anlng,&inclin,&zdot,
	&eorbit,&porbit,&t2_latc,&t2_long);

/*
  Get satellite lat, long, and radius at the center of the pass
*/
t1 += step_delta;
brouwer(&t1,model,&truea,&radii[2],&thetas,&anlng,&inclin,&model->cpass,
	&eorbit,&porbit,&cen_latc,&cen_long);

/*
  Determine if the pass is a day or night pass
*/
c_daynite(&model->epoch_jday,&model->epoch_year,&t1,&cen_latc,&cen_long,
	  &model->sunzen,&model->day_night);

/*
  Get satellite lat, long, and radius at 3/4 of the pass
*/
t1 += step_delta;
brouwer(&t1,model,&truea,&radii[3],&thetas,&anlng,&inclin,&zdot,
	&eorbit,&porbit,&t4_latc,&t4_long);

/*
  Get satellite lat, long, and radius at the end of the pass
*/
t1 += step_delta;
brouwer(&t1,model,&truea,&radii[4],&thetas,&anlng,&inclin,&model->epass,
	&model->eorbit,&porbit,&end_latc,&end_long);

/*
  Calculate the satellite altitude coefficients
*/
calcrcoefs(radii,&step_delta,model->alt_coefs);

/*
  Add the altitude corrections to the altitude of the satellite
*/
model->alt_coefs[0] += model->delta_alt;

/*
  Back off earth's rotation from the satellite's position
*/
str_long += (WE * (-model->scene_ctime - model->delta_time));
cen_long += (WE * (-model->delta_time));
end_long += (WE * (model->scene_ctime - model->delta_time));

/*
  Calculate inertial earth center vectors referenced at the center of the pass
*/
sph2cart(&str_latc,&str_long,&radii[0],&strvec);
sph2cart(&cen_latc,&cen_long,&radii[2],&cenvec);
sph2cart(&end_latc,&end_long,&radii[4],&endvec);

/*
  Find the angle between the start, center and end vectors
*/
anglebetwn(&strvec,&cenvec,&angle1);
anglebetwn(&cenvec,&endvec,&angle2);
if (overhalf)
    {
    angle1 = PIE2 - angle1;
    angle2 = PIE2 - angle2;
    }

/*
  Calculate mean motions
*/
model->mean_motion1 = angle1 / model->scene_ctime;
model->mean_motion2 = angle2 / model->scene_ctime;

/*
  Store vector values for the satellite at the center of image
  as the reference position of the satellite
*/
model->satint.target.x = cenvec.x;
model->satint.target.y = cenvec.y;
model->satint.target.z = cenvec.z;
model->satint.mag = radii[2];

/*
  Calculate velocity vector of satellite at center of image
*/
model->satint.vmag = sqrt(AMU / radii[2]);
newvec(&cenvec,&endvec,overhalf,&vel_angle,&model->satint.vmag,
       &model->satint.vel);

return;
}
 
