/*******************************************************************************
NAME:		C_SUNPOS

PURPOSE:	Get the sun position (lat/long) as a function of time

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		REASON
-------		----	------		------
  2.0		02/91	D. Hollaren	Conversion to LAS5.0
  2.0	        12/93   D. Hollaren	Problems occurred when the total
					seconds from epoch of the current year
					goes over to the next year.  Need to
					subtract a years worth of days from
					the calculated julian date.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <math.h>
#include "worgen.h"
#include "avhrr.h"
#include "adapsutil.h"
#include "constants.h"

#define SUMSOL		172		/* Julian date of the summer solstice */
#define DAY_PER_YEAR	365.25
#define DAY_PER_LYEAR	366.0

void c_sunpos
(
    long *jdate,		/* Julian date				*/
    long *year,			/* Year					*/
    double *tsec,		/* Total seconds from midnight GMT	*/
    double *sunlat,		/* Sun latitude location		*/
    double *sunlng		/* Sun longitude location		*/
)
{
/*
  Days Per Month
*/
static long  dpm[12] = {31,28,31,30,31,30,31,31,30,31,30,31};

/*
  Days Per Month for Leap year
*/
static long  ldpm[12] = {31,29,31,30,31,30,31,31,30,31,30,31};

/*
  Greenwich hour angle at midnight GMT at start of each month
  The Greenwich hour angle at midnight GMT is the arc length in radians
  past the Greenwich meridian.
*/
static float gha[12] = {179.23,176.63,176.88,179.0,180.72,180.56,179.06,
			178.4,179.98,182.56,184.91,182.75};

long	month,		/* Month					*/
	ist,		/* Starting month				*/
	iend,		/* Ending month					*/
	i;		/* Looping variable				*/

double	we,		/* Earth rotation rate				*/
	fjd,		/* Fractional julian date at the specified time	*/
	djd,		/* Julian date					*/
	dmo,		/* Day of the month				*/
	sungha,		/* Greenwich hour angle at specified date	*/
	earc;		/* The arc length the earth as moved from	*/
			/* summer solstice (RADIANS)			*/

/*
  Calculate the earth rotation rate
*/
we = 0.0041666678 * RAD_PER_DEG;

/*
  Calculate fractional portion of the julian date
*/
fjd = *jdate + *tsec / SEC_PER_DAY;

/*
  Determine if time went over the next year and subtract a years worth
  of days if it did.
*/
if (fjd > DAY_PER_YEAR)
    {
    if (c_isleap(year) && (fjd > DAY_PER_LYEAR))
	fjd -= DAY_PER_LYEAR;
    else
	fjd -= DAY_PER_YEAR;
    }

/*
  Subtract one for indexing
*/
djd = fjd - 1.0;

/*
  Find the month and calculate the day of the month
*/
if (c_isleap(year))
    {
    for (i = 0; i < 12; i++)
    	{
    	djd -= ldpm[i];
    	if (djd < 0.0)
	    {
	    month = i + 1;
	    dmo = (ldpm[i] + djd) / ldpm[i];
	    break;
	    }
    	}
    }
else
    {
    for (i = 0; i < 12; i++)
    	{
    	djd -= dpm[i];
    	if (djd < 0.0)
	    {
	    month = i + 1;
	    dmo = (dpm[i] + djd) / dpm[i];
	    break;
	    }
    	}
    }

/*
  Save the index into the Greenwich hour angle array
*/
if (month == 12)
    {
    ist = 12;
    iend = 1;
    }
else
    {
    ist = month;
    iend = month + 1;
    }

/*
  Calculate the Greenwich hour angle at midnight GMT for the specific date.
*/
sungha = (((gha[iend-1] - gha[ist-1]) * dmo) + gha[ist-1]) * RAD_PER_DEG;

/*
  Calculate the sun longitude angle.  Use the Greenwich hour angle at
  midnight GMT as the origin and subtract the amount the earth has 
  rotated since midnight GMT.  This value is then subtraced from 2PI
  so the longitude result is referenced from Greenwich meridian.
*/
*sunlng = PIE2 - sungha - (we * (*tsec));
if (*sunlng < 0.0)
    *sunlng = PIE2 + *sunlng;

/*
  The maximum sun angle (23.5) occures at julian date 172 (Summer Solstice).
  Also the change in the sun angle from day to day models a cosine function.
  These two facts are used to scale the sun angles between 23.5 and -23.5
  degrees using the cosine of the distance the earth has progresed in it's
  orbit from the summer solstace.

  Calculate the distance the earth has moved and use that to determine the
  sun's latitude.
*/
earc = ((fjd - SUMSOL) / DAY_PER_YEAR) * PIE2;
*sunlat = (23.5 * cos(earc)) * RAD_PER_DEG;

return;
}
 
