/*******************************************************************************
FUNCTION:	C_WINDDRP

PURPOSE:   	Given a drop line structure for an entire image and a window
		within the image, return a drop line structure relative to the
		windowed portion of the image.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/97  T. Baltzer       Development

PROJECT:        ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"


int c_winddrp
(
    struct MODEL *model,	/* IN     - AVHRR model structure             */
    struct DROPPED *drop,	/* IN     - AVHRR dropped line structure      */
    struct DROPPED *window_drop,/* OUT    - windowed AVHRR dropped line struct*/
    long window[],		/* IN/OUT - window being applied to the model */
    long bound_flg		/* IN     - Bound flag:			      */
				/*	TRUE:  Allow the window to be bounded */
				/*	       by drop lines.		      */
				/*	FALSE: Don't allow the window to be   */
				/*	       bounded by drop lines.	      */
)
{

long i, j, k;		/* Loop control                                       */
long flip_flag;		/* Flag indicating whether flipped or not	      */
long within_a_gap;	/* Flag specifying if within a drop line gap	      */
long wlinenum;		/* Window line number				      */

char *drop_line;	/* Ptr to drop line array			      */


/* Initialize an array with one element for each line of the input image.  This
   array will be used to indicate if a line is dropped or bad.  A value of zero
   indicates a good line while a value of 1 indicates a dropped line.  Calloc()
   will set all elements to zero.
------------------------------------------------------------------------------*/
drop_line = (char *)calloc(model->nlines , sizeof(char));
if (drop_line == NULL)
    {
    c_lasmsg(FALSE,"c_winddrp-alloc","Error allocating memory");
    return(E_FAIL);
    }


/* Populate the drop line array using the drop line gaps in the original model.
   The image must be flipped if it is an ascending pass not in satellite view
   (if it is an ascending pass in satellite view the window has already been
   flipped).
------------------------------------------------------------------------------*/
if ((model->cpass == ASCENDING) && (!model->sat_view))
    flip_flag = TRUE;
else
    flip_flag = FALSE;

for (i = 0; i < drop->ngaps; i++)
    {
    j = drop->lmissed[i] + drop->nmissed[i] - 1;
    for (k = drop->lmissed[i] - 1; k < j; k++)
	{
	if (flip_flag)
	    drop_line[model->nlines - k - 1] = TRUE;
	else
	    drop_line[k] = TRUE;
	}
    }


/* If the boundary flag equals FALSE, do not allow the windowed image to be
   bounded at either end by drop lines.
------------------------------------------------------------------------------*/
if (bound_flg == FALSE)
    {
    if (drop_line[window[SL] - 1] == TRUE)
	{
	for (i = window[SL] - 1; drop_line[i] == TRUE; i++);
	c_lasmsg(FALSE,"c_winddrp-sl","Window SL changed from %d to %d",
		 window[SL],i + 1);
	c_lasmsg(FALSE,"c_winddrp-nl","Window NL changed from %d to %d",
		 window[NL],window[NL] - (i + 1 - window[SL]));
	window[NL] = window[NL] - (i + 1 - window[SL]);
	window[SL] = i + 1;
	}

    if (drop_line[window[SL] + window[NL] - 2] == TRUE)
	{
	for (i = window[SL] + window[NL] - 2; drop_line[i] == TRUE; i--);
	c_lasmsg(FALSE,"c_winddrp-nl","Window NL changed from %d to %d",
		 window[NL],i - window[SL] + 2);
	window[NL] = i - window[SL] + 2;
	}
    }


/* Define the window drop line structure, taking into account the specified
   window.
------------------------------------------------------------------------------*/
window_drop->ngaps = 0;
within_a_gap = FALSE;
for (i = window[SL] - 1, wlinenum = 1; wlinenum <= window[NL]; i++, wlinenum++)
    {
    if (drop_line[i] == TRUE)
	{
	if (within_a_gap)
	    {
	    (window_drop->nmissed[window_drop->ngaps - 1])++;
	    }
	else
	    {
	    (window_drop->ngaps)++;
	    window_drop->lmissed[window_drop->ngaps - 1] = wlinenum;
	    window_drop->nmissed[window_drop->ngaps - 1] = 1;
	    within_a_gap = TRUE;
	    }
	}
    else
	within_a_gap = FALSE;
    }


/* Free allocated memory
------------------------------------------------------------------------------*/
free(drop_line);


return(E_SUCC);
}
