/*******************************************************************************
NAME:		C_WINDMOD	

PURPOSE:	Given an AVHRR model and drop line structure for an entire
		image, generate a model and drop line structure which reflects
		a specified window within the image.

		NOTE: Technically speaking, windowing an image in the "samples"
		      direction will alter the start/end time of the image
		      (167 milliseconds across a HRPT scan line).  However,
		      for the purposes of c_windmod(), start/end times are only
		      affected by windowing in the "lines" direction.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/97  T. Baltzer       Development
  1.1    01/98  T. Baltzer       Fix so that the raw satellite time gets passed
				 to c_acqtmcor().

				
PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h> 
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "acqio.h"
#include "adapsutil.h"
#include "local_protos.h"



int c_windmod
(
    struct MODEL *model,	/* IN     - AVHRR model information           */
    struct DROPPED *drop,	/* IN     - AVHRR dropped line structure      */
    struct MODEL *window_model,	/* OUT    - AVHRR model of windowed info      */
    struct DROPPED *window_drop,/* OUT    - AVHRR windowed drop line structure*/
    long window[],		/* IN/OUT - Window for the AVHRR archive image*/
    long bands[],		/* IN     - Band spec for the output image    */
    long bound_flg              /* IN - Bound flag:			      */
                                /*      TRUE:  Allow the window to be bounded */
                                /*             by drop lines.                 */
                                /*      FALSE: Don't allow the window to be   */
                                /*             bounded by drop lines.         */
)
{


long i;				/* Loop counter                               */
long msecoff;			/* Time offset in milliseconds		      */
long status;			/* Return status variable                     */


/* Window the drop line structure in the new model as specified by the user.
   Do this first because it could possibly change the window.
-----------------------------------------------------------------------------*/
status = c_winddrp(model, drop, window_drop, window, bound_flg);
if (status != E_SUCC)
    return(E_FAIL);


/* Determine the starting time of the new model based on the starting time of
   the entire image and the number of lines elapsed into the entire image prior
   to the start of the specified window.  The window must be flipped if it is
   an ascending pass not in satellite view (if it is an ascending pass in
   satellite view the window has already been flipped).
-----------------------------------------------------------------------------*/
if ((model->cpass == ASCENDING) && (!model->sat_view))
    {
    if (model->dtype == GAC)
	msecoff = (long)((model->nlines - (window[SL] + window[NL] - 1))
		  / GAC_PER_SEC * MIL_PER_SEC + 0.5);
    else
	msecoff = (long)((model->nlines - (window[SL] + window[NL] - 1))
		  / LAC_PER_SEC * MIL_PER_SEC + 0.5);
    }
else
    {
    if (model->dtype == GAC)
	msecoff = (long)((window[SL] - 1) / GAC_PER_SEC * MIL_PER_SEC + 0.5);
    else
	msecoff = (long)((window[SL] - 1) / LAC_PER_SEC * MIL_PER_SEC + 0.5);
    }


/* Adjust the offset to take into account the NBS time correction for the model
   of the entire image.  (We want to start the window model with no NBS offset
   applied yet, just the raw satellite time.  We will do it this way because
   the NBS correction for the model of the entire image may be different than
   the NBS correction for the window model). 
-----------------------------------------------------------------------------*/
msecoff += model->clock_err;


/* Using this calculated offset, adjust the starting time/date for the window.
-----------------------------------------------------------------------------*/
c_cvtoffsete(model->year,model->month,model->day,model->shour,
	     model->smin,model->ssec,model->sfrac,msecoff,
	     &window_model->year,&window_model->month,&window_model->day,
	     &window_model->shour,&window_model->smin,&window_model->ssec,
	     &window_model->sfrac);


/* Get the NBS offset from the table for the windowed portion of the pass.
-----------------------------------------------------------------------------*/
status = c_acqtmcor(&model->satnum,&window_model->month,&window_model->day,
	   &window_model->year,&window_model->shour,&window_model->smin,
	   &window_model->ssec,&window_model->sfrac,&window_model->clock_err);
if (status != E_SUCC)
    return(E_FAIL);


/* If the NBS offset in the table is zero, (1) it couldn't find a value in the
   table (use the value found in the original header), or (2) it found a value
   of zero in the table (the value found in the original header is probably
   close to zero then too, so use it).
-----------------------------------------------------------------------------*/
if (window_model->clock_err == 0)
    {
    window_model->clock_err = model->clock_err;
    c_lasmsg(FALSE,"windmod-info","NBS offset for window set to %d",
	     model->clock_err);
    }


/* Apply the NBS time correction to the window model.  Now that we have gotten
   the appropriate NBS offset, adjust the starting time and date accordingly.
-----------------------------------------------------------------------------*/
c_cvtoffsete(window_model->year,window_model->month,window_model->day,
	     window_model->shour,window_model->smin,window_model->ssec,
	     window_model->sfrac,-(window_model->clock_err),
	     &window_model->year,&window_model->month,&window_model->day,
	     &window_model->shour,&window_model->smin,&window_model->ssec,
	     &window_model->sfrac);


/* Initialize the model using this new start time and the number of lines in
   the window.  Make sure the end time is not set so that it will be computed.
-----------------------------------------------------------------------------*/
window_model->ehour = 0;
window_model->emin = 0;
window_model->esec = 0;
window_model->efrac = 0;
status = c_initmod(&model->satnum,&window_model->year,&window_model->month,
		   &window_model->day,&window_model->shour,
		   &window_model->smin,&window_model->ssec,
		   &window_model->sfrac,&model->dtype,&window[NL],
		   window_model);
if (status != E_SUCC)
    return(E_FAIL);


/* The call to c_initmod() will have filled in the nsamples field of the
   model with the default for the particular data type.  This may not be
   the actual number of requested samples in the output image; reset the
   field with window[NS].
------------------------------------------------------------------------*/
window_model->nsamples = window[NS];


/* Window the bands in the new model as specified by the user.
-----------------------------------------------------------------------------*/
status = wind_bands(model,window_model,bands);
if (status != E_SUCC)
    return(E_FAIL);


/* Compute the total number of missed lines in the window model
------------------------------------------------------------------------------*/
window_model->totmiss = 0;
for (i = 0; i < window_drop->ngaps; i++)
    {
    window_model->totmiss += window_drop->nmissed[i];
    }


/* Preserve misc. fields in the window model.
-----------------------------------------------------------------------------*/
strcpy(window_model->station,model->station);
window_model->sat_view = model->sat_view;


return(E_SUCC);
}
