/****************************************************************************
NAME:		CENTH2DET

PURPOSE:	Calculate the geodetic latitude and height of a point above
		the ellipse, given the geocentric latitude and the radius  

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:
	ESCOBAL, METHODS OF ORBIT DETERMINATION (page 398)

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "ellipsoid.h"

void centh2det
(
    double *latc,	/* Geocentric latitude			*/
    double *radius,	/* Radius of the point			*/
    double *latd,	/* Geodetic latitude			*/
    double *height	/* Height of point			*/
)
{
double	rsq,		/* Radius of the point squared			     */
	radius2,	/* Calculated radius of the point		     */
	r2sq,		/* Calculated radius of the point squared	     */
	height2,	/* Height of the satellite			     */
	phis,		/* Latitude of the satellite			     */
	phip,		/* Latitude of the point			     */
	dphip,		/* Delta latitude of the point			     */
	dphis;		/* Delta latitude of the satellite		     */

double	last_height = 0.0; /* Previous height of the satellite		     */

long	i;		/* Looping variable				     */

phip = *latc;
rsq  = *radius * *radius;

for (i = 0; i < 20; i++)
    {
    /*
      Calculate the radius at the target
    */
    radius2 = REQUA * sqrt(Q / (1.0 - EE * cos(phip) * cos(phip)));
    r2sq = radius2 * radius2;

    /*
      Calculate the latitude of the satellite
    */
    phis = atan(1.0 / Q * tan(phip));
    dphis = phis - phip;

    /*
      Calculate the height of the satellite
    */
    height2 = sqrt(rsq - r2sq * sin(dphis) * sin(dphis)) -
	      radius2 * cos(dphis);

    /*
      Check the tolerance
    */
    if (fabs(height2 - last_height) <= 0.0001)
	break;

    /*
      Save the current height
    */
    last_height = height2;

    /*
      Calculate the delta latitude correction for the new height
    */
    dphip = asin(height2 / *radius * sin(dphis));
    phip = *latc - dphip;
    }

/*
  Return the new latitude and height
*/
*latd = phis;
*height = height2;

return;
}
 
