/******************************************************************************
NAME:		ELRATES

PURPOSE:	Change original ephemeris data into the units that the model 
		needs.  Also calculate the rates of change for the elements.

PROGRAM HISTORY:
VERSION         PROGRAMMER           DATE       COMMENTS
-------         ----------           ----       --------
 2.0            C.  WIVELL           2/91	Original development

COMPUTER SOFTWARE OR HARDWARE LIMITATIONS:

PROJECT:             ADAPS

ALGORITHM DESCRIPTION:
Convert elements and calulate rates

ALGORITHM REFERENCES:
	"FUNDAMENTALS OF ASTRODYNAMICS"  Bates,Mueller,White
             
*******************************************************************************/
#include <math.h>
#include "worgen.h"
#include "model.h"
#include "avhrr.h"
#include "local_protos.h"

#define	G2 	541.3172e-6	/* Gravitational constant number 2	*/
#define	G4	0.624e-6	/* Gravitational constant number 4	*/

void elrates
(
    struct MODEL *model	  /* Model and orbital information	*/
)

{
double	ex,		  /* Eccentricity squared		*/
	cosi,		  /* Cosine of inclination		*/
	cosisq,		  /* Cosine of inclination squared	*/
	cosiqd,		  /* Cosine of inclination quad		*/
	p,		  /* Semi-latus rectum			*/
	ppa,		  /* p per a				*/
	pp,		  /* p squared (p * p)			*/
	pppp,		  /* p quad (p * p * p * p)		*/
	q;		  /* Sqrt(ppa)				*/

/*
  Calculate the semi-major axis
*/
model->smajor_axis = pow(model->mean_motion,-0.66666667);

/*
  Find the total seconds since epoch (model->sec_epoch)
*/
secsincepoch(model);

/*
  Related constants
*/
ex = model->ecc * model->ecc;
cosi = cos(model->inclin);
cosisq = cosi * cosi;
cosiqd = cosisq * cosisq;

p = model->smajor_axis * (1.0 - ex);
ppa = p / model->smajor_axis;
pp = p * p;
pppp = pp * pp;
q = sqrt(1.0- ex);

/*
  Find a0 (semi-major axis)
*/
model->smajor_axis *= ( pow((1.0 + 1.5 * G2 * q / pp * (3.0 * cosisq - 1.0)
		    + 0.09375 * G2 * G2 * q / pppp * (16.0 * q + 25.0 * ppa
		    - 15.0 + (30.0 - 96.0 * q - 90.0 * ppa) * cosisq
		    + (105.0 + 144.0 * q + 25.0 * ppa) * cosiqd)
		    + 0.9375 * G4 * q / pppp * ex * (3.0 - 30.0 * cosisq
		    + 35.0 * cosiqd)),( 2.0 / 3.0)) );

p = model->smajor_axis * ppa;
pp = p * p;
pppp = pp * pp;

/*
  Calculate the derivitive of the argument of perigee w/r mean motion
  d(arg. of perigee)/dm
*/
model->der_argper = -1.5 * G2 / pp * (1.0 - 5.0 * cosi * cosi)
		  + .09375 * G2 * G2 / pppp * (-35.0 + 24.0 * q + 25.0 * ppa
		  + (90.0 - 192.0 * q - 126.0 * ppa) * cosisq
		  + (385.0 + 360.0 * q + 45.0 * ppa) * cosiqd)
		  + 0.3125 * G4 / pppp * (21.0 - 9.0 * ppa
		  + (-270.0 + 126.0 * ppa) * cosisq
		  + (385.0 - 189.0 * ppa) * cosiqd);

/*
  Calculate the derivitive of longitude of ascending node w/r mean motion
  inertial space.
  d(long.of asc. node)/dm
*/
model->der_anode = -3.0 * G2 / pp * cosi
		 + 0.375 * G2 * G2 / pppp * cosi * (-5.0 + 12.0 * q + 9.0 * ppa
		 - (35.0 + 36.0 * q + 5.0 * ppa) * cosisq)
		 + 1.25 * G4 / pppp * cosi * (5.0 - 3.0 * ppa)
		 * (3.0 - 7.0 * cosisq);

/*
  adot(da/dt)
*/
model->adot = -4.0 / 3.0 * model->smajor_axis / model->mean_anomaly 
	    * model->decay;

return;
}
 
