/*******************************************************************************
FUNCTION:	FINDTARVEC

PURPOSE:	Find the target vector from the satellite position and the line
		of sight vector.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  3.0    11/92  C.Wivell         Development
  3.1    10/93  J.Fenno          Added code to ensure values passed to asin()
				 are between -1.0 and 1.0 inclusive.

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "vector.h"
#include "model.h"
#include "local_protos.h"


void findtarvec
(
    struct MVECTOR *satvec,	/* Satellite location vector		     */
    struct MVECTOR *losv,	/* Line of sight to target vector	     */
    struct MVECTOR *tarvec,	/* Target vector			     */
    double *tarlatc,	/* Target latitude				     */
    double *tarlong,	/* Target longitude				     */
    double *tarrad	/* Radius of the target				     */
)
{
double	angle,		/* Angle between satellite and LOS vectors	     */
	sat_latc,	/* Latitude of the satellite			     */
	sat_long,	/* Longitude of the satellite			     */
	earth_radius,	/* Radius of the earth				     */
	sat_radius,	/* Radius of the satellite			     */
	max_tan,	/* Maximum tangent angle from satellite to earth     */
	maglos,		/* Magnitude of the line of sight vector	     */
	psi,		/* Earth centeral angle				     */
	latc;		/* Temporary variable to hold a latitude	     */

double	t1,t2;		/* Temporary double variables                        */

long	i;		/* Looping and index variables			     */

/*
  Find the angle between the satellite location and the line of sight
*/
anglebetwn(satvec,losv,&angle);
angle = PIE - angle;

/*
  Convert the satellite cartesian location to spherical coordinates
*/
cart2sph(satvec,&sat_latc,&sat_long,&sat_radius);
latc = sat_latc;

for (i = 0; i < 3; i++)
    {
    /*
      Find the radius of the earth for the specified latitude
    */
    earthradius(latc,&earth_radius);

    /*
      Calculate the maximum angle to the tangent to the earth
    */
    t1 = earth_radius / sat_radius;
    if (t1 < -1.0)
	t1 = -1.0;
    else if (t1 > 1.0)
	t1 = 1.0;
    max_tan = asin(t1);

    /*
      Check to determine if the angle is greater than the max tan angle
    */
    if (angle > max_tan)
	{
	/*
	  Fudge to appoximate earth centeral angle
	*/
	t1 = sin(max_tan) * sat_radius / earth_radius;
	if (t1 < -1.0)
	    t1 = -1.0;
	else if (t1 > 1.0)
	    t1 = 1.0;
	t2 = sin(angle - max_tan) * sat_radius / earth_radius;
	if (t2 < -1.0)
	    t2 = -1.0;
	else if (t2 > 1.0)
	    t2 = 1.0;
	psi = asin(t1) + asin(t2) - angle;
	}
    else
	{
	t1 = sin(angle) * sat_radius / earth_radius;
	if (t1 < -1.0)
	    t1 = -1.0;
	else if (t1 > 1.0)
	    t1 = 1.0;
	psi = asin(t1) - angle;
	}

    /*
      Calculate the magnitude of the line of sight vector
    */
    maglos = sqrt(sat_radius * sat_radius + earth_radius * earth_radius - 
		  2.0 * sat_radius * earth_radius * cos(psi));

    /*
      Set the target location
    */
    tarvec->x = satvec->x + maglos * losv->x;
    tarvec->y = satvec->y + maglos * losv->y;
    tarvec->z = satvec->z + maglos * losv->z;

    /*
      Convert the target cartesian location to spherical coordinates
    */
    cart2sph(tarvec,tarlatc,tarlong,tarrad);
    latc = *tarlatc;
    }

return;
}
 
