/*******************************************************************************
FUNCTION:	RELAZIMUTH

PURPOSE:	Calculate the relative azimuth angle

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  3.0    11/92  C.Wivell         Development
  3.1    11/93  D.Hollaren/      Modified so that the relative azimuth angle is
                J.Fenno          returned as a scaled value (0-360).  This
                                 will be used for illumination corrections.
  3.2    03/94  J.Fenno          Ensure that arguments to acos() are in the
				 range -1 to 1.

ALGORITHM REFERENCES:
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "model.h"
#include "local_protos.h"

void relazimuth
(
    double *tarlatd,	/* Target latitude				     */
    double *tarlong,	/* Target longitude				     */
    double *satlat,	/* Satellite latitude				     */
    double *satlong,	/* Satellite longitude				     */
    double *sunlat,	/* Sun latitude					     */
    double *sunlong,	/* Sun longitude				     */
    double *relaz	/* Relative azimuth value			     */
)
{
double	gcir,		/* Great circle from target			     */
	tarlatc,	/* Geocentric latitude				     */
	sat_az,		/* Satellite azimuth				     */
	sun_az,		/* Solar azimuth				     */
	elev = 0.0,	/* Elevation					     */
	rc;		/* Radius of satellite				     */

/*
  Convert the geodetic latitude to geocentric
*/
det2centh(tarlatd,&elev,&tarlatc,&rc);

/*
  Calculate the great circle from the target to the satellite
*/
greatcircle(&tarlatc,tarlong,satlat,satlong,&gcir);

/*
  Calculate the satellite azimuth.  Ensure the value supplied to acos is valid.
*/
sat_az = (sin(*satlat) - sin(tarlatc) * cos(gcir)) / (cos(tarlatc) * sin(gcir));
if (sat_az < -1.0) sat_az = -1.0;
if (sat_az >  1.0) sat_az =  1.0;
sat_az = acos(sat_az);

/*
  Check the sign of the satellite azimuth.  This must be clockwise from North.
*/
if (sin(*satlong - *tarlong) < 0.0)
    sat_az  = PIE2 - sat_az;

/*
  Calculate the great circle from the target to the sun
*/
greatcircle(&tarlatc,tarlong,sunlat,sunlong,&gcir);

/*
  Calculate the solar azimuth.  Ensure the value supplied to acos is valid.
*/
sun_az = (sin(*sunlat) - sin(tarlatc) * cos(gcir)) / (cos(tarlatc) * sin(gcir));
if (sun_az < -1.0) sun_az = -1.0;
if (sun_az >  1.0) sun_az =  1.0;
sun_az = acos(sun_az);

/*
  Check the sign of the solar azimuth.  This must be clockwise from North.
*/
if (sin(*sunlong - *tarlong) < 0.0)
    sun_az  = PIE2 - sun_az;

/*
  Calculate the relative sun/satellite azimuth
*/
*relaz = sun_az - sat_az;


/*
  Ensure the magnitude value of the relative azimuth is 0-180
*/
if (*relaz < -PIE)
    *relaz += PIE2;
else if (*relaz > PIE)
    *relaz -= PIE2;

/*
  Now scale the relative azimuth so it is 0-360
*/
*relaz += PIE;

return;
}
 
