/*******************************************************************************
FUNCTION: c_pg2struct

PURPOSE:  Given a string read from the standing order table scan the individual
          fields into a standing order structure.  Return a filled standing
	  order structure and E_SUCC if successful, E_FAIL otherwise.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development
  1.1    J.Fenno          10/92  Separated prodgen and prodedit into separate
				 functions and created a product generation
				 object library.
  1.2    J.Fenno          02/93  Added age field in prodgen table
  3.4    A. Vander Vorste 03/96  Added output directory field to prodgen.table 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Run under TAE

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "avhrr.h"
#include "products.h"

int c_pg2struct
(
    char *str,             /* IN  - String read from the prodgen table        */
    struct ORDER *order   /* OUT - Ptr to a standing order structure          */
)
{
char tmpstr[CMLEN];       /* Temporary string for converting DMS to degrees   */

double tmpdbl;            /* Temporary double variable                        */

long i;                   /* Loop control                                     */
long nonfatal = 1;        /* Nonfatal error flag                              */
long tmplng;              /* Temporary long variable                          */


/* If the string isn't long enough error out
--------------------------------------------*/
if (strlen(str) < 100)
   {
   c_errmsg("String must be at least 100 characters long",
      "pg2struct-len", &nonfatal);
   return(E_FAIL);
   }


/* Retrieve the account number
------------------------------*/
sscanf(str, "%d", &(order->account));


/* Retrieve the order number
----------------------------*/
sscanf(&str[10], "%d", &(order->ordernum));


/* Retrieve the starting and ending dates
-----------------------------------------*/
sscanf(&str[14], "%d", &(order->start_date));
sscanf(&str[23], "%d", &(order->end_date));


/* Retrieve the maximum age of the requested data
-------------------------------------------------*/
sscanf(&str[32], "%d", &(order->age));

/* Retrieve the Station ID
__________________________*/
strncpy(order->station, &str[37], 3);
order->description[3] = '\0';


/* Retrieve the Satellite number
--------------------------------*/
sscanf(&str[41], "%hd", &(order->satnum));


/* Retrieve the Day/Night flag
------------------------------*/
sscanf(&str[44], "%s", tmpstr);
if (strcmp(tmpstr, "day") == 0)
   order->day_night = DAY;
else if (strcmp(tmpstr, "night") == 0)
   order->day_night = NIGHT;
else
   order->day_night = DAY_NIGHT;


/* Retrieve the type of data requested
--------------------------------------*/
sscanf(&str[50], "%s", tmpstr);
if (strcmp(tmpstr, "GAC") == 0)
   order->dtype = GAC;
else
   order->dtype = HRPT;


/* Retrieve the Northern Latitude.  Convert from DMS to degrees
---------------------------------------------------------------*/
sscanf(&str[55], "%s", tmpstr);     /* seconds */
sscanf(&tmpstr[5], "%d", &tmplng);
tmpdbl = (double)tmplng / 3600.0;
order->northlat = tmpdbl;
tmpstr[5] = '\0';

sscanf(&tmpstr[3], "%d", &tmplng);  /* minutes */
tmpdbl = (double)tmplng / 60.0;
order->northlat += tmpdbl;
tmpstr[3] = '\0';

sscanf(&tmpstr[1], "%d", &tmplng);  /* degrees */
order->northlat += (double)tmplng;
if (tmpstr[0] == '-')
   order->northlat *= -1;


/* Retrieve the Southern Latitude.  Convert from DMS to degrees
---------------------------------------------------------------*/
sscanf(&str[63], "%s", tmpstr);     /* seconds */
sscanf(&tmpstr[5], "%d", &tmplng);
tmpdbl = (double)tmplng / 3600.0;
order->southlat = tmpdbl;
tmpstr[5] = '\0';

sscanf(&tmpstr[3], "%d", &tmplng);  /* minutes */
tmpdbl = (double)tmplng / 60.0;
order->southlat += tmpdbl;
tmpstr[3] = '\0';

sscanf(&tmpstr[1], "%d", &tmplng);  /* degrees */
order->southlat += (double)tmplng;
if (tmpstr[0] == '-')
   order->southlat *= -1;


/* Retrieve the Westernmost Longitude.  Convert from DMS to degrees
-------------------------------------------------------------------*/
sscanf(&str[71], "%s", tmpstr);     /* seconds */
sscanf(&tmpstr[6], "%d", &tmplng);
tmpdbl = (double)tmplng / 3600.0;
order->westlong = tmpdbl;
tmpstr[6] = '\0';

sscanf(&tmpstr[4], "%d", &tmplng);  /* minutes */
tmpdbl = (double)tmplng / 60.0;
order->westlong += tmpdbl;
tmpstr[4] = '\0';

sscanf(&tmpstr[1], "%d", &tmplng);  /* degrees */
order->westlong += (double)tmplng;
if (tmpstr[0] == '-')
   order->westlong *= -1;


/* Retrieve the Easternmost Longitude.  Convert from DMS to degrees
-------------------------------------------------------------------*/
sscanf(&str[80], "%s", tmpstr);     /* seconds */
sscanf(&tmpstr[6], "%d", &tmplng);
tmpdbl = (double)tmplng / 3600.0;
order->eastlong = tmpdbl;
tmpstr[6] = '\0';

sscanf(&tmpstr[4], "%d", &tmplng);  /* minutes */
tmpdbl = (double)tmplng / 60.0;
order->eastlong += tmpdbl;
tmpstr[4] = '\0';

sscanf(&tmpstr[1], "%d", &tmplng);  /* degrees */
order->eastlong += (double)tmplng;
if (tmpstr[0] == '-')
   order->eastlong *= -1;


/* Determine the pole crossing flag 
-----------------------------------*/
if ((order->eastlong == 0.0) && (order->westlong == 0.0))
   {
   if (order->northlat == 0.0)
      order->polecross = SOUTHPOLE;
   else
      order->polecross = NORTHPOLE;
   }
else
   order->polecross = NOPOLE;


/* Retrieve the minimum number of lines to be processed
-------------------------------------------------------*/
sscanf(&str[89], "%d", &(order->minlines));

/* Retrieve the windowing flag
______________________________*/
sscanf(&str[95], "%c", &(order->window));

/* Retrieve the product code
----------------------------*/
sscanf(&str[97], "%d", &(order->product_code));


/* Retrieve the band specifications
-----------------------------------*/
sscanf(&str[101], "%s", tmpstr);
for (i = 0; i < 5; i++)
   order->bands[i] = (tmpstr[i] != '0');


/* Retrieve the description 
---------------------------*/
strncpy(order->description, &str[107], 17);
order->description[17] = '\0';
i = (long)strlen(order->description);

/* Retrieve the output directory (watch for \n at end of string)
----------------------------------------------------------------*/
strncpy(order->output, &str[125], CMLEN);
order->output[CMLEN-1] = '\0';
i = (long)strlen(order->output);
if (order->output[--i] == '\n')
   order->output[i] = '\0';

return(E_SUCC);
}
