/*******************************************************************************
FUNCTION: c_pgpoints

PURPOSE:  Given the model structure for an archive image and the number of
	  pixels to skip between points, create a list of points that lie
	  around the edge of the image separated by the specified number
	  of pixels.  Each point will contain the line and sample and the
	  latitude and longitude of a pixel.  The points will be stored in
	  a FIFO queue.

	  Return E_SUCC if successful, E_FAIL if not.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development
  1.1    J.Fenno          10/92  Separated prodgen and prodedit into separate
				 functions and created a product generation
				 object library.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Run under TAE

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "model.h"
#include "constants.h"
#include "products.h"
#include "queue.h"

/* Prototype declaration
-------------------------*/
static short create_pt
(
    long q,                /* IN - Queue of points                            */
    long line,             /* IN - Line number of the point to be added       */
    long samp,             /* IN - Sample number of the point to be added     */
    struct MODEL *model    /* IN - Ptr to the model data structure            */
);



int c_pgpoints
(
    struct MODEL *model,  /* IN  - Ptr to the model and orbital info structure*/
    long npixels,	  /* IN  - Number of pixels to skip between points    */
    long *q               /* OUT - Ptr to the queue that is created           */
)
{
long line;              /* Loop control                                       */
long samp;              /* Loop control                                       */
long nonfatal =  1;     /* Nonfatal error flag                                */


/* Create a new queue
---------------------*/
*q = c_qcreate("fifo", "dups");
if (*q == -1)
   {
   c_errmsg("Error creating queue", "pgpoints-qcreate", &nonfatal);
   return(E_FAIL);
   }


/* Add the points along the top and bottom of the image to the queue
--------------------------------------------------------------------*/
line = 1;
while (line <= model->nlines)
   {
   for (samp = 1; samp <= model->nsamples; samp += npixels)
      {
      /* Create a new point structure, calculate the latitude and
	 longitude for this point, and add it to the queue of points
      --------------------------------------------------------------*/
      if (create_pt(*q, line, samp, model))
	 {
	 c_qdestroy(*q);
	 return(E_FAIL);
	 }


      /* Reset samp so it always equals the last sample on the final iteration
      ------------------------------------------------------------------------*/
      if (((samp + npixels) > model->nsamples) && (samp < model->nsamples))
	 samp = model->nsamples - npixels;
      }

   line += model->nlines - 1;
   }


/* Add the points along the edges of the image to the queue
-----------------------------------------------------------*/
for (line = npixels + 1; line < model->nlines; line += npixels)
   {
   /* Create a new point structure, calculate the latitude and
      longitude for this point, and add it to the queue of points
   --------------------------------------------------------------*/
   samp = 1;
   if (create_pt(*q, line, samp, model))
      {
      c_qdestroy(*q);
      return(E_FAIL);
      }


   /* Create a new point structure, calculate the latitude and
      longitude for this point, and add it to the queue of points
   --------------------------------------------------------------*/
   samp = model->nsamples;
   if (create_pt(*q, line, samp, model))
      {
      c_qdestroy(*q);
      return(E_FAIL);
      }
   }
return(E_SUCC);
}


/*==============================================================================
Given:  The number of a queue, the line and sample for a point in an archive
        image, and a ptr to the model structure.
Task:   Calculate the latitude and longitude for the given line/sample and add
	this point to a queue of points.
Return: 0 if successful, -1 if not.
==============================================================================*/
static short create_pt
(
    long q,                /* IN - Queue of points                            */
    long line,             /* IN - Line number of the point to be added       */
    long samp,             /* IN - Sample number of the point to be added     */
    struct MODEL *model    /* IN - Ptr to the model data structure            */
)
{
float fline;              /* Float value of the current line                 */
float fsamp;              /* Float value of the current sample               */

double elev = 0.0;	  /* Elevation					     */

long  nonfatal = 1;       /* Nonfatal error flag                             */

struct POINT *ptr;        /* Ptr to a point structure                        */


/* Create a new pixel structure
-------------------------------*/
ptr = (struct POINT *)malloc(sizeof(struct POINT));
if (ptr == NULL)
   {
   c_errmsg("Error allocating memory", "pgpoints-alloc", &nonfatal);
   return(E_FAIL);
   }


/* Store the line/sample values in the structure
------------------------------------------------*/
ptr->line   = line;
ptr->sample = samp;


/* Calculate the latitude and longitude for this
   point and convert from radians to degrees
------------------------------------------------*/
fline = (float)line;
fsamp = (float)samp;
c_ls2ll(&fline, &fsamp, &elev, model, &(ptr->lat), &(ptr->lon), NULL);
ptr->lat *= DEG_PER_RAD;
ptr->lon *= DEG_PER_RAD;


/* Insert this point in the queue.  Since this is a FIFO queue
   and the compare function is not used, just pass NULL to c_qinsert.
--------------------------------------------------------------------*/
if (c_qinsert(q, (void *)ptr, NULL))
   {
   c_errmsg("Error inserting into Q", "pgpoints-qinsert", &nonfatal);
   return(E_FAIL);
   }
return(E_SUCC);
}
