/*******************************************************************************
FUNCTION: c_pgread

PURPOSE:  Read the standing order table from ADAPSTABLES.  It is assumed that
	  the table has been locked before this function is called to prevent
	  another function from accessing or updating the table at the same
	  time.  Return E_SUCC if successful, E_FAIL otherwise.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development
  1.1    J.Fenno          10/92  Separated prodgen and prodedit into separate
				 functions and created a product generation
				 object library.
  1.2    J.Fenno          02/93  Added age field in prodgen table
  3.3    A. Vander Vorste 03/96  Added output directory field to prodgen.table 


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Run under TAE

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adapsutil.h"
#include "products.h"
#include "queue.h"


int c_pgread
(
    struct DESCRIPT *descript, /* OUT - Ptr to struct containing description  */
    long *orders_q             /* OUT - Q holding all standing order strings  */
)
{
char *buffer_ptr;         /* Ptr into a buffer of characters                  */
char filename[CMLEN];     /* Full path specification of TABLE                 */
char *ptr;                /* Ptr a character string                           */
char string[CMLEN];       /* Temporary working string                         */
char string2[2*CMLEN];    /* Temporary working string                         */

long i;                   /* Loop control                                     */
long nonfatal = 1;        /* Nonfatal error flag                              */
long norders;             /* Number of standing orders in the table           */
long unlock = UNLOCK;     /* Flag for c_lockfil to unlock a file              */

FILE *table;              /* Ptr to the table file                            */


/* Get the file name for TABLE
------------------------------*/
ptr = (char *)getenv("ADAPSTABLES");
if (ptr == NULL)
   {
   c_errmsg("Error retrieving ADAPSTABLES", "pgread-env", &nonfatal);
   return(E_FAIL);
   }
strcpy(filename, ptr);
#ifdef unix
strcat(filename, "/");
#endif
strcat(filename, TABLE);


/* Open the table for reading
-----------------------------*/
table = fopen(filename, "r");
if (table == NULL)
   {
   c_lockfil(filename, &unlock);
   sprintf(string, "Error opening %s", filename);
   c_errmsg(string, "pgread-open", &nonfatal);
   return(E_FAIL);
   }


/* Read the first line to determine the number of lines
   in the description.  Remove the '\n' from the string.
--------------------------------------------------------*/
if (fgets(string, CMLEN, table) == NULL)
   {
   fclose(table);
   c_lockfil(filename, &unlock);
   c_errmsg("Error reading table", "pgread-read", &nonfatal);
   return(E_FAIL);
   }
ptr = strchr(string, '\n');
if (ptr != NULL)
   *ptr = '\0';


/* Determine the number of lines in the description
   portion of the table (Must be >= 2)
---------------------------------------------------*/
if ((sscanf(string, "%d", &(descript->nlines)) != 1) || (descript->nlines < 2))
   {
   fclose(table);
   c_lockfil(filename, &unlock);
   c_errmsg("Error reading table", "pgread-read", &nonfatal);
   return(E_FAIL);
   }


/* Allocate enough space for the description
--------------------------------------------*/
descript->buffer = (char *)calloc(descript->nlines, CMLEN);
if (descript->buffer == NULL)
   {
   fclose(table);
   c_lockfil(filename, &unlock);
   c_errmsg("Error allocating memory", "pgread-alloc", &nonfatal);
   return(E_FAIL);
   }


/* Copy the first line into the buffer (this is the second line of the
   table - which contains the number of orders) and then read the 
   remaining lines of the description
------------------------------------------------------------------------*/
buffer_ptr = descript->buffer;
strcpy(buffer_ptr, string);
for (i = 1; i < descript->nlines; i++)
   {
   /* Read one line from the table
   -------------------------------*/
   if (fgets(string, CMLEN, table) == NULL)
      {
      fclose(table);
      c_lockfil(filename, &unlock);
      free(descript->buffer);
      c_errmsg("Number of records does not match line two of table",
	 "pgread-nrecs", &nonfatal);
      return(E_FAIL);
      }


   /* Remove the trailing '\n' character from the string
   -----------------------------------------------------*/
   ptr = strchr(string, '\n');
   if (ptr != NULL)
      *ptr = '\0';
   

   /* Store the string in the buffer
   ---------------------------------*/
   buffer_ptr += CMLEN;
   strcpy(buffer_ptr, string);
   }


/* Determine the number of standing orders in the table.
   This is the first number on the second line of the table.
------------------------------------------------------------*/
buffer_ptr = descript->buffer;
buffer_ptr += CMLEN;
if (sscanf(buffer_ptr, "%d", &norders) != 1)
   {
   fclose(table);
   c_lockfil(filename, &unlock);
   free(descript->buffer);
   c_errmsg("Error reading table", "pgread-read", &nonfatal);
   return(E_FAIL);
   }


/* Create a new Q for the standing order strings
------------------------------------------------*/
*orders_q = c_qcreate("ascending", "nodups");
if (*orders_q == -1)
   {
   fclose(table);
   c_lockfil(filename, &unlock);
   free(descript->buffer);
   c_errmsg("Error creating queue", "pgread-qcreate", &nonfatal);
   return(E_FAIL);
   }


/* Read all of the standing orders and place them in a queue
------------------------------------------------------------*/
for (i = 0; i < norders; i++)
   {
   /* Allocate enough space for one standing order string
   ------------------------------------------------------*/
   buffer_ptr = (char *)malloc(CMLEN*2);
   if (buffer_ptr == NULL)
      {
      fclose(table);
      c_lockfil(filename, &unlock);
      free(descript->buffer);
      c_qdestroy(*orders_q);
      c_errmsg("Error allocating memory", "pgread-alloc", &nonfatal);
      return(E_FAIL);
      }


   /* Read the next standing order from the table
   ----------------------------------------------*/
   if (fgets(string2, CMLEN, table) == NULL)
      {
      fclose(table);
      c_lockfil(filename, &unlock);
      free(descript->buffer);
      c_qdestroy(*orders_q);
      c_errmsg("Error reading table", "pgread-read", &nonfatal);
      return(E_FAIL);
      }

   
   /* Remove the trailing '\n' character
   -------------------------------------*/
   ptr = strchr(string2, '\n');
   if (ptr != NULL)
      *ptr = '\0';
   strcpy(buffer_ptr, string2);

   
   /* Insert the standing order string into the queue
   --------------------------------------------------*/
   if (c_qinsert(*orders_q, buffer_ptr, (long (*)())strcmp))
      {
      fclose(table);
      c_lockfil(filename, &unlock);
      free(descript->buffer);
      c_qdestroy(*orders_q);
      c_errmsg("Error inserting into Q", "pgread-qinsert", &nonfatal);
      return(E_FAIL);
      }
   }


/* Close the table
------------------*/
fclose(table);
return(E_SUCC);
}
