/*******************************************************************************
FUNCTION: c_pgtest

PURPOSE:  Given the model structure for an archive image, a structure
	  containing a standing order, and a queue of points surrounding
	  the edge of an image, determine if the archive image meets the
	  requirements for the standing order.  If the image should be
	  processed for the standing order, return the starting line,
	  number of lines, and a status of PROCESS.  If the image should
	  not be processed for the standing order, return zero for the
	  starting line, -1 for the number of lines, and a status of
	  NOPROCESS.  If an error occurs, return a status of E_FAIL.


Version  Author           Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development
  1.1    J.Fenno          10/92  Separated prodgen and prodedit into separate
				 functions and created a product generation
				 object library.
  1.2    J.Fenno          12/92  Added the age field to the prodgen table


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Run under TAE

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "products.h"
#include "queue.h"

static void get_dates
(
   long year,           /* IN  - Year of acquisition                          */
   long month,          /* IN  - Month of acquisition                         */
   long day,            /* IN  - Day of acquisition                           */
   long age,            /* IN  - # of days after acquisition                  */
   long *agedate,       /* OUT - YYYYMMDD form of "age" days after acquisition*/
   long *curdate        /* OUT - YYYYMMDD form of today's date                */
);


int c_pgtest
(
   struct MODEL  *model,  /* IN - Ptr to a archive model structure            */
   struct ORDER  *order,  /* IN - Ptr to a standing order structure           */
   long points_q,         /* IN - Queue of points along the edge of the image */
   long *start,           /* OUT - First line of the image in the product     */
   long *nlines           /* OUT - Number of lines in the product             */
)
{
double lat_rads;          /* Latitude of a point in radians                   */
double lon_rads;          /* Longitude of a point in radians                  */
double elev = 0.0;	  /* Elevation					      */

float  fline[4];          /* Floating pt line and sample of the 4 corner pts  */
float  fsamp[4];          /*    of the area of interest for the standing order*/

long   agedate;           /* Last day of customer's interest for the image    */
long   curdate;           /* Today's date                                     */
long   date;              /* Date the image was acquired                      */
long   i;                 /* Loop control                                     */
long   maxline;           /* Maximum line number that lies within the image   */
long   minline;           /* Minimum line number that lies within the image   */
long   nonfatal = 1;      /* Nonfatal error flag                              */

void   *ptr;              /* Ptr to a void object returned by queue functions */

struct POINT *pt;         /* Ptr to a point along the edge of the image       */
char   astation[4];	  /* Temporary station for comparisons	              */
char   kstation[4];	  /* Temporary station for comparisons		      */
char   os[4];
char   ms[4];


/* Initialize start, nlines, maxline, and minline
-------------------------------------------------*/
*start  = 0;
*nlines = -1;
maxline = 0;
minline = model->nlines + 1;


/* Convert the image's acquisition date date to YYYYMMDD
--------------------------------------------------------*/
if (model->year < 70)
   model->year += 2000;
else if (model->year < 100)
   model->year += 1900;
date = (model->year * 10000) + (model->month * 100) + model->day;


/* Return if the acquisition date isn't within the specified period
-------------------------------------------------------------------*/
if ((date < order->start_date) || (date > order->end_date))
   return(NOPROCESS);


/* Return if the current date is past the data age date.  This means the
   customer does not want the data if it was acquired too long ago.
------------------------------------------------------------------------*/
get_dates(model->year,model->month,model->day,order->age,&agedate,&curdate);
if (curdate > agedate)
   return(NOPROCESS);


/* Compare the satellite numbers
--------------------------------*/
if (model->satnum != order->satnum)
   return(NOPROCESS);


/* Compare the DAY/NIGHT indicators
-----------------------------------*/
if ((order->day_night != DAY_NIGHT) && (order->day_night != model->day_night))
   return(NOPROCESS);


/* Compare the types of data (GAC or HRPT).  Process
   LAC data if the order requires HRPT.
----------------------------------------------------*/
if (((order->dtype == HRPT) && (model->dtype == GAC)) ||
    ((order->dtype == GAC)  && (model->dtype != GAC)))
   return(NOPROCESS);

/* Compare the station id's.  Don't process data if
   the image's station id not = the order's station id
   OR if the order's station id = *** and the 
   image's station id is = KM* (KM1, KME, etc)
______________________________________________________*/
strcpy(astation,"***");
strcpy(kstation,"KM*");
strcpy(os,order->station);
strcpy(ms,model->station);
if (strcmp(ms,os) != 0)
    {
    /* if first 2 char's of kstation == ms */
    if (strncmp(kstation, ms, 2) == 0)
	return(NOPROCESS);
    /* if astation == os */
    if (strcmp(astation,os) != 0) 
	return(NOPROCESS); 

    }/* end-if (strcmp(ms,os) != 0) */


/* Get a pointer to the first point in the queue
------------------------------------------------*/
if (c_qfirst(points_q, &ptr))
   {
   c_errmsg("Error accessing queue", "pgtest-queue", &nonfatal);
   return(E_FAIL);
   }


/* Check all of the points in the queue
---------------------------------------*/
i = 0;
while (ptr != NULL)
   {
   /* Type cast the returned pointer into a POINT structure
   --------------------------------------------------------*/
   pt = (struct POINT *)ptr;

   /* Prepare yourself for a monster if condition.  The first line determines if
      the area of interest does not cross either pole.  The next four lines
      determine if the area of interest crosses the International Date Line and
      if the current point lies within it.  The last five lines determine if
      the area of interest does not cross the date line and if the current point
      lies within it.

                 Date Line                  Does not cross the Date Line
                     |               
             --------|--------                    -----------------
             |       |       |                    |               |
             |       |  x    |                    |               |
             |   x   |       |                    |     x         |
             |       |       |                    |               |
             |       |       |                    |               |
             --------|--------                    -----------------
           west      |     east                 west            east

      if ((x >= west) || (x <= east))      if ((x >= west) && (x <= east))
         The point lies within                The point lies within
   ---------------------------------------------------------------------------*/
   if ((order->polecross == NOPOLE) &&
       (((order->westlong > order->eastlong) &&
	 ((pt->lon >= order->westlong) || (pt->lon <= order->eastlong)) &&
	 (pt->lat  >= order->southlat) &&
	 (pt->lat  <= order->northlat)) ||
        ((order->westlong < order->eastlong) &&
	 (pt->lon >= order->westlong) &&
	 (pt->lon <= order->eastlong) &&
	 (pt->lat >= order->southlat) &&
	 (pt->lat <= order->northlat))))
      {
      i++;
      /* Check for a new minimum or maximum line
      ------------------------------------------*/
      if (pt->line > maxline)
	 maxline = pt->line;
      if (pt->line < minline)
	 minline = pt->line;
      }
   else if ((order->polecross == NORTHPOLE) && (pt->lat >= order->northlat))
      {
      i++;
      if (pt->line > maxline)
	 maxline = pt->line;
      if (pt->line < minline)
	 minline = pt->line;
      }
   else if ((order->polecross == SOUTHPOLE) && (pt->lat <= order->southlat))
      {
      i++;
      if (pt->line > maxline)
	 maxline = pt->line;
      if (pt->line < minline)
	 minline = pt->line;
      }


   /* Get the next point in the queue
   ----------------------------------*/
   if (c_qnext(points_q, &ptr))
      {
      c_errmsg("Error accessing queue", "pgtest-queue", &nonfatal);
      return(E_FAIL);
      }
   }


/* If minline or maxline are equal to their initial values then we must
   check to see if the area of interest falls entirely within the image.  In
   addition, if minline == maxline (ie. we are only going to return one line of
   the image that matches the standing order) then check the corner points of
   the area of interest in an attempt to find more than one line of the image.
------------------------------------------------------------------------------*/
if ((minline == model->nlines + 1) || (maxline == 0) || (minline == maxline))
   {
   if (order->polecross == NOPOLE)
      {
      /* Compute the line/sample of the four corners of the area of interest
      ----------------------------------------------------------------------*/
      lat_rads = (double)order->northlat * RAD_PER_DEG;
      lon_rads = (double)order->westlong * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[0], &fsamp[0], NULL);
      lon_rads = (double)order->eastlong * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[1], &fsamp[1], NULL);
      lat_rads = (double)order->southlat * RAD_PER_DEG;
      lon_rads = (double)order->westlong * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[2], &fsamp[2], NULL);
      lon_rads = (double)order->eastlong * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[3], &fsamp[3], NULL);
      }
   else if (order->polecross == NORTHPOLE)
      {
      /* Compute 4 line/sample points around the latitude of interest
      ---------------------------------------------------------------*/
      lat_rads = (double)order->northlat * RAD_PER_DEG;
      lon_rads = 0.0   * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[0], &fsamp[0], NULL);
      lon_rads = 90.0  * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[1], &fsamp[1], NULL);
      lon_rads = -90.0 * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[2], &fsamp[2], NULL);
      lon_rads = 180.0 * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[3], &fsamp[3], NULL);
      }
   else if (order->polecross == SOUTHPOLE)
      {
      /* Compute 4 line/sample points around the latitude of interest
      ---------------------------------------------------------------*/
      lat_rads = (double)order->southlat * RAD_PER_DEG;
      lon_rads = 0.0   * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[0], &fsamp[0], NULL);
      lon_rads = 90.0  * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[1], &fsamp[1], NULL);
      lon_rads = -90.0 * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[2], &fsamp[2], NULL);
      lon_rads = 180.0 * RAD_PER_DEG;
      c_ll2ls(&lat_rads, &lon_rads, &elev, model, &fline[3], &fsamp[3], NULL);
      }

   /* Determine if any of the four points lie within the image
   -----------------------------------------------------------*/
   for (i = 0; i < 4; i++)
      {
      if ((fline[i] > (float)maxline) && (fline[i] <= (float)model->nlines) &&
	  (fsamp[i] >= 1) && (fsamp[i] <= (float)model->nsamples))
	 maxline = (long)fline[i];
      if ((fline[i] < (float)minline) && (fline[i] >= 1) &&
	  (fsamp[i] >= 1) && (fsamp[i] <= (float)model->nsamples))
	 minline = (long)fline[i];
      }
   }


/* If minline and maxline are not equal to their initial values then we must
   have some lines that fall within the area of interest and start and nlines
   must be set accordingly
-----------------------------------------------------------------------------*/
if ((minline != model->nlines + 1) && (maxline != 0))
   {
   *start  = minline;
   *nlines = maxline - minline + 1;
   }


/* If nlines is less than the required number of lines then return NOPROCESS
----------------------------------------------------------------------------*/
if (*nlines < order->minlines)
   return(NOPROCESS);

return(PROCESS);
}



/****************************************
* Static function for determining dates *
****************************************/
static void get_dates
(
   long year,           /* IN  - Year of acquisition                          */
   long month,          /* IN  - Month of acquisition                         */
   long day,            /* IN  - Day of acquisition                           */
   long age,            /* IN  - # of days after acquisition                  */
   long *agedate,       /* OUT - YYYYMMDD form of "age" days after acquisition*/
   long *curdate        /* OUT - YYYYMMDD form of today's date                */
)
{
long done;              /* Loop control                                       */
long ndays;             /* Number of days in a month                          */
long now;               /* Current date and time encoded as a long            */

struct tm *today;       /* Structure containing the current date and time     */


/* Determine current date and time
----------------------------------*/
now   = time(NULL);
today = localtime(&now);


/* Retrieve the current date
----------------------------*/
*curdate =  (today->tm_year + 1900) * 10000;
*curdate += (today->tm_mon + 1) * 100;
*curdate += today->tm_mday;


/* Determine the year, month, and day of "age" days after acquisition
---------------------------------------------------------------------*/
day += age;
done = FALSE;
while (!done)
   {
   switch (month)
      {
      /* Months with 31 days */
      case  1:
      case  3:
      case  5:
      case  7:
      case  8:
      case 10:
      case 12:
	 ndays = 31;
	 break;

      /* Months with 30 days */
      case 4:
      case 6:
      case 9:
      case 11:
	 ndays = 30;
	 break;

      /* February */
      case 2:
	 ndays = 28;
	 if (c_isleap(&year))
	    ndays = 29;
	 break;
      }
   if (day <= ndays)
      done = TRUE;
   else if (month < 12)
      {
      day -= ndays;
      month++;
      }
   else
      {
      day -= ndays;
      month = 1;
      year++;
      }
   }

/* Create an integer representing the age date
----------------------------------------------*/
*agedate =  year * 10000;
*agedate += month * 100;
*agedate += day;
}
