/****************************************************************************
NAME:		C_LOCKFIL

PURPOSE:	Lock or unlock a file.  A file is locked by creating a
		lock file from the root of the input host file and an
		extension of "lock".  The file is not actually lock for
		exclusive use, but like processes should check for the
		existance of this lock file.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 D. Hollaren   03/92	Original development
  5.1	 D.Lloyd       07/98    Modified for Y2K compliance
  5.2    D.Lloyd       06/01    Added temporary diagnostics CR4506

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include "las.h"
#include "labserv.h"
#include "vll.h"
#include "adapsutil.h"

#define LOCK	1
#define UNLOCK	0
#define LOCKED	1

extern int errno;

int c_lockfil
(
   char *hname,		/* Host file name of the file to be locked           */

   long *lockflg 	/* Locking operation (1 - LOCK, 0 - UNLOCK)	     */
)
{
struct tm 	*lctm;	   	/* Local time structure			     */

int	fd;			/* File descriptor to the lock file	     */

long	time_val,		/* Time variable			     */
	nbytes,			/* Number of bytes to write		     */
	zero = 0,		/* Constant of zero			     */
	nonfatal = 1,		/* Nonfatal error flag			     */
	status;			/* Return status			     */

char	lockfile[CMLEN],	/* Lock file name			     */
	msg[ERRLEN],		/* Error message			     */
	tstamp[20];		/* Current time				     */

if (*lockflg == LOCK)
    {
    c_lsmknm(hname,"lock",lockfile);
    status = c_lsstat(lockfile,&zero);
    if (status == E_SUCC)
	{
	/*
	  Indicate that the file is already locked
	*/
	sprintf(msg,"File %s already locked",hname);
	c_errmsg(msg,"lockfil-locked",&nonfatal);
	return(LOCKED);
	}
    else
	{
	/*
	  Get the current date and time
	*/
	time(&time_val);
	lctm = localtime (&time_val);
	/*******************************************************
	DLLOYD Y2K
	  in year 2000 lctm->tm_year = 100 so needs to be MODed
	********************************************************/
	sprintf(tstamp,"%02d/%02d/%02d  %02d:%02d:%02d",lctm->tm_mon+1,
		lctm->tm_mday, lctm->tm_year%100, lctm->tm_hour, lctm->tm_min,
		lctm->tm_sec);

	/*
	  Create the lock file
	*/
	fd = creat(lockfile,zero);
	if (fd < 0)
	    {
	    /* The next 2 lines are temporary
	    diagnostic messages see CR4506.
	    ---------------------------------*/
	    perror("c_lockfil Error creating the lock file");
	    c_lasmsg(FALSE,"lockfil-errno","Error number is: %d", errno);
	    c_errmsg("Error creating the lock file","lockfil-lock",&nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Write the time code to the file
	*/
	nbytes = (long)strlen(tstamp);
	write(fd,tstamp,nbytes);

	/*
	  Close the lock file
	*/
	close(fd);
	}
    }
else
    {
    /*
      Create the lock file name
    */
    c_lsmknm(hname,"lock",lockfile);

    /*
      Determine if the file exists
    */
    status = c_lsstat(lockfile,&zero);
    if (status == E_SUCC)
	{
	/*
	  Remove the lock file
	*/
	status = deletefile(lockfile,"file",zero,zero);
	if (status != E_SUCC)
	    {
	    c_errmsg("Error removing the lock file","lockfil-unlock",&nonfatal);
	    return(E_FAIL);
	    }
	}
    }

return(E_SUCC);
}
 
