/*************************************************************************
FUNCTION:	C_PCK32

PURPOSE:	Pack 10-bit data into 32-bit format

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0                                    Original development
 5.0 	   12/91    T. Rockvam		LAS 5.0 Conversion


PROJECT		ADAPS

ALGORITHM 
    Top two bits are unused.  Three 10-bit words will be packed right
    justified into one 32-bit word.

    Format of input buffer:

    +--------------------+
    !000000:   Word 1    !  1st 16-bit word
    +--------------------+
    !000000:   Word 2    !  2nd 16-bit word
    +--------------------+
    etc.

    Format of output buffer:

    +------------------------------------------------+
    | 00:    Word 1    :    Word 2    :    Word 3    |  1st 32-bit long word
    +------------------------------------------------+
    | 00:    Word 4    :    Word 5    :    Word 6    |  2nd 32-bit long word
    +------------------------------------------------+
    etc.

    After byte swapping:

    +--------+------+--+----+----+--+------+
    | word 3 |word 2:w3|wd 1:wd 2|00:word 1|
    +--------+------+--+----+----+--+------+

ALGORITHM REFERENCES
                NOAA Polar Orbiter Data User's Guide
                  K. Kidwell, NOAA
                  Section 2 Level 1B Data Base

*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "adapsutil.h"

#define MASK	0X03ff

void c_pck32
(
    short *inbuf,		/* Series of 10-bit words stored in 16 bits 	*/
    unsigned char *outbuf,	/* Pointer to the 32-bit output buffer 		*/
    long *nwords	/* Number of input 16-bit shorts to pack	*/
)
{
short	*iptr;		/* Pointer to the 16-bit input buffer		*/

long	*optr,		/* Pointer to the 32-bit output buffer		*/
	num,		/* Number of full 32-bit long words		*/
	extra,		/* Number of extra shorts (num mod 3)		*/
	i;		/* loop counter					*/

/*
  Set pointers to the input and output buffers
*/
iptr = inbuf;
optr = (long *)outbuf;

/*
  Calculate the number of full 32-bit long words
*/
num = *nwords / 3;

/*
  Pack sets of three 10-bit words into a long
*/
for (i = 0; i < num; i++)
    {
    *optr = *iptr & MASK;
    iptr += 1;

    *optr = (*optr << 10) + (*iptr & MASK);
    iptr += 1;

    *optr = (*optr << 10) + (*iptr & MASK);
    iptr += 1;

    optr += 1;
    }

/*
  Handle any 10-bit words which are left over
*/
extra = (*nwords % 3);
if (extra >= 1)
    {
    /*
      Pack the first extra word
    */
    *optr = *iptr & MASK;

    /*
      Pack the second extra word and then shift them
    */
    if (extra == 2)
	{
	iptr += 1;
	*optr = (*optr << 10) + (*iptr & MASK);
	*optr = (*optr << 10);
	}
    else
	{
	/*
	  Shift the one word
	*/
	*optr = (*optr << 20);
	}
    }

return;
}
 
