/*******************************************************************************
FUNCTION:	c_pixscale

PURPOSE:	Unscale a buffer of pixels given a scaling factor and an offset
		and rescale the buffer using another scaling factor and offset.
		The data type may be converted during the rescaling operation.
		Note this function supports the new data types of unsigned word
		(EUWORD) and unsigned long (EULONG).

		At the beginning of the rescaling operation a status flag is
		set to E_SUCC.  If the rescaling operation generates a value
		too large or too small for the output data type, the status
		flag is or'd with TOOBIG or TOOSMALL and this status is
		returned by the function.  In addition, the output pixel value
		will be set to the largest or smallest valid value for the
		output data type.

		The pixel values in the input buffer are unscaled using the
		formula:

		    pixel = (*in - ioffset) * iscale
		
		and they are rescaled and stored in the output buffer using
		this formula:

		    *out = (pixel / oscale) + ooffset


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    01/94  D.Hollaren       Development.  Partially copied from c_pxcov().
  1.1    06/94  J.Fenno          Placed in the ADAPS util support library.

*******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "typlim.h"
#include "c_pixscale.h"


int c_pixscale
(
    long *idtype,	/* IN  - Data type of the input buffer                */
    long *odtype,	/* IN  - Data type of the output buffer               */
    unsigned char *in,	/* IN  - Input buffer                                 */
    unsigned char *out,	/* OUT - Output buffer                                */
    long *ns,		/* IN  - Number of samples in the buffers             */
    double iscale,	/* IN  - Scaling factor used in the input buffer      */
    double ioffset,	/* IN  - Offset used in the input buffer              */
    double oscale,	/* IN  - Scaling factor to be used in output buffer   */
    double ooffset,	/* IN  - Offset to be used in the output buffer       */
    long *rndopt	/* IN  - Rounding option (TRUE, FALSE)                */
)
{
long status = E_SUCC;	/* Return status                                      */

float rnd;		/* Rounding value                                     */


if (*rndopt)
    rnd = 0.5;
else
    rnd = 0.0;

switch (*idtype)
    {
    case EBYTE:
	switch (*odtype)
	    {
	    case EBYTE:
		byt_byt(in,out,*ns,&status,iscale,ioffset,oscale,ooffset,rnd);
		break;
	    case EWORD:
		byt_wrd(in,(short *)out,*ns,&status,iscale,ioffset,oscale,
			ooffset,rnd);
		break;
	    case EUWORD:
		byt_uwrd(in,(unsigned short *)out,*ns,&status,iscale,ioffset,
			 oscale,ooffset,rnd);
		break;
	    case ELONG:
		byt_lng(in,(long *)out,*ns,&status,iscale,ioffset,oscale,
			ooffset,rnd);
		break;
	    case EULONG:
		byt_ulng(in,(unsigned long *)out,*ns,&status,iscale,ioffset,
			 oscale,ooffset,rnd);
		break;
	    case EREAL:
		byt_flt(in,(float *)out,*ns,iscale,ioffset,oscale,ooffset);
		break;
	    case EDOUBLE:
		byt_dbl(in,(double *)out,*ns,iscale,ioffset,oscale,ooffset);
		break;
	    }
	break;

    case EWORD:
	switch (*odtype)
	    {
	    case EBYTE:
		wrd_byt((short *)in,out,*ns,&status,iscale,ioffset,oscale,
			ooffset,rnd);
		break;
	    case EWORD:
		wrd_wrd((short *)in,(short *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EUWORD:
		wrd_uwrd((short *)in,(unsigned short *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case ELONG:
		wrd_lng((short *)in,(long *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EULONG:
		wrd_ulng((short *)in,(unsigned long *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case EREAL:
		wrd_flt((short *)in,(float *)out,*ns,iscale,ioffset,oscale,
			ooffset);
		break;
	    case EDOUBLE:
		wrd_dbl((short *)in,(double *)out,*ns,iscale,ioffset,oscale,
			ooffset);
		break;
	    }
	break;

    case EUWORD:
	switch (*odtype)
	    {
	    case EBYTE:
		uwrd_byt((unsigned short *)in,out,*ns,&status,iscale,ioffset,
			 oscale,ooffset,rnd);
		break;
	    case EWORD:
		uwrd_wrd((unsigned short *)in,(short *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case EUWORD:
		uwrd_uwrd((unsigned short *)in,(unsigned short *)out,*ns,
			  &status,iscale,ioffset,oscale,ooffset,rnd);
		break;
	    case ELONG:
		uwrd_lng((unsigned short *)in,(long *)out,*ns,&status,iscale,ioffset,oscale,ooffset,rnd);
		break;
	    case EULONG:
		uwrd_ulng((unsigned short *)in,(unsigned long *)out,*ns,
			  &status,iscale,ioffset,oscale,ooffset,rnd);
		break;
	    case EREAL:
		uwrd_flt((unsigned short *)in,(float *)out,*ns,iscale,ioffset,
			 oscale,ooffset);
		break;
	    case EDOUBLE:
		uwrd_dbl((unsigned short *)in,(double *)out,*ns,iscale,ioffset,
			 oscale,ooffset);
		break;
	    }
	break;

    case ELONG:
	switch (*odtype)
	    {
	    case EBYTE:
		lng_byt((long *)in,out,*ns,&status,iscale,ioffset,oscale,
			ooffset,rnd);
		break;
	    case EWORD:
		lng_wrd((long *)in,(short *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EUWORD:
		lng_uwrd((long *)in,(unsigned short *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case ELONG:
		lng_lng((long *)in,(long *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EULONG:
		lng_ulng((long *)in,(unsigned long *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case EREAL:
		lng_flt((long *)in,(float *)out,*ns,iscale,ioffset,oscale,
			ooffset);
		break;
	    case EDOUBLE:
		lng_dbl((long *)in,(double *)out,*ns,iscale,ioffset,oscale,
		        ooffset);
		break;
	    }
	break;

    case EULONG:
	switch (*odtype)
	    {
	    case EBYTE:
		ulng_byt((unsigned long *)in,out,*ns,&status,iscale,ioffset,
			 oscale,ooffset,rnd);
		break;
	    case EWORD:
		ulng_wrd((unsigned long *)in,(short *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case EUWORD:
		ulng_uwrd((unsigned long *)in,(unsigned short *)out,*ns,
			  &status,iscale,ioffset,oscale,ooffset,rnd);
		break;
	    case ELONG:
		ulng_lng((unsigned long *)in,(long *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case EULONG:
		ulng_ulng((unsigned long *)in,(unsigned long *)out,*ns,&status,
			  iscale,ioffset,oscale,ooffset,rnd);
		break;
	    case EREAL:
		ulng_flt((unsigned long *)in,(float *)out,*ns,iscale,ioffset,
			 oscale,ooffset);
		break;
	    case EDOUBLE:
		ulng_dbl((unsigned long *)in,(double *)out,*ns,iscale,ioffset,
			 oscale,ooffset);
		break;
	    }
	break;

    case EREAL:
	switch (*odtype)
	    {
	    case EBYTE:
		flt_byt((float *)in,out,*ns,&status,iscale,ioffset,oscale,
			ooffset,rnd);
		break;
	    case EWORD:
		flt_wrd((float *)in,(short *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EUWORD:
		flt_uwrd((float *)in,(unsigned short *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case ELONG:
		flt_lng((float *)in,(long *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EULONG:
		flt_ulng((float *)in,(unsigned long *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case EREAL:
		flt_flt((float *)in,(float *)out,*ns,iscale,ioffset,oscale,
			ooffset);
		break;
	    case EDOUBLE:
		flt_dbl((float *)in,(double *)out,*ns,iscale,ioffset,oscale,
			ooffset);
		break;
	    }
	break;

    case EDOUBLE:
	switch (*odtype)
	    {
	    case EBYTE:
		dbl_byt((double *)in,out,*ns,&status,iscale,ioffset,oscale,
			ooffset,rnd);
		break;
	    case EWORD:
		dbl_wrd((double *)in,(short *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EUWORD:
		dbl_uwrd((double *)in,(unsigned short *)out,*ns,&status,
			 iscale,ioffset,oscale,ooffset,rnd);
		break;
	    case ELONG:
		dbl_lng((double *)in,(long *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset,rnd);
		break;
	    case EULONG:
		dbl_ulng((double *)in,(unsigned long *)out,*ns,&status,iscale,
			 ioffset,oscale,ooffset,rnd);
		break;
	    case EREAL:
		dbl_flt((double *)in,(float *)out,*ns,&status,iscale,ioffset,
			oscale,ooffset);
		break;
	    case EDOUBLE:
		dbl_dbl((double *)in,(double *)out,*ns,iscale,ioffset,oscale,
			ooffset);
		break;
	    }
	break;
    }

return(status);
}

static void byt_byt
(
    unsigned char *in,		/* Input buffer				*/
    unsigned char *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUTINY)
	{
	*out = MAXUTINY;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUTINY)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}

static void byt_wrd
(
    unsigned char *in,		/* Input buffer				*/
    short *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXWORD)
	{
	*out = MAXWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINWORD)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}

static void byt_uwrd
(
    unsigned char *in,		/* Input buffer				*/
    unsigned short *out,	/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    if (tmp > MAXUWORD)
	{
	*out = MAXUWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUWORD)
	{
	*out = MINUWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}

static void byt_lng
(
    unsigned char *in,		/* Input buffer				*/
    long *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXLONG)
	{
	*out = MAXLONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINLONG)
	{
	*out = MINLONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}

static void byt_ulng
(
    unsigned char *in,		/* Input buffer				*/
    unsigned long *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXULONG)
	{
	*out = MAXULONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINULONG)
	{
	*out = MINULONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}

static void byt_flt
(
    unsigned char *in,		/* Input buffer				*/
    float *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (float)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void byt_dbl
(
    unsigned char *in,		/* Input buffer				*/
    double *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (double)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void wrd_byt
(
    short *in,			/* Input buffer				*/
    unsigned char *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUTINY)
	{
	*out = MAXUTINY;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUTINY)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}

static void wrd_wrd
(
    short *in,			/* Input buffer				*/
    short *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXWORD)
	{
	*out = MAXWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINWORD)
	{
	*out = MINWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}

static void wrd_uwrd
(
    short *in,			/* Input buffer				*/
    unsigned short *out,	/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUWORD)
	{
	*out = MAXUWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUWORD)
	{
	*out = MINUWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}

static void wrd_lng
(
    short *in,			/* Input buffer				*/
    long *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXLONG)
	{
	*out = MAXLONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINLONG)
	{
	*out = MINLONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}

static void wrd_ulng
(
    short *in,			/* Input buffer				*/
    unsigned long *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXULONG)
	{
	*out = MAXULONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINULONG)
	{
	*out = MINULONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}

static void wrd_flt
(
    short *in,			/* Input buffer				*/
    float *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (float)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void wrd_dbl
(
    short *in,			/* Input buffer				*/
    double *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (double)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void uwrd_byt
(
    unsigned short *in,		/* Input buffer				*/
    unsigned char *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUTINY)
	{
	*out = MAXUTINY;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUTINY)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}

static void uwrd_wrd
(
    unsigned short *in,		/* Input buffer				*/
    short *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXWORD)
	{
	*out = MAXWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINWORD)
	{
	*out = MINWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}

static void uwrd_uwrd
(
    unsigned short *in,		/* Input buffer				*/
    unsigned short *out,	/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUWORD)
	{
	*out = MAXUWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUWORD)
	{
	*out = MINUWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}

static void uwrd_lng
(
    unsigned short *in,		/* Input buffer				*/
    long *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXLONG)
	{
	*out = MAXLONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINLONG)
	{
	*out = MINLONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}

static void uwrd_ulng
(
    unsigned short *in,		/* Input buffer				*/
    unsigned long *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXULONG)
	{
	*out = MAXULONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINULONG)
	{
	*out = MINULONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}

static void uwrd_flt
(
    unsigned short *in,		/* Input buffer				*/
    float *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (float)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void uwrd_dbl
(
    unsigned short *in,		/* Input buffer				*/
    double *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (double)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void lng_byt
(
    long *in,			/* Input buffer				*/
    unsigned char *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUTINY)
	{
	*out = MAXUTINY;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUTINY)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}

static void lng_wrd
(
    long *in,			/* Input buffer				*/
    short *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXWORD)
 	{
	*out = MAXWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINWORD)
	{
	*out = MINWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}

static void lng_uwrd
(
    long *in,			/* Input buffer				*/
    unsigned short *out,	/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUWORD)
 	{
	*out = MAXUWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUWORD)
	{
	*out = MINUWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}

static void lng_lng
(
    long *in,			/* Input buffer				*/
    long *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXLONG)
	{
	*out = MAXLONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINLONG)
	{
	*out = MINLONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}

static void lng_ulng
(
    long *in,			/* Input buffer				*/
    unsigned long *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXULONG)
	{
	*out = MAXULONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINULONG)
	{
	*out = MINULONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}

static void lng_flt
(
    long *in,			/* Input buffer				*/
    float *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (float)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void lng_dbl
(
    long *in,			/* Input buffer				*/
    double *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (double)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void ulng_byt
(
    unsigned long *in,		/* Input buffer				*/
    unsigned char *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUTINY)
	{
	*out = MAXUTINY;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUTINY)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}

static void ulng_wrd
(
    unsigned long *in,		/* Input buffer				*/
    short *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXWORD)
	{
	*out = MAXWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINWORD)
	{
	*out = MINWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}

static void ulng_uwrd
(
    unsigned long *in,		/* Input buffer				*/
    unsigned short *out,	/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUWORD)
	{
	*out = MAXUWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUWORD)
	{
	*out = MINUWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}

static void ulng_lng
(
    unsigned long *in,		/* Input buffer				*/
    long *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXLONG)
	{
	*out = MAXLONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINLONG)
	{
	*out = MINLONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}

static void ulng_ulng
(
    unsigned long *in,		/* Input buffer				*/
    unsigned long *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXULONG)
	{
	*out = MAXULONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINULONG)
	{
	*out = MINULONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}

static void ulng_flt
(
    unsigned long *in,		/* Input buffer				*/
    float *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (float)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void ulng_dbl
(
    unsigned long *in,		/* Input buffer				*/
    double *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (double)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void flt_byt
(
    float *in,			/* Input buffer				*/
    unsigned char *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUTINY)
	{
	*out = MAXUTINY;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUTINY)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}

static void flt_wrd
(
    float *in,			/* Input buffer				*/
    short *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXWORD)
	{
	*out = MAXWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINWORD)
	{
	*out = MINWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}

static void flt_uwrd
(
    float *in,			/* Input buffer				*/
    unsigned short *out,	/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUWORD)
	{
	*out = MAXUWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUWORD)
	{
	*out = MINUWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}

static void flt_lng
(
    float *in,			/* Input buffer				*/
    long *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXLONG)
	{
	*out = MAXLONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINLONG)
	{
	*out = MINLONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}

static void flt_ulng
(
    float *in,			/* Input buffer				*/
    unsigned long *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
float	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXULONG)
	{
	*out = MAXULONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINULONG)
	{
	*out = MINULONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}

static void flt_flt
(
    float *in,			/* Input buffer				*/
    float *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (float)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void flt_dbl
(
    float *in,			/* Input buffer				*/
    double *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (double)((((*in - ioffset) * iscale) / oscale) + ooffset);
}

static void dbl_byt
(
    double *in,			/* Input buffer			*/
    unsigned char *out,		/* Output buffer		*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
double	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUTINY)
	{
	*out = MAXUTINY;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUTINY)
	{
	*out = MINUTINY;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}

static void dbl_wrd
(
    double *in,			/* Input buffer				*/
    short *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
double	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXWORD)
	{
	*out = MAXWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINWORD)
	{
	*out = MINWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}

static void dbl_uwrd
(
    double *in,			/* Input buffer				*/
    unsigned short *out,	/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
double	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXUWORD)
	{
	*out = MAXUWORD;
	*flag |= TOOBIG;
	}
    else if (tmp < MINUWORD)
	{
	*out = MINUWORD;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}

static void dbl_lng
(
    double *in,			/* Input buffer				*/
    long *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
double	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXLONG)
	{
	*out = MAXLONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINLONG)
	{
	*out = MINLONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}

static void dbl_ulng
(
    double *in,			/* Input buffer				*/
    unsigned long *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset,		/* Output scale and offset values	*/
    double rnd			/* Rounding value			*/
)
{
double	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXULONG)
	{
	*out = MAXULONG;
	*flag |= TOOBIG;
	}
    else if (tmp < MINULONG)
	{
	*out = MINULONG;
	*flag |= TOOSMALL;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}

static void dbl_flt
(
    double *in,			/* Input buffer				*/
    float *out,			/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    long *flag,			/* Bounds status flag			*/
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
double	tmp;			/* Temporary calculation buffer		*/

for(; ns--; in++, out++)
    {
    tmp = (((*in - ioffset) * iscale) / oscale) + ooffset;

    /*
      Check the bounds for the data type
    */
    if (tmp > MAXREAL)
	{
	*out = MAXREAL;
	*flag |= TOOBIG;
	}
    else if (tmp < MINREAL)
	{
	*out = MINREAL;
	*flag |= TOOSMALL;
	}
    else
	*out = (float)tmp;
    }
}

static void dbl_dbl
(
    double *in,			/* Input buffer				*/
    double *out,		/* Output buffer			*/
    long ns,			/* Number of sample in the buffers      */
    double iscale,
    double ioffset,		/* Input scale and offset values	*/
    double oscale,
    double ooffset		/* Output scale and offset values	*/
)
{
for(; ns--; in++, out++)
    *out = (double)((((*in - ioffset) * iscale) / oscale) + ooffset);
}
