/******************************************************************************
FUNCTION:	C_WINDPAR

PURPOSE:	Get the input parameters related to the window and return a
		window in "line/sample" format.

Who might want to call this routine?  Module SOMEMOD with the following
subcommands and parameters:

SOMEMOD-LINE
    PARM SL
    PARM NL
SOMEMOD-LATRANGE
    PARM SLAT
    PARM ELAT
    PARM QUADNUM
    PARM AREAOPT
SOMEMOD-LATLONG
    PARM SLAT
    PARM SLONG
    PARM ELAT
    PARM ELONG

What is returned from this module?  A window to process:
    window[SL]
    window[SS]
    window[NL]
    window[NS]

Note that an AVHRR model must be available before this support routine can be
called.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0	 11/97  T.Baltzer	 Development

PROJECT:	ADAPS
*******************************************************************************/
#include "tae.h"
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"
#include "local_protos.h"


int c_windpar
(
    struct PARBLK *vblock,	/* INPUT: Variable block		*/
    struct MODEL *model,	/* INPUT: Model and orbital information	*/
    long window[4]		/* OUTPUT: Window specification		*/
)


{
long	areaopt,	/* Flag indicating area of interest		*/
	one = 1,	/* The value of one				*/
	quadnum,	/* Quadrant number for the latrange subcommand  */
	slen = 7,	/* Length of subcommand string			*/
	status,		/* Status returned from support calls   	*/
	wflag;		/* Window subcommand indicator			*/

long	inwind[4];	/* Input window (lines)				*/
float   inlat[4];	/* Input window (lats/lons or lat range)	*/

char	subcmd[7];	/* Buffer to contain the subcommand 		*/
char    tmp[CMLEN];	/* Temporary string				*/

struct VARIABLE	*val;		/* Pointer to a TAE variable 	        */


/*
  Get the windowing option info
*/
status = c_getstr(vblock,"_SUBCMD",&one,&slen,subcmd);


if (strcasecmp(subcmd,"LINE") == 0)		/* -LINE subcmd	*/
    {
    inwind[SS] = 1;		/* Cannot window samples */
    if (model->dtype != GAC)
	inwind[NS] = LAC_SAMPS;
    else
	inwind[NS] = GAC_SAMPS;

    wflag = 0;

    /*
      Retrieve the starting line number
      if SL is not specified, then set SL = 1
    */
    val = p_find(vblock,"SL");
    if (val->v_count == 0)
	inwind[SL] = 1;
    else
	inwind[SL] = IVAL(*val,0);

    if (inwind[SL] < 1)
	{
	c_lasmsg(FALSE,"c_windpar-stline",
	    "Specified starting line was < 1, reset to 1");
	inwind[SL] = 1;
	}
    
    /*
      Retrieve the number of lines to process.  If the user defaulted this
      parameter, the entire image starting at the line "SL" will be processed.
    */
    val = p_find(vblock,"NL");
    if (val->v_count != 0)
       inwind[NL] = IVAL(*val,0);  /* Number of lines was specified */
    else
       inwind[NL] = -1;		   /* Number of lines not specified */
    
    if (inwind[NL] < 1 && inwind[NL] != -1)
	{
	c_lasmsg(FALSE,"c_windpar-end",
	    "Specified ending line was beyond image, reset to image ending line");
	inwind[NL] = -1;
	}
    
    }

else if (strcasecmp(subcmd,"LATRANGE") == 0)		/* -LATRANGE subcmd */
   {
   wflag = 1;

   /* getting the starting latitude */
   val = p_find(vblock,"SLAT");
   inlat[0] = (float)RVAL(*val,0); 

   /* getting the ending latitude */
   val = p_find(vblock,"ELAT");
   inlat[2] = (float)RVAL(*val,0); 

   /* getting the quadrant number */
   val = p_find(vblock,"QUADNUM");
   quadnum = IVAL(*val,0);

   /* getting the area option */
   val = p_find(vblock,"AREAOPT");
   strcpy(tmp,SVAL(*val,0));

   if (strcmp(tmp,"FIRST") == 0)
	areaopt = 0;
   else if (strcmp(tmp,"SECOND") == 0)
	areaopt = 1;
   }

else if (strcasecmp(subcmd,"LATLONG") == 0)	/* -LATLONG subcmd */
   {
   wflag = 2;

   /* getting the starting latitude */
   val = p_find(vblock,"SLAT");
   inlat[0] = (float)RVAL(*val,0); 

   /* getting the ending latitude */
   val = p_find(vblock,"ELAT");
   inlat[2] = (float)RVAL(*val,0); 

   /* getting the starting longitude */
   val = p_find(vblock,"SLONG");
   inlat[1] = (float)RVAL(*val,0);

   /* getting the ending longitude */
   val = p_find(vblock,"ELONG");
   inlat[3] = (float)RVAL(*val,0);
   }

/*
  Now call the function to convert the parameters into a line/sample window
*/
status = convert_wind(inwind,inlat,wflag,quadnum,areaopt,model,window);
if (status != E_SUCC)
    return(E_FAIL);


return(E_SUCC);
}
