/******************************************************************************
FUNCTION:	convert_window

PURPOSE:	Convert starting lat/long values to appropriate image lines

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0                              	Original development
 5.0	   12/91    T. Rockvam		LAS 5.0 conversion:
					new algorithm for finding line numbers
					for a lat range
 5.1	   11/96    A.Vander Vorste	Modified for Window1b (From MKLEVEL1B)
 6.0	   11/97    T. Baltzer		Made part of support routine c_windpar()

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/
#include "las.h"
#include "adaps.h"
#include "adapssup.h"
#include "adapsutil.h"


#define EL 2


/* MACROS */
/* convert radians to degrees */
#ifndef DEG
#define DEG(p)    (p) * 180 / PIE
#endif

/* convert degress to radians */
#ifndef RAD
#define RAD(p)    (p) * PIE / 180
#endif

/* find the min of two inputs */
#ifndef MIN
#define MIN(a,b)  ((a) < (b) ? (a) : (b))
#endif

/* find the max of two inputs */
#ifndef MAX
#define MAX(a,b)  ((a) > (b) ? (a) : (b))
#endif

/* find the min of four inputs */
#ifndef MIN4
#define MIN4(a,b,c,d) (MIN(a,b) < MIN(c,d) ? MIN(a,b) : MIN(c,d))
#endif

/* find the max of four inputs */
#ifndef MAX4
#define MAX4(a,b,c,d) (MAX(a,b) > MAX(c,d) ? MAX(a,b) : MAX(c,d))
#endif


int convert_wind
(
   long inwind[4],		/* Input window (lines)			*/
   float inlat[4],		/* Input window (lats/lons or lat range)*/
   long wflag,			/* window option			*/
   long	quadnum,		/* quad code number			*/
   long	areaopt,		/* first/second area option		*/
   struct MODEL *model,         /* Model and orbital information        */
   long window[4] 		/* Window returned for image		*/
)
{

long	sl[3],			/* starting line			*/
	el[3],			/* ending line				*/
	i = 0,j = 0,		/* loop counter 			*/
	linerange[2],	        /* linerange to be windowed		*/
	numfound = 0,		/* number of areas found		*/
	status = 0;		/* function return status		*/

double  dwind[4],		/* double values of window		*/
	elev = 0.0,		/* elevation of data			*/
	*view_angles = NULL;	/* Zenith and relative azimuth angles	*/

float	l1=0,l2=0,l3=0,l4=0, 	/* temporary lines for lat,long choice	*/
	s1=0,s2=0,s3=0,s4=0; 	/* temporary sample for lat,long choice	*/



/*
  Initialize lineranges
*/
for (j = 0; j<2; j++)
    linerange[j] = 0;


if (wflag == 0) /* -LINE: window by lines */
    {

    /* if the user entered values were outside the original image */
    if (inwind[SL] > model->nlines)
	{
    	c_lasmsg(FALSE,"c_windpar-beyimg",
	    "Specified starting line was beyond image");
	return(E_FAIL);
	}
    if (inwind[NL] > model->nlines - inwind[SL] + 1) 
	{
    	c_lasmsg(FALSE,"c_windpar-end",
	"Specified number of lines reset from %d to %d",inwind[NL],
	model->nlines - inwind[SL] + 1);
        inwind[NL] = model->nlines - inwind[SL] + 1;
	}
    if (inwind[NL] == -1)		/* NL wasn't specified */
	{
        inwind[NL] = model->nlines - inwind[SL] + 1;
    	c_lasmsg(FALSE,"c_windpar-nln"," Number of lines set to %d",
	inwind[NL]);
	}

    window[SL] = inwind[SL];
    window[SS] = 1;
    window[NL] = inwind[NL];
    window[NS] = model->nsamples;
    c_lasmsg(FALSE," ","Window specified for image is (%d %d %d %d)",
	     window[SL],window[SS],window[NL],window[NS]);
    }

else if (wflag == 2)/* -LATLONG: Window contains lats,longs of two corners */
    {
    /*
	l1,s1-----------------l2,s2
	  |			|
	  |			|
	l3,s3-----------------l4,s4
      Convert lat/long corners of window to line/sample
    */
    for (i = 0; i < 4; i++)
	dwind[i] = (double)RAD(inlat[i]);
    c_ll2ls(&dwind[0],&dwind[1],&elev,model,&l1,&s1,view_angles);
    c_ll2ls(&dwind[0],&dwind[3],&elev,model,&l2,&s2,view_angles);
    c_ll2ls(&dwind[2],&dwind[1],&elev,model,&l3,&s3,view_angles);
    c_ll2ls(&dwind[2],&dwind[3],&elev,model,&l4,&s4,view_angles);

    /*
      set the input lat/long window to line/sample window
    */
    inlat[SL] = MIN4(l1,l2,l3,l4);
    inlat[EL] = MAX4(l1,l2,l3,l4);

    /* if the windowed area is out of the bounds of the image */
    if (inlat[SL] < 1.0)
	{
	inlat[SL] = 1.0;
	c_lasmsg(FALSE,"c_windpar-stline",
    	     "Specified starting line was before image, reset to 1");
	}
    if ((inlat[SL] > (float)model->nlines) || (inlat[EL] < 1.0))
	{
    	c_lasmsg(FALSE,"c_windpar-nointsct","Area did not intersect image");
	return(E_FAIL);
	}
    if (inlat[EL] > (float)model->nlines) 
	{
        inlat[EL] = (float)model->nlines;
    	c_lasmsg(FALSE,"c_windpar-end",
	"Specified ending lat was beyond image, reset to image ending line");
	}

    /* setting up the linerange -> to be converted to timerange so
       can account for drop lines not being in the l1B file       */
    linerange[0] = (long)inlat[SL];
    linerange[1] = (long)(inlat[EL] + 0.5);
    
    /*
      Inform the user of the window we are using:
      line numbers found are relative to the archive image;
      must flip the line numbers for ASCENDING passes
    */
    if (model->cpass == ASCENDING)
	{
	window[SL] = (long)(model->nlines - linerange[1] + 1);
	window[NL] = (long)(linerange[1] - linerange[0] + 1);
	}
    else
	{
	window[SL] = linerange[0];
	window[NL] = (long)(linerange[1] - linerange[0] + 1);
	}
    window[SS] = 1;
    window[NS] = model->nsamples;
    c_lasmsg(FALSE," ","Window specified for image is (%d %d %d %d)",
	     window[SL],window[SS],window[NL],window[NS]);
    }

else if (wflag == 1) /* -LATRANGE: Window contains 2 lats */
    {
    /* convert the latitudes to radians */
    dwind[0] = (double)RAD(inlat[0]);
    dwind[1] = (double)RAD(inlat[2]);

    /* find the lineranges that correspond to this latrange 
       only one range will be used @@@ (for GAC data sometimes
       up to 3 areas will fall with in a latrange)            */
    status = findrange(dwind[0],dwind[1],sl,el,quadnum,areaopt,&numfound,model);
    if (status != E_SUCC)
	{
        c_lasmsg(FALSE,"c_windpar-lrnfnd","Latitude range not found");
	return(E_FAIL);
	}

    /*
      Define window (sl,el) from range found (the first set!)
    */
    inlat[SL] = (float)MIN(sl[0],el[0]);
    inlat[EL] = (float)MAX(sl[0],el[0]);

    /*
      Define linerange (sl el)

      Inform the user of the window we are using:
      line numbers found are relative to the archive image;
      must flip the line numbers for ASCENDING passes.

      Only one of the lineranges returned will be used and is
      defined by areaopt.
    */
    linerange[0] = sl[0];
    linerange[1] = el[0];
    if (model->cpass == ASCENDING)
	{
	window[SL] = model->nlines - linerange[1] + 1;
	window[NL] = linerange[1] - linerange[0] + 1;
	}
    else
	{
	window[SL] = linerange[0];
	window[NL] = linerange[1] - linerange[0] + 1;
	}
    window[SS] = 1;
    window[NS] = model->nsamples;
    c_lasmsg(FALSE," ","Window specified for image is (%d %d %d %d)",
	     window[SL],window[SS],window[NL],window[NS]);
    }


return(E_SUCC);
}
