/****************************************************************************
NAME:		PROCESS

PURPOSE:	Process each line of the input to correct the satellite
		zenith angle and then the relative azimuth angle.

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "correct.h"

FUNCTION process(vblock,ddr,hosout,window,totbnd,odtype,msgtim,minmax,
		 minval,maxval)

struct PARBLK *vblock;		/* TAE parameter block			     */
struct DDR    *ddr;		/* Data Descriptor Record		     */

char	*hosout;		/* Host name of output image		     */

long	window[],		/* Window specification of input images      */
	totbnd,			/* Total number of bands to be processed     */
	odtype,			/* Data type of output image                 */
	minmax,			/* Min/max flag                              */
	msgtim;			/* Elapsed time between display messages     */

double  minval[],		/* Min pixel value array                     */
	maxval[];		/* Max pixel value array                     */
{

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern int fdesc[5];		/* Image file descriptors used for image I/O */
extern int gdesc;		/* Group descriptor used for image I/O       */
extern long offset1[4];		/* Offsets into thebuf for input bands	     */
extern long offset2[4]; 	/* Offsets into "thebuf" for output bands    */
extern long curent;		/* Current processing line		     */
extern long curcat;		/* Current processing band		     */

extern unsigned char *thebuf;   /* Image I/O buffer                          */

long	curlin,			/* Current line being processed              */
	size,			/* Number of bytes per pixel 	 	     */
	one = 1,		/* One					     */
	nonfatal = 1,		/* Nonfatal error code			     */
	msg_flag = ALLCAT;	/* Flag for processing message               */

struct PROJ proj;		/* Projection information		     */

/*
  Setup for data type
*/
switch(odtype)
    {
    case EBYTE:  size = 1; break;
    case EWORD:  size = 2; break;
    case ELONG:  size = 4; break;
    case EREAL:  size = 4; break;
    }

/*
  Initialize the projection package
*/
init_proj(ddr,&proj);

/*
  Setup for processing message utility
*/
c_inittm("ANGCOR","Processed","line","bands"," ",&window[NL],&totbnd,&msgtim,
	 &msg_flag);
curent = 1;    
c_startm();

/*
   Initialize min/max arrays
*/
if (minmax)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/*
  Main processing loop
*/
for (curlin = 1; curlin <= window[NL]; curlin++)
    {
    /*
      Get the next line
    */
    if (c_estep(&gdesc) != E_SUCC)
	c_clean(vblock,"angcor",hosout,&one);

    /*
      Calculate the satellite zenith and the relative azimuth angles
    */
    calc_views(thebuf + (*(offset1 + BAND1) * size),
	       thebuf + (*(offset1 + BAND3) * size),
	       thebuf + (*(offset2 + BAND1) * size),
	       thebuf + (*(offset2 + BAND3) * size),
	       window[NS],odtype,curlin,&proj);

    /*
      Copy the solar zenith angles (No changed required)
    */
    c_pxcopy(thebuf + (*(offset1 + BAND2) * size),
	     thebuf + (*(offset2 + BAND2) * size),
	     &odtype,&window[NS]);

    /*
      Find the minimum and maximum values for each band
    */
    if (minmax)
	{
	if (c_pxmnmx(thebuf + (*(offset2 + BAND1) * size),&window[NS],
                     &minval[BAND1],&maxval[BAND1],&odtype) != E_SUCC)
	    c_errmsg("Nonfatal error occured","angcor-warn",&nonfatal);

	if (c_pxmnmx(thebuf + (*(offset2 + BAND2) * size),&window[NS],
                     &minval[BAND2],&maxval[BAND2],&odtype) != E_SUCC)
	    c_errmsg("Nonfatal error occured","angcor-warn",&nonfatal);

	if (c_pxmnmx(thebuf + (*(offset2 + BAND3) * size),&window[NS],
                     &minval[BAND3],&maxval[BAND3],&odtype) != E_SUCC)
	    c_errmsg("Nonfatal error occured","angcor-warn",&nonfatal);
	}

   /*
     Increment the processing message line counter
   */
   curent++;
   }

/*
  Final write
*/
if (c_estep(&gdesc) != E_SUCC)
      c_clean(vblock,"angcor",hosout,&one);

/*
  Turn off display messages
*/
c_stoptm();

return;
}
