
/* getin.c  by David Zokaites June 1995;  prevents memory problems with huge
   input image names.
   
   Changes to the supporting file c_getin.c to produce getin.c:  function
   getin2() was renamed getin2_changed().  Called c_getalias_changed() instead
   of c_getalias().  The length of some char variables changed to 2000 */

/* function declarations to prevent link problems with c_getin.c */ 
   static int c_getin(); 
   static int imgprs(); 
   static int compress(); 
   static int image_info(); 
   static int make_window(); 
   static int make_bands(); 
   static int make_proj(); 



/*******************************************************************************
NAME				C_GETIN

PURPOSE		Break input specification into individual image specifications.

PROGRAM HISTORY
	PROGRAMMER	  DATE		REASON
	----------	  ----		------
	P. Mumford	 3/15/85	Original development for NEWLAS.
	K. Gacke	 9/09/85	Updated the include files and fixed a
					bug involving parameter "same_window"
	L. Huewe	10/17/85	Reimplemented QFLAG parameter.
	K. Gacke         9/15/86        Added parameters "nbands" and "totbnd"
					to return number of bands per image and
					total number of bands.
	K. Gacke	12/12/86	Allow user to name images with any
					extension and not just "img"
	K. Gacke	 2/2/87		Modified to allow device to be specified
					on VMS.  The colon character has two
					meanings depending on it position: 
					1) separator of device and directory
					2) separator of window and bands
	K. Gacke	 2/2/87		Ensure no spaces trail the image name
	K. Zanter	 12/87		1.  switch to raw C types 
					2.  strings descriptor arguments to 
					    character strings
				        3.  changed include file specification
					4.  renamed source from "getin" to
					    "c_getin"
				        5.  moved bridge routine to separate
					    source file
	D. Hollaren	 04/88		changed include spec to las
					changed cerrmsg to c_errmsg and
					passed the status code by address
	B. Ailts	07/88		Added better error checking
        B. Ailts	09/88		Returns *images as zero if the
					image_name[*element] was not specified
        B. Ailts	09/88		Returns a null values in the host and
					LAS names if the image_name[*element] 
					was not found in the parameter block
	D. VanderZee 	07/90		Standardized error handling 
	D. Etrheim	09/90		Allow a band range to be specified
        T. Mittan	09/92		type cast pos and left to ints
					in comparision statments for ANSII
					standard compilers
	T. Mittan	09/92		Changed type casts and placed 
					(char *) on constants instead
	T. Mittan	07/93		Changed PRS_LST to double, added
					new windowing options, added 
					make_proj
        D. Bridges      10/93           Split c_getin into c_getin and c_getin2
                                        where c_getin is called from TAE and
                                        c_getin2 is called from outside TAE
	D. Etrheim	10/93		Return status from c_getin2


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	C_GETIN must be run under TAE.

PROJECT				LAS

ALGORITHM:
	Get the value of the input image parameter
 	If the count of the input image names are less than *element + 1
	   String copy a blank into the tae file names
	   String copy a blank into the host file names
	   Set *images to zero
	   Return
	Substitute any aliases in the input image parameter
	Parse the string and return the name, window, and bands in an array,
		also the number of images
	For each image
		Get the full TAE name
		Get the host name
		Check the window for accuracy and default value
		Check the bands for accuracy and default value
	End for


ALGORITHM REFERENCES
	This algorithm is roughly based on the GET_IN utility in LAS.

******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <ctype.h> 

#include "las.h"



FUNCTION static c_getin( block, param_buf, element, tnames, hnames, images, window, /* line changed */ 
		bands, nbands, totbnd, imglimit, same_window, qflag )

	struct	PARBLK	*block;		   /* TAE parameter block             */
		char	*param_buf;	   /* Input image parameter name      */
		long    *element;	   /* element array number of param.  */
		char	tnames[][CMLEN];   /* TAE names of images specified   */
		char	hnames[][CMLEN];   /* host names of images specified  */
		long	*images;	   /* Number of images                */
		long	(*window)[4];	   /* Image windows (sl ss nl ns)     */
		long	(*bands)[MAXBND+1];/* Image bands                     */
		long	*nbands;	   /* Number of bands per image       */
		long	*totbnd;	   /* total number of bands specified */
		long	*imglimit;	   /* Maximum images caller can handle*/

		long	*same_window;	   /* 1 if all windows the same	      */
					   /* 0 otherwise                     */
                long    (*qflag)[2];       /* 1 if caller specified the window*/
                                           /* and bands                       */
                                           /* 0 if default window and bands   */

/****************************************************************************** 
    Purpose:
       Parse image names, bands and windows.  This is used by applications
       that use the TAE interface.  Calls c_getpar2 to do the actual parse.
*******************************************************************************/

{
	struct	VARIABLE *i_spec;	   /* Input image parameter value     */

		char	inparm[CMLEN];	   /* Input image string with         */
              				   /* optional window specification   */
                long    nonfatal = 1;      /* non-fatal key for c_errmsg      */
long status;				   /* return status */

/* Get image specification and parse it. */
if( NULL == (i_spec = p_find( block, param_buf )))
	{
	c_errmsg("Error: Input image parameter does not exist",
		 "getin-param",&nonfatal);
	return(E_FAIL);
	};

if (i_spec->v_count < *element + 1)
   {
   *images = 0;
   strcpy(hnames[0],"\0");
   strcpy(tnames[0],"\0");
   return(E_SUCC);
   }
strcpy(inparm,SVAL(*i_spec,*element));

status = c_getin2(inparm,tnames,hnames,images,window,bands,nbands,
                totbnd,imglimit,same_window,qflag);
return(status);
}



FUNCTION c_getin2_changed( inparm, tnames, hnames, images, window, bands,  /* line was changed */ 
                  nbands, totbnd, imglimit, same_window, qflag )

                char    inparm[];          /* Input image string with         */
 					   /* optional window specificarion   */
                char    tnames[][CMLEN];   /* TAE names of images specified   */
                char    hnames[][CMLEN];   /* host names of images specified  */
                long    *images;           /* Number of images                */
                long    (*window)[4];      /* Image windows (sl ss nl ns)     */
                long    (*bands)[MAXBND+1];/* Image bands                     */
                long    *nbands;           /* Number of bands per image       */
                long    *totbnd;           /* total number of bands specified */
                long    *imglimit;         /* Maximum images caller can handle*/

                long    *same_window;      /* 1 if all windows the same       */
                                           /* 0 otherwise                     */

                long    (*qflag)[2];       /* 1 if caller specified the window*/
                                           /* and bands                       */
                                           /* 0 if default window and bands   */

/******************************************************************************
    Purpose:
        Parse image names, bands, and windows.  May be called by applications
        that do not use the TAE interface.  For the applications that do use
        TAE, c_getpar must be called.
*******************************************************************************/

{
static	struct	PRS_LST	p_list[MAXIMG];	   /* Image list returned from imgprs */

                char    outparm[2000];     /* Output parm spec without aliases*/ /* line changed */ 
                long    status;            /* status code returned by CM      */
                char    err_text[ERRLEN+1];/* error message text              */
                unsigned short  image;     /* image counter and index         */
                char    tae[CMLEN];        /* current TAE name                */
                char    *ptr;
                long    mode = 0;          /* file access                     */
                long    nonfatal = 1;      /* Non-fatal key for c_errmsg      */
                long    temp;

/* Substitute any alias 
-------------------------*/
if (c_getalias_changed(inparm,outparm) != E_SUCC) /* this line was changed */
	{
	c_errmsg("Error returned from getalias_changed","getin-call",&nonfatal); /* changed */ 
	return(E_FAIL);
	}

if( E_FAIL == imgprs(outparm,p_list,images))
	{
	c_errmsg("Error: Input image parse was unsuccessful","getin-parse",
		 &nonfatal);
	return(E_FAIL);
	};

if( *images > *imglimit )
	{
	sprintf( err_text, "%s %d %s %d %s.", "Error: There were", *images,
			"parsed which is greater than the limit of",
			*imglimit, "images");
	c_errmsg(err_text,"getin-imglmt",&nonfatal);
	return(E_FAIL);
	}

for ( image = 0; image < *images; image++ )
   {
   strcpy(tae,p_list[image].name);

   if (strchr(tae, TAE_DIR_BEG) || strchr(tae, TAE_EXT))
      {
      if ((ptr = strrchr(tae,TAE_EXT)) == 0)
         sprintf(&tnames[image][0],"%s%c%s",tae,TAE_EXT,"img");
      else 
         {
         if (strchr(tae,TAE_EXT) != ptr)
            {
            c_errmsg(
             "Error: Invalid LAS image name: Multiple extensions not allowed",
             "getin-name",&nonfatal);
            return(E_FAIL);
            }
         strcpy(&tnames[image][0],tae);
         }
      }
   else
      {
      temp = (long)strlen(tae);
      if (strcmp(&tae[temp-4], ".img") != 0)
         sprintf(tnames[image],"%s%c%s", tae, HOST_EXT, "img");
      else
         strcpy(tnames[image], tae);
      }
   }

/* Get the full TAE and host names. */
for ( image = 0; image < *images; image++ )
   {
   if (c_cmhost(&tnames[image][0],&hnames[image][0]) != E_SUCC)
      {
      c_errmsg("Error returned from cmhost","getin-call",&nonfatal);
      sprintf(err_text,"Error retrieving host file name for %s",
		&tnames[image][0]);
      c_errmsg(err_text,"getin-cmhost",&nonfatal);
      return(E_FAIL);
      }

   strcpy(tnames[image], hnames[image]);
   if (c_lsstat(&hnames[image][0],&mode) != E_SUCC)
      {
      sprintf(err_text,"Error: Image file %s does not exist",&tnames[image][0]);
      c_errmsg(err_text,"getin-exist",&nonfatal);
      return(E_FAIL);
      }
   }

/* Check the windows and bands. */
*totbnd = 0;
*same_window = 1;
for ( image = 0; image < *images; image++ )
   {

   /* This is where projection coordinates can be converted to line sample
   ---------------------------------------------------------------------*/
   if (p_list[image].proj[0] != 0)
      {
      status = c_wndconv(&hnames[image][0],p_list[image].window,
   		         p_list[image].proj);
      if (status != E_SUCC)
         {
         c_errmsg("Error returned from wndconv","getin-call",&nonfatal);
	 return(E_FAIL);
	 }
      }

   status = c_wndchk(&hnames[image][0],p_list[image].window,window[image]);
   if (status != E_SUCC)
      	{
        c_errmsg("Error returned from wndchk","getin-call",&nonfatal);
      	sprintf(err_text,"Error: Window specified for %s was invalid",
		 &tnames[image][0]);
	c_errmsg(err_text,"getin-invwnd",&nonfatal);
      	return(E_FAIL);
      	}

   if (image > 0)
      	if ((window[image][2] != window[image-1][2]) ||
	    (window[image][3] != window[image-1][3]) )
		*same_window = 0;

   status = c_bndchk(&hnames[image][0],p_list[image].bands,bands[image],
		     &nbands[image]);
   if (status != E_SUCC)
      	{
        c_errmsg("Error returned from bndchk","getin-call",&nonfatal);
      	sprintf(err_text,"Error: Bands specified for %s were invalid",
      		&tnames[image][0]);
	c_errmsg(err_text,"getin-invbnds",&nonfatal);
      	return(E_FAIL);
      	}

   *totbnd += nbands[image];
   qflag[image][0] = p_list[image].qflag[0];
   qflag[image][1] = p_list[image].qflag[1];
   }

if (*totbnd > MAXBND)
   {
   c_errmsg("Error: Too many bands specified in input image(s)","getin-invband",
	    &nonfatal);
   return(E_FAIL);
   }
return(E_SUCC);
}


 FUNCTION static imgprs( i_specs, list, images )/* line changed */ 
		char	*i_specs;
	struct	PRS_LST *list;
		long	*images;
	{
		char	line[2000+1],	/* holds compressed i_specs */  /* line was changed */ 
			*spec_begin,	/* address of next image spec */
			*pos,		/* position of '+' */
			err_text[ERRLEN+1]; /* text of error message */
		long	nonfatal = 1;	   /* Non-fatal key for c_errmsg     */

/* Compress i_specs so that there is no white space except a single space
   between numbers */
	compress( i_specs, line );

/* For each image specification (seperated by '+'s), find the image name,
   window, and bands.*/
	spec_begin = line;
	*images = 0;
	do
		{
		if ( *images > MAXIMG )
			{
			sprintf( err_text," %s %s %d %s.",
				"Error: Too many images were specified.",
				"  Only",MAXIMG,"images are allowed");
			c_errmsg(err_text,"getin-maximg",&nonfatal);
			return (E_FAIL);
			}

		pos = strchr( spec_begin, '+' );
		if (pos != (char *)0 )
			*pos = '\0';

		if (image_info(spec_begin,&list[*images]) != E_SUCC)
			return (E_FAIL);

		if (pos != (char *)0 ) spec_begin = pos + 1;
		(*images)++;
		}
	while ( pos > (char *)0 );
	return(E_SUCC);
	}

 FUNCTION static compress( filled, small )/* line changed */ 
		char	*filled,
			*small;
	{
		unsigned short	i,
				j;
	enum	states
		{
		digit, non_digit
		}	p_state;

/* Copy all characters except white space.  If a digit was just read, copy a
   space and then set back to a non-digit state. */
	p_state = non_digit;
	i = j = 0;
	while ( filled[i] != '\0' )
		{
/*
		if (( isspace(filled[i]) )  || (filled[i] == ','))
			{
			if ( p_state == digit )
				{
				small[j++] = ' ';
				i++;
				}
			else
				{
				i++;
				};
*/
		if ( isspace(filled[i]) )
			{
			if ( p_state == digit )
				{
				small[j++] = ' ';
				i++;
				}
			else
				{
				i++;
				}
			p_state = non_digit;
                        }
		else
		if ((filled[i] == ','))
			{
				{
				small[j++] = ' ';
				i++;
				}
			p_state = non_digit;
			}
		else if ( (j>0) && (filled[i]==':') && (small[j-1]==' ') )
			{
			small[j-1] = ':';
			i++;
			p_state = non_digit;
			}
		else if ( (j>0) && (filled[i]==')') && (small[j-1]==' ') )
			{
			small[j-1] = ')';
			i++;
			p_state = non_digit;
			}
		else
			{
			if ( isdigit(filled[i]) )	{ p_state = digit; }
				else		{ p_state = non_digit; };
			small[j++] = filled[i++];
			}
		};
	small[j] = '\0';
	return;
	}

 FUNCTION static image_info( spec, i_record )/* line changed */ 
		char	*spec;		/* single image specification*/
	struct	PRS_LST	*i_record;	/* record for name, window, & bands */
	{
		char	*left,		/* position of left parenthesis */
			*colon,		/* position of colon */
			*right,		/* position of right parenthesis */
			*semi,		/* position of ';'		*/
			*ptr,		/* temporary pointer */
			err_text[ERRLEN];
		unsigned short	wind;		/* window index and counter */
		long	nonfatal = 1;	   /* Non-fatal key for c_errmsg      */

/* Find position of (, :, and ). */
	left = strchr( spec, '(' );
	if (left != NULL)
	   {
	   colon = strchr( left, ':' );
	   semi = strchr( left, ';' );
	   }
	else
	   {
	   colon = NULL;
	   semi = NULL;
	   }

	right = strchr( spec, ')' );
	if ( (left==0) && (colon==0) && (right==0) && (semi == 0))
					/* no window, bands or projection 
					   specified */
		{
		strcpy( i_record->name, spec);
		for ( wind=0; wind<4; wind++ ) i_record->window[wind] = 0;
		i_record->bands[0] = 0;
		i_record->qflag[0] = i_record->qflag[1] = 0;
		i_record->proj[0] = 0;
		}
	else if (( left > (char *) 0 ) && (semi == (char *)0))
		{
		i_record->proj[0] = 0;
		*left = '\0';
		strcpy( i_record->name, spec );
		*left = '(';
		if ( (colon==0) && (left<right) )
					/*	IMAGE(WINDOW)	*/
			{
			*right = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   }

			*right = ')';
			i_record->bands[0] = 0;
			i_record->qflag[0] = 1;
			}
		else if ( (left<colon) && (colon+1==right) )
					/*	IMAGE(WINDOW:)	*/
			{
			*colon = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *colon = ':';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*colon = ':';
			i_record->bands[0] = 0;
			i_record->qflag[0] = 1;
			}
		else if ( (left+1==colon) && (colon<right) )
					/*	IMAGE(:BANDS)	*/
			{
			for ( wind=0; wind<4; wind++ )
				i_record->window[wind] = 0;
			*right = '\0';
			if (make_bands(colon+1,i_record->bands) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Band format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[1] = 1;
			}
		else if ( (left<colon) && (colon<right) )
					/*	IMAGE(WINDOW:BANDS)	*/
			{
			*colon = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *colon = ':';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",

				   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*colon = ':';
			*right = '\0';
			if (make_bands(colon+1,i_record->bands) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Band format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[0] = i_record->qflag[1] = 1;
			}
		else
					/* cannot parse window and bands */
			{
			sprintf( err_text,
				"Error: Specification %s cannot be parsed",
				spec );
			c_errmsg(err_text,"getin-invspec",&nonfatal);
			return(E_FAIL);
			};
		}
	else if (( left > (char *) 0 ) && (colon == (char *)0))
		{
		i_record->bands[0] = 0;
		*left = '\0';
		strcpy( i_record->name, spec );
		*left = '(';
		if ( (left<semi) && (semi+1==right) )
					/*	IMAGE(WINDOW;)	*/
			{
			*semi = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *semi = ';';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*semi = ';';
			i_record->proj[0] = 0;
			i_record->qflag[0] = 1;
			}
		else if ( (left+1==semi) && (semi<right) )
					/*	IMAGE(;PROJECTION	*/
			{
			for ( wind=0; wind<4; wind++ )
				i_record->window[wind] = 0;
			*right = '\0';
			if (make_proj(semi+1,i_record->proj) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Projection format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[1] = 1;
			}
		else if ( (left<semi) && (semi<right) )
					/*	IMAGE(WINDOW;PROJECTION	*/
			{
			*semi = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *semi = ';';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",

				   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*semi = ';';
			*right = '\0';
			if (make_proj(semi+1,i_record->proj) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Projection format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[0] = i_record->qflag[1] = 1;
			}
		else
					/* cannot parse window and bands */
			{
			sprintf( err_text,
				"Error: Specification %s cannot be parsed",
				spec );
			c_errmsg(err_text,"getin-invspec",&nonfatal);
			return(E_FAIL);
			};
		}
	else if (( left > (char *) 0 ) && (semi != (char *)0) && 
		 (colon != (char *) 0))
		{
		i_record->proj[0] = 0;
		*left = '\0';
		strcpy( i_record->name, spec );
		*left = '(';
		if ( (left<colon) && (colon+1==semi) && (semi+1==right) )
					/*	IMAGE(WINDOW:;)	*/
			{
			*colon = '\0';
			*semi = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *colon = ':';
			   *semi = ';';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*colon = ':';
			*semi = ';';
			i_record->bands[0] = 0;
			i_record->qflag[0] = 1;
			i_record->proj[0] = 0;
			}
		else if ( (left+1==colon) && (colon+1==semi) && (semi<right))
					/*	IMAGE(:;PROJECTION)	*/
			{
			for ( wind=0; wind<4; wind++ )
				i_record->window[wind] = 0;
			*right = '\0';
			if (make_proj(semi+1,i_record->proj) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Projection format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[1] = 1;
			i_record->bands[0] = 0;
			}
		else if ( (left+1==colon) && (colon<semi) && (semi+1==right))
					/*	IMAGE(:BANDS;)	*/
			{
			for ( wind=0; wind<4; wind++ )
				i_record->window[wind] = 0;
			*right = '\0';
			if (make_bands(colon+1,i_record->bands) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Band format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[1] = 1;
			i_record->proj[0] = 0;
			}
		else if ( (left+1==colon) && (colon<semi) && (semi<right))
					/*	IMAGE(:BANDS;PROJECTION)*/
			{
			for ( wind=0; wind<4; wind++ )
				i_record->window[wind] = 0;
			*right = '\0';
			if (make_bands(colon+1,i_record->bands) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Band format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };
			if (make_proj(semi+1,i_record->proj) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Projection format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[1] = 1;
			}
		else if ( (left<colon) && (colon<semi) && (semi+1==right) )
					/*	IMAGE(WINDOW:BANDS;)	*/
			{
			*colon = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *colon = ':';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",

				   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*colon = ':';
			*right = '\0';
			if (make_bands(colon+1,i_record->bands) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Band format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[0] = i_record->qflag[1] = 1;
			i_record->proj[0] = 0;
			}
		else if ( (left<colon) && (colon+1==semi) && (semi<right) )
					/*	IMAGE(WINDOW:;PROJECTION)*/
			{
			*colon = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *colon = ':';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",

				   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*colon = ':';
			*right = '\0';
			if (make_proj(semi+1,i_record->proj) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Band format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[0] = i_record->qflag[1] = 1;
			i_record->bands[0] = 0;
			}
		else if ( (left<colon) && (colon<semi) && (semi<right) )
					/*	IMAGE(WINDOW:BANDS;PROJECTION)*/
			{
			*colon = '\0';
			if (make_window(left+1,i_record->window) != E_SUCC)
			   {
			   *colon = ':';
			   sprintf( err_text,
			   "Error: Window format error was generated by %s",

				   spec );
			   c_errmsg(err_text,"getin-invwnd",&nonfatal);
			   return(E_FAIL);
			   };

			*colon = ':';
			*right = '\0';
			if (make_bands(colon+1,i_record->bands) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Band format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };
			if (make_proj(semi+1,i_record->proj) != E_SUCC)
			   {
			   *right = ')';
			   sprintf( err_text,
			   "Error: Projection format error was generated by %s",
			   spec );
			   c_errmsg(err_text,"getin-invbnds",&nonfatal);
			   return(E_FAIL);
			   };

			*right = ')';
			i_record->qflag[0] = i_record->qflag[1] = 1;
			}
		else
					/* cannot parse window and bands */
			{
			sprintf( err_text,
				"Error: Specification %s cannot be parsed",
				spec );
			c_errmsg(err_text,"getin-invspec",&nonfatal);
			return(E_FAIL);
			};
	        }
	else
					/* cannot parse, no left paren */
		{
		sprintf(err_text,"Error: No left parenthesis in {%s}.", spec );
		c_errmsg(err_text,"getin-lparen",&nonfatal);
		return(E_FAIL);
		};

	ptr = i_record->name + strlen(i_record->name) - 1;
  	for (; (*ptr == ' ') && (ptr > i_record->name); ptr--)
	   *ptr = '\0';
	return E_SUCC;
	}

 FUNCTION static make_window( numerals, window )/* line changed */ 
		char	*numerals;
		double	window[];
	{
		unsigned short	wind;		/* window counter and index */
		char	*n_begin,	/* first numeral in window component */
			*n_end;		/* the space following the numeral */
		long	nonfatal = 1;	   /* Non-fatal key for c_errmsg      */

	wind = 0;
	n_begin = numerals;
	do
		{
		n_end = strchr( n_begin, ' ' );
		if ( n_end == 0 )
			{
/*
			window[wind++] = atoi( n_begin );
*/
			sscanf(n_begin,"%lf",&window[wind++]);
			}
		else
			{
			*n_end = '\0';
/*
			window[wind++] = atoi( n_begin );
*/
			sscanf(n_begin,"%lf",&window[wind++]);
			*n_end = ' ';
			n_begin = n_end + 1;
			};
		}

	while ( (wind<4) && (n_end!=0) );
	if ( (wind==4) && (n_end!=0) )
		{
		c_errmsg("Error: There are too many window components",
			 "getin-invwnd",&nonfatal);
		return (E_FAIL);
		}
	else if ( (wind<4) && (n_end==0) )
		{
		c_errmsg("Error: There are not enough window components",
			 "getin-invwnd",&nonfatal);
		return (E_FAIL);
		};
	return E_SUCC;
	}

 FUNCTION static make_bands( numerals, bands )/* line changed */ 
		char	*numerals;
		long	bands[];
	{
		unsigned short	band;		/* band counter and index */
		char	*n_begin,	/* first numeral in band component */
			*n_end,		/* the space following the numeral */
			*n_dash;	/* postion of the '-' for band range */
		long	nonfatal = 1;	   /* Non-fatal key for c_errmsg     */
		long	end,i,start;	/* loop variables for band range     */

	band = 0;
	n_begin = numerals;
	do
		{
		n_end = strchr( n_begin, ' ' );
		n_dash = strchr( n_begin, '-');
		if ( n_end == n_dash )
			{
			bands[band++] = atoi( n_begin );
			}
		else if (((n_end < n_dash) && (n_end !=0)) || n_dash==0 )
		    {
		    *n_end = '\0';
		    bands[band++] = atoi( n_begin );
		    *n_end = ' ';
		    n_begin = n_end + 1;
		    }
		else
		    {
		    start = atoi( n_begin );
		    n_begin = n_dash + 1;
		    end = atoi( n_begin );
		    for (i = start; i <= end; i++)
			bands[band++] = i;
		    n_begin = n_end + 1;
		    }
		}
	while ( (band<=MAXBND) && (n_end!=0) );
	if ( band>MAXBND ) 
		{
		c_errmsg("Error: There are too many band components",
			 "getin-invbnds",&nonfatal);
		bands[band] = 0;
		return (E_FAIL);
		};

	bands[band] = 0;
	return(E_SUCC);
	}
 FUNCTION static make_proj( numerals, proj )/* line changed */ 
		char	*numerals;
		long	proj[];
	{
		unsigned short	band;		/* band counter and index */
		char	*n_begin,	/* first numeral in band component */
			*n_end,		/* the space following the numeral */
			*n_dash;	/* postion of the '-' for band range */
		long	nonfatal = 1;	   /* Non-fatal key for c_errmsg     */

	band = 0;
	n_begin = numerals;
	do
		{
		n_end = strchr( n_begin, ' ' );
		n_dash = strchr( n_begin, '-');
		if ( n_end == n_dash )
			{
                        if ((strcmp(n_begin,"LS") == 0) || 
					(strcmp(n_begin,"ls") == 0))
			   {
                           proj[0] = 0;
			   band++;
			   }
			else
                        if ((strcmp(n_begin,"DEG") == 0) || 
					(strcmp(n_begin,"deg") == 0))
			   {
                           proj[0] = 4;
			   band++;
			   }
			else
                        if ((strcmp(n_begin,"DMS") == 0) || 
					(strcmp(n_begin,"dms") == 0))
			   {
                           proj[0] = 5;
			   band++;
			   }
			else
                        if ((strcmp(n_begin,"PRO") == 0) || 
					(strcmp(n_begin,"pro") == 0))
			   {
                           proj[0] = 3;
			   band++;
			   }
                        else
                           proj[0] = 0;
			}
		}
	while ( (band == 0) && (n_end!=0) );
	if ( band > 1 ) 
		{
		c_errmsg("Error: There are too many projection components",
			 "getin-invbnds",&nonfatal);
		proj[band] = 0;
		return (E_FAIL);
		}

	proj[band] = 0;

	return(E_SUCC);
	}
