
/* image.c:  functions to consolidate LAS image processing calls.
   image.c, template.pdf, and associated source was written and revised by 
   David Zokaites 

   Dec 1992  Developed imageio proc:  functions and data structures for 
	     sequential IO.
   Feb 1993  Enabled random IO by writing openrin(), openrout(), rread(), 
	     rwrite(), rimage();  created global vars for easier error 
	     messages;  wrote minmax(). 
   May 1993  Enabled block IO by writing bread(), bwrite(), bimage();  wrote 
	     make_ddr(). 
   Jul 1993  Renamed proc from imageio to template;  eliminated functions 
	     getin2() and getout2();  wrote help for template. 
   Aug 1993  Allowed for data conversion upon read and write;  wrote 
	     convert[srb](). 
   May 1994  Wrote openbin(), openbout(), minmax[srb](), parse_file(); 
             expanded help in template.pdf;  removed make.com;  added parms 
	     iomode, readtype, procmeth, writconv. 
   Jun 1995  Modified openrin() for COPY2 to call c_getin2_changed() instead of 
	     c_getin() 
   Mar 1996  Changed end_las() so that the images would only be closed once. */






#include "copy2.h"    /* proc header file */ 
#include "pixman.h"   /* for TOOBIG, TOOSMALL macros */ 
#include <unistd.h>   /* for call to access() */
#include "util.h"





/* FUNCTION: initialize TAE and LAS processing.  Must be called before all
   other functions in this file */ 
void begin_las (proc) 
   char *proc;             /* input:  name of las proc */ 
{
/* declare vars */ 
   long length,            /* length of a char string */ 
        std_out;           /* standard output */ 
/* initialize las */ 
   c_inlas (&pblock, &std_out); 
/* set global vars for proc name, some are uppercase, some are lowercase */ 
   strcpy  (procname, proc); 
   length = (long)strlen (procname);   
   c_low2up (procname, &length);  
   length = (long)strlen (proc);       
   c_up2low (proc, &length); 
   sprintf (procfatal, "%s-fatal", proc); 
   sprintf (procwarn,  "%s-warn",  proc); 
}





/* FUNCTION close input image files and free buffers */ 
void close_inimage (ini)
   struct lasinimage *ini;  /* input:  close these files */ 
{
/* declare vars */
   long imagenum;  /* loop index, image number */ 
/* close image files, close individual file descriptors */ 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_eclose (ini->fdarray+imagenum))
	 c_errmsg ("Error closing image files.", procwarn, &go_on); 
/* free buffers */ 
   free (ini->buffer); 
}





/* FUNCTION close output image files and free buffers */ 
void close_outimage (outi)
   struct lasoutimage *outi;  /* input:  close these files */ 
{
/* close image files, close individual file descriptors */ 
   if (c_eclose (outi->fdarray))
      c_errmsg ("Error closing image files.", procwarn, &go_on); 
/* free buffers */ 
   free (outi->buffer); 
}





/* FUNCTION: cleanup from an error and delete output image files */ 
void errclean (message) 
   char *message;  /* input:  message to write */ 
   /* additional inputs are inimage and outimage global variables */ 
{
/* declare var, write message to user */ 
   long one=1;     /* number of output file names in outimage.hostname */ 
   c_errmsg (message, procfatal, &go_on); 
/* close image files and free buffers */ 
   close_inimage  (&inimage); 
   close_outimage (&outimage); 
/* cleanup disk files */ 
   c_clean (&pblock, procname, outimage.hostname, &one); 
}





/* FUNCTION: open an input image for random access and allocate data buffers, 
   return TRUE if an error was returned from c_getin(), else return FALSE */ 
long openrin (imagename, dtypeproc, ini)
   char *imagename;         /* input:  image to open */ 
   long dtypeproc;          /* input:  convert image data to this type upon
			       reading.  pass a 0 if not converting */ 
   struct lasinimage *ini;  /* output: opened input image */ 
{
/* declare vars */ 
   long accessm=IREAD,      /* image access mode, arg to c_eopenr() */ 
        bandnum,            /* loop index, band number */ 
        imagenum,           /* loop index, image number */ 
        maximg=MAXIMG,      /* max number of images allowed */ 
	one=1,              /* value of one */ 
        qflag[MAXIMG][2],   /* flag, do image components have a band or
			       window specified */ 
	samewindow,         /* flag, do image components have the same 
			       window specified? */ 
        tband,              /* loop index, total band number */ 
        zero=0;             /* value of zero */ 
   struct DDR ddr;          /* ddr for input image */ 
/* parse input image spec */ 
   /* call c_getin2_changed() */
      if (c_getin2_changed (imagename, ini->taename, ini->hostname,
      &ini->ncomponents, ini->window, ini->bands, ini->nbands,
      &ini->totalbands, &maximg, &samewindow, qflag))
      {
	 sprintf (message, "Error getting input image for parm IN."); 
	 c_errmsg (message, procwarn, &go_on); 
	 return (TRUE); 
      }
   /* error check, different window sizes causes logic in process.c to fail */ 
      if (!samewindow)
      {
	 sprintf (message, 
	 "Input images specified by parm IN do not have the same window size.");
	 c_errmsg (message, procfatal, &fatal); 
      }
   /* set nl, ns */ 
      ini->nl = ini->window[0][NL]; 
      ini->ns = ini->window[0][NS]; 
/* update last used fields of input images */ 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_lused (ini->hostname[imagenum] ))
	 c_errmsg ("Error updating last used fields of input image ddr.", 
	 procwarn, &go_on); 
/* open image file and set vars */ 
   ini->dtypeproc=dtypeproc; 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_eopenr (ini->fdarray+imagenum, ini->hostname[imagenum], 
      ini->nbands+imagenum, 
      &ini->window[imagenum][SL], &ini->window[imagenum][SS], 
      &ini->window[imagenum][NL], &ini->window[imagenum][NS], 
      &accessm, &ini->dtypeproc, &zero, &one)) 
      {
	 sprintf (message, "Error opening image file for image %s",
	 ini->taename[imagenum]); 
	 c_errmsg (message, procfatal, &fatal); 
      }
/* set dsizeproc and dtypein */ 
   switch (ini->dtypeproc)
   {
      case EBYTE:  ini->dsizeproc=1;  break; 
      case EWORD:  ini->dsizeproc=2;  break; 
      case ELONG:  ini->dsizeproc=4;  break; 
      case EREAL:  ini->dsizeproc=4;  break; 
   }
   if (c_getddr (ini->hostname[0], &ddr) )
      c_errmsg ("Error retrieving ddr for input image.", procfatal, &fatal); 
   ini->dtypein = ddr.dtype; 
/* build luts for later access by rread() */ 
   for (tband=imagenum=0; imagenum<ini->ncomponents; imagenum++)
      for (bandnum=0; bandnum<ini->nbands[imagenum]; bandnum++, tband++)
      {
	 /* Due to subbanding, the parm eband to c_eread() is relative to the 
	 bands in the input -- not output -- image files.  This is accommodated 
	 by cblut.  */ 
	 ini->cblut[tband] = ini->bands[imagenum][bandnum];
	 ini->fdlut[tband] = imagenum; 
      } 
/* allocate memory for image buffer */ 
   if (( ini->buffer = (unsigned char *) malloc (ini->ns * ini->dsizeproc) ) 
   == NULL)
      c_errmsg ("Unable to allocate memory for input image buffer.", 
      procfatal, &fatal); 
/* return */ 
   return (FALSE);  /* no problems in c_getin() */
}





/* FUNCTION: open an output image for random access and allocate data buffers,
   must be called after openrin(), return TRUE if output image pre-existed,
   else return FALSE */ 
long openrout (writconv, ini, outi)
   long writconv;             /* input:  flag. if TRUE, data will be converted 
                                 to ODTYPE upon writing to disk. */ 
   struct lasinimage *ini;    /* input:  input image, with nl, ns, 
				 nbands to be used for outi */ 
   struct lasoutimage *outi;  /* output: opened output image */
{
/* declare vars */ 
   char odtype[10];           /* tae parm odtype, output image data type */ 
   long accessm=IWRITE,       /* image access mode, arg to c_eopenr() */ 
        nocheck=IUPDATE,      /* file access mode, arg to c_getnam() */
	preexist=FALSE,       /* output flag, did the output image preexist */ 
        one =1,               /* value of one */ 
	opt[2],               /* optional parm to c_eopenr(), used for data 
				 type conversion */ 
        zero=0;               /* value of 0 */ 
   struct VARIABLE *tparm;    /* points to a tae parm in pblock */ 
/* get output image name, 
   some of this logic is duplicated in get_par(),
   don't call c_getout() as it gives an error if the image already exists. */
   /* determine host name */ 
      if (c_getnam (&pblock, "out", &zero, outi->hostname, "img", &nocheck))
	 c_errmsg ("Error getting output image OUT.", procfatal, &fatal); 
   /* determine tae name */
      strcpy(outi->taename, outi->hostname);
   /* delete image if it exists */ 
      if (! access(outi->hostname,0) )
      {
	 c_delfil (outi->hostname);
	 sprintf (message, "Output image %s pre-existed.", outi->taename);
	 c_errmsg (message, "informational", &go_on); 
	 preexist = TRUE; 
      }
/* set data type and size 
   /* set dtypeout after getting tae parm odtype */ 
      tparm = p_find (&pblock, "odtype");    
      strcpy (odtype, SVAL(*tparm, 0)); 
      if      (!strcmp (odtype, "SAME")) outi->dtypeout = ini->dtypein; 
      else if (!strcmp (odtype, "BYTE")) outi->dtypeout = EBYTE; 
      else if (!strcmp (odtype, "I*2"))  outi->dtypeout = EWORD; 
      else if (!strcmp (odtype, "I*4"))  outi->dtypeout = ELONG; 
      else if (!strcmp (odtype, "R*4"))  outi->dtypeout = EREAL; 
      else c_errmsg ("Parm ODTYPE has an illegal value.", procfatal, &fatal); 
   /* dtypeproc */ 
      if (writconv)
	 outi->dtypeproc = ini->dtypeproc; 
      else 
	 outi->dtypeproc = outi->dtypeout; 
   /* dsizeproc */ 
      switch (outi->dtypeproc)
      {
	 case EBYTE:  outi->dsizeproc=1;  break; 
	 case EWORD:  outi->dsizeproc=2;  break; 
	 case ELONG:  outi->dsizeproc=4;  break; 
	 case EREAL:  outi->dsizeproc=4;  break; 
      }
/* set vars in data structure */ 
   outi->nl = ini->nl; 
   outi->ns = ini->ns; 
   outi->totalbands = ini->totalbands; 
   outi->truncate = E_SUCC; /* output data was tentatively not truncated */ 
/* open image file */ 
   if (writconv)
   {
      opt[0] = outi->dtypeout; 
      opt[1] = 0; 
   }
   else 
      opt[0] = 0; 
   if (c_eopenr (outi->fdarray, outi->hostname, &outi->totalbands, &one, &one, 
   &outi->nl, &outi->ns, &accessm, &outi->dtypeproc, opt, &one))
   {
      sprintf (message, "Error opening output image file for image %s",
      outi->taename); 
      errclean (message); 
   }
/* allocate memory for image buffer */ 
   if ((outi->buffer = (unsigned char *) 
   malloc (outi->ns * outi->dsizeproc) ) == NULL)
      errclean ("Unable to allocate memory for output image buffer."); 
/* initialize min max arrays */ 
   tparm = p_find (&pblock, "$minmax");    
   if (*SVAL(*tparm,0) == 'Y')  /* Y for Yes */
      outi->minmax = TRUE;
   else 
      outi->minmax = FALSE;
   if (outi->minmax && c_inmnmx (&outi->dtypeproc, &outi->totalbands, 
   outi->minval, outi->maxval))
      errclean ("Error initializing min/max arrays."); 
/* return */ 
   return (preexist); 
}





/* FUNCTION:  read data from input image file, using random IO */ 
void rread (curline, curband, ini)
   long curline;           /* input: starting line of block of data to read, 
			      0 relative */ 
   long curband;           /* input: band to read, 0 relative */ 
   struct lasinimage *ini; /* input and output: data structure of input image 
			      to read */
{
/* declare vars */ 
   long imagenum,          /* image number, index to fdarray[] */ 
	one=1;             /* n image lines to read at a time */ 
/* first determine which image file to access then translate from curband as 
   input to curband relative to input image file */ 
   imagenum = ini->fdlut[curband]; 
   curband  = ini->cblut[curband]; 
/* convert from 0 relative to 1 relative */ 
   curline++; 
/* read data from disk image */ 
   if (c_eread (ini->fdarray+imagenum, &curband, &curline, ini->buffer, 
   &one) != ini->ns)
      errclean ("Error reading input image data.");
}





/* FUNCTION:  write data to disk image, using random IO */ 
void rwrite (curline, curband, outi)
   long curline;             /* input: starting line of block of data to write, 
			        0 relative */ 
   long curband;             /* input: band to write, 0 relative */ 
   struct lasoutimage *outi; /* input: data structure of image to write */
{
/* declare var */ 
   long one=1;               /* n lines to write at a time */ 
/* convert from 0 relative to 1 relative */ 
   curline++; 
   curband++; 
/* write data to disk */ 
   if (c_ewrite (outi->fdarray, &curband, &curline, outi->buffer, &one) 
   != outi->ns)
      errclean ("Error writing output image data.");
}





/* FUNCTION:  convert image data type and correct for roundoff error;
   this function is for random processing */ 
void convertr (ini, outi) 
   struct lasinimage *ini;   /* input:  data to convert */ 
   struct lasoutimage *outi; /* output:  converted data */ 
{
/* declare variables */ 
   long sample;             /* for loop index */ 
   unsigned char *inbuf;    /* pixel location in input buffer */
/* correct for round off error */ 
   if (ini->dtypeproc == EREAL && outi->dtypeout != EREAL) 
      for ( inbuf = ini->buffer,
      sample=0; sample<ini->ns; sample++, inbuf+=4) 
      {
	 if ( *(float *) inbuf >= 0)
	    *(float *) inbuf += (float) .4999999;
	 else 
	    *(float *) inbuf -= (float) .4999999;
      }
/* convert data type */ 
   outi->truncate = outi->truncate | 
   c_pxconv (&ini->dtypeproc, &outi->dtypeproc, ini->buffer, outi->buffer, 
   &ini->ns);
}





/* FUNCTION:  optionally update minmax for a line of image data for random
   processing */ 	
void minmaxr (band, outi)
   long band;                /* input:  image buffer holds data for this band */
   struct lasoutimage *outi; /* input and output:  structure of image data to 
				update min/max for */ 
{
/* return if min/max does not need updated */ 
   if (!outi->minmax)
      return; 
/* update min max */ 
   if (c_pxmnmx (outi->buffer, &outi->ns, outi->minval+band, 
   outi->maxval+band, &outi->dtypeproc) )
      errclean ("Error updating min max arrays."); 
}





/* FUNCTION close image files, create ddr, and end the proc, last function in 
   this file to be called */ 
void end_las (ini, outi)
   struct lasinimage *ini;    /* input:  image input to the proc */ 
   struct lasoutimage *outi;  /* input:  image output from the proc */ 
{
/* vars for ddr */ 
   long cflag=NOCOMB,   /* combine flag for c_upddr() */ 
        one=1;          /* value of one */ 
   double oned=1.0;     /* one as a double */ 
/* check for data type truncation.  check will only find problems when 
   c_pxconv() is used for data type conversion. */ 
   if (outi->truncate==TOOSMALL || outi->truncate== (TOOBIG | TOOSMALL))
      c_errmsg (
      "Data values beyond lower limit of output data type were truncated.",
      procwarn, &go_on); 
   if (outi->truncate==TOOBIG   || outi->truncate== (TOOBIG | TOOSMALL))
      c_errmsg (
      "Data values beyond upper limit of output data type were truncated.",
      procwarn, &go_on); 
/* create ddr for output image.  this call takes care of last used fields */ 
   if (c_upddr (&pblock, ini->hostname, outi->hostname, &ini->ncomponents, 
   ini->window, ini->bands, ini->nbands, &oned, &oned, &outi->totalbands, 
   &cflag))
   {
      c_errmsg ("Error creating ddr for output image.", procfatal, &go_on); 
      c_clean (&pblock, procname, outi->hostname, &one); 
   }
/* update ddr with min max */ 
   if (outi->minmax && c_svmnmx (outi->hostname, &outi->totalbands, 
   outi->minval, outi->maxval, "YES"))
      c_errmsg ("Error updating min/max values in ddr.", procwarn, &go_on); 
/* create history file */ 
   if (c_hstout (&pblock, outi->hostname, ini->hostname, &ini->ncomponents, ""))
   {
      c_errmsg ("Error creating history file for output image.", procfatal,
      &go_on); 
      c_clean (&pblock, procname, outi->hostname, &one); 
   }
/* end program */ 
   c_complt(procname, outi->taename, &one);
}






