
/*******************************************************************************
NAME:     datelog.c 
PURPOSE:  program main()
	  DATELOG maintains scene and processing logs for AVHRR compositing
	  projects.


PROGRAM HISTORY
-----------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  --------------------------------------------------
Apr 1990  David Zokaites   Wrote DATEBAND procedure to associate SCENEID's
			   with INDEX's for the WHOLUS AVHRR project.  This
			   routine had 2 input parms (SCENEID and BEGINPER),
			   one output parm (INDEX);  maintained one permanent
			   log;  and called RESTORE-VARIABLE, SAVE-VARIABLE,
			   and PUTPARM.
Jun 1990  David Zokaites   Wrote DATELOG process for WHOLUS AVHRR project to
			   log scene information and return an index.  The
			   input parms were SCENEID, DATES, NEWDATES, TAPEID,
			   UPLEFT, LORITE, SL, SS, NL, NS, BEGINPER, ENDWK1,
			   MAKEPERM, PERMLOG, NAMEBASE, and DATEFILE.  The only
			   output parm was INDEX.  In addition to one permanent
			   log, two short-term logs were created for each
			   compositing period.
Nov 1990  David Zokaites   Deleted references to specific projects;
			   expanded help, changed SCENEID from (string 15)
			   to (string 16).
Mar 1991  David Zokaites   Deleted parms UPLEFT, LORITE, ENDWK1;  renamed
			   NEWDATES parm CURDATES;  added parm GROUPPER to
			   optionally group two compositing periods.  Used
			   function prototypes;  broke main function into
			   several smaller functions;  simplified error
			   traps.
Jun 1991  David Zokaites   Changed strupr() to strlower() to convert strings
			   to lower case before comparing them.  Upper case
			   file names presented problems on Unix systems.  This
			   version was released to $LASCONTRIB but updated
			   versions were maintained in $DCONTRIB (ddps contrib)
			   then $LASDITTY.
Jan 1994  David Zokaites   Rewrote application to provide 4 subcommands
			   instead of none;  created subcommand -MAKEPERM to
			   make the permlog;  -COMPOSIT to log composite
			   tapes;  -GETINFO to return more info to calling
			   application.
Feb 1994  David Zokaites   Added parms PROT, PSTATUS, and INCOMP.
Mar 1994  David Zokaites   Added subcommand -MAKESHORT.  It is now a fatal
			   error if BEGINPER=YES and a short-term log
			   exists;  also a fatal error if BEGINPER=NO and a
			   short-term log does not exist.
Apr 1994  David Zokaites   Added subcommand -APLOGS to append logs when
			   formerly done if GROUPPER=YES and ENDPER=YES for
			   -SCENE subcommand.  Changed fprintf %2d to %02d
			   for composite band number.
Jun 1994  Ron Kanengieter  Added parm MRLCLOG and function write_mrlc();
			   wrote system time to log files.
Aug 1994  David Zokaites   Added parm CBANDS.
Sep 1994  David Zokaites   Fixed processing for parm DBNAME (formerly MRLCLOG),
			   added parm DBUID, added function system_date(), 
			   fixed some bugs in aplogs.c.
Oct 1994  David Zokaites   No longer a fatal error if lockfile exists after
			   several checks.
Mar 1995  David Zokaites   A major revision:  wrote -SCENEOUT subcommand;
			   renamed -GETINFO subcommand -GETDATE;  added parm
			   SCALOPT to -GETDATE and -SCENE subcommands;  allowed
			   parms DBNAME, DBUID, and PERMLOG to have null
			   value;  deleted parm PERNUM and other parms;  added
			   some parms;  converted short-term and database logs
			   to the same format as the permlog;  changed format of
			   datefile;  name of database logs in now based on
			   system time instead of process id;  zillions of
			   small changes.
Sep 1995  David Zokaites   Renamed APLOGS subcommand COMLOGS;  deleted parm
			   LOCKFILE;  for DATELOG-SCENEIN, added parm DATEIND,
			   deleted parms SCALOPT, GROUPPER, and INCOMP;
			   updated user guide;  called c_lasmsg() instead of
			   c_errmsg();  updated Makefile format;  changed
			   indentation and error messages per LAS conventions;
			   kept track of dynamic memory;  released application
			   to $ADAPSCONTRIB 
Mar 1997  David Zokaites   Added parameter WRITSHOR to DATELOG-COMPOSIT;
			   changed variable inline to linein for gcc compiler.
Apr 1997  David Zokaites   Converted from C to C++;  changed write_output() to 
			   write filenames only once;  for any DATELOG 
			   execution, all records for all logs have exactly the
			   same system date;  increased MAXMEMORY;  implemented
			   better file locking in lock.C;  added more error
			   messages to the user guide.  
*******************************************************************************/


#include <string.h>    // for calling string functions  
#define GLOBALVARS     // defined only here, flags datelog.h  
#include "datelog.h"   // application header file  





/* Except for host_tae(), file names are referenced only by host name, 
   never by TAE name */ 

FUNCTION int main ()  // int required by C++ compiler 
{
// declare variables  
   char  nofiles[1][CMLEN],  // array of output files, parm to c_complt()  
        *subcmd;             // subcommand to execute  
   long std_out,             // standard output  
        zero=0;              // value of zero  
   struct VARIABLE *tparm;   // points to a TAE variable  

// initialize las and TAE  
   c_inlas (&pblock, &std_out); 

// initialize variables 
   outfiles.newcount = outfiles.upcount = 0;  // number of output files  
   init_count();                              // count of dynamic memory 

// execute appropriate subcommand  
   tparm = p_find (&pblock, "_subcmd");
   subcmd = SVAL(*tparm,0); 

   if      (!strcmp (subcmd, "COMLOGS"  ))  comlogs(); 
   else if (!strcmp (subcmd, "COMPOSIT" ))  composit(); 
   else if (!strcmp (subcmd, "GETDATE"  ))  get_date(); 
   else if (!strcmp (subcmd, "MAKEPERM" ))  makeperm(); 
   else if (!strcmp (subcmd, "MAKESHORT"))  makeshort(); 
   else if (!strcmp (subcmd, "SCENEIN"  ))  scenein(); 
   else if (!strcmp (subcmd, "SCENEOUT" ))  sceneout(); 
   else 
      errclean ("datelog-fatal", "Switch error in main()"); 

// end program, write_output() lists files which were updated or created 
   write_output (outfiles.upcount,  outfiles.upfiles,  "updated file"); 
   write_output (outfiles.newcount, outfiles.newfiles, "created file"); 
   free_memory ();  // free dynamically allocated memory  
   c_complt ("DATELOG", nofiles, &zero); 
   return (0);  // prevent compiler warning 
}







#ifdef ANOTE // this ifdef'ed function call will pop up from    % lasmsg  
   c_lasmsg (ReadThisNote, 
   The unix application lasmsg can not be used directly on this application
   because of the function errclean.  Run tests/lasmsg2 instead.);
#endif 

