
/*******************************************************************************
NAME:     functions.c 
PURPOSE:  functions called by more than one subcommand 
*******************************************************************************/

#include <ctype.h>     // for calling tolower()  
#include <stdlib.h>    // for calling getenv()  
#include <string.h>    // for calling string functions  
#include <sys/stat.h>  // for calling chmod() 
#include <sys/types.h> // for calling time()  
#include <time.h>      // for calling time(), localtime() 
#include <unistd.h>    // for calling access(), sleep()  
#include "datelog.h"   // application header file  

/* max number of memory locations to free, must be at least 5 * count of 
   dateind parm for datelog-sceneout.  set to 20* for good luck */ 
#define MAXMEMORY 200 

// data structure to keep track of dynamically allocated memory  
   struct 
      {
      long count;                // number of pointers to free 
      char *address[MAXMEMORY];  // pointers to free 
      }
   tofree;  





/*******************************************************************************
NAME:     init_count()
PURPOSE:  initialize count of dynamic memory.  struct tofree is only known in
	  functions.c, hence the need for this function 
*******************************************************************************/
FUNCTION void init_count (void) 
{
   tofree.count = 0;
}






/*******************************************************************************
NAME:     integer_parm()
PURPOSE:  return values of TAE integer parm with multiple values 
*******************************************************************************/

FUNCTION void integer_parm (
   char *parmname,         // input:  name of parameter  
   struct iparm *parmval)  // output: parm values  
{

// declare variables  
   long pcount;            // loop index, parameter count  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

// find count of parm  
   tparm = p_find (&pblock, parmname);
   parmval->count = tparm->v_count;

// set parm values  
   for (pcount=0;  pcount<parmval->count;  pcount++)
      parmval->value[pcount] = IVAL(*tparm,pcount);
}






/*******************************************************************************
NAME:     name_parm()
PURPOSE:  return value of nullable TAE string parm used for a file name  
*******************************************************************************/

FUNCTION void name_parm (
   char *parmname,         // input:  name of parameter  
   long access,            // input:  file access mode, parm to c_getnam()  
   struct fparm *parmval)  // output: parm values  
{

// declare variables  
   long zero = 0;          // value of 0  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

// find count of parm  
   tparm = p_find (&pblock, parmname);
   parmval->count = tparm->v_count;
   if (!parmval->count)
      return; 

// get parm value  
   if (c_getnam (&pblock, parmname, &zero, parmval->value, "", &access)!=E_SUCC)
      errclean ("datelog-fatal", "Fatal error encountered"); 
}







/*******************************************************************************
NAME:     string_parm()
PURPOSE:  return values of TAE string parm with multiple values  
*******************************************************************************/

FUNCTION void string_parm (
   char *parmname,         // input:  name of parameter  
   struct sparm *parmval)  // output: parm values  
{

// declare variables  
   long pcount;            // loop index, parameter count  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

// find count of parm  
   tparm = p_find (&pblock, parmname);
   parmval->count = tparm->v_count;

// set parm values  
   for (pcount=0;  pcount<parmval->count;  pcount++)
      parmval->value[pcount] = SVAL(*tparm,pcount);
}







/*******************************************************************************
NAME:     check_parm()
PURPOSE:  check TAE parm for white space and zero length.  abort upon error  
*******************************************************************************/

FUNCTION void check_parm (
   const char *parmname,   /* name of the parameter to check, used for error
                              messages */ 
   const char *value)      // parameter value to check  
{

// check for white space  
   if (strchr(value,' ') != NULL || strchr(value,'\t') != NULL)
      errclean ("datelog-white", 
	       "Value '%s' for parameter %s has white space character(s)", 
	       value, parmname);

// check for zero length  
   if (*value == '\0') 
      errclean ("datelog-0length", 
	       "A value for parameter %s has zero characters", parmname);
}







/*******************************************************************************
NAME:     get_data()
PURPOSE:  read datefile and return pernum and oldname
*******************************************************************************/

FUNCTION void get_data (
         char *datefile,  // host file containing composite period names 
         char *pername,   // current composite period name  
         char *oldname,   // output:  name of previous composite period 
         long *pernum)    // output:  current composite period number  
{

// declare variables  
   char  linein[LINELEN],    // one line in input file  
         validd[2][LINELEN]; // two composite period names found in datefile 
   FILE *filep;		    // points to datefile  
   long  found=FALSE,       // flag, was pername found in datefile  
	 header,            // loop index, header lines to skip  
         vindex; 	    // index to validd, alternates between 1 and 0  

// open file  
   if ((filep = fopen(datefile,"r")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for reading", 
	       host_tae(datefile));

// skip table header  
   for (header=0;  header<DATEHEADER;  header++)
      if (fgets (linein, LINELEN, filep) == NULL)
	 errclean ("datelog-end", "End of file while reading DATEFILE %s", 
		  host_tae (datefile)); 

// find pername in datefile  
   for (*pernum=FIRSTPERIOD, vindex=*pernum%2;  
   fgets (linein, LINELEN, filep) != NULL;)
      {
      // get composite period name from linein, skip over blank lines in file 
	 if (sscanf (linein, "%s", validd[vindex]) != 1)
	    continue; 

      // check string length  
	 if (strlen (validd[vindex]) > PNAME -1)
	    errclean ("datelog-plength",
	             "%s %s in DATEFILE %s has more than %d characters",
		     "Composite period name", validd[vindex], 
		     host_tae(datefile), PNAME-1); 

      // save composite period name  
	 strlower (validd[vindex]); 

      // see if composite period name matches pername   
	 if (!strcmp(validd[vindex], pername))
	    {
	    found = TRUE;
	    break; 
	    }

      // increment loop indices  
	 (*pernum)++;
	 vindex=*pernum%2;  // vindex = 1, 0, 1, 0, ...  
      } 
   fclose (filep); 

// pername was not found, abort 
   if (!found)
      errclean ("datelog-nopername", "PERNAME %s was not found in DATEFILE %s", 
	       pername, host_tae(datefile));

// set oldname  
   if (*pernum == FIRSTPERIOD) 
      *oldname = '\0';  // there is no previous composite period  
   else 
      strcpy (oldname, validd[!vindex]);   // !vindex = old value of vindex 
}








/*******************************************************************************
NAME:     get_dateind()
PURPOSE:  read curlog and return dateind, date index for this scene.  This 
	  function is similar to get_scene() in sceneout.c 
*******************************************************************************/

FUNCTION void get_dateind (
   const char *scalopt,   // input:  scale option used to process avhrr data  
   const char *sceneid,   // input:  current sceneid 
   const char *groupper,  // input:  option to group 2 periods 
         char *curlog,    // input:  log file to check for old sceneid 
   const long  pernum,    // input:  current composite period 
         long *dateind)   // output: date index 
{

// declare variables  
   char  linein[LINELEN],   // one line in input file  
	 oldscene[LINELEN]; // old sceneid found in file  
   FILE *filep;             // points to curlog  
   long  anyscenes,         // flag, were any scenes found in this log  
         header,            // loop index, header lines to skip  
         maxindex,          // max dateind found in curlog  
         oldindex;          // old dateind found in curlog  

// open curlog and skip header  
   if (access(curlog,0) )
      errclean ("datelog-nolog", "Log file %s does not exist", 
	       host_tae(curlog));

   if ((filep = fopen(curlog,"r")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for reading", 
	       host_tae(curlog));
      
   for (header=0;  header<LOGHEADER;  header++)
      if (fgets (linein, LINELEN, filep) == NULL)
	 errclean ("datelog-fileend", 
		  "End of file while reading short-term log %s", 
		  host_tae(curlog)); 

// compare sceneid with old sceneids in curlog  
   for (anyscenes=FALSE, maxindex=0;   fgets(linein, LINELEN, filep) != NULL;)
      {
      // log files include two types of records.  skip inappropriate records  
	 if (sscanf (linein, " %*s %*s %d %s", &oldindex, oldscene) != 2)
	    continue; 

      // set flag, curlog contains some scenes  
	 if (oldindex != NOTINCOMP) 
	    anyscenes=TRUE;        

      // make comparison case in-sensitive  
	 strlower (oldscene);  

      // use this scene's previous date index as long as it is != NOTINCOMP  
	 if (!strcmp(sceneid,oldscene) && oldindex != NOTINCOMP) 
	    {
	    *dateind = oldindex;  
	    fclose (filep); 
	    return; 
	    }

      // find max date index in curlog  
	 if (maxindex < oldindex) 
	    maxindex = oldindex; 
      }
   fclose (filep); 

// new scene did not match any old scene, calculate a new index  
   if (anyscenes) 
      *dateind = maxindex + 1; 
   else 
      {
      // At the beginning of a period, dateind=1,101, or  10,110  
      if (*groupper=='Y' && !(pernum%2) )   // Y for yes  
	 {
	 // user is grouping periods, this is an odd numbered period  
	 if (*scalopt == 'G') // G for global 
	    *dateind = 110; 
	 else 
	    *dateind = 101;
	 }
      else
	 {
	 if (*scalopt == 'G') // G for global 
	    *dateind = 10; 
	 else 
	    *dateind = 1;  
	 }
      } 
}







/*******************************************************************************
NAME:     get_log_name() 
PURPOSE:  return name of short-term log  
*******************************************************************************/

FUNCTION void get_log_name (
         char *namebase,  /* input:  base of names for short-term log.  the 
			     trailing '.' will be deleted if it exists */ 
   const char *pername,   // input:  period name to append to namebase  
         char *logname)   // output: name of short-term log  
{

// eliminate trailing '.' on namebase, '.' was returned from c_getnam() 
   if ( namebase [strlen(namebase)-1 ] == '.')
        namebase [strlen(namebase)-1 ] = '\0'; 

/* set file name.  If namebase is only a dir name, "pername.log" will be 
   appended to namebase.  If namebase includes a dir name and part of a file 
   name, "_pername.log" will be appended. */ 
   if ( namebase [strlen(namebase)-1 ] == '/')
      sprintf (logname, "%s%s.log",  namebase, pername);
   else 
      sprintf (logname, "%s_%s.log", namebase, pername);
}








/*******************************************************************************
NAME:     dbase_name()
PURPOSE:  create a host file with a unique name.  File will be used for 
	  database info.  A sample output name is ./1995-03-16-14:42:07.scenein 
*******************************************************************************/

FUNCTION void dbase_name (
         char *basename,  /* input:  base for output file name.  the trailing 
			     '.' will be deleted if it exists */ 
   const char *extension, /* input:  file extension for output file name 
			     without a leading '.' */ 
         char *dbname)    // output: name of new file  
{

// declare variables  
   char *sdate;     // system date for file name  
   FILE *filep;     // points to output file  
   long exists,     // flag, does file exist  
        wait;       /* loop index, seconds to wait to change file name, 
		       if system date changes, file name will change */ 

// eliminate trailing '.' on basename, '.' was returned from c_getnam() 
   if ( basename [strlen(basename)-1 ] == '.')
        basename [strlen(basename)-1 ] = '\0'; 

/* For DATELOG-SCENEOUT, when multiple values are given for DATEIND, several
   database logs will be created.  This should be the only instance of a file
   existing on disk of the calculated name.  File locking effectively prevents 
   two concurrent DATELOG executions.  This block of code makes a unique file 
   name. */

   for (exists=TRUE, wait=0;  exists && wait<3;  wait++)
      {
      // get file name  
	 sdate = system_date (TRUE); 
	 sprintf (dbname, "%s%s.%s", basename, sdate, extension);
      // check file existence  
	 if (access (dbname, 0))
	    exists = FALSE; 
	 else
	    sleep (1); 
      }

   // error check:  file exists  
      if (exists) 
         errclean ("datelog-dbase", "Database file %s already exists",
		  host_tae(dbname)); 

/* create file.  Local functions to write data to files require that files
   exist */

   if ((filep = fopen (dbname, "w")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for writing", 
	       host_tae(dbname));
      
   fclose (filep); 
}








/*******************************************************************************
NAME:     host_tae()
PURPOSE:  return a TAE name given a host name as would be used in writing a 
	  user message.  This function never issues a fatal error as DATELOG
	  may already be trying to issue a fatal error. 
*******************************************************************************/

FUNCTION char * host_tae (
   char *hostname)  // input:  host name to translate  
{
// declare variable  
   char *taename;    // output:  TAE name  

/* error check:  too much dynamically allocated memory, error was never 
   encountered during testing */ 
   if (tofree.count >= MAXMEMORY)
   {
      c_lasmsg (FALSE, "datelog-maxm", "MAXMEMORY exceeded in host_tae()"); 
      return (char *) (hostname); // cast away constness for return value 
   }

// allocate memory for return value 
   taename = (char *) malloc (sizeof (char) * CMLEN);
   if (taename == NULL)
      {
      c_lasmsg (FALSE, "datelog-memory", "Unable to allocate memory"); 
      return (char *) (hostname); // cast away constness for return value 
      } 

// save location of new memory  
   tofree.address[tofree.count++] = taename;

// translate name from TAE to host
   strcpy(taename, hostname);

// return 
   return (taename); 
}






/*******************************************************************************
NAME:     system_date()
PURPOSE:  return system date  
*******************************************************************************/

FUNCTION char * system_date (
   const long seconds)
   /* input:  if (seconds), return date in yyyy-mm-dd-hh:mm:ss 
	      format;  else use            yyyy-mm-dd-hh:mm 
	      example output string is 1995-03-16-14:42:07 */ 
{
// declare variables  
   char *time_string;      // output: local system time as a string 
   time_t green_time;      // Greenwich system time 
   struct tm *timestruct;  // time structure used for local time 

// error check:  too much dynamically allocated memory 
   if (tofree.count >= MAXMEMORY)
      errclean ("datelog-max", "MAXMEMORY exceeded in system_date()");

// allocate memory 
   time_string = (char *) malloc (sizeof (char) * 30);
   if (time_string == NULL)
      errclean ("datelog-memory", "Unable to allocate memory"); 

// save location of new memory  
   tofree.address[tofree.count++] = time_string;

// convert greenwich time to local time  
   if (time (&green_time) == -1)
      errclean ("datelog-time", "Error getting system time");
   timestruct = localtime (&green_time);

/* add year to system year.  
   what happens in year 2000 is not defined by the localtime man page */
/*
  DLLOYD Y2K -- this is okay, in 2000 tm_year = 100
----------------------------------------------------*/
   timestruct->tm_year += 1900;

// create time string  
   sprintf (time_string, "%4d-%02d-%02d-%02d:%02d", timestruct->tm_year,
	   timestruct->tm_mon + 1, timestruct->tm_mday, timestruct->tm_hour, 
	   timestruct->tm_min);
   if (seconds)
      sprintf (time_string + strlen(time_string), ":%02d", timestruct->tm_sec); 

// return  
   return (time_string);
}







/*******************************************************************************
NAME:     strlower()
PURPOSE:  convert input string to lower case 
*******************************************************************************/

FUNCTION void strlower (char *string)
{
   for (;  *string!='\0';  string++)
      *string = tolower(*string); 
}







/*******************************************************************************
NAME:     set_prot()
PURPOSE:  set protection of a file.  implicit input: TAE parm prot
*******************************************************************************/

FUNCTION void set_prot (
         char *infile,    // input:  set protection for this file  
   const long lockmode)   /* input:  flag, set file protection to enable 
			     mandatory file locking, prototype this parameter 
			     as long lockmode=FALSE */ 
{

// declare variables  
   char converter[20];     // format string used to convert type  
   long taeprot,           // file protection as input TAE parm  
	octprot;           // file protection in octal  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

// get TAE parm  
   tparm = p_find (&pblock, "prot");
   taeprot = IVAL (*tparm, 0);

// convert from octal to int  
   sprintf (converter, "%d",   taeprot); 
   sscanf  (converter, "%od", &octprot); 

// optionally add flags for file locking
   if (lockmode)
      octprot |= S_ISGID;

// set protection  
   if (chmod (infile, octprot) ) 
      errclean ("datelog-prot", "Error setting protection %d for file %s", 
	       taeprot, host_tae(infile)); 
}









/*******************************************************************************
NAME:     write_output()
PURPOSE:  write a message about the output files.  This function gives a better 
	  description of the output created/updated than c_complt()  
*******************************************************************************/

FUNCTION void write_output (
   const long nfiles,                // input:  number of files created  
         char outnames[MAXF][CMLEN], // input:  host names of files created  
   const char *action)         /* input:  action performed as in "created file"
				  or "updated file".  This function will append
				  an "s" to the action if nfiles>1 */ 
{

// declare variables  
   long checkf,    // loop index, files checked for having been written  
        outf,      // loop index, number of output files  
	written;   // flag, was file name written yet  

// optionally return  
   if (!nfiles)
      return;  

// output header  
   c_lasmsg (FALSE, "informational", "DATELOG %s%s:", action, 
	     nfiles>1 ? "s" : ""    ); 

// write file names, translating host names to TAE names  
   for (outf=0; outf<nfiles; outf++)
   {
      // check for having already written this file name  
	 for (written=FALSE, checkf=0;  checkf<outf;  checkf++)
	    if (!strcmp (outnames[outf], outnames[checkf]) )
	    {
	       written=TRUE;
	       break;
	    }

      // write file name  
	 if (!written) 
	    c_lasmsg(FALSE, "informational", "  %s", host_tae(outnames[outf]) );
   }
}







/*******************************************************************************
NAME:     file_header()
PURPOSE:  write file header (table heading) to a log file 
*******************************************************************************/

FUNCTION void file_header (
         FILE *filep,  /* input: write header to this file.  File existence was 
			  already checked in the calling routines. */ 
   const char *line1)  // input: text for first line of file header  
{

// declare variables  
   long spaces,  // number of leading spaces to write  
	written; // loop index, number of spaces written  

// write leading spaces to center first line  
   spaces = strlen ("--------------------------------------------------------------------------------------------------------------")
    - strlen (line1);
   spaces /= 2;
   if (spaces < 0)
      spaces = 0; 
   for (written=0;  written<spaces;  written++)
      fputc (' ', filep); 

// write header lines.  Function must write LOGHEADER lines  
   fprintf(filep, "%s\n", line1); 
   fprintf(filep, "---------------------------------------------------------------------------------------------------------------\n");
   fprintf(filep, "Period   Study   Date   Scene/Composite   Processing        Processing      In/Out     Window in Study Area    \n");
   fprintf(filep, "Name     Area    Index  Identifier        Date and Time     Status          TapeId    SL     SS     NL     NS  \n");
   fprintf(filep, "-------  ------  -----  ----------------  ----------------  --------------  ------  ------ ------ ------ ------\n");
}








/*******************************************************************************
NAME:     write_log()
PURPOSE:  write scene data to a log file, all arguments are input 
*******************************************************************************/

FUNCTION void write_log (
         char *logfile,          // name of log file to write to  
   const char *sysdate,          // system date  
   const char *pername,          // name of current composite period  
   const char *sarea,            // study area  
   const long dateind,           // date index  
   const char *sceneid,          // scene id  
   const char *tapeid,           // tape id  
   const struct iparm *swindow,  // scene window, includes a validity flag  
   const char *swstring,         /* place holder for scene window, used if 
	 		            swindow.count == BADSWINDOW */ 
   const char *pstatus,          // processing status  
   const long newfile)           // flag, is this a new file?  
{

// declare variables  
   FILE *filep;   // points to output file  
   long index;    // loop index  

// check file existence  
   if (access (logfile, 0))
      errclean ("datelog-nolog", "Log file %s does not exist", 
	       host_tae(logfile));

// open and write to file  
   // open file  
      if ((filep = fopen (logfile, "a")) == NULL)
	 errclean ("datelog-open", "Unable to open file %s for appending", 
		  host_tae(logfile));

   // write most info  
      fprintf (filep, "%-9s",    pername ); 
      fprintf (filep, "%-8s",    sarea   ); 
      fprintf (filep, "%4d   ",  dateind ); 
      fprintf (filep, "%-18s",   sceneid ); 
      fprintf (filep, "%-18s",   sysdate ); 
      fprintf (filep, "%-16s",   pstatus ); 
      fprintf (filep, "%-8s",    tapeid  ); 

   // write swindow info, checking swindow for validity  
      if (swindow->count == BADSWINDOW)
	 fprintf (filep, "%-28s", swstring); 
      else 
	 for (index=0; index<swindow->count; index++)
	    fprintf (filep, "%6d ", swindow->value[index]); 
      fprintf (filep, "\n"); 

   // close file  
      fclose (filep); 

// increment count of output files  
   if (newfile) 
      strcpy (outfiles.newfiles[outfiles.newcount++], logfile); 
   else
      strcpy (outfiles.upfiles[outfiles.upcount++], logfile); 
}







/*******************************************************************************
NAME:     free_memory()
PURPOSE:  free dynamic memory
*******************************************************************************/

FUNCTION void free_memory (void) 
{
// declare variable  
   long index;  // loop index, pointers to free  

// free  
   for (index=0;  index<tofree.count;  index++)
      free (tofree.address[index] ); 
}



