
/*******************************************************************************
NAME:     lock.c 
PURPOSE:  Functions dealing with file locking, see user note 4 in datelog.pdf
          and comments at end of file. 
*******************************************************************************/

#include <fcntl.h>     // for calling open() and lockf()
#include <stdio.h>     // for C style file IO 
#include <string.h>    // for various string functions  
#include <sys/stat.h>  // for calling chmod()
#include <sys/types.h> // for calling chmod()
#include <unistd.h>    // for calling access()
#include "datelog.h"   // application header file  






/*******************************************************************************
NAME:     make_lock()
PURPOSE:  enable file locking
*******************************************************************************/

FUNCTION void make_lock (
   const char *lockbase)    // input:  base of host name for a lockfile  
{

// declare variables  
   char  lockfile[CMLEN]; // host name of file used for locking  
   FILE *filep;           // points to lockfile 
   int   filedes;         // file descriptor for the lock file 

// get lockfile name  
   make_name (lockbase, lockfile); 

// optionally create a lock file, enable mandatory (not advisory) file locking 
   if (access(lockfile,0) )   // file does not exist 
   {
      // create lock file 
	 if ((filep = fopen (lockfile, "w") ) == NULL)
	    errclean ("datelog-makelock", "Unable to create lockfile %s",
		     host_tae(lockfile)); 
	 fprintf (filep, 
	  "This file is used by LAS>DATELOG to lock access to log files, \n"
	  "thus preventing concurrent writes and possible file corruption.\n");
	 fclose (filep); 

      // enable locking
	 set_prot (lockfile, TRUE); 
   }

// lock the lockfile.  file is automatically unlocked upon process termination

   // the file must be open before calling lockf() 
   if (( filedes = open (lockfile, O_WRONLY, 0) ) == -1)
      errclean ("datelog-open", "Unable to open lockfile %s", 
	       host_tae(lockfile));

   if (lockf (filedes, F_LOCK, 0))
      errclean ("datelog-lockf", "Unable to lock lockfile %s", 
	       host_tae(lockfile));
}








/*******************************************************************************
NAME:     make_name()
PURPOSE:  build the name of lockfile much how c_lsmknm() would:  
	  delete characters after the last '/', append "lock.file" 
	  c_lsmknm() was not used because it created a file named .lock when 
	  lockbase was a directory name 
*******************************************************************************/

FUNCTION void make_name (
   const char lockbase[CMLEN],  // input:  base of host name for a lockfile  
         char lockfile[CMLEN])  // output: host name of lockfile  
{

// declare variable  
   char *lastchar;        // points to last character of lockfile  

// copy file name to output variable  
   strcpy (lockfile, lockbase); 

// delete trailing chars up to the trailing '/'  
   for (lastchar=lockfile+strlen(lockfile)-1;  
       *lastchar!='/' && lastchar!=lockfile;
       lastchar--)
	  *lastchar = '\0';

// append "lock.file" 
   strcat (lockfile, "lock.file");
}







/* a programmer note: 
   These functions have functionality similar to c_lockfil() but make_lock()
   waits for a concurrent process to complete while c_lockfil() does not;
   c_lockfil() calls c_lsmknm() while make_lock() does not;  these functions
   use lockf() while c_lockfil() does not */ 
