
/*******************************************************************************
NAME:     errclean.C
PURPOSE:  Consolidate functions for error handling.
*******************************************************************************/

// include files 
   #include <stdarg.h>      // for variable argument processing  
   #include "gacosdates.h"  // application header file 

// declare a global structure to keep track of all images  
   #define MAXIMAGES 20  // maximum number of input/output images 

   struct IMAGELIST
   {
      long delflg;                          // TAE global $delflg 
      struct INIMAGE  *inlist [MAXIMAGES];  // list of input  images  
      struct OUTIMAGE *outlist[MAXIMAGES];  // list of output images  
      short inputs,  // number of input  images 
	    outputs; // number of output images 
      IMAGELIST ();  // constructor 
   }
   imagelist;






/*******************************************************************************
NAME:     IMAGELIST::IMAGELIST()
PURPOSE:  constructor, initialize 
*******************************************************************************/
FUNCTION IMAGELIST::IMAGELIST (void)  
{
// initialze count of images 
   this->inputs = this->outputs = 0;
}





/*******************************************************************************
NAME:     set_delflg
PURPOSE:  set $delflg in IMAGELIST, this code should be part of the constructor
	  for IMAGELIST, but PARBLK *pblock is not available until main() is 
	  active, and I was unable to acquire PBLOCK with the TAE function 
	  p_inim()
*******************************************************************************/
FUNCTION void set_delflg (struct PARBLK *pblock) // TAE parameter block
{
// set delflg 
   struct VARIABLE *tparm;
   tparm = p_find (pblock, "$delflg");
   if ( *(SVAL(*tparm,0)) == 'Y')  // Y for yes 
      imagelist.delflg = TRUE;
   else
      imagelist.delflg = FALSE;
}






/******************************************************************************
NAME:     no_memory()
PURPOSE:  handle errors for new() in case system has no more memory available
******************************************************************************/
FUNCTION void no_memory (void)
{
   errclean ("gdates-memory", "Unable to allocate memory in no_memory()"); 
}








/******************************************************************************
NAME:     INIMAGE::INIMAGE()
PURPOSE:  Constructor, initialize status flags for an input image,
	  increment information in imagelist.
******************************************************************************/
FUNCTION INIMAGE::INIMAGE (void)
{
// initialize flags in INIMAGE  
   this->opened = FALSE;  // the image has not been opened  
   this->closed = FALSE;  // the image has not been closed  

// increment imagelist  
   if (imagelist.inputs == MAXIMAGES)
      c_lasmsg (TRUE, "gdates-maximages", 
		"MAXIMAGES was exceeded and should be increased."); 

   imagelist.inlist[ imagelist.inputs++ ] = this;
}




/******************************************************************************
NAME:     OUTIMAGE::OUTIMAGE()
PURPOSE:  Constructor, initialize status flags for an output image,
	  increment information in imagelist.
******************************************************************************/
FUNCTION OUTIMAGE::OUTIMAGE (void)
{
// initialize flags in OUTIMAGE  
   this->opened = FALSE;  // the image has not been opened  
   this->closed = FALSE;  // the image has not been closed  

// increment imagelist  
   if (imagelist.outputs == MAXIMAGES)
      c_lasmsg (TRUE, "gdates-maximages", 
		"MAXIMAGES was exceeded and should be increased."); 

   imagelist.outlist[ imagelist.outputs++ ] = this;
}





/*******************************************************************************
NAME:     INIMAGE::~INIMAGE()
PURPOSE:  destructor, reset list of active images
*******************************************************************************/
FUNCTION INIMAGE::~INIMAGE (void)
{
// declare variable
   long inum;  // loop index, image number 

// find this in imagelist.inlist[]
   for (inum=0;  inum<imagelist.inputs;  inum++)
      if (imagelist.inlist[ inum ] == this) 
	 break;

// delete this from imagelist.inlist[]
   for (;  inum+1<imagelist.inputs;  inum++)
      imagelist.inlist[ inum ] = imagelist.inlist[ inum+1 ];

// reset number of images 
   imagelist.inputs --;
}





/*******************************************************************************
NAME:     OUTIMAGE::~OUTIMAGE()
PURPOSE:  destructor, reset list of active images
*******************************************************************************/
FUNCTION OUTIMAGE::~OUTIMAGE (void)
{
// declare variable
   long inum;  // loop index, image number 

// find this in imagelist.outlist[]
   for (inum=0;  inum<imagelist.outputs;  inum++)
      if (imagelist.outlist[ inum ] == this) 
	 break;

// delete this from imagelist.outlist[]
   for (;  inum+1<imagelist.outputs;  inum++)
      imagelist.outlist[ inum ] = imagelist.outlist[ inum+1 ];

// reset number of images 
   imagelist.outputs --;
}






/*******************************************************************************
NAME:     errclean() 
PURPOSE:  errclean is called in the event of a fatal error.  It closes any open
	  images;  if $delflg="yes" output files are deleted;  a fatal error
	  message is issued.  This function can be called from anywhere in an
	  application except for constructors that are executed before main().
*******************************************************************************/
FUNCTION void errclean (
   char *mkey,    // message key passed to c_lasmsg()
   char *format,  // message text to print, corresponds to a similar
                        // argument to c_lasmsg()
         ...)           // additional arguments to print 
{
// declare arguments passed to this function  
   va_list argumentp; // lists a variable number of function arguments  

// declare variables 
   char message[3*ERRLEN];   // text of message printed by c_lasmsg() 
   long inum;                // loop index, image number  
   struct INIMAGE *input;    // points to an input image  
   struct OUTIMAGE *output;  // points to an output image  

// get input variables and format message string 
   va_start (argumentp, format);  
   vsprintf (message, format, argumentp); 
   va_end (argumentp);

// issue error message
   c_lasmsg (FALSE, mkey, message); 

// close any open image files and free buffers 
   for (inum=0;  inum<imagelist.inputs;  inum++)
   {
      input = imagelist.inlist[inum];
      if ( input->opened && ! input->closed)  
	 close_inimage (input); 
   }

   for (inum=0;  inum<imagelist.outputs;  inum++)
   {
      output = imagelist.outlist[inum];
      if (output->opened && !output->closed)  
	 close_outimage(output); 
   }

// optionally delete output files and images 
   if (imagelist.delflg) 
      delete_outfiles();

// terminate processing 
   c_lasmsg (TRUE, "gdates-fatal", "Fatal error encountered");
}






