
/*******************************************************************************
NAME:     gacosdates.h
PURPOSE:  application header file 
*******************************************************************************/

// include files 
extern "C"
{
#include "las.h"    // LAS include file
#include "image.h"  // for image data structures and function prototypes
#include "util.h"
}

// symbolic constants 
   #define FLAGDATE     -1  // flag value to indicate invalid date index 
   #define HEADER_LINES  5  // number of header lines in a short term log 
   #define LINELEN     200  // more than N characters on line (about 115) of log
   #define BYTEMAX     255  // max value for a byte image, the date band is byte
   #define BYTEVALS    256  // number of byte values, 0-255 
   #define MININDEX     10  // minimum date index for global scaling
   #define NAREAS        9  // # of non-fill study areas in global composite
   #define SCENEID      16  // string length of a scene id 

// declare several types of data structures 

   // a file name
      struct FILENAME 
      {
	 char tae [CMLEN],  // TAE  file name
	      host[CMLEN];  // host file name
      };

   /* user TAE parameters, other parameters (especially INDATE and OUTDATE) are
      in image data structures */ 
      struct UPARMS 
      {
	 char *pername;  // name of compositing period
	 long msgtim,    // $msgtim 
	      scalfact;  // scale factor used to reduce image size for testing
	 struct FILENAME dbname,   // base of name for database logs 
			 outinlog, // name of output inlog, built from TAE parms
			 inlogs[NAREAS],  // names of all input  short term logs
			 outlogs[NAREAS]; // names of all output short term logs
      }; 

   // scenes and date indices in one short-term log or in one study area 
      struct SCEDATES 
      {
	 long nrecords;    // number of records with sceneid and date index
	 long *dateindex;  // date index in each record 
	 char **sceneid;   // sceneid in each record 
      };

   // describe a global dateband image
      struct DATEBANDI
      {
	 long dtype,          // data type
	      numbands,       // number of bands   in overall image 
	      numlines,       // number of lines   in overall image 
	      numsamps,       // number of samples in overall image 
	      // the following windows in overall image are 1 relative
	      sl[NAREAS],     // starting line     for each study area 
	      ss[NAREAS],     // starting sample   for each study area 
	      nl[NAREAS],     // number of lines   for each study area 
	      ns[NAREAS],     // number of samples for each study area 
	      number[NAREAS]; // translates from 9-non fill area numbers to 12
			      // overall study area numbers 
	 DATEBANDI (void);             // constructor
	 void scale_size (const long); // reduce expected image size for testing
      };

// a global variable to describe the dateband image 
   #ifdef GLOBALVAR 
      struct DATEBANDI datebandi;
   #else 
      extern struct DATEBANDI datebandi;
   #endif 







// function prototypes for local source files 

// check.C
   void check_indate (const struct INIMAGE *indate); 

// dbase.C
   void dbase_name (const struct FILENAME basename, 
		    const char * const extension, 
		    struct FILENAME &dbname); 

// errclean.C 
   void no_memory (void); 
   void set_delflg (struct PARBLK *pblock);
   void errclean (char *mkey, char *format, ...); 

// getpar.C
   void getpar (struct PARBLK *pblock, struct UPARMS &uparms); 

// histogram.C
   void histogram (struct PARBLK *pblock, 
		   short wasUsed[NAREAS][BYTEVALS]); 

// image.c prototyped in image.h

// image2.C
   void open_inimage (struct PARBLK *pblock, const long areanum, 
		      struct INIMAGE *inimage, long accessm=IREAD, 
		            char *parmname="INDATE",
		      const long dtypeap=EBYTE, const long iomode=BSQIO, 
		      const long blocksize=NOTAPPLY); 
   void read_bsq2 (long curline, const long areanum, 
		   struct INIMAGE *inimage);
   void write_bsq2 (long curline, const long areanum, 
		    struct INIMAGE *inimage);

// mode.C
   void get_modes (struct PARBLK *pblock); 
   void change_prot (const struct FILENAME filename); 
   void change_owner (const struct FILENAME filename); 

// new_images.C 
   void make_lut (const struct SCEDATES oldinfo[NAREAS], 
		  const struct SCEDATES newinfo, 
		  short imageLut[NAREAS][BYTEVALS]); 
   void copy_image (struct PARBLK *pblock);
   void map_image (struct PARBLK *pblock, 
		   const short imageLut[NAREAS][BYTEVALS]);

// new_logs.C
   void make_inlog (const struct UPARMS uparms, 
		    const struct SCEDATES newinfo); 
   void make_short (const short wasUsed[NAREAS][BYTEVALS], 
		          struct UPARMS uparms, 
		    const struct SCEDATES oldinfo[NAREAS],
		    const struct SCEDATES newinfo);

// old_logs.C 
   void read_logs (const struct UPARMS uparms, 
		   struct SCEDATES oldinfo[NAREAS]); 
   void check_mapping (const struct SCEDATES oldinfo[NAREAS], 
		       const struct UPARMS uparms);
   void check_index (const short wasUsed[NAREAS][BYTEVALS], 
		     const struct SCEDATES oldinfo[NAREAS]);

// outfiles.C 
   void add_outfile_name (const struct FILENAME filename); 
   void add_outfile_name (char *filename); 
   void delete_outfiles(void);
   void complete(void);

// util.C
   void allocate_memory (const long nrecords, struct SCEDATES &onearea);
   void free_memory (struct SCEDATES &onearea); 
   int compareStrings (const void* input1, const void* input2);
   int unique_sort (void * base, const long nel, const long width,
		int (*compar) (const void *, const void *) );
   void new_indices (const struct SCEDATES oldinfo[NAREAS], 
		     struct SCEDATES &newinfo);
   long new_date (const char * const sceneid, const struct SCEDATES &newinfo);
   long get_nrecords (FILE *filep, const char *taename); 

