
/*******************************************************************************
NAME:     image.h
PURPOSE:  header file for image IO 
	  includes data structures for LAS images, 
	  symbolic constants and function prototypes
*******************************************************************************/




/* macro:  use this value when a function parameter does not apply.  
   must be 0 for the supports */
#define NOTAPPLY 0 

/* types of image IO allowed for, they all use LAS random IO.  random supports
   seem to work a little better than sequential supports */ 
   enum iomodes 
   {
      BSQIO,   // BSQ processing  
      BILIO,   // BIL processing  
      BLOCKIO  // in blocks       
   };

// GACOSDATES uses BSQ processing 


/* a note on data types: 
   An image can potentially have 3 different data types:
   dtypein:  data type as exists in input disk image
   dtypeap:  data type as processed internally by the application
   dtypeout:  data type as will exist in output disk image.  Data types
   can be EBYTE, EWORD, ELONG, EREAL. */ 



/* data structure for one LAS input image, used to facilitate IO.  Because LAS
   image specs allow for grouping image components, (e.g. in="image1 + image2")
   one input image may correspond to multiple disk image files.  The same
   structure is used for all types of IO, although all structure members are 
   not always used. */
   struct INIMAGE
   {
      char hostname[MAXIMG][CMLEN],// host name for image files  
           taename[MAXIMG][CMLEN]; // TAE name for image files  
      struct FDESC *fdarray[MAXIMG+1];      // file descriptor array  
      long bands[MAXIMG][MAXBND+1],// band numbers for each image file  
           blocksize,              // size of blocks for BLOCKIO  
	   blocknl, blockns,       /* image size after expansion for blocking,
				      for BLOCKIO */
           cblut[MAXIMG*MAXBND],   // lut for current band  
 	   dsizeap,                // # bytes per pixel as seen by the app 
 	   dtypeap,                // image data type as seen by the app  
 	   dtypein,                // image data type as read = EBYTE, ... 
           fdlut[MAXIMG*MAXBND],   // lut for file descriptor 
	   fillns,                 /* ns needed to fill last block, for 
				      BLOCKIO */
           iomode,                 // mode of io, = BILIO, BSQIO, BLOCKIO  
           lblocks,                /* number of blocks in line direction, 
				      for BLOCKIO */
           nbands[MAXIMG],         // number of bands in each image file  
           nmrimg,                 // number of disk image files  
           nl, ns,                 /* size of input image = window[][NL], 
				      window[][NS] */ 
           sblocks,                /* number of blocks in sample direction, 
				      for BLOCKIO */ 
           totalbands,             // total # of bands for all image files 
           window[MAXIMG][4];      // window (sl ss nl ns) for each image file
      unsigned char **bufloc,      /* location in data buffer for each line of
	                              data, for BLOCKIO */ 
	       *buffer,            // holds one image line for BSQIO  
	       *linebuf[MAXBND];   /* holds one line for each band.  for BILIO 
	                              and BLOCKIO */ 
      // status true/false flags used for error handling  
      long closed,                 // has the image been closed  
	   opened;                 // has the image been opened 

      // constructor to initialize flags
      INIMAGE (void);

      // destructor to reset list of active images 
      ~INIMAGE (void); 
   }; 



/* data structure for one LAS output image, used to facilitate IO, 
   structure includes some TAE globals */ 
   struct OUTIMAGE
   {
      char hostname[1][CMLEN],     /* host name for image file, supporting
                                      routines expect a 2-d array */ 
           taename[1][CMLEN];      /* TAE name for image, supporting 
				      routines expect a 2-d array */
      struct FDESC *fdarray[1+1];           // file descriptor array  
      long blocksize,              // size of blocks for BLOCKIO  
	   blocknl, blockns,       /* image size after expansion for blocking,
				      for BLOCKIO */
 	   dsizeap,                // # bytes per pixel as seen by the app 
 	   dtypeap,                // image data type as seen by the app  
 	   dtypeout,               // image data type to write = EBYTE, ... 
	   fillns,                 /* ns needed to fill last block, for 
				      BLOCKIO */
           iomode,                 // mode of io, = BILIO, BSQIO, BLOCKIO  
           lblocks,                /* number of blocks in line direction, 
				      for BLOCKIO */
           minmax,                 /* TAE global $minmax, flag, determine 
				      min/max for ddr? = T,F */
	   msgtim,                 /* TAE global $msgtim, time interval for
	                              processing message */ 
           nl, ns,                 // size of output image  
           sblocks,                /* number of blocks in sample direction, for
				      BLOCKIO */ 
           totalbands,             // total number of bands  
	   truncate;               /* flag, was the output data truncated to
	                              fit in the allowed range, used only if
				      c_pxconv() is called */ 
      unsigned char **bufloc,      /* location in data buffer for each line of
	                              data, for BLOCKIO */ 
	       *buffer,            // holds one image line for BSQIO  
	       *linebuf[MAXBND];   /* holds one line for each band.  for BILIO 
	                              and BLOCKIO */ 
      double minval[MAXBND],       // min pixel value, used to update ddr  
             maxval[MAXBND];       // max pixel value, used to update ddr  
      // status flags used for error handing, = true/false  
      long closed,                 // has the image been closed  
	   delflg,                 // TAE global $delflg, delete images if err
	   opened;                 // has the image been opened 

      // constructor to initialize flags
      OUTIMAGE (void);

      // destructor to reset list of active images 
      ~OUTIMAGE (void); 
   }; 



/* data structure giving processing parameters for output image, 
   simplifies the programmer interface for open_outimage() */ 
   struct OUTINFO 
   {
      char hostname[CMLEN], /* host name for output image, use "" if image name
			       is to be found in pblock */ 
           *parmname; /* TAE parameter name for output image, used to get name
                         in pblock */ 
      long dtypein,   /* data type of input image, use NOTAPPLY if there is 
			 no input image.  otherwise use EBYTE, EWORD, ELONG, or
			 EREAL */ 
           dtypeap,   /* output image data type as processed by the application,
			 use NOTAPPLY if the app handles data type conversion 
			 and conversion does not occur upon writing to disk */ 
           dtypeout,  /* image data type as output to disk image.  use
			 NOTAPPLY if dtypeout should be based on parm odtype */ 
           iomode,    // mode of io. = BSQIO, BILIO, or BLOCKIO  
           blocksize, /* size of blocks for BLOCKIO.  ignored if not using 
			 block io */ 
           nl,        // nl for output image  
           ns,        // ns for output image  
           nbands;    // nbands for output image  
   };




// prototypes functions in image.c 
   void open_bsq_in (struct PARBLK *pblock, char *parmname, long dtypeap, 
	struct INIMAGE *inimage); 
   void open_inimage (struct PARBLK *pblock, char *parmname, long dtypeap, 
	long iomode, long blocksize, struct INIMAGE *inimage); 
   void open_bsq_out (struct PARBLK *pblock, char *parmname, long dtypeap, 
	struct INIMAGE *inimage, struct OUTIMAGE *outimage); 
   void open_outimage (struct PARBLK *pblock, struct OUTINFO *outinfo, 
	struct OUTIMAGE *outimage); 
   void read_bsq (long curline, long curband, struct INIMAGE *inimage);
   void write_bsq (long curline, long curband, struct OUTIMAGE *outimage);
   void close_inimage (struct INIMAGE *inimage);
   void close_outimage (struct OUTIMAGE *outimage);
   void finish_out (struct PARBLK *pblock, struct INIMAGE *inimage,
	struct OUTIMAGE *outimage);

