
/******************************************************************************
NAME:     new_logs.C
PURPOSE:  create new inlog and short term logs  
******************************************************************************/

// include files 
   #include <stdio.h>      // for C style IO 
   #include <string.h>     // for calling strcmp() 
   #include <unistd.h>     // for calling access() 
   #include "gacosdates.h" // application header file 

// information on files being processed 
   struct FILEINFO 
   {
      FILE *filep;          // points to open file 
      struct FILENAME name; // TAE and host file name
      // functions in other files use struct FILENAME 
   };

// prototype local functions 
   static void open_logs (const long lognum, 
			  const struct UPARMS &uparms, 
			  struct FILEINFO &inlog, 
			  struct FILEINFO &outlog);
   static void copy_file (const struct SCEDATES &newinfo, 
			  struct FILEINFO &inlog, 
			  struct FILEINFO &outlog, 
			  struct SCEDATES &delWritten); 
   static void translate_index (const struct SCEDATES &newinfo, 
			        struct SCEDATES &delWritten, char *line);
   static void update_log (const struct UPARMS uparms, 
			   const long areanum, 
			   const short wasUsed[BYTEVALS], 
			   const struct SCEDATES oldinfo, 
			   const struct SCEDATES newinfo, 
			   const struct SCEDATES delWritten, 
			   const struct FILENAME logfile);
   static long check_deletes (const char *sceneid, const SCEDATES *delWritten);
   static void write_log (const struct FILENAME logfile, const long newfile, 
			  const char *pername, const long sarea, const long
			  dateindex, const char *sceneid);





/******************************************************************************
NAME:     make_inlog()
PURPOSE:  make a new inlog used by LAS>GACOSADD for calculating date index
******************************************************************************/
FUNCTION void make_inlog (
   const struct UPARMS uparms,    // user TAE parameters 
   const struct SCEDATES newinfo) // scenes and new indices
{
// check for existence of file outinlog 
   if (!access (uparms.outinlog.host, F_OK))
      c_lasmsg (FALSE, "gdates-overwrite", "OUTINLOG %s already exists and "
      "will be overwritten", uparms.outinlog.tae); 

// open outinlog 
   FILE *filep;
   filep = fopen (uparms.outinlog.host, "w");
   if (filep == NULL)
      errclean ("gdates-open", "Unable to open file %s for writing",
      uparms.outinlog.tae); 

// write header 
   fprintf (filep, "Date indices to be used for %s\n", uparms.pername); 
   fprintf (filep, "-----------------------------------\n"); 
   fprintf (filep, "Sceneid           Date Index\n"); 
   fprintf (filep, "----------------  ----------\n"); 

// write scenes and date indices 
   for (long counter=0;  counter<newinfo.nrecords;  counter++)
      fprintf (filep, "%s %7d\n", 
	      newinfo.sceneid[counter], 
	      newinfo.dateindex[counter]); 

// close file
   if (fclose (filep))
       errclean ("gdates-close", "Error closing file %s", uparms.outinlog.tae); 

// change file protection 
   change_prot (uparms.outinlog); 

// update list of files created
   add_outfile_name (uparms.outinlog); 
}






/*******************************************************************************
NAME:     make_short()
PURPOSE:  create new short term logs with new date indices.  This function is 
	  similar to read_logs() in old_logs.C 
*******************************************************************************/
FUNCTION void make_short (
   const short wasUsed[NAREAS][BYTEVALS], // histogram of INDATE image 
         struct UPARMS uparms,            // user TAE parameters 
   const struct SCEDATES oldinfo[NAREAS], // scenes and old date indices 
   const struct SCEDATES newinfo)         // sceneids and new date indices 
{
// declare variables 

   struct FILEINFO 
   inlog,                  // input  short term log 
   outlog;                 // output short term log 

   struct SCEDATES delWritten;  // list of scenes for which "delete" or
   // "overwritten" was written to the short term log.  delWritten.dateindex[] 
   // is not used 

// initialize and start processing line message
   extern long curent;   // current entity, for processing message

   long messform = GEN2,  // parameter to c_inittm(), message format
        nareas = NAREAS,  // parameter to c_inittm()
        notused = -1;     // parameter to c_inittm() which is ignored

   curent = 1;
   c_inittm ("GACOSDATES", "Updated logs for study area", "", "", "", 
             &nareas, &notused, &uparms.msgtim, &messform);
   c_startm ();

// loop through all logs, there is one log for each study area  
   for (long lognum=0;  lognum<NAREAS;  lognum++, curent++)
   {
      // open log files 
	 open_logs (lognum, uparms, inlog, outlog); 

      // determine total number of records in input file
	 long nrecords = get_nrecords (inlog.filep, inlog.name.tae); 

      // allocate memory for delWritten 
	 delWritten.nrecords=0;   // initialize count of records with scenes
	 allocate_memory (nrecords, delWritten);   

      // copy file
	 copy_file (newinfo, inlog, outlog, delWritten); 

      // close files
	 if (fclose (inlog.filep) )
	     errclean ("gdates-close", "Error closing file %s", inlog.name.tae);

	 if (fclose (outlog.filep) ) 
	     errclean ("gdates-close", "Error closing file %s",outlog.name.tae);

      // update short term log to describe scenes which were deleted from the 
      // composite, but for which "delete"/"overwritten" was not written to the
      // short term log.  also make a data base file 
         update_log (uparms, lognum, wasUsed[lognum], oldinfo[lognum], 
		     newinfo, delWritten, outlog.name); 

      // change protection of short term log
	 change_prot (outlog.name); 

      // update list of files created
	 add_outfile_name (outlog.name); 

      // free memory
	 free_memory (delWritten); 
   }
   c_stoptm(); 
}






/*******************************************************************************
NAME:     open_logs()
PURPOSE:  open log files.  called only by make_short()
*******************************************************************************/
FUNCTION static void open_logs (
   const long lognum,             // log or study area being processed 
   const struct UPARMS &uparms,   // user TAE parameters 
	 struct FILEINFO &inlog,  // input  short term log 
	 struct FILEINFO &outlog) // output short term log 
{
// set log file names 
   inlog.name  = uparms.inlogs [lognum]; 
   outlog.name = uparms.outlogs[lognum]; 

// check file existence 
   if (access (inlog.name.host, F_OK))
      errclean ("gdates-nolog", "Short term log %s does not exist", 
		inlog.name.tae); 

   if (!access (outlog.name.host, F_OK))
      c_lasmsg (FALSE, "gdates-overwrite", "Output short term log %s "
		"already exists and will be overwritten", outlog.name.tae);

// open files
   inlog.filep = fopen (inlog.name.host, "r");
   if (inlog.filep == NULL)
      errclean ("gdates-open", "Unable to open file %s for reading",
	       inlog.name.tae);

   outlog.filep = fopen (outlog.name.host, "w");
   if (outlog.filep == NULL)
      errclean ("gdates-open", "Unable to open file %s for writing", 
	       outlog.name.tae);
} 








/*******************************************************************************
NAME:     copy_file()
PURPOSE:  copy inlog to outlog, update delWritten.  called only by make_short()
*******************************************************************************/
FUNCTION static void copy_file (
   const struct SCEDATES &newinfo,    // scenes and new date indices 
	 struct FILEINFO &inlog,      // input  short term log 
	 struct FILEINFO &outlog,     // output short term log 
	 struct SCEDATES &delWritten) // list of scenes for which 
	 // "delete" or "overwritten" was written to the short term logs 
{
// declare variable 
   char linein[LINELEN];   // a line of text being processed 

// copy header 
   for (long headline=0;  headline<HEADER_LINES;  headline++)
   {
      // read header 
      if (fgets (linein, LINELEN, inlog.filep) == NULL)
      {
	 fclose(inlog.filep);  
	 fclose(outlog.filep); 
	 errclean ("gdates-noheader", "End of file detected in short "
		   "term log %s while reading the header", inlog.name.tae);
      }

      // write header 
      if (fputs (linein, outlog.filep) == EOF)
      {
	 fclose(inlog.filep);  
	 fclose(outlog.filep); 
	 errclean ("gdates-write", "Error writing to short term log %s", 
		   outlog.name.tae); 
      }
   }

// copy the rest of the file, translating date index from old to new  
   while ((fgets (linein, LINELEN, inlog.filep) != NULL))
   {
      // transate date index, update delWritten 
      translate_index (newinfo, delWritten, linein); 

      // write text 
      if (fputs (linein, outlog.filep) == EOF)
      {
	 fclose(inlog.filep);  
	 fclose(outlog.filep); 
	 errclean ("gdates-write", "Error writing to short term log %s", 
		   outlog.name.tae); 
      }
   } 
}







/*******************************************************************************
NAME:     translate_index()
PURPOSE:  translate the date index on a line of text from the old calculation
          method to the new.  line of text is from a short term log.
	  update delWritten.  Function is called only by copy_file()  
*******************************************************************************/
FUNCTION static void translate_index(
   const struct SCEDATES &newinfo,    // scenes and new date indices 
         struct SCEDATES &delWritten, // list of scenes with "delete" or
	                              // "overwritten" in the log
         char *line)                  // date index is on this line 

{
// declare variables 
   char pername[LINELEN],    // compositing period name in *line 
        sceneid[LINELEN],    // sceneid in *line
        studyarea[LINELEN];  // study area in *line, declared LINELEN to
			     // prevent potential memory errors 
   long dateindex;           // date index in *line 

// determine if this record is relevant.  skip information on composites
   long fields = sscanf (line, "%s %s %d %s",
                 pername, studyarea, &dateindex, sceneid);

   if (fields != 4)
      return;

// update delWritten if *line contains "delete" or "overwritten" 
   if ( strstr(line,"delete") != NULL)
      strcpy (delWritten.sceneid[ delWritten.nrecords++ ], sceneid); 
   else if ( strstr(line,"overwritten") != NULL)
      strcpy (delWritten.sceneid[ delWritten.nrecords++ ], sceneid); 

// don't change date index of FLAGDATE 
   if (dateindex == FLAGDATE)
      return; 

// determine new date index for this sceneid 
   long newDate = new_date (sceneid, newinfo); 

// write newDate to *line.  code was borrowed from LAS>DATELOG
   short nchars = sprintf (line, "%-9s%-8s%4d", pername, studyarea, newDate); 

// replace null added by sprintf 
   *(line + nchars) = ' ';
} 








/*******************************************************************************
NAME:     update_log()
PURPOSE:  update logs to describe scenes which were deleted from the composite,
	  but for which "delete" or "overwritten" was not written to the short
	  term log.  Function is called only by make_short()
*******************************************************************************/
FUNCTION static void update_log (
   const struct UPARMS uparms,      // user TAE parameters 
   const long areanum,              // study area being processed 
   const short wasUsed[BYTEVALS],   // histogram of INDATE for this area 
   const struct SCEDATES oldinfo,   // scenes and old date indices for this area
   const struct SCEDATES newinfo,   // scenes and new date indices 
   const struct SCEDATES delWritten,// list of scenes with "delete" or 
				    // "overwritten" in the log
   const struct FILENAME logfile)   // write data to this log 
{
// declare variables
   long incomp;             // flag, was the scene in the composite 
   struct SCEDATES towrite; // list of scenes to write "overwritten" for 

// allocate memory for towrite
   allocate_memory (oldinfo.nrecords, towrite); 
   towrite.nrecords = 0;  // initialize number of records 

// loop for all scenes in oldinfo 
   for (long scenenum=0;  scenenum<oldinfo.nrecords;  scenenum++)
   {
      // set variables for the scene being processed 
      long olddate  = oldinfo.dateindex[scenenum]; 
      char *sceneid = oldinfo.sceneid  [scenenum];

      // if scene had FLAGDATE index, skip it 
      if (olddate == FLAGDATE)
	 continue; 

      // set incomp flag
      incomp = wasUsed[olddate]; 

      // set flag, was a "delete"/"overwritten" record written
      long wasdel = check_deletes (sceneid, &delWritten);

      // add selected scenes to towrite
      // if scene is not in composite, and a record was not written
      if (!incomp && !wasdel)
	 strcpy (towrite.sceneid[ towrite.nrecords++ ], sceneid); 
   }

// sort scenes in towrite, eliminate duplicates
   long unique = unique_sort (towrite.sceneid, towrite.nrecords, sizeof(char *),
			      compareStrings);
   towrite.nrecords = unique; 

// loop for all scenes in towrite
   for (long writescenes=0;  writescenes<towrite.nrecords;  writescenes++)
   {
      // get new date index 
      char *sceneid = towrite.sceneid[writescenes]; 
      long newdate = new_date(sceneid, newinfo);

      // write to short term log 
      write_log (logfile, FALSE, uparms.pername, areanum, newdate, sceneid);

      // write to database file 
      struct FILENAME dbname; // name of database file 
      dbase_name (uparms.dbname, "sceneout", dbname);
      write_log (dbname, TRUE, uparms.pername, areanum, newdate, sceneid);
   }

// free memory in towrite
   free_memory (towrite); 
}


   



/*******************************************************************************
NAME:     check_deletes()
PURPOSE:  return TRUE if a deleted record was written.  
	  Function is called only by update_log() 
*******************************************************************************/
FUNCTION static long check_deletes (
   const char *sceneid,        // see if deleted was written for this scene
   const SCEDATES *delWritten) // list of scenes with deleted written
{
   for (long checked=0;  checked<delWritten->nrecords;  checked++)
      if (!strcmp (sceneid, delWritten->sceneid[checked] ))
	 return TRUE;
   return FALSE; 
} 





/*******************************************************************************
NAME:     write_log()
PURPOSE:  write scene data to a log file;  function was modified from
          LAS>DATELOG;  function called only by update_log().
*******************************************************************************/
FUNCTION static void write_log (
   const struct FILENAME logfile, // name of log file to write to  
   const long newfile,   // flag, is this a new database file.  if !newfile, 
			 // logfile is a short term log that will be appended to
   const char *pername,  // name of compositing period  
   const long areanum,   // study area number
   const long dateindex, // date index  
   const char *sceneid)  // scene id  

{
// declare variables  
   FILE *filep;   // points to output file  

// check file existence  
   if (newfile) 
   {
      if (!access (logfile.host, F_OK))
	 errclean ("gdates-dbase", "Database file %s already exists", 
		   logfile.tae); 
   }
   else 
      if (access (logfile.host, F_OK))
	 errclean ("gdates-nolog", "Short term log %s does not exist", 
		   logfile.tae); 

// open file  
   if (newfile)
   {
      if ((filep = fopen (logfile.host, "w")) == NULL)
	 errclean ("gdates-open", "Unable to open file %s for writing", 
		   logfile.tae);
   }
   else 
      if ((filep = fopen (logfile.host, "a")) == NULL)
	 errclean ("gdates-open", "Unable to open file %s for appending", 
		   logfile.tae);

/* write to file.  
   The comments for this section describe variables used by LAS>DATELOG. 
   datebandi.number[] translates from 9 non-fill area numbers to 12 overall 
   study area numbers */ 

   fprintf (filep, "%-9s",    pername       ); 
   fprintf (filep, "%-8d",    datebandi.number[areanum] ); 
   fprintf (filep, "%4d   ",  dateindex     ); 
   fprintf (filep, "%-18s",   sceneid       ); 
   fprintf (filep, "%-18s",   "unknown    " ); // system date (sysdate) 
   fprintf (filep, "%-16s",   "overwritten" ); // processing status (pstatus)
   fprintf (filep, "%-8s",    "not-a"       ); // tapeid 
   fprintf (filep, "%-28s",   "entire-image-processed");  // swstring
   fprintf (filep, "\n"); 

// close file  
   fclose (filep); 

// processing for new database files
   if (newfile)
   {
      change_prot (logfile);       // change file protection 
      change_owner (logfile);      // change file owner
      add_outfile_name (logfile);  // update list of files created
   }
}




