
/******************************************************************************
NAME:     old_logs.C 
PURPOSE:  process existing short term logs 
******************************************************************************/

// include files 
   #include <string.h>      // for calling strcpy()
   #include <stdio.h>       // for C style file IO 
   #include <unistd.h>      // for calling access() 
   #include "gacosdates.h"  // application header file 

// prototype local functions 
   void static get_data (const char * const pername, const char * const logname,
			 const char * const linein, struct SCEDATES &onearea);





/******************************************************************************
NAME:     read_logs()
PURPOSE:  get all date indices and sceneid's from the short-term logs. 
	  This function is similar to make_short() in new_logs.C 
******************************************************************************/
FUNCTION void read_logs (
   const struct UPARMS uparms,            // input TAE parameters 
	 struct SCEDATES oldinfo[NAREAS]) // output:  date indices, sceneids
{
// declare variables 
   FILE *filep;              // points to a short term log file 
   char linein[LINELEN];     // a line of text being read 
   struct FILENAME logfile;  // host and TAE names of log file 
   long nrecords;   // number of records in file for which to allocate memory,
                    // may include a few records which are later ignored 

// loop through all logs, there is 1 log per study area 
   for (long lognum=0;  lognum<NAREAS;  lognum++)
   {
      // initialize number of records 
	 oldinfo[lognum].nrecords = 0;

      // make name of log file 
	 logfile = uparms.inlogs[lognum]; 

      // check file existence 
	 if (access (logfile.host, F_OK))
	    errclean ("gdates-nolog", "Short term log %s does not exist", 
	             logfile.tae);

      // open file 
	 filep = fopen (logfile.host, "r");
	 if (filep == NULL)
	    errclean ("gdates-open", "Unable to open file %s for reading", 
		     logfile.tae);

      // determine number of records in file 
	 nrecords = get_nrecords (filep, logfile.tae);

      // allocate memory for oldinfo[lognum]
	 allocate_memory (nrecords, oldinfo[lognum]);  

      // skip header in file 
	 for (long skipline=0;  skipline<HEADER_LINES;  skipline++)
	    if (fgets (linein, LINELEN, filep) == NULL)
	    {
	       fclose (filep);
	       errclean ("gdates-noheader", "End of file detected in short "
			 "term log %s while reading the header", logfile.tae);
	    } 

      // determine sceneid and dateindex for all records 
	 while (fgets (linein, LINELEN, filep) != NULL)
	    get_data (uparms.pername, logfile.tae, linein, oldinfo[lognum]);

      // close file 
	 if (fclose (filep))
	     errclean ("gdates-close", "Error closing file %s", logfile.tae); 
   }
}










/******************************************************************************
NAME:     get_data()
PURPOSE:  determine sceneid and dateindex for a record of a short term log 
******************************************************************************/
FUNCTION void static get_data (
   const char * const pername, // PERNAME as input to TAE 
   const char * const logname, // name of log file being processed, for err text
   const char * const linein,  // input line of text 
         struct SCEDATES &oldinfo) //update this info with sceneid and dateindex
{
// declare variables
   char Lperiod[LINELEN],    // compositing period name in *line
        sceneid[LINELEN],    // sceneid in *line
        studyarea[LINELEN];  // study area in *line, declared LINELEN to
                             // prevent improbable memory errors
   long dateindex;           // date index in *line

// part 1 of    "skip information on composites"
   long fields = sscanf (linein, "%s %s %d %s", 
		 Lperiod, studyarea, &dateindex, sceneid);

// check pername
   if ( strcmp (pername, Lperiod)) 
       errclean ("gdates-pername", "Parameter PERNAME has a value of "
                "%s but the period in short term log %s has a value of %s", 
                pername, logname, Lperiod); 

// part 2 of    "skip information on composites "
   if (fields != 4)
      return;

// check length of sceneid 
   if (strlen (sceneid) != SCENEID)
      errclean ("gdates-sceneid", "Sceneid %s in short term log %s has other "
                "than %d characters", sceneid, logname, SCENEID); 

// check date index 
   if (dateindex != FLAGDATE && dateindex < MININDEX)
       errclean ("gdates-minindex", "A date index less than %d was "
                "encountered in short term log %s", MININDEX, logname);

   if (dateindex > BYTEMAX)
       errclean ("gdates-maxindex", "A date index greater than %d was "
                "encountered in short term log %s", BYTEMAX, logname);

// set return data 
   strcpy ( oldinfo.sceneid [oldinfo.nrecords], sceneid);
   oldinfo.dateindex [oldinfo.nrecords] = dateindex;
   oldinfo.nrecords++;
}






/******************************************************************************
NAME:     check_mapping()
PURPOSE:  make sure there is a 1-1 mapping between date index and sceneid for
          each study area 
******************************************************************************/
FUNCTION void check_mapping (
   const struct SCEDATES oldinfo[NAREAS], // check mappings in this data 
   const struct UPARMS uparms)            // user parameters, for error text
{

// loop for all study areas
for (long narea=0;  narea<NAREAS;  narea++)

// loop for all records
   for (long recA=0;  recA<oldinfo[narea].nrecords;  recA++)
   {
   // skip over date indices of FLAGDATE, they are invalid 
      if (oldinfo[narea].dateindex[recA] == FLAGDATE)
	 continue;

   // loop through all other records 
      for (long recB=recA;  recB<oldinfo[narea].nrecords;  recB++)
      {

      // copy relevant info to simpler variables for easier understanding
	 char *sceneA = oldinfo[narea].sceneid[recA];
	 char *sceneB = oldinfo[narea].sceneid[recB];

	 long indexA = oldinfo[narea].dateindex[recA];
	 long indexB = oldinfo[narea].dateindex[recB];

      // skip over date indices of FLAGDATE, they are invalid 
	 if (indexB == FLAGDATE)
	    continue;

      // look for problems:  scenes match  && indices differ ||
      //                     scenes differ && indices match 
	 if (!strcmp (sceneA,sceneB) && indexA != indexB)
	    errclean ("gdates-badlogs1", "In short term log %s sceneid %s is "
		      "associated with date indices %d and %d", 
		      uparms.inlogs[narea].tae, sceneA, indexA, indexB); 

	 if (strcmp (sceneA,sceneB) && indexA == indexB)
	    errclean ("gdates-badlogs2", "In short term log %s date index %d "
		      "is associated with sceneids %s and %s", 
		      uparms.inlogs[narea].tae, indexA, sceneA, sceneB); 
      }
   }
} 
	    







/*******************************************************************************
NAME:     check_index()
PURPOSE:  make sure every dateindex in the histogram appears in the input logs
*******************************************************************************/
FUNCTION void check_index (
   const short wasUsed[NAREAS][BYTEVALS],  // histogram of INDATE
   const struct SCEDATES oldinfo[NAREAS])  // scenes and old date indices
{
// declare variables 
   char badlist[1000]; // list of bad date indices as one long string 
   long anybad,        // flag, were any bad date indices found in any areas
        areabad,       // flag, were any bad date indices found for this area
	areanum,       // loop index, study area number 
	checked,       // loop index, list of scenes which were checked 
	indexnum,      // loop index, date index number 
        matched;       // flag, was a particular scene matched for this area

// loop for all study areas and all date indices 
   for (anybad=FALSE, areanum=0;  areanum<NAREAS;  areanum++)
   {
      for (areabad=FALSE, *badlist='\0', indexnum=0;  
      indexnum<BYTEMAX;  indexnum++)
      {
	 // skip indices that have no data
	    if (! wasUsed[areanum][indexnum])
	       continue;  

	 // skip indices that correspond to fill values
	    if (indexnum < MININDEX)
	       continue; 

	 // look for date index in oldinfo, loop through all records 
	    for (matched=FALSE, checked=0;  
	    !matched && checked<oldinfo[areanum].nrecords;  
	    checked++)
	       if (indexnum == oldinfo[areanum].dateindex[checked])
		  matched = TRUE;

	 // increment list of bad indices
	    if (!matched)
	    {
	       sprintf (badlist+strlen(badlist), "%d ", indexnum); 
	       areabad = TRUE;
	       anybad  = TRUE;  
	    }
      }

   // write error message describing bad indices 
      if (areabad)
	 c_lasmsg (FALSE, "gdates-index", "For study area %d the following "
		   "date indices were found in image INDATE but not in the "
		   "short term log:  %s", 
		   datebandi.number[areanum], badlist); 
   }

// abort if any bad scenes were found in any areas
   if (anybad)
      c_lasmsg (TRUE, "gdates-fatal", "Fatal error encountered"); 
}

