
/******************************************************************************
NAME:     util.C
PURPOSE:  miscellaneous utilities 
******************************************************************************/

// include files 
   #include <stdlib.h>     // for calling qsort() 
   #include <string.h>     // for calling strcmp()
   #include "gacosdates.h" // application header file  







/*******************************************************************************
NAME:     allocate_memory()
PURPOSE:  allocate memory for struct SCEDATES
*******************************************************************************/
FUNCTION void allocate_memory (
   const long nrecords,             // allocate memory for this many records 
         struct SCEDATES &onearea)  // memory allocated here 
{
// allocate memory 
   onearea.dateindex = new long   [ nrecords ];
   onearea.sceneid   = new char * [ nrecords ]; 

   for (long rec=0;  rec<nrecords;  rec++)
      onearea.sceneid[rec] = new char [SCENEID+1];
}






/*******************************************************************************
NAME:     free_memory()
PURPOSE:  free memory for struct SCEDATES
*******************************************************************************/
FUNCTION void free_memory (
         struct SCEDATES &onearea)  // memory freed here 
{
// free memory 
   for (long rec=0;  rec<onearea.nrecords;  rec++)
      delete onearea.sceneid[rec]; 

   delete[]  onearea.dateindex;
   delete[]  onearea.sceneid  ;
}






/*******************************************************************************
NAME:     compareStrings()
PURPOSE:  compare two strings to enable sorting via qsort().  Function was 
	  designed to operate on     char **varname.  
*******************************************************************************/
FUNCTION int compareStrings (
   const void* input1, const void* input2) // values to compare
{
   return  strcmp (*(char **)input1, *(char **)input2); 
} 






/*******************************************************************************
NAME:     unique_sort()
PURPOSE:  sort a list of elements;  eliminate duplicates from the list;  return
	  number of elements in list.  function declaration is much like qsort()
*******************************************************************************/
FUNCTION int unique_sort (
         void * base, // points to the element at the base of the list
   const long nel,    // number of elements in the list
   const long width,  // size of each element in bytes
         int (*compar) (const void *, const void *)  
	 // function to compare elements
   )
{
// declare variables 
   // char* is used as a generic pointer, void* can not be incremented
   char * copyfrom, // copy pointer from this location 
	* copyto;   // copy pointer to   this location 
   long checked,    // loop index, number of values that have been checked 
	unique;     // number of unique elements in list 

// sort the list with qsort
   qsort (base, nel, width, compar); 

// loop for all elements in the list 
   for (checked=unique=0, copyfrom=copyto=(char *)base;   checked<nel;  
	copyto+=width, unique++)
   {
      // copy non-duplicate members in list 
      for (long index=0;  index<width;  index++)
	 copyto[index] = copyfrom[index]; 

      // skip over duplicates in list 
      for (;  checked<nel && !(*compar)(copyfrom,copyto);  
      copyfrom+=width, checked++)
	 ; 
   }

// return number of unique members in list 
   return unique; 
}








/******************************************************************************
NAME:     new_indices()
PURPOSE:  calculate and return new date indices for all scenes 
******************************************************************************/
FUNCTION void new_indices (
   const struct SCEDATES oldinfo[NAREAS],  // sceneids and date indices 
	 struct SCEDATES &newinfo)         // output:  scenes and new indices
{
// declare variables
   long areanum;  // loop index, area number 

// determine the total number of sceneids in oldinfo 
   long Nscenes = 0;
   for (areanum=0;  areanum<NAREAS;  areanum++)
      Nscenes += oldinfo[areanum].nrecords;

// make a list of all sceneids in oldinfo 
   char ** sceneList = new char * [Nscenes]; 

   long sceneCounter=0;   // index to sceneList[] 
   for (areanum=0;  areanum<NAREAS;  areanum++)
      for (long rec=0;  rec<oldinfo[areanum].nrecords;  rec++, sceneCounter++)
	 sceneList[sceneCounter] = oldinfo[areanum].sceneid[rec];

// sort the list
   long unique =unique_sort(sceneList, Nscenes, sizeof(char *), compareStrings);

// allocate memory for returned data
   allocate_memory (unique, newinfo);  

// set return data in newinfo.  
   // For global scaling, dateindices start at MININDEX and are incremented by 1
   newinfo.nrecords = unique; 
   for (long counter=0;  counter<unique;  counter++)  // loop through scenes
   {
      strcpy (newinfo.sceneid[counter], sceneList[counter]);
      newinfo.dateindex[counter] = counter+MININDEX;
   }

// free memory
   delete[] sceneList; 
}






/*******************************************************************************
NAME:     new_date()
PURPOSE:  return new date index for an input sceneid 
*******************************************************************************/
FUNCTION long new_date (
   const char * const sceneid,      // return new date index for this scene
   const struct SCEDATES &newinfo)  // scenes and their new date indices 
{
// look through all scenes 
   for (long checker=0;  checker<newinfo.nrecords;  checker++)
      if (! strcmp(sceneid, newinfo.sceneid[checker]))
	 return newinfo.dateindex[checker];   

// error check:  match not found.  This should never occur.
   errclean ("gdates-nomatch", "An internal error occurred in new_date()");

// prevent compiler warning, statement is never ever executed
   return E_FAIL; 
}






/*******************************************************************************
NAME:     get_nrecords()
PURPOSE:  return the number of records in a short term log 
*******************************************************************************/
FUNCTION long get_nrecords (
         FILE *filep,    // input:  points to a file which was already opened 
			 // file will be rewound to the beginning of file 
   const char *taename)  // TAE name of file being read, for error messages
{
// declare variables 
   char linein[LINELEN];  // holds a line of text on the file 
   long nrecords;         // return value, number of records in the short term 
			  // log excluding header 

// skip header 
   for (long skipline=0;  skipline<HEADER_LINES;  skipline++)
      if (fgets (linein, LINELEN, filep) == NULL)
      {
	 fclose (filep);
	 errclean ("gdates-noheader", "End of file detected in "
	          "short term log %s while reading the header", taename); 
      } 

// test last header line
   if ( (strstr (linein, "-------  ------  -----  ----------------"))  
   != linein)
      errclean ("gdates-badheader", "The last header line in short "
		"term log %s was corrupted", taename); 

// determine number of records
   for (nrecords=0;   fgets(linein,LINELEN,filep)!=NULL;   nrecords++)
      ;

// rewind file 
   rewind (filep); 

// return 
   return nrecords;
} 



