/****************************************************************************
FUNCTION:	acquire

PURPOSE:	Reads and formats data from the 225W framesync through
		a socket connection.  Data is written to a standard
		SF2 (big endian) format file.

INPUT:		required:
		    output file name
		optional:
		    IP address of frame sync
		 or logical name of frame sync
		    port number of frame sync

OUTPUT:		formatted file

RETURNS:	none

USAGE:		acquire -file <output file name> -ipadd <frame sync address>
                        -port <frame sync port number>

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: acquire.c,v $
Revision 1.6  2009/07/13 18:08:26  adaps
*** empty log message ***

Revision 1.5  2005/06/08 17:30:07  dlloyd
Merging NOAA 18 branch onto trunk

Revision 1.4.4.1  2005/06/08 17:28:48  dlloyd
Added satellite ID of 0d

Revision 1.4  2003/04/21 15:30:20  dlloyd
Added code for time correction algorithm.

Revision 1.3  2002/07/01 18:01:13  dlloyd
Minor documentation change.  Removed diagnostic call to fflush().

Revision 1.2  2002/05/20 14:48:24  dlloyd
Added support for creating and removing a lock file.

Revision 1.1  2002/05/20 13:23:24  dlloyd
Initial checkin of files

****************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "acquire.h"
#include <sys/time.h>

/* Forward definition of signal handler
---------------------------------------*/
void sighandler( int );

/* Static variables local to this function.  Declared here so that
the signal handler can access them closing the files on exit.
-----------------------------------------------------------------*/
int   socket_fd;	/* file descriptor for the socket		*/
FILE *image_file;	/* output file descriptor for the image file	*/
FILE *misc_file;	/* output file descriptor for miscellaneous data*/
FILE *tstamp_file;	/* output file descriptor for the time stamps	*/
char  lock_file_name[STRLEN];	/* lock file name			*/


int main
    (
    int argc,			/* count of command line arguments	*/
    char *argv[]		/* array of command line arguments	*/
    )
    {
    unsigned char buf[BUFLEN];	/* I/O buffer filled from socket	*/
    unsigned char allzero[ALLZERO] = {0};/* buffer of zeros used to fill*/
    				/* in the missing data from frame sync	*/
    char IPaddress[STRLEN];	/* IP address of the frame sync		*/
    char image_file_name[STRLEN];/* output image file name		*/
    char misc_file_name[STRLEN];/* miscellaneous data output file name	*/
    char tstamp_file_name[STRLEN];/* time stamp output file name	*/

    int  num_read;		/* number of bytes read from socket	*/
    int  msglength;		/* length of message to read		*/
    int  *bufptr = (int *)&buf;	/* integer pointer to the buffer	*/
    int  forever = TRUE;	/* while loop control variable		*/
    int  port_num;		/* port number of frame sync		*/

    FILE *lock_file;		/* file descriptor for lock file	*/

    struct timeval now;
    struct timezone tz;

    char satid[3];
    int  satnum;
    char *strptr;



    /* set up the signal handler
    ----------------------------*/
    signal( SIGHUP, sighandler );


    /* parse the input arguments
    ----------------------------*/
    parse_args( argv, IPaddress, &port_num, image_file_name,
                misc_file_name, tstamp_file_name, lock_file_name );
    

    /* Create a lock file
    ---------------------*/
    lock_file = fopen( lock_file_name, "w" );
    if ( !lock_file )
        {
	write_log(FALSE,"main","Error","Error creating lock file");
	lock_file_name[0] = '\0';
	}
    else
        {
	fprintf( lock_file, "%s is locked", lock_file_name );
	fclose( lock_file );
	}

    /* Open the output files
    ------------------------*/
    image_file  = fopen( image_file_name,  "w" );
    tstamp_file = fopen( tstamp_file_name, "w" );
    misc_file   = fopen( misc_file_name,   "w" );
    if ( !image_file || !misc_file || !tstamp_file )
	{
	write_log(TRUE,"main","file-open","Error: problem opening output file");
	exit(1);
	}  /* END while */


    /* create the socket connection with the frame sync
    ---------------------------------------------------*/
    if (( socket_fd = create_socket( IPaddress, port_num )) == FAIL )
        exit(1);



    /* Put the frame sync in the proper configuration.  If the proper
    configuration is not set the frame sync may not send the proper
    data.  Note: this has been a little touchy.  On occasion the
    frame sync will refuse to acknowledge.  In current testing this
    call has been commented out.
    if ( configure_framesync( socket_fd ) == FAIL )
        exit(1);
    -----------------------------------------------------------------*/
    strptr = strrchr( image_file_name, '/' );
    strptr++;
    sscanf( strptr, "ah%02d", &satnum );
    switch ( satnum )
        {
	case 15 : strcpy( satid, "07" ); break;
	case 16 : strcpy( satid, "03" ); break;
	case 17 : strcpy( satid, "0b" ); break;
	case 18 : strcpy( satid, "0d" ); break;
        case 19 : strcpy( satid, "0f" ); break;
	default : strcpy( satid, "00" );
	}


    /* Place a dummy header at the beginning of the image file
    ----------------------------------------------------------*/
    fwrite( allzero, sizeof(unsigned char), SFLHEADER, image_file );


    /* determine the sat num
    ------------------------*/


    /* Main processing loop.  Read from the framesync, determine
    the block type and write the data to the proper file.
    ------------------------------------------------------------*/
    while ( forever )
        {
	/* The first four bytes from the frame sync contain
	the number of bytes in the message, this is an
	integer value.  Get the message length.
	--------------------------------------------------*/
	num_read = read( socket_fd, buf, 4 );
	msglength = *bufptr;


	/* Read the entire message
	--------------------------*/
	num_read = 0;
	while ( num_read < msglength )
	    num_read += read( socket_fd, &buf[num_read], msglength-num_read );
	

	/* If the block type is 'Video', write
	the image and time stamp data.
	--------------------------------------*/
	if ( strncmp( (char *)&buf[1], "Video", 5 ) == 0 &&
	     strncmp( (char *)&buf[20], satid, 2 ) == 0 )
	    {
	    /* Write the time stamp information
	    -----------------------------------*/
	    gettimeofday( &now, & tz );
	    fwrite( &now.tv_sec, sizeof(time_t), 1, tstamp_file );
	    fwrite( &now.tv_usec, sizeof(long), 1, tstamp_file );
	    fwrite( buf, sizeof(unsigned char), TSTAMP, tstamp_file );

	    /* Write the image data filling in
	    the missing data with zero data.
	    ----------------------------------*/
	    fwrite( &buf[MINOR], sizeof(unsigned char), MINORSIZE, image_file );
	    fwrite( allzero,     sizeof(unsigned char), ALLZERO,   image_file );
	    fwrite( &buf[VIDEO], sizeof(unsigned char), VIDEOSIZE, image_file );
	    fwrite( allzero,     sizeof(unsigned char), POSTVIDEO, image_file );
	    }  /* END if Video block */
	
	/* Otherwise it's a miscellaneous block
	---------------------------------------*/
	else
	    fwrite( buf, sizeof(unsigned char), msglength, misc_file );
	}  /* END while forever */
    
    return 0;
    }  /* END main */



/*************************************************************************
FUNCTION:	sighandler

PURPOSE:	Catch the SIGHUP signal, and terminate gracefully closing
		the socket and output files.

INPUT:		signal number

OUTPUT:		none

RETURNS:	none, does not return, program is exited here
*************************************************************************/
void sighandler
    (
    int signal_number		/* signal number of the interrupt	*/
    )
    {
    if ( signal_number == SIGHUP )
        {
	fclose( image_file  );
	fclose( misc_file   );
	fclose( tstamp_file );
	close ( socket_fd   );
	if ( lock_file_name[0] != '\0' )
	    {
	    if ( remove( lock_file_name ) != 0 )
	        write_log(FALSE,"sighandler","Error",
		    "Could not remove lock file");
	    }
	write_log(FALSE,"sighandler","SIGHUP","Exiting program");
	exit(0);
	}  /* END if */
    else
        return;
    }  /* END sighandler */

