/**************************************************************************
FUNCTION:	fs_status

PURPOSE:	To obtain the status of the 225W frame synchronizer and, if
		it is not in the proper configuration, set up the correct
		configuration for reception.

INPUT:		input is obtained from the frame synchronizer

OUTPUT:		Messages to standard out giving status

RETURNS:	0 on success
		1 of failure, either frame synchronizer is not in remote
		mode or configuration attempts failed.

12/00	D.Lloyd		Development

REFERENCES:	Model 225W Frame Syncronizer Operation Manual, May 2000
		GDP Space Systems.

		Advanced Programming in the UNIX Environment, Stevens

$Log: fs_status.c,v $
Revision 1.1  2003/04/24 16:27:20  dlloyd
Initial checkin of fs_status

**************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>

/* Local function prototypes
----------------------------*/
int  execute_command ( int, char * );
int  get_status( int, const char *, char * );

#define BUFLEN 256
#define TRUE 1
#define FALSE !TRUE


int main( int argc, char *argv[] )
    {
    char	   IPaddress[BUFLEN];	/* IP address of frame sync	*/
    char	   portnum[120];	/* server port on frame sync	*/

    int		   sk;			/* socket descriptor		*/

    struct hostent *hp;			/* host name of address		*/
    struct sockaddr_in skaddr;		/* socket information structure	*/

    char	   buf[BUFLEN];		/* message buffer received	*/
    int            receptionflag;	/* flag: TRUE FALSE		*/
    int            configureflag;	/* flag: TRUE FALSE		*/



    /* Set up IP address and port strings
    -------------------------------------*/
    strcpy( IPaddress, "192.168.10.2" );
    strcpy( portnum, "1024" );

    /* create a socket
    ------------------*/
    if (( sk = socket( PF_INET, SOCK_STREAM, 0 )) < 0 )
	{
	printf("Problem creating socket\n");
	exit(1);
	}

    /* Set the client name and port
    -------------------------------*/
    skaddr.sin_family = AF_INET;
    if (( hp = gethostbyname( IPaddress )) != 0 )
	memcpy( &skaddr.sin_addr.s_addr, hp->h_addr, hp->h_length );
    else
	{
	if ( inet_aton( IPaddress, &skaddr.sin_addr ) == 0 )
	    {
	    printf("Invalid IP address: %s\n",IPaddress);
	    exit(1);
	    }
	}
    skaddr.sin_port = htons(atoi(portnum));

    /* establish the connection
    ---------------------------*/
    if (( connect( sk, (struct sockaddr *) &skaddr, sizeof( skaddr ))) < 0  )
	{
	printf("Problem connecting socket\n");
	exit(1);
	}

    /* Execute a status command
    ---------------------------*/
    if ( get_status( sk, "SSS", buf ))
	{
	printf("Unable to get status\n");
	close( sk );
	return 1;
	}


    /* Check to see if frame sync is set up for remote.
    If it isn't, it's a fatal error.
    ---------------------------------------------------*/
    if (( buf[1] & 0x04 ) == 0x04 )
	printf("\n*** System set for remote. ***\n\n");
    else
	printf("\n##### System NOT set for remote! #####\n\n");

    /* Set the flags
    ----------------*/
    receptionflag=TRUE;
    configureflag=FALSE;

    /* Check the remaining configuration items.  For each
    item: check the status, if it's correct print the message,
    if it isn't try to correct the situation, on error set the
    flag to false, otherwise print the status with a * and
    set the set flag to true.  This code was pulled from a
    more intense diagnostic routine and the other cases were
    removed which is why the switch statements remain.
    -----------------------------------------------------------*/
    switch ( buf[4] )
	{
	case '7' : printf("Format type, FTP: HRPT\n"); break;
	default  : if ( execute_command( sk, "FTP 7"))
		       {
		       printf( "Error setting format type\n" );
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Format type, FTP: HRPT*\n");
		       configureflag = TRUE;
		       }
	}
    switch ( buf[10] )
	{
	case '1' : printf("Data type,   DTP: KLM\n"); break;
	default  : if ( execute_command( sk, "DTP 1"))
		       {
		       printf( "Error setting data type\n" ); 
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Data type,   DTP: KLM*\n");
		       configureflag = TRUE;
		       }
	}
    switch ( buf[9] )
	{
	case '0' : printf("Clock rate,  CRT: Normal 1X\n");break;
	default  : if ( execute_command( sk, "CRT 0"))
		       {
		       printf( "Error setting clock rate\n" ); 
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Clock rate,  CRT: Normal 1X*\n");
		       configureflag = TRUE;
		       }
	}
    switch ( buf[7] )
	{
	case 'N' : printf("Input code,  COD: NRZ-L\n");break;
	default  : if ( execute_command( sk, "COD N"))
		       {
		       printf( "Error setting input code\n" ); 
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Input code,  COD: NRZ-L*\n");
		       configureflag = TRUE;
		       }
	}
    switch ( buf[5] )
	{
	case 'F' : printf("Data direct, DIR: Forward\n");break;
	default  : if ( execute_command( sk, "DIR F"))
		       {
		       printf( "Error setting data direction\n" ); 
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Data direct, DIR: Forward*\n");
		       configureflag = TRUE;
		       }
	}
    switch ( buf[3] )
	{
	case '1' : printf("Input port,  INP: port 1\n");break;
	default  : if ( execute_command( sk, "INP 1"))
		       {
		       printf( "Error setting input port\n" ); 
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Input port,  INP: port 1*\n");
		       configureflag = TRUE;
		       }
	}
    switch ( buf[8] )
	{
	case 'N' : printf("Clk polarity,CPL: Normal\n");break;
	default  : if ( execute_command( sk, "CPL N"))
		       {
		       printf( "Error setting clock polarity\n" ); 
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Clk polarity,CPL: Normal*\n");
		       configureflag = TRUE;
		       }
	}
    switch ( buf[106] )
	{
	case 'E' : printf("Time stamping enabled\n");break;
	default  : if ( execute_command( sk, "TSP E"))
		       {
		       printf( "Error enabling time stamping\n" ); 
		       receptionflag=FALSE;
		       }
		   else
		       {
		       printf("Time stamping enabled*\n");
		       configureflag = TRUE;
		       }
	}

    /* There was no position in the status buffer for
    the video enable.  So always set the video.
    -------------------------------------------------*/
    if ( execute_command( sk, "VID E" ))
	{
	printf( "Error enabling video\n" );
	receptionflag=FALSE;
	}
    else
	printf("Video enabled\n");

    /* If any of the configuration items had to be set, except 
    for the video, print this message.
    ----------------------------------------------------------*/
    if ( configureflag )
	printf("Items marked * were not previously set\n");

    /* If the reception flag is FALSE that means there is
    some item required for reception wasn't set so print the
    error message if it's TRUE then the fs is ready.
    --------------------------------------------------------*/
    if ( receptionflag )
	printf("\n*** Ready for reception. ***\n\n");
    else
	printf( "\n##### Not ready for reception!  #####\n\n");


    /* Close the socket
    -------------------*/
    close( sk );
	    
    return 0;
    }  /* END main */




/* This function sends a command and checks that
the command was successful.
------------------------------------------------*/
int execute_command
    (
    int socket,
    char *command_str
    )
    {
    char message[BUFLEN];	/* message to send to fs		*/
    char response[BUFLEN];	/* response from fs			*/
    int  n;			/* number of bytes sent or received	*/
    int  msg_length;	/* length of message				*/
    int  res_length;	/* length of response				*/
    int *msg_ptr = (int *)message; /* integer pointer to message buffer	*/
    int *res_ptr = (int *)response;/* integer pointer to response buffer*/


    /* Build the message.  Messages consist of a integer message length
    followed by a ':', the message and terminating with a ';'.  Build
    the string by placing a dummy value in the first four bytes of the
    string followed by :msg;.  Next get the message length and plaxe it
    in the first four reserved bytes -- this is an integer value not
    character data.and it doesn't include the 4-byte integer length.
    -------------------------------------------------------------------*/
    sprintf( message, "0000:%s;", command_str );
    msg_length = strlen( message );
    *msg_ptr = msg_length - 4;

    /* Send the message, if the entire message
    isn't accepted, return an error
    ------------------------------------------*/
    n = write( socket, message, msg_length );
    if ( n != msg_length )
	{
	printf("Command: %s not sent\n", command_str);
	return 1;
	}

    /* Read the first four bytes of response.  This is
    the length of the string the fs is returning.  It
    must be converted to an integer.
    --------------------------------------------------*/
    n = read( socket, response, 4 );
    res_length = *res_ptr;

    /* Now read the remainder of the message overwriting
    the message length.  Null terminate the string by
    overwriting the ';' then compare the original 
    command string against the response excluding the ':'
    If the two are the same the fs accepted the command.
    ----------------------------------------------------*/
    n = read( socket, response, res_length );
    response[res_length-1] = '\0';
    if ( strcmp( command_str, &response[1] ) == 0 )
	{
	printf("Error in response: %s\n", response );
	return 1;
	}

    return 0;
    }




/* This functions sends a command and returns the response
----------------------------------------------------------*/
int get_status
    (
    int socket,
    const char *command_str,
    char *response
    )
    {
    char message[BUFLEN];	/* message to send to fs		*/
    int  n;			/* number of bytes sent or received	*/
    int  msg_length;		/* length of message			*/
    int  res_length;		/* length of response			*/
    int *msg_ptr = (int *)message; /* integer pointer to message buffer	*/
    int *res_ptr = (int *)response;/* integer pointer to response buffer*/

    /* Build the message.  Messages consist of a integer message length
    followed by a ':', the message and terminating with a ';'.  Build
    the string by placing a dummy value in the first four bytes of the
    string followed by :msg;.  Next get the message length and plaxe it
    in the first four reserved bytes -- this is an integer value not
    character data.and it doesn't include the 4-byte integer length.
    -------------------------------------------------------------------*/
    sprintf( message, "0000:%s;", command_str );
    msg_length = strlen( message );
    *msg_ptr = msg_length - 4;

    /* Send the message, if the entire message
    isn't accepted, return an error
    ------------------------------------------*/
    n = write( socket, message, msg_length );
    if ( n != msg_length )
	{
	printf("Command: %s not sent\n", command_str);
	return 1;
	}

    /* Read the first four bytes of response.  This is
    the length of the string the fs is returning.  It
    must be converted to an integer.
    --------------------------------------------------*/
    n = read( socket, response, 4 );
    res_length = *res_ptr;

    /* Now read the remainder of the message overwriting the message length.
    Null terminate the beginning of the string.  Compare the ogiginal command
    with the beginning of the response string, excluding the ':'.
    If the two are the same the fs accepted the command.
    ----------------------------------------------------*/
    n = read( socket, response, res_length );
    response[res_length] = '\0';
    if ( strcmp( command_str, &response[1] ) == 0 )
	{
	printf("Error in response: %s\n", response );
	return 1;
	}

    return 0;
    }  /* END get_status */
