/****************************************************************************
FUNCTION:	SEND_COMMAND

PURPOSE:	Send and verify a command to the socket connection with
		the 225W frame synchronizer.

INPUT:		socket file descriptor
		message: the command string to send.  See 225W operation
			 manual section 4.7.3 for valid commands.

OUTPUT:		no return parameters modified
		message is sent to socket

RETURNS:	SUCCESS
		FAIL

REFERENCES:	Model 225W Frame Synchronizer Operation Manual, document
		    number 640-225005, GDP Space Systems, undated.
		HRPT Video Installation Guide For the Frame Synchronizer
		    Model 225W, document number M225W-VIDEO-IG-1.00.00, GDP
		    Space Systems, April 8, 2002
****************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "acquire.h"


int send_command
    (
    int   socket_fd,		/* IN - socket file descriptor		*/
    char *message		/* IN - 225W command message string	*/
    )
    {
    char msg[150];		/* message sent to the socket		*/
    char readbuf[150];		/* response from the socket		*/

    int  buf_len;		/* length of response			*/
    int *bufptr = (int *)readbuf;/* pointer to the response buffer	*/
    int  msg_len;		/* length of message			*/
    int *msgptr = (int *)msg;	/* pointer to the message buffer	*/
    int  num_read;		/* number of bytes read from socket	*/
    int  num_write;		/* number of bytes written to socket	*/
    int n;

    /* Create the command message.  The first four bytes
    are the message length excluding the 4-byte header.
    ----------------------------------------------------*/
    sprintf( msg, "0000:%s;", message );
    msg_len = strlen( msg );
    *msgptr = msg_len - 4;

    /* send the command
    -------------------*/
    num_write = write( socket_fd, msg, msg_len );
    if ( num_write != msg_len )
	return FAIL;

    /* read the 4-byte command response message length 
    --------------------------------------------------*/
    num_read = 0;
    while ( num_read < 4 )
	{
	n = recv( socket_fd, readbuf, 4-num_read, MSG_DONTWAIT );
	if ( n < 0 )
	    break;
	num_read += n;
	}
    if ( num_read != 4 )
	return FAIL;
    buf_len = *bufptr;

    /* read the command response.  The first two bytes
    indicate the block type and comand status respectively.
    as follows:
	block type byte 1
	    0000 = 0 = data block
	    0001 = 1 = data quality block
	    0010 = 2 = command response block
	    0011 = 3 = status reply block
	comand status byte 2
	    bit 0 = command executed
		    0 = executed
		    1 = error
	    bit 1 = CRC check
		    0 = CRC passed
		    1 = CRC failed
	    bit 2 = local/remote selection
		    0 = local
		    1 = remote

    The remainder of the command response is identical to
    the command message if all went well or a '?' is not.

    Add 2 to the message length just in case a <CR><LF>
    is sent.
    -------------------------------------------------------*/
    num_read = 0;
    while ( num_read < buf_len+2 )
	{
	n = recv( socket_fd, readbuf, buf_len-num_read, MSG_DONTWAIT );
	if ( n < 0 )
	    break;
	num_read += n;
	}
    if ( num_read != buf_len )
	return FAIL;
    readbuf[buf_len] = '\0';

    /* byte 0 should be 0x02
    ------------------------*/
    if ( readbuf[0] == 0x02 )
        {
	/* this is a command reply block so check
	the status.  Did the command fail?
	-----------------------------------------*/
	if (( readbuf[1] & 0x01 ) == 1 )
	    {
	    /* Is the 225w in remote?
	    ------------------------*/
	    if (( readbuf[1] & 0x04 ) == 0 )
		{
		write_log(TRUE,"send_command","Error",
		    "Frame sync not in remote");
		}
	    /* or was it a CRC error?
	    ------------------------*/
	    else if (( readbuf[1] & 0x02 ) == 1 )
		write_log(TRUE,"send_command","Error","CRC error");
	    
	    /* in either case return failure
	    --------------------------------*/
	    return FAIL;
	    }  /* END if */
	}  /* END if */
	
    /* Check the returned message.  It
    should be the same as the one sent.
    -----------------------------------*/
    if ( strcmp( &msg[4], &readbuf[2] ) == 0 )
	return SUCCESS;
    else
	return FAIL;
    }  /* END send_command */

