#!/bin/csh -f
#
# pushacqu.csh
#
# Purpose: Copy live acquisition scenes and modsid files to the production
#          system.  Make backup copies of the scene, time stamp and 
#          miscellaneous files.
#
#   Usage: pushacqu.csh <production system name>
# Example: pushacqu.csh edcsgs4
#
# Messages are written to $ADAPSLOG/pushacqu.log
#
#-------------------------------------------------------------------------

# Set up the production LAS environment,
# if is hasn't already been setup.
#---------------------------------------
if (! $?EDCSOFT) then
    setenv EDCSOFT /avhrr/software/las/build
    source $EDCSOFT/env/las/assign/lasenv.csh $EDCSOFT
endif

# Set up the log file
#--------------------
alias wrtlog $ADAPSBASE/write_log
set logfile = $ADAPSLOG/pushacqu
wrtlog $logfile Start pushacqu-start "Starting PUSHACQU"

# Check for command line arguments
#---------------------------------
if ( $#argv != 1 ) then
    echo "pushacqu-args Production system must be specified"
    echo "pushacqu-fatal Fatal error encountered"
    wrtlog $logfile Fatal pushacqu-arg "Must specify production system"
    exit 1
endif

# Clear the time correction file flags
#-------------------------------------
set noaa15 = ""
set noaa16 = ""
set noaa17 = ""
set noaa18 = ""
set noaa19 = ""

# Copy the all image files to the production system
#--------------------------------------------------
cd $ADAPSACQ
set images = `ls *.sf2`
if ( ${?images} ) then
    foreach image ($images)

	# Copy with a prefix so that ingest doesn't 
	# grab it until the copy is complete.
	#------------------------------------------
	rcp $image ${1}:'$ADAPSACQ'/LXT$image

	# On error move the file to ADAPSERR
	if ( $status ) then
	    mv $image $ADAPSERR/.
	    wrtlog $logfile $image:r pushacqu-err "File moved to ADAPSERR"
	# Otherwise remove the prefix
	else
	    rsh $1 mv '$ADAPSACQ'/LXT$image '$ADAPSACQ'/$image
	endif

	# Move the local image to ADAPSARC as a temporary backup copy.
	# It will be removed by a clean up script at a later date.
	#-------------------------------------------------------------
	mv $image $ADAPSARC/sfl/sf2/.
	wrtlog $logfile $image:r pushacqu-copy "Scene copied to $1 acquire"
	# Set the time correction flag so that the appropriate time
	# correction file will be copied to the production system.
	#----------------------------------------------------------
	set noaa15 = `echo $image | awk /ah15/'{ print "noaa15.timcor"}'`
	set noaa16 = `echo $image | awk /ah16/'{ print "noaa16.timcor"}'`
	set noaa17 = `echo $image | awk /ah17/'{ print "noaa17.timcor"}'`
	set noaa18 = `echo $image | awk /ah18/'{ print "noaa18.timcor"}'`
        set noaa19 = `echo $image | awk /ah19/'{ print "noaa19.timcor"}'`

    # Set this flag to indicate that something was done
    set didstuff = "didstuff"
    end
endif


# Copy any time correction file that's needed
#--------------------------------------------
if ( $noaa15 == "noaa15.timcor" ) then
    rcp $ADAPSTABLES/$noaa15 ${1}:'$ADAPSTABLES'/noaa15.timcor
    wrtlog $logfile $noaa15 pushacqu-timecor "File copied to $1 adapstables"
endif
if ( $noaa16 == "noaa16.timcor" ) then
    rcp $ADAPSTABLES/$noaa16 ${1}:'$ADAPSTABLES'/noaa16.timcor
    wrtlog $logfile $noaa16 pushacqu-timecor "File copied to $1 adapstables"
endif
if ( $noaa17 == "noaa17.timcor" ) then
    rcp $ADAPSTABLES/$noaa17 ${1}:'$ADAPSTABLES'/noaa17.timcor
    wrtlog $logfile $noaa17 pushacqu-timecor "File copied to $1 adapstables"
endif
if ( $noaa18 == "noaa18.timcor" ) then
    rcp $ADAPSTABLES/$noaa18 ${1}:'$ADAPSTABLES'/noaa18.timcor
    wrtlog $logfile $noaa18 pushacqu-timecor "File copied to $1 adapstables"
endif
if ( $noaa19 == "noaa19.timcor" ) then
    rcp $ADAPSTABLES/$noaa19 ${1}:'$ADAPSTABLES'/noaa19.timcor
    wrtlog $logfile $noaa19 pushacqu-timecor "File copied to $1 adapstables"
endif



# Copy all modsid files to the production system Modsid
# files are used by schedit to update the master schedule.
#---------------------------------------------------------
cd $ADAPSING
set modsids = `ls modsid*`
if ( ${?modsids} ) then
    foreach modsid ($modsids)
	rcp $modsid ${1}:'$ADAPSTABLES'/$modsid
	if ( $status ) then
	    mv $modsid $ADAPSERR/.
	    set text = "$modsid:r copied to ADAPSERR"
	    wrtlog $logfile modsid pushacqu-err $text
	else
	    set text = "$modsid:r copied to $1 adapstables"
	    wrtlog $logfile modsid pushacqu-modsid $text
	endif
	rm -f $modsid
    set didstuff = "didstuff"
    end
endif


# Make a local backup of any time stamp file
#-------------------------------------------
cd $ADAPSACQ
set tstamps = `ls *.tstamp`
if ( ${?tstamps} ) then
    foreach tstamp ($tstamps)
	mv $tstamp $ADAPSARC/tstamp/.
	wrtlog $logfile $tstamp:r pushacqu-tstamp "File moved to ADAPSARC"
    set didstuff = "didstuff"
    end
endif

# Make a local backup of any miscellaneous file
#----------------------------------------------
set miscs = `ls *.misc`
if ( ${?miscs} ) then
    foreach misc ($miscs)
	mv $misc $ADAPSARC/misc/.
	wrtlog $logfile $misc:r pushacqu-misc "File moved to ADAPSARC"
    set didstuff = "didstuff"
    end
endif

# Write an appropriate completion message
#----------------------------------------
if ( $?didstuff ) then
    wrtlog $logfile Complete pushacqu-cmplt "PUSHACQU completed successfully"
else
    wrtlog $logfile Complete pushacqu-cmplt "PUSHACQU had nothing to do"
endif
