/*************************************************************************
FUNCTION:	acquman

PURPOSE:	The acquisition manager coordinates AVHRR acquisition
		processes.  The main function tries to establish
		communication with the moving window display and enters
		a loop.  Main control remains in this loop until the
		user places a stop file in $ADAPSTABLES.  The loop
		checks for a stop file, checks the alarm, either tries
		to establish communication or checks the socket for
		usability and checks the schedule.

INPUT:		Reads from 	$ADAPSTABLES/master.sch

OUTPUT:		messages logged in $ADAPSLOG/acquman.log

RETURNS:	0 successful completion
		-1 fatal error

PROJECT:	ADAPS AVHRR Acquisition System

*************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "acquman.h"

/* Signal handler for broken pipe associated with socket
--------------------------------------------------------*/
void sighandler( int signal_number );

/* Declared static for signal handler
-------------------------------------*/
static int shoacqu_socket = -1;	/* moving window socket descriptor	*/ 


int main( void )
    {
    int start_stop_flag = START;/* flag indicating alarm type:		*/
    				/* START = start acquisition alarm	*/
				/* STOP = stop acquisition alarm	*/
    int i;			/* loop control variable		*/
    int forever = TRUE;		/* loop control to stifle compiler warn	*/

    char day_night_flag[10];	/* flag indicating a day or night pass	*/
    char sceneid[17];		/* scene identifier			*/
    char local_start_time[25];	/* local start time: -next HH:MM:SS XM	*/
    char local_stop_time[25];	/* local stop time:  HH:MM:SS XM	*/
    char adapstables[STRINGLEN];/* ADAPSTABLES environment variable	*/
    char adapsing[STRINGLEN];	/* ADAPSING environment variable        */
    char adapsacq[STRINGLEN];	/* Incomming directory environment      */
    char adapserr[STRINGLEN];	/* ADAPSERR environment variable        */
    char message[STRINGLEN];	/* string for sending messages to log	*/

    pid_t acquire_id = 0;	/* acquire child process id		*/
    pid_t timecorr_id = 0;	/* time correction child process id	*/
    pid_t status;

    				/* BOT is 1/1/1986 00:00:00 UTC		*/
    time_t curr_time;		/* current UTC time in secs since BOT	*/
    time_t start_time;		/* UTC start time in seconds since BOT	*/
    time_t stop_time;		/* UTC stop time in seconds since BOT	*/
    time_t alarm_time;		/* UTC alarm time in seconds since BOT	*/
    time_t seconds2alarm;	/* number of seconds until alarm expires*/
    time_t acu_alarm;		/* time to put ACU in program auto	*/
    time_t seconds2acu_alarm;	/* number of seconds until acu_alarm	*/


    /* Setup the signal handler
    ---------------------------*/
    signal( SIGPIPE, sighandler );


    /* Get all the environment variables
    ------------------------------------*/
    if ( get_environment( adapstables, adapsing, adapsacq, adapserr ))
        {
	write_log(TRUE,"main","Error","Couldn't get environment variables");
	return -1;
	}


    /* Write a startup message
    --------------------------*/
    write_log(FALSE,"acquman","info","ACQUMAN started");
    

    /* Get the next pending scene information and
    set the start alarm for the acquisition
    ---------------------------------------------*/
    get_next_scene( adapstables, sceneid, day_night_flag, local_start_time,
        local_stop_time, &start_time, &stop_time, &curr_time );
    if ( start_time != TIME_ERROR )
	{
	alarm_time = start_time;
	acu_alarm  = start_time - ACUALARMOFFSET;
	}
    else
	{
	/* If there was an error getting the next scene time,
	push the alarm ahead so that an inadvertent start is
	not forced.  The alarm must be set beyond one normal
	sleep interval to prevent a start that's why two times
	the interval is added to the current time.
	-----------------------------------------------------*/
        alarm_time = curr_time + NAPTIME * 2;
	acu_alarm  = curr_time + ACUALARMOFFSET * 2;
	write_log(FALSE,"acquman","Error", "Unable to get next pending scene");
	}


    /* Attempt to set up communication with SHOACQU.  ACQUMAN is
    generally run at startup and there won't be an X-window session
    for SHOACQU.  However, if the operators restart ACQUMAN using
    the restart script SHOACQU will be executed immediately.  We
    want an immediate display of the next acquisition time as
    feedback to the operator.  If we wait until the while loop to
    do this SHOACQU may not have tried to connect to the socket and
    it will be NAPTIME seconds until the next time is displayed.
    --------------------------------------------------------------*/
    for ( i = 0; i < 10; i++ )
        {
	shoacqu_socket = create_socket( adapstables );
	if ( shoacqu_socket >= 0 )
	    {
	    /* Send it the next scene acquisition time
	    ------------------------------------------*/
	    send_message( shoacqu_socket, local_start_time );
	    break;
	    }
	else
	    sleep(1);
	}  /* END for i */

	
    /* Main processing loop
    -----------------------*/
    while ( forever )
        {
	/* Are we supposed to stop?  Check
	for a stop file in $ADAPSTABLES.
	----------------------------------*/
	if ( check_for_stop( adapstables ) == TRUE )
	    break;
	
	/* If the alarm is about to expire,
	call the appropriate function.
	-----------------------------------*/
	seconds2alarm = check_alarm( alarm_time );
	seconds2acu_alarm = check_alarm( acu_alarm );
	if ( seconds2alarm <= MINSLEEPINT )
	    {
	    if ( start_stop_flag == START )
		{
		/* Set the next alarm to the scene stop time */
		alarm_time = stop_time;
		seconds2alarm = check_alarm( alarm_time );

		/* Begin the acquisition */
	        start_stop_flag = start_acquisition( adapsacq, shoacqu_socket,
		    sceneid, day_night_flag, local_stop_time, &acquire_id );

		/* Add the acquire process ID to the list */
		status = wait_for_children( acquire_id );
		if ( status == -1 )
		    {
		    sprintf(message,"Couldn't add PID %d to list",acquire_id);
		    write_log(FALSE,"acquman","Error",message);
		    }
		}
	    else
		{
	        start_stop_flag = stop_acquisition( adapsacq, adapsing,
		    adapserr, acquire_id, sceneid, &timecorr_id );

		/* Add the timecorr process ID to the list */
		status = wait_for_children( timecorr_id );
		if ( status == -1 )
		    {
		    sprintf(message,"Couldn't add PID %d to list",timecorr_id);
		    write_log(FALSE,"acquman","Error",message);
		    }
		}
	    
	    }  /* END if seconds2alarm */


	/* Check the status of the two children processes. 
	ACQUIRE and TIMECORR were started by start_acquisition
	and stop_acquisition respectively.  To prevent zombie
	processes wait for them to terminate, without blocking,
	and clear their process ID's.  A value greater than
	0 the a process ID of a process that has terminated.
	-------------------------------------------------------*/
	status = wait_for_children( 0 );
	if ( status )
	    {
	    if ( status == acquire_id )
		acquire_id = 0;
	    else if ( status == timecorr_id )
		timecorr_id = 0;
	    else
	        {
		sprintf(message,"Returned Process ID was: %d",status);
		write_log(FALSE,"acquman","Error",message);
		}
	    }


	/* Check the schedule again and, if the entry is
	different, set a new alarm and inform SHOACQU.
	Only check when not doing a live acquisition.
	-----------------------------------------------*/
	if ( start_stop_flag == START )
	    {
	    get_next_scene( adapstables, sceneid, day_night_flag,
	        local_start_time, local_stop_time, &start_time, &stop_time,
		&curr_time );
	    if ( start_time != alarm_time )
		{
		if ( start_time != TIME_ERROR )
		    {
		    alarm_time = start_time;
		    seconds2alarm = check_alarm( alarm_time );

		    /* The ACU needs some time to get going so subtract
		    a few minutes from the start_time.
		    ---------------------------------------------------*/
		    acu_alarm  = start_time - ACUALARMOFFSET;
		    seconds2acu_alarm = check_alarm( acu_alarm );

		    send_message( shoacqu_socket, local_start_time );
		    }
		else if ( check_alarm( alarm_time ) > NAPTIME * 2 )
		    {
		    /* Keep the current alarm for now */
		    write_log(FALSE,"acquman","Error",
		        "Unable to get next pending scene, keeping current");
		    }
		else
		    {
		    /* If there was an error getting the next scene time,
		    push the alarm ahead so that an inadvertent start is
		    not forced.  The alarm must be set beyond one normal
		    sleep interval to prevent a start that's why two times
		    the interval is added to the current time.
		    -----------------------------------------------------*/
		    alarm_time = curr_time + NAPTIME * 2;
		    acu_alarm = curr_time + ACUALARMOFFSET * 2;
		    write_log(FALSE,"acquman","Error",
		        "Unable to get next pending scene");
		    }
		}  /* END if start_time != alarm_time */
	    }  /* END if start_stop_flag */
	

	/* If there is no connection with SHOACQU, see if we can get one
	----------------------------------------------------------------*/
	if ( shoacqu_socket < 0 )
	    {
	    /* On success send it the next scheduled time */
	    shoacqu_socket = create_socket( adapstables );
	    if ( shoacqu_socket  >= 0 )
	        send_message( shoacqu_socket, local_start_time );
	    }

	/* Otherwise make sure the connection is still valid
	----------------------------------------------------*/
	else
	    {
	    /* If an error is returned the connection is lost */
	    if ( poll_socket( shoacqu_socket ))
		{
		close( shoacqu_socket );
	        shoacqu_socket = -1;
		}
	    }


	/* Write a log message
	----------------------*/
	if ( shoacqu_socket >= 0 )
	    write_log(FALSE,"main","while-loop","All is well");
	else
	    write_log(FALSE,"main","while-loop","SHOACQU is not running");
	

	/* If we are within ACUALARMOFFSET seconds of the
	acu_alarm tell the ACU to go into program auto.
	-------------------------------------------------*/
	if ( seconds2acu_alarm == 0 )
	    {
	    char tempsatid[3];
	    int sat,i,stat=1;

	    write_log(FALSE,"acquman","Info","Telling ACU to go");
	    /* Attempt to put the acu into program mode.
	    Give it at most five tries before giving up.
	    -------------------------------------------*/
	    /* Turn on track for new acu */
	    strncpy( tempsatid, &sceneid[2], 2 );
	    sscanf( tempsatid, "%d", &sat );
	    for ( i = 0; i < 5; i++ )
	        {
		stat = acu_control( TRACK, sat );
		if ( stat == 0 )
		    break;
		sleep(1);
		}
	    if ( stat )
	        write_log(FALSE,"acquman","Error","Could not set NEW ACU");

	    /* Reset the acu_alarm to five times the offset in the
	    future so this code isn't repeated until the next pass.
	    -------------------------------------------------------*/
	    acu_alarm += ACUALARMOFFSET * 5;

	    /* for now tell the receiver to change as well
	    ----------------------------------------------*/
	    set_rcvrfreq( sat );
	    }  /* END if seconds2acu_alarm */


	/* Sleep until the alarm or NAPTIME seconds, whichever is less.
	However, if the alarm is less than the minimum sleep interval,
	don't sleep at all.
	--------------------------------------------------------------*/
	if ( seconds2alarm > MINSLEEPINT && seconds2alarm < NAPTIME )
	    sleep( seconds2alarm );
        else if ( seconds2acu_alarm > MINSLEEPINT &&
                  seconds2acu_alarm < NAPTIME )
            {
            sleep( seconds2acu_alarm );
            }
	else if ( seconds2alarm > MINSLEEPINT )
	    sleep( NAPTIME );
	}  /* END while */
    
    return 0;
    }  /* END main */


/* Handle broken pipes by closing
the moving window display socket.
---------------------------------*/
void sighandler( int signal_number )
    {
    /* Reset the signal handler
    ---------------------------*/
    signal( SIGPIPE, sighandler );

    if ( signal_number == SIGPIPE )
	{
        close( shoacqu_socket );
	shoacqu_socket = -1;
	}

    return;
    }  /* END sighandler */
