/*************************************************************************
FUNCTION:       create_socket

PURPOSE:        Create a server socket and attempt to connect to a client.

INPUT:          none

OUTPUT:         none

RETURNS:        client socket descriptor

PROJECT:        ADAPS AVHRR Acquisition System

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "acquman.h"


int create_socket
    (
    const char *adapstables     /* IN - ADAPSTABLES environment variable*/
    )
    {

    static int server_socket = 0;	/* server socket descriptor	*/
    struct sockaddr_in server_address;	/* server address structure	*/
    struct sockaddr_in client_address;	/* client address structure	*/
    int    server_length;	/* length of server address structure	*/
    int    client_length;	/* length of client address structure	*/
    int    client_socket;	/* socket descriptor of client		*/
    char   message[STRINGLEN];	/* log message string			*/


    /* If the server socket has not been established, create it
    -----------------------------------------------------------*/
    if ( !server_socket )
	{
	/* create an endpoint socket for communication
	----------------------------------------------*/
	server_socket = socket( AF_INET, SOCK_STREAM, 0 );
	if ( server_socket < 0 )
	    {
	    write_log(FALSE,"create_socket","socket-error",
		"Problem creating socket");
	    return -1;
	    }

	/* Set the socket to non-blocking.  The server socket
	must be non-blocking in case there are no client sockets
	trying to connect.  If this is the case the accept call
	later will block until there is a client trying to connect.
	Acquman must not block in the acept call.
	-----------------------------------------------------------*/
	if ( fcntl( server_socket, F_SETFL, O_NONBLOCK ) != 0 )
	    {
	    write_log(FALSE,"create_socket","fcntl-error",
		"couldn't set to non-block");
	    close( server_socket );
	    server_socket = 0;
	    return -1;
	    }

	/* Set the other socket values
	------------------------------*/
	server_address.sin_family = AF_INET;
	server_address.sin_addr.s_addr = htonl(INADDR_ANY);
	server_address.sin_port = htons( 0 );

	/* Bind a name to the socket created above
	------------------------------------------*/
	if ( bind(server_socket, (struct sockaddr *) &server_address,
		sizeof(server_address)) < 0 )
	    {
	    write_log( FALSE,"create_socket","bind-error",
	        "Problem binding socket");
	    close( server_socket );
	    server_socket = 0;
	    return -1;
	    }

	/* Get the current name for the socket created above.
	The name is necessary to know the port number assigned.
	------------------------------------------------------*/
	client_length = sizeof( server_address );
	if ( getsockname( server_socket, (struct sockaddr *) &server_address,
		&client_length) < 0 )
	    {
	    write_log(FALSE,"create_socket","Error",
	        "Error calling getsockname");
	    close( server_socket );
	    server_socket = 0;
	    return -1;
	    }

	/* Save the port number in a known location
	-------------------------------------------*/
	sprintf( message, "Server port number: %d",
	    (int)ntohs( server_address.sin_port ));
	write_log(FALSE,"create_socket","Information",message);
	save_port( adapstables, (int)ntohs( server_address.sin_port ));

	/* Allow the socket to accept incoming connections
	--------------------------------------------------*/
	if ( listen( server_socket, 5 ) < 0 )
	    {
	    write_log(FALSE,"create_socket","listen-error",
	        "Error calling listen");
	    close( server_socket );
	    server_socket = 0;
	    return -1;
	    }
	}  /* END if !server_socket */


    /* Accept a connection on the socket
    --------------------------------------*/
    client_socket = accept( server_socket, (struct sockaddr*)&client_address,
                            &server_length );
    if ( client_socket < 0 )
	{
	write_log(FALSE,"create_socket","accept-error","No socket accepted");
	return -1;
	}

printf("client socket number: %d\n",client_socket);
    return client_socket;
    }  /* END create_socket */
