/******************************************************************************
FUNCTION:	get_next_scene

PURPOSE:	Read the master schedule and determine the next pending
		EROS scene to be acquired.  In this context 'Pending' means
		the stop time is in the future.  And, since the schedule
		is ordered by start time, the first stop time that is
		in the future with an 'E' in the station column is the one
		that is returned.

INPUT:		ADAPSTABLES environment variable
		reads data from $ADAPSTABLES/master.sch

OUTPUT:		sceneid: string AHSSMMDDYYhhmmss
		day_night_flag: string "-dnflag D" = day "-dnflag N" = night
		local_start_time: string "-next hh:mm:ss XM"
		start_time All times in UTC seconds since 1/1/86 00:00:00
		stop_time

RETURNS:	none

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include "adapsutil.h"
#include "acquman.h"


void get_next_scene
    (
    const char *adapstables,    /* IN - ADAPSTABLES environment variable     */
    char *sceneid,              /* OUT - scene identifier                    */
    char *day_night_flag,       /* OUT - flag indicating a day or night pass */
    char *local_start_time,     /* OUT - local start time as HH:MM:SS A/PM   */
    char *local_stop_time,      /* OUT - local stop time as HH:MM:SS A/PM    */
    time_t *start_time,         /* OUT - UTC start time in seconds since BOT */
    time_t *stop_time,          /* OUT - UTC stop time in seconds since BOT  */
    time_t *curr_time           /* OUT - current UTC time in secs since BOT  */
    )
    {
    char schedule_name[STRINGLEN];/* master schedule file name		*/
    char curr_station;		/* current record station ID		*/
    char curr_dnflag;		/* current record day/night flag	*/
    char ampm;			/* AM/PM indicator			*/
    char curr_sceneid[17];	/* current record scene ID		*/
    char cmd[STRINGLEN];

    FILE *schedule;		/* master scedule file descriptor	*/

    time_t now;		/* local time seconds since 1/1/70 00:00:00 UTC	*/
    struct tm *time_struct;	/* local time structure			*/

    long timearray[3];		/* time (hour, min, sec) for makesecs	*/
    long datearray[3];		/* date (mon, day, year) for makesecs	*/
    long local_time;		/* local time in seconds since BOT	*/
    long utc_time;		/* UTC time in seconds since BOT	*/
    long curr_start;		/* current record start time		*/
    long curr_stop;		/* current record stop time		*/
    long seconds2utc;		/* number of seconds from local to UTC	*/

    int fd;			/* file descriptor for locking schedule	*/
    int num_records;		/* number of record entries in schedule	*/
    int record;			/* index of current record		*/
    int local_start_hour;	/* start hour in local time		*/
    int local_start_min;	/* starting minute			*/
    int local_start_sec;	/* starting second			*/


    /* Make a copy of the master schedule to work on
    ------------------------------------------------*/
    sprintf( cmd, "%s/mastercp", getenv( "LASTOOLS" ));
    if ( system( cmd ))
    {
        write_log( FALSE,"get_next_scene","copy-error",
	    "Could not copy schedule");
        return;
    }

    /* NULL the string output parameters and set the times to -1
    ------------------------------------------------------------*/
    sceneid[0] = day_night_flag[0] = local_start_time[0] = '\0';
    local_stop_time[0] = '\0';
    *start_time = *stop_time = TIME_ERROR;


    /* Get the local and UTC time structures
    ----------------------------------------*/
    now = time( NULL );

    time_struct = localtime( &now );
    timearray[0] = time_struct->tm_hour;
    timearray[1] = time_struct->tm_min;
    timearray[2] = time_struct->tm_sec;
    datearray[0] = time_struct->tm_mon + 1;   /* make the month 1 relative */
    datearray[1] = time_struct->tm_mday;
    datearray[2] = time_struct->tm_year;
    c_makesecs( datearray, timearray, &local_time );

    time_struct = gmtime( &now );
    timearray[0] = time_struct->tm_hour;
    timearray[1] = time_struct->tm_min;
    timearray[2] = time_struct->tm_sec;
    datearray[0] = time_struct->tm_mon + 1;   /* make the month 1 relative */
    datearray[1] = time_struct->tm_mday;
    datearray[2] = time_struct->tm_year;
    c_makesecs( datearray, timearray, &utc_time );
    *curr_time = (time_t)utc_time;


    /* Compute the difference between local and UTC
    -----------------------------------------------*/
    seconds2utc = utc_time - local_time;


    /* Get the master schedule file name
    ------------------------------------*/
    strcpy( schedule_name, adapstables );
    strcat( schedule_name, "/master.sch.copy" );

    /* Open and lock the schedule to prevent the schedule
    from being updated while reading.  This file descriptor
    is only used for locking the file not for reading (I
    didn't want to change the rest of the tested code).  If
    either the open or the lock are unsuccessful, return to
    try again later.
    -------------------------------------------------------*/
    fd = open( schedule_name, O_RDONLY|O_NONBLOCK );
    if ( fd < 0 )
	{
        write_log( FALSE,"get_next_scene","open-error",
	    "Could not open schedule");
	return;
	}
    if ( flock( fd, LOCK_EX|LOCK_NB ) == -1 )
	{
        write_log( FALSE,"get_next_scene","lock-error",
	    "Could not lock schedule");
	if ( close( fd ) < 0 )
	    {
	    write_log(FALSE,"get_next_scene","close-error",
	        "Could not close master schedule");
	    }
	return;
	}


    /* Open the master schedule
    ---------------------------*/
    schedule = fopen( schedule_name, "r" );
    if ( schedule == NULL )
        {
	write_log(FALSE,"get_next_scene","fopen-error",
	    "Cannot open master schedule");
	return;
	}

    /* Get the number of records
    ----------------------------*/
    if ( fscanf( schedule, "%d\n", &num_records ) != 1 )
        {
	write_log(FALSE,"get_next_scene","fscanf-error",
	    "Could not get number of schedule records");
	if ( fclose( schedule ) == EOF )
	    write_log(FALSE,"get_next_scene","fclose-error",
		"Could not close master schedule");
	return;
	}

    /* Scan each record in the schedule until the next pending
    scene is found or all the records have been read.
    ----------------------------------------------------------*/
    for ( record = 0; record < num_records; record++ )
        {
	/* read the next record */
	if ( fscanf( schedule,
		"%s %*s %ld %ld %*d %*d %*d %*s %*d %*s %c %c %*s %*s\n",
		curr_sceneid, &curr_start, &curr_stop, &curr_dnflag,
		&curr_station ) != 5 )
	    {
	    write_log(FALSE,"get_next_scene","fscanf-error",
	        "Scan count of schedule record is incorrect");
	    if ( fclose( schedule ) == EOF )
	        write_log(FALSE,"get_next_scene","fclose-error",
		    "Could not close master schedule");
	    return;
	    }  /* END if fscanf */
	
	/* If the station is "E" we have an EROS scene.  Further, if
	the stop time is later than the current UTC time, we have
	found the next scene to acquire.  Adding 120 seconds to the
	current time is so that pending scenes that are nearly
	complete will be ignored.  120 seconds is the minimum scene
	length that will be archived.
	-----------------------------------------------------------*/
	if (( curr_station == 'E' ) && ( curr_stop >= utc_time + 120 ))
	    {
            char sss[3];
	    /* Copy the scene ID and lower case the AH */
	    strcpy( sceneid, curr_sceneid );
	    sceneid[0] = 'a';
	    sceneid[1] = 'h';
            sss[0] = sceneid[2];
            sss[1] = sceneid[3];
            sss[2] = '\0';

	    /* Copy the day.night flag */
	    sprintf( day_night_flag, "-dnflag %c", curr_dnflag );

	    /* Copy the start and stop times */
	    *start_time = (time_t)curr_start;
	    *stop_time = (time_t)curr_stop;

	    /* Create the local start time string */
	    local_start_sec = ( curr_start - seconds2utc ) % 86400;
	    local_start_hour = local_start_sec / 3600;
	    local_start_sec -= local_start_hour * 3600;
	    local_start_min = local_start_sec / 60;
	    local_start_sec -= local_start_min * 60;
	    if ( local_start_hour > 11 )
	        {
		ampm = 'P';
		if ( local_start_hour > 12 )
		    local_start_hour -= 12;
		}
	    else
	        ampm = 'A';
	    sprintf( local_start_time, "-next NOAA %s %d:%02d:%02d %cM", sss,
	        local_start_hour, local_start_min, local_start_sec, ampm );

	    /* Create the local stop time string */
	    local_start_sec = ( curr_stop - seconds2utc ) % 86400;
	    local_start_hour = local_start_sec / 3600;
	    local_start_sec -= local_start_hour * 3600;
	    local_start_min = local_start_sec / 60;
	    local_start_sec -= local_start_min * 60;
	    if ( local_start_hour > 11 )
	        {
		ampm = 'P';
		if ( local_start_hour > 12 )
		    local_start_hour -= 12;
		}
	    else
	        ampm = 'A';
	    sprintf( local_stop_time, "%d:%02d:%02d %cM",
	        local_start_hour, local_start_min, local_start_sec, ampm );

	    break;
	    }  /* END if curr_station ... */
	}  /* END for record ... */

    if ( fclose( schedule ) == EOF )
        {
	write_log(FALSE,"get_next_scene","fclose-error",
	    "Could not close master schedule");
	}

    /* Close and unlock the schedule.  The
    close implicitly unlocks the file.
    --------------------------------------*/
    if ( fd >= 0 )
	{
        if ( close( fd ) < 0 )
	    write_log(FALSE,"get_next_scene","close-error",
	        "Could not close master schedule");
	}
    }  /* END get_next_scene */
