/*************************************************************************
FUNCTION:	set_rcvrfreq

PURPOSE:	Set the frequency of the receiver by recalling the channel
		information.  The recall command is 'XN' where N is the
		channel to be recalled.

INPUT:		2-digit satellite ID: 16, 17 etc.

OUTPUT:		character string "XN" where N is a single digit number
		of the receiver channel to recall

RETURNS:	0 on SUCCESS
		-1 on FAILURE

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <arpa/inet.h>
#include <ctype.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <termios.h>
#include <unistd.h>

#include "acquman.h"

static int execute_command( int, const char *, char * );
#define BUFLEN 1024

int set_rcvrfreq
(
    int satellite		/* IN - satellite number: 16, 17 etc.	*/
)
{
    char IPaddress[25];         /* Receiver IP address                  */
    char mode_change[BUFLEN];   /* Command string                       */
    char freq_change[BUFLEN];   /* Command string                       */
    char dcoff_change[BUFLEN];   /* Command string                       */
    char portnum[10];           /* Receiver port number                 */
    char response[BUFLEN];      /* Message returned by receiver         */

    int  sk;                    /* Socket descriptor                    */

    struct hostent     *hp;     /* host name of address                 */
    struct sockaddr_in skaddr;  /* socket information structure         */
    char message[STRINGLEN];	/* log message string			*/
    char the_mode[25];
    char the_sat[25];


    /* Set up IP address and port strings
    -------------------------------------*/
    strcpy( IPaddress, "192.168.10.4" );
    strcpy( portnum, "10001" );

    /* Create a socket
    ------------------*/
    if (( sk = socket( PF_INET, SOCK_STREAM, 0 )) < 0 )
    {
	write_log(FALSE,"set_rcvrfreq","Error","Creating socket");
        return 1;
    }

    /* Set the client name and port
    -------------------------------*/
    skaddr.sin_family = AF_INET;
    if (( hp = gethostbyname( IPaddress )) != 0 )
        memcpy( &skaddr.sin_addr.s_addr, hp->h_addr, hp->h_length );
    else
    {
        if ( inet_aton( IPaddress, &skaddr.sin_addr ) == 0 )
        {
            sprintf( message, "Invalid IP address: %s", IPaddress );
	    write_log(FALSE,"set_rcvrfreq","Error",message);
            return 1;
        }
    }
    skaddr.sin_port = htons(atoi(portnum));


    /* establish the connection
    ---------------------------*/
    if (( connect( sk, (struct sockaddr *) &skaddr, sizeof( skaddr ))) < 0  )
    {
	write_log(FALSE,"set_rcvrfreq","Error","Connecting socket");
        return 1;
    }

    /* Set the satellite frequency and mode.  Note: there may be other
    things that need to be set when switching between NOAA and METOP.
    ------------------------------------------------------------------*/
    switch ( satellite )
    {
        case 2 :
	    strcpy( freq_change, "f1701.300" );
	    strcpy( dcoff_change, "o1565.300" );
	    strcpy( mode_change, "m2" );
	    strcpy( the_mode, "DMSP RTD" );
	    strcpy( the_sat, "METOP A" );
	    break;
        case 17 :
	    strcpy( freq_change, "f1707.000" );
	    strcpy( dcoff_change, "o1565.000" );
	    strcpy( mode_change, "m3" );
	    strcpy( the_mode, "HRPT" );
	    sprintf( the_sat, "NOAA %d", satellite );
	    break;
        case 18 :
	    strcpy( freq_change, "f1698.000" );
	    strcpy( dcoff_change, "o1565.000" );
	    strcpy( mode_change, "m3" );
	    strcpy( the_mode, "HRPT" );
	    sprintf( the_sat, "NOAA %d", satellite );
	    break;
         case 19 :
            strcpy( freq_change, "f1698.000" );
            strcpy( dcoff_change, "o1565.000" );
            strcpy( mode_change, "m3" );
            strcpy( the_mode, "HRPT" );
            sprintf( the_sat, "NOAA %d", satellite );
            break;
	default :
	    return 1;
    }

/* TODO: Hack added to look for an alternate frequency.
-------------------------------------------------------*/
{
    FILE *inf;
    int  stl;
    char freq[10];
    char dcoff[10];
    char msg[80];

    inf = fopen("/avhrr/software/las/build/env/adaps/tables/alt_freq.txt","r");
    if ( inf )
    {
        while ( fscanf( inf, "%d %s %s\n", &stl, dcoff, freq ) == 3 )
        {
            if ( stl == satellite )
            {
                sprintf( freq_change, "f%s", freq );
                sprintf( dcoff_change, "o%s", dcoff );
                sprintf( msg, "Changing to alternate frequency %s", freq);
                write_log(FALSE,"set_rcvrfreq","Info", msg );
            }
        }  /* END while */
    }  /* END else */
}  /* END TODO: Hack */


    /* We want the receiver in brief mode so we know what to expect.
    However, if it's already in brief mode and you send the 'b' command,
    it doesn't return anything.  Since the socket is initially in a
    blocking mode it will hang.  The same isn't true of putting it in
    verbose mode -- it always returns something.  So put it in verbose
    mode then in brief mode.
    -------------------------------------------------------------------*/
    if ( execute_command( sk, "v", response ))
    {
	write_log(FALSE,"set_rcvrfreq","Error","Changing to verbose mode");
    }
    if ( strncmp( response, "verbose mode", 12 )  != 0 &&
         strncmp( response, "vverbose mod", 12 )  != 0 )
    {
        sprintf( message, "Unexpected response: '%s'", response );
	write_log(FALSE,"set_rcvrfreq","Error",message);
    }
    /* Now put it in brief mode.
    ----------------------------*/
    if ( execute_command( sk, "b", response ))
    {
	write_log(FALSE,"set_rcvrfreq","Error","Changing to brief mode");
    }
    if ( strcmp( response, "b" ) != 0 )
    {
        sprintf( message, "Unexpected response: '%s'", response );
	write_log(FALSE,"set_rcvrfreq","Error",message);
    }

    /* Execute the command, in both cases the single
    character 'k' should be returned.
    ------------------------------------------------*/
    if ( execute_command( sk, mode_change, response ))
    {
	write_log(FALSE,"set_rcvrfreq","Error","Changing mode");
    }
    if ( strcmp( response, "k" ) != 0 )
    {
        sprintf( message, "Unexpected response: '%s'", response );
	write_log(FALSE,"set_rcvrfreq","Error",message);
    }
    else
    {
        sprintf( message, "Changed mode to: %s", the_mode );
	write_log(FALSE,"set_rcvrfreq","Information",message);
    }
    if ( execute_command( sk, dcoff_change, response ))
    {
	write_log(FALSE,"set_rcvrfreq","Error",
            "Changing down converter offset");
    }
    if ( strcmp( response, "k" ) != 0 )
	write_log(FALSE,"set_rcvrfreq","Error",
            "Changing down converter offset");
    else
    {
        sprintf( message,
            "Changed down converter offset to %s", dcoff_change );
	write_log(FALSE,"set_rcvrfreq","Information",message);
    }
    if ( execute_command( sk, freq_change, response ))
    {
	write_log(FALSE,"set_rcvrfreq","Error","Changing frequency");
    }
    if ( strcmp( response, "k" ) != 0 )
	write_log(FALSE,"set_rcvrfreq","Error","Changing frequency");
    else
    {
        sprintf( message, "For %s changed frequency to %s", the_sat,
            freq_change );
	write_log(FALSE,"set_rcvrfreq","Information",message);
    }

    close( sk );
    return 0;
}  /* END set_rcvrfreq */


int execute_command( int socket, const char *command_str, char *response )
{
    char msg[STRINGLEN];	/* log message string			*/
    char message[BUFLEN+2];
    int  mlen;
    int  n, i;

    /* Copy the command into local string and add the \r
    ---------------------------------------------------------*/
    n = strlen( command_str );
    strcpy( message, command_str );
    message[n++] = 0x0d;
    message[n]   = '\0';
    mlen = n;

    /* Write message to the socket
    ------------------------------*/
    n = write( socket, message, mlen );
    if ( n != mlen )
    {
        sprintf( msg, "Error sending message: %s", message);
	write_log(FALSE,"set_rcvrfreq","Error",msg);
        return 1;
    }

    /* There should always be one character returned */
    i = 0;
    n = read( socket, &response[i++], 1 );

    /* unblock the socket */
    if ( fcntl( socket, F_SETFL, O_NDELAY ))
    {
	write_log(FALSE,"set_rcvrfreq","Error","Setting socket to non-block");
    }

    /* and read any remaining characters */
    while ( n > 0 )
    {
        n = read( socket, &response[i], 1 );
        i++;
    }

    /* null terminate the string */
    response[i-1] = '\0';

    /* and replace any \r with space */
    for ( n = 0; n < i; n++ )
    {
        if ( response[n] == 0x0d )
            response[n] = ' ';
    }

    /* Put the socket back in to async mode */
    if ( fcntl( socket, F_SETFL, O_ASYNC ))
    {
	write_log(FALSE,"set_rcvrfreq","Error","Setting socket to async");
    }

    return 0;

}  /* END execute_command */
