/*************************************************************************
FUNCTION:	start_acquisition

PURPOSE:	Initiate the processes necessary for live acquisition.

INPUT:		Incoming data directory
		moving window display socket descriptor
		sceneid
		day/night flag

OUTPUT:		acquire process id

RETURNS:	STOP if all was successful

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "acquman.h"


int start_acquisition
    (
    const char *adapsacq,       /* IN - Incoming data directory         */
    int shoacqu_socket,		/* IN - socket descriptor for MWD	*/
    const char *sceneid,	/* IN - 16-digit scene ID		*/
    const char *day_night_flag,	/* IN - day/night flag string		*/
    const char *stop_time,	/* IN - scene stop time			*/
    pid_t  *acquire_id		/* OUT - acquire child process ID	*/
    )
    {
    char output_filename[STRINGLEN];/* output file name of acquisition	*/
    char message[STRINGLEN];	/* message string			*/
    char dnflag;		/* actual flag value of day_night_flag	*/


    /* Build an output file name
    ----------------------------*/
    strcpy( output_filename, adapsacq );
    strcat( output_filename, "/" );
    strcat( output_filename, sceneid );
    strcat( output_filename, ".sf2" );


    /* Notify the display to start displaying the acquisition.
    ---------------------------------------------------------*/
    sscanf( day_night_flag, "%*s %c", &dnflag );
    switch ( sceneid[3] )
        {
	case '5' :
	    /* NOAA 15 is a descending daylight satellite so flip the day
	    night flag so that shoacqu will display the scene correctly.
	    ------------------------------------------------------------*/
	    if ( dnflag == 'd' || dnflag == 'D' )
		sprintf( message, "-file %s -dnflag N", output_filename );
	    else
		sprintf( message, "-file %s -dnflag D", output_filename );
	    break;
	case '6' :
	    sprintf( message, "-file %s %s", output_filename, day_night_flag );
	    break;
	case '7' :
	    /* NOAA 17 is a descending daylight satellite so flip the day
	    night flag so that shoacqu will display the scene correctly.
	    ------------------------------------------------------------*/
	    if ( dnflag == 'd' || dnflag == 'D' )
		sprintf( message, "-file %s -dnflag N", output_filename );
	    else
		sprintf( message, "-file %s -dnflag D", output_filename );
	    break;
	default  :
	    sprintf( message, "-file %s %s", output_filename, day_night_flag );
	    break;
	}  /* END switch */
    if ( dnflag == 'd' || dnflag == 'D' )
	strcat( message, " -rgb 2 2 1" );
    else
	strcat( message, " -rgb 3 3 3" );

    /* Add the stop time to the message
    -----------------------------------*/
    strcat( message, " -stop " );
    strcat( message, stop_time );

    send_message( shoacqu_socket, message );
    
    
    /* Fork an acquire child to do the acquisition
    ----------------------------------------------*/
    *acquire_id = fork_child( "acquire", "-port", FSPORT, "-file",
        output_filename );
    if ( acquire_id <= 0 )
	{
	write_log(FALSE,"start_acquisition","Error","Could not start acquire");

	/* Return a start condition since acquire did not start
	properly.  This will force another try at starting the
	acquisition.
	------------------------------------------------------*/
	return START;
	}
    else
        write_log(FALSE,"start_acquisition","Info","Initiated acquisition");
    
    return STOP;
    }  /* END start_acquisition */
