/*************************************************************************
FUNCTION:	stop_acquisition

PURPOSE:	To stop the current acquisition, make sure the scene is
		larger than the minimum size and start the time stamp
		process.

INPUT:		ADAPSACQ environment variable
		ADAPSING environment variable
		ADAPSERR environment variable
		process ID of acquire
		scene ID of image

OUTPUT:		Process ID of time correction process

RETURNS:	START if successful

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include "acquman.h"



int stop_acquisition
    (
    const char *adapsacq,       /* IN - ADAPSACQ environment variable   */
    const char *adapsing,       /* IN - ADAPSING environment variable   */
    const char *adapserr,       /* IN - ADAPSERR environment variable   */
    pid_t acquire_id,		/* IN - process ID of acquire		*/
    const char *sceneid,	/* IN - scene ID of image		*/
    pid_t *timecorr_id		/* OUT - process ID of time correction	*/
    )
    {
    char file_name[STRINGLEN];	/* full file name of scene		*/
    char error_file[STRINGLEN];	/* file name for moving to ADAPSERR	*/

    struct stat file_stats;	/* file statistics structure		*/



    /* Terminate the acquire process.  Caution: if for some reason the
    acquire child died during acquisition the acquire_id will be 0. If
    we try to kill process 0 we will kill this process, and all its
    children, instead.
    ------------------------------------------------------------------*/
    if ( acquire_id )
	kill( acquire_id, SIGHUP );


    /* Stow the ACU
    ---------------*/
    {
    int i,stat=1;
    write_log(FALSE,"stop_acqusition","Info","Telling ACU to stow");
    for ( i = 0; i < 5; i++ )
        {
	stat = acu_control( STOW, 0 );
	if ( stat == 0 )
	    break;
	sleep(1);
	}
    if ( stat )
        write_log(FALSE,"stop_acqusition","Error","Could not stow NEW ACU");
    }


    /* Check the size of the image to insure that we have acquired
    an image.  If the size is smaller than MINSCENESIZE move the
    image and associated files to ADAPSERR.
    --------------------------------------------------------------*/
    sprintf( file_name, "%s/%s.sf2", adapsacq, sceneid );
    if ( stat( file_name, &file_stats ) != 0 )
	write_log(FALSE,"stop_acquisition","stat","Error getting file stats");
    else
	{
	/* If the file is too small send it to ADAPSERR
	-----------------------------------------------*/
	if ( file_stats.st_size < MINSCENESIZE )
	    {
	    sprintf( error_file, "%s/%s.sf2", adapserr, sceneid );
	    if ( rename( file_name, error_file ) == -1 )
	        write_log(FALSE,"stop_acquisition","rename-error",file_name);
	    sprintf( file_name, "%s/%s.tstamp", adapsacq, sceneid );
	    sprintf( error_file, "%s/%s.tstamp", adapserr, sceneid );
	    if ( rename( file_name, error_file ) == -1 )
	        write_log(FALSE,"stop_acquisition","rename-error",file_name);
	    sprintf( file_name, "%s/%s.misc", adapsacq, sceneid );
	    sprintf( error_file, "%s/%s.misc", adapserr, sceneid );
	    if ( rename( file_name, error_file ) == -1 )
	        write_log(FALSE,"stop_acquisition","rename-error",file_name);
	    write_log(FALSE,"stop_acquisition","toosmall",
	        "Error complete scene was not acquired");
	    }  /* END if file_stats... */

	/* Otherwise, complete processing of the file
	---------------------------------------------*/
	else
	    {
	    /* Create a master schedule update file
	    ---------------------------------------*/
	    create_modsid( adapsing, sceneid );

	    /* Start the time correction process.  Fork_child requires
	    four arguments hence the two dummy arguments in the call.
	    ----------------------------------------------------------*/
	    *timecorr_id = fork_child( "timecorr", "-file", file_name, "", "" );
	    if ( *timecorr_id == -1 )
		{
	        write_log(FALSE,"stop_acquisition","fork-error",
		    "Could not start timecorr");
		}

	    write_log(FALSE,"stop_acquisition",sceneid,"Completed processing");
	    }
	}  /* END else */

    return START;
    }  /* END stop_acquisition */
