/*************************************************************************
FUNCTION:	wait_for_children

PURPOSE:	Prevent zombie child processes.  If called with a process
		ID (value greater than zero), the ID is added to a local
		list of ID's.  Waitpid is called for each of the ID's in
		the local list.  If more than PIDWARN process ID's are in
		the list a warning message is printed in the log.  This
		function will return at most one process ID of a child that
		has terminated 0 otherwise or -1 if a new ID structure
		could not be added.

INPUT:		valid process ID  (value greater than zero), all other
		    values are ignored.

OUTPUT:		writes warning message to log

RETURNS:	0 if process ID successfully added to list
		-1 if process ID not added to list
		Process ID of child that has terminated

PROJECT:	ADAPS AVHRR Acquisition System

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "acquman.h"


typedef struct ID
    {
    pid_t processid;		/* process id				*/
    struct ID *next;		/* pointer to next ID in list		*/
    } id;


pid_t wait_for_children
    (
    pid_t processid		/* IN - process ID to add to list	*/
    )
    {
    static id *head = NULL;	/* head of process id list		*/
    id *curr;			/* pointer to current process id	*/
    id *prev;			/* pointer to previous process id	*/
    id *newid;			/* pointer to id to add to list		*/

    char message[STRINGLEN];	/* log message string			*/

    int count = 0;		/* count of ID nodes in the list	*/
    int found;
    pid_t status = 0;		/* return status			*/


    /* If a process ID was passed add it to the list
    ------------------------------------------------*/
    if ( processid > 0 )
        {
	/* make sure the process ID is not in the list
	----------------------------------------------*/
	curr = head;
	found = FALSE;
	while ( curr != NULL )
	    {
	    if ( curr->processid == processid )
		{
	        found = TRUE;
	        break;
		}
	    curr = curr->next;
	    }  /* END while */

	if ( !found )
	    {
	    /* allocate a new node for the list */
	    newid = (id *)malloc( sizeof( id ));
	    if ( newid == NULL )
		{
		write_log(FALSE,"wait_for_children","Error","Unable to malloc");
		return -1;
		}
	    else
		{
		/* fill the structure */
		newid->processid = processid;
		newid->next = head;

		/* Add the new ID to the head of the list */
		head = newid;

		}  /* END else */
	    }  /* EMD if !found */
	}  /* END if processid > 0 */

    /* Go through the list and wait for each process, if a
    process terminates, break from the loop since we can
    only return the status of one process at a time.
    ------------------------------------------------------*/
    prev = curr = head;
    while ( curr != NULL )
        {
	/* If waitpid returns the same process ID
	as it was sent the process has completed
	----------------------------------------*/
	if ( waitpid( curr->processid, NULL, WNOHANG ) == curr->processid )
	    {
	    /* Return the process ID that has terminated */
	    status = curr->processid;

	    /* Remove the process ID from the list
	    --------------------------------------*/
	    if ( curr == head )
	        {
		/* removing from head of list */
		head = prev = curr->next;
		free( curr );
		curr = head;
		}
	    else
	        {
		/* removing from middle or end of list */
		prev->next = curr->next;
		free( curr );
		curr = prev->next;
		}

	    /* Break from loop since we can only return the
	    value of one process id that has terminated.
	    -----------------------------------------------*/
	    curr = NULL;
	    }  /* END if waidpid */

	/* Otherwise go to the next node in the list
	--------------------------------------------*/
	else
	    {
	    prev = curr;
	    curr = curr->next;
	    }
	}  /* END while curr != NULL */
    
    /* Issue a warning if PIDWARN is exceeded
    -----------------------------------------*/
    if ( head != NULL )
        {
	/* Count the number of nodes in the list The
	nodes can't be counted in the wait loop since
	it breaks out once a child process terminates.
	----------------------------------------------*/
	count = 0;
	curr = head;
	while ( curr != NULL )
	    {
	    count++;
	    curr = curr->next;
	    }  /* END while */

	if ( count > PIDWARN )
	    {
	    sprintf( message, "Warning there are %d zombied processes", count );
	    write_log(FALSE,"wait_for_children","Error",message);
	    }
	}  /* END if head != NULL */
    
    return status;
    }  /* END wait_for_children */
