/******************************************************************************
FUNCTION:	AUTOGEN

PURPOSE:	Produces a standard list of passes to be acquired for a set of
		NOAA satellites, for a specified period of days and period
		of time for each day.  The list can also be trimmed down such
		that all passes must be over a specified duration.  A 
		schedule update file can optionally be generated to update the 
		master schedule.


PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne	updated to receive the output file names as 
				parameters so their version numbers are already
				known.  This eliminates problems in batch.
5.0	   10/92    T. Rockvam	Updated to LAS 5.0
				Uses new model
6.0        11/98    D.Lloyd	Resurected from the Attic and modified for use
				with SCHEDCNFLT for conflict resolution.
				It now generates a listing without scheduled
				passes marked.  If input parameter STODATE is
				defaulted to NULL, 6 days are added to STADATE
				and used as the ending date.
6.1        02/00    D.Lloyd	Modified for Y2K

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
			None.

PROJECT			ADAPS/DOMSAT
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "util2.h"
#include "adaps.h"
#include "adapsutil.h"
#include "autogen.h"

float circle[NUMPNT][2];	/* target coverage circle		*/
long  lincnt = 1;		/* report line count			*/

void main ( void )
{
struct  PARBLK	vblock;		/* TAE variable block			*/

FILE	*report,		/* autogen report			*/
	*sched;			/* schedule update file			*/

long	stdlun,			/* standard output			*/
	status,			/* function return status		*/
	opmode,			/* operation mode for schedule		*/
	minsecs,		/* minimum pass length			*/
	varsecs,		/* variance time			*/
	msgtim,			/* message processing time		*/
	sdate[3],		/* user's start date			*/
	edate[3],		/* user's end date			*/
	stime[3],		/* user's start time			*/
	etime[3],		/* user's end time			*/
	schadd = 0,		/* schedule update flag			*/
	outcount = 0;           /* number of output files		*/

long	satnum;			/* satellite number 	(9,10,11,12)	*/
char	satnam[8],		/* satellite name 	(NOAA-11)	*/
	satno[6],		/* satellite number 	(19531)		*/
	satcode[2],		/* satellite code 	(I)		*/
	satid[4];		/* satellite id 	(N11)		*/

static char key[] = "SATNO", 	/* what type of satellite info we want	*/
	pname[] = "autogen";	/* program name				*/

char 	outnames[3][CMLEN];     /* output file names			*/

float	target[3],		/* target lat/long			*/
	mxdist;			/* maximum distance (meters)		*/


/*
  Initialize TAE
*/
c_inlas(&vblock,&stdlun);

/*
  Get the parameters from the variable block
*/
status = getpar(&vblock,&report,&sched,satid,sdate,edate,stime,etime,
	 target,&opmode,&minsecs,&varsecs,&mxdist,&schadd,outnames,
	 &outcount,&msgtim);
if (status != E_SUCC)
    c_clean(&vblock,pname,(const char (*)[CMLEN])outnames,&outcount);

/*
  Set up the coverage circle for the default target
*/
set_circle(target[0]*DEG_PER_RAD,target[1]*DEG_PER_RAD);
  
/*
  Set key values for the satellite being processed
ex: c_getsat("SATNUM",11,"NOAA-11","N11","19531","I")
*/
if (c_getsat(key,&satnum,satnam,satid,satno,satcode) != E_SUCC)
    c_clean(&vblock,pname,(const char (*)[CMLEN])outnames,&outcount);

/*
  Write the header for the output report
*/
header(report,satnam,satno,sdate,stime,edate,etime,
		target[0],target[1]);

/*
  Use the starting and ending times to find the passes we want
*/
status = findpasses(sched,report,satnum,satnam,satno,target[0],target[1],
		   sdate,stime,edate,etime,minsecs,varsecs,mxdist, opmode);
if (status != E_SUCC)
    c_clean(&vblock,pname,(const char (*)[CMLEN])outnames,&outcount);

/*
  Close the appropriate files
*/
fclose(report);
if (schadd) 
    fclose(sched);

z_exit(1,"AUTOGEN");

}
