#! /bin/csh -f
# FUNCTION:	autogen.csh
#
# PURPOSE:      Generate a temporary PDF that will allow a user to run a LAS
#               function as a cron job.  The temporary PDF will be created in
#               the specified directory and the LAS process will be run from
#               that directory as well.


#--------------------
# Check the arguments
#--------------------
if ($#argv < 1) then
    echo 'usage: autogen.csh dir [sat1 sat2 ...]'
    echo ''
    echo '       Create a temporary PDF that will allow a user to run the ADAPS'
    echo '       autogen as a cron job.  The temporary PDF will be created in'
    echo '       dir and the LAS process will be run from that directory as'
    echo '       well.  The list of satellite numbers should reflect which'
    echo '       ephemeris is entered in the ACU eg. 12 14'
    echo ''
    echo '       The temporary PDF will be removed as soon as the LAS process'
    echo '       begins.'
    exit 1
else if ($#argv > 5) then
    echo 'autogen.csh limited to 5 arguments: 1 directory and 4 satellites'
    exit 1
endif
cd $1
if ($#argv == 2) then
    set satellite1 = "autogen$2.prt"
    set satellite2 = ""
    set satellite3 = ""
    set satellite4 = ""
else if ($#argv == 3) then
    set satellite1 = "autogen$2.prt"
    set satellite2 = "autogen$3.prt"
    set satellite3 = ""
    set satellite4 = ""
else if ($#argv == 4) then
    set satellite1 = "autogen$2.prt"
    set satellite2 = "autogen$3.prt"
    set satellite3 = "autogen$4.prt"
    set satellite4 = ""
else if ($#argv == 5) then
    set satellite1 = "autogen$2.prt"
    set satellite2 = "autogen$3.prt"
    set satellite3 = "autogen$4.prt"
    set satellite4 = "autogen$5.prt"
endif

#---------------------------
# Set up the LAS environment
#---------------------------
if ((-r /etc/login) && (! $?EDCSOFT)) then
    source /etc/login
    eval `$masterlog response comfile las70d`
    unset masterlog
    unset response
    unset comfile
endif


#------------------------------
# Create a unique log file name
#------------------------------
set tstamp = `date '+%m%d%y_%H%M'`
set stadate = `date '+(%m,%d,%y)'`


#-------------------------------------------------------
# Generate a temporary PDF to create the autogen reports
#-------------------------------------------------------
cat << EOF > autogen_$tstamp.pdf
procedure autogen_$tstamp

body
autogen stadate=$stadate
exit
end-proc
EOF


#--------------------------------------------
# Start LAS and submit the PDF as a batch job
#--------------------------------------------
cat << EOJ | tae >& autogen_$tstamp.stdout
autogen_$tstamp |runtype=batch|
EOJ


#-----------------------------------------------------------
# The temporary pdf will not run from cron unless it it run
# in batch mode so test for the existance of the final
# product of autogen (autogen17.prt) before continuing
#-----------------------------------------------------------
while (! -e autogen18.prt )
    sleep 10
end


#-------------------------
# Remove the temporary PDF
#-------------------------
rm -f autogen_$tstamp.pdf


#-----------------------------------------
# Concatenate the autogen reports into one
# Don't use the autogen12.prt since it's
# ephemeris is no longer in the acu.
#-----------------------------------------
set filename = "schedcnflt_$tstamp.prt"
set autoname = "autogen_$tstamp.prt"
cat $satellite1 $satellite2 $satellite3 $satellite4 > autogen.prt
rm -f autogen18.prt autogen15.prt autogen16.prt autogen17.prt


#----------------------------------------------------
# Get a copy of the master schedule (try 5 times) and
# create the conflict report if successful
#----------------------------------------------------
set test = 1
set count = 1
set sch_dir = `rsh  edcsgs20 echo '$ADAPSTABLES'`
while ($test)
    rcp edcsgs20:$sch_dir/master.sch master.sch
    if ($status == 0) then
	set test = 0
	$ADAPSBASE/schedcnflt autogen.prt $filename master.sch
	echo "schedcnflt completed" >> autogen_$tstamp.stdout
        lp -s -c -dops < $filename
	# make a copy on edclxs1
	scp $filename edclxs1:'$ADAPSTABLES/schedcnflt.prt'
	if ( $status != 0 ) then
	    echo "Unable to copy to edclxs1" >> autogen_$tstamp.stdout
	endif
	mv $filename $ADAPSACQ/forms/$filename
    else
	echo "Unable to copy master schedule" >> autogen_$tstamp.stdout
        sleep 300
    endif
    if ($count == 5) set test = 0
    @ count += 1
end


#------------------------------------------
# Move files to the acquire forms directory
#------------------------------------------
mv autogen_$tstamp.stdout $ADAPSACQ/forms/autogen_$tstamp.stdout
mv master.sch $ADAPSACQ/forms/master_$tstamp.sch
mv autogen.prt $ADAPSACQ/forms/$autoname
mv autogen_*.log* $ADAPSACQ/forms/.


#---------------------------------------
# clean up 60 day old files in ACQ forms
#---------------------------------------
cd $ADAPSACQ/forms
set files = `ls *`
foreach file ($files)
    set test = 0
    set test = `find $ADAPSACQ/forms -name $file -mtime +60 -exec "echo" 1 \;`
    if ($test) then
        rm $ADAPSACQ/forms/$file
    endif
end

exit 0

