/******************************************************************************
FUNCTION:	findpasses

PURPOSE:	Find passes that match our criteria for date/time and length

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne
5.0        10/92    T.Rockvam   Rewritten with LAS5.0 and new model.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "autogen.h"


int findpasses
(
    FILE        *sched,		/* the schedule update file             */
    FILE        *report,	/* report of all passes                 */
    long        satid,		/* satellite id                         */
    char        *satnam,	/* satellite name                       */
    char        *satno,		/* satellite code                       */
    float       targlat,	/* station latitude                     */
    float       targlon,	/* station longitude                    */
    long        sdate[],	/* start date (mm dd yy)                */
    long        stime[],	/* start time (hh mm ss)                */
    long        edate[],	/* end date (mm dd yy)                  */
    long        etime[],	/* end time (hh mm ss)                  */
    long        minsecs,	/* minimum total pass time              */
    long        varsecs,	/* time variance each side of center    */
    float       mxdist,		/* maximum dist (meters) from station to sat*/
    long        opmode		/* operation mode                       */
)
{
struct MODEL model;    		/* orbital parameters			*/

/************************************************************************
  TIME variables:
   Some variables are in total seconds from 1986 for a single point of 
   reference, and for ease in looping thru days.
   The rest of the variables are in total seconds on the day 
   of the pass.  The model routines expect total seconds from 
   the day of the pass, even if the day of the pass is not the 
   same as the day of the ephemeris information.  			*/

double	dtotsecs,		/* double value of total seconds-used to*/
				/* increment time by exactly one orbit	*/
	approachsecs;		/* total seconds from start of scene 	*/
				/* jday to time of closest approach 	*/

long	totsecs,		/* total seconds at time of current orbit*/
	stopsecs,		/* total seconds at user's end time	*/
	daystart,
	daystop,
	zerotime[3],		/* start of day	(00:00:00)		*/
	midnight[3],		/* end of day	(23:59:59)		*/
	offtime,		/* offset in time from current start 	*/
				/* time to closest approach 		*/
	closestsecs,		/* total seconds since 1986 to time of 	*/
				/* closest approach			*/
	startoff,		/* offset from closest approach to start*/
				/* of pass 				*/
	startsecs,		/* seconds from 1986 to start of pass	*/
	startdate[3],		/* start date	(month day year)	*/
	starttime[3],		/* start time	(hour min sec)		*/
	endoff,			/* time offset from closest approach to */
				/* end of pass 				*/
	endsecs,		/* seconds from 1986 to end of pass	*/
	enddate[3],		/* end date (month day year)		*/
	endtime[3],		/* end time (hour min sec)		*/
	lasttime = 0,		/* seconds from 1986 to last pass	*/
	time_in_view;		/* total seconds satellite is in cov circle*/
/************************************************************************/

long	orbit,			/* orbit number				*/
	passdir,		/* pass direction			*/
	jday,			/* julian day				*/
	year,			/* two-digit year			*/
	sfsec = 0,		/* fractional seconds			*/
	status,			/* function return status		*/
	dtype = HRPT,		/* data type of live acquisition is HRPT*/
	nonfatal = 1,		/* nonfatal error code			*/
	nlines = 5400;		/* default nlines for 15-minute pass	*/

double	tlat,			/* target latitude			*/
	tlon,			/* target longitude			*/
	passlat,		/* pass latitude of closest approach	*/
	passlon,		/* pass longitude of closest approach	*/
	eqxing,			/* equatorial crossing			*/
	angdist,		/* angular distance bet us & satellite	*/
	dist,			/* distance in meters bet us & satellite*/
	zero = 0.0;		/* double value of zero			*/

float	line, samp;		/* line & sample			*/

char	sceneid[17];		/* scene id for the pass		*/
char	select[3];


/*
  Set up limits for looping thru all passes of the date range regardless of 
  user's time range
*/
zerotime[0] = zerotime[1] = zerotime[2] = 0;
midnight[0] = 23;
midnight[1] = midnight[2] = 59;
makesecs(sdate,zerotime,&totsecs);
makesecs(edate,midnight,&stopsecs);

/*
  Set start time to the first orbit in this time range
  if it returns E_FAIL, totsecs was not reset, which is ok
*/
status = setfirsttime(&totsecs,satid,&model);
dtotsecs = (double)totsecs;

/*
  Loop until we hit all orbits
*/
while (totsecs <= stopsecs)
    {
    /*
      Convert total seconds to hour,min,sec
    */
    consec(&totsecs,&startdate[2],&jday,&startdate[0],&startdate[1],
	   &starttime[0],&starttime[1],&starttime[2]);

    /*
      Find the time limits for this day
    */
    makesecs(startdate,stime,&daystart);
    makesecs(startdate,etime,&daystop);
    /* if end time is after midnight, add one day to the time */
    if (daystop < daystart)
        daystop += SEC_PER_DAY;

    /*
      Initialize the model for this time
    */
    year = startdate[2] % 100;
    model.ehour = model.emin = model.esec = model.efrac = 0;
    status = c_initmod(&satid,&year,&startdate[0],&startdate[1],
	&starttime[0],&starttime[1],&starttime[2],&sfsec,&dtype,&nlines,&model);
    if (status == E_FAIL)
	{
	c_errmsg("Error occurred initializing model","autogen-initmod",
	    &nonfatal);
        return(E_FAIL);
	}

    /*
      Find the time of closest approach for this orbit
    */
    tlat = (double)targlat;
    tlon = (double)targlon;
    status = findclosest(&model,tlat,tlon,&offtime,&passlat,&passlon);
    if (status == E_FAIL)
	{
        /*
          Add one orbit on to time
        */
        dtotsecs += PIE2 * HERG / model.mean_motion;
        totsecs = (long)(dtotsecs + 0.5);
	continue;
	}

    /* 
      getorb expects seconds from 0:00:00 of the scene's jday, 
      not ephem jday, if they are different 
    */
    approachsecs = model.shour*3600 + model.smin*60 + model.ssec 
	    + offtime;
    closestsecs = totsecs + offtime;

    /*
      Get orbit number for this time
    */
    c_getorb(&approachsecs,&model,&orbit,&passdir,&eqxing);

    /*
      Find the distance from the station (at the point of closest approach??)
    */
    calcdist(tlat,tlon,passlat,passlon,model.mean_motion,
	     &angdist,&dist,&time_in_view);

    /*
      If the pass is close enough and was not already found
      continue checking it out
    */
    if ((abs(lasttime - closestsecs) > 10) && (time_in_view > 0))
	{
	/*
	  Hang on the time of last pass in case the next orbit
	  returns the same one
	*/
	lasttime = closestsecs;

        /*
          Find the starting and ending time of the pass
        */
	findends(&model,closestsecs-varsecs,closestsecs+varsecs,
		 closestsecs,passlat,passlon,&startoff,&endoff);
        if (startoff == endoff)
            {
            /* Outside the U.S.-closest point is not in coverage circle */
            /* Start and end times would have been +/- half the  */
            /* time in view */
            startsecs = closestsecs - time_in_view/2;
            endsecs   = closestsecs + time_in_view/2;
            }
        else
            {
	    startsecs = closestsecs + startoff;
	    endsecs = closestsecs + endoff;
	    }
        consec(&startsecs,&startdate[2],&jday,&startdate[0],&startdate[1],
	   &starttime[0],&starttime[1],&starttime[2]);
        consec(&endsecs,&enddate[2],&jday,&enddate[0],&enddate[1],
	   &endtime[0],&endtime[1],&endtime[2]);

    	/*
      	  Find out if this is a day pass - look at the center of the pass
    	*/
	line = ((startsecs + endsecs) / 2.0 - totsecs) * LAC_PER_SEC + 1.0;
	samp = LAC_NADIR;
	c_ls2ll(&line,&samp,&zero,&model,&passlat,&passlon,NULL);

        /*
          Find the length of the pass; see if meets our length criteria
          If long enough, close enough, and in the user's time period,
          save record for schedule update
        */
	strcpy(select,"  ");
        if (((endsecs - startsecs) >= minsecs) &&
             (dist <= mxdist) &&
             (startsecs <= daystop) && (endsecs >= daystart))
	    {
	    strcpy(select,"**");
	    sprintf(sceneid,"AH%02d%02d%02d%02d%02d%02d%02d", 
		satid,startdate[0],startdate[1],(startdate[2] % 100),
		starttime[0],starttime[1],starttime[2]);
	    if (sched != NULL)
                /* Generate a record in the schedule update file */
	    	savsched(sched,sceneid,satid,startsecs,endsecs,opmode,orbit);
	    }
	/*
	  Write the record to the list of all passes
	*/
        write_report(report,orbit,satnam,satno,sdate,stime,edate,etime,
            startdate,starttime,enddate,endtime,endsecs-startsecs,
            closestsecs,time_in_view,targlat,targlon,dist,opmode);
	}

    /*
      Add one orbit on to time
    */
    dtotsecs += PIE2 * HERG / model.mean_motion;
    totsecs = (long)(dtotsecs + 0.5);
    }

return(E_SUCC);
}
