/******************************************************************************
FUNCTION:       setfirsttime

PURPOSE: 	This routine adjusts the input time to be the start time of 
		the current orbit.  The start time of an orbit is the time of 
		the ascending node of the orbit (the time the satellite crosses
		the equator on an ascending pass).
		FOR THE U.S.:
		For N11, day passes over the U.S. are ascending and night 
		passes are descending.  Thus, day passes over the U.S. will 
		occur soon after the ascending node, and the time of the pass 
		will be close to (within about 10 minutes of) the ascending 
		node.  Night passes will be about half an orbit (usually about 
		30-40 minutes) from the ascending node.
		For N12, the reverse is true.
			Sat  Day/Night PassDir    GMT		CST
			N11	N	Desc	0800-1300    0200-0700
			N11	D	Asc	1800-2400    1200-1800
			N12	N	Asc	2200-0300    1600-2100
			N12	D	Desc	1300-1700    0700-1100

PROGRAM HISTORY:
Version    Date     Author      Change Request
-------    -----    ---------   --------------
5.0	   10/92    T.Rockvam	New development for LAS 5.0 scheduling

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "autogen.h"


int setfirsttime
(
    long        *totsecs,	/* user start time/orbit start time */
    long        satid,		/* satellite id                 */
    struct MODEL *model		/* model structure              */
)
{
long	startdate[3],		/* start date (mm dd yy)	*/
	starttime[3],		/* start time (hh mm ss)	*/
	jday,			/* julian day			*/
	year,			/* 2-digit year			*/
	status,			/* function return status 	*/
	passdir,		/* pass direction		*/
	orbit,			/* orbit number			*/
	nonfatal = 1,		/* nonfatal error message	*/
	nlines = 5400,		/* default #lines in 15-min pass*/
	dtype = HRPT,		/* data type			*/
	sfsec = 0;		/* fractional seconds		*/

float	line,			/* line number			*/
	samp;			/* sample number		*/

double  eqxing,			/* equatorial crossing		*/
	zero = 0.0;		/* value of zero		*/

/*
  Convert total seconds to hour,min,sec
*/
consec(totsecs,&startdate[2],&jday,&startdate[0],&startdate[1],
           &starttime[0],&starttime[1],&starttime[2]);

/*
  Set up model information for this time
*/
year = startdate[2] % 100;
model->ehour = model->emin = model->esec = model->efrac = 0;
status = c_initmod(&satid,&year,&startdate[0],&startdate[1],
    &starttime[0],&starttime[1],&starttime[2],&sfsec,&dtype,&nlines,model);
if (status == E_FAIL)
    {
    c_errmsg("Error occurred initializing model","autogen-initmod",&nonfatal);
    return(E_FAIL);
    }

/*
  Find the equatorial crossing (the start of the orbit)
*/
c_getorb(&model->sec_epoch,model,&orbit,&passdir,&eqxing);

/*
  Find the line number at this point
*/
c_ll2ls(&zero,&eqxing,&zero,model,&line,&samp,NULL);

/*
  Adjust start time for start of the orbit
*/
*totsecs += (long)(line - 1.0)/LAC_PER_SEC;

/*
  Add 10 seconds to be sure we get into the correct orbit,
  not the tail end of the previous one
*/
*totsecs += 10;

return(E_SUCC);
}
