/*******************************************************************************
FUNCTION:	CHIPSETUP

PURPOSE:	Determines the AVHRR band and area to use for chip creation and
		navigation
		This routine assumes a LAS image with associated ADDR file.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  5.0    03/92  D.Hollaren       Original development
  5.1    11/92  J.Fenno          Added the FAXOPT parameter to allow the user
				 to specify if a facsimile should or should not
				 be generated.
				 Updated the User's Guide.
  5.2    07/93  J.Fenno          Modified so the scene ID is extracted from the
				 filename if it is valid.
  5.2    07/93  T.Rockvam        Taken from dbsetup

PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "archio.h"

main()
{
struct PARBLK	vblock;		/* TAE parameter block			     */
struct MODEL 	model;		/* Archive header model info		     */	

long	std_out,		/* LUN to standard output		     */
	status,			/* Return status			     */
	band,			/* Band to process			     */
	count     = 1,		/* Number of corners in the area	     */
	savcount  = 0,		/* Greatest # of corners in the area	     */
	zero      = 0,		/* Constant of zero			     */
	nonfatal  = 1,		/* Nonfatal error flag			     */
	fatal     = -1,		/* Fatal error flag			     */
	access    = IREAD;	/* Access flag				     */

char	area[5][10],		/* Area of the image for DCW files         */
	hosin[CMLEN],		/* Input host file name			     */
	*svec[5];		/* Array of text pointers to send params     */

double  tmplon1,tmplon2,
	tmplon3,tmplon4;	/* Temporary longitudes			     */

/*
  Crude Latitude/longitude maximum and minimum areas
  (SOAMAFR, EURNASIA, SASAUS, NOAMER)
  Array is set up as: max lat, min lat, max long, min long per area
*/
				/* SOAMAFR  */
static double regions[4][4] = {30.0 * RAD_PER_DEG,  -90.0 * RAD_PER_DEG,
			      360.0 * RAD_PER_DEG,    0.0 * RAD_PER_DEG,
				/* EURNASIA */
			       90.0 * RAD_PER_DEG,   35.0 * RAD_PER_DEG,
			      195.0 * RAD_PER_DEG,  -50.0 * RAD_PER_DEG,
				/* SASAUS   */
			       55.0 * RAD_PER_DEG,  -55.0 * RAD_PER_DEG,
			      230.0 * RAD_PER_DEG,   30.0 * RAD_PER_DEG,
				/* NOAMER   */
			       90.0 * RAD_PER_DEG,    5.0 * RAD_PER_DEG,
			       -5.0 * RAD_PER_DEG, -180.0 * RAD_PER_DEG};

/*
  Initialization
*/
c_inlas(&vblock,&std_out);

/*
  Retrieve input avhrr file name
*/
access = IREAD;
status = c_getnam(&vblock, "IN", &zero, hosin, "img", &access);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","chipsetup-fatal",&fatal);


/*
  Get the AVHRR header information
*/
status = c_getaddr(hosin,&model);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered", "chipsetup-fatal", &fatal);


/*
  Determine which band to process
*/
if (model.day_night == NIGHT)
    band = 4;
else
    band = 2;

strcpy(area[0],"NONE");
/*
  Check for South America/Africa
  This area goes -180 to +180 so check latitude only
*/
count = 0;
if ((model.nw[0] <= regions[0][0]) && (model.nw[0] >= regions[0][1]))
    count++;
if ((model.sw[0] <= regions[0][0]) && (model.sw[0] >= regions[0][1]))
    count++;
if ((model.se[0] <= regions[0][0]) && (model.se[0] >= regions[0][1]))
    count++;
if ((model.ne[0] <= regions[0][0]) && (model.ne[0] >= regions[0][1]))
    count++;
if ((count >= savcount) && (count > 0))
    {
    savcount = count;
    strcpy(area[0],"SOAMAFR");
    }

/*
  Check for Europe/N.Asia
  This area's longitude goes from -50 -> 180 and -180 -> -165
  So we will check -50 -> 195
*/
count = 0;
if (model.nw[1] <= -165.0) tmplon1 = model.nw[1] + 360.0;
else 			   tmplon1 = model.nw[1];
if (model.sw[1] <= -165.0) tmplon2 = model.sw[1] + 360.0;
else 			   tmplon2 = model.sw[1];
if (model.se[1] <= -165.0) tmplon3 = model.se[1] + 360.0;
else 			   tmplon3 = model.se[1];
if (model.ne[1] <= -165.0) tmplon4 = model.ne[1] + 360.0;
else 			   tmplon4 = model.ne[1];
if ((model.nw[0] <= regions[1][0]) && (model.nw[0] >= regions[1][1]) &&
    (tmplon1     <= regions[1][2]) && (tmplon1     >= regions[1][3]))
    count++;
if ((model.sw[0] <= regions[1][0]) && (model.sw[0] >= regions[1][1]) &&
    (tmplon2     <= regions[1][2]) && (tmplon2     >= regions[1][3]))
    count++;
if ((model.se[0] <= regions[1][0]) && (model.se[0] >= regions[1][1]) &&
    (tmplon3     <= regions[1][2]) && (tmplon3     >= regions[1][3]))
    count++;
if ((model.ne[0] <= regions[1][0]) && (model.ne[0] >= regions[1][1]) &&
    (tmplon4     <= regions[1][2]) && (tmplon4     >= regions[1][3]))
    count++;
if ((count >= savcount) && (count > 0))
    {
    savcount = count;
    strcpy(area[0],"EURNASIA");
    }

/*
  Check for S.Asia/Australia
  This area's longitude goes from 25 -> 180 and 180 -> -130
  So we will check  25 -> 230
*/
count = 0;
if (model.nw[1] <= 0.0) tmplon1 = model.nw[1] + 360.0;
else 			tmplon1 = model.nw[1];
if (model.sw[1] <= 0.0) tmplon2 = model.sw[1] + 360.0;
else 			tmplon2 = model.sw[1];
if (model.se[1] <= 0.0) tmplon3 = model.se[1] + 360.0;
else 			tmplon3 = model.se[1];
if (model.ne[1] <= 0.0) tmplon4 = model.ne[1] + 360.0;
else 			tmplon4 = model.ne[1];
if ((model.nw[0] < regions[2][0]) && (model.nw[0] > regions[2][1]) &&
    (tmplon1     < regions[2][2]) && (tmplon1     > regions[2][3]))
    count++;
if ((model.sw[0] < regions[2][0]) && (model.sw[0] > regions[2][1]) &&
    (tmplon2     < regions[2][2]) && (tmplon2     > regions[2][3]))
    count++;
if ((model.se[0] < regions[2][0]) && (model.se[0] > regions[2][1]) &&
    (tmplon3     < regions[2][2]) && (tmplon3     > regions[2][3]))
    count++;
if ((model.ne[0] < regions[2][0]) && (model.ne[0] > regions[2][1]) &&
    (tmplon4     < regions[2][2]) && (tmplon4     > regions[2][3]))
    count++;
if ((count >= savcount) && (count > 0))
    {
    savcount = count;
    strcpy(area[0],"SASAUS");
    }

/*
  Check for North America
*/
count = 0;
if ((model.nw[0] < regions[3][0]) && (model.nw[0] > regions[3][1]) &&
    (model.nw[1] < regions[3][2]) && (model.nw[1] > regions[3][3]))
    count++;
if ((model.sw[0] < regions[3][0]) && (model.sw[0] > regions[3][1]) &&
    (model.sw[1] < regions[3][2]) && (model.sw[1] > regions[3][3]))
    count++;
if ((model.se[0] < regions[3][0]) && (model.se[0] > regions[3][1]) &&
    (model.se[1] < regions[3][2]) && (model.se[1] > regions[3][3]))
    count++;
if ((model.ne[0] < regions[3][0]) && (model.ne[0] > regions[3][1]) &&
    (model.ne[1] < regions[3][2]) && (model.ne[1] > regions[3][3]))
    count++;
if ((count >= savcount) && (count > 0))
    {
    savcount = count;
    strcpy(area[0],"NOAMER");
    }

if (strcmp(area[0],"NONE") == 0)
    {
    c_errmsg("No DCW area found","chipsetup-area",&nonfatal);
    c_errmsg("Fatal error encountered","chipsetup-fatal",&fatal);
    }

/*
  Write the band to process to the parameter block
*/
status = q_intg(&vblock,"BAND",1,&band,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending BAND to TAE", "chipsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","chipsetup-fatal",&fatal);
    }


/*
  Write the area of the image for DCW
*/
svec[0] = area[0];
count = 1;
status = q_string(&vblock,"AREA",count,svec,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending AREA to TAE", "chipsetup-send", &nonfatal);
    c_errmsg("Fatal error encountered","chipsetup-fatal",&fatal);
    }


/*
  Write the parameter block to TAE
*/
status = q_out(&vblock);
if (status != SUCCESS)
    {
    c_errmsg("Error sending variable block to TAE","chipsetup-qout",&nonfatal);
    c_errmsg("Fatal error encountered","chipsetup-fatal",&fatal);
    }


/*
  Write a completion message 
*/
c_complt ("CHIPSETUP",hosin,&zero);
}
