! $Id: cloud_type.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!---------------------------------------------------------------------
! This module performs a cloud typing decision on pixel by pixel basis
! the resulting cloud mask codes are
!
! cloud ctype codes
!  0 - clear
!  1 - partly cloud
!  2 - liquid cloud
!  3 - mixed phase cloud
!  4 - glaciated
!  5 - cirrus cloud
!  6 - cirrus over lower
!
! Authors:
! Michael Pavolonis
! Andrew Heidinger
!----------------------------------------------------------------------
module CLOUD_TYPING
 use CONSTANTS
 use AVHRR_ROUTINES

 implicit none
 private
 public:: CLOUD_TYPE

 contains

 subroutine CLOUD_TYPE()

 real (kind=real4):: NIR_RAT_THRES,NIR_THRES,BTD_OVERLAP_THRES,&
		     CIRRUS_WIN_THRES,BT3B4_THRES,low_bound
 real(kind=real8), dimension(7):: A1
 real(kind=real8), dimension(7):: B1
 real(kind=real8), dimension(7):: C1
 real(kind=real8), dimension(7):: D1
 real(kind=real8), dimension(7):: E1
 real(kind=real8), dimension(7):: MIN_WINDOW
 real(kind=real8), dimension(7):: A2
 real(kind=real8), dimension(7):: B2
 real(kind=real8), dimension(7):: C2
 real(kind=real8), dimension(7):: D2
 real(kind=real8), dimension(7):: E2
 real(kind=real8), dimension(7,8) :: A3
 real(kind=real8), dimension(7,8) :: B3
 real(kind=real8), dimension(7,8) :: C3
 real(kind=real8), dimension(7,8) :: D3
 real(kind=real8), dimension(7,8) :: E3
 real(kind=real8), dimension(7,8) :: MIN_OVER

 integer:: i,j,nx,ny,index1,index2,wflg
 logical:: day

!----------------------------------------------------------------------------		
! Coefficients needed to determine the 11um - 12um BTD for cirrus detection 
! thresholds as a function of 11 um BT.  All coefficients are for 7
! different viewing zenith angles.
!----------------------------------------------------------------------------

 A1 = reshape((/-3.21578298543e+03, -2.94034788795e+03, -3.21256258031e+03,   &
                -3.47061397845e+03, -3.50486299804e+03, -5.08846627537e+03,   &
                -5.09507382017e+03/),(/7/))
           
 B1 = reshape((/ 4.88462861927e+01,  4.47332404792e+01,  4.86993920820e+01,   &
                 5.27678258867e+01,  5.32849278382e+01,  7.75358901756e+01,   &
                 7.80030571000e+01/),(/7/))
           
 C1 = reshape((/-2.76528129603e-01, -2.53525879101e-01, -2.75138895479e-01,   &
                -2.99072421299e-01, -3.01969990521e-01, -4.40956004022e-01,   &
                -4.45699597692e-01/),(/7/))
          
 D1 = reshape((/ 6.90692856276e-04,  6.33594236649e-04,  6.85787480971e-04,   &
                 7.48047893283e-04,  7.55160035017e-04,  1.10843453838e-03,   &
                 1.12561057802e-03/),(/7/))
           
 E1 = reshape((/-6.41179656790e-07, -5.88096462349e-07, -6.35205859105e-07,   &
                -6.95627948240e-07, -7.02035486892e-07, -1.03800396763e-06,   &
                -1.05900437811e-06/),(/7/)) 
		
 MIN_WINDOW = reshape((/1.0, 1.0, 0.9,   &
                        0.8, 0.6, 0.6,   &
                        0.6/),(/7/))     
 
!----------------------------------------------------------------------------
! Coefficients needed to determine the 3.75um - 11um BTD thresholds that 
! differentiate between ice and water as a function of 11um - 12um BTD.  
! All coefficients are for 7 different viewing zenith angles.
!----------------------------------------------------------------------------

 A2 = reshape((/ 3.67107643481e-01,  3.36458606683e-01,  2.41676270282e-01,   &
                 1.74658763419e-01,  2.87704447664e-01,  2.17330552664e-02,   &
                -7.31011031866e-01/),(/7/))
           
 B2 = reshape((/ 3.78078114458e+00,  3.76731335795e+00,  3.76913819531e+00,   &
                 4.11018993353e+00,  4.47071677565e+00,  4.55202437527e+00,   &
                 4.97669171136e+00/),(/7/))
           
 C2 = reshape((/ 1.68542169247e+00,  1.69898191715e+00,  1.71234157395e+00,   &
                 1.50137445131e+00,  1.08453312818e+00,  1.02913912350e+00,   &
                 1.47244717791e+00/),(/7/))
          
 D2 = reshape((/-1.29320690316e+00, -1.27347975779e+00, -1.25712279459e+00,   &
                -1.24807113787e+00, -1.09555849488e+00, -1.00754783102e+00,   &
                -1.31340559947e+00/),(/7/))
           
 E2 = reshape((/ 1.82935664832e-01,  1.72289757442e-01,  1.66813773502e-01,   &
                 1.76165053220e-01,  1.53279656708e-01,  1.28047566835e-01,   &
                 1.74999237696e-01/),(/7/))

!----------------------------------------------------------------------------
! Coefficients needed to determine the 11um - 12um BTD used to find cloud
! overlap as a function of 0.65 um reflectance.  All coefficients are for 7
! different viewing zenith angles and 8 different solar zenith angles.
!----------------------------------------------------------------------------

 A3 = reshape((/&
       2.94e+00, 3.14e+00, 3.15e+00, 3.03e+00, 3.27e+00, 3.77e+00, 3.77e+00,  &
       2.94e+00, 3.14e+00, 3.15e+00, 3.03e+00, 3.27e+00, 3.77e+00, 3.77e+00,  &
       2.76e+00, 3.04e+00, 3.14e+00, 3.20e+00, 3.23e+00, 3.25e+00, 3.25e+00,  &
       2.95e+00, 2.75e+00, 3.03e+00, 3.15e+00, 3.34e+00, 3.48e+00, 3.48e+00,  &
       2.62e+00, 2.71e+00, 2.65e+00, 2.80e+00, 2.80e+00, 2.97e+00, 2.97e+00,  &
       2.26e+00, 2.59e+00, 2.33e+00, 2.43e+00, 2.62e+00, 3.01e+00, 3.01e+00,  &
       1.94e+00, 1.29e+00, 1.65e+00, 1.65e+00, 1.88e+00, 6.49e-01, 6.49e-01,  &
      -2.33e+00,-1.83e+00, 4.17e-01,-2.67e+00,-7.20e-01, 2.34e-01, 2.34e-01/),&
      (/7,8/))

 B3 = reshape((/&
       9.36e-01,-3.25e+00,-2.60e+00, 1.71e+00,-7.43e-01,-8.27e+00,-8.27e+00,  &
       9.36e-01,-3.25e+00,-2.60e+00, 1.71e+00,-7.43e-01,-8.27e+00,-8.27e+00,  &
       4.48e+00,-1.20e+00,-2.31e+00,-1.98e+00, 1.48e-01, 2.65e+00, 2.65e+00,  &
       3.65e-01, 4.94e+00,-2.40e-01,-1.20e+00,-2.60e+00,-2.09e+00,-2.09e+00,  &
       6.62e+00, 4.96e+00, 6.72e+00, 5.76e+00, 8.27e+00, 9.71e+00, 9.71e+00,  &
       1.21e+01, 6.67e+00, 1.12e+01, 1.05e+01, 9.62e+00, 7.24e+00, 7.24e+00,  &
       1.67e+01, 2.45e+01, 1.99e+01, 1.99e+01, 1.81e+01, 3.37e+01, 3.37e+01,  &
       6.92e+01, 6.26e+01, 3.53e+01, 6.57e+01, 4.58e+01, 3.56e+01, 3.56e+01/),&
       (/7,8/))

 C3 = reshape((/&
      -4.12e+01,-2.41e+01,-2.77e+01,-4.57e+01,-3.60e+01,-8.56e+00,-8.56e+00,  &
      -4.12e+01,-2.41e+01,-2.77e+01,-4.57e+01,-3.60e+01,-8.56e+00,-8.56e+00,  &
      -5.47e+01,-3.20e+01,-2.77e+01,-2.96e+01,-3.80e+01,-5.29e+01,-5.29e+01,  &
      -3.76e+01,-5.51e+01,-3.41e+01,-3.01e+01,-2.40e+01,-3.02e+01,-3.02e+01,  &
      -6.08e+01,-5.30e+01,-5.77e+01,-5.38e+01,-6.41e+01,-7.67e+01,-7.67e+01,  &
      -8.14e+01,-5.96e+01,-7.29e+01,-6.59e+01,-6.21e+01,-4.88e+01,-4.88e+01,  &
      -1.02e+02,-1.27e+02,-1.06e+02,-1.00e+02,-9.37e+01,-1.38e+02,-1.38e+02,  &
      -3.09e+02,-2.80e+02,-1.69e+02,-2.56e+02,-1.86e+02,-1.23e+02,-1.23e+02/),&
      (/7,8/))

 D3 = reshape((/&
       8.58e+01, 6.05e+01, 6.69e+01, 9.35e+01, 7.91e+01, 4.24e+01, 4.24e+01,  &
       8.58e+01, 6.05e+01, 6.69e+01, 9.35e+01, 7.91e+01, 4.24e+01, 4.24e+01,  &
       1.05e+02, 7.15e+01, 6.50e+01, 6.78e+01, 7.92e+01, 1.07e+02, 1.07e+02,  &
       7.88e+01, 1.03e+02, 7.16e+01, 6.47e+01, 5.46e+01, 6.82e+01, 6.82e+01,  &
       1.11e+02, 9.79e+01, 1.01e+02, 9.37e+01, 1.08e+02, 1.33e+02, 1.33e+02,  &
       1.41e+02, 1.08e+02, 1.20e+02, 1.03e+02, 9.63e+01, 7.01e+01, 7.01e+01,  &
       1.78e+02, 2.08e+02, 1.70e+02, 1.53e+02, 1.43e+02, 1.87e+02, 1.87e+02,  &
       5.08e+02, 4.55e+02, 2.75e+02, 3.69e+02, 2.66e+02, 1.40e+02, 1.40e+02/),&
       (/7,8/))


 E3 = reshape((/&
      -5.09e+01,-3.83e+01,-4.20e+01,-5.52e+01,-4.80e+01,-3.16e+01,-3.16e+01,  &
      -5.09e+01,-3.83e+01,-4.20e+01,-5.52e+01,-4.80e+01,-3.16e+01,-3.16e+01,  &
      -6.02e+01,-4.36e+01,-4.01e+01,-4.14e+01,-4.67e+01,-6.36e+01,-6.36e+01,  &
      -4.67e+01,-5.81e+01,-4.20e+01,-3.79e+01,-3.25e+01,-4.11e+01,-4.11e+01,  &
      -6.25e+01,-5.49e+01,-5.42e+01,-4.99e+01,-5.63e+01,-7.13e+01,-7.13e+01,  &
      -7.72e+01,-6.02e+01,-6.28e+01,-5.13e+01,-4.70e+01,-3.22e+01,-3.22e+01,  &
      -1.00e+02,-1.12e+02,-8.96e+01,-7.65e+01,-7.04e+01,-8.40e+01,-8.40e+01,  &
      -2.85e+02,-2.52e+02,-1.49e+02,-1.82e+02,-1.28e+02,-5.21e+01,-5.21e+01/),&
      (/7,8/))

       
!----------------------------------------------------------------------------
! The minimum 11um - 12um BTD allowed for overlap detection.  All values are 
! for 7 different viewing zenith angles and 8 different solar zenith angles.
!----------------------------------------------------------------------------
	     
  MIN_OVER = reshape((/&
             0.70,0.70,0.70,0.70,0.75,0.80,0.80,  &
             0.70,0.70,0.70,0.70,0.75,0.80,0.80,  &
             0.70,0.70,0.70,0.70,0.75,0.80,0.80,  &
             0.70,0.70,0.70,0.70,0.75,0.80,0.80,  &
             0.70,0.70,0.70,0.70,0.75,0.80,0.80,  &
             0.70,0.70,0.70,0.70,0.75,0.90,0.90,  &
             0.75,0.75,0.75,0.80,0.80,0.90,0.90,  &
             0.75,0.75,0.75,0.80,0.80,0.90,0.90/),&
	     (/7,8/))
 
!----------------------------------------------------------------------------
! Begin cloud typing.
!----------------------------------------------------------------------------
 
 nx = size(alb1,1)
 ny = size(alb1,2)
 
i_loop: do  i = 1,nx

 j_loop: do j = 1,ny
   
!----------------------------------------------------------------------------
! Determine the viewing zenith angle bin.
!----------------------------------------------------------------------------
   
   index1 = min(7,max(1,int(satzen(i,j)/10.0) + 1))
   
!----------------------------------------------------------------------------
! Determine the solar zenith angle bin.
!----------------------------------------------------------------------------
   
   index2 = min(8,max(1,int(solzen(i,j)/10.0) + 1))
   
!----------------------------------------------------------------------------
! Set the surface dependent 1.6 um and 1.6um/0.6um thresholds.
!----------------------------------------------------------------------------
     
   if ((sfc_type(i,j) == 0) .or. (sfc_type(i,j) == 14) ) then
     NIR_RAT_THRES = 0.7
     NIR_THRES = 17.0
   else
     NIR_RAT_THRES = 0.9
     NIR_THRES = 32.0   
   endif
   
!----------------------------------------------------------------------------
! Set 11um - 12um overlap thresholds.
!----------------------------------------------------------------------------
   
   if ((alb1(i,j) >= 35.0) .and. (alb1(i,j) <= 60.0)) then
     BTD_OVERLAP_THRES = A3(index1,index2) + B3(index1,index2)*alb1(i,j) + C3(index1,index2)*alb1(i,j)**2 + &
                         D3(index1,index2)*alb1(i,j)**3 + E3(index1,index2)*alb1(i,j)**4
   elseif (alb1(i,j) > 60.0 .and. alb1(i,j) < 90.0) then
     BTD_OVERLAP_THRES = MIN_OVER(index1,index2)
   else
     BTD_OVERLAP_THRES = 9999.0
   endif
   
   if (BTD_OVERLAP_THRES < MIN_OVER(index1,index2)) then
     BTD_OVERLAP_THRES = MIN_OVER(index1,index2)
   endif
   
   if ((sfc_type(i,j) == 14) .or. (solzen(i,j) >= 80.0)) then
     BTD_OVERLAP_THRES = 9999.0
   endif
     
!----------------------------------------------------------------------------
! Set 11um - 12um cirrus thresholds.
!----------------------------------------------------------------------------

   CIRRUS_WIN_THRES = A1(index1) + B1(index1)*bt4(i,j) + C1(index1)*bt4(i,j)**2 + &
                      D1(index1)*bt4(i,j)**3 + E1(index1)*bt4(i,j)**4
		      
   
   low_bound = MIN_WINDOW(index1)
   CIRRUS_WIN_THRES = max(1.0,min(3.0,CIRRUS_WIN_THRES))
   
!----------------------------------------------------------------------------
! Set 3.75um - 11um thresholds.
!----------------------------------------------------------------------------

   BT3B4_THRES = A2(index1) + B2(index1)*bt45(i,j) + C2(index1)*bt45(i,j)**2 + &
                 D2(index1)*bt45(i,j)**3 + E2(index1)*bt45(i,j)**4
		 
   BT3B4_THRES = min(8.0,max(-2.0,BT3B4_THRES))

!----------------------------------------------------------------------------
! Check if daytime or nighttime algorithm is to be used.
!---------------------------------------------------------------------------- 
   
   day = .false.
   if (solzen(i,j) < 80.0) then
     day = .true.
   endif 
   
!----------------------------------------------------------------------------
! Check for clear or partly cloudy grid cells.
!----------------------------------------------------------------------------

   if (cld_mask(i,j) == 0) then
     cld_type(i,j) = 0
   endif 
   if (cld_mask(i,j) == 1) then
     cld_type(i,j) = 1
   endif
   
!----------------------------------------------------------------------------
! If daytime, use daytime algorithm.
!----------------------------------------------------------------------------
   
   if ((cld_mask(i,j) >= 2) .and. (day .eqv. .true.)) then
   
!----------------------------------------------------------------------------
! Perform initial temperature typing.
!----------------------------------------------------------------------------     
     
     wflg = 1
     
     if (bt4(i,j) <= 243.16) then
       cld_type(i,j) = 4
       wflg = 0
     elseif ((bt4(i,j) > 243.16) .and. (bt4(i,j) <= 253.16)) then
       cld_type(i,j) = 4
     elseif ((bt4(i,j) > 253.16) .and. (bt4(i,j) <= 273.16)) then
       cld_type(i,j) = 3
     else
       cld_type(i,j) = 2
     endif
     
!----------------------------------------------------------------------------
! Use 1.6 um algorithm, if available.
!----------------------------------------------------------------------------
       
     if ((ch3a_on(j) == 1) .and. (alb1(i,j) > 0.0) ) then 
       
     
!----------------------------------------------------------------------------
! Perform the 1.6um typing test.
!----------------------------------------------------------------------------
     
       if (cld_type(i,j) == 3 .and. alb3a(i,j) <= NIR_THRES .and. bt4(i,j) < 263.16) then
         cld_type(i,j) = 4
       endif

       if (cld_type(i,j) == 4 .and. wflg == 1 .and. alb3a(i,j) > NIR_THRES) then
         cld_type(i,j) = 3
       endif
     
!----------------------------------------------------------------------------
! Perform the 3.75um reflectance test (used only if 1.6um is not available).
!----------------------------------------------------------------------------

     elseif ((ch3a_on(j) == 0) .and. (alb1(i,j) > 0.0) ) then
       
       NIR_THRES = 6.0
       
       if (cld_type(i,j) == 3 .and. alb3b(i,j) <= NIR_THRES .and. bt4(i,j) < 263.16) then
         cld_type(i,j) = 4
       endif
       
       if (cld_type(i,j) == 4 .and. wflg == 1 .and. alb3b(i,j) > NIR_THRES) then
         cld_type(i,j) = 3
       endif
       
     endif
       
!----------------------------------------------------------------------------
! Perform the cloud overlap test.
!----------------------------------------------------------------------------

     if((bt45(i,j) > BTD_OVERLAP_THRES) .and. (bt4(i,j) < 270.0) .and. sfc_type(i,j) /= 14) then
       cld_type(i,j) = 6
     endif

!----------------------------------------------------------------------------
! Look for cirrus clouds.
!----------------------------------------------------------------------------

     if ((cld_type(i,j) /= 6) .and. (bt45(i,j) > CIRRUS_WIN_THRES)) then
       cld_type(i,j) = 5
     endif       
       
!----------------------------------------------------------------------------
! If nighttime, use tri-spectral nighttime algorithm.
!----------------------------------------------------------------------------
   
   elseif((cld_mask(i,j) >= 2) .and. (day .eqv. .false.)) then

!----------------------------------------------------------------------------
! Perform initial temperature typing.
!----------------------------------------------------------------------------

     wflg = 1
     
     if (bt4(i,j) <= 243.16) then
       cld_type(i,j) = 4
       wflg = 0
     elseif ((bt4(i,j) > 243.16) .and. (bt4(i,j) <= 253.16)) then
       cld_type(i,j) = 4
     elseif ((bt4(i,j) > 253.16) .and. (bt4(i,j) <= 273.16)) then
       cld_type(i,j) = 3
     else
       cld_type(i,j) = 2
     endif  
       
!----------------------------------------------------------------------------
! Perform 3.75um/11um/12um trispectral test.
!----------------------------------------------------------------------------

     if (cld_type(i,j) == 3 .and. bt3b4(i,j) >= BT3B4_THRES .and. bt4(i,j) < 263.16) then
       cld_type(i,j) = 4
     endif

     if (cld_type(i,j) == 4 .and. wflg == 1 .and. bt3b4(i,j) < BT3B4_THRES) then
       cld_type(i,j) = 3
     endif
     
!----------------------------------------------------------------------------
! Look for cirrus clouds.
!----------------------------------------------------------------------------

     if ((cld_type(i,j) /= 6) .and. (bt45(i,j) > CIRRUS_WIN_THRES)) then
       cld_type(i,j) = 5
     endif
       
   endif
        
 end do j_loop
   
end do i_loop

!----------------------------------------------------------------------------
! End of cloud typing algorithm.
!----------------------------------------------------------------------------
 
  end subroutine CLOUD_TYPE

end module CLOUD_TYPING
