! $Id: num_mod.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!------------------------------------------------------------------
!
!-------------------------------------------------------------------
 module NUMERICAL_ROUTINES
  use CONSTANTS
  implicit none
  private
  public:: LOCATE,ROBERTS_CON,PLANCK_RADIANCE,PLANCK_TEMPERATURE, &
           GAMMA_LN,HISTOGRAM_ARRAY,HISTOGRAM_VECTOR,GET_NCEP_T_U, &
           POSSOL,JULIAN
  contains

!--------------------------------------------------------------------
! compute T and u knowing P
!
! note this routine assumes that levels are even spaced (except sfc) and
! the first level is not zero
!--------------------------------------------------------------------
 subroutine GET_NCEP_T_U(P,T,u,p_level,t_p,u_p)
  real (kind=ipre), intent(in), dimension(:):: P,T,u
  real (kind=ipre), intent(in):: p_level
  real (kind=ipre), intent(out):: t_p, u_p
  real:: x,x2
  integer:: ilev,nlev

  nlev = size(P)

!----------------------------------------------------
! interpolate profile to desired pressure level
!----------------------------------------------------
 ilev = (p_level / P(1))
 ilev = min(max(ilev, 1),nlev-1)
 if (P(ilev+1) /= P(ilev)) then
   x = (p_level - P(ilev)) / (P(ilev+1) - P(ilev))
   x2 = (p_level**2 - P(ilev)**2) /( P(ilev+1)**2 - P(ilev)**2)
 else
   x = 0.0
   x2 = 0.0
 endif
 t_p = T(ilev)  + x * (T(ilev+1) - T(ilev))

 u_p = u(ilev)  + x2 * (u(ilev+1) - u(ilev))

 u_p = max(0.0_ipre,u_p)

! print *," in get t u"
! print *,"P_prof = ",P
! print *,"T_prof = ",T
! print *,"u_prof = ",u
! print *,"p_level, t_p, u_p", p_level, t_p, u_p
 end subroutine GET_NCEP_T_U

!-----------------------------------------------------------------------
! Roberts et al (1976) Continuum
!----------------------------------------------------------------------
 subroutine ROBERTS_CON(nlayers,u_h2o,e_h2o,P,T,wavel,tau_con)
  integer, intent(in):: nlayers
  real (kind=ipre), dimension(:), intent(in):: u_h2o
  real (kind=ipre), dimension(:), intent(in):: P,T,e_h2o
  real (kind=ipre), intent(in):: wavel
  real (kind=ipre), dimension(:), intent(out):: tau_con

  real (kind=ipre):: wavnum,Lv,Rv,gamma,phi0,ec,P_ref,phi
  real (kind=ipre):: e_mean,P_mean,T_mean,psi,kv,g

  integer:: lay

  wavnum = 10000.0/wavel
   P_ref = 1000.0
   g = 9.80
   Lv = 2.5e06  !latent heat of water vaporozation J/kg/K
   Rv = 461.0   !Water Vapor Gas constant J/kg/K
   gamma = 0.002  !constant for pressure broadening correction
   phi0 = exp(1800.0/296.0)
   do lay = 1, nlayers
    P_mean = 0.5*(P(lay)+P(lay+1))
    T_mean = 0.5*(T(lay)+T(lay+1))
!   e_mean = 0.5*(e_h2o(lay)+e_h2o(lay+1))
    e_mean = 0.5*(e_h2o(lay)*P(lay)+e_h2o(lay+1)*P(lay+1))/(P(lay)+P(lay+1))
    ec = (e_mean + gamma*(P_mean-e_mean) ) /P_ref  ! ec in atm
    phi = exp(1800.0/T_mean)     
    psi = 4.2 + 5588.0*exp(-0.00787*wavnum)      ! cm^2/g/atm
    kv = (phi/phi0) * psi * ec                   ! cm^2/g
    tau_con(lay) = kv*u_h2o(lay)
   enddo
 end subroutine ROBERTS_CON
!------------------------------------------------------------------
  subroutine LOCATE(xx,n,x,j)
   integer, intent(in):: n
   integer, intent(out):: j
   real (kind=ipre), intent(in):: x
   real (kind=ipre), dimension(:), intent(in):: xx
   integer:: i,jl,jm,ju
     jl=0
     ju=n+1
     do i = 1,2*n
      if( ju-jl <= 1) then
        exit
      endif
      jm=(ju+jl)/2
      if((xx(n) >= xx(1)) .eqv.(x >= xx(jm)))then
        jl=jm
      else
        ju=jm
      endif
     enddo
      if(x == xx(1))then
        j=1
      else if(x == xx(n))then
        j=n-1
      else
        j=jl
      endif
 end subroutine LOCATE

!---------------------------------------------------
! ye olde planck function
! T - temperature in K
! lambda - wavelength in microns
! planck_radiance - blackbody emission in W/m^2/str
!------------------------------------------------------
function PLANCK_RADIANCE(T,lambda) result(B)
  real (kind=ipre), intent(in):: lambda,T
  real (kind=ipre):: B
  real (kind=ipre), parameter:: c1= 1.191062e8, c2 = 1.438786e4
  B = c1/( lambda**5.0*(exp(c2/(lambda*T)) -1.0)  )
end function PLANCK_RADIANCE
!-------------------------------------------
function PLANCK_TEMPERATURE(B,lambda) result(T_b)
   real (kind=ipre), intent(in):: lambda,B
   real (kind=ipre):: T_b
   real (kind=ipre), parameter:: c1= 1.191062e08, c2 = 1.438786e04
   T_b = c2 /( lambda * log( 1.0 + c1/(B*(lambda**5))) )
end function PLANCK_TEMPERATURE

!---------------------------------------------------------------------------
function GAMMA_LN(xx) result(gammln)
  real(kind=ipre), intent(in):: xx
  real(kind=real8):: gammln
  integer:: j
  real(kind=real8):: ser,tmp,x,y
  real(kind=real8), parameter:: stp= 2.5066282746310005e0_real8
  real (kind=real8), dimension(6), parameter :: cof =                 &
  reshape((/76.18009172947146e0_real8,-86.50532032941677e0_real8,       &
           24.01409824083091e0_real8,-1.231739572450155e0_real8,  &
           0.1208650973866179e-2_real8, -0.5395239384953e-5_real8/),(/6/))
      x=xx
      y=x
      tmp=x+5.5_real8
      tmp=(x+0.5_real8)*log(tmp)-tmp
      ser=1.000000000190015_real8
      do j=1,6
        y=y+1.0_real8
        ser=ser+cof(j)/y
      enddo
      gammln=tmp+log(stp*ser/x)
 end function GAMMA_LN

!--------------------------------------------------------------------
! perform a histrogram on an two-dimensional array of any size
!
! input:
!         A - the array
!         bin_min - the minimum value to consider for histogram
!         delta - the width of the histrogram bins
!         nbins - the number of histogram bins
!         
! output:
!         h - number of occurences in each bin
!         p - pdf which integrated to unity
!         x - value of bin centers
!--------------------------------------------------------------------
  subroutine HISTOGRAM_ARRAY(A,bin_min,delta,nbins,h,p,x)
    real(kind=ipre), dimension(:,:), intent(in):: A
    real(kind=ipre), intent(in):: bin_min, delta
    integer, intent(in):: nbins 
    real(kind=ipre), dimension(:), intent(out):: h,p,x
    integer:: i,j,ibin

    do i = 1,size(A,1)
       do j = 1,size(A,2)
          ibin =  min(int((A(i,j)-bin_min)/delta) + 1,nbins)  !find correct bin
          h(ibin) = h(ibin) + 1
       enddo
    enddo
    p = h/(size(A)*delta)
    do ibin = 1,nbins
      x(ibin) = bin_min + (ibin-1)*delta + 0.5*delta
    enddo
 end subroutine HISTOGRAM_ARRAY 
!--------------------------------------------------------------------
! perform a histrogram on a vector of any length
!
! input:
!         A - the vector
!         bin_min - the minimum value to consider for histogram
!         delta - the width of the histrogram bins
!         nbins - the number of histogram bins
!         
! output:
!         h - number of occurences in each bin
!         p - pdf which integrated to unity
!         x - value of bin centers
!--------------------------------------------------------------------
  subroutine HISTOGRAM_VECTOR(A,bin_min,delta,nbins,h,p,x)
    real(kind=ipre), dimension(:), intent(in):: A
    real(kind=ipre), intent(in):: bin_min, delta
    integer, intent(in):: nbins 
    real(kind=ipre), dimension(:), intent(out):: h,p,x
    integer:: i,ibin

    do i = 1,size(A)
        ibin =  min(int((A(i)-bin_min)/delta) + 1,nbins)  !find correct bin
        h(ibin) = h(ibin) + 1
    enddo
    p = h/(size(A)*delta)
    do ibin = 1,nbins
      x(ibin) = bin_min + (ibin-1)*delta + 0.5*delta
    enddo
 end subroutine HISTOGRAM_VECTOR
!---------------------------------------------------------------------
! compute solar angles
!
! input:
!         jday - julian day
!         tu -  time of day - fractional hours
!         lon - latitude in degrees
!         lat - latitude in degrees
!      
!  output:
!         asol - solar zenith angle 
!         phi - solar azimuth angle
!
!----------------------------------------------------------------------
 subroutine POSSOL(jday,tu,xlon,xlat,asol,phis)
  integer, intent(in):: jday 
  real (kind=ipre), intent(in):: tu,xlon,xlat
  real (kind=ipre), intent(out):: asol, phis
  real (kind=ipre):: tsm, xlo, xla, xj, a1, a2, et, tsv, ah, a3, delta, amuzero, elev, &
         az, caz, azim, pi2
  real (kind=ipre), parameter:: pi = 3.14159265,dtor = pi / 180.0
!
! solar position (zenithal angle asol, azimuthal angle phi
! in degrees
! jday is the number of the day in the month
!
!
! mean solar time
!
      tsm = tu + xlon/15.0
      xlo = xlon*dtor
      xla = xlat*dtor
      xj = real(jday)
!
! time equation (mn.dec)
!
      a1 = (1.00554*xj-6.28306)*dtor
      a2 = (1.93946*xj+23.35089)*dtor
      et = -7.67825*sin(a1) - 10.09176*sin(a2)
!
! true solar time
!
      tsv = tsm + et/60.0
      tsv = (tsv-12.0)
!
! hour angle
!
      ah = tsv*15.0*dtor
!
! solar declination (in radian)
!
      a3 = (0.9683*xj-78.00878)*dtor
      delta = 23.4856*sin(a3)*dtor
!
! elevation, azimuth
!
      amuzero = sin(xla)*sin(delta)+cos(xla)*cos(delta)*cos(ah)
      elev=asin(amuzero)
      az = cos(delta)*sin(ah)/cos(elev)
      caz=(-cos(xla)*sin(delta)+sin(xla)*cos(delta)*cos(ah))/cos(elev)
!
      if (az >= 1.0) then
        azim = asin(1.0)
      elseif (az <= -1.0) then
        azim = asin(-1.0)
      else
        azim = asin(az)
      endif
!
      if (caz <= 0.0) then
        azim = pi - azim
      endif

      if ((caz > 0.0) .and. (az <= 0.0)) then
          azim = 2 * pi + azim
      endif
      azim = azim + pi
      pi2 = 2 * pi
      if (azim> pi2) then
          azim = azim - pi2
      endif
!
! conversion in degrees
!
      elev = elev / dtor
      asol = 90.0 - elev
      phis = azim / dtor

 end subroutine POSSOL
!-------------------------------------------------
! compute julian day
! input:
!         iday - integer day
!         imonth - integer month
!         iyear - integer year (2 or four digits)
!         
! output : jday - julian day
!--------------------------------------------------
 subroutine JULIAN(iday,imonth,iyear,jday)

!-- Computes julian day (1-365/366)
        integer, intent(in)::  iday,imonth,iyear
        integer, intent(out):: jday
        integer::  j
        integer, dimension(12)::  jmonth

        jmonth = reshape ((/31,28,31,30,31,30,31,31,30,31,30,31/),(/12/))

        jday = iday
        if (modulo(iyear,4) == 0) then
            jmonth(2)=29
        endif

        do j = 1,imonth-1
           jday = jday + jmonth(j)
        end do

   end subroutine JULIAN

end module NUMERICAL_ROUTINES
