! $Id: process_avhrr_clavr.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!------------------------------------------------------------------
! AVHRR 1b PROCESSING SOFTWARE
!
! author: Andrew Heidinger
! Date: June, 2002
! 
! this code does the following:
! read in AVHRR 1b GAC,LAC or HRPT data from all pre and post klm formats
! extract and apply navigation and calibration
! apply a cloud mask with or without dynamic thresholds
! apply a cloud thermodynamic phase typing algorithm
! write out a 1bx file which is the 1b + cloud mask 
! write out a clavr file which contains the results mapped into equal-area grid-cells
! 
! files pertaining the avhrr specific parameters are read in for
! each avhrr instrument. All avhrr's are treated as having a dual gain
! reflectance calibration. The calibration parameters for the 
! reflectance channels are taken from these files, not the avhrr data.
!
! the output for all avhrr's has ch3a included even though this channel
! did not exist until NOAA-15
!
! See comments in cloud_mask.f90 and cloud_type.f90 for a description
! of the output of these subroutines.
!
! in the 1b data lac, has one scan line split over 2 data records
! For consistency with gac, this code combines 4 lac records to produce two
! scan lines per record as in gac data
!
! July 2000- added ndvi(land), sst(water) into missing channel(ch3a/b) in 1c
!
!June 2002 - added the ability to write a .pcm file (cloud mask only)
!July 2002 - added the ability to write a clavr file (mapped data)
!July 2002 - added the the day and year to the records in clavr file
!September 2002 - moved some routines to avhrr_mod.f90 for cleanliness
!October 2002 - added ist and lst
!November 2002 - stuck cell_type_count into output by using some blank clavr bytes
!                this changes the number of blank clavr bytes from 51 to 37
!December 2002 - put in compilation of ice and water cloud radiances for cloudy pixels
!                and logic to store most common as the cloudy radiance distribution
!
! January - 2003 - Placed most functionality into subroutines
!
! internal switches
!   klm - T if this for NOAA-15,16,17... , else F
!   gac - T is this GAC data, F if LAC or HRPT
!   1bx - T if a 1bx is to be made
!   clavr - T if  a clavr file is to be made
!   dyn - T is dynamic thermal cloud tests are to be used
!   type - T if cloud type is turned on 
! 
!   irewound - this code processes two scanlines at a time, if there is an
!              odd number of scanlines, the code rewinds one scan line at the
!              end to process it
!-------------------------------------------------------------------------
 program PROCESS_AVHRR_CLAVR

!*****************************************************************************
! SECTION 1
! ACCESS MODULES 
!******************************************************************************
  use CONSTANTS
  use AVHRR_ROUTINES
  use CLAVR_ROUTINES
  use ELEVATION_MAP
  use SST_CLIMATOLOGY
  use SURFACE_PROPERTIES
  use CLOUD_MASK
  use CLOUD_TYPING
  use CLOUD_MASK_THRESHOLDS
  use EQUAL_AREA_GRID

  implicit none


!*************************************************************************
! SECTION 2 
! DECLARE AND INITIALIZE VARIABLES
!*************************************************************************
  integer(kind=int4), parameter::  num_chan=5,num_anchors=51
  integer(kind=int4), parameter:: nchan_clavr = 6, nmask_clavr=5, ntype = 7, &
                       num_blank_clavr_bytes=33
  integer(kind=int4):: rec_length,rec_length_pcm,rec_length_sst, &
            nword_clavr,nword_clavr_start, &
            ipix,iword,word_offset

  integer(kind=int2):: sc_id,data_type,start_year,start_day, &
                       end_year,end_day,month, &
                       num_scans,num_loc,tip_parity,aux_sync,ramp_auto_cal, &
                       num_scans_pcm
  integer(kind=int1):: sc_id_new,clavr_file_type
  integer(kind=int4):: start_time,end_time,rec_length_clavr
  logical:: gac,klm,bx,pcm,clavr,dyn,type,therm_cal_1b,high_res_sst
  integer(kind=int2):: first_scan_number
  real(kind=real4):: max_sst,max_lst,std_sst,std_lst

  integer(kind=int4):: irec,irec_header,irec_next,irec_1bx, &
                       ilon_land_mask,ilat_land_mask, &
                       ilon_elev,ilat_elev,ilon_sst_clim,ilat_sst_clim, &
                       irec_pcm,nrec_clavr,irec_sst,nrec_sst
  real(kind=real4):: sun_earth_distance,xlon
  integer(kind=int2):: num_pix
  integer(kind=int4):: i,j,rec_per_two_scans
  integer(kind=int1):: cld_mask_max
  character(len=7):: proc_block_id
  character(len=72):: file_1b,file_1bx,file_pcm,file_sst,file_clavr, &
                      data_dir,dir_1b,dir_1bx,dir_sst, &
                      instr_const_file,algo_const_file,dir_pcm,dir_clavr
  integer(kind=int4):: orbit_number

  real(kind=real4):: time_since_launch
  integer:: iskip,irewound,name_len

  integer:: ios0,file_number
  integer, dimension(4):: ios
  real(kind=real4):: max_bt,min_bt,max_alb,min_alb

!------- grid-cell parameters
real(kind=real4):: dlat
integer(kind=int4):: count_thresh

!** remove in odspd version*************************
 integer:: ioutput,asc_flag,idiag,isubset,i_skip,j_output
 real(kind=real4):: lat_min,lat_max

 ioutput = 0
 i_skip = 0
 j_output = 1
 isubset = 0
 idiag = 0

 asc_flag = 1      !0 = ascending
 lat_max = 45.0
 lat_min = 25.0

!********************************************

 dir_sst = "./sst"
 high_res_sst = .false.

 
 count_thresh = 1
 clavr_file_type = 0   

!--- set min and max values to constrain measurements
  max_alb = 120.0
  min_alb = -10.0
  max_bt = 340.0
  min_bt = 180.0


!*************************************************************************
! SECTION 3
! READ IN AVHRR_INPUT AND CREATE ARRAYS
!*************************************************************************

!-----------------------------------------------------------------
!--- open and read avhrr_input file
!-----------------------------------------------------------------
   print *, "***PROCESS_AVHRR - unit 55 avhrr_input"
   open(unit=55,file="avhrr_input",action="read",status="old",position="rewind")
   read(unit=55,fmt=*) klm
   read(unit=55,fmt=*) gac
   read(unit=55,fmt=*) therm_cal_1b
   read(unit=55,fmt=*) bx
   read(unit=55,fmt=*) pcm 
   read(unit=55,fmt=*) clavr
   read(unit=55,fmt=*) dyn
   read(unit=55,fmt=*) type
   read(unit=55,fmt=*) dlat
   read(unit=55,fmt="(a)") dir_1b
   read(unit=55,fmt="(a)") dir_1bx
   read(unit=55,fmt="(a)") dir_pcm
   read(unit=55,fmt="(a)") dir_clavr
   read(unit=55,fmt="(a)") data_dir

   if (dyn .eqv. .true.) then
     print *,"Dynamic Thermal Test is on"
   endif

   if (type .eqv. .true.) then
     print *,"Cloud Typing is turned on"
   endif

   if ((klm .eqv. .false.) .and. (bx .eqv. .true.)) then
      print *,"Error: 1bx option not available for pre-klm data"
      bx = .false.
   endif

!-------------------------------------------------------------------
!-- based on file type (klm and gac), determine parameters needed
!-- to read in header and data records
!-------------------------------------------------------------------
  call DEFINE_1B_DATA(gac,klm,num_pix,rec_length,irec_header, &
                      rec_per_two_scans,nword_clavr_start,nword_clavr)

!----------------------------------------------------------------------
!-- define record lengths for output files
!-----------------------------------------------------------------------
   rec_length_pcm = 2 + 6*num_pix
   rec_length_clavr =  1*6 + 2*13 +  &
                       4*(1 + 5 + (nmask_clavr+1)*nchan_clavr*2 + 11 + 1) + &
                       14 + num_blank_clavr_bytes

!-----------------------------------------------------------------------
!-- set up pixel level arrays
!-----------------------------------------------------------------------
  call  CREATE_PIXEL_ARRAYS(num_pix,rec_per_two_scans,rec_length,num_anchors)

!-----------------------------------------------------------------------
!-- set up gridcell and associated arrays
!-----------------------------------------------------------------------
  if (clavr .eqv. .true. ) then  

    call SETUP_EQUAL_AREA(dlat)

!-----------------------------------------------------------------------
!--- allocate memory for gridcell arrays
!-----------------------------------------------------------------------
    call CREATE_GRIDCELL_ARRAYS(nchan_clavr,nmask_clavr,ntype,num_pix, &
                            num_blank_clavr_bytes,nequal)

  endif

!*************************************************************************
! SECTION 4
! READ IN ANCILLARY DATA
!*************************************************************************

!-----------------------------------------------------------------------
!--- Read in land-sea mask and elevation data
!-----------------------------------------------------------------------
 call READ_LAND_COVER_UMD(trim(data_dir)//"sfc_data/land_cover_umd_8km_mod.dat")
 call READ_ELEVATION_MAP(trim(data_dir)//"sfc_data/digelev_le.map")
 call READ_SST_CLIMATOLOGY_MAP(trim(data_dir)//"sfc_data/sst_climatology_50km.dat")

!-----------------------------------------------------------------------
!---- set up surface radiative properties 
!-----------------------------------------------------------------------
 call SETUP_UMD_PROPS()

!-----------------------------------------------------------------------
!--- read in cloud mask thresholds
!-----------------------------------------------------------------------
 call READ_THRESHOLDS(data_dir)

!-----------------------------------------------------------------------
!--- read in grid-cell surface
!-----------------------------------------------------------------------
 if (clavr .eqv. .true. ) then  
   call READ_GRIDCELL_SFCTYPE(dlat,data_dir)
 endif


!----------------------------------------------------------------------
!START LOOP THROUGH AVHRR 1b FILES LISTED IN AVHRR_INPUT
!----------------------------------------------------------------------
file_number = 1

file_loop: do

!----------------------------------------------------------------------
! READ IN AVHRR_INPUT FILE AND SET FLAGS 
!----------------------------------------------------------------------
   read(unit=55,fmt="(a)",iostat=ios0) file_1b
   if (ios0 /= 0) then
    exit
   endif

   irec_pcm = 0
   file_number = file_number + 1
   iskip = 0
   irewound = 0
  
!-----------------------------------------------------------------------
!--- reset dynamic sst/lst thresholds for this orbit
!-----------------------------------------------------------------------
  max_lst = -1.0
  max_sst = -1.0
  std_sst = 2.0
  std_lst = 5.0

!----------------------------------------------------------------------
!-- reset thermal calibration stats for this orbit
!----------------------------------------------------------------------
  spinup_new_therm_cal = .true.
  iprt = 0
  jcal = 0
  mean_T_prt = 0.0
  mean_space_count_1 = 0.0
  mean_space_count_2 = 0.0
  mean_space_count_3 = 0.0
  mean_space_count_4 = 0.0
  mean_space_count_5 = 0.0
  mean_bb_count_3 = 0.0
  mean_bb_count_4 = 0.0
  mean_bb_count_5 = 0.0
!-----------------------------------------------------------------------
!--- reset grid-cell arrays for this orbit
!-----------------------------------------------------------------------
  if (clavr .eqv. .true.) then
     call INITIALIZE_GRIDCELL_ARRAYS()
  endif

!------------------------------------------------------------------
!--- open diagnostic files
!------------------------------------------------------------------
 if (idiag == 1) then
    call OPEN_DIAG_FILES(num_pix)
 endif


!*************************************************************************
! SECTION 5
! OPEN FILE AND READ IN HEADER AND DETERMINE SOME CONSTANTS
!*************************************************************************

!-------------------------------------------------------------------
!--- open avhrr 1b file
!-------------------------------------------------------------------
  call system("date")
  print *,"PROCESS_AVHRR - unit 9 read 1b==>",dir_1b
  print *,"processing file ", trim(file_1b)
  open(unit=9,file=trim(dir_1b)//trim(file_1b),  &
          status="old",form="unformatted",&
          access="direct",recl=rec_length,action="read")



!-------------------------------------------------------------------
!-- read in header
!-------------------------------------------------------------------
  if (gac .eqv. .true.) then
    read(unit=9,rec=1) buffer(1:rec_length)    
 
    if (klm .eqv. .true.) then
     call unpack_avhrr_header_record_klm(sc_id,data_type,start_year, &
                 start_day,start_time,num_scans,end_year,end_day,end_time, &
                 tip_parity,aux_sync,ramp_auto_cal,proc_block_id)
    else
     call unpack_avhrr_header_record(sc_id,data_type,start_year, &
                 start_day,start_time,num_scans,end_year,end_day,end_time, &
                  tip_parity,aux_sync,ramp_auto_cal,proc_block_id)

     end_year = end_year + 1900   !pre klm used 2 digit year
     start_year = start_year + 1900
    endif
  else
    read(unit=9,rec=1) buffer(1:rec_length)
    read(unit=9,rec=2) buffer(rec_length+1:2*rec_length)
    if (klm .eqv. .true.) then
     call unpack_avhrr_header_record_klm(sc_id,data_type,start_year, &
                 start_day,start_time,num_scans,end_year,end_day,end_time, &
                 tip_parity,aux_sync,ramp_auto_cal,proc_block_id)
     else
    call unpack_avhrr_header_record(sc_id,data_type,start_year, &
              start_day,start_time,num_scans,end_year,end_day,end_time, &
               tip_parity,aux_sync,ramp_auto_cal,proc_block_id)
     end_year = end_year + 1900   !pre klm used 2 digit year
     start_year = start_year + 1900
    endif
  endif

!------------------------------------------------------------------
! compute the month
!------------------------------------------------------------------
   month = COMPUTE_MONTH(start_day)
!-------------------------------------------------------------------
!-- define a sc_id that is unique value for each satellite
!-------------------------------------------------------------------
 sc_id_new = 0
 if (klm .eqv. .false.) then
   if(sc_id == 1) then   !TIROS-N
      sc_id_new = 1
   endif
   if(sc_id == 2) then   !NOAA-6
      sc_id_new = 6
   endif
   if(sc_id == 4) then   !NOAA-7
      sc_id_new = 7
   endif
   if(sc_id == 6) then   !NOAA-8
      sc_id_new = 8      
   endif
   if(sc_id == 7) then   !NOAA-9
      sc_id_new = 9      
   endif
   if(sc_id == 8) then   !NOAA-10
      sc_id_new = 10      
   endif
   if(sc_id == 1) then   !NOAA-11
      sc_id_new = 11      
   endif
   if(sc_id == 5) then   !NOAA-12
      sc_id_new = 12     
   endif
   if(sc_id == 3) then   !NOAA-14
      sc_id_new = 14     
   endif
 else
   if(sc_id == 4) then   !NOAA-15
      sc_id_new = 15      
   endif
   if(sc_id == 2) then   !NOAA-16
      sc_id_new = 16      
   endif
   if(sc_id == 6) then   !NOAA-17
      sc_id_new = 17      
   endif
 endif


!--------------------------------------------------------------
!--- make an integer orbit_number from proc_block_id
!--------------------------------------------------------------
   orbit_number = 0
   do i = 1,len(proc_block_id)
     orbit_number = orbit_number + (ichar(proc_block_id(i:i))-48)* &
                             (10**(len(proc_block_id) - i))
   enddo

!--------------------------------------------------------------
!--- output orbit parameters to screen
!--------------------------------------------------------------
   print *,"Orbit Number = ", orbit_number
   print *,"spacecraft id = ", sc_id, sc_id_new
   print *,"data type = ", data_type
   print *,"start year, day, time = ",start_year,start_day, &
                                      start_time/60.0/60.0/1000.0
   print *,"end year, day, time = ", end_year,end_day, &
                                     end_time/60.0/60.0/1000.0
   print *,"number of scans = ", num_scans

!--------------------------------------------------------------
!--- based on spacecraft id, set up constants
!--------------------------------------------------------------
  if (sc_id_new < 10) then
   instr_const_file = trim(data_dir)//"avhrr_data/"// &
                         "avhrr_"//char(sc_id_new+48)//"_instr.dat"
   algo_const_file = trim(data_dir)//"avhrr_data/"// &
                         "avhrr_"//char(sc_id_new+48)//"_algo.dat"
  else
   instr_const_file = trim(data_dir)//"avhrr_data/"// &
                "avhrr_1"//char(sc_id_new - 10 + 48)//"_instr.dat" 
   algo_const_file = trim(data_dir)//"avhrr_data/"// &
                "avhrr_1"//char(sc_id_new - 10 + 48)//"_algo.dat" 
  endif

  call READ_AVHRR_INSTR_CONSTANTS(trim(instr_const_file))
  call READ_AVHRR_ALGO_CONSTANTS(trim(algo_const_file))


!--------------------------------------------------------------
!--- name and open and write header to 1bx file
!--------------------------------------------------------------
  if (bx .eqv. .true.) then
   file_1bx = trim(file_1b)//"x"
   print *,"PROCESS_AVHRR unit 11 out 1bx==>",dir_1bx
   print *,"creating file ",trim(file_1bx)
   open(unit=11,file=trim(dir_1bx)//trim(file_1bx),  &
           status="replace",form="unformatted", &
           access="direct",recl=rec_length,action="write")
   if (gac .eqv. .true.) then
     write(unit=11,rec=1) buffer(1:rec_length)    
   else
    write(unit=11,rec=1) buffer(1:rec_length)
    write(unit=11,rec=2) buffer(rec_length+1:2*rec_length)
   endif
  endif

  print *,"opened and wrote header to 1bx"

!---------------------------------------------------------
!-- name and open and write header to pcm file
!---------------------------------------------------------
  if (pcm .eqv. .true.) then
   name_len = len_trim(file_1b)
   file_pcm = trim(file_1b)//".pcm"
   print *,"PROCESS_AVHRR unit 12 out cpm==>",dir_pcm
   print *,"creating file ",trim(file_pcm)
   open(unit=12,file=trim(dir_pcm)//trim(file_pcm),  &
           status="replace",form="unformatted", &
           access="direct",recl=rec_length_pcm,action="write")
  endif
  print *,"opened cm"

!--------------------------------------------------
! name and open high resolution sst file
!---------------------------------------------------
  if (high_res_sst .eqv. .true.) then
   name_len = len_trim(file_1b)
   file_sst = trim(file_1b)//".sst"
   open(unit=14,file=trim(dir_sst)//trim(file_sst),  &
           status="replace",form="unformatted", &
           access="direct",recl=rec_length_sst,action="write")
  endif

!--------------------------------------------------
! name and open clavr file
!---------------------------------------------------
  if (clavr .eqv. .true.) then
   name_len = len_trim(file_1b)
   file_clavr = trim(file_1b)//".clavr"
   print *,"PROCESS_AVHRR unit 13 out clavr==>",dir_clavr
   print *,"creating file ",trim(file_clavr)
   open(unit=13,file=trim(dir_clavr)//trim(file_clavr),  &
           status="replace",form="unformatted", &
           access="direct",recl=rec_length_clavr,action="write")
    blank_clavr =  0
  endif

!-----------------------------------------------------------
!--- compute sun-earth distance
!------------------------------------------------------------
   sun_earth_distance = 1.0 - 0.016729*cos(0.9856*(start_day-4.0)*dtor)
   print *,"sun earth distance = ", sun_earth_distance

!------------------------------------------------------------
!--- compute time since launch needed for reflectance calibration
!------------------------------------------------------------
   time_since_launch = start_year + start_day / 365.25  - launch_date

!*************************************************************************
! SECTION 6
! READ IN DATA RECORDS AND PERFORM QUALITY CHECKING
!*************************************************************************
ios = 0
irec_next = irec_header + 1
irec_1bx = irec_next
irec_pcm = 1

records: do 

   irec = irec_next

   q_flag = 0
   ch3a_on = 0

   if (gac .eqv. .true.) then
    if (klm .eqv. .true. ) then
      read(unit=9,rec=irec,iostat=ios(1)) buffer(1:rec_length)    
      read(unit=9,rec=irec+1,iostat=ios(2)) buffer(rec_length+1:2*rec_length)    
      call unpack_avhrr_data_record_klm(gac,scan_number, &
              ch3a_on,resync,fatal,ascend,year,day,time,num_loc)

    else
     read(unit=9,rec=irec,iostat=ios(1)) buffer    
     call unpack_avhrr_data_record(gac,scan_number,resync,fatal, &
                  ascend,year,day,time,num_loc)

    endif
   else
    if (klm .eqv. .true. ) then
     read(unit=9,rec=irec,iostat=ios(1)) buffer(1:rec_length)
     read(unit=9,rec=irec+1,iostat=ios(2)) buffer(rec_length+1:2*rec_length)
      call unpack_avhrr_data_record_klm(gac,scan_number, &
                  ch3a_on,resync,fatal,ascend,year,day,time,num_loc)
    else
     read(unit=9,rec=irec,iostat=ios(1)) buffer(1:rec_length)
     read(unit=9,rec=irec+1,iostat=ios(2)) buffer(rec_length+1:2*rec_length)
     read(unit=9,rec=irec+2,iostat=ios(3)) buffer(2*rec_length+1:3*rec_length)
     read(unit=9,rec=irec+3,iostat=ios(4)) buffer(3*rec_length+1:4*rec_length)

     call unpack_avhrr_data_record(gac,scan_number,resync,fatal, &
                                  ascend,year,day,time,num_loc)
    endif
   endif

   
   if (irec == irec_header + 1) then
      first_scan_number = scan_number(1)
      print *,"first scan number = ", first_scan_number
   endif

!----------------------------------------------------------------------
! generate new thermal calibration coefficients
!----------------------------------------------------------------------
   if (therm_cal_1b .eqv. .false.) then
      if (klm .eqv. .true.) then
       call COMPUTE_NEW_THERM_CAL_COEF_KLM()
      else
       call COMPUTE_NEW_THERM_CAL_COEF()
      endif

!---- determine if valid coefficients are made
      if (valid_new_therm_cal .eqv. .false.) then
         fatal = 1
      endif
!--- if sufficient scans lines have not been processed, this is fatal
      if (spinup_new_therm_cal .eqv. .true.) then
         fatal = 1
      endif
   endif
!----------------------------------------
! exit if end of file received
!----------------------------------------
   if (maxval(scan_number-first_scan_number) > num_scans) then
     print *," exceeded expected number of scans, exiting on rec = ", irec
     exit
   endif

   if ((sum(ios) > 0) .and. (irewound == 0)) then
     if( (klm .eqv. .false.) .and. (gac .eqv. .true.)) then
       exit
     else
      irec_next = irec -  rec_per_two_scans/2
      irec_1bx = irec_1bx - rec_per_two_scans/2
      irewound = 1
      print *, "rewinding ", irec, irec_next, irewound
      cycle
     endif
   elseif ((sum(ios) > 0) .and. (irewound == 1)) then
      print *, "end of 1b file reached on record", irec
      exit
   else 
      irec_next = irec +  rec_per_two_scans
   endif

!------------------------------------------------------------------------------
! check for bad scan lines here
!------------------------------------------------------------------------------
   if ( (sum(lat_anchor(:,1)) == 0.00).or. (sum(lon_anchor(:,1)) == 0.00)) then
       print *,"error in navigation"
       q_flag(1) = 1
   endif
   if ( (sum(lat_anchor(:,2)) == 0.00).or. (sum(lon_anchor(:,2)) == 0.00)) then
       q_flag(2) = 1
       print *,"error in navigation"
   endif
   if (fatal(1) > 0) then
       q_flag(1) = 1
   elseif (fatal(2) > 0) then
       q_flag(2) = 1
   endif
  if (ascend(1) > 1) then
       q_flag(1) = 1
  elseif (ascend(2) > 1) then
       q_flag(2) = 1
  endif
  if (sum(q_flag) > 0) then
     print *,"fatal flaw found", irec
     cycle
  endif

!--- check to see if new thermal cal is valid if in use
  if ((therm_cal_1b .eqv. .false.) .and. (valid_new_therm_cal .eqv. .false.)) then
      print *,"new thermal cal failed, skipping scanlines " ,irec
      cycle
  endif
!-----------------------------------------------------------------------
! check if subsetting turned on and if region entered or exited
! note, iskip is used to ensure region is not entered on  orbit overlap
!-----------------------------------------------------------------------
   if ( (isubset==1) .and. (ascend(1) == asc_flag) .and.(irec < 5) .and. &
        (lat_anchor(26,1) > lat_min) .and.  &
        (lat_anchor(26,1) < lat_max)) then
        iskip = 1
   endif
   if ( (isubset==1) .and. (ascend(1) == asc_flag) .and.(iskip == 1) .and. &
        ((lat_anchor(26,1) < lat_min) .or.  &
        (lat_anchor(26,1) > lat_max))) then
        iskip = 0
   endif

   if ((ioutput == 0).and.(isubset==1).and.(iskip==0)) then
       if ((ascend(1) == asc_flag) .and.  &
           (lat_anchor(26,1) >= lat_min) .and.  &
           (lat_anchor(26,1) <= lat_max)) then
             ioutput= 1
             print *,"data in desired subset region found", scan_number(1), irec, lat_anchor(26,1)
       endif
   endif
   if ((isubset == 1).and.(ioutput == 1))  then
        if ((lat_anchor(26,1) < lat_min) .or. (lat_anchor(26,1) > lat_max)) then
          print *,"desired subset region left", scan_number(1), irec, lat_anchor(26,1)
          ioutput = 0
          exit
        endif
   endif

!--------------------------------------------------
! skip if subsetting and outside of region
!--------------------------------------------------
   if ((isubset == 1).and.(ioutput == 0)) then
     cycle
   endif


!--------------------------------------------------------------------------
! print out info every 1000th scan line to screen
!---------------------------------------------------------------------------
  if ((modulo(scan_number(1),1000_int2) == 0).or.(modulo(scan_number(2),1000_int2) == 0)) then
        write (unit=6,fmt="(2i6,f7.3,i3,3f8.2)")  & 
         scan_number,time(1)/60.0/60.0/1000.0,ascend(1), &
         lat_anchor(26,1),lon_anchor(1,1),lon_anchor(51,1)
   endif



!*************************************************************************
! SECTION 7
! LOOP THROUGH EACH SCANLINE IN SCANLINE PAIR AND NAVIGATE & CALIBRATE
!*************************************************************************

!--------------------------------------------------------------------------
! REFLECTANCE CALIBRATION (1,2,3a)
!--------------------------------------------------------------------------
   call REF_CAL(time_since_launch)

!--------------------------------------------------------------------------
! RADIANCE CALIBRATION (3b,4,5)
!--------------------------------------------------------------------------
   call THERM_CAL(therm_cal_1b,klm,ch3a_on)

scans1: do j = 1,2
!----------------------------------------------------------------------
! COMPUTE ANGLE ANCHOR POINTS
!----------------------------------------------------------------------
!----------------------------------------------------------------------
! for NOAA-14 and earlier, compute anchors points for satzen, relaz
! and glintzen - also compute solar zenith angle to a higher precision
! note klm notation is different than that used here
!---------------------------------------------------------------------
  if (klm .eqv. .false.) then
    call compute_angle_anchors(day(j),time(j),lon_anchor(:,j),lat_anchor(:,j),satzen_anchor(:,j),&
                               solzen_anchor(:,j),relaz_anchor(:,j))
   else
     relaz_anchor(:,j) = 180.0 - abs(relaz_anchor(:,j))
  endif

!---------------------------------------------------------------
! MAKE GLINT ZENITH ANGLE ANCHORS 
!---------------------------------------------------------------
    call COMPUTE_GLINTZEN(solzen_anchor(:,j),satzen_anchor(:,j), &
                          relaz_anchor(:,j),glintzen_anchor(:,j))
                          
!---------------------------------------------------------------------------
! INTERPOLATE ANCHORS TO EACH PIXEL
!---------------------------------------------------------------------------
    call lagrangian_anchor_interp(lat_anchor(:,j),lat(:,j))
    call lagrangian_anchor_interp(lon_anchor(:,j),lon(:,j))
    call linear_anchor_interp(satzen_anchor(:,j),satzen(:,j))
    call linear_anchor_interp(solzen_anchor(:,j),solzen(:,j))
    call linear_anchor_interp(relaz_anchor(:,j),relaz(:,j))
    call linear_anchor_interp(glintzen_anchor(:,j),glintzen(:,j))

!---------------------------------------------------------------------------
!--- convert visible albedoes to standard reflectances
!---------------------------------------------------------------------------
    alb1(:,j) = alb1(:,j) / cos(solzen(:,j)*dtor) * (sun_earth_distance**2)
    alb2(:,j) = alb2(:,j) / cos(solzen(:,j)*dtor) * (sun_earth_distance**2)
    alb3a(:,j) = alb3a(:,j)/cos(solzen(:,j)*dtor) * (sun_earth_distance**2)

end do scans1

!-----------------------------------------------------------
! produce pixel arrays needed for subsequent processing
!-----------------------------------------------------------
call COMPUTE_PIXEL_ARRAYS()

!----------------------------------------------------------------------------
! COMPUTE CHANNEL3b ALBEDO 
!----------------------------------------------------------------------------
  call CH3B_ALB(A0,B0,C0,sun_earth_distance)

!----------------------------------------------------------
! Produce pixel level products for clavr file
!----------------------------------------------------------
 if (clavr .eqv. .true.) then
  call COASTWATCH_SST()
  call COMPUTE_LAND_SURFACE_TEMP() 
  call COMPUTE_ICE_SURFACE_TEMP() 
 endif
!---------------------------------------------------------
! constrain output
!---------------------------------------------------------
     alb1 = min(max_alb,max(min_alb,alb1))
     alb2 = min(max_alb,max(min_alb,alb2))
     alb3a = min(max_alb,max(min_alb,alb3a))
     alb3b = min(max_alb,max(min_alb,alb3b))
     alb3b_mask = min(max_alb,max(min_alb,alb3b_mask))
     ems3b = min(2.0,max(0.0,ems3b))
     bt3b = min(max_bt,max(min_bt,bt3b))
     bt4 = min(max_bt,max(min_bt,bt4))
     bt5 = min(max_bt,max(min_bt,bt5))

!------------------------------------------------------
! set night-time albedoes to zero (except alb3b)
!------------------------------------------------------
      where (solzen >= 90.0)
        alb1 = 0.0
        alb2 = 0.0
        alb3a = 0.0
      end where


!*************************************************************************
! SECTION 8
! DETERMINE PIXEL SURFACE TYPE, ELEVATION, AND LAND MASK FOR EACH PIXEL
! IN SCANLINE PAIR
!*************************************************************************

!--- compute land, desert,snow and terrain masks for these scanlines
 do j = 1, size(land_mask,2)
   do i = 1,size(land_mask,1)

!------ compute pixel surface type
    ilon_land_mask = max(1,min(num_lon_land_mask,int((lon(i,j) - first_lon_land_mask)/del_lon_land_mask) + 1))
    ilat_land_mask = max(1,min(num_lat_land_mask,int((lat(i,j) - first_lat_land_mask)/del_lat_land_mask) + 1))

    sfc_type(i,j) = land_cover_umd(ilon_land_mask,ilat_land_mask)

!------------------------------------------------------------------------------------
!-- compute pixel elevation
!------------------------------------------------------------------------------------
    if (lon(i,j) < 0.0) then
        xlon = lon(i,j) + 360.0
    else
        xlon = lon(i,j)
    endif

    ilon_elev = max(1,min(num_lon_elev,int((xlon - first_lon_elev)/del_lon_elev) + 1))
    ilat_elev = max(1,min(num_lat_elev,int((lat(i,j) - first_lat_elev)/del_lat_elev) + 1))
    zsfc(i,j) = elev_map(ilon_elev,ilat_elev)

!-------------------------------------------------------------------------------------------------
! climatological sst
!--------------------------------------------------------------------------------------------------
   sst_clim(i,j) = 270.0
   if ((lat(i,j) < 85.0) .and. (lat(i,j) > -80.0)) then
    ilon_sst_clim = max(1,min(num_lon_sst_clim,int((xlon - first_lon_sst_clim)/del_lon_sst_clim) + 1))
    ilat_sst_clim = max(1,min(num_lat_sst_clim,int((lat(i,j) - first_lat_sst_clim)/del_lat_sst_clim) + 1))
    sst_clim(i,j) = max(270.0,sst_clim_map(month,ilon_sst_clim,ilat_sst_clim))
   endif

!------------------------------------------------------------------------------------
!--- assign land/water based on surface type
!------------------------------------------------------------------------------------
    if (sfc_type(i,j) > 0) then
     land_mask(i,j) =  1
    else
     land_mask(i,j) =  0
    endif


!------------------------------------------------------------------------------------
!--- fill in snow input from sfc type
!------------------------------------------------------------------------------------
    if (sfc_type(i,j) == 14) then
      snow_input(i,j) = 1
    else
      snow_input(i,j) = 0
    endif


!------------------------------------------------------------------------------------
!--- compute desert flags from sfc type
!------------------------------------------------------------------------------------
    if ((sfc_type(i,j)==12)) then
       desert(i,j) = 1
    else
       desert(i,j) = 0
    endif 

    if ((sfc_type(i,j)==12).or.(sfc_type(i,j)==8).or.(sfc_type(i,j)==9) ) then
       desert3b(i,j) = 1
    else
       desert3b(i,j) = 0
    endif 


   enddo

 enddo

!*************************************************************************
! SECTION 9
! DETERMINE CLOUD MASK and CLOUD TYPE FOR EACH PIXEL IN SCANLINE PAIR
!*************************************************************************

!------------------------------------------------------------------------------------
!--- call static cloud mask
!------------------------------------------------------------------------------------

   clavr_status = 0

   call CLOUD_MASK_22(max_lst,max_sst)

   clavr_status = 1

!------------------------------------------------------------------------------
!--- compute running mean of max sst and lst for cloud masking
!------------------------------------------------------------------------------
   call MAX_SFC_TEMP(max_sst,std_sst,max_lst,std_lst)

!-------------------------------------------------------------------------------
!--- call dynamic thermal cloud mask enhancement
!-------------------------------------------------------------------------------
  if (dyn .eqv. .true.) then
    call DYN_WIN1_CLOUD_MASK(lat_anchor(26,1), &
                             max_sst,max_lst,&
                             std_sst,std_lst)
  endif

!-------------------------------------------------------------------------------
!--- call cloud typing algorithm
!------------------------------------------------------------------------------
    if (type .eqv. .true.) then
       call CLOUD_TYPE()
    endif


!*************************************************************************
! SECTION 10 
! COMPOSITE PIXEL VALUES INTO GRID_CELLS
!*************************************************************************
if (clavr .eqv. .true.) then
   call COMPILE_GRIDCELL_ARRAYS(nmask_clavr,num_pix)
endif

!*************************************************************************
! SECTION 11
! PACK CLOUD MASK AND CLOUD TYPE INTO ALLOCATED BYTES (klm only)
!*************************************************************************
  if (bx .eqv. .true.) then

   word_offset = rec_per_two_scans * rec_length / 2

   do j = 1,2

!----- clavr status flag
    iword =  nword_clavr_start + 3 + word_offset*(j-1)
    buffer(iword) = clavr_status(j)

!----- cloud mask 
    ipix = 0
    do iword = nword_clavr_start + 8 + word_offset*(j-1),  &
              nword_clavr_start+8+nword_clavr + word_offset*(j-1)

      buffer(iword) = 0
      do i = 1,min(4,num_pix-ipix) 
         buffer(iword) = buffer(iword) + ishft(min(cld_mask_max,cld_mask(ipix+i,j)),8-2*i)
      enddo 

!-------------------------------
      ipix = ipix + 4
    enddo

   enddo
  endif  

   
!*************************************************************************
! SECTION 12
! COMPOSITE PIXEL VALUES INTO GRID_CELLS
!*************************************************************************
if (clavr .eqv. .true.) then
   call COMPILE_GRIDCELL_ARRAYS(nmask_clavr,num_pix)
endif

!*************************************************************************
! SECTION 13
! PIXEL_LEVEL OUTPUT
!*************************************************************************
  if (bx .eqv. .true.) then
   write(unit=11,rec=irec_1bx) buffer(1:rec_length)    
   write(unit=11,rec=irec_1bx+1) buffer(rec_length+1:2*rec_length)    
   irec_1bx = irec_1bx +  rec_per_two_scans
  endif  
!------------------------------------------------------------------------
!--- write to pixel level cloud mask file
!-------------------------------------------------------------------------
     if (pcm .eqv. .true.) then
      do j = 1,2
       irec_pcm = irec_pcm + 1
       write (unit=12,rec=irec_pcm) scan_number(j),cld_mask(:,j),cld_type(:,j), &
                                   data_bit(:,j),mask_byte_1(:,j), &
                                   mask_byte_2(:,j),mask_byte_3(:,j)
      enddo
     endif
!--------------------------------------------------------------------------
! write to sst file
!-------------------------------------------------------------------------
  if (high_res_sst .eqv. .true.) then
   do j = 1,2
    irec_sst = irec_sst + 1
    write(unit=14,rec=irec_sst + 1) scan_number(j), ascend(j),time(j),ch3a_on(j),cld_mask(:,j),  &
                               sfc_type(:,j), lat(:,j), lon(:,j), solzen(:,j),sst(:,j)

   enddo
  endif
  nrec_sst = irec_sst

!-----------------------------------------------------------------------
!--- write out to diagnostic files
!-----------------------------------------------------------------------
    if ((ioutput == 1).and.(idiag ==1)) then
      call WRITE_TO_DIAG_FILES(j_output)
    end if

   end do records

   close(unit=9)
   close(unit=11)

   num_scans_pcm = irec_pcm - 1
   print *, "number of scanlines written to cm = ",num_scans_pcm
   print *,"number of scanlines written to diagnostic output = ", j_output-1

!*************************************************************************
! SECTION 14
! COMPUTE and WRITE GRID-CELL STATISTICS FOR CLAVR FILE
!*************************************************************************
if (clavr .eqv. .true.) then
   call COMPUTE_WRITE_GRIDCELL_ARRAYS(orbit_number,start_day,end_day, &
                                      start_year,end_year,count_thresh, &
                                      nmask_clavr,sc_id_new,nrec_clavr)

endif
!*************************************************************************
! SECTION 15
! WRITE OUT HEADERS FOR PIXEL-LEVEL OUTPUT
!*************************************************************************

!------------------------------------------------------------------------
!--- write out header for clavr file
!------------------------------------------------------------------------
if (clavr .eqv. .true.) then
  write(unit=13,rec=1) clavr_file_type,rec_length_clavr, &
                 sc_id_new,data_type,orbit_number,&
                 start_time,start_day,start_year,end_time,end_day,end_year, &
                 dlat,nequal,nrec_clavr,solar_3b,ew_3b,nu_3b,&
                 a1_3b,a2_3b,nu_4,a1_4,a2_4,nu_5,a1_5,a2_5,c1,c2

  close(unit=13)
endif

!------------------------------------------------------------------------
!--- write out header for pcm file
!------------------------------------------------------------------------
  if (pcm .eqv. .true.) then
     write(unit=12,rec=1) sc_id_new,data_type,orbit_number,start_year, &
                 start_day,start_time,num_scans_pcm,end_year,end_day,end_time, &
                 tip_parity,aux_sync,ramp_auto_cal,therm_cal_1b,dyn,type
     close(unit=12)
 endif
!--------------------------------------------------------------------------
! write out header for sst file
!--------------------------------------------------------------------------
  if (high_res_sst .eqv. .true.) then
     write(unit=14,rec=1) sc_id_new,data_type,orbit_number,start_year, &
                 start_day,start_time,end_year,end_day,end_time, &
                 tip_parity,aux_sync,ramp_auto_cal,dyn,type, &
                 nrec_sst,a1_day,a2_day,a3_day,a4_day, &
                 a1_night,a2_night,a3_night,a4_night,a5_night
     close(unit=14)
 endif

!------------------------------------------------------------------------
!-- if in diagnostic mode, exit
!------------------------------------------------------------------------
  if (idiag == 1) then
       exit
  endif
 
 end do file_loop
 close(unit=55)

!*************************************************************************
! SECTION 16
! DEALLOCATE MEMORY AND END PROGRAM
!*************************************************************************

!-------------------------------------------------------------------------
!--- Deallocate memory from pixels arrays
!-------------------------------------------------------------------------
  call DESTROY_PIXEL_ARRAYS() 

!-------------------------------------------------------------------------
!--- Deallocate memory from grid-cell arrays
!-------------------------------------------------------------------------
 if (clavr .eqv. .true.) then
    call DESTROY_GRIDCELL_ARRAYS()
 endif

 end program PROCESS_AVHRR_CLAVR
