/******************************************************************************
FUNCTION:	CLAVR

PURPOSE:	Create a cloud code mask for AVHRR single scene or composite
		images.  The algorithm uses a modified NOAA CLAVR-1 and
		CLAVR-x algorithm

INPUT:		From parameter block
		    IN - input file name
		    SATNUM - NOAA satellite number
		    ACQDATE - acquisition date
		    SCALING - input image scaling type
		    THRESHOLD - optional threshold file name
		    BARREN - optional barren image name
		    OUT - optional output image name
		From ADAPSTABLES
		    default threshold values

OUTPUT:		If OUT is specified, a cloud code mask the same size and space
		    as the input image.
		If OUT is not specified, an additional band containing the 
		    cloud code mask is added to the input image.

RETURNS:	E_SUCC on success
		E_FAIL on failure

REFERENCES:
    Modified CLAVR-1 and CLAVR-x Cloud Masking Algorithm for AVHRR Single
    Scenes and Composites Software Requirements Specification, May 6, 2003.
    Located in DOCS directory.

    Modified CLAVR-1 and CLAVR-x Cloud Masking Algorithm for AVHRR Single
    Scenes and Composites Design Document, May 13, 2003.  Located in DOCS
    directory.

    Scientific Basis and Initial Evaluation of the CLAVR-1 Global Clear/Cloud
    Classification Algorithm for the Advanced Very High Resolution Radiometer,
    Stowe, Larry L., et al. Journal of Atmospheric and Oceanic Technology, Vol
    16, June 99, pp 656-681.  Hard copy maintained in ADAPS files.

    NOAA KLM User's Guide.  http://www2.ncdc.noaa.gov/docs/klm/index.htm

    CLAVR-1 source code obtained from Andrew Heidinger, NOAA.  Source is
    maintained in clavr/CLAVR/CLAVR1 directory.

    CLAVR-x source code obtained from Andrew Heidinger, NOAA.  Source is
    maintained in clavr/CLAVR/CLAVRX directory.

PROGRAM HISTORY:

******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "util2.h"
#include "worgen.h"
#include "clavr.h"
#include "calcoef.h"
#include "model.h"

/* REMOVE this when reprocessing of NOAA 16 composites is complete */
int channel3override = 0;

int main( void )
    {
    char    barren_name[CMLEN];     /* barren mask file name                */
    char    default_threshold[CMLEN];/* default threshold file name         */
    char    input_file[CMLEN];      /* input image file name                */
    char    output_name[CMLEN];     /* output image file name               */
    char    temporary_file[CMLEN];  /* temporary output file name           */
    char    user_threshold[CMLEN];  /* user-supplied threshold file name    */

    int     barren_mask_flag;       /* flag: barren mask file specified     */
                                    /* values: TRUE or FALSE                */
    int     input_image_type;       /* flag: type of input image, values:   */
                                    /* SINGLE_SCENE or COMPOSITE            */
    int     julian_day;             /* julian day of year from acquisition  */
    int     output_file_flag;       /* flag: output file name specified     */
                                    /* values: TRUE or FALSE                */
    int     satellite_number;       /* NOAA satellite number designations   */
    int     scaled_flag;            /* flag: input file scaled or not       */
                                    /* values: TRUE or FALSE                */
    int     status;                 /* function return status E_SUCC, E_FAIL*/
    int     user_threshold_flag;    /* flag: user supplied threshold file   */
                                    /* values: TRUE or FALSE                */

    long    idtype;		    /* input image data type                */
    long    input_buffer_offsets[13];/* offsets into input image buffer     */
    long    msgtim;                 /* time between processing messages     */
    long    minorframe_fdesc = 0;   /* minor frame file descriptor          */
    long    output_buffer_offset[1];/* offset into output image buffer      */
    long    std_out;                /* Standard output                      */
    long    window[WINDOW];	    /* input window specification	    */

    struct  DDR barren_ddr;         /* barren mask ddr                      */
    struct  DDR input_ddr;	    /* input image ddr                      */
    struct  FDESC *fdesc[4] = {0};  /* file descriptors for all images      */
    struct  GDESC *group_fdesc;     /* group file descriptor                */
    struct  MODEL input_model;      /* input image model (from AVHRR ddr)   */
    struct  PARBLK  vblock;         /* TAE paramter block                   */
    struct  SCOFF scale_factors[NUM_SCALEINDICES];
    				    /* structure of scaling factors         */
    struct  THRESHOLD thresholds;   /* threshold structure                  */

    unsigned char *thebuf;          /* image I/O buffer                     */
    


    /* Initialize las and retrieve the parameters
    ---------------------------------------------*/
    c_inlas( &vblock, &std_out );
    status = getpar( &vblock, input_file, &input_ddr, &input_model,
                     user_threshold, default_threshold, barren_name,
		     &barren_ddr, output_name, &input_image_type,
		     window, &user_threshold_flag, &barren_mask_flag,
		     &output_file_flag, &scaled_flag, scale_factors,
		     &satellite_number, &julian_day, &msgtim );
    if ( status == E_FAIL )
        {
	c_lasmsg(TRUE,"clavr-getpar","Error retrieving input parameters");
	return E_FAIL;
	}


    /* Retrieve the default threshold values for clavr
    and, if specified, the user-supplied values.
    --------------------------------------------------*/
    status = getthresh( user_threshold, user_threshold_flag,
                        default_threshold, &thresholds );
    if ( status == E_FAIL )
        {
	c_lasmsg(TRUE,"clavr-thresh","Error retrieving threshold values");
	return E_FAIL;
	}
    

    /* Open and group the input images.  This function
    also allocates buffer space for I/O.
    --------------------------------------------------*/
    status = openimages( &vblock, input_file, barren_name, output_name,
                         barren_mask_flag, output_file_flag, input_image_type,
			 &idtype, window, fdesc, &group_fdesc,
			 &minorframe_fdesc, temporary_file,
			 input_buffer_offsets, output_buffer_offset, &thebuf );
    if ( status == E_FAIL )
        {
	/* clean issues a fatal message */
	las_clean( &vblock, "CLAVR", &temporary_file , 1 );
	return E_FAIL;
	}
    

    /* Process the input image and create either the
    output cloud mask image or the temporary one.
    ------------------------------------------------*/
    status = process( &group_fdesc, thebuf, input_buffer_offsets, 
                      output_buffer_offset, &minorframe_fdesc,
		      barren_mask_flag, input_image_type, scaled_flag,
		      scale_factors, thresholds, &input_ddr, &input_model,
		      window, julian_day, satellite_number, msgtim );
    if ( status == E_FAIL )
        {
        closeimages( fdesc, &minorframe_fdesc, input_file, input_image_type );
	/* clean issues a fatal message */
	las_clean( &vblock, "CLAVR", &temporary_file , 1 );
	return E_FAIL;
	}


    /* Close the images.  After this point, on any other failure,
    leave the output and temporary files as they are and let the
    user know what has been left lying around.
    -------------------------------------------------------------*/
    status = closeimages( fdesc, &minorframe_fdesc, input_file,
                          input_image_type );
    if ( status == E_FAIL )
        {
	c_lasmsg(TRUE,"clavr-close","Error closing images");
	return E_FAIL;
	}
    

    /* If the output file was not specified, the temporary
    output image must be added, as the last band, to the
    input image.  The easiest way is to copy IN+temp to 
    another temporary file, remove the original input file
    and rename the temporary file to original input file.
    ------------------------------------------------------*/
    if ( !output_file_flag )
        {
	status = combine_images( &vblock, input_file, temporary_file );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(TRUE,"clavr-combine",
	        "Error combining mask and input image");
	    return E_FAIL;
	    }
	}  /* END if !output_file_flag */


    /* Issue a completion message and exit las.  The
    return here is for supressing the compiler warning
    since c_complt does the actual exit.
    --------------------------------------------------*/
    if ( output_file_flag )
	las_complt( "CLAVR", &output_name, 1 );
    else
	las_complt( "CLAVR", &output_name, 0 );
    return E_SUCC;
    }  /* END main */
