/*****************************************************************************
Include file for the clavr process
******************************************************************************/
#ifndef CLAVR_H
#define CLAVR_H

#include "las.h"
#include "model.h"

/* General definitions
----------------------*/
#define SINGLE_SCENE   1
#define COMPOSITE      2
#define PI             3.141592653589793238
#define NADIR          1024	/* Nadir pixel location in AVHRR data	*/
#define NUMBANDS       11	/* number of input bands max		*/

/* ??? The question remains as to whether to use the CLAVR-x channel 3
temp to albedo conversion or the CLAVR-1 conversion for KLM satellites
in the klm_chan3temp2alb function.  To use CLAVR-x define CLAVRX.
**********************************************************************/
#define CLAVRX

/* Threshold structure definition.  It is important that
all of these match.  There must be NUMTHRESHOLDS enumerated
values in THRESHOLDNAME and NUMTHRESHOLDS strings in
threshold_names.  The enum's and names should be identical,
except for case, and in the same order.  USETHRESHOLDS 
should be defined in each file that needs reference to the
threshold names.  Use of the define prevents compiler 
warnings about duplicate definitions of threshold_names.
#define NUMTHRESHOLDS  13
-----------------------------------------------------------*/
enum THRESHOLDNAME { RGCT = 0, TGCR1, C3AR, C3AR_KLM, C3AR_KLM3A, GAMMA,
                     RRCT_MIN, RRCT_MAX, TGCR2, C3AT, C3AT_KLM, C3AT_KLM3A,
		     TGCT, LAT_MIN, LAT_MAX, NUMTHRESHOLDS };
#ifdef USETHRESHOLDS
const char *threshold_names[] =
     {"rgct","tgcr1","c3ar","c3ar_klm", "c3ar_klm3a","gamma","rrct_min",
      "rrct_max","tgcr2","c3at","c3at_klm","c3at_klm3a","tgct","lat_min",
      "lat_max"};
#else
extern const char *threshold_names[];
#endif

struct THRESHOLD {
    int flag[NUMTHRESHOLDS];	/* Indicates where the threshold value
				originated and is only used for filling the
				structure.  A value of 0 indicates the value
				is not set; 1 indicates the value is from
				default table; and 2 indicates the value is
				user-supplied */
    float value[NUMTHRESHOLDS];	/* Threshold value.  This array of values is
    				in the same order as the enumerated types */
    };


/* Structure used for storing scaling, offset quantities */
/*-------------------------------------------------------*/
struct SCOFF
   {
   long   scaling;		/* type of scaling: US, NONE, GLOBAL	 */
   long   odtype;		/* output data type			 */
   double minval;               /* Minimum value of appropriate quantity */
   double maxval;               /* Maximum value of appropriate quantity */
   double scale;                /* Scale value to apply                  */
   double offset;               /* Offset value to apply                 */
   double unscale;              /* Value to apply to unscale             */
   };

/* An array of SCOFF's will be declared and these are the
indices that will be used to setup and access the array.
---------------------------------------------------------*/
enum SCALEINDICES { SC_REFLECTANCE = 0, SC_THERMAL, SC_SATZEN, SC_RELAZ,
                    SC_SOLZEN, NUM_SCALEINDICES };


/* Satellite constant structure.  The satellite
specific constants for converting temperature
values to radiance and reflectance.
-----------------------------------------------*/
struct SATELLITE_CONSTANTS
    {
    double A;
    double B;
    double C;
    double D;
    double S3PI;
    double CNUO;
    double C1nuc_cubed;
    double C2nuc;
    double A0;
    double A1;
    double A2;
    double B0;
    double C0;
    double solar_3b_nu;
    };
/* Planck's blackbody constants.  Non-satellite specific
constants for converting temperature to radiance.
--------------------------------------------------------*/
#define C1 1.1910427e-5
#define C2 1.4387752


/* Function prototypes
----------------------*/
int getpar
    (
    struct  PARBLK  *vblock,    /* IN  - TAE parameter block            */
    char *input_file,           /* OUT - input image file name          */
    struct DDR *input_ddr,      /* OUT - input image ddr                */
    struct MODEL *input_model,  /* OUT - input image model              */
    char *user_threshold,       /* OUT - user-supplied threshold name   */
    char *default_threshold,    /* OUT - default threshold file name    */
    char *barren_name,          /* OUT - barren image file name         */
    struct DDR *barren_ddr,     /* OUT - barren image ddr               */
    char *output_name,          /* OUT - output image file name         */
    int *input_image_type,      /* OUT - input image type: SINGLE_SCENE */
                                /*       or COMPOSITE                   */
    long *window,		/* OUT - input window specification	*/
    int *user_threshold_flag,   /* OUT - flag: user-supplied threshold  */
                                /*       TRUE or FALSE                  */
    int *barren_mask_flag,      /* OUT - flag: barren mask file present */
                                /*       TRUE or FALSE                  */
    int *output_file_flag,      /* OUT - flag: output file specified    */
                                /*       TRUE or FALSE                  */
    int *scaled_flag,           /* OUT - flag: input file values scaled */
                                /*       TRUE or FALSE                  */
    struct SCOFF scale_factors[],/* OUT - scale factors structure       */
    int *satellite_number,      /* OUT - NOAA satellite number          */
                                /*       11, 12, 14, 15, 16 or 17       */
    int *julian_day,            /* OUT - julian day of acquisition      */
                                /*       range: [1...366]               */
    long *msgtim                /* OUT - time between processing messages*/
    );


int getthresh
    (
    char *user_threshold,       /* IN - user-supplied file name         */
    int user_threshold_flag,    /* IN - flag: user-supplied file        */
                                /*      values: TRUE or FALSE           */
    char *default_threshold,    /* IN - default threshold file name     */
    struct THRESHOLD *thresholds/* OUT - threshold structure            */
    );

int openimages
    (
    struct PARBLK *vblock,      /* IN - TAE parameter block             */
    char *input_file,           /* IN - input file name                 */
    char *barren_name,          /* IN - barren mask file name           */
    char *output_name,          /* IN - output file name                */
    int barren_mask_flag,       /* IN - flag: barren mask file name is  */
                                /*      specified, TRUE or FALSE        */
    int output_file_flag,       /* IN - flag: output file name is       */
                                /*      specified, TRUE or FALSE        */
    int input_image_type,       /* IN - flag: type of input image either*/
                                /*      SINGLE_SCENE or COMPOSITE       */
    long *idtype,               /* IN - input data type                 */
    long *window,               /* IN - image window                    */
    struct FDESC **fdesc,       /* OUT - array of file descriptors      */
    struct GDESC **group_fdesc, /* OUT - group file descriptor          */
    long *minorframe_fdesc,     /* OUT - minor frame file descriptor    */
    char *temporary_file,       /* OUT - file name for temporary output */
    long *input_offsets,        /* OUT - offsets into thebuf for input  */
    long *output_offsets,       /* OUT - offsets into thebuf for output */
    unsigned char **thebuf      /* OUT - image I/O buffer               */
    );

int closeimages
    (
    struct FDESC **fdesc,       /* IN - Null terminated array of descriptors*/
    long *minorframe_fdesc,     /* IN - minor frame file descriptor         */
    char *input_image,          /* IN - input image name                    */
    int input_image_type        /* IN - input image type: SINGLE_SCENE or   */
                                /*      COMPOSITE                           */
    );


int process
    (
    struct GDESC **group_fdesc, /* IN - grouped file descriptors        */
    unsigned char *thebuf,      /* IN - image I/O buffer                */
    long *input_buffer_offsets, /* IN - offsets into thebuf for input   */
    long *output_buffer_offset, /* IN - offsets into thebuf for output  */
    long *minorframe_fdesc,     /* IN - minor frame file descriptor     */
    int barren_mask_flag,       /* IN - flag: barren mask specified     */
                                /*      TRUE or FALSE                   */
    int input_image_type,       /* IN - flag: input image type          */
                                /*      SINGLE_SCENE or COMPOSITE       */
    int scaled_flag,            /* IN - flag: input image scaled        */
                                /*      TRUE or FALSE                   */
    struct SCOFF *scale_factors,/* IN - scaling factors                 */
    struct THRESHOLD thresholds,/* IN - threshold value structure       */
    struct DDR *input_ddr,      /* IN - input image ddr                 */
    struct MODEL *input_model,  /* IN - input image AVHRR model         */
    long *window,               /* IN - input window specification      */
    int julian_day,             /* IN - julian date of acquisition      */
    int satellite_number,       /* IN - NOAA satellite number           */
    long msgtim                 /* IN - time between processing messages*/
    );

float chan3temp2alb
    (
    float three,                /* IN - channel three temperature       */
    float four,                 /* IN - channel four temperature        */
    float five,                 /* IN - channel five temperature        */
    float sedsq,                /* IN - sun/earth distance squared      */
    float solzen,               /* IN - solar zenith angle (in radians) */
    struct SATELLITE_CONSTANTS constant/* IN - satellite constants      */
    );

float klm_chan3temp2alb
    (
    float chan3btemp,           /* IN - channel 3B temperature          */
    float chan4temp,            /* IN - channel 4 temperature           */
    float chan5temp,            /* IN - channel 5 temperature           */
    float sedsq,                /* IN - sun/earth distance squared      */
    float solar_zenith,         /* IN - solar zenith angle (in radians) */
    struct SATELLITE_CONSTANTS constants /* IN - satellite constants    */
    );

int getconstants
    (
    int satellite_number,       /* IN - satellite number                */
    struct SATELLITE_CONSTANTS *constants /* OUT - constant structure   */
    );

int clavr
    (
    float r1,                   /* IN - channel 1 albedo                */
    float r2,                   /* IN - channel 1 albedo                */
    float r3,                   /* IN - channel 1 albedo                */
    float t4,                   /* IN - channel 4 temperature           */
    float t5,                   /* IN - channel 5 temperature           */
    float lat,                  /* IN - latitude                        */
    float lon,                  /* IN - longitude                       */
    float gamma,                /* IN - gamma angle                     */
    int barren,                 /* IN - barren mask flag                */
    int sat,                    /* IN - NOAA satellite number           */
    int band3isthermal,		/* IN - flag: TRUE/FALSE		*/
    struct THRESHOLD thresholds /* IN - threshold structure             */
    );

int combine_images
    (
    struct PARBLK *vblock,	/* IN - TAE parameter block		*/
    char *input_file,           /* IN - input file name                 */
    char *temporary_file        /* IN - temporary output file name      */
    );

#endif
