/*****************************************************************************
FUNCTION:	CLAVR

PURPOSE:	Compute the cloud code using modified CLAVR-1 and CLAVR-x
		algorithms.

INPUT:		channel 1 albedo (in percent)
		channel 2 albedo (in percent)
		channel 3 albedo (in percent)
		channel 4 temperature (in Kelvin)
		channel 5 temperature (in Kelvin)
		latitude (in degrees)
		longitude (in degrees)
		gamma angle (in degrees)
		barren flag (1 or 0)
		satellite_number
		threshold structure

OUTPUT:		none

RETURNS:	cloud code

*****************************************************************************/
#include <math.h>
#include "clavr.h"


int clavr
    (
    float r1,			/* IN - channel 1 albedo		*/
    float r2,			/* IN - channel 1 albedo		*/
    float r3,			/* IN - channel 1 albedo		*/
    float t4,			/* IN - channel 4 temperature		*/
    float t5,			/* IN - channel 5 temperature		*/
    float latitude,		/* IN - latitude			*/
    float longitude,		/* IN - longitude			*/
    float gamma,		/* IN - gamma angle			*/
    int barren,			/* IN - barren mask flag		*/
    int sat,			/* IN - NOAA satellite number		*/
    int band3isthermal,		/* IN - flag: TRUE/FALSE		*/
    struct THRESHOLD thresholds	/* IN - threshold structure		*/
    )
    {
    int code;			/* cloud code				*/
    int desert;			/* flag: is desert or not, 1 or 0	*/
    int foft4_index;		/* index into the function of t4	*/

    float c3ar;			/* threshold that depends on satellite	*/
    float c3at			/* threshold that depends on satellite	*/;

    /* This is the function of t4.  Pulled directly from original
    CLAVR-1 code and changed from fortran to C.
    ------------------------------------------------------------*/
    static float foft4[] = { 0.00,
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
          0.01, 0.03, 0.05, 0.08, 0.11, 0.14, 0.18, 0.23, 0.28, 0.34,
          0.41, 0.48, 0.57, 0.66, 0.76, 0.87, 1.00, 1.13, 1.27, 1.42,
          1.59, 1.76, 1.94, 2.14, 2.34, 2.55, 2.77, 3.00, 3.24, 3.48,
          3.73, 3.99, 4.26, 4.52, 4.80, 5.08, 5.35, 5.64, 5.92, 6.20,
          6.48, 6.76, 7.03, 7.30, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80,
          7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80 };


    /* Compute the index into the foft4 function.  The first function
    value is for 200K, last value 320K.  For temperatures less than 200K
    set index to 0 and for values greater than 320 set index to 119.
    Round channel four temperatures, which are always positive, to the
    nearest integral value while computing the index.
    --------------------------------------------------------------------*/
    foft4_index = (int)(t4 + 0.5) - 200;
    if ( foft4_index < 0 )
        foft4_index = 0;
    else if ( foft4_index > 120 )
        foft4_index = 120;

    
    /* Choose the appropriate channel 3 albedo
    thresholds based on satellite series.
    ------------------------------------------*/
    switch ( sat )
        {
	case 15 :
	case 16 :
        case 17 :
	case 18 :
	    if ( band3isthermal )
		{
		c3ar = thresholds.value[C3AR_KLM];
		c3at = thresholds.value[C3AT_KLM];
		}
	    else
		{
		c3ar = thresholds.value[C3AR_KLM3A];
		c3at = thresholds.value[C3AT_KLM3A];
		}
	    break;
        case 19 :
            if ( band3isthermal )
                {
                c3ar = thresholds.value[C3AR_KLM];
                c3at = thresholds.value[C3AT_KLM];
                }
            else
                {
                c3ar = thresholds.value[C3AR_KLM3A];
                c3at = thresholds.value[C3AT_KLM3A];
                }
            break;
	default :
	    c3ar = thresholds.value[C3AR];
	    c3at = thresholds.value[C3AT];
	    break;
	}


    /* Start with code equal to one and add to it based on result of following
    tests.  See either the SRS or Design document App. A for the algorithm
    diagram.  The algorithm was designed such that codes less than 100 are
    clear of restored clear codes and those greater than or equal to 100 are
    cloud codes.
    ------------------------------------------------------------------------*/
    code = 1;

    if ( r1 > thresholds.value[RGCT] )
        {
	code += 99;
	if ( t4 > thresholds.value[TGCR1] )
	    {
	    code += 10;
	    }
	else
	    {
	    if ( r3 < c3ar )
		{
		code += 20;
		}
	    else
	        {
		if ( latitude < thresholds.value[LAT_MIN] &&
		     gamma < thresholds.value[GAMMA] )
		    code += 30;
		else
		    return code;
		}
	    }
	if ( t4 - t5 > foft4[foft4_index] )
	    {
	    code += 1;
	    return code;
	    }
	if ( latitude > thresholds.value[LAT_MAX] )
	    {
	    code -= 99;
	    return code;
	    }
	if ( t4 < thresholds.value[TGCT] )
	    {
	    code += 3;
	    return code;
	    }
	code -=100;
	return code;
	}  /* END if ( r1 > thresholds.value[RGCT] ) */
    
    /* Determine desert.  The NOAA algorithm defines several desert
    regions based on latitude and longitude.  The following lat/lon
    bounding boxes are considered to be desert.
    ---------------------------------------------------------------*/
    if ((latitude >=  10.0 &&  latitude <=   35.0 &&
        longitude >=  20.0 && longitude <=   30.0 ) ||
	(latitude >=   5.0 &&  latitude <=   50.0 &&
	longitude >=  30.0 && longitude <=   60.0 ) ||
	(latitude >=  25.0 &&  latitude <=   50.0 &&
	longitude >=  60.0 && longitude <=  110.0 ) ||
	(latitude >= -31.0 &&  latitude <= -19.0 &&
	longitude >= 121.0 && longitude <= 141.0 ))
        {
	desert = 1;
	}
    else
        desert = 0;
    
    /* The barren term was established at EDC and is used the
    in the same manner as the NOAA defined desert regions.
    ---------------------------------------------------------*/
    if ( barren || desert )
	{
	code += 1;
	}
    else
        {
	if ((( r2 / r1 ) >= thresholds.value[RRCT_MIN] ) &&
	    (( r2 / r1 ) <  thresholds.value[RRCT_MAX] ))
	    {
	    code += 100;
	    if ( t4 > thresholds.value[TGCR2] )
		{
		code += 39;
		}
	    else
	        {
		if ( r3 < c3ar )
		    {
		    code += 49;
		    }
		else
		    {
		    if ( latitude < thresholds.value[LAT_MIN] &&
		         gamma < thresholds.value[GAMMA] )
			code += 59;
		    else
			return code;
		    }
		}
	    if ( t4 - t5 > foft4[foft4_index] )
		{
		code += 1;
		return code;
		}
	    if ( latitude > thresholds.value[LAT_MAX] )
		{
		code -= 99;
		return code;
		}
	    if ( t4 < thresholds.value[TGCT] )
		{
		code += 3;
		return code;
		}
	    code -=100;
	    return code;
	    }  /* END if ((( r2 / r1 ) >= thresholds... */
	
	if ( r3 > c3at )
	    {
	    code += 101;
	    if ( t4 > thresholds.value[TGCR2] )
	        code += 68;
	    else
	        return code;
	    if ( t4 - t5 > foft4[foft4_index] )
		{
		code += 1;
		return code;
		}
	    if ( latitude > thresholds.value[LAT_MAX] )
		{
		code -= 99;
		return code;
		}
	    if ( t4 < thresholds.value[TGCT] )
		{
		code += 3;
		return code;
		}
	    code -= 100;
	    return code;
	    }  /* END if ( r3 > c3at ) */
	}  /* END if ( !barren && !desert ) */

    if ( t4 - t5 > foft4[foft4_index] )
        {
	code += 102;
	return code;
	}
    
    if ( latitude < thresholds.value[LAT_MAX] )
        {
	code += 2;
	if ( t4 < thresholds.value[TGCT] )
	    {
	    code += 102;
	    return code;
	    }
	}
    
    return code;
    }  /* END clavr */
