/******************************************************************************
FUNCTION:	COMBINE_IMAGES

PURPOSE:	Combine the input and temporary output images into a single
		image.  The resulting image is renamed as the original input
		image.

INPUT:		TAE variable block
		input image name
		temporary image name

OUTPUT:		combined image is written

RETURNS:	E_SUCC on success
		E_FAIL on failure

PROGRAM HISTORY:

******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "diskio.h"
#include "clavr.h"


int combine_images
    (
    struct PARBLK *vblock,	/* IN - TAE parameter block		*/
    char *input_file,		/* IN - input file name			*/
    char *temporary_file	/* IN - temporary output file name	*/
    )
    {
    char dir[CMLEN];		/* file directory specification		*/
    char ext[CMLEN];		/* file extension specification		*/
    char file[CMLEN];		/* file name returned by getfiles	*/
    char hosin[2][CMLEN];	/* names of files to copy		*/
    char hosout[CMLEN];		/* name of file to copy to		*/
    char input_fullname[CMLEN];	/* full file specification of input file*/
    char input_head[CMLEN];	/* directory and root of input file	*/
    char logfile[CMLEN];	/* full file specification of log file	*/
    char message[CMLEN];	/* message string for lasmsg		*/
    char minmax[4] = {0};	/* minmax flag for tae			*/
    char root[CMLEN];		/* file root specification		*/
    char wildfile[CMLEN];	/* wildcarded file specification	*/

    int bands[2][MAXBND+1] = {{0}};/* copy all bands			*/
    int nbands[2] = {0};	/* copy all bands in all images		*/
    int pid = getpid();		/* process ID for temporary name	*/
    int window[2][WINDOW] = {{0}};/* copy entire image			*/
    int nfiles;			/* number of files returned by getfiles	*/
    int status;			/* function return status		*/
    int success;		/* flag: while loop success values are	*/
    				/*       TRUE or FALSE			*/



    /* Combine input and temporary output
    -------------------------------------*/
    strcpy( hosin[0], input_file );
    strcpy( hosin[1], temporary_file );
    sprintf( hosout, "CLAVR%d_tmp.img", pid );
    status = c_copy_image( vblock, hosin, hosout,
			   FALSE, /* don't compress the image */
			   2,     /* two input images */
			   window,  bands, nbands,
			   0,     /* default bands */
			   SAME,  /* same data type */
			   FALSE, /* truncate input data */
			   10,    /* ten second message display */
			   minmax );
    if ( status == E_FAIL )
	{
	c_lasmsg(FALSE,"clavr-copy","Unable to combine input and mask");
	c_lasmsg(FALSE,"clavr-info","Input file has not been modified");
	sprintf(message,"Cloud mask is in image %s", temporary_file);
	c_lasmsg(FALSE,"clavr-info",message);
	return E_FAIL;
	}
    
    /* A special case of US composites is a log file that is usually
    named with the same root file name as the composite with a .log
    extension.  The copy image does not carry this file along with it
    so make a specific copy of this file, if it exists, since it will
    be unconditionally removed by the ensuing code.
    -----------------------------------------------------------------*/
    c_splitname( input_file, dir, root, ext );
    sprintf( logfile, "%s%s.log", dir, root );
    status = c_getfiles( logfile, file, &nfiles, 1, TRUE, FALSE );
    if ( status != E_SUCC )
        c_lasmsg(FALSE,"clavr-getfile","Error getting log file");
    else if ( nfiles == 1 )
        {
	c_splitname( hosout, dir, root, ext );
	sprintf( logfile, "%s%s.log", dir, root );
	status = c_dkcopy( file, logfile, dir );
	if ( status != E_SUCC )
	    c_lasmsg(FALSE,"clavr-copy","Error making copy of log file");
	}

    
    /* The copy was successful so remove the input file
    ---------------------------------------------------*/
    c_splitname( input_file, dir, root, ext );
    sprintf( wildfile, "%s%s.*", dir, root );
    success = TRUE;
    /* get the first matching file */
    status = c_getfiles( wildfile, file, &nfiles, 1, TRUE, FALSE );
    if ( status != E_SUCC || nfiles == 0 )
	success = FALSE;
    while ( nfiles > 0 )
        {
	
	/* remove the file */
	status = c_dkdel( file );
	if ( status != E_SUCC )
	    {
	    success = FALSE;
	    break;
	    }
	
	/* get a matching file */
	status = c_getfiles( wildfile, file, &nfiles, 1, TRUE, FALSE );
	if ( status != E_SUCC )
	    {
	    success = FALSE;
	    break;
	    }
	}  /* END while */
    if ( !success )
        {
	c_lasmsg(FALSE,"clavr-delete","Error deleting input file");
	sprintf(message,"Combined Input and Cloud mask is: %s", hosout);
	c_lasmsg(FALSE,"clavr-info",message);
	sprintf(message,"Cloud mask is: %s", temporary_file);
	c_lasmsg(FALSE,"clavr-info",message);
	return E_FAIL;
	}
    

    /* The input file was successfully deleted so move the temporary
    file to the input file.  Dir and root already have the input file
    specification in them.so set up a spec less the extension.
    -----------------------------------------------------------------*/
    sprintf( input_head, "%s%s.", dir, root );
    c_splitname( hosout, dir, root, ext );
    sprintf( wildfile, "%s%s.*", dir, root );
    success = TRUE;
    /* get the first matching file */
    status = c_getfiles( wildfile, file, &nfiles, 1, TRUE, FALSE );
    if ( status != E_SUCC || nfiles == 0 )
	success = FALSE;
    while ( nfiles > 0 )
        {
	/* move the file.  need to get the extension */
	c_splitname( file, dir, root, ext );
	sprintf( input_fullname, "%s%s", input_head, ext );
	status = c_dkmove( file, input_fullname, NULL );
	if ( status == E_FAIL )
	    {
	    success = FALSE;
	    break;
	    }

	/* get the next matching file */
	status = c_getfiles( wildfile, file, &nfiles, 1, TRUE, FALSE );
	if ( status != E_SUCC )
	    {
	    success = FALSE;
	    break;
	    }
	}  /* END while */
    if ( !success )
        {
	c_lasmsg(FALSE,"clavr-move","Unable to rename combined image");
	sprintf(message,"Combined image is %s", hosout);
	c_lasmsg(FALSE,"clavr-info",message);
	sprintf(message,"Cloud mask is: %s", temporary_file);
	c_lasmsg(TRUE,"clavr-info",message);
	return E_FAIL;
	}
    

    /* The final image was successfully
    moved so delete the temporary file.  
    -----------------------------------*/
    c_splitname( temporary_file, dir, root, ext );
    sprintf( wildfile, "%s%s.*", dir, root );
    success = TRUE;
    /* get the first matching file */
    status = c_getfiles( wildfile, file, &nfiles, 1, TRUE, FALSE );
    if ( status != E_SUCC || nfiles == 0 )
	success = FALSE;
    while ( nfiles > 0 )
        {
	/* remove the file */
	status = c_dkdel( file );
	if ( status != E_SUCC )
	    {
	    success = FALSE;
	    break;
	    }
	
	/* get the next matching file */
	status = c_getfiles( wildfile, file, &nfiles, 1, TRUE, FALSE );
	if ( status != E_SUCC )
	    {
	    success = FALSE;
	    break;
	    }
	}  /* END while */
    if ( !success )
        {
	c_lasmsg(FALSE,"clavr-delete","Error deleting temp cloud mask file");
	sprintf(message,"Cloud mask is: %s", temporary_file);
	c_lasmsg(FALSE,"clavr-info",message);
	return E_FAIL;
	}
    

    return E_SUCC;
    }  /* END combine_images */
