/*****************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Retrieve the parameters from the TAE variable block, and do
                initial validation of parameters.  Determine input image type,
		insure input images exist and are valid and check that output
		file, if specified, does not exist.

INPUT:		vblock

OUTPUT:		input file name
		input file ddr
		input file model
		user-suplied threshold file name
		default threshold file name
		barren mask file name
		barren mask ddr
		output image file name
		input image type
		window
		threshold flag
		barren mask flag
		output file flag
		scaling flag
		scaling factors
		satellite number
		julian day of acquisition date
		processing message interval

RETURNS:	E_SUCC on success
		E_FAIL on failure

******************************************************************************/
#include <string.h>
#include "las.h"
#include "ddr.h"
#include "util.h"
#include "util2.h"
#include "labserv.h"
#include "archio.h"
#include "adapssup.h"
#include "model.h"
#include "calcoef.h"
#include "clavr.h"

/* REMOVE this when reprocessing of NOAA 16 composites is complete */
extern int channel3override;

int getpar
    (
    struct  PARBLK  *vblock,	/* IN  - TAE parameter block		*/
    char *input_file,		/* OUT - input image file name		*/
    struct DDR *input_ddr,	/* OUT - input image ddr		*/
    struct MODEL *input_model,	/* OUT - input image model		*/
    char *user_threshold,	/* OUT - user-supplied threshold name	*/
    char *default_threshold,	/* OUT - default threshold file name	*/
    char *barren_name,		/* OUT - barren image file name		*/
    struct DDR *barren_ddr,	/* OUT - barren image ddr		*/
    char *output_name,		/* OUT - output image file name		*/
    int *input_image_type,	/* OUT - input image type: SINGLE_SCENE	*/
    				/*       or COMPOSITE			*/
    long *return_window,	/* OUT - input image window spec.	*/
    int *user_threshold_flag,	/* OUT - flag: user-supplied threshold	*/
    				/*       TRUE or FALSE			*/
    int *barren_mask_flag,	/* OUT - flag: barren mask file present	*/
    				/*       TRUE or FALSE			*/
    int *output_file_flag,	/* OUT - flag: output file specified	*/
    				/*       TRUE or FALSE			*/
    int *scaled_flag,		/* OUT - flag: input file values scaled	*/
    				/*       TRUE or FALSE			*/
    struct SCOFF scale_factors[],/* OUT - scale factors structure	*/
    int *satellite_number,	/* OUT - NOAA satellite number		*/
    				/*       11, 12, 14, 15, 16 or 17	*/
    int *julian_day,		/* OUT - julian day of acquisition	*/
    				/*       range: [1...366]		*/
    long *msgtim                /* OUT - time between processing messages*/
    )
    {
    char *adapstables;		/* pointer to ADAPSTABLES environment	*/
    char addr_file[CMLEN];	/* name of associated file addr		*/

    int i;			/* loop variable			*/
    int num_out;		/* number of out images from getout call*/
    int status;			/* function return status		*/

    long month,day,year,jday;	/* acquisition date for c_grejul call	*/
    long soption;		/* scaling option for getscale call	*/

    struct VARIABLE *var;	/* pointer to a TAE variable		*/

    /* Variables specifically for c_getin calls */
    char hnames[1][CMLEN];	/* host names from parameter		*/
    char tnames[1][CMLEN];	/* TAE names from parameter		*/
    long bands[1][MAXBND+1];	/* band numbers selected for each image	*/
    long element = 0;		/* array element is zero in C		*/
    long imglimit = 1;		/* max number of images to be handled	*/
    long nbands;		/* num bands specified for each image	*/
    long nmrimg;		/* number of images in image spec	*/
    long qflag[1][2];		/* window and bands option flags	*/
    long same_window = 1;	/* all images must have same window	*/
    long totbnd;		/* total bands to be processed		*/
    long window[1][WINDOW];	/* window specificaton			*/


    /* Retrieve the input file name and check the ddr.
    --------------------------------------------------*/
    status = c_getin( vblock, "IN", &element, tnames, hnames, &nmrimg, window,
                      bands, &nbands, &totbnd, &imglimit, &same_window, qflag );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-getin","Unable to get input image specification");
	return E_FAIL;
	}
    strcpy( input_file, hnames[0] );
    status = c_getddr( input_file, input_ddr );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-ddr","Could not get input file DDR");
	return E_FAIL;
	}
    
    /* Copy the window specified in the input
    image to the return_window specification.
    -----------------------------------------*/
    return_window[SL] = window[0][SL];
    return_window[SS] = window[0][SS];
    return_window[NL] = window[0][NL];
    return_window[NS] = window[0][NS];
    
    /* Check for the presence of the addr file.  If it exists load up the
    model information.  If it doesn't exist, the assumption is that the image
    is a composite.  There is no need to check the ddr fields of a single-
    scene image since the projection information is contained in the addr.
    -----------------------------------------------------------------------*/
    c_lsmknm( input_file, "addr", addr_file );
    status = c_labstat( addr_file, 0 );
    if ( status == E_SUCC )
	{
	status = c_getaddr( input_file, input_model );
	if ( status == E_SUCC )
	    {
	    if ( input_ddr->nbands != 5 )
		{
		c_lasmsg(FALSE,"clavr-ddr",
		    "Single scene image must be 5 bands");
		return E_FAIL;
		}
	    *input_image_type = SINGLE_SCENE;
	    }
	else
	    {
	    c_lasmsg(FALSE,"clavr-info", "Could not open addr");
	    return E_FAIL;
	    }
	}
    else
        {
	c_lasmsg(FALSE,"clavr-info",
	    "No addr, assuming a Composite image");
	if ( input_ddr->nbands != 13 )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Composite image must be 13 bands");
	    return E_FAIL;
	    }
	
	/* Check for valid projection parameters
	----------------------------------------*/
	if ( input_ddr->valid[DDPCV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid projection code");
	    return E_FAIL;
	    }
	if ( input_ddr->valid[DDZCV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid zone code");
	    return E_FAIL;
	    }
	if ( input_ddr->valid[DDDCV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid datum code");
	    return E_FAIL;
	    }
	if ( input_ddr->valid[DDPPV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid projection coefficients");
	    return E_FAIL;
	    }
	if ( input_ddr->valid[DDPUV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid projection units");
	    return E_FAIL;
	    }
	if ( input_ddr->valid[DDPDV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid projection distance");
	    return E_FAIL;
	    }
	if ( input_ddr->valid[DDCCV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid corner coordinates");
	    return E_FAIL;
	    }
	if ( input_ddr->valid[DDINCV] != VALID )
	    {
	    c_lasmsg(FALSE,"clavr-ddr","Invalid line/sample increments");
	    return E_FAIL;
	    }

	/* If all the tests pass
	------------------------*/
	*input_image_type = COMPOSITE;
	}  /* END else */


    /* Get satellite number
    -----------------------*/
    var = p_find( vblock, "SATNUM" );
    if ( var->v_count != 1 )
        {
	c_lasmsg(FALSE,"clavr-find","Error finding SATNUM paramter");
	return E_FAIL;
	}
    *satellite_number = IVAL( *var, 0 );
    switch ( *satellite_number )
        {
	case 11 :
	case 12 :
	case 14 :
	case 15 :
	case 16 :
	case 17 :
        case 18 :
        case 19 :break; 
	default :
	    c_lasmsg(FALSE,"clavr-satnum","Invalid satellite number");
	    return E_FAIL;
	}  /* END switch */
    
    /* Get the scaling parameter
    ----------------------------*/
    var = p_find( vblock, "SCALING" );
    if ( var->v_count != 1 )
        {
	c_lasmsg(FALSE,"clavr-find","Error finding SCALING paramter");
	return E_FAIL;
	}
    if ( strcasecmp( SVAL( *var, 0 ), "NONE" ) == 0 )
        *scaled_flag = FALSE;
    else if ( strcasecmp( SVAL( *var, 0 ), "US" ) == 0 )
        {
	*scaled_flag = TRUE;
	c_getscaleopt( "US", input_ddr->dtype, &soption );
	}
    else if ( strcasecmp( SVAL( *var, 0 ), "GLOBAL" ) == 0 )
        {
	*scaled_flag = TRUE;
	c_getscaleopt( "GLOBAL", input_ddr->dtype, &soption );
	}
    else
        {
	c_lasmsg(FALSE,"clavr-scaling","Invalid scaling parameter");
	return E_FAIL;
	}
    
    /* Get the scaling parameters
    -----------------------------*/
    if ( *scaled_flag )
        {
	status = c_getscale( soption, "reflectance",
	    &scale_factors[SC_REFLECTANCE].minval,
	    &scale_factors[SC_REFLECTANCE].maxval,
	    &scale_factors[SC_REFLECTANCE].scale,
	    &scale_factors[SC_REFLECTANCE].unscale,
	    &scale_factors[SC_REFLECTANCE].offset);
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-scaling",
	        "Unable to get reflectance scale factors");
	    return E_FAIL;
	    }
	status = c_getscale( soption, "thermal",
	    &scale_factors[SC_THERMAL].minval,
	    &scale_factors[SC_THERMAL].maxval,
	    &scale_factors[SC_THERMAL].scale,
	    &scale_factors[SC_THERMAL].unscale,
	    &scale_factors[SC_THERMAL].offset);
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-scaling",
	        "Unable to get thermal scale factors");
	    return E_FAIL;
	    }
	
	if ( *input_image_type == COMPOSITE )
	    {
	    status = c_getscale( soption, "relazimuth",
		&scale_factors[SC_RELAZ].minval,
		&scale_factors[SC_RELAZ].maxval,
		&scale_factors[SC_RELAZ].scale,
		&scale_factors[SC_RELAZ].unscale,
		&scale_factors[SC_RELAZ].offset);
	    if ( status != E_SUCC )
		{
		c_lasmsg(FALSE,"clavr-scaling",
		    "Unable to get relazimuth scale factors");
		return E_FAIL;
		}
	    status = c_getscale( soption, "satzenith",
		&scale_factors[SC_SATZEN].minval,
		&scale_factors[SC_SATZEN].maxval,
		&scale_factors[SC_SATZEN].scale,
		&scale_factors[SC_SATZEN].unscale,
		&scale_factors[SC_SATZEN].offset);
	    if ( status != E_SUCC )
		{
		c_lasmsg(FALSE,"clavr-scaling",
		    "Unable to get satzenith scale factors");
		return E_FAIL;
		}
	    status = c_getscale( soption, "solzenith",
		&scale_factors[SC_SOLZEN].minval,
		&scale_factors[SC_SOLZEN].maxval,
		&scale_factors[SC_SOLZEN].scale,
		&scale_factors[SC_SOLZEN].unscale,
		&scale_factors[SC_SOLZEN].offset);
	    if ( status != E_SUCC )
		{
		c_lasmsg(FALSE,"clavr-scaling",
		    "Unable to get solzenith scale factors");
		return E_FAIL;
		}
	    }  /* END if input_image_type == COMPOSITE */
	}  /* END if scaled_flag */
    

    /* Get the acquisition date and calculate a julian date
    -------------------------------------------------------*/
    var = p_find( vblock, "ACQDATE" );
    if ( var->v_count != 3 )
        {
	c_lasmsg(FALSE,"clavr-find","Error finding ACQDATE parameter");
	return E_FAIL;
	}
    month = IVAL( *var, 0 );
    day   = IVAL( *var, 1 );
    year  = IVAL( *var, 2 );
    if ( year < 1900 || year > 2100 )
        {
	c_lasmsg(FALSE,"clavr-year","Year must be entered as a 4-digit value");
	return E_FAIL;
	}
    c_grejul( &year, &month, &day, &jday );
    if ( jday < 1 || jday > 366 )
        {
	c_lasmsg(FALSE,"clavr-julian","Error computing julian day");
	return E_FAIL;
	}
    else
        *julian_day = jday;
    

    /* Get the threshold name and create a default name
    ---------------------------------------------------*/
    var = p_find( vblock, "THRESHOLD" );
    if ( var->v_count == 1 )
	{
	strcpy( user_threshold, SVAL( *var, 0 ));
        *user_threshold_flag = TRUE;
	}
    else if ( var->v_count == 0 )
        *user_threshold_flag = FALSE;
    else
        {
	c_lasmsg(FALSE,"clavr-find","Error finding THRESHOLD parameter");
	return E_FAIL;
	}
    adapstables = getenv( "ADAPSTABLES" );
    if ( !adapstables )
        {
	c_lasmsg(FALSE,"clavr-tables","Unable to get ADAPSTABLES");
	return E_FAIL;
	}
    sprintf( default_threshold, "%s/CLAVR_threshold_%02d.dat", adapstables,
        (int)month );


    /* Get the barren mask file name
    --------------------------------*/
    status = c_getin( vblock, "BARREN", &element, tnames, hnames, &nmrimg,
              window, bands, &nbands, &totbnd, &imglimit, &same_window, qflag );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-getin",
	    "Unable to get barren image specification");
	return E_FAIL;
	}
    if ( nmrimg == 1 )
	{
	strcpy( barren_name, hnames[0] );
	if ( *input_image_type == SINGLE_SCENE )
	    {
	    c_lasmsg(FALSE,"clavr-warn",
	        "Barren mask cannot be used with single scene input");
	    c_lasmsg(FALSE,"clavr-warn","Barren mask will be ignored");
	    *barren_mask_flag = FALSE;
	    }
	else
	    {
	    /* Compare the window specification with
	    that of the input -- they must be the same
	    ------------------------------------------*/
	    if ( return_window[SL] != window[0][SL] ||
	         return_window[SS] != window[0][SS] ||
	         return_window[NL] != window[0][NL] ||
	         return_window[NS] != window[0][NS] )
		{
		c_lasmsg(FALSE,"clavr-window",
		    "Input and barren mask image specification not the same");
		return E_FAIL;
		}

	    /* Get the barren mask ddr and compare it
	    with the input file ddr projection and size
	    -------------------------------------------*/
	    status = c_getddr( barren_name, barren_ddr );
	    if ( status != E_SUCC )
	        {
		c_lasmsg(FALSE,"clavr-ddr","Unable to get barren mask ddr");
		return E_FAIL;
		}
	    
	    for ( i = 0; i < 7; i++ )
	        {
		if ( barren_ddr->valid[i] != VALID )
		    {
		    c_lasmsg(FALSE,"clavr-ddr",
		        "Invalid flag found in barren ddr");
		    return E_FAIL;
		    }
		}  /* END for */

	    if ( barren_ddr->nl != input_ddr->nl ||
	         barren_ddr->ns != input_ddr->ns ||
	         barren_ddr->proj_code != input_ddr->proj_code ||
	         barren_ddr->zone_code != input_ddr->zone_code ||
	         barren_ddr->datum_code != input_ddr->datum_code ||
	         barren_ddr->pdist_y != input_ddr->pdist_y ||
	         barren_ddr->pdist_x != input_ddr->pdist_x )
		{
		c_lasmsg(FALSE,"clavr-ddr",
		    "Barren mask ddr not equal to input ddr");
		return E_FAIL;
		}
	    
	    if ( strcasecmp( barren_ddr->proj_units, input_ddr->proj_units )
		!= 0 )
		{
		c_lasmsg(FALSE,"clavr-ddr",
		    "Barren mask projection units not equal to input");
		return E_FAIL;
		}
	    
	    for ( i = 0; i < 2; i++ )
	        {
		if ( barren_ddr->upleft[i] != input_ddr->upleft[i] ||
		     barren_ddr->loleft[i] != input_ddr->loleft[i] ||
		     barren_ddr->upright[i] != input_ddr->upright[i] ||
		     barren_ddr->loright[i] != input_ddr->loright[i] )
		    {
		    c_lasmsg(FALSE,"clavr-ddr",
		        "Barren mask corner coordinates not equal to input");
		    return E_FAIL;
		    }
		}
	    
	    for ( i = 0; i < 15; i++ )
	        {
		if ( barren_ddr->proj_coef[i] != input_ddr->proj_coef[i] )
		    {
		    c_lasmsg(FALSE,"clavr-ddr",
		     "Barren mask projection coefficients not equal to input");
		    return E_FAIL;
		    }
		}
	    *barren_mask_flag = TRUE;
	    }  /* END else */
	}  /* END if nmrimg == 1 */
    else
        *barren_mask_flag = FALSE;

	    
    /* Get the output file
    ----------------------*/
    status = c_get_multiple_outs( vblock, 1, "OUT", output_name, &num_out );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-getout","Error finding OUT file");
	return E_FAIL;
	}
    if ( num_out == 1 )
        *output_file_flag = TRUE;
    else
	{
        *output_file_flag = FALSE;

	/* If no output image is specified and the input image is windowed
	it is a fatal error because an attempt will be made to add the cloud
	mask to the input image as another band.  Since the input and output
	images will be different sizes the copy will error out.  There is no
	point in continuing processing if the error is detected here.
	-------------------------------------------------------------------*/
	if ( return_window[SL] != 1 ||
	     return_window[SS] != 1 ||
	     return_window[NL] != input_ddr->nl ||
	     return_window[NS] != input_ddr->ns )
	    {
	    c_lasmsg(FALSE,"clavr-window",
	        "Windowed output cannot be added to input image");
	    return E_FAIL;
	    }
        }  /* END else */


    /* Get the processing message interval
    --------------------------------------*/
    var = p_find( vblock, "$MSGTIM" );
    if ( var->v_count == 1 )
        *msgtim = IVAL( *var, 0 );
    else
        *msgtim = 10;

    /* get the override value. REMOVE this code when reprocessing
    of NOAA 16 composites is complete.
    -------------------------------------------------------------*/
    var = p_find( vblock, "C3OVERIDE" );
    if ( var->v_count == 1 )
        {
	char c[3];

	strcpy( c, SVAL( *var, 0 ));
	if ( c[1] == 'a' || c[1] == 'A' )
	    channel3override = 1;
	else if ( c[1] == 'b'|| c[1] == 'B'  )
	    channel3override = 2;
	else
	    channel3override = 0;
	}
    else
        channel3override = 0;

    return E_SUCC;
    }  /* END getpar */
