/*****************************************************************************
FUNCTION:	GETTHRESH

PURPOSE:	To fill the threshold structure with values.  The default
		values from ADAPSTABLES/CLAVR_threshold_MM.dat, where MM is
		the 2-digit month of acquisition, are used to fill the
		structure first.  If the user supplied a threshold file name,
		that file is used to update any threshold values it contains.

INPUT:		user-supplied threshold file name
		user-supplied threshold flag
		default threshold file name

OUTPUT:		threshold structure

RETURNS:	E_SUCC on success
		E_FAIL on failure

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#define USETHRESHOLDS
#include "clavr.h"


int getthresh
    (
    char *user_threshold,	/* IN - user-supplied file name		*/
    int user_threshold_flag,	/* IN - flag: user-supplied file	*/
    				/*      values: TRUE or FALSE		*/
    char *default_threshold,	/* IN - default threshold file name	*/
    struct THRESHOLD *thresholds/* OUT - threshold structure		*/
    )
    {
    FILE *inf;			/* file descriptor for input file	*/

    char filename[CMLEN];	/* current file name in use		*/
    char inbuf[CMLEN];		/* input buffer				*/
    char message[CMLEN];	/* message string			*/
    char test_name[20];		/* test name extracted from input buffer*/

    float threshold_value;	/* value extracted from input buffer	*/

    int flag_value;		/* value for flag structure member	*/
    				/* 1 indicates threshold value is from	*/
				/* default file 2 from user-supplied	*/
				/* file and 0 indicates value is not set*/
    int found;			/* flag: found test name in valid list	*/
    int missing_value;		/* flag: one or more values missing	*/
    int num_scanned;		/* number of values scanned from inbuf	*/
    int threshold_index;	/* loop variable			*/


    /* Clear all the structure values
    ---------------------------------*/
    for (threshold_index = 0;threshold_index <= NUMTHRESHOLDS;threshold_index++)
        {
	thresholds->flag[threshold_index] = 0;
	thresholds->value[threshold_index] = 0.0;
	}
    

    /* Open the default threshold file and load up the structure.
    If the user supplied an alternate file open it and overwrite
    structure values that it contains.  This is done because
    the user is allowed to specify one or more values and all
    values must be filled for the clavr algorithm.
    -------------------------------------------------------------*/
    flag_value = 1;
    do
        {
        if ( flag_value == 1 )
	    strcpy( filename, default_threshold );
	else
	    strcpy( filename, user_threshold );
	inf = fopen( filename, "r" );
	if ( !inf )
	    {
	    sprintf(message,"Unable to open: %s", filename);
	    c_lasmsg(FALSE,"clavr-open",message);
	    return E_FAIL;
	    }
	

	/* Read the input file until data is exhausted
	----------------------------------------------*/
	while ( fgets( inbuf, 80, inf ) != NULL )
	    {
	    /* Grab the two items that are supposed to
	    be in the line.  If both are not retrieved
	    it is a fatal error.
	    ------------------------------------------*/
	    num_scanned = sscanf( inbuf, "%s %f", test_name, &threshold_value);
	    if ( num_scanned != 2 )
	        {
		c_lasmsg(FALSE,"clavr-scan","Error scanning input buffer");
		fclose( inf );
		return E_FAIL;
		}
	    

	    /* Look through the valid list of test names
	    for the current test name.  If it is found
	    break out of the loop.  Going through the loop
	    without finding a matching name is a fatal error.
	    -------------------------------------------------*/
	    found = FALSE;
	    for ( threshold_index = 0; threshold_index < NUMTHRESHOLDS;
		threshold_index++ )
		{
		if ( strcasecmp( test_name, threshold_names[threshold_index])
		    == 0 )
		    {
		    /* If a duplicate entry is found, it will
		    be used, but issue a warning message that
		    the duplicate entry was found.
		    -----------------------------------------*/
		    if ( thresholds->flag[threshold_index] == flag_value )
		        {
			sprintf( message, "Duplicate entry %s in %s",
			    threshold_names[threshold_index],
			    filename );
			c_lasmsg(FALSE,"clavr-warn",message);
			sprintf( message, "Value %f will be used",
			    thresholds->value[threshold_index] );
			c_lasmsg(FALSE,"clavr-warn",message);
			}
		    
		    thresholds->flag[threshold_index] = flag_value;
		    thresholds->value[threshold_index] = threshold_value;
		    found = TRUE;
		    break;
		    }  /* END if */
		}  /* END for */
	    

	    /* An invalid test name is fatal
	    --------------------------------*/
	    if ( ! found )
	        {
		sprintf(message,"Invalid test %s found in %s",
		    test_name, filename );
		c_lasmsg(FALSE,"clavr-invalid",message);
		fclose( inf );
		return E_FAIL;
		}  /* END if */
	    }  /* END while */
	    
	fclose( inf );

	if ( user_threshold_flag )
	    flag_value++;
	else
	    break;
	}
    while ( flag_value != 3 );
    

    /* Make sure that every structure member
    has an entry.  If not, it is a fatal error.
    -------------------------------------------*/
    missing_value = FALSE;
    for (threshold_index = 0;threshold_index < NUMTHRESHOLDS;threshold_index++)
        {
	if ( thresholds->flag[threshold_index] == 0 )
	    {
	    sprintf( message, "Missing %s threshold",
		threshold_names[threshold_index] );
	    c_lasmsg(FALSE,"clavr-noval",message);
	    missing_value = TRUE;
	    }
	}  /* END for */
    if ( missing_value )
        return E_FAIL;
    
    return E_SUCC;
    }  /* END getthresh */
