/****************************************************************************
PROCESS:	klm_chan3temp2alb

PURPOSE:	Convert channel 3B temperature to an equivalent albedo.

INPUT:		channel 3B temperature (in Kelvin)
		channel 4 temperature (in Kelvin)
		channel 5 temperature (in Kelvin)
		sun/earth distance (in AU) squared
		solar zenith angle (in radians)
		satellite conversion constants

OUTPUT:		none

RETURNS:	channel 3B albedo (in percent)

ALGORITHM:	convert channel 3B temperature to radiance (see KLM User's
		    Guide, Sec 7 and app. D)
		compute channel 3B emmision radiance (see CLAVR-x code: file
		    avhrr_mod.f90 function ch3b_alb)

*****************************************************************************/
#include <math.h>
#include "clavr.h"

#define NINETY (PI / 2.0)

float klm_chan3temp2alb
    (
    float chan3btemp,		/* IN - channel 3B temperature		*/
    float chan4temp,		/* IN - channel 4 temperature		*/
    float chan5temp,		/* IN - channel 5 temperature		*/
    float sedsq,		/* IN - sun/earth distance squared	*/
    float solar_zenith, 	/* IN - solar zenith angle (in radians)	*/
    struct SATELLITE_CONSTANTS constants/* IN - satellite constants	*/
    )
    {
    double alb3b;		/* channel 3B albedo			*/
    double ems3b;		/* channel 3B emissivity		*/
    double rad3b;		/* channel 3B radiance			*/
    double rad3b_ems;		/* channel 3B emmision radiance		*/
    double t0;			/* emission brightness temperature	*/

    /* Convert temperature to radiance
    ----------------------------------*/
    rad3b = constants.C1nuc_cubed /
        (exp( constants.A2 * constants.C2nuc / ( chan3btemp - constants.A1 ))
	 - 1.0 );

    /* Compute the 3B emission brightness temperature
    -------------------------------------------------*/
    if ( chan4temp - chan5temp > 1.0 )
	{
        t0 = constants.A0 + constants.B0 * chan4temp + constants.C0 *
	     ( chan4temp - chan5temp );
	}
    else
        t0 = chan4temp;

    /* Compute channel 3B emission radiance and emissivity
    ------------------------------------------------------*/
    if ( chan4temp > 0.0 )
        {
	rad3b_ems = constants.C1nuc_cubed / ( exp( constants.C2nuc /
	    ((t0 - constants.A1) / constants.A2 )) - 1.0 );
	ems3b = rad3b / rad3b_ems;
	}
    else
        rad3b_ems = ems3b = 0.0;
    
/* Unsure of which one of these to use at this time.  Defer decision
until testing is complete and some more experimentation is done. ?????
**********************************************************************/
    if ( solar_zenith < NINETY && rad3b_ems > 0.0 && rad3b > 0.0 )
	{
#ifdef CLAVRX
	alb3b = 100.0 * PI * ( rad3b - rad3b_ems ) / (( constants.solar_3b_nu *
	    cos( (double)solar_zenith )) / sedsq ) - PI * rad3b_ems;
#else
	alb3b = 100.0 * PI * ( rad3b - rad3b_ems ) / (( constants.solar_3b_nu *
	    cos( (double)solar_zenith )) / sedsq );
#endif
	}

    if ( solar_zenith >= PI / 2.0 && ems3b > 0.0 )
        alb3b = 100.0 * ( 1.0 - ems3b );
    
    return (float)alb3b;
    }  /* END klm_chan3temp2alb */
