/*****************************************************************************
FUNCTION:	CHAN3TEMP2ALB

PURPOSE:	Compute a channel 3 albedo using channel 3, 4 and 5
		temperatures.  This is taken directly from the function tms3.f
		in the CLAVR1 source.

INPUT:		channel 3 temperature (in Kelvin)
		channel 4 temperature (in Kelvin)
		channel 5 temperature (in Kelvin)
		sun/earth distance (in AU) squared
		solar zenith angle (in radians)
		satellite conversion constants

OUTPUT:		none

RETURNS:	channel 3 albedo

*****************************************************************************/
#include <math.h>
#include "clavr.h"

/* 89.5 degrees in radians
--------------------------*/
#define EIGHTYNINE_FIVE 1.562069680534925 

float chan3temp2alb
    (
    float channel3temp,		/* IN - channel three temperature	*/
    float channel4temp,		/* IN - channel four temperature	*/
    float channel5temp,		/* IN - channel five temperature	*/
    float sedsq,		/* IN - sun/earth distance squared	*/
    float solar_zenith,		/* IN - solar zenith angle (in radians)	*/
    struct SATELLITE_CONSTANTS constants/* IN - satellite constants	*/
    )
    {
    /* Please no complaints about variable names and lack of comments.  This
    code was essentially pasted directly from the NOAA CLAVR source code, case
    changed and statements changed to C-syntax.  There was no point in trying
    to decipher its meaning so this is as good as it's going to get.
    -------------------------------------------------------------------------*/
    double delt3;
    double t0, c2t3, c2t0, expt0, expt3;
    double exdiff, ext31, ext01, ext3t0, delr3;
    double tms;
    double albedo;

    if ( solar_zenith > EIGHTYNINE_FIVE )
        return 0.0;
    else if (channel3temp == 0.0 || channel4temp == 0.0 || channel5temp == 0.0)
        return 0.0;
    else if ( fabs( channel4temp - channel5temp ) > 20.0 )
        return 0.0;
    
    tms = constants.A * (double)channel3temp + constants.B *
          (double)channel4temp + constants.C *
	  (double)channel5temp + constants.D;
    delt3 = tms / constants.A;
    t0 = channel3temp - delt3;
    c2t3 = constants.C2nuc / (double)channel3temp;
    c2t0 = constants.C2nuc / t0;
    expt0 = exp( c2t0 );
    expt3 = exp( c2t3 );
    exdiff = expt0 - expt3;
    ext31 = expt3 - 1.0;
    ext01 = expt0 - 1.0;
    ext3t0 = ext31 * ext01;
    delr3 = constants.C1nuc_cubed * ( exdiff / ext3t0 );
    albedo =
        (( delr3 * sedsq ) / ( cos( solar_zenith ) * constants.S3PI )) * 100.0;
    if ( albedo < 0.0 )
	albedo = 0.0;
    if ( albedo > 100.0 )
	albedo = 100.0;

    return (float)albedo;
    }  /* END chan3temp2alb */
