/****************************************************************************
FUNCTION:	GETCONSTANTS

PURPOSE:	Load a constant structure with satellite specific constants
		and precalculate a few constants based on these values.  There
		are no appropriate ADAPS routines to get these constant values
		from ADAPSTABLES and most are not stored in the units needed
		by the two routines that use them.  In addition there is no
		documentation as to how certain values were derived for
		example: ew_3b and solar_3b.  That is why I chose to implement
		in this manner.

INPUT:		satellite number (11, 12, 14, 15, 16, 17)

OUTPUT:		constant structure

RETURNS:	E_SUCC on success
		E_FAIL on failure

*****************************************************************************/
#include <string.h>
#include "las.h"
#include "clavr.h"


/***********************************************************************
These values are pulled directly from the constants files in CLAVR1/data

Noaa-11
  0.962422                   //  [Coefficients for defining
 -2.127582                   //   Ch. 3  Emission Temperatures from
  1.16516                    //   Ch. 4 & Ch. 5 Temperatures]
 -0.744                      //
 5.115                       // Ch.3 Solar SPECTRAL IRRAD. divided by PI
 2670.95                     // Central Wave No. for Ch.3 Alb. over Ocean

Noaa-12
  1.000000                   //  [Coefficients for defining
 -2.454250                   //   Ch. 3  Emission Temperatures from
  1.47937                    //   Ch. 4 & Ch. 5 Temperatures]
 -5.61764                    // 
 5.011                       // Ch.3 Solar SPECTRAL IRRAD. divided by PI
 2639.61                     // Central Wave No. for Ch.3 Alb. over Ocean 

Noaa-14
  1.000000                   //  [Coefficients for defining
 -2.915924                   //   Ch. 3  Emission Temperatures from
  1.92754                    //   Ch. 4 & Ch. 5 Temperatures]
 -1.21284                    // 
 5.031                       // Ch.3 Solar SPECTRAL IRRAD. divided by PI
 2645.90                     // Central Wave No. for Ch.3 Alb. over Ocean 

These constants were pulled directly from the CLAVRX/data constant files
n15         !sat_name
3.781       !solar_3b
236.53016   !ew_3b
2695.9743   !nu_3b
2.53662     !A0
0.99252     !B0
1.42640     !C0

n16         !sat_name
3.8299466   !solar_3b
239.12594   !ew_3b
2681.2540   !nu_3b
3.98914     !A0
0.98110     !B0
1.99224     !C0

n17         !sat_name
4.2443861   !solar_3b
269.79606   !ew_3b
2669.1414   !nu_3b
6.82947     !A0
0.97232     !B0
1.66366     !C0
*************************************************************************/


int getconstants
    (
    int satellite_number,	/* IN - satellite number		*/
    struct SATELLITE_CONSTANTS *constants /* OUT - constant structure	*/
    )
    {
    memset( constants, 0, sizeof(struct SATELLITE_CONSTANTS));
    switch ( satellite_number )
        {
        case 11 :
	    constants->A = 0.962422;
	    constants->B = -2.127582;
	    constants->C = 1.16516;
	    constants->D = -0.744;
	    constants->S3PI = 5.115;
	    constants->CNUO = 2670.95;
	    break;
        case 12 :
	    constants->A = 1.000000;
	    constants->B = -2.454250;
	    constants->C = 1.47937;
	    constants->D = -5.61764;
	    constants->S3PI = 5.011;
	    constants->CNUO = 2639.61;
	    break;
        case 14 :
	    constants->A = 1.000000;
	    constants->B = -2.915924;
	    constants->C = 1.92754;
	    constants->D = -1.21284;
	    constants->S3PI = 5.031;
	    constants->CNUO = 2645.90;
	    break;
	case 15 :
	    constants->A = 1.621256;
	    constants->B = 0.998015;
	    constants->solar_3b_nu = 1000.0 * 3.781 / 236.53016;
	    constants->A0 = 2.53662;
	    constants->B0 = 0.99252;
	    constants->C0 = 1.42640;
	    constants->CNUO = 2695.9743;
	    constants->A1 = -1.624481;
	    constants->A2 = 1.001989;
	    break;
        case 16 :
	    constants->A = 1.592459;
	    constants->B = 0.998147;
            constants->solar_3b_nu = 1000.0 * 3.8299466 / 239.12594;
            constants->A0 = 3.98914;
            constants->B0 = 0.98110;
            constants->C0 = 1.99224;
	    constants->CNUO = 2700.1148;
	    constants->A1 = -1.6774586;
	    constants->A2 = 1.0017316;
            break;
        case 17 :
	    constants->A = 1.702380;
	    constants->B = 0.997378;
            constants->solar_3b_nu = 1000.0 * 4.2443861 / 269.79606;
            constants->A0 = 6.82947;
            constants->B0 = 0.97232;
            constants->C0 = 1.66366;
	    constants->CNUO = 2669.3554;
	    constants->A1 = -1.7002941;
	    constants->A2 = 1.0026724;
	    break;
	default :
	    c_lasmsg(FALSE,"clavr-satnum","Invalid satellite number");
	    return E_FAIL;
	}
    
    /* Calculate these constants so that they don't have to
    be recalculated for every pixel in the image.
    -------------------------------------------------------*/
    constants->C1nuc_cubed = C1 * constants->CNUO * constants->CNUO *
	constants->CNUO;
    constants->C2nuc = C2 * constants->CNUO;

    return E_SUCC;
    }  /* END getconstants */
