/*****************************************************************************
FUNCTION:	OPENIMAGES

PURPOSE:	Open input and output images, allocate memory for I/O and 
		group images for LAS I/O routines.  In the case the user
		didn't specify an output file, a temporary file is created
		for output.

INPUT:		TAE parameter block
		input file name
		barren image file name
		output file name
		barren mask flag
		output file flag
		input image type
		input image data type
		image window

OUTPUT:		array of file descriptors
		group file descriptor
		minor frame file descriptor
		temporary file name
		input buffer offsets
		output buffer offset
		I/O buffer

RETURNS:	E_SUCC on success
		E_FAIL on failure

*****************************************************************************/
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "clavr.h"


int openimages
    (
    struct PARBLK *vblock,	/* IN - TAE parameter block		*/
    char *input_file,		/* IN - input file name			*/
    char *barren_name,		/* IN - barren mask file name		*/
    char *output_name,		/* IN - output file name		*/
    int barren_mask_flag,	/* IN - flag: barren mask file name is	*/
    				/*      specified, TRUE or FALSE	*/
    int output_file_flag,	/* IN - flag: output file name is	*/
    				/*      specified, TRUE or FALSE	*/
    int input_image_type,	/* IN - flag: type of input image either*/
    				/*      SINGLE_SCENE or COMPOSITE	*/
    long *image_type,		/* IN - input data type 		*/
    long *window,		/* IN - image window			*/
    struct FDESC **fdesc,	/* OUT - array of file descriptors	*/
    struct GDESC **group_fdesc,	/* OUT - group file descriptor		*/
    long *minorframe_fdesc,	/* OUT - minor frame file descriptor	*/
    char *temporary_file,	/* OUT - file name for temporary output	*/
    long *input_offsets,	/* OUT - offsets into thebuf for input	*/
    long *output_offsets,	/* OUT - offsets into thebuf for output	*/
    unsigned char **thebuf	/* OUT - image I/O buffer		*/
    )
    {
    char hosmnr[CMLEN];		/* minor fram file name			*/
    char tempname[CMLEN];	/* used for building temp output name	*/
    char directory[CMLEN];	/* directory root file name and 	*/
    char rootname[CMLEN];	/* extension for building a temporary	*/
    char extension[CMLEN];	/* output file name			*/

    int num_bands;		/* number of input bands		*/
    int nbands;			/* number of input bands for upddr call	*/
    int status;			/* function return status		*/

    long access;		/* image I/O access type		*/
    long bnds[MAXBND+1];	/* bands for open call			*/
    long bufsiz;		/* size of buffer returned from ealloc	*/
    long idtype = EREAL;	/* type conversion for input image	*/
    long image;			/* index for images			*/
    long inbands[MAXBND+1];	/* bands for open and upddr calls	*/
    long odtype;		/* output data type 			*/
    long one = 1;		/* contant of one			*/
    long opt[3] = {0};		/* data conversion option		*/
    long zero = 0;		/* contant of zero			*/

    pid_t pid;			/* used for building temp output name	*/


    /* Open the input image.  Set up the bands array which tell eopen what
    bands to use from input image.  All input images will utilize the first
    five bands (the channel information).
    -----------------------------------------------------------------------*/
    access = IREAD;
    inbands[0] = 1;
    inbands[1] = 2;
    inbands[2] = 3;
    inbands[3] = 4;
    inbands[4] = 5;
    inbands[5] = 0;
    num_bands = 5;
    /* If it's a composite the viewing angles are in bands 7-9
    and QC band is band 12.
    ----------------------------------------------------------*/
    if ( input_image_type == COMPOSITE )
        {
	inbands[5] = 7;
	inbands[6] = 8;
	inbands[7] = 9;
	inbands[8] = 12;
	inbands[9] = 0;
	num_bands = 9;
	}
    image = 0;		/* set the image index and make sure the desc	*/
    fdesc[image] = NULL;/* is NULL so that on error the closeimage func	*/
    			/* will not try to close an unopened image	*/
    status = c_eopens( &fdesc[image], input_file, inbands, &window[SL],
                       &window[SS], &window[NL], &window[NS], 
		       &input_offsets[0], &access, &idtype, &zero, opt );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-open","Unable to open input file");
	return E_FAIL;
	}

    /* On successful open, NULL terminate the decsriptor
    array for any following error condition.
    ---------------------------------------------------*/
    fdesc[image+1] = NULL;

    
    /* Open the barren image if specified
    -------------------------------------*/
    if ( barren_mask_flag )
	{
	bnds[0] = 1;	/* Set up for a second, single-band, image open	*/
	bnds[1] = 0;
	image++;
	status = c_eopens( &fdesc[image], barren_name, bnds, &window[SL],
	                   &window[SS], &window[NL], &window[NS],
			   &input_offsets[num_bands],&access, &idtype,
			   &zero, opt );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-open","Unable to open barren mask");
	    closeimages(fdesc, minorframe_fdesc, input_file, input_image_type);
	    return E_FAIL;
	    }
	/* On successful open, NULL terminate the decsriptor
	array for any following error condition.
	---------------------------------------------------*/
	fdesc[image+1] = NULL;
	}
    

    /* Open the output file.  If a separate image was specified,
    output_file_flag is TRUE, open it with the specified output
    name and as BYTE.  If the output wasn't specified, the output
    will be added as the last band to the input file.  Because
    of this open a file with a temporary name and with the same
    data type as the input image.  Note that we want to do all
    processing using floats so the data type for the call is
    EREAL with the conversion set to the desired output type.
    ----------------------------------------------------------*/
    access = IWRITE;
    odtype = EREAL;
    bnds[0] = 1;
    bnds[1] = 0;
    image++;
    if ( output_file_flag )
	{
        strcpy( tempname, output_name );
	opt[0] = EBYTE;
	}
    else
        {
	c_splitname( input_file, directory, rootname, extension );
	opt[0] = *image_type;
	pid = getpid();
	sprintf( tempname, "%s%s_mask%d.img", directory, rootname, pid );
	strcpy ( temporary_file, tempname );
	}
    opt[1] = opt[2] = 0;
    status = c_eopens( &fdesc[image], tempname, bnds, &one, &one, &window[NL],
		       &window[NS], &output_offsets[0], &access, &odtype,
		       &zero, opt );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-open","Unable to open output file");
	closeimages(fdesc, minorframe_fdesc, input_file, input_image_type);
	return E_FAIL;
	}
    fdesc[image+1] = NULL;

    /* Create a history file
    ------------------------*/
    status = las_hstout( vblock, tempname, "", 0, "" );
    if ( status != E_SUCC )
        c_lasmsg(FALSE,"clavr-his","Unable to create history file");

    /* Update the output ddr with information from the input ddr
    ------------------------------------------------------------*/
    nbands = num_bands;
    status = las_upddr( vblock, input_file, tempname, 1, window, inbands,
                      &nbands, 1, 1, 1, COMB );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-upddr","Error updating output DDR");
	closeimages(fdesc, minorframe_fdesc, input_file, input_image_type);
	return E_FAIL;
	}

    /* Allocate the I/O buffer
    --------------------------*/
    *thebuf = (unsigned char *)c_ealloc( fdesc, &bufsiz );
    if ( thebuf == NULL )
        {
	c_lasmsg(FALSE,"clavr-alloc","Unable to allocate I/O buffer");
	closeimages(fdesc, minorframe_fdesc, input_file, input_image_type);
	return E_FAIL;
	}
    
    /* Group the images
    -------------------*/
    status = c_egroup( fdesc, group_fdesc, *thebuf, &bufsiz );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-group","Unable to group images");
	closeimages(fdesc, minorframe_fdesc, input_file, input_image_type);
	return E_FAIL;
	}
    
    /* Open the minor frame if single scene
    ---------------------------------------*/
    if ( input_image_type == SINGLE_SCENE )
        {
	c_lsmknm( input_file, "mnr", hosmnr );
	access = 0;
	status = c_lsopen( minorframe_fdesc, hosmnr, &access );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-open","Unable to open minor frame");
	    closeimages(fdesc, minorframe_fdesc, input_file, input_image_type);
	    return E_FAIL;
	    }
	}

    return E_SUCC;
    }  /* END openimages */
