/*****************************************************************************
FUNCTION:	PROCESS

PURPOSE:	Read through the input files, calculating or converting data
		for the clavr input.  Call clavr and write output cloud codes
		to output or temporary image.

INPUT:		grouped file descriptors
		image I/O buffer
		input buffer offsets
		output buffer offset
		minor frame file descriptor
		barren mask flag
		input image type
		scaled flag
		scale factors
		thresholds
		input image ddr
		input image model
		window
		julian day of acquisition
		satellite number
		processing message interval

OUTPUT:		writes to output or temporary file

RETURNS:	E_SUCC on success
		E_FAIL on failure

*****************************************************************************/
#include <math.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "vll.h"
#include "dltf.h"
#include "geompak.h"
#include "archio.h"
#include "model.h"
#include "constants.h"
#include "adapssup.h"
#include "clavr.h"

/* REMOVE this when reprocessing of NOAA 16 composites is complete */
extern int channel3override;

/* ??? This is here to supress a warning about no prototype
for sunearthdis() function.  Remove once the ADAPSLIB is
updated to include an external c_sunearthdis prototype.
**************************************************************************/
void sunearthdis
    (
    long jday,          /* Julian day                           */
    float *aus          /* Number of astronomical units         */
    );

/* processing message counter
-----------------------------*/
extern long curent;

int process
    (
    struct GDESC **group_fdesc,	/* IN - grouped file descriptors	*/
    unsigned char *thebuf,	/* IN - image I/O buffer		*/
    long *input_buffer_offsets,	/* IN - offsets into thebuf for input	*/
    long *output_buffer_offset,	/* IN - offsets into thebuf for output	*/
    long *minorframe_fdesc,	/* IN - minor frame file descriptor     */
    int barren_mask_flag,	/* IN - flag: barren mask specified	*/
    				/*      TRUE or FALSE			*/
    int input_image_type,	/* IN - flag: input image type		*/
    				/*      SINGLE_SCENE or COMPOSITE	*/
    int scaled_flag,		/* IN - flag: input image scaled	*/
    				/*      TRUE or FALSE			*/
    struct SCOFF *scale_factors,/* IN - scaling factors			*/
    struct THRESHOLD thresholds,/* IN - threshold value structure	*/
    struct DDR *input_ddr,	/* IN - input image ddr			*/
    struct MODEL *input_model,	/* IN - input image AVHRR model		*/
    long *window,		/* IN - input window specification	*/
    int julian_day,		/* IN - julian date of acquisition	*/
    int satellite_number,	/* IN - NOAA satellite number		*/
    long msgtim			/* IN - time between processing messages*/
    )
    {
    float channel1;		/* channel 1 data extracted from thebuf */
    float channel1alb;		/* channel 1 converted to albedo	*/
    float channel2;             /* channel 2 data extracted from thebuf */
    float channel2alb;		/* channel 2 converted to albedo	*/
    float channel3;             /* channel 3 data extracted from thebuf */
    float channel3alb;		/* channel 3 converted to albedo	*/
    float channel4;             /* channel 4 data extracted from thebuf */
    float channel5;             /* channel 5 data extracted from thebuf */
    float gamma;		/* specular gamma angle in degrees	*/
    float latitude, longitude;	/* lat/long of pixel in degrees		*/
    float *ptr[NUMBANDS] = {NULL}; /* pointer to image buffer		*/
    float relative_azimuth;	/* Viewing angle in radians		*/
    float satellite_zenith;	/* Viewing angle in radians		*/
    float sedsq;		/* sun/earth distance in AU's squared   */
    float solar_zenith;		/* Viewing angle in radians		*/

    int band3is_thermal = TRUE;	/* flag: band 3 is thermal, TRUE or FALSE*/
    int band3isNOTdetermined = FALSE;/* flag: TRUE/FALSE, band 3 not known*/
    int band;			/* loop variable			*/
    int barren; 		/* barren flag value extracted from thebuf*/
    int line;			/* loop variable			*/
    int sample; 		/* loop variable			*/
    int barren_index;		/* barren flag index for the ptr array	*/
    int c1,c2,c3,c4,c5;		/* channel indices for the ptr array	*/
    int input_bands;		/* number of input bands		*/
    int klm_flag;		/* satellite is a KLM series		*/
    int latlon_error = FALSE;	/* flag: error calculating lat and lon	*/
    				/* value is TRUE or FALSE		*/
    int out;			/* output value index for the ptr array	*/
    int qc;			/* QC band index for ptr array		*/
    int qcvalue;		/* value read from QC band		*/
    int rel,sat,sol;		/* viewing anngle indices for ptr array	*/
    int status;			/* function return status		*/

    long flag = ALLCAT;		/* processing message format flag	*/
    long num_bands;		/* number of bands for processing msg	*/

    struct SATELLITE_CONSTANTS constants;/* satellite constant structure*/
    
    /* variables for determining lat/lon from projection parameters         */
    double  all_zero[15] = {0.0};   /* zero projection parameters           */
    double  lat,lon;		    /* latitude and longitude from c_rtcoef */
    double  proj_coord[8];          /* corner coords in projection space    */
    double  proj_y, proj_x;         /* coordinates from c_trans call        */
    double  rot_coeffs[6];          /* rotation coefficients                */
    long    degrees = 4;            /* units code (degrees) for geo proj    */
    long    geo = 0;                /* geographic projection flag           */
    long    image_coord[8];         /* corner coords in image space         */
    long    msgflag = TRUE;         /* flag for printing error messages     */
    long    proj_units;             /* projection units value               */
    long    prtprm[2];              /* print flags for c_transinit()        */
    long    utm = 62;               /* UTM projection code                  */
    /* Local copies of ddr for c_transinit and c_trans calls                */
    long datum_code = input_ddr->datum_code;	/* datum code		    */
    long proj_code = input_ddr->proj_code;	/* projection code	    */
    long zone_code = input_ddr->zone_code;	/* zone			    */


    /* variables for reading data from the minor frame label services file  */
    char    char_buf[1];            /* buffer for label services calls      */
    char    dtype[DTYPL];           /* Data type for label services calls   */
    char    linekey[KEYL];          /* line number key for minor frame file */
    long    minor_nbytes = MNR_WORDS*sizeof(short); /* num bytes in min frm */
    long    one = 1;                /* Constant                             */
    short   min_frm[MNR_WORDS];     /* minor frame buffer                   */

    /* variables for getting the lat, lon and viewing angles from model */
    double elev = 0.0;		/* elevation for ls2ll call, always use 0*/
    double view[3];		/* viewing angles returned in here	*/
    float l;			/* line value for ls2ll call		*/
    float s;			/* sample value for ls2ll call		*/



/* Initial setup section
*****************************************************************************/
    switch ( satellite_number )
        {
	case 15 :
	case 16 :
	case 17 : 
        case 18 :
        case 19 : klm_flag = TRUE; break;
	default : klm_flag = FALSE; break;
	}
    
    /* There will always be at least five bands and the first five will always
    be channels 1 through 5.  These indices, along with sat, sol, rel, bare
    and out are used with the ptr array of pointers into thebuf to extract the
    appropriate data.
    -------------------------------------------------------------------------*/
    input_bands = 0;
    c1 = input_bands++;
    c2 = input_bands++;
    c3 = input_bands++;
    c4 = input_bands++;
    c5 = input_bands++;

    /* For single scene input images, reinitialize the 
    AVHRR model.  This is to insure the most recent
    ephemeris and time correction values are used.
    -------------------------------------------------*/
    if ( input_image_type == SINGLE_SCENE )
        {
	/* Initmod needs a bunch of long values to be passed.  These
	values are already contained in the model structure from getpar
	so just need to extract and assign them to long variables.
	---------------------------------------------------------------*/
	long satnum = satellite_number;
	long year = input_model->year;
	long month = input_model->month;
	long day = input_model->day;
	long shour = input_model->shour;
	long smin = input_model->smin;
	long ssec = input_model->ssec;
	long sfrac = input_model->sfrac;
	long dtype = input_model->dtype;
	long nlines = input_model->nlines;

	status = c_initmod( &satnum, &year, &month, &day, &shour, &smin,
	                    &ssec, &sfrac, &dtype, &nlines, input_model );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-initmod",
	        "Error initializing the AVHRR model");
	    return E_FAIL;
	    }
	}  /* END if */
    else
	{
	/* Initialize the transformation routines
	-----------------------------------------*/
	proj_units = c_string2unit( input_ddr->proj_units );
	if ( proj_units == E_FAIL )
	    {
	    c_lasmsg(FALSE,"clavr-projunit",
                "Incorrect projection units in DDR");
	    return E_FAIL;
	    }

	prtprm[0] = prtprm[1] = FALSE;
	status = c_transinit( &proj_code, &proj_units, &zone_code, &datum_code,
                              input_ddr->proj_coef, &geo, &degrees, &utm,
		              &datum_code, all_zero, prtprm, "\0" );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-transinit",
		"Error initializing projection transformation");
	    return E_FAIL;
	    }

	/* Set up rotation coefficients in case the image
	has been rotated.  In most cases we are using square
	unrotated images but to be safe set this up anyway.
	----------------------------------------------------*/
	proj_coord[0] = input_ddr->upleft[0];
	proj_coord[1] = input_ddr->upleft[1];
	proj_coord[2] = input_ddr->loleft[0];
	proj_coord[3] = input_ddr->loleft[1];
	proj_coord[4] = input_ddr->upright[0];
	proj_coord[5] = input_ddr->upright[1];
	proj_coord[6] = input_ddr->loright[0];
	proj_coord[7] = input_ddr->loright[1];
	image_coord[0] = 1;             /* ul y */
	image_coord[1] = 1;             /* ul x */
	image_coord[2] = input_ddr->nl; /* ll y */
	image_coord[3] = 1;             /* ll x */
	image_coord[4] = 1;             /* ur y */
	image_coord[5] = input_ddr->ns; /* ur x */
	image_coord[6] = input_ddr->nl; /* lr y */
	image_coord[7] = input_ddr->ns; /* lr x */
	status = c_rtcoef( proj_coord, image_coord, rot_coeffs, &msgflag );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-rtcoef","Error initializing rotation coord");
	    return E_FAIL;
	    }

	/* Since this is a composite add the additional
	viewing angles bands to total input bands and
	set the pointer indices for them.
	-----------------------------------------------*/
	sat = input_bands++;
	sol = input_bands++;
	rel = input_bands++;
	qc  = input_bands++;
	}  /* END else */
    
    /* If the barren mask has been specified increase
    the number of bands and add the barren index.
    -------------------------------------------------*/
    if ( barren_mask_flag )
	barren_index = input_bands++;
    
    /* Always make the out index the last ptr
    -----------------------------------------*/
    out = NUMBANDS-1;

    /* Calculate the sun/earth distance squared
    -------------------------------------------*/
    sunearthdis( julian_day, &sedsq );
    sedsq = sedsq * sedsq;

    /* Get the satellite dependent constants for
    conversion of temperature and radiance.
    --------------------------------------------*/
    status = getconstants( satellite_number, &constants );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-const","Error retrieving satellite constants");
	return E_FAIL;
	}
    

    /* Get the first line of input data
    -----------------------------------*/
    status = c_estep( group_fdesc );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"clavr-estep","Error accessing input data");
	return E_FAIL;
	}


    /* Initailize and start the processing messages
    -----------------------------------------------*/
    curent = 1;
    num_bands = input_bands;
    c_inittm( "CLAVR", "Processed", "line", "bands", " ", &window[NL],
              &num_bands, &msgtim, &flag );
    c_startm();
    

/* Main loop section
*****************************************************************************/
    /* Line processing loop.  The input image may have been windowed
    so use the window number of lines as the stopping condition.
    ----------------------------------------------------------------*/
    for ( line = 0; line < window[NL]; line++, curent++ )
        {
	/* Each estep call updates the offsets into thebuf, so the pointers
	for the individual bands can be set.  The ptr array can then be used
	later with the c1, c2, c3, c4, c5, sol, sat, rel, barren_index and
	out indices to extract the desired information.
	-------------------------------------------------------------------*/
	for ( band = 0; band < input_bands; band++ )
	    {
	    ptr[band] =
		(float *)(thebuf + input_buffer_offsets[band] * sizeof(float));
	    }
	ptr[out] = (float *)(thebuf + output_buffer_offset[0] * sizeof(float));

	/* Read a line of minor frame data.  The minor frame is a label
	services routine and is unaffected by the estep so must read it.
	Input may be windowed so take this into account when assigning
	the line number to read.
	----------------------------------------------------------------*/
	if ( input_image_type == SINGLE_SCENE )
	    {
	    sprintf( linekey, "%d", line + (int)window[SL] );
	    status = c_lsread( minorframe_fdesc, linekey, &one,
	                       &minor_nbytes, char_buf,
	                       (unsigned char *)min_frm, dtype );
	    if ( status != E_SUCC )
	        {
		c_lasmsg(FALSE,"clavr-lsread","Error reading minor frame");
		return E_FAIL;
		}
	    }  /* END if input_image_type */
	
	/* Determine what band 3 is
	---------------------------*/
	if ( klm_flag )
	    {
	    band3isNOTdetermined = FALSE;
	    if ( input_image_type == SINGLE_SCENE )
		{
		/* The least significant bit of the 7th word in the
		minor frame determines which sensor was recorded:
		     0 = Chan 3B (thermal)
		     1 = Chan 3A (optical)
		---------------------------------------------------*/
		if (( min_frm[6] &1 ) == 0 )
		    band3is_thermal = TRUE;
		else
		    band3is_thermal = FALSE;
		}
	    else
	        {
		/* For a composite this determination must be done
		on a pixel by pixel basis.  Set the flag to 
		undetermined and ???
		--------------------------------------------------*/
		band3isNOTdetermined = TRUE;
		}
	    }
	else
	    {
	    band3is_thermal = TRUE;
	    band3isNOTdetermined = FALSE;
	    }

	/* Sample processing loop (this is not an example of a loop but a
	real loop for processing the samples).  The input image may be
	windowed so use the window to set up the loop starting and stopping
	conditions.  Sample will be used to index into thebuf and, since
	the offsets already take the window into account the sample index
	must start at 0;
	------------------------------------------------------------------*/
	for ( sample = 0; sample < window[NS]; sample++ )
	    {
	    if ( input_image_type == COMPOSITE && band3isNOTdetermined )
	        {
		band3isNOTdetermined = FALSE;
		qcvalue = ptr[qc][sample] /10;
		qcvalue %= 10;
		if ( qcvalue == 1 )
		    band3is_thermal = FALSE;
		else if ( qcvalue == 2 )
		    band3is_thermal = TRUE;
		else
		    {
		    /* use of channel3override should be REMOVED when
		     * reprocessing of NOAA 16 composites is complete.
		     -------------------------------------------------*/
		    if ( channel3override == 1 )
		        band3is_thermal = FALSE;
		    else if ( channel3override == 2 )
		        band3is_thermal = TRUE;
		    else
		        band3isNOTdetermined = TRUE;
		    }
		}

	    /* Unscale the data if needed
	    -----------------------------*/
	    if ( scaled_flag )
	        {
		channel1 = UNSCALE( ptr[c1][sample],
		    scale_factors[SC_REFLECTANCE].unscale,
		    scale_factors[SC_REFLECTANCE].offset );
		channel2 = UNSCALE( ptr[c2][sample],
		    scale_factors[SC_REFLECTANCE].unscale,
		    scale_factors[SC_REFLECTANCE].offset );
		if ( band3is_thermal )
		    channel3 = UNSCALE( ptr[c3][sample],
			scale_factors[SC_THERMAL].unscale,
			scale_factors[SC_THERMAL].offset );
		else
		    channel3 = UNSCALE( ptr[c3][sample],
			scale_factors[SC_REFLECTANCE].unscale,
			scale_factors[SC_REFLECTANCE].offset );
		channel4 = UNSCALE( ptr[c4][sample],
		    scale_factors[SC_THERMAL].unscale,
		    scale_factors[SC_THERMAL].offset );
		channel5 = UNSCALE( ptr[c5][sample],
		    scale_factors[SC_THERMAL].unscale,
		    scale_factors[SC_THERMAL].offset );

		if ( input_image_type == COMPOSITE )
		    {
		    /* Extract the angles, which are in
		    degrees, and convert to radians.
		    -----------------------------------*/
		    satellite_zenith = RAD_PER_DEG * UNSCALE( ptr[sat][sample],
		        scale_factors[SC_SATZEN].unscale,
			scale_factors[SC_SATZEN].offset );
		    solar_zenith = RAD_PER_DEG * UNSCALE( ptr[sol][sample],
		        scale_factors[SC_SOLZEN].unscale,
			scale_factors[SC_SOLZEN].offset );
		    relative_azimuth = RAD_PER_DEG * UNSCALE( ptr[rel][sample],
		        scale_factors[SC_RELAZ].unscale,
			scale_factors[SC_RELAZ].offset );
		    }

		}  /* END if scaled_flag */

	    /* If the data is not scaled then copy
	    the data from the input buffer as is.
	    --------------------------------------*/
	    else
	        {
		channel1 = ptr[c1][sample];
		channel2 = ptr[c2][sample];
		channel3 = ptr[c3][sample];
		channel4 = ptr[c4][sample];
		channel5 = ptr[c5][sample];
		if ( input_image_type == COMPOSITE )
		    {
		    /* Extract the angles, which are in
		    degrees, and convert to radians.
		    -----------------------------------*/
		    satellite_zenith = ptr[sat][sample] * RAD_PER_DEG;
		    solar_zenith = ptr[sol][sample] * RAD_PER_DEG;
		    relative_azimuth = ptr[rel][sample] * RAD_PER_DEG;
		    }
		}  /* END else */
	    
	    /* Calulate the latitude and longitude
	    and view angles for the pixel
	    --------------------------------------*/
	    if ( input_image_type == SINGLE_SCENE )
		{
		/* The call to ls2ll is very expensive.  So only
		make the call occasionally.  A typical 4800 line
		2048 sample image will take about 16 minutes to
		complete.  Cutting this to every other line and
		pixel takes 4:24, and every 5th line and sample
		1:02.  A good compromise is every 5th pixel on
		every line which takes about 3 1/2 minutes.
		if ( line % 2 == 0 && sample % 5 == 0 )
		------------------------------------------------*/
		if ( sample % 5 == 0 )
		    {
		    /* Remember the AVHRR model is not windowed
		    so the window SL and SS must be added.
		    -------------------------------------------*/
		    l = (float)(line   + window[SL]);
		    s = (float)(sample + window[SS]);

                    /* Also, if the scene is ascending the window 
                    must be flipped.
                    ---------------------------------------------*/
                    if ( input_model->spass == ASCENDING )
                        {
                        l = (float)input_model->nlines - l + 1.0;
                        s = (float)input_model->nsamples - s + 1.0;
                        }

		    c_ls2ll( &l, &s, &elev, input_model, &lat, &lon, view );

		    /* The function returns the lat/lon
		    in radians we want them in degrees.
		    -----------------------------------*/
		    latitude  = (float)(lat * DEG_PER_RAD);
		    longitude = (float)(lon * DEG_PER_RAD);

		    /* The function returns the angles in radians and that's
		    what we want.  However, the satellite zenith angles are
		    defined as negative from 1st scanned pixel to nadir and
		    positive after nadir so that needs to be handled.  Since
		    the image is not in satellite space, the first scanned
		    pixel for an ascending pass is sample 2048.  Relative
		    azimuth is defined as -PI to PI so make sure it's in that
		    range.
		    --------------------------------------------------------*/
		    satellite_zenith = view[0];
		    if (( input_model->cpass == ASCENDING &&  s >  NADIR ) ||
		        ( input_model->cpass == DESCENDING && s <= NADIR ))
			satellite_zenith *= -1.0;
		    solar_zenith     = view[1];
		    relative_azimuth = view[2] - PI;
		    }
		}  /* END if */
	    else
		{
		/* Determine the projection coordinates taking
		into account any rotation of the image.
		-----------------------------------------------*/
		proj_y = ( rot_coeffs[0] * (double)(line+window[SL])) +
			 ( rot_coeffs[1] * (double)(sample+window[SS])) +
			   rot_coeffs[2];
		proj_x = ( rot_coeffs[3] * (double)(line+window[SL])) +
			 ( rot_coeffs[4] * (double)(sample+window[SS])) +
			   rot_coeffs[5];

		/* Compute the latitude and longitude of the location. 
		If c_trans does not return E_SUCC we assume we are
		in a invalid region such as an interrupted Goode region
		so set the lat lon to invalid values.
		-------------------------------------------------------*/
		if ( c_trans( &proj_code, &proj_units, &geo,
		              &degrees, &proj_x, &proj_y, &lon, &lat )
			  != E_SUCC )
		    {
		    latlon_error = TRUE;
		    }
		else
		    latlon_error = FALSE;

		latitude  = (float)lat;
		longitude = (float)lon;
		}  /* END else */

	    /* Calculate the gamma angle
	    ----------------------------*/
	    gamma = DEG_PER_RAD * ( acos( cos( solar_zenith ) *
	                               cos( satellite_zenith ) +
				       sin( solar_zenith ) *
				       sin( satellite_zenith ) *
				       cos( relative_azimuth )));

	    /* Calculate channel 3 albedo if necessary
	    ------------------------------------------*/
	    if ( !klm_flag )
		{
	        channel3alb = chan3temp2alb( channel3, channel4, channel5,
		    sedsq, solar_zenith, constants );
		}
	    else if ( band3is_thermal )
		{
	        channel3alb = klm_chan3temp2alb( channel3, channel4, channel5,
		    sedsq, solar_zenith, constants );
		}
	    else
	        channel3alb = channel3 * cos(solar_zenith) / sedsq;

	    /* Calculate channel 1 and 2 albedo
	    -----------------------------------*/
	    channel1alb = channel1 * cos(solar_zenith) / sedsq;
	    channel2alb = channel2 * cos(solar_zenith) / sedsq;

	    /* Extract the barren value
	    ---------------------------*/
	    if ( barren_mask_flag )
	        barren = (int)ptr[barren_index][sample];
	    else
	        barren = 0;

	    /* Calculate the cloud code
	    ---------------------------*/
	    if ( latlon_error )
	        ptr[out][sample] = 99;
	    else if ( band3isNOTdetermined )
	        ptr[out][sample] = 97;
	    else
		{
		ptr[out][sample] = (float)clavr( channel1alb, channel2alb,
		    channel3alb, channel4, channel5, latitude, longitude,
		    gamma, barren, satellite_number, band3is_thermal,
		    thresholds );
		}

	    if ( input_image_type == COMPOSITE && klm_flag )
	        band3isNOTdetermined = TRUE;
	    }  /* END for sample loop */

	/* Read the next line of input and write
	write the output.
	----------------------------------------*/
	status = c_estep( group_fdesc );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-estep","Error accessing input data");
	    return E_FAIL;
	    }

	}  /* END for line loop */

    /* Stop the processing messages
    -------------------------------*/
    c_stoptm();

    return E_SUCC;
    }  /* END process */
