#!/bin/csh
#
# FUNCTION:     cleannsidcfix.csh
#
# PURPOSE:      Clean up unmatched ".ahdr" files residing in /sgs7/disk2/nsidc
#               (".ahdr" files without a corresponding ".arch" file).
#
#               For each ".ahdr" file residing in /sgs7/disk2/nsidc
#               without a matching associated ".arch" file,
#               relink the associated ".arch" file to the ".arch" file
#               residing on the silo.  If there is no associated ".arch"
#               file on the silo, delete the ".ahdr" file residing in
#               /sgs7/disk2/nsidc (and the ".sum" file). 
#
# BACKGROUND:   When NSIDC downloads an image from EDC, they are
#               responsible for deleting the associated header
#               file ('.ahdr').  EDC is responsible for eventually
#               unlinking the image file ('.arch') file in
#               /sgs7/disk2/nsidc from the image file residing on the silo.
#
# ALGORITM:     For each file ".ahdr" file found in the pwd,
#               if a matching ".arch" file does not exist in
#               the pwd, but one does on the silo, then create a
#               link to the one on the silo.  If there isn't one
#               on the silo, delete the ".ahdr" file found in pwd,
#               and delete the ".sum" file found in pwd.
#
# NOTES:      - To get around the 'too many files in the directory'
#               problem, the selective num0/num1/num2 matching
#               was implemented.
#             - This script will also identify local ".arch" files which are
#               linked to non-existent files on the silo, and treat them as
#               if the link didn't even exist.  (this should be OK).
#
# DIRECTIONS:   1) go into directory to clean up (/sgs7/disk2/nsidc)
#               2) execute script: "csh cleannsidcfix.csh"
#
#
#
unalias ls
echo "======= Running cleannsidcfix.csh on `date` ========" >> cleannsidcfix.log
#
foreach  num0 (0 1 2 3 4 5 6 7 8 9)
   foreach num1 (0 1 2 3 4 5)
      foreach  num2 (0 1 2 3 4 5 6 7 8 9)
         foreach ahdrfile (`ls *$num2$num1$num0.ahdr`)
	    set archfile  = `echo $ahdrfile | awk 'FS=".ahdr" {print $1}'`
	    set archfile = "$archfile.arch"
	    if (! -e $archfile) then
	       set satnum = `echo $ahdrfile | cut -c3,4`
	       set month = `echo $ahdrfile | cut -c5,6`
	       set day = `echo $ahdrfile | cut -c7,8`
	       set siloname = "./avhrr/sat$satnum/month$month/day$day/$archfile"
	       if ( -e $siloname ) then
	          echo "Linking $archfile to $siloname for $ahdrfile" >> cleannsidcfix.log
 		  ln -s $siloname $archfile
	       else
	          echo "Cannot link $archfile to $siloname for $ahdrfile" >> cleannsidcfix.log
		  echo "Removing header file $ahdrfile" >> cleannsidcfix.log
       	          rm -f $ahdrfile
	          set sumfile  = `echo $ahdrfile | awk 'FS=".ahdr" {print $1}'`
	          set sumfile = "$sumfile.sum"
	          if ( -e $sumfile) then
		     echo "Removing sum file $sumfile" >> cleannsidcfix.log
 	             rm -f $sumfile
		  endif
	       endif
	    endif
         end
      end
   end
end

