/*******************************************************************************FUNCTION:	CLEANUCFM

PURPOSE:	To remove files on Unitree (silo) that are older than the
		number of days specified using the uftp commands udir and
		umdel.

USAGE:		cleanucfm <age in days> <unitree directory> <file specification>

		age in days: files that have not been modified in the number
		    of days specified will be deleted.
		unitree directory: the directory in which to search for files
		    to delete.  The search is not recursive.
		        Eg. /avhrr/sat14/month05/day29
			 or /unitree/avhrr/sat14/month05/day29
		    Note: In certain cases -- eg. a directory specification
		    that is created using CMHOST -- it is desireable to 
		    include a /unitree prefix to prevent a conflict with a
		    local home directory.  Cleanucfm accepts /unitree and
		    will remove it prior to calling udir or umdel.  Also
		    a trailing '/' will be removed to prevent problems with
		    udir or umdel.
		file specification: a file name or wildcard specification.
		    Files that meet the specification criteria will be checked
		    for their age.  Eg. 'a[hl]140514*'
		                     or 'ah16012301123456_152445.*'

PROJECT:	ADAPS

$Log: cleanucfm.c,v $
Revision 3.1  2001/06/13 11:55:59  dlloyd
Merging branch cr4681.  Added new function to use uftp commands instead of
NFS mounts to remove files from unitree.  CR4681

Revision 1.1.2.3  2001/06/07 20:02:45  dlloyd
Modified algorithm for determining the modification year when a time value
is returned instead of a year value.
Added additional comments and other changes from code review.

Revision 1.1.2.2  2001/06/06 15:45:22  dlloyd
Clean up and comment, added error messages.

Revision 1.1.2.1  2001/06/01 19:56:29  dlloyd
Removed cleanone.csh and replaced with cleanucfm.c.  Modified Makefile.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

/* Local definitions
--------------------*/
#define  CMLEN        256
#define  SEC_PER_DAY  86400
#define  SUCCESS      0
#define  FAILURE      -1

/* Local function prototypes
----------------------------*/
static int month_atoi( char * );


int main( int argc, char *argv[] )
    {
    char  buf[CMLEN];		/* one udir directory entry		*/
    char  directory[CMLEN];	/* directory specification		*/
    char  filename[CMLEN];	/* file name from directory entry	*/
    char  month_str[4];		/* modification month string		*/
    char  time_year_str[10];	/* time or year from directory entry	*/
    char  udir_command[CMLEN];	/* unitree directory command		*/
    char  umdel_command[CMLEN];	/* unitree delete command		*/

    FILE  *fp;		/* file pointer to system command pipe		*/

    int	current_year;	/* current year, years since 1900		*/
    int current_yday;	/* current year day, days since Jan 1		*/
    int	cutoff_date;	/* cutoff date, seconds since Jan 1, 1970	*/
    int	file_year;	/* file in question year, years since 1900	*/
    int	modification_day;/* day file in question was last modified	*/
    int	month;		/* integer value of file modification month	*/
    int	status;		/* function return status			*/

    struct tm file_tm;	/* file modification time structure		*/
    struct tm *tm_ptr;	/* pointer to tm structure			*/

    time_t file_date;	/* file modification date, seconds since 1/1/70	*/
    time_t now;		/* local time, seconds since Jan 1, 1970	*/


    /* Check that the required number of parameters were sent
    ---------------------------------------------------------*/
    if ( argc != 4 )
	{
	fprintf( stderr, "CLEANUCFM: Insufficient arguments\n" );
	fprintf( stderr,
	  "Usage: %s <age in days> <unitree directory> <file specification>\n",
	  argv[0] );
	return FAILURE;
	}


    /* If '/unitree' was specified in the directory argument, remove it
    -------------------------------------------------------------------*/
    if ( strncmp( argv[2], "/unitree", 8 ) == 0 )
	strcpy( directory, &argv[2][8] );
    else
	strcpy( directory, argv[2] );


    /* If a trailing '/' was specified, remove it
    ---------------------------------------------*/
    if ( directory[strlen(directory)-1] == '/' )
        directory[strlen(directory)-1] = '\0';


    /* Calculate the cutoff date
    ----------------------------*/
    if ( !isdigit( argv[1][0] ))
        {
	fprintf( stderr, "CLEANUCFM: First argument, %s, is not numeric\n",
	    argv[1] );
	return FAILURE;
	}
    now = time( NULL );
    cutoff_date = now -
	(time_t)( atoi( argv[1] ) * SEC_PER_DAY + SEC_PER_DAY );


    /* Get the current year
    -----------------------*/
    tm_ptr = localtime( &now );
    current_year = tm_ptr->tm_year;
    current_yday = tm_ptr->tm_yday;


    /* Build the directory listing command.  The pipe into the awk
    command is used to exclude extraneous listing entries and extract
    the four fields we need.  A udir listing includes several lines of
    information that are not directory listings.  eg.

    [informational] Remote system is <edcsgs3>
    200 PORT command successful.
    150 Opening ASCII mode data connection for /usr/unitree/bin/ddir (0 bytes).
    -rw-r--r--  1 dlloyd    avhrr    DK  common      2253 Jun  6 14:33 Makefile

    Entries with 'DK' or 'AR' in the 5th field are directory listings.
    The awk passes on any entries that match the expression:
        ($5 == \"DK\") || ($5 == \"AR\")
    where $5 is field 5 of the entry.  In addition, those entries that are
    passed to the output are formatted by the {print ...} statement.  In
    this case we are printing fields 8 through 11 which are:
        8  - the month of last modification
	9  - the day of last modification
	10 - either the year or time of last modification
	11 - the file name
    -------------------------------------------------------------------------*/
    sprintf( udir_command, "udir -d '%s' '%s'", directory, argv[3] );
    strcat( udir_command,
        " | awk '($5 == \"DK\") || ($5 == \"AR\") {print $8,$9,$10,$11}'" );


    /* Open a pipe to the udir_command
    ----------------------------------*/
    fp = popen( udir_command, "r" );
    if ( !fp )
	{
	fprintf( stderr, "CLEANUCFM: Unable to open pipe for udir\n" );
	return FAILURE;
	}


    /* loop through all the directory entries
    -----------------------------------------*/
    while ( fgets( buf, CMLEN-1, fp ) != NULL )
	{
	/* Scan the needed information
	------------------------------*/
	status = sscanf( buf, "%s %d %s %s", month_str, &modification_day,
	    time_year_str, filename );
	if ( status != 4 )
	    {
	    fprintf( stderr, "CLEANUCFM: Error scanning directory listing\n" );
	    return FAILURE;
	    }

	/* Convert the month string to an integer
	-----------------------------------------*/
	month = month_atoi( month_str );

	/* See if there is a modification time or year in the 3rd field.
	The presence of a ':' indicates it is a time value.  If it is a
	time value, determine if the modification date occurred before
	today or after today (in calendar year terms).  Eg.  if today is
	Jun 6 and the modification time is Jan 12, the file was modified
	in the current year; however, if the modification time is Aug 2,
	the file was modified last year.  If the 3rd field is a year
	value convert it to years since 1900.
	----------------------------------------------------------------*/
	if ( strchr( time_year_str, ':' ) != NULL )
	    {
	    /* Get an estimate of the days since Jan 1.  This is
	    only an estimate since we don't know the actual year.
	    It may be off by one if the 'actual' year the file
	    was modified and the current year are not the same
	    length because one is a leap year.
	    -----------------------------------------------------*/
	    file_tm.tm_sec   = 0;
	    file_tm.tm_min   = 0;
	    file_tm.tm_hour  = 0;
	    file_tm.tm_mday  = modification_day;
	    file_tm.tm_mon   = month;
	    file_tm.tm_year  = current_year;
	    file_tm.tm_wday  = 0;
	    file_tm.tm_yday  = 0;
	    file_tm.tm_isdst = 0;
	    /* mktim updates the tm_yday field */
	    mktime( &file_tm );
            
	    /* We must assume that any file modified earlier than
	    the current day of the year was modified in the current
	    year and those modified later in the year were modified
	    in the previous year.
	    -------------------------------------------------------*/
	    if ( file_tm.tm_yday <= current_yday )
		file_year = current_year;
	    else
	        file_year = current_year - 1;
	    }
	else
	    file_year = atoi( time_year_str ) - 1900;

	/* Load up a tm structure with the file modification date,
	using the file modification  year, and convert it to seconds.
	-------------------------------------------------------------*/
	file_tm.tm_sec   = 0;
	file_tm.tm_min   = 0;
	file_tm.tm_hour  = 0;
	file_tm.tm_mday  = modification_day;
	file_tm.tm_mon   = month;
	file_tm.tm_year  = file_year;
	file_tm.tm_wday  = 0;
	file_tm.tm_yday  = 0;
	file_tm.tm_isdst = 0;
	file_date = mktime( &file_tm );

	/* Compare the file date with cutoff
	date and, if it's older, delete it.
	------------------------------------*/
	if ( file_date < cutoff_date )
	    {
	    /* build a umdel command */
	    sprintf( umdel_command, "umdel -d '%s' '%s'", directory, filename );
	    status = system( umdel_command );
	    if ( status == -1 )
		{
		fprintf( stderr, "CLEANUCFM: Error deleting %s/%s\n",
		    directory, filename );
		return FAILURE;
		}
	    fprintf( stdout, "CLEANUCFM: Deleted %s/%s\n",
	        directory, filename );
	    }  /* END if */

	}  /* END while */


    /* Close the pipe.  On error just print an error
    message and return SUCCESS.  This will allow the
    calling pdf to continue without aborting.
    ------------------------------------------------*/
    if ( pclose( fp ) == -1 )
        fprintf( stderr, "CLEANUCFM: Error closing pipe\n" );

    return SUCCESS;
    }  /* END cleanucfm */

/******************************************************
Local function to convert a 3-digit month abbreviation,
Jan, Feb, Mar ..., to an integer value that equates to
months since January.  Eg. Jan = 0, Feb = 1, Dec = 11.
*******************************************************/
int month_atoi( char *month_str )
    {
    int mon;	/* month value (0..11)			*/

    switch ( toupper( month_str[0] ))
	{
	case 'F' :
	    mon = 1;
	    break;
	case 'S' :
	    mon = 8;
	    break;
	case 'O' :
	    mon = 9;
	    break;
	case 'N' :
	    mon = 10;
	    break;
	case 'D' :
	    mon = 11;
	    break;
	case 'J' :
	    if ( toupper( month_str[1] ) == 'A' )
		mon = 0;
	    else if ( toupper( month_str[2] ) == 'N' )
		mon = 5;
	    else 
		mon = 6;
	    break;
	case 'M' :
	    if ( toupper( month_str[2] ) == 'R' )
		mon = 2;
	    else
		mon = 4;
	    break;
	case 'A' :
	    if ( toupper( month_str[1] ) == 'P' )
		mon = 3;
	    else
		mon = 7;
	}  /* END switch */

    return mon;
    }  /* END month_atoi */
