#!/bin/csh
#
# FUNCTION: Cleanup
#
# PURPOSE: Clean up files that may exist because the system went
# 	   down during ADAPS processing.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# T. Baltzer       12/93  Development
# T. Baltzer       08/94  Updated to reflect changes in ADAPS software
#
#

#-------------------------
# save current directory
#-------------------------
set curdir=`pwd`

#-------------------------------------------------------
# Browse files created by REFAID which have a temporary
# file name extension of "tbrw".
#-------------------------------------------------------
rm -f $ADAPSBRW/*.tbrw

#--------------------
# Ingest files
#--------------------
rm -f $ADAPSING/A*.tmp
rm -f $ADAPSING/*.lock

#--------------------
# Acquire files
#--------------------
rm -f $ADAPSACQ/z*.tmp
rm -f $ADAPSACQ/AH*

#--------------------
# Microfiche files
#--------------------
rm -f $ADAPSMIC/mic_t*.img
rm -f $ADAPSMIC/mic_t*.ddr
rm -f $ADAPSMIC/mic_t*.his

#----------------------------------------------------------
# If there is a micro_aa.pid file, where pid is a literal
# process id, then rename it to micro_aa.sav.
# Likewise for a micro_ah file.
#----------------------------------------------------------
if ( -r $ADAPSTABLES/micro_aa.* ) then
   mv $ADAPSTABLES/micro_aa.* $ADAPSTABLES/micro_aa.sav
endif

if ( -r $ADAPSTABLES/micro_ah.* ) then
   mv $ADAPSTABLES/micro_ah.* $ADAPSTABLES/micro_ah.sav
endif

#--------------------
# Images files
#--------------------
rm -f $ADAPSIMG/ann_m*
rm -f $ADAPSIMG/ann_t*
rm -f $ADAPSIMG/las_t*
rm -f $ADAPSIMG/map_t*
rm -f $ADAPSIMG/mic_s*
rm -f $ADAPSIMG/fax_s*

#-----------------------------------
# Microfiche processing in ADAPSIMG
#-----------------------------------
rm -f $ADAPSIMG/annot_*
rm -f $ADAPSIMG/bordr_*
rm -f $ADAPSIMG/fiche*

#---------------------------
# Archive conversion files
#---------------------------
rm -f $ADAPSPROD/*.tmp
rm -f $ADAPSPROD/*.lock
rm -f $ADAPSTABLES/readeaf.lock

#--------------------
# Grid files
#--------------------
rm -f ~opsavhrr/adaps/*.grid

#---------------------------------------------------------
# Move AUTOSTACK files back into their proper directory.
# Search all of $ADAPSARC/esa/* and $ADAPSARC/sfl/* for
# subdirectories of the form ansistack*.  If there are
# files there, rename them into their sceneid name if
# they have a tapeid name (they were being stacked at
# the time of system crash):
# 'ihsceneid.timestamp' goes to 'sceneid_timestamp.arch'
# 'hhsceneid.timestamp' goes to 'sceneid_timestamp.ahdr'
# 'igsceneid.timestamp' goes to 'sceneid_timestamp.arch'
# 'hgsceneid.timestamp' goes to 'sceneid_timestamp.ahdr'
# 'ilsceneid.timestamp' goes to 'sceneid_timestamp.arch'
# 'hlsceneid.timestamp' goes to 'sceneid_timestamp.ahdr'
# Once renamed, move the files up one directory.
#---------------------------------------------------------
rm -f $ADAPSARC/esa/*/ansistack.lock
rm -f $ADAPSARC/sfl/*/ansistack.lock
foreach dir (`ls -d $ADAPSARC/esa/*/ansistack* $ADAPSARC/sfl/*/ansistack*`)
   cd $dir
   if (-e zzztapelist.list) rm -f zzztapelist.list
   set imagelist = ""
   foreach file (`ls`)
      if ($file =~ [ih][ghl]*) set imagelist = "$imagelist $file"
   end
   foreach file ($imagelist)
      if ($file =~ i*) then
         set name=`echo $file | sed -e 's/^i/a/;s/\./\_/;s/$/.arch/'`
      else if ($file =~ h*) then
         set name=`echo $file | sed -e 's/^h/a/;s/\./\_/;s/$/.ahdr/'`
      endif
      mv $file $name
   end
   foreach file (`ls`)
      mv $file ../.
   end
end

#------------------------------------------------
# Check for AVHRRSTACK lock files in ADAPSTABLES.
# These lock files are used to reserve the tape
# drives.  The file name format is
# avhrrstack_DRIVE.lock where DRIVE is the drive
# name.
#------------------------------------------------
cd $ADAPSTABLES
foreach file (`ls avhrrstack*.lock`)
   set drive = `echo $file | awk -F_ '{print $2}' | awk -F\. '{print $1}'`
   echo "Unlocking drive $drive"
   rm -f avhrrstack_$drive.lock
end
if ($?drive) then
   echo ""
   echo "If a 4 mm or 8 mm tape drive was in use when the system went"
   echo "down, all but the last file on the tape should be valid data."
   echo "A tape label and/or a shipping form must to be manually"
   echo "generated for this tape."
   echo "If a 9-track tape or a 3480 tape drive was in use when the"
   echo "system went down, the entire tape will be regenerated.  No"
   echo "manual operator intervention is required in this case."
   echo ""
endif

#---------------------------------------------------
# Delete any orbital stitching files that might be
# hanging around.
#---------------------------------------------------
#----------------------------------------------------
# If there are line detection fragments, delete them.
# ORBLN reads from ADAPSORB/lndetect and write to
# ADAPSORB/process with a "_tmp" extension added to the
# name.  Once the scene has been completely LNDETECT'ed,
# the "_tmp" part of the file name is again eliminated.
#----------------------------------------------------
cd $ADAPSORB/lndetect
foreach file (`ls`)
   if ($file =~ *.lock) then
      rm -f $file
      set ahdrname=`echo $file | sed -e 's/^i/a/;s/\.lock/\_tmp.ahdr/'`
      set archname=`echo $file | sed -e 's/^i/a/;s/\.lock/\_tmp.arch/'`
      cd $ADAPSORB/process
      rm -f $ahdrname
      rm -f $archname
      cd $ADAPSORB/lndetect
   endif
end

#---------------------------------------------
# If there are MAKORB fragments, delete them.
#---------------------------------------------
rm -f $ADAPSORB/orbits/*.tmp

#---------------------------------------------
# If there are archive file fragments in
# ADAPSORB/lndetect, ADAPSORB/process, ADAPSORB/orbits
# move them into the ADAPSORB/error directory.
#---------------------------------------------
foreach dir (`ls -d $ADAPSORB/lndetect $ADAPSORB/process $ADAPSORB/orbits`)
   cd $dir
   foreach file (`ls`)
      if ($file =~ *.ahdr) then
         set archname=`echo $file | sed -e 's/^i/a/;s/\.ahdr/\.arch/'`
         if (! -r $archname) mv $file ../error/.
      endif
      if ($file =~ *.arch) then
         set ahdrname=`echo $file | sed -e 's/^i/a/;s/\.arch/\.ahdr/'`
         if (! -r $ahdrname) mv $file ../error/.
      endif
   end
end

#-------------------------------------
# change back to origonal directory
#-------------------------------------
cd $curdir

#-----------------------------
# Delete all of the stop files
#-----------------------------
rm -f $ADAPSTABLES/*.stop


echo "End of CLEANUP procedure"
