/*******************************************************************************
NAME:		GET_GEOGRAPHIC_IMAGES

PURPOSE:	Get the user specified images in geographic space

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T.Baltzer      11/99           CSC           Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"	
#include "util.h"	
#include "adaps.h"	
#include "compatm.h"


void get_geographic_images
(
    struct PARBLK *vblock,			/* IN:  TAE parameter block   */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],	/* OUT: Water vapor buffers   */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],	/* OUT: Ozone image buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],	/* OUT: QC band buffers	      */
    int	*nwater 				/* OUT: Number of water bands */
)

{

long	bands[MAXDCOMP][MAXBND+1],   /* bands specified			*/
	element	= 0,		/* parameter element number		*/
	imglimit = 1,           /* image limit for getin call           */
	nbands[MAXDCOMP],       /* number of bands specified		*/
	numimages,              /* number of images specified		*/
	one = 1,		/* Constant value			*/
	qflag[1][2],            /* window band flags for getin call     */
	same_window,            /* same window flag for getin call      */
	totband,                /* total bands specified		*/
	wind[1][4];             /* window specification			*/

int	i,			/* looping counter			*/
	nozone;			/* number of ozone images		*/

char ozone[MAXDCOMP][CMLEN],	/* Ozone image names			*/
     qc[MAXDCOMP][CMLEN],	/* QC image names			*/
     qcdir[CMLEN],		/* QC image directory specification	*/
     tnames[MAXDCOMP][CMLEN],	/* tae file name specified		*/
     tmpstr[CMLEN],		/* Temporary string			*/
     water[MAXDCOMP][CMLEN],	/* Water vapor image names		*/
     waterdir[CMLEN];		/* Water vapor image dir specification	*/

struct VARIABLE	*val;		/* pointer to a TAE variable		*/


/* Retrieve the number of ozone images specified.
--------------------------------------------------*/
val = p_find(vblock,"NOZONE");
nozone = IVAL(*val,0);

/* Retrieve the number of water vapor and qc images specified.
---------------------------------------------------------------*/
val = p_find(vblock,"NWATER");
*nwater = IVAL(*val,0);

/* Retrieve the water vapor image directory specification.
NOTE:  This parameter was introduced to avoid a TAE block
limitation error.
---------------------------------------------------------------*/
val = p_find(vblock,"WATERDIR");
strcpy(waterdir,SVAL(*val,0));

/* Retrieve the quality control image directory specification.
NOTE:  This parameter was introduced to avoid a TAE block
limitation error.
---------------------------------------------------------------*/
val = p_find(vblock,"QCDIR");
strcpy(qcdir,SVAL(*val,0));

/* Make sure that the correct number of water vapor images has
   been specified
---------------------------------------------------------------*/
val = p_find(vblock,"WATER");
if (val->v_count != *nwater)
    {
    c_lasmsg(FALSE,"compatm-numv",
		"Invalid number of water vapor images specified");
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
    }

/* Assemble the water vapor image names.
-----------------------------------------*/
for (i = 0; i < *nwater; i++)
    {
    strcpy( tmpstr, waterdir );
    strcat( tmpstr, SVAL( *val, i ));
    c_cmhost( tmpstr, water[i] );
    strcat( water[i], ".img" );
    }

/* Make sure that the correct number of qc images has been
   specified
---------------------------------------------------------------*/
val = p_find(vblock,"QC");
if (val->v_count != *nwater)
    {
    c_lasmsg(FALSE,"compatm-numv",
		"Invalid number of quality control images specified");
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
    }

/* Assemble the qc image names.
---------------------------------*/
for (i = 0; i < *nwater; i++)
    {
    strcpy( tmpstr, qcdir );
    strcat( tmpstr, SVAL( *val, i ));
    c_cmhost( tmpstr, qc[i] );
    strcat( qc[i], ".img" );
    }

/* Get the water vapor and qc image data, read into buffers
------------------------------------------------------------*/
if ( get_geographic_buffers(water, *nwater, &one, waterbuf) != E_SUCC)
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
if ( get_geographic_buffers(qc, *nwater, &one, qcbuf) != E_SUCC)
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");


/* Make sure that the correct number of ozone images has
   been specified
---------------------------------------------------------------*/
val = p_find(vblock,"OZONE");
if (val->v_count != nozone)
    {
    c_lasmsg(FALSE,"compatm-numv",
		"Invalid number of ozone images specified");
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
    }

for (i = 0; i < nozone; i++)
    {
    /* Get the ozone image names.
    -------------------------------*/
    element = i;
    if (c_getin( vblock, "OZONE", &element, tnames[i],
		 ozone[i], &numimages,
		 wind, &bands[i][0], &nbands[i], &totband, &imglimit,
		 &same_window, qflag) != E_SUCC)
	c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");

    /* Get the ozone image data, read into the buffer
    ---------------------------------------------------*/
    if ( get_geographic_buffers(&ozone[i][0], one, &bands[i][0],
				&ozonebuf[i][0][0]) != E_SUCC)
	c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");

    }


}

