/******************************************************************************
NAME:   			PROCESS 

PURPOSE:    Process the data to generate the necessary output images

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T.Baltzer      11/99           CSC           Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS

******************************************************************************/

#include <string.h>
#include "las.h"
#include "adaps.h"
#include "util.h"
#include "pixman.h"
#include "compatm.h"

#define UNDEFINED     -1	/* Value signifying an undefined value  */
#define UNDEFINEDQC  255	/* Value signifying an undefined QC	*/
#define DTBND_IN   0		/* Date band index			*/
#define SOLZN_IN   1		/* Solar zenith index			*/
#define LINES_IN   2		/* 'lines' band index			*/
#define SAMPS_IN   3		/* 'samples' band index			*/

#define WATER_OUT  0		/* Water band ndex			*/
#define OZONE_OUT  1		/* Ozone band index			*/
#define QCBND_OUT  2		/* Quality control band index		*/


extern long curent;			/* Current line being processed		*/


void process
(
    struct PARBLK *vblock,		/* IN: TAE parameter block	     */
    char hosin[][CMLEN],		/* IN: host names of input image     */
    char hosout[][CMLEN],		/* IN: host name of output image     */
    struct MYFD myfd[],			/* IN: File descriptors to the images */
    struct GDESC **gdesc,		/* IN: group descriptor for image I/O */
    long offset1[MAXIMG*MAXBND],	/* IN: offsets for input bands	     */
    long offset2[MAXBND],   		/* IN: offsets for output bands	     */
    char *thebuf,			/* IN: image I/O buffer              */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],	/* IN: Water vapor buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],	/* IN: QC band buffers	     */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],	/* IN: Ozone image buffers   */
    int scenelist[MAXSCENE],			/* IN: Scene listing	     */
    char datelist[MAXDCOMP][7],			/* IN: Date listing	     */
    int nwater,				/* IN: Number of water bands	     */
    long msgtim,			/* IN: time between display messages */
    char *minmax,			/* IN: min max flag                  */
    char *scale				/* IN: Scaling technique US, GLOBAL  */
)

{


static double  minval[MAXBND];	/* min pixel value array                     */
static double  maxval[MAXBND];	/* max pixel value array                     */

float   *rthebuf;		/*  pointer to thebuf			     */

long	bands_index,		/* Date band index			     */
	i,			/* Counter			     	     */
	sample,			/*  counter -- current pixel		     */
	lines_index,		/* 'Lines' index into geographic images	     */
	mmflag = FALSE,		/* Min max flag				     */
	msg_flag=GEN2,		/* flag for processing message               */
	nl,			/* number of image lines                     */
	ns,			/* number of image samples                   */
	num_in_img = NUM_IN_IMG,/* Number of input images		     */
	num_out_bnd,		/* Number of output bands		     */
	ozone_index,		/* Ozone image band index  		     */
	ozonelist[MAXDCOMP],	/* Ozone index list	     		     */
	rdtype = EREAL,		/* Floating point data type		     */
	samps_index,		/* 'Samples' index into geographic images    */
	solzn_index,		/* Solar Zenith angle index		     */
	two = 2,		/* constant				     */
	UNDEFINED_DB_MAX;	/* Undefined date band maximum value	     */


/* Assign the undefined date band values based on the type of composite:

   US:     0 = undefined 

   GLOBAL: 0 = missing over land
	   1 = pixel over ocean
	   2 = pixel in Goodes interupt area

   NOTE: Don't have to check for less than zero because values are coming from
	 a byte image.
-----------------------------------------------------------------------------*/
if (strcmp(scale,"GLOBAL") == 0)
    UNDEFINED_DB_MAX = 2;
else
    UNDEFINED_DB_MAX = 0;


/* Setup parameters
---------------------------*/
nl = myfd[0].nlines;
ns = myfd[0].nsamples;
rthebuf = (float *) thebuf;

/* Initialize min/max arrays
   mmflag is set to be used within the processing loop -- quicker than strcmp
-----------------------------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   mmflag = TRUE;	/* set flag			 */

   num_out_bnd = 2;	/* water vapor band & ozone band */
   c_inmnmx(&rdtype,&num_out_bnd,&minval[0],&maxval[0]);

   num_out_bnd = 1;	/* qc band			 */
   c_inmnmx(&rdtype,&num_out_bnd,&minval[2],&maxval[2]);
   }


/* Determine the ozone indexing list based on the date list.  For example, a
   datelist = "083099","083199","090199" would yield an ozone index list
   = (0, 0, 1).  The month in datelist[0] is always considered as the "0"
   value in ozonelist.  There are never more than two months in the
   ozone list.
---------------------------------------------------------------------------- */
ozonelist[0] = 0;
for (i = 1; i < nwater; i++)
   {
   if (strncmp(datelist[i],datelist[0],2) == 0)
      ozonelist[i] = 0;
   else
      ozonelist[i] = 1;
   }

/* Setup for processing message utility
---------------------------------------*/
curent = 1;    
c_inittm("COMPATM","Processed","line","bands"," ",&nl,&num_in_img,&msgtim,
	 &msg_flag);
c_startm();

/* Main processing loop, let the processing begin
--------------------------------------------------*/
for (curent = 0; curent < nl; curent++)
   {
   /* Read / write a line of the image */
   if (c_estep(gdesc) != E_SUCC)
      c_clean(vblock,"compatm",hosout,&two);

   for (sample = 0; sample < ns; sample++)
      {
      /* Get the value of each sample for each input band 
      ----------------------------------------------------*/
      lines_index = (long) (*(rthebuf + (*(offset1+LINES_IN) + sample)));
      samps_index = (long) (*(rthebuf + (*(offset1+SAMPS_IN) + sample)));
      bands_index = (long) (*(rthebuf + (*(offset1+DTBND_IN) + sample)));
      solzn_index = (long) (*(rthebuf + (*(offset1+SOLZN_IN) + sample)));

      /* Make sure we are not pointing to an unspecified scene index */
      if ((bands_index > UNDEFINED_DB_MAX) &&
	  ((bands_index >= MAXSCENE) ||
	  (scenelist[bands_index] == UNDEFINED)))
	 {
	 c_lasmsg(FALSE,"compatm-band","Invalid date band specification");
	 c_clean(vblock,"compatm",hosout,&two);
	 }

      /* Assign a pixel value for each of the three output bands,
	 undefined or otherwise.
      --------------------------------------------------------*/
      if ((lines_index == UNDEFINED) || (samps_index == UNDEFINED) ||
	  (bands_index <= UNDEFINED_DB_MAX))
	 {
	 *(rthebuf + (*(offset2+WATER_OUT) + sample)) = UNDEFINED;
	 *(rthebuf + (*(offset2+OZONE_OUT) + sample)) = UNDEFINED;
	 *(rthebuf + (*(offset2+QCBND_OUT) + sample)) = UNDEFINEDQC;
	 }
      else
	 {

         /* 'bands_index' is the value from out of the date band image.  This
	    value is used as an index into the 'scenelist' list.  This value
	    is used as an index into the 'ozonelist' list.  
         ---------------------------------------------------------------*/
         ozone_index = ozonelist[scenelist[bands_index]];

	 *(rthebuf + (*(offset2+WATER_OUT) + sample)) =
		waterbuf[scenelist[bands_index]][lines_index-1][samps_index-1];
	 *(rthebuf + (*(offset2+OZONE_OUT) + sample)) =
		ozonebuf[ozone_index][lines_index-1][samps_index-1];
	 *(rthebuf + (*(offset2+QCBND_OUT) + sample)) =
		qcbuf[scenelist[bands_index]][lines_index-1][samps_index-1];

	 /* Flag extreme solar zenith angle values in the qc band
	 -----------------------------------------------------*/
	 if (solzn_index > SZANGLE)
	     *(rthebuf + (*(offset2+QCBND_OUT) + sample)) += SZSCALE;
	 }
      }

      /* Calculate the min/max values for the output bands, if specified 
       ----------------------------------------------------------------*/
      if (mmflag)
	 {
         c_pxmnmx((long *)(rthebuf + *(offset2+WATER_OUT)),&ns,
                  &minval[WATER_OUT],&maxval[WATER_OUT],&rdtype); 
         c_pxmnmx((long *)(rthebuf + *(offset2+OZONE_OUT)),&ns,
                  &minval[OZONE_OUT],&maxval[OZONE_OUT],&rdtype); 
         c_pxmnmx((long *)(rthebuf + *(offset2+QCBND_OUT)),&ns,
                  &minval[QCBND_OUT],&maxval[QCBND_OUT],&rdtype); 
	 }
   }

/* Write the final line of output data
---------------------------------------*/
if (c_estep(gdesc) != E_SUCC)
      c_clean(vblock,"compatm",hosout,&two);

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Save the associated min/max values
------------------------------------------------------------------------*/
num_out_bnd = 2;	/* water vapor band & ozone band */
if (c_svmnmx(hosout[0],&num_out_bnd,&minval[0],&maxval[0],minmax) != E_SUCC)
   c_lasmsg(FALSE,"compatm-warn","Nonfatal error encountered");

num_out_bnd = 1;	/* qc band */
if (c_svmnmx(hosout[1],&num_out_bnd,&minval[2],&maxval[2],minmax) != E_SUCC)
   c_lasmsg(FALSE,"compatm-warn","Nonfatal error encountered");


/* Update the DDR's with the projection information
---------------------------------------------------*/
update_ddr(vblock,hosin,hosout);

/* Update the BDDR's with the data source
------------------------------------------*/
update_bdr(vblock,hosout);

}
