/*******************************************************************************
NAME:	DCWCHIPS

PURPOSE:	Rasterize data from the Digital Chart of the World,
		creating separate 64x64 image "chips" that can be used
		in automatic correlation of the image.  The chips are specified
		as 64x64 pixel images surrounding tie points in a tps file.
		The input image must have an associated AVHRR Data Descriptor 
		Record (ADDR) if using the satellite projection, or must have 
		a valid DDR if using another projection.

PROGRAM HISTORY:
Version      Date    Programmer		Request
-------      ----    ----------         -------
  1.0	     04/93   T. Rockvam		Original development; much stolen from
					DCW2TAB and WDB2RAS
					Improved the rasterization routine 
					(scani) to create finer lines for use 
					in correlation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	Must be run under TAE

PROJECT:	ADAPS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "dcw.h"
#include "adaps.h"
#include "geompak.h"
#include "dcwchips.h"

main()
{
extern long curent;		/* Current processing line		*/

				/* coverage definitions, initialized to none*/
static struct DCW_COVERAGE dcw_coverage[] = {
    {"Political/Oceans",        "po", 0, 0, 3, FALSE, NULL, NULL},
    {"Drainage",                "dn", 0, 0, 3, FALSE, NULL, NULL},
    {"Roads",                   "rd", 0, 0, 2, FALSE, NULL, NULL},
    {"Railroads",               "rr", 0, 0, 2, FALSE, NULL, NULL},
    {"Hypsography",             "hy", 0, 0, 3, FALSE, NULL, NULL},
    {"Hypsography-Sup",         "hs", 0, 0, 2, FALSE, NULL, NULL},
    {NULL,                      NULL, 0, 0, 0, FALSE, NULL, NULL},
    };

struct	PARBLK	vblock;		/* TAE parameter block			*/
struct  DDR     ddr;    	/* DDR information                      */
struct  MODEL   model;		/* AVHRR model				*/
struct  MTPDATA mtp_data;       /* search tps data record 		*/

char	cdpath[CMLEN],		/* location of dcw			*/
	chip_name[CMLEN], 	/* name of output chip image		*/
	hosin[1][CMLEN],  	/* input host file			*/
	library[4][CMLEN],	/* libraries (areas) of interest	*/
	msg[ERRLEN],		/* informational message		*/
	mtpfile[CMLEN],	  	/* mtp file name			*/
	proj[6],		/* projection type (sat or proj)	*/
	taein[1][CMLEN],  	/* input TAE image name			*/
	taeout[CMLEN],		/* output TAE chip image name		*/
	tpsfile[CMLEN];	  	/* tps file name			*/

int     *tps_desc,              /* tps file descriptor                  */
	*mtp_desc;		/* mtp file descriptor			*/

long    acount,			/* number of areas to rasterize		*/
	backgrnd,		/* background value for rasterized image*/
	bands[1][MAXBND+1], 	/* band number to be processed		*/
	chip_window[4], 	/* window spec for chip area		*/
	curchip = 0,		/* current chip image			*/
	fatal = -1,		/* fatal error code			*/
	imgback = FALSE, 	/* image data in background flag	*/
	msgtim,			/* time between processing messages	*/
	nbands[1],		/* number of bands per input image	*/
	nchips = 0,		/* number of chips successfully created	*/
	nonfatal = 1,		/* nonfatal error code			*/
	nout = 1,		/* number of output chips to create	*/
	pixval,			/* intensity value for rasterized pixels*/
	odtype,			/* output data type			*/
	status,			/* return status variable		*/
	std_out;		/* standard output			*/

double  corner[4]; 		/* corner (lat/long) spec for each chip	*/
double	elev;			/* elevation of tie point		*/

/*
  Initialize the TAE parameter block
*/
c_inlas(&vblock,&std_out);

/*
  Get the input parameters
*/
status = getpar(&vblock,&ddr,&model,cdpath,&acount,library,dcw_coverage,taein,
	 hosin,tpsfile,mtpfile,bands,nbands,&odtype,&pixval,
	 &backgrnd,&imgback,proj,&msgtim);
if (status == E_FAIL)
    c_errmsg("Fatal error encountered","dcwchips-fatal",&fatal);

/*
  Set up tps files and projection stuff
*/
status = set_up(proj,&ddr,tpsfile,mtpfile,&tps_desc,&mtp_desc);
if (status == E_FAIL)
    c_errmsg("Fatal error encountered","dcwchips-fatal",&fatal);

while (TRUE)
    {
    /*
      Get the next chip to process
    */
    curchip++;
    if (strcmp(proj,"SAT") == 0)
        {
        status = get_sat_chip(&tps_desc,&mtp_data,&model,&ddr,&elev,
			      chip_window,corner,chip_name,taeout,nchips+1);
        }
    else
        {
        status = get_proj_chip(&tps_desc,&mtp_data,&ddr,chip_window,corner,
			       chip_name,taeout,nchips+1);
        }
    if (status == E_EOF) 
	break;
    else if (status != E_SUCC)
	{
        c_delfil(mtpfile);
    	c_errmsg("Fatal error encountered","dcwchips-fatal",&fatal);
	}

    /*
      Create the chip
    */
    if (strcmp(proj,"SAT") == 0)
        status = satsel(&ddr,&model,&elev,hosin[0],bands[0],nbands[0],
			chip_name,cdpath,library,acount,dcw_coverage,pixval,
			chip_window,backgrnd,imgback,&msgtim,odtype,
			corner[MINLAT],corner[MINLON],
			corner[MAXLAT],corner[MAXLON]);
    else
        status = projsel(&ddr,hosin[0],bands[0],nbands[0],chip_name,
		     cdpath,library,acount,dcw_coverage,
		     pixval,chip_window,backgrnd,imgback,&msgtim,odtype,
		     corner[MINLAT],corner[MINLON],
		     corner[MAXLAT],corner[MAXLON]);
    if (status != E_SUCC)
	{
        sprintf(msg,"Error occurred creating %s",taeout);
        c_errmsg(msg,"dcwchips-nocreate",&nonfatal);
	}
    else
	{
	nchips++;
    	finish(hosin,chip_name,&vblock,chip_window,bands,nbands);
        /*sprintf(msg,"Successfully created %s",taeout);*/
        sprintf(msg,"Created %s",taeout);
        c_errmsg(msg," ",&nonfatal);

    	/*
      	  Finish up (update mtp and associated files)
      	  Only write to mtp if the chip was created
    	*/
    	if (c_ptio(&mtp_desc,&mtp_data) == E_FAIL)
	    c_errmsg("Error writing merged tie point file","dcwchips-write",
		&nonfatal);
	}
    }

/*
  Close tie points files
*/
c_ptclse(&tps_desc);
c_ptclse(&mtp_desc);

/*
  If no chips were found, don't keep an empty mtp file
*/
strcpy(taeout, mtpfile);
if (nchips == 0)
    {
    c_delfil(mtpfile);
    nout = 0;
    }

/*
  Issue completion message
*/
sprintf(msg,"Successfully created %d chip images",nchips);
c_errmsg(msg," ",&nonfatal);
c_complt("DCWCHIPS",taeout,&nout);
}
