/*******************************************************************************
NAME:		GET_PROJ_CHIP

PURPOSE:	Get a valid chip from the tps file and calculate its 
		line,sample and lat,long extents for a projected image
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "proj.h"
#include "dcw.h"
#include "geompak.h"
#include "dcwchips.h"

FUNCTION get_proj_chip(tps_desc,mtp_data,ddr,window,corners,chip_name,
			taeout,chip_no)

int	**tps_desc;		/* tps file descriptor			*/

struct 	MTPDATA *mtp_data;      /* search tps data record 		*/
struct  DDR	*ddr;		/* ddr for the image			*/

long	window[4],		/* window specification for each chip	*/
	chip_no;		/* current chip number			*/

double	corners[4];		/* lat/long range for each chip		*/

char	*chip_name;		/* output chip image name		*/
char	*taeout;		/* tae output chip image name		*/
{
struct TPSDATA tps_data;        /* tps data record 			*/

long	i,			/* looping variable			*/
	nonfatal = 1,		/* non-fatal error flag 		*/
	flag = 1,		/* flag for errors from c_rtcoef	*/
	status,			/* function return status		*/
	line,samp,		/* line/samp at center & corners	*/
	ulline, ulsamp,		/* upper left corner of chip		*/
	geo_code,               /* projection code                      */
        geo_zone,               /* projection zone                      */
        geo_units,              /* projection units                     */
        ddr_units,              /* projection units from image ddr      */
        prtprm[2],              /* Print parameter for c_transinit      */
	ls_corners[8];       	/* image corner line,samp coordinates 	*/

char    cname[80];              /* current chip name                    */

double 	lat,lon;		/* lat/long at center & corners		*/
double	proj_coord[2];		/* corner pts in projection coordinates	*/
double 	coef[6];      		/* transformation coefficients 		*/
double	geo_parms[15];          /* geo projection parameters		*/

/*
  Set up projection information
*/
geo_code = GEO;
geo_zone = NULZONE;
geo_units = DEGREES;
status = c_ckunit(ddr->proj_units,&ddr_units);
if (status != E_SUCC)
    {
    c_errmsg("Invalid projection information in DDR","dcwchips-proj",
    	&nonfatal);
    return(E_FAIL);
    }
for (i = 0; i < 15; i++)
    geo_parms[i] = 0.0;

/*
  Get coefficients for converting from line/samp to proj coord
*/
ls_corners[0] = ls_corners[1] = ls_corners[3] = ls_corners[4] = 1;
ls_corners[2] = ls_corners[6] = ddr->nl;
ls_corners[5] = ls_corners[7] = ddr->ns;
c_rtcoef(ddr->upleft, ls_corners, coef, &flag);

/*
  Process each control point
*/
while (TRUE)
    {
    status = c_ptio(tps_desc, &tps_data);
    if (status == E_EOF) break;
    if (status == E_FAIL)
	{
	c_errmsg("Error reading tie point file","dcwchips-read",&nonfatal);
	return(status);
	}

    /*
      Get the name of this chip
    */
    if (strlen(tps_data.chip_name) != 0)
        {
        status = c_mkimg(tps_data.chip_name,"",taeout,chip_name,&flag);
        if (status != E_SUCC)
            return(status);
        }
    else
        {
        sprintf(cname,"chip%d",chip_no);
        status = c_mkimg(cname,"",taeout,chip_name,&flag);
        if (status != E_SUCC)
            return(status);
        }

    /*
      Calculate nominal coordinates in input image
    */
    lat = tps_data.coord[0];
    lon = (tps_data.coord[1] < 0.0) ? tps_data.coord[1] + 360.0:
                                      tps_data.coord[1];
    if (cal_lin_smp(ddr,&geo_code,&geo_units,&ddr_units,&geo_zone,geo_parms,
	&lat,&lon,&line,&samp) != E_SUCC)
	{
	c_errmsg("Error calculating line/samp","dcwchips-calc",&nonfatal);
	return(E_FAIL);
	}

    /*
      Find latitude, longitude extent of chip
    */
    if ((line > BORDERWIDTH) && (line < ddr->nl-BORDERWIDTH) &&
	(samp > BORDERWIDTH) && (samp < ddr->ns-BORDERWIDTH))
	{
	/* 
	  Save into to mtp file
	*/
        mtp_data->geo_coord[0] = tps_data.coord[0];
        mtp_data->geo_coord[1] = tps_data.coord[1];
        mtp_data->elevation    = tps_data.elevation;
        mtp_data->zoom_factor  = tps_data.zoom_factor;
        mtp_data->nominal[0]   = line;
        mtp_data->nominal[1]   = samp;
        mtp_data->ref_coord[0] = CHIPSIZE/2.0 + (line - (long)line);
        mtp_data->ref_coord[1] = CHIPSIZE/2.0 + (samp - (long)samp);
        mtp_data->chip_size[0] = CHIPSIZE;
        mtp_data->chip_size[1] = CHIPSIZE;
        mtp_data->offsets[0]   = CHIPSIZE/4;
        mtp_data->offsets[1]   = CHIPSIZE/4;
        strcpy(mtp_data->pt_id,tps_data.pt_id);
        strcpy(mtp_data->ref_chip,taeout);
        strcpy(mtp_data->sea_chip,"");

	ulline = (long)line - CHIP_OFF;
	ulsamp = (long)samp - CHIP_OFF;

	/*
	  For each corner:
	  Get the proj coordinates of the line/samp 
	  Use proj to convert the proj coord to lat/long 
	*/
	line = ulline;
	samp = ulsamp;
	proj_coord[0] = (coef[0] * line) + (coef[1] * samp) + coef[2];
	proj_coord[1] = (coef[3] * line) + (coef[4] * samp) + coef[5];
	c_transinit(&ddr->proj_code,&ddr_units,&ddr->zone_code,
               &ddr->datum_code,ddr->proj_coef, &geo_code,&geo_units,&geo_zone,
               &ddr->datum_code,geo_parms,prtprm,"\0");
	c_trans(&ddr->proj_code,&ddr_units, &geo_code,&geo_units,
	       &proj_coord[1],&proj_coord[0],&lon,&lat);
	window[SL] = line;
	window[SS] = samp;
	corners[MINLAT] = lat;
	corners[MAXLAT] = lat;
	corners[MINLON] = lon;
	corners[MAXLON] = lon;

	samp = ulsamp + CHIPSIZE - 1;
	proj_coord[0] = (coef[0] * line) + (coef[1] * samp) + coef[2];
	proj_coord[1] = (coef[3] * line) + (coef[4] * samp) + coef[5];
	c_trans(&ddr->proj_code,&ddr_units, &geo_code,&geo_units,
               &proj_coord[1],&proj_coord[0],&lon,&lat);
	if (lat < corners[MINLAT]) corners[MINLAT] = lat;
	if (lat > corners[MAXLAT]) corners[MAXLAT] = lat;
	if (lon < corners[MINLON]) corners[MINLON] = lon;
	if (lon > corners[MAXLON]) corners[MAXLON] = lon;

	line = ulline + CHIPSIZE - 1;
	proj_coord[0] = (coef[0] * line) + (coef[1] * samp) + coef[2];
	proj_coord[1] = (coef[3] * line) + (coef[4] * samp) + coef[5];
	c_trans(&ddr->proj_code,&ddr_units, &geo_code,&geo_units,
                 &proj_coord[1],&proj_coord[0],&lon,&lat);
	if (lat < corners[MINLAT]) corners[MINLAT] = lat;
	if (lat > corners[MAXLAT]) corners[MAXLAT] = lat;
	if (lon < corners[MINLON]) corners[MINLON] = lon;
	if (lon > corners[MAXLON]) corners[MAXLON] = lon;

	samp = ulsamp;
	proj_coord[0] = (coef[0] * line) + (coef[1] * samp) + coef[2];
	proj_coord[1] = (coef[3] * line) + (coef[4] * samp) + coef[5];
	c_trans(&ddr->proj_code,&ddr_units, &geo_code,&geo_units,
                &proj_coord[1], &proj_coord[0],&lon,&lat);
	if (lat < corners[MINLAT]) corners[MINLAT] = lat;
	if (lat > corners[MAXLAT]) corners[MAXLAT] = lat;
	if (lon < corners[MINLON]) corners[MINLON] = lon;
	if (lon > corners[MAXLON]) corners[MAXLON] = lon;

	/*
	  Keep results--pad min/max lat/longs to make sure we get all the area
	*/
	window[NL] = CHIPSIZE;
	window[NS] = CHIPSIZE;

	corners[MINLON] = (corners[MINLON] > 360.0) ? 
			   corners[MINLON] - 360.0 : 
			   corners[MINLON];
	corners[MAXLON] = (corners[MAXLON] > 360.0) ? 
			   corners[MAXLON] - 360.0 : 
			   corners[MAXLON];

	/*
	  When a good one is found, we can return
	*/
	break;
	}

    }

return(status);
}
