/*******************************************************************************
NAME:		GET_SAT_CHIP

PURPOSE:	Get one valid chip from the tps file and calculate its 
		line,sample and lat,long extents for an image in satellite 
		projection

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "dcw.h"
#include "geompak.h"
#include "dcwchips.h"

FUNCTION get_sat_chip(tps_desc,mtp_data,model,ddr,elev,
		      window,corners,chip_name,taeout,chip_no)

int	**tps_desc;		/* tps file descriptor			     */

struct 	MTPDATA *mtp_data;      /* search tps data record */
struct  MODEL   *model;   	/* AVHRR model                              */
struct  DDR   	*ddr;   	/* AVHRR image's ddr                        */

long	window[4],		/* window specification for each chip	     */
	chip_no;		/* number of current chip		     */

double	corners[4],		/* lat/long range for each chip		     */
	*elev;			/* elevation at center of chip		     */

char	*chip_name;		/* name of output chip image		     */
char	*taeout;		/* tae names of output chip images	     */
{
struct TPSDATA tps_data;        /* Search tps data record */

char 	cname[80];		/* current directory			     */

long	nonfatal = 1,		/* non-fatal error flag 		     */
	flag = 1,		/* check image existence flag		     */
	status,			/* function return status		     */
	ulline, ulsamp;		/* upper left corner of chip 		     */

float 	line,samp;		/* line/samp of center based on model	     */
float 	lasline,lassamp;	/* line/samp of center based on master l/s   */
float 	leftsamp,rightsamp;	/* left & right sample limits for selection  */

double 	lat,lon;		/* lat/long of center and each corner	     */

/*
  Set up limits
*/
leftsamp = EXCLUDE * model->nsamples;
rightsamp = model->nsamples - (EXCLUDE * model->nsamples);

/*
  Find a valid control point
*/
while (TRUE)
    {
    status = c_ptio(tps_desc,&tps_data);
    if (status == E_EOF) break;
    if (status == E_FAIL)
	{
	c_errmsg("Error reading tie point file","dcwchips-read",&nonfatal);
	break;
	}

    /*
      Get the name of this chip
    */
    if (strlen(tps_data.chip_name) != 0)
	{
        status = c_mkimg(tps_data.chip_name,"",taeout,chip_name,&flag);
	if (status != E_SUCC)
	    return(status);
	}
    else
        {
        sprintf(cname,"chip%d",chip_no);
        status = c_mkimg(cname,"",taeout,chip_name,&flag);
	if (status != E_SUCC)
	    return(status);
        }

    /*
      Calculate nominal coordinates in raw AVHRR image
    */
    lat = tps_data.coord[0] * RAD_PER_DEG;
    lon = (tps_data.coord[1] < 0.0) ? (tps_data.coord[1] + 360.0) * RAD_PER_DEG:
                                       tps_data.coord[1] * RAD_PER_DEG;
    *elev = tps_data.elevation;
    c_ll2ls(&lat,&lon,elev,model,&line,&samp,NULL);

    /*
      The model reports line/samp based on the original image specified by
      the model structure.  If the LAS image we are working with has been
      windowed we need to calculate the actual line/samp in the windowed image.
    */
    lasline = line - ddr->master_line;
    lassamp = samp - ddr->master_sample;

    /*
      See if it's a keeper
      Make sure the LAS line/sample is within the image and the
      AVHRR sample is within the defined extents
    */
    if ((lasline > BORDERWIDTH) && (lasline < (ddr->nl-BORDERWIDTH)) && 
	(lassamp > BORDERWIDTH) && (lassamp < (ddr->ns-BORDERWIDTH)) &&
	(samp > leftsamp) && (samp < rightsamp))
	{

	/*
	  Write results to mtp file
	  The output MTP file needs line/sample coords of the LAS image
	  for proper windowing
	*/
	mtp_data->geo_coord[0] = tps_data.coord[0];
	mtp_data->geo_coord[1] = tps_data.coord[1];
	mtp_data->elevation    = tps_data.elevation;
	mtp_data->zoom_factor  = tps_data.zoom_factor;
	mtp_data->nominal[0]   = lasline;
	mtp_data->nominal[1]   = lassamp;
	mtp_data->ref_coord[0] = CHIPSIZE/2.0 + (lasline - (long)lasline);
	mtp_data->ref_coord[1] = CHIPSIZE/2.0 + (lassamp - (long)lassamp);
	mtp_data->chip_size[0] = CHIPSIZE;
	mtp_data->chip_size[1] = CHIPSIZE;
	mtp_data->offsets[0]   = CHIPSIZE/4;
	mtp_data->offsets[1]   = CHIPSIZE/4;
	strcpy(mtp_data->pt_id,tps_data.pt_id);
	strcpy(mtp_data->ref_chip,taeout);
	strcpy(mtp_data->sea_chip,"");

	/*
          Find latitude, longitude extent of chip
	  Use the original model line/samp
	*/
	ulline = (long)line - CHIP_OFF;
	ulsamp = (long)samp - CHIP_OFF;

	line = ulline;
	samp = ulsamp;
    	c_ls2ll(&line,&samp,elev,model,&lat,&lon,NULL);
	window[SL] = lasline - CHIP_OFF;
	window[SS] = lassamp - CHIP_OFF;
	corners[MINLAT] = lat;
	corners[MAXLAT] = lat;
	corners[MINLON] = lon;
	corners[MAXLON] = lon;

	samp = ulsamp + CHIPSIZE - 1;
    	c_ls2ll(&line,&samp,elev,model,&lat,&lon,NULL);
	if (lat < corners[MINLAT]) corners[MINLAT] = lat;
	if (lat > corners[MAXLAT]) corners[MAXLAT] = lat;
	if (lon < corners[MINLON]) corners[MINLON] = lon;
	if (lon > corners[MAXLON]) corners[MAXLON] = lon;

	line = ulline + CHIPSIZE - 1;
    	c_ls2ll(&line,&samp,elev,model,&lat,&lon,NULL);
	if (lat < corners[MINLAT]) corners[MINLAT] = lat;
	if (lat > corners[MAXLAT]) corners[MAXLAT] = lat;
	if (lon < corners[MINLON]) corners[MINLON] = lon;
	if (lon > corners[MAXLON]) corners[MAXLON] = lon;

	samp = ulsamp;
    	c_ls2ll(&line,&samp,elev,model,&lat,&lon,NULL);
	if (lat < corners[MINLAT]) corners[MINLAT] = lat;
	if (lat > corners[MAXLAT]) corners[MAXLAT] = lat;
	if (lon < corners[MINLON]) corners[MINLON] = lon;
	if (lon > corners[MAXLON]) corners[MAXLON] = lon;

	/*
	  Keep results
	*/
	window[NL] = CHIPSIZE;
	window[NS] = CHIPSIZE;

	corners[MINLAT] *= DEG_PER_RAD;
	corners[MAXLAT] *= DEG_PER_RAD;
	corners[MINLON] = (corners[MINLON] > PIE) ? 
			  (corners[MINLON] * DEG_PER_RAD) - 360.0 : 
			   corners[MINLON] * DEG_PER_RAD;
	corners[MAXLON] = (corners[MAXLON] > PIE) ? 
			  (corners[MAXLON] * DEG_PER_RAD) - 360.0 : 
			   corners[MAXLON] * DEG_PER_RAD;

	/*
	  After one valid one is found, we can return
	*/
	break;
	}
    }

return(status);
}
