/*******************************************************************************
NAME:		SCANI

PURPOSE:	Find the line/sample coordinates to create a segment between
		the two given endpoints

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"

FUNCTION scani(chipbuf,l1,s1,l2,s2,window,pixval)

float	*chipbuf;	/* buffer of chip pixels 			*/

long	l1,		/* first y coordinate				*/
	s1,		/* first x coordinate				*/
	l2,		/* second y coordinate				*/
	s2;		/* second x coordinate				*/

long	window[],	/* window to process				*/
	pixval;		/* pixel value for the line			*/ 
{
float	slope;		/* inverse of the slope of the line segment	*/
float	xrasti;		/* current sample				*/

long	yhi,		/* high point line value of line segment	*/
	ylo,		/* low point line value of the line segment	*/
	xhi,		/* high point sample value of line segment	*/
	xlo;		/* low point sample value of line segment	*/

long	rastmax,	/* maximum sample value				*/
	rastmin,	/* minimum sample value				*/
	scanmax,	/* maximum scan line value			*/
	scanmin,	/* minimum line value				*/
	xstart,		/* first pixel to turn on			*/
	xstop,		/* last pixel to turn on			*/
	xscani,		/* current sample				*/
	yscani,		/* current line					*/
	temp;		/* temp variable				*/


/*
  Determine the orientation of the line segment
  a smaller line number is further north (higher)
*/
if (l1 < l2)
    {
    /* the line falls */
    yhi = l1;
    ylo = l2;
    xhi = s1;
    xlo = s2;
    }
else
    {
    /* the line rises */
    ylo = l1;
    yhi = l2;
    xlo = s1;
    xhi = s2;
    }

/*
  Calculate the inverse slope of the line segment
*/
if (yhi != ylo)
    {
    if (xhi > xlo)
        slope = (float)( xhi - xlo + 1);
    else if (xhi < xlo)
        slope = (float)( xhi - xlo - 1);
    else
        slope = 0.0;
    if (yhi > ylo)
        slope /= (float)( yhi - ylo + 1);
    else
        slope /= (float)( yhi - ylo - 1);
    }
else
    slope = 0.0;

/*
  Set start/stop points acording to the line weight
*/
scanmin = yhi;
scanmax = ylo;

if (xhi > xlo)
    {
    rastmax = xhi;
    rastmin = xlo;
    }
else
    {
    rastmax = xlo;
    rastmin = xhi;
    }

/*
  Validate the range of scan lines to be scanned.  Line segments
  that lie entirely above or below the image are ignored.  Line
  segments that lie partially above or below the image are included.
*/
if ((scanmax < window[SL]) || (scanmin > (window[SL] + window[NL] -1)))
    return;

/*
  Validate the range of pixels to be scanned.  Line segments
  that lie entirely to the left or right of the image are
  ignored.  Line segments that lie partially to the left or
  right of the image are included.
*/
if (((rastmax < window[SS]) && (rastmin < window[SS])) ||
    ((rastmax > (window[SS] + window[NS] - 1)) &&
     (rastmin > (window[SS] + window[NS] - 1))))
    return;

/*
  Scan the segments that intersect only 1 scan line
*/
if (slope == 0.0)
    {
    xstart = rastmin;
    xstop  = rastmax;

    /*
      Check the bounds
    */
    if (scanmin < window[SL])
	scanmin = window[SL];

    if (scanmax > (window[SL] + window[NL] - 1))
	scanmax =  window[SL] + window[NL] - 1;

    if (xstart < window[SS])
	xstart = window[SS];

    if (xstop > (window[SS] + window[NS] - 1))
	xstop =  window[SS] + window[NS] - 1;

    /*
      Write the scan line segment to the scan file
    */
    for (yscani = scanmin; yscani <= scanmax; yscani++)
	{
	for (xscani = xstart; xscani <= xstop; xscani++)
	    chipbuf[(yscani-window[SL])*window[NS]+xscani-window[SS]] = pixval;
	}
    }
else
    {
    /*
      Set the initial line and sample for the line segment
    */
    yscani = scanmin;
    xrasti = (float)xhi;

    /*
      Find each scan line intersected by this line segment
    */
    do
	{
	/*
	  Make sure the scan line is within the window
	*/
	if (yscani <= (window[SL] + window[NL] - 1))
	    {
	    /*
	      Determine the scan line segment limits with negative slope
	    */
	    if (slope < 0.0)
		    {
		    xstart = (long)(ceil(xrasti));
		    if (slope <= -1.0)
		        xstop  = (long)(xrasti + slope) + 1;
		    else
		        xstop  = (long)(ceil(xrasti));

		    /*
		      Check the bounds
		    */
		    if (xstart < rastmin)
			xstart = rastmin;
		    if (xstop > rastmax)
			xstop = rastmax;
		    if (xstop < rastmin)
			xstop = rastmin;
		    if (xstart > rastmax)
			xstart = rastmax;
		    }
	    else
		    {
		    /*
		      Determine the scan line segment limits with positive
		      slope
		    */
		    xstart = (long)xrasti;
		    if (slope >= 1.0)
		        xstop  = (long)(xrasti + slope) - 1;
		    else
		        xstop  = (long)xrasti;

		    /*
		      Check the bounds
		    */
		    if (xstop > rastmax)
			xstop = rastmax;
		    if (xstart < rastmin)
			xstart = rastmin;
		    if (xstart > rastmax)
			xstart = rastmax;
		    if (xstop < rastmin)
			xstop = rastmin;
		    }

	    /*
	      Determine if the scan line segment intersects
	      the image window
	    */
	    if (yscani >= window[SL])
		{
		/*
		  Order the samples
		*/
		if (xstart > xstop)
		    {
		    temp = xstart;
		    xstart = xstop;
		    xstop = temp;
		    }

		if ((xstart <= (window[SS] + window[NS] - 1)) &&
		    (xstop > 0))
		    {
		    /*
		      Check the bounds
		    */
		    if (xstart < window[SS])
		        xstart = window[SS];

		    if (xstop > (window[SS] + window[NS] - 1))
		        xstop = window[SS] + window[NS] - 1;

		    /*
      		      Save the rasterized pixels in the chip buffer
		    */
		    for (xscani = xstart; xscani <= xstop; xscani++)
	    	       chipbuf[(yscani-window[SL])*window[NS]+xscani-window[SS]]
			    = pixval;
		    }
		}

	    /*
	      Next sample
	    */
	    xrasti += slope;
	    }

	/*
	  Next line
	*/
	yscani++;
	}
    while (yscani <= scanmax);
    }

return;
}
