#!/bin/csh -f
#
# FUNCTION: esacquire
#
# PURPOSE:  Acquire ESA orbits from tape to disk.
#
#
# Version  Programmer       Date   Change Request
# -------  ---------------  -----  ---------------------------------------------
#   1.0    T.Baltzer        04/96  Original development (modified TPACQUIRE)

#
# COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
#   cshell script called from ESACQUIRE.PDF or from command line if drive is
#   allocated.
#
#
# NOTES:
#   1. When using remotely mounted tapes the user must have an account on the
#      remote machine and they must have a .rhosts file on that machine.
#
#   2. Shell metacharacters which are not quoted are interpreted on local
#      machine, while quoted metacharacters are interpreted on the remote
#      machine.  Thus the command
#
#         rsh otherhost cat remotefile >> localfile
#
#      appends the remote file remotefile to the localfile localfile, while
#
#         rsh otherhost cat remotefile ">>" otherremotefile
#
#      appends remotefile to otherremotefile.
#
#   3. To redirect standard output and standard error to separate files, use
#      (cmd > file1) >& file2.  On the DG, there is a bug in the csh which does
#      not allow this double redirection.
#
#   4. The dd, bsf, fsf junk at the end is to try to check for end of volume.
#      We didn't just capture the status of the dd that reads the image because
#      this may indicate a problem with the tape or drive within the image
#      file.  Each tape/system/station combination has special cases:
#
#      DG: Use fsr to check for EOV.  Use fsr instead of dd because dd gets
#          hung up at EOV.
#
#      Other systems: Use dd to check for EOV
#
#      8mm/9-track: When at EOV a dd creates a 0-size file.  We must do an
#          extra fsf to get to the correct side of the FMK for further dd's to
#          fail.
#
#      3480: fsf, dd.  dd fails at EOV for 3480 
#
#   5. The ESA tapes are composed of tar files.  A tar archive is a
#      stream of 512-byte header structures which may be followed by file data
#      in 512-byte blocks.  We set the block size to 10240, so when checking
#      for EOV, we fsr or dd ahead 1 record, which will be a 10240-byte header
#      in the tar file.
#
#   6. The variable free_bytes designates how many free bytes of disk space
#      must be available before esacquire will read an image(s) from tape.
#      This value is currently much higher than the typical mbytes required for
#      an ESA orbit because we cannot fill up a disk with acquired images,
#      leaving us no disk space for processing the images.
#
#
#
#


#---------------------------
# On interrupt go to CLEANUP
#---------------------------
onintr CLEANUP
set noglob


#------------------------------
# Check the number of arguments
#------------------------------
if ($#argv < 9) goto USAGE


#---------------------
# Set up the variables
#---------------------
set tape_id  = `echo $1 | awk '{printf "%-6s",$1}' | tr "[A-Z]" "[a-z]"`
set drive_no = $2
set ceos_id  = `echo $3 | tr "[A-Z]" "[a-z]"`
set destdir  = $4


#---------------------------------------------------------
# Set up aliases for checkspace, get_device, and write_log
#---------------------------------------------------------
if ("$5" == ".") then
   alias checkspace `pwd`/checkspace
else
   alias checkspace $5/checkspace
endif
if ("$6" == ".") then
   alias get_device `pwd`/get_device
else
   alias get_device $6/get_device
endif
if ("$7" == ".") then
   alias wrtlog `pwd`/write_log
else
   alias wrtlog $7/write_log
endif


#--------------------------------------------------------------------------
# Determine if the user is specifying a range or list of image numbers.  In
# either case, set up the variable $images which is a list of all the image
# numbers to be read from tape.
#--------------------------------------------------------------------------
set cmd = `echo $8 | tr "[A-Z]" "[a-z]"`
if ("$cmd" == "range") then
   set i = ($9)
   if (($#argv != 10) || ($#i != 1)) goto USAGE
   set images = ""
   while ($i <= "$10")
      set images = ($images $i)
      @ i += 1
   end
else if ("$cmd" == "select") then
   set i = 0
   while ($i < 8)
      @ i += 1
      shift
   end
   set images = ($*)
else
   goto USAGE
endif
unset cmd


#---------------------------------------------------------------
# Check the length of the CEOS ID.  Note that wc -c
# reports one more character than the length of a string.
#---------------------------------------------------------------
@ len = `echo $ceos_id | wc -c` - 1
if ($len != 3) then
   echo "[esacquire-len] CEOSID must be three characters"
   exit 1
endif


#---------------------------------------------------------------------------
# Unalias all commands used in this shell script to get rid of user aliases.
# If there is a /usr/bsd/rsh then use this command instead of the /bin/rsh
# because the Berkley version actually works with remote systems while the
# /bin/rsh version does not.  If there isn't a /usr/bsd/rsh then /bin/rsh
# usually works.
#---------------------------------------------------------------------------
unalias cd
unalias dd
unalias mt
unalias rm
unalias rsh
if (-e /usr/bsd/rsh) alias rsh /usr/bsd/rsh


#--------------------------------------------------------------------
# Set up the processing variables.  $logfile must be a LAS file name.
#--------------------------------------------------------------------
set disk_space    = "NO"
set eov_reached   = "NO"
set free_bytes    = 500000000
set logfile       = `echo "[$ADAPSLOG]esacquire" | tr / . | sed  's/\[\./\[/'`
set remote_system = "NO"


#------------------------------------------------------------------------------
# Get the device name of the drive and set the TAPE environment variable for mt
#------------------------------------------------------------------------------
set drive_name = `get_device $drive_no | awk '{print $1}'`
setenv TAPE $drive_name


#-------------------------------------------------------------
# Determine if we are using a drive on a remote system.  This
# is indicated by having a drive number of the form edcsgs4@4.
#-------------------------------------------------------------
set system_name = `uname -n`
set drive_num = `echo $drive_no | awk -F@ '{print $2}'`
if ($drive_num != "") then
   set system_name = `echo $drive_no | awk -F@ '{print $1}'`
   set remote_system = "YES"
endif


#------------------------------------------------------------------------------
# Create an alias for the dd and mt commands if using a remote system.
#
# NOTE: Silicon Graphics systems use a -t option for the mt command while all
#       other systems us a -f option.
#------------------------------------------------------------------------------
if ($remote_system == "YES") then
   #-------------
   # Alias for mt
   #-------------
   if (("$system_name" =~ sg*) || ("$system_name" =~ edcsg*)) then
      alias mt rsh $system_name mt -t $drive_name
   else
      alias mt rsh $system_name mt -f $drive_name
   endif

   #-------------
   # Alias for dd
   #-------------
   alias dd rsh $system_name dd
endif


#-----------------------------------------------------------------------------
# Set up the tape block size.
#-----------------------------------------------------------------------------
set blksz = 10240


#-------------------------------------
#	Skip over the first tape header file
#-------------------------------------
if (! { mt fsf >>& /dev/null }) then
    set text = "Error forward skipping file"
    wrtlog $logfile ${tape_id}-0-${ceos_id} esacq-error "$text"
    echo "[esacquire-fsf] $text"
    exit 1
endif


#---------------------------------------------------
# cd to the destination directory because you cannot
# specify the destination for the output of tar.
#---------------------------------------------------
cd $destdir


#----------------------------------------------------------------------
# If on remote system, reset name of the tape drive to use.  tar allows
# remote tape drives by using the syntax: [user@]system:/dev/....
#----------------------------------------------------------------------
if (($remote_system == "YES") && ($system_name !~ edcsg*) && \
    ($system_name !~ sg*)) then
    set text = "Remote system not allowed for tar"
    wrtlog $logfile ${tape_id}-0-${ceos_id} esacq-error "$text"
    echo "[esacquire-remote] $text"
    exit 1
else if ($remote_system == "YES") then
    set drive_name = "${system_name}:$drive_name"
endif


#---------------------------
# Log the start of esacquire
#---------------------------
set text = "Starting the ESACQUIRE procedure"
wrtlog $logfile ${tape_id}-0-${ceos_id} esacq-start "$text"
echo "[informational] $text"


#--------------------------
# Loop until EOV is reached
#--------------------------
set file_no     = 1
set images_read = 0
while (($eov_reached == "NO") && ($#images > 0))
   #----------------------------------------
   # Get the first image number from $images
   #----------------------------------------
   set image = $images[1]


   #------------------------------------------------
   # If we want to read this image, check disk space
   #------------------------------------------------
   if ($file_no == $image) then
      set i = 0
      while ($disk_space == "NO")
	 @ i += 1
	 if ({ checkspace "" $free_bytes $destdir }) then
	    set disk_space = "YES"
	 else if ($i < 61) then
	    sleep 60
	 else
	    set i = 1
	    echo "[informational] ESACQUIRE waiting for disk space"
	    sleep 60
         endif
      end
      
      #-------------------------------
      # Issue an informational message
      #-------------------------------
      if ($i > 1) then
	 echo "[informational] Disk space available.  Continuing"
      endif
   endif


   #-------------------
   # Process ESA orbit
   #-------------------
   if ($file_no == $image) then
      #--------------------------------------------------------------------
      # Capture the output of tar to get the name of the image file.  On an
      # interrupt continue on to the next tar file.  An alternate tar
      # command for remotely accessing a drive on a Sun
      # `rsh -n $system_name dd if=/dev/rmt0 bs=20b | tar xvBfb - 20`
      #--------------------------------------------------------------------
      onintr  CONTIN
      set tar_stat = `tar -xvf $drive_name |& awk '{print}'`

      onintr  CLEANUP
      if ($tar_stat[1] != "x") goto READERR

      set tar_name=`echo $tar_stat | awk '{print $2}' | awk -F, '{print $1}'`
      if (! { mt fsf >>& /dev/null }) goto FSFERR
      goto CONTIN2

      #-------------------------------------------
      # If tar asks for another tape the user must
      # interrupt the shell script with <ctrl-c>.
      #-------------------------------------------
      CONTIN:
      onintr CLEANUP
      echo ""
      echo -n "Interrupt <ctrl-c> within 10 seconds to abort"
      sleep 10
      set text = "Partial tar file was file $file_no"
      wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} esacq-error $text
      echo "[esacquire-partial] $text"
      set disk_space = "NO"
      @ file_no     += 1
      @ images_read += 1
      shift images
      if ($#images == 0) break
      continue

      CONTIN2:
   else if (! { mt fsf >>& /dev/null }) then
      goto FSFERR
   endif
 

   #-----------------------------------------------------------------
   # Check to see if the output file has anything in it.  A file size
   # of zero indicates that we hit another EOF which implies EOV.
   #-----------------------------------------------------------------
   if ($file_no == $image) then
      if (! -e $tar_name) break
      if (-z $tar_name) then
	 rm -f $tar_name
	 break
      endif


      #------------------------------------------------------
      # Create a new name for the output file and rename it.
      # This signifies that the file has been fully copied.
      # Document the file renaming for recovery purposes.
      #------------------------------------------------------
      set t = "$tar_name.done"
      mv $tar_name $destdir/$t
      set sceneid=`echo $tar_name | awk -F. '{print $1}'`
      set text = "File $file_no from $tape_id is scene $sceneid"
      wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} esacq-renam "$text"
      echo "[informational] $text"


      #--------------------------------
      # Issue a message to the log file
      #--------------------------------
      set text = "Successfully acquired file $file_no from $tape_id"
      wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} esacq-success "$text"
      echo "[informational] $text"


      #-----------------------------------
      # Shift the image numbers in $images
      #-----------------------------------
      @ images_read += 1
      shift images
   endif


   #--------------------------------
   # Update the processing variables
   #--------------------------------
   set disk_space = "NO"
   @ file_no += 1
   if ($#images == 0) break


   #-----------------------------------------
   # Read ahead to see if we have reached EOV
   #-----------------------------------------
   if ($system_name =~ dg*) then
      #-----------------------------------------------------------
      # Hand csh the command so we don't see the ugly error at EOV
      #-----------------------------------------------------------
      if ({ csh -c mt fsr >>& /dev/null }) then
	 #------------------------------------------------
	 # If ok then back up to the beginning of the file
	 #------------------------------------------------
	 mt bsr 1 >>& /dev/null
      else
	 #----------------------------
	 # dg fsr returns error at EOV
	 #----------------------------
	 set eov_reached = "YES"
      endif
   else
      if ({ (dd if=$drive_name bs=$blksz count=1 > _temp1) >>& \
	 /dev/null }) then
	 #-----------------------------------------------------------
	 # If check was successful, see if we actually read something
	 #-----------------------------------------------------------
	 if (-e _temp1) then
	    if (-z _temp1) then
	       set eov_reached = "YES"
            endif
	    rm -f _temp1
	 else
	    set eov_reached = "YES"
         endif

         #--------------------------------------------------------------------
	 # If all is ok, back up to the beginning of the file.  We must bsf
	 # once to back up before the filemark and then fsf to position at the
	 # beginning of the file we dd'd.
         #--------------------------------------------------------------------
	 mt bsf >>& /dev/null
	 mt fsf >>& /dev/null
      else
	 if (-e _temp1) rm -f _temp1
	 set eov_reached = "YES"
      endif
   endif
end


#-----------------------------------------------------------
# Issue a message stating how many files were read from tape
#-----------------------------------------------------------
set text = "Read $images_read images from drive $drive_no"
wrtlog $logfile ${tape_id}-0-${ceos_id} esacq-drive "$text"
echo "[informational] $text"
exit 0



#---------------
# User interrupt
#---------------
CLEANUP:
   onintr -
   set text = "Read $images_read images from drive $drive_no"
   wrtlog $logfile ${tape_id}-0-${ceos_id} esacq-drive "$text"
   echo "[informational] $text"

   set text = "User interrupt"
   wrtlog $logfile ${tape_id}-0-${ceos_id} esacq-usrint "$text"
   echo "[esacquire-user] $text"
   exit 1


#---------------------------
# Error forward spacing file
#---------------------------
FSFERR:
   set text = "Error forward skipping file"
   wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} esacq-error "$text"
   echo "[esacquire-fsf] $text"
   rm -f $tar_name
   exit 1


#--------------------
# Error reading image
#--------------------
READERR:
   set text = "Error reading image $file_no from tape"
   wrtlog $logfile ${tape_id}-${file_no}-${ceos_id} esacq-error "$text"
   echo "[esacquire-read] $text"
   rm -f $tar_name
   exit 1


#--------------------
# Usage error message
#--------------------
USAGE:
   echo ''
   echo 'USAGE: esacquire tape drive ceos dir cpath gpath wpath cmd ...'
   echo ''
   echo 'WHERE:'
   echo '   tape    = 6 character tape ID'
   echo '   drive   = Tape drive number'
   echo '   ceos    = 3 character CEOS ID'
   echo '   dir     = Host destination directory'
   echo '   cpath   = Path for the command checkspace'
   echo '   gpath   = Path for the command get_device'
   echo '   wpath   = Path for the command write_log'
   echo '   cmd     = Command type:'
   echo '                range  -> $10 = start image'
   echo '                          $11 = stop image'
   echo '                select -> $10, $11, $12, ... = images to read'
   echo ''
   echo ''
   echo 'EXAMPLES:'
   echo ''
   echo '   esacquire km1000 sg2@2 lsu /tmp ~/bin ~/bin ~/bin range 2 10'
   echo '      Read images 2 through 10 from tape km1000 which is mounted on'
   echo '      tape drive sg2@2.'
   echo ''
   echo '   esacquire km1000 2 lsu /tmp ~/bin ~/bin ~/bin select 2 5 9'
   echo '      Read images 2, 5, and 9 from tape km1000 which is mounted on'
   echo '      tape drive 2 of the current computer system.  It is assumed that'
   echo '      the list of images to read is sorted in ascending order, if not,'
   echo '      some of the images will not be read from tape.'
   echo ''
   exit 1

