/******************************************************************************
NAME:   			GEN_LUTS 

PURPOSE:    Generate a Look-Up Table to convert incorrectly calibrated
	    temperatures into correctly calibrated temperatures.
	    This Look-Up Table will actually correlate input pixel values to
	    output pixel values (not temperatures).


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   FIXCAL must be run under TAE.

PROJECT:			LAS

ALGORITHM:

   ALGORITHM REFERENCES:		none
******************************************************************************/

#include "las.h"
#include "adaps.h"
#include "fixcal.h"


FUNCTION gen_luts(vblock,odtype,channel,lut)

   struct PARBLK *vblock;	/* TAE parameter block			*/
   long	odtype;			/* data type of output image		*/
   long channel;		/* AVHRR data channel number		*/
   short lut[];			/* Look-up Table			*/

{

long i;				/* Counter				*/
long min_pixel_value;		/* Minimum valid pixel value for d type	*/
long max_pixel_value;		/* Maximum valid pixel value for d type	*/
long soption;			/* Scaling option as expected by c_getscale()*/
long status;			/* Return error status			*/
long zero = 0;			/* Value of zero			*/

double cwn[3];			/* Central wave numbers for temp 3	*/
double T1, T2;			/* Input, output temperatures		*/
double t_unscale;		/* Temperature unscale factor		*/

struct  SCALESTR t_scale; /* Scaling values for temperature            */


/* Determine the valid range of pixel values based on the data type
-----------------------------------------------------------------------------*/
if (odtype == EBYTE)
    {
    min_pixel_value = MIN_PIX_VALUE_US;
    max_pixel_value = MAX_PIX_VALUE_US;
    }
else
    {
    min_pixel_value = MIN_PIX_VALUE_1KM;
    max_pixel_value = MAX_PIX_VALUE_1KM;
    }

/* Load the scaling structure.  If byte data assume that it is US processing,
   otherwise assume it is i*2 Global 1KM processing.
-----------------------------------------------------------------------------*/
if (odtype == EBYTE)
    soption = US_BYTE;
else
    soption = AVHRR_10BIT;

status = c_getscale(soption,"thermal",&t_scale.minval,&t_scale.maxval,
                    &t_scale.scale,&t_unscale,&t_scale.offset);
if (status != E_SUCC)
       c_clean(vblock,"fixcal","",&zero);


/* Central wave number for Temperature range 3
-----------------------------------------------*/
cwn[0] = 2645.899 ;
cwn[1] = 929.3323 ;
cwn[2] = 845.12 ;


/* Preserve the "0" value and the mask values
-----------------------------------------------*/
for (i = 0; i < min_pixel_value; i++)
    lut[i] = i;


/* Generate a look-up value (output pixel) for each valid input pixel value.
   The index to the look-up table will coincide with the input pixel value.
   Only the appropriate range of values will be loaded into the array.
---------------------------------------------------------------------------*/
for (i = min_pixel_value; i <= max_pixel_value; i++)
    {
    /* Cast the input pixel value into a double */
    T1 = (double)i;

    /* Unscale the value */
    T1 = UNSCALE(T1,t_unscale,t_scale.offset);

    /* Get the appropriate new temperature value here. */
    noaa14Tfix(T1, cwn[channel-3], channel, &T2);

    /* Scale the value */
    T2 = SCALE(T2,t_scale.minval,t_scale.maxval,t_scale.scale,t_scale.offset);

    /* Round the value into the look-up table */
    if (T2 > 0.0)
	T2 += 0.5;
    else
	T2 -= 0.5;
    if (odtype == EBYTE)
	lut[i] = (unsigned char)T2;
    else if (odtype == EWORD)
	lut[i] = (short)T2;

    }

return;
}
