/*******************************************************************************
NAME 	fixtimes


PURPOSE  

If the starting time, ending time, or number of lines stored
within the AVHRR archive header file (.ahdr suffix) is inconsistant
with the actual archive image file (.arch suffix), modify the 
header file.  Also, if the NBS offset stored within the
AVHRR archive header file (.ahdr suffix) is inconsistant the
NBS offset contained within the on-line table, modify the
header file.

PROGRAM HISTORY
	PROGRAMMER	DATE		REASON
	---------- 	----		------
	T. Baltzer	10/93		Original development
	T. Baltzer	01/94		Check the NBS offset
	T. Baltzer	01/94		Subtract offset from first line time
					to determine header start time
	T. Baltzer	06/94		Revamp to always check .arch for
					start, end time and number of lines.
	T. Baltzer	07/94		Add writing to the badscenes log file.



PROJECT				ADAPS


*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "worgen.h"
#include "adaps.h"
#include "archio.h"

main()
{
struct MODEL	model;		/* Archive model information		     */
struct DROPPED  ahdr_drop;	/* Archive dropped line information	     */
struct PARBLK vblock;		/* TAE parameter block                       */

char	directory[CMLEN];	/* Directory portion of file name	     */
char	extension[CMLEN];	/* Extension portion of the file name	     */
char	filename[CMLEN];	/* ahdr file name                            */
char	fixflag[4];		/* Header fixed or not flag		     */
char	hostnm[CMLEN];		/* Host file name 			     */
char	hostay[1][CMLEN];	/* Host file name array			     */
char	key[17];		/* Scene id key 			     */
char	logfile[CMLEN];		/* Bad scenes log file name		     */
char	*charptr;		/* Pointer to char			     */
char	root[CMLEN];		/* Root portion of the file name	     */
char	startime[7];		/* Start time as defined in the header	     */

int	status;			/* return status			     */

short	*arcbuf;		/* Ptr to buffer of pixel data from AVHRR    */
short	*mnrbuf;		/* Ptr to Minor Frame buffer from c_arcread()*/

long	access = IUPDATE;	/* File access mode			     */
long	adesc;			/* Archive image descriptor record	     */
long	count = 0;		/* Number of files geneated by routine	     */
long	clockerr;		/* Clock error drift			     */
long	element;		/* Element to be retrieved		     */
long	end_time;		/* ending time of orbit in milliseconds	     */
long	etime;			/* Adjusted ending time in milliseconds	     */
long	fatal = -1;		/* fatal error status			     */
long	imgacc;			/* Access of image file			     */
long	len;			/* String length			     */
long	newlines;		/* Number of lines as defined by image size  */
long	nonfatal = 1;		/* non-fatal error status		     */
long	start_time;		/* starting time of orbit in milliseconds    */
long	std_out;		/* Standard output file			     */
long	stime;			/* Adjusted start time in milliseconds	     */
long	tpunit = NULL;		/* Image on disk not on tape		     */
long	window[4];		/* Window of image			     */

double	imgsize[1];		/* Size of image in bytes		     */

char   *ptr[1];			/* Array of pointers to send string to TAE   */

/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Initialize the FIXFLAG (did NOT fix the ahdr)
--------------------------------------------------*/
strcpy(fixflag,"no");


/* Get the input file name.
----------------------------*/
element = 0;
if (c_getnam(&vblock, "IN", &element, filename, "ahdr", &access) != E_SUCC)
    c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);

c_splitname(filename, directory, root, extension);

if (strcmp(root,"") == 0)
   {
   c_errmsg("Invalid file name specification","fixtimes-file",&nonfatal);
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);
   }

if ((strcmp(extension,"ahdr") != 0) && (strcmp(extension,"") != 0))
   {
   c_errmsg("Invalid extension specification","fixtimes-ext",&nonfatal);
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);
   }

strcpy(hostnm,directory);
strcat(hostnm,root);
strcat(hostnm,".arch");


/* Load the structure with values from the AVHRR image header file
------------------------------------------------------------------*/
status = c_getahdr(filename,&model,&ahdr_drop);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);


/* Compute the start time, end time in milliseconds based on
   the information in the header.
-------------------------------------------------------------*/
start_time = (((((model.shour * 60) + model.smin) * 60)
                  + model.ssec) * MIL_PER_SEC) + model.sfrac;

end_time =   (((((model.ehour * 60) + model.emin) * 60)
                  + model.esec) * MIL_PER_SEC) + model.efrac;


/* --------------------------------------------------------------
   Get the NBS offset from the time corrections table.  If the
   offset from the table is not the same as the header then
   we must adjust the offset in the header.  The assumption
   is that the offset currently retrieved is better than the
   one stored in the header.
---------------------------------------------------------------*/
status = c_acqtmcor(&model.satnum,&model.month,&model.day,&model.year,
		    &model.shour,&model.smin,&model.ssec,&model.sfrac,
		    &clockerr);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);

if (clockerr != model.clock_err)
   {
   c_errmsg("NBS offset in header file being updated",
	    "fixtimes-nbs",&nonfatal);
   model.clock_err = clockerr;
   strcpy(fixflag,"yes");
   }


/*------------------------------------------------------------
   Determine the image start time using the time stamp found
   on the first line of the archive image as well as the NBS
   offset just found in the table.  This start time is more
   reliable than the one stored in the header file.
   Conseqently, if there is a discrepancy, the header file
   will be updated.
------------------------------------------------------------*/

/* Open the AVHRR archive file
------------------------------*/
imgacc = IREAD;
window[0] = window[1] = window[2] = window[3] = 0;
status = c_arcopnrd(hostnm,&imgacc,window,&model,&tpunit,&adesc);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);

/* Read the first line in the archive file
-------------------------------------------*/
status = c_arcread(&adesc,&mnrbuf,&arcbuf);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);

/* Close the AVHRR archive file
------------------------------*/
status = c_arclose(hostnm,&adesc);
if (status != E_SUCC)
   {
   c_errmsg("Error closing archive image","fixtimes-clos",&nonfatal);
   c_errmsg("Non-fatal error encountered","fixtimes-nonf",&nonfatal);
   }

/* Determine the time of the first line, Don't worry
   about the date (assume it will be not be changed)
-----------------------------------------------------*/
stime = mnrbuf[9] & 0177;
stime = (stime << 10) + (mnrbuf[10] & 01777);
stime = (stime << 10) + (mnrbuf[11] & 01777);

/* Subtract the clock error just retrieved from the
   table from the time on the first line to determine
   the correct start time.
-----------------------------------------------------*/
stime -= clockerr;

/* Check to see if this start time from the archive file
   differs from the start time in the origonal header.
   If it does, update the header to contain the new
   start time.
----------------------------------------------------- */
if (stime != start_time)
   {
   c_errmsg("Starting time in header file being updated",
	    "fixtimes-sta",&nonfatal);
   c_cvtmil(&stime,&model.shour,&model.smin,&model.ssec,
            &model.sfrac);
   strcpy(fixflag,"yes");
   if (abs(start_time - stime) > 7000)
      {
      c_errmsg("Starting time in header file off significantly",
	       "fixtimes-toff",&nonfatal);
      charptr = (char *)getenv("ADAPSLOG");
      strcpy(logfile,charptr);
      strcat(logfile,"/badscenes.log");
      len = min(strlen(root),16);
      strncpy(key,root,len);
      key[len] = '\0';
      c_wrtlog(logfile,key,"fixtimes-toff",
	       "Starting time in header file off significantly");
      }
   }


/* -----------------------------------------------------
   Now determine the correct number of lines in the 
   image by looking at the size of the image file.  If
   this number of lines disagrees with the number of 
   lines stored in the header file, the header file
   will be modified.
-----------------------------------------------------*/

/* Determine the size of the image file
----------------------------------------*/
strcpy(hostay[0],hostnm);
c_getsizes(hostay,1,imgsize);
if ((long)imgsize[0] == -1)
   {
   c_errmsg("Error determining image size","fixtimes-size",&nonfatal);
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);
   }

/* Define the number of lines based on the file size
-------------------------------------------------------*/
if (model.dtype == GAC)
   newlines = (long)imgsize[0] / GAC_BYTES;
else
   newlines = (long)imgsize[0] / LAC_BYTES;

/* Update the header file if a discrepancy exists
--------------------------------------------------*/
if (model.nlines != newlines)
   {
   c_errmsg("Line count in header file being updated",
	    "fixtimes-nln",&nonfatal);
   model.nlines = newlines;
   strcpy(fixflag,"yes");
   }


/* -------------------------------------------------------
   Recompute the ending time based on the new start time
   and the new number of lines.  If a discrepancy exists
   with the header file, update the header file.
----------------------------------------------------------*/

/* Compute the ending time 
------------------------------*/
if (model.dtype == GAC)
   etime = stime + (newlines / GAC_PER_SEC * MIL_PER_SEC);
else
   etime = stime + (newlines / LAC_PER_SEC * MIL_PER_SEC);
if (etime > MIL_PER_DAY)
   etime = etime - MIL_PER_DAY;

/* If the end time as just computed differs from the
   end time stored in the header, replace the one
   found in the header.
------------------------------------------------------*/
if (end_time != etime)
   {
   c_errmsg("Ending time in header file being updated",
	    "fixtimes-etm",&nonfatal);
   c_cvtmil(&etime,&model.ehour,&model.emin,&model.esec,
            &model.efrac);
   strcpy(fixflag,"yes");
   if (abs(end_time - etime) > 7000)
      {
      c_errmsg("Ending time in header file off significantly",
	       "fixtimes-toff",&nonfatal);
      charptr = (char *)getenv("ADAPSLOG");
      strcpy(logfile,charptr);
      strcat(logfile,"/badscenes.log");
      len = min(strlen(root),16);
      strncpy(key,root,len);
      key[len] = '\0';
      c_wrtlog(logfile,key,"fixtimes-toff",
	       "Ending time in header file off significantly");
      }
   }


/* ------------------------------------------------------------
   If anything within the model structure changed, the header
   file will need to be updated.
--------------------------------------------------------------*/
if (strcmp(fixflag,"yes") == 0)
   {

   /* Calculate the new corner lat/longs, orbit number, etc.
   ----------------------------------------------------------*/
   status = c_initmod(&model.satnum,&model.year,&model.month,&model.day,
		      &model.shour,&model.smin,&model.ssec,&model.sfrac,
		      &model.dtype,&model.nlines,&model);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);

   /* Write the corrected structure values back out to the header file
   -------------------------------------------------------------------*/
   status = c_putahdr(filename,&model,&ahdr_drop);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);

   }

/* Send the FIXFLAG back to tae land
--------------------------------------*/
ptr[0] = fixflag;
if (q_string(&vblock,"FIXFLAG",1,ptr,P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","fixtimes-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);
   }

/* Send the start time in the header file back to TAE land.
   This is done whether it was just modified or not.
-------------------------------------------------------------*/
sprintf(startime,"%02d%02d%02d",model.shour,model.smin,model.ssec);
ptr[0] = startime;
if (q_string(&vblock,"STARTIME",1,ptr,P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","fixtimes-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);
   }

if (q_out(&vblock) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","fixtimes-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","fixtimes-fatal",&fatal);
   }


/*
  Exit with a success status
*/
c_complt("FIXTIMES","",&count);
}
