/*******************************************************************************
FUNCTION:	process

PURPOSE:   	Process the image.

*******************************************************************************/
#include <stdio.h>
#include "acqio.h"
#include "gapexp.h"

extern long curent;		/* Current processing line                    */
extern long indesc;             /* OUT - AVHRR descriptor for input image     */
extern long outdesc;            /* OUT - AVHRR descriptor for output image    */


FUNCTION process (drop, model, minlines)

    struct DROPPED *drop;	/* IN     - AVHRR dropped line structure      */
    struct MODEL *model;	/* IN     - AVHRR model structure             */
    long minlines;		/* IN     - Minimum line gap to be expanded   */
{

long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long k;			/* Loop control                                       */
long nonfatal =  1;	/* Nonfatal error flag                                */
long status;		/* Function return status                             */
long within_a_gap;	/* Flag specifying if within a drop line gap	      */

short *img;		/* Ptr to buffer of image data                        */
short *mnr;		/* Ptr to buffer of Minor Frame data                  */
short *mnrzero;		/* Minor frame for missing lines		      */
short *zero;		/* Buffer of zero'd BIP image data		      */

char *drop_line;	/* Ptr to drop line array			      */


/* Initialize an array with one element for each line of the input image.  This
   array will be used to indicate if a line is dropped or bad.  A value of zero
   indicates a good line while a value of 1 indicates a dropped line.  Calloc()
   will set all elements to zero.
------------------------------------------------------------------------------*/
drop_line = (char *)calloc(model->nlines , sizeof(char));
if (drop_line == NULL)
    {
    c_errmsg("Error allocating memory", "gapexp-alloc", &nonfatal);
    errout();
    }


/* Starting from the header data, expand each gap by 2 lines in both
   directions, if the gap is MINLINES lines or larger.  Make sure to not go
   off the image boundaries.
------------------------------------------------------------------------------*/
for (i = 0; i < drop->ngaps; i++)
    {
    if (drop->nmissed[i] >= minlines)
	{
	drop->lmissed[i] -= 2;
	if (drop->lmissed[i] < 1)
	    drop->lmissed[i] = 1;
	drop->nmissed[i] += 4;
	if ((drop->lmissed[i] + drop->nmissed[i] - 1) > model->nlines)
	    drop->nmissed[i] = model->nlines - drop->lmissed[i] + 1;
	}
    }


/* Populate the drop line array using the expanded drop line gaps.
------------------------------------------------------------------------------*/
for (i = 0; i < drop->ngaps; i++)
    {
    j = drop->lmissed[i] + drop->nmissed[i] - 1;
    for (k = drop->lmissed[i] - 1; k < j; k++)
	drop_line[k] = BAD;
    }


/* Allocate a buffer of zero'd BIP data.  This buffer will be used so that the
   actual image data does not have to be zero'd.
------------------------------------------------------------------------------*/
zero = (short *)calloc(model->nsamples * NCHANS, sizeof(short));
if (zero == NULL)
    {
    c_errmsg("Error allocating memory", "gapexp-alloc", &nonfatal);
    errout();
    }


/* Allocate a buffer of zero'd minor frame data.  This buffer will be used so
   that the actual minor frame data does not have to be zero'd.
------------------------------------------------------------------------------*/
mnrzero = (short *)calloc(MAX_ACQ, sizeof(short));
if (mnrzero == NULL)
    {
    c_errmsg("Error allocating memory", "gapexp-alloc", &nonfatal);
    errout();
    }


/* Read and write the entire image.
------------------------------------------------------------------------------*/
curent = 0;
for (i = 0; i < model->nlines; i++)
    {
    /* Read the image data from the input file
    ------------------------------------------*/
    status = c_arcread(&indesc, &mnr, &img);
    if (status != E_SUCC)
	errout();

    /* Write the input image line or a drop line
    ------------------------------------------*/
    if (drop_line[i] == BAD)
	{
	status = c_arcwrite(&outdesc, mnrzero, zero);
	if (status != E_SUCC)
	    errout();
	}
    else
	{
	status = c_arcwrite(&outdesc, mnr, img);
	if (status != E_SUCC)
	    errout();
	}

    curent++;
    }


/* Redefine the drop line structure, taking into account the expanded gap
   segments.  Use the drop line array to redefine the drop line structure
   since the expansion of the gaps may have caused 2 gaps to become 1 gap,
   etc.
------------------------------------------------------------------------------*/
drop->ngaps = 0;
within_a_gap = FALSE;
for (i = 0; i <model->nlines; i++)
    {
    if (drop_line[i] == BAD)
	{
	if (within_a_gap)
	    {
	    (drop->nmissed[drop->ngaps - 1])++;
	    }
	else
	    {
	    (drop->ngaps)++;
	    drop->lmissed[drop->ngaps - 1] = i + 1;
	    drop->nmissed[drop->ngaps - 1] = 1;
	    within_a_gap = TRUE;
	    }
	}
    else
	within_a_gap = FALSE;
    }


/* Change the total number of missed lines in the model
------------------------------------------------------------------------------*/
model->totmiss = 0;
for (i = 0; i < drop->ngaps; i++)
    {
    model->totmiss += drop->nmissed[i];
    }


/* Free allocated memory
------------------------------------------------------------------------------*/
free(drop_line);
free(zero);
free(mnrzero);


return(E_SUCC);
}
