/*******************************************************************************
NAME:		LS2LL

PURPOSE:	Convert a line/sample to a lat/long for a given pixel in a
		Level-0 image.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    11/97  T. Baltzer       Development

         6/99   K. Hoback        Changed Line/sample to use real values.

                                 Added capability to read Line/Sample values
			         from an input file.

			         Added a print oarameter to control 
                                 destination of output.

PROJECT:        ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "adaps.h"
#include "archio.h"


main()
{

long access,               /* File access flag			*/
     element = 0,	   /* Constant value			*/
     fatal = -1,           /* fatal error message flag          */
     flag = FALSE,         /* flag for valid data points        */
     n_outfiles  = 0,	   /* Number of output files		*/
     status,		   /* Return status			*/
     std_out,              /* LUN to standard output		*/
     zero = 0;		   /* Constant zero                     */

long lnpr;                 /* flag for output to lnpr           */
long file;                 /* flag for output to file           */
long term;                 /* flag for output to terminal       */

float   fline,		   /* Floating point version of line	*/
	fsample;	   /* Floating point version of sample	*/

double  dlat,dlng,	   /* Double version of lat,long	*/
	delev = 0.0;	   /* Elevation value			*/

char infil[CMLEN],         /* Input ascii LINE-SAMPLE file      */
     hosin[1][CMLEN];      /* Host names of input images	*/
char filnm[CMLEN];         /* User specified filename           */
char tmpfil[CMLEN];        /* temporary file name for fileutil  */


struct PARBLK  vblock;	   /* TAE parameter block		*/

struct MODEL   model;	   /* AVHRR model information		*/
struct DROPPED  drop;      /* dropped line data structure	*/

struct VARIABLE *p_find(); /* Function to find a TAE variable	*/
struct VARIABLE *val;	   /* Pointer to a TAE variable within v-block  */

FILE *fp_out,              /* Output file descriptor            */
     *fp_in;        	   /* LINE-SAMPLE ascii file descriptor */


/* Initialize LAS
----------------------------------------------------------------------------*/
c_inlas(&vblock, &std_out);

/* get print option, make a file name if necessary, open temp file
------------------------------------------------------------------*/
strcpy(tmpfil,"ls2ll");
status = c_setprt(&vblock,&term,&file,&lnpr,tmpfil,filnm);
if (status != E_SUCC)
   c_lasmsg(TRUE,"ls2ll-fatal","Fatal error encountered");
if (file == 1)
   {
   n_outfiles = 1;
   }
else
   {
   strcpy(filnm," ");
   n_outfiles = 0;
   }

/* Get the name of the input image
----------------------------------------------------------------------------*/
access = IREAD;
status = c_getnam(&vblock, "IN", &element, hosin, "arch", &access);
if (status != E_SUCC)
    c_lasmsg(TRUE,"ls2ll-fatal","Fatal error encountered");


/* Load the AVHRR model from the header file
----------------------------------------------------------------------------*/
status = c_getahdr(hosin, &model, &drop);
if (status != E_SUCC)
   c_lasmsg(TRUE,"ls2ll-fatal","Fatal error encountered");


/* Open output file, Print header to file. 
----------------------------------------------------------------------------*/
fp_out = fopen(tmpfil, "w");     /* Open the LINE-SAMPLE LAT/LONGITUDE file */
if (fp_out == NULL)
   c_lasmsg(TRUE,"ls2ll-open", "Error opening tmpfil file");

fprintf(fp_out,"LINE/SAMPLE                             LATITUDE/LONGITUDE\n");
fprintf(fp_out,"-----------------------------------------------------------------\n");


/* Check if INFILE is specified, ignore LINE and SAMPLE parameters if it is
----------------------------------------------------------------------------*/
access = IREAD;
status = c_getnam(&vblock,"INFILE",&zero,infil,"txt",&access);
if (status != E_SUCC)
   c_lasmsg(TRUE, "ls2ll-open", "Error opening infile file");
if (strlen(infil) != 0)
   {
   fp_in = fopen(infil, "r");	     /* Open the LINE-SAMPLE file */

   if (fp_in == NULL)
      c_lasmsg(TRUE, "ls2ll-open", "Error opening infile file");

   while((status = fscanf(fp_in,"%f %f",&fline,&fsample)) != EOF)
      {
	 if (status != 2)
	    {
	    c_lasmsg(TRUE, "ls2ll-pairs", 
                    "The LINE and SAMPLE values in INFILE must be in pairs");
	    }
         /* Convert the line/sample to a lat/long
         -------------------------------------------------------------------*/
         c_ls2ll(&fline,&fsample,&delev,&model,&dlat,&dlng,NULL);
         dlat = dlat * DEG_PER_RAD;
         dlng = dlng * DEG_PER_RAD;

         /* Print set of data to file.
         -------------------------------------------------------------------*/
         fprintf(fp_out,"(%13f, %13f)        (%11f, %11f) ",fline,fsample,
                  dlat,dlng);
         if((fline > model.nlines) || (fline < 1) || (fsample > model.nsamples)
                 || (fsample < 1))
            {
            flag = TRUE;
            fprintf(fp_out,"**");
            }
         fprintf(fp_out,"\n");
      }
    fclose(fp_in);
    }
else
   {
   /* Get the sample of interest
   -------------------------------------------------------------------------*/
   val = p_find(&vblock, "LINE");
   fline = RVAL(*val, 0);

   /* Get the sample of interest
   -------------------------------------------------------------------------*/
   val = p_find(&vblock, "SAMPLE");
   fsample = RVAL(*val, 0);

   /* Convert the line/sample to a lat/long
   -------------------------------------------------------------------------*/
   c_ls2ll(&fline,&fsample,&delev,&model,&dlat,&dlng,NULL);
   dlat = dlat * DEG_PER_RAD;
   dlng = dlng * DEG_PER_RAD;

   /* Print out the results
   -------------------------------------------------------------------------*/
   fprintf(fp_out,"(%13f, %13f)        (%11f, %11f) ",fline,fsample,dlat,dlng);
   if((fline > model.nlines) || (fline < 1) || (fsample > model.nsamples)
                 || (fsample < 1))
      {
      flag = TRUE;
      fprintf(fp_out,"**");
      }
   fprintf(fp_out,"\n");
   }

/* if flag set print out a warning message
---------------------------------------------------------------------------*/
if(flag)
   fprintf(fp_out,"\nNOTE: **   Means value outside specified image!\n");

/* Insert a new line at end of file and Close output file pointer.
---------------------------------------------------------------------------*/
fprintf(fp_out,"\n");
fclose(fp_out);

/* Output completion message
----------------------------*/
c_complt("LS2LL", filnm, &n_outfiles);
}
