/******************************************************************************
NAME:                           MULLIINFO 

PURPOSE:    Get the necessary information from the composite date log file.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      04/00          CSB           Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none

   FILE FORMAT REFERENCE:  For a sample input date band log file, see
			   the file 'us_232245.log' ('long' format) or
			   the file '1992p19.log' ('short' format)
			   stored under CVS as part of the MULLI module.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "mulliinfo.h"


int main(void)
    {


    long access;		/* Type of access for getnam call	*/
    long element = 0;		/* Element number for getnam call	*/

    int i;			/* counter                              */
    int file_format;		/* File format flag (long, short)	*/
    int header_lines;		/* Number of header lines in log file	*/
    int line_count;		/* Line counter				*/
    int ndays;			/* Number of days in composite		*/
    int nperiods;		/* Number of periods in 'periods'	*/
    int nscanned;		/* Number of items scanned from string	*/
    int sceneptlist[MAXSCENE];	/* List of scene pointers		*/
    int status;			/* Return status			*/

    long std_out = 0;		/* Standard output device		*/

    char sceneid[17];		/* 16 char scene id in line buffer      */
    char sceneidlist[MAXSCENE][17];/* List of scenes used in the composite */
    char filename[CMLEN];	/* Half orbit file name                 */
    char linebuf[CMLEN+1];      /* Buffer containing line of data       */
    char periods[MAXPERIOD][CMLEN];/* Periods used in the composite	*/
    char satid[3];		/* Satellite id number			*/

    struct VARIABLE *val;	/* pointer to a TAE variable		*/
    struct PARBLK vblock;	/* TAE parameter block			*/

    FILE *infp;                 /* LUT file pointer			*/


    /* Initialize LAS
    ---------------------------*/
    c_inlas(&vblock,&std_out);


    /* Initialize the variables
    ---------------------------*/
    ndays = 0;
    for (i = 0; i < MAXSCENE; i++)
	sceneptlist[i] = -1;


    /* Get the period name(s)
    --------------------------------------------*/
    val = p_find(&vblock,"PERIODS");
    nperiods = val->v_count;
    for (i = 0; i < nperiods; i++)
	strcpy(periods[i],SVAL(*val,i));


    /* Get the name of the file to read and make
    sure the file exists with read permission
    --------------------------------------------*/
    access = IREAD;
    status = c_getnam(&vblock,"DATELOG",&element,filename,"",&access);
    if (status != E_SUCC)
	c_lasmsg(TRUE,"mullii-fatal","Fatal error encountered");


    /* Open the input file
    -----------------------*/
    infp = fopen(filename,"r");
    if (infp == NULL)
	{
	c_lasmsg(FALSE,"mullii-open","Error opening input file %s",
		 filename);
	c_lasmsg(TRUE,"mullii-fatal","Fatal error encountered");
	}


    /* Read the header to determine if it is a 'long' or 'short'
       format log file.  Determine this using the third line
       in the file.  Then read past all of the header information.
    --------------------------------------------------------------*/
    for (line_count = 0; line_count < 3; line_count++)
	{
	if ( fgets(linebuf,CMLEN+1,infp) == NULL)
	    {
	    fclose(infp);
	    c_lasmsg(FALSE,"mullii-read","Error reading line %d in input file",
		     line_count+1);
	    c_lasmsg(TRUE,"mullii-fatal","Fatal error encountered");
	    }
	}

    if (strncmp(linebuf,"Scenei",6) == 0)
	{
	file_format = SHORT_FORMAT;
	header_lines = HEADER_LINES_SHORT;
	}
    else if (strncmp(linebuf,"Period",6) == 0)
	{
	file_format = LONG_FORMAT;
	header_lines = HEADER_LINES_LONG;
	}
    else
	{
	fclose(infp);
	c_lasmsg(FALSE,"mullii-form","Error determining log file format");
	c_lasmsg(TRUE,"mullii-fatal","Fatal error encountered");
	}

    for (      ; line_count <= header_lines; line_count++)
	{
	if ( fgets(linebuf,CMLEN+1,infp) == NULL)
	    {
	    fclose(infp);
	    c_lasmsg(FALSE,"mullii-read","Error reading line %d in input file",
		     line_count+1);
	    c_lasmsg(TRUE,"mullii-fatal","Fatal error encountered");
	    }
	}


    /* Get the satellite identifier number
    --------------------------------------------------*/
    if (file_format == LONG_FORMAT)
	nscanned = sscanf(linebuf,"%*s %*s %*s %s", sceneid);
    else
	nscanned = sscanf(linebuf,"%s",sceneid);
    if ( nscanned != 1 )
	c_lasmsg(TRUE,"mullii-scan","Error scanning satellite ID");
    strncpy(satid,&sceneid[2],2);
    satid[2] = '\0';


    /* Process one line of data.
       Read a line, if NULL is returned, assume the EOF has been
       encountered.  Don't print an error message.
    ---------------------------------------------------------------*/
    for (;;)
	{
	if (file_format == LONG_FORMAT)
	    process_long(linebuf,sceneptlist,sceneidlist,&ndays,periods,
			 nperiods);
	else
	    process_short(linebuf,sceneptlist,sceneidlist,&ndays);

	if ( fgets(linebuf,CMLEN+1,infp) == NULL)
	    break;
	}


    /* Close the date log file
    --------------------------------------*/
    if ( fclose(infp) != E_SUCC )
	{
	c_lasmsg(FALSE,"mullii-clos","Error closing input file %s",
		 filename);
	c_lasmsg(FALSE,"mullii-warn","Nonfatal error encountered");
	}


    /* Range check the 'ndays' parameter here
    ------------------------------------------*/
    if (ndays > MAXSCENE)
	{
	c_lasmsg(FALSE,"mullii-days","Maximum number of days (%d) exceeded",
		 ndays);
	c_lasmsg(FALSE,"mullii-fatal","Fatal error encountered");
	}


    /* Send the information back to TAE land
    --------------------------------------------*/
    taeland(&vblock,sceneptlist,sceneidlist,&ndays);


    /* Exit with a success status, no message
    ------------------------------------------*/
    z_exit(1,"");


    /* The return is placed here to make the compiler happy
    --------------------------------------------------------*/
    return (SUCCESS);
    }

