/******************************************************************************

NAME:   			PRE_PROCESS

PURPOSE:    Open, group the images; get ready to process.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T.Baltzer	   04/00	EDC/CSB		original development
  1.1      D.Lloyd	   02/01	Added code to update the output image
  					ddr with input image geometry.  The
					output image will have valid values
					if the input image does.  CR 4645


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   MULLI must be run under TAE.

PROJECT:			ADAPS

******************************************************************************/

#include "las.h"
#include "adaps.h"
#include "adapssup.h"
#include "util.h"
#include "mulli.h"

void pre_process
(
    struct PARBLK *vblock,	/* IN: TAE parameter block		     */
    struct FILEINFO  fileinfo[],/* IN: File information structure	     */
    char hosin[][CMLEN],	/* IN: host names of input images	     */
    char hosout[][CMLEN],	/* IN: host name of output image	     */
    int sceneptlist[MAXSCENE],		/* IN: Scene pointer listing	     */
    char sceneidlist[MAXSCENE][17],	/* IN: Scene listing	             */
    int nsceneidlist,			/* IN: Number in sceneidlist         */
    long msgtim,			/* IN: Time between messages         */
    char minmax[4] 			/* IN: Min max flags	             */
)

{

long	access,			/* image I/O access type                     */
	bufsiz,			/* size of image I/O buffer                  */
	dtype = EREAL,		/* input data type			     */
	nl,			/* number of image lines                     */
	ns,			/* number of image samples                   */
	offset1[MAXIMG*MAXBND],	/* offsets into thebuf for input bands	     */
	offset2[MAXBND],	/* offsets into "thebuf" for output bands    */
	opt[3],			/* option array				     */
	outbands[1],		/* Output number of bands		     */
	scale_option,		/* scaling option			     */
	sl,			/* starting image line                       */
	ss,			/* starting image sample                     */
	status,			/* function return status code               */
	one = 1,		/* value of one				     */
	zero = 0;		/* value of zero                             */
long	two = 2;		/* value of two				     */
long	window[1][4];		/* image window				     */
long	bands[1][MAXBND];	/* output image bands			     */
long	nbands;			/* number of output bands		     */
long	cflag = NOCOMB;		/* combine flag for upddr		     */

double	dbl_one = 1.0;		/* value of one				     */

int	band,			/* loop index counting the bands             */
	i,			/* Counter				     */
	image,			/* loop index counting input images          */
	save_nl,		/* Retain previous number of lines in image  */
	save_ns;		/* Retain previous number of samples in img  */

float	aus;			/* sun-earth distance correction factor	     */
float	aus2[MAXSCENE];		/* sun-earth distance squared (per scene)    */

char    *thebuf;		/* image I/O buffer                          */

struct DDR   ddr;		/* DDR structure 			     */
struct FDESC *fdesc[MAXIMG+1];	/* image file descriptors used for image I/O */
struct GDESC *gdesc;		/* group descriptor used for image I/O       */
struct MODEL model[MAXSCENE];	/* Array of AVHRR models		     */
struct SCOFF scoff;		/* scaling coefficient structure	     */

/* Open input images
----------------------*/
access = IREAD;
sl = 1;  /* no windowing allowed */
ss = 1;  /* no windowing allowed */
for (band=0,image=0;image<NUM_IN_IMG;image++)
   {
   nl = 0;  /* get number of lines from the ddr */
   ns = 0;  /* get number of samples from the ddr */
   status = c_eopens(&fdesc[image],hosin[image],fileinfo[image].bands,
		     &sl,&ss,&nl,&ns,&offset1[band],&access,&dtype,&zero,&zero);
   if (status != E_SUCC)
      c_clean(vblock,"mulli","",&zero);
   if (image == 0)
      {
      save_nl = nl;
      save_ns = ns;
      }
   else
      {
      if ((nl != save_nl) || (ns != save_ns))
	 {
	 c_lasmsg(FALSE,"mulli-imgs","Invalid sized input image specified");
	 c_clean(vblock,"mulli","",&zero);
	 }
      }
   band += fileinfo[image].nbands;
   }


/* Open output image
-----------------------*/

opt[0] = EBYTE;		/* Output data type equal input data type */
opt[1] = COMPRES;	/* Always compress the output image */
opt[2] = 0;		/* Standard option terminator	*/
access = IWRITE;	/* Write access for output image */
outbands[0] = 2;	/* Two output bands		*/

status = c_eopens(&fdesc[image],hosout[image-NUM_IN_IMG],outbands,
		  &sl,&ss,&nl,&ns,&offset2[0],&access,&dtype,&zero,opt);
if (status != E_SUCC)
    c_clean(vblock,"mulli",hosout,&one);

image++;
fdesc[image] = 0;

/* Update the output ddr with information from the input ddr CR 4645
--------------------------------------------------------------------*/
bands[0][0] = 1;
bands[0][1] = 2;
bands[0][2] = 0;
nbands = 2;
window[0][SL] = sl;
window[0][NL] = nl;
window[0][SS] = ss;
window[0][NS] = ns;
status = c_upddr( vblock, hosin, hosout, &one, window, bands, &nbands,
		&dbl_one, &dbl_one, &two, &cflag );
if ( status != E_SUCC )
    c_clean(vblock,"mulli",hosout,&one);


/* Allocate I/O buffer
----------------------*/
if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_lasmsg(FALSE,"mulli-alloc","Error allocating dynamic memory");
   c_clean(vblock,"mulli",hosout,&one);
   }


/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,(unsigned char *)thebuf,&bufsiz) != E_SUCC)
    {
    free(thebuf);
    c_clean(vblock,"mulli",hosout,&one);
    }


/* Load the scaling structure.   MULLI only processes US scaled composites.
   It is also assumed that these US composites were always calibrated to
   "reflectance".
--------------------------------------------------------------------------*/
c_getscaleopt("US",EBYTE,&scale_option);
status = c_getscale( scale_option, "reflectance", &scoff.minval,
                     &scoff.maxval, &scoff.scale, &scoff.unscale,
                     &scoff.offset );
if ( status != E_SUCC )
    {
    free(thebuf);
    c_egclse(&gdesc);
    c_clean(vblock,"mulli",hosout,&one);
    }


/* Load the AVHRR models for each acquisition used in the composite.
   Determine the earth-sun distance squared for each model.
-------------------------------------------------------------------*/
for (i = 0; i < nsceneidlist; i++)
    {
    if ( load_model(sceneidlist[i],&model[i]) != E_SUCC )
	{
	free(thebuf);
	c_egclse(&gdesc);
	c_clean(vblock,"mulli",hosout,&one);
	}

    sunearthdis(model[i].jday,&aus);
    aus2[i] = aus * aus;
    }


/* Get the image ddr
----------------------------------*/
status = c_getddr(hosin[0],&ddr);
if (status != E_SUCC)
    {
    free(thebuf);
    c_egclse(&gdesc);
    c_clean(vblock,"mulli",hosout,&one);
    }


/* Finally do the processing here!
----------------------------------*/
if ( process(hosout,fileinfo,&gdesc,offset1,offset2,thebuf,
	sceneptlist,model,msgtim,minmax,&scoff,&ddr,aus2) != E_SUCC)
    {
    free(thebuf);
    c_egclse(&gdesc);
    c_clean(vblock,"mulli",hosout,&one);
    }



/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
    c_lasmsg(FALSE,"mulli-warn","Nonfatal error encountered");


/* Update the BDDR's with the data source
------------------------------------------*/
if ( update_bdr(hosout) != E_SUCC)
    c_lasmsg(FALSE,"mulli-warn","Nonfatal error encountered");


/* Free buffer
--------------*/
free(thebuf);


}
