/******************************************************************************
NAME:   			PROCESS 

PURPOSE:    Process the data to generate the necessary output image

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         REASON
-------    ----------      ----         ------
  1.0      T.Baltzer      04/00         Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   MULLI must be run under TAE.

PROJECT:                        ADAPS

******************************************************************************/

#include <string.h>
#include <math.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "adaps.h"
#include "util.h"
#include "pixman.h"
#include "mulli.h"


#define   CH1_IN   0		/* AVHRR Channel 1 input index		*/
#define   CH2_IN   1		/* AVHRR Channel 2 input index		*/
#define DTBND_IN   2		/* Date band index			*/

#define CH1_OUT    0		/* AVHRR Channel 1 output index		*/
#define CH2_OUT    1		/* AVHRR Channel 2 output index		*/

#define UNDEFINED  -1		/* Value signifying an undefined value	*/

long curent;			/* Current line being processed		*/


int process
(
    char hosout[][CMLEN],		/* IN: host name of output image     */
    struct FILEINFO fileinfo[],		/* IN: File information structure    */
    struct GDESC **gdesc,		/* IN: group descriptor for image I/O */
    long offset1[MAXIMG*MAXBND],	/* IN: offsets for input bands	     */
    long offset2[MAXBND],   		/* IN: offsets for output bands	     */
    char *thebuf,			/* IN: image I/O buffer              */
    int sceneptlist[MAXSCENE],		/* IN: Scene pointer listing	     */
    struct MODEL model[MAXSCENE],	/* IN: Array of AVHRR models         */
    long msgtim,			/* IN: time between display messages */
    char *minmax,			/* IN: min max flag                  */
    struct SCOFF *scoff,		/* IN: Scaling coefficient structure */
    struct DDR *ddr, 			/* IN: Composite image DDR	     */
    float aus2[MAXSCENE] 		/* IN: sun-earth distance squared    */
)

{

double	cos_zenith,		/* Cosine of the solar zenith angle	     */
	elev = 0.0,		/* Elevation value			     */
	lat,			/* latitude for one pixel 		     */
	lon,			/* longitude for one pixel		     */
	minval[MAXBND],		/* min pixel value array		     */
	maxval[MAXBND],		/* max pixel value array		     */
	proj_x,			/* projection coordinate of a grid point     */
	proj_y,			/* projection coordinate of a grid point     */
	rot_coeffs[6],		/* rotation coefficients		     */
	view_angles[3];		/* Zenith and Azimuth angles		     */

float	ch1_index,		/* AVHRR channel 1 index		     */
	ch2_index,		/* AVHRR channel 2 index		     */
	line,			/* Line variable			     */
	nadir_samp = 1024.0,	/* Nadir sample value			     */
	*rthebuf,		/*  pointer to thebuf			     */
	samp;			/* Sample variable			     */

long	bands_index,		/* Date band index			     */
	degrees = DEGREE,	/* units code for projection (degrees)	     */
	geo = GEO,		/* geographic projection flag		     */
	sample,			/*  counter -- current pixel		     */
	mmflag = FALSE,		/* Min max flag				     */
	msg_flag=GEN2,		/* flag for processing message               */
	nl,			/* number of image lines                     */
	ns,			/* number of image samples                   */
	num_in_img = NUM_IN_IMG,/* Number of input images		     */
	num_out_bnd,		/* Number of output bands		     */
	*offset1_CH1_IN,	/* offset1 plus CH1_IN			     */
	*offset1_CH2_IN,	/* offset1 plus CH2_IN			     */
	*offset1_DTBND_IN,	/* offset1 plus DTBND_IN		     */
	*offset2_CH1_OUT,	/* offset2 plus CH1_OUT			     */
	*offset2_CH2_OUT,	/* offset2 plus CH2_OUT			     */
	proj_units,		/* Projection units value		     */
	rdtype = EREAL,		/* Floating point data type		     */
	undefined_db_max;	/* Undefined date band maximum value	     */


/* Assign the undefined date band values based on the type of composite:

   US:     0 = undefined 

   GLOBAL: 0 = missing over land
	   1 = pixel over ocean
	   2 = pixel in Goodes interupt area

   NOTE: Don't have to check for less than zero because values are coming from
	 a byte image.  Also, MULLI only processes US scaled images, so the
	 above global documentation is only informational.
-----------------------------------------------------------------------------*/
undefined_db_max = 0;


/* Setup parameters
---------------------------*/
nl = fileinfo[0].nlines;
ns = fileinfo[0].nsamples;
rthebuf = (float *) thebuf;

/* Initialize min/max arrays: mmflag is set to be used within
    the processing loop -- quicker than strcmp
-----------------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   mmflag = TRUE;	/* set flag			 */
   num_out_bnd = 2;	/* channel 1 and channel 2 band  */
   c_inmnmx(&rdtype,&num_out_bnd,&minval[0],&maxval[0]);
   }


/* Initialize the transformation information for the composite image
----------------------------------------------------------------------*/
if ( my_trans_init( ddr, &proj_units, rot_coeffs) != E_SUCC)
    return(E_FAIL);


/* Setup for processing message utility
---------------------------------------*/
curent = 1;    
c_inittm("MULLI","Processed","line","bands"," ",&nl,&num_in_img,&msgtim,
	 &msg_flag);
c_startm();

/* Main processing loop, let the processing begin
--------------------------------------------------*/
for (curent = 0; curent < nl; curent++)
   {
   /* Read / write a line of the image */
   if (c_estep(gdesc) != E_SUCC)
      return(E_FAIL);

   /* Assign the offsets for each channel, do this before going into
      the 'sample' loop (for the sake of efficiency).
   ----------------------------------------------------------------*/
   offset1_CH1_IN   = offset1 + CH1_IN;
   offset1_CH2_IN   = offset1 + CH2_IN;
   offset1_DTBND_IN = offset1 + DTBND_IN;
   offset2_CH1_OUT  = offset2 + CH1_OUT;
   offset2_CH2_OUT  = offset2 + CH2_OUT;
   
   for (sample = 0; sample < ns; sample++)
      {
      /* Get the value of each sample for each input band 
      ----------------------------------------------------*/
      ch1_index = *(rthebuf + (*offset1_CH1_IN + sample));
      ch2_index = *(rthebuf + (*offset1_CH2_IN + sample));
      bands_index = (long) (*(rthebuf + (*offset1_DTBND_IN + sample)));

      /* Make sure we are not pointing to an unspecified scene index   */
      if ((bands_index > undefined_db_max) &&
	  ((bands_index >= MAXSCENE) ||
	  (sceneptlist[bands_index] == -1)))
	 {
	 c_lasmsg(FALSE,"mulli-band","Invalid date band specification");
         return(E_FAIL);
	 }

      /* Assign a pixel value for each of the two output bands,
	 undefined or otherwise. 
      ----------------------------------------------------------------*/
      if (((long)ch1_index == UNDEFINED) || ((long)ch2_index == UNDEFINED))
	 {
	 *(rthebuf + (*offset2_CH1_OUT + sample)) = UNDEFINED;
	 *(rthebuf + (*offset2_CH2_OUT + sample)) = UNDEFINED;
	 }
      else if (bands_index <= undefined_db_max)
	 {
	 *(rthebuf + (*offset2_CH1_OUT + sample)) = (float)bands_index;
	 *(rthebuf + (*offset2_CH2_OUT + sample)) = (float)bands_index;
	 }
      else
	 {

        /* Determine the projection coordinates taking
         into account any rotation of the image.
         -----------------------------------------------*/
         proj_y = ( rot_coeffs[0] * (double)(curent + 1) ) +
                  ( rot_coeffs[1] * (double)(sample + 1) ) + rot_coeffs[2];
         proj_x = ( rot_coeffs[3] * (double)(curent + 1) ) +
                  ( rot_coeffs[4] * (double)(sample + 1) ) + rot_coeffs[5];

         /* Compute the latitude and logitude of the location.
         If c_trans does not return E_SUCC we assume we are
         in an invalid region such as an interupted Goode region.
         -----------------------------------------------------------*/
         if ( c_trans( &ddr->proj_code, &proj_units, &geo, &degrees,
                       &proj_x, &proj_y, &lon, &lat ) != E_SUCC )
	     {
	     *(rthebuf + (*offset2_CH1_OUT + sample)) = UNDEFINED;
	     *(rthebuf + (*offset2_CH2_OUT + sample)) = UNDEFINED;
	     }
	 else
	     {
	     /* Call c_ll2ls() to determine the line/sample within the
	        acquisition image.  First convert lat/lon to radians.
             ---------------------------------------------------------*/
	     lat *= RAD_PER_DEG;
	     lon *= RAD_PER_DEG;
	     c_ll2ls(&lat,&lon,&elev,&model[sceneptlist[bands_index]],
		     &line,&samp,view_angles);

	     /* Call c_ls2ll() to determine the solar zenith angle at the
	        nadir pixel of the line in question.
             ------------------------------------------------------*/
	     c_ls2ll(&line,&nadir_samp,&elev,&model[sceneptlist[bands_index]],
		     &lat,&lon,view_angles);

	     /* Get the cosine of the solar zenith angle.
		Restrict this value from getting too small,
		to reverse what was applied during calibration.
             ---------------------------------------------------*/
	     cos_zenith = cos(view_angles[1]);
	     if (cos_zenith < 0.1)
	         cos_zenith = 0.1;

	     /* Unscale the AVHRR Channel 1,2 values and
	        remove the effects of illumination correction.
	     -------------------------------------------------------------*/
	     ch1_index = UNSCALE(ch1_index,scoff->unscale,scoff->offset)
		         * cos_zenith / aus2[sceneptlist[bands_index]];
	     ch2_index = UNSCALE(ch2_index,scoff->unscale,scoff->offset)
		         * cos_zenith / aus2[sceneptlist[bands_index]];

	     /* Scale the AVHRR Channel 1,2 values and
	        assign the output values to the I/O buffer.
	     -------------------------------------------------------------*/
	     *(rthebuf + (*offset2_CH1_OUT + sample)) = 
	         SCALE(ch1_index,scoff->minval,scoff->maxval,
		       scoff->scale,scoff->offset);
	     *(rthebuf + (*offset2_CH2_OUT + sample)) =
	         SCALE(ch2_index,scoff->minval,scoff->maxval,
		       scoff->scale,scoff->offset);
	     }
	 }
      }

      /* Calculate the min/max values for the output bands, if specified 
       ----------------------------------------------------------------*/
      if (mmflag)
	 {
         c_pxmnmx((long *)(rthebuf + *offset2_CH1_OUT),&ns,
                  &minval[CH1_OUT],&maxval[CH1_OUT],&rdtype); 
         c_pxmnmx((long *)(rthebuf + *offset2_CH2_OUT),&ns,
                  &minval[CH2_OUT],&maxval[CH2_OUT],&rdtype); 
	 }
   }

/* Write the final line of output data
---------------------------------------*/
if (c_estep(gdesc) != E_SUCC)
    return(E_FAIL);

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Save the associated min/max values
------------------------------------------------------------------------*/
if (mmflag)
    {
    if (c_svmnmx(hosout[0],&num_out_bnd,&minval[0],&maxval[0],minmax) != E_SUCC)
	c_lasmsg(FALSE,"mulli-warn","Nonfatal error encountered");
    }

return(E_SUCC);
}
