/******************************************************************************
FUNCTION:	CALC_ATTITUDES

PURPOSE: 	This routine find the attitude and attitude rates given a 
		control point file.
		The roll, pitch, yaw, and delta_alt are recalculated.
*******************************************************************************/
#include <math.h>
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

int calc_attitudes(
	FILE	*fptr,			/* Output report		      */
	struct MODEL *model,		/* AVHRR model info from search image */
	struct DDR *ddr,		/* AVHRR image ddr		      */
	double dline[][PTS_PER_GROUP],	/* delta lines			      */
	double dsamp[][PTS_PER_GROUP],	/* delta samples		      */
	double refline[][PTS_PER_GROUP],/* reference lines		      */
	double refsamp[][PTS_PER_GROUP],/* reference samples		      */
	char id[][PTS_PER_GROUP][20],	/* buffer of tie point identifiers    */
	long used[][PTS_PER_GROUP],	/* flag if point was used	      */
	long npts,			/* # of points read from tpl file     */
	double linecoefs[],		/* Least Squares coeff's mapping lines*/
	double samplecoefs[],		/* Least Squares coeff's mapping samps*/
	long *msgtim,			/* processing message time	      */
	long ngroups,			/* number of point groups	      */
	long pts_in_group[] 		/* number of points in each group     */
)

{
double	zdot,			/* pass direction			*/
	satalt,			/* satellite altitude in meters		*/
	avhrrlines,		/* number of lines in AVHRR scene	*/
	totlines,		/* number of lines in image		*/
	totsamps,		/* number of samps in image		*/
	offline,		/* line offset				*/
	offsample,		/* sample offset			*/
	dalt = 0.0;		/* new altitude correction		*/

long	i,j,			/* looping variable			*/
	status;			/* function return status		*/


fprintf(fptr,"\n");

/* Initialize used flag
   --------------------*/
for (i = 0; i < ngroups; i++)
    for (j = 0; j < pts_in_group[i]; j++)
        used[i][j] = TRUE;

/* Initialize plane coefficients
   -----------------------------*/
for (i = 0; i < 3; i++)
    {
    linecoefs[i] = 0.0;
    samplecoefs[i] = 0.0;
    }

satalt = SAT_ALTITUDE * 1000.0;
zdot = (double)model->cpass;
totlines = ddr->nl;
totsamps = ddr->ns;
avhrrlines = model->nlines;
offline = -ddr->master_line;
offsample = -ddr->master_sample;

/* 8 or more points; least median of squares 
   fit;  First find the least squares
   This is done only in case lmsplane fails and
   we want to calculate coeffs anyway
   lsplane(npts,refline,refsamp,dline,linecoefs);
   lsplane(npts,refline,refsamp,dsamp,samplecoefs);
   ----------------------------------*/

/* Now do least median of squares
   ------------------------------*/
status = lmsplane(fptr,refline,refsamp,dline,dsamp,id,used,npts,
		  totlines,totsamps,linecoefs,samplecoefs,msgtim,
		  ngroups,pts_in_group);
if (status != E_SUCC)
    return(E_FAIL);

fprintf(fptr,"Final line coefficients   %lf %lf %lf\n",linecoefs[0],
	linecoefs[1],linecoefs[2]);
fprintf(fptr,"Final sample coefficients %lf %lf %lf\n",samplecoefs[0],
	samplecoefs[1],samplecoefs[2]);

/* Calculate roll, pitch, and yaw deltas which are added to
   the corresponding model coeffs, and an altitude delta 
   which is added to both delta_alt and alt_coefs in the model
   -----------------------------------------------------------*/
plane2attit(linecoefs,samplecoefs,zdot,model->dtype,satalt,avhrrlines,
	    offline,offsample,model->roll_coefs,model->pitch_coefs,
	    model->yaw_coefs,&dalt);

/* Add the dalt back in to complete 
   re-initialization of the model
   --------------------------------*/
model->delta_alt    += dalt;
model->alt_coefs[0] += dalt;

/* Report on the attitude correction coefficients
   ----------------------------------------------*/
fprintf(fptr,"Roll coefficients  %4.5lf %4.5lf %4.5lf %4.5lf %4.5lf\n",
	model->roll_coefs[0]*DEG_PER_RAD,model->roll_coefs[1]*DEG_PER_RAD,
	model->roll_coefs[2]*DEG_PER_RAD,model->roll_coefs[3]*DEG_PER_RAD,
	model->roll_coefs[4]*DEG_PER_RAD);
fprintf(fptr,"Pitch coefficients %4.5lf %4.5lf %4.5lf %4.5lf %4.5lf\n",
	model->pitch_coefs[0]*DEG_PER_RAD,model->pitch_coefs[1]*DEG_PER_RAD,
	model->pitch_coefs[2]*DEG_PER_RAD,model->pitch_coefs[3]*DEG_PER_RAD,
	model->pitch_coefs[4]*DEG_PER_RAD);
fprintf(fptr,"Yaw coefficients   %4.5lf %4.5lf %4.5lf %4.5lf %4.5lf\n",
	model->yaw_coefs[0]*DEG_PER_RAD,model->yaw_coefs[1]*DEG_PER_RAD,
	model->yaw_coefs[2]*DEG_PER_RAD,model->yaw_coefs[3]*DEG_PER_RAD,
	model->yaw_coefs[4]*DEG_PER_RAD);
fprintf(fptr,"Alt coefficients   %4.5lf %4.5lf %4.5lf %4.5lf %4.5lf\n",
	model->alt_coefs[0],model->alt_coefs[1],model->alt_coefs[2],
	model->alt_coefs[3],model->alt_coefs[4]);
fprintf(fptr,"Delta alt          %4.5lf\n",model->delta_alt);

return(E_SUCC);
}
