/*******************************************************************************
FUNCTION:       LMSPLANE

PURPOSE:        This program finds a plane through three data points which has
                the minimum value of the median of the squares of the residuals.
                (least median square fit (LMS))

                Output a plane of the form:
  deltaline=linecoefs[0]*line + linecoefs[1]*sample + linecoefs[2];
  deltasample=samplecoefs[0]*line + samplecoefs[1]*sample + samplecoefs[2];

ALGORITHM REFERENCES
 NOTE: points are divided into groups of at most PTS_PER_GROUP points
 For each group: find the best plane
   For each 5-point combination in the group:
     Find a least squares plane through 5 points.
     Find the squares of the residuals.
     Find the median.
     Is this value less than the last one?
      yes: keep it and find next plane,
      no: go on to next plane
 After the best least squares plane has been calculated for each group,
   calculate a new plane by averaging the planes of each group
 For each of these planes:
   Calculate residuals from the whole set of points
   Count the number of points that are <= MAXMEDIAN residual
 Choose the plane that agrees with the most points
 Calculate a plane using all the points that agree with the chosen set
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

int lmsplane(
	FILE *fptr,				/* output file pointer	      */
	double refline[][PTS_PER_GROUP],	/* reference line	      */
	double refsample[][PTS_PER_GROUP],	/* reference sample	      */
	double dline[][PTS_PER_GROUP],		/* delta line (search - ref)  */
	double dsample[][PTS_PER_GROUP],	/* delta sample	(search - ref)*/
	char id[][PTS_PER_GROUP][20],		/* buffer to hold tie point id*/
	long	used[][PTS_PER_GROUP],		/* flag if point was used     */
	long 	npoints,			/* # input points	      */
	double tlines,				/* total # lines in image     */
	double tsamples,			/* total # samples in image   */
	double linecoefs[],			/* final calc'd line coeffs   */
	double samplecoefs[],			/* final calc'd sample coeffs */
	long	*msgtim,			/* processing message time    */
	long	ngroups,			/* # of point groups	      */
	long	pts_in_group[] 			/* # of points in each group  */
)

{
extern long curent;             /* Current processing plane 		*/

double 	residuall[MAXPTS],	/* line residual			*/
	residuals[MAXPTS],	/* sample residual			*/
	residualt[MAXPTS],	/* total residual			*/
	key[MAXGRPS+1][MAXPTS],	/* key in sort (order of points)	*/
	coefl[5],		/* temporary line coeffs		*/
	coefs[5],		/* temporary sample coeffs		*/
	lcoefs[MAXGRPS+1][5],	/* group line coeffs			*/
	scoefs[MAXGRPS+1][5],	/* group sample coeffs			*/
	pline[5],		/* 5 lines used in each plane calc	*/
	psample[5],		/* 5 samples used in each plane calc	*/
	pdline[5],		/* 5 dlines used in each plane calc	*/
	pdsample[5],		/* 5 dsamples used in each plane calc	*/
	xline[MAXPTS],		/* lines that agree with the fit	*/
	xsample[MAXPTS],	/* samples that agree with the fit	*/
	xdline[MAXPTS],		/* dlines that agree with the fit	*/
	xdsample[MAXPTS],	/* dsamples that agree with the fit	*/
	median,			/* median residual			*/
	savemedian = 999999999.9,/* least median so far			*/
	disp[5],		/* displacements between the 5 points */
	vecl,			/* line distance between 2 of the 5 points */
	vecs,			/* sample distance between 2 of the 5 points*/
	mindisp,		/* minimum displacement bet 2 of the 5 points */
	minmindisp;		/* minimum allowed displacement bet 2 points */

long 	i, j, k, l, m, n, q,	/* loop counters 			*/
	ii, jj,			/* loop counters			*/
	grp,pt,			/* group,point loop counter		*/
	nn, o,			/* median indices			*/
	index,			/* index key in sorted array		*/
	over[MAXGRPS+1],	/* index of 1st point over maxmedian	*/
	even,			/* flag if even # points		*/
	nonfatal = 1,		/* nonfatal error code			*/
	nplane = 0,		/* # planes calculated 			*/
	p = 5,			/* # points to take at a time		*/
	maxmatch,		/* maximum # pts that worked in plane   */
	ncplane = 0,		/* # planes that pass constraints	*/
	planefound = FALSE,	/* found at least one passing plane	*/
	planeidx,		/* index of best plane			*/
	planes = 0,		/* # of groups with a successful plane	*/
	savepts[MAXGRPS][5];	/* points used for the best LMS fit	*/
long	msgfmt = GEN2,          /* general message format one           */
        max_entity = 0,      	/* maximum entity for processing message*/
	zero = 0;		/* constant 0 				*/
char	msg[CMLEN];
	

minmindisp = sqrt(tlines * tlines + tsamples * tsamples) / 10.0;

for (grp = 0; grp < ngroups; grp++)
    {
    /* Start the processing message
       ----------------------------*/
    curent = 1;
    max_entity = comb(pts_in_group[grp],5);
    sprintf(msg,"for group %d",grp+1);
    c_inittm("NAVIGATE","Calculated","plane"," ",msg,&max_entity,&zero,
	msgtim,&msgfmt);
    c_startm();
    
    nplane = ncplane = 0;
    planefound = FALSE;
    savemedian = 999999999.9;
    even = pts_in_group[grp] % 2;
    nn = pts_in_group[grp] / 2;
    o = nn + 1;
    
    /* Process the points in combinations of 5
       ---------------------------------------*/
    for (i = 0; i < (pts_in_group[grp] - 4); i++)
        {
        pline[0]    = refline[grp][i];
        psample[0]  = refsample[grp][i];
        pdline[0]   = dline[grp][i];
        pdsample[0] = dsample[grp][i];
    
        for (j = i + 1; j < (pts_in_group[grp] - 3); j++)
	    {
            pline[1]    = refline[grp][j];
            psample[1]  = refsample[grp][j];
            pdline[1]   = dline[grp][j];
            pdsample[1] = dsample[grp][j];
    
            for (k = j + 1; k < (pts_in_group[grp] - 2); k++)
                {
                pline[2]    = refline[grp][k];
                psample[2]  = refsample[grp][k];
                pdline[2]   = dline[grp][k];
                pdsample[2] = dsample[grp][k];
                   
                for (l = k + 1; l < (pts_in_group[grp] - 1); l++)
                    {
                    pline[3]    = refline[grp][l];
                    psample[3]  = refsample[grp][l];
                    pdline[3]   = dline[grp][l];
                    pdsample[3] = dsample[grp][l];
    
                    for (m = l + 1; m < pts_in_group[grp]; m++)
                        {
		        curent++;
                        nplane      = nplane + 1;
                        pline[4]    = refline[grp][m];
                        psample[4]  = refsample[grp][m];
                        pdline[4]   = dline[grp][m];
                        pdsample[4] = dsample[grp][m];
    
		        /* Calculate total displacements
		           -----------------------------*/
                        mindisp = 99999999.9;
                        for (ii = 0; ii < p; ii++)
                            {
                            disp[ii] = 0.0;
                            for (jj = 0; jj < p; jj++)
                                {
                                if (ii != jj)
                                    {
                                    vecl = pline[jj] - pline[ii];
                                    vecs = psample[jj] - psample[ii];
				    /* find the min dist between any 2 points */
                                    /* disp[ii] = sqrt(vecl*vecl+vecs*vecs); */
                         	    /* if (disp[ii] < mindisp)
                             	        mindisp = disp[ii]; */
                                    disp[ii] += sqrt(vecl*vecl+vecs*vecs);
                                    } 
                                }
                             if (disp[ii] < mindisp)
                                 mindisp = disp[ii];
                             }
    
		        /* Calculate plane for points i,j,k,l,m
		           ------------------------------------*/
		        lsplane(p,pline,psample,pdline,coefl);
                        lsplane(p,pline,psample,pdsample,coefs);
    
                        if (fabs(coefl[0]) <= MAXPITCHRATE && 
                            fabs(coefs[0]) <= MAXROLLRATE && 
                            fabs(coefl[1]) <= MAXYAW &&
                            fabs(coefs[1]) <= MAXALT &&
                            mindisp >= minmindisp)
			    {
			    /* Calculate residuals of points
			       in this group
			       -----------------------------*/
                     	    for (n = 0; n < pts_in_group[grp]; n++)
                                {
                                residuall[n] = coefl[0] * refline[grp][n] + 
                                               coefl[1] * refsample[grp][n] + 
					       coefl[2] - dline[grp][n];
                                residuals[n] = coefs[0] * refline[grp][n] + 
                                               coefs[1] * refsample[grp][n] + 
					       coefs[2] - dsample[grp][n];
                                }
                
			    /* Square the residuals
			       --------------------*/
			    for (n = 0; n < pts_in_group[grp]; n++) 
                                {
                                residuall[n] = residuall[n] * residuall[n];
                                residuals[n] = residuals[n] * residuals[n];
                                residualt[n] = residuals[n] + residuall[n];
                                }
    
			    /* Sort the squares of the residuals
			       ---------------------------------*/
                            for (q = 0; q < pts_in_group[grp]; q++)  
			        key[0][q] = (double)(q);
                            heapsort(pts_in_group[grp],residualt,key[0]);
    
			    /* Find the median value of the squares
			       ------------------------------------*/
                            if (even == 0) /* if even # pts, avg middle pts */
                                median = (residualt[nn] + residualt[o]) / 2.0; 
                            else
                                median = residualt[o];
    
			    /* Save the minimum median coefs for line & sample
			       -----------------------------------------------*/
                            if (residualt[pts_in_group[grp]-1] < MAXRESID)
                                {   
                                ncplane += 1;
                                if (median < savemedian)
                                    {
                                    planefound = TRUE;
                                    savemedian = median;
                                    lcoefs[planes][0] = coefl[0];
                                    lcoefs[planes][1] = coefl[1];
                                    lcoefs[planes][2] = coefl[2];
                                    scoefs[planes][0] = coefs[0];
                                    scoefs[planes][1] = coefs[1];
                                    scoefs[planes][2] = coefs[2];
                                    savepts[planes][0] = i;
                                    savepts[planes][1] = j;
                                    savepts[planes][2] = k;
                                    savepts[planes][3] = l;
                                    savepts[planes][4] = m;
                                    }
			        }
			    } /* end if coef.s check */
                        } /* m */
                    } /* l */
                } /* k */
	    } /* j */
        } /* i */
    
    curent--;
    c_stoptm();
    
    
    /* If no planes were found go to next group
       ----------------------------------------*/
    if (!planefound)
        {
        sprintf(msg,"Group %d does not pass constraints",grp+1);
        c_errmsg(msg,
	    "navigate-noplane",&nonfatal);
        continue;
        }

    fprintf(fptr,"For group %d, plane %d:\n",grp+1,planes+1);
    fprintf(fptr," Line coefs % f % f % f\n Samp coefs % f % f % f\n",
	lcoefs[planes][0],lcoefs[planes][1],lcoefs[planes][2],
	scoefs[planes][0],scoefs[planes][1],scoefs[planes][2]);
    fprintf(fptr," Number of possible planes was %d\n",nplane);
    fprintf(fptr," Number of planes that passed constraints was %d\n",ncplane);
    
    planes++;
    }

/* Add the average of all group coefs as a possible coef set
   if there is more than one
   ---------------------------------------------------------*/
if (planes > 1)
    {
    lcoefs[planes][0] = lcoefs[planes][1] = lcoefs[planes][2] = 0.0;
    scoefs[planes][0] = scoefs[planes][1] = scoefs[planes][2] = 0.0;
    for (i = 0; i < planes; i++)
	{
	lcoefs[planes][0] += lcoefs[i][0];
	lcoefs[planes][1] += lcoefs[i][1];
	lcoefs[planes][2] += lcoefs[i][2];
	scoefs[planes][0] += scoefs[i][0];
	scoefs[planes][1] += scoefs[i][1];
	scoefs[planes][2] += scoefs[i][2];
	}
    lcoefs[planes][0] /= planes;
    lcoefs[planes][1] /= planes;
    lcoefs[planes][2] /= planes;
    scoefs[planes][0] /= planes;
    scoefs[planes][1] /= planes;
    scoefs[planes][2] /= planes;
    fprintf(fptr,"For average, plane %d:\n",planes+1);
    fprintf(fptr," Line coefs % f % f % f\n Samp coefs % f % f % f\n",
	    lcoefs[planes][0],lcoefs[planes][1],lcoefs[planes][2],
	    scoefs[planes][0],scoefs[planes][1],scoefs[planes][2]);
    planes++;
    }


/* Calculate residual information for each plane
   from the whole set of points
   ---------------------------------------------*/
for (i = 0; i < planes; i++)
    {
    for (grp = 0, pt = 0, m = 0; m < npoints; m++)
        {
        residuall[m] = lcoefs[i][0] * refline[grp][pt] + 
		       lcoefs[i][1] * refsample[grp][pt] + 
		       lcoefs[i][2] - dline[grp][pt];
        residuals[m] = scoefs[i][0] * refline[grp][pt] + 
		       scoefs[i][1] * refsample[grp][pt] + 
		       scoefs[i][2] - dsample[grp][pt];
        residuall[m] = residuall[m] * residuall[m];
        residuals[m] = residuals[m] * residuals[m];
        residualt[m] = residuals[m] + residuall[m];
        key[i][m] = (double)(m);
        pt++;
        if (pt == pts_in_group[grp])
	    {
	    grp++;
	    pt = 0;
	    }
        }
        
    
    /* Sort the squares of the residuals
       ---------------------------------*/
    heapsort(npoints,residualt,key[i]);
    
    if (residualt[npoints / 2 + 1] > 1.0) 
        {
        sprintf(msg,"Warning: final median value is greater than 1 pixel");
        c_errmsg(msg,"navigate-warn",&nonfatal);
        }
    
    /* Gather the points that agree with the fit
       -----------------------------------------*/
    for (over[i] = 0, m = 0; m < npoints; m++) 
        {
        if (residualt[m] > MAXMEDIAN)
	    {
	    over[i] = m;
	    break;
	    } 
        } 
    fprintf(fptr,"Plane %d used %d points\n",i+1,over[i]);
    
    /* Make sure we use at least 60% of the pts
       ----------------------------------------*/
    if ((100.0*over[i]/npoints) < 60.0)
        {
        over[i] = 0;
	/* This just means the coeffs for this group won't be used
        sprintf(msg,"Final coefficients: too few points used in final fit");
        c_errmsg(msg,"navigate-toofew",&nonfatal);
	*/
        }
    }

/* Choose the plane with the most points agreeing (largest over)
   -------------------------------------------------------------*/
for (planeidx = -1, maxmatch = 0, i = 0; i < planes; i++)
    {
    if (over[i] > maxmatch)
	{
	planeidx = i;
	maxmatch = over[i];
	}
    }
if (planeidx == -1)
    {
    c_errmsg("No planes were found that pass the constraints",
	"navigate-noplane",&nonfatal);
    return(E_FAIL);
    }
fprintf(fptr,"Selecting points based on plane %d\n",planeidx+1);

/* Make sure we use at least 60% of the pts
   ----------------------------------------*/
if ((100.0*maxmatch/npoints) < 60.0)
    {
    c_errmsg("Too few points used in final fit","navigate-toofew",&nonfatal);
    return(E_FAIL);
    }

/* Calculate the final plane from the selected plane
   -------------------------------------------------*/
for (grp = 0, pt = 0,i = 0; i < npoints; i++)
    {
    used[grp][pt] = FALSE;
    pt++;
    if (pt == pts_in_group[grp])
	{
	grp++;
	pt = 0;
	}
    }

for (m = 0; m < maxmatch; m++)
    {
    index = (long)(key[planeidx][m]);
    grp = 0; 
    while (index >= pts_in_group[grp]) {index-=pts_in_group[grp]; grp++;}
    xline[m] = refline[grp][index];
    xsample[m] = refsample[grp][index];
    xdline[m] = dline[grp][index];
    xdsample[m] = dsample[grp][index];
    used[grp][index] = TRUE;
    }
                     
lsplane(maxmatch,xline,xsample,xdline,linecoefs);
lsplane(maxmatch,xline,xsample,xdsample,samplecoefs);

fprintf(fptr,"Of %d points, %d were used in final fit (%5.2lf%c)\n",
    npoints,maxmatch,100.0*maxmatch/npoints,'%');
if (planeidx != (planes-1))	/* only print pts if a non-average was used */
    fprintf(fptr,"Points that were used for final fit were %s %s %s %s %s\n",
    id[grp][savepts[planeidx][0]],id[grp][savepts[planeidx][1]],
    id[grp][savepts[planeidx][2]],id[grp][savepts[planeidx][3]],
    id[grp][savepts[planeidx][4]]);

return(E_SUCC);
}
