/**************************************************************************
FUNCTION:	LSPLANE

PURPOSE: 	This function calculates the least squares fit for a plane 
		given "n" points around that plane. (two independent variables)

Returns a plane of the form:

y = coefs[0] * x1 + coefs[1] * x2 + coefs[2]

ALGORITHM REFERENCES
page 234 of Applied Linear Statistical Models by John Neter and
William Wasserman (Richard D. Irwin, INC 1974)
**************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

void lsplane(
	long	npoints,	/* Number of points			*/
	double	x1vec[],	/* first independent values (lines)	*/
	double	x2vec[],	/* second independent values (samples)	*/
	double	yvec[],		/* dependant values (errors)		*/
	double	coefs[] 	/* Return coefficients			*/
)


{

/*
           _     _                _     _
          | a b c |              | A B C |
    X'X=  | d e f |   (X'X)^-1 = | D E F | * 1 / Z
          | g h i |              | G H I |
           -     -                -     -
where:
	a = npoints
	b = sum(x1)
	c = sum(x2)
	d = sum(x1)
	e = sum(x1^2)
	f = sum(x1*x2)
	g = sum(x2)
	h = sum(x1*x2)
	i = sum(x2^2)

   Note: b = d, c = g, and f = h
*/

double 	sumx1 = 0.0,
	sumx2 = 0.0,
	sumx1x2 = 0.0,
	sumx1x1 = 0.0,
	sumx2x2 = 0.0,
	sumy = 0.0,
	sumyx1 = 0.0,
	sumyx2 = 0.0,
	a, b, c,
	d, e, f,
	g, h, i,

	A, B, C,
	D, E, F,
	G, H, I,
	Z;
          
long 	index;

for (index = 0; index < npoints; index++)
    {
    sumx1   += x1vec[index];
    sumx2   += x2vec[index];
    sumx1x2 += x1vec[index] * x2vec[index];
    sumx1x1 += x1vec[index] * x1vec[index];
    sumx2x2 += x2vec[index] * x2vec[index];
    sumy    += yvec[index];
    sumyx1  += yvec[index] * x1vec[index];
    sumyx2  += yvec[index] * x2vec[index];
    }

a = npoints;
b = sumx1;
c = sumx2;
d = sumx1;
e = sumx1x1;
f = sumx1x2;
g = sumx2;
h = sumx1x2;
i = sumx2x2;

/*
   Note: b = d, c = g, and f = h
*/
A = e * i - f * h;
B = c * h - b * i;
C = b * f - c * e;
D = B; 			/* D = f * g - d * i; */
E = a * i - c * g;
F = c * d - a * f;
G = C; 			/* G = d * h - e * g; */
H = F; 			/* H = b * g - a * h; */
I = a * e - b * d;
Z = a * A + b * D + c * G;

coefs[2] = (A * sumy + B * sumyx1 + C * sumyx2) / Z;
coefs[0] = (D * sumy + E * sumyx1 + F * sumyx2) / Z;
coefs[1] = (G * sumy + H * sumyx1 + I * sumyx2) / Z;

return;
}
