/****************************************************************************
NAME:				NAVIGATE

PURPOSE:   Perform automatic navigation based on a tie point location file

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
  5.0	   10/93    D.Steinwand Original Development
		    C.Wivell
		    T.Rockvam

PROJECT			ADAPS/1KM AVHRR
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

main()		
{
struct MODEL model;		/* AVHRR model information		*/
struct DDR ddr;			/* AVHRR image ddr			*/

double 	dline[MAXGRPS][PTS_PER_GROUP],		/* err in ver pt in line dir  */
	dsamp[MAXGRPS][PTS_PER_GROUP],		/* err in ver pt in sample dir*/
	refline[MAXGRPS][PTS_PER_GROUP],	/* err in ver pt in line dir  */
	refsamp[MAXGRPS][PTS_PER_GROUP],	/* err in ver pt in sample dir*/
	srcline[MAXGRPS][PTS_PER_GROUP],	/* tie point search lines     */
	srcsamp[MAXGRPS][PTS_PER_GROUP],	/* tie point search samples   */
	latitude[MAXGRPS][PTS_PER_GROUP],	/* tie point latitudes 	      */
	longitude[MAXGRPS][PTS_PER_GROUP],	/* tie point longitudes       */
	elevation[MAXGRPS][PTS_PER_GROUP],	/* tie point elevations       */
	linecoefs[3],		/* Least Squares coeffs mapping lines	*/
	sampcoefs[3];		/* Least Squares coeffs mapping samples	*/

long	active[MAXGRPS][PTS_PER_GROUP];		/* tie point active flag      */

long 	num_out = 0,		/* number of output files		*/
	nonfatal = 1,		/* nonfatal error code			*/
	npts = 0,   		/* total number of verification points 	*/
	sarea,			/* study area				*/
	status,			/* function return status 		*/
	fatal = -1,		/* fatal flag for errmsg 		*/
	msgtim,			/* message timer 			*/
	ngroups,		/* number of groups			*/
	pts_in_group[MAXGRPS],	/* number of points in each group	*/
	used[MAXGRPS][PTS_PER_GROUP]; /* flag if point was used		*/

char 	filename[3][CMLEN],	/* name for PRINT parameter 		*/
        print_file[CMLEN],      /* host print file name                 */
        plot_file[CMLEN],	/* host plot file name			*/
        db_file[CMLEN],		/* data base file name			*/
	host_name[CMLEN],	/* host file name 			*/
	tae_name[CMLEN],	/* TAE file name 			*/
	msg[CMLEN],		/* informational message		*/
	id[MAXGRPS][PTS_PER_GROUP][20];    	/* buffer to hold tie point id*/

int 	*in_tpl;		/* input TPL file descriptor		*/
FILE 	*dbptr;			/* data base file			*/
FILE 	*fptr;			/* output report			*/
FILE 	*pptr;			/* output plot file			*/


/* Get parameters & open files
   ---------------------------*/
getpar(&model,&ddr,&msgtim,&num_out,&in_tpl,filename,print_file,plot_file,
       db_file,&fptr,&pptr,&dbptr,host_name,tae_name,&sarea);

/* Read the active control points  
   ------------------------------*/
status = readpnts(&in_tpl,&npts,id,dline,dsamp,refline,refsamp,srcline,srcsamp,
		  latitude,longitude,elevation,active,&ngroups,pts_in_group);
if (status == E_FAIL) 
    {
    fclose(fptr);
    c_delfil(print_file);
    if (strlen(plot_file) != 0)
	{
	fclose(pptr);
	c_delfil(plot_file);
	}
    if (strlen(db_file) != 0)
	{
	fclose(dbptr);
	c_delfil(db_file);
	}
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
    }

/* Calculate the attitude and attitude rates on the selected points
   ----------------------------------------------------------------*/
if (npts >= 8)
    {
    status = calc_attitudes(fptr,&model,&ddr,dline,dsamp,refline,refsamp,id,
	     used,npts,linecoefs,sampcoefs,&msgtim,ngroups,pts_in_group);
    if (status != E_SUCC)
    	{
    	fclose(fptr);
    	c_delfil(print_file);
	if (strlen(plot_file) != 0)
	    {
	    fclose(pptr);
	    c_delfil(plot_file);
	    }
	if (strlen(db_file) != 0)
	    {
	    fclose(dbptr);
	    c_delfil(db_file);
	    }
        c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
	}
    }
else
    {
    fclose(fptr);
    c_delfil(print_file);
    if (strlen(plot_file) != 0)
	{
	fclose(pptr);
	c_delfil(plot_file);
	}
    if (strlen(db_file) != 0)
	{
	fclose(dbptr);
	c_delfil(db_file);
	}
    c_errmsg("Not enough points to do LMS fit","navigate-npts",&nonfatal);
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
    }

/* Save the coefficients in the header ???
   -----------------------------------*/
sprintf(msg,"Updating ADDR file for %s",tae_name);
c_errmsg(msg," ",&nonfatal);
if (c_putaddr(host_name,&model) != E_SUCC)
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);


/* Report on navigate registration accuracy
   ----------------------------------------*/
write_report(&model,&ddr,fptr,pptr,npts,refline,refsamp,srcline,srcsamp,
	     latitude,longitude,elevation,active,linecoefs,sampcoefs,id,
	     used,ngroups,pts_in_group,plot_file);


/* Generate a data base file if called for
   ----------------------------------------*/
if (strlen(db_file) != 0)
    write_db(&model,dbptr,id,used,sarea);


/* Write the standard completion message and exit
   ----------------------------------------------*/
c_complt("NAVIGATE",filename,&num_out);
}
