/*************************************************************************
FUNCTION:	PLANE2ATTIT

PURPOSE: 	This routine changes the plane fitted values to attitude 
		correction values(roll,droll/dt,pitch,dpitch/dt,yaw,altitude).

		NOTE:
		All corrections are deltas from the previous roll,pitch,yaw,alt
		values because they were taken into account in the creation of 
		the chips, which carried through the correlation process.  The 
		model roll, pitch, and yaw are passed in to plane2attit() to 
		have the new correction deltas added to them.  Since our model 
		structure has both a delta_alt and alt_coefs, the new altitude 
		delta itself will be passed back and added to both.
		(alt_coefs[0] should reflect delta_alt.  We need both because 
		alt_coefs are used in model calculations and delta_alt is 
		saved in the .addr file).

		NOTE: 
		Roll, pitch and yaw coefs are output in terms of degrees.
      		Altitude coefs are output in terms of meters.
*************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

void plane2attit(
	double linecoefs[],		/* Plane line coefficients	*/
	double samplecoefs[],		/* Plane sample coefficients	*/
	double zdot,			/* Pass direction ASC/DESC	*/
	long   dtype,			/* Pass data type LAC/HRPT/GAC	*/
	double satalt,			/* Satellite altitude		*/
	double totlines,		/* Total # lines in scene 	*/
	double offline,			/* Line offset (master line)	*/
	double offsample,		/* Sample offset (master sample)*/
	double rollcoefs[],		/* New roll coefficients	*/
	double pitchcoefs[],		/* New pitch coefficients	*/
	double yawcoefs[],		/* New yaw coefficients		*/
	double *altcoef 		/* Delta altitude correction	*/
)

{
double 	midline,		/* nadir line of pass relative to window*/
	midsample,		/* nadir sample of pass rel to window	*/
	nadirsiz,		/* size of nadir pixel			*/
	dlinec,			/* line coeff at center of pass		*/
	dlines,			/* line coeff at start of pass		*/
	dsamplec,		/* sample coeff at center of pass	*/
	dsamples,		/* sample coeff at start of pass	*/
	pitchc,			/* pitch at center of pass		*/
	pitchs,			/* pitch at start of pass		*/
	dpitchpdt,		/* change in pitch			*/
	rollc,			/* roll at center of pass		*/
	rolls,			/* roll at start of pass		*/
	drollpdt,		/* change in roll			*/
	yaw,			/* yaw correction			*/
	dalt,			/* altitude correction			*/
	radpsamp;		/* radians per sample			*/

if (dtype == GAC)
    {
    nadirsiz = 1091.344 * 5;
    midline = totlines / 2.0 + offline;
    midsample = GAC_NADIR + offsample;
    radpsamp = 1.0 / GAC_SAMPLE_SLOPE;
    }
else
    {
    nadirsiz = 1091.344;
    midline = totlines / 2.0 + offline;
    midsample = LAC_NADIR + offsample;
    radpsamp = 1.0 / LAC_SAMPLE_SLOPE;
    /* midsample = 1024.5 + offsample; */
    /* degpsamp = 55.42 / 1024.5; */
    }

/*
  Calc. pitch at scene center and start of pass 
  to find dpitch / dt 
*/
dlinec = linecoefs[0] * midline + linecoefs[1] * midsample + linecoefs[2];
if (zdot > 0.0)
    {
    pitchc = atan(nadirsiz * dlinec / satalt);
    dlines = linecoefs[0] * (totlines + offline) + 
             linecoefs[1] * midsample + 
             linecoefs[2];
    pitchs = atan(nadirsiz * dlines / satalt);
    }
else
    {
    pitchc = atan(nadirsiz * -dlinec / satalt);
    dlines = linecoefs[0] + linecoefs[1] * midsample + linecoefs[2];
    pitchs = atan(nadirsiz * -dlines / satalt);
    }

dpitchpdt = (pitchc - pitchs) / ((totlines / 2.0) / 6.0);

/*
  calc. roll at scene center and start of pass
  to find droll / dt 
*/
dsamplec=samplecoefs[0] * midline + samplecoefs[1] * midsample + samplecoefs[2];
if (zdot > 0.0)
      {
      rollc = radpsamp * dsamplec;
      dsamples = samplecoefs[0] * (totlines + offline) + 
                 samplecoefs[1] * midsample + 
                 samplecoefs[2];
      rolls = radpsamp * dsamples;
      }
else
      {
      rollc = radpsamp * -dsamplec;
      dsamples = samplecoefs[0] + 
                 samplecoefs[1] * midsample + 
                 samplecoefs[2];
      rolls = radpsamp * -dsamples;
      }

drollpdt = (rollc - rolls) / ((totlines / 2.0) / 6.0);

/*
  calc. yaw for the scene
*/
yaw = atan(linecoefs[1] * zdot);

/*
  calc. altitude error 
*/
dalt = samplecoefs[1] * satalt / (1.0 - samplecoefs[1]);
dalt = -dalt * zdot;

/*
  Save angle values as degrees and alt. values as meters
*/
pitchcoefs[0] += pitchc;
pitchcoefs[1] += dpitchpdt;

rollcoefs[0] += rollc;
rollcoefs[1] += drollpdt;

yawcoefs[0] += yaw;

*altcoef = dalt;

return;
}
