/****************************************************************************
NAME:		READPNTS

PURPOSE:   	Read the active points from the tpl file
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "adaps.h"
#include "navigate.h"

/* Define a structure to use in a queue to save the point info
   temporarily as we read it to see how many points there are
   -----------------------------------------------------------*/
struct PTSET {
    char id[20];
    double refline;
    double refsamp;
    double srcline;
    double srcsamp;
    double dline;
    double dsamp;
    double latitude;
    double longitude;
    double elevation;
    long active;
    };

int readpnts(
	int 	**in_tpl,		    /* input TPL file descriptor      */
	long	*npts,			    /* # of points read from tpl file */
	char 	id[][PTS_PER_GROUP][20],    /* buffer of tie point identifiers*/
	double 	dline[][PTS_PER_GROUP],	    /* err in ver pt in line dir      */
	double	dsamp[][PTS_PER_GROUP],	    /* err in ver pt in sample dir    */
	double	refline[][PTS_PER_GROUP],   /* tie pt ref lines (orig AVHRR)  */
	double	refsamp[][PTS_PER_GROUP],   /* tie pt ref samps (orig AVHRR)  */
	double	srcline[][PTS_PER_GROUP],   /* tie pt search lines ("true")   */
	double	srcsamp[][PTS_PER_GROUP],   /* tie pt search samps ("true")   */
	double	latitude[][PTS_PER_GROUP],  /* tie pt latitudes 	      */
	double	longitude[][PTS_PER_GROUP], /* tie pt longitudes 	      */
	double	elevation[][PTS_PER_GROUP], /* tie pt elevations 	      */
	long	active[][PTS_PER_GROUP],    /* tie pt active flag	      */
	long	*ngroups,		    /* number of groups 	      */
	long	pts_in_group[] 		    /* number of points in each group */
)

{
struct TPLDATA tpldata;		/* tie point location file data record 	*/
struct PTSET *thispt;		/* set of point info in queue		*/

long 	grpindex = 0,		/* loop counter for groups		*/
	i,			/* loop counter				*/
	nonfatal = 1,		/* nonfatal error code			*/
	ptindex = 0,		/* loop counter for points		*/
	queue,			/* queue identifier			*/
	status;			/* function return status 		*/

char	msg[CMLEN];		/* informational message		*/


/* Create a fifo queue allowing duplicates
   ---------------------------------------*/
queue = c_qcreate("fifo","");
if (queue == -1)
    {
    c_errmsg("Queue error","navigate-queue",&nonfatal);
    return(E_FAIL);
    }

for (;;)
    {
    /* Read a point from the tpl file
       ------------------------------*/
    status = c_ptio(in_tpl, &tpldata);
    if (status == E_EOF) break;
    if (status == E_FAIL) 
	return(E_FAIL);

    /* Calculate differences for active points
       -----------------------------------------*/
    if ((tpldata.active == 0) || (tpldata.active == 1) || (tpldata.active > 9))
   	{
	thispt = (struct PTSET *)malloc(sizeof(struct PTSET));
        if (thispt == NULL)
	    {
    	    c_errmsg("Queue error","navigate-queue",&nonfatal);
	    return(E_FAIL);
	    }
	strcpy(thispt->id,  tpldata.pt_id);
	thispt->refline   = tpldata.ref_coord[0];
	thispt->refsamp   = tpldata.ref_coord[1];
	thispt->srcline   = tpldata.sea_coord[0];
	thispt->srcsamp   = tpldata.sea_coord[1];
   	thispt->dline     = tpldata.sea_coord[0] - tpldata.ref_coord[0];
   	thispt->dsamp     = tpldata.sea_coord[1] - tpldata.ref_coord[1];
	thispt->latitude  = tpldata.geo_coord[0];
	thispt->longitude = tpldata.geo_coord[1];
	thispt->elevation = tpldata.elevation;
	thispt->active    = tpldata.active;
	if (c_qinsert(queue,thispt,qcompare) != 0)
	    {
    	    c_errmsg("Queue error","navigate-queue",&nonfatal);
	    return(E_FAIL);
	    }
   	(*npts)++;
   	}

    if (*npts > MAXPTS)
	{
	sprintf(msg,"Only the first %d points will be used",MAXPTS);
	c_errmsg(msg,"navigate-many",&nonfatal);
	*npts = MAXPTS;
	break;
	}
    }

/* Close tpl file
   --------------*/
c_ptclse(in_tpl);

/* Reset queue to split the points up
   ----------------------------------*/
status = c_qfirst(queue, &thispt);
if (status != 0)
    {
    c_errmsg("Queue error","navigate-queue",&nonfatal);
    return(E_FAIL);
    }
if (thispt == NULL)	/* empty queue */
    return(E_SUCC);

*ngroups = ceil((double)(*npts)/(double)(PTS_PER_GROUP));
if (*ngroups > 1)
    {
    sprintf(msg,"Dividing points into %d groups",*ngroups);
    c_errmsg(msg," ",&nonfatal);
    }

for (i = 0; i < MAXGRPS; i++)
    pts_in_group[i] = 0;

for (i = 0; i < *npts; i++)
    {
    strcpy(id[grpindex][ptindex],thispt->id);
    refline[grpindex][ptindex]   = thispt->refline;
    refsamp[grpindex][ptindex]   = thispt->refsamp;
    srcline[grpindex][ptindex]   = thispt->srcline;
    srcsamp[grpindex][ptindex]   = thispt->srcsamp;
    dline[grpindex][ptindex]     = thispt->dline;
    dsamp[grpindex][ptindex]     = thispt->dsamp;
    latitude[grpindex][ptindex]  = thispt->latitude;
    longitude[grpindex][ptindex] = thispt->longitude;
    elevation[grpindex][ptindex] = thispt->elevation;
    active[grpindex][ptindex]    = thispt->active;

    pts_in_group[grpindex]++;
    grpindex++;
    if (grpindex == *ngroups)
	{
	grpindex = 0;
	ptindex++;
	}

    status = c_qnext(queue,&thispt);
    if (status != 0)
	{
	c_errmsg("navigate-queue","Queue error",&nonfatal);
	return(E_FAIL);
	}
    }

/* Remove all space used by the queue
   ----------------------------------*/
c_qdelete(queue);

return(E_SUCC);
}

/* This routine doesn't need to order anything 
   -------------------------------------------*/
long qcompare(
	void	*ptr1,
	void	*ptr2
)

{
return(1);
}
