/****************************************************************************
NAME:		WRITE_REPORT

PURPOSE:   	Report on navigate registration accuracy.
		Generate a report and a plot file.
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

void write_report(
	struct MODEL *model,		/* AVHRR model information	      */
	struct DDR *ddr,		/* AVHRR image ddr		      */
	FILE 	*fptr,			/* output report		      */
	FILE	*pptr,			/* output plot file		      */
	long	npts,   		/* total number of verification points*/
	double 	refline[][PTS_PER_GROUP],/* err in verification pt in line dir*/
	double	refsamp[][PTS_PER_GROUP],/* err in verification pt in samp dir*/
	double	srcline[][PTS_PER_GROUP],/* tie point search lines 	      */
	double	srcsamp[][PTS_PER_GROUP],/* tie point search samples 	      */
	double	latitude[][PTS_PER_GROUP],/* tie point latitudes 	      */
	double	longitude[][PTS_PER_GROUP],/* tie point longitudes 	      */
	double	elevation[][PTS_PER_GROUP],/* tie point elevations 	      */
	long	active[][PTS_PER_GROUP],/* tie point active flag 	      */
	double	linecoefs[],		/* Least Squares coeffs mapping lines */
	double	sampcoefs[],		/* Least Squares coeffs mapping samps */
	char	id[][PTS_PER_GROUP][20],/* buffer to hold tie point identifier*/
	long	used[][PTS_PER_GROUP],  /* flag if point was used	      */
	long	ngroups,		/* number of point groups	      */
	long	pts_in_group[],		/* number of points in each group     */
	char	plot_file[] 		/* plot file name		      */
)

{
double	dblline,		/* double version of "line"		*/
	dblsamp,		/* double version of "samp"		*/
	delta_line, 		/* diff bet model & observed line  	*/
	delta_samp,		/* diff bet model & observed samp	*/
	rmssq = 0.0, 		/* total RMSE accumulator		*/
	rmses = 0.0, 		/* sample RMSE accumulator		*/
	rmsel = 0.0,     	/* line RMSE accumulator		*/
	linesq,sampsq,magsq,    /* line, sample, magnitude squared error*/
	medianline,		/* median line value			*/
	mediansamp,		/* median sample value			*/
	avgl = 0.0, avgs = 0.0, /* accumulators for avg error calc	*/
	lsmag,                  /* err magnitude from Least Squares fit */
	mag,                    /* err magnitude from satellite model fit */
	lat,lon;		/* latitude,longitude			*/

double  dline_arry[MAXGRPS*PTS_PER_GROUP], /* array of all "deleta_line"*/
	dsamp_arry[MAXGRPS*PTS_PER_GROUP], /* array of all "deleta_samp"*/
	key[1][MAXPTS];		/* key in sort (order of points)        */

float 	line,samp;		/* AVHRR line/sample			*/

long	arryindex = 0;		/* array index to delta line/sample	*/
long 	i,j,q;			/* looping variables			*/
long	npts_used;		/* Current number of points used	*/

/* Print out the column headers
   ----------------------------*/
fprintf(fptr,"\n\n");
fprintf(fptr,
"Point ID    Lat     Long   Elev    Line    Sample   Dline   Dsamp     Mag  LSmag\n");
fprintf(fptr,
"-------- ------- ------- ------ -------- -------- ------- ------- ------- ------\n");


/* Write the number of points used to the plot file
   -------------------------------------------------*/
npts_used = 0;
for (i = 0; i < ngroups; i++)
   for (j = 0; j < pts_in_group[i]; j++)
      if (used[i][j])
	 npts_used++;
if (strlen(plot_file) != 0)
    fprintf(pptr,"%d\n",npts_used);

/* Calculate residuals based on the new attitude coefficients
   ----------------------------------------------------------*/
for (i = 0; i < ngroups; i++)
   {
for (j = 0; j < pts_in_group[i]; j++)
   {
   if (!used[i][j])
       continue;
   delta_line=linecoefs[0]*refline[i][j] + linecoefs[1]*refsamp[i][j] + linecoefs[2];
   delta_samp=sampcoefs[0]*refline[i][j] + sampcoefs[1]*refsamp[i][j] + sampcoefs[2];
   delta_line = srcline[i][j] - (refline[i][j] + delta_line);
   delta_samp = srcsamp[i][j] - (refsamp[i][j] + delta_samp);
   lsmag = sqrt(delta_line*delta_line + delta_samp*delta_samp);
   lat = latitude[i][j] * RAD_PER_DEG;
   lon = (longitude[i][j] < 0.0) ? (longitude[i][j]+360.0)*RAD_PER_DEG: 
				 longitude[i][j]*RAD_PER_DEG;
   c_ll2ls(&lat,&lon,&elevation[i][j],model,&line,&samp,NULL);
   /* Correct for sub-windowed image */
   line -= ddr->master_line;
   samp -= ddr->master_sample;
   delta_line = srcline[i][j] - line;
   delta_samp = srcsamp[i][j] - samp;
   avgl += delta_line;
   avgs += delta_samp;
   linesq = delta_line*delta_line;
   sampsq = delta_samp*delta_samp;
   magsq = linesq + sampsq;
   mag = sqrt(magsq);
   rmssq += magsq;
   rmses += sampsq;
   rmsel += linesq;
   fprintf(fptr,"%-8s %7.2f %7.2f %6.1f %8.2f %8.2f %7.3f %7.3f %7.3f %6.3f\n",
     id[i][j],latitude[i][j],longitude[i][j],elevation[i][j],line,samp,
     delta_line,delta_samp,mag,lsmag);
   if (strlen(plot_file) != 0)
      {
      dblline = (double)line;
      dblsamp = (double)samp;
      fprintf(pptr,"%s %f %f %f %f %d\n",id[i][j],dblsamp,dblline,delta_line,
      	      delta_samp,active[i][j]);
      }
   dline_arry[arryindex  ] = delta_line;
   dsamp_arry[arryindex++] = delta_samp;
   }
fprintf(fptr,"\n");
   }
avgl = avgl / npts;
avgs = avgs / npts;
rmssq = sqrt(rmssq / npts);
rmses = sqrt(rmses / npts);
rmsel = sqrt(rmsel / npts);
fprintf(fptr,"\n");
fprintf(fptr,"Line error     Average = %10.6f   RMSE = %10.6f pixels\n", avgl,rmsel);
fprintf(fptr,"Sample error   Average = %10.6f   RMSE = %10.6f pixels\n", avgs,rmses);
fprintf(fptr,"Total error                           RMSE = %10.6f satellite (raw) pixels\n\n", rmssq);

/* Sort the delta line and sample values
   --------------------------------------*/
for (q = 0; q < arryindex; q++)
   key[0][q] = (double)(q);
heapsort(arryindex,dline_arry,key[0]);
heapsort(arryindex,dsamp_arry,key[0]);

/* Find the median values.  If there are an even number
   of points, average the middle two points.
   -----------------------------------------------------*/
if ((arryindex % 2) == 0)
   {
   medianline = (dline_arry[arryindex / 2] + dline_arry[(arryindex / 2) - 1]) / 2;
   mediansamp = (dsamp_arry[arryindex / 2] + dsamp_arry[(arryindex / 2) - 1]) / 2;
   }
else
   {
   medianline = dline_arry[(arryindex - 1) / 2];
   mediansamp = dsamp_arry[(arryindex - 1) / 2];
   }

fprintf(fptr,"\n\n");
fprintf(fptr,"Minimum Dline = %10.6f   ", dline_arry[0]);
fprintf(fptr,"Minimum Dsamp = %10.6f\n",  dsamp_arry[0]);
fprintf(fptr,"Maximum Dline = %10.6f   ", dline_arry[arryindex-1]);
fprintf(fptr,"Maximum Dsamp = %10.6f\n",  dsamp_arry[arryindex-1]);
fprintf(fptr,"Median Dline  = %10.6f   ",  medianline);
fprintf(fptr,"Median Dsamp  = %10.6f\n",   mediansamp);


if (strlen(plot_file) != 0)
   {
   fprintf(pptr,"%lf\n",rmssq);
   fclose(pptr);
   }

fclose(fptr);

return;
}
