/******************************************************************************
NAME:                           ELIMINAT 

PURPOSE:    Eliminate half orbits if certain conditions are met.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      03/94          CSB           Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "stitch.h"             /* Orbital stitch include file         */


FUNCTION eliminat(procopt,sceneid,locate,nscenes,lndflgs,orbflgs,doneflg,elimflg)

char procopt[];         /* Processing option                    */
char sceneid[][17];     /* Scene ids                            */
char locate[][5];       /* Scene location DISK, SILO, TAPE      */
long nscenes[];         /* Number of passes in the orbit        */
char lndflgs[];         /* Line detect flags 'L' or ' '         */
char orbflgs[];         /* Orbital stitch flags 'O' or ' '      */
long doneflg[];         /* Done processing all orbits flag      */
long *elimflg;          /* Eliminate flag TRUE or FALSE         */

{
long i;                 /* Counter                              */


/* Initialize to not eliminate this half orbit
-------------------------------------------------------*/
*elimflg = FALSE;


/*-----------------------------------------------------
Determine if we need to set the done flag to false:
If the calling routine is ORBREAD then set the done
flag to false unless all of the scenes are on disk.
If the calling routine is ORBSTITCH then set the done
flag to false unless all of the scenes have been
stitched.
If the calling routine is ORBLN then set the done
flag to false unless all of the scenes have been
line detected.
-------------------------------------------------------*/
if (strcmp(procopt,"READ") == 0)
    for (i = 0; i < nscenes[0]; i++)
		{
        if (strcmp(locate[i],"DISK") != 0)
            {
            doneflg[0] = 0;
            break;
            }
		}
else if (strcmp(procopt,"STITCH") == 0)
    for (i = 0; i < nscenes[0]; i++)
        {
        if (orbflgs[i] != 'O')
            {
            doneflg[0] = 0;
            break;
            }
        }
else if (strcmp(procopt,"DETECT") == 0)
    for (i = 0; i < nscenes[0]; i++)
        {
        if (lndflgs[i] != 'L')
            {
            doneflg[0] = 0;
            break;
            }
        }


/* Eliminate the entire half orbit if there is only 1 scene in
   the entire half orbit and that scene is an "ao" (a previously
   stitched scene).
----------------------------------------------------------------*/
if ((nscenes[0] == 1) && (strncmp("ao",sceneid[0],2) == 0))
    {
    *elimflg = TRUE;
    return;
    }


/* If this routine is called by ORBREAD then we only want to return this half
   orbit if at least one scene resides on the silo, and all of the scenes are
   either on the silo or disk.  This is because we are only interested in scenes
   which need to be copied from the silo to the disk.  If not all of the scenes
   are on the disk or silo we do not want to populate the disk until all of the
   scenes are readily available.
----------------------------------------------------------------------------*/
if (strcmp(procopt,"READ") == 0)
    {
    for (i = 0; i < nscenes[0]; i++)
        if (strcmp(locate[i],"SILO") == 0)
            break;
    if (i == nscenes[0])
        {
        *elimflg = TRUE;
        return;
        }
    
    for (i = 0; i < nscenes[0]; i++)
        if ((strcmp(locate[i],"SILO") != 0) && (strcmp(locate[i],"DISK") != 0))
            {
            *elimflg = TRUE;
            return;
            }
    }


/* If this routine is called by ORBSTITCH then we only want to return this half 
  orbit if all of the input scenes have been line detected but not yet stitched.
------------------------------------------------------------------------------*/
if (strcmp(procopt,"STITCH") == 0)
    for (i = 0; i < nscenes[0]; i++)
        if ((lndflgs[i] != 'L') || (orbflgs[i] != ' '))
            {
            *elimflg = TRUE;
            return;
            }


/* If this routine is called by ORBLN then we only want to return this half
   orbit if at least one of the input scenes resides on the disk but has not
   yet been line detected.
----------------------------------------------------------------------------*/
if (strcmp(procopt,"DETECT") == 0)
    {
    for (i = 0; i < nscenes[0]; i++)
        if ((strcmp(locate[i],"DISK") == 0) && (lndflgs[i] != 'L'))
            return;
    *elimflg = TRUE;
    return;
    }


return;
}
