/******************************************************************************
NAME:                           GETORB 

PURPOSE:    Get the scene id's and other information associated with
            one satellite half orbit.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      01/93          CSB           Original Development
  1.1      T. Baltzer      12/97          CSB           Fix to properly handle
							spanning days
 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "stitch.h"             /* Orbital stitch include file         */

#define ORBIT_GAP_TIME  1860	/* Minimum time between scenes which	*/
				/* constitutes a new orbit (seconds)	*/

FUNCTION getorb(filnam,tapeid,sceneid,locate,srclnk,lndflgs,orbflgs,nscenes,linenum)

   char filnam[];               /* Name of input file                   */
   char tapeid[][7];            /* Tape identifier                      */
   char sceneid[][17];          /* Scene identifier                     */
   char locate[][5];            /* Location of scene                    */
   char srclnk[][7];            /* Data base source link                */
   char lndflgs[];              /* Line detect flags                    */
   char orbflgs[];              /* Orbital stitch flags                 */
   long nscenes[];              /* Number of passes in the orbit        */
   long linenum[];              /* line number to read                  */

{

long j;                         /* counter                              */
long location;                  /* Location of scene                    */
/*   0 = undetermined                   */
/*   1 = on disk                        */
/*   2 = on the silo                    */
/*   3 = apparently on tape             */
long status;                    /* return status value                  */
long stoptime;                  /* Acquisition stop time                */
long startime;                  /* Acquisition start time               */
long ndays;                     /* days between refrdate, currdate      */

char refrdate[7];               /* Reference date, date of first scene	*/
char currdate[7];               /* Date of current  line MMDDYY         */

FILE *infp;                     /* LUT file pointer                     */

struct TAPESTR oneline;         /* Structure with one line of info      */


/* Get the values from the first line of data for this orbital stitch 
----------------------------------------------------------------------*/
status = get_1st_line(filnam,linenum,&oneline,&infp);
if (status != E_SUCC)
    return(status);


/* Get the reference date, the date of the first scene within this orbit
----------------------------------------------------------------------*/
strncpy(refrdate, &(oneline.sceneid[4]),6);
refrdate[6] = '\0';


/* Now get the variables that need to be obtained once
per input scene (for one entire orbit to be stitched).
------------------------------------------------------*/
for (j = 0, nscenes[0] = 0; ;j++)
    {

    /* Check if too many scenes will be acquired
    --------------------------------------------*/
    if (nscenes[0] == MAX_PASS)
        {
        c_lasmsg(FALSE,"halforb-numscenes",
            "Too many scenes found within an orbit");
        c_lasmsg(FALSE,"halforb-numscenes",
            "Maximum number of scenes per orbit is %d", MAX_PASS);
        c_lasmsg(FALSE,"halforb-numscenes",
            "Last sceneid processed is %s", sceneid[MAX_PASS-1]);
        return (E_FAIL);
        }
    
    /* Store the tapeid
    --------------------*/
    strcpy(tapeid[j],oneline.tapeid);
    

    /* Store the scene id
    ---------------------*/
    strcpy(sceneid[j],oneline.sceneid);
    

    /* Store the data base source link
    -----------------------------------*/
    strcpy(srclnk[j],oneline.srclnk);
    

    /* Store the line detect flag, orbital stitch flag
    ----------------------------------------------------*/
    lndflgs[j] = oneline.lndflg;
    orbflgs[j] = oneline.orbflg;
    

    /* Determine if the scene is on disk, silo, tape, unknown
    ---------------------------------------------------------*/
    scene_loc(&oneline,&location);
    switch(location)
        {
        case 0: strcpy(locate[j],"UNKW"); break;
        case 1: strcpy(locate[j],"DISK"); break;
        case 2: strcpy(locate[j],"SILO"); break;
        case 3: strcpy(locate[j],"TAPE"); break;
        }
    

    /* Increment number of passes in this orbit
    --------------------------------------------*/
    (nscenes[0])++;
    

    /* Get the acquisition stop time for this scene
    ------------------------------------------------*/
    status = calctime(oneline.stoptm,&stoptime);
    if (status != E_SUCC)
        return(E_FAIL);

    
    /* Get the acquisition start time for this scene.
       Get it to check if this scene spans midnight.
       If it does, add one days worth of sec to stop time.
    ------------------------------------------------------*/
    status = calctime(&(oneline.sceneid[10]),&startime);
    if (status != E_SUCC)
        return(E_FAIL);
    if (stoptime < startime)
	stoptime += 86400;


    /* If day has been spanned, add time of day(s) between
    -------------------------------------------------------*/
    strncpy(currdate, &(oneline.sceneid[4]),6);
    currdate[6] = '\0';
    status = days_ago(currdate,refrdate,&ndays);
    if (status != E_SUCC)
        return(E_FAIL);
    stoptime += (ndays * 86400);
    

    /* Get the values from the next line of data.  If a line is
       not successfully gotten, assume that the EOF has been
       encountered.  Processing should not abort since at least
       one line of data was already obtained for this orbit.
    --------------------------------------------------------------*/
    (linenum[0])++;
    status = line_info(infp,&oneline);
    if (status != E_SUCC)
        break;
    

    /* Get the acquisition start time for the next scene
    ------------------------------------------------------*/
    status = calctime(&(oneline.sceneid[10]),&startime);
    if (status != E_SUCC)
        return(E_FAIL);
    

    /* If day has been spanned, add time of day(s) between
    -------------------------------------------------------*/
    strncpy(currdate, &(oneline.sceneid[4]),6);
    currdate[6] = '\0';
    status = days_ago(currdate,refrdate,&ndays);
    if (status != E_SUCC)
        return(E_FAIL);
    startime += (ndays * 86400);
    

    /* If start time - stop time is > ORBIT_GAP_TIME
    then this next line is part of the next orbit
    ---------------------------------------------*/
    if ((startime - stoptime) > ORBIT_GAP_TIME)
        break;
    }


/* Close the input file
------------------------------*/
if ( fclose(infp) != E_SUCC )
    {
    c_lasmsg(FALSE,"halforb-clos","Error closing input file");
    c_lasmsg(FALSE,"halforb-warn","Nonfatal error encountered");
    }

return(E_SUCC);
}
