/******************************************************************************
FUNCTION:       PROCES

PURPOSE:        Main processing procedure for stitching scenes into a half
                orbit.  This routine will return the info needed to stitch
                the appropriate scenes into the next half orbit.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0       01/93    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT                 ADAPS

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>              /* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"                /* LAS include                         */


FUNCTION proces(tapeid,sceneid,locate,srclnk,lndflgs,orbflgs,orbitnm,
                nscenes,linenum,procflg,strscene,endscene,filnam,logfile)

   char tapeid[][7];            /* Tape identifier                      */
   char sceneid[][17];          /* Scene identifier                     */
   char locate[][5];            /* Location of scene                    */
   char srclnk[][7];            /* Data base source link                */
   char lndflgs[];              /* Line detect flags                    */
   char orbflgs[];              /* Orbital stitch flags                 */
   char orbitnm[];              /* Orbit name of stitched output image  */
   long nscenes[];              /* Number of passes in the orbit        */
   long linenum[];              /* Line number to begin processing at   */
   long procflg[];              /* Processing flag                      */
   char strscene[];             /* Scene id at which to start processing*/
   char endscene[];             /* Scene id at which to end processing  */
   char filnam[];               /* Name of input file                   */
   char logfile[];              /* Name of log file                     */

{

long i,k;                       /* Loop counter                         */
long status;                    /* Return status                        */


do
    {
    /* Get the scene id's, etc.  associated with a half orbit
    ---------------------------------------------------------*/
    status = getorb(filnam,tapeid,sceneid,locate,srclnk,
    lndflgs,orbflgs,nscenes,linenum);
    if (status != E_SUCC)
        return(status);

    /*--------------------------------------------------------
    If the starting scene id has not yet been found, see if
    it exists in the orbit just returned in getorb().
    If the starting scene id is not the first id in the orbit
    list, delete the id's preceding the starting scene id.
    Warn the user that id's are being discarded, consequently
    only a partial orbit will be stitched together.
    ----------------------------------------------------------*/
    if (procflg[0] == 0)
        {
        for (i = 0; i < nscenes[0]; i++)
            {
            if (strcmp(strscene,sceneid[i]) == 0)
                {
                procflg[0] = 1;
                nscenes[0] -= i;
                for (k = 0; k < nscenes[0]; k++)
                    {
                    strcpy(sceneid[k],sceneid[k+i]);
                    strcpy(tapeid[k],tapeid[k+i]);
                    strcpy(locate[k],locate[k+i]);
                    strcpy(srclnk[k],srclnk[k+i]);
                    lndflgs[k] = lndflgs[k+i];
                    orbflgs[k] = orbflgs[k+i];
                    }
                if (i > 0)
                    c_lasmsg(FALSE,"halforb-pre",
					"Processing partial orbit following STRSCENE");
                break;
                }
            }
        }
    }
while (procflg[0] == 0);


/* Check to see if this orbit contains the ending
   scene id.  If it does, set the processing flag
   to denote the final orbit to be stitched.
   Delete any scenes in the orbit which occur
   subsequent to the ending scene ENDSCENE, and
   warn the user of these deletions.
-------------------------------------------------*/
for (i = 0; i < nscenes[0]; i++)
    {
    if (strcmp(endscene,sceneid[i]) == 0)
        {
        procflg[0] = 2;
        if ((i + 1) != nscenes[0])
            {
            nscenes[0] = i + 1;
            c_lasmsg(FALSE,"halforb-end",
			"Processing partial orbit preceding ENDSCENE");
            }
        }
    }


/* Check to see if there are duplicate entries in this half orbit.  
   If there are, get rid of them, warn the user.
------------------------------------------------------------------*/
check_dup(tapeid,sceneid,locate,srclnk,lndflgs,orbflgs,nscenes);


/* Check to see if the scenes do not exist on disk, silo, tape.  
   If they don't, get rid of them, warn the user.
-----------------------------------------------------------------------*/
check_not(tapeid,sceneid,locate,srclnk,lndflgs,orbflgs,nscenes,logfile);

/* Determine the name of the half orbit.  This name will be derived from the
   first scene in the sceneid array even though there may have been scenes in
   this half orbit which were eliminated by the STRSCENE spec.  Replace the
   second character with an "o".
---------------------------------------------------------------------------*/
strcpy(orbitnm,sceneid[0]);
orbitnm[1] = 'o';


return(E_SUCC);
}
