/******************************************************************************
NAME:   			PROCESS 

PURPOSE:    Process the data to generate the necessary output image

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         REASON
-------    ----------      ----         ------
  1.0      T.Baltzer      04/00         Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   REANGLE must be run under TAE.

PROJECT:                        ADAPS

******************************************************************************/

#include <string.h>
#include <math.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "adaps.h"
#include "util.h"
#include "pixman.h"
#include "reangle.h"


#define DTBND_IN   0		/* Date band index			*/

#define CH1_OUT    0		/* AVHRR angle band 1 output index	*/
#define CH2_OUT    1		/* AVHRR angle band 2 output index	*/
#define CH3_OUT    2		/* AVHRR angle band 3 output index	*/

#define UNDEFINED  -1		/* Value signifying an undefined value	*/

long curent;			/* Current line being processed		*/


int process
(
    char hosout[][CMLEN],		/* IN: host name of output image     */
    struct FILEINFO fileinfo[],		/* IN: File information structure    */
    struct GDESC **gdesc,		/* IN: group descriptor for image I/O */
    long offset1[MAXIMG*MAXBND],	/* IN: offsets for input bands	     */
    long offset2[MAXBND],   		/* IN: offsets for output bands	     */
    char *thebuf,			/* IN: image I/O buffer              */
    int sceneptlist[MAXSCENE],		/* IN: Scene pointer listing	     */
    struct MODEL model[MAXSCENE],	/* IN: Array of AVHRR models         */
    long msgtim,			/* IN: time between display messages */
    char *minmax,			/* IN: min max flag                  */
    struct SCOFF *scoff1,		/* IN: Scaling coefficient structure */
    struct SCOFF *scoff2,		/* IN: Scaling coefficient structure */
    struct SCOFF *scoff3,		/* IN: Scaling coefficient structure */
    struct DDR *ddr  			/* IN: Composite image DDR	     */
)

{

double
	elev = 0.0,		/* Elevation value			     */
	lat,			/* latitude for one pixel 		     */
	lon,			/* longitude for one pixel		     */
	minval[MAXBND],		/* min pixel value array		     */
	maxval[MAXBND],		/* max pixel value array		     */
	proj_x,			/* projection coordinate of a grid point     */
	proj_y,			/* projection coordinate of a grid point     */
	rot_coeffs[6],		/* rotation coefficients		     */
	view_angles[3];		/* Zenith and Azimuth angles		     */

float	line,			/* Line variable			     */
	*rthebuf,		/*  pointer to thebuf			     */
	samp;			/* Sample variable			     */

long	bands_index,		/* Date band index			     */
	degrees = DEGREE,	/* units code for projection (degrees)	     */
	geo = GEO,		/* geographic projection flag		     */
	sample,			/*  counter -- current pixel		     */
	mmflag = FALSE,		/* Min max flag				     */
	msg_flag=GEN2,		/* flag for processing message               */
	nl,			/* number of image lines                     */
	ns,			/* number of image samples                   */
	num_in_img = NUM_IN_IMG,/* Number of input images		     */
	num_out_bnd,		/* Number of output bands		     */
	*offset1_DTBND_IN,	/* offset1 plus DTBND_IN		     */
	*offset2_CH1_OUT,	/* offset2 plus CH1_OUT			     */
	*offset2_CH2_OUT,	/* offset2 plus CH2_OUT			     */
	*offset2_CH3_OUT,	/* offset2 plus CH3_OUT			     */
	proj_units,		/* Projection units value		     */
	rdtype = EREAL,		/* Floating point data type		     */
	undefined_db_max;	/* Undefined date band maximum value	     */


/* Assign the undefined date band values based on the type of composite:

   US:     0 = undefined 

   GLOBAL: 0 = missing over land
	   1 = pixel over ocean
	   2 = pixel in Goodes interupt area

   NOTE: Don't have to check for less than zero because values are coming from
	 a byte image.  Also, REANGLE only processes US scaled images, so the
	 above global documentation is only informational.
-----------------------------------------------------------------------------*/
undefined_db_max = 0;


/* Setup parameters
---------------------------*/
nl = fileinfo[0].nlines;
ns = fileinfo[0].nsamples;
rthebuf = (float *) thebuf;

/* Initialize min/max arrays: mmflag is set to be used within
    the processing loop -- quicker than strcmp
-----------------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   mmflag = TRUE;	/* set flag			 */
   num_out_bnd = 3;	/* three angles bands output     */
   c_inmnmx(&rdtype,&num_out_bnd,&minval[0],&maxval[0]);
   }


/* Initialize the transformation information for the composite image
----------------------------------------------------------------------*/
if ( my_trans_init( ddr, &proj_units, rot_coeffs) != E_SUCC)
    return(E_FAIL);


/* Setup for processing message utility
---------------------------------------*/
curent = 1;    
c_inittm("REANGLE","Processed","line","bands"," ",&nl,&num_in_img,&msgtim,
	 &msg_flag);
c_startm();

/* Main processing loop, let the processing begin
--------------------------------------------------*/
for (curent = 0; curent < nl; curent++)
   {
   /* Read / write a line of the image */
   if (c_estep(gdesc) != E_SUCC)
      return(E_FAIL);

   /* Assign the offsets for each channel, do this before going into
      the 'sample' loop (for the sake of efficiency).
   ----------------------------------------------------------------*/
   offset1_DTBND_IN = offset1 + DTBND_IN;
   offset2_CH1_OUT  = offset2 + CH1_OUT;
   offset2_CH2_OUT  = offset2 + CH2_OUT;
   offset2_CH3_OUT  = offset2 + CH3_OUT;
   
   for (sample = 0; sample < ns; sample++)
      {
      /* Get the value of each sample for each input band 
      ----------------------------------------------------*/
      bands_index = (long) (*(rthebuf + (*offset1_DTBND_IN + sample)));

      /* Make sure we are not pointing to an unspecified scene index   */
      if ((bands_index > undefined_db_max) &&
	  ((bands_index >= MAXSCENE) ||
	  (sceneptlist[bands_index] == -1)))
	 {
	 c_lasmsg(FALSE,"reangle-band","Invalid date band specification");
         return(E_FAIL);
	 }

      /* Assign a pixel value for each of the three output bands.
	 Note that we don't worry about an undefined GOODES region
	 because we are worried about US composites where this doesn't
	 occur...
      ----------------------------------------------------------------*/
      if (bands_index <= undefined_db_max)
	 {
	 *(rthebuf + (*offset2_CH1_OUT + sample)) = (float)bands_index;
	 *(rthebuf + (*offset2_CH2_OUT + sample)) = (float)bands_index;
	 *(rthebuf + (*offset2_CH3_OUT + sample)) = (float)bands_index;
	 }
      else
	 {

        /* Determine the projection coordinates taking
         into account any rotation of the image.
         -----------------------------------------------*/
         proj_y = ( rot_coeffs[0] * (double)(curent + 1) ) +
                  ( rot_coeffs[1] * (double)(sample + 1) ) + rot_coeffs[2];
         proj_x = ( rot_coeffs[3] * (double)(curent + 1) ) +
                  ( rot_coeffs[4] * (double)(sample + 1) ) + rot_coeffs[5];

         /* Compute the latitude and logitude of the location.
         If c_trans does not return E_SUCC we assume we are
         in an invalid region such as an interupted Goode region.
         -----------------------------------------------------------*/
         if ( c_trans( &ddr->proj_code, &proj_units, &geo, &degrees,
                       &proj_x, &proj_y, &lon, &lat ) != E_SUCC )
	     {
	     *(rthebuf + (*offset2_CH1_OUT + sample)) = UNDEFINED;
	     *(rthebuf + (*offset2_CH2_OUT + sample)) = UNDEFINED;
	     *(rthebuf + (*offset2_CH3_OUT + sample)) = UNDEFINED;
	     }
	 else
	     {
	     /* Call c_ll2ls() to determine the line/sample within the
	        acquisition image.  First convert lat/lon to radians.
             ---------------------------------------------------------*/
	     lat *= RAD_PER_DEG;
	     lon *= RAD_PER_DEG;
	     c_ll2ls(&lat,&lon,&elev,&model[sceneptlist[bands_index]],
		     &line,&samp,view_angles);

	     /* Convert the view angles to degrees
             ---------------------------------------------------------*/
	     view_angles[0] = view_angles[0] * DEG_PER_RAD;
	     view_angles[1] = view_angles[1] * DEG_PER_RAD;
	     view_angles[2] = view_angles[2] * DEG_PER_RAD;

             /* Satellite zenith angles are negative from the first
                scanned pixel to nadir and positive after nadir.
             Note: Added for clarification 09/14/2001 DEL.  The above
             statement is correct as well as the logic below.  Since
             the models used are in satellite space, the line/sample
             values coming from the model are in satellite space.
             For a more complete explaination for this logic, refer to
             RECTIFY coding!
             ---------------------------------------------------------*/
	     if ((( model[sceneptlist[bands_index]].cpass == ASCENDING ) &&
		    ( samp <= 1024.0)) ||
		    (( model[sceneptlist[bands_index]].cpass == DESCENDING ) &&
		    ( samp > 1024.0)))
			view_angles[0] = view_angles[0] * -1.0;

	     /* relative azimuth is defined as -180 to +180
		so make sure it's in that range.
             --------------------------------------------------*/
	     view_angles[2] = fabs(view_angles[2] - 180.0);

	     /* Scale the AVHRR angle bands and
	        assign the output values to the I/O buffer.
	     -------------------------------------------------------------*/
	     *(rthebuf + (*offset2_CH1_OUT + sample)) = 
	         SCALE(view_angles[0],scoff1->minval,scoff1->maxval,
		       scoff1->scale,scoff1->offset);
	     *(rthebuf + (*offset2_CH2_OUT + sample)) =
	         SCALE(view_angles[1],scoff2->minval,scoff2->maxval,
		       scoff2->scale,scoff2->offset);
	     *(rthebuf + (*offset2_CH3_OUT + sample)) =
	         SCALE(view_angles[2],scoff3->minval,scoff3->maxval,
		       scoff3->scale,scoff3->offset);
	     }
	 }
      }

      /* Calculate the min/max values for the output bands, if specified 
       ----------------------------------------------------------------*/
      if (mmflag)
	 {
         c_pxmnmx((long *)(rthebuf + *offset2_CH1_OUT),&ns,
                  &minval[CH1_OUT],&maxval[CH1_OUT],&rdtype); 
         c_pxmnmx((long *)(rthebuf + *offset2_CH2_OUT),&ns,
                  &minval[CH2_OUT],&maxval[CH2_OUT],&rdtype); 
         c_pxmnmx((long *)(rthebuf + *offset2_CH3_OUT),&ns,
                  &minval[CH3_OUT],&maxval[CH3_OUT],&rdtype); 
	 }
   }

/* Write the final line of output data
---------------------------------------*/
if (c_estep(gdesc) != E_SUCC)
    return(E_FAIL);

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Save the associated min/max values
------------------------------------------------------------------------*/
if (mmflag)
    {
    if (c_svmnmx(hosout[0],&num_out_bnd,&minval[0],&maxval[0],minmax) != E_SUCC)
	c_lasmsg(FALSE,"reangle-warn","Nonfatal error encountered");
    }

return(E_SUCC);
}
