/******************************************************************************
NAME:                           REATMOINFO 

PURPOSE:    Get the necessary information from the composite date log file.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      10/99          CSB           Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none

   FILE FORMAT REFERENCE:  For a sample input date band log file, see
			   the file 'us_232245.log' ('long' format) or
			   the file '1992p19.log' ('short' format)
			   stored under CVS as part of the REATMO module.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "reatmoinfo.h"


int main(void)
{


long access;			/* Type of access for getnam call	*/
long element = 0;		/* Element number for getnam call	*/

int i;				/* counter                              */
int file_format;		/* File format flag (long, short)	*/
int header_lines;		/* Number of header lines in log file	*/
int line_count;			/* Line counter				*/
int ndays;			/* Number of days in composite		*/
int nmonths;			/* Number of months in composite	*/
int nperiods;			/* Number of periods in 'periods'	*/
int nscanned;			/* Number of things scanned from string	*/
int satnum;			/* Satellite number			*/
int scenelist[MAXSCOMP];	/* List of scene in composite		*/
int status;			/* Return status			*/

long std_out = 0;		/* Standard output device		*/

char datelist[MAXDAYS][7];  	/* List of dates used in the composite	*/
char filename[CMLEN];		/* Half orbit file name                 */
char linebuf[CMLEN+1];          /* Buffer containing line of data       */
char ozonelist[2][3];		/* List of months used in the composite */
char periods[MAXPERIOD][CMLEN];	/* Periods used in the composite	*/
char satid[3];			/* Satellite id number			*/
char sceneid[17];		/* 16 char scene id in line buffer	*/

struct VARIABLE *val;		/* pointer to a TAE variable		*/
struct PARBLK vblock;		/* TAE parameter block			*/

FILE *infp;                 	/* LUT file pointer			*/


/* Initialize LAS
---------------------------*/
c_inlas(&vblock,&std_out);


/* Initialize the variables
---------------------------*/
for (i = 0; i < MAXSCOMP; i++)
    scenelist[i] = -1;
ndays = 0;
nmonths = 0;


/* Get the period name(s)
--------------------------------------------*/
val = p_find(&vblock,"PERIODS");
nperiods = val->v_count;
for (i = 0; i < nperiods; i++)
    strcpy(periods[i],SVAL(*val,i));


/* Get the name of the file to read and make
sure the file exists with read permission
--------------------------------------------*/
access = IREAD;
status = c_getnam(&vblock,"DATELOG",&element,filename,"",&access);
if (status != E_SUCC)
    c_lasmsg(TRUE,"reatmoi-fatal","Fatal error encountered");


/* Open the input file
-----------------------*/
infp = fopen(filename,"r");
if (infp == NULL)
    {
    c_lasmsg(FALSE,"reatmoi-open","Error opening input file");
    c_lasmsg(TRUE,"reatmoi-fatal","Fatal error encountered");
    }


/* Read the header to determine if it is a 'long' or 'short'
   format log file.  Determine this using the third line
   in the file.  Then read past all of the header information.
--------------------------------------------------------------*/
for (line_count = 0; line_count < 3; line_count++)
    {
    if ( fgets(linebuf,CMLEN+1,infp) == NULL)
        {
	fclose(infp);
        c_lasmsg(FALSE,"reatmoi-read","Error reading line in input file");
        c_lasmsg(TRUE,"reatmoi-fatal","Fatal error encountered");
        }
    }

if (strncmp(linebuf,"Scenei",6) == 0)
    {
    file_format = SHORT_FORMAT;
    header_lines = HEADER_LINES_SHORT;
    }
else if (strncmp(linebuf,"Period",6) == 0)
    {
    file_format = LONG_FORMAT;
    header_lines = HEADER_LINES_LONG;
    }
else
    {
    fclose(infp);
    c_lasmsg(FALSE,"reatmoi-form","Error determining log file format");
    c_lasmsg(TRUE,"reatmoi-fatal","Fatal error encountered");
    }

for (      ; line_count <= header_lines; line_count++)
    {
    if ( fgets(linebuf,CMLEN+1,infp) == NULL)
        {
	fclose(infp);
        c_lasmsg(FALSE,"reatmoi-read","Error reading line in input file");
        c_lasmsg(TRUE,"reatmoi-fatal","Fatal error encountered");
        }
    }


/* Get the satellite identifier number
--------------------------------------------------*/
if (file_format == LONG_FORMAT)
    nscanned = sscanf(linebuf,"%*s %*s %*s %s", sceneid);
else
    nscanned = sscanf(linebuf,"%s",sceneid);
if ( nscanned == 0 )
    {
    c_lasmsg(FALSE,"reatmoi-scan","Error scanning satellit ID");
    c_lasmsg(TRUE,"reatmoi-scan","Fatal error encountered");
    }
strncpy(satid,&sceneid[2],2);
satid[2] = '\0';
satnum = atoi(satid);


/* Process one line of data.
   Read a line, if NULL is returned, assume the EOF has been
   encountered.  Don't print an error message.
---------------------------------------------------------------*/
for (;;)
    {
    if (file_format == LONG_FORMAT)
	process_long(linebuf,datelist,&ndays,scenelist,periods,nperiods);
    else
	process_short(linebuf,datelist,&ndays,scenelist);

    if ( fgets(linebuf,CMLEN+1,infp) == NULL)
	break;
    }


/* Close the date log file
--------------------------------------*/
if ( fclose(infp) != E_SUCC )
    {
    c_lasmsg(FALSE,"reatmoi-clos","Error closing input file %s",
	     filename);
    c_lasmsg(FALSE,"reatmoi-warn","Nonfatal error encountered");
    }


/* Range check the 'ndays' parameter here
------------------------------------------*/
if (ndays > MAXDAYS)
    {
    c_lasmsg(FALSE,"reatmoi-days","Maximum number of days exceeded");
    c_lasmsg(FALSE,"reatmoi-fatal","Fatal error encountered");
    }


/* Determine the ozone month information here.  Assumptions:
   (1) There will always be at least one month in the composite.
   (2) There will never be more than two months in the composite.
   (3) The month specified in the first element of 'datelist'
       becomes the first element in 'ozonelist'.
-----------------------------------------------------------------*/
strncpy(&ozonelist[0][0],&datelist[0][0],2);
ozonelist[0][2] = '\0';
nmonths = 1;
for (i = 1; i < ndays; i++)
    {
    if (strncmp(&ozonelist[0][0],&datelist[i][0],2) != 0)
	{
	strncpy(&ozonelist[1][0],&datelist[i][0],2);
	ozonelist[1][2] = '\0';
	nmonths = 2;
	break;
	}
    }


/* Send the information back to TAE land
--------------------------------------------*/
taeland(&vblock,datelist,scenelist,ozonelist,&ndays,&nmonths,&satnum);


/* Exit with a success status, no message
------------------------------------------*/
z_exit(1,"");


/* The return is placed here to make the compiler happy
--------------------------------------------------------*/
return (E_SUCC);
}

