/*******************************************************************************
FUNCTION:	display_image

PURPOSE:	Display the input file in the drawing area widget

INPUT:		the graphics context
		pointer to the display image
		pixmap for the background display
		input file descriptor
		day/night flag
		RGB band array
		RGB shift values structure

OUTPUT:		the image

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: display_image.c,v $
Revision 1.3  2002/07/18 17:39:30  dlloyd
Periodic check in during development

Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <stdio.h>
#include <Xm/ScrollBar.h>
#include "shoacqu.h"


/* Global variables referenced
------------------------------*/
extern int              imagedisplay;
extern Widget		toplevel;


void display_image
    (
    GC     gc,			/* IN - the graphics context		*/
    XImage *theimage,		/* IN - pointer to the display image	*/
    Pixmap pixmap,		/* IN - background pixmap for display	*/
    int    infd,                /* IN - input file descriptor           */
    int    daynightflag,        /* IN - flag DAY or NIGHT               */
    int    *rgb_bands,		/* IN - arrays of bands to display rgb	*/
    const struct SHIFT *shift   /* IN - RGB shift values                */
    )
    {
    char	*imageptr;	/* pointer into the image buffer	*/
    char	wstr[STRLEN];	/* widget name string			*/

    Display     *display;	/* pointer to the display		*/

    int		i;		/* loop counter				*/
    int		line;		/* line number of the image buffer	*/
    int		sample;		/* sample number of the image buffer	*/
    int		vsbmax;		/* maximum value of vertical scroll bar	*/
    int		vsbmin;		/* minimum value of vertical scroll bar	*/
    int		vsbpos;		/* vertical scroll bar position		*/
    int		vsbsize;	/* size of vertical scroll bar		*/

    long	foreground;	/* foreground color of pixel		*/

    short	bandvals[6];	/* AVHRR band values			*/
    short	*sptr;		/* pointer into input buffer		*/

    unsigned char inbuf[LINESIZE];/* input buffer contains 1 line of data*/

    Widget	draw_area;	/* the drawing area widget		*/
    Widget	vsb;		/* vertical scroll bar widget		*/
    Window	vsbwindow;	/* vertical scroll bar window		*/
    Window	window;		/* drawing area window			*/

    XEvent	event;		/* event				*/


    /* Get the drawing area widget and image data
    ---------------------------------------------*/
    draw_area = getdrawingarea();
    imageptr = theimage->data;
    display = XtDisplay( draw_area );
    window = XtWindow( draw_area );
    
    /* Get the vertical scroll bar widget,
    its size and position.
    -------------------------------------*/
    sprintf(wstr,"%s.%s.scroll_window.VertScrollBar", WnMAINFORM, WnDRAWFORM );
    vsb = XtNameToWidget( toplevel, wstr );
    XtVaGetValues( vsb,
	XmNsliderSize, &vsbsize,
        XmNmaximum, &vsbmax,
	XmNminimum, &vsbmin,
	XmNvalue,   &vsbpos,
	NULL );
    vsbwindow = XtWindow( vsb );
    
    /* If this is a daytime (descending) pass
    set the scroll bar position to the bottom
    -----------------------------------------*/
    if ( daynightflag == DAY )
        {
	vsbpos = vsbmax - vsbsize;
	XmScrollBarSetValues( vsb, vsbpos, 0, 0, 0, True );
	}
    /* Otherwise set it to the top
    ------------------------------*/
    else
        {
	vsbpos = vsbmin;
	XmScrollBarSetValues( vsb, vsbpos, 0, 0, 0, True );
	}

    /* If the scene is a daytime (ascending) scene, it is displayed
    from the bottom up and the samples are flipped left to right.
    If the scene is a nighttime (descending) scene it is displayed
    from the top down and samples are not flipped.
    --------------------------------------------------------------*/
    /* loop through the lines of the image buffer
    ---------------------------------------------*/
    if ( daynightflag == DAY )
	line = IMGHEIGHT-1;
    else
        line = 0;

    /* While the displayed image is not completely filled
    -----------------------------------------------------*/
    while ( line >= 0 && line < IMGHEIGHT )
	{
	/* If the image display flag is not TRUE immediately
	break from the loop because we are done.
	----------------------------------------------------*/
	if ( !imagedisplay )
	    {
	    write_log(FALSE,"display_image","imagedisplay",
	        "Break from loop because of flag");
	    break;
	    }
	
	/* If the return value is not a full line
	of data immediately break from the loop.
	-----------------------------------------*/
	if (( get_next_line( infd, inbuf )) < LINESIZE )
	    {
	    write_log(FALSE,"display_image","LINESIZE",
	        "Break from loop because of line size");
	    break;
	    }
	
	/* loop through the samples of the image buffer
	-----------------------------------------------*/
	sptr = (short *)&inbuf[VIDEOPTR];
	if ( daynightflag == DAY )
	    sample = IMGWIDTH-1;
	else
	    sample = 0;
	while ( sample >= 0 && sample < IMGWIDTH )
	    {
	    /* Extract the band values from the input buffer.
	    Currently these values are in the range [0..1023].
	    They are stored in big endian format.  Extract
	    the band and scale to the range [0..255].
	    --------------------------------------------------*/
	    for ( i = 1; i <= 5; i++, sptr++ )
		{
		bandvals[i]  = (*sptr & 0x03)   << 6;
		bandvals[i] += (*sptr & 0xff00) >> 10;
		}
		/*
		bandvals[i] = *sptr >> 2;
		*/

/* Note: if we run into efficiency problems
the contrast stretch and clamp code would
be good candidates for removal.
-------------------------------------------*/
	    /* From experience the band 1, 2, 4 and 5 values
	    seldom exceed counts of 1000.  We want to stretch
	    the range a little bit for a brighter display.
	    Band 3 values are generally much higher so we
	    need to compress its range slightly.
	    ------------------------------------------------*/
	    bandvals[1] *= 1.2;
	    bandvals[2] *= 1.1;
	    bandvals[3] *= 0.8;
	    bandvals[4] *= 1.1;
	    bandvals[5] *= 1.1;

	    /* In case a vaue does exceed 255, clamp the stretched
	    values to 255.  Only worry about the displayed bands.
	    ------------------------------------------------------*/
	    if ( bandvals[rgb_bands[0]] > 255 )
	        bandvals[rgb_bands[0]] = 255;
	    if ( bandvals[rgb_bands[1]] > 255 )
	        bandvals[rgb_bands[1]] = 255;
	    if ( bandvals[rgb_bands[2]] > 255 )
	        bandvals[rgb_bands[2]] = 255;

	    /* Build a foreground color by applying the appropriate
	    shift to the bands for an RGB display.
	    ------------------------------------------------------*/
	    foreground =  bandvals[rgb_bands[0]] << shift->red;
	    foreground += bandvals[rgb_bands[1]] << shift->green;
	    foreground += bandvals[rgb_bands[2]] << shift->blue;
	    XPutPixel( theimage, sample, 0, foreground );

	    /* Go to next display sample
	    ----------------------------*/
	    if ( daynightflag == DAY )
		sample--;
	    else
		sample++;

	    /* Go to the next sample in the input buffer
	    inptr += SUBSAMPLE_INTERVAL;
	    --------------------------------------------*/
	    sptr += NUMBANDS * (SUBSAMPLE_VALUE - 1);
	    }  /* END for sample loop */
	
	/* Display the line created.  The image is actually displayed
	as a background pixmap.  The put image copies the line just
	created into the pixmap.  The copy area copies the pixmap to
	the background and generates the expose event.
	-------------------------------------------------------------*/
	XPutImage( display, pixmap, gc, theimage,
	    0, 0, 0, line, IMGWIDTH, 1 );
	XCopyArea( display, pixmap, window, gc, 
	    0, line, IMGWIDTH, 1, 0, line );

	/* Go to the next line of the display
	-------------------------------------*/
	if ( daynightflag == DAY )
	    {
	    line--;
	    /* If we are at the top of the display
	    move the scroll bar down one line
	    --------------------------------------*/
	    if ( line < IMGHEIGHT - DISPHEIGHT )
	        {
		vsbpos--;
		XmScrollBarSetValues( vsb, vsbpos, 0, 0, 0, True );
		}
	    }
	else
	    {
	    line++;
	    /* If we are at the bottom of the display
	    move the scroll bar up one line
	    -----------------------------------------*/
	    if ( line > DISPHEIGHT )
	        {
		vsbpos++;
		XmScrollBarSetValues( vsb, vsbpos, 0, 0, 0, True );
		}
	    }
	
	/* Dispatch any pending events except button events related
	to the scroll bar.  Accepting scroll bar events while in
	this loop causes the displayed image to scroll erratically.
	??? This only handles one event at a time
	if ( XPending( display ) > 0 )
	??? Also want to try with real-time system
	while ( XPending( display ) > 0 )
	-----------------------------------------------------------*/
	while ( XPending( display ) > 0 )
	    {
	    XNextEvent( display, &event );
	    if ( event.type == ButtonPress )
	        {
		/* Prevent user interaction with scroll bar */
	        if ( event.xany.window == vsbwindow )
		    /* throw event away */;
		else
		    XtDispatchEvent( &event );
		}
	    else
		XtDispatchEvent( &event );
	    }
	}  /* END for line loop */
    
    /* If we got here because the display height was reached
    the image display flag will not have been set to FALSE.
    Returning without setting the flag will put us into a
    display loop, redisplaying the input file until signaled.
    Therefore, unconditionally set the flag to FALSE.
    --------------------------------------------------------*/
    imagedisplay = FALSE;

    return;
    }  /* END display_image */
