/*******************************************************************************
FUNCTIONS:	get_next_line
		readline
		openinput

PURPOSE:	input file functions

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: infile.c,v $
Revision 1.5  2002/08/26 12:12:47  dlloyd
Added code to toss button press events on the scroll bar.  Corrected the
if statement that checks header size -- added equal sign.

Revision 1.4  2002/07/18 17:39:30  dlloyd
Periodic check in during development

Revision 1.2  2002/02/28 12:58:07  dlloyd
Slight modifications to work with acquman.

Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include "shoacqu.h"


/* Global variables referenced
------------------------------*/
extern int imagedisplay;
extern Display     *display;
extern Widget       toplevel;


/* variables local to this file
-------------------------------*/
static long file_size;
static Window vsbwindow;


/*******************************************************************************
FUNCTION:	openinput

PURPOSE:	Get the next line of data to display.

INPUT:		input file name

OUTPUT:		none

RETURNS:	int - input file descriptor
*******************************************************************************/
int openinput
    (
    char *filename		/* IN - input file name			*/
    )
    {
    char buf[HEADER];		/* buffer to read header into		*/
    char        wstr[STRLEN];   /* widget name string                   */

    int	infd;			/* input file descriptor		*/
    int nbytes;			/* number of bytes read from input file	*/

    struct stat statbuf;	/* file statistics buffer		*/

    /* Get the vertical scroll bar widget
    -------------------------------------*/
    sprintf(wstr,"%s.%s.scroll_window.VertScrollBar", WnMAINFORM, WnDRAWFORM );
    vsbwindow = XtWindow( XtNameToWidget( toplevel, wstr ));

    /* initialize the static local file size
    ----------------------------------------*/
    file_size = 0;

    /* Attemp to open the input file, sleep on failure.
    This function will block here until the file exists
    or the pass duration is exceeded and the parent
    signals to continue ( imagedisplay flag is FALSE).
    It may be several seconds before Acquire creates
    the file and there is no reason to do hundreds of
    opens, hence the sleep.
    --------------------------------------------------*/
    while((( infd = open( filename, O_RDONLY)) == -1 ) && imagedisplay )
	{
	XEvent      event;

	socket_select();
	while ( XPending( display ))
	    {
	    XNextEvent( display, &event );
	    XtDispatchEvent( &event );
	    }
	sleep(1);
	}
    
    /* After successfully opening the file wait until
    the full header exists.  Read and discard the
    header.  Again the function will block here until
    the header is stripped or the imagesdisplay is set
    to FALSE by a message forom the parent.
    -------------------------------------------------*/
    while( imagedisplay )
        {
	/* It may be several seconds before Acquire writes
	the header and there is no reason to read until
	it exists.  Same comment about the sleep as above.
	--------------------------------------------------*/
	fstat( infd, &statbuf );
	if ( statbuf.st_size >= HEADER )
	    {
	    nbytes = 0;
	    while (( nbytes < HEADER ) && imagedisplay )
		nbytes += read( infd, buf, HEADER - nbytes );
	    file_size = nbytes;
	    break;
	    }
	else
	    {
	    XEvent      event;

	    socket_select();
	    while ( XPending( display ))
		{
		XNextEvent( display, &event );
		XtDispatchEvent( &event );
		}
	    sleep(1);
	    }
	}
    
    return infd;
    }  /* END openinput */

    

/*******************************************************************************
FUNCTION:	get_next_line

PURPOSE:	Get the next line of data to display.

INPUT:		input file descriptor

OUTPUT:		buffer of image data

RETURNS:	int number of bytes read
*******************************************************************************/
int get_next_line
    (
    int		infd,		/* IN - input file descriptor		*/
    char	*buf		/* OUT - buffer of image data to display*/
    )
    {
    int		bytes_read = 0;	/* number of bytes read from file	*/
    int		i;		/* lopp control variable		*/


    /* Based on the subsampling interval, read
    and discard the appropriate number of lines,
    except, of course, the last line read.
    -------------------------------------------*/
    for ( i = 0; i < SUBSAMPLE_VALUE; i++ )
        {
	bytes_read = readline( infd, buf );
	if ( bytes_read == 0 )
	    break;
	}

    return bytes_read;
    }  /* END get_next_line */


/*******************************************************************************
FUNCTION:	readline

PURPOSE:	Read one line (LINESIZE) of data from file.  It first checks
		that the file has at least LINESIZE more bytes in it since
		the last read before reading from the file.  This function
		blocks until the file is read from or the global imagedisplay
		variable is set to FALSE.

INPUT:		file descriptor

OUTPUT:		buffer of data

RETURNS:	number of bytes read
*******************************************************************************/
int readline
    (
    int		infd,		/* IN - input file descriptor		*/
    char	*buf		/* OUT - buffer of image data to display*/
    )
    {
    int		nbytes;		/* number of bytes read			*/
    struct stat	statbuf;	/* input file statistics		*/
    XEvent	event;		/* event				*/

    nbytes = 0;

    /* As long as the imagedisplay flag is TRUE continue
    to try to read from the file.  The flag will be set
    to FALSE when a SIGTERM is received from the parent.
    ----------------------------------------------------*/
    while( imagedisplay )
        {
	/* See if there is a message from the parent
	--------------------------------------------*/
	socket_select();

	/* We don't want this routine eating up cpu cycles
	unnecessarily waiting for data.  In addition we
	don't want to block in a read or we'll be blocked
	if Acquire quits writing to the file.  This process
	could hang indefinitely if that were to happen.
	---------------------------------------------------*/
	fstat( infd, &statbuf );
	if ( statbuf.st_size > file_size + LINESIZE )
	    {
	    nbytes = read( infd, &buf[nbytes], LINESIZE );
	    file_size += nbytes;
	    break;
	    }
	else
	    {
	    /* Dispatch any pending events
	    if we haven't read any data.
	    ------------------------------*/
	    while ( XPending( display ))
		{
		XNextEvent( display, &event );
		if ( event.type == ButtonPress )
		    {
		    /* Prevent user interaction with scroll bar */
		    if ( event.xany.window == vsbwindow )
		        /* throw event away */;
		    else
			XtDispatchEvent( &event );
		    }
		else
		    XtDispatchEvent( &event );
		}
	    }
	}  /* END while */
    
    /* If the imagedisplay flag has been set to
    FALSE set nbytes to 0 so that the calling
    routine will return immediately.
    -------------------------------------------*/
    if ( !imagedisplay )
	{
	write_log(FALSE,"readline","imagedisplay","display flag set to FALSE");
        nbytes = 0;
	}
    return nbytes;
    }  /* END readline */
