/******************************************************************************
FUNCTION:	read_socket

PURPOSE:	To read from the socket and set the appropriate variables

INPUT:		socket descriptor
		RGB band array

OUTPUT:		imagedisplay flag
		if applicable sets:
		    input file name
		    next start time
		    day/night flag
		    rgb bands
		    stop time

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: read_socket.c,v $
Revision 1.7  2008/06/03 17:48:17  adaps
Accomodated changes in acquman that is sending the satellite type now

Revision 1.6  2002/08/26 13:02:21  dlloyd
Added support for displaying a stop time.

Revision 1.5  2002/08/26 12:13:48  dlloyd
Removed temporary diagnostic message.

Revision 1.4  2002/07/19 13:56:26  dlloyd
Minor changes and diagnostics

Revision 1.3  2002/07/18 17:39:30  dlloyd
Periodic check in during development

Revision 1.1  2002/05/15 13:26:07  dlloyd
Added this as a combined get_next_scene() and get_next_time() function.

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include "shoacqu.h"

extern int imagedisplay;

int read_socket
    (
    int sock_fd,                /* IN - socket descriptor               */
    int rgb_bands[]		/* OUT - band combinations for display	*/
    )
    {
    char buf[STRLEN];		/* buffer used to read from socket	*/
    char *strptr;		/* pointer into buf			*/
    char start_time[9];		/* start time to display HH:MM:SS	*/
    char stop_time[9];		/* stop time to display HH:MM:SS	*/
    char am_pm_flag[3];		/* AM PM indicator to display		*/
    char file_name[STRLEN];	/* input file name			*/
    char display_string[STRLEN];/* full time display string		*/
    char dnflag;		/* flag indicating a day or night pass	*/
    char error_msg[10];		/* short error message from socket	*/

    int  msglength;		/* message length expected from socket	*/
    int  num_read;		/* number of bytes read from socket	*/
    int  n;			/* number of bytes returned from recv	*/

    Widget w = getlabelw();     /* the text label widget for the display*/

    XmString text;              /* text to display in the label         */

    struct pollfd poll_struct[2];
    int status;


    /* If there is no socket connection return
    ------------------------------------------*/
    if ( sock_fd < 0 )
        return -1;
    
    /* Poll the socket connection
    poll_struct[0].fd = sock_fd;
    poll_struct[0].events = POLLIN;
    poll_struct[0].revents = 0;
    status = poll( poll_struct, 1, 10 );
    if ( status > 0 )
	if ( poll_struct[0].revents & (POLLHUP|POLLNVAL|POLLERR))
	    {
	    write_log(TRUE,"read_socket","poll","Lost socket connection");
	    exit(1);
	    }
    -----------------------------*/
	    
    /* Read the first three bytes of the message.  This contains the
    ASCII representation of the message length expected.  Convert the
    string to an integer count.
    -----------------------------------------------------------------*/
    num_read = recv( sock_fd, &buf, 3, MSG_DONTWAIT );
    if ( num_read != 3 )
	{
	/* Poll the socket connection
	-----------------------------*/
	poll_struct[0].fd = sock_fd;
	poll_struct[0].events = POLLIN;
	poll_struct[0].revents = 0;
	status = poll( poll_struct, 1, 10 );
	if ( status > 0 )
	    if ( poll_struct[0].revents & (POLLHUP|POLLNVAL|POLLERR))
		{
		write_log(TRUE,"read_socket","poll","Lost socket connection");
		exit(1);
		}
	return num_read;
	}
    buf[num_read] = '\0';
    sscanf( buf, "%d", &msglength );

    /* Read the rest of the message from the socket.
    ------------------------------------------------*/
    num_read = recv( sock_fd, &buf, msglength, MSG_DONTWAIT );
    if ( num_read != msglength )
	{
	/* Poll the socket connection
	-----------------------------*/
	poll_struct[0].fd = sock_fd;
	poll_struct[0].events = POLLIN;
	poll_struct[0].revents = 0;
	status = poll( poll_struct, 1, 10 );
	if ( status > 0 )
	    if ( poll_struct[0].revents & (POLLHUP|POLLNVAL|POLLERR))
		{
		write_log(TRUE,"read_socket","poll","Lost socket connection");
		exit(1);
		}
	return num_read;
	}
    buf[num_read] = '\0';

    /* Parse the message.  The expected configuration is:
        
    or
        -poll
    or
	-next <HH:MM XM>
    or
        -error <8-char or less error message>
    or
	-file <file name> -dnflag [dDnN] -rgb <b1 b2 b3> -stop <HH:MM XM>
    -----------------------------------------------------*/
    strptr = &buf[0];
    while (( strptr = strchr( strptr, '-' )) != NULL )
        {
	if ( strncmp( strptr, "-next", 5 ) == 0 )
	    {
            char stype[10], ssat[4];
	    sscanf( strptr, " %*s %s %s %s %s", stype, ssat, 
                start_time, am_pm_flag );
	    sprintf( display_string, "Next acquisition %s %s at %s %s",
	        stype, ssat, start_time, am_pm_flag );
/*
	    sprintf( display_string, "Next acquisition at %s %s",
	        start_time, am_pm_flag );
*/
	    text = XmStringCreateLocalized( display_string );
	    XtVaSetValues( w, XmNlabelString, text, NULL );
	    XmStringFree( text );
	    imagedisplay = FALSE;
	    }
	
	else if ( strncmp( strptr, "-error", 6 ) == 0 )
	    {
	    sscanf( strptr, " %*s %s", error_msg );
	    sprintf( display_string, "An Error occurred: %s", error_msg );
	    text = XmStringCreateLocalized( display_string );
	    XtVaSetValues( w, XmNlabelString, text, NULL );
	    XmStringFree( text );
	    imagedisplay = FALSE;
	    }

	else if ( strncmp( strptr, "-file", 5 ) == 0 )
	    {
	    sscanf( strptr, " %*s %s", file_name );
	    putinfile( file_name );
	    imagedisplay = TRUE;
	    }

	else if ( strncmp( strptr, "-dnflag", 7 ) == 0 )
	    {
	    sscanf( strptr, " %*s %c", &dnflag );
	    if ( dnflag == 'N' || dnflag == 'n' )
	        putdaynightflag( NIGHT );
	    else
	        putdaynightflag( DAY );
	    }
	
	else if ( strncmp( strptr, "-rgb", 4 ) == 0 )
	    {
	    sscanf( strptr, " %*s %d %d %d", &rgb_bands[0], &rgb_bands[1],
	        &rgb_bands[2] );
	    }

	else if ( strncmp( strptr, "-poll", 5 ) == 0 )
	    /* No op, parent is simply checking the connection */;

	else if ( strncmp( strptr, "-stop", 5 ) == 0 )
	    {
	    sscanf( strptr, " %*s %s %s", stop_time, am_pm_flag );
	    putstoptime( stop_time, am_pm_flag );
	    }

	else
	    {
	    write_log( FALSE, "read_socket", "what?", strptr );
	    }
	strptr++;
	}  /* END while */
    write_log( FALSE, "read_socket", "received", buf );

    return num_read;
    }  /* END read_socket */
