/******************************************************************************r
FUNCTION:	directory

PURPOSE:	Return at most maxfiles files matching the infile specification
		on the silo.  If lockflg is true, ignore those files that are
		locked.


Version  Programmer       Date   Reason
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          04/94  Development

PROJECT:  ADAPS
*******************************************************************************/
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <time.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "queue.h"
#include "silo.h"


extern int errno;	/* Error status from system                           */

static long fnamecmp();	/* Function for comparing file structures             */
static long rootcmp();	/* Function for comparing root file names             */


struct FNAME
    {
    char name[CMLEN];
    char location[5];
    };




int directory
(
    struct PARBLK *vblock,/* IN - TAE parameter block                         */
    char *infile,	/* IN - silo file to get the location for             */
    long lockflg,	/* IN - Flag to ignore locked files                   */
    long maxfiles	/* IN - Maximum number of files to return             */
)
{
char cmd_files[CMLEN];	/* Command to get find all files matching infile      */
char cmd_locks[CMLEN];	/* Command to find all lock files in the directory    */
char dir[CMLEN];	/* Directory portion of a file name                   */
char ext[CMLEN];	/* Extension portion of a file name                   */
char location[20];	/* File location DISK or TAPE                         */
char *lockname;		/* Ptr to a lock file in the queue                    */
char name[CMLEN];	/* Name of a file                                     */
char permission[20];	/* Permission string for a file (ie. from ls -l)      */
char root[CMLEN];	/* Root portion of a file name                        */
char str[CMLEN];	/* Temporary string                                   */
char **strings;		/* Array of string pointers to return to TAE          */

int status;		/* Status variable for closing pipes                  */

long i;			/* Loop control                                       */
int nfiles;		/* The number of files to be returned to TAE          */
long nonfatal = 1;	/* Nonfatal error flag                                */
long position;		/* Position of a file name in the queue               */
long qfiles;		/* Queue of file names matching infile                */
long qlocks;		/* Queue of lock files                                */

FILE *fp;		/* Ptr to a pipe containing file names, sizes         */

struct FNAME *fileptr;	/* Ptr to a file name structure in the queue          */
struct FNAME tmpstruct;	/* Temporary FNAME structure for use with c_qfind()   */




/* Split the input file specification into a directory and file name
--------------------------------------------------------------------*/
c_splitname(infile, dir, root, ext);
if (strlen(ext))
    {
    strcat(root, ".");
    strcat(root, ext);
    }
else if (strlen(root) == 0)
    strcpy(root, "*");


/* Create the system command for retrieving the matching files and lock files
-----------------------------------------------------------------------------*/
if (strlen(dir))
    {
    sprintf(cmd_files, "udir -d '%s' '%s'", dir, root);
    sprintf(cmd_files, "ssh edcsns7 ls -l '%s/%s'", dir, root);
    sprintf(cmd_locks, "udir -d '%s' '*.lock'", dir);
    sprintf(cmd_locks, "ssh edcsns7 ls -l '%s/*.lock'", dir);
    }
else
    {
    sprintf(cmd_files, "udir '%s'", root);
    sprintf(cmd_locks, "udir '*.lock'");
    sprintf(cmd_files, "ssh edcsns7 ls -l '%s'", root);
    sprintf(cmd_locks, "ssh edcsns7 ls -l '*.lock'");
    }
/*
strcat(cmd_files," | awk '($5 == \"DK\") || ($5 == \"AR\") {print $1,$5,$11}'");
strcat(cmd_locks," | awk '($5 == \"DK\") || ($5 == \"AR\") {print $11}'");
*/


/* Create two queues to hold the file names and lock files
----------------------------------------------------------*/
qfiles = c_qcreate("ascending", "dups");
qlocks = c_qcreate("ascending", "dups");
if ((qfiles == -1) || (qlocks == -1))
    {
    c_errmsg("Queue operation error", "silo-queue", &nonfatal);
    return(E_FAIL);
    }


/* Open a pipe to the command that returns the matching file names
------------------------------------------------------------------*/
fp = popen(cmd_files, "r");
if (fp == NULL)
    {
    c_errmsg("Error returned from system call", "silo-syscall", &nonfatal);
    sprintf(str, "System errno = %d", errno);
    c_errmsg(str, "silo-errno", &nonfatal);
    return(E_FAIL);
    }


/* Read all file names matching infile and place them in the queue
------------------------------------------------------------------*/
while (fscanf(fp, "%9s%s %*s %*s %*s %*s %*s %*s %s", permission, location, name) == 3)
    {
    /* If the file is a symbolic link call get_location() to follow the link
       and retrieve the location of the target file.  If the file is not a
       symbolic link, convert the location from T to TAPE and from - to DISK.
       The T and - are from the last element of the permission argument
       eg: rw-r--r-- or rw-r--r-T
    --------------------------------------------------------------------------*/
    if (permission[0] == 'l')
	{
	sprintf(str, "%s%s", dir, name);
	get_location(str, FALSE, str, location);
	}
    else if (strcmp(location, "T") == 0)
	strcpy(location, "TAPE");
    else
	strcpy(location, "DISK");


    /* Allocate a structure to store the file name and location
    -----------------------------------------------------------*/
    fileptr = (struct FNAME *)malloc(sizeof(struct FNAME));
    strcpy(fileptr->name, name);
    strcpy(fileptr->location, location);
    if (c_qinsert(qfiles, fileptr, fnamecmp))
	{
	c_errmsg("Queue operation error", "silo-queue", &nonfatal);
	c_qdestroy(qfiles);
	pclose(fp);
	wait(&status);
	return(E_FAIL);
	}
    }
pclose(fp);
wait(&status);


/* If only unlocked files are to be returned, find all lock files
-----------------------------------------------------------------*/
if (lockflg && c_qsize(qfiles))
    {
    /* Open a pipe to the command that returns all lock files
    ---------------------------------------------------------*/
    fp = popen(cmd_locks, "r");
    if (fp == NULL)
	{
	c_errmsg("Error returned from system call", "silo-syscall", &nonfatal);
	sprintf(str, "System errno = %d", errno);
	c_errmsg(str, "silo-errno", &nonfatal);
	c_qdestroy(qfiles);
	return(E_FAIL);
	}


    /* Read all file names matching infile and place them in the queue
    ------------------------------------------------------------------*/
    while (fscanf(fp, "%s", str) == 1)
	{
	lockname = (char *)malloc(strlen(str) + 1);
	strcpy(lockname, str);
	if (c_qinsert(qlocks, lockname, strcmp))
	    {
	    c_errmsg("Queue operation error", "silo-queue", &nonfatal);
	    c_qdestroy(qfiles);
	    c_qdestroy(qlocks);
	    pclose(fp);
	    wait(&status);
	    return(E_FAIL);
	    }
	}
    pclose(fp);
    wait(&status);
    }


/* For every lock file in the queue delete each file that it locks.
-------------------------------------------------------------------*/
while (c_qsize(qfiles) && c_qsize(qlocks))
    {
    /* Remove the first lock file from the queue
    --------------------------------------------*/
    if (c_qremove(qlocks, 1, (void **)&lockname))
	{
	c_errmsg("Queue operation error", "silo-queue", &nonfatal);
	c_qdestroy(qfiles);
	c_qdestroy(qlocks);
	return(E_FAIL);
	}
    

    /* Store the name of the lock file in a temporary
       FNAME structure for the c_qfind() function.
    -------------------------------------------------*/
    strcpy(tmpstruct.name, lockname);
    

    /* Remove every file from the queue of file names that has
       the same root file name as the current lock file.
    ----------------------------------------------------------*/
    do  {
	if (c_qfind(qfiles, &tmpstruct, (void **)&fileptr, rootcmp, &position))
	    {
	    c_errmsg("Queue operation error", "silo-queue", &nonfatal);
	    c_qdestroy(qfiles);
	    c_qdestroy(qlocks);
	    return(E_FAIL);
	    }
	if (fileptr != NULL)
	    c_qdelete(qfiles, position);
	} while (fileptr != NULL);
    }
    

/* Determine the number of files to return
------------------------------------------*/
if (c_qsize(qfiles) < maxfiles)
    nfiles = c_qsize(qfiles);
else
    nfiles = maxfiles;


/* Create an array of pointers to the file names to be returned
---------------------------------------------------------------*/
if (nfiles)
    {
    /* Allocate an array of string ptrs to return nfiles file names
    ---------------------------------------------------------------*/
    strings = (char **)calloc(nfiles, sizeof(char *));
    if (strings == NULL)
	{
	c_errmsg("Error allocating memory", "silo-alloc", &nonfatal);
	c_qdestroy(qfiles);
	c_qdestroy(qlocks);
	return(E_FAIL);
	}
    

    /* Retrieve pointers to each of the strings to be returned
    ----------------------------------------------------------*/
    i = 0;
    c_qfirst(qfiles, (void **)&fileptr);
    while (i < nfiles)
	{
	strings[i++] = fileptr->name;
	c_qnext(qfiles, (void **)&fileptr);
	}


    /* Return the file names to TAE
    -------------------------------*/
    if (q_string(vblock, "OUTFILES", nfiles, strings, P_UPDATE) != SUCCESS)
	{
	c_errmsg("Error sending variable to TAE", "silo-tae", &nonfatal);
	free(strings);
	c_qdestroy(qfiles);
	c_qdestroy(qlocks);
	return(E_FAIL);
	}
    

    /* Retrieve pointers to each of the strings to be returned
    ----------------------------------------------------------*/
    i = 0;
    c_qfirst(qfiles, (void **)&fileptr);
    while (i < nfiles)
	{
	strings[i++] = fileptr->location;
	c_qnext(qfiles, (void **)&fileptr);
	}


    /* Return the file locations to TAE
    -----------------------------------*/
    if (q_string(vblock, "LOCATION", nfiles, strings, P_UPDATE) != SUCCESS)
	{
	c_errmsg("Error sending variable to TAE", "silo-tae", &nonfatal);
	free(strings);
	c_qdestroy(qfiles);
	c_qdestroy(qlocks);
	return(E_FAIL);
	}


    /* Return the number of files to TAE
    ------------------------------------*/
    if ((q_intg(vblock, "NFILES", 1, &nfiles, P_UPDATE) != SUCCESS) ||
	(q_out(vblock) != SUCCESS))
	{
	c_errmsg("Error sending variable to TAE", "silo-tae", &nonfatal);
	free(strings);
	c_qdestroy(qfiles);
	c_qdestroy(qlocks);
	return(E_FAIL);
	}
    }


/* Free up all of the allocated memory in the queues
----------------------------------------------------*/
free(strings);
c_qdestroy(qfiles);
c_qdestroy(qlocks);

return(E_SUCC);
}






static long fnamecmp (file1, file2)

    void *file1;	/* IN - Ptr to a FNAME structure                      */
    void *file2;	/* IN - Ptr to a FNAME structure                      */
{
struct FNAME *f1;	/* Ptr to a FNAME structure                           */
struct FNAME *f2;	/* Ptr to a FNAME structure                           */


f1 = (struct FNAME *)file1;
f2 = (struct FNAME *)file2;

return(strcmp(f1->name, f2->name));
}



static long rootcmp (file1, file2)

    void *file1;	/* IN - Ptr to a FNAME structure                      */
    void *file2;	/* IN - Ptr to a FNAME structure                      */
{
char root1[CMLEN];	/* Root portion of a file name                        */
char root2[CMLEN];	/* Root portion of a file name                        */
char str[CMLEN];	/* Directory portion of a file name                   */

struct FNAME *f1;	/* Ptr to a FNAME structure                           */
struct FNAME *f2;	/* Ptr to a FNAME structure                           */


f1 = (struct FNAME *)file1;
f2 = (struct FNAME *)file2;

c_splitname(f1->name, str, root1, str);
c_splitname(f2->name, str, root2, str);

return(strcmp(root1, root2));
}
