/******************************************************************************r
FUNCTION:	get_location

PURPOSE:	Get the location of a file.  Every file on the silo will be on
		the disk cache or on tape.  If a symbolic link is encountered,
		follow the link to the target file and return the status of
		that file.

		If the waitflg is TRUE, do not return until the file is located
		on disk.


Version  Programmer       Date   Reason
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          04/94  Development

PROJECT:  ADAPS
*******************************************************************************/
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "silo.h"


extern int errno;	/* Error status from system                           */


int get_location
(
    char *infile,	/* IN  - silo file to get the location for            */
    long waitflg,	/* IN  - Flag to wait until the status becomes disk   */
    char *target,	/* OUT - Name of file the returned location is for    */
    char *location	/* OUT - Location of the file (disk, tape)            */
)
{
char access[CMLEN];	/* List of file access codes from /bin/ls -l          */
char dir[CMLEN];	/* Directory portion of a file name                   */
char ext[CMLEN];	/* Extension portion of a file name                   */
char filename[CMLEN];	/* Name of the file to which infile points it a link  */
char root[CMLEN];	/* Root portion of a file name                        */
char str[CMLEN];	/* Temporary string                                   */
char syscmd[CMLEN];	/* Command sent to the operating system               */
char *ptr;

int status;		/* Status variable for closing pipes                  */

long i;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */
long nread;		/* Number of strings read by fscanf()                 */

FILE *fp;		/* Ptr to a pipe containing file names, sizes         */




/* Copy infile to target and assume we are getting the location of infile.
   If infile is a symbolic link, target will be reset on the next level of
   recursion.
--------------------------------------------------------------------------*/
strcpy(target, infile);


/* Split the input file specification into a directory and file name
--------------------------------------------------------------------*/
c_splitname(infile, dir, root, ext);
if (strlen(ext))
    {
    strcat(root, ".");
    strcat(root, ext);
    }


/* Create the system command for retrieving the file status
-----------------------------------------------------------*/
if (strlen(dir))
    sprintf(syscmd, "udir -d '%s' '%s'", dir, root);
else
    sprintf(syscmd, "udir '%s'", root);
/*
strcat(syscmd, " | awk '($5 == \"DK\") || ($5 == \"AR\") {print}'");
*/


/* Determine the status of the file
-----------------------------------*/
i = 0;
do  {
    /* If this is not the first time through, sleep
       before rechecking the status of the file.
    -----------------------------------------------*/
    if (i == 1)
	c_errmsg("Waiting for file to be cached", " ", &nonfatal);
    if (i++ > 0)
	sleep(20);


    /* Open a pipe to the udir command
    ----------------------------------*/
    fp = popen(syscmd, "r");
    if (fp == NULL)
	{
	c_errmsg("Error returned from system call", "silo-syscall", &nonfatal);
	sprintf(str, "System errno = %d", errno);
	c_errmsg(str, "silo-errno", &nonfatal);
	strcpy(location, "unknown");
	return(E_FAIL);
	}


    /* Read the file access information, location, and name
    -------------------------------------------------------*/
    nread = fscanf(fp, "%9s%s %*s %*s %*s %*s %*s %*s %*s %s",
	access, location, filename);
    if (nread != 3)
	{
	pclose(fp);
	wait(&status);
	c_errmsg("Error retrieving file status", "silo-status", &nonfatal);
	strcpy(location, "unknown");
	return(E_FAIL);
	}
    

    /* Udir now returns the filename with a full path.  Remove the path
    -------------------------------------------------------------------*/
    ptr = strrchr( filename, '/' );
    ptr++;
    strcpy( filename, ptr );


    /* Ensure filename returned by the udir command is
       the same as the root file name from infile.
    --------------------------------------------------*/
    if (strcmp(root, filename))
	{
	pclose(fp);
	wait(&status);
	strcpy(location, "unknown");
	c_errmsg("Error retrieving file status", "silo-status", &nonfatal);
	return(E_FAIL);
	}
    

    /* If the desired file is a link, follow the link
       and get the location of the target file.
    -------------------------------------------------*/
    if (access[0] == 'l')
	{
	nread = fscanf(fp, "%s %s", access, filename);
	if ((strcmp(access, "->") != 0) || (nread != 2))
	    {
	    pclose(fp);
	    wait(&status);
	    strcpy(location, "unknown");
	    c_errmsg("Error retrieving file status", "silo-status", &nonfatal);
	    return(E_FAIL);
	    }
	
	pclose(fp);
	wait(&status);

	sprintf(str, "%s linked to %s", root, filename);
	c_errmsg(str, " ", &nonfatal);
	return(get_location(filename, waitflg, target, location));
	}
    

    /* Close the pipe
    -----------------*/
    pclose(fp);
    wait(&status);

    } while (0);
/* Until the uftp routines support reprting the actual location
   we will fall through this routine once and report based on
   the value of location regardless of actual location.
    } while ((strcmp(location,"T") == 0) && waitflg);
*/


/* Convert the location code to DISK or TAPE
--------------------------------------------*/
if (strcmp(location, "T") == 0)
    strcpy(location, "TAPE");
else
    strcpy(location, "DISK");

return(E_SUCC);
}
