/*******************************************************************************
NAME:    SILO

PURPOSE: Read and write files to the silo file system.


Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
T. Baltzer       04/93  Initial development
J. Fenno         04/94  Added the LINK and WAIT subcommands.
J. Fenno	 08/95	Added the -l option for following the DATAPATH
			environment variable.
B. Olson         06/97  Added multiple file writes to the WRITE subcommand.
B. Olson         08/97  Added multiple files for the READ & DELETE subcommands.
D. Lloyd         06/01  Remove trailing /'s from WRITE command outdir to
                        prevent hangs from occurring CR4997.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Since the Unitree Central File Manager (Silo File System) is
  only supported on UNIX systems, this routine is not to be
  run under VMS.

PROJECT:  ADAPS
*******************************************************************************/
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "silo.h"


#define NTRIES  3	/* Number of tries before exiting with error status   */

extern int errno;	/* Error status from system                           */

void main( void )
{
char cache[4];		/* Cache flag                                         */
char dir[10][CMLEN];	/* Directory portion of a file name                   */
char errkey[CMLEN];	/* Standard output error key                          */
char errmsg[ERRLEN];	/* Standard output message                            */
char ext[10][CMLEN];	/* Extension portion of a file name                   */
char infile[10][CMLEN];	/* Input file specification                           */
char link[CMLEN];	/* Host name of the symbolic link to create           */
char location[CMLEN];	/* Location of a file on the silo (disk or tape)      */
char outdir[CMLEN];	/* Output directory specification (las  name)         */
char outfile[10][CMLEN];/* Output name for completion message                 */
char *ptr;		/* Temporary pointer to a string                      */
char root[10][CMLEN];	/* Root portion of a file name                        */
char str[10][CMLEN];	/* Temporary string                                   */
char subcmd[10];	/* SILO subcommand                                    */
char syscmd[CMLEN];	/* Command sent to the operating system               */
char target[CMLEN];	/* Target file if infile is a symbolic link           */
char umgetcmd[CMLEN];	/* The umget command with optional -l argument        */

long ii,jj;		/* Counting variable				      */
long j;			/* Count number of tries before failing		      */
long lockflg;		/* Flag to ignore locked files                        */
long maxfiles;		/* Maximum number of file names to return             */
long mode = 511;	/* Directory mode				      */
long stdlun;		/* Standard output                                    */
long sysstat;		/* System return status                               */
long nfiles;            /* Number of input files                              */

struct PARBLK vblock;	/* TAE parameter block                                */



/* Initialize LAS
-----------------*/
c_inlas(&vblock, &stdlun);


/* Get the user input parameters
--------------------------------*/
if ( getpar(&vblock, subcmd, infile, outdir, cache, link,
       &lockflg, &maxfiles, &nfiles) != E_SUCC )
    c_lasmsg(TRUE,"silo-fatal","Fatal error encountered");


/* Create the appropriate system command
----------------------------------------*/
if (strcmp(subcmd,"delete") == 0)
    {
    for (ii=0;ii<nfiles;ii++)
      { 
      /* Split the input file specification into a directory and file name
      --------------------------------------------------------------------*/
      c_splitname(infile[ii], dir[ii], root[ii], ext[ii]);
      if (strlen(ext[ii]))
	sprintf(infile[ii], "%s.%s", root[ii], ext[ii]);
      else
	sprintf(infile[ii], "%s", root[ii]);


      /* Create an error message and key
      ----------------------------------*/
      sprintf(errmsg, "Error deleting %s on silo", infile[ii]);
      sprintf(errkey, "silo-ddiag");


      /* Create the system command for deleting the files
      ---------------------------------------------------*/
      if (strlen(dir[ii]))
	sprintf(syscmd, "umdel -v -d '%s' '%s'", dir[ii], infile[ii]);
      else
	sprintf(syscmd, "umdel -v '%s'", infile[ii]);
      for (j = 0; j < NTRIES; j++)
        {
        if (sysstat = system(syscmd))
          {
          c_lasmsg(FALSE, "silo-syscall", "Error returned from system call");
          c_lasmsg(FALSE, "silo-systatus","Return status from system call = %d",
                    sysstat);
          c_lasmsg(FALSE, "silo-errno", "System errno = %d", errno);

          if (j + 1 == NTRIES)
            {
            c_lasmsg(FALSE, errkey, "%s %s", errmsg, "giving up");
            c_lasmsg(TRUE, "silo-fatal", "Fatal error encountered");
            }
          else
            {
            c_lasmsg(FALSE, errkey, "%s %s", errmsg, "retry");
            system("sleep 5");
            }
          }
        else
          break;
        }
      }
    }
else if (strcmp(subcmd,"dir") == 0)
    {
    if (directory(&vblock, infile[0], lockflg, maxfiles) != E_SUCC)
	c_lasmsg(TRUE, "silo-fatal", "Fatal error encountered");


    /* Issue a success message and complete
    ---------------------------------------*/
    ii = 0;
    c_lasmsg(FALSE, " ", "Successfully retrieved %s on silo", infile);
    c_complt("SILO", outfile, &ii);
    }
else if (strcmp(subcmd,"link") == 0)
    {
    /* Split the input file specification into a directory and file name
    --------------------------------------------------------------------*/
    c_splitname(link, dir[0], root[0], ext[0]);
    if (strlen(ext[0]))
	sprintf(link, "%s.%s", root[0], ext[0]);
    else
	strcpy(link, root[0]);


    /* Create an error message and key
    ----------------------------------*/
    sprintf(errmsg, "Error creating link %s on silo", link);
    sprintf(errkey, "silo-ldiag");


    /* Create the system command for creating the link
    --------------------------------------------------*/
    if (strlen(dir[0]))
	sprintf(syscmd, "umput -v -d '%s' -l '%s' '%s'", dir, infile, link);
    else
	sprintf(syscmd, "umput -v -l '%s' '%s'", infile, link);
    }
else if (strcmp(subcmd,"read") == 0)
    {
    for (ii=0;ii<nfiles;ii++)
      { 

      /* Change into the destination directory
      ----------------------------------------*/
      if (!isdir(outdir))
        if (mkdir(outdir,mode) != 0) 
	  c_lasmsg(TRUE, "silo-mkdir", "Error creating specified directory");

      if (chdir(outdir))
	c_lasmsg(TRUE, "silo-chdir", "Error moving to specified directory");

      /* Split the input file specification into a directory and file name.  
       NOTE:
	   Due to an error in the silo software, when a symbolic link is cached
	   the location of the "real" file is not updated properly.  The
	   function get_location() can be used to follow symbolic links and
	   return the name and current location of the "real" file.  By caching
	   this file, instead of the actual symbolic link, the silo will update
	   the location correctly.  get_location() cannot handle wildcard
	   characters.  If this becomes a problem, we will have to not allow
	   wildcards when caching or create a function that will expand the
	   wildcards and cache each file independently.  Unitree is aware of
	   the problem and will be coming out with a fix.
    --------------------------------------------------------------------------*/
      if (!wildcard(infile[ii]) && (strcmp(cache, "-sv") == 0))
	{
	if (get_location(infile[ii], FALSE, target, location) != E_SUCC)
	    c_lasmsg(TRUE, "silo-link", "Error following symbolic link");
	c_splitname(target, dir[ii], root[ii], ext[ii]);
	}
      else
	c_splitname(infile[ii], dir[ii], root[ii], ext[ii]);
      if (strlen(ext[ii]))
	{
	strcat(root[ii], ".");
	strcat(root[ii], ext[ii]);
	}


      /* Create an error message and key
      ----------------------------------*/
      sprintf(errmsg, "Error reading %s from silo", root[ii]);
      sprintf(errkey, "silo-rdiag");


    /* If $DATAPATH is defined, add the -l option to the umget command.  This
       will tell umget that it should place the files in the appropriate
       directory defined by $DATAPATH.

       **** This may need fixing if anyone actually uses it ****
    -------------------------------------------------------------------------*/
      /*
      ptr = (char *) getenv("DATAPATH");
      if (ptr == NULL)
  	strcpy(umgetcmd, "umget");
      else
  	strcpy(umgetcmd, "umget -l");
      */


      /* Create the system command for reading the files.
        This depends on an NFS mount being available, but 
        that seems to be present wherever the compositing
        is done.
      --------------------------------------------------*/
      
      sprintf( syscmd, "cp %s .", infile[ii] );
      if (sysstat = system(syscmd))
        {
        c_lasmsg(FALSE, "silo-syscall", "Error returned from system call");
        c_lasmsg(FALSE, "silo-systatus","Return status from system call = %d",
                  sysstat);
        c_lasmsg(FALSE, "silo-errno", "System errno = %d", errno);

        c_lasmsg(FALSE, errkey, "%s %s", errmsg, "giving up");
        c_lasmsg(TRUE, "silo-fatal", "Fatal error encountered");
        
        }
      }
    }
else if (strcmp(subcmd,"wait") == 0)
    {
    /* Create an error message and key
    ----------------------------------*/
    sprintf(errmsg, "Error getting status for %s on silo", infile);
    sprintf(errkey, "silo-sdiag");


    /* Get the location of the desired file
    ---------------------------------------*/
    if (get_location(infile[0], TRUE, target, location) != E_SUCC)
	c_lasmsg(TRUE, errkey, errmsg);


    /* Use just the file name for the informational message
    -------------------------------------------------------*/
    c_splitname(infile[0], dir[0], root[0], ext[0]);
    if (strlen(ext[0]))
	{
	strcat(root[0], ".");
	strcat(root[0], ext[0]);
	}
    

    /* Issue a success message and complete
    ---------------------------------------*/
    ii = 0;
    c_lasmsg(FALSE, " ", "File %s is on silo cache disk", root);
    c_complt("SILO", outfile, &ii);
    }
else if (strcmp(subcmd,"write") == 0)
    {
    /* make sure /hsm is in the directory name
    ---------------------------------------------*/
    if (strlen(outdir))
        siloname(outdir);
    for (ii=0;ii<nfiles;ii++)
      {
      /* Create an error message and key
      ----------------------------------*/
      c_splitname(infile[ii], dir[ii], root[ii], ext[ii]);
      if (strlen(ext[ii]))
	  sprintf(errmsg, "Error writing %s.%s to silo", root[ii], ext[ii]);
      else
	  sprintf(errmsg, "Error writing %s to silo", root[ii]);
      sprintf(errkey, "silo-wdiag");


      /* Create the system command for writing the files
      --------------------------------------------------*/
      if (strlen(outdir))
	  {
	  while ( outdir[strlen(outdir)-1] == '/' )
	      outdir[strlen(outdir)-1] = '\0';
	  sprintf(syscmd, "umput -v -m -d '%s' '%s'", outdir, infile[ii]);
	  }
      else
	  sprintf(syscmd, "umput -v '%s'", infile[ii]);

    /* Make the necessary system call.  Try NTRIES times before
       terminating if a failure occurs.  Sleep between tries.
       Before terminating with an error status, attempt to delete 
       the partially written file.
       ------------------------------------------------------------*/
      for (j = 0; j < NTRIES; j++)
        {
        if (sysstat = system(syscmd))
          {
          c_lasmsg(FALSE, "silo-syscall", "Error returned from system call");
          c_lasmsg(FALSE, "silo-systatus", "Return status from system call = %d",
              sysstat);
          c_lasmsg(FALSE, "silo-errno", "System errno = %d", errno);
          c_lasmsg(FALSE, errkey, "%s %s", errmsg, "giving up");
          if (strlen(outdir))
            if (strlen(ext[0]))
              sprintf(syscmd, "umdel -v -d '%s' '%s.%s'", outdir, root[ii],
                      ext[ii]);
            else
              sprintf(syscmd, "umdel -v -d '%s' '%s'", outdir, root[ii]);
          else
            if (strlen(ext[ii]))
              sprintf(syscmd, "umdel -v '%s.%s'", root[ii], ext[ii]);
            else
              sprintf(syscmd, "umdel -v '%s'", root[ii]);
          system(syscmd);
          c_lasmsg(TRUE, "silo-fatal", "Fatal error encountered");
          }
        else
          break;
        }
      }
    }


/* Make the necessary system call.  Try NTRIES times before
   terminating if a failure occurs.  Sleep between tries.
------------------------------------------------------------------*/
if ((strcmp(subcmd,"dir") == 0) || (strcmp(subcmd,"link") == 0))
  for (j = 0; j < NTRIES; j++)
    {
    if (sysstat = system(syscmd))
      {
      c_lasmsg(FALSE, "silo-syscall", "Error returned from system call");
      c_lasmsg(FALSE, "silo-systatus", "Return status from system call = %d",
	  sysstat);
      c_lasmsg(FALSE, "silo-errno", "System errno = %d", errno);

      if (j + 1 == NTRIES)
	 {
	 c_lasmsg(FALSE, errkey, "%s %s", errmsg, "giving up");
	 c_lasmsg(TRUE, "silo-fatal", "Fatal error encountered");
	 }
      else
	 {
	 c_lasmsg(FALSE, errkey, "%s %s", errmsg, "retry");
	 system("sleep 5");
	 }
      }
    else
      break;
    }


/* Issue an informational message and determine the file created
----------------------------------------------------------------*/
if ((strcmp(subcmd, "read") == 0) && (strcmp(cache, "-sv") == 0))
    {
    for (jj=0;jj<nfiles;jj++)
      {
      ii = 0;
      sprintf(str[0], "Successfully staged %s from silo", root[jj]);
      strcpy(outfile[jj], root[jj]);
      c_lasmsg(FALSE, " ", str[0]);
      }
    }
else if (strcmp(subcmd, "read") == 0)
    {
    ii = nfiles;
    for (jj=0;jj<nfiles;jj++)
      {
      sprintf(str[0], "Successfully read %s from silo", root[jj]);
      strcpy(outfile[jj], root[jj]);
      c_lasmsg(FALSE, " ", str[0]);
      }
    }
else if (strcmp(subcmd, "write") == 0)
    {
    ii=nfiles;
    for (jj=0;jj<nfiles;jj++)
      {
      c_splitname(infile[jj], dir[jj], root[jj], ext[jj]);
      if (strlen(ext[jj]))
	{
	strcat(root[jj], ".");
	strcat(root[jj], ext[jj]);
	}
      sprintf(str[0], "Successfully wrote %s to silo", root[jj]);
      
      strcpy(outfile[jj], root[jj]);
      c_lasmsg(FALSE, " ", str[0]);
      }
    }
else if (strcmp(subcmd, "link") == 0)
    {
    ii = 1;
    sprintf(str[0], "Successfully created link %s on silo", link);
    strcpy(outfile[0], link);
    c_lasmsg(FALSE, " ", str[0]);
    }
else if (strcmp(subcmd, "delete") == 0)
    {
    ii = 0;
    for (jj=0;jj<nfiles;jj++)
      {
      sprintf(str[0], "Successfully deleted %s on silo", infile[jj]);
      strcpy(outfile[jj], infile[jj]);
      c_lasmsg(FALSE, " ", str[0]);
      }
    }

/* Issue a completion message
-----------------------------*/
c_complt("SILO", outfile, &ii);
}
