/******************************************************************************
FUNCTION:	timecor_predict

PURPOSE:	To predict a time correction from a series of previous timcor
		entries.

INPUT:		At least 20 lines from a previous time correction series.

OUTPUT:		The input time correction series and two predicted values, one
		from a linear fit the other from a multifit.

USAGE:		tail -20 $ADAPSTABLES | timecor_predict

EXAMPLE:
02/10/05 15:55:44.000 -0033
02/10/05 17:35:22.000 -0033
02/10/05 19:16:43.000 -0033
02/11/05 15:33:33.000 -0033
02/11/05 17:12:17.000 -0033
02/11/05 18:53:29.000 -0032
02/12/05 15:11:59.000 -0032
02/12/05 16:49:36.000 -0032
02/12/05 18:30:09.000 -0032
02/13/05 16:26:51.000 -0031
02/13/05 18:07:01.000 -0031
02/14/05 16:04:17.000 -0031
02/14/05 17:44:02.000 -0031
02/14/05 19:25:44.000 -0031
02/15/05 15:42:07.000 -0031
02/15/05 17:20:58.000 -0031
02/15/05 19:02:18.000 -0031
02/16/05 15:20:07.000 -0031
02/16/05 16:58:20.000 -0031
02/16/05 18:39:01.000 -0031
Linear fit
02/17/05 18:39:01.000 -0030
Multi fit
02/17/05 18:39:01.000 -0031

COMPILE NOTE:	This funciton uses the public GSL library.  Compile as follows:

	cc timecor_predict.c -o timecor_predict \
	-L/sgs20/csb/software/build/public/gsl -lgsl \
	-I/sgs20/csb/software/build/public

    or:

	cc timecorrfit.c -g -o timecorrfit -L/home/dlloyd/lib -lgsl \
	-I/home/dlloyd/include -lgslcblas -static -lm

The second method will allow to compile on edclxw26 and run on edclxs1.

******************************************************************************/
#include <stdio.h>
#include <time.h>
#include <gsl/gsl_fit.h>
#include <gsl/gsl_multifit.h>

#define NUM  20

int mfit( double *x, double *y, int n, double *c0, double *c1, double *c2 );

int main( int argc, char *argv[] )
{
time_t times[NUM+1];
int    corrs[NUM+1];
int i;
char datestr[9];
char timestr[13];
int correction;
struct tm timeptr;
struct tm *gmt;
int status;

double x[NUM], y[NUM];
double c0;
double c1;
double c2;
double cov00;
double cov01;
double cov11;
double sumsq;



for ( i = 0; i < NUM; i++ )
    {
    status = scanf( "%s %s %d", datestr, timestr, &correction );
    if ( status != 3 )
        {
	printf( "Error reading from input\n" );
	return 1;
	}
    sscanf( datestr, "%d/%d/%d", &timeptr.tm_mon, &timeptr.tm_mday,
                                 &timeptr.tm_year );
    timeptr.tm_mon--;
    timeptr.tm_year += 100;

    sscanf( timestr, "%d:%d:%d", &timeptr.tm_hour, &timeptr.tm_min,
                                 &timeptr.tm_sec );
    
    times[i] = mktime( &timeptr );
    x[i] = (double)times[i];
    corrs[i] = correction;
    y[i] = (double)corrs[i];
    /*
    printf( " %d  %d\n", (int)times[i], corrs[i] );
    */
    printf( "%s %s %05d\n", datestr, timestr, correction );
    }
times[NUM] = times[NUM-1] + (3600 * 24);

status = gsl_fit_linear( x, 1, y, 1, NUM, &c0, &c1, &cov00, &cov01,
                         &cov11, &sumsq );
if ( status )
    {
    printf( "Error calculating fit\n" );
    return 1;
    }

corrs[NUM] = (int)(c0 + c1*times[NUM]);

/*
printf( "%d %d\n", times[NUM], corrs[NUM] );
printf( "%lf %lf\n", c0, c1 );
*/

/*
gmt = gmtime( &times[NUM] );
*/
gmt = localtime( &times[NUM] );
printf( "Linear fit\n" );
printf( "%02d/%02d/%02d %02d:%02d:%02d.000 %05d\n", gmt->tm_mon+1,
        gmt->tm_mday, gmt->tm_year-100, gmt->tm_hour, gmt->tm_min,
	gmt->tm_sec, corrs[NUM]
      );

mfit( x, y, NUM, &c0, &c1, &c2 );
corrs[NUM] = (int)(c0 + c1*times[NUM] + c2*times[NUM]*times[NUM] );
printf( "Multi fit\n" );
printf( "%02d/%02d/%02d %02d:%02d:%02d.000 %05d\n", gmt->tm_mon+1,
        gmt->tm_mday, gmt->tm_year-100, gmt->tm_hour, gmt->tm_min,
	gmt->tm_sec, corrs[NUM]
      );



return 0;
}


int mfit( double *x, double *y, int n, double *c0, double *c1, double *c2 )
{
int i;
double xi, yi, ei, chisq;
gsl_matrix *X, *cov;
gsl_vector *Y, *w, *c;
gsl_multifit_linear_workspace *work;

X = gsl_matrix_alloc( n, 3 );
Y = gsl_vector_alloc( n );
w = gsl_vector_alloc( n );
c = gsl_vector_alloc( 3 );
cov = gsl_matrix_alloc( 3, 3 );

for ( i = 0; i < n; i++ )
    {
    xi = x[i];
    yi = y[i];
    ei = 1.0;

    gsl_matrix_set( X, i, 0, 1.0 );
    gsl_matrix_set( X, i, 1, xi );
    gsl_matrix_set( X, i, 2, xi*xi );
    gsl_vector_set( Y, i, yi );
    gsl_vector_set( w, i, 1.0/(ei*ei));
    }

work = gsl_multifit_linear_alloc( n, 3 );
gsl_multifit_wlinear( X, w, Y, c, cov, &chisq, work );
gsl_multifit_linear_free( work );

*c0 = gsl_vector_get( c, 0 );
*c1 = gsl_vector_get( c, 1 );
*c2 = gsl_vector_get( c, 2 );

return 0;
}
