/*************************************************************************
FUNCTION:	calcdelta

PURPOSE:	Calculate a delta time correction based on the current
		measured delta and past history.

                Note: This function was taken from old vax code in
                acquire.c hence the cryptic naming conventions and
                the scarcity of comments.

IN:		measured delta
		satellite ID

OUT:		no output values
		past delta file is updated

RETURNS:	calculated delta

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: calcdelta.c,v $
Revision 1.4  2003/12/24 19:07:54  dlloyd
Modified to move one file at a time in case errors occur.  On errors the
image is no longer moved to error.  Fixed a bug in th savedelta routine.

Revision 1.3  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "timecorr.h"

/* local prototype
------------------*/
void gauss
    (
    double matrix[ORDER+1][ORDER+1],
    double solut[ORDER+1],
    int order
    );

int calcdelta
    (
    int measured_delta,		/* IN - measured delta value		*/
    long startime,		/* IN - start time in seconds		*/
    int satid			/* IN - satellite ID, 15, 16 17 etc.	*/
    )
    {
    FILE *old_deltas;		/* file descriptor for old delta values	*/

    int entries;		/* number of entries in file		*/
    int oldest;			/* index pointer to oldest entry in file*/
    int satime;
    int millierror;
    int i, j;			/* loop control				*/
    int row, col;		/* loop control				*/
    int order = ORDER;
    
    char old_deltas_name[STRLEN];/* file name of old deltas		*/
    char *adapstables;		/* ADAPSTABLES environment variable	*/
    char message[STRLEN];

    double matrix[ORDER+1][ORDER+1];
    double solut[ORDER+1];
    double sumx2n[2*ORDER+1];
    double power;


    /* Initialize arrays
    --------------------*/
    for ( j = 0; j <= ORDER; j++ )
        solut[j] = 0.0;
    for ( j = 0; j <= ORDER*2; j++ )
        sumx2n[j] = 0.0;

    
    /* Open the old deltas file
    ---------------------------*/
    adapstables = getenv( "ADAPSTABLES" );
    if ( adapstables == NULL )
	{
        write_log(FALSE,"calcdelta","getenv","Unable to get ADAPSTABLES");
	return MEAN_ERROR;
	}
    sprintf( old_deltas_name, "%s/noaa%02d.delta", adapstables, satid );
    old_deltas = fopen( old_deltas_name, "r" );
    if ( !old_deltas )
        {
	write_log(FALSE,"calcdelta","fopen","Unable to open deltas file");
	return MEAN_ERROR;
	}
    
    /* Get the number of entries and oldest entry
    ---------------------------------------------*/
    fscanf( old_deltas, "%d %d\n", &entries, &oldest );


    /* Make sure there are enough entries
    -------------------------------------*/
    if ( entries <= ORDER )
        {
	write_log(FALSE,"calcdelta","entries","Too few entries in delta file");
	order = entries - 1;
	/*
	return TOOFEWENTRIES;
	*/
	}
    sprintf( message, "Using regression of order %d", order );
    write_log(FALSE,"calcdelta","Information",message);

    /* Load up the arrays from the file
    -----------------------------------*/
    for ( i = 0; i < entries; i++ )
        {
	fscanf( old_deltas, "%d %d\n", &satime, &millierror );

	power = 1;
	for ( j = 0; j <= order; j++ )
	    {
	    sumx2n[j] += power;
	    solut[j]  += millierror * power;
	    power     *= ( satime - startime );
	    }
	for ( ; j <= order*2; j++ )
	    {
	    sumx2n[j] += power;
	    power     *= ( satime - startime );
	    }

	for ( row = 0; row <= order; row++ )
	    for ( col = 0; col <= order; col++ )
	        matrix[row][col] = sumx2n[row+col];
	}  /* END for i */

    fclose( old_deltas );

    gauss( matrix, solut, order );
    sprintf( message, "Calculated delta %d", (int)rint( solut[0] ));
    write_log(FALSE,"calcdelta","information",message);
    return (int)rint( solut[0] );
    }  /* END calcdelta */




void gauss
    (
    double matrix[ORDER+1][ORDER+1],
    double solut[ORDER+1],
    int order
    )
    {
    int i, j, k;

    for ( i = 0; i <= ORDER; i++ )
        {
	solut[i] /= matrix[i][i];
	for( j = order; j >= i; j-- )
	    matrix[i][j] /= matrix[i][i];
	for( j = 0; j <= order; j++ )
	    {
	    if( j != i )
		{
		solut[j] -= solut[i] * matrix[j][i];
		for( k = order; k >= i; k-- )
		    matrix[j][k]-=matrix[i][k]*matrix[j][i];
		}  /* END if */
	    }  /* END for j */
	}  /* END for i */
    }  /* END gauss */

