/***********************************************************************
FUNCTION:	timecorr

PURPOSE:	Create a new average time correction value entry in
		the $ADAPSTABLES/noaaXX.timcor file, where XX is the
		satellite number, using the sceneid.tstamp file,
		where sceneid is a standard 16-digit ADAPS scene ID.

INPUT:		input file name as: /path/sceneid.sf2

OUTPUT:		new entry in $ADAPSTABLES/noaaXX.timcor

RETURNS:	0 on success
		-1 on failure

PROJECT:	ADAPS, AVHRR Aquisition System

$Log: timecorr.c,v $
Revision 1.12  2007/12/13 20:16:54  dlloyd
Added code to make a link of the file

Revision 1.11  2007/10/02 19:59:53  dlloyd
Changed destinations and added an install target

Revision 1.10  2004/11/09 14:01:13  dlloyd
Additional mods for daylight saving time change

Revision 1.9  2004/10/13 11:58:55  dlloyd
Added a bit more logic to check for discrepancies between calculated and
measured deltas in an attempt to bypass the problems encountered with change
to and from daylight savings time.

Revision 1.8  2003/12/24 19:07:54  dlloyd
Modified to move one file at a time in case errors occur.  On errors the
image is no longer moved to error.  Fixed a bug in th savedelta routine.

Revision 1.7  2003/08/07 19:11:32  dlloyd
Trying to prevent problems when the clock resets.  This is a test.

Revision 1.6  2003/04/18 12:29:22  dlloyd
Put in a sanity check for reasonable value (1000ms)

Revision 1.5  2003/04/18 11:59:00  dlloyd
Only save calculated time corr entry in delta file.

Revision 1.4  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "timecorr.h"



int main( int argc, char *argv[] )
    {
    char  tstamp_file_name[STRLEN];	/* time stamp file name		*/
    char  timecorr_file_name[STRLEN];	/* time correction file name	*/
    char  lock_file_name[STRLEN];	/* lock file name		*/
    char *substr_ptr1;			/* pointer into string		*/
    char *substr_ptr2;			/* pointer into string		*/
    char  timecorr_entry[STRLEN];	/* time correction entry	*/
    char  root_filename[STRLEN];

    FILE *tstamp_file;		/* time stamp file descriptor		*/
    FILE *timecorr_file;	/* time correction file descriptor	*/
    FILE *lock_file;		/* lock file descriptor			*/

    int  time_correction;	/* time correction from process()	*/
    long  satime;

    struct stat lockfilestat;	/* status structure for lock file	*/

    int   month;		/* variables used to extract the date	*/
    int   day;			/* and time from the input sceneid and	*/
    int   year;			/* to create the time correction	*/
    int   hour;			/* entry				*/
    int   minute;
    int   second;
    int   calc_delta;
    int   satid;
    int   seconds;


    /* Insure that command line argument was specified
    --------------------------------------------------*/
    if ( argc < 3 )
        {
	write_log(TRUE,"main","Error","No arguments specified on command line");
	write_log(TRUE,"main","Usage","timecorr -file <file name>");
	return -1;
	}
    else
        {
	/* A full file specification is expected
	----------------------------------------*/
	if (( substr_ptr1 = strrchr( argv[2], '.' )) == NULL )
	    {
	    write_log(TRUE,"main","Error","Incorrect file specification");
	    return -1;
	    }
	else
	    /* Temporarily NULL terminate the argument */
	    *substr_ptr1 = '\0';

	/* Create the time stamp and lock file names
	--------------------------------------------*/
	strcpy( tstamp_file_name, argv[2] );
	strcat( tstamp_file_name, ".tstamp" );
	strcpy( lock_file_name, argv[2] );
	strcat( lock_file_name, ".lock" );
	if (( substr_ptr2 = strrchr( argv[2], '/' )) == NULL )
	    {
	    /* Assume it's just a file spec without path
	    --------------------------------------------*/
	    substr_ptr2 = &argv[2][0];
	    }
	/* Otherwise point to the beginning of the root file name
	---------------------------------------------------------*/
	else
	    substr_ptr2++;
	strcpy( root_filename, substr_ptr2 );
	/* Put the argument back the way it was */
	*substr_ptr1 = '.';

	/* Get the satellite ID
	-----------------------*/
	sscanf( root_filename, "%*c%*c%2d", &satid );
	}
    
    /* Move only the image file so that errors in timecorr
    don't cause problems with production flow.  Also link
    the file to ADAPSARC/sfl/sf2
    -----------------------------------------------------*/
    move1file( argv[2], "ADAPSACQ" );

    /* Check for the lock file.  If it's still there wait
    for up to 20 minutes.  If it isn't unlocked by then,
    print an error message and continue.
    -----------------------------------------------------*/
    for ( seconds = 0; seconds < TWENTYMINUTES; seconds++ )
        {
	if (( stat( lock_file_name, &lockfilestat )) == 0 )
	    sleep(1);
	else
	    break;
	}
    if ( seconds >= TWENTYMINUTES )
        write_log(FALSE,"main","lock-file","Exceded wait, continuing");
    else
	{
	/* Now re-lock the file
	-----------------------*/
	lock_file = fopen( lock_file_name, "w" );
	if ( !lock_file )
	    write_log(FALSE,"main","lockfile","Unable to create lock file");
	else
	    {
	    fprintf( lock_file, "locked by time_corr\n" );
	    fclose( lock_file );
	    }
	}  /* END else */


    /* Open the time stamp file
    ---------------------------*/
    tstamp_file = fopen( tstamp_file_name, "r" );
    if ( !tstamp_file )
        {
	write_log(FALSE,"main","tstamp","Unable to open time stamp file");
	movefiles( argv[2], "ADAPSERR" );
	write_log(TRUE,"main","errmov","files moved to ADAPSERR");
	return -1;
	}


    /* Determine the time correction file and open it
    -------------------------------------------------*/
    substr_ptr1 = getenv( "ADAPSTABLES" );
    if ( substr_ptr1 == NULL )
        {
	write_log(FALSE,"main","getenv","Unable to get ADAPSTABLES");
	movefiles( argv[2], "ADAPSERR" );
	write_log(TRUE,"main","errmov","files moved to ADAPSERR");
	fclose( tstamp_file );
	return -1;
	}
    sprintf( timecorr_file_name, "%s/noaa%c%c.timcor",
             substr_ptr1, root_filename[2], root_filename[3] );
    timecorr_file = fopen( timecorr_file_name, "a" );
    if ( !timecorr_file )
        {
	write_log(FALSE,"main","Error","Unable to open time correction file");
	movefiles( argv[2], "ADAPSERR" );
	write_log(TRUE,"main","errmov","files moved to ADAPSERR");
	fclose( tstamp_file );
	return -1;
	}
    

    /* Now process the input file
    -----------------------------*/
    time_correction = process( tstamp_file, &satime );
    if ( time_correction != MEAN_ERROR )
        time_correction += PROPDELAY;

    /* Calculate the delta using the delta file
    -------------------------------------------*/
    satime = makeseconds( root_filename );
    calc_delta = calcdelta( time_correction, satime, satid );

    /* Okay, it seems that it takes a while for the frame sync clock to get
    in line when there is a change to and from daylight saving time.  There is
    no reason to reset the time series when this occurs.  I'm changing the
    following logic to exit if both values are unreasonable.  If the measured
    value is unreasonable, as happens after the time change the calulated
    value will be used.
    -------------------------------------------------------------------------*/
    if ( abs( calc_delta ) > REASONABLE_VALUE &&
         abs( time_correction ) > REASONABLE_VALUE )
        {
	sprintf( timecorr_entry, "%02d/%02d/%02d %02d:%02d:%02d.000 %05d",
		 month, day, year, hour, minute, second, calc_delta );
	write_log(FALSE,"main","create-entry",timecorr_entry);
	sprintf( timecorr_entry, "%02d/%02d/%02d %02d:%02d:%02d.000 %d",
		 month, day, year, hour, minute, second, time_correction );
	write_log(FALSE,"main","measured-delta",timecorr_entry);
	write_log(TRUE,"main","Error","Unreasonable value encountered");
	remove( lock_file_name );
	movefiles( argv[2], "ADAPSERR" );
	return -1;
	}
    else if ( abs( time_correction ) > REASONABLE_VALUE &&
              calc_delta != TOOFEWENTRIES )
        {
	savedelta( satid, satime, calc_delta, calc_delta );
	}
    else
        {
	savedelta( satid, satime, time_correction, calc_delta );
	}

    /* Create the entry and append to time correction file
    Entry format:
        MM/DD/YY hh:mm:ss.000 CCCCC
    
    Where: MM, DD and YY are 2-digit month, day and year;
           hh, mm and ss are 2-digit hour, minute and second;
	   CCCCC is the 5-digit time correction

    Also write it to the log file.
    ---------------------------------------------------------*/
    if ( calc_delta != TOOFEWENTRIES )
	{
	sscanf ( &root_filename[4], "%02d%02d%02d%02d%02d%02d", &month, &day,
		 &year, &hour, &minute, &second );
	sprintf( timecorr_entry, "%02d/%02d/%02d %02d:%02d:%02d.000 %05d",
		 month, day, year, hour, minute, second, calc_delta );
	if ( fseek( timecorr_file, 0L, SEEK_END ) == 0 )
	    fprintf( timecorr_file, "%s\n", timecorr_entry );
	else
	    write_log(TRUE,"main","fseek","Failure seeking to end of file");
	write_log(FALSE,"main","create-entry",timecorr_entry);
	sprintf( timecorr_entry, "%02d/%02d/%02d %02d:%02d:%02d.000 %d",
		 month, day, year, hour, minute, second, time_correction );
	write_log(FALSE,"main","measured-delta",timecorr_entry);
	}
    else
        write_log(FALSE,"main","create-entry","Unable to determine time corr");


    /* Close the files, remove the lock file
    and move the files to $ADAPSING.
    ----------------------------------------*/
    fclose( tstamp_file );
    fclose( timecorr_file );
    remove( lock_file_name );
    movefiles( argv[2], "ADAPSMISC" );
    write_log(FALSE,"main","move-files","files moved to ADAPSING");

    
    /* Write a successful completion message
    ----------------------------------------*/
    write_log(FALSE,"main","SUCCESS","time_corr completed successfully");
    return 0;
    }
