/* These routines were stolen from the old vax acquire

$Log: weighted_mean.c,v $
Revision 1.2  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

-------------------------------------------------------*/
#include <math.h>
#include "timecorr.h"


/*************************************************************************
FUNCTION:	calc_stdev

PURPOSE:	Calculate the mean and standard deviation of a set of
		values.  The first and last index of the array determine
		the set of values from the input array to use in the
		calculation.  An error is returned if last - first is
		less than one.

INPUT:		integer array 
		integer first: index of first element to use
		integer last: index of last element to use

OUTPUT:		float mean: the mean of the elements

RETURN:		float, standard deviation of elements
		float MEAN_ERROR, if only one element is specified
*************************************************************************/
float calc_stdev( int satmil[], int first, int last, float *mean )
{
float	variance;	/* variance of the elements			*/
float	sum;		/* sum of the elements				*/

int	i;		/* loop index					*/
int	num_elements;	/* number of elements in the set		*/

/* Determine the number of elements in the set
and return an error if num_elements <= 1
----------------------------------------------*/
num_elements = last - first + 1;
if ( num_elements <= 1 )
    return MEAN_ERROR;

/* Compute the mean of the element set
--------------------------------------*/
sum = 0.0;
for ( i = first; i <= last; i++ )
    sum += (float)satmil[i];
*mean = sum / (float)num_elements;

/* Compute the variance of the element set
-----------------------------------------*/
variance = 0.0;
for ( i = first; i <= last; i++ )
    variance = (satmil[i] - *mean) * (satmil[i] - *mean);
variance /= (float)num_elements - 1.0;

/* Return the standard deviation
--------------------------------*/
return sqrt( variance );
}  /* END calc_stdev */


/*************************************************************************
FUNCTION:       weighted_mean

PURPOSE:        Determine the mean of a sorted array but only using the
                most common values.  The reason for this is to eliminate
                the effects of noise or loss of data on the mean.  The
                weighted mean returned will be from the largest set of
                values with a standard deviation less than or equal to
                STDDEV_THRESH.  The final set of values will have at
                least MIN_MEAN_COUNT members.

INPUTS:         int satmil[SAMPLES] -- sorted in ascending order

OUTPUTS:        none

RETURNS:        weighted mean -- if one was found
                MEAN_ERROR    -- if mean couldn't be determined

ALGORITHM:
    Determine stddev of all values in the set
    Set number of values in each set to SAMPLES-1
    While the stddev is greater than STDDEV_THRESH and number of values in
      set is greater than or equal to MIN_MEAN_COUNT
        Determine the number of sets with numval values
        Loop through the numval sets: from first = 0 to numsets
            determine index of last value in set
            calculate stdev and mean of current set
            if the current stdev is less than stdev
                let stdev = current stdev
                let mean = current mean
            end-if
        end-loop
        subtract one from numval
    End while

    if the number of values is less than MIN_MEAN_COUNT
        let mean = MEAN_ERROR
    return mean

*************************************************************************/
int weighted_mean( int satmil[] )
{
float   current_mean;   /* mean of current set of values                */
float   current_stdev;  /* standard deviation of current set of values  */
float   mean;           /* mean of set of values with lowest stdev      */
float   stdev;          /* lowest standard deviation                    */

int     first;          /* index of first value in set                  */
int     numsets;        /* number of sets in satmil with numval values  */
int     numval;         /* number of values in the set                  */
int     last;           /* index of last value in set                   */


/* Get the standard deviation and mean of the entire data set
-------------------------------------------------------------*/
stdev = calc_stdev( satmil, 0, SAMPLES-1, &mean );

/* Calculate standard deviation and mean of succesively smaller sets
of values until the number of values in the data set are less than the
minimum (MIN_MEAN_COUNT) or a stndard deviation is calculated that is
less than or equal to the maximum (STDDEV_THRESH).
----------------------------------------------------------------------*/
numval = SAMPLES-1;
while ( numval >= MIN_MEAN_COUNT && stdev > STDDEV_THRESH )
    {
    /* Determine the number of data sets of numval size.
    ---------------------------------------------------*/
    numsets = SAMPLES - numval + 1;

    /* Loop through each data set calculating its stdev and mean
    -----------------------------------------------------------*/
    for ( first = 0; first < numsets; first++ )
        {
        last = first + numval - 1;
        current_stdev = calc_stdev( satmil, first, last, &current_mean );

        /* If the current stdev is less than any previous
        one, save it and the mean
        -------------------------------------------------*/
        if ( current_stdev < stdev && current_stdev != MEAN_ERROR )
            {
            stdev = current_stdev;
            mean = current_mean;
            }
        }  /* END for */

    /* Decrement the number of values in a data set
    -----------------------------------------------*/
    numval--;
    }  /* END while */

/* If we got out of the while loop with less than the
minimum number of values return the error value.
-----------------------------------------------------*/
if ( numval < MIN_MEAN_COUNT )
    return MEAN_ERROR;

return (int)( mean + 0.5 );
}  /* END weighted_mean */



/***********************************************************************
FUNCTION:	insertion_sort

PURPOSE:	Sort a small ( n < 50 ) integer array.  The input array
		is overwritten with sorted values.

INPUT:		integer array: array to be sorted
		integer n: number of elements in array

OUTPUT:		input array is overwritten with sorted values

RETURNS:	none
***********************************************************************/
void insertion_sort( int array[], int n )
{
int	i, j;	/* loop index variables		*/
int	temp;	/* temporary swapping variable	*/

for ( j = 1; j < n; j++ )
    {
    temp = array[j];
    i = j - 1;
    while ( i >= 0 && array[i] > temp )
        {
	array[i+1] = array[i];
	i--;
	}
    array[i+1] = temp;
    }
}  /* END insertion_sort */
