/*******************************************************************************
FUNCTION:	write_log

PURPOSE:	Write a message to the standard log file for timecorr.
		The message is written to the file as follows:
		    key  date  time  code  message
		
		If the fatal flag is TRUE an additional fatal error message
		is written.

USAGE NOTE:	Although there is are no restrictions to the lengths of the 
		message key and code, their actual output strings will be
		limited to 16 characters.

INPUT:		TRUE/FALSE fatality flag          
		error/message key                       
		error/message code              
		message string                    

OUTPUT:		message to log file

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: write_log.c,v $
Revision 1.3  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include "timecorr.h"


void write_log
    (
    int  flag,			/* IN - TRUE/FALSE fatality flag	*/
    				/*      TRUE = issue fatal error message*/
    char *key,			/* IN - error/message key		*/
    char *code,			/* IN - error/message code		*/
    char *msg			/* IN - message string			*/
    )
    {
    FILE *ouf;			/* output file descriptor of log file	*/

    char *adapslog;		/* pointer to system variable		*/
    char date_time[20];		/* string representation of time	*/
    char filename[STRLEN];	/* log file name			*/
    char errmsg[STRLEN];	/* error message string			*/

    struct tm *tm_time;		/* pointer to the time structure	*/

    time_t now;			/* seconds since beginning of unix time	*/


    /* Open the output file, on error just return because
    we don't want file output errors to halt the process
    -----------------------------------------------------*/
    adapslog = getenv( "ADAPSLOG" );
    if ( !adapslog )
	{
	/* output this message somehow
	------------------------------*/
	sprintf( errmsg, "Error getting ADAPSLOG environment variable" );
        return;
	}
    sprintf( filename, "%s/timecorr.log", adapslog );
    ouf = fopen( filename, "a" );
    if ( !ouf )
	{
	/* output this message somehow
	------------------------------*/
	sprintf( errmsg, "Error opening file: %s", filename );
        return;
	}

    /* get the current time and convert to a viewable string
    --------------------------------------------------------*/
    now = time( 0 );
    tm_time = localtime( &now );
    sprintf( date_time, "%02d/%02d/%02d  %02d:%02d:%02d",
        tm_time->tm_mon+1, tm_time->tm_mday, tm_time->tm_year % 100,
	tm_time->tm_hour, tm_time->tm_min, tm_time->tm_sec );
    
    /* Print the message, limit the key and
    code to 16 characters for readability.
    ---------------------------------------*/
    fprintf( ouf, "%-16.16s  %s  %-16.16s  %s\n", key, date_time, code, msg );

    /* if the fatal flag is set also issue a fatal message
    ------------------------------------------------------*/
    if ( flag == TRUE )
        {
	fprintf( ouf, "%-16.16s  %s  %-16.16s  Fatal error encountered\n",
	    key, date_time, code );
	}
    
    fclose( ouf );
    }  /* END write_log */
