/******************************************************************************

SUBROUTINE FONT2
	
PURPOSE:
	Subroutine FONT2 is a routine which computes the "font" for
	a single line of a text string.  It is assumed that the calling
	program will do all of the error checking (such as making sure
	the length of the text will stay within the bounds of the out-
	put buffer).  The complete set of printable characters is shown
	below. Any character not in this list will appear as a question
	mark.
		  SP  !  "  #  $  %  &  '  (  )  *  +  ,  -  .  /
		   0  1  2  3  4  5  6  7  8  9  :  ;  <  =  >  ?
		   @  A  B  C  D  E  F  G  H  I  J  K  L  M  N  O
		   P  Q  R  S  T  U  V  W  X  Y  Z  [  \  ]  ^  _
		   `  a  b  c  d  e  f  g  h  i  j  k  l  m  n  o
		   p  q  r  s  t  u  v  w  x  y  z  {  |  }  ~  


	The "font" used by this routine has a dimension of 32 x 58 
	array of points.  The input parameter of SF will allow the 
	"font" to be scaled.  The scaling factor must be an integer,
	otherwise the characters will not be proportioned correctly.

	The "font" uses the last four columns as character separation.
	Also, the last eleven lines are used for line separation.
	   
ARGUMENTS:
   Name	   Type	           I/O	  Description
   ----	   ----            ---    -----------
   text[]   char            I     Text string to annotated
   curlin   int             I     Line number within the font that is to be
			          processed. Will have a value of 0 - 35.
   sf	    int	            I     Scaling factor of the font
   buf[]    unsigned char   O     Buffer containing sample values of the 
			          annotation.

VARIABLES:
  Name	       Type            I/O     Description
  ----         ----            ---     -----------
  back         unsigned char    O      background gray value
  font[96][58] unsigned int     O      Font table for the 96 characters. 
  fore	       unsigned char    O      foreground gray vlaue
  greysc       unsigned char    O      Grey value of background or foreground
  ichr	       int		O      integer value of a character
  len	       int		O      length of text string
  ptr	       int              O      Current index into the output buffer
  temp	       unsigned int     O      Temporary value after the byte shift

PROGRAM HISTORY:
  Version   Date       Author		Request
  -------   ----       ------       	-------
    1.0	    5/85       K. Gacke	     	initial development
    2	   11/89	K. Gacke	added spot copyright symbol
    3	    2/90	K. Gacke	PRF 5332 - color fire automation

PROJECT:     Color FIRE

LANGUAGE:    C

PDL:
    SUBROUTINE FONT2
    BEGIN
	SET PTR equal to zero -- pointer into the output buffer
	SET BACK equal to zero -- background gray value
	SET FORE equal to 255  -- foreground gray value
	SET LEN equal to the length of text string to be rasterized
	IF CURLIN greater than 46 THEN -- annotation is black line
	   BEGIN
	   SET NMR to the number of samples of the annotation
	   FOR PTR = 0 TO NMR DO
	      BEGIN
	      SET BUF[PTR] = BACK
	      END
	   RETURN to calling program
	   END
	DO FOR each character within the text string -- I is the loop index
	   BEGIN
	   Store into ICHR the integer value of the current character 
	   SET ICHR equal to (ICHR - 31) -- ICHR is numerical representation 
		of the character. Subtract 31 from it to obtain the index 
		into the font table.
	   IF the character is invalid THEN set the character equal to a 
		question mark.
	   SET VALUE to FONT[ICHR][CURLIN] -- The integer value of all 32 bits
	       for the current line of the annotation for the current character
	   FOR J = 31 DOWNTO 0 DO
	      BEGIN
	      SET GREYSC equal to BACK
	      Check to see if the current bit is a 1 by shifting the bits
			within the character to the right and perform a logical
			"and" of the shifted value.
	      IF the right most bit is on THEN
		 SET GREYSC equal to FORE
	      DO FOR K = 1 To SF -- Do for each scale factor
		 BEGIN
		 Increment PTR by 1
		 SET BUF(PTR) = GREYSC -- grey scale is assigned to buffer
	         END
	      END
	   END
	RETURN to calling Program
    END of subroutine FONT2

******************************************************************************/

#include <stdio.h>

font2(text,curlin,sf,buf)
   char    text[];
   int     curlin,sf;
   unsigned char buf[];
{
int    i,j,k,ptr;
int    ichr,len,nmr;
unsigned int  temp,value;
unsigned char back,fore,greysc;

/******************************************************************************

  Initialize the font table.  Note that the table is static; threrfore the
  table is not reinitialized every time the routine is called, which results
  in faster execution.  Characters are defined in ASCII order, and each 
  character is separated by { }.

*xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

static unsigned int font[96][58] = {
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		          0,         0,         0,         0,     12288, 
		      30720,     30720,     12288,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    493440,    493440,    493440,    493440,    493440, 
		     493440,    493440,    493440,    493440,    493440, 
		     493440,    493440,    493440,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,   1966560,   1966560, 
		    1966560,   1966560,   1966560,   1966560,   1966560, 
		    1966560,  67108863,  67108863,  67108863,  67108863, 
		    1966560,   1966560,   1966560,   1966560,   1966560, 
		    1966560,   1966560,  67108863,  67108863,  67108863, 
		   67108863,   1966560,   1966560,   1966560,   1966560, 
		    1966560,   1966560,   1966560,   1966560,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     30720,     30720,     30720,   1048568, 
		    4194296,  16777208,  33554424,  32536576,  65042432, 
		   62945280,  62945280,  65042432,  65042432,  66091008, 
		   33060864,  33454080,  16777088,   8388576,   2097144, 
		     524284,     31230,     30846,     30783,     30751, 
		      30751,     30735,     30735,     30751,     30782, 
		    8388606,   8388604,   8388592,   8388544,     30720, 
		      30720,     30720,     30720,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         3,         7,   1966095,   8355854,  16760862, 
		   31580220,  62976056,  58749048,  58749168,  62976224, 
		   31580640,  16761792,   8356736,   1968000,      3840, 
		       3584,      7680,     15360,     14336,     30720, 
		      61440,     57344,    122880,    245760,    229856, 
		     493560,    987132,    925214,   1981455,   3946503, 
		    3684359,   7879695,  15736350,  14684156,  31459320, 
		   62915040,  58720256,  50331648,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    130560,    524160,   2097120,   4194288,   8258040, 
		    8126712,  16253052,  16253052,   7864444,   8126588, 
		    3932408,   3932656,   1967072,   1048448,   1048320, 
		     523264,   2095104,   4159488,   8141824,  16268288, 
		   15736320,  32513536,  31461120,  65015552,  62916480, 
		   62916480,  62915523,  62915527,  65012207,  31457775, 
		   32506110,  15728894,  16253052,   8126716,   4194302, 
		    2097150,    524175,    130575,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{        15,        63,       126,       252,       504, 
		       1008,      2016,      1984,      4032,      8064, 
		      16128,     15872,     32256,     31744,     64512, 
		      63488,     63488,     63488,     63488,     63488, 
		      63488,     63488,     63488,     64512,     31744, 
		      32256,     15872,     16128,      8064,      4032, 
		       1984,      2016,      1008,       504,       252, 
		        126,        63,        15,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914560,  66060288,  33030144,  16515072,   8257536, 
		    4128768,   2064384,   1015808,   1032192,    516096, 
		     258048,    126976,    129024,     63488,     64512, 
		      31744,     31744,     31744,     31744,     31744, 
		      31744,     31744,     31744,     64512,     63488, 
		     129024,    126976,    258048,    516096,   1032192, 
		    1015808,   2064384,   4128768,   8257536,  16515072, 
		   33030144,  66060288,  62914560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,   7340088,   7864440,   3932400, 
		    1966560,    984000,    984000,    493440,    249600, 
		     249600,    130560,  67108863,  67108863,  67108863, 
		   67108863,    130560,    249600,    249600,    493440, 
		     984000,   1966560,   1966560,   3932400,   7864440, 
		    7340088,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,  67108863,  67108863,  67108863, 
		   67108863,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     30720, 
		      30720,     30720,     30720,     14336,     28672, 
		      57344,    114688,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  67108863,  67108863,  67108863, 
		   67108863,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     30720, 
		      30720,     30720,     30720,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         3,         7,        15,        14,        30, 
		         60,        56,       120,       240,       224, 
		        480,       960,       896,      1920,      3840, 
		       3584,      7680,     15360,     14336,     30720, 
		      61440,     57344,    122880,    245760,    229376, 
		     491520,    983040,    917504,   1966080,   3932160, 
		    3670016,   7864320,  15728640,  14680064,  31457280, 
		   62914560,  58720256,  50331648,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     64512,    261888,   1048512,   2097120,   4129776, 
		    8258040,   8126712,  16253052,  16253052,  15728700, 
		   32505918,  32505982,  32506110,  31457790,  65012703, 
		   65013663,  65015583,  62922255,  62929935,  62945295, 
		   62976015,  65134623,  65257503,  65503263,  32440350, 
		   33423422,  33292350,  33030206,  15728700,  16253052, 
		   16253052,   8126712,   8258040,   4129776,   2097120, 
		    1048512,    261888,     64512,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     63488,    129024,    260096,    489472, 
		     948224,   1865728,   1865728,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,   1048512, 
		    1048512,   1048512,   1048512,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    523776,   2097024,   8388576,  16516080,  32506104, 
		   31457404,  62914622,  62914622,        31,        31, 
		         31,        31,        63,        62,       126, 
		        252,       504,      1008,      2016,      4032, 
		       8064,     16128,     32256,     64512,    129024, 
		     258048,    516096,   1032192,   2064384,   4128768, 
		    8257536,  16515072,  33030144,  66060288,  67108863, 
		   67108863,  67108863,  67108863,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67108863,  67108863,  67108863,  67108863,        31, 
		         62,       124,       248,       496,       992, 
		       1984,      3968,      7936,     15872,     31744, 
		      63488,    126976,    130048,     32512,      8128, 
		       2016,      1008,       248,       124,        62, 
		         62,        31,  62914591,  62914622,  65011774, 
		   32505980,  16253176,  16253424,   8259552,   4194240, 
		    2097024,    523776,    129024,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{       240,       496,      1008,      2032,      4080, 
		       7920,     15600,     30960,     61680,    123120, 
		     246000,    491760,    983280,   1966320,   3932400, 
		    7864560,  15728880,  31457520,  67108863,  67108863, 
		   67108863,  67108863,       240,       240,       240, 
		        240,       240,       240,       240,       240, 
		        240,       240,       240,       240,       240, 
		        240,       240,       240,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67108856,  67108856,  67108856,  67108856,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  63045120, 
		   63438720,  65011680,  67108848,  66584824,  66060412, 
		   65011772,        62,        30,        31,        15, 
		         15,        15,        15,        31,  62914590, 
		   65011774,  31457340,  32505980,  16253176,   8388592, 
		    2097120,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     63488,    126976,    253952,    507904,   1015808, 
		    2031616,   1966080,   4063232,   3932160,   8126464, 
		    7864320,  16252928,  15728640,  32505856,  32636416, 
		   31981440,  66060256,  67108848,  66978296,  66584700, 
		   66060348,  66060350,  65011742,  65011743,  65011743, 
		   65011743,  65011743,  65011743,  65011743,  31457310, 
		   32505918,  15728700,  16253052,   8258040,   4194288, 
		    2097120,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67108863,  67108863,  67108863,  67108863,        63, 
		         62,        62,       126,       124,       252, 
		        504,       496,      1008,      2016,      1984, 
		       4032,      8064,      7936,     16128,     32256, 
		      31744,     64512,    129024,    126976,    258048, 
		     516096,    507904,   1032192,   2064384,   2031616, 
		    4128768,   8257536,   8126464,  16515072,  33030144, 
		   32505856,  66060288,  65011712,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    130560,    524160,   2097120,   4194288,   8258040, 
		    8126712,  16253052,  16253052,  16253052,  16253052, 
		    8126712,   4063728,   2032608,   1048512,    524160, 
		     524160,   2097120,   4129776,   8126712,  16253052, 
		   15728700,  32505918,  31457310,  65011743,  62914575, 
		   62914575,  62914575,  62914575,  65011743,  31457310, 
		   32505918,  15728700,  16253052,   8126712,   4194288, 
		    2097120,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    130560,    524160,   2097120,   4194288,   8258040, 
		   16253052,  15728700,  32505918,  31457310,  65011743, 
		   65011743,  65011743,  65011743,  65011743,  65011743, 
		   31457311,  32505919,  15728703,  16253055,   8258047, 
		    4194303,   2097135,    524175,    130590,        60, 
		         60,       124,       120,       248,       240, 
		        496,       480,       992,      1984,      3968, 
		       7936,     15872,     31744,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,     30720,     30720, 
		      30720,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     30720, 
		      30720,     30720,     30720,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,     30720,     30720, 
		      30720,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,     30720, 
		      30720,     30720,     30720,     14336,     28672, 
		      57344,    114688,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         7,        31,        62,       124,       248, 
		        496,      2016,      3968,      7936,     15872, 
		      64512,    126976,    253952,   1032192,   2031616, 
		    4063232,  16515072,  32505856,  65011712,  65011712, 
		   32505856,  16515072,   4063232,   2031616,   1032192, 
		     253952,    126976,     64512,     15872,      7936, 
		       3968,      2016,       496,       248,       124, 
		         62,        31,         7,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,  67108863,  67108863, 
		   67108863,  67108863,         0,         0,         0, 
		          0,  67108863,  67108863,  67108863,  67108863, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  58720256,  65011712,  32505856,  16252928,   8126464, 
		    4063232,   2064384,    507904,    253952,    126976, 
		      64512,     15872,      7936,      4032,       992, 
		        496,       252,        62,        31,        31, 
		         62,       252,       496,       992,      4032, 
		       7936,     15872,     64512,    126976,    253952, 
		     507904,   2064384,   4063232,   8126464,  16252928, 
		   32505856,  65011712,  58720256,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     64512,    261888,   1048512,   2097120,   4129776, 
		    8258040,   8126712,  16253052,  15728700,  32505918, 
		   31457310,  65011743,  62914591,  62914591,        30, 
		         62,       252,       496,       992,      4032, 
		       7936,     15872,     31744,     63488,    126976, 
		     122880,    122880,    122880,    122880,    122880, 
		          0,         0,         0,         0,    122880, 
		     122880,    122880,    122880,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    524032,   2097088,   4194272,   8388592,  16515576, 
		   16253176,  33030396,  32505980,  66060414,  65011774, 
		   65011775,  65011743,  62979103,  63045135,  63176463, 
		   63164175,  63145743,  63145743,  63145743,  63145743, 
		   63145758,  63145758,  63176700,  63045624,  62980080, 
		   62930880,  62914560,  65011712,  65011712,  32505856, 
		   32505856,  33030144,  16515072,  16646144,   8388544, 
		    4194240,   2097088,    524224,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     64512,    130560,    261888,    249600, 
		     493440,   1017792,    984000,   2032608,   2032608, 
		    1966560,   1966560,   4063728,   4063728,   3932400, 
		    3932400,   8126712,   8126712,   7864440,   7864440, 
		   16253052,  16253052,  15728700,  16777212,  33554430, 
		   33554430,  33554430,  31457310,  31457310,  31457310, 
		   65011743,  65011743,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67107840,  67108608,  67108800,  67108832,  15730672, 
		   15729144,  15728892,  15728764,  15728702,  15728702, 
		   15728671,  15728671,  15728671,  15728702,  15728702, 
		   15728764,  15728888,  16777184,  16777088,  16777088, 
		   16777184,  15728888,  15728764,  15728702,  15728702, 
		   15728671,  15728671,  15728655,  15728671,  15728671, 
		   15728670,  15728702,  15728764,  15728888,  16777200, 
		   16777184,  16777088,  16776704,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    130560,    524160,   1048512,   2097120,   4163568, 
		    8324088,   8258040,  16515324,  16253052,  32505918, 
		   32505918,  65011743,  65011743,  65011743,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  65011743, 
		   65011743,  65011743,  32505918,  32505918,  16253052, 
		   16515324,   8258040,   8324088,   4163568,   2097120, 
		    1048512,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67106816,  67108352,  67108736,  67108800,  15732704, 
		   15730672,  15729648,  15729144,  15728888,  15728892, 
		   15728764,  15728702,  15728702,  15728670,  15728671, 
		   15728671,  15728655,  15728655,  15728655,  15728655, 
		   15728655,  15728655,  15728671,  15728671,  15728670, 
		   15728702,  15728702,  15728764,  15728892,  15728888, 
		   15729144,  15729648,  15730672,  15732704,  16777152, 
		   16777088,  16776704,  16775168,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67108863,  67108863,  67108863,  67108863,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  67108352,  67108352,  67108352, 
		   67108352,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  67108863, 
		   67108863,  67108863,  67108863,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67108863,  67108863,  67108863,  67108863,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  67108352,  67108352,  67108352, 
		   67108352,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    130560,    524160,   1048512,   2097120,   4163568, 
		    8324088,   8258040,  16515324,  16253052,  32505918, 
		   32505918,  65011743,  65011712,  65011712,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62947327,  65044479, 
		   65044479,  65044479,  32505918,  32505918,  16253052, 
		   16515324,   8258040,   8324088,   4163568,   2097120, 
		    1048512,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  67108863,  67108863,  67108863, 
		   67108863,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{   1048512,   1048512,   1048512,   1048512,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,   1048512, 
		    1048512,   1048512,   1048512,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     16383,     16383,     16383,     16383,       480, 
		        480,       480,       480,       480,       480, 
		        480,       480,       480,       480,       480, 
		        480,       480,       480,       480,       480, 
		        480,       480,       480,       480,       480, 
		        480,  62915040,  62915040,  65012192,  65012192, 
		   66061280,  32506848,  33032128,  33034176,  16777088, 
		    8388352,   4193792,   1046528,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914623,  62914686,  62914812,  62915064,  62915568, 
		   62916576,  62918592,  62922624,  62930688,  62946816, 
		   62979072,  63043584,  63172608,  63430656,  63946752, 
		   64978944,  67043328,  66977792,  66846720,  66846720, 
		   66977792,  67043328,  64978944,  63946752,  63430656, 
		   63172608,  63043584,  62979072,  62946816,  62930688, 
		   62922624,  62918592,  62916576,  62915568,  62915064, 
		   62914812,  62914686,  62914623,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  67108863, 
		   67108863,  67108863,  67108863,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  66584703,  66846975,  66846975,  64881135,  63832527, 
		   63898575,  63374223,  63408015,  63145743,  63164175, 
		   63032847,  63045135,  62979087,  62979087,  62945295, 
		   62945295,  62926863,  62926863,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  65011727,  66060303,  66584591,  66584591,  66846735, 
		   66846735,  66977807,  64880655,  64946191,  63897615, 
		   63930383,  63406095,  63422479,  63160335,  63168527, 
		   63037455,  63041551,  62976015,  62978063,  62946319, 
		   62929935,  62930447,  62922255,  62922511,  62918415, 
		   62918543,  62916495,  62916559,  62915535,  62915567, 
		   62915055,  62915071,  62914815,  62914815,  62914687, 
		   62914687,  62914623,  62914591,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    130560,    524160,   2097120,   4194288,   4129776, 
		    8258040,   8126712,  16253052,  16253052,  15728700, 
		   32505918,  32505918,  32505918,  31457310,  65011743, 
		   65011743,  65011743,  62914575,  62914575,  62914575, 
		   62914575,  65011743,  65011743,  65011743,  31457310, 
		   32505918,  32505918,  32505918,  15728700,  16253052, 
		   16253052,   8126712,   8258040,   4129776,   4194288, 
		    2097120,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67107840,  67108608,  67108800,  67108832,  62915576, 
		   62914812,  62914684,  62914622,  62914590,  62914591, 
		   62914591,  62914591,  62914590,  62914622,  62914684, 
		   62914812,  62915576,  67108832,  67108800,  67108608, 
		   67107840,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,  62914560,  62914560, 
		   62914560,  62914560,  62914560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    130560,    524160,   2097120,   4194288,   4129776, 
		    8258040,   8126712,  16253052,  16253052,  15728700, 
		   32505918,  32505918,  32505918,  31457310,  65011743, 
		   65011743,  65011743,  62914575,  62914575,  62914575, 
		   62914575,  65011743,  65011743,  65026079,  31464478, 
		   32513598,  32509758,  32507710,  15730620,  16253948, 
		   16253436,   8126712,   8258044,   4129788,   4194302, 
		    2097134,    524175,    130567,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67107840,  67108608,  67108800,  67108832,  62915576, 
		   62914812,  62914684,  62914622,  62914590,  62914591, 
		   62914591,  62914591,  62914590,  62914622,  62914684, 
		   62914812,  62915576,  67108832,  67108800,  67108608, 
		   67107840,  64946176,  63930368,  63422464,  63168512, 
		   63041536,  62978048,  62946304,  62930432,  62922496, 
		   62918528,  62916544,  62915552,  62915056,  62914808, 
		   62914684,  62914622,  62914591,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    524160,   2097120,   4194288,   8388600,  16646652, 
		   16253052,  32505918,  32505886,  31457311,  65011727, 
		   65011727,  65011712,  31457280,  32505856,  16515072, 
		    8323072,   4177920,   1044480,    261632,     65408, 
		       8160,      2040,       508,       124,        62, 
		         30,        31,  62914591,  62914591,  65011742, 
		   31457342,  32505918,  16253052,  16646652,   8388600, 
		    4194288,   2097120,    524160,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67108863,  67108863,  67108863,  67108863,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  65011743,  65011743,  31457310,  32505918, 
		   16253052,  16515324,   8258040,   8324088,   4194288, 
		    2097120,   1048512,    261888,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914575,  65011743,  65011743,  31457310,  32505918, 
		   32505918,  15728700,  16253052,  16253052,   7864440, 
		    8126712,   8126712,   3932400,   4063728,   4063728, 
		    1966560,   2032608,   2032608,    984000,   1017792, 
		    1017792,    493440,    511872,    511872,    249600, 
		     261888,    261888,    130560,    130560,    130560, 
		      64512,     64512,     64512,     30720,     30720, 
		      30720,     12288,     12288,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62914575,  62914575, 
		   62914575,  62914575,  62914575,  62926863,  62926863, 
		   62945295,  62945295,  62979087,  62979087,  63045135, 
		   63032847,  63164175,  63145743,  63408015,  63374223, 
		   63898575,  63832527,  64881135,  64749807,  66846975, 
		   66584703,  66584703,  66060351,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914575,  31457310,  15728700,  15728700,   7864440, 
		    7864440,   3932400,   3932400,   1966560,   1966560, 
		     984000,    984000,    493440,    493440,    249600, 
		     249600,    130560,    130560,     64512,     64512, 
		     130560,    130560,    249600,    249600,    493440, 
		     493440,    984000,    984000,   1966560,   1966560, 
		    3932400,   3932400,   7864440,   7864440,  15728700, 
		   15728700,  31457310,  62914575,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  62914575,  31457310,  15728700,  15728700,   7864440, 
		    7864440,   3932400,   3932400,   1966560,   1966560, 
		     984000,    984000,    493440,    493440,    249600, 
		     249600,    130560,     64512,     64512,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  67108863,  67108863,  67108863,  67108863,        15, 
		         30,        30,        60,       120,       240, 
		        480,       480,       960,      1920,      3840, 
		       3840,      7680,     15360,     30720,     30720, 
		      61440,    122880,    245760,    245760,    491520, 
		     983040,   1966080,   1966080,   3932160,   7864320, 
		   15728640,  31457280,  31457280,  62914560,  67108863, 
		   67108863,  67108863,  67108863,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{   1048512,   1048512,   1048512,   1048512,    983040, 
		     983040,    983040,    983040,    983040,    983040, 
		     983040,    983040,    983040,    983040,    983040, 
		     983040,    983040,    983040,    983040,    983040, 
		     983040,    983040,    983040,    983040,    983040, 
		     983040,    983040,    983040,    983040,    983040, 
		     983040,    983040,    983040,    983040,   1048512, 
		    1048512,   1048512,   1048512,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  50331648,  58720256,  62914560,  29360128,  31457280, 
		   15728640,   7340032,   7864320,   3932160,   1835008, 
		    1966080,    983040,    458752,    491520,    245760, 
		     114688,    122880,     61440,     28672,     30720, 
		      15360,      7168,      7680,      3840,      1792, 
		       1920,       960,       448,       480,       240, 
		        112,       120,        60,        28,        30, 
		         15,         7,         3,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{   1048512,   1048512,   1048512,   1048512,       960, 
		        960,       960,       960,       960,       960, 
		        960,       960,       960,       960,       960, 
		        960,       960,       960,       960,       960, 
		        960,       960,       960,       960,       960, 
		        960,       960,       960,       960,       960, 
		        960,       960,       960,       960,   1048512, 
		    1048512,   1048512,   1048512,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     64512,    130560,    261888,    511872, 
		    1017792,   2032608,   4063728,   8126712,  16253052, 
		   32505918,  65011743,  62914575,  58720263,  50331651, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,  67108863, 
		   67108863,  67108863,  67108863,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     30720,     30720,     30720,     14336, 
		      28672,     57344,    114688,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,   4192256,  16776704,  33554304, 
		   62915520,  58720736,       224,    258288,   1047792, 
		    4194288,   8130544,  15729648,  31457776,  29360368, 
		   62914800,  58720368,  58720368,  58720368,  58720368, 
		   62914800,  29360368,  31457776,  15729656,   8130492, 
		    4194078,   1047567,    258055,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  58720256,  58720256,  58720256,  58720256,  58720256, 
		   58720256,  58720256,  58720256,  58720256,  58720256, 
		   58720256,  58720256,  58850816,  59244416,  60817376, 
		   62850032,  66846968,  66584700,  66060348,  65011742, 
		   65011742,  62914574,  62914575,  58720263,  58720263, 
		   58720263,  58720263,  62914575,  62914574,  65011742, 
		   65011742,  66060348,  66584700,  66846968,  62850032, 
		   60817376,  59244416,  58850816,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,   1048512,   4194288,   8388600, 
		   16515324,  32505918,  31457310,  65011743,  62914575, 
		   62914575,  58720256,  58720256,  58720256,  58720256, 
		   58720256,  58720256,  58720256,  58720256,  62914575, 
		   62914575,  65011743,  31457310,  32505918,  16515324, 
		    8388600,   2097136,    524224,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         7,         7,         7,         7,         7, 
		          7,         7,         7,         7,         7, 
		          7,         7,    130567,    524167,   2097127, 
		    4129783,   8126719,  16253055,  15728703,  31457311, 
		   31457311,  29360143,  62914575,  58720263,  58720263, 
		   58720263,  58720263,  62914575,  29360143,  31457311, 
		   31457311,  15728703,  16253055,   8126719,   4129783, 
		    2097127,    524167,    130567,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,    130560,    524160,   2097120, 
		    4129776,   8126712,  16253052,  15728700,  31457310, 
		   31457310,  29360142,  62914575,  58720263,  67108863, 
		   67108863,  67108863,  62914560,  29360128,  31457280, 
		   31457280,  15728640,  16252984,   8126584,   4129008, 
		    2097120,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{      7168,     15872,     16128,     31616,     29120, 
		      28896,     28896,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,    524032,    524032, 
		     524032,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,    130567,    524167,   2097127, 
		    4129783,   8126719,  16253055,  15728703,  31457311, 
		   29360143,  62914575,  62914575,  62914575,  62914575, 
		   29360143,  31457311,  15728703,  16253055,   8126719, 
		    4129783,   2097127,    524167,    130567,         7, 
		          7,         7,         7,  58720263,  58720263, 
		   62914575,  31457310,  15728700,   7864440,   4194288, 
		    1048512,    261888,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  58720256,  58720256,  58720256,  58720256,  58720256, 
		   58720256,  58720256,  58720256,  58720256,  58720256, 
		   58720256,  58720256,  58850816,  59244416,  60817376, 
		   62850032,  66846968,  66584700,  66060348,  65011742, 
		   65011742,  62914574,  62914575,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		      14336,     14336,     14336,         0,         0, 
		          0,         0,    129024,    129024,    129024, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		     130816,    130816,    130816,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		        224,       224,       224,         0,         0, 
		          0,         0,      2016,      2016,      2016, 
		        224,       224,       224,       224,       224, 
		        224,       224,       224,       224,       224, 
		        224,       224,       224,       224,       224, 
		        224,       224,       224,       224,       224, 
		        224,       224,       224,   1835232,   1835232, 
		    1966560,    917952,    984000,    493440,    261888, 
		     130560,     30720,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{  14680064,  14680064,  14680064,  14680064,  14680064, 
		   14680064,  14680064,  14680064,  14680064,  14680064, 
		   14680064,  14680064,  14680092,  14680124,  14680184, 
		   14680304,  14680544,  14681024,  14681984,  14683904, 
		   14687744,  14695424,  14710784,  14741504,  14802944, 
		   14925824,  15187968,  15720448,  16707584,  16545792, 
		   16268288,  15736320,  14683904,  14681984,  14681024, 
		   14680544,  14680304,  14680184,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    129024,    129024,    129024,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		      14336,     14336,     14336,     14336,     14336, 
		     130816,    130816,    130816,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  59507072,  60687296,  60687296, 
		   62851040,  62851040,  66293360,  66293360,  65133624, 
		   65133624,  62978076,  62978076,  58748942,  58748942, 
		   58748942,  58748942,  58748942,  58748942,  58748942, 
		   58748942,  58748942,  58748942,  58748942,  58748942, 
		   58748942,  58748942,  58748942,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  58850816,  59244416,  60817376, 
		   62850032,  66846968,  66584700,  66060348,  65011742, 
		   65011742,  62914574,  62914575,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,    130560,    524160,   2097120, 
		    4129776,   8126712,  16253052,  15728700,  31457310, 
		   31457310,  29360142,  62914575,  58720263,  58720263, 
		   58720263,  58720263,  62914575,  29360142,  31457310, 
		   31457310,  15728700,  16253052,   8126712,   4129776, 
		    2097120,    524160,    130560,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  58850816,  59244416,  60817376, 
		   62850032,  66846968,  66584700,  66060348,  65011742, 
		   62914574,  62914575,  62914575,  62914575,  62914575, 
		   62914574,  65011742,  66060348,  66584700,  66846968, 
		   62850032,  60817376,  59244416,  58850816,  58720256, 
		   58720256,  58720256,  58720256,  58720256,  58720256, 
		   58720256,  58720256,  58720256,  58720256,  58720256, 
		   58720256,  58720256,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,    130567,    524167,   2097127, 
		    4129783,   8126719,  16253055,  15728703,  31457311, 
		   29360143,  62914575,  62914575,  62914575,  62914575, 
		   29360143,  31457311,  15728703,  16253055,   8126719, 
		    4129783,   2097127,    524167,    130567,         7, 
		          7,         7,         7,         7,         7, 
		          7,         7,         7,         7,         7, 
		          7,         7,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,   7405312,   7602112,   7864288, 
		    8323312,   8257776,   8126576,   7864320,   7340032, 
		    7340032,   7340032,   7340032,   7340032,   7340032, 
		    7340032,   7340032,   7340032,   7340032,   7340032, 
		    7340032,   7340032,   7340032,   7340032,   7340032, 
		    7340032,   7340032,   7340032,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,   1048512,   4194288,   8388600, 
		   16515324,  15728700,  31457310,  29360143,  62914567, 
		   58720256,  62914560,  33554304,  16777184,   4194296, 
		        124,        30,        15,        15,         7, 
		   58720271,  62914574,  31457310,  15728700,  16515324, 
		    8388600,   4194288,   1048512,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,   4194272,   4194272, 
		    4194272,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28672,     28672,     28672, 
		      28672,     28672,     28784,     28784,     30944, 
		      16320,      8064,      3840,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  62914575,  29360143,  31457311, 
		   31457311,  15728703,  16253055,   8126719,   4129783, 
		    2097127,    524167,    130567,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  50331651,  58720263,  58720263, 
		   29360142,  29360142,  14680092,  14680092,   7340088, 
		    7340088,   3670128,   3670128,   1835232,   1835232, 
		     917952,    917952,    459648,    459648,    231168, 
		     231168,    118272,    118272,     64512,     64512, 
		      30720,     30720,     12288,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  58720270,  58720270,  58720270, 
		   58720270,  58748942,  58748942,  58748942,  58748942, 
		   58748942,  58748942,  58748942,  58748942,  58748942, 
		   58748942,  58748942,  29423644,  29423644,  14801976, 
		   14801976,   7573104,   7573104,   4130784,   4130784, 
		    1967040,   1967040,    786816,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  58720263,  62914575,  31457310, 
		   15728700,   7864440,   3932400,   1966560,    984000, 
		     493440,    249600,    130560,     64512,     30720, 
		      30720,     64512,    130560,    249600,    493440, 
		     984000,   1966560,   3932400,   7864440,  15728700, 
		   31457310,  62914575,  58720263,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  58720263, 
		   58720263,  58720263,  58720263,  58720263,  62914575, 
		   29360143,  31457311,  15728703,  16253055,   8126719, 
		    4129783,   2097127,    524167,    130567,         7, 
		          7,         7,         7,  58720263,  58720263, 
		   62914575,  31457310,  15728700,   7864440,   4194288, 
		    1048512,    261888,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,  67108863,  67108863,  67108863, 
		         30,        60,       120,       240,       480, 
		        960,      1920,      3840,      7680,   4194272, 
		    4194272,   4194272,    245760,    491520,    983040, 
		    1966080,   3932160,   7864320,  15728640,  31457280, 
		   67108863,  67108863,  67108863,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{      8128,     16320,     32704,     64512,     63488, 
		      61440,     61440,     61440,     61440,     61440, 
		      61440,     61440,     61440,     61440,     61440, 
		     122880,    245760,    983040,   1966080,   1966080, 
		     983040,    245760,    122880,     61440,     61440, 
		      61440,     61440,     61440,     61440,     61440, 
		      61440,     61440,     61440,     63488,     64512, 
		      32704,     16320,      8128,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{     30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,     30720,     30720, 
		      30720,     30720,     30720,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{   1040384,   1044480,   1046528,     64512,     31744, 
		      15360,     15360,     15360,     15360,     15360, 
		      15360,     15360,     15360,     15360,     15360, 
		       7680,      3840,       960,       480,       480, 
		        960,      3840,      7680,     15360,     15360, 
		      15360,     15360,     15360,     15360,     15360, 
		      15360,     15360,     15360,     31744,     64512, 
		    1046528,   1044480,   1040384,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{    786432,   4128768,   8355840,  15974400,  14802944, 
		   31514627,  29421575,  62945295,  58735630,  50338846, 
		       7708,      3900,      2040,      1008,       192, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 },
		{         0,         0,    130560,    524160,   2097120, 
		    4163568,   4063728,   8126712,   7864440,  15728700, 
		   15728700,  14680092,  31489054,  31522334,  31584030, 
		   29590414,  63373775,  63307983,  63832079,  63832079, 
		   63307983,  63373775,  29590414,  31584030,  31522334, 
		   31489054,  14680092,  15728700,  15728700,   7864440, 
		    8126712,   4063728,   4163568,   2097120,    524160, 
		     130560,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0,         0,         0, 
		          0,         0,         0 }
				   };

/*xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/

ptr = 0;				/* index pointer into output buffer  */
back = 0;				/* background value		     */
fore = 255;				/* foreground pointer		     */
len = strlen(text);			/* if curlin greater than 46 then    */
if (curlin > 46)			/*   buffer is filled with value of  */
   return;

for (i=0;i<len;i++)			/* do for each character of string   */
   {					/*   begin			     */
   if (text[i] == '\f')			/*   if char is form feed then       */
      ichr = 95;			/*     set ichr to be copyright char */
   else					/*   else 			     */
      ichr = text[i] - 32;		/*      ichr = integer value of char */
   if ((ichr > 95) || (ichr < 0))	/*   if invalid character then	     */
      ichr = 31;			/*     set ichr = question mark      */
   value = font[ichr][curlin];		/*   value = font value of character */
					/*      for the current line         */
   for (j=31;j>=0;j--)			/*   do for each 32 bits of the line */
      {					/*     begin			     */
      temp = value >> j;		/*     perform a right shift	     */
      for (k=0;k<sf;k++,ptr++)		/*     do for each scaling factor    */
	if (temp & 01)
         buf[ptr] = fore;		/*       assign greysc to output buf */
      }					/*     end			     */
   }					/*   end			     */
return;					/* return to calling program	     */
}					/* end of FONT2 subroutine	     */
