/******************************************************************************
Known problems;

	- If record is dropped due to disk write lag, sequence shows more
	  records written to disk than actual.

Notes;
	o Passes on the old frame synch, connected to BCA0: will exhibit
	  bit synch loss errors in the leftmost light and in the quality
	  words.  This is due to there being no bit synch to provide a
	  lock signal.  The other frame synch has this disabled since we
	  have no need to connect to a bit synch.

	o GAC is currently read three records at a time in order to allow
	  disk to keep up.  Inhale treats each three record chunk as one
	  record as far as examining the data for bad synch, sat etc.  No
	  delta time adjustment is made but delta time is stashed in the
	  header.

Modification history beginning 8/3/90;

08/03/90 TAB	Added code to find "best fit" for a pass against the schedule
		based on the median time of the pass (had been taking the first
		entry which contained the median time).  Added code to discard
		any LAC/GAC pass which contains less than 75% of the scheduled
		pass.

08/08/90 TAB	Modified startio() to reduce time needed for reset.  Also
		modified all remaining LOGENTRY's which were in the old ACQUIRE
		format.

08/14/90 TAB	Added times and datatype to disk write lag messages.

08/31/90 TAB	Modified droplog() to increase sceneid to 16 characters.

09/04/90 TAB	Modified to set protection of output to O:RWED G:RWED W:RE.

09/05/90 TAB	Fixed problem in day-of-year in disk lag messages.  Fixed
		calculation for median time of GAC scenes.  Fixed calculation
		for expected lines of GAC.  Added display of expected lines
		for all passes.

09/13/90 TAB	Fixed problem in 75% error message.  Beautified messages.

10/05/90 TAB	Added buffer number to LAG message.  Added disk full message.

10/09/90 TAB	Replaced several printf's with LOGENTRY's.  Added loop limit
		to nbstime().  Modified so that loss of NBS clock is not fatal.
		Always deassign the NBS device ISA0: before exiting with error.

05/21/91 TAB	Modified to add NOAA-12.  Stop receiving GAC for now.

05/28/92 TAB	Modified header() in acquire to create new header information
                needed for the 1k system.  Info is at offset 256, and old
                header info is unchanged but may be removed later.  Also
		changed output filename to reflect new station naming.

08/13/92 TAB	Fixed some bugs in previous modification.  Removed year as a
		parameter passed to header() and simply grabbed it out of
		the global variable.

09/13/92 TAB	Fixed problem in obtaining julian date for inclusion in new
		header.  Also clarified message for drop of incomplete LAC and
		GAC scenes.

12/30/92 TAB	Fixed problem in julian date calculation in INHALE.H with day
		year overflow.

01/12/93 TAB	Added code to suspend and resume netcopy - only if this is
		the Wallops inhale.  We'll let Gilmore co-exist with netcopy.

06/28/93 TAB	Always suspend/resume ingest1, ingest2 and netcopy.  Also
		changed so suspends and resumes occur while at elevated prior.

06/07/94 TAB	Modified to suspend and raise priority only if open succeeds.

11/29/94 TAB	Modified markstat() so that it always closes the schedule.

01/10/95 TAB	Modified to handle NOAA-14.  Added code to log unknown satid's
		so we can determine these much easier next time.

05/23/95 TAB	Changed percentage of scene received in order to keep it
		to 25%.

08/05/98 TTB	Modified to handle NOAA-15.

07/22/99 DEL	Modified for y2k compliance and to accomodate a system year
		offset in case the system fails to work reliably in 2000.
		Removed the calls to time.h localtime() and replaced with
		system function and a local function, get_localtime() since
		time.h localtime() failed Y2K testing.

01/08/01 DEL    Modified to handle NOAA-16.
04/08/02 DEL    Modified to handle NOAA-17.

*******************************************************************************/

#include	"inhale.h"

#define		SCAN_RATE 500		/* scan rate in milliseconds */
#define		TIME_OUT  5000		/* read timeout in milliseconds */
#define		INITIAL	1000		/* initial size of file in frames */
#define		GROW	500		/* number of frames to extend file */
#define		MISSED_AGE 21600	/* seconds before a pass is marked
						missed in the schedule */
#define		SATLOSS 10		/* consec bad synch frames allowed */
#define		LOGENTRY fprintf(LOGFILE,
#define		MBX_BUFSZ 512
#define		MBX_BUFQUO 10000
#define		ACQUIRE_SAYS_GO 1
#define		ACQUIRE_SAYS_STOP 2
#define		SCH$C_SUSP	9
#define		SCH$C_SUSPO	10

short	isb[4];			/* Must be global, because if we exit this
				   routine, BCDRIVER keeps writing it!!! */
char	nulls[128];		/* Used to zero out fib */
FILE	*LOGFILE;

/* This stuff is for event timing */
#ifdef TIMING
unsigned int shift = 10000, avg_1, avg_2;
unsigned short	elapsed[20000][2];
unsigned int	start_time[20000][2],stop_time[2],elapsed_time[2];
unsigned int	elapsed_millisec,cnt;
unsigned int	lcl_elapsed_time[2];
#endif

int	days,hours,minutes,seconds,ticks;

short	bcchan, pipechan= -1, datatype=0, filenbr=0, feed, delta;
int	astrap(), alarm, timex[2] = {-100000000,-1};
struct	tm	*tmptr;
int	inttime, attained, lost, sequence, bad, overwrite,mult;
int 	mbx_chan,mbx_status,recnum,diff1,diff2,synch_errs,time_errs;

struct
{
	short status;
	short count;
	int devdep;
} mbx_iosb;

char mbx_buffer[MBX_BUFSZ];
$DESCRIPTOR(mbx_logical,"WALLOPS_MBX");

char	datatype_name[3][5] = {"HRPT","LAC","GAC"};
char	iam[10], logname[64], proclogname[60];
char	disk_name[2][7] = {"HRPT1:","HRPT2:"};
char	filename[64], text_buff[60];
int	num_iam, expected, total_lines;

int	satcode, unknown_sat = 0, median_time;
char	satid[16][4] =
/*	  0  1     2  3    4   5     6  7     8  9    10  11   12 13 14  15 */
	{"","N11","","N16","","N14","","N15","","N12","","N17","","","","N10"};

/* Following lines used in checking delta logicals */
unsigned char acmode[1] = '\2';
char	satname[4] = "   ";
$DESCRIPTOR(logical, &satname);
$DESCRIPTOR(lnm_table, "LNM$SYSTEM_TABLE");
char	equiv[25];
int	equiv_len;

struct {
	unsigned short	s_buffer_length;
	unsigned short	s_item_code;
	char		*s_buffer_address;
	short		*s_return_length_address;

	int	end;
	} trnlnm_itmlst = {25 , 2, &equiv, &equiv_len, 0};

int     start_hour, start_minute, start_second, start_mil, start_julian;
int     start_month, start_dom, start_raw_time;
int	pid,state;

struct tm *get_localtime();


main()
{
char	my_prcnam[16];
int 	mbx_ast(), status;

struct	ITEM
{
	short	len;			/* buffer length */
	short	item_code;		/* information item requested */
	char	*buf;			/* return buffer */
	int	*rlen;			/* return length */
};

struct
{
	struct	ITEM items[1];
	int	endoflist;
} itemlist1				=
					{
		/* process name	*/	15 ,JPI$_PRCNAM,my_prcnam	,0
					};
int spare=0,i;

/* First, find out which DOMSAT I am */
itemlist1.endoflist = 0;
sys$getjpiw(0,0,0,&itemlist1,0,0,0);
my_prcnam[15] = 0;
if (strncmp(my_prcnam,"INHALE_WALLOPS",14) == 0)
	{
	strcpy(iam,"WALLOPS");
	num_iam = 0;
	}
  else if (strncmp(my_prcnam,"INHALE_GILMORE",14) == 0)
	{
	strcpy(iam,"FAIRBANKS");
	num_iam = 1;
	}
  else
	exit(1);

sprintf(logname,"ADAPSTABLES:%s.log",iam);
sprintf(proclogname,"ADAPSTABLES:%s.proclog",iam);

LOGFILE = fopen(logname,"w","shr=get","mbc=16");

/* If Wallops then we need to set up communications with ACQUIRE */
mbx_buffer[0] = ACQUIRE_SAYS_GO;
if(num_iam == 0)
	{
	mbx_status = sys$assign(&mbx_logical,&mbx_chan,0,0);
	if (!(mbx_status & 1 ))
		LOGENTRY"!\tError %d opening mailbox\n", mbx_status); 
	    else
		{
		mbx_status = sys$qio(0,mbx_chan,IO$_READVBLK,&mbx_iosb,
			&mbx_ast,0,mbx_buffer,512,0,0,0,0);
		if (!(mbx_status & 1 ))
			LOGENTRY"!\tError %d reading mailbox\n", mbx_status); 
		}
	sleep(5);				/* wait for mbx to flush */
	}

if (num_iam == 0)
	feed = 1;
     else
	feed = 0;
for(i=0;i<128;i++)
	nulls[i] = 0;
for(datatype=0;;)
	{
	/* Make sure acquire hasn't told us to back off */
	if (mbx_buffer[0] != ACQUIRE_SAYS_GO)
		{
		sys$dassgn(pipechan);
		pipechan = -1;
		sys$cancel(bcchan);
		sys$dassgn(bcchan);
		sleep(5);
		continue;
		}
	if(OK != setupipe())
		{
		LOGENTRY"!\tUnable to initialize data pipeline\n");
		sys$dassgn(pipechan);
		pipechan = -1;
		sys$cancel(bcchan);
		sys$dassgn(bcchan);
		sleep(15);
		continue;
		}
	if(OK != startio(&bcchan, isb))
		{
		LOGENTRY"!\tUnable to start cyclic input\n");
		sys$dassgn(pipechan);
		pipechan = -1;
		sys$cancel(bcchan);
		sys$dassgn(bcchan);
		sleep(15);
		continue;
		}

	timex[1] = -1;
	timex[0] = 0 - (SCAN_RATE * 10000);
	alarm = 0;
	sys$setimr(EFTIM,timex,astrap,0);
	sys$waitfr(EFDAT);
	sys$cantim(0,0);
	sys$clref(EFTIM);
	if(alarm == 1)
		{
		/* try each mode for each feed */
		datatype++;
/*		if (datatype >= 3) Uncomment for GAC */
		if (datatype >= 2)
			datatype = 0;
		continue;
		}
LOGENTRY
"**************************************************************************\n");
	tmptr = get_localtime();
	status = nbstime(&days,&hours,&minutes,&seconds,&ticks);
	if (status == OK)
		{
		JULTIM_TO_TOTSECS(seconds,minutes,hours,days,tmptr->tm_year,
			attained);
		LOGENTRY"!\tLock attained %03d %02d:%02d:%02d.%03d (%09d)\n",
			days,hours,minutes,seconds,ticks,attained);
		}
	    else
		{
		LOGENTRY"?\tCouldn't read NBS to get attained, continuing!");
		JULTIM_TO_TOTSECS(tmptr->tm_sec,tmptr->tm_min,tmptr->tm_hour,
			tmptr->tm_yday,tmptr->tm_year,attained);
		ticks = 0;
		LOGENTRY"!\tLock attained %03d %02d:%02d:%02d.%03d (%09d)\n",
			tmptr->tm_yday,tmptr->tm_hour,tmptr->tm_min,
			tmptr->tm_sec,ticks,attained);
		}
	if (datatype == 2)
		mult = 3;
	    else
		mult = 1;

	acquire();

	fclose(LOGFILE);
	LOGFILE = fopen(logname,"a","shr=get","mbc=16");
	filenbr++;
	}
}

acquire()
{
int	consecerr, future=0;
short	stamp[17];
int	millisec, secofyr, stpofyr;
int	year, month, date;
int	day, hour, minute, second, mil, tick;
int	oldmil[4] = {0,0,0,0};
int	i, j, dumy, status;
int	defpri;

int	ff = 0, tpe = 0, aspe = 0, bs = 0, mp = 0, spe = 0;

int	stat, wfordata, first_time,first_day,last_time,last_day;

sprintf(filename,"%s[INHALE]%04d%s.%s",
	disk_name[num_iam],filenbr,&iam[0],datatype_name[datatype]);
LOGENTRY"!\tInhaling %s data from %s to %s\n",
	datatype_name[datatype],iam,filename);

if (OK != bufopen(filename))
	{
	fflush(LOGFILE);
	/* vital that we shutdown cyclic (rotational) input */
	stat = sys$cancel(bcchan);
	stat = sys$dassgn(bcchan);
	sleep(30);
	return(NOTOK);
	}

stat = sys$setpri(0,0,0xf,&defpri);
pid = scan("INGEST1",&state);
if (pid != 0)
	{
	stat = sys$suspnd(&pid,0,0);
	if (!(stat & 1))
		LOGENTRY"!\tSuspend error %d on INGEST1\n",stat);
	    else
		LOGENTRY"!\tSuspended INGEST1 = %08x\n",pid);
	}
pid = scan("INGEST2",&state);
if (pid != 0)
	{
	stat = sys$suspnd(&pid,0,0);
	if (!(stat & 1))
		LOGENTRY"!\tSuspend error %d on INGEST2\n",stat);
	    else
		LOGENTRY"!\tSuspended INGEST2 = %08x\n",pid);
	}
pid = scan("NETCOPY",&state);
if (pid != 0)
	{
	stat = sys$suspnd(&pid,0,0);
	if (!(stat & 1))
		LOGENTRY"!\tSuspend error %d on NETCOPY\n",stat);
	    else
		LOGENTRY"!\tSuspended NETCOPY = %08x\n",pid);
	}

i = overwrite = bad = wfordata = consecerr = first_time = first_day = 0;
total_lines = last_time = last_day = synch_errs = time_errs = 0;

/* Build timestamp which is appended to each frame */
sprintf(stamp,"TIMECODE                SEQUENCE");
stamp[4] = year % 100;
stamp[5] = month;
stamp[6] = date;
stamp[11] = 1000;

satcode = 16;
delta = 0;

for(sequence=0;;sequence++)			/* Loop until signal loss */
	{
	if (mbx_buffer[0] != ACQUIRE_SAYS_GO)
		{
		LOGENTRY"!\tChannel seized by ACQUIRE\n");
		sys$dassgn(pipechan);
		pipechan = -1;
		goto GIVEUP;
		}
	overwrite = 0;
	if(SS$_WASSET != sys$readef(EFDAT,&dumy))
		wfordata++;
	timex[1] = -1;
	timex[0] = 0 - (TIME_OUT * 10000);
	alarm = 0;
	sys$setimr(EFTIM,timex,astrap,0);
	stat = sys$waitfr(EFDAT);
	/* This is where we take off from when 760 record arrives */
	/* This code is for event timing */
#ifdef TIMING
	if ((sequence >= 0) && (sequence <= 19999))
		{
		sys$gettim(&start_time[sequence]);
		elapsed[sequence][0] = elapsed[sequence][1] = 0;
		}
#endif
	sys$cantim(0,0);
	sys$clref(EFTIM);
	if(alarm == 1)
		break;

	sys$clref(EFDAT);

	if(isb[0] != SS$_NORMAL)
		{
		LOGENTRY"?\tcyclic read error after %d records\n",
							isb[2]);
		LOGENTRY"?\t\tI/O status = %d\n",isb[0]);
		goto GIVEUP;
		}

	/* i points to the rotational buffer (0-3) just filled */
	i = (isb[2] - 1) % NUMBUF;

	/* extract timecode, will be appended to HRPT & LAC  frame */
	millisec = cycbuff[i][11];
	millisec += cycbuff[i][10] << 10;
	millisec += (cycbuff[i][9] & 0x007f) << 20;

	mil = millisec % 1000;
	second = millisec / 1000;
	minute = second   / 60;		second	%= 60;
	hour = minute	  / 60;		minute	%= 60;
	day = hour	  / 24;		hour	%= 24;

	/* look for bad sync pattern */
	if((cycbuff[i][0]!=0x0284)||(cycbuff[i][1]!=0x016f)
	||	(cycbuff[i][2]!=0x035c)||(cycbuff[i][3]!=0x019d)
	||	(cycbuff[i][4]!=0x020f)||(cycbuff[i][5]!=0x0095))
		{
		bad++;
		synch_errs++;
		consecerr++;
		if(--sequence < -1)	/* was 1 */
			sequence = -1;

		/* If frame synch pattern has shifted one or two words, then
	           we have experienced the frame shift syndrome and will not
		   recover without resetting the 760 */
		if(((cycbuff[i][1]==0x0284)&&(cycbuff[i][2]==0x016f)
		    &&(cycbuff[i][3]==0x035c)&&(cycbuff[i][4]==0x019d)
		    &&(cycbuff[i][5]==0x020f))
				||
		   ((cycbuff[i][2]==0x0284)&&(cycbuff[i][3]==0x016f)
		    &&(cycbuff[i][4]==0x035c)&&(cycbuff[i][5]==0x019d)))
			{
			LOGENTRY"!\t***Detected frame shift syndrome, ");
			LOGENTRY"760 interface reset! (Seq %d)\n",
				(sequence+1)*mult);
			if(OK != startio(&bcchan, isb))
				goto GIVEUP;
			consecerr = 1;
			}

		if(consecerr >= SATLOSS)
			{
			LOGENTRY"!\t***%d consecutive bad frames, ",
				consecerr);
LOGENTRY"! Last sync was %08x %08x %08x %08x %08x %08x\n", cycbuff[i][0],
cycbuff[i][1],cycbuff[i][2],cycbuff[i][3],cycbuff[i][4],cycbuff[i][5]);
			LOGENTRY"760 interface reset (Seq %d)\n",
				(sequence+1)*mult);
			if(OK != startio(&bcchan, isb))
				goto GIVEUP;

			consecerr = 1;
			}
		fflush(LOGFILE);
		continue;
		}
	    else
		{	/* if FRAME SYNC PATTERN is perfect, then */
		consecerr = 0;
		last_time = millisec;
		last_day = cycbuff[i][8]>>1;
		/* If we haven't determined sat & delta yet, do it! */
		if (satcode == 16)
			{
			satcode = (cycbuff[0][6]>>3) & 0x0f;
			if (satid[satcode][0] != 0)
				{
				first_time = millisec;
				start_raw_time = millisec;
				start_julian = first_day = cycbuff[i][8]>>1;
				LOGENTRY"!\tFrame %d has sat %d (%s) ",
					sequence*mult,satcode,satid[satcode]);
				/* get delta */
				strcpy(satname,satid[satcode]);
				stat = SYS$TRNLNM(0, &lnm_table, &logical, 
					acmode, &trnlnm_itmlst);
				if (stat == 1)
					{
					equiv[equiv_len] = '\0';
					sscanf(equiv,"%d",&delta);
					}
				LOGENTRY" Delta = %d\n",delta);
				if (datatype !=2)
					start_raw_time -= delta;
				}
			    else
				{
				unknown_sat = satcode;
				satcode = 16;
				}
			}

		/* apply delta to HRPT and LAC, not to GAC */
		if (datatype != 2)
			millisec -= delta;
		mil = millisec % 1000;
		second = millisec / 1000;
		minute = second   / 60;		second	%= 60;
		hour = minute	  / 60;		minute	%= 60;
		day = hour	  / 24;		hour	%= 24;

		if ((millisec<oldmil[2]) && (millisec<oldmil[3]))
			{ 
			bad++;
			time_errs++;
			consecerr++;
			if(--sequence<-1)
				sequence= -1;
/*			LOGENTRY"!\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY" Timestamp < two previous frames\n");
			LOGENTRY"!\t\tFrame %d discarded",(sequence+1)*mult);
			LOGENTRY" (%d/%d/%d)\n",
				oldmil[2], oldmil[3], millisec);
*/
			future=0;
			continue;
			}
		else if (millisec<oldmil[3]) 
			{ 
			bad++;	
			time_errs++;
			consecerr++;
			if(--sequence < 0)
				sequence = 0;
			overwrite = 1;
/*			LOGENTRY"!\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY" Timestamp < previous frame\n");
			LOGENTRY"!\t\tPrevious frame %d discarded",
				sequence*mult);
			LOGENTRY" (%d/%d/%d)\n",
				oldmil[2], oldmil[3], millisec);
*/
			oldmil[3]=oldmil[2];
			future=0;
			}
		else if ((millisec > (oldmil[3] + 60000)) && (oldmil[3] != 0))
			{
			future++;
			if(--sequence<-1)
				sequence= -1;
			if (future >= 3)
				{
				if((sequence -= 2) < 0)
					sequence = 0;
				LOGENTRY"!\tConcatenated images encountered");
				LOGENTRY" after %d frames\n",sequence*mult);
				goto GIVEUP;
				}
/*			LOGENTRY"!\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY" Timestamp more than 1 min. in future\n");
			LOGENTRY"!\t\tFrame %d discarded",(sequence+1)*mult);
			LOGENTRY" (%d/%d/%d)\n",
				oldmil[2], oldmil[3], millisec);
*/
			continue;
			}
		else
			future = 0;
				
		oldmil[0] = oldmil[1];
		oldmil[1] = oldmil[2];
		oldmil[2] = oldmil[3];
		oldmil[3] = millisec;

		if((cycbuff[i][11089]&0x0ffff)!=0)
			{
			if((cycbuff[i][11089]&0x04000)!=0)
				ff++;
			if((cycbuff[i][11089]&0x03f00)!=0)
				for(dumy=0x02000;dumy>=0x00100;dumy>>=1)
					if((dumy&cycbuff[i][11089])!=0)
						tpe++;
			if((cycbuff[i][11089]&0x000F0)!=0)
				aspe += (cycbuff[i][11089]&0x000F0)>>4;
			if((cycbuff[i][11089]&0x080)!=0)
				bs++;
			if((cycbuff[i][11089]&0x040)!=0)
				mp++;
			if((cycbuff[i][11089]&0x020)!=0)
				spe++;
			}

		}

	/* append timestamp to HRPT and LAC frame */
	if ((datatype == 0) || (datatype == 1))
		{
		for(j=0;j<17;j++)
			cycbuff[i][HRPT_WORDCNT -100 +j] = stamp[j];
		if(day != 0)
			cycbuff[i][HRPT_WORDCNT -100 +6] = date + day;
		cycbuff[i][HRPT_WORDCNT -100 +  7] = hour;
		cycbuff[i][HRPT_WORDCNT -100 +  8] = minute;
		cycbuff[i][HRPT_WORDCNT -100 +  9] = second;
		cycbuff[i][HRPT_WORDCNT -100 + 10] = millisec%1000;
		cycbuff[i][HRPT_WORDCNT -100 + 16] = sequence;
		}

	/* queue up the write of this record to disk */
	if(OK != bufrit(i,sequence))
		goto GIVEUP;
	}
GIVEUP:
/* vital that we shutdown cyclic (rotational) input, else we corrupt memory */
stat = sys$cancel(bcchan);
stat = sys$dassgn(bcchan);

if (sequence < 0)
	sequence = 0;
tmptr = get_localtime();
status = nbstime(&days,&hours,&minutes,&seconds,&ticks);
if (status == OK)
	{
	JULTIM_TO_TOTSECS(seconds,minutes,hours,days,tmptr->tm_year,lost);
	LOGENTRY"!\tLock lost     %03d %02d:%02d:%02d.%03d (%09d)\n",
		days,hours,minutes,seconds,ticks,lost);
	}
    else
	{
	LOGENTRY"?\tCouldn't read NBS clock to find lost, continuing!\n");
	JULTIM_TO_TOTSECS(tmptr->tm_sec,tmptr->tm_min,tmptr->tm_hour,
		tmptr->tm_yday,tmptr->tm_year,lost);
	ticks = 0;
	LOGENTRY"!\tLock lost     %03d %02d:%02d:%02d.%03d (%09d)\n",
		tmptr->tm_yday,tmptr->tm_hour,tmptr->tm_min,
		tmptr->tm_sec,ticks,lost);
	}
pid = scan("INGEST1",&state);
if ((pid != 0) && ((state == SCH$C_SUSP) || (state == SCH$C_SUSPO)))
	{
	stat=sys$resume(&pid,0);
	if (!(stat & 1))
		LOGENTRY"!\tResume error %d on INGEST1\n",stat);
	    else
		LOGENTRY"!\tResumed INGEST1 = %08x\n",pid);
	}
pid = scan("INGEST2",&state);
if ((pid != 0) && ((state == SCH$C_SUSP) || (state == SCH$C_SUSPO)))
	{
	stat=sys$resume(&pid,0);
	if (!(stat & 1))
		LOGENTRY"!\tResume error %d on INGEST2\n",stat);
	    else
		LOGENTRY"!\tResumed INGEST2 = %08x\n",pid);
	}
pid = scan("NETCOPY",&state);
if ((pid != 0) && ((state == SCH$C_SUSP) || (state == SCH$C_SUSPO)))
	{
	stat = sys$resume(&pid,0);
	if (!(stat & 1))
		LOGENTRY"!\tResume error %d on NETCOPY\n",stat);
	    else
		LOGENTRY"!\tResumed NETCOPY = %08x\n",pid);
	}
stat = sys$setpri(0,0,defpri,&defpri);

/* Determine median timestamp for identification purposes */
if (first_time == 0)
	{
	LOGENTRY"?\tCouldn't find good start time, using NBS\n");
	first_time = attained;
	}
second = first_time / 1000;
minute = second   / 60;		second	%= 60;
hour = minute	  / 60;		minute	%= 60;
day = hour	  / 24;		hour	%= 24;
day += first_day;
tmptr = get_localtime();
year = tmptr->tm_year;
JULTIM_TO_TOTSECS(second,minute,hour,day,year,first_time);
LOGENTRY"!\tSat start  %04d %03d:%02d%:%02d:%02d (%09d)\n",
	year,day,hour,minute,second,first_time);

/* Calculate median time by adding to sat start time one half the number of
elapsed seconds of the acquisition.  Note that we must subtract the TIME_OUT
since that time was wasted waiting for a timeout.  Also, we must consider that
LAC and GAC are being played back at twice realtime, and therefore must double
the number of elapsed seconds which we add.   While we're at it, calculate
total_lines which is used in culling out useless fragments of passes. */
if (datatype == 0)
	{
	median_time = first_time + ((lost - attained - (TIME_OUT/1000))/2);
	total_lines = (lost - attained - (TIME_OUT/1000)) * 6;
	}
    else if (datatype == 1)
	{
	median_time = first_time + lost - attained - (TIME_OUT/1000);
	total_lines = (lost - attained - (TIME_OUT/1000)) * 12;
	}
    else if (datatype == 2)
	{
	median_time = first_time + ((lost - attained - (TIME_OUT/1000))*10);
	total_lines = (lost - attained - (TIME_OUT/1000)) * 40;
	}
if (total_lines < (wfordata * mult))
	total_lines = wfordata * mult;
LOGENTRY"!\tSat median                   (%09d)\n",median_time);

second = last_time / 1000;
minute = second   / 60;		second	%= 60;
hour = minute	  / 60;		minute	%= 60;
day = hour	  / 24;		hour	%= 24;
day += last_day;
tmptr = get_localtime();
year = tmptr->tm_year;
JULTIM_TO_TOTSECS(second,minute,hour,day,year,last_time);
LOGENTRY"!\tSat end    %04d %03d:%02d%:%02d:%02d (%09d)\n",
	year,day,hour,minute,second,last_time);

LOGENTRY"!\tSummary of frame sync. errors: ");
LOGENTRY"FF=%d TPE=%d ASPE=%d BS=%d MP=%d SPE=%d\n",
	ff,tpe,aspe,bs,mp,spe);
LOGENTRY"!\tFile actually contains:	%d FRAMES\n",sequence*mult);
LOGENTRY"!\tFrames with bad sync or timecode: 		%d\n",bad*mult);
if ((synch_errs > 0) || (time_errs > 0))
	LOGENTRY"!\tTimecode sequence errors = %d, sync errors = %d\n",
		time_errs,synch_errs);
LOGENTRY"!\tWait for data on %d lines\n",wfordata);

/* This code is for event timing */
#ifdef TIMING
for(cnt=avg_1=avg_2=0;cnt<sequence-1;cnt++)
	{
	if (cnt > 19998)
		break;
	subquad (&start_time[cnt+1],&start_time[cnt],&lcl_elapsed_time);
	ediv (&lcl_elapsed_time,&shift,&elapsed_millisec);
	elapsed[cnt][1] = elapsed_millisec;
	avg_2 += elapsed[cnt][1];
	}
if (sequence > 0)
	LOGENTRY"!\tAverage record interval = %4d ms.\n", avg_2 / sequence);
#endif
ident();
return(OK);
}

startio(bcchan,isb)
short	*bcchan, isb[];
{
int	sys$assign(), sys$qiow(), sys$qio(), sys$cancel(), sys$dassgn();
int	stat, i, do_reset=0;
$DESCRIPTOR(desc1,"_BCB0");
$DESCRIPTOR(desc2,"_BCA0");
int	modtab[2*7] = {1, 1, 2, 4, 3, 0, 4, 0, 5, 1, 6, 0, 8, 1};
int	buftab[2*NUMBUF];

RESET:;
stat = sys$dassgn(*bcchan);

for(i=0;i<NUMBUF;i++)
	{
	buftab[(2*i)+0] = &cycbuff[i][0];
	buftab[(2*i)+1] = framesiz[datatype];
	}

if (num_iam == 0)
	stat = sys$assign (&desc1,bcchan,0,0);
    else
	stat = sys$assign (&desc2,bcchan,0,0);

if(stat & 0x01 ==0)
	{
	LOGENTRY"?\tError attaching to the 760\n");
	return(NOTOK);
	}
stat = sys$qiow(EFDAT,*bcchan,IO$_RESET,isb,0,0,0,0,0,0,0,0);
if	( stat & 0x01 == 0 )
	{
	LOGENTRY"?\tError initializing the 760: stat = %d\n",
		stat);
	return(NOTOK);
	}
    else if	(isb[0] != SS$_NORMAL)
	{
	LOGENTRY"?\tError initializing the 760:isb[0] = %d\n",isb[0]);
	return(NOTOK);
	}

i = 0;
STARTCYC:
stat = sys$qio (EFDAT,*bcchan,IO$_RCYCLIC,isb,0,0,
		buftab,NUMBUF*2*4,modtab,7*2*4,0,0);
if((stat & 0x01) == 0)
	{
	LOGENTRY"?\tError initializing cyclic I/O: stat = %d\n",stat);
	if(stat == 0x02c4)
		{
		if(++i >= 10)
			goto RESET;
		sleep(1);
		goto STARTCYC;
		}
	return(NOTOK);
	}
fflush(LOGFILE);
return(OK);
}

int	filptr, writcount;
struct	FAB	thefab;
int	sys$qio(), sys$qiow(), stat;
unsigned short	isbfil[4][4];
int size;

static struct	fibdef	thefib;
static struct dsc$descriptor fibdesc =
    {sizeof(thefib),DSC$K_DTYPE_Z,DSC$K_CLASS_S,&thefib};

struct FAT
	{
	char	fat$b_rtype[1];
	char	fat$b_rattrib[1];
	unsigned short fat$w_rsize;
	unsigned int fat$l_hiblk;
	unsigned int fat$l_efblk;
	unsigned short fat$w_ffbyte;
	char	fat$b_bktsize[1];
	char	fat$b_vfcsize[1];
	unsigned short fat$w_maxrec;
	unsigned short fat$w_defext;
	unsigned short fat$w_gbc;
	char	filler[6];
	unsigned short fat$w_unused;
	unsigned short fat$w_versions;
	};

struct FAT thefat;

struct THEATR
	{
	struct atrdef atr;
	unsigned int terminator;
	};

struct THEATR theatr;

struct
	{
	unsigned short status;
	unsigned short next;					
	unsigned int extend;
	}iosb;

bufopen(filename)
char	filename[];
{
thefab = cc$rms_fab;
/* In order to truncate, file must not be shared ! */
thefab.fab$l_fna = filename;
thefab.fab$b_fns = strlen(filename);
thefab.fab$b_fac = FAB$M_BIO | FAB$M_PUT | FAB$M_TRN;
thefab.fab$l_fop = FAB$M_UFO | FAB$M_CBT;
thefab.fab$l_alq = ((bufsiz[datatype]/512)*INITIAL) +1;
thefab.fab$b_shr = FAB$M_NIL;
thefab.fab$w_ifi = 0;

stat = sys$create(&thefab);
if(stat != RMS$_NORMAL)
	{
	LOGENTRY"?\tImage file create error %d (stv=%d)\n",
		stat,thefab.fab$l_stv);
	if(stat == RMS$_FUL)
	    LOGENTRY"?\tThe disk is full!\n");
	return(NOTOK);
	}

writcount = 0;
sys$setef(32);
sys$setef(33);
sys$setef(34);
sys$setef(35);
filptr = 2;
isbfil[0][0] = isbfil[1][0] = isbfil[2][0] = isbfil[3][0] = 1;

size = thefab.fab$l_alq;

/* Must zero out the fib between uses */
strncpy(&thefib,nulls,sizeof(struct fibdef));

return(OK);
}

bufrit(bufnum,sequence)
int	bufnum, sequence;
{
int	i, dumy;
unsigned int stat;
int disk_ast();

if(SS$_WASSET != sys$readef(bufnum+32,&dumy))
    {
    tmptr = get_localtime();
    LOGENTRY"?\t%s Rec %d dropped due to disk lag on rec %d",
	datatype_name[datatype],sequence*mult,(sequence-4)*mult);
    LOGENTRY" (%03d %02d:%02d:%02d) Buf %d\n",tmptr->tm_yday+1,tmptr->tm_hour,
	tmptr->tm_min,tmptr->tm_sec,bufnum);
    return(OK);
    }

if(!(isbfil[bufnum][0] & 1))
	{
	LOGENTRY"?\tBad IOSB from prior disk write\n");
	LOGENTRY"?\t\tIOSB = %d %d %d %d\n",isbfil[bufnum][0],
		isbfil[bufnum][1],isbfil[bufnum][2],isbfil[bufnum][3]);
	LOGENTRY"?\t\tsequence number:%d\n",sequence);
	return(NOTOK);
	}
writcount++;

/* Get time, calculate elapsed millisec since 760 I/O completed.  Stash
result in millisec array indexed by sequence */
#ifdef TIMING
if ((sequence >= 0) && (sequence <= 19999))
	{
	sys$gettim(&stop_time);
	subquad (&stop_time,&start_time[sequence],&elapsed_time);
	ediv (&elapsed_time,&shift,&elapsed_millisec);
	elapsed[sequence][0] = elapsed_millisec;
	}
#endif
/* Adjust file pointer for overwriting previous record(s) */
if (overwrite > 0)
	{
	filptr -= ((bufsiz[datatype]/512) * overwrite);
	if (filptr < 1)
		filptr = 1;
	overwrite = 0;
	writcount--;
	}
stat = sys$qio (bufnum+32,thefab.fab$l_stv,IO$_WRITEVBLK,isbfil[bufnum],
	0,0,&cycbuff[bufnum][0],bufsiz[datatype],filptr,0,0,0);
/*stat = sys$qio (bufnum+32,thefab.fab$l_stv,IO$_WRITEVBLK,isbfil[bufnum],
	disk_ast,sequence,&cycbuff[bufnum][0],bufsiz[datatype],filptr,0,0,0);
*/
filptr += bufsiz[datatype]/512;
if(!(stat & 1))
	{
	LOGENTRY"?\tBad status from disk write\n");
	LOGENTRY"?\t\tsequence number:%d\n",sequence);
	return(NOTOK);
	}

/* If we're getting close to the end of the file, extend it by GROW frames */
if (filptr + (64*(bufsiz[datatype]/512)) > size)
	{
	thefib.fib$r_exctl_overlay.fib$w_exctl = FIB$M_EXTEND;
	thefib.fib$l_exsz = GROW * (bufsiz[datatype]/512);
	thefib.fib$l_exvbn = 0;
	stat = sys$qiow (0,thefab.fab$l_stv,IO$_MODIFY,&iosb,
		0,0,&fibdesc,0,0,0,0,0);
	if ((stat == 1) && (iosb.status == 1))
		size+= (GROW * (bufsiz[datatype]/512));
	    else
		{
		LOGENTRY"!\tFile extension failed (%d, %d, %d)\n",
			stat, iosb.status, iosb.extend);
		if (iosb.extend == 2128)
			LOGENTRY"!\tDisk is full!\n");
		return(NOTOK);
		}
	}

return(OK);
}

buflsh(records,orbit)
int	records,orbit;
{
int	ritesize, counter;

for(counter=0;counter<4;counter++)
	{
	stat = sys$waitfr(counter+32);
	if(!(stat & 1))
		LOGENTRY"?\tBad status from disk wait: stat = %d\n",stat);
	if(!(isbfil[counter][0] & 1))
	LOGENTRY"?\tBad iosb.status from disk write (#%d):iosb[0] = %d\n",
		counter,isbfil[counter][0]);
	}

LOGENTRY"!\tThe disk was accessed %d times\n",writcount);

/* note that the last two parameters were orbit (was always 0) and year */
header(&cycbuff[0][0],records,orbit);
stat = sys$qiow(32,thefab.fab$l_stv,IO$_WRITEVBLK,
	isbfil[0],0,0,&cycbuff[0][0],512,1,0,0,0);
if(!(stat & 1))
	LOGENTRY"?\tHeader record write failed!: status = %d\n",stat);
if(!(isbfil[0][0] & 1))
	LOGENTRY"?\tBad iosb.status from header write:iosb[0] = %d\n"
			,isbfil[0][0]);

thefib.fib$r_exctl_overlay.fib$w_exctl = FIB$M_TRUNC;
thefib.fib$l_exsz = 0;
thefib.fib$l_exvbn = filptr;

/* Fix the eof pointer */
strncpy(&thefat,nulls,sizeof(struct FAT));
strncpy(&theatr,nulls,sizeof(struct THEATR));
thefat.fat$l_efblk = (filptr<<16) + (filptr>>16);
thefat.fat$w_ffbyte = 0;
theatr.atr.atr$w_size = ATR$S_RECATTR;
theatr.atr.atr$w_type = ATR$C_RECATTR;
theatr.atr.atr$l_addr = &thefat;
theatr.terminator = 0;

stat = sys$qiow (0,thefab.fab$l_stv,IO$_DEACCESS,&iosb,0,0,
	&fibdesc,0,0,0,&theatr,0);
if ((stat != 1) || (iosb.status != 1))
	LOGENTRY"!\tFile deaccess/truncation failed (%d, %d, %d, %d)\n",
		stat, iosb.status, iosb.extend, thefib.fib$l_exsz);

stat = sys$dassgn(thefab.fab$l_stv);
if (stat != 1)
	LOGENTRY"!\tFile deassign failed, status=%d\n",stat);

return(OK);
}

header(buffer,records,orbit)
char	buffer[];
int	records, orbit;
/*
==========================================================================
==									==
==	This routine creates the header record that is appended to	==
==	the beginning of the HRPT image file.  The output of this	==
==	routine is left in 'buffer' to be written by the caller.	==
==									==
==========================================================================
*/
{
int	i, tmp_sat=0, tmp_samples, year;
char	tmp_stn[4];

year = tmptr->tm_year;
/* old header info from 0 to 255 */
buffer[0] = buffer[1] = buffer[2] = buffer[3] = 0xaa;	/* magic sync. number*/
if (datatype != 2) /* LAC, HRPT */
	{
	buffer[4] = (43*512) & 0x00ff;	/* bytes per record	(low part)   */
	buffer[5] = ((43*512)>>8) & 0x00ff; /* bytes per record	(high part)  */
	}
    else	   /* GAC */
	{
	buffer[4] = 6654 & 0x00ff;	/* bytes per record	(low part)   */
	buffer[5] = (6654>>8) & 0x00ff; /* bytes per record	(high part)  */
	}
if (datatype != 2)
	{
	buffer[6] = (HRPT_BYTECNT-200)& 0x00ff;	/* offset to time code (low part)   */
	buffer[7] = ((HRPT_BYTECNT-200)>>8)&0x00ff;/* offset to time code(high part)  */
	}
    else
	{
	buffer[6] = 16 & 0x00ff;	/* offset to time code	(low part)   */
	buffer[7] = 0;			/* offset to time code	(high part)  */
	}
buffer[8] = 1;buffer[9] = 0;		/* record offset to data (next rec.) */
buffer[10] = 1;buffer[11] = 0;		/* type type number (unused )	     */
buffer[12] = (records*mult) & 0x00ff;	/* number of records	(low part)   */
buffer[13] = ((records*mult) >> 8) & 0x0ff; /* number of records(high part)  */
buffer[14] = orbit & 0x00ff;		/* orbit number		(low part)   */
buffer[15] = (orbit >> 8) & 0x0ff;	/* orbit number 	(high part)  */
buffer[16] = year & 0x00ff;		/* year of data acq.	(low part)   */
buffer[17] = (year >> 8) & 0x0ff;	/* year of data acq.	(high part)  */
buffer[18] = 1;buffer[19] = 0;		/* tape rate. ??		     */
buffer[20] = 0;buffer[21] = 0;		/* data compression indicat.(0=no)   */
buffer[22] = 1;buffer[23] = 0;		/* byte swapping indicator. (1=no)   */
for(i=24;i<512;i++)
	buffer[i] = 0;
if (datatype == 2)
	{
	buffer[50] = delta & 0x00ff;		/* GAC delta	(low part)   */
	buffer[51] = (delta >> 8) & 0x0ff;	/* GAC delta	(high part)  */
	}
for(i=256;i<512;i++)buffer[i] = 32;     /* space fill new header             */
/* new header info from 256 to 511 */
if (satid[satcode][0] == 0)
	tmp_sat = 0;
    else
	sscanf(&satid[satcode][1],"%d",&tmp_sat);

if (num_iam == 0)
	strcpy(tmp_stn,"NOA");
    else if (num_iam == 1)
	strcpy(tmp_stn,"NGC");
    else
	strcpy(tmp_stn,"???");
if (datatype == 2)
	tmp_samples = 409;
    else
	tmp_samples = 2048;

/* start_raw_time is milliseconds since 00:00:00.00 today */
start_mil = start_raw_time % 1000;
start_second = start_raw_time / 1000;
start_minute = start_second   / 60;         start_second  %= 60;
start_hour = start_minute     / 60;         start_minute  %= 60;
					    start_hour    %= 24;

/* now find out the month, and day-of-month based on year and julian date */
JULGRE(year, start_julian, start_month, start_dom);

sprintf(&buffer[256],
"%5d %2d %4s %3s %4d %4d %02d/%02d/%02d %3d %02d:%02d:%02d.%03d",
delta, tmp_sat, datatype_name[datatype], tmp_stn, records, tmp_samples,
start_month, start_dom, year, start_julian, start_hour,
start_minute, start_second, start_mil);
}

disk_ast(sequence)
int sequence;
{
unsigned int	lcl_stop_time[2],lcl_elapsed_time[2];
unsigned int	lcl_elapsed_millisec;

sys$gettim(&lcl_stop_time);

#ifdef TIMING
if ((sequence >= 0) && (sequence <= 19999))
	{
	subquad (&lcl_stop_time,&start_time[sequence],&lcl_elapsed_time);
	ediv (&lcl_elapsed_time,&shift,&lcl_elapsed_millisec);
	elapsed[sequence][1] = lcl_elapsed_millisec;
	}
#endif
}

setupipe()
{
int	status;
short	outbuf[13];

/* Last digit is source (0 = 1, 1 = 2, 2 = 3, 3 = 4) */
short	framcom[4] = 	{
			0x1078,	/* HRPT 2x command */
			0x1038,	/* LAC	command */
			0x1058,	/* GAC	command */
			};
int	i;

for(i=0;i<13;i++)
	outbuf[i] = 0;

outbuf[0] = (framcom[datatype] | feed);
return(outpipe(outbuf,1));
}
outpipe(outbuf,words)
short	outbuf[];
int	words;
{
int	i, j;
int	sys$assign(), sys$qiow();
short	isb[4];
$DESCRIPTOR(desc1,"_BCB0");
$DESCRIPTOR(desc2,"_BCA0");

if (pipechan == -1)
	{
	if (num_iam == 0)
		j = sys$assign (&desc1,&pipechan,0,0);
	    else
		j = sys$assign (&desc2,&pipechan,0,0);
	if(j&0x01==0)
		{
		LOGENTRY"?\tError %d attaching to the 760\n",j);
		j = sys$dassgn(pipechan);
		pipechan = -1;
		return(NOTOK);
		}
	}

j = sys$qiow(EFPIP,pipechan,IO$_RESET,isb,0,0,0,0,0,0,0,0);
if(j&0x01==0)
	{
	LOGENTRY"?\tError %d initializing the 760\n",j);
	j = sys$dassgn(pipechan);
	pipechan = -1;
	return(NOTOK);
	}
if(isb[0] != SS$_NORMAL)
	{
	LOGENTRY"?\tError %d initializing the 760\n",isb[0]);
	j = sys$dassgn(pipechan);
	pipechan = -1;
	return(NOTOK);
	}

for(i=0;i<words;i++)
	{
	if (outbuf[i] == 0)
		continue;
	j = sys$qiow(EFPIP,pipechan,IO$_WREGISTER,isb,0,0,&outbuf[i],2,0,0,0,0);
	if (!(j&0x01))
		{
		LOGENTRY"?\tError writing to the 760\n");
		LOGENTRY"?\t\treturn status = %d\n",j);
		LOGENTRY"?\t\tIOSB = %d %d %d %d\n",
			isb[0],isb[1],isb[2],isb[3]);
		j = sys$dassgn(pipechan);
		pipechan = -1;
		return(NOTOK);
		}
	}
return(OK);
}

astrap()
{
alarm = 1;
sys$setef(EFDAT);
}

ident()
{
struct SCHED wo;		/* WORK ORDER: one schedule record*/
register status;
char	newname[64], tmp_stn[4];
FILE	*stream;
int	i, records;
extern	int noshare	errno;
extern	int noshare	vaxc$errno;


for( ;NULL==(stream=fopen(SCHEDULE,"r+")); sleep(5) )
	{
	if(errno == EVMSERR)
    	if (vaxc$errno == 98954)
	    LOGENTRY"!\tSchedule file locked by another user\n");
	  else
	    LOGENTRY"!\tvax c error number = %x\n",vaxc$errno);
	}

if(0 == fscanf(stream,"%d\n",&records))
	{
	LOGENTRY"!\tFirst schedule file read failed\n");
	if(0!=fclose(stream))
		LOGENTRY"!\tSchedule file close failed:errno = %d\n",errno);
	buflsh(sequence,0);
	return(NOTOK);
	}

diff1 = diff2 = 0;
recnum = -1;
for(i=0;i<records;i++)
	{
	GETSCHREC(stream,&wo);
	if((wo.station[0] == iam[0])
	    && (strncmp(wo.atype,&datatype_name[datatype],3) == 0)
	    && (strncmp(wo.satid,&satid[satcode],3) == 0)
	    && ((median_time > wo.rstart) && (median_time < wo.rstop)))
		{
		/* Ok... this MAY be it, but check for a better candidate */
		diff2 = abs(median_time - ((wo.rstart + wo.rstop) / 2));
		LOGENTRY"!\tSchedule entry %s within %d second(s) of median\n",
			wo.scene_id,diff2);
		if ((diff2 < diff1) || (diff1 == 0))
			{
			diff1 = diff2;
			recnum = i;
			}
		}
	}

if (recnum >= 0)
	{
	fseek(stream,0,0);			/* Rewind to BOF */
	fscanf(stream,"%d\n",&records);		/* Skip record count */
	fseek(stream,recnum * RECSIZ,1);	/* Skip to record */
	GETSCHREC(stream,&wo);
	LOGENTRY"!\tPass identified as %s in the schedule\n",wo.scene_id);
	if (datatype <= 1)
		expected = (wo.rstop - wo.rstart) * 6;
	    else if (datatype == 2)
		expected = (wo.rstop - wo.rstart) * 2;
	LOGENTRY"!\tPass contains %d of %d expected lines.\n",
		total_lines,expected);
	if((wo.stat[0] == PENDING) || (wo.stat[0] == MISSED) ||
	   (wo.stat[0] == DISK))
		{
		if(wo.stat[0] == MISSED)
			{
			LOGENTRY"!\tPass previously marked missed\n");
			droplog(wo.scene_id,"INHALE-DONE",
			    "Pass previously marked missed",
			    proclogname);
			}
		if(wo.stat[0] == DISK)
			{
			LOGENTRY"!\tPass is a rebroadcast\n");
			droplog(wo.scene_id,"INHALE-DONE",
			    "Pass is a rebroadcast",proclogname);
			}
		fclose(stream);
		if (num_iam == 0)
        		strcpy(tmp_stn,"NOA");
		    else if (num_iam == 1)
		        strcpy(tmp_stn,"NGC");
		    else
		        strcpy(tmp_stn,"XXX");

		sprintf(newname,"%s[IMAGES]%s.%s",
			disk_name[num_iam],wo.scene_id,tmp_stn);
		/* wait for write completion, write header record*/
		buflsh(sequence,wo.orbit);
		/* Throw LAC or GAC pass away if too much missing */
		if ((total_lines < ((expected * 1) / 4)) &&
		    (datatype != 0))
			{
			LOGENTRY"!\t %s pass less than 25%%",
				datatype_name[datatype]);
			LOGENTRY" complete, deleting.\n");
			sprintf(text_buff,
		 	   "Recieved only %d of %d lines, deleted!",
			   total_lines,expected);
			droplog(wo.scene_id,"INHALE-DONE",text_buff,
				proclogname);
			delete(filename);
			return(OK);
			}
		chmod(filename,509);	/* prot=(O:RWED,G:RWED,W:RE) */
		rename(filename,newname);
		LOGENTRY"!\tPass renamed %s\n",newname);
		sprintf(text_buff,
 			"%s contains %d lines",newname,sequence*mult);
		droplog(wo.scene_id,"INHALE-DONE",text_buff,proclogname);
		sprintf(text_buff,
 			"%d lines written, %d bad lines detected",
			sequence*mult,bad);
		droplog(wo.scene_id,"INHALE-DONE",text_buff,proclogname);
		wo.stat[0] = 'D';
		markstat(&wo);
		return(OK);
		}
	    else if(wo.stat[0] == NONE)
		{
		fclose(stream);
		LOGENTRY"!\tPass not scheduled for acquisition, deleting.\n");
		droplog(wo.scene_id,"INHALE-DONE",
		    "Pass not scheduled to be retained, deleted it",
		    proclogname);
		buflsh(sequence,0);
		delete(filename);
		return(OK);
		}
	}

fclose(stream);
if (satcode == 16)
	LOGENTRY"?\t Unknown satcode %d\n",unknown_sat);
LOGENTRY"!\t%s %s from %s at %09d not in the schedule, deleting\n",
	satid[satcode],datatype_name[datatype],iam,median_time);
buflsh(sequence,0);
delete(filename);
return(OK);
}

markstat(newo)
struct	SCHED	*newo;
{
FILE	*stream;
int	i, records, now, status;
struct	SCHED	wo;

if(NULL == (stream = fopen(SCHEDULE,"r+")) )
	{
	LOGENTRY"?\tSchedule file open failed\n");
	return(NOTOK);
	}

if(0 == fscanf(stream,"%d\n",&records))
	{
	LOGENTRY"?\tFirst schedule file read failed\n");
	fclose(stream);
	return(NOTOK);
	}

status = nbstime(&days,&hours,&minutes,&seconds,&ticks);
if (status == OK)
	JULTIM_TO_TOTSECS(seconds,minutes,hours,days,tmptr->tm_year,now);
    else
	{
	LOGENTRY"?\tCouldn't read NBS clock!  Continuing!\n");
	now = 0;	/* Don't mark any missed! */
	}

for(i=0;i<records;i++)
	{
	GETSCHREC(stream,&wo);
	if ((newo->lstart == wo.lstart) && (wo.station[0] == iam[0]))
		{
		fseek(stream,-RECSIZ,1);
		PUTSCHREC(stream,newo);
		fclose(stream);
		return(OK);
		}
	    else if ((wo.stat[0] == PENDING) 
	          && (wo.station[0] == iam[0])
		  && (now - wo.lstart > MISSED_AGE))
		{
		fseek(stream,-RECSIZ,1);
		wo.stat[0] = 'M';
		droplog(wo.scene_id,"INHALE-MISSED",
			"Pass marked as missed",proclogname);
		LOGENTRY"!\tMarked old pass %s as missed\n",wo.scene_id);
		PUTSCHREC(stream,&wo);
		}
	}
fclose(stream);
LOGENTRY"?\tSchedule entry not found!\n");
return(NOTOK);
}

# define PSCAN$_NODENAME 13
# define PSCAN$_GETJPI_BUFFER_SIZE 24

scan(usr,state)
char *usr;
int *state;
{
int 	context;		/* scan context */
int	pid;			/* process identification */
char	username[13];		/* process name */
int	owner;			/* owner if subprocess */
char	nodename[6] = "EROSH";	/* nodename of remote system */
int	scan_iosb[2], proc_state;

struct	JPI_ITEM
{
	short	len;			/* buffer length */
	short	item_code;		/* information item requested */
	char	*buf;			/* return buffer */
	int	*rlen;			/* return length */
};

struct	SCAN_ITEM
{
	short	len;			/* buffer length */
	short	item_code;		/* information item requested */
	char	*buf;			/* return buffer */
	int	code;			/* item code */
};

struct
{
	struct	JPI_ITEM items[4];
	int	endoflist;
} itemlist				=
					{
		/* pid		*/	 4 ,JPI$_PID	,&pid		,0
		/* process name */	,12,JPI$_USERNAME,username	,0
		/* owner	*/	,4 ,JPI$_OWNER	,&owner		,0
		/* state	*/	,4 ,JPI$_STATE	,&proc_state	,0
					};
struct
{
	struct	SCAN_ITEM items[2];
	int	endoflist;
} scanlist				=
					{
		/* node		*/	 5 ,PSCAN$_NODENAME,nodename,0
		/* buffer	*/	,0 ,PSCAN$_GETJPI_BUFFER_SIZE,1000,0
					};

short	status;				/* status returned by calls */
int	counter;

	itemlist.endoflist = scanlist.endoflist = 0;
	status = sys$process_scan(&context,&scanlist);
	if (status != SS$_NORMAL)
		{
		LOGENTRY"?\tscan status = %d\n",status);
		return(0);
		}

	for(status=0;status != SS$_NOMOREPROC;)
		{
		status = sys$getjpiw(1,&context,0,&itemlist,&scan_iosb,0,0);
		/* Check next process if this one returns a bad status */
		if (status == SS$_NORMAL)
			status = scan_iosb[0];
		if (status != SS$_NORMAL)
			continue;

		/* Un-pad the user name */
		for (counter=12;counter >= 0;counter--)
			{
			if (username[counter] == 32)
				username[counter] = 0;
			}
		if ((strcmp(usr,username) == 0) && (owner == 0))
			{
			*state = proc_state;
			return(pid);
			}
		}
	return(0);
}

nbstime(days,hours,minutes,seconds,ticks)
int	*days, *hours, *minutes, *seconds, *ticks;
{
int	i, j, clock_loop=0;
int	funct;
int	sys$assign(), sys$qiow();
short	isb[4], ischan, timecode[4], temp[4];
$DESCRIPTOR(desc,"_ISA0");

j = sys$assign (&desc,&ischan,0,0);
if(j&0x01==0)
	{
	LOGENTRY"?\tERROR ATTACHING TO THE DSS-11\n");
	return(NOTOK);
	}

READCLOCK:;
clock_loop++;
if (clock_loop >= 10)
	{
	sys$dassgn(ischan);
	return(NOTOK);
	}

funct = IO$_READVBLK;
j = sys$qiow(1,ischan,funct,isb,0,0,timecode,0,0,0,0,0);
if(j&0x01==0)
	{
	LOGENTRY"?\tERROR INITIATING A READ FROM THE DSS-11\n");
	LOGENTRY"?\t\treturn status = %d\n",j);
	sys$dassgn(ischan);
	return(NOTOK);
	}
else if(isb[0]&0x01 == 0)
	{
	LOGENTRY"?\tERROR READING FROM THE DSS-11\n");
	LOGENTRY"?\t\tIOSB[0] = %d\n",isb[0]);
	sys$dassgn(ischan);
	return(NOTOK);
	}

if((timecode[1]&0x0001)!=0)
	{
	LOGENTRY"?\tINVALID CLOCK READ: try again\n");
	goto READCLOCK;
	}

sys$dassgn(ischan);

*days	= 100 * (0x03 & (timecode[2] >> 14))
	+ 10  * (0x0f & (timecode[2] >> 10))
	+	(0x0f & (timecode[2] >> 06));

*hours	= 10 *	(0x03 & (timecode[2] >> 04))
	+	(0x0f & (timecode[2] >> 00));

*minutes = 10 *	(0x07 & (timecode[1] >> 13))
	+	(0x0f & (timecode[1] >> 09));

*seconds = 10 *	(0x07 & (timecode[1] >> 06))
	+	(0x0f & (timecode[1] >> 02));

*ticks	= 100 *	(0x0f & (timecode[0] >> 12))
	+ 10  *	(0x0f & (timecode[0] >> 08))
	+	(0x0f & (timecode[0] >> 04));
return(OK);
}

droplog(scid,mess,text,logname)
char	scid[], mess[], text[], logname[];
{
struct	DESC	siddesc, mesdesc, txtdesc, logdesc;
char	sceneid[17], message[17], explain[61], log[61], pad[61];
sprintf(pad,"                                                            ");

strncpy(sceneid,scid,16);
strncpy(message,mess,16);
strncpy(explain,text,60);
strncpy(log,logname,60);

strncat(sceneid,pad,16-strlen(sceneid));
strncat(message,pad,16-strlen(message));
strncat(explain,pad,60-strlen(explain));
strncat(log,pad,60-strlen(log));

siddesc.addr = sceneid;		siddesc.length = 16;
mesdesc.addr = message;		mesdesc.length = 16;
txtdesc.addr = explain;		txtdesc.length = 60;
logdesc.addr = log;		logdesc.length = 60;

return( logwrt(&siddesc,&mesdesc,&txtdesc,&logdesc) );
}

mbx_ast()
{
mbx_buffer[1] = 0;
mbx_status = sys$qio(0,mbx_chan,IO$_READVBLK,&mbx_iosb,&mbx_ast,0,
		mbx_buffer,512,0,0,0,0);
}

/*-----------------------------------------------------------------------
Mimic the time.h localtime() function call.  This function sets several
of the tm structure fields: tm_mday, tm_year, tm_hour, tm_min, tm_sec,
tm_mon, and tm_yday.  The tm_year is a 2-digit year NOT a count of years
since 1900.  The other fields are as described in the man pages for
localtime() except that no attemp was made for time zone.  The structure
simply returns the system time.
------------------------------------------------------------------------*/
struct tm *get_localtime()
{
unsigned int    bintim[2];	/* vax binary time			*/
struct  DESC time_desc;		/* time descriptor for asctim() call	*/
int     date_len;		/* length of date string		*/
char    date_str[30];		/* the date string from asctim()	*/
static	struct  tm *tmptr = NULL;	/* the time structure		*/
int     i;			/* loop counter				*/
char    mon_str[4];		/* month string extracted from date	*/
int     leap = 0;		/* leap year flag			*/
char    *month[12] = { "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
		       "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};

/* Allocate a tm structure
--------------------------*/
if ( tmptr == NULL )
    tmptr = (struct tm *)malloc( sizeof(struct tm *));

/* Get the system binary time and convert it to an ASCII string
---------------------------------------------------------------*/
sys$gettim( &bintim );
time_desc.addr = &date_str;
time_desc.length = 25;
sys$asctim( &date_len, &time_desc, &bintim, 0 );

/* Remove the delimiters from the date string
---------------------------------------------*/
date_len = strlen( date_str );
for ( i = 0; i < date_len; i++ )
    if ( date_str[i] == '-' || date_str[i] == ':' || date_str[i] == '.' )
        date_str[i] = ' ';

/* Extract the date and time info from the date string
------------------------------------------------------*/
sscanf( date_str, "%d %s %d %d %d %d", &tmptr->tm_mday, mon_str,
        &tmptr->tm_year, &tmptr->tm_hour, &tmptr->tm_min, &tmptr->tm_sec );

/* Adjust for any year offset, set the leap year flag
and scrunch year to 2 digits
------------------------------------------------------*/
tmptr->tm_year += YEAR_OFFSET;
if ( tmptr->tm_year % 4 == 0 )
    leap = 1;
tmptr->tm_year %= 100;

/* Convert the month string to an 0-based month index
-----------------------------------------------------*/
for ( i = 0; i < 12; i++ )
    if ( strcmp( mon_str, month[i] ) == 0 )
        tmptr->tm_mon = i;

/* Count the number of days since the beginning of the year
-----------------------------------------------------------*/
tmptr->tm_yday = 0;
switch ( tmptr->tm_mon )
    {
    case  11 : tmptr->tm_yday += 30;
    case  10 : tmptr->tm_yday += 31;
    case   9 : tmptr->tm_yday += 30;
    case   8 : tmptr->tm_yday += 31;
    case   7 : tmptr->tm_yday += 31;
    case   6 : tmptr->tm_yday += 30;
    case   5 : tmptr->tm_yday += 31;
    case   4 : tmptr->tm_yday += 30;
    case   3 : tmptr->tm_yday += 31;
    case   2 : if ( leap )
                   tmptr->tm_yday += 29;
               else
                   tmptr->tm_yday += 28;
    case   1 : tmptr->tm_yday += 31;
    }
tmptr->tm_yday += tmptr->tm_mday;
    
return tmptr;
}

