/*******************************************************************************
NAME:		GETPAR

PURPOSE:	Retreive the user specified input parameters

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "typlim.h"
#include "wvschips.h"

FUNCTION getpar(vblock,ddr,model,wvspath,taein,hosin,tpsfile,mtpfile,
		bands,nbands,odtype,pixval,backgrnd,imgback,proj,msgtim)

struct	PARBLK	*vblock;	/* TAE parameter block			     */
struct  DDR	*ddr;		/* DDR information			     */
struct  MODEL  	*model;   	/* AVHRR model                              */

long	bands[][MAXBND+1],	/* bands to be processed		     */
	nbands[],		/* band numbers selected		     */
	*odtype,		/* output data type			     */
	*pixval,		/* pixel intensity value for rasterized pixel*/
	*backgrnd,		/* background value for rasterized image     */
	*imgback,		/* background filled with image data flag    */
	*msgtim;		/* time between processing messages	     */

char	*wvspath,		/* location of wvs			     */
	taein[][CMLEN],		/* input TAE image name			     */
	hosin[][CMLEN],		/* input host image name		     */
	*tpsfile,		/* input tps file name			     */
	*mtpfile,		/* output mtp file name		     	     */
	*proj;			/* projection type (satellite or rectified)  */
{
struct VARIABLE	*var;		/* TAE variable structure		     */
struct VARIABLE *p_find();	/* TAE variable finder function		     */

long	access,			/* file access mode			     */
	nmrimg,			/* number of input image specified	     */
	nonfatal = 1,		/* non-fatal error code			     */
	one = 1,		/* constant of one			     */
	qflag[1][2],		/* flag indicating if subwindows specified   */
	same_window,		/* flag indicating if images hav same window */
	status,			/* status return code			     */
	totbnd,			/* Total number of bands		     */
	window[1][4],		/* input image window specification	     */
	zero = 0;		/* constant of zero			     */

/*
 * Get the input image name defining the area in which the vector
 * data is to be rasterized
 */
status = c_getin(vblock,"IN",&zero,taein,hosin,&nmrimg,window,bands,nbands,
		 &totbnd,&one,&same_window,qflag);
if (status != E_SUCC)
   return(E_FAIL);

/*
  Get the DDR information
  and validate projection parameters
*/
status = c_getddr(hosin,ddr);
*odtype = ddr->dtype;

/*
  Get the projection type
*/
var = p_find(vblock,"PROJ");
strcpy(proj,SVAL(*var,0));

if (strcmp(proj,"OTH") == 0)
    {
    if ((ddr->valid[0] == INVAL) || 
	(ddr->valid[DDCCV] == INVAL) ||
        (ddr->valid[DDPDV] == INVAL))
    	{
    	c_errmsg("Invalid projection information contained in DDR",
		"wvschips-proj",&nonfatal);
    	return(E_FAIL);
    	}
    }

/*
  Get the input tps file name
*/
access = IREAD;
if (c_getnam(vblock,"INTS",&zero,tpsfile,"tps",&access) != E_SUCC) 
    return(E_FAIL);

/*
  Get the output mtp file name
*/
access = IWRITE;
if (c_getnam(vblock,"OUTMT",&zero,mtpfile,"mtp",&access) != E_SUCC) 
    return(E_FAIL);

/*
  Get the pixel value intensity for the line work
*/
var = p_find(vblock,"PIXVAL");
*pixval = IVAL(*var,0);

/*
  Get the background pixel value
*/
var = p_find(vblock,"BACKGRND");
if (var->v_count == 0)
    {
    /*
      If BACKGRND is NULL, we need to be sure only 1 band was 
      specified in IN for the source of the chip background pixel values 
    */
    *imgback = TRUE;
    *backgrnd = 0;
    if (totbnd > 1)
        {
        c_errmsg("Only a single band image allowed on input",
	     "wvschips-onebnd",&nonfatal);
    	return(E_FAIL);
        }
    }
else
    {
    *imgback = FALSE;
    *backgrnd = IVAL(*var,0);
    }

/*
  Check that the background and pixel value are in a valid range for the data
  type of the input image.
*/
if (*odtype == EBYTE)
    {
    if ((*backgrnd > MAXUTINY) || (*backgrnd < MINUTINY))
	{
	c_errmsg("BACKGRND is not within the valid range for byte data",
	    "wvschips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXUTINY) || (*pixval < MINUTINY))
	{
	c_errmsg("PIXVAL is not within the valid range for byte data",
	    "wvschips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }
else if (*odtype == EWORD)
    {
    if ((*backgrnd > MAXWORD) || (*backgrnd < MINWORD))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "wvschips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXWORD) || (*pixval < MINWORD))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "wvschips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }
else if (*odtype == ELONG)
    {
    if ((*backgrnd > MAXLONG) || (*backgrnd < MINLONG))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*4 data",
	    "wvschips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXLONG) || (*pixval < MINLONG))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*4 data",
	    "wvschips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }
else /* (*odtype == EREAL) */
    {
    if ((*backgrnd > MAXREAL) || (*backgrnd < MINREAL))
	{
	c_errmsg("BACKGRND is not within the valid range for real data",
	    "wvschips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXREAL) || (*pixval < MINREAL))
	{
	c_errmsg("PIXVAL is not within the valid range for real data",
	    "wvschips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }

/*
  Get the location of the WVS
*/
var = p_find(vblock, "VECTSRC");
strcpy(wvspath, SVAL(*var, 0));

/*
  Retrieve the AVHRR DDR when the input image is in satellite prespective
*/
if (strcmp(proj,"SAT") == 0)
    {
    status = c_getaddr(hosin,model);
    if (status != E_SUCC)
    	return(E_FAIL);
    }

/*
  Get the time between processing messages
*/
var = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*var,0);

return(E_SUCC);
}
