/*******************************************************************************
NAME:	WVSCHIPS

PURPOSE:	Rasterize data from the World Vector Shorelines,
		creating separate 64x64 image "chips" that can be used
		in automatic correlation of the image.  The chips are specified
		as 64x64 pixel images surrounding tie points in a tps file.
		The input image must have an associated AVHRR Data Descriptor 
		Record (ADDR) if using the satellite projection, or must have 
		a valid DDR if using another projection.

PROGRAM HISTORY:
Version      Date    Programmer		Request
-------      ----    ----------         -------
  1.0	     06/94   T. Rockvam		Original development; much stolen from
					DCWCHIPS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	Must be run under TAE

PROJECT:	ADAPS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "geompak.h"
#include "wvschips.h"

main()
{
extern 	long curent;		/* Current processing line		*/

struct	PARBLK	vblock;		/* TAE parameter block			*/
struct  DDR     ddr;    	/* DDR information                      */
struct  MODEL   model;		/* AVHRR model				*/
struct  MTPDATA mtp_data;       /* search tps data record 		*/

char	wvspath[CMLEN],		/* location of wvs			*/
	wvsfile[CMLEN],		/* wvs file name			*/
	chip_name[CMLEN], 	/* name of output chip image		*/
	hosin[1][CMLEN],  	/* input host file			*/
	msg[ERRLEN],		/* informational message		*/
	mtpfile[CMLEN],	  	/* mtp file name			*/
	proj[6],		/* projection type (sat or proj)	*/
	taein[1][CMLEN],  	/* input TAE image name			*/
	taeout[CMLEN],		/* output TAE chip image name		*/
	tpsfile[CMLEN];	  	/* tps file name			*/

int     *tps_desc,              /* tps file descriptor                  */
	*mtp_desc;		/* mtp file descriptor			*/

long    backgrnd,		/* background value for rasterized image*/
	bands[1][MAXBND+1], 	/* band number to be processed		*/
	chip_window[4], 	/* window spec for chip area		*/
	curchip = 0,		/* current chip image			*/
	fatal = -1,		/* fatal error code			*/
	imgback = FALSE, 	/* image data in background flag	*/
	msgtim,			/* time between processing messages	*/
	nbands[1],		/* number of bands per input image	*/
	nchips = 0,		/* number of chips successfully created	*/
	nonfatal = 1,		/* nonfatal error code			*/
	nout = 1,		/* number of output chips to create	*/
	pixval,			/* intensity value for rasterized pixels*/
	odtype,			/* output data type			*/
	status,			/* return status variable		*/
	std_out;		/* standard output			*/

double  corner[4]; 		/* corner (lat/long) spec for each chip	*/
double	elev;			/* elevation of tie point		*/

/*
  Initialize the TAE parameter block
*/
c_inlas(&vblock,&std_out);

/*
  Get the input parameters
*/
status = getpar(&vblock,&ddr,&model,wvspath,taein,
	 hosin,tpsfile,mtpfile,bands,nbands,&odtype,&pixval,
	 &backgrnd,&imgback,proj,&msgtim);
if (status == E_FAIL)
    c_errmsg("Fatal error encountered","wvschips-fatal",&fatal);

/*
  Set up tps files and projection stuff
*/
status = set_up(proj,&ddr,tpsfile,mtpfile,&tps_desc,&mtp_desc);
if (status == E_FAIL)
    c_errmsg("Fatal error encountered","wvschips-fatal",&fatal);

while (TRUE)
    {
    /*
      Get the next chip to process
    */
    curchip++;
    if (strcmp(proj,"SAT") == 0)
        {
        status = get_sat_chip(&tps_desc,&mtp_data,&model,&ddr,&elev,chip_window,
			      corner,chip_name,taeout,nchips+1,wvsfile);
        }
    else
        {
        status = get_proj_chip(&tps_desc,&mtp_data,&ddr,chip_window,corner,
			       chip_name,taeout,nchips+1,wvsfile);
        }
    if (status == E_EOF) 
	break;
    else if (status != E_SUCC)
	{
	c_ptclse(&tps_desc);
	c_ptclse(&mtp_desc);
    	c_errmsg("Fatal error encountered","wvschips-fatal",&fatal);
	}

    /*
      Create the chip
    */
    if (strcmp(proj,"SAT") == 0)
        status = satsel(&vblock,&ddr,&model,&elev,hosin,bands,nbands,
			chip_name,wvspath,wvsfile,pixval,
			chip_window,backgrnd,imgback,&msgtim,odtype,
			corner[MINLAT],corner[MINLON],
			corner[MAXLAT],corner[MAXLON]);
    else
        status = projsel(&vblock,&ddr,hosin,bands,nbands,chip_name,
		     wvspath,wvsfile,
		     pixval,chip_window,backgrnd,imgback,&msgtim,odtype,
		     corner[MINLAT],corner[MINLON],
		     corner[MAXLAT],corner[MAXLON]);
    if (status != E_SUCC)
	{
        sprintf(msg,"Error occurred creating %s",taeout);
        c_errmsg(msg,"wvschips-nocreate",&nonfatal);
	}
    else
	{
	nchips++;
        sprintf(msg,"Created %s",taeout);
        c_errmsg(msg," ",&nonfatal);

    	/*
      	  Finish up (update mtp and associated files)
      	  Only write to mtp if the chip was created
    	*/
    	if (c_ptio(&mtp_desc,&mtp_data) == E_FAIL)
	    c_errmsg("Error occurred writing merged tie point file",
		"wvschips-mwrite",&nonfatal);
	}
    }

/*
  Close tie points files
*/
c_ptclse(&tps_desc);
c_ptclse(&mtp_desc);

/*
  If no chips were found, don't keep an empty mtp file
*/
strcpy(taeout, mtpfile);
if (nchips == 0)
    {
    c_delfil(mtpfile);
    nout = 0;
    }

/*
  Issue completion message
*/
sprintf(msg,"Successfully created %d chip images",nchips);
c_errmsg(msg," ",&nonfatal);
c_complt("WVSCHIPS",taeout,&nout);
}
