/******************************************************************************
FUNCTION:	DELALIAS

PURPOSE:	Deletes the specified aliases.
		If an alias name is specified, the alias is deleted.
		If alias text is specified, all aliases assigned to the text
		are deleted.
		If "$*" is specified, all aliases are deleted
		(the user's alias file is removed).

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      10/86    C. Greenhagen	New Development
  1.1	   12/87    B. Ailts	 	Change include dir. spec.
					Use raw 'C' types
  5.0      04/88    D. Hollaren		changed include spec to las
					changed call to cerrmsg to c_errmsg
					and passed code by address
					SUCCESS/FAIL flag to E_SUCC/E_FAIL
  5.1      11/90    D. Etrheim          Corrected for alias of length 9.  Need
					ASZ+2 to handle lf and NULL chars.
  5.2      12/90    D. Etrheim		If an error occurs opening the new
					alias file, attempt to delete it and
					try again.
  5.3      1/91     B. Davis		Standardized errmsg calls and status
					returns according to EDC guidelines.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 

PROJECT			LAS

ALGORITHM 
Get the user's ALIST LIST file directory and name spec.
If "$*" is specified
   remove the user's ALIAS LIST file.
Otherwise,
   open ALIAS LIST file and a temporary alias file.
Make sure that the ALIAS LIST file is not empty.
Copy all the alias names into the temporary alias file
    except the specified name.
Close both files.
Rename the temp file to be the user's new ALIAS LIST file.

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <math.h>

#ifdef vms
#include <file.h>
#else
#include <sys/types.h>
#include <sys/file.h>
#endif

#ifdef sun
#include <fcntl.h> 
#endif

#include "las.h"
#include "diskio.h"   /* c_dkdel prototype */
#include "alias.h"


/**
 * delalias:
 * @instring:  I: input string
 *
 * Deletes the specified aliases.
 * If an alias name is specified, the alias is deleted.
 * If alias text is specified, all aliases assigned to the text
 * are deleted.
 * If "$*" is specified, all aliases are deleted
 * (the user's alias file is removed).
 *
 * Returns: E_SUCC:  Successful completion.
 *          E_FAIL:  Unsuccessful completion.
 */
int delalias
(
    char *instring			/* input string			*/
)
{
#define ASZ	9			/* max length of alias name	*/
#define TSZ	CMLEN-1			/* max length of alias text	*/
char
    aliasfile[CMLEN],			/* name of user's alias file	*/
    aliasname[ASZ+2],			/* alias name			*/
    aliastext[TSZ+1],			/* alias text			*/
    newfile[CMLEN];			/* name of temporary alias file	*/

int
    byname=0,				/* delete by name t=1,f=0	*/
    bytext=0,				/* delete by text t=1,f=0	*/
    deleted=0,				/* found record to delete	*/
    fdnew,				/* file descriptor for new file	*/
    fdold,				/* file descriptor for old file	*/
    flags,				/* flags for file open		*/
    more,				/* more records to read		*/
    status;				/* cmlock status		*/

long  nonfatal = 1;			/* code for non-fatal errors	*/

char  msg[ERRLEN + 1];			/* Error message string		*/

FILE
    *aliasf = NULL,			/* current alias file		*/
    *newf = NULL;			/* new alias file		*/
    
/************************************************************************/

bldalias (aliasfile);				/* build alias file name*/

strcpy (newfile,aliasfile);			/* build temp file name	*/
strcat (newfile,"new");

if (strcmp (instring,"$*") == 0)		/* delete all aliases	*/
    {						/* (delete file)	*/
#   ifdef vms
	delete (aliasfile);
#   else
	unlink (aliasfile);
#   endif
    }

else						/* delete selected alias*/
    {
    if (instring[0] == '$')			/* delete by alias name	*/
	{
	byname = 1;
	if (strlen (instring) > ASZ)
	    {
	    c_errmsg ("Alias name too long","delalias-name",&nonfatal);
	    return(E_FAIL);
	    }
	}
    else					/* delete by alias text	*/
	{
	bytext = 1;
	}

    flags = O_RDONLY;				/* open old alias file	*/
    status = cmlock (aliasfile,&fdold,flags,60);
    if (status == E_SUCC)
	aliasf = fdopen (fdold,"r");	

    if ((status != E_SUCC) || (aliasf == NULL))
	{
	sprintf(msg,"Error opening file %s ",aliasfile);
	c_errmsg(msg,"delalias-open",&nonfatal);
	strcpy(msg,"Error returned from cmlock");
	c_errmsg(msg,"delalias-call",&nonfatal);
	return(E_FAIL);
	}

    flags = O_TRUNC | O_WRONLY | O_CREAT;	/* open new alias file	*/
    status = cmlock (newfile,&fdnew,flags,60);
    if (status == E_SUCC)
	newf = fdopen (fdnew,"w");
    else
       {
       status = c_dkdel(newfile);		/* delete (old)new alias file */
       status = cmlock (newfile,&fdnew,flags,60); /* and try again */
       if (status == E_SUCC)
	  newf = fdopen (fdnew,"w");
       }

    if ((status != E_SUCC) || (newf == NULL))
	{
	c_errmsg ("Unable to create new alias file","delalias-file",&nonfatal);
	strcpy(msg,"Error returned from cmlock");
	c_errmsg(msg,"delalias-call",&nonfatal);
	return(E_FAIL);
	}

    more = 1;		/* copy alias file, omitting selected records	*/
    if ((fgets(aliasname,ASZ+2,aliasf)) == NULL)
	more = 0;
    else if ((fgets(aliastext,TSZ+1,aliasf)) == NULL)
	more = 0;

    while (more)
	{
	aliasname[strlen(aliasname)-1] = 0;
	aliastext[strlen(aliastext)-1] = 0;

	if ((byname && (strcmp (aliasname,instring)!=0)) ||
	    (bytext && (strcmp (aliastext,instring)!=0)))
	    {						/* no match,	*/
	    fputs   (aliasname,newf);			/* copy alias	*/
	    fprintf (newf,"\n");
	    fputs   (aliastext,newf);
	    fprintf (newf,"\n");
	    }
	else
	    deleted = 1;

	if ((fgets (aliasname,ASZ+2,aliasf)) == NULL)
	    more = 0;
	else if ((fgets (aliastext,TSZ+1,aliasf)) == NULL)
	    more = 0;
	}

    if (deleted == 0)
        {
	c_errmsg ("Specified alias not found","delalias-found",&nonfatal);
	return (E_FAIL);
        }

    fclose (aliasf);
    fclose (newf);

#   ifdef vms
    delete (aliasfile);
#   endif

    rename (newfile,aliasfile);			/* rename new file	*/
    }

return (E_SUCC);
}
