#!/bin/csh
#
# FUNCTION:     bldcron.csh
#
# PURPOSE:      Main function to build LAS/ADAPS code.
#               This function must be placed in the cron jobs for the release
#               account on the "main source code" system.  It is currently in
#               release@edcsgs20.cr.usgs.gov:cron/cron.jobs.
#               The output from running this script is put in a log file in
#               "$HOME/bldlog/bldcron_$stamp.log" and then sent to the LAS
#               administrator.
#
#
# PARAMETERS:
#   The first command line parameter should be the CVS repository tag to
#   checkout for the build
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development
# T. Beckmann      07/00  Get the tag to build from the command line


#-----------------------------------------------
# Define the systems this function should run on
#-----------------------------------------------
set admin    = (tbeckman@usgs.gov)
set usr      = release
set bldsys   = (edcsgs20 edcsns1)
set halfline = "====================================="
set tag      = $1

#-----------------------------------------------------------
# If the tag is defined, grab the list of defined builds
#-----------------------------------------------------------
if ($tag != "") then
    set tags = `cat /sgs20/csb/release/bldlog/build.list`
else
    # flag that only the release build is happening
    set tags = (BUILD)
endif

#--------------------------------------
# build the code for each build defined
#--------------------------------------
foreach currtag ($tags)
    # set the output file name for this build
    set stamp    = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`
    set output   = "$HOME/bldlog/bldcron_${tag}_$stamp.log"

    # handle the BUILD tag as the release build
    if ($currtag == "BUILD") then
        set tag = ""
    else
        set tag = $currtag
        # developer defined build, so check if it is really needed
        set build_file = "/sgs20/csb/release/bldlog/build_$tag"
        if (-e $build_file) then
            # echo the build file into the output so it can be determined who
            # requested the build and why
            echo "$halfline$halfline"                      >>& $output
            echo "developer build requested by:"           >>& $output
            cat $build_file                                >>& $output
            echo "$halfline$halfline"                      >>& $output
            rm $build_file
        else
            continue        
        endif
    endif

    #-----------------------
    # Build on each system
    #-----------------------
    foreach system ($bldsys)
        #------------------------------
        # Ensure the build system is up
        #------------------------------
        ssh -q $system -l $usr -n date > /dev/null
        if ($status) then
            echo "ERROR: $system is down at `date`"        >>& $output
            continue
        endif

        #---------------------
        # Build on this system
        #---------------------
        echo "$halfline$halfline"                      >>& $output
        echo "Building on $system at `date`"           >>& $output
        echo "$halfline$halfline"                      >>& $output
        ssh -q $system -l $usr -n build_software.csh $tag >>& $output
        if ($status) then
            echo "ERROR: Building software on $system" >>& $output
        endif

        #------------------------------------------------------------------------
        # Clean up old release directories where applications that are used for
        # making lug notes or force releases when doing a release build
        #------------------------------------------------------------------------
        if ($tag == "") then
            echo "$halfline$halfline"                      >>& $output
            echo "Cleaning up old releases on $system"     >>& $output
            echo "$halfline$halfline"                      >>& $output
            ssh -q $system -l $usr -n clean_release_dir.pl >>& $output
            if ($status) then
                echo "Warning: Release cleanup error on $system" >>& $output
            endif
        endif
        
        #------------------------
        # Spacing in the log file
        #------------------------
        echo ""                                        >>& $output
    end

    #---------------------------------------------
    # Start the ADAPS benchmark for release builds
    #---------------------------------------------
    echo "$halfline$halfline"                          >>& $output
    echo "Starting ADAPS benchmark on sgs20 at `date`" >>& $output
    echo "$halfline$halfline"                          >>& $output

    # start the ADAPS benchmark for the correct build
    if ($tag == "") then
        # release build
        $ADAPSBASE/adapsbench.csh /sgs20/km1/opsavhrr/bmark /sgs20/csb/release/las/release_build >>& $output
    else
        # test build
        $ADAPSBASE/adapsbench.csh /sgs20/km1/opsavhrr/bmark /sgs20/csb/release/las/${tag}_build >>& $output
    endif

    #----------------------------------
    # Mail the results to $adimistrator
    #----------------------------------
    elm -s"$system release `date '+%D %T'`" $admin < $output >& /dev/null
/bin/rm -f $output
end
