#!/bin/perl
# This script is used to build a configured item list
#
# Limitations: The version of code to build a configured item list must be
#              checked out
#
# Parameters:
#   A single directory of the code to build the configured item list
#   is accepted.
#
# Example: 
#   build_cil.pl [-i filename] las
#      will build the configured item list for the las directory
#

use Getopt::Std;

# get the defined options
getopts("i:");

# make sure a directory and only one directory was passed in
if ($#ARGV != 0)
{
    print "usage: build_cil.pl [-i filename] <dir>\n" .
      "    where <dir> is a directory that was checked out of CVS.\n" .
      "      and [-i filename] is a file containing directories to ignore\n";
    exit 1;
}

$dir = $ARGV[0];

# read the ignore file if one was provided
if ($opt_i)
{
    if (! -f $opt_i)
    {
        print "Error: $opt_i does not exist\n";
        exit 1;
    }
    
    open (IGNORE, "< $opt_i") || die ("Error opening $opt_i: $!");
    @ignore_list = <IGNORE>;
    close (IGNORE);
    chomp @ignore_list;
}


# error out if the provided parameter is not a directory
if (! -d $dir)
{
    print "Error: $dir is not a directory.\n";
    exit 1;
}

# find all the files in the directory passed in
open( FINDPIPE, "find $dir -print |" ) || die( "Find pipe: $!" );

# process all the files found
while( <FINDPIPE> )
{
    $file = $_;
    chop( $file );
    if ( -f $file ) 
    {
        # separate the file name into the directory and base file name
        $file =~ ( /^(.+)\/([^\/]+)/ );
        $directory = $1;
        $base = $2;

        # skip the CVS directories
        next if ( $directory =~ /CVS$/ );

        # skip the directory if it matches one in the ignore list
        my $ignored = 0;
        foreach $ignore_pattern (@ignore_list)
        {
            if ($directory =~ /$ignore_pattern/)
            {
                $ignored = 1;
                last;
            }
        }
        next if ($ignored);

        # extract the version number for the current file
        #  Note: this could probably be done faster by using the entries in
        #        the CVS directory
        open( STATUSPIPE, "cvs status \"$file\" |" ) || die("Status pipe: $!");
        $version = "";
        while( <STATUSPIPE> ) 
        {
            $version = $1 if ( /Working revision:\s+(\d+\.\d+)/ );
        }
        close (STATUSPIPE);

        printf( "$file,$version\n") if ($version);
    }
}

close( FINDPIPE );

exit 0;
