#!/bin/csh
#
# FUNCTION:     check_obsolete.csh
#
# PURPOSE:      Identify any obsolete files in the release directories
#               (relative to $EDCSOFT).
#               This routine compares the files in the release directories to
#               the files in the specified build directory (assumed to be the
#               latest build).  If a file in the release directory does not
#               exist in a build directory it is identified and the LAS
#               administrator should check the log file and move the
#               appropriate file to the old directory.
#
#         NOTE: The move has to be a manual operation since there is
#               a possiblity that an application had an error during the
#               build and would not have been removed from the release
#               directory had it compiled.  Also there may be cases when
#               old applications need to remain in the release even
#               though they are no longer built.
#
#         NOTE: This function assumes the directory structures required for
#               a release exist in both $EDCSOFT and the specified build
#               directory so the comparisons can be done.
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

#-------------------------------------------
# Verify the <newdir> argument was specified
#-------------------------------------------
if ($#argv < 1) then
    cat << EOM - 
    Usage: check_obsolete.csh <new_dir>
        Identify the obsolete file in release directories.  This function
        finds obsolete files by comparing the files in the release directory
        (relative to \$EDCSOFT) to the files in the <new_dir> directories.
        When a file in the release directory does not exist in <new_dir>
        directories it is identified in the obsolete report file.  The
        identified files in the report should be reviewd to determine if
        they are actually obsolete files.

        The obsolete files are reported in the $HOME/bldlog/obsolete_tstamp.rpt
        file.
EOM
    exit 1
endif

#--------------------------------------
# Verify the specified directory exists
#--------------------------------------
set new_release = $1
if (! -d $new_release) then
   echo "The new release directory $new_release does not exist"
   exit 1
endif

#--------------------
# Set shell variables
#--------------------
unalias cd
set halfline = "----------------------------------------"
set stamp = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set log_file = "$HOME/bldlog/obsolete_$stamp.rpt"
set obsolete = 0

echo "Identify obsolete files"
#------------------------------------------------------------
# Check for existance of a file in release but not in the new
#------------------------------------------------------------
foreach dir ( \
	build/las/include \
	build/adaps/include \
	build/las/lib \
	build/adaps/lib \
	run/adaps/base \
	run/adaps/contrib \
	run/adaps/local \
	run/las/base \
	run/las/contrib \
	run/las/ditto \
	run/las/local \
	run/bin \
	env/las/tables \
	env/adaps/tables \
	env/las/assign \
	)

    #--------------------------------------
    # Setup directory paths to the software
    #--------------------------------------
    set new_path = $new_release/$dir
    set rel_path = $EDCSOFT/$dir

    #-----------------------------
    # Verify directory paths exist
    #-----------------------------
    if (! -d $new_path) then
        echo "The new release directory $new_path does not exist" >> $log_file
    else if (! -d $rel_path) then
        echo "The release directory $rel_path does not exist" >> $log_file
    else
        #--------------------------------------------------------------
        # Check binaries in the release directory to see if they exists
        # in the new build binaries.
        #--------------------------------------------------------------
        cd $rel_path
        echo "Checking obsolete files in $dir"  >> $log_file
        echo "$halfline$halfline" >> $log_file
        set obsolete = 1

        foreach file (*)
            #------------------------------------------------------------
            # When the file in the release directory doesn't exist in the
            # new directory log it.  See NOTES above.
            #------------------------------------------------------------
            if (! -e $new_path/$file) then
                echo "$file is obsolete, move it to old" >> $log_file
                set obsolete = 2
            endif
        end

        #------------------------
        # Spacing in the log file
        #------------------------
        echo "" >> $log_file
        echo "" >> $log_file

        #----------------------------------------------
        # Log a message if no obsolete files were found
        #----------------------------------------------
        if ($obsolete == 1) then
            echo "No obsolete files found in $dir" >> $log_file
        endif
    endif
end

#-----------------------------------------------------
# Log that an error occured or a successful completion
#-----------------------------------------------------
if ($obsolete == 0) then
    echo "ERROR: No directories found for checking obsolete files"
    echo "ERROR: No directories found for checking obsolete files" >> $log_file
    exit 1
else
    echo "Obsolete file checking completed successfully"
    echo "Obsolete file checking completed successfully"  >> $log_file
endif

exit 0
