#!/usr/local/bin/perl
#
# This script is used to recursively decend into an directory structure that
# has been checked out of CVS and checkout a version of code that has the 
# tag defined.  For the directory where the tag does not exist, the version 
# already checked out there will remain.

# flush output buffers immediately
$| = 1;

# make sure only one tag is provided
if ($#ARGV != 1)
{
    print "usage: checkout_tag.pl <cvs_dir> <cvs_tag>\n";
    exit 1;
}

# get the directory from the parameters and make sure it exists
my $cvs_dir = $ARGV[0];
if (!(-d $cvs_dir))
{
    print "ERROR: The directory $cvs_dir does not exist\n";
    exit 1;
}

# get the tag from the argument (tag is global)
my $tag = $ARGV[1];

#checkout the directory
checkout_dir ($cvs_dir, $tag);

exit 0;

# checkout_dir recursively searches for directories where the tag is defined
# and checks out the associated files
#    inputs: directory name to search
#            tag 
#    returns: nothing
sub checkout_dir
{
    my $dir = @_[0];
    my $tag = @_[1];

    # change into the directory 
    chdir $dir || die "Error changing to directory $dir: $!\n";

    # check if there are any files in the directory to check for a tag
    my $file = find_CVS_file($dir);

    if ($file ne "")
    {
        my $tag_found = check_for_tag ($file, $tag);

        if ($tag_found) 
        {
            # The tag was found, so update the code with that tag
            # and return since all directories under this one will be
            # recusively updated
            my $current_dir = `pwd`;
            chomp $current_dir;
            print "------------------------------------\n";
            print "Checking out tag $tag in $current_dir\n";
            my $status = system ("cvs update -r $tag"); 
            $status >>= 8;
            print "Error checking out $tag tag in $current_dir\n" 
                if ($status != 0);
            return;
        }
    }

    # recursively parse the directories found in this directory since the 
    # tag was not found in this directory
    foreach my $recurse_dir (<*>)
    {
        # recursively process all directories except the CVS directory
        if ($recurse_dir !~ /^CVS$/)
        {
            if (-d $recurse_dir)
            {
                # recursively check the next directory
                checkout_dir($recurse_dir, $tag);

                # change back to the current directory
                chdir ".." || die "Error changing to directory $dir: $!\n";
            }
        }
    }
}


# find_CVS_file looks for the name of a file in CVS by looking in the 
# CVS/Entries file for a filename that exists
#    inputs: current directory name
#    returns: name of file found or an empty string if no file found
sub find_CVS_file
{
    my $dir = @_[0];
    my $file = "";
    
    # look in the CVS/Entries file for a non-directory entry
    open (ENTRIES, "CVS/Entries") || die "Error opening $dir/CVS/Entries: $!\n";

    while (<ENTRIES>)
    {
        # skip directories
        next if (/^D/);

        # get the filename from the line
        my @fileline = split('/');
        $file = $fileline[1];

        # if this file exists, exit the loop
        last if (-e $file);
    }
    close (ENTRIES);
    
    return $file;
}

# check_for_tag searches a CVS log for a tag
#   inputs: filename to perform the CVS log on
#           tag
#   returns: 1 if the tag is found, 0 if the tag is not found
sub check_for_tag
{
    my $file = @_[0];
    my $tag = @_[1];

    # initialize flags
    my $in_tags = 0;
    my $tag_found = 0;

    # open a pipe to a cvs log of the file and look for the tag
    open (CVSLOG, "cvs log -h $file|") || die "Can't open a cvs log: $!\n";
    while (<CVSLOG>)
    {
        if (!$in_tags)
        {
            # looking for start of tags marker
            if (/symbolic names/)
            {
                $in_tags = 1;
            }
        }
        else
        {
            # bail out if the end of tags marker is found
            last if (/keyword substitution/);

            # check for the requested tag
            if (/\b$tag\b/)
            {
                $tag_found = 1;
                last;
            }
        }
    }
    close (CVSLOG);

    return $tag_found;
}

