#!/bin/csh
#
# FUNCTION:     force_distribute.csh
#
# PURPOSE:      Distribute software to associated systems.
#
# $1 - Module to be force distributed.  This is actually a directory that
#      contains the binaries for the module to be released.
#      e.g.) $LASBASE/release/<module>
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

#-----------------------------------------------
# Define the systems this function should run on
#-----------------------------------------------
set usr      = release
set halfline = "====================================="
set stamp    = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set force_pattern = "FORCE_*.log"
set force_file = "FORCE_$stamp.log"
set assoc_sun  = ()
set assoc_sgs20  = ()
set system = `uname -n`
set found_dir = 0

#------------------------------------------------------
# Verify the correct number of arguments were specified
#------------------------------------------------------
if ( $#argv < 1 ) then
    goto USAGE
endif

#----------------------------------------------
# Get the name of the file to be force released
#----------------------------------------------
set module = $1

#-------------------------------------------------
# Determine the associated systems to copy code to
#-------------------------------------------------
if ($system == "edcsgs20") then
    set assoc_systems = ($assoc_sgs20)
    set install_dir = "/sgs20/csb/release/las/release_build"
else if ($system == "edcsns1") then
    set assoc_systems = ($assoc_sun)
    set install_dir = "/edcsns1/csb2/release/release_build"
else
    set assoc_systems = ""
endif

#-------------------------------------------
# Ensure the all distribution systems are up
#-------------------------------------------
foreach sys ($assoc_systems)
    ssh -q $sys -l $usr -n date > /dev/null
    if ($status) then
        echo "ERROR: $sys is down at `date`"
        echo "       Please force distribute when the system is up."
        exit 1
    else
        echo "  $sys is up"
    endif
end

#------------------------
# Spacing in the log file
#------------------------
echo ""

# ON the distribution system

#------------------------------
# Find the directory to release
#------------------------------
foreach dir ( \
    run/adaps/base \
    run/adaps/contrib \
    run/adaps/local \
    run/las/base \
    run/las/contrib \
    run/las/ditto \
    run/las/local \
    run/aircraft/base \
    env/las/tables \
    env/adaps/tables \
    )

    set force_path = $EDCSOFT/$dir/release/$module
    if ( -d $force_path) then
        set found_dir = 1
        cd $force_path
        echo "Distributing $module on $system at `date`"

        #---------------------------------------------------------------
        # Remove any already existing force files in case someone forces
        # the release more than once
        #---------------------------------------------------------------
        /bin/rm $force_path/$force_pattern

        #------------------------------------------------
        # Release the software on the distribution system
        #------------------------------------------------
        release_copy.csh $force_path $EDCSOFT/$dir
        if ($status) then
            echo "ERROR: Force releasing $module to $system"
            exit 1
        endif

        #------------------------------------------------------------
        # Copy the software to the weekly build directory in case the
        # force release is happening after the weekly build
        #------------------------------------------------------------
        release_copy.csh $force_path $install_dir/$dir
        if ($status) then
            echo "ERROR: Force releasing $module to $system in $install_dir"
            exit 1
        endif

        #------------------------
        # Spacing in the log file
        #------------------------
        echo ""

        #-------------------------------------------------------
        # Distribute the software to the systems associated with
        # the distribution system
        #-------------------------------------------------------
        foreach sys ($assoc_systems)
            echo "Distributing $module from $system to $sys"

            #---------------------
            # Distribute the files
            #---------------------
            scp -qp -r * ${user}@${sys}:'$EDCSOFT'/$dir
            if ($status) then
                echo "ERROR: Distributing $module to $sys"
                exit 1
            endif
        end

        #----------------------------------------------------------------
        # Create a "force" file that the weekly LUG notes can key off of.
        #----------------------------------------------------------------
        /bin/ls -1 > $force_path/$force_file

        #----------------
        # Force released!
        #----------------
        echo "Successfully force released $module"
        exit 0
    endif
end 

if (found_dir == 0) then
    echo "ERROR: $module NOT released, directory not found"
endif
exit 0

USAGE:
cat << EOM
usage: force_distribute.csh <module>

       Force distribute the specified module.

       <module> - Name of the module to force distribute.  The <module> name
                  is actually a directory name under a "release" directory.
                  e.g.) $LASBASE/release/<module>
                  This directory contains the actual binaries to distribute.
EOM
exit 1
