#!/bin/csh
#
# FUNCTION:     taelas_build.csh
#
# PURPOSE:      Build TAE, LAS, and ADAPS software in the specified
#               installation directory.  The installation directories
#               are cleaned of old object and binary files and then built.
#               A "clean.log" and "build.log" are created in the installation
#               directory being cleaned/built.
#
#        NOTE:  The installation directory must exist and contain the
#               sub-directories necessary for the build.  Run lasdir_setup.csh
#               in the installation directory to create the proper directories.
#
# Usage:
#       -  Option to build -tae, -las, or -all
#   $1  -  Installation directory
#   $2  -  Debug option.  Set to 1 to set compile flags for debug
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

#------------------------------------------------------
# Verify the correct number of arguments were specified
#------------------------------------------------------
if ( $#argv < 1 ) then
    goto USAGE
endif

#------------------------------------------------------
# Save the input arguments to meaningful variable names
#------------------------------------------------------
set install_dir = $1
set tae_dir = $install_dir/tae
set source_dir = $install_dir/src
set debug_flag = $2
unset debug

#----------------------------------------
# Make sure the source directories exists
#----------------------------------------
if (! (-d $tae_dir)) then
   echo "TAE source directory $tae_dir does not exist"
   echo ""
   exit 1
endif

if (! (-d $source_dir/las)) then
   echo "LAS source directory $source_dir/las does not exist"
   echo ""
   exit 1
endif

if (! (-d $source_dir/adaps)) then
   echo "ADAPS source directory $source_dir/adaps does not exist"
   echo ""
   exit 1
endif

#---------------------------------------------------------------
# Distribute the scripts for assigning the LAS/ADAPS environment
# to appropriate locations so the setup scripts can be run
#---------------------------------------------------------------
cd $source_dir/las/env/assign
cp lasinit.generic lasinit
cp masterlog.generic masterlog
cp pgmassign.generic pgmassign
cp usrassign.generic usrassign

setenv DSTDIR $install_dir/env/las/assign
make las
unsetenv DSTDIR

#-----------------------------------------
# Change to the LAS/ADAPS source directory
#-----------------------------------------
cd $source_dir

#---------------------------------------------------------------
# Copy the scripts necessary to setup the system for a new build
#---------------------------------------------------------------
# LAS build script
cp las/env/install/Build.las .
chmod 755 Build.las

#----------------------------------------
# Set environment variables for the build
#----------------------------------------
setenv EDCSOFT $install_dir
setenv TAE $EDCSOFT/tae
source $install_dir/env/las/assign/lasenv.csh

#-------------------------------------------------------------------
# Use the default compile flags unless the debug option flag was set
#-------------------------------------------------------------------
if ($debug_flag == 1) then
    set flags = `echo $CFLAGS | sed s/-O/-g/ | sed s/" -s"// `
    setenv CFLAGS "$flags"
    set flags = `echo $FFLAGS | sed s/-O/-g/ | sed s/" -s"// `
    setenv FFLAGS "$flags"
    set debug = "-debug"
endif

#-----------------------------------
# Change to the TAE source directory
#-----------------------------------
cd $tae_dir
echo "Building TAE in $tae_dir"

#----------
# Build TAE
#----------
#-----------------------------------------------------------------------------
# Determine what target to use for the make.  If there is a Build script, it
# will take care of making the application.  If there isn't, search for "las:"
# or "all:" in the Makefile.  Do not use the -s option of egrep because it is
# not supported on the DGs.
#-----------------------------------------------------------------------------
if (-e Makefile) then
    make clean  >& clean.log
    if ($status) then
        echo "ERROR: Cleaning TAE"
        exit 1
    endif

    # Set environment variables that are referenced in the Makefiles.
    setenv TAETM $TAE/src/tm
    setenv TAETAELIB $TAE/src/lib/tae
    make install >& build.log
    if ($status) then
        unsetenv TAETM
        unsetenv TAETAELIB
        echo "ERROR: Building TAE"
        exit 1
    endif
    unsetenv TAETM
    unsetenv TAETAELIB
else
    installtae >& build.log
    if ($status) then
        echo "ERROR: Installing TAE"
        exit 1
    endif
endif

#-----------------------------------------
# Change to the LAS/ADAPS source directory
#-----------------------------------------
cd $source_dir

#------------------------------------------------------
# Clean the LAS/ADAPS directories before building.
# This is done in case a previous build existed before.
#------------------------------------------------------
echo "Cleaning LAS/ADAPS in $source_dir"
Build.las -clean >& clean.log
if ($status) then
    echo "ERROR: Cleaning LAS/ADAPS"
    exit 1
endif

#----------------
# Build LAS/ADAPS
#----------------
echo "Building LAS/ADAPS in $source_dir"
Build.las >& build.log
if ($status) then
    echo "ERROR: Building LAS/ADAPS"
    exit 1
endif

echo "TAE/LAS/ADAPS build completed successfully"
exit 0

USAGE:
cat << EOM
usage: taelas_build.csh <install_dir> [<debug_flag>]

       Build TAE, LAS, and ADAPS software in the specified <install_dir>
       directory.  The <install_dir> sub-directories are cleaned of old object
       and binary files and then built.  A "clean.log" and "build.log" are
       created in the installation directory being cleaned/built.

       This function assumes that the <install_dir> exists, TAE source exits
       under <install_dir>/tae, and LAS/ADAPS source exists under
       <install_dir>/src.

       <install_dir> - Directory to install the TAE, LAS, and ADAPS executables.
                       This directory must exist and contain the sub-directories
                       needed for the installation.  Run "lasdir_setup.csh" in
                       the <install_dir> to create the proper directories.

       <debug_flag> -  When set to 1 the compile flags will be set for debug.
EOM
exit 1
