#!/bin/csh
#
# FUNCTION:     compile_one_system.csh
#
# PURPOSE:      Compile one module on the current (distribution) system.  It is
#               assumed the user running this function has scp keys setup for
#               the specified host and source directory.  All output should be
#		        echo'd to standard output so the invoking command can capture
#		        the messages.  User is release or ddp.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# J.Fenno          04/94  Development
# J.Fenno          06/94  Added code to distribute the executables to other
#                         associated systems.
# J.Fenno          12/94  Added additional argument for the documentation
#                         directory.  If the makefile contains a 'doc:' target,
#                         DSTDIR is set to $EDCSOFT/$docdir and 'make doc' is
#                         run after the 'make las'.
# J.Fenno          10/95  Modified so the documentation for all applications is
#                         only built on $docsys.
# D.Etrheim        07/96  Modified to build to "release/func_name" to allow for 
#                         "forced" distributions 
# D.Etrheim        07/96  Changes due to the demise of sg2 
# D Hollaren       02/00  The new release procedure doesn't promote from the
#                         release directories any more, so distributing to
#                         associated systems is no longer required.  Removed
#                         the distribution to other systems.  Use the
#                         force_release.csh to distribute in emergencies.
#
#-------------------------------------------------------------------------------
# Constants
#    copyonly:  File containing the name of each file that must be copied for
#               compilation.  This is only used for a destination directory
#               matching build/adaps/lib or build/las/lib.
#    docsys:    Documentation system.  This is the WWW server for all LAS and
#               ADAPS documentation.
#-------------------------------------------------------------------------------
set copyonly = copy.these.supports.only
set docsys   = edcsgs20

#-----------------
# Set up the shell
#-----------------
if ($?noclobber) then
    unset noclobber
endif
unalias cd

#------------------
# Get the arguments
#------------------
if ($#argv < 4) then
    echo 'ERROR: invalid number of arguments'
    echo -n 'usage: compile_one_system.csh <srcsys> <srcdir> <dstdir> <docdir>'
    echo ''
    echo '       Compile the source code for one application or module on the'
    echo '       current system.'
    echo ''
    echo '       <srcsys> is the name of the computer where the source code'
    echo '       resides.'
    echo ''
    echo '       <srcdir> is the directory that contains the source code.  The'
    echo '       directory may be an absolute or relative path.'
    echo ''
    echo '       <dstdir> is a directory, relative to $EDCSOFT, where the'
    echo '       make las command should place the appropriate files.'
    echo ''
    echo '       <docdir> is a directory, relative to $EDCSOFT, where the'
    echo '       make doc command should place the documentation files.'
    echo ''
    echo '       <systems> is a list of other computer systems to which the'
    echo '       generated files should be scpd to.'
    exit 1
endif

set srcsys = $1
set srcdir = $2
set dstdir = $3
set docdir = $4
shift
shift
shift
shift
set assoc_systems = ( $* )

#-----------------------------
# Create a temporary directory
#-----------------------------
set stamp = `date '+%d_%m_%y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set tmpdir = "compiledir.$stamp"
onintr -
mkdir $tmpdir
if ($status) then
    echo "ERROR: creating temporary directory"
    exit 1
endif

#-----------------------------
# Ensure the group is software
#-----------------------------
chgrp software $tmpdir
chmod 775 $tmpdir
chmod g+s $tmpdir
cd $tmpdir
if ($status) then
    echo "ERROR: accessing temporary directory"
    /bin/rm -rf $tmpdir
    exit 1
endif
onintr FATAL

#------------------------------------------------------------------------------
# If we are compiling a support library, copy only those
# files that are named in the $copyonly file in the source directory.  If we
# are not compiling a support library, copy all source code that exists in the
# source directory.
#------------------------------------------------------------------------------
if (("$dstdir" == "build/adaps/lib") || \
    ("$dstdir" == "build/aircraft/lib") || \
    ("$dstdir" == "build/elips/lib") || \
    ("$dstdir" == "build/las/lib")) then
    scp -qpr ${srcsys}:"$srcdir/$copyonly" .
    foreach file (`cat $copyonly`)
	    scp -qpr ${srcsys}:"$srcdir/$file" .
    end
    # remove the file that contains the list of library files 
    /bin/rm -f $copyonly
else
    scp -qpr ${srcsys}:"$srcdir/*" .
endif

# TEMP DIR (DSTDIR) FOR EXECUTABLES
#------------------------------------------------------------------------------
# Create a temporary directory and set the environment variable DSTDIR to point
# to this directory.  When the application is compiled, all executable code
# will be placed in this directory.  These files can then be copied to the
# associated systems and then moved into the $EDCSOFT/$dstdir directory.  This
# is done in case the compile failed.
#------------------------------------------------------------------------------
mkdir _tmp_dstdir_
chmod 775 _tmp_dstdir_
chmod g+s _tmp_dstdir_

setenv DSTDIR `pwd`/_tmp_dstdir_

# BUILD THE CODE
#-----------------------------------------------------------------------------
# Determine what target to use for the make.  If there is a Build script, it
# will take care of making the application.  If there isn't, search for "las:"
# or "all:" in the Makefile.  Do not use the -s option of egrep because it is
# not supported on the DGs.
#-----------------------------------------------------------------------------
if ((-e Build) && (-x Build)) then
    Build
    if ($status) goto BUILDERR
else if (`egrep las: Makefile | wc -l`) then
    make las
    if ($status) goto BUILDERR
else if (`egrep all: Makefile | wc -l`) then
    make all
    if ($status) goto BUILDERR
else
    make
    if ($status) goto BUILDERR
endif

#------------------------------------------------------------------------------
# Move into the temporary destination directory, copy all existing files to the
# associated systems, and then move them into $EDCSOFT/$dstdir on the current
# system.
#
# This step must be done for library distribution.
# This step is also done for applications so emergency releases can be done and
# as a verification of the release process.  The lug note generation process
# will clean the directory and report any difference.
#------------------------------------------------------------------------------
cd _tmp_dstdir_
set nfiles=`/bin/ls | /bin/wc -l`
if ($nfiles != 0) then
    #---------------------------------------------------------
    # The distribution should only occur for ddps applications
    #---------------------------------------------------------
    foreach sys ($assoc_systems)
        echo ""
        echo "Distributing files to $sys"
        ssh -q ${user}@${sys} "if (! -e "'$EDCSOFT'"/$dstdir) then \
            echo mkdir "'$EDCSOFT'"/$dstdir \
            mkdir "'$EDCSOFT'"/$dstdir \
            endif" 
        echo scp -qpr * ${user}@${sys}:'$EDCSOFT'/$dstdir
        scp -qpr * ${user}@${sys}:'$EDCSOFT'/$dstdir
    end

    echo ""
    echo "Placing files in DSTDIR"
    if (! -d $EDCSOFT/$dstdir) then
        echo "Creating directory $EDCSOFT/$dstdir"
        /bin/mkdir -p $EDCSOFT/$dstdir
    endif

    echo "Copying binaries to $EDCSOFT/$dstdir"
    /bin/cp -pr * $EDCSOFT/$dstdir
endif

cd ../

#-----------------------------------------
# Distribute documentation on $docsys only
#-----------------------------------------
if (`uname -n` == "$docsys") then
    #-------------------------------------------------------------------------
    # Make the documenation.  Only attempt to run "make doc" if the specified
    # documentation directory is not "invalid" and there is a "doc:" target in
    # the makefile.  Do not use the -s option of egrep because it is not
    # supported on all systems.
    #-------------------------------------------------------------------------
    if (("$docdir" != "invalid") && (-e Makefile) && \
        (`egrep doc: Makefile | wc -l`)) then
	    echo ""
	    echo "Making documentation on $docsys"
	    setenv DSTDIR $EDCSOFT/$docdir
	    make doc
	    if ($status) goto DOCERR
    endif
endif


#----------------------------------------------------------------------------
# Compile was successful.  Remove the temporary directory and all source code
#----------------------------------------------------------------------------
cd ../
/bin/rm -rf $tmpdir

#--------------
# End of script
#--------------
exit 0

#-----------------
# All build errors
#-----------------
BUILDERR:
    onintr -
    echo "ERROR: compile failed on `uname -n`"
    goto FATAL

#-------------------------
# All documentation errors
#-------------------------
DOCERR:
    onintr -
    echo "ERROR: documentation installation failed on `uname -n`"
    goto FATAL

#-------------------------------
# All fatal errors or user abort
#-------------------------------
FATAL:
    onintr -
    cd
    /bin/rm -rf $tmpdir
    exit 1
