#include "ditbrowse.h"

#define STRLEN    256		// Length of strings for reading from a file
#define SIZE      121		// Bytes per line in the listing file
#define PATHLEN    78		// Number of characters allowed for the path
#define HEADER    (23 * SIZE)	// Offset to start of the first scene ID
#define JPEG_FLAG  38		// Index of jpeg completion flag in a line

#define ANOMOLY_STR "*****"	// String that begins the anomoly section

extern class Log log;


/*******************************************************************************
                        make_line
*******************************************************************************/
static void make_line (
    char *id,		// IN  - Scene ID
    long fnum,		// IN  - Expected file number on tape
    long tnum,		// IN  - Actual file number on the tape
    long quality,	// IN  - JPEG quality value
    long tflag,		// IN  - Has the image been read from tape yet?
    long cflag,		// IN  - Were the corner coordinates rotated 90 degrees?
    long fflag,		// IN  - Was the image flipped?
    long jflag,		// IN  - Has a jpeg image been created yet?
    long eflag,		// IN  - Was an error encountered with this image?
    long nflag,		// IN  - Do the image names conflict?
    char *path,		// IN  - Location of the created jpeg image
    char *outline	// OUT - Formatted line for the listing file
    )
{
char tmp[STRLEN];	// Copy of the jpeg file output path
char cflg;		// Character for the corner rotation flag
char eflg;		// Character for the error flag
char fflg;		// Character for the flip flag
char nflg;		// Character for the name flag
char jflg;		// Character for the jpeg creation flag
char tflg;		// Character for the tape read flag


/*
 * Ensure the path of the image isn't too long
 */
strcpy(tmp, path);
if (strlen(tmp) > PATHLEN)
    {
    tmp[PATHLEN - 4] = '\0';
    strcat(tmp, "...");
    }

/*
 * Convert the flag values to the correct character
 */
tflg = cflg = fflg = jflg = eflg = nflg = '-';
if (tflag)
    tflg = 'T';
if (cflag)
    cflg = 'C';
if (fflag)
    fflg = 'F';
if (jflag)
    jflg = 'J';
if (eflag)
    eflg = 'E';
if (nflag)
    nflg = 'N';


/*
 * Create the output line
 */
sprintf(outline, "%-16.16s %06d %06d %03d %c%c%c%c%c%c %*s",
    id, fnum, tnum, quality, tflg, cflg, fflg, jflg, eflg, nflg, -PATHLEN, tmp);
}



/*******************************************************************************
                        List::start
*******************************************************************************/
void List::start(char *logpath, char *tapeid)
{
char word[STRLEN];	/* -the first word from 'line'.         */
char fname[STRLEN];	/* -the input file name                 */
char line[STRLEN];	/* -one line of text from a file        */

int fnum = 0;		/* -current file number in listing      */
int trial;		/* -First try the newest (3rd) ...      */

FILE *ifp;		/* -ifp to the tape directory file      */
FILE *ofp;		/* -ofp to the listing file             */


/*
 * Open the file
 */
sprintf(list_file_name, "%s/%s/%s", logpath, tapeid, "ditbrowse.list");
if ((ofp = fopen(list_file_name, "r")) == NULL)
    {
    if ((ofp = fopen(list_file_name, "w")) == NULL)
        {
	fprintf(stderr,
	    "[ditbrowse-nolist] Could not create listing file: %s\n",
	    list_file_name);
	exit(1);
        }
    else
        log.Report(FALSE,"newlist", "Created new listing file: %s",
            list_file_name);
    }
else 
    { 
    /*
     * The file already exists so use it
     */
    log.Report(FALSE,"oldlist", "Using existing listing file");
    return;
    }


/*
 * Creating a new file so print the header information
 */
fprintf(ofp, "Tape ID:         %-6.6s %96s\n", tapeid, " ");
fprintf(ofp, "Total/Completed: 000000/000000 %89s\n", " ");
fprintf(ofp, "Anomalies:       000000 (listed after \"%s\") %73s\n",
    ANOMOLY_STR, " ");
fprintf(ofp, "%120s\n", " ");


/*
 * Print the next eight lines
 */
fprintf(ofp, "%-120s\n%-120s\n%-120s\n%-120s\n%-120s\n%-120s\n%-120s\n%-120s\n",
    "Scene ID = ID of the browse image",
    "FNum     = Expected file number on the tape",
    "TNum     = Actual file number on the tape",
    "Q        = JPEG quality factor",
    "Status   = Six characters indicating the current status of the browse",
    "           image.  Six dashes indicate the image has not been read",
    "           from tape.  The first character will be set to 'T' when the",
    "           image is read from tape, the second will be set to 'C' if");


/*
 * Print the next seven lines
 */
fprintf(ofp, "%-120s\n%-120s\n%-120s\n%-120s\n%-120s\n%-120s\n%-120s\n",
    "           the corner coordinates are rotated ninety degrees, the",
    "           third character will be set to an 'F' if the image is",
    "           flipped (rotated 180 degrees), the fourth character will be",
    "           set to 'J' when the jpeg image is written to disk, the",
    "           fifth character will be set to 'E' if an error is",
    "           encountered, and the sixth character is set to 'N' if the",
    "           scene ID from the browse header does not agree with the");


/*
 * Print the last four lines
 */
fprintf(ofp, "%-120s\n%-120s\n%-120s\n%s\n",
    "           scene ID in the ANSI header file.",
    " ",
    "Scene ID         FNum   TNum   Q   Status Path",
    "---------------- ------ ------ --- ------ ------------------------------------------------------------------------------");

/*
 * Open the newest tape directory file from Kodak.  Some tapes have several
 * copies of this file.  They are all named as <tapeid>.copy<x>.listing where
 * <tapeid> is the ID of the tape and <x> is the number of the file.
 */
for (trial = 10; trial>0; trial--)
    {
    sprintf(fname,"%s/%s/%s.copy%d.listing", logpath, tapeid, tapeid, trial);
    if ((ifp = fopen(fname, "r")) != NULL)
        break;
    }
if (ifp == NULL)
    log.Report(TRUE, "needlist", "Need file: %s\n", fname);


/*
 * Extract the scene IDs sequentialy and build the listing file
 */
while (fgets(line, STRLEN, ifp) != NULL)
    {
    /*
     * Skip all blank lines
     */
    if (sscanf(line, "%s", word) != 1)
	continue;
    

    /*
     * If the line begins with "End" then we have found the end of the file
     */
    if (strcmp(word, "End") == 0)
	break;
    

    /*
     * If this is a scene ID then copy it to the listing file
     */
    if (strncmp(word, "DS", 2) == 0)
	{
	make_line(word, ++fnum, 0, 0, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
	    " ", line);
	fprintf(ofp, "%-120.120s\n", line);
	}
    }


/*
 * Add a string that indicates the start of the anomoly section
 */
fprintf(ofp, "%-120s\n", ANOMOLY_STR);

fclose (ofp);
fclose (ifp);
}



/*******************************************************************************
                        List::update
*******************************************************************************/
void List::update (
    char *scene,	// IN - Scene ID to be updated
    long tnum,		// IN - Actual file number on the tape
    long quality,	// IN - JPEG quality factor
    long tflag,		// IN - Has the image been read from tape yet?
    long cflag,		// IN - Were the corner coordinates rotated 90 degrees?
    long fflag,		// IN - Was the image flipped?
    long jflag,		// IN - Has a jpeg image been created yet?
    long eflag,		// IN - Was an error encountered with this image?
    long nflag,		// IN - Is there a conflict in the image names?
    char *path		// IN - File path for the jpeg image
    )
{
char line[STRLEN];	// Current line from the listing file
char id[STRLEN];	// Scene ID found in first word of line

int found = FALSE;	// Flag if scene ID is found in listing file

long fnum;		// File number expected on the tape

FILE *listfp;		// The listing file pointer


/*
 * Open the listing file
 */
if ((listfp = fopen(list_file_name,"r+")) == NULL)
    {
    fprintf(stderr,
	"[ditbrowse-nolist] Could not open listing file: %s\n",
	list_file_name);
    exit(1);
    }


/*
 * Seek to the expected location for this image and check the scene IDs
 */
fseek(listfp, (HEADER + (tnum - 1) * SIZE), 0);
fgets(line, STRLEN, listfp);
sscanf(line, "%s %d", id, &fnum);


/*
 * Found the expected scene ID so update it and quit
 */
if (strcmp(scene, id) == 0)
    {
    fseek(listfp, -SIZE, 1);
    make_line(scene, fnum, tnum, quality, tflag, cflag, fflag, jflag, eflag,
	nflag, path, line);
    fprintf(listfp, "%-120.120s\n", line);
    fclose(listfp);
    return;
    }


/*
 * The scene ID was not located where we expected it.  Start searching through
 * the listing file in sequential order.
 */
fseek(listfp, HEADER, 0);
while (fgets(line, STRLEN, listfp) != NULL)
    {
    /*
     * Read one line and check the scene ID
     */
    sscanf(line, "%s %d", id, &fnum);
    if (strcmp(scene, id) == 0)
	{
	found = TRUE;
	break;
	}
    }

/*
 * If we found the scene ID then set the file pointer to update this line.
 * If the scene ID was not found, we must be at the end of the file.  Print a
 * line at the current file position (ie. in the Anomoly section.
 */
if (found)
    fseek(listfp, -SIZE, 1);
else
    fnum = 0;  // We didn't expect this scene 


/*
 * Generate the new line for the file
 */
make_line(scene, fnum, tnum, quality, tflag, cflag, fflag, jflag, eflag,
    nflag, path, line);
fprintf(listfp, "%-120.120s\n", line);
fclose(listfp);
}



/*******************************************************************************
                        List::finish
*******************************************************************************/
void List::finish(void)
{
char line[STRLEN];	// Line of text from a file
char id[20];		// Scene ID for the current line of the file

int Anomoly    = FALSE;
int numanomoly = 0;
int numdone    = 0;
int numfiles   = 0;

FILE *listfp;		// File pointer to the listing file


/*
 * Open the listing file
 */
if ((listfp = fopen(list_file_name,"r+")) == NULL)
    {
    fprintf(stderr,
	"[ditbrowse-nolist] Could not open listing file: %s\n",
	list_file_name);
    exit(1);
    }


/*
 * Gather statistics from the file
 */
fseek(listfp, HEADER, 0);
while (fgets(line, STRLEN, listfp) != NULL)
    {
    sscanf(line,"%s", id);
    
    /*
     * Is this the beginning of the anomoly section?
     */
    if (strcmp(id, ANOMOLY_STR) == 0)
        {
        Anomoly = TRUE;
        continue;
        }


    if  (Anomoly)
        numanomoly++;
    else
        {
	/*
	 * Count the total number of images and the number that are done
	 */
        numfiles++;
        if (line[JPEG_FLAG] == 'J')
            numdone++;
        }
    }


/*
 * Update the statistical portion of the file
 */
fseek(listfp, SIZE, 0);
fprintf(listfp, "Total/Completed: %06d/%06d %79s\n", numfiles, numdone, " ");
fprintf(listfp, "Anomolies:       %06d (listed after %s%-76s\n",
    numanomoly, ANOMOLY_STR, ")");
fclose (listfp);
}



/*******************************************************************************
                        List::expect
*******************************************************************************/
short List::expect(void)
{
char line[STRLEN];	// Line of text from a file
char id[20];		// Scene ID for the current line of the file

int numfiles   = 0;

FILE *listfp;		// File pointer to the listing file


/*
 * Open the listing file
 */
if ((listfp = fopen(list_file_name,"r")) == NULL)
    {
    fprintf(stderr,
	"[ditbrowse-nolist] Could not open listing file: %s\n",
	list_file_name);
    exit(1);
    }


/*
 * Gather statistics from the file
 */
fseek(listfp, HEADER, 0);
while (fgets(line, STRLEN, listfp) != NULL)
    {
    sscanf(line,"%s", id);
    
    /*
     * Is this the beginning of the anomoly section?
     */
    if (strcmp(id, ANOMOLY_STR) == 0)
        break;

    /*
     * Count the total number of images and the number that are done
     */
    numfiles++;
    }


/*
 * Close the file and return the number of expected images
 */
fclose (listfp);
return(numfiles);
}
