/*------------------------------------------------------------------------------
    ######                  ######
    #     #     #     ##### #     #  #####    ####   #    #   ####   ######
    #     #     #       #   #     #  #    #  #    #  #    #  #       #
    #     #     #       #   ######   #    #  #    #  #    #   ####   #####
    #     #     #       #   #     #  #####   #    #  # ## #       #  #
    #     #     #       #   #     #  #   #   #    #  ##  ##  #    #  #
    ######      #       #   ######   #    #   ####   #    #   ####   ######

    PROGRAM: DITBROWSE

    WRITTEN BY: TROY WOLBRINK

    NOTES:  c_tpread() calls were replaced by calls to read().  This brought
            the average tested time from 5.24 seconds per scene to 1.83 sec.
            Since the JPEG compression time was tested to an average of .973
            seconds.  The respective tape i/o time would be 4.26 seconds
            using c_tpread() and .860 seconds using read().  This five-fold
            improvement comes with the sacrifice of not being able to
            use remotely mounted tapes.

------------------------------------------------------------------------------*/
#include <stdio.h>
#include <errno.h>
#include "ditbrowse.h"

extern "C"
{
#include "flip_scene.h"
}


class Tape tape;	// Global so that class Log can close the tape drive
class Log  log;		// Global so it is available for Tape and List
class List List;	// Global so it is available for Tape
extern int errno;


int main (int argc, char *argv[])
{
char drive[CMLEN] = "";		// Tape drive
char header[HEADER_DATA_SIZE+1];// Browse image header record
char jpeg_file_name[CMLEN];	// Name of the jpeg output file
char jpegdir[CMLEN];		// Directory to place jpeg images in
char logdir[CMLEN];		// Directory for log and listing files
char sceneid[CMLEN];		// Scene ID of current image

long count = 0;			// Number of current image on tape
long end;			// Last image on tape to process
long first_record;		// Is this the first record of an image?
long i, j;			// Loop control
long len;			// Length of a scene ID
long nlines;			// Number of lines in image
long nsamps;			// Number of samples in the image
long offset;			// Offset into browse image buffer
long quality;			// JPEG quality value
long start;			// First image on tape to process
long status;			// Function return status

long cflag;			// Rotate the corner coordinates?
long eflag;			// Error processing current image?
long fflag;			// Does the image need to be flipped?
long pflag;			// Process the current image?



/*
 * Retrieve the command line arguments
 */
getparm(argc, argv, jpegdir, &quality, &start, &end, drive, logdir);


/*
 * Mount a tape
 */
tape.mount(drive);


/*
 * Extract the tape ID from first record of the ANSI header file.  The first
 * ANSI header file contains three records.  The first 80 byte record contains
 * the tape ID.  All other ANSI header files will only contain two 80 byte
 * records.
 */
tape.fetch(HT_RECSIZE, HT_RECSIZE);
tape.GetTapeid(logdir, jpegdir);


/*
 * Log the start of this tape
 */
log.Report(FALSE, "tpopen", "Tape mounted on %s", drive);
log.Report(FALSE, "jpegqual", "JPEG Quality factor is %d",quality);


/*
 * Process each browse image on tape.  This statement reads the record of the
 * ANSI header file that contains the scene ID of the image.  This record is
 * stored in buffer1.
 */
while ((status = tape.Tread(HT_RECSIZE,0)) == HT_RECSIZE)
    {
    /*
     * Initialize all processing flags for this image
     */
    cflag = FALSE;	// Rotate corner coordinates?
    eflag = FALSE;	// Error encountered?
    fflag = FALSE;	// Flip the image?
    pflag = FALSE;	// Process the image?


    /*
     * Print a starting message
     */
    if (++count == start)
        {
        log.Report(FALSE, "skip", "%d scene(s) skipped", start-1);
        timer_start();
        }


    /*
     * Should this image be processed?
     */
    if ((count >= start) && ((count <= end) || (end == NO_END)))
        pflag = TRUE;


    /*
     * Stop processing yet?
     */
    if ((end != NO_END) && (count > end))
        {
        count--;
        status = 0;
        break;
        }


    /*
     * Retrieve the scene ID from the ANSI header record.  Stop after
     * encountering the first white space character.
     */
    for (i=0, j=4; i<16; i++, j++)
	{
	if (tape.buffer1[j] == ' ')
	    break;
	sceneid[i] = tape.buffer1[j];
	}
    sceneid[i] = '\0';


    /*
     * If this image is supposed to be processed.  Update the listing file so
     * it shows an error was encountered while processing the image.  If an
     * error is encountered the listing file will be accurate.  If processing
     * is successful, the file will be updated below.
     */
    if (pflag)
	{
	List.update(sceneid, count, quality, FALSE, cflag, fflag, FALSE, TRUE,
	    FALSE, jpegdir);
	}


    /*
     * Skip the last record of the ANSI header file and expect to find an EOF
     */
    tape.fetch(HT_RECSIZE, HT_RECSIZE);
    tape.fetch(HT_RECSIZE, 0);

    /*
     * Read the browse image
     */
    first_record = TRUE;
    offset = 0;
    while ((status = tape.Tread(BR_RECSIZE, offset)) == BR_RECSIZE)
	{
        if (pflag)
            {
            offset += BR_RECSIZE;
            if (first_record)
		{
		/*
		 * Examine the browse header information stored in the first
		 * HEADER_DATA_SIZE bytes of the image.  This will ensure the
		 * buffer sizes are large enough to contain the entire browse
		 * image
		 */
                tape.proces_head(header, sceneid, &nlines, &nsamps);
		first_record = FALSE;
		}
            }
	}
    if (status != 0)
        log.Report(TRUE, "read", "Error reading from tape");


    /*
     * Skip past the ANSI trailer file
     */
    tape.fetch(HT_RECSIZE, HT_RECSIZE);
    tape.fetch(HT_RECSIZE, HT_RECSIZE);
    tape.fetch(HT_RECSIZE, 0);


    /*
     * Continue with the next image if this image shouldn't be processed
     */
    if (!pflag)
	continue;


    /*
     * Issue an error message if the scene ID is too short
     */
    len = strlen(sceneid);
    if (len != 16)
        log.Report(FALSE, "short", "%s is only %d characters", sceneid, len);


    /*
     * Determine if the image should be flipped.  We must flip all images that
     * are "mixed" and are listed in the static array flip_scenes.  The 12th
     * character of the scene ID indicates if the image is ascending,
     * descending, mixed mode, or an engineering shot.
     */
    if ((len == 16) && (sceneid[11] == 'M'))
        {
        int i;

        for (i = 0; i < NUM_FLIP; i++)
	    {
            if (strcmp(sceneid, flip_scenes[i]) == 0)
                {
                fflag = TRUE;
                break;
                }
	    }
        }


    /*
     * Corner coordinates should be rotated for all ascending images and those
     * images that need to be flipped.
     */
    if (fflag || ((len == 16) && (sceneid[11] == 'A')))
	cflag = TRUE;


    /*
     * Rotate corner coordinates and flip the image if necessary
     */
    if (cflag)
	rotate(header);
    if (fflag)
	tape.flip(nlines, nsamps);


    /*
     * Create the jpeg image
     */
    sprintf(jpeg_file_name,"%s/%s.%s.jpg", jpegdir, tape.tapeid, sceneid);
    write_JPEG_file (&tape.buffer1[HEADER_DATA_SIZE], jpeg_file_name,
	(int)quality, header, nlines, nsamps, 1);
    c_lasmsg(FALSE, "ditbrowse-informational", "Processed image %s (%d)",
	sceneid, count);


    /*
     * Update the entry for this scene in the listing file
     */
    List.update(sceneid, count, quality, TRUE, cflag, fflag, TRUE, FALSE,
	FALSE, jpegdir);
    }


/*
 * Stop the timer
 */
timer_stop(count-start+1);


/*
 * Check the tape status and close the tape drive.  When reading to EOT we are
 * encountering an EIO error.  Ignore this error if we have read to EOT.
 */
if (status != 0)
    {
    if (! ((status == -1) && (errno == EIO) && (List.expect() <= count)))
	log.Report(TRUE, "read", "Error reading from tape");
    }
tape.close();

log.Report(FALSE, "done", "DITBROWSE completed successfully");
return(0);
}
