#include <stdio.h>
#include <sys/signal.h>

#ifndef SEEK_SET
#define SEEK_SET 0
#endif
#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif
#ifndef SEEK_END
#define SEEK_END 2
#endif


/* Arrays representing AVHRR frame sync values for packed and unpacked data
---------------------------------------------------------------------------*/
static int unpacked_0swap[12] = {2,132,1,111,3,92,1,157,2,15,0,149};
static int unpacked_2swap[12] = {132,2,111,1,92,3,157,1,15,2,149,0};
static int unpacked_4swap[12] = {111,1,132,2,157,1,92,3,149,0,15,2};
static int packed_0swap[8]    = {40,69,191,92,25,216,60,149};
static int packed_2swap[8]    = {69,40,92,191,216,25,149,60};
static int packed_4swap[8]    = {92,191,69,40,149,60,216,25};

static long byte         = 0;	 /* Current byte being processed              */
static long found        = 0;	 /* Number of frame syncs found               */
static long last         = 0;	 /* Byte position of last frame sync found    */
static long maxbytes     = 0;	 /* Max number of bytes to search through     */
static long maxfind	 = 0;	 /* Max number of frame syncs to find         */
static double percentage = 0.75; /* % of frame sync that must be correct      */

#define FALSE         0	/* Value indicating false                             */
#define TRUE          1	/* Value indicating true                              */
#define NTYPES        6	/* Number of different types of frame syncs           */



/*******************************************************************************
FUNCITON: usage

PURPOSE:  Print the usage message and error out.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
int usage ()
{
printf("usage: framesync [-b] [-n] [-p] [-s] <file>\n");
printf("\n");
printf("       Framesync will search through a file looking for AVHRR frame\n");
printf("       syncs.  It will search for frame syncs in the following\n");
printf("       order: packed with no byte swapping; packed with two bytes\n");
printf("       swapped; packed with four bytes swapped; unpacked with no\n");
printf("       byte swapping; unpacked with two bytes swapped; and unpacked\n");
printf("       with four bytes swapped.\n");
printf("\n");
printf("          -b <xx>  Only search through the first xx bytes of the\n");
printf("                   image.  The default is the entire file\n");
printf("\n");
printf("          -n <xx>  Stop after locating xx frame syncs.\n");
printf("\n");
printf("          -p <xx>  Search for xx%% partially correct frame syncs.\n");
printf("\n");
printf("          -s       Operate in silent mode.  Print the format of the\n");
printf("                   image, the location of the first frame sync, and\n");
printf("                   the number of bytes separating the last two\n");
printf("                   frame syncs.  For example, p 4 513 20992\n");
printf("                   indicates that the image is packed, is byte\n");
printf("                   swapped in groups of four bytes, the first frame\n");
printf("                   sync is located in byte 513, and the subsequent\n");
printf("                   frame syncs are separated by 20,992 bytes.\n");
exit(1);
}



/*******************************************************************************
FUNCITON: complete

PURPOSE:  Compare a circular buffer of values against one of the frame sync
	  arrays.  If the buffer contains a valid frame sync then return TRUE,
	  otherwise return FALSE.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
int complete(buffer, framesync, start, nvalues)
    int *buffer;     /* IN - Circular buffer of values from the input file    */
    int *framesync;  /* IN - Array of frame sync values to search for         */
    int start;       /* IN - Index of the first byte value in the buffer      */
    int nvalues;     /* IN - Number of frame sync values                      */
{
register int i;
register int compares;

for (i = start, compares = 0; compares < nvalues; i++, compares++)
    {
    if (i >= nvalues) i = 0;
    if (buffer[i] != framesync[compares])
	return(FALSE);
    }
return(TRUE);
}



/*******************************************************************************
FUNCITON: partial

PURPOSE:  Compare a circular buffer of values against one of the frame sync
	  arrays.  If the buffer contains a partially correct frame sync then
	  return TRUE, otherwise return FALSE.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
int partial (buffer, framesync, start, nvalues)
    int *buffer;     /* IN - Circular buffer of values from the input file    */
    int *framesync;  /* IN - Array of frame sync values to search for         */
    int start;       /* IN - Index of the first byte value in the buffer      */
    int nvalues;     /* IN - Number of frame sync values                      */
{
register int   i;
register int   compares;
register float matches;

matches = 0.0;
for (i = start, compares = 0; compares < nvalues; i++, compares++)
    {
    if (i >= nvalues) i = 0;
    if (buffer[i] == framesync[compares]) matches += 1.0;
    }

matches /= (float)nvalues;
if (matches >= percentage)
    return(TRUE);

return(FALSE);
}



/*******************************************************************************
FUNCITON: framesync

PURPOSE:  Find all of the frame syncs in the specified data file.  When a frame
	  sync is found print its byte location.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
main(argc, argv)
   int  argc;
   char *argv[];
{
char format[10];       /* Format that we are looking for                      */

int  buffer[12];       /* Circular buffer containing values from input file   */
int  *framesync;       /* Pointer to an array of valid frame sync values      */
int  i;                /* Loop control                                        */
int  nvalues;          /* Number of frame sync values (8 or 12)               */

long first_sync;       /* Byte number where the first frame sync is located   */
long line_size;        /* Number of bytes between consecutive frame syncs     */

short lastline;        /* Flag indicating if last line size is reported       */
short msg;             /* Flag indicating if messages should be printed or not*/
short stop = FALSE;    /* Flag to stop searching                              */
short type;            /* Type of frame sync being searched for               */


/* Define a pointer to the comparison function.  Depending upon the user
   options, we need to run one of two functions.  The easiest way to do this
   is set a pointer to the appropriate function.
------------------------------------------------------------------------------*/
#ifdef sun
int (*compare)();
#else
int (*compare)(int *, int *, int, int); /* Comparison function to use         */
#endif

FILE *ifile;           /* Ptr to the input file                               */


/* Set up the default processing options
----------------------------------------*/
msg = TRUE;		/* Print interactive messages         */
compare = complete;


/* Check the arguments
----------------------*/
if (argc < 2)
    usage();
for (i = 1; i < (argc - 1); i++)
    {
    if (strcmp(argv[i], "-b") == 0)
	{
	i++;
	if (sscanf(argv[i], "%d", &maxbytes) != 1)
	    usage();
	}
    else if (strcmp(argv[i], "-n") == 0)
	{
	i++;
	if (sscanf(argv[i], "%d", &maxfind) != 1)
	    usage();
	if (maxfind < 1)
	    {
	    fprintf(stderr, "%s %s\n", "Number of frame syncs must be greater",
		"than zero.  Using default value.");
	    maxfind = 0;
	    }
	}
    else if (strcmp(argv[i], "-p") == 0)
	{
	i++;
	if (sscanf(argv[i], "%lf", &percentage) != 1)
	    usage();
	percentage /= 100.0;
	compare = partial;
	}
    else if (strcmp(argv[i], "-s") == 0)
	{
	msg = FALSE;
	}
    else
	usage();
    }


/* Was a file specified?
------------------------*/
if (i == argc)
    usage();


/* Open the input file for reading
----------------------------------*/
ifile = fopen(argv[i], "r");
if (ifile == NULL)
    {
    if (msg)
	printf("Error opening %s\n", argv[i]);
    else
	printf("p 0 0 0\n");
    exit(1);
    }


/* If the entire file is to be searched then set a flag to
   inform the user of the size of the last line in the file
-----------------------------------------------------------*/
if (!maxfind && !maxbytes)
    lastline = TRUE;
else
    lastline = FALSE;


/* Determine the size of the file if we are going search the entire thing
-------------------------------------------------------------------------*/
if (!maxbytes)
    {
    fseek(ifile, 0, SEEK_END);
    maxbytes = ftell(ifile);
    }


/* Test for frame syncs
-----------------------*/
for (type = 0; !stop && (type < NTYPES); type++)
    {
    /* Seek to the beginning of the input file and reset the number of bytes
       read from the file, the number of frame syncs found, and where the last
       one was found.
    --------------------------------------------------------------------------*/
    fseek(ifile, 0, SEEK_SET);
    byte       = 0;
    first_sync = 0;
    found      = 0;
    last       = 1; /* corrects the size of the first line */
    line_size  = 0;


    /* Set a pointer to the current frame sync type that we are searching for
    -------------------------------------------------------------------------*/
    switch (type)
	{
	case 0:
	    if (msg)
		printf("Packed, 0 bytes swapped\n");
	    else
		strcpy(format, "p 0");
	    framesync = packed_0swap;
	    nvalues   = 8;
	    break;

	case 1:
	    if (msg)
		printf("Packed, 2 bytes swapped\n");
	    else
		strcpy(format, "p 2");
	    framesync = packed_2swap;
	    nvalues   = 8;
	    break;

	case 2:
	    if (msg)
		printf("Packed, 4 bytes swapped\n");
	    else
		strcpy(format, "p 4");
	    framesync = packed_4swap;
	    nvalues   = 8;
	    break;

	case 3:
	    if (msg)
		printf("Unpacked, 0 bytes swapped\n");
	    else
		strcpy(format, "u 0");
	    framesync = unpacked_0swap;
	    nvalues   = 12;
	    break;

	case 4:
	    if (msg)
		printf("Unpacked, 2 bytes swapped\n");
	    else
		strcpy(format, "u 2");
	    framesync = unpacked_2swap;
	    nvalues   = 12;
	    break;

	case 5:
	    if (msg)
		printf("Unpacked, 4 bytes swapped\n");
	    else
		strcpy(format, "u 4");
	    framesync = unpacked_4swap;
	    nvalues   = 12;
	    break;
	
	default:
	    fprintf(stderr, "Programmer error!  Invalid case.\n");
	    exit(1);
	}


    /* Load the first (nvalues - 1) bytes into the buffer
    -----------------------------------------------------*/
    i = 0;
    while (i < nvalues - 1)
	if ((buffer[i++] = getc(ifile)) == EOF)
	    {
	    if (msg)
		printf("No frame syncs found.\n");
	    else
		printf("%s 0 0\n", format);
	    exit(0);
	    }


    /* Search through maxbytes bytes of the image file for a frame sync
    -------------------------------------------------------------------*/
    while (!stop && (byte < maxbytes) && ((buffer[i++] = getc(ifile)) != EOF))
	{
	byte++;
	if (i >= nvalues)
	    i = 0;
	if (compare(buffer, framesync, i, nvalues))
	    {
	    line_size = byte - last;
	    if (msg)
		printf("Found at byte %1d, line size = %1d\n",byte,line_size);
	    found++;
	    last = byte;
	    if (found == 1)
		first_sync = byte;
	    if (maxfind && (found >= maxfind))
		stop = TRUE;
	    }
	}


    /* If we found any frame syncs we should stop.  We have already searched
       the specified portion of the file and reported the appropriate number
       of framesyncs.
    ------------------------------------------------------------------------*/
    if (found)
	stop = TRUE;
    }


/* Close the input file
-----------------------*/
fclose(ifile);


/* Issue a message indicating no frame syncs were found
-------------------------------------------------------*/
if (found == 0)
    {
    if (msg)
	printf("No frame syncs found.\n");
    else
	printf("%s 0 0\n", format);
    exit(0);
    }


if (!msg)
    printf("%s %d %d\n", format, first_sync, line_size);
else if (lastline)
    printf("Last line size = %d\n", maxbytes - last + 1);

exit(0);
}
