/*******************************************************************************
FUNCTION:	qtrack

PURPOSE:	Extract the Qtrack information from an AVHRR image

*******************************************************************************/
#include <stdio.h>
#include <time.h>

#define FALSE         0	/* Value indicating false                             */
#define TRUE          1	/* Value indicating true                              */
#define NBYTES    22180 /* Number of bytes in a line of data                  */
#define NWORDS    11090 /* Number of integers in a line of data               */
#define YEAR          1 /* Flag for a four digit year                         */
#define YR            2 /* Flag for a two digit year                          */

#define QTRACK           -32768  /* 0x8000 Qtrack flag for their data         */
#define QTRACK_EPHEM     -32768  /* 0x8000 Qtrack flag for ephemeris data     */
#define QTRACK_ACQ_START -32512  /* 0x8100 Qtrack flag for acquisition start  */
#define QTRACK_RESYNC    -32256  /* 0x8200 Qtrack flag for acquisition resync */
#define QTRACK_ACQ_END   -32000  /* 0x8300 Qtrack flag for acquisition stop   */


int usage ()
{
printf("usage: qtrack [-year] [-yr] <file>\n");
printf("\n");
printf("       qtrack will examine an AVHRR image acquired with the QTRACK\n");
printf("       software.  At the time of development these images were only\n");
printf("       produced in Yakutsk, CIS (former Soviet Union).  This\n");
printf("       software places four types of special lines in the image.\n");
printf("       This function scans for those lines and prints them to\n");
printf("       stdout.\n");
printf("\n");
printf("       The Qtrack lines are:\n");
printf("\n");
printf("           Satellite ephemeris data\n");
printf("           Start of data time stamp\n");
printf("           Resync time stamp\n");
printf("           File close time stamp\n");
printf("\n");
printf("       Options:\n");
printf("           -year  Prints the 4 digit year of acquisition\n");
printf("           -yr    Prints the 2 digit year of acquisition\n");
printf("\n");
exit(1);
}


void swap (buffer, bytes)
    
    char *buffer;	/* IN - Data buffer to by byte swapped                */
    int bytes;		/* IN - Number of bytes in the buffer                 */
{
int i;		/* Loop control         */
int j;		/* Loop control         */
char tmp;	/* Temporary swap space */

for (i = 0, j = 1; i < bytes; i += 2, j += 2)
    {
    tmp = buffer[i];
    buffer[i] = buffer[j];
    buffer[j] = tmp;
    }
}


void printdate (buffer, yearflg)

    short buffer[];	/* IN - Qtrack buffer containing a C time code        */
    short yearflg;	/* IN - Flag to just print the year or entire date    */
{
unsigned short left;	/* Leftmost 16 bits of the 32 bit time code           */
unsigned short right;	/* Rightmost 16 bits of the 32 bit time code          */
unsigned long acqtime;	/* 32 bit time code                                   */

struct tm *acqdate;	/* Ptr to a C time structure                          */


/*
  Extract the C time code from a short buffer and convert to a long
*/
left  = (unsigned short)buffer[3];
right = (unsigned short)buffer[4];
acqtime = left;
acqtime = acqtime << 16;
acqtime += right;


/*
  Generate a C time structure from the stored code
*/
acqdate = gmtime(&acqtime);


/*
  Print the year of acquisition or the entire date and time
*/
if (yearflg == YEAR)
    printf("%d\n", acqdate->tm_year + 1900);
else if (yearflg == YR)
    printf("%d\n", acqdate->tm_year);
else
    printf("%s", asctime(acqdate));
}



main(argc, argv)
   int  argc;
   char *argv[];
{
short buffer[NWORDS];	/* Buffer to read one line of YKT data                */
short yearflg = FALSE;	/* Flag to just print the year of acquisition         */
int i;			/* Loop control                                       */
int line = 0;		/* Current line number                                */
int nwords;		/* Number of int words read from the file             */
FILE *fp;		/* File pointer for the AVHRR image                   */


/*
  Check the arguments
*/
if (argc < 2)
    usage();
for (i = 1; i < (argc - 1); i++)
    {
    if (strcmp(argv[i], "-year") == 0)
	yearflg = YEAR;
    else if (strcmp(argv[i], "-yr") == 0)
	yearflg = YR;
    else
	usage();
    }


/*
  Open the input file for reading
*/
fp = fopen(argv[i], "r");
if (fp == NULL)
    {
    printf("Error opening %s\n", argv[i]);
    exit(1);
    }


/*
  Scan the entire image looking for the Qtrack lines
*/
do {

    /*
      Read one line of data
    */
    nwords = fread(buffer, sizeof(short), NWORDS, fp);
    if (nwords == 0) break;
    if (nwords != NWORDS)
	{
	printf("Error reading file.  Read %d words when expecting %d.\n",
	    nwords, NWORDS);
	exit(1);
	}
    line++;


    /*
      Byte swap the buffer to Unix byte order
    */
    swap((char *)buffer, NBYTES);


    /*
      If this line starts with the unique Qtrack number print the information.
      If the year flag was specified, it is assumed this function is being
      called from a script and it should only report the year at the start of
      acquisition.
    */
    if (buffer[0] == QTRACK)
	{
	if (!yearflg && (buffer[1] == QTRACK_EPHEM))
	    printf("line %4d: Satellite ephemeris data\n", line);
	else if (buffer[1] == QTRACK_ACQ_START)
	    {
	    if (!yearflg)
		printf("line %4d: Start of data ", line);
	    printdate(buffer, yearflg);


	    /*
	      If the year flag is set, we can quit
	    */
	    if (yearflg) exit(0);
	    }
	else if (!yearflg && (buffer[1] == QTRACK_RESYNC))
	    {
	    printf("line %4d: Resync at     ", line);
	    printdate(buffer, yearflg);
	    }
	else if (!yearflg && (buffer[1] == QTRACK_ACQ_END))
	    {
	    printf("line %4d: File close at ", line);
	    printdate(buffer, yearflg);
	    }
	}
    } while (nwords == NWORDS);


/*
  Close the input file
*/
fclose(fp);


/*
  If the year flag is set, we never found the correct info
*/
if (yearflg)
    exit(1);

exit(0);
}
