
/******************************************************************************
NAME:     host2las.c 
PURPOSE:  a unix utility to convert filename(s) from host to LAS format, 
          developed for LAS>LASSHELL 

PROGRAM HISTORY
-------------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  ----------------------------------------------------
Apr 1995  David Zokaites   Original development for $LASDITTY
May 1995  David Zokaites   Checked pclose() return code 
Oct 1995  David Zokaites   Changed indentation, released to $LASTOOLS 
******************************************************************************/




#include <ctype.h>  /* for call to isspace() */ 
#include <stdio.h>  /* for calls to fprintf(), popen(), ... */ 
#include <stdlib.h> /* for calls to exit(), popen() */
#include <string.h> /* for calls to various string functions */ 
#include "las.h"    /* las header file */ 





/* function prototypes, ifdef allows for K&R compilers */ 
#ifdef __DGUX__
   int main (int argc, char *argv[]); 
   void print_usage (void);
   void parse_tilde (char *inoutname);
#else 
   int main (); 
   void print_usage ();
   void parse_tilde ();
#endif






/* PROGRAM MAIN */ 
FUNCTION int main (argc, argv)
   int argc;      /* number of input arguments */
   char *argv[];  /* array of input arguments */ 
{
/* declare variables */ 
   char curfile[CMLEN],  /* current filename being processed */ 
        outname[CMLEN];  /* output:  filename in las format */ 
   int filenum,          /* for loop index, filenames */ 
       oneline;          /* flag, does user want one file printed per line */ 

/* check number of input arguments */ 
   if (argc < 2)
      print_usage (); 

/* get oneline parm, set curfile */ 
   if (strcmp (argv[1], "-1"))
      {
      oneline = FALSE;  
      filenum = 1; 
      }
   else
      {
      oneline = TRUE;
      filenum = 2; 
      if (argc < 3)
	 print_usage (); 
      }

/* convert filenames */ 
   for (;  filenum<argc;  filenum++)
      {
      /* set curfile */ 
	 strcpy (curfile, argv[filenum]); 

      /* check name for '~' character */
	 parse_tilde (curfile);

      /* convert name from host to las */ 
         strcpy(outname, curfile);

      /* print filename with spaces or newlines between filenames */ 
	 fprintf (stdout, "%s", outname); 
	 if (oneline)
	    fprintf (stdout, "\n"); 
	 else if (argc > 1 && filenum != argc -1)
	    fprintf (stdout, " "); 
      }

   if (!oneline) 
      fprintf (stdout, "\n"); 

/* successful completion */
   return (0); 
} 









/******************************************************************************
NAME:     print_usage() 
PURPOSE:  print out a usage error message 
******************************************************************************/

FUNCTION void print_usage ()
{
   fprintf (stderr, "Host2las converts input file and directory names "); 
   fprintf (stderr, "from host to LAS format.\n");
   fprintf (stderr, "Usage:  host2las [-1] filename(s)\n"); 
   fprintf (stderr, "The -1 flag results in printing one filename per line.\n");
   fprintf (stderr, "The ~ character (as in ~diane/k) will be translated ");
   fprintf (stderr, "by host2las via the cshell.\n");
   exit (1);
}







/******************************************************************************
NAME:     parse_tilde()
PURPOSE:  parse an input path to expand the '~' character.  Any wildcards in
	  the path will not be expanded.  The '~' character is parsed via
	  "popen (csh -c echo ...)".  This requires two sub-shells.  Reading
	  /etc/passwd could be faster but it fails on systems that run yppasswd
	  and would also require getting an environment variable ($user)
******************************************************************************/

FUNCTION void parse_tilde (inoutname)
   char *inoutname;   /* input:   filename with a '~' optionally imbedded 
                        output:  expanded filename, with no '~' */ 
{
/* declare variables */
   char *copied,           /* a character that was copied */ 
	*newline,          /* points to a trailing newline char */ 
	 syscmd[CMLEN],    /* an operating system command */ 
	 toreturn[CMLEN],  /* file name to return */ 
        *tocopy,           /* points to a character to copy */ 
	 rootdir[CMLEN];   /* user's root directory with a '~' character */ 
   FILE *comout;           /* used to open a pipe to a unix command */ 

/* if ~ is not the first character, no parsing is necessary */ 
   if (*inoutname != '~')
      return;

/* get rootdir (user's root dir as in ~diane) */ 
   for (copied=rootdir, tocopy=inoutname;  
   *tocopy !='/' && *tocopy !='\0';  copied++, tocopy++)
      *copied = *tocopy; 
   *copied = '\0';  /* terminate string */ 

/* open pipe to translate rootdir via a cshell */ 
   sprintf (syscmd, "csh -c \" echo %s \"", rootdir);
   comout = popen (syscmd, "r");
   if (comout == NULL)
      {
      fprintf (stderr, "host2las:  error opening pipe via popen() \n");
      exit (1);
      }

/* read pipe */ 
   if (fgets (toreturn, CMLEN, comout) == NULL)
      {
      fprintf (stderr, "host2las:  error reading command output\n");
      pclose (comout);
      exit (1);
      }
   if (pclose (comout))
      exit (1); 

/* eliminate trailing newline from pipe output */ 
   newline = toreturn + strlen(toreturn) -1;
   if (isspace(*newline))
      *newline = '\0';  

/* append rest of path to user's root directory */ 
   for (copied=toreturn + strlen(toreturn);  
   *tocopy !='\0';  copied++, tocopy++)
      *copied = *tocopy; 
   *copied = '\0';  /* terminate string */ 

/* set return value */ 
   strcpy (inoutname, toreturn); 
}
