#!/bin/csh
#
# FUNCTION: newcr.csh
#
# PURPOSE:  Determine if a message from the change request system is a new
#           request or not.  If it is a new request then reformat the message
#           using fmtlug.csh or throw the message away.  This should be called
#           by your mail filter program.
#
#	    NOTE: When running as a mail filter, you do not have $HOME or ~
#             defined.  You cannot rely upon your normal path as well.
#
# $Author: dlabahn $
# $Date: 2002/09/26 21:30:18 $
# $Source: /edclxs64/www/adaps/software/src/bin/lug/newcr.csh,v $
# $Revision: 1.2 $
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# J.Fenno          09/95  Development
# J.Willems        07/97  Changed to be executed for LUG notes once a week.
# D Hollaren       03/00  Got rid of hard coded dependancies as much as possible
#

#---------------------
# Check for parameters
#---------------------
if ($#argv != 2) then
    echo 'usage: newcr.csh <new_crs> <file>'
    echo ''
    echo '       If the mail message supplied through stand input is a new'
    echo '       change request, use fmtlug.csh to reformat the mail message'
    echo '       and save the reformatted output to <file>.  The <new_crs>'
    echo '       file contains the list of new change requests numbers.  The'
    echo '       numbers are used to extract information, for the LUG notes,'
    echo '       from the gnats-db.'
    exit 1
endif
set new_crs = $1
set lugmsg = $2

#----------------------
# Setup shell variables
#----------------------
set hline = "----------------------------------------"
set curdir = `pwd`
set gnats_dir = /sgs20/csb/gnats/gnats-db
set adaps   = $gnats_dir/adaps
set las     = $gnats_dir/las
set errfile = $curdir/newcr.err
set errlog  = $curdir/fmtlug$$.log

#-------------------------------------------
# Make sure the file that has new CRs exists
#-------------------------------------------
if (! -e $new_crs) then
    echo "$new_crs does not exist."
    exit 1
endif

#----------------------------
# For each new number in file
#----------------------------
foreach number (`/bin/cat $new_crs`)
  if ( -r $adaps/$number ) then
    set cr_file = $adaps/$number
  else if ( -r $las/$number ) then
    set cr_file = $las/$number
  else
    echo "$number not found in $adaps or $las" >> $errfile
  endif

  #--------------------------------------
  # Ensure a description field is present
  #--------------------------------------
  grep ">Description:" $cr_file > /dev/null
  if ($status) then
    echo "" >> $errfile
    echo "" >> $errfile
    echo "$hline$hline" >> $errfile
    echo "NEWCR: Description field not present" >> $errfile
    echo "$hline$hline" >> $errfile
    cat $cr_file >> $errfile
    /bin/rm -f $cr_file
    exit 0
  endif

  #-----------------------------------------------------------------
  # All of the necessary fields were located so reformat the message
  #-----------------------------------------------------------------
  cat $cr_file | fmtlug.csh $lugmsg >& $errlog
  if ($status) then
    echo "" >> $errfile
    echo "" >> $errfile
    echo "$hline$hline" >> $errfile
    cat $errlog >> $errfile
    echo "$hline$hline" >> $errfile
    cat $cr_file >> $errfile
  endif

end

#--------------------------
# Remove the temporary file
#--------------------------
/bin/rm -f $errlog
