%{
/*************************************************************
   Copyright (c) 1993-1995 by Paul Long  All rights reserved.
**************************************************************/

/*************************************************************
   gram.y - This source file contains the YACC grammar and
            actions for Metre's Standard C parser. It also
            contains main(), error-handling functions,
            trigger-firing functions, functions that
            maintain the typedef symbol table, functions
            that administer the execution of Metre,
            functions that can be called from the rules()
            function, and a replacement for YACC's
            yyerror(). As you can see, most of the grammar
            rules have no actions. This is because the
            grammar is largely used to just skip over the
            parts of C that Metre doesn't care about.
**************************************************************/

#include <stdio.h>
#include <limits.h>
#include <stdarg.h>
#include <stdlib.h>
#include "metreint.h"

/*
   NOTE: Someone said that that the default stack size for MKS yacc and
   AT&T (SCO) yacc of 150 was not sufficient for the code he was running
   through Metre, so he redefined the stack size to 1000. If you
   encounter the same situation, use one of the following directives to
   increase the stack size for the the version of yacc that you're
   using.
*/
#if 0
#define YYSSIZE      1000  /* MKS yacc stack size */
#define YYMAXDEPTH   1000  /* AT&T yacc stack size */
#endif

/* String containing all of the white-space characters. */
#define WHITE_SPACE_CHARS  " \t\r\n\v\f"

/* VMS has different ideas about what constitutes a successful return value. */
#ifdef VAX
#define RETURN_FROM_MAIN_VALUE 1
#else
#define RETURN_FROM_MAIN_VALUE 0
#endif


/* External-function prototypes. */

/* Defines behavior for this Metre tool. */
extern void rules(void);
/* YACC-generated parser function. */
extern yyparse();
/* lex-generated scanner function. */
extern yylex();
/* Error function that yacc-generated parser calls. */
extern void yyerror(char *);


/* External variables. */

/*
   Metre maintains statistics in the int_* structures, e.g., int_prj.
   The structures with the unadorned names, e.g., prj, are used to
   communicate with the rules. They are zero-filled except when a
   trigger is being fired. At that time, the contents of the
   corresponding int_* structure is copied to it, the rules() function
   is called, and then the structure is zero-filed again.
*/
PRJ prj, int_prj;    /* Project. */
MOD mod, int_mod;    /* Module. */
FCN fcn;             /* Function. */
static FCN int_fcn;
STM stm;             /* Statement. */
static STM int_stm;
LIN lin, int_lin;    /* Line. */
GRM grm, int_grm;    /* Grammatical. */
LEX lex, int_lex;    /* Lexical. */

/*
   Which command-line argument (argv) at which to start looking for the
   next file name to process and original file name, respectively.
*/
unsigned next_cmd_line_file;
unsigned next_cmd_line_file_orig_n;

/* Number of decision points, conditions, and functions in module (file). */
unsigned mod_decisions;
unsigned mod_conditions;
unsigned mod_functions;

/*
   Indicates whether we are looking for a tag. Parser writes it, lexer
   reads it. Since tags are in a separate name space from other
   identifiers, this helps the lexer determine whether a lexeme is an
   identifier or a type name.
*/
BOOLEAN looking_for_tag;

/*
   Indicates whether we are within a function definition. Used to
   determine whether to use the last identifier as the function name.
*/
BOOLEAN is_within_function;

/*
   Name of current input file and name of original file, if specified on
   command line with the substitute-file-name option. If substitute not
   provided, input_file_orig_name points to the same name as input_file.
*/
char *input_file;
char *input_file_orig_name;

/* Where all output is written. */
FILE *out_fp;

static char *command_name = DEFAULT_COMMAND_NAME;

/* Global versions of main's argc and argv. */
int cmd_line_argc;
char **cmd_line_argv;

/*
   Block within which typedef type names are held in the typedef
   symbol-table.
*/
typedef struct typedef_sym_blk {
   struct typedef_sym_blk *next;          /* Next block. */
   unsigned total;                        /* Total in this block. */
   char *id[TYPEDEF_SYMBOLS_PER_BLOCK]; /* Array of pointers to type names. */
} TYPEDEF_SYM_BLK;

/*
   Head of the typedef symbol-table. NOTE: This is the only symbol table
   in Metre, and it is used just to hold typedef type names, not all
   symbols.
*/
TYPEDEF_SYM_BLK *typedef_sym_tbl_head = NULL;

/* Command-line argument. Used to hold arguments taken from script file. */
typedef struct arg {
   struct arg *next;
   char *string;
} ARG;


/* Static variables. */

/* Control depth--current number of nested control structures. */
static unsigned depth;

/* Used to determine how many statements are on a line. */
static unsigned previous_statements_line_number;

/* Number of decision points and conditions in function. */
static unsigned fcn_decisions;
static unsigned fcn_conditions;

/*
   Metre maintains the number of statements appearing on a line. The
   is_if and is_else BOOLEANs are used principally to relax the criteria
   for what constitutes a statement. For example, "else if," is not
   considered two statements because this is a common idiom.
*/
static BOOLEAN is_else, is_if;

/*
   is_typedef indicates whether we are parsing a typedef. Since typedefs
   cannot be nested, it's okay that this is just a BOOLEAN. This is used
   to find the type name of a typedef so that it can be added to the
   typedef symbol table. If the type name is subsequently encountered by
   the lexer, the lexer returns a token for a type name rather than for
   an identifier. This is the ugly part of C that cannot practically be
   expressed by a YACC grammar specification.
*/
static BOOLEAN is_typedef;

/*
   nested_decl_specs is used to distinguish the name of the thing being
   declared, such as the type name in a typedef, from other identifiers
   in the declaration.
*/
static unsigned nested_decl_specs;

/*
   Whether the last statement was a case label. Used to consider
   adjacent case labels as one decision point. The transition from
   is_case_label==TRUE to is_case_label==FALSE is a decision point,
   instead of each case label being a decision point. Adjacent case
   labels are considered one decision point because this is analogous to
   an if statement with or logical operators, ||. For example,

      switch (expr) {
      case 5:
      case 10:
         a;
      }

   is, in this regard, analogous to

      if (expr == 5 || expr == 10)
         a;

   I did not want to bias the metric against the switch statement.
*/
static BOOLEAN is_case_label;

static IDENTIFIER function_name;
static IDENTIFIER last_ident;
static IDENTIFIER last_decl;

/*
   Pointer to current operator, keyword, or identifier as passed to
   rules if such a token is encountered.
*/
static char *current_operator;
static char *current_keyword;
static char *current_identifier;
static char *current_type_name;

/* Static function declarations. */

static void begin_project(char *);
static void fire_fcn(void);
static void fire_stm(void);
static void print_exception(int, char *, char *, va_list);
static void typedef_symbol_table_add(char *);
static void case_label(void);
static void not_case_label(void);
static void check_starting_options(void);
static void print_banner(void);
static void print_help(void);
static void stat_func_begin(void);
static void stat_func_end(void);
static void check_multiple_statements(void);
static void int_warn(int, char *, ...);
static void fire_operator(void);
static void fire_keyword(void);
static void fire_identifier(void);
static void fire_declarator(void);
static void fire_type_name(void);
static void found_constant(void);
static void found_string(void);
static void found_expression_statement(BOOLEAN);
static void function_call(void);
static void after_compound_declarations(void);
static void before_compound_declarations(void);
static void after_initializer(void);
static void before_initializer(void);
static void get_options_from_file(char *);
static void build_new_argv(ARG *);
%}

%token TK_IDENTIFIER TK_CONSTANT TK_STRING_LITERAL TK_SIZEOF
%token TK_PTR_OP TK_INC_OP TK_DEC_OP TK_LEFT_OP TK_RIGHT_OP
%token TK_LE_OP TK_GE_OP TK_EQ_OP TK_NE_OP
%token TK_AND_OP TK_OR_OP TK_MUL_ASSIGN TK_DIV_ASSIGN TK_MOD_ASSIGN
%token TK_ADD_ASSIGN TK_SUB_ASSIGN TK_LEFT_ASSIGN TK_RIGHT_ASSIGN TK_AND_ASSIGN
%token TK_XOR_ASSIGN TK_OR_ASSIGN TK_TYPE_NAME

%token TK_TYPEDEF TK_EXTERN TK_STATIC TK_AUTO TK_REGISTER
%token TK_CHAR TK_SHORT TK_INT TK_LONG TK_SIGNED TK_UNSIGNED
%token TK_FLOAT TK_DOUBLE TK_CONST TK_VOLATILE TK_VOID
%token TK_STRUCT TK_UNION TK_ENUM TK_ELIPSIS TK_RANGE

%token TK_CASE TK_DEFAULT TK_IF TK_ELSE TK_SWITCH TK_WHILE TK_DO TK_FOR
%token TK_GOTO TK_CONTINUE TK_BREAK TK_RETURN

%start translation_unit

/* Used to disambiguate if from if/else. */
%nonassoc THEN
%nonassoc TK_ELSE

%%

tk_lparen
   : '('
      { fire_operator(); }
   ;

tk_rparen
   : ')'
      { fire_operator(); }
   ;

tk_lsquare_brace
   : '['
      { fire_operator(); }
   ;

tk_rsquare_brace
   : ']'
      { fire_operator(); }
   ;

tk_dot
   : '.'
      { fire_operator(); }
   ;

tk_ptr_to
   : TK_PTR_OP
      { fire_operator(); }
   ;

tk_inc
   : TK_INC_OP
      { fire_operator(); }
   ;

tk_dec
   : TK_DEC_OP
      { fire_operator(); }
   ;

tk_comma
   : ','
      { fire_operator(); }
   ;


tk_sizeof
   : TK_SIZEOF
      { fire_keyword(); }
   ;

tk_ampersand
   : '&'
      { fire_operator(); }
   ;

tk_asterisk
   : '*'
      { fire_operator(); }
   ;

tk_plus
   : '+'
      { fire_operator(); }
   ;

tk_minus
   : '-'
      { fire_operator(); }
   ;

tk_tilde
   : '~'
      { fire_operator(); }
   ;

tk_exclamation
   : '!'
      { fire_operator(); }
   ;

tk_slash
   : '/'
      { fire_operator(); }
   ;

tk_percent
   : '%'
      { fire_operator(); }
   ;

tk_left
   : TK_LEFT_OP
      { fire_operator(); }
   ;

tk_right
   : TK_RIGHT_OP
      { fire_operator(); }
   ;

tk_lt
   : '<'
      { fire_operator(); }
   ;

tk_gt
   : '>'
      { fire_operator(); }
   ;

tk_le
   : TK_LE_OP
      { fire_operator(); }
   ;

tk_ge
   : TK_GE_OP
      { fire_operator(); }
   ;

tk_eq
   : TK_EQ_OP
      { fire_operator(); }
   ;

tk_ne
   : TK_NE_OP
      { fire_operator(); }
   ;

tk_circumflex
   : '^'
      { fire_operator(); }
   ;

tk_bar
   : '|'
      { fire_operator(); }
   ;

tk_land
   : TK_AND_OP
      { fire_operator(); }
   ;

tk_lor
   : TK_OR_OP
      { fire_operator(); }
   ;

tk_question
   : '?'
      { fire_operator(); }
   ;

tk_colon
   : ':'
      { fire_operator(); }
   ;

tk_assign
   : '='
      { fire_operator(); }
   ;

tk_mul_assign
   : TK_MUL_ASSIGN
      { fire_operator(); }
   ;

tk_div_assign
   : TK_DIV_ASSIGN
      { fire_operator(); }
   ;

tk_mod_assign
   : TK_MOD_ASSIGN
      { fire_operator(); }
   ;

tk_add_assign
   : TK_ADD_ASSIGN
      { fire_operator(); }
   ;

tk_sub_assign
   : TK_SUB_ASSIGN
      { fire_operator(); }
   ;

tk_left_assign
   : TK_LEFT_ASSIGN
      { fire_operator(); }
   ;

tk_right_assign
   : TK_RIGHT_ASSIGN
      { fire_operator(); }
   ;

tk_and_assign
   : TK_AND_ASSIGN
      { fire_operator(); }
   ;

tk_xor_assign
   : TK_XOR_ASSIGN
      { fire_operator(); }
   ;

tk_or_assign
   : TK_OR_ASSIGN
      { fire_operator(); }
   ;

tk_semicolon
   : ';'
      { fire_operator(); }
   ;

tk_typedef
   : TK_TYPEDEF
      { fire_keyword(); }
   ;

tk_extern
   : TK_EXTERN
      { fire_keyword(); }
   ;

tk_static
   : TK_STATIC
      { fire_keyword(); }
   ;

tk_auto
   : TK_AUTO
      { fire_keyword(); }
   ;

tk_register
   : TK_REGISTER
      { fire_keyword(); }
   ;

tk_char
   : TK_CHAR
      { fire_keyword(); }
   ;

tk_short
   : TK_SHORT
      { fire_keyword(); }
   ;

tk_int
   : TK_INT
      { fire_keyword(); }
   ;

tk_long
   : TK_LONG
      { fire_keyword(); }
   ;

tk_signed
   : TK_SIGNED
      { fire_keyword(); }
   ;

tk_unsigned
   : TK_UNSIGNED
      { fire_keyword(); }
   ;

tk_float
   : TK_FLOAT
      { fire_keyword(); }
   ;

tk_double
   : TK_DOUBLE
      { fire_keyword(); }
   ;

tk_const
   : TK_CONST
      { fire_keyword(); }
   ;

tk_volatile
   : TK_VOLATILE
      { fire_keyword(); }
   ;

tk_void
   : TK_VOID
      { fire_keyword(); }
   ;

tk_struct
   : TK_STRUCT
      { fire_keyword(); }
   ;

tk_union
   : TK_UNION
      { fire_keyword(); }
   ;

tk_enum
   : TK_ENUM
      { fire_keyword(); }
   ;

tk_type_name
   : TK_TYPE_NAME
      { fire_type_name(); }
   ;

tk_lcurly_brace
   : '{'
      { fire_operator(); }
   ;

tk_rcurly_brace
   : '}'
      { fire_operator(); }
   ;

tk_elipsis
   : TK_ELIPSIS
      { fire_operator(); }
   ;

tk_case
   : TK_CASE
      { fire_keyword(); }
   ;

tk_default
   : TK_DEFAULT
      { fire_keyword(); }
   ;

tk_if
   : TK_IF
      { fire_keyword(); }
   ;

tk_switch
   : TK_SWITCH
      { fire_keyword(); }
   ;

tk_else
   : TK_ELSE
      { fire_keyword(); }
   ;

tk_while
   : TK_WHILE
      { fire_keyword(); }
   ;

tk_do
   : TK_DO
      { fire_keyword(); }
   ;

tk_for
   : TK_FOR
      { fire_keyword(); }
   ;

tk_goto
   : TK_GOTO
      { fire_keyword(); }
   ;

tk_continue
   : TK_CONTINUE
      { fire_keyword(); }
   ;

tk_break
   : TK_BREAK
      { fire_keyword(); }
   ;

tk_return
   : TK_RETURN
      { fire_keyword(); }
   ;

tk_string
   : TK_STRING_LITERAL
      { found_string(); }
   ;

tk_constant
   : TK_CONSTANT
      { found_constant(); }
   ;

identifier
   : TK_IDENTIFIER
      {
         /* Fire the identifier trigger. */
         fire_identifier();
         /*
            Save this identifier in case token buffer (yytext[]) gets
            overwritten by the time the identifier is needed.
         */
         strcpy(last_ident, token());
      }
   ;

primary_expr
   : identifier
   | tk_constant
   | string_list
   | tk_lparen expr tk_rparen
   ;

/* (In case a preprocessor wasn't run to join adjacent strings.) */
string_list
   : tk_string
   | string_list tk_string
   ;

postfix_expr
   : primary_expr
   | postfix_expr tk_lsquare_brace expr tk_rsquare_brace
   | postfix_expr tk_lparen
      { function_call(); }
   tk_rparen
   | postfix_expr tk_lparen
      { function_call(); }
   argument_expr_list tk_rparen
   | postfix_expr tk_dot identifier
   | postfix_expr tk_ptr_to identifier
   | postfix_expr tk_inc
   | postfix_expr tk_dec
   ;

argument_expr_list
   : assignment_expr
   | argument_expr_list tk_comma assignment_expr
   ;

unary_expr
   : postfix_expr
   | tk_inc unary_expr
   | tk_dec unary_expr
   | unary_operator cast_expr
   | tk_sizeof unary_expr
   | tk_sizeof tk_lparen type_name tk_rparen
   ;

unary_operator
   : tk_ampersand
   | tk_asterisk
   | tk_plus
   | tk_minus
   | tk_tilde
   | tk_exclamation
   ;

cast_expr
   : unary_expr
   | tk_lparen type_name tk_rparen cast_expr
   ;

multiplicative_expr
   : cast_expr
   | multiplicative_expr tk_asterisk cast_expr
   | multiplicative_expr tk_slash cast_expr
   | multiplicative_expr tk_percent cast_expr
   ;

additive_expr
   : multiplicative_expr
   | additive_expr tk_plus multiplicative_expr
   | additive_expr tk_minus multiplicative_expr
   ;

shift_expr
   : additive_expr
   | shift_expr tk_left additive_expr
   | shift_expr tk_right additive_expr
   ;

relational_expr
   : shift_expr
   | relational_expr tk_lt shift_expr
   | relational_expr tk_gt shift_expr
   | relational_expr tk_le shift_expr
   | relational_expr tk_ge shift_expr
   ;

equality_expr
   : relational_expr
   | equality_expr tk_eq relational_expr
   | equality_expr tk_ne relational_expr
   ;

and_expr
   : equality_expr
   | and_expr tk_ampersand equality_expr
   ;

exclusive_or_expr
   : and_expr
   | exclusive_or_expr tk_circumflex and_expr
   ;

inclusive_or_expr
   : exclusive_or_expr
   | inclusive_or_expr tk_bar exclusive_or_expr
   ;

logical_and_expr
   : inclusive_or_expr
   | logical_and_expr tk_land
      /* Logical-and operator counts as a condition. */
      { ++fcn_conditions; }
      inclusive_or_expr
   ;

logical_or_expr
   : logical_and_expr
   | logical_or_expr tk_lor
      /* Logical-or operator counts as a condition. */
      { ++fcn_conditions; }
      logical_and_expr
   ;

conditional_expr
   : logical_or_expr
   | logical_or_expr tk_question expr tk_colon conditional_expr
      /*
         I consider the conditional operator to be a decision point and a
         condition.
      */
      { ++fcn_decisions; ++fcn_conditions; }
   ;

assignment_expr
   : conditional_expr
   | unary_expr assignment_operator assignment_expr
   ;

assignment_operator
   : tk_assign
   | tk_mul_assign
   | tk_div_assign
   | tk_mod_assign
   | tk_add_assign
   | tk_sub_assign
   | tk_left_assign
   | tk_right_assign
   | tk_and_assign
   | tk_xor_assign
   | tk_or_assign
   ;

expr
   : assignment_expr
   | expr tk_comma assignment_expr
   ;

constant_expr
   : conditional_expr
   ;

declaration
   /*
      Regardless of whether we were really parsing a typedef, indicate
      that we are no longer by setting is_typedef to FALSE.
   */
   : declaration_specifiers tk_semicolon
      {
         is_typedef = FALSE;

         ++int_fcn.decl;         /* Declaration statement. */
         ++int_mod.decl;

         /*
            Indicate at the end of a declaration statement then fire the
            end-of-statement trigger.
         */
         int_stm.is_decl = int_stm.end = TRUE;
         fire_stm();
         ZERO(int_stm);
      }
   | declaration_specifiers init_declarator_list tk_semicolon
      {
         is_typedef = FALSE;

         ++int_fcn.decl;         /* Declaration statement. */
         ++int_mod.decl;

         /*
            Indicate at the end of a declaration statement then fire the
            end-of-statement trigger.
         */
         int_stm.is_decl = int_stm.end = TRUE;
         fire_stm();
         ZERO(int_stm);
      }
   ;

declaration_specifiers
   /*
      nested_decl_specs is incremented and decremented here so that the
      name of the root-level item can be distinguished from other
      identifiers in the declaration.
   */
   : storage_class_specifier
   | storage_class_specifier
      { ++nested_decl_specs; }
      declaration_specifiers
      { --nested_decl_specs; }
   | type_specifier
   | type_specifier
      { ++nested_decl_specs; }
      declaration_specifiers
      { --nested_decl_specs; }
   ;

init_declarator_list
   : init_declarator
   | init_declarator_list tk_comma init_declarator
   ;

init_declarator
   : declarator
   | declarator
      { before_initializer(); }
   tk_assign initializer
      { after_initializer(); }
   ;

storage_class_specifier
   : tk_typedef
      /*
         Indicate that we are within a typedef. This is used to find the
         type name so that it can be added to the typedef symbol table.
      */
      { is_typedef = TRUE; }
   | tk_extern
   | tk_static
   | tk_auto
   | tk_register
   ;

type_specifier
   : tk_char
   | tk_short
   | tk_int
   | tk_long
   | tk_signed
   | tk_unsigned
   | tk_float
   | tk_double
   | tk_const
   | tk_volatile
   | tk_void
   | struct_or_union_specifier
   | enum_specifier
   | tk_type_name
   ;

struct_or_union_specifier
   /*
      In the following rules, regardless of whether there was a tag,
      indicate that we are no longer looking for one by setting
      looking_for_tag to FALSE.
   */
   : struct_or_union identifier
      { looking_for_tag = FALSE; }
      tk_lcurly_brace struct_declaration_list tk_rcurly_brace
   | struct_or_union tk_lcurly_brace
      { looking_for_tag = FALSE; }
      struct_declaration_list tk_rcurly_brace
   | struct_or_union identifier
      { looking_for_tag = FALSE; }
   ;

struct_or_union
   /*
      Indicate that we are looking for a tag. This information is passed
      to the lexer. Since tags are in a separate name space from other
      identifiers, this helps the lexer determine whether a lexeme is an
      identifier or a type name.
   */
   : tk_struct
      { looking_for_tag = TRUE; }
   | tk_union
      { looking_for_tag = TRUE; }
   ;

struct_declaration_list
   : struct_declaration
   | struct_declaration_list struct_declaration
   ;

struct_declaration
   : type_specifier_list struct_declarator_list tk_semicolon
      {
         ++int_fcn.decl;         /* Declaration statement. */
         ++int_mod.decl;

         /*
            Indicate at the end of a declaration statement then fire the
            end-of-statement trigger.
         */
         int_stm.is_decl = int_stm.end = TRUE;
         fire_stm();
         ZERO(int_stm);
      }
   ;

struct_declarator_list
   : struct_declarator
   | struct_declarator_list tk_comma struct_declarator
   ;

struct_declarator
   : declarator
   | tk_colon constant_expr
   | declarator tk_colon constant_expr
   ;

enum_specifier
   /*
      In the following rules, regardless of whether there was a tag,
      indicate that we are no longer looking for one by setting
      looking_for_tag to FALSE.
   */
   : enum enumeration
      { looking_for_tag = FALSE; }
   | enum identifier
      { looking_for_tag = FALSE; }
      enumeration
   | enum identifier
      { looking_for_tag = FALSE; }
   ;

enumeration
   /*
      Note that an enumerator list can have a trailing comma. This is
      not Standard C, but some C compilers allow it anyway.
   */
   : tk_lcurly_brace enumerator_list tk_rcurly_brace
   | tk_lcurly_brace enumerator_list tk_comma tk_rcurly_brace
   ;

enum
   /*
      Indicate that we are looking for a tag. See struct_or_union for
      more info.
   */
   : tk_enum
      { looking_for_tag = TRUE; }
   ;

enumerator_list
   : enumerator
   | enumerator_list tk_comma enumerator
   ;

enumerator
   : identifier
   | identifier tk_assign constant_expr
   ;

declarator
   : declarator2
   | pointer declarator2
   ;

declarator2
   : identifier
      {
         if (nested_decl_specs == 0)
         {
            strcpy(last_decl, last_identifier());

            /* Fire the declarator trigger. */
            fire_declarator();

            /*
               If we are parsing a typedef and this is the root-level
               declaration, this identifier must be the type name that
               is being being defined by the typedef. Therefore, add it
               to the typedef symbol table so that the lexer can
               distinguish type names from identifiers.
            */
            if (is_typedef)
              typedef_symbol_table_add(token());
         }
      }
   | tk_lparen declarator tk_rparen
   | declarator2 tk_lsquare_brace tk_rsquare_brace
   | declarator2 tk_lsquare_brace constant_expr tk_rsquare_brace
   /*
      For the next three rules, if we are not within a function definition
      already, use the last identifier as the function name.
   */
   | declarator2 tk_lparen
      {
         if (!is_within_function)
            strcpy(function_name, last_identifier());
      }
      tk_rparen
   /*
      nested_decl_specs is incremented and decremented in the following
      two rules so that the name of the root-level item in a declaration
      can be distinguished from other identifiers in the declaration.
   */
   | declarator2 tk_lparen
      {
         ++nested_decl_specs;
         if (!is_within_function)
            strcpy(function_name, last_identifier());
      }
      parameter_type_list
      { --nested_decl_specs; }
      tk_rparen
   | declarator2 tk_lparen
      {
         ++nested_decl_specs;
         if (!is_within_function)
            strcpy(function_name, last_identifier());
      }
      parameter_identifier_list
      { --nested_decl_specs; }
      tk_rparen
   ;

pointer
   : tk_asterisk
   | tk_asterisk type_specifier_list
   | tk_asterisk pointer
   | tk_asterisk type_specifier_list pointer
   ;

type_specifier_list
   : type_specifier
   | type_specifier_list type_specifier
   ;

parameter_identifier_list
   : identifier_list
   | identifier_list tk_comma tk_elipsis
   ;

identifier_list
   : identifier
   | identifier_list tk_comma identifier
   ;

parameter_type_list
   : parameter_list
   | parameter_list tk_comma tk_elipsis
   ;

parameter_list
   : parameter_declaration
   | parameter_list tk_comma parameter_declaration
   ;

parameter_declaration
   : declaration_specifiers declarator
   | declaration_specifiers
   | declaration_specifiers abstract_declarator
   ;

type_name
   : type_specifier_list
   | type_specifier_list abstract_declarator
   ;

abstract_declarator
   : pointer
   | direct_abstract_declarator
   | pointer direct_abstract_declarator
   ;

direct_abstract_declarator
   : tk_lparen abstract_declarator tk_rparen
   | tk_lsquare_brace tk_rsquare_brace
   | tk_lsquare_brace constant_expr tk_rsquare_brace
   | direct_abstract_declarator tk_lsquare_brace tk_rsquare_brace
   | direct_abstract_declarator tk_lsquare_brace constant_expr tk_rsquare_brace
   | tk_lparen tk_rparen
   | tk_lparen parameter_type_list tk_rparen
   | direct_abstract_declarator tk_lparen tk_rparen
   | direct_abstract_declarator tk_lparen parameter_type_list tk_rparen
   ;

initializer
   : assignment_expr
   | tk_lcurly_brace initializer_list tk_rcurly_brace
   | tk_lcurly_brace initializer_list tk_comma tk_rcurly_brace
   ;

initializer_list
   : initializer
   | initializer_list tk_comma initializer
   ;

statement
   : labeled_statement
   | compound_statement
      {
         is_else = is_if = FALSE;

         ++int_fcn.high;      /* High-level statement. */

         /* Used to consider adjacent case labels as one decision point. */
         not_case_label();

         /*
            Indicate at the end of a compound, high-level statement,
            then fire the end-of-statement trigger.
         */
         int_stm.is_comp = int_stm.is_high = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      }
   | expression_statement
   | selection_statement
      {
         ++int_fcn.high;      /* High-level statement. */

         /* Used to consider adjacent case labels as one decision point. */
         not_case_label();

         /*
            Indicate at the end of a selection, high-level statement,
            then fire the end-of-statement trigger.
         */
         int_stm.is_select = int_stm.is_high = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      }
   | iteration_statement
      {
         ++int_fcn.high;      /* High-level statement. */

         /* Used to consider adjacent case labels as one decision point. */
         not_case_label();

         /*
            Indicate at the end of an iteration, high-level statement,
            then fire the end-of-statement trigger.
         */
         int_stm.is_iter = int_stm.is_high = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      }
   | jump_statement
      {
         check_multiple_statements();

         ++int_fcn.low;          /* Low-level statement. */

         /* Used to consider adjacent case labels as one decision point. */
         not_case_label();

         /*
            Indicate at the end of a jump, low-level statement, then
            fire the end-of-statement trigger.
         */
         int_stm.is_jump = int_stm.is_low = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      }
   ;

labeled_statement
   : identifier tk_colon
      {
         /*
            Indicate a label (target of a goto) and the depth at which
            it occurs.
         */
         int_stm.is_label = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      }
   statement
      { is_else = is_if = FALSE; }
   | tk_case constant_expr tk_colon
      {
         check_multiple_statements();

         /* Used to consider adjacent case labels as one decision point. */
         case_label();

         /*
            Each case counts as a condition, but not necessarily a
            decision point. This is because adjacent cases count as one
            decision point, but each of those cases count as a distinct
            condition.
         */
         ++fcn_conditions;
      }
      statement
   | tk_default tk_colon
      { check_multiple_statements(); }
      statement
   ;

compound_statement
   : compound_intro
      { after_compound_declarations(); }
   tk_rcurly_brace
   | compound_intro
      { after_compound_declarations(); }
   statement_list tk_rcurly_brace
   | compound_intro declaration_list
      { after_compound_declarations(); }
   tk_rcurly_brace
   | compound_intro declaration_list
      { after_compound_declarations(); }
   statement_list tk_rcurly_brace
   ;

compound_intro
   : tk_lcurly_brace
      { before_compound_declarations(); }
   ;

declaration_list
   : declaration
   | declaration_list declaration
   ;

statement_list
   : statement
   | statement_list statement
   ;

expression_statement
   : tk_semicolon
      { found_expression_statement(TRUE); }
   | expr tk_semicolon
      { found_expression_statement(FALSE); }
   ;

selection_statement
   : tk_if tk_lparen expr tk_rparen
      {
         is_if = TRUE;
         check_multiple_statements();
         ++depth;                /* Increase control depth. */

         /* This is a decision point and a condition. */
         ++fcn_decisions;
         ++fcn_conditions;
      }
      statement
      { --depth;                 /* Decrease control depth. */ }
      opt_else
   | tk_switch tk_lparen expr tk_rparen
      {
         check_multiple_statements();
         ++depth;                /* Increase control depth. */
      }
      statement
      { --depth;                 /* Decrease control depth. */ }
   ;

opt_else
   :                              %prec THEN
   | tk_else
      {
         check_multiple_statements();
         is_else = TRUE;         /* Must be after check_...() */
         ++depth;                /* Increase control depth. */
      }
      statement
      { --depth;                 /* Decrease control depth. */ }
   ;

iteration_statement
   : tk_while tk_lparen expr tk_rparen
      {
         check_multiple_statements();
         ++depth;                /* Increase control depth. */

         /* This is a decision point and a condition. */
         ++fcn_decisions;
         ++fcn_conditions;
      }
      statement
      { --depth;                 /* Decrease control depth. */ }
   | tk_do
      {
         ++depth;                /* Increase control depth. */

         /* This is a decision point and a condition. */
         ++fcn_decisions;
         ++fcn_conditions;
      }
      statement
      { --depth;                 /* Decrease control depth. */ }
      tk_while tk_lparen expr tk_rparen tk_semicolon
      { check_multiple_statements(); }
   | tk_for tk_lparen opt_expr tk_semicolon tk_semicolon opt_expr tk_rparen
      {
         check_multiple_statements();
         ++depth;                /* Increase control depth. */
         /*
            NOTE: This for statement does not contain a decision point or a
            condition.
         */
      }
      statement
      { --depth;                 /* Decrease control depth. */ }
   | tk_for tk_lparen opt_expr tk_semicolon expr tk_semicolon opt_expr tk_rparen
      {
         check_multiple_statements();
         ++depth;                /* Increase control depth. */

         /* This is a decision point and a condition. */
         ++fcn_decisions;
         ++fcn_conditions;
      }
      statement
      { --depth;                 /* Decrease control depth. */ }
   ;

opt_expr
   :
   | expr
   ;

jump_statement
   : tk_goto identifier tk_semicolon
   | tk_continue tk_semicolon
   | tk_break tk_semicolon
   | tk_return opt_expr tk_semicolon
   ;

/*
   Standard C requires there to be one or more external declarations in
   a translation unit. However, header files often contain preprocessor
   directives but no external declarations. So that header files can be
   parsed independent of any source files that may include them, Metre
   makes external declarations optional in a translation unit.
*/
translation_unit
   :
   | body
   ;

body
   : external_declaration
   | body external_declaration
   ;

external_declaration
   : function_definition
      { stat_func_end(); }
   | declaration
   ;

function_definition
   : declarator
      { stat_func_begin(); }
      function_body
   | declaration_specifiers declarator
      { stat_func_begin(); }
      function_body
   ;

function_body
   : compound_statement
   | declaration_list compound_statement
   ;
%%
/* Functions. */

int main(int main_argc, char *main_argv[])
{
   ZERO(grm);
   ZERO(int_grm);

   /* Input not determined yet. */
   yyin = NULL;
   /* Use stdout until/if output is redirected with command-line option. */
   out_fp = stdout;

   /* Fire the need-command-name trigger. */
   ZERO(int_prj);
   int_prj.need_cmd_name = TRUE;
   fire_prj();
   int_prj.need_cmd_name = FALSE;

   print_banner();

   /* Save so that the argument list may be accessed globally. */
   cmd_line_argv = main_argv;
   cmd_line_argc = main_argc;

   /* Get command-line options that affect Metre at the start. */
   check_starting_options();

   /* Initialize YACC and Lex. */
   init_yacc();
   init_lex();

   if (option(HELP_OPT_CHAR))
      print_help();
   else
   {
      /*
         Look for the first input file and first original name of that
         input file.
      */
      next_cmd_line_file = 0;
      next_cmd_line_file_orig_n = 0;
      if ((input_file = get_next_input_file(&next_cmd_line_file)) != NULL)
         if ((yyin = fopen(input_file, "r")) != NULL)
            begin_project(input_file);
         else
            int_warn(W_CANNOT_OPEN_FILE, input_file);
      else
         /* Since no input file specified on command line, process stdin. */
         begin_project("stdin");
   }

   return RETURN_FROM_MAIN_VALUE;
}

/* Begin the project by firing initial triggers and calling parser. */
static void begin_project(char *input_file)
{
   input_file_orig_name =
         get_next_input_file_orig_name(&next_cmd_line_file_orig_n);
   /* If no original name given, use the actual as the original, also. */
   if (input_file_orig_name == NULL)
      input_file_orig_name = input_file;

   /* Fire the beginning-of-project trigger. */
   ZERO(int_prj);
   int_prj.begin = TRUE;
   fire_prj();
   int_prj.begin = FALSE;

   /* Fire the beginning-of-module trigger. */
   ZERO(int_mod);
   int_mod.begin = TRUE;
   fire_mod();
   int_mod.begin = FALSE;

   /*
      Call the YACC-generated parser to wade through the input file(s).
      When all done, close the last input file. yywrap() could have
      opened any number of subsequent input files.
   */
   yyparse();

   fclose(yyin);
}

/* Common exception-handling function. */
static void print_exception(int n, char *severity_str, char *format, va_list ap)
{
   fflush(out_fp);

   /*
      Display input line if supposed to, then display line with marker
      character in it if appropriate.
   */
   if (mod_name() != NULL && strlen(mod_name()) > 0 &&
         !int_mod.end && !int_fcn.end)
   {
      if (!display_input)
         fputs(line(), out_fp);

      if (!int_lin.end)
         fputs(marker(), out_fp);
   }

   /*
      Display module name and line number if a module name has been
      established. If one hasn't, the error must not relate to a
      location in the input file.
   */
   if (mod_name() != NULL && strlen(mod_name()) > 0)
      fprintf(out_fp, "%s(%u): ", mod_name(), lineno());
   fprintf(out_fp, "%s%04u: ", severity_str, n);

   vfprintf(out_fp, format, ap);
   fputc('\n', out_fp);
}

/* Print message for internal fatal error. */
void int_fatal(int n, char *format, ...)
{
   va_list ap;

   va_start(ap, format);

   print_exception(n, "Fatal Error ME", format, ap);

   exit(1);
}

/* Print message for fatal error that occured in the rules() function. */
void fatal(int n, char *format, ...)
{
   va_list ap;

   va_start(ap, format);

   print_exception(n, "Fatal Error E", format, ap);

   exit(1);
}

/* Print message for internal warning. */
static void int_warn(int n, char *format, ...)
{
   va_list ap;

   if (!option(NO_WARNINGS_OPT_CHAR))
   {
      va_start(ap, format);

      print_exception(n, "Warning MW", format, ap);
   }
}

/* Print message for warning that occured in the rules() function. */
void warn(int n, char *format, ...)
{
   va_list ap;

   if (!option(NO_WARNINGS_OPT_CHAR))
   {
      va_start(ap, format);

      print_exception(n, "Warning W", format, ap);
   }
}

/* Function called by YACC when it detects an error. */
void yyerror(char *s)
{
#ifdef DEBUG_TYPEDEF
   /* Used for debugging typedef processing. */
   typedef_symbol_table_dump();
#endif
   int_fatal(0, "%s", s);
}

/* Add type name to the typedef symbol table. */
static void typedef_symbol_table_add(char *p_symbol)
{
   TYPEDEF_SYM_BLK *block;
   /* Allocate room for string and its '\0' terminator. */
   char *str_buf = (char *)malloc(strlen(p_symbol) + 1);

   if (str_buf == NULL)
      int_fatal(E_NO_HEAP);

   /* Make copy of argument. */
   strcpy(str_buf, p_symbol);

   /* Is symbol table empty? */
   if (typedef_sym_tbl_head == NULL)
   {
      /* Allocate first block and make the head of the table list. */
      typedef_sym_tbl_head = block = (TYPEDEF_SYM_BLK *)malloc(sizeof *block);
      if (block == NULL)
         int_fatal(E_NO_HEAP);

      block->total = 0;
      block->next = NULL;
   }
   else
   {
      TYPEDEF_SYM_BLK *prev_block;

      /* Find first block in symbol-table list that isn't full. */
      for (prev_block = NULL, block = typedef_sym_tbl_head;
            block != NULL && block->total == DIM_OF(block->id);
            prev_block = block, block = block->next)
         ;

      /* If all blocks are full, allocate a new one and append to list. */
      if (block == NULL)
      {
         block = (TYPEDEF_SYM_BLK *)malloc(sizeof *block);
         if (block == NULL)
            int_fatal(E_NO_HEAP);

         block->total = 0;
         block->next = NULL;
         if (prev_block == NULL)
            typedef_sym_tbl_head = block;
         else
            prev_block->next = block;
      }
   }

   /* Save pointer to the new type name in first empty slot of this block. */
   block->id[block->total++] = str_buf;
}

/* Return TRUE if symbol exists in the typedef symbol table. */
BOOLEAN typedef_symbol_table_find(char *p_symbol)
{
   TYPEDEF_SYM_BLK *block;
   BOOLEAN found = FALSE;

   /* Loop for each block of symbols. */
   for (block = typedef_sym_tbl_head; block != NULL; block = block->next)
   {
      unsigned i;

      /* Loop for each symbol in this block. */
      for (i = 0; i < block->total; ++i)
         if (strcmp(block->id[i], p_symbol) == 0)
            found = TRUE;
   }

   return found;
}

/*
   Remove all symbols (and symbol blocks) from the symbol table--make
   the table empty.
*/
void typedef_symbol_table_flush(void)
{
   TYPEDEF_SYM_BLK *block;

   /* Loop for each block of symbols. */
   for (block = typedef_sym_tbl_head; block != NULL; )
   {
      unsigned i;
      TYPEDEF_SYM_BLK *temp_block;

      /* Loop for each symbol in this block. */
      for (i = 0; i < block->total; ++i)
         free(block->id[i]);              /* Free symbol. */

      temp_block = block->next;
      free(block);                        /* Free block now that it's empty. */
      block = temp_block;
   }

   typedef_sym_tbl_head = NULL;
}

#ifdef DEBUG_TYPEDEF
/*
   Display the contents of the typedef symbol table. Used for debugging
   typedef processing.
*/
void typedef_symbol_table_dump(void)
{
   TYPEDEF_SYM_BLK *block;

   fputs("typedef symbol table dump:\n", out_fp);

   /* Loop for each block of symbols. */
   for (block = typedef_sym_tbl_head; block != NULL; block = block->next)
   {
      unsigned i;

      /* Loop for each symbol in this block. */
      for (i = 0; i < block->total; ++i)
         fprintf(out_fp, "%s\n", block->id[i]);
   }
}
#endif

/* Record that the last statement was a case label. */
static void case_label(void)
{
   is_case_label = TRUE;
}

/*
   Record that the last statement was not a case label. If the previous
   statement was a case label, increment the decision-point
   counter--this ends a run of adjacent case labels which count as one
   decision point.
*/
static void not_case_label(void)
{
   if (is_case_label)
   {
      is_case_label = FALSE;
      ++fcn_decisions;        /* This is a decision point. */
   }
}

/* Initialize parser. */
void init_yacc(void)
{
   /* Make the typedef symbol table empty. */
   typedef_symbol_table_flush();

   /*
      See the definition of each of these variables, above, for more
      information.
   */
   is_typedef = FALSE;
   nested_decl_specs = 0;
   is_case_label = FALSE;

   function_name[0] = '\0';
   last_ident[0] = '\0';
   last_decl[0] = '\0';

   mod_decisions = 0;
   mod_conditions = 0;
   mod_functions = 0;
   is_else = is_if = FALSE;
   looking_for_tag = FALSE;

   is_within_function = FALSE;

   current_operator = "";
   current_keyword = "";
   current_identifier = "";
   current_type_name = "";
}

/*
   Returns 1 if the indicated option character was specified on the
   command line. E.g., for -S, it would return 1 for option('S'). If
   option character is immediately followed by a decimal integer, this
   function will instead return the value of that number, e.g. for -S3,
   it would return 3. NOTE: 0 would be ambiguous, e.g., -S0, because 0
   would also be returned if -S were not specified.
*/
int option(int opt_char)
{
   char *opt_str = str_option(opt_char);

   return opt_str == NULL ? 0 : strlen(opt_str) == 0 ? 1 : atoi(opt_str);
}

/*
   Returns pointer to the part of an option following the option character.
   E.g., for -Xabc=def, it would point to the "abc=def" part.
*/
char *str_option(int opt_char)
{
   unsigned i;

   /* (Skip the command itself, argv[0].) */
   for (i = 1; i < cmd_line_argc; ++i)
      if (strchr(OPT_INTRO_CHARS, cmd_line_argv[i][0]) != NULL)
         if (cmd_line_argv[i][OPTION_START] == opt_char)
            break;

   return i < cmd_line_argc ? &cmd_line_argv[i][OPTION_STRING_START] : NULL;
}

/*
   Return pointer to the name of the next input file in the argv array
   starting with the argument that p_i points to. An input file name is
   distinguished from the command-line options by not starting with one
   of characters in the OPT_INTRO_CHARS string.

   NOTE: argument 0 is the name of the command, not the first argument,
   per se.
*/
char *get_next_input_file(unsigned *p_i)
{
   for (++*p_i; *p_i < cmd_line_argc &&
         strchr(OPT_INTRO_CHARS, cmd_line_argv[*p_i][0]) != NULL; ++*p_i)
      ;

   return *p_i < cmd_line_argc ? cmd_line_argv[*p_i] : NULL;
}

/*
   Return pointer to the original name of the next input file in the
   argv array starting with the argument that p_i points to. An original
   name is specified as a command-line option. That is how an original
   name is distinguished from the name of an actual input file

   NOTE: argument 0 is the name of the command, not the first argument,
   per se.
*/
char *get_next_input_file_orig_name(unsigned *p_i)
{
   for (++*p_i; *p_i < cmd_line_argc; ++*p_i)
      if (strchr(OPT_INTRO_CHARS, cmd_line_argv[*p_i][0]) != NULL &&
            cmd_line_argv[*p_i][OPTION_START] == SUBST_FILE_OPT_CHAR)
         break;

   return *p_i < cmd_line_argc ?
         &cmd_line_argv[*p_i][OPTION_STRING_START] : NULL;
}

/* Returns pointer to current module (file) name. */
char *mod_name(void)
{
   return input_file_orig_name;
}

/* Process the command-line options that affect the start of Metre. */
static void check_starting_options(void)
{
   static char *output_file_name;
   char *script_file_name;

   /* Get options from file rather than command line? */
   script_file_name = str_option(SCRIPT_FILE_OPT_CHAR);
   if (script_file_name != NULL)
      get_options_from_file(script_file_name);

   /* Interleave the input with the output? */
   display_input = option(COPY_INPUT_OPT_CHAR);

   /*
      Name of listing file to contain output. This for those OS's that don't
      support command-line redirection, e.g., VMS.
   */
   output_file_name = str_option(LISTING_OPT_CHAR);
   if (output_file_name != NULL && strlen(output_file_name) > 0)
   {
      out_fp = fopen(output_file_name, "w");
      if (out_fp == NULL)
      {
         out_fp = stdout;     /* Restore to previous, good output file. */
         int_fatal(E_CANT_OPEN_LISTING_FILE);   /* Die. */
      }
   }
}

/*
   Build a "fake" argument list and count, copy command name (argv[0])
   to it, then append the options that are in the file to it. (Sneaky,
   aren't I?) (Perhaps the arguments from the file should be appended
   to whatever is on the command line. Maybe next time.)
*/
static void get_options_from_file(char *file)
{
   FILE *cmd_fp;

   /*
      If other options on command line, generate warning but continue.
      Note: 2 = the command itself + the file option.
   */
   if (cmd_line_argc > 2)
      int_warn(W_OPTIONS_IGNORED);

   /* Attempt to open the specified script file. */
   if (strlen(file) == 0 || (cmd_fp = fopen(file, "r")) == NULL)
      int_fatal(E_CANT_OPEN_COMMAND_FILE);
   else
   {
      ARG *args = NULL, *args_tail = NULL;
      char buf[BUFSIZ];

      /*
         Initialize argc to 1 for the command name which is carried over
         from the orginal argv.
      */
      cmd_line_argc = 1;

      /* Loop for each line of input. */
      while (fgets(buf, sizeof buf, cmd_fp) != NULL)
      {
         char *start;   /* Buffer address 1st time through loop, then NULL. */
         char *token;   /* Pointer to token in buffer. */

         /* Loop for each token in this line of input. */
         for (start = buf; (token = strtok(start, WHITE_SPACE_CHARS)) != NULL;
               start = NULL)
         {
            ARG *arg;

            /*
               Allocate buffers for this argument and its string
               (including its '\0' terminator).
            */
            if ((arg = (ARG *)malloc(sizeof *arg)) == NULL)
               int_fatal(E_NO_HEAP);
            if ((arg->string = (char *)malloc(strlen(token) + 1)) == NULL)
               int_fatal(E_NO_HEAP);

            /* Copy argument into string buffer. */
            strcpy(arg->string, token);

            /* Attach this argument to end of argument linked list. */
            if (args_tail == NULL)
               args = arg;
            else
               args_tail->next = arg;

            args_tail = arg;
            arg->next = NULL;

            ++cmd_line_argc;
         }
      }

      /* Allocate buffer for argv array then move arguments from linked list. */
      build_new_argv(args);
   }
}

/* Build a new list of "command-line arguments" pointed to by cmd_line_argv. */
static void build_new_argv(ARG *args)
{
   unsigned i;
   char *argv_cmd_name;

   /* Save pointer to command name from original argv. */
   argv_cmd_name = cmd_line_argv[0];

   /* Allocate buffer to hold array of pointers to arguments. */
   cmd_line_argv = (char **)malloc(cmd_line_argc * sizeof *cmd_line_argv);
   if (cmd_line_argv == NULL)
      int_fatal(E_NO_HEAP);

   /* Restore pointer to command name. */
   cmd_line_argv[0] = argv_cmd_name;

   /*
      Fill in new argv from linked list of arguments, freeing list
      entries but not the strings from the list.  (Skip the place for
      the "command" argument.)
   */
   for (i = 1; i < cmd_line_argc; ++i)
   {
      ARG *arg;

      cmd_line_argv[i] = args->string;

      arg = args->next;
      free(args);
      args = arg;
   }
}

/* Print name, version, and copyright notice. */
static void print_banner(void)
{
   fprintf(out_fp, "METRE Version %s  ", metre_version);
   fputs("Copyright (c) 1993-1995 by Paul Long  All rights reserved.\n", out_fp);
}

/* Print Metre command-usage information. */
static void print_help(void)
{
   fprintf(out_fp, "Syntax: %s [ option | file ]...\n", command_name);
   fprintf(out_fp, "-%cxxx=[xxx] Define identifier          "
         "-%cxxx       Name of output file\n",
         DEFINE_OPT_CHAR, LISTING_OPT_CHAR);
   fprintf(out_fp, "-%cxxx       Substitute file name       "
         "-%c          Copy input to output\n",
         SUBST_FILE_OPT_CHAR, COPY_INPUT_OPT_CHAR);
   fprintf(out_fp, "-%c          Suppress warnings          "
         "-%c          This help\n", NO_WARNINGS_OPT_CHAR, HELP_OPT_CHAR);
   fprintf(out_fp, "-%cxxx       Response file\n", SCRIPT_FILE_OPT_CHAR);

   /* Fire the project-help trigger. */
   ZERO(int_prj);
   int_prj.help = TRUE;
   fire_prj();
   int_prj.help = FALSE;
}

/* Called at the beginning of a function definition. */
static void stat_func_begin(void)
{
   /* Fire the beginning-of-function trigger. */
   ZERO(int_fcn);
   int_fcn.begin = TRUE;
   fire_fcn();
   int_fcn.begin = FALSE;

   /* Initialize variables relating to a function. */
   depth = 0;
   is_within_function = TRUE;
   fcn_decisions = 0;
   fcn_conditions = 0;
   previous_statements_line_number = 0;

   /* Install starting counts. Replaced with deltas at end-of-function. */
   int_fcn.lines.white = int_mod.lines.white;
   int_fcn.lines.com = int_mod.lines.com;
   int_fcn.lines.code = int_mod.lines.code;
   int_fcn.lines.total = lineno();
}

/* Called at the end of a function definition. */
static void stat_func_end(void)
{
   /*
      Replace initial counts with their deltas to arrive at totals for
      the function.
   */
   int_fcn.lines.white = int_mod.lines.white - int_fcn.lines.white;
   int_fcn.lines.com = int_mod.lines.com - int_fcn.lines.com;
   int_fcn.lines.code = int_mod.lines.code - int_fcn.lines.code;
   int_fcn.lines.total = lineno() - int_fcn.lines.total;

   ++mod_functions;                 /* Increment function count. */

   /* Accumulate function decision and condition counts into module counts. */
   mod_decisions += fcn_decisions;
   mod_conditions += fcn_conditions;

   /*
      Install the rest of the function-related information into the
      function structure and fire the end-of-function trigger.
   */
   int_fcn.decisions = fcn_decisions;
   int_fcn.conditions = fcn_conditions;
   int_fcn.end = TRUE;
   fire_fcn();
   ZERO(int_fcn);

   /*
      Clear variables that would otherwise indicate that we are still
      parsing a function.
   */
   strcpy(function_name, "");
   is_within_function = FALSE;
}

/*
   Returns pointer to current function name, or an empty string if not
   in a function.
*/
char *fcn_name(void)
{
   return function_name;
}

/*
   Increment the number of statements for the current line. This
   function is called whenever a statement is encountered. If the
   current line number is the same as when the last statement was
   encountered and if this is not the special situation of "else if,"
   the counter is incremented.
*/
static void check_multiple_statements(void)
{
   if (previous_statements_line_number == lineno())
      if (is_else && is_if)
         ;        /* do nothing */
      else
         ++int_lin.statements;
   else
      previous_statements_line_number = lineno();

   is_else = is_if = FALSE;
}


/*
   This is true for all of the fire_*() functions: Copy the internal
   structure to the exported structure, allow the triggers to fire in
   the rules() function, then zero-fill the exported structure so that
   unrelated rules won't execute when other triggers are fired.
*/

void fire_prj(void)
{
   prj = int_prj;
   rules();
   ZERO(prj);
}

void fire_mod(void)
{
   mod = int_mod;
   rules();
   ZERO(mod);
}

static void fire_fcn(void)
{
   fcn = int_fcn;
   rules();
   ZERO(fcn);
}

static void fire_stm(void)
{
   stm = int_stm;
   rules();
   ZERO(stm);
}

void fire_lin(void)
{
   lin = int_lin;
   rules();
   ZERO(lin);
}

void fire_lex(void)
{
   lex = int_lex;
   rules();
   ZERO(lex);
}

void fire_grm(void)
{
   grm = int_grm;
   rules();
   ZERO(grm);
}

/* Fire the operator trigger. */
static void fire_operator(void)
{
   current_operator = token();
   int_grm.is_operator = TRUE;
   fire_grm();
   int_grm.is_operator = FALSE;
   current_operator = "";
}

/* Fire the keyword trigger. */
static void fire_keyword(void)
{
   current_keyword = token();
   int_grm.is_keyword = TRUE;
   fire_grm();
   int_grm.is_keyword = FALSE;
   current_keyword = "";
}

/* Fire the identifier trigger. */
static void fire_identifier(void)
{
   current_identifier = token();
   int_grm.is_identifier = TRUE;
   fire_grm();
   int_grm.is_identifier = FALSE;
   current_identifier = "";
}

/* Fire the declarator trigger. */
static void fire_declarator(void)
{
   int_grm.is_declarator = TRUE;
   fire_grm();
   int_grm.is_declarator = FALSE;
}

/* Fire the type-name trigger. */
static void fire_type_name(void)
{
   current_type_name = token();
   int_grm.is_type_name = TRUE;
   fire_grm();
   int_grm.is_type_name = FALSE;
   current_type_name = "";
}

/* Fire the grammar trigger with is_constant set to TRUE. */
static void found_constant(void)
{
   int_grm.is_constant = TRUE;
   fire_grm();
   ZERO(int_grm);
}

/* Fire the grammar trigger with is_string set to TRUE. */
static void found_string(void)
{
   int_grm.is_string = TRUE;
   fire_grm();
   ZERO(int_grm);
}

/* Indicate a function call. */
static void function_call(void)
{
   int_grm.is_call = TRUE;
   fire_grm();
   ZERO(int_grm);
}

/* Indicate an expression statement. Can be empty, i.e. just ";". */
static void found_expression_statement(BOOLEAN is_empty)
{
   check_multiple_statements();

   ++int_fcn.low;          /* Low-level statement. */

   /* Used to consider adjacent case labels as one decision point. */
   not_case_label();

   /*
      Indicate at the end of an expression, low-level statement, whether
      an empty expression, then fire the end-of-statement trigger.
   */
   int_stm.is_expr = int_stm.is_low = int_stm.end = TRUE;
   int_stm.is_empty_expr = is_empty;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/* Return whether the specified operator is the current operator. */
BOOLEAN oper(char *op)
{
   return grm.is_operator && strcmp(current_operator, op) == 0;
}

/* Return whether the specified keyword is the current keyword. */
BOOLEAN keyword(char *name)
{
   return grm.is_keyword && strcmp(current_keyword, name) == 0;
}

/* Return whether the specified identifier is the current identifier. */
BOOLEAN identifier(char *name)
{
   return grm.is_identifier && strcmp(current_identifier, name) == 0;
}

/*
   Indicate the beginning of the declarations within a compound
   statement, whether or not there were any, and the depth at which they
   occured.
*/
static void before_compound_declarations(void)
{
   int_stm.before_compound_declarations = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/*
   Indicate the end of the declarations within a compound statement,
   whether or not there were any, and the depth at which they occured.
*/
static void after_compound_declarations(void)
{
   int_stm.after_compound_declarations = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/*
   Indicate the beginning of an initializer (before the "=") and the
   depth at which it occurs.
*/
static void before_initializer(void)
{
   int_stm.before_initializer = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/* Indicate the end of an initializer and the depth at which it occured. */
static void after_initializer(void)
{
   int_stm.after_initializer = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/*
   Return pointer to identifier that corresponds to when grammar
   identifier trigger is fired.
*/
char *this_identifier(void)
{
   return current_identifier;
}

/* Return pointer to last-encountered identifier. */
char *last_identifier(void)
{
   return last_ident;
}

/* Return pointer to last-encountered declarator. */
char *last_declarator(void)
{
   return last_decl;
}

/*
   When prj.cmd_name is true, a rule may provide the name of the command
   that invokes this program. The default is "METRE".
*/
void cmd_name(char *name)
{
   if (name != NULL)
      command_name = name;
}

/* Return command-line argument count. */
int argc(void)
{
   return cmd_line_argc;
}

/* Return pointer to command-line-argument array. */
char **argv(void)
{
   return cmd_line_argv;
}
