/*************************************************************
   Copyright (c) 1993-1995 by Paul Long  All rights reserved.
**************************************************************/

/*************************************************************
   metrules.c  This source file contains a set of rules for
               the METRE metrics tool.
**************************************************************/


#include <stdarg.h>
#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "metre.h"


/* Manifest constants. */

/* Do Halstead metrics. Do not normally because slows things down. */
#define HALSTEAD_OPT_CHAR     'H'
/* Adjust function points. (Nonstandard adjustment calculation.) */
#define ADJUST_FP_OPT_CHAR    'a'
/* Just print the summaries (no function-specific info)? */
#define SUMMARY_OPT_CHAR      's'
/*
   Value for the "Stroud number" in Halstead's Programming Time metric.
   18 is the default.
*/
#define STROUD_OPT_CHAR       'T'
/* Produce comma-separated-value, machine-readable output. */
#define CSV_OPT_CHAR          'c'
/* Print a particluar metric, e.g., -p=fnp. Can be repeated. */
#define SELECT_METRIC_OPT_CHAR   'm'

/* Defines for diagnostics that are generated by the rules, not the parser. */
#define F_MEMORY              0, "Out of memory"
#define F_BAD_CYCLO           1, "Cyclomatic complexity of 0"
#define W_GOTO                0, "Goto statement used (%u time%s)"
#define W_CONTINUE            1, "Continue statement used (%u time%s)"
#define W_RETURNS             2, "Multiple return statements (%u)"
#define W_TAB_SPACE_INDENT    3, "Tab & space used on same line for indention"
#define W_MULT_STATEMENTS     4, "Multiple statements (%u) per line"
#define W_NONSTANDARD         5, "Non-standard character (0x%x) encountered"
#define W_BAD_METRIC          6, "Invalid metric identifier (%s)"

/* Nicety to evaluate as "s" if argument is greater than one; otherwise, "". */
#define NUMBER(x)       ((x) > 1 ? "s" : "")

/* Evaluate to stringified x and value of x. */
#define STRX_X(x) #x,x

/* Assign max value to integer argument regardless of type, e.g., long int. */
#define TYPE_MAX(x) ((x = ~0) < 0 ? x = ~(1 << ((sizeof x * CHAR_BIT) - 1)) : x)

/* Halstead requires log2(). This macro uses log10() to calculate log2(). */
#define log2(x)         (log10(x) / log10_to_2_divisor)

/*
   Function points are backfired by dividing the number of statements by
   this factor.
*/
#define STMNT_TO_FP_DIVISOR   (128.0)

/* Default Stroud number used in calculating Halstead Programming Time. */
#define DEFAULT_STROUD_NUMBER (18)

/* Conversion factor from seconds to hours. */
#define SECONDS_TO_HOURS_FACTOR  (60.0 * 60.0)

/*
   Since C truncates when converting float to integral, force rounding
   by adding 0.5 before conversion. This manifest constant provides the
   proper addend which is to be added to the float value before converting
   to an integral type.
*/
#define ROUND  (0.5)

/* Number of columns on output device. */
#define OUTPUT_WIDTH 80


/* General-purpose statistics structure. */
typedef struct {
   unsigned long times; /* Number of times stat recorded. */
   unsigned long total; /* Summation of stat being measured. */
   unsigned long min;   /* Minimum and maximum value of stat. */
   unsigned long max;
} STAT;


/*
   The following structures are for statistics that are maintained at
   the function, module, and project level.
*/

/* Function-level statistics. */
static struct {
   STAT idents;               /* Identifiers. */
   STAT max_depth;            /* Nesting level. */
   unsigned comp;             /* How many compound statements. */
   unsigned empty_expr;       /* How many empty expression statements. */
   unsigned label;            /* How many (goto) labels. */
   unsigned gotos;            /* How many gotos. */
   unsigned continues;        /* How many continues. */
   unsigned returns;          /* How many returns. */

   /* For Halstead's Software Science metric. */
   unsigned long operator_length;
   unsigned long operand_length;
} function;

/* Module-level statistics. */
static struct {
   /* ...across all functions in a module. */
   struct {
      STAT decisions,         /* Binary decision points. */
         conditions,          /* How many conditions, not just decisions. */
         /* Halstead's Software Science metrics. */
         length, vocabulary, volume, level, effort, intelligence, lang_level,
         /* Various LOC metrics. */
         lines, code_lines, comment_lines, white_lines,
         decls,               /* Number of declaration statements. */
         stmnts,              /* Number of executable statements. */
         max_depth;           /* Maximum control depth. */
   } function;

   STAT idents;               /* Identifiers. */
   /* Various LOC metrics. */
   unsigned lines, code_lines, comment_lines, white_lines;
} module;

/* Project-level statistics. */
static struct {
   /* ...across all functions in a module. */
   struct {
      STAT decisions,         /* Binary decision points. */
         conditions,          /* How many conditions, not just decisions. */
         /* Halstead's Software Science metrics. */
         length, vocabulary, volume, level, effort, intelligence, lang_level,
         /* Various LOC metrics. */
         lines, code_lines, comment_lines, white_lines,
         decls,               /* Number of declaration statements. */
         stmnts,              /* Number of executable statements. */
         max_depth;           /* Maximum control depth. */
   } function;

   /* ...across all modules in a module. */
   struct {
      /* Various LOC metrics. */
      STAT lines, code_lines, comment_lines, white_lines,
         decls,               /* Number of declaration statements. */
         stmnts,              /* Number of executable statements. */
         pp,                  /* Number of preprocessor statements. */
         funcs;               /* Number of functions. */
   } module;

   STAT idents;               /* Identifiers. */
   unsigned long modules;     /* Number of modules. */
} project;

/* Used by Halstead metrics to record each term in a vocabulary. */
typedef struct term {
   struct term *next;   /* Next term in the linked list. */
   char *string;        /* Lexeme */
   unsigned count;      /* Number of occurences. */
} TERM;
/* Vocabularies as table of string terms.*/
static TERM *operator_vocabulary = NULL;
static TERM *operand_vocabulary = NULL;

/* Conversion divisor from 1og 10 to 2.*/
static double log10_to_2_divisor;

/* Moments per hour. */
static float moments_per_hour;

/* Types of metrics. */
enum metric_type {
   bll,  /* Blank lines. */
   cyc,  /* Cyclomatic complexity. */
   cdl,  /* Code lines. */
   cml,  /* Comment lines. */
   dcs,  /* Declaration statements. */
   exc,  /* Extended complexity. */
   exs,  /* Executable statements. */
   fnc,  /* Function count. */
   fnp,  /* Function points. */
   inc,  /* Intelligence content. */
   idc,  /* Identifier count. */
   idl,  /* Identifier length. */
   llv,  /* Language level. */
   mdc,  /* Module count. */
   mcd,  /* Maximum control depth. */
   pef,  /* Programming effort. */
   pln,  /* Program length. */
   plv,  /* Program level. */
   pps,  /* Preprocessor statements. */
   ptm,  /* Programming time. */
   pvl,  /* Program volume. */
   pvc,  /* Program vocabulary. */
   scl   /* Source lines. */
};

static struct {
   char *name;             /* Abbreviated name of metric (same as enum id). */
   enum metric_type type;  /* Type of metric. */
   BOOLEAN print;          /* Whether to print this metric. */

   /* Whether this metric is part of a particular section in the output. */
   BOOLEAN fcn,            /* Function metrics. */
         mod_fcn,          /* Function-related module metrics. */
         mod,              /* Module-related module metrics. */
         prj_fcn,          /* Function-related project metrics. */
         prj_mod,          /* Module-related project metrics. */
         prj;              /* Project-related projecrt metrics. */
} metric[] = {
   /* name type   print    fcn      mod-fcn  mod      prj-fcn  prj-mod  prj */
   { STRX_X(bll), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(cyc), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(cdl), FALSE,   TRUE,    TRUE,    TRUE,    TRUE,    TRUE,    TRUE  },
   { STRX_X(cml), FALSE,   TRUE,    TRUE,    TRUE,    TRUE,    TRUE,    TRUE  },
   { STRX_X(dcs), FALSE,   TRUE,    TRUE,    TRUE,    TRUE,    TRUE,    TRUE  },
   { STRX_X(exc), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(exs), FALSE,   TRUE,    TRUE,    TRUE,    TRUE,    TRUE,    TRUE  },
   { STRX_X(fnc), FALSE,   FALSE,   FALSE,   TRUE,    FALSE,   TRUE,    TRUE  },
   { STRX_X(fnp), FALSE,   FALSE,   FALSE,   TRUE,    FALSE,   FALSE,   TRUE  },
   { STRX_X(inc), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(idc), FALSE,   TRUE,    FALSE,   TRUE,    FALSE,   FALSE,   TRUE  },
   { STRX_X(idl), FALSE,   TRUE,    FALSE,   TRUE,    FALSE,   FALSE,   TRUE  },
   { STRX_X(llv), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(mdc), FALSE,   FALSE,   FALSE,   FALSE,   FALSE,   FALSE,   TRUE  },
   { STRX_X(mcd), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(pef), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(pln), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(plv), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(pps), FALSE,   FALSE,   FALSE,   TRUE,    FALSE,   TRUE,    TRUE  },
   { STRX_X(ptm), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(pvl), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(pvc), FALSE,   TRUE,    TRUE,    FALSE,   TRUE,    FALSE,   FALSE },
   { STRX_X(scl), FALSE,   TRUE,    TRUE,    TRUE,    TRUE,    TRUE,    TRUE  },
};

/* Local-function prototypes. */
static void print_help(void);
static BOOLEAN mark_metrics_to_print(void);
static void mark_sections_to_print(BOOLEAN *, BOOLEAN *, BOOLEAN *, BOOLEAN *,
      BOOLEAN *, BOOLEAN *);
static BOOLEAN print_metric(enum metric_type);
static void print_project_summary(BOOLEAN, BOOLEAN, BOOLEAN, BOOLEAN, BOOLEAN,
      BOOLEAN);
static void print_module_summary(BOOLEAN, BOOLEAN, BOOLEAN, BOOLEAN, BOOLEAN);
static void print_function_summary(BOOLEAN, BOOLEAN, unsigned, float,
      long unsigned, long unsigned, unsigned, unsigned, float);
static void init_function_stats(void);
static void init_module_stats(BOOLEAN);
static void init_project_stats(BOOLEAN);
static void update_project_stats(BOOLEAN);
static void update_module_stats(BOOLEAN, unsigned, float, long unsigned,
      long unsigned, unsigned, unsigned, float);
static float fp_adjustment_factor(unsigned long);
static unsigned executable_function_statements(void);
static int out(const char *, ...);
static void init_stat(STAT *);
static void record_stat(STAT *, unsigned long);
static void absorb_stat(STAT *, STAT *);
static unsigned long min_stat(STAT *);
static unsigned long max_stat(STAT *);
static unsigned long tim_stat(STAT *);
static unsigned long avg_stat(STAT *);
static unsigned long tot_stat(STAT *);
static unsigned clear_vocab(TERM **);
static void add_term(TERM **, char *);
static TERM *lookup_term(TERM *, char *);
static void remove_term(TERM **, char *);

void rules(void)
{
   /* Whether spaces & tabs in indent. */
   static BOOLEAN mixed_indent_lines;
   /* Whether within logic, as opposed to a declaration. */
   static BOOLEAN is_logic = FALSE;
   /* Whether we are within a function definition. */
   static BOOLEAN is_within_function = FALSE;
   /* Adjust function points with non-standard algorithm? */
   static BOOLEAN adjust_function_points = FALSE;
   /* Whether to calculate the Halsetad metrics. */
   static BOOLEAN do_halstead = FALSE;
   /* Just print the summaries? */
   static BOOLEAN summaries_only = FALSE;
   /* Whether to generate output in Comma-Separated-Value format. */
   static BOOLEAN csv;
   /* Whether whole sections are printed. */
   static BOOLEAN
         print_fcn,     /* Function metrics. */
         print_mod_fcn, /* Function-related module metrics. */
         print_mod,     /* Module-related module metrics. */
         print_prj_fcn, /* Function-related project metrics. */
         print_prj_mod, /* Module-related project metrics. */
         print_prj;     /* Project-related project metrics. */

   /* Addendum to main help info when -h is specified. */
   if (prj.help)
      print_help();

   if (prj.begin)
   {
      unsigned stroud_number;

      /* Save log10-to-log2 factor so do not have to recalculate. */
      log10_to_2_divisor = log10(2);

      stroud_number = option(STROUD_OPT_CHAR);
      if (stroud_number == 0)
         stroud_number = DEFAULT_STROUD_NUMBER;
      moments_per_hour = stroud_number * SECONDS_TO_HOURS_FACTOR;

      do_halstead = option(HALSTEAD_OPT_CHAR);
      adjust_function_points = option(ADJUST_FP_OPT_CHAR);
      summaries_only = option(SUMMARY_OPT_CHAR);
      csv = option(CSV_OPT_CHAR);

      /* Mark which metrics should be printed. */
      if (mark_metrics_to_print())
         /*
            If at least one of the Halstead metrics has been selected to be
            printed, calculate the root Halstead metrics. Same as if the user
            had used the -H command-line option.
         */
         if (print_metric(pln) || print_metric(pvc) || print_metric(pvl) ||
               print_metric(plv) || print_metric(pef) || print_metric(inc) ||
               print_metric(ptm) || print_metric(llv))
            do_halstead = TRUE;

      /* Determine which entire sections are to be printed. */
      mark_sections_to_print(&print_fcn, &print_mod_fcn, &print_mod,
         &print_prj_fcn, &print_prj_mod, &print_prj);

      init_project_stats(do_halstead);
   }

   /*
      At the end of the project, print a project summary if multiple
      modules and at least one function-, module-, or project-related
      project metric has been selected.
   */
   if (prj.end && project.modules > 1 &&
         (print_prj_fcn || print_prj_mod || print_prj))
      print_project_summary(csv, print_prj_fcn, print_prj_mod, print_prj,
            do_halstead, adjust_function_points);

   if (mod.begin)
   {
      if (!csv) {
         out("\n****************************************************");
         out("\n\n%s:\n", mod_name());
      }

      mixed_indent_lines = FALSE;

      init_module_stats(do_halstead);
   }

   /*
      At the end of each module, update project statistics and print a
      module summary.
   */
   if (mod.end)
   {
      update_project_stats(do_halstead);

      /*
         Only print module metrics if at least one function- or
         module-related module metric has been selected.
      */
      if (print_mod || print_mod_fcn)
         print_module_summary(csv, print_mod, print_mod_fcn,
               do_halstead, adjust_function_points);

      if (mixed_indent_lines)
         warn(W_TAB_SPACE_INDENT);
   }

   if (fcn.begin)
   {
      init_function_stats();

      is_within_function = TRUE;
   }

   /*
      At the end of each function, update module statistics and print
      function statistics.
   */
   if (fcn.end)
   {
      unsigned unique_operators, unique_operands, volume;
      float level;
      long unsigned effort;
      long unsigned length;
      unsigned vocabulary;
      unsigned intelligence;
      float lang_level;

      if (do_halstead) {
         unsigned unique_terms;

         unique_operators = clear_vocab(&operator_vocabulary);
         unique_operands = clear_vocab(&operand_vocabulary);
         unique_terms = unique_operators + unique_operands;

         /*
            Program Length                  N = N1 + N2
            Program Vocabulary              n = n1 + n2
            Program Volume                  V = N * log2(n)
            Program Level                   L^ = (2 / n1) * (n2 / N2)
            Programming Effort              E = V / L^
            Programming Time (in hours)     T^ = E / (f * S)
            Intelligence Content            I = L^ * V
            Language Level                  L' = L^ * I
         */

         /* Detect log2(0) before it happens. */
         if (unique_terms == 0)
            volume = 0;
         else
            volume = (function.operator_length + function.operand_length) *
                  log2(unique_terms) + ROUND;

         if (unique_operators == 0 || function.operand_length == 0)
            level = 0.0;
         else
            level = (2.0 / unique_operators) *
                  ((float)unique_operands / function.operand_length);

         if (level == 0.0)
            effort = 0;
         else
            effort = (long unsigned)(volume / level + ROUND);

         length = function.operator_length + function.operand_length;
         vocabulary = unique_operators + unique_operands;
         intelligence = level * volume + ROUND;
         lang_level = level * level * volume;

         is_logic = FALSE;
      }

      is_within_function = FALSE;

      update_module_stats(do_halstead, volume, level, effort, length,
            vocabulary, intelligence, lang_level);

      /*
         Do not just print summaries and there is at least one metric to
         print for a function.
      */
      if (!summaries_only && print_fcn)
         print_function_summary(csv, do_halstead, volume, level, effort,
               length, vocabulary, intelligence, lang_level);

      if (function.gotos > 0)
         warn(W_GOTO, function.gotos, NUMBER(function.gotos));
      if (function.continues > 0)
         warn(W_CONTINUE, function.continues, NUMBER(function.continues));
      if (function.returns > 1)
         warn(W_RETURNS, function.returns);
   }

   /* Only apply these rules if calculating Halstead metrics. */
   if (do_halstead)
   {
      /* Before an initializer in a function? */
      if (is_within_function && stm.before_initializer)
      {
         ++function.operand_length;
         add_term(&operand_vocabulary, last_declarator());
         is_logic = TRUE;
      }

      /* Past the declarations in a compound statement? */
      if (stm.after_compound_declarations)
         is_logic = TRUE;

      /* End of an initializer or beginning of a compound block? */
      if ((is_within_function && stm.after_initializer) ||
            stm.before_compound_declarations)
         is_logic = FALSE;

      /* Count and record terms for Halstead metrics. */
      if (is_logic)
      {
         if (grm.is_identifier || grm.is_type_name ||
               grm.is_constant || grm.is_string)
         {
            ++function.operand_length;
            add_term(&operand_vocabulary, token());
         }
         else if (keyword("break") || keyword("case") ||
               keyword("continue") || keyword("default") || keyword("do") ||
               keyword("else") || keyword("for") || keyword("goto") ||
               keyword("if") || keyword("return") || keyword("sizeof") ||
               keyword("switch") || keyword("while") ||
               (grm.is_operator && !(oper("}") || oper(")") || oper("]"))))
         {
            ++function.operator_length;
            add_term(&operator_vocabulary, token());
         }
      }

      /*
         Halstead says that labels are just to be considered comments, so
         "uncount" them.
      */
      if (is_logic && stm.is_label)
      {
         /* Decrement length for both the label's identifier and colon. */
         --function.operand_length;
         --function.operator_length;

         /*
            Decrement term count. If these were the only instances of the
            label's identifier and colon, remove them from the vocabulary
            table.
         */
         if (--lookup_term(operand_vocabulary, last_identifier())->count == 0)
            remove_term(&operand_vocabulary, last_identifier());
         if (--lookup_term(operator_vocabulary, ":")->count == 0)
            remove_term(&operator_vocabulary, ":");
      }
   }

   /* At the end of each executable statement, record depth statistics. */
   if (stm.end && !stm.is_decl)
      record_stat(&function.max_depth, stm.depth);

   /* Remember if line found with mixed indention (spaces and tabs). */
   if (lin.is_mixed_indent)
      mixed_indent_lines = TRUE;

   /* Print warning if a line contains more than one statement. */
   if (lin.statements > 0)
      warn(W_MULT_STATEMENTS, lin.statements + 1);

   /*
      Count compound statements to later subtract from high-statement
      total. This is because compound statements are counted as
      high-level statements in fcn.high, but SPR code-counting rules do
      not consider them to be statements, so they have to be subtracted
      back out when reporting executable statements.
   */
   if (stm.is_comp)
      ++function.comp;

   /*
      Count (goto) labels because the SPR code-counting rules consider
      them to be executable statements.  (Curiously, Halstead considers
      labels to be merely comments.)
   */
   if (stm.is_label)
      ++function.label;

   /*
      Count empty statements, i.e., ";". According to Capers Jones, SPR
      does not consider this C anomoly to be a "statement" in its
      code-counting rules. Subtract from total later because fcn.low
      includes empty expressions.
   */
   if (stm.is_empty_expr)
      ++function.empty_expr;

   /* Count gotos, continues, and returns. */
   if (keyword("goto"))
      ++function.gotos;

   if (keyword("continue"))
      ++function.continues;

   if (keyword("return"))
      ++function.returns;

   /* Record identifier statistics at the function and module level. */
   if (grm.is_identifier) {
      unsigned str_len = strlen(this_identifier());

      record_stat(&function.idents, str_len);
      record_stat(&module.idents, str_len);
   }


   /*
      If character encountered that is not in Standard C's source
      character set, print warning.
   */
   if (lex.nonstandard)
      warn(W_NONSTANDARD, lex.nonstandard);
}

/* Tell user how to use Metre and interpret its output. */
static void print_help(void)
{
   int i;
   unsigned column;

   out("-%c          Summaries only             "
          "-%c          Halstead metrics\n",
          SUMMARY_OPT_CHAR, HALSTEAD_OPT_CHAR);
   out("-%c          Adjust function points     "
          "-%cN         Stroud number [%d]\n",
          ADJUST_FP_OPT_CHAR, STROUD_OPT_CHAR, DEFAULT_STROUD_NUMBER);
   out("-%c          Comma-separated-value      "
          "-%cxxx       Select specific metric\n",
          CSV_OPT_CHAR, SELECT_METRIC_OPT_CHAR);

   out("\nNotation and Abbreviations:\n");
   out("<           Minimum                    ");
   out("Cyclomatic  Cyclomatic Complexity\n");
   out(">           Maximum                    ");
   out("Max Depth   Max control-structure depth\n");
   out("~           Average                    ");
   out("Exec        Executable\n");
   out("(nothing)   Total                      ");
   out("Decl        Declaration\n");
   out("Lang        Language                   ");
   out("PP          Preprocessor\n");

   column = out("\nFor -%c option: ", SELECT_METRIC_OPT_CHAR);
   for (i = 0; i < DIM_OF(metric); ++i)
   {
      if (column + strlen(metric[i].name) + 1 >= OUTPUT_WIDTH)
      {
         out("\n");
         column = 0;
      }
      column += out("%s ", metric[i].name);
   }
   out("\n");
}

/* Mark which metrics to print in the metric[] array. */
static BOOLEAN mark_metrics_to_print(void)
{
   static BOOLEAN metric_selected_to_print = FALSE;
   int i, j;
   int arg_count = argc();
   char **arg_values = argv();

   /* Start off assuming that no metrics are to be printed. */
   for (j = 0; j < DIM_OF(metric); ++j)
      metric[j].print = FALSE;

   for (i = 1; i < arg_count; ++i)
      if (strchr(OPT_INTRO_CHARS, arg_values[i][0]) != NULL &&
            arg_values[i][OPTION_START] == SELECT_METRIC_OPT_CHAR)
      {
         for (j = 0; j < DIM_OF(metric); ++j)
            if (strcmp(metric[j].name,
                  &arg_values[i][OPTION_STRING_START]) == 0)
            {
               metric[j].print = TRUE;
               metric_selected_to_print = TRUE;
               break;
            }

         if (j == DIM_OF(metric))
            warn(W_BAD_METRIC, &arg_values[i][OPTION_STRING_START]);
      }

   /* If no metrics explicitly selected to print, print all metrics. */
   if (!metric_selected_to_print)
      for (j = 0; j < DIM_OF(metric); ++j)
         metric[j].print = TRUE;

   return metric_selected_to_print;
}

/* Determine whether whole sections are to be printed. */
static void mark_sections_to_print(BOOLEAN *print_fcn, BOOLEAN *print_mod_fcn,
      BOOLEAN *print_mod, BOOLEAN *print_prj_fcn, BOOLEAN *print_prj_mod,
      BOOLEAN *print_prj)
{
   int i;

   /* Function metrics. */
   *print_fcn = FALSE;
   for (i = 0; i < DIM_OF(metric); ++i)
      if (metric[i].print && metric[i].fcn)
      {
         *print_fcn = TRUE;
         break;
      }

   /* Function-related module metrics. */
   *print_mod_fcn = FALSE;
   for (i = 0; i < DIM_OF(metric); ++i)
      if (metric[i].print && metric[i].mod_fcn)
      {
         *print_mod_fcn = TRUE;
         break;
      }

   /* Module-related module metrics. */
   *print_mod = FALSE;
   for (i = 0; i < DIM_OF(metric); ++i)
      if (metric[i].print && metric[i].mod)
      {
         *print_mod = TRUE;
         break;
      }

   /* Function-related project metrics. */
   *print_prj_fcn = FALSE;
   for (i = 0; i < DIM_OF(metric); ++i)
      if (metric[i].print && metric[i].prj_fcn)
      {
         *print_prj_fcn = TRUE;
         break;
      }

   /* Module-related project metrics. */
   *print_prj_mod = FALSE;
   for (i = 0; i < DIM_OF(metric); ++i)
      if (metric[i].print && metric[i].prj_mod)
      {
         *print_prj_mod = TRUE;
         break;
      }

   /* Project-related project metrics. */
   *print_prj = FALSE;
   for (i = 0; i < DIM_OF(metric); ++i)
      if (metric[i].print && metric[i].prj)
      {
         *print_prj = TRUE;
         break;
      }
}

/* Return whether to print the indicated metric. */
static BOOLEAN print_metric(enum metric_type type)
{
   BOOLEAN print_it = TRUE;
   int i;

   for (i = 0; i < DIM_OF(metric); ++i)
      if (metric[i].type == type)
         print_it = metric[i].print;

   return print_it;
}

/* Initialize project-level statistics. */
static void init_project_stats(BOOLEAN do_halstead)
{
   init_stat(&project.function.decisions);
   init_stat(&project.function.conditions);
   if (do_halstead)
   {
      init_stat(&project.function.length);
      init_stat(&project.function.vocabulary);
      init_stat(&project.function.volume);
      init_stat(&project.function.level);
      init_stat(&project.function.effort);
      init_stat(&project.function.intelligence);
      init_stat(&project.function.lang_level);
   }
   init_stat(&project.function.lines);
   init_stat(&project.function.code_lines);
   init_stat(&project.function.comment_lines);
   init_stat(&project.function.white_lines);
   init_stat(&project.function.decls);
   init_stat(&project.function.stmnts);
   init_stat(&project.function.max_depth);
   init_stat(&project.module.lines);
   init_stat(&project.module.code_lines);
   init_stat(&project.module.comment_lines);
   init_stat(&project.module.white_lines);
   init_stat(&project.module.decls);
   init_stat(&project.module.stmnts);
   init_stat(&project.module.pp);
   init_stat(&project.module.funcs);
   init_stat(&project.idents);

   project.modules = 0;
}

/* Print project summary. */
static void print_project_summary(BOOLEAN csv,
      BOOLEAN print_prj_fcn, BOOLEAN print_prj_mod, BOOLEAN print_prj,
      BOOLEAN do_halstead, BOOLEAN adjust_function_points)
{
   /*
      Executable + declaration + preprocessor statements.  For
      function-point backfiring.
   */
   float total_statements;

   /* Backfired function points. */
   float function_points;

   out(csv ? "" : "\n****************************************************");
   out(csv ? "prj" : "\n\nProject Summary\n");

   /*
      Only print function-related stats if at least one function and
      there is at least one function-related metric to print for the
      project.
   */
   if (tot_stat(&project.module.funcs) != 0 && print_prj_fcn)
   {
      out(csv ? "" : "   -Function-\n");

      if (print_metric(cyc))
         out(csv ? ",%lu,%lu,%lu" : "   Cyclomatic:      <%lu >%lu ~%lu\n",
               min_stat(&project.function.decisions) + 1,
               max_stat(&project.function.decisions) + 1,
               avg_stat(&project.function.decisions) + 1);

      if (print_metric(exc))
         out(csv ? ",%lu,%lu,%lu" : "     Extended:      <%lu >%lu ~%lu\n",
               min_stat(&project.function.conditions) + 1,
               max_stat(&project.function.conditions) + 1,
               avg_stat(&project.function.conditions) + 1);

      if (do_halstead)
      {
         if (print_metric(pln))
            out(csv ? ",%lu,%lu,%lu" : "   Halstead Length: <%lu >%lu ~%lu\n",
                  min_stat(&project.function.length),
                  max_stat(&project.function.length),
                  avg_stat(&project.function.length));

         if (print_metric(pvc))
            out(csv ? ",%lu,%lu,%lu" : "     Vocabulary:    <%lu >%lu ~%lu\n",
                  min_stat(&project.function.vocabulary),
                  max_stat(&project.function.vocabulary),
                  avg_stat(&project.function.vocabulary));

         if (print_metric(pvl))
            out(csv ? ",%lu,%lu,%lu" : "     Volume:        <%lu >%lu ~%lu\n",
                  min_stat(&project.function.volume),
                  max_stat(&project.function.volume),
                  avg_stat(&project.function.volume));

         if (print_metric(plv))
            out(csv ? ",%.3f,%.3f,%.3f" :
                  "     Level:         <%.3f >%.3f ~%.3f\n",
                  min_stat(&project.function.level) / 1000.0,
                  max_stat(&project.function.level) / 1000.0,
                  avg_stat(&project.function.level) / 1000.0);

         if (print_metric(pef))
            out(csv ? ",%lu,%lu,%lu" : "     Effort:        <%lu >%lu ~%lu\n",
                  min_stat(&project.function.effort),
                  max_stat(&project.function.effort),
                  avg_stat(&project.function.effort));

         if (print_metric(inc))
            out(csv ? ",%lu,%lu,%lu" : "     Intelligence:  <%lu >%lu ~%lu\n",
                  min_stat(&project.function.intelligence),
                  max_stat(&project.function.intelligence),
                  avg_stat(&project.function.intelligence));

         if (print_metric(ptm))
            out(csv ? ",%.1f,%.1f,%.1f,%.1f" :
                  "     Time (hours):  <%.1f >%.1f ~%.1f %.1f\n",
                  min_stat(&project.function.effort) / moments_per_hour,
                  max_stat(&project.function.effort) / moments_per_hour,
                  avg_stat(&project.function.effort) / moments_per_hour,
                  tot_stat(&project.function.effort) / moments_per_hour);

         if (print_metric(llv))
            out(csv ? ",%.2f,%.2f,%.2f" :
                  "     Lang Level:    <%.2f >%.2f ~%.2f\n",
                  min_stat(&project.function.lang_level) / 100.0,
                  max_stat(&project.function.lang_level) / 100.0,
                  avg_stat(&project.function.lang_level) / 100.0);
      }

      if (print_metric(scl))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "   Lines:           <%lu >%lu ~%lu %lu\n",
               min_stat(&project.function.lines),
               max_stat(&project.function.lines),
               avg_stat(&project.function.lines),
               tot_stat(&project.function.lines));

      if (print_metric(cdl))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "     Code:          <%lu >%lu ~%lu %lu\n",
               min_stat(&project.function.code_lines),
               max_stat(&project.function.code_lines),
               avg_stat(&project.function.code_lines),
               tot_stat(&project.function.code_lines));

      if (print_metric(cml))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "     Comment:       <%lu >%lu ~%lu %lu\n",
               min_stat(&project.function.comment_lines),
               max_stat(&project.function.comment_lines),
               avg_stat(&project.function.comment_lines),
               tot_stat(&project.function.comment_lines));

      if (print_metric(bll))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "     Blank:         <%lu >%lu ~%lu %lu\n",
               min_stat(&project.function.white_lines),
               max_stat(&project.function.white_lines),
               avg_stat(&project.function.white_lines),
               tot_stat(&project.function.white_lines));

      if (print_metric(exs))
         out(csv ? ",%lu,%lu,%lu" : "   Exec Statements: <%lu >%lu ~%lu\n",
               min_stat(&project.function.stmnts),
               max_stat(&project.function.stmnts),
               avg_stat(&project.function.stmnts));

      if (print_metric(dcs))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "   Decl Statements: <%lu >%lu ~%lu %lu\n",
               min_stat(&project.function.decls),
               max_stat(&project.function.decls),
               avg_stat(&project.function.decls),
               tot_stat(&project.function.decls));

      if (print_metric(mcd))
         out(csv ? ",%lu,%lu,%lu" : "   Max Depth:       <%lu >%lu ~%lu\n",
               min_stat(&project.function.max_depth),
               max_stat(&project.function.max_depth),
               avg_stat(&project.function.max_depth));

      out(csv ? "" : "\n");
   }

   /*
      Only print module-related stats if there is at least one
      module-related metric to print for the project.
   */
   if (print_prj_mod)
   {
      out(csv ? "" : "   -Module-\n");

      if (print_metric(scl))
         out(csv ? ",%lu,%lu,%lu" : "   Lines:           <%lu >%lu ~%lu\n",
               min_stat(&project.module.lines),
               max_stat(&project.module.lines),
               avg_stat(&project.module.lines));

      if (print_metric(cdl))
         out(csv ? ",%lu,%lu,%lu" : "     Code:          <%lu >%lu ~%lu\n",
               min_stat(&project.module.code_lines),
               max_stat(&project.module.code_lines),
               avg_stat(&project.module.code_lines));

      if (print_metric(cml))
         out(csv ? ",%lu,%lu,%lu" : "     Comment:       <%lu >%lu ~%lu\n",
               min_stat(&project.module.comment_lines),
               max_stat(&project.module.comment_lines),
               avg_stat(&project.module.comment_lines));

      if (print_metric(bll))
         out(csv ? ",%lu,%lu,%lu" : "     Blank:         <%lu >%lu ~%lu\n",
               min_stat(&project.module.white_lines),
               max_stat(&project.module.white_lines),
               avg_stat(&project.module.white_lines));

      if (print_metric(exs))
         out(csv ? ",%lu,%lu,%lu" : "   Exec Statements: <%lu >%lu ~%lu\n",
               min_stat(&project.module.stmnts),
               max_stat(&project.module.stmnts),
               avg_stat(&project.module.stmnts));

      if (print_metric(dcs))
         out(csv ? ",%lu,%lu,%lu" : "   Decl Statements: <%lu >%lu ~%lu\n",
               min_stat(&project.module.decls),
               max_stat(&project.module.decls),
               avg_stat(&project.module.decls));

      if (print_metric(pps))
         out(csv ? ",%lu,%lu,%lu" : "   PP Statements:   <%lu >%lu ~%lu\n",
               min_stat(&project.module.pp),
               max_stat(&project.module.pp),
               avg_stat(&project.module.pp));

      if (print_metric(fnc))
         out(csv ? ",%lu,%lu,%lu" : "   Functions:       <%lu >%lu ~%lu\n",
               min_stat(&project.module.funcs),
               max_stat(&project.module.funcs),
               avg_stat(&project.module.funcs));

      out(csv ? "" : "\n");
   }

   /*
      Only print project-related stats if there is at least one
      project-related metric to print for the project.
   */
   if (print_prj)
   {
      out(csv ? "" : "   -Project-\n");

      if (print_metric(scl))
         out(csv ? ",%lu" : "   Lines:           %lu\n",
               tot_stat(&project.module.lines));

      if (print_metric(cdl))
         out(csv ? ",%lu" : "     Code:          %lu\n",
               tot_stat(&project.module.code_lines));

      if (print_metric(cml))
         out(csv ? ",%lu" : "     Comment:       %lu\n",
               tot_stat(&project.module.comment_lines));

      if (print_metric(bll))
         out(csv ? ",%lu" : "     Blank:         %lu\n",
               tot_stat(&project.module.white_lines));

      if (print_metric(exs))
         out(csv ? ",%lu" : "   Exec Statements: %lu\n",
               tot_stat(&project.function.stmnts));

      if (print_metric(dcs))
         out(csv ? ",%lu" : "   Decl Statements: %lu\n",
               tot_stat(&project.module.decls));

      if (print_metric(pps))
         out(csv ? ",%lu" : "   PP Statements:   %lu\n",
               tot_stat(&project.module.pp));

      total_statements = tot_stat(&project.function.stmnts) +
            tot_stat(&project.module.decls) +
            tot_stat(&project.module.pp);
      function_points = total_statements / STMNT_TO_FP_DIVISOR + ROUND;
      /*
         If function(s) and supposed to, adjust function points based on
         average complexity.
      */
      if (tot_stat(&project.module.funcs) != 0 && adjust_function_points)
      {
         function_points /= fp_adjustment_factor(
               avg_stat(&project.function.decisions) + 1);
      }
      if (print_metric(fnp))
         out(csv ? ",%u" : "   Function Points: %u\n",
               (unsigned)function_points);

      if (print_metric(idc))
         out(csv ? ",%lu" : "   Identifiers:     %lu\n",
               tim_stat(&project.idents));

      if (print_metric(idl))
         out(csv ? ",%lu,%lu,%lu" : "     Length:        <%lu >%lu ~%lu\n",
               tim_stat(&project.idents) == 0 ? 0 : min_stat(&project.idents),
               tim_stat(&project.idents) == 0 ? 0 : max_stat(&project.idents),
               tim_stat(&project.idents) == 0 ? 0 : avg_stat(&project.idents));

      if (print_metric(fnc))
         out(csv ? ",%lu" : "   Functions:       %lu\n",
               tot_stat(&project.module.funcs));

      if (print_metric(mdc))
         out(csv ? ",%lu" : "   Modules:         %lu\n", project.modules);
   }

   out(csv ? "\n" : "");
}

/* Update project-level statistics. */
static void update_project_stats(BOOLEAN do_halstead)
{
   absorb_stat(&project.function.decisions, &module.function.decisions);
   absorb_stat(&project.function.conditions, &module.function.conditions);

   if (do_halstead)
   {
      absorb_stat(&project.function.length, &module.function.length);
      absorb_stat(&project.function.vocabulary, &module.function.vocabulary);
      absorb_stat(&project.function.volume, &module.function.volume);
      absorb_stat(&project.function.level, &module.function.level);
      absorb_stat(&project.function.effort, &module.function.effort);
      absorb_stat(&project.function.intelligence,
            &module.function.intelligence);
      absorb_stat(&project.function.lang_level, &module.function.lang_level);
   }

   absorb_stat(&project.function.lines, &module.function.lines);
   absorb_stat(&project.function.code_lines, &module.function.code_lines);
   absorb_stat(&project.function.comment_lines, &module.function.comment_lines);
   absorb_stat(&project.function.white_lines, &module.function.white_lines);

   record_stat(&project.module.lines, mod.lines.total);
   record_stat(&project.module.code_lines, mod.lines.code);
   record_stat(&project.module.comment_lines, mod.lines.com);
   record_stat(&project.module.white_lines, mod.lines.white);

   absorb_stat(&project.idents, &module.idents);

   absorb_stat(&project.function.decls, &module.function.decls);
   absorb_stat(&project.function.stmnts, &module.function.stmnts);
   absorb_stat(&project.function.max_depth, &module.function.max_depth);

   record_stat(&project.module.decls, mod.decl);
   record_stat(&project.module.stmnts, tot_stat(&module.function.stmnts));

   record_stat(&project.module.pp, mod.pp);
   record_stat(&project.module.funcs, mod.functions);

   ++project.modules;
}

/* Initialize module-level statistics. */
static void init_module_stats(BOOLEAN do_halstead)
{
   init_stat(&module.function.decisions);
   init_stat(&module.function.conditions);
   if (do_halstead)
   {
      init_stat(&module.function.length);
      init_stat(&module.function.vocabulary);
      init_stat(&module.function.volume);
      init_stat(&module.function.level);
      init_stat(&module.function.effort);
      init_stat(&module.function.intelligence);
      init_stat(&module.function.lang_level);
   }
   init_stat(&module.function.lines);
   init_stat(&module.function.code_lines);
   init_stat(&module.function.comment_lines);
   init_stat(&module.function.white_lines);
   init_stat(&module.function.decls);
   init_stat(&module.function.stmnts);
   init_stat(&module.function.max_depth);
   init_stat(&module.idents);

   module.lines = 0;
   module.code_lines = 0;
   module.comment_lines = 0;
   module.white_lines = 0;
}

/* Print module summary. */
static void print_module_summary(BOOLEAN csv, BOOLEAN print_mod,
      BOOLEAN print_mod_fcn, BOOLEAN do_halstead,
      BOOLEAN adjust_function_points)
{
   /*
      Executable + declaration + preprocessor statements.  For
      function-point backfiring.
   */
   float total_statements;

   /* Backfired function points. */
   float function_points;

   out(csv ? "mod,%s" : "\n\n   Module Summary\n", mod_name());

   /*
      Only print function-related stats if at least one function and
      there is at least one function-related metric to print for a
      module.
   */
   if (mod.functions != 0 && print_mod_fcn)
   {
      out(csv ? "" : "      -Function-\n");

      if (print_metric(cyc))
         out(csv ? ",%lu,%lu,%lu" : "      Cyclomatic:      <%lu >%lu ~%lu\n",
               min_stat(&module.function.decisions) + 1,
               max_stat(&module.function.decisions) + 1,
               avg_stat(&module.function.decisions) + 1);

      if (print_metric(exc))
         out(csv ? ",%lu,%lu,%lu" : "        Extended:      <%lu >%lu ~%lu\n",
               min_stat(&module.function.conditions) + 1,
               max_stat(&module.function.conditions) + 1,
               avg_stat(&module.function.conditions) + 1);

      if (do_halstead)
      {
         if (print_metric(pln))
            out(csv ? ",%lu,%lu,%lu" :
                  "      Halstead Length: <%lu >%lu ~%lu\n",
                  min_stat(&module.function.length),
                  max_stat(&module.function.length),
                  avg_stat(&module.function.length));

         if (print_metric(pvc))
            out(csv ? ",%lu,%lu,%lu" :
                  "        Vocabulary:    <%lu >%lu ~%lu\n",
                  min_stat(&module.function.vocabulary),
                  max_stat(&module.function.vocabulary),
                  avg_stat(&module.function.vocabulary));

         if (print_metric(pvl))
            out(csv ? ",%lu,%lu,%lu" :
                  "        Volume:        <%lu >%lu ~%lu\n",
                  min_stat(&module.function.volume),
                  max_stat(&module.function.volume),
                  avg_stat(&module.function.volume));

         if (print_metric(plv))
            out(csv ? ",%.3f,%.3f,%.3f" :
                  "        Level:         <%.3f >%.3f ~%.3f\n",
                  min_stat(&module.function.level) / 1000.0,
                  max_stat(&module.function.level) / 1000.0,
                  avg_stat(&module.function.level) / 1000.0);

         if (print_metric(pef))
            out(csv ? ",%lu,%lu,%lu" :
                  "        Effort:        <%lu >%lu ~%lu\n",
                  min_stat(&module.function.effort),
                  max_stat(&module.function.effort),
                  avg_stat(&module.function.effort));

         if (print_metric(inc))
            out(csv ? ",%lu,%lu,%lu" :
                  "        Intelligence:  <%lu >%lu ~%lu\n",
                  min_stat(&module.function.intelligence),
                  max_stat(&module.function.intelligence),
                  avg_stat(&module.function.intelligence));

         if (print_metric(ptm))
            out(csv ? ",%.1f,%.1f,%.1f,%.1f" :
                  "        Time (hours):  <%.1f >%.1f ~%.1f %.1f\n",
                  min_stat(&module.function.effort) / moments_per_hour,
                  max_stat(&module.function.effort) / moments_per_hour,
                  avg_stat(&module.function.effort) / moments_per_hour,
                  tot_stat(&module.function.effort) / moments_per_hour);

         if (print_metric(llv))
            out(csv ? ",%.2f,%.2f,%.2f" :
                  "        Lang Level:    <%.2f >%.2f ~%.2f\n",
                  min_stat(&module.function.lang_level) / 100.0,
                  max_stat(&module.function.lang_level) / 100.0,
                  avg_stat(&module.function.lang_level) / 100.0);
      }

      if (print_metric(scl))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "      Lines:           <%lu >%lu ~%lu %lu\n",
               min_stat(&module.function.lines),
               max_stat(&module.function.lines),
               avg_stat(&module.function.lines),
               tot_stat(&module.function.lines));

      if (print_metric(cdl))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "        Code:          <%lu >%lu ~%lu %lu\n",
               min_stat(&module.function.code_lines),
               max_stat(&module.function.code_lines),
               avg_stat(&module.function.code_lines),
               tot_stat(&module.function.code_lines));

      if (print_metric(cml))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "        Comment:       <%lu >%lu ~%lu %lu\n",
               min_stat(&module.function.comment_lines),
               max_stat(&module.function.comment_lines),
               avg_stat(&module.function.comment_lines),
               tot_stat(&module.function.comment_lines));

      if (print_metric(bll))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "        Blank:         <%lu >%lu ~%lu %lu\n",
               min_stat(&module.function.white_lines),
               max_stat(&module.function.white_lines),
               avg_stat(&module.function.white_lines),
               tot_stat(&module.function.white_lines));

      if (print_metric(exs))
         out(csv ? ",%lu,%lu,%lu" : "      Exec Statements: <%lu >%lu ~%lu\n",
               min_stat(&module.function.stmnts),
               max_stat(&module.function.stmnts),
               avg_stat(&module.function.stmnts));

      if (print_metric(dcs))
         out(csv ? ",%lu,%lu,%lu,%lu" :
               "      Decl Statements: <%lu >%lu ~%lu %lu\n",
               min_stat(&module.function.decls),
               max_stat(&module.function.decls),
               avg_stat(&module.function.decls),
               tot_stat(&module.function.decls));

      if (print_metric(mcd))
         out(csv ? ",%lu,%lu,%lu" : "      Max Depth:       <%lu >%lu ~%lu\n",
               min_stat(&module.function.max_depth),
               max_stat(&module.function.max_depth),
               avg_stat(&module.function.max_depth));

      out(csv ? "" : "\n");
   }

   /*
      Only print module-related stats if there is at least one
      module-related metric to print for this module.
   */
   if (print_mod)
   {
      out(csv ? "" : "      -Module-\n");

      if (print_metric(scl))
         out(csv ? ",%u" : "      Lines:           %u\n", mod.lines.total);

      if (print_metric(cdl))
         out(csv ? ",%u" : "        Code:          %u\n", mod.lines.code);

      if (print_metric(cml))
         out(csv ? ",%u" : "        Comment:       %u\n", mod.lines.com);

      if (print_metric(bll))
         out(csv ? ",%u" : "        Blank:         %u\n", mod.lines.white);

      if (print_metric(exs))
         out(csv ? ",%lu" : "      Exec Statements: %lu\n",
               tot_stat(&module.function.stmnts));

      if (print_metric(dcs))
         out(csv ? ",%u" : "      Decl Statements: %u\n", mod.decl);

      if (print_metric(pps))
         out(csv ? ",%u" : "      PP Statements:   %u\n", mod.pp);

      total_statements = tot_stat(&module.function.stmnts) + mod.decl + mod.pp;
      function_points = total_statements / STMNT_TO_FP_DIVISOR + ROUND;
      /*
         If function(s) and supposed to, adjust function points based on
         average complexity.
      */
      if (mod.functions != 0 && adjust_function_points)
      {
         function_points /= fp_adjustment_factor(
               avg_stat(&module.function.decisions) + 1);
      }
      if (print_metric(fnp))
         out(csv ? ",%u" : "      Function Points: %u\n",
               (unsigned)function_points);

      if (print_metric(idc))
         out(csv ? ",%lu" : "      Identifiers:     %lu\n",
               tim_stat(&module.idents));

      if (print_metric(idl))
         out(csv ? ",%lu,%lu,%lu" : "        Length:        <%lu >%lu ~%lu\n",
               tim_stat(&module.idents) == 0 ? 0 : min_stat(&module.idents),
               tim_stat(&module.idents) == 0 ? 0 : max_stat(&module.idents),
               tim_stat(&module.idents) == 0 ? 0 : avg_stat(&module.idents));

      if (print_metric(fnc))
         out(csv ? ",%u" : "      Functions:       %u\n", mod.functions);
   }

   out(csv ? "\n" : "");
}

/* Update module-level statistics. */
static void update_module_stats(BOOLEAN do_halstead,
      unsigned volume, float level, long unsigned effort,
      long unsigned length, unsigned vocabulary, unsigned intelligence,
      float lang_level)
{
   record_stat(&module.function.decisions, fcn.decisions);
   record_stat(&module.function.conditions, fcn.conditions);

   if (do_halstead)
   {
      record_stat(&module.function.length, length);
      record_stat(&module.function.vocabulary, vocabulary);
      record_stat(&module.function.volume, volume);
      /*
         In order to maintain precision to the third decimal place while
         holding in an unsigned long, multiply by 1000. Will be divided
         by 1000 when extracted.
      */
      record_stat(&module.function.level,
            (unsigned long)(level * 1000.0 + ROUND));
      record_stat(&module.function.effort, effort);
      record_stat(&module.function.intelligence, intelligence);
      record_stat(&module.function.lang_level,
            (unsigned long)(lang_level * 100.0 + ROUND));
   }

   record_stat(&module.function.lines, fcn.lines.total);
   record_stat(&module.function.code_lines, fcn.lines.code);
   record_stat(&module.function.comment_lines, fcn.lines.com);
   record_stat(&module.function.white_lines, fcn.lines.white);

   record_stat(&module.function.max_depth, max_stat(&function.max_depth));

   record_stat(&module.function.decls, fcn.decl);
   record_stat(&module.function.stmnts, executable_function_statements());
}

/* Initialize function-level statistics. */
static void init_function_stats(void)
{
   init_stat(&function.idents);
   init_stat(&function.max_depth);

   function.comp = 0;
   function.empty_expr = 0;
   function.label = 0;
   function.gotos = 0;
   function.continues = 0;
   function.returns = 0;
   function.operand_length = 0;
   function.operator_length = 0;
}

/* Print function summary. */
static void print_function_summary(BOOLEAN csv, BOOLEAN do_halstead,
      unsigned volume, float level, long unsigned effort,
      long unsigned length, unsigned vocabulary, unsigned intelligence,
      float lang_level)
{
   if (csv)
      out("fcn,%s,%s", mod_name(), fcn_name());
   else
      out("\n   %s()\n", fcn_name());

   if (print_metric(cyc))
      out(csv ? ",%u" :
            "      Cyclomatic:      %u\n", fcn.decisions + 1);

   if (print_metric(exc))
      out(csv ? ",%u" :
            "        Extended:      %u\n", fcn.conditions + 1);

   if (do_halstead)
   {
      if (print_metric(pln))
         out(csv ? ",%lu" :
               "      Halstead Length: %lu\n", length);

      if (print_metric(pvc))
         out(csv ? ",%u" :
               "        Vocabulary:    %u\n", vocabulary);

      if (print_metric(pvl))
         out(csv ? ",%u" :
               "        Volume:        %u\n", volume);

      if (print_metric(plv))
         out(csv ? ",%.3f" :
               "        Level:         %.3f\n", level);

      if (print_metric(pef))
         out(csv ? ",%lu" :
               "        Effort:        %lu\n", effort);

      if (print_metric(inc))
         out(csv ? ",%u" :
               "        Intelligence:  %u\n", intelligence);

      if (print_metric(ptm))
         out(csv ? ",%.1f" :
               "        Time (hours):  %.1f\n",
               effort / moments_per_hour);

      if (print_metric(llv))
         out(csv ? ",%.2f" :
               "        Lang Level:    %.2f\n", lang_level);
   }

   if (print_metric(scl))
      out(csv ? ",%u" :
            "      Lines:           %u\n", fcn.lines.total);

   if (print_metric(cdl))
      out(csv ? ",%u" :
            "        Code:          %u\n", fcn.lines.code);

   if (print_metric(cml))
      out(csv ? ",%u" :
            "        Comment:       %u\n", fcn.lines.com);

   if (print_metric(bll))
      out(csv ? ",%u" :
            "        Blank:         %u\n", fcn.lines.white);

   if (print_metric(exs))
      out(csv ? ",%u" :
            "      Exec Statements: %u\n",
            executable_function_statements());

   if (print_metric(dcs))
      out(csv ? ",%u" :
            "      Decl Statements: %u\n", fcn.decl);

   if (print_metric(mcd))
      out(csv ? ",%lu" :
            "      Max Depth:       %lu\n", max_stat(&function.max_depth));

   if (print_metric(idc))
      out(csv ? ",%lu" :
            "      Identifiers:     %lu\n", tim_stat(&function.idents));

   if (print_metric(idl))
      out(csv ? ",%lu,%lu,%lu" :
            "        Length:        <%lu >%lu ~%lu\n",
            tim_stat(&function.idents) == 0 ? 0 : min_stat(&function.idents),
            tim_stat(&function.idents) == 0 ? 0 : max_stat(&function.idents),
            tim_stat(&function.idents) == 0 ? 0 : avg_stat(&function.idents));

   out(csv ? "\n" : "");
}

/*
   Return factor that adjusts an initial function-point estimate based
   on the average functional complexity of a program. This results in a
   more accurate assessment of backfired function points. Divide the
   initial function-point estimate by the returned factor to arrive at
   the adjusted function-point estimate. Cyclomatic complexity is passed
   into this function.

   This function uses a table that is similar to the one in Capers Jones
   book, _Applied Software Measurement_. Capers' table bases the
   adjustment on the sum of these three subjective bounded complexity
   metrics: program, data, and code complexity. The table in this
   function bases the adjustment on McCabe's objective unbounded
   cyclomatic complexity metric. I adapted ideas from Capers' book to
   come up with this table but have no idea how well founded it is.
*/
static float fp_adjustment_factor(unsigned long cyclo)
{
   static struct {
      unsigned long min;
      float factor;
   } table[] = {
      { 0,  0.70 },     /* 1 */
      { 1,  0.80 },     /* 2 */
      { 2,  0.85 },     /* 3 */
      { 3,  0.90 },     /* 4-5 */
      { 5,  0.95 },     /* 6-8 */
      { 8,  1.00 },     /* 9-11 */
      { 11, 1.05 },     /* 12-16 */
      { 16, 1.10 },     /* 17-21 */
      { 21, 1.15 },     /* 22-27 */
      { 27, 1.20 },     /* 28-36 */
      { 36, 1.25 },     /* 37-53 */
      { 53, 1.30 },     /* 54- */
   };
   unsigned i;

   if (cyclo == 0)
      fatal(F_BAD_CYCLO);

   for (i = 0; i < DIM_OF(table) && cyclo > table[i].min; ++i)
      ;

   return table[i - 1].factor;
}

/* Return the total number of executable statements in the current function. */
static unsigned executable_function_statements(void)
{
   /*
      Subtract out compound statements from the total in fcn.high, then
      add 1 back in for the outer-most compound statement that contains
      the function itself.
   */
   return (fcn.low - function.empty_expr) + (fcn.high - function.comp + 1) +
         function.label;
}

/* Common printf-like output function. */
static int out(const char *format, ...)
{
   va_list args;
   int chars_written;

   va_start(args, format);
   chars_written = vfprintf(out_fp, format, args);
   va_end(args);

   return chars_written;
}


/*
   The following *_stat() functions are accessor functions for STAT
   structures.
*/

/* Initialize statistic. */
static void init_stat(STAT *stat)
{
   stat->times = 0;
   stat->total = 0;
   TYPE_MAX(stat->min);
   stat->max = 0;
}

/* Record a value for this statistic. */
static void record_stat(STAT *stat, unsigned long value)
{
   if (value > stat->max)
         stat->max = value;

   if (value < stat->min)
         stat->min = value;

   ++stat->times;

   stat->total += value;
}

/* Absorb source statistic into destination statistic. */
static void absorb_stat(STAT *stat_to, STAT *stat_from)
{
   if (stat_from->max > stat_to->max)
         stat_to->max = stat_from->max;

   if (stat_from->min < stat_to->min)
         stat_to->min = stat_from->min;

   stat_to->times += stat_from->times;

   stat_to->total += stat_from->total;
}

/* Return minimum value ever recorded for this statistic. */
static unsigned long min_stat(STAT *stat)
{
   unsigned long min;

   if (stat->times == 0)
      min = 0;
   else
      min = stat->min;

   return min;
}

/* Return maximum value ever recorded for this statistic. */
static unsigned long max_stat(STAT *stat)
{
   return stat->max;
}

/* Return sum of all values ever recorded for this statistic. */
static unsigned long tot_stat(STAT *stat)
{
   return stat->total;
}

/* Return number of times a value was recorded for this statistic. */
static unsigned long tim_stat(STAT *stat)
{
   return stat->times;
}

/*
   Return the average value of this statistic, rounding to the nearest
   integer.
*/
static unsigned long avg_stat(STAT *stat)
{
   unsigned long avg;

   if (stat->times == 0)
      avg = 0;
   else
      avg = (unsigned long)((float)tot_stat(stat) / stat->times + ROUND);

   return avg;
}


/*
   The following functions (clear_vocab(), add_term(), lookup_term(),
   and remove_term()) manipulate the linked list whos head is pointed to
   by the vocabulary. These functions are used for the Halstead metrics.
*/

/* Remove all vocabulary-table entries and return the number of entries. */
static unsigned clear_vocab(TERM **vocabulary)
{
   TERM *term = *vocabulary;
   unsigned entries = 0;

   while (term != NULL)
   {
      TERM *next_term;

      free(term->string);
      next_term = term->next;
      free(term);
      term = next_term;
      ++entries;
   }

   *vocabulary = NULL;

   return entries;
}

/* Add string to vocabulary table. */
static void add_term(TERM **vocabulary, char *string)
{
   TERM *term;

   /* If term not found, add a copy of it to the front of the vocabulary. */
   if ((term = lookup_term(*vocabulary, string)) == NULL)
   {
      term = (TERM *)malloc(sizeof *term);
      if (term == NULL)
         fatal(F_MEMORY);
      term->next = *vocabulary;
      *vocabulary = term;
      term->string = (char *)malloc(strlen(string) + 1);
      if (term->string == NULL)
         fatal(F_MEMORY);
      strcpy(term->string, string);
      term->count = 0;
   }

   ++term->count;
}

/*
   Lookup term in vocabulary table. Return pointer to TERM entry or NULL
   if not found.
*/
static TERM *lookup_term(TERM *vocabulary, char *string)
{
   TERM *term;

   /* Search through vocabulary for this term. */
   for (term = vocabulary; term != NULL && strcmp(term->string, string) != 0;
         term = term->next)
      ;

   return term;
}

/* Remove term from vocabulary table. */
static void remove_term(TERM **vocabulary, char *string)
{
   TERM *term, *prev_term;

   /* Search through vocabulary for this term. */
   for (prev_term = NULL, term = *vocabulary;
         term != NULL && strcmp(term->string, string) != 0;
         prev_term = term, term = term->next)
      ;

   if (term != NULL)
   {
      /* If prev_term is NULL, this is the first entry in the linked list. */
      if (prev_term == NULL)
         *vocabulary = term->next;
      else
         prev_term->next = term->next;
      free(term->string);
      free(term);
   }
   /* (Else if term was not found, do not do anything. */
}
