/*************************************************************
   Copyright (c) 1993-1995 by Paul Long  All rights reserved.
**************************************************************/

/*************************************************************
   trerules.c  This source file contains a set of rules for
               the MTREE call-hierarchy tool.
**************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "metre.h"


/* Manifest constants. */

/* Function from which to build call tree if not main or first function. */
#define ROOT_FUNC_OPT_CHAR    'r'
/* Number of columns to indent for each level. */
#define INDENTION_OPT_CHAR    'c'
/* Use alternate graphic characters for tree lines. */
#define ALT_GRAPH_OPT_CHAR    'g'
/* Don't print tree lines. */
#define NO_GRAPH_OPT_CHAR     'n'
/* Print table of contents? */
#define TOC_OPT_CHAR          't'
/* Ignore library calls? */
#define IGNORE_LIB_OPT_CHAR   'l'
/* Width allotted to file-name section in left margin. */
#define FILE_WIDTH_OPT_CHAR   'm'

/* Number of columns to indent if not specified or in error. */
#define DEFAULT_INDENTION  4
#define DEFAULT_FILE_WIDTH 12

#define GREATEST_STRING "~"

#define TOC_INDENT               4
#define DOT_SPACES_RIGHT_MARGIN  70

/* Defines for diagnostics that are generated by the rules, not the parser. */
#define F_MEMORY           0, "Out of memory"
#define F_INTERNAL         1, "Internal error"
#define W_DUP_FUNC         0, "Duplicate function definition for %s ignored"
#define W_NO_FUNCS         1, "No functions from which to build call tree"
#define W_FUNC_NOT_FOUND   2, "%s not found; using %s"
#define W_INDENTION        3, "Invalid indention specified; using %d"

#define DIM_OF(x)          (sizeof (x) / sizeof (x[0]))

/* Characters used for tree lines. */
#define PRI_T_CHAR    '+'     /* ASCII characters. */
#define PRI_HORZ_CHAR '-'
#define PRI_VERT_CHAR '|'
#define PRI_TURN_CHAR '`'
#define ALT_T_CHAR    '\303'  /* IBM graphic characters. */
#define ALT_HORZ_CHAR '\304'
#define ALT_VERT_CHAR '\263'
#define ALT_TURN_CHAR '\300'

/* Whether call location is recorded. (Location currently not used.) */
#define CALL_LOCATION   FALSE


/* Structure definitions. */

/*
   Represents a function call in the list of calls attached to each
   function structure.
*/
typedef struct call {
   struct call *next;
   char *name;
#if CALL_LOCATION
   char *file;
   unsigned line;
#endif
} CALL;

/* Represents a function definition. */
typedef struct function {
   struct function *next;
   char *name;
   char *file;
   unsigned line;
   CALL *calls;               /* List of functions that are called from
                                 within this function. Only one entry
                                 per function regardless of how many
                                 times it is called. */
   BOOLEAN visited;           /* Has this function definition been
                                 visited? Used to detect secondary
                                 roots after the main tree has been
                                 printed. */
} FUNCTION;

/*
   Represents a stack frame in the call hierarchy. Used to determine
   recursive calls.
*/
typedef struct frame {
   struct frame *next;
   FUNCTION *function;
   BOOLEAN last_call;         /* Is the call that this represents the
                                 last entry in the calling function's
                                 call list? Used for drawing tree
                                 diagram.
                              */
} FRAME;


/* Function prototypes for local functions. */

static void check_options(void);
static void print_trees(void);
static void consider_function(void);
static void add_function(void);
static void consider_call(void);
static FUNCTION *get_root(void);
static void print_calls(FUNCTION *);
static void print_call_line(FUNCTION *);
static void print_tree_line(FRAME *);
static void push_call(FUNCTION *);
static BOOLEAN is_last_call(FRAME *, char *);
static void pop_call(void);
static BOOLEAN recursive_call(FUNCTION *);
static void print_toc(void);
static void print_toc_dots(char *);
static int cmpstr(const char *, const char * const *);
static BOOLEAN library_call(char *);
static void print_indent(int, int);
static FUNCTION *lookup_function(FUNCTION *, char *);


/* Local data definitions. */

/* Head of linked list of function definitions. */
static FUNCTION *functions = NULL;

/*
   Pointer to the structure of the function currently being parsed
   which is always at the end of the functions linked list.
*/
static FUNCTION *current_function = NULL;

/* Head of call stack; used for detecting recursion. */
static FRAME *stack = NULL;

/* Number of columns to indent in tree for each call level. */
static unsigned indention;

/* "Graphical" characters to use when drawing call tree. */
static char t_char, horz_char, vert_char, turn_char;

/* Whether to ignore calls to Standard C library functions. */
static BOOLEAN ignore_library_calls;

/*
   Width of tree left margin within which is printed the name of the
   file within which each function is defined. 
*/
static unsigned file_width;

/*
   Line on which the last declaration occured. Used for determining on
   what line a function is defined.
*/
static unsigned last_decl_lineno = 0;

/* Has function definition already been encountered with this name? */
static BOOLEAN duplicate_function = FALSE;


/* Function definitions. */

/* Set of rules that define the behavior of this call-hierarchy tool. */
void rules(void)
{
   /* Whether we are within a function definition. */
   static BOOLEAN is_within_function = FALSE;

   if (prj.need_cmd_name)
      cmd_name("MTREE");

   /* Addendum to main help info when -h is specified. */
   if (prj.help)
   {
      fprintf(out_fp, "-%cxxx       Root function name         "
             "-%cN         Indent columns per level [%d]\n",
            ROOT_FUNC_OPT_CHAR, INDENTION_OPT_CHAR, DEFAULT_INDENTION);
      fprintf(out_fp, "-%c          Alternate-character tree   "
             "-%c          Print table of contents\n",
            ALT_GRAPH_OPT_CHAR, TOC_OPT_CHAR);
      fprintf(out_fp, "-%c          No tree                    "
             "-%c          Ignore library calls\n",
            NO_GRAPH_OPT_CHAR, IGNORE_LIB_OPT_CHAR);
      fprintf(out_fp, "-%cN         File name margin [%d]\n",
            FILE_WIDTH_OPT_CHAR, DEFAULT_FILE_WIDTH);
   }

   /* Project initialization. Check starting command-line options. */
   if (prj.begin)
      check_options();

   /* Generate output. */
   if (prj.end)
      if (functions == NULL)
         warn(W_NO_FUNCS);
      else
      {
         /* Print primary and secondary call trees. */
         print_trees();

         /* Optionally print table of contents. */
         if (option(TOC_OPT_CHAR))
            print_toc();
      }

   /* Function definition. */
   if (fcn.begin) {
      consider_function();

      is_within_function = TRUE;
   }

   if (fcn.end)
      is_within_function = FALSE;

   /*
      Is this what appears to be a function call (not simply a macro
      invokation outside a function definition) that is not within a
      duplicate function definition? If so, if this is a library call,
      am I supposed to ignore it?
   */
   if (grm.is_call && is_within_function && !duplicate_function &&
         !(ignore_library_calls && library_call(last_identifier())))
      consider_call();

   /*
      Remember line on which every base-level declarator is declared in
      case we learn later that it is a function definition.
   */
   if (grm.is_declarator)
      last_decl_lineno = lineno();
}

/* Check starting command-line options. */
static void check_options(void)
{
   char *indention_str;
   char *file_width_str;

   /*
      Get width of left margin within which file names are printed.
      Zero means do not print file names at all.
   */
   if ((file_width_str = str_option(FILE_WIDTH_OPT_CHAR)) == NULL)
      file_width = DEFAULT_FILE_WIDTH;
   else
      file_width = atoi(file_width_str);

   ignore_library_calls = option(IGNORE_LIB_OPT_CHAR);

   /*
      Get how many columns to indent in the tree diagram for each
      call level.
   */
   if ((indention_str = str_option(INDENTION_OPT_CHAR)) == NULL)
      indention = DEFAULT_INDENTION;
   else if ((indention = atoi(indention_str)) < 1)
      warn(W_INDENTION, indention = DEFAULT_INDENTION);

   /* Determine which set of "line" characters to use for the call tree. */
   if (option(NO_GRAPH_OPT_CHAR))
      /* No tree */
      t_char = horz_char = vert_char = turn_char = ' ';
   else if (option(ALT_GRAPH_OPT_CHAR))
   {
      /* Use alternate character set (IBM graphic characters). */
      t_char      = ALT_T_CHAR;
      horz_char   = ALT_HORZ_CHAR;
      vert_char   = ALT_VERT_CHAR;
      turn_char   = ALT_TURN_CHAR;
   }
   else
   {
      /* Use ASCII characters to simulate lines. */
      t_char      = PRI_T_CHAR;
      horz_char   = PRI_HORZ_CHAR;
      vert_char   = PRI_VERT_CHAR;
      turn_char   = PRI_TURN_CHAR;
   }
}

/* Print call tree for primary and secondary roots. */
static void print_trees(void)
{
   FUNCTION *function;

   /* Print the primary, possibly only, call tree. */
   fprintf(out_fp, "\nCall Hierarchies\n");
   print_calls(get_root());
   fprintf(out_fp, "\n");

   /*
      For each function that was not part of primary tree, print its
      tree. This is intended for libraries without a single root.
   */
   for (function = functions; function != NULL; function = function->next)
      if (!function->visited) {
         print_calls(function);
         fprintf(out_fp, "\n");
      }
}

/* If not a duplicate function definition, add it to the functions list. */
static void consider_function(void)
{
   /*
      Search for a function that has the same name as this one.
      Remember so can ignore calls within this function.
   */
   duplicate_function = lookup_function(functions, fcn_name()) != NULL;

   /* If duplicate definition, print warning and ignore. */
   if (duplicate_function)
      warn(W_DUP_FUNC, fcn_name());
   else
      add_function();
}

/* Add current function to functions list. */
static void add_function(void)
{
   FUNCTION *function;

   if ((function = (FUNCTION *)malloc(sizeof *function)) == NULL)
      fatal(F_MEMORY);

   /* Add this function to the list of functions. */
   if (functions == NULL)
      functions = function;
   else
      current_function->next = function;
   current_function = function;
   current_function->next = NULL;

   /*
      Attach buffers containing function name and file name to function
      structure.
   */
   current_function->name = (char *)malloc(strlen(fcn_name()) + 1);
   if (current_function->name == NULL)
      fatal(F_MEMORY);

   strcpy(current_function->name, fcn_name());

   current_function->file = (char *)malloc(strlen(mod_name()) + 1);
   if (current_function->file == NULL)
      fatal(F_MEMORY);

   strcpy(current_function->file, mod_name());

   /*
      Line on which this function is defined (line on which identifier
      appears).
   */
   current_function->line = last_decl_lineno;

   current_function->calls = NULL;
   current_function->visited = FALSE;
}

/* If not a duplicate call from within the current function, add it to list. */
static void consider_call(void)
{
   CALL *temp_call, *prev_call;

   assert(current_function != NULL);
   assert(last_identifier() != NULL);

   /* Search for a call to the same function as this call. */
   for (temp_call = current_function->calls, prev_call = NULL;
         temp_call != NULL && strcmp(temp_call->name, last_identifier()) != 0;
         prev_call = temp_call, temp_call = temp_call->next)
      ;

   /* Ignore if have already seen a call to this function. */
   if (temp_call == NULL)
   {
      CALL *call;

      if ((call = (CALL *)malloc(sizeof *call)) == NULL)
         fatal(F_MEMORY);

      /*
         Add this call to the call list for the current function
         definition.
      */
      if (prev_call == NULL)
         current_function->calls = call;
      else
         prev_call->next = call;
      call->next = NULL;

      /*
         Attach buffers containing function name and file name to call
         structure.
      */
      if ((call->name = (char *)malloc(strlen(last_identifier()) + 1)) == NULL)
         fatal(F_MEMORY);

      strcpy(call->name, last_identifier());

#if CALL_LOCATION
      if ((call->file = (char *)malloc(strlen(mod_name()) + 1)) == NULL)
         fatal(F_MEMORY);

      strcpy(call->file, mod_name());

      /* Line on which call is made. */
      call->line = lineno();
#endif
   }
}

/*
   Return pointer to root of primary call tree. Order of which function
   name to use for root: -rfunc, "main", then first function.
*/
static FUNCTION *get_root(void)
{
   char *root_name;
   FUNCTION *function;

   /* If root function name not specified on command line, try "main". */
   if ((root_name = str_option(ROOT_FUNC_OPT_CHAR)) == NULL)
      function = lookup_function(functions, root_name = "main");
   else
      /* If root from cmd line does not exist, print warning and try "main". */
      if ((function = lookup_function(functions, root_name)) == NULL)
      {
         warn(W_FUNC_NOT_FOUND, root_name, "main");
         function = lookup_function(functions, root_name = "main");
      }

   /* If neither root from cmd line or "main" exists, use first function. */
   if (function == NULL)
   {
      warn(W_FUNC_NOT_FOUND, root_name, functions->name);
      function = functions;
   }

   return function;
}

/* Print call tree (or subtree) whose root is pointed to by argument. */
static void print_calls(FUNCTION *calling)
{
   CALL *call;

   /*
      Push this function on call stack in order to be able to detect
      recursion.
   */
   push_call(calling);

   /* Print the part of the call tree for this function. */
   print_call_line(calling);

   /*
      Mark this function definition as being visited so that call trees
      can be subsequently printed for unvisited functions.
   */
   calling->visited = TRUE;

   /*
      Recursively print subtree for each function that is called from
      within this function.
   */
   for (call = calling->calls; call != NULL; call = call->next)
   {
      /*
         Dummy function structure with no calls for undefined or
         recursive functions.
      */
      static FUNCTION function = { NULL, NULL, NULL };
      FUNCTION *called;

      /* If function not defined, just print function name (no calls). */
      if ((called = lookup_function(functions, call->name)) == NULL)
      {
         function.name = call->name;
         called = &function;
         print_calls(called);
      }
      /* If recursive function call, just print function name with note. */
      else if (recursive_call(called))
      {
         function.name = call->name;
         called = &function;
         print_calls(called);
         fprintf(out_fp, " (recursive)");
      }
      else
         /* Normal case. Recursively print function with any calls. */
         print_calls(called);
   }

   /* Pop this function from the call stack. */
   pop_call();
}

/*
   Print the part of the call tree for this function--a single line.

   In left margin, print file name within which this function is
   defined. Right justify so that if name is too big for the margin,
   truncation occurs on the left so that some path information may be
   lost but the file name is still visible. If no margin at all for
   printing file name, also do not print spaces between it and the
   horizontal line made up from characters from the chosen graphical
   character set. The function name is printed at the end of the line.
*/
static void print_call_line(FUNCTION *calling)
{
   unsigned file_len;

   fprintf(out_fp, "\n%*s", file_width, calling->file == NULL ? "" :
         &calling->file[file_width < (file_len = strlen(calling->file)) ?
         file_len - file_width : 0]);

   if (file_width > 0)
      fprintf(out_fp, "  ");

   print_tree_line(stack);

   fprintf(out_fp, "%s", calling->name);
}

/*
   Print the horizontal line for the function at the top of the call
   stack. Together, the horizontal lines for each call make up the call
   tree. The tree is drawn with the graphical characters in turn_char,
   horz_char, t_char, and vert_char.
*/
static void print_tree_line(FRAME *frame)
{
   if (frame->next != NULL)
   {
      print_tree_line(frame->next);
      if (frame->last_call)
         if (stack == frame)
            print_indent(turn_char, horz_char);
         else
            print_indent(' ', ' ');
      else
         if (stack == frame)
            print_indent(t_char, horz_char);
         else
            print_indent(vert_char, ' ');
   }
}

/* Push function onto call stack. */
static void push_call(FUNCTION *function)
{
   FRAME *frame;

   if ((frame = (FRAME *)malloc(sizeof *frame)) == NULL)
      fatal(F_MEMORY);

   frame->function = function;
   frame->next = stack;
   stack = frame;

   /* Is this last call of the function within which this function is called? */
   frame->last_call = is_last_call(frame, function->name);
}

/*
   Return whether the function whos call is represented by this frame is
   the last call of the function within which it is called.
*/
static BOOLEAN is_last_call(FRAME *frame, char *name)
{
   BOOLEAN last_call = FALSE;

   if (frame->next != NULL)
   {
      CALL *call;

      for (call = frame->next->function->calls;
            call != NULL && strcmp(call->name, name) != 0; call = call->next)
         ;

      if (call == NULL)
         fatal(F_INTERNAL);

      if (call->next == NULL)
         last_call = TRUE;
   }

   return last_call;
}

/* Pop function off of top of call stack. */
static void pop_call(void)
{
   FRAME *new_top;

   if (stack == NULL)
      fatal(F_INTERNAL);

   new_top = stack->next;
   free(stack);
   stack = new_top;
}

/*
   Return whether this function is being called recursively--is already
   on the call stack.
*/
static BOOLEAN recursive_call(FUNCTION *function)
{
   FRAME *frame;

   for (frame = stack; frame != NULL && frame->function != function;
         frame = frame->next)
      ;

   return frame != NULL;
}

/*
   Print table of contents containing function definitions, organized by
   file (module). In comparison with a book's table of contents, a file
   is a chapter and a function is a heading under that chapter.
*/
static void print_toc(void)
{
   FUNCTION *function;
   char *file = "";

   fprintf(out_fp, "\n\nTable of Contents\n\n");

   for (function = functions; function != NULL; function = function->next)
   {
      unsigned i;

      /* Only print file name if different than last one. */
      if (strcmp(file, function->file) != 0)
         fprintf(out_fp, "%s\n", (file = function->file));

      /* Indent function name. */
      for (i = 0; i < TOC_INDENT; ++i)
         fputc(' ', out_fp);

      fprintf(out_fp, "%s", function->name);

      /* Print dots between file name and line number. */
      print_toc_dots(function->name);

      fprintf(out_fp, "%6u\n", function->line);
   }
}

/* Print dots presumably between function name and line number. */
static void print_toc_dots(char *name)
{
   int columns_to_fill;
   int dot_spaces;
   unsigned i;

   columns_to_fill = DOT_SPACES_RIGHT_MARGIN - (TOC_INDENT + strlen(name));
   dot_spaces = columns_to_fill / 2;

   /*
      Print extra space so that dots for each function line up with each
      other.
   */
   if (columns_to_fill % 2 == 1)
      fputc(' ', out_fp);

   for (i = 0; i < dot_spaces; ++i)
      fprintf(out_fp, " .");
}

/*
   Return how the first string compares with the string where the second
   argument points.
*/
static int cmpstr(const char *key, const char * const *element)
{
   return strcmp(key, *element);
}

/*
   Return whether the argument is the name of a function in the standard
   library.
*/
static BOOLEAN library_call(char *name)
{
   /*
      (If you add names to this array, insert them alphabetically so
      that the binary search continues to work correctly.)
   */
   static const char * const library[] = {
      "abort", "abs", "acos", "asctime", "asin", "assert", "atan",
      "atan2", "atexit", "atof", "atoi", "atol", "bsearch", "calloc",
      "ceil", "clearerr", "clock", "cos", "cosh", "ctime", "difftime",
      "div", "exit", "exp", "fabs", "fclose", "feof", "ferror",
      "fflush", "fgetc", "fgetpos", "fgets", "floor", "fmod", "fopen",
      "fprintf", "fputc", "fputs", "fread", "free", "freopen", "frexp",
      "fscanf", "fseek", "fsetpos", "ftell", "fwrite", "getc",
      "getchar", "getenv", "gets", "gmtime", "isalnum", "isalpha",
      "iscntrl", "isdigit", "isgraph", "islower", "isprint", "ispunct",
      "isspace", "isupper", "isxdigit", "labs", "ldexp", "ldiv",
      "localeconv", "localtime", "log", "log10", "longjmp", "malloc",
      "mblen", "mbstowcs", "mbtowc", "memchr", "memcmp", "memcpy",
      "memmove", "memset", "mktime", "modf", "perror", "pow", "printf",
      "putc", "putchar", "puts", "qsort", "raise", "rand", "realloc",
      "remove", "rename", "rewind", "scanf", "setbuf", "setjmp",
      "setlocale", "setvbuf", "signal", "sin", "sinh", "sprintf",
      "sqrt", "srand", "sscanf", "strcat", "strchr", "strcmp",
      "strcoll", "strcpy", "strcspn", "strerror", "strftime", "strlen",
      "strncat", "strncmp", "strncpy", "strpbrk", "strrchr", "strspn",
      "strstr", "strtod", "strtok", "strtol", "strtoul", "strxfrm",
      "system", "tan", "tanh", "time", "tmpfile", "tmpname", "tolower",
      "toupper", "ungetc", "va_arg", "va_end", "va_start", "vfprintf",
      "vprintf", "vsprintf", "wcstombs", "wctomb",
   };

   return bsearch(name, library, DIM_OF(library), sizeof library[0],
         (int (*)(const void *, const void *))cmpstr) != NULL;
}

/*
   Print one level of indention for the call tree. The left_most_char
   is printed followed by N pad_char's, where N is the value of the
   global variable, indention.
*/
static void print_indent(int left_most_char, int pad_char)
{
   unsigned i;

   fputc(left_most_char, out_fp);
   for (i = 1; i < indention; ++i)
      fputc(pad_char, out_fp);
}

/*
   Return whether the function whos name is the second argument is the
   name of a function in the linked list whos head is pointed to by the
   first argument.
*/
static FUNCTION *lookup_function(FUNCTION *head, char *name)
{
   FUNCTION *function;

   for (function = head; function != NULL && strcmp(function->name, name) != 0;
         function = function->next)
      ;

   return function;
}
