/* c:/mks\bin\yacc -d gram.y */
#line 1 "gram.y"

/*************************************************************
   Copyright (c) 1993-1995 by Paul Long  All rights reserved.
**************************************************************/

/*************************************************************
   gram.y - This source file contains the YACC grammar and
            actions for Metre's Standard C parser. It also
            contains main(), error-handling functions,
            trigger-firing functions, functions that
            maintain the typedef symbol table, functions
            that administer the execution of Metre,
            functions that can be called from the rules()
            function, and a replacement for YACC's
            yyerror(). As you can see, most of the grammar
            rules have no actions. This is because the
            grammar is largely used to just skip over the
            parts of C that Metre doesn't care about.
**************************************************************/

#include <stdio.h>
#include <limits.h>
#include <stdarg.h>
#include <stdlib.h>
#include "metreint.h"

/*
   NOTE: Someone said that that the default stack size for MKS yacc and
   AT&T (SCO) yacc of 150 was not sufficient for the code he was running
   through Metre, so he redefined the stack size to 1000. If you
   encounter the same situation, use one of the following directives to
   increase the stack size for the the version of yacc that you're
   using.
*/
#if 0
#define YYSSIZE      1000  /* MKS yacc stack size */
#define YYMAXDEPTH   1000  /* AT&T yacc stack size */
#endif

/* String containing all of the white-space characters. */
#define WHITE_SPACE_CHARS  " \t\r\n\v\f"

/* VMS has different ideas about what constitutes a successful return value. */
#ifdef VAX
#define RETURN_FROM_MAIN_VALUE 1
#else
#define RETURN_FROM_MAIN_VALUE 0
#endif


/* External-function prototypes. */

/* Defines behavior for this Metre tool. */
extern void rules(void);
/* YACC-generated parser function. */
extern yyparse();
/* lex-generated scanner function. */
extern yylex();
/* Error function that yacc-generated parser calls. */
extern void yyerror(char *);


/* External variables. */

/*
   Metre maintains statistics in the int_* structures, e.g., int_prj.
   The structures with the unadorned names, e.g., prj, are used to
   communicate with the rules. They are zero-filled except when a
   trigger is being fired. At that time, the contents of the
   corresponding int_* structure is copied to it, the rules() function
   is called, and then the structure is zero-filed again.
*/
PRJ prj, int_prj;    /* Project. */
MOD mod, int_mod;    /* Module. */
FCN fcn;             /* Function. */
static FCN int_fcn;
STM stm;             /* Statement. */
static STM int_stm;
LIN lin, int_lin;    /* Line. */
GRM grm, int_grm;    /* Grammatical. */
LEX lex, int_lex;    /* Lexical. */

/*
   Which command-line argument (argv) at which to start looking for the
   next file name to process and original file name, respectively.
*/
unsigned next_cmd_line_file;
unsigned next_cmd_line_file_orig_n;

/* Number of decision points, conditions, and functions in module (file). */
unsigned mod_decisions;
unsigned mod_conditions;
unsigned mod_functions;

/*
   Indicates whether we are looking for a tag. Parser writes it, lexer
   reads it. Since tags are in a separate name space from other
   identifiers, this helps the lexer determine whether a lexeme is an
   identifier or a type name.
*/
BOOLEAN looking_for_tag;

/*
   Indicates whether we are within a function definition. Used to
   determine whether to use the last identifier as the function name.
*/
BOOLEAN is_within_function;

/*
   Name of current input file and name of original file, if specified on
   command line with the substitute-file-name option. If substitute not
   provided, input_file_orig_name points to the same name as input_file.
*/
char *input_file;
char *input_file_orig_name;

/* Where all output is written. */
FILE *out_fp;

static char *command_name = DEFAULT_COMMAND_NAME;

/* Global versions of main's argc and argv. */
int cmd_line_argc;
char **cmd_line_argv;

/*
   Block within which typedef type names are held in the typedef
   symbol-table.
*/
typedef struct typedef_sym_blk {
   struct typedef_sym_blk *next;          /* Next block. */
   unsigned total;                        /* Total in this block. */
   char *id[TYPEDEF_SYMBOLS_PER_BLOCK]; /* Array of pointers to type names. */
} TYPEDEF_SYM_BLK;

/*
   Head of the typedef symbol-table. NOTE: This is the only symbol table
   in Metre, and it is used just to hold typedef type names, not all
   symbols.
*/
TYPEDEF_SYM_BLK *typedef_sym_tbl_head = NULL;

/* Command-line argument. Used to hold arguments taken from script file. */
typedef struct arg {
   struct arg *next;
   char *string;
} ARG;


/* Static variables. */

/* Control depth--current number of nested control structures. */
static unsigned depth;

/* Used to determine how many statements are on a line. */
static unsigned previous_statements_line_number;

/* Number of decision points and conditions in function. */
static unsigned fcn_decisions;
static unsigned fcn_conditions;

/*
   Metre maintains the number of statements appearing on a line. The
   is_if and is_else BOOLEANs are used principally to relax the criteria
   for what constitutes a statement. For example, "else if," is not
   considered two statements because this is a common idiom.
*/
static BOOLEAN is_else, is_if;

/*
   is_typedef indicates whether we are parsing a typedef. Since typedefs
   cannot be nested, it's okay that this is just a BOOLEAN. This is used
   to find the type name of a typedef so that it can be added to the
   typedef symbol table. If the type name is subsequently encountered by
   the lexer, the lexer returns a token for a type name rather than for
   an identifier. This is the ugly part of C that cannot practically be
   expressed by a YACC grammar specification.
*/
static BOOLEAN is_typedef;

/*
   nested_decl_specs is used to distinguish the name of the thing being
   declared, such as the type name in a typedef, from other identifiers
   in the declaration.
*/
static unsigned nested_decl_specs;

/*
   Whether the last statement was a case label. Used to consider
   adjacent case labels as one decision point. The transition from
   is_case_label==TRUE to is_case_label==FALSE is a decision point,
   instead of each case label being a decision point. Adjacent case
   labels are considered one decision point because this is analogous to
   an if statement with or logical operators, ||. For example,

      switch (expr) {
      case 5:
      case 10:
         a;
      }

   is, in this regard, analogous to

      if (expr == 5 || expr == 10)
         a;

   I did not want to bias the metric against the switch statement.
*/
static BOOLEAN is_case_label;

static IDENTIFIER function_name;
static IDENTIFIER last_ident;
static IDENTIFIER last_decl;

/*
   Pointer to current operator, keyword, or identifier as passed to
   rules if such a token is encountered.
*/
static char *current_operator;
static char *current_keyword;
static char *current_identifier;
static char *current_type_name;

/* Static function declarations. */

static void begin_project(char *);
static void fire_fcn(void);
static void fire_stm(void);
static void print_exception(int, char *, char *, va_list);
static void typedef_symbol_table_add(char *);
static void case_label(void);
static void not_case_label(void);
static void check_starting_options(void);
static void print_banner(void);
static void print_help(void);
static void stat_func_begin(void);
static void stat_func_end(void);
static void check_multiple_statements(void);
static void int_warn(int, char *, ...);
static void fire_operator(void);
static void fire_keyword(void);
static void fire_identifier(void);
static void fire_declarator(void);
static void fire_type_name(void);
static void found_constant(void);
static void found_string(void);
static void found_expression_statement(BOOLEAN);
static void function_call(void);
static void after_compound_declarations(void);
static void before_compound_declarations(void);
static void after_initializer(void);
static void before_initializer(void);
static void get_options_from_file(char *);
static void build_new_argv(ARG *);
#define TK_IDENTIFIER	257
#define TK_CONSTANT	258
#define TK_STRING_LITERAL	259
#define TK_SIZEOF	260
#define TK_PTR_OP	261
#define TK_INC_OP	262
#define TK_DEC_OP	263
#define TK_LEFT_OP	264
#define TK_RIGHT_OP	265
#define TK_LE_OP	266
#define TK_GE_OP	267
#define TK_EQ_OP	268
#define TK_NE_OP	269
#define TK_AND_OP	270
#define TK_OR_OP	271
#define TK_MUL_ASSIGN	272
#define TK_DIV_ASSIGN	273
#define TK_MOD_ASSIGN	274
#define TK_ADD_ASSIGN	275
#define TK_SUB_ASSIGN	276
#define TK_LEFT_ASSIGN	277
#define TK_RIGHT_ASSIGN	278
#define TK_AND_ASSIGN	279
#define TK_XOR_ASSIGN	280
#define TK_OR_ASSIGN	281
#define TK_TYPE_NAME	282
#define TK_TYPEDEF	283
#define TK_EXTERN	284
#define TK_STATIC	285
#define TK_AUTO	286
#define TK_REGISTER	287
#define TK_CHAR	288
#define TK_SHORT	289
#define TK_INT	290
#define TK_LONG	291
#define TK_SIGNED	292
#define TK_UNSIGNED	293
#define TK_FLOAT	294
#define TK_DOUBLE	295
#define TK_CONST	296
#define TK_VOLATILE	297
#define TK_VOID	298
#define TK_STRUCT	299
#define TK_UNION	300
#define TK_ENUM	301
#define TK_ELIPSIS	302
#define TK_RANGE	303
#define TK_CASE	304
#define TK_DEFAULT	305
#define TK_IF	306
#define TK_ELSE	307
#define TK_SWITCH	308
#define TK_WHILE	309
#define TK_DO	310
#define TK_FOR	311
#define TK_GOTO	312
#define TK_CONTINUE	313
#define TK_BREAK	314
#define TK_RETURN	315
#define THEN	316
extern int yychar, yyerrflag;
#ifndef YYSTYPE
#define YYSTYPE int
#endif
extern YYSTYPE yyval;
extern YYSTYPE yylval;
#line 1472 "gram.y"
/* Functions. */

int main(int main_argc, char *main_argv[])
{
   ZERO(grm);
   ZERO(int_grm);

   /* Input not determined yet. */
   yyin = NULL;
   /* Use stdout until/if output is redirected with command-line option. */
   out_fp = stdout;

   /* Fire the need-command-name trigger. */
   ZERO(int_prj);
   int_prj.need_cmd_name = TRUE;
   fire_prj();
   int_prj.need_cmd_name = FALSE;

   print_banner();

   /* Save so that the argument list may be accessed globally. */
   cmd_line_argv = main_argv;
   cmd_line_argc = main_argc;

   /* Get command-line options that affect Metre at the start. */
   check_starting_options();

   /* Initialize YACC and Lex. */
   init_yacc();
   init_lex();

   if (option(HELP_OPT_CHAR))
      print_help();
   else
   {
      /*
         Look for the first input file and first original name of that
         input file.
      */
      next_cmd_line_file = 0;
      next_cmd_line_file_orig_n = 0;
      if ((input_file = get_next_input_file(&next_cmd_line_file)) != NULL)
         if ((yyin = fopen(input_file, "r")) != NULL)
            begin_project(input_file);
         else
            int_warn(W_CANNOT_OPEN_FILE, input_file);
      else
         /* Since no input file specified on command line, process stdin. */
         begin_project("stdin");
   }

   return RETURN_FROM_MAIN_VALUE;
}

/* Begin the project by firing initial triggers and calling parser. */
static void begin_project(char *input_file)
{
   input_file_orig_name =
         get_next_input_file_orig_name(&next_cmd_line_file_orig_n);
   /* If no original name given, use the actual as the original, also. */
   if (input_file_orig_name == NULL)
      input_file_orig_name = input_file;

   /* Fire the beginning-of-project trigger. */
   ZERO(int_prj);
   int_prj.begin = TRUE;
   fire_prj();
   int_prj.begin = FALSE;

   /* Fire the beginning-of-module trigger. */
   ZERO(int_mod);
   int_mod.begin = TRUE;
   fire_mod();
   int_mod.begin = FALSE;

   /*
      Call the YACC-generated parser to wade through the input file(s).
      When all done, close the last input file. yywrap() could have
      opened any number of subsequent input files.
   */
   yyparse();

   fclose(yyin);
}

/* Common exception-handling function. */
static void print_exception(int n, char *severity_str, char *format, va_list ap)
{
   fflush(out_fp);

   /*
      Display input line if supposed to, then display line with marker
      character in it if appropriate.
   */
   if (mod_name() != NULL && strlen(mod_name()) > 0 &&
         !int_mod.end && !int_fcn.end)
   {
      if (!display_input)
         fputs(line(), out_fp);

      if (!int_lin.end)
         fputs(marker(), out_fp);
   }

   /*
      Display module name and line number if a module name has been
      established. If one hasn't, the error must not relate to a
      location in the input file.
   */
   if (mod_name() != NULL && strlen(mod_name()) > 0)
      fprintf(out_fp, "%s(%u): ", mod_name(), lineno());
   fprintf(out_fp, "%s%04u: ", severity_str, n);

   vfprintf(out_fp, format, ap);
   fputc('\n', out_fp);
}

/* Print message for internal fatal error. */
void int_fatal(int n, char *format, ...)
{
   va_list ap;

   va_start(ap, format);

   print_exception(n, "Fatal Error ME", format, ap);

   exit(1);
}

/* Print message for fatal error that occured in the rules() function. */
void fatal(int n, char *format, ...)
{
   va_list ap;

   va_start(ap, format);

   print_exception(n, "Fatal Error E", format, ap);

   exit(1);
}

/* Print message for internal warning. */
static void int_warn(int n, char *format, ...)
{
   va_list ap;

   if (!option(NO_WARNINGS_OPT_CHAR))
   {
      va_start(ap, format);

      print_exception(n, "Warning MW", format, ap);
   }
}

/* Print message for warning that occured in the rules() function. */
void warn(int n, char *format, ...)
{
   va_list ap;

   if (!option(NO_WARNINGS_OPT_CHAR))
   {
      va_start(ap, format);

      print_exception(n, "Warning W", format, ap);
   }
}

/* Function called by YACC when it detects an error. */
void yyerror(char *s)
{
#ifdef DEBUG_TYPEDEF
   /* Used for debugging typedef processing. */
   typedef_symbol_table_dump();
#endif
   int_fatal(0, "%s", s);
}

/* Add type name to the typedef symbol table. */
static void typedef_symbol_table_add(char *p_symbol)
{
   TYPEDEF_SYM_BLK *block;
   /* Allocate room for string and its '\0' terminator. */
   char *str_buf = (char *)malloc(strlen(p_symbol) + 1);

   if (str_buf == NULL)
      int_fatal(E_NO_HEAP);

   /* Make copy of argument. */
   strcpy(str_buf, p_symbol);

   /* Is symbol table empty? */
   if (typedef_sym_tbl_head == NULL)
   {
      /* Allocate first block and make the head of the table list. */
      typedef_sym_tbl_head = block = (TYPEDEF_SYM_BLK *)malloc(sizeof *block);
      if (block == NULL)
         int_fatal(E_NO_HEAP);

      block->total = 0;
      block->next = NULL;
   }
   else
   {
      TYPEDEF_SYM_BLK *prev_block;

      /* Find first block in symbol-table list that isn't full. */
      for (prev_block = NULL, block = typedef_sym_tbl_head;
            block != NULL && block->total == DIM_OF(block->id);
            prev_block = block, block = block->next)
         ;

      /* If all blocks are full, allocate a new one and append to list. */
      if (block == NULL)
      {
         block = (TYPEDEF_SYM_BLK *)malloc(sizeof *block);
         if (block == NULL)
            int_fatal(E_NO_HEAP);

         block->total = 0;
         block->next = NULL;
         if (prev_block == NULL)
            typedef_sym_tbl_head = block;
         else
            prev_block->next = block;
      }
   }

   /* Save pointer to the new type name in first empty slot of this block. */
   block->id[block->total++] = str_buf;
}

/* Return TRUE if symbol exists in the typedef symbol table. */
BOOLEAN typedef_symbol_table_find(char *p_symbol)
{
   TYPEDEF_SYM_BLK *block;
   BOOLEAN found = FALSE;

   /* Loop for each block of symbols. */
   for (block = typedef_sym_tbl_head; block != NULL; block = block->next)
   {
      unsigned i;

      /* Loop for each symbol in this block. */
      for (i = 0; i < block->total; ++i)
         if (strcmp(block->id[i], p_symbol) == 0)
            found = TRUE;
   }

   return found;
}

/*
   Remove all symbols (and symbol blocks) from the symbol table--make
   the table empty.
*/
void typedef_symbol_table_flush(void)
{
   TYPEDEF_SYM_BLK *block;

   /* Loop for each block of symbols. */
   for (block = typedef_sym_tbl_head; block != NULL; )
   {
      unsigned i;
      TYPEDEF_SYM_BLK *temp_block;

      /* Loop for each symbol in this block. */
      for (i = 0; i < block->total; ++i)
         free(block->id[i]);              /* Free symbol. */

      temp_block = block->next;
      free(block);                        /* Free block now that it's empty. */
      block = temp_block;
   }

   typedef_sym_tbl_head = NULL;
}

#ifdef DEBUG_TYPEDEF
/*
   Display the contents of the typedef symbol table. Used for debugging
   typedef processing.
*/
void typedef_symbol_table_dump(void)
{
   TYPEDEF_SYM_BLK *block;

   fputs("typedef symbol table dump:\n", out_fp);

   /* Loop for each block of symbols. */
   for (block = typedef_sym_tbl_head; block != NULL; block = block->next)
   {
      unsigned i;

      /* Loop for each symbol in this block. */
      for (i = 0; i < block->total; ++i)
         fprintf(out_fp, "%s\n", block->id[i]);
   }
}
#endif

/* Record that the last statement was a case label. */
static void case_label(void)
{
   is_case_label = TRUE;
}

/*
   Record that the last statement was not a case label. If the previous
   statement was a case label, increment the decision-point
   counter--this ends a run of adjacent case labels which count as one
   decision point.
*/
static void not_case_label(void)
{
   if (is_case_label)
   {
      is_case_label = FALSE;
      ++fcn_decisions;        /* This is a decision point. */
   }
}

/* Initialize parser. */
void init_yacc(void)
{
   /* Make the typedef symbol table empty. */
   typedef_symbol_table_flush();

   /*
      See the definition of each of these variables, above, for more
      information.
   */
   is_typedef = FALSE;
   nested_decl_specs = 0;
   is_case_label = FALSE;

   function_name[0] = '\0';
   last_ident[0] = '\0';
   last_decl[0] = '\0';

   mod_decisions = 0;
   mod_conditions = 0;
   mod_functions = 0;
   is_else = is_if = FALSE;
   looking_for_tag = FALSE;

   is_within_function = FALSE;

   current_operator = "";
   current_keyword = "";
   current_identifier = "";
   current_type_name = "";
}

/*
   Returns 1 if the indicated option character was specified on the
   command line. E.g., for -S, it would return 1 for option('S'). If
   option character is immediately followed by a decimal integer, this
   function will instead return the value of that number, e.g. for -S3,
   it would return 3. NOTE: 0 would be ambiguous, e.g., -S0, because 0
   would also be returned if -S were not specified.
*/
int option(int opt_char)
{
   char *opt_str = str_option(opt_char);

   return opt_str == NULL ? 0 : strlen(opt_str) == 0 ? 1 : atoi(opt_str);
}

/*
   Returns pointer to the part of an option following the option character.
   E.g., for -Xabc=def, it would point to the "abc=def" part.
*/
char *str_option(int opt_char)
{
   unsigned i;

   /* (Skip the command itself, argv[0].) */
   for (i = 1; i < cmd_line_argc; ++i)
      if (strchr(OPT_INTRO_CHARS, cmd_line_argv[i][0]) != NULL)
         if (cmd_line_argv[i][OPTION_START] == opt_char)
            break;

   return i < cmd_line_argc ? &cmd_line_argv[i][OPTION_STRING_START] : NULL;
}

/*
   Return pointer to the name of the next input file in the argv array
   starting with the argument that p_i points to. An input file name is
   distinguished from the command-line options by not starting with one
   of characters in the OPT_INTRO_CHARS string.

   NOTE: argument 0 is the name of the command, not the first argument,
   per se.
*/
char *get_next_input_file(unsigned *p_i)
{
   for (++*p_i; *p_i < cmd_line_argc &&
         strchr(OPT_INTRO_CHARS, cmd_line_argv[*p_i][0]) != NULL; ++*p_i)
      ;

   return *p_i < cmd_line_argc ? cmd_line_argv[*p_i] : NULL;
}

/*
   Return pointer to the original name of the next input file in the
   argv array starting with the argument that p_i points to. An original
   name is specified as a command-line option. That is how an original
   name is distinguished from the name of an actual input file

   NOTE: argument 0 is the name of the command, not the first argument,
   per se.
*/
char *get_next_input_file_orig_name(unsigned *p_i)
{
   for (++*p_i; *p_i < cmd_line_argc; ++*p_i)
      if (strchr(OPT_INTRO_CHARS, cmd_line_argv[*p_i][0]) != NULL &&
            cmd_line_argv[*p_i][OPTION_START] == SUBST_FILE_OPT_CHAR)
         break;

   return *p_i < cmd_line_argc ?
         &cmd_line_argv[*p_i][OPTION_STRING_START] : NULL;
}

/* Returns pointer to current module (file) name. */
char *mod_name(void)
{
   return input_file_orig_name;
}

/* Process the command-line options that affect the start of Metre. */
static void check_starting_options(void)
{
   static char *output_file_name;
   char *script_file_name;

   /* Get options from file rather than command line? */
   script_file_name = str_option(SCRIPT_FILE_OPT_CHAR);
   if (script_file_name != NULL)
      get_options_from_file(script_file_name);

   /* Interleave the input with the output? */
   display_input = option(COPY_INPUT_OPT_CHAR);

   /*
      Name of listing file to contain output. This for those OS's that don't
      support command-line redirection, e.g., VMS.
   */
   output_file_name = str_option(LISTING_OPT_CHAR);
   if (output_file_name != NULL && strlen(output_file_name) > 0)
   {
      out_fp = fopen(output_file_name, "w");
      if (out_fp == NULL)
      {
         out_fp = stdout;     /* Restore to previous, good output file. */
         int_fatal(E_CANT_OPEN_LISTING_FILE);   /* Die. */
      }
   }
}

/*
   Build a "fake" argument list and count, copy command name (argv[0])
   to it, then append the options that are in the file to it. (Sneaky,
   aren't I?) (Perhaps the arguments from the file should be appended
   to whatever is on the command line. Maybe next time.)
*/
static void get_options_from_file(char *file)
{
   FILE *cmd_fp;

   /*
      If other options on command line, generate warning but continue.
      Note: 2 = the command itself + the file option.
   */
   if (cmd_line_argc > 2)
      int_warn(W_OPTIONS_IGNORED);

   /* Attempt to open the specified script file. */
   if (strlen(file) == 0 || (cmd_fp = fopen(file, "r")) == NULL)
      int_fatal(E_CANT_OPEN_COMMAND_FILE);
   else
   {
      ARG *args = NULL, *args_tail = NULL;
      char buf[BUFSIZ];

      /*
         Initialize argc to 1 for the command name which is carried over
         from the orginal argv.
      */
      cmd_line_argc = 1;

      /* Loop for each line of input. */
      while (fgets(buf, sizeof buf, cmd_fp) != NULL)
      {
         char *start;   /* Buffer address 1st time through loop, then NULL. */
         char *token;   /* Pointer to token in buffer. */

         /* Loop for each token in this line of input. */
         for (start = buf; (token = strtok(start, WHITE_SPACE_CHARS)) != NULL;
               start = NULL)
         {
            ARG *arg;

            /*
               Allocate buffers for this argument and its string
               (including its '\0' terminator).
            */
            if ((arg = (ARG *)malloc(sizeof *arg)) == NULL)
               int_fatal(E_NO_HEAP);
            if ((arg->string = (char *)malloc(strlen(token) + 1)) == NULL)
               int_fatal(E_NO_HEAP);

            /* Copy argument into string buffer. */
            strcpy(arg->string, token);

            /* Attach this argument to end of argument linked list. */
            if (args_tail == NULL)
               args = arg;
            else
               args_tail->next = arg;

            args_tail = arg;
            arg->next = NULL;

            ++cmd_line_argc;
         }
      }

      /* Allocate buffer for argv array then move arguments from linked list. */
      build_new_argv(args);
   }
}

/* Build a new list of "command-line arguments" pointed to by cmd_line_argv. */
static void build_new_argv(ARG *args)
{
   unsigned i;
   char *argv_cmd_name;

   /* Save pointer to command name from original argv. */
   argv_cmd_name = cmd_line_argv[0];

   /* Allocate buffer to hold array of pointers to arguments. */
   cmd_line_argv = (char **)malloc(cmd_line_argc * sizeof *cmd_line_argv);
   if (cmd_line_argv == NULL)
      int_fatal(E_NO_HEAP);

   /* Restore pointer to command name. */
   cmd_line_argv[0] = argv_cmd_name;

   /*
      Fill in new argv from linked list of arguments, freeing list
      entries but not the strings from the list.  (Skip the place for
      the "command" argument.)
   */
   for (i = 1; i < cmd_line_argc; ++i)
   {
      ARG *arg;

      cmd_line_argv[i] = args->string;

      arg = args->next;
      free(args);
      args = arg;
   }
}

/* Print name, version, and copyright notice. */
static void print_banner(void)
{
   fprintf(out_fp, "METRE Version %s  ", metre_version);
   fputs("Copyright (c) 1993-1995 by Paul Long  All rights reserved.\n", out_fp);
}

/* Print Metre command-usage information. */
static void print_help(void)
{
   fprintf(out_fp, "Syntax: %s [ option | file ]...\n", command_name);
   fprintf(out_fp, "-%cxxx=[xxx] Define identifier          "
         "-%cxxx       Name of output file\n",
         DEFINE_OPT_CHAR, LISTING_OPT_CHAR);
   fprintf(out_fp, "-%cxxx       Substitute file name       "
         "-%c          Copy input to output\n",
         SUBST_FILE_OPT_CHAR, COPY_INPUT_OPT_CHAR);
   fprintf(out_fp, "-%c          Suppress warnings          "
         "-%c          This help\n", NO_WARNINGS_OPT_CHAR, HELP_OPT_CHAR);
   fprintf(out_fp, "-%cxxx       Response file\n", SCRIPT_FILE_OPT_CHAR);

   /* Fire the project-help trigger. */
   ZERO(int_prj);
   int_prj.help = TRUE;
   fire_prj();
   int_prj.help = FALSE;
}

/* Called at the beginning of a function definition. */
static void stat_func_begin(void)
{
   /* Fire the beginning-of-function trigger. */
   ZERO(int_fcn);
   int_fcn.begin = TRUE;
   fire_fcn();
   int_fcn.begin = FALSE;

   /* Initialize variables relating to a function. */
   depth = 0;
   is_within_function = TRUE;
   fcn_decisions = 0;
   fcn_conditions = 0;
   previous_statements_line_number = 0;

   /* Install starting counts. Replaced with deltas at end-of-function. */
   int_fcn.lines.white = int_mod.lines.white;
   int_fcn.lines.com = int_mod.lines.com;
   int_fcn.lines.code = int_mod.lines.code;
   int_fcn.lines.total = lineno();
}

/* Called at the end of a function definition. */
static void stat_func_end(void)
{
   /*
      Replace initial counts with their deltas to arrive at totals for
      the function.
   */
   int_fcn.lines.white = int_mod.lines.white - int_fcn.lines.white;
   int_fcn.lines.com = int_mod.lines.com - int_fcn.lines.com;
   int_fcn.lines.code = int_mod.lines.code - int_fcn.lines.code;
   int_fcn.lines.total = lineno() - int_fcn.lines.total;

   ++mod_functions;                 /* Increment function count. */

   /* Accumulate function decision and condition counts into module counts. */
   mod_decisions += fcn_decisions;
   mod_conditions += fcn_conditions;

   /*
      Install the rest of the function-related information into the
      function structure and fire the end-of-function trigger.
   */
   int_fcn.decisions = fcn_decisions;
   int_fcn.conditions = fcn_conditions;
   int_fcn.end = TRUE;
   fire_fcn();
   ZERO(int_fcn);

   /*
      Clear variables that would otherwise indicate that we are still
      parsing a function.
   */
   strcpy(function_name, "");
   is_within_function = FALSE;
}

/*
   Returns pointer to current function name, or an empty string if not
   in a function.
*/
char *fcn_name(void)
{
   return function_name;
}

/*
   Increment the number of statements for the current line. This
   function is called whenever a statement is encountered. If the
   current line number is the same as when the last statement was
   encountered and if this is not the special situation of "else if,"
   the counter is incremented.
*/
static void check_multiple_statements(void)
{
   if (previous_statements_line_number == lineno())
      if (is_else && is_if)
         ;        /* do nothing */
      else
         ++int_lin.statements;
   else
      previous_statements_line_number = lineno();

   is_else = is_if = FALSE;
}


/*
   This is true for all of the fire_*() functions: Copy the internal
   structure to the exported structure, allow the triggers to fire in
   the rules() function, then zero-fill the exported structure so that
   unrelated rules won't execute when other triggers are fired.
*/

void fire_prj(void)
{
   prj = int_prj;
   rules();
   ZERO(prj);
}

void fire_mod(void)
{
   mod = int_mod;
   rules();
   ZERO(mod);
}

static void fire_fcn(void)
{
   fcn = int_fcn;
   rules();
   ZERO(fcn);
}

static void fire_stm(void)
{
   stm = int_stm;
   rules();
   ZERO(stm);
}

void fire_lin(void)
{
   lin = int_lin;
   rules();
   ZERO(lin);
}

void fire_lex(void)
{
   lex = int_lex;
   rules();
   ZERO(lex);
}

void fire_grm(void)
{
   grm = int_grm;
   rules();
   ZERO(grm);
}

/* Fire the operator trigger. */
static void fire_operator(void)
{
   current_operator = token();
   int_grm.is_operator = TRUE;
   fire_grm();
   int_grm.is_operator = FALSE;
   current_operator = "";
}

/* Fire the keyword trigger. */
static void fire_keyword(void)
{
   current_keyword = token();
   int_grm.is_keyword = TRUE;
   fire_grm();
   int_grm.is_keyword = FALSE;
   current_keyword = "";
}

/* Fire the identifier trigger. */
static void fire_identifier(void)
{
   current_identifier = token();
   int_grm.is_identifier = TRUE;
   fire_grm();
   int_grm.is_identifier = FALSE;
   current_identifier = "";
}

/* Fire the declarator trigger. */
static void fire_declarator(void)
{
   int_grm.is_declarator = TRUE;
   fire_grm();
   int_grm.is_declarator = FALSE;
}

/* Fire the type-name trigger. */
static void fire_type_name(void)
{
   current_type_name = token();
   int_grm.is_type_name = TRUE;
   fire_grm();
   int_grm.is_type_name = FALSE;
   current_type_name = "";
}

/* Fire the grammar trigger with is_constant set to TRUE. */
static void found_constant(void)
{
   int_grm.is_constant = TRUE;
   fire_grm();
   ZERO(int_grm);
}

/* Fire the grammar trigger with is_string set to TRUE. */
static void found_string(void)
{
   int_grm.is_string = TRUE;
   fire_grm();
   ZERO(int_grm);
}

/* Indicate a function call. */
static void function_call(void)
{
   int_grm.is_call = TRUE;
   fire_grm();
   ZERO(int_grm);
}

/* Indicate an expression statement. Can be empty, i.e. just ";". */
static void found_expression_statement(BOOLEAN is_empty)
{
   check_multiple_statements();

   ++int_fcn.low;          /* Low-level statement. */

   /* Used to consider adjacent case labels as one decision point. */
   not_case_label();

   /*
      Indicate at the end of an expression, low-level statement, whether
      an empty expression, then fire the end-of-statement trigger.
   */
   int_stm.is_expr = int_stm.is_low = int_stm.end = TRUE;
   int_stm.is_empty_expr = is_empty;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/* Return whether the specified operator is the current operator. */
BOOLEAN oper(char *op)
{
   return grm.is_operator && strcmp(current_operator, op) == 0;
}

/* Return whether the specified keyword is the current keyword. */
BOOLEAN keyword(char *name)
{
   return grm.is_keyword && strcmp(current_keyword, name) == 0;
}

/* Return whether the specified identifier is the current identifier. */
BOOLEAN identifier(char *name)
{
   return grm.is_identifier && strcmp(current_identifier, name) == 0;
}

/*
   Indicate the beginning of the declarations within a compound
   statement, whether or not there were any, and the depth at which they
   occured.
*/
static void before_compound_declarations(void)
{
   int_stm.before_compound_declarations = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/*
   Indicate the end of the declarations within a compound statement,
   whether or not there were any, and the depth at which they occured.
*/
static void after_compound_declarations(void)
{
   int_stm.after_compound_declarations = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/*
   Indicate the beginning of an initializer (before the "=") and the
   depth at which it occurs.
*/
static void before_initializer(void)
{
   int_stm.before_initializer = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/* Indicate the end of an initializer and the depth at which it occured. */
static void after_initializer(void)
{
   int_stm.after_initializer = TRUE;
   int_stm.depth = depth;
   fire_stm();
   ZERO(int_stm);
}

/*
   Return pointer to identifier that corresponds to when grammar
   identifier trigger is fired.
*/
char *this_identifier(void)
{
   return current_identifier;
}

/* Return pointer to last-encountered identifier. */
char *last_identifier(void)
{
   return last_ident;
}

/* Return pointer to last-encountered declarator. */
char *last_declarator(void)
{
   return last_decl;
}

/*
   When prj.cmd_name is true, a rule may provide the name of the command
   that invokes this program. The default is "METRE".
*/
void cmd_name(char *name)
{
   if (name != NULL)
      command_name = name;
}

/* Return command-line argument count. */
int argc(void)
{
   return cmd_line_argc;
}

/* Return pointer to command-line-argument array. */
char **argv(void)
{
   return cmd_line_argv;
}
static short yydef[] = {
	  -1,  287,  300,  302,   -5,  -21,  -37,  299,  301,  -41, 
	 -47,  -51,  -55,  -63,  308,  310,  312,  314,  316,  318, 
	 321,  326,  329,  332,  333,  334,  303,  -67,  -71,  298, 
	 296,  297,  -79,  295,  174,  -83,  148,  293,  294,  313, 
	 315,  317,  319,  320,  322,  323,  324,  325,  327,  328, 
	 330,  331,  294,  304,  289,  -87,  292,  309,  311,  -91, 
	 -95,  291
};
static short yyex[] = {
	   0,  288,   -1,    1,  257,  306,   40,  306,   41,  306, 
	  91,  306,   44,  306,   42,  306,   59,  306,   -1,    6, 
	 257,  307,   40,  307,   41,  307,   91,  307,   44,  307, 
	  42,  307,   59,  307,   -1,    5,    0,    0,   -1,    1, 
	 257,   14,   41,   12,   -1,   13,  123,   10,   -1,   11, 
	 123,    8,   -1,    9,   44,  305,   61,    7,   59,  305, 
	  -1,   19,  125,   15,   -1,   16,  125,   17,   -1,   18, 
	  44,  305,   61,    7,   59,  305,   -1,    1,   41,    3, 
	  -1,    4,   59,  290,   -1,    1,   59,  290,   -1,    1, 
	  41,  290,   -1,    1,   41,  290,   -1,    1
};
static short yyact[] = {
	-330, -319, -249, -267, -286, -285, -284, -283, -282, -281, 
	-280, -279, -278, -277, -276, -275, -274, -273, -272, -271, 
	-270, -269, -268,  301,  300,  299,  298,  297,  296,  295, 
	 294,  293,  292,  291,  290,  289,  288,  287,  286,  285, 
	 284,  283,  282,  257,   42,   40, -319, -267, -281, -280, 
	-279, -278, -277, -276, -275, -274, -273, -272, -271, -270, 
	-269, -268,  301,  300,  299,  298,  297,  296,  295,  294, 
	 293,  292,  291,  290,  289,  288,  282,   42, -330, -319, 
	-249,  257,   42,   40, -330, -249,  257,   40, -330, -328, 
	  91,   40, -266, -249,  257,  123, -330, -319, -287, -249, 
	 257,   59,   42,   40, -266, -267, -286, -285, -284, -283, 
	-282, -281, -280, -279, -278, -277, -276, -275, -274, -273, 
	-272, -271, -270, -269, -268,  301,  300,  299,  298,  297, 
	 296,  295,  294,  293,  292,  291,  290,  289,  288,  287, 
	 286,  285,  284,  283,  282,  123, -329,   41, -315, -320, 
	-330, -319, -318, -317, -327, -316, -249, -250, -251, -321, 
	-324, -323,  263,  262,  260,  259,  258,  257,  126,   93, 
	  45,   43,   42,   40,   38,   33, -249,  257, -267, -286, 
	-285, -284, -283, -282, -281, -280, -279, -278, -277, -276, 
	-275, -274, -273, -272, -271, -270, -269, -268,  301,  300, 
	 299,  298,  297,  296,  295,  294,  293,  292,  291,  290, 
	 289,  288,  287,  286,  285,  284,  283,  282, -322, -287, 
	  59,   44, -327,   93, -300, -301,  271,   63, -302,  270, 
	-303,  124, -304,   94, -320,   38, -306, -305,  269,  268, 
	-310, -309, -308, -307,  267,  266,   62,   60, -312, -311, 
	 265,  264, -318, -317,   45,   43, -313, -319, -314,   47, 
	  42,   37, -315, -320, -330, -319, -318, -317, -316, -249, 
	-250, -251, -321, -324, -323,  263,  262,  260,  259,  258, 
	 257,  126,   45,   43,   42,   40,   38,   33, -330, -326, 
	-328, -325, -324, -323,  263,  262,  261,   91,   46,   40, 
	-315, -320, -330, -319, -318, -317, -316, -249, -250, -251, 
	-321, -324, -323, -267, -281, -280, -279, -278, -277, -276, 
	-275, -274, -273, -272, -271, -270, -269, -268,  301,  300, 
	 299,  298,  297,  296,  295,  294,  293,  292,  291,  290, 
	 289,  288,  282,  263,  262,  260,  259,  258,  257,  126, 
	  45,   43,   42,   40,   38,   33, -251,  259, -298,   61, 
	-322, -265,  125,   44, -266,  123, -267, -281, -280, -279, 
	-278, -277, -276, -275, -274, -273, -272, -271, -270, -269, 
	-268,  301,  300,  299,  298,  297,  296,  295,  294,  293, 
	 292,  291,  290,  289,  288,  282, -315, -320, -330, -319, 
	-318, -317, -287, -266, -316, -249, -250, -251, -321, -324, 
	-323, -263, -262, -261, -260, -258, -257, -256, -255, -254, 
	-253, -252,  315,  314,  313,  312,  311,  310,  309,  308, 
	 306,  305,  304,  263,  262,  260,  259,  258,  257,  126, 
	 123,   59,   45,   43,   42,   40,   38,   33, -265,  125, 
	-322,   44, -330, -319, -328, -249,  257,   91,   42,   40, 
	-330, -319, -328, -267, -281, -280, -279, -278, -277, -276, 
	-275, -274, -273, -272, -271, -270, -269, -268,  301,  300, 
	 299,  298,  297,  296,  295,  294,  293,  292,  291,  290, 
	 289,  288,  282,   91,   42,   40, -298, -297, -296, -295, 
	-294, -293, -292, -291, -290, -289, -288,  281,  280,  279, 
	 278,  277,  276,  275,  274,  273,  272,   61, -329, -322, 
	  44,   41, -265, -249,  257,  125, -330, -319, -299, -249, 
	-267, -281, -280, -279, -278, -277, -276, -275, -274, -273, 
	-272, -271, -270, -269, -268,  301,  300,  299,  298,  297, 
	 296,  295,  294,  293,  292,  291,  290,  289,  288,  282, 
	 257,   58,   42,   40, -265, -267, -281, -280, -279, -278, 
	-277, -276, -275, -274, -273, -272, -271, -270, -269, -268, 
	 301,  300,  299,  298,  297,  296,  295,  294,  293,  292, 
	 291,  290,  289,  288,  282,  125, -315, -320, -330, -319, 
	-318, -317, -266, -316, -249, -250, -251, -321, -324, -323, 
	 263,  262,  260,  259,  258,  257,  126,  123,   45,   43, 
	  42,   40,   38,   33, -287,   59, -330,   40, -315, -320, 
	-330, -319, -318, -317, -287, -266, -265, -316, -249, -250, 
	-251, -321, -324, -323, -263, -262, -261, -260, -258, -257, 
	-256, -255, -254, -253, -252,  315,  314,  313,  312,  311, 
	 310,  309,  308,  306,  305,  304,  263,  262,  260,  259, 
	 258,  257,  126,  125,  123,   59,   45,   43,   42,   40, 
	  38,   33, -299,   58, -249, -264,  302,  257, -330, -329, 
	-319, -328, -249, -267, -286, -285, -284, -283, -282, -281, 
	-280, -279, -278, -277, -276, -275, -274, -273, -272, -271, 
	-270, -269, -268,  301,  300,  299,  298,  297,  296,  295, 
	 294,  293,  292,  291,  290,  289,  288,  287,  286,  285, 
	 284,  283,  282,  257,   91,   42,   41,   40, -330, -328, 
	-249,  257,   91,   40, -267, -286, -285, -284, -283, -282, 
	-281, -280, -279, -278, -277, -276, -275, -274, -273, -272, 
	-271, -270, -269, -268, -264,  302,  301,  300,  299,  298, 
	 297,  296,  295,  294,  293,  292,  291,  290,  289,  288, 
	 287,  286,  285,  284,  283,  282, -322, -299,   58,   44, 
	-322, -327,   93,   44, -330, -329, -319, -328, -267, -286, 
	-285, -284, -283, -282, -281, -280, -279, -278, -277, -276, 
	-275, -274, -273, -272, -271, -270, -269, -268,  301,  300, 
	 299,  298,  297,  296,  295,  294,  293,  292,  291,  290, 
	 289,  288,  287,  286,  285,  284,  283,  282,   91,   42, 
	  41,   40, -329, -267, -286, -285, -284, -283, -282, -281, 
	-280, -279, -278, -277, -276, -275, -274, -273, -272, -271, 
	-270, -269, -268,  301,  300,  299,  298,  297,  296,  295, 
	 294,  293,  292,  291,  290,  289,  288,  287,  286,  285, 
	 284,  283,  282,   41, -330, -319, -299, -249,  257,   58, 
	  42,   40, -315, -320, -330, -319, -318, -317, -266, -265, 
	-316, -249, -250, -251, -321, -324, -323,  263,  262,  260, 
	 259,  258,  257,  126,  125,  123,   45,   43,   42,   40, 
	  38,   33, -315, -320, -330, -319, -318, -317, -287, -316, 
	-249, -250, -251, -321, -324, -323,  263,  262,  260,  259, 
	 258,  257,  126,   59,   45,   43,   42,   40,   38,   33, 
	-258,  309, -259,  307,   -1
};
static short yypact[] = {
	  23,   23,   62,   90,   96,   96,  104,   62,   90,  148, 
	 178,  178,  218,  198,  226,  229,  231,  233,  235,  238, 
	 244,  250,  254,  259,  294,  357,  359,  198,  450,  451, 
	 456,  451,  460,  478,  507,  275,  683,   90,  741,  233, 
	 235,  238,  244,  244,  250,  250,  250,  250,  254,  254, 
	 259,  259,   90,  683,  451,  275,   90,  229,  231,  275, 
	 275,  953,  422,  422,  625,  422,  520,  147,  275,  147, 
	 422,  422,  422,  627,  220,  422,  951,  936,  907,  275, 
	 223,  147,  422,  422,  520,  520,  520,  625,  362,  888, 
	 275,  520,  275,  147,  147,  162,  863,  223,  683,  275, 
	 275,  275,  422,  625,  625,  655,  610,  580,  220,  275, 
	 275,  275,  275,  818,  792,  147,  275,  147,  275,  275, 
	 788,  147,  765,  713,  162,  147,  686,  220,  275,  683, 
	 655,  627,  627,  627,  627,  177,  625,  625,  449,  422, 
	 610,  381,  580,  545,  524,  275,  520,  147,  275,  177, 
	 177,  275,  328,  275,  275,  275,  275,  275,  275,  275, 
	 275,  275,  275,  275,  275,  275,  275,  275,  275,  275, 
	 449,  422,   81,  359,  125,  365,  381,  365,  362,  328, 
	 275,  275,  275,  275,  223,  147,  198,  177,  100,  125, 
	 220,  198,  198,  177,  162,  147,  125,  100,   94,   94, 
	  86,   81
};
static short yygo[] = {
	  -7, -202, -202,  -10,  -10,  -33, -124, -114,  -97, -124, 
	-114,  -97,  -69, -202, -114, -124, -100, -101, -102,  -56, 
	-202, -202, -152, -152, -153, -202, -202, -202, -202, -180, 
	 201,  200,  197,  188,  183,  181,  180,  172,  143,  134, 
	 133,  132,  131,  123,  113,   89,   73,   56,   52,   38, 
	  37,   33,   30,   24,    8,    3,    1,    0,  -65, -206, 
	-208, -440, -219, -217, -213, -339, -432, -438, -439, -338, 
	-350, -417, -418, -333, -113, -416, -413, -437,  195,  185, 
	 147,  146,  125,  121,  117,  115,   96,   94,   93,   91, 
	  86,   85,   84,   81,   69,   67,   66, -195, -195, -149, 
	 -96,  -96, -125,   56,   37,   24,    8,    3, -436, -435, 
	-434, -337, -433, -415, -414,  184,  124,  114,   97,   95, 
	  80, -150, -151, -342, -181,   24, -343, -182,   24, -127, 
	-123,  -79,  -80,  -90, -145, -173, -111,  190,  178,  108, 
	  91,   88,   31,   29, -184, -167, -167, -351,   40,   18, 
	  -3,   -3,   -3,   -3, -154,   -3,   -3, -154, -154,   -3, 
	  -3,   -3,   -3,   -3,   -3,   -3,   -3, -352,  201,  197, 
	 188,  172,  143,  123,  113,   89,   51,   50,   33,   30, 
	  23,    7,    2,    1,    0, -157, -157, -157, -353,   49, 
	  48,   22, -158, -158, -158, -354,   49,   48,   22, -355, 
	-356, -155, -156, -159, -160, -161, -162, -163, -164, -165, 
	-166, -168, -169, -248, -247, -170, -226,  -91, -224,  -93, 
	-223, -110,  129,  120,   98,   53,   36, -365, -141, -146, 
	 173,   34, -366, -367, -368, -369, -370, -371, -372, -373, 
	-374, -375, -209,  -61,  -60,  -78, -459, -462, -236, -220, 
	-460, -461, -245, -244, -245, -221,  197,  190,  188,  137, 
	 136,  127,  108,  104,  103,   87,   77,   74,   64, -240, 
	-381, -382, -383, -384, -385, -386, -387, -388, -389, -390, 
	-391, -392, -393, -394, -395, -238, -237, -234, -398, -107, 
	-107, -107, -142, -194, -239, -194, -222,  199,  198,  177, 
	 175,  140,  106,   78, -443, -442, -453, -399, -451, -452, 
	-400, -409, -450, -408,  170,  144,  142,  138,  130,  107, 
	 105,   88,   78, -426, -423,  122, -129, -130, -132, -133, 
	-215,  -74, -134,   76, -211, -135, -136, -137, -138,  -36, 
	-335, -334,   25, -332, -233, -233, -233, -233,  -37,  -37, 
	 -37,  -37,  -37,  -37,  -37,  -37,  -37, -233,  -37,  -37, 
	-233, -425,  -37, -104,  -37, -233,  -27, -340, -341,  -37, 
	-233, -424, -233,  -27, -233,  -12,  -11, -233, -233, -331, 
	 201,  200,  199,  198,  197,  193,  188,  187,  172,  171, 
	 150,  149,  144,  143,  139,  135,  130,  126,  123,  105, 
	 102,   89,   83,   82,   75,   72,   71,   70,   65,   63, 
	  62,   38,   30,    1,    0, -336,  -26,  -55,  -55,  -55, 
	 -55,  -67,  -75,  -85,  -86,  -87, -115, -147, -147, -121, 
	-147, -128,  179,  169,  152,  151,  148,  101,  100,   99, 
	  77,   68,   60,   59,   55,   35,  -25, -116, -117,  -92, 
	-441, -345, -441, -377, -364, -344, -441, -376,  140,  116, 
	 111,  110,  106,   79,   78, -357, -357, -357, -357, -357, 
	-357, -357, -357, -357, -357, -357, -357, -357, -357, -357, 
	-357, -357, -357, -357, -357, -357, -357, -357, -357, -357, 
	-357, -346, -347, -357, -349, -357,  -35,  194,  183,  182, 
	 181,  180,  168,  167,  166,  165,  164,  163,  162,  161, 
	 160,  159,  158,  157,  156,  155,  154,  153,  145,  128, 
	 124,  119,  118,  112,  109,   95,   92,   90, -183, -358, 
	-360, -361, -362, -348, -359,  182,  155,  154,  153,  112, 
	-118, -148,  152,  -52,  -51,  -24,  157,  156,  -50,  -49, 
	 -23,  159,  158,  -48,  -47,  -46,  -45,  -22,  163,  162, 
	 161,  160,  -44,  -43,  -21,  165,  164,  -42,  -20,  166, 
	 -41,  -19,  167,  -40,  -18,  168,  -59,  -17,  118,  -58, 
	 -16,  119, -119,  -15, -120, -378, -246, -378, -378, -378, 
	-378, -378, -378, -363,  194,  145,  128,  124,  109,   95, 
	  92,   90, -112, -406,  -81, -405,  -98,  -99, -412, -185, 
	 145,  128,  124,  109,   95,   90, -465, -465, -455, -455, 
	-454,  189,   27,    1,    0, -198, -198,  -31,  -31,  -31, 
	 -31,  -31, -243, -242, -189,  192,  191,  186,  123,  122, 
	 113,   96,    1,    0, -191,   -6, -192, -421, -422, -422, 
	-421, -421, -421, -422, -421, -421, -421,   -5,  179,  176, 
	 152,  143,  142,  141,  107,   33,    7,    2, -193, -380, 
	-379,  172, -429,  -54, -196,  -54,  -29,  -29,  -13, -196, 
	-203,  201,  197,  188,  172,  143,  123,   89,   30, -174, 
	-445, -444, -241,  106,   78, -396, -397, -199, -176, -108, 
	-143,  141, -177, -402, -402, -401,  142,  107,   -8,  -34, 
	 -34, -144,  179,  152,    2, -109, -404, -403,   89, -200, 
	-407, -235,  177, -178, -179, -411, -410,  144,   -9,   -9, 
	  -4,  200,   38, -419, -420,  -39,  -53,  -53,  -39, -201, 
	 123,  113,   33,   30,    7,    2, -186, -187,  -82, -232, 
	 -95,  186,   96, -122, -188, -231, -126,  -30,  -32, -428, 
	-427,  122, -430, -431,  -94,   33,   30,  -57,  -57,  -38, 
	  52,   38,  -89, -214, -205, -207, -218, -216, -212, -448, 
	-225, -449, -210, -457, -457, -456,  130,  105,  102,   83, 
	  82,   75,   72,   71,   70,   65,   63,   62, -446, -468, 
	-469, -468, -230,  196,  189,  174, -447, -229, -228, -227, 
	 -83,  -76,  -84,  -14, -171, -172, -106, -131,  139,  -28, 
	-190,   13, -139, -140,  -71,  -62, -458,  -72,  -63,  -73, 
	-103,  -77,  -88,  -70,  -68, -105,   60,   59,   55,  -66, 
	 -64,   -2, -464, -463,    1, -204, -197, -467, -466,  174, 
	-175,   -1
};
static short yypgo[] = {
	   0,    0,    0,  447,  448,  646,  668,  689,  698,  695, 
	 723,  696,  746,  747,  754,  814,  815,  822,  823,  850, 
	 846,  843,  808,  840,  808,  839,  808,  831,  830,  808, 
	 829,  826,  828,  807,  827,  825,  824,  806,  806,  813, 
	 812,  811,  798,  810,  785,  785,  785,  785,  756,  753, 
	 730,  719,  696,  705,  697,  697,  702,  645,  670,  634, 
	 634,  620,  620,  593,  584,  582,  379,  343,  341,  339, 
	 338,  337,  336,  335,  334,  332,  330,  329,  328,  327, 
	 326,  324,  313,  296,  288,  287,  286,  285,  284,  283, 
	 282,  281,  280,  279,  278,  277,  276,  275,  274,  273, 
	 272,  271,  270,  269,  255,  241,  240,  239,  238,  237, 
	 236,  235,  234,  233,  232,  229,  221,  215,  214,  213, 
	 212,  211,  210,  209,  208,  207,  206,  205,  204,  203, 
	 202,  201,  200,  199,  195,  188,  167,  147,  144,  136, 
	 127,  124,  122,  121,  114,  102,   77,   29,  415,  415, 
	 415,  416,  416,  446,  446,  446,  446,  446,  446,  446, 
	 446,  449,  449,  496,  496,  496,  496,  496,  528,  528, 
	 528,  528,  528,  528,  534,  534,  545,  545,  545,  545, 
	 457,  457,  602,  602,  602,  602,  602,  602,  602,  602, 
	 602,  602,  602,  431,  431,  609,  644,  644,  645,  645, 
	 645,  645,  657,  657,  657,  657,  657,  657,  657,  657, 
	 657,  657,  657,  657,  657,  657,  695,  695,  700,  700, 
	 715,  715,  717,  717,  696,  721,  721,  724,  724,  726, 
	 730,  730,  730,  730,  730,  730,  739,  739,  711,  711, 
	 755,  757,  757,  750,  758,  758,  760,  760,  541,  769, 
	 769,  769,  769,  769,  769,  769,  769,  769,  692,  692, 
	 692,  772,  772,  785,  785,  798,  798,  802,  802,  802, 
	 802,  820,  820,  817,  817,  807,  809,  809,  809,  809, 
	 841,  841,  843,  845,  845,  848,  848,    0,    0,  835, 
	 835,  826,  764,  764,  764,  541,  760,  750,  755,  739, 
	 739,  680,  680,  726,  717,  670,  634,  634,  593,  583, 
	 583,  580,  580,  577,  577,  574,  574,  571,  571,  568, 
	 568,  568,  564,  564,  564,  564,  564,  557,  557,  557, 
	 550,  550,  550,  496,  415,    0
};
static short yyrlen[] = {
	   0,    0,    0,    0,    0,    0,    0,    0,    0,    2, 
	   0,    2,    0,    0,    0,    0,    0,    0,    0,    0, 
	   0,    1,   10,    0,    9,    0,    9,    0,    0,    6, 
	   0,    3,    0,    6,    0,    0,    0,    2,    1,    1, 
	   0,    0,    4,    0,    1,    1,    1,    1,    0,    0, 
	   1,    1,    2,    3,    1,    1,    0,    1,    4,    3, 
	   3,    3,    2,    5,    0,    0,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   3,    1,    2,    1,    4,    4,    5,    3,    3,    2, 
	   2,    1,    3,    2,    2,    2,    2,    4,    1,    1, 
	   1,    1,    1,    1,    1,    4,    1,    3,    3,    3, 
	   1,    3,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    3,    1,    1,    3,    1,    1, 
	   1,    1,    1,    1,    1,    1,    1,    1,    1,    1, 
	   1,    1,    1,    1,    1,    1,    6,    5,    1,    2, 
	   1,    3,    2,    3,    4,    3,    4,    1,    3,    3, 
	   3,    3,    4,    4,    6,    6,    2,    3,    1,    2, 
	   3,    1,    3,    3,    1,    3,    2,    2,    2,    3, 
	   2,    3,    3,    4,    2,    3,    3,    4,    1,    3, 
	   4,    1,    3,    1,    1,    5,    4,    3,    4,    4, 
	   5,    1,    2,    1,    2,    8,    3,    2,    2,    3, 
	   1,    2,    1,    3,    4,    1,    2,    1,    0,    1, 
	   0,    0,    2,    1,    1,    1,    1,    1,    1,    2, 
	   1,    2,    1,    1,    1,    1,    1,    1,    1,    4, 
	   1,    4,    1,    3,    1,    3,    1,    3,    1,    3, 
	   3,    1,    3,    3,    3,    3,    1,    3,    3,    1, 
	   3,    3,    1,    1,    1,    2
};
#define YYS0	0
#define YYDELTA	182
#define YYNPACT	202
#define YYNDEF	62

#define YYr333	0
#define YYr334	1
#define YYr335	2
#define YYr91	3
#define YYr93	4
#define YYr166	5
#define YYr169	6
#define YYr174	7
#define YYr195	8
#define YYr199	9
#define YYr211	10
#define YYr213	11
#define YYr227	12
#define YYr229	13
#define YYr232	14
#define YYr283	15
#define YYr285	16
#define YYr287	17
#define YYr289	18
#define YYr329	19
#define YYr327	20
#define YYr325	21
#define YYr314	22
#define YYr313	23
#define YYr312	24
#define YYr311	25
#define YYr310	26
#define YYr309	27
#define YYr308	28
#define YYr307	29
#define YYr306	30
#define YYr305	31
#define YYr304	32
#define YYr302	33
#define YYr301	34
#define YYr299	35
#define YYr298	36
#define YYr297	37
#define YYr296	38
#define YYr291	39
#define YYr281	40
#define YYr279	41
#define YYr278	42
#define YYr277	43
#define YYr276	44
#define YYr275	45
#define YYr274	46
#define YYr272	47
#define YYr233	48
#define YYr230	49
#define YYr223	50
#define YYr216	51
#define YYr210	52
#define YYr204	53
#define YYr201	54
#define YYr200	55
#define YYr197	56
#define YYr176	57
#define YYr175	58
#define YYr170	59
#define YYr167	60
#define YYr164	61
#define YYr163	62
#define YYr146	63
#define YYr143	64
#define YYr140	65
#define YYr82	66
#define YYr81	67
#define YYr80	68
#define YYr79	69
#define YYr78	70
#define YYr77	71
#define YYr76	72
#define YYr75	73
#define YYr74	74
#define YYr73	75
#define YYr72	76
#define YYr71	77
#define YYr70	78
#define YYr69	79
#define YYr68	80
#define YYr67	81
#define YYr66	82
#define YYr65	83
#define YYr64	84
#define YYr63	85
#define YYr62	86
#define YYr61	87
#define YYr60	88
#define YYr59	89
#define YYr58	90
#define YYr57	91
#define YYr56	92
#define YYr55	93
#define YYr54	94
#define YYr53	95
#define YYr52	96
#define YYr51	97
#define YYr50	98
#define YYr49	99
#define YYr48	100
#define YYr47	101
#define YYr46	102
#define YYr45	103
#define YYr44	104
#define YYr43	105
#define YYr42	106
#define YYr41	107
#define YYr40	108
#define YYr39	109
#define YYr38	110
#define YYr37	111
#define YYr36	112
#define YYr35	113
#define YYr34	114
#define YYr33	115
#define YYr32	116
#define YYr31	117
#define YYr30	118
#define YYr29	119
#define YYr28	120
#define YYr27	121
#define YYr26	122
#define YYr25	123
#define YYr24	124
#define YYr23	125
#define YYr22	126
#define YYr21	127
#define YYr20	128
#define YYr19	129
#define YYr18	130
#define YYr17	131
#define YYr16	132
#define YYr15	133
#define YYr14	134
#define YYr13	135
#define YYr12	136
#define YYr11	137
#define YYr10	138
#define YYr9	139
#define YYr8	140
#define YYr7	141
#define YYr6	142
#define YYr5	143
#define YYr4	144
#define YYr3	145
#define YYr2	146
#define YYr1	147
#define YYrACCEPT	YYr333
#define YYrERROR	YYr334
#define YYrLR2	YYr335
#line 2 "c:/mks/etc/yyparse.c"

/*
 * Copyright 1985, 1990 by Mortice Kern Systems Inc.  All rights reserved.
 * 
 * Automaton to interpret LALR(1) tables.
 *
 *	Macros:
 *		yyclearin - clear the lookahead token.
 *		yyerrok - forgive a pending error
 *		YYERROR - simulate an error
 *		YYACCEPT - halt and return 0
 *		YYABORT - halt and return 1
 *		YYRETURN(value) - halt and return value.  You should use this
 *			instead of return(value).
 *		YYREAD - ensure yychar contains a lookahead token by reading
 *			one if it does not.  See also YYSYNC.
 *		YYRECOVERING - 1 if syntax error detected and not recovered
 *			yet; otherwise, 0.
 *
 *	Preprocessor flags:
 *		YYDEBUG - includes debug code.  The parser will print
 *			 a travelogue of the parse if this is defined
 *			 and yydebug is non-zero.
 *		YYSSIZE - size of state and value stacks (default 150).
 *		YYSTATIC - By default, the state stack is an automatic array.
 *			If this is defined, the stack will be static.
 *			In either case, the value stack is static.
 *		YYALLOC - Dynamically allocate both the state and value stacks
 *			by calling malloc() and free().
 *		YYLR2 - defined if lookahead is needed to resolve R/R or S/R conflicts
 *		YYSYNC - if defined, yacc guarantees to fetch a lookahead token
 *			before any action, even if it doesnt need it for a decision.
 *			If YYSYNC is defined, YYREAD will never be necessary unless
 *			the user explicitly sets yychar = -1
 *
 *	Copyright (c) 1983, by the University of Waterloo
 */

/* GENTEXT: yyerror */
#ifndef I18N
#define	gettext(x)	x
#endif

#ifndef YYSSIZE
# define YYSSIZE	150
#endif

#define YYERROR		goto yyerrlabel
#define yyerrok		yyerrflag = 0
#define yyclearin	yychar = -1
#define YYACCEPT	YYRETURN(0)
#define YYABORT		YYRETURN(1)
#define YYRECOVERING()	(yyerrflag != 0)
#ifdef YYALLOC
# define YYRETURN(val)	{ retval = (val); goto yyReturn; }
#else
# define YYRETURN(val)	return(val)
#endif
#ifdef YYDEBUG
/* The if..else makes this macro behave exactly like a statement */
# define YYREAD	if (yychar < 0) {					\
			if ((yychar = yylex()) < 0)			\
				yychar = 0;				\
			if (yydebug)					\
				printf(gettext("read %s (%d)\n"), 	\
				yyptok(yychar),				\
				yychar);				\
		} else
#else
# define YYREAD	if (yychar < 0) {					\
			if ((yychar = yylex()) < 0)			\
				yychar = 0;				\
		} else
#endif
#define YYERRCODE	256		/* value of `error' */
#if 0 && defined(__TURBOC__) && __SMALL__
	/* THIS ONLY WORKS ON TURBO C 1.5 !!! */
#define	YYQYYP	*(int *)((int)yyq + ((int)yyq-(int)yyp))
#else
#define	YYQYYP	yyq[yyq-yyp]
#endif

YYSTYPE	yyval;				/* $$ */
YYSTYPE	*yypvt;				/* $n */
YYSTYPE	yylval;				/* yylex() sets this */

int	yychar,				/* current token */
	yyerrflag,			/* error flag */
	yynerrs;			/* error count */

#ifdef YYDEBUG
int yydebug = 0;		/* debug flag & tables */
extern char	*yysvar[], *yystoken[], *yyptok();
extern short	yyrmap[], yysmap[];
extern int	yynstate, yynvar, yyntoken, yynrule;
# define yyassert(condition, msg, arg) \
	if (!(condition)) { printf(gettext("\nyacc bug: ")); printf(msg, arg); YYABORT; }
#else /* !YYDEBUG */
# define yyassert(condition, msg, arg)
#endif

yyparse()
{

	register short		yyi, *yyp;	/* for table lookup */
	register short		*yyps;		/* top of state stack */
	register short		yystate;	/* current state */
	register YYSTYPE	*yypv;		/* top of value stack */
	register short		*yyq;
	register int		yyj;

#ifdef YYSTATIC
	static short	yys[YYSSIZE + 1];
	static YYSTYPE	yyv[YYSSIZE + 1];
#else
#ifdef YYALLOC
	YYSTYPE *yyv;
	short	*yys;
	YYSTYPE save_yylval;
	YYSTYPE save_yyval;
	YYSTYPE *save_yypvt;
	int save_yychar, save_yyerrflag, save_yynerrs;
	int retval;
#if 0	/* defined in <stdlib.h>*/
	extern char	*malloc();
#endif
#else
	short		yys[YYSSIZE + 1];
	static YYSTYPE	yyv[YYSSIZE + 1];	/* historically static */
#endif
#endif

#ifdef YYALLOC
	yys = (short *) malloc((YYSSIZE + 1) * sizeof(short));
	yyv = (YYSTYPE *) malloc((YYSSIZE + 1) * sizeof(YYSTYPE));
	if (yys == (short *)0 || yyv == (YYSTYPE *)0) {
		yyerror("Not enough space for parser stacks");
		return 1;
	}
	save_yylval = yylval;
	save_yyval = yyval;
	save_yypvt = yypvt;
	save_yychar = yychar;
	save_yyerrflag = yyerrflag;
	save_yynerrs = yynerrs;
#endif

	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;
	yyps = yys;
	yypv = yyv;
	yystate = YYS0;		/* start state */

yyStack:
	yyassert((unsigned)yystate < yynstate, gettext("state %d\n"), yystate);
	if (++yyps > &yys[YYSSIZE]) {
		yyerror("Parser stack overflow");
		YYABORT;
	}
	*yyps = yystate;	/* stack current state */
	*++yypv = yyval;	/* ... and value */

#ifdef YYDEBUG
	if (yydebug)
		printf(gettext("state %d (%d), char %s (%d)\n"),yysmap[yystate],
			yystate, yyptok(yychar), yychar);
#endif

	/*
	 *	Look up next action in action table.
	 */
yyEncore:
#ifdef YYSYNC
	YYREAD;
#endif
	if (yystate >= sizeof yypact/sizeof yypact[0]) 	/* simple state */
		yyi = yystate - YYDELTA;	/* reduce in any case */
	else {
		if(*(yyp = &yyact[yypact[yystate]]) >= 0) {
			/* Look for a shift on yychar */
#ifndef YYSYNC
			YYREAD;
#endif
			yyq = yyp;
			yyi = yychar;
#if 0 && defined(__TURBOC__) && __SMALL__
	/* THIS ONLY WORKS ON TURBO C 1.5 !!! */
			/* yyi is in di, yyp is in si */
		L01:
			asm lodsw	/* ax = *yyp++; */
			asm cmp yyi, ax
			asm jl L01
#else
			while (yyi < *yyp++)
				;
#endif
			if (yyi == yyp[-1]) {
				yystate = ~YYQYYP;
#ifdef YYDEBUG
				if (yydebug)
					printf(gettext("shift %d (%d)\n"),
						yysmap[yystate], yystate);
#endif
				yyval = yylval;		/* stack what yylex() set */
				yychar = -1;		/* clear token */
				if (yyerrflag)
					yyerrflag--;	/* successful shift */
				goto yyStack;
			}
		}

		/*
	 	 *	Fell through - take default action
	 	 */

		if (yystate >= sizeof yydef /sizeof yydef[0])
			goto yyError;
		if ((yyi = yydef[yystate]) < 0)	 { /* default == reduce? */
											/* Search exception table */
			yyassert((unsigned)~yyi < sizeof yyex/sizeof yyex[0],
				gettext("exception %d\n"), yystate);
			yyp = &yyex[~yyi];
#ifndef YYSYNC
			YYREAD;
#endif
			while((yyi = *yyp) >= 0 && yyi != yychar)
				yyp += 2;
			yyi = yyp[1];
			yyassert(yyi >= 0,
				 gettext("Ex table not reduce %d\n"), yyi);
		}
	}

#ifdef YYLR2
yyReduce:	/* reduce yyi */
#endif
	yyassert((unsigned)yyi < yynrule, gettext("reduce %d\n"), yyi);
	yyj = yyrlen[yyi];
#ifdef YYDEBUG
	if (yydebug) printf(gettext("reduce %d (%d), pops %d (%d)\n"), 
		yyrmap[yyi], yyi, yysmap[yyps[-yyj]], yyps[-yyj]);
#endif
	yyps -= yyj;		/* pop stacks */
	yypvt = yypv;		/* save top */
	yypv -= yyj;
	yyval = yypv[1];	/* default action $$ = $1 */
	switch (yyi) {		/* perform semantic action */
		
case YYr1: {	/* tk_lparen :  '(' */
#line 282 "gram.y"
 fire_operator(); 
} break;

case YYr2: {	/* tk_rparen :  ')' */
#line 287 "gram.y"
 fire_operator(); 
} break;

case YYr3: {	/* tk_lsquare_brace :  '[' */
#line 292 "gram.y"
 fire_operator(); 
} break;

case YYr4: {	/* tk_rsquare_brace :  ']' */
#line 297 "gram.y"
 fire_operator(); 
} break;

case YYr5: {	/* tk_dot :  '.' */
#line 302 "gram.y"
 fire_operator(); 
} break;

case YYr6: {	/* tk_ptr_to :  TK_PTR_OP */
#line 307 "gram.y"
 fire_operator(); 
} break;

case YYr7: {	/* tk_inc :  TK_INC_OP */
#line 312 "gram.y"
 fire_operator(); 
} break;

case YYr8: {	/* tk_dec :  TK_DEC_OP */
#line 317 "gram.y"
 fire_operator(); 
} break;

case YYr9: {	/* tk_comma :  ',' */
#line 322 "gram.y"
 fire_operator(); 
} break;

case YYr10: {	/* tk_sizeof :  TK_SIZEOF */
#line 328 "gram.y"
 fire_keyword(); 
} break;

case YYr11: {	/* tk_ampersand :  '&' */
#line 333 "gram.y"
 fire_operator(); 
} break;

case YYr12: {	/* tk_asterisk :  '*' */
#line 338 "gram.y"
 fire_operator(); 
} break;

case YYr13: {	/* tk_plus :  '+' */
#line 343 "gram.y"
 fire_operator(); 
} break;

case YYr14: {	/* tk_minus :  '-' */
#line 348 "gram.y"
 fire_operator(); 
} break;

case YYr15: {	/* tk_tilde :  '~' */
#line 353 "gram.y"
 fire_operator(); 
} break;

case YYr16: {	/* tk_exclamation :  '!' */
#line 358 "gram.y"
 fire_operator(); 
} break;

case YYr17: {	/* tk_slash :  '/' */
#line 363 "gram.y"
 fire_operator(); 
} break;

case YYr18: {	/* tk_percent :  '%' */
#line 368 "gram.y"
 fire_operator(); 
} break;

case YYr19: {	/* tk_left :  TK_LEFT_OP */
#line 373 "gram.y"
 fire_operator(); 
} break;

case YYr20: {	/* tk_right :  TK_RIGHT_OP */
#line 378 "gram.y"
 fire_operator(); 
} break;

case YYr21: {	/* tk_lt :  '<' */
#line 383 "gram.y"
 fire_operator(); 
} break;

case YYr22: {	/* tk_gt :  '>' */
#line 388 "gram.y"
 fire_operator(); 
} break;

case YYr23: {	/* tk_le :  TK_LE_OP */
#line 393 "gram.y"
 fire_operator(); 
} break;

case YYr24: {	/* tk_ge :  TK_GE_OP */
#line 398 "gram.y"
 fire_operator(); 
} break;

case YYr25: {	/* tk_eq :  TK_EQ_OP */
#line 403 "gram.y"
 fire_operator(); 
} break;

case YYr26: {	/* tk_ne :  TK_NE_OP */
#line 408 "gram.y"
 fire_operator(); 
} break;

case YYr27: {	/* tk_circumflex :  '^' */
#line 413 "gram.y"
 fire_operator(); 
} break;

case YYr28: {	/* tk_bar :  '|' */
#line 418 "gram.y"
 fire_operator(); 
} break;

case YYr29: {	/* tk_land :  TK_AND_OP */
#line 423 "gram.y"
 fire_operator(); 
} break;

case YYr30: {	/* tk_lor :  TK_OR_OP */
#line 428 "gram.y"
 fire_operator(); 
} break;

case YYr31: {	/* tk_question :  '?' */
#line 433 "gram.y"
 fire_operator(); 
} break;

case YYr32: {	/* tk_colon :  ':' */
#line 438 "gram.y"
 fire_operator(); 
} break;

case YYr33: {	/* tk_assign :  '=' */
#line 443 "gram.y"
 fire_operator(); 
} break;

case YYr34: {	/* tk_mul_assign :  TK_MUL_ASSIGN */
#line 448 "gram.y"
 fire_operator(); 
} break;

case YYr35: {	/* tk_div_assign :  TK_DIV_ASSIGN */
#line 453 "gram.y"
 fire_operator(); 
} break;

case YYr36: {	/* tk_mod_assign :  TK_MOD_ASSIGN */
#line 458 "gram.y"
 fire_operator(); 
} break;

case YYr37: {	/* tk_add_assign :  TK_ADD_ASSIGN */
#line 463 "gram.y"
 fire_operator(); 
} break;

case YYr38: {	/* tk_sub_assign :  TK_SUB_ASSIGN */
#line 468 "gram.y"
 fire_operator(); 
} break;

case YYr39: {	/* tk_left_assign :  TK_LEFT_ASSIGN */
#line 473 "gram.y"
 fire_operator(); 
} break;

case YYr40: {	/* tk_right_assign :  TK_RIGHT_ASSIGN */
#line 478 "gram.y"
 fire_operator(); 
} break;

case YYr41: {	/* tk_and_assign :  TK_AND_ASSIGN */
#line 483 "gram.y"
 fire_operator(); 
} break;

case YYr42: {	/* tk_xor_assign :  TK_XOR_ASSIGN */
#line 488 "gram.y"
 fire_operator(); 
} break;

case YYr43: {	/* tk_or_assign :  TK_OR_ASSIGN */
#line 493 "gram.y"
 fire_operator(); 
} break;

case YYr44: {	/* tk_semicolon :  ';' */
#line 498 "gram.y"
 fire_operator(); 
} break;

case YYr45: {	/* tk_typedef :  TK_TYPEDEF */
#line 503 "gram.y"
 fire_keyword(); 
} break;

case YYr46: {	/* tk_extern :  TK_EXTERN */
#line 508 "gram.y"
 fire_keyword(); 
} break;

case YYr47: {	/* tk_static :  TK_STATIC */
#line 513 "gram.y"
 fire_keyword(); 
} break;

case YYr48: {	/* tk_auto :  TK_AUTO */
#line 518 "gram.y"
 fire_keyword(); 
} break;

case YYr49: {	/* tk_register :  TK_REGISTER */
#line 523 "gram.y"
 fire_keyword(); 
} break;

case YYr50: {	/* tk_char :  TK_CHAR */
#line 528 "gram.y"
 fire_keyword(); 
} break;

case YYr51: {	/* tk_short :  TK_SHORT */
#line 533 "gram.y"
 fire_keyword(); 
} break;

case YYr52: {	/* tk_int :  TK_INT */
#line 538 "gram.y"
 fire_keyword(); 
} break;

case YYr53: {	/* tk_long :  TK_LONG */
#line 543 "gram.y"
 fire_keyword(); 
} break;

case YYr54: {	/* tk_signed :  TK_SIGNED */
#line 548 "gram.y"
 fire_keyword(); 
} break;

case YYr55: {	/* tk_unsigned :  TK_UNSIGNED */
#line 553 "gram.y"
 fire_keyword(); 
} break;

case YYr56: {	/* tk_float :  TK_FLOAT */
#line 558 "gram.y"
 fire_keyword(); 
} break;

case YYr57: {	/* tk_double :  TK_DOUBLE */
#line 563 "gram.y"
 fire_keyword(); 
} break;

case YYr58: {	/* tk_const :  TK_CONST */
#line 568 "gram.y"
 fire_keyword(); 
} break;

case YYr59: {	/* tk_volatile :  TK_VOLATILE */
#line 573 "gram.y"
 fire_keyword(); 
} break;

case YYr60: {	/* tk_void :  TK_VOID */
#line 578 "gram.y"
 fire_keyword(); 
} break;

case YYr61: {	/* tk_struct :  TK_STRUCT */
#line 583 "gram.y"
 fire_keyword(); 
} break;

case YYr62: {	/* tk_union :  TK_UNION */
#line 588 "gram.y"
 fire_keyword(); 
} break;

case YYr63: {	/* tk_enum :  TK_ENUM */
#line 593 "gram.y"
 fire_keyword(); 
} break;

case YYr64: {	/* tk_type_name :  TK_TYPE_NAME */
#line 598 "gram.y"
 fire_type_name(); 
} break;

case YYr65: {	/* tk_lcurly_brace :  '{' */
#line 603 "gram.y"
 fire_operator(); 
} break;

case YYr66: {	/* tk_rcurly_brace :  '}' */
#line 608 "gram.y"
 fire_operator(); 
} break;

case YYr67: {	/* tk_elipsis :  TK_ELIPSIS */
#line 613 "gram.y"
 fire_operator(); 
} break;

case YYr68: {	/* tk_case :  TK_CASE */
#line 618 "gram.y"
 fire_keyword(); 
} break;

case YYr69: {	/* tk_default :  TK_DEFAULT */
#line 623 "gram.y"
 fire_keyword(); 
} break;

case YYr70: {	/* tk_if :  TK_IF */
#line 628 "gram.y"
 fire_keyword(); 
} break;

case YYr71: {	/* tk_switch :  TK_SWITCH */
#line 633 "gram.y"
 fire_keyword(); 
} break;

case YYr72: {	/* tk_else :  TK_ELSE */
#line 638 "gram.y"
 fire_keyword(); 
} break;

case YYr73: {	/* tk_while :  TK_WHILE */
#line 643 "gram.y"
 fire_keyword(); 
} break;

case YYr74: {	/* tk_do :  TK_DO */
#line 648 "gram.y"
 fire_keyword(); 
} break;

case YYr75: {	/* tk_for :  TK_FOR */
#line 653 "gram.y"
 fire_keyword(); 
} break;

case YYr76: {	/* tk_goto :  TK_GOTO */
#line 658 "gram.y"
 fire_keyword(); 
} break;

case YYr77: {	/* tk_continue :  TK_CONTINUE */
#line 663 "gram.y"
 fire_keyword(); 
} break;

case YYr78: {	/* tk_break :  TK_BREAK */
#line 668 "gram.y"
 fire_keyword(); 
} break;

case YYr79: {	/* tk_return :  TK_RETURN */
#line 673 "gram.y"
 fire_keyword(); 
} break;

case YYr80: {	/* tk_string :  TK_STRING_LITERAL */
#line 678 "gram.y"
 found_string(); 
} break;

case YYr81: {	/* tk_constant :  TK_CONSTANT */
#line 683 "gram.y"
 found_constant(); 
} break;

case YYr82: {	/* identifier :  TK_IDENTIFIER */
#line 688 "gram.y"

         
         fire_identifier();
         
         strcpy(last_ident, token());
      
} break;

case YYr91: {	/* postfix_expr :  postfix_expr tk_lparen */
#line 716 "gram.y"
 function_call(); 
} break;

case YYr93: {	/* postfix_expr :  postfix_expr tk_lparen */
#line 719 "gram.y"
 function_call(); 
} break;

case YYr140: {	/* logical_and_expr :  logical_and_expr tk_land */
#line 807 "gram.y"
 ++fcn_conditions; 
} break;

case YYr143: {	/* logical_or_expr :  logical_or_expr tk_lor */
#line 815 "gram.y"
 ++fcn_conditions; 
} break;

case YYr146: {	/* conditional_expr :  logical_or_expr tk_question expr tk_colon conditional_expr */
#line 826 "gram.y"
 ++fcn_decisions; ++fcn_conditions; 
} break;

case YYr163: {	/* declaration :  declaration_specifiers tk_semicolon */
#line 863 "gram.y"

         is_typedef = FALSE;

         ++int_fcn.decl;         
         ++int_mod.decl;

         
         int_stm.is_decl = int_stm.end = TRUE;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr164: {	/* declaration :  declaration_specifiers init_declarator_list tk_semicolon */
#line 878 "gram.y"

         is_typedef = FALSE;

         ++int_fcn.decl;         
         ++int_mod.decl;

         
         int_stm.is_decl = int_stm.end = TRUE;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr166: {	/* declaration_specifiers :  storage_class_specifier */
#line 902 "gram.y"
 ++nested_decl_specs; 
} break;

case YYr167: {	/* declaration_specifiers :  storage_class_specifier $166 declaration_specifiers */
#line 904 "gram.y"
 --nested_decl_specs; 
} break;

case YYr169: {	/* declaration_specifiers :  type_specifier */
#line 907 "gram.y"
 ++nested_decl_specs; 
} break;

case YYr170: {	/* declaration_specifiers :  type_specifier $169 declaration_specifiers */
#line 909 "gram.y"
 --nested_decl_specs; 
} break;

case YYr174: {	/* init_declarator :  declarator */
#line 920 "gram.y"
 before_initializer(); 
} break;

case YYr175: {	/* init_declarator :  declarator $174 tk_assign initializer */
#line 922 "gram.y"
 after_initializer(); 
} break;

case YYr176: {	/* storage_class_specifier :  tk_typedef */
#line 931 "gram.y"
 is_typedef = TRUE; 
} break;

case YYr195: {	/* struct_or_union_specifier :  struct_or_union identifier */
#line 962 "gram.y"
 looking_for_tag = FALSE; 
} break;

case YYr197: {	/* struct_or_union_specifier :  struct_or_union tk_lcurly_brace */
#line 965 "gram.y"
 looking_for_tag = FALSE; 
} break;

case YYr199: {	/* struct_or_union_specifier :  struct_or_union identifier */
#line 968 "gram.y"
 looking_for_tag = FALSE; 
} break;

case YYr200: {	/* struct_or_union :  tk_struct */
#line 979 "gram.y"
 looking_for_tag = TRUE; 
} break;

case YYr201: {	/* struct_or_union :  tk_union */
#line 981 "gram.y"
 looking_for_tag = TRUE; 
} break;

case YYr204: {	/* struct_declaration :  type_specifier_list struct_declarator_list tk_semicolon */
#line 991 "gram.y"

         ++int_fcn.decl;         
         ++int_mod.decl;

         
         int_stm.is_decl = int_stm.end = TRUE;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr210: {	/* enum_specifier :  enum enumeration */
#line 1023 "gram.y"
 looking_for_tag = FALSE; 
} break;

case YYr211: {	/* enum_specifier :  enum identifier */
#line 1025 "gram.y"
 looking_for_tag = FALSE; 
} break;

case YYr213: {	/* enum_specifier :  enum identifier */
#line 1028 "gram.y"
 looking_for_tag = FALSE; 
} break;

case YYr216: {	/* enum :  tk_enum */
#line 1046 "gram.y"
 looking_for_tag = TRUE; 
} break;

case YYr223: {	/* declarator2 :  identifier */
#line 1066 "gram.y"

         if (nested_decl_specs == 0)
         {
            strcpy(last_decl, last_identifier());

            
            fire_declarator();

            
            if (is_typedef)
              typedef_symbol_table_add(token());
         }
      
} break;

case YYr227: {	/* declarator2 :  declarator2 tk_lparen */
#line 1093 "gram.y"

         if (!is_within_function)
            strcpy(function_name, last_identifier());
      
} break;

case YYr229: {	/* declarator2 :  declarator2 tk_lparen */
#line 1104 "gram.y"

         ++nested_decl_specs;
         if (!is_within_function)
            strcpy(function_name, last_identifier());
      
} break;

case YYr230: {	/* declarator2 :  declarator2 tk_lparen $229 parameter_type_list */
#line 1110 "gram.y"
 --nested_decl_specs; 
} break;

case YYr232: {	/* declarator2 :  declarator2 tk_lparen */
#line 1113 "gram.y"

         ++nested_decl_specs;
         if (!is_within_function)
            strcpy(function_name, last_identifier());
      
} break;

case YYr233: {	/* declarator2 :  declarator2 tk_lparen $232 parameter_identifier_list */
#line 1119 "gram.y"
 --nested_decl_specs; 
} break;

case YYr272: {	/* statement :  compound_statement */
#line 1198 "gram.y"

         is_else = is_if = FALSE;

         ++int_fcn.high;      

         
         not_case_label();

         
         int_stm.is_comp = int_stm.is_high = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr274: {	/* statement :  selection_statement */
#line 1217 "gram.y"

         ++int_fcn.high;      

         
         not_case_label();

         
         int_stm.is_select = int_stm.is_high = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr275: {	/* statement :  iteration_statement */
#line 1233 "gram.y"

         ++int_fcn.high;      

         
         not_case_label();

         
         int_stm.is_iter = int_stm.is_high = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr276: {	/* statement :  jump_statement */
#line 1249 "gram.y"

         check_multiple_statements();

         ++int_fcn.low;          

         
         not_case_label();

         
         int_stm.is_jump = int_stm.is_low = int_stm.end = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr277: {	/* labeled_statement :  identifier tk_colon */
#line 1270 "gram.y"

         
         int_stm.is_label = TRUE;
         int_stm.depth = depth;
         fire_stm();
         ZERO(int_stm);
      
} break;

case YYr278: {	/* labeled_statement :  identifier tk_colon $277 statement */
#line 1281 "gram.y"
 is_else = is_if = FALSE; 
} break;

case YYr279: {	/* labeled_statement :  tk_case constant_expr tk_colon */
#line 1283 "gram.y"

         check_multiple_statements();

         
         case_label();

         
         ++fcn_conditions;
      
} break;

case YYr281: {	/* labeled_statement :  tk_default tk_colon */
#line 1299 "gram.y"
 check_multiple_statements(); 
} break;

case YYr283: {	/* compound_statement :  compound_intro */
#line 1305 "gram.y"
 after_compound_declarations(); 
} break;

case YYr285: {	/* compound_statement :  compound_intro */
#line 1308 "gram.y"
 after_compound_declarations(); 
} break;

case YYr287: {	/* compound_statement :  compound_intro declaration_list */
#line 1311 "gram.y"
 after_compound_declarations(); 
} break;

case YYr289: {	/* compound_statement :  compound_intro declaration_list */
#line 1314 "gram.y"
 after_compound_declarations(); 
} break;

case YYr291: {	/* compound_intro :  tk_lcurly_brace */
#line 1320 "gram.y"
 before_compound_declarations(); 
} break;

case YYr296: {	/* expression_statement :  tk_semicolon */
#line 1335 "gram.y"
 found_expression_statement(TRUE); 
} break;

case YYr297: {	/* expression_statement :  expr tk_semicolon */
#line 1337 "gram.y"
 found_expression_statement(FALSE); 
} break;

case YYr298: {	/* selection_statement :  tk_if tk_lparen expr tk_rparen */
#line 1342 "gram.y"

         is_if = TRUE;
         check_multiple_statements();
         ++depth;                

         
         ++fcn_decisions;
         ++fcn_conditions;
      
} break;

case YYr299: {	/* selection_statement :  tk_if tk_lparen expr tk_rparen $298 statement */
#line 1352 "gram.y"
 --depth;                  
} break;

case YYr301: {	/* selection_statement :  tk_switch tk_lparen expr tk_rparen */
#line 1355 "gram.y"

         check_multiple_statements();
         ++depth;                
      
} break;

case YYr302: {	/* selection_statement :  tk_switch tk_lparen expr tk_rparen $301 statement */
#line 1360 "gram.y"
 --depth;                  
} break;

case YYr304: {	/* opt_else :  tk_else */
#line 1366 "gram.y"

         check_multiple_statements();
         is_else = TRUE;         
         ++depth;                
      
} break;

case YYr305: {	/* opt_else :  tk_else $304 statement */
#line 1372 "gram.y"
 --depth;                  
} break;

case YYr306: {	/* iteration_statement :  tk_while tk_lparen expr tk_rparen */
#line 1377 "gram.y"

         check_multiple_statements();
         ++depth;                

         
         ++fcn_decisions;
         ++fcn_conditions;
      
} break;

case YYr307: {	/* iteration_statement :  tk_while tk_lparen expr tk_rparen $306 statement */
#line 1386 "gram.y"
 --depth;                  
} break;

case YYr308: {	/* iteration_statement :  tk_do */
#line 1388 "gram.y"

         ++depth;                

         
         ++fcn_decisions;
         ++fcn_conditions;
      
} break;

case YYr309: {	/* iteration_statement :  tk_do $308 statement */
#line 1396 "gram.y"
 --depth;                  
} break;

case YYr310: {	/* iteration_statement :  tk_do $308 statement $309 tk_while tk_lparen expr tk_rparen tk_semicolon */
#line 1398 "gram.y"
 check_multiple_statements(); 
} break;

case YYr311: {	/* iteration_statement :  tk_for tk_lparen opt_expr tk_semicolon tk_semicolon opt_expr tk_rparen */
#line 1400 "gram.y"

         check_multiple_statements();
         ++depth;                
         
      
} break;

case YYr312: {	/* iteration_statement :  tk_for tk_lparen opt_expr tk_semicolon tk_semicolon opt_expr tk_rparen $311 statement */
#line 1409 "gram.y"
 --depth;                  
} break;

case YYr313: {	/* iteration_statement :  tk_for tk_lparen opt_expr tk_semicolon expr tk_semicolon opt_expr tk_rparen */
#line 1411 "gram.y"

         check_multiple_statements();
         ++depth;                

         
         ++fcn_decisions;
         ++fcn_conditions;
      
} break;

case YYr314: {	/* iteration_statement :  tk_for tk_lparen opt_expr tk_semicolon expr tk_semicolon opt_expr tk_rparen $313 statement */
#line 1420 "gram.y"
 --depth;                  
} break;

case YYr325: {	/* external_declaration :  function_definition */
#line 1454 "gram.y"
 stat_func_end(); 
} break;

case YYr327: {	/* function_definition :  declarator */
#line 1460 "gram.y"
 stat_func_begin(); 
} break;

case YYr329: {	/* function_definition :  declaration_specifiers declarator */
#line 1463 "gram.y"
 stat_func_begin(); 
} break;
#line 237 "c:/mks/etc/yyparse.c"
	case YYrACCEPT:
		YYACCEPT;
	case YYrERROR:
		goto yyError;
#ifdef YYLR2
	case YYrLR2:
#ifndef YYSYNC
		YYREAD;
#endif
		yyj = 0;
		while(yylr2[yyj] >= 0) {
			if(yylr2[yyj] == yystate && yylr2[yyj+1] == yychar
			&& yylook(yys+1,yyps,yystate,yychar,yy2lex(),yylr2[yyj+2]))
					break;
			yyj += 3;
		}
		if(yylr2[yyj] < 0)
			goto yyError;
		if(yylr2[yyj+2] < 0) {
			yystate = ~ yylr2[yyj+2];
			goto yyStack;
		}
		yyi = yylr2[yyj+2];
		goto yyReduce;
#endif
	}

	/*
	 *	Look up next state in goto table.
	 */

	yyp = &yygo[yypgo[yyi]];
	yyq = yyp++;
	yyi = *yyps;
#if	0 && defined(__TURBOC__) && __SMALL__
	/* THIS ONLY WORKS ON TURBO C 1.5 !!! */
	/* yyi is in di, yyp is in si */
L02:
	asm lodsw		/* ax = *yyp++; */
	asm cmp yyi, ax
	asm jl L02
#else
	while (yyi < *yyp++)
		;
#endif
	yystate = ~(yyi == *--yyp? YYQYYP: *yyq);
	goto yyStack;

yyerrlabel:	;		/* come here from YYERROR	*/
/*
#pragma used yyerrlabel
 */
	yyerrflag = 1;
	if (yyi == YYrERROR)
		yyps--, yypv--;
	
yyError:
	switch (yyerrflag) {

	case 0:		/* new error */
		yynerrs++;
		yyi = yychar;
		yyerror("Syntax error");
		if (yyi != yychar) {
			/* user has changed the current token */
			/* try again */
			yyerrflag++;	/* avoid loops */
			goto yyEncore;
		}

	case 1:		/* partially recovered */
	case 2:
		yyerrflag = 3;	/* need 3 valid shifts to recover */
			
		/*
		 *	Pop states, looking for a
		 *	shift on `error'.
		 */

		for ( ; yyps > yys; yyps--, yypv--) {
			if (*yyps >= sizeof yypact/sizeof yypact[0])
				continue;
			yyp = &yyact[yypact[*yyps]];
			yyq = yyp;
			do
				;
			while (YYERRCODE < *yyp++);
			if (YYERRCODE == yyp[-1]) {
				yystate = ~YYQYYP;
				goto yyStack;
			}
				
			/* no shift in this state */
#ifdef YYDEBUG
			if (yydebug && yyps > yys+1)
				printf(
	gettext("Error recovery pops state %d (%d), uncovers %d (%d)\n"),
					yysmap[yyps[0]], yyps[0],
					yysmap[yyps[-1]], yyps[-1]);
#endif
			/* pop stacks; try again */
		}
		/* no shift on error - abort */
		break;

	case 3:
		/*
		 *	Erroneous token after
		 *	an error - discard it.
		 */

		if (yychar == 0)  /* but not EOF */
			break;
#ifdef YYDEBUG
		if (yydebug)
			printf(gettext("Error recovery discards %s (%d), "),
				yyptok(yychar), yychar);
#endif
		yyclearin;
		goto yyEncore;	/* try again in same state */
	}
	YYABORT;

#ifdef YYALLOC
yyReturn:
	yylval = save_yylval;
	yyval = save_yyval;
	yypvt = save_yypvt;
	yychar = save_yychar;
	yyerrflag = save_yyerrflag;
	yynerrs = save_yynerrs;
	free((char *)yys);
	free((char *)yyv);
	return(retval);
#endif
}

#ifdef YYLR2
yylook(s,rsp,state,c1,c2,i)
short *s;		/* stack		*/
short *rsp;		/* real top of stack	*/
int state;		/* current state	*/
int c1;			/* current char		*/
int c2;			/* next char		*/
int i;			/* action S < 0, R >= 0	*/
{
	int j;
	short *p,*q;
	short *sb,*st;
#ifdef YYDEBUG
	if(yydebug) {
	printf(gettext("LR2 state %d (%d) char %s (%d) lookahead %s (%d)"),
			yysmap[state],state,yyptok(c1),c1,yyptok(c2),c2);
		if(i > 0)
			printf(gettext("reduce %d (%d)\n"), yyrmap[i], i);
		else
			printf(gettext("shift %d (%d)\n"), yysmap[i], i);
	}
#endif
	st = sb = rsp+1;
	if(i >= 0)
		goto reduce;
  shift:
	state = ~i;
	c1 = c2;
	if(c1 < 0)
		return 1;
	c2 = -1;

  stack:
  	if(++st >= &s[YYSSIZE]) {
		yyerror("Parser Stack Overflow");
		return 0;
	}
	*st = state;
	if(state >= sizeof yypact/sizeof yypact[0])
		i = state- YYDELTA;
	else {
		p = &yyact[yypact[state]];
		q = p;
		i = c1;
		while(i < *p++)
			;
		if(i == p[-1]) {
			state = ~q[q-p];
			c1 = c2;
			if(c1 < 0)
				return 1;
			c2 = -1;
			goto stack;
		}
		if(state >= sizeof yydef/sizeof yydef[0])
			return 0;
		if((i = yydef[state]) < 0) {
			p = &yyex[~i];
			while((i = *p) >= 0 && i != c1)
				p += 2;
			i = p[1];
		}
	}
  reduce:
  	j = yyrlen[i];
	if(st-sb >= j)
		st -= j;
	else {
		rsp -= j+st-sb;
		st = sb;
	}
	switch(i) {
	case YYrERROR:
		return 0;
	case YYrACCEPT:
		return 1;
	case YYrLR2:
		j = 0;
		while(yylr2[j] >= 0) {
			if(yylr2[j] == state && yylr2[j+1] == c1)
				if((i = yylr2[j+2]) < 0)
					goto shift;
				else
					goto reduce;
		}
		return 0;
	}
	p = &yygo[yypgo[i]];
	q = p++;
	i = st==sb ? *rsp : *st;
	while(i < *p++);
	state = ~(i == *--p? q[q-p]: *q);
	goto stack;
}
#endif
		
#ifdef YYDEBUG
	
/*
 *	Print a token legibly.
 *	This won't work if you roll your own token numbers,
 *	but I've found it useful.
 */
char *
yyptok(i)
{
	static char	buf[10];

	if (i >= YYERRCODE)
		return yystoken[i-YYERRCODE];
	if (i < 0)
		return "";
	if (i == 0)
		return "$end";
	if (i < ' ')
		sprintf(buf, "'^%c'", i+'@');
	else
		sprintf(buf, "'%c'", i);
	return buf;
}
#endif
#ifdef YYDEBUG
char * yystoken[] = {
	"error",
	"TK_IDENTIFIER",
	"TK_CONSTANT",
	"TK_STRING_LITERAL",
	"TK_SIZEOF",
	"TK_PTR_OP",
	"TK_INC_OP",
	"TK_DEC_OP",
	"TK_LEFT_OP",
	"TK_RIGHT_OP",
	"TK_LE_OP",
	"TK_GE_OP",
	"TK_EQ_OP",
	"TK_NE_OP",
	"TK_AND_OP",
	"TK_OR_OP",
	"TK_MUL_ASSIGN",
	"TK_DIV_ASSIGN",
	"TK_MOD_ASSIGN",
	"TK_ADD_ASSIGN",
	"TK_SUB_ASSIGN",
	"TK_LEFT_ASSIGN",
	"TK_RIGHT_ASSIGN",
	"TK_AND_ASSIGN",
	"TK_XOR_ASSIGN",
	"TK_OR_ASSIGN",
	"TK_TYPE_NAME",
	"TK_TYPEDEF",
	"TK_EXTERN",
	"TK_STATIC",
	"TK_AUTO",
	"TK_REGISTER",
	"TK_CHAR",
	"TK_SHORT",
	"TK_INT",
	"TK_LONG",
	"TK_SIGNED",
	"TK_UNSIGNED",
	"TK_FLOAT",
	"TK_DOUBLE",
	"TK_CONST",
	"TK_VOLATILE",
	"TK_VOID",
	"TK_STRUCT",
	"TK_UNION",
	"TK_ENUM",
	"TK_ELIPSIS",
	"TK_RANGE",
	"TK_CASE",
	"TK_DEFAULT",
	"TK_IF",
	"TK_ELSE",
	"TK_SWITCH",
	"TK_WHILE",
	"TK_DO",
	"TK_FOR",
	"TK_GOTO",
	"TK_CONTINUE",
	"TK_BREAK",
	"TK_RETURN",
	"THEN",
	0
};
char * yysvar[] = {	"$accept",
	"translation_unit",
	"tk_lparen",
	"tk_rparen",
	"tk_lsquare_brace",
	"tk_rsquare_brace",
	"tk_dot",
	"tk_ptr_to",
	"tk_inc",
	"tk_dec",
	"tk_comma",
	"tk_sizeof",
	"tk_ampersand",
	"tk_asterisk",
	"tk_plus",
	"tk_minus",
	"tk_tilde",
	"tk_exclamation",
	"tk_slash",
	"tk_percent",
	"tk_left",
	"tk_right",
	"tk_lt",
	"tk_gt",
	"tk_le",
	"tk_ge",
	"tk_eq",
	"tk_ne",
	"tk_circumflex",
	"tk_bar",
	"tk_land",
	"tk_lor",
	"tk_question",
	"tk_colon",
	"tk_assign",
	"tk_mul_assign",
	"tk_div_assign",
	"tk_mod_assign",
	"tk_add_assign",
	"tk_sub_assign",
	"tk_left_assign",
	"tk_right_assign",
	"tk_and_assign",
	"tk_xor_assign",
	"tk_or_assign",
	"tk_semicolon",
	"tk_typedef",
	"tk_extern",
	"tk_static",
	"tk_auto",
	"tk_register",
	"tk_char",
	"tk_short",
	"tk_int",
	"tk_long",
	"tk_signed",
	"tk_unsigned",
	"tk_float",
	"tk_double",
	"tk_const",
	"tk_volatile",
	"tk_void",
	"tk_struct",
	"tk_union",
	"tk_enum",
	"tk_type_name",
	"tk_lcurly_brace",
	"tk_rcurly_brace",
	"tk_elipsis",
	"tk_case",
	"tk_default",
	"tk_if",
	"tk_switch",
	"tk_else",
	"tk_while",
	"tk_do",
	"tk_for",
	"tk_goto",
	"tk_continue",
	"tk_break",
	"tk_return",
	"tk_string",
	"tk_constant",
	"identifier",
	"primary_expr",
	"string_list",
	"expr",
	"postfix_expr",
	"$91",
	"$93",
	"argument_expr_list",
	"assignment_expr",
	"unary_expr",
	"unary_operator",
	"cast_expr",
	"type_name",
	"multiplicative_expr",
	"additive_expr",
	"shift_expr",
	"relational_expr",
	"equality_expr",
	"and_expr",
	"exclusive_or_expr",
	"inclusive_or_expr",
	"logical_and_expr",
	"$140",
	"logical_or_expr",
	"$143",
	"conditional_expr",
	"assignment_operator",
	"constant_expr",
	"declaration",
	"declaration_specifiers",
	"init_declarator_list",
	"storage_class_specifier",
	"$166",
	"type_specifier",
	"$169",
	"init_declarator",
	"declarator",
	"$174",
	"initializer",
	"struct_or_union_specifier",
	"enum_specifier",
	"struct_or_union",
	"$195",
	"struct_declaration_list",
	"$197",
	"struct_declaration",
	"type_specifier_list",
	"struct_declarator_list",
	"struct_declarator",
	"enum",
	"enumeration",
	"$211",
	"enumerator_list",
	"enumerator",
	"declarator2",
	"pointer",
	"$227",
	"$229",
	"parameter_type_list",
	"$230",
	"$232",
	"parameter_identifier_list",
	"$233",
	"identifier_list",
	"parameter_list",
	"parameter_declaration",
	"abstract_declarator",
	"direct_abstract_declarator",
	"initializer_list",
	"statement",
	"labeled_statement",
	"compound_statement",
	"expression_statement",
	"selection_statement",
	"iteration_statement",
	"jump_statement",
	"$277",
	"$279",
	"$281",
	"compound_intro",
	"$283",
	"$285",
	"statement_list",
	"declaration_list",
	"$287",
	"$289",
	"$298",
	"$299",
	"opt_else",
	"$301",
	"$304",
	"$306",
	"$308",
	"$309",
	"opt_expr",
	"$311",
	"$313",
	"body",
	"external_declaration",
	"function_definition",
	"$327",
	"function_body",
	"$329",
	0
};
short yyrmap[] = {
	 333,  334,  335,   91,   93,  166,  169,  174,  195,  199, 
	 211,  213,  227,  229,  232,  283,  285,  287,  289,  329, 
	 327,  325,  314,  313,  312,  311,  310,  309,  308,  307, 
	 306,  305,  304,  302,  301,  299,  298,  297,  296,  291, 
	 281,  279,  278,  277,  276,  275,  274,  272,  233,  230, 
	 223,  216,  210,  204,  201,  200,  197,  176,  175,  170, 
	 167,  164,  163,  146,  143,  140,   82,   81,   80,   79, 
	  78,   77,   76,   75,   74,   73,   72,   71,   70,   69, 
	  68,   67,   66,   65,   64,   63,   62,   61,   60,   59, 
	  58,   57,   56,   55,   54,   53,   52,   51,   50,   49, 
	  48,   47,   46,   45,   44,   43,   42,   41,   40,   39, 
	  38,   37,   36,   35,   34,   33,   32,   31,   30,   29, 
	  28,   27,   26,   25,   24,   23,   22,   21,   20,   19, 
	  18,   17,   16,   15,   14,   13,   12,   11,   10,    9, 
	   8,    7,    6,    5,    4,    3,    2,    1,   83,   84, 
	  86,   87,   88,   89,   90,   92,   94,   95,   96,   97, 
	  98,   99,  100,  102,  103,  104,  105,  106,  107,  108, 
	 109,  110,  111,  112,  113,  114,  115,  116,  117,  118, 
	 147,  148,  149,  150,  151,  152,  153,  154,  155,  156, 
	 157,  158,  159,  160,  161,  162,  171,  172,  177,  178, 
	 179,  180,  181,  182,  183,  184,  185,  186,  187,  188, 
	 189,  190,  191,  192,  193,  194,  196,  198,  202,  203, 
	 205,  206,  208,  209,  212,  214,  215,  217,  218,  220, 
	 224,  225,  226,  228,  231,  234,  237,  238,  239,  240, 
	 242,  243,  244,  246,  247,  248,  249,  251,  253,  257, 
	 258,  259,  260,  261,  262,  263,  264,  265,  266,  267, 
	 268,  269,  270,  271,  273,  280,  282,  284,  286,  288, 
	 290,  292,  293,  294,  295,  300,  317,  318,  319,  320, 
	 323,  324,  326,  328,  330,  331,  332,  322,  321,  316, 
	 315,  303,  256,  255,  254,  252,  250,  245,  241,  236, 
	 235,  222,  221,  219,  207,  173,  168,  165,  145,  144, 
	 142,  141,  139,  138,  137,  136,  135,  134,  133,  132, 
	 131,  130,  129,  128,  127,  126,  125,  124,  123,  122, 
	 121,  120,  119,  101,   85,    0
};
short yysmap[] = {
	   0,    5,    6,   10,   35,   36,   61,   66,   68,   69, 
	  73,   77,   80,   90,  102,  103,  104,  105,  106,  107, 
	 108,  109,  110,  111,  124,  128,  142,  157,  160,  162, 
	 164,  166,  214,  218,  222,  240,  261,  277,  281,  287, 
	 288,  289,  290,  291,  292,  293,  294,  295,  296,  297, 
	 298,  299,  310,  341,  354,  358,  380,  385,  386,  440, 
	 448,  456,  466,  464,  459,  457,  455,  454,  449,  447, 
	 445,  444,  443,  442,  441,  433,  428,  427,  425,  421, 
	 416,  414,  412,  410,  409,  408,  407,  405,  401,  397, 
	 396,  389,  384,  379,  377,  376,  375,  373,  366,  362, 
	 361,  360,  359,  357,  353,  351,  348,  347,  343,  340, 
	 335,  324,  312,  309,  308,  307,  306,  303,  286,  285, 
	 284,  283,  282,  280,  276,  275,  274,  260,  253,  252, 
	 251,  248,  247,  246,  244,  243,  242,  241,  239,  238, 
	 236,  234,  233,  231,  227,  225,  223,  221,  215,  213, 
	 212,  208,  205,  201,  200,  199,  198,  197,  194,  193, 
	 188,  187,  186,  185,  182,  181,  180,  178,  176,  170, 
	 159,  158,  152,  151,  150,  147,  146,  145,  144,  127, 
	 123,  122,  121,  120,   99,   98,   97,   96,   91,   87, 
	  82,   79,   78,   72,   70,   67,   62,   37,   15,   12, 
	   9,    7,    1,    3,  467,  458,  463,  453,  465,  406, 
	 245,  450,  429,  468,  460,  451,  430,  452,  431,  367, 
	 249,   89,  365,  411,  434,  368,  254,  255,  256,  258, 
	 163,  167,    8,   11,   74,  398,   13,   14,   76,   34, 
	 350,  148,  149,  153,   83,  420,  171,  174,   38,  131, 
	 132,  262,  263,  264,  265,  266,  267,  268,  462,  269, 
	 270,  271,  272,  371,  229,   75,   39,   40,   41,   42, 
	  43,   44,   45,   46,   47,   48,   49,   50,   51,   52, 
	  53,   54,   55,   56,   57,   58,   84,  325,  326,  327, 
	 328,  329,  330,  331,  332,  333,  334,  226,  344,  172, 
	 173,  175,  177,  179,  183,  184,  189,  190,  191,  192, 
	 195,  196,  202,  203,  133,  134,  135,  136,   59,  137, 
	 138,  154,  139,  140,  216,  217,  141,   71,   95,   60, 
	 130,  129,  336,  126,  224,  125,  391,  390,  422,  305, 
	 304,  211,  210,  388,  437,  209,  207,  206,  204,  387, 
	 119,  118,  117,  116,  115,  114,  113,  392,  112,  302, 
	 301,  300,  220,  393,  323,  322,  321,  320,  319,  318, 
	 317,  316,  315,  314,  313,  219,  394,  101,   81,  237, 
	  33,   32,   31,   30,   29,   28,   27,   26,   25,   24, 
	  23,   22,   21,   20,   19,   18,   17,   16,  399,  346, 
	 232,  345,  342,  424,  395,  423,  230,  228,  339,  143, 
	 338,  337,   94,  100,  169,  168,  383,  372,   65,   93, 
	  64,   92,  370,  161,  369,  382,  165,  381,  279,  278, 
	 311,  419,  374,  413,  417,  436,  378,  418,  415,  435, 
	 349,  426,  439,  400,  438,  259,  257,  446,  432,  273, 
	 364,  352,  402,   88,  156,  250,  363,  461,  404,  356, 
	 355,  403,    4,   63,    2,   86,  235,   85,  155
};
int yyntoken = 86;
int yynvar = 186;
int yynstate = 469;
int yynrule = 336;
#endif
