#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "tstamp.h"

#define TRUE           1	/* True                                       */
#define FALSE          0	/* False                                      */
#define NPACKED        4	/* Number of longs in the packed buffer       */
#define NUNPACKED     12	/* Number of shorts in the unpacked buffer    */

#define MASK1 0X3ff00000
#define MASK2 0X000ffc00
#define MASK3 0X000003ff

#ifndef SEEK_SET
#define SEEK_SET 0
#endif



/*******************************************************************************
FUNCITON: usage

PURPOSE:  Print the usage message and error out.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          09/93  Development

PROJECT:  ADAPS
*******************************************************************************/
void usage( void )
{ 
printf("\n");
printf("usage: tstamp [-l] [-nosync] [-noklm] <image>\n");
printf("\n");
printf("       The function framesync is called using the -s flag to determine if the\n");
printf("       image is packed or unpacked, if it needs to be byte swapped, the location\n");
printf("       of the first frame sync, and the size of each line.  Once this\n");
printf("       information is returned by framesync, this function will then read the\n");
printf("       Julian day and time stamp from every line in the specified image.\n");
printf("\n");
printf("       -l <xx>  Number of lines to report on.  The default is all lines in the\n");
printf("                image.\n");
printf("\n");
printf("       -nosync <p> <b> <start> <nbytes>  If <p> is the character 'p' then the\n");
printf("                image is assumed to be packed, otherwise the data is assumed to\n");
printf("                be unpacked.  <b> is the number of bytes that needs to be byte\n");
printf("                swapped in the image (0, 2, 4).  <start> is the byte in which\n");
printf("                the frame sync should start if it existed and <nbytes> is the\n");
printf("                number of bytes contained in a single scan line of data.\n");
printf("\n");
printf("                Examples:\n");
printf("                    EDC archive image:  -nosync p 0 1 13796\n");
printf("                    Level 1B image:     -nosync p 0 15231 14800\n");
printf("\n");
printf("       -noklm   Image is not a KLM series satellite.  The default is to assume\n");
printf("                the scene may be from N15, N16 or N17.  Use this flag if tstamp\n");
printf("                reports that the scene is a KLM series satellite and you suspect\n");
printf("                it isn't.  KLM satellites use the same ID's as earlier \n");
printf("                satellites and were launched starting in Nov 98.\n");
printf("\n");
exit(1);
}



/*******************************************************************************
FUNCITON: byte_swap

PURPOSE:  Byte swap a buffer of data.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          09/93  Development

PROJECT:  ADAPS
*******************************************************************************/
void byte_swap
    (
    char *buffer,	/* IN/OUT - Buffer containing the data to be swapped  */
    short nswap,	/* IN     - Number of bytes in each group to swap     */
    short nbytes	/* IN     - Total number of bytes contained in buffer */
    )
{
char *left;		/* Ptr to the leftmost byte value in a group          */
char *right;		/* Ptr to the rightmost byte value in a group         */
char tmp;		/* Temporary variable used for swapping values        */

long i;			/* Loop control                                       */

for (i = 0; i < nbytes; i += nswap)
    {
    left = buffer + i;
    right = left + nswap - 1;

    while (left < right)
	{
	tmp = *left;
	*left = *right;
	*right = tmp;
	left++;
	right--;
	}
    }
}



/*******************************************************************************
FUNCITON: tstamp

PURPOSE:  Call the function framesync to determine the format and type of iamge
	  specified by the user.  Then extract the Julian day and time stamp
	  from each line of the image.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          09/93  Development
  1.1	 D.Lloyd	  08/98  Changed satid 7 to NOAA 15 and added band 3
  				 information message for NOAA 15
  1.2    D.Lloyd          10/00  Added a -noklm flag and modified the logic
  				 to determine satellite ids.  Added NOAA 16
				 as satid 3.

PROJECT:  ADAPS
*******************************************************************************/
void main
    (
    int argc,
    char *argv[]
    )
{
char cmd[256];		/* UNIX command                                       */
char image[256];	/* Name of the AVHRR archive image to check           */
char tmp[256];		/* Temporary working string                           */
char noaa[10];		/* NOAA satellite ID                                  */

int status;		/* Status variable used to close the pipe to framesync*/

long first_byte = 0;	/* Location of the first frame sync                   */
long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long jday       = 0;	/* Julian day that a line was acquired                */
long maxlines   = 0;	/* Maximum number of lines to report                  */
long msec       = 0;	/* Milliseconds since midnight at time of acquisition */
long nbytes     = 0;	/* Number of bytes per line in the archive image      */
long nlines     = 0;	/* Number of lines processed                          */
long packed[NPACKED];	/* Buffer of packed data from the image               */
long prev_msec  = 0;	/* Time stamp of the previous line                    */
long seek;		/* Number of the byte to seek to                      */
long band3;		/* Band 3A/3B flag -- 0 is 3A, 1 is 3B -- NOAA KLM    */

short bswap;		/* Indicates if the image is byte swapped (0, 2, 4)   */
short done;		/* Loop control                                       */
short sync      = TRUE;	/* Flag indicating frame sync contained in image      */
short packflg;		/* Flag indicating if the image is packed or not      */
short satid;		/* Satellite ID                                       */
short unpacked[NUNPACKED];/* Buffer of unpacked data from the image           */
short noklm = FALSE;    /* Flag to indicate if the satellite is from KLM series */

FILE *fp;


/* Check the arguments
----------------------*/
if (argc < 2)
    usage();
for (i = 1; i < (argc - 1); i++)
    {
    if (strcmp(argv[i], "-l") == 0)
	{
	i++;
	sscanf(argv[i], "%d", &maxlines);
	if (maxlines < 1)
	    maxlines = 0;
	}
    else if (strcmp(argv[i], "-nosync") == 0)
	{
	sync = FALSE;
	i++;
	sscanf(argv[i], "%s", tmp);
	if (strcmp(tmp, "p") == 0)
	    packflg = TRUE;
	else
	    packflg = FALSE;
	i++;
	sscanf(argv[i], "%hd", &bswap);
	i++;
	sscanf(argv[i], "%d", &first_byte);
	i++;
	sscanf(argv[i], "%d", &nbytes);
	}
    else if ( strcmp( argv[i], "-noklm" ) == 0 )
        noklm = TRUE;
    else
	usage();
    }
strcpy(image, argv[i]);


/* Open a pipe and determine the location of the first frame
   sync and the number of bytes between each frame sync.
------------------------------------------------------------*/
if (sync)
    {
    sprintf(cmd, "framesync -b 5000000 -s -n 10 %s", image);
    fp = popen(cmd, "r");
    if (fp == NULL)
	{
	printf("error opening pipe\n");
	exit(1);
	}
    fscanf(fp, "%s %hd %d %d", tmp, &bswap, &first_byte, &nbytes);
    pclose(fp);
    wait(&status);
    if (strcmp(tmp, "p") == 0)
	packflg = TRUE;
    else
	packflg = FALSE;
    }


/* If either the location of the first frame sync or the number of
   bytes between consecutive frame syncs are zero then error out.
------------------------------------------------------------------*/
if ((first_byte == 0) || (nbytes == 0))
    {
    printf("Framesync could not locate an AVHRR frame sync.\n");
    exit(1);
    }


/* Open the specified image for reading
---------------------------------------*/
fp = fopen(image, "r");
if (fp == NULL)
    {
    printf("Error opening image.\n");
    exit(1);
    }


/* Process the specified number of time stamp codes
---------------------------------------------------*/
done = FALSE;
first_byte -= 1;  /* subtract one so it is an offset */
nlines = 0;
while (!done)
    {
    /* Seek to the next HRPT minor frame
    ------------------------------------*/
    seek = first_byte + nlines * nbytes;
    if (fseek(fp, seek, SEEK_SET))
	{
	done = TRUE;  /* must have hit eof */
	continue;
	}
    

    /* Read the frame sync, ID, and the time stamp code from the image
    ------------------------------------------------------------------*/
    if (packflg)
	{
	/* Read the data from the image
	-------------------------------*/
	if (fread(packed, sizeof(long), NPACKED, fp) != NPACKED)
	    {
	    printf("Error reading from image.\n");
	    fclose(fp);
	    exit(1);
	    }
	

	/* Byte swap the data if necessary
	----------------------------------*/
	if (bswap)
	    byte_swap((char *)packed, bswap, NPACKED * sizeof(long));

	
	/* Unpack the NPACKED long values into the unpacked buffer
	----------------------------------------------------------*/
	for (i = 0, j = 0; i < NPACKED; i++)
	    {
	    unpacked[j++] = (short)((packed[i] & MASK1) >> 20);
	    unpacked[j++] = (short)((packed[i] & MASK2) >> 10);
	    unpacked[j++] = (short)(packed[i] & MASK3);
	    }
	}
    else
	{
	/* Read the data from the image
	-------------------------------*/
	if (fread(unpacked, sizeof(short), NUNPACKED, fp) != NUNPACKED)
	    {
	    printf("Error reading from image.\n");
	    fclose(fp);
	    exit(1);
	    }


	/* Byte swap the data if necessary
	----------------------------------*/
	if (bswap)
	    byte_swap((char *)unpacked, bswap, NUNPACKED * sizeof(short));
	}



    /* Extract the Satellite ID, Julian day, and the time stamp
    -----------------------------------------------------------*/
    nlines++;
    satid = (unpacked[6] >> 3) & 0xf;
    switch (satid)
	{
	case 1:
	    strcpy(noaa,"11");
	    break;
	case 3:
	    if ( noklm )
		strcpy(noaa,"06");
	    else
		{
		strcpy(noaa,"16");
		band3 = unpacked[6] & 0x1;
		}
	    break;
	case 5:
	    strcpy(noaa,"14");
	    break;
	case 7:
	    if ( noklm )
		strcpy(noaa,"07");
	    else
	        {
		strcpy(noaa,"15");
		band3 = unpacked[6] & 0x1;
		}
	    break;
	case 9:
	    strcpy(noaa,"12");
	    break;
	case 11:
	    if ( noklm )
		strcpy(noaa,"09");
	    else
	        {
		strcpy(noaa,"17");
		band3 = unpacked[6] & 0x1;
		}
	    break;
	case 13:
	    if ( noklm )
		strcpy(noaa,"08");
	    else
	        {
		strcpy(noaa,"18");
		band3 = unpacked[6] & 0x1;
		}
	    break;
	case 15:
	    strcpy(noaa,"10");
	    break;
	default:
	    strcpy(noaa, "??");
	}

    jday  = (unpacked[8] >> 1) & 0x3ff;
    msec  = unpacked[9] & 0x7f;
    msec  = (msec << 10) | (unpacked[10] & 0x3ff);
    msec  = (msec << 10) | (unpacked[11] & 0x3ff);


    /* Report the information to the user
    -------------------------------------*/
    printf("line: %4d  NOAA-%s  jdate: %03d  msec: %8d  diff: %d",
       nlines, noaa, jday, msec, msec - prev_msec);
    
    /* For NOAA KLM series (15,16,17) report band
    3A/3B information also
    -------------------------------------------------*/
    if ( !noklm )
	{
	if (( satid == 11 || satid == 7 || satid == 3 ) && band3 )
	   printf( "  Band 3A" );
	else if ( satid == 11 || satid == 7 || satid == 3 )
	   printf( "  Band 3B" );
	}
    printf( "\n" );

    prev_msec = msec;


    /* Determine if we have reported on the correct number of lines
    ---------------------------------------------------------------*/
    if ((maxlines > 0) && (nlines >= maxlines))
	done = TRUE;
    }


/* Close the image
------------------*/
fclose(fp);


exit(0);
}
