/*******************************************************************************
NAME			      NOTAE_UPDDR

PURPOSE	     Updates records 1 and 2 of a DDR file.  This routine is the same
	     as the LAS routine c_upddr with the exception that the TAE
	     parameter block is not required.

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
B. Ailts	      Jan. 1988 	original development
B. Ailts	      Apr. 1988		Replaced newlas.h with las.h
					Replaced cerrmsg with c_errmsg
					Changed calling sequences to the put
					and get ddr calls
					Added the rotated projection space
					subroutine
B. Ailts	      jul. 1988		COMB option can now have more output
					bands than input bands
B. Ailts	      Sept. 1988	Increased the buffer space for the 
					band records of the ddr when multiple
					bands are being combined
B.Ailts		      Jan. 1989		If COMB, all bands of one input image
					are not compared with the corresponding
					bands of another input image,  all the
					input bands of all the input images are
					compared with one another
B.Ailts		      Dec. 1989		Added code to free dynamic allocated 
					buffers
B.Ailts		      Dec. 1990		Updated error messages
L. Huewe	      Oct. 1993		Created a version which is not 
					dependent on TAE.

PROJECT       LAS

ALGORITHM 
   Retreive the ouptut DDR band indepenent records

   If no. of input images is equal to one
      Retreive the input DDR band indepenent records
      Copy the fields to the output ddr
   Else
      Retreive the band indepenent records of all the input DDRs
      Compare validity flags and values to see if the values of the
        output DDR will be copied from the input DDR or made invalid
   End if

    Based on the flags set by the above comparisons or the validity
      flag if there was only one input image, copy or recalculate the
      individual fields of the output DDR.
    Write the band indepenent records to the output DDR file

    If cflag is set to COMB
       Read the band depenent data of the input DDRs
       Compare the fields to the correct band of the output DDR
       If the band dependent fields are equal then
	  Copy the input band dependent fields to the output DDR
       Else
	  Invalidate the band depenent fields of the output DDR
    Else
       Read the band depenent data of the input DDRs
       Copy the fields to the correct band of the output DDR
       Write the output DDR to the DDR file
    Endif

    Return

ALGORITHM REFERENCES	The LAS routine c_upddr.c
*******************************************************************************/
#include "las.h"

FUNCTION notae_upddr(ddrvflag,projtol,in_ddr,out_ddr,in_bddr,out_bddr,nmrimg,
		 window,bands,nbands,line_inc,samp_inc,out_nbands,cflag)

struct DDR in_ddr[];            /* input ddrs                        */
struct DDR *out_ddr;      	/* output ddr                        */
struct BDDR in_bddr[];          /* input band dependent ddr records  */
struct BDDR out_bddr[];         /* output band dependent ddr record  */

long ddrvflag;		 	/*  Validity flag to use when not valid      */
long *nmrimg;			/*  Total number of input images	     */
long window[][4];		/*  Window specifications for each image     */
long bands[][MAXBND +1];	/*  band numbers for each input image	     */
long nbands[];			/*  number of bands for each input image     */
double *line_inc;		/*  increment in the line direction	     */
double *samp_inc;		/*  increment in the sample direction        */
long *out_nbands;	 	/*  number of output bands		     */
long *cflag;			/*  combination of bands flag		     */
float projtol;			/*  Tolerance global		     	     */

{ /* notae_upddr.c  */

long corn_index;		/*  index to first DDR with valid corner flag*/
long corner_flag;		/*  Processing corner validity flag	     */
long curbnd;			/*  loop counter -- current band	     */
long i;				/*  loop counter			     */
long in_bands;			/*  Total number of input bands		     */
long inc_flag;			/*  Processing increment validity flag	     */
long inc_index;			/*  index to first DDR with valid increm. flag*/
long j;				/*  loop counter			     */
long one = 1;			/*  nonfatal error status		     */
long pdist_flag;		/*  Processing proj. distance validity flag  */
long pdist_index;		/*  index to first DDR with valid increm. flag*/
long ppar_flag;			/*  Processing corner proj. parameter flag   */
long proj_flag;			/*  Processing corner projection flag	     */
long status;			/*  status flag				     */
long zone_flag;			/*  Processing corner zone flag	     	     */

char vflag[3];			/* ddr valid flag			     */

/*  Allocate space for the input DDRs
-------------------------------------*/
in_bands = 0;
for (i = 0; i < *nmrimg; i++)
   in_bands += nbands[i];
/*
in_ddr = (struct DDR *)calloc(*nmrimg,sizeof(struct DDR));
out_bddr = (struct BDDR *)calloc(*out_nbands,sizeof(struct BDDR));
if (*cflag == COMB)
   in_bddr = (struct BDDR *)calloc(in_bands,sizeof(struct BDDR));
else
   in_bddr = (struct BDDR *)malloc(sizeof(struct BDDR));
if ((in_ddr == NULL) || (out_bddr == NULL) || (in_bddr == NULL))
   {
   c_errmsg("Error allocating dynamic memory","upddr-alloc",&one);
   return(E_FAIL);
   }
*/

/*  if only one input image copy needed information from the input ddr
----------------------------------------------------------------------*/
if (*nmrimg == 1)
   {
   /*  Set the validity flags used by set_corners routines
   -------------------------------------------------------*/
   corner_flag = in_ddr[0].valid[DDCCV];
   corn_index= 0;
   proj_flag = in_ddr[0].valid[DDPCV];
   pdist_flag = in_ddr[0].valid[DDPDV];
   pdist_index = 0;
   inc_flag = in_ddr[0].valid[DDINCV];
   inc_index = 0;
   zone_flag = in_ddr[0].valid[DDZCV];
   ppar_flag = in_ddr[0].valid[DDPPV];
   set_master(&in_ddr[0],out_ddr,inc_flag,window[0]);
   
   /*  Copy certain fields from the input DDR to the output DDR
   ------------------------------------------------------------*/
   out_ddr->valid[DDPCV] = in_ddr[0].valid[DDPCV];
   out_ddr->valid[DDZCV] = in_ddr[0].valid[DDZCV];
   out_ddr->valid[DDDCV] = in_ddr[0].valid[DDDCV];
   out_ddr->valid[DDPPV] = in_ddr[0].valid[DDPPV];
   out_ddr->valid[DDPUV] = in_ddr[0].valid[DDPUV];
   out_ddr->proj_code = in_ddr[0].proj_code;
   out_ddr->zone_code = in_ddr[0].zone_code;
   out_ddr->datum_code = in_ddr[0].datum_code;
   strcpy(out_ddr->proj_units,in_ddr[0].proj_units);

   for (i = 0; i < 15; i++)
       out_ddr->proj_coef[i] = in_ddr[0].proj_coef[i];

   }

else  /* multiple input images */
   {
   /*  Copy or null the individual fields of the ddr
   -------------------------------------------------*/ 
   set_proj(in_ddr,out_ddr,&proj_flag,*nmrimg);
   set_datum(in_ddr,out_ddr,*nmrimg);
   set_zone(in_ddr,out_ddr,&zone_flag,*nmrimg);
   set_ppar(in_ddr,out_ddr,&ppar_flag,&proj_flag,*nmrimg);
   set_punit(in_ddr,out_ddr,*nmrimg);

   /*  Set the flags of the individual fields of the ddr which may need to
       be recalculated.
   -----------------------------------------------------------------------*/
   set_pdis_flag(in_ddr,&pdist_flag,&pdist_index,*nmrimg);
   set_inc_flag(in_ddr,&inc_flag,&inc_index,*nmrimg);
   set_corn_flag(in_ddr,&corner_flag,window[0],&corn_index,
	  	 pdist_flag,projtol,pdist_index,ddrvflag,*nmrimg);
   out_ddr->master_line = 1;
   out_ddr->master_sample = 1;
   } /* else multiple input images */

/*  Recalculate the individual fields of the ddr based upon their flags which
    were set above
-----------------------------------------------------------------------------*/
set_increment(inc_flag,&in_ddr[inc_index],out_ddr,*line_inc,*samp_inc);
set_pdist(pdist_flag,&in_ddr[pdist_index],out_ddr,*line_inc,*samp_inc);
set_corners(corner_flag,proj_flag,pdist_flag,zone_flag,ppar_flag,
	    &in_ddr[corn_index],out_ddr,*line_inc,*samp_inc,
            window[corn_index]);

/*  Compare the band dependent fields of each input ddr to check equality.
    This is done only when two or more individual bands will be combined
    into one band(i.e. applications like CONCAT,MATH)
--------------------------------------------------------------------------*/
if (*cflag == COMB)
   {
   /*  Compare the fields for each band
   ------------------------------------*/  
   set_capture(&(in_bddr[0]),&(out_bddr[0]),in_bands);
   } /* cflag == COMB  */
else /* there is one output band for each input band of each image */
   { /* else cflag == NOCOMB */
   curbnd = 0;
   for (j = 0; j < *nmrimg; j++)
      {
      for (i = 0; i < nbands[j]; i++)
         {
	 /*  Copy the band fields of the input DDR to the band fields of
	     the output DDR except for minval, maxval, bandno, and validity
  	 ------------------------------------------------------------------*/
	 strcpy(out_bddr[curbnd].source,in_bddr->source);
	 strcpy(out_bddr[curbnd].instrument,in_bddr->instrument);
	 strcpy(out_bddr[curbnd].direction,in_bddr->direction);
	 strcpy(out_bddr[curbnd].date,in_bddr->date);
	 strcpy(out_bddr[curbnd].time,in_bddr->time);
         curbnd++;
         } /* for i to bands[j][i] */
      } /* for j to *nmrimg */
   } /* else cflag == NOCOMB */

return (E_SUCC);
} /*  notae_upddr  */
