/****************************************************************************/
/*   PROGRAM NAME  : ct_stretchoperations.c                                 */
/*   AUTHOR        : g e nostrand                                           */
/*   DATE          : aug 30, 1993                                           */
/*   LOCATION      : DSRC PRINCETON, NJ                                     */
/*                                                                          */
/*   PROGRAM DESCRIPTION:                                                   */
/*		This module contains functions for manipulating the 	    */
/*		stretch list including loading, saving, and updating	    */
/*		Additionally, some generic utility functions are 	    */
/*		maintained for displaying errors/warnings and obtaining     */
/*              confirmation.						    */
/*   PROGRAM MODIFICATIONS:                                                 */
/****************************************************************************/

#include <malloc.h>
#include "xid.h"
#include "ct_interface.h"
/*****************************************************************************/
/*
	The stretch list is a mechanism for storing and retrieving 
	contrast stretches in order to provide both a quick comparison
	mechanism between multiple stretches and to store predefined
	stretches for reuse.  Two version of the stretch file format 
	have been defined.  The original version contained stretches
	with Red, Green and Blue mappings.  Version 2.0 allows for 
	Saturation stretches.  Both versions can be read but version 2.0
	is the current default for writing.  The format of the stretch file
	is unfortunately binary.  The file name conventions for the stretch
	file is to suffix the file with .strh.   

	Stretch File Format
	    char version[] = "USGS_XIDSTRETCHFILEHEADER_VERSION_2.0"
	    int	NumStretch;	Total number of stretches in file 

	    Each stretch element for NumStretch contains the following
	        int redindex;	index of red breakpoints
	        int redfrom[redindex+1]; red from values
	        int redto [redindex+1]; red to values	
	        int greenindex;	index of green breakpoints
	        int greenfrom[greenindex+1]; green from values
	        int greento [greenindex+1]; green to values	
	        int blueindex;	index of blue breakpoints
	        int bluefrom[blueindex+1]; blue from values
	        int blueto [blueindex+1]; blue to values	
	        int satindex;	index of sat breakpoints
	        int satfrom[satindex+1]; sat from values
	        int satto [satindex+1]; sat to values	

*/
/*****************************************************************************/


#define  OLDXIDSTRETCHFILEHEADER "USGS_XIDSTRETCHFILEHEADER_VERSION_1.0"
#define  XIDSTRETCHFILEHEADER "USGS_XIDSTRETCHFILEHEADER_VERSION_2.0"

/*****************************************************************************/
/*
	Internally, the stretch list is stored using structures the same
	as that used within the current XID_COLORTOOL info.  The first
	three elements of the array are reserverd for programmatic use
	to store the current, previous and linear stretches.  
*/
/*****************************************************************************/


XID_COLORSTRETCHPTR  	Stretch[MAXSTRETCHES];
int                  	CurrStretch = -1;	/* Current selected stretch */
int 			NumStretch=0;	/* Number of stretches in array	    */
char			*StretchFile = NULL;  /* File name of stretch file  */

/* List of names for first three stretch list entries	*/
char    *ListNames [] = {
	"Current",
	"Previous",
	"Linear"
	};

/* 
   Data structures maintained in ct_graphicoperations.c to store information
   on the current contrast stretch
*/
extern XID_COLORTOOLPTR CT_Color;
extern XID_COLORTOOL        red, green, blue, allrgb, satrgb;

extern XID_COLORTOOLPTR		CT_Colors[];

/*****************************************************************************/
/*
	The function ct_stretchlistentytext formats the stretch list entries
	for display in the Stretch List Tool selection list.  The format is in 
	the form of from-to pairs for each color.
*/
/*****************************************************************************/


#ifdef _NO_PROTO
void ct_stretchlistentytext(str, stretch)
char *str;	/* Preallocated string to hold formatted stretch entry  */
int stretch;	/* Index of stretch list entry 				*/
#else
void ct_stretchlistentytext(char *str, int stretch)
#endif
{
   int          n;
   char         tmp[16];

#define CURRSTRETCH
#ifdef CURRSTRETCH
   /* Provide names to first three entries, "Stretch #" to all user added */
   if (stretch <= LINEAR)
       sprintf (str, "%s ", ListNames[stretch]);
   else
       sprintf(str,"Stretch #% 2d ",stretch-LINEAR);
#else
       sprintf(str,"Stretch #% 2d ",stretch+1);
#endif
   for (n=0;n<=Stretch[stretch]->color[RED].index;n++) {
       sprintf(tmp,"R(%d->%d), ",Stretch[stretch]->color[RED].from[n],
                                 Stretch[stretch]->color[RED].to[n]);
       strcat(str,tmp);
   }
   for (n=0;n<=Stretch[stretch]->color[GREEN].index;n++) {
       sprintf(tmp,"G(%d->%d), ",Stretch[stretch]->color[GREEN].from[n],
                                 Stretch[stretch]->color[GREEN].to[n]);
       strcat(str,tmp);
   }
   for (n=0;n<=Stretch[stretch]->color[BLUE].index;n++) {
       sprintf(tmp,"B(%d->%d), ",Stretch[stretch]->color[BLUE].from[n],
                                 Stretch[stretch]->color[BLUE].to[n]);
       strcat(str,tmp);
   }
   for (n=0;n<=Stretch[stretch]->color[STRETCHSAT].index;n++) {
       sprintf(tmp,"S(%3.2f->%3.2f), ",
			Stretch[stretch]->color[STRETCHSAT].from[n] / 255.0,
			Stretch[stretch]->color[STRETCHSAT].to[n] / 255.0);
       strcat(str,tmp);
   }
   str [strlen(str) - 2] = '\0';	/* Strip last comma	*/
}
/*****************************************************************************/
/*
	The function ct_init_stretch_list initializes the stretch list
	and allocates the predefined three stretch list entries.  The stretch
	tool selection list is updated with the first entries.  
*/
/*****************************************************************************/


void ct_init_stretch_list ()
{
    int		list, clr;
    long	*from_offset, *to_offset;
    char	*str;
    int		strsize;
#ifdef _NO_PROTO
    void ct_displayerror();
#else
    void ct_displayerror(char *msg);
#endif



    for (list=CURRENT; list <=LINEAR; list++)
    {
	/*  Allocate the whole area at the beginning and calculate offsets for 
	    other pointers.  This should simplify the code organization.
	*/
	Stretch[list]= (XID_COLORSTRETCHPTR)
	    malloc(sizeof(XID_COLORSTRETCH) + 
	    ((STRETCHSAT+1) *((sizeof(long) * 256 * 2) )));

	if (Stretch[list] == NULL) {
	  ct_displayerror("Could not allocate memory for colorstretch structure");
	  break;
	}

	from_offset = (long *) ((char *)Stretch[list] 
			+ sizeof(XID_COLORSTRETCH));
	to_offset = (long *) ((char *)from_offset + 
		((STRETCHSAT+1) * sizeof(long) * 256));
	for (clr=RED; clr <= STRETCHSAT; clr++)
	{
	    Stretch[list]->color[clr].from= from_offset;
	    Stretch[list]->color[clr].to= to_offset;
	    Stretch[list]->color[clr].index = 1;
	    Stretch[list]->color[clr].from[0] = 0;
	    Stretch[list]->color[clr].to[0]   = 0;
	    Stretch[list]->color[clr].from[1] = 255;
	    Stretch[list]->color[clr].to[1]   = 255;
	    from_offset += 256;
	    to_offset += 256;
	}

	/* calculate size of string sufficient to hold formatted entry  */
	/* depends on format specified in ct_stretchlistentytext	*/
	strsize= 14 * ( Stretch[list]->color[RED].index + 
		       Stretch[list]->color[GREEN].index + 
		       Stretch[list]->color[BLUE].index   +
		       Stretch[list]->color[STRETCHSAT].index   
		       + 4);

	if ((str=(char *)malloc(sizeof(char) * strsize)) == NULL) {
	  ct_displayerror(" Could not allocate memory for list entry string...exitting");
	  exit(0);
	}
	ct_stretchlistentytext(str,list); /* convienence format list entry */

	XmListAddItem(colortool_stretch_list, 
		XmStringCreateLtoR(str,XmSTRING_DEFAULT_CHARSET), list+1);
	XmListSelectPos(colortool_stretch_list, list, False);
	XmListSetBottomPos(colortool_stretch_list, list);
	free(str);
    }

    NumStretch = LINEAR + 1;
}
/*****************************************************************************/
/*
	The function ct_upd_list_entry updates the selection list item 
	for the specified stretch list entry.  This is called when updating
	current and previous entries or when a user has pressed the update
	option button.
*/
/*****************************************************************************/


#ifdef _NO_PROTO
void ct_upd_list_entry (list)
int list;
#else
void ct_upd_list_entry (int 	list)
#endif
{
    int strsize;
    char	*str;
    XmString	xmstr;

#ifdef _NO_PROTO
    void ct_displayerror();
#else
    void ct_displayerror(char *msg);
#endif

	/* calculate size of string sufficient to hold formatted entry  */
	/* depends on format specified in ct_stretchlistentytext	*/

    strsize= 14 * ( Stretch[list]->color[RED].index + 
		   Stretch[list]->color[GREEN].index + 
		   Stretch[list]->color[BLUE].index   +
		   Stretch[list]->color[STRETCHSAT].index   
		   + 4);

    if ((str=(char *)malloc(sizeof(char) * strsize)) == NULL) {
      ct_displayerror(" Could not allocate memory for list entry string...exitting");
      exit(0);
    }
    ct_stretchlistentytext(str,list); /* convienence format list entry */

    xmstr = XmStringCreateLtoR(str,XmSTRING_DEFAULT_CHARSET);
    XmListReplaceItemsPos(colortool_stretch_list, &xmstr, 1, list+1);
    XmStringFree (xmstr);
    free(str);

}
/*****************************************************************************/
/*
	The function ct_new_curr_stretch is called whenever the current
	stretch list entry is modified.  The current entry is copied to the
	previous entry and the current entry is updated.  The Undo/Redo 
	button on the Color Tool display is reset to Undo.
	
*/
/*****************************************************************************/

void ct_new_curr_stretch ()
{
    int	clr, i;

    if (NumStretch == 0)
    {
	ct_init_stretch_list();
    }
    /*  Save CURRENT stretch to PREVIOUS	*/
    for (clr=RED; clr <= STRETCHSAT; clr++)
    {
	Stretch[PREVIOUS]->color[clr].index =Stretch[CURRENT]->color[clr].index;
	for (i=0; i <= Stretch[CURRENT]->color[clr].index; i++)
        {
	   Stretch[PREVIOUS]->color[clr].from[i] = 
		Stretch[CURRENT]->color[clr].from[i];
	   Stretch[PREVIOUS]->color[clr].to[i] = 
		Stretch[CURRENT]->color[clr].to[i];
	}
    }

    /*  Update CURRENT to lastest stretch	*/
    for (clr=RED; clr <= STRETCHSAT; clr++)
    {
	Stretch[CURRENT]->color[clr].index =CT_Colors[clr]->adjust.index;
	for (i=0; i <= Stretch[CURRENT]->color[clr].index; i++)
        {
	   Stretch[CURRENT]->color[clr].from[i] = 
		CT_Colors[clr]->adjust.from[i];
	   Stretch[CURRENT]->color[clr].to[i] = 
		CT_Colors[clr]->adjust.to[i];
	}
    }
    ct_upd_list_entry (CURRENT);
    ct_upd_list_entry (PREVIOUS);

#define UNDO    1
#define REDO    2

    {
    Arg         args[2];
    int         argcnt;
    Boolean     argok;
    XmString    xmstr[256];
    extern int Undo_Redo;

    XtSetSensitive (ct_undo_redo_button, True);
    if (Undo_Redo == REDO)
    {
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Undo", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (ct_undo_redo_button, args, argcnt);
	XmStringFree (xmstr[0]);

	Undo_Redo = UNDO;
    }
    }
}
/*****************************************************************************/
/*
	The function ct_clear_stretch_list is called to free the space 
	occupied by the current stretch list and clear the selection list
	on the Stretch List Tool display.  
*/
/*****************************************************************************/

void ct_clear_stretch_list()
{
   int n;

#ifdef CURRSTRETCH
   for (n=NumStretch;n>FIRSTSTRETCH;n--) {
#else
   for (n=NumStretch;n>0;n--) {
#endif
       free(Stretch[n-1]->color[RED].from);
       free(Stretch[n-1]->color[RED].to);
       free(Stretch[n-1]->color[GREEN].from);
       free(Stretch[n-1]->color[GREEN].to);
       free(Stretch[n-1]->color[BLUE].from);
       free(Stretch[n-1]->color[BLUE].to);
       free(Stretch[n-1]->color[STRETCHSAT].from);
       free(Stretch[n-1]->color[STRETCHSAT].to);
       free(Stretch[n-1]);
       XmListDeletePos(colortool_stretch_list,n);
   }
#ifdef CURRSTRETCH
   NumStretch=FIRSTSTRETCH;
#else
   NumStretch=0;
#endif
   CurrStretch=0;
   XFlush(xid.display);
}
/*****************************************************************************/
/*
	The function ct_get_confirmation is used when a user request requires
	confirmation prior to proceeding.  A dialog is displayed with a message
	requesting the confirmation.  The user can then confirm or cancel the 
	request.   Note that confirmation cannot be stacked.
*/
/*****************************************************************************/

static void (*confirm_function)();/* Function to call on confirmation	*/
static void (*cancel_function)(); /* Function to call on calcellation	*/

#ifdef _NO_PROTO
void ct_get_confirmation(msg, confirm_func, cancel_func)
char *msg;		/* Message for user dialog	*/
void (*confirm_func)(); /* Function to call on confirmation	*/
void (*cancel_func)();  /* Function to call on calcellation	*/
#else
void ct_get_confirmation(char *msg, 
			 void (*confirm_func)(), void (*cancel_func)())
#endif
{
   Arg	args[10];
   int	argcnt=0;

   XtSetArg(args[argcnt], XmNmessageString, XmStringCreateLtoR(msg, XmSTRING_DEFAULT_CHARSET)); argcnt++;
   XtSetValues(colortool_confirm_operation,args,argcnt);
   XtManageChild(colortool_confirm_operation);
   confirm_function = confirm_func;
   cancel_function = cancel_func;
}
/*****************************************************************************/
/*
	The function ct_displayinfo display an information dialog to the 
	user.  The user can remove the display by pressing OK.  No further
	action is required of the application.
*/
/*****************************************************************************/

#ifdef _NO_PROTO
void ct_displayinfo(msg)
char *msg;
#else
void ct_displayinfo(char *msg)
#endif
{
   Arg	args[10];
   int	argcnt=0;

    XWindowAttributes xwa;
    XEvent	event;

   XtSetArg(args[argcnt], XmNmessageString, XmStringCreateLtoR(msg, XmSTRING_DEFAULT_CHARSET)); argcnt++;
   XtSetValues(colortool_warning_message,args,argcnt);
   XtUnmanageChild(XmMessageBoxGetChild(colortool_warning_message,XmDIALOG_OK_BUTTON));
   XtManageChild(colortool_warning_message);
   /*
   The following attempts to make the widget visible XtMainLoop
   */
   while 
   (XGetWindowAttributes(xid.display,XtWindow(colortool_warning_message), &xwa)
   && xwa.map_state != IsViewable)
   {
     XtAppNextEvent (XtWidgetToApplicationContext(colortool_warning_message), 
		     &event);
     XtDispatchEvent (&event);
   }

   XmUpdateDisplay (colortool_warning_message);
}
/*****************************************************************************/
/*
	The function ct_undisplayinfo is used to remove the informational
	dialog when loading an image into the separate Color Tool image
	window after the load has been completed.
*/
/*****************************************************************************/

void ct_undisplayinfo ()
{

   XtManageChild(XmMessageBoxGetChild(colortool_warning_message,XmDIALOG_OK_BUTTON));
   XtUnmanageChild(colortool_warning_message);

}
/*****************************************************************************/
/*
	The function ct_displaywarning displays a warning dialog
*/
/*****************************************************************************/

#ifdef _NO_PROTO
void ct_displaywarning(msg)
char *msg;
#else
void ct_displaywarning(char *msg)
#endif
{
   Arg	args[10];
   int	argcnt=0;

   XtSetArg(args[argcnt], XmNmessageString, XmStringCreateLtoR(msg, XmSTRING_DEFAULT_CHARSET)); argcnt++;
   XtSetValues(colortool_warning_message,args,argcnt);
   XtManageChild(colortool_warning_message);
}
/*****************************************************************************/
/*
	The function ct_displayerror displays an error dialog
*/
/*****************************************************************************/


#ifdef _NO_PROTO
void ct_displayerror(msg)
char *msg;
#else
void ct_displayerror(char *msg)
#endif
{
   Arg	args[10];
   int	argcnt=0;

   XtSetArg(args[argcnt], XmNmessageString, XmStringCreateLtoR(msg, XmSTRING_DEFAULT_CHARSET)); argcnt++;
   XtSetValues(colortool_error_message,args,argcnt);
   XtManageChild(colortool_error_message);
}
/*****************************************************************************/
/*
	The function ct_confirm is the callback for the OK button of a 
	confirmation dialog.  If a confirm function has been specified,
	the confirm function is executed.
*/
/*****************************************************************************/

#ifdef _NO_PROTO
void ct_confirm(w, data, call_data)
Widget w;
XtPointer data;
XtPointer call_data;
#else
void ct_confirm(Widget w, XtPointer data, XtPointer call_data)
#endif
{
   XtUnmanageChild(w);
   if (confirm_function)
       (*confirm_function)();
   XBell(xid.display,0);
}
/*****************************************************************************/
/*
	The function ct_cancel is the callback for the Cancel button of a 
	confirmation dialog.  If a cancel function has been specified,
	the cancel function is executed.
*/
/*****************************************************************************/

#ifdef _NO_PROTO
void ct_cancel(w, data, call_data)
Widget w;
XtPointer data;
XtPointer call_data;
#else
void ct_cancel(Widget w, XtPointer data, XtPointer call_data)
#endif
{
   if (cancel_function)
       (*cancel_function)();
   XtUnmanageChild(w);
}
/*****************************************************************************/
/*
	The function ct_savestretchfile_func saves the stretch list entries
	to the specified stretch file name.  The format of the stretch file
	is specified at the beginning of this source file.  
*/
/*****************************************************************************/

void
#ifdef _NO_PROTO
ct_savestretchfile_func (tmpname)
char *tmpname;
#else
ct_savestretchfile_func (char *tmpname)
#endif
{
   FILE *fp;
   int  n, strsize;
   char *str, tmp[80], msg[256];
   int	numstretch;


   if ((fp=fopen(tmpname,"w"))==NULL) {		
      XtUnmanageChild(colortool_load_stretch_file_selector);
      sprintf(msg,"File %s could not be opened for write",tmpname);
      ct_displayerror(msg);
      free(tmpname);
      return;
   }
   XtUnmanageChild(colortool_save_stretch_file_selector);
   sprintf(tmp,"%s",XIDSTRETCHFILEHEADER);
   if (fwrite(&tmp[0],sizeof(char),strlen(tmp),fp) !=  strlen(tmp)) {
      sprintf(msg,"Could not write header for file %s",tmpname);
      ct_displayerror(msg);
      free(tmpname);
      fclose(fp);
      return;
   }

#ifdef CURRSTRETCH
   numstretch = NumStretch - FIRSTSTRETCH;
   if (fwrite(&numstretch,sizeof(int),1,fp) ==  0) {
#else
   if (fwrite(&NumStretch,sizeof(int),1,fp) ==  0) {
#endif
      sprintf(msg,"Could not write NumStretch for file %s",tmpname);
      ct_displayerror(msg);
      free(tmpname);
      fclose(fp);
      return;
   }
#ifdef CURRSTRETCH
   for (n=FIRSTSTRETCH;n<NumStretch;n++) {
#else
   for (n=0;n<NumStretch;n++) {
#endif
      if (fwrite(&Stretch[n]->color[RED].index,sizeof(long),1,fp) ==  0) {
         sprintf(msg,"Could not write red index of stretch %d for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[RED].from,sizeof(long),
                Stretch[n]->color[RED].index+1,fp) != Stretch[n]->color[RED].index+1) {
         sprintf(msg,"Could not write red 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[RED].to,sizeof(long),
                Stretch[n]->color[RED].index+1,fp) != Stretch[n]->color[RED].index+1) {
         sprintf(msg,"Could not write red 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }

      if (fwrite(&Stretch[n]->color[GREEN].index,sizeof(long),1,fp) ==  0) {
         sprintf(msg,"Could not write green index of stretch %d for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[GREEN].from,sizeof(long),
                Stretch[n]->color[GREEN].index+1,fp) != Stretch[n]->color[GREEN].index+1) {
         sprintf(msg,"Could not write green 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[GREEN].to,sizeof(long),
                Stretch[n]->color[GREEN].index+1,fp) != Stretch[n]->color[GREEN].index+1) {
         sprintf(msg,"Could not write green 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }

      if (fwrite(&Stretch[n]->color[BLUE].index,sizeof(long),1,fp) ==  0) {
         sprintf(msg,"Could not write blue index of stretch %d for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[BLUE].from,sizeof(long),
                Stretch[n]->color[BLUE].index+1,fp) != Stretch[n]->color[BLUE].index+1) {
         sprintf(msg,"Could not write blue 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[BLUE].to,sizeof(long),
                Stretch[n]->color[BLUE].index+1,fp) != Stretch[n]->color[BLUE].index+1) {
         sprintf(msg,"Could not write blue 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(&Stretch[n]->color[STRETCHSAT].index,sizeof(long),1,fp) ==  0) {
         sprintf(msg,"Could not write blue index of stretch %d for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[STRETCHSAT].from,sizeof(long),
                Stretch[n]->color[STRETCHSAT].index+1,fp) != Stretch[n]->color[STRETCHSAT].index+1) {
         sprintf(msg,"Could not write blue 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
      if (fwrite(Stretch[n]->color[STRETCHSAT].to,sizeof(long),
                Stretch[n]->color[STRETCHSAT].index+1,fp) != Stretch[n]->color[STRETCHSAT].index+1) {
         sprintf(msg,"Could not write blue 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         return;
      }
   }
   fflush(fp);
   fclose(fp);
   sprintf(msg,"Stretch file %s sucessfully saved",tmpname);
   ct_displaywarning(msg);
}
/*****************************************************************************/
/*
	The function ct_savestretchfile is the XmNokCallback for the 
	PromptDialog widget colortool_save_stretch_file_selector.  This 
	allows the user to specify the filename for the stretch file 
	when saving a new stretch file.  The stretch file suffix is set
	to .strh if no suffix is provided.  When saving new, a check is 
	performed to ensure that there is no existing file with the same
	name.  A confirmation dialog allows an overwrite of an existing
	file. 
*/
/*****************************************************************************/

#ifdef _NO_PROTO
void ct_savestretchfile(w, client, call_data)
Widget w;
XtPointer client;
XtPointer call_data;
#else
void ct_savestretchfile(Widget w, XtPointer client, XtPointer call_data)
#endif
{
   FILE *fp;
   int  n, strsize;
   char *filename, *tmpname, *str, tmp[80], msg[256];
   XmSelectionBoxCallbackStruct *savedata = (XmSelectionBoxCallbackStruct*)call_data;

   /* check that is a file with correct suffix (.strh)
      unmanage the fileselection box
      check to see if file exists, if it does, return and force user to
      pick another name.  (later write routine to confirm with user the overwrite)
      save the stretch_list to file
      message dialog back to confirm save
   */

   XmStringGetLtoR(savedata->value,XmSTRING_DEFAULT_CHARSET,&filename);
   if ( strlen(filename) == 0) {
      XtUnmanageChild(colortool_load_stretch_file_selector);
      ct_displayerror("You must enter valid filename for this operation");
      XtFree(filename);
      return;
   }

   tmpname=(char *)malloc(sizeof(char) * strlen(filename)+5);
   if ( tmpname == NULL) {
      ct_displayerror("Could not allocate memory in write stretch file routine");
      XtUnmanageChild(colortool_load_stretch_file_selector);
      XtFree(filename);
      return;
   }

   if ( strncmp(&filename[strlen(filename)-5],".strh",5)) {       /* if matches returns 0 */
      sprintf(msg,"%s: Filename entered does not have\n correct type suffix... appending '.strh'",filename);
      ct_displaywarning(msg);
      sprintf(tmpname,"%s.strh",filename);
   }
   else {
      sprintf(tmpname,"%s",filename);
   }
   XtFree(filename);

   if ((fp=fopen(tmpname,"r"))!=NULL) {		/* don't allow overwrite of existing file for now */
      XtUnmanageChild(colortool_load_stretch_file_selector);
      sprintf(msg,"File %s already exists, choose another name( for now )",tmpname);
      ct_displayerror(msg);
      free(tmpname);
      return;
   }

   ct_savestretchfile_func (tmpname);
   StretchFile = tmpname;
}
/*****************************************************************************/
/*
	The function ct_accept_loadclear is the confirmation function when
	the user requests that a stretch file be loaded and there are existing
	stretch list entries.
	
*/
/*****************************************************************************/

void
ct_accept_loadclear ()
{

       ct_clear_stretch_list();

       XtManageChild(colortool_load_stretch_file_selector); 
}
/*****************************************************************************/
/*
	The function ct_loadstretchfile is the XmNokCallback for the 
	FileSelectionDialog widget colortool_load_stretch_file_selector.
	The stretch list file name is verified and loaded into the 
	Stretch List Tool selection list.  
*/
/*****************************************************************************/

#ifdef _NO_PROTO
void ct_loadstretchfile(w, client, call_data)
Widget w;
XtPointer client;
XtPointer call_data;
#else
void ct_loadstretchfile(Widget w, XtPointer client, XtPointer call_data)
#endif
{
   FILE *fp;
   int  n, strsize;
   char *filename, *tmpname, *str, tmp[80], msg[256];
   int	numstretch;
   int fileversion;

   XmFileSelectionBoxCallbackStruct *loaddata = (XmFileSelectionBoxCallbackStruct*)call_data;

   /* check that is a file with correct suffix (.strh)
      if not correct suffix, append it to the filename 
      see if the file  exists
      unmanage the fileselection box
      check that the file exists, if it does not, notify user and return 
      deallocate the stretch_list structure
      initialize the stretch list widget
      load the stretch file
      write the stretch list
      update the stretch filename label over the stretch list 
   */
   XmStringGetLtoR(loaddata->value,XmSTRING_DEFAULT_CHARSET,&filename);

   tmpname=(char *)malloc(sizeof(char) * strlen(filename)+5);
   if ( tmpname == NULL) {
      ct_displayerror("Could not allocate memory in read stretch file routine");
      XtUnmanageChild(colortool_load_stretch_file_selector);
      XtFree(filename);
      return;
   }
   if ( strcmp(&filename[strlen(filename)-5],".strh")) {	/* if matches returns 0 */
      sprintf(tmpname,"%s.strh",filename);
      sprintf(msg,"%s: Filename entered does not have\n correct type suffix... appending '.strh'",filename);
      ct_displaywarning(msg);
   }
   else {
      sprintf(tmpname,"%s",filename);
   }

   XtFree(filename);

   if ((fp=fopen(tmpname,"r"))==NULL) {
      XtUnmanageChild(colortool_load_stretch_file_selector);
      sprintf(msg,"Could not open file %s",tmpname);
      ct_displayerror(msg);
      free(tmpname);
      return;
   }
   XtUnmanageChild(colortool_load_stretch_file_selector);
   if ((n=fread(&tmp[0],sizeof(char),strlen(XIDSTRETCHFILEHEADER),fp)) ==  0) {
      sprintf(msg,"Could not read header for file %s",tmpname);
      ct_displayerror(msg);
      free(tmpname);
      fclose(fp);
      return;
   }
   tmp[strlen(XIDSTRETCHFILEHEADER)]='\0';

   fileversion = 2;
   if ( strncmp(tmp,XIDSTRETCHFILEHEADER,strlen(XIDSTRETCHFILEHEADER))) 
     if ( strncmp(tmp,OLDXIDSTRETCHFILEHEADER,strlen(OLDXIDSTRETCHFILEHEADER))) 
     {
      sprintf(msg,"Header for file %s is not correct (%s)",tmpname,tmp);
      ct_displayerror(msg);
      free(tmpname);
      fclose(fp);
      return;
     }
     else
	 fileversion = 1;
   if (fread(&numstretch,sizeof(int),1,fp) ==  0) {
      sprintf(msg,"Could not read NumStretch for file %s",tmpname);
      ct_displayerror(msg);
      free(tmpname);
      fclose(fp);
      return;
   }
   if (numstretch >= MAXSTRETCHES ) {
      sprintf(msg,"Number_of_stretches > %d for file %s",numstretch,MAXSTRETCHES,tmpname);
      ct_displayerror(msg);
      free(tmpname);
      fclose(fp);
      return;
   }
#ifdef CURRSTRETCH
   NumStretch = numstretch + FIRSTSTRETCH;
   for (n=FIRSTSTRETCH;n<NumStretch;n++) {
#else
   NumStretch = numstretch;
   for (n=0;n<NumStretch;n++) {
#endif
      Stretch[n]= (XID_COLORSTRETCHPTR)malloc(sizeof(XID_COLORSTRETCH));
      if (Stretch[n] == NULL) {
         sprintf(msg,"Could not allocate memory for colorstretch structure %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fread(&Stretch[n]->color[RED].index,sizeof(long),1,fp) ==  0) {
         sprintf(msg,"Could not read red index of stretch %d for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (Stretch[n]->color[RED].index > 255 ) {
         sprintf(msg,"Red Index for stretch %d exceeds 255 for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      Stretch[n]->color[RED].from=(long *)malloc(sizeof(long) * (Stretch[n]->color[RED].index+1));
      if (Stretch[n]->color[RED].from == NULL) {
         sprintf(msg,"Could not allocate memory for red 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fread(Stretch[n]->color[RED].from,sizeof(long),
                Stretch[n]->color[RED].index+1,fp) != Stretch[n]->color[RED].index+1) {
         sprintf(msg,"Could not read red 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      Stretch[n]->color[RED].to=(long *)malloc(sizeof(long) * (Stretch[n]->color[RED].index+1));
      if (Stretch[n]->color[RED].to == NULL) {
         sprintf(msg,"Could not allocate memory for red 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fread(Stretch[n]->color[RED].to,sizeof(long),
                Stretch[n]->color[RED].index+1,fp) != Stretch[n]->color[RED].index+1) {
         sprintf(msg,"Could not read red 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }

      if (fread(&Stretch[n]->color[GREEN].index,sizeof(long),1,fp) ==  0) {
         sprintf(msg,"Could read green index of stretch %d for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (Stretch[n]->color[GREEN].index > 255 ) {
         sprintf(msg,"Green index for stretch %d exceeds 255 for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      Stretch[n]->color[GREEN].from=(long *)malloc(sizeof(long) * (Stretch[n]->color[GREEN].index+1));
      if (Stretch[n]->color[GREEN].from == NULL) {
         sprintf(msg,"Could not allocate memory for green 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fread(Stretch[n]->color[GREEN].from,sizeof(long),
                Stretch[n]->color[GREEN].index+1,fp) != Stretch[n]->color[GREEN].index+1) {
         sprintf(msg,"Could not read green 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      Stretch[n]->color[GREEN].to=(long *)malloc(sizeof(long) * (Stretch[n]->color[GREEN].index+1));
      if (Stretch[n]->color[GREEN].to == NULL) {
         sprintf(msg,"Could not allocate memory for green 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fread(Stretch[n]->color[GREEN].to,sizeof(long),
                Stretch[n]->color[GREEN].index+1,fp) != Stretch[n]->color[GREEN].index+1) {
         sprintf(msg,"Could not read green 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }

      if (fread(&Stretch[n]->color[BLUE].index,sizeof(long),1,fp) ==  0) {
         sprintf(msg,"Could read blue index of stretch %d for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (Stretch[n]->color[BLUE].index > 255 ) {
         sprintf(msg,"Green index for stretch %d exceeds 255 for file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      Stretch[n]->color[BLUE].from=(long *)malloc(sizeof(long) * (Stretch[n]->color[BLUE].index+1));
      if (Stretch[n]->color[BLUE].from == NULL) {
         sprintf(msg,"Could not allocate memory for blue 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fread(Stretch[n]->color[BLUE].from,sizeof(long),
                Stretch[n]->color[BLUE].index+1,fp) != Stretch[n]->color[BLUE].index+1) {
         sprintf(msg,"Could not read blue 'from' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[BLUE].from);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      Stretch[n]->color[BLUE].to=(long *)malloc(sizeof(long) * (Stretch[n]->color[BLUE].index+1));
      if (Stretch[n]->color[BLUE].to == NULL) {
         sprintf(msg,"Could not allocate memory for blue 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[BLUE].from);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fread(Stretch[n]->color[BLUE].to,sizeof(long),
                Stretch[n]->color[BLUE].index+1,fp) != Stretch[n]->color[BLUE].index+1) {
         sprintf(msg,"Could not read blue 'to' array for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[BLUE].from);
         free(Stretch[n]->color[BLUE].to);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      if (fileversion == 2)
      {
	  if (fread(&Stretch[n]->color[STRETCHSAT].index,sizeof(long),1,fp) ==  0) {
	     sprintf(msg,"Could read blue index of stretch %d for file %s",n,tmpname);
	     ct_displayerror(msg);
	     free(tmpname);
	     free(Stretch[n]->color[GREEN].from);
	     free(Stretch[n]->color[GREEN].to);
	     free(Stretch[n]->color[RED].from);
	     free(Stretch[n]->color[RED].to);
	     free(Stretch[n]);
	     fclose(fp);
	     NumStretch=n;
	     return;
	  }
	  if (Stretch[n]->color[STRETCHSAT].index > 255 ) {
	     sprintf(msg,"Green index for stretch %d exceeds 255 for file %s",n,tmpname);
	     ct_displayerror(msg);
	     free(tmpname);
	     free(Stretch[n]->color[GREEN].from);
	     free(Stretch[n]->color[GREEN].to);
	     free(Stretch[n]->color[RED].from);
	     free(Stretch[n]->color[RED].to);
	     free(Stretch[n]);
	     fclose(fp);
	     NumStretch=n;
	     return;
	  }
	  Stretch[n]->color[STRETCHSAT].from=(long *)malloc(sizeof(long) * (Stretch[n]->color[STRETCHSAT].index+1));
	  if (Stretch[n]->color[STRETCHSAT].from == NULL) {
	     sprintf(msg,"Could not allocate memory for blue 'from' array for stretch %d, file %s",n,tmpname);
	     ct_displayerror(msg);
	     free(tmpname);
	     free(Stretch[n]->color[GREEN].from);
	     free(Stretch[n]->color[GREEN].to);
	     free(Stretch[n]->color[RED].from);
	     free(Stretch[n]->color[RED].to);
	     free(Stretch[n]);
	     fclose(fp);
	     NumStretch=n;
	     return;
	  }
	  if (fread(Stretch[n]->color[STRETCHSAT].from,sizeof(long),
		    Stretch[n]->color[STRETCHSAT].index+1,fp) != Stretch[n]->color[STRETCHSAT].index+1) {
	     sprintf(msg,"Could not read blue 'from' array for stretch %d, file %s",n,tmpname);
	     ct_displayerror(msg);
	     free(tmpname);
	     free(Stretch[n]->color[BLUE].from);
	     free(Stretch[n]->color[GREEN].from);
	     free(Stretch[n]->color[GREEN].to);
	     free(Stretch[n]->color[RED].from);
	     free(Stretch[n]->color[RED].to);
	     free(Stretch[n]);
	     fclose(fp);
	     NumStretch=n;
	     return;
	  }
	  Stretch[n]->color[STRETCHSAT].to=(long *)malloc(sizeof(long) * (Stretch[n]->color[STRETCHSAT].index+1));
	  if (Stretch[n]->color[STRETCHSAT].to == NULL) {
	     sprintf(msg,"Could not allocate memory for blue 'to' array for stretch %d, file %s",n,tmpname);
	     ct_displayerror(msg);
	     free(tmpname);
	     free(Stretch[n]->color[BLUE].from);
	     free(Stretch[n]->color[GREEN].from);
	     free(Stretch[n]->color[GREEN].to);
	     free(Stretch[n]->color[RED].from);
	     free(Stretch[n]->color[RED].to);
	     free(Stretch[n]);
	     fclose(fp);
	     NumStretch=n;
	     return;
	  }
	  if (fread(Stretch[n]->color[STRETCHSAT].to,sizeof(long),
		    Stretch[n]->color[STRETCHSAT].index+1,fp) != Stretch[n]->color[STRETCHSAT].index+1) {
	     sprintf(msg,"Could not read blue 'to' array for stretch %d, file %s",n,tmpname);
	     ct_displayerror(msg);
	     free(tmpname);
	     free(Stretch[n]->color[BLUE].from);
	     free(Stretch[n]->color[BLUE].to);
	     free(Stretch[n]->color[GREEN].from);
	     free(Stretch[n]->color[GREEN].to);
	     free(Stretch[n]->color[RED].from);
	     free(Stretch[n]->color[RED].to);
	     free(Stretch[n]);
	     fclose(fp);
	     NumStretch=n;
	     return;
	  }
      }
      else
      {
	  Stretch[n]->color[STRETCHSAT].index = 1;
	  Stretch[n]->color[STRETCHSAT].from=(long *)malloc(sizeof(long) * (Stretch[n]->color[STRETCHSAT].index+1));
	  Stretch[n]->color[STRETCHSAT].to=(long *)malloc(sizeof(long) * (Stretch[n]->color[STRETCHSAT].index+1));
	    Stretch[n]->color[STRETCHSAT].from[0] = 0;
	    Stretch[n]->color[STRETCHSAT].to[0]   = 0;
	    Stretch[n]->color[STRETCHSAT].from[1] = 255;
	    Stretch[n]->color[STRETCHSAT].to[1]   = 255;

      }
      strsize= 14 * (Stretch[n]->color[RED].index +
                     Stretch[n]->color[GREEN].index +
                     Stretch[n]->color[BLUE].index +
                     Stretch[n]->color[STRETCHSAT].index   + 4);

      if ((str=(char *)malloc(sizeof(char) * strsize)) == NULL) {
         sprintf(msg,"Could allocate string for label for stretch %d, file %s",n,tmpname);
         ct_displayerror(msg);
         free(tmpname);
         free(Stretch[n]->color[BLUE].from);
         free(Stretch[n]->color[BLUE].to);
         free(Stretch[n]->color[GREEN].from);
         free(Stretch[n]->color[GREEN].to);
         free(Stretch[n]->color[RED].from);
         free(Stretch[n]->color[RED].to);
         free(Stretch[n]);
         fclose(fp);
         NumStretch=n;
         return;
      }
      ct_stretchlistentytext(str,n);
      CurrStretch=n;
      XmListAddItem(colortool_stretch_list, XmStringCreateLtoR(str,XmSTRING_DEFAULT_CHARSET), n+1);
/* 
      XmListSelectPos(colortool_stretch_list, n+1, False);
*/
      XmListSetBottomPos(colortool_stretch_list, n+1);
      free(str);
   }
   fclose(fp);
   StretchFile = tmpname;
}
/*****************************************************************************/
/*
	The function ct_set_stretch_color sets the current stretch information
	for the color specified from the CurrentStretch entry. 
*/
/*****************************************************************************/
 
void
#ifdef _NO_PROTO
ct_set_stretch_color (color)
int color;
#else
ct_set_stretch_color (int color)
#endif
{
    
   void		ct_apply();
   int		n;

   CT_Colors[color]->adjust.index= Stretch[CurrStretch]->color[color].index;
   for (n=0;n<=Stretch[CurrStretch]->color[color].index;n++) {
       CT_Colors[color]->adjust.from[n]=
			Stretch[CurrStretch]->color[color].from[n];
       CT_Colors[color]->adjust.to[n]=
			Stretch[CurrStretch]->color[color].to[n];
   }
   ct_apply(CT_Colors[color]);
}
/*****************************************************************************/
/*
	The function ct_set_stretch sets the current stretch information for
	all colors specified from the CurrentStretch entry.  This 
	occurs when the user selects a stretch from the selection list or when
	the user selects UNDO/REDO.  
*/
/*****************************************************************************/

void
ct_set_stretch ()
{
   int 		n, size=SIZEBREAKPTBOX;
   void 	ct_drawline(),
		ct_histo_draw(),
		ct_apply();

   /*  Erase current contrast stretch in drawing area  		*/
   ct_drawline(CT_Color->window, 
		CT_Color->gc, 
		CT_Color->adjust.from, 
		CT_Color->adjust.to, 
	        CT_Color->adjust.index, 
		CT_Color->background, size+5);

   for (n=0; n<= STRETCHSAT; n++)
       ct_set_stretch_color (n);

    if (CT_Color->adjust.flags & MAP_ALL)
    {
	void	ct_cancel_all();
	int	i,j, diff = False, curr_color;
	/*  
	    Check if any colors have been updated and
	    warn user that ALL will override these updates
	*/
	for (i=0; i<BLUE;i++)
	{
	    if (Stretch[CurrStretch]->color[i].index 
	    != Stretch[CurrStretch]->color[i+1].index)
		diff = True;
	    else
		for (j=0; j<= Stretch[CurrStretch]->color[i].index;j++)
		{
		    if (Stretch[CurrStretch]->color[i].from[j] 
		    != Stretch[CurrStretch]->color[i+1].from[j]) 
			diff = True;
		    if (Stretch[CurrStretch]->color[i].to[j] 
		    != Stretch[CurrStretch]->color[i+1].to[j]) 
			diff = True;
		}
	}
	if (diff)
	{
	    ct_displaywarning (
		"Stretch selected RGB values inconsistent with ALL");
	    CT_Color = &red;
	    ct_cancel_all ();
	}
	else
	{
	   CT_Colors[4]->adjust.index= Stretch[CurrStretch]->color[BLUE].index;
	   for (n=0;n<=Stretch[CurrStretch]->color[BLUE].index;n++) {
	       CT_Colors[4]->adjust.from[n]=
				Stretch[CurrStretch]->color[BLUE].from[n];
	       CT_Colors[4]->adjust.to[n]=
				Stretch[CurrStretch]->color[BLUE].to[n];
	   }
	}

    }
   /* Redraw with new contrast stretch			*/
   ct_drawline(CT_Color->window, 
		CT_Color->gc, 
		CT_Color->adjust.from, 
		CT_Color->adjust.to, 
                CT_Color->adjust.index, 
		CT_Color->linecolor, size);
    /* Redraw histogrm for new contrast stretch 	*/
    ct_histo_draw (CT_Color);

    /* Make new contrast stretch the current stretch	*/
    ct_new_curr_stretch ();
}
/*****************************************************************************/
/*
	The function ct_selectstretch is the XmNsingleSelectionCallback for the
	stretch selection list widget ct_stretch_list.  This is called when the
	user double clicks on the selection list item.  The selected contrast
	stretch replaces the current stretch.
*/
/*****************************************************************************/

#ifdef _NO_PROTO
void ct_selectstretch(w, data, call_data)
Widget w;
XtPointer data;
XmListCallbackStruct *call_data;
#else
void ct_selectstretch(Widget w, XtPointer data, XmListCallbackStruct *call_data)
#endif
{
   /* recover the item position of the selected stretch 
      erase current graphic displays
      copy the XID_COLORSTRETCH structure into the XID_COLORTOOL structure
      apply the stretch to the colortable
      update the graphic displays.
   */
   CurrStretch=call_data->item_position-1;/* start array from zero, position starts from 1 */
 

   ct_set_stretch ();
}
/*****************************************************************************/
/*
	The fuction ct_add_stretch_item updates the stretch list entry 
	specified by stretch with the current contrast stretch information. 
*/
/*****************************************************************************/

void
#ifdef _NO_PROTO
ct_add_stretch_item(stretch)
int stretch;
#else
ct_add_stretch_item(int stretch)
#endif
{
    int n;


    Stretch[stretch]= (XID_COLORSTRETCHPTR)malloc(sizeof(XID_COLORSTRETCH));
    if (Stretch[stretch] == NULL) {
      ct_displayerror(
	 "Could not allocate memory for colorstretch structure");
      return;
    }

    Stretch[stretch]->color[RED].from=
	(long *)malloc(sizeof(long) * (CT_Colors[RED]->adjust.index+1));
    if (Stretch[stretch]->color[RED].from == NULL) {
      ct_displayerror(
	"Could not allocate memory for red colorstretch_list from structure");
      return;
    }
    Stretch[stretch]->color[RED].to=
	(long *)malloc(sizeof(long) * (CT_Colors[RED]->adjust.index+1));
    if (Stretch[stretch]->color[RED].to == NULL) {
      ct_displayerror(
	  "Could not allocate memory for red colorstretch_list to structure");
      return;
    }
    Stretch[stretch]->color[RED].index=CT_Colors[RED]->adjust.index;
    for (n=0;n<=CT_Colors[RED]->adjust.index;n++) {
       Stretch[stretch]->color[RED].from[n]= CT_Colors[RED]->adjust.from[n];
       Stretch[stretch]->color[RED].to[n]=   CT_Colors[RED]->adjust.to[n];
    }
    Stretch[stretch]->color[GREEN].from=
	(long *)malloc(sizeof(long) * (CT_Colors[GREEN]->adjust.index+1));
    if (Stretch[stretch]->color[GREEN].from == NULL) {
      ct_displayerror(
       " Could not allocate memory for green colorstretch_list from structure");
      return;
    }
    Stretch[stretch]->color[GREEN].to=
	(long *)malloc(sizeof(long) * (CT_Colors[GREEN]->adjust.index+1));
    if (Stretch[stretch]->color[GREEN].to == NULL) {
      ct_displayerror(
	 " Could not allocate memory for green colorstretch_list to structure");
      return;
    }
    Stretch[stretch]->color[GREEN].index=CT_Colors[GREEN]->adjust.index;
    for (n=0;n<=CT_Colors[GREEN]->adjust.index;n++) {
       Stretch[stretch]->color[GREEN].from[n]= CT_Colors[GREEN]->adjust.from[n];
       Stretch[stretch]->color[GREEN].to[n]=   CT_Colors[GREEN]->adjust.to[n];
    }
    Stretch[stretch]->color[BLUE].from=
	(long *)malloc(sizeof(long) * (CT_Colors[BLUE]->adjust.index+1));
    if (Stretch[stretch]->color[BLUE].from == NULL) {
      ct_displayerror(
	" Could not allocate memory for blue colorstretch_list from structure");
      return;
    }
    Stretch[stretch]->color[BLUE].to=
	(long *)malloc(sizeof(long) * (CT_Colors[BLUE]->adjust.index+1));
    if (Stretch[stretch]->color[BLUE].to == NULL) {
      ct_displayerror(
	  " Could not allocate memory for blue colorstretch_list to structure");
      return;
    }
    Stretch[stretch]->color[BLUE].index=CT_Colors[BLUE]->adjust.index;
    for (n=0;n<=CT_Colors[BLUE]->adjust.index;n++) {
       Stretch[stretch]->color[BLUE].from[n]=CT_Colors[BLUE]->adjust.from[n];
       Stretch[stretch]->color[BLUE].to[n]=   CT_Colors[BLUE]->adjust.to[n];
    }
    Stretch[stretch]->color[STRETCHSAT].from=
	(long *)malloc(sizeof(long) * (CT_Colors[STRETCHSAT]->adjust.index+1));
    if (Stretch[stretch]->color[STRETCHSAT].from == NULL) {
      ct_displayerror(
	" Could not allocate memory for blue colorstretch_list from structure");
      return;
    }
    Stretch[stretch]->color[STRETCHSAT].to=
	(long *)malloc(sizeof(long) * (CT_Colors[STRETCHSAT]->adjust.index+1));
    if (Stretch[stretch]->color[STRETCHSAT].to == NULL) {
      ct_displayerror(
	  " Could not allocate memory for blue colorstretch_list to structure");
      return;
    }
    Stretch[stretch]->color[STRETCHSAT].index=CT_Colors[STRETCHSAT]->adjust.index;
    for (n=0;n<=CT_Colors[STRETCHSAT]->adjust.index;n++) {
       Stretch[stretch]->color[STRETCHSAT].from[n]=CT_Colors[STRETCHSAT]->adjust.from[n];
       Stretch[stretch]->color[STRETCHSAT].to[n]=   CT_Colors[STRETCHSAT]->adjust.to[n];
    }
}
/*****************************************************************************/
/*
	The function ct_add_stretch adds the current contrast stretch 
	information to the stretch list.  This occurs when the user
	selects Add from the Stretch Tool.
*/
/*****************************************************************************/

void
ct_add_stretch ()
{
    int		n, strsize;
    char	*str;

    /*
      1. check that maximum number of stretches is not exceeded 
      2. allocate all of structure XID_COLORSTRETCH
      3. copy each element of XID_COLORTOOL into matching 
	 elements of XID_COLORSTRETCH
      4. update NumStretch
    */
    if ( NumStretch == MAXSTRETCHES ) {
      ct_displaywarning("Cannot store any more stretches");
      return;
    }

    ct_add_stretch_item (NumStretch);

    strsize= 14 * ( Stretch[NumStretch]->color[RED].index + 
		   Stretch[NumStretch]->color[GREEN].index + 
		   Stretch[NumStretch]->color[BLUE].index + 
		   Stretch[NumStretch]->color[STRETCHSAT].index   + 4);

    if ((str=(char *)malloc(sizeof(char) * strsize)) == NULL) {
      ct_displayerror(
	  " Could not allocate memory for list entry string...exitting");
      exit(0);
    }
    ct_stretchlistentytext(str,NumStretch); /* convienence format list entry */

    CurrStretch=NumStretch;
    NumStretch++;
    XmListAddItem(colortool_stretch_list, 
	XmStringCreateLtoR(str,XmSTRING_DEFAULT_CHARSET), NumStretch);
    XmListSelectPos(colortool_stretch_list, NumStretch, False);
    XmListSetBottomPos(colortool_stretch_list, NumStretch);
    free(str);
}
/*****************************************************************************/
/*
	The function ct_delete_stretch deletes the currently selected 
	stretch from the stretch list.  This occurs when the user selects
	Delete from the Stretch List Tool after first selecting the stretch
	to delete with MB1.
*/
/*****************************************************************************/


void
ct_delete_stretch ()
{
    int		n, strsize;
    char	*str;


     /*
	  1. check that there is a stretch_list and that one stretch is selected
	  2. free structures in stretch_list, from, to for each color, and main for selected element 
	  3. move stretch_list array pointers down (if necessary) to fill array.
	  4. delete selected item in list
	  5. deselect the current position in list.
	  6. decrement NumStretch variable.
      */
       if ( NumStretch == 0 ) {
	  ct_displaywarning("No stretches to delete");
	  return;
       }
       if ( CurrStretch == -1 ) {
	  ct_displaywarning("No stretch selected");
	  return;
       }
#ifdef CURRSTRETCH
       if ( CurrStretch <= LINEAR)
       {
	   ct_displaywarning ("Cannot delete standard stretch list item");
	   return;
	}
#endif

       free(Stretch[CurrStretch]->color[RED].from);
       free(Stretch[CurrStretch]->color[RED].to);
       free(Stretch[CurrStretch]->color[GREEN].from);
       free(Stretch[CurrStretch]->color[GREEN].to);
       free(Stretch[CurrStretch]->color[BLUE].from);
       free(Stretch[CurrStretch]->color[BLUE].to);
       free(Stretch[CurrStretch]->color[STRETCHSAT].from);
       free(Stretch[CurrStretch]->color[STRETCHSAT].to);
       free(Stretch[CurrStretch]);

       XmListDeletePos(colortool_stretch_list,CurrStretch+1);
       XmListDeselectPos(colortool_stretch_list,CurrStretch+1);
       for (n=CurrStretch;n<(NumStretch - 1);n++) {
	   Stretch[n]= Stretch[n+1];
	   ct_upd_list_entry (n);
       }
       NumStretch--;
       CurrStretch = -1;
}
/*****************************************************************************/
/*
	The function ct_update_stretch updates the selected stretch list
	entry with information from the current contrast stretch.  
	This occurs when the user selects Update from the Stretch List Tool.
*/
/*****************************************************************************/


void
ct_update_stretch ()
{
    int		n, strsize;
    char	*str;
    XmString	xmstr;


     /*
	  1. check that there is a stretch_list and that one stretch is selected
	  2. free structures in stretch_list, from, to for each color, and main for selected element 
	  3. update structure in stretch_list, from, to for each color
	  5. replace current position in list with new value
      */
       if ( NumStretch == 0 ) {
	  ct_displaywarning("No stretches to update");
	  return;
       }
       if ( CurrStretch == -1 ) {
	  ct_displaywarning("No stretch selected");
	  return;
       }
#ifdef CURRSTRETCH
       if ( CurrStretch <= LINEAR)
       {
	   ct_displaywarning ("Cannot update standard stretch list item");
	   return;
	}
#endif

       free(Stretch[CurrStretch]->color[RED].from);
       free(Stretch[CurrStretch]->color[RED].to);
       free(Stretch[CurrStretch]->color[GREEN].from);
       free(Stretch[CurrStretch]->color[GREEN].to);
       free(Stretch[CurrStretch]->color[BLUE].from);
       free(Stretch[CurrStretch]->color[BLUE].to);
       free(Stretch[CurrStretch]->color[STRETCHSAT].from);
       free(Stretch[CurrStretch]->color[STRETCHSAT].to);
       free(Stretch[CurrStretch]);

	ct_add_stretch_item(CurrStretch);

	ct_upd_list_entry (CurrStretch);

	XmListSelectPos(colortool_stretch_list, CurrStretch+1, True);
	XmListSetBottomPos(colortool_stretch_list, CurrStretch+1);
}

/*****************************************************************************/
/*
	The function ct_reset_stretch initializes the current contrast stretch
	to a linear ramp for the current color selected.  This function is
	called when the user selects Linear from the X Color Tools display.
*/
/*****************************************************************************/


void
ct_reset_stretch ()
{
   /*
      1. erase current graphic displays
      2. initialize each of the XID_COLORTOOL structures to initial values
      3. apply the stretch to the colortable
      4. update the graphic displays
      5. unhighlight the list entry if any.
   */
   int	color_index, color_begin_index, color_end_index;
   int 		n, size=SIZEBREAKPTBOX;

   void         ct_drawline(),
                ct_histo_draw(),
                ct_apply();

   if (CT_Color->adjust.flags & MAP_ALL)
   {
       color_begin_index = 0;
       color_end_index = 4;
   }
   else
   {
       for (color_index=0; color_index<3; color_index++)
	    if (CT_Color == CT_Colors[color_index]) 
		    break;
       color_begin_index = 
       color_end_index = color_index;
   }
   for (color_index = color_begin_index; 
	color_index <=color_end_index; color_index++) 
   {
       CT_Color = CT_Colors [color_index];
       ct_drawline(CT_Color->window, 
	    CT_Color->gc, 
	    CT_Color->adjust.from, 
	    CT_Color->adjust.to,
	    CT_Color->adjust.index, 
	    CT_Color->background, size+5);

       for (n=0;n<=CT_Color->adjust.index;n++) {
	   CT_Color->adjust.from[n]=  0;
	   CT_Color->adjust.to[n]=    0;
       }

       CT_Color->adjust.index=        1;
       CT_Color->adjust.from[1]=      255;
       CT_Color->adjust.to[1]=        255;
       ct_apply(CT_Color);

       ct_drawline(CT_Color->window, 
		   CT_Color->gc, 
		   CT_Color->adjust.from, 
		   CT_Color->adjust.to,
		   CT_Color->adjust.index, 
		   CT_Color->linecolor, size);
    }
    ct_histo_draw (CT_Color);
    ct_new_curr_stretch();
}
/*****************************************************************************/
/*
	The function ct_stretchoperations is a common function for stretch
	list callbacks.  The function is called with an integer data parameter
	to specify the specific action to be performed.  
	Valid ct_stretchoperations cases
		case 0:	 Add - ct_stretch_add_button
		case 1:  Delete - ct_stretch_del_button
		case 2:  Linear - ct_linear_button
		case 3:  Clear - ct_stretch_clear
		case 4:  Load - ct_stretch_load_button
		case 5:  Save - ct_stretch_save_button
		case 6:  Restore - ct_restore_button
		case 7:  Update - ct_stretch_upd_button
		case 8:  Save New - ct_stretch_save_new_button

*/
/*****************************************************************************/


#ifdef _NO_PROTO
void ct_stretchoperations(w, data, call_data)
Widget w;
XtPointer data;
XmAnyCallbackStruct *call_data;
#else
void ct_stretchoperations(Widget w, XtPointer data, XmAnyCallbackStruct *call_data)
#endif
{
   XtAppContext context=XtWidgetToApplicationContext(w);
   extern void  ct_drawline();
   char         *str, msg[160];
   int          i, m, n, strsize, size=5;

   switch((int)data) {
       case 0: /* Save stretch to list:   */

	       ct_add_stretch ();
               break;

       case 1: /* delete stretch from list */

	       ct_delete_stretch ();
               break;

       case 2: /* reset stretch to initial values */
	       
	       ct_reset_stretch ();
               break;

       case 3: /* clear stretch list 
                  1. free each of the XID_COLORSTRETCH structures
                  2. remove each entry from the list.
               */
               if ( NumStretch != 0 ) {
                  ct_get_confirmation(
		      "Do you really want to clear the stretch list?", 
		      ct_clear_stretch_list, NULL);
               }
               break;
       case 4: /* load stretch file 
                  if a stretch list exists, warn user that existing stretch list will be overwritten
                  give user opportunity to cancel
		  if a stretch list exists, clear the stretch list and deallocate structures 
               */
	       if (NumStretch == FIRSTSTRETCH)
		   XtManageChild(colortool_load_stretch_file_selector); 
	       else
	       {
		   sprintf(msg,
		       "This operation will overwrite current stretch list!\n");
		   strcat(msg,"Use the cancel button to abort load");
		   ct_get_confirmation(msg, ct_accept_loadclear, NULL);
		}
               break;
       case 5: /* save stretch file */
               if ( NumStretch == 0) {
		  ct_displaywarning("No stretches on list to save!");
                  break;
               }
	       if (StretchFile == NULL)
		   XtManageChild(colortool_save_stretch_file_selector); 
		else
		    ct_savestretchfile_func (StretchFile);
               break;
	case 6:	/*  Restore original colormap	*/
		/*  Tom K.  Restore the original colormap */
		map_restore (ct_restore_button, 
			(XtPointer) NULL, (XtPointer) NULL);
		if (CT_Color == &allrgb)
 		    { 
		    allrgb.adjust.index=   xid.red_count - 1;
		    for (i=0; i < xid.red_count; i++)
		        {
		        allrgb.adjust.from[i] = xid.red_from[i];
		        allrgb.adjust.to[i] = xid.red_to[i];
		        }
		    piece(xid.red_from, xid.red_to, &xid.red_count, allrgb.map);
		    }
		else if (CT_Color == &red)
 		    { 
		    red.adjust.index=   xid.red_count - 1;
		    for (i=0; i < xid.red_count; i++)
		        {
		        red.adjust.from[i] = xid.red_from[i];
		        red.adjust.to[i] = xid.red_to[i];
		        }
		    piece(xid.red_from, xid.red_to, &xid.red_count, red.map);
		    }
		else if (CT_Color == &green)
 		    { 
		    green.adjust.index=   xid.green_count - 1;
		    for (i=0; i < xid.green_count; i++)
		        {
		        green.adjust.from[i] = xid.green_from[i];
		        green.adjust.to[i] = xid.green_to[i];
		        }
		    piece(xid.green_from, xid.green_to, &xid.green_count, 
			green.map);
		    }
		else if (CT_Color == &blue)
 		    { 
		    blue.adjust.index=   xid.blue_count - 1;
		    for (i=0; i < xid.blue_count; i++)
		        {
		        blue.adjust.from[i] = xid.blue_from[i];
		        blue.adjust.to[i] = xid.blue_to[i];
		        }
		    piece(xid.blue_from, xid.blue_to, &xid.blue_count, 
			blue.map);
		    }
		XClearArea(xid.display, XtWindow(ct_stretch_drawing), 0, 0,
		    0, 0, True);
		/*
		XClearArea(xid.display, XtWindow(ct_mainForm), 0, 0,
		    0, 0, True);
		*/
		break;

	case 7: /* Update current list element	*/
		
	       ct_update_stretch();

		break;

	case 8: /* Save New */
		
	       XtManageChild(colortool_save_stretch_file_selector); 

		break;
      default:				/* no currently defined operation */
               printf("no currently defined operation for %d\n",data);
/*
	       ct_displaywarning("Stretch Operations -- No currently defined operation");
*/
               break;
   }
}
