/*
 *  This is the fill function source file for the fill panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By L.Huewe, USGS/EROS Data Center, 7/94.
 *    - X Image Display, v1.0.
 *
 */
#include <math.h>
#include "xid.h"
#include "pixmaps.h"
#include "../xid_widgets/ProcessM.h"
#include "../xid_widgets/Context.h"

#define SETPOS_FROM	1
#define SETPOS_TO	2

int long_compare(e1, e2)
void *e1, *e2;
/*****************************************************************************
Purpose:
    This routine is the comparsion function used by the "qsort" utility.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
******************************************************************************/
{
long *v1, *v2;

v1 = (long *)(e1);
v2 = (long *)(e2);
return(*v1 < *v2) ? -1 : (*v1 > *v2) ? 1 : 0;
}

void
fill_apply(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    FILL_APPLY determines the options chosen by the user, calculates the
    bounding rectangle of each polygon selected, calculates any mappings 
    necessary, generates the new image, recalculates the histogram information 
    and displays the filled image.

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
void	fill_setarea();		/* Function for setting bounding rectangles.  */

long	amax,                   /* Maximum line of all polygons.              */
	amin,                   /* Minimum line of all polygons.              */
	*area,   		/* Bounding rectangle for each polygon.       */
	count,			/* Histogram count returned from c_pxhist.    */
	dtype = EBYTE,		/* Data type of image data.		      */
	hist_type = ELONG,	/* Histogram data type.			      */
	map[256],		/* Mapping to be applied to image data.       */
	nbins = 256,		/* Number of histogram bins.		      */
	np,			/* Number of pixels in image data array.      */
	zero = 0;		/* Constant zero.			      */

int	display_idx[3],		/* Memory plane numbers to display.	      */
	i,			/* Looping index.			      */
	nplanes,		/* Number of planes currently displayed.      */
        orig_plane_idx[3],      /* Indices into memory plane structure of the
				   currently viewed image.                    */
        plane_idx[3],      	/* Indices into memory plane structure of the
				   currently viewed image.                    */
	status;			/* Status indicator.			      */

/*
    Return if no polygons were selected to fill.
*/
if (xid_fill.num_poly == 0)
    {
    xid_message("User must select which polygons to fill");
    return;
    }

/*
    If the user has chosen to fill in the full resolution window but one
    is not displayed, return.
*/
if ((xid_fill.flags & FILL_FULLRES) && (xid_fullres == NULL))
    {
    xid_message("No full resolution window currently displayed");
    return;
    }

/*
    If the user is filling polygons with a piecewise linear mapping, make sure 
    at least 2 break points have been selected.
*/
if ((xid_fill.flags & FILL_PWL) && (xid_fill.num < 2))
    {
    xid_message("User must specify at least 2 break points for piecewise linear mapping");
    return;
    }
/*
    If the user is filling polygons with a renumber mapping, make sure 
    at least 1 from/to pair has been selected.
*/
else if ((xid_fill.flags & FILL_RENUMBER) && (xid_fill.num < 1))
    {
    xid_message("User must specify at least 1 from/to assignment for renumbering");
    return;
    }

/*
    Set up the processing message widget with the correct title.
*/
XtVaSetValues(load_dialog_shell, XmNtitle, "Fill - Processing", NULL);

/*
    Display a processing message.
*/
XeProcessMBegin(process_widget, 0, 1, 
    "Calculating filled image, please wait ...  ", True, False, False);

/*
    Allocate space for the bounding rectangle of each polygon selected.
*/
area = (long *)malloc(xid_fill.num_poly * 4 * sizeof(long));
if (area == NULL)
    {
    xid_message("Insufficient amount of memory available");
    return;
    }

/*
    Find the memory plane indices of the displayed image.
*/
if ((xid_fill.flags & FILL_UPDATE) || (xid_fill.flags & FILL_FULLRES))
    {
    plane_idx[RED] = xid_view.rgbplanes[RED] - 1;
    plane_idx[GREEN] = xid_view.rgbplanes[GREEN] - 1;
    plane_idx[BLUE] = xid_view.rgbplanes[BLUE] - 1;

/*
    Determine the number of planes displayed.
*/
    if ((plane_idx[RED] == plane_idx[GREEN]) && (plane_idx[GREEN] == 
	plane_idx[BLUE]))
	    nplanes = 1;
    else
	    nplanes = 3;
    }
else
    {
/*
    Find the memory plane indices of the displayed image.
*/
    orig_plane_idx[RED] = xid_view.rgbplanes[RED] - 1;
    orig_plane_idx[GREEN] = xid_view.rgbplanes[GREEN] - 1;
    orig_plane_idx[BLUE] = xid_view.rgbplanes[BLUE] - 1;
/*
    Determine the number of planes displayed.
*/
    if ((orig_plane_idx[RED] == orig_plane_idx[GREEN]) && 
	(orig_plane_idx[GREEN] == orig_plane_idx[BLUE]))
	    nplanes = 1;
    else
	    nplanes = 3;

/*
    Find the appropriate number of memory planes for the new image and copy
    information from the original planes to the new planes.
*/
    status = fill_new_image(nplanes, orig_plane_idx, plane_idx);
    if (status != XID_S)
	return;
    }

/*
    Calculate the bounding rectangle for each selected polygon.
*/
if (xid_fill.flags & FILL_MAIN)
    fill_setarea(xid_planes[plane_idx[RED]].window, &amin, &amax, area);
else
    fill_setarea(xid_fullres[RED].window, &amin, &amax, area);

/*
    Generate the appropriate mapping.
*/
if (xid_fill.flags & FILL_PWL)
    piece(xid_fill.from, xid_fill.to, &xid_fill.num, map);
else if (xid_fill.flags & FILL_RENUMBER)
    renumber(xid_fill.from, xid_fill.to, &xid_fill.num, map);

/*
    Calculate the filled image.
*/
status = fill_pfill(plane_idx, map, &amin, &amax, area);
if (status != XID_S)
    {
    xid_message("Error calculating filled image"); 
    return;
    }
free(area);

/*
    Recalculate the histogram and statistics based on the new image data.
*/
if (xid_fill.flags & FILL_MAIN)
    {
    for (i=0; i < nplanes; i++)
	{
	np = xid_planes[plane_idx[i]].dspwin[NL] * xid_planes[plane_idx[i]].
	    dspwin[NS];
	c_pxfill(xid_planes[plane_idx[i]].histogram, &hist_type, &nbins, &zero);
	c_pxhist(&np, xid_planes[plane_idx[i]].image_data, &dtype, 
	    xid_planes[plane_idx[i]].histogram, &count);
	xid_histcalc(xid_planes[plane_idx[i]].histogram, 
	    &xid_planes[plane_idx[i]].minval, &xid_planes[plane_idx[i]].maxval, 
	    &xid_planes[plane_idx[i]].mean, &xid_planes[plane_idx[i]].stdev);
	}
    }
else
    {
    for (i=0; i < nplanes; i++)
	{
	np = xid_fullres[i].dspwin[NL] * xid_fullres[i].dspwin[NS];
	c_pxfill(xid_planes[plane_idx[i]].histogram, &hist_type, &nbins, &zero);
	c_pxhist(&np, xid_fullres[i].image_data, &dtype, 
	    xid_fullres[i].histogram, &count);
	xid_histcalc(xid_fullres[i].histogram, &xid_fullres[i].minval, 
	    &xid_fullres[i].maxval, &xid_fullres[i].mean, 
	    &xid_fullres[i].stdev);
	}
    }

/*
    Free any previously allocated pixmaps.
*/
if (xid_fill.flags & FILL_MAIN)
    {
    if (xid_mem.imgpix)
	{
	XFreePixmap(xid.display,xid_mem.image_pixmap);
        XSetWindowBackground(xid.display,XtWindow(main_image_bulletinboard),
            xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
        XClearWindow(xid.display,XtWindow(main_image_bulletinboard));
	xid_mem.imgpix = FALSE;
	xid_mem.image_pixmap = (Pixmap)NULL;
	}
    }
else
    for (i=0; i < xid.num_fullres; i++)
        if (xid_fullres[i].image_pixmap != NULL)
            {
            XFreePixmap(xid.display, xid_fullres[i].image_pixmap);
	    XSetWindowBackground(xid.display,XtWindow(
		fullres_win_bulletinboard), xid.color_table[RGBBW_IDX + 
 		BLACK_COL].pixel);
	    XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));
            xid_fullres[i].image_pixmap = (Pixmap)NULL;
            }

/*
    Call load_display to display the filled image. 
*/
display_idx[RED] = plane_idx[RED] + 1;
display_idx[GREEN] = plane_idx[GREEN] + 1;
display_idx[BLUE] = plane_idx[BLUE] + 1;
if ((xid_fill.flags & FILL_MAIN) && (nplanes == 1))
    load_display(display_idx, &nplanes, XtWindow(
	main_image_bulletinboard), Shell_MainWin, main_image_bulletinboard,
	&xid_planes[plane_idx[RED]].dspwin[NL],
	&xid_planes[plane_idx[RED]].dspwin[NS],
	&xid_planes[plane_idx[RED]].maxval,
	xid_planes[plane_idx[RED]].image_data, NULL, NULL, TRUE,
	&xid_mem.image_pixmap, TRUE, FALSE, FALSE);
else if (xid_fill.flags & FILL_MAIN)
    load_display(display_idx, &nplanes, XtWindow(
	main_image_bulletinboard), Shell_MainWin, main_image_bulletinboard,
        &xid_planes[plane_idx[RED]].dspwin[NL],
        &xid_planes[plane_idx[RED]].dspwin[NS],
	&xid_planes[plane_idx[RED]].maxval,
	xid_planes[plane_idx[RED]].image_data,
	xid_planes[plane_idx[GREEN]].image_data,
	xid_planes[plane_idx[BLUE]].image_data, TRUE,
	&xid_mem.image_pixmap, TRUE, FALSE, FALSE);
else if ((xid_fill.flags & FILL_FULLRES) && (nplanes == 1))
    load_display(display_idx, &nplanes, XtWindow(fullres_win_bulletinboard),
	Shell_FullResWin, fullres_win_bulletinboard, &xid_fullres[0].dspwin[NL],
        &xid_fullres[0].dspwin[NS], &xid_planes[plane_idx[RED]].maxval, 
	xid_fullres[0].image_data, NULL, NULL, FALSE, &xid_fullres[0].
        image_pixmap, FALSE, FALSE, FALSE);
else if (xid_fill.flags & FILL_FULLRES)
    load_display(display_idx, &nplanes, XtWindow(fullres_win_bulletinboard),
	Shell_FullResWin, fullres_win_bulletinboard, &xid_fullres[0].dspwin[NL],
        &xid_fullres[RED].dspwin[NS], &xid_planes[plane_idx[RED]].maxval, 
	xid_fullres[RED].image_data, xid_fullres[GREEN].image_data, 
	xid_fullres[BLUE].image_data, FALSE, &xid_fullres[0].
        image_pixmap, FALSE, FALSE, FALSE);
/*
    Close the processing message window.
*/
XeProcessMEnd(process_widget);
}

void fill_breakpts()

/*******************************************************************************
Purpose:
    This routine creates a dialog panel for entering break point values for
    a piece-wise linear mapping or for entering from and to values for a 
    renumber mapping.

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
Arg    	args[512];
Cardinal
	argcnt;
XmString
	xmstr[32];

void	fill_move_scrollbar();
void	fill_ok_breakpts();

Widget	fill_pwl_form;
Widget	fill_pwl_help_button;
Widget	fill_set_button;
Widget	fill_from_scrolled_window;
Widget	fill_to_scrolled_window;
Widget	fill_delete_button;
Widget	fill_clear_button;
Widget	fill_from_label;
Widget	fill_to_label;


/*
    This does not need to be created every time since it is not destroyed 
    until the fill panel is closed.
*/
if (fill_breakpt_dialog == NULL)
    {
/*
    Create the prompt dialog. 
*/
    argcnt = 0;
    if (xid_fill.flags & FILL_PWL)
	{
	XtSetArg(args[argcnt], XmNtitle, "Fill -- Piece-Wise Linear"); argcnt++;
	}
    else
	{
	XtSetArg(args[argcnt], XmNtitle, "Fill -- Renumber"); argcnt++;
	}
    XtSetArg(args[argcnt], XmNautoUnmanage, False); argcnt++;
    XtSetArg(args[argcnt], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);argcnt++;
    XtSetArg(args[argcnt], XmNdeleteResponse, XmDESTROY); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
    XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
    fill_breakpt_dialog = XmCreatePromptDialog(fill_form,
		"breakptsDialog",
		args,
		argcnt);

    XtUnmanageChild(XmSelectionBoxGetChild(fill_breakpt_dialog, XmDIALOG_TEXT));
    XtUnmanageChild(XmSelectionBoxGetChild(fill_breakpt_dialog, 
	XmDIALOG_SELECTION_LABEL));
    XtUnmanageChild(XmSelectionBoxGetChild(fill_breakpt_dialog, 
	XmDIALOG_HELP_BUTTON));
    XtUnmanageChild(XmSelectionBoxGetChild(fill_breakpt_dialog, 
	XmDIALOG_CANCEL_BUTTON));

/*
    Set up OK callback.
*/
    XtAddCallback(fill_breakpt_dialog, XmNokCallback, fill_ok_breakpts, 0);

/*
    Create the form widget.
*/
    XtInitializeWidgetClass(xmFormWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNautoUnmanage, False); argcnt++;
    XtSetArg(args[argcnt], XmNresizePolicy, XmRESIZE_GROW); argcnt++;
    XtSetArg(args[argcnt], XmNx, 0); argcnt++;
    XtSetArg(args[argcnt], XmNy, 0); argcnt++;
    fill_pwl_form = XtCreateWidget("fill_pwl_form",
		xmFormWidgetClass,
		fill_breakpt_dialog,
		args,
		argcnt);
    XtManageChild(fill_pwl_form);

/*
    Create the "To" label.
*/
    XtInitializeWidgetClass(xmLabelWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
		"To List:", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNrecomputeSize, True); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNbottomOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 25); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 0); argcnt++;
    fill_to_label = XtCreateWidget("fill_to_label",
		xmLabelWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );
    XtManageChild(fill_to_label);

/*
    Create the "From" label.
*/
    XtInitializeWidgetClass(xmLabelWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
		"From List:", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNrecomputeSize, True); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNbottomOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 20); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 0); argcnt++;
    fill_from_label = XtCreateWidget("fill_from_label",
		xmLabelWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );
    XtManageChild(fill_from_label);

/*
    Create the "Delete" button.
*/
    XtInitializeWidgetClass(xmPushButtonWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNshadowThickness, 2); argcnt++;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
		"Delete", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNrecomputeSize, False); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 215); argcnt++;
    XtSetArg(args[argcnt], XmNbottomOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNx, 10); argcnt++;
    XtSetArg(args[argcnt], XmNy, 215); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 83); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 30); argcnt++;
    fill_delete_button = XtCreateWidget("fill_delete_button",
		xmPushButtonWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );
    XtAddCallback(fill_delete_button, XmNactivateCallback, fill_delete_breakpt, (XtPointer)0);
    XtManageChild(fill_delete_button);

/*
    Create the "Clear" button.
*/
    XtInitializeWidgetClass(xmPushButtonWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNshadowThickness, 2); argcnt++;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
		"Clear", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNrecomputeSize, False); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 175); argcnt++;
    XtSetArg(args[argcnt], XmNbottomOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNx, 10); argcnt++;
    XtSetArg(args[argcnt], XmNy, 190); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 83); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 30); argcnt++;
    fill_clear_button = XtCreateWidget("fill_clear_button",
		xmPushButtonWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );

    XtAddCallback(fill_clear_button, XmNactivateCallback, fill_clear_breakpts, (XtPointer)0);
    XtManageChild(fill_clear_button);

/*
    Create the "To" scrolled list.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNscrollBarDisplayPolicy, XmAS_NEEDED); argcnt++;
    XtSetArg(args[argcnt], XmNlistSizePolicy, XmCONSTANT); argcnt++;
    XtSetArg(args[argcnt], XmNscrollingPolicy, XmAPPLICATION_DEFINED); argcnt++;
    XtSetArg(args[argcnt], XmNvisualPolicy, XmCONSTANT); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 25); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNtopWidget, fill_to_label); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 5); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNbottomOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 54); argcnt++;
    XtSetArg(args[argcnt], XmNvisibleItemCount, 5); argcnt++;
    fill_to_list = XmCreateScrolledList(fill_pwl_form, "fill_to_list", args, 
	argcnt);
    XtAddCallback(fill_to_list, XmNbrowseSelectionCallback, fill_list,
	(XtPointer)SETPOS_FROM);
    XtManageChild(fill_to_list);
    fill_to_scrolled_window = XtParent(fill_to_list);

/*
    Set a callback on the vertical scrollbar so that the from list 
    can scroll along with the to list.
*/
    XtVaGetValues(fill_to_scrolled_window, XmNverticalScrollBar, 
	&fill_to_scrollBar, NULL);
    XtAddCallback(fill_to_scrollBar, XmNdragCallback, fill_move_scrollbar, 
	(XtPointer)NULL);
    XtAddCallback(fill_to_scrollBar, XmNvalueChangedCallback, 
	fill_move_scrollbar, (XtPointer)NULL);
    XtAddCallback(fill_to_scrollBar, XmNdecrementCallback, fill_move_scrollbar, 
	(XtPointer)NULL);
    XtAddCallback(fill_to_scrollBar, XmNincrementCallback, fill_move_scrollbar, 
	(XtPointer)NULL);
    XtAddCallback(fill_to_scrollBar, XmNpageDecrementCallback, 
	fill_move_scrollbar, (XtPointer)NULL);
    XtAddCallback(fill_to_scrollBar, XmNpageIncrementCallback, 
	fill_move_scrollbar, (XtPointer)NULL);

/*
    Create the "From" scrolled list.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNscrollBarDisplayPolicy, XmAS_NEEDED); argcnt++;
    XtSetArg(args[argcnt], XmNlistSizePolicy, XmCONSTANT); argcnt++;
    XtSetArg(args[argcnt], XmNscrollingPolicy, XmAPPLICATION_DEFINED); argcnt++;
    XtSetArg(args[argcnt], XmNvisualPolicy, XmCONSTANT); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 25); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNleftWidget, fill_delete_button); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNtopWidget, fill_from_label); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 5); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNbottomOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 54); argcnt++;
    XtSetArg(args[argcnt], XmNvisibleItemCount, 5); argcnt++;
    fill_from_list = XmCreateScrolledList(fill_pwl_form, "fill_from_list", args, 
	argcnt);
    XtAddCallback(fill_from_list, XmNbrowseSelectionCallback, fill_list,
	(XtPointer)SETPOS_TO);
    XtManageChild(fill_from_list);
    fill_from_scrolled_window = XtParent(fill_from_list);

/*
    Set a callback on the vertical scrollbar so that the to list 
    can scroll along with the from list.
*/
    XtVaGetValues(fill_from_scrolled_window, XmNverticalScrollBar,
	&fill_from_scrollBar, NULL);
    XtAddCallback(fill_from_scrollBar, XmNdragCallback, fill_move_scrollbar, 
	(XtPointer)NULL);
    XtAddCallback(fill_from_scrollBar, XmNvalueChangedCallback, 
	fill_move_scrollbar, (XtPointer)NULL);
    XtAddCallback(fill_from_scrollBar, XmNdecrementCallback, 
	fill_move_scrollbar, (XtPointer)NULL);
    XtAddCallback(fill_from_scrollBar, XmNincrementCallback, 
	fill_move_scrollbar, (XtPointer)NULL);
    XtAddCallback(fill_from_scrollBar, XmNpageDecrementCallback, 
	fill_move_scrollbar, (XtPointer)NULL);
    XtAddCallback(fill_from_scrollBar, XmNpageIncrementCallback, 
	fill_move_scrollbar, (XtPointer)NULL);

/*
    Create the "Set" button.
*/
    XtInitializeWidgetClass(xmPushButtonWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNshadowThickness, 2); argcnt++;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
		"Set", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNrecomputeSize, False); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 50); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 20); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 83); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 30); argcnt++;
    fill_set_button = XtCreateWidget("fill_set_button",
		xmPushButtonWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );
    XtAddCallback(fill_set_button, XmNactivateCallback, fill_set_breakpt, (XtPointer)0);
    XtManageChild(fill_set_button);

/*
    Create the "To" scale widget.
*/
    XtInitializeWidgetClass(xmScaleWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNmaximum, 255); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, 0); argcnt++;
    XtSetArg(args[argcnt], XmNorientation, XmHORIZONTAL); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"To", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNshowValue, True); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNbottomOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 20); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 277); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNrightWidget, fill_set_button); argcnt++;
    fill_to_scale = XtCreateWidget("fill_to_scale",
		xmScaleWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );
    XtManageChild(fill_to_scale);

/*
    Create the "From" scale widget.
*/
    XtInitializeWidgetClass(xmScaleWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNmaximum, 255); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, 0); argcnt++;
    XtSetArg(args[argcnt], XmNorientation, XmHORIZONTAL); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"From", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNshowValue, True); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 20); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 277); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNrightWidget, fill_set_button); argcnt++;
    fill_from_scale = XtCreateWidget("fill_from_scale",
		xmScaleWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );
    XtManageChild(fill_from_scale);

/*
    Create the "Help" button.
*/
    XtInitializeWidgetClass(xmPushButtonWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNlabelType, 1); argcnt++;
    XtSetArg(args[argcnt], XmNlabelPixmap,
	XPM_PIXMAP(fill_form, helpxbm_width,helpxbm_height,
	helpxbm_ncolors, helpxbm_chars_per_pixel, helpxbm_colors,
	helpxbm_pixels)); argcnt++;
    XtSetArg(args[argcnt], XmNrecomputeSize, False); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNx, 369); argcnt++;
    XtSetArg(args[argcnt], XmNy, 10); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 31); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 31); argcnt++;
    fill_pwl_help_button = XtCreateWidget("fill_pwl_help_button",
		xmPushButtonWidgetClass,
		fill_pwl_form,
		args,
		argcnt);
    XtAddCallback(fill_pwl_help_button, XmNactivateCallback, help_update, (XtPointer)"fill.html#pwl");
    XtManageChild(fill_pwl_help_button);

/*
    Set form attachments.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtopOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNtopWidget, fill_from_scale); argcnt++;
    XtSetValues(fill_to_scale, args, argcnt);

    argcnt = 0;
    XtSetArg(args[argcnt], XmNleftOffset, 25); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNleftWidget, fill_from_scrolled_window); argcnt++;
    XtSetValues(fill_to_scrolled_window, args, argcnt);

    argcnt = 0;
    XtSetArg(args[argcnt], XmNtopOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNtopWidget, fill_to_scale); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 20); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNleftWidget, fill_delete_button); argcnt++;
    XtSetValues(fill_from_label, args, argcnt);

    argcnt = 0;
    XtSetArg(args[argcnt], XmNtopOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNtopWidget, fill_to_scale); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 25); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_WIDGET); argcnt++;
    XtSetArg(args[argcnt], XmNleftWidget, fill_from_scrolled_window); argcnt++;
    XtSetValues(fill_to_label, args, argcnt);
    }
return;
}

void
fill_clear_breakpts(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine removes all break points from the "from" and "to" lists.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
/*
    Delete all break points from the "from" list.
*/
if (fill_from_list != NULL)
    XmListDeleteAllItems(fill_from_list);
/*
    Delete all break points from the "to" list.
*/
if (fill_to_list != NULL)
    XmListDeleteAllItems(fill_to_list);
xid_fill.num = 0;
}

void fill_constant()

/*******************************************************************************
Purpose:
    This routine creates a dialog panel for entering a constant fill value. 

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
Arg         args[512];
Cardinal    argcnt;
XmString    xmstr[32];

Widget      fill_constant_form;
Widget      fill_constant_help_button;

void	    fill_ok_constant();

/*
    This does not need to be created every time since it is not destroyed
    until the fill panel is closed.
*/
if (fill_const_dialog == NULL)
    {
/*
    Create a prompt dialog.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "Fill -- Constant"); argcnt++;
    XtSetArg(args[argcnt], XmNautoUnmanage, False); argcnt++;
    XtSetArg(args[argcnt], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);argcnt++;
    XtSetArg(args[argcnt], XmNdeleteResponse, XmDESTROY); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
    XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
    fill_const_dialog = XmCreatePromptDialog(fill_form,
		"constantDialog",
		args,
		argcnt);

    XtUnmanageChild(XmSelectionBoxGetChild(fill_const_dialog, XmDIALOG_TEXT));
    XtUnmanageChild(XmSelectionBoxGetChild(fill_const_dialog, 
	XmDIALOG_SELECTION_LABEL));
    XtUnmanageChild(XmSelectionBoxGetChild(fill_const_dialog, 
	XmDIALOG_HELP_BUTTON));
    XtUnmanageChild(XmSelectionBoxGetChild(fill_const_dialog, 
	XmDIALOG_CANCEL_BUTTON));

/*
    Set up OK callback.
*/
    XtAddCallback(fill_const_dialog, XmNokCallback, fill_ok_constant, 0);

/*
    Create a form widget.
*/
    XtInitializeWidgetClass(xmFormWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNautoUnmanage, False); argcnt++;
    XtSetArg(args[argcnt], XmNresizePolicy, XmRESIZE_GROW); argcnt++;
    XtSetArg(args[argcnt], XmNx, 0); argcnt++;
    XtSetArg(args[argcnt], XmNy, 0); argcnt++;
    fill_constant_form = XtCreateWidget("fill_constant_form",
		xmFormWidgetClass,
		fill_const_dialog,
		args,
		argcnt);
    XtManageChild(fill_constant_form);

/*
    Create the constant value scale widget.
*/
    XtInitializeWidgetClass(xmScaleWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNmaximum, 255); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, 0); argcnt++;
    XtSetArg(args[argcnt], XmNorientation, XmHORIZONTAL); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"Constant Value", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNshowValue, True); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 45); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 350); argcnt++;
    fill_constant_scale = XtCreateWidget("fill_constant_scale",
		xmScaleWidgetClass,
		fill_constant_form,
		args,
		argcnt);
    XmStringFree( xmstr[0] );
    XtManageChild(fill_constant_scale);

/*
    Create the "Help" button.
*/
    XtInitializeWidgetClass(xmPushButtonWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNlabelType, 1); argcnt++;
    XtSetArg(args[argcnt], XmNlabelPixmap,
	XPM_PIXMAP(fill_form, helpxbm_width,helpxbm_height,
	helpxbm_ncolors, helpxbm_chars_per_pixel, helpxbm_colors,
	helpxbm_pixels)); argcnt++;
    XtSetArg(args[argcnt], XmNrecomputeSize, False); argcnt++;
    XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_NONE); argcnt++;
    XtSetArg(args[argcnt], XmNrightAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNtopAttachment, XmATTACH_FORM); argcnt++;
    XtSetArg(args[argcnt], XmNleftOffset, 0); argcnt++;
    XtSetArg(args[argcnt], XmNrightOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNtopOffset, 10); argcnt++;
    XtSetArg(args[argcnt], XmNx, 371); argcnt++;
    XtSetArg(args[argcnt], XmNy, 10); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 31); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 31); argcnt++;
    fill_constant_help_button = XtCreateWidget("fill_constant_help_button",
		xmPushButtonWidgetClass,
		fill_constant_form,
		args,
		argcnt);
    XtAddCallback(fill_constant_help_button, XmNactivateCallback, help_update, (XtPointer)"fill.html#constant");
    XtManageChild(fill_constant_help_button);
    }
return;
}

void
fill_delete_breakpt(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine removes the selected break point from the "from" and "to"
    lists.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
int	count,				/* Number of selected items.	      */
	i,				/* Looping index.		      */
	*pos_list;			/* List of selected items.	      */

/*
    Get the position of the selected item from the "from" list.
*/
XmListGetSelectedPos(fill_from_list, &pos_list, &count);

/*
    If there are no break points selected, there is nothing to delete.
*/
if (count == 0)
    {
    xid_message("No break point selected to delete");
    return;
    }

/*
    Delete the selected item from the scrolled lists.
*/
XmListDeletePos(fill_from_list, pos_list[0]);
XmListDeletePos(fill_to_list, pos_list[0]);
XtFree((char *)pos_list);

/*
    Remove the item from the "from" and "to" arrays.
*/
for (i = pos_list[0] - 1; i < xid_fill.num - 1; i++)
    {
    xid_fill.from[i] = xid_fill.from[i+1];
    xid_fill.to[i] = xid_fill.to[i+1];
    }
xid_fill.num--;
}

void
fill_dest(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether to generate the
    filled image in new image planes or to update the displayed image.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
/*
    Set the flag indicating the destination for the fill operation. 
*/
xid_fill.flags = xid_fill.flags & ~(FILL_NEW | FILL_UPDATE);
switch (*client)
    {
    case 'U':
	xid_fill.flags = xid_fill.flags | FILL_UPDATE;
	break;
    case 'N':
	xid_fill.flags = xid_fill.flags | FILL_NEW;
	break;
    }
}

void
fill_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    Close the FILL window and perform necessary clean up. 
    
By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
/*
    Close the fill window and set the fill button on the main 
    panel to sensitive.
*/
XtUnmanageChild(fill_form);
XtPopdown(Shell_Fill);
XtSetSensitive(main_fill_button,True);

/*
    Remove the "Fill" context.
*/
if (xid_fill.flags & FILL_USER)
    xid_contextdel(CON_FILL);

/*
    Unselect all polygons.
*/
fill_unselect_all();

/*
    Destroy any dialog widgets.
*/
if (fill_breakpt_dialog != NULL)
    {
    XtDestroyWidget(fill_breakpt_dialog);
    fill_breakpt_dialog = NULL;
    xid_fill.num = 0;
    }
if (fill_const_dialog != NULL)
    {
    XtDestroyWidget(fill_const_dialog);
    fill_const_dialog = NULL;
    xid_fill.constant = 0;
    }

/*
    Reset the active flag. 
*/
xid_fill.flags = xid_fill.flags ^ FILL_ACTIVE;
}

int
fill_isect(u, cline, x, y, nvert, sect, nsect)

float	*cline,		/* Current line number.	(I)		     	      */
	*x,		/* X coordinates of the polygon vertices. (I) 	      */
	*y;		/* Y coordinates of the polygon vertices. (I)	      */

long	*nvert,		/* Number of vertices. (I)		     	      */
	u[],		/* X coordinates of the line segment. (I)	      */
	**sect,		/* Intersection points of the current line with
 			   the line segments connecting the vertices.
			   (O) 						      */
	*nsect;		/* Number of intersection points. (O)	     	      */

/****************************************************************************
Purpose:
	Given the current line number and a list of polygon vertices, ISECT 
	determines the X coordinates of the intersection of the line with the 
	polygon.  ISECT always returns pairs of intersection points.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
	This routine was adopted from the DMS function named FILL.
*****************************************************************************/
{
static float 
	ep0 = 1.0e-5,	/* Constants used for determining where the	      */
	ep1l = 0.99999, /* intersection of the current line lies	      */
	ep1h = 1.0001;	/* in relation to the input polygon.		      */

float	du,		/* Change in the X coords of the line segment.	      */
	x1,y1,		/* XY coordinates of a polygon vertice.	     	      */
	x2,y2,		/* XY coordinates of a polygon vertice.	     	      */
	dx,dy,		/* Change in the line segment.		     	      */
	det,		/* Determinant.				     	      */
	dux,dvy,	/* Distance from U1 to X1 and from V1 to Y1.  	      */
	r,		/* Determines where intersection lies.	     	      */
	rd;		/* Relative distance from u1 to current line. 	      */

long	snum=0,		/* Number of sides that lie on the current 
			   line.				     	      */
	i,j,		/* Looping index.			     	      */
	inum=0,		/* Number of intersections found.	     	      */
	isect_max,	/* Current maximum index for "tsect".		      */
	*side, 		/* Sides of the polygon that lie directly on
			   the current line.			     	      */
	side_max,	/* Current maximum index for "side".		      */
    	*tsect;		/* Temporary array for storing intersections.

/*
    Allocate space for the array containing polygon sides that lie directly
    on the current line.
*/
side = (long *)malloc(FILL_MAXISECT * sizeof(long));
if (side == NULL)
    {
    xid_message("Insufficient memory available");
    return(XID_F);
    }
side_max = FILL_MAXISECT;

/*
    Allocate space for the array containing intersection points. 
*/
tsect = (long *)malloc(FILL_MAXISECT * sizeof(long));
if (tsect == NULL)
    {
    xid_message("Insufficient memory available");
    return(XID_F);
    }
isect_max = FILL_MAXISECT;

/*
    Calculate the change in X of the line segment passed in.
*/
du = (float)(u[1] - u[0]);

*nsect = 0;
x2 = x[*nvert - 1];
y2 = y[*nvert - 1];

/*
    Loop through all line segments of the polygon.
*/
for (i=0; i < *nvert; i++)
    {
    x1 = x2;
    y1 = y2;
    x2 = x[i];
    y2 = y[i];

/*
    Calculate the change in the line segment.
*/
    dx = x2 - x1;
    dy = y2 - y1;

/*
    Calculate the determinant.
*/
    det = - (du * dy);

    if (fabs(det) < ep0)
	{
	if (*cline == y1)

/*
    The line segment is lying on the current line.
*/
	    {
	    side[snum] = (long) min(x1,x2);
	    side[snum+1] = (long) max(x1,x2);
	    snum+=2;
/*
    If we've run out of room in the sides array, allocate more space.
*/
	    if (snum >= side_max) 
		{
		side_max += FILL_MAXISECT;
		side = (long *)realloc(side, side_max * sizeof(long));
		if (side == NULL)
		    {
		    xid_message("Insufficient memory available");
		    return(XID_F);
		    }
		}
	    }
	continue;
	}
/*
    Calculate the distance from the first vertice to the first endpoint of the 
    line segment passed in.
*/
    dux = (float)u[0] - x1;
    dvy = *cline - y1;

    r = (-(du * dvy)) / det;

/*
    Intersection is outside.
*/
    if ((r < -ep0) || (r > ep1h))
	continue;

    if (((r < ep0) && (det < 0)) || ((r > ep1l) && (det > 0)))
	continue;

/*
    Calculate the relative distance from u1 to v1.
*/
    rd = ((dy * dux) - (dx * dvy)) / det;

/*
    Calculate the intersection coordinate.
*/
    tsect[inum] = (long)(u[0] + rd * du + .5);
    inum++;
/*
    If we've run out of space in the intersection array, allocate more space.
*/
    if (inum >= isect_max) 
	{
	isect_max += FILL_MAXISECT;
	tsect = (long *)realloc(tsect, isect_max * sizeof(long));
	if (tsect == NULL)
	    {
	    xid_message("Insufficient memory available");
	    return(XID_F);
	    }
	}
    }
/*
    Sort the intersection points.
*/
qsort((char *)tsect, inum, sizeof(long), long_compare);

/*
    Make sure none of the intersection points are a side of the 
    polygon.
*/
for (i=0; i < snum; i+=2)
    for (j=0; j < inum; j+=2)
	if ((tsect[j] == side[i]) && (tsect[j+1] == side[i+1]))
	    tsect[j] = tsect[j+1] = -1;

for (i=0; i < inum; i++)
    if (tsect[i] != -1)
	(*nsect)++;

/*
    Allocate space for the array of intersection points returned.
*/
if (*nsect != 0)
    {
    *sect = (long *)malloc(*nsect * sizeof(long));
    if (*sect == NULL)
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
    }
/*
    Copy intersection points into the returned array.
*/
for (i=0, j=0; i < inum; i++)
    if (tsect[i] != -1)
	{
	*(*sect + j) = tsect[i];
	j++;
	}
/*
    Free dynamically allocated space.
*/
free(side);
free(tsect);
return(XID_S);
}

void
fill_list(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine retrieves the position of the selected item from the "from" 
    or to "list" and selects the corresponding item in the opposite list.

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
int	count,				/* Number of items selected.	      */
	option,				/* Which list are setting position
					   for?			      	      */
	*pos_list;			/* Array of selected items.	      */

/*
   Determine which list we are setting the position for.
*/
option = (int)client;

switch (option)
    {
    case SETPOS_FROM:
/*
    Get the position of the selected item from the "to" list.
*/
	XmListGetSelectedPos(fill_to_list, &pos_list, &count);
/*
    Set the corresponding position in the "from" list.
*/
	XmListSelectPos(fill_from_list, *pos_list, False);
	XtFree((char *)pos_list);
	break;
    case SETPOS_TO:
/*
    Get the position of the selected item from the "from" list.
*/
	XmListGetSelectedPos(fill_from_list, &pos_list, &count);
/*
    Set the corresponding position in the "to" list.
*/
	XmListSelectPos(fill_to_list, *pos_list, False);
	XtFree((char *)pos_list);
	break;
    }
}

int
fill_mark(element, plane_idx)

int	element,		/* Index to polygon record. (I)	              */
        plane_idx;              /* Index into memory plane structure of the
				   currently viewed image.  (I)               */


/*******************************************************************************
Purpose:
    This routine redraws the specified polygon with dashed lines indicating
    that it has been selected for filling.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
struct  POLY_ELEMENT    *pptr;  /* Pointer to polygon specific information.   */

float	x1,			/* First X coordinate of line segment.	      */
	x2,			/* Second X coordinate of line segment.	      */
	y1,			/* First Y coordinate of line segment.	      */
	y2;			/* Second Y coordinate of line segment.	      */

long    draw = FALSE,		/* Indicates whether or not there is text
				   (annotation or label) to draw.	      */
	i,			/* Looping index.			      */
	line_width,		/* Line width.				      */
	num_vert,		/* Number of vertices.			      */
	xwin,           	/* Window X coordinate.                       */
	x1win,          	/* Window X coordinate.                       */
        ywin,           	/* Window Y coordinate.                       */
	y1win;          	/* Window Y coordinate.                       */

char    *string;                /* Label string.                	      */

/*
    Erase the selected polygon from any displayed windows.
*/
graphics_deletelinepoly(0, element, plane_idx);
graphics_deletelinepoly(1, element, plane_idx);
graphics_deletelinepoly(2, element, plane_idx);

/*
    Make sure the background and foreground values are set properly in the GC.
*/
XSetBackground(xid.display,xid_graphics.rb_gc,0);
XSetForeground(xid.display,xid_graphics.rb_gc,1);

/*
    Determine how many vertices the polygon has and the line width of the 
    polygon.
*/
pptr = (struct POLY_ELEMENT *)xid_element[element].graph_ptr;
num_vert = pptr->num_vert;
line_width = pptr->line_width;

for (i=0; i < num_vert; i++)
    {
/*
    Convert the file coordinates to main image window coordinates.
*/
    if (i == num_vert - 1)
	{
	x1 = pptr->x[0];
	y1 = pptr->y[0];
	x2 = pptr->x[i];
	y2 = pptr->y[i];
	}
    else
	{
	x1 = pptr->x[i];
	y1 = pptr->y[i];
	x2 = pptr->x[i+1];
	y2 = pptr->y[i+1];
	}

    xid_file2win(plane_idx,0,x1,y1,&xwin,&ywin);
    xid_file2win(plane_idx,0,x2,y2,&x1win,&y1win);
/*
    Set the GC to draw dashed lines.
*/
    XSetLineAttributes(xid.display,xid_graphics.rb_gc,line_width,
	LineOnOffDash,CapButt,JoinMiter);
/*
    Draw the line segment.
*/
    XDrawLine(xid.display,xid_graphics.pixmap[xid_element[element].
	color],xid_graphics.rb_gc,xwin,ywin,x1win,y1win); 
/*
    Draw the line into the full resolution image window.
*/
    if ((xid_fullres != NULL) && (xid_fullres[0].image_pixmap != NULL))
	{
/*
    Convert the file coordinates to full resolution window coordinates.
*/
	xid_file2win(plane_idx,1,x1,y1,&xwin,&ywin);
	xid_file2win(plane_idx,1,x2,y2,&x1win,&y1win);
/*
    Set the GC to draw dashed lines.
*/
	XSetLineAttributes(xid.display,xid_graphics.rb_gc,line_width,
	    LineOnOffDash,CapButt,JoinMiter);
/*
    Draw the line segment.
*/
	XDrawLine(xid.display,xid_graphics.full_pix[xid_element
	    [element].color],xid_graphics.rb_gc,xwin,ywin,x1win,y1win); 
	}
/*
    Draw the line into the zoomed window.
*/
    if ((xid_zoom.flags & ZOOM_ACTIVE) && (xid_zoom.image_pixmap != 
	NULL))
	{
/*
    Convert the file coordinates to zoom window coordinates.
*/
	xid_file2win(plane_idx,2,x1,y1,&xwin,&ywin);
	xid_file2win(plane_idx,2,x2,y2,&x1win,&y1win);
/*
    Set the GC to draw dashed lines.
*/
	XSetLineAttributes(xid.display,xid_graphics.rb_gc,line_width,
	    LineOnOffDash,CapButt,JoinMiter);
/*
    Draw the line segment.
*/
	XDrawLine(xid.display,xid_graphics.zoom_pix[xid_element
	    [element].color],xid_graphics.rb_gc,xwin,ywin,x1win,y1win); 
	}
    }

/*
    Draw the label if one exists for the polygon.
*/
if (xid_element[element].labelx != -1)
    {
    string = (char *)malloc(strlen(xid_element[element].label)+1);
    if (string == NULL)
	{
	xid_message("Insufficient amount of memory available");
	return(XID_F);
	}
    strcpy(string,xid_element[element].label);
    draw = TRUE;
    }

if (draw)
    {
/*
    Convert the file coordinates to the main image window coordinates and
    draw the string.
*/
    xid_file2win(plane_idx,0,xid_element[element].labelx,
	xid_element[element].labely,&x1win,&y1win);
    XSetFont(xid.display,xid_graphics.rb_gc,xid_graphics.font_info[
	xid_element[element].font]->fid);
    XDrawString(xid.display,xid_graphics.pixmap[xid_element[element].color],
	xid_graphics.rb_gc,x1win,y1win + xid_graphics.font_info[
	xid_element[element].font]->max_bounds.ascent,string,
	strlen(string));

/*
    Convert the file coordinates to full resolution window coordinates and
    draw the string.
*/
    if ((xid_fullres != NULL) && (xid_fullres[0].image_pixmap != NULL))
	{
	xid_file2win(plane_idx,1,xid_element[element].labelx,
	    xid_element[element].labely,&x1win,&y1win);
	XSetFont(xid.display,xid_graphics.rb_gc,xid_graphics.font_info[
	    xid_element[element].font]->fid);
	XDrawString(xid.display,xid_graphics.full_pix[xid_element[element].
	    color],xid_graphics.rb_gc,x1win,y1win + xid_graphics.font_info[
	    xid_element[element].font]->max_bounds.ascent,string,
	    strlen(string));
	}

/*
    Convert the file coordinates to zoom window coordinates and draw the string.
*/
    if ((xid_zoom.flags & ZOOM_ACTIVE) && (xid_zoom.image_pixmap != NULL))
	{
	xid_file2win(plane_idx,2,xid_element[element].labelx,
	    xid_element[element].labely,&x1win,&y1win);
	XSetFont(xid.display,xid_graphics.rb_gc,xid_graphics.font_info[
	    xid_element[element].font]->fid);
	XDrawString(xid.display,xid_graphics.zoom_pix[xid_element[element].
	    color],xid_graphics.rb_gc,x1win,y1win + xid_graphics.font_info[
	    xid_element[element].font]->max_bounds.ascent,string,
	    strlen(string));
	}

    free(string);
    }
return(XID_S);
}

void
fill_method(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    chosen to fill inside or outside selected polygons.

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
/*
    Set the flag indicating whether to fill inside or outside selected polygons.
*/
xid_fill.flags = xid_fill.flags & ~(FILL_INTERNAL | FILL_EXTERNAL);
switch (*client)
    {
    case 'I':
	xid_fill.flags = xid_fill.flags | FILL_INTERNAL;
	break;
    case 'E':
	xid_fill.flags = xid_fill.flags | FILL_EXTERNAL;
	break;
    }
}

void
fill_mode(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether to fill all
    polygons or only user-selected polygons.

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
void    fill_select_all();    	/* Function to select all polygons.           */

/*
    Remove the "Fill" context.
*/
if (xid_fill.flags & FILL_USER)
    xid_contextdel(CON_FILL);
else
/*
    Unselect all currently selected polygons.
*/
    fill_unselect_all();

/*
    Set the flag indicating whether to fill all polygons or only user-selected 
    ones. 
*/
xid_fill.flags = xid_fill.flags & ~(FILL_USER | FILL_ALL);
switch (*client)
    {
    case 'U':
	xid_fill.flags = xid_fill.flags | FILL_USER;
/*
    Add the "Fill" context and make it active.
*/
	xid_contextadd(CON_FILL);
	XeContextSetActive((XeContextWidget)context_mouse,
	    context_names[CON_FILL]);
	break;
    case 'A':
	xid_fill.flags = xid_fill.flags | FILL_ALL;
/*
    Select all displayed polygons.
*/
	fill_select_all();
	break;
    }
}

void fill_move_scrollbar(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
/*******************************************************************************
Purpose:
    This routine keeps the scrollbars of the "from" and "to" lists in sync.
    It does this by retrieving the scrollbar position of the list that 
    generated the callback and setting the scrollbar position of the opposite
    list.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
int         scrollbar_pos,  /* Scrollbar position.                 	      */
	    increment,      /* Current scrollbar increment value.             */
	    page,           /* Current scrollbar page increment value.        */
	    size;           /* Scrollbar size.                                */

/*
    Get the current position of the scrollbar which triggered the callback.
*/
XmScrollBarGetValues(w, &scrollbar_pos, &size, &increment, &page);

/*
    Set the new position for the scrollbar which did not trigger the callback. 
*/
if (w == fill_from_scrollBar)
    {
    XmScrollBarSetValues(fill_to_scrollBar, scrollbar_pos, size, increment, 
	page, True);
    XmListSetPos(fill_to_list, scrollbar_pos+1);
    }
else
    {
    XmScrollBarSetValues(fill_from_scrollBar, scrollbar_pos, size, increment, 
	page, True);
    XmListSetPos(fill_from_list, scrollbar_pos+1);
    }
}

int
fill_new_image(nplanes, orig_plane_idx, plane_idx)
int	nplanes,
	*orig_plane_idx,
	*plane_idx;
/*******************************************************************************
Purpose:
    This routine finds new image planes for the output filled image and copies
    the appropriate information from the original image's memory plane structure
    to the new memory plane structure.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
struct XID_IMGINFO
	*save_infoptr;		/* Pointer to the image information structure.*/

int	i;			/* Looping index.			      */

unsigned char
	*save_imgptr;		/* Pointer to the image data array.	      */

/*
    Lock the original image planes so that we don't overwrite them.
*/
for (i=0; i < nplanes; i++)
    xid_planes[orig_plane_idx[i]].flags = xid_planes[orig_plane_idx[i]].flags |
	LOCK;

/*
    Get the required number of image planes.  Display an error message if there
    are not enough free memory planes.
*/
for (i=0; i < nplanes; i++)
    {
    if (xid_getmem(&plane_idx[i]) == XID_F)
	{
        xid_message("Not enough memory planes available");
        return(XID_F);
	}
    plane_idx[i]--;
/*
    Lock each plane as we get it so that we don't reuse it.
*/
    xid_planes[plane_idx[i]].flags = xid_planes[plane_idx[i]].flags | LOCK;
    }

/*
    Unlock the image planes.
*/
for (i=0; i < nplanes; i++)
    {
    xid_planes[orig_plane_idx[i]].flags = xid_planes[orig_plane_idx[i]].flags ^
	LOCK;
    xid_planes[plane_idx[i]].flags = xid_planes[plane_idx[i]].flags ^ LOCK;
/*
    Copy the appropriate information from the input memory plane structure to
    the output memory plane structure.
*/
    save_imgptr = xid_planes[plane_idx[i]].image_data;
    save_infoptr = xid_planes[plane_idx[i]].image_info;
    memcpy(&xid_planes[plane_idx[i]], &xid_planes[orig_plane_idx[i]], 
	sizeof(struct XID_MEMPLANE));
    xid_planes[plane_idx[i]].image_data = save_imgptr;
    xid_planes[plane_idx[i]].image_info = save_infoptr;
    memcpy(xid_planes[plane_idx[i]].image_data, xid_planes[orig_plane_idx[i]].
	image_data, xid_layout.mem_width * xid_layout.mem_height);
    memcpy(xid_planes[plane_idx[i]].image_info, xid_planes[orig_plane_idx[i]].
	image_info, sizeof(struct XID_IMGINFO)); 
    }
if (nplanes == 1)
    plane_idx[GREEN] = plane_idx[BLUE] = plane_idx[RED];

return(XID_S);
}

void fill_ok_breakpts(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
/*******************************************************************************
Purpose:
    This routine closes the panel for entering break point/renumber values. 

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
while(!XtIsVendorShell(w))
    w = XtParent(w);

XtPopdown(w);
}

void fill_ok_constant(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
/*******************************************************************************
Purpose:
    This routine retrieves the constant fill value and closes the panel.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
int	scale_val;		/* Constant scale value.		      */

/*
    Get the selected fill value.
*/
XmScaleGetValue(fill_constant_scale, &scale_val);
xid_fill.constant = (long)scale_val;

/*
    Close the panel.
*/
while(!XtIsVendorShell(w))
    w = XtParent(w);

XtPopdown(w);
}

int
fill_pfill(plane_idx, map, amin, amax, area)
int	*plane_idx;	/* Indices of the memory planes to fill. (I)	      */

long	*amin,		/* Minimum line for all polygons. (I)	     	      */
	*amax,		/* Maximum line for all polygons. (I)	     	      */
	area[][4],	/* Bounding rectangle for each polygon.	(I)    	      */
	map[];		/* Mapping PWL or renumber option. (I)	     	      */

/****************************************************************************
Purpose:
	This routine is designed to fill the selected polygons based on the
	user selected method (internal or external), the user selected type
	(piece-wise linear mapping, renumber mapping or constant value), and
	the window selected (main image window or full resolution window).

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
	This routine was adopted from the DMS function named FILL.
*****************************************************************************/
{
struct  POLY_ELEMENT    *pptr;  
			/* Pointer to polygon specific information.   	      */

float	fend,		/* Last sample of polygon that the current line crosses 
			   (file coords).			     	      */
	fline,		/* Current line number in file coordinates.	      */
	fnull,		/* Dummy variable for xid_win2file.		      */
	fstart;		/* First sample of polygon that the current line 
			   crosses (file coords).			      */

long	dtype = EBYTE,	/* Data type of buffer to copy.			      */
	end,		/* Last sample of polygon that the current line crosses 
			   (image coords).			     	      */
	nsect,		/* Number of intersection points.	     	      */
	null = 1,	/* Null coordinate for conversion routines.   	      */
	*sect,		/* X coordinates of intersection points.	      */
	start;		/* First sample of polygon that the current line 
			   crosses (image coords).			      */

int	band,		/* Band loop index.				      */
	i,		/* Looping index.				      */
	ipt,		/* Intersection points loop index.		      */
	last,           /* Last sample of bounding rectangle.                 */
	line,		/* Line loop index.				      */
	nbands,         /* Number of bands displayed.                         */
	nfill,		/* Number of samples to fill.			      */
	nl,		/* Number of lines in the displayed image.	      */
	ns,		/* Number of samples in the displayed image.	      */
	poly,		/* Polygon loop index.				      */
	status,		/* Return status indicator.			      */
	xcoord[2];	/* Starting and ending X coordinates of the displayed
			   image.					      */

unsigned char
	fillval,	/* Fill value.					      */
	*imgptr[3],	/* Pointers to displayed image data arrays.	      */
	*savebuf;	/* Original image data.				      */

/*
    Set starting and ending x coordinate based on the window the fill operation
    is to be performed in.
*/
if (xid_fill.flags & FILL_MAIN)
    {
    xcoord[0] = xid_planes[plane_idx[RED]].window[SS];
    xcoord[1] = xid_planes[plane_idx[RED]].window[SS] + 
	xid_planes[plane_idx[RED]].window[NS] - 1;
    }
else
    {
    xcoord[0] = xid_fullres[RED].window[SS];
    xcoord[1] = xid_fullres[RED].window[SS] + xid_fullres[RED].window[NS] - 1;
    }

/*
    Determine the number of bands displayed.
*/
if ((plane_idx[RED] == plane_idx[GREEN]) && (plane_idx[GREEN] == 
    plane_idx[BLUE]))
    nbands = 1;
else
    nbands = 3;

/*
    Determine the number of lines and samples based on the window selected.
    Also, set a pointer to the appropriate image data.
*/
if (xid_fill.flags & FILL_MAIN)
    {
    for (band=0; band < nbands; band++)
	imgptr[band] = xid_planes[plane_idx[band]].image_data;
    nl = xid_planes[plane_idx[RED]].dspwin[NL];
    ns = xid_planes[plane_idx[RED]].dspwin[NS];
    }
else
    {
    for (band=0; band < nbands; band++)
	imgptr[band] = xid_fullres[band].image_data;
    nl = xid_fullres[RED].dspwin[NL];
    ns = xid_fullres[RED].dspwin[NS];
    }

/*
    Allocate space for a temporary buffer to store image data.
*/
savebuf = (unsigned char *)malloc(ns * nbands);
if (savebuf == NULL)
    {
    xid_message("Insufficient memory available");
    return(XID_F);
    }

/*
    This is kind of unorthodox, but pre-decrement the pointer to the image
    data to eliminate the need for a test inside the loop for the first
    pass.
*/
for (band=0; band < nbands; band++)
    imgptr[band] -= ns;

for (line=0; line < nl; line++)
    {
/*
    Increment the image pointer to the next line of image data.
*/
    for (band=0; band < nbands; band++)
	imgptr[band] += ns;
/*
    Convert the current line number to a file coordinate based on the 
    selected window.
*/
    if (xid_fill.flags & FILL_MAIN)
        xid_win2file(plane_idx[RED], 0, null, line, &fnull, &fline); 
    else
        xid_win2file(plane_idx[RED], 1, null, line, &fnull, &fline); 
/*
    If filling externally with a constant value, save the image data from the
    current line and copy the constant value into the entire image line.
*/
    if ((xid_fill.flags & FILL_CONSTANT) && (xid_fill.flags & 
	FILL_EXTERNAL))
	{
	fillval = (unsigned char)xid_fill.constant;
	for (band=0; band < nbands; band++)
	    {
	    c_pxcopy(imgptr[band], savebuf + (ns * band), &dtype, &ns); 
	    c_pxfill(imgptr[band], &dtype, &ns, &fillval); 
	    }
	}
/*
    If filling externally with a piece-wise linear or renumber mapping, save 
    the image data from the current line and map the entire image line.
*/
    else if (((xid_fill.flags & FILL_PWL) || (xid_fill.flags & 
	FILL_RENUMBER)) && (xid_fill.flags & FILL_EXTERNAL))
	{
	for (band=0; band < nbands; band++)
	    {
	    c_pxcopy(imgptr[band], savebuf + (ns * band), &dtype, &ns); 
	    for (i=0; i < ns; i++)
		*(imgptr[band] + i) = map[*(imgptr[band] + i)];
	    }
	}
/*
    Continue to the next line if the current line is outside the bounds of all
    selected polygons.
*/
    if ((fline < *amin) || (fline > *amax))
	continue;
/*
    Loop through all selected polygons ...
*/
    for (poly=0; poly < xid_fill.num_poly; poly++)
	{
/*
    Continue to the next polygon if the current line is outside the bounding
    rectangle of this polygon.
*/
	if ((fline < area[poly][0]) || (fline > (area[poly][0] + 
	    area[poly][2] - 1)))
	    continue;
/*
    Calculate the intersection points between the current line and polygon.
*/
	pptr = (struct POLY_ELEMENT *)xid_element[xid_fill.polys[poly]].
	    graph_ptr;
	status = fill_isect(xcoord, &fline, pptr->x, pptr->y, &(pptr->num_vert),
	    &sect, &nsect);
	if (status != XID_S)
	    return(status);
	if (nsect != 0)
	    {
/*
    Loop through all intersection points ...
*/
	    for (ipt=0; ipt < nsect; ipt+=2)
		{
/*
    Convert the starting and ending points to window coordinates.
*/
		fstart = (float)max(sect[ipt], area[poly][1] - 1);
		last = area[poly][1] + area[poly][3];
		fend = (float)min(sect[ipt + 1], last);
		if (xid_fill.flags & FILL_MAIN)
		    {
		    xid_file2win(plane_idx[RED], 0, fstart, (float)null,
			&start, &null);
		    xid_file2win(plane_idx[RED], 0, fend, (float)null, &end, 
			&null);
		    }
		else
		    {
		    xid_file2win(plane_idx[RED], 1, fstart, (float)null,
			&start, &null);
		    xid_file2win(plane_idx[RED], 1, fend, (float)null, &end, 
			&null);
		    }
		if ((xid_fill.flags & FILL_CONSTANT) && (xid_fill.flags & 
		    FILL_INTERNAL))
		    {
/*
    Fill between the starting and ending points with the selected constant 
    value.
*/
		    nfill = end - start + 1;
		    if (nfill > 0)
			{
			fillval = (unsigned char)xid_fill.constant;
			for (band = 0; band < nbands; band++)
			    c_pxfill(imgptr[band] + start, &dtype, 
				&nfill, &fillval); 
			}
		    }
		else if (((xid_fill.flags & FILL_PWL) || (xid_fill.flags & 
		    FILL_RENUMBER)) && (xid_fill.flags & FILL_INTERNAL))
		    {
/*
    Fill between the starting and ending points with the mapped values.
*/
		    nfill = end - start + 1;
		    if (nfill > 0)
			for (band = 0; band < nbands; band++)
			    for (i=0; i < nfill; i++)
			        *(imgptr[band] + start + i) = 
				    map[*(imgptr[band] + start + i)];
		    }
		else if (xid_fill.flags & FILL_EXTERNAL)
		    {
/*
    Fill between the starting and ending points with the saved image data.
*/
		    nfill = end - start + 1;
		    if (nfill > 0)
			for (band=0; band < nbands; band++)
			    c_pxcopy(savebuf + (ns * band) + start, 
				imgptr[band] + start, &dtype, &nfill); 
		    }
		}
	    free(sect);
	    }
	}
    }
free(savebuf);
return(XID_S);
}

void
fill_select(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    This routine finds the displayed polygon closest to the current mouse
    position and marks it as one selected to be filled.  Marking the polygon
    consists of redrawing it with dashed lines.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
XButtonEvent    *event;         /* X Event structure.                         */

float           xyfil[2];       /* XY file coordinate.                        */

long            i,		/* Looping index.			      */
		poly[1],    	/* Index of polygon found.                    */
		nfnd,           /* Number of polygons actually found.         */
		nreq = 1,       /* Number of polygons requested.     	      */
		plane_idx,      /* Index into memory plane structure of the
                                   currently viewed image.                    */
                tol = 5,        /* Area around specified point to be searched.*/
		win;            /* Window from which coordinates are to be
				   calculated.			 	      */

event = (XButtonEvent *)call;

/*
    Determine which window generated the event.
*/
if (w == main_image_bulletinboard)
    win = 0;
else if (w == fullres_win_bulletinboard)
    win = 1;
else if (w == zoom_win_bulletinboard)
    win = 2;

/*
    Find the memory plane index of the first band of the displayed image.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Convert the window coordinate to an actual file coordinate based on the
    image currently displayed.
*/
xid_win2file(plane_idx, win, event->x, event->y, &xyfil[0], &xyfil[1]);

/*
    Find the polygon (or rectangle) closest to the point selected.
*/
graphics_closest(plane_idx, POLY, tol, xyfil, nreq, poly, &nfnd);
if (nfnd == 0)
    {
    graphics_closest(plane_idx, RECT, tol, xyfil, nreq, poly, &nfnd);
    if (nfnd == 0)
        return;
    }

/*
    If the user has already selected this polygon, return.
*/
for (i=0; i < xid_fill.num_poly; i++)
    if (xid_fill.polys[i] == poly[0])
	return;

/*
    Add the polygon that was found to be the closest to the list of selected 
    polygons and allocate more space if we've run out of room in the list.
*/
xid_fill.polys[xid_fill.num_poly] = poly[0];
xid_fill.num_poly++;
if (xid_fill.num_poly >= xid_fill.max_polys)
    {
    xid_fill.max_polys += FILL_MAXPOLY;
    xid_fill.polys = (long *)realloc(xid_fill.polys, xid_fill.max_polys *
	sizeof(long));
    if (xid_fill.polys == NULL)
        {
        xid_message("Insufficient memory available");
        return;
        }
    }

/*
    Mark the polygon as selected.
*/
fill_mark(poly[0], plane_idx);

}

void
fill_select_all()
/*******************************************************************************
Purpose:
    This routine selects and marks all displayed polygons for filling. 

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
int	i,			/* Looping index.			      */
	num_elements,		/* The number of current graphics elements.   */
	plane_idx;              /* Index into memory plane structure of the
				   currently viewed image.                    */
/*
    Find the memory plane index of the first band of the displayed image.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Loop through all the graphics elements and select the polygons. 
*/
if (xid_graphics.flags & GR_CLOSE)
    num_elements = xid_graphics.current + 1;
else
    num_elements = xid_graphics.current;

for (i=0; i < num_elements; i++)
    {
    if ((xid_element[i].type == POLY) || (xid_element[i].type == RECT))
	{
	xid_fill.polys[xid_fill.num_poly] = i;
	xid_fill.num_poly++;
	if (xid_fill.num_poly >= xid_fill.max_polys)
	    {
	    xid_fill.max_polys += FILL_MAXPOLY;
	    xid_fill.polys = (long *)realloc(xid_fill.polys, 
		xid_fill.max_polys * sizeof(long));
	    if (xid_fill.polys == NULL)
		{
		xid_message("Insufficient memory available");
		return;
		}
	    }
/*
    Mark the polygon as selected.
*/
	fill_mark(i, plane_idx);
	}
    }
}

void
fill_setarea(window, amin, amax, area)

long	window[],	/* File window of displayed image. (I)	     	      */
	*amin, 		/* Minimum line for all polygons. (O)	     	      */
	*amax,		/* Maximum line for all polygons. (O)	     	      */
	area[][4];	/* Bounding rectangle for each polygon (image). (O)   */

/*****************************************************************************
Purpose:
	Determines the bounding rectangle for each selected polygon and the 
	minimum and maximum line for all polygons.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
	This routine was adopted from the DMS function named FILL.
******************************************************************************/
{
struct  POLY_ELEMENT    *pptr;  /* Pointer to polygon specific information.   */

long	i,j,		/* Looping index.			     	*/
	minx,		/* Minimum X coordinate.			*/
	miny,		/* Minimum Y coordinate.		        */
	maxx,		/* Maximum X coordinate.			*/
	maxy,		/* Maximum Y coordinate.			*/
	botx,		/* Bottom X coordinate of the input window.   	*/
	boty;		/* Bottom Y coordinate of the input window.   	*/

/*
    Initial set up.
*/
*amin = 99999;
*amax = 0;
botx = window[SS] + window[NS] - 1;
boty = window[SL] + window[NL] - 1;

for (i=0; i < xid_fill.num_poly; i++)
    {
    pptr = (struct POLY_ELEMENT *)xid_element[xid_fill.polys[i]].graph_ptr;
/*
    Calculate the bounding rectangle for each polygon specified, based 
    on the polygon vertices and the input file window.
*/
    minx = miny = 99999;
    maxx = maxy = 0;
    for (j=0; j < pptr->num_vert; j++)
	{
	if (pptr->x[j] < minx) minx = pptr->x[j]; 
	if (pptr->x[j] > maxx) maxx = pptr->x[j]; 
	if (pptr->y[j] < miny) miny = pptr->y[j]; 
	if (pptr->y[j] > maxy) maxy = pptr->y[j]; 
	}
    area[i][1] = max(minx, window[SS]);
    area[i][0] = max(miny, window[SL]);
    area[i][3] = ((min(maxx,botx)) - area[i][1] + 1);
    area[i][2] = ((min(maxy,boty)) - area[i][0] + 1);

/*
    Calculate the minimum and maximum line for all polygons processed.
*/
    if (area[i][0] < *amin) 
	*amin = area[i][0];
    if ((area[i][0] + area[i][2] - 1) > *amax)
	*amax = area[i][0] + area[i][2] - 1;
    }
return;
}

void
fill_set_breakpt(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine retrieves the selected from and to values and places them in 
    the proper position in the scrolled lists. 

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
XmString
	xmstr;				/* Motif format string.		      */

int	i,j,				/* Looping variables.		      */
	fromBreak,			/* "From" break point value.	      */
	toBreak;			/* "To" break point value.	      */

char	charstr[5];

/*
    Get the current from and to values.
*/
XmScaleGetValue(fill_from_scale, &fromBreak);
XmScaleGetValue(fill_to_scale, &toBreak);

/*
    Place the values in the proper position in the from and to arrays (these
    arrays are stored in ascending order).
*/
if (xid_fill.num == 0)
    {
    xid_fill.from[xid_fill.num] = fromBreak;
    xid_fill.to[xid_fill.num] = toBreak;
    xid_fill.num++;
    }
else
    {
    for (i=0; i < xid_fill.num; i++)
	{
	if (fromBreak < xid_fill.from[i])
	    {
	    for (j=xid_fill.num; j > i; j--)
		{
		xid_fill.from[j] = xid_fill.from[j-1];
		xid_fill.to[j] = xid_fill.to[j-1];
		}
	    xid_fill.from[i] = fromBreak;
	    xid_fill.to[i] = toBreak;
	    xid_fill.num++;
	    break;
	    }
	}
    if (i == xid_fill.num)
	{
	xid_fill.from[i] = fromBreak;
	xid_fill.to[i] = toBreak;
	xid_fill.num++;
	}
    }

/*
    If we've run out of room to store values, allocate more space.
*/
if (xid_fill.num >= xid_fill.max_breakpts)
    {
    xid_fill.max_breakpts += FILL_MAXBREAK;
    xid_fill.from = (long *)realloc(xid_fill.from, xid_fill.max_breakpts * 
	sizeof(long));
    if (xid_fill.from == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
    xid_fill.to = (long *)realloc(xid_fill.to, xid_fill.max_breakpts * 
	sizeof(long));
    if (xid_fill.from == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
    }

/*
    Delete all items from both lists.
*/
XmListDeleteAllItems(fill_from_list);
XmListDeleteAllItems(fill_to_list);

/*
    Display the new from and to lists.
*/
for (i=0; i < xid_fill.num; i++)
    {
    sprintf(charstr, "%3d", xid_fill.from[i]);
    xmstr = XmStringCreateLtoR(charstr, XmSTRING_DEFAULT_CHARSET);
    XmListAddItemUnselected(fill_from_list, xmstr, 0);
    XmStringFree(xmstr);

    sprintf(charstr, "%3d", xid_fill.to[i]);
    xmstr = XmStringCreateLtoR(charstr, XmSTRING_DEFAULT_CHARSET);
    XmListAddItemUnselected(fill_to_list, xmstr, 0);
    XmStringFree(xmstr);
    }
}

void
fill_type(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating the type of mapping to
    apply to the filled polygons.

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
void		fill_breakpts(),	/* Routine to create break points 
					   panel.			      */
		fill_constant();	/* Routine to create constant value
					   panel.			      */

int		clear;			/* Flag indicating whether or not to
					   clear the break point list.	      */
/*
    If the user has not changed mapping type, don't clear the break point list
    (i.e., the user may be re-opening the panel to modify currently selected
    break points).
*/
if ((*client == 'P') && (xid_fill.flags & FILL_PWL))
    clear = False;
else if ((*client == 'R') && (xid_fill.flags & FILL_RENUMBER))
    clear = False;
else 
    clear = True;

/*
    Clear the break points list.
*/
if (clear)
    fill_clear_breakpts(w, client, call);

/*
    Set the flag indicating what type of mapping to apply. 
*/
xid_fill.flags = xid_fill.flags & ~(FILL_PWL | FILL_RENUMBER | FILL_CONSTANT);
switch (*client)
    {
    case 'P':  /* Piece-wise linear mapping */
	xid_fill.flags = xid_fill.flags | FILL_PWL;
/*
    Create the panel for entering break points.
*/
	fill_breakpts();
    	XtManageChild(fill_breakpt_dialog);
	XtPopup(XtParent(fill_breakpt_dialog), XtGrabNone);
	break;
    case 'R':  /* Renumber */
	xid_fill.flags = xid_fill.flags | FILL_RENUMBER;
/*
    Create the panel for entering renumber values. 
*/
	fill_breakpts();
    	XtManageChild(fill_breakpt_dialog);
	XtPopup(XtParent(fill_breakpt_dialog), XtGrabNone);
	break;
    case 'C':  /* Constant value */
	xid_fill.flags = xid_fill.flags | FILL_CONSTANT;
/*
    Create the panel for entering a constant fill value. 
*/
	fill_constant();
    	XtManageChild(fill_const_dialog);
	XtPopup(XtParent(fill_const_dialog), XtGrabNone);
	break;
    }
}

void
fill_unselect(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    This routine finds the displayed polygon closest to the current mouse
    position and removes it from the list of polygons to be filled (this
    includes redrawing the polygon without dashed lines).

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
XButtonEvent    *event;         /* X Event structure.                         */

float           xyfil[2];       /* XY file coordinate.                        */

long            i,j,		/* Looping indices.			      */
		poly[1],    	/* Index of polygon found.                    */
		nfnd,           /* Number of polygons actually found.         */
		nreq = 1,       /* Number of polygons requested.     	      */
		plane_idx,      /* Index into memory plane structure of the
                                   currently viewed image.                    */
                tol = 5,        /* Area around specified point to be searched.*/
		win;            /* Window from which coordinates are to be
				   calculated.			 	      */
event = (XButtonEvent *)call;

/*
    Determine which window generated the event.
*/
if (w == main_image_bulletinboard)
    win = 0;
else if (w == fullres_win_bulletinboard)
    win = 1;
else if (w == zoom_win_bulletinboard)
    win = 2;

/*
    Find the memory plane index of the first band of the displayed image.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Convert the window coordinate to an actual file coordinate based on the
    image currently displayed.
*/
xid_win2file(plane_idx, win, event->x, event->y, &xyfil[0], &xyfil[1]);

/*
    Find the polygon closest to the point selected.
*/
graphics_closest(plane_idx, POLY, tol, xyfil, nreq, poly, &nfnd);
if (nfnd == 0)
    {
    graphics_closest(plane_idx, RECT, tol, xyfil, nreq, poly, &nfnd);
    if (nfnd == 0)
        return;
    }

/*
    Find the polygon in the list of selected polygons.
*/
for (i=0; i < xid_fill.num_poly; i++)
    {
    if (xid_fill.polys[i] == poly[0])
	break;
    }
/*
    If the polygon is not found in the list of selected polygons, it must
    not be currently selected.
*/
if (i >= xid_fill.num_poly)
    return;

/*
    Remove it from the list of selected polygons.
*/
for (j=i; j < xid_fill.num_poly - 1; j++)
    xid_fill.polys[j] = xid_fill.polys[j+1];
xid_fill.num_poly--;

/*
    Redraw the polygon without dashed lines and then generate an expose event
    in any displayed windows.
*/
graphics_draw(-1, plane_idx, poly[0]);
XClearArea(xid.display, XtWindow(main_image_bulletinboard), 0, 0, 0, 0, True);

if ((xid_fullres != NULL) && (xid_fullres[0].image_pixmap != NULL))
    XClearArea(xid.display, XtWindow(fullres_win_bulletinboard), 0, 0, 0, 0, 
	True);

if ((xid_zoom.flags & ZOOM_ACTIVE) && (xid_zoom.image_pixmap != NULL))
    XClearArea(xid.display, XtWindow(zoom_win_bulletinboard), 0, 0, 0, 0, True);
}

void
fill_unselect_all()
/*******************************************************************************
Purpose:
    This removes all currently selected polygons from the list of selected
    polygons.  This includes redrawing these polygons without dashed lines.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
int	i,			/* Looping index.			      */
	plane_idx;              /* Index into memory plane structure of the
				   currently viewed image.                    */
/*
    Find the memory plane index of the first band of the displayed image.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Redraw each of the selected polygons and then generate an expose event in
    all displayed windows.
*/
for (i=0; i < xid_fill.num_poly; i++)
    graphics_draw(-1, plane_idx, xid_fill.polys[i]);

XClearArea(xid.display, XtWindow(main_image_bulletinboard), 0, 0, 0, 0, True);

if ((xid_fullres != NULL) && (xid_fullres[0].image_pixmap != NULL))
    XClearArea(xid.display, XtWindow(fullres_win_bulletinboard), 0, 0, 0, 0, 
	True);

if ((xid_zoom.flags & ZOOM_ACTIVE) && (xid_zoom.image_pixmap != NULL))
    XClearArea(xid.display, XtWindow(zoom_win_bulletinboard), 0, 0, 0, 0, True);

/*
    Set the number of selected polygons to zero.
*/
xid_fill.num_poly = 0;
}

void
fill_window(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating the window to perform
    the fill operation in (main image window or full resolution window).  Note 
    that the option to generate the filled image in new image planes or update 
    the current image does not apply when the full resolution window is 
    selected.  This is because there is ever only one occurrence of the full 
    resolution window.

By L. Huewe, USGS/EROS Data Center, 8/94

Algorithm Description:
*******************************************************************************/
{
/*
    Set the flag indicating which window to do the fill operation in. 
*/
xid_fill.flags = xid_fill.flags & ~(FILL_MAIN | FILL_FULLRES);
switch (*client)
    {
    case 'M':
	xid_fill.flags = xid_fill.flags | FILL_MAIN;
	XtSetSensitive(fill_dest_radiobox, True);
	break;
    case 'F':
	xid_fill.flags = xid_fill.flags | FILL_FULLRES;
	XtSetSensitive(fill_dest_radiobox, False);
	break;
    }
}
