/*
 *  This is the flicker function source file for the flicker panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By L.Huewe, USGS/EROS Data Center, 7/93.
 *    - X Image Display, v1.0.
 *
 */
#include "xid.h"
#include "../xid_widgets/ProcessM.h"

void
flicker_next(client)
Widget client;

/*******************************************************************************
Purpose:
    This rouine is called after the specified time has elapsed and displays 
    the next image in the defined sequence.  This routine is also responsible
    for calling XtAppAddTimeOut to make sure it is called again after the
    appropriate amount of time has elapsed.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
XtAppContext	context;		/* Application context.		      */

long		img_idx,                /* Index into colormap of first image 
					   color.  			      */
		num_imgcolors,          /* Number of image colors.            */
		ms;			/* Flicker rate in milliseconds.      */

/*
    Set up the appropriate values for storing the colormap.
*/
if (*xid.colormap_ptr == xid.colormap)
    {
    num_imgcolors = NUM_IMGCOLORS;
    img_idx = IMG_IDX;
    }
else
    {
    num_imgcolors = 256;
    img_idx = 0;
    }

/*
    If the server being used supports the multi-buffering extension, display
    the next buffer and write out the appropriate colormap.
*/
if (xid_flicker.flags & FLICKER_MULTI)
    {
#ifdef sun
    XmbufDisplayBuffers(xid.display,1,&xid_flicker.buffers[xid_flicker.
	current_image],0,0);
#endif
    XStoreColors(xid.display,*xid.colormap_ptr,&xid_flicker.colortable[
	xid_flicker.current_image][img_idx],num_imgcolors);
    XFlush(xid.display);
    }
/*
    Display the appropriate image in the main image window and write out the
    corresponding colormap.
*/
else
    {
    XSetWindowBackgroundPixmap(xid.display,XtWindow(main_image_bulletinboard),
	xid_flicker.pixmaps[xid_flicker.current_image]);
    XStoreColors(xid.display,*xid.colormap_ptr,&xid_flicker.colortable[
	xid_flicker.current_image][img_idx],num_imgcolors);
/*
    Clear the window to update the background.
*/
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,0,0,0,True);
    }

/*
    Set the timer again.
*/
context = XtWidgetToApplicationContext(client); 
if (xid_flicker.rate >= 0)
    {
    ms = 1000 * xid_flicker.rate;
    xid_flicker.timer = XtAppAddTimeOut(context, ms, 
	(XtTimerCallbackProc)flicker_next, client);
    xid_flicker.flags |= FLICKER_ING;
    }

/*
    Increment the current image index.
*/
xid_flicker.current_image++;
if (xid_flicker.current_image > (xid_flicker.nplanes - 1))
    xid_flicker.current_image = 0;
}

void
flicker_start(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine starts the flickering process.  It sets up the buffers
    for the multi-buffer extension (if the current server supports the
    multi-buffer extension) and sets the timer so the routine which displays
    the appropriate image is called after the specified amount of time.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
XtAppContext	context;		/* Application context.		      */

long		buf_count,		/* Multi-buffer count.		      */
		i,			/* Looping index.		      */
		max_nl,			/* Multi-buffer number of lines.      */
		max_ns,			/* Multi-buffer number of samples.    */
		ms;			/* Flicker rate in milliseconds.      */

/*
    Report an error and return if the user has not selected at least 2 images
    to flicker.
*/
if (xid_flicker.nplanes < 2)
    {
    xid_message("User must select at least 2 images to flicker");
    return;
    }

/*
    Return of already flickering.
*/
if (xid_flicker.flags & FLICKER_ING)
    return;

/*
   Resize the main image window based on the largest image that is to be
   flickered.
*/
max_nl = xid_planes[xid_flicker.planes[0][RED]-1].dspwin[NL];
max_ns = xid_planes[xid_flicker.planes[0][RED]-1].dspwin[NS];
for (i=0; i < xid_flicker.nplanes; i++)
    {
    if (xid_planes[xid_flicker.planes[i][RED]-1].dspwin[NS] > max_ns)
	max_ns = xid_planes[xid_flicker.planes[i][RED]-1].dspwin[NS];
    if (xid_planes[xid_flicker.planes[i][RED]-1].dspwin[NL] > max_nl) 
	max_nl = xid_planes[xid_flicker.planes[i][RED]-1].dspwin[NL];
    }
XtVaSetValues(main_image_bulletinboard,XmNwidth,max_ns,XmNheight,
    max_nl,XmNmaxWidth,max_ns,XmNmaxHeight,max_nl,NULL);
XtResizeWidget(main_image_bulletinboard,max_ns,max_nl,0);

XtVaSetValues(Shell_MainWin,XmNmaxWidth,max_ns + xid_mem.view_xoffset,
    XmNmaxHeight,max_nl + xid_mem.view_yoffset,NULL);

/*
   Create buffers for the multi-buffer extension.
*/
if (xid_flicker.flags & FLICKER_MULTI)
    { 
#ifdef sun
    buf_count = XmbufCreateBuffers(xid.display,XtWindow(
	main_image_bulletinboard),xid_flicker.nplanes,
	MultibufferUpdateActionUntouched,MultibufferUpdateHintFrequent,
	xid_flicker.buffers);
    if (buf_count != xid_flicker.nplanes)
	{
	xid_message("Error creating image buffers");
	xid_flicker.flags &= (~FLICKER_MULTI);
	if (buf_count > 0)
	    XmbufDestroyBuffers(xid.display,XtWindow(main_image_bulletinboard));
	}
/*
    Check sizes and write the image data to the buffers.
*/
    else
	{
	for (i=0; i < xid_flicker.nplanes; i++)
	    XCopyArea(xid.display,xid_flicker.pixmaps[i],xid_flicker.buffers[i],
		xid_flicker.gc,0,0,xid_planes[xid_flicker.planes[i][RED]-1].
		dspwin[NS],xid_planes[xid_flicker.planes[i][RED]-1].dspwin[NL],
		0,0);
	}
#endif
    xid_flicker.flags |= FLICKER_BUF;
    }

/*
    The user may not have access to the "Clear" and "Step" buttons or the
    "Extension" toggle while flickering.
*/
XtSetSensitive(flicker_step_button,False);
XtSetSensitive(flicker_clear_button,False);
#ifdef sun
XtSetSensitive(flicker_extension_toggle,False);
#endif

/*
    Set the timer.
*/
context = XtWidgetToApplicationContext(w);
if (xid_flicker.flags & FLICKER_ING)
    XtRemoveTimeOut(xid_flicker.timer);
if (xid_flicker.rate >= 0)
    {
    ms = 1000 * xid_flicker.rate;
    xid_flicker.timer = XtAppAddTimeOut(context, ms, 
	(XtTimerCallbackProc)flicker_next, w);
    xid_flicker.flags |= FLICKER_ING;
    }
}

/* ARGSUSED */
void
flicker_accept_combo(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    This routine takes the red, green, blue combination or the saved composite
    view specified by the user and adds it to the list of images to flicker.
    The corresponding X image is also created at this time.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
extern void xid_find_composite();
				/* Function to find the composite index of the 
				   specified entry.			      */
XmString
	entry;			/* A motif format string specifying the 
				   entry to be added to the flicker list.     */

long    composite_index,        /* Composite index.                           */
	i,			/* Looping index.			      */
	select_save = FALSE,    /* Indicates whether or not a saved composite
				   image has been selected.		      */
	status,			/* Status indicator.			      */
	zero;			/* Constant zero.			      */

char	entry_string[100];	/* The ascii string specifying the entry to
				   be added to the flicker list.	      */

XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

/*
    Report an error if the user selected the "Accept" button before any
    items were selected.
*/
if (!(xid_flicker.flags & FLICKER_SEL))
    {
    xid_message("User must choose an item before selecting ACCEPT");
    return;
    }

/*
    Check whether or not a saved composite view has been selected. 
*/
if (xid_flicker.planes[xid_flicker.nplanes][RED] == -1)
    {
    select_save = TRUE;
/*
    Find the composite view specified.
*/
    xid_find_composite(xid_flicker.composite,&composite_index);
    if (composite_index < 0)
        {
        xid_message("Unable to find specified composite image.");
        return;
        }
/*
    Get the red, green and blue plane numbers as well as the colormap of the
    composite view.
*/
    for (i=0; i < 3; i++)
        xid_flicker.planes[xid_flicker.nplanes][i] = xid_composite[
 	    composite_index].rgbplanes[i];
    for (i=0; i < 256; i++)
	xid_flicker.colortable[xid_flicker.nplanes][i] = xid_composite[
	    composite_index].color_table[i];
/*
    Set a flag indicating this entry is a saved composite view.
*/
    xid_flicker.cflags[xid_flicker.nplanes] = TRUE;
    }

/*
    If a value was not specified for one of the display components, copy it
    from a specified component.
*/
for (i=0,zero=0; i < 3; i++)
    if (xid_flicker.planes[xid_flicker.nplanes][i] == 0)
	{
	zero++;
	switch(i)
	    {
	    case 0:
		xid_flicker.planes[xid_flicker.nplanes][i] = 
		    (xid_flicker.planes[xid_flicker.nplanes][i+1] > 0) ? 
		    xid_flicker.planes[xid_flicker.nplanes][i+1] : 
		    xid_flicker.planes[xid_flicker.nplanes][i+2]; 
		break;
	    case 1:
		xid_flicker.planes[xid_flicker.nplanes][i] = 
		    (xid_flicker.planes[xid_flicker.nplanes][i-1] > 0) ? 
		    xid_flicker.planes[xid_flicker.nplanes][i-1] : 
		    xid_flicker.planes[xid_flicker.nplanes][i+1]; 
		break;
	    case 2:
		xid_flicker.planes[xid_flicker.nplanes][i] = 
		    (xid_flicker.planes[xid_flicker.nplanes][i-1] > 0) ? 
		    xid_flicker.planes[xid_flicker.nplanes][i-1] : 
		    xid_flicker.planes[xid_flicker.nplanes][i-2]; 
		break;
	    }
	}

/*
    Set up the processing message widget with the correct title. 
*/
XtVaSetValues(load_dialog_shell,XmNtitle,"Flicker - Processing",NULL);

/*
    Display a processing message.
*/
XeProcessMBegin(process_widget,0,1,"Creating flicker image, please wait ...  ",
    True,False,False);

/*
    Construct the text string to be displayed in the flicker list.
*/
if (!select_save)
    {
    sprintf(entry_string,"R %d G %d B %d",xid_flicker.planes[xid_flicker.
	nplanes][RED],xid_flicker.planes[xid_flicker.nplanes][GREEN],xid_flicker.
	planes[xid_flicker.nplanes][BLUE]);
    entry = XmStringCreate(entry_string,XmSTRING_DEFAULT_CHARSET);
    XmListDeselectAllItems(flicker_redlist);
    XmListDeselectAllItems(flicker_greenlist);
    XmListDeselectAllItems(flicker_bluelist);
    }
else
    {
    entry = XmStringCreate(xid_flicker.composite,XmSTRING_DEFAULT_CHARSET);
    XmListDeselectAllItems(flicker_savelist);
    }

/*
    Create the X image for the selected image view.
*/
status = xid_make_ximage();
if (status != XID_S)
    {
    xid_message("Error creating images to flicker");
    return;
    }

/*
    Add the item to the flicker list.
*/
XmListAddItem(flicker_flickerlist,entry,0);
xid_flicker.flags &= ~FLICKER_SEL;

/*
    Increment the number of planes(images) to be flickered.
*/
xid_flicker.nplanes++;

/*
    Free resources.
*/
XmStringFree(entry);

/*
    Close the processing message window.
*/
XeProcessMEnd(process_widget);
}

/* ARGSUSED */
void
flicker_rate(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    This routine modifies the flicker rate.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
XmScaleCallbackStruct *acs=(XmScaleCallbackStruct*)call;

XtAppContext	context;		/* Application context.		      */

long		ms;			/* Flicker rate in milliseconds.      */

/*
    Assign the new flicker rate.
*/
xid_flicker.rate = acs->value;

/*
    Reset the timer if already flickering.
*/
if (xid_flicker.flags & FLICKER_ING)
    {
    XtRemoveTimeOut(xid_flicker.timer);
    if (xid_flicker.rate >= 0)
	{
	context = XtWidgetToApplicationContext(w);
	ms = 1000 * xid_flicker.rate;
	xid_flicker.timer = XtAppAddTimeOut(context, ms, 
	    (XtTimerCallbackProc)flicker_next, w);
	}
    }
}

/* ARGSUSED */
void
flicker_select(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    This routine is called each time the user selects an item from the
    red, green or blue list or from the save list.  The value selected
    is saved until the combination is accepted (using the "Accept" button). 

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
long	plane_num;			/* Plane number selected.	      */

char	text[6],			/* Dummy text variable.		      */
	*tmpstr;			/* Temporary string containing the
					   value of the selected list element.*/

XmListCallbackStruct *acs=(XmListCallbackStruct*)call;

/*
    Get the item the user selected.
*/
XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&tmpstr);

/*
    Set the flag indicating an item has been selected.
*/
xid_flicker.flags |= FLICKER_SEL;

/*
   Deselect all items in the save and flicker lists and extract the plane
   number from the text string.
*/
if (*((char *)client) != 'S')
    {
    sscanf(tmpstr,"%s%d",text,&plane_num);
    XmListDeselectAllItems(flicker_savelist);
    XmListDeselectAllItems(flicker_flickerlist);
    }
/*
    Deselect all items in the red, green, blue and flicker lists.
*/
else
    {
    XmListDeselectAllItems(flicker_redlist);
    XmListDeselectAllItems(flicker_greenlist);
    XmListDeselectAllItems(flicker_bluelist);
    XmListDeselectAllItems(flicker_flickerlist);
    }

/*
    Assign the plane number to the appropriate element in the RGB array.
*/
if (*((char *)client) == 'R')
     xid_flicker.planes[xid_flicker.nplanes][RED] = plane_num; 
else if (*((char *)client) == 'G')
     xid_flicker.planes[xid_flicker.nplanes][GREEN] = plane_num; 
else if (*((char *)client) == 'B')
     xid_flicker.planes[xid_flicker.nplanes][BLUE] = plane_num; 
/*
    If a saved composite name was selected, assign the plane numbers to a 
    special value and retrieve the composite name.
*/
else if (*((char *)client) == 'S')
    {
    xid_flicker.planes[xid_flicker.nplanes][RED] = xid_flicker.planes[
	xid_flicker.nplanes][GREEN] = xid_flicker.planes[xid_flicker.nplanes]
 	[BLUE] = -1;
    xid_flicker.composite = malloc(strlen(tmpstr) + 1);
    if (xid_flicker.composite == NULL)
        {
	free(tmpstr);
        xid_message("Insufficient memory available");
        return;
        }
    strcpy(xid_flicker.composite,tmpstr);
    }

/*
    Free appropriate resources.
*/
free(tmpstr);
}

/* ARGSUSED */
void
flicker_stop(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    This routine stops the image flickering, frees appropriate resources
    and makes the current flicker image the currently displayed image
    (using load_display).

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
long	current,		/* Current image in flicker list.     	      */
	i,			/* Looping index.			      */
	nplanes;		/* Number of planes to display.		      */

XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

if (!(xid_flicker.flags & FLICKER_ING))
    return;

/*
    Stop the timer.
*/
XtRemoveTimeOut(xid_flicker.timer);
xid_flicker.flags &= (~FLICKER_ING);

/*
    Destroy the buffers if the multi-buffer extension was being used.
*/
#ifdef sun
if (xid_flicker.flags & FLICKER_MULTI)
    {
    XmbufDestroyBuffers(xid.display,XtWindow(main_image_bulletinboard));
    xid_flicker.flags &= ~FLICKER_BUF;
    }
#endif

/*
    Allow access to the "Step", "Clear" and "Extension" functions.
*/
XtSetSensitive(flicker_step_button,True);
XtSetSensitive(flicker_clear_button,True);
#ifdef sun
XtSetSensitive(flicker_extension_toggle,True);
#endif

/*
    Determine which image to display. 
*/
current = xid_flicker.current_image - 1;
if (current < 0)
    current = xid_flicker.nplanes - 1;

/*
    If no images have been selected for flickering, return.
*/
if (current < 0)
    return;

/*
    Determine the number of planes to display.
*/
if ((xid_flicker.planes[current][RED] == xid_flicker.planes[current][GREEN]) && 
    (xid_flicker.planes[current][GREEN] == xid_flicker.planes[current][BLUE]))
	nplanes = 1;
else
	nplanes = 3;

/*
    Free any previously allocated pixmaps.
*/
if (xid_mem.imgpix)
    {
    XFreePixmap(xid.display,xid_mem.image_pixmap);
    xid_mem.imgpix = FALSE;
    xid_mem.image_pixmap = (Pixmap)NULL;
    }

/*
    Call load_display to display the appropriate image data in the
    appropriate RGB order.
*/
if (nplanes == 1)
    load_display(xid_flicker.planes[current],&nplanes,XtWindow(
	main_image_bulletinboard),Shell_MainWin,main_image_bulletinboard,
	&xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NL],
	&xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NS],
	&xid_planes[xid_flicker.planes[current][RED] - 1].maxval,
	xid_planes[xid_flicker.planes[current][RED] - 1].image_data,NULL,NULL,
	TRUE,&xid_mem.image_pixmap,TRUE,FALSE,FALSE);
else
    load_display(xid_flicker.planes[current],&nplanes,XtWindow(
	main_image_bulletinboard),Shell_MainWin,main_image_bulletinboard,
        &xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NL],
        &xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NS],
	&xid_planes[xid_flicker.planes[current][RED] - 1].maxval,
	xid_planes[xid_flicker.planes[current][RED] - 1].image_data,
	xid_planes[xid_flicker.planes[current][GREEN] - 1].image_data,
	xid_planes[xid_flicker.planes[current][BLUE] - 1].image_data,TRUE,
	&xid_mem.image_pixmap,TRUE,FALSE,FALSE);
/*
    Apply the appropriate color table.
*/
XStoreColors(xid.display,*xid.colormap_ptr,xid_flicker.colortable[current],
    256);
for (i=0; i < 256; i++)
    xid.colortable_ptr[i] = xid_flicker.colortable[current][i];

/*
    Change the title to reflect the name of the image displayed in the
    main image window.
*/
XtVaSetValues(Shell_MainWin,XmNtitle,xid_planes[xid_flicker.planes[current][RED]
    - 1].filename,NULL);
}

void
flicker_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine frees the appropriate resources and closes the flicker panel. 

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
long	i;			/* Looping index.			      */

/*
    If currently flickering, stop.
*/
if (xid_flicker.flags & FLICKER_ING)
    flicker_stop(NULL,NULL,NULL);

/*
    Destroy buffers if the multi-buffer extension was being used.
*/
#ifdef sun
if ((xid_flicker.flags & FLICKER_BUF) && (xid_flicker.flags & FLICKER_MULTI))
    {
    XClearWindow(xid.display,XtWindow(main_image_bulletinboard));
    XmbufDestroyBuffers(xid.display,XtWindow(main_image_bulletinboard));
    xid_flicker.flags &= ~FLICKER_BUF;
    }
#endif

/*
    Free the pixmap created for each flicker image.
*/
for (i=0; i < xid_flicker.nplanes; i++)
    XFreePixmap(xid.display,xid_flicker.pixmaps[i]);
XFreeGC(xid.display,xid_flicker.gc);

/*
    Close the flicker panel.
*/
XtUnmanageChild(flicker_form);
XtDestroyWidget(flicker_form);
XtPopdown(Shell_Flicker);
XtSetSensitive(main_flicker_button,True);
flicker_form = NULL;
xid_flicker.flags &= ~FLICKER_ACTIVE;
}

void
flicker_delete(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is the callback for 2 different widgets.  If it is triggered
    by a selection in the flicker list, this routine simply gives the user
    access to the "Delete" button.  If is is triggered from the "Delete" button,
    this routine removes the selected image view from the flicker list.  This
    includes freeing any resources associated with the deleted image view.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
int	count,				/* Number of items selected.	      */
	i,j,				/* Looping indices.		      */
	*positions,			/* Positions of selected items.	      */
	status;				/* Status indicator.		      */

/*
    If a selection has been made in the flicker list, allow access to the
    "Delete" button.
*/
if (w == flicker_flickerlist)
    {
    XtSetSensitive(flicker_delete_button,True);
    return;
    }

/*
    Get the position of the selected item.
*/
status = XmListGetSelectedPos(flicker_flickerlist,&positions,&count);
if (count == 0)
    {
    xid_message("User must select an item from the flicker list to be deleted");
    return;
    }

/*
    Delete the selected item from the list.
*/
XmListDeleteItemsPos(flicker_flickerlist,1,*positions);

/*
    Free the pixmap associated with the deleted entry.
*/
XFreePixmap(xid.display,xid_flicker.pixmaps[*positions - 1]);

/*
    Make sure all information associated with the entries in the flicker
    list remains contiguous.
*/
for (i=(*positions); i < xid_flicker.nplanes; i++)
    {
    xid_flicker.pixmaps[i-1] = xid_flicker.pixmaps[i];
    xid_flicker.cflags[i-1] = xid_flicker.cflags[i];
    for(j=0; j < 3; j++)
        xid_flicker.planes[i-1][j] = xid_flicker.planes[i][j];
    for(j=0; j < 256; j++)
        xid_flicker.colortable[i-1][j] = xid_flicker.colortable[i][j];
    }
xid_flicker.planes[xid_flicker.nplanes-1][RED] = xid_flicker.planes[xid_flicker.
    nplanes-1][GREEN] = xid_flicker.planes[xid_flicker.nplanes-1][BLUE] = 0;

/*
    Reduce the number of planes (images) to flicker.
*/
xid_flicker.nplanes--;

/*
    If the current image is greater than the number images to flicker,
    reset the current image.
*/
if ((xid_flicker.current_image > (xid_flicker.nplanes - 1)) && (xid_flicker.
    nplanes > 0))
    xid_flicker.current_image = xid_flicker.nplanes - 1;

/*
    Free appropriate resources.
*/
if (status)
    free(positions);
/*
    Deselect all items in the red, green, blue and save lists.
*/
XmListDeselectAllItems(flicker_redlist);
XmListDeselectAllItems(flicker_greenlist);
XmListDeselectAllItems(flicker_bluelist);
XmListDeselectAllItems(flicker_savelist);

/*
    Do not allow access to the "Delete" button until another item has been
    selected.
*/
XtSetSensitive(flicker_delete_button,False);
}

void
flicker_step(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine displays (in the main image window) the next image in the
    series of images to be flickered.  This is like a manual flicker (i.e., the
    user controls when the next image will be displayed).

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
long	current,		/* Current image in flicker list.     	      */
	i,			/* Looping variable.		      	      */
	nplanes;		/* Number of planes to display.		      */

/*
    At least 2 images must be defined in the flicker list.
*/
if (xid_flicker.nplanes < 2)
    {
    xid_message("User must select at least 2 images to step through");
    return;
    }

current = xid_flicker.current_image;

/*
    Determine the number of planes to display.
*/
if ((xid_flicker.planes[current][RED] == xid_flicker.planes[current][GREEN]) && 
    (xid_flicker.planes[current][GREEN] == xid_flicker.planes[current][BLUE]))
	nplanes = 1;
else
	nplanes = 3;

/*
    Free any previously allocated pixmaps.
*/
if (xid_mem.imgpix)
    {
    XFreePixmap(xid.display,xid_mem.image_pixmap);
    xid_mem.imgpix = FALSE;
    xid_mem.image_pixmap = (Pixmap)NULL;
    }

/*
    Call load_display to display the appropriate image data in the
    appropriate RGB order.
*/
if (nplanes == 1)
    load_display(xid_flicker.planes[current],&nplanes,XtWindow(
	main_image_bulletinboard),Shell_MainWin,main_image_bulletinboard,
	&xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NL],
	&xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NS],
	&xid_planes[xid_flicker.planes[current][RED] - 1].maxval,
	xid_planes[xid_flicker.planes[current][RED] - 1].image_data,NULL,NULL,
	TRUE,&xid_flicker.pixmaps[current],TRUE,FALSE,TRUE);
else
    load_display(xid_flicker.planes[current],&nplanes,XtWindow(
	main_image_bulletinboard),Shell_MainWin,main_image_bulletinboard,
        &xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NL],
        &xid_planes[xid_flicker.planes[current][RED] - 1].dspwin[NS],
	&xid_planes[xid_flicker.planes[current][RED] - 1].maxval,
	xid_planes[xid_flicker.planes[current][RED] - 1].image_data,
	xid_planes[xid_flicker.planes[current][GREEN] - 1].image_data,
	xid_planes[xid_flicker.planes[current][BLUE] - 1].image_data,TRUE,
	&xid_flicker.pixmaps[current],TRUE,FALSE,TRUE);
/*
    Apply the appropriate color table.
*/
XStoreColors(xid.display,*xid.colormap_ptr,xid_flicker.colortable[current],
    256);
for (i=0; i < 256; i++)
    xid.colortable_ptr[i] = xid_flicker.colortable[current][i];

/*
    Set the image pixmap to be the background of the image window.
XSetWindowBackgroundPixmap(xid.display,XtWindow(main_image_bulletinboard),
    xid_flicker.pixmaps[current]);
*/

/*
    Apply the appropriate color table.
XStoreColors(xid.display,*xid.colormap_ptr,xid_flicker.colortable[current],
    256);
*/

/*
    Clear the window to update the background.
XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,0,0,0,True);
*/

/*
    Change the title to reflect the name of the image displayed in the
    main image window.
*/
XtVaSetValues(Shell_MainWin,XmNtitle,xid_planes[xid_flicker.planes[current][RED]
    - 1].filename,NULL);

/*
    Determine which image is to be displayed next.
*/
xid_flicker.current_image++;
if (xid_flicker.current_image > (xid_flicker.nplanes - 1))
    xid_flicker.current_image = 0;

}

void flicker_clear(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is used to clear the entire flicker list.  This includes 
    freeing approriate resources.  
    
By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
long	i,j;				/* Looping indices.		      */

/*
    Delete all items from the flicker list.
*/
XmListDeleteAllItems(flicker_flickerlist);

/*
    Free buffers if the multi-buffer extension was used.
*/
#ifdef sun
if ((xid_flicker.flags & FLICKER_BUF) && (xid_flicker.flags & FLICKER_MULTI))
    {
    XClearWindow(xid.display,XtWindow(main_image_bulletinboard));
    XmbufDestroyBuffers(xid.display,XtWindow(main_image_bulletinboard));
    xid_flicker.flags &= ~FLICKER_BUF;
    }
#endif

/*
    Free all associated pixmaps.
*/
for (i=0; i < xid_flicker.nplanes; i++)
    {
    XFreePixmap(xid.display,xid_flicker.pixmaps[i]);
    for (j=0; j < 3; j++)
	xid_flicker.planes[i][j] = 0;
    }

/*
    Reset the number of planes (images) to flicker as well as the current image.
*/
xid_flicker.nplanes = xid_flicker.current_image = 0;
}

void flicker_extension(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose: 
    This routine is triggered when the user indicates he/she would like to
    use the multi-buffer extension for flickering if it is available.
    
By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
long	first_event,		/* Value of first event code.		      */
	first_error,		/* Value of first error code.		      */
	major_opcode,		/* Major opcode of the extension.	      */
	multibuf;		/* Flag indicating whether or not the multi-
				   buffer extension is available for the 
				   current display.      		      */

#ifdef sun
if (XmToggleButtonGetState(flicker_extension_toggle))
    {
    multibuf = XQueryExtension(xid.display,"Multi-Buffering",&major_opcode,
	&first_event,&first_error);
    if (multibuf)
	{
	xid_flicker.flags |= FLICKER_MULTI;
	printf("Using multi-buffer extension\n");
	}
    else
	{
	XmToggleButtonSetState(flicker_extension_toggle,False,False);
	xid_message("The multi-buffer extension is not available for the current display");
	}
    }
else
    xid_flicker.flags &= ~FLICKER_MULTI;
#endif
}
