/*
 *  This is the init function source file for the init panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the initialize panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "xid.h"

void
init_all(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    elected to initialize all image planes. 

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
/*
   Set the flag indicating all image planes are to be initialized.
*/
xid_init.flags = (xid_init.flags ^ INIT_ALL);
}

void
init_apply(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine checks the initialization flags set and 1) displays a 
    verification panel if the entire main image window or all image planes
    are being initialized or 2) calls the appropriate subroutine to 
    initialize the mapping.

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
XmString	xmstr[1];	/* XmString value.		              */
Arg    		args[1];	/* Arguments for setting resource
				   values.			              */
int    		argcnt;		/* Number of arguments to set.		      */

long		save_flags;	/* Current value of xid_map.flags.	      */

void		init_planes();	/* Initialize planes function.		      */

/*
    If the main window is to be initialized, display a verification panel to
    make sure the user knows he/she is destroying all currently loaded image
    data.
*/
if (xid_init.flags & INIT_MAIN)
    {
    argcnt = 0;
    XtSetArg(args[argcnt], XmNmessageString, (xmstr[0]=XmStringCreateLtoR(
		"Are you sure you want to\n\
initialize the main image\n\
window ?",XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues(init_messagebox,args,1);
    XmStringFree( xmstr[0] );
    XtManageChild(init_messagebox);
    }
else if (xid_init.flags & INIT_PLANE)
    {
/*
    If the all image planes are to be initialized, display a verification 
    panel to make sure the user knows he/she is destroying all currently 
    loaded image data.
*/
    if (xid_init.flags & INIT_ALL)
	{
	argcnt = 0;
        XtSetArg(args[argcnt], XmNmessageString, (xmstr[0]=XmStringCreateLtoR(
		"Are you sure you want to\n\
initialize all image planes?", XmSTRING_DEFAULT_CHARSET))); argcnt++;
        XtSetValues(init_messagebox,args,1);
        XmStringFree( xmstr[0] );
        XtManageChild(init_messagebox);
	}
/*
    Initialize the specified image plane.
*/
    else
	{
        init_destroy(w, client, call);
	init_planes(xid_init.rotate_value);
	}
    }
/*
    Initialize the mapping (i.e. load a linear mapping).
*/
else if (xid_init.flags & INIT_MAP)
    {
    init_destroy(w, client, call);
    save_flags = xid_map.flags;
    xid_map.flags = MAP_ALL;
    map_linear(w, client, call);
    xid_map.flags = save_flags;
    }
}

void
init_create()

/*******************************************************************************
Purpose:
    This routine sets some initial values at the time the initialize panel
    is created.

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
/*
    The default initialization function intializes the entire main image window.
*/
xid_init.flags = INIT_MAIN;

/*
    The default image plane to intialize is the first image plane.
*/
xid_init.rotate_value = 1;
}

void
init_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine closes the initialize panel and sets the main menu option to 
    sensitive.

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
/*
    Close the initialize panel and set the initialize main menu button to
    be sensitive again.
*/
XtUnmanageChild(Init_bulletinboard);
XtDestroyWidget(Init_bulletinboard);
XtPopdown(Shell_Init);
XtSetSensitive(main_init_button,True);
}

void
init_rotator(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine increments the current rotator value which represents the
    image plane to be intialized.  The new value is output to the display.

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
char    outval[3];		/* The new image plane number.		      */

/*
    Increment the rotator value and verify that it has not exceeded the 
    maximum plane number allowed by the current configuration.  If the
    current rotator value is -99 (indicating the graphics planes), reset
    it to 1.
*/
if (xid_init.rotate_value == -99)
    xid_init.rotate_value = 1;
else
    xid_init.rotate_value++;

xid_init.rotate_value = (xid_init.rotate_value > xid.max_plane) ? -99 : 
    xid_init.rotate_value;

/*
    Output the new value to the text widget.
*/
if (xid_init.rotate_value == -99)
    strcpy(outval,"G");
else
    sprintf(outval,"%d",xid_init.rotate_value);
XmTextSetString(init_text,outval);
}

void
init_type(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating the type of 
    initialization to be done. 

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
/*
    Initialize the main image window (this includes all image planes, graphics
    planes and mappings).
*/
if (strcmp(client,"MW") == 0)
    {
    xid_init.flags = xid_init.flags & ~(INIT_MAIN | INIT_MAP | INIT_PLANE);
    xid_init.flags = xid_init.flags | INIT_MAIN;
    xid_init.flags = (xid_init.flags & ~INIT_ALL);
    XtUnmapWidget(init_all_toggle);
    }
/*
    Initialize a single image plane specified by the user.
*/
else if (strcmp(client,"P") == 0)
    {
    xid_init.flags = xid_init.flags & ~(INIT_MAIN | INIT_MAP | INIT_PLANE);
    xid_init.flags = xid_init.flags | INIT_PLANE;
    XtMapWidget(init_all_toggle);
    XmToggleButtonSetState(init_all_toggle,False,True);
    xid_init.flags = (xid_init.flags & ~INIT_ALL);
    }
/*
    Initialize the mapping (i.e. load a linear mapping).
*/
else
    {
    xid_init.flags = xid_init.flags & ~(INIT_MAIN | INIT_MAP | INIT_PLANE);
    xid_init.flags = xid_init.flags | INIT_MAP;
    xid_init.flags = (xid_init.flags & ~INIT_ALL);
    XtUnmapWidget(init_all_toggle);
    }
}

void
init_verify_cancel(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is called in response to the user's request to cancel
    the initialization process.  The initialization panel is closed and
    no further action is taken.

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
init_destroy(w, client, call);
}

void
init_verify_ok(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is called in response to the user's request to continue
    the initialization process.  The initialization panel is closed, all
    image planes are initialized and if requested, the mapping is intialized.

By L. Huewe, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
{
void		init_planes();	/* Initialize planes function.		      */

/*
    Close the initialization panel.
*/
init_destroy(w, client, call);

/*
    Initialize all planes. 
*/
init_planes(-1);

/*
    Initialize the mapping if initializing the main image window.
*/
if (xid_init.flags & INIT_MAIN)
    map_linear(w, client, call);
}

void
init_planes(plane)
int	plane;			/* Plane number to initialize (-1 indicates
				   all planes) (I).			      */

/*******************************************************************************
Purpose:
    This routine intializes the specified image plane.  A -99 indicates the
    graphics planes are to be initialized.  A negative one indicates all 
    image planes are to be initialized.  If a single image plane is being 
    initialized, an error message is output if that plane is locked.

By L. Huewe, USGS/EROS Data Center, 2/92
Modified by, L. Huewe, USGS/EROS Data Center, 10/93
  - Modified the initialize graphics to free graphics pixmaps if the graphics
    panel is not active.

Algorithm Description:
*******************************************************************************/
{
extern void xid_clear_composite(),
				/* Function to remove entries from composite
				   structure.				      */
	    xid_image_makelist();
				/* Function to build a list of plane numbers. */

Arg     args[3];        	/* Variable used to set widget resources.     */

struct  LINE_ELEMENT    *lptr;  /* Pointer to line specific information.      */

struct  POLY_ELEMENT    *pptr;  /* Pointer to polygon specific information.   */

struct  ANNOT_ELEMENT   *aptr;  /* Pointer to annotation specific information.*/

int	check_lock,		/* Indicates whether or not to check if the
				   image plane is locked before initializing
				   it.					      */
	i,j,			/* Looping indices.			      */
	maxpl,			/* Maximum plane to initialize.		      */
	show_flg,               /* Indicates whether or not the show panel
				   is active.                                 */
	start;			/* Plane to start initializing.		      */

/*
    Check whether or not the graphics planes are to be initialized.
*/
if ((plane == -99) || (plane == -1))
    {
/*
    Print an error message if the user has not either cancelled or finished
    selecting the current element.
*/
    if ((xid_graphics.temp_num != 0) || (xid_graphics.flags & GR_LABEL))
	{
	xid_message("User must cancel or finish selection of current type before initializing graphics planes"); 
	return;
	}
/*
    Print an error message if the user has not either cancelled or finished
    editting the current element.
*/
    if (xid_graphics.flags & GR_EDIT)
	{
	xid_message("User must cancel or finish modifying current element before initializing graphics planes");
	return;
	}
/*
    Clear bitmaps if the graphics panel is active. 
*/
    if (xid_graphics.flags & GR_ACTIVE)
	{
	if (xid_graphics.rb_gc != NULL)
	    XSetForeground(xid.display,xid_graphics.rb_gc,0);
	for (i=0; i < USER_GRCOLORS; i++)
	    {
	    if (xid_graphics.pixmap[i] != NULL)
		XFillRectangle(xid.display,xid_graphics.pixmap[i],xid_graphics.
		    rb_gc,0,0,xid_mem.mem_width,xid_mem.mem_height);
	    if ((xid_graphics.full_pix[i] != NULL) && (xid_fullres != NULL))
		XFillRectangle(xid.display,xid_graphics.full_pix[i],
		    xid_graphics.rb_gc,0,0,xid_fullres[0].width,
		    xid_fullres[0].height);
	    if ((xid_graphics.zoom_pix[i] != NULL) && (xid_zoom.flags & 
		ZOOM_ACTIVE))
		XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
		    xid_graphics.rb_gc,0,0,xid_zoom.parms.factor * 
		    xid_zoom.parms.in_samps,xid_zoom.parms.factor * 
		    xid_zoom.parms.in_lines);
	    }
	}
/*
    Free bitmaps if the graphics panel is not active. 
*/
    else
	{
	for (i=0; i < USER_GRCOLORS; i++)
	    {
	    if (xid_graphics.pixmap[i] != NULL)
		{
		XFreePixmap(xid.display,xid_graphics.pixmap[i]);
		xid_graphics.pixmap[i] = (Pixmap)NULL;
		}
	    if ((xid_graphics.full_pix[i] != NULL) && (xid_fullres != NULL))
		{
		XFreePixmap(xid.display,xid_graphics.full_pix[i]);
	        xid_graphics.full_pix[i] = (Pixmap)NULL;
		}
	    if ((xid_graphics.zoom_pix[i] != NULL) && (xid_zoom.flags & 
		ZOOM_ACTIVE))
		{
		XFreePixmap(xid.display,xid_graphics.zoom_pix[i]);
	        xid_graphics.zoom_pix[i] = (Pixmap)NULL;
		}
	    }
	}
/*
    Clear the windows.
*/
    XClearWindow(xid.display,XtWindow(main_image_bulletinboard));
    if (xid_fullres != NULL)
	XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));
    if (xid_zoom.flags & ZOOM_ACTIVE)
	XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
/*
    Free all current elements from the graphics element structure.
*/
    for (i=0; i < xid_graphics.current; i++)
	{
	if (xid_element[i].type == LINE)
	    {
	    if (xid_element[i].graph_ptr != NULL)
		{
		lptr = (struct LINE_ELEMENT *)xid_element[i].graph_ptr;
		if (lptr->x != NULL)
		    {
		    free(lptr->x);
		    lptr->x = NULL;
		    }
		if (lptr->y != NULL)
		    {
		    free(lptr->y);
		    lptr->y = NULL;
		    }
		free(xid_element[i].graph_ptr);
		xid_element[i].graph_ptr = NULL;
		}
	    }
	if (xid_element[i].type == POLY)
	    {
	    if (xid_element[i].graph_ptr != NULL)
		{
		pptr = (struct POLY_ELEMENT *)xid_element[i].graph_ptr;
		if (pptr->x != NULL)
		    {
		    free(pptr->x);
		    pptr->x = NULL;
		    }
		if (pptr->y != NULL)
		    {
		    free(pptr->y);
		    pptr->y = NULL;
		    }
		free(xid_element[i].graph_ptr);
		xid_element[i].graph_ptr = NULL;
		}
	    }
	if (xid_element[i].type == TEXT)
	    {
	    if (xid_element[i].graph_ptr != NULL)
		{
		aptr = (struct ANNOT_ELEMENT *)xid_element[i].graph_ptr;
		if (aptr->string != NULL)
		    {
		    free(aptr->string);
		    aptr->string = NULL;
		    }
		free(xid_element[i].graph_ptr);
		xid_element[i].graph_ptr = NULL;
		}
	    }
	xid_element[i].type = NO_TYPE;
	xid_element[i].offset = xid_element[i].writeflg = xid_element[i].color 
	    = xid_element[i].font = 0;
	xid_element[i].x = xid_element[i].y = -1;
	xid_element[i].labelx = xid_element[i].labely = -1;
    	xid_element[i].upleft[0] = xid_element[i].upleft[1] = 0.0;
	xid_element[i].loright[0] = xid_element[i].loright[1] = 0.0;
        for (j=0; j < MAXATT; j++)
	    strcpy(xid_element[i].attributes[j],"");
	strcpy(xid_element[i].label,"");
	}
    xid_graphics.current = 0;
/*
    Free "edit graphics" structures.
*/
    if (xid_editorig.graph_ptr != NULL)
	{
	if (xid_editorig.type == LINE)
	    {
	    lptr = (struct LINE_ELEMENT *)xid_editorig.graph_ptr;
	    if (lptr->x != NULL)
		free(lptr->x);
	    if (lptr->y != NULL)
		free(lptr->y);
	    free(xid_editorig.graph_ptr);
	    xid_editorig.graph_ptr = NULL;
	    }
	if ((xid_editorig.type == POLY) || (xid_editorig.type == RECT))
	    {
	    pptr = (struct POLY_ELEMENT *)xid_editorig.graph_ptr;
	    if (pptr->x != NULL)
		free(pptr->x);
	    if (pptr->y != NULL)
		free(pptr->y);
	    free(xid_editorig.graph_ptr);
	    xid_editorig.graph_ptr = NULL;
	    }
	if (xid_editorig.type == TEXT)
	    {
	    aptr = (struct ANNOT_ELEMENT *)xid_editorig.graph_ptr;
	    if (aptr->string != NULL)
		free(aptr->string);
	    free(xid_editorig.graph_ptr);
	    xid_editorig.graph_ptr = NULL;
	    }
	}

    if (xid_editundo.graph_ptr != NULL)
	{
	if (xid_editundo.type == LINE)
	    {
	    lptr = (struct LINE_ELEMENT *)xid_editundo.graph_ptr;
	    if (lptr->x != NULL)
		free(lptr->x);
	    if (lptr->y != NULL)
		free(lptr->y);
	    free(xid_editundo.graph_ptr);
	    xid_editundo.graph_ptr = NULL;
	    }
	if ((xid_editundo.type == POLY) || (xid_editundo.type == RECT))
	    {
	    pptr = (struct POLY_ELEMENT *)xid_editundo.graph_ptr;
	    if (pptr->x != NULL)
		free(pptr->x);
	    if (pptr->y != NULL)
		free(pptr->y);
	    free(xid_editundo.graph_ptr);
	    xid_editundo.graph_ptr = NULL;
	    }
	if (xid_editundo.type == TEXT)
	    {
	    aptr = (struct ANNOT_ELEMENT *)xid_editundo.graph_ptr;
	    if (aptr->string != NULL)
		free(aptr->string);
	    free(xid_editundo.graph_ptr);
	    xid_editundo.graph_ptr = NULL;
	    }
	}
    if (plane == -99)
        return;
    }

/*
    If all planes are to be initialized, set the maximum plane number based 
    on the current configuration.
*/
if (plane == -1)
    {
    check_lock = FALSE;
    maxpl = xid.max_plane;
    start = 1;
    }
else
    {
    check_lock = TRUE;
    start = maxpl = plane;
    }

/*
    Make sure the plane to be initialized is not locked.
*/
if (xid_planes != NULL)
    {
    if (check_lock)
        for (i=start-1; i < maxpl; i++)
	    {
	    if (xid_planes[i].flags != LOCK)
		{
	        xid_planes[i].flags = 0;
	        xid_clear_composite(i+1);
		}
	    else
		xid_message("Unable to initialize requested image plane");
	    }
    else
/*
    Set the flag indicating the specified plane is no longer in use.
*/
        for (i=start-1; i < maxpl; i++)
	    {
	    xid_planes[i].flags = 0;
	    xid_clear_composite(i+1);
	    }
    }
/*
    Clear the main image window if the initialized plane was currently
    being viewed.
*/
if ((xid_view.rgbplanes[RED] == plane) ||  
    (xid_view.rgbplanes[GREEN] == plane) ||
    (xid_view.rgbplanes[BLUE] == plane) || (plane == -1))
    {
    XSetWindowBackground(xid.display,XtWindow(main_image_bulletinboard),
	xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
    XClearWindow(xid.display,XtWindow(main_image_bulletinboard));
    xid_view.rgbplanes[RED] = xid_view.rgbplanes[GREEN] = 
	xid_view.rgbplanes[BLUE] = 0;
    xid.view = -1;
/*
    Clear the full resolution and zoom windows.
*/
    if (xid_fullres != NULL)
        {
	XSetWindowBackground(xid.display,XtWindow(fullres_win_bulletinboard),
	    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
	XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));
        for (i=0; i < xid.num_fullres; i++)
            {
            if (xid_fullres[i].image_pixmap != NULL)
                {
                XFreePixmap(xid.display,xid_fullres[i].image_pixmap);
                xid_fullres[i].image_pixmap = (Pixmap)NULL;
                }
            if (xid_fullres[i].image_data != NULL)
                {
                free(xid_fullres[i].image_data);
                xid_fullres[i].image_data = NULL;
                }
            }
	}
    if (xid_zoom.flags & ZOOM_ACTIVE)
	{
	XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
	    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
	XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
	if (xid_zoom.image_pixmap != NULL)
	    {
	    XFreePixmap(xid.display,xid_zoom.image_pixmap);
	    xid_zoom.image_pixmap = (Pixmap)NULL;
	    }
	if (xid_zoom.image_data != NULL)
	    {
	    free(xid_zoom.image_data);
	    xid_zoom.image_data = NULL;
	    }
	}
    }

/*
    If the show panel and/or the flicker panel is open, update the plane lists
    and save lists.
*/
show_flg = XtIsRealized(Shell_Show);
if (show_flg)
    xid_image_makelist(show_redlist,show_greenlist,show_bluelist,
	show_savelist,TRUE);
if (xid_flicker.flags & FLICKER_ACTIVE)
    {
    xid_image_makelist(flicker_redlist,flicker_greenlist,flicker_bluelist,
	flicker_savelist,FALSE);
    flicker_clear(flicker_clear_button,NULL,NULL);
    }
}
