/*
 *  This is the power spectrum function source file for the power spectrum
 *  panel of the X Image Display application.  It contains the callbacks, 
 *  action routines, event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 5/93.
 *    - X Image Display, v1.0.
 *
 */
#include <math.h>
#include "xid.h"
#include "complex.h"

void
power_am(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected automatic or manual mode.  It then adds the appropriate event
    handler based on that mode.

By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
/*
   Set the appropriate flags.
*/
xid_power.flags = xid_power.flags & ~(POWER_MAN | POWER_AUTO);
if (*client == 'A')
    xid_power.flags = xid_power.flags | POWER_AUTO;
else
    xid_power.flags = xid_power.flags | POWER_MAN;

/*
   Add the appropriate event handler for the mode selected.
*/
if (xid_power.flags & POWER_MAN)
    {
    XtRemoveEventHandler(main_image_bulletinboard,
        PointerMotionMask|ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,
	NULL);
    XtAddEventHandler(main_image_bulletinboard,
        ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,NULL);
    }
else
    {
    XtRemoveEventHandler(main_image_bulletinboard,
        ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,NULL);
    XtAddEventHandler(main_image_bulletinboard,
        PointerMotionMask|ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,
	NULL);
    }
}

void
power_cd(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user would
    like the power spectrum plot displayed as a continuous line or as discrete 
    points.

By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
/*
   Set the appropriate flags.
*/
xid_power.flags = xid_power.flags & ~(POWER_CONT | POWER_DIS);
if (*client == 'C')
    xid_power.flags = xid_power.flags | POWER_CONT;
else
    xid_power.flags = xid_power.flags | POWER_DIS;
}

/*
    Power spectrum plot constants.
*/

#define NVALUES         256             /* Total # of (byte) values.          */
#define INDENT_LEFT     105             /* # of pixels to indent.             */
#define INDENT_TOP      45
#define INDENT_BOTTOM   20
#define MAX_WIDTH       440             /* Arbitrary viewport maximums.       */
#define MAX_HEIGHT      400
#define MAX_VHEIGHT	460
#define WIN_SIZE	MAX_WIDTH - 2 * INDENT_LEFT
					/* Size of power spectrum window which
					   does not require scroll bars.      */

float	power_scale;			/* Scaling factor used to space
					   the output points appropriately.   */

void
power_create()

/*******************************************************************************
Purpose:
    This routine is used to allocate the point array for the power spectrum
    plot, draw and label the axis, create the GC, resize the viewport and 
    add the appropriate event handler based on whether automatic or manual 
    mode has been selected.
    
By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
Arg	args[6];		/* Arguments used to reconfigure the power 
				   spectrum widgets.			      */

XGCValues values;		/* Values array for the GC.		      */

Window	power_win;		/* Window ID for plot window.	      	      */

Pixel	back;			/* Background pixel value.		      */

Dimension
	scroll_height,		/* Current scrolled window height.	      */
	scroll_width,		/* Current scrolled window width.	      */
	shell_height,		/* Current shell height.		      */
	shell_width;		/* Current shell width.			      */

int line_style = LineSolid,	/* GC attributes.			      */
    cap_style = CapRound,
    join_style = JoinRound;

float	fmark[5];		/* Tic mark values.			      */

long	count,			/* Number of image planes containing data.    */
	depth,			/* Window/pixmap plane depth.		      */
	len,			/* Working string length.		      */
	mark[5],		/* Tic mark values.			      */
	nvalues,		/* Number of values in the range to be 
				   plotted.				      */
	pwidth,			/* Working string pixel width.		      */
	x_axis,			/* X axis coodinate.			      */
	y_axis,			/* Y axis coordinate.			      */
	xstring,		/* X coordinate of a text string.	      */
	roi_y,			/* Region of interest Y coordinate.	      */
	npts,			/* Number of points to plot.		      */
	pstore,			/* Size of point storage.		      */
	view_width,		/* Working profile viewport dimensions.	      */
	view_height,
	w_change,		/* Width change.			      */
	i;			/* Looping index.			      */

unsigned int line_width = 0;	/* Specifies optimal for server.	      */

char axis[25];			/* Working string array.		      */

/*
    Determine whether or not the Z option is valid.
*/
for (i=0, count=0; i < xid.max_plane; i++)
    if (xid_planes[i].flags & USED)
	count++;
if (count < 2)
    {
    XtSetSensitive(power_z_button,False);
    if (xid_power.flags & POWER_Z)
	{
        xid_power.flags = xid_power.flags & ~POWER_Z;
        xid_power.flags = xid_power.flags | POWER_X;
	XmToggleButtonSetState(power_x_button,True,True);
	}
    }
else
    XtSetSensitive(power_z_button,True);

/*
    Determine the number of points.
*/
if ((xid_power.flags & POWER_ENT) && (xid_power.flags & POWER_PER))
    {
    if (xid_power.ent_npts > xid_power.per_npts)
	npts = xid_power.ent_npts;
    else
	npts = xid_power.per_npts;
    }
else if (xid_power.flags & POWER_ENT)
    npts = xid_power.ent_npts;
else if (xid_power.flags & POWER_PER)
    npts = xid_power.per_npts;

/*
    If the number of points is less than 175 (this is an arbitrary number),
    scale accordingly.  This is done so that with smaller numbers of points,
    the shape of the results plot is more easily visible.
*/
if (npts < 175)
    {
    if (npts == 1)
        power_scale = WIN_SIZE;
    else
        power_scale = (WIN_SIZE) / (float)(npts - 1);
    }
else
    power_scale = 1.0;

/*
    Allocate point arrays.
*/
pstore = xid_power.ent_npts * sizeof(XPoint);
xid_power.ent_points = (XPoint *)malloc(pstore);

pstore = xid_power.per_npts * sizeof(XPoint);
xid_power.per_points = (XPoint *)malloc(pstore);

if ((xid_power.ent_points == NULL) || (xid_power.per_points == NULL))
    {
    xid_message("Insufficient memory available"); 
    return;
    }

/*
    Calculate plot width and height, x and y axis coordinates and region
    of interest values.
*/
if ((xid_power.maxdb - xid_power.mindb + 1) > 1)
    xid_power.scale = (xid_power.maxdb - xid_power.mindb + 1) / 
	(float)MAX_HEIGHT;
else
    xid_power.scale = MAX_HEIGHT / (float)(xid_power.maxdb - xid_power.mindb 
	+ 1);

if (npts == 1)
    xid_power.width = (long) (power_scale * npts + 2 * INDENT_LEFT);
else
    xid_power.width = (long) (power_scale * (npts-1) + 2 * INDENT_LEFT);
xid_power.height= MAX_HEIGHT;
x_axis = (INDENT_TOP + 1 + (xid_power.height - 2 * INDENT_TOP - INDENT_BOTTOM));
y_axis = (INDENT_LEFT + 1);
roi_y = (x_axis - (xid_power.height - 2 * INDENT_TOP - INDENT_BOTTOM));

/*
    Get the power spectrum plot window ID and visual depth.
*/
power_win = XtWindow(power_form);
depth = xid.visual_info[0].depth;

/*
    Create the clear and working pixmaps for the power spectrum plot. 
*/
xid_power.info_pixmap = XCreatePixmap(xid.display,power_win,
    xid_power.width,xid_power.height,depth);
xid_power.clear_pixmap = XCreatePixmap(xid.display,power_win,
    xid_power.width,xid_power.height,depth);
 
/*
    Create the working GC for the power spectrum window.
*/
values.function = GXclear;
values.plane_mask = AllPlanes;
xid_power.gc = XCreateGC(xid.display,power_win,GCFunction | GCPlaneMask,
    &values);

/*
    Clear the pixmaps.
*/
XSetFunction(xid.display,xid_power.gc,GXcopy);

XtVaGetValues(power_form,XmNbackground,&back,NULL);
XSetForeground(xid.display,xid_power.gc,back);

XSetBackground(xid.display,xid_power.gc,xid.color_table[RGBBW_IDX + 
    BLACK_COL].pixel);

XFillRectangle(xid.display,xid_power.info_pixmap,xid_power.gc,0,0,
    xid_power.width,xid_power.height);
XSetFont(xid.display,xid_power.gc,xid.font_info->fid);
XSetLineAttributes(xid.display,xid_power.gc,line_width,line_style,cap_style,
    join_style);
XSetForeground(xid.display,xid_power.gc,xid.color_table[RGBBW_IDX + 
    BLACK_COL].pixel);
XSetBackground(xid.display,xid_power.gc,xid.color_table[RGBBW_IDX + 
    WHITE_COL].pixel);

/*
    Set values for labelling the axes.
*/
nvalues = xid_power.maxdb - xid_power.mindb + 1;
mark[0] = xid_power.mindb;
mark[1] = xid_power.mindb + (nvalues * .25 + 1);
mark[2] = xid_power.mindb + (nvalues * .50 + 1);
mark[3] = xid_power.mindb + (nvalues * .75 + 1);
mark[4] = xid_power.maxdb;

/*
    Draw the axis for the plot.
*/

XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,roi_y,
    y_axis,x_axis);					            /* Y axis */
if (npts == 1)
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
        x_axis,(long)(y_axis + power_scale * npts),x_axis);   /* X axis */
else
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
        x_axis,(long)(y_axis + power_scale * (npts - 1)),x_axis);   /* X axis */

/*
    Label the Y axis.
*/
strcpy(axis,"PSD");
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,
    (y_axis - pwidth - 40),(x_axis - (long)((xid_power.height - 2 * 
    INDENT_TOP - INDENT_BOTTOM) * .62)) + xid.font_info->descent,axis,len);

strcpy(axis,"(dB)");
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,
    (y_axis - pwidth - 40),(x_axis - (long)((xid_power.height - 2 * 
    INDENT_TOP - INDENT_BOTTOM) * .62)) + xid.font_info->descent * 2 + 
    xid.font_info->ascent,axis,len);

XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
    (x_axis - 1),(y_axis - 5),(x_axis - 1));
XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
    (x_axis - (long)((xid_power.height - 2 * INDENT_TOP - INDENT_BOTTOM) * 
    .25)),(y_axis - 5),(x_axis - (long)((xid_power.height - 2 * INDENT_TOP - 
    INDENT_BOTTOM) * .25)));
XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
    (x_axis - (long)((xid_power.height - 2 * INDENT_TOP - INDENT_BOTTOM) * 
    .50)),(y_axis - 5),(x_axis - (long)((xid_power.height - 2 * INDENT_TOP - 
    INDENT_BOTTOM) * .50)));
XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
    (x_axis - (long)((xid_power.height - 2 * INDENT_TOP - INDENT_BOTTOM) * 
    .75)),(y_axis - 5),(x_axis - (long)((xid_power.height - 2 * INDENT_TOP - 
    INDENT_BOTTOM) * .75)));
XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
    (x_axis - (xid_power.height - 2 * INDENT_TOP - INDENT_BOTTOM)),(y_axis - 5),
    (x_axis - (xid_power.height - 2 * INDENT_TOP - INDENT_BOTTOM)));

sprintf(axis,"%d",mark[0]);
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,
    (y_axis - pwidth - 5),(x_axis - 1) + xid.font_info->descent,
    axis,len);

sprintf(axis,"%d",mark[1]);
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,
    (y_axis - pwidth - 5), (x_axis - (long)((xid_power.height - 2 * 
    INDENT_TOP - INDENT_BOTTOM) * .25)) + xid.font_info->descent,axis,len);

sprintf(axis,"%d",mark[2]);
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,
    (y_axis - pwidth - 5),(x_axis - (long)((xid_power.height - 2 * 
    INDENT_TOP - INDENT_BOTTOM) * .50)) + xid.font_info->descent,axis,len);

sprintf(axis,"%d",mark[3]);
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,
    (y_axis - pwidth - 5),(x_axis - (long)((xid_power.height - 2 * 
    INDENT_TOP - INDENT_BOTTOM) * .75)) + xid.font_info->descent,axis,len);

sprintf(axis,"%d",mark[4]);
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,
    (y_axis - pwidth - 5),(x_axis - (xid_power.height - 2 * 
    INDENT_TOP - INDENT_BOTTOM)) + xid.font_info->descent,axis,len);

/*
   Label the X axis.
*/
strcpy(axis,"Normalized Frequency");
len = strlen(axis);
pwidth = XTextWidth(xid.font_info,axis,len);
xstring = y_axis + ((power_scale * npts) / 2.0) - (pwidth / 2.0);
if (xstring < 0)
    xstring = 0;
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,xstring,x_axis + 
    xid.font_info->ascent + 30,axis,len);

fmark[0] = xid_power.start_freq;
fmark[1] = xid_power.start_freq + ((xid_power.end_freq - xid_power.start_freq)
    * .25);
fmark[2] = xid_power.start_freq + ((xid_power.end_freq - xid_power.start_freq)
    * .50);
fmark[3] = xid_power.start_freq + ((xid_power.end_freq - xid_power.start_freq)
    * .75);
fmark[4] = xid_power.end_freq;

XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,x_axis,
    y_axis,(x_axis + 5));

if (npts == 1)
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
        npts * power_scale * .25),x_axis,(long)(y_axis + npts * 
        power_scale * .25),(x_axis + 5));
else
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
        (npts - 1) * power_scale * .25),x_axis,(long)(y_axis + (npts - 1) * 
        power_scale * .25),(x_axis + 5));

if (npts == 1)
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	npts * power_scale * .50),x_axis,(long)(y_axis + npts * 
	power_scale * .50),(x_axis + 5));
else
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	(npts - 1) * power_scale * .50),x_axis,(long)(y_axis + (npts - 1) * 
	power_scale * .50),(x_axis + 5));

if (npts == 1)
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	npts * power_scale * .75),x_axis,(long)(y_axis + npts * 
	power_scale * .75),(x_axis + 5));
else
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	(npts - 1) * power_scale * .75),x_axis,(long)(y_axis + (npts - 1) * 
	power_scale * .75),(x_axis + 5));

if (npts == 1)
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	npts * power_scale),x_axis,(long)(y_axis + npts * 
	power_scale), (x_axis + 5));
else
    XDrawLine(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	(npts - 1) * power_scale),x_axis,(long)(y_axis + (npts - 1) * 
	power_scale), (x_axis + 5));

sprintf(axis,"%.3f",fmark[0]);
len = strlen(axis);
XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,y_axis,
    x_axis + xid.font_info->ascent + 10,axis,len);

sprintf(axis,"%.3f",fmark[1]);
len = strlen(axis);
if (npts == 1)
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	npts * power_scale * .25),x_axis + xid.font_info->ascent + 10,
	axis,len);
else
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	(npts - 1) * power_scale * .25),x_axis + xid.font_info->ascent + 10,
	axis,len);

sprintf(axis,"%.3f",fmark[2]);
len = strlen(axis);
if (npts == 1)
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	npts * power_scale * .50),x_axis + xid.font_info->ascent + 10,
	axis,len);
else
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	(npts - 1) * power_scale * .50),x_axis + xid.font_info->ascent + 10,
	axis,len);

sprintf(axis,"%.3f",fmark[3]);
len = strlen(axis);
if (npts == 1)
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	npts * power_scale * .75),x_axis + xid.font_info->ascent + 10,
	axis,len);
else
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	(npts - 1) * power_scale * .75),x_axis + xid.font_info->ascent + 10,
	axis,len);

sprintf(axis,"%.3f",fmark[4]);
len = strlen(axis);
if (npts == 1)
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	npts * power_scale),x_axis + xid.font_info->ascent + 10,axis,len);
else
    XDrawString(xid.display,xid_power.info_pixmap,xid_power.gc,(long)(y_axis + 
	(npts - 1) * power_scale),x_axis + xid.font_info->ascent + 10,axis,len);

/*
    Copy the pixmap to the "clear template."
*/
XCopyArea(xid.display,xid_power.info_pixmap,xid_power.clear_pixmap,
    xid_power.gc,0,0,xid_power.width,xid_power.height,0,0);

/*
    Assign width, height, maximum width and height for the power spectrum 
    scrolled window.
*/
XtSetArg(args[0],XmNwidth,xid_power.width);
XtSetArg(args[1],XmNheight,xid_power.height);
XtSetArg(args[2],XmNmaxWidth,xid_power.width);
XtSetArg(args[3],XmNmaxHeight,xid_power.height);
XtSetValues(power_scrollwin_bulletinboard,args,4);
XFlush(xid.display);

/*
    Resize the power spectrum viewport widget.
*/
if (npts == 1)
    view_width = (long)(power_scale * npts + 2 * INDENT_LEFT);
else
    view_width = (long)(power_scale * (npts - 1) + 2 * INDENT_LEFT);
view_width = (view_width > MAX_WIDTH) ? MAX_WIDTH : view_width;
view_height= xid_power.height;
view_height= (view_height > MAX_VHEIGHT) ? MAX_VHEIGHT : view_height;

XtVaGetValues(power_scrollwin,XmNwidth,&scroll_width,XmNheight,&scroll_height,
    NULL);
XtResizeWidget(power_scrollwin,view_width+xid_power.view_xoffset,
    view_height+xid_power.view_yoffset,1);

/*
    Resize the entire shell based on the amount the scrolled window changed.
*/
XtVaGetValues(Shell_Power,XmNwidth,&shell_width,XmNheight,&shell_height,NULL);
w_change = scroll_width - (view_width + xid_power.view_xoffset);
XtResizeWidget(Shell_Power,shell_width - w_change,POWER_SHELLH,0);

/*
    Set the power spectrum window flag to active.
*/
xid_power.flags = xid_power.flags | POWER_ACTIVE; 

return;
}

void
power_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is used to restore any image data destroyed by the horizontal
    and/or vertical cursor tracking line.  The appropriate event handlers and
    callbacks are removed and resources are freed.  POWER_DESTROY then 
    closes the power spectrum window and sets the main panel button to 
    sensitive.
    
By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
long	plane_idx;		/* Memory plane index.			      */

/*
    Remove any event handlers and callbacks no longer needed.
*/
if (xid_power.flags & POWER_AUTO)
    XtRemoveEventHandler(main_image_bulletinboard,
	PointerMotionMask|ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,
	NULL);
else
    XtRemoveEventHandler(main_image_bulletinboard,
	ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,NULL);

/*
    Determine which (if any) tracking line was displayed.
*/
if (xid_power.flags & POWER_X)
    xid.num_horiz--;
else if (xid_power.flags & POWER_Y)
    xid.num_vert--;

/*
    Restore any image data written over by the vertical or horizontal cursor 
    tracking line if it is no longer needed.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

if ((xid.num_vert == 0) && (xid.line_x != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,
        1,xid_planes[plane_idx].dspwin[NL],True);
    xid.line_x = -1;
    xid.flags = xid.flags & (~XID_VERT);
    }
if ((xid.num_horiz == 0) && (xid.line_y != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
        xid_planes[plane_idx].dspwin[NS],1,True);
    xid.line_y = -1;
    xid.flags = xid.flags & (~XID_HORIZ);
    }
if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
    {
    XFreeGC(xid.display,xid.line_gc);
    xid.line_gc = (GC)NULL;
    }

/*
   Free resources.
*/
if (xid_power.gc != NULL)
    XFreeGC(xid.display,xid_power.gc);
XFreePixmap(xid.display,xid_power.info_pixmap);
XFreePixmap(xid.display,xid_power.clear_pixmap);
xid_power.gc = (GC)NULL;

free(xid_power.ent_points);
free(xid_power.per_points);

/*
    Close the power spectrum window and set the main panel button to sensitive.
*/
XtUnmanageChild(Power_form);
XtPopdown(Shell_Power);
XtSetSensitive(main_power_button,True);

/*
    Reset the active flag.
*/
xid_power.flags  = xid_power.flags ^ POWER_ACTIVE;
}

void
power_expose(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is used to redraw the power spectrum plot after an expose 
    event. 
    
By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
/*
    Copy the pixmap to the power spectrum window.
*/
XCopyArea(xid.display,xid_power.info_pixmap,
    XtWindow(power_scrollwin_bulletinboard),xid_power.gc,0,0,
    xid_power.width,xid_power.height,0,0);
}

void
power_dir(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating which dimension to
    calculate the power spectrum in and adds the appropriate event handlers.  
    It then determines whether the cursor tracking line is on and/or needs to 
    be turned on/off.  After the appropriate flags are set and resources 
    freed, the window is cleared and the new power spectrum plot is created.

By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
XGCValues values;               /* Values array for the GC.                   */

unsigned long
	valuemask = 0;          /* Specify no values.                         */

long	i,			/* Looping index.			      */
	nzplanes,		/* Number of planes for the Z dimension plot. */
	plane_idx;              /* Index to memory plane structure.           */

char	points[10];             /* Number of points string.	   	      */

/*
    Determine whether or not a tracking line is currently in use.
*/
if (xid_power.flags & POWER_X)
    xid.num_horiz--;
else if (xid_power.flags & POWER_Y)
    xid.num_vert--;

/*
    Set the flag indicating the dimension.
*/
if (strcmp(client,"X") == 0)
    {
    xid_power.flags = xid_power.flags & ~(POWER_X | POWER_Y | POWER_Z | 
	POWER_XY);
    xid_power.flags = xid_power.flags | POWER_X;
    }
else if (strcmp(client,"Y") == 0)
    {
    xid_power.flags = xid_power.flags & ~(POWER_X | POWER_Y | POWER_Z | 
	POWER_XY);
    xid_power.flags = xid_power.flags | POWER_Y;
    }
else if (strcmp(client,"Z") == 0)
    {
    xid_power.flags = xid_power.flags & ~(POWER_X | POWER_Y | POWER_Z | 
	POWER_XY);
    xid_power.flags = xid_power.flags | POWER_Z;
    }
else
    {
    xid_power.flags = xid_power.flags & ~(POWER_X | POWER_Y | POWER_Z | 
	POWER_XY);
    xid_power.flags = xid_power.flags | POWER_XY;
    }

/*
    Get the memory plane index of the first band displayed.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Set flags for the appropriate cursor tracking line.
*/
if (!(xid_power.flags & POWER_Z))
    {
    if (xid_power.flags & POWER_X)
        xid.num_horiz++;
    else if (xid_power.flags & POWER_Y)
        xid.num_vert++;

/*
    Create a GC for the cursor tracking line.
*/
    if (xid.line_gc == NULL)
        xid.line_gc = XCreateGC(xid.display,XtWindow(main_image_bulletinboard),
	    valuemask,&values);
    XSetForeground(xid.display,xid.line_gc,
	xid.color_table[RGBBW_IDX + WHITE_COL].pixel);
    }

/*
    If a tracking line is no longer needed, restore the image data.
*/
if ((xid.num_vert == 0) && (xid.line_x != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,
        1,xid_planes[plane_idx].dspwin[NL],True);
    xid.line_x = -1;
    xid.flags = xid.flags & (~XID_VERT);
    }
if ((xid.num_horiz == 0) && (xid.line_y != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
        xid_planes[plane_idx].dspwin[NS],1,True);
    xid.line_y = -1;
    xid.flags = xid.flags & (~XID_HORIZ);
    }
if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
    {
    XFreeGC(xid.display,xid.line_gc);
    xid.line_gc = (GC)NULL;
    }

/*
    Free the appropriate resources.
*/
if (xid_power.gc != NULL)
    {
    XFreeGC(xid.display,xid_power.gc); 
    XFreePixmap(xid.display,xid_power.info_pixmap); 
    XFreePixmap(xid.display,xid_power.clear_pixmap);
    xid_power.gc = (GC)NULL;

    free(xid_power.ent_points);
    free(xid_power.per_points);
    }

/*
   Calculate the number of points for both the maximum entropy and
   periodogram plots.
*/
if (xid_power.flags & POWER_Y)
    {
    xid_power.per_npts = (long)(nearest_power2(xid_planes[plane_idx].
	dspwin[NL]) / 2.0);
    sprintf(points,"%d",xid_power.per_npts);
    XmTextFieldSetString(power_ppoints_text,points);
    if (xid_power.flags & POWER_PER)
	{
	xid_power.ent_npts = xid_power.per_npts; 
	sprintf(points,"%d",xid_power.ent_npts);
	XmTextFieldSetString(power_epoints_text,points);
	}
    else
	{
	xid_power.ent_npts = (long)(xid_planes[plane_idx].dspwin[NL] / 2.0);
	sprintf(points,"%d",xid_power.ent_npts);
	XmTextFieldSetString(power_epoints_text,points);
	}
    }
else if (xid_power.flags & POWER_X)
    {
    xid_power.per_npts = (long)(nearest_power2(xid_planes[plane_idx].
	dspwin[NS]) / 2.0);
    sprintf(points,"%d",xid_power.per_npts);
    XmTextFieldSetString(power_ppoints_text,points);
    if (xid_power.flags & POWER_PER)
	{
	xid_power.ent_npts = xid_power.per_npts; 
	sprintf(points,"%d",xid_power.ent_npts);
	XmTextFieldSetString(power_epoints_text,points);
	}
    else
	{
	xid_power.ent_npts = (long)(xid_planes[plane_idx].dspwin[NS] / 2.0);
	sprintf(points,"%d",xid_power.ent_npts);
	XmTextFieldSetString(power_epoints_text,points);
	}
    }
else if (xid_power.flags & POWER_Z)
    {
/*
    Determine the number of planes for the Z plot.
*/
    for (i=0, nzplanes = 0; i < xid.max_plane; i++)
	{
	if (xid_planes[i].flags & USED)
	    nzplanes++; 
	}

    xid_power.per_npts = (long)(nearest_power2(nzplanes) / 2.0);
    sprintf(points,"%d",xid_power.per_npts);
    XmTextFieldSetString(power_ppoints_text,points);
    if (xid_power.flags & POWER_PER)
	{
	xid_power.ent_npts = xid_power.per_npts; 
	sprintf(points,"%d",xid_power.ent_npts);
	XmTextFieldSetString(power_epoints_text,points);
	}
    else
	{
	xid_power.ent_npts = (long)(nzplanes / 2.0);
	sprintf(points,"%d",xid_power.ent_npts);
	XmTextFieldSetString(power_epoints_text,points);
	}
    }
/*
    Clear the window and create the new power spectrum plot.
*/
XClearWindow(xid.display,XtWindow(power_scrollwin_bulletinboard));
power_create();
XCopyArea(xid.display,xid_power.info_pixmap,
    XtWindow(power_scrollwin_bulletinboard),xid_power.gc,0,0,
    xid_power.width,xid_power.height,0,0);
}

void
power_draw(w, client_data, call_data)
Widget  w;
caddr_t client_data;
caddr_t call_data;

/*******************************************************************************
Purpose:
    This routine is used to plot the current power spectrum information based 
    on the cursor location.  
    
By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
extern	float	power_scale;		/* Scaling factor used to space
					   the output points appropriately.   */

XEvent	*ev;		/* The input event.				      */

Pixel	ent_color,	/* Color for maximum entropy plot.		      */
	per_color;	/* Color for periodogram plot.			      */

COMPLEX	*freq;		/* Frequency buffer.				      */

float	*data,		/* Input data (floating point).			      */
	*out,		/* Output buffer.			 	      */
	delta,		/* Step size by which to evaluate spectrum.	      */
	evlmem(),	/* Routine to evaluate the maximum entropy 
			   coefficients.				      */
	fdt,		/* Frequency at which to evaluate spectrum.	      */
	scale,		/* Value used to scale output values to fit within
			   the bounds of the plot graph.		      */
	temp;		/* Temporary variable.			    	      */

long	in_pts,		/* Number of input points.			      */
	line,		/* Current line location in image.		      */
	sample,		/* Current sample location in line.		      */
        nvalues,        /* Number of values in the range to be plotted.       */
	i,		/* Loop index.					      */
	rplane_idx,     /* Index to red plane number of the currently
			   viewed image.			      	      */
	npts,		/* Number of points.				      */
	nzplanes,	/* Number of planes to profile in the Z dimension.    */
	*zplanes,	/* Plane numbers to profile in the Z dimension.	      */
	zpoint,		/* Index into image data buffer for profile in
			   the Z dimension.				      */
	roi_x,		/* Region of Interest X coordinate.		      */
	roi_y,		/* Region of Interest Y coordinate.		      */
	roi_w,		/* Region of Interest width.			      */
	roi_h;		/* Region of Interest height.			      */

unsigned char
	*idata;		/* Image data.					      */

/*
    Set a pointer to the event structure.
*/
ev = (XEvent *)call_data;

/*
    Determine the number of points.
*/
if ((xid_power.flags & POWER_ENT) && (xid_power.flags & POWER_PER))
    {
    if (xid_power.ent_npts > xid_power.per_npts)
	npts = xid_power.ent_npts;
    else
	npts = xid_power.per_npts;
    }
else if (xid_power.flags & POWER_ENT)
    npts = xid_power.ent_npts;
else if (xid_power.flags & POWER_PER)
    npts = xid_power.per_npts;

/*
    The displayed image must be black and white (single band).
*/
if ((xid_view.rgbplanes[RED] != xid_view.rgbplanes[GREEN]) || (
    xid_view.rgbplanes[GREEN] != xid_view.rgbplanes[BLUE]))
    {
    xid_message("Power spectrum calculations may be done on black and white images only");
    return;
    }

/*
    Validate the cursor coordinates.
*/
rplane_idx = xid_view.rgbplanes[RED] - 1;

if ((ev->xmotion.x < 0) || (ev->xmotion.x > xid_planes[rplane_idx].dspwin[NS] 
    - 1) || (ev->xmotion.y < 0) || (ev->xmotion.y > 
    xid_planes[rplane_idx].dspwin[NL] - 1))
    return;

/*
    Set the current line and sample based on the cursor location.
*/
line = ev->xmotion.y;
sample = ev->xmotion.x;

/*
    Determine the plane numbers of the planes to use in the Z dimension.
*/
if (xid_power.flags & POWER_Z)
    {
    zplanes = (long *) calloc(xid.max_plane,sizeof(long));
    if (zplanes == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
    for (i=0, nzplanes = 0; i < xid.max_plane; i++)
	{
	if (xid_planes[i].flags & USED)
	    {
	    zplanes[nzplanes] = i;
	    nzplanes++; 
	    }
	}
    }

/*
    Set the number of output values. 
*/
nvalues = xid_power.maxdb - xid_power.mindb + 1;

/*
    Set region of interest and clear the plot.
*/
roi_x = (INDENT_LEFT + 1 + 1);
roi_y = (INDENT_TOP + 1);
if (npts == 1)
    roi_w = (long)(power_scale * npts);
else
    roi_w = (long)(power_scale * (npts - 1));
roi_h = xid_power.height - (2 * INDENT_TOP) - INDENT_BOTTOM;

/*
    Clear the old data.
*/
XCopyArea(xid.display,xid_power.clear_pixmap,xid_power.info_pixmap,
    xid_power.gc,roi_x,roi_y,roi_w,roi_h,roi_x,roi_y);

/*
    Plot the maximum entropy results. 
*/
if (xid_power.flags & POWER_ENT)
    {
/*
    Calculate the scale factor used for plotting the Y coordinates.
*/
    scale = roi_h / (float)nvalues;

/*
    Allocate space for the results of the maximum entropy calculation.
*/
    out = (float *)malloc(xid_power.ent_npts * sizeof(float));
    if (out == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
/*
    Allocate buffer space for the input data.
*/
    if (xid_power.flags & POWER_X)
	{
        data = (float *)malloc(xid_planes[rplane_idx].dspwin[NS] * 
	    sizeof(float));
	in_pts = xid_planes[rplane_idx].dspwin[NS];
	}
    else if (xid_power.flags & POWER_Y)
	{
        data = (float *)malloc(xid_planes[rplane_idx].dspwin[NL] * 
	    sizeof(float));
	in_pts = xid_planes[rplane_idx].dspwin[NL];
	}
    else if (xid_power.flags & POWER_Z)
	{
        data = (float *)malloc(nzplanes * sizeof(float));
	in_pts = nzplanes; 
	}
    if (data == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
/*
    Assign the appropriate data to the input buffers.
*/
    if (xid_power.flags & POWER_X)
	for (i=0; i < xid_planes[rplane_idx].dspwin[NS]; i++)
	    data[i] = *(xid_planes[rplane_idx].image_data + (line * 
		xid_planes[rplane_idx].dspwin[NS] + i));
    else if (xid_power.flags & POWER_Y)
	for (i=0; i < xid_planes[rplane_idx].dspwin[NL]; i++)
	    {
	    line = i * xid_planes[rplane_idx].dspwin[NS] + sample;
	    data[i] = *(xid_planes[rplane_idx].image_data + line);
	    }
    else if (xid_power.flags & POWER_Z)
	for (i=0; i < nzplanes; i++)
	    {
	    zpoint = (line * xid_planes[rplane_idx].dspwin[NS]) + sample;
	    data[i] = (*(xid_planes[zplanes[i]].image_data + zpoint));
	    }

/*
    Calculate the maximum entropy coefficients.
*/
    memcof(data-1,in_pts,xid_power.order,&xid_power.b0,xid_power.coef-1);
    temp = xid_power.ent_npts;
    delta = (xid_power.end_freq - xid_power.start_freq) / temp; 

    for (i=0; i < xid_power.ent_npts; i++)
	{
/*
    Calculate the power spectrum estimate P(f) as a function of fdt.
*/
	fdt = xid_power.start_freq + (float)i * delta;
        out[i] = evlmem(fdt,xid_power.coef,xid_power.order,xid_power.b0);
/*
    Convert the output value to dB (decibels).
*/
	out[i] = (float)(20.0 * log10((double)(1 + out[i])));
/*
    Calculate the x and y positions.
*/
	xid_power.ent_points[i].x = (long)(roi_x + i * power_scale); 
	xid_power.ent_points[i].y = (roi_h + INDENT_TOP + 1) - (long)(out[i] * 
	    scale);
	}
/*
    Determine the color to be used and plot the points.
*/
    XtVaGetValues(power_entropy_button,XmNselectColor,&ent_color,NULL);
    XSetForeground(xid.display,xid_power.gc,ent_color);
    if (xid_power.flags & POWER_CONT)
        XDrawLines(xid.display,xid_power.info_pixmap,xid_power.gc,
            xid_power.ent_points,xid_power.ent_npts,CoordModeOrigin);
    else
        XDrawPoints(xid.display,xid_power.info_pixmap,xid_power.gc,
            xid_power.ent_points,xid_power.ent_npts,CoordModeOrigin);
/*
    Free resources.
*/
    free(data);
    free(out);
    }

/*
    Plot the periodogram results. 
*/
if (xid_power.flags & POWER_PER)
    {
/*
    Calculate the scale factor used for plotting the Y coordinates.
*/
    scale = roi_h / (float)nvalues;

/*
    Allocate buffer space for the input data.
*/
    if (xid_power.flags & POWER_X)
	{
        idata = (unsigned char *)malloc(xid_planes[rplane_idx].dspwin[NS]);
	in_pts = xid_planes[rplane_idx].dspwin[NS];
	}
    else if (xid_power.flags & POWER_Y)
	{
        idata = (unsigned char *)malloc(xid_planes[rplane_idx].dspwin[NL]);
	in_pts = xid_planes[rplane_idx].dspwin[NL];
	}
    else if (xid_power.flags & POWER_Z)
	{
        idata = (unsigned char *)malloc(nzplanes);
	in_pts = nzplanes; 
	}
    if (idata == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
/*
    Allocate buffer space for the output data.
*/
    out = (float *)malloc(xid_power.per_npts * sizeof(float));
    if (out == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
/*
    Allocate buffer space for the frequency data.
*/
    freq = (COMPLEX *)malloc(xid_power.per_npts * 4 * sizeof(float));
    if (freq == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
/*
    Assign the appropriate data to the input buffers.
*/
    if (xid_power.flags & POWER_X)
	for (i=0; i < xid_planes[rplane_idx].dspwin[NS]; i++)
	    idata[i] = *(xid_planes[rplane_idx].image_data + (line * 
		xid_planes[rplane_idx].dspwin[NS] + i));
    else if (xid_power.flags & POWER_Y)
	for (i=0; i < xid_planes[rplane_idx].dspwin[NL]; i++)
	    {
	    line = i * xid_planes[rplane_idx].dspwin[NS] + sample;
	    idata[i] = *(xid_planes[rplane_idx].image_data + line);
	    }
    else if (xid_power.flags & POWER_Z)
	for (i=0; i < nzplanes; i++)
	    {
	    zpoint = (line * xid_planes[rplane_idx].dspwin[NS]) + sample;
	    idata[i] = (*(xid_planes[zplanes[i]].image_data + zpoint));
	    }
/*
    Pad the input data array with zeros to the specified power of 2.
*/
    pad_to_pow2(idata,in_pts,(float *)freq,xid_power.per_npts * 2);
/*
    Replace the input array with its discrete Fourier transform.
*/
    fft(freq,xid_power.per_npts*2,FORWARD);

/*
    Calculate the periodogram results.
*/
    for (i=0; i < xid_power.per_npts; i++)
	{
	out[i] = (freq[i].re * freq[i].re + freq[i].im * freq[i].im) / 
	    (xid_power.per_npts * 2);
/*
    Convert the output value to dB (decibels).
*/
	out[i] = (float)(20.0 * log10((double)(1 + out[i])));
/*
    Calculate the x and y positions.
*/
	xid_power.per_points[i].x = (long)(roi_x + i * power_scale); 
	xid_power.per_points[i].y = (roi_h + INDENT_TOP + 1) - (long)(out[i] * 
	    scale);
	}
/*
    Determine the color to be used and plot the points.
*/
    XtVaGetValues(power_periodogram_button,XmNselectColor,&per_color,NULL);
    XSetForeground(xid.display,xid_power.gc,per_color);
    if (xid_power.flags & POWER_CONT)
        XDrawLines(xid.display,xid_power.info_pixmap,xid_power.gc,
            xid_power.per_points,xid_power.per_npts,CoordModeOrigin);
    else
        XDrawPoints(xid.display,xid_power.info_pixmap,xid_power.gc,
            xid_power.per_points,xid_power.per_npts,CoordModeOrigin);
/*
    Freee resources.
*/
    free(idata);
    free(out);
    free(freq);
    }

/*
    Copy the working power spectrum pixmap to the power spectrum window.
*/
XCopyArea(xid.display,xid_power.info_pixmap,
    XtWindow(power_scrollwin_bulletinboard),xid_power.gc,roi_x,roi_y,roi_w+1,
    roi_h+1,roi_x,roi_y);
}

/* ARGSUSED */
void
power_entropy(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating the user has selected
    to perform a one-dimensional power spectral density calculation using
    the maximum entropy (Burg's) method.

By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

/*
    Set the flag indicating the type of plot.
*/
if (acs->set)
    xid_power.flags |= POWER_ENT;
else
    xid_power.flags &= ~POWER_ENT;

/*
    Free the appropriate resources.
*/
if (xid_power.gc != NULL)
    {
    XFreeGC(xid.display,xid_power.gc); 
    XFreePixmap(xid.display,xid_power.info_pixmap); 
    XFreePixmap(xid.display,xid_power.clear_pixmap);
    xid_power.gc = (GC)NULL;

    free(xid_power.ent_points);
    free(xid_power.per_points);
    }

/*
   Clear the window and create the new power spectrum plot.
*/
XClearWindow(xid.display,XtWindow(power_scrollwin_bulletinboard));
power_create();
XCopyArea(xid.display,xid_power.info_pixmap,
    XtWindow(power_scrollwin_bulletinboard),xid_power.gc,0,0,
    xid_power.width,xid_power.height,0,0);
}

/* ARGSUSED */
void
power_periodogram(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating the user has selected
    to perform a one-dimensional power spectral density calculation using
    a periodogram (Barlett estimate).

By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
char	tmpstr[10];             /* Number of points string.                   */

XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

/*
    Set the flag indicating the type of plot.
*/
if (acs->set)
    xid_power.flags |= POWER_PER;
else
    xid_power.flags &= ~POWER_PER;

/*
    Free the appropriate resources.
*/
if (xid_power.gc != NULL)
    {
    XFreeGC(xid.display,xid_power.gc); 
    XFreePixmap(xid.display,xid_power.info_pixmap); 
    XFreePixmap(xid.display,xid_power.clear_pixmap);
    xid_power.gc = (GC)NULL;

    free(xid_power.ent_points);
    free(xid_power.per_points);
    }

/*
   If the maximum entropy plot is also being displayed, force the maximum
   entropy start frequency, end frequency and number of points to be the
   same as for the periodogram.
*/
if ((xid_power.flags & POWER_ENT) && (acs->set))
    {
    sprintf(tmpstr,"%d",xid_power.per_npts);
    XmTextFieldSetString(power_epoints_text,tmpstr);
    XtVaSetValues(power_epoints_text,XmNeditable,False,NULL);
    xid_power.ent_npts = xid_power.per_npts;
    XmTextFieldSetString(power_estart_text,"0");
    XtVaSetValues(power_estart_text,XmNeditable,False,NULL);
    xid_power.start_freq = 0.0;
    XmTextFieldSetString(power_eend_text,".5");
    XtVaSetValues(power_eend_text,XmNeditable,False,NULL);
    xid_power.end_freq = 0.5;
    }
else if ((xid_power.flags & POWER_ENT) && !(acs->set))
    {
    XtVaSetValues(power_epoints_text,XmNeditable,True,NULL);
    XtVaSetValues(power_estart_text,XmNeditable,True,NULL);
    XtVaSetValues(power_eend_text,XmNeditable,True,NULL);
    }

/*
   Clear the window and create the new power spectrum plot.
*/
XClearWindow(xid.display,XtWindow(power_scrollwin_bulletinboard));
power_create();
XCopyArea(xid.display,xid_power.info_pixmap,
    XtWindow(power_scrollwin_bulletinboard),xid_power.gc,0,0,
    xid_power.width,xid_power.height,0,0);
}

/* ARGSUSED */
void
power_text(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
/*******************************************************************************
Purpose:
    This routine is used to retrieve user-specified values including start
    frequency, end frequency, number of points or order.  A new power
    spectrum plot is created based on the new values entered.

By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
char    *tmpstr;                /* Temporary storage for a text string.       */

XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

/*
    Get the user-specified value.
*/
tmpstr = XmTextGetString(w);

/*
    Determine which parameter needs to be changed (start frequency, end
    frequency, number of points or order).
*/
if (w == power_epoints_text)
    sscanf(tmpstr,"%d",&xid_power.ent_npts);
else if (w == power_eend_text)
    sscanf(tmpstr,"%f",&xid_power.end_freq);
else if (w == power_estart_text)
    sscanf(tmpstr,"%f",&xid_power.start_freq);
else if (w == power_order_text)
    {
    sscanf(tmpstr,"%d",&xid_power.order);
    if (xid_power.coef != NULL)
	free(xid_power.coef);
    xid_power.coef = (float *)malloc(xid_power.order * sizeof(float));
    if (xid_power.coef == NULL)
	{
	xid_message("Insufficient memory available"); 
	return;
	}
    }
XtFree(tmpstr);

/*
    Free the appropriate resources.
*/
if (xid_power.gc != NULL)
    {
    XFreeGC(xid.display,xid_power.gc); 
    XFreePixmap(xid.display,xid_power.info_pixmap); 
    XFreePixmap(xid.display,xid_power.clear_pixmap);
    xid_power.gc = (GC)NULL;

    free(xid_power.ent_points);
    free(xid_power.per_points);
    }

/*
   Clear the window and create the new power spectrum plot.
*/
XClearWindow(xid.display,XtWindow(power_scrollwin_bulletinboard));
power_create();
XCopyArea(xid.display,xid_power.info_pixmap,
    XtWindow(power_scrollwin_bulletinboard),xid_power.gc,0,0,
    xid_power.width,xid_power.height,0,0);
}

memcof(data,n,m,pm,cof)

float data[];
float *pm;
float cof[];
int n,m;
/*******************************************************************************
Purpose:
    Calculate a coefficient vector and scalar which are the coefficients
    of the maximum entropy method.

By D. Steinwand, USGS/EROS Data Center, 4/92

Algorithm Description:
    Obtained from the Numerical Recipes in C text.
*******************************************************************************/
{
int k,j,i;
float p=0.0,*wk1,*wk2,*wkm,*vector();
float num,denom;
void free_vector();

wk1=vector(1,n);
wk2=vector(1,n);
wkm=vector(1,m);
for (j=1;j<=n;j++) p += (data[j] * data[j]);
*pm=p/n;
wk1[1]=data[1];
wk2[n-1]=data[n];
for (j=2;j<=n-1;j++)
   {
   wk1[j]=data[j];
   wk2[j-1]=data[j];
   }
for (k=1;k<=m;k++)
   {
   num=0.0;
   denom=0.0;
   for (j=1;j<=(n-k);j++)
      {
      num += wk1[j]*wk2[j];
      denom += ((wk1[j] * wk1[j]) + (wk2[j] * wk2[j]));
      }
   if (denom < 1.0e-6) cof[k] = 0.0;
   else cof[k]=2.0*num/denom;
   *pm *= (1.0 - (cof[k] * cof[k]));
   if (k != 1)
      for (i=1;i<=(k-1);i++)
	 cof[i]=wkm[i]-cof[k]*wkm[k-i];

/* The algorithm is recursive, building up the answer for larger and larger
   values of m until the desired value is reached.
  -----------------------------------------------*/
   if (k == m)
      {
      free_vector(wkm,1,m);
      free_vector(wk2,1,n);
      free_vector(wk1,1,n);
      return;
      }
   for (i=1;i<=k;i++) wkm[i]=cof[i];
   for (j=1;j<=(n-k-1);j++)
      {
      wk1[j] -= wkm[k]*wk2[j];
      wk2[j]=wk2[j+1]-wkm[k]*wk1[j+1];
      }
   }
return;
}

float evlmem(fdt,cof,m,pm)
float fdt,cof[],pm;
int m;
/*******************************************************************************
Purpose:
    This routine returns the power spectrum estimate P(f) as a function of fdt.
    
By D. Steinwand, USGS/EROS Data Center, 4/92

Algorithm Description:
    Obtained from the Numerical Recipes in C text.
*******************************************************************************/
{
int i;
float sumr=1.0,sumi=0.0;
double wr=1.0,wi=0.0,wpr,wpi,wtemp,theta;
float temp;

theta=6.28318530717959*fdt;
wpr=cos(theta);
wpi=sin(theta);
for (i=0;i<m;i++)
   {
   wr=(wtemp=wr)*wpr-wi*wpi;
   wi=wi*wpr+wtemp*wpi;
   sumr -= cof[i]*wr;
   sumi -= cof[i]*wi;
   }
temp = sumr*sumr+sumi*sumi;
if (temp < 1.0e-6) return 0.0;
else return (float)(pm/temp);
}

/* Misc Support Routines--Dynamically allocate/free memory
  --------------------------------------------------------*/
void nrerror(error_text) char error_text[]; {
printf("Run-time error...\n");
printf("%s\n",error_text);
printf("...now exiting to system...\n");
exit(1);}

float *vector(nl,nh)  int nl,nh; { float *v;
v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
if (!v) nrerror("allocation failure in vector()");
return v-nl; }

void free_vector(v,nl,nh)float *v;int nl,nh;{free((char*) (v+nl));}

nearest_power2(ns) long ns; 

/*******************************************************************************
Purpose:
    Given the input values, this routine returns the next greatest power of two.

By D. Steinwand, USGS/EROS Data Center, 4/92

Algorithm Description:
*******************************************************************************/
{
long pow2;
long exp;

for (pow2 = 1, exp = 0;; exp++, pow2 = pow2 << 1)
    if (pow2 >= ns) break;
return(pow2);
}

pad_to_pow2(in, in_ns, out, out_ns) 
unsigned char *in; 			/* Input array */
long in_ns;				/* Number of samples in input array */
float *out;				/* Output array */
long out_ns;				/* Number of samples in output */
/*******************************************************************************
Purpose:
    This routine pads the input array (with zeros) to the specified power of 
    two.

By D. Steinwand, USGS/EROS Data Center, 4/92

Algorithm Description:
*******************************************************************************/
{
register long i;
register unsigned char *in_ptr;
register float *out_ptr;

for(in_ptr=in,out_ptr=out,i=in_ns;i--;*out_ptr++=(float)*in_ptr++,*out_ptr++=0.0);
for (i = out_ns - in_ns; i--; *out_ptr++ = 0.0, *out_ptr++ = 0.0);
return;
}

fft(dat,n,isign)
COMPLEX *dat;
int n,isign;
/*******************************************************************************
Purpose:
    FFT replaces the "dat" array by its discrete Fourier transform, if
    ISIGN = 1.  If ISIGN = -1, this routine performs the inverse discrete
    Fourier transform, multiplied by N.  "dat" is a complex array of length
    N, with N an integer power of 2.

    In this code, sines and cosines are computed via trigonometric recurrence.
    Other optimizations are also incorporated.

By D. Steinwand, USGS/EROS Data Center, 1/92

Algorithm Description:
    This routine was derrived from:  "Numerical Recipes, the Art of Scientific
     Computing", by Press, Flannery, Teukolsky, and Vetterling, Cambridge
     University Press, 1986.
*******************************************************************************/
{
int mmax,m,j,istep,i,half_n,n_1;
register float *djre,*djim,*dire,*diim;
register float tempr,tempi;
register double wtemp,wr,wpr,wpi,wi,theta;
COMPLEX *data;

data=dat;

/* Perform bit reversal
  --------------------*/
for (j=0,i=1,half_n=n>>1,n_1=n-1;i<n_1;i++) 
   {
   m=half_n;
   while(m<=j){j-=m;m>>=1;}
   j+=m;
   if (i > j) { SWAP(data[j].re,data[i].re);SWAP(data[j].im,data[i].im); }
   }

/* The Danielson-Lanczos section of the routine begins here.  The outter
   loop is executed log2NN times
  -----------------------------*/
mmax=1;
while (n > mmax) 
   {
   istep=mmax<<1;

/* Initialize for the trigonometric recurrence
  -------------------------------------------*/
   theta=PI/(isign*mmax);
   wtemp=sin(0.5*theta);
   wpr = -2.0*wtemp*wtemp;
   wpi=sin(theta);
   wr=1.0; wi=0.0;
   for (m=0;m<mmax;m++) 
      {

/* The Danielson-Lanczos formula...
  --------------------------------*/
      for (i=m;i<n;i+=istep) 
         {
	 j=i+mmax;
	 djre= &(data[j].re);
	 dire= &(data[i].re);
	 djim= &(data[j].im);
	 diim= &(data[i].im);
	 tempr=wr* *djre-wi* *djim;
	 tempi=wr* *djim+wi* *djre;
	 *djre= *dire-tempr;
	 *djim= *diim-tempi;
	 *dire += tempr;
	 *diim += tempi;
	 }

/* The trigonometric recurrence
  ----------------------------*/
      wr+=(wtemp=wr)*wpr-wi*wpi;
      wi+=wi*wpr+wtemp*wpi;
      }
   mmax=istep;
   }
}
