/*
 *   Context.c
 */

/*
 *  Implementation file for the Context compound widget.
 */

/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or 
 * publicity pertaining to distribution of the software without specific, 
 * written prior permission.  USGS makes no representations about the 
 * suitability of this software for any purpose.  It is provided "as is" 
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS 
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *  Change log:
 *
 *	D. Akkerman, USGS/EROS Data Center, 7/92
 *	 - inception.
 *
 */

/*
 *  Overview
 */

/*
 *  This module implements a context object for the DLG-E processor.  The
 *  context object stores, manages and dispatches the current mouse button
 *  context.  In addition, it provides visual information to the user for
 *  the function of each mouse button in the given context and provides for
 *  the switching of contexts as well as additional functions not available on
 *  mouse buttons but available within a given context.  Finally, it also
 *  provides public functions for the setting, adding, removing of contexts
 *  as well as the callback hooks used to implement a given context.
 *  Currently, a three-button mouse is assumed.
 *
 *  The context object itself is constructed of a Motif widget tree and is
 *  a subclass of the BulletinBoard manager widget.
 *
 */

/*
 *  Required include files.
 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DrawingA.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MenuShell.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>

#include "ContextP.h"

/*
 *  Working constants.
 */

#define MAGIC_NUMBER	7	/* Option menu extra width. (??)	*/
#define DEFAULT_OPTION_BUTTON_BORDER_WIDTH	0
#define DEFAULT_OPTION_MENU_BORDER_WIDTH	0
#define DEFAULT_DRAWINGAREA_BORDER_WIDTH	1
#define DEFAULT_ADDITIONAL_BUTTON_BORDER_WIDTH	0
#define DEFAULT_ADDITIONAL_MENU_BORDER_WIDTH	0

/*
 *  Resource table.
 */

#define res_offset(field) XtOffset(XeContextWidget, field)

static XtResource resources[] =
{
	{XeNlabelFont, XeCLabelFont, XtRFontStruct, sizeof(XFontStruct *),
	 res_offset(context.label_font), XtRString, "XtDefaultFont"},
	{XeNbuttonFont, XeCButtonFont, XtRFontStruct, sizeof(XFontStruct *),
	 res_offset(context.button_font), XtRString, "XtDefaultFont"},
	{XeNvisual, XeCVisual, XtRVisual, sizeof(Visual*),
	 res_offset(context.visual), XtRImmediate, CopyFromParent},
	{XeNoptionButtonBorderWidth, XeCOptionButtonBorderWidth, XtRDimension,
	 sizeof(Dimension), res_offset(context.option_button_border_width),
	 XtRImmediate, (caddr_t)DEFAULT_OPTION_BUTTON_BORDER_WIDTH},
	{XeNoptionMenuBorderWidth, XeCOptionMenuBorderWidth, XtRDimension,
	 sizeof(Dimension), res_offset(context.option_menu_border_width),
	 XtRImmediate, (caddr_t)DEFAULT_OPTION_MENU_BORDER_WIDTH},
	{XeNdrawingAreaBorderWidth, XeCDrawingAreaBorderWidth, XtRDimension,
	 sizeof(Dimension), res_offset(context.drawingarea_border_width),
	 XtRImmediate, (caddr_t)DEFAULT_DRAWINGAREA_BORDER_WIDTH},
	{XeNadditionalButtonBorderWidth, XeCAdditionalButtonBorderWidth,
	 XtRDimension, sizeof(Dimension),
	 res_offset(context.additional_button_border_width),
	 XtRImmediate, (caddr_t)DEFAULT_ADDITIONAL_BUTTON_BORDER_WIDTH},
	{XeNadditionalMenuBorderWidth, XeCAdditionalMenuBorderWidth,
	 XtRDimension, sizeof(Dimension),
	 res_offset(context.additional_menu_border_width),
	 XtRImmediate, (caddr_t)DEFAULT_ADDITIONAL_MENU_BORDER_WIDTH},
};

/*
 *  Class methods.
 */

static void Initialize();
static void Destroy();
static Boolean SetValues();

/*
 *  Action functions.
 */


/*
 *  Private support functions.
 */

static void get_font_dimensions();


/*
 *  Class storage definition.
 */

XeContextClassRec XecontextClassRec =
{
	{ /* core_class variables */
	 (WidgetClass) &xmBulletinBoardClassRec,/* ancestor */
	 "XeContext",				/* class name */
	 sizeof(XeContextRec),			/* widget size */
	 NULL,					/* class initialize */
	 NULL,					/* class part init. */
	 NULL,					/* class inited */
	 Initialize,				/* initialize */
	 NULL,					/* initialize hook */
	 XtInheritRealize,			/* realize */
	 NULL,					/* actions */
	 0,					/* number of actions */
	 resources,				/* resources */
	 XtNumber(resources),			/* number of resources */
	 NULLQUARK,				/* xrm class */
	 TRUE,					/* compress motions */
	 XtExposeCompressMaximal,		/* compress exposures */
	 TRUE,					/* compress enter/leave */
	 FALSE,					/* visibility interest */
	 Destroy,				/* destroy */
	 XtInheritResize,			/* resize */
	 XtInheritExpose,			/* expose */
	 SetValues,				/* set values */
	 NULL,					/* set values hook */
	 XtInheritSetValuesAlmost,		/* set values almost */
	 NULL,					/* get values hook */
	 NULL,					/* accept focus */
	 XtVersion,				/* version */
	 NULL,					/* callback private */
	 NULL,					/* translation table */
	 XtInheritQueryGeometry,		/* query geometry */
	 XtInheritDisplayAccelerator,		/* display accelerator */
	 NULL,					/* extension */
	},
	{ /* composite_class variables */
	 XtInheritGeometryManager,		/* geometry manager */
	 XtInheritChangeManaged,		/* change managed */
	 XtInheritInsertChild,			/* insert child */
	 XtInheritDeleteChild,			/* delete child */
	 NULL,					/* extension */
	},
	{ /* constraint_class fields */
	 NULL,					/* subresources */
	 0,					/* number of subresources */
	 NULL,					/* record size */
	 NULL,					/* initialize */
	 NULL,					/* destroy */
	 NULL,					/* set values */
	 NULL,					/* extension */
	},
	{ /* manager_class fields */
         XtInheritTranslations,                 /* default translations   */
         NULL,                         		/* syn_resources          */
         0,               			/* num_syn_resources      */
         NULL,                                  /* syn_cont_resources     */
         0,                                     /* num_syn_cont_resources */
         NULL,            			/* parent_process         */
         NULL,                                  /* extension              */
	},
	{ /* bulletinBoard class fields */
	 FALSE,                                 /*always_install_accelerators*/
	 NULL,                                  /* geo_matrix_create */
	 XtInheritFocusMovedProc,           	/* focus_moved_proc */
	 NULL                                   /* extension */
	},
	{ /* context class fields */
	 0,
	}
 }; /* XecontextClassRec */

WidgetClass xeContextWidgetClass = (WidgetClass) &XecontextClassRec;

/*
 *  Private callbacks.
 */
 
static void option_callback(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
{
XeContextWidget context_widget;	/* Convenience pointer.                 */

int	i;
 
/*
 *  Assign convenience pointers.
 */
 
 context_widget = (XeContextWidget)client;

/*
 *  Search the option menu pushbutton widget list for the number of
 *  the widget pressed.  This value is used as an index to determine
 *  which context to activate.
 */
 
 i = 0;
 while (i < MAX_CONTEXTS)
  {
   if (w == context_widget->context.option_pushButton[i])
        break;
   i++;
  }

/*
 *  Set the requested context to active.
 */

 XeContextSetActive(context_widget,
		context_widget->context.context_list[i]->context_token); 
}
 
static void additional_callback(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
{
XeContextPart *context_part;	/* Convenience pointer.			*/

XtPointer client_data;		/* Convenience pointer.			*/

long i,				/* Element index.			*/
     current;			/* Current context value.		*/


/*
 *  Assign convenience pointers.
 */

 context_part = (XeContextPart *)client;
 current = context_part->current_context;

/*
 *  Search the additional menu pushbutton widget list for the number of
 *  the widget pressed.  This value is used as an index into a callback
 *  list.
 */

 i = 0;
 while (i < (MAX_ELEMENTS - NUM_BUTTONS))
  {
   if (w == context_part->additional_pushButton[i])
	break;
   i++;
  }

/*
 *  Increment the value found past the number of mouse buttons.
 */

 i+= NUM_BUTTONS;

/*
 *  Fire off the corresponding callback.
 */

 client_data = (context_part->context_list[current]->client_data[i]);

 (*context_part->context_list[current]->element_handlers[i])
				(w, client_data, call);
}

/*
 *  Private eventhandlers.
 */

void
context_eventhandler (w, client, event)
Widget w;
XtPointer client;
XEvent* event;
{
XeContextPart *context_part;    /* Convenience pointer.                 */

XtPointer client_data;          /* Convenience pointer.                 */
 
long i,                         /* Element index.                       */
     current;                   /* Current context value.               */
 
 
/*
 *  Assign convenience pointers.
 */
 
 context_part = (XeContextPart *)client;
 current = context_part->current_context;

/*
 *  Call the appropriate handler based on the mouse button activated.
 */

 switch (event->xbutton.button) /* Need to handle motion as well!!!! */
  {
   case Button1:
	client_data = (context_part->context_list[current]->client_data[0]);
	(*context_part->context_list[current]->element_handlers[0])
		(w, client_data, (caddr_t)event);
	break;

   case Button2:
	client_data = (context_part->context_list[current]->client_data[1]);
	(*context_part->context_list[current]->element_handlers[1])
		(w, client_data, (caddr_t)event);
	break;

   case Button3:
	client_data = (context_part->context_list[current]->client_data[2]);
	(*context_part->context_list[current]->element_handlers[2])
		(w, client_data, (caddr_t)event);
	break;
  }
}

/*
 *  XeContextWidget methods.
 */

/*
 *  Initialize() initializes each instance of a context widget.
 */

static void Initialize(request, new)
XeContextWidget	request,
	 	new;
{
Arg args[512];			/* Xt resource argument list.		*/
 
XmString xms,			/* Working Motif strings.		*/
	 cxms;
				/* Assign standard X variables.		*/
Display *display = XtDisplay(new);

Screen *scr_ptr = XtScreen(new);

int scr_num = XScreenNumberOfScreen(scr_ptr);

Widget button_gadget,		/* Option menu internal gadgets.	*/
       label_gadget;

Dimension option_menu_width,    /* Option menu dimensions.              */
          option_menu_height,
          additional_height,    /* Additional menu dimensions.          */
          additional_width,
          frame_width,          /* Frame dimensions.                    */
          frame_height,
          label_font_width,     /* Label font dimensions.               */
          label_font_height,
          button_font_width,    /* Button font dimensions.              */
          button_font_height,
          gadget_width,         /* Option button gadget width.          */
          work_frame_height,	/* Used to divide the frame into 3 parts.*/
	  info_area_width,	/* Working information area dimensions.	*/
	  info_area_height;
 
Position option_menu_x,		/* Option menu position.		*/
	 option_menu_y,
	 additional_menu_x,	/* Additional menu position.		*/
         additional_menu_y,
         frame_x,		/* Frame position.			*/
         frame_y,
         context_label_x,	/* Context label position.		*/
         context_label_y,
         button_label_x,	/* Button label x position (in frame).	*/
         button_label_y,	/* Button label y position (in frame).	*/
         drawingarea_x,		/* Drawingarea x position (in frame).	*/
         drawingarea_y,		/* Drawingarea y position (in frame).	*/
	 info_area_x,		/* Working information area position.	*/
	 info_area_y;

char resource_string[25],	/* Working resource name.		*/
     work_string[ELEMENT_MAX_LENGTH+1],
     *work_ptr;			/* String to calculate maximum string
				   dimension in pixels.			*/

long i,				/* Loop counter.			*/
     argcnt,			/* Xt resource argument count.		*/
     label_max,			/* Maximum # of pixels using label font.*/
     button_max;		/* Max. # of pixels using button font.	*/

/*
 *  Set some working variables to assist in the positioning of the widget
 *  tree based on the size of the parent widget and the size of the fonts
 *  used for the mouse and button labels.
 */

/*
 *  Get the font dimensions and create the Motif font lists for each.
 */

 get_font_dimensions(new->context.label_font,&label_font_width,
			&label_font_height);

 get_font_dimensions(new->context.button_font,&button_font_width,
			&button_font_height);

 new->context.label_font_list = XmFontListCreate(new->context.label_font,
			XmSTRING_ISO8859_1);

 new->context.button_font_list = XmFontListCreate(new->context.button_font,
			XmSTRING_ISO8859_1);

/*
 *  Initialize the current context and number.
 */

 new->context.current_context = 0;
 new->context.num_contexts = 0;

/*
 *  Initialize the current eventhandler for the context.
 */

 new->context.current_eventhandler = NULL;

/*
 *  Find the widest string possible (in pixels) and include it in dimension
 *  calculations.
 */

 work_ptr = work_string;
 for (i = 0; i < ELEMENT_MAX_LENGTH; i++, work_ptr++)
	*work_ptr = 'x';
 *work_ptr = '\0';
/* Dan trial */
/* xms = XmStringCreateLtoR(work_string, XmSTRING_ISO8859_1);*/
/* Dan end */
 xms = XmStringCreateLtoR(work_string, XmSTRING_DEFAULT_CHARSET);

 label_max = XmStringWidth(new->context.label_font_list,xms);
 button_max = XmStringWidth(new->context.button_font_list,xms);

 info_area_height= (label_font_height + GAP + GAP + PIXMAP_HEIGHT);
 info_area_width = label_max;

 frame_width = (GAP + info_area_width + GAP);

/*
 *  Create a label for the option menu.
 */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNlabelString, (cxms=XmStringCreateLtoR(
                "Context:", XmSTRING_ISO8859_1))); argcnt++;
 context_label_x = ((frame_width / 2) - 
	(XmStringWidth(new->context.label_font_list,cxms) / 2) + GAP);
 context_label_y = GAP;
 XtSetArg(args[argcnt], XmNx, context_label_x); argcnt++;
 XtSetArg(args[argcnt], XmNy, context_label_y); argcnt++;
 XtSetArg(args[argcnt], XmNfontList, new->context.label_font_list); argcnt++;
 new->context.context_label = XtCreateWidget("additional_cascadeButton",
                xmLabelWidgetClass,
                (Widget)new,
                args,
                argcnt);
 XtManageChild(new->context.context_label);
 XmStringFree( cxms );

/*
 *  Create an option menu for switching contexts.
 */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNwidth, 1); argcnt++;
 XtSetArg(args[argcnt], XmNheight, 1); argcnt++;
 XtSetArg(args[argcnt], XmNfontList, new->context.button_font_list); argcnt++;
 XtSetArg(args[argcnt], XmNcolormap, new->core.colormap); argcnt++;
 XtSetArg(args[argcnt], XmNvisual, new->context.visual); argcnt++;
 XtSetArg(args[argcnt], XmNdepth, new->core.depth); argcnt++;
 new->context.option_menuShell = XtCreatePopupShell("option_menuShell",
                xmMenuShellWidgetClass,
                (Widget)new,
                args,
                argcnt);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNrowColumnType, XmMENU_PULLDOWN); argcnt++;
 XtSetArg(args[argcnt], XmNx, 0); argcnt++;
 XtSetArg(args[argcnt], XmNy, 0); argcnt++;
 new->context.option_pulldownMenu = XtCreateWidget("option_pulldownMenu",
                xmRowColumnWidgetClass,
                new->context.option_menuShell,
                args,
                argcnt);

/*
 *  Create placeholder buttons for the option menu.
 */

 for (i = 0; i < MAX_CONTEXTS; i++)
  {
   sprintf(resource_string,"option_pushbutton%d",i);
   argcnt = 0;
   XtSetArg(args[argcnt], XmNborderWidth, 
	new->context.option_button_border_width); argcnt++;
   XtSetArg(args[argcnt], XmNlabelString, xms); argcnt++;
   XtSetArg(args[argcnt], XmNfontList, new->context.button_font_list); argcnt++;
   new->context.option_pushButton[i] = XtCreateWidget(resource_string,
                xmPushButtonWidgetClass,
                new->context.option_pulldownMenu,
                args,
                argcnt);

   XtAddCallback(new->context.option_pushButton[i],XmNactivateCallback,
	option_callback,new); 
  }

/*
 *  Always manage the first menu button for size calculations.
 */

 XtManageChild(new->context.option_pushButton[0]);

 option_menu_y = (context_label_y + label_font_height + GAP);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNborderWidth, 
	new->context.option_menu_border_width); argcnt++;
 XtSetArg(args[argcnt], XmNpacking, XmPACK_TIGHT); argcnt++;
 XtSetArg(args[argcnt], XmNrowColumnType, XmMENU_OPTION); argcnt++;
 XtSetArg(args[argcnt], XmNy, option_menu_y); argcnt++;
 XtSetArg(args[argcnt], XmNsubMenuId, new->context.option_pulldownMenu);
	 argcnt++;
 new->context.optionMenu = XtCreateWidget("optionMenu",
                xmRowColumnWidgetClass,
                (Widget)new,
                args,
                argcnt);

 XtManageChild(new->context.optionMenu); 

/*
 *  Retrieve the option menu cascade button gadget and set its
 *  font list resource.
 */

 button_gadget = XmOptionButtonGadget(new->context.optionMenu);
 argcnt = 0;
 XtSetArg(args[argcnt], XmNfontList, new->context.button_font_list); argcnt++;
 XtSetValues(button_gadget, args, argcnt);

/*
 *  Retrieve the option menu gadget button width.
 */

 label_gadget = XmOptionLabelGadget(new->context.optionMenu);
 argcnt = 0;
 XtSetArg(args[argcnt], XmNwidth, &gadget_width); argcnt++;
 XtGetValues(label_gadget, args, argcnt);

/* Dan test */
argcnt = 0;
XtSetArg(args[argcnt], XmNlabelType, XmPIXMAP); argcnt++;
XtSetArg(args[argcnt], XmNlabelPixmap, XmUNSPECIFIED_PIXMAP); argcnt++;
XtSetValues(label_gadget, args, argcnt);
/* Dan end */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNwidth, &option_menu_width); argcnt++;
 XtSetArg(args[argcnt], XmNheight, &option_menu_height); argcnt++;
 XtGetValues(new->context.option_pulldownMenu, args, argcnt);
 option_menu_width += gadget_width + 15;

/*
 *  Adjust the frame width to be the same size as the option menu if
 *  the frame is less.
 */

 frame_width = (frame_width < option_menu_width) ? 
					option_menu_width : frame_width;

 option_menu_x = ((frame_width / 2) - (option_menu_width / 2) - MAGIC_NUMBER);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNx, option_menu_x); argcnt++;
 XtSetValues(new->context.optionMenu,args,argcnt);

/*
 *  Create a pulldown menu for the (any) additional context elements
 *  not available on the mouse buttons.
 */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNborderWidth, 
	new->context.additional_menu_border_width); argcnt++;
 XtSetArg(args[argcnt], XmNrowColumnType, XmMENU_BAR); argcnt++;
 new->context.additional_menuBar = XtCreateWidget("additional_menuBar",
                xmRowColumnWidgetClass,
                (Widget)new,
                args,
                argcnt);
 
 XtManageChild(new->context.additional_menuBar);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNwidth, 1); argcnt++;
 XtSetArg(args[argcnt], XmNheight, 1); argcnt++;
 XtSetArg(args[argcnt], XmNcolormap, new->core.colormap); argcnt++;
 XtSetArg(args[argcnt], XmNvisual, new->context.visual); argcnt++;
 XtSetArg(args[argcnt], XmNdepth, new->core.depth); argcnt++;
 new->context.additional_menuShell = 
	XtCreatePopupShell("additional_menuShell",
                xmMenuShellWidgetClass,
                new->context.additional_menuBar,
                args,
                argcnt);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNrowColumnType, XmMENU_PULLDOWN); argcnt++;
 XtSetArg(args[argcnt], XmNx, 0); argcnt++;
 XtSetArg(args[argcnt], XmNy, 0); argcnt++;
 new->context.additional_pulldownMenu = 
	XtCreateWidget("additional_pulldownMenu1",
                xmRowColumnWidgetClass,
                new->context.additional_menuShell,
                args,
                argcnt);

/*
 *  Create placeholder buttons for the additional menu.
 */

 for (i = 0; i < (MAX_ELEMENTS - NUM_BUTTONS); i++)
  {
   sprintf(resource_string,"additional_pushbutton%d",i);
   argcnt = 0;
   XtSetArg(args[argcnt], XmNborderWidth,
	new->context.additional_button_border_width); argcnt++;
   XtSetArg(args[argcnt], XmNlabelString, xms); argcnt++;
   XtSetArg(args[argcnt], XmNfontList, new->context.button_font_list); argcnt++;
   new->context.additional_pushButton[i] = 
	XtCreateWidget(resource_string,
                xmPushButtonWidgetClass,
                new->context.additional_pulldownMenu,
                args,
                argcnt);

   XtAddCallback(new->context.additional_pushButton[i],XmNactivateCallback,
        additional_callback,&new->context);
  } 

/*
 *  Always manage the first menu button for size calculations.
 */

 XtManageChild(new->context.additional_pushButton[0]);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNlabelString, (cxms=XmStringCreateLtoR(
                "Additional", XmSTRING_ISO8859_1))); argcnt++;
 XtSetArg(args[argcnt], XmNsubMenuId, new->context.additional_pulldownMenu);
	argcnt++;
 XtSetArg(args[argcnt], XmNfontList, new->context.button_font_list); argcnt++;
 new->context.additional_cascadeButton = 
	XtCreateWidget("additional_cascadeButton",
                xmCascadeButtonWidgetClass,
                new->context.additional_menuBar,
                args,
                argcnt);
 XmStringFree( cxms );
 
 XtManageChild(new->context.additional_cascadeButton);

/*
 *  Retrieve the additional menu dimensions.
 */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNheight, &additional_height); argcnt++;
 XtSetArg(args[argcnt], XmNwidth, &additional_width); argcnt++;
 XtGetValues(new->context.additional_cascadeButton, args, argcnt);

/*
 *  Center the additional menu.
 */

 additional_menu_x = ((frame_width / 2) - (additional_width / 2) + GAP);
 additional_menu_y = (new->core.height - additional_height - GAP);
 argcnt = 0;
 XtSetArg(args[argcnt], XmNx, additional_menu_x); argcnt++;
 XtSetArg(args[argcnt], XmNy, additional_menu_y); argcnt++;
 XtSetValues(new->context.additional_menuBar, args, argcnt);

/*
 *  Create a frame for the mouse button information.
 */

 frame_x = GAP;
 frame_y    = (option_menu_y + option_menu_height + (2 * GAP));
 frame_height = (additional_menu_y - GAP - GAP - frame_y);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNshadowType, XmSHADOW_OUT); argcnt++;
 XtSetArg(args[argcnt], XmNx, frame_x); argcnt++;
 XtSetArg(args[argcnt], XmNy, frame_y); argcnt++;
 new->context.frame = XtCreateWidget("frame",
                xmFrameWidgetClass,
                (Widget)new,
                args,
                argcnt);
 
 XtManageChild(new->context.frame);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNresizePolicy, XmRESIZE_NONE); argcnt++;
 XtSetArg(args[argcnt], XmNx, 0); argcnt++;
 XtSetArg(args[argcnt], XmNy, 0); argcnt++;
 XtSetArg(args[argcnt], XmNheight, frame_height); argcnt++;
 XtSetArg(args[argcnt], XmNwidth, frame_width); argcnt++;
 new->context.frame_bulletinBoard = XtCreateWidget("frame_bulletinBoard",
                xmBulletinBoardWidgetClass,
                new->context.frame,
                args,
                argcnt);
 
 XtManageChild(new->context.frame_bulletinBoard);

/*
 *  Create labels for each mouse button.
 */

/*
 *  Split the frame into sections.
 */

 work_frame_height = (frame_height * (1.0 / (float)NUM_BUTTONS));
 info_area_y = ((work_frame_height / 2) - (info_area_height / 2));
 info_area_x = ((frame_width / 2) - (label_max / 2));

 button_label_x = info_area_x;
 button_label_y = info_area_y;

 for (i = 0; i < NUM_BUTTONS; i++)
  {
   sprintf(resource_string,"button%d_label",i);

   argcnt = 0;
   XtSetArg(args[argcnt], XmNlabelString, xms); argcnt++;
   XtSetArg(args[argcnt], XmNrecomputeSize, True); argcnt++;
   XtSetArg(args[argcnt], XmNx, button_label_x); argcnt++;
   XtSetArg(args[argcnt], XmNy, button_label_y); argcnt++;
   XtSetArg(args[argcnt], XmNfontList, new->context.label_font_list); argcnt++;
   new->context.button_label[i] = XtCreateWidget(resource_string,
                xmLabelWidgetClass,
                new->context.frame_bulletinBoard,
                args,
                argcnt);
 
   XtManageChild(new->context.button_label[i]);
   button_label_y = (button_label_y + work_frame_height);
  }

/*
 *  Create drawingareas for the mouse information pixmaps.
 */

 drawingarea_x  = ((frame_width / 2) - (PIXMAP_WIDTH / 2));
 drawingarea_y = (info_area_y + label_font_height + GAP + GAP);

 for (i = 0; i < NUM_BUTTONS; i++)
  {
   new->context.button_pix[i] = XPM_PIXMAP(XtParent(new), pix_width,pix_height,
        pix_ncolors, pix_chars_per_pixel, pix_colors, pix_pix[i]);
   argcnt = 0;
   XtSetArg(args[argcnt], XmNborderWidth, 
	new->context.drawingarea_border_width); argcnt++;
   XtSetArg(args[argcnt], XmNx, drawingarea_x); argcnt++;
   XtSetArg(args[argcnt], XmNy, drawingarea_y); argcnt++;
   XtSetArg(args[argcnt], XmNwidth, PIXMAP_WIDTH); argcnt++;
   XtSetArg(args[argcnt], XmNheight, PIXMAP_HEIGHT); argcnt++;
   XtSetArg(args[argcnt], XmNbackgroundPixmap, new->context.button_pix[i]);
	argcnt++;
   sprintf(resource_string,"drawingArea%d",i);
   new->context.drawingArea[i] = XtCreateWidget(resource_string,
                xmDrawingAreaWidgetClass,
                new->context.frame_bulletinBoard,
                args,
                argcnt);

   XtManageChild(new->context.drawingArea[i]);
   drawingarea_y = (drawingarea_y + work_frame_height);
  }
XmStringFree(xms);
} /* Initialize */

/*
 *  Destroy() cleans up dynamic data structures.
 */

static void Destroy(w)
XeContextWidget w;
{
 long i;


 for (i = 0; i < NUM_BUTTONS; i++)
  XFreePixmap(XtDisplay(w),w->context.button_pix[i]);

 XtDestroyWidget((Widget)w);

} /* Destroy */

/*
 *  SetValues() handles modifications to resources.
 */
 
static Boolean SetValues(current, request, parent)
Widget  current,
        request,
        parent;
{
 return FALSE;
} /* SetValues -- not needed. */

/*
 *  Support functions.
 */

static void get_font_dimensions(font_struct, font_width, font_height)
XFontStruct *font_struct;
Dimension *font_width,
	  *font_height;
{
 *font_width = font_struct->max_bounds.width;
 *font_height= (font_struct->max_bounds.ascent +
	       font_struct->max_bounds.descent);
}

/*
 *  Public interface.
 */

/*
 *  This module allows a programmer to add a context to the context
 *  widget by use of the Context public structure.
 */

void XeContextAdd(context_widget, context_struct)
XeContextWidget context_widget;
Context context_struct;
{
Arg args[2];			/* Xt resource argument list.		*/

int argcnt;			/* Argument list count.			*/

				/* Convenience pointer.			*/
Context_Internal *internal = NULL;

long i;				/* Loop index variable.			*/

/*
 *  Check if room is available for the new context.
 */

 if (context_widget->context.num_contexts == MAX_CONTEXTS)
  {
   fprintf(stderr,"Error adding context %s: table full\n",
	context_struct.context_name);
   return;
  }

/*
 *  Allocate a new internal context structure.
 */

 internal = (Context_Internal *)malloc(sizeof(Context_Internal));
 if (internal == NULL)
  {
   fprintf(stderr,"Error allocating memory for context %s\n",
		context_struct.context_name);
   exit(-1);
  }

/*
 *  Now fill in the private structure with values from the public.
 */

 strcpy(internal->context_token,context_struct.context_name);

 internal->context_name = 
	XmStringCreateLtoR(context_struct.context_name,XmSTRING_ISO8859_1);

 internal->num_elements = context_struct.num_elements;

 for (i = 0; i < internal->num_elements; i++)
  {
   internal->element_names[i] = 
	XmStringCreateLtoR(context_struct.element_names[i],
                           XmSTRING_ISO8859_1);

   internal->client_data[i] = context_struct.client_data[i];
   internal->element_handlers[i] = context_struct.element_handlers[i];
  }

 internal->event_handler = context_struct.event_handler;
 internal->nonmaskable   = context_struct.nonmaskable;
 internal->event_mask    = context_struct.event_mask;

 if (context_struct.num_widget_ids < 1)
   fprintf(stderr,"Warning!  Context widget has no event widget\n");

 if (context_struct.num_widget_ids > MAX_WIDGETS)
  {
   fprintf(stderr,"Warning!  Context widget has more than %d event widgets\n",
	MAX_WIDGETS);
   internal->num_widget_ids = MAX_WIDGETS;
  }
 else
  {
   internal->num_widget_ids = context_struct.num_widget_ids;
  }

 for (i = 0; i < internal->num_widget_ids; i++)
  internal->event_widgets[i] = context_struct.event_widgets[i];

 internal->activate_callback = context_struct.activate_callback;
 internal->active_client_data = context_struct.active_client_data;

/*
 *  Now assign the new internal context structure to the instance record.
 */

 context_widget->context.context_list[context_widget->context.num_contexts] =
	internal;

/*
 *  Assign a pushbutton for the new context.
 */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNborderWidth, 
	context_widget->context.option_button_border_width); argcnt++;
 XtSetArg(args[argcnt], XmNlabelString, internal->context_name); argcnt++;
 XtSetValues(context_widget->
	context.option_pushButton[context_widget->context.num_contexts],
	args, argcnt);

 XtManageChild(context_widget->
	context.option_pushButton[context_widget->context.num_contexts]);

/*
 *  Now increment the number of contexts variable.
 */

 context_widget->context.num_contexts++;
}

/*
 *  This module allows a programmer to set the active context from the
 *  context list.  This allows programmatic control in addition to the
 *  use of the option menu by the user.
 */

void XeContextSetActive(context_widget, context_name)
XeContextWidget context_widget;
char *context_name;
{
Arg args[20];			/* Xt resource argument list.		*/

int argcnt;			/* Resource argument count.		*/

Context_Internal *internal;	/* Convenience pointer.			*/

long i,				/* Loop indices.			*/
     j,
     num_elements,
     num_blank,
     current_context;		/* Working context index.		*/

Boolean found = FALSE;		/* Flag for context search.		*/

Widget gadget_button;		/* Gadget id for the option menu.	*/

XmString xms;			/* Working Motif string.		*/


/*
 *  Search the context list for the requested member.
 */

 for (i = 0; i < context_widget->context.num_contexts; i++)
  {
   if ((strcmp(context_name,
	context_widget->context.context_list[i]->context_token)) == 0)
    {
     /*
      * Assign a working pointer for convenience.
      */

      internal = context_widget->context.context_list[i];

     /*
      * Set the current context index and assign the new one.
      */

      current_context = context_widget->context.current_context;
      context_widget->context.current_context = i;

     /*
      *  Change the option menu to reflect the active context.
      */

      gadget_button = XmOptionButtonGadget(context_widget->context.optionMenu);

      argcnt = 0;
      XtSetArg(args[argcnt], XmNlabelString, internal->context_name); argcnt++;
      XtSetValues(gadget_button, args, argcnt);

     /*
      *  Unmanage the additional menu buttons.
      */

      for (j = 0; j < (MAX_ELEMENTS - NUM_BUTTONS); j++)
	XtUnmanageChild(context_widget->context.additional_pushButton[j]);

     /*
      *  Change the button labels and additional menu buttons
      *  for the active context.
      */

      num_elements = internal->num_elements;
      for (j = 0; j < num_elements; j++)
	{
	 if (j < NUM_BUTTONS)	/* Mouse buttons. */
	  {
           argcnt = 0;
           XtSetArg(args[argcnt], XmNlabelString, internal->element_names[j]);
              argcnt++;
           XtSetValues(context_widget->context.button_label[j],args,argcnt);
	  }
	 else			/* Additional menu items. */
	  {
	   argcnt = 0;
	   XtSetArg(args[argcnt], XmNlabelString,internal->element_names[j]);
		argcnt++;
	   XtSetValues(context_widget->context.additional_pushButton[(j -
			NUM_BUTTONS)], args, argcnt);
	   XtManageChild(context_widget->context.additional_pushButton[(j -
			NUM_BUTTONS)]);
	  }
	}

     /*
      *  If the number of elements is less than the number of mouse buttons
      *  then blank any remaining labels.
      */

      if (num_elements < NUM_BUTTONS)
       {
	xms = XmStringCreateLtoR("",XmSTRING_ISO8859_1);
        num_blank = (NUM_BUTTONS - num_elements);
        while (num_blank > 0)
	 {
          argcnt = 0;
          XtSetArg(args[argcnt], XmNlabelString, xms); argcnt++;

	  XtSetValues(context_widget->context.button_label[(NUM_BUTTONS - 
			num_blank)],args,argcnt);

          num_blank--;
         }
        XmStringFree(xms);
       }

     /*
      *  If no additional menu items are needed then make the additional
      *  menu insensitive.
      */

      if (num_elements <= NUM_BUTTONS)
        XtSetSensitive(context_widget->context.additional_menuBar, False);
      else
	XtSetSensitive(context_widget->context.additional_menuBar, True);

     /*
      *  Now assign an eventhandler for the context.  If a specific one has
      *  not been specified then use a default.
      */

      if (context_widget->context.current_eventhandler != NULL)
       {
        for (j = 0; j < 
	context_widget->context.context_list[current_context]->num_widget_ids;
	 j++)
	 {
          XtRemoveEventHandler(context_widget->context.context_list[
		current_context]->event_widgets[j],
	   context_widget->context.current_eventmask,
	   context_widget->context.current_nonmaskable,
	   context_widget->context.current_eventhandler,
	   context_widget->context.current_client);
         }
       }

      if (context_widget->context.context_list[i]->event_handler)
       {
	context_widget->context.current_eventhandler = 
		context_widget->context.context_list[i]->element_handlers[0];
	context_widget->context.current_eventmask =
		context_widget->context.context_list[i]->event_mask;
	context_widget->context.current_nonmaskable =
		context_widget->context.context_list[i]->nonmaskable;
	context_widget->context.current_client = 
		context_widget->context.context_list[i]->client_data[0];

	for (j = 0; j < 
		context_widget->context.context_list[i]->num_widget_ids; j++)
	 {
	  XtAddEventHandler(
	  context_widget->context.context_list[i]->event_widgets[j],
	  context_widget->context.current_eventmask,
	  context_widget->context.current_nonmaskable,
	  context_widget->context.current_eventhandler,
	  context_widget->context.current_client);
	 }
       }
      else
       {
	context_widget->context.current_eventhandler = 
	    (XtEventHandler)context_eventhandler;
	context_widget->context.current_eventmask    = EVENT_MASK;
	context_widget->context.current_nonmaskable  = False;
	context_widget->context.current_client       = 
					(XtPointer)&context_widget->context;

        for (j = 0; j <
                context_widget->context.context_list[i]->num_widget_ids; j++)
         {
          XtAddEventHandler(
          context_widget->context.context_list[i]->event_widgets[j],
          context_widget->context.current_eventmask,
          context_widget->context.current_nonmaskable,
          context_widget->context.current_eventhandler,
          context_widget->context.current_client);
         }
       }
      found = TRUE;

      /*
       *  If an active callback is set then execute it.
       */

       if (context_widget->context.context_list[i]->activate_callback != NULL)
	(*context_widget->context.context_list[i]->activate_callback)
	(context_widget,
	  context_widget->context.context_list[i]->active_client_data, NULL);

      break;
    }
  }

 if (!found)
  {
   fprintf(stderr,"Error setting context %s: context not found\n",context_name);
  }
}

/*
 *  This module allows the programmer to remove a context from the context
 *  list.  If the target context is also the currently active context then
 *  the active context is set to the first context in the list.
 */

void XeContextRemove(context_widget, context_name)
XeContextWidget context_widget;
char *context_name;
{
Context_Internal *context_internal;	/* Convenience pointer.		*/

Widget move_id;				/* Widget id to be removed.	*/

long i,					/* Loop indices.		*/
     j;

Boolean found = FALSE;			/* Error check flag.		*/


/*
 *  Check if only one context exists -- if it does then output an error.
 */

 if (context_widget->context.num_contexts == 1)
  {
   fprintf(stderr,"Error removing context %s: empty list\n",context_name);
   return;
  }

 /*
  *  Search the context list for the requested member.
  */

  for (i = 0; i < context_widget->context.num_contexts; i++)
   {
    if ((strcmp(context_name,
        context_widget->context.context_list[i]->context_token)) == 0)
     {
      /*
       *  If the member is currently the active context then revert to
       *  the first context in the list by default.
       */

       if (context_widget->context.current_context == i)
	XeContextSetActive(context_widget,
		context_widget->context.context_list[0]->context_token);
      /*
       *  If the currently active context follows the context being
       *  deleted, the index of the currently active context needs to
       *  be adjusted to reflect the removal of the context being
       *  deleted.
       */

       if (context_widget->context.current_context > i)
          context_widget->context.current_context--;

      /*
       *  Remove the member from the list and compress the list.
       *  Also, remove the member from the option menu list.
       */

       context_internal = context_widget->context.context_list[i];

       XmStringFree(context_internal->context_name);
       for (j = 0; j < context_internal->num_elements; j++)
		XmStringFree(context_internal->element_names[j]);

       free(context_internal);
       context_widget->context.context_list[i] = NULL;

       XtUnmanageChild(context_widget->context.option_pushButton[i]);

       move_id = context_widget->context.option_pushButton[i];
       for (j = i; j < MAX_CONTEXTS; j++)
        {
         if (j == (MAX_CONTEXTS - 1))
          {
	   context_widget->context.option_pushButton[j] = move_id;

           context_widget->context.context_list[j] = NULL;
	  }
         else
	  {
           context_widget->context.option_pushButton[j] =
		context_widget->context.option_pushButton[j+1];

	   context_widget->context.context_list[j] = 
		context_widget->context.context_list[j+1];
   	  }
        }

       context_widget->context.num_contexts--;

      /*
       *  Set the error check flag to true.
       */

       found = TRUE;
       break;
     }
    }

  /*
   *  Check if an incorrect context was specified.
   */

   if (!found)
     fprintf(stderr,"Error removing context %s: context not found\n",
	context_name);
}

/*
 *  This module returns the currently active context.
 */
 
void XeContextGetActive(context_widget, context_name)
XeContextWidget context_widget;
char *context_name;
{
/*
 *  Copy the currently active context name into the input string.
 */
 
 strcpy(context_name, context_widget->context.context_list[
        context_widget->context.current_context]->context_token);
 
}

/*
 *  This module performs a search on the context list for the specified
 *  context and returns a boolean as a result:  TRUE = found.
 */
 
Boolean XeContextCheckList(context_widget, context_name)
XeContextWidget context_widget;
char *context_name;
{
long i;         /* Loop iterator.                               */
 
/*
 *  Search the context list for the requested context's existence.
 */
 
  for (i = 0; i < context_widget->context.num_contexts; i++)
   {
    if ((strcmp(context_name,
        context_widget->context.context_list[i]->context_token)) == 0)
        return(TRUE);
   }
return(FALSE);
}

/*
 *  This module performs a search on the context list for the specified
 *  context and desensitizes the corresponding option pushbutton.
 */

void XeContextDesensitize(context_widget, context_name)
XeContextWidget context_widget;
char *context_name;
{
long i;		/* Loop iterator.			*/

/*
 *  Search the context list for the specified context.
 */

  for (i = 0; i < context_widget->context.num_contexts; i++)
   {
    /*
     *  If the context is found then set its option button insensitive.
     */

     if ((strcmp(context_name,
        context_widget->context.context_list[i]->context_token)) == 0)
      {
 	XtSetSensitive(context_widget->context.option_pushButton[i], False);
        break;
      }
   }

/*
 *  Check if a bad context name was specified.
 */

 if (i == context_widget->context.num_contexts)
  fprintf(stderr, "Context: Bad context name specified!\n");

return;
}

/*
 *  This module performs a search on the context list for the specified
 *  context and sensitizes the corresponding option pushbutton.
 */

void XeContextSensitize(context_widget, context_name)
XeContextWidget context_widget;
char *context_name;
{
long i;		/* Loop iterator.				*/

/*
 *  Search the context list for the specified context.
 */

  for (i = 0; i < context_widget->context.num_contexts; i++)
   {
    /*
     *  If the context is found then set its option button sensitive.
     */

     if ((strcmp(context_name,
        context_widget->context.context_list[i]->context_token)) == 0)
      {
 	XtSetSensitive(context_widget->context.option_pushButton[i], True);
        break;
      }
   }

/*
 *  Check if a bad context name was specified.
 */

 if (i == context_widget->context.num_contexts)
  fprintf(stderr, "Context: Bad context name specified!\n");

return;
}


