/*
 *   Context.h
 */

/*
 *  Public declarations/definitions for Context.c
 */

#ifndef _XeContext_h
#define _XeContext_h

/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or 
 * publicity pertaining to distribution of the software without specific, 
 * written prior permission.  USGS makes no representations about the 
 * suitability of this software for any purpose.  It is provided "as is" 
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS 
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *  Change log:
 *
 *	D. Akkerman, USGS/EROS Data Center, 7/92
 *	 - inception.
 *
 */

#include <X11/Shell.h>
#include <X11/Xmu/Converters.h>

/*
 *  This module implements a context object for the DLG-E processor.  The
 *  context object stores, manages and dispatches the current mouse button
 *  context.  In addition, it provides visual information to the user for
 *  the function of each mouse button in the given context and provides for
 *  the switching of contexts as well as additional functions not available on
 *  mouse buttons but available within a given context.  Finally, it also 
 *  provides public functions for the setting, adding, removing of contexts
 *  as well as the callback hooks used to implement a given context.  
 *  Currently, a three-button mouse is assumed.
 *
 *  The context object itself is constructed of a Motif widget tree and is
 *  a subclass of the BulletinBoard manager widget.
 *
 */

/*
 *  Resources:
 *
 *     Name	      Class	   Data Type	  Default	Access
 *
 *  XeNlabelFont   XeCLabelFont   XFontStruct*  XtDefaultFont	  C
 *  XeNbuttonFont  XeCButtonFont  XFontStruct*	XtDefaultFont	  C
 *  XeNvisual      XeCVisual      Visual*	CopyFromParent    C
 *  XeNoptionButtonBorderWidth
 *		   XeCOptionButtonBorderWidth
 *				  Dimension  	      0  	  C
 *  XeNoptionMenuBorderWidth
 *		   XeCOptionMenuBorderWidth
 *				  Dimension	      0		  C
 *  XeNdrawingAreaBorderWidth
 *		   XeCDrawingAreaBorderWidth
 *				  Dimension	      1		  C
 *  XeNadditionalButtonBorderWidth
 *		   XeCAdditionalButtonBorderWidth
 *				  Dimension	      0		  C
 *  XeNadditionalMenuBorderWidth
 *		   XeCAdditionalMenuBorderWidth	      0		  C
 *
 */

/*
 *  Public interfaces.
 */

/*
 *  Resource definitions.
 */

#define XeNlabelFont			"labelFont"
#define XeNbuttonFont			"buttonFont"
#define XeNvisual			XtNvisual
#define XeNoptionButtonBorderWidth	"optionButtonBorderWidth"
#define XeNoptionMenuBorderWidth	"optionMenuBorderWidth"
#define XeNdrawingAreaBorderWidth	"drawingAreaBorderWidth"
#define XeNadditionalButtonBorderWidth	"additionalButtonBorderWidth"
#define XeNadditionalMenuBorderWidth	"additionalMenuBorderWidth"

#define XeCLabelFont			"LabelFont"
#define XeCButtonFont			"ButtonFont"
#define XeCVisual			XtCVisual
#define XeCOptionButtonBorderWidth	"OptionButtonBorderWidth"
#define XeCOptionMenuBorderWidth	"OptionMenuBorderWidth"
#define XeCDrawingAreaBorderWidth	"DrawingAreaBorderWidth"
#define XeCAdditionalButtonBorderWidth	"AdditionalButtonBorderWidth"
#define XeCAdditionalMenuBorderWidth	"AdditionalMenuBorderWidth"

typedef struct _XeContextClassRec *XeContextWidgetClass;
typedef struct _XeContextRec *XeContextWidget;

extern WidgetClass xeContextWidgetClass;

/*
 *  Public constants.
 */

#define ELEMENT_MAX_LENGTH	16	/* Maximum element name (15 chars.)  */
#define NUM_BUTTONS		3	/* A 3-button mouse (see NOTE below).*/
#define MAX_ELEMENTS		10	/* Maximum # of elements per context.*/
#define MAX_CONTEXTS		10	/* Maximum # of contexts.	     */
#define MAX_WIDGETS		10	/* Maximum # of input widget Ids.    */

/*
 * NOTE:  A three-button mouse is assumed within.  If a different sort of
 *        mouse needs to be supported it will require the adding of additional
 *        pixmaps into the ContextI.h file and possibly a change to the pixmap
 *	  size in both ContextI.h and Context.c.  This approach is sufficient
 *	  for now. -- D. Akkerman
 */

/*
 *  Public structures.
 */

typedef struct _Context
 {
					/* Context and element names.	     */
  char context_name[ELEMENT_MAX_LENGTH],
       element_names[MAX_ELEMENTS][ELEMENT_MAX_LENGTH];

  long num_elements,			/* Total # of elements.		     */
       num_widget_ids;			/* Total # of input widgets.	     */

  Widget event_widgets[MAX_WIDGETS];	/* Event widget IDs.		     */

  Boolean event_handler;		/* If TRUE then first element_handler
					   is an eventhandler for the mouse. */

  Boolean nonmaskable;			/* Call eventhandler on nonmaskable
					   events? (almost always FALSE)     */

  EventMask event_mask;			/* Event mask for eventhandler.	     */

					/* Pointers to callbacks for elements.
					   If event_handler is TRUE then the
					   first function pointer must be to
					   the eventhandler for the mouse.   */
  void (*element_handlers[MAX_ELEMENTS])();

  XtPointer client_data[MAX_ELEMENTS];	/* Argument to pass to callback.     */

  void (*activate_callback)();		/* Callback when context is
					   activated.			     */

  XtPointer active_client_data;		/* Client data for active callback.  */
 } Context;


/*
 *  Public functions.
 */

extern void XeContextSetActive();
/*
 *  XeContextWidget context_widget;
 *  char *context_name;
 */

extern void XeContextAdd();
/*
 *  XeContextWidget context_widget;
 *  Context context_struct;
 */

extern void XeContextRemove();
/*
 *  XeContextWidget context_widget;
 *  char *context_name;
 */

extern void XeContextGetActive();
/*
 *  XeContextWidget context_widget;
 *  char *context_name;
 */

extern Boolean XeContextCheckList();
/*
 *  XeContextWidget context_widget;
 *  char *context_name;
 */

extern void XeContextDesensitize();
/*
 *  XeContextWidget context_widget;
 *  char *context_name;
 */

extern void XeContextSensitize();
/*
 *  XeContextWidget context_widget;
 *  char *context_name;
 */
#endif /* _XeContext_h */

