/*
**********************************************************************

NAME: ALBEDO

PURPOSE: Computes the albedo of a landsat image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0        06/15/84   K. Gacke        
   2.0        07/28/89   K. A. Narayanan    conversion to 5.0
   3.0 	      09/21/92	 T. Mittan	    Incoporated global .h file to
					    eliminate warning messages on 
					    link.
   3.1        03/04/93   D. Etrheim         Corrected windowing problem

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get and validate parameters and allocate
      dynamic storage
   Call FILSPEC to set up file specifications
   Call GETLUT to create radiance table
   Call APPLYLUT to apply radiance table and generate output image
   Issue completion message

ALGORITHM REFERENCE:
   Charles J. Robinave, "Computation with Physical Values from Landsat 
   Digital Data," Photogrammetric Engineering and Remote Sensing, Vol. 48, 
   No. 5, 1982, pp. 781-784.

**********************************************************************
*/

#include <stdio.h>

#include "las.h"
#include "util.h"
#include "protos.h"

#define MAXCH   4

struct PARBLK block;             /* TAE parameter block     		*/
float   *iobuf;                  /* image i/o buffer        		*/

struct FDESC * fdesc[3];                /* file descriptors        		*/
struct GDESC * gdesc;                   /* group descriptor        		*/
long    ipoint[MAXCH + 1],       /* offsets into image i/o  		*/
                opoint;          /* for input and output bands 		*/
long    min_max;                 /* minmax option flag      		*/
long    msgtim;                  /* message interval        		*/
long    fatal_err = -1;
long    inf_err = 1;

char    *prog_name = "ALBEDO";  /* program name            		*/

void main
(void) 
{
float	lookup[MAXCH][256];	 /* radiance lookup table	   	*/
float	sunangle;		 /* sun angle			    	*/
	
extern long    curcat;           /* parameters for processing 		*/
extern long    curent;           /* message utility         		*/
long	band_num[1][MAXBND + 1]; /* image band numbers	      	*/
long	dark[MAXCH];		 /* dark values			     	*/
long	num_inbands;             /* number of input image bands      	*/
long	one = 1;
long	satellite_code;	         /* satellite identification 	     	*/
long	wind[1][4];              /* input image windows              	*/

char	hos_in[1][CMLEN];         /* input image host name            	*/
char	hos_out[1][CMLEN];        /* output image host name           	*/
char	las_out[1][CMLEN];       /* output image las name            	*/

/*
get user parameters 
-------------------
*/
getpar (hos_in, las_out, hos_out, band_num, &num_inbands, 
        wind, &satellite_code, &sunangle, dark);

/*
open image files
----------------
*/
filspec (hos_in, hos_out, &num_inbands, band_num, wind);

/*
compute radiance tables
-----------------------
*/
getlut (lookup, dark, sunangle, satellite_code, num_inbands, band_num[0]);

/*
generate output image using radiance tables
-------------------------------------------
*/
applylut (hos_in, hos_out[0], num_inbands, band_num[0], wind[0], lookup);

/*
write completion message
------------------------
*/
c_complt ("ALBEDO", las_out, &one); 
}
