/*********************************************************************
NAME: ARC2LAS

PURPOSE: To take the four files generated by GridImage (or GridFloat)
         in ARC with the bil option and to generate a LAS .DDR and .HIS
         file.  If the input grid coverage is 1-bit, then a LAS .IMG
         file will be generated also.
 
PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  --------------   ----  ------ 
  1.0    08/92  K. Larson         C    Initial development
  1.1    12/92  K. Larson         C    Set pixel size value
  1.2    01/93  K. Larson         C    

   Fix error with pixel size value and allow invalid information to
   be in the *.stx file and continue processing.  The invalid flag
   value will be set for each band that has invalid information in
   the *.stx file

   2.0   02/94  K. Lowell         C    Conversion to LAS6.0
                                       standards
   2.1   09/94  K. Lowell         C    Upgraded to ARC7.0.  Incorporated
                                       changes made by R.White/PSU 
   2.2   9/96   D.Etrheim	       Raised buffers in bit2byte MAXNS

$Log: arc2las.c,v $
Revision 1.20  2002/11/04 18:16:09  shimitz
Merge development branch for CRs 5170, 5174, & 5792 with the trunk.
5170: Enhance file clean-up to remove the "temptn" files generated during
      processing.
5174: Enhance to support floating-point grids.
5792: Allow the user to enter the ingrid in any case (upper, lower, mixed) and
      still read the lower-case grid from disk.

Revision 1.19.2.7  2002/09/19 20:33:47  shimitz
Added call to c_lused in the floating-point code.
Modified the explanation of the program in the PDF file and added a user
note about the corner point precision.

Revision 1.19.2.6  2002/04/25 21:03:03  shimitz
Modifications applied after code review.

Revision 1.19.2.5  2002/04/23 19:23:10  shimitz
Added support for non-byte grid stacks. The need for these changes were
identified when attempting to process a floating point grid stack. The problem
was found that gridfloat will not process a grid stack, as gridimage will.
The solution is to process each of the grids that compose the stack individually
and then combine them with COPY after each BIL file gets converted to a LAS
image with TRANSLAS. CR 5174

Revision 1.19.2.4  2002/04/11 16:32:18  shimitz
Modified to create unique temp file name used to retrieve info from ARCInfo.

Revision 1.19.2.3  2002/02/01 20:24:14  shimitz
Minor changes to clean up compiler warnings.

Revision 1.19.2.2  2002/01/30 15:47:07  shimitz
Updated arc2las to properly convert floating-point arc grids and integer grids
with pixel values exceeding the byte range (0-255) to LAS imagery by using the
arc command gridfloat. CR 5174


************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "labserv.h"
#include "vll.h"
#include "arc2las.h"

#define STXLEN  448     /* max length of readin buffer for stx file */

struct PARBLK vblk;     /*TAE parameter block     */

char imagename[1][CMLEN];   /* Name of the output image. */
char tmp_imagename[CMLEN];  /* Copy of imagename without any extension */
char ddrname[CMLEN];        /* Name of the output image .ddr. */
char tmp_ddrname[CMLEN];    /* Name of the output image .ddr for grid stacks. */
char hisname[CMLEN];        /* Name of the output image .his. */
char tmp_hisname[CMLEN];    /* Name of the output image .his for grid stacks. */
char gridstx[CMLEN];        /* Name of the .stx file. */
char gridbilw[CMLEN];       /* Name of the .blw file. */
char gridbilw2[CMLEN];      /* Name of the other .blw file. */
char gridhdr[CMLEN];        /* Name of the .hdr file. */
char gridbil[CMLEN];        /* Name of the .bil file. */
char gridprj[CMLEN];        /* Name of the .prj file */
char hosout[CMLEN];         /* Output root name      */
char workspace[CMLEN];
char geotype[CMLEN];        /* geo dataset type (grid, stack, ...) */
char ingrid[CMLEN];         /* Name of the input ARC grid */
char layernum[CMLEN]   ;    /* Layer number to append to DDR filename */

FILE *hisout;             /* Output image .his file pointer. */
FILE *hdrin;              /* .hdr file pointer. */
FILE *bilin;              /* .bil file pointer. */
FILE *stxin;              /* .stx file pointer. */
FILE *bilwin;             /* .blw file pointer. */

void main
(
  void
)
{
    char readin[STXLEN];       /* String used for reading in values.         */
    char hostin[1][CMLEN];     /* Array for hstout                           */
    char text[CMLEN];          /* Text string for hstout                     */
    char proc_num[CMLEN];      /* Current number being extraced from 
                                  input string.                              */
    char *error_ptr;           /* Points to an invalid character when
                                  converting a string to a number.           */
    char byteorder[5];         /* Contains the byte order of the input bil.  */
    char layout[5];            /* Indicates the image format of the Arc
                                  output, Assumed to be BFORM, because this
                                  will be the name extension that will be
                                  looking for.                               */
    char systrg[CMLEN];        /* System type                                */
    char progname[CMLEN];      /* Program name                               */

    int nbands;                /* number of bands in image                   */
    int dtype;                 /* data type of output LAS image              */
    int nodata_val;            /* nodata filler pixel value                  */
    int i, j;                  /* general looping counters                   */

    long length;               /* String lengths */
    long outcount = 1;         /* Output file count                          */
    long index;                /* Used for indexing readin.                  */
    long loop,loop2;           /* Loop counters                              */
    long status;               /* Status indicator                           */
    long key = DDSYS;          /* DDR field key                              */
    long lvalue;               /* Long value retrieved from hdr              */
    long tmp_long;             /* dummy long to obtain lng value             */
    long std_out;              /* logical unit number for standard output */
    long ep3flag;              /* 8.3 naming convention flag
                                    0 - 8.3 naming convention in use
                                    1 - NO8.3 naming convention in use */
    long zero = 0;             /* Constant of zero                           */
    long ii;                   /* Counter                                    */

    long nbits = -1;           /* # of bits in # from input .bil. defaulted
                                  to -1 since never referenced if gridfloat
                                  was used to create the BIL                 */
    long bandbytes;            /* # of bytes per line of input .bil.         */
    long totalbytes;           /* # of bytes per band of input .bil.         */
    long msgtim;               /* message processing time */

    long validflag;            /* used when determining min and max */

    double curmin;            /* current minimum pixel value */
    double curmax;            /* current maximum pixel value */
    double minmin;            /* smallest minimum of all bands               */
    double maxmax;            /* largest maximum of all bands                */
    double cellsize;          /* pixel size as retrieved from .hdr file      */
    double b_rotation = 0;    /* Rotation of image to world transformation.  */
    double d_rotation = 0;    /* Rotation of image to world transformation.  */
    double xdim;              /* Size in the x dimension of the cell.        */
    double ydim;              /* Size in the y dimension of the cell.        */
    double ulxmap;            /* Upper left cell, center coordinate for x.   */
    double ulymap;            /* Upper left cell, center coordinate for y.   */
    double xllcorner;         /* Lower left cell, center coordinate for x.   */
    double yllcorner;         /* Lower left cell, center coordinate for y.   */
    double tmp;               /* temporarily holds number of bands */

    /* STRUCT THAT CONTAINS ALL THE PARAMETERS IN
       THE FIRST 2 RECORDS OF THE DDR */
    struct DDR  ddrstuff;

    /* STRUCT THAT CONTAINS THE PARAMETERS FOR
       THIRD OR MORE RECORDS IN THE DDR */
    struct BDDR bddrstuff;

    c_inlas (&vblk, &std_out);

    /* GET THE VARIABLES FROM THE TAE VARIABLE BLOCK
       ---------------------------------------------*/
    getpar(&vblk,hosout,&msgtim,&ep3flag,workspace,geotype,ingrid,&dtype,
           &nbands);

    length = (long)strlen(hosout);
    if (strstr(hosout,".img") == NULL)
        strcpy(imagename[0],hosout);
    else
    {
        strncpy(imagename[0],hosout,(length-4));
        imagename[0][(length-3)]=0;
        strcpy(hosout,imagename[0]);
    }

    strcpy(tmp_imagename, imagename[0]);
    c_lsmknm(imagename[0],"ddr",ddrname);
    c_lsmknm(imagename[0],"his",hisname);
    c_lsmknm(imagename[0],"img",imagename[0]);

    if (dtype == EBYTE)
    {
        if (ep3flag == 1)
        {
            c_lsmknm(hosout,"hdr",gridhdr);
            c_lsmknm(hosout,"stx",gridstx);
            c_lsmknm(hosout,"blw",gridbilw);
            c_lsmknm(hosout,"bilw",gridbilw2);
            c_lsmknm(hosout,"prj",gridprj);
            c_lsmknm(hosout,"bil",gridbil);
        }
        else
        {
            c_lsmknm(hosout,"hdr",gridhdr);
            c_lsmknm(hosout,"stx",gridstx);
            c_lsmknm(hosout,"bilw",gridbilw);
            c_lsmknm(hosout,"blw",gridbilw2);
            c_lsmknm(hosout,"proj",gridprj);
            c_lsmknm(hosout,"bil",gridbil);
        }
    }
    else
    {
        if(strcmp(geotype, "Stack") == 0)
        {
            /* Just use the header and projection information from
               the first layer. Note that arc2las will only be able to
               properly process grid stacks whose layers all share the
               same corner points and are the same resolution. */
            strcat(hosout, "_1");
        }
        c_lsmknm(hosout,"hdr",gridhdr);
        c_lsmknm(hosout,"prj",gridprj);
    }

    /* CHECKING IF FILES ALREADY EXIST
       -------------------------------*/
    /* Checking if .ddr already exists. */
    if ((c_lsstat(ddrname,&zero)) == E_SUCC)
        c_lasmsg(TRUE, "arc2las-exist", "DDR file already exists");

    /* Checking if .his already exists. */
    if ((c_lsstat(hisname,&zero)) == E_SUCC)
    {
        fclose(hisout);
        c_lasmsg(TRUE, "arc2las-exist", "HIS file already exists");
    }

    /* INITIALIZING DDR STRUCTURE.  SET INVALID FLAGS
       ----------------------------------------------*/
    c_intddr(&ddrstuff);
    ddrstuff.line_inc = 1.0;
    ddrstuff.sample_inc = 1.0;

    for (ii = 0; ii < 8; ii++)
        ddrstuff.valid[ii] = INVAL;

    /* OPENING THE .HDR */
    if ((hdrin = fopen(gridhdr,"r")) == NULL)
        c_lasmsg(TRUE, "arc2las-open", "Error opening .hdr");

    /* READING IN ALL OF THE .HDR INFORMATION */
    if (dtype == EBYTE) /* gridimage used to create BIL file */
    {
        if ((fscanf(hdrin," BYTEORDER %c LAYOUT %s NROWS %d NCOLS %d",
                    byteorder, layout, &ddrstuff.nl, &ddrstuff.ns) < 4)
            || (fscanf(hdrin,
                       " NBANDS %d NBITS %d BANDROWBYTES %d TOTALROWBYTES %d",
                       &ddrstuff.nbands, &nbits, &bandbytes, &totalbytes) < 4))
        {
            fclose(hdrin);
            c_lasmsg(TRUE, "arc2las-read", "Error reading .hdr file");
        }
    }
    else /* gridfloat used to create BIL file */
    {
        if ((fscanf(hdrin," ncols %d nrows %d xllcorner %lf yllcorner %lf",
                    &ddrstuff.ns, &ddrstuff.nl, &xllcorner, &yllcorner) < 4)
            || (fscanf (hdrin," cellsize %lf NODATA_value %d byteorder %c",
                        &cellsize, &nodata_val, byteorder) < 3))
        {
            fclose(hdrin);
            c_lasmsg(TRUE, "arc2las-read", "Error reading .hdr file");
        }
        strcpy(layout, "BIL");
        ddrstuff.nbands = nbands;
    }
    lvalue = ddrstuff.nl;
    if (E_SUCC != lng2tae(&lvalue,1,1,"NL",&tmp_long))
        c_lasmsg(TRUE, "arc2las-qout", "Error passing NL variable to TAE");
    lvalue = ddrstuff.ns;
    if (E_SUCC != lng2tae(&lvalue,1,1,"NS",&tmp_long))
        c_lasmsg(TRUE, "arc2las-qout", "Error passing NS variable to TAE");
    lvalue = ddrstuff.nbands;
    if (E_SUCC != lng2tae(&lvalue,1,1,"NBANDS",&tmp_long))
        c_lasmsg(TRUE, "arc2las-qout", "Error passing NBANDS variable to TAE");
    lvalue = nbits;
    if (E_SUCC != lng2tae(&lvalue,1,1,"NBITS",&tmp_long))
        c_lasmsg(TRUE, "arc2las-qout", "Error passing NBITS variable to TAE");
    lvalue = dtype;
    if (E_SUCC != lng2tae(&lvalue,1,1,"DTYPE",&tmp_long))
        c_lasmsg(TRUE, "arc2las-qout",
                 "Error passing DTYPE variable to TAE");

    if (q_out(&vblk) != SUCCESS)
        c_lasmsg(TRUE, "arc2las-qout", "Error passing variables to TAE");

    fclose(hdrin);

    /*  Process stx and bilw files - only do this if BYTE data (BIL file
        was created using gridimage)
        ----------------------------------------------------------------*/
    if (dtype == EBYTE)
    {
        /*  RETRIEVING THE MIN AND MAX FROM STX
            -----------------------------------*/
        if ((stxin = fopen(gridstx,"r")) == NULL)
            c_lasmsg(TRUE, "arc2las-open", "Error opening .stx");
        for (loop = 1; loop <= ddrstuff.nbands; loop++)
        {
            /* SETTING THE READIN'S INDEXING VARIABLE TO 0 */
            index = 0;

            /* READING IN THE BAND INFORMATION */
            if (fgets(readin,STXLEN,stxin) == NULL)
            {
                fclose(stxin);
                c_lasmsg(TRUE, "arc2las-read", "Error reading .stx file");
            }

            length = (long)strlen(readin);
            error_ptr = NULL;

            /* ASSUMING THE READIN PARAMETERS ARE VALID */
            validflag = VALID;

            /* GETTING THE BAND NUMBER, MINIMUM AND MAXIMUM VALUES */
            for (loop2 = 0; loop2 < 3 && validflag == VALID; loop2++)
            {
                strip_num(readin,proc_num,&index);
                switch (loop2)
                {
                    case 0:
                        break;
                    case 1: curmin = strtod(proc_num,&error_ptr);
                            break;
                    case 2: curmax = strtod(proc_num,&error_ptr);
                            break;
                    default:
                            fclose(stxin);
                            c_lasmsg(TRUE, "arc2las-proc",
                                     "Error processing .stx file");
                }

                /* NEXT TO STATEMENT, JUST SETTING THE BAND NUMBER IF AN
                   ERROR OCCURS AND SETTING THE BAND FLAG TO INVALID */
                if (((error_ptr != NULL)&&((*error_ptr) != 0)) ||
                    (strlen(proc_num)==0))
                    validflag = INVAL;

                /* MAKING SURE INDEX IS NOT LARGER THAN THE LENGTH OF READIN */
                if (length < index)
                    validflag = INVAL;

                if (nbits == 1)
                {
                    curmin=0;
                    curmax= 1;
                    validflag=2;
                }

                if (loop == 1) /*FIRST BAND*/
                {
                    if (loop2==1)minmin = curmin;
                    if (loop2==2)maxmax = curmax;
                }
                else
                {
                    if (curmin < minmin)
                        minmin = curmin;
                    if (curmax > maxmax)
                        maxmax = curmax;
                }
            }
        }
        fclose(stxin);

        /* OPENING THE .BLW */
        if ((bilwin = fopen(gridbilw,"r")) == NULL)
        {
            if ((bilwin = fopen(gridbilw2,"r")) == NULL)
                c_lasmsg(TRUE, "las2bil-open", "Error opening %s", gridbilw);
        }

        /* READING IN ALL OF THE .BLW INFORMATION */
        if (fscanf(bilwin,"%lf %lf %lf %lf %lf %lf",&xdim,&d_rotation,
                   &b_rotation,&ydim,&ulxmap,&ulymap) < 0)
        {
            fclose(bilwin);
            c_lasmsg(TRUE, "arc2las-read", "Error reading .blw file");
        }

        fclose(bilwin);

        /* RESETTING YDIM TO A POSITIVE VALUE */
        ydim = ydim * -1;

        /* SETTING DTYPE */
        if (nbits == 8)
        {
            if ((maxmax >= 255.5) || (minmin <= -0.5))
                c_lasmsg(TRUE, "arc2las-dtype", "Cannot process data");
        }
        else
        {
            if (nbits != 1)
                c_lasmsg(TRUE, "arc2las-dtype", "Cannot process data");
        }

        ddrstuff.dtype = EBYTE;

        /* PROCESSING THE .BILW TO COMPUTE CORNER COORDINATES.
           ---------------------------------------------------*/
        coordinates(xdim,ydim,b_rotation,d_rotation,ulxmap,ulymap,&ddrstuff);

        /* GET ARC/INFO PROJECTION INFORMATION
           -----------------------------------*/
        if (proj2ddr(&ddrstuff,ep3flag,workspace,ingrid) != E_SUCC)
        {
            ddrstuff.valid[DDPCV] = INVAL;
            c_lasmsg(FALSE,"arc2las-editddr",
                     "Use EDITDDR to enter projection data");
        }

        strcpy(systrg,"ieee-std");
        c_putddf(&ddrstuff,&key,(unsigned char *)systrg);

        /* PRINTING OUT THE DDR INFORMATION */
        c_putddr(ddrname,&ddrstuff);
        c_lused(ddrname);

        /* PROCESSING THE .STX FILE
           ------------------------*/
        if ((stxin = fopen(gridstx,"r")) == NULL)
            c_lasmsg(TRUE, "arc2las-open", "Error opening .stx");

        /*  INTIALIZING BDDR, ONLY NEEDS TO BE DONE ONCE */
        c_intbdr(&bddrstuff);

        /* GOING THROUGH A LOOP TO PROCESS ALL BANDS */
        for (loop = 1; loop <= ddrstuff.nbands; loop++)
        {
            /* SETTING THE READIN'S INDEXING VARIABLE TO 0 */
            index = 0;

            /* READING IN THE BAND INFORMATION */
            if (fgets(readin,STXLEN,stxin) == NULL)
            {
                fclose(stxin);
                c_lasmsg(TRUE, "arc2las-read", "Error reading .stx file");
            }

            length = (long)strlen(readin);
            error_ptr = NULL;

            /* ASSUMING THE READIN PARAMETERS ARE VALID */
            bddrstuff.valid = VALID;

            /* GETTING THE BAND NUMBER, MINIMUM AND MAXIMUM VALUES */
            for (loop2 = 0; loop2 < 3 && bddrstuff.valid == VALID; loop2++)
            {
                strip_num(readin,proc_num,&index);

                switch (loop2)
                {
                    case 0: bddrstuff.bandno = strtol(proc_num,&error_ptr,10);
                            break;
                    case 1: bddrstuff.minval = strtod(proc_num,&error_ptr);
                            break;
                    case 2: bddrstuff.maxval = strtod(proc_num,&error_ptr);
                            break;
                    default:
                            fclose(stxin);
                            c_lasmsg(TRUE, "arc2las-proc",
                                     "Error processing .stx file");
                }

                /* NEXT TO STATEMENT, JUST SETTING THE BAND NUMBER IF AN
                   ERROR OCCURS AND SETTING THE BAND FLAG TO INVALID */
                if (((error_ptr != NULL) && ((*error_ptr) != 0)) ||
                        (strlen(proc_num) == 0))
                {
                    bddrstuff.bandno = loop;
                    bddrstuff.valid = INVAL;
                }

                /* MAKING SURE INDEX IS NOT LARGER THAN THE LENGTH OF READIN */
                if (length < index)
                {
                    bddrstuff.bandno = loop;
                    bddrstuff.valid = INVAL;
                }
                if (nbits == 1)
                {
                    bddrstuff.minval = 0;
                    bddrstuff.maxval= 1;
                    bddrstuff.valid = 2;
                }
            }

            /* INVALIDATE MIN AND MAX PIXEL VALUES (DON'T TRUST ORIGINATOR) */
            bddrstuff.valid = INVAL;

            /* WRITING OUT THIS BDDR INFORMATION */
            c_putbdr(ddrname,&bddrstuff);
        }
        fclose(stxin);
    }
    else /* binary image file created by gridfloat so bilw, stx files
            not available */
    {
        /* Set the pixel size */
        ddrstuff.pdist_x = cellsize;
        ddrstuff.pdist_y = cellsize;
        ddrstuff.valid[DDPDV] = VALID;

        /* Set the corner coordinates. Only the LL corner is given in the
           .hdr file, so the others need to be calculated. This is a simple
           calculation using the nl, ns, and pixsize since Arc grids have
           no rotation to them (they are rectangular). Arc references the
           pixel at the center of the pixel, which is what we want. */
        ddrstuff.loleft[0] = ddrstuff.loright[0] = yllcorner;
        ddrstuff.loleft[1] = ddrstuff.upleft[1] = xllcorner;
        ddrstuff.upleft[0] = ddrstuff.upright[0] = yllcorner +
                                                    (ddrstuff.nl * cellsize);
        ddrstuff.loright[1] = ddrstuff.upright[1] = xllcorner +
                                                     (ddrstuff.ns * cellsize);
        ddrstuff.valid[DDCCV] = VALID;

        /* Determine if the data type needs to be upgraded to accomodate
           the NODATA pixel value, which is not included in the calculation
           of the zmin and zmax values retrieved in the PDF file */
        if((dtype == EWORD) && ((nodata_val < -32768) || (nodata_val > 32767)))
        {
            dtype = ELONG;
            lvalue = dtype;
            if (E_SUCC != lng2tae(&lvalue,1,1,"DTYPE",&tmp_long))
            {
                c_lasmsg(TRUE, "arc2las-qout",
                         "Error passing DTYPE variable to TAE");
            }

            if (q_out(&vblk) != SUCCESS)
            {
                c_lasmsg(TRUE, "arc2las-qout",
                         "Error passing variables to TAE");
            }
        }
        ddrstuff.dtype = dtype;

        /* GET ARC/INFO PROJECTION INFORMATION
           -----------------------------------*/
        if (proj2ddr(&ddrstuff,ep3flag,workspace,ingrid) != E_SUCC)
        {
            ddrstuff.valid[DDPCV] = INVAL;
            c_lasmsg(FALSE,"arc2las-editddr",
                     "Use EDITDDR to enter projection data");
        }

        strcpy(systrg,"ieee-std");
        c_putddf(&ddrstuff,&key,(unsigned char *)systrg);

        /* PRINTING OUT THE DDR INFORMATION */
        /* If the LAS image is being created from a grid stack, make a DDR
           for each layer so the information will be relayed to the final
           image file that COPY will create. */
        if(strcmp(geotype, "Stack") == 0)
        {
            /* It needs to look like a 1 band image at this point, so
               temporarily modify the number of bands */
            tmp = ddrstuff.nbands;
            ddrstuff.nbands = 1;
            for(i = 1; i <= nbands; i++) // for each layer, create separate DDR
            {
                strcpy(tmp_ddrname, tmp_imagename);
                sprintf(layernum, "_%d", i);
                strcat(tmp_ddrname, layernum);
                c_lsmknm(tmp_ddrname,"ddr",tmp_ddrname);
                c_putddr(tmp_ddrname,&ddrstuff);
            }
            ddrstuff.nbands = tmp;
        }
        else
            c_putddr(ddrname,&ddrstuff);

        /*  Intializing BDDR, only needs to be done once */
        c_intbdr(&bddrstuff);

        /* Save the band DDR information */
        /* Don't trust min/max pixel values, so set to 0 and 255 here and
           let MINMAX (run in the PDF later) figure out the true min/max */
        for(i = 0; i < ddrstuff.nbands; i++)
        {
            bddrstuff.bandno = i + 1;
            bddrstuff.minval = 0.0;
            bddrstuff.maxval = 255.0;
            bddrstuff.valid = INVAL; /* min/max pixel validity flag */

            /* WRITING OUT THIS BDDR INFORMATION */
            if(strcmp(geotype, "Stack") == 0)
            {
                /* If this is a grid stack, we're creating a DDR that
                   needs to reflect one band, so break out of loop after
                   first putbdr */
                for(j = 1; j <= nbands; j++) // for each layer, write bddr
                {
                    strcpy(tmp_ddrname, tmp_imagename);
                    sprintf(layernum, "_%d", j);
                    strcat(tmp_ddrname, layernum);
                    c_lsmknm(tmp_ddrname,"ddr",tmp_ddrname);
                    if(c_putbdr(tmp_ddrname,&bddrstuff) != E_SUCC)
                    {
                       c_lasmsg(TRUE, "arc2las-bdr",
                                "Error writing band DDR record");
                    }
                }
                break; // only write one band to DDR
            }
            else
            {
                if(c_putbdr(ddrname,&bddrstuff) != E_SUCC)
                   c_lasmsg(TRUE, "arc2las-bdr",
                            "Error writing band DDR record");
            }
        }
        c_lused(ddrname);
    }
    /* CREATING THE .HIS FILE. If the LAS image is being created from
       a grid stack, make a HIS file for each layer so the information
       will be relayed to the final image file that COPY will create.
       ---------------------------------------------------------------*/
    strcpy(text,"");
    if(strcmp(geotype, "Stack") == 0)
    {
        for(j = 1; j <= nbands; j++)
        {
            strcpy(tmp_hisname, tmp_imagename);
            sprintf(layernum, "_%d", j);
            strcat(tmp_hisname, layernum);
            c_lsmknm(tmp_hisname,"his",tmp_hisname);
            status = c_hstout(&vblk,tmp_hisname,hostin,&zero,text);
            if (status != E_SUCC)
                c_lasmsg(TRUE, "arc2las-open", "Error opening .his file");
        }
    }
    else
    {
        status = c_hstout(&vblk,hisname,hostin,&zero,text);
        if (status != E_SUCC)
            c_lasmsg(TRUE, "arc2las-open", "Error opening .his file");
    }
    fclose(bilin);

    /* CREATING THE .IMG FILE
       ----------------------*/
    if (nbits == 1)
    {
        bit2byte(gridbil, imagename[0], ddrstuff.nbands, nbits, ddrstuff.ns,
                ddrstuff.nl, msgtim, bandbytes);
    }

    strcpy(progname,"ARC2LAS");
    c_complt(progname, imagename, &outcount);
}
