/*  arcproj.h -- Definitions of Arc/Info Projections and Spheroids

PURPOSE:  
    This file contains definitions of structures, macros, and related
    constants to generate tables associating Arc/Info projection
    keywords and names, projection parameters, and spheroid keywords and
    names with the corresponding LAS keywords, names, and coefficients.
    The tables contain one record for each projection or spheroid, in
    the same order as they are specified in $LASINCLUDE/proj.h and
    $EDCSOFT/src/las/base/support/geolib/spheroid.c, respectively.
    Projections and spheroids which are defined in Arc/Info but not in
    LAS are added, in alphabetical order, at the ends of the tables.

    NOTE:  When proj.h or spheroid.c is updated, arcproj.h must also be
    updated.
  
USAGE:
    To allocate space for a table named tablename and initialize it with
    the desired values, use statements of the form

	#include "proj.h"
	#include "arcproj.h"
	SETPROJKN (tablename);		[Projection keywords and names]
	SETPROJCOEFMAP (tablename);	[Projection coeff. mappings]
        SETSPHERDEF (tablename);	[Spheroid keywords, names, and
					 semi-major/minor axes]

    Individual records of the tables generated by the first two macros
    may be accessed using the index values defined in proj.h.  Records
    of the third table are indexed using the LAS "datum" (actually,
    spheroid) code.  The contents of each record in these tables are
    described in the corresponding definitions of structures PROJKN,
    PROJCOEFMAP, and SPHERDEF; the "contcnt" ("continuation count") 
    and "map" fields of
    PROJCOEFMAP, however, require further explanation.

    PROJCOEFMAP.contcnt is normally zero.
    However, if alternate mappings are associated with a projection (as
    specified by the first record of the Arc/Info parameters), the
    "contcnt" field for the main entry contains the number of additional
    mappings.  Records defining these additional mappings are located
    after the end of the main table, beginning with record ARCPROJCOUNT,
    and may be identified by searching for a matching key entry.  The
    last record for a given projection is indicated by contcnt = 0.
    NOTE:  The value contcnt = -1 indicates that no parameter mapping
    has been defined for the given projection.

    PROJCOEFMAP.map contains two characters for each LAS projection
    coefficient in the DDR proj_coef array.  The first character
    specifies the format of the record in the Arc/Info .prj file from
    which the coefficient value is to be taken or an alternative source for
    it.  If the value is to be taken from the .prj file, the second
    character gives the record number within the Arc/Info .prj parameter
    list from which the value is to be taken, counting the first
    parameter value as record one.

    The format/source character has the following interpretations:
    
      blank	LAS coefficient unused.
	a	Arc/Info parameter is angle, in form dd mm ss.sss.
	d	Arc/Info parameter is decimal number.
	j	LAS coefficient is semi-maJor axis of spheroid.
	n	LAS coefficient is semi-miNor axis of spheroid.
	o	LAS coefficient is constant one.
	r	LAS coefficient is radius of sphere -- Arc/Info uses
		6370997 meters; LAS uses 0 to default to this value.
	t	LAS coefficient is constant two.
	z	LAS coefficient is constant zero.

    Two special cases are not fully supported by this table.  UTM may
    have zero or two paramaters and coefficients depending upon whether
    or not the zone is specified.  And Arc/Info UNIVERSAL_POLAR maps
    into POLAR with parameters (as defined below) UPSLCM, UPSLTS, UPSFE,
    and UPSFN.  Extra code is needed to handle these special cases.

    There are also two Arc/Info spheroid names which are not supported
    by LAS, but which are synoynms for supported spheroids.  These are
    NAD83 and WGS84, which are identical (within a few millimeters) to
    GRS80.
*/

/*  Constants defining sizes of arrays  */

#define	ARCPROJKEYLEN	24
#define	ARCPROJNAMELEN	40
#define	ARCPROJCOUNT	48
#define	ARCSPHERKEYLEN	24
#define	ARCSPHERNAMELEN	32
#define	ARCSPHERCOUNT	27
#define EXTRATBLRECS	 7
#define LASPROJKEYLEN	 8    

/*  Constants for mapping UPS to standard polar stereographic  */

#define UPSLCM	00 00 00
#define UPSLTS  81 06 52.3
#define UPSFE	2000000
#define UPSFN	2000000

/*  Structures for table records  */

struct PROJKN {		/* Projection keywords and name */
    char    lkey[LASPROJKEYLEN];	/* LAS or brief Arc keyword */
    char    akey[ARCPROJKEYLEN];	/* Arc/Info keyword */
    char    name[ARCPROJNAMELEN];	/* Arc/Info descriptive name */
    };

struct PROJCOEFMAP {	/* Projection coeffcient mapping record */
    char    lkey[LASPROJKEYLEN];	/* LAS or brief Arc keyword */
    short   contcnt;			/* Continuation record count */
    char    map[2 * COEFCT + 1];	/* Coeffcient mapping codes
					   and .prj record numbers */
    };

struct SPHERDEF {	/* Spheroid keywords, names, and semi-axes */
    char    akey[ARCSPHERKEYLEN];	/* Arc/Info spheroid keyword */
    char    name[ARCSPHERNAMELEN];	/* Arc/Info descriptive name */
    double  smjaxis;			/* Semi-major axis */
    double  smnaxis;			/* semi-minor axis */
    };

/*  Projection key definitions  */

#define SETPROJKN(tablename) \
  struct PROJKN tablename[ARCPROJCOUNT] = { \
    {"GEO",    "GEOGRAPHIC",	    "Latitude/longitude (not a projection)"}, \
    {"UTM",    "UTM",		    "Universal Transverse Mercator"}, \
    {"SPCS",   "STATEPLANE",	    "State Plane Coordinate System (SPCS)"}, \
    {"ALBERS", "ALBERS",	    "Albers conic equal area"}, \
    {"LAMCC",  "LAMBERT",	    "Lambert conformal conic"}, \
    {"MERCAT", "MERCATOR",	    "Standard Mercator"}, \
    {"PS",     "POLAR",		    "Polar Stereograhic"}, \
    {"POLYC",  "POLYCONIC",	    "Polyconic"}, \
    {"EQUIDC", "EQUIDISTANT",	    "EQUIDISTANT_CONIC"}, \
    {"TM",     "TRANSVERSE",	    "Transverse Mercator"}, \
    {"STEREO", "STEREOGRAPHIC",	    "Stereographic"}, \
    {"LAMAZ",  "LAMBERT_AZIMUTHAL", "Lambert azimuthal equal area"}, \
    {"AZMEQD", "AZIMUTHAL",	    "Azimuthal equidistant"}, \
    {"GNOMON", "GNOMONIC",	    "Gnomonic"}, \
    {"ORTHO",  "ORTHOGRAPHIC",	    "Orthographic"}, \
    {"GVNSP",  "PERSPECTIVE",	    "General vertical near-side perspective"}, \
    {"SNSOID", "SINUSOIDAL",	    "Sinusoidal (Sansom-Flamsteed)"}, \
    {"EQRECT", "EQUIRECTANGULAR",   "Equirectangular simple cylindrical"}, \
    {"MILLER", "MILLER",	    "Miller cylindrical"}, \
    {"VGRINT", "GRINTEN",	    "Van der Grinten I"}, \
    {"HOM",    "OBLIQUE_MERCATOR",  "Oblique Mercator (or Hotine)"}, \
    {"ROBIN",  "ROBINSON",	    "Robinson"}, \
    {"SOM",    "SPACE",		    "SPACE_OBLIQUE_MERCA"}, \
    {"ALASKA", "ALASKA_GRID",	   "Alaska modified stereographic conformal"}, \
    {"GOOD",   "",		    "Interrupted Goode homolosine"}, \
    {"MOLL",   "MOLLWEIDE",	    "Mollweide"}, \
    {"IMOLL",  "",		    "Interrupted Mollweide"}, \
    {"HAMMER", "HAMMER-AITOFF",	    "Hammer-Aitoff equal area"}, \
    {"WAGIV",  "",		    "Wagner IV"}, \
    {"WAGVII", "",		    "Wagner VII"}, \
    {"OBEQA",  "",		    "Oblated equal area"}, \
    {"BPLOBL", "BIPOLAR_OBLIQUE",   "Bipolar oblique conic conformal"}, \
    {"BONNE",  "BONNE",		    "Bonne"}, \
    {"CASSIN", "CASSINI",	    "Cassini"}, \
    {"CHAMBR", "CHAMBERLIN",	    "Chamberlain"}, \
    {"CRASPB", "CRASTER_PARABOLIC", "Craster parabolic"}, \
    {"CYLIND", "CYLINDRICAL",	    "Cylindrical equal area"}, \
    {"ECKRIV", "ECKERTIV",	    "Eckert IV"}, \
    {"ECKRVI", "ECKERTVI",	    "Eckert VI"}, \
    {"FLATPQ", "FLAT_POLAR_QUARTIC", "McBryde-Thomas flat polar quartic"}, \
    {"GALLST", "GALLS_STEREOGRAPHIC", "Galls stereographic"}, \
    {"GBGRID", "GREATBRITAIN_GRID", "Great Britain National Grid"}, \
    {"NZGRID", "NEWZEALAND_GRID",   "New Zealand National Grid"}, \
    {"RSO",    "RSO",		    "Rectified skewed orthomorphic"}, \
    {"SMPLCN", "SIMPLE_CONIC",	    "Simple conic"}, \
    {"TIMES",  "TIMES",		    "Times"}, \
    {"TWOPT",  "TWO_POINT_EQUIDISTANT", "Two-point equidistant"}, \
    {"UPS",    "UNIVERSAL_POLAR",   "Universal Polar Stereogrpahic"} }

/*  Create table for mapping projection coefficients  */

#define SETPROJCOEFMAP(tablename) \
    struct PROJCOEFMAP tablename[ARCPROJCOUNT + EXTRATBLRECS] = { \
        {"GEO",     0, "                              "}, \
        {"UTM",     0, "a1a2                          "}, \
        {"SPCS",    0, "                              "}, \
        {"ALBERS",  0, "j n a1a2a3a4d5d6              "}, \
        {"LAMCC",   0, "j n a1a2a3a4d5d6              "}, \
        {"MERCAT",  0, "j n     a1a2d3d4              "}, \
        {"PS",      0, "j n     a1a2d3d4              "}, \
        {"POLYC",   0, "j n     a1a2d3d4              "}, \
        {"EQUIDC",  1, "j n a2  a3a4d5d6z             "}, \
        {"TM",      0, "j n d1  a2a3d4d5              "}, \
        {"STEREO",  1, "d2      a3a4d5d6z             "}, \
        {"LAMAZ",   0, "d1      a2a3d4d5              "}, \
        {"AZMEQD",  0, "d1      a2a3d4d5              "}, \
        {"GNOMON",  0, "d1      a2a3d4d5              "}, \
        {"ORTHO",   0, "d1      a2a3d4d5              "}, \
        {"GVNSP",   0, "d1  d2  a3a4d5d6              "}, \
        {"SNSOID",  0, "j       a2  d3d4              "}, \
        {"EQRECT",  0, "d1      a2z d3d4              "}, \
        {"MILLER",  0, "d1      a2  d3d4              "}, \
        {"VGRINT",  0, "d1      a2z d3d4              "}, \
        {"HOM",     1, "j n d2    a3d8d9a4a5a6a7z     "}, \
        {"ROBIN",   0, "r       a1  d2d3              "}, \
        {"SOM",     0, "j n d1d2                o     "}, \
        {"ALASKA",  0, "j n         z z               "}, \
        {"GOOD",   -1, "                              "}, \
        {"MOLL",    0, "r       a1  z z               "}, \
        {"IMOLL",  -1, "                              "}, \
        {"HAMMER",  0, "r       a1  z z               "}, \
        {"WAGIV",  -1, "                              "}, \
        {"WAGVII", -1, "                              "}, \
        {"OBEQA",  -1, "                              "}, \
        {"BPLOBL",  0, "                              "}, \
        {"BONNE",   0, "        a1a2d3d4              "}, \
        {"CASSIN",  0, "        a1a2d3d4              "}, \
        {"CHAMBR",  0, "    a1a2        a3a4a5a6      "}, \
        {"CRASPB",  0, "        a1                    "}, \
        {"CYLIND",  2, "        a2a3            z     "}, \
        {"ECKRIV",  0, "        a1                    "}, \
        {"ECKRVI",  0, "        a1                    "}, \
        {"FLATPQ",  0, "        a1                    "}, \
        {"GALLST",  0, "        a1                    "}, \
        {"GBGRID",  0, "                              "}, \
        {"NZGRID",  0, "                              "}, \
        {"RSO",     1, "                        z     "}, \
        {"SMPLCN",  0, "    a3  a2              z     "}, \
        {"TIMES",   0, "d1      a2  d3d4              "}, \
        {"TWOPT",   0, "                a1a2a3a4      "}, \
        {"UPS",     0, "                              "}, \
	{"EQUIDC",  0, "j n a2a3a4a5d6d7o             "}, \
	{"STEREO",  0, "                              "}, \
	{"HOM",     0, "j n d2a5a3a4d6d7        o     "}, \
	{"CYLIND",  1, "    d2          a3a4a5a6o     "}, \
	{"CYLIND",  0, "    d2a3a4a5            t     "}, \
	{"RSO",     0, "                        o     "}, \
	{"SMPLCN",  0, "    a3a4a2              o     "} }


/*  Create table of spheroid keys, names, and semi-axes  */

#define SETSPHERDEF(tablename) \
    struct SPHERDEF tablename[ARCSPHERCOUNT] = { \
	{"CLARKE1866",		"Clarke 1866", \
			6378206.4,	6356583.8}, \
	{"CLARKE1880",		"Clarke 1880", \
			6378249.145,	6356514.86955}, \
	{"BESSEL",		"Bessel", \
			6377397.155,	6356078.9628}, \
	{"INT1967",		"New International 1967", \
			6378157.5,	6356772.2}, \
	{"INT1909",		"International 1909", \
			6378388.0,	6356911.94613}, \
	{"WGS72",		"WGS 1972", \
			6378135.0,	6356750.519915}, \
	{"EVEREST",		"Everest", \
			6377276.3452,	6356075.4133}, \
	{"WGS66",		"WGS 1966", \
			6378145.0,	6356759.769356}, \
	{"GRS80",		"GRS 1980", \
			6378137.0,	6356752.31414}, \
	{"AIRY",		"Airy", \
			6377563.396,	6356256.91}, \
	{"MODEVEREST",		"Modified Everest", \
			6377304.063,	6356103.039}, \
	{"MODAIRY",		"Modified Airy", \
			6377341.89,	6356036.143}, \
	{"WALBECK",		"Walbeck", \
			6376896.0,	6355834.8467}, \
	{"SEASIA",		"Southeast Asia", \
			6378155.0,	6356773.3205}, \
	{"AUSTRALIAN",		"Australian National", \
			6378160.0,	6356774.719}, \
	{"KRASOVSKY",		"Krasovsky", \
			6378245.0,	6356863.0188}, \
	{"HOUGH",		"Hough", \
			6378270.0,	6356794.343479}, \
	{"MERCURY1960",		"Mercury 1960", \
			6378166.0,	6356784.283666}, \
	{"MERCURY1968",		"Modified Mercury 1968", \
			6378150.0,	6356768.337303}, \
	{"SPHERE",		"Sphere, R = 6370997 meters", \
			6370997.0,	6370997.0}, \
	{"FISCHER1960",		"Fischer 1960", \
			6378166.0,	6356784.28}, \
	{"FISCHER1968",		"Fischer 1968", \
			6378150.0,	6356768.33}, \
	{"NAD83",		"NAD83 (same as GRS 1980)", \
			6378137.0,	6356752.31414}, \
	{"HELMERT1906",		"Helmert 1906", \
			6378200.0,	6356818.17}, \
	{"MODIFIEDFISCHER1960",	"Modified Fischer 1960", \
			6378155.0,	6356773.32}, \
	{"SOUTHAMERICAN1969",	"South American 1969", \
			6378160.0,	6356774.72}, \
	{"WGS84",		"WGS 1984", \
			6378137.0,	6356752.31} }
