/******************************************************************************
FUNCTION: bit2byte

PURPOSE: Convert 1 & 4 bit images to BYTE

HISTORY:
$Log: bit2byte.c,v $
Revision 1.9  2002/11/04 18:16:09  shimitz
Merge development branch for CRs 5170, 5174, & 5792 with the trunk.
5170: Enhance file clean-up to remove the "temptn" files generated during
      processing.
5174: Enhance to support floating-point grids.
5792: Allow the user to enter the ingrid in any case (upper, lower, mixed) and
      still read the lower-case grid from disk.

Revision 1.8.6.2  2002/01/30 15:47:07  shimitz
Updated arc2las to properly convert floating-point arc grids and integer grids
with pixel values exceeding the byte range (0-255) to LAS imagery by using the
arc command gridfloat. CR 5174

******************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "arc2las.h"

int bit2byte
(
  char *gridbil,             /* I: Input BIL image                 */
  char *hostout,             /* I: Output LAS image                */
  long nbands,               /* I: Number of bands                 */
  long nbits,                /* I: Number of bits                  */
  long ns,                   /* I: number of samples               */
  long nl,                   /* I: number of lines                 */
  long msgtim,               /* I: Message processing time         */
  long bandbytes             /* I: Bytes per band                  */
)
{
    FILE *imageout;            /* Output LAS image                */
    FILE *bilin;               /* Input 1-bit bil image           */

    extern long curent;        /* Current line being processed    */
    extern long curcat;        /* Current band being processed    */
    long blocksize = 512;      /* Output write blocksize          */
    long i;                    /* Loop control variable           */
    long j;                    /* Loop control variable           */
    long linesize;             /* Size of processing line         */
    long sizefield;            /* Size of a character             */
    long byteskip;             /* Number of bytes to skip         */
    long fullinput;            /* Number of bits per band         */
    long diffbyte=0;           /* Remainder byte                  */
    long msg_flag=ALLCAT;      /* Message processing flag         */
    long lineget;              /* Number of lines to get          */
    long leftget=0;

    unsigned char *chinput;    /* Input processing buf            */
    unsigned char *choutput;   /* Output processing buf           */

    /* Determine if the output image exists
       ------------------------------------*/
    if (access(hostout, R_OK) == 0)
        c_lasmsg(TRUE,"arc2las-exists","Output image already exists");

    /* Open the output image for binary writing
       ----------------------------------------*/
    if ((imageout=fopen(hostout, "wb")) == NULL)
        c_lasmsg(TRUE,"arc2las-open","Error opening output image");

    /* Open the input image for binary reading
       ---------------------------------------*/
    if ((bilin=fopen(gridbil,"rb")) == NULL)
    {
        fclose(imageout);
        c_lasmsg(TRUE,"arc2las-open","Error opening input image");
    }

    /* Initializing the needed variables
       ---------------------------------*/
    linesize = ns * nbits;
    byteskip = linesize * (nbands - 1);
    fullinput = (long)(linesize / 8);
    lineget = (long)(fullinput * 8);
    leftget = linesize - lineget;
    sizefield = sizeof(char);

    /* Allocating space for the input and output buffers
       -------------------------------------------------*/
    chinput = (unsigned char *)malloc(sizeof(unsigned char) * (ns/8));
    choutput = (unsigned char *)malloc(sizeof(unsigned char) * ns);

    /* Determining if there is a left over byte in the nband or not
       ------------------------------------------------------------*/
    if (((linesize % 8) == 0) && (linesize > 8))
        diffbyte = 0;
    else
        diffbyte = 1;

    /* Initialize the message processing tools
       ---------------------------------------*/
    c_inittm("ARC2LAS","Processed","line","bands"," ",
            &nl,&nbands,&msgtim,&msg_flag);
    curent = 1;
    curcat = 1;
    c_startm();

    /* Reading in the image to create a bil
       ------------------------------------*/
    for (i = 1; i <= nbands; i++)
    {
        curcat = i;
        for (j = 1; j <= nl; j++)
        {
            curent = j;
            if (fread((void *)chinput,sizefield,fullinput,bilin) != fullinput)
            {
                fclose(bilin);
                fclose(imageout);
                c_lasmsg(TRUE,"arc2as-read1","Error reading input image");
            }

            /* Checking if bits size is 1 or 4,
               and ajusting for it
               --------------------------------*/
            if (nbits <= 4)
            {
                sep_shift(chinput,choutput, lineget);
                if (fwrite((void *)choutput,sizefield,lineget,imageout)
                    != lineget)
                {
                    fclose(imageout);
                    fclose(bilin);
                    c_lasmsg(TRUE,"arc2las-write1",
                             "Error writing output image");
                }
                if (fread((void *)chinput,sizefield,diffbyte,bilin) != diffbyte)
                {
                    fclose(bilin);
                    fclose(imageout);
                    c_lasmsg(TRUE,"arc2as-read11","Error reading input image");
                }
                sep_shift(chinput,choutput, leftget);
                if (fwrite((void *) choutput,sizefield,leftget,imageout)
                    != leftget)
                {
                    fclose(imageout);
                    fclose(bilin);
                    c_lasmsg(TRUE,"arc2las-write1",
                             "Error writing output image");
                }
            }
            else if(fwrite((void*)choutput,sizefield,blocksize,imageout)
                    != blocksize)
            {
                fclose(imageout);
                fclose(bilin);
                c_lasmsg(TRUE,"arc2las-write8","Error writing output image");
            }

            if ((j != nl) && (nbands != 1))
                if (fseek(bilin, byteskip, SEEK_CUR) != NULL)
                    c_lasmsg(TRUE,"arc2las-readnl","Error reading input image");

        } /* END OF LOOP 1 */

        /* If only one band, it's not necessary
           to reposition file pointer
           ------------------------------------*/
        if (nbands != 1)
            if (fseek(bilin,(bandbytes * i), SEEK_SET) != NULL)
                c_lasmsg(TRUE,"arc2las-read","Error moving pointer");

    } /* END OF LOOP 2 */

    c_stoptm();
    fclose(imageout);
    fclose(bilin);
    return(E_SUCC);
}
