/* @(#)infodir.c	1.7 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infodir.c
 Description: Contains infolib directory file functions.  The following
                functions are contained in this module:

                  DirGetRecord
                  DirWriteRecord

    History: 12/91  Todd Stellhorn  - original coding
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"

/***************************************************************************/


/****************************************************************************
       Name: DirGetRecord
    Purpose: This function searches the input DR9 file for a record
               entry of type RecordStatus (set using macros DirEMPTY or
               DirNotEMPTY) and matching the input INFO external file
               name.  The matched record and its file position are
               returned.  If RecordStatus is DirEMPTY, the first empty
               record is returned.  If no empty records are found, an
               error is generated and Position is set to a negative number
               which equals the size of the DR9 file.  This allows for
               appending of new records onto the end of the DR9 file.
     Inputs: DirFilePath ( char * ) INFO DR9 file path
             RecordStatus ( short int ) Directory record type
               (DirEMPTY or DirNotEMPTY)
             FileName ( char * ) INFO external file name
             ReturnRecord ( DirRecord * ) Directory record buffer
             Position ( long int * ) Directory record file position
               (record number)
    Outputs: Function - ( short int ) Returned function status
                        0 == error returning record
                        1 == matching record found
               On error, RecordStatus and Position are undefined unless
               Position is less than 0, then Position * -1 indicates
               the last directory entry position.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int DirGetRecord
(
      char      *DirFilePath,
      short int  RecordStatus,
      char      *FileName,
      DirRecord *ReturnRecord,
      long int  *Position
)
{

      char          InternalName[INTERNALNAME_SIZE];
      FILE         *DirFile = ( FILE * ) NULL;
      DirRecord     Record;

      *Position = 0;

      if(( DirFile = OpenFile( DirFilePath, InfoREAD )) ==
           ( FILE * ) NULL )
      {
           return( 0 );
      }

      while( 1 ) /* loop until end of file or matching record found */
      {
           if( fread( &Record, sizeof( DirRecord ), 1, DirFile ) != 1 )
           {
                CloseFile( DirFile );
                return( 0 );
           }

           if( feof( DirFile ))
           {
                if( ! CloseFile( DirFile ))
                {
                     return( 0 );
                }

                /* if file not found, flag Position and return error */

                *Position = *Position * -1;
                return( 0 );
           }

           if(( Record.EmptyFlag == RecordStatus ) &&
                ( Record.FileType == DirDataFileTYPE ))
           {
                if( RecordStatus == DirNotEMPTY )
                {
                     TerminateString( Record.InternalName,
                          sizeof( Record.InternalName ), InternalName );

                     if(( CompareStrings( InternalName, FileName,
                          NONCASE_SENSITIVE )) == 0 )
                     {
                          break;
                     }
                }
                else
                     break;
           }
           *Position = *Position + 1;
      }

      if( ! CloseFile( DirFile ))
      {
           return( 0 );
      }

      memcpy( ReturnRecord, &Record, sizeof( DirRecord ));

      return( 1 );
}                                           /* end of DirGetRecord         */



/****************************************************************************
       Name: DirWriteRecord
    Purpose: This function writes the input directory record to the input
               DR9 file based on the record's position.  No checking of
               the input record or its file position is performed.  An
               error is generated if the directory file cannot be
               opened for write, the file position cannot be located, an
               error occurs while writing to the DR9 file, or a file close
               error occurs.  On success, the subject DR9 file is updated 
               with the input record.
     Inputs: DirFilePath ( char * ) INFO directory file path
             Record ( DirRecord * ) DR9 record
             Position ( long int ) DR9 record position (record number)
    Outputs: Function - ( short int ) Returned function status
                        0 == write error
                        1 == write ok
               On success, the subject DR9 file is updated, on error, 
               results are undefined.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int DirWriteRecord
(
      char       *DirFilePath,
      DirRecord  *Record,
      long int    Position
)
{

      FILE         *DirFile;

      if(( DirFile = OpenFile( DirFilePath, InfoWRITE )) ==
           ( FILE * ) NULL )
      {
           return( 0 );
      }

      if( fseek( DirFile,
           ( long int ) ( sizeof( DirRecord ) * Position ), 0 ))
      {
           CloseFile( DirFile );
           return( 0 );
      }

      if(( fwrite( Record, sizeof( DirRecord ), 1, DirFile )) != 1 )
      {
           CloseFile( DirFile );
           return( 0 );
      }

      if( ! CloseFile( DirFile ))
      {
           return( 0 );
      }

      return( 1 );

}                                           /* end of DirWriteRecord       */

/*                                             end of infodir.c            */
