/* @(#)infoetc.c	1.9 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infoetc.c
 Description: Contains infolib miscellaneous functions.   The following
                functions are contained in this module:

                  OpenFile
                  CloseFile
                  CompareValues
                  GetExternalPath
                  IOBufferAllocate
                  InfoFileFree
                  ItemDefFree

    History: 12/91  Todd Stellhorn  - original coding
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"

/***************************************************************************/


/****************************************************************************
       Name: OpenFile
    Purpose: This function attempts to open a file based on the input
               file name and file access.  File access is set using the
               macros InfoREAD, InfoWRITE, InfoINIT.  If a file is opened
               with InfoINIT, the file will be truncated to zero length.
               All file access types open files in binary mode.  On
               success, a file pointer is returned or NULL on error.
     Inputs: FileName ( char * ) File name
             Access ( short int ) File open access type (access type
               set using macros InfoREAD, InfoWRITE and InfoINIT) - all
               files are opened in binary mode
    Outputs: Function - ( FILE * ) Returned pointer to opened file, NULL
               on error
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
FILE *OpenFile
(
      char      *FileName,
      short int Access
)
{

      FILE       *FilePtr;

      switch ( Access )
      {
           case InfoREAD:

                if(( FilePtr = fopen( FileName, "rb" )) == NULL )
                {
                     return(( FILE * ) NULL );
                }

                break;

           case InfoWRITE:

                if(( FilePtr = fopen( FileName, "r+b" )) == NULL )
                {
                     return(( FILE * ) NULL );
                }

                break;

           case InfoINIT:

                if(( FilePtr = fopen( FileName, "w+b" )) == NULL )
                {
                     return(( FILE * ) NULL );
                }

                break;

           default:

                return(( FILE * ) NULL );

      }

      return( FilePtr );

}                                           /* end of OpenFile             */



/****************************************************************************
       Name: CloseFile
    Purpose: This function closes an open file pointed to by the input
               file pointer and returns the file close status.
     Inputs: FilePtr ( FILE * ) File pointer
    Outputs: Function - ( short int ) Returned close status
                        0 == file close error
                        1 == file close ok
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int CloseFile
(
      FILE *FilePtr
)
{

      if( fclose( FilePtr ) != 0 )
      {
           return( 0 );
      }

      return( 1 );
}                                           /* end of CloseFile            */


/****************************************************************************
       Name: CompareValues
    Purpose: This function compares two values and returns a comparison
               status.  The first value is contained in either
               SearchString or SearchInt and the second value is contained
               in the InfoFilePtr's IOBuffer.  The value from the IOBuffer
               is decoded based on the input item definition and used for
               the comparison.  This function can only be used with C,
               I, or B INFO data types (macros INFO_CHARACTER_TYPE,
               INFO_INTEGER_TYPE, or INFO_BINARY_TYPE).  An error will be
               generated if any other data type is used.  The comparison 
               status value is returned via CompareCode.  Function
               status is also returned.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             ItemDef ( InfoItemDef * ) INFO item definition
             SearchString ( char * ) Search string
             SearchInt ( long int ) Search integer value
             CompareCode ( short int * ) Compare status
    Outputs: Function - ( short int )  Returned function status
                          0 == compare error
                          1 == compare ok
               Returned comparison status (CompareCode):
                          < 0 == SearchString or SearchInt is less than
                                   IOBuffer value
                         == 0 == SearchString or SearchInt is equal
                                   to IOBuffer value
                          > 0 == SearchString or SearchInt is greater
                                   than IOBuffer value
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int CompareValues
(
      InfoFile     *InfoFilePtr,
      InfoItemDef  *ItemDef,
      char         *SearchString,
      long int      SearchInt,
      short int    *CompareCode
)
{

     char               RetString[TMPRECORD_SIZE];
     double             DRetVal;
     long int           CompareVal;

     if( ! InfoDecode( InfoFilePtr, ItemDef, RetString, &DRetVal ))
     {
          return( 0 );
     }

     switch( ItemDef->ItemType )
     {

          case INFO_CHARACTER_TYPE :

              CompressString( RetString, ItemDef->ItemWidth );
              *CompareCode = CompareStrings( SearchString, RetString,
                   CASE_SENSITIVE );

              break;

          case INFO_INTEGER_TYPE   :
          case INFO_BINARY_TYPE    :

              CompareVal = SearchInt - ( long int ) DRetVal;

              if( ! CompareVal )
                   *CompareCode = 0;
              else
                   if( CompareVal < 0 )
                        *CompareCode = -1;
                   else
                        *CompareCode = 1;

              break;

          default :

               return( 0 );
     }

     return( 1 );

}                                           /* end of CompareValues        */


/****************************************************************************
       Name: GetExternalPath
    Purpose: This function returns the pathname string from an
               INFO externalled data file.  Function assumes a full
               pathname to an INFO data file (xxx###dat file) and the
               full pathname to the INFO directory.  The function 
               reads the pathname string from the data file and tests
               if the path name is a relative path (such as generated
               with ARC/INFO Rev6.0).  If a relative path is found, an
               attempt is made to generate a full pathname using the 
               pathname to the INFO directory.  This function does 
               not expand relative paths which do not start with "..".   
               Expanded file pathnames are not guaranteed to open
               the externalled data file. Function status and a 
               fully expanded data file path are returned.
     Inputs: DataFilePath ( char * ) External data file pathname
             InfoFilePath ( char * ) INFO directory pathname
             ExternalFilePath ( char * ) Expanded external data file
               pathname
    Outputs: Function - ( short int ) Returned function status
                          0 == status error
                          1 == status ok
               Also returned is expanded external data file pathname
               (ExternalFilePath).
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int GetExternalPath
(
      char *DataFilePath,
      char *InfoFilePath,
      char *ExternalFilePath
)
{

      char        TmpString[PATH_SIZE];
      char        TmpPath[PATH_SIZE];
      char       *StringPtr;
      FILE       *FilePointer = ( FILE * ) NULL;

      if(( FilePointer = fopen( DataFilePath, "r" )) == NULL )
      {
           return( 0 );
      }

      if(( fgets( TmpString, sizeof( TmpString ), FilePointer )) == NULL )
      {
           CloseFile( FilePointer );
           return( 0 );
      }

      if( ! CloseFile( FilePointer ))
      {
           return( 0 );
      }
      
      /* terminate and remove trailing blanks from string and convert 
         string to lowercase 
       */

      TerminateString( TmpString, sizeof( TmpString ), TmpPath );
      StringtoLower( TmpPath );

      /* expand relative path to fullpath */

      if(( TmpPath[0] == '.' ) && ( TmpPath[1] == '.' ))
      {
           /* extract directory path from info file pathname, first
              copy info file pathname into temp string buffer */
           strcpy( TmpString, InfoFilePath );

           /* point to end of string buffer */
           StringPtr = &TmpString[ strlen( TmpString ) - 1 ];

           /* search backwards until operating system separator found */
           while( *StringPtr != OPSYS_SEP[0] )
                StringPtr--;

           /* terminate string and copy result */
           *StringPtr = '\0';

           sprintf( ExternalFilePath, "%s%s", TmpString, &TmpPath[2] );
      }
      else
      {
           strcpy( ExternalFilePath, TmpPath );
      }

      return( 1 );
}                                           /* end of GetExternalPath      */



/****************************************************************************
       Name: IOBufferAllocate
    Purpose: This function allocates a character buffer based on an
               input buffer size.  Allocated buffer is initialized with
               blanks (' ').  Pointer to buffer is returned or NULL on
               error.
     Inputs: BufferSize ( short int ) Character buffer size
    Outputs: Function - ( char * ) Character pointer to allocated
               buffer or NULL on error.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
char *IOBufferAllocate
(
      short int BufferSize
)
{

      char          *Buffer;

      if(( Buffer = ( char * ) malloc( BufferSize )) == NULL )
      {
           return(( char * ) NULL );
      }

      memset( Buffer, ' ', BufferSize );

      return( Buffer );

}                                           /* end of IOBufferAllocate     */


/****************************************************************************
       Name: InfoFileFree
    Purpose: This function returns INFO file pointer memory to free
               memory.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
    Outputs: Function - ( void )
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void InfoFileFree
(
      InfoFile *InfoFilePtr
)
{

      if( InfoFilePtr == ( InfoFile * ) NULL )
           return;

      if( InfoFilePtr->FileName != ( char * ) NULL )
           free( InfoFilePtr->FileName );

      if( InfoFilePtr->DirFilePath != ( char * ) NULL )
           free( InfoFilePtr->DirFilePath );

      if( InfoFilePtr->NitFilePath != ( char * ) NULL )
           free( InfoFilePtr->NitFilePath );

      if( InfoFilePtr->InfoFILE != ( FILE * ) NULL )
           free( InfoFilePtr->InfoFILE );

      if( InfoFilePtr->IOBuffer != ( char * ) NULL )
           free( InfoFilePtr->IOBuffer );

      free( InfoFilePtr );
}                                           /* end of InfoFileFree         */



/****************************************************************************
       Name: ItemDefFree
    Purpose: This function returns item definition memory to free memory.
     Inputs: ItemDef ( InfoItemDef * ) Pointer to item definition
    Outputs: Function - ( void )
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void ItemDefFree
(
      InfoItemDef *ItemDef
)
{

      if( ItemDef != ( InfoItemDef * ) NULL )
           free( ItemDef );
}                                           /* end of ItemDefFree          */

/*                                             end of infoetc.c            */
