/* @(#)infofile.c	1.11 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infofile.c
 Description: Contains infolib file functions.  The following
                functions are contained in this module:

                  InfoOpenFile
                  InfoCloseFile
                  InfoFileExists
                  InfoReadRecord
                  InfoWriteRecord

    History: 12/91  Todd Stellhorn  - original coding
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"

/***************************************************************************/


/****************************************************************************
       Name: InfoOpenFile
    Purpose: This function opens an INFO file based on an input INFO file
               pathname and access type.  Access types are set using the
               macros InfoREAD, InfoWRITE and InfoINIT (InfoINIT will
               truncate the data file to zero length on open).  ARC/INFO
               style INFO file pathnames are supported, such as:
                 /sleet1/todd/infolib/info:arc:test
               where:
                 /sleet1/todd/infolib/info - full pathname to INFO
                   directory
                 : - INFO path separator -- ":" is used under UNIX because
                   /bin/csh uses "!" as history character.  this value is
                   set using the infodefs.h macro PATH_DELIMITER.
                 test - external name of the INFO file
               InfoOpenFile returns an InfoFile pointer or NULL on
               error.
     Inputs: InfoFilePath ( char * ) INFO file pathname
             AccessType ( short int ) File access type (access types
               are defined by macros InfoREAD, InfoWRITE and InfoINIT)
    Outputs: Function - ( InfoFile * ) Returned pointer to open INFO file
               on success or NULL on error.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
InfoFile *InfoOpenFile
(
      char *InfoFilePath,
      short int AccessType
)
{

      char          InfoDirPath[PATH_SIZE];
      char          UserName[USERNAME_SIZE];
      char          FileName[INTERNALNAME_SIZE];
      char          DirFilePath[PATH_SIZE];
      char          ExternalName[PATH_SIZE];
      char          ExternalPath[PATH_SIZE];
      char          NitFilePath[PATH_SIZE];
      char          TmpPath[PATH_SIZE];
      DirRecord     Record;
      long int      DirPosition = 0;
      InfoFile     *InfoFilePtr = ( InfoFile * ) NULL;

      if( ! ParsePathName( InfoFilePath, InfoDirPath, UserName, FileName ))
      {
           return( InfoFilePtr );
      }

      sprintf( DirFilePath, "%s%s%s%s", InfoDirPath, OPSYS_SEP,
           UserName, DIR_EXT );

      StringtoLower( DirFilePath );

      if( ! DirGetRecord( DirFilePath, DirNotEMPTY, FileName,
           &Record, &DirPosition ))
      {
           return( InfoFilePtr );
      }

      if( Record.RecordLength == 0 )  /* zero record length, return error */
      {
           return( InfoFilePtr );
      }

      TerminateString( Record.ExternalName, sizeof( Record.ExternalName ),
           ExternalName );

      sprintf( ExternalPath, "%s%s%s%s", InfoDirPath, OPSYS_SEP,
           ExternalName, DATA_EXT );
      sprintf( NitFilePath, "%s%s%s%s", InfoDirPath, OPSYS_SEP,
           ExternalName, NIT_EXT );

      StringtoLower( ExternalPath );
      StringtoLower( NitFilePath );
                                          /* test for external data file */
      if( Record.ExternalFlag[0] == 'X' )
      {
           strcpy( TmpPath, ExternalPath );

           if( ! GetExternalPath( TmpPath, InfoDirPath, ExternalPath ))
           {
                return(( InfoFile * ) NULL );
           }

      }

      if(( InfoFilePtr = ( InfoFile * )
           malloc( sizeof ( InfoFile ))) == NULL )
      {
           return(( InfoFile * ) NULL );
      }

      InfoFilePtr->FileName = ( char * ) NULL;
      InfoFilePtr->DirFilePath = ( char * ) NULL;
      InfoFilePtr->NitFilePath = ( char * ) NULL;
      InfoFilePtr->InfoFILE = ( FILE * ) NULL;
      InfoFilePtr->NumberItems = Record.NumberItems;
      InfoFilePtr->RecordLength = Record.RecordLength;
      InfoFilePtr->NumberRecords = Record.NumberRecords;
      InfoFilePtr->AccessFlag = AccessType;
      InfoFilePtr->CurrentRecord = 0;
      InfoFilePtr->IOBuffer = ( char * ) NULL;

      if(( InfoFilePtr->FileName =
           ( char * ) malloc( strlen( FileName ) + 1 )) == NULL )
      {
           InfoFileFree( InfoFilePtr );
           return(( InfoFile * ) NULL );
      }

      strcpy( InfoFilePtr->FileName, FileName );

      if(( InfoFilePtr->DirFilePath =
           ( char * ) malloc( strlen( DirFilePath ) + 1 )) == NULL )
      {
           InfoFileFree( InfoFilePtr );
           return(( InfoFile * ) NULL );
      }

      strcpy( InfoFilePtr->DirFilePath, DirFilePath );

      if(( InfoFilePtr->NitFilePath =
           ( char * ) malloc( strlen( NitFilePath ) + 1 )) == NULL )
      {
           InfoFileFree( InfoFilePtr );
           return(( InfoFile * ) NULL );
      }

      strcpy( InfoFilePtr->NitFilePath, NitFilePath );

      if(( InfoFilePtr->InfoFILE = OpenFile( ExternalPath,
           AccessType )) == ( FILE * ) NULL )
      {
           InfoFileFree( InfoFilePtr );
           return(( InfoFile * ) NULL );
      }

      if( AccessType == InfoINIT )    /* if init, update number of records */
           InfoFilePtr->NumberRecords = 0;

      if(( InfoFilePtr->IOBuffer =
           IOBufferAllocate( InfoFilePtr->RecordLength )) == NULL )
      {
           InfoFileFree( InfoFilePtr );
           return(( InfoFile * ) NULL );
      }

      return( InfoFilePtr );

}                                           /* end of InfoOpenFile         */


/****************************************************************************
       Name: InfoCloseFile
    Purpose: This function closes an open INFO file and returns the
               InfoFile memory to free memory.  If the file was opened
               using macros InfoWRITE or InfoINIT, DR9 file is updated
               with the current number of records.  INFO file close
               status is returned.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
    Outputs: Function - ( short int )  INFO file close status
                          0 == file close error
                          1 == file close ok
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoCloseFile
(
      InfoFile *InfoFilePtr
)
{

      if( ! CloseFile( InfoFilePtr->InfoFILE ))
      {
           return( 0 );
      }

      if( InfoFilePtr->AccessFlag != InfoREAD )
      {
           long int         DirPosition = 0;
           DirRecord        Record;

           if( ! DirGetRecord( InfoFilePtr->DirFilePath, DirNotEMPTY,
                InfoFilePtr->FileName, &Record, &DirPosition ))
           {
                return( 0 );
           }

           Record.NumberRecords = InfoFilePtr->NumberRecords;

           if( ! DirWriteRecord( InfoFilePtr->DirFilePath, &Record,
                DirPosition ))
           {
                return( 0 );
           }
      }

      InfoFileFree( InfoFilePtr );

      return( 1 );
}                                           /* end of InfoCloseFile        */



/****************************************************************************
       Name: InfoFileExists
    Purpose: This function tests for the existence of the input INFO.  The 
               test is performed by searching the DR9 file for a non-empty
               record with a matching file name.  A valid return does not
               guarantee that the INFO file can successfully be opened,
               only that the file name exists in a non-empty record in
               the DR9 file.  Additionally, an invalid return indicates
               either that the file does not exist or that an error
               occurred while searching the DR9 file.  In any event,
               something is wrong with the input InfoFilePath or the
               DR9 file has been corrupted so we return an error.
     Inputs: InfoFilePath ( char * ) INFO file pathname
    Outputs: Function - ( short int ) Returned file existence status or
               error
                          0 == file does not exist or error
                          1 == file exists (see discussion above)
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoFileExists
(
char *InfoFilePath
)
{

      char          InfoDirPath[PATH_SIZE];
      char          UserName[USERNAME_SIZE];
      char          FileName[INTERNALNAME_SIZE];
      char          DirFilePath[PATH_SIZE];
      DirRecord     Record;
      long int      DirPosition = 0;

      if( ! ParsePathName( InfoFilePath, InfoDirPath, UserName, FileName ))
      {
           return( 0 );
      }

      sprintf( DirFilePath, "%s%s%s%s", InfoDirPath, OPSYS_SEP,
           UserName, DIR_EXT );

      StringtoLower( DirFilePath );

      if( ! DirGetRecord( DirFilePath, DirNotEMPTY, FileName,
           &Record, &DirPosition ))
      {
           return( 0 );
      }

      return( 1 );
}                                           /* end of InfoFileExists       */



/****************************************************************************
       Name: InfoReadRecord
    Purpose: This function reads a record from an open INFO file based
               on an InfoFile pointer and a record number.  The record
               is placed in the InfoFile's IOBuffer and the value for
               CurrentRecord is updated.  An error occurs if RecordNumber
               is less than or equal to zero or greater than the
               current number of records.  On function error, the
               contents of the IOBuffer and the value of CurrentRecord
               are undefined.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             RecordNumber ( long int ) Record number
    Outputs: Function - ( short int )  Returned read status
                          0 == read error
                          1 == read ok
               On success, the IOBuffer is updated with the read record
               and the value of CurrentRecord is updated -- on error,
               IOBuffer and CurrentRecord are undefined.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoReadRecord
(
      InfoFile *InfoFilePtr,
      long int RecordNumber
)
{

      if( InfoFilePtr == ( InfoFile * ) NULL )
      {
           return( 0 );
      }

      if(( InfoFilePtr->NumberRecords <= 0 ) ||
           ( RecordNumber > InfoFilePtr->NumberRecords ))
      {
           return( 0 );
      }

      if( fseek( InfoFilePtr->InfoFILE,
           ( long int ) (( InfoFilePtr->RecordLength ) *
           ( RecordNumber - 1 )), 0 ))
      {
           return( 0 );
      }

      if(( fread( InfoFilePtr->IOBuffer, InfoFilePtr->RecordLength,
           1, InfoFilePtr->InfoFILE )) != 1 )
      {
           return( 0 );
      }

      InfoFilePtr->CurrentRecord = RecordNumber;

      return( 1 );

}                                           /* end of InfoReadRecord       */


/****************************************************************************
       Name: InfoWriteRecord
    Purpose: This function writes a record to an open INFO file based
               on an InfoFile pointer and a record number.  The record
               which is written is contained in the InfoFile's IOBuffer.
               An error occurs if the INFO file has been opened for read
               (using macro InfoREAD), or RecordNumber is less than or
               equal to zero or greater than the number of records + 1.
               If the write operation creates a new record, the number
               of records is updated.  Write status is returned.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             RecordNumber ( long int ) Record number
    Outputs: Function - ( short int )  Returned write status
                          0 == write error
                          1 == write ok
               On success, the IOBuffer is written to the open INFO data
               file and the number of records is updated -- on error
               the status of the data file and the number of records are
               undefined.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoWriteRecord
(
      InfoFile *InfoFilePtr,
      long int RecordNumber
)
{

      if( InfoFilePtr == ( InfoFile * ) NULL )
      {
           return( 0 );
      }

      if( InfoFilePtr->AccessFlag == InfoREAD )
      {
           return( 0 );
      }

      if(( RecordNumber <= 0 ) ||
           ( RecordNumber > ( InfoFilePtr->NumberRecords + 1 )))
      {
           return( 0 );
      }

      if( fseek( InfoFilePtr->InfoFILE,
           ( long int ) (( InfoFilePtr->RecordLength ) *
           ( RecordNumber - 1 )), 0 ))
      {
           return( 0 );
      }

      if(( fwrite( InfoFilePtr->IOBuffer, InfoFilePtr->RecordLength,
           1, InfoFilePtr->InfoFILE )) != 1 )
      {
           return( 0 );
      }

      if( RecordNumber > InfoFilePtr->NumberRecords )
           InfoFilePtr->NumberRecords++;

      return( 1 );

}                                           /* end of InfoWriteRecord      */

/*                                             end of infofile.c           */
