/* @(#)infoitem.c	1.18 10/14/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infoitem.c
 Description: Contains infolib item functions.  The following
                functions are contained in this module:

                  InfoGetItemDef
                  InfoItemExists
                  InfoDecode
                  InfoEncode

    History: 12/91  Todd Stellhorn  - original coding
             10/92  Todd Stellhorn  - InfoDecode updated
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"

/***************************************************************************/


/****************************************************************************
       Name: InfoGetItemDef
    Purpose: This function returns a pointer to an INFO item definition
               structure InfoItemDef based on an input INFO file pointer
               and an item name.  Regular, redefined and the pseudo item
               $RECNO are supported.  An attempt is made to test if the
               InfoFilePtr is valid by testing for NULL.  NULL is
               returned if an error occurs while reading the NIT file,
               if the item does not exist or if a memory allocation error
               occurs.  On success, a valid InfoItemDef pointer is returned.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             ItemName ( char * ) INFO item name
    Outputs: Function - ( InfoItemDef * ) Returned pointer to INFO item
               structure on success or NULL if item is not found or on error.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
InfoItemDef *InfoGetItemDef
(
      InfoFile *InfoFilePtr,
      char *ItemName
)
{

      long int      NitPosition = 0;
      NitRecord     Record;
      InfoItemDef  *ItemDef = ( InfoItemDef * ) NULL;


      if( InfoFilePtr == ( InfoFile * ) NULL )
      {
           return( ItemDef );
      }

      /* test for record number pseudo item */

      if(( CompareStrings( ItemName, INFO_RECNO, NONCASE_SENSITIVE )) == 0 )
      {
           if(( ItemDef = ( InfoItemDef * )
                malloc( sizeof( InfoItemDef ))) == NULL )
           {
                return( ItemDef );
           }

           ItemDef->ItemPosition = -1;
           ItemDef->ItemWidth = -1;
           ItemDef->ItemType = INFO_RECNO_TYPE;
           ItemDef->NumberDecimals = 0;
           ItemDef->OutputWidth = 5;
           strcpy( ItemDef->ItemName, ItemName );

           return( ItemDef );  /* return here for $recno test */
      }

      if( ! NitGetRecord( InfoFilePtr->NitFilePath, ItemName,
           &Record, &NitPosition ))
      {
           return( ItemDef );
      }

      if(( ItemDef = ( InfoItemDef * )
           malloc( sizeof( InfoItemDef ))) == NULL )
      {
           return( ItemDef );
      }

      ItemDef->ItemPosition = Record.ItemPosition;
      ItemDef->ItemWidth = Record.ItemWidth;
      ItemDef->ItemType = Record.ItemType;
      ItemDef->NumberDecimals = Record.NumberDecimals;
      ItemDef->OutputWidth = Record.OutputWidth;
      strcpy( ItemDef->ItemName, ItemName );

      return( ItemDef );

}                                           /* end of InfoGetItemDef       */


/****************************************************************************
       Name: InfoItemExists
    Purpose: This function tests for the existence of an item on an open 
               INFO file.  An attempt is made to test if the InfoFilePtr 
               is valid by testing for NULL.  A successful return indicates 
               that the item exists in the NIT for the input InfoFilePtr.  
               An error occurs if the item does not exist or an error
               occurred while reading the NIT file.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             ItemName ( char * ) INFO item name
    Outputs: Function - ( short int ) Returned existence status or error
                          0 == item does not exist or error
                          1 == item exists
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoItemExists
(
      InfoFile *InfoFilePtr,
      char *ItemName
)
{

      long int      NitPosition = 0;
      NitRecord     Record;

      if( InfoFilePtr == ( InfoFile * ) NULL )
      {
           return( 0 );
      }

      if(( CompareStrings( ItemName, INFO_RECNO, NONCASE_SENSITIVE )) == 0 )
      {
           return( 1 );
      }

      if( ! NitGetRecord( InfoFilePtr->NitFilePath, ItemName, &Record,
           &NitPosition ))
      {
           return( 0 );
      }

      return( 1 );

}                                           /* end of InfoItemExists       */


/****************************************************************************
       Name: InfoDecode
    Purpose: This function decodes and returns string or double values
               from the IOBuffer pointed to by InfoFilePtr based on an
               item definition pointed to by ItemDef.  The decode process
               extracts the values from the IOBuffer and places it into
               the output buffers pointed to by StringPtr or DoublePtr.
               All supported numeric values are returned via DoublePtr.
               The following INFO data types are supported:
                 C - character (INFO_CHARACTER_TYPE)
                 I - integer (INFO_INTEGER_TYPE)
                 N - numeric (INFO_NUMERIC_TYPE)
                 B - binary (2 and 4 byte) (INFO_BINARY_TYPE)
                 F - float (4 and 8 byte) (INFO_FLOAT_TYPE)
                 $RECNO - record number pseudo item (INFO_RECNO_TYPE)
               An error occurs if an attempt is made to decode an
               unsupported data type.  On error, the contents of the 
               buffers pointed to by StringPtr and DoublePtr are
               undefined.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             ItemDef ( InfoItemDef * ) INFO item definition
             StringPtr ( char * ) Decoded string
             DoublePtr ( double * ) Decoded double
    Outputs: Function - ( short int )  Returned decode status
                          0 == decode error
                          1 == decode ok
               Decoded values (string or double) are returned.
    History: 12/91  Todd Stellhorn  - original coding
             10/92  Todd Stellhorn  - fixed binary 4 decode loop value
****************************************************************************/
short int InfoDecode
(
      InfoFile    *InfoFilePtr,
      InfoItemDef *ItemDef,
      char        *StringPtr,
      double      *DoublePtr
)
{
      switch ( ItemDef->ItemType )
      {
           case INFO_DATE_TYPE :            /* unsupported data type */

                return( 0 );

           case INFO_CHARACTER_TYPE :
           {
                strncpy( StringPtr,
                     &InfoFilePtr->IOBuffer[ItemDef->ItemPosition - 1],
                     ItemDef->ItemWidth );

                StringPtr[ItemDef->ItemWidth] = '\0';

                return( 1 );
           }

           case INFO_INTEGER_TYPE :
           case INFO_NUMBER_TYPE :
           {
                char        TmpString[TMPRECORD_SIZE];

                strncpy( TmpString,
                     &InfoFilePtr->IOBuffer[ItemDef->ItemPosition - 1],
                     ItemDef->ItemWidth );

                TmpString[ItemDef->ItemWidth] = '\0';

                *DoublePtr = atof( TmpString );

                return( 1 );
           }

           /* for binary, float and double types move each byte individually */

           case INFO_BINARY_TYPE :
           {
                switch ( ItemDef->ItemWidth )
                {
                     case 2 :
                     {
                          short int       TmpShortInt;
                          short int       k;

                          for( k = 0; k < sizeof( short int ); k++ )
                          {
                               (( char * ) &TmpShortInt )[k] =
                                    (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k];
                          }

                          *DoublePtr = ( double ) TmpShortInt;

                          return( 1 );
                     }
                     case 4 :
                     {
                          long int        TmpInt;
                          short int       k;

                          for( k = 0; k < sizeof( long int ); k++ )
                          {
                               (( char * ) &TmpInt )[k] =
                                    (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k];
                          }

                          *DoublePtr = ( double ) TmpInt;

                          return( 1 );
                     }
                     default :

                          return( 0 );
                }
           }

           case INFO_FLOATING_TYPE :
           {
                switch ( ItemDef->ItemWidth )
                {
                     case 4 :
                     {
                          float           TmpFloat;
                          short int       k;

                          for( k = 0; k < sizeof( float ); k++ )
                          {
                               (( char * ) &TmpFloat )[k] =
                                    (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k];
                          }

                          *DoublePtr = ( double ) TmpFloat;

                          return( 1 );
                     }
                     case 8 :
                     {
                          double          TmpDouble;
                          short int       k;

                          for( k = 0; k < sizeof( double ); k++ )
                          {
                               (( char * ) &TmpDouble )[k] =
                                    (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k];
                          }

                          *DoublePtr = ( double ) TmpDouble;

                          return( 1 );
                     }
                     default :

                          return( 0 );
                }

           }

           case INFO_LEADFILL_TYPE     :    /* unsupported data types */
           case INFO_PACKED_TYPE       :
           case INFO_ZEROFILL_TYPE     :
           case INFO_OVERPUNCH_TYPE    :
           case INFO_TRAILINGSIGN_TYPE :

                return( 0 );

           case INFO_RECNO_TYPE        :    /* record number pseudo item */
           {
                *DoublePtr = ( double ) InfoFilePtr->CurrentRecord;

                return( 1 );
           }

           default :

                return( 0 );
      }
}                                           /* end of InfoDecode           */


/****************************************************************************
       Name: InfoEncode
    Purpose: This function encodes string or double values into the 
               IOBuffer pointed to by InfoFilePtr based on an item 
               definition pointed to by ItemDef.  The encode process 
               updates the IOBuffer by inserting the input values.  All 
               supported numeric values are accepted by the routine using 
               DoubleVal.  The following INFO data types are supported:
                 C - character (INFO_CHARACTER_TYPE)
                 I - integer (INFO_INTEGER_TYPE)
                 N - numeric (INFO_NUMERIC_TYPE)
                 B - binary (2 and 4 byte) (INFO_BINARY_TYPE)
                 F - float (4 and 8 byte) (INFO_FLOAT_TYPE)
               An error occurs if an attempt is made to encode an
               unsupported data type or the pseudo item $RECNO.  This 
               function does not return an error is data value truncation
               occurs.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             ItemDef ( InfoItemDef * ) INFO item definition
             StringPtr ( char * ) Encode string pointer
             DoubleVal ( double  ) Encode double value
    Outputs: Function - ( short int )  Returned encode status
                          0 == encode error
                          1 == encode ok
               IOBuffer is updated with encoded values.  On error, the
               contents of the IOBuffer is undefined.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoEncode
(
      InfoFile    *InfoFilePtr,
      InfoItemDef *ItemDef,
      char        *StringPtr,
      double       DoubleVal
)
{
      switch ( ItemDef->ItemType )
      {
           case INFO_DATE_TYPE :            /* unsupported data type */

                return( 0 );

           case INFO_CHARACTER_TYPE :
           {
                FillStringBuffer( StringPtr, ItemDef->ItemPosition,
                     ItemDef->ItemWidth, InfoFilePtr->IOBuffer);

                return( 1 );
           }

           case INFO_INTEGER_TYPE :
           {
                char        TmpString[TMPSTRING_SIZE];
                char        ConvertFormat[TMPSTRING_SIZE];

                sprintf( ConvertFormat, "%%%dld", ItemDef->ItemWidth );
                sprintf( TmpString, ConvertFormat, ( long int ) DoubleVal );

                FillStringBuffer( TmpString, ItemDef->ItemPosition,
                     ItemDef->ItemWidth, InfoFilePtr->IOBuffer);

                return( 1 );
           }

           case INFO_NUMBER_TYPE :
           {
                char        TmpString[TMPSTRING_SIZE];
                char        ConvertFormat[TMPSTRING_SIZE];

                sprintf( ConvertFormat, "%%%d.%dlf", ItemDef->ItemWidth,
                     ItemDef->NumberDecimals );
                sprintf( TmpString, ConvertFormat, DoubleVal );

                FillStringBuffer( TmpString, ItemDef->ItemPosition,
                     ItemDef->ItemWidth, InfoFilePtr->IOBuffer);

                return( 1 );
           }

           /* for binary, float and double types move each byte individually */

           case INFO_BINARY_TYPE :
           {
                switch ( ItemDef->ItemWidth )
                {
                     case 2 :
                     {
                          short int       TmpShortInt;
                          short int       k;

                          TmpShortInt = ( short int ) DoubleVal;

                          for( k = 0; k < sizeof( short int ); k++ )
                          {
                               (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k] =
                                    (( char * ) &TmpShortInt )[k];
                          }

                          return( 1 );
                     }
                     case 4 :
                     {
                          long int             TmpInt;
                          short int            k;

                          TmpInt = ( long int ) DoubleVal;

                          for( k = 0; k < sizeof( long int ); k++ )
                          {
                               (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k] =
                                    (( char * ) &TmpInt )[k];
                          }

                          return( 1 );
                     }
                     default :

                          return( 0 );
                }
           }

           case INFO_FLOATING_TYPE :
           {
                switch ( ItemDef->ItemWidth )
                {
                     case 4 :
                     {
                          float           TmpFloat;
                          short int       k;

                          TmpFloat = ( float ) DoubleVal;

                          for( k = 0; k < sizeof( float ); k++ )
                          {
                               (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k] =
                                    (( char * ) &TmpFloat )[k];
                          }

                          return( 1 );
                     }
                     case 8 :
                     {
                          short int       k;

                          for( k = 0; k < sizeof( double ); k++ )
                          {
                               (( char * ) &InfoFilePtr->IOBuffer
                                    [ItemDef->ItemPosition - 1] )[k] =
                                    (( char * ) &DoubleVal )[k];
                          }

                          return( 1 );
                     }
                     default :

                          return( 0 );
                }

           }

           case INFO_LEADFILL_TYPE     :    /* unsupported data types */
           case INFO_PACKED_TYPE       :
           case INFO_ZEROFILL_TYPE     :
           case INFO_OVERPUNCH_TYPE    :
           case INFO_TRAILINGSIGN_TYPE :
           case INFO_RECNO_TYPE        :    /* $recno cannot be encoded */
           default :

                return( 0 );
      }
}                                           /* end of InfoEncode           */

/*                                             end of infoitem.c           */
