/* @(#)infoprs.c	1.7 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infoprs.c
 Description: Contains infolib parsing functions.  The following
                functions are contained in this module:

                  ParsePathName
                  FetchNextWord

    History: 12/91  Todd Stellhorn  - removed from infolib source
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"

/***************************************************************************/


/****************************************************************************
       Name: ParsePathName
    Purpose: This function parses an input pathname into its pathname
               components.  ARC/INFO style INFO file pathnames are
               supported, such as:
                 /sleet1/todd/infolib/info:arc:test
               where:
                 /sleet1/todd/infolib/info - full pathname to INFO
                   directory
                 : - INFO path separator -- ":" is used under UNIX because
                   /bin/csh uses "!" as history character.  this value is
                   set using the infodefs.h macro PATH_DELIMITER.
                 test - external name of the INFO file
               This function returns the full pathname of the INFO
               directory, the user name (default value is set using 
               DEFAULT_USER), and the external INFO file name.  If 
               pathname and user name are not contained in the input 
               string (delimited by PATH_DELIMITER) default values are added.
     Inputs: InString ( char * ) Pointer to input string
             PathName ( char * ) INFO directory pathname
             User ( char * ) INFO user name
             FileName ( char * ) INFO external filename
    Outputs: Function - ( short int ) Returned function status
                         0 == ParsePathName error
                         1 == ParsePathName ok
               Function also returns the full INFO directory pathname, the
               INFO user name, and the INFO external filename.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int ParsePathName
(
      char *InString,
      char *PathName,
      char *User,
      char *FileName
)
{

      short int    DelimiterCount = 0;
      char        *TmpStringPtr;

      TmpStringPtr = InString;

      /* count the number of path delimiters */

      while( *TmpStringPtr )
      {
           if( *TmpStringPtr == PATH_DELIMITER )
           {
                DelimiterCount++;
           }

           TmpStringPtr++;
      }

      /* switch on number of delimiters */

      switch( DelimiterCount )
      {
           case 0 :
           {
                sprintf( PathName, "%s%s%s",
                     getcwd( NULL, PATH_SIZE ), OPSYS_SEP, DEFAULT_PATH );
                strcpy( User, DEFAULT_USER );
                strcpy( FileName, InString );

                break;
           }
           case 1 :
           {
                TmpStringPtr =
                      FetchNextWord( InString, PATH_DELIMITER, PathName );
                strcpy( User, DEFAULT_USER );
                strcpy( FileName, TmpStringPtr );

                break;
           }
           case 2 :
           {
                TmpStringPtr =
                     FetchNextWord( InString, PATH_DELIMITER, PathName );
                TmpStringPtr =
                     FetchNextWord( TmpStringPtr, PATH_DELIMITER, User );
                FetchNextWord( TmpStringPtr, PATH_DELIMITER, FileName );

                break;
           }
           default :

                return( 0 );

      }

      return( 1 );

}                                           /* end of ParsePathname        */


/****************************************************************************
       Name: FetchNextWord
    Purpose: This function returns the next word based on an input string
               pointer and an input character delimiter.  A word is defined
               as one or more characters delimited by the value of
               Delimiter.  The position within the input string
               following the character delimiter is also returned.
     Inputs: StringPtr ( char * ) Input string pointer
             Delimiter ( char ) Input delimiter character
             NextWord ( char * ) Input/Returned updated string
    Outputs: Function - ( char * ) Returned next input string word
               delimited by the value of Delimiter.  Returned value is 
               placed in the character buffer NextWord.  The string 
               position following the returned word is also returned.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
char *FetchNextWord
(
      char *StringPtr,
      char  Delimiter,
      char *NextWord
)
{

     while(( *StringPtr != Delimiter ) && ( *StringPtr ))
     {
          *NextWord = *StringPtr;

          NextWord++;
          StringPtr++;
     }

     *NextWord = '\0';

     return( ++StringPtr );

}                                           /* end of FetchNextWord        */

/*                                             end of infoprs.c            */
