/* @(#)infostr.c	1.7 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infostr.c
 Description: Contains infolib string processing functions.  The
                following functions are contained in this module:

                  TerminateString
                  StringtoLower
                  StringtoUpper
                  FillStringBuffer
                  CompressString
                  ExpandString
                  CompareStrings

    History: 12/91  Todd Stellhorn  - original coding
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"

/***************************************************************************/


/****************************************************************************
       Name: TerminateString
    Purpose: This function copies the values from the string buffer
               pointed to by Source into the string buffer pointed to
               by Target.  The string buffer Target is NULL terminated
               on return.  The value of SourceSize is used to indicate
               the length of Source.  Trailing spaces (' ') are truncated
               from Target.  The function is used to convert INFO's
               non-terminated FORTRAN strings into terminated C strings.
     Inputs: Source ( char * ) String buffer
             SourceSize ( short int ) Maximum size of non-terminated
               string pointed to by Source
             Target ( char * ) Return buffer for terminated string
    Outputs: Function - ( void ) Terminated string pointed to by Target.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void TerminateString
(
      char      *Source,
      short int  SourceSize,
      char      *Target
)
{

      short int         i;

      for( i = 0; i < SourceSize; i++)
      {
           if( Source[i] == ' ' )
                break;

           Target[i] = Source[i];
      }

      Target[i] = '\0';

}                                           /* end of TerminateString      */


/****************************************************************************
       Name: StringtoLower
    Purpose: This function translates the string pointed to by
               StringBuffer to lower case.
     Inputs: StringBuffer ( char * ) Updated string buffer
    Outputs: Function - ( void ) Updated string buffer pointed to by
               StringBuffer.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void StringtoLower
(
      char *StringBuffer
)
{

      while( *StringBuffer )
      {
           *StringBuffer = tolower( *StringBuffer );
           StringBuffer++;
      }

}                                           /* end of StringtoLower        */


/****************************************************************************
       Name: StringtoUpper
    Purpose: This function translates the string pointed to by
               StringBuffer to upper case.
     Inputs: StringBuffer ( char * ) Updated string buffer
    Outputs: Function - ( void ) Updated string buffer pointed to by
               StringBuffer.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void StringtoUpper
(
      char *StringBuffer
)
{

      while( *StringBuffer )
      {
           *StringBuffer = toupper( *StringBuffer );
           StringBuffer++;
      }

}                                           /* end of StringtoUpper        */


/****************************************************************************
       Name: FillStringBuffer
    Purpose: This function places the values pointed to by InBuffer into
               the buffer pointed to by OutBuffer.  The insertion position
               within OutBuffer is input via Position and the length of
               the inserted values is input via Length.  If the length
               of InBuffer is less than Length, OutBuffer is padded with
               spaces (' ').  Function is used to insert a string of values
               into the IOBuffer.
     Inputs: InBuffer ( char * ) String buffer
             Position ( short int ) Buffer string position
             Length ( short int ) Buffer string length
             OutBuffer ( char * ) Updated string buffer
    Outputs: Function - ( void )  Updated string buffer pointed to by
               OutBuffer.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void FillStringBuffer
(
      char      *InBuffer,
      short int  Position,
      short int  Length,
      char      *OutBuffer
)
{

      short int        i;
      short int        k;

      for( i = 0, k = ( Position - 1 ); i < Length; i++, k++ )
      {
           if( i < strlen( InBuffer ))
                OutBuffer[k] = InBuffer[i];
           else
                OutBuffer[k] = ' ';
      }

}                                           /* end of FillStringBuffer     */


/****************************************************************************
       Name: CompressString
    Purpose: This function removes trailing spaces (' ') from the input
               string based on an input string width (Width) and a string
               buffer pointed to by String.  The updated string buffer
               is returned.
     Inputs: String ( char * ) String buffer
             Width ( short int ) String buffer width
    Outputs: Function - ( void ) Updated string pointed to by String.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void CompressString
(
      char      *String,
      short int  Width
)
{

     short int         i;

     for( i = ( Width - 1 ); ( i > 0 ) && ( String[i] == ' ' ); --i );

     String[ i + 1 ] = '\0';

}                                           /* end of CompressString       */


/****************************************************************************
       Name: ExpandString
    Purpose: This function pads the input string pointed to by String
               based on the value of Width.  Trailing spaces are added
               to expand the string.  An updated string buffer is returned.
     Inputs: String ( char * ) String buffer
             Width ( short int ) String buffer width
    Outputs: Function - ( void ) Updated string pointed to by String
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
void ExpandString
(
      char *String,
      short int Width
)
{

    short int        i;

     for( i = ( strlen( String )); i < Width; i++ )
          String[i] = ' ';

}                                           /* end of ExpandString         */


/****************************************************************************
       Name: CompareStrings
    Purpose: This function compares two input strings based on an input
               compare rule (CASE_SENSITIVE or NONCASE_SENSITIVE) and
               returns the string compare status.
     Inputs: String1 ( char * ) First string
             String2 ( char * ) Second sting
             CompareRule ( short int ) Compare rule (CASE_SENSITIVE or 
               NONCASE_SENSITIVE)
    Outputs: Function - ( short int ) Returned compare status
                          < 0 - String1 less than String2
                         == 0 - String1 equal to String2
                          > 0 - String1 greater than String2
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int CompareStrings
(
      char *String1,
      char *String2,
      short int CompareRule
)
{

      short int       Compare;

      while(( *String1 ) && ( *String2 ))
      {
           switch( CompareRule )
           {
                case CASE_SENSITIVE     :

                     Compare = *String1 - *String2;
                     break;

                case NONCASE_SENSITIVE  :

                     Compare =
                          ( toupper( *String1 )) - ( toupper( *String2 ));
                     break;
           }

           if( Compare )
           {
                return( Compare );
           }

           String1++;
           String2++;

      }

      Compare = ( strlen( String1 )) - ( strlen( String2 ));

      if( Compare )
      {
           return( Compare );
      }

      return( 0 );
}                                           /* end of CompareStrings       */

/*                                             end of infostr.c            */
