/* @(#)infoutil.c	1.7 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infoutil.c 
 Description: Contains infolib utility functions.  The following
                functions are contained in this module:

                  InfoBinarySearch
                  InfoSeqSearch
                  InfoGetTemplate

    History: 12/91  Todd Stellhorn  - original coding
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"


/****************************************************************************
       Name: InfoBinarySearch
    Purpose: This function performs a binary search on the input open INFO
               file based on the item definition and value of SearchString
               or SearchInt.  The lowest record number of the matching
               records and function status are returned.  Additionally,
               the values of IOBuffer and CurrentRecord pointed to by
               InfoFilePtr are updated.  The function operates only on
               INFO C, I or B item types and the pseudo item $RECNO
               (INFO_CHARACTER_TYPE, INFO_INTEGER_TYPE, INFO_BINARY_TYPE
               or INFO_RECNO_TYPE).  It is assumed that the input INFO
               file is sorted on the item pointed to by ItemDef.
               This function will produce undefined results if the
               input file is not sorted on ItemDef.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             ItemDef ( InfoItemDef * ) INFO item definition
             SearchString ( char * ) Search string
             SearchInt ( long int ) Search integer value
             RecordNumber ( long int * ) Record number
    Outputs: Function - ( short int )  Returned binary search status
                          0 == record not found or binary search error
                          1 == record found
               The lowest record number of the matching records is returned
               via RecordNumber and the values of IOBuffer and CurrentRecord
               pointed to by InfoFilePtr are updated.  On error or non-
               matching record, the values of RecordNumber, IOBuffer and
               CurrentRecord are undefined.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoBinarySearch
(
     InfoFile    *InfoFilePtr,
     InfoItemDef *ItemDef,
     char        *SearchString,
     long int     SearchInt,
     long int    *RecordNumber
)
{

     long int        MinRecord;
     long int        MaxRecord;
     long int        CurrentRecord;
     short int       CompareCode;

     if( ItemDef->ItemType == INFO_RECNO_TYPE )
     {
          if( ! InfoReadRecord( InfoFilePtr, SearchInt ))
          {
               return( 0 );
          }

          *RecordNumber = SearchInt; 

          return( 1 );
     }

     if(( ItemDef->ItemType != INFO_CHARACTER_TYPE ) && 
        ( ItemDef->ItemType != INFO_INTEGER_TYPE ) && 
        ( ItemDef->ItemType != INFO_BINARY_TYPE ))
     {
        return( 0 );
     }

     CurrentRecord = 0;
     MinRecord = 1;
     MaxRecord = InfoFilePtr->NumberRecords;

     while( 1 )
     {
          if( MinRecord > MaxRecord )
          {
              return( 0 );
          }
     
          CurrentRecord = ( MinRecord + MaxRecord ) / 2;

          if( ! InfoReadRecord( InfoFilePtr, CurrentRecord ))
          {
               return( 0 );
          }

          if( ! CompareValues( InfoFilePtr, ItemDef, SearchString, 
                SearchInt, &CompareCode ))
          {
                return( 0 );
          }
          
          if( CompareCode == 0 )
          {
               break;
          }

          if( CompareCode > 0 )
               MinRecord = CurrentRecord + 1;
          else
               MaxRecord = CurrentRecord - 1;

     }

     while( 1 )
     {
          if( CurrentRecord == 1 )
          {
               *RecordNumber = 1;
               return( 1 );
          }

          if( ! InfoReadRecord( InfoFilePtr, --CurrentRecord ))
          {
               return( 0 );
          }

          if( ! CompareValues( InfoFilePtr, ItemDef, SearchString, 
                SearchInt, &CompareCode ))
          {
                return( 0 );
          }
          
          if( CompareCode != 0 )
          {
               if( ! InfoReadRecord( InfoFilePtr, ++CurrentRecord ))
               {
                    return( 0 );
               }

               *RecordNumber = CurrentRecord;
               return( 1 );
          }
     }
}                                           /* end of InfoBinarySearch     */


/****************************************************************************
       Name: InfoSeqSearch
    Purpose: This function performs a sequential search on the input open
               INFO file based on the item definition and value of
               SearchString or SearchInt.  The record number of the
               first matching record and function status are returned.
               Additionally, the values of IOBuffer and CurrentRecord
               pointed to by InfoFilePtr are updated.  The function
               operates only on INFO C, I or B item types and the pseudo
               item $RECNO (INFO_CHARACTER_TYPE, INFO_INTEGER_TYPE,
               INFO_BINARY_TYPE or INFO_RECNO_TYPE). The function
               searches the input file by starting at the top of
               the file and sequentially searching for the required value.
     Inputs: InfoFilePtr ( InfoFile * ) INFO file pointer
             ItemDef ( InfoItemDef * ) INFO item definition
             SearchString ( char * ) Search string
             SearchInt ( long int ) Search integer value
             RecordNumber ( long int * ) Record number
    Outputs: Function - ( short int )  Returned sequential search status
                          0 == record not found or sequential search error
                          1 == record found
               The record number of the first matching record is returned
               via RecordNumber and the values of IOBuffer and CurrentRecord
               pointed to by InfoFilePtr are updated.  On error the values
               for RecordNumber, IOBuffer and CurrentRecord are undefined.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int InfoSeqSearch
(
     InfoFile    *InfoFilePtr,
     InfoItemDef *ItemDef,
     char        *SearchString,
     long int     SearchInt,
     long int    *RecordNumber
)
{

     long int        CurrentRecord;
     short int       CompareCode;

     if( ItemDef->ItemType == INFO_RECNO_TYPE )
     {
          if( ! InfoReadRecord( InfoFilePtr, SearchInt ))
          {
               return( 0 );
          }

          *RecordNumber = SearchInt; 

          return( 1 );
     }

     if(( ItemDef->ItemType != INFO_CHARACTER_TYPE ) && 
        ( ItemDef->ItemType != INFO_INTEGER_TYPE ) && 
        ( ItemDef->ItemType != INFO_BINARY_TYPE ))
     {
          return( 0 );
     }

     CurrentRecord = 1;

     while( CurrentRecord <= InfoFilePtr->NumberRecords )
     {
          if( ! InfoReadRecord( InfoFilePtr, CurrentRecord ))
          {
               return( 0 );
          }

          if( ! CompareValues( InfoFilePtr, ItemDef, SearchString, 
                SearchInt, &CompareCode ))
          {
                return( 0 );
          }
          
          if( CompareCode == 0 )
          {
               *RecordNumber = CurrentRecord;
               return( 1 );
          }

          CurrentRecord++;
     }

     return( 0 );
}                                           /* end of InfoSeqSearch        */


/****************************************************************************
       Name: InfoGetTemplate
    Purpose: This function returns a pointer to an array of type
               TemplateList based on an open INFO file pointed to by
               InfoFilePtr.  The array of type TemplateList contains
               the item definitions for the input INFO file.  Only
               real items are contained in the array -- redefined
               items are excluded.  A pointer to the TemplateList array
               is returned or NULL on error.
     Inputs: InfoFilePtr ( InfoFile * ) Input INFO file pointer
    Outputs: Function - ( TemplateList * ) Returned pointer to template 
               item list or NULL on error
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
TemplateList *InfoGetTemplate
(
      InfoFile *InfoFilePtr
)
{

      short int     i;
      char          TmpItemName[ITEMNAME_SIZE];
      NitRecord     Record;
      TemplateList *Template = ( TemplateList * ) NULL;
      FILE         *NitFile = ( FILE * ) NULL;

      if( InfoFilePtr == ( InfoFile * ) NULL )
      {
           return(( TemplateList * ) NULL );
      }

      if(( Template = ( TemplateList * ) malloc( sizeof( TemplateList )
           * InfoFilePtr->NumberItems )) == NULL )
      {
           return(( TemplateList * ) NULL );
      }

      if(( NitFile = OpenFile( InfoFilePtr->NitFilePath, InfoREAD )) == 
           ( FILE * ) NULL )
      {
           return(( TemplateList * ) NULL );
      }

      for( i = 0; i < InfoFilePtr->NumberItems; i++ )
      {
           if(( fread( &Record, sizeof( NitRecord ), 1, NitFile )) != 1 )
           {
                free( Template );
                CloseFile( NitFile );
                return(( TemplateList * ) NULL );
           }

           if( feof( NitFile ))
           {
                free( Template );
                CloseFile( NitFile );
                return(( TemplateList * ) NULL );
           }

           Template[i].ItemPosition = Record.ItemPosition;
           Template[i].ItemWidth = Record.ItemWidth;
           Template[i].ItemType = Record.ItemType;
           Template[i].NumberDecimals = Record.NumberDecimals;
           Template[i].OutputWidth = Record.OutputWidth;

           TerminateString( Record.ItemName, sizeof( TmpItemName ),
                TmpItemName );

           strcpy( Template[i].ItemName, TmpItemName );

      }

      if( ! CloseFile( NitFile ))
      {
           free( Template );
           return(( TemplateList * ) NULL );
      }

      return( Template );

}                                           /* end of InfoGetTemplate      */

/*                                             end of infoutil.c           */
