/*****************************************************************************
NAME: PROJ2DDR  
 
PURPOSE: Called by ARC2LAS to access the Arc/Info prj file and enter the
         data from it into a DDR data structure.
 
PROGRAM HISTORY:                                  
PROGRAMMER         DATE           REASON
----------         ----           ------
K.Lowell           12-93          Initial development ARC2LAS
R.White            08-94          Replaced projection-specific with 
                                  table-driven conversions.
K.Lowell           10-94          Upgraded to Arc7.0
T.Wolbrink         07-95          If non-UTM or non-StatePlane
                                    Zone_Code = 62
                                    zone-code-validity-flag = VALID
                                  If spheroid is given but not supported
                                    datum-code-validity-flag = VALID
                                  Always set increment-validity-flag = VALID
D.Hames            11-98          Needed to change Lambert_Azimuth to
                                  Lambert_Azimuthal since arc uses both

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    This program runs under the TAE/LAS environment.
    This program only runs under UNIX.

PROJECT: LAS 

ALGORITHM DESCRIPTION:
    Look for projection file in directory with same name as input grid name
    If projection file not found
      Add "c1" to end of directory name and look there for projection file
    If projection file does not exist
      Inform user and return from function
    Open the prj file under the given directory
    Read all records from the prj file
    Close prj file
    For each record read from prj file prior to "Parameter" keyword
    Interpret and save value associated with keyword
    If projection is UTM
      If zone code not specified
        Determine zone from first parameter
      Else if zone specified but no coordinates
        Modify coefficient map to suppress copying
    If non-UTM or non-StatePlane
      Zone_code <-- 62
      Zone-code-validity-flag <-- VALID
    If projection is UPS
      Enter parameters for UPS version of polar sterographic
    If spheroid not specified
      Enter "INT1909"
    Get the datum number, given the datum name
    If datum number < 0
      get the spheroid number, given the spheroid name
    Else if spheroid not specified or spheroid not supported
      Use datum code 0
    If projection is not supported by LAS
      Enter code for user defined
    Find coefficient mapping record for projection
    For each projection coefficient in DDR
    If coefficient value is to be taken from prj file
      Convert value to required format and store in DDR
    Else if radius or spheroid semi-axis is to be used
      Get value from specified spheroid and store in DDR
    Else if constant value is to be entered
      Enter specified value into DDR
    Set increment-validity-flag <-- VALID

ALGORITHM REFERENCES:
    The map projections, parameters for them, and spheroids used by
    Arc/Info are described in "Map Projections & Coordinate Management",
    (Environmental Systems Research Institute, Inc., Redlands, CA); some
    additional information is contained in the online doucmention
    distributed with the current release of Arc/Info.
*****************************************************************************/
#include <stdio.h>            /* Standard I/O definitions           */
#include <string.h>           /* String manipulation functions      */
#include <unistd.h>           /* Unix standard interfaces           */
#include <ctype.h>
#include "las.h"              /* Standard LAS include files         */
#include "math.h"             /* Standard LAS include files         */
#include "proj.h"             /* Geographic projection codes        */
#include "cproj.h"
#include "util.h"
#include "geompak.h"
#include "datum.h"            /* Contains datum infor structure     */
#include "arcproj.h"          /* Arc/Info projection information    */
#include "arctables.h"        /* Arc/Info datum and ellipsoid info  */
#include "arc2las.h"
#include "infolib/infolib.h"
 
SETPROJKN (projkn);              /* Table of projection keys and names */
SETPROJCOEFMAP (coefmap);        /* Coefficient mapping table          */

#define RECLN   128              /* Maximum length of prj file records */
#define STDRAD  6370997          /* Radius for unspecified spheroids   */

int get_datum_number
( 
    char *datum_name       /* IN: Arc datum name                    */
);
int get_spheroid_number
( 
    char *name             /* IN: Arc spheroid name                 */
);

int proj2ddr
(
    struct DDR *ddr,         /* O: DDR structure to fill              */
    long ep3flag,            /* I: 8.3 naming flag                    */
    char *workspace,         /* I: Directory holding ARC information  */
    char *ingrid             /* I: Input ARC image                    */
)
{
    char comment[RECLN];              /* Comment record from prj file        */
    char *cptr;                       /* Current field in coefmap.map        */
    char datum[CMLEN];                /* Datum, in correct meaning           */
    char *idend;                      /* End of id field in prj record       */
    char prjfile[CMLEN] = "";         /* Name of projection-data file        */
    char projkey[ARCPROJKEYLEN];      /* Arc/Info projection key             */
    char parmstart;                   /* Set when "parameter" keyword found  */
    char quadrant[12];                /* Quadrant specifier                  */
    char *recbuf;                     /* Buffer space for prj records        */
    char *recstart;                   /* Start of current prj record         */
    char spheroid[ARCSPHERNAMELEN];   /* Arc spheroid keyword                */
    char tmpdtm[CMLEN];
    char units[20];                   /* Horizontal measurement units        */
    char zunits[12];                  /* Units of vertical measurement       */
    char InfoFilePath[CMLEN];
    char Item[CMLEN];
    char stkpath[CMLEN];
    char lbprjfile[CMLEN];
    float deg;                        /* Degrees part of angle               */
    float minn;                       /* Minutes part of angle               */
    float sec;                        /* Seconds part of angle               */
    float xshift;                     /* Value of xshift from prj file       */
    float yshift;                     /* Value of yshift from prj file       */
    int  unit_code;                   /* Numeric value of projection units   */
    long dtmnum = -1;                 /* Datum number, defaulted to "NONE"   */
    long ii;                          /* Loop counter/index                  */
    long length;                      /* Length of id field or keyword       */
    long maprec;                      /* Record for multiple-map projections */
    long nalloc;                      /* Number of record buffers allocated  */
    long nrec;                        /* Number of prj records read          */
    long projndx;                     /* Projection index/code               */
    long rec;                         /* Current record of prj file          */
    long status;

    struct PROJCOEFMAP *mapptr;       /* Coeff. map record for projection    */
    struct DATUMDEF dtm_info;         /* Information about datum             */

    FILE *fp;                         /* Pointer to current input file       */

    /* Determine location of Arc/Info projection file
       ----------------------------------------------*/
    strcpy(lbprjfile,workspace);
    strcat(lbprjfile,"/");
    strcat(lbprjfile,ingrid);
    if (ep3flag ==0)
        strcat(lbprjfile,"/prj");
    else
        strcat(lbprjfile,"/prj.adf");
    if (access (lbprjfile, R_OK) == 0)
        strcpy(prjfile,lbprjfile);
    else
    {

        /* Since the prj.adf file wasn't found in the main directory for the 
           grid, check the first band directory to see if it has the prj.adf
           file.  The assumptions made are that c# is tacked on to the grid 
           name for the band directories and that all the bands have the same 
           prj.adf contents, so taking the first band's is valid. 
           ------------------------------------------------------------------*/
        strcpy(lbprjfile,workspace);
        strcat(lbprjfile,"/");
        strcat(lbprjfile,ingrid);
        if (ep3flag ==0)
            strcat(lbprjfile,"c1/prj");
        else
            strcat(lbprjfile,"c1/prj.adf");
        if (access (lbprjfile, R_OK) == 0)
            strcpy(prjfile,lbprjfile);
    }
    if (!(prjfile[0]))
    {
        strcpy(InfoFilePath,workspace);
        strcat(InfoFilePath,"/info:");
        strcat(InfoFilePath,ingrid);
        strcat(InfoFilePath,".stk");
        strcpy(Item,"GRID");
        if (getpath(InfoFilePath,Item,stkpath) != E_SUCC)
        {
            c_lasmsg(FALSE,"arc2las-grprj","Fatal error encountered");
            return(E_FAIL);
        }
        strcpy(prjfile,workspace);
        strcat(prjfile,"/");
        strcat(prjfile,stkpath);
        if (ep3flag == 0)
            strcat(prjfile,"/prj");
        if (ep3flag != 0)
            strcat(prjfile,"/prj.adf");
    }

    /* Check whether projection file exists
       ------------------------------------*/
    if (access(prjfile, R_OK) != 0)
    {
        return (E_FAIL);
    }

    /* Open the prj file
       -----------------*/
    if ((fp = fopen (prjfile, "r")) == NULL)
    {
        c_lasmsg(FALSE,"arc2las-openprj","Error opening prj file");
        return (E_FAIL);
    }

    /* Read in projection records
       --------------------------*/
    nalloc = 10;
    nrec = 0;
    recbuf = (char*) malloc (RECLN * (nalloc + 1));
    while (fgets(recbuf + RECLN * nrec, RECLN, fp) != NULL)
    {
        if (++nrec > nalloc)
        {
            nalloc += nalloc;
            recbuf = (char*) realloc (recbuf, RECLN * (nalloc+1));
        }
    }
    fclose (fp);
    if (nrec == 0)
    {
        c_lasmsg(FALSE,"arc2las-emptyprj","Projection file %s is empty", 
                prjfile);
        return (E_FAIL);
    }

    /* Prepare to extract non-parameter projection data
       ------------------------------------------------*/
    for (ii = 0; ii < 15; ii++)
        ddr->proj_coef[ii] = 0.0;
    xshift = yshift = 0;
    parmstart = FALSE;
    spheroid[0] = datum[0] = '\0';
    comment[0] = quadrant[0] = zunits[0] = '\0';

    /* Interpret records based on record identifier
       --------------------------------------------*/
    for (rec = 0; (rec < nrec) && ( ! parmstart); rec++)
    {
        recstart = recbuf + RECLN * rec;
        for (; (isalpha (*recstart) == 0)
                && (recstart < recbuf + RECLN * (rec + 1) ); recstart++);
        for (idend = recstart; (isalpha (*idend) != 0)
                && (idend < recbuf + RECLN * (rec + 1) ); idend++);
        if (idend >= recbuf + RECLN * (rec + 1))
        {
            c_lasmsg(FALSE,"arc2las-badprjrec","Cannot interpret Arc %s %s",
                    "projection record -- record ignored.  Record text is",
                    recbuf + RECLN * rec);
            continue;
        }
        *idend++ = '\0';
        for (cptr = idend + strlen(idend) - 1; isspace(*cptr); cptr--);
        *++cptr = '\0';
        length = (long)strlen(recstart);
        c_up2low(recstart,&length);

        /* Convert projection keyword to projection code
           ---------------------------------------------*/
        if (strcmp (recstart, "projection") == 0)
        {
            sscanf (idend, " %s ", projkey);                   

            /* Arcproj.h will only recognize LAMBERT_AZIMUTHAL
               as vaild, so we concatinate an 'AL' to the
               'LAMBERT_AZIMUTH'
               -----------------------------------------------*/
            if (!strcmp(projkey , "LAMBERT_AZIMUTH"))
                strcat (projkey,"AL");   

            length = (long)strlen (projkey);
            c_low2up(projkey, &length);
            for (projndx = 0; (projndx < ARCPROJCOUNT) &&
                    (strcmp(projkey,projkn[projndx].akey)) &&
                    (strcmp(projkey,projkn[projndx].name)); projndx++);
        }
        else if (strcmp (recstart, "units") == 0)
        {
            /* Process Units: record
               ---------------------*/
            ddr->valid[DDPUV] = VALID;
            sscanf(idend," %s ",units);
            length = (long)strlen(units);
            c_up2low(units,&length);

            if (strcmp(units,"radians") == 0)
                unit_code = RADIAN;
            else if (strcmp(units,"dms") == 0)
                unit_code = DMS;
            else if (strcmp(units,"feet") == 0)
                unit_code = FEET;
            else if (strcmp(units,"meters") == 0)
                unit_code = METER;
            else if (strcmp(units,"dd") == 0)
                unit_code = DEGREE;
            else if (strcmp(units,"ds") == 0)
                unit_code = SECOND;
            else
            {
                ddr->valid[DDPUV] = INVAL;
                c_lasmsg(FALSE,"arc2las-unknwnuni",
                        "LAS does not support %s \"%s\"",
                        "Arc projection units", units);
            }

            if (ddr->valid[DDPUV] == VALID)
            {
                if (c_unit2string(unit_code,ddr->proj_units) != E_SUCC)
                {
                    /* c_unit2string displays the error message */
                    c_lasmsg(FALSE,"arc2las-unit", "Setting units to invalid");
                    ddr->valid[DDPUV] = INVAL;
                }
            }
        }
        else if (strcmp (recstart, "datum") == 0)
        {
            /* Process datum code
               ------------------*/
            sscanf(idend," %s ",datum);
            length = (long)strlen(datum);
            c_low2up(datum,&length);
        }
        else if (strcmp (recstart, "spheroid") == 0)
        {
            /* Capture spheroid keyword
               ------------------------*/
            sscanf(idend," %s ",spheroid);
            length = (long)strlen(spheroid);
            c_low2up(spheroid,&length);
        }
        else if (strcmp(recstart,"zone") == 0)
        {
            /* Capture zone code
               -----------------*/
            if (sscanf (idend, " %d ", &(ddr->zone_code)) < 1)
                c_lasmsg(FALSE,"arc2las-emptyzone","%s %s", 
                        "Arc prj file contains",
                        "\"zone\" keyword but no value -- record ignored");
            else
                ddr->valid[DDZCV] = VALID;
        }
        else if (strcmp (recstart, "parameters") == 0)
        {
            /* Set flag for start of parameters
               --------------------------------*/
            parmstart = TRUE;
        }
        /* Check for user defined shifts
           -----------------------------*/
        else if (strcmp (recstart,"xshift") == 0)
            sscanf (idend, "%f", &xshift);
        else if (strcmp (recstart, "yshift") == 0)
            sscanf (idend, "%f", &yshift);
        else
        {
            /* Pick up miscellaneous entries not saved by LAS
               ----------------------------------------------*/
            if (strcmp (recstart, "zunits") == 0)
                sscanf(idend,"%s",zunits);
            else if (strcmp(recstart,"quadrant") == 0)
                sscanf(idend,"%s",quadrant);
            else if (strcmp(recstart,"comment") == 0)
                strcpy(comment,idend);
            c_lasmsg(FALSE,"arc2las-prjignord","%s \"%s%s\" -- record ignored",
                    "Unsupported prj record type: ",recstart,idend);
        }
    }

    /* Determine UTM zone if not specified explicitly
       ----------------------------------------------*/
    if (projndx == UTM)
    {
        if ((ddr->valid[DDZCV] != VALID) && (rec <= nrec))
        {
            sscanf((recbuf + RECLN * rec),"%f %f %f",&deg,&minn,&sec);
            ddr->zone_code = (long)((3600.0*(deg + 186.0) + 60.0*minn + sec)/
                    (6.0*3600));
            if ((ddr->zone_code >= 1) && (ddr->zone_code <= 60))
                ddr->valid[DDZCV] = VALID;
        }

        /* If zone specified without coords, suppress copying
           --------------------------------------------------*/
        else if (rec == nrec)
            strncpy(coefmap[UTM].map,"    ",4);
    }

    /* If Non-UTM and non-State_Plane, set zone<--62 (valid)
       -----------------------------------------------------*/
    if ((projndx != UTM) && (projndx != SPCS))
    {
        ddr->zone_code = 62;
        ddr->valid[DDZCV] = VALID;
    }

    /* Convert UPS to Polar Stereogrpahic with appropriate parameters
       --------------------------------------------------------------*/
    if (strcmp (projkey, "UNIVERSAL_POLAR") == 0)
    {
        projndx = PS;
        strcpy(recbuf,"UPSCLM");
        strcpy(recbuf + RECLN,"UPSLTS");
        strcpy(recbuf + 2 * RECLN,"UPSFE");
        strcpy(recbuf + 3 * RECLN,"UPSFN");
        rec = 0;
        nrec = 4;
        recstart = recbuf - RECLN;
        if (spheroid[0] == '\0')
            strcpy(spheroid,"INT1909");
    }

    /* convert the datum or spheroid keyword
       -------------------------------------*/
    if (*datum && strcmp(datum,"NONE"))
    {
        strcpy(tmpdtm,datum);
        dtmnum = get_datum_number(datum);
        if (dtmnum < 0)
            c_lasmsg(FALSE,"arc2las-nodatum", "Datum %s unavailable", tmpdtm);
    }
    else if (*spheroid)
    {
        strcpy(tmpdtm, spheroid);
        dtmnum = get_spheroid_number(spheroid);
        if (dtmnum < 0)
            c_lasmsg(FALSE,"arc2las-spherunkn", 
                    "Spheroid type %s not supported by LAS", tmpdtm);
    }
    else
        c_lasmsg(FALSE,"arc2las-notspec", 
                "Neither the datum nor spheroid is specified in the Arc file");

    /* Retrieve datum information 
       --------------------------*/
    if (dtmnum >= 0)
    {
        dtm_info.spherenum = dtmnum;
        dtm_info.datumnum  = dtmnum;
        if (c_getdatum(&dtm_info) != E_SUCC)
        {
            dtmnum = -1;
            ddr->datum_code = 0;
            ddr->valid[DDDCV] = INVAL;
            dtm_info.datumnum = -1;
            dtm_info.spherenum = -1;
            c_lasmsg(FALSE,"arc2las-nodatum", "Datum %s unavailable", tmpdtm);
        }
        else
        {
            ddr->datum_code = dtmnum;
            ddr->valid[DDDCV] = VALID;
        }
    }
    else
    {
        dtm_info.spherenum = dtmnum;
        dtm_info.datumnum  = dtmnum;
        ddr->datum_code = 0;
        ddr->valid[DDDCV] = INVAL;
    }

    /* Check whether projection is supported by LAS
       --------------------------------------------*/
    if (projndx > MAXPROJ)
    {
        if (projndx < ARCPROJCOUNT)
        {
            c_lasmsg(FALSE,"arc2las-prjnotlas",
                    "Projection %s:  \"%s\" %s",projkey,projkn[projndx].name,
                    "not supported by LAS -- entering "
                    "projection as \"user defined\"");
        }
        else
        {
            c_lasmsg(FALSE,"arc2las-prjnotlas",
                    "File %s contains \"Projection\" %s",prjfile,
                    "keyword, but no value -- entering "
                    "projection as \"user defined\"");
        }
        ddr->proj_code = USDEF;
    }
    else
    {
        ddr->proj_code = projndx;
        ddr->valid[DDPCV] = VALID;
    }

    /* Find coeffcient mapping record
       ------------------------------*/
    if (projndx < ARCPROJCOUNT)
    {
        mapptr = &coefmap[projndx];
        if (mapptr->contcnt > 0)
        {
            sscanf((recbuf + rec * RECLN),"%d",&maprec);
            maprec--; 
            while (maprec--)
            {
                for (mapptr = &coefmap[ARCPROJCOUNT];
                     mapptr < coefmap + ARCPROJCOUNT + EXTRATBLRECS; mapptr++)
                {
                    if (!strcmp(coefmap[projndx].lkey,mapptr->lkey))
                        break;
                }
            }
        }
        if (mapptr->contcnt < 0)
        {
            c_lasmsg(FALSE,"arc2las-nocoefmap","%s %s %s","Mapping of Arc",
                    "projection parameters to LAS projection coefficents",
                    "not defined -- not entered into DDR");
            projndx = ARCPROJCOUNT;
        }
    }

    /* Enter projection coefficients into DDR
       --------------------------------------*/
    if (projndx < ARCPROJCOUNT)
    {
        ddr->valid[DDPPV] = VALID;
        for (ii = 0, cptr = mapptr->map; ii < COEFCT; ii++, cptr += 2)
            switch (*cptr)
            {
                /* Get an angle value
                   ------------------*/
                case 'a':
                    status = sscanf((recstart + ( *(cptr + 1) - '0') * RECLN),
                            "%f %f %f",&deg,&minn,&sec);
                    if (status != 3)
                    {
                        status=sscanf((recstart + ( *(cptr + 1) - '0') * RECLN),
                                "%f %f %f",&deg,&minn,&sec);
                        if (status == 1)
                        {
                            minn = (deg - (float)((int)(deg)))*60.0;
                            deg = (float)((int)(deg));
                            sec = (minn - (float)((int)(minn)))*60.0;
                            minn = (float)((int)(minn));
                        }
                        else      
                        {
                            c_lasmsg(FALSE,"arc2las-badangle",
                                    "Misformatted angle parameter");
                            ddr->valid[DDPPV] = INVAL;
                            break;
                        }
                    }
                    if (deg > 0)
                        ddr->proj_coef[ii] = 1000000.0 * deg + 1000.0 * minn 
                            + sec;
                    else
                        ddr->proj_coef[ii] = -(1000000.0 * fabs(deg) + 1000.0 *
                                minn + sec);
                    break;

                    /* Get a simple numeric value
                       --------------------------*/
                case 'd':
                    status = sscanf((recstart + ( *(cptr + 1) - '0') * RECLN),
                            "%lf", &(ddr->proj_coef[ii]));
                    if (status != 1)
                    {
                        c_lasmsg(FALSE,"arc2las-badnumber",
                                "Invalid numeric parameter");
                        ddr->valid[DDPPV] = INVAL;
                    }
                    break;

                    /* Insert spheroid semi-axis
                       -------------------------*/
                case 'j':
                    if (ddr->valid[DDDCV] == INVAL)
                        ddr->proj_coef[ii] = STDRAD;
                    else
                        ddr->proj_coef[ii] = dtm_info.smajor;
                    break;
                case 'n':
                    if (ddr->valid[DDDCV] == INVAL)
                        ddr->proj_coef[ii] = STDRAD;
                    else
                        ddr->proj_coef[ii] = dtm_info.sminor;
                    break;

                    /* Insert constant value
                       ---------------------*/
                case 'o':
                    ddr->proj_coef[ii] = 1.0;
                    break;
                case 't':
                    ddr->proj_coef[ii] = 2.0;
                    break;
                case 'z':
                    ddr->proj_coef[ii] = 0.0;
                    break;
                case 'r':   /* Zero tells LAS it's std sphere */
                case ' ':   /* Coefficient not used */
                    break;
            }
    }

    /* Set increment-validity-flag to VALID
       ------------------------------------*/
    ddr->valid[DDINCV] = VALID;

    if ((dtm_info.spherenum != 0) && (dtm_info.spherenum != 1) &&
            (dtm_info.spherenum !=2) && (dtm_info.spherenum != 24) &&
            (dtm_info.spherenum != 26))
    {
        if (xshift != 0)
            ddr->proj_coef[6] = xshift;
        if (yshift != 0)
            ddr->proj_coef[7] = yshift;
    }

    free(recbuf);
    return (E_SUCC);
}


/****************************************************************************
FUNCTION:  get_datum_number

PURPOSE:   retrieves the datum number matching the Arc datum name

INPUT:     name - Arc datum name 

OUTPUT:    none

RETURNS:   datum number if valid, else -1, meaning "not found" or "invalid"
*****************************************************************************/
int get_datum_number
( 
    char *name                          /* IN: Arc datum name               */
)
{
    int i;                                /* Counter                          */
    int list_total;                       /* Total number of elements in list */

    /* Return "invalid" if not a valid datum number
       --------------------------------------------*/
    if (!strcmp(name, "NOTDEFINED"))
        return -1;

    /* Return the datum number if found in datum list
       ----------------------------------------------*/
    list_total = num_arc_datums();   
    for (i=0; i<list_total; i++)
    {
        if (!strcmp(arc_datums[i], name))
            return i+100;
    }
    return -1;
} /* get_datum_number */


/****************************************************************************
FUNCTION:  get_spheroid_number

PURPOSE:   retrieves the spheroid number matching the Arc spheroid name

INPUT:     name - Arc spheroid name 

OUTPUT:    none

RETURNS:   spheroid number if valid, else -1, meaning "not found" or "invalid"
*****************************************************************************/
int get_spheroid_number
( 
    char *name                          /* IN: Arc spheroid name            */
)
{
    int i;                              /* Counter                          */
    int list_total;                     /* Total number of elements in list */

    /* Return "invalid" if not a valid spheroid number
       ----------------------------------------------*/
    if (!strcmp(name, "NOTDEFINED"))
        return -1;

    /* Return the spheroid number if found in spheroid list
       ----------------------------------------------------*/
    list_total = num_arc_spheroids();
    for (i=0; i<list_total; i++)
    {
        if (!strcmp(arc_spheroids[i], name))
            return i;
    }
    return -1;
} /* get_spheroid_number */
