/******************************************************************************
NAME:        ASCII2DDR

PURPOSE:     Converts an ASCII DDR file to binary format.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
EXIT_SUCCESS    Successful completion
EXIT_FAILURE    An error occurred during processing

PROGRAM HISTORY:
$Log: ascii2ddr.c,v $
Revision 1.10  2004/02/13 16:42:46  dlabahn
Make 1 minor change (remove a space) in order to commit ascii2ddr changes.
For some reason it did not release properly the first time.

Revision 1.9  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.8.2.1  2001/11/21 19:52:48  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.8  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.

PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Initialize TAE interface
   Get user specified parameters and TAE globals with get_par
   Call parse_ascddr to retrieve DDR and BDDR field values from the ASCII file.
   Set the signal handler for the standard TAE signals
   Write the band independent fields
   Write the band dependent fields for each band
   Reset the signal handlers
   Write completion message and exit
******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

/* Define the global variables that are used by the cleanup routine
-------------------------------------------------------------------*/
int  g_delflg;                 /* Value of env variable $DELFLG              */
char g_outfile[CMLEN];         /* Output binary DDR filename in host format  */

int main()
{
    char   *ext_ptr = NULL;       /* Pointer for modifying the output file
                                     name for use by c_lused                 */
    char   in_hpath[CMLEN];       /* Host path for input ASCII DDR filename  */
    char   mock_outfile[CMLEN];   /* Mock output file name with .img
                                     extension for use with c_lused          */
    char   out_files[1][CMLEN];   /* out_hpath name for c_complt             */
    char   out_hpath[CMLEN];      /* Host path for output DDR filename       */
    char   system[12];            /* Computer system data is on from value
                                     of global $DATASYS variable             */
    int    i;                     /* Looping counter                         */
    long   one = 1;               /* For c_complt call                       */
    long   std_out = 0;           /* Logical unit number for standard output */
    struct DDR ddr;               /* DDR structure                           */
    struct BDDR *bddrs;           /* BDDR structures                         */
    struct PARBLK vblock;         /* TAE parameter block                     */

    /* Initialization of parameter block
    ------------------------------------*/
    c_inlas(&vblock, &std_out);

    /* Get user specified parameters and TAE globals
    ------------------------------------------------*/
    if(get_par(&vblock, in_hpath, out_hpath, system, &g_delflg) != E_SUCC)
    {
        c_lasmsg(TRUE, "ascii2ddr-fatal", "Fatal error encountered");
    }

    /* Set g_outfile variable for use by the cleanup routine
    --------------------------------------------------------*/
    strcpy(g_outfile, out_hpath);

    /* Call parse_ascddr to retrieve DDR and BDDR field values from the
       ASCII DDR file.  It validates each value and places them into
       the DDR and BDDR structures.
    -------------------------------------------------------------------*/
    if(parse_ascddr(in_hpath, system, &ddr, &bddrs) != E_SUCC)
    {
        c_lasmsg(TRUE, "ascii2ddr-fatal", "Fatal error encountered");
    }

    /* Set the signal handler for the standard TAE signals so that they
       are caught and we cleanup all the files that have been created
    --------------------------------------------------------------------*/
    if (c_install_cleanup_handler(signal_cleanup) != E_SUCC)
    {
        free(bddrs);
        c_lasmsg(FALSE, "ascii2ddr-signal", "Error setting the signal "
                "handlers for the TAE standard signals");
        c_lasmsg(TRUE, "ascii2ddr-fatal", "Fatal error encountered");
    }

    /* Write the band independent fields to the binary DDR file
    -----------------------------------------------------------*/
    if(c_putddr(out_hpath, &ddr) != E_SUCC)
    {
        free(bddrs);
        c_lasmsg(FALSE, "ascii2ddr-write", "Error writing to DDR file");
        cleanup_files(out_hpath);
        c_lasmsg(TRUE, "ascii2ddr-fatal", "Fatal error encountered");
    }

    /* Write the band dependent fields to the binary DDR file
    ---------------------------------------------------------*/
    for(i = 0; i < ddr.nbands; i++)
    {
        if(c_putbdr(out_hpath, &bddrs[i]) != E_SUCC)
        {
            free(bddrs);
            c_lasmsg(FALSE, "ascii2ddr-write", "Error writing to DDR file");
            cleanup_files(out_hpath);
            c_lasmsg(TRUE, "ascii2ddr-fatal", "Fatal error encountered");
        }
    }
    free(bddrs);

    /* Set the last modified date/time in the DDR to current date/time.
       First copy the output file name and change the "ddr" extension
       to "img" so we can use c_lused to accomplish this.
    -------------------------------------------------------------------*/
    strcpy(mock_outfile, out_hpath);

    /* Find last '.' in the filename to get a pointer to the file extension
    -----------------------------------------------------------------------*/
    ext_ptr = strrchr(mock_outfile, '.');
    ext_ptr++; /* pointing at "ddr" */
    strcpy(ext_ptr, "img"); /* overwrite "ddr" with "img" */
    if(c_lused(mock_outfile) != E_SUCC)
    {
        c_lasmsg(FALSE, "ascii2ddr-lastmod",
                "Error updating last modified date/time in DDR");
    }

    /* Reset the signal handlers. Errors are not caught since messages
       are printed by the routine and we don't want to terminate the
       program if this fails
    ------------------------------------------------------------------*/
    c_remove_cleanup_handler();

    /* Write completion message and exit
    ------------------------------------*/
    strcpy(out_files[0], out_hpath);
    c_complt("ASCII2DDR", &out_files[0], &one);
    return(EXIT_SUCCESS);
}
