/******************************************************************************
NAME:          GET_CORN_COORDS

PURPOSE:       Retrieves the corner coordinates' validity flag and the
               corner coordinates from the ASCII DDR file. It then
               validates the values and sets the values appropriately in
               the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_corn_coords.c,v $
Revision 1.7  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.6.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.6  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Corner coordinates are not validated
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

int get_corn_coords
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char   line_buf[PAGE_WIDTH];   /* Temp line buffer to hold file data     */
    char   temp_str[46];           /* Used to eat up the X/Y or LAT/LONG     */
    char   val_flg_str[13];        /* Validity flag (string)                 */
    double corn_coords[8];         /* Corner coordinates in the order:
                                      UL lat
                                      UL long
                                      UR lat
                                      UR long
                                      LL lat
                                      LL long
                                      LR lat
                                      LR long                             */
    int    status;                 /* Function return status                 */
    long   val_flg_int = -1;       /* Validity flag (integer)                */

    /* Next line contains the corner coordinates' validity flag
       --------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " CORNER COOR:%46cValid:%12s", temp_str,
            val_flg_str);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving corner coordinates validity"
                " flag; set to INVALID");
        val_flg_int = INVAL;
    }

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(val_flg_int != INVAL)
    {
        if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
        {
            c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for corner coordinates");
            c_lasmsg(FALSE, "ascii2ddr-modflag",
                    "Corner coordinates validity flag being set to UNKNOWN");
        }
    }

    /* Next 4 lines contain the corner coordinates values
       --------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " ULcorner:%lE %lE", &corn_coords[0],
            &corn_coords[1]);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving upper left corner coordinates;"
                " set to 0.0");
        corn_coords[0] = corn_coords[1] = 0.0;
    }

    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " URcorner:%lE %lE", &corn_coords[2],
            &corn_coords[3]);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving upper right corner coordinates;"
                " set to 0.0");
        corn_coords[2] = corn_coords[3] = 0.0;
    }

    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " LLcorner:%lE %lE", &corn_coords[4],
            &corn_coords[5]);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving lower left corner coordinates;"
                " set to 0.0");
        corn_coords[4] = corn_coords[5] = 0.0;
    }

    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " LRcorner:%lE %lE", &corn_coords[6],
            &corn_coords[7]);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving lower right corner coordinates;"
                " set to 0.0");
        corn_coords[6] = corn_coords[7] = 0.0;
    }

    /* No validation done beyond setting them to 0.0 if validity flag is inval
       -----------------------------------------------------------------------*/
    if(val_flg_int == INVAL)
    {
        /* The flag was INVALID so set the coordinates to 0.0
           --------------------------------------------------*/
        ddr->upleft[0] = ddr->upleft[1] = 0.0;
        ddr->upright[0] = ddr->upright[1] = 0.0;
        ddr->loleft[0] = ddr->loleft[1] = 0.0;
        ddr->loright[0] = ddr->loright[1] = 0.0;

        ddr->valid[DDCCV] = val_flg_int;
    }
    else
    {
        /* Carry over the corner coordinates
           ---------------------------------*/
        ddr->upleft[0] = corn_coords[0];
        ddr->upleft[1] = corn_coords[1];
        ddr->upright[0] = corn_coords[2];
        ddr->upright[1] = corn_coords[3];
        ddr->loleft[0] = corn_coords[4];
        ddr->loleft[1] = corn_coords[5];
        ddr->loright[0] = corn_coords[6];
        ddr->loright[1] = corn_coords[7];

        ddr->valid[DDCCV] = val_flg_int;
    }

    return(E_SUCC);
}
