/******************************************************************************
NAME:          GET_DATUM_CODE

PURPOSE:       Retrieves the datum code and its validity flag from the
               ASCII DDR file. It then validates their values and sets the
               values appropriately in the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_datum_code.c,v $
Revision 1.10  2004/02/12 20:15:26  shimitz
Committing branch changes for CR 8779.

Revision 1.9.4.1  2003/12/31 18:53:27  shimitz
Updated to accept a single integer value for both the datum and projection codes
as a fall-back if the pattern matching of "(%d) %s" doesn't work. Also updated
to allow a single blank line to end the ascii file and not throw an error about
too many band records being in the file. CR 8779

Revision 1.9  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.8.2.2  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.8.2.1  2001/11/14 14:28:23  shimitz
Fixed an error in the sscanf that retrieved the datum code. Changed
get_zone_code to accept negative zone codes ([-60,-1]) as valid. These are
issues that were identified by Kelly F. during testing. CR 2841

Revision 1.8  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Datum code must be a valid datum code used by LAS else its validity flag
      will be set to INVAL
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "ascii2ddr.h"

int get_datum_code
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char line_buf[PAGE_WIDTH];     /* Temp line buffer to hold file data     */
    char dat_str[47];              /* Datum code string                      */
    char val_flg_str[13];          /* Validity flag (string)                 */
    int  ind;                      /* Looping index                          */
    int  status;                   /* Function return status                 */
    long dat_len;                  /* Length of datum code string            */
    long dat_num;                  /* Datum code                             */
    long val_flg_int = -1;         /* Validity flag (integer)                */

    /* Next line contains the datum code and its validity flag
       -------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " DATUM CODE:%46c Valid:%12s", dat_str,
            val_flg_str);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving datum code validity flag; set"
                " to INVALID");
        val_flg_int = INVAL;
        strcpy(dat_str, "");
    }

    /* Chop all trailing white space off the datum string
       and then put the \0 on the end of the string
       --------------------------------------------------*/
    for(ind = 45; ind >= 0; ind--)
    {
        if(isspace((int)dat_str[ind]) == 0)
            break;
    }
    dat_str[ind + 1] = '\0';

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(val_flg_int != INVAL)
    {
        if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
        {
            c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for datum code");
            c_lasmsg(FALSE, "ascii2ddr-modflag",
                    "Datum code validity flag being set to UNKNOWN");
        }
    }

    /* Make datum string all uppercase for comparisons
       -----------------------------------------------*/
    dat_len = (long)strlen(dat_str);
    c_low2up(dat_str, &dat_len);

    /* Retrieve the datum number from inside the parenthesis and
       validate the datum code if its validity flag is "valid"
       ---------------------------------------------------------*/
    if(strcmp(dat_str, "") != 0)
    {
        status = sscanf(dat_str, "(%ld)%*s", &dat_num);
        if(status != 1)
        {
            /* Try to match just an integer value by itself
               --------------------------------------------*/
            status = sscanf(dat_str, "%ld", &dat_num);
            if (status != 1)
            {
                c_lasmsg(FALSE, "ascii2ddr-retddr",
                        "Error encountered retrieving datum code; set to 0");
                dat_num = 0;
            }
        }
    }

    if(val_flg_int == INVAL)
    {
        /* The flag was INVALID so set the code to 0
           -----------------------------------------*/
        ddr->datum_code = 0;
        ddr->valid[2] = val_flg_int;
    }
    else if(val_flg_int == UNKNOW)
    {
        /* Carry over the datum code without validation
           --------------------------------------------*/
        ddr->datum_code = dat_num;
        ddr->valid[2] = val_flg_int;
    }
    else
    {
        /* The validity flag was VALID, so go ahead and validate the
           datum code.  This could represent a spheroid as well, so
           valid range will be [0,MAX_DATUM].
           ---------------------------------------------------------*/
        if((dat_num < 0) || (dat_num > MAX_DATUM))
        {
            /* Invalid datum code
               ------------------*/
            c_lasmsg(FALSE, "ascii2ddr-inval",
                    "Invalid datum code; validity flag set to INVALID");
            ddr->valid[2] = INVAL;
        }
        else
            ddr->valid[2] = val_flg_int;

        /* Copy the value to the DDR structure
           -----------------------------------*/
        ddr->datum_code = dat_num;
    }

    return(E_SUCC);
}
