/******************************************************************************
NAME:          GET_INC

PURPOSE:       Retrieves the line/sample increments and their validity flag
               from the ASCII DDR file. It then validates their values and
               sets the values appropriately in the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_inc.c,v $
Revision 1.9  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.8.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.8  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Line/sample increments must be greater than 0 else their values will be
      set to 1.0 and their validity flag set to UNKNOW
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

int get_inc
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char   line_buf[PAGE_WIDTH];  /* Temp line buffer to hold file data      */
    char   val_flg_str[13];       /* Validity flag (string)                  */
    double inc[2];                /* Increments (line,sample)                */
    int    status;                /* Function return status                  */
    long   val_flg_int = -1;      /* Validity flag (integer)                 */

    /* Next line contains the line/sample increments and their validity flag
       ---------------------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " INCREMENT:%lE %lE Valid:%12s", &inc[0], &inc[1],
            val_flg_str);
    if(status != 3)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving line/sample validity flag;"
                " set to INVALID");
        val_flg_int = INVAL;
    }

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(val_flg_int != INVAL)
    {
        if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
        {
            c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for line/samp increments");
            c_lasmsg(FALSE, "ascii2ddr-modflag",
                    "Increment validity flag being set to UNKNOWN");
        }
    }

    /* Validate the line/sample increment if its validity flag is "valid"
       ------------------------------------------------------------------*/
    if(val_flg_int == INVAL)
    {
        /* Set the line/sample increments to 0.0
           -------------------------------------*/
        ddr->line_inc = 0.0;
        ddr->sample_inc = 0.0;
        ddr->valid[DDINCV] = val_flg_int;
    }
    else if(val_flg_int == UNKNOW)
    {
        /* Carry over the line/sample increments without validation
           --------------------------------------------------------*/
        ddr->line_inc = inc[0];
        ddr->sample_inc = inc[1];
        ddr->valid[DDINCV] = val_flg_int;
    }
    else
    {
        /* The validity flag was VALID, so go ahead and validate the
           line/sample increments. If they are invalid, set them to 1.0
           ------------------------------------------------------------*/
        if((inc[0] < 0) || (inc[1] < 0))
        {
            /* Invalid line/sample increments
               ------------------------------*/
            c_lasmsg(FALSE, "ascii2ddr-inc",
                    "Invalid increment; set to 1.0 and validity flag set to"
                    " UNKNOWN");
            ddr->valid[DDINCV] = UNKNOW;

            /* Set the increment values to 1.0
               -------------------------------*/
            ddr->line_inc = 1.0;
            ddr->sample_inc = 1.0;
        }
        else
        {
            ddr->valid[DDINCV] = val_flg_int;

            /* Copy the values to the DDR structure
               ------------------------------------*/
            ddr->line_inc = inc[0];
            ddr->sample_inc = inc[1];
        }
    }

    return(E_SUCC);
}
