/******************************************************************************
NAME:          GET_MAST_COORDS

PURPOSE:       Retrieves the master line/sample from the ASCII DDR file.
               It then validates the values and sets them appropriately
               in the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_mast_coords.c,v $
Revision 1.7  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.6.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.6  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Master line and master sample must be greater than or equal to 0 else
      their values will be set to 1
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ascii2ddr.h"

int get_mast_coords
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char line_buf[PAGE_WIDTH];    /* Temp line buffer to hold file data      */
    int  status;                  /* Function return status                  */
    long mast_coords[2];          /* Master coordinates (line,sample)        */

    /* Next line contains the master line/sample
       -----------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " MASTER COOR:%ld %ld", &mast_coords[0],
            &mast_coords[1]);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving master coordinates; set to 1");
        mast_coords[0] = mast_coords[1] = 1;
    }

    /* Validate the master line/sample values
       --------------------------------------*/
    if(mast_coords[0] < 0)
    {
        /* Invalid master line value
           -------------------------*/
        c_lasmsg(FALSE, "ascii2ddr-mastlin",
                "Invalid master line value; reset to 1");
        ddr->master_line = 1;
    }
    else
        ddr->master_line = mast_coords[0];

    if(mast_coords[1] < 0)
    {
        /* Invalid master sample value
           ---------------------------*/
        c_lasmsg(FALSE, "ascii2ddr-mastsam",
                "Invalid master sample value; reset to 1");
        ddr->master_sample = 1;
    }
    else
        ddr->master_sample = mast_coords[1];

    return(E_SUCC);
}
