/******************************************************************************
NAME:        GET_PAR

PURPOSE:     This function retrieves the parameters from TAE and verifies
             that they are valid.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_par.c,v $
Revision 1.7  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.6.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.6  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Get TAE global $DELFLG
   Get TAE global $DATASYS
   Get input ASCII DDR filename
   Get output binary DDR filename
   Validate the nonexistence of the output binary DDR file
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

int get_par
(
    struct PARBLK *vblock,       /* I: TAE parameter block                   */
    char   in_hpath[],           /* O: Host path to input ASCII DDR file     */
    char   out_hpath[],          /* O: Host path to output binary DDR file   */
    char   system[],             /* O: Computer system data is on from
                                       value of global $DATASYS              */
    int    *delflg               /* O: Value of env variable $DELFLG         */
)
{
//    char   *ext_ptr = NULL;   /* Pointer to extension portion of out_hpath   */
//    char   *name_ptr = NULL;  /* Pointer to file name portion of out_hpath   */
    char   infile[CMLEN];     /* User specified input binary DDR file        */
    char   temp_str[4];       /* Temp string for retrieval of delflg         */
    char   temp_hpath[CMLEN]; /* The INFILE with ".prt" appended to it       */
    int    mode = R_OK;       /* Access mode - to check if the file exists
                                 and can be read                             */
    long   acc = IWRITE;      /* Type of I/O access                          */
    long   element = 0;       /* Element of parameter                        */
    struct VARIABLE *val;     /* Pointer to TAE variable within parm block   */

    /* Get TAE global $DELFLG
       ----------------------*/
    val = p_find(vblock, "$DELFLG");
    strcpy(temp_str, SVAL(*val, 0));
    if(strncmp(temp_str, "YES", 3) == 0)
        *delflg = TRUE;
    else
        *delflg = FALSE;

    /* Get TAE global $DATASYS
       ----------------------*/
    if(c_getsys(system) != E_SUCC)
    {
        /* Don't want to stop the program for this, so just
           set it to standard "ieee-std"
           ------------------------------------------------*/
        strcpy(system, "ieee-std");
    }

    /* Make sure it isn't more than 11 characters; crop if it is.
       Don't notify the user if it's too long since this isn't a
       value being retrieved from the ASCII DDR file.
       ----------------------------------------------------------*/
    if(strlen(system) > 11)
        system[11] = '\0';

    /* Get input ASCII filename.
       -------------------------*/
    val = p_find(vblock, "INFILE");
    strcpy(infile, SVAL(*val, 0));

    /* Create full host path
       ---------------------*/
    if(c_cmhost(infile, in_hpath) != E_SUCC)
    {
        c_lasmsg(FALSE, "ascii2ddr-host", "Error creating host path name for"
                "%s", infile);
        return(E_FAIL);
    }

    /* Validate the existence of the input ASCII file
       ----------------------------------------------*/
    if(access(in_hpath, mode) != 0)
    {
        /* Before erroring out, check to see if the file exists with a
           ".prt" extension appended to it (this is the extension used
           by dspddr when its output is redirected to a file).
           -----------------------------------------------------------*/
        strcpy(temp_hpath, in_hpath);
        strcat(temp_hpath, ".prt");
        if(access(temp_hpath, mode) != 0)
        {
            c_lasmsg(FALSE, "ascii2ddr-infile",
                    "Error retrieving parameter INFILE");
            return(E_FAIL);
        }
        else
        {
            /* The file with ".prt" appended to it was found to exist. Set
               in_hpath to this filename to be used as the file to process.
               An informational message is needed since c_getnam already
               printed "Error: ddr does not exist", which isn't true now.
               ------------------------------------------------------------*/
            c_lasmsg(FALSE, "ascii2ddr-info", "Using file %s", temp_hpath);
            strcpy(in_hpath, temp_hpath);
        }
    }

    /* Get output DDR filename. c_getnam creates the full host
       path and tests for existence of the file, returning E_FAIL
       if it does not exist.
       ----------------------------------------------------------*/
    if(c_getnam(vblock, "OUT", &element, out_hpath, "ddr", &acc) != E_SUCC)
    {
        c_lasmsg(FALSE, "ascii2ddr-exist", "OUT file %s already exists",
                out_hpath);
        return(E_FAIL);
    }

    return(E_SUCC);
}
