/******************************************************************************
NAME:          GET_PROJ_CODE

PURPOSE:       Retrieves the projection code and its validity flag from the
               ASCII DDR file. It then validates their values and sets the
               values appropriately in the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_proj_code.c,v $
Revision 1.9  2004/02/12 20:15:26  shimitz
Committing branch changes for CR 8779.

Revision 1.8.4.1  2003/12/31 18:53:27  shimitz
Updated to accept a single integer value for both the datum and projection codes
as a fall-back if the pattern matching of "(%d) %s" doesn't work. Also updated
to allow a single blank line to end the ascii file and not throw an error about
too many band records being in the file. CR 8779

Revision 1.8  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.7.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.7  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Projection code must be a valid proj code used by LAS else its validity
      flag will be set to INVAL
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "proj.h"
#include "ascii2ddr.h"

int get_proj_code
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char line_buf[PAGE_WIDTH];     /* Temp line buffer to hold file data     */
    char proj_str[47];             /* Projection code string                 */
    char val_flg_str[13];          /* Validity flag (string)                 */
    int  ind;                      /* Looping index                          */
    int  status;                   /* Function return status                 */
    long proj_len;                 /* Length of projection code string       */
    long proj_num;                 /* Projection code                        */
    long val_flg_int = -1;         /* Validity flag (integer)                */

    /* Next line contains the projection code and its validity flag
       ------------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " PROJ. CODE:%46cValid:%12s", proj_str,
            val_flg_str);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection code validity string;"
                " set to INVALID");
        val_flg_int = INVAL;
        strcpy(proj_str, "");
    }

    /* Chop all trailing white space off the proj code string
       and then put the \0 on the end of the string
       ------------------------------------------------------*/
    for(ind = 45; ind >= 0; ind--)
    {
        if(isspace((int)proj_str[ind]) == 0)
            break;
    }
    proj_str[ind + 1] = '\0';

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
    {
        c_lasmsg(FALSE, "ascii2ddr-valflg",
                "Unrecognized validity flag for projection code");
        c_lasmsg(FALSE, "ascii2ddr-modflag",
                "Projection code validity flag being set to UNKNOWN");
    }

    /* Make projection code string all uppercase for comparisons
       ---------------------------------------------------------*/
    proj_len = (long) strlen(proj_str);
    c_low2up(proj_str, &proj_len);

    /* Retrieve the projection number from inside the parenthesis
       and validate the projection code if its validity flag is "valid"
       ----------------------------------------------------------------*/
    if(strcmp(proj_str, "") != 0)
    {
        status = sscanf(proj_str, "(%ld)", &proj_num);
        if(status != 1)
        {
            /* Try to match just an integer value by itself
               --------------------------------------------*/
            status = sscanf(proj_str, "%ld", &proj_num);
            if (status != 1)
            {
                c_lasmsg(FALSE, "ascii2ddr-retddr",
                        "Error encountered retrieving projection code");
                return(E_FAIL);
            }
        }
    }

    if(val_flg_int == INVAL)
    {
        /* The flag was INVALID so set the code to 0
           -----------------------------------------*/
        ddr->proj_code = 0;
        ddr->valid[DDPCV] = val_flg_int;
    }
    else if(val_flg_int == UNKNOW)
    {
        /* Carry over the projection code without validation
           -------------------------------------------------*/
        ddr->proj_code = proj_num;
        ddr->valid[DDPCV] = val_flg_int;
    }
    else
    {
        /* The validity flag was VALID, so go ahead and validate the proj
           code. Projection numbers up to 99 are allowed for user-defined
           projection codes.
           --------------------------------------------------------------*/
        if((proj_num < 0) || (proj_num > MAXPROJ))
        {
            /* Invalid projection code
               -----------------------*/
            c_lasmsg(FALSE, "ascii2ddr-inval",
                    "Invalid projection code; validity flag set to INVALID");
            ddr->valid[DDPCV] = INVAL;
        }
        else
            ddr->valid[DDPCV] = val_flg_int;

        /* Copy the value to the DDR structure
           -----------------------------------*/
        ddr->proj_code = proj_num;
    }

    return(E_SUCC);
}
