/******************************************************************************
NAME:          GET_PROJ_DIST

PURPOSE:       Retrieves the ground distance and its validity flag from the
               ASCII DDR file. It then validates their values and sets the
               values appropriately in the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_proj_dist.c,v $
Revision 1.8  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.7.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.7  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Ground distance must be greater than 0 else their validity flag will be
      set to INVAL
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

int get_proj_dist
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char   line_buf[PAGE_WIDTH];    /* Temp line buffer to hold file data    */
    char   val_flg_str[13];         /* Validity flag (string)                */
    double proj_dist[2];            /* Projection distances (y,x)            */
    int    status;                  /* Function return status                */
    long   val_flg_int = -1;        /* Validity flag (integer)               */

    /* Next line contains the ground distances and their validity flag
       ---------------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " PROJ. DIST:%lE %lE Valid:%12s", &proj_dist[0],
            &proj_dist[1], val_flg_str);
    if(status != 3)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving ground distances validity flag;"
                " set to INVALID");
        val_flg_int = INVAL;
    }

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(val_flg_int != INVAL)
    {
        if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
        {
            c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for ground distance");
            c_lasmsg(FALSE, "ascii2ddr-modflag",
                    "Ground distance validity flag being set to UNKNOWN");
        }
    }

    /* Validate the ground distance if its validity flag is "valid"
       ------------------------------------------------------------*/
    if(val_flg_int == INVAL)
    {
        /* Set the ground distances to 0.0
           -------------------------------*/
        ddr->pdist_y = 0.0;
        ddr->pdist_x = 0.0;
        ddr->valid[DDPDV] = val_flg_int;
    }
    else if(val_flg_int == UNKNOW)
    {
        /* Carry over the ground distances without validation
           --------------------------------------------------*/
        ddr->pdist_y = proj_dist[0];
        ddr->pdist_x = proj_dist[1];
        ddr->valid[DDPDV] = val_flg_int;
    }
    else
    {
        /* The validity flag was VALID, so go ahead and validate the
           ground distances
           ---------------------------------------------------------*/
        if((proj_dist[0] <= 0) || (proj_dist[1] <= 0))
        {
            /* Invalid ground distance
               -----------------------*/
            c_lasmsg(FALSE, "ascii2ddr-inval",
                    "Invalid ground distance; validity flag set to INVALID");
            ddr->valid[DDPDV] = INVAL;
        }
        else
            ddr->valid[DDPDV] = val_flg_int;

        /* Copy the values to the DDR structure
           ------------------------------------*/
        ddr->pdist_y = proj_dist[0];
        ddr->pdist_x = proj_dist[1];
    }

    return(E_SUCC);
}
