/******************************************************************************
NAME:          GET_PROJ_PARMS

PURPOSE:       Retrieves the projection parameters' validity flag and the
               projection parameters from the ASCII DDR file. It then
               validates the values and sets the values appropriately in
               the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_proj_parms.c,v $
Revision 1.8  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.7.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.7  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Projection parameters will be validated by c_check_proj_params and if
      invalid their validity flag will be set to INVAL
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "ascii2ddr.h"

int get_proj_parms
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char   line_buf[PAGE_WIDTH];     /* Temp line buffer to hold file data   */
    char   val_flg_str[13];          /* Validity flag (string)               */
    double proj_coef[15];            /* Projection parameters                */
    int    ind;                      /* Looping index                        */
    int    status;                   /* Function return status               */
    long   val_flg_int = -1;         /* Validity flag (integer)              */

    /* Next line contains the projection parameters' validity flag
       -----------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " PROJ. PARM: Valid:%12s", val_flg_str);
    if(status != 1)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection parameters validity"
                " flag; set to INVALID");
        val_flg_int = INVAL;
    }

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(val_flg_int != INVAL)
    {
        if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
        {
            c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for projection parameters");
            c_lasmsg(FALSE, "ascii2ddr-modflag",
                    "Projection parameters validity flag being set to UNKNOWN");
        }
    }

    /* Next 5 lines contain the projection parameters
       ----------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " A: %lE %lE %lE", &proj_coef[0], &proj_coef[1],
            &proj_coef[2]);
    if(status != 3)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection parameters 1-3; set"
                " to 0.0");
        proj_coef[0] = proj_coef[1] = proj_coef[2] = 0.0;
    }

    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " B: %lE %lE %lE", &proj_coef[3], &proj_coef[4],
            &proj_coef[5]);
    if(status != 3)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection parameters 4-6; set"
                " to 0.0");
        proj_coef[3] = proj_coef[4] = proj_coef[5] = 0.0;
    }

    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " C: %lE %lE %lE", &proj_coef[6], &proj_coef[7],
            &proj_coef[8]);
    if(status != 3)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection parameters 7-9; set"
                " to 0.0");
        proj_coef[6] = proj_coef[7] = proj_coef[8] = 0.0;
    }

    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " D: %lE %lE %lE", &proj_coef[9], &proj_coef[10],
            &proj_coef[11]);
    if(status != 3)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection parameters 10-12; set"
                " to 0.0");
        proj_coef[9] = proj_coef[10] = proj_coef[11] = 0.0;
    }

    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " E: %lE %lE %lE", &proj_coef[12], &proj_coef[13],
            &proj_coef[14]);
    if(status != 3)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection parameters 13-15; set"
                " to 0.0");
        proj_coef[12] = proj_coef[13] = proj_coef[14] = 0.0;
    }

    /* Validate the projection parameters if their validity flag is "valid"
       --------------------------------------------------------------------*/
    if(val_flg_int == INVAL)
    {
        /* The flag was INVALID so set the coefficients to 0.0
           ---------------------------------------------------*/
        for(ind = 0; ind < 15; ind++)
            ddr->proj_coef[ind] = 0.0;
        ddr->valid[DDPPV] = val_flg_int;
    }
    else if(val_flg_int == UNKNOW)
    {
        /* Carry over the projection coefficients without validation
           ---------------------------------------------------------*/
        for(ind = 0; ind < 15; ind++)
            ddr->proj_coef[ind] = proj_coef[ind];
        ddr->valid[DDPPV] = val_flg_int;
    }
    else
    {
        /* The validity flag was VALID, go ahead and validate the parameters
           as long as the projection code was valid
           -----------------------------------------------------------------*/
        if(ddr->valid[DDPCV]) /* validity flag for the projection code */
        {
            if(c_check_proj_params(proj_coef, ddr->proj_code) != E_SUCC)
            {
                /* Invalid projection parameters
                   -----------------------------*/
                c_lasmsg(FALSE, "ascii2ddr-inval",
                        "Invalid projection parameters; validity flag set to "
                        "INVALID");
                ddr->valid[DDPPV] = INVAL;
            }
            else
                ddr->valid[DDPPV] = val_flg_int;
        }
        else
        {
            c_lasmsg(FALSE, "ascii2ddr-projpar",
                    "Could not validate projection parameters due to invalid"
                    " projection code");
            ddr->valid[DDPPV] = val_flg_int;
        }

        /* Copy the values to the DDR structure
           ------------------------------------*/
        for(ind = 0; ind < 15; ind++)
            ddr->proj_coef[ind] = proj_coef[ind];
    }

    return(E_SUCC);
}
