/******************************************************************************
NAME:          GET_PROJ_UNIT

PURPOSE:       Retrieves the projection units and its validity flag from the
               ASCII DDR file. It then validates their values and sets the
               values appropriately in the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_proj_unit.c,v $
Revision 1.9  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.8.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.8  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Projection units must be "radian","feet","meter","second","degree", or
      "dms" (case insensitive) else its validity flag will be set to INVAL
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

int get_proj_unit
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char line_buf[PAGE_WIDTH];     /* Temp line buffer to hold file data     */
    char unit_str[46];             /* Projection units string                */
    char unit_str_cpy[46];         /* Copy of projection units string        */
    char val_flg_str[13];          /* Validity flag (string)                 */
    int  ind;                      /* Looping index                          */
    int  status;                   /* Function return status                 */
    long unit_len;                 /* Length of projection units string      */
    long val_flg_int = -1;         /* Validity flag (integer)                */

    /* Next line contains the projection units and its validity flag
       -------------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " PROJ. UNITS:%46cValid:%12s", unit_str,
            val_flg_str);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving projection units validity"
                " flag; set to INVALID");
        val_flg_int = INVAL;
        strcpy(unit_str, "");
    }

    /* If the last character read is not a space then the
       string is obviously too long
       --------------------------------------------------*/
    if(isspace((int)unit_str[45]) == 0)
    {
        c_lasmsg(FALSE, "ascii2ddr-length", "Projection units string too long; "
                "truncated to 11 chars");
        unit_str[11] = '\0';
    }
    else
    {
        /* Chop all trailing white space off the projection units string
           and then put the \0 on the end of the string
           -------------------------------------------------------------*/
        for(ind = 44; ind >= 0; ind--)
        {
            if(isspace((int)unit_str[ind]) == 0)
                break;
        }
        unit_str[ind + 1] = '\0';
    }

    /* Check if the string exceeds its max length allowed by the DDR structure
       -----------------------------------------------------------------------*/
    unit_len = (long) strlen(unit_str);
    if(unit_len > 11)
    {
        c_lasmsg(FALSE, "ascii2ddr-length", "Projection units string too long; "
                "truncated to 11 chars");
        unit_str[11] = '\0';
    }

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(val_flg_int != INVAL)
    {
        if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
        {
            c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for projection units");
            c_lasmsg(FALSE, "ascii2ddr-modflag",
                    "Projection units validity flag being set to UNKNOWN");
        }
    }

    /* Make units strings all uppercase for comparisons.
       Don't want to actually modify the units string,
       so make a copy of it.
       -----------------------------------------------*/
    strcpy(unit_str_cpy, unit_str);
    c_low2up(unit_str_cpy, &unit_len);

    /* Validate the projection units if its validity flag is "valid"
       -------------------------------------------------------------*/
    if(val_flg_int == INVAL)
    {
        /* The flag was INVALID so set the units to an empty string
           --------------------------------------------------------*/
        strcpy(ddr->proj_units, "");
        ddr->valid[DDPUV] = val_flg_int;
    }
    else if(val_flg_int == UNKNOW)
    {
        /* Carry over the projection code without validation
           -------------------------------------------------*/
        strcpy(ddr->proj_units, unit_str);
        ddr->valid[DDPUV] = val_flg_int;
    }
    else
    {
        /* The validity flag was VALID, so go ahead and validate the units
           ---------------------------------------------------------------*/
        if((strstr(unit_str_cpy, "RAD") == NULL) &&
                (strstr(unit_str_cpy, "FEET") == NULL) &&
                (strstr(unit_str_cpy, "METER") == NULL) &&
                (strstr(unit_str_cpy, "SEC") == NULL) &&
                (strstr(unit_str_cpy, "DEG") == NULL) &&
                (strstr(unit_str_cpy, "DMS") == NULL))
        {
            /* Invalid projection units
               ------------------------*/
            c_lasmsg(FALSE, "ascii2ddr-inval",
                    "Invalid projection units; validity flag set to INVALID");
            ddr->valid[DDPUV] = INVAL;
        }
        else
            ddr->valid[DDPUV] = val_flg_int;

        /* Copy the value to the DDR structure
           -----------------------------------*/
        strcpy(ddr->proj_units, unit_str);
    }

    return(E_SUCC);
}
