/******************************************************************************
NAME:          GET_ZONE_CODE

PURPOSE:       Retrieves the zone code and its validity flag from the
               ASCII DDR file. It then validates their values and sets the
               values appropriately in the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: get_zone_code.c,v $
Revision 1.7  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.6.2.2  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.6.2.1  2001/11/14 14:28:23  shimitz
Fixed an error in the sscanf that retrieved the datum code. Changed
get_zone_code to accept negative zone codes ([-60,-1]) as valid. These are
issues that were identified by Kelly F. during testing. CR 2841

Revision 1.6  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Validity flag must be "invalid","valid", or "unknown" (case insensitive)
      else it will be set to UNKNOW
   Zone code must be in the set {[-60,-1] (indicates imagery is in southern
      hemisphere), [1,60], 62} if proj code is UTM or in the range [101,5400]
      if projcode is SPCS else its validity flag will be set to INVAL
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "proj.h"
#include "util.h"
#include "ascii2ddr.h"

int get_zone_code
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char line_buf[PAGE_WIDTH];    /* Temp line buffer to hold file data      */
    char val_flg_str[13];         /* Validity flag (string)                  */
    int  status;                  /* Function return status                  */
    long zone_num;                /* Zone code                               */
    long val_flg_int = -1;        /* Validity flag (integer)                 */

    /* Next line contains the zone code and its validity flag
       ------------------------------------------------------*/
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " ZONE CODE:%ld Valid:%12s", &zone_num,
            val_flg_str);
    if(status != 2)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving zone code validity flag; set to"
                " INVALID");
        val_flg_int = INVAL;
    }

    /* Determine the integer value of the string validity flag
       -------------------------------------------------------*/
    if(val_flg_int != INVAL)
    {
        if(det_val_flag(val_flg_str, &val_flg_int) != E_SUCC)
        {
            c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for zone code");
            c_lasmsg(FALSE, "ascii2ddr-modflag",
                    "Zone code validity flag being set to UNKNOWN");
        }
    }

    /* Validate the zone code if its validity flag is "valid"
       ------------------------------------------------------*/
    if(val_flg_int == INVAL)
    {
        /* Set the code to 62
           ------------------*/
        ddr->zone_code = 62;
        ddr->valid[DDZCV] = val_flg_int;
    }
    else if(val_flg_int == UNKNOW)
    {
        /* Carry over the zone code without validation
           -------------------------------------------*/
        ddr->zone_code = zone_num;
        ddr->valid[DDZCV] = val_flg_int;
    }
    else
    {
        /* The validity flag was VALID, so validate the zone code.
           -------------------------------------------------------*/
        if(((ddr->valid[DDPCV]) && (ddr->proj_code == UTM) &&
                    ((zone_num < -60) || (zone_num > 60) || (zone_num == 0)) &&
                    (zone_num != 62)) || /* for UTM valid set is
                                            {[-60,-1],[1,60],62} */
                ((ddr->valid[DDPCV]) && (ddr->proj_code == SPCS) &&
                 ((zone_num < 101) || (zone_num > 5400)))) /* for SPCS valid
                                                              range is
                                                              [101,5400] */
        {
            /* Invalid zone code
               -----------------*/
            c_lasmsg(FALSE, "ascii2ddr-inval",
                    "Invalid zone code; validity flag set to INVALID");
            ddr->valid[DDZCV] = INVAL;
        }
        else
            ddr->valid[DDZCV] = val_flg_int;

        /* Copy the value to the DDR structure. If projection is not UTM
           or SPCS, just copy the value anyway, since it is inconsequential
           ----------------------------------------------------------------*/
        ddr->zone_code = zone_num;
    }

    return(E_SUCC);
}
