/******************************************************************************
NAME:        PARSE_ASCDDR

PURPOSE:     Retrieves DDR and BDDR field values from the ASCII DDR file,
             validates the values, and places them into the DDR and BDDR
             structures.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: parse_ascddr.c,v $
Revision 1.9  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.8.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.8  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Open the ASCII DDR file for read access
   Call read_criticals
   Call read_noncriticals
   Set system to value of $DATASYS
   Call read_bddrs
   Close the ASCII DDR file
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "ascii2ddr.h"

int parse_ascddr
(
    const char  hpath[CMLEN], /* I: Host path to input DDR file to parse     */
    const char  system[12],   /* I: Computer system data is on from value of
                                   global $DATASYS variable                  */
    struct DDR  *ddr,         /* O: Output DDR structure with DDR values
                                   retrieved from the ASCII DDR file         */
    struct BDDR **bddrs       /* O: Output BDDR structure with Band DDR
                                   values retrieved from the ASCII DDR file  */
)
{
    FILE *fp = NULL;   /* File pointer for the ASCII DDR file                */

    /* Open ASCII DDR file with read access
       ------------------------------------*/
    fp = fopen(hpath, "r");
    if(fp == NULL)
    {
        c_lasmsg(FALSE, "ascii2ddr-open", "Error opening file %s",
                hpath);
        return(E_FAIL);
    }

    /* Call read_criticals to read the critical DDR field values - those
       values that need to be present and valid to make the DDR at all
       useable. Invalid values will cause a fatal error
       ------------------------------------------------------------------*/
    if(read_criticals(fp, ddr) != E_SUCC)
    {
        c_lasmsg(FALSE, "ascii2ddr-parse", "Error retrieving critical DDR"
                " fields");

        /* Close ASCII DDR file
           --------------------*/
        fclose(fp);

        return(E_FAIL);
    }

    /* Call read_noncriticals to read the noncritical DDR field values
       ---------------------------------------------------------------*/
    if(read_noncriticals(fp, ddr) != E_SUCC)
    {
        c_lasmsg(FALSE, "ascii2ddr-parse", "Error retrieving non-critical DDR"
                " fields");

        /* Close ASCII DDR file
           --------------------*/
        fclose(fp);

        return(E_FAIL);
    }

    /* Set system string to value of $DATASYS variable
       -----------------------------------------------*/
    strcpy(ddr->system, system);

    /* Call read_bddrs to read the band records from the ASCII DDR file
       ----------------------------------------------------------------*/
    if(read_bddrs(fp, ddr, bddrs) != E_SUCC)
    {
        c_lasmsg(FALSE, "ascii2ddr-parse", "Error retrieving Band DDR fields");

        /* Close ASCII DDR file
           --------------------*/
        fclose(fp);

        return(E_FAIL);
    }

    /* Close the ASCII DDR file
       ------------------------*/
    if(fclose(fp) != 0)
    {
        c_lasmsg(FALSE, "ascii2ddr-close", "Error closing the ASCII DDR file");
        return(E_FAIL);
    }

    return(E_SUCC);
}

/******************************************************************************
NAME:		DET_VAL_FLAG

PURPOSE:	Determines the integer value of the validity flag from the
		string value retrieved from the ASCII DDR file.

RETURN VALUE:
Type = int
Value		Description
-----		-----------
E_SUCC		Valid validity flag string found. Returning the
		comparable integer value.
E_FAIL		Validity flag string not recognized. Set the integer value
		to UNKNOW. Caller should generate warning message.

PROGRAM HISTORY:
$Log: parse_ascddr.c,v $
Revision 1.9  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.8.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.8  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:	LAS
******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"

int det_val_flag
(
    char val_flg_str[13],      /* I: Validity flag (string) */
    long *val_flg_int          /* O: Validity flag (integer) */
)
{
    long val_flg_len;          /* Length of validity flag string */

    /* Make string all uppercase for comparison
       ----------------------------------------*/
    val_flg_len = (long) strlen(val_flg_str);
    c_low2up(val_flg_str, &val_flg_len);

    /* Need the validity flag in its integer form for the DDR structure
       ----------------------------------------------------------------*/
    if(strcmp(val_flg_str, "INVALID") == 0)
        *val_flg_int = INVAL;
    else if(strcmp(val_flg_str, "VALID") == 0)
        *val_flg_int = VALID;
    else if(strcmp(val_flg_str, "UNKNOWN") == 0)
        *val_flg_int = UNKNOW;
    else
    {
        *val_flg_int = UNKNOW;
        return(E_FAIL);
    }

    return(E_SUCC);
}
