/******************************************************************************
NAME:        READ_BDDRS

PURPOSE:     Retrieves the Band DDR record fields, validates the values, and
             places them into the BDDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: read_bddrs.c,v $
Revision 1.11  2004/02/12 20:15:26  shimitz
Committing branch changes for CR 8779.

Revision 1.10.4.1  2003/12/31 18:53:27  shimitz
Updated to accept a single integer value for both the datum and projection codes
as a fall-back if the pattern matching of "(%d) %s" doesn't work. Also updated
to allow a single blank line to end the ascii file and not throw an error about
too many band records being in the file. CR 8779

Revision 1.10  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.9.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.9  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   For each band
      Read next 11 lines to retrieve the band DDR record values
         Band numbers must be in the range [1,MAXBND] else the associated BDDR
            record will not be copied to the binary DDR file
         Validity flags for BDDR min/max intensity values must be "invalid",
            "valid","bounded", or "unacceptable" else the flag will be set to
            INVAL
         Min/max intensity values have different valid ranges based on the data
            type, if not valid the validity flag will be set to INVAL
         Source is not validated
         Sensor is not validated
         Direction of capture is not validated
         Acquisition date/time is not validated
   If not at EOF, fail
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

int read_bddrs
(
    FILE   *fp,               /* I: File pointer for the ASCII DDR file      */
    struct DDR *ddr,          /* I/O: DDR structure - filled in by
                                    read_criticals and read_noncriticals
                                    prior to this routine, though its nbands
                                    value may need to be modified            */
    struct BDDR **bddrs       /* O: Output BDDR structure with BDDR values
                                    retrieved from the ASCII DDR file        */
)
{
    char   capt_dir[70];         /* Capture direction                        */
    char   data_source[70];      /* Data source                              */
    char   date[70];             /* Acquisition date                         */
    char   line_buf[PAGE_WIDTH]; /* Temporary line buffer to hold file data  */
    char   sensor[70];           /* Sensor type                              */
    char   temp_char[1];         /* Temporary character to determine if a
                                    field has a value in it                  */
    char   time[70];             /* Acquisition time                         */
    char   val_flg_str[13];      /* Validity flag (string)                   */
    double max;                  /* Maximum intensity value                  */
    double min;                  /* Minimum intensity value                  */
    int    band_no;              /* Band number                              */
    int    cur_band;             /* Counter for looping - the current band # */
    int    dtype;                /* Data type                                */
    int    ind;                  /* Looping index                            */
    int    nbands;               /* Number of bands as specified by the
                                    DDR file                                 */
    int    status;               /* Function return status                   */
    long   val_flg_int = -1;     /* Validity flag (integer)                  */
    long   val_flg_len;          /* Length of validity flag string           */
    struct BDDR *bptr = NULL;    /* Pointer to the current BDDR record       */

    /* Retrieve needed info from DDR structure
       ---------------------------------------*/
    nbands = ddr->nbands;
    dtype = ddr->dtype;

    /* Allocate band ddr structures and initialize values - initialize
       all the band numbers to the sequential ordering, but zero everything
       else or assign emtpy strings
       --------------------------------------------------------------------*/
    if((*bddrs = (struct BDDR *)malloc(nbands * sizeof(struct BDDR))) == NULL)
    {
        c_lasmsg(FALSE, "ascii2ddr-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }


    bptr = *bddrs;
    for(ind = 0; ind < nbands; ind++)
    {
        /* Initialize the current band ddr structure
           -----------------------------------------*/
        c_intbdr(bptr);
        bptr->bandno = ind + 1;
        bptr++;
    }

    /* Retrieve each band record
       -------------------------*/
    for(cur_band = 0; cur_band < nbands; cur_band++)
    {
        /* Next line contains the image name; disregard
           --------------------------------------------*/
        if(skip_line(fp, 1) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* skip_line generates message */
        }

        /* Next line contains the band number
           ----------------------------------*/
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }
        status = sscanf(line_buf, " BAND NO:%d", &band_no);
        if(status != 1)
        {
            c_lasmsg(FALSE, "ascii2ddr-retbddr",
                    "Error encountered retrieving band number; set to 0");
            band_no = 0;
        }

        /* Validate the band number - if it is not valid skip the rest
           of this Band DDR record and continue with next one
           -----------------------------------------------------------*/
        if((band_no < 1) || (band_no > nbands))
        {
            c_lasmsg(FALSE, "ascii2ddr-bandnum",
                    "Invalid band number %ld; band record #%d not processed",
                    band_no, cur_band + 1);

            /* Skip the rest of the lines in this
               band DDR record - continue with
               processing of next band record
               ----------------------------------*/
            if(skip_line(fp, 9) != E_SUCC)
            {
                free(bddrs);
                return(E_FAIL); /* skip_line generates message */
            }
            continue;
        }

        /* Otherwise, point the bptr at the appropriate band ddr structure
           and make sure the band numbers match
           ---------------------------------------------------------------*/
        else
        {
            bptr = *bddrs; /* start by pointing it at first bddr record */
            for(ind = 1; ind < band_no; ind++)
                bptr++;

            if(bptr->bandno != band_no) /* make sure pointing at correct
                                           struct */
            {
                free(bddrs);
                c_lasmsg(FALSE, "ascii2ddr-bandstr",
                        "Error placing band %ld's band record into a band ddr"
                        " structure", band_no);
                return(E_FAIL);
            }
        }

        /* Next line is blank line
           -----------------------*/
        if(skip_line(fp, 1) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* skip_line generates message */
        }

        /* Next line contains the minimum intensity value and
           its validity flag
           --------------------------------------------------*/
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }
        status = sscanf(line_buf, " MINIMUM:%lE Valid:%12s", &min, val_flg_str);
        if(status != 2)
        {
            c_lasmsg(FALSE, "ascii2ddr-retbddr",
                    "Error encountered retrieving minimum intensity validity"
                    " flag; set to INVALID (band %d)", band_no);
            val_flg_int = INVAL;
        }

        if(val_flg_int != INVAL)
        {
            /* Make validity flag uppercase for comparison
               -------------------------------------------*/
            val_flg_len = (long) strlen(val_flg_str);
            c_low2up(val_flg_str, &val_flg_len);

            /* Need the validity flag in its integer form for
               the BDDR structure
               ----------------------------------------------*/
            if(strcmp(val_flg_str, "INVALID") == 0)
                val_flg_int = INVAL;
            else if(strcmp(val_flg_str, "VALID") == 0)
                val_flg_int = VALID;
            else if(strcmp(val_flg_str, "BOUNDED") == 0)
                val_flg_int = BOUND;
            else if(strcmp(val_flg_str, "UNACCEPTABLE") == 0)
                val_flg_int = INVAL;
            else
            {
                c_lasmsg(FALSE, "ascii2ddr-valflg",
                     "Unrecognized validity flag for min intensity (band %ld)",
                     band_no);
                c_lasmsg(FALSE, "ascii2ddr-modflag",
                        "Min intensity validity flag being set to INVALID");
                val_flg_int = INVAL;
            }
        }

        /* Start the min/max validity flag off at VALID
           --------------------------------------------*/
        bptr->valid = VALID;

        /* Validate the min intensity value if its validity flag is "valid"
           or "bounded"
           ----------------------------------------------------------------*/
        if(val_flg_int == INVAL)
        {
            /* The flag was INVALID so set the minval to 0.0
               ---------------------------------------------*/
            bptr->minval = 0.0;
            bptr->valid = val_flg_int;
        }
        else
        {
            /* The validity flag was not INVALID, so go ahead and validate
               the minimum intensity value
               -----------------------------------------------------------*/
            if(((dtype == EBYTE) && ((min < CHAR_MIN) || (min > CHAR_MAX))) ||
               ((dtype == EWORD) && ((min < SHRT_MIN) || (min > SHRT_MAX))) ||
               ((dtype == ELONG) && ((min < INT_MIN) || (min > INT_MAX))))
                /* EREAL - no range checking necessary */
            {
                c_lasmsg(FALSE, "ascii2ddr-inval",
                        "Invalid min intensity; validity flag set to INVALID"
                        " (band %ld)", band_no);
                bptr->valid = INVAL;
            }
            bptr->minval = min;

            /* If the validity flag was "bounded" in the ASCII DDR, set
               it to BOUND; otherwise, leave it at VALID
               --------------------------------------------------------*/
            if(val_flg_int == BOUND)
                bptr->valid = val_flg_int;
        }

        /* Next line contains the maximum intensity value
           and its validity flag
           ----------------------------------------------*/
        val_flg_int = -1;
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }
        status = sscanf(line_buf, " MAXIMUM:%lE Valid:%12s", &max, val_flg_str);
        if(status != 2)
        {
            c_lasmsg(FALSE, "ascii2ddr-retbddr",
                    "Error encountered retrieving maximum intensity validity"
                    " flag; set to INVALID (band %d)", band_no);
            val_flg_int = INVAL;
        }

        if(val_flg_int != INVAL)
        {
            /* Make validity flag uppercase for comparison
               -------------------------------------------*/
            val_flg_len = (long) strlen(val_flg_str);
            c_low2up(val_flg_str, &val_flg_len);

            /* Need the validity flag in its integer form
               for the BDDR structure
               ------------------------------------------*/
            if(strcmp(val_flg_str, "INVALID") == 0)
                val_flg_int = INVAL;
            else if(strcmp(val_flg_str, "VALID") == 0)
                val_flg_int = VALID;
            else if(strcmp(val_flg_str, "BOUNDED") == 0)
                val_flg_int = BOUND;
            else if(strcmp(val_flg_str, "UNACCEPTABLE") == 0)
                val_flg_int = INVAL;
            else
            {
                c_lasmsg(FALSE, "ascii2ddr-valflg",
                    "Unrecognized validity flag for max intensity (band %ld)",
                    band_no);
                c_lasmsg(FALSE, "ascii2ddr-modflag",
                        "Max intensity validity flag being set to INVALID");
                val_flg_int = INVAL;
            }
        }

        /* Validate the max intensity value if its validity flag is "valid"
           or "bounded" and the min/max intensity validity flag has not been
           determined to be invalid yet.
           -----------------------------------------------------------------*/
        if(val_flg_int == INVAL)
        {
            /* The flag was INVALID so set the maximum value to 0.0
               ----------------------------------------------------*/
            bptr->maxval = 0.0;
            bptr->valid = val_flg_int;
        }
        else if(bptr->valid)
        {
            /* The validity flag was not INVALID, so go ahead and validate
               the maximum intensity value
               -----------------------------------------------------------*/
            if(((dtype == EBYTE) && ((max < CHAR_MIN) || (max > CHAR_MAX))) ||
               ((dtype == EWORD) && ((max < SHRT_MIN) || (max > SHRT_MAX))) ||
               ((dtype == ELONG) && ((max < INT_MIN) || (max > INT_MAX))))
                /* EREAL - no range checking necessary */
            {
                c_lasmsg(FALSE, "ascii2ddr-inval",
                        "Invalid max intensity; validity flag set to INVALID"
                        " (band %ld)", band_no);
                bptr->valid = INVAL;
            }
            bptr->maxval = max;

            /* If the validity flag was "bounded" in the ASCII DDR, set
               it to BOUND; otherwise leave it at VALID
               --------------------------------------------------------*/
            if(val_flg_int == BOUND)
                bptr->valid = val_flg_int;
        }

        /* If the min/max validity flag has already been changed to INVALID,
           there is no reason to validate the max value since the flag will
           be set to INVAL anyway.
           -----------------------------------------------------------------*/
        else
        {
            c_lasmsg(FALSE, "ascii2ddr-minmax", "Min intensity was invalid; Max"
                    " value not validated (band %d)", band_no);
            bptr->maxval = max;
        }

        /* Make sure the max intensity value is greater than the min
           ---------------------------------------------------------*/
        if((bptr->valid) && (bptr->maxval < bptr->minval))
        {
            c_lasmsg(FALSE, "ascii2ddr-minmax",
                    "Max value less than min; validity flag set to INVALID"
                    " (band %ld)", band_no);
            bptr->valid = INVAL;
        }

        /* Reset the validity flag for next band
        ----------------------------------------*/
        val_flg_int = -1;

        /* Next line is a blank line
           -------------------------*/
        if(skip_line(fp, 1) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* skip_line generates message */
        }

        /* Next line contains the data source - this is not validated
           ----------------------------------------------------------*/
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }

        /* First find out if there is a value in the data source field.
           This is done because using sscanf with the %s conversion
           will fail if there is no value in the field - it returns EOF
           which is unclear if there was an error reading the string or
           if there just wasn't a string to retrieve.
        ----------------------------------------------------------------*/
        status = sscanf(line_buf, " DATA SOURCE:%c", temp_char);
        if(status != 1)
        {
            /* Nothing specified, so just set it to an empty string */
            strcpy(data_source, "");
        }
        else
        {
            /* If not a blank space, retrieve the value. Retrieve the
               string as characters since the data source string commonly
               has spaces in it (e.g. "landsat 5").
            -------------------------------------------------------------*/
            if(isspace((int)temp_char[0]) == 0)
            {
                status = sscanf(line_buf, " DATA SOURCE:%69c", data_source);
                if(status != 1)
                {
                    c_lasmsg(FALSE, "ascii2ddr-retbddr",
                            "Error encountered retrieving data source; set to"
                            " empty string (band %d)", band_no);
                    strcpy(data_source, "");
                }
                else
                {
                    /* Got the string. Remove any trailing white space it
                       may have (e.g. "landsat 5      ").
                    -----------------------------------------------------*/
                    data_source[69] = '\0'; /* end the string */
                    for(ind = strlen(data_source) - 1; ind >= 0; ind--)
                    {
                       if(isspace((int)data_source[ind]) == 0)
                          break;
                    }
                    data_source[ind + 1] = '\0';

                    /* Check if the string length exceeds its maximum length
                       allowed by the band record structure
                       -----------------------------------------------------*/
                    if(strlen(data_source) > 31)
                    {
                        c_lasmsg(FALSE, "ascii2ddr-length", "Data source"
                                " string too long; truncated to 31 chars"
                                " (band %ld)", band_no);
                        data_source[31] = '\0';
                    }
                }
            }
            else
                strcpy(data_source, "");
        }
        strcpy(bptr->source, data_source);

        /* Next line contains the sensor type - this is not validated
           ----------------------------------------------------------*/
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }

        /* First find out if there is a value in the sensor field.
           This is done because using sscanf with the %s conversion
           will fail if there is no value in the field - it returns EOF.
        ----------------------------------------------------------------*/
        status = sscanf(line_buf, " SENSOR TYPE:%c", temp_char);
        if(status != 1)
        {
            /* Nothing specified, so just set it to an empty string */
            strcpy(sensor, "");
        }
        else
        {
            /* If not a blank space, retrieve the value
            -------------------------------------------*/
            if(isspace((int)temp_char[0]) == 0)
            {
                status = sscanf(line_buf, " SENSOR TYPE:%69s", sensor);
                if(status != 1)
                {
                    c_lasmsg(FALSE, "ascii2ddr-retbddr",
                             "Error encountered retrieving sensor; set to empty"
                             " string (band %d)", band_no);
                    strcpy(sensor, "");
                }
                else
                {
                    /* Check if the string length exceeds its maximum length
                       allowed by the band record structure
                       -----------------------------------------------------*/
                    if(strlen(sensor) > 31)
                    {
                        c_lasmsg(FALSE, "ascii2ddr-length",
                                "Sensor string too long; truncated to 31 chars"
                                " (band %ld)", band_no);
                        sensor[31] = '\0';
                    }
                }
            }
            else
                strcpy(sensor, "");
        }
        strcpy(bptr->instrument, sensor);

        /* Next line contains the capture direction - this is not validated
           ----------------------------------------------------------------*/
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }

        /* First find out if there is a value in the capt dir field.
           This is done because using sscanf with the %s conversion
           will fail if there is no value in the field - it returns EOF.
        ----------------------------------------------------------------*/
        status = sscanf(line_buf, "CAPT. DIRECTION:%c", temp_char);
        if(status != 1)
        {
            /* Nothing specified, so just set it to an empty string */
            strcpy(capt_dir, "");
        }
        else
        {
            /* If not a blank space, retrieve the value
            -------------------------------------------*/
            if(isspace((int)temp_char[0]) == 0)
            {
                status = sscanf(line_buf, "CAPT. DIRECTION:%69s", capt_dir);
                if(status != 1)
                {
                    c_lasmsg(FALSE, "ascii2ddr-retbddr",
                             "Error encountered retrieving capture direction;"
                             " set to empty string (band %d)", band_no);
                    strcpy(capt_dir, "");
                }
                else
                {
                    /* Check if the string length exceeds its maximum length
                       allowed by the band record structure
                    --------------------------------------------------------*/
                    if(strlen(capt_dir) > 63)
                    {
                        c_lasmsg(FALSE, "ascii2ddr-length", "Capture direction"
                                 " sting too long; truncated to 63 chars"
                                 " (band %ld)", band_no);
                        capt_dir[63] = '\0';
                    }
                }
            }
            else
                strcpy(capt_dir, "");
        }
        strcpy(bptr->direction, capt_dir);

        /* Next line contains the date of capture - validate format but not
           value
           ----------------------------------------------------------------*/
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }

        /* First find out if there is a value in the date field.
           This is done because using sscanf with the %s conversion
           will fail if there is no value in the field - it returns EOF.
        ----------------------------------------------------------------*/
        status = sscanf(line_buf, " DATE:%c", temp_char);
        if(status != 1)
        {
            /* Nothing specified, so just set it to an empty string */
            strcpy(date, "");
        }
        else
        {
            /* If not a blank space, retrieve the value
            -------------------------------------------*/
            if(isspace((int)temp_char[0]) == 0)
            {
                status = sscanf(line_buf, " DATE:%69s", date);
                if(status != 1)
                {
                    c_lasmsg(FALSE, "ascii2ddr-retbddr",
                             "Error encountered retrieving date; set to empty"
                             " string (band %d)", band_no);
                    strcpy(date, "");
                }
                else
                {
                    /* Check if the string length exceeds its maximum length
                       allowed by the band record structure
                       -----------------------------------------------------*/
                    if(strlen(date) > 9)
                    {
                        c_lasmsg(FALSE, "ascii2ddr-length",
                                "Date string too long; truncated to 9 chars"
                                " (band %ld)", band_no);
                        date[9] = '\0';
                    }

                    /* Verify the date is in the proper format
                       (dd-mmm-yy e.g. 31-dec-86)
                       ---------------------------------------*/
                    for(ind=0; ind < strlen(date); ind++)
                    {
                        switch(ind)
                        {
                            case 0:
                            case 1:
                            case 7:
                            case 8:
                                if(isdigit((int)date[ind]) == 0)
                                {
                                    c_lasmsg(FALSE, "ascii2ddr-format",
                                             "Acquisition date not in the LAS"
                                             " DDR date format (band %ld)",
                                             band_no);
                                    ind = strlen(date); /* to break out of the
                                                           for loop */
                                }
                                break;
                            case 2:
                            case 6:
                                if(date[ind] != '-')
                                {
                                    c_lasmsg(FALSE, "ascii2ddr-format",
                                             "Acquisition date not in the LAS"
                                             " DDR date format (band %ld)",
                                             band_no);
                                    ind = strlen(date); /* to break out of the
                                                           for loop */
                                }
                                break;
                            case 3:
                            case 4:
                            case 5:
                                if(isalpha((int)date[ind]) == 0)
                                {
                                    c_lasmsg(FALSE, "ascii2ddr-format",
                                             "Acquisition date not in the LAS"
                                             " DDR date format (band %ld)",
                                             band_no);
                                    ind = strlen(date); /* to break out of the
                                    for loop */
                                }
                                break;
                        }
                    }
                }
            }
            else
                strcpy(date, "");
        }
        strcpy(bptr->date, date);

        /* Next line contains the time of capture - validate format but not
           value
           ----------------------------------------------------------------*/
        if(read_line(fp, line_buf) != E_SUCC)
        {
            free(bddrs);
            return(E_FAIL); /* read_line generates message */
        }

        /* First find out if there is a value in the time field.
           This is done because using sscanf with the %s conversion
           will fail if there is no value in the field - it returns EOF.
        ----------------------------------------------------------------*/
        status = sscanf(line_buf, " TIME:%c", temp_char);
        if(status != 1)
        {
            /* Nothing specified, so just set it to an empty string */
            strcpy(time, "");
        }
        else
        {
            /* If not a blank space, retrieve the value
            -------------------------------------------*/
            if(isspace((int)temp_char[0]) == 0)
            {
                status = sscanf(line_buf, " TIME:%69s", time);
                if(status != 1)
                {
                    c_lasmsg(FALSE, "ascii2ddr-retbddr",
                             "Error encountered retrieving time; set to empty"
                             " string (band %d)", band_no);
                    strcpy(time, "");
                }
                else
                {
                    /* Check if the string length exceeds its maximum length
                       allowed by the band record structure
                       -----------------------------------------------------*/
                    if(strlen(time) > 7)
                    {
                        c_lasmsg(FALSE, "ascii2ddr-length",
                                 "Time string too long; truncated to 7 chars"
                                 " (band %ld)", band_no);
                        time[7] = '\0';
                    }

                    /* Verify the time is in the proper format
                       (e.g. 1:05:55 represented as 1305:55). '-' and ' '
                       are valid place holders.
                       --------------------------------------------------*/
                    for(ind=0; ind < strlen(time); ind++)
                    {
                        if(ind == 4)
                        {
                            if(time[ind] != ':')
                            {
                                c_lasmsg(FALSE, "ascii2ddr-format",
                                        "Acquisition time not in the LAS DDR"
                                        " time format (band %ld)", band_no);
                                break;
                            }
                        }
                        else
                        {
                            if((isdigit((int)time[ind]) == 0) &&
                               (time[ind] != '-') && (time[ind] != ' '))
                            {
                                c_lasmsg(FALSE, "ascii2ddr-format",
                                        "Acquisition time not in the LAS DDR"
                                        " time format (band %ld)", band_no);
                                break;
                            }
                        }
                    }
                }
            }
            else
               strcpy(time, "");
        }
        strcpy(bptr->time, time);

        /* Clear out the data source string for next time. This needs
           to be done because the data source is retrieved with the
           %c conversion code.
        -------------------------------------------------------------*/
        memset(data_source, ' ', 70);
    }

    /* If not at EOF, warning (excess band ddr records)
       ------------------------------------------------*/
    if(fgets(line_buf, PAGE_WIDTH, fp) != NULL)
    {
        /* Successful read of next line, which indicates we're
           not at the end of the file yet and there are more
           band records than was indicated by the number of bands
           field in the beginning of the DDR. Sometimes there could
           be a blank line at end - only error if we find another
           line after this.
           ------------------------------------------------------*/
        if(strcmp(line_buf,"") != 0)
        {
            /* The line is not just a blank line like may be found at
               the end of a band record. This could be another, extra
               band record, or it could just be junk, but we cannot
               be too careful in attempting to cipher out junk.
               ------------------------------------------------------*/
            c_lasmsg(FALSE, "ascii2ddr-bandrec", "Excess band records may"
                    " exist in the ASCII DDR file");
        }
        else
        {
            /* Found a blank line, only error if there is another
               line following this in the file still.
               --------------------------------------------------*/
            if(fgets(line_buf, PAGE_WIDTH, fp) != NULL)
            {
                /* Still found another line, so now go ahead and error
                   ---------------------------------------------------*/
                c_lasmsg(FALSE, "ascii2ddr-bandrec", "Excess band records"
                        " may exist in the ASCII DDR file");
            }
        }
    }

    /* Otherwise determine if fgets didn't return NULL because of EOF
       or because of a general error
       --------------------------------------------------------------*/
    else
    {
        /* Check to see if the NULL was caused by an EOF, which means the
           file contained the same number of band records as was indicated
           by the number of bands field in the beginning of the DDR
           ---------------------------------------------------------------*/
        if(feof(fp) != 0) /* fgets failed because EOF reached */
            return(E_SUCC);
        else /* fgets just had a general failure */
        {
            c_lasmsg(FALSE, "ascii2ddr-read",
                    "Error reading from ASCII DDR file");
            return(E_FAIL);
        }
    }
    return(E_SUCC);
}
