/******************************************************************************
NAME:        READ_CRITICALS

PURPOSE:     Retrieves critical DDR field values from the ASCII DDR file,
             validates the values, and places them into the DDR structure. If
             any of these values are not valid the appliation will exit with a
             fatal error.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: read_criticals.c,v $
Revision 1.8  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.7.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.7  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Read first 2 lines of the ASCII DDR file, containing the critical DDR info
      Number of lines must be greater than 0 else fatal error
      Number of samples must be greater than 0 else fatal error
      Number of bands must be in the range [1,MAXBND] else fatal error
      Data type must be "byte","integer*2","integer*4",or "real*4" (case
         insensitive) else fatal error
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "ascii2ddr.h"

int read_criticals
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    char dtype[10];            /* Data type                                  */
    char line_buf[PAGE_WIDTH]; /* Temporary line buffer to hold file data    */
    int  status;               /* Function return status                     */
    long dlen;                 /* Length of data type string                 */
    long int_dtype;            /* Data type in its integer form              */
    long nb;                   /* Number of bands                            */
    long nl;                   /* Number of lines                            */
    long ns;                   /* Number of samples                          */

    /* Skip first line containing image name.
       --------------------------------------*/
    if(skip_line(fp, 1) != E_SUCC)
        return(E_FAIL); /* no message needed since skip_line generates one */

    /* Next line contains number of lines, number of samples, number of
       bands, and data type
       ----------------------------------------------------------------*/  
    if(read_line(fp, line_buf) != E_SUCC)
        return(E_FAIL); /* no message needed since read_line generates one */
    status = sscanf(line_buf, " NL:%ld NS:%ld NB:%ld DTYPE:%9s", &nl, &ns, &nb,
            dtype);
    if(status != 4)
    {
        c_lasmsg(FALSE, "ascii2ddr-retddr",
                "Error encountered retrieving number of lines, number of"
                " samples, number of bands, and data type");
        return(E_FAIL);
    }

    /* Validate the values
       -------------------*/
    if(nl < 1)
    {
        c_lasmsg(FALSE, "ascii2ddr-invalid", "Invalid number of lines");
        return(E_FAIL);
    }
    else
        ddr->nl = nl;

    if(ns < 1)
    {
        c_lasmsg(FALSE, "ascii2ddr-invalid", "Invalid number of samples");
        return(E_FAIL);
    }
    else
        ddr->ns = ns;

    if((nb < 1) || (nb > MAXBND))
    {
        c_lasmsg(FALSE, "ascii2ddr-invalid", "Invalid number of bands");
        return(E_FAIL);
    }
    else
        ddr->nbands = nb;

    /* Make dtype string all uppercase for comparison
       ----------------------------------------------*/
    dlen = (long)strlen(dtype);
    c_low2up(dtype, &dlen);

    /* Need the data type in its integer form for the DDR structure
       ------------------------------------------------------------*/
    if(strcmp(dtype, "BYTE") == 0)
        int_dtype = EBYTE;
    else if(strcmp(dtype, "INTEGER*2") == 0)
        int_dtype = EWORD;
    else if(strcmp(dtype, "INTEGER*4") == 0)
        int_dtype = ELONG;
    else if(strcmp(dtype, "REAL*4") == 0)
        int_dtype = EREAL;
    else
    {
        c_lasmsg(FALSE, "ascii2ddr-invalid", "Invalid data type");
        return(E_FAIL);
    }
    ddr->dtype = int_dtype;

    return(E_SUCC);
}
