/******************************************************************************
NAME:        READ_NONCRITICALS

PURPOSE:     Retrieves non-critical DDR field values from the ASCII DDR file,
             validates the values, and places them into the DDR structure.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: read_noncriticals.c,v $
Revision 1.5  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.4.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.4  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS

ALGORITHM DESCRIPTION:
   Skip line containing last modified date/time
   Call get_proj_code to retrieve and validate the projection code
   Call get_zone_code to retrieve and validate the zone code
   Call get_datum_code to retrieve and validate the datum code
   Call get_proj_parms to retrieve and validate the projection parameters
   Call get_corn_coords to retrieve the corner coordinates
   Call get_proj_dist to retrieve and validate the ground distance
   Call get_proj_unit to retrieve and validate the projection units
   Call get_inc to retrieve and validate the line/sample increments
   Call get_mast_coords to retrieve and validate the master line/sample
******************************************************************************/
#include "las.h"
#include "ascii2ddr.h"

int read_noncriticals
(
    FILE   *fp,             /* I: File pointer for the ASCII DDR file        */
    struct DDR *ddr         /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
)
{
    /* Skip the line containing the last modified date/time
       ----------------------------------------------------*/
    if(skip_line(fp, 1) != E_SUCC)
        return(E_FAIL); /* no message needed since skip_line generates one */

    /* Retrieve and validate the projection code and its validity flag
       ---------------------------------------------------------------*/
    if(get_proj_code(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_proj_code generates them */

    /* Retrieve and validate the zone code and its validity flag
       ---------------------------------------------------------*/
    if(get_zone_code(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_zone_code generates them */

    /* Retrieve and validate the datum code and its validity flag
       ----------------------------------------------------------*/  
    if(get_datum_code(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_datum_code generates them */

    /* Retrieve and validate the projection parameters and their validity flag
       -----------------------------------------------------------------------*/
    if(get_proj_parms(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_proj_parms generates them */

    /* Retrieve and validate the corner coordinates and their validity flag
       --------------------------------------------------------------------*/
    if(get_corn_coords(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_corn_coords generates them */

    /* Retrieve and validate the ground distance and its validity flag
       ----------------------------------------------------------------*/
    if(get_proj_dist(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_proj_dist generates them */

    /* Retrieve and validate the projection units and their validity flag
       ------------------------------------------------------------------*/
    if(get_proj_unit(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_proj_unit generates them */

    /* Retrieve and validate the line/sample incs and their validity flag
       ------------------------------------------------------------------*/
    if(get_inc(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_inc generates them */

    /* Retrieve and validate the master line/sample values
       ---------------------------------------------------*/
    if(get_mast_coords(fp, ddr) != E_SUCC)
        return(E_FAIL); /* no message since get_mast_coords generates them */

    return(E_SUCC);
}
