/******************************************************************************
PROGRAM:        asterhdf2las

PURPOSE:        Converts ASTER HDF image file to LAS image file(s).

RETURN VALUE:
Type = int
Value           Description
-----           -----------
EXIT_SUCCESS    Successful completion
EXIT_FAILURE    An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "mfhdf.h"
#include "worgen.h"
#include "diskio.h"
#include "util.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"
#include "process_hdf_image.h"
#include "get_metadata.h"

#define NUM_DDR_FLAGS DDNVAL       /* the count of flags in ddr */

/* Define the global variables that are used by the cleanup routine when
   the SIGKILL or SIGTERM signals are caught
-------------------------------------------------------------------------*/
int g_num_processed_files = 0;/* Number of files processed for output       */
char g_processed_files[NUMFILES][CMLEN];
                             /* Array of filenames that have been created 
                                for output from ASTERHDF2LAS                */
int g_delflg = FALSE;        /* Flag to represent the global DELFLG flag
                                (TRUE or FALSE)                             */
int main()
{
    char acquisition_date[20];/* Capture date                                */
    char acquisition_time[20];/* Capture time                                */
    char instrument_name[32]; /* Instrument name ie ASTER                    */
    char bands[NUMBANDS][3];  /* Bands to be processed                       */
    char infile[1][CMLEN];    /* Input HDF filename in host format. It's 
                                 2 dimensional to avoid type casts           */
    char intermediate_files[3][CMLEN]; /* Intermediate files needed for
                                          SDP Toolkit                        */
    char kinetic_temp[4];     /* Flag indicating if the product is ast08
                                 kinetic temperature                         */
    char outfiles[NUMFILES][CMLEN]; /* Output LAS filename(s) in host format */
    char projection_units[8]; /* Projection units (degrees or meters)        */
    char subdirectory[CMLEN]; /* Name of subdirectory which stores the
                                 temporary intermediate files                */
    char units[4];            /* Value from units field                      */
    double corner_coordinates[8];/* Corner coordinates in either degrees for
                                    product level 1A or DEM or meters for
                                    the level 1B or 2                        */ 
    double projection_values[15]; /* Projection parameters of the scene      */
    double window[4];         /* Window values requested by user             */
    int band_groupings[NUMBANDS]; /* The grouping of bands. Each band grouping
                                     represents one output LAS image         */
    int data_type[NUMBANDS];  /* Data type of each band                      */
    int ddr_valids[NUM_DDR_FLAGS]; /* Flags indicating if fields in the DDR are
                                      valid or not (=0 not valid, =1 valid)  */
    int index;                /* indexing variable used for looping          */
    int nl[NUMBANDS];         /* Array of total number of lines              */
    int ns[NUMBANDS];         /* Array of total number of samples            */
    int num_band_groupings;   /* Count of band_groupings                     */
    int num_bands;            /* Count of bands                              */
    int num_outs;             /* Count of outfiles                           */
    int num_window;           /* Count of window values                      */
    int process_all_bands;    /* Flag indicating if all bands in file should
                                 be processed (TRUE or FALSE)                */
    int product_level;        /* Values can be Level 1A=L1A,  Level 1B=L1B,
                                 Level 2 =L2 or Level 3 DEM =L3              */
    int projection_code;      /* Value will be either TM=9, LAMCC=4, PS=6,
                                 EQRECT=17, SOM=22, GEO=0                    */
    int projection_distance[NUMBANDS]; /* Resolution of each band            */
    int window_specified;     /* Flag indicating if the window field was
                                 specified                                   */
    int32 data_set_index_num[NUMBANDS]; /* Index number of the data set      */
    int32 sd_id;              /* SD interface identifier                     */
    int32 sds_index[NUMBANDS];/* Index of data set                           */
    long long_num_band_groupings; /* Count of band_groupings with long value */
    long msgtim;              /* Time between display messages               */
    long std_out = 0;         /* Standard output                             */
    long zero = 0;            /* Constant of 0. This is needed for c_hstout  */
    long zone;                /* Zone needed for UTM projection              */
    struct PARBLK vblock;     /* TAE parameter block                         */

    /* Initialize TAE interface
    --------------------------*/
    c_inlas (&vblock, &std_out);

    /* Get user specified parameters and TAE globals
    -----------------------------------------------*/
    if (get_par(&vblock, infile[0],outfiles,&num_outs, bands, &num_bands, 
               band_groupings, &num_band_groupings, window, &num_window,
               units, &g_delflg, &msgtim, &process_all_bands, 
               &window_specified) != E_SUCC)
    {
        c_lasmsg (TRUE, "asterhdf2las-parm", 
                  "Error parsing the TAE parameter block");
    }

    /* Open HDF file for read access
    -------------------------------*/
    sd_id = SDstart(infile[0], DFACC_READ);
    if (sd_id == HDF_FAIL)
    {   
        c_lasmsg(TRUE, "asterhdf2las-file", "Error opening file %s", 
                 infile[0]);
    }

    /* Initialize ddr_valids array to invalid
    ----------------------------------------*/
    for (index = 0; index < NUM_DDR_FLAGS; index++) 
        ddr_valids[index] = INVAL;

     /* Set the signal handler for the standard TAE signals so that they
        are caught and we cleanup all the files that have been created
     --------------------------------------------------------------------*/
    if (c_install_cleanup_handler (signal_cleanup) != E_SUCC)
    {
        c_lasmsg (TRUE, "asterhdf2las-signal", "Error setting the signal "
                  "handlers for the TAE standard signals");
    }

    /* Get metadata information
    --------------------------*/
    if (get_metadata(sd_id,infile[0],outfiles,num_outs,bands,num_bands,
                     band_groupings,&num_band_groupings,process_all_bands,
                     sds_index, &product_level, ddr_valids, acquisition_date,
                     acquisition_time,corner_coordinates,projection_values,
                     &projection_code, data_set_index_num, projection_distance,
                     projection_units,instrument_name,data_type,nl,ns, 
                     kinetic_temp,intermediate_files,subdirectory,
                     &zone) != E_SUCC)
    {
        for (index = 0; index < 3; index++)
        {
            if (c_dkdel(intermediate_files[index]) != E_SUCC)
                c_lasmsg(FALSE, "asterhdf2las-warn","Error deleting %s file",
                         intermediate_files[index]);
        }
        if (rmdir(subdirectory) != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-warn", "Error removing the "
                     "subdirectory %s", subdirectory);
        }
        c_lasmsg(TRUE, "asterhdf2las-meta", "Fatal error encountered");
    }

    /* Delete temporary files and the temporary subdirectory
    -------------------------------------------------------*/
    for (index = 0; index < 3; index++)
    {
        if (c_dkdel(intermediate_files[index]) != E_SUCC)
            c_lasmsg(FALSE, "asterhdf2las-warn","Error deleting %s file",
                     intermediate_files[index]);
    }

    if (rmdir(subdirectory) != E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-warn", "Error removing the "
                 "subdirectory %s", subdirectory);
    }
    

    /* Read and process the HDF image, apply windowing if specified
       and write to the LAS image(s).
    --------------------------------------------------------------*/
    if (process_hdf_image(sd_id,sds_index,window,projection_code,
                      projection_values,corner_coordinates,projection_distance,
                      projection_units,ddr_valids,outfiles,band_groupings,nl,
                      ns,infile[0],window_specified,num_band_groupings,
                      data_type, msgtim,units,num_window,bands,
                      instrument_name,acquisition_date,acquisition_time,
                      product_level,kinetic_temp,zone,g_processed_files,
                      &g_num_processed_files) != E_SUCC) 
    {
        if (cleanup_files(&g_num_processed_files,g_processed_files,
                          g_delflg)!=E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-warn", 
                     "Error deleting LAS image file");
        }
        c_lasmsg(TRUE, "asterhdf2las-proc",
                 "Error encountered while processing image");
    }

    /* Create a history file for each outfile 
    ----------------------------------------*/
    for (index=0; index < num_band_groupings; index++)
    {
        if(c_hstout(&vblock,outfiles[index],infile, &zero, " ") != E_SUCC)
        c_lasmsg(FALSE,"asterhdf2las-warn","Error creating history record %s",
                 outfiles[index]);
    }

    /* Remove our user-defined cleanup handlers
    -----------------------------------------*/
    if (c_remove_cleanup_handler () != E_SUCC)
        c_lasmsg(TRUE,"asterhdf2las-hand","Error from remove_cleanup_handler");

    /* Write completion message
    -------------------------*/
    /* copy to a long to avoid type casts */
    long_num_band_groupings = num_band_groupings; 

    c_complt("ASTERHDF2LAS", outfiles, &long_num_band_groupings);

    return SUCCESS;
}

