/******************************************************************************
NAME:           convert_proj_parms_coordinates

PURPOSE:        This function converts the projection parameters from 
                radians to DMS. It also converts the corner coordinates 
                from degrees (lat lon) to meters (Y X). 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "PGS_MET.h"
#include "PGS_tk.h"
#include "mfhdf.h"
#include "PGS_SMF.h"
#include "proj.h"
#include "cproj.h"
#include "geompak.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

int convert_proj_parms_coordinates
(
    double corner_coordinates[], /* I/O: Input lat/lon in degrees, output 
                                         y/x in meters                      */
    double projection_values[],  /* I/O: Input angle values in radians output 
                                         in degrees/minutes/seconds for all
                                         projections except geographic      */
    int projection_code,         /* I: Projection of the scene              */
    long zone                    /* I: Zone number for UTM                   */
)
{
    double dms = 0.0;            /* output angle value in
                                    degrees/minutes/seconds                 */
    double y_x_coordinates[NUM_COORDINATES];  
                                 /* corner coordinates in y x meters        */
    int index;                   /* looping index                           */
    int start_index;             /* beginning index number used for looping */ 
    struct DATUMDEF datuminfo;   /* structure for datum information         */

    /* The semi minor axis in the metadata portion of the HDF file rounds
       up to the nearest 10th. A more accurate semi minor is needed to
       initialize the projection and datum information for c_trans. 
    -------------------------------------------------------------------------*/
    if (projection_code != UTM) /* minor is not listed in UTM parameters */
    {
        datuminfo.datumnum = WGS84; 
        if (c_getdatum(&datuminfo) != E_SUCC)
        {
            c_lasmsg(FALSE,"asterhdf2las-axis",
                     "Error returning semi minor axis from LASTABLES");
            return (E_FAIL);
        }
        projection_values[1] = datuminfo.sminor; 
    }

    /* All angles in the projection values array need to be in DMS. This only
       applies to LAMCC, PS, TM, EQRECT and SOM.  First convert those angles
       from radians to degrees. 
    ------------------------------------------------------------------------*/
    if (projection_code != UTM)
    {
        /* The angles will only be located in projection_values[2]-[5]. The 
        exception is TM which scale factor (at projection_values[2]) does not
        need to be converted*/
        if (projection_code == TM)
            start_index = 3;
        else
            start_index = 2;

        for (index = start_index; index <= 5; index++)  
        {
            if (projection_values[index] != 0.0)
            {
                projection_values[index] = projection_values[index] * R2D;
            }
        }
 
        /* Now convert angles in the projection values array from degrees to 
           DMS, depending on the projection
        -------------------------------------------------------------------*/
        if (projection_code == LAMCC)
        {
            if (c_degdms (&projection_values[2], &dms,"DEG", "LAT") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms", 
                     "Error converting standard parallel 1 from degree to DMS");
            }
            projection_values[2] = dms;

            if (c_degdms (&projection_values[3], &dms,"DEG", "LAT") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms", 
                     "Error converting standard parallel 2 from degree to DMS");
            }
            projection_values[3] = dms;

            if (c_degdms (&projection_values[4], &dms,"DEG", "LON") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms",
                         "Error converting longitude from degree to DMS");
            }
            projection_values[4] = dms;

            if (c_degdms (&projection_values[5], &dms,"DEG", "LAT") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms",
                         "Error converting latitude from degree to DMS");
            }
            projection_values[5] = dms;
        }

        else if (projection_code == PS)
        {
            if (c_degdms (&projection_values[4], &dms,"DEG", "LON") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms",
                         "Error converting longitude from degree to DMS");
            }
            projection_values[4] = dms;

            if (c_degdms (&projection_values[5], &dms,"DEG", "LAT") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms",
                         "Error converting latitude from degree to DMS");
            }
            projection_values[5] = dms;
        }

        else if (projection_code == TM)
        {
            if (c_degdms (&projection_values[4], &dms,"DEG", "LON") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms",
                         "Error converting longitude from degree to DMS");
            }
            projection_values[4] = dms;

            if (c_degdms (&projection_values[5], &dms,"DEG", "LAT") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms",
                         "Error converting latitude from degree to DMS");
            }
            projection_values[5] = dms;
        }

        else if (projection_code == EQRECT)
        {
            if (c_degdms (&projection_values[4], &dms,"DEG", "LON") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms",
                         "Error converting longitude from degree to DMS");
            }
            projection_values[4] = dms;

            if (c_degdms (&projection_values[5], &dms,"DEG", "LAT") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms", 
                         "Error converting latitude from degree to DMS");
            }
            projection_values[5] = dms;
        }

        else if (projection_code == SOM)
        {
            if (c_degdms (&projection_values[4], &dms,"DEG", "LON") != E_SUCC)
            {
                c_lasmsg(FALSE, "asterhdf2las-dms", 
                         "Error converting longitude from degree to DMS");
            }
            projection_values[4] = dms;
        }
    
        else
        {     
            c_lasmsg(FALSE, "asterhdf2las-proj", "Projection code is invalid");
            return (E_FAIL);
        }
    } 

    /* Convert the corner coordinates from Lat Long to projection coordinates 
       in meters.
    ------------------------------------------------------------------------*/
    if (latlon2yx(projection_values,projection_code,corner_coordinates,
                  zone,y_x_coordinates) != E_SUCC)
    {
        /* A warning message is issued in get_corner_coordinates */
        return (E_FAIL);   
    }

    /* Copy y_x_coordinates back to corner_coordinates
    --------------------------------------------------*/
    for (index = 0; index < NUM_COORDINATES; index++)
        corner_coordinates[index] = y_x_coordinates[index];

    return (E_SUCC);

}
