/******************************************************************************
NAME:           convert_verify_window

PURPOSE:        This function will use a DDR structure to convert the 
                window values to line/sample from their input values.   
                The window values will then be verified.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "ddr.h"
#include "util.h"
#include "diskio.h"
#include "geompak.h"
#include "cproj.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

int convert_verify_window
(
    int window_specified,     /* I: Flag indicating if the window field was
                                    specified (TRUE or FALSE)                */
    double window[],          /* I: Window values requested by user            
                                    Image window (If window_units is LS then
                                    sl, ss, nl, ns.  If window_units is DMS or
                                    DEG then UL_lat, UL_long, LR_lat, LR_long.
                                    If window_units is PRO then UL_Y, UL_X,
                                    LR_Y, LR_X.)                             */ 
    char window_units[],      /* I: Value from units field (LS,DEG,DMS,PRO)  */
    int num_window,           /* I: Count of window values                   */
    int nl,                   /* I: Total number of lines in the image       */
    int ns,                   /* I: Total number of samples in the image     */
    int projection_code,      /* I: Value will be either TM=9, LAMCC=4
                                    PS=6, EQRECT=17, SOM=22, GEO=0, UTM=1    */
    double projection_values[],/* I: Projection parameters of the scene      */
    double corner_coordinates[], /* I: Coordinates of the scene, no windowing 
                                       is applied. Value is degrees or meters*/
    int projection_distance,  /* I: Resolution of the image 15,30 or 90 meters*/ 
    int product_level,        /* I: Values can be Level 1A=L1A, Level 1B=L1B
                                    Level 2=L2 or Level 3 DEM=L3             */
    long zone,                /* I: UTM zone code                            */
    long valid_window[]       /* O: Window values in line/sample             */
)

{

    char *env_ptr = NULL;     /* pointer to the DATASYS environment variable */ 
    int index;                /* indexing variable                           */
    int intunits;             /* int value of the units 
                                 DEG=DEGREE, DMS=DMS, PRO=METER              */
    struct DDR ddr;           /* DDR to output                               */


    /* If the window was not specified then set the valid window to 1,1,nl,ns
    ------------------------------------------------------------------------*/
    if (window_specified == FALSE)
    {
        valid_window[SL] = 1;
        valid_window[SS] = 1;
        valid_window[NL] = nl;
        valid_window[NS] = ns;
    }
    else
    {
        /* Process the window only if the units are not LS
        -------------------------------------------------*/
        if (strcmp(window_units,"LS") != 0)
        {

            /* Fill the DDR structure for the current band.  This structure  
               will be used to call c_check_window.
            ----------------------------------------------------------------*/
            if (product_level != L1A) 
                strcpy(ddr.proj_units,"meters"); 
            else
                strcpy(ddr.proj_units,"degrees");

            ddr.proj_code = projection_code;

            if (projection_code != UTM)
                ddr.zone_code = DEFAULT_ZONE;
            else
                ddr.zone_code = zone;

            for (index = 0; index < 15; index++)
                ddr.proj_coef[index] = projection_values[index];

            ddr.datum_code = WGS84;
            ddr.nl = nl;
            ddr.ns = ns;
            ddr.dtype = EBYTE;   /* need only a dummy value */
            ddr.upleft[0] = corner_coordinates[0];
            ddr.upleft[1] = corner_coordinates[1];
            ddr.loleft[0] = corner_coordinates[2];
            ddr.loleft[1] = corner_coordinates[3];
            ddr.upright[0] = corner_coordinates[4];
            ddr.upright[1] = corner_coordinates[5];
            ddr.loright[0] = corner_coordinates[6];
            ddr.loright[1] = corner_coordinates[7];
            ddr.pdist_x = (double) projection_distance;
            ddr.pdist_y = (double) projection_distance;
            ddr.nbands = 1;  /* only creating info for one band */

            ddr.valid[0] = VALID;  /* projection code */
            ddr.valid[1] = VALID;  /* zone code */
            ddr.valid[2] = VALID;  /* datum code */
            ddr.valid[3] = VALID;  /* proj_coef */
            ddr.valid[4] = VALID;  /* ground units */
            ddr.valid[5] = VALID;  /* ground distance */
            ddr.valid[6] = VALID;  /* corner coordinates */

            /* get the system from the DATASYS environment variable */
            env_ptr = getenv("DATASYS");
            if (env_ptr != NULL)
                strncpy(ddr.system,env_ptr,12); /* 12 length */
            else
            {
                c_lasmsg(FALSE,"asterhdf2las-sys",
                        "DATASYS environment variable not set");
                strcpy (ddr.system, "UNKNOWN");
            }


            /* Convert the string window_units value to a long value
               DEG = 4, DMS = 5, PRO = 2 (meters).
            --------------------------------------------------------*/
            if (strcmp (window_units,"DEG") == 0)
                intunits = DEGREE;
            else if (strcmp (window_units,"DMS") == 0)
                intunits = DMS;
            else if (strcmp (window_units,"PRO") == 0)
                intunits = METER;

            /* Convert the window parameters from their units to line/sample.
            -----------------------------------------------------------------*/
            c_lasmsg(FALSE,"asterhdf2las-info", "Converting window units from "
                     "%s to LS",window_units);
            if (c_window_proj_to_ls (&ddr, window, &intunits) != E_SUCC)
            {
                c_lasmsg (FALSE, "asterhdf2las-wind", 
                          "Error converting window coordinates");
                return (E_FAIL);
            }
        }

        /* Copy the window to valid_window
        ----------------------------------*/
        for (index = 0; index < 4; index++)
            valid_window[index] = (long)window[index];

        /* If sl or ss is greater than the nl or ns then issue a fatal error
        --------------------------------------------------------------------*/
        if (valid_window[SL] > nl)
        {
            c_lasmsg (FALSE,"asterhdf2las-wind","The starting line %ld cannot "
                      "be larger than",valid_window[SL]);
            c_lasmsg (FALSE, "asterhdf2las-wind","the number of lines %ld",nl);
            return (E_FAIL);
        }

        if (valid_window[SS] > ns)
        {
            c_lasmsg (FALSE, "asterhdf2las-wind", "The starting sample %ld "
                      "cannot be larger than", valid_window[SS]);
            c_lasmsg (FALSE,"asterhdf2las-wind","the number of samples %ld",ns);
            return (E_FAIL);
        }

        /* If sl or ss is less than 1 then reset to 1
        -----------------------------------------------*/
        if (valid_window[SL] < 1)
        {
            c_lasmsg(FALSE, "asterhdf2las-wind",
                     "Starting line < 1, reset to 1");
            valid_window[SL] = 1;
        }

        if (valid_window[SS] < 1)
        {
             c_lasmsg (FALSE, "asterhdf2las-window",
                      "Starting sample < 1, reset to 1");
        valid_window[SS] = 1;
        }

        /* Check the valid window
        ------------------------*/
        if ((strcmp (window_units,"LS") == 0) && (num_window == 2))
        {
            /* If the window_units are LS and the number of window values is 2,
            then calculate the number of lines and samples for the window. Add
            1 since starting line and starting sample are 1 based.
            ----------------------------------------------------------------*/
            valid_window[NL] = nl - valid_window[SL] + 1;
            valid_window[NS] = ns - valid_window[SS] + 1;
        }

        /* If nl or ns extends past the LR corner then reset the nl and ns
        ----------------------------------------------------------------*/
        if (valid_window[SL] + valid_window[NL] > nl)
        {
            c_lasmsg(FALSE, "asterhdf2las-wind", 
                     "Window extends past image bounds so the number of lines");
            c_lasmsg(FALSE, "asterhdf2las-wind", 
                     "will be reset to stop at the LR corner of the image");
            valid_window[NL] = nl - valid_window[SL] + 1;
        }

        if (valid_window[SS] + valid_window[NS] > ns)
        {
            c_lasmsg(FALSE, "asterhdf2las-wind", 
                   "Window extends past image bounds so the number of samples");
            c_lasmsg(FALSE, "asterhdf2las-wind",
                   "will be reset to stop at the LR corner of the image");
            valid_window[NS] = ns - valid_window[SS] + 1;
        }
    }
    return (E_SUCC);
}
