/******************************************************************************
NAME:           final_output_names

PURPOSE:        This function will assign an outfile name to each band  
                grouping.  If there are not enough outfile names provided by 
                the user then one will be created with either "_vnir", "_swir",
                "_tir" extension added to the outnames.  Since there will be
                additional outfile names then band_groupings will be adjusted
                to reflect this.
                
RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

int final_output_names
(
    char outfiles[][CMLEN],     /* I/O: Output LAS file names                */
    int *num_outs,              /* I/O: Count of outfiles                    */
    int band_groupings[],       /* I/O: The grouping of bands. Each band
                                        grouping represents one LAS image    */
    char bands[][3],            /* I: Bands to be processed                  */
    int num_bands,              /* I: Count of bands                         */
    int proj_distance[],        /* I/O: Resolution of each band that is to 
                                        be processed                         */
    int *num_band_groupings     /* I/O: Count of band_groupings              */
)
{
    char temp_outs[NUMFILES][CMLEN]; /* The outfiles (with the added
                                        extensions) are stored here          */
    int band_group_totals = 0;    /* Keeps a running total of the bands in
                                     the band groupings. This includes the 
                                     total of the previous band groupings.   */
    int fo_index;                 /* Index for looping                       */
    int len;                      /* Variable used to hold length of a string*/
    int multi_res;                /* Flag indicating if the band grouping has
                                     multiple resolutions. (TRUE or FALSE)   */
    int nb_index;                 /* Index for looping                       */
    int temp_band_groupings[NUMBANDS]; /* Running count of bands in each band 
                                          grouping                           */
    int temp_num_outs;            /* Running count of number of outs         */
    int while_index;              /* Index for looping                       */
    int b3_b_index;               /* Variable used for looping               */
    int temp_number = 0;          /* Number used to flag a 3B band           */

    /* Initialize the temp_band_groupings to zero 
    -------------------------------------------*/
    memset(temp_band_groupings, 0, NUMBANDS * sizeof(int));

    /* Loop through bands and if band 3B exists, then set the resolution
    to a temporary number. This makes it easier to add the _vnir_3b extension 
    ------------------------------------------------------------------------*/
    b3_b_index = 0;
    
    while (b3_b_index < num_bands)
    {
        if (strcmp (bands[b3_b_index], "3B") == 0)
        {
            proj_distance[b3_b_index] = temp_number;
            break;
        }
        else
            b3_b_index++;
    }

    /* Loop through the band groupings
    ---------------------------------*/
    fo_index = 0;
    nb_index = 0;
    temp_num_outs = 0;
    while_index = 0;

    for (nb_index = 0; nb_index < *num_band_groupings; nb_index++)
    {
        /* If the .img extension exists then copy outfiles (without the 
           ".img" extension) to a temporary array */
        if (strstr(outfiles[nb_index], ".img") == NULL)
        {
            strcpy(temp_outs[fo_index], outfiles[nb_index]);
        }
        else
        {
            len = strlen(outfiles[nb_index]);
            strncpy(temp_outs[fo_index], outfiles[nb_index], len-4); 
            temp_outs[fo_index][len-4]='\0';
            temp_num_outs++;
            temp_band_groupings[fo_index]++;
        }

        /* Set multi_res flag to FALSE because it is assumed that the band 
        grouping has the same resolution */
        multi_res = FALSE;

        band_group_totals = band_group_totals+band_groupings[nb_index];

        /* Increment while_index once for each band groupings */
        while_index++;

        /* Compare the bands in the band grouping to see if the band
        grouping has multiple resolutions*/
        while (while_index < band_group_totals)
        {
            if (proj_distance[while_index] != proj_distance[while_index - 1])
            {
                /* Set multi_res flag to TRUE because another resolution was
                found in the band grouping */
                multi_res = TRUE;

                /* Add the extension, which describes the resolution of the
                output image, to the out file name */
                if (proj_distance[while_index-1] == 15)
                    strcat(temp_outs[fo_index],"_vnir");
                else if(proj_distance[while_index-1] == temp_number)
                    strcat(temp_outs[fo_index],"_vnir_3b");
                else if(proj_distance[while_index-1] == 30)
                    strcat(temp_outs[fo_index],"_swir");
                else if(proj_distance[while_index-1] == 90)
                    strcat(temp_outs[fo_index],"_tir");
                else
                {
                    c_lasmsg(FALSE, "asterhdf2las-outs",
                             "Error getting output image extension");
                    return(E_FAIL);
                }
                fo_index++; 
                strncpy(temp_outs[fo_index],outfiles[nb_index],len-4);
                temp_outs[fo_index][len-4] = '\0'; 
                temp_num_outs++; 
            }
            while_index++;             
            temp_band_groupings[fo_index]++;
        }

        /* This is the last band of the band grouping.  If the multi_res
        flag is set to TRUE then an extension is needed since the temp_outs
        file prior to this one has an extension */
        if ((while_index == band_group_totals) && (multi_res == TRUE))
        {
            if (proj_distance[while_index-1] == 15)
                strcat(temp_outs[fo_index],"_vnir");
            else if (proj_distance[while_index-1] == temp_number)
                strcat(temp_outs[fo_index],"_vnir_3b");
            else if(proj_distance[while_index-1] == 30)
                strcat(temp_outs[fo_index],"_swir");
            else if(proj_distance[while_index-1] == 90)
                strcat(temp_outs[fo_index],"_tir");
            else
            {
                c_lasmsg(FALSE, "asterhdf2las-outs", "Error getting output "
                         "image extension for projection distance", 
                         proj_distance[while_index-1]);
                return(E_FAIL);
            }
        }
        fo_index++;
    } 

    /* Set the temp_number resolution back to 15
    --------------------------------------------*/
    proj_distance[b3_b_index] = 15;

    /* Get the number of outs
    ------------------------*/
    *num_outs = temp_num_outs;

    /* Since the count of outs equals the count of band groupings, 
       num_band_groupings can be set using num_outs
    -----------------------------------------------------------------*/
    *num_band_groupings = *num_outs;

    /* Copy temp_band_groupings back to band_groupings. 
    --------------------------------------------------*/
    for (nb_index = 0; nb_index < *num_band_groupings; nb_index++)
        band_groupings[nb_index] = temp_band_groupings[nb_index];  

    /* Initialize outfiles array to empty
    -----------------------------------*/
    for (fo_index = 0; fo_index < *num_outs; fo_index++)
        strcpy(outfiles[fo_index], "");

    /* Copy the temporary outfiles back to outfile array and attach the .img
       extension.
    ------------------------------------------------------------------------*/
    for (fo_index = 0; fo_index < *num_outs; fo_index++)
    {
        strcpy(outfiles[fo_index], temp_outs[fo_index]);
        strcat(outfiles[fo_index], ".img");
    }

    return (E_SUCC);
}
