/******************************************************************************
NAME:           get_corner_coordinates

PURPOSE:        This function retrieves the corner coordinates from the
                metadata information.

RETURN VALUE:
Type = void

PROGRAM HISTORY:

PROJECT:        LAS

NOTE: The corner coordinate location of ASTER HDF Level 1 Data is referenced
      by the center of pixel which is the same with LAS. 
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "PGS_MET.h"
#include "PGS_tk.h"
#include "mfhdf.h"
#include "proj.h"
#include "las.h"
#include "get_corner_coordinates.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

void get_corner_coordinates
(
    PGSt_integer fileid,        /* I: File id for the file containing
                                      parameter data                         */ 
    int projection_code,        /* I: Projection of the scene TM,LAMCC,PS,
                                      EQRECT,SOM, UTM or GEO                 */
    int product_level,          /* I: Level 1A=L1A Level 1B=L1B Level 2=L2
                                      Level 3 DEM=L3                         */
    long *zone,                 /* O: Zone code needed for UTM projection    */
    double corner_coordinates[],/* O: Corner coordinates in lat/lon for 
                                      Level 1A and DEM. They are in y/x meters
                                      for rest of products                   */
    int ddr_valids[],           /* O: Flag indicating if corner coordinates
                                      field is valid or not (VALID or INVAL) */
    double projection_values[]  /* I/O: Input angle values in radians output
                                        in degrees/minutes/seconds for all 
                                        projections except geographic        */

)

{
    int index;                  /* looping variable                          */
    double coordinates[2];      /* array used to extract coordinates from
                                   the HDF metadata                          */
    double coordinates_dem[4];  /* array used to extract DEM coordinates
                                   from the HDF metadata                     */
    double scene_center[2];     /* array used to extract center of scene
                                   (lat/long) from the HDF metadata          */
    PGSt_SMF_status ret;        /* Return status                             */

    /* Initialize corner coordinates validity flag to VALID. If an error 
       occurs it will be set to INVAL.
    ------------------------------------------------------------------*/
    ddr_valids[6] = VALID;

    /* Get the four corners coordinates values tag and copy to
       corner_coordinates array
     --------------------------------------------------------*/
    index = 0;
    if (product_level != L3)
    {

        ret = PGS_MET_GetPCAttr(fileid,1,"productmetadata.0","UPPERLEFT",
                &coordinates);
        if (ret != PGS_S_SUCCESS) 
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                    "UPPERLEFT. The corner coordinates are invalid");
            ddr_valids[6] = INVAL;
        }
        corner_coordinates[index] = coordinates[0];
        corner_coordinates[index+1] = coordinates[1];  
        index = index + 2;

        ret = PGS_MET_GetPCAttr(fileid,1,"productmetadata.0","LOWERLEFT",
                &coordinates);
        if (ret != PGS_S_SUCCESS)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                    "LOWERLEFT. The corner coordinates are invalid");
            ddr_valids[6] = INVAL;
        }
        corner_coordinates[index] = coordinates[0];
        corner_coordinates[index+1] = coordinates[1];
        index = index + 2;

        ret = PGS_MET_GetPCAttr(fileid,1,"productmetadata.0","UPPERRIGHT",
                &coordinates);
        if (ret != PGS_S_SUCCESS)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                    "UPPERRIGHT. The corner coordinates are invalid");
            ddr_valids[6] = INVAL;
        }
        corner_coordinates[index] = coordinates[0];
        corner_coordinates[index+1] = coordinates[1];
        index = index + 2;

        ret = PGS_MET_GetPCAttr(fileid,1,"productmetadata.0","LOWERRIGHT",
                &coordinates);
        if (ret != PGS_S_SUCCESS)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                    "LOWERRIGHT. The corner coordinates are invalid");
            ddr_valids[6] = INVAL;
        }

        corner_coordinates[index] = coordinates[0];
        corner_coordinates[index+1] = coordinates[1];
        index = index + 2;

    }
    else
    {
        /* GRINGPOINTLONGITUDE and GRINGPOINTLATITUDE. The coordinates are
           listed in the metadata as upper left, upper right, lower right
           and lower left
        --------------------------------------------------------------------*/
        ret = PGS_MET_GetPCAttr(fileid,1,"coremetadata.0",
                                "GRINGPOINTLATITUDE.1", coordinates_dem);
        if (ret != PGS_S_SUCCESS)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                 "GRINGPOINTLATITUDE. The corner coordinates are invalid");
            ddr_valids[6] = INVAL;
        }

        corner_coordinates[0] = coordinates_dem[0]; /* UL */ 
        corner_coordinates[4] = coordinates_dem[1]; /* UR */ 
        corner_coordinates[6] = coordinates_dem[2]; /* LR */
        corner_coordinates[2] = coordinates_dem[3]; /* LL */ 
        
        ret = PGS_MET_GetPCAttr(fileid,1,"coremetadata.0", 
                                "GRINGPOINTLONGITUDE.1", &coordinates_dem);

        if (ret != PGS_S_SUCCESS)
        { 
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag " 
                  "GRINGPOINTLONGITUDE. The corner coordinates are invalid"); 
            ddr_valids[6] = INVAL;
        } 

        corner_coordinates[1] = coordinates_dem[0]; /* UL */ 
        corner_coordinates[5] = coordinates_dem[1]; /* UR */
        corner_coordinates[7] = coordinates_dem[2]; /* LR */ 
        corner_coordinates[3] = coordinates_dem[3]; /* LL */ 
    }

    /* Get scene center which is needed to calculate the UTM zone 
       code for DEM products
    ----------------------------------------------------------------*/
    if (projection_code == UTM)
    {
        ret = PGS_MET_GetPCAttr(fileid,1,"productmetadata.0","SCENECENTER",
                                &scene_center);
        if (ret != PGS_S_SUCCESS) 
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                     "SCENECENTER. Therefore UTM zone cannot be calculated");
            c_lasmsg(FALSE, "asterhdf2las-meta", "Set UTM zone to invalid");
            ddr_valids[1] = INVAL;
            *zone = DEFAULT_ZONE;
        }
        else
        {
            /* Calculate UTM zone number for Level 3 DEMs -- NOTE Longitude
               entered in DEGREES
            --------------------------------------------------------------*/
            *zone = (long)(((scene_center[1] + 180.0) / 6.0) + 1.0); 

            /* Check if zone code is valid
            -----------------------------*/
            if ((*zone < 1) || (*zone > 60))
            {                
                c_lasmsg(FALSE,"asterhdf2las-meta", "UTM zone code %ld is "
                        "invalid", *zone);
                ddr_valids[1] = INVAL;
            }
            else
                ddr_valids[1] = VALID;
        }
    }
    /* The metadata information for all product levels has the corner
       coordinates in degrees and the projection distance in meters.  If 
       the product is level 1A, leave the corner coordinates in degrees.
       If the projection code is GEO (it is set to GEO if an error occured
       while retrieving the projection code from the HDF metadata) then leave
       the corner coordinates in degrees.  The projection distance is 
       calculated to degrees in update_ddr. If it is either one of the other 
       products (1B or 2 or 3) then convert the corner coordinates to meters,
       leave the projection distance as meters and convert the projection
       parameters angles from radians to DMS. 
    --------------------------------------------------------------------*/
    if ((product_level != L1A) && (projection_code != GEO) && 
        (ddr_valids[3] != INVAL) && (ddr_valids[6] != INVAL))  
    {
        if (convert_proj_parms_coordinates(corner_coordinates,
            projection_values, projection_code,*zone) != E_SUCC)
        {
            c_lasmsg(FALSE,"asterhdf2las-warn", 
                     "Error converting corner coordinates. Set to invalid");

            ddr_valids[6] = INVAL;
        }     
     }   
   
    /* If the projection code is GEO but is not a level 1a then set
       the corner coordinates validity flag to invalid.
    --------------------------------------------------------------------*/
    if ((product_level != L1A) && (projection_code == GEO))
    {
        ddr_valids[6] = INVAL;
    }
  
    /* If the projection parameters are invalid and the projection code is 
       not GEO, then the corner coordinates are left in degrees and set to
       invalid
    ---------------------------------------------------------------------*/
    if ((projection_code != GEO) && (ddr_valids[3] != VALID))
        ddr_valids[6] = INVAL;

}
