/******************************************************************************
NAME:           get_l1a_l1b_metadata

PURPOSE:        This function retrieves Level 1A and Level 1B metadata specific
                information required for LAS DDR.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "PGS_MET.h"
#include "PGS_tk.h"
#include "mfhdf.h"
#include "PGS_SMF.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

int get_l1a_l1b_metadata
(
    int fileid,               /* I: File id for the file containing
                                    parameter data                           */
    int product_level,        /* I: Flag values can be Level 1A =L1A, or
                                    Level 1B =L1B                            */
    char band_sensor[],       /* I: Indicates if band is VNIR, SWIR or TIR   */

    char projectionparameters[],/* I: The tag name that is to be retrieved 
                                      ie PROJECTIONPARAMETERS1               */
    char mpmethod[],          /* I: The tag name that is to be retrieved
                                    ie MPMETHOD1                             */
    char *instrument_name,    /* O: Instrument name ie ASTER                 */
    char *projection_name,    /* O: Names in metadata will be either UTM,
                                    LAMCC, PS, EQRECT, SOM                   */
    double projection_values[],/* O: The 15 projection parameters            */
    int ddr_valids[]           /* O: Flags indicating if fields in the DDR
                                    are valid or not (VALID or INVAL)        */
) 

{
    char product_metadata[20];/* File attribute name                         */
    PGSt_SMF_status ret;      /* Return status                               */

    /* Get instrument name from INSTRUMENTSHORTNAME tag
    --------------------------------------------------*/
    ret = PGS_MET_GetPCAttr(fileid,1,"coremetadata.0","INSTRUMENTSHORTNAME", 
                            &instrument_name);
    if (ret != PGS_S_SUCCESS)
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                 "INSTRUMENTSHORTNAME. Unable to retrieve the sensor type.");
    }

    /* The rest of the information applies only to Level 1B products
    ---------------------------------------------------------------*/

    /* Find out what type of band exists.  This information helps distinguish
       which file attribute name is to be read.
    ------------------------------------------------------------------------*/
    if (product_level == L1B)
    {
        if (strcmp (band_sensor, "VNIR") == 0)
            strcpy(product_metadata, "productmetadata.v");
        else if (strcmp (band_sensor, "SWIR") == 0)
            strcpy(product_metadata, "productmetadata.s");
        else if (strcmp (band_sensor, "TIR") == 0)
            strcpy(product_metadata, "productmetadata.t");
        else
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", 
                     "Error retrieving file attribute productmetadata");
            return (E_FAIL);
        }

        /* Get the projection parameters of the first band using tag 
           projectionparameters 
        ------------------------------------------------------------*/
        ret = PGS_MET_GetPCAttr(fileid,1,product_metadata,
                                projectionparameters,projection_values);

        if (ret != PGS_S_SUCCESS)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta","Error retrieving the tag %s. "
                     "Projection parameters are invalid.",projectionparameters);
            ddr_valids[3] = INVAL;
        }
        else
            ddr_valids[3] = VALID;


        /* Get the projection name of the first band using tag mpmethod
        --------------------------------------------------------------*/
        ret = PGS_MET_GetPCAttr(fileid,1,product_metadata,mpmethod,
                                &projection_name);

        if (ret != PGS_S_SUCCESS)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta",
                     "Error retrieving the tag %s", product_metadata);
            ddr_valids[0] = INVAL;
        }
    }
    return (E_SUCC);
}
