/******************************************************************************
NAME:           get_metadata

PURPOSE:        This function retrieves the metadata information required
                for LAS DDR.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS

NOTE: 1) The software of ASTER Level-1 Data Processing Subsystem uses the TM
         projection parameters but refers to the projection name and projection
         information in the metadata as UTM.  The LAS DDR will use the 
         projection name TM. 

      2) The SDP (Science Data Processing) Toolkit software is used for 
         metadata handling, process control and status message handling.  
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "PGS_MET.h"  /* for SDP Toolkit */ 
#include "PGS_tk.h"
#include "PGS_SMF.h"
#include "proj.h"
#include "geompak.h"
#include "get_metadata.h" 
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"
#include "get_corner_coordinates.h"

#define PB_LENGTH 30          /* String length of PROCESSEDBANDS */

int get_metadata
(
    int32 sd_id,              /* I: SD interface identifier                  */
    char infile[],            /* I: Input HDF filename                       */
    char outfiles[][CMLEN],   /* I/O: Output LAS filename(s)                 */
    int num_outs,             /* I/O: Count of outfiles                      */
    char bands[][3],          /* I/O: Bands to be processed                  */
    int num_bands,            /* I/O: Count of bands                         */
    int band_groupings[],     /* I/O: The grouping of bands. Each band group 
                                      represents one output LAS image        */
    int *num_band_groupings,  /* I/O: Count of band_groupings                */
    int process_all_bands,    /* I: Flag indicating if all bands in file
                                    should be processed (process all = TRUE
                                    else FALSE)                              */
    int32 sds_index[],        /* I/O: Index of data set                      */
    int *product_level,       /* O: Values can be Level 1A=L1A, Level 1B=L1B
                                    Level 2=L2 or Level 3=L3                 */
    int ddr_valids[],         /* O: Flags indicating if fields in the DDR 
                                    are valid or not (VALID or INVAL)        */
    char *acquisition_date,   /* O: Capture date                             */
    char *acquisition_time,   /* O: Capture time                             */
    double corner_coordinates[],/* O: Corner coordinates in either degrees 
                                      or meters                              */
    double projection_values[],/* O: Projection parameters                   */
    int *projection_code,      /* O: Either GEO, TM, LAMCC, PS, EQRECT or SOM*/
    int32 data_set_index_num[],/* O: Index number of the data set            */
    int projection_distance[], /* O: Resolution of each band(15,30or90 meters)*/
    char projection_units[],  /* O: Projection units (degrees or meters)     */
    char *instrument_name,    /* O: Type of sensor                           */
    int dtype[],              /* O: Data types for each band                 */
    int nl[],                 /* O: Array of total number of lines           */
    int ns[],                 /* O: Array of total number of samples         */
    char kinetic_temp[],      /* O: Indicates if product is Kinetic 
                                    Temperature (yes or no)                  */ 
    char intermediate_files[][CMLEN],/* O: Path and file names of the 
                                           temporary files                   */ 
    char subdirectory[],      /* O: Name of the subdirectory which stores
                                    the intermediate files                   */
    long *zone                /* O: Zone code needed for UTM projection      */
) 

{   
    PGSt_SMF_status ret;      /* Return status                               */
    PGSt_integer fileid;      /* File id for the file containing parameter
                                 data                                        */
    char *acquisition_date_ptr=NULL; /* pointer used to point to chars in
                                        acquisition date                     */
    char *acquisition_time_ptr=NULL; /* pointer to acquisition_time          */
    char *pb_ptr = NULL;      /* Pointer to processedbands                   */
    char processedbands[PB_LENGTH+1]; /* Bands available in HDF file         */
    char projection_name[8];  /* name of projection(TM,LAMCC,PS,EQRECTorSOM) */

    char shortname[20];       /* Product id                                  */
    char *shortname_ptr = NULL; /* Pointer to shortname                      */
    char *temp_files_directory_ptr; /* Pointer to the directory where 
                                       temporary files will be stored        */
    char available_bands[NUMBANDS][3]; /* Bands available in HDF file        */
    char band_sensor[5];      /* Indicates if band is VNIR, SWIR or TIR      */
    char data_set_name[NUMBANDS][20]; /* Index name of the data set          */
    char filename[CMLEN];     /* filetable.temp and the current working 
                                 directory path                              */
    char found_files[NUMFILES][CMLEN]; /* Existing output files found on disk*/
    char mpmethod[20];        /* used to hold MPMETHOD tag name              */
    char nonexist_bands[NUMBANDS][3]; /* Bands requested that do not 
                                         exist in HDF file                   */
    char outfiles_that_exist[NUMFILES][CMLEN]; /* array of files that 
                                                  already exist on disk      */
    char projectionparameters[20]; /* used to hold PROJECTIONPARAMETERS tag
                                      name                                   */
    char sds_name[MAX_NC_NAME]; /* name of the data set                      */
    char *subdir_ptr;         /* pointer to subdirectory name                */
    char temp[3];             /* Temporary string used to hold band numbers  */
    char temp_acquisition_date[9]; /* temp string to hold acquisition_date   */
    char temp_acquisition_time[9]; /* temp string to hold acquisition_time   */ 
    int ab_index;             /* Index for available bands                   */
    int band_found;           /* Flag indicating if a band exists in HDF file*/
    int index2;               /* Index value used for data set looping       */
    int index;                /* Index used for looping                      */
    int index_num;            /* Index number of the data set used for looping*/
    int nonex_index;          /* Index used for nonexist bands loop          */
    int num_available_bands;  /* Count of available bands in HDF file        */
    int num_nonexist_bands;   /* Count of nonexisting bands                  */
    int pb_index;             /* Index for processedbands                    */
    int proj_unit_code;       /* Value either METER or DEGREE                */
    int sum;                  /* Sum of adjusted band_groupings              */
    int32 data_type;          /* data type of data in data set               */
    int32 dim_sizes[3];       /* size of each dimension in the data set      */
    int32 n_attrs;            /* number of attributes in data set            */
    int32 rank;               /* number of dimensions in data set            */
    int32 sds_id;             /* data set identifier                         */
    intn status_n;            /* status for functions returning an intn      */
    long current_time;        /* current time                                */
    int nfiles;               /* Number of outfiles found from c_getfiles    */
    mode_t mode = S_IRWXU;    /* S_IRWXU = Read, write, execute by owner     */
    struct  tm *tm_ptr;       /* pointer to local time structure             */

    /* The file that holds the input file name is $(PWD)/filetable.temp.
       The SDP Toolkit expects filetable.temp to be located in the current
       working directory. In order to avoid possible problems with running
       asterhdf2las multiple times (at the same time and in the same working
       directory), a subdirectory called test_ will be created and the time
       stamp will be appended to the subdirectory name (test_hhmmss).  This
       subdirectory will store the temporary intermediate files. 
       ---------------------------------------------------------------------*/
    /* Get the time, format it and append it to the subdirectory name */
    current_time = time(NULL);
    tm_ptr = localtime(&current_time);

    strcpy(subdirectory, "temp_");

    subdir_ptr = subdirectory + strlen(subdirectory);

    sprintf (subdir_ptr,"%2d",tm_ptr->tm_hour);
    if (tm_ptr->tm_hour < 10)
        *subdir_ptr = '0';
    subdir_ptr += 2;

    sprintf (subdir_ptr,"%2d",tm_ptr->tm_min);
    if (tm_ptr->tm_min < 10)
        *subdir_ptr = '0';
    subdir_ptr += 2;

    sprintf (subdir_ptr,"%2d",tm_ptr->tm_sec);
    if (tm_ptr->tm_sec < 10)
        *subdir_ptr = '0';
    subdir_ptr += 2; 

    /* Create the subdirectory */
    if (mkdir(subdirectory, mode) != E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-dir", "Error creating subdirectory %s",
                subdirectory);
        return (E_FAIL);
    } 

    /* Get current working directory */
    temp_files_directory_ptr = getenv ("PWD");
    if (temp_files_directory_ptr == NULL)
    {
        c_lasmsg(FALSE, "asterhdf2las-env", "Error retrieving environmental "
                "variable PWD");
        return (E_FAIL);
    }

    /* Concat the subdirectory */
    strcat(temp_files_directory_ptr, "/");
    strcat(temp_files_directory_ptr, subdirectory);

    /* Change the value of the environmental variable PWD */
    if (putenv (temp_files_directory_ptr) != E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-env", "Error setting environmental "
                "variable PWD");
        return (E_FAIL);
    }

    /* Add filetable.temp to the filename path */
    sprintf(filename, "%s/filetable.temp", temp_files_directory_ptr);

    /* Copy the path and file names to an array.
       -------------------------------------------*/
    strcpy(intermediate_files[0],filename);
    sprintf(intermediate_files[1], "%s/LogStatus", temp_files_directory_ptr);
    sprintf(intermediate_files[2], "%s/GetAttrtemp", temp_files_directory_ptr);

    /* Create the filetable.temp file and assign the input HDF file a
       logical id
       -----------------------------------------------------------------*/
    if (create_filetable(filename,infile,temp_files_directory_ptr) != E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-file", 
                "Error creating file %s", filename);
        return (E_FAIL);
    }  

    /* Set logical id for the HDF file
       ----------------------------------*/
    ret = PGS_MET_SetFileId();
    if (ret != PGS_S_SUCCESS)
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", "Error assigning HDF logical id");
        return (E_FAIL);
    }   

    /* Get file id for the HDF file which is retrieved from filetable.temp
       ----------------------------------------------------------------------*/
    fileid = PGS_MET_GetFileId(infile);
    if (fileid == 0)
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving file id");
        return (E_FAIL);
    }
    /* Now we can retrieve the metadata information 
       ----------------------------------------------*/

    /* Get product id from tag SHORTNAME
       ----------------------------------*/
    shortname_ptr = shortname;

    ret = PGS_MET_GetPCAttr(fileid,1,"coremetadata.0","SHORTNAME", 
            &shortname_ptr);
    if (ret != PGS_S_SUCCESS)
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the HDF tag "
                "SHORTNAME");
        return (E_FAIL); 
    } 

    /* Set the flag to indicate the product level
       ---------------------------------------------*/
    if ((strcmp(shortname,"AST01")==0) || (strcmp(shortname,"AST_01")==0)
            || (strcmp(shortname,"ASTL1A")==0))
        *product_level = L1A;
    else if ((strcmp(shortname,"AST03")==0) || (strcmp(shortname,"AST_03")==0)
            || (strcmp(shortname,"ASTL1B")==0))
        *product_level = L1B;
    else if ((strcmp(shortname,"AST14")==0) || (strcmp(shortname,"AST_14")==0)
            || (strcmp(shortname,"AST14DEM") == 0))
        *product_level = L3;
    else
        *product_level = L2;

    /* PROCESSEDBANDS will not be used for the DEM since the DEM image is 
       only one band 
       --------------------------------------------------------------------*/
    if (*product_level != L3)
    {
        /* Get the available bands in the file from PROCESSEDBANDS.
           NOTE: If PROCESSEDBANDS string value is
           "01023N3B0405060708091011121314", then all bands exist in the
           file.  To show a band does not exist in the file, a "XX" will be 
           used instead of the band number.  For example, a file that has all
           bands but "07" will be shown as "01023N3B040506XX08091011121314"
           -----------------------------------------------------------------*/

        pb_ptr = processedbands; 
        ret = PGS_MET_GetPCAttr(fileid,1,"productmetadata.0", "PROCESSEDBANDS",
                &pb_ptr); 

        if (ret != PGS_S_SUCCESS)  
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the HDF tag"
                    " PROCESSEDBANDS. Band numbers do not exist in the HDF "
                    "metadata.");
            return (E_FAIL);
        } 

        num_available_bands = 0;
        pb_index = 0;
        while (pb_index < PB_LENGTH) 
        {
            strncpy(temp, pb_ptr+pb_index,2);
            temp[2] = '\0';
            if (strcmp (temp, "XX") != 0)
            {
                strcpy(available_bands[num_available_bands], temp);
                num_available_bands ++;
            }
            pb_index = pb_index + 2;
        }

        /* Copy the available bands to bands array if the user didn't 
           specify BANDS
           ---------------------------------------------------------------*/
        if (process_all_bands == TRUE)
        {
            for(ab_index = 0; ab_index < num_available_bands; ab_index++)
                strcpy(bands[ab_index],available_bands[ab_index]);
            *num_band_groupings = 1;
            num_bands = num_available_bands;
            band_groupings[0] = num_available_bands;
        } 

        /* Look for the nonexisting bands and copy them to nonexist bands array
           -------------------------------------------------------------------*/
        else    
        {
            /* Make sure each band consists of 2 characters. This is used for 
               the string comparisons for the nonexist bands */
            for (index = 0; index < num_bands; index++)
            {
                if (strlen(bands[index]) == 1)
                {
                    strcpy(temp, bands[index]);
                    strcpy(bands[index], "0");
                    strcat(bands[index], temp);
                }
            }

            num_nonexist_bands = 0;/* Keep a total of the bands that don't 
                                      exist in the HDF file*/
            nonex_index = 0;
            /* Loop through each band */
            for (index = 0; index < num_bands; index ++)
            {  
                band_found = FALSE;
                pb_index = 0;
                while (pb_index < NUMBANDS)
                {
                    /* Compare the available bands the current [index] band */
                    if (strcmp(bands[index], available_bands[pb_index]) != 0) 
                    {
                        pb_index++;
                    }
                    else
                    {
                        band_found = TRUE;
                        break;
                    }
                }
                /* If the band does not exist in the HDF file, copy it to a 
                   nonexisting bands array */
                if (band_found == FALSE) 
                {
                    strcpy (nonexist_bands[nonex_index], bands[index]);
                    nonex_index ++;
                    num_nonexist_bands ++; /*Add 1 to nonexisting bands total*/
                }
            }

            /* Warn the user of nonexisting bands.  Adjust the band array,
               band grouping array and out image array according to the bands
               that don't exist.
               --------------------------------------------------------------*/
            if (num_nonexist_bands > 0)
            {
                if (nonexisting_bands(nonexist_bands,num_nonexist_bands,bands,
                            &num_bands,band_groupings,num_band_groupings,
                            outfiles, &num_outs)!= E_SUCC)
                {
                    c_lasmsg(FALSE,"asterhdf2las-band",
                            "Error from nonexisting_bands");
                    return (E_FAIL);
                }
            }
        }     

        /* Get the sum of bands
           ----------------------*/
        sum = 0;
        for (index = 0; index < *num_band_groupings; index++)
            sum = sum + band_groupings[index];

        /* If sum of band_groupings equals the count of band_grouping then each
           band goes into a separate output file.  There is no need to sort the 
           bands.  Otherwise sort the bands.
           -------------------------------------------------------------------*/
        if (sum != *num_band_groupings)
            sort_bands_in_band_groupings(bands, band_groupings, num_bands, 
                    *num_band_groupings);

        /* Get the data set name.  The data set names will vary according to
           the products.
           ------------------------------------------------------------------*/
        if (*product_level != L1A)
        {
            /*Level 1B products will have the same data set name for each band*/
            if (*product_level == L1B)
            {
                for (index = 0; index < num_bands; index ++)
                {
                    strcpy (data_set_name[index],"ImageData");
                    /* The ImageData names cannot have a '0'. ie ImageData1
                       instead of ImageData01 */
                    if ((strchr(bands[index],'0') != NULL) &&
                            (strcmp(bands[index], "10") != 0))
                        strncat (data_set_name[index], bands[index]+1, 1);
                    else
                        strcat (data_set_name[index], bands[index]);
                }
            }

            /* Level 2 products will have the same data set names for each band.
               There is an exception for the Level 2 product Surface Kinetic
               Temperature (also identified with ShortName AST_08). The data set
               name for this is KineticTemperature */
            else if (*product_level == L2)
            {
                if ((strcmp(shortname, "AST_08") != 0)
                        && (strcmp (shortname, "AST08") != 0))
                {
                    for (index = 0; index < num_bands; index ++)
                    {
                        strcpy (data_set_name[index],"Band");
                        /* The Band names cannot have a '0'. ie Band1 instead 
                           of Band01 */
                        if ((strchr(bands[index],'0') != NULL) && 
                                (strcmp(bands[index], "10") != 0)) 
                            strncat (data_set_name[index], bands[index]+1, 1);
                        else
                            strcat (data_set_name[index], bands[index]);
                    }
                }
                else
                {
                    strcpy (data_set_name[0], "KineticTemperature");
                    num_bands = 1; /*KineticTemperature in only one band even
                                     though more may be listed in metadata*/
                    *band_groupings = 1;
                    *num_band_groupings = 1;
                    strcpy(kinetic_temp, "yes");
                }
            }

            else
            {
                c_lasmsg(FALSE, "asterhdf2las-name",
                        "Error generating data set name");
                return (E_FAIL);
            }

            /* Find index of the data set with the name specified in 
               data_set_name and use the index to select the data set
               --------------------------------------------------------*/
            for (index = 0; index < num_bands; index++)
            {
                sds_index[index] = SDnametoindex (sd_id, data_set_name[index]);
                sds_id = SDselect (sd_id, sds_index[index]);
                status_n = SDgetinfo (sds_id, sds_name, &rank, dim_sizes,
                        &data_type, &n_attrs);  

                if (status_n == HDF_FAIL)
                {
                    c_lasmsg(FALSE, "asterhdf2las-meta",
                            "Error reading image information from file %s", 
                            infile);
                    return (E_FAIL);
                }
                /* copy dim_sizes and data_type */
                nl[index] = dim_sizes[0];
                ns[index] = dim_sizes[1];
                dtype[index] = data_type;
            }
        }
        else
        {
            /* Level 1A products identify all image data sets with the same
               name.  The name is ImageData.  Using the bands array, loop
               through each available band and compare the band numbers.  Once
               the band is found in the available bands array, then set data
               set number array to the available bands index + 1.  The data set
               number array will be used to distinguish which data sets need to
               be read to process the new bands.  */

            ab_index = 0;
            for (index = 0; index < num_bands; index ++)
            {
                while (strcmp(bands[index], available_bands[ab_index]) != 0)
                {
                    ab_index++;
                    /* add check to avoid a possible infinite loop */
                    if (ab_index > NUMBANDS)
                        break;
                }
                data_set_index_num[index] = ab_index + 1;

                ab_index = 0;
            }

            /* Loop through each data set and if the name of the data set
               is "ImageData" then that is the first available band, the second
               data set with the name "ImageData" is the second available band
               etc..  */

            for (index = 0; index < num_bands; index++)
            {
                index2 = 0;
                index_num = 0;
                while(index2 < data_set_index_num[index])
                {
                    sds_id = SDselect(sd_id, index_num);
                    status_n = SDgetinfo (sds_id, sds_name, &rank, dim_sizes,
                            &data_type, &n_attrs);
                    if (status_n == HDF_FAIL)
                    {
                        c_lasmsg(FALSE, "asterhdf2las-meta",
                                "Error reading image information from file %s", 
                                infile);
                        return (E_FAIL);
                    }
                    if (strcmp(sds_name, "ImageData") == 0)
                        index2++;
                    index_num++;
                }
                sds_index[index] = index_num - 1;

                /* Copy the nl, ns and data type information for the bands
                   that are going to be processed */
                nl[index] = dim_sizes[0];
                ns[index] = dim_sizes[1];
                dtype[index] = data_type;
            }
        }

        /* Loop through each band and assign a resolution. Then set the 
           ddr_valids flag to valid
           ------------------------------------------------------------------*/
        for (index = 0; index < num_bands; index++)
        {
            if ((strcmp(bands[index],"01") == 0) ||
                    (strcmp(bands[index],"02") == 0) ||
                    (strcmp(bands[index],"3N") == 0) ||
                    (strcmp(bands[index],"3B") == 0))
            {
                projection_distance[index] = 15;
            }
            else if ((strcmp(bands[index],"04") == 0) ||
                    (strcmp(bands[index],"05") == 0) ||
                    (strcmp(bands[index],"06") == 0) ||
                    (strcmp(bands[index],"07") == 0) ||
                    (strcmp(bands[index],"08") == 0) ||
                    (strcmp(bands[index],"09") == 0))
            {
                projection_distance[index] = 30;
            }
            else if ((strcmp(bands[index],"10") == 0) ||
                    (strcmp(bands[index],"11") == 0) ||
                    (strcmp(bands[index],"12") == 0) ||
                    (strcmp(bands[index],"13") == 0) ||
                    (strcmp(bands[index],"14") == 0))
            {
                projection_distance[index] = 90;
            }
            else
            {
                c_lasmsg(FALSE, "asterhdf2as-meta",
                        "Error setting projection distance");
                return (E_FAIL);
            }
        }
        /* Since the user may not know how many image sizes are in a scene,
           there may not be enough outfiles requested. Since we cannot put two 
           bands of different sizes (or resolutions) in the same output image,
           then an additional outfile will be created using a "_vnir", "_swir"
           or "_tir" extension.  The extension depends on the resolution.
           ------------------------------------------------------------------*/
        if(final_output_names(outfiles,&num_outs,band_groupings,bands,num_bands,
                    projection_distance, num_band_groupings) != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-name", 
                    "Error retrieving output file name");
            return (E_FAIL);
        }
    }
    else   /* product level is Level 3 DEM */
    {

        /* Assign the resolution. Since DEM's are derived from bands 
           3N and 3B it will always be 15.
          -----------------------------------------------------------*/
        projection_distance[0] = 15;

        /* DEM is only one band */
        strcpy (data_set_name[0],"Band1");
        num_bands = 1;
        *num_band_groupings = 1;
        band_groupings[0] = 1;

        /* Get data set index number using the data set name */
        sds_index[index] = SDnametoindex (sd_id, data_set_name[0]);
        if (sds_index[index] == HDF_FAIL)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta", "Data set with the name %s "
                    "does not exist", data_set_name[0]);
            return (E_FAIL);
        }

        /* Select the data set and get dimension sizes and data type */
        sds_id = SDselect (sd_id, sds_index[index]);
        status_n = SDgetinfo (sds_id, sds_name, &rank, dim_sizes,
                &data_type, &n_attrs);
        if (status_n == HDF_FAIL)
        {
            c_lasmsg(FALSE, "asterhdf2las-meta",
                    "Error reading image information from file %s", infile);
            return (E_FAIL);
        }

        /* Copy dim_sizes and data type */
        nl[0] = dim_sizes[0];
        ns[0] = dim_sizes[1];
        dtype[0] = data_type;
    }

    /* Check the output file names and issue a fatal error if it already exists
       -----------------------------------------------------------------------*/
    index2 = 0;
    for (index = 0; index < num_outs; index++)
    {
        if (c_getfiles(outfiles[index],found_files,&nfiles,NUMFILES,1,0)
                != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-file", "An error occurred while "
                    "checking disk for existing output LAS file name");
        }

        if (nfiles == 1)
        {
            strcpy(outfiles_that_exist[index2],outfiles[index]);
            index2++;
        }
    }

    if (index2 > 0)
    {
        for (index = 0; index2 > index; index++)
        {
            c_lasmsg(FALSE, "asterhdf2las-file", "File %s already exists", 
                    outfiles_that_exist[index]);
        }
        return (E_FAIL);
    }


    /* Select a LAS data type based on SD data type
       ----------------------------------------------*/
    for (index=0; index < num_bands; index++)
    {
        switch (dtype[index])
        {
            case DFNT_UINT8:
                dtype[index] = EBYTE;
                break;
            case DFNT_UINT16: 
            case DFNT_INT16:
                dtype[index] = EWORD;
                break;
            case DFNT_FLOAT32:
                dtype[index] = EREAL;
                break;
            default: 
                {
                    c_lasmsg(FALSE, "asterhdf2las-type",
                            "HDF image contains the HDF data type %ld which is "
                            "not supported by this routine", dtype[index]);
                    return (E_FAIL);
                }
        }
    }

    /* Get tags common to all products
       ---------------------------------*/ 
    /* Get time
       ----------*/
    ret = PGS_MET_GetPCAttr(fileid,1,"coremetadata.0","TIMEOFDAY",
            &acquisition_time); 
    if (ret != PGS_S_SUCCESS) 
    {
        c_lasmsg(FALSE, "asterhdf2las-warn", "Error retrieving TIMEOFDAY. "
                "Capture time will not be added to the DDR");
        strcpy(acquisition_time, " ");
    }
    else
    {
        /* The ASTER Level 1 Data Products Spec shows that in the metadata
           TIMEOFDAY should be in hhmmssd->dZ format. However some of the
           Level 1 and Level 2 data products have hh:mm:ss.d->dZ as the format.
           Eliminate ':' and proceed.
           NOTE: Values for hour (hh) are 0-23, minute (mm) are 0-59,
           seconds (ss) are 0-59. The d->d is an n-character subfield (n <= 6)
           for decimal fraction of a second, with each digit in range 0-9.
           This will not be used. Finally Z is a terminator.
           ------------------------------------------------------------------*/
        /* first make sure the string length is large enough */
        if (strlen (acquisition_time) < 6)
        {
            c_lasmsg(FALSE, "asterhdf2las-warn", "The string length of the "
                    "HDF tag TIMEOFDAY must be at least 6 characters long. "
                    "Capture time will not be added to the DDR");
            strcpy(acquisition_time, " ");
        }

        acquisition_time_ptr = acquisition_time;

        if (acquisition_time_ptr == NULL)
        {
            c_lasmsg(FALSE, "asterhdf2las-time",
                    "Error retrieving acquisition time");
            return (E_FAIL);
        }

        if (strchr(acquisition_time_ptr, ':') != NULL)
        {
            strncat(temp_acquisition_time, acquisition_time_ptr,2);
            strncat(temp_acquisition_time,acquisition_time_ptr+3,2);
            strncat(temp_acquisition_time,acquisition_time_ptr+6,2);
        }
        else
        {
            strncat(temp_acquisition_time,acquisition_time_ptr,6);
        }

        strcpy(acquisition_time,temp_acquisition_time);
    }

    /* Get date
       ----------*/
    ret = PGS_MET_GetPCAttr(fileid,1,"coremetadata.0","CALENDARDATE",
            &acquisition_date); 

    if (ret != PGS_S_SUCCESS) 
    {
        c_lasmsg(FALSE, "asterhdf2las-warn", "Error retrieving CALENDARDATE. "
                "Capture date will not be added to the DDR.");
        strcpy(acquisition_date, " ");
    }
    else
    {
        /* The ASTER Level 1 Data Products Spec shows that in the metadata
           CALENDARDATE should be in YYYYMMDD format. However some of the 
           Level 1 and Level 2 data products have YYYY-MM-DD as the format. 
           Eliminate '-' and proceed.
           -------------------------------------------------------------------*/ 
        /* first make sure the string length is large enough */
        if (strlen (acquisition_date) < 8)
        {
            c_lasmsg(FALSE, "asterhdf2las-warn", "The string length of the "
                    "HDF tag CALENDARDATE must be at least 8 characters long. "
                    "Capture date will not be added to the DDR");
            strcpy(acquisition_date, " ");
        }

        acquisition_date_ptr = acquisition_date;

        if (acquisition_date_ptr == NULL)
        {
            c_lasmsg(FALSE, "asterhdf2las-date",
                    "Error retrieving acquisition date");
            return (E_FAIL);
        }

        if (strchr(acquisition_date_ptr, '-') != NULL)
        {
            strncat(temp_acquisition_date, acquisition_date_ptr,4);
            strncat(temp_acquisition_date,acquisition_date_ptr+5,2);
            strncat(temp_acquisition_date,acquisition_date_ptr+8,2);
            strcpy(acquisition_date,temp_acquisition_date);
        }
    }

    /* Each available band has its own projection information. Since all bands 
       have the same projection parameters and projection, then only one of the
       bands metadata information will be used to retrieve this information.The
       first bands metadata information will be used. In order to retrieve the
       first bands metadata information, the object name PROJECTIONPARAMETERS
       will be used, but it needs to have the band number added to it. 
       ie. PROJECTIONPARAMETERS1. This is also true for MPMETHOD.
       -----------------------------------------------------------------------*/
    strcpy(projectionparameters, "PROJECTIONPARAMETERS");
    strcpy(mpmethod, "MPMETHOD");

    /* The preceeding '0' is not needed. ie. PROJECTIONPARAMETERS1 not 
       PROJECTIONPARAMETERS01 */
    if ((strchr(bands[0],'0') != NULL) && (strcmp(bands[0],"10") != NULL))
    {
        strncat (projectionparameters, bands[0]+1, 1);
        strncat (mpmethod, bands[0]+1, 1); 
    }
    else
    {
        strcat(projectionparameters,bands[0]);
        strcat(mpmethod,bands[0]);
    }

    /* The band (using the same one above) sensor name is also needed. The
       band sensor flag will indicate which portion of the metadata needs
       to be read in order to retrieve that bands metadata information.
       The band sensor is not needed in retrieving the DEM (product level 3)
       metadata. 
       ---------------------------------------------------------------------*/
    if (*product_level != L3)
    {
        if ((strcmp(bands[0],"01") == 0) || (strcmp(bands[0],"02") == 0) ||
                (strcmp(bands[0],"3N") == 0) || (strcmp(bands[0],"3B") == 0)) 
        {
            strcpy(band_sensor,"VNIR");
        }
        else if ((strcmp(bands[0],"04") == 0) || (strcmp(bands[0],"05") == 0) ||
                (strcmp(bands[0],"06") == 0) || (strcmp(bands[0],"07") == 0) ||
                (strcmp(bands[0],"08") == 0) || (strcmp(bands[0],"09") == 0))
        {
            strcpy(band_sensor,"SWIR");        
        }
        else if ((strcmp(bands[0],"10") == 0) || (strcmp(bands[0],"11") == 0) ||
                (strcmp(bands[0],"12") == 0) || (strcmp(bands[0],"13") == 0) ||
                (strcmp(bands[0],"14") == 0))             
        {
            strcpy(band_sensor,"TIR");
        }
        else
        {
            c_lasmsg(FALSE, "asterhdf2las-meta",
                    "Error setting band sensor flag");
            return (E_FAIL);
        }
    }

    /* The projection units for product Level 1A will always be set to 
       degrees because the projection is geographic.  Otherwise the units 
       are set to meters.
       ---------------------------------------------------------------------*/
    if (*product_level == L1A) 
        proj_unit_code= DEGREE;
    else
        proj_unit_code = METER;

    if (c_unit2string(proj_unit_code,projection_units) != E_SUCC)
    {
        /* c_unit2string displays the error message */
        c_lasmsg(FALSE,"asterhdf2las-warn", "Setting units to invalid");
        ddr_valids[4] = INVAL;
    }
    else
        ddr_valids[4] = VALID;  /* Set projection units to valid */

    /* Initialize projection values to 0.0
       --------------------------------------*/
    for(index = 0; index < 15; index++) /*15 is the # of projection values*/
        projection_values[index] = 0.0;

    /* Set the ddr_valids flag to VALID for the projection values.  Do this
       for Level 1A products because the projection values are geographic. Do 
       this for Level 3 DEM because the projection is always UTM therefore 
       there aren't any projection parameters.
       -----------------------------------------------------------------------*/
    if ((*product_level == L1A) || (*product_level == L3))
        ddr_valids[3] = VALID;

    /* Get the rest of the product specific metadata
       ----------------------------------------------*/
    if ((*product_level == L1A) || (*product_level == L1B))
    {
        if (get_l1a_l1b_metadata(fileid,*product_level,band_sensor,
                    projectionparameters,mpmethod, instrument_name,
                    projection_name, projection_values,ddr_valids) != E_SUCC)
        {
            c_lasmsg(FALSE,"asterhdf2las-meta",
                    "Error retrieving Level 1 metadata");
            return (E_FAIL);
        }
    }
    else if (*product_level == L3)
    {
        if (get_dem_metadata(fileid,instrument_name) != E_SUCC)
        {
            c_lasmsg(FALSE,"asterhdf2las-meta",
                    "Error retrieving Level 3 DEM metadata");
            return (E_FAIL);
        }    
    }
    else
    {
        if (get_l2_metadata(fileid,projectionparameters,mpmethod,
                    instrument_name,projection_name, projection_values,
                    ddr_valids) != E_SUCC)
        {
            c_lasmsg(FALSE,"asterhdf2las-meta",
                    "Error retrieving Level 2 metadata");
            return (E_FAIL);
        }
    }

    /* Get projection code using the projection name. Set ddr_valids 
       flag to invalid if is an unsupported projection
       ------------------------------------------------------------*/
    /* First initialize projection ddr_valids flag to valid */
    ddr_valids[0]=VALID;

    /*Level 1A will always be geographic */
    if (*product_level == L1A) 
    {
        *projection_code = GEO;
    }
    /* Level 3 DEM will always be UTM */
    else if (*product_level == L3)
    {
        *projection_code = UTM;
    }
    else
    {
        if (strcmp(projection_name,"UTM") == 0)
            *projection_code = TM;
        else if (strcmp(projection_name,"LAMCC") == 0)
            *projection_code = LAMCC;
        else if (strcmp(projection_name,"PS") == 0)
            *projection_code = PS;
        else if (strcmp(projection_name,"EQRECT") == 0)
            *projection_code = EQRECT;
        else if (strcmp(projection_name,"SOM") == 0)
            *projection_code = SOM;
        else
        { 
            c_lasmsg(FALSE,"asterhdf2las-meta", "Projection name %s is "
                    "unsupported, set projection code to geographic and "
                    "invalid", projection_name);
            *projection_code = GEO;
            ddr_valids[0] = INVAL;
        }
    }      

    /* There are some questions as to whether the projection parameters
       are correct for projections LAMCC, PS, EQRECT and SOM. See CR 3754.
       So until we hear otherwise asterhdf2las will not support them. 
       UTM will be supported for L3 DEM's, GEO will be supported for L1A
       and TM will be supported for L1B and L2. 
       -------------------------------------------------------------------*/
       if ((*projection_code == LAMCC) || (*projection_code == PS) ||
          (*projection_code == EQRECT) || (*projection_code == SOM))
       {
           c_lasmsg(FALSE, "asterhdf2las-proj",  
                    "Projection %s has not been tested and will not be "
                    "supported until it is known if the parameters listed "
                    "in the metadata file are valid", projection_name);
           c_lasmsg(FALSE, "asterhdf2las-proj",
                   "Contact the LAS Administrator who will check the validity " 
                   "this ASTER data.");

           return (E_FAIL);
       }

    /* Get the corner coordinates 
       ----------------------------*/
    get_corner_coordinates(fileid,*projection_code,*product_level,zone,
                          corner_coordinates,ddr_valids, projection_values);

    return (E_SUCC);
}
        
