/*******************************************************************************
NAME:           GET_PAR

PURPOSE:        This function retrieves the parameters for TAE and verifies
                that they are correct.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT: LAS

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

int get_par
(
    struct PARBLK *vblock,    /* I: TAE parameter block                      */
    char infile_host[],       /* O: Input HDF filename in host format        */
    char outfiles[][CMLEN],   /* O: Output LAS filename(s) in host format    */
    int *num_outs,            /* O: Count of outfiles                        */
    char bands[][3],          /* O: Bands requested by user                  */
    int *num_bands,           /* O: Count of bands                           */
    int band_groupings[],     /* O: Grouping of bands requested by the user  */
    int *num_band_groupings,  /* O: Count of band_groupings                  */
    double window[],          /* O: Window values requested by user          */
    int *num_window,          /* O: Count of window values                   */
    char units[],             /* O: Value from units field (DEG,DMS,PRO,LS)  */
    int *delflg,              /* O: Flag for global value DELFLG (TRUE to
                                    remove intermediate files or FALSE)      */
    long *msgtim,             /* O: Time between display messages in seconds */
    int *process_all_bands,   /* O: Flag indicating if all band in file should
                                    be processed (TRUE or FALSE)             */
    int *window_specified     /* O: Flag indicating if the window field was
                                    specified                                */
)

{
    char temp_str[4];         /* Temporary string for band number and delflg */
    int index;                /* Looping variable                            */ 
    int index_2;              /* Looping variable                            */
    int sum;                  /* Sum of the band_groupings values            */ 
    long access = IREAD;      /* Type of I/O access                          */
    long element = 0;         /* Element of parameter                        */
    struct VARIABLE *val;     /* Pointer to TAE variable within the parameter 
                                 block                                       */

    /* Get TAE global DELFLG
    -----------------------*/
    val = p_find (vblock, "$DELFLG");
    strcpy(temp_str, SVAL(*val,0)); 
    if (strncmp (temp_str, "YES", 3) == 0)
        *delflg = TRUE;
    else
        *delflg = FALSE;
   
    /* Get TAE global MSGTIM 
    -----------------------*/
    val = p_find (vblock, "$MSGTIM");
    *msgtim = IVAL(*val, 0);

    /* Get input HDF file name
    -------------------------*/
    if (c_getnam(vblock, "INFILE", &element, infile_host, "", &access)!=E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-par", "Error retrieving INFILE");
        return (E_FAIL);
    } 

    /* Get OUT file name(s) and its count
    ------------------------------------*/
    if (c_get_multiple_outs(vblock,NUMBANDS,"OUT",outfiles,num_outs) != E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-getpar", "Error retrieving OUT");
        return (E_FAIL);
    } 

    /* Get requested BANDS and its count
    ----------------------------------*/ 
    val = p_find(vblock, "BANDS");
    *num_bands = val->v_count;

    if (*num_bands == 0)
        *process_all_bands = TRUE;
    else
    {
        for (index = 0; index < *num_bands; index++)
            strcpy(bands[index],SVAL(*val,index));
        *process_all_bands = FALSE;
    }

    /* Check for duplicate BANDS numbers specifications
    --------------------------------------------------*/
    for (index = 0; index < *num_bands; index++)
    {
        strcpy(temp_str,bands[index]);
        for (index_2 = index + 1; index_2 < *num_bands; index_2++)
        {
            if (strcmp(temp_str,bands[index_2]) == 0)
            {
                c_lasmsg(FALSE, "asterhdf2las-par", 
                         "Band %s can only be specified once", temp_str);
                return(E_FAIL);
            }
        }
    }  
 
    /* Get requested NBANDS, its count and do some error checking
    -----------------------------------------------------------*/
    val = p_find(vblock, "NBANDS");
    *num_band_groupings = val->v_count;  /* Get count of band groupings */

    if (*num_band_groupings > 0)
    { 
        /* Get sum of num_band_groupings */
        sum = 0;
        for (index = 0; index < *num_band_groupings; index++)
        {
            band_groupings[index] = IVAL(*val, index);
            sum = sum + band_groupings[index];
        }
   
        /* Make sure the sum of NBANDS equals the total count of BANDS */
        if (sum != *num_bands)
        {    
            c_lasmsg(FALSE, "asterhdf2las-par", 
                     "The sum of NBANDS must equal the total count of BANDS");
            return (E_FAIL);
        } 

        /* Make sure the count of OUT equals the count of NBANDS */
        if (*num_outs != *num_band_groupings)
        {
            c_lasmsg(FALSE, "asterhdf2las-par",
                     "The count of OUT must equal the count of NBANDS");
            return (E_FAIL);
        }
    }
    else
    {
        if (*num_bands > 0)
        {
            /* Make sure the count of OUT (if the count of OUT is not 1) 
               equals the count of BANDS */
            if ((*num_outs != *num_bands) && (*num_outs != 1))
            {
                c_lasmsg(FALSE, "asterhdf2las-par",
                         "The count of OUT must equal the count of BANDS");
                return (E_FAIL);
            }

            /* Since NBANDS was not specified and BANDS was, it is assumed
            that all bands will go into one band grouping. (If there is only
            one output name) Set num_band_groupings to 1 and band_groupings to
            the count of bands */
            if (*num_outs == 1)
            {
                *num_band_groupings = 1;
                *band_groupings = *num_bands;
            }

            /* If the count of outs is greater than one, then the count of 
            outs must equal the count of bands.  If this is true, then set
            band_groupings to one band per file */
            else 
            {
                if (*num_outs == *num_bands)
                {
                    *num_band_groupings = *num_outs;
                    for (index = 0; index < *num_band_groupings; index++)
                        band_groupings[index] = 1;
                }
            }
        }
        else
        {
            /* Do some error checking */
            if (*num_outs != 1)
            {
                c_lasmsg(FALSE, "asterhdf2las-par",
                         "There can be only one OUT name");
                c_lasmsg(FALSE, "asterhdf2las-par",
                         "if BANDS and NBANDS is not specified.");
                return (E_FAIL);
            }
        }
    }

    /* Get WINDOW values, its count and do some error checking
    ---------------------------------------------------------*/
    val = p_find(vblock,"WINDOW");
    *num_window = val->v_count;      /* Get count of window values */

    if (*num_window == 0)
    {
        *window_specified = FALSE;
        strcpy(units,"LS");
    }
    else
    {
        *window_specified = TRUE;
        for (index = 0; index < *num_window; index++)
            window[index] = RVAL(*val, index);
        val = p_find(vblock,"UNITS");
        strcpy(units, SVAL(*val, 0));
      
        /* Do some error checking */ 
        if (strcmp(units,"LS") == 0)
        {
            if (*num_window != 2 && *num_window != 4)
            {
                c_lasmsg(FALSE, "asterhdf2las-par", 
                         "WINDOW specification must have 2 or 4 values");
                return (E_FAIL);
            }
        }
        else 
        {
            if (*num_window != 4)
            {
                c_lasmsg(FALSE, "asterhdf2las-par",
                         "WINDOW specification must have 4 values");
                return (E_FAIL);
            }
        }

    }             
    return (E_SUCC);
}
