/******************************************************************************
NAME:           nonexisting_bands

PURPOSE:        This function will warn the user of nonexisting bands. The
                band array, band grouping array and output LAS image array 
                will be adjusted according to the nonexisting bands.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

int nonexisting_bands
(
    char nonex_bands[][3],      /* I: Bands requested that do not exist in 
                                      in the HDF file                        */
    int num_nonex_bands,        /* I: Count of nonexisting bands             */
    char bands[][3],            /* I/O: Bands to be processed                */
    int *num_bands,             /* I/O: Count of bands                       */
    int band_groupings[],       /* I/O: The grouping of bands. Each band 
                                        grouping represents one LAS image    */
    int *num_band_groupings,    /* I/O: Count of band_groupings              */
    char outfiles[][CMLEN],     /* I/O: Output LAS filename(s)               */
    int *num_outs               /* I/O: Number of output files               */
)

{
    char temp_outfiles[NUMBANDS][CMLEN]; /* Temporary array of adjusted 
                                            output LAS filename(s)           */
    char tmp_bands[NUMBANDS][NUMBANDS][3]; /* Temporary band array           */
    int adj_bands_index;        /* Index used for adjusted bands array       */
    int b_index;                /* Index used for band array                 */
    int bands_in_band_groupings;/* Total number of bands in each band 
                                   grouping                                  */
    int bg_index;               /* Index used for band_grouping array        */
    int i;                      /* Looping variable                          */
    int nonex_index;            /* Index used for nonexisting array          */
    int tmp_band_groupings[NUMBANDS]; /* Temporary array of adjusted band 
                                         grouping values                     */
 
    /* Warn user of nonexisting bands
    -------------------------------*/
    for (i = 0; i < num_nonex_bands; i++)
    {
        c_lasmsg(FALSE, "asterhdf2las-warn", 
             "Band %s is not available for processing", nonex_bands[i]);
    }

    /* Group bands according to band groupings into a temporary 2 dimensional 
       array so it can be easily compared.
    -----------------------------------------------------------------------*/
    b_index = 0;
    for (bg_index = 0; bg_index < *num_band_groupings; bg_index ++)
    {
        for (i = 0; i < band_groupings[bg_index]; i++)
        {
            strcpy (tmp_bands[bg_index][i], bands[b_index]);
            b_index = b_index + 1;
        }
    }

    /* Using the nonexist bands array, compare it to the temporary 2
       dimensional band array.  Leave blank the bands that do not exist.
    ------------------------------------------------------------------*/
    for (nonex_index = 0; nonex_index < num_nonex_bands; nonex_index ++)
    {
        for (bg_index = 0; bg_index < *num_band_groupings; bg_index ++)
        {
            for (i = 0; i < band_groupings[bg_index]; i++)    
            {
                if(strcmp(tmp_bands[bg_index][i],nonex_bands[nonex_index]) == 0)
                    strcpy(tmp_bands[bg_index][i], " ");
            }
        }
    }
    
    /* Total the new total number of band groupings that exist and copy it 
       to a temporary band groupings array.
    -------------------------------------------------------------------*/ 
    for (bg_index = 0; bg_index < *num_band_groupings; bg_index ++)
    {
        bands_in_band_groupings = 0;
        for (i = 0; i < band_groupings[bg_index]; i++)
        {
            if (strcmp (tmp_bands[bg_index][i], " ") != 0)
                bands_in_band_groupings ++;
        }
        tmp_band_groupings[bg_index] = bands_in_band_groupings;
    }

    /* Set bands to empty
    --------------------*/
    for (b_index = 0; b_index < *num_bands; b_index++)
        strcpy(bands[b_index], "");

    /* Now copy the temporary bands to bands
    ---------------------------------------*/
    adj_bands_index = 0;
    for (bg_index = 0; bg_index < *num_band_groupings; bg_index++)
    {
        for (i = 0; i < band_groupings[bg_index]; i++)
        {
            if (strcmp(tmp_bands[bg_index][i], " ") != 0)
            {
                strcpy(bands[adj_bands_index],tmp_bands[bg_index][i]);
                adj_bands_index++;
            }
        }
    } 

    /* Initialize band groupings to 0
    ---------------------------------*/
    memset(band_groupings, 0, NUMBANDS * sizeof(int));

    /* Now copy the temporary band groupings to band groupings and the  
       outfiles to temporary outfiles
    ------------------------------------------------------------------*/ 
    i = 0;
    for (bg_index = 0; bg_index < *num_band_groupings; bg_index ++)
    {
        if (tmp_band_groupings[bg_index] != 0)
        {
            band_groupings[i] = tmp_band_groupings[bg_index];
            strcpy(temp_outfiles[i], outfiles[bg_index]); 
            i++;
        }
    }

    /* The number of outs is equal to i
    ----------------------------------*/
    *num_outs = i;

    /* Copy temporary outfiles back to outfiles array
    ------------------------------------------------*/
    for (i = 0; i < *num_outs; i++)
        strcpy(outfiles[i], temp_outfiles[i]);

    /* Get the ajusted count of band groupings
    -----------------------------------------*/
    *num_band_groupings = i;   

    /* Get the adjusted count of bands
    ---------------------------------*/
    *num_bands = adj_bands_index;

    return (E_SUCC);
}
