/******************************************************************************
NAME:           process_hdf_image

PURPOSE:        This function will read the HDF image and output it to a 
                LAS image(s).

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "PGS_MET.h"
#include "PGS_tk.h"
#include "PGS_SMF.h"
#include "las.h"
#include "geompak.h"
#include "lasmath.h"
#include "util.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"
#include "process_hdf_image.h"


#define NUM_COORDINATES 8     /* the total number of coordinates             */
#define MAX_LINES_IN_BLOCK 20 /* Number of lines in a proccessing block      */

extern long curent;           /* current processing line */
extern long curcat;           /* current processing band */

void get_corners
(
    double *in_corners,     /* input image data corners:
                                0:ul y, 1:ul x
                                2:ur y, 3:ur x
                                4:ll y, 5:ll x
                                6:lr y, 7:lr x */
    double *ls_corners,     /* line/sample corners as above */
    double *window,         /* image window */
    double *out_corners     /* output image data corners as above */
)
{
    /* file names 
       ----------*/
    long one = 1;           /* one */
    long i;                 /* loop control variable */
    double A[16];           /* matrix of line/sample of qr function */
    double projy[4];        /* vector of line search space */
    double projx[4];        /* vector of sample serach space */
    double v[4];
    double sample;
    double line;


    /* Get assign line/sample values to the matrix
       -------------------------------------------*/
    for (i = 0; i < 4; i++)
    {
        A[i] = 1.0;
        A[i+4] = ls_corners[i*2 + 1];
        A[i+8] = ls_corners[i*2];
        A[i+12] = A[i+4] * A[i+8];
    }

    /* assign x/y values
       ----------------*/

    projx[0] = in_corners[UPLEFT_X];
    projx[1] = in_corners[UPRIGHT_X];
    projx[2] = in_corners[LOLEFT_X];
    projx[3] = in_corners[LORIGHT_X];
    projy[0] = in_corners[UPLEFT_Y];
    projy[1] = in_corners[UPRIGHT_Y];
    projy[2] = in_corners[LOLEFT_Y];
    projy[3] = in_corners[LORIGHT_Y];

    /* find the polynomial values by QR factorization
       --------------------------------------------*/
    c_qrdecomp (A, 4, 4, v, 0);
    c_qrsolve (A, 4, 4, v, projy, 0);
    c_qrsolve (A, 4, 4, v, projx, 0);

    /* find the ul corner projection x/y value
       ---------------------------------------*/
    line = window[SL];
    sample = window[SS];
    out_corners[UPLEFT_Y] = c_eval (&one, projy, &sample, &line);
    out_corners[UPLEFT_X] = c_eval (&one, projx, &sample, &line);

    /* find the ur corner projection x/y value
       ---------------------------------------*/
    line = window[SL];
    sample = window[NS] + window[SS] - 1;
    out_corners[UPRIGHT_Y] = c_eval (&one, projy, &sample, &line);
    out_corners[UPRIGHT_X] = c_eval (&one, projx, &sample, &line);

    /* find the lr corner projection x/y value
       ---------------------------------------*/
    line = window[NL] + window[SL] - 1;
    sample = window[NS] + window[SS] - 1;
    out_corners[LORIGHT_Y] = c_eval (&one, projy, &sample, &line);
    out_corners[LORIGHT_X] = c_eval (&one, projx, &sample, &line);

    /* find the ll corner projection x/y value
       ---------------------------------------*/
    line = window[NL] + window[SL] - 1;
    sample = window[SS];
    out_corners[LOLEFT_Y] = c_eval (&one, projy, &sample, &line);
    out_corners[LOLEFT_X] = c_eval (&one, projx, &sample, &line);

}
/* routine to convert corner coordinate reference points from the values 
   in the ASTER HDF file to be referenced to the center of the four corner
   points of the image needed for LAS.  The ASTER corners are messed up in two
   different ways:
       1) The corners points are relative to the center of the 15 meter pixels
          and the 30 meter and 90 meter bands pixel edges are lined up with the
          edge of the 15 meter pixels.  A crude diagram is:
            --------------------------
            |15m|    |               |
            | x |    |               |
            |   |    |               |
            |---|    |               |
            | 30m    |               |
            |        |               |
            |--------|               |
            |                        |
            |   90m pixel            |
            |                        |
            |                        |
            |                        |
            |                        |
            --------------------------
          where x is the 15m pixel center.  So, for the 30 meter and 90 meter
          bands, the corner points have to be adjusted to be at the center of
          the pixels.
       2) The corner points for the upper right and lower right corners are the
          coordinate of the pixel that lies 1 pixel past the right edge of the
          image.  Likewise, the lower left and right corner coordinates are 1 
          pixel past the bottom edge of the image.
   Both of these issues need to be adjusted for to make the LAS image corners
   be reported correctly. 
*/
int convert_coordinates
(
    double *raw_corners,/* I: corners of the original HDF file(ul,ll,ur,lr y/x)*/
    int lines,          /* I: number of lines in the input image */
    int samples,        /* I: number of samples across the input image */
    int pixel_size,     /* I: size of pixels in meters */
    double *corners     /* O: adjusted corners */
)
{
    double ls_coords[8]; /* line/sample coordinates in ul,ll,ur,lr order */
    double window[4]; /* window coordinates to use to adjust the corner points */
    double factor;    /* factor between the 15 meter nominal resolution and the
                         current resolution */
    double temp_corners[8];
    double out_corners[8];
    double offset;
    double temp;

    factor = pixel_size / 15.0;

    temp_corners[0] = raw_corners[0];
    temp_corners[1] = raw_corners[1];
    temp_corners[2] = raw_corners[4];
    temp_corners[3] = raw_corners[5];
    temp_corners[4] = raw_corners[2];
    temp_corners[5] = raw_corners[3];
    temp_corners[6] = raw_corners[6];
    temp_corners[7] = raw_corners[7];

    /* load the line/sample coordinates for calculating the coefficients.
       Note that the coordinate system being used defines the upper left corner
       pixel to be 1,1 since c_rotcor assumes the upper left corner of the image
       is at 1,1 */
    ls_coords[0] = 1;
    ls_coords[1] = 1;
    ls_coords[2] = 1;
    ls_coords[3] = (samples * factor) + 1;
    ls_coords[4] = (lines * factor) + 1;
    ls_coords[5] = 1;
    ls_coords[6] = (lines * factor) + 1;
    ls_coords[7] = (samples * factor) + 1;

    /* specify the window to calculate the corner coordinates for.  Note this
       depends on the resolution since a different adjustment needs to be made
       due to how the pixels are lined up. */
    offset = ((pixel_size - 15.0)/2.0)/15.0;
    window[SL] = 1.0 + offset;
    window[SS] = window[SL];
    temp = lines * factor - offset;
    window[NL] = temp - window[SL] + 1;
    temp = samples * factor - offset;
    window[NS] = temp - window[SS] + 1;

    /* calculate the real corners coordinates */
    get_corners(temp_corners,ls_coords,window,out_corners);

    corners[0] = out_corners[0];
    corners[1] = out_corners[1];
    corners[2] = out_corners[4];
    corners[3] = out_corners[5];
    corners[4] = out_corners[2];
    corners[5] = out_corners[3];
    corners[6] = out_corners[6];
    corners[7] = out_corners[7];

    return E_SUCC;
}


int process_hdf_image
(
    int32 sd_id,              /* I: SD interface identifier                  */
    int32 sds_index[],        /* I: Index of data set                        */ 
    double window[],          /* I: Window values requested by user          */
    int projection_code,      /* I: Value will be either TM=9, LAMCC=4, 
                                    PS=6, EQRECT=17, SOM=22, GEO=0           */
    double projection_values[],/* I: Projection parameters of the scene     */
    double input_coordinates[],/* I: Coordinates of the scene, no windowing 
                                      is applied. Value is degrees or meters */
    int projection_distance[],/* I: Resolution of each band                  */ 
    char projection_units[],  /* I: Projection units (degrees or meters)     */ 
    int ddr_valids[],         /* I: Flags indicating if fields in the DDR are
                                    valid or not (0=not valid, 1=valid)      */
    char outfiles[][CMLEN],   /* I: Output LAS filename(s) in host format    */ 
    int band_groupings[],     /* I: Number of bands in each band grouping    */
    int nl[],                 /* I: Total number of lines for each band      */
    int ns[],                 /* I: Total number of samples for each band    */
    char infile[],            /* I: Input HDF filename in host format        */
    int window_specified,     /* I: Flag indicating if the window field was
                                    specified (TRUE or FALSE)                */
    int num_band_groupings,   /* I: Count of band groupings                  */
    int data_type[],          /* I: Data type of the each band in the HDF 
                                    image. The output image will be the same
                                    data type as the input image.
                                    Value are either EBYTE or EWORD          */
    long msgtim,              /* I: Time between display messages            */
    char window_units[],      /* I: Units the window was specified in        */
    int num_window,           /* I: Count of window values                   */
    char bands[][3],          /* I: Bands to be processed                    */
    char instrument_name[],   /* I: Instrument name ie ASTER                 */
    char acquisition_date[],  /* I: Capture date                             */
    char acquisition_time[],  /* I: Capture time                             */
    int product_level,        /* I: Values can be Level 1A=L1A, Level 1B=L1B
                                    Level 2=L2 or Level 3 DEM=L3             */
    char kinetic_temp[],      /* I: Indicates if product is kinetic 
                                    temperature (yes or no)                  */
    long zone,                /* I: Zone code for UTM projection             */
    char processed_files[][CMLEN], /* O: array of filenames created for 
                                         output from this application        */
    int *num_processed_files  /* O: count of out files that have been
                                    processed                                */
)

{
    char bands_in_outfile[NUMBANDS][3]; /* bands numbers in the LAS outfile  */
    double ddr_coordinates[NUM_COORDINATES];
                              /* original corner coordinates before 
                                 windowing is applied                        */
    int b_index = 0;          /* index variable used for band looping        */
    int bg_index;             /* index variable used for band grouping loop  */
    int block_index;          /* index value for looping on number of blocks */
    int index2;               /* index variable used for looping             */
    int index=0;              /* index variable                              */ 
    int sd_index = 0;         /* index variable used for the sds_index       */
    int32 edges[2];           /* number of data elements to be read          */
    int32 sds_id;             /* data set identifier                         */
    int32 start[2];           /* position at which read will start           */
    intn status;              /* status for functions returning an intn      */
    long acc = IWRITE;        /* Image file write access                     */
    long flag = ONECAT;       /* Display message flag                        */
    long line_num_to_write;   /* the first line number of the block that is
                                 to be written out to the LAS image          */
    long lines_in_block=MAX_LINES_IN_BLOCK; 
                              /* Maximum number of lines to be read at a time*/
    long one = 1;             /* constant of 1 used for sl and ss for
                                 c_ewrite and c_eopenr                       */
    long valid_window[4];     /* window values in line/sample                */
    long opt[3]={0,0,0};      /* Image file open option                      */
    struct FDESC *fdesc;      /* Image file descriptors used for image I/O   */
    unsigned char *hdfbuf;    /* Input buffer                                */

    double corner_coordinates[8];

    /* Initialize to 0
    -----------------*/
    *num_processed_files = 0;

    /* Loop through each band grouping
    --------------------------------*/
    for(bg_index = 0; bg_index < num_band_groupings; bg_index++)
    {
        /* adjust the corner coordinates from the ASTER HDF locations to 
           the locations LAS expects. */
        if (convert_coordinates(input_coordinates, nl[index], ns[index], 
                projection_distance[index],corner_coordinates) != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-wind",
                     "Error encountered while adjusting corner coordinates");
            return (E_FAIL);
        }

        /* Verify that the window parameters are valid and if the input units
           are not in line/sample, convert the window values to line/sample.
        --------------------------------------------------------------------*/
        if (convert_verify_window(window_specified,window,window_units,
                                  num_window,nl[index],ns[index],
                                  projection_code,projection_values,
                                  corner_coordinates,projection_distance[index],
                                  product_level,zone,valid_window) != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-wind",
                     "Error encountered while verifying window parameters");
            return (E_FAIL);
        } 


        /*for example if band_groupings[0]=2 then use data_type[2] info*/
        index = index+band_groupings[bg_index];

        /* Allocate memory for the input buffer 
        --------------------------------------*/
        if ((hdfbuf=(unsigned char *)malloc(valid_window[NS]
                     *data_type[index-1]*lines_in_block)) == NULL)  
        {
            c_lasmsg(FALSE, "asterhdf2las-alloc", 
                     "Error allocating dynamic memory");
            return (E_FAIL);
        }  

        /* Initialize the processing display message
        ------------------------------------------*/
        curcat=1;
        curent=1;
        c_inittm("ASTERHDF2LAS","Processed","line","bands"," ",
                  &valid_window[NL], (long *)&band_groupings[bg_index],
                  &msgtim,&flag); 


        /* Start display message utility
        ------------------------------*/
        c_startm(); 

        /* Open the output file
        ----------------------*/
        if (c_eopenr(&fdesc,outfiles[bg_index],(long*)&band_groupings[bg_index],
                     &one,&one,&valid_window[NL],&valid_window[NS],&acc,
                     (long*)&data_type[index-1],opt,&lines_in_block) != E_SUCC) 
                     
        {
            c_lasmsg(FALSE, "asterhdf2las-open", 
                     "Error opening output image %s", outfiles[bg_index]); 
            free (hdfbuf); 
            return (E_FAIL);
        }

        /* Add the output filename to the list of files that have been
           created
        --------------------------------------------------------------*/
        strcpy(processed_files[bg_index], outfiles[bg_index]);
        *num_processed_files = *num_processed_files + 1;

        /* Loop through each band in the band grouping
        ----------------------------------------------*/
        for (curcat = 1; curcat <= band_groupings[bg_index]; curcat++)
        {
            /* Initialize to 1 for line 1
            ----------------------------*/
            line_num_to_write = 1;

            curent = 1;

            /* Select the data set
            ----------------------*/
            sds_id = SDselect(sd_id,sds_index[sd_index]);
            sd_index++;

            /* Set elements of array start to 0, elements of array edges
            to SDS dimensions,and use NULL for the argument stride in 
            SDreaddata to read the entire data.
            -----------------------------------------------------------------*/
            start[0] = valid_window[SL]-1; /* -1 because line starts at 0 */ 
            start[1] = valid_window[SS]-1; /* -1  because sample starts at 0 */
            edges[0] = lines_in_block;  /* read # of lines_in_block at a time */
            edges[1] = (int32)valid_window[NS]; 
            

            /* Loop on each number of blocks and read into a data array
            ----------------------------------------------------------*/
            for (block_index=0; block_index < valid_window[NL]; 
                              block_index = block_index + MAX_LINES_IN_BLOCK)
            {
                /* If the block of lines in the image is less than
                MAX_LINES_IN_BLOCK then the lines in block will be
                be recalculated */
                if (line_num_to_write > valid_window[NL] - MAX_LINES_IN_BLOCK)
                {
                    lines_in_block = valid_window[NL] - line_num_to_write + 1;
                    edges[0] = lines_in_block;
                }

                status = SDreaddata(sds_id,start,NULL,edges,(VOIDP)hdfbuf);
                if (status == HDF_FAIL)
                {
                    c_lasmsg(FALSE, "asterhdf2las-read",
                           "Error reading image data from file %s",infile);
                    status = SDendaccess (sds_id);
                    if (status == HDF_FAIL)
                    {
                        c_lasmsg(FALSE, "asterhdf2las-warn","Error terminating"
                            " access to Scientific Data Set in %s", infile);
                    }
                    status = SDend (sd_id);
                    if (status == HDF_FAIL)
                    {
                        c_lasmsg(FALSE, "asterhdf2las-warn",
                                "Error closing file %s", infile);
                    }
                    if (c_eclose(&fdesc) != E_SUCC)
                    {
                        c_lasmsg(FALSE, "asterhdf2las-warn",
                                "Error closing file %s",outfiles[bg_index]);
                    }
                    free(hdfbuf);
                    return (E_FAIL);
                }

                /* Write to file */
                status = c_ewrite(&fdesc,&curcat,&line_num_to_write,
                                  hdfbuf,&lines_in_block);

                if (status != valid_window[NS]*lines_in_block)
                {
                    c_lasmsg(FALSE, "asterhdf2las-wind","Error writing to %s",
                             outfiles[bg_index]);
                    status = SDendaccess (sds_id);
                    if (status == HDF_FAIL)
                    {
                        c_lasmsg(FALSE, "asterhdf2las-warn","Error terminating"
                            " access to Scientific Data Set in %s", infile);
                    }
                    status = SDend (sd_id);
                    if (status == HDF_FAIL)
                    {
                        c_lasmsg(FALSE, "asterhdf2las-warn",
                                "Error closing file %s", infile);
                    }

                    if (c_eclose(&fdesc) != E_SUCC)
                    {
                        c_lasmsg(FALSE, "asterhdf2las-warn",
                                "Error closing file %s",outfiles[bg_index]);
                    }
                    free(hdfbuf);
                    return (E_FAIL);
                }
                line_num_to_write = line_num_to_write + lines_in_block;
                start[0] = start[0] + lines_in_block;
                curent = curent + lines_in_block;
            }

            /* Reset lines in block back to the maximum number of line
               assigned to the block to process the next band
            ---------------------------------------------------------*/
            lines_in_block = MAX_LINES_IN_BLOCK;


            /* Terminate access to the data set and to the SD interface
            ----------------------------------------------------------*/
            status = SDendaccess (sds_id);
            sds_id = NULL;
            if (status == HDF_FAIL)
            {
                c_lasmsg(FALSE, "asterhdf2las-warn",
                  "Error terminating access to SD data set in file %s", infile);
            }
        }
  
        /* Get band numbers that apply to outfiles 
        ------------------------------------------*/
        for (index2 = 0; index2 < band_groupings[bg_index]; index2++)
        {
            strcpy(bands_in_outfile[index2], bands[index2+b_index]);
        }     
        b_index = b_index+band_groupings[bg_index];

        /* Save the original corner coordinates. This is done because the 
           corner coordinates values change, (if windowing is applied), to 
           reflect the recalculated window coordinates. The original corner
           coordinates are needed for the processing of the next band grouping.
        ---------------------------------------------------------------------*/ 
        for (index2 = 0; index2 < NUM_COORDINATES; index2++)
            ddr_coordinates[index2] = corner_coordinates[index2];

        /* Update the DDR  
        ----------------*/
        if (update_ddr(outfiles[bg_index],valid_window,bands_in_outfile,
                       band_groupings[bg_index],data_type[index-1], 
                       projection_code,projection_values,window_specified,
                       ddr_coordinates,projection_distance[index-1],
                       projection_units,ddr_valids,instrument_name,
                       acquisition_date,acquisition_time,nl[index-1],
                       ns[index-1],product_level,kinetic_temp,zone) != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-ddr", 
                     "An error occurred while updating the DDR");
        }
        free (hdfbuf);
        hdfbuf = NULL;  
        c_eclose(&fdesc);
        fdesc = NULL; 
        curent = valid_window[NL]; 

        /* Stop the display message utility
        -----------------------------------*/
        c_stoptm(); 

    }
    status = SDend (sd_id);
    if (status == HDF_FAIL)
    {
        c_lasmsg(FALSE, "asterhdf2las-warn", "Error closing file %s", infile);
    }
 
    return (E_SUCC);
}
