/******************************************************************************

NAME: recalc_corners

PURPOSE: Recalculate the corner points for a subwindow of an image.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT: LAS

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "asterhdf2las_protos.h"

int recalc_corners
(
    double corners[],         /* I/O: corner points of the image          */
    long nl,                  /* I: original size of the image NL         */
    long ns,                  /* I: original size of the image NS         */
    long window[]             /* I: image window specified                */
)

{
    double coefs[6];              /* transformation coefficients              */
    double proj_coords[8];        /* image corner projection coordinates      */
    long flag = TRUE;             /* c_rtcoef error flag                      */
    long img_coords[8];           /* image corner line,sample coordinates     */
    long line;                    /* current line                             */
    long samp;                    /* current sample                           */
    long status;                  /* function return status                   */

    /* Copy the corner points in the order c_rtcoef wants.
    The corners are (x,y) in UL, UR, LL, LR order
    --------------------------------------------------*/
    proj_coords[0] = corners[1]; proj_coords[1] = corners[0]; /* UL */
    proj_coords[2] = corners[3]; proj_coords[3] = corners[2]; /* LL */
    proj_coords[4] = corners[5]; proj_coords[5] = corners[4]; /* UR */
    proj_coords[6] = corners[7]; proj_coords[7] = corners[6]; /* LR */

    /* set up the image coordinates
    ---------------------------- */
    img_coords[0] = 1;                                        /* UL */
    img_coords[1] = 1;                        
    img_coords[2] = nl;                                       /* LL */
    img_coords[3] = 1;
    img_coords[4] = 1;                                        /* UR */
    img_coords[5] = ns;
    img_coords[6] = nl;                                       /* LR */
    img_coords[7] = ns;        

    /* compute the transformation coefficients
    ----------------------------------------*/
    status = c_rtcoef(proj_coords, img_coords, coefs, &flag);
    if(status == E_FAIL)
    {
        c_lasmsg(FALSE,"asterhdf2las-coef",
           "Error computing the transformation coefficients");
        return (E_FAIL);
    }

    /* now compute the new corner points
    ---------------------------------*/
    /* UL */
    line = window[SL];
    samp = window[SS];
    corners[0] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
    corners[1] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

    /* UR */
    samp += window[NS] - 1;
    corners[4] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
    corners[5] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

    /* LL */
    line = window[SL] + window[NL] - 1;
    samp = window[SS];
    corners[2] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
    corners[3] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

    /* LR */
    samp += window[NS] - 1;
    corners[6] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
    corners[7] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

    return (E_SUCC);
}
