/******************************************************************************
NAME:           update_ddr

PURPOSE:        This function will update the DDR with the information
                obtained from the metadata of the HDF file.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "ddr.h"
#include "geompak.h"
#include "proj.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"
 
#define NUM_VALID_FIELDS DDNVAL  /* Total of valid fields in the ddr         */
#define NUM_PROJ_PARMS 15        /* Total number of projection parameters    */

int update_ddr
(
    char outfile[],              /* I: LAS file name in host format          */
    long window[],               /* I: Window values in lines/samples        */
    char bands[][3],             /* I: Bands in the LAS image                */ 
    int band_groupings,          /* I: Number of bands in the LAS image      */
    int data_type,               /* I: Data type of the LAS image
                                       value is either EBYTE or EWORD        */
    int projection_code,         /* I: Value will be either UTM=1, TM=9,
                                       LAMCC=4, PS=6, EQRECT=17, SOM=22, 
                                       GEO=0                                 */
    double projection_parameters[], /* I: The 15 parameters in the scene     */
    int window_specified,        /* I: Flag indicating if the window field 
                                       was specified (TRUE or FALSE)         */
    double corner_coordinates[], /* I: Coordinates of the scene,windowing isn't
                                       applied. Values are degrees or meters */
    int projection_distance,     /* I: Resolution of the image               */
    char projection_units[],     /* I: Units in degrees or meters            */
    int ddr_valids[],            /* I: Flags indicating if fields in the DDR 
                                       are valid or not (0=not valid,1=valid)*/
    char instrument_name[],      /* I: Instrument name ie ASTER              */
    char acquisition_date[],     /* I: Capture date                          */
    char acquisition_time[],     /* I: Capture time                          */
    int orig_nl,                 /* I: Total lines in the image (no windowing
                                       applied)                              */
    int orig_ns,                 /* I: Total samples in the image (no windowing
                                       applied)                              */
    int product_level,           /* I: Values can be Level 1A=L1A, Level 1B=L1B,
                                       Level 2 =L2 or Level 3 DEM =L3        */
    char kinetic_temp[],         /* I: Indicates if product is kinetic
                                       temperature (yes or no)               */
    long utm_zone                /* I: Zone code for UTM projection          */
)

{
    char date[10];               /* capture date set in DDR standard format  */
    char month[4];               /* month name                               */
    char sensor_name[5];         /* names either VNIR, SWIR, TIR             */
    char temp_instrument_name[32];/* used to hold instrument_name temporarily*/
    char time[8];                /* capture time set in DDR standard format  */
    int index;                   /* index variable used for looping          */
    struct BDDR *bptr;           /* pointer to the current BDDR record       */
    struct BDDR bddrs[NUMBANDS]; /* local bddr structure                     */
    struct DDR ddr;              /* local ddr structure                      */

    /* Take care of DDR information
    ------------------------------*/

    /* Retrieve DDR 
    --------------*/
    if (c_getddr(outfile,&ddr) != E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-ddr", "DDR for %s does not exist",
                 outfile);
        return (E_FAIL);
    }

    /* Fill in the DDR structure
    ---------------------------*/
    ddr.nl = window[NL];
    ddr.ns = window[NS];
    ddr.nbands = band_groupings; 
    ddr.dtype = data_type;
    ddr.proj_code = projection_code; 
    if (projection_code == UTM)
        ddr.zone_code = utm_zone;
    else
    {
        ddr.zone_code = DEFAULT_ZONE;
        ddr_valids[1] = VALID;
    }
    ddr.datum_code = WGS84;
    ddr_valids[2] = VALID;   /* Set datum to valid */

    for (index = 0; index < NUM_PROJ_PARMS; index++)
        ddr.proj_coef[index] = projection_parameters[index];

    /* If windowing was applied then the corner coordinates need to be 
       recalculated
    ------------------------------------------------------------------*/
    if (window_specified == TRUE)
    {
        if (recalc_corners(corner_coordinates,orig_nl,orig_ns,window) != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-coor", 
                     "Error recalculating corner coordinates");
            return (E_FAIL);
        }
                    
    }

    /* Set corner coordinates
    ------------------------*/
    ddr.upleft[0] = corner_coordinates[0];
    ddr.upleft[1] = corner_coordinates[1];
    ddr.loleft[0] = corner_coordinates[2];
    ddr.loleft[1] = corner_coordinates[3];
    ddr.upright[0] = corner_coordinates[4];
    ddr.upright[1] = corner_coordinates[5];
    ddr.loright[0] = corner_coordinates[6];
    ddr.loright[1] = corner_coordinates[7];

 
    /* Set projection distance
    -------------------------*/
    if (product_level != L1A)  
    {
        ddr.pdist_x = (double)projection_distance;
        ddr.pdist_y = ddr.pdist_x;
    }
    else
    {   /* Approximate projection distance in degrees 
        --------------------------------------------*/
        /* ((ULX - URX) / *ns) */ 
        ddr.pdist_x = fabs ((ddr.upleft[1] - ddr.upright[1]) / window[NS]); 
        /* ((ULY - LLY) / *nl) */ 
        ddr.pdist_y = fabs ((ddr.upleft[0] - ddr.loleft[0]) / window[NL]);  
    }
 
    ddr_valids[5] = VALID;    /* Set proj dist to VALID */

    /* Set projection units
    ----------------------*/
    strcpy (ddr.proj_units,projection_units);

    /* master line, master sample, line increment, and sample increment 
       will always be 1
    -------------------------------------------------------------------*/
    ddr.master_line = 1;
    ddr.master_sample = 1;
    ddr.line_inc = 1.0;
    ddr.sample_inc = 1.0;
    ddr_valids[7] = VALID;   /* Set line/sample increments to valid */

    /* Set DDR fields to vaild or invalid by copying ddr_valids array to DDR
    ---------------------------------------------------------------------*/
    for (index = 0; index < NUM_VALID_FIELDS; index++) 
        ddr.valid[index] = ddr_valids[index];

    /* Take care BDDR information
    ----------------------------*/
    
    /* Assign a sensor name as long as it is not a DEM product.  Since all 
       bands are the same resolution, then only the first band needs to be 
       checked
    -----------------------------------------------------------------------*/
    if (product_level != L3)
    {
        if ((strcmp (bands[0],"01") == 0) || (strcmp (bands[0],"02") == 0) ||
            (strcmp (bands[0],"3N") == 0) || (strcmp (bands[0],"3B") == 0))
        {
            strcpy(sensor_name,"VNIR");
        }
 
        else if ((strcmp (bands[0],"04") == 0)||(strcmp (bands[0],"05") == 0) ||
                 (strcmp (bands[0],"06") == 0)||(strcmp (bands[0],"07") == 0) ||
                 (strcmp (bands[0],"08") == 0)||(strcmp (bands[0],"09") == 0))
        {
            strcpy(sensor_name,"SWIR");
        }

        else if ((strcmp (bands[0],"10") == 0)||(strcmp (bands[0],"11") == 0) ||
                 (strcmp (bands[0],"12") == 0)||(strcmp (bands[0],"13") == 0) ||
                 (strcmp (bands[0],"14") == 0))
        {
            strcpy(sensor_name,"TIR");
        }

        else
        {
            c_lasmsg(FALSE, "asterhdf2las-name",
                     "Error assigning a sensor name");
            strcpy(sensor_name, " ");
        }
    }
 
    /* Get the BDDR 
    --------------*/
    for (index = 1; index < band_groupings+1; index++)
    {
        if (c_getbdr(outfile,&bddrs[index-1],(long*)&index) != E_SUCC)
        {
            c_lasmsg(FALSE,"asterhdf2las-bddr",
                     "Error retrieving the BDDR for file %s", outfile);
            return (E_FAIL);
        }
    }

    /* Write to DDR
    --------------*/
    if (c_putddr(outfile,&ddr) != E_SUCC)
    {
        c_lasmsg(FALSE, "asterhdf2las-ddr",
                 "Error writing to DDR for %s",outfile);
        return (E_FAIL);
    }

    /* Store acquisition date according as the DDR standard of ie 31-dec-86
    ----------------------------------------------------------------------*/
    /* get numerical month and get its name*/
    strncpy(month, acquisition_date+4, 2);
    month[2] = '\0';
    if (strcmp (month, "01") == 0)
        strcpy (month, "jan");
    else if (strcmp (month, "02") == 0)
        strcpy (month, "feb");
    else if (strcmp (month, "03") == 0)
        strcpy (month, "mar");
    else if (strcmp (month, "04") == 0)
        strcpy (month, "apr");
    else if (strcmp (month, "05") == 0)
        strcpy (month, "may");
    else if (strcmp (month, "06") == 0)
        strcpy (month, "jun");
    else if (strcmp (month, "07") == 0)
        strcpy (month, "jul");
    else if (strcmp (month, "08") == 0)
        strcpy (month, "aug");
    else if (strcmp (month, "09") == 0)
        strcpy (month, "sep");
    else if (strcmp (month, "10") == 0)
        strcpy (month, "oct");
    else if (strcmp (month, "11") == 0)
        strcpy (month, "nov");
    else if (strcmp (month, "12") == 0)
        strcpy (month, "dec");
    else
    {
        c_lasmsg(FALSE, "asterhdf2las-warn",
                 "Unable to retrieve month from acquisition date");
    }

    strncpy(date,acquisition_date+6,2);
    date[2] =  '\0';  
    strcat(date,"-");
    strcat(date, month);
    strcat(date, "-");
    strncat(date,acquisition_date+2,2);
    date[9] = '\0';

    /* Store acquisition time according as the DDR standard of ie 1305:55
       for 1:05:55 pm
    ----------------------------------------------------------------------*/
    strncpy (time,acquisition_time,2);
    time[2] =  '\0';
    strncat (time,acquisition_time+2,2);
    time[4] = '\0';
    strcat (time, ":");
    strncat (time, acquisition_time+4,2);
    time[7] = '\0';

    /* Fill in the BDDR structure and output to BDDR
    -----------------------------------------------*/
    bptr = bddrs;
    for (index = 0; index < band_groupings; index++)
    {
        bptr->bandno = index+1;
        bptr->valid = INVAL;   /* Since min max is invalid  */
        bptr->minval = 0.0;      /* Since it is not available */
        bptr->maxval = 0.0;      /* Since it is not available */ 
        strcpy(bptr->source, "TERRA");
        strcpy(bptr->direction, " "); /* Since it is not available */
 
        /* Concat instrument name, sensor name and band together */
        strcpy(temp_instrument_name, instrument_name);
        strcat(temp_instrument_name, "_");
        if (strcmp (kinetic_temp,"yes") == 0)
        {
            strcat(temp_instrument_name, "Kinetic_Temperature");
        }
        else if (product_level == L3)
        {
            strcat(temp_instrument_name, "DEM");
        }    
        else
        {
            strcat(temp_instrument_name, sensor_name);
            strcat(temp_instrument_name, "_band_");
            strcat(temp_instrument_name, bands[index]);
        }

        strcpy(bptr->instrument, temp_instrument_name); 
        
        strcpy(bptr->date, date);
        strcpy(bptr->time, time);

        /*Output to BDDR */
        if (c_putbdr(outfile,&bddrs[index]) != E_SUCC)
        {
            c_lasmsg(FALSE,"asterhdf2las-bddr",
                     "Error writing to BDDR for file %s", outfile);
            return (E_FAIL);
        }
        bptr++;
    }

    return (E_SUCC); 
}
