/****************************************************************************
NAME:				AUTOFIT

PURPOSE:  AUTOFIT uses a Tie Point Location file containing tie point pairs
          to derive bivariate polynomials using the method of least squares.
	  Editing of the TPL file is done by the "leave-one-out" method.  
	  Editing continues until the fit "settles-down", that is the RMS
	  errors between two successive fits are less than TOLVAL, the 
	  threshold value.

VERSION	 DATE	AUTHOR	   
-------	 ----	------	  
  1.0	 5/94	D. Steinwand
  1.1	 7/94	T. Mittan			Updated to standards	
  1.2	 1/95	T. Mittan			Modified to use realloc
						as number of Tie points
						increases.

****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

#define TERM 0
#define PFILE 1
#define LP 2


void main
(void)
{
struct TPLHEAD tie_head;	/* Tie point location file header record */
struct PARBLK vblk;		/* TAE parameter block 			*/
struct TPLDATA *tie_data;	/* Ptr to Tie point location data records */
long degree;			/* Degree of polynomial used in fit 	*/
long fatal = -1;		/* fatal error flag			*/
long msgtim;			/* message time				*/
long num_out=1;			/* Number of output files 		*/
long nonfatal = 1;		/* nonfatal error flag			*/
long npts;			/* Number of points in first TPL file 	*/
long pltflg;       	    	/* plot flag                            */
long std_out;			/* Dummy variable to standard output 	*/
float minrms;          		/* Minimum rms value to complete autofit*/
float tolval;			/* Tolerance value 			*/
char filename[CMLEN];         	/* print file name                      */
char tpl[CMLEN];		/* Input Tie Point Location file name 	*/
char outtpl[CMLEN];		/* Output Tie Point Location file name 	*/
char outbuf[2][CMLEN];		/* Tae name of output files for complete*/
char plotfile[CMLEN];         	/* plot filename                        */
char prtfile[CMLEN];          	/* print file name                      */
long print[3];          	/* print option                         */
FILE *fptr;			/* print file pointer			*/



c_inlas(&vblk, &std_out);

/* Allocate buffer space for tie point records
  -------------------------------------------*/
tie_data=(struct TPLDATA *)malloc((sizeof(struct TPLDATA)*MAX_TIE_POINTS));
if (tie_data == NULL) 
   {
   c_errmsg("Unable to allocate dynamic memory","autofit-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","autofit-fatal",&fatal);
   }

if (getpar(&vblk,tpl,outtpl,&degree,&tolval,&pltflg,plotfile,print,
                    prtfile,filename,&msgtim,&minrms) == E_FAIL)
   {
   free(tie_data);
   c_errmsg("Fatal error encountered","autofit-fatal",&fatal);
   }


if (get_tie_points(tpl, &tie_head, &tie_data, &npts) == E_FAIL)
   {
   free(tie_data);
   c_errmsg("Fatal error encountered","autofit-fatal",&fatal);
   }

/* open print file
----------------*/
fptr=fopen(prtfile,"w");
if(!fptr)
  {
  free(tie_data);
  c_errmsg("Error opening print file","autofit-print",&nonfatal);
  c_errmsg("Fatal error encountered","autofit-fatal",&fatal);
  }

if (editpts(tie_data, npts, degree, tolval,&pltflg,plotfile,fptr,
                      &msgtim,&minrms) == E_FAIL)
  {
  /* clean up if error
  ------------------*/
  fclose(fptr);
  num_out = 0;
  if (print[PFILE])
     {
     strcpy(outbuf[0],filename);
     num_out++;
     }
  if (pltflg)
     {
     strcpy(outbuf[1], plotfile);
     num_out++;
     }
  free(tie_data);
  c_clean (&vblk,"AUTOFIT", outbuf, &num_out);
  }

if (put_tie_points(outtpl, &tie_head, tie_data, npts) == E_FAIL)
  {
  /* clean up if error
  ------------------*/
  fclose(fptr);
  num_out = 0;
  strcpy(outbuf[0], outtpl);
  if (print[PFILE])
     {
     strcpy(outbuf[1],filename);
     num_out++;
     }
  if (pltflg)
     {
     strcpy(outbuf[1], plotfile);
     num_out++;
     }
  free(tie_data);
  c_clean (&vblk, "AUTOFIT", outbuf, &num_out);
  }

free(tie_data);

/* completion message
-------------------*/
strcpy(outbuf[0], outtpl);
if (print[PFILE])
   {
   strcpy(outbuf[num_out],filename);
   num_out++;
   }
if (pltflg)
   {
   strcpy(outbuf[num_out], plotfile);
   num_out++;
   }


c_complt("AUTOFIT", outbuf, &num_out);
}
