/****************************************************************************
NAME:				FIT

PURPOSE:  Performs a least-squares fit and calculates residual errors


****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "lasmath.h"
#include "protos.h"

int fit
(
    struct TPLDATA *tie_data,	/* Tie Point Location data records 	*/
    long npts,			/* Number of points in TPL file 	*/
    long npts_used,			/* Current number of points used 	*/
    long degree,			/* Degree of polynomial used 		*/
    double *rmse,			/* Root Mean Squared Error 		*/
    double *minerr,			/* minimum error			*/
    double *maxerr,			/* maximum error			*/
    long plot,			/* Flag--write plot file? 		*/
    char *plotfile			/* plot file name			*/
)
{
struct TPLDATA *tt;		/* Ptr to TPL data structure 		*/
double *v;			/* Work buffer for LSQ fit 		*/
double *b1;			/* Vector containing X coordinates 	*/
double *b2;			/* Vector containing Y coordinates 	*/
double *A;			/* Matrix A (used for LSQ fit) 		*/
double x,y;			/* Calculated x & y locactions 		*/
double delta_x,delta_y;		/* Absolute value of x & y residuals 	*/
double rms_acc; 		/* Accumulator for RMSe calc 		*/
double x_residual;		/* Residual in X 			*/
double y_residual;		/* Residual in Y 			*/
double temp;			/* temporary variable			*/
long i,k;			/* Loop counters 			*/
long nonfatal = 1;		/* non fatal error flag			*/
long size;			/* Size of vectors (based on poly degree) */
FILE *fptr;

/* allocate space depending on polynomial degree
----------------------------------------------*/
if(degree==1)
   size=3; 
else 
   size=6;

A = (double *)malloc(npts_used * size * sizeof(double));
if (A == NULL) 
   {
   c_errmsg("Unable to allocate dynamic memory","autofit-alloc",&nonfatal);
   return(E_FAIL);
   }
b1 = (double *)malloc(npts_used * sizeof(double));
if (b1 == NULL) 
   {
   c_errmsg("Unable to allocate dynamic memory","autofit-alloc",&nonfatal);
   return(E_FAIL);
   }
b2 = (double *)malloc(npts_used * sizeof(double));
if (b2 == NULL) 
   {
   c_errmsg("Unable to allocate dynamic memory","autofit-alloc",&nonfatal);
   return(E_FAIL);
   }
v = (double *)malloc(npts_used * sizeof(double));
if (v == NULL) 
   {
   c_errmsg("Unable to allocate dynamic memory","autofit-alloc",&nonfatal);
   return(E_FAIL);
   }

if (plot) 
   fptr = fopen(plotfile,"w");

/* Loop on tie point pairs.  If the tie point is active, use it in the
   fit calculations; first for x, then for y.
  ------------------------------------------*/
for(tt=tie_data, k=0, i=0; i < npts; i++, tt++)
   {
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
      A[k] =               1.0;
      A[k+npts_used] =     tt->ref_coord[1];
      A[k+(2*npts_used)] = tt->ref_coord[0];
      if(degree==2)
           {
           A[k+(3*npts_used)]=tt->ref_coord[1] * tt->ref_coord[1];
           A[k+(4*npts_used)]=tt->ref_coord[1] * tt->ref_coord[0];
           A[k+(5*npts_used)]=tt->ref_coord[0] * tt->ref_coord[0];
           }
      b1[k] = tt->sea_coord[1];
      b2[k] = tt->sea_coord[0];
      k++;
      }
   }
c_qrdecomp(A, npts_used, size, v, FALSE);
c_qrsolve(A, npts_used, size, v, b1, 0);
c_qrsolve(A, npts_used, size, v, b2, 0);

/* Calculate Residuals
  -------------------*/
rms_acc = 0.0; 
*maxerr = 0.0; 
*minerr = 9999999.0;

/* Loop on tie point pairs.  If the tie point is active, evaluate the
   polynomial and calculate residuals & rms errors
  -----------------------------------------------*/
if (plot) 
   fprintf(fptr,"%d\n",npts_used);

for(tt=tie_data, npts_used=0, i=0; i < npts; i++, tt++)
   {
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
       x = c_eval(&degree,b1,&(tt->ref_coord[1]),&(tt->ref_coord[0]));
       y = c_eval(&degree,b2,&(tt->ref_coord[1]),&(tt->ref_coord[0]));
       x_residual = tt->sea_coord[1] - x;
       y_residual = tt->sea_coord[0] - y;
       delta_x = fabs(x_residual);
       delta_y = fabs(y_residual);
       if (plot) 
          fprintf(fptr,"%s %f %f %f %f 0\n",tt->pt_id,tt->sea_coord[1],
		tt->sea_coord[0], x_residual, y_residual);
       temp = (delta_x * delta_x) + (delta_y * delta_y);
       if (temp > *maxerr) 
          *maxerr = temp;
       if (temp < *minerr) 
          *minerr = temp;
       rms_acc += temp;
       npts_used++;
       }
   }

/* Calculate average residuals
  ---------------------------*/
*maxerr = sqrt(*maxerr);
*minerr = sqrt(*minerr);
*rmse = sqrt(rms_acc / npts_used);
if (plot) 
   fprintf(fptr,"%lf\n",*rmse);
free(A); 
free(b1); 
free(b2); 
free(v);
if (plot) 
   fclose(fptr);

return(E_SUCC);
}
