/*******************************************************************************

NAME: BANDRATIO

PURPOSE: Compute the ratio of one band to the weighted sum of several bands

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        12/20/83   G. Linebaugh, SASC      
     2	       3/20/86   K. Gacke, CSB        
     3            6/87   M. Mazzella        Conversion to NEWLAS
     4         1/26/89   K. Narayanan       Retrocode to 5.0 standards
     5        2-NOV-89   P. Pease	    Change call from inlas to c_inlas
     6         7/31/90   T. Sturman	    Added statics to run on pn9050
     6.1      05/23/94   D. Eichacker       Added opt[] for c_eopens.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   RunS under TAE.

PROJECT:        LAS 

ALGORITHM DESCRIPTION:
   Call GETPAR to get user-specified parameters
   Open the input and output images 
   For each line
      Sum the samples of the denominator image 
      Divide samples of the numerator image by this denominator sum
   Update output image min/max and history 

ALGORITHM REFERENCES     none

*******************************************************************************/


#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

#define curlin curent

struct	{                                    /* offsets into the image  */
	int	ip1[MAXIMG * MAXBND + 1];    /* I/O buffer for input    */
	int	ip2;                         /* and output bands  	*/
	int	ip3;
	}points;

struct FDESC *fdesc[MAXIMG + 1]; /* image file descriptors used 
                                     for image I/O  */
struct GDESC * gdesc;		 /* group descriptor used for image I/O        */

void main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		      */

double line_inc = 1.0;          /* line increment                             */
double maxval;			/* maximum intensity value of output image    */
double minval;			/* minimum intensity value of output image    */
double samp_inc = 1.0;          /* sample increment                           */

float denom;			/* value to be output when denominator is 0   */
float fone = 1.0;		/* float value of one			      */
float fzero = 0.0;		/* float value of zero			      */
float wa[24];			/* weight factor array for denominator images */
float wn;			/* weight factor for numerator image	      */

long access;			/* image I/O access type		      */
long band;			/* loop index counting the bands 	      */
long bands1[MAXIMG][MAXBND + 1];/* band specification for denominator images  */
long bands2[1][MAXBND + 1];     /* band specification for numerator image     */
long bufsiz;			/* size of image I/O buffer		      */
extern long curlin;		/* current line number for display message    */
long fatal = -1;
long ftype = EREAL;	        /* data type for processing                   */
long msg_flag;			/* flag for processing message		      */
long msgtime;			/* elpased time between display messages      */
long nim1;			/* number of input images for denominator     */
long i;				/* loop index				      */
long ione = 1;			/* integer value of one			      */
long mnmx = FALSE;		/* minmax boolean flag			      */
long nib1[MAXIMG+1];		/* number of bands per denominator image      */
long nlout;			/* number of lines in output image	      */
long nonfatal = 1;
long npout;			/* number of pixels per line in output image  */
long one = 1;
long size;			/* number of bytes per pixel		      */
long status;			/* function return status code	              */
long std_out;			/* logical unit # for standard output         */
long totbnd1;			/* total number of bands in denominator       */
long wind1[MAXIMG][4];		/* window specification for denominator images*/
long wind2[1][4];		        /* window specification for numerator image   */
long zero = 0;			/* integer value of zero		      */

static	long	opt[3]={0,0,0}; /* for c_eopens                               */

unsigned char *tbuf;		/* temporary summing buffer		      */
unsigned char *thebuf;		/* image I/O buffer			      */

static  char h1[MAXIMG][CMLEN];	/* denominator image host names               */
char	h2[1][CMLEN];	    /* numerator image host name                  */
char	h3[1][CMLEN];	        /* output image host name                     */
char    minmax[4];	        /* minmax flag buffer 			      */
static  char t1[MAXIMG][CMLEN];	/* denominator image TAE names                */
char	t2[1][CMLEN];	        /* numerator image TAE name                   */
char	t3[1][CMLEN];	        /* output image TAE name                      */
static char tmpbuf[MAXIMG+1][CMLEN];/* temporary host names array             */

c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/* get user-specified parameters and TAE globals
*/
getpar(&vblock, wa, &wn, &denom, &msgtime, t1, h1, 
       t2, h2, t3, h3, wind1, wind2, &nlout, 
       &npout, nib1, minmax, bands1, bands2, &nim1, 
       &totbnd1);

/* open input images for denominator
*/
access = IREAD;
band = 0;
for ( i = 0; i < nim1; ++i)
   {
   status = c_eopens(&fdesc[i], h1[i], bands1[i], &wind1[i][SL], &wind1[i][SS],
                   &nlout, &npout, (long *) &points.ip1[band], &access,
                   &ftype, &zero, opt);
   if (status != E_SUCC)
       c_errmsg("Fatal error encountered", "bandratio-fatal",&fatal);
   band += nib1[i];
   }

/* open input image for numerator
*/
status = c_eopens(&fdesc[i], h2[0], bands2[0], &wind2[0][SL], &wind2[0][SS], 
                &nlout,&npout,&points.ip2, &access, &ftype, &zero, opt);
if (status != E_SUCC)
       c_errmsg("Fatal error encountered", "bandratio-fatal",&fatal);

/* open output image
*/
access = IWRITE;
status = c_eopens (&fdesc[i+1], h3[0], &ione, &ione, &ione, &nlout, &npout, 
                   &points.ip3, &access, &ftype, &zero,opt);
if (status != E_SUCC)
       c_errmsg("Fatal error encountered", "bandratio-fatal",&fatal);

fdesc[i+2] = 0;

/* allocate space for image I/O buffer
*/
if ((thebuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_errmsg("Error allocating dynamic memory", "bandratio-alloc",&fatal);

/* allocate space for temporary summing buffer
*/
size = sizeof(float);
tbuf = (unsigned char *) malloc(npout * size);
if (tbuf == NULL)
   c_errmsg("Error allocating dynamic memory","bandratio-alloc",&fatal);

/* set up file group
*/
if (c_egroup(fdesc, &gdesc, thebuf, &bufsiz) != E_SUCC)
   c_errmsg("Fatal error encountered","bandratio-fatal",&fatal);

/* initialize min and max 
*/
if (strcmp(minmax,"YES") == 0)
   {
   mnmx = TRUE;
   c_inmnmx(&ftype,&ione,&minval,&maxval);
   }

/* set up and start timed process message
*/
msg_flag = GEN2;
c_inittm("BANDRATIO", "Processed", "line", "bands", "", &nlout, &ione, 
         &msgtime, &msg_flag);
curlin = 1;
c_startm();

/* compute bandratio
*/
for (curlin = 0; curlin < nlout; ++curlin)
   {
   if (c_estep(&gdesc) != E_SUCC)
      c_errmsg("Fatal error encountered","bandratio-fatal",&fatal);

   c_pxfill(tbuf, &ftype, &npout, (unsigned char *)  &fzero);

   for (i = 0; i < totbnd1; ++i)
      {
      c_pxadd (&npout, (long *) &thebuf[points.ip1[i] * size],(long *)tbuf,
                (long *)tbuf, &wa[i], &fone, &fzero, &ftype);
      }
   status = c_pxdiv(&npout, (long *) &thebuf[points.ip2 * size], (long *) tbuf,
                  (long *) &thebuf[points.ip3 * size], &wn, &fzero, &ftype,
                  (long *) &denom);

   if (mnmx)
      status = c_pxmnmx( (long *) &thebuf[points.ip3*size],&npout,
                          &minval,&maxval,&ftype);
   }

/* final buffer flush
*/
if (c_estep(&gdesc) != E_SUCC)
   c_errmsg("Fatal error encountered","bandratio-fatal",&fatal);

/* turn off process message
*/
c_stoptm();

/* close images
*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","bandratio-warn",&nonfatal);

/* free the buffers
*/
free(thebuf);
free(tbuf);

/* update output image ddr, min, and max
*/
c_upddr (&vblock, h2, h3[0], &one, wind2, bands2, &one, &line_inc, 
         &samp_inc, &one, &zero);
if (mnmx)
   if (c_svmnmx(h3[0],&ione,&minval,&maxval,"YES") != E_SUCC)
      c_errmsg("Nonfatal error encountered","bandratio-warn",&nonfatal);

/* create associated history file
*/
for (i=0; i < nim1; i++)
    strcpy(tmpbuf[i], h1[i]);
strcpy(tmpbuf[i], h2[0]);
i++;
status = c_hstout(&vblock, h3[0], tmpbuf, &i, " ");
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","bandratio-warn",&nonfatal);

/* inform user of successful completion
*/
c_complt("BANDRATIO", t3, &ione);
}
