#include "bayes.h"

/***********************************************************************

NAME:	classify

FUNCTION:
	Do bayesian maximum likelihood classification

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0      27-May-82     C. Gurney
    2.0         Jun-87     N. Deodhar/SAR    New LAS & coversion to C
    3.0         Oct-88     B. Allam/SAR      Retrocode
    5.0       6-Apr-90	   D. VanderZee      Changed processing algorithm
					     to process an entire line one 
					     pixel at a time.  This allowed
					     for the addition of checking 
					     each pixel against the mask value
					     if present.  Also eliminated the
					     use of IMSL matrix manipulation
					     routines in order to increase
					     speed. 

***********************************************************************/

void classify
(
    float *mean[MAXCLS],          /* i   cluster means */
    double *covinv[MAXCLS],        /* i   inverse covariance matricies for clust */
    float *x[MAXBND],		/* i   image data                             */
    long totbnd,			/* i   number of bands                        */
    long ns,			/* i   number of pixels to be classified      */
    long ncx,			/* i   number of classes                      */
    float cconst[],		/* i   class constants                        */
    float *g[MAXCLS],		/* o   g values                               */
    unsigned char class[],		/* o   class assignments                      */
    float gvals[],		/* o   maximum g values                       */
    long chi,			/* i   output indicator flag                  */
    long mask,			/* i   mask presence indicator               */
    double maskval		/* i   pixel mask value */
)
{
float	gmax,			/* Maximum g value                            */
	scale1,			/* Scale for scaling                          */
	x1,			/* Value to clip g values to                  */
	x2;			/* Value to clip g values to                  */
double   *covinv_ptr, 		/* pointer to inverse covariance matrix */
	temp[MAXBND],		/* temporary work buffer */ 
	temp1[MAXBND],	        /* Temporary work buffer                      */
	*temp1_ptr,		/* pointer to temp1 buffer */
	temp2;	 		/* Temporary work buffer                      */
long    temp_prod;  		/* temporary product for pointer arithmetic */

long    imax = 0,			/* Index of class with the maximum g value    */
	i, j,			/* Loop counter on bands                      */
	k, mk,			/* Loop counter on classes                    */
	kk,			/* loop counter for matrix multiplication */
	px,			/* Loop counter on pixels                     */
	limit, 			/* Loop limit for number of classes */
	doit = TRUE;		/* Flag idicating if pixel is masked or not */ 

x1 = -88.7361;
x2 = 0.0;
scale1 = 255.0 / 88.7361;

for (px = 0; px < ns; px++)
  {
   gmax = -999999.0;
   if (mask)
     {
     doit = FALSE;
     for (i = 0; i < totbnd; i ++)
       {
       if (x[i][px] != maskval)
	 {
	 doit = TRUE;
	 break;
	 } 
       }
     }
 
  if (doit)
   {
   if (mask)
     /* the value of ncx reflects the extra mask class */
     limit = ncx - 1;
   else
     limit = ncx;
   for (k = 0; k < limit; k++)
      {
      if (mask)
	 /* class zero is reserved for masked values */ 
         mk = k + 1;
      else 
         mk = k;

      /* Get difference vectors
      Subtract class mean values, mean values have been multiplied
      by -1.0 in subroutine do_stats
      ------------------------------------------------------------ */
      for (j = 0; j < totbnd; j++)
         {
	 temp[j] = x[j][px] + mean[k][j];
         }

      /* Do bayes classification

      Multiply the transpose of difference vectors and the inverse covariance
      ----------------------------------------------------------------------- */

      covinv_ptr = covinv[k];
      for (j = 0; j < totbnd; j++)
         {
         temp1_ptr = &temp1[j];
	 *temp1_ptr = 0;
	 temp_prod = j * totbnd;
	 for (kk = 0; kk < totbnd; kk++)
	    {
	    *temp1_ptr += (temp[kk] *  *(covinv_ptr + temp_prod + kk));
	    }
	 }
      /* Multiplied with the difference vectors
      -------------------------------------- */
      for (i = 0, temp2 = 0; i < totbnd; i++)
         {
	 temp2 += temp1[i] * temp[i];
	 }

      /* Compute g values
         ---------------- */
      g[mk][px] = (temp2 * -0.5) + cconst[k];
      /* Find the class index with maximum g value
         ----------------------------------------- */
      if (g[mk][px] > gmax)
	 {
	 gmax = g[mk][px];
         imax = mk;
	 }
      /* Clip g values to -88.7361 - 0.0
         ------------------------------- */
      if (g[mk][px] < x1)
         g[mk][px] = x1;
      else if (g[mk][px] > x2)
         g[mk][px] = x2;

      }
   if (mask) 
      {
      /* imax already reflects actual class number */ 
      class[px] = (unsigned char) imax;
      /* set mask class g value for pixel to minimum */ 
      g[0][px] = x1; 
      }
   else
      /* class number has to be incremented because counter started at 0 */
      class[px] = (unsigned char) (imax + 1);

   gvals[px] = gmax;
   }
  else
   {
   /* mask has to be true to get here */
   /* set class to mask class (0) */
   class[px] = 0;
   /* set mask class g value and gvals for pixel to maximum */
   gvals[px] = g[0][px] = x2;
   /* set g values for all other classes to zero */
   for (k = 1; k < ncx; k++)
      g[k][px] = x1;
   } 

  /* scale the g values 
     ------------------ */
  for (k = 0; k < ncx; k++)
     g[k][px] = (g[k][px] * scale1) + 255.5;

  if (!chi)
     {
     /* clip and scale gvals */
     if (gvals[px] < x1)
	gvals[px] = x1;
     else if (gvals[px] > x2)
	gvals[px] = x2;

     gvals[px] = (gvals[px] * scale1) + 255.0;  
     }
  }

return;
}
