#include "bayes.h"
#include "pixman.h"
#include "lasmath.h"

/************************************************************************

NAME:	do_stats

FUNCTION:
	Retrieves and controls processing of statistics from
	stats file and controls receipt and processing of
	prior probabilities

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0      27-May-82     C. Gurney
    2.0         Jun-87     N. Deodhar/SAR    New LAS & coversion to C
    3.0         Oct-88     B. Allam/SAR      Retrocode
    4.0      10-Apr-89     K. Narayanan      Enhanced to accept others'
                                             stats files in the general case
    4.1         Apr-89     D. Hollaren       Added static to compile on PN9050
    5.0	      6-Apr-90     D. VanderZee      unpacked inverse covariance matrix
					     for more effiecient processing
					     in classify 

***********************************************************************/

void do_stats
(
    float *mean[MAXCLS],          /* o   mean values for classes */
    double *covinv[MAXCLS],       /* o   inverse covariance matrices for classes*/
    char sthos[],	        /* i   stats file host name                   */
    long *nclass,		/* o   # classes in stats file                */
    char aprior[6],		/* i   option for prior probabilities         */
    long totbnd,			/* i   # input images                         */
    float cconst[MAXCLS],	        /* o   class constants                        */
    long nuse,			/* i   number of classes                      */
    long clind[MAXCLS]	        /* o   class indices                          */
)
{
long zero = 0;

static double	*cfile,         /* covariance matrix returned from stats file */
	cmean[MAXBND],		/* mean vector returned from stats file       */
	*covar,         	/* covariance matrix for a class              */
	*t_covinv, 	 	/* temporary pointer to packed form of 
				   symetric inverse covariance matrix */	
	*t_cursor,		/* temp cursor used to index into t_covinv */  
	dtemp,			/* Temporary storage for determinant          */
	pi2;			/* 2 * PI                                     */

static float	cdet,		/* Log of the determinant                     */
	det[2],			/* Temporary float value                      */
	determ,			/* Determinant of the class matrix            */
	logpi2,			/* Temporary variable for multiplication      */
	offset,			/* Offset for pxline */
	prior[MAXCLS], 	        /* prior probabilities                        */
	scale1;			/* Scale value                                */

static long	access ,	/* Access to open stats file                  */
	cshape[9],		/* stats data descriptor for covariance       */
	dtype = EREAL,		/* The data type we are working with          */
	i,			/* Loop counter                               */
	j,
	ierr,			/* Return status                              */
	ind,			/* Loop counter on classes                    */
	jj,			/* Loop counter on classes                    */
	k,			/* A class number                             */
        c_size, t_size,
	mshape[9],		/* stats data descriptor for mean             */
	ncx,			/* Number of clases in stats file             */
	nval;			/* Number of values in covariance matrix      */

static char	msg[80];	/* Output message buffer                      */


/* Allocate storage for temporary buffers
  -------------------------------------- */
c_size = (long) (totbnd * (totbnd+1)/2 * sizeof (double));
t_size = (long) (totbnd * (totbnd+1)/2 * sizeof (double));
cfile = (double *) malloc (t_size);
covar = (double *) malloc (c_size);
t_covinv = (double *) malloc (c_size);

if ((cfile == NULL) || (covar == NULL) || (t_covinv == NULL))
   iso_err("!Error allocating dynamic memory","bayes-alloc");

/* Open stats file
   --------------- */
if (strcmp (aprior, "ENTER") == 0) access = IUPDATE;
else access = IREAD;
ierr = c_stopen (&sdesc[0], sthos, &access, &ncx);
if (ierr != E_OPOLD)
   iso_err ("!Error opening input statistics file", "bayes-open");

if (ncx > 255)
   {
   ncx = 255;
   iso_err("?Statistics file has too many classes - first 255 will be used",
	"bayes-class");
   } 

if (nuse == 0)
   {
   *nclass = ncx;

   for (jj = 0; jj < *nclass; jj++)
      clind[jj] = jj + 1;
   }
else
   {
/*
   Prompt for the selection of subset of classes
   --------------------------------------------- */
   if (nuse > ncx)
      {
      iso_err ("?NCLASS is larger than the total number of classes", 
	"bayes-nclass");
      nuse = ncx;
      sprintf (msg, "?Number of classes to be used is now reset to: %d", nuse);
      iso_err (msg, "bayes-nclass");
      }

   clprompt (&sdesc[0], ncx, &nuse, clind);
   *nclass = nuse;
   }
/*
   Get prior probabilities
   ----------------------- */
priori_prob (prior, *nclass, aprior, clind);
/*
   Get mean and covariance from stats file
   Check # values corresponds to #bands
   --------------------------------------- */
nval = (totbnd * (totbnd + 1)) / 2;
pi2 = 3.1415926 * 2.0;
logpi2 = 0.5 * totbnd * log (pi2);

/* Allocate dynamic storage 
  ------------------------ */
dyn_alloc (totbnd, *nclass, mean, covinv);

for (ind = 0; ind < *nclass; ind++)
   {
   k = clind[ind];
   ierr = c_stget (&sdesc[0], "MEAN_VECTOR", &k, &zero, mshape, cmean);
   if (ierr != 0)
      {
      sprintf (msg, "!No mean vector present for class # %d", k);
      iso_err (msg, "bayes-nomean");
      }
   if (mshape[2] != totbnd)
      {
      sprintf (msg, "!Mean vector does not have %d values for class # %d",
               totbnd,k);
      iso_err (msg, "bayes-mean");
      }
   ierr = c_stget (&sdesc[0], "COVARIANCE_MATRIX", &k, &zero, cshape, cfile);
   if (ierr != 0)
      {
      sprintf (msg, "!No covariance matrix present for class # %d", k);
      iso_err (msg, "bayes-nocovar");
      }
   if (cshape[2] != nval)
      {
      sprintf (msg, "!Covariance matrix has too few/too many values for class # %d", k);
      iso_err (msg, "bayes-covar");
      }
/*
   Multiply class means by -1.0 for later use
   Later the addition of vectors will be used instead of substraction
   ------------------------------------------------------------------ */
   for (i = 0; i < totbnd; i++)
      mean[ind][i] = cmean[i];
   scale1 = -1.0;
   offset = 0.0;
   c_pxline(mean[ind],mean[ind],&totbnd,&dtype,&scale1,&offset);
/*
   Compute inverse of covariance matrix
   The covariance and inverse of covariance are stored in symmetric mode
   --------------------------------------------------------------------- */
   for (i = 0; i < nval; i++)
      covar[i] = cfile[i];

   linv2p (covar, &totbnd, t_covinv, det);

/*
   transfer and expand inverse of covariance matrix for later processing 
   --------------------------------------------------------------------- */
   t_cursor = t_covinv; 
   for (i = 0; i < totbnd; i++)
      {
      for (j = 0; j <= i; j++)
	 {
	 *(covinv[ind] + i*totbnd + j) = *t_cursor;  
	 if (i != j) 
	    *(covinv[ind] + j*totbnd + i) = *t_cursor;  
	 t_cursor++;
	 }   
      }
/*
   Compute determinant of class matrix
   ----------------------------------- */
   determ = det[0] * pow (10.0, ((double) det[1]));
   if (determ <= 0.0)
      {
      sprintf (msg, "!Covariance matrix contains all zeroes for class # %d", k);
      iso_err (msg, "bayes-zcovar");
      }
/*
   Compute class constant
   ---------------------- */
   dtemp = determ;
   cdet = log (dtemp);
   dtemp = prior[ind];
   cconst[ind] = log (dtemp) - logpi2 - 0.5 * cdet;
   }
free (covar);
free (cfile);
free (t_covinv);
return;
}
