#include "lasmath.h"
#include "bdist.h"
#include "protos.h"

void getdist
(
    long nc,		/* no. classes			*/
    long numsets,	/* no. feature sets		*/
    double *scov[],	/* covariance matrix		*/
    float *smean[],	/* mean vector			*/
    float b_dist[], 	/* Bhattacharyya distances	*/
    float det[]		/* determinants of covariances  */
)
/*
**********************************************************************

NAME: GETDIST

FUNCTION: Calculate distances between pairs of classes.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	      23-Mar-83  C. Gurney
   2           1-Oct-85  F. Irani
   3           5-Jun-89  K. Narayanan     Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   BD = 0.25*((X1-X2)~(C1+C2)'(X1-X2)) + 0.5*LN((C1+C2)"/(2*SQRT(C1"*C2")))
   where:
      BD= Bhattacharyya distance between classes 1 & 2
      X1= mean vector for class 1
      C1= covariance matrix for class 1
      ~=  transpose
      '=  inverse
      "=  determinant

**********************************************************************
*/
{
double	*covinv; 		/* inverse of covariance	*/
float	determ[2];		/* determinant of covariance	*/
float	mult[MAXBND];
float	sbmean[MAXBND];
float	sum;
double	*sumcov;
float	xdet;
float	xsum;
float	ysum;

long    fatal_err = -1;
long    inf_err = 1;


long	i, j, jk, jm, mi, mj, n,
	npair, nptot, nval, order;

nptot = (nc*(nc-1)/2);
sprintf (buf, "Calculating distances between each of %d class pairs", nptot); 
c_errmsg (buf, "bdist-calc", &inf_err);
/*
allocate storage for sum of covariances and its inverse
-------------------------------------------------------
*/
nval = numsets*(numsets+1)/2;

if ((sumcov = (double *) malloc (nval * sizeof (double))) == NULL ||
    (covinv = (double *) malloc (nval * sizeof (double))) == NULL)
   g_errmsg ("Error allocating dynamic memory", "bdist-alloc",
             &fatal_err);
/*
for each class pair, get inverse of summed covariances,
and determinant of same
-------------------------------------------------------
*/
npair = 0;
sum = 0.;
for (jk = 1; jk < nc; jk++)
   {
   for (jm = 0; jm < jk; jm++)
      {
      if (det[jk] <= 1.0e-6 || det[jm] <= 1.0e-6)
         {
         b_dist[npair++] = -1.0;
         }
      else
         {
         xsum=0.;
         for (n=0; n < nval; n++)
            sumcov[n] = scov[jk][n] + scov[jm][n];
/*
         invert covariance matrix and get its determinant
         ------------------------------------------------
*/
         order = numsets;
         linv2p (sumcov, &order, covinv, determ);
         xdet = determ[0] * pow (10.0, (double) determ[1]);
/*
         get difference in means
         -----------------------
*/
         for (i = 0; i < numsets; i++)
            {
            sbmean[i] = smean[jk][i] - smean[jm][i];
            mult[i] = 0.;
            }
/*
         multiply diff in means by inverted sum of covars
         and result by diff in means again
         ------------------------------------------------
*/
         for (i = 0; i < numsets; i++)
            {
            for (j = 0; j < numsets; j++)
               {
               mi = max (i, j);
               mj = min (i, j);
               mult[i] += sbmean[j] * covinv[mi*(mi+1)/2+mj];
               }
            xsum += mult[i] * sbmean[i];
            }
/*
         get pairwise bhattacharyya distance
         -----------------------------------
*/
         ysum = 0.5 * log((double) xdet/(2*sqrt((double) det[jk]*det[jm])));
         b_dist[npair] = xsum*0.25 + ysum;
         sum += b_dist[npair];
         npair++;
         }
      }
   }
/*
get mean distance over all pairs
--------------------------------
*/
b_dist[npair] = sum/( (float) npair );
/*
free storage
------------
*/
free (sumcov);
free (covinv);
}
