#include "canal.h"
#include "protos.h"

void getmat
(
    float *p[],        /* between-class covariance matrix */
    float *w[],        /* within-class covariance matrix  */
    long nclass,      /* no. of classes                  */
    long nsamp,       /* total no. of samples            */
    long ibands,      /* no. of input image bands        */
    long *ncomp,      /* max no. of output components    */
    float *a[],        /* transformation matrix           */
    long print[]     /* output destination for info     */
)
/*
**********************************************************************

NAME: GETMAT                            

PURPOSE: Computes transformation matrix

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         08/17/88  K. A. Narayanan
  1.0         02/25/83  K. A. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Output the between- and within-class covariance matrices
   Call TRIDIA to convert w matrix to tridiagonal form
   Call EGVCVL to find eigenvalues and eigvectotrs of tridiagonal matrix
   Call MATRANS to make transpose of eigenvectors
   Call MATDIV to divide eigen vector matriix by squateroot of eigenvalues
   Call MATMUL three times to get q matrix
   Call TRIDIA and then EGVCVL to get eigenvalues and and eigenvectors of q
   Call EGVORDER to arrage eigenvalues and eigenvectors in descending order 
   Perform Bartlett's test to determine NCOMP = number of significant 
      eigen values
   Inform user about the number of significant eigen values
   Inform user about the number of eigen values discarded      
   Call MATRANS to take transpose of eigenvectors 
   Call MATMUL to compute the final transformation matrix 
   Display the transformation matrix on the user's terminal

************************************************************************
*/
{
float	chisq;                     /* chi-square value       */
float	lamda[MAXBND];
float	*eig_vec;                  /* eigen vectors          */
float	*eig[MAXBND];              /* eigen values           */
float	off[MAXBND];
float	prod;

long	i, idf, ierr, ind, is, j, k;

char	buf[ERRLEN];
/*
display the between- and within-class covariance matrices
---------------------------------------------------------
*/
c_txtout ("\n Statistical Output From Canal :", print, fd);
c_txtout (" -----------------------------\n", print, fd);
c_txtout ("\n Between-class Covariance Matrix P :\n ", print, fd);      

for (i = 0; i < ibands; i++)
   {
   sprintf( buf, "\n Row %2d  :", i+1);
   c_txtout( buf, print, fd);

   for (k = 0; k < ibands; )
      {
      buf[0] = '\0';
      for (j = 0; j < 4; j++)
         {
         sprintf( &buf[strlen( buf)], "%15.8e  ", p[i][k]);
         if (++k == ibands) break;
         }
      c_txtout( buf, print, fd);
      }
   }  

c_txtout ("\n\n Within-class Covariance Matrix W : \n", print, fd);
for (i = 0; i < ibands; i++)
   {
   sprintf( buf, "\n Row %2d  :", i+1);
   c_txtout( buf, print, fd);

   for (k = 0; k < ibands; )
      {
      buf[0] = '\0';
      for (j = 0; j < 4; j++)
         {
         sprintf( &buf[strlen( buf)], "%15.8e  ", w[i][k]);
         if (++k == ibands) break;
         }
      c_txtout( buf, print, fd);
      }
   }

if ((eig_vec = (float *) malloc (ibands*ibands*sizeof (float))) == NULL)
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error allocating dynamic memory", "canal-alloc", 
            prog_name, out_files, &num_outfiles);
   }

for (i = 0; i < ibands; i++)
   eig[i] = &eig_vec[i*ibands];

/*
transform w to tridiagonal form
-------------------------------
*/
tridia (ibands, ibands, w, lamda, off, eig);   
 

/*
compute eigen values and eigenvector
------------------------------------
*/
egvcvl (ibands, ibands, lamda, off, eig, ierr);
 
/*
compute square root of eigenvalues
----------------------------------
*/
egsqrt (lamda, ibands);

/*
compute transpose of eig eigenvectors and store in w
----------------------------------------------------
*/
matrans (eig, w, ibands, ibands); 

/*
treat inverse square root of lamda vector as diagonal matrix
and post multiply it with eigen vector matrix i.e. eig/sqrt(lamda)
------------------------------------------------------------------
*/
matdiv (eig, lamda, ibands, ibands);

/*
get inverse sqrt of  w by matrix multiplication
-----------------------------------------------
*/
matmul (eig, w, w, ibands, ibands);

/*
get f matrix
------------
*/
matmul (w, p, eig, ibands, ibands);
matmul (eig, w, p, ibands, ibands);

/*
transform p to tridiagonal form
-------------------------------
*/
tridia (ibands, ibands, p, lamda, off, eig);   

/*
compute eigen values and eigenvectors
-------------------------------------
*/
egvcvl (ibands, ibands, lamda, off, eig, ierr);

/*
eigenvectors are in ascending order rearrange them 
descending order
--------------------------------------------------
*/
egvorder (lamda, eig, ibands, ibands);

/*
perform bartlett's test
-----------------------
*/
ind = TRUE;
i = 0;
is = (nclass-1 < ibands) ? nclass-1 : ibands;
while (i  <  is  &&  ind)
   {
   idf = (ibands-i)*(nclass-i-1);
   prod = 1.0;
   for (j = i+1; j <= is; j++)
      {
      prod = prod*(1+lamda[j-1]);
      }  
   chisq = ((nsamp-1)-(ibands+nclass)/2.0)*log ((double) prod);
   if (chisq  <  (float) idf) ind = FALSE;
   i++;
   }  
if (! ind) 
   {
   i = i-1;
   sprintf (buf, "Last %d eigen values are discarded", is-i);
   c_errmsg (buf, "canal-eigenvalue", &inf_err);
   }  

if (i  ==  0) 
   {
   c_errmsg ("Transformation not possible;", "canal-trans1", &inf_err);
   c_errmsg ("Regenerate statistics file defining different set of classes",
      "canal-trans2", &fatal_err);
   }
*ncomp = i;


/*
compute transpose of da eigenvectors and store deig
---------------------------------------------------
*/
matrans (eig, eig, ibands, ibands); 

/*
compute transformation matrix and store in a
--------------------------------------------
*/
for (i = 0; i < ibands; i++)
   {
   for (k = 0; k < ibands; k++)
      {
      a[i][k] = 0.0;
      for (j = 0; j < ibands; j++)
         {
         a[i][k] += eig[i][j] * w[j][k];
         }
      }
   }

sprintf (buf, "\n Maximum possible number of output components: %d", *ncomp);
c_txtout (buf, print, fd);

/*
display the transformation matrix
---------------------------------
*/
c_txtout ("\n Transformation Matrix :\n", print, fd);

for (i = 0; i < *ncomp; i++)
   {
   sprintf( buf, "\n Row %2d  :", i+1);
   c_txtout( buf, print, fd);

   for (k = 0; k < ibands; )
      {
      buf[0] = '\0';
      for (j = 0; j < 4; j++)
         {
         sprintf( &buf[strlen( buf)], "%15.8e  ", a[i][k]);
         if (++k == ibands) break;
         }
      c_txtout( buf, print, fd);
      }
   }

free (eig_vec);
}
