#include "canal.h"
#include "protos.h" 


void getstat
(
    float *a[],       /* transformation matrix              */
    long ibands,     /* no. of input image bands           */
    long obands,	    /* no. of output images desired       */
    long *ncomp,     /* no. of bands actually output       */
    float *p[],       /* between-class covariance matrix    */
    float *w[],       /* within-class covariance matrix     */
    long nclass,     /* no. of classes                     */
    long *nsamp,     /* total no. of samples               */
    long itrns,      /* matrix availability indicator      */
    long print[],    /* destination for info output        */
    FILE * fd_sfin,    /* input statistics file descriptor   */
    FILE * fd_sfout   /* output statistics file descriptor  */
)

/*
**********************************************************************

NAME: GETSTAT

PURPOSE:
    Gets transformation matrix if available. If not
    gets mean vectors and covariance matrices, computes
    within- and between-class covariance matrices, and copies
    contents of input statistics file to output statistics file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         08/17/88  K. A. Narayanan    Conversion to 5.0
  1.0         02/25/83  K. A. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   If transformation matrix is to be obtained from statistics file
      Get transformation matrix
      If not avilable issue error message and abort
      If size of matrix is incorrect issue error message and abort
      Get NCOMP = output dimension for the matrix
      Output the matrix 
      IF NCOMP > OBANDS set NCOMP = OBANDS
   Else
      Do for number of classes
         Get number of samples in the class
         Get mean and covariance statistics
         If size is incorrect issue error message and abort
         Update within- and between-class covariance matrices and
            total number of samples
      Expand matrices to full symmetric matrices
   If transformation matrix is to be computed 
      Call STAT_COPY to copy input statistics file to output file

***********************************************************************
*/
{
double	*ap,                   /* between-class covariance matrix   */
	*aw,                   /* within-class covariance matrix    */
	*c,
	m[MAXBND], 
	pm[MAXBND];

long	i, j, k, k1;
long	buf_size;
long	cov_size;
long	cshape[9];
long	mshape[9];
long	ns;
long	nval;
long	sshape[9]; 
long	stat;
long	zero = 0;

char	buf[ERRLEN];
unsigned char	class_name[CMLEN];

/*
allocate dynamic storage for covariance matrices
------------------------------------------------
*/
cov_size = ibands * (ibands+1) / 2;
buf_size = (long) ( cov_size * sizeof (double) );
if ((ap = (double *) malloc (buf_size)) == NULL ||
    (aw = (double *) malloc (buf_size)) == NULL ||
    (c = (double *) malloc (buf_size)) == NULL)
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error allocating dynamic memory", "canal-alloc", 
            prog_name, out_files, &num_outfiles);
   }

for (i = 0; i < MAXBND; i++)    pm[i] = 0; 
for (i = 0; i < cov_size; i++) ap[i] = aw[i] = 0;
   
if (itrns  ==  0) 
   {
/*
   get transformation matrix
   -------------------------
*/
   stat = c_stget(&fd_sfin, "CANAL_TRANS_MATRIX", &zero, &zero, sshape,
                   (unsigned char *) a[0]); 
   if (stat != E_SUCC)
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error getting transformation matrix", "canal-matrix", 
            prog_name, out_files, &num_outfiles);
   }

   *ncomp = sshape[2];

/*
   display the transformation matrix
   ---------------------------------
*/
   c_txtout (" Transformation Matrix:\n", print, fd);      

   for (i = 0; i < *ncomp; i++)
      {
      sprintf( buf, "\n Row %2d  :", i+1);
      c_txtout( buf, print, fd);

      for (k = 0; k < ibands; )
         {
         buf[0] = '\0';
         for (j = 0; j < 4; j++)
            {
            sprintf( &buf[strlen( buf)], "%15.8e  ", a[i][k]);
            if (++k == ibands) break;
            }
         c_txtout( buf, print, fd);
         }
      }  

/*
   get the number of bands that will be output
   -------------------------------------------
*/
   if (*ncomp  >  obands) *ncomp = obands;
   }
else
   {

/*
   compute within- and between-class variances
   -------------------------------------------
*/
   *nsamp = 0;
   for (i = 1; i <= nclass; i++)
      {
/*
      get number of samples in each class
      -----------------------------------
*/
      stat = c_stget (&fd_sfin, "NPOINTS", &i, &zero, sshape,
                        (unsigned char *) &ns); 
      if (stat !=  E_SUCC)
         {
         c_stget (&fd_sfin, "NAME", &i, &zero, mshape, class_name); 
         sprintf (buf, "Number of samples for class %s not found in stats file",
                  class_name);
         if (fd) fclose (fd);
         c_abort (&block, buf, "canal-nosamp", prog_name, out_files, 
                  &num_outfiles);
         }  
/*
      get mean and covariance - check no. of elements
      -----------------------------------------------
*/
      stat = c_stget (&fd_sfin, "MEAN_VECTOR", &i, &zero, mshape,
                        (unsigned char *) m); 
      if (stat  !=  E_SUCC)
         {
         c_stget (&fd_sfin, "NAME", &i, &zero, mshape, class_name); 
         sprintf (buf, "Mean vector for class %s not found in statistics file",
                  class_name);
         if (fd) fclose (fd);
         c_abort (&block, buf, "canal-nomean", prog_name, out_files, 
                  &num_outfiles);
         }  
      if (mshape[2]  !=  ibands) 
         {
         c_stget (&fd_sfin, "NAME", &i, &zero, mshape, class_name); 
         sprintf (buf, 
             "Mean vector for class %s does not have correct no. of elements",
              class_name);
         if (fd) fclose (fd);
         c_abort (&block, buf, "canal-mean", prog_name, out_files, 
                  &num_outfiles);
         }  
      nval = ibands*(ibands+1)/2;
      stat = c_stget (&fd_sfin, "COVARIANCE_MATRIX", &i, &zero, cshape,
                       (unsigned char *) c); 
      if (stat !=  0)
         {
         c_stget (&fd_sfin, "NAME", &i, &zero, mshape, class_name); 
         sprintf (buf, 
              "Covariance matrix for class %s not found in statistics file",
              class_name);
         if (fd) fclose (fd);
         c_abort (&block, buf, "canal-nocov", prog_name, out_files, 
                  &num_outfiles);
         }  
      if (cshape[2]  !=  nval)  
         {
         c_stget (&fd_sfin, "NAME", &i, &zero, mshape, class_name); 
         sprintf (buf, 
         "Covariance matrix for class %s does not have correct no. of elements",
              class_name);
         if (fd) fclose (fd);
         c_abort (&block, buf, "canal-cov", prog_name, out_files, 
                  &num_outfiles);
         }  

      *nsamp += ns;

      k1 = 0;
      for (j = 0; j < ibands; j++)
         {
         pm[j] = pm[j] + ns * m[j];
         for (k = 0; k <= j; k++, k1++)
            {
            aw[k1] += (ns-1) * c[k1];
            ap[k1] += ns * m[j] * m[k];
            }  
         }  
      }  

/*
   expand covariance vectors to full matrices
   ------------------------------------------
*/
   k = 0;
   for (i = 0; i < ibands; i++)
      {
      for (j = 0;  j <= i; j++, k++)
         {
         w[i][j] = aw[k] / (*nsamp-nclass);
         w[j][i] = w[i][j];
         p[i][j] = (ap[k] - pm[i] * pm[j] / *nsamp) / (nclass-1);
         p[j][i] = p[i][j]; 
         }  
      }  

   }  

if (itrns  !=  0) 
   {
/*
   copy input statistics file to output; close input statistics file;
   use output statistics file from hereon
   ------------------------------------------------------------------
*/
   stat_copy (fd_sfin, fd_sfout, nclass);
   }  

free (ap);
free (aw);
free (c);
}

