#include "canal.h"
#include "protos.h"

void newcorel
(
    long ibands,       /* number of input image bands     */
    double tcov[],       /* covariance matrix               */
    long class_index,  /* class index                     */
    long site_index,   /* site index                      */
    FILE * fd_sf        /* stats file descriptor           */
)
/*
**********************************************************************

NAME: NEWCOREL                       

PURPOSE: Generates correlation matrix from covariance matrix.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         08/17/88  K. A. Narayanan   Conversion to 5.0
  1.0         12/24/84  S. Ross

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Expand the covariance to a 2-dimensional matrix
   Call DETERM to check the determinant of matrix
   If the determinant <= zero issue a warning message
   Compute the correlation based on the covariance
   Output the new correlation matrix to the statistics file

**********************************************************************
*/
{
double	*cov;          /* expanded covariance matrix        */
double	fact; 
double	*tcor;         /* correlation matrix in vector form */


long	ib, ii, jb, jj, kk;
long	cshape[9];
long	ncov;
long	shape[9];
long	stat;
long	zero = 0;

char	buf[ERRLEN];
char	cnam[80];
char	*data_type;
char	snam[80];

ncov = (ibands * (ibands + 1))/2;
cshape[0] = EDOUBLE;
cshape[1] = 1;
cshape[2] = ncov;

/*
allocate dynamic storage for correlation and expanded covariance matrices
-------------------------------------------------------------------------
*/
if ((tcor = (double *) malloc (ncov * sizeof (double))) == NULL ||
    (cov = (double *) malloc (ibands*ibands*sizeof (double))) == NULL)
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error allocating dynamic memory", "canal-alloc", 
            prog_name, out_files, &num_outfiles);
   }
/*
expand covariance matrix
------------------------
*/
for (ib = 0, kk = 0; ib < ibands; ib++)
   for (jb = 0; jb <= ib; jb++, kk++)
      cov[ib*ibands+jb] = cov[jb*ibands+ib] = tcov[kk];
      
/*
check determinant of covariance matrix for this site
----------------------------------------------------
*/
if (determ (cov, ibands)  <=  0.00001) 
   {
   c_errmsg ("Determinant of Covariance matrix <= 0 for", "canal-zerodet1", 
             &inf_err);
   data_type = "NAME";
   c_stget (&fd_sf, data_type, &class_index, &zero, shape, 
                (unsigned char *) cnam); 
   cnam[shape[2]] = '\0';
   sprintf (buf, "Class: %s", cnam);
   if (site_index != 0)
      {
      c_stget (&fd_sf, data_type, &class_index, &site_index, shape,
                (unsigned char *) snam); 
      snam[shape[2]] = '\0';
      sprintf (&buf[strlen (buf)], ", Site: %s", snam);
      }
   c_errmsg (buf, "canal-zerodet2", &inf_err);
   }  

/*
compute correlation from covariance
-----------------------------------
*/
for (ib = 1, kk = 0; ib <= ibands; ib++)
   {
   ii = ib*(ib+1)/2;
   for (jb = 1; jb <= ib; jb++, kk++)
      {
      jj = jb*(jb+1)/2;
      fact = tcov[ii-1] * tcov[jj-1];
      if (fact  <=  0.00000001) 
         {
         c_errmsg ("Diagonal element <= 0 in Covariance matrix for",
                  "canal-diag1", &inf_err);
         data_type = "NAME";
         c_stget (&fd_sf, data_type, &class_index, &zero, shape,
                   (unsigned char *) cnam); 
         cnam[shape[2]] = '\0';
         sprintf (buf, "Class: %s", cnam);
         if (site_index != 0)
            {
            c_stget (&fd_sf, data_type, &class_index, &site_index, shape, 
                      (unsigned char *) snam); 
            snam[shape[2]] = '\0';
            sprintf (&buf[strlen (buf)], ", Site: %s", snam);
            }
         c_errmsg (buf, "canal-diag2", &inf_err);
         c_errmsg ("Correlation matrix not computed", "canal-diag3", &inf_err);
         return;
         }
      else
         tcor[kk] = tcov[kk] / sqrt((double) fact);
      }  
   }  

/*
save correlation matrix into statistics file
--------------------------------------------
*/
stat = c_stput(&fd_sf, "CORRELATION_MATRIX", &class_index, &site_index, 
               cshape, (unsigned char *) tcor); 
if (stat != E_DTNEW)
   {
   if (fd) fclose (fd);
   c_abort (&block, 
        "Error copying correlation matrix from input to output statistics file",
        "canal-copystats", prog_name, out_files, &num_outfiles);
   }

/*
free dynamic storage
--------------------
*/
free (tcor);
free (cov);
}

