/******************************************************************************
FUNCTION:	CHECK_LINE

PURPOSE:	Determines whether or not the window specified intersects
		the quadrants input based on the starting line and number
		of lines.  If the window does not intersect the quadrants
		input, the array containing the quadrants to be processed
		is adjusted accordingly. 

ALGORITHM 
  Determine the quadrandts the window intersects
  If the window starting line is greater than the number of lines in quad
      The window is entirely in the bottom half of the image
      If the shape of the output image is 'l' or 'r'
	  The window intersects one quad only
      else
	  The window intersects the lower half only
  If the window starting line plus number of lines is greater than the
  number of lines in quad
      The window is entirely in the top half of the image
      If the shape of the output image is 'l' or 'r'
	  The window intersects one quad only
      else
	  The window intersects the upper half only

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tm.h"
#include "protos.h"

void check_line
(
    long window[], 		/* user specified window			*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    long *nl,		/* number of lines in quad			*/
    long *ns,		/* numbber of samples in quad			*/
    long quads[],		/* quadrants to process				*/
    long *nquads		/* number of quads to process			*/
)
{
extern long com_idx[4];	/* index into comment array			*/

long nonfatal = 1;	/* Non-fatal error code				*/
char mesg[80];		/* informational message text			*/

/*
  Determine the quadrandts the window intersects
*/
if (window[0] > *nl)
    /*
      The window is entirely in the bottom half of the image
    */
    {
    if ((*imshape == 'l') || (*imshape == 'r')) 

	/*
	  The window intersects only one quad
	*/
 	{
	sprintf (mesg,"Window intersects only 1 quad.  Processing quad %d",
	    quads[0]+2);
	c_errmsg (mesg,"informational",&nonfatal);
	*imshape = 'q';
	quads[0] = quads[0]+2;
	*nquads = 1;

	if ((quads[0] % 2) == 0)
	    {
	    window[1] = *ns - (TMSAMP - window[1]);
	    com_idx[0] = com_idx[1];
	    }

	window[0] = window[0] - *nl;
	}
    else
	/*
	  The window intersects the lower half only
	*/
	{
	sprintf (mesg,"%s%s","Window intersects only lower half.  ",
		 "Processing quadrants 3 and 4");
 	c_errmsg(mesg,"tipsp-onehalf",&nonfatal);
	*imshape = 'b';
	quads[0] = 3;
	quads[1] = 4;
	*nquads = 2;
	window[0] = window[0] - *nl;
	com_idx[0] = com_idx[2];
	com_idx[1] = com_idx[3];
	}
    }
else
    {
    if (window[0]+window[2]-1 <= *nl)
	/*
	  The window is entirely in the upper half of the image
	*/
	{
	if ((*imshape == 'l') || (*imshape == 'r'))
	    {
	    /*
	      The window intersects only one quad
	    */
	    sprintf (mesg,"Window intersects only 1 quad.  Processing quad %d",
	        quads[0]);
	    c_errmsg(mesg,"informational",&nonfatal);
	    *imshape = 'q';
	    *nquads = 1;
	    if ((quads[0] % 2) == 0)
	        window[1] = *ns - (TMSAMP - window[1]);
	    }
	else

	    /*
	      The window intersects the upper half only
	    */
	    {
	    sprintf (mesg,"%s%s","Window intersects only upper half.  ",
		     "Processing quadrants 1 and 2");
 	    c_errmsg(mesg,"tipsp-onehalf",&nonfatal);
	    *imshape = 'u';
	    quads[0] = 1;
	    quads[1] = 2;
	    *nquads = 2;
	    }
    	}
    }

return;
}
