/******************************************************************************
FUNCTION:	CHECK_WIND

PURPOSE:	Used to insure compatibility of user specified window.  Reset
		window to indicate the intersection of the user specified
		window with the image, if necessary (this is a version
		similiar to CHKWIND except that the input image is not
		required to exist).

ALGORITHM 
  Set the window flag to false
  Validate the syntax of the window specified
  Make sure the window specified intersects the image
  Determine whether the whole image is being processed
  Determine whether the user specified window can be processed or whether
  part of the image falls outside the image so that a subset of the window
  must be calculated.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "protos.h"

void check_wind
(
    long *wind,		/* user specified window  (SL,SS,NL,NS)		*/
    long *imsiz,		/* image size  (NL,NS) 				*/
    long *flag		/* flag indicating if window being applied	*/
)
{
long i;			/* index variable				*/
long tl, tp;
long xl, xp;
long fatal = -1;	/* Fatal error code				*/
long nonfatal = 1;	/* Non-fatal error code				*/
long zero_count = 0;
long entire = FALSE;

char message[80];	/* Message string				*/

/*
  Set the window flag to false
*/
*flag = FALSE;	

/*
  Validate the syntax of the window specified
*/
for (i = 0; i < 4; i++)
   {
   if (wind[i] < 0)
      {
      sprintf(message,"(%d,%d,%d,%d) Invalid window syntax.",wind[0],
              wind[1],wind[2],wind[3]);
      c_errmsg(message,"tipsp-badwin",&fatal);
      }

   if (wind[i] == 0)
      zero_count++;
   }

if ((zero_count > 0) && (zero_count < 4))
   {
   sprintf(message,"(%d,%d,%d,%d) Invalid window syntax.",wind[0],
           wind[1],wind[2],wind[3]);
   c_errmsg(message,"tipsp-badwin",&fatal);
   }

/*
  Make sure the window specified intersects the image
*/
if ((wind[0] > imsiz[0]) || (wind[1] > imsiz[1]))
   {
   sprintf(message,"(%d,%d,%d,%d) does not intersect (%d,%d).",
           wind[0],wind[1],wind[2],wind[3],imsiz[0],imsiz[1]);
   c_errmsg(message,"tipsp-outside",&fatal);
   }

/*
  Determine whether the whole image is being processed
*/
if ((wind[2] >= imsiz[0]) && (wind[3] >= imsiz[1]))
   entire = TRUE;

if ((zero_count == 4) || (entire))
   {
   wind[0] = wind[1] = 1;
   wind[2] = imsiz[0];
   wind[3] = imsiz[1];
   sprintf(message,"(%d,%d,%d,%d) entire image being processed.",
           wind[0],wind[1],wind[2],wind[3]);
   c_errmsg(message,"informational",&nonfatal);
   return;
   }

/*
  Determine whether the user specified window can be processed or whether
  part of the image falls outside the image so that a subset of the window
  must be calculated.
*/
*flag = TRUE;
tp = wind[1] + wind[3] - 1;
tl = wind[0] + wind[2] - 1;

if (tp < imsiz[1])
   xp = tp - wind[1] + 1;
else
   xp = imsiz[1] - wind[1] + 1;

if (tl < imsiz[0])
   xl = tl - wind[0] + 1;
else
   xl = imsiz[0] - wind[0] + 1;

if ((xp < wind[3]) || (xl < wind[2]))
   {
   wind[3] = xp;
   wind[2] = xl;
   sprintf(message,"Window reset to subset (%d,%d,%d,%d) intersecting file.",
           wind[0],wind[1],wind[2],wind[3]);
   c_errmsg(message,"tipsp-subset",&nonfatal);
   }
else
   {
   sprintf(message,"(%d,%d,%d,%d) User specified window being processed.",
           wind[0],wind[1],wind[2],wind[3]);
   c_errmsg(message,"informational",&nonfatal);
   }

return;
}
