/******************************************************************************
FUNCTION:	GET_ANNOT

PURPOSE:	Gets the annotation information found in the supplemental
		logical volume at the end of the tape.

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development
  2.0		12/90		BALTZER		Check return status's

ALGORITHM 
  Display a message about skipping to end of volume for the annoation records
  Skip to the EOV mark
  Backup to the desired file containing the annotation
  Skip over the file descriptor record
  Process the image quality records
    if the annotation information is to be saved
	write the image quality records to a file
  Read in the annotation record
  If this is the first time that "get_annot" has been called, then save the
  information needed for the printout
      Get the WRS path row
      Get the acquisition date
      Get the image scene center--lat long
      Get the resampling technique
      Get the data source
      Get the scene ID
  If the annotation information is to be saved
      Write the annotation record to disk

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

int get_annot
(
    long *unit,		/* tape unit number			      */
    long *flag,		/* flag indicating whether to save annotation */
    long bands[],		/* band specification			      */
    char *date,		/* acquisition date			      */
    char *lat_long,		/* lat long at scene center		      */
    char *path_row,		/* WRS path row				      */
    char *scene_id,		/* scene id				      */
    char *source,		/* data source				      */
    char *resamp,		/* resampling technique			      */
    char *quality		/* overall geometric quality code	      */
)
{
extern long annot_fd;	/* file descriptor			      */
extern short andsk;	/* annotation file flag			      */

long i, j;		/* loop index				      */
long mode;		/* tape I/O error message mode		      */
long nbytes;		/* number of bytes to be read from tape	      */
long n_written;	 	/* number of bytes to be written	      */
long skip;		/* number of files/records to skip	      */
long status;		/* function return status code		      */
long nonfatal = 1;	/* Non-fatal error code			      */

unsigned char inbuf[3600];	/* input buffer			      */

/*
  Display a message about skipping to end of volume for the annoation records
*/
printf ("\n");
c_errmsg("Skipping to end of tape to get annotation file","get_annot-skip",
	 &nonfatal);

/*
  Skip to the EOV mark, documented return status's from tpfsf are:
    TAPE_SUCC   success
    TPFSF_SKIP  device error   <- check for this one, except its not
    TAPE_EOT    end of tape       defined anywhere, so don't check status
    TAPE_EOV    end of volume
*/
skip = 100;
mode=NO_HANDLE;
status = c_tpfsf(unit,&skip,&mode);

/*
  Backup to the desired file containing the annotation
*/
skip = 2;
status = c_tpbsf(unit,&skip,&mode);
if (status != TAPE_SUCC)
    return(TIPS_READ);

/*
  Skip over the file descriptor record
*/
nbytes = 540;
mode = HANDLE;
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);
  
/*
  Process the image quality records
*/
j = 0;
for (i = 1; i <= 7; i++)
    {
    nbytes = 3600;
    status = c_tpread(unit,inbuf,&nbytes,&mode);
    if (status != nbytes)
        return(TIPS_READ);

    quality[0] = inbuf[1036];
    quality[1] = '\0';

    if (andsk)
        {
        if (bands[j] == i)
            {
            n_written = write(annot_fd,inbuf,nbytes);
            if (n_written != nbytes)
	        {
	        c_errmsg("Error writing to associated annotation file",
			 "get_annot-annot",&nonfatal);
	        c_errmsg("Nonfatal error encountered--Image data saved",
			 "get_annot-warn",&nonfatal);
        	return(TIPS_WRITE);
	        }
            j++;
            }
        }
    }

/*
  Read in the annotation record
*/
nbytes = 540;
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);

/*
  If this is the first time that "get_annot" has been called, then save the
  information needed for the printout
*/
if (*flag == 0)
    {
    /*
      Get the WRS path row
    */
    strncpy(path_row,(char *)&inbuf[12],8);
    path_row[8] = '\0';

    /*
      Get the acquisition date
    */
    strncpy(date,(char *)&inbuf[24],8);
    date[8] = '\0';
   
    /*
      Get the image scene center--lat long
    */
    strncpy(lat_long,(char *)&inbuf[49],17);
    lat_long[17] = '\0';

    /*
      Get the resampling technique
    */
    resamp[0] = inbuf[94];
    resamp[1] = '\0';

    /*
      Get the data source
    */
    strncpy(source,(char *)&inbuf[102],13);
    source[13] = '\0';
   
    /*
      Get the scene ID
    */
    strncpy(scene_id,(char *)&inbuf[115],25);
    scene_id[25] = '\0';

    }

/*
  Write the annotation record to disk
*/
if (andsk)
    {
    nbytes = 540;
    n_written = write(annot_fd,inbuf,nbytes);
    if (n_written != nbytes)
        {
        c_errmsg("Error writing to associated annotation file",
		 "get_annot-annot",&nonfatal);
       	c_errmsg("Nonfatal error encountered--Image data saved",
		 "get_annot-warn",&nonfatal);
        return(TIPS_WRITE);
      	}
    }

return(E_SUCC);
}
