/******************************************************************************
FUNCTION:	GET_CALIB

PURPOSE:	Gets the header calibration information

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development
  2.0		12/90		BALTZER		Check return status's

ALGORITHM 
  If the header flag is set process calibration record or records.
	For images in BSQ format,
	    there is one calibration record in each leader file of each band.
	    Read the header from tape
	    Write the header information to the associated file.

	For images in BIL format,
	    the calibration records for all 7 bands are in the same file.
	    Read the header for each band from tape
	    If the band was selected
		write the header to the associated file.
  else
	Skip over the header calibration record

  Skip over the header EOF mark

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

int get_calib
(
    long *unit,		/* tape unit number				*/
    long bands[],  		/* Band specification		      	      	*/
    long *format		/* image file format (BSQ or BIL)		*/
)
{
extern long head_fd;	/* file descriptor for header file		*/
extern short hdsk;	/* header file flag				*/

long j, k;		/* loop index				      */
long nbytes = 4320;	/* number of bytes to be read from tape		*/
long n_written;		/* number of bytes written			*/
long skip;		/* number of files to skip on tape		*/
long status;		/* function return status code			*/
long mode = HANDLE;	/* tape I/O outputs error mesages		*/
long nonfatal = 1;	/* Non-fatal error code				*/

unsigned char inbuf[4320]; /* input buffer			        */

/*
  Process calibration record or records.  For images in BSQ format,
  there is one calibration record in each leader file of each band.
  For images in BIL format, the calibration records for all 7 bands
  are in the same file.
*/
if (hdsk)
    {
    if (*format == BSQ)
	{
	/*
	  Read the header from tape
	*/
	status = c_tpread(unit,inbuf,&nbytes,&mode);
	if (status != nbytes)
	    return(TIPS_READ);

	/*
	  Write the header information to the output file
	*/
	n_written = write(head_fd,inbuf,nbytes);
	if (n_written != nbytes)
	    {
	    c_errmsg("Error writing to associated header file",
		     "get_calib-head",&nonfatal);
	    c_errmsg("Nonfatal error encountered--Image data saved",
		     "get_calib-warn",&nonfatal);
	    return(TIPS_WRITE);
	    }
	}
    else
	{
	for(j = 0, k = 0; j < 7; j++)
	    {
	    /*
	      Read the header for each band from tape
	    */
	    status = c_tpread(unit,inbuf,&nbytes,&mode);
	    if (status != nbytes)
		return(TIPS_READ);

	    /*
	       If the band was selected write the header to the associated
	       file.
	    */
	    if (bands[k] == j + 1)
		{
		n_written = write(head_fd,inbuf,nbytes);
		if (n_written != nbytes)
		    {
		    c_errmsg("Error writing to associated header file",
			     "get_calib-head",&nonfatal);
		    c_errmsg("Nonfatal error encountered--Image data saved",
			     "get_calib-warn",&nonfatal);
	    	    return(TIPS_WRITE);
		    }
		k++;
		}
	    }
	}
    }
else
    {
    /*
      Skip over the header calibration record
    */
    if (*format == BSQ)
	skip = 1;
    else
	skip = 7;

    status = c_tpfsr(unit,&skip,&mode);
    if (status != TAPE_SUCC)
	return(TIPS_READ);
    }

/*
  Skip over the header EOF mark
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != TAPE_EOF)
    return(TIPS_READ);

return(E_SUCC);
}
