/******************************************************************************
FUNCTION:	GET_HEAD

PURPOSE:	Gets the header information needed to pack the DDR, output the
		associated header file, and for printing.

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development
  2.0		12/90		BALTZER		COMPUTE the full scene center
						projection coordinates
  3.0		12/90		BALTZER		Check return status's
  4.0      	08/91		Ailts		Fixed the calling seauence to
						c_proj.
	    
ALGORITHM 
  Skip header file descriptor record
  Process scene header record
  Only need to get information from the header the first time
      Retrieve the full scene center lat/long
      Set the full scene center line/sample
      Get the WRS path number
      Get the orbital direction
      Get the map projection
      If header file is to be saved
	Write the map projection record to a file
  Process the map projection record
  Only need to retrieve the projection (UTM or SOM) related information the
  first time through.
	If the map projection is UTM
	    Retrieve the UTM zone
	    COMPUTE the full scene center projection coordinates
	    Retrieve the display rotation angle
	Else the map projection is SOM
	    Retrieve the full scene center projection coordinates
	    Retrieve the display rotation angle
	Retrieve the interpixel and nominal interline distances
	Retrieve the line and sample numbers at WRS center
	If header file is to be saved
	    Write the map projection record to a file
  Process the calibration record

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"     
#include "tapeio.h"
#include "proj.h"
#include "tm.h"
#include "geompak.h"
#include "protos.h"

int get_head
(
    long *unit,		/* tape unit number				*/
    long *flag,		/* flag indicating whether to save header info	*/
    long *format,		/* image file format (BSQ or BIL)		*/
    long bands[],  		/* Band specification		      	      	*/
    double fcenll[],	/* Full scene center latitude/longitude degrees	*/
    double fcenls[],	/* Full scene center line/sample		*/
    double fcproj[],	/* Full scene center in projection coordinates	*/
    double *angle,		/* Angle of rotation converted into radians	*/
    long *path,		/* WRS path number				*/
    long *utmzone,		/* UTM zone number				*/
    char *direction,	/* Orbital direction				*/
    char *maprj,		/* Map projection identifier			*/
    long *line_center,	/* Line number at scene center			*/
    long *samp_center	/* sample number at scene center		*/
)

{
extern long head_fd;	/* file descriptor for header file		*/
extern short hdsk;	/* header file flag				*/

long fatal = -1;	/* Fatal error code				*/
long i;		/* loop index					*/
long indatum = 0;	/* code for datum	       	       		*/
long inproj;		/* input projection system	       		*/
long inunit = 4;	/* code for input units           		*/
long inzone = 62;	/* code for zone	       	       		*/
long mode = HANDLE;	/* tape I/O outputs error mesages		*/
long nbytes;		/* number of bytes to be read from tape		*/
long nonfatal = 1;	/* Non-fatal error code				*/
long n_written;		/* number of bytes written			*/
long outdatum = 0;	/* code for output datum  	       		*/
long outproj;		/* output projection system       		*/
long outunit = 2;	/* code for output units  	       		*/
long prtprm[2];		/* report to term and/or file     		*/
long skip;		/* number of files to skip on tape		*/
long status;		/* function return status code			*/
long tempzone = 62;

double inparm[15];	/* input projection parameters    		*/
double outparm[15];     /* output projection parameters   		*/
double pdist_x;		/* Interline distance  (km)			*/
double pdist_y;		/* Interline distance  (km)			*/
double pi = 3.141592653589793238;/* Constant value of PI		*/

char inbuf[4320]; /* input buffer			        */
char temp[17];		/* Temp buffer for buffer conversions		*/

/*
  Skip header file descriptor record
*/
skip = 1;
status = c_tpfsr(unit,&skip,&mode);
if (status != TAPE_SUCC)
    return(TIPS_READ);

/*
  Process scene header record
*/
nbytes = 4320;
status = c_tpread(unit,(unsigned char *) inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);

/*
  Only need to get information from the header the first time
*/
if (*flag == 0)
    {
    /*
      Retrieve the full scene center lat/long
    */
    strncpy(temp,&inbuf[52],16);
    temp[16] = '\0';
    sscanf(temp,"%lf",&fcenll[0]);

    strncpy(temp,&inbuf[68],16);
    temp[16] = '\0';
    sscanf(temp,"%lf",&fcenll[1]);

    /*
      Set the full scene center line/sample
    */
    fcenls[0] = 2983.0;
    fcenls[1] = 3484.0;

    /*
      Get the WRS path number
    */
    strncpy(temp,&inbuf[165],3);
    temp[3] = '\0';
    sscanf(temp,"%d",path);

    /*
      Get the orbital direction
    */
    strncpy(direction,&inbuf[356],10);
    direction[10] = '\0';

    /*
      Get the map projection
    */
    strncpy(maprj,&inbuf[1556],3);
    maprj[3] = '\0';

    /*
      Write the map projection record to a file if requested
    */
    if (hdsk)
	{
	n_written = write(head_fd,inbuf,nbytes);
	if (n_written != nbytes)
	    {
	    c_errmsg("Error writing to associated header file",
		     "get_head-head",&nonfatal);
	    c_errmsg("Nonfatal error encountered--Image data saved",
		     "get_head-warn",&nonfatal);
    	    return(TIPS_WRITE);
	    }
	}
    }

/*
  Process the map projection record
*/
status = c_tpread(unit,(unsigned char *) inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);

/*
  Only need to retrieve the projection (UTM or SOM) related information the
  first time through.
*/
if (*flag == 0)
    {
    if (strcmp(maprj,"UTM") == 0)
	{
	/*
	  Retrieve the UTM zone
	*/
	strncpy(temp,&inbuf[98],10);
	temp[10] = '\0';
	sscanf(temp,"%d",utmzone);
	
	/*
	  Compute the full scene center projection coordinates
	*/
	c_pronum("GEO",&inproj);
	c_pronum(maprj,&outproj);

	for (i=0;i<15;i++)
	   inparm[i] = 0.0;

	outproj = 9;

	if ( utmprm(utmzone,&fcenll[0],outparm) != E_SUCC )
	   return(E_FAIL);

	prtprm[0] = 0;
	prtprm[1] = 0;
	if ( c_transinit(&inproj,&inunit,&inzone,&indatum,inparm,&outproj,
                    &outunit,&tempzone,&outdatum,outparm,prtprm,"") != E_SUCC )
	   return(E_FAIL);

	if ( c_trans(&inproj,&inunit,&outproj,&outunit, 
                     &fcenll[1],&fcenll[0],&fcproj[1],&fcproj[0]) != E_SUCC )
	   return(E_FAIL);

	/*
	  Retrieve the display rotation angle
	*/
	strncpy(temp,&inbuf[204],16);
	temp[16] = '\0';
	sscanf(temp,"%lf",angle);
	*angle = *angle * (pi / 180.0);
	}
    else if ((strcmp(maprj,"SOM") == 0)) /* || (strcmp(maprj,"PS") == 0))*/
	{
	/*
	  Retrieve the full scene center projection coordinates
	*/
	strncpy(temp,&inbuf[252],16);
	temp[16] = '\0';
	sscanf(temp,"%lf",&fcproj[0]);
	fcproj[0] = fcproj[0] * 1000.0;

	strncpy(temp,&inbuf[268],16);
	temp[16] = '\0';
	sscanf(temp,"%lf",&fcproj[1]);
	fcproj[1] = fcproj[1] * 1000.0;

	/*
	  Retrieve the display rotation angle
	*/
	strncpy(temp,&inbuf[316],16);
	temp[16] = '\0';
	sscanf(temp,"%lf",angle);
	*angle = *angle * (pi / 180.0);
	}
    else
	{
	c_errmsg("Image is not in one of the supported projections",
		 "get_head-badprj",&nonfatal);
	c_errmsg ("Fatal error encountered", "get_head-fatal", &fatal);
	}

    /*
      Retrieve the interpixel and nominal interline distances
    */
    strncpy(temp,&inbuf[364],16);
    temp[16] = '\0';
    sscanf(temp,"%lf",&pdist_x);
    pdist_x = pdist_x * 1000.0;

    strncpy(temp,&inbuf[380],16);
    temp[16] = '\0';
    sscanf(temp,"%lf",&pdist_y);
    pdist_y = pdist_y * 1000.0;

    /*
      Retrieve the line and sample numbers at WRS center
    */
    strncpy(temp,&inbuf[412],16);
    temp[16] = '\0';
    sscanf(temp,"%d",line_center);

    strncpy(temp,&inbuf[428],16);
    temp[16] = '\0';
    sscanf(temp,"%d",samp_center);

    /*
      Write the map projection record to a file if requested
    */
    if (hdsk)
	{
	n_written = write(head_fd,inbuf,nbytes);
	if (n_written != nbytes)
	    {
	    c_errmsg("Error writing to associated header file",
		     "get_head-head",&nonfatal);
	    c_errmsg("Nonfatal error encountered--Image data saved",
		     "get_head-warn",&nonfatal);
    	    return(TIPS_WRITE);
	    }
	}
    }

/*
  Process the calibration record
*/
status = get_calib(unit,bands,format);
if (status != E_SUCC)
    return(status);

return(E_SUCC);
}
