/******************************************************************************
FUNCTION:	PROC_ASSOC

PURPOSE:	Process the associated files only

ALGORITHM 
  Open the necessary associated files
  Processing loop -- read in the specified quads
      Mount the the tape the the quadrant to be processed
      Check that the appropriate volume is mounted
      Process the header information -- if the header file is not to be
      written to disk, the header file only needs to be read once (for
      the first quad processed) to get information needed for the DDR.
      Read the file descriptor record to get the size of the input quad
      Process the BSQ or BIL formatted image
	  Process each band
	      Skip over the image data
	      Process the trailer record
		  Skip to the calibration record
		  Process the calibration record
	  Skip over the image data
	  Process the trailer record
      Process the annotation information
      Dismount the tape
  Close the associated files
  Write the parameter information to the output print file and TAE parameter
  block

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

void proc_assoc
(
    long bands[],		/* band specification				*/
    long *nbands,		/* number of bands				*/
    long quads[],		/* quad specification				*/
    long *nquads,		/* number of quads				*/
    char *annot_host,	/* annotation host name				*/
    char *head_host,	/* header host name				*/
    char *trail_host,	/* trailer host name				*/
    long window[]		/* window specification				*/
)

{
extern short print_flag;	/* Flag for print file open		*/
extern short andsk;		/* annotation file flag		        */
extern long  term;		/* Flag for terminal output		*/

long i, j;		/* loop index					*/
long unit;		/* tape unit number				*/
long format;		/* Image file format (BSQ or BIL)		*/
long cur_quad;		/* current quad number				*/
long ex_quad;		/* quadrant expected to be mounted		*/
long skip;		/* number of files to skip on tape		*/
long path;		/* WRS path number				*/
long utmzone;		/* UTM zone number				*/
long line_center;	/* Line number at scene center			*/
long samp_center;	/* sample number at scene center		*/
long status;		/* function return status code			*/

long nl[MAX_QUADS];     /* number of lines				*/
long ns[MAX_QUADS];     /* number of samples per line			*/
long zero = 0;		/* constant zero				*/
long nonfatal = 1;	/* Non-Fatal error code				*/
long mode = HANDLE;	/* tape I/O outputs error messages		*/

double angle;		/* Angle of rotation converted into radians	*/
double fcenll[2];	/* Full scene center latitude/longitude degrees	*/
double fcenls[2];	/* Full scene center line/sample		*/
double fcproj[2];	/* Full scene center in projection coordinates	*/

char date[9];		/* acquisition date				*/
char lat_long[18];	/* lat long at scene center			*/
char path_row[9];	/* WRS path row					*/
char quality[11];	/* overall geometric quality code		*/
char resamp[18];	/* resampling technique				*/
char sceneid[11];	/* scene id read from text record		*/
char scene_id[26];	/* scene id					*/
char source[14];	/* data source					*/
char direction[11];	/* Orbital direction				*/
char maprj[4];		/* Map projection identifier			*/
char msg[ERRLEN];	/* Message string				*/

/*
  Open the necessary associated files
*/
open_assoc(annot_host,head_host,trail_host);

/*
  Processing loop -- read in the specified quads
*/
for (i = 0; i < *nquads; i++)
    {
    /*
      Mount the the tape the the quadrant to be processed
    */
    if (i == 0)
	status = mount_first(&unit,&ex_quad,nquads,quads);
    else
	{
	status = mount_tape(&unit,&i,quads[i]);
	ex_quad = quads[i];
	}

    if (status != E_SUCC)
	clean_up(&unit,status);

    /*
      Check that the appropriate volume is mounted
    */
    check_vol(&unit,&cur_quad,&ex_quad,sceneid,&format);

    /*
      Process the header information -- if the header file is not to be
      written to disk, the header file only needs to be read once (for
      the first quad processed) to get information needed for the DDR.
    */
    status = get_head(&unit,&i,&format,bands,fcenll,fcenls,fcproj,&angle,&path,
		      &utmzone,direction,maprj,&line_center,&samp_center);
    if (status != E_SUCC)
	clean_up(&unit,status);

    /*
      Read the file descriptor record to get the size of the input quad
    */
    status = get_imgfd(&unit,&nl[i],&ns[i]);
    if (status != E_SUCC)
	clean_up(&unit,status);

    /*
      Process the BSQ or BIL formatted image
    */
    if (format == BSQ)
	{
	/*
	  Process each band
	*/
	for (j = 0; j < *nbands; j++)
	    {
	    sprintf(msg,"Skipping over image band %d",j+1);
	    c_errmsg(msg,"tipsp-skip",&nonfatal);

	    /*
	      Skip over the image data
	    */
	    skip = 1;
	    status = c_tpfsf(&unit,&skip,&mode);
	    if (status != TAPE_SUCC)
		clean_up(&unit,status);
	
	    /*
	      Process the trailer record
	    */
	    status = get_trail(&unit,&j);
	    if (status != E_SUCC)
		clean_up(&unit,status);
	
	    if (j < *nbands - 1)
		{
		/*
		  Skip to the calibration record
		*/
		skip = 3;
		status = c_tpfsr(&unit,&skip,&mode);
	
		/*
		  Process the calibration record
		*/
		status = get_calib(&unit,bands,&format);
		if (status != E_SUCC)
		    clean_up(&unit,status);
		}
	    }
	}
    else
	{
	c_errmsg("Skipping over the image data","tipsp-skip",&nonfatal);

	/*
	  Skip over the image data
	*/
	skip = 1;
	status = c_tpfsf(&unit,&skip,&mode);
	if (status != TAPE_SUCC)
	    clean_up(&unit,status);
	
	/*
	  Process the trailer record
	*/
	status = get_trail(&unit,&zero);
	if (status != E_SUCC)
	    clean_up(&unit,status);
	}

    /*
      Process the annotation information
    */
    if ((andsk) || (((term) || (print_flag)) && (i == 0)))
	get_annot(&unit,&i,bands,date,lat_long,path_row,scene_id,source,resamp,
		  quality);

    /*
      Dismount the tape
    */
    dismount_tape(&unit);
    }

/*
  Close the associated files
*/
close_assoc();

/*
  Write the parameter information to the output print file and TAE parameter
  block
*/
write_parm(date,lat_long,path_row,scene_id,source,resamp,quality,&line_center,
	   &samp_center,window,nquads,quads,sceneid);

return;
}
