/******************************************************************************
FUNCTION:	PROC_WINDOW

PURPOSE:	Process the image if a window was specified and process the
		image when the mosaic flag is set to YES.

ALGORITHM 
  Open the necessary associated files
  Verify the window entered is valid
  Processing loop -- read in the specified quads
      If all the specified samples were processed in the left quad, there
      is no need to process the right quad.
      Mount the the tape the the quadrant to be processed
      Set up current quad number for the display message
      Check that the appropriate volume is mounted
      Position tape to first specified band
      Process the header information -- if the header file is not to be
      written to disk, the header file only needs to be read once (for
      the first quad processed) to get information needed for the DDR.
      Read the file descriptor record to get the size of the input quad
	  Determine whether or not the window specified intersects the
	  quadrants input (based on starting line and number of lines)
	      The window specified did not intersect the first quad specified,
	      so prompt the operator to mount the tape of the first quadrant
	      the window does intersect
	      Delete any current associated files so ones for the
	      appropriate quad can be created.
	      Set up current quad number for the display message

	  Determine whether or not the window specified intersects the
	  quadrants input (based on starting sample and number of samples)
	      Read the first line of the image to be processed to get the
	      fill counts.
	      If the first quadrant specified is not available -- calculate
	      what the right fill for that quadrant would have been based on
	      the left fill of the quadrant mounted (the total of the right
	      fill of quadrant 1 and the left fill of quadrant 2 is 1473.  The
	      same holds true for quadrants 3 and 4).
	      Determine whether or not the window specified intersects the
	      quadrants input based on the starting sample and number of
	      samples.
	      The window specified intersects the first quadrant specified, so
	      back up one record (because of the record that was read above for
	      determining fill counts)
	  The window specified did not intersect the first quad specified,
	  so prompt the operator to mount the tape of the first quadrant
	  the window does intersect
	      Delete any current associated files so ones for the appropriate
	      quad can be created.
	      Set up current quad number for the display message
      Process the BSQ or BIL formatted image
      Process the annotation information
      Dismount the tape
  Close the associated files
  Update the DDR information
  Create the associated history file
  Create the associated min max file
  Write the parameter information to the output print file and TAE parameter
  block

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "protos.h"

void proc_window
(
    char hosout[][CMLEN],		/* output host name				*/
    long bands[],		/* band specification				*/
    long *nbands,		/* number of bands				*/
    long quads[],		/* quad specification				*/
    long *nquads,		/* number of quads				*/
    char *annot_host,	/* annotation host name				*/
    char *head_host,	/* header host name				*/
    char *trail_host,	/* trailer host name				*/
    long window[],		/* window specification				*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    char *minmax,		/* minmax flag buffer				*/
    long *msgtim		/* elapsed time between display messages	*/
)
{
extern struct PARBLK vblock;	/* pointer to TAE block			*/
extern long com_idx[4];		/* index into comment array		*/
extern char *comment[4];	/* mount messages for each tape		*/
extern short print_flag;	/* Flag for print file open		*/
extern short andsk;		/* annotation file flag		        */
extern long  term;		/* Flag for terminal output		*/

static long qline[MAX_QUADS] = {2983,2983,2982,2982};

			/* number of samples in each quad		*/
static long qsamp[MAX_QUADS] = {4220,4220,4220,4220};

long i;			/* loop index					*/
long unit;		/* tape unit number				*/
long format;		/* Image file format (BSQ or BIL)		*/
long cur_quad;		/* current quad number				*/
long ex_quad;		/* quadrant expected to be mounted		*/
#ifdef vms
long four = 4;
#endif
long lastns;		/* NS written to the last line processed	*/
long nbytes;		/* number of bytes to be read from tape		*/
long skip;		/* number of files to skip on tape		*/
long path;		/* WRS path number				*/
long utmzone;		/* UTM zone number				*/
long line_center;	/* Line number at scene center			*/
long samp_center;	/* sample number at scene center		*/
long status;		/* function return status code			*/
long right_fill;	/* right fill value				*/
long left_fill;		/* left fill value				*/
long tape_rec;		/* current tape record     			*/
long *tmp;		/* pointer to temporary variable		*/

long quads_used[4];	/* the quadrants actually used			*/
long nl[MAX_QUADS];     /* number of lines				*/
long ns[MAX_QUADS];     /* number of samples per line			*/
long origwin[4];	/* original window before line and sample check	*/
long one = 1;		/* constant one					*/
long zero = 0;		/* value of zero				*/
long nonfatal = 1;	/* Non-Fatal error code				*/
long nquads_used = 0;	/* number of quads used				*/
long mode = HANDLE;	/* tape I/O outputs error messages		*/
short ldone = FALSE;	/* has line check on window been done?		*/
short pdone = FALSE;	/* has pixel check on window been done? 	*/

char date[9];		/* acquisition date				*/
char lat_long[18];	/* lat long at scene center			*/
char path_row[9];	/* WRS path row					*/
char quality[11];	/* overall geometric quality code		*/
char resamp[18];	/* resampling technique				*/
char sceneid[11];	/* scene id read from text record		*/
char scene_id[26];	/* scene id					*/
char source[14];	/* data source					*/
char direction[11];	/* Orbital direction				*/
char maprj[4];		/* Map projection identifier			*/
char text[11];		/* text in display message			*/
char mesg[ERRLEN];	/* informational message			*/

double angle;		/* Angle of rotation converted into radians	*/
double fcenll[2];	/* Full scene center latitude/longitude degrees	*/
double fcenls[2];	/* Full scene center line/sample		*/
double fcproj[2];	/* Full scene center in projection coordinates	*/
double outmax[MAX_BANDS]; /* array of max values for each output band	*/
double outmin[MAX_BANDS]; /* array of min values for each output band	*/

unsigned char inbuf[4320];/* input buffer				*/

/*
  Open the necessary associated files
*/
open_assoc(annot_host,head_host,trail_host);

/*
  Verify the window entered is valid
*/
verify_win(window,imshape,quads);
for (i=0; i < 4; i++)
    origwin[i] = window[i];

/*
  Processing loop -- read in the specified quads
*/
for (i = 0, lastns = 0; i < *nquads; i++)
    {
    /*
      If all the specified samples were processed in the left quad, there
      is no need to process the right quad.
    */
    if ((lastns == window[3]) && (*imshape == 'w') && (i == 1))
	{
	sprintf(mesg,"%s%s","Window does not intersect quadrant 2, ",
		"processing quadrant 3");
	c_errmsg(mesg,"tipsp-nointer",&nonfatal);
	lastns = 0;
	continue;
	}
    else if ((lastns == window[3]) && ((*imshape == 'u') || 
	     (*imshape == 'b') || (*imshape == 'w')))
	{
	sprintf(mesg,
		"Window does not intersect quadrant %d, processing complete",
		quads[i]);
	c_errmsg (mesg,"tipsp-nointer",&nonfatal);
	continue;
	}

    /*
      Mount the the tape the the quadrant to be processed
    */
    if (i == 0)
	status = mount_first(&unit,&ex_quad,nquads,quads);
    else
	{
	status = mount_tape(&unit,&i,quads[i]);
	ex_quad = quads[i];
	}

    if (status != E_SUCC)
	clean_up(&unit,status);

    /*
      Set up current quad number for the display message
    */
    sprintf(text,"for quad %d",ex_quad);

    /*
      Check that the appropriate volume is mounted
    */
    check_vol(&unit,&cur_quad,&ex_quad,sceneid,&format);

startover:
    /*
      Position tape to first specified band
    */
    if ((bands[0] != 1) && (format == BSQ))
	{
	c_errmsg("Skipping to specified input band","tipsp-skip",&nonfatal);
	skip = 3 * (bands[0]-1);
	status = c_tpfsf(&unit,&skip,&mode);
	if (status != TAPE_SUCC)
	    clean_up(&unit,TIPS_READ);
	}

    /*
      Process the header information -- if the header file is not to be
      written to disk, the header file only needs to be read once (for
      the first quad processed) to get information needed for the DDR.
    */
    status = get_head(&unit,&i,&format,bands,fcenll,fcenls,fcproj,&angle,&path,
		      &utmzone,direction,maprj,&line_center,&samp_center);
    if (status != E_SUCC)
	clean_up(&unit,status);

    /*
      Read the file descriptor record to get the size of the input quad
    */
    status = get_imgfd(&unit,&nl[i],&ns[i]);
    if (status != E_SUCC)
	clean_up(&unit,status);

    tape_rec = 2;

    if ((i == 0) && (ldone == FALSE)) 
	{
	ldone = TRUE;

	/*
	  Determine whether or not the window specified intersects the
	  quadrants input (based on starting line and number of lines)
	*/
    	if ((*imshape == 'l') || (*imshape == 'r') || (*imshape == 'w'))
	    check_line(window,imshape,&qline[quads[0]-1],&qsamp[quads[0]-1],
			quads,nquads);

	if ((cur_quad != quads[0]) && (strcmp(comment[com_idx[0]],"NA") != 0))
	    {
	    /*
	      The window specified did not intersect the first quad specified,
	      so prompt the operator to mount the tape of the first quadrant
	      the window does intersect
	    */
	    status = swap_tape(&unit,&quads[0],&i,&format);
	    if (status != E_SUCC)
          	clean_up(&unit,status);

	    /*
	      Delete any current associated files so ones for the
	      appropriate quad can be created.
	    */
	    close_assoc();
	    delete_assoc(annot_host,head_host,trail_host);
	    open_assoc(annot_host,head_host,trail_host);

	    /*	
	      Set up current quad number for the display message
	    */
       	    sprintf(text,"for quad %d",quads[i]);
	    tape_rec = 0;
	    cur_quad = quads[0];
	    goto startover;
	    }
	}

    if ((i == 0) && (pdone == FALSE)) 
	{
	pdone = TRUE;

	/*
	  Determine whether or not the window specified intersects the
	  quadrants input (based on starting sample and number of samples)
	*/
    	if ((*imshape == 'u') || (*imshape == 'b') || (*imshape == 'w'))
	    {
	    /*
	      Read the first line of the image to be processed to get the
	      fill counts.
	    */
	    if (format == BSQ)
		skip = window[0] - 1;
	    else
		skip = (window[0] - 1) * 7;

	    if (skip > 0)
		{
		c_errmsg("Skipping to first input record","tipsp-skip",
			 &nonfatal);
		status = c_tpfsr(&unit,&skip,&mode);
		if (status != TAPE_SUCC)
          	    clean_up(&unit,TIPS_READ);
		}

	    if (format == BSQ)
		tape_rec = window[0] + 1;
	    else
		tape_rec = ((window[0] - 1) * 7) + 2;

	    nbytes = 4320;
	    status = c_tpread(&unit,inbuf,&nbytes,&mode);
	    if (status != nbytes)
          	clean_up(&unit,status);

	    tmp = (long*) &inbuf[28];
	    right_fill = *tmp;
	    tmp = (long*) &inbuf[24];
	    left_fill = *tmp;

#ifdef vms
	    status = c_pxswap(&right_fill,&one,&four);
	    status = c_pxswap(&left_fill,&one,&four);
#endif

	    /*
	      If the first quadrant specified is not available -- calculate
	      what the right fill for that quadrant would have been based on
	      the left fill of the quadrant mounted (the total of the right
	      fill of quadrant 1 and the left fill of quadrant 2 is 1473.  The
	      same holds true for quadrants 3 and 4).
	    */
	    if (strcmp(comment[com_idx[quads[0]-1]],"NA") == 0)
		right_fill = 1473 - left_fill;

	    /*
	      Determine whether or not the window specified intersects the
	      quadrants input based on the starting sample and number of
	      samples.
	    */
	    check_pixel(window,imshape,&qsamp[quads[0]-1],quads,nquads,
			&right_fill);

	    /*
	      The window specified intersects the first quadrant specified, so
	      back up one record (because of the record that was read above for
	      determining fill counts)
	    */
	    if (cur_quad == quads[0])
		{
		status = c_tpbsr(&unit,&one,&mode);
		if (status != TAPE_SUCC)
		    clean_up(&unit,TIPS_READ);
		}
	    }

	/*
	  The window specified did not intersect the first quad specified,
	  so prompt the operator to mount the tape of the first quadrant
	  the window does intersect
	*/
	if (cur_quad != quads[0])
	    {
	    status = swap_tape(&unit,&quads[0],&i,&format);
	    if (status != E_SUCC)
          	clean_up(&unit,status);

	    /*
	      Delete any current associated files so ones for the appropriate
	      quad can be created.
	    */
	    close_assoc();
	    delete_assoc(annot_host,head_host,trail_host);
	    open_assoc(annot_host,head_host,trail_host);

	    /*
	      Set up current quad number for the display message
	    */
       	    sprintf(text,"for quad %d",quads[i]);
	    tape_rec = 0;
	    cur_quad = quads[0];
	    goto startover;
	    }
	}

    /*
      Process the BSQ or BIL formatted image
    */
    if (format == BSQ)
	winbsq(&unit,hosout[0],&i,&nl[i],&ns[i],window,bands,nbands,quads,
		quads_used,&nquads_used,&lastns,&tape_rec,imshape,outmin,
		outmax,minmax,msgtim,text);
    else
	winbil(&unit,hosout[0],&i,&nl[i],&ns[i],nbands,bands,window,quads,
		imshape,&tape_rec,&lastns,quads_used,&nquads_used,outmin,
		outmax,minmax,msgtim,text);

    /*
      Process the annotation information
    */
    if ((andsk) || (((term) || (print_flag)) && (i == 0)))
	get_annot(&unit,&i,bands,date,lat_long,path_row,scene_id,source,resamp,
		  quality);

    /*
      Dismount the tape
    */
    dismount_tape(&unit);
    }

/*
  Close the associated files
*/
close_assoc();

/*
  Update the DDR information
*/
update_ddr(hosout[0],origwin,nbands,scene_id,maprj,direction,&path,&utmzone,
	   fcproj,fcenll,fcenls,&angle,date);

/*
  Create the associated history file
*/
status = c_hstout(&vblock,hosout[0],hosout,&zero,"ccttipsp");
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered--Image data saved","tipsp-warn",
	     &nonfatal);

/*
  Create the associated min max file
*/
status = c_svmnmx(hosout[0],nbands,outmin,outmax,minmax);
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered--Image data saved","tipsp-warn",
	     &nonfatal);

/*
  Write the parameter information to the output print file and TAE parameter
  block
*/
write_parm(date,lat_long,path_row,scene_id,source,resamp,quality,&line_center,
	   &samp_center,origwin,&nquads_used,quads_used,sceneid);

return;
}
