/******************************************************************************
FUNCTION:	READ_WBIL

PURPOSE:	Reads in the specified window from all specified bands of
		in image in band interleaved by line format.   This routine
		is used when a window was specified or when the quadrants
		are to be mosaicked.

ALGORITHM 
  Determine whether or not mosaicking is necessary for the quadrant being
  processed
  Find the number of records to skip based on the tape start record previously
  calculated (the number of records to skip is 2 less than this number because
  the file decriptor record has already been read)
  Start the processing message
  Loop on the number of tape records to read (previously determined by INITQUAD)
	  If the proccessing band is back to one write the image bands to
	  the image
          Read a record from tape
	  Determine whether this band is to be processed
	  Get the fill counts
	  Swap the bytes of the filll counts if on a vax
	      Determine where to start retrieving data from the buffer read
	      from tape (START), where to place this data in the buffer to be
	      written to disk (START_SAMP) and the number of samples to
	      retrieve from the tape buffer (NS)
	      If the current line is already complete, continue to the next line
	  If mosaicing fill the output buffer with zero's.  The buffer is the
	  size of the full moasaiced image line
	  Write the input line to the output buffer
	  Calculated minimum and maximum values if necessary
  Flush the output buffer
  Stop the processing message
  If the total number of lines have not been written to the image, fill
  in the remaining lines with zeros.  This is done so the full image size
  is defined.  If this is not done, it causes an error in eopenr when
  the file is reopened to process the bottom part of the image
  Close all image files in the group
  Free the dynamically allocated memory
  read EOF mark after image file
  Set the last sample number written to the image

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

int read_wbil
(
    long *unit,	   	/* tape unit number				*/
    long *count,		/* loop counter				  	*/
    long window[],		/* user specified window               		*/
    long bands[],		/* band numbers to be processed		 	*/
    long quads[],		/* quads to process			  	*/
    long *nsquad,		/* number of samples in the quad	  	*/
    long *lastns,		/* number of samples written to last line 	*/
    long disk_rec[],	/* current disk record number             	*/
    long disk_startrec[],	/* disk record to start writing 	  	*/
    long *tape_rec,		/* current tape record number		  	*/
    long tape_startrec[],	/* tape record number of start reading    	*/
    long *tape_lastrec,	/* tape record number of last record to read	*/
    char *imshape,	 	/* shape of the output image
				w - whole image
                                q - single quad
				u - upper half (quads 1 and 2)
				b - bottom half (quads 3 and 4)
				l - left half (quads 1 and 3)
				r - right half (quads 2 and 4)		*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
)
{
extern struct GDESC * gdesc;		/* group file descriptor (seq I/O)	*/
extern long offset[MAXBND+1];	/* offsets into output buffer (seq I/O) */
extern long curent;	/* current line number - needed just so that 	*/
			/* the display messages start with a one.	*/

extern char *buffer;	/* I/O buffer					*/

short mosaic;		/* mosaicking flag 				*/

#ifdef vms
long four = 4;
#endif
long ns;		/* number of samples to copy            	*/
long skip;		/* number of records to skip            	*/
long start_samp;	/* first sample in disk file to write   	*/
long end_samp;		/* last sample in disk file to write    	*/
long start;		/* first sample from tape data buffer to copy 	*/
long write_flag;	/* flag to write current image line to disk	*/
long right_fill;	/* right fill count                     	*/
long left_fill;		/* left fill count				*/
long temp;		/* temporary value				*/
long *tmp;		/* pointer to temporary variable        	*/
long status;		/* function return status code          	*/
long one = 1;		/* constant one					*/
unsigned char zero = 0;		/* constant zero				*/
long bndidx = 0;	/* index into bands array			*/
long band_num = 0;	/* band number of record read from tape		*/
long nonfatal = 1;	/* Nonfatal error code				*/
long nbytes = 4320;	/* number of bytes to be read from tape 	*/
long mode = HANDLE;	/* tape I/O outputs error messages 		*/
long dtype = EBYTE;	/* BYTE data type				*/

unsigned char inbuf[4320]; /* input data buffer				*/

curent = disk_startrec[*count];

/*
  Determine whether or not mosaicking is necessary for the quadrant being
  processed
*/
if (((quads[*count] % 2) == 0) && ((*imshape == 'u') || (*imshape == 'b') ||
    (*imshape == 'w')))
    mosaic = TRUE;
else
    mosaic = FALSE;

/*
  Find the number of records to skip based on the tape start record previously
  calculated (the number of records to skip is 2 less than this number because
  the file decriptor record has already been read)
*/
if (*tape_rec != tape_startrec[*count])
    {
    c_errmsg("Skipping to first input record","tipsp-skip",&nonfatal);
    skip = tape_startrec[*count] - 2;
    if (skip > 0)
        status = c_tpfsr(unit,&skip,&mode);
    }

/*
  Start the processing message
*/
c_startm();

/*
  Loop on the number of tape records to read (previously determined by INITQUAD)
*/
disk_rec[*count] = disk_startrec[*count];
for (*tape_rec=tape_startrec[*count]; *tape_rec <= *tape_lastrec; (*tape_rec)++)
	{
	(band_num + 1 > 7) ? band_num = 1 : band_num++;

	/*
	  If the proccessing band is back to one write the image bands to
	  the image
	*/
	if (band_num == 1)
	    {
            if (c_estep(&gdesc) != E_SUCC)
	        return (TIPS_WRITE);

	    bndidx = 0;
	    curent++;
	    disk_rec[*count]++;
	    }

	/*
          Read a record from tape
	*/
        status = c_tpread(unit,inbuf,&nbytes,&mode);
        if (status != nbytes)
            return(TIPS_READ);

	/*
	  Determine whether this band is to be processed
	*/
	if (band_num != bands[bndidx])
	    continue;

	/*
	  Get the fill counts
	*/
	tmp = (long*) &inbuf[28];
        right_fill = *tmp;
        tmp = (long*) &inbuf[24];
        left_fill = *tmp;


	/*
	  Swap the bytes of the filll counts if on a vax
	*/
#ifdef vms
	status = c_pxswap(&right_fill,&one,&four);
	status = c_pxswap(&left_fill,&one,&four);
#endif

	write_flag = TRUE;
	if ((quads[*count] % 2) == 0)
	    {
	    /*
	      Determine where to start retrieving data from the buffer read
	      from tape (START), where to place this data in the buffer to be
	      written to disk (START_SAMP) and the number of samples to
	      retrieve from the tape buffer (NS)
	    */
	    if ((*imshape == 'u') || (*imshape == 'b') || (*imshape == 'w'))
		temp = *nsquad - (TMSAMP - window[1]);
	    else
		temp = window[1];

	    if (left_fill > temp - 1)
	        start = left_fill + 32;
	    else
		start = temp - 1 + 32;

	    if (((temp + window[3] - 1) - left_fill) < window[3])
		start_samp = (window[3] - ((temp + window[3]-1) - left_fill))+1;
	    else
		start_samp = 1;

	    ns = window[3] - start_samp + 1;

	    /*
	      If the current line is already complete, continue to the next line
	    */
	    if (ns <= 0)
		{
		write_flag = FALSE;
		ns = window[3];
		}
	    }
	else
	    {
	    start = window[1] - 1 + 32;
	    start_samp = 1;
	    if (((*nsquad - window[1] + 1) - right_fill) < window[3])
		end_samp = (*nsquad - window[1] + 1) - right_fill; 
	    else
		end_samp = window[3];

	    if (end_samp <= 0)
		break;

	    ns = end_samp - start_samp + 1;
	    }

	/*
	  If mosaicing fill the output buffer with zero's.  The buffer is the
	  size of the full moasaiced image line
	*/
	if (mosaic != TRUE)
	    c_pxfill(buffer + (*(offset+bndidx)),&dtype,&window[3],
			      &zero);

	/*
	  Write the input line to the output buffer
	*/
	if (write_flag == TRUE)
	    c_pxcopy(&inbuf[start],buffer + (*(offset+bndidx)) + (start_samp-1),
		     &dtype,&ns);

	/*
	  Calculated minimum and maximum values if necessary
	*/
    	if (((ns == window[3]) || (mosaic == TRUE)) &&
	     (strcmp(minmax,"yes") == 0))
	    status = c_pxmnmx(buffer + (*(offset+bndidx)),&ns,&outmin[bndidx],
			      &outmax[bndidx],&dtype);
	bndidx++;
    }

/*
  Flush the output buffer
*/
if (c_estep(&gdesc) != E_SUCC)
    return (TIPS_WRITE);

/*
  Stop the processing message
*/
c_stoptm();

/*
  If the total number of lines have not been written to the image, fill
  in the remaining lines with zeros.  This is done so the full image size
  is defined.  If this is not done, it causes an error in eopenr when
  the file is reopened to process the bottom part of the image
if ((*tape_rec <= *tape_lastrec) || (disk_rec[*count] <= window[2]))
    {
    startfill = disk_rec[*count];
    stopfill = window[2];
    for (bndidx=0; bndidx < *nbands; bndidx++)
        status = c_pxfill(buffer + (*(offset+bndidx)),&dtype,&window[3],&zero);

    for (i = startfill; i <= stopfill; i++,curent++)
        if (c_estep(&gdesc) != E_SUCC)
	    return (TIPS_WRITE);
    }
*/

/*
  Close all image files in the group
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing image group -- Image data saved","tipsp-close",
	    &nonfatal);
*/

/*
  Free the dynamically allocated memory
*/
free(buffer);

/*
  read EOF mark after image file
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != TAPE_EOF)
   {
   status = c_tpfsf(unit,&one,&mode);
   if (status != TAPE_SUCC)
       return(TIPS_READ);
   }

/*
  Set the last sample number written to the image
*/
*lastns = ns;

return(E_SUCC);
}
