/******************************************************************************
FUNCTION:	READ_WIN

PURPOSE:	Reads in the specified window of a band from the current quad.
		This routine is used when a window was specified or when the
		quadrants are to be mosaicked.

ALGORITHM 
  Determine whether or not mosaicking is necessary for the quadrant being
  processed
  Find the number of records to skip based on the tape start record previously
  calculated (the number of records to skip is 2 less than this number because
  the file decriptor record has already been read
  Start the processing message
  Loop on the number of tape records to read (previously determined by INITQUAD
	  Read a line from tape
	  Get the fill counts
	  Swap the bytes if on a vax
	      Determine where to start retrieving data from the buffer read
	      from tape (START), where to place this data in the buffer to
	      be written to disk (START_SAMP) and the number of samples to
	      retrieve from the tape buffer (NS)
	      If the current line is already complete, continue to the next line
	  If mosaic flag is set re-read the line from the output image
	  The next quad will be added to the end of this buffer
	  Write copy the input line to the output buffer
	  Calculated minimum and maximum values if necessary
	  Write the image line to the disk file
  Stop the processing message
  read EOF mark after image file
  Set the last sample number written to the image

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

int read_win
(
    long *unit,		/* tape unit number				*/
    long *count,		/* loop counter					*/
    long window[],		/* user specified window			*/
    long *band_num,		/* current output band number			*/
    long quads[],		/* quads to process				*/
    long *nsquad,		/* number of samples in the quad		*/
    long *lastns,		/* number of samples written to last line	*/
    long disk_rec[],	/* current disk record number			*/
    long disk_startrec[],	/* disk record to start writing			*/
    long *tape_rec,		/* current tape record number			*/
    long tape_startrec[],	/* tape record number of start reading		*/
    long *tape_lastrec,	/* tape record number of last record to read	*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
)
{
extern struct FDESC * fdesc[2];	/* image I/O file descriptors 		*/
extern long curent;	/* current line number				*/
			/* needed just so that the display messages	*/
			/* don't start with "processed line 0"		*/

short mosaic;		/* mosaicking flag				*/

#ifdef vms
long four = 4;
#endif
long lnum;		/* line number for ewrite			*/
long ns;		/* number of samples to copy    		*/
long left_fill;		/* left fill count				*/
long right_fill;	/* right fill count             		*/
long skip;		/* number of records to skip         		*/
long start;		/* first sample from tape data buffer to copy	*/
long start_samp;	/* first sample in disk file to write 		*/
long end_samp;		/* last sample in disk file to write  		*/
long status;		/* function return status code        		*/
long temp;		/* temporary value				*/
long *tmp;		/* pointer to temporary variable		*/
long write_flag;	/* whether to write the current image line to disk */

long one = 1;		/* constant one				 	*/
unsigned char zero = 0;		/* constant zero				*/
long nonfatal = 1;	/* nonfatal error code				*/
long nbytes = 4320;	/* number of bytes to be read from tape  	*/
long outsiz = 6967;	/* output buffer size			 	*/
long mode = HANDLE;	/* tape I/O outputs error messages 	 	*/
long dtype = EBYTE;	/* BYTE data type			 	*/

unsigned char inbuf[4320];	/* input data buffer			*/
unsigned char outbuf[6967];	/* output data buffer		        */

curent = disk_startrec[*count];

/*
  Determine whether or not mosaicking is necessary for the quadrant being
  processed
*/
if (((quads[*count] % 2) == 0) && ((*imshape == 'u') || (*imshape == 'b') ||
    (*imshape == 'w')))
	mosaic = TRUE;
else
	mosaic = FALSE;

/*
  Find the number of records to skip based on the tape start record previously
  calculated (the number of records to skip is 2 less than this number because
  the file decriptor record has already been read
*/
if (*tape_rec != tape_startrec[*count])
    {
    c_errmsg("Skipping to first input record","tipsp-skip",&nonfatal);
    skip = tape_startrec[*count] - 2;
    if (skip > 0)
        status = c_tpfsr(unit,&skip,&mode);
    }

/*
  Start the processing message
*/
c_startm();

/*
  Loop on the number of tape records to read (previously determined by INITQUAD
*/
disk_rec[*count] = disk_startrec[*count];
for (*tape_rec = tape_startrec[*count], lnum = 1; *tape_rec <= *tape_lastrec;
    (*tape_rec)++, curent++, lnum++)
	{
        status = c_tpread(unit,inbuf,&nbytes,&mode);
        if (status != nbytes)
            return(TIPS_READ);

	/*
	  Get the fill counts
	*/
	tmp = (long*) &inbuf[28];
        right_fill = *tmp;
        tmp = (long*) &inbuf[24];
        left_fill = *tmp;

	/*
	  Swap the bytes if on a vax
	*/
#ifdef vms
	status = c_pxswap(&right_fill,&one,&four);
	status = c_pxswap(&left_fill,&one,&four);
#endif

	write_flag = TRUE;
	if ((quads[*count] % 2) == 0)
	    {
	    /*
	      Determine where to start retrieving data from the buffer read
	      from tape (START), where to place this data in the buffer to
	      be written to disk (START_SAMP) and the number of samples to
	      retrieve from the tape buffer (NS)
	    */
	    if ((*imshape == 'u') || (*imshape == 'b') || (*imshape == 'w'))
		temp = *nsquad - (TMSAMP - window[1]);
	    else
		temp = window[1];

	    if (left_fill > temp-1)
	        start = left_fill + 32;
	    else
		start = temp - 1 + 32;

	    if (((temp + window[3] - 1) - left_fill) < window[3])
		start_samp = (window[3] - ((temp + window[3]-1) - left_fill))+1;
	    else
		start_samp = 1;

	    ns = window[3] - start_samp + 1;

	    /*
	      If the current line is already complete, continue to the next line
	    */
	    if (ns <= 0)
		{
		write_flag = FALSE;
		ns = window[3];
		}
	    }
	else
	    {
	    start = window[1] - 1 + 32;
	    start_samp = 1;

	    if (((*nsquad - window[1] + 1) - right_fill) < window[3])
		end_samp = (*nsquad - window[1] + 1) - right_fill; 
	    else
		end_samp = window[3];

	    if (end_samp <= 0)
		break;

	    ns = end_samp - start_samp + 1;
	    }

	/*
	  If mosaic flag is set re-read the line from the output image
	  The next quad will be added to the end of this buffer
	*/
	if (mosaic == TRUE)
	    {
	    status = c_eread(&fdesc[0],band_num,&lnum,outbuf,&one);
	    if (status != window[3])
		return(TIPS_READ);
	    }
	else
	    c_pxfill(outbuf,&dtype,&outsiz,&zero);

	/*
	  Write copy the input line to the output buffer
	*/
	if (write_flag == TRUE)
	    c_pxcopy(&inbuf[start],&outbuf[start_samp-1],&dtype,&ns);

	/*
	  Calculated minimum and maximum values if necessary
	*/
    	if (((ns == window[3]) || (mosaic == TRUE)) &&
	     (strcmp(minmax,"yes") == 0))
	    status = c_pxmnmx(outbuf,&ns,&outmin[*band_num - 1],
			      &outmax[*band_num-1],&dtype);

	/*
	  Write the image line to the disk file
	*/
	if (write_flag == TRUE)
	    {
    	    status = c_ewrite(&fdesc[0],band_num,&lnum,outbuf,&one);
    	    if (status != window[3]) 
                return(TIPS_WRITE);
	    }

	disk_rec[*count]++;
    }

/*
  Stop the processing message
*/
c_stoptm();

/*
  If the total number of lines have not been written to the image, fill
  in the remaining lines with zeros.  This is done so the full image size
  is defined.  If this is not done, it causes an error in eopenr when
  the file is reopened to process the bottom part of the image
if ((*tape_rec <= *tape_lastrec) || (disk_rec[*count] <= window[2]))
    {
    startfill = disk_rec[*count];
    stopfill = window[2];
    c_pxfill(outbuf,&dtype,&outsiz,&zero);

    for (i = startfill; i <= stopfill; i++,curent++)
    	status = c_ewrite(&fdesc[0],band_num,&i,outbuf,&one);
    }
*/

/*
  read EOF mark after image file
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != TAPE_EOF)
   {
   status = c_tpfsf(unit,&one,&mode);
   if (status != TAPE_SUCC)
       return(TIPS_READ);
   }

/*
  Set the last sample number written to the image
*/
*lastns = ns;

return(E_SUCC);
}
