/******************************************************************************
FUNCTION:	SEPARATE

PURPOSE:	Controls the processing of separate output quad files

ALGORITHM 
  Processing loop -- each quad becomes a separate output image
      Open the necessary associated files
      Initialize min max arrays
      Mount the input tape volume
      Check that the appropriate volume is mounted
      Position tape to first specified band
      Process the header information -- if the header file is not to be
      written to disk, the header file only needs to be read once (for
      the first quad processed) to get information needed for the DDR.
      Read the image file descriptor record to get the size of the input quad
      Display the size of the output image
      Process BSQ or BIL formatted image data
      Process the annotation information -- if the annotation file is not to be
      written to disk and the print_flag is not set, the annotation file only
      needs to be read once (for the first quad processed)
      Close the associated files
      Create the associated history file
      Create the associated min max file
      Update the DDR information
  Write the parameter information to the output print file and TAE parameter
  block

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "protos.h"

void separate
(
    char hosout[][CMLEN],	/* output host names				*/
    long bands[],		/* band specification				*/
    long *nbands,		/* number of bands				*/
    long quads[],		/* quad specification				*/
    long *nquads,		/* number of quads				*/
    char annot_host[][CMLEN],/* annotation host name(s)			*/
    char head_host[][CMLEN],/* header host names				*/
    char trail_host[][CMLEN],/* trailer host names				*/
    char *minmax,		/* min max flag					*/
    long *msgtim		/* elapsed time between display messages	*/
)
{
extern struct PARBLK vblock; /*  TAE parameter block			*/
extern short print_flag;/* Flag for print file open			*/
extern short andsk;	/* annotation file flag		        	*/
extern long  term;		/* Flag for terminal output		*/

long unit;		/* tape unit number				*/
long cur_quad;		/* current quad number				*/
long i, j;		/* loop index					*/
long skip;		/* number of files to skip on tape		*/
long status;		/* function return status code			*/
long format;		/* image file format (BSQ or BIL)		*/
long path;		/* WRS path number				*/
long utmzone;		/* UTM zone number				*/
long line_center;	/* Line number at scene center			*/
long samp_center;	/* sample number at scene center		*/
long window[4];		/* window value for TAE global			*/
long nl[MAX_QUADS];	/* number of lines				*/
long ns[MAX_QUADS];	/* number of samples per line			*/

long zero = 0;		/* value of zero				*/
long mode = HANDLE;	/* tape I/O outputs error mesages		*/
long nonfatal = 1;	/* Non-fatal error code				*/

char date[9];		/* acquisition date				*/
char lat_long[18];	/* lat long at scene center			*/
char path_row[9];	/* WRS path row					*/
char quality[11];	/* overall geometric quality code		*/
char resamp[18];	/* resampling technique				*/
char sceneid[11];	/* scene id read from text record		*/
char scene_id[26];	/* scene id					*/
char source[14];	/* data source					*/
char direction[11];	/* Orbital direction				*/
char maprj[4];		/* Map projection identifier			*/
char msg[ERRLEN];	/* size message					*/

double angle;		/* Angle of rotation converted into radians	*/
double fcenll[2];	/* Full scene center latitude/longitude degrees	*/
double fcenls[2];	/* Full scene center line/sample		*/
double fcproj[2];	/* Full scene center in projection coordinates	*/
double outmin[MAX_BANDS]; /* array of min values for each output band	*/
double outmax[MAX_BANDS]; /* array of max values for each output band	*/


/*
  Processing loop -- each quad becomes a separate output image
*/
for (i = 0; i < *nquads; i++)
    {
    /*
      Open the necessary associated files
    */
    open_assoc(annot_host[i],head_host[i],trail_host[i]);

    /*
      Initialize min max arrays
    */
    for (j = 0; j < *nbands; j++)
	{
        outmin[j] = 255.0;
        outmax[j] = 0.0;
  	}

    /*
      Mount the input tape volume
    */
    status = mount_tape(&unit,&i,quads[i]);
    if (status != E_SUCC)
        clean_up(&unit,status);

    /*
      Check that the appropriate volume is mounted
    */
    check_vol(&unit,&cur_quad,&quads[i],sceneid,&format);

    /*
      Position tape to first specified band
    */
    if ((bands[0] != 1) && (format == BSQ))
 	{
        c_errmsg("Skipping to specified input band","tipsp-skip",&nonfatal);
        skip = 3 * (bands[0] - 1);
        status = c_tpfsf(&unit,&skip,&mode);
 	if (status != TAPE_SUCC)
   	    clean_up(&unit,TIPS_READ);
	}

    /*
      Process the header information -- if the header file is not to be
      written to disk, the header file only needs to be read once (for
      the first quad processed) to get information needed for the DDR.
    */
    status = get_head(&unit,&i,&format,bands,fcenll,fcenls,fcproj,&angle,&path,
		      &utmzone,direction,maprj,&line_center,&samp_center);
    if (status != E_SUCC)
	clean_up(&unit,status);

    /*
      Read the image file descriptor record to get the size of the input quad
    */
    status = get_imgfd(&unit,&nl[i],&ns[i]);
    if (status != E_SUCC)
	clean_up(&unit,status);

    /*
      Display the size of the output image
    */
    sprintf(msg,"Size of output image is %d x %d",nl[i],ns[i]);
    c_errmsg(msg,"informational",&nonfatal);

    /*
      Process BSQ or BIL formatted image data
    */
    if (format == BSQ)
	sepbsq (&unit,hosout[i],&quads[i],&nl[i],&ns[i],nbands,bands,outmin,
		outmax,minmax,msgtim);
    else
	sepbil (&unit,hosout[i],&quads[i],&nl[i],&ns[i],nbands,bands,outmin,
		outmax,minmax,msgtim);

    /*
      Process the annotation information -- if the annotation file is not to be
      written to disk and the print_flag is not set, the annotation file only
      needs to be read once (for the first quad processed)
    */
    if ((andsk) || (((term) || (print_flag)) && (i == 0)))
	get_annot(&unit,&i,bands,date,lat_long,path_row,scene_id,source,
		  resamp,quality);

    /*
      Close the associated files
    */
    close_assoc();

    /*
      Create the associated history file
    */
    status = c_hstout(&vblock,hosout[i],hosout,&zero,"ccttipsp");
    if (status != E_SUCC)
   	c_errmsg("Nonfatal error encountered--Image data saved","tipsp-warn",
		 &nonfatal);

    /*
      Create the associated min max file
    */
    status = c_svmnmx(hosout[i],nbands,outmin,outmax,minmax);
    if (status != E_SUCC)
        c_errmsg("Nonfatal error encountered--Image data saved","tipsp-warn",
		 &nonfatal);

/*
  Set the starting window location relative to the full scene to get the
  georeference locations for the DDR.
*/
    if (quads[i] == 1)
	{
	window[0] = 1;
	window[1] = 1;
	}
    else if (quads[i] == 2)
	{
	window[0] = 1;
	window[1] = 4220 - 1473 + 1;
	}
    else if (quads[i] == 3)
	{
	window[0] = 2984;
	window[1] = 1;
	}
    else if (quads[i] == 4)
	{
	window[0] = 2984;
	window[1] = 4220 - 1473 + 1;
	}

    window[2] = nl[i];
    window[3] = ns[i];

    /*
      Update the DDR information
    */
    update_ddr(hosout[i],window,nbands,scene_id,maprj,direction,&path,
	       &utmzone,fcproj,fcenll,fcenls,&angle,date);

    dismount_tape(&unit);
    }


    for (i = 0; i < 4; i++)
	window[i] = 0;
/*
  Write the parameter information to the output print file and TAE parameter
  block
*/
write_parm(date,lat_long,path_row,scene_id,source,resamp,quality,&line_center,
	  &samp_center,window,nquads,quads,sceneid);

return;
}
