/******************************************************************************
FUNCTION:	SEPBSQ

PURPOSE:	Processing loop for band sequential images that are not being
		mosaicked.

ALGORITHM 
  Set up current quad number for the display message
  Set up number of lines for the display message
  Open the output image
  Set the image random open LAS flag (2)
  Process each band
      Read in the image file for the currend quad
      Process the trailer record
      Position tape to next specified input band
	  Skip to the calibration record
	  Process the calibration record
	  Skip over file descriptor record of the image file
  Close the output image

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "protos.h"

void sepbsq
(
    long *unit,		/* Tape unit number			      	*/
    char *hname,	/* Desc for host file name of output image    	*/
    long *quad,		/* Current quadrant number		      	*/
    long *nl,		/* Number of lines in the quad		      	*/
    long *ns,		/* Number of samples in the quad	      	*/
    long *nbands,	/* Number of bands being processed	      	*/
    long bands[],  	/* Band specification		      	      	*/
    double outmin[],	/* Array of min values for each output band	*/
    double outmax[],	/* Array of max values for each output band	*/
    char *minmax,	/* Min max flag			      	   	*/
    long *msgtim	/* Number of seconds between processing messages*/
)
{
extern short hdsk;		/* header file flag			*/
extern short las_flag;		/* Flag for las open			*/
extern struct FDESC * fdesc[2];		/* image I/O file descriptors 		*/
extern long curcat;		/* Current band number       	        */

static	long	opt[3]={0,0,0};	/* optional parameter array for EOPENR  */
long j;			/* Loop index				      	*/
long nbs;		/* Number of bands to skip  		      	*/
long skip;		/* Number of files to skip on tape	      	*/
long status;		/* Status indicator			      	*/
long one = 1;		/* one						*/
long nonfatal = 1;	/* Non-fatal error code				*/
long format = BSQ;	/* Image data format (BSQ or BIL)	  	*/
long flag = ONECAT;	/* Display message flag--one band at a time 	*/
long mode = HANDLE;	/* Tape I/O outputs error mesages             	*/
long dtype = EBYTE;	/* Data type of image (always BYTE)		*/
long access = IWRITE;	/* Open file for write access			*/

char text[11];		/* Text for processing message		      	*/

/*
  Set up current quad number for the display message
*/
sprintf(text,"in quad %d",*quad);

/*
  Set up number of lines for the display message
*/
c_inittm("CCTTIPSP","processed","line","band",text,nl,nbands,msgtim,&flag);

/*
  Open the output image
*/
status = c_eopenr(&fdesc[0],hname,nbands,&one,&one,nl,ns,&access,&dtype,opt,
		  &one);
if (status != E_SUCC)
    clean_up(unit,TIPS_OPEN);

/*
  Set the image random open LAS flag (2)
*/
las_flag = 2;

/*
  Process each band
*/
for (j = 0, curcat = 0; j < *nbands; j++)
    {
    curcat++;

    /*
      Read in the image file for the currend quad
    */
    status = read_quad(unit,nl,ns,outmin,outmax,minmax);
    if (status != E_SUCC)
	clean_up(unit,status);

    /*
      Process the trailer record
    */
    status = get_trail(unit,&j);
    if (status != E_SUCC)
	clean_up(unit,status);

    /*
      Position tape to next specified input band
    */
    if ((j + 1) < *nbands)
	{
	if ((bands[j] + 1) == bands[j + 1])
	    {
	    if (hdsk)
		skip = 0;
	    else
		skip = 1;
	    }
	else 
	    {
	    printf ("\n");
  	    c_errmsg("Skipping to next input band","tipsp-skip",&nonfatal);
   	    nbs = (bands[j + 1] - bands[j]) - 1;

	    if (hdsk)
		skip = 3 * nbs;
	    else
		skip = (3 * nbs) + 1;
	    }

	status = c_tpfsf(unit,&skip,&mode);
	if (status != TAPE_SUCC)
	    clean_up(unit,TIPS_READ);

	if (hdsk)
	    {
	    /*
	      Skip to the calibration record
	    */
            skip = 3;
            status = c_tpfsr(unit,&skip,&mode);

	    /*
	      Process the calibration record
	    */
	    status = get_calib(unit,bands,&format);
	    if (status != E_SUCC)
		clean_up(unit,status);
	    }

	/*
	  Skip over file descriptor record of the image file
	*/
        skip = 1;
        status = c_tpfsr(unit,&skip,&mode);
        if (status != TAPE_SUCC)
	    clean_up(unit,TIPS_READ);
	}
    }

/*
  Close the output image
*/
status = c_eclose(&fdesc[0]);
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered--Image data saved","tipsp-warn",
	     &nonfatal);

las_flag = 0;

return;
}
