/******************************************************************************
FUNCTION:	CCTTIPSP

PURPOSE:	Reads the P-type TM scenes that are recorded as quadrants
		on four nine-track 6250 bpi magnetic tapes in band sequential
		format.  If no window is specified, the user is given the
		option to have the specified quadrants mosaicked into one
		output image or to have each quadrant output as a separate
		image.  If a window is specified and it intersects more than
		one quadrant, the pieces of the window are automatically
		mosaicked.  The user may also specify a subset of bands to
		be processed.  Finally, information from the leader, trailer,
		and supplemental data files may be written to associated disk
		files.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      03/86    K. Zanter		Original development
  2.0      11/87    L. Huewe		Added windowing capabilities
  5.0      03/89    D. Hollaren		Conversion to LAS 5.0 and added BIL
					processing
  5.1      05/89    D. Hollaren		Fixed problem with trying to process
					calibration when processing quads 
					seperately
  5.2      08/89    D. Hollaren         Fixed problem updating DDR when windowed
  5.3      11/93    D. Etrheim          Correct density specification

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
The values of the TAE input parameters and the global variables
are retrieved.  The proper processing function is then called.  There
are two different processing control functions:
   1)  Separate
       If the user does not want mosaicking performed or the input
       is only a single quad and no window is specified, this function 
       is used.  Each input quad becomes a separate output image with its 
       own associated files.

   2)  Proc_window 
       If the user specified a window or specified that mosaicking was
       required, this function is used.  If a window was specified,
       proc_window determines which of the input quadrants are
       intersected.  If the window intersects a quadrant that is not
       specified, an error occurs.  The window specified is always
       relative the full TM scene (5965 x 6967).  For the first quadrant
       processed, an image defining the entire input area is opened and 
       the appropriate image data is read from tape.  For each remaining
       quadrant, the only the area containing the lines of the image to be 
       processed is opened (processing always starts at sample 1) and 
       the appropriate image data read from tape and output to the
       defined image.  This function creates just one image file with
       one set of associated files.
       
History and minmax files are created to represent the image(s).  If
necessary, a file is made containg the defined information from the 
header and annotation records and is sent to the terminal, line printer
or a text file.  

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "tm.h"
#include "protos.h"

struct FDESC * fdesc[2];	/* image file descriptors used for image I/O    */
struct GDESC * gdesc;		/* group file descriptor for sequential I/O	*/
long offset[MAXBND+1]; /* offsets into output buffer for sequential I/O*/
long com_idx[4];	/* index into comment array			*/
long out_count;		/* number of output images			*/

long annot_fd;		/* pointer to annotation file descriptor	*/
long head_fd;		/* pointer to header file descriptor  		*/
long trail_fd;		/* pointer to trailer file descriptor		*/

unsigned char *buffer;	/* I/O buffer					*/

FILE *out_fp;		/* File pointer to print file			*/

short las_flag;		/* Flag for las open				*/
short tape_flag;	/* Flag for tape open				*/
short print_flag;	/* Flag for print file open			*/
short assoc_flag;	/* Flag for associated files open		*/
short andsk;		/* annotation file flag		                */
short hdsk;		/* header file flag			        */
short tdsk;		/* trailer file flag			        */
long  term;		/* Flag for terminal output			*/

char *comment[4];	/* mount messages for each tape	                */
char tmdrive[CMLEN];	/* Flag for remount of TM tape			*/
char out[MAXIMG][CMLEN]; /* the output files created			*/

struct PARBLK vblock;	/* TAE parameter block				*/

int main
(void)
{
long bands[MAX_BANDS+1];/* band specification				*/
long msgtim;		/* elapsed time between display messages	*/
long nbands;		/* number of bands				*/
long nquads;		/* number of quads				*/
long quads[4];		/* quad specification				*/
long status;		/* function return status code			*/
long wcount;		/* number of window values specified		*/
long window[4];	        /* window to process				*/
long lp;		/* Flag for line printer output			*/
long file;		/* Flag for file output				*/
long zero = 0;		/* Constant zero				*/
long fatal = -1;	/* Fatal error code				*/
long nonfatal = 1;	/* Non-fatal error code				*/

char imshape[1];	/* shape of the output image
				w - whole image
                                q - single quad
				u - upper half (quads 1 and 2)
				b - bottom half (quads 3 and 4)
				l - left half (quads 1 and 3)
				r - right half (quads 2 and 4)		  */

static char minmax[4];			 /* min max flag		  */
static char mflag[6];	        	 /* mosaicking flag		  */
static char hosout[MAX_QUADS][CMLEN];	 /* buffer storage for hosout	  */
static char taeout[MAX_QUADS][CMLEN];	 /* buffer storage for taeout	  */
static char annot_host[MAX_QUADS][CMLEN];/* annotation host name(s)	  */
static char head_host[MAX_QUADS][CMLEN]; /* buffer storage for head_host  */
static char trail_host[MAX_QUADS][CMLEN];/* buffer storage for trail_host */
static char prtfile[CMLEN];		 /* Printer working file	  */
static char filename[CMLEN];		 /* Output print file name	  */

/*
  Signals to clean_up function that the print file, las file and tape
  have not been opened yet.
*/
print_flag = FALSE;
tape_flag  = FALSE;
assoc_flag = FALSE;
las_flag   = 0;

/*
  Get user-specified parameters and global variable values from TAE
*/
getpar(hosout,taeout,bands,&msgtim,&nbands,&nquads,quads,minmax,annot_host,
	head_host,trail_host,imshape,window,&wcount,mflag);

/*
  Get the working output print file
*/
strcpy(prtfile,"tipsp");
status = c_setprt(&vblock,&term,&file,&lp,prtfile,filename);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","tipsp-fatal",&fatal);

/*
  Open the temporary print file if going to the line printer or a file
*/
if (lp || file)
    {
    if ((out_fp = fopen(prtfile,"a+")) == NULL)
	{
	c_errmsg("Error opening the print file","tipsp-open",&nonfatal);
	clean_up(&zero,TIPS_PRT);
	}
    print_flag = TRUE;

    /*
      Hold onto the print host file name so clean up can be done when an
      error occurs
    */
    strcpy(out[out_count],prtfile);
    out_count++;
    }

/*
  Process the image
*/
if (strcmp(mflag,"ASSOC") == 0)
    proc_assoc(bands,&nbands,quads,&nquads,annot_host[0],
	       head_host[0],trail_host[0],window);
else if ((strcmp(mflag,"YES") == 0) || (wcount != 0))
    proc_window(hosout,bands,&nbands,quads,&nquads,annot_host[0],
		head_host[0],trail_host[0],window,imshape,minmax,&msgtim);
else
    separate(hosout,bands,&nbands,quads,&nquads,annot_host,head_host,trail_host,
	     minmax,&msgtim);

if (strcmp(mflag,"ASSOC") != 0)
    {
    for (out_count = 0; out_count < nquads; out_count++)
	strcpy(out[out_count],taeout[out_count]);
    }

if (hdsk)
    {
    strcpy(out[out_count], head_host[0]);
    out_count++;
    }

if (andsk)
    {
    strcpy(out[out_count], annot_host[0]);
    out_count++;
    }

if (tdsk)
    {
    strcpy(out[out_count], trail_host[0]);
    out_count++;
    }

if (file)
    strcpy(out[out_count],filename);


/*
  Write a completion message to the user
*/
c_complt("CCTTIPSP",out,&out_count);

return EXIT_SUCCESS;

}
