/******************************************************************************
FUNCTION:	UPDATE_DDR


PURPOSE:	Updates the DDR information for an image

ALGORITHM 
  Get the satellite mission number
  Get the hours, minutes, and seconds of observation
  Get the day, month, and year of acquisition
  Get the DDR information from the image file
  Initialize the validity flags
  Update the values that are known
  Find the projection coordinates for the corners
  Update the Band DDR information
      Get the Band DDR information from the image
      Update the Band DDR information
      Update the direction of the pass
      Update the date and time
  Put the updated DDR information to the image
  Put the updated Band DDR information to the image

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "proj.h"
#include "tm.h"
#include "protos.h"

#define SL  0
#define SS  1
#define NL  2
#define NS  3

void update_ddr
(
    char *hname,		/* Host file name				*/
    long window[],		/* Input window					*/
    long *totbnd,		/* Total number of bands to process		*/
    char *scene_id,		/* Scene id					*/
    char *maprj,		/* Map projection identifier			*/
    char *direction,	/* Orbital direction				*/
    long *path,		/* WRS path number				*/
    long *utmzone,		/* UTM zone number				*/
    double fcproj[],	/* Full scene center in projection coordinates	*/
    double fcenll[],	/* Full scene center latitude/longitude degrees	*/
    double fcenls[],	/* Full scene center line/sample		*/
    double *angle,		/* Angle of rotation converted into radians	*/
    char *date		/* acquisition date				*/
)
{
struct DDR ddr;		/* DDR structure				*/
struct BDDR bddr[7];	/* Band DDR structure				*/

long i;			/* Looping variable				*/
long band;		/* Band looping variable			*/
long status;		/* Status variable				*/
long line_samp[2];	/* Line/Sample corner coordinates		*/
long fatal = -1;	/* Fatal error code				*/

char satid[2];		/* Satellite id				*/
char day[3];		/* Day of exposure			*/
char month[4];		/* Month of exposure			*/
char year[3];		/* Year of exposure			*/
char hrmn[5];		/* Hour and minute of exposure		*/
char sec[3];		/* Seconds of exposure			*/

/*
  Get the satellite mission number
*/
strncpy(satid,&scene_id[2],1);
satid[1] = '\0';

/*
  Get the hours, minutes, and seconds of observation
*/
strncpy(hrmn,&scene_id[8],4);
hrmn[4] = '\0';

strncpy(sec,&scene_id[12],2);
sec[2] = '\0';

/*
  Get the day, month, and year of acquisition
*/
strncpy(day,&date[0],2);
day[2] = '\0';

strncpy(month,&date[2],3);
month[3] = '\0';

strncpy(year,&date[5],2);
year[2] = '\0';

/*
  Get the DDR information from the image file
*/
status = c_getddr(hname,&ddr);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","tipsp-fatal",&fatal);

/*
  Initialize the validity flags
*/
for (i = 0; i < 7; i++)
    ddr.valid[i] = VALID;

ddr.master_line   = window[0];
ddr.master_sample = window[1];

/*
  Update the values that are known
*/
if (strcmp(maprj,"SOM") == 0)
    {
    somprm(path,ddr.proj_coef);
    ddr.proj_code = SOM;
    ddr.zone_code = 62;
    }
else
    {
    utmprm(utmzone,&fcenll[0],ddr.proj_coef);
    ddr.proj_code = TM;
    ddr.zone_code = 62;
/*
    ddr.proj_code = UTM;
    ddr.zone_code = &utmzone;
*/
    }

ddr.datum_code = 4;
strcpy(ddr.proj_units,"meters");
ddr.pdist_y = 28.5;
ddr.pdist_x = 28.5;

/*
  Find the projection coordinates for the corners
*/
line_samp[0] = window[SL];
line_samp[1] = window[SS];
ls2proj(angle,&ddr.proj_code,fcproj,fcenls,ddr.upleft,line_samp);

line_samp[0] = window[SL] + window[NL] - 1;
line_samp[1] = window[SS];
ls2proj(angle,&ddr.proj_code,fcproj,fcenls,ddr.loleft,line_samp);

line_samp[0] = window[SL];
line_samp[1] = window[SS] + window[NS] - 1;
ls2proj(angle,&ddr.proj_code,fcproj,fcenls,ddr.upright,line_samp);

line_samp[0] = window[SL] + window[NL] - 1;
line_samp[1] = window[SS] + window[NS] - 1;
ls2proj(angle,&ddr.proj_code,fcproj,fcenls,ddr.loright,line_samp);

/*
  Update the Band DDR information
*/
for (band = 1; band <= *totbnd; band++)
    {
    /*
      Get the Band DDR information from the image
    */
    c_getbdr(hname,&bddr[band-1],&band);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","tipsp-fatal",&fatal);

    /*
      Update the Band DDR information
    */
    sprintf(bddr[band-1].source,"landsat %s",satid);
    strcpy(bddr[band-1].instrument,"tm");

    /*
      Update the direction of the pass
    */
    strcpy(bddr[band-1].direction,direction);

    /*
      Update the date and time
    */
    sprintf(bddr[band-1].date,"%s-%s-%s",day,month,year);
    sprintf(bddr[band-1].time,"%s:%s",hrmn,sec);
    }

/*
  Put the updated DDR information to the image
*/
c_putddr(hname,&ddr);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","tipsp-fatal",&fatal);

/*
  Put the updated Band DDR information to the image
*/
for (band = 0; band < *totbnd; band++)
    c_putbdr(hname,&bddr[band]);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","tipsp-fatal",&fatal);

return;
}
