/******************************************************************************
FUNCTION:	WINBIL

PURPOSE:	Processing loop for band interleaved by line images that are 
		being mosaicked and/or windowed.

ALGORITHM 
  Initial processing for each quad
  Keep track of which quadrants were intersected
  Set up the output image bands to process
  Set up number of lines for the display message
  Open the output image file if it is the first time through the loop,
  otherwise reopen the image file
      Display the size of the output image
  Set the image sequential open LAS flag
  Null terminat the file descriptor array
  Allocate I/O buffer
  Put output image in a processing gropu
  Read in the image file for the current quad
  Close all image files in the group
  Process the trailer record

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "protos.h"

void winbil
(
    long *unit,		/* Tape unit number				*/
    char *hname,		/* host file name of output image    		*/
    long *count,		/* Loop count					*/
    long *nl,		/* Number of lines in the quadrant		*/
    long *ns,		/* Number of samples in the quadrant		*/
    long *nbands,		/* Number of bands being processed		*/
    long bands[],		/* Band specification		      	 	*/
    long window[],		/* Window to be processed			*/
    long quads[],		/* Quadrant numbers to process			*/
    char *imshape,		/* shape of the output image
				 w - whole image
                                 q - single quad
				 u - upper half (quads 1 and 2)
				 b - bottom half (quads 3 and 4)
				 l - left half (quads 1 and 3)
				 r - right half (quads 2 and 4)		*/
    long *tape_rec,		/* Current tape record          		*/
    long *lastns,		/* Number of samples written to the last line	*/
    long quads_used[],	/* The quadrants actually used			*/
    long *nquads_used,	/* Number of quads used				*/
    double outmin[],	/* Min values for each output band		*/
    double outmax[],	/* Max values for each output band		*/
    char *minmax,		/* Min max flag					*/
    long *msgtim,		/* Number of seconds between processing messages*/
    char *text		/* Text for processing message		    	*/
)
{
extern unsigned char *buffer;	/* I/O buffer				*/
extern struct FDESC * fdesc[2];		/* image file descriptors 		*/
extern struct GDESC * gdesc;		/* group file descriptor		*/
extern short las_flag;		/* Flag for las open			*/
extern long offset[MAXBND+1];	/* offsets into output buffer 		*/

				/* current disk record number		*/
static long disk_rec[MAX_QUADS] = {1,1,1,1};
				/* disk record to start writing         */
static long disk_startrec[MAX_QUADS] = {1,1,1,1};
				/* tape record to start reading         */
static long tape_startrec[MAX_QUADS] = {2,2,2,2};
static long opt[3]={0,0,0};	/* optional parameter array for EOPENS  */

long i;			/* Loop index					*/
long status;		/* Status indicator				*/
long access;		/* Open access for the image file		*/
long bufsiz;		/* Output buffer size				*/
long numlin;		/* Number of output lines			*/
long numsamps;		/* Number of output samples			*/
long tape_lastrec;	/* Last tape record to read             	*/
long one = 1;		/* One						*/
long zero = 0;		/* Zero						*/
long nonfatal = 1;	/* Non-fatal error code				*/
long format = BIL;	/* Image data format  (BSQ or BIL)		*/
long flag = ALLCAT;	/* Display message flag--one band at a time	*/
long dtype = EBYTE;	/* Data type of the image (always BYTE)		*/
long obands[MAX_BANDS+1]; /* Output image bands to process		*/

char mesg[ERRLEN];	/* Informational message			*/

/*
  Initial processing for each quad
*/
initquad(window,nl,count,imshape,quads,tape_startrec,&tape_lastrec,disk_rec,
	 disk_startrec,&format);

/*
  Keep track of which quadrants were intersected
*/
quads_used[*nquads_used] = quads[*count];
(*nquads_used)++;

/*
  Set up the output image bands to process
*/
for (i = 0; i < *nbands; i++)
    obands[i] = i + 1;
obands[*nbands] = 0; 

/*
  Set up number of lines for the display message
*/
c_inittm("CCTTIPSP","processed","line","bands",text,&window[2],nbands,
	 msgtim,&flag);

/*
  Open the output image file if it is the first time through the loop,
  otherwise reopen the image file
*/
if (*count == 0)
    {
    access = IWRITE;
    status = c_eopens(&fdesc[0],hname,nbands,&one,&one,&window[2],&window[3],
		    offset,&access,&dtype,&zero,opt);
    status = c_eclose(&fdesc[0]);

    if (*nl < window[2])
       numlin = *nl;
    else
       numlin = window[2];

    if (*ns < window[3])
       numsamps = *ns;
    else
       numsamps = window[3];

    access = IUPDATE;
    status = c_eopens(&fdesc[0],hname,obands,&one,&one,&numlin,&numsamps,
		    offset,&access,&dtype,&zero,opt);

    /* 
      Display the size of the output image
    */
    sprintf(mesg,"Size of output image is %d x %d",window[2],window[3]);
    c_errmsg(mesg,"informational",&nonfatal);
    }
else
    {
    access = IUPDATE;
    numlin = window[2] - disk_startrec[*count] + 1;
    numsamps = window[3];
    if (quads[*count] == 2)
       {
       if (numlin > *nl)
          numlin = *nl - disk_startrec[*count] + 1;
       }

    status = c_eopens(&fdesc[0],hname,obands,&disk_startrec[*count],&one,
		      &numlin,&numsamps,offset,&access,&dtype,&zero,opt);
    }

/*
  Set the image sequential open LAS flag
*/
las_flag = 1;

/*
  Null terminat the file descriptor array
*/
fdesc[1] = 0;

/*
  Allocate I/O buffer
*/
if ((buffer = (unsigned char *) c_ealloc(fdesc,&bufsiz)) == NULL)
    clean_up(unit,TIPS_ALLOC);

/*
  Put output image in a processing gropu
*/
status = c_egroup(fdesc,&gdesc,buffer,&bufsiz);
if (status != E_SUCC)
    clean_up(unit,TIPS_GROUP);

/*
  Read in the image file for the current quad
*/
status = read_wbil(unit,count,window,bands,quads,ns,lastns,disk_rec,
		   disk_startrec,tape_rec,tape_startrec,&tape_lastrec,imshape,
		   outmin,outmax,minmax);
if (status != E_SUCC)
    clean_up(unit,status);

/*
  Close all image files in the group
*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing image group -- Image data saved","tipsp-close",
	    &nonfatal);

las_flag = 0;

/*
  Process the trailer record
*/
status = get_trail(unit,&zero);
if (status != E_SUCC)
    clean_up(unit,status);

return;
}
