/******************************************************************************
FUNCTION:	WINBSQ

PURPOSE:	Processing loop for band sequential images that are being
		mosaicked and/or windowed.

ALGORITHM 
  Initial processing for each quad
  Keep track of which quadrants were intersected
 Set up number of lines for the display message
  Open the output image file if it is the first time through the loop,
  otherwise reopen the image file
      Display the size of the output image
  Set the image random open LAS flag (2)
  Process each band
      Read in the image file for the current band
      Process the trailer record
      Position tape to next specified input band
	      Skip to the calibration record
	      Process the calibration record
	  Skip over file descriptor record of the image file
  Close the output image

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "protos.h"

void winbsq
(
    long *unit,		/* Tape unit number				*/
    char *hname,		/* host file name of output image		*/
    long *count,		/* Loop count					*/
    long *nl,		/* Number of lines in the quadrant		*/
    long *ns,		/* Number of samples in the quadrant		*/
    long window[],		/* Window to be processed			*/
    long bands[],		/* Band specification		   		*/
    long *nbands,		/* Number of bands being processed		*/
    long quads[],		/* Quadrant numbers to process			*/
    long quads_used[],	/* The quadrants actually used			*/
    long *nquads_used,	/* Number of quads used				*/
    long *lastns,		/* Number of samples written to the last line	*/
    long *tape_rec,		/* Current tape record                  	*/
    char *imshape, 		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)   */
    double outmin[],	 /* Array of min values for each output band	*/
    double outmax[],	/* Array of max values for each output band	*/
    char *minmax,		/* Min max flag			      	      	*/
    long *msgtim,		/* Number of seconds between processing message */
    char *text		/* Text for processing message			*/
)
{
extern struct FDESC * fdesc[2];	/* image file descriptors for image I/O 	*/
extern long curcat;	/* Current band number       	        	*/
extern short hdsk;	/* header file flag				*/
extern short las_flag;	/* Flag for las open				*/

				/* current disk record number	 	*/
static long disk_rec[MAX_QUADS] = {1,1,1,1};
				/* disk record to start writing  	*/
static long disk_startrec[MAX_QUADS] = {1,1,1,1};
				/* tape record to start reading  	*/
static long tape_startrec[MAX_QUADS] = {2,2,2,2};
static long opt[3]={0,0,0};	/* optional parameter arraly for EOPENR	*/

long j;			/* Loop index				  	*/
long nbs;		/* Number of bands to skip  		  	*/
long skip;		/* Number of files to skip on tape	  	*/
long numlin;		/* Number of output lines			*/
long tape_lastrec;	/* Last tape record to read               	*/
long access;		/* Open access for the imag file		*/
long status;		/* Status indicator			  	*/
long one = 1;		/* Constant one					*/
long nonfatal = 1;	/* Non-fatal error code				*/
long format = BSQ;	/* Image data format (BSQ or BIL)	  	*/
long dtype = EBYTE;	/* Data type of image (always BYTE)		*/
long flag = ONECAT;	/* Display message flag--one band at a time	*/
long mode = HANDLE;	/* Tape I/O outputs error mesages          	*/

char mesg[80];		/* Informational message			*/

/*
  Initial processing for each quad
*/
initquad (window,nl,count,imshape,quads,tape_startrec,&tape_lastrec,
	  disk_rec,disk_startrec,&format);

/*
  Keep track of which quadrants were intersected
*/
quads_used[*nquads_used] = quads[*count];
(*nquads_used)++;

/*
 Set up number of lines for the display message
*/
c_inittm("CCTTIPSP","processed","line","band",text,&window[2],nbands,
	 msgtim,&flag);

/*
  Open the output image file if it is the first time through the loop,
  otherwise reopen the image file
*/
if (*count == 0)
    {
    access = IWRITE;
    status = c_eopenr(&fdesc[0],hname,nbands,&one,&one,&window[2],&window[3],
		     &access,&dtype,opt,&one);

    /*
      Display the size of the output image
    */
    sprintf(mesg,"Size of output image is %d x %d",window[2],window[3]);
    c_errmsg(mesg,"informational",&nonfatal);
    }
else
    {
    access = IUPDATE;
    numlin = window[2] - disk_startrec[*count] + 1;
    status = c_eopenr(&fdesc[0],hname,nbands,&disk_startrec[*count],&one,
		      &numlin,&window[3],&access,&dtype,opt,&one);
    }

if (status != E_SUCC)
    clean_up(unit,status);

/*
  Set the image random open LAS flag (2)
*/
las_flag = 2;

/*
  Process each band
*/
for (j = 0, curcat = 0; j < *nbands; j++)
    {
    curcat++;

    /*
      Read in the image file for the current band
    */
    status = read_win(unit,count,window,&curcat,quads,ns,lastns,disk_rec,
		      disk_startrec,tape_rec,tape_startrec,&tape_lastrec,
		      imshape,outmax,outmin,minmax);
    if (status != E_SUCC)
	clean_up(unit,status);

    /*
      Process the trailer record
    */
    status = get_trail(unit,&j);
    if (status != E_SUCC)
	clean_up(unit,status);

    /*
      Position tape to next specified input band
    */
    if ((j + 1) < *nbands)
	{
        if ((bands[j] + 1) == bands[j + 1])
	    {
	    if (hdsk)
		skip = 0;
	    else
                skip = 1;
	    }
	else 
            {
 	    printf ("\n");
  	    c_errmsg("Skipping to next input band","tipsp-skip",&nonfatal);
   	    nbs = (bands[j + 1] - bands[j]) - 1;
	    if (hdsk)
		skip = 3 * nbs;
	    else
  	        skip = 1 + (3 * nbs);
	    }

	status = c_tpfsf(unit,&skip,&mode);
	if (status != TAPE_SUCC)
		clean_up(unit,TIPS_READ);

	if (hdsk)
	    {
	    /*
	      Skip to the calibration record
	    */
            skip = 3;
            status = c_tpfsr(unit,&skip,&mode);

	    /*
	      Process the calibration record
	    */
	    status = get_calib(unit,bands,&format);
	    if (status != E_SUCC)
		clean_up(unit,status);
	    }

	/*
	  Skip over file descriptor record of the image file
	*/
        skip = 1;
        status = c_tpfsr(unit,&skip,&mode);
	if (status != TAPE_SUCC)
	    clean_up(unit,TIPS_READ);

	*tape_rec = 2;
	}
    }

/*
  Close the output image
*/
status = c_eclose(&fdesc[0]);
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered--Image data saved","tipsp-warn",
	     &nonfatal);

las_flag = 0;

return;
}
