/******************************************************************************
FUNCTION:	WRITE_PARM

PURPOSE:	Write the input parameters to a disk file for printing and
		set the TAE globals.

ALGORITHM 
  Get the day, month, and year of acquisition
  Write parameter information to the terminal
  Write parameter information to a file
      Write out the information
      Close the print file
  Set the TAE globals
  Convert the acquisition date from the form ddmmmyy to the form mm/dd/yy
  Add the appropriate characters to the scene ID -- the EROS scene id has the
  form YmddddhhmmsX0 where m is the mission number (4 or 5), ddd is the number
  of days since launch at time of observation, hh is the GMT hour at 
  observation, mm is the GMT minute at observation, and s is the GMT tens of
  seconds at time of observation.  The first character (Y) and the last two
  characters (X0) are constant.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"     
#include "tapeio.h"
#include "util.h"
#include "protos.h"

void write_parm
(
    char *date,		/* acquisition date				*/
    char *lat_long,		/* lat long at scene center			*/
    char *path_row,		/* WRS path row					*/
    char *scene_id,		/* scene id					*/
    char *source,		/* data source					*/
    char *resamp,		/* resampling technique				*/
    char *quality,		/* overall geometric quality code		*/
    long *line_center,	/* line number at scene center			*/
    long *samp_center,	/* sample number at scene center		*/
    long window[],		/* window value					*/
    long *nquads,		/* number of quads				*/
    long quads[],		/* quad specification				*/
    char *sceneid		/* scene id read from text record		*/
)
{
extern struct PARBLK vblock; /* TAE parameter block			*/
extern short print_flag;     /* print flag				*/
extern long  term;	     /* Flag for terminal output		*/
extern FILE *out_fp;	     /* File pointer to output print file	*/

			     /* months of the year			*/
static char monthlst[12][4] = 
     {"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};

char day[3];		/* Day of exposure				*/
char month[4];		/* Month of exposure				*/
char year[3];		/* Year of exposure				*/
char newdate[9];	/* new format of date				*/
char newscene[14];	/* new format of scene id			*/
char *strptr[1];	/* array of pointers to string variables	*/
char text[CMLEN];	/* Text for terminal output			*/

long i;			/* loop index					*/
long zero = 0;		/* zero constant				*/

/*
  Get the day, month, and year of acquisition
*/
strncpy(day,&date[0],2);
day[2] = '\0';

strncpy(month,&date[2],3);
month[3] = '\0';

strncpy(year,&date[5],2);
year[2] = '\0';

/*
  Write parameter information to the terminal
*/
if (term)
    {
    sprintf(text,"     Scene ID                         %s",scene_id);
    c_pterm(text,&zero);
     
    sprintf(text,"     WRS Path/Row                     %s",path_row);
    c_pterm(text,&zero);

    sprintf(text,"     Image Scene Center (Lat/Long)    %s",lat_long);
    c_pterm(text,&zero);

    sprintf(text,"     Image Scene Center (Line)        %d",*line_center);
    c_pterm(text,&zero);

    sprintf(text,"     Image Scene Center (Samp)        %d",*samp_center);
    c_pterm(text,&zero);

    sprintf(text,"     Acquisition Date                 %s-%s-%s",
	    day,month,year);
    c_pterm(text,&zero);

    sprintf(text,"     Data Source                      %s",source);
    c_pterm(text,&zero);
    }

/*
  Write parameter information to a file
*/
if (print_flag)
    {
    /*
      Write out the information
    */
    fprintf(out_fp,"     Scene ID                         ");
    fprintf(out_fp,"%s\n",scene_id);
     
    fprintf(out_fp,"     WRS Path/Row                     ");
    fprintf(out_fp,"%s\n",path_row);

    fprintf(out_fp,"     Image Scene Center (Lat/Long)    ");
    fprintf(out_fp,"%s\n",lat_long);

    fprintf(out_fp,"     Image Scene Center (Line)        ");
    fprintf(out_fp,"%d\n",*line_center);

    fprintf(out_fp,"     Image Scene Center (Samp)        ");
    fprintf(out_fp,"%d\n",*samp_center);

    fprintf(out_fp,"     Acquisition Date                 ");
    fprintf(out_fp,"%s-%s-%s\n",day,month,year);

    fprintf(out_fp,"     Data Source                      ");
    fprintf(out_fp,"%s\n",source);

    /*
      Close the print file
    */
    fclose(out_fp);
    }

/*
  Set the TAE globals
*/
q_intg(&vblock,"TMWIND",4, window,P_UPDATE);
q_intg(&vblock,"TMQUADS",*nquads, quads,P_UPDATE);

strptr[0] = path_row;
q_string(&vblock,"TMPTHRW",1,strptr,P_UPDATE);

/*
  Convert the acquisition date from the form ddmmmyy to the form mm/dd/yy
*/
for (i = 0; i < 12; i++)
    {
    if (strcmp(monthlst[i],month) == 0)
	{
	sprintf(newdate,"%d/%s/%s",i+1,day,year);
	break;
	}
    }

strptr[0] = newdate;
q_string(&vblock,"TMACQ",1,strptr,P_UPDATE);

/*
  Add the appropriate characters to the scene ID -- the EROS scene id has the
  form YmddddhhmmsX0 where m is the mission number (4 or 5), ddd is the number
  of days since launch at time of observation, hh is the GMT hour at 
  observation, mm is the GMT minute at observation, and s is the GMT tens of
  seconds at time of observation.  The first character (Y) and the last two
  characters (X0) are constant.
*/
sprintf(newscene,"%s%s%s","Y",sceneid,"X0");
strptr[0] = newscene;
q_string(&vblock,"TMSCENE",1,strptr,P_UPDATE);

if (resamp[0] == 'C')
    strcpy(resamp,"CUBIC CONVOLUTION");
else if (resamp[0] == 'N')
    strcpy(resamp,"NEAREST NEIGHBOR");

strptr[0] = resamp;
q_string(&vblock,"TMRESAMP",1,strptr,P_UPDATE);

if (quality[0] == 'A')
    strcpy(quality,"ACCEPTABLE");
else if (quality[0] == 'G') 
    strcpy(quality,"GOOD");
else if (quality[0] == 'E')
    strcpy(quality,"EXCELLENT");
strptr[0] = quality;
q_string(&vblock,"TMQUAL",1,strptr,P_UPDATE);
q_out (&vblock);

return;
}
