/****************************************************************************
FILENAME:  cib2las.c

FUNCTION(S): main()
             calc_zone()
             cib_print() (debugging only)

PURPOSE:  Ingest specified portions of a Controlled Image Base (CIB) image 
          and write out to a LAS image

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB          Original Development
  1.1      A. Shimitz      10/00           CSB          Added tilenum parameter
                                                        being passed to
                                                        read_toc. Also prevent
                                                        it from calculating the
                                                        zone yet if the user
                                                        specified any boundary
                                                        rectangles (CR 4215)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:    LAS

ALGORITHM DESCRIPTION:
    read in TAE parameters
    if CIB image on remote CD
        mount CD
    convert output image name to a host name
    Read in CIB image Table of Contents file (TOC) file
    Calculate the range of rows and columns for this CIB image (min & max)    
    Create a list of pointers to frame file records
    Set all elements of list to NULL
    set list of pointers to the appropriate record
    write out CIB image
    display completion message

ALGORITHM REFERENCES:  none
******************************************************************************/
#include<string.h>
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"limits.h"  /* for LONG_MAX */
#include"worgen.h" 
#include"las.h"
#include"util.h"
#include"cproj.h"
#include"locate_frame.h"
#include"cib2las.h"
#include"cib2lasout.h"
#include"mount_cd.h"
#include"update_ddr.h"



static char calc_zone 
(
    double latitude /* I: Latitude of point to find zone for */
);

void main
(
    void
)
{
    char  taeout[1][CMLEN];      /* tae formatted output image name         */
    char  hosout[1][CMLEN];      /* host formatted output image name        */
    char  comment[CMLEN]="";     /* comment from user to send to computer
                                    operator with instructions on which
                                    CD to install (if necessary)            */
    char  device[CMLEN]="";      /* CD device                               */
    char  drive[CMLEN];          /* LAS drive name                          */ 
    char  zone;
    int   mounted=0;             /* CD mounted flag                         */
    int   framecount=0;          /* number of frame files in volume         */
    int   status;                /* return value from cib2lasout            */
    int   i,j,k;
    long  std_out = 0;           /* standard out                            */
    long  one = 1;
    long  maxrow=0;              /* maximum row number in zone              */
    long  minrow=LONG_MAX;       /* minimum row number in zone              */ 
    long  maxcolumn=0;           /* maximum column number in volume         */
    long  mincolumn=LONG_MAX;    /* minimum column in volume                */
    long  framerow;              /* variables used when setting up list     */
    long  framecol;              /* of frame file pointers to appropriate   */
    long  framenumber;           /* frame files                             */
    float latitude;              /* used to determine CIB zone images 
                                    requested is in.                        */
    FRAMEFILE *ffiles = NULL,    /* a list of framefiles holding 
                                    infomation on all frame files in 
                                    CIB volume                              */
              **matrix;          /* a array of pointers to frame files 
                                    that is used to locate them into 
                                    rows and columns as assigned in zone    */
    struct PARBLK vblk;
    CIB_DATA  cib_dim;               /* cib_dimensions: struct holding CIB 
                                    volume information                      */
    INPARAMS input;              /* input parameters: struct holding user
                                    input parameters                        */

    /* initialize the zone
    ----------------------*/
    zone = ' ';

    /* Initialize tae block
    -----------------------*/
    c_inlas (&vblk, &std_out);

    /* get parameters from TAE
    --------------------------*/    
    if(get_par(&vblk,taeout,comment,&input)!= E_SUCC)
        c_lasmsg(TRUE,"cib2las-tae",
            "An error occurred retrieving TAE parameters");

    /* if the data is on a CD, mount the CD
    ---------------------------------------*/
    if(strlen(comment) > 0)
    {
        if (mount_cd(&vblk,comment,drive,device,&mounted) != E_SUCC)
            c_lasmsg(TRUE,"cib2las-fatal","Fatal error encountered");    
        if (mounted == FALSE)
            c_lasmsg(TRUE,"cib2las-cd","CD not mounted");
    }
    
    /* If user included extension already, do not append
    ----------------------------------------------------*/
    if(strstr(taeout[0],";img") == NULL)
        strncat(taeout[0],";img",4);
   
    if(c_cmhost(taeout[0],hosout[0]) != E_SUCC)
    {  
        c_lasmsg(TRUE,"cib2las-out",
            "Error converting LAS output name to host output name");
    }
    
    /* find Latitude based on a point just above the midpoint of
       the operator specified window.  don't set the zone yet if
       the user specified boundary rectangles (TILENUM) - the zone
       will be determined in read_toc based on the first frame file
       found that is inside one of the specified boundary rects.
    --------------------------------------------------------------*/
    if((input.type != DEFAULT) && (input.tilenum[0] == -1))
    {
        if(strcmp(input.units,"PRO") == 0)
        { 
            if(input.type == COORS4)
                latitude = ( ((input.nw_y - input.se_y) / 1.99 +
                    input.se_y ) / EARTH_RADIUS) * R2D;
            else
                latitude = (input.nw_y / EARTH_RADIUS) * R2D;
        }
        else
        {
            if(input.type == COORS4)
                latitude = (input.nw_lat - input.se_lat) / 1.99 + input.se_lat;
            else
                latitude = input.nw_lat;
        }
        zone = calc_zone(latitude);

        /* If CIB zone is a polar zone ('9' and 'J') exit
        --------------------------------------------------*/
        if(zone == '9' || zone == 'J')
        {
            c_lasmsg(TRUE,
                "cib2las-data","CIB2LAS does not ingest polar zones");
        }
    }
   
    /* read in Table of contents, filling 
       f_files with frame file infomation 
    --------------------------------------*/
    ffiles =  read_toc(input.indir,device,input.tilenum,&zone,&framecount);
    if(framecount < 1)
    {
        if(mounted == TRUE)
            unmount_cd(&vblk,drive,&mounted);
        c_lasmsg(TRUE,"cib2las-toc","No frame files found");    
    }
    if(ffiles == NULL)
    {
        if(mounted == TRUE)
            unmount_cd(&vblk,drive,&mounted);
        c_lasmsg(TRUE,"cib2las-fatal","Fatal error occurred");
    }
    
    /* find minimum and maximum rows and columns in CIB volume 
    -----------------------------------------------------------*/
    for(i = 0; i < framecount; i++)
    {
        /* if file is an overview file or there was an error calculating
           its location in the zone do not include in output image
        ----------------------------------------------------------------*/
        if(ffiles[i].column > 0 )
        {
            if(ffiles[i].column > maxcolumn)
                maxcolumn = ffiles[i].column;
           
            if(ffiles[i].column < mincolumn)
                mincolumn = ffiles[i].column;
            
            if(ffiles[i].row > maxrow)
                maxrow = ffiles[i].row;
            
            if(ffiles[i].row < minrow)
                minrow = ffiles[i].row;
        }
    }

    /* if max and min rows and columns were not modified, no frame
       files were found in the zone.
    ---------------------------------------------------------------*/
    if(maxcolumn == 0 && mincolumn == LONG_MAX &&
       maxrow == 0 && minrow == LONG_MAX)
    {
        if(mounted == TRUE)
            unmount_cd(&vblk,drive,&mounted);
        c_lasmsg(FALSE,"cib2las-toc","No frame files found");
        c_lasmsg(TRUE,"cib2las-fatal","Fatal error occurred");
    }
    
    cib_dim.numcolumns = maxcolumn - mincolumn + 1;
    cib_dim.numrows = maxrow - minrow + 1;
    cib_dim.maxrow = cib_dim.numrows - 1;
    cib_dim.maxcolumn = cib_dim.numcolumns - 1;

    /* create a list of pointers to frame files to 
       use to sort frame files into rows and columns
    -------------------------------------------------*/
    if((matrix = (FRAMEFILE **) malloc (cib_dim.numcolumns * 
        cib_dim.numrows * sizeof(FRAMEFILE *))) == NULL)
    {
        free(ffiles);
        if(mounted == TRUE)
            unmount_cd(&vblk,drive,&mounted);
        c_lasmsg(TRUE,"cib2las-alloc","Error allocating dynamic memory");
    } 

    for( j = cib_dim.numrows - 1 ; j >= 0; j--)
        for( k = 0 ; k < cib_dim.numcolumns; k++)
            matrix[k + (j * cib_dim.numcolumns) ] = NULL;

    
    /* set up pointers in list to appropriate locations
    ---------------------------------------------------*/
    for( i = 0; i < framecount; i++)
    {
        /* if record is a frame file image 
           set appropriate pointer to it
        -----------------------------------*/       
        if(ffiles[i].column != NOTAFRAMEFILE)
        {
            framerow = ffiles[i].row - minrow;
            framecol = ffiles[i].column - mincolumn;
            framenumber = framecol + framerow * cib_dim.numcolumns;
            matrix[framenumber] = &ffiles[i];
        }
    } 

    /* finally, write out the frame files to the las image
    ------------------------------------------------------*/
    status = cib2lasout(&vblk,matrix,&input,&cib_dim,hosout,device);
    if(mounted == TRUE)
        unmount_cd(&vblk,drive,&mounted);
    free(matrix);
    free(ffiles);
    if(status == E_SUCC)
        c_complt("CIB2LAS",hosout,&one);
    else
        c_clean(&vblk,"cib2las",hosout,&one);
}
/*****************************************************************************
function NAME: calc_zone

PURPOSE:   Calculate the CIB zone based on Latitude passed in. 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:


PROJECT:    LAS

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCES:
RETURNS: character representing CIB zone.
******************************************************************************/
char calc_zone 
(
    double latitude /* I: Latitude of point to find zone for */
)
{
    if (latitude > 80)
        return ('9');
    else if (latitude >  76)
        return ('8');
    else if (latitude >  72)
        return ('7');
    else if (latitude >  68)
        return ('6');
    else if (latitude >  64)
        return ('5');
    else if (latitude >  56)
        return ('4');
    else if (latitude >  48)
        return ('3');
    else if (latitude >  32)
        return ('2');
    else if (latitude >   0)
        return ('1');
    else if (latitude > -32)
        return ('A');
    else if (latitude > -48)
        return ('B');
    else if (latitude > -56)
        return ('C');
    else if (latitude > -64)
        return ('D');
    else if (latitude > -68)
        return ('E');
    else if (latitude > -72)
        return ('F');
    else if (latitude > -76)
        return ('G');
    else if (latitude > -80)
        return ('H');
    else 
        return ('J');
}
#ifdef DEBUG


/* Debuging Routine to print out information about the CIB volume 
  
   eg. function call from main:

   cib_print(2,matrix,minrow,mincolumn,&input,&cib_dim);    

-----------------------------------------------------------------------------
option 1:  prints out all boundary rectangles in CIB volume.

    e.g.
    
Boundary Rectangle ID's in CIB volume

0165 0166 0167 0168 0169 0170 0171 0172 0173 0174 0175 0176 0177   rows
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----  ------ 
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005  15  15
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005  14  14
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005  13  13
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005  12  12
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005  11  11
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005  10  10
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   9   9
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   8   8
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   7   7
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   6   6
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   5   5
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   4   4
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   3   3
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   2   2
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   1   1
0002 0002 0002 0002 0002 0002 0005 0005 0005 0005 0005 0005 0005   0   0


-----------------------------------------------------------------------------
option 2: prints out last four digits of filename for all frame files in 
          the CIB volume.
   e.g.

Frame File names in CIB volume

0165 0166 0167 0168 0169 0170 0171 0172 0173 0174 0175 0176 0177   rows
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ----  ------ 
0TR8 0TR9 0TRA 0TRB 0TRC 0TRD 0TRE 0TRF 0TRG 0TRH 0TRJ 0TRK 0TRL  15  15
0RWP 0RWQ 0RWR 0RWS 0RWT 0RWU 0RWV 0RWW 0RWX 0RWY 0RWZ 0RX0 0RX1  14  14
0Q24 0Q25 0Q26 0Q27 0Q28 0Q29 0Q2A 0Q2B 0Q2C 0Q2D 0Q2E 0Q2F 0Q2G  13  13
0N7K 0N7L 0N7M 0N7N 0N7P 0N7Q 0N7R 0N7S 0N7T 0N7U 0N7V 0N7W 0N7X  12  12
0LD0 0LD1 0LD2 0LD3 0LD4 0LD5 0LD6 0LD7 0LD8 0LD9 0LDA 0LDB 0LDC  11  11
0JJF 0JJG 0JJH 0JJJ 0JJK 0JJL 0JJM 0JJN 0JJP 0JJQ 0JJR 0JJS 0JJT  10  10
0GPW 0GPX 0GPY 0GPZ 0GQ0 0GQ1 0GQ2 0GQ3 0GQ4 0GQ5 0GQ6 0GQ7 0GQ8   9   9
0EVB 0EVC 0EVD 0EVE 0EVF 0EVG 0EVH 0EVJ 0EVK 0EVL 0EVM 0EVN 0EVP   8   8
0D0S 0D0T 0D0U 0D0V 0D0W 0D0X 0D0Y 0D0Z 0D10 0D11 0D12 0D13 0D14   7   7
0B67 0B68 0B69 0B6A 0B6B 0B6C 0B6D 0B6E 0B6F 0B6G 0B6H 0B6J 0B6K   6   6
09BN 09BP 09BQ 09BR 09BS 09BT 09BU 09BV 09BW 09BX 09BY 09BZ 09C0   5   5
07H3 07H4 07H5 07H6 07H7 07H8 07H9 07HA 07HB 07HC 07HD 07HE 07HF   4   4
05NJ 05NK 05NL 05NM 05NN 05NP 05NQ 05NR 05NS 05NT 05NU 05NV 05NW   3   3
03TZ 03U0 03U1 03U2 03U3 03U4 03U5 03U6 03U7 03U8 03U9 03UA 03UB   2   2
01ZE 01ZF 01ZG 01ZH 01ZJ 01ZK 01ZL 01ZM 01ZN 01ZP 01ZQ 01ZR 01ZS   1   1
004V 004W 004X 004Y 004Z 0050 0051 0052 0053 0054 0055 0056 0057   0   0


----------------------------------------------------------------------------*/
int cib_print
(
    int type,               /* I: option to print, see above                */
    FRAMEFILE **matrix,     /* I: list containing information on all frame
                                  files                                     */
    long minrow,            /* I: minimum row in zone                       */
    long mincolumn,         /* I: minimum column in zone                    */
    INPARAMS *input_params, /* I: user entered input parameters             */
    CIB_DATA *cib_dim       /* I: information on CIB Volumne contents       */
) 
{
    int  j,k,
          maxcolumns = 25;  /* maximum number of columns to print out */
    char str[7],
          *bstr,
          cstr[6];
    FRAMEFILE * ff;
     
    /* print header for matrix columns
    ----------------------------------*/
    printf("\n\n");
    if(type == 1)
        printf("Boundary Rectangle ID's in CIB volume\n\n");
    else if (type == 2)
        printf("Frame file names (partial) in CIB volume\n\n");
    
    for(k = 0; k < cib_dim->numcolumns; k++)
        if(k < maxcolumns)
            printf("%04d ",k + mincolumn);
    printf("  rows");
    printf("\n");
    for(k = 0; k < cib_dim->numcolumns; k++)
        if(k < maxcolumns)
            printf("---- ");
    printf(" ------ ");
    printf("\n");  
    for(j = cib_dim->numrows-1; j >= 0 ; j--)
    {
        for(k = 0 ; k < cib_dim->numcolumns; k++)
        {
            if(k < maxcolumns)
            {
                if(matrix[k+(j*cib_dim->numcolumns)] != NULL)
                {
                    if(type == 1)
                    {
                        printf("%04ld ",matrix[k+(j*cib_dim->numcolumns)]
                            ->brect_record);
                    }
                    else if (type == 2)
                    {
                        strncpy(str,
                                matrix[k+(j*cib_dim->numcolumns)]->filename,6);
                        bstr = &str[2];
                        strcpy(cstr,bstr);
                        cstr[4]='\0';
                        printf("%s ",cstr);
                    }
                }
            else 
                printf("____ ");
            }
        } 
        printf("%3d %3d\n",j + minrow,j);
    }
    if(cib_dim->numcolumns > maxcolumns)
    {
        printf("%d columns not listed due to screen size\n\n",
            cib_dim->numcolumns-maxcolumns);
    }
    return(E_SUCC);
}
#endif   /* endif DEBUG */

