/****************************************************************************
FILENAME:     cib2lasout.c

FUNCTION(S):  cib2lasout()
              getminmax()
              calc_corners()
              check_window_deg()
              calc_rectangular_coors()
              calc_geometric_coors() 
              check_window_geo() debugging only

PURPOSE:  Write out specified CIB image 

PROGRAM HISTORY:

VERSION    PROGRAMMER    DATE     CODE/CONT       REASON
-------    ----------    ----     ---------       ------
  1.0      B. Gross      5/00      CSB          Original Development
  1.1      A.Shimitz     10/00     CSB          Modified the check_window_deg
                                                routine where it verifies the
                                                specified corner coord.  fall
                                                within the image because it was
                                                checking the given SE lat's
                                                with the image's long's

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:    LAS

ALGORITHM DESCRIPTION:

    find the first non-empty frame file in the first row to use to obtain the
      upper left corner coordinates of the CIB Volume image.
    get frame file's name and path
    get the frame file's zone code
    if the frame file is a polar image,
        raise fatal error and exit, (CIB2LAS doesn't support Polar Zone images)
    if window parameters entered in degrees or packed DMS
       convert them (input window values) to meters 
    open frame file and read coverage section (Lat. & Long.)
    correct the longitude value of the coverage section to be coverage
        for first column (coverage for entire volume including 
            empty frame files)
    correct latitude and longitude to be center pixels based 
        vs. corner pixel based like (CIB)
    read in pixel-pixel resolution from frame file for calculations
    convert upper left corner of CIB image from
           degrees to Projection coordinates in meters
    read source attribute from frame file
    if user specified SS NS SL NL calculate portions of CIB Volume to output
        if specified values exceed input image size
           raise fatal error, and exit
        correct sl,ss to 0 (zero) based vales
        calculate first column of CIB image to output
        calculate last column of CIB image to output
        calculate first row of CIB image to output
        calculate last row of CIB image to output
        calculate the corner coordinates of the specified output image
        calculate the number of lines in the first row
    end if SLSSNLNS
    if user specified COORS4
        if latitude specified > upper left corner of CIB image
            raise fatal error, exit
        if longitude specified < longitude of left edge of CIB volume 
             raise fatal error, exit
        create a dummy ddr containing info on overall CIB image
        call c_wndconv to calculate SL SS NL NS
        calculate corners of CIB image base on CIB image pixels values 
        calculate first column of CIB image to output
        calculate last column of CIB image to output
        calculate first row of CIB image to output
        calculate last row of CIB image to output
        calculate the number of lines in first row to output
    end if COORS4
    if user specified no image parameters (DEFAULT)
        calculate first column of CIB image to output
        calculate last column of CIB image to output
        calculate first row of CIB image to output
        calculate last row of CIB image to output
        calculate corners of CIB image base on CIB image pixels values 
        set starting sample of CIB image to output to 0
        calculate number of samples of CIB image to output 
        set starting line of CIB image to output to
        calculate number of lines of CIB image to output
        set number of lines to output in first row to 1536 (1 frame file)
    endif DEFAULT   
        check number of samples and number of lines against samples and lines
            in CIB image
        calculate output image size
        if output image size > 10Gb (CIBMAXIMGSIZE)
             raise fatal error, free allocated memory, exit
        allocate memory to store a list that contains information on 
            each column to
        set value for first column
        set default values into remaining columns
        set values for last column
        allocate memory for a list of pointers to frame files, used to
            organize frame files into rows and columns
        allocate memory for linebuf, will hold one line of output image        
        create output file
        set current output line to 1 (outline)
        initialize output message routine
        calculate number of rows in output image
        for each row in output image
            calculate actual row in CIB image
            for each column in output image
                if frame file exists for this row and column
                   get frame file path and name from list
                   read frame file image into framebuf[column]
                 else - empty frame file
                   fill framebuf[column] with fill value
                 endif
            end for each col...       
            calculate starting line of row to output
            calculate ending line of row to output
            for row starting line to row ending line
                write out line to output image
                if minmax specified
                    update min/max to include this line
            end for star...
        end for each row...
        stop message routine
        if size of output image not size specified
            raise fatal error, free allocated memory, exit
        look through attributes of corner frame file for datasoure
            if datasource found (paramid==1)
                 get value
        save data in DDR
        free allocated memory

ALGORITHM REFERENCES:  none
******************************************************************************/
#include<stdlib.h> 
#include<ctype.h>
#include<math.h>
#include<float.h>
#include<string.h>
#include"las.h"
#include"util.h"
#include"cproj.h"
#include"diskio.h" /* for c_dkdel */
#include"cib2las.h"
#include"cibframe.h"
#include"cib2lasout.h"
#include"update_ddr.h"
#include"get_par.h"

static char  zone=' '; 
extern long curent; 

static int calc_geometric_coors
(
    double *lat,               /* O: output latitude of point degrees       */
    double *lon,               /* O: output longitude of point degrees      */
    double y,                  /* I: input Y projection coordinate meters   */
    double x                   /* I: input X projection coordinate meters   */
);

static int check_window_deg
(
    INPARAMS *in,     /* I:  Input Parameters                      */
    struct CIBCoverage *corner  /* information on frame file in upper        */
);

static int calc_rectangular_coors
(
    double lat,            /* I: input latitude of point degrees            */
    double lon,            /* I: input longitude of point degrees           */
    double *y,             /* O: output Y projection coordinate meters      */
    double *x              /* O: output X projection coordinate meters      */
);

int cib2lasout
(
    struct PARBLK *vblk,       /* I:  TAE parameter block                   */
    FRAMEFILE **list,          /* I:  List of Frame Files                   */
    INPARAMS *input_params,    /* I:  Input Parameters                      */
    const CIB_DATA *cib_data,  /* I:  Input Dimensions                      */
    char hosout[][CMLEN],      /* I:  host name of output file              */
    const char *device         /* I:  device (remote drive                  */
)
{
    unsigned char **framebuf;  /* buffer for a row of frame files images    */
    unsigned char *linebuf;    /* buffer for one line of the output image   */
    unsigned char  minpixel=255;/* minimum pixel value                      */
    unsigned char  maxpixel=0; /* maximum pixel value                       */  
    char  datasource [15]="";  /* image data source                         */
    char  tempstr[CMLEN];      /* temporary string used to mangle filenames */
    char  ldir[CMLEN];
    char  lfile[50];
    char  filename[CMLEN];
    int   one=1;
    int   i = 0,j = 0;
    int   status;              /* contains return status for some calls     */
    int   col = 0;             /* incremental counter for columns           */
    int   start_row = 0;       /* Starting Frame Row                        */
    int   end_row = 0;         /* Ending Frame Row                          */
    int   filled_rows;         /* number of filled rows in output image     */
    int   minmax_flg;          /* Value of global minmax flag               */
    int   start_column = 0;    /* Starting Frame column                     */
    int   end_column = 0;      /* Ending Frame column                       */
    int   columns = 0;         /* number of horiz. frame files (colums)     */
    int   startline = 0;       /* starting line for frame working on        */
    int   endline = 0;         /* ending line for frame working on          */
    int   srowlines = 0;       /* number of samples in first frame of row   */
    int   linebufloc = 0;      /* location to write to in linebuffer        */
    int   line = 0;            /* line counter                              */
    int   sample = 0;          /* sample counter                            */
    int   cib_row = 0;         /* row number in zone                        */
    int   row = 0;             /* counter for rows                          */
    int   column = 0;          /* counter for columns                       */
    long  odtype = EBYTE;      /* data type for output file                 */
    long  str_length;          /* length of frame file name                 */
    long  acc = IWRITE;        /* access type for output file               */
    long  opt = 0;             /* option (compressed/non-compressed)        */
    long  nline = 1;           /* max number lines written at once          */
    long  ss = 1;              /* starting sample for output image          */
    long  sl = 1;              /* starting line for output image            */
    long  outline;             /* line being written to output file         */
    long  flag = GEN2;         /* flag indicating message output type       */
    long  nl_blk = 1;          /* number of blocks for c_ewrite()           */
    long  bands = 1;           /* bands in images, defined to be 1          */
    long  imglines=0;          /* number of lines in CIB image              */
    long  imgsamples=0;        /* number of samples in CIB image            */
    double image_size;         /* size of output image                      */
    double x,y;                /* temp variable for call to calc
                                  geometric_coors                           */
    FILE  *ffptr;                  /* file pointer for reading frame file 
                                   coordinates coors2, coors4               */
    struct CIBAttribute cibatt;/* structure to hold image attributes        */
    COLUMN *coldata;           /* structure to hold info on a column        */
    struct CIBCoverage         /* information on frame file in upper        */
           corner;             /*     left corner of CIB volume             */
    struct CIBCoverage                            
           image;              /* corner infomation on output image         */

    struct FDESC *fdesc;

    
    /* Since the CIB image row numbering starts at the bottom
       of the image, and we want to start with the top rows,
       calculate cib_row which is the CIB row we want to read from.   
    -------------------------------------------------------------------*/
    cib_row = cib_data->maxrow;

    /* calculate necessary data for determination of which frames to read
    --------------------------------------------------------------------*/
 
    /* read corner coordinates from first frame file found in first row
       of CIB volume, elements in list that are NULL are empty frame files
    ----------------------------------------------------------------------*/
    column=0;
    while(list[column + (cib_row * cib_data->numcolumns)] == NULL)
    {
        column++;
        if(column > cib_data->numcolumns)
        {
            c_lasmsg(FALSE,"cib2las-list","No columns in first row");
            return (E_FAIL);
        }
    }

    /* get zone code from first frame file in first row of list, 
       filename is ffffffvp.ccz where z is the zone code
    ------------------------------------------------------------*/
    zone = toupper(list[column 
        + (cib_row * cib_data->numcolumns)]->filename[11]);

    /* if window parameters entered in degrees or packed DMS
       convert them to meters 
    ---------------------------------------------------------*/
    if(strcmp(input_params->units,"PRO") == 0) 
    {
        if(calc_geometric_coors(&input_params->nw_lat,&input_params->nw_long,
            input_params->nw_y, input_params->nw_x) != E_SUCC)
        {
            c_lasmsg(FALSE,"cib2las-input",
                "Error converting corner coordinates");
            return (E_FAIL);
        }
     
        if(input_params->type == COORS4)
        {
            if(calc_geometric_coors(&input_params->se_lat,
                &input_params->se_long,input_params->se_y,input_params->se_x)
                != E_SUCC)
            {
                c_lasmsg(FALSE,"cib2las-input",
                   "Error converting corner coordinates");
                return (E_FAIL);
            }
        }
    }
   
   
    /* read coverage infomation and attribute infomation 
       found in the first row from first frame file 
    ------------------------------------------------------*/

    /* column now points to first non-empty frame file in first row,
       read in corner frame file to use for output image calculation
    ------------------------------------------------------------------*/
    strcpy(ldir,list[column
        + (cib_row * cib_data->numcolumns)]->dir);
    strcpy(lfile,list[column 
        + (cib_row * cib_data->numcolumns)]->filename);
    sprintf(filename,"%s%s%s",device,ldir,lfile);

    if ((ffptr = fopen(filename,"r")) == NULL)
    {
        /* if unable to open file file, convert directory and 
           file name to lower case and try again.  Do not convert 
           device.  Some sytems convert file and
           directory names to all lower or upper case.
        -----------------------------------------------------------*/
        /* convert directory and filename to lower case 
        -----------------------------------------------*/
        sprintf(tempstr,"%s%s",ldir,lfile);
        str_length = (long) strlen(tempstr);
        c_up2low(tempstr,&str_length);
        sprintf(filename,"%s%s",device,tempstr);
        if ((ffptr = fopen(filename ,"r")) == NULL)
        {
            sprintf(tempstr,"%s%s",ldir,lfile);
            c_low2up(tempstr,&str_length);
            sprintf(filename,"%s%s",device,tempstr);
            if ((ffptr = fopen(filename ,"r")) == NULL)
            {
                /* get original filename to print out
                --------------------------------------*/
                sprintf(filename,"%s%s%s",device,ldir,lfile);                        
                fclose(ffptr);
                c_lasmsg(FALSE,"cib2las-open",
                    "Error opening frame file %s",filename);
                return (E_FAIL);
            }
        }
    }

    /* read Latitude and Longitude infomation on first frame file found
    --------------------------------------------------------------------*/
    if (read_frame_coverage(ffptr,&corner) != E_SUCC)
    {
        fclose(ffptr);
        c_lasmsg(FALSE,"cib2las-read","Error reading frame file %s coverage",
            filename);
        return (E_FAIL);
    }
    else
    {
        /* correct corner Latitude and Longitude for 
           cases where corner frame file isn't present
        ----------------------------------------------*/
        corner.ULLong -= column * SAMPLES_FRAME * corner.LongRes;

        /* correct corner lat and longitude to a pixel centered location
        ----------------------------------------------------------------*/
        corner.ULLat -= (0.5 * corner.LatRes);
        corner.ULLong += (0.5 * corner.LongRes);

        /* convert upper left corner of CIB image from
           degrees to Projection coordinates in meters
        -----------------------------------------------*/
        if(calc_rectangular_coors(corner.ULLat,corner.ULLong,&y,&x) != E_SUCC)
        {
            c_lasmsg(FALSE,"cib2las-coors",
                "Error converting corner coordinates");
            return (E_FAIL);   
        }
        else
        {
            corner.ULy = y;
            corner.ULx = x;
        }

        /* set *.yRes and *.xRes to the resolution
           currently being used (meters)
        -------------------------------------------*/
        corner.yRes = corner.NSRes;      
        corner.xRes = corner.EWRes;
        
        /* set output image geographic resolutions to resolution and 
           projection coordinates resolution to the resolutions of 
           the first frame file (the corner)     
        ---------------------------------------------------------------------*/
        image.LatRes = corner.LatRes;  /* Latitude/vertical interval degrees */
        image.LongRes = corner.LongRes;/* Long./horiz. interval degrees      */
        image.EWRes = corner.EWRes;    /* Latitude/vertical interval meters  */
        image.NSRes = corner.NSRes;    /* Long./horiz. interval meters       */
        
        /* set the current resolution of the output image to meters which
           is the current resolution for the image 
        ------------------------------------------------------------------*/
        image.yRes = corner.yRes;      /* Y resolution used in output image */
        image.xRes = corner.xRes;      /* X resolution used in output image */
    }

 
    
    /* read source infomation on first frame file found
    ---------------------------------------------------*/   
    fseek(ffptr,0l,SEEK_SET);

    if(read_frame_attributes(ffptr,&cibatt) != E_SUCC)
        c_lasmsg(FALSE, "cib2las-source","Error reading image data source");
    
    fclose(ffptr);
     
    /* Based on user inputs, determine portion of image to output */

    /* Calculate SL SS NS NL from user input   
    ----------------------------------------*/
    if(input_params->type == SLSSNLNS || input_params->type ==  SLSS)
    {   
        /* correct ss and sl line to 0 based
        ------------------------------------*/
        input_params->sl--;
        input_params->ss--;
        
        imgsamples =  cib_data->numcolumns * SAMPLES_FRAME;
        imglines = cib_data->numrows * LINES_FRAME;

        if(input_params->type == SLSSNLNS)
        {
            /* check to see if specified lines and samples are
               within input image
            ----------------------------------------------------*/
            if(input_params->nl + input_params->sl > imglines)
            {
                input_params->nl = imglines - input_params->sl;
                c_lasmsg(FALSE,"Informational",
                    "Specified lines exceed CIB image size");
                c_lasmsg(FALSE,"Informational","defaulted to CIB image lines");
            }


            if(input_params->ns + input_params->ss > imgsamples)
            {
                input_params->ns = imgsamples - input_params->ss;
                c_lasmsg(FALSE,"Informational",
                    "Specified samples exceed CIB image size");
                c_lasmsg(FALSE,"Informational",
                         "defaulted to CIB image samples");
            }            
        }
        else  
        {
            /* user did not enter number of lines or number of samples
               so default to include the remainder of the CIB image
            ----------------------------------------------------------*/
            input_params->ns = imgsamples - input_params->ss;
            input_params->nl = imglines - input_params->sl;
        }
    

        /* calculate what portion of the CIB image is to be output
        -----------------------------------------------------------*/  
        start_column = (int) input_params->ss / SAMPLES_FRAME;
        end_column = (int) ((input_params->ss + input_params->ns - 1)
            / SAMPLES_FRAME);
        columns = end_column - start_column + 1;
        start_row = (int) input_params->sl / LINES_FRAME;
        srowlines = LINES_FRAME - (input_params->sl - start_row * LINES_FRAME);
        end_row = (int) ((input_params->sl + input_params->nl - 1) 
            / LINES_FRAME);
        
        /* calculate the corners of the output image 
        --------------------------------------------*/
        if(calc_corners(&corner,input_params,&image) != E_SUCC)
            return(E_FAIL);

    }

    /* USER INPUT:  2 or 4 Window Parameters 
    ----------------------------------------*/
    else if(input_params->type == COORS4 || input_params->type == COORS2)
    {
        /* make sure specified output window within input image
        -------------------------------------------------------*/
        
        /* calculate corner parameters of the CIB image
        -----------------------------------------------*/
        corner.LLy = corner.ULy - 
            ((cib_data->numrows * LINES_FRAME - 1) * corner.yRes);  
        corner.LLx = corner.ULx;                              
        corner.URy = corner.ULy;                               
        corner.URx = corner.ULx + 
            ((cib_data->numcolumns * SAMPLES_FRAME - 1) 
            * corner.xRes);
        corner.LRy = corner.LLy;                              
        corner.LRx = corner.URx;
        
        corner.LRLat = corner.ULLat - 
            ((cib_data->numrows * LINES_FRAME - 1) * corner.LatRes);
        corner.LRLong = corner.ULLong + 
            ((cib_data->numcolumns * SAMPLES_FRAME - 1) * corner.LongRes);
        corner.LLLat = corner.LRLat;
        corner.LLLong = corner.ULLong;
        
        /* check if window outside of CIB input image, reset to
           CIB image limits if necessary
        --------------------------------------------------------*/
        if(check_window_deg(input_params,&corner) != E_SUCC)
                return (E_FAIL);
        
        /* calculate starting sample and starting line 
        add 0.5 for half a pixel dimension - rounding error
        -----------------------------------------------------*/
        input_params->sl = (long)((corner.ULLat - input_params->nw_lat) / 
            image.LatRes + 1.5);
        input_params->ss = (long)((input_params->nw_long - corner.ULLong) / 
            image.LongRes + 1.5);
        
        image.ULLat = corner.ULLat - (input_params->sl * image.LatRes);    
        image.URLat = image.ULLat;                              

        image.ULLong = corner.ULLong + (input_params->ss * image.LongRes); 
        image.LLLong = image.ULLong;                            
      
        /* calculate number of lines and number of samples 
        --------------------------------------------------*/
        input_params->nl = (long) ((image.ULLat - input_params->se_lat) / 
            image.LatRes + 1.5);
        input_params->ns = (long) ((input_params->se_long - image.ULLong) / 
            image.LongRes + 1.5);

        image.LLLat = image.ULLat - (input_params->nl) * image.LatRes;     
        image.LRLat = image.LLLat;                              
              
        image.URLong = image.ULLong + (input_params->ns) * image.LongRes;  
        image.LRLong = image.URLong;                            
               
        /* increment ns and nl to include staring line and sample 
        ---------------------------------------------------------*/   
        input_params->nl++;
        input_params->ns++; 
       
        /* correct to 0 base number system
        -----------------------------------*/
        input_params->sl--;
        input_params->ss--;

        /* calculate the corners of the output image 
        ---------------------------------------------*/
        if(calc_corners(&corner,input_params,&image) != E_SUCC)
            return(E_FAIL);

        /* calculate what portion of the CIB image is to be output
        -----------------------------------------------------------*/  
        start_column = (int) input_params->ss / SAMPLES_FRAME;
        end_column = (int) ((input_params->ss + input_params->ns - 1)
            / SAMPLES_FRAME);
        columns = end_column - start_column + 1;
        
        start_row = (int) input_params->sl / LINES_FRAME;
        srowlines = LINES_FRAME - (input_params->sl - start_row * LINES_FRAME);
        end_row = (int) ((input_params->sl + input_params->nl - 1) 
            / LINES_FRAME);
    }

    /* USER INPUT:  DEFAULT ENTIRE WINDOW
    -------------------------------------*/
    if(input_params->type == DEFAULT)
    {
        start_row = 0;
        end_row = cib_data->maxrow;

        /* setup column data 
        --------------------*/
        start_column = 0;
        end_column = cib_data->maxcolumn;  
        columns =  cib_data->numcolumns;

        /* set up sample and line data
        ------------------------------*/
        input_params->ss = 0;
        input_params->ns = cib_data->numcolumns * SAMPLES_FRAME;
        input_params->sl = 0;
        input_params->nl = cib_data->numrows * LINES_FRAME;
        srowlines = LINES_FRAME;

        /* calculate the corners of the output image 
        --------------------------------------------*/ 
        if(calc_corners(&corner,input_params,&image) != E_SUCC)
            return(E_FAIL);
    } 
    
    /* check to see if a window exists 
    -----------------------------------*/
    if((input_params->sl < 0) || (input_params->ss < 0))
    {
        c_lasmsg(FALSE,"check-window","no window");
        return (E_FAIL);
    }

    if((input_params->nl) < 1 || (input_params->ns < 1))
    {
        c_lasmsg(FALSE,"check-window","no window");
        return (E_FAIL);
    }
    
    /* check image size
    -------------------*/

    image_size = (double)input_params->nl * (double)input_params->ns;
        
    if(image_size > 1e9)
    {
        c_lasmsg(FALSE,"Informational","Specified Image size [ %4.2f Gb ]",
            image_size / 1e9);
    }
    else if(image_size > 1e6)
    {
        c_lasmsg(FALSE,"Informational","Specified Image size [ %5.2f Mb ]",
            image_size / 1e6);
    }
    else 
    {
        c_lasmsg(FALSE,"Informational","Specified Image size [ %4.1f Kb ]",
            image_size / 1e3);
    }
    
    if(image_size > CIBMAXIMGSIZE)
    {
        c_lasmsg(FALSE,"cib2las-size",
            "Specified Image size [%4.2fGb] > max image size [ %4.2f Gb ]",
            image_size/1e9,CIBMAXIMGSIZE/1e9);
        return (E_FAIL);
    } 

    /* create an array of information on each column
    ------------------------------------------------*/
    if((coldata = (COLUMN *)malloc(columns * sizeof(COLUMN))) == NULL)
    {
        c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
        return (E_FAIL);
    } 

    /* set appropriate values for first column to be output
    -------------------------------------------------------*/
    if(input_params->ss > SAMPLES_FRAME)
    {
        coldata[0].startsample = input_params->ss 
            - (int)(input_params->ss / SAMPLES_FRAME) * SAMPLES_FRAME;
    }
    else
    {
        coldata[0].startsample = input_params->ss;
    }
    
    if((coldata[0].startsample + input_params->ns) >= SAMPLES_FRAME)
        coldata[0].endingsample = SAMPLES_FRAME - 1;
    else
        coldata[0].endingsample = coldata[0].startsample + input_params->ns;

    coldata[0].framesamples = coldata[0].endingsample 
        - coldata[0].startsample + 1;

    /* fill remaining columns with default values
    ----------------------------------------------*/
    for (column = 1; column < columns - 1; column++)
    {
        coldata[column].startsample = 0;
        coldata[column].endingsample = SAMPLES_FRAME - 1;
        coldata[column].framesamples = SAMPLES_FRAME;
    }

    /* if more than one column to output,
           calculate data for last column
    --------------------------------------*/
    if(columns > 1)
    {
        /* column samples start at 0
        -----------------------------*/
        coldata[columns - 1].startsample = 0;
        /* number of samples in last column is 
            total number of samples 
                - number of filled columns
                - number of samples in first column
        --------------------------------------------*/
        coldata[columns - 1].framesamples = input_params->ns
            - (columns - 2) * SAMPLES_FRAME - coldata[0].framesamples;
        /* ending sample is the start sample 
               plus the number of samples in this column -1
        ---------------------------------------------------*/
        coldata[columns - 1].endingsample = coldata[columns - 1].startsample
            - 1 + coldata[columns - 1].framesamples;
    }

    /*  Allocate space for frame buffer ,framebuf is an array
        of pointers to frames.  Each element in framebuf points
        to a buffer of the size of one frame
    ------------------------------------------------------------*/
    if((framebuf = (unsigned char **) malloc (columns * 
        sizeof(unsigned char *))) == NULL)
    {
        c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
        free(coldata);
        return (E_FAIL);
    }
    
    /*  Allocate enough memory for each frame file
    ----------------------------------------------*/
    for (j = 0; j < columns ; j++)
    {
       if((framebuf[j] = (unsigned char *) malloc 
           (SAMPLES_FRAME * LINES_FRAME * sizeof(unsigned char))) == NULL)
       {
          c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
          for(i = j - 1; i >= 0; i--)
             free(framebuf[i]);
          free(framebuf);
          free (coldata);
          return (E_FAIL);
       }
    }
         
    /* Allocate memory for the line buffer, the line buffer
       is one complete line across all columns in a row 
    -------------------------------------------------------*/
    if((linebuf = (unsigned char *) malloc 
        (SAMPLES_FRAME * columns * sizeof(unsigned char))) == NULL)
    {
        c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
        for (j = 0; j < columns ; j++)
           free (framebuf[j]);
        free(framebuf);
        free(coldata);
        return (E_FAIL);
    }

    /* Create output file 
    ---------------------*/
    status = c_eopenr(&fdesc,hosout[0],&bands,&sl,&ss,
        &input_params->nl,&input_params->ns,&acc,&odtype,&opt,&nline);
    if (status != E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-open","Error creating output image %s",
            hosout[0]);
        free(linebuf);
        for (j = 0; j < columns ; j++)
           free (framebuf[j]);
        free(framebuf);
        free (coldata);
        return (E_FAIL);
    }
    
    /* set fileflag variable in vblock to indicate 
       that an output file was created
    -----------------------------------------------*/
    if (SUCCESS != q_intg(vblk,"fileflag",1,&one,P_UPDATE))
    {
        free(linebuf);
        for (j = 0; j < columns ; j++)
           free (framebuf[j]);
        free(framebuf);
        free (coldata);
        c_lasmsg(FALSE,"cib2las-tae","Error passing FILEFLAG to TAE");
        return (E_FAIL);
    }
    
    /* return vblock to TAE with new changes 
    ----------------------------------------*/
    if (q_out(vblk) != SUCCESS)
    {
        free(linebuf);
        for (j = 0; j < columns ; j++)
           free (framebuf[j]);
        free(framebuf);
        free (coldata);
        c_lasmsg(FALSE,"cib2las-tae","Error passing variables to TAE");
        return (E_FAIL);
    }
    
    /* set outline to first line 
    -----------------------------*/
    outline = 1;                        

    /* initialize output message routine
    ------------------------------------*/
    curent=0;
    c_inittm("CIB2LAS","Processed","line","category","",
        &input_params->nl,&bands,&input_params->msgtime,&flag);
    c_startm();
   
    /* set the minmax flag based on the TAE global $minmax */
    if(input_params->minmax == TRUE)
       minmax_flg = TRUE;

    /* 
         Main output section:  for each row  in output image
                                   for each column in row
                                       if frame exits
                                           read in frame
                                       else
                                           fill with empty frame
                                    ...end for each column
                                    for each line to output in column
                                        write line to output file
                                    ...end for each line
                               ...end for each row */
    for (row = start_row; row <= end_row; row++)
    {
        /* calculate actual row to output - cib_row
        --------------------------------------------*/
        cib_row = cib_data->maxrow - row;

        /* for each column in this row
        ------------------------------*/   
        for(column = start_column, col = 0; column <= end_column;
            column++, col++)
        {
            /* if not an empty frame file
            -----------------------------*/
            if (list[column + (cib_row * cib_data->numcolumns)] != NULL)
            {
                /* read in frame file path and name
                -----------------------------------*/
                strcpy(ldir,list[column
                    + (cib_row * cib_data->numcolumns)]->dir);
                strcpy(lfile,list[column + 
                    (cib_row * cib_data->numcolumns)]->filename);
                sprintf(filename,"%s%s%s",device,ldir,lfile);

                /* read in a frame file image into framebuffer # col 
                ----------------------------------------------------*/
                if(read_frame(filename,input_params->fill,framebuf[col])
                    != E_SUCC)
                {
                    /* convert directory and filename to lower case
                    -----------------------------------------------*/
                    sprintf(tempstr,"%s%s",ldir,lfile); 
                    str_length = (long) strlen(tempstr);
                    c_up2low(tempstr,&str_length);
                    sprintf(filename,"%s%s",device,tempstr);

                    if(read_frame(filename,input_params->fill,framebuf[col]) 
                        != E_SUCC)
                    {
                        /* convert directory and filename to upper case
                        ------------------------------------------------*/
                        sprintf(tempstr,"%s%s",ldir,lfile);
                        c_low2up(tempstr,&str_length);
                        sprintf(filename,"%s%s",device,tempstr);

                        if(read_frame(filename,input_params->fill,
                            framebuf[col]) 
                            != E_SUCC)
                        {
                            /* restore original name for error message
                            -------------------------------------------*/
                            sprintf(filename,"%s%s%s",device,ldir,lfile);
                            c_lasmsg(FALSE,"cib2las-open",
                                "Error opening frame file %s",filename);
                            free(linebuf);
                            for(col = 0; col < columns; col++)
                                free (framebuf[col]);
                            free(framebuf);
                            free(coldata);
                            return (E_FAIL);
                        }
                    }
                   
                }
            }
            else
            {
                /* empty frame file (doesn't exist) fill frame with user
                   specified fill value 
                --------------------------------------------------------*/
                memset(framebuf[col],input_params->fill,
                    LINES_FRAME * SAMPLES_FRAME);
            }
        } /* end for col... */
         
        /* calculate first line (frame_startingline) in 
           CIB image to output in this row of frames
        -----------------------------------------------*/ 
        if(row == start_row)
            startline = input_params->sl - start_row * LINES_FRAME;
        else
            startline = 0;
        if(row == end_row)
        {
            if(end_row != start_row)
            {
                filled_rows =  end_row - start_row - 1;
                endline = input_params->nl - srowlines - 1 
                    - filled_rows * LINES_FRAME;
            }
            else
            {
                endline = startline + input_params->nl - 1;
                srowlines = input_params->nl;
            }
        }
        else
            endline = LINES_FRAME - 1; 

        /* write out appropriate lines from this row of frame files
        -----------------------------------------------------------*/
        for(line = startline; line <= endline; line++)
        {
            linebufloc=0;
            for(col = 0; col < columns; col++)
            {
                for(sample = coldata[col].startsample; 
                    sample <= coldata[col].endingsample; sample++, linebufloc++)
                {
                    linebuf[linebufloc] = framebuf[col][(line * SAMPLES_FRAME)
                           + sample];
                } 
                
            }
    
            status = c_ewrite(&fdesc,&bands,&outline,linebuf,&nl_blk);
            
            if(status == E_FAIL)
            {
                free(linebuf);
                for (col = 0; col < columns; col++)
                    free(framebuf[col]);
                free (framebuf);
                free(coldata);
                c_lasmsg(FALSE,
                    "cib2las-write","Error writing output image %s",hosout[0]);
                return (E_FAIL);
            }
            else
            {   
                if(minmax_flg)
                    getminmax(input_params->ns,linebuf,&minpixel,&maxpixel);
                outline++;
                curent = outline;
            }
        }   
    }

    /* close the new LAS image
    --------------------------*/
    if(c_eclose(&fdesc) != E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-close",
            "Error closing output image %s",hosout[0]);
        free(linebuf);
        for (col = 0; col < columns; col++)
            free(framebuf[col]);
        free (framebuf);
        free(coldata);
        return (E_FAIL);
    }
    
    /* stop messaging routine
    --------------------------*/
    c_stoptm();

    /* look through attributes for source attribute 
    -----------------------------------------------*/
    for(i = 0;i < cibatt.numatt; i++)
    {
        if (cibatt.attoff[i].paramid == 1)
            strcpy(datasource,cibatt.attribute[i].param.c);
    }

    /* output ddr information 
    -------------------------*/
    if(update_ddr(hosout[0],input_params,&image,&minpixel,&maxpixel,minmax_flg,
        datasource,zone,0) != E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-close",
            "Error closing output image %s",hosout[0]);
        free(linebuf);
        for (col = 0; col < columns; col++)
            free(framebuf[col]);
        free (framebuf);
        free(coldata);
        return (E_FAIL);
    }
    free(linebuf);
    for(col = 0; col < columns; col++)
        free( framebuf[col] );
    free(framebuf);
    free(coldata);
    return (E_SUCC);
}          

/*****************************************************************************
function NAME: getminmax() 


PURPOSE:   get minimum and maximum values in line passed.  values in line
           are compared to current passed in values and if new minimums or
           maximums are found, their value is placed in minval and maxval.

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:


PROJECT:    LAS

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCES:  none
RETURNS:  The variable minpixel will contain the minimum pixel value 
          contained in the image line (line) or its original value (passed 
          in) whichever is the smallest.  maxpixel will contain the largest 
          pixel value contained in the line or its value when passed in, 
          whichever is the largest.
******************************************************************************/
void getminmax
(
    const long numpixels,      /* I:   number of pixels in line             */
    const unsigned char *line, /* I:   image line to check                  */
    unsigned char *minpixel,   /* I/O: current image minimum pixel value    */
    unsigned char *maxpixel    /* I/O: current image maximum pixel value    */
)
{
    long i;
    
    for(i = 0; i < numpixels; i++)
    {
        if(line[i] > *maxpixel)
            *maxpixel = line[i];
        else if(line[i] < *minpixel)
            *minpixel = line[i];
    }
}

/*****************************************************************************
function NAME: calc_corners()

PURPOSE: A function for calculating corner values based on the upper
         left corner of the CIB image, and the starting lines, starting 
         sample, number of lines and number of samples to be take from
         CIB image

PROGRAM HISTORY:
VERSION      PROGRAMMER      DATE         CODE/CONT       REASON
-------      ----------      ----         ---------       ------
1.0          B. Gross        5/00            CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS
ALGORITHM DESCRIPTION:
ALGORITHM REFERENCES:  none
RETURNS:  correct values for output image corners based on upper left corner
          of CIB image and the SL, SS, NL, and NS specified, and 
          
          E_SUCC on success 
          E_FAIL on failure

notes:  inputs->sl + inputs->ss are 0 (zero) based
******************************************************************************/
int calc_corners
(
    const struct CIBCoverage * corner, /* I: Upper left corner of CIB image */
    const INPARAMS *inputs,            /* I: SL SS NL NS for output image   */
    struct CIBCoverage * image         /* I/O: Corners of output image      */
)
{  
    /* calculate North West (Upper Left corner)
    -------------------------------------------*/
    image->ULLat = corner->ULLat - ((inputs->sl) * image->LatRes);    
    image->ULLong = corner->ULLong + ((inputs->ss) * image->LongRes); 
    if(calc_rectangular_coors(image->ULLat,
        image->ULLong,&image->ULy,&image->ULx)!= E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-coors",
            "Error converting corner coordinates");
        return (E_FAIL);
    }
    
    /* calculate South West (Lower Left corner)
    -------------------------------------------*/
    image->LLLong = image->ULLong;  
    image->LLLat = image->ULLat - (inputs->nl - 1) * image->LatRes;     
    if(calc_rectangular_coors(image->LLLat,
        image->LLLong,&image->LLy,&image->LLx)!= E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-coors",
            "Error converting corner coordinates");
       return (E_FAIL);
    }
    
    /* calculate North East (Upper Right corner)
    -------------------------------------------*/
    image->URLat = image->ULLat;
    image->URLong = image->ULLong + (inputs->ns - 1) * image->LongRes;  
    if(calc_rectangular_coors(image->URLat,
        image->URLong,&image->URy,&image->URx)!= E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-coors",
            "Error converting corner coordinates");
        return (E_FAIL);
    }
   
    /* calculate South East (Lower Right corner)
    -------------------------------------------*/
    image->LRLat = image->LLLat;                              
    image->LRLong = image->URLong;       
    if(calc_rectangular_coors(image->LRLat,
        image->LRLong,&image->LRy,&image->LRx)!= E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-coors",
            "Error converting corner coordinates");
        return (E_FAIL);
    }
    return (E_SUCC);
}
#ifdef DEBUG
/****************************************************************************
FUNCTION NAME: check_window_geo()

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:DESC:
    
    check to see if a window exists, 
    if a window exists, and the input parameters
    exceed the limits of the CIB image, 
        default them to the limit of the CIB image
RETURNS:
    
    E_FAIL if no window exists, 
    resets input parameters to CIB image limits if exceeded and issures a 
    warning
*****************************************************************************/
int check_window_geo
(
    INPARAMS *in,               /* I: Input Parameters                      */
    struct CIBCoverage *corner  /* I: CIB image information                 */
)
{
    
    /* check to ensure that input window and CIB image info. are the same
       polarity, return fail if not
    ----------------------------------------------------------------------*/
    if(((in->nw_y > 0 && corner->ULy > 0) || (in->nw_y < 0 && corner->ULy < 0))
                                          && 
       ((in->nw_x > 0 && corner->ULx > 0) || (in->nw_x < 0 && corner->ULx < 0)))
    {
        
        if(in->nw_y < in->se_y || in->se_x < in->nw_x)
        {
            c_lasmsg(FALSE,"check-window","no window");
            return (E_FAIL);
        }
               
        /* check to see if a window exists
        -----------------------------------
           are both x parameters left of (less than) the CIB input image
        or are both x parameters right of (greater than) the CIB input image
        or are both y parameters above the CIB input image
        or are both y parameters below the CIB input image 
        --------------------------------------------------------------------*/
        if(((in->nw_x < corner->ULx) && (in->se_x < corner->ULx)) ||
           ((in->nw_x > corner->LRx) && (in->se_x > corner->LRx)) ||
           ((in->nw_y > corner->ULy) && (in->se_y > corner->ULy)) ||
           ((in->nw_y < corner->LRy) && (in->se_y < corner->LRy)))
        {
            c_lasmsg(FALSE,"check-window","no window");
            return (E_FAIL);
        }
        
        /* check upper left corner 
        -----------------------------------------------------*/
        
        /* if both parameters are outside of CIB image
           set to CIB corner
        -----------------------------------------------*/
        if( in->nw_y > corner->ULy &&
            in->nw_x < corner->ULx)
        {
            c_lasmsg(FALSE,"check-window",
                    "resetting to cib nw corner");
            /* set upper left Lat./y to corner Lat./y
            ----------------------------------------*/
            in->nw_y = corner->ULy;
            
            /* set upper left Long./x to corner Long./x
            ----------------------------------------*/
            in->nw_x = corner->ULx;
        }
        /* else if one parameter outside of the window
        -----------------------------------------------*/
        else if(in->nw_y > corner->ULy)
        {
            in->nw_y = corner->ULy;
            c_lasmsg(FALSE,"check-window",
                    "resetting nw corner y to cib y");
        }
        else if(in->nw_x < corner->ULx)
        {    
            in->nw_x = corner->ULx;
            c_lasmsg(FALSE,"check-window",
                "resetting nw corner x to cib x");
        }

        /* Check South East (Lower Right) corner
        -----------------------------------------------------------*/
        /* if all parameters same sign
        -------------------------------*/
        if((((in->se_y > 0) && (corner->LRy > 0)) || 
            ((in->se_y < 0) && (corner->LRy < 0)))  
                            && 
           (((in->se_x > 0) && (corner->LRx > 0)) ||
            ((in->se_x < 0) && (corner->LRx < 0))))
        {
            if( in->se_y < corner->LRy && 
                in->se_x > corner->LRx )
            {
                c_lasmsg(FALSE,"check-window",
                        "resetting to cib se corner");
                /* set lower right Lat./y to corner Lat./y
                 ------------------------------------------*/
                in->se_y = corner->LRy;
                in->se_lat = corner->LRLat;
                
                /* set lower right Long./x to corner Long./x
                --------------------------------------------*/
                in->se_x = corner->LRx;
                in->se_long = corner->LRLong;
            }
            /* else if one parameter outside of the window
            -----------------------------------------------*/
            else if(in->se_y < corner->LRy)
            {
                c_lasmsg(FALSE,"check-window",
                    "resetting se corner y to cib y");
                in->se_y = corner->LRy;
            }
            else if(in->se_x > corner->LRx)
            {
                in->se_x = corner->LRx;
                c_lasmsg(FALSE,"check-window",
                    "resetting se corner x to cib x");
            }
        }/* all parameters same sign */
        
        return (E_SUCC);
    } 
    else
    {
        c_lasmsg(FALSE,"check-window","no window");
        return (E_FAIL);
    
    }
    
}/* end of function */
#endif

/****************************************************************************
FUNCTION NAME: check_window_deg()

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:DESC:
    
    check to see if a window exists, 
    if a window exists, and the input parameters
    exceed the limits of the CIB image, 
        default them to the limit of the CIB image
RETURNS:
    
    E_FAIL if no window exists, 
    resets input parameters to CIB image limits if exceeded and issures a 
    warning
*****************************************************************************/
int check_window_deg
(
    INPARAMS *in,               /* I: Input Parameters                      */
    struct CIBCoverage *corner  /* I: CIB image information                 */
)
{
    /* check to ensure that input window and CIB image info. are the same
       polarity, return fail if not
    ------------------------------------------------------------------------*/
    if(((in->nw_lat > 0 && corner->ULLat > 0)   ||
        (in->nw_lat < 0 && corner->ULLat < 0))
                                                    && 
       ((in->nw_long > 0 && corner->ULLong > 0) ||
        (in->nw_long < 0 && corner->ULLong < 0)))
    {
        
        if(in->nw_lat < in->se_lat || in->se_long < in->nw_long)
        {
            c_lasmsg(FALSE,"check-window","no window");
            return (E_FAIL);
        }
               
        /* check to see if a window exists
        -----------------------------------
           are both x parameters left of (less than) the CIB input image
        or are both x parameters right of (greater than) the CIB input image
        or are both y parameters above the CIB input image
        or are both y parameters below the CIB input image 
        --------------------------------------------------------------------*/
        if(((in->nw_long < corner->ULLong) && (in->se_long < corner->ULLong)) ||
           ((in->nw_long > corner->LRLong) && (in->se_long > corner->LRLong)) ||
           ((in->nw_lat > corner->ULLat) && (in->se_lat > corner->ULLat)) ||
           ((in->nw_lat < corner->LRLat) && (in->se_lat < corner->LRLat)))
        {
            c_lasmsg(FALSE,"check-window","no window");
            return (E_FAIL);
        }
        
        /* check upper left corner 
        -----------------------------------------------------*/
        
        /* if both parameters are outside of CIB image
           set to CIB corner
        -----------------------------------------------*/
        if( in->nw_lat > corner->ULLat &&
            in->nw_long < corner->ULLong)
        {
            c_lasmsg(FALSE,"check-window",
                    "resetting to cib nw corner");
            /* set upper left Lat./y to corner Lat./y
            ----------------------------------------*/
            in->nw_lat = corner->ULLat;
            
            /* set upper left Long./x to corner Long./x
            ----------------------------------------*/
            in->nw_long = corner->ULLong;
        }
        /* else if one parameter outside of the window
        -----------------------------------------------*/
        else if(in->nw_lat > corner->ULLat)
        {
            in->nw_lat = corner->ULLat;
            c_lasmsg(FALSE,"check-window",
                    "resetting nw corner y to cib y");
        }
        else if(in->nw_long < corner->ULLong)
        {    
            in->nw_long = corner->ULLong;
            c_lasmsg(FALSE,"check-window",
                "resetting nw corner x to cib x");
        }

        /* Check South East (Lower Right) corner
        -----------------------------------------------------------*/
        /* if all parameters same sign
        -------------------------------*/
        if((((in->se_lat > 0) && (corner->LRLat > 0)) || 
            ((in->se_lat < 0) && (corner->LRLat < 0)))  
                            && 
           (((in->se_long > 0) && (corner->LRLong > 0)) ||
            ((in->se_long < 0) && (corner->LRLong < 0))))
        {
            if( in->se_lat < corner->LRLat && 
                in->se_long > corner->LRLong )
            {
                c_lasmsg(FALSE,"check-window",
                        "resetting to cib se corner");
                /* set lower right Lat./y to corner Lat./y
                 ------------------------------------------*/
                in->se_lat = corner->LRLat;
                              
                /* set lower right Long./x to corner Long./x
                --------------------------------------------*/
                in->se_long = corner->LRLong;
            }
            /* else if one parameter outside of the window
            -----------------------------------------------*/
            else if(in->se_lat < corner->LRLat)
            {
                c_lasmsg(FALSE,"check-window",
                    "resetting se corner y to cib y");
                in->se_lat = corner->LRLat;
            }
            else if(in->se_long > corner->LRLong)
            {
                in->se_long = corner->LRLong;
                c_lasmsg(FALSE,"check-window",
                    "resetting se corner x to cib x");
            }
        }/* all parameters same sign */
        
        return (E_SUCC);
    } 
    else
    {
        c_lasmsg(FALSE,"check-window","no window");
        return (E_FAIL);
    
    }
    
}/* end of function */
/*****************************************************************************
returns CIB zone 
*****************************************************************************/
char get_zone
( 
    void 
)
{
    return (zone);
}
/*****************************************************************************
function NAME:    calc_rectangular_coors()   


PURPOSE:  Convert the geometric coordinates (Lat. & Long.) in degress to
Projection coordinates in meters.  Geometric coordinates must be in degrees. 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:
  
    get necessary information 
        false Easting
        false Northing
        Latitude of true scale of projection
        Central Meridian of projection
    convert all values in degrees to radians 
    initialize gctpc function for conversion in Equirectangular projection
        from lat. and long. (in radians) to projection coordinates in meters
    perform conversion and update x and y coordinates 
    
RETURNS: Returns y and x projections coordinates for Lat. and Long passed.
         Returns (E_SUCC) if successful, (E_FAIL) otherwise
NOTES:   Not static because also called by function cib2lasout() in cib2lasout.c
******************************************************************************/
int calc_rectangular_coors
(
    double lat,               /* I: input latitude of point degrees         */
    double lon,               /* I: input longitude of point degrees        */
    double *y,                /* O: output Y projection coordinate meters   */
    double *x                 /* O: output X projection coordinate meters   */
) 
{
    double lat_truescale;     /* equirectangular latitude of true scale 
                                  (no distortion)                           */
    double fe;                /* false easting in projection                */
    double fn;                /* false northing in projection               */
    double cent_mer;          /* central meridian of projection             */
    double radius=EARTH_RADIUS; /* radius of sphere for transformations
                                 taken from sphsz function (sphdz.c) in 
                                 lasgeolib,                                 */
    double latitude;
    double longitude;
    char zone;                 /* CIB zone                                  */
    
    /* get CIB zone 
    ---------------*/
    zone = get_zone();

    /* get Latitude of true scale 
    ------------------------------*/ 
    if(get_truescalelat(zone,&lat_truescale) == E_FAIL)
    {
        c_lasmsg(FALSE, "cib2las-tae", 
            "Error retrieving Latitude of True Scale for Zone %c",zone);
        return (E_FAIL);
    }

    /* convert degrees to radians 
    ------------------------------*/
    fe = CIB_FE * D2R;
    fn = CIB_FN * D2R;
    cent_mer = CIB_CENT_MER * D2R;
    lat_truescale = lat_truescale * D2R;
    latitude  = lat * D2R;
    longitude = lon * D2R;
    
    /* initialize Equirectangular conversion from 
       (degrees Lat. and Long.) to meters (X and Y).
    ------------------------------------------------*/
    equiforint(radius,cent_mer,lat_truescale,fe,fn);
    
    /* perform conversion (degrees Lat. and Long.)
       to meters (X and Y).
    ----------------------------------------------*/
    equifor(longitude,latitude,x,y);

    return (E_SUCC);
}

/****************************************************************************
NAME:     calc_geometric_coors  


PURPOSE:  Convert the Projection coordinates enterred by operator to 
geometric coordinates (Lat. & Long.) used by CIB2LASout.   Latitude and 
Longitude returned in radians.  Projection coordinates must be in meters

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:
  
    get necessary information 
        false Easting
        false Northing
        Latitude of true scale of projection
        Central Meridian of projection
    convert central meridian and latitude of true scale from ddr format
        of packed dms to degrees
    convert all values in degrees to radians 
    initialize gctpc function for conversion in Equirectangular projection
        from lat. and long. (in radians) to projection coordinates in meters
    perform conversion and return x and y coordinates  
    
*****************************************************************************/
int calc_geometric_coors
(
    double *lat,               /* O: output latitude of point degrees       */
    double *lon,               /* O: output longitude of point degrees      */
    double y,                  /* I: input Y projection coordinate meters   */
    double x                   /* I: input X projection coordinate meters   */
) 
{
    double lat_truescale,        /* equirectangular latitude of true scale 
                                    (no distortion)                         */
           fe,                   /* false easting in projection             */
           fn,                   /* false northing in projection            */
           cent_mer,             /* central meridian of projection          */
           radius=EARTH_RADIUS;  /* radius of sphere for transformations
                                    taken from sphsz function (sphdz.c) in 
                                    lasgeolib,                              */
    char zone;                   /* CIB zone                                */
    /* get cib zone
    ----------------*/
    zone = get_zone();
    
    /* get Latitude of true scale 
    -----------------------------*/ 
    if(get_truescalelat(zone,&lat_truescale) != E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-tae", 
            "Error retrieving Latitude of True Scale for Zone %c",zone);
        return (E_FAIL);
    }

    /* get False Easting and False Northing 
    ---------------------------------------*/
    fe = CIB_FE * D2R;
    fn = CIB_FN * D2R;

    /* get Central Meridiam of projection  
    --------------------------------------*/
    cent_mer = CIB_CENT_MER * D2R;

    /* convert degress to radians (required for equiforint & equifor)
    -----------------------------------------------------------------*/
    lat_truescale = lat_truescale * D2R;

    /* initialize Equirectangular conversion from latitude and longitude
       to geographic coordinates X and Y.
    --------------------------------------------------------------------*/
    equiinvint(radius,cent_mer,lat_truescale,fe,fn);

    /* perform conversion meters (X and Y) to (degrees lat. and long) 
    -----------------------------------------------------------------*/
    equiinv(x,y,lon,lat);

    *lon *= R2D;
    *lat *= R2D;
    
    return(E_SUCC);
}
