#ifndef _CIBFRAME_H_
#define _CIBFRAME_H_
#define	CIB_HIST_LEN 216   /* histogram size is fixed for CIB               */

struct CIBHeader {
    unsigned char endianness;
                      /* little/big endian indicator, a boolean 
                         value indicating the endianess of this file , FF 
                         hex is little endian, 00 (hex) is big endian.      */                         
    unsigned short length;
                      /* header section length, after the header 
                         section is the location section, used to locate 
                         start of location section of file                  */  
    char     filename[13];
                      /* 12 byte ASCII name, the name of this file A.TOC    */ 
    unsigned char update;
                      /* boolean, indicates if this is a new, replacement,
                         or updated file                                    */
    char     specnum[16]; 
                      /* 15 byte ascii string defining the document number
                         of the standard to which the format of this Table
                         of Contents file conforms                          */
    char     specdate[9];
                      /* 8 byte ascii character string in the format YYYYMMDD
                         defining the effective date of the standard to which
                         this Table of contents file conforms.              */
    char     sclass; 
                      /* security classification:  a 1-byte ASCII code 
                         string coded to indicate the security level 
                         (e.g. unclassified, confidential, secret) of this
                         file, as  specified in MIL-STD-2411-1, section 
                         5.1.8                                              */   
    char     scode[3];
                      /* security country/international code, a 2-byte ASCII
                         string coded to indicate the originating country or
                         international affiliation of the security classifi-
                         cation of this file, as defined in MIL-STD-2411-1,
                         section 5.1.7                                      */ 
    char     smark[3]; 
                      /* security release marking, a 2 bytpe ASCII character
                         string coded to indicate any special handling or 
                         releasibility restrictions assigned to the contents
                         of this file, as defined in MIL-STD-2411-1, section
                         5.1.9                                              */
    long    location;
                      /* location section location, a 4 byte unsigned int 
                         defining the absolute address (i.e. byte number)
                         of the first byte of the [location section] 
                         relative to the beginning of the file              */
};

/*  The [location section] designates the beginning byte addresses 
    relative to the beginning of the file for the sections/
    components in a RPF.  It's structure and data types shall be
    as defined in MIL-STD-2411.  The component IDs are as defined 
    in MIL-STD-2411-1.                                                      */

struct CIBLocation {
    unsigned short length;     /* location section length                   */
    unsigned long tbloffset;   /* component loc tbl offset                  */
    unsigned short numrec;     /* numb component loc recs                   */
    unsigned short reclength;  /* component loc rec length                  */
    unsigned long agglength;   /* component aggregate len                   */
    struct CompRec{
        unsigned short compid; /* component id                              */
        unsigned long length;  /* component length                          */
        long    location;      /* component location                        */
    } comptable[17];           /* component location table                  */
};

/*  The [coverage section] defines the geographic extent of the 
    frame.  The structure and data types for the coverage section
    in the CIB product shall be as defined in MIL-STD-2411.  This
    section icludes four sets of latitude and longitude vertices
    that shall define the geographic extent of the [frame file].
    It also contains, six parameters that define the pixel 
    resolution and their intervals.                                                                                                                         */

struct CIBCoverage {
    double    ULLat;    /* northwest/upper left latitude                    */
    double    ULy;
    
    double    ULLong;   /* northwest/upper left longitude                   */
    double    ULx;

    double    LLLat;    /* southwest/lower left latitude                    */
    double    LLy;
    
    double    LLLong;   /* southwest/lower left longitude                   */
    double    LLx;
    
    double    URLat;    /* northeast/upper right latitude                   */
    double    URy;
    
    double    URLong;   /* northeast/upper right long                       */
    double    URx;
    
    double    LRLat;    /* southeast/lower right latitude                   */
    double    LRy;

    double    LRLong;   /* southeast/lower right long                       */
    double    LRx;
    
    double    NSRes;    /* north-south/vertical res                         */
    double    EWRes;    /* east-west/horizontal res                         */
    
    double    LatRes;   /* latitude/vertical interval                       */
    double    LongRes;  /* longitude/horizontal interval                    */
    
    double    yRes;     /* y pixel - pixel resolution used in 
                           calculations                                     */
    double    xRes;     /* x pixel - pixel resolution used in 
                           calculations                                     */
};

/* The [compression section] is applicable for compressed data 
   only.  VO CIB compression provides tables of pixel vector
   codebook values necessary to decode the compressed image for 
   the vector quantization algorithm used for CIB.                          */

#define CMP_TBLS 4
#define IMAGE_CODES 4096
#define CMP_LOOKUP_VALS 4

struct CIBCompress {

    /* compression section subheader                                        */
    unsigned short algoid;   /* compress algorithm id
                                FIXED by CIB 1 (VQ)                         */
    unsigned short numoff;   /* numb compression lookup offset
                                records FIXED by CIB 4                      */
    unsigned short numparam; /* num compression parameter offset
                                records FIXED by CIB 0                      */

    /* compression lookup subsection                                        */
    unsigned long LOTbloff;  /* compress lookup offset tbl offset
                                FIXED by CIB 6                              */
    unsigned short LOTblrec; /* compress lookup tbl offset record
                                length FIXED by CIB 14                      */

    /* compression lookup offset table                                      */
    struct LORec{
        unsigned short tblid;  /* compression lookup table id
                                  FIXED by CIB (1,2,3,4)                    */
        unsigned long numrec;  /* numb compression lookup
                                  recs FIXED by CIB 4096                    */
        unsigned short numval; /* num values/compress lookup
                                  record FIXED by CIB 4                     */
        unsigned short bitlen; /* compress lookup value bit
                                  length FIXED by CIB 8                     */
        unsigned long LToff;   /* compress lookup tbl offset
                                  LOTbloffset+4*LOTblreclen+
                                  (tblid-1)*numrec*numval                   */
    } LOTbl[4];

    /* compression lookup table                                             */
    unsigned char vals[CMP_TBLS][IMAGE_CODES][CMP_LOOKUP_VALS];
    /* compress LUT vals                                                    */
};

/*  The [color/grayscale table] contains 216 instances of grayscale
    values that are pointed to by the indicies in the [compression
    section].  They define the grayscale value of a pixel for 
    display or printing purposes.  The structure and data types of
    the [color/grayscale] section are defined in MIL-STD-2411.              */

#define NUM_GRAYSCALE_VALUES 217   

struct CIBColor {

    /* color/grayscale section subheader                                    */
    unsigned char numLUT; /* number of grayscale offset
                             records                                        */
    unsigned char numCON; /* number of color converter
                             offset records                                 */
    char efname[13];      /* external grayscale file
                             name                                           */
    /* colormap subsection                                                  */
    unsigned long cmapoff;     /* cmap offset table offset                  */
    unsigned short moffreclen; /* grayscale offset rec len                  */

    /* colormap offset table                                                */
    struct ColorOffset {
        unsigned short tblid;   /* grayscale table id
                                    FIXED by CIB 3                          */
        unsigned int   numrec;  /* num grayscale records                    */
        unsigned char  elemlen; /* grayscale element length                 */
        unsigned short histlen; /* histogram record length                  */
        unsigned int   grayoff; /* grayscale table offset                   */
        unsigned int   histoff; /* histogram table offset                   */
    } COffTbl[2];

    /* color/grayscale element group                                        */
    unsigned char CTable[2][NUM_GRAYSCALE_VALUES];/* grayscale elements     */

    /* histogram element group                                              */
    unsigned long Histogram[2][CIB_HIST_LEN];/* histogram elements          */

    /* color converter subsection                                           */
    unsigned long  ccoffset;    /* color converter offset table offset      */
    unsigned short coffreclen;  /* color converter offset record length     */
    unsigned short ccreclen;    /* color converter rec len                  */

    /* color converter offset table */
    struct CCRec {
        unsigned short cctblid;  /* color converter table id                */
        unsigned long  nccrecs;  /* num color converter recs                */
        unsigned long  cctbloff; /* color conv tbl offset                   */
        unsigned long  srcoff;   /* source grayscale offset table offset    */
        unsigned long  tgtoff;   /* target grayscale offset table offset    */
    } ccoff;

    /* color converter table                                                */
    /* target grayscale table entry number                                  */
    unsigned long CCTbl[NUM_GRAYSCALE_VALUES]; 
};

/*  For compressed data, the [image section] contains indices of
    VQ compression kernels in the [compression section], that in
    turn, specify grayscale indices in the primary color table
    within the [color/grayscale table section].  For uncompressed
    data, the [image section] contains grayscale indices in the
    primary color table.                                                    */

#define CIB_SUBFRAMES 36
#define CMP_IMAGE_ROWS 64
#define CMP_IMAGE_COLUMNS 64
#define UCMP_IMAGE_ROWS 256
#define UCMP_IMAGE_COLUMNS 256

struct CIBImage {
    /* image description subheader                                          */
    unsigned short numgrp;   /* num spectral grps FIXED CIB 1               */
    unsigned short numstbl;  /* num subframe tbls FIXED CIB 36              */
    unsigned short numbtbl;  /* num spectral band tbls
                               FIXED by CIB 1                               */
    unsigned short numbline; /* num spectral band lines/image row
                               FIXED by CIB 1                               */
    unsigned short EWsubfrm; /* num subframes in east-west
                               direction FIXED by CIB 6                     */
    unsigned short NSsubfrm; /* numb subframes in north-south
                               direction FIXED by CIB 6                     */
    unsigned long  colsubf;  /* num output cols/subframe
                               FIXED by CIB 256                             */
    unsigned long  rowsubf;  /* num output rows/subframe 
                               FIXED by CIB 256                             */
    unsigned long  smaskoff; /* subframe mask table offset
                               FIXED by RPF 4                               */
    unsigned long  tmaskoff; /* transparency mask table offset
                               4+2*36 or 4 if no mask                       */

    /* mask subsection                                                      */
    /* mask subheader                                                       */
    unsigned short subrecl;  /* subframe sequence record length
                                4 or 0 if none                              */
    unsigned short tranrecl; /* transparency sequence rec length
                                4 or 0 if none                              */
    unsigned short tcodelen; /* transparent output pixel code len
                                FIXED by CIB 8                              */
    unsigned char  tcode;    /* transparent output pixel code
                                FIXED by CIB 216                            */
    /* subframe mask table                                                  */
    unsigned long  sframes[CIB_SUBFRAMES];
    /* subframe byte offset                                                 */
    
    /* transparency mask table                                              */
    unsigned long  tframes[CIB_SUBFRAMES];
    /* transparent subframe byte offset*/

    /* image display parameters subheader                                   */
    unsigned long  numrow;  /* number of image rows
                              FIXED by CIB 64                               */
    unsigned long  numcol;  /* number of image codes per row
                              FIXED by CIB 64                               */
    unsigned char  codelen; /* image code bit length
                              FIXED by CIB 12                               */
    /* spatial data subsection */
    /* compressed image codes                                       
       36 subframes consisiting of 64 rows 
       containing 64 image codes                                            */
    unsigned short codes[CIB_SUBFRAMES][CMP_IMAGE_ROWS][CMP_IMAGE_COLUMNS]; 

    /* uncompressed image pixels
       36 subframes containting 256 rows 
       containing 256 columns                                               */
    unsigned char  pixels[CIB_SUBFRAMES][UCMP_IMAGE_ROWS][UCMP_IMAGE_COLUMNS];
};



struct CIBAttribute {

    /* attribute section subheader                                          */
    unsigned short numatt;    /* num attribute offset recs                  */
    unsigned short numcov;    /* num explicit areal cov recs                */
    unsigned long  tbloff;    /* attribute offset table offset              */
    unsigned short offrecl;   /* attribute offset record length             */

    /* attribute subsection                                                 */
    /* attribute offset table                                               */
    struct AttOffset {
        unsigned short attid;    /* attribute id                            */
        unsigned char  paramid;  /* parameter id                            */
        unsigned char  covseq;   /* areal coverage seq num                  */
        unsigned int   attroff;  /* attribute rec offset                    */
    } attoff[250];

    /* attribute table */
    struct AttRec {
        union ParamVal{
            char           c[13]; /* ASCII param value                      */
            unsigned long  l;     /* long param value                       */
            unsigned short s;     /* short param value                      */
        } param;
    } attribute[250];

    /* explicit areal coverage subsection                                   */
    unsigned long  covtbloff;   /* explicit areal coverage table offset     */
    unsigned short covreclen;   /* explicit areal coverage record length    */
    unsigned short coordreclen; /* corner coords rec len                    */

    /* explicit areal coverage table                                        */
    struct AreaCov {
        unsigned short numvert; /* number of vertices                       */
        struct Vertex{
            double    lat;     /* latitude                                  */
            double    lng;     /* longitude                                 */
        } coords[8];
    } attarea[100];

};

struct CIBUpdate {

    /* replace/update section subheader                                     */
    unsigned long  rtbloff;  /* replace/update table offset                 */
    unsigned short numrec;   /* numb replace/update records                 */
    unsigned short rreclen;  /* replace/update record length                */

    /* replace/update subsection                                            */
    /* replace/update table                                                 */
    struct UpRec {
        char    nfname[13];   /* new file name                              */
        char    ofname[13];   /* old file name                              */
        unsigned char upstat; /* replace/update status                      */
    } update[34];
};

int read_frame_coverage
(
  FILE *ffptr,                /* I:  pointer to frame file                  */
  struct CIBCoverage *cibcov  /* O:  structure containing coverage          */
);

#endif
