/****************************************************************************
FILENAME:  get_par.c

FUNCTION(S): get_par()
             check_limits()

PURPOSE:   Read in TAE paramaters 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB          Original Development
  1.1      A. Shimitz      10/00           CSB          Added retrieval of
                                                        TILENUM parameter value
                                                        (CR 4215)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:    LAS

ALGORITHM DESCRIPTION:
        
    Retrieve the required parameters and globals from the TAE parameter block
        Retrieve the output image name
        Retrieve the fill value
        Retrieve the tilenum values
        Retrieve the windows units type "UNITS"
        Retreive the output window specification
        If all four variables present
            If units LS
                if four window parameters entered
                    Set intype SLSSNLNS 
                else if two window parameters entered
                    Set intype SLSS
                If parameters exceed input image size,
                    default NL and/or NS to remainder of CIB image
                If intype is SLSS
                    default NL and/or NS to remainder of CIB image
                Set corner parameters (meters)
            Else-If units DEG or DMS
                if 4 window parmeters
                    set intype to COORS4
                else if 2 window parameters
                    set intype to COORS2
                Check limits of corner params (deg.)
                Check limits on corners, lower right
                    corner must at at lower right of
                    upper left corner
            Else-If units PRO
                if 4 window parameters 
                    Set intype COORS4
                else if 2 window parameters
                    set intype COORS2
                set corner parmas (meters)
        Else
            Error missing input parameters
            Return fail
        Read in globals
            Read in message time interval
            Read in calculate minmax flag
     Return success

RETURNS: Returns user entered parameters in structure INPARAMS
         return (E_SUCC) if successful, (E_FAIL) otherwise

ALGORITHM REFERENCES:  none
*****************************************************************************/
#include<string.h>
#include"las.h"
#include"util.h"
#include"cproj.h"
#include"geompak.h"
#include"worgen.h"
#include"cib2las.h"
#include"cib2lasout.h"
#include"get_par.h"
#include"update_ddr.h"


static int check_limits
(
    const INPARAMS *in     /* I: structure latitudes and longitude to check */
);


int get_par
(
    struct PARBLK *vblock,   /* I: Pointer to TAE parameter block           */
    char taeout[][CMLEN],    /* O: TAE file name of output image            */
    char comment[CMLEN],     /* O: Comment to load CD                       */  
    INPARAMS *in             /* O: storage for user inputted parameters     */
)
{
    struct  VARIABLE *var;   /* Struct for TAE parameters                   */
    long    ltemp;
    long    i;
    char    temp_str[CMLEN]; /* value of the $MINMAX TAE global             */

    /* set input type to default 
    ----------------------------*/
    in->type = DEFAULT;
    
    /* Retrieve the comment to send to commputer room operator
    ----------------------------------------------------------*/
    var = p_find(vblock,"COMMENT");
    if(var->v_count==0)
        strcpy(comment,"");
    else
       strcpy(comment,SVAL(*var,0)); 


    /* Retrieve the directory containing the CIB image data.
    --------------------------------------------------------*/
    var = p_find(vblock, "INDIR");
    if (var->v_count == 0)
    {
       if(strlen(comment) == 0)
       {
           c_lasmsg(FALSE,"cib2las-tae",
           "You must specify input directory or comment to mounting a CD");
           return (E_FAIL);
       }
    }
    else
    {
        strcpy (in->indir, SVAL(*var, 0));
        /* if no comment is present concatenate '/' to end of input
           directory, not needed if ingesting off a remote CD
        -----------------------------------------------------------*/
       if(strlen(comment) < 1)
           strncat(in->indir,"/",1);
    }
  
    /* Retrieve output image name
    -----------------------------*/
    var = p_find(vblock, "OUT");
    if (var->v_count == 0)
    {
       c_lasmsg(FALSE, "cib2las-tae", "You must specify the output name");
       return (E_FAIL);
    }
    strcpy (taeout[0], SVAL(*var, 0));

    /* Get fill value for empty frames, and transparent pixels
    ----------------------------------------------------------*/ 
    var = p_find(vblock,"FILL");
    if(var->v_count == 0)
        in->fill = 0;
    else
        in->fill = IVAL(*var,0);

    /* Get the boundary rectangles the user wants to ingest
    -------------------------------------------------------*/
    var = p_find(vblock,"TILENUM");
    if(var->v_count == 0)
        in->tilenum[0]=-1;
    else
    {
        for(i = 0; i < var->v_count; i++)
            in->tilenum[i] = IVAL(*var,i);
        in->tilenum[i] = -1;
    }

    /* Get output image specifications, by window coordinates,
    ----------------------------------------------------------*/
    var = p_find(vblock,"UNITS");
    if(var->v_count == 0)
    {
        c_lasmsg(FALSE, "cib2las-input",
                "No input units specified for window");
        return (E_FAIL);
    }
    else
    {
        strcpy(in->units,SVAL(*var,0)); 
        ltemp = (long) strlen(in->units);
        c_low2up(in->units,&ltemp);
    }
     
    /* Get window parameters 
    ------------------------*/
    var = p_find(vblock,"WINDOW");
    
    if(var->v_count == 4 || var->v_count == 2)
    {
        /* If units line & sample
        -------------------------*/
        if(strcmp(in->units,"LS")==0)
        {
           
            in->sl = (long) RVAL(*var,0);  
            in->ss = (long) RVAL(*var,1);  
            
            if (in->sl < 1 || in->ss  < 1 )
            {
                c_lasmsg(FALSE,"cib2las-input",
                    "Error in window starting line/samples parameters");
                return (E_FAIL);
            }
            
            if(var->v_count == 4)
            {
                in->type = SLSSNLNS; 
                in->nl = (long) RVAL(*var,2);
                in->ns = (long) RVAL(*var,3);  
                if (in->nl < 1 || in->ns  < 1 )
                {
                    c_lasmsg(FALSE,"cib2las-input",
                        "Error in window number of line/samples parameters");
                    return (E_FAIL);
                }
            }
            else 
                in->type = SLSS; 
        }
        else if(strcmp(in->units,"DMS") == 0 || strcmp(in->units,"DEG") == 0)
        {
            in->nw_lat  = RVAL(*var,0);
            in->nw_long = RVAL(*var,1);
            if(var->v_count == 4)
            {
                in->type = COORS4;
                in->se_lat  = RVAL(*var,2);
                in->se_long = RVAL(*var,3);
            }
            else
                in->type = COORS2;
                   
            /* If units packed degrees minutes seconds (Lat. & Long.) 
            ---------------------------------------------------------*/
            if(strcmp(in->units,"DMS")==0)
            {
                c_decdeg(&in->nw_lat,"DMS","LAT");
                c_decdeg(&in->nw_long,"DMS","LON");
                if(var->v_count == 4)
                {
                    in->type = COORS4;
                    c_decdeg(&in->se_lat,"DMS","LAT"); 
                    c_decdeg(&in->se_long,"DMS","LON");
                }
                 else
                    in->type = COORS2;
            }
            
            /* check window parameters against normal limits for Lat. & Long.
            -----------------------------------------------------------------*/
            if(check_limits(in) != E_SUCC)
               return (E_FAIL);
        }
            
        /* If units = PROjection coordinates (X & Y) in meters
        ------------------------------------------------------*/
        else if(strcmp(in->units,"PRO")==0)
        {
            
            in->nw_y  = RVAL(*var,0);
            in->nw_x = RVAL(*var,1);
            if(var->v_count == 4)
            {
                in->type = COORS4;
                in->se_y = RVAL(*var,2);
                in->se_x = RVAL(*var,3);
            }
            else
                in->type = COORS2;
        } 

    }
    else if (var->v_count > 0)
    { 
        c_lasmsg(FALSE, "cib2las-input",
                 "Missing window specifications");
        return (E_FAIL);
    }
    /* no window parameters entered, output the entire CIB volume (image)
    ---------------------------------------------------------------------*/
    else
    {
      in->type = DEFAULT;
    }
    
    /* Retrieve globals
    -------------------*/
    var = p_find(vblock,"$msgtim");
    if(var->v_count == 1)
        in->msgtime = IVAL(*var,0);
    else
        in->msgtime = 0;

    var = p_find(vblock,"$minmax");
    if(var->v_count==0)
        in->minmax = 0;
    else
    {
        strcpy(temp_str,SVAL(*var,0)); 
        ltemp = (long) strlen(temp_str);
        c_low2up(temp_str,&ltemp);
        if(*temp_str == 'Y')
            in->minmax = 1;
        else
            in->minmax = 0;
    }
        
    return(E_SUCC);
}

/*****************************************************************************
function NAME: check_limits()

PURPOSE:   Check Latitudes and Longitudes against appropriate limits

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:


PROJECT:    LAS

ALGORITHM DESCRIPTION: 

ALGORITHM REFERENCES: 

******************************************************************************/
static int check_limits
(
    const INPARAMS *in  /* I: structure latitudes and longitude to check */
)
{
    if(in->nw_lat > 90.0 ||  in->nw_lat < -90.0)
    {
        c_lasmsg(FALSE, "cib2las-tae","Error in UL Latitude (%f)",
            in->nw_lat);
        return (E_FAIL);
    }
    
    if(in->nw_long > 180.0 ||  in->nw_long < -180.0)
    {
        c_lasmsg(FALSE, "cib2las-tae","Error in UL Longitude (%f)",
            in->nw_long);
        return (E_FAIL);
    }        
  
    if(in->se_lat > 90.0 ||  in->se_lat < -90.0)
    {
        c_lasmsg(FALSE, "cib2las-tae","Error in LR Latitude (%f)",
            in->se_lat); 
        return (E_FAIL);
    }        
  
    if(in->se_long > 180.0 ||  in->se_long < -180.0)
    {
        c_lasmsg(FALSE, "cib2las-tae","Error in LR Longitude (%f)",
            in->se_long);
        return (E_FAIL);
    }        
    return (E_SUCC);
}
