/*****************************************************************************
FILE NAME: locate_frame.c

FUNCITON(S):

    locate_frame(),
    get_val(),
    get_columns(),
    get_rows()

PURPOSE:   Calculates the zone row and column for a given frame file name 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
   1.0      B. Gross        2/00            CSB         original development 
   1.1      A.Shimitz      10/00            CSB         added check to see if
                                                        the frame file exists
                                                        in one of the user-
                                                        specified boundary
                                                        rectangles (CR 4215)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS
ALGORITHM DESCRIPTION:

    if file is a overview file
       set file info to invalid
       exit
    Parse frame file name obtaining
        data series (identifies image resolution)
        zone
        cummulative frame file number in zone (base 34 value)
    convert cumulative frame file number to decimal (base 10)
    get number of frame file rows in zone
    get number of frame file columns in zone
    calculate maximum number of a frame file in a zone
    if cumulative frame file number > maximum
        set frame file data to invalid
        exit function
    calculate this frame files's row number in zone
    calculate this frame file's column number in zone

ALGORITHM REFERENCES:  
    
    MIL-PRF-89041 Performance Specification
    Controlled Image Base
    
    APPENDIX A:  for calculation for frame file
                                row and column numbers

    CIB frame file naming convention    

        "ffffffvp.ccz"

         'ffffff' = base 34 of cumulative frame file number in zone 'z'
                    (f = values are 0-9, A-Z, excluding I and O)
         'v'      = version 
         'p'      = producer code
         cc       = product type    I1 =  5 meter CIB
                                    I2 = 10 meter CIB
         'z'      = zone                      
    
    MIL-STD-2411-1 Military Standard
    Registered Data Value for Raster Product Format

    for registered product type 

    I1 =  5 meter CIB
    I2 = 10 meter CIB
    
******************************************************************************/
#include<stdlib.h> 
#include<string.h> 
#include<ctype.h>  
#include<stdio.h>
#include<math.h>
#include<ctype.h>       
#include"las.h"
#include"cib2las.h"
#include"locate_frame.h"


int locate_frame
(
    char zone,             /* I: CIB zone of requested image                */
    int tilenum[MAXTILES], /* I: boundary rectangles to ingest              */
    FRAMEFILE *ff          /* I/O:  frame file record to update             */
)
{
    char fname[7];         /* storage for alphanumeric representation
                              of frame file's cumulative frame file
                              number, nrz or "fffff" above, also the first
                              five letters/numbers of the frame file name   */
    char ctemp;            /* temporary storage for a char                  */
static int warn_flag = FALSE; /* warning flag used to track if a warning 
                              has  been sent that other zones are present,
                              and will not be in the output image           */
    int br_id_flag = FALSE;/* Flag used to determine if the current frame
                              file is in a boundary rectangle specified
                              by the user                                   */
    int  i;                /* counter                                       */
    long Rz;               /* the number of columns in zone                 */
    long Cz;               /* the number of columns in zone                 */ 
    long rFz;              /* the frames row in zone                        */
    long cFz;              /* the frames column in zone                     */
    long nrz;              /* cumalative frame file number in zone          */ 
    long nrzMAX;           /* maximum nrz (cumalative frame number)
                              in zone                                       */
    double value=0.0;      /* number calculated to determine 
                              frame file number in zone                     */
    
    /* if filename is not correct length, fail 
    ------------------------------------------*/
    if(strlen(ff->filename) != 12)
    {
        ff->row = NOTAFRAMEFILE;
        ff->column = NOTAFRAMEFILE;
        ff->fnum = NOTAFRAMEFILE;
        return(E_SUCC);
    }

    /* if file is an overview file, set this file's values to invalid
    -----------------------------------------------------------------*/   
    if(strncmp(&ff->filename[9],"OVR",3) == 0)
    {
        ff->row = NOTAFRAMEFILE;
        ff->column = NOTAFRAMEFILE;
        ff->fnum = NOTAFRAMEFILE;
        return(E_SUCC);
    }
    
    /* save filename into record 
    ----------------------------*/
    strncpy(fname,ff->filename,6);
    fname[6]='\0';

    /* parse zone from filename 
    ----------------------------*/
    ff->zone = ff->filename[11];   

    /* check to see if this frame file is in the desired zone.
       If this frame file isn't in the zone, do not include.
       Added if(zone != ' ') to keep this check from being done
       if the zone has not been determined yet.
    ------------------------------------------------------------*/
    if(zone != ' ')
    {
        if(ff->zone != zone)
        {
            if(warn_flag == FALSE)
            {
                c_lasmsg(FALSE,"Informational",
                    "More than one zone present on volume");
                c_lasmsg(FALSE,"Informational",
                    "Only data for zone %c being ingested", zone);
                warn_flag = TRUE;
            }
            ff->row = NOTAFRAMEFILE;
            ff->column = NOTAFRAMEFILE;
            ff->fnum = NOTAFRAMEFILE;
            return(E_SUCC);
        }
    }

    /* check to see if this frame file is in one of the specified
       boundary rectangles and don't include if it isn't.  Only do
       this if the user specified some boundary rectangles
    --------------------------------------------------------------*/
    if(tilenum[0] != -1)
    {
        br_id_flag = FALSE;
        for(i = 0; tilenum[i] != -1; i++)
        {
            if(ff->brect_record == tilenum[i])
            {
                br_id_flag = TRUE;
                break;
            }
        }
        if(!br_id_flag)
        {
            ff->row = NOTAFRAMEFILE;
            ff->column = NOTAFRAMEFILE;
            ff->fnum = NOTAFRAMEFILE;
            return(E_SUCC);
        }
    }

    /* get the resolution from the frame file filename extension 
    -------------------------------------------------------------*/
    ff->res = ff->filename[10] - '0';

    /* reorder filename storing it in increasing magnitude in fname array 
    ---------------------------------------------------------------------*/
    ctemp = fname[5];
    fname[5]=fname[0];
    fname[0]=ctemp;
    ctemp = fname[4];
    fname[4]=fname[1];
    fname[1]=ctemp;
    ctemp = fname[3];
    fname[3]=fname[2];
    fname[2]=ctemp; 
   
    /* calculate the decimal value of the 6 alpha-numeric number that
       represents the frame number in its zone the number is base 34.
    ------------------------------------------------------------------*/
    for (i = 0; i < 6; i++)
        value += get_val(fname[i]) * pow(34,i);
    
    nrz = (long) value;
    
    /* Get Rz the number of rows in this zone 
    -----------------------------------------*/   
    Rz = get_rows(ff->zone,ff->res);

    /* Get Cz the number of columNs in this zone
    ---------------------------------------------*/
    Cz = get_columns(ff->zone,ff->res);

    if(Rz < 0 || Cz < 0)
    {
        ff->row = NOTAFRAMEFILE;
        ff->column = NOTAFRAMEFILE;
        ff->fnum =  NOTAFRAMEFILE;
        c_lasmsg(FALSE,"cib2las-locate",
            "Error calculating frame file %s location",ff->filename);
        return (E_FAIL);
    }
 
    /* calculate nrz, the maximnum number of frame files in zone
    -------------------------------------------------------------*/
    nrzMAX = Rz * Cz - 1; 

    if(nrz > nrzMAX)
    {
        ff->row = NOTAFRAMEFILE;
        ff->column = NOTAFRAMEFILE;
        ff->fnum =  NOTAFRAMEFILE;
        c_lasmsg(FALSE,"cib2las-locate",
            "Error calculating frame file %s location",ff->filename);
        return (E_FAIL);
    }
    /* calculate rFz, the frame file row in the zone 
    ------------------------------------------------*/
    rFz =  (nrz/Cz); 

    /* calcualte cFz, the frame file column in the zone 
    ---------------------------------------------------*/
    cFz =  nrz - (rFz * Cz);

    ff->column = cFz;
    ff->row = rFz;
    ff->fnum = nrz;

    return (E_SUCC);
}

/*****************************************************************************
NAME:       get_val  

PURPOSE:    return the integer value for a given character as specified by
            MIL-PRF-89041 Frame  naming convention Section A.3.6

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       --------------------
  1.0      B. Gross        5/00            CSB          Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS
   
DESC:       Return the value of the letter/number from frame file filename.  
            the numbers start at 0, to through 9, and then A - Z, excluding
            I and O, which could be mistook for 1 (one) and 0 (zero).

ALGORITHM REFERENCES:  
    
        MIL-PRF-89041 Performance Specification
        Controlled Image Base

RETURNS:  The decimal value of the character passed in as specified in 
          MIL-PRF-89041 Appendix

*****************************************************************************/
int get_val(char ch) 
{
    if( isdigit((int)ch) )
        return( ch- '0' );
    switch(toupper(ch)){
        case 'A': return 10;
        case 'B': return 11;
        case 'C': return 12;
        case 'D': return 13;
        case 'E': return 14;
        case 'F': return 15;
        case 'G': return 16;
        case 'H': return 17;
        case 'J': return 18;
        case 'K': return 19;
        case 'L': return 20;
        case 'M': return 21;
        case 'N': return 22;
        case 'P': return 23;
        case 'Q': return 24;
        case 'R': return 25;
        case 'S': return 26;
        case 'T': return 27;
        case 'U': return 28;
        case 'V': return 29;
        case 'W': return 30;
        case 'X': return 31;
        case 'Y': return 32;
        case 'Z': return 33;
        default:  return -100;
        }
}
/*****************************************************************************
NAME: get_columns


PURPOSE:    Lookup number of frame file columns in a zone at the indicated
            resolution 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        5/00            CSB         Original Development 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

RETURNS:  The number of columns in the CIB zone passed in (zone), for the 
          resolution passed in (res).

ALGORITHM REFERENCES:  
    
    MIL-PRF-89041 Performance Specification
    Controlled Image Base
              
******************************************************************************/
long get_columns
(
    char zone,   /* I:  zone frame file contained in              */
    int res      /* I:  resolution of frame file e.g. 5m, 10m etc. */
)
{   
    if (res == FIVE_METER)
    {
        switch(toupper(zone)){
            case '1': 
            case 'A': return 4814;
            case '2': 
            case 'B': return 3940;
            case '3': 
            case 'C': return 3200;
            case '4': 
            case 'D': return 2594;
            case '5': 
            case 'E': return 2127;
            case '6': 
            case 'F': return 1787;
            case '7': 
            case 'G': return 1434;
            case '8': 
            case 'H': return 1074;
            case '9': 
            case 'J': return 291; /* na for polar zones */
            }
     }
     else if (res == TEN_METER)
     {
        switch(toupper(zone)){
            case '1':  
            case 'A': return 2407;
            case '2': 
            case 'B': return 1970;
            case '3': 
            case 'C': return 1600;
            case '4': 
            case 'D': return 1297;
            case '5': 
            case 'E': return 1064;
            case '6': 
            case 'F': return 894; 
            case '7': 
            case 'G': return 717;
            case '8': 
            case 'H': return 537;
            case '9':   
            case 'J': return 147; /* na for polar zones */
            }
      }
return  UNSUPPORTED_RES;  /* resolution not supported */
}
/****************************************************************************
NAME:     get_rows()


PURPOSE:  Lookup the number of frame file rows in zone at the indicated
          resolution 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE   CODE/CONT       REASON
-------    ----------      ----   ---------       ------
  1.0      B. Gross        5/00   CSB             original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  LAS

RETURNS:  The number of rows in the CIB zone passed in (zone), for the 
          resolution passed in (res).

ALGORITHM REFERENCES:  
    
    MIL-PRF-89041 Performance Specification
    Controlled Image Base
              
*****************************************************************************/
long get_rows
(
    char zone,   /* I:  zone frame file contained in                        */
    int res      /* I:  resolution of frame file e.g. 5m, 10m etc.          */
)
{
    if (res == FIVE_METER)
    {
        switch(toupper(zone)){
            case '1': 
            case 'A': return 464L;
            case '2': 
            case 'B': return 233L;
            case '3': 
            case 'C': return 116L;
            case '4': 
            case 'D': return 117L;
            case '5': 
            case 'E': return 59L;
            case '6': 
            case 'F': return 59L;
            case '7': 
            case 'G': return 59L;
            case '8': 
            case 'H': return 59L;
            case '9': 
            case 'J': return 291L;
            }
    }
    else if (res == TEN_METER)
    {
        switch(toupper(zone)){
            case '1':  
            case 'A': return 232L;
            case '2': 
            case 'B': return 117L;
            case '3': 
            case 'C': return 59L;
            case '4': 
            case 'D': return 59L;
            case '5': 
            case 'E': return 30L;
            case '6': 
            case 'F': return 30L; 
            case '7': 
            case 'G': return 30L;
            case '8': 
            case 'H': return 30L;
            case '9':   
            case 'J': return 147L;
            }
    }
    /* resolution not supported
    ---------------------------*/
    return UNSUPPORTED_RES; 
}
