/*--------------------------------------------------------------------------
FILENAME: mount_cd.c

ROUTINES: mount_cd(),
          unmount_cd()

PURPOSE:
        Routines to mount and unmount a CD

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
 1.0         B. Gross       06/00       Original development - taken mostly
                                        from mount_media.c in ndfin

MOUNT_CD:
INPUTS:
        vblk - TAE parameter block pointer
        comment - operator messages for mounting CD 


OUTPUTS:
        drive - Drive the CD was mounted on
        device - Device the CD was mounted on (mount point)
        mounted - CD mounted flag (1=mounted, 0=not mounted)
                  
ALGORITHM:
        Open the tape/CD drive
        Set the mounted flag
        Extract the drive and device name
        Set the mounted flag, drive, and device names in the TAE block
            so the drive can be unmounted if a failure occurs
        

UNMOUNT_CD:
INPUTS:
        vblk - TAE parameter block pointer
        drive - Drive to unmount
        
OUTPUTS:
        mounted - mounted flag (1=mounted, 0=not mounted)

ALGORITHM:
        Reopen the drive without rewinding to get the unit number
        Close the drive, taking it offline
        Clear the mounted flag and update the TAE block

RETURNS: mount_cd, unmount_cd 
         return E_SUCC if successful, E_FAIL otherwise
--------------------------------------------------------------------------*/
#include <string.h> /* string function prototypes */
#include <unistd.h> /* gethostname prototype */
#include "las.h"    /* las header file */
#include "tapeio.h" /* tapeio header file */
#include "util.h"   /* c_up2low prototype */
#include "mount_cd.h" /* prototypes for this module */

int  mount_cd
(
    struct PARBLK *vblk,   /* I/O: TAE parameter block */
    char comment[CMLEN],   /* I: mount messages */
    char *drive,           /* O: drive CD was mounted on */
    char *device,          /* O: device CD was mounted on */
    int *mounted           /* O: CD mounted flag */
)
{
    char density[5] = "0"; /* Density of mounted tape                       */
    char *string_ptrs[1];  /* used to return variables to tae               */
    char tape_id[CMLEN];   /* id of tape or CD mounted                      */
    int  status;           /* status returned by tapeio routines            */ 
    long access = TREAD;   /* Indicates tape access mode                    */
    long mode   = HANDLE;  /* Let tapeio handle tape errors                 */
    long rewopt;           /* Rewind tape upon close                        */
    long skip = 0;         /* Files to skip upon tape open                  */
    long unit;             /* Tape unit number                              */
    
    /* issue a mount request */

    /* c_tpopen gets an error if you attempt to rewind
       a CD on open, so don't rewind the CD on open
    ---------------------------------------------------*/
    rewopt = NOREW;
    status = c_tpopen(&unit, &skip, &mode, &rewopt, density, &access, 
                      comment, tape_id);
    if (status != TAPE_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-cd", "Error encountered mounting CD ");
        return (E_FAIL);
    }
    
    /* set the mounted flag
    -----------------------*/
    *mounted = TRUE;
    
    /* get the drive name and device
    ---------------------------------*/
    strcpy(drive, comment);
    
    /* If the DRIVE doesn't have an @ sign in it then this is a 
       local CD drive.  If the DRIVE does have an @ sign then this 
       is a remote drive.  If this is a local DRIVE, then we need 
       to get the hostname and compute the DRIVE from hostname + 
       @ + DRIVE.
       -------------------------------------------------------------*/
    
    /* create the device name for the CD drive
    ------------------------------------------*/
    if (strchr(drive,'@') == NULL)
    {   /* This is a local drive */
        char lhost[CMLEN];
        long length;
        
        if (gethostname(lhost,CMLEN) != 0)
        {
            c_lasmsg(FALSE,"cib2las-hostname","Error reading the hostname");
            return (E_FAIL);
        }
        length = (long)strlen(lhost);
        c_up2low(lhost,&length);
        sprintf (device, "/%s@%s/", lhost, drive);
    }
    else
    {
        /* This is a remote drive
        -------------------------*/
        sprintf (device, "/%s/", drive);
    }

    /* return the mounted flag, drive, and device right away so they
       can be unmounted in the pdf if needed
    -----------------------------------------------------------------*/
    if (SUCCESS != q_intg(vblk,"MOUNTED",1,mounted,P_UPDATE))
    {
        c_lasmsg(FALSE,"cib2las-tae","Error passing MOUNTED to TAE");
        return (E_FAIL);
    }
    string_ptrs[0] = drive;
    if (q_string(vblk,"DRIVE",1,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_lasmsg(FALSE,"cib2las-tae","Error passing DRIVE to TAE");
        return (E_FAIL);
    }
    string_ptrs[0] = device;
    if (q_string(vblk,"DEVICE",1,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_lasmsg(FALSE,"cib2las-tae","Error passing DEVICE to TAE");
        return (E_FAIL);
    }

    if (q_out(vblk) != SUCCESS)
    {
        c_lasmsg(FALSE,"cib2las-tae","Error passing variables to TAE");
        return (E_FAIL);
    }
        
    return (E_SUCC);
}


int unmount_cd
(
    struct PARBLK *vblk, /* I/O: TAE parameter block                        */
    char *drive,         /* I: drive to unmount CD                          */
    int *mounted         /* O: CD mounted flag                              */
)
{
    char density[5] = "0"; /* Density of mounted tape                       */
    /* LAS constants */
    long zero = 0;         /* Constant of zero                              */
    long unit;             /* Tape unit number                              */
    long skip = 0;         /* Files to skip upon tape open                  */
    long rewopt;           /* Rewind option                                 */
    long mode   = HANDLE;  /* Let tapeio handle tape errors                 */
    long access = TREAD;   /* Indicates tape access mode                    */

    /* reopen the drive to get the unit for the close call
    ------------------------------------------------------*/
    rewopt = NOREW;
    if (c_tpreop(&unit, &skip, &mode, &rewopt, density, &access, drive) 
            != TAPE_SUCC) 
    {
        c_lasmsg(FALSE,"cib2las-cd","Error encountered un-mounting CD");
        return (E_FAIL);
    }

    /* take CD offline
    ------------------*/
    rewopt = REWOFFL;
    if (c_tpclos(&unit, &mode, &rewopt, &zero) != TAPE_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-cd","Error encountered closing CD ");
        return (E_FAIL);
    }

    /* update the state of the mounted flag and immediately write it to the
       tae block in case a failure is encountered later
    -----------------------------------------------------------------------*/
    *mounted = FALSE;
    
    if ( q_intg(vblk,"MOUNTED",1,mounted,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FALSE,"cib2las-tae","Error passing MOUNTED to TAE");
        return (E_FAIL);
    }

    if (q_out(vblk) != SUCCESS)
    {
        c_lasmsg(FALSE,"cib2las-tae","Error passing variables to TAE");
        return (E_FAIL);
    }

    return (E_SUCC);
}
