/*****************************************************************************
FILENAME:  nitfaccess.c

FUNCTION(S): loc_rpf_header()

PURPOSE: Access NIFT formatted portion of CIB files 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
   1.0      B. Gross        2/00            CSB         original development 

   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCES: 

    MIL-STD-2500B Dept. of Defense Interface Standard
    National Imagery Transmission Format Version 2.1

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "nitfaccess.h"

/* loc_rpf_header - locate RPF Header in User Defined Header
                    Data Section of NITF Header. 
   
   Positions file pointer to the User Defined Header
   Data secion of the NITF formatted RFP Table of Contents file
   When formatted into NITF, the RPF Table of contents files is
   placed in the UDHD section of the NITF Header
   
----------------------------------------------------------------------------*/

int loc_rpf_header
(
    FILE *ntfimg    /* I: file pointer to a NITF formatted file             */
)
{
        char field[81];  /* temporary storage for NITF field read in */
    
    /* move past following fields,
       until we get to the file an image header length
   
    field abrev. # bytes   description
    -------------------------------------------------------
    FHDR           9      File type and version
    CLEVEL         2      Complexity level
        STYPE          4      System type
        OSTAID        10      Originating Station ID (OSTAID) 
        FDT           14      File Date and Time
        FTITLE        80      File Title
        FSCLAS         1      File Security Classification
        FSCODE        40      File Codewords
        FSCTLN        40      File Control and Handling
    --------------------------------------------------------
                 200  bytes                                                 */
                
    if(fseek(ntfimg,200L,SEEK_SET) != 0)
        return (E_FAIL);

        if(fread(field,1,6,ntfimg) != 6)         /* FSDWNG */
        return (E_FAIL);

    if(strncmp(field,"999998", 6) == 0)      /* FSDEVT */
        {
        if(fseek(ntfimg,40,SEEK_CUR) !=0)
            return (E_FAIL);                                 
    }
    /* move past following fields,
       until we get to the file an image header length
   
    field abrev. # bytes   description
    -------------------------------------------------------   
        FSREL         40      File Releasing Instructions
        FSCAUT        20      File Classification Authority
        FSCTLN        20      File Security Control Control Number
    FSCOP          5      File copy number 
    FSCPYS         5      File number of copies
    ENCRYP         1      Encryption
    ONAME         27      Originator's Name
    OPHONE        18      Originator's Phone Number
    FL            12      File length
    HL             6      Header Length
    --------------------------------------------------------
                 154 bytes
                    
    ------------------------------------------------------------------------*/
  
    if(fseek(ntfimg,154L,SEEK_CUR) != 0)
        return (E_FAIL);
        
    /* read NUMI: Number of Images
    ------------------------------*/
    if(fread(field,1,3,ntfimg) != 3)
        return (E_FAIL);

    if(fseek(ntfimg,16 * atoi(field),SEEK_CUR) != 0)
        return (E_FAIL);
    
    /* read NUMS : Numer of Graphics
    --------------------------------*/
    if(fread(field,1,3,ntfimg) != 3)
        return (E_FAIL);

    if(fseek(ntfimg,10 * atoi(field),SEEK_CUR) != 0)
       return (E_FAIL);

    /* read NUML: 
    -------------*/
    if(fread(field,1,3,ntfimg) != 3)
        return (E_FAIL);
    
    if(fseek(ntfimg,7 * atoi(field),SEEK_CUR) != 0)
        return (E_FAIL);
    
    /* read NUMT: Number of Text Files
    ----------------------------------*/
    if(fread(field,1,3,ntfimg) != 3)
        return (E_FAIL);

    if(fseek(ntfimg,9 * atoi(field),SEEK_CUR) != 0)
       return (E_FAIL);
    
    /* read NUMDES: Number of Data Extension Segments
    --------------------------------------------------*/
    if(fread(field,1,3,ntfimg) != 3)
        return (E_FAIL);

    if(fseek(ntfimg,13 * atoi(field),SEEK_CUR) != 0)
        return (E_FAIL);
        
    /* read NUMRES: Number of Reserved Data Extension Segments
    -----------------------------------------------------------*/       
    if(fread(field,1,3,ntfimg) != 3)
       return (E_FAIL);
    
    if(fseek(ntfimg,11 * atoi(field),SEEK_CUR) != 0)
        return (E_FAIL);

    /* move past UDHDL: User Defined Header Data Length
    ---------------------------------------------------*/
    if(fseek(ntfimg,5,SEEK_CUR) != 0)
        return (E_FAIL);
    /* move past UDHD: User Defined Header Data
    -------------------------------------------*/
    if(fseek(ntfimg,3,SEEK_CUR) != 0)
       return (E_FAIL);

    /* move past RETAG
    ------------------*/
    if(fseek(ntfimg,6,SEEK_CUR) != 0)
        return (E_FAIL);

    /* move past REL
    ----------------*/
    if(fseek(ntfimg,5,SEEK_CUR) != 0)
        return (E_FAIL);

    return (E_SUCC);
}
