/****************************************************************************
FILENAME:    parse_locations.c

FUNCTION(S): parse_locations()
  

PURPOSE: This function is used to find the file locations for necessary 
         sections of the CIB RPF Table of Contents (TOC) files.  The Table of 
         Contents file contains a locations section that lists the file
         locations of each of the remaining sections of the TOC file.  The
         desired sections are passed in by filling their id into the passed
         array.  

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE   CODE/CONT    REASON
-------    ----------      ----   ---------    ------
  1.0      B. Gross        5/00   CSB       Original Development
                                            (most taken from sgibrowse)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCES:  none

PRE:    toc must point to a CIB Table of Contents file "A.TOC".  *locs must 
        point to a LOCATION structure whose id member contains the id of the
        sections you are looking for.  count must contain the number of
        sections needed, as indicated by the id member of *locs.

POST:   The physical location in the CIB TOC for each section id 
        location is filled in.

ALGORITHM REFERENCES:  

    MIL-STD-2411 Military Standard
    Raster Product Format        

    MIL-STD-2411-1 Military Standard
    Registered Data Value for Raster Product Format

*****************************************************************************/
#include<stdio.h>
#include "cib2las.h"

int parse_locations
(
    FILE *toc,         /* I: file pointer to CIB RPF TOC file               */
    int count,         /* I: how many sections we're trying to fill in
                             the locations for                              */
    char *filename,    /* I: filename of file reading from                  */
    LOCATION *locs     /* I/O: pointer to array of LOCATIONS structures.    */ 
)
{
    int         i, j;        /* counter variables                            */
    unsigned short      n;   /* number of sections in file, read from TOC    */
    unsigned short      id;  /* section id in TOC read from file             */
    unsigned int phys_index; /* physical address of section                  */
 
  
    /* Initialize indices so we can later tell if they weren't found
    ----------------------------------------------------------------*/
    for (j = 0; j < count; j++)
        locs[j].phys_index = FALSE;

    /* skip location section length (2 bytes) and
       component location table offset (4 bytes)
    ----------------------------------------------*/
    if(fseek(toc,6,SEEK_CUR) != 0)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
            "Error moving file pointer in %s",filename);
        return (E_FAIL);
    } 

    /* Read the number sections: # of section location records
    -----------------------------------------------------------*/
    if(fread(&n,2,1,toc)!= 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
        return (E_FAIL);
    } 
    swap_endian((void *)&n,2);

    /* skip location record length (2 bytes) and
       component aggregate length (4 bytes)
    ---------------------------------------------*/
    if(fseek(toc,6,SEEK_CUR) != 0)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
            "Error moving file pointer in %s",filename);
        return (E_FAIL);
    } 

    /* Now go find the ones we want
    -------------------------------*/
    for (i = 0; i < n; i++)
    {
        /* read the component id
        ------------------------*/
        if(fread(&id, 2, 1, toc)!= 1)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                        "Error reading from %s",filename);
            return (E_FAIL);
        } 
        swap_endian((void *)&id,2);

        /* skip component length
        ------------------------*/
        if(fseek(toc,4,SEEK_CUR) != 0)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error moving file pointer in %s",filename);
            return (E_FAIL);
        } 

        /* read the component location and assign it to the locs structure
           whose id matches the current component id
        ------------------------------------------------------------------*/
        if(fread(&phys_index,4,1,toc)!= 1)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                        "Error reading from %s",filename);
            return (E_FAIL);
        } 
        swap_endian((void *)&phys_index,4);
    
        for (j = 0; j < count; j++)
        {
            if (locs[j].id == id)
                locs[j].phys_index = (long)phys_index;
        }
    }  
    return (E_SUCC);
}   
