/*****************************************************************************
FILENAME:       read_frame.c

FUNCTIONS:      read_frame()
                find_rpf_hdr()
                get_location_address()
                find_loc()
                get_loc()
                get_cov()
                get_comphdr()
                get_comptbl()
                get_colhdr()
                get_colmap()
                get_imghdr()
                get_imgdsp()
                get_imgmask()
                get_colcon()
                get_imgpix()
                get_uimgpix()
                get_atthdr()
                get_atttbl()
                bld_img()
                bld_uimg()
                wrt_img()
                read_frame_coverage()
                read_frame_attributes()
          * remainder for debug only *
                prt_hdr()
                prt_loc()
                prt_cov()
                prt_com()
                prt_col()
                prt_img()
                prt_att()
                prt_new()
                prt_nitfhdr()
                get_nitfhdr()
                get_hdr()
                get_attcov()
                get_newhdr()
                get_newtbl()

PURPOSE:   Reads a Controlled Image Base (CIB) frame file

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        2/00            CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS
ALGORITHM DESCRIPTION:
    
    open frame file specified
    find location of header
    read frame file header
    find file locations section of frame file header 
    for each frame file record
       read in frame file information
    end for
    set transparent pixel value to user specified fill value
    if image compressed
      decompress and build frame file image 
    else 
      build frame file image
      
ALGORITHM REFERENCES:  
    
    MIL-PRF-89041 Performance Specification
    Controlled Image Base

    MIL-STD-2500B Dept. of Defense Interface Standard
    National Imagery Transmission Format Version 2.1

    MIL-STD-2411 Military Standard
    Raster Product Format

    MIL-STD-2411-1 Military Standard
    Registered Data Value for Raster Product Format

RETURNS:    A flat buffer containing the frame file image, E_SUCC on 
            successful completion, otherwise E_FAIL
            
******************************************************************************/
#include<string.h>
#include<stdio.h>
#include<ctype.h>
#include"las.h"
#include"worgen.h"
#include"util.h"
#include"cib2las.h"
#include"cibframe.h"
#include"locate_frame.h"
#include"nitfaccess.h"


int read_frame
(
    char *fname,            /* I: path to and name of frame file to read */
    char fillval,           /* I: specified image fill value             */
    unsigned char *obuffer  /* O: output (array) buffer for frame data   */
)
{
    char frmname[80];       /* local copy of frame file name             */
    int  i;
    int  compressed = FALSE;/* is image compressed                       */
    FILE *ffptr;                    /* frame file pointer                        */
    struct CIBHeader    cibhdr; /* structure to hold header info.        */
    struct CIBLocation  cibloc; /* structure to hold location info.      */
    struct CIBCoverage  cibcov; /* structure to hold coverage info.      */
    struct CIBCompress  cibcom; /* structure to hold compression info.   */
    struct CIBColor     cibcol; /* structure to hold color lut info.     */
    struct CIBImage         cibimg; /* structure to hold image infol         */
    struct CIBAttribute cibatt; /* structure to hold attribute info.     */
#ifdef DEBUG
    struct CIBUpdate       cibnew; /* structure to hold new/update info.    */
#endif /* DEBUG */
    
    /* if couldn't open file, return failure
    -----------------------------------------*/
    if((ffptr=fopen(fname,"r"))== NULL)
        return (E_FAIL);

    /*  find and read frame file header section 
    ---------------------------------------*/
    if(loc_rpf_header(ffptr)!= E_SUCC)
    {
        fclose(ffptr);
        c_lasmsg(FALSE,"cib2las-read","Error locating RPF header %s"
            ,frmname);
        return (E_FAIL);
    }
    
    if(get_location_address(ffptr,&cibhdr)!= E_SUCC)
    {
        fclose(ffptr);
        c_lasmsg(FALSE,"cib2las-read","Error reading RPF header %s location",
            fname);
        return (E_FAIL);
    }

    /*  find and read frame file [location] section 
    ------------------------------------------------*/
    if(find_loc(ffptr,&cibhdr)!= E_SUCC)
     {
        fclose(ffptr);
        c_lasmsg(FALSE,"cib2las-read","Error finding location section of %s",
            frmname);
        return (E_FAIL);
    }
    if(get_loc(ffptr,&cibloc)!= E_SUCC)
    {
        fclose(ffptr);        
        c_lasmsg(FALSE,"cib2las-read","Error reading location section of %s",
            frmname);
        return (E_FAIL);
    }

   /*  compid as listed below is obtained from MIL-STD-2411-1 
      (Registered Data Values for RPF) Section 5.1 page 6   
   -----------------------------------------------------------*/
    for(i = 0; i < cibloc.numrec; i++)
    {
        if( fseek(ffptr,cibloc.comptable[i].location,SEEK_SET) != 0)
        {
            c_lasmsg(FALSE, "cib2las-read","Error moving file pointer");
            return (E_FAIL);
        }

        switch ((int)cibloc.comptable[i].compid)
        {
                
                 /* get frame file coverage
                 --------------------------*/
            case LOC_COVERAGE_SECTION: 
                if(get_cov(ffptr,&cibcov) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                } 
                                
                        break;
                
                /* get compression header info 
                ------------------------------*/
            case LOC_COMPRESSION_SECTION: 
                if(get_comphdr(ffptr,&cibcom) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                compressed = TRUE;
                break;
                
                /* get compression tables
                --------------------------*/
            case LOC_COMPRESSION_LOOKUP_SUBSECTION: 
                if(get_comptbl(ffptr,&cibcom) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /* get color table header
                -------------------------*/
            case LOC_COLORGRAY_SECTION_SUBHEADER: 
                if(get_colhdr(ffptr,&cibcol)!= E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /*  get color map
                -----------------*/
            case LOC_COLORMAP_SUBSECTION:
                if(get_colmap(ffptr,&cibcol) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                } 
                break;
                
                /*  get image header
                --------------------*/
            case LOC_IMAGE_DESCR_SUBHEADER:
                if(get_imghdr(ffptr,&cibimg) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;
            case LOC_IMAGE_DISPLAY_PARAM_SUBHEADER:
                if(get_imgdsp(ffptr,&cibimg) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /*  get image mask
                ------------------*/
            case LOC_MASK_SUBSECTION: 
                if(get_imgmask(ffptr,&cibimg) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /* get color table info
               -------------------------*/
            case LOC_COLOR_CONVERTER_SUBSECTION:
                if(get_colcon(ffptr,&cibcol) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /* get compressed image pixel values (conversion table)
                -------------------------------------------------------*/
            case LOC_SPATIAL_DATA_SUBSECTION:
                if(compressed == TRUE)
                {
                    if(get_imgpix(ffptr,&cibimg)!= E_SUCC)
                    {
                        fclose(ffptr);
                        c_lasmsg(FALSE,"cib2las-read",
                            "Error Reading Frame File %s",frmname);
                        return (E_FAIL);
                    }
                }
                else
                {
                    if(get_uimgpix(ffptr,&cibimg)!= E_SUCC)
                    {
                        fclose(ffptr);
                        c_lasmsg(FALSE,"cib2las-read",
                            "Error Reading Frame File %s",frmname);
                        return (E_FAIL);
                    }
                }
                        break;

                /* get attributes' header for frame file 
                ----------------------------------------*/
            case LOC_ATTRIBUTE_SECTION_SUBHEADER:
                if(get_atthdr(ffptr,&cibatt) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /* get attributes list (table) for frame file
                --------------------------------------*/
            case LOC_ATTRIBUTE_SUBSECTION:
                if(get_atttbl(ffptr,&cibatt) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /* get expicit areal coverage table from frame file 
                ----------------------------------------------------*/
#ifdef DEBUG  /* not used/needed in CIB2LAS */
            case LOC_EXPLICIT_AREAL_TABLE:
                if(get_attcov(ffptr,&cibatt)!= E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /* get replace/update header from frame file
                ---------------------------------------------*/
            case LOC_REPLACE_UPDATE_SECTION_SUBHEADER:
                if(get_newhdr(ffptr,&cibnew)!= E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;

                /* get replace/update table from frame file
                ---------------------------------------------*/
            case LOC_REPLACE_UPDATE_TABLE:
                if(get_newtbl(ffptr,&cibnew) != E_SUCC)
                {
                    fclose(ffptr);
                    c_lasmsg(FALSE,"cib2las-read",
                        "Error Reading Frame File %s",frmname);
                    return (E_FAIL);
                }
                break;
#endif /* DEBUG */
            default:    break;
        }

    }
    fclose(ffptr);

    /* update color maps to use fill value for transparent pixel
    ------------------------------------------------------------*/

    /* look in each color Look Up Table, 
           If there are 217 items,
               set last item (transparent pixel value)
               to operator specified fill value              */
               

    for(i = 0; i < cibcol.numLUT; i++)
    {
        /* if there are 217 records present (then there are transparent
           pixels in this frame file)
        ----------------------------------------------------------------*/  
        if(cibcol.COffTbl[i].numrec == 217)
        {
            /* set transparent pixel value to operator specified fill value 
            ---------------------------------------------------------------*/
            cibcol.CTable[i][216] = fillval;
        }
    }

    /* if image is compressed, uncompress and build image
    ------------------------------------------------------*/
    if (compressed == TRUE)
        bld_img(&cibcom,&cibcol,&cibimg);
    else
        bld_uimg(&cibcol,&cibimg);
    /* write out image to buffer obuffer
    ------------------------------------*/
    wrt_img(&cibimg,obuffer ); 
    
    return (E_SUCC);
}
/* ################### END OF MAIN READ FILE ROUTINES###################### */  


/* ################### SUPPORT FUNCTION DEFINITIONS ####################### */

/* find RPF file header section, places file pointer (ffptr) at RPF section
   of file                                                                  

ALGORITHM REFERENCES:

    MIL-STD-2411 Military Standard
    Raster Product Format

----------------------------------------------------------------------------*/
  
int find_rpf_hdr
(
    FILE        *ffptr  /* I: FILE pointer to CIB image frame file              */
)
{
    if(fseek(ffptr,0,SEEK_SET) != 0)
        return (E_FAIL);

    if(loc_rpf_header(ffptr) != E_SUCC)
        return (E_FAIL);    
    return (E_SUCC);
}

/***************************************************************************
FUNCTION NAME:    get_location_address()

PURPOSE:  Get the address of the locations section of the RPF header located
   in the RPF Table of Contents (TOC) file.

ALGORITHM REFERENCES:

    MIL-STD-2411 Military Standard
    Raster Product Format

RETURNS:

    Returns the [location] section address (starting byte in file) in struct
    cibhdr->location field
*****************************************************************************/
int get_location_address
(
    FILE *ffptr,               /* I: FILE pointer to CIB image frame file   */
    struct CIBHeader *cibhdr   /* O: contains TOC information               */ 
)
{
    if(fread(&cibhdr->endianness,1,1,ffptr) != 1)
        return (E_FAIL);

    /* check for endianess mismatch 
    -------------------------------*/
    check_endian(cibhdr->endianness);
    
    /* move to location address byte
    ---------------------------------*/
    if(fseek(ffptr,43,SEEK_CUR) != 0) 
        return (E_FAIL);
    if(fread(&cibhdr->location,4,1,ffptr) != 1)  
        return (E_FAIL);
    swap_endian(&cibhdr->location,4);
    return (E_SUCC);
}  

/***************************************************************************
FUNCTION NAME: find_loc()
PURPOSE:       Positions the file pointer to the [location] section of 
               frame file in CIB Volume Table of Contents RPF header.

ALGORITHM REFERENCES:

    MIL-STD-2411 Military Standard
        Raster Product Format
   
RETURNS:       Returns E_SUCC on success, E_FAIL otherwise.
*****************************************************************************/
int find_loc
(
  FILE                     *ffptr,  /* I: FILE pointer to CIB image frame file      */
  struct CIBHeader *cibhdr  /* I: contains file location of [location] 
                                  section of frame file                     */
)
{
    if(fseek(ffptr, cibhdr->location,SEEK_SET) != 0)
        return(E_FAIL);
    return (E_SUCC);
}

/***************************************************************************
FUNCTION NAME: get_loc()
PURPOSE:       Fills structure cibloc with information on the remaining
               sections of the RPF formatted header.

ALGORITHM REFERENCES:

    MIL-STD-2411 Military Standard
    Raster Product Format

RETURNS:   

   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_loc
(
    FILE  *ffptr,              /* I: FILE pointer to CIB image frame file   */
    struct CIBLocation *cibloc /* O:   structure to fill with frame file 
                                       location information                 */
)
{
    int i;

    if(fread(&cibloc->length,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibloc->tbloffset,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibloc->numrec,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibloc->reclength,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibloc->agglength,4,1,ffptr) != 1)  
        return (E_FAIL);

    /* read in each record
    -----------------------*/
    for(i = 0; i < cibloc->numrec; i++)
    {
        if(fread(&cibloc->comptable[i].compid,2,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibloc->comptable[i].length,4,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibloc->comptable[i].location,4,1,ffptr) != 1)  
            return (E_FAIL);
    }
    return (E_SUCC);
}

/***************************************************************************
FUNCTION NAME:  get_cov()

PURPOSE:  read in frame file coverage coverage information

ALGORITHM REFERENCES:  

    MIL-PRF-89041 Performance Specification
    Controlled Image Base

    MIL-STD-2411 Military Standard
    Raster Product Format

    MIL-STD-2411-1 Military Standard
    Registered Data Value for Raster Product Format

RETURNS

   Returns the corner coordinates of the frame file indicated by ffptr.
   Returns E_SUCC on success, E_FAIL otherwise.
   
*****************************************************************************/
int get_cov
(
    FILE *ffptr,               /* I: FILE pointer to CIB image frame file   */
    struct CIBCoverage *cibcov /* O: struct to store coverage frame file    */
)
{
    if(fread(&cibcov->ULLat,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->ULLong,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->LLLat,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->LLLong,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->URLat,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->URLong,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->LRLat,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->LRLong,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->NSRes,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->EWRes,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->LatRes,8,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcov->LongRes,8,1,ffptr) != 1)  
        return (E_FAIL);
    return(E_SUCC);
}

/* read in compression information into structure cibcom

   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_comphdr
(
  FILE  *ffptr,               /* I: FILE pointer to CIB image frame file    */
  struct CIBCompress *cibcom  /* O: struct to hold ff compress info         */
)
{
    if(fread( &cibcom->algoid,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread( &cibcom->numoff,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread( &cibcom->numparam,2,1,ffptr) != 1)  
        return (E_FAIL);
    return(E_SUCC);
}

/* read in compression table information into structure cibcom
  
   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_comptbl
(
    FILE *ffptr,               /* I: FILE pointer to CIB image frame file   */
    struct CIBCompress *cibcom /* O: struct to hold ff compress info        */
)
{
    int i,j,k;

    if(fread(&cibcom->LOTbloff,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcom->LOTblrec,2,1,ffptr) != 1)  
        return (E_FAIL);
    for(i=0 ;i < cibcom->numoff; i++)
    {
        if(fread(&cibcom->LOTbl[i].tblid,2,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcom->LOTbl[i].numrec,4,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcom->LOTbl[i].numval,2,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcom->LOTbl[i].bitlen,2,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcom->LOTbl[i].LToff,4,1,ffptr) != 1)  
            return (E_FAIL);
    }
    for (i = 0; i < cibcom->numoff; i++)
        for (j = 0; j < cibcom->LOTbl[i].numrec; j++)
            for (k = 0; k < cibcom->LOTbl[i].numval; k++)
                if(fread(&cibcom->vals[i][j][k],1,1,ffptr) != 1)  
                    return (E_FAIL);
    return(E_SUCC);
}

/*  read in number of color/grayscale records -> number of color/grayscale 
    lookup tables - minimum of one 
    
    read in the number of color conversion tables 
    
    Returns the number of the color lookup table (CLUT), the number of colors
    present, and the external CLUT filename.
    
    Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_colhdr
(
    FILE *ffptr,            /* I: FILE pointer to CIB image frame file      */
    struct CIBColor *cibcol /* O: struct to hold ff color table info        */
)
{
    if(fread(&cibcol->numLUT,1,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcol->numCON,1,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(cibcol->efname,1,12,ffptr) != 12)  
        return (E_FAIL);
    cibcol->efname[12] = 0;
    return (E_SUCC);
}

/* read in color map from frame file into structure cibcol

   Returns the color lookup table in the struct cibcol
   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_colmap
(
    FILE *ffptr,             /* I: FILE pointer to CIB image frame file     */
    struct CIBColor     *cibcol  /* O: struct to hold ff color table info       */
)
{
    int i;
    long j;
    long curr;
    
    curr = ftell(ffptr);
    if(fread(&cibcol->cmapoff,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibcol->moffreclen,2,1,ffptr) != 1)  
        return (E_FAIL); 
    for(i=0;i<cibcol->numLUT;i++)
    {
        if(fread(&cibcol->COffTbl[i].tblid,2,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcol->COffTbl[i].numrec,4,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcol->COffTbl[i].elemlen,1,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcol->COffTbl[i].histlen,2,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcol->COffTbl[i].grayoff,4,1,ffptr) != 1)  
            return (E_FAIL);
        if(fread(&cibcol->COffTbl[i].histoff,4,1,ffptr) != 1)  
            return (E_FAIL);
    }
    for(i=0;i<cibcol->numLUT;i++)
    {
      if(fseek(ffptr,(long)(curr+cibcol->COffTbl[i].grayoff),SEEK_SET) != 0)
      {
          return (E_FAIL);
      }
      j = (long) cibcol->COffTbl[i].numrec;
      /* j is number of record in this color look up table "i" 
      --------------------------------------------------------*/
      if(fread(&cibcol->CTable[i][0],1,j,ffptr) != j)
          return (E_FAIL);
    }
    for(i=0; i<cibcol->numLUT; i++)
    {
      if(cibcol->COffTbl[i].histlen != 0xFFFF)
      {
        if(fseek(ffptr,(long)(curr+cibcol->COffTbl[i].histoff),SEEK_SET) != 0)
        {
            return (E_FAIL);
        }
        j = (long)cibcol->COffTbl[i].numrec;
        if(fread(&cibcol->Histogram[i][0],4,j,ffptr) != j)
        {
            return (E_FAIL);
        }
      }
    }
  return(E_SUCC);
}

/* read frame file image header info. into structure cibimg

   Returns the image header from the frame file (indicated by ffptr) in
   the struct cibimg.
   
   Returns E_SUCC on success, E_FAIL otherwise.
   -------------------------------------------------------------------------*/
int get_imghdr
(
    FILE *ffptr,             /* I: FILE pointer to CIB image frame file     */
    struct CIBImage     *cibimg  /* O: struct to hold ff image info             */
)
{
    if(fread(&cibimg->numgrp,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->numstbl,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->numbtbl,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->numbline,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->EWsubfrm,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->NSsubfrm,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->colsubf,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->rowsubf,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->smaskoff,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->tmaskoff,4,1,ffptr) != 1)  
        return (E_FAIL);
    return(E_SUCC);
}

/* read frame file image data description info into structure cibimg
   Returns E_SUCC on success, E_FAIL otherwise.
   -------------------------------------------------------------------------*/
int get_imgdsp
(
    FILE *ffptr,            /* I: FILE pointer to CIB image frame file      */
    struct CIBImage     *cibimg /* O: struct to hold image info                 */
)
{

    int i;
    int nmask;
    unsigned long LNULL = 0xFFFFFFFF;

    if(fread(&cibimg->numrow,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->numcol,4,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->codelen,1,1,ffptr) != 1)  
        return (E_FAIL);
    nmask = cibimg->EWsubfrm * cibimg->NSsubfrm;
    if(cibimg->smaskoff == LNULL)
    {
        for(i = 0; i < nmask; i++)
        {
            cibimg->sframes[i] = i * 
                (cibimg->numrow * cibimg->numcol * cibimg->codelen) / 8;
        }
    }
    if(cibimg->tmaskoff == LNULL)
    {
        for(i = 0; i < nmask; i++)
            cibimg->tframes[i] = LNULL;
    }
    return(E_SUCC);
}

/* read image mask into structure cibimg.  The mask indicates which frame
   files exist, and which ones do not.

   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_imgmask
(
    FILE *ffptr,            /* I: FILE pointer to CIB image frame file      */
    struct CIBImage     *cibimg /* O: struct to hold image info                 */
)
{
    int     nmask,
                i;
    long    curr;
    unsigned long LNULL = 0xFFFFFFFF;

    nmask = cibimg->EWsubfrm * cibimg->NSsubfrm;
    curr = ftell(ffptr);
    if(fread(&cibimg->subrecl,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->tranrecl,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->tcodelen,2,1,ffptr) != 1)  
        return (E_FAIL);
    if(fread(&cibimg->tcode,1,1,ffptr) != 1)  
        return (E_FAIL);
    if((cibimg->subrecl != 0) && (cibimg->smaskoff != LNULL))
    {
        if(fseek(ffptr,(long)(curr+cibimg->smaskoff),SEEK_SET) != 0)
            return (E_FAIL);
        if(fread(cibimg->sframes,cibimg->subrecl,nmask,ffptr) != nmask)
            return (E_FAIL);
    }
    else
    {
        for(i = 0; i < nmask; i++)
        {
            cibimg->sframes[i] = i * (cibimg->numrow * cibimg->numcol
                * cibimg->codelen)/8;
        }
    }
  
    if((cibimg->tranrecl != 0) && (cibimg->tmaskoff != LNULL))
    {
        if(fseek(ffptr,(long)(curr+cibimg->tmaskoff),SEEK_SET) != 0)
            return (E_FAIL);        
        if(fread(cibimg->tframes,cibimg->tranrecl,nmask,ffptr) != nmask)
            return (E_FAIL);
    }
    else
    {
        for(i = 0; i < nmask; i++)
            cibimg->tframes[i] = LNULL;
    }
    return (E_SUCC);
}

/* Read color table info into structure cibcol.

   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_colcon
(
    FILE *ffptr,            /* I: FILE pointer to CIB image frame file      */
    struct CIBColor     *cibcol /* O: struct to hold color table info           */
)
{

    long  curr, j;
    
    curr = ftell(ffptr);
    if(fread(&cibcol->ccoffset,4,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibcol->coffreclen,2,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibcol->ccreclen,2,1,ffptr) != 1)
        return (E_FAIL);
    if(cibcol->numCON > 0)
    {
        if(fseek(ffptr,curr+(long)cibcol->ccoffset,SEEK_SET) != 0)
            return (E_FAIL);
        if(fread(&cibcol->ccoff.cctblid,2,1,ffptr) != 1)
            return (E_FAIL);
        if(fread(&cibcol->ccoff.nccrecs,4,1,ffptr) != 1)
            return (E_FAIL);
        if(fread(&cibcol->ccoff.cctbloff,4,1,ffptr) != 1)
            return (E_FAIL);
        if(fread(&cibcol->ccoff.srcoff,4,1,ffptr) != 1)
            return (E_FAIL);
        if(fread(&cibcol->ccoff.tgtoff,4,1,ffptr) != 1)
            return (E_FAIL);
    }
    if(cibcol->numCON > 0)
    {
        if(fseek(ffptr,curr+(long)cibcol->ccoff.cctbloff,SEEK_SET) != 0)
            return (E_FAIL);
        j = (long) cibcol->ccoff.nccrecs;
        if(fread(cibcol->CCTbl,4,j,ffptr) != j)
            return (E_FAIL);
    }
    return (E_SUCC);
}

/* Place compressed image pixel values into structure cibimg.

   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_imgpix
(
    FILE *ffptr,            /* I: FILE pointer to CIB image frame file      */
    struct CIBImage *cibimg /* O: struct to hold frame file image data      */
)  
{
    int         i, j, k, l, m;
    int         sfloc;                  /* index of current subframe */
    unsigned char codebuf[3];           /* VQ code buffer */
    unsigned short scode[3];
    unsigned long LNULL = 0xFFFFFFFF;
    long    curr;                       /* current section pointer */;

    curr = ftell(ffptr);
    for(i = 0; i < cibimg->NSsubfrm; i++)
    {
        for(j = 0; j < cibimg->EWsubfrm; j++)
        {
            sfloc = i * cibimg->EWsubfrm + j;
            if(cibimg->sframes[sfloc] != LNULL)
            {
                if(fseek(ffptr,curr+(long)cibimg->sframes[sfloc],SEEK_SET)
                    != 0)
                {
                    return (E_FAIL);
                }
                for(k = 0; k < cibimg->numrow; k++)
                {
                    for(l = 0; l < cibimg->numcol; l+= 2)
                    {
                        if(fread(codebuf,1,3,ffptr) != 3)
                            return (E_FAIL);
                                for(m = 0; m < 3; m++)
                        {
                                        scode[m] = (unsigned short)codebuf[m];
                                    cibimg->codes[sfloc][k][l] = 
                                (scode[0] << 4)|((scode[1] & 0x00F0) >> 4);
                                    cibimg->codes[sfloc][k][l+1] =
                                            ((scode[1] & 0x000F) << 8)| scode[2];
                        }
                            }
                }
            }
                else
            {
                    for(k = 0; k < cibimg->numrow; k++)
                {
                            for(l = 0; l < cibimg->numcol; l++)
                                cibimg->codes[sfloc][k][l] = 0x0FFF;
                }
            }
        }
    }
    return(E_SUCC);
}

/* Place un-compressed image pixel values into structure cibimg
----------------------------------------------------------------------------*/
int get_uimgpix
(
    FILE *ffptr,            /* I: FILE pointer to CIB image frame file      */
    struct CIBImage *cibimg /* O: struct to hold frame file image data      */
)
{
    int i, j, k, l;
    int sfloc;                  /* index of current subframe */
    long curr;                  /* current section pointer   */
    unsigned long LNULL = 0xFFFFFFFF;

    curr = ftell(ffptr);

    /* for each vertical row of subframes 
    -------------------------------------*/
    for(i = 0; i < cibimg->NSsubfrm; i++)
    {
      /* for each horizontal row of subframes 
      ----------------------------------------*/
      for(j = 0; j < cibimg->EWsubfrm; j++)
      {
          sfloc = i * cibimg->EWsubfrm + j;
          if(cibimg->sframes[sfloc] != LNULL)
          {
              if(fseek(ffptr,curr+(long)cibimg->sframes[sfloc],SEEK_SET) != 0)
              {
                  return (E_FAIL);
              }
              /* for each pixel in number of row in image
              -------------------------------------------*/    
              for(k = 0; k < cibimg->numrow; k++)
                  if(fread(cibimg->pixels[sfloc][k], 1,
                    cibimg->numcol,ffptr) != cibimg->numcol)
                  {
                      return (E_FAIL);
                  }  
          }
          else
          {
              for(k = 0; k < cibimg->numrow; k++)
                  for(l = 0 ; l < cibimg->numcol; l++) 
                      cibimg->pixels[sfloc][k][l] = 0;
          }
      }
  }
  return(E_SUCC);
}

/* Read attribute data header into structure cibatt

   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_atthdr
(
    FILE *ffptr,                 /* I: FILE pointer to CIB image frame file */
    struct CIBAttribute *cibatt  /* O: struct to hold ff attributes         */
)
{
    if(fread(&cibatt->numatt,2,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibatt->numcov,2,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibatt->tbloff,4,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibatt->offrecl,2,1,ffptr) != 1)
        return (E_FAIL);
    return (E_SUCC);
}

/* Read attributes into structure cibatt

   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int get_atttbl
(
    FILE *ffptr,                 /* I: FILE pointer to CIB image frame file */
    struct CIBAttribute *cibatt  /* O: struct to hold ff attributes         */
)
{

    int i;
    long  curr;

    curr = ftell(ffptr);
    if(fseek(ffptr,curr+(long)cibatt->tbloff,SEEK_SET) != 0)
        return (E_FAIL);

    for(i = 0; i < cibatt->numatt; i++)
    {
        if(fread(&cibatt->attoff[i].attid,2,1,ffptr) != 1)
            return (E_FAIL);
        if(fread(&cibatt->attoff[i].paramid,1,1,ffptr) != 1)
            return (E_FAIL);
        if(fread(&cibatt->attoff[i].covseq,1,1,ffptr) != 1)
            return (E_FAIL);
        if(fread(&cibatt->attoff[i].attroff,4,1,ffptr) != 1)
            return (E_FAIL);
    }
    for(i = 0; i < cibatt->numatt; i++)
    {
        if(fseek(ffptr,curr+(long)cibatt->attoff[i].attroff,SEEK_SET) != 0)
            return (E_FAIL);
        switch ((int)cibatt->attoff[i].attid ){
         
            case 1:     /* Currency Date
                    ----------------*/
            case 2:     /* Production Date
                    ------------------*/
            case 3:     /* Significant Date
                    -------------------*/
                    if(fread(cibatt->attribute[i].param.c,1,8,ffptr) != 8)
                        return (E_FAIL);
                    cibatt->attribute[i].param.c[8] = 0;
                    break;
            case 9:     /* Horizontal Absolute Accuracy 
                    -------------------------------*/
            case 11:/* Horizontal Relative Accuracy
                    -------------------------------*/
                    if(cibatt->attoff[i].paramid == 1)
                    {
                        if(fread(&cibatt->attribute[i].param.l,4,1,ffptr) != 1)
                            return (E_FAIL);
                    }
                    else if(cibatt->attoff[i].paramid == 2)
                    {
                        if(fread(&cibatt->attribute[i].param.s,2,1,ffptr) != 1)
                            return (E_FAIL);
                    }
                    break;
            case 22:/* Image Source 
                    ---------------*/
                    if(cibatt->attoff[i].paramid == 1)
                    {
                        if(fread(cibatt->attribute[i].param.c,1,12,ffptr)
                            != 12)
                            return (E_FAIL);
                        cibatt->attribute[i].param.c[12] = 0;
                    }
                    else if(cibatt->attoff[i].paramid == 2)
                    {
                        if(fread(&cibatt->attribute[i].param.l,4,1,ffptr) != 1)
                            return (E_FAIL);
                    }  
                    break;
            default:  break;  /* some attributes we don't care about and
                                 additional ones may be added, so ignore */
          }
    }
    return (E_SUCC);
}


/*  build compressed image i.e. uncompress image and store in CIBimage
    Returns CIB frame file image in struct cibimg
----------------------------------------------------------------------------*/
void bld_img
(
    struct CIBCompress   *cibcom, /* I:  CIB compression info               */
    struct CIBColor      *cibcol, /* I:  CIB color lookup table(s)          */
    struct CIBImage      *cibimg  /* O:  complete CIB image                 */
)
{
    int  i, j, k, l, m, n;
    int  sfloc;                   /* index of to a  subframe                */
    unsigned char tmpix;          /* temporary pixel variable               */

    for(i = 0; i < cibimg->NSsubfrm; i++)
    {
        for(j = 0; j < cibimg->EWsubfrm; j++)
        {
            sfloc = i * cibimg->EWsubfrm + j;
            for(k = 0; k < cibimg->numrow; k++)
            {
                for(l = 0; l < cibimg->numcol; l++)
                {
                    for(m = 0; m < 4; m++)
                    {
                        for( n = 0; n < 4; n++)
                        {
                            tmpix = 
                                cibcom->vals[m][cibimg->codes[sfloc][k][l]][n];
                            cibimg->pixels[sfloc][k * 4 + m][l * 4 + n] = 
                                cibcol->CTable[0][tmpix];
                        }
                    }
                }
            }
        }    
    }
}

/*  build uncompressed image in structure cibimg

    Returns CIB frame file image in struct cibimg
    ------------------------------------------------------------------------*/
void bld_uimg
(
    struct CIBColor  *cibcol, /* I:  color conversion table for frame file  */
    struct CIBImage  *cibimg  /* O:  frame file image stored in subframes   */
) 
{
    int  i, j, k, l;
    int  sfloc;               /* index to a subframe                        */
    unsigned char tmpix;      /* index into color table for a pixel         */

    for(i = 0; i < cibimg->NSsubfrm; i++)
    {
        for(j = 0; j < cibimg->EWsubfrm; j++)
        {
            sfloc = i * cibimg->EWsubfrm + j;
            for(k = 0; k < cibimg->numrow; k++)
            {
                /* perform color restoration
                ----------------------------*/      
                for(l = 0; l < cibimg->numcol; l++)
                {
                    tmpix = cibimg->pixels[sfloc][k][l];
                    cibimg->pixels[sfloc][k][l] = cibcol->CTable[0][tmpix];
                }
            }
        }
    }
}

/*  write frame file image to obuffer.
    each frame is made up of 6 x 6 subframes of 256 x 256 pixels. 
    
    Returns CIB frame file image in obuffer 
    ------------------------------------------------------------------------*/
void wrt_img
(
    struct CIBImage *cibimg, /* I: structure holding uncompressed 
                                   frame file image subframes               */
    unsigned char *obuffer   /* O: output buffer frame file 
                                   image is written to                      */
)
{
    int   i, j, k, l;
    int   sfloc;        /* index of subframe accessing in cibimg            */
    int   row, col;     /* frame row and column                             */
   
    for(i = 0; i < cibimg->NSsubfrm; i++)
    {
        for(j = 0; j < cibimg->EWsubfrm; j++)
        {
            sfloc = i * cibimg->EWsubfrm + j;
            for(k = 0; k < LINES_SUBFRAME; k++)
            { 
                row = i * LINES_SUBFRAME + k;
                for(l = 0; l < SAMPLES_SUBFRAME; l++)
                {
                    col = j * SAMPLES_SUBFRAME + l;
                    obuffer[ row * LINES_FRAME + col] = 
                        cibimg->pixels[sfloc][k][l];
                }
            }
        }
    }
}

/*  Read coverage of frame file whose name is passed in structure

    Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int read_frame_coverage
(
    FILE *ffptr,
    struct CIBCoverage *cibcov /*  O: coverage of frame file indicated      */
)
{
    int  i=0;
    struct CIBHeader cibhdr;
    struct CIBLocation cibloc;
             
    /* find and read frame file header section 
    -------------------------------------------*/
    if(loc_rpf_header(ffptr) != E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-read",
            "Error moving file pointer");
        return (E_FAIL);
    }
    
    if(get_location_address(ffptr,&cibhdr) != E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-read",
            "Error reading location address"); 
        return (E_FAIL);
    }

    /* find and read frame file location section 
    ---------------------------------------------*/
    if(find_loc(ffptr,&cibhdr)!= E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-read",
            "Error moving file pointer"); 
        return (E_FAIL);
    }
    
    if(get_loc(ffptr,&cibloc) != E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-read",
            "Error reading Locations section of frame file"); 
        return (E_FAIL);
    }

    /* compid as listed below is obtained from MIL-STD-2411-1 
       (Registered Data Values for RPF) Section 5.1 page 6   
       look through all records, if not found, issue an error
       and return, otherwise read coverage
    ----------------------------------------------------------*/
       while (cibloc.comptable[i].compid != LOC_COVERAGE_SECTION)
       {
           if(i > cibloc.numrec)
           {
              c_lasmsg(FALSE, "cib2las-read",
                  "Error reading frame file coverage"); 
              return(E_FAIL); 
           }
           i++;
       }
       
       if(get_cov(ffptr,cibcov) != E_SUCC)
       {
              c_lasmsg(FALSE, "cib2las-read",
                  "Error reading frame file coverage"); 
              return(E_FAIL); 
       }
       return (E_SUCC);
}   
/* ######################################################################### 
   read_frame_attributes
   
   DESC:  read frame file attributes from frame file passed fname.
          Store attributes in structure cibatt passed.
   PRE:   fname is the complete path and name to the frame file
          whose attributes are to be read.
   POST:  If frame file opened successfully, attributes are stored
          in passed structure cibatt.
   Returns E_SUCC on success, E_FAIL otherwise.
----------------------------------------------------------------------------*/
int read_frame_attributes
(
    FILE *ffptr,                 /* I: file pointer to frame file     */
    struct CIBAttribute  *cibatt /* O: structure to hold attributes   */
)
{
    int    i;
    struct CIBHeader cibhdr;
    struct CIBLocation   cibloc;
  
    /* find and read frame file header section 
    -------------------------------------------*/
    if(loc_rpf_header(ffptr) != E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-read",
            "Error moving file pointer"); 
        return (E_FAIL);
    }
    
    if(get_location_address(ffptr,&cibhdr) != E_SUCC)
        return (E_FAIL);    

    /* find and read frame file location section 
    --------------------------------------------*/
    if(find_loc(ffptr,&cibhdr) != E_SUCC)
        return (E_FAIL);    
    if(get_loc(ffptr,&cibloc) != E_SUCC)
        return (E_FAIL);    

    /* compid as listed below is obtained from MIL-STD-2411-1 
       (Registered Data Values for RPF) Section 5.1 page 6   
    ---------------------------------------------------------*/
    for(i = 0; i < cibloc.numrec; i++)
    {
        if(fseek(ffptr,cibloc.comptable[i].location,SEEK_SET) != 0)
            return (E_FAIL);
        switch ((int)cibloc.comptable[i].compid)
        {
            case LOC_ATTRIBUTE_SECTION_SUBHEADER:
                 if(get_atthdr(ffptr,cibatt) != E_SUCC)
                     return (E_FAIL);    
                 break;
            case LOC_ATTRIBUTE_SUBSECTION: 
                 if(get_atttbl(ffptr,cibatt) != E_SUCC)
                     return (E_FAIL);    
                 break;
            default: break;
        }

    }
    return(E_SUCC);
}

/* ###############  remainder of functions for debugging ################### */

#ifdef DEBUG

/* print frame file header section
   -------------------------------*/
void prt_hdr
(
    struct CIBHeader *cibhdr     /* I: struct holing frame file header info */
)
{
    printf("\nFrame file header section:\n" );
    printf("\tEndianness:\t\t\t%u\n", cibhdr->endianness );
    printf("\tLength:\t\t\t\t%u\n", cibhdr->length );
    printf("\tFilename:\t\t\t%s\n", cibhdr->filename );
    printf("\tReplace/update:\t\t\t%u\n", cibhdr->update );
    printf("\tSpecification Number:\t\t%s\n", cibhdr->specnum );
    printf("\tSpecification Date:\t\t%s\n", cibhdr->specdate );
    printf("\tSecurity Classification:\t%c\n", cibhdr->sclass );
    printf("\tCountry Code:\t\t\t%s\n", cibhdr->scode );
    printf("\tSecurity Markings:\t\t%s\n", cibhdr->smark );
    printf("\tLocation Section Offset:\t%u\n\n",
        (unsigned int)cibhdr->location );
    fflush(stdin);
}

/* print frame file location section
   ---------------------------------*/
void prt_loc
(
    struct CIBLocation *cibloc /* I: struct holing frame file location info */
)
{
    int i;

    printf("\nFrame File Location Section:\n");
    printf("\tLength:\t\t\t\t%u\n", cibloc->length);
    printf("\tTable Offset:\t\t\t%u\n", cibloc->tbloffset);
    printf("\tNumber of Records:\t\t%u\n", cibloc->numrec);
    printf("\tRecord Length:\t\t\t%u\n", cibloc->reclength);
    printf("\tAggregate Length:\t\t%u\n", cibloc->agglength);
    for(i = 0; i < cibloc->numrec; i++)
    {
        printf("\tLocation Record Number:\t\t%u\n", i+1 );
        printf("\t\tComponent ID:\t\t%u\n", cibloc->comptable[i].compid);
        printf("\t\tComponent Length:\t%u\n", cibloc->comptable[i].length);
        printf("\t\tComponent Location:\t%u\n", cibloc->comptable[i].location);
    }
    printf("\n");
    fflush(stdin);
}

/* print frame file coverage section
   ---------------------------------*/
void prt_cov
(
  struct CIBCoverage *cibcov  /* I: struct holding frame file coverage info */
)
{
    printf("\nFrame File Coverage Section:\n" );
    printf("\tUpper Left Corner Latitude:\t%f\n", cibcov->ULLat );
    printf("\tUpper Left Corner Longitude:\t%f\n", cibcov->ULLong );
    printf("\tLower Left Corner Latitude:\t%f\n", cibcov->LLLat );
    printf("\tLower Left Corner Longitude:\t%f\n", cibcov->LLLong );
    printf("\tUpper Right Corner Latitude:\t%f\n", cibcov->URLat );
    printf("\tUpper Right Corner Longitude:\t%f\n", cibcov->URLong );
    printf("\tLower Right Corner Latitude:\t%f\n", cibcov->LRLat );
    printf("\tLower Right Corner Longitude:\t%f\n", cibcov->LRLong );
    printf("\tNorth-South Resolution:\t\t%f meters\n", cibcov->NSRes );
    printf("\tEast-West Resolution:\t\t%f meters\n", cibcov->EWRes );
    printf("\tLatitude Resolution:\t\t%f seconds\n", cibcov->LatRes*3600.0);
    printf("\tLongitude Resolution:\t\t%f seconds\n\n", cibcov->LongRes*3600.0);
    fflush(stdin);
}

/* print frame file compression section
   ------------------------------------*/
void prt_com
(
    struct CIBCompress *cibcom   /* I: struct holding file compression info */
)
{
    int i, j, k;

    printf("\nFrame File Compression Section:\n");
    printf("\tCompression Algorithm ID:\t\t%u\n",cibcom->algoid);
    printf("\tNumber of Lookup Offset Records:\t%u\n",cibcom->numoff);
    printf("\tNumber of Parameter Offset Records:\t%u\n",cibcom->numparam);
    printf("\tLookup Offset Table Offset:\t\t%u\n",cibcom->LOTbloff);
    printf("\tLookup Offset Table Record Length:\t%u\n",cibcom->LOTblrec);
    for(i = 0; i < cibcom->numoff; i++)
    {
        printf("\t\tLookup Table Number:\t\t%u\n",i + 1);
        printf("\t\tLookup Table ID:\t\t%u\n",
            cibcom->LOTbl[i].tblid);
        printf("\t\tNumber of Lookup Records:\t%u\n",
            cibcom->LOTbl[i].numrec);
        printf("\t\tNumber of Values per Record:\t%u\n",
            cibcom->LOTbl[i].numval);
        printf("\t\tLookup Value Bit Length:\t%u\n",
            cibcom->LOTbl[i].bitlen);
        printf("\t\tLookup Table Offset:\t\t%u\n",
            cibcom->LOTbl[i].LToff);
        for (j = 0; j < cibcom->LOTbl[i].numrec; j += 256)
        {
          printf("\t\tEntry:\t%4d\tValues:\t",j);
          for (k = 0; k < cibcom->LOTbl[i].numval; k++)
              printf(" %4d", cibcom->vals[i][j][k]);
          printf("\n");
        }
    }
    fflush(stdin);
}

/* print frame file color table section
   ------------------------------------*/
void prt_col
(
    struct CIBColor     *cibcol      /* I: struct holding file color table info */
) 
{
    int numhist;
    int i, j;

    printf("\nFrame File Color Table Section:\n" );
    printf("\tNumber of Lookup Tables:\t\t%u\n", cibcol->numLUT );
    printf("\tNumber of Color Conversion Tables:\t%u\n", cibcol->numCON );
    printf("\tExternal Color Table File Name:\t%s\n", cibcol->efname );
    printf("\tColormap Offset Table Offset:\t\t%u\n", cibcol->cmapoff );
    printf("\tOffset Table Record Length:\t\t%u\n", cibcol->moffreclen );
    for(i = 0;i < cibcol->numLUT; i++)
    {
        printf("\tColor Table Number:\t\t\t%u\n", i + 1);
        printf("\tColor Table ID:\t\t\t\t%u\n",
            cibcol->COffTbl[i].tblid);
        printf("\tNumber of Records:\t\t\t%u\n",
            cibcol->COffTbl[i].numrec);
        printf("\tElement Length:\t\t\t\t%u\n",
            cibcol->COffTbl[i].elemlen);
        printf("\tHistogram Length:\t\t\t%u\n",
            cibcol->COffTbl[i].histlen);
        printf("\tColor Table Offset:\t\t\t%u\n",
            cibcol->COffTbl[i].grayoff);
        printf("\tHistogram Offset:\t\t\t%u\n",
            cibcol->COffTbl[i].histoff);
    }
    for(i = 0; i < cibcol->numLUT; i++)
    {
        printf("\tValues for Color Table Number:\t\t%u", i + 1);
        for (j = 0; j < cibcol->COffTbl[i].numrec; j++)
        {
          if (j%16 == 0)
              printf("\n\t");
          printf("%4d", cibcol->CTable[i][j] );
        }
        printf("\n");
    }
    for (i = 0; i < cibcol->numLUT; i++)
    {
        if (cibcol->COffTbl[i].histlen != 0xFFFF)
        {
          printf("\tValues for Histogram Number:\t\t%u", i+1);
          numhist = CIB_HIST_LEN;
          for (j = 0; j < cibcol->COffTbl[i].numrec; j++)
          {
            if (j%8 == 0)
                printf("\n\t");
            printf("%8d", cibcol->Histogram[i][j]);
          }
          printf("\n");
        }
      }
    printf("\tColor Conversion Subsection:\n");
    printf("\tColor Conv Offset Table Offset:\t\t%u\n",
        cibcol->ccoffset);
    printf("\tColor Conv Offset Table Record Length:\t%u\n",
        cibcol->coffreclen);
    printf("\tColor Conv Table Record Length:\t\t%u\n",
        cibcol->ccreclen);
    if (cibcol->numCON > 0)
    {
        printf("\tColor Conversion Offset Table:\n");
        printf("\tColor Conv Table ID:\t%u\n",
            cibcol->ccoff.cctblid);
        printf("\tNumber of Color Conv Records:\t%u\n",
            cibcol->ccoff.nccrecs);
        printf("\tColor Conv Table Offset:\t%u\n",
            cibcol->ccoff.cctbloff);
        printf("\tSource Color Table Offset:\t%u\n",
            cibcol->ccoff.srcoff);
        printf("\tTarget Color Table Offset:\t%u\n",
            cibcol->ccoff.tgtoff);
    }
    if(cibcol->numCON > 0)
    {
        printf("\tValues for Color Conversion Table:");
        for (j = 0; j < cibcol->ccoff.nccrecs; j++)
        {
            if ( j%8 == 0 )
                printf("\n\t");
            printf("%6d", cibcol->CCTbl[j]);
        }
        printf("\n");
    }
    fflush(stdin);
}

/* print frame file image section
   ------------------------------------*/
void prt_img
(
    struct CIBImage *cibimg      /* I: struct holding frame file image      */
)
{
    int nmask;
    int i;
    unsigned long LNULL = 0xFFFFFFFF;

 /* Header subsection
    -----------------*/
    printf("\nFrame File Image Section:\n");
    printf("\tNumber of Spectral Groups:\t\t%u\n",
        cibimg->numgrp);
    printf("\tNumber of Subframe Tables:\t\t%u\n",
        cibimg->numstbl);
    printf("\tNumber of Spectral Band Tables:\t\t%u\n",
        cibimg->numbtbl);
    printf("\tNumber of Lines per Image Row:\t\t%u\n",
        cibimg->numbline);
    printf("\tNumber of EW Subframes:\t\t\t%u\n",
        cibimg->EWsubfrm);
    printf("\tNumber of NS Subframes:\t\t\t%u\n",
        cibimg->NSsubfrm);
    printf("\tNumber of Columns per Subframe:\t\t%u\n",
        cibimg->colsubf);
    printf("\tNumber of Rows per Subframe:\t\t%u\n",
        cibimg->rowsubf );
    if(cibimg->smaskoff != LNULL)
        printf("\tSubframe Mask Table Offset:\t\t%u\n",
            cibimg->smaskoff);
    else
        printf("\tSubframe Mask Table Offset:\t\t<null>\n");
    if(cibimg->tmaskoff != LNULL)
        printf("\tTransparency Mask Table Offset:\t\t%u\n",
            cibimg->tmaskoff);
    else
        printf("\tTransparency Mask Table Offset:\t\t<null>\n");

    /* Display subsection
    ---------------------*/
    printf("\tNumber of Image Rows:\t\t\t%u\n", cibimg->numrow);
    printf("\tNumber of Codes per Row:\t\t%u\n", cibimg->numcol);
    printf("\tCode Length:\t\t\t\t%u\n", cibimg->codelen);

    /* Mask subsection
    -------------------*/
    nmask = cibimg->EWsubfrm * cibimg->NSsubfrm;
    printf("\tSubframe Sequence Record Length:\t%u\n",
        cibimg->subrecl);
    printf("\tTransparency Sequence Record Length:\t%u\n",
        cibimg->tranrecl);
    printf("\tTransparent Pixel Length:\t\t%u\n",
        cibimg->tcodelen);
    printf("\tTransparent Output Pixel:\t\t%u\n",
        cibimg->tcode);
    printf("\tSubframe Mask:");
    for (i = 0; i < nmask; i++)
    {
        if( i%8 == 0 )
            printf("\n\t");
        if( cibimg->sframes[i] != LNULL )
            printf("%8d", cibimg->sframes[i] );
        else
            printf("  <null>" );
    }
    printf("\n");
    printf("\tTransparency Mask:");
    for(i = 0; i < nmask; i++)
    {
        if(i%8 == 0)
            printf("\n\t");
        if(cibimg->tframes[i] != LNULL)
            printf("%8d",cibimg->tframes[i]);
        else
            printf("  <null>" );
    }
    printf("\n");
    fflush(stdin);
}

/* print frame file attributes
   ---------------------------*/
void prt_att
(
    struct CIBAttribute *cibatt /* I:  struct holding frame file attributes */
) 
{
    int i,j;

    printf("\nFrame File Attribute Section:\n");
    printf("\tNumber of Attribute Records:\t\t%u\n",
        cibatt->numatt);
    printf("\tNumber of Areal Coverage Records:\t%u\n",
        cibatt->numcov);
    printf("\tAttribute Offset Table Offset:\t\t%u\n",
        cibatt->tbloff);
    printf("\tAttribute Offset Record Length:\t\t%u\n",
        cibatt->offrecl);
      
    for(i = 0; i < cibatt->numatt; i++)
    {
        printf("\tAttribute ID:\t\t\t\t%u\n",
            cibatt->attoff[i].attid);
        printf("\tParameter ID:\t\t\t\t%u\n",
            cibatt->attoff[i].paramid);
        printf("\tCoverage Sequence Number:\t\t%u\n",
            cibatt->attoff[i].covseq);
        printf("\tAttribute Record Offset:\t\t%u\n",
            cibatt->attoff[i].attroff);
    }

    printf("\tAttribute Table:\n");
    for (i = 0; i < cibatt->numatt; i++)
    {
      switch ((int)cibatt->attoff[i].attid )
      {
          case 1:   /* Currency Date
                    ----------------*/
                    printf("\tCurrency Date:\t\t%s\n",
                        cibatt->attribute[i].param.c);
                    break;
          case 2:   /* Production Date
                    ------------------*/
                    printf("\tProduction Date:\t%s\n",
                        cibatt->attribute[i].param.c);
                    break;
          case 3:   /* Significant Date
                    -------------------*/
                    printf("\tSignificant Date:\t%s\n",
                        cibatt->attribute[i].param.c);
                    break;
          case 9:   /* Horizontal Absolute Accuracy
                    --------------------------------*/
                    if ( cibatt->attoff[i].paramid == 1 )
                    {
                        printf("\tAbsolute Accuracy:\t%u\n",
                            cibatt->attribute[i].param.l);
                    }
                    else if ( cibatt->attoff[i].paramid == 2 )
                    {
                        printf("\tAccuracy Units:\t\t%u\n",
                            cibatt->attribute[i].param.s);
                    }
                    break;
          case 11:   /* Horizontal Relative Accuracy
                     -------------------------------*/
                    if ( cibatt->attoff[i].paramid == 1 )
                    {
                        printf("\tRelative Accuracy:\t%u\n",
                            cibatt->attribute[i].param.l);
                    }
                    else if ( cibatt->attoff[i].paramid == 2 )
                    {
                        printf("\tAccuracy Units:\t\t%u\n",
                            cibatt->attribute[i].param.s);
                    }
                    break;
          case 22:  /* Image Source 
                    ----------------*/
                    if ( cibatt->attoff[i].paramid == 1 )
                        printf("\tImage Source:\t\t%s\n",
                            cibatt->attribute[i].param.c);
                    else if ( cibatt->attoff[i].paramid == 2 )
                        printf("\tGSD:\t\t\t%u\n",
                            cibatt->attribute[i].param.l);
                    break;
          default:  break;
          }
    }
    if(cibatt->numcov > 0)
    {
        printf("\tExplicit Areal Coverage:\n");
        printf("\tCoverage Table Offset:\t\t%u\n", cibatt->covtbloff );
        printf("\tCoverage Record Length:\t\t%u\n", cibatt->covreclen );
        printf("\tCoordinate Record Length:\t%u\n", cibatt->coordreclen );

        for(i = 0; i < cibatt->numcov; i++)
        {
          printf("\tNum Vertices:\t%u\n", cibatt->attarea[i].numvert );
          for(j = 0; j < cibatt->attarea[i].numvert; j++ )
          {
              printf("\tLatitude:\t%lf\n", cibatt->attarea[i].coords[j].lat);
              printf("\tLongitude:\t%lf\n", cibatt->attarea[i].coords[j].lng);
          }
        }
    }
}

/* print frame file replace/update section
   ----------------------------------------*/
void prt_new
(
    struct CIBUpdate *cibnew /* I: struct holding ff update info.           */
)
{
    int   i;

    printf("\nFrame File Replace/Update Section:\n");
    printf("\tTable Offset:\t\t\t%u\n", cibnew->rtbloff);
    printf("\tNumber of Records:\t\t%u\n", cibnew->numrec);
    printf("\tRecord Length:\t\t\t%u\n", cibnew->rreclen);

    for(i = 0; i < cibnew->numrec; i++)
    {
        printf("\tReplace/Update Record Number:\t\t%u\n",i+1);
        printf("\t\tNew FIle Name:\t\t%s\n", cibnew->update[i].nfname);
        printf("\t\tOld FIle Name:\t\t%s\n", cibnew->update[i].ofname);
        printf("\t\tUpdate Status:\t%u\n", cibnew->update[i].upstat);
    }
    printf("\n");
    fflush(stdin);
}

/* print out NITF file header
   --------------------------*/
void prt_nitfhdr
(
)
{
    char        Title1[41];    /* first half of file title                      */
    char        Title2[41];    /* second half of file title                     */
  
    printf("\nNITF File Header:\n");
    printf("\tFHDR:\t\t%s\n", nitfhdr.FHDR );
    printf("\tCLEVEL:\t\t%s\n", nitfhdr.CLEVEL );
    printf("\tSTYPE:\t\t%s\n", nitfhdr.STYPE );
    printf("\tOSTAID:\t\t%s\n", nitfhdr.OSTAID );
    printf("\tFDT:\t\t%s\n", nitfhdr.FDT );
    strncpy( Title1, &nitfhdr.FTITLE[0], 40 );
    strncpy( Title2, &nitfhdr.FTITLE[40], 40 );
    printf("\tFTITLE[01-40]:\t%s\n\tFTITLE[41-80]:\t%s\n", Title1, Title2 );
    printf("\tFSCLAS:\t\t%s\n", nitfhdr.FSCLAS );
    printf("\tFSCODE:\t\t%s\n", nitfhdr.FSCODE );
    printf("\tFSCTLH:\t\t%s\n", nitfhdr.FSCTLH );
    printf("\tFSREL:\t\t%s\n", nitfhdr.FSREL );
    printf("\tFSCAUT:\t\t%s\n", nitfhdr.FSCAUT );
    printf("\tFSCTLN:\t\t%s\n", nitfhdr.FSCTLN );
    printf("\tFSDWNG:\t\t%s\n", nitfhdr.FSDWNG );
    if ( !strcmp( nitfhdr.FSDWNG, "999998" ) )
        printf("\tFSDEVT:\t\t%s\n", nitfhdr.FSDEVT );
    printf("\tFSCOP:\t\t%s\n", nitfhdr.FSCOP );
    printf("\tFSCPYS:\t\t%s\n", nitfhdr.FSCPYS );
    printf("\tENCRYP:\t\t%s\n", nitfhdr.ENCRYP );
    printf("\tONAME:\t\t%s\n", nitfhdr.ONAME );
    printf("\tOPHONE:\t\t%s\n", nitfhdr.OPHONE );
    printf("\tFL:\t\t%s\n", nitfhdr.FL );
    printf("\tHL:\t\t%s\n", nitfhdr.HL );
    printf("\tNUMI:\t\t%s\n", nitfhdr.NUMI );
    if ( strcmp( nitfhdr.NUMI, "000" ) )
    {
        printf("\tLISH001:\t%s\n", nitfhdr.LISH001 );
        printf("\tLI001:\t\t%s\n", nitfhdr.LI001 );
    }
    printf("\tNUMS:\t\t%s\n", nitfhdr.NUMS );
    if ( strcmp( nitfhdr.NUMS, "000" ) )
    {
        printf("\tLSSH001:\t%s\n", nitfhdr.LSSH001 );
        printf("\tLS001:\t\t%s\n", nitfhdr.LS001 );
    }
    printf("\tNUML:\t\t%s\n", nitfhdr.NUML );
    if ( strcmp( nitfhdr.NUML, "000" ) )
    {
        printf("\tLLSH001:\t%s\n", nitfhdr.LLSH001 );
        printf("\tLL001:\t\t%s\n", nitfhdr.LL001 );
    }
    printf("\tNUMT:\t\t%s\n", nitfhdr.NUMT );
    if ( strcmp( nitfhdr.NUMT, "000" ) )
    {
        printf("\tLTSH001:\t%s\n", nitfhdr.LTSH001 );
        printf("\tLT001:\t\t%s\n", nitfhdr.LT001 );
    }  
    printf("\tNUMDES:\t\t%s\n", nitfhdr.NUMDES );
    if ( strcmp( nitfhdr.NUMDES,  "000" ) )
    {
        printf("\tLDSH001:\t%s\n", nitfhdr.LDSH001 );
        printf("\tLD001:\t\t%s\n", nitfhdr.LD001 );
    }
    printf("\tNUMRES:\t\t%s\n", nitfhdr.NUMRES );
    if ( strcmp( nitfhdr.NUMRES,  "000" ) )
    {
        printf("\tLRSH001:\t%s\n", nitfhdr.LRSH001 );
        printf("\tLR001:\t\t%s\n", nitfhdr.LR001 );
    }  
    printf("\tUDHDL:\t\t%s\n", nitfhdr.UDHDL );
    printf("\tUDHD:\t\t%s\n", nitfhdr.UDHD );
    printf("\tRETAG:\t\t%s\n", nitfhdr.RETAG );
    printf("\tREL:\t\t%s\n", nitfhdr.REL );

}

/* Read the NITF file header */
int get_nitfhdr
(
    FILE    *ffptr                   /* I: FILE pointer to CIB image frame file */
)
{
    int stat;                    /* I/O return status                       */
    int NUMI;                    /* number of images                        */
    int NUMS;                    /* number of symbols                       */
    int NUML;                    /* number of labels                        */
    int NUMT;                    /* number of text files                    */
    int NUMDES;                  /* number of Data Extension Segments       */
    int NUMRES;                  /* number of Reserved Extension Segments   */
    int i;                           /* counter                                 */

    stat = fread( nitfhdr.FHDR, 1, 9, ffptr );
    nitfhdr.FHDR[9] = '\0';
    stat += fread( nitfhdr.CLEVEL, 1, 2, ffptr );
    nitfhdr.CLEVEL[2] = '\0';
    stat += fread( nitfhdr.STYPE, 1, 4, ffptr );
    nitfhdr.STYPE[4] = '\0';
    stat += fread( nitfhdr.OSTAID, 1, 10, ffptr );
    nitfhdr.OSTAID[10] = '\0';
    stat += fread( nitfhdr.FDT, 1, 14, ffptr );
    nitfhdr.FDT[14] = '\0';
    stat += fread( nitfhdr.FTITLE, 1, 80, ffptr );
    nitfhdr.FTITLE[80] = '\0';
    stat += fread( nitfhdr.FSCLAS, 1, 1, ffptr );
    nitfhdr.FSCLAS[1] = '\0';
    stat += fread( nitfhdr.FSCODE, 1, 40, ffptr );
    nitfhdr.FSCODE[40] = '\0';
    stat += fread( nitfhdr.FSCTLH, 1, 40, ffptr );
    nitfhdr.FSCTLH[40] = '\0';
    stat += fread( nitfhdr.FSREL, 1, 40, ffptr );
    nitfhdr.FSREL[40] = '\0';
    stat += fread( nitfhdr.FSCAUT, 1, 20, ffptr );
    nitfhdr.FSCAUT[20] = '\0';
    stat += fread( nitfhdr.FSCTLN, 1, 20, ffptr );
    nitfhdr.FSCTLN[20] = '\0';
    stat += fread( nitfhdr.FSDWNG, 1, 6, ffptr );
    nitfhdr.FSDWNG[6] = '\0';
    if ( !strcmp( nitfhdr.FSDWNG, "999998" ) )
    {
        stat += fread( nitfhdr.FSDEVT, 1, 40, ffptr );
        nitfhdr.FSDEVT[40] = '\0';
    }
    stat += fread( nitfhdr.FSCOP, 1, 5, ffptr );
    nitfhdr.FSCOP[5] = '\0';
    stat += fread( nitfhdr.FSCPYS, 1, 5, ffptr );
    nitfhdr.FSCPYS[5] = '\0';
    stat += fread( nitfhdr.ENCRYP, 1, 1, ffptr );
    nitfhdr.ENCRYP[1] = '\0';
    stat += fread( nitfhdr.ONAME, 1, 27, ffptr );
    nitfhdr.ONAME[27] = '\0';
    stat += fread( nitfhdr.OPHONE, 1, 18, ffptr );
    nitfhdr.OPHONE[18] = '\0';
    stat += fread( nitfhdr.FL, 1, 12, ffptr );
    nitfhdr.FL[12] = '\0';
    stat += fread( nitfhdr.HL, 1, 6, ffptr );
    nitfhdr.HL[6] = '\0';
    stat += fread( nitfhdr.NUMI, 1, 3, ffptr );
    nitfhdr.NUMI[3] = '\0';
    sscanf( nitfhdr.NUMI, "%3d", &NUMI );
    for ( i=0; i<NUMI; i++ )
    {
        stat += fread(nitfhdr.LISH001, 1, 6, ffptr );
        nitfhdr.LISH001[6] = '\0';
        stat += fread( nitfhdr.LI001, 1, 10, ffptr );
        nitfhdr.LI001[10] = '\0';   
    }
    stat += fread( nitfhdr.NUMS, 1, 3, ffptr );
    nitfhdr.NUMS[3] = '\0';
    sscanf( nitfhdr.NUMS, "%3d", &NUMS );
    for ( i=0; i<NUMS; i++ )
    {
        stat += fread( nitfhdr.LSSH001, 1, 4, ffptr );
       nitfhdr.LSSH001[4] = '\0';
        stat += fread( nitfhdr.LS001, 1, 6, ffptr );
        nitfhdr.LS001[6] = '\0';   
    }
    stat += fread( nitfhdr.NUML, 1, 3, ffptr );
    nitfhdr.NUML[3] = '\0';
    sscanf( nitfhdr.NUML, "%3d", &NUML );
    for ( i=0; i<NUML; i++ )
    {
        stat += fread( nitfhdr.LLSH001, 1, 4, ffptr );
        nitfhdr.LLSH001[4] = '\0';
        stat += fread(nitfhdr.LL001, 1, 3, ffptr );
        nitfhdr.LL001[3] = '\0';   
    }
    stat += fread( nitfhdr.NUMT, 1, 3, ffptr );
    nitfhdr.NUMT[3] = '\0';
    sscanf( nitfhdr.NUMT, "%3d", &NUMT );
    for ( i=0; i<NUMT; i++ )
    {
        stat += fread( nitfhdr.LTSH001, 1, 4, ffptr );
        nitfhdr.LTSH001[4] = '\0';
        stat += fread( nitfhdr.LT001, 1, 5, ffptr );
        nitfhdr.LT001[5] = '\0';   
    }
    stat += fread( nitfhdr.NUMDES, 1, 3, ffptr );
    nitfhdr.NUMDES[3] = '\0';
    sscanf( nitfhdr.NUMDES, "%3d", &NUMDES );
    for ( i=0; i<NUMDES; i++ )
    {
        stat += fread( nitfhdr.LDSH001, 1, 4, ffptr );
        nitfhdr.LDSH001[4] = '\0';
        stat += fread( nitfhdr.LD001, 1, 9, ffptr );
        nitfhdr.LD001[9] = '\0';
    }
    stat += fread( nitfhdr.NUMRES, 1, 3, ffptr );
    nitfhdr.NUMRES[3] = '\0';
    sscanf( nitfhdr.NUMRES, "%3d", &NUMRES );
    for ( i=0; i<NUMRES; i++ )
    {
        stat += fread( nitfhdr.LRSH001, 1, 4, ffptr );
        nitfhdr.LRSH001[4] = '\0';
        stat += fread( nitfhdr.LR001, 1, 7, ffptr );
        nitfhdr.LR001[7] = '\0';
    }
    stat += fread( nitfhdr.UDHDL, 1, 5, ffptr );
    nitfhdr.UDHDL[5] = '\0';
    stat += fread( nitfhdr.UDHD, 1, 3, ffptr );
    nitfhdr.UDHD[3] = '\0';
    stat += fread( nitfhdr.RETAG, 1, 6, ffptr );
    nitfhdr.RETAG[6] = '\0';
    stat += fread( nitfhdr.REL, 1, 5, ffptr );
    nitfhdr.REL[5] = '\0';
    prt_nitfhdr();
    return( stat );
}


/*  read CIB Volume header section 
----------------------------------*/
int get_hdr
(
  FILE             *ffptr,/* I: FILE pointer to CIB image frame file        */
  struct CIBHeader *cibhdr/* O: contains TOC information                    */ 
)
{
    int stat = 0;

    if(fread(&cibhdr->endianness,1,1,ffptr)!= 1)  
    {
        printf("read endianness\n");
        return (E_FAIL);
    }
    if(fread(&cibhdr->length,2,1,ffptr)!= 1)  
    {
         printf("read length\n");
         return (E_FAIL);
    }
    if(fread(cibhdr->filename,1,12,ffptr)!= 12)  
    {
         printf("read filename\n");
         return (E_FAIL);
    }
    cibhdr->filename[12] = 0;
    if(fread(&cibhdr->update,1,1,ffptr)!= 1)  
    {
         printf("read update\n");
         return (E_FAIL);
    }
    if(fread(cibhdr->specnum,1,15,ffptr)!= 15)  
    {
         printf("read specnum\n");
         return (E_FAIL);
    }
    cibhdr->specnum[15] = 0;
    if(fread(cibhdr->specdate,1,8,ffptr)!= 8)  
    {
         printf("read specdate\n");
         return (E_FAIL);
    }
    cibhdr->specdate[8] = 0;
    if(fread(&cibhdr->sclass,1,1,ffptr)!= 1)  
    {
         printf("read sclass\n");
         return (E_FAIL);
    }
    if(fread(cibhdr->scode,1,2,ffptr)!= 2)  
    {
         printf("read scode\n");
         return (E_FAIL);
    }
    cibhdr->scode[2] = 0;
    if(fread(cibhdr->smark,1,2,ffptr)!= 2)  
    {
         printf("read smark\n");
         return (E_FAIL);
    }
    cibhdr->smark[2] = 0;

    if(fread(&cibhdr->location,4,1,ffptr)!= 1)  
    {
         printf("location address is %ld\n",cibhdr->location);
         return (E_FAIL);
    }
    printf("location address is %ld\n",cibhdr->location);
    return (E_SUCC);
}


/*
    Returns E_SUCC on success, E_FAIL otherwise.
    ------------------------------------------------------------------------*/
int get_attcov
(
    FILE *ffptr,                 /* I: FILE pointer to CIB image frame file */
    struct CIBAttribute *cibatt  /* O: array to hold ff attributes          */
)
{
    int i,j;

    if(fread(&cibatt->covtbloff,4,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibatt->covreclen,2,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibatt->coordreclen,2,1,ffptr) != 1)
        return (E_FAIL);

    for(i = 0; i < cibatt->numcov; i++)
    {
        if(fread(&cibatt->attarea[i].numvert,2,1,ffptr) != 1)
            return (E_FAIL);
        for(j = 0; j < cibatt->attarea[i].numvert; j++)
        {
            if(fread(&cibatt->attarea[i].coords[j].lat,8,1,ffptr) != 1)
                return (E_FAIL);
            if(fread(&cibatt->attarea[i].coords[j].lng,8,1,ffptr) != 1)
                return (E_FAIL);
        }
    }
    return (E_SUCC);
}

/*  get frame file new/update header information

    Returns E_SUCC on success, E_FAIL otherwise.
    ------------------------------------------------------------------------*/
    
int get_newhdr
(
    FILE  *ffptr,                /* I: FILE pointer to CIB image frame file */
    struct CIBUpdate *cibnew     /* O: struct to hold update/new ff data    */
)
{
    if(fread(&cibnew->rtbloff,4,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibnew->numrec,2,1,ffptr) != 1)
        return (E_FAIL);
    if(fread(&cibnew->rreclen,2,1,ffptr) != 1)
        return (E_FAIL);
    return (E_SUCC);
}


/*  get frame file new/update table 
    Returns E_SUCC on success, E_FAIL otherwise.
    ------------------------------------------------------------------------*/

int get_newtbl
(
    FILE *ffptr,               /* I: FILE pointer to CIB image frame file   */
    struct CIBUpdate *cibnew   /* O: struct to hold new/update information  */
)
{
    int i;

    for(i = 0; i < cibnew->numrec; i++)
    {
        if(fread(cibnew->update[i].nfname,1,12,ffptr) != 12)
            return (E_FAIL);
        cibnew->update[i].nfname[12] = 0;
        if(fread(cibnew->update[i].ofname,1,12,ffptr) != 12)
            return (E_FAIL);
        cibnew->update[i].ofname[12] = 0;
        if(fread(&cibnew->update[i].upstat,1,1,ffptr) != 12)
            return (E_FAIL);
    }
    return (E_SUCC);
}

#endif /* DEBUG */






