/*****************************************************************************
FILENAME:  read_toc.c

FUNCTION(S): read_toc()
             setup_bound_rects()
             in_list()

PURPOSE:    Read in TOC (Table of Contents) from a CIB image. This function
            returns a pointer to a list of FRAMEFILE type structures which
            contain information about the location of all of the frame files
            in the CIB volume.  

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        2/00            CSB          Original Development
  1.1      A. Shimitz      10/00           CSB          Added code to determine 
                                                        the zone based on the
                                                        nw boundary rectangle 
                                                        and added the in_list &
                                                        setup_bound_rects
                                                        routines (CR 4215)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
            run under LAS/TAE
PROJECT:    LAS

ALGORITHM DESCRIPTION:

    set zoneflag to TRUE if the user did not specify any boundary rects
    open TOC file
    read mil standard name/number for product
    read mil standard date
    skip over NIFT header 
    read endianess
    check for endian mismatch between file and current OS
    read TOC [header section length]
    read header filename
    read header new
    read header standard number 
    read header standard date
    read header classification
    read header country
    read header release
    read head loc section physical locations
    check if data is CIB by comparing to standard number listed
        to CIB standard number
    call parse_locations() to obtain desired file locations
    check if appropriate header file locations were found
    read frame file index section table offset
    read number of frame file index records
    read number of pathname records
    read index record length
    read boundary rectangle section table offset
    read number of boundary rectangle records
    read boundary rectangle record length
    call setup_bound_rects() to determine the numbering of the
        boundary retangles (the numbering that appears on the CD case)
    create an array of frame file records
    skip to frame file index section of TOC file
    for each frame file index record
        read frame file boundary record id - use this id as the index into
            the bound_num_array to retrieve its tile number that correlates
            with the numbering on the CD
        read frame file path
        read frame file name
        call locate_frame() to update frame file zone row and column
    end for each frame file 
    close TOC file
    return list of frame files

ALGORITHM REFERENCE(S):  

    MIL-PRF-89041 Performance Specification
    Controlled Image Base

    MIL-STD-2500B Dept. of Defense Interface Standard
    National Imagery Transmission Format Version 2.1

    MIL-STD-2411 Military Standard
    Raster Product Format

    MIL-STD-2411-1 Military Standard
    Registered Data Value for Raster Product Format

RETURNS:  a pointer to a list of frame file records

******************************************************************************/
#include<string.h>
#include<stdio.h>
#include<unistd.h>
#include<string.h>
#include<ctype.h>
#include<math.h>
#include"las.h"
#include"util.h"
#include"cib2las.h"
#include"nitfaccess.h"
#include"locate_frame.h"
#define RPF_DIR "Rpf/"

#define RECTHDRSEC 0
#define RECTTBL    1
#define FFHDRSEC   2
#define FFTBL      3

static int setup_bound_rects
(
    FILE  *toc,                /* I: file pointer to TOC file                */
    unsigned short bound_rect_tbl_offset,
                               /* I: offset between the beginning of the
                                     boundary rectangle subsection and the
                                     boundary rectangle table                */
    long  num_bound_rect_recs, /* I: number of boundary rectangles records   */
    unsigned short bound_rec_len,
                               /* I: boundary rectangle record length        */
    long  bound_rect_location, /* I: boundary rectangle subsection physical
                                     location                                */
    unsigned int num_index_recs,
                               /* I: number of frame file index records      */
    unsigned int frm_index_tbl_off,
                               /* I: Frame file index table offset           */
    unsigned short index_rec_len,
                               /* I: frame file index record length          */
    long  ff_location,         /* I: frame file subsection physical location */
    char  *filename,           /* I: TOC filename including path             */
    int   *bound_num_array     /* O: array with new boundary rectangle #'s   */
);

static int in_list
(
    int cur_id,                /* I: current boundary rectangle sequence id  */
    int b_list[MAXTILES]       /* I: the list of boundary rectangle sequence
                                     numbers for those that contain overview
                                     image frame files only                  */
);

FRAMEFILE *read_toc
(   
    char *dir,                    /* I: path to header file                 */
    char *device,                 /* I: drive CD located on                 */
    int  tilenum[MAXTILES],       /* I: boundary rectangles to ingest       */
    char *zone,                   /* I/O: CIB zone of requested image       */
    int  *fcount                  /* O: number of frame files               */
)    
{
    char filename[CMLEN];        /* TOC filename including path             */
    char *directory;             /* path to TOC file                        */
    char rpfdir[5] = RPF_DIR;    /* RPF sub-directory name                  */
    char tocfile[6];             /* path and file name to RPF TOC file      */
    unsigned short boundary_id;  /* boundary record id                      */
    unsigned short frame_row;    /* frame file row in boundary rec.         */
    unsigned short frame_col;    /* frame file column in boudary rec.       */
    unsigned short path_length;
    unsigned short num_pathname_recs;/* # frame file pathname records       */
    unsigned short index_rec_len;/* frame file index record length          */
    unsigned short num_bound_rect_recs;
                                 /* number of boundary rectangle records -
                                    the number of boundary rectangles is
                                    half this                               */
    unsigned short bound_rect_tbl_offset;
                                 /* the displacement between the beginning
                                    of the boundary rectangle subsection and
                                    the boundary rectangle table            */
    unsigned short bound_rec_len;/* boundary rectangle record length        */
    unsigned int num_index_recs; /* number of frame file index records      */
    unsigned int path_off;       /* offset to path section of TOC file      */
    unsigned int frm_index_tbl_off;/* Frame file index table offset         */
    int  *bound_num_array=NULL;  /* array of boundary rectangle numbers
                                    corresponding to the boundary rectangle
                                    numbers on the CD case                  */
    int  i, j;              
    int zoneflag;                /* indicates if the zone has been determined*/
    long  curr;                  /* used with fseek, ftell                  */
    long  length;                /* length                                  */ 
    FILE  *toc;                  /* file pointer to TOC file                */
    FRAMEFILE *frameitem,        /* structure to hold info for a frame file */
              *flist;            /* list of stuctures holding data for all
                                    frame files in CIB volume               */
    struct CIBHeader   head;     /* structure to hold header infomation     */
    LOCATION locations[4];       /* location records for the frame file
                                    section subheader, frame file index table,
                                    boundary rectangle section subheader,
                                    and the boundary rectangle table        */

    /* set the zoneflag to FALSE if the user specified the
       boundary rectangles to ingest - indicating that we
       have not determined the zone code yet.
    ------------------------------------------------------*/
    if(tilenum[0] != -1)
        zoneflag = FALSE;
    else
        zoneflag = TRUE;

    /* Open CIB Table of Contents File  "A.TOC"
       Table of contents file found in RPF directory below input directory 
    -----------------------------------------------------------------------*/
    strcpy(tocfile,RGPF_TOC);
    
    /* try opening file as original
    -------------------------------*/
    sprintf(filename,"%s%s%s%s",device,dir,rpfdir,tocfile);
    if ((toc = fopen(filename, "r")) == NULL)
    {
        /* set toc to lower case
        ------------------------*/
        length = (long)strlen(tocfile);
        c_up2low(tocfile,&length); 
        sprintf(filename,"%s%s%s%s",device,dir,rpfdir,tocfile);
        if ((toc = fopen(filename, "r")) == NULL)
        {
            /* try all lower case
            ---------------------*/  
            length = (long) strlen(rpfdir);
            c_up2low(rpfdir,&length);
            sprintf(filename, "%s%s%s%s",device,dir,rpfdir,tocfile);
            if ((toc = fopen(filename, "r")) == NULL)
            {
                /* try all uppercase
                --------------------*/   
                c_low2up(rpfdir,&length);
                length = (long) strlen(tocfile);
                c_low2up(tocfile,&length);
                sprintf(filename, "%s%s%s%s",device,dir,rpfdir,tocfile);
                if ((toc = fopen(filename, "r")) == NULL)
                {
                    c_lasmsg(FALSE, "cib2las-toc", 
                        "Error opening RPF TOC file %s",filename);
                    return (NULL);
                }
            }
        }
    }

    /* set name back to original unmangled name for error messages
    --------------------------------------------------------------*/
    sprintf(filename,"%s%s%s%s",device,dir,rpfdir,tocfile);
    
    /* move file pointer to User Defined Header Data Section of NITF
       Header.  The RPF TOC Header is located in this portion of the 
       NITF Header.
    ----------------------------------------------------------------*/
	if(loc_rpf_header(toc) != E_SUCC)
        return (NULL);

    /* Read TOC file header information */

    /* Compare file endian to OS endian, set flag swap if mismatch 
    --------------------------------------------------------------*/
    if(fread(&head.endianness, 1,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
            "Error reading from %s",filename);
        return (NULL);
    }
    
    /* check for endianess mismatch 
    -------------------------------*/
    check_endian(head.endianness);
    
    if(fread(&head.length,2,1,toc) != 1)  
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        return (NULL);
    }    
    swap_endian(&head.length,2);
        
    if(fread(head.filename,12,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        return (NULL);
    } 
    
    if(fread(&head.update,1,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        return (NULL);
    }
    
    if(fread(head.specnum,15,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        return (NULL);
    }
    
    if (strncmp(&head.specnum[9],"41",2) != 0)
    {
        c_lasmsg(FALSE,"cib2las-toc",
            "Error in head.standard_num:%15s\n",head.specnum);
        fclose(toc);
        return (NULL);
    }  
    
    if(fread(head.specdate,8,1,toc)!= 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }
    
    if(fread(&head.sclass,1,1,toc)!= 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }
    
    if(fread(head.scode,2,1,toc)!= 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }
    
    if(fread(head.smark,2,1,toc)!= 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    } 
    
    if(fread(&head.location,4,1,toc)!= 1)
    {
        c_lasmsg(FALSE, "cib2las-toc","Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    } 
    swap_endian(&head.location,4);

    /* fseek to start of location section: 
    --------------------------------------*/
    if(fseek(toc,head.location,SEEK_SET) != 0) 
    {
        c_lasmsg(FALSE, "cib2las-toc",
            "Error moving file pointer in %s",filename);
        fclose(toc);
        return (NULL);
    }    
    
    /* Locate the correct sections
    ------------------------------*/
    locations[RECTHDRSEC].id = LOC_BOUNDARY_SECTION_SUBHEADER;
    locations[RECTTBL].id = LOC_BOUNDARY_RECTANGLE_TABLE;
    locations[FFHDRSEC].id = LOC_FRAME_FILE_INDEX_SUBHEADER;
    locations[FFTBL].id = LOC_FRAME_FILE_INDEX_SUBSECTION;

    /* find the correct locations 
    ------------------------------*/
    if(parse_locations(toc,4,filename,locations) != E_SUCC)
    {
        fclose(toc);
        return (NULL);
    }

    for(i = 0; i < 4; i++)
    {
        if(locations[i].phys_index == FALSE)
        {
           c_lasmsg(FALSE,"cib2las-toc",
                    "Error finding RPF TOC file %s locations",filename);
           fclose(toc);
           return(NULL);
        }
    }

    /* Read # of frame file index records */
    /* Skip 1 byte security classification */
    /* locations[FFHDRSEC] is loc of frame file index section subheader
    ------------------------------------------------------------*/
    if(fseek(toc,locations[FFHDRSEC].phys_index + 1,SEEK_SET) != 0)
    {
        c_lasmsg(FALSE, "cib2las-toc",
            "Error moving file pointer in %s",filename);
        fclose(toc);
        return (NULL);
    }    
    
    if(fread(&frm_index_tbl_off,4,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
            "Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }     
    swap_endian(&frm_index_tbl_off,4);

    if(fread(&num_index_recs,4,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    } 
    swap_endian(&num_index_recs,4);

    if(fread(&num_pathname_recs,2,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    } 
    swap_endian(&num_pathname_recs,2);

    if(fread(&index_rec_len,2,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }     
    swap_endian(&index_rec_len,2);

    /* Read the number of boundary rectangle records and set the number of
       boundary rectangles. Also read the boundary rectangle table offset.
    ----------------------------------------------------------------------*/
    if(fseek(toc,locations[RECTHDRSEC].phys_index,SEEK_SET) != 0)
    {
        c_lasmsg(FALSE, "cib2las-toc",
            "Error moving file pointer in %s",filename);
        fclose(toc);
        return (NULL);
    }    
    
    if(fread(&bound_rect_tbl_offset,4,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
            "Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }     
    swap_endian(&bound_rect_tbl_offset,4);

    if(fread(&num_bound_rect_recs,2,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
            "Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }     
    swap_endian(&num_bound_rect_recs,2);

    if(fread(&bound_rec_len,2,1,toc) != 1)
    {
        c_lasmsg(FALSE, "cib2las-toc", 
            "Error reading from %s",filename);
        fclose(toc);
        return (NULL);
    }     
    swap_endian(&bound_rec_len,2);

    /* allocate memory for the boundary rectangle numbers array and
       set all values to -1 initially
    ---------------------------------------------------------------*/
    if((bound_num_array = (int *) malloc (num_bound_rect_recs *
                                          sizeof(int))) == NULL)
    {
        c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
        fclose(toc);
        return(NULL);
    }
    for(i = 0; i < num_bound_rect_recs; i++)
        bound_num_array[i] = -1;

    /* have setup_bound_rects determine the boundary rectangle numbering
    --------------------------------------------------------------------*/
    if(setup_bound_rects(toc,bound_rect_tbl_offset,num_bound_rect_recs,
                         bound_rec_len,locations[RECTTBL].phys_index,
                         num_index_recs,frm_index_tbl_off,index_rec_len,
                         locations[FFTBL].phys_index,filename,
                         bound_num_array) != E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-boundnumber", 
            "Error determining boundary rectangle numbering");
        fclose(toc);
        free(bound_num_array);
        return (NULL);
    }

    /* if the user specified the boundary rectangles to ingest, use
       the zone value for the first boundary rectangle in the array.
       this will be the the most northwesterly boundary rectangle.
    ----------------------------------------------------------------*/
    if(tilenum[0] != -1)
    {
        for(i = (num_bound_rect_recs - 1); i >= 0; i--)
        {
            for(j = 0; tilenum[j] != -1; j++)
            {
                if(bound_num_array[i] == tilenum[j])
                {
                    /* point TOC file pointer to the first boundary rectangle
                       record - skip the product data type, compression ratio
                       and scale values
                    ---------------------------------------------------------*/
                    if(fseek(toc,locations[RECTTBL].phys_index +
                             bound_rect_tbl_offset + (bound_rec_len * i) + 22,
                             SEEK_SET) != 0)
                    {
                        c_lasmsg(FALSE, "cib2las-toc", 
                            "Error moving file pointer in %s",filename);
                        fclose(toc);
                        free(bound_num_array);
                        return (NULL);
                    }
    
                    /* read in the zone code the nw boundary rectangle is in
                    --------------------------------------------------------*/
                    if(fread(zone,1,1,toc) != 1)
                    {
                        c_lasmsg(FALSE, "cib2las-toc", 
                            "Error reading from %s",filename);
                        fclose(toc);
                        free(bound_num_array);
                        return (NULL);
                    }     
                    zoneflag = TRUE;
                }
            }
            if(zoneflag)
                break;
        }
        if(*zone == ' ')
        {
            c_lasmsg(FALSE, "cib2las-zone", 
                "Could not determine zone code from boundary rectangle");
            fclose(toc);
            free(bound_num_array);
            return (NULL);
        }
    }

    /* allocate an array to hold frame file information
    ---------------------------------------------------*/
    if((flist=(FRAMEFILE *)malloc(num_index_recs * sizeof(FRAMEFILE))) == NULL)
    {
        c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
        fclose(toc);
        free(bound_num_array);
        return(NULL);
    } 

    /* update frame file count
    --------------------------*/
    *fcount = num_index_recs;
    
    if(num_index_recs < 1)
    {
        c_lasmsg(FALSE,"cib2las-toc","No frame files found");
        fclose(toc);
        free(bound_num_array);
        free(flist);
        return(NULL);
    } 

    /* Read frame file index records 
    --------------------------------*/
    for(i = 0; i < num_index_recs; i++)
    {
        frameitem=&flist[i];
     
        /* locations[FFTBL] is frame file index table subsection 
        ----------------------------------------------------*/
        if(fseek(toc,(locations[FFTBL].phys_index + index_rec_len*i),SEEK_SET)
           != 0)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error moving file pointer in %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }

        if(fread(&boundary_id,2,1,toc) != 1)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }     
        swap_endian(&boundary_id,2);

        /* read frame record row and column 
        -----------------------------------*/
        if(fread(&frame_row,2,1,toc) != 1)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }     
        swap_endian(&frame_row,2);
        
        if(fread(&frame_col,2,1,toc) != 1)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }     
        
        swap_endian(&frame_col,2);
        
        /* read pathname offset
        -----------------------*/
        if(fread(&path_off,4,1,toc) != 1)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }     
        swap_endian(&path_off,4);

        /* Save file position for later
        -------------------------------*/
        curr = ftell(toc);

        /* Go to start of pathname record */

        /* Add path offset to frame file index table subsection (loc[3])
        ---------------------------------------------------------------*/
        if(fseek(toc,(long)(locations[FFTBL].phys_index + path_off),SEEK_SET)
           != 0)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                    "Error moving file pointer in %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }

        if(fread(&path_length,2, 1, toc) != 1)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }            
        swap_endian(&path_length,2);
        
        if((frameitem->dir = (char *)malloc(path_length + 1 + strlen(dir)))
            == NULL)
        {
            c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
            fclose(toc);
            free(bound_num_array);
            free(flist);
            return(NULL);
        } 
        
        if((directory = (char *) malloc(path_length + 1)) == NULL)
        {
            c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }     
                
        /* 1st part of directory name is passed as arg: e.g. "../RPF2/" 
        ---------------------------------------------------------------*/
        if (dir[strlen(dir)-1] == '1')
            dir[strlen(dir)-1] = '\0';
        
        strcpy(frameitem->dir, dir);

        /* concatenate rpf directory to input directory name
        ----------------------------------------------------*/
        strcat(frameitem->dir,rpfdir);
        
        if(fread(directory,1,path_length,toc) != path_length)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            fclose(toc);
            free(flist);
            free(directory);
            free(bound_num_array);
            return (NULL);
        }            
        
        directory[path_length] = '\0';

        /* concatenate frame file subdirectory below rpf directory
           to frame file directroy,  skip 1st 2 chars: "./": 
        -----------------------------------------------------------*/
        if(strlen(directory) >= 3)
           strcat(frameitem->dir,&directory[2]);
        
        free(directory);

        /* Go back to get filename tail
        -------------------------------*/
        if(fseek(toc,curr,SEEK_SET) != 0)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error moving file pointer in %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }
        
        if(fread(frameitem->filename,1,12,toc) != 12)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }            
        
        frameitem->filename[12] = '\0';
     
        /* Remove last space in name 
        -----------------------------*/
        for(j = 11; j >= 0; --j)
        {
            if(frameitem->filename[j] == ' ')
                frameitem->filename[j] = '\0';   
        }

        /* update boundary record field for frame file 
        -----------------------------------------------*/
        frameitem->brect_record = bound_num_array[boundary_id];

        /* get zone code not set, image specified by default 
           filename is ffffffvp.ccz where z is the zone code
           only do this if the user did not specify any boundary
           rectangles
        ------------------------------------------------------------*/
        if((*zone == ' ') && zoneflag)
        {
            if(strncmp(&frameitem->filename[9],"OVR",3) != 0)
            {
                *zone = toupper(frameitem->filename[11]);

                /* If CIB zone is a polar zone ('9' and 'J') exit
                -------------------------------------------------*/
                if(*zone == '9' || *zone == 'J')
                {
                    c_lasmsg(FALSE,"cib2las-data",
                             "CIB2LAS does not ingest polar zones");
                    fclose(toc);
                    free(flist);
                    free(bound_num_array);
                    return (NULL);
                }
            }
        }

        frameitem->exists = 1;

        /* Update row and column fields in flist[] for this frame file 
        --------------------------------------------------------------*/
        if(locate_frame(*zone,tilenum,frameitem) != E_SUCC)
        {
            fclose(toc);
            free(flist);
            free(bound_num_array);
            return (NULL);
        }
    }/* end for(i=num_index_recs */

    /* close table of contents file and free memory 
    -----------------------------------------------*/
    fclose(toc);
    free(bound_num_array);

    return(flist);
}
/*****************************************************************************
function NAME: setup_bound_rects() 


PURPOSE:   match up the internal boundary rectangle sequence number to the
           numbering of the boundary rectangles/tiles on the CD case image.

PROGRAM HISTORY:

VERSION	   PROGRAMMER      DATE	        CODE/CONT       REASON
-------	   ----------      ----	        ---------       ------
  1.0      A. Shimitz      10/00          CSB          Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:
    initialize max & min lat and longs
    step through each boundary rectangle to determine the outer limits
           of the geographic area covered by the image
    allocate memory for the boundary rectangle matrix and initialize all
           entries to -1
    for each boundary rectangle
        read the rectangle's lat and long values and calculate the average vals
        determine the latitudinal row in the matrix to place the rectangle in
        place the rectangle's sequence number in the appropriate longitudinal
               location in the row just determined
        if a location for the rectangle was not found in the matrix error
    step through the matrix and assign a new number to each boundary rectangle
           to correspond with the numbering found on the graphic on the CD case

ALGORITHM REFERENCES:  none
RETURNS:   bound_num_array will be setup so the boundary rectangle sequence
           number specified in the TOC file is used as the index into the
           array to retrieve its boundary rectangle number on the CD image.
******************************************************************************/
static int setup_bound_rects
(
    FILE  *toc,                /* I: file pointer to TOC file                */
    unsigned short bound_rect_tbl_offset,
                               /* I: offset between the beginning of the
                                     boundary rectangle subsection and the
                                     boundary rectangle table                */
    long  num_bound_rect_recs, /* I: number of boundary rectangles records   */
    unsigned short bound_rec_len,
                               /* I: boundary rectangle record length        */
    long  bound_rect_location, /* I: boundary rectangle subsection physical
                                     location                                */
    unsigned int num_index_recs,
                               /* I: number of frame file index records      */
    unsigned int frm_index_tbl_off,
                               /* I: Frame file index table offset           */
    unsigned short index_rec_len,
                               /* I: frame file index record length          */
    long  ff_location,         /* I: frame file subsection physical location */
    char  *filename,           /* I: TOC filename including path             */
    int   *bound_num_array     /* O: array with new boundary rectangle #'s   */
)
{
    long  i,j;                   /* general loop counters                   */
    long  cur_rect;              /* loop counter for boundary rectangles    */
    long  lat_range;             /* total north-south distance covered by
                                    the boundary rectangles                 */
    long  long_range;            /* total east-west distance covered by
                                    the boundary rectangles                 */
    long  long_index;            /* longitudinal index into the
                                    bound_rect_matrix                       */
    long  lat_index;             /* latitudinal index within the longitudinal
                                    bucket of boundary rectangles           */
    long  whole_max_lat;         /* ceil(max_lat)                           */
    long  whole_min_lat;         /* floor(min_lat)                          */
    long  whole_max_long;        /* ceil(max_long)                          */
    long  whole_min_long;        /* floor(min_long)                         */
    long  b_index;               /* index for the b_list array              */
    int   *bound_rect_matrix;    /* matrix of boundary rectangle geographic
                                    locations                               */
    int   rect_number;           /* new boundary rectangle number           */
    int   b_list[MAXTILES];      /* list containing the boundary rectangle
                                    sequence numbers for those boundary
                                    rectangles that contain the overview
                                    image frame files                       */
    long double max_lat;         /* maximum latitude found in the
                                    boundary rectangles                     */
    long double min_lat;         /* minimum latitude found in the
                                    boundary rectangles                     */
    long double max_long;        /* maximum longitude found in the
                                    boundary rectangles                     */
    long double min_long;        /* minimum longitude found in the
                                    boundary rectangles                     */
    long double cur_upper_lat;   /* current boundary rectangle's greatest
                                    latitude - the greater of the nw and ne */
    long double cur_lower_lat;   /* current boundary rect's least latitude -
                                    the least of the sw and se values       */
    long double cur_upper_long;  /* current boundary rect's greatest
                                    longitude - the greater of nw and sw    */
    long double cur_lower_long;  /* current boundary rect's lease longitude -
                                    the least of the ne and se values       */
    long double avg_lat;         /* average latitudinal val for bound rect  */
    long double avg_long;        /* average longitudinal val for bound rect */
    unsigned short bound_id;     /* boundary rectangle sequence id
                                    the frame file belongs to               */
    char     ff_filename[CMLEN]; /* string representing the current frame
                                    file filename                           */

    /* set intial values for max_lat, min_lat, max_long, and min_long.  We will
       use decimal degree values, since that is how they are represented in
       the TOC file.
    -------------------------------------------------------------------------*/
    max_lat = -90.0;
    min_lat = 90.0;
    max_long = -180.0;
    min_long = 180.0;

    /* identify those boundary rectangle that contain overview image frame
       files since we do not want to include these in our matrix
    ----------------------------------------------------------------------*/
    b_index = 0;
    for(i = 0; i < num_index_recs; i++)
    {
        if(fseek(toc,(long)(ff_location + frm_index_tbl_off +
                 (index_rec_len * i)), SEEK_SET) != 0)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error moving file pointer in %s",filename);
            return (E_FAIL);
        }
        if(fread(&bound_id,1,2,toc) != 2)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            return (E_FAIL);
        }
        swap_endian(&bound_id,2);
        if(fseek(toc,8,1) != 0)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            return (E_FAIL);
        }
        if(fread(&ff_filename,1,12,toc) != 12)
        {
            c_lasmsg(FALSE, "cib2las-toc", 
                "Error reading from %s",filename);
            return (E_FAIL);
        }            
        
        ff_filename[12] = '\0';
     
        /* Remove last space in name 
        -----------------------------*/
        for(j = 11; j >= 0; --j)
        {
            if(ff_filename[j] == ' ')
            {
                ff_filename[j] = '\0';   
                break;
            }
        }

        /* if this frame file is for an overview image, add its boundary
           id to the b_list
        ----------------------------------------------------------------*/
        if(strncmp(&ff_filename[9],"OVR",3) == 0)
        {
            b_list[b_index] = bound_id;
            b_index++;
        }
    }

    /* mark the end of the array
    ----------------------------*/
    b_list[b_index] = -1;

    /* step through each boundary rectangle to find the greatest nw_lat and
       nw_long as well as the lowest se_lat and se_long. these values will
       be used to place the boundary rectangles in the bound_rect_matrix.
       Don't include the boundary rectangles that are for the overview
       images - their boundary rectangle sequence number will be in the b_list
    --------------------------------------------------------------------------*/
    for(cur_rect = 0; cur_rect < num_bound_rect_recs; cur_rect++)
    {
        if(!in_list(cur_rect,b_list))
        {
            cur_upper_lat = 0.0;
            cur_lower_lat = 0.0;
            cur_upper_long = 0.0;
            cur_lower_long = 0.0;
    
            /* point TOC file pointer to the next boundary rectangle
               record - skip the product data type, compression ratio,
               scale, zone, and producer values
            ----------------------------------------------------------*/
            if(fseek(toc,bound_rect_location + bound_rect_tbl_offset +
                     (bound_rec_len * cur_rect) + 28,SEEK_SET) != 0)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error moving file pointer in %s",filename);
                return (E_FAIL);
            }
    
            /* read the current boundary rectangle's lat and long values and
               modify the min and max lat and long values if needed.  
            ----------------------------------------------------------------*/
            if(fread(&cur_upper_lat,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                return (E_FAIL);
            }
            swap_endian(&cur_upper_lat,8);
            if(fread(&cur_lower_long,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                return (E_FAIL);
            }
            swap_endian(&cur_lower_long,8);
            if(fread(&cur_lower_lat,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                return (E_FAIL);
            }
            swap_endian(&cur_lower_lat,8);
            if(fseek(toc,16,1) != 0)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                return (E_FAIL);
            }
            if(fread(&cur_upper_long,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                return (E_FAIL);
            }
            swap_endian(&cur_upper_long,8);
    
            /* compare current boundary rectangle lat/long limits to
               max/min vals
            --------------------------------------------------------*/
            if(cur_upper_lat > max_lat)
                max_lat = cur_upper_lat;
            if(cur_lower_lat < min_lat)
                min_lat = cur_lower_lat;
            if(cur_upper_long > max_long)
                max_long = cur_upper_long;
            if(cur_lower_long < min_long)
                min_long = cur_lower_long;
        }
    } /* end for each boundary rectangle */
    
    /* calculate the floor and ceil of the min and max lat and long
    ---------------------------------------------------------------*/
    whole_min_lat = (long)floor(min_lat);
    whole_max_lat = (long)ceil(max_lat);
    whole_min_long = (long)floor(min_long);
    whole_max_long = (long)ceil(max_long);

    /* allocate memory for the boundary rectangle matrix and initialize all
       values to -1
    -----------------------------------------------------------------------*/
    lat_range = whole_max_lat - whole_min_lat;
    long_range = whole_max_long - whole_min_long;
    if((bound_rect_matrix = (int *) malloc (long_range * lat_range *
                                            sizeof(int))) == NULL)
    {
        c_lasmsg(FALSE,"cib2las-alloc","Error allocating dynamic memory");
        return(E_FAIL);
    }
    for(i = 0; i < (long_range * lat_range); i++)
       bound_rect_matrix[i] = -1;

    /* sort into buckets by latitude
    --------------------------------*/
    for(cur_rect = 0; cur_rect < num_bound_rect_recs; cur_rect++)
    {
        if(!in_list(cur_rect,b_list))
        {
            /* point TOC file pointer to the next boundary rectangle
               record - skip the product data type, compression ratio,
               scale, zone, and producer values
            ----------------------------------------------------------*/
            if(fseek(toc,bound_rect_location + bound_rect_tbl_offset +
                     (bound_rec_len * cur_rect) + 28,SEEK_SET) != 0)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error moving file pointer in %s",filename);
                free(bound_rect_matrix);
                return (E_FAIL);
            }    
    
            /* read the current boundary rectangle's lat and long values and
               calculate the average lat and average long values
            ----------------------------------------------------------------*/
            if(fread(&cur_upper_lat,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                free(bound_rect_matrix);
                return (E_FAIL);
            }
            swap_endian(&cur_upper_lat,8);
            if(fread(&cur_lower_long,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                free(bound_rect_matrix);
                return (E_FAIL);
            }
            swap_endian(&cur_lower_long,8);
            if(fread(&cur_lower_lat,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                free(bound_rect_matrix);
                return (E_FAIL);
            }
            swap_endian(&cur_lower_lat,8);
            if(fseek(toc,16,1) != 0)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                free(bound_rect_matrix);
                return (E_FAIL);
            }
            if(fread(&cur_upper_long,1,8,toc) != 8)
            {
                c_lasmsg(FALSE, "cib2las-toc", 
                    "Error reading from %s",filename);
                free(bound_rect_matrix);
                return (E_FAIL);
            }
            swap_endian(&cur_upper_long,8);
    
            avg_lat = (cur_upper_lat + cur_lower_lat) / 2.0;
            avg_long = (cur_upper_long + cur_lower_long) / 2.0;
    
            /* determine the latitudinal row to place the rectangle in
            ----------------------------------------------------------*/
            lat_index = (long)floor(whole_max_lat - avg_lat) * long_range;
            long_index = (long)floor(avg_long - whole_min_long);
    
            /* verify that the index into the array does not exceed its limit
            -----------------------------------------------------------------*/
            if((lat_index + long_index) >= (lat_range * long_range))
            {
                c_lasmsg(FALSE, "cib2las-boundnumber", 
                         "Boundary rectangle position within matrix "
                         "exceeds limit");
                free(bound_rect_matrix);
                return (E_FAIL);
            }
    
            /* if another boundary rectangle has already been
               placed in this location, we cannot put another one
               here so error out
            -----------------------------------------------------*/
            if(bound_rect_matrix[lat_index + long_index] != -1)
            {
                c_lasmsg(FALSE, "cib2las-boundnumber", 
                         "Attempting to place multiple boundary "
                         "rectangle","references into same matrix "
                         "location");
                free(bound_rect_matrix);
                return (E_FAIL);
            }
            bound_rect_matrix[lat_index + long_index] = cur_rect;
        }
    } /* end for each boundary rectangle record */

    /* now assign the numbers to the boundary rectangles to correspond with
       the numbers on the CD case insert
    -----------------------------------------------------------------------*/
    rect_number = 1; /* starts at 1 because the numbering on the CD does */
    for(i = 0; i < (long_range * lat_range); i++)
    {
        if(bound_rect_matrix[i] != -1)
        {
            bound_num_array[bound_rect_matrix[i]] = rect_number;
            rect_number++;
        }
    }
    free(bound_rect_matrix);
    return (E_SUCC);
}

/*****************************************************************************
function NAME: in_list() 


PURPOSE:   determine if the current boundary rectangle sequence number is in
           the list of those that contain only overview image frame files.

PROGRAM HISTORY:

VERSION	   PROGRAMMER      DATE	        CODE/CONT       REASON
-------	   ----------      ----	        ---------       ------
  1.0      A. Shimitz      10/00          CSB          Original Development
                                                       (CR 4215)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS

ALGORITHM DESCRIPTION:
RETURNS:    TRUE/FALSE
******************************************************************************/
static int in_list
(
    int cur_id,                /* I: current boundary rectangle sequence id  */
    int b_list[MAXTILES]       /* I: the list of boundary rectangle sequence
                                     numbers for those that contain overview
                                     image frame files only                  */
)
{
    long i;                    /* counter                                    */

    /* compare the current boundary rectangle sequence id to each
       id known to be a boundary rectangle for an overview image
    -------------------------------------------------------------*/
    for(i = 0; b_list[i] != -1; i++)
    {
        /* if the id is in the list, return TRUE
        ----------------------------------------*/
        if(cur_id == b_list[i])
            return(TRUE);
    }

    /* the id was not found in the list so this is not a boundary rectangle
       for an overview image
    -----------------------------------------------------------------------*/
    return (FALSE);
}
