/****************************************************************************
FILENAME:    update_ddr.c

FUNCTION(S): update_ddr()
             get_truescalelat()

PURPOSE:  Write out bdr and ddr information for CIB ingest routine 

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      B. Gross        6/00            CSB          original development 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    run under LAS/TAE

PROJECT:    LAS
ALGORITHM DESCRIPTION:

    set image corner values
    set datum code WGS-84 by defn. CIB
    get semi-major of datum
    set projection paramaters
    set valid flags
    set projection pixel-pixel dimensions
    set bdr info
    save ddr file
    save ddr and bdr.
    set last used date/time

ALGORITHM REFERENCES:  NONE

******************************************************************************/
#include<ctype.h>
#include<string.h>
#include <stdlib.h> /* getenv prototype */
#include"las.h"
#include"util.h"
#include"geompak.h"
#include"worgen.h"
#include"math.h"
#include"cproj.h"
#include"cib2las.h"
#include"cibframe.h"
#include"update_ddr.h"

int update_ddr
(
    char *fname,              /* I: filename for image file                 */
    INPARAMS *in,             /* I: paramaters sl, ss, nl, ns etc.          */
    struct CIBCoverage *out,  /* I: corner coordinates,lat,long,res etc.    */ 
    unsigned char *minval,    /* I: minvalue in band if flag set            */
    unsigned char *maxval,    /* I: maxvalue in band if flag set            */ 
    int minmax_flg,           /* I: value of global minmax flag             */
    char *datasource,         /* I: image datasource                        */
    char cibzone,             /* I: CIB zone of image                       */
    char units                /* I: output units, 1 = degrees, 0 = meters   */
)
{
    char   tempstr[15];
    char   *env_ptr;          /* pointer to environment variables           */
    int    i;
    long   length;            /* length of string used for case conversions */
    double lat_truescale;     /* projection Latitude of true scale          */
    double f_zero;            /* intermediate variable used for conver
                                 sion from degrees to dms for fe,fn, 
                                 and cm                                     */
    struct DATUMDEF dtmparm;  /* struct to hold datum info.                 */
    struct DDR ddr;           /* struct to hold ddr info.                   */
    struct BDDR bddr;         /* struct to hold bder info.                  */

    /* initialize output ddr and bdr information 
    --------------------------------------------*/    
    c_intddr(&ddr);
    c_intbdr(&bddr);

    ddr.ns = in->ns;
    ddr.nl = in->nl;
    ddr.nbands = 1;           /* CIB images are monochromatic               */
    ddr.dtype = EBYTE;       
    ddr.zone_code = 62;       /* 62 is the invalid zone code                */

    ddr.proj_code = 17;       /* CIB Non-Polar Zone ARC Projections - 
                                 Equirectangular Proj. p.g. 90
                                 Map Projections - a working Manual 
                                 U.S.G.S 1395  */
    /* update ddr datum code, by definition CIB 
       horizontal datum is WGS-84 LAS datum 317
    -------------------------------------------*/
    ddr.datum_code = 317; 
 
    /* Retrieve the datum structure
    -------------------------------*/
    dtmparm.datumnum = ddr.datum_code;
    if (c_getdatum(&dtmparm) != E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-datum","Error retrieving datum info");
        return (E_FAIL);
    }

    /* Use the datum structure to set the semi-major axis 
    -----------------------------------------------------*/
    ddr.proj_coef[SMAJOR] = dtmparm.smajor;

    /* update ddr semi-minor axis because c_traninit will not run under
       XID without it, but not necessary for Equirectangular Projection
       by definition (c_transinit checks major and minor axis values 
       against correct values for datum in ddr.)
    -------------------------------------------------------------------*/
    ddr.proj_coef[SMINOR] = dtmparm.sminor;
 
    /* Central Meridian for ARC projection system, Non-Polar Zone Projection
       - Equirectangular projection Central Meridain is located at 0.00 
       degress West Longitude (from MIL-A-89007 ARC Digitized Raster Graphics
       Fig. 22 pg. 68 )
    -------------------------------------------------------------------------*/
    f_zero = CIB_CENT_MER;
    if(c_degdms(&f_zero,&ddr.proj_coef[CENTMER],"DEG","LON") != E_SUCC)
    {
         c_lasmsg(FALSE,"cib2las-ddr","Error setting Central Meridian");
         return (E_FAIL);
    }
 
    /* Get True Scale Latitude (Latitude of True Scale)
    ---------------------------------------------------*/
    if(get_truescalelat(cibzone,&lat_truescale) != E_SUCC)
    {
        c_lasmsg(FALSE, "cib2las-tae", 
            "Error retrieving Latitude of True Scale for Zone %c",cibzone);
        return (E_FAIL);
    }

    if(c_degdms(&lat_truescale,&ddr.proj_coef[TRUESCALE],"DEG","LAT") != E_SUCC)
    {
         c_lasmsg(FALSE,"cib2las-ddr","Error setting True Scale Latitude");
         return (E_FAIL);
    }
     
    /* convert false easting and false northing to packed DMS data
       in DDR projection coordinates must be stored in packed DMS.
    ---------------------------------------------------------------*/
    f_zero = CIB_FE;
    if(c_degdms(&f_zero,&ddr.proj_coef[FE],"DEG","LAT") != E_SUCC)
    {
         c_lasmsg(FALSE,"cib2las-ddr","Error setting False Easting");
         return (E_FAIL);
    }

    f_zero = CIB_FN;
    if(c_degdms(&f_zero,&ddr.proj_coef[FN],"DEG","LON") != E_SUCC)
    {
         c_lasmsg(FALSE,"cib2las-ddr","Error setting False Northing");
         return (E_FAIL);
    } 

    /* update DDR code validity flags
    ---------------------------------*/
    for(i = 0; i < 8; i++)
        ddr.valid[i] = VALID;

    /* update ddr projection units value, CIB data listed degrees by defn,
       converted to meters (above) due to limitations of gctpc
    ----------------------------------------------------------------------*/
    if(units == 0)
    {
        strcpy(ddr.proj_units,"meters");

        ddr.upleft[0]  = out->ULy;
        ddr.upleft[1]  = out->ULx;
        ddr.upright[0] = out->URy;
        ddr.upright[1] = out->URx;
        ddr.loright[0] = out->LRy;
        ddr.loright[1] = out->LRx;
        ddr.loleft[0]  = out->LLy;
        ddr.loleft[1]  = out->LLx;
       
        /* update ddr pixel - pixel dimensions 
        --------------------------------------*/
        ddr.pdist_y = out->yRes; 
        ddr.pdist_x =  out->xRes; 
    } 
    else
    {
        strcpy(ddr.proj_units,"degrees");
        ddr.upleft[0]  = out->ULLat;
        ddr.upleft[1]  = out->ULLong;
        ddr.upright[0] = out->URLat;
        ddr.upright[1] = out->URLong;
        ddr.loright[0] = out->LRLat;
        ddr.loright[1] = out->LRLong;
        ddr.loleft[0]  = out->LLLat;
        ddr.loleft[1]  = out->LLLong;
       
        /* update ddr pixel - pixel dimensions 
        --------------------------------------*/
        ddr.pdist_y = out->LatRes; 
        ddr.pdist_x =  out->LongRes; 
    }

    
    /* update ddr source of data
    ----------------------------*/
    ddr.master_line = 1;
    ddr.master_sample = 1;
    ddr.line_inc = 1.0;
    ddr.sample_inc = 1.0;

    bddr.bandno = 1;           /* only one band in CIB data                 */
    if(!minmax_flg)
    {
        *minval = *maxval = 0;
        bddr.valid = INVAL;
    }
    else
        bddr.valid = VALID;
    bddr.minval = *minval;     /* minimum value of the band                 */
    bddr.maxval = *maxval;     /* maximum value of the band                 */
   
    /* get the system from the DATASYS environment variable 
    --------------------------------------------------------*/
    env_ptr = getenv("DATASYS");
    
    if (env_ptr != NULL)
        strncpy(ddr.system,env_ptr,12); /* 12 length */
    else
    {
        c_lasmsg(FALSE,"cib2las-env","Error DATASYS undefined");
        return (E_FAIL);
    }
 
    /* format data source for ddr 
    -----------------------------*/   
    length = (long)strlen(datasource);
    c_up2low(datasource,&length);
    
    /* if source is landsat, it is formatted "Landsat7",
        convert it to "landsat-7"
    ------------------------------------------------------------*/
    if (strncmp(datasource,"landsat",7) == 0)
    {
        strncpy(bddr.source,datasource,7);
        if(length > 7)
        {
            sprintf(tempstr,"-%c",datasource[8]);
            strcat(bddr.source,tempstr);
        }
    }   
    else
    {
        strcpy(bddr.source,datasource); 
    }   
  
    if(c_putddr(fname,&ddr) != E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-ddr","Error writing DDR %s",fname);
        return (E_FAIL);
    }

    if(c_putbdr(fname,&bddr) != E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-bdr","Error writing band DDR %s",fname);
        return (E_FAIL);
    }
    
    /* set the last date/time used field 
    ------------------------------------*/
    if(c_lused(fname)!=E_SUCC)
    {
        c_lasmsg(FALSE,"cib2las-ddr","Error Updating Last Used fields");
        return (E_FAIL);
    }
    return (E_SUCC);
}

/* Get True Scale Latitude, (Latitude of no distortion [shrink or stretch])

   Function takes alphanumeric value representing CIB zone which contains
   the CIB image, and returns the Latitude of true scale for that CIB zone.

ALGORITHM REFERENCES:  

   Latitudes of True Scale taken from MIL-A-89007 ARC Digitized Raster
   Graphics (ADRG) Appendix Table IV Page 64.
----------------------------------------------------------------------------*/
int get_truescalelat
(
    char zone,             /* I: alphanumeric value representing CIB 
                                 zone containing the image                  */
    double *truescalelat   /* O: Latitude of true scale                     */
)
{
    switch(toupper(zone))
    {
        case '1': *truescalelat =  22.94791772; return (E_SUCC);
        case 'A': *truescalelat = -22.94791772; return (E_SUCC);
        case '2': *truescalelat =  41.12682127; return (E_SUCC);
        case 'B': *truescalelat = -41.12682127; return (E_SUCC);
        case '3': *truescalelat =  52.28859923; return (E_SUCC);
        case 'C': *truescalelat = -52.28859923; return (E_SUCC);
        case '4': *truescalelat =  60.32378942; return (E_SUCC);
        case 'D': *truescalelat = -60.32378942; return (E_SUCC);
        case '5': *truescalelat =  66.09421768; return (E_SUCC);
        case 'E': *truescalelat = -66.09421768; return (E_SUCC);
        case '6': *truescalelat =  70.10896259; return (E_SUCC);
        case 'F': *truescalelat = -70.10896259; return (E_SUCC);
        case '7': *truescalelat =  74.13230145; return (E_SUCC);
        case 'G': *truescalelat = -74.13230145; return (E_SUCC);
        case '8': *truescalelat =  78.17283750; return (E_SUCC);
        case 'H': *truescalelat = -78.17283750; return (E_SUCC);
        
        /* n/a for polar zones, polar data in a different projection 
        ------------------------------------------------------------*/
        case '9': 
        case 'J': 
        default : return (E_FAIL);
        }
}
