#include <stdio.h>
#include <string.h>
#include "las.h"
#include "externs.h"
#include "util.h"
#include "protos.h"
/******************************************************************************

NAME:	CLASSMAP

FUNCTION:   Controls overall class map formatted picture generation.

PROGRAM HISTORY:
  Version	Date       Author      	           Request
  -------	----	   ------                  -------
   1.0        19-Feb-82    J. Hwang
   2.0         7-Jan-88    M. Mazzella, SAR/GSFC   Conversion to C     
   3.0         4-Apr-89    K. Narayanan, STX       Corrected getpar, getimg,
                                                   gensqr, gentik, and genchr
                                                   routines, and array 
                                                   declarations
   4.0        27-Jun-89    K. Narayanan, STX       Corrected genchr for
                                                   annotation on non-vms 
                                                   systems
   5.0	       2-Nov-89.   P. Pease  		   Changed classmap.c to call 
						   c_inlas instead of inlas.
   6.0        20-Dec-90    K. Donaldson            Added comments; Updated
                                                   error messages; added code
                                                   to handle DLTF's
   7.0        05-Feb-91    K. Donaldson            Added code to process only
                                                   the 1st band if input image
                                                   has multiple bands.
   8.0        08-Oct-92	   S. Nelson               Made correction to getpar.c
						   to accept null value for 
						   INDLTF.
   9.0        18-Oct-99    D. Hames                Remove reference to MAXNS
                                                   and replace with ns to
                                                   support images >2gig.
                                                   Add prototypes.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):

   Initialize tae.
   Call INIT to intialize the picture description table.
   Call GETINF to get tae parameters.
   Call OPNIMG to open image files.
   Call GENANO to generate annotation.
   Call GENPIC to generate the output image.
   Close the input and output images.
   Free the dynamic memory.
   Calculate the min/max values of output image.
   Generate the output image history and print the completion message.

******************************************************************************/

unsigned char	*bufferin;	/* Imageio buffer for input image */
unsigned char 	*bufferout;	/* Imageio buffer for output image */
unsigned char	*workbuf;	/* Scratch work buffer */

long	fdescin[2],		/* Input file descriptor array for imageio */
	fdescout[2],		/* Output file descriptor array for imageio */
	gdescin,		/* Input group descriptor for imageio */
	gdescout,		/* Input group descriptor for imageio */
	offset[4];		/* Array of offsets into the input image */
struct  PARBLK  block;          /* TAE parameter block */

static  char    *prog_name = "CLASSMAP"; /* Program name */
char    clean_files[2][CMLEN];  /* Clean file names */
char    out_files[2][CMLEN];    /* Output file names */

long    min_max;                /* Minmax option flag */
long    num_outfiles;           /* Number of output files */

static  long    fatal_err = -1;         /* Fatal error return status */
static  long    inf_err = 1;            /* Info error return status  */

static  long    one = 1;
static  long    three = 3;


void main
(void) /* classmap */
{

char    clschr[32][8],		/* Class names  */
	dltfentr[CMLEN],	/* User specified dltfentry */
	fname[CMLEN],		/* Facility name */
	hosin[CMLEN],		/* Input image host name */
	hosout[CMLEN],		/* Output image host name */
	hsinfo[12],		/* Info to be added to history file */
	hstext[450],		/* Text to output to history file */
	indltf[CMLEN],		/* Dltf name */
	taeout[CMLEN],		/* Output image tae name */
	usrano[63];		/* User supplied annotation */

double  *amax,			/* Maximum values in the image */
	*amin;			/* Minimum values in the image */

long    *bands, 		/* Number of band in the input image */
	gry[2],			/* Holds line numbers for grey scale on image */
	i,			/* Loop counter */
	imgsiz[4],		/* Image dimensions */
	msgtim,			/* Time for the processing message */
	numcls,			/* Number of input classes */
	status,			/* Retrun status */
	usrcol_flg,		/* Flag for user specified colors      */
	window[4],		/* Image window  specification (SL,SS,NL,NS) */
        ns;			/* total number of samples in image */

static long
       num_img_bands[MAXIMG][MAXBND+1]; /* Band numbers for each image */

unsigned char	class[32];	/* Input color numbers for each class */

struct	DDR	ddr;		/* Data descriptor record */


num_outfiles = 0;

/*
Initialize tae.
----------------
*/
c_inlas(&block, &status); 

/*
Call INIT to intialize the picture description table.
-------------------------------------------------------
*/
initc();

/*
Allocate memory.
----------------
*/

if (((amax = (double *) malloc(MAXBND * sizeof(double))) == NULL)||
    ((amin = (double *) malloc(MAXBND * sizeof(double))) == NULL)||
    ((bands = (long *)malloc(MAXIMG * sizeof(long))) == NULL))
    c_errmsg ("!Error allocating dynamic memory", "classmap-alloc",&inf_err);

/*
Call GETINF to get tae parameters.
----------------------------------
*/
getinf (hosin, window, bands, num_img_bands, &ddr, usrano, taeout, hosout, 
        clschr, class, fname, &msgtim, imgsiz, dltfentr, indltf, 
        &usrcol_flg, &numcls, &ns);

/* Allocate memory for workbuf
-----------------------------
*/
if  ((workbuf=(unsigned char *)malloc(ns * 4)) == NULL)
    c_errmsg ("!Error allocating dynamic memory", "classmap-alloc",&inf_err);


/*
Call OPNIMG to open image files.
--------------------------------
*/
opnimg (hosin, taeout, &ddr, num_img_bands, window, offset, hosout, imgsiz);

/*
Call GENANO to generate annotation.
-----------------------------------
*/
genano (indltf, dltfentr, usrano, clschr, class, gry, fname, usrcol_flg);

/*
Call GENPIC to generate the output image.
------------------------------------------
*/
genpic (imgsiz, msgtim, gry); /* class, clschr);*/

/*
Close the input and output images.
----------------------------------
*/
status = c_egclse ((struct GDESC **)&gdescin);
if (status != E_SUCC)
    c_errmsg("Error closing input image","classmap-inclose", &fatal_err);

status = c_egclse ((struct GDESC **)&gdescout);
if (status != E_SUCC)
    {
    c_errmsg("Error closing output image","classmap-outclose", &inf_err);
    c_clean(&block,"classmap",(char (*)[256])hosout,&one);
    }

/*
Calculate the min/max values of output image.
---------------------------------------------
*/
if (min_max)
   {
   imgsiz[SL] = imgsiz[SS] = one;
   bands[0]=1;
   bands[1]=2;
   bands[2]=3;
   bands[3]=0;
   c_minmax ((char (*)[256])hosout, &one, (long (*)[4])imgsiz,
	     (long (*)[257])bands, &three, amin, amax);  
   c_svmnmx (hosout, &three, amin, amax, "YES");
   }

/*
Free the dynamic memory.
------------------------
*/
free (bufferin);
free (bufferout);
free (workbuf);
free (amax);
free (amin);
free (bands);

/*
Generate the output image history and print the completion message.
--------------------------------------------------------------------
*/

if (usrcol_flg)
    {
    strcpy (hstext, "User specified lut as follows:");
    for (i=0;i<numcls;i++)
	{
        sprintf (hsinfo, "%s:%2d,",clschr[i],class[i]);
	strncat(hstext, hsinfo, 11);
	}
    c_hstout (&block, hosout, (char (*)[256])hosin, &one, hstext); 
    }
else
    {
    c_hstout (&block, hosout, (char (*)[256])hosin, &one, " ");
    }
c_complt (prog_name, (char (*)[256])taeout, &one); 

}
