#include <stdio.h>
#include "las.h"
#include <string.h>  
#include "dltf.h"
#include "util.h"
#include "protos.h"
/******************************************************************************

NAME:  GENANO

FUNCTION:   Generate annotations and build the lookup table.

PROGRAM HISTORY:
  Version	Date       Author      		  Request
  -------	----	   ------            	  -------
   1.0       19-Feb-82    J. Hwang/SASC
   2.0        7-Jan-88    M. Mazzella SAR/GSFC    Convert to c
   3.0       20-Dec-90    K. Donaldson            Added more informative 
                                                  comments; added code to 
                                                  handle DLTF's
   4.0       15-Oct-98    D. Hames                Add code to comply
                                                  with localtime & y2k 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):

   Get the grey scale vertical range (this portion of picture should not 
      go through the lookup table.
   Get history label (3 lines)
   Get the look up table.
   Inform the user which LUT will be used.
   If user defined LUT is being used, 
      Store the gray level values corresponding to the selected colors in 
      the lookup table; 
      Remove from the lookup table any gray level values for the default 
      colors.
   End-if
   Save the user defined annotation.
   Save the facility name and generation date in the last line of annotation.
   Place the class names and colors (user defined or default) into the 
      character picture descripiton table.

******************************************************************************/

void genano
(
    char indltf[],              /* Dltf (lookup table file) name */
    char dltfentry[],           /* Dltf entry */
    char usrano[],              /* User defined annotation */
    char clschr[][8],           /* Class names */
    unsigned char class[],      /* Color numbers for classes */
    long gry[2],                /* Line numbers of grey scale on image */
    char fname[],               /* Facility name to be printed on image */
    long usrcol_flg             /* Flag for user specified colors      */
)

{
#include "externs.h"
#include "classmap.h"
#include <time.h>

struct	tm	*loc_time;	/* Structure for time */
struct	ASDLTF dpf_info;	/* DLTF field information */

char	desc[81],		/* User specified description of DLTF entry */
	dpf_type[3],		/* Data type of the DLTF entry */
	lookup[3][256];		/* Buffer for data portion of  DLTF entry */

unsigned char	tcol[3][256];	/* Temporary buffer for lookup data */

double	offset;			/* Offset of the LUT in the DLTF entry */
double	slope;			/* Slope of the LUT in the DLTF entry */

long    day, month, year,	/* Date elements    */
	default_dltf,		/* Default DLTF flag - set to TRUE if DLTF */
				/* file name was not specified */
	dpf_len,		/* Length of data portion of second DLTF */
                                /* record */
	end_lut[3],		/* Last entry in LUT data */
	fd_dltf,		/* File descriptor for DLTF file */
	i,j,			/* Loop counters and array indexes */
	lbnd,			/* Lookup table band counter */
	lut_bands,		/* Number of bands in the lookup table */
	secs,			/* Time in seconds */
	start_lut[3];		/* First entry in the LUT data */

/*
Initialize tcol.
-----------------
*/
for (i = 0; i < 3; i++)
   for (j = 0; j < 256; j++)
      tcol[i][j] = (unsigned char)zero;

/*
Get the grey scale vertical range (this portion of picture should not 
go through the lookup table.
----------------------------------------------------------------------
*/
gry[0] = tb[160];
gry[1] = tb[160] + tb[162] - 1;

/*
Get history label (3 lines)
----------------------------
*/
for (i = 0; i < hist1[6]; i++)
   {
   chr[hist1[7]+i-1] = ' ';
   chr[hist2[7]+i-1] = ' ';
   chr[hist3[7]+i-1] = ' ';
   }

/*
Get the look up table.
----------------------
*/
if ((strlen (indltf) == 0) || (usrcol_flg)) default_dltf = TRUE;
else 
   {
   default_dltf = FALSE;
   if (c_r1dltf (&fd_dltf, indltf, dltfentry, desc, &dpf_info, 
                 &dpf_len, dpf_type) != E_SUCC)
      c_abort (&block, "Error reading DLTF", "classmap-dltfread", prog_name,
               clean_files, &one);
   else
      {
      lut_bands = 3;
      for (lbnd = 1; lbnd <= lut_bands; lbnd++)
         {
         if (c_r2dltf (&fd_dltf, dltfentry, &lbnd, &start_lut[lbnd-1], 
                    &end_lut[lbnd-1], &dpf_len, (unsigned char *)lookup[lbnd-1],
	            &slope, &offset) != E_SUCC)
            c_abort (&block, "Error reading DLTF", "classmap-dltfread", 
                     prog_name, clean_files, &one);
         }
      
      for (i = 0; i < 3; i++)
         {
         if (!usrcol_flg)
            {
            for (j = 0; j < 256; j++)
               coltbl[i][j] = lookup[i][j];
            }
         coltbl[i][0] = 0;
         coltbl[i][255] = 255;
         }
      }
   }

/*
Inform the user which LUT will be used.
---------------------------------------
*/
if (usrcol_flg)
   printf ("\nThe color LUT defined by the user is being used");
else
   {
   if (default_dltf)
      printf ("\nDefault color LUT is being used");
   else
      printf ("\nThe LUT in input DLTF is being used");
   }

/*
If user defined LUT is being used, store the gray level values corresponding
to the selected colors in the lookup table; remove from the lookup table
any gray level values for the default colors.
----------------------------------------------------------------------------
*/
if (usrcol_flg)
   {
   for (j = 0; j < 3; j++)
      {
      for (i = 0; i < 32; i++)
         tcol[j][i+1] = coltbl[j][class[i]];
      }
   for (j = 0; j < 3; j++)
      {
      for (i = 0; i <= 32; i++)
         coltbl[j][i] = tcol[j][i];
      }
   }

/*
Save the user defined annotation.
---------------------------------
*/

strncpy(&chr[usanot[7]-1], usrano, usanot[6]);

/*
Save the last line annotation.
------------------------------
*/
time (&secs);
loc_time = localtime (&secs);
month = loc_time->tm_mon + 1;
day = loc_time->tm_mday;
year = loc_time->tm_year; 
year = year%100;                          /* used to retrieve 2 year digits for 
                                            dates after the year 2000 */

sprintf (&chr[lstano[7]-1], "%2d/%2d/%02d", month, day, year); 
strcpy (&chr[lstano[7]-1+9], fname);

/*
Place the class names and colors (user defined or default) into the 
character picture descripiton table.
--------------------------------------------------------------------
*/
strcpy(&chr[clas01[7]-1], clschr[0]);
clas01[5] = 1;
cls001[4] = 1;
strcpy(&chr[clas02[7]-1], clschr[1]);
clas02[5] = 2;
cls002[4] = 2;
strcpy(&chr[clas03[7]-1], clschr[2]);
clas03[5] = 3;
cls003[4] = 3;
strcpy(&chr[clas04[7]-1], clschr[3]);
clas04[5] = 4;
cls004[4] = 4;
strcpy(&chr[clas05[7]-1], clschr[4]);
clas05[5] = 5;
cls005[4] = 5;
strcpy(&chr[clas06[7]-1], clschr[5]);
clas06[5] = 6;
cls006[4] = 6;
strcpy(&chr[clas07[7]-1], clschr[6]);
clas07[5] = 7;
cls007[4] = 7;
strcpy(&chr[clas08[7]-1], clschr[7]);
clas08[5] = 8;
cls008[4] = 8;
strcpy(&chr[clas09[7]-1], clschr[8]);
clas09[5] = 9;
cls009[4] = 9;
strcpy(&chr[clas10[7]-1], clschr[9]);
clas10[5] = 10;
cls010[4] = 10;
strcpy(&chr[clas11[7]-1], clschr[10]);
clas11[5] = 11;
cls011[4] = 11;
strcpy(&chr[clas12[7]-1], clschr[11]);
clas12[5] = 12;
cls012[4] = 12;
strcpy(&chr[clas13[7]-1], clschr[12]);
clas13[5] = 13; 
cls013[4] = 13;                
strcpy(&chr[clas14[7]-1], clschr[13]);
clas14[5] = 14;
cls014[4] = 14;
strcpy(&chr[clas15[7]-1], clschr[14]);
clas15[5] = 15;
cls014[5] = 15;
strcpy(&chr[clas16[7]-1], clschr[15]);
clas16[5] = 16;
cls016[4] = 16;
strcpy(&chr[clas17[7]-1], clschr[16]);
clas17[5] = 17;
cls017[4] = 17;
strcpy(&chr[clas18[7]-1], clschr[17]);
clas18[5] = 18;
cls018[4] = 18;
strcpy(&chr[clas19[7]-1], clschr[18]);
clas19[5] = 19;
cls019[4] = 19;
strcpy(&chr[clas20[7]-1], clschr[19]);
clas20[5] = 20;
cls020[4] = 20;
strcpy(&chr[clas21[7]-1], clschr[20]);
clas21[5] = 21;
cls021[4] = 21;
strcpy(&chr[clas22[7]-1], clschr[21]);
clas22[5] = 22;
cls022[4] = 22;
strcpy(&chr[clas23[7]-1], clschr[22]);
clas23[5] = 23;
cls023[4] = 23;
strcpy(&chr[clas24[7]-1], clschr[23]);
clas24[5] = 24;
cls024[4] = 24;
strcpy(&chr[clas25[7]-1], clschr[24]);
clas25[5] = 25;
cls025[4] = 25;
strcpy(&chr[clas26[7]-1], clschr[25]);
clas26[5] = 26;
cls026[4] = 26;
strcpy(&chr[clas27[7]-1], clschr[26]);
clas27[5] = 27;
cls027[4] = 27;
strcpy(&chr[clas28[7]-1], clschr[27]);
clas28[5] = 28;
cls028[4] = 28;
strcpy(&chr[clas29[7]-1], clschr[28]);
clas29[5] = 29;
cls029[4] = 29;
strcpy(&chr[clas30[7]-1], clschr[29]);
clas30[5] = 30;
cls030[4] = 30;
strcpy(&chr[clas31[7]-1], clschr[30]);
clas31[5] = 31;
cls031[4] = 31;
strcpy(&chr[clas32[7]-1], clschr[31]);
clas32[5] = 32;
cls032[4] = 32;

}
