#include <stdio.h>
#include "las.h"
#include <ctype.h>
#include <string.h>
#include "util.h"
#include "pixman.h"
#include "protos.h"
/******************************************************************************

NAME:  GETPAR

FUNCTION:  Get user-specified parameters from tae.

PROGRAM HISTORY:
  Version	Date       Author      		     Request
  -------	----	   ------            	     -------
   1.0        19-Feb-82  J. Hwang, SASC
   2.0         8-Jan-88  M. Mazzella SAR/GSFC      Convert to C
   3.0         4-Apr-89  K. Narayanan, STX         Didn't accept input/output 
                                                   images and class numbers 
                                                   not initialized correctly
   4.0        20-Dec-90  K. Donaldson              Added comments
   5.0        05-Feb-91  K. Donaldson              Added code to process only
                                                   the 1st band if input image
                                                   has multiple bands.
   6.0        08-Oct-92	 S. Nelson		   Made a correction so that
					   	   INDLTF can be null.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Initialize parameters.
   Retrieve the input image name.
   If the number of input image bands exceeds 1, 
      Print an error message.
      Reset the window and band variables to handle only one band.
   End-If
   Retrieve the data type of the input image.
   If data type is not byte, 
      Print a fatal error message.
   Retrieve the output image name.
   Retrieve the truncation option flag.
   Get the number of classes wanted.
   If batch mode and interactive parameters are specified,
      inform and abort.
   Get user annotation if defined, 
   Otherwise get the tae name.
   Initialize terminal i/o.
   If the number of classes specified by the user is not 0, 
      Display the color lookup table.
   For the number of classes specified by the user, 
      Get the class name or color number.
      Ring bell to get attention.
      Prompt the user for the class name or color number.
      Read in the grey scale of the class and the class name.
      Call PACK to pack the user's response and convert lower case characters 
         to upper case.
      Validate the user's answer.
   Extract the grey scale and class name from the line buffer.
   If the number of classes specified by the user is 0, 
      Generate the default class numbers.
   Retrieve the input display lookup table file and its entry name.
   Retrieve the facility name.
   Get minmax flag and processing message interval

******************************************************************************/

void getpar
(
    char hosin[],		/*  Input host file name		*/
    long windin[4],		/*  Input image window			*/
    long window[][4],		/*  Initial multi-band input window     */
    long bands[MAXIMG],		/*  Input image bands			*/
    long (*num_img_bands)[MAXBND+1],  /* Band numbers for each image     */
    struct DDR *ddr,		/*  Input image ddr 			*/
    char usrano[],		/*  User defined annotation		*/
    char taeout[],		/*  Output tae name			*/
    char hosout[],		/*  Output host file name		*/
    long *trnc,			/*  Image truncation flag		*/
    char clschr[][8],		/*  Class descriptions			*/
    unsigned char class[],	/*  user defined class numbers		*/
    char facility_name[],	/*  Facility name			*/
    long *msgtim,		/*  Time for processing message         */
    char indltf[],		/*  User specified dltf name		*/
    char dltfentr[],		/*  User specified dltf option		*/
    long *usrcol_flg,		/*  Flag for user specified colors      */
    long *numcls		/*  Number of classes specified		*/
)

{
#include "externs.h"
#include "classmap.h"


char    fill_val,		/*  Filling value                       */
	*ii,			/*  Index into input buffer		*/
	line[80],		/*  Input line				*/
	mnmx[4],		/*  Minmax flag				*/
	param[CMLEN],		/*  Parameter name  			*/
	tempstr[80],		/*  Temporary string variable		*/
	taein[CMLEN];		/*  Tae input name			*/

long	access,			/*  Access type                         */
	badresp,		/*  Bad response flag			*/
	cols,			/*  Number of columns to initialize	*/
	element = 0,		/*  Element of the input paramter array */
	fill_num,		/*  Number fill values                  */
	fill_type,		/*  Data type of the fill values        */
	i,			/*  Loop counter			*/
	imglimit = 1,		/*  Input image limit			*/
	j,			/*  Loop counter			*/
	length,			/*  Length of class name,number		*/
	lines,			/*  Number of lines to initialize	*/	
	nim,			/*  Number of input images		*/
	num,			/*  Number finding flag                 */
	qflag[MAXIMG][2],	/*  Flag for user spec window and band  */
	samwind,		/*  Same window flag			*/
	status,			/*  Error checking status		*/
	temp_band,		/*  Temporary band number		*/
	temp_len,		/*  Temporary storage for string length */
	type,			/*  Data type of images			*/
	term,			/*  Terminal 				*/
	totbnd;			/*  Total number of input bands		*/

struct VARIABLE	*var;		/*  Pointer into tae variable block	*/

/* 
Initialize parameters.
----------------------
*/
*usrcol_flg = FALSE;
for (i = 0; i < 32; ++i)
   {
   class[i] = 0;
   strcpy (clschr[i],"       ");
   }

/*
Retrieve the input image name.
If the number of input image bands exceeds 1, print an error message
and reset the window and band variables to handle only one band.
-----------------------------------------------------------------------
*/
strcpy (param, "IN");
status = c_getin (&block, param, &element, (char (*)[256])taein, 
		 (char (*)[256])hosin, &nim, window, num_img_bands, bands,
		 &totbnd, &imglimit, &samwind, qflag);
if (status != E_SUCC)
   c_errmsg ("Error getting input image", "classmap-getin", &fatal_err);

for (i = 0; i < 4; i++)
   windin[i] = window[0][i];

if (totbnd > 1) 
   {
   temp_band = num_img_bands[0][0];
   for (i = 0; i < MAXIMG; i++)
      {
      for (j = 0; j < MAXBND+1; j++)
         num_img_bands[i][j] = 0;
      bands[i] = 0;
      }
   bands[0] = 1;
   num_img_bands[0][0] = temp_band;
   c_errmsg ("Multiple bands not allowed-only the first band used",
             "classmap-bands",&inf_err);
   }

/*
Retrieve the data type of the input image.
If data type is not byte, print a fatal error message.
------------------------------------------------------
*/
c_getddr(hosin, ddr);
if (ddr->dtype != EBYTE)
   c_errmsg ("Input image must be byte", "classmap-dtype", &fatal_err ); 

/*
Retrieve the output image name.
--------------------------------
*/	
strcpy (param, "OUT");
status =  c_getout (&block, param, (char (*)[256])taeout,
		    (char (*)[256])hosout);
if (status != E_SUCC)
   c_errmsg ("Error getting output image","classmap-getout", &fatal_err);

/*
Retrieve the truncation option flag.
-------------------------------------
*/
fill_num = 79;
fill_type = EBYTE;
fill_val = ' ';
c_pxfill ((unsigned char *)tempstr, &fill_type, &fill_num,
	 (unsigned char *)&fill_val);
tempstr[0] = '\0';
var = p_find (&block, "TRUNOPT");
strcpy (tempstr, SVAL(*var,0));
temp_len = (long)strlen (tempstr);
c_low2up (tempstr, &temp_len);
if (tempstr[0] == 'T')
   *trnc = TRUE;
else
   *trnc = FALSE;

/*
Get the number of classes wanted.
---------------------------------
*/
var = p_find (&block, "NCLASS");
*numcls = IVAL(*var, 0);

/*
If batch mode and interactive parameters are specified inform and abort.
-------------------------------------------------------------------------
*/
var = p_find (&block, "$runtype");

if ((*numcls > 0) && SVAL (*var, 0)[0] == 'B')
   c_errmsg ("Classmap must be run interactively with NCLASS > 0", 
             "classmap-batch", &fatal_err);

/*
Get user annotation if defined, otherwise get the tae name.
------------------------------------------------------------
*/
c_pxfill ((unsigned char *)tempstr, &fill_type, &fill_num,
	 (unsigned char *)&fill_val);
tempstr[0] = '\0';
var = p_find (&block, "ANNOTE");
if ((*var).v_count != 0) 
   strcpy (tempstr, SVAL(*var,0));
if (strlen (tempstr) == 0)
   strcpy (usrano, taein);
else
   strcpy (usrano, tempstr);

/*
Initialize terminal i/o.
-------------------------
*/
t_init (&lines, &cols, &type);

/*
If the number of classes specified by the user is not 0, display the
color lookup table.
---------------------------------------------------------------------
*/
if (*numcls != 0) 
   {
   t_write("                 color table",T_STDCC);
   t_write("                -------------",T_STDCC);
   t_write(" 1: black     2: dkblue    3: blgreen   4: ltblue" ,T_STDCC);
   t_write(" 5: dkgreen   6: medgreen  7: ltgreen   8: olive" ,T_STDCC);
   t_write(" 9: purple   10: magenta  11: maroon   12: red  " ,T_STDCC);
   t_write("13: brown    14: orange   15: yellow   16: gray " ,T_STDCC);
   t_write("17: pink     18: tan      19: sand     20: peach" ,T_STDCC);
   t_write("21: dkgray   22: violet   23: medblue  24: aqua " ,T_STDCC);
   t_write("25: white    26: spare    27: spare    28: spare" ,T_STDCC);
   t_write("29: spare    30: spare    31: spare    32: spare" ,T_STDCC);
   t_write(" ('spare' numbers are currently white)   ",T_STDCC);
   }  

/*
For the number of classes specified by the user, get the class name or 
color number.
-----------------------------------------------------------------------
*/
for (i = 0; i < *numcls; ++i)
   {
   badresp = 0;
   do
      {
/*
Ring bell to get attention.
----------------------------
*/
      t_bell();      
/*
Prompt the user for the class name or color number.
-----------------------------------------------------
*/
      sprintf (tempstr, "%d type in [class name, color number] :", i+1);
      t_write (tempstr, T_PROMPT);
   
/*
Read in the grey scale of the class and the class name.
-------------------------------------------------------
*/
      t_read (line,(int*)&term);

/*
Call PACK to pack the user's response and convert lower case characters 
to upper case.
Validate the user's answer.
--------------------------------------------------------------------------
*/
      pack (line, &length);   
      ii = strchr (line, ',');
      if (ii == 0)  
         {
         c_errmsg ("Input not in proper format, use comma", "classmap-format",
                    &inf_err);
         badresp++;
         }  
      else
         {
         class[i] = (atoi(ii+1));
         *usrcol_flg = TRUE;
         if ((class[i] > 32) || (class[i] < 1))
            {
            c_errmsg ("Invalid color number specified", "classmap-color",
                      &inf_err);
            badresp++;
            }
          else
            { 
            /*if (length == 0) 
               {
               badresp++;
               if (badresp >= 3) 
                  c_errmsg ("Invalid input for more than 3 times",
                            "classmap-warn", &inf_err); 
               }  
            else*/
               badresp = 0;
            }
         } 
      } while (badresp > 0);

/*
Extract the grey scale and class name from the line buffer.
------------------------------------------------------------
*/
   num = FALSE;
   c_low2up (line, &length);
   for (j = 0; j < length; ++j)
      {
      if ((line[j] != ',') && (!num)) 
         /* clschr[i][j] = toupper (line[j]); */
         clschr[i][j] = line[j];
      else if (line[j] == ',') 
         {
         if (j == 0)      /*  set default */
            sprintf (clschr[i], "CLASS%d", i+1);
         num = TRUE;
         }
      }

   }
/*
If the number of classes specified by the user is 0, generate the 
default class numbers.
----------------------------------------------------------------------
*/
if (*numcls == 0) 
   {
   for (i = 0; i < 32; ++i)
      {
      class[i] = (unsigned char)i + 1;
      sprintf (clschr[i], "CLASS%d", i+1);      
      } 
   }  

/*
Retrieve the input display lookup table file and its entry name.
----------------------------------------------------------------
*/
var = p_find (&block, "INDLTF");
if ((*var).v_count  !=  0)
   {
   access = IREAD;
   if (c_getnam (&block, "indltf", &zero, indltf, "dltf", &access) !=  E_SUCC) 
      c_errmsg ("Error getting DLTF", "classmap-dltf", &fatal_err);
   if ((strlen (indltf) != 0) && (*numcls == 0))
      {
      var = p_find (&block, "DLTFENTR");
      if ((*var).v_count  ==  0) 
         c_errmsg ("No DLTF entry specified", "classmap-dltfent", &fatal_err); 
      strcpy (dltfentr, SVAL(*var, 0));
      }
   }
else
   strcpy (indltf, "");

/*
Get the facility address for output title.
--------------------------------------------
*/
strcpy (facility_name, SVAL (*p_find (&block, "$faddr"), 0));
length = (long)strlen (facility_name);
for (i = length; i < lstano[6]; i++) facility_name[i] = ' ';
facility_name[lstano[6]] = '\0';

/*
Get minmax flag and processing message interval
------------------------------------------------
*/
var = p_find (&block, "$MSGTIM");
*msgtim = IVAL (*var, 0);
var = p_find  (&block, "$MINMAX"); 
strcpy(mnmx, SVAL(*var,0));
if (mnmx[0] == 'Y' || mnmx[0] == 'y') 
   min_max = TRUE;
else 
   min_max = FALSE;
}
