/**************************************************************
NAME: CONSTRUCT_FL

PURPOSE:  construct takes the data stored in inpix and returns a
          `good' lookup table stored in outlut.  The size of the pixel
          arrays are stored in pixsz and the desired size of the of
          the lookup table is in lutsz.  count is a temporary array
          that should be at least as large as lutsz.  Part of
          the CONSTRUCT program.

***************************************************************/
#include <stdio.h>
#include <math.h>
#include "cluster.h"

long construct_fl 
(
    float *inpix,	/* 1-D array of pixel data of size [pixsiz][ndim] */
    long pixsz,		/* size of pixel image		*/
    int ndim,
    float  **outlut,	/* 2-D array of pixel data of size [ndim][lutsz] */ 
    long lutsz,		/* size of look up table	*/
    long niter, 
    long maxsamp, 
    long ilmax,
    int *lookup,		/* an array used to speed up the search */
    float  **means,
    long init,		/* a flag to initialize the lut */
    FILE *hist_file_ptr,
    long *count,
    float *vecs_pc, 
    float *pc
)
{
long i, j, k, l, m, index5;
long *ix, lsize;
long fatal = -1;
float scale;
float magnitude;

float idist, jdist, maxdist;
float errsq=0.0;
float pixfc;

pixfc = (float)pixsz;
pixfc = pixfc / 0xffffff;

/*------------------------ Allocate Memory ---------------------------- */	
  
ix = (long *)malloc((unsigned)(ndim * sizeof(long)));
if(!ix )
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }

/*---------------------- Done Allocate Memory ------------------------- */	

if(init) 
  {				/* initialize the luts */
  for(i=0;i<ndim;i++) 
    {
    for(j=0;j<lutsz;j++) 
      {
      outlut[j][i] = 0.0;
      }
    }
  }

lsize = lutsz;
i = j = 0;			

if (init) 
  {
  while ( i < lutsz && j < maxsamp) 
    {
    j++;
    m = j * ndim;	/* calculate indicie into data */
    					/* unique points are found.   */
    for(l=0;l<ndim;l++) 
      {
      outlut[i][l] =  inpix[m + l] ;
      }
    count [i] = 1;

    idist = 1;
    for (k=0; k<i;k++) 	/* eliminate duplicates      */
      {
      jdist = 1;
      for(l=0;l<ndim;l++) 
        {
	if (outlut[k][l] != outlut[i][l]) 
          {
	  jdist = 0;
	  break;
	  }
	}
      if (jdist == 1) 
        {
	idist = 0;
	break;
	}
      }

    if (idist == 1) i++;
  }

lsize = i;


/*---Free Memory ---*/	
 	 
free((char *) ix);

/*---Done Free Memory ---*/	
  
return(lsize);
  
}




   magnitude = 0.0;
   for ( i = 0 ; i < ndim ; i ++ ) 
     {
      pc [i] = 0.0;
      for ( j = 0 ; j < lutsz ; j ++ ) 
      {
          pc [i] += outlut [j][i] * (float)count [j];
      }
      magnitude += pc [i] * pc [i];
      }

   magnitude = sqrt ( magnitude );
   if ( magnitude < 1.0E-10 ) 
     {
     c_errmsg("PC magnitude error","cluster-fatal",&fatal);
     }
   for ( i = 0 ; i < ndim ; i ++ ) pc [i] /= magnitude;




   for ( j = 0 ; j < lutsz ; j ++ ) 
    {
    vecs_pc [j] = 0.0;
    for ( i = 0 ; i < ndim ; i ++ ) vecs_pc [j] += outlut [j][i] * pc [i];
    }

bsort3 ( outlut, ndim, lutsz, vecs_pc ); 
build_indices ( vecs_pc, lutsz, lookup, 0, 255, 256.0 );

scale = ((float) ilmax) / ( vecs_pc [lutsz-1] - vecs_pc [0] );

for(i=0;i<lutsz;i++) 
  {
  count[i] = 0;      
  }

for(i = 0; i < ndim;  i++) 
  {
  for(j = 0; j < lutsz; ++j)
    {
    means[j][i] = 0.0;    
    }
  }

for(i=0; i < niter; i++) 
  {
  m = i * ndim;		
  index5 = nn_fl_pc (ndim,&(inpix[m]),lutsz,outlut,&maxdist,
                lookup,vecs_pc,scale,pc,ilmax);
  errsq += maxdist;
  for(l = 0; l < ndim; ++l)
    {
    means[index5][l] +=  inpix[m+l] ;
    }
  ++(count[index5]);
  }
  
for(i = 0; i < lutsz; ++i)
  {
  for(j = 0; j < ndim; ++j)
    {
    if( count[i] != 0)
      {
      outlut[i][j]  = means[i][j] / (float)count[i];
      }
    }
  }

/*----------------------------- Free Memory ----------------------------- */	
  
free((char *) ix);
 
/*-------------------------- Done Free Memory --------------------------- */	
  
fprintf ( hist_file_ptr, "Construct   TSE: %f   MSE: %f   SQRT: %f\n", 
	errsq, errsq/(float)(ndim*niter),
        (float)sqrt(errsq/(float)(ndim*niter)) );
fflush (hist_file_ptr);
  
return(lsize);
}
