/************************************************************
NAME: GETDATA_UC

PURPOSE: Generate a random set of pixel values to pass
         to a monte-carlo statistics program.  Part of
         the CONSTRUCT program

************************************************************/
#include <math.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include "iio.h"
#include "las.h"
#include "cluster.h"

#ifdef sun
#define rand_int(high) ((long)(rand())%high)
#else
#define rand_int(high) ((long)(random())%high)
#endif

static long compar(long *left,long *right)
{
     return (*left - *right);
}

long getdata_uc 
(
    struct RDESC rdesc[],	      /* File descriptors to the images      */
    float **outlut,               /* the output centroids */
    float **means,
    long numlines,                /* the number of lines */
    long numsamp,                 /* the number of samples per line */
    long numbands,                /* the number of bands */
    long iters,                   /* the iterations */
    long numclust,                /* the number of desired clusteres */
    long bnd,                     /* the the band to sort on */
    long ilmax,                   /* the max poss value */
    int  *lookup,                 /* an array used to speed up the search */
    long totalsize,
    FILE *hist_file_ptr,         /* history file */
    float *conv_table,
    long *count,
    float **pc_mults_uc, 
    float *vecs_pc, 
    float scale, 
    float *pc,
    long use_log,
    long mfact,                  /* Multiplicity factor   */
    long window[],		     /* Processing window     */
    long nmrimg  		     /* number of input images         */
)
{
long index1,index2,index3,index4;    /* various counters */
long numpix;                     /* the number of pixels */
long *indicies;                  /* the random number indicies */
long imagesiz;                   /* the total # of pixels in image */
long init = 1;                   /* a flag to initialize the luts */
long ii,ll;			/* loop conters		*/
double dcurl;			/* current line		*/
long curl;			/* current line		*/
long curp;			/* current pixel	*/
double dcurp;			/* current pixel	*/
long linep;			/* curp location in curl  */
long fatal = -1;		/* fatal error constant */
long nonfatal = 1;              /* nonfatal error constant */
long bandnum;			/* Image band number	*/

unsigned char *outdata;          /* the sampled data */
unsigned char *bandbuf;		/* band buffer	    */
     
double dnumpix;


     
/* Set Parameters
 --------------------------------------- */   
 imagesiz = numlines*numsamp;

/* Find total table size
------------------------*/
if ( totalsize < (10 * numclust) ) 
  {
  totalsize = mfact *10 * numclust;
  fprintf(hist_file_ptr,"Total Size too small.... reset.\n");
  fflush(hist_file_ptr);
  }

if ( totalsize > imagesiz ) 
  {
  totalsize = imagesiz;
  fprintf(hist_file_ptr,"Total Size too large.... reset.\n");
  fflush(hist_file_ptr);
  }
     
/* Find maximum size of outdata
--------------------------------*/

numpix = 10 * numclust;

dnumpix = (double)numclust * pow((double) 2.0,(double) iters);
if( ((long) dnumpix) > numpix )
  {
  numpix  = (long) dnumpix;
  }

if( numpix > totalsize) numpix = totalsize;

fprintf (hist_file_ptr,"Image Size: %d  Numpix: %d  Totalsize: %d\n",
          imagesiz,numpix, totalsize);
fflush (hist_file_ptr);


/* Allocate Memory 
--------------------------------------- */   

indicies = (long *)malloc((unsigned)(totalsize * 2 * sizeof(long)));
if(!indicies)
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }

bandbuf = (unsigned char *)malloc((unsigned)(numsamp*sizeof(unsigned char)));
if(!bandbuf)
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }

outdata = (unsigned char *)malloc((unsigned)(totalsize*numbands*
           sizeof(unsigned char)));
if(!outdata)
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }

  
for ( ii = 0; ii <= iters; ++ii )
  {
  if ( ii == 0 ) 
    {

/* INITIALIZE THE CLUSTERS; NEED numclust UNIQUE PIXELS 
--------------------------------------------------------*/

     numpix = numclust * 10;
     ll = totalsize;     /* GUARANTEE READ IN DATA FIRST TIME */
     }
   else if ( ii == 1 ) 
     {
     numpix =  2 * numclust;
     ll = numclust +1;
     }
   else 
     {
     ll += numpix;
     numpix *= 2;
     }
    
   if (numpix > totalsize) numpix = totalsize;
    
   if( (ll+numpix) >= totalsize )
     {
     ll = 0; 

	/*--------- Get Random Indicies -------------------------- */ 
     
      for(index1=0; index1 < 2*totalsize; index1 += 2)
        {  
	indicies[index1]   = rand_int(imagesiz);
	indicies[index1+1] = index1/2;
	}
     
	/*--------- Done Get Random Indicies --------------------- */
     
     qsort((long *)indicies,totalsize,2*sizeof(long),compar); 

	/*------ Sample Data ---------------------------------- */
     open_las(rdesc,window,nmrimg);
     if (alloc_las(rdesc,nmrimg) == E_FAIL)
         c_errmsg("Fatal error encountered","cluster-fatal",&fatal);

     index4 = 0;
     index2 = 0;

     while (index4 < totalsize)
       {
       curp = indicies[index2];
       dcurp = curp;
       dcurl = (floor(dcurp/numsamp) + 1);
       curl = (long)dcurl;
       linep = curp - ((curl-1)*numsamp);
        
       for (index3 = 0; index3 < numbands; ++index3)
         {
         bandnum = index3 + 1;
         if (bandlineread(rdesc,curl,nmrimg,bandnum,bandbuf,numsamp) != E_SUCC)
           {
           c_errmsg("Error reading image file","cluster-read",&fatal);
           }

           outdata[indicies[2*index4+1]*numbands+index3]
             = bandbuf[linep];
         }
       ++index4;
       index2 +=2;
       if (index4 >= totalsize) break;
       }

	/*----------- Done Sample Data ----------------------------- */

       if(close_las(rdesc,nmrimg) != E_SUCC)
         {
         free(bandbuf);
         bandbuf=NULL;
	 c_errmsg("Error closing image file","cluster-nonfatal",&nonfatal);
         }
		}
     
		fprintf(hist_file_ptr,
			" At Iteration %d with samples %d and location %d\n", 
			ii,numpix,ll);

		fflush (hist_file_ptr);
	
		/*-------- Construct Tree ------------------------------- */
     
	numclust = construct_uc (&outdata[ll*numbands], numpix, numbands, 
			outlut, numclust, numpix, numpix, lookup, 
			means, init, hist_file_ptr,conv_table,count,
			pc_mults_uc, vecs_pc, scale, pc, use_log);
          
	/*------- Done Construct Tree --------------------------- */
       
	init = 0;

	}

/*------------Deallocate Memory & Close File ----------------------- */

free((char *) indicies);
free((char *) outdata);
free(bandbuf);
     
/*------------Done Deallocate Memory & Close File ------------------- */   
     
return (TRUE) ;
}
