
/****************************************************************
NAME: LPROJECT.C

PURPOSE: Apply the cluster means generated with the construct
         modules of CLUSTER to the specified input image and
         to generate a single band clustered image (BYTE ONLY)

PROGRAM HISTORY:
Version   Date     Programmer        Code   Reason
-------   ----     ----------        -----  ------
  1.0     02/94    K. Lowell           C    MRLC

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

PROJECT:  LAS

****************************************************************/

#include <stdlib.h>
#include "las.h"
#include "iio.h"
#include <math.h>
#include "cluster.h"


long lproject 
(
    struct PARBLK *vblk,	   /* pointer to initial TAE parameter block */
    struct RDESC rdesc[],
    char *infile,              /* The input image to project */
    float **outlut,            /* The codebook file to project through */
    char *outfile,             /* The resultant output image file */
    FILE *hist_file_ptr,       /* The history file */
    long numclust,		   /* Number of clusters  */
    struct DDR *ddr,	   /* The DDR structure   */
    long use_log,              /* Use logarithm of data */
    long msgtim,		   /* Processing time     */
    long window[],		   /* Processing window   */
    long nmrimg 	           /* Number of input images */
)
{

long time0, time1; /* Internal clock counters */
long i, j;		   /* Local increment counters  */

float **calc_means;        /* Mean vectors */

long fatal = -1;           /* Fatal error flag */
long nonfatal = 1;         /* Nonfatal error flag */
long numbands = 0;         /* Number of data bands in image */
long numlines = 0;         /* Number of rows */
long numsamp = 0;          /* Number of cols */

long k, ii;		   /* Local increment counters  */

float *conv_table;

int  *count;               /* Number of elements */
float ***cov_upper;        /* Upper triangular covariance matrix */

long ilmax;                /* for speeding up searches */
int  *lookup;              /* for speeding up searches */

float *pc;                 /* Principal component of vectors */
float magnitude;           /* Magnitude of vector */
float *means_pc;           /* Principal components of the mean vectors */
float **pc_mult_uc;        /* Multiplication values using PC */


/*  Do some initializations.
----------------------------- */
for (ii=0;ii<nmrimg;ii++)
  numbands += rdesc[ii].nbands;
numlines = window[2];
numsamp = window[3];


/*    Get the codebook read in and organized.
----------------------------------------------*/
   count = NULL;
   cov_upper = NULL;

if ( outlut == NULL ) 
  {
  c_errmsg("Mean vectors not present","cluster-nolut",&nonfatal);
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
  }

if ( cov_upper != NULL ) 
  {
  /*
  fprintf (stderr,"Covariance matrices present in codebook -- ignored ");
  fprintf (stderr,"and not deleted.\n");
  */
  }

cov_upper = (float ***) malloc((unsigned) numclust * sizeof(float **));
if ( !cov_upper ) 
  {
  c_errmsg(" Error allocating memory","cluster-alloc",&nonfatal);
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
  }
for(i=0; i<numclust; i++) 
  {
  cov_upper[i] =  SM_get_matrix (numbands) ;
  }

fprintf(hist_file_ptr,"Input File         = %s\n",infile);
fprintf(hist_file_ptr,"Output File        = %s\n",outfile);
fprintf(hist_file_ptr,"Rows               = %d\n",numlines);
fprintf(hist_file_ptr,"Columns            = %d\n",numsamp);
fprintf(hist_file_ptr,"Bands              = %d\n",numbands);
fprintf(hist_file_ptr,"Number of Clusters = %d\n",numclust);
if ( use_log ) 
  {
  fprintf ( hist_file_ptr, "NOTE:  Using logarithm of data...\n" );
  }

fflush (hist_file_ptr);

/* Find principal component of mean vectors.
---------------------------------------------- */
pc = (float *) malloc ( sizeof(float) * numbands );
if ( pc == NULL ) 
  {
  c_errmsg (" Error allocating memory","cluster-alloc",&nonfatal);
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
  }

if ( ! count ) 
  {
  magnitude = 0.0;
  for ( i = 0 ; i < numbands ; i ++ ) 
    {
    pc [i] = 0.0;
    for ( j = 0 ; j < numclust ; j ++ ) pc [i] += outlut [j][i];
    magnitude += pc [i] * pc [i];
    }
  } 
else 
  {
  magnitude = 0.0;
  for ( i = 0 ; i < numbands ; i ++ ) 
    {
    pc [i] = 0.0;
    for ( j = 0 ; j < numclust ; j ++ ) 
    {
        pc [i] += outlut [j][i] * (float)count [j];
    }
    magnitude += pc [i] * pc [i];
    }
  }
   
magnitude = sqrt ( magnitude );
if ( magnitude < 1.0E-10 ) 
  {
  c_errmsg("PC magnitude error","cluster-lookup",&nonfatal);
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
  }

for ( i = 0 ; i < numbands ; i ++ ) pc [i] /= magnitude;

/* Calculate principal components of the mean vectors.
---------------------------------------------------------- */
means_pc = (float *) malloc ( sizeof(float) * numclust ) ;
if ( means_pc == NULL ) 
  {
  c_errmsg("Error allocating memory","cluster-alloc",&nonfatal);
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
  }
for ( j = 0 ; j < numclust ; j ++ ) 
  {
  means_pc [j] = 0.0;
  for ( i = 0 ; i < numbands ; i ++ ) means_pc [j] += outlut [j][i] * pc [i];
  }

/*  Calculate multiplications needed for doing PC calculations.
---------------------------------------------------------------- */
pc_mult_uc = (float **)contig_matrix ( numbands, 256 );
if ( use_log ) 
  {
  for ( i = 0 ; i < numbands ; i ++ ) 
     for ( j = 0 ; j < 256 ; j ++ ) 
            pc_mult_uc [i][j] = pc [i] * log ( (double) (1 + j) ) ;
  } 
else 
  {
  for ( i = 0 ; i < numbands ; i ++ ) 
     for ( j = 0 ; j < 256 ; j ++ ) pc_mult_uc [i][j] = pc [i] * (float) j;
   }

/* Set up needed structures for fast lookups.
---------------------------------------------------- */
ilmax = 256;
lookup=(int *) malloc((unsigned) (ilmax)*sizeof(long));
if (!lookup)
  {
  c_errmsg("Error allocating memory","cluster-alloc",&nonfatal);
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
  }

bsort3 ( outlut, numbands, numclust, means_pc );

calc_means = (float **)contig_matrix ( numclust, numbands );
for ( i = 0 ; i < numclust ; i ++ ) 
  {
  for ( j = 0 ; j < numbands ; j ++ ) 
    {
    calc_means [i][j] = 0.0;
    }
  }

/*  Project the data.
---------------------------- */
if ( (count = (int *) malloc( sizeof(long) * numclust )) == NULL ) 
  {
  c_errmsg("Error allocating memory","cluster-alloc",&nonfatal);
  c_errmsg("Fatal err encountered","cluster-fatal",&fatal);
  }

time1 = timer ();

fprintf(hist_file_ptr,"Entering Projection Routine ...\n") ;
fflush (hist_file_ptr);

switch (ddr->dtype) 
  {
  case 1:
 
    conv_table = (float *) malloc ( sizeof(float) * 256 );
    if ( conv_table == NULL ) 
      {
      c_errmsg("Error allocating memory","cluster-alloc",&nonfatal);
      c_errmsg("Fatal err encountered","cluster-fatal",&fatal);
      }
 
    if ( use_log ) 
      {
      for ( k = 0 ; k < 256 ; k ++ ) 
        {
        conv_table [k] = (float) log ( (double) (1 + k) );
        }
      } 
    else 
      {
      for ( k = 0 ; k < 256 ; k ++ ) 
        {
        conv_table [k] = (float) k ;
        }
      }
 
    build_indices ( means_pc, numclust, lookup, 0, 255, 256.0 );
 
    if (project_uc_uc (
               vblk,rdesc, outlut, numbands, numlines, numsamp, 
               numclust,
               outfile, lookup, ilmax, cov_upper, calc_means,
               count, infile, conv_table, pc_mult_uc, means_pc, msgtim,
               window,nmrimg) 
        != E_SUCC)
      c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
 
    break;
/*
case 4:
  
  build_indices ( means_pc, numclust, lookup, 0, 255, 256.0 );

  project_fl_uc (
           &iio_in, outlut, numbands, numlines, numsamp, numclust, 
           outfile, lookup, ilmax, cov_upper, calc_means, 
           count, infile, conv_table, pc_mult_uc, means_pc, pc ) ;
  break;
*/

default:
  c_errmsg("Invalid data type","cluster-dtype",&fatal);
      }


time0 = timer ();

fprintf(hist_file_ptr,"Time to complete = %d seconds.\n", (int)(time0-time1));
fflush (hist_file_ptr);


return(E_SUCC);
}

