/********************************************************************
NAME: NN_SH_PC

PURPOSE: Given a set of short vectors, and a new vector x.
         find the nearest neighbor in terms of Euclidian
         distance.  Part of the CONSTRUCT program.


*  WARNING:  IN LOGICAL OPERATIONS TO SET try_up AND try_dn, SOME
*            COMPILERS MAY NOT KICK OUT IF FIRST CONDITIONAL IS
*            FALSE.  THIS COULD RESULT IN A SEGMENTATION VIOLATION.
*

 ********************************************************************/

#include <stdio.h>
#include <math.h>
#include "cluster.h"


long nn_sh_pc
(
    long dim,		/* Dimension of the input vector */
    short *x,		/* Vector to find nearest neighbor of */
    long num_vecs,		/* Number of vectors in the set */
    float **vecs,		/* Set of vectors dimensioned [num_vecs][dim] */
    float *dist,		/* (OUTPUT) Squared distance to nearest vector */
    int  *start,		/* Indices to close vectors */
    float *vecs_pc,		/* Principal component of the vectors */
    double scale,		/* Scale value for finding starting position */
    float *pc,		/* Principal component */
    long start_length	/* Length of start */
)

{
long ret_val;		/* Index of nearest vector */
float sep;		/* Separation */
long loc;		/* Current vector we are searching around */
long loc_up;		/* Search up the sorted list */
long loc_dn;		/* Search down the sorted list */
long try_up;		/* Try in the upward search direction? */
long try_dn;		/* Try in the downward search direction? */
float vector_pc;	/* Principal component of input vector */

long i, j;		/* Loop control */
float tmp1, tmp2;	/* Temporary */

/*  Calculate principal component of the new vector.
---------------------------------------------------------- */
vector_pc = 0.0;
for ( j = 0 ; j < dim ; j ++ ) vector_pc += pc [j] * x [j] ;

/*  Start by finding a close vector in some dimension.
-------------------------------------------------------- */

	if ( vector_pc <= vecs_pc [0] ) {
		loc = 0;
	} else if ( vector_pc >= vecs_pc [num_vecs-1] ) {
		loc = start_length - 1 ;
	} else {
		loc = (long) (( vector_pc - vecs_pc [0] ) * scale);
		if ( loc < 0 ) {
			loc = 0 ;
		} else if ( loc >= start_length ) {
			loc = start_length - 1 ;
		}
	}

	loc = start [loc];

	*dist = 0.0;
	for ( i = 0 ; i < dim ; i ++ ) {
		tmp1 = x [i] - vecs [loc][i];
		(*dist) += tmp1 * tmp1;
	}

	sep = (float) sqrt ( *dist );
	ret_val = loc;

/*  Search forward and backward from loc.  First check
    separations, then distance.
------------------------------------------------------- */

	loc_up = loc + 1;
	loc_dn = loc - 1;

	try_up = ( loc_up < num_vecs ) && 
		( (vecs_pc [loc_up] - vector_pc) < sep );
	try_dn = ( loc_dn >= 0 ) && 
		( (vector_pc - vecs_pc [loc_dn]) < sep );

	while ( try_up && try_dn ) {

		tmp2 = 0.0;
		for ( i = 0 ; i < dim ; i ++ ) 
		  {
		  tmp1 = x [i] - vecs [loc_up][i];
		  tmp2 += tmp1 * tmp1;
		  if ( tmp2 > (*dist) ) break;
		  }
		if ( tmp2 < (*dist) ) 
		  {
		  *dist = tmp2;
		  sep = (float) sqrt ( tmp2 );
		  ret_val = loc_up;
		  }

		tmp2 = 0.0;
		for ( i = 0 ; i < dim ; i ++ ) 
		  {
		  tmp1 = x [i] - vecs [loc_dn][i];
		  tmp2 += tmp1 * tmp1;
		  if ( tmp2 > (*dist) ) break;
		  }
		if ( tmp2 < (*dist) ) 
		  {
		  *dist = tmp2;
		  sep = (float) sqrt ( tmp2 );
		  ret_val = loc_dn;
		  }

		loc_up ++;
		loc_dn --;

		try_up = ( loc_up < num_vecs ) && 
			( (vecs_pc [loc_up] - vector_pc) < sep );
		try_dn = ( loc_dn >= 0 ) && 
			( (vector_pc - vecs_pc [loc_dn]) < sep );

	}

	while ( try_up ) {

		tmp2 = 0.0;
		for ( i = 0 ; i < dim ; i ++ ) {
			tmp1 = x [i] - vecs [loc_up][i];
			tmp2 += tmp1 * tmp1;
			if ( tmp2 > (*dist) ) break;
		}
		if ( tmp2 < (*dist) ) {
			*dist = tmp2;
			sep = (float) sqrt ( tmp2 );
			ret_val = loc_up;
		}

		loc_up ++;

		try_up = ( loc_up < num_vecs ) && 
			( (vecs_pc [loc_up] - vector_pc) < sep );

	}

	while ( try_dn ) {

		tmp2 = 0.0;
		for ( i = 0 ; i < dim ; i ++ ) 
		  {
		  tmp1 = x [i] - vecs [loc_dn][i];
		  tmp2 += tmp1 * tmp1;
		  if ( tmp2 > (*dist) ) break;
	      	  }
		if ( tmp2 < (*dist) ) 
		  {
		  *dist = tmp2;
		  sep = (float) sqrt ( tmp2 );
		  ret_val = loc_dn;
		  }

		loc_dn --;

		try_dn = ( loc_dn >= 0 ) && 
			( (vector_pc - vecs_pc [loc_dn]) < sep );

	}

	return ( ret_val );
}

