/************************************************************
NAME: NN_UC_PC

PURPOSE: Given a set of char vectors, and a new vector x,
         find the nearest neighbor in terms of Euclidean
         distance.  Part of the CONSTRUCT program.


 *  WARNING:  IN LOGICAL OPERATIONS TO SET try_up AND try_dn, SOME
 *            COMPILERS MAY NOT KICK OUT IF FIRST CONDITIONAL IS
 *            FALSE.  THIS COULD RESULT IN A SEGMENTATION VIOLATION.


PROGRAM HISTORY:
Version    Date    Programmer     Code    Reason
-------    ----    ----------     ----    ------
  1.0      07/91  This routine was an updated (but re-written) version
                  of a routine named rngh_uc (), which started as a
                  translation of FORTRAN code written by Jim White.
 
  1.1             Avoids many type-casting operations by using a 
                  look-up-table indexed by the unsigned character
                  input vector components.  This table, then, can 
                  store any function (i.e. logarithmic or linear)
                  of the input vector components.
 
   1.2     10/91   Now uses vectors sorted by the principal component
                   of the data.  This provides a large amount of 
                   separation in one dimension for the data.

   2.0     01/94   K.Lowell         C     Conversion to LAS 
 *
 *  NOTES:      If the logarithm of the data is being used, "lut" and
 *              "pc_mults_uc" should contain values reflecting this.
 *
 *  RETURNS:    Returns the index of the nearest vector as an integer.
 *
 *             
 **********************************************************************/

#include <stdio.h>
#include <math.h>
#include "cluster.h"

long nn_uc_pc
(
    long dim,		/* Dimension of the input vector */
    unsigned char *x,	/* Vector to find nearest neighbor of */
    long num_vecs,		/* Number of vectors in the set */
    float **vecs,		/* Set of vectors dimensioned [num_vecs][dim] */
    float *dist,		/* (OUTPUT) Squared distance to nearest vector */
    float *lut,	 	/* Look-Up-Table for char to float conversion */
    int *start,		/* Indices to close vectors */
    float **pc_mults_uc, 	/* Table with multiplication results */
    float *vecs_pc,		/* Principal component of the vectors */
    double scale		/* Scale value for finding starting position */
)

{
long ret_val;		/* Index of nearest vector */
float sep;		/* Separation */
long loc;		/* Current vector we are searching around */
long loc_up;		/* Search up the sorted list */
long loc_dn;		/* Search down the sorted list */
long try_up;		/* Try in the upward search direction? */
long try_dn;		/* Try in the downward search direction? */
float vector_pc;	/* Principal component of input vector */

long i, j;		/* Loop control */
float tmp1, tmp2;	/* Temporary */

/*
 *  (1)  Calculate principal component of the new vector.
 *
 *       The table "pc_mults_pc" holds the values created by 
 *       doing the multiplications:
 *
 *       pc_mults_pc [i][j] = principal_component [i] * j
 *------------------------------------------------------------*/

vector_pc = 0.0;
for ( j = 0 ; j < dim ; j ++ ) vector_pc += pc_mults_uc [ j ] [ x[j] ];

/*  Start by finding a close vector in some dimension.
----------------------------------------------------------------*/

if ( vector_pc <= vecs_pc [0] ) 
  {
  loc = 0;
  } 
else if ( vector_pc >= vecs_pc [num_vecs-1] ) 
  {
  loc = 255;
  } 
else 
  {
  loc = (long) (( vector_pc - vecs_pc [0] ) * scale);
  }

loc = start [loc];

*dist = 0.0;
for ( i = 0 ; i < dim ; i ++ ) 
  {
  tmp1 = lut [ x [i] ] - vecs [loc][i];
  (*dist) += tmp1 * tmp1;
  }

sep = (float) sqrt ( *dist );
ret_val = loc;

/*  (3)  Search forward and backward from loc.  First check
*       separations, then distance.
--------------------------------------------------------- */

loc_up = loc + 1;
loc_dn = loc - 1;

try_up = ( loc_up < num_vecs ) && 
	( (vecs_pc [loc_up] - vector_pc) < sep );
try_dn = ( loc_dn >= 0 ) && 
	( (vector_pc - vecs_pc [loc_dn]) < sep );

while ( try_up && try_dn ) 
  {
  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ x [i] ] - vecs [loc_up][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (*dist) ) break;
    }
  if ( tmp2 < (*dist) ) 
    {
    *dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_up;
    }

  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ x [i] ] - vecs [loc_dn][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (*dist) ) break;
    }
  if ( tmp2 < (*dist) ) 
    {
    *dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_dn;
    }

  loc_up ++;
  loc_dn --;

  try_up = ( loc_up < num_vecs ) && 
	( (vecs_pc [loc_up] - vector_pc) < sep );
  try_dn = ( loc_dn >= 0 ) && 
	( (vector_pc - vecs_pc [loc_dn]) < sep );

}

while ( try_up ) 
  {
  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ x [i] ] - vecs [loc_up][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (*dist) ) break;
    }
  if ( tmp2 < (*dist) ) 
    {
    *dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_up;
    }

  loc_up ++;

  try_up = ( loc_up < num_vecs ) && 
	( (vecs_pc [loc_up] - vector_pc) < sep );

}

while ( try_dn ) 
  {
  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ x [i] ] - vecs [loc_dn][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (*dist) ) break;
    }
  if ( tmp2 < (*dist) ) 
    {
    *dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_dn;
    }

  loc_dn --;

  try_dn = ( loc_dn >= 0 ) && 
	( (vector_pc - vecs_pc [loc_dn]) < sep );

  }

return ( ret_val );
}

