
/*
 *   Copyright, 1991, The Regents of the University of California.
 *   This software was produced under a U.S. Government contract
 *   (W-7405-ENG-36) by the Los Alamos National Laboratory, which is
 *   operated by the University of California for the U.S. Department
 *   of Energy.  The U.S. Government is licensed to use, reproduce,
 *   and distribute this software.  Neither the Government nor the
 *   University makes any warranty, express or implied, or assumes
 *   any liability responsibility for the use of this software.
 */

#include <stdio.h>
#include <math.h>
#include "cluster.h"

/*
 *  WARNING:  IN LOGICAL OPERATIONS TO SET try_up AND try_dn, SOME
 *            COMPILERS MAY NOT KICK OUT IF FIRST CONDITIONAL IS
 *            FALSE.  THIS COULD RESULT IN A SEGMENTATION VIOLATION.
 */

/*
 *  nn_uc_pc_ul ()    Given a set of (float) vectors, and a new vector x,
 *                    find the nearest neighbor in terms of Euclidean 
 *                    distance.
 *
 *                    SAME AS nn_uc_pc () EXCEPT IT EXPECTS UN-INTERLEAVED
 *                    INPUT DATA.  INSTEAD OF RECEIVING AN INPUT VECTOR x,
 *                    IT TAKES A SET OF DATA AND AN INDEX.  THE VECTOR SET
 *                    IS INDEXED vset [dim][vector].
 *
 *                    Patrick M. Kelly     12-6-91
 *
 *                 This routine was an updated (but re-written) version
 *                 of a routine named rngh_uc (), which started as a
 *                 translation of FORTRAN code written by Jim White.
 *
 *  CHANGES:    (1)  Avoids many type-casting operations by using a 
 *                   look-up-table indexed by the unsigned character
 *                   input vector components.  This table, then, can 
 *                   store any function (i.e. logarithmic or linear)
 *                   of the input vector components.
 *
 *              (2)  Now uses vectors sorted by the principal component
 *                   of the data.  This provides a large amount of 
 *                   separation in one dimension for the data.
 *
 *                   11/7/91  Patrick M. Kelly
 *
 *  NOTES:      If the logarithm of the data is being used, "lut" and
 *              "pc_mults_uc" should contain values reflecting this.
 *
 *  RETURNS:    Returns the index of the nearest vector as an integer.
 *              
 *  Written by:  Patrick M. Kelly
 *  Date:        10/24/91
 */

int nn_uc_pc_ul
(
    int dim,		/* Dimension of the input vector */
    unsigned char **vset,	/* Vector set containing vector in question */
    int which,		/* The vector of interest */
    int num_vecs,		/* Number of vectors in the set */
    float **vecs,		/* Set of vectors dimensioned [num_vecs][dim] */
    float *lut,		/* Look-Up-Table for char to float conversion */
    int *start,		/* Indices to close vectors */
    float **pc_mults_uc,	/* Table with multiplication results */
    float *vecs_pc,		/* Principal component of the vectors */
    double scale		/* Scale value for finding starting position */
)

{

int ret_val;	/* Index of nearest vector */
float sep;	/* Separation */
int loc;	/* Current vector we are searching around */
int loc_up;	/* Search up the sorted list */
int loc_dn;	/* Search down the sorted list */
int try_up;	/* Try in the upward search direction? */
int try_dn;	/* Try in the downward search direction? */
float vector_pc;	/* Principal component of input vector */
float dist;    /* Squared distance to nearest vector */
int i, j;	/* Loop control */
float tmp1, tmp2;	/* Temporary */

/*
 *  (1)  Calculate principal component of the new vector.
 *
 *       The table "pc_mults_pc" holds the values created by 
 *       doing the multiplications:
 *
 *       pc_mults_pc [i][j] = principal_component [i] * j
 */

vector_pc = 0.0;
for ( j = 0 ; j < dim ; j ++ ) 
  vector_pc += pc_mults_uc [ j ] [ vset [j][which] ];

/*
 *  (2)  Start by finding a close vector in some dimension.
 */

if ( vector_pc <= vecs_pc [0] ) 
  {
  loc = 0;
  }
else if ( vector_pc >= vecs_pc [num_vecs-1] ) 
  {
  loc = 255;
  }
else 
  {
  loc = (int) (( vector_pc - vecs_pc [0] ) * scale);
  }

loc = start [loc];
dist = 0.0;
for ( i = 0 ; i < dim ; i ++ ) 
  {
  tmp1 = lut [ vset [i][which] ] - vecs [loc][i];
  dist += tmp1 * tmp1;
  }

sep = (float) sqrt (dist );
ret_val = loc;

/*
 *  (3)  Search forward and backward from loc.  First check
 *       separations, then distance.
 */

loc_up = loc + 1;
loc_dn = loc - 1;

try_up = ( loc_up < num_vecs ) && 
	( (vecs_pc [loc_up] - vector_pc) < sep );
try_dn = ( loc_dn >= 0 ) && 
	( (vector_pc - vecs_pc [loc_dn]) < sep );

while ( try_up && try_dn ) 
  {
  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ vset [i][which] ] - vecs [loc_up][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (dist) ) break;
    }
  if ( tmp2 < (dist) ) 
    {
    dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_up;
    }

  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ vset [i][which] ] - vecs [loc_dn][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (dist) ) break;
    }
  if ( tmp2 < (dist) ) 
    {
    dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_dn;
    }

  loc_up ++;
  loc_dn --;

try_up = ( loc_up < num_vecs ) && 
	( (vecs_pc [loc_up] - vector_pc) < sep );
try_dn = ( loc_dn >= 0 ) && 
	( (vector_pc - vecs_pc [loc_dn]) < sep );

}

while ( try_up ) 
  {
  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ vset [i][which] ] - vecs [loc_up][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (dist) ) break;
    }
  if ( tmp2 < (dist) ) 
    {
    dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_up;
    }

  loc_up ++;

  try_up = ( loc_up < num_vecs ) && 
	( (vecs_pc [loc_up] - vector_pc) < sep );

}

while ( try_dn ) 
  {
  tmp2 = 0.0;
  for ( i = 0 ; i < dim ; i ++ ) 
    {
    tmp1 = lut [ vset [i][which] ] - vecs [loc_dn][i];
    tmp2 += tmp1 * tmp1;
    if ( tmp2 > (dist) ) break;
    }
  if ( tmp2 < (dist) ) 
    {
    dist = tmp2;
    sep = (float) sqrt ( tmp2 );
    ret_val = loc_dn;
    }

  loc_dn --;

  try_dn = ( loc_dn >= 0 ) && 
	( (vector_pc - vecs_pc [loc_dn]) < sep );

  }

return ( ret_val );
}

