/*******************************************************************************
NAME:    COMPOSIT

PURPOSE: Generate a composite using up to 52 different images.

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    R.McKinney       02/88  EDC   Update for standards
  1.1    R.McKinney       06/88  EDC   Conversion to LAS 5.0
  1.2    R.McKinney       12/88  EDC   Insured all files are closed
  2.0    B.Ailts          06/89  EDC   Fixed problem with 3 or more input images
  2.1    B.Ailts          08/89  EDC   Changed upddr_flag to COMB
  2.2    D.Etrheim        05/91  EDC   Changed to use type limits 
  2.3    T.Mittan         08/91  EDC   Initialized minmax arrays
  5.0    J.Fenno          01/92  EDC   Completely rewrote COMPOSIT in response
				       to adp5352
  5.1	 T.Mittan	  09/92	 EDC   Declared curent and curcat as extern
  5.2   D. Etrheim        09/92        Check pixel size against an Epsilon
                                       value.  Comparing floating point values
                                       directly can cause problems if an image
                                       is transfered from another machine.
  5.3    S. Nelson	  10/92	       set COMPRES off so that output image
				       can be opened.
  6.0    D. Bridges       02/94        Added BANDFLG parameter to give option
				       of including identifier band in output
  7.0    J. Willems       10/97        Changed algorithm for assigning in_window
                                       to pass to upddr.
  7.1    D. Hames	  11/99	       Remove 50000 limit of NL and NS from 
                                       composit.pdf. Add prototypes.
  7.2    D. Hames	  01/00	       Fixed a typecast error that occurred 
				       while prototyping composit.c.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"
#include "util.h"

/* The following variables must be declared global for image I/O routines
=========================================================================*/
extern long curcat;          /* current image being processed                 */
extern long curent;          /* current processing line                       */
int  fdesc_in[MAXIMG+1];     /* file descriptors for the input images         */
int  fdesc_out[2];           /* file descriptor for the output image          */
int  gdesc_in;               /* group descriptor for the input images         */
int  gdesc_out;              /* group descriptor for the output image         */
long *offset_in;             /* offsets into the input images buffer          */
long *offset_out;            /* offsets into the output image buffer          */

void main
(void)
{
struct DDR    ddr;           /* DDR structure of a LAS image                  */
struct PARBLK vblock;        /* TAE parameter block                           */

char   *hostin;              /* Host file names of the input images           */
char   *hostout;             /* Host file name for the output image           */
char   *minmax;              /* Minmax flag buffer                            */
char   *taeout;              /* TAE file name for the output image            */
char   *tmp_str;             /* Temporary string for printing messages        */

double line_inc = 1;         /* (Sub)sampling value in line direction         */
double *maxval;              /* Array of maximum pixel values for each band   */
double *minval;              /* Array of minimum pixel values for each band   */
double samp_inc = 1;         /* (Sub)sampling value in sample direction       */

long   auto_subcommand;      /* Flag indicating if auto subcommand was used   */
long   band_to_compare;      /* Band to be compared                           */
long   *bands;               /* Band numbers used from each input image       */
long   combine_flag = COMB;  /* Flag indicating the input images are combined */
long   fatal = -1;           /* Fatal error flag                              */
long   i,j,k;                /* Loop control                                  */
long   in_window[MAXIMG][4]; /* Window specification for each input image     */
long   images_in_group[52];  /* Number of images in each image group          */
long   last_flag;            /* Use the last image or first image for ties    */
long   max_flag;             /* Use the max pixel value or the min value      */
long   msgtim;               /* Elapsed time between display messages         */
long   nbands[MAXIMG+1];     /* Number of bands in each input image           */
long   nl;                   /* Number of lines in the output image           */
long   nonfatal = 1;         /* Nonfatal error flag                           */
long   ns;                   /* Number of samples in the output image         */
long   num_images;           /* Total number of input images specified        */
long   num_img_groups;       /* Number of images groups (entries in IN)       */
long   odtype;               /* Data type of the output image                 */
long   one = 1;              /* Value of one                                  */
long   out_window[52][4];    /* Output window spec. for each image group      */
long   size;                 /* # of bytes needed for the output data type    */
long   status;               /* Function return status code                   */
long   std_out;              /* Logical unit number for standard output       */
long   totbnd;               /* Total number of output bands                  */
long   update_mode;          /* Flag to update the first input image          */
long   ident_band;           /* Flag to include identifier band in image      */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);

/* Allocate dynamic memory to avoid stack overflow on Gould PowerNode
---------------------------------------------------------------------*/
hostin  = (char *)calloc((CMLEN * sizeof(char)),        MAXIMG);
hostout = (char *)calloc((CMLEN * sizeof(char)),        1);
minmax  = (char *)calloc((4 * sizeof(char)),            1);
taeout  = (char *)calloc((CMLEN * sizeof(char)),        1);
tmp_str = (char *)calloc((CMLEN * sizeof(char)),        1);
bands   = (long *)calloc(((MAXBND + 1) * sizeof(long)), MAXIMG);
if ((hostin == NULL) || (hostout == NULL) || (minmax == NULL) ||
    (taeout == NULL) || (tmp_str == NULL) || (bands == NULL))
   {
   free(hostin); free(hostout); free(minmax);
   free(taeout); free(tmp_str); free(bands);
   c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
   c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
   }

/* Get user supplied parameters and TAE globals
-----------------------------------------------*/
status = getpar(&vblock, (char (*)[256])hostin, hostout, minmax, taeout, 
                &auto_subcommand, &band_to_compare, (long (*)[257])bands,
                images_in_group, in_window, &last_flag, &max_flag, &msgtim,
                nbands, &nl, &ns, &num_img_groups, &num_images, out_window,
                &totbnd, &update_mode, &ident_band);
if (status != E_SUCC)
   {
   free(hostin); free(hostout); free(minmax);
   free(taeout); free(tmp_str); free(bands);
   c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
   }

/* Determine the data type of the output image
----------------------------------------------*/
status = get_dtype((char (*)[256])hostin, &odtype, num_images, &size,
                   update_mode);
if (status != E_SUCC)
   {
   free(hostin); free(hostout); free(minmax);
   free(taeout); free(tmp_str); free(bands);
   c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
   }

/* Allocate dynamic memory needed for image i/o routines
--------------------------------------------------------*/
offset_in = (long *)calloc(sizeof(long), (totbnd + 2));
if (offset_in == NULL)
   {
   free(hostin); free(hostout); free(minmax);
   free(taeout); free(tmp_str); free(bands);
   c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
   c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
   }
offset_out = (long *)calloc(sizeof(long), (totbnd + 2));
if (offset_out == NULL)
   {
   free(hostin); free(hostout); free(minmax); free(taeout);
   free(tmp_str); free(bands); free(offset_in);
   c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
   c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
   }

/* Initialize the output image
------------------------------*/
status = init_output((char (*)[256])hostin,hostout, (long (*)[257])bands,
                     images_in_group, in_window, max_flag, msgtim, nbands, nl, 
                     ns, odtype, out_window, size, totbnd, update_mode, 
                     ident_band);
if (status != E_SUCC)
   {
   free(hostin); free(hostout); free(minmax); free(taeout);
   free(tmp_str); free(bands); free(offset_in); free(offset_out);
   if (update_mode)
      c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
   else
      c_clean(&vblock, "composit",(char (*)[256])hostout, &one);
   }

/* Composite each of the remaining input images to the output image
-------------------------------------------------------------------*/
for (i = 1; i < num_img_groups; i++)
   {
   status = composite_image ((char (*)[256])hostin, hostout,band_to_compare - 1,
                             (long (*)[257])bands, i, images_in_group, 
                             in_window, last_flag, max_flag, msgtim, nbands, 
                             odtype, out_window, size, totbnd,
			     update_mode, ident_band);
   if (status != E_SUCC)
      {
      free(hostin); free(hostout); free(minmax); free(taeout);
      free(tmp_str); free(bands); free(offset_in); free(offset_out);
      if (update_mode)
	 c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
      else
	 c_clean(&vblock, "composit",(char (*)[256])hostout, &one);
      }
   }

/* Free the memory used for image i/o 
-------------------------------------*/
free(offset_in);
free(offset_out);

/* totbnd is the number of bands being composited add 1 to include all bands
----------------------------------------------------------------------------*/
if (ident_band)
   totbnd += 1;

/* If not in update mode and the auto subcommand is being used then copy all
   of the relevant data to the output image's DDR.  For c_upddr to work
   correctly we must deceive it into thinking we are combining images of the
   same size and placing each one over the top of the others.  Note that for
   any image that does not start at line 1 sample 1 in the output image,
   in_window is set so that is has a negative starting line and sample.
----------------------------------------------------------------------------*/
k=0;
if (!update_mode && auto_subcommand)
   {
   for (i = 0; i < num_img_groups; i++)
      {
      for (j = 0; j < images_in_group[i]; j++)
          {
          in_window[k][SL] = in_window[k][SL] - out_window[i][SL] + 1;
          in_window[k][SS] = in_window[k][SS] - out_window[i][SS] + 1;
          in_window[k][NL] = nl;
          in_window[k][NS] = ns;
          k++;
          }
      }
   c_upddr(&vblock,(char (*)[256])hostin, hostout, &num_images, in_window, 
           (long (*)[257])bands, nbands, &line_inc, &samp_inc, &totbnd,
           &combine_flag);
   }

/* Invalidate the min and max values when in update mode
--------------------------------------------------------*/
if (update_mode)
   {
   if (E_SUCC != c_svmnmx(hostout, &totbnd, minval, maxval, "NO"))
      c_errmsg("Error occurred while invalidating min/max values",
	       "composit-svmnmx", &nonfatal);
   }

/* If the user has requested minimum and maximum values then calculate them
---------------------------------------------------------------------------*/
if (strcmp(minmax, "YES") == 0)
   {
   /* Allocate storage for the min/max arrays
   ------------------------------------------*/
   minval = (double *)calloc(sizeof(double), totbnd);
   maxval = (double *)calloc(sizeof(double), totbnd);
   if ((minval == NULL) || (maxval == NULL))
      {
      free(hostin); free(hostout); free(minmax); free(taeout);
      free(tmp_str); free(bands); free(minval); free(maxval);
      c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
      c_errmsg("Fatal error encountered", "composit-fatal", &fatal);
      }

   /* Set up temporary variables for the c_minmax call
   ---------------------------------------------------*/
   in_window[0][SL] = 1;
   in_window[0][SS] = 1;
   in_window[0][NL] = nl;
   in_window[0][NS] = ns;
   if (update_mode)
      {
      /* Retrieve the number of lines and samples for the entire image
      ----------------------------------------------------------------*/
      if (E_SUCC != c_getddr(hostout, &ddr))
	 c_errmsg("Error occurred while calculating the min/max values",
		  "composit-minmax", &nonfatal);
      else
	 {
	 in_window[0][NL] = ddr.nl;
	 in_window[0][NS] = ddr.ns;
	 }
      }
   else
      {
      /* Set up the bands array for the new output image
      --------------------------------------------------*/
      for (i = 0; i < totbnd; i++)
	 *(bands + i) = i + 1;
      *(bands + i) = 0;
      }

   /* The DDR will be updated as long as the given window is the entire image
   --------------------------------------------------------------------------*/
   status = c_minmax((char (*)[256])hostout,&one,in_window,(long (*)[257])bands,
                      &totbnd,
                      minval,maxval);
   if (status != E_SUCC)
      c_errmsg("Error occurred calculating the min/max values",
	       "composit-minmax", &nonfatal);
   free(minval);
   free(maxval);
   }

/* Create associated history file
---------------------------------*/
if (update_mode)
   {
   num_images--; /* don't include the output image */
   i = 1;
   }
else
   i = 0;
if (E_SUCC != c_hstout(&vblock,hostout,(char (*)[256])(hostin + i * CMLEN),
                       &num_images," "))
   {
   sprintf(tmp_str,"Error occurred while updating history file for %s",taeout);
   c_errmsg(tmp_str,"composit-history",&nonfatal);
   }

/* Inform user of successful completion
---------------------------------------*/
c_complt("COMPOSIT",(char (*)[256])taeout, &one);

/* Free all dynamic memory
--------------------------*/
free(hostin);
free(hostout);
free(minmax);
free(taeout);
free(tmp_str);
free(bands);
}
