/*******************************************************************************
NAME:    COMPOSITE_IMAGE

PURPOSE: Composite one input image group with the output image

PROGRAM HISTORY:
VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Rewrote COMPOSIT to work like CONCAT.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
=========================================================================*/
extern long curcat;              /* current image being processed             */
extern long curent;              /* current line being processed              */
extern int  fdesc_in[];          /* file descriptors for the input images     */
extern int  fdesc_out[];         /* file descriptor for the output image      */
extern int  gdesc_in;            /* group descriptor for input images         */
extern int  gdesc_out;           /* group descriptor for output image         */
extern long *offset_in;          /* offsets into the input image buffer       */
extern long *offset_out;         /* offsets into the output image buffer      */


int composite_image
(
    char hostin[][CMLEN],  /* I: Host file names of the input images          */
    char hostout[],        /* I: Host file name of the output image           */
    long band_to_compare,  /* I: # of the band to be compared (zero indexed)  */
    long bands[][MAXBND+1],/* I: The numbers of the bands to use for ea image */
    long group_number,    /* I: # of image grp to be composited (zero indexed)*/
    long images_in_group[],/* I: Number of images in each image group         */
    long in_window[][4],   /* I: Window specification for each input image    */
    long last_flag,        /* I: Use the last image or first image for ties   */
    long max_flag,         /* I: Use the max pixel value or the min value     */
    long msgtime,          /* I: Number of seconds between processing messages*/
    long nbands[],         /* I: Number of bands in each input image          */
    long odtype,           /* I: Output data type                             */
    long out_window[][4],  /* I:Window specification for ea output image group*/
    long size,             /* I: Number of bytes for the output data type     */
    long totbnd,       /* I: # of bands to composite (not including last band)*/
    long update_mode,      /* I: Flag for updating the first input image      */
    long ident_band        /* I: Flag for including identifier band in image  */
)

{
char tmp_str[CMLEN];      /* Temporary string for printing messages           */

long band;                /* Loop control                                     */
long buffer_size;         /* Size of the input buffer returned by open_image  */
long i;                   /* Loop control                                     */
long line;                /* Loop control                                     */
long nonfatal =  1;       /* Nonfatal error flag                              */
long sample;              /* Loop control                                     */
long status;              /* Return status from LAS functions                 */
long tmp_bands[1][MAXBND+1];/* Numbers of the bands to use for output image   */
long zero     = 0;        /* Value of 0                                       */

unsigned char *current_pixel; /* ptr to pixel value in output image           */
unsigned char *in_buffer;     /* Input buffer returned by open_image          */
unsigned char *new_pixel;     /* ptr to pixel value in input image            */
unsigned char *out_buffer;    /* Buffer for reading/writing to output image   */


/* Check for 0 lines or samples to be placed in the output image
----------------------------------------------------------------*/
if ((out_window[group_number][NL] <= 0) || (out_window[group_number][NS] <= 0))
   {
   sprintf(tmp_str,"Image %1d lies outside the output image", group_number + 1);
   c_errmsg(tmp_str,"composit-outside", &nonfatal);
   return(E_SUCC);
   }

/* Set up temporary variables needed by open_image and then
   open the output image for sequential reading and writing
-----------------------------------------------------------*/
if (ident_band)
   {
   if (update_mode)
      for (i = 0; i <= totbnd; i++)
         tmp_bands[0][i] = bands[0][i];
   else
      for (i = 0; i <= totbnd; i++)
         tmp_bands[0][i] = i + 1;
   }
else
   {
   if (update_mode)
      for (i = 0; i < totbnd; i++)
         tmp_bands[0][i] = bands[0][i];
   else
      for (i = 0; i < totbnd; i++)
         tmp_bands[0][i] = i + 1;
   }
tmp_bands[0][i] = 0;
status = open_image ((char (*)[256])hostout, tmp_bands, &buffer_size, 
                     group_number, images_in_group, FALSE, nbands, odtype,
		     (long (*)[4])out_window[group_number], &out_buffer);
if (status != E_SUCC)
   return(E_FAIL);


/* Open the specified image group for sequential reading
--------------------------------------------------------*/
status = open_image (hostin, bands, &buffer_size, group_number,
		     images_in_group, TRUE, nbands, odtype,
		     in_window, &in_buffer);
if (status != E_SUCC)
   {
   free(out_buffer);
   if (c_egclse((struct GDESC **)&gdesc_out) != E_SUCC)
      c_errmsg("Error closing image", "composit-close", &nonfatal);
   return(E_FAIL);
   }

/* Start the processing message
-------------------------------*/
curent = 1;
curcat += 1;
sprintf(tmp_str,"for image %1d", group_number + 1);
i = GEN2;
c_inittm("COMPOSIT", "Processed", "line", "image", tmp_str,
	 &out_window[group_number][NL], &zero, &msgtime, &i);
c_startm();

/* For each line of the output image
------------------------------------*/
for (line = out_window[group_number][SL];
     line < out_window[group_number][SL] + out_window[group_number][NL];
     line++)
   {
   /* Read the current line from all bands of the output image
   -----------------------------------------------------------*/
   if (c_estep((struct GDESC **)&gdesc_out) != E_SUCC)
      {
      free(in_buffer);
      free(out_buffer);
      if (c_egclse((struct GDESC **)&gdesc_out) != E_SUCC)
	 c_errmsg("Error closing image", "composit-close", &nonfatal);
      if (c_egclse((struct GDESC **)&gdesc_in) != E_SUCC)
	 c_errmsg("Error closing image", "composit-close", &nonfatal);
      return(E_FAIL);
      }

   /* Read the current line from all bands of the current input image
   ------------------------------------------------------------------*/
   if (c_estep((struct GDESC **)&gdesc_in) != E_SUCC)
      {
      free(in_buffer);
      free(out_buffer);
      if (c_egclse((struct GDESC **)&gdesc_out) != E_SUCC)
	 c_errmsg("Error closing image", "composit-close", &nonfatal);
      if (c_egclse((struct GDESC **)&gdesc_in) != E_SUCC)
	 c_errmsg("Error closing image", "composit-close", &nonfatal);
      return(E_FAIL);
      }

   /* The global variables offset_in and offset_out are set up with the
      number of pixels that each band is offset from the beginning of the
      buffer, convert this to bytes
   ----------------------------------------------------------------------*/
   for (i = 0; i <= totbnd; i++)
      {
      *(offset_out + i) *= size;
      *(offset_in + i)  *= size;
      }

   /* Perform the appropriate compositing function.  Note that the only
      difference between the different data types is the type casting and the
      only difference between the two cases for each data type is the > or < in
      the if statements.
   --------------------------------------------------------------------------*/
   current_pixel = out_buffer + *(offset_out + band_to_compare);
   new_pixel     = in_buffer  + *(offset_in  + band_to_compare);
   switch (odtype)
      {
      case EBYTE:
	 if (max_flag)
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*new_pixel > *current_pixel) ||
		   (last_flag && (*new_pixel == *current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(out_buffer + *(offset_out + band) + sample) =
			*(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(out_buffer + *(offset_out + band) + sample) =
		        (unsigned char)(group_number + 1);
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 else
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*new_pixel < *current_pixel) ||
		   (last_flag && (*new_pixel == *current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(out_buffer + *(offset_out + band) + sample) =
			*(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(out_buffer + *(offset_out + band) + sample) =
		        (unsigned char)(group_number + 1);
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 break;
      case EWORD:
	 if (max_flag)
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*(short *)new_pixel > *(short *)current_pixel) ||
		   (last_flag &&
		   (*(short *)new_pixel == *(short *)current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(short *)(out_buffer + *(offset_out + band) + sample) =
			*(short *)(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(short *)(out_buffer + *(offset_out + band) + sample) =
		        (short)(group_number + 1);
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 else
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*(short *)new_pixel < *(short *)current_pixel) ||
		   (last_flag &&
		   (*(short *)new_pixel == *(short *)current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(short *)(out_buffer + *(offset_out + band) + sample) =
			*(short *)(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(short *)(out_buffer + *(offset_out + band) + sample) =
		        (short)(group_number + 1);
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 break;
      case ELONG:
	 if (max_flag)
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*(long *)new_pixel > *(long *)current_pixel) ||
		   (last_flag &&
		   (*(long *)new_pixel == *(long *)current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(long *)(out_buffer + *(offset_out + band) + sample) =
			*(long *)(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(long *)(out_buffer + *(offset_out + band) + sample) =
		        group_number + 1;
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 else
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*(long *)new_pixel < *(long *)current_pixel) ||
		   (last_flag &&
		   (*(long *)new_pixel == *(long *)current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(long *)(out_buffer + *(offset_out + band) + sample) =
			*(long *)(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(long *)(out_buffer + *(offset_out + band) + sample) =
		        group_number + 1;
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 break;
      case EREAL:
	 if (max_flag)
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*(float *)new_pixel > *(float *)current_pixel) ||
		   (last_flag &&
		   (*(float *)new_pixel == *(float *)current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(float *)(out_buffer + *(offset_out + band) + sample) =
			*(float *)(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(float *)(out_buffer + *(offset_out + band) + sample) =
		        (float)(group_number + 1);
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 else
	    for (sample = 0; sample < out_window[group_number][NS] * size;
		 sample += size)
	       {
	       if ((*(float *)new_pixel < *(float *)current_pixel) ||
		   (last_flag &&
		   (*(float *)new_pixel == *(float *)current_pixel)))
		  {
		  for (band = 0; band < totbnd; band++)
		     *(float *)(out_buffer + *(offset_out + band) + sample) =
			*(float *)(in_buffer + *(offset_in + band) + sample);
                  if (ident_band)
		     *(float *)(out_buffer + *(offset_out + band) + sample) =
		        (float)(group_number + 1);
		  }
	       current_pixel += size;
	       new_pixel     += size;
	       }
	 break;
      } /* switch (odtype) */

   curent++;
   } /* for each line */

/* Write the last line of the output image
------------------------------------------*/
if (c_estep((struct GDESC **)&gdesc_out) != E_SUCC)
   {
   free(in_buffer);
   free(out_buffer);
   if (c_egclse((struct GDESC **)&gdesc_out) != E_SUCC)
      c_errmsg("Error closing image", "composit-close", &nonfatal);
   if (c_egclse((struct GDESC **)&gdesc_in) != E_SUCC)
      c_errmsg("Error closing image", "composit-close", &nonfatal);
   return(E_FAIL);
   }

/* Close the input and output images
------------------------------------*/
if (c_egclse((struct GDESC **)&gdesc_in) != E_SUCC)
   c_errmsg("Error closing image", "composit-close", &nonfatal);
if (c_egclse((struct GDESC **)&gdesc_out) != E_SUCC)
   c_errmsg("Error closing image", "composit-close", &nonfatal);

free(in_buffer);
free(out_buffer);
c_stoptm();
return(E_SUCC);
}
