/*******************************************************************************
NAME     GETPAR 

PURPOSE  Obtain the necessary parameters for the COMPOSIT function.

PROGRAM HISTORY

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    R.McKinney       06/88  EDC   Converted to LAS 5.0
  1.1    R.McKinney       12/88  EDC   Insured all files are closed
  2.0    B.Ailts          06/89  EDC   Fixed bug with 3 or more input files
  3.0    J.Fenno          01/92  EDC   Rewrote to make COMPOSIT like CONCAT.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS		                

ALGORITHM:
*******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock, /* I: pointer to TAE block                         */
    char hostin[][CMLEN],  /* O: host file names of input images              */
    char *hostout,         /* O: host file name of output image               */
    char *minmax,          /* O: flag to calculate min & max values for DDR   */
    char *taeout,          /* O: TAE file name of output image                */
    long *auto_subcommand, /* O: Flag indicating if auto subcommand was used  */
    long *band_to_compare, /* O: band to be compared among all images         */
    long bands[][MAXBND+1],/* O: band specification                           */
    long images_in_group[],/* O: number of images in each image group         */
    long in_window[][4],   /* O: input window specification for each image    */
    long *last_flag,       /* O: flag to use last image on ties,else the first*/
    long *max_flag,        /* O: flag to use the max pixel value, else the min*/
    long *msgtim,          /* O: elapsed time between display messages        */
    long *nbands,          /* O: number of bands for each input image         */
    long *nl,              /* O: number of lines in the output image          */
    long *ns,              /* O: number of samples in the output image        */
    long *num_img_groups,  /* O: number of image groups                       */
    long *num_images,      /* O: number of input images                       */
    long out_window[][4], /* O: output window specification for ea image group*/
    long *totbnd,          /* O: number of bands in each input image to be
			         composited.  In update mode image 1 will
			         contain (totbnd + 1) bands.                  */
    long *update_mode,     /* O: flag for updating the first input image      */
    long *ident_band       /* O: flag for including identifier band in image  */
)

{
struct VARIABLE *var;     /* pointer to TAE variable within v-block           */

char   taein[MAXIMG][CMLEN];/* TAE file names of input images                 */
char   tmp_str[CMLEN];    /* temp string for printing error messages          */

double outsize[2];        /* output image size                                */
double upleft[2];         /* upper left corner coordinates                    */

float  projtol;           /* projection coordinate tolerance                  */

long   i;                 /* loop control                                     */
long   j;                 /* loop control                                     */
long   k;                 /* loop control                                     */
long   last_line;         /* number of last line in the output image          */
long   last_sample;       /* number of last sample in the output image        */
long   max_images;        /* max number of images to be processed             */
long   nmrbnd;            /* number of bands for each entry in IN             */
long   nmrimg;            /* number of images for each entry in IN            */
long   nonfatal = 1;      /* nonfatal error flag                              */
long   outsize_flag;      /* flag to indicate if user specified OUTSIZE       */
long   qflag[MAXIMG][2];  /* flag indicating if subwindows specified          */
long   same;              /* flag indicating if images have same window       */
long   sl[52];            /* starting line in the output image where each
			     image group will be placed                       */
long   ss[52];            /* starting sample in the output image where each
			     image group will be placed                       */
long   status;            /* function return status code                      */
long   upleft_flag;       /* flag to indicate if user specified UPLEFT        */

/* Get the TAE global variables
-------------------------------*/
var = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*var,0);
var = p_find(vblock,"$MINMAX");
strcpy(minmax,SVAL(*var,0));

/* Get output image and determine if we are in UPDATE mode or not
-----------------------------------------------------------------*/
var = p_find(vblock,"OUT");
if (var->v_count == 1)
   {
   *update_mode = FALSE;
   status = c_getout(vblock,"OUT",(char (*)[256])taeout,(char (*)[256])hostout);
   if (status != E_SUCC)
      return(E_FAIL);
   }
else
   *update_mode = TRUE;

/* Get the band identifier flag. If BANDFLG is Yes, then the extra band that 
   is used to identify which image a pixel came from is created for the output
   image. 
-----------------------------------------------------------------------------*/
var = p_find(vblock, "BANDFLG");
if (strcmp(SVAL(*var,0), "YES") == 0)
   *ident_band = TRUE;
else
   *ident_band = FALSE;

/* Process each of the files specified by IN.  The number of input files can
   differ from the number of image groups.  For example, if one of the entries
   for IN is given as "image1(:3) + image2(:4 5)" this is one image group with
   3 bands and 2 images.
   If in UPDATE mode the first image group must contain only one image file and
   it must have one more band than all the other image groups.  The last band
   will be used for storing the image numbers.  If not in UPDATE mode each image
   group must contain the same number of bands.
   If the indentifier band is not included in the output image, each image
   group must contain the same number of bands.
----------------------------------------------------------------------------*/
var = p_find(vblock,"IN");
*num_img_groups = var->v_count;
*num_images = 0;
*totbnd     = 0;
nmrimg      = 0;
max_images  = MAXIMG;
for (i = 0; i < *num_img_groups; i++)
   {
   status = c_getin(vblock, "IN", &i,(char (*)[256])taein[*num_images], 
                    (char (*)[256])hostin[*num_images],
		    &nmrimg,(long (*)[4])in_window[*num_images], 
                    (long (*)[257])bands[*num_images],
		    &nbands[*num_images], &nmrbnd, &max_images, &same,
		    (long (*)[2])qflag[*num_images]);
   if (status != E_SUCC)
      return(E_FAIL);
   if (!same)
      {
      sprintf(tmp_str,
	 "Images in position %1d of IN do not have equal window sizes",i+1);
      c_errmsg(tmp_str,"composit-window",&nonfatal);
      return(E_FAIL);
      }
   
   if (i == 0)
      {
      if (*update_mode && (nmrimg != 1))
	 {
	 sprintf(tmp_str,"First entry of IN contains %1d images.  %s", nmrimg,
		 "Must contain 1 when updating.");
	 c_errmsg(tmp_str, "composit-numimage",&nonfatal);
	 return(E_FAIL);
	 }
      else if (*update_mode && *ident_band)
	 *totbnd = nmrbnd - 1;  /* all other images must have 1 fewer bands */
      else
	 *totbnd = nmrbnd;
      }
   else if (nmrbnd != *totbnd)
      {
      sprintf(tmp_str,"Image %1d has %1d bands, expecting %1d",
	      i + 1, nmrbnd, *totbnd);
      c_errmsg(tmp_str, "composit-nbands", &nonfatal);
      return(E_FAIL);
      }
   images_in_group[i] = nmrimg;
   *num_images += nmrimg;
   }

/* Copy the name of the image to be updated to the output names
---------------------------------------------------------------*/
if (*update_mode)
   {
   strcpy(hostout, hostin[0]);
   strcpy(taeout,  taein[0]);
   }

/* Get the band to be compared
------------------------------*/
var = p_find(vblock,"BAND");
*band_to_compare = IVAL(*var,0);
if (*band_to_compare > *totbnd)
   {
   sprintf(tmp_str,"Band specified for compositing is greater than number %s",
	   "of bands in the input image(s)");
   c_errmsg(tmp_str, "composit-band", &nonfatal);
   return(E_FAIL);
   }

/* Get the replacement option
-----------------------------*/
var = p_find(vblock,"REPLOPT");
if (strcmp(SVAL(*var,0), "LAST") == 0)
   *last_flag = TRUE;
else
   *last_flag = FALSE;

/* Get the minimum or maximum pixel option
------------------------------------------*/
var = p_find(vblock,"SELOPT");
if (strcmp(SVAL(*var,0), "MAX") == 0)
   *max_flag = TRUE;
else
   *max_flag = FALSE;

/* Based on the subcommand retrieve or compute the sl, ss, nl, and ns
---------------------------------------------------------------------*/
var = p_find(vblock,"_SUBCMD");
if (strcmp(SVAL(*var,0), "MANUAL") == 0)
   {
   *auto_subcommand = FALSE;

   /* Get the output image size
   ----------------------------*/
   if (*update_mode)
      {
      *nl = in_window[0][NL];
      *ns = in_window[0][NS];
      }
   else
      {
      var = p_find(vblock, "NL");
      if (var->v_count == 0)
	 {
	 c_errmsg("NL must be specified when creating an output image",
		  "composit-required",&nonfatal);
	 return(E_FAIL);
	 }
      *nl = IVAL(*var, 0);
      var = p_find(vblock, "NS");
      if (var->v_count == 0)
	 {
	 c_errmsg("NS must be specified when creating an output image",
		  "composit-required",&nonfatal);
	 return(E_FAIL);
	 }
      *ns = IVAL(*var, 0);
      }

   /* Get the starting line locations
   ----------------------------------*/
   var = p_find(vblock,"SL");
   if (var->v_count < *num_img_groups)
      {
      c_errmsg("SL and IN must have the same number of entries",
	       "composit-unequal", &nonfatal);
      return(E_FAIL);
      }
   for (i = 0; i < *num_img_groups; i++)
      sl[i] = IVAL(*var, i);

   /* Get the starting sample locations
   ------------------------------------*/
   var = p_find(vblock,"SS");
   if (var->v_count < *num_img_groups)
      {
      c_errmsg("SS and IN must have the same number of entries",
	       "composit-unequal", &nonfatal);
      return(E_FAIL);
      }
   for (i = 0; i < *num_img_groups; i++)
      ss[i] = IVAL(*var, i);

   /* Check for starting location outside of the image boundary
   ------------------------------------------------------------*/
   for (i = 0; i < *num_img_groups; i++)
      {
      if (sl[i] > *nl)
	 {
	 c_errmsg("Starting line is outside the output image",
		  "composit-location", &nonfatal);
	 return(E_FAIL);
	 }
      if (ss[i] > *ns)
	 {
	 c_errmsg("Starting sample is outside the output image",
		  "composit-location", &nonfatal);
	 return(E_FAIL);
	 }
      }
   }
else
   {
   *auto_subcommand = TRUE;

   /* Get the projection coordinate tolerance
   ------------------------------------------*/
   var = p_find(vblock, "$PROJTOL");
   projtol = RVAL(*var, 0);

   /* Get the upper left corner coordinates
   ----------------------------------------*/
   var = p_find(vblock, "UPLEFT");
   if (var->v_count == 0)
      upleft_flag = FALSE;
   else
      {
      upleft_flag = TRUE;
      sscanf(SVAL(*var,0), "%lf", &upleft[0]);
      sscanf(SVAL(*var,1), "%lf", &upleft[1]);
      }

   /* Get the output image size
   ----------------------------*/
   var = p_find(vblock, "OUTSIZE");
   if (var->v_count == 0)
      outsize_flag = FALSE;
   else
      {
      outsize_flag = TRUE;
      sscanf(SVAL(*var,0), "%lf", &outsize[0]);
      sscanf(SVAL(*var,1), "%lf", &outsize[1]);
      }

   /* Calculate the sl and ss in the output image for each of the input images
      and calculate nl and ns for the output image.  If in update mode
      cal_lin_smp should be called for the image to be updated and then called
      for all of the other input images.  This will ensure that sl, ss, nl, and
      ns are all set according to the image to be updated.
   ---------------------------------------------------------------------------*/
   if (*update_mode)
      {
      long tmp_images = 1;
      long tmp_img_groups = 1;

      status = cal_lin_smp(hostin[0], &tmp_images, &tmp_img_groups,
			   images_in_group, in_window, &projtol, &upleft_flag,
			   &outsize_flag, upleft, outsize, sl, ss, nl, ns);
      if (status != E_SUCC)
	 return(E_FAIL);
      upleft_flag = TRUE;
      outsize_flag = TRUE;
      tmp_images = *num_images - 1;
      tmp_img_groups = *num_img_groups - 1;
      status = cal_lin_smp(hostin[1], &tmp_images, &tmp_img_groups,
			   &images_in_group[1], (long (*)[4])in_window[1], 
                           &projtol, &upleft_flag, &outsize_flag, upleft,
                           outsize, &sl[1], &ss[1], nl, ns);
      if (status != E_SUCC)
	 return(E_FAIL);
      }
   else
      {
      status = cal_lin_smp(hostin[0], num_images, num_img_groups,
			   images_in_group, in_window, &projtol, &upleft_flag,
			   &outsize_flag, upleft, outsize, sl, ss, nl, ns);
      if (status != E_SUCC)
	 return(E_FAIL);
      }
   }

/* Combine the sl and ss arrays returned from cal_lin_smp into an output
   window for each image group.  This will define the placement of each
   image group in the output image.  Remember that there is one input
   window per IMAGE but only one output window per IMAGE GROUP.
------------------------------------------------------------------------*/
if (*update_mode)
   {
   last_line   = in_window[0][SL] + in_window[0][NL] - 1;
   last_sample = in_window[0][SS] + in_window[0][NS] - 1;
   }
else
   {
   last_line   = *nl;
   last_sample = *ns;
   }
j = 0;
for (i = 0; i < *num_img_groups; i++)
   {
   out_window[i][SL] = sl[i];
   out_window[i][SS] = ss[i];
   if (last_line < (sl[i] + in_window[j][NL] - 1))
      out_window[i][NL] = last_line - out_window[i][SL] + 1;
   else
      out_window[i][NL] = in_window[j][NL];

   if (last_sample < (ss[i] + in_window[j][NS] - 1))
      out_window[i][NS] = last_sample - out_window[i][SS] + 1;
   else
      out_window[i][NS] = in_window[j][NS];
   j += images_in_group[i];
   }

/* Reset the input window's NL and NS.  This will ensure that we only open
   that portion of the input images that will be placed in the output.
--------------------------------------------------------------------------*/
k = 0;
for (i = 0; i < *num_img_groups; i++)
   {
   if (out_window[i][NL] < 0)
      out_window[i][NL] = 0;
   if (out_window[i][NS] < 0)
      out_window[i][NS] = 0;
   for (j = 0; j < images_in_group[i]; j++)
      {
      in_window[k][NL] = out_window[i][NL];
      in_window[k][NS] = out_window[i][NS];
      k++;
      }
   }

return(E_SUCC);
}
