/*******************************************************************************
NAME:    INIT_OUTPUT

PURPOSE: Initialize the output image.  When updating an image set every pixel
	 in the last band to 1.  When creating a new image copy the first input
	 image to the output storing either a 0 or 1 in the last band.
	 0 indicates that a pixel was not taken from the image while 1 indicates
	 that a pixel came from the image.

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Developed for the new version of COMPOSIT
  2.0    S. Nelson	  10/92  EDC   Changed COMPRES off for output so that
				       image can be opened.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
=========================================================================*/
extern long curcat;              /* current image being processed             */
extern long curent;              /* current line being processed              */
extern int  fdesc_in[];          /* file descriptor for the input image       */
extern int  fdesc_out[];         /* file descriptor for the output image      */
extern int  gdesc_in;            /* group descriptor for the input image      */
extern long *offset_in;          /* offsets into the buffer for input image   */


int init_output
(
    char hostin[][CMLEN],  /* I: Host file names of the input images          */
    char hostout[],        /* I: Host file name of the output image           */
    long bands[][MAXBND+1],/* I: The numbers of the bands to use for ea image */
    long images_in_group[],/* I: Number of images in each image group         */
    long in_window[][4],   /* I: Input window specification for ea input image*/
    long max_flag,         /* I: Flag to find max pixel value, else the min   */
    long msgtime,          /* I: Num of seconds between processing messages   */
    long nbands[],         /* I: Number of bands in each input image          */
    long nl,               /* I: Number of lines in the output image          */
    long ns,               /* I: Number of samples in the output image        */
    long odtype,           /* I: Output data type                             */
    long out_window[][4],  /* I:Output window specification for ea image group*/
    long size,             /* I: Num of bytes needed for the output data type */
    long totbnd,           /* I: total number of bands in each input image    */
    long update_mode,      /* I: Flag for updating the first input image      */
    int ident_band	   /* I: Flag to include identifier band in image     */ 
)

{
long access;              /* File access mode                                 */
long band;                /* Loop control                                     */
long buffer_size;         /* Size, in bytes, of the input buffer              */
long i;                   /* Loop control                                     */
long line;                /* Loop control                                     */
long nonfatal =  1;       /* Nonfatal error flag                              */
long one      =  1;       /* Value of 1                                       */
long opt[3];		  /* Array to set COMPRES to off when opening output  */
long sample;              /* Loop control                                     */
long sl;                  /* Starting line of the image to be opened          */
long ss;                  /* Starting sample of the image to be opened        */
long status;              /* Status code returned from LAS functions          */
long zero     =  0;       /* Value of 0                                       */

unsigned char *in_buffer;        /* Buffer for reading from input images      */
unsigned char *out_buffer[2];    /* Buffer for writing to the output file     */

opt[0]=opt[2]=zero;
opt[1]=NO_COMPRES;

/* Open the output file for random reading and writing
------------------------------------------------------*/
if (update_mode)
   {
   access = IUPDATE;	
   sl = in_window[0][SL];
   ss = in_window[0][SS];
   status = c_eopenr((struct FDESC **)&fdesc_out[0], hostout, &nbands[0], &sl,
                     &ss, &nl, &ns, &access, &odtype, opt, &one);
   }
else
   {
   access = IWRITE;	
   sl = 1;
   ss = 1;
   if(ident_band)
      i = totbnd + 1;
   else
      i = totbnd;
   status = c_eopenr((struct FDESC **)&fdesc_out[0], hostout, &i, &sl, &ss, &nl,
                       &ns, &access, &odtype, opt, &one);
   }
if (status != E_SUCC)
   return(E_FAIL);
fdesc_out[1] = 0;

/* Start the processing message
-------------------------------*/
curent = 1;
curcat = 1;
i = GEN2;
c_inittm("COMPOSIT", "Processed", "line", "image", "for image 1", &nl, &zero,
	 &msgtime, &i);
c_startm();

if (update_mode)
   {
   /* Allocate a buffer for writing to the output image
   ----------------------------------------------------*/
   if ((out_buffer[0] = (unsigned char *)malloc(ns * size)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
      if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	 c_errmsg("Error closing image", "composit-close", &nonfatal);
      return(E_FAIL);
      }

   /* Initialize the out_buffer to contain all 1's
   -----------------------------------------------*/
   switch (odtype)
      {
      case EBYTE:
	 for (sample = 0; sample < ns * size; sample += size)
	    *(unsigned char *)(out_buffer[0] + sample) = 1;
	 break;
      case EWORD:
	 for (sample = 0; sample < ns * size; sample += size)
	    *(short *)(out_buffer[0] + sample) = 1;
	 break;
      case ELONG:
	 for (sample = 0; sample < ns * size; sample += size)
	    *(long *)(out_buffer[0] + sample) = 1;
	 break;
      case EREAL:
	 for (sample = 0; sample < ns * size; sample += size)
	    *(float *)(out_buffer[0] + sample) = 1.0;
	 break;
      }

   /* totbnd is the number of bands being composited.  When in update mode
      bands[0][totbnd] is the band used to store the image numbers.
   -----------------------------------------------------------------------*/
   band = bands[0][totbnd];

   /* For the portion of the output image that is being composited store a 1
      in the last band to indicate that the pixels have come from image 1,
      if the identifier band is to included in the output image.
   -------------------------------------------------------------------------*/
   if (ident_band)
      {
      for (line = 1; line <= nl; line++)
         {
         status = c_ewrite((struct FDESC **)fdesc_out, &band, &line,    
                           out_buffer[0], &one);
         if (status == E_FAIL)
	    {
	    free(out_buffer[0]);
	    if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	       c_errmsg("Error closing image", "composit-close", &nonfatal);
	    return(E_FAIL);
	    }
         curent++;
         }
      }
   free(out_buffer[0]);
   }
else
   {
   /* If not in update mode then copy the first image group to the output image
      and store either a 0 or a 1 in the last band.  A 0 indicates that a value
      was not taken from the group and a 1 indicates that one was.
   ---------------------------------------------------------------------------*/

   /* Allocate a buffer for writing to the output image.  Output_buffer[0] is
      used for copying the pixel values from the input image while
      out_buffer[1] is used for creating the last band, the compositing band.
   ---------------------------------------------------------------------------*/
   if ((out_buffer[0] = (unsigned char *)malloc(ns * size)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
      if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	 c_errmsg("Error closing image", "composit-close", &nonfatal);
      return(E_FAIL);
      }
   if (ident_band)
      {
      if ((out_buffer[1] = (unsigned char *)malloc(ns * size)) == NULL)
         {
         free(out_buffer[0]);
         c_errmsg("Error allocating dynamic memory", "composit-alloc", 
                   &nonfatal);
         if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	    c_errmsg("Error closing image", "composit-close", &nonfatal);
         return(E_FAIL);
         }
      }

   /* Initialize out_buffer to contain the fill values
   ---------------------------------------------------*/
   switch (odtype)
      {
      case EBYTE:
	 for (sample = 0; sample < ns * size; sample += size)
	    {
	    if (max_flag)
	       *(unsigned char *)(out_buffer[0] + sample) = MINUTINY;
	    else
	       *(unsigned char *)(out_buffer[0] + sample) = MAXUTINY;
            if (ident_band)
	       *(unsigned char *)(out_buffer[1] + sample) = 0;
	    }
	 break;
      case EWORD:
	 for (sample = 0; sample < ns * size; sample += size)
	    {
	    if (max_flag)
	       *(short *)(out_buffer[0] + sample) = MINWORD;
	    else
	       *(short *)(out_buffer[0] + sample) = MAXWORD;
            if (ident_band)
	       *(short *)(out_buffer[1] + sample) = 0;
	    }
	 break;
      case ELONG:
	 for (sample = 0; sample < ns * size; sample += size)
	    {
	    if (max_flag)
	       *(long *)(out_buffer[0] + sample) = MINLONG;
	    else
	       *(long *)(out_buffer[0] + sample) = MAXLONG;
            if (ident_band)
	       *(long *)(out_buffer[1] + sample) = 0;
	    }
	 break;
      case EREAL:
	 for (sample = 0; sample < ns * size; sample += size)
	    {
	    if (max_flag)
	       *(float *)(out_buffer[0] + sample) = MINREAL;
	    else
	       *(float *)(out_buffer[0] + sample) = MAXREAL;
            if (ident_band)
	       *(float *)(out_buffer[1] + sample) = 0.0;
	    }
	 break;
      }

   /* Blank out all lines in the output image that lie above image group 1
   -----------------------------------------------------------------------*/
   for (line = 1; line < out_window[0][SL]; line++)
      {
      for (band = 1; band <= totbnd + 1; band++)
	 {
	 if (band <= totbnd)
	    status = c_ewrite((struct FDESC **)fdesc_out, &band, &line,
                               out_buffer[0], &one);
	 else if (ident_band)
	    status = c_ewrite((struct FDESC **)fdesc_out, &band, &line,  
                                out_buffer[1], &one);
         else status = E_SUCC;
	 if (status == E_FAIL)
	    {
	    free(out_buffer[0]);
            if (ident_band)
	       free(out_buffer[1]);
	    if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	       c_errmsg("Error closing image", "composit-close", &nonfatal);
	    return(E_FAIL);
	    }
	 }
      curent++;
      }

   /* Blank out all lines in the output image that lie below image group 1
   -----------------------------------------------------------------------*/
   for (line = out_window[0][SL] + out_window[0][NL]; line < nl; line++)
      {
      for (band = 1; band <= totbnd + 1; band++)
	 {
	 if (band <= totbnd)
	    status = c_ewrite((struct FDESC **)fdesc_out, &band, &line, 
                              out_buffer[0], &one);
	 else if (ident_band)
	    status = c_ewrite((struct FDESC **)fdesc_out, &band, &line, 
                              out_buffer[1], &one);
         else status = E_SUCC;
	 if (status == E_FAIL)
	    {
	    free(out_buffer[0]);
            if (ident_band)
	       free(out_buffer[1]);
	    if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	       c_errmsg("Error closing image", "composit-close", &nonfatal);
	    return(E_FAIL);
	    }
	 }
      curent++;
      }
   
   /* Store a 1 in the output buffer for the last band to
      indicate that the pixel value came from image group 1
   --------------------------------------------------------*/
   if (ident_band)
      {
      switch (odtype)
         {
         case EBYTE:
       	    for (sample = (out_window[0][SS] - 1) * size;
	         sample < (out_window[0][SS] + out_window[0][NS] - 1) * size;
	         sample += size)
	       *(unsigned char *)(out_buffer[1] + sample) = 1;
	    break;
         case EWORD:
	    for (sample = (out_window[0][SS] - 1) * size;
	         sample < (out_window[0][SS] + out_window[0][NS] - 1) * size;
	         sample += size)
	       *(short *)(out_buffer[1] + sample) = 1;
	    break;
         case ELONG:
   	    for (sample = (out_window[0][SS] - 1) * size;
	         sample < (out_window[0][SS] + out_window[0][NS] - 1) * size;
	         sample += size)
	       *(long *)(out_buffer[1] + sample) = 1;
	    break;
         case EREAL:
	    for (sample = (out_window[0][SS] - 1) * size;
	         sample < (out_window[0][SS] + out_window[0][NS] - 1) * size;
	         sample += size)
	       *(float *)(out_buffer[1] + sample) = 1.0;
	    break;
         }
      }

   status = open_image(hostin, bands, &buffer_size, 0, images_in_group,
		       TRUE, nbands, odtype, in_window, &in_buffer);
   if (status != E_SUCC)
      {
      free(out_buffer[0]);
      if (ident_band)
         free(out_buffer[1]);
      if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	 c_errmsg("Error closing image", "composit-close", &nonfatal);
      return(E_FAIL);
      }

   /* Copy image group 1 to the output image
   -----------------------------------------*/
   for (line = out_window[0][SL];
	line < out_window[0][SL] + out_window[0][NL];
	line++)
      {
      if (c_estep((struct GDESC **)&gdesc_in) != E_SUCC)
	 {
	 free(in_buffer);
	 free(out_buffer[0]);
         if (ident_band)
	    free(out_buffer[1]);
	 if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	    c_errmsg("Error closing image", "composit-close", &nonfatal);
	 if (c_egclse((struct GDESC **)&gdesc_in) != E_SUCC)
	    c_errmsg("Error closing image", "composit-close", &nonfatal);
	 return(E_FAIL);
	 }
      for (band = 1; band <= totbnd + 1; band++)
	 {
	 if (band <= totbnd)
	    {
	    c_pxcopy(
	       (in_buffer  + *(offset_in + band - 1) * size),
	       (out_buffer[0] + (out_window[0][SS] - 1) * size),
	       &odtype, &out_window[0][NS]);
	    status = c_ewrite((struct FDESC **)fdesc_out, &band, &line,  
                               out_buffer[0], &one);
	    }
	 else if (ident_band)
      	    status = c_ewrite((struct FDESC **)fdesc_out, &band, &line, 
                               out_buffer[1], &one);
         else status = E_SUCC;
	 if (status == E_FAIL)
	    {
	    free(in_buffer);
	    free(out_buffer[0]);
            if (ident_band)
	       free(out_buffer[1]);
	    if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
	       c_errmsg("Error closing image", "composit-close", &nonfatal);
	    if (c_egclse((struct GDESC **)&gdesc_in) != E_SUCC)
	       c_errmsg("Error closing image", "composit-close", &nonfatal);
	    return(E_FAIL);
	    }
	 }
      curent++;
      }
   
   /* Close the first image group
   ------------------------------*/
   if (c_egclse((struct GDESC **)&gdesc_in) != E_SUCC)
      c_errmsg("Error closing image","composit-close",&nonfatal);

   free(in_buffer);
   free(out_buffer[0]);
   if (ident_band)
      free(out_buffer[1]);
   }

/* Close the output file
------------------------*/
if (c_eclose((struct FDESC **)fdesc_out) != E_SUCC)
   c_errmsg("Error closing image", "composit-close", &nonfatal);

c_stoptm();
return(E_SUCC);
}
