/*******************************************************************************
NAME:    OPEN_IMAGE

PURPOSE: Using c_eopens, c_ealloc, and c_egroup prepare an input image for
	 sequential reading or prepare the output image for sequential updating.

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Developed to reduce code
  2.0    S.Nelson	  10/92  EDC   Changed COMPRES off for output so that
				       image can be opened.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"
#include "imageio.h"

/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
extern int  fdesc_in[];   /* file descriptors for the input images            */
extern int  fdesc_out[];  /* file descriptor for the output image             */
extern int  gdesc_in;     /* group descriptor for the input image groups      */
extern int  gdesc_out;    /* group descriptor for the output image            */
extern long *offset_in;   /* offsets into input image buffer                  */
extern long *offset_out;  /* offsets into output image buffer                 */

int open_image
(
    char hostnames[][CMLEN],/*I: Host file names of the images to be opened   */
    long bands[][MAXBND+1],/* I: The numbers of the bands to use for ea image */
    long *buffer_size,     /* I: Size of the buffer allocated by c_ealloc     */
    long group_number,     /* I: # of image group to be opened (zero indexed) */
    long images_in_group[],/* I: Number of images in each image group         */
    long input_file,       /* I: Flag to open either input or output image(s) */
    long nbands[],         /* I: Number of bands in each input image          */
    long odtype,           /* I: Output data type                             */
    long window[][4],      /* I: Window specification for each input image    */
    unsigned char **buffer /* I: Buffer allocated by c_ealloc for reading     */
)
{
long access;              /* Access type for the input images                 */
long band;                /* Loop control                                     */
long i;                   /* Loop control                                     */
long image;               /* Loop control                                     */
long nonfatal =  1;       /* Nonfatal error flag                              */
long opt[3];		  /* Compression off option     		      */
long status;              /* Function return status                           */
long zero     = 0;        /* Value of 0                                       */


if (input_file)
   {
   /* Initialize local variables
   -----------------------------*/
   access = IREAD;
   band   = 0;
   image  = 0;

   /* Determine the first input image to be opened
   -----------------------------------------------*/
   for (i = 0; i < group_number; i++)
      image += images_in_group[i];

   /* Open all of the images in this image group
   ---------------------------------------------*/
   opt[0]=opt[1]=opt[2]=0;
   for (i = 0; i < images_in_group[group_number]; i++)
      {
      status = c_eopens((struct FDESC **)&fdesc_in[i], hostnames[image], 
                        bands[image],
			&window[image][SL], &window[image][SS],
			&window[image][NL], &window[image][NS],
			(offset_in + band), &access, &odtype, &zero, opt);
      if (status != E_SUCC)
	 return(E_FAIL);
      band += nbands[image];
      image++;
      }
   fdesc_in[i] = 0;

   /* Allocate a buffer for this group
   -----------------------------------*/
   *buffer = (unsigned char *) c_ealloc((struct FDESC **)fdesc_in, buffer_size);
   if (*buffer == NULL)
      {
      c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
      return(E_FAIL);
      }

   /* Group all input images together
   ----------------------------------*/
   status = c_egroup((struct FDESC **)fdesc_in,(struct GDESC **) &gdesc_in,
                      *buffer,buffer_size);
   if (status != E_SUCC)
      {
      free(*buffer);
      return(E_FAIL);
      }
   }
else
   {
   /* Initialize local variables
   -----------------------------*/
   access = IUPDATE;
   opt[0]=opt[2]=zero;
   opt[1]=NO_COMPRES;

   /* Open the image
   -----------------*/
   status = c_eopens((struct FDESC **)&fdesc_out[0],(char *)hostnames, bands[0],
                     &window[0][SL], &window[0][SS], &window[0][NL], 
                     &window[0][NS],
		     offset_out, &access, &odtype, &zero, opt);
   if (status != E_SUCC)
      return(E_FAIL);
   fdesc_out[1] = 0;

   /* Allocate a buffer for this group
   -----------------------------------*/
   *buffer = (unsigned char *)c_ealloc((struct FDESC **)fdesc_out, buffer_size);
   if (*buffer == NULL)
      {
      c_errmsg("Error allocating dynamic memory", "composit-alloc", &nonfatal);
      return(E_FAIL);
      }

   /* Group all input images together
   ----------------------------------*/
   status = c_egroup((struct FDESC **)fdesc_out,(struct GDESC **) &gdesc_out,
                     *buffer, buffer_size);
   if (status != E_SUCC)
      {
      free(*buffer);
      return(E_FAIL);
      }
   }

return(E_SUCC);
}
