/*******************************************************************************
NAME:      		   	CONCAT

PURPOSE:    Create an output image by overlaying each input image into
            the output at the specified locations 

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey      Aug   1984	Original development
D. Akkerman    Aug   1987       PR #4247 modified to allow for single band 
				 specification of a multi-band images
K. Zanter      Sept. l987       NEWLAS (conversion to 'C')
B. Ailts       July  1988       LAS5.0 conversion -- added -AUTO subcommand
B. Ailts       Nov   1988       Changed name of cal_line_samp and Set_data_type
				in order to work on the IBM RT
T. Mittan      Jan   1992	Added update, and averaging options, also 
				maskval can now be a range
T. Mittan      Mar   1992	Modified averaging option to choose which 
				direction the average will be performed.
D. Etrheim     Sep   1992       Check pixel size against an Epsilon value.
                                Comparing floating point values directly can
                                cause problems if an image is transfered from
                                another machine.
D. Etrheim     Apr   1993       Correct problem in update mode of deleting the
				first input image.  Free allocated space.
D. Bridges     Oct   1993       Modified to allow NL and NS to accept NULL when
				in undate mode.
D. Bridges     Feb   1994       Corrected a problem with OVEROPT so that it
   				works correctly when it has 1 less argument 
				than specified in IN.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   CONCAT must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Initialization of variables
Retrieve the user-specified parameters.  
The data type of the output image is determined by the maximum data type 
 of the input images.
If subcommand equals AUTO
  For each of the input images
      The projection coordinates are retrieved from the DDR files
      The projection coordinates are re-calculated to compensate for specified
       windows
      The starting line and sample locations within the output image are
       calculated for each input image
Else
  The starting line and sample parameters are retrieved
Output image is opened to the specified dimensions (NL and NS parameters)
If not in update mode output image is padded with the first MASK value
For each input image
   The ouput image is opened for update
   The input image is overlaid into the output image according to the 
    appropriate starting line and sample values
If $MINMAX equals YES second pass is made to determine min and max values of 
 each band
The associated min/max and history files are created for the output image 
The completion message is output.

ALGORITHM REFERENCES:     none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "concat.h"

#define NIMGS 50

/* The following variables must be declared global for image I/O routines */
struct FDESC *fdesc[MAXIMG];	     /* image file descriptors	      */
struct GDESC *gdesc;			     /* group descriptor 	      */
long offset1[MAXIMG * MAXBND + 1];/* offsets into thebuf for input */
long offset2[MAXBND+1];		     /* offsets into thebuf for output*/

void main
(
  void
)
{
struct PARBLK vblk;		 /* initial TAE parameter block		      */
struct VARIABLE *var;		 /* pointer to parameter value		      */

double line_inc;		 /* increment value in line direction	      */
double samp_inc;		 /* increment value in sample direction	      */
static double	maxval[MAXBND];	 /* maximum intensity values	       	      */
static double	minval[MAXBND];	 /* minimum intensity values	       	      */
float  maskval[NIMGS][2]; 	 /* mask values				      */

static long    bands[MAXIMG * NIMGS][MAXBND+1];/* band specification	      */
long	aver;			 /* average option for overlap condition      */
long	*laver; 	  	 /* average option for overlap condition      */
long    cflag;			 /* combination flag for C_UPDDR	      */
long    fatal = -1;		 /* fatal flag 				      */
long    image_count;		 /* total number of images specified	      */
long 	in_count;		 /* number of IN entries specified 	      */
long    index;			 /* index				      */
long	i,j,k;			 /* counter				      */
long	update;			 /* output image flag			      */
long	leave;			 /* leave option for overlap condition	      */
long    limage;			 /* current logical image		      */
long    msgtim;			 /* elapsed time between display messages     */
long    nbands[MAXIMG * NIMGS];	 /* number of bands in each image	      */
long    nl;			 /* number of lines in output image           */
long    nl1;			 /* number of lines in output image           */
long    nonfatal = 1;		 /* nonfatal error code			      */
long    ns;			 /* number of samples per line in output image*/
long    ns1;			 /* number of samples per line in output image*/
long    numimg[NIMGS];		 /* number of images in each IN entry	      */
long    one = 1;		 /* value of one			      */
long    out_dtype;		 /* data type of output image		      */
long	replace;		 /* replace option for overlap condition      */
long    sl[NIMGS];		 /* starting line locations		      */
long    ss[NIMGS];		 /* starting sample locations		      */
long	status;			 /* function return status code		      */
long 	std_out;		 /* logical unit number for standard output   */
long    totbnd;			 /* total number of output bands              */
long    twindow[MAXIMG][4];	 /* temporary window specifications	      */
long    window[NIMGS * MAXIMG][4]; /* window specification		      */
long    out_window[NIMGS][4]; 	 /* window specification		      */
long	zero = 0;		 /* value of zero			      */

char    *hostin; 		 /* buffer storage for "hostin"		      */
char    *hosti_ptr; 		 /* buffer storage for "hostin"		      */
char    *hosto_ptr; 		 /* buffer storage for "hostin"		      */
char    hostout[CMLEN];          /* buffer storage for "hostout" 	      */
char    minmax[4];  	 	 /* minmax flag buffer			      */
char    option[NIMGS][7];	 /* option values			      */
char    taeout[CMLEN];           /* buffer storage for "taeout"   	      */

/* initialization of parameter block
------------------------------------*/
c_inlas(&vblk,&std_out);	 

/* Get the number of logical images specified
---------------------------------------------*/
var = p_find(&vblk,"IN");
in_count = var->v_count;

/* Allocate memory for the host names
-------------------------------------*/
hostin = (char *)malloc(in_count * MAXIMG * CMLEN);
if (hostin == NULL)
  {
  c_lasmsg(FALSE,"concat-alloc","Error allocating memory");
  c_lasmsg(TRUE,"concat-fatal","Fatal error encountered");
  }

/* get user-specified parameters and TAE globals 
------------------------------------------------*/
getpar(&vblk,hostin,&in_count,window,bands,&totbnd,nbands,
       numimg,&image_count,taeout,hostout,&msgtim,minmax,sl,ss,
       &nl,&ns,option,maskval,&update,out_window);

/* Allocate memory for the overlay option
-----------------------------------------*/ 
if ((laver = (long *)calloc(ns,(sizeof(long *)))) == NULL)
   {
   c_errmsg("Error dynamically alocating memory","concat-warn",&nonfatal);
   c_errmsg("Fatal error encountered","concat-warn",&fatal);
   }

/* determine output data type 
-----------------------------*/
set_dtype(hostin,image_count,&out_dtype,maskval,&in_count);

/* initialize output image by padding with mask value 
-----------------------------------------------------*/
if (!update)
   pad(&vblk,hostout,&nl,&ns,&totbnd,&out_dtype,&maskval[0][0],&msgtim);

/* overlay inset images 
-----------------------*/
index = 0;
hosti_ptr = hostin;
if (update)
  {
  hosto_ptr = hostin;
  hosti_ptr += (numimg[0] * CMLEN);
  index += numimg[0];
  limage = 1;
  nl1 = nl;
  ns1 = ns;
  }
else 
  {
  limage = 0;
  hosto_ptr = hostout;
  }

for (; limage < in_count; limage++)
    {
    /*if (update)
       {
       if (window[0][NL] < nl1)
          nl = window[0][NL] + sl[limage] - 1;
       else
          nl = nl1;
       if (window[0][NS] < ns1)
          ns = window[0][NS] + ss[limage] - 1;
       else
          ns = ns1;
       }*/
    if (strcmp(option[limage],"REPLAC") == 0)
       {
       replace = TRUE;
       leave = FALSE;
       aver = FALSE;
       }
    else
    if (strcmp(option[limage],"LEAVE") == 0)
       {
       replace = FALSE;
       leave = TRUE;
       aver = FALSE;
       }
    else
       {
       replace = FALSE;
       leave = FALSE;
       aver = TRUE;
       for (i = 0 ; i < ns; i++)
           {
           laver[i] = FALSE;
           }
       averopt(laver,out_window,&limage);
       }
   if ((window[index][NL] > 0) && (window[index][NS] > 0))
       overlay_img(&vblk,hosti_ptr,numimg[limage],window[index],bands[index],
	   &nbands[index],&sl[limage],&ss[limage],&maskval[limage][0],hosto_ptr,
	   &out_dtype,&totbnd,&nl,&ns,replace,leave,aver,&msgtim,&update,laver);
	
    hosti_ptr += (numimg[limage] * CMLEN);
    index += numimg[limage];
    }

/* Update the DDR fields.  Since the input images all start at different places
   in the output image.  Set a temporary window of each of the input images
   to show how they relate to the output space.
------------------------------------------------------------------------------*/
k=0;
for (i = 0; i < in_count; i++)
   {
   for (j =0; j < numimg[i]; j++)
       {
       twindow[k][SL] = window[k][SL] - sl[i] + 1;
       twindow[k][SS] = window[k][SS] - ss[i] + 1;
       twindow[k][NL] = nl;
       twindow[k][NS] = ns;
       k++;
       }
   }
line_inc = 1.0;
samp_inc = 1.0;
cflag = COMB;
if (!update)
   status = c_upddr(&vblk,hostin,hosto_ptr,&image_count,twindow,bands,nbands,
		 &line_inc,&samp_inc,&totbnd,&cflag);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);

/* if necessary, determine min and max values 
---------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
     mnmx(&vblk,hosto_ptr,&nl,&ns,&out_dtype,&totbnd,minval,maxval,&msgtim,
          &update);

/* create associated min/max file 
---------------------------------*/
status = c_svmnmx(hosto_ptr,&totbnd,minval,maxval,minmax);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);

/* create associated history file
--------------------------------- */
hosti_ptr = hostin;
if (update)
  {
  hosti_ptr += (numimg[0] * CMLEN);
  image_count--;
  }
status = c_hstout(&vblk,hosto_ptr,hosti_ptr,&image_count," ");
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);

/* inform user of successful completion 
---------------------------------------*/
if (update)
   c_complt("CONCAT"," ",&zero);
else
   c_complt("CONCAT",taeout,&one);
}
