/*******************************************************************************
NAME:      		   	GETPAR

PURPOSE:   	To retreive the TAE paramters

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey      Aug   1984	Original development
D. Akkerman    Aug   1987       PR #4247 modified to allow for single band 
				 specification of a multi-band images
K. Zanter      Sept. l987       NEWLAS (conversion to 'C')
B. Ailts       July  1988       LAS5.0 conversion -- added -AUTO subcommand
T. Mittan      Jan   1992	Added update option, and averaging option
				also, maskval can now be a range
T. Mittan      Mar   1992	Modified averaging option to choose which
				direction the average is performed 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   GETPAR must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Initialization 
Allocation of space for TAE names 
Get globals
Get first input image specifications
Get the other input image specifications
Get output image name
Get mask values
If the subcommand specified is -MANUAL
   Get output image size
   Get starting line locations
   Get starting sample locations
   Check for starting location outside of image boundary 
If the subcommand specified is -AUTO
   Retreive the ULCOORS and OUTSIZE parameter
   Calculate the SL, SS, NL, and NS or the output image
Return
ALGORITHM REFERENCES:     none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "concat.h"

#define NIMGS 50

void getpar
(
  struct PARBLK *vblk,        /* I: TAE parameter block */
  char *hostin,               /* O: Host file names */
  long *in_count,             /* O: Number of input images */
  long window[][4],           /* O: Input window specifications */
  long bands[][MAXBND+1],     /* O: Band specifications */
  long *num_out_bands,        /* O: Number of output bands */
  long nbands[],              /* O: Number of bands in each image */
  long numimg[],              /* O: Number of images in each IN element */
  long *index,                /* O: Total number of images specified */
  char *taeout,               /* O: Output TAE file name */
  char *hostout,              /* O: Output host file name */
  long *msgtim,               /* O: Time between display messages */
  char *minmax,               /* O: MinMax flag buffer */
  long sl[],                  /* O: Starting line locations */
  long ss[],                  /* O: Starting sample locations */
  long *nl,                   /* O: Number of lines */
  long *ns,                   /* O: Number of samples */
  char option[][7],           /* O: Option values */
  float maskval[][2],         /* O: Mask values */
  long *update,               /* O: Output image flag */
  long out_window[][4]        /* O: Output window spcification */
)
{
struct VARIABLE *var;		   /* pointer to a TAE variable               */

double  outsize[2];		   /* output image size			      */
double  ulcoors[2];		   /* upper left coordinates		      */

float   projtol;		   /* projection coordinate tolerance	      */
long    fatal = -1;		   /* fatal error flag			      */
long    i;			   /* loop index			      */
long    imglimit = MAXIMG;	   /* maximum number of images per IN         */
long    nonfatal = 1;		   /* nonfatal error flag		      */
long    outsize_flag;		   /* output size flag			      */
long    one = 1;
long    qflag[MAXIMG][2];	   /* flag indicating if subwindows specified */
long    same;			   /* flag if images have same window	      */
long	status;			   /* function return status code	      */
long tmp_images = 1;
long tmp_img_groups = 1;
long    totbnd;			   /* number of bands specified per IN        */
long    ulcoors_flag;		   /* upper left corrdinate flag	      */
long    zero = 0;		   /* zero				      */
char    *temp;

char    subcmd[7];		   /* subcommand buffer			      */
char    mask[NIMGS][10];	   /* mask values			      */
char    *taein;			   /* TAE file names			      */
char    *hostin_ptr;		   /* temporary HOST character pointer	      */

/*  Initialization 
------------------*/
hostin_ptr = hostin;

/* Allocation of space for TAE names 
------------------------------------*/
taein = (char *)malloc(MAXIMG * CMLEN);
if (taein == NULL)
   {
   c_errmsg("Error allocating dynamic memory","concat-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   }

/* get globals
--------------*/
var = p_find(vblk,"$msgtim");
*msgtim = IVAL(*var,0);


var = p_find(vblk,"$minmax");
strcpy(minmax,SVAL(*var,0));

/* get first input image specifications
---------------------------------------*/
status = c_getin(vblk,"IN",&zero,taein,hostin_ptr,&numimg[0],window[0],
		bands[0],&nbands[0],&totbnd,&imglimit,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","concat-fatal",&fatal);

if (same == 0)
   {
   c_errmsg("Images do not have same window size","concat-window",&nonfatal);
   c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   }

/* initialization 
-----------------*/
hostin_ptr += (numimg[0] * CMLEN);
*num_out_bands = totbnd;

/* process remaining IN entries
-------------------------------*/
*index = numimg[0];

for (i = 1; i < *in_count; i++)
    {
    status = c_getin(vblk,"IN",&i,taein,hostin_ptr,&numimg[i],
		   window[*index],bands[*index],&nbands[*index],&totbnd,
		   &imglimit,&same,qflag);
    if (status != E_SUCC)
       c_errmsg("Fatal error encountered","concat-fatal",&fatal);

    if (same == 0) 
       {
       c_errmsg("Images do not have same window size",
		"concat-window",&nonfatal);
       c_errmsg("Fatal error encountered","concat-fatal",&fatal);
       }

    if (*num_out_bands != totbnd)
       {
       c_errmsg("IN entries do not have the same number of bands",
	       "concat-nbands",&nonfatal);
       c_errmsg("Fatal error encountered","concat-fatal",&fatal);
       }

    *index += numimg[i];
    hostin_ptr += (numimg[i] * CMLEN);
    }

/*  Free TAE name space
-----------------------*/
free(taein);

/* get output image name
------------------------*/
var = p_find(vblk,"OUT");
if (var->v_count == 1)
  {
  *update = FALSE;
  status = c_getout(vblk,"OUT",taeout,hostout);
  if (status != E_SUCC)
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
  }
else
  *update = TRUE;

    

/* get mask values
------------------*/
var = p_find(vblk,"maskval");

if (*update && (var->v_count < *in_count))
  {
  if (var->v_count < (*in_count - 1))
     {
     c_errmsg("The number of values specified for IN and MASKVAL are not equal",
              "concat-unequal",&nonfatal);
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
      }

  for (i = 1; i <= var->v_count; i++)
    {
    strcpy(mask[i],SVAL(*var,i-1));
    temp = (char *)strchr(mask[i],' ');
    if ((temp == NULL) || (temp ==(&mask[i][0] + strlen(mask[i]) - 1)))
      {
      sscanf(mask[i],"%f",&maskval[i][0]);
      maskval[i][1] = maskval[i][0];
      }
    else
      sscanf(mask[i],"%f%f",&maskval[i][0],&maskval[i][1]);
    }
    maskval[0][0] = maskval[1][0];
    maskval[0][1] = maskval[1][1];
 }
else
  {
  if (var->v_count < *in_count )
     {
     c_errmsg("The number of values specified for IN and MASKVAL are not equal",
              "concat-unequal",&nonfatal);
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
      }
  for (i = 0; i < var->v_count; i++)
    {
    strcpy(mask[i],SVAL(*var,i));
    temp = (char *)strchr(mask[i],' ');
    if ((temp == NULL) || (temp ==(&mask[i][0] + strlen(mask[i]) - 1)))
      {
      sscanf(mask[i],"%f",&maskval[i][0]);
      maskval[i][1] = maskval[i][0];
      }
    else
      sscanf(mask[i],"%f%f",&maskval[i][0],&maskval[i][1]);
    }
  }

/* get option
-------------*/
var = p_find(vblk,"OVEROPT");

if (*update && (var->v_count < *in_count))
  {
  if (var->v_count < (*in_count - 1))
     {
     c_errmsg("The number of values specified for IN and OVEROPT are not equal",
              "concat-unequal",&nonfatal);
     c_errmsg("Fatal error encounterd","concat-fatal",&fatal);
     }

   for (i = 1; i <= var->v_count; i++)
       strcpy(option[i],SVAL(*var,i-1));
   }
else
   {
  if (var->v_count < *in_count )
     {
     c_errmsg("The number of values specified for IN and OVEROPT are not equal",
              "concat-unequal",&nonfatal);
     c_errmsg("Fatal error encounterd","concat-fatal",&fatal);
     }
   for (i = 0; i < var->v_count; i++)
       strcpy(option[i],SVAL(*var,i));
   }

var = p_find(vblk,"_subcmd");
strcpy(subcmd,SVAL(*var,0));

/*  Based upon the subcommand specified--1) retreive nl, ns, sl, ss for -MANUAL 
    2) retreive or calculate the ULCOORS and OUTSIZE parameters and then 
    calculate the nl, ns, sl, and ss for -AUTO
------------------------------------------------------------------------------*/
if (strcmp(subcmd,"MANUAL") == 0)
   {
   /* get output image size
   ------------------------*/
   var = p_find(vblk,"nl");
   if (var->v_count == 0)
      {
      if(!*update)
         {
         c_errmsg("NL and NS must be specified","concat-size",
                  &nonfatal);
         c_errmsg("Fatal error encountered","concat-fatal",&fatal);
         }
      else
         *nl = window[0][NL];
      }
   else 
      *nl = IVAL(*var,0);
  
   var = p_find(vblk,"ns");
   if (var->v_count == 0)
      {
      if(!*update)
         {
         c_errmsg("NL and NS must be specified","concat-size",
                   &nonfatal);
         c_errmsg("Fatal error encountered","concat-fatal",&fatal);
         }
      else
         *ns = window[0][NS];
      }
   else 
      *ns = IVAL(*var,0);

   /* get starting line locations
   ------------------------------*/
   var = p_find(vblk,"sl");
   
  if (*update && (var->v_count < *in_count))
     {
     if (var->v_count < (*in_count - 1))
        {
        c_errmsg("The number of values specified for IN and SL are not equal",
                "concat-unequal",&nonfatal);
        c_errmsg("Fatal error encountered","concat-fatal",&fatal);
        }
     for (i = 1; i <= var->v_count; i++)
        {
        sl[i] = IVAL(*var,i-1);
        out_window[i][0] = sl[i];
        }
     }
  else
     {
     /* the number of values for sl cannot be more than for in, otherwise
	it cannot be detected if a value was not specified
     ----------------------------------------------------*/
     if (var->v_count != *in_count)
        {
        c_errmsg("The number of values specified for IN and SL are not equal",
                "concat-unequal",&nonfatal);
        c_errmsg("Fatal error encountered","concat-fatal",&fatal);
        }
     for (i = 0; i < var->v_count; i++)
         {
         sl[i] = IVAL(*var,i);
         out_window[i][0] = sl[i];
	 }
     }
   
   /* get starting sample locations
   --------------------------------*/
   var = p_find(vblk,"ss");
   
  if (*update && (var->v_count < *in_count))
     {
     if (var->v_count < (*in_count - 1))
        {
        c_errmsg("The number of values specified for IN and SS are not equal",
                "concat-unequal",&nonfatal);
        c_errmsg("Fatal error encountered","concat-fatal",&fatal);
        }
   
     for (i = 1; i <= var->v_count; i++)
         {
         ss[i] = IVAL(*var,i-1);
         out_window[i][1] = ss[i];
         }
     }
  else
     {
     /* the number of values for ss cannot be more than for in, otherwise
        it cannot be detected if a value was not specified
     ----------------------------------------------------*/
     if (var->v_count != *in_count)
        {
        c_errmsg("The number of values specified for IN and SS are not equal",
                "concat-unequal",&nonfatal);
        c_errmsg("Fatal error encountered","concat-fatal",&fatal);
        }
     for (i = 0; i < var->v_count; i++)
         {
         ss[i] = IVAL(*var,i);
         out_window[i][1] = ss[i];
         }
     }
   
  /* check for starting location outside of image boundary 
  --------------------------------------------------------*/
  if (*update)
     {
     if (numimg[0] > 1)
       {
       c_errmsg("Do not plus first image in update mode","concat-fatal",&fatal);
       c_errmsg("Fatal error encountered","concat-fatal",&fatal);
       }
      if ((window[0][SS] == 1) && (window[0][NS]) < *ns)
        *ns = window[0][NS];
      if ((window[0][SL] == 1) && (window[0][NL]) < *nl)
        *nl = window[0][NL];
      *index = numimg[0];
      for (i = 1; i < *in_count; i++)
      {
      if (window[0][SS] > ss[i])
          {
          window[*index][SS] += window[0][SS] - ss[i];
          window[*index][NS] -= window[0][SS] - ss[i];
          ss[i] = window[0][SS];
          }
      if (window[0][SL] > sl[i])
          {
          window[*index][SL] += window[0][SL] - sl[i];
          window[*index][NL] -= window[0][SL] - sl[i];
          sl[i] = window[0][SL];
          }
       *index += numimg[i];
      }
     i = 1;
     }
  else 
         i = 0;
  *index = 0;
  for (; i < *in_count; i++)
       {
       if (sl[i] > *nl)
          {
          c_errmsg("Starting line location is outside output image",
   	          "concat-location",&nonfatal);
          c_errmsg("Fatal error encountered","concat-fatal",&fatal);
          }
   
       if (ss[i] > *ns)
          {
          c_errmsg("Starting sample location is outside output image",
                  "concat-location",&nonfatal);
          c_errmsg("Fatal error encountered","concat-fatal",&fatal);
          }
       if ((window[*index][NL] - sl[i]) > *nl)
	  out_window[i][NL] = *nl; 
       else
          out_window[i][NL] = window[*index][NL];
       if ((window[*index][NS] - ss[i]) > *ns)
	  out_window[i][NS] = *ns; 
       else
          out_window[i][NS] = window[*index][NS];
       *index += numimg[i];
       }
   }
else /*calculate the starting line and sample from DDR projection coordinates */
   {
   var = p_find(vblk,"$PROJTOL");
   projtol = RVAL(*var,0);

   var = p_find(vblk,"ULCOORS");
   if (var->v_count != 0)
      {
      sscanf(SVAL(*var,0),"%lf",&ulcoors[0]);
      sscanf(SVAL(*var,1),"%lf",&ulcoors[1]);
      ulcoors_flag = TRUE;
      }
   else
      ulcoors_flag = FALSE;

   var = p_find(vblk,"outsize");
   if (var->v_count != 0)
      {
      sscanf(SVAL(*var,0),"%lf",&outsize[0]);
      sscanf(SVAL(*var,1),"%lf",&outsize[1]);
      outsize_flag = TRUE;
      }
   else
      outsize_flag = FALSE;

if (*update)
   {
   if (numimg[0] > 1)
     {
     c_errmsg("Do not plus first image in update mode","concat-fatal",&fatal);
     c_errmsg("Fatal error encountered","concat-fatal",&fatal);
     }
   /*  Calculate the SL, SS, NL, and NS or the output image
   --------------------------------------------------------*/
   hostin_ptr = hostin;
   cal_lin_smp(hostin_ptr,&tmp_images,&tmp_img_groups,&one,window,&projtol,
		 &ulcoors_flag,&outsize_flag,ulcoors,outsize,sl,ss,nl,ns,
		 out_window);
   if ((window[0][NS]) < *ns)
      *ns = window[0][NS];
   if ((window[0][NL]) < *nl)
      *nl = window[0][NL];
   out_window[0][NL] = *nl;
   out_window[0][NS] = *ns;
   ulcoors_flag = TRUE;
   outsize_flag = TRUE;
   tmp_images = *in_count - 1;
   tmp_img_groups = *in_count - 1;
   hostin_ptr += (numimg[0] * CMLEN);
   cal_lin_smp(hostin_ptr, &tmp_images, &tmp_img_groups, &numimg[1],
               window[1], &projtol, &ulcoors_flag, &outsize_flag,
               ulcoors, outsize, &sl[1], &ss[1], nl, ns,out_window[1]);
   *index = numimg[0];
   /*(*nl)++;
   (*ns)++;*/
   for (i = 1; i < *in_count; i++)
     {
     if (window[0][SS] > 1)
          {
          ss[i] += window[0][SS] - 1;
          out_window[i][SS] = ss[i];
          }
     if (window[0][SL] > 1)
          {
          sl[i] += window[0][SL] - 1;
          out_window[i][SL] = sl[i];
          }
     if ((window[*index][NL] + sl[i]) > *nl)
          out_window[i][NL] = *nl - sl[i]; 
     else
          out_window[i][NL] = window[*index][NL];
     if ((window[*index][NS] + ss[i]) > *ns)
          out_window[i][NS] = *ns - ss[i]; 
     else
          out_window[i][NS] = window[*index][NS];
     *index += numimg[i];
     }
   }
 else
   {
   cal_lin_smp(hostin,index,in_count,numimg,window,&projtol,&ulcoors_flag,
	 	&outsize_flag,ulcoors,outsize,sl,ss,nl,ns,out_window);
   *index = 0;
   for ( i = 0; i < *in_count; i++)
      {
      if ((window[*index][NL] - sl[i]) > *nl)
           out_window[i][NL] = *nl; 
      else
           out_window[i][NL] = window[*index][NL];
      if ((window[*index][NS] - ss[i]) > *ns)
           out_window[i][NS] = *ns; 
      else
           out_window[i][NS] = window[*index][NS];
      *index += numimg[i];
      }
   }
 }/*  subcmd == AUTO  */
}
