/*******************************************************************************
NAME:      		   	MNMX

PURPOSE:   	To calculate the min and max values of the output image

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey      Aug   1984	Original development
D. Akkerman    Aug   1987       PR #4247 modified to allow for single band 
				 specification of a multi-band images
K. Zanter      Sept. l987       NEWLAS (conversion to 'C')
B. Ailts       July  1988       LAS5.0 conversion -- added -AUTO subcommand
P. Pease/GSFC  Nov.  1989       change call to c_startm from startm
D. Etrheim     Apr   1993       Correct problem in update mode of deleting the
                                first input image.  Free allocated space.
D. Eichacker   May.  1994       added opt[].
D. Eichacker   Apr.  1995       Added datasize[] 


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   MNMX must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Initialization 
Initialize min and max arrays
Open image for reading
Allocate space for image I/O buffer
Set up file group
Set up and start timed processing message
For each line
   For each band
      Read image 
      Calculate the min and max values 
Close image
Free buffer
Return
ALGORITHM REFERENCES:     none
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "concat.h"

void mnmx
(
  struct PARBLK *vblk,     /* I: TAE parameter block */
  char *hostout,           /* I: Output host file name */
  long *nl,                /* I: Number of lines */
  long *ns,                /* I: Number of samples */
  long *dtype,             /* I: Data type of output image */
  long *totbnd,            /* I: Number of bands in output image */
  double bmin[],           /* O: BYTE minimum intensity for each band */
  double bmax[],           /* O: BYTE maximum intensity for each band */
  long *msgtim,            /* I: Time between display messages */
  long *update             /* I: Update flag */
)
{
extern  long curent;	  	 /* current line number for display messages  */
extern  long fdesc[MAXIMG];	 /* image file descriptors	              */
extern  long gdesc;		 /* group descriptor 	       		      */
extern  long offset2[MAXBND+1];	 /* offsets into thebuf for output	      */

static	long	opt[3]={0,0,0};	 /* optional parameter array for EOPENS	      */
static  long	datasize[NDTYPES+1] = {0,1,2,4,4,8,1,2,4};
				 /* offset muliplier for the output buffer    */

long	access = IREAD;		 /* image I/O access type		      */
long	band;			 /* band counter			      */
long	bufsiz;			 /* size of image I/O buffer 		      */
long    fatal;			 /* fatal error code			      */
long	i;			 /* loop index				      */
long	msg_flag;		 /* flag for processing message		      */
long    nonfatal = 1;		 /* nonfatal error message		      */
long	one = 1;		 /* value of one			      */
long	status;			 /* function return status code		      */
long    temp_bands[MAXBND+1];	 /* temporary bands array		      */
long	zero = 0;		 /* value of zero			      */

char    *thebuf;	 	 /* pointer to image I/O buffer		      */
char    *c_ealloc();	 	 /* declaration of function c_ealloc	      */

/* initialize min and max arrays
--------------------------------*/
status = c_inmnmx(dtype,totbnd,bmin,bmax);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","concat-fatal",&fatal);

/* open image for reading
-------------------------*/
for (i = 0; i < *totbnd; i++)
    temp_bands[i] = i + 1;
temp_bands[*totbnd] = 0;
status = c_eopens(&fdesc[0],hostout,temp_bands,&one,&one,nl,ns,offset2,
   	        &access,dtype,&zero,opt);
if (status != E_SUCC)
   {
   if (*update)
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   else
      c_clean(vblk,"concat",hostout,&one);
   }

fdesc[1] = 0;

/* allocate space for image I/O buffer
--------------------------------------*/
if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","concat-alloc",&nonfatal);
   if (c_eclose(fdesc) != E_SUCC)
      {
      c_errmsg("Error closing file(s)","concat-close",&nonfatal);
      c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
      }
   if (*update)
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   else
      c_clean(vblk,"concat",hostout,&one);
   }
   
/* set up file group
--------------------*/
status = c_egroup(fdesc,&gdesc,thebuf,&bufsiz);
if (status != E_SUCC)
   {
   if (c_eclose(fdesc) != E_SUCC)
      {
      c_errmsg("Error closing file","concat-close",&nonfatal);
      c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
      }
   if (*update)
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   else
      c_clean(vblk,"concat",hostout,&one);
   }

/* set up and start timed processing message
--------------------------------------------*/
msg_flag = ALLCAT;
c_inittm("CONCAT","Processed minmax for","line","bands","",nl,totbnd,msgtim,
	&msg_flag);
curent = 1;
c_startm();

/* read image to determine min and max values 
---------------------------------------------*/
for (curent = 0; curent < *nl; curent++)
    {
    if (c_estep(&gdesc) != E_SUCC)
       { 
       if (c_egclse(&gdesc) != E_SUCC)
         {
         c_errmsg("Error closing file","concat-close",&nonfatal);
         c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
         }
       if (*update)
          c_errmsg("Fatal error encountered","concat-fatal",&fatal);
       else
          c_clean(vblk,"concat",hostout,&one);
       }
    for (band = 0; band < *totbnd; band++)
	c_pxmnmx(thebuf + (offset2[band] * datasize[*dtype]), ns, 
		&bmin[band], &bmax[band], dtype);
    }

/* turn off processing message
------------------------------*/
c_stoptm();

/* close image
--------------*/
if (c_egclse(&gdesc) != E_SUCC)
   {
   c_errmsg("Error closing file","concat-close",&nonfatal);
   c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
   }

/* free buffer
--------------*/
free(thebuf);
}
