/*******************************************************************************
NAME:      	 OVERLAY_IMG

PURPOSE:   	To overlay the input image into the output image

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey      Aug   1984	Original development
D. Akkerman    Aug   1987       PR #4247 modified to allow for single band 
				 specification of a multi-band images
K. Zanter      Sept. l987       NEWLAS (conversion to 'C')
B. Ailts       July  1988       LAS5.0 conversion -- added -AUTO subcommand
T. Mittan      Jan   1992	Added update option, and averaging option.
				also, maskval can be a range
T. Mittan      Mar   1992	Modified averaging option to choose which
				direction the averaging should be performed
D. Etrheim     Apr   1993       Correct problem in update mode of deleting the
                                first input image.  Free allocated space.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   OVERLAY must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Initialization 
Ensure that window specification does not exceed the image bounds
Open image for reading
Allocate space for image I/O buffer
Set up file group
Do initial read
Set up and start timed processing message
Switch on dtype -- EBYTE, EWORD, ELONG, or EREAL
    Type cast to correct pointer
    For each line
       For each band
          For each sample
             If input image sample does not eqaul mask
	        If replace or (leave and output image sample equals mask)
		   Output sample equals input sample
       Read/write each image 
    Break for switch statement
Close images
Free buffer
Return

ALGORITHM REFERENCES:     none
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "concat.h"

int overlay_img
(
  struct PARBLK *vblk,        /* I: TAE parameter block */
  char *hostin,               /* I: Input host file name */
  long nmrimg,                /* I: Number of images in current spec */
  long wind[][4],             /* I/O: Window specification */
  long bands[][MAXBND+1],     /* I: Band specification */
  long nbands[],              /* I: Number of bands in each image */
  long *sl,                   /* I: Starting line */
  long *ss,                   /* I: Starting sample */
  float *mask,                /* I: Mask value */
  char *hostout,              /* I: Output host file name */
  long *dtype,                /* I: Data type of output image */
  long *totbnd,               /* I: Number of bands in output image */
  long *nl,                   /* I: Number of lines in output image */
  long *ns,                   /* I: Number of samples in output image */
  long replace,               /* I: Replace option for overlap condition */
  long leave,                 /* I: Leave option for overlap condition */
  long aver,                  /* I: Averag option for overlap condition */
  long *msgtim,               /* I: Time between messages */
  long *update,               /* I: Update flag */
  long *laver                 /* I: Left average array */
)
{
float   *rthebuf;		 /* float pointer to thebuf		      */

long	access;			 /* image I/O access type		      */
long	band;			 /* loop index counting the bands	      */
long    bufsiz;			 /* size of image I/O buffer		      */
long    distance;                /* number of overlap pixels on a certain line*/
extern  long	curent;		 /* current line number for display message   */
extern struct FDESC *fdesc[MAXIMG]; /* image file descriptors	              */
extern struct GDESC *gdesc;	 /* group descriptor 	                      */
long	msg_flag;		 /* flag for processing message		      */
long   	i;			 /* loop index				      */
long    index;			 /* index				      */
long    lmask;			 /* long mask value		      	      */
long    lmask1;			 /* long mask value		      	      */
long    *lthebuf;		 /* long pointer to thebuf		      */
long    *ltemp;
long    *ltemp1;
long    nl1;			 /* true number of lines		      */
long    nonfatal = 1;		 /* nonfatal error code			      */
long    ns1;			 /* true number of samples		      */
extern  long offset1[MAXIMG * MAXBND + 1];/* offsets into thebuf for input    */
extern  long offset2[MAXBND+1];	 /* offsets into thebuf for output            */
long	one = 1;		 /* value of one			      */
long	samp;			 /* loop index counting samples		      */
long	sl1;			 /* true starting line			      */
long	ss1;			 /* true starting sample		      */
long	status;			 /* function return status code		      */
long    temp_bands[MAXBND+1];	 /* temporary bands array for (update) eopens */
long	temp_val;		 /* temporary value			      */
long	zero = 0;		 /* value of zero			      */
long    opt[3];
long    fatal = -1;

float   mfactor;		 /* multiplication factor		      */
float   rate;			 /* rate of change per pixel		      */
float   *ftemp;
float   *ftemp1;

short   wmask;			 /* short mask value			      */
short   wmask1;			 /* short mask value			      */
short	*wtemp;
short	*wtemp1;
short   *wthebuf;		 /* short pointer to thebuf		      */

char *hname_ptr;		 /* pointer to current host name	      */
unsigned char  bmask;		 /* BYTE mask value			      */
unsigned char  bmask1;		 /* BYTE mask value			      */
unsigned char  *btemp;		 /* BYTE mask value			      */
unsigned char  *btemp1;		 /* BYTE mask value			      */
unsigned char  *thebuf;		 /* pointer to image I/O buffer		      */

/* Initialization
-----------------*/
hname_ptr = hostin;

/* ensure that window specification does not exceed the image bounds
--------------------------------------------------------------------*/
for (i = 0; i < nmrimg; i++)
    {
if (!*update)
    {
    temp_val = *nl - *sl + 1;
    wind[i][NL] = wind[i][NL] < temp_val ? wind[i][NL] : temp_val;
    temp_val = *ns - *ss + 1;
    wind[i][NS] = wind[i][NS] < temp_val ? wind[i][NS] : temp_val;
    } 
else /* if in update mode */
    {
    temp_val = *nl - *sl + 1;
    wind[i][NL] = wind[i][NL] < temp_val ? wind[i][NL] : temp_val;
    temp_val = *ns - *ss + 1;
    wind[i][NS] = wind[i][NS] < temp_val ? wind[i][NS] : temp_val;
    /*if ((wind[i][SL] + wind[i][NL]) < (*nl + *sl))
        wind[i][NL] = wind[i][NL] - *sl + 1;
    if ((wind[i][SS] + wind[i][NS]) < (*ns + *ss))
        wind[i][NS] = wind[i][NS] - *ss + 1;*/
/*
    wind[i][NS] = abs(wind[i][NS] - wind[i][SS] + *ss) < 
    abs(temp_val) ? wind[i][NL] : temp_val;
    temp_val = *nl - *sl + 1;
    wind[i][NL] = wind[i][NL] < temp_val ? wind[i][NL] : *nl;
    temp_val = *ns + *ss;
    wind[i][NS] = wind[i][NS] < temp_val ? wind[i][NS] : *ns;
*/
    }
    }

/*  Check for negative offsets
------------------------------*/
if (*sl <= zero) 
   {
   sl1 = 1;
   nl1 = wind[0][NL] - abs(*sl) - 1;
   }
else
   {
   sl1 = *sl;
   nl1 = wind[0][NL];
   }
if (nl1 <= 0)
   {
   c_errmsg("Offsets outside output image bounds","informational",&nonfatal);
   return(E_SUCC);
   }

if (*ss <= zero)
   {
   ss1 = 1;
   ns1 = wind[0][NS] - abs(*ss) - 1;
   }
else
   {
   ss1 = *ss;
   ns1 = wind[0][NS];
   }
if (ns1 <= 0)
   {
   c_errmsg("Offsets outside output image bounds","informational",&nonfatal);
   return(E_SUCC);
   }

/* open output "padded" image for update
----------------------------------------*/
opt[0] = opt[2] = 0;
opt [1] = NO_COMPRES;
access = IUPDATE;
for (i = 0; i < *totbnd; i++)
    temp_bands[i] = i+1;
temp_bands[*totbnd] = 0;
   status = c_eopens(&fdesc[0],hostout,temp_bands,&sl1,&ss1,&nl1,
   	          &ns1,offset2,&access,dtype,&zero,opt);
   if (status != E_SUCC)
      {
      if (*update)
         c_errmsg("Fatal error encountered","concat-fatal",&fatal);
      else
         c_clean(vblk,"concat",hostout,&one);
      }

/* open bands of "inset" image for reading
------------------------------------------*/
access = IREAD;
index = 0;
for (i = 0; i < nmrimg; i++)
   {
   /*  Check for negative offsets
   ------------------------------*/
   if (*sl <= zero) 
      {
      sl1 = abs(*sl) + wind[i][SL] + 1;
      nl1 = wind[i][NL] - abs(*sl) - 1;
      }
   else
      {
      sl1 = wind[i][SL];
      nl1 = wind[i][NL];
      }
   if (nl1 <= 0)
	continue;

   if (*ss <= zero)
      {
      ss1 = abs(*ss) + wind[i][SS] + 1;
      ns1 = wind[0][NS] - abs(*ss) - 1;
      }
   else
      {
      ss1 = wind[i][SS];
      ns1 = wind[i][NS];
      }
   if (ns1 <= 0)
	continue;

   opt[0]= opt[1]= opt[2] = 0;
   status = c_eopens(&fdesc[i+1],hname_ptr,bands[i],&sl1,&ss1,&nl1,&ns1,
		    &offset1[index],&access,dtype,&zero,opt);
   if (status != E_SUCC)
      {
      if (*update)
         c_errmsg("Fatal error encountered","concat-fatal",&fatal);
      else
         c_clean(vblk,"concat",hostout,&one);
      }

   hname_ptr += CMLEN;
   index += nbands[i];
   }

fdesc[nmrimg+1] = 0;

/* allocate space for image I/O buffer
--------------------------------------*/
if ((thebuf = (unsigned char *) c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","concat-alloc",&nonfatal);
   if (c_eclose(fdesc) != E_SUCC)
      {
      c_errmsg("Error closing file","concat-close",&nonfatal);
      c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
      }
   if (*update)
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   else
      c_clean(vblk,"concat",hostout,&one);
   }

/* set up file group
--------------------*/
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_eclose(fdesc) != E_SUCC)
      {
      c_errmsg("Error closing file","concat-close",&nonfatal);
      c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
      }
   if (*update)
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   else
      c_clean(vblk,"concat",hostout,&one);
   }
/* initial estep
----------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
      {
      c_errmsg("Error closing file","concat-close",&nonfatal);
      c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
      }
   if (*update)
      c_errmsg("Fatal error encountered","concat-fatal",&fatal);
   else
      c_clean(vblk,"concat",hostout,&one);
   }

/* set up and start timed processing message
--------------------------------------------*/
msg_flag = ALLCAT;
c_inittm("CONCAT","Processed","line","bands","",&wind[0][NL],totbnd,
         msgtim,&msg_flag);
curent = 1;
c_startm();

/* overlay "inset" image into "padded" image.  Based on the data type execute 
   the correct processing loop.
-----------------------------------------------------------------------------*/
switch (*dtype)
   {
   case EBYTE:
      bmask  = (unsigned char )mask[0];
      bmask1 = (unsigned char )mask[1];
      for (curent = 0; curent < nl1; curent++)
         {
         for (band = 0; band < *totbnd; band++)
          {
          btemp  = (thebuf + *(offset1 + band));
          btemp1 = (thebuf + *(offset2 + band));
	  if (replace) 
            {
            for (samp = 0; samp < ns1; samp++, btemp++, btemp1++)
               if ((*btemp < bmask) || (*btemp > bmask1))
                  *btemp1 = *btemp;
            }
          else
          if (leave) 
            for (samp = 0; samp < ns1; samp++,btemp++,btemp1++)
               {
               if ((*btemp  <  bmask) || (*btemp  >  bmask1))
                  if ((*btemp1 >= bmask) && (*btemp1 <= bmask1))
	               *btemp1 = *btemp;
	        }  
          else
	    { /* if (aver) */
            /* calculate the overlap distance
            -------------------------------*/
            for (distance = 1,samp = 0; samp < ns1; samp++, btemp++, btemp1++)
              {
              if ((( *(btemp)  < bmask)   || 
                   ( *(btemp)  > bmask1)) &&
                  (( *(btemp1) < bmask)   || 
                   ( *(btemp1) > bmask1)))
	      distance += 1;
	      }
            btemp  = (thebuf + *(offset1 + band));
            btemp1 = (thebuf + *(offset2 + band));
	    rate = 1.0 / distance;
	    mfactor = 1.0;
            for (samp = 0; samp < ns1; samp++, btemp++, btemp1++)
               {
               if ((*btemp < bmask) || (*btemp > bmask1))
                  {
	          if ((*btemp1 >= bmask) &&
	             ( *btemp1 <= bmask1))
	               *btemp1  = *btemp;
	          else
	             {
	             mfactor -= rate;
                     if (laver[curent])
                       {
	               *btemp1  = ((1.0-mfactor) * (*btemp1)) + 
	               (mfactor * (*btemp));
                       }
                     else
                       {
	               *btemp1  = (mfactor * (*btemp1)) + 
	               ((1.0-mfactor) * (*btemp));
                       }
                     }
                  }
	        }
	      }
             }
	    if (c_estep(&gdesc) != E_SUCC)
              {
              if (c_egclse(&gdesc) != E_SUCC)
                {
                c_errmsg("Error closing file","concat-close",&nonfatal);
                c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
                }
              if (*update)
                 c_errmsg("Fatal error encountered","concat-fatal",&fatal);
              else
                 c_clean(vblk,"concat",hostout,&one);
              }
         }  /* for curent < nl1  */
      break;

   case EWORD:
      wmask = (short ) mask[0];
      wmask1= (short ) mask[1];
      wthebuf = (short *)thebuf;
      for (curent = 0; curent < nl1; curent++)
         {
         for (band = 0; band < *totbnd; band++)
            {
/*
	calculate the overlap distance if averaging
-------------------------------------------------------*/
          wtemp  = (wthebuf + *(offset1 + band));
          wtemp1 = (wthebuf + *(offset2 + band));
	  if (aver)
	  {
	  distance = 1;
          for (samp = 0; samp < ns1; samp++)
            {
            if (((*(wtemp+samp)  < wmask) || 
                ( *(wtemp+samp)  > wmask1)) &&
                ((*(wtemp1+samp) < wmask) || 
                ( *(wtemp1+samp) > wmask1)))
	    distance += 1;
	    }
	  rate = 1.0 / distance;
	  mfactor = 1.0;
	  }
            for (samp = 0; samp < ns1; samp++)
               {
               if ((*(wtemp+samp) < wmask) || 
                  ( *(wtemp+samp) > wmask1))
                  {
	          if ((replace) ||
	              ((leave) && ((*(wtemp1+samp) >= wmask)
                      && (*(wtemp1+samp) <= wmask1))))
	                  *(wtemp1+samp) = *(wtemp+samp);
		  else
	          if (aver)
		     {
	              if ((*(wtemp1+samp) >= wmask) &&
	                  (*(wtemp1+samp) <= wmask1))
	                 *(wtemp1+samp)  = *(wtemp+samp);
		      else
			{
			mfactor -= rate;
                        if (laver[curent])
	                    *(wtemp1+samp)  = ((1.0-mfactor) * (*(wtemp1+samp)))
			    + (mfactor * (*(wtemp+samp)));
                        else
	                    *(wtemp1+samp)  = (mfactor * (*(wtemp1+samp)))
			    + ((1.0 - mfactor) * (*(wtemp+samp)));
                        }
		     }
	          }
	       }  /* for samp < ns1  */
            }
	    if (c_estep(&gdesc) != E_SUCC)
              {
              if (c_egclse(&gdesc) != E_SUCC)
                {
                c_errmsg("Error closing file","concat-close",&nonfatal);
                c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
                }
              if (*update)
                 c_errmsg("Fatal error encountered","concat-fatal",&fatal);
              else
                 c_clean(vblk,"concat",hostout,&one);
              } 
         }  /* for curent < nl1  */
      break;

   case ELONG:
      lmask = (long ) mask[0];
      lmask1= (long ) mask[1];
      lthebuf = (long *)thebuf;
      for (curent = 0; curent < nl1; curent++)
         {
         for (band = 0; band < *totbnd; band++)
            {
/*
	calculate the overlap distance if averaging
-------------------------------------------------------*/
          ltemp  = (lthebuf + *(offset1 + band));
          ltemp1 = (lthebuf + *(offset2 + band));
	  if (aver)
	  {
	  distance = 1;
          for (samp = 0; samp < ns1; samp++)
            {
            if (((*(ltemp+samp)  < lmask) || 
                ( *(ltemp+samp)  > lmask1)) &&
                ((*(ltemp1+samp) < lmask) || 
                ( *(ltemp1+samp) > lmask1)))
	    distance += 1;
	    }
	  rate = 1.0 / distance;
	  mfactor = 1.0;
	  }
            for (samp = 0; samp < ns1; samp++)
               {
               if (( *(ltemp+samp) < lmask) || 
                  (  *(ltemp+samp) > lmask1))
                  {
	          if ((replace) ||
	              ((leave) && ((*(ltemp1+samp) >= lmask)
                      && (*(ltemp1+samp) <= lmask1))))
	                 *(ltemp1+samp) = *(ltemp+samp);
		  else
	          if (aver)
		     {
	              if ((*(ltemp1+samp) >= lmask) &&
	                  (*(ltemp1+samp) <= lmask1))
	                 *(ltemp1+samp)  = *(ltemp+samp);
		      else
			{
			mfactor -= rate;
                        if (laver[curent])
	                    *(ltemp1+samp)  = ((1.0-mfactor) * (*(ltemp1+samp)))
			    + (mfactor * (*(ltemp+samp)));
                        else
	                    *(ltemp1+samp)  = (mfactor * (*(ltemp1+samp)))
			    + ((1.0 - mfactor) * (*(ltemp+samp)));
                        }
		     }
	          }
	       }  /* for samp < ns1  */
	    }
	    if (c_estep(&gdesc) != E_SUCC)
              {
              if (c_egclse(&gdesc) != E_SUCC)
                {
                c_errmsg("Error closing file","concat-close",&nonfatal);
                c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
                }
              if (*update)
                 c_errmsg("Fatal error encountered","concat-fatal",&fatal);
              else
                 c_clean(vblk,"concat",hostout,&one);
              } 
         }  /* for curent < nl1  */
      break;

   case EREAL:
      rthebuf = (float *)thebuf;
      for (curent = 0; curent < nl1; curent++)
         {
         for (band = 0; band < *totbnd; band++)
            {
/*
	calculate the overlap distance if averaging
-------------------------------------------------------*/
          ftemp  = (rthebuf + *(offset1 + band));
          ftemp1 = (rthebuf + *(offset2 + band));
	  if (aver)
	  {
	  distance = 1;
          for (samp = 0; samp < ns1; samp++)
            {
            if (((*(ftemp+samp)  < mask[0]) || 
                ( *(ftemp+samp)  > mask[1])) &&
                ((*(ftemp1+samp) < mask[0]) || 
                ( *(ftemp1+samp) > mask[1])))
	    distance += 1;
	    }
	  rate = 1.0 / distance;
	  mfactor = 1.0;
	  }
            for (samp = 0; samp < ns1; samp++)
               {
               if (( *(ftemp+samp) < mask[0]) || 
                  (  *(ftemp+samp) > mask[1]))
                  {
	          if ((replace) ||
	              ((leave) && ((*(ftemp1+samp) >= mask[0])
                      && (*(ftemp1+samp) <= mask[1]))))
	                 *(ftemp1+samp) = *(ftemp+samp);
		  else
	          if (aver)
		     {
	              if ((*(ftemp1+samp) >= mask[0]) &&
	                  (*(ftemp1+samp) <= mask[1]))
	                 *(ftemp1+samp)  = *(ftemp+samp);
		      else
			{
			mfactor -= rate;
			if (laver[curent])
	                    *(ftemp1+samp)  = ((1.0-mfactor) * (*(ftemp1+samp)))
			    + (mfactor * (*(ftemp+samp)));
                        else
	                    *(ftemp1+samp)  = (mfactor * (*(ftemp1+samp)))
			    + ((1.0 - mfactor) * (*(ftemp+samp)));
                        }
		     }
	          }
	       }  /* for samp < ns1  */
	    }
	    if (c_estep(&gdesc) != E_SUCC)
              {
              if (c_egclse(&gdesc) != E_SUCC)
                {
                c_errmsg("Error closing file","concat-close",&nonfatal);
                c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
                }
              if (*update)
                 c_errmsg("Fatal error encountered","concat-fatal",&fatal);
              else
                 c_clean(vblk,"concat",hostout,&one);
              } 
         }  /* for curent < nl1  */
      break;
   }  /* switch datatype  */

/* turn off processing message
------------------------------*/
c_stoptm();

/* close images
---------------*/
if (c_egclse(&gdesc) != E_SUCC)
   {
   c_errmsg("Error closing file","concat-close",&nonfatal);
   c_errmsg("Nonfatal error encountered","concat-warn",&nonfatal);
   }

/* free buffer
--------------*/
free(thebuf);

return(E_SUCC);
}
