/*******************************************************************************
NAME: PAD

PURPOSE: To pad the output image with the fill value

PROGRAM HISTORY:
PROGRAMMER     DATE           REASON
----------     ----           ------
B. Wilsey      08/84          Original development
D. Akkerman    08/87          PR #4247 modified to allow for single band 
                                specification of a multi-band images
K. Zanter      09/87          NEWLAS (conversion to 'C')
B. Ailts       07/88          LAS5.0 conversion -- added -AUTO subcommand
T. Mittan      01/92          Improved performance by only pixcopying to a 
                                buffer once

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PAD must be run under TAE.

PROJECT: LAS		                

ALGORITHM DESCRIPTION:
Initialization 
Switch on dtype -- EBYTE, EWORD, ELONG, or EREAL
   Type cast mask value to a temporary pointer
   Set correct data type size
Open image for writing
Allocate space for image I/O buffer
Set up file group
Do initial read
Set up and start timed processing message
    For each line
       For each band
          Fill buffer with mask value
       Write each image 
Close images
Free buffer

ALGORITHM REFERENCES: None
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "concat.h"

void pad
(
  struct PARBLK *vblk,      /* I: TAE parameter block */
  char *hostout,            /* I: Output host file name */
  long *nl,                 /* I: Number of lines in output image */
  long *ns,                 /* I: Number of samples in output image */
  long *totbnd,             /* I: Number of bands in output image */
  long *dtype,              /* I: Data type of output image */
  float *mask,              /* I: Mask value */
  long *msgtim              /* I: Display message delay */
)
{
  extern long curent;           /* Current line */
  extern struct FDESC *fdesc[MAXIMG];    /* Image file descriptors */
  float	tmask;                  /* Tempory buffer to mask value */

  long access = IWRITE;         /* Image I/O access type */
  long band;                    /* Band counter */
  long *lmask;                  /* LONG mask value */
  long msg_flag;                /* Message type flag */
  long one = 1;                 /* Value of one */
  long size;                    /* Number of bytes per sample */
  long status;                  /* Function return status code */
  long opt[3];

  short	*smask;                 /* SHORT mask value */

  unsigned char	*bmask;         /* BYTE mask value */
  char *thebuf;                 /* Pointer to image I/O buffer */

  /* Initialize the pointer to the temporary mask value
  -----------------------------------------------------*/
  bmask = (unsigned char *)&tmask;
  smask = (short *)&tmask;
  lmask = (long *)&tmask;

  /* Setup based on data type 
  ---------------------------*/
  switch (*dtype)
  {
    case EBYTE:
      *bmask = (unsigned char) *mask;
      size = 1;
      break;
    case EWORD:
      *smask = (short) *mask;
      size = 2;
      break;
    case ELONG:
      *lmask = (long) *mask;
      size = 4;
      break;
    case EREAL:
      size = 4;
      tmask = *mask; 
      break;
   }

  /* Allocate space for image I/O buffer
  --------------------------------------*/
  thebuf = (char *)calloc(*ns,(size * sizeof(char)));
  if ( thebuf == NULL)
  {
    c_lasmsg(FALSE,"concat-alloc","Error allocating dynamic memory");
    c_clean(vblk,"concat",hostout,&one);
  }

  /* Open output image to full size
  ---------------------------------*/
  opt[0] = opt[2] = 0;
  opt [1] = NO_COMPRES;
  fdesc[1] = 0;
  status = c_eopenr(&fdesc[0],hostout,totbnd,&one,&one,nl,ns,
                    &access,dtype,opt,&one);
  if (status != E_SUCC)
    c_clean(vblk,"concat",hostout,&one);

  /* Fill the buffer with the proper value
  --------------------------------------*/
  c_pxfill(thebuf,dtype,ns,&tmask);

  /* Set up and start timed processing message
  --------------------------------------------*/
  msg_flag = ALLCAT;
  c_inittm("CONCAT","Padded","line","bands","",nl,totbnd,msgtim,&msg_flag);
  curent = 1;
  c_startm();

  /* Process padded image 
  -----------------------*/
  for (curent = 1; curent <= *nl; curent++)
    for (band = 1; band <= *totbnd; band++)
      if (c_ewrite(&fdesc[0],&band,&curent,thebuf,&one) != *ns)
      {
        c_eclose(&fdesc[0]);
        c_clean(vblk,"concat",hostout,&one);
      }

  /* Turn off processing message
  ------------------------------*/
  c_stoptm();

  /* Close image
  --------------*/
  if (c_eclose(&fdesc[0]) != E_SUCC)
  {
    c_lasmsg(FALSE,"concat-close","Error closing file");
    c_lasmsg(FALSE,"concat-warn","Nonfatal error encountered");
    c_clean(vblk,"concat",hostout,&one);
  }

  /* Free buffer
  --------------*/
  free(thebuf);
}
