/*******************************************************************************
NAME: SET_DTYPE

PURPOSE: Sets the data type of the output image by determining the 
           maximum data type of the input images.

PROGRAM HISTORY:
PROGRAMMER      DATE          REASON
----------      ----          ------
B. Wilsey       08/84         Original development
D. Akkerman     08/87         PR #4247 modified to allow for single band 
                                specification of a multi-band images
K. Zanter       09/87         NEWLAS (conversion to 'C')
B. Ailts        07/88         LAS5.0 conversion -- added -AUTO subcommand
B. Ailts        11/89         Shortened name to rn on IBM-RT

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   SET_DTYPE must be run under TAE.

PROJECT: LAS		                

ALGORITHM DESCRIPTION:
Initialization 
For every image
   Retreive the data type
   If current data type is greater than output data type
      Output data type equals current data type

Switch on dtype -- EBYTE, EWORD, ELONG, or EREAL
   For each mask value
     Check to see if the mask value is within the range of the data type

ALGORITHM REFERENCES: None
*******************************************************************************/
#include "las.h"
#include "pixman.h"
#include "typlim.h"
#include "concat.h"

void set_dtype
(
  char *hostin,             /* I: Input host file name */
  long image_count,         /* I: Number of input images */
  long *out_dtype,          /* O: Data type of output image */
  float maskval[][2],       /* I: Mask value */
  long *in_count            /* I: Number of mask values */
)
{
  long i;                   /* Loop counter */
  long one = 1;	            /* Value of one */
  long status;              /* Function return status code */
  long type;                /* Data type of current image */

  char *temp;               /* Temporary descriptor */

  /* Initialization of variables
  ------------------------------*/
  *out_dtype = 0;
  temp = hostin;

  for (i = 0; i < image_count; i++)
  {
    status = c_gettyp(temp,&one,&type);
    if (status != E_SUCC)
    {
      c_lasmsg(FALSE,"concat-ddr","Error reading DDR file");
      c_lasmsg(TRUE,"concat-fatal","Fatal error encountered");
    }

    if (type > *out_dtype)
      *out_dtype = type;

    temp += CMLEN;
  }

  /* Check the range of mask values based on data type 
  ----------------------------------------------------*/
  switch (*out_dtype)
  {
    case EBYTE:
      for (i = 0; i < *in_count; i++)
      {
        if ((maskval[i][0] < MINUTINY) || (maskval[i][0] > MAXUTINY) ||
            (maskval[i][1] < MINUTINY) || (maskval[i][1] > MAXUTINY))
        {
          c_lasmsg(FALSE,"concat-maskval",
                   "Mask value is not in range of output image");
          c_lasmsg(TRUE,"concat-fatal","Fatal error encountered");
        }
      }
      break;

    case EWORD:
      for (i = 0; i < *in_count; i++)
      {
        if ((maskval[i][0] < MINWORD) || (maskval[i][0] > MAXWORD) ||
            (maskval[i][1] < MINWORD) || (maskval[i][1] > MAXWORD))
        {
          c_lasmsg(FALSE,"concat-maskval",
                   "Mask value is not in range of output image");
          c_lasmsg(TRUE,"concat-fatal","Fatal error encountered");
        }
      }
      break;

    case ELONG:
      for (i = 0; i < *in_count; i++)
      {
        if ((maskval[i][0] < MINLONG) || (maskval[i][0] > MAXLONG) ||
            (maskval[i][1] < MINLONG) || (maskval[i][1] > MAXLONG))
        {
          c_lasmsg(FALSE,"concat-maskval",
                   "Mask value is not in range of output image");
          c_lasmsg(TRUE,"concat-fatal","Fatal error encountered");
        }
      }
      break;

    case EREAL:
      for (i = 0; i < *in_count; i++)
      {
        if ((maskval[i][0] < MINREAL) || (maskval[i][0] > MAXREAL) ||
            (maskval[i][1] < MINREAL) || (maskval[i][1] > MAXREAL))
        {
        c_lasmsg(FALSE,"concat-maskval",
                 "Mask value is not in range of output image");
        c_lasmsg(TRUE,"concat-fatal","Fatal error encountered");
        }
      }
      break;
  }
}
