/*
************************************************************************
NAME: cellmnmx

PURPOSE: Get the cell min/max pixel values for a given set of cells.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           05/99    L. Johnson   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

  for all cell rows do
    for all cell columns do
      Initialize min and max values for cell.
      for all lines in cell do
        Get min and max pixel values.
        Check wheter value is masked.
        Set cell min to minimum of cell min and line min.
        Set cell max to maximum of cell max and line max.
      end do
    end do
  end do

*****************************************************************
	arguments:
	name		description
	----		-----------
	ibuff		input buffer (input)
        itype           data type (input)
        pxminc, pxmaxc  min and max pixel value arrays (output)
	nsbuf		number of samples in input buffer (input)
	nlc		number of rows in the cell (input)
	nsc		number of samples in the cell (input)
        ncrows, nccols  number of cell rows and columns (input)
        totccols        total number of cell rows in frame (input)
        nmask           number of mask values(input)
        maskval         mask values(input)

************************************************************************
*/
#include "las.h"
#include "typlim.h"       /* defines data type limits */
#include "constretch.h"   /* inf_err defined          */


int  cellmnmx
(
    void *ibuff,             /* input buffer (input)                        */
    long itype,              /* data type (input)                           */
    double *pxminc,          /* min pixel value arrays (output)             */
    double *pxmaxc,          /* max pixel value arrays (output)             */
    long nsbuf,              /* number of samples in input buffer (input)   */
    long *nlc,               /* number of rows in the cell (input)          */
    long *nsc,               /* number of samples in the cell (input)       */
    long ncrows,             /* number of cell rows (input)                 */
    long nccols,             /* number of cell columns (input)              */
    long totccols,           /* total number of cell rows in frame (input)  */
    long nmask,              /* number of mask values(input)                */
    double maskval[][2]      /* mask values(input)                          */
)
{
  unsigned char *badd;     /* line buffer pointers for diff. data types    */
  short *wadd;
  long *ladd;
  float *radd;
  long   ii, jj, kk, ll;   /* counter                                      */
  long   imask;            /* loop counter                                 */
  long   maskflag;         /* flag denoting whether pixel is masked        */
  long   ncell = 0;        /* cell counter                                 */
  long   nl, ns;           /* line and sample counters                     */
  long   offset;           /* address offset                               */
  double dmax, dmin;       /* max/min values for data type                 */


  /* Set up max/min values for data type.
  ---------------------------------------*/
  switch (itype)
     {
     case EBYTE:
        dmax = MAXUTINY;
        dmin = MINUTINY;
        break;

     case EWORD:
        dmax = MAXWORD;
        dmin = MINWORD;
        break;

     case ELONG:
        dmax = MAXLONG;
        dmin = MINLONG;
        break;

     case EREAL:
        dmax = MAXREAL;
        dmin = MINREAL;
        break;

     default:
        c_errmsg("Invalid data type.", "constretch-dtype", &inf_err);
        c_errmsg("Fatal error encountered", "constretch-fatal", &inf_err);
        return (E_FAIL);
     }

  nl = 0;
  for (ii=0; ii<ncrows; ii++)                   /* Loop over cell rows.     */
     {
     ns = 0;
     for (jj=0; jj<nccols; jj++, ncell++)       /* Loop over cell columns.  */
        {
        /* Initialize max/min to data type max/min.
        -------------------------------------------*/
        pxminc[ncell] = dmax;
        pxmaxc[ncell] = dmin;

        /* Find max/min values for cell.  Must consider data type for
           pointer incrementation.
        -------------------------------------------------------------*/
        switch (itype)            
           {
           case EBYTE:
              for (kk=0; kk<nlc[ii]; kk++)      /* Loop over cell lines.    */
                 {
                 offset = (nl + kk)*nsbuf + ns;
                 badd = (unsigned char *)ibuff + offset;
                 for (ll=0; ll<nsc[jj]; ll++, badd++) /* Loop over          */
                    {                                 /* line samples       */
                    /* Check pixel to determine whether it is masked.
                    -------------------------------------------------*/
                    maskflag = 0;
                    for (imask=0; imask<nmask; imask++)
                       {
                       if ((*badd >= maskval[imask][0]) &&
                           (*badd <= maskval[imask][1]))
                          maskflag = 1;
                       }
                    if (!maskflag)             /* If pixel is not masked... */
                       {
                       if (*badd < pxminc[ncell])
                           pxminc[ncell] = *badd;     /* new cell minimum   */
                       else if (*badd > pxmaxc[ncell])
                           pxmaxc[ncell] = *badd;     /* new cell maximum   */
                       }
                    }                                 /* line sample loop   */
                 }                                    /* cell line loop     */
              break;

           case EWORD:
              for (kk=0; kk<nlc[ii]; kk++)      /* Loop over cell lines.    */
                 {
                 offset = (nl + kk)*nsbuf + ns;
                 wadd = (short *)ibuff + offset;
                 for (ll=0; ll<nsc[jj]; ll++, wadd++) /* Loop over          */
                    {                                 /* line samples       */
                    /* Check pixel to determine whether it is masked.
                    -------------------------------------------------*/
                    maskflag = 0;
                    for (imask=0; imask<nmask; imask++)
                       {
                       if ((*wadd >= maskval[imask][0]) &&
                           (*wadd <= maskval[imask][1]))
                          maskflag = 1;
                       }
                    if (!maskflag)             /* If pixel is not masked... */
                       {
                       if (*wadd < pxminc[ncell])
                           pxminc[ncell] = *wadd;     /* new cell minimum   */
                       else if (*wadd > pxmaxc[ncell])
                           pxmaxc[ncell] = *wadd;     /* new cell maximum   */
                       }
                    }                                 /* line sample loop   */
                 }                                    /* cell line loop     */
              break;

           case ELONG:
              for (kk=0; kk<nlc[ii]; kk++)      /* Loop over cell lines.    */
                 {
                 offset = (nl + kk)*nsbuf + ns;
                 ladd = (long *)ibuff + offset;
                 for (ll=0; ll<nsc[jj]; ll++, ladd++) /* Loop over          */
                    {                                 /* line samples       */
                    /* Check pixel to determine whether it is masked.
                    -------------------------------------------------*/
                    maskflag = 0;
                    for (imask=0; imask<nmask; imask++)
                       {
                       if ((*ladd >= maskval[imask][0]) &&
                           (*ladd <= maskval[imask][1]))
                          maskflag = 1;
                       }
                    if (!maskflag)             /* If pixel is not masked... */
                       {
                       if (*ladd < pxminc[ncell])
                           pxminc[ncell] = *ladd;     /* new cell minimum   */
                       else if (*ladd > pxmaxc[ncell])
                           pxmaxc[ncell] = *ladd;     /* new cell maximum   */
                       }
                    }                                 /* line sample loop   */
                 }                                    /* cell line loop     */
              break;

           case EREAL:
              for (kk=0; kk<nlc[ii]; kk++)      /* Loop over cell lines.    */
                 {
                 offset = (nl + kk)*nsbuf + ns;
                 radd = (float *)ibuff + offset;
                 for (ll=0; ll<nsc[jj]; ll++, radd++) /* Loop over          */
                    {                                 /* line samples       */
                    /* Check pixel to determine whether it is masked.
                    -------------------------------------------------*/
                    maskflag = 0;
                    for (imask=0; imask<nmask; imask++)
                       {
                       if ((*radd >= maskval[imask][0]) &&
                           (*radd <= maskval[imask][1]))
                          maskflag = 1;
                       }
                    if (!maskflag)             /* If pixel is not masked... */
                       {
                       if (*radd < pxminc[ncell])
                           pxminc[ncell] = *radd;     /* new cell minimum   */
                       else if (*radd > pxmaxc[ncell])
                           pxmaxc[ncell] = *radd;     /* new cell maximum   */
                       }
                    }                                 /* line sample loop   */
                 }                                    /* cell line loop     */
              break;
           }                                          /* itype switch       */

        /* Increase sample counter by one cell line.
        ---------------------------------------------*/
        ns += nsc[jj];
        }                                       /* cell column loop         */
     /* Increase cell counter to point to 1st cell in next row for relevant
        block of cells.
     ----------------------------------------------------------------------*/
     ncell += totccols - nccols;

     /* Increase the line counter by the number of lines in the previous
        cell.
     -------------------------------------------------------------------*/
     nl += nlc[ii];
     }                                          /* cell row loop            */

  return(E_SUCC);

}    /*  end of cellmnmx()  */

