/*
*************************************************************************

NAME: CONSTRETCH

PURPOSE: Performs a local range modification (LRM) on an input image. An
         LRM is an adaptive contrast stretch which uses block max/min
         pixel values to increase the contrast within the block.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           12/98   L. Johnson  

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
       Get the user-specified parameters and TAE global variables.
       Open input image(s). 
       for each input image do
          for each input band do
             Get the cell min and max values for all cells.
          end do
       end do
       Compute the node min and max values.
       for each input image do
          for each input band do
             Compute the new pixel values based on linear interpolation
               between the four corner node min and max values.
          end do
       end do
       Update the output image DDR, min/max values, and history.

ALGORITHM REFERENCES:
     Schowengerdt, Robert A., Remote Sensing, Academic Press, San Diego,
     1997, p. 214.


*****************************************************************
*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "imageio.h"
#include "util.h"
#include "constretch.h"


int main() 
{
  struct PARBLK parblk;              /* TAE parameter block             */
  double line_inc = 1;               /* line interval for upddr         */
  double maskval[10][2];             /* mask values                     */
  double maxbnd[MAXBND+1];           /* output image maximum value      */
  double minbnd[MAXBND+1];           /* output image minimum value      */
  double maxval;                     /* max value for contrast stretch  */
  double minval;                     /* min value for contrast stretch  */
  double samp_inc = 1;               /* sample interval for upddr       */
  double thresh;                     /* contrast range threshold        */
  long bands[MAXIMG][MAXBND+1];      /* image band numbers              */
  long bdrflag;                      /* contrast stretch border flag    */
  long comb_flag = NOCOMB;           /* for upddr                       */
  long itype;                        /* input image data type           */
  long min_max;                      /* minmax option flag              */
  long msgtim;                       /* message interval                */
  long nbands[MAXIMG];               /* no. bands in each image         */
  long nl;                           /* no. lines in image              */
  long nmask;                        /* number of mask values           */
  long nmrimg;                       /* no. images                      */
  static long nonfatal = 1;	     /* non-fatal error status	        */
  long nrows, ncols;                 /* number of cells in x and y dir. */
  long ns;                           /* no. samples/line in image       */
  static long one = 1;	             /* constant		        */
  long otype;                        /* output image data type          */
  long stat;                         /* status                          */
  long totbnd;                       /* total no. bands                 */
  long totsam;		             /* total samples per band 	        */
  long window[MAXIMG][4];            /* image windows                   */
  static long zero = 0;		     /* constant		        */
  char prog_name[11];	    	     /* program name		        */
  char hosin[MAXIMG][CMLEN];         /* Input file name(s)              */
  char hosout[CMLEN];                /* Output file name (host format)  */
  char taeout[CMLEN];                /* Output file name (TAE format)   */


  strcpy(prog_name,"CONSTRETCH");
  /* Get user-specified parameters and tae global variables.
  ----------------------------------------------------------*/
  getpar(&parblk, hosin, taeout, hosout, &totbnd, bands, nbands, &nl, &ns,
	 window, &nmrimg, &nrows, &ncols, &itype, &otype, &maxval, &minval,
         &min_max, &msgtim, &bdrflag, &thresh, &nmask, maskval);

  /* Initialize display message.
  ------------------------------*/
  totsam = nl*ns;
  c_inittm ("CONSTRETCH", "Processed", "sample", "bands", " ", &totsam,
            &totbnd, &msgtim, &zero);

  /* Process the image(s).
  ------------------------*/
  stat = pro_img(nmrimg, window, bands, nbands, totbnd, hosin, hosout,
                 itype, otype, maxval, minval, min_max, maxbnd, minbnd,
                 nrows, ncols, bdrflag, thresh, nmask, maskval);
  if (stat == E_FAIL)  return EXIT_FAILURE;


  /* Update ddr and minmax and for output image.
  ----------------------------------------------*/
  if (c_upddr(&parblk, (const char (*)[CMLEN])hosin, hosout, &nmrimg,
              (const long (*)[4])window, (const long (*)[MAXBND+1])bands,
              nbands, &line_inc, &samp_inc, &totbnd, &comb_flag) == E_FAIL)
     c_errmsg("Nonfatal error encountered","Constretch-warn",&nonfatal);

  if (min_max)
     {
     if (c_svmnmx(hosout, &totbnd, minbnd, maxbnd, "YES") == E_FAIL)
        c_errmsg("Nonfatal error encountered","Constretch-warn",&nonfatal);
     }

  /* Create associated history file for output image.
  ---------------------------------------------------*/
  if (c_hstout(&parblk, hosout, hosin, &nmrimg, " ") == E_FAIL)
     c_errmsg("Nonfatal error encountered","Constretch-warn",&nonfatal);

  /* Output completion message.
  -----------------------------*/
  c_complt ("CONSTRETCH", taeout, &one);

  return 0;
}    /* end of constretch()  */
