/*
*************************************************************************
NAME: GETPAR

PURPOSE: Gets the user-specified parameters and tae globals.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           12/98   L. Johnson

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name			description
	----			-----------
        parblk          TAE parameter block
	hosin  		host names of input images
	taeout		tae name of output image
	hosout		host name of output image
	totbnd		total number of output bands
	bands           band specification
	nbands 		number of bands in each image
	nl		number of lines to be processed
	ns		number of samples per line to be processed
	window		window specification
	nmrimg		number of input images
	nrows		number of rows in input images
	ncols		number of columns in input images
	itype		data type of input image
	otype		output data type
        maxval          maximum value for contrast stretch
        minval          minimum value for contrast stretch
        min_max         minmax option flag
        bdrflag         contrast stretch border flag
        thresh          contrast range threshold
        nmask           number of mask values
        maskval         mask values

************************************************************************
*/
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "typlim.h"
#include "constretch.h"

void getpar
(
    struct PARBLK *parblk,   /* TAE parameter block                         */
    char hosin[][CMLEN],     /* host names of input images                  */
    char taeout[],           /* tae name of output image                    */
    char hosout[],           /* host name of output image                   */
    long *totbnd,            /* total number of output bands                */
    long bands[][MAXBND+1],  /* band specification                          */
    long nbands[],           /* number of bands in each image               */
    long *nl,                /* number of lines to be processed             */
    long *ns,                /* number of samples per line to be processed  */
    long window[][4],        /* window specification                        */
    long *nmrimg,            /* number of input images                      */
    long *nrows,             /* number of rows in input images              */
    long *ncols,             /* number of columns in input images           */
    long *itype,             /* data type of input image                    */
    long *otype,             /* output data type                            */
    double *maxval,          /* maximum value for contrast stretch          */
    double *minval,          /* minimum value for contrast stretch          */
    long *min_max,           /* minmax option flag                          */
    long *msgtim,            /*                                             */
    long *bdrflag,           /* contrast stretch border flag                */
    double *thresh,          /* contrast range threshold                    */
    long *nmask,             /* number of mask values                       */
    double maskval[][2]      /* mask values                                 */
)
{
  struct VARIABLE *var;       /* Pointer to a parameter in vblk             */
  static long fatal_err = -1; /* Fatal error flag                           */
  long	dtype = ELONG;        /* Data type for TAE input values             */
  long  ftype = EDOUBLE;      /* Data type for real numbers                 */
  long  ii;                   /* Loop counter                               */
  long	numcells[2];          /* Number of grid cells in x and y directions */
  long	length;               /* String length                              */
  long	maximg = MAXIMG;      /* Maximum number of images                   */
  static long  one = 1;       /* Constant                                   */
  long	qflag[MAXIMG][2];     /* Window and band option flags for images.   */
  long	same;                 /* Flag denoting whether input images are     */
                              /*      same size                             */
  long	status;               /* Return error status for functions          */
  long	std_out;              /* Standard output ID                         */
  static long two = 2;        /* Constant                                   */
  static long zero = 0;       /* Constant                                   */
  double temp;                /* Temporary (scratch) variable               */
  char  borders[4];           /* Contrast stretch border string (ON, OFF)   */
  char  mask[10][50];         /* mask values                                */
  char	minmax[4];            /* TAE global variable $minmax                */
  char	odtype[5];            /* Output data type                           */
  char  *ptr;                 /* Temporary storage space for pointer        */
  char  *ptr2;                /* Temporary storage space for pointer        */
  char	taein[MAXIMG][CMLEN]; /* Input file name(s)                         */
  char  tempstr[80];          /* Temporary storage space for string         */


  /* Initialization.
  ------------------*/
  c_inlas(parblk, &std_out);

  /* Get globals.
  ---------------*/
  var = p_find(parblk, "$msgtim");
  *msgtim = IVAL(*var, 0);

  var = p_find(parblk, "$minmax");
  strcpy(minmax, SVAL( *var, 0));
  if (minmax[0] == 'Y')
     *min_max = TRUE;
  else
     *min_max = FALSE;

  /* Get input file information.
  ------------------------------*/
  status = c_getin(parblk, "IN", &zero, taein, hosin, nmrimg, window, bands, 
                   nbands, totbnd, &maximg, &same, qflag);
  if (status  !=  E_SUCC) 
     c_errmsg("Fatal error encountered", "constretch-fatal", &fatal_err);

  if (same  !=  1) 
     c_errmsg("Input images do not have the same window size",
              "constretch-windsiz", &fatal_err);

  *nl = window[0][NL];
  *ns = window[0][NS];

  /* Get output file name.
  ------------------------*/
  status = c_getout(parblk, "OUT", taeout, hosout);
  if (status  !=  E_SUCC) 
     c_errmsg ("Fatal error encountered", "constretch-fatal", &fatal_err);

  /* Get the number of cells in the x and y directions.
  -----------------------------------------------------*/
  c_getnum(parblk, "NUMCELLS", &two, &dtype, numcells);
  *nrows = numcells[0];
  *ncols = numcells[1];

  /* Validate cell dimensions.
  ------------------------------*/
  if(*nrows <1 || *ncols < 1) 
     c_errmsg("Invalid number of cells -- Must be greater than 0",
              "constretch-small", &fatal_err);
  else if((*ncols > *ns)  ||  (*nrows > *nl)) 
    c_errmsg("Invalid number of cells -- Cannot have more cells than samples",
             "constretch-large", &fatal_err);

  /* Get output data type.
  ------------------------*/
  c_getstr(parblk, "ODTYPE", &one, &length, odtype);
  if (strcmp(odtype, "SAME") == 0)
      *otype = 0;
  else if (strcmp(odtype, "BYTE") == 0)
      *otype = EBYTE;
  else if (strcmp(odtype, "I*2") == 0)
      *otype = EWORD;
  else if (strcmp(odtype, "I*4") == 0)
      *otype = ELONG;
  else if (strcmp(odtype, "R*4") == 0)
      *otype = EREAL;
  else
      c_errmsg("Invalid data type","constretch-dtype",&fatal_err);

   status = c_gettyp(hosin, nmrimg, itype);
   if (status  !=  E_SUCC) 
      c_errmsg ("Input images are not of same data type", "constretch-diftyp",
	         &fatal_err);
  if (*itype != EBYTE && *itype != EWORD && *itype != ELONG &&
                                                              *itype != EREAL)
      c_errmsg("Invalid data type","constretch-dtype",&fatal_err);
  if (*otype == 0) 
      *otype = *itype;

  /* Get the requested minimum and maximum values.
   -----------------------------------------------*/
  switch (*otype)
      {
      case EBYTE:
          *maxval = MAXUTINY;
          *minval = MINUTINY;
          break;

      case EWORD:
          *maxval = MAXWORD;
          *minval = MINWORD;
          break;

      case ELONG:
          *maxval = MAXLONG;
          *minval = MINLONG;
          break;

      case EREAL:
          *maxval = MAXREAL;
          *minval = MINREAL;
          break;
      }
  if (c_getnum(parblk, "MAXVAL", &one, &ftype, &temp) != 0)
      {
      if (temp > *maxval || temp < *minval)
          c_errmsg ("MAXVAL outside range for data type", "constretch-minmax",
                    &fatal_err);
      else
          *maxval = (double)temp;
      }
  if (c_getnum(parblk, "MINVAL", &one, &ftype, &temp) != 0)
      {
      if (temp >= *maxval)
          c_errmsg ("MAXVAL less than MINVAL", "constretch-minmax",
                    &fatal_err);
      else if (temp < *minval)
          c_errmsg ("MINVAL outside range for data type", "constretch-minmax",
                    &fatal_err);
      else
          *minval = (double)temp;
      }

  /* Get the "border stretch" flag.
  ---------------------------------*/
  c_getstr(parblk, "BORDERS", &one, &length, borders);
  if (strcmp(borders, "ON") == 0)
      *bdrflag = 1;
  else
      *bdrflag = 0;

  /* Get the threshold value.
  ---------------------------*/
  if (c_getnum(parblk, "THRSHOLD", &one, &ftype, thresh) == 0)
      *thresh = 0.0;    /* THRSHOLD is null , set thresh to zero */
  else                  /* THRSHOLD is an actual value, check it */
      {
      if (*thresh < 0.0)
          c_errmsg ("Invalid value for THRVAL", "constretch-thresh",
                    &fatal_err);
      }

  /* Get the mask values.
  -----------------------*/
  var = p_find(parblk, "MASKVAL");
  *nmask = var->v_count;
  for (ii=0; ii<*nmask; ii++)
      {
      strcpy(mask[ii], SVAL(*var, ii));

      /* Locate and delete spaces.
      ----------------------------*/
      ptr = mask[ii];
      ptr2 = mask[ii];
      while (*ptr2 != '\0')
          {
          if (!isspace(*ptr2))
              *(ptr++) = *(ptr2++);
          else
              ptr2++;
          }
      *ptr = '\0';

      /* Locate the dash and divide mask values accordingly.
      ------------------------------------------------------*/
      ptr = (char *)strchr(mask[ii], '-');
      if (ptr == mask[ii])
          {
          /* Handle case where first value may be negative.
          -------------------------------------------------*/
          sscanf(mask[ii], "-%s", tempstr);
          ptr = (char *)strchr(tempstr, '-');
          }
      if (ptr == NULL)
          {
          sscanf(mask[ii], "%lf", &maskval[ii][0]);
          maskval[ii][1] = maskval[ii][0];
          }
      else
          sscanf(mask[ii],"%lf-%lf",&maskval[ii][0],&maskval[ii][1]);

      /* Verify that range mask values are in ascending order.
      --------------------------------------------------------*/
      if (maskval[ii][0] > maskval[ii][1])
          c_errmsg("Error in mask value range", "constretch-mask",&fatal_err);
      }
}
