/*
******************************************************************************
NAME: nodemnmx

PURPOSE: Set the max and min values for all cell nodes.

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1           02/99   L. Johnson

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

  Set max and min values for data type.
  For all cells do
    if contrast range < threshold then
      Set cell max and min values to max and min for data type.
    end if
  end do
  Find max and min values for first row of nodes.
  Find max and min values for nodes in rows 2 through N-1.
  Find max and min values for nodes in last (Nth) row.

******************************************************************************
        arguments:
        name            description
        ----            -----------
        pxminc, pxmaxc  min and max pixel values for cells
        pxminn, pxmaxn  min and max pixel values for nodes
        ncrows, nccols  number of cell rows and columns
        bdrflag         contrast stretch border flag
        dtype           input data type
        thresh          contrast range threshold

******************************************************************************
*/
#include "las.h"
#include "constretch.h"
#include "typlim.h"


int  nodemnmx
(
    double *pxminc,  /* min pixel values for cells                          */
    double *pxmaxc,  /* max pixel values for cells                          */
    double *pxminn,  /* min pixel values for nodes                          */
    double *pxmaxn,  /* max pixel values for nodes                          */
    long ncrows,     /* number of cell rows                                 */
    long nccols,     /* number of cell columns                              */
    long bdrflag,    /* contrast stretch border flag                        */
    long dtype,      /* input data type                                     */
    double thresh    /* contrast range threshold                            */
)
{
  long    ncell;       /* cell counter                              */
  long    nnode;       /* node counter                              */
  long	  ii, jj;      /* counter                                   */
  double  pmax, pmin;  /* max and min values for data type          */
  double  temp;        /* temporary storage                         */


  /* Get max and min values for data type.
  ----------------------------------------*/
  switch (dtype)
     {
     case EBYTE:
         pmax = MAXUTINY;
         pmin = MINUTINY;
         break;
     case EWORD:
         pmax = MAXWORD;
         pmin = MINWORD;
         break;
     case ELONG:
         pmax = MAXLONG;
         pmin = MINLONG;
         break;
     case EREAL:
         pmax = MAXREAL;
         pmin = MINREAL;
         break;
     default:
         c_errmsg("Invalid data type.","constretch-dtype",&inf_err);
         c_errmsg("Fatal error encountered","constretch-fatal",&inf_err);
         return(E_FAIL);
     }

  /* Compare all cell contrast ranges to threshold.  If cell range below
     threshold, set range to max so the cell will not be stretched.
  ----------------------------------------------------------------------*/
  for (ii=0; ii<ncrows*nccols; ii++)
      if ((pxmaxc[ii] - pxminc[ii]) < thresh)
          {
          pxminc[ii] = pmin;
          pxmaxc[ii] = pmax;
	  }

  /* Find max and min values for first row of nodes.
  --------------------------------------------------*/
  if (bdrflag)
      {
      /* Border flag set, so entire cell will be stretched.
      -----------------------------------------------------*/
      pxminn[0] = pxminc[0];
      pxmaxn[0] = pxmaxc[0];
      for (nnode=1; nnode<nccols; nnode++)
          {
          pxminn[nnode] = min(pxminc[nnode-1], pxminc[nnode]);
          pxmaxn[nnode] = max(pxmaxc[nnode-1], pxmaxc[nnode]);
          }
      pxminn[nnode] = pxminc[nnode-1];
      pxmaxn[nnode] = pxmaxc[nnode-1];
      nnode++;
      }
  else
      {
      /* Border flag not set, so cell boundary will not be stretched.
         This is enforced by setting the boundary range to max.
      ---------------------------------------------------------------*/
      for (nnode=0; nnode<=nccols; nnode++)
          {
          pxminn[nnode] = pmin;
          pxmaxn[nnode] = pmax;
          }
      }

  /* Find max and min values for nodes in rows 2 through N-1.
  -----------------------------------------------------------*/
  ncell = 0;
  for (ii=1; ii<ncrows; ii++)                      /* Loop over cell rows. */
      {
      if (bdrflag)
          {
          /* Border flag set, so entire cell will be stretched.
          -----------------------------------------------------*/
          pxminn[nnode] = min(pxminc[ncell], pxminc[ncell+nccols]);
          pxmaxn[nnode] = max(pxmaxc[ncell], pxmaxc[ncell+nccols]);
          }
      else
          {
          /* Border flag not set, so cell boundary will not be stretched.
             This is enforced by setting the boundary range to max.
          ---------------------------------------------------------------*/
          pxminn[nnode] = pmin;
          pxmaxn[nnode] = pmax;
          }
      nnode++;
      for (jj=1; jj<nccols; jj++, ncell++, nnode++)/*Loop over cell columns.*/
          {
          pxminn[nnode] = min(pxminc[ncell], pxminc[ncell+1]);
          temp = min(pxminc[ncell+nccols], pxminc[ncell+nccols+1]);
          if (temp < pxminn[nnode])
              pxminn[nnode] = temp;
          pxmaxn[nnode] = max(pxmaxc[ncell], pxmaxc[ncell+1]);
          temp = max(pxmaxc[ncell+nccols], pxmaxc[ncell+nccols+1]);
          if (temp > pxmaxn[nnode])
              pxmaxn[nnode] = temp;
          }
      if (bdrflag)
          {
          /* Border flag set, so entire cell will be stretched.
          -----------------------------------------------------*/
          pxminn[nnode] = min(pxminc[ncell], pxminc[ncell+nccols]);
          pxmaxn[nnode] = max(pxmaxc[ncell], pxmaxc[ncell+nccols]);
          }
      else
          {
          /* Border flag not set, so cell boundary will not be stretched.
             This is enforced by setting the boundary range to max.
          ---------------------------------------------------------------*/
          pxminn[nnode] = pmin;
          pxmaxn[nnode] = pmax;
          }
      nnode++;
      ncell++;
      }

  /* Find max and min values for nodes in last (Nth) row.
  -------------------------------------------------------*/
  if (bdrflag)
      {
      /* Border flag set, so entire cell will be stretched.
      -----------------------------------------------------*/
      pxminn[nnode] = pxminc[ncell];
      pxmaxn[nnode] = pxmaxc[ncell];
      nnode++;
      for (ii=1; ii<nccols; ii++, ncell++, nnode++)
          {
          pxminn[nnode] = min(pxminc[ncell], pxminc[ncell+1]);
          pxmaxn[nnode] = max(pxmaxc[ncell], pxmaxc[ncell+1]);
          }
      pxminn[nnode] = pxminc[ncell];
      pxmaxn[nnode] = pxmaxc[ncell];
      nnode++;
      }
  else
      {
      /* Border flag not set, so cell boundary will not be stretched.
         This is enforced by setting the boundary range to max.
      ---------------------------------------------------------------*/
      for (ii=0; ii<=nccols; ii++, nnode++)
          {
          pxminn[nnode] = pmin;
          pxmaxn[nnode] = pmax;
          }
      }

  return(E_SUCC);

}    /*  end of nodemnmx()  */
