/*************************************************************************
NAME: pro_blk

PURPOSE:  process an image block

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           02/99   L. Johnson

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

  Initialize arrays and flags.
  while more data in input image(s) do
    for all images do
      for all bands in image do
        Read image block.
        Find the cell column and row for the origin of the present
             image block.
        if finding minmax then
          Get min and max pixel values for cells in block.
        else (creating new image)
          Compute the new pixel gray levels.
          Write new pixel values to output file.
          if band min and max requested then
            Calculate band min, max value for the ddr.
          end if
        end if
      end do
    end do
  end do

*****************************************************************
	arguments:
	name		description
	----		-----------
        proflag         Process flag (input):
                                      0 = get min and max pixel values
                                      1 = Compute the new pixel values
	nmrimg		number of images (input)
	window		window of image (input)
	bands		band numbers to be processed (input)
	nbands		number of bands in each image (input)
	totbnd		total number of bands (input)
	hosin		input image host name (input)
	hosout		output image host name (input)
	min_max		min, max flag from TAE land (input)
	maxbnd		array of max pixel values (output)
	minbnd		array of min pixel values (output)
        nicols          number of cell columns in image
        ibuff           data buffer to read (input)
        obuff           data buffer to write (input)
	itype		input data type (input)
	otype		output data type (input)
    	nlib		number of lines in 'ibuff' (input)
	nsib		number of samples in 'ibuff' (input)
        nlb             number of lines in the boundary (input)
        nsb             number of samples in the boundary (input)
    	nli		number of lines in the image (input)
	nsi		number of samples in the image (input)
        pxminc, pxmaxc  min and max pixel values for cells (input/output)
        pxminn, pxmaxn  min and max pixel values for nodes (input)
        nicells         total number of cells per frame (input)
        ninodes         total number of nodes per frame (input)
        nlc, nsc        number of lines and samples per cell (input)
        ncrb, nccb      number of cell rows and columns in buffer (input)
        ncrb1, nccb1    number of cell rows and columns in last buffer
                        row and column, respectively (input)
        totlb, totsb    total number of buffer blocks in x and y dir (input)
        maxval          maximum value for contrast stretch(input)
        minval          minimum value for contrast stretch(input)
        nmask           number of mask values(input)
        maskval         mask values(input)
	
******************************************************************/


#include <stdio.h>
#include "las.h"
#include "imageio.h"
#include "constretch.h"

int pro_blk
(
    long proflag,                 /* I: Process flag:
                                          0 = get min and max pixel values
                                          1 = Compute the new pixel values  */
    long nmrimg,                  /* I: number of images                    */
    long window[MAXIMG][4],       /* I: window of image                     */
    long bands[MAXIMG][MAXBND+1], /* I: band numbers to be processed        */
    long nbands[MAXIMG],          /* I: number of bands in each image       */
    long totbnd,                  /* I: total number of bands               */
    char hosin[MAXIMG][CMLEN],    /* I: input image host name               */
    char hosout[CMLEN],           /* I: output image host name              */
    long min_max,                 /* I: min, max flag from TAE land         */
    double maxbnd[MAXBND+1],      /* O: array of max pixel values           */
    double minbnd[MAXBND+1],      /* O: array of min pixel values           */
    long nicols,                  /* I: number of cell columns in image     */
    unsigned char *ibuff,         /* I: data buffer to read                 */
    unsigned char *obuff,         /* I: data buffer to write                */
    long itype,                   /* I: input data type                     */
    long otype,                   /* I: output data type                    */
    long *nlib,                   /* I: number of lines in 'ibuff'          */
    long *nsib,                   /* I: number of samples in 'ibuff'        */
    long nlb,                     /* I: number of lines in the boundary     */
    long nsb,                     /* I: number of samples in the boundary   */
    long nli,                     /* I: number of lines in the image        */
    long nsi,                     /* I: number of samples in the image      */
    double *pxminc,               /* I/O: min pixel values for cells        */
    double *pxmaxc,               /* I/O: max pixel values for cells        */
    double *pxminn,               /* I: min pixel values for nodes          */
    double *pxmaxn,               /* I: max pixel values for nodes          */
    long nicells,                 /* I: total number of cells per frame     */
    long ninodes,                 /* I: total number of nodes per frame     */
    long *nlc,                    /* I: number of lines per cell            */
    long *nsc,                    /* I: number of samples per cell          */
    long ncrb,                    /* I: number of cell rows in buffer       */
    long nccb,                    /* I: number of cell columns in buffer    */
    long ncrb1,                   /* I: num. of cell rows in last buffer row*/
    long nccb1,                   /* I: num. of cell cols in last buffer col*/
    long totlb,                   /* I: tot. num. of buffer blocks in y dir */
    long  totsb,                  /* I: tot. num. of buffer blocks in x dir */
    double maxval,                /* I: maximum value for contrast stretch  */
    double minval,                /* I: minimum value for contrast stretch  */
    long nmask,                   /* I: number of mask values               */
    double maskval[][2]           /* I: mask values                         */
)
{
  struct FDESC *fdesc[MAXIMG]; /* input image file descriptor             */
  struct FDESC * odesc;	/* output image file descriptor                   */
  long   done;          /* flag whether finished                          */
  long	 fill = MIRR;	/* boundary fill technique                        */
  long	 ftthru;  	/* first time thru flag	                          */
  long	 iband, jj, im;	/* counters                                       */
  long   iframe;        /* image frame counter                            */
  long   lbcnt, sbcnt;  /* block counters in line and sample directions.  */
  long   ncrows, nccols;/* number of cell rows and columns                */
  long   ncell;         /* cell number                                    */
  long	 nextflg;	/* next flag for input image                      */
  long	 nextflp;	/* next flag for output image                     */
  long	 nlav;		/* number of actual valid lines in 'ibuff'        */
  long   nnode;         /* node number                                    */
  long	 nsav;		/* number of actual valid samples in 'ibuff'      */
  long   ntmp, ntmp1;   /* temporary storage                              */
  long	 oband;		/* output band number                             */
  long	 sldat[MAXIMG];	/* starting line of input image                   */
  long	 sldato;	/* starting line of output image                  */
  long	 ssdat[MAXIMG];	/* starting sample of input image                 */
  long	 ssdato;	/* starting sample of output image                */
  long	 stat;		/* return status                                  */


  nextflg = INIT;	         /* first call to get_buf()               */
  if (proflag == 1)
     {
     nextflp = CREAT;            /* first call to put_buf()               */
     oband = 1;		         /* initialize output band number         */
     }
  ftthru = 1;                    /* init. first time through flag         */
  done = 0;
  lbcnt = 0;
  sbcnt = 0;

  for (im = 0; im < nmrimg; im++) /* init input starting origins          */
     {
     sldat[im] = window[im][0];
     ssdat[im] = window[im][1];
     }
  sldato = 1;		         /* initialize output starting origins    */
  ssdato = 1;

  while(1)                                            /* image block loop */
     {     

     iframe = 0;              /* reset frame counter for next image block */

     for (im = 0; im < nmrimg; im++)                  /* image loop       */
        {
        /* Work-around for get_buf limitations.
        ---------------------------------------*/
        if (lbcnt > 0)
	   {
           sldat[im] += nlib[lbcnt-1] - nlib[lbcnt];
           sldato += nlib[lbcnt-1] - nlib[lbcnt];
	   }
        else if (sbcnt > 0 && sbcnt < totsb)
	   {
           ssdat[im] += nsib[sbcnt-1] - nsib[sbcnt];
           ssdato += nsib[sbcnt-1] - nsib[sbcnt];
	   }

        for (iband = 0; iband < nbands[im]; iband++, iframe++) /* band loop */
           {
           /* Get the next image block.
           ----------------------------*/
           stat = get_buf(ibuff,&nlib[lbcnt],&nsib[sbcnt],&nlav,&nsav,
                          &window[im][0],&window[im][1],&nli,&nsi,&nlb,&nsb,
                          &sldat[im],&ssdat[im],&nextflg,&bands[im][iband],
                          &fill, &fdesc[im],&hosin[im][0],&itype);

           /* If the number of lines and samples in the block aren't what we
              expected, we have failed.
           -----------------------------------------------------------------*/
           if (stat == E_SUCC && (nlav != nlib[lbcnt] || nsav != nsib[sbcnt]))
              stat = E_FAIL;

           if (stat != E_SUCC)
              {
              /* Done reading images.
              -----------------------*/

              if (stat == E_EOIW)  /* "Read Origin beyond Input WIndow"? */
                 /* This is OK.  We've just finished reading the input
                    images.
                 -----------------------------------------------------*/
     	         stat = E_SUCC;

              /* Depending on proflag, we are either done with cell min/max
                 or done processing.  Either way exit the routine.
              -------------------------------------------------------------*/
              done = 1;

              break;
              }

           /* Find the cell/node column and row for the origin of the
              present image block.
           ----------------------------------------------------------*/
           ntmp = ncrb*lbcnt;
           ntmp1 = nccb*sbcnt;
           ncell = nicols*ntmp + ntmp1 + iframe*nicells;
           nnode = (nicols + 1)*ntmp + ntmp1 + iframe*ninodes;

           /* Set the correct number of rows/columns depending on whether
              this is the last row/column of the window.
           --------------------------------------------------------------*/
           if (lbcnt < totlb - 1)
              ncrows = ncrb;
           else
              ncrows = ncrb1;
           if (sbcnt < totsb - 1)
              nccols = nccb;
           else
              nccols = nccb1;

           /* Depending on the processing flag, either
              get min and max pixel values for cells in block or
              compute the new pixel gray levels.
           -----------------------------------------------------*/
           if (proflag == 0)
              stat = cellmnmx(ibuff, itype, &pxminc[ncell], &pxmaxc[ncell],
                              nsib[sbcnt], &nlc[ntmp], &nsc[ntmp1], ncrows,
                              nccols, nicols, nmask, maskval);
           else
              stat = strtch(ibuff, itype, obuff, otype, &pxminn[nnode],
                            &pxmaxn[nnode], nsib[sbcnt], &nlc[ntmp],
                            &nsc[ntmp1], ncrows, nccols, nicols, maxval,
                            minval, nmask, maskval);

           if (stat == E_FAIL)
              break;

           if (proflag == 1)
	      {
              /* Write new pixel values to output file.
              -----------------------------------------*/
              stat = put_buf(obuff,&nlib[lbcnt],&nsib[sbcnt],&nlav,&nsav,
                             &sldato,&ssdato,&nli,&nsi,&nextflp,&oband,
                             &totbnd,&odesc,hosout,&otype,&otype);

              if (stat  ==  E_FAIL)
                 /* Failed to write buffer to file.
                 ----------------------------------*/
                 break;

              if (min_max)
                 /* Calculate band min, max value for the ddr.
                 ---------------------------------------------*/
                 stat = extreme(obuff, otype, nsib[sbcnt], nlav, nsav, oband,
                                maxbnd, minbnd);

              if (stat == E_FAIL)
                 break;

              oband++;		/* increment output band number            */
              nextflp = CURNT;	/* moving to next block	                   */
              }

           nextflg = CURNT;	/* handle all bands before incr. block     */

           }                    /* bottom of band loop                     */

        if (stat == E_FAIL || done)
           break;

        if (ftthru)
           nextflg = INIT;	/* first open on input image               */  
        else
           nextflg = NEXT;

        }                       /* bottom of image loop                    */

     if (stat == E_FAIL || done)
        break;

     if (lbcnt < totlb - 1)
        lbcnt++;
     else
        {
        lbcnt = 0;
        sbcnt++;
        }
     ftthru = 0;	        /* no longer first time thru               */
     nextflg = NEXT;	        /* move to next block before reading image */
     if (proflag == 1)
        {
        oband = 1;	        /* reset starting output band number       */
        nextflp = NEXT;	        /* move to next block before writing image */
	}

     }                          /* bottom of image block loop              */

  /* Close the input image files.
  -------------------------------*/
  for (jj = 0; jj <= im; jj++)
     if (c_eclose(&fdesc[jj]) == E_FAIL)
        c_errmsg("Nonfatal error encountered","filter-warn", &inf_err);

  if (proflag == 1)
     /* Close the output image file.
     -------------------------------*/
     if (c_eclose(&odesc) == E_FAIL)
        c_errmsg("Nonfatal error encountered","filter-warn", &inf_err);

  return (stat);
}
