/*
************************************************************************
NAME: pro_img

PURPOSE:  process an image window (all specified bands)

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           02/99   L. Johnson

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

  Compute the total number of image frames from all images and bands.
  Allocate storage for the various node and cell arrays.
  Compute the number of lines and samples in each cell.
  Compute the size of the input and output storage buffers
       and allocate space for them.
  Find the number of lines and samples in each buffer read.
  Get min and max pixel values for all cells.
  Set the max and min values for all cell nodes.
  Compute the new pixel gray levels.
  
*****************************************************************
	arguments:
	name		description
	----		-----------
	nmrimg		number of images (input)
	window		window of image (input)
	bands		band numbers to be processed (input)
	nbands		number of bands in each image (input)
	totbnd		total number of bands (input)
	hosin		input image host name (input)
	hosout		output image host name (input)
	itype		input data type (input)
	otype		output data type (input)
        maxval          maximum value for contrast stretch (input)
        minval          minimum value for contrast stretch (input)
	min_max		min, max flag from TAE land (input)
	maxbnd		array of max pixel values (output)
	minbnd		array of min pixel values (output)
        nirows          number of cell rows in image (input)
        nicols          number of cell columns in image (input)
        bdrflag         contrast stretch border flag (input)
        thresh          contrast range threshold (input)
        nmask           number of mask values (input)
        maskval         mask values (input)
	
*****************************************************************
*/


#include <stdio.h>
#include "las.h"
#include "util.h"
#include "imageio.h"
#include "constretch.h"


int  pro_img
(
    long nmrimg,                  /* I: number of images                    */
    long window[MAXIMG][4],       /* I: window of image                     */
    long bands[MAXIMG][MAXBND+1], /* I: band numbers to be processed        */
    long nbands[MAXIMG],          /* I: number of bands in each image       */
    long totbnd,                  /* I: total number of bands               */
    char hosin[MAXIMG][CMLEN],    /* I: input image host name               */
    char hosout[CMLEN],           /* I: output image host name              */
    long itype,                   /* I: input data type                     */
    long otype,                   /* I: output data type                    */
    double maxval,                /* I: maximum value for contrast stretch  */
    double minval,                /* I: minimum value for contrast stretch  */
    long min_max,                 /* I: min, max flag from TAE land         */
    double maxbnd[MAXBND+1],      /* O: array of max pixel values           */
    double minbnd[MAXBND+1],      /* O: array of min pixel values           */
    long nirows,                  /* I: number of cell rows in image        */
    long nicols,                  /* I: number of cell columns in image     */
    long bdrflag,                 /* I: contrast stretch border flag        */
    double thresh,                /* I: contrast range threshold            */
    long nmask,                   /* I: number of mask values               */
    double maskval[][2]           /* I: mask values                         */
)
{
  unsigned char	*ibuff;	/* data buffer to read                            */
  unsigned char	*obuff;	/* data buffer to write                           */
  long   iframe;        /* frame counter                                  */
  long	 maxram;	/* maximum available bytes in RAM                 */
  long   ncrb, nccb;    /* number of cell rows and columns in buffer      */
  long   ncrb1, nccb1;  /* number of cell rows and columns in last buffer */
  long   nframes;       /* total number of frames (bands) in all images   */
  long   nicells;       /* total number of cells per frame                */
  long   ninodes;       /* total number of nodes per frame                */
  long   nlb = 0;       /* number of lines in the boundary                */
  long	 *nlib;		/* number of lines in 'ibuff'                     */
  long   *nlc, *nsc;    /* number of lines and samples per cell           */
                        /* row and column, respectively                   */
  long	 nlob;		/* number of lines in the output buffer           */
  long   nrcnt, nccnt;  /* number of cell rows and columns with an extra  */
                        /* line or sample, respectively                   */
  long   nsb = 0;       /* number of samples in the boundary              */
  long	 *nsib;		/* number of samples in 'ibuff'                   */
  long	 nsob;		/* number of samples in the output buffer         */
  long   ntmp, ntmp1;   /* temporary storage                              */
  long   proflag;       /* Process flag: 0 = get min and max pixel values */
                        /*               1 = Compute the new pixel values */
  long	 stat;		/* error return status                            */
  long	 ii, jj, im;	/* counters                                       */
  long	 nli;		/* number of lines in the image                   */
  long	 nsi;		/* number of samples in the image                 */
  long   totlb, totsb;  /* total number of buffer blocks in x and y dir.  */
  double *pxminc, *pxmaxc; /* min and max pixel values for cells          */
  double *pxminn, *pxmaxn; /* min and max pixel values for nodes          */


  maxram = MAXRAM;

  nli = window[0][2];
  nsi = window[0][3];
  nicells = nicols*nirows;
  ninodes = (nicols + 1)*(nirows + 1);
  
  /* Compute the total number of image frames from all images and bands.
  ----------------------------------------------------------------------*/
  nframes = 0;
  for (im = 0; im < nmrimg; im++)
     nframes += nbands[im];

  /* Allocate storage for the various node and cell arrays.
  ---------------------------------------------------------*/
  if (aloc_buf(nirows, 1, ELONG, (char **)&nlc) == 0)
     return(E_FAIL);
  if (aloc_buf(nicols, 1, ELONG, (char **)&nsc) == 0)
     return(E_FAIL);
  if (aloc_buf(nirows+1, 1, ELONG, (char **)&nlib) == 0)
     return(E_FAIL);
  if (aloc_buf(nicols+1, 1, ELONG, (char **)&nsib) == 0)
     return(E_FAIL);
  if (aloc_buf(nicells, nframes, EDOUBLE, (char **)&pxminc) == 0)
     return(E_FAIL);
  if (aloc_buf(nicells, nframes, EDOUBLE, (char **)&pxmaxc) == 0)
     return(E_FAIL);
  if (aloc_buf(ninodes, nframes, EDOUBLE, (char **)&pxminn) == 0)
     return(E_FAIL);
  if (aloc_buf(ninodes, nframes, EDOUBLE, (char **)&pxmaxn) == 0)
     return(E_FAIL);

  /* Compute the number of lines and samples in each cell.
  --------------------------------------------------------*/
  nrcnt = nli % nirows;
  nccnt = nsi % nicols;
  nlc[0] = nli/nirows;
  for (ii=1; ii<nirows; ii++)
     nlc[ii] = nlc[0];
  nsc[0] = nsi/nicols;
  for (ii=1; ii<nicols; ii++)
     nsc[ii] = nsc[0];
  for (ii=0; ii<nrcnt; ii++)
     nlc[ii]++;
  for (ii=0; ii<nccnt; ii++)
     nsc[ii]++;

  /* Compute the size of the input and output storage buffers
     and allocate space for them.
  ------------------------------------------------------------*/
  if (size_buf(&nli, &nsi, &nlc[0], &nsc[0], &nlb, &nsb, &itype, &otype,
               &maxram, &nlib[0], &nsib[0], &nlob, &nsob) == E_FAIL)
     return(E_FAIL);
  if (aloc_buf(nlib[0], nsib[0], itype, (char **)&ibuff) == 0)
     return(E_FAIL);
  if (aloc_buf(nlob, nsob, otype, (char **)&obuff) == 0)
     return(E_FAIL);

  ncrb = nlib[0]/nlc[0]; /* number of cell rows and columns in input buffer */
  nccb = nsib[0]/nsc[0];

  /* Find the number of lines and samples in each buffer read.
  ------------------------------------------------------------*/
  ii = 0;
  totlb = 0;
  while (ii < nirows)
     {
       nlib[totlb] = 0;
       for (jj=0; jj<ncrb; jj++, ii++)
	  {
          if (ii < nirows)
             nlib[totlb] += nlc[ii];
          else
             break;
          }
       if (jj > 0)
          totlb++;
     }
  nlib[totlb] = nlib[totlb-1];
  ncrb1 = jj;
  ii = 0;
  totsb = 0;
  while (ii < nicols)
     {
       nsib[totsb] = 0;
       for (jj=0; jj<nccb; jj++, ii++)
	  {
          if (ii < nicols)
             nsib[totsb] += nsc[ii];
          else
             break;
          }
       if (jj > 0)
          totsb++;
     }
  nsib[totsb] = nsib[totsb-1];
  nccb1 = jj;

  c_startm();

  /* Get min and max pixel values for all cells.
  ----------------------------------------------*/
  proflag = 0;
  stat = pro_blk(proflag, nmrimg, window, bands, nbands, totbnd, hosin,
                 hosout, min_max, maxbnd, minbnd, nicols, ibuff,
                 obuff, itype, otype, nlib, nsib, nlb, nsb, nli, nsi,
                 pxminc, pxmaxc, pxminn, pxmaxn, nicells, ninodes, nlc, nsc,
                 ncrb, nccb, ncrb1, nccb1, totlb, totsb, maxval, minval,
                 nmask, maskval);

  /* Free the image buffers and exit if there has been a failure.
  ---------------------------------------------------------------*/
  if (stat != E_SUCC)
     {
     free_buf((unsigned char **)&nlc);
     free_buf((unsigned char **)&nsc);
     free_buf((unsigned char **)&nlib);
     free_buf((unsigned char **)&nsib);
     free_buf((unsigned char **)&pxminc);
     free_buf((unsigned char **)&pxmaxc);
     free_buf((unsigned char **)&pxminn);
     free_buf((unsigned char **)&pxmaxn);
     free_buf((unsigned char **)&ibuff);
     free_buf((unsigned char **)&obuff);
     c_stoptm();
     return (stat);
     }

  /* Set the max and min values for all cell nodes.
  -------------------------------------------------*/
  iframe = 0;
  for (im = 0; im < nmrimg; im++)
     {
     for (ii = 0; ii < nbands[im]; ii++, iframe++)
        {
        ntmp = nicells*iframe;
        ntmp1 = ninodes*iframe;
        stat = nodemnmx(&pxminc[ntmp], &pxmaxc[ntmp], &pxminn[ntmp1],
                        &pxmaxn[ntmp1], nirows, nicols, bdrflag, itype,
                        thresh);
        if (stat != E_SUCC)
           return (stat);
        }
     }

  /* Compute the new pixel gray levels.
  -------------------------------------*/
  proflag = 1;
  stat = pro_blk(proflag, nmrimg, window, bands, nbands, totbnd, hosin,
                 hosout, min_max, maxbnd, minbnd, nicols, ibuff,
                 obuff, itype, otype, nlib, nsib, nlb, nsb, nli, nsi,
                 pxminc, pxmaxc, pxminn, pxmaxn, nicells, ninodes, nlc, nsc,
                 ncrb, nccb, ncrb1, nccb1, totlb, totsb, maxval, minval,
                 nmask, maskval);

  /* Free the image buffers.
  --------------------------*/
  free_buf((unsigned char **)&nlc);
  free_buf((unsigned char **)&nsc);
  free_buf((unsigned char **)&nlib);
  free_buf((unsigned char **)&nsib);
  free_buf((unsigned char **)&pxminc);
  free_buf((unsigned char **)&pxmaxc);
  free_buf((unsigned char **)&pxminn);
  free_buf((unsigned char **)&pxmaxn);
  free_buf((unsigned char **)&ibuff);
  free_buf((unsigned char **)&obuff);

  c_stoptm();
  return (stat);

}    /* end of pro_img()   */
