/*
******************************************************************************
NAME: SIZE_BUF

PURPOSE: determines the size of the input and output buffer required
by the sliding window.

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
     1          12/98    L. Johnson


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
	The following function determines the maximum allowable size
	for the input and output buffers.  The following assumptions
	are made:
	  (1) The sum of the input buffer and the output buffer should
	      not exceed maxram bytes.
	  (2) The output buffer size will be that of the input buffer
	      less the boundary.  Since the output buffer size is a 
	      function of the input buffer size, both buffer sizes must
	      be calculated simultaneously.
	  (3) An approximately square buffer is the most efficient
	      buffer shape for general processing.
          (4) The buffers must hold a rectangular grouping of image
              cells, with the cell size defined by nlk, nsk.

******************************************************************************
	arguments:
	name		description
	----            -----------
	 nli            number of lines in the input image (input)
	 nsi            number of samples in the input image (input)
	 nlk            number of lines in the kernel (input)
	 nsk            number of samples in the kernel (input)
	 nlb            number of lines in the boundary (input)
	 nsb            number of samples in the boundary (input)
	 itype	        input data type (input)
	 otype	        output data type (input)
	 maxram         maximum buffer memory size in bytes (input)
	 nlib           number of lines in the input buffer (output)
	 nsib           number of samples in the input buffer (output)
	 nlob           number of lines in the output buffer (output)
	 nsob           number of samples in the output buffer (output)

******************************************************************************
*/

#include "las.h"
#include "constretch.h"


int  size_buf
(
    long *nli,     /* I: number of lines in the input image                 */
    long *nsi,     /* I: number of samples in the input image               */
    long *nlk,     /* I: number of lines in the kernel                      */
    long *nsk,     /* I: number of samples in the kernel                    */
    long *nlb,     /* I: number of lines in the boundary                    */
    long *nsb,     /* I: number of samples in the boundary                  */
    long *itype,   /* I: input data type                                    */
    long *otype,   /* I: output data type                                   */
    long *maxram,  /* I: maximum buffer memory size in bytes                */
    long *nlib,    /* O: number of lines in the input buffer                */
    long *nsib,    /* O: number of samples in the input buffer              */
    long *nlob,    /* O: number of lines in the output buffer               */
    long *nsob     /* O: number of samples in the output buffer             */
)
{
  long ibsize, obsize; /* number of bytes per element for input and output  */
  long status;         /* return status flag                                */
  long nonfatal = 1;   /* non-fatal error status                            */
  long maxsam;         /* number of maximum samples                         */
  long total;          /* total buffer size                                 */
  long lors = 0;       /* line or sample flag                               */


  /* Error check the input parameters.
  ------------------------------------*/
  status = err_sbuf(*nli,*nsi,*nlk,*nsk,*nlb,*nsb,*itype,*maxram);
  if (status == E_FAIL)
     return(E_FAIL);

  /* Get the number of bytes for the data types.
  ----------------------------------------------*/
  switch (*itype)
     {
     case EREAL:
         ibsize = EREALSIZE;
         break;
     case ELONG:
         ibsize = ELONGSIZE;
         break;
     case EWORD:
         ibsize = EWORDSIZE;
         break;
     case EBYTE:
         ibsize = EBYTESIZE;
     }
  switch (*otype)
     {
     case EREAL:
         obsize = EREALSIZE;
         break;
     case ELONG:
         obsize = ELONGSIZE;
         break;
     case EWORD:
         obsize = EWORDSIZE;
         break;
     case EBYTE:
         obsize = EBYTESIZE;
     }

  /* Get the number of samples that will fit in the allotted memory.
  ------------------------------------------------------------------*/
  maxsam = *maxram/(ibsize + obsize);

  /* Initialize the amount of memory used based on one image cell.
  ----------------------------------------------------------------*/
  *nlib = *nlk;
  *nsib = *nsk;
  *nlob = *nlib - *nlb*2;
  *nsob = *nsib - *nsb*2;
  total = *nlib*(*nsib);

  /* If there is not enough memory allotted for even one image cell,
     there's not much we can do.
  ------------------------------------------------------------------*/
  if (total > maxsam)
     {
     c_errmsg("Buffer too small for full kernel","filter-smallb",&nonfatal);
     c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
     return(E_FAIL);	         /* Not enough memory to fit in one kernel. */
     }

  while (total <= maxsam && lors != 2)
     {
     lors = 0;                   /* Default to incrementing lines.          */
     if ( *nlib < *nli)          /* If max lines not reached ...            */
        {
        /* If max samples not reached and number of samples less than
           number of lines, set flag to increment samples.
        -------------------------------------------------------------*/
        if (*nsib < *nsi && *nsib < *nlib)
           lors = 1;
        }
     else                        /* If max lines reached ...                */
        {
        if (*nsib < *nsi)        /* If max samples not reached ...          */
           {
	   lors = 1;		 /* Set flag to increment samples.          */
	   }
        else
	   {
	   lors = 2;             /* Max lines and samples reached.          */
	   }
        }

     if (lors == 0)
        {                        /* Increment lines.   */
        *nlib += *nlk;
        *nlob += *nlk;
        total += *nsib*(*nlk);
        }
     else if (lors == 1)
        {                        /* Increment samples. */
        *nsib += *nsk;
        *nsob += *nsk;
        total += *nlib*(*nsk);
        }

     }  /* end of while loop */

  if (total > maxsam)
     {
     if (lors == 0)
        {                        /* Decrement lines.   */
        *nlib -= *nlk;
        *nlob -= *nlk;
        }
     else if (lors == 1)
        {                        /* Decrement samples. */
        *nsib -= *nsk;
        *nsob -= *nsk;
        }
     }

  return(E_SUCC);

}  /*  end of size_buf()   */
