/*
************************************************************************
NAME: strtch

PURPOSE: Compute the new pixel gray levels.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           05/99   L. Johnson

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

for cell rows do
  for cell columns do
    for lines in cell do
      for samples in line do
        Interpolate min and max values for pixel.
        Compute new pixel value.
      end do
    end do
  end do
end do

*****************************************************************
	arguments:
	name		description
	----		-----------
	ibuff		input buffer (input)
        itype           input data type (input)
	obuff		output buffer (output)
        otype           output data type (input)
        pxminn, pxmaxn  min and max pixel values for nodes (input)
	nsbuf		number of samples in input buffer (input)
	nlc		number of rows in the cell (input)
	nsc		number of samples in the cell (input)
        ncrows, nccols  number of cell rows and columns
        totccols        total number of cell rows in frame (input)
        maxval          maximum value for contrast stretch
        minval          minimum value for contrast stretch
        nmask           number of mask values(input)
        maskval         mask values(input)

************************************************************************
*/

#include "las.h"
#include "constretch.h"


int strtch
(
    void *ibuff,         /* input buffer (input)                            */
    long itype,          /* input data type (input)                         */
    void *obuff,         /* output buffer (output)                          */
    long otype,          /* output data type (input)                        */
    double *pxminn,      /* min pixel values for nodes (input)              */
    double *pxmaxn,      /* max pixel values for nodes (input)              */
    long nsbuf,          /* number of samples in input buffer (input)       */
    long *nlc,           /* number of rows in the cell (input)              */
    long *nsc,           /* number of samples in the cell (input)           */
    long ncrows,         /* number of cell rows                             */
    long nccols,         /* number of cell columns                          */
    long totccols,       /* total number of cell rows in frame (input)      */
    double maxval,       /* maximum value for contrast stretch              */
    double minval,       /* minimum value for contrast stretch              */
    long nmask,          /* number of mask values(input)                    */
    double maskval[][2]  /* mask values(input)                              */
)
{
  unsigned char *ibadd;     /* input buffer pointers for diff. data types   */
  short *iwadd;
  long *iladd;
  float *iradd;
  unsigned char *obadd;     /* output buffer pointers for diff. data types  */
  short *owadd;
  long *oladd;
  float *oradd;
  long   ii, jj, kk, ll;    /* counters                                     */
  long   imask;             /* loop counter                                 */
  long   maskflag;          /* flag denoting whether pixel is masked        */
  long   nnode;             /* node counter                                 */
  long   nl, ns;            /* line and sample counters                     */
  long   offset;            /* address offset                               */
  double factor;            /* cell size multiplication factor              */
  double pxmin, pxmax;      /* min and max pixel values for pixels          */
  double itemp, otemp;      /* temporary storage                            */
  double xx, yy;            /* sample, line offsets for pixel centers       */


  nl = 0;
  nnode = 0;
  for (ii=0; ii<ncrows; ii++, nnode++)          /* Loop over cell rows.     */
     {
     ns = 0;
     for (jj=0; jj<nccols; jj++, nnode++)       /* Loop over cell columns.  */
        {
        factor = 1./(double)(nlc[ii]*nsc[jj]);

        /* Contrast stretch pixels in cell.
        -----------------------------------*/
        for (kk=0; kk<nlc[ii]; kk++)    /* Loop over lines in cell.   */
           {
           offset = (nl + kk)*nsbuf + ns;

           /* Must consider data types for pointer incrementation.
           -------------------------------------------------------*/
           switch (itype)            
              {
              case EBYTE:
                 ibadd = (unsigned char *)ibuff + offset;
                 break;
              case EWORD:
                 iwadd = (short *)ibuff + offset;
                 break;
              case ELONG:
                 iladd = (long *)ibuff + offset;
                 break;
              case EREAL:
                 iradd = (float *)ibuff + offset;
              }

           switch (otype)            
              {
              case EBYTE:
                 obadd = (unsigned char *)obuff + offset;
                 break;
              case EWORD:
                 owadd = (short *)obuff + offset;
                 break;
              case ELONG:
                 oladd = (long *)obuff + offset;
                 break;
              case EREAL:
                 oradd = (float *)obuff + offset;
              }
           for (ll=0; ll<nsc[jj]; ll++) /* Loop over samples in line. */
              {

              /* Cast input buffer data into double.
              --------------------------------------*/
              switch (itype)
                 {
                 case EBYTE:
                    itemp = (double)*ibadd;
                    break;
                 case EWORD:
                    itemp = (double)*iwadd;
                    break;
                 case ELONG:
                    itemp = (double)*iladd;
                    break;
                 case EREAL:
                    itemp = (double)*iradd;
                 }

              /* Check pixel to determine whether it is masked.
              -------------------------------------------------*/
              maskflag = 0;
              for (imask=0; imask<nmask; imask++)
                 {
                 if ((itemp >= maskval[imask][0]) &&
                     (itemp <= maskval[imask][1]))
                    {
                    maskflag = 1;
                    break;
                    }
                 }

              if (!maskflag)
                 {
                 /* Interpolate min and max values for pixel.
                 --------------------------------------------*/
                 yy = (double)ll + 0.5;
                 xx = (double)kk + 0.5;
                 pxmin = factor*((yy*pxminn[nnode+1] +
                         (nsc[jj] - yy)*pxminn[nnode])*(nlc[ii] - xx) +
                         (yy*pxminn[nnode+totccols+2] +
                         (nsc[jj] - yy)*pxminn[nnode+totccols+1])*xx);
                 pxmax = factor*((yy*pxmaxn[nnode+1] +
                         (nsc[jj] - yy)*pxmaxn[nnode])*(nlc[ii] - xx) +
                         (yy*pxmaxn[nnode+totccols+2] +
                         (nsc[jj] - yy)*pxmaxn[nnode+totccols+1])*xx);

                 /* Compute new pixel value.
                 ---------------------------*/
                 if ((pxmax - pxmin) > 1e-5)
                    otemp = (maxval - minval)/(pxmax - pxmin)*
                            (itemp - pxmin) + minval;
                 }
              else
                 {
                 if (itemp > maxval)
                    otemp = maxval;
                 else if (itemp < minval)
                    otemp = minval;
                 else
                    otemp = itemp;
                 }

              /* Store data in ouput buffer and increment buffer pointers.
                 Add 0.5 for all non-float types before cast truncation.
                 ---------------------------------------------------------*/
              switch (otype)
                 {
                 case EBYTE:
                    *obadd = (unsigned char)(otemp + 0.5);
                    obadd++;
                    break;
                 case EWORD:
                    *owadd = (short)(otemp + 0.5);
                    owadd++;
                    break;
                 case ELONG:
                    *oladd = (long)(otemp + 0.5);
                    oladd++;
                    break;
                 case EREAL:
                    *oradd = (float)otemp;
                    oradd++;
                 }
              switch (itype)
                 {
                 case EBYTE:
                    ibadd++;
                    break;
                 case EWORD:
                    iwadd++;
                    break;
                 case ELONG:
                    iladd++;
                    break;
                 case EREAL:
                    iradd++;
                 }

              } /* end of samples loop */

           } /* end of line loop */

        ns += nsc[jj];

        } /* end of cell column loop */

     nnode += totccols - nccols;
     nl += nlc[ii];

     } /* end of cell row loop */

  return(E_SUCC);

}    /*  end of strtch()  */
