/*******************************************************************************
NAME:                           COPY


PURPOSE:    Copies an image into another image.  It allows the input
            image(s) to be sub-windowed and sub-banded when copied to
            the output image.  The input image may be BYTE, INTEGER*2,
            INTEGER*4, or REAL*4.  The output image may be converted
            to any one of the four data types (the default is no data
            conversion).

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      N.Donahue       4/83           CSC           Original Development

  4.0      F. Irani        8/87           SAR           Rename to COPY from
                                                        FILECOPY and modify to
                                                        LAS 4.0 standards

  5.0      B. Ailts        6/88           EDC           Add UPDDR routine
                                                        Take descriptors out
                                                        of calling sequences
                                                        Replace newlas.h with
                                                        las.h
                                                        Delete tae.h
                                                        Place 'c_' in front of
                                                        las subroutines
                                                        Place subroutines in a
                                                        seperate file
 5.1      B. Ailts         10/88         EDC            Deleted flag argument
                                                        from the call to hstout
 5.2       B.Ailts        12/88         EDC/CSB         Added subroutine RNDPROC                                                        in order to round when
                                                        the parameter ROUNDOPT
                                                        is set to ROUND and
                                                        the data is being
                                                        converted from REAL to
                                                        a lower data type
 5.3     B.Ailts         04/88          EDC/CSB         Corrected problem with
                                                        rounding negative num.
 7.2     G.Schmidt       07/00          EDC/CSB         Changed to call the
                                                        c_copy_image routine

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COPY must be run under TAE.

PROJECT:                        LAS

ALGORITHM DESCRIPTION:
        The input parameters and globals are retrieved.
        Error checking is done on the input image(s) to ensure that they all
        have equal window sizes.
        Each line of the input image is copied to the corresponding line of the
        output image.
        The associated history file is created to represent the output
        image.
        A completion message is written.

ALGORITHM REFERENCES:           none
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

extern long curent;            /* current processing line                   */
extern long curcat;            /* current processing band                   */

int main ()
{
  struct PARBLK vblock;            /* TAE parameter block               */

  long bands[MAXIMG][MAXBND+1];    /* Bands to process - all images     */
  long fatal = -1;                 /* Value for fatal error messages    */
  long msgtim;                     /* Elapsed time between display msg  */
  long nbands[MAXIMG];             /* Number of bands per input image   */
  long nmrimg;                     /* Number of input images            */
  long odtype;                     /* Data type of output image         */
  long one = 1;                    /* Value of one                      */
  long std_out;                    /* LUN to standard output            */
  long totbnd;                     /* Total number of bands             */
  long window[MAXIMG][4];          /* Window for each input image       */
  int comp;                        /* Compression flag integer value
                                      TRUE = Compress output image
                                      FALSE = Don't compress output image
                                      EXT_COMP = Use external compres_opt
                                        or COMPRES1 environment variable*/
  int rtopt;                       /* Round/truncation option int value
                                      TRUE = Round the input data
                                      FALSE = Truncate the input data   */

  char hosin[MAXIMG][CMLEN];       /* Host names of input images        */
  char hosout[1][CMLEN];           /* Host name of output image         */
  char minmax[4];                  /* Min max flag                      */
  char taein[MAXIMG][CMLEN];       /* TAE names of input images         */
  char taeout[1][CMLEN];           /* TAE name of output image          */

  /* Initialization
  -----------------*/
  c_inlas (&vblock, &std_out);

  /* Get the TAE parameters
  -------------------------*/
  if (getpar (&vblock, taein, hosin, taeout, hosout, &comp, &nmrimg, window,
      bands, nbands, &totbnd, &odtype, &msgtim, minmax, &rtopt) != E_SUCC)
  {
    c_errmsg ("Error returned from getpar", "copy-fatal", &one);
    return (EXIT_FAILURE);
  }

  /* Copy the images
  ------------------*/
  if (c_copy_image (&vblock, hosin, hosout[0], comp, nmrimg, window, bands,
      nbands, totbnd, odtype, rtopt, msgtim, minmax) != E_SUCC)
  {
    c_errmsg ("Error copying the images", "copy_copy_image", &one);
    c_errmsg ("Fatal error occurred", "copy-fatal", &fatal);
    return (EXIT_FAILURE);
  }

  /*  Update history file
  -----------------------*/
  if (c_hstout (&vblock, hosout[0], hosin, &nmrimg, " ") != E_SUCC)
    c_errmsg ("Nonfatal error encountered", "copy-warn", &one);

  /* Output completion message
  ----------------------------*/
  c_complt ("COPY", taeout, &one);
  return (EXIT_SUCCESS);
}
