/******************************************************************************
NAME:   			GETPAR 

ALGORITHM:
   Retrieve the required parameters and globals from the TAE parameter block
   Parse the input image specification by calling GETIN which will validate 
    the window and band specifications  
   Checking to ensure that all the input images have equal window sizes  
   Retreive the output image name
   Retreive the output data type
   Retreive the global parameters
   Return
******************************************************************************/
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,        /* I: Pointer to TAE parameter block     */
    char taein[][CMLEN],          /* O: TAE file names of input images     */
    char hosin[][CMLEN],          /* O: Host file names of input images    */
    char taeout[][CMLEN],         /* O: TAE file name of output image      */
    char hosout[][CMLEN],         /* O: Host file name of output image     */
    int *comp,                    /* O: Compression flag                   */
    long *nmrimg,                 /* O: Number of input images             */
    long window[][4],             /* O: Window specs for each input image  */
    long bands[][MAXBND+1],       /* O: Band specs for each input image    */
    long *nbands,                 /* O: Number of bands per input image    */
    long *totbnd,                 /* O: Total number of bands to process   */
    long *odtype,                 /* O: Output data type                   */
    long *msgtim,                 /* O: Time between display messages      */
    char *minmax,                 /* O: Min max flag buffer                */
    int *rtopt                    /* O: Round/truncation option            */
)

{
  struct VARIABLE *val;         /* Struct for TAE parameters           */
  long count;                   /* Counter variable                    */
  long i;                       /* Counter -- current image            */
  long imglim;                  /* Max number of images to process     */
  long qflag[MAXIMG][2];        /* Denotes if subwindows specified     */
  long same;                    /* Denotes if images have same window  */
  long status;                  /* Function return status code         */
  long bndproc;                 /* Bands to process per IN element     */
  long nimgs;                   /* Number of images per IN element     */
  long totimg;                  /* Total number of images to process   */

  char comp_str[4];             /* Compression flag
                                   YES = compress the image
                                   NO = don't compress the image
                                   NULL = use environment variable     */
  char dtype[5];                /* Data type buffer                    */
  char rtopt_str[6];            /* Round/truncation option
                                   ROUND = Round the input data
                                   TRUNC = Truncate the input data     */

  /* Retrieve and parse input image specification 
  -----------------------------------------------*/
  imglim = MAXIMG;
  *totbnd = 0;
  *nmrimg = 0;
  totimg = 0;
  if (c_getcnt(vblock,"IN",&count) != E_SUCC)
    return(E_FAIL);

  for (i = 0; i < count; i++)
    {
    /* Parses each element of the IN parameter pulling
       the bands,image names, and windowing information
    ---------------------------------------------------*/
      status = c_getin(vblock,"IN",&i,&taein[totimg],&hosin[totimg],&nimgs,
                     &window[totimg],&bands[totimg],&nbands[totimg],&bndproc,
                     &imglim,&same,qflag);
    if (status != E_SUCC)
      return(E_FAIL);

    /* If the windowing of the IN element is a match,
       determine if the windowing across the IN
       elements match
    -------------------------------------------------*/
    if ((same != 0) && (totimg > 0))
      if ((window[totimg-1][2] != window[totimg][2]) ||
          (window[totimg-1][3] != window[totimg][3]))
        same = 0;

    /* Issue message if input image windows don't match
    ---------------------------------------------------*/
    if (same == 0)
      {
      c_lasmsg(FALSE,"copy-window",
               "Input images do not have same window sizes");
      return(E_FAIL);
      }

    /* Accumulate the total number of images
       and the total number of bands.
    ----------------------------------------*/
    totimg += nimgs;
    *totbnd += bndproc;
    imglim -= nimgs;

    if (totimg > MAXIMG)
      {
      c_lasmsg(FALSE,"copy-maximg",
               "Number of images exceeds maximum of %d",MAXIMG);
      return(E_FAIL);
      }
    }

  /* Capture the total number of images to process
  ------------------------------------------------*/
  *nmrimg = totimg;

  /* Retrieve output image name
  -----------------------------*/
  if (c_getout(vblock,"OUT",taeout,hosout) != E_SUCC)
    return(E_FAIL);

  /* Retrieve compression flag
  ----------------------------*/
  if (c_getcnt(vblock,"COMPFLG",&count) != E_SUCC)
    return(E_FAIL);

  if (count > 0)
    {
    val = p_find(vblock,"compflg");	
    strcpy(comp_str,SVAL(*val,0));
    }
  else
    strcpy(comp_str,"\0");

  if (comp_str == NULL)
    *comp = EXT_COMP;
  else if (!strcmp (comp_str, "YES"))
    *comp = TRUE;
  else
    *comp = FALSE;

  /* Retrieve output data type
  ----------------------------*/
  val = p_find(vblock,"odtype");	
  strcpy(dtype,SVAL(*val,0));
  if (strcmp(dtype,"SAME") == 0)
    *odtype = 0;
  else if (strcmp(dtype,"BYTE") == 0)
    *odtype = EBYTE;
  else if (strcmp(dtype,"I*2") == 0)
    *odtype = EWORD;
  else if (strcmp(dtype,"I*4") == 0)
    *odtype = ELONG;
  else if (strcmp(dtype,"R*4") == 0)
    *odtype = EREAL;

  /* Retrieve the round/truncate option
  -------------------------------------*/
  val = p_find(vblock,"ROUNDOPT");
  strcpy(rtopt_str,SVAL(*val,0));
  if (!strcmp (rtopt_str, "ROUND"))
    *rtopt = TRUE;
  else
    *rtopt = FALSE;

  /* Retrieve globals
  -------------------*/
  val = p_find(vblock,"$msgtim");
  *msgtim = IVAL(*val,0);
  val = p_find(vblock,"$minmax");
  strcpy(minmax,SVAL(*val,0));

  return(E_SUCC);
}
