/*******************************************************************************
NAME        	    	           APPLY_COR	

PURPOSE     APPLY_COR applies systematic corrections and resamples the data
            from the input MSS image.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
K. Zanter	 10/1985        PRFs #3319 and #3334 
   				for systematic corrections
K. Zanter        10/1986        LAS updates
K. Zanter	 12/1986	Users did not want the image deleted
			        when a fatal error was encountered.
R. McKinney      12/1988        Converted to LAS 5.0
D. Eichacker	 05/1994	Added opt[]

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
     APPLY_COR must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM:
   Systematic corrections are applied to the image data using the
method described in the user's guide.  The data is then resampled 
along-line using the specified algorithm and the result written to the 
output image.

ddr file is updated for the output image.

ALGORITHM REFERENCES 		
1.   Leo Nady of the Technique Development Section of the Technique Development
     and Applications Branch of EROS Data Center for correction algorithm

2.   Appendix G (Algorithmic Processes For Conversion of CCT-A Data to CCT-P
     Data) of the TRW contract for EDIPS (dated 6/19/80)

3.   Dan Steinwand of the Scientific Software Section of Computer Services 
     Branch of EROS Data Center for resampling methods
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

#define  FACT1		0.03125
#define  FACT2		0.015625
#define  FACT3		32.0

#define  SQUARE(x)      x * x
#define  CUBE(x)   	x * x * x
#define  TBLSIZ		528

/* The number of samples in an uncorrected MSS image from EDIPS 0.0 is 3548  */
#define  MAXSAMP        3548

/* The number of lines in an uncorrected MSS image from EDIPS 0.0 is 2400    */
#define  MAXLINE        2400

/* CORRECT only processes one input image at a time */
#define  IMGMAX         1
/*
   The following variables must be declared global for image I/O routines
*/

extern struct FDESC *  fdesc[IMGMAX+2];    /* image file descriptors used 
                                              for image I/O */
extern struct GDESC *  gdesc;	        /* group descriptor used 
                                              for image I/O       */
extern long offset1[IMGMAX*MAXBND+1];/* offsets into "thebuf" for input bands*/
extern long offset2[MAXBND+1];  /* offsets into "thebuf" for output bands    */
extern long curent;	        /* current processing line		     */
extern long curcat;             /* current processing band		     */

void apply_cor
(
    struct PARBLK *vblock, /* TAE parameter block			  */
    double lindis,             /* nominal scale of input inter-line distance */
    double pixdis,	     /* nominal scale of input inter-pixel distance*/
    double mir[],             /* MSS mirror model coefficients		   */
    double nomalt,	     /* nominal spacecraft altitude		   */
    double nomwid,	     /* nominal input image width		   */
    float boff[],            /* MSS band-to-band offsets                   */
    float sdelay[][7],      /* MSS sampling delay constants               */
    double erthrot,            /* Earth rotation parameter		   */
    long linlen[],          /* acutal line lengths			   */
    long nomlen,            /* nominal line length     		   */
    long msgtim, 	     /* elapsed time between display messages      */
    char minmax[],          /* min max flag				   */
    long bands[][MAXBND+1], /* band specification			   */
    long btot,	             /* total number of output image bands         */
    char method[],          /* resampling method		           */
    char hosin[][CMLEN],	/* pointer to input host file descriptor	*/
    char hosout[][CMLEN],        /* pointer to output host file descriptor	*/
    long bckgnd,            /* background fill value 		  	   */
    double alpha,              /* cubic convolution alpha parameter	   */
    long maxlim,            /* maximum interpolation value     	   */
    long minlim,            /* minimum interpolation value		   */
    char *tabhos    	/* pointer to table host file descriptor	*/
)
{
double   calc1;		     /* constant term used in calculating "inpix"  */
double   calc2;		     /* constant term used in calculating "inpix"  */
double   calc3;		     /* constant term used in calculating "inpix"  */
double   erthadj;	     /* term used in calculating "inpix"	   */
double   inpix;              /* pixel location in input image   	   */
double   inter;		     /* constant term used in calculating "inpix"  */
double   linadj;	     /* term used in calculating "inpix"	   */
double   mv1;		     /* term used in calculating "inpix"	   */
double   mv2;		     /* term used in calculating "inpix"	   */
double   rsamp;              /* resampled pixel value			   */
double   term1;		     /* term used in calculating "inpix"	   */
double   tot;		     /* term used in calculating "inpix"	   */

float    table [TBLSIZ];     /* weight table used for resampling           */

static long opt[3]={0,0,0};  /* optional parameter array for EOPENS	   */

long	  access;	     /* image I/O access type                      */
long      band;	             /* loop index counting the bands              */
long   	  bufsiz;            /* size of "thebuf"			   */
long      konst;	     /* constant term used in resampling	   */
long      count;	     /* loop index			           */
long      det;               /* current detector number			   */
long      dtype;	     /* data type				   */
long      flag = ALLCAT;     /* process all bands at once 		   */
long      fill[5];           /* fill offset   	                           */
long      i;		     /* loop index			           */
long      index;	     /* loop index				   */
long      line;	             /* loop index counting image lines            */
long      lowlim;            /* lower limit			  	   */
long	  nl;		     /* number of image lines                      */
long	  ns;		     /* number of image samples                    */
long      nonfatal = 1;      /* nonfatal error flag		           */
long      fatal = -1;        /* fatal error flag		           */
long      odtype;	     /* output image data type		           */
long	  one = 1;	     /* value of one                               */
long      outsamp;           /* pixel location in output image		   */
long      pixloc;            /* pixel location of beginning of kernel	   */   
long      samp;	             /* loop index counting samples per line       */
long	  status;	     /* function return status code                */
long      tabsiz;	     /* size of resampling weight table		   */
long      tblloc;            /* location in weight table		   */ 
long      zero = 0;          /* value of zero				   */

double	linc = 0.0,
	sinc = 0.0;

long	window[1][4],
	nbands[1][2],
	cflag = NOCOMB;

double minval [MAXBND];       /* array of minimum pixel values		   */
double maxval [MAXBND];       /* array of maximum pixel values		   */

unsigned char *thebuf;	      /* image input buffer			   */

/* Initialize detector number */
det = 1;

/* Initialize the fill offsets */
fill[1] = 75;
fill[2] = 73;
fill[3] = 71;
fill[4] = 69;

/* If the user wants the min max pixel values computed for each band,         */
/* initialize the array of min max values				      */
if (strcmp(minmax,"YES") == 0)
	{
	dtype = EBYTE;
	c_inmnmx(&dtype,&btot,minval,maxval);
	}

/* Make the resampling weight table */
make_table(table,&tabsiz,method,alpha,tabhos);

/* Open the input image for sequential read access */
access = IREAD;
nl = zero;
ns = zero;
odtype = zero;
status = c_eopens(&fdesc[0],hosin[0],bands[0],&one,&one,&nl,&ns,
     		    offset1,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","correct-fatal",&fatal);

if (odtype != EBYTE)
   c_errmsg("Input image is not BYTE","correct-notbyte",
		&fatal);

if ((nl != MAXLINE) || (ns != MAXSAMP))
   c_errmsg("Input image is not a full-sized MSS scene","correct-sizerr",
		&fatal);

/* Open the output image for sequential write access */
access = IWRITE;
status = c_eopens(&fdesc[1],hosout[0],&btot,&one,&one,&nl,&ns,offset2,
         &access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","correct-fatal",&fatal);

/* Allocate the I/O buffer  */
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","correct-alloc",&nonfatal);
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","correct-close",&nonfatal);
   c_clean(vblock,"correct",hosout,&one);
   }
    

/* Group the input image and the output image   */
fdesc[2] = 0;
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","correct-close",&nonfatal);
   c_clean(vblock,"correct",hosout,&one);
   }

/* Calculate the constants in the correction formula and resampling */
calc1 = nomlen * nomalt;
calc2 = 2 * mir[2];
calc3 = 3 * mir[3];
inter = (lindis / pixdis) * erthrot;
konst = 1 - (tabsiz / 2);
lowlim = (tabsiz / 2) - 1;
/*
printf("nomlen %d\n",nomlen);
printf("nomalt %G\n",nomalt);
printf("mir[0] %G\n",mir[0]);
printf("mir[1] %G\n",mir[1]);
printf("mir[2] %G\n",mir[2]);
printf("mir[3] %G\n",mir[3]);
printf("calc1 %G\n",calc1);
printf("calc2 %G\n",calc2);
printf("calc3 %G\n",calc3);
printf("lindis %G\n",lindis);
printf("pixdis %G\n",pixdis);
printf("erthrot %G\n",erthrot);
printf("inter %G\n",inter);
printf("konst %d\n",konst);
printf("lowlim %d\n",lowlim);
*/

/* Initialize and start the display message utility */
c_inittm("CORRECT","Processed","line","bands"," ",&nl,&btot,&msgtim,&flag);
curent = 1;
c_startm();

/* Process all the image bands */
for (line = 0; line < MAXLINE; line++)
    {
    /* Read in the input image line */
    if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
	    c_errmsg("Error closing images","correct-close",&nonfatal);
	c_clean(vblock,"correct",hosout,&one);
	}

    linadj = (double)linlen[line] / (double)nomlen;
    erthadj = inter * (line - MAXLINE);

    outsamp = 0;
    for (samp = 0; samp < MAXSAMP; samp++)
        {
        outsamp++;
        mv1 = mir[1] + (calc2 * outsamp) + (calc3 * SQUARE(outsamp));
        term1 = linadj * (outsamp + erthadj);

        for (band = 0; band < btot; band++)
	    {
            index = bands[0][band];
	    mv2 = (calc1 * boff[index])/nomwid;
	    tot = term1 + boff[index] - (mv2 * mv1) + sdelay[index][det];
 
            /* Inpix is the result of all these calculations    */
            /* It is the pixel location in the input space for  */
            /* the present output sample                        */

            inpix = tot + *(fill + index);

            /* Adjust inpix for the zero relative array */
            inpix = inpix - 1;

            /* Resampling */
            if ((inpix < lowlim) || (inpix > linlen[line]))
		{
                *(thebuf + *(offset2 + band) + samp) = (unsigned char) bckgnd;
		}
 	     else 
		{
		pixloc = (long) inpix +  konst;
                tblloc = tabsiz * (long)(FACT3 * (FACT2 + inpix -
  			 (long) inpix));

     	        for (count = 0, rsamp = 0; count < tabsiz; count++)
    		   rsamp += *(table + tblloc + count) *
    		   	     *(thebuf + *(offset1 + band) + (pixloc + count));

 		if (rsamp < minlim) 
  		   *(thebuf + *(offset2 + band) + samp ) = (unsigned char) minlim;
	        else if (rsamp > maxlim)
 		   *(thebuf + *(offset2 + band) + samp ) = (unsigned char) maxlim;
 		else
	          *(thebuf + *(offset2 + band) + samp ) = (unsigned char) rsamp;
  	      	}
	    }
        }
     
    if (strcmp(minmax,"YES") == 0)
	{
	for (i = 0; i < btot; i++)
	     status = c_pxmnmx(thebuf + (*(offset2 + i)),&ns,&minval[i],
     		       &maxval[i], &odtype);
        }

    if (det == 6)
        det = 1;
    else
        det++;

    curent ++;
    }

/* Final c_estep */
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","correct-close",&nonfatal);
      c_clean(vblock,"correct",hosout,&one);
      }

/* Disable the display message utility */
c_stoptm();

/* Close the images in the group */
status = c_egclse(&gdesc);
if (status != E_SUCC) 
   {
   c_errmsg("Nonfatal error encountered--Image data saved",
	    "correct-warn",&nonfatal);
   c_clean(vblock,"correct",hosout,&one);
   }

/*
  Update the DDR information based on the input file
*/
window[0][0] = one;
window[0][1] = one;
window[0][2] = nl;
window[0][3] = ns;
nbands[0][0] = btot;
status = c_upddr(vblock,hosin,hosout[0],&one,window,bands,nbands[0],&linc,&sinc,
		 &btot,&cflag);
if (status != E_SUCC)
   {
   c_errmsg("Nonfatal error encountered--Image data saved",
	    "correct-warn",&nonfatal);
   c_clean(vblock,"correct",hosout,&one);
   }

/* update the ddr with minmax, if appropriate */
status = c_svmnmx(hosout[0],&btot,minval,maxval,minmax);
if (status != E_SUCC)
   {
   c_errmsg("Nonfatal error encountered--Image data saved",
	    "correct-warn",&nonfatal);
   c_clean(vblock,"correct",hosout,&one);
   }
/*
  Free the buffer
*/
free(thebuf);

return;
}
