/******************************************************************************
NAME        	    	            CORRECT	

PURPOSE    CORRECT performs along-line systematic corrections on a full-sized,
	   non-geometrically corrected Landsat MSS scene.  The image data is
	   corrected for the following high frequency systematic errors:
                1)  Line length adjustment
	        2)  Mirror scan non-linearity
	        3)  Band-to-band offsets
                4)  Earth rotation
	        5)  Detector-to-detector sampling delay
           
  	   After the systematic corrections are applied, the data is 
           resampled along-line using the specified algorithm.
            
EXTERNAL ASSOCIATIONS		

    The input to CORRECT is a MSS image and its associated corrections file
    which was created by LAS function EDIPSIN. 

file REFERENCES			

1.    Image's associated corrections file     		ASCII text file

      This file is created by LAS function EDIPSIN
      with the TAE name "IMAGE NAME;EDIPS.COR"

      The file format is as follows:
   	Landsat mission number		     	     	integer
	Nominal line length				integer
	Nominal scale of input inter-pixel distance     double precision
        Nominal scale of input inter-line distance      double precision
        Nominal spacecraft altitude			double precision
	Nominal input image width			double precision
        4 MSS mirror model coefficients			double precision
 	26 MSS sampling delay constants			floating point
	5 MSS band-to-band offsets			floating point
        Scene center latitude				double precision
	Earth rotation parameter			floating point
	2400 actual line lengths			integer
	Band specification				integer


2.    The file containing the user-entered 	        ASCII text file
      resampling weight table.
   
      This file is referenced only for the -TABLE subcommand.
      It is created by a text editor or a user-supplied table
      generation function.  Its extension is "rwt". 

	Horizontal kernel size		(integer; maximum is 16)
	Horizontal resampling table	(A table consisting of 33 rows 
					by HORIZONTAL KERNEL SIZE
					columns of floating point 
					values.  Each row of values 
					must add to one.)

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
K. Zanter	 10/1985        PRFs #3319 and #3334 
   				for systematic corrections
K. Zanter        10/1986        LAS updates
K. Zanter	 12/1986	Users did not want the image deleted
			        when a fatal error was encountered.
R. McKinney      12/1988        Converted to LAS 5.0
B. Davis	 04/1992	1. Adjusted global fdesc declaration from
				   IMGMAX+1 to IMGMAX+2. This is the actual
				   requirment, and all of the subroutines had
				   this declaration.  This caused memory address
				   conflicts on sun sparc's, but only
				   intermittantly.
				   Also eliminated the global extern
				   declarations from those subroutines that were
				   not using them.
				2. Changed the scanf formats in get_info from
				   e to lf. Very large values were getting lost.
				3. Added a low2up call in getpar for minmax
				   flag. Changed subsequent strcmp's of minmax
				   to upper case YES, this is ultimately what
				   c_svmnmx wants.
				4. Fixed the call to svmnmx, passing the actual
				   minmax string. Previously, a dummy char with
				   no contents was passed, therefore the ddr
				   was never updated.
				5. Above 4 fixes tested for extreme data on:
					sg2	yes
					dg2	yes
					sunm	yes
					pn9050	yes

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
     CORRECT must be run under TAE.

PROJECT   	         LAS		                

ALGORITHM:
   The values of the TAE input parameters and the global variables
are retrieved.  The existence of the associated file containing the
ancillary information needed to apply systematic corrections is checked.
This file is then opened and the values are retrieved.  Next, the weight
table to be used in resampling is generated when using the -CUBIC or
-BILINEAR subcommands.  If the -TABLE subcommand is specified, the weight
table is read from INFILE, the disk file containing the table of user-
entered resampling weights.
 
   Systematic corrections are applied to the image data using the
method described in the user's guide.  The data is then resampled 
along-line using the specified algorithm and the result written to the 
output image.

History file is created for the output image.

ALGORITHM REFERENCES 		
1.   Leo Nady of the Technique Development Section of the Technique Development
     and Applications Branch of EROS Data Center for correction algorithm

2.   Appendix G (Algorithmic Processes For Conversion of CCT-A Data to CCT-P
     Data) of the TRW contract for EDIPS (dated 6/19/80)

3.   Dan Steinwand of the Scientific Software Section of Computer Services 
     Branch of EROS Data Center for resampling methods

*******************************************************************************/

#include <desc.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"


#define  SQUARE(x)      x * x
#define  CUBE(x)   	x * x * x

/* The following are factors used in resampling */
#define  FACT1          0.03125
#define  FACT2          0.015625
#define  FACT3          32.0

#define  MATSIZ         33
#define  TBLSIZ         528

/* The number of samples in an uncorrected MSS image from EDIPS 0.0 is 3548  */
#define  MAXSAMP        3548

/* The number of lines in an uncorrected MSS image from EDIPS 0.0 is 2400    */
#define  MAXLINE        2400

/* CORRECT only processes one input image at a time */
#define  IMGMAX         1

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC *  fdesc[IMGMAX+2]; /* image file descriptors used for image I/O */
struct GDESC *  gdesc;		     /* group descriptor used for image I/O       */
long offset1[IMGMAX*MAXBND+1];  /* offsets into "thebuf" for input bands     */
long offset2[MAXBND+1];         /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int main
(void)

{

struct PARBLK vblock;        /* TAE parameter block		           */

double   lat;		     /* scene center latitude			   */
double   lindis;             /* nominal scale of input inter-line distance */
double   mir [4];            /* MSS mirror model coefficients		   */
double   nomalt;	     /* nominal spacecraft altitude		   */
double   nomwid;	     /* nominal input image width		   */
double   pixdis;	     /* nominal scale of input inter-pixel distance*/

float    alpha;              /* cubic convolution alpha parameter          */
float    boff [7];           /* MSS band-to-band offsets                   */
float    erthrot;            /* earth rotation parameter		   */
float    sdelay [5] [7];     /* MSS sampling delay constants               */

long	bands[IMGMAX][MAXBND+1];/* band specification 			   */  
long    bckgnd;              /* background fill value			   */
long    btot;	  	     /* total number of output image bands	   */
long    linlen [MAXLINE];    /* actual line lengths			   */
long    maxlim;              /* maximum interpolation value     	   */
long    minlim;              /* minimum interpolation value		   */
long	msgtim;		     /* elapsed time between display messages      */
long    nomlen;              /* nominal line length			   */
long    nonfatal = 1;        /* nonfatal error flag			   */
long	one = 1;	     /* value of one			           */
long	status;		     /* function return status code                */
long	std_out;	     /* LUN for standard output			   */

char    corhos[CMLEN];       /* buffer storage for "corhos"                */
char	hosin[IMGMAX][CMLEN];/* buffer storage for "hosin"                 */
char	hosout[1][CMLEN];       /* buffer storage for "hosout"                */
char    method [9];	     /* resampling method		           */
char	minmax[4];	     /* min max flag                               */
char    tabhos[CMLEN];       /* buffer storage for "tabhos"                */
char    taein[IMGMAX][CMLEN];/* buffer storage for "taein"		   */
char	taeout[1][CMLEN];       /* buffer storage for "taeout"                */

/* Initialization  */
c_inlas(&vblock,&std_out);

/* Get user-specified parameters and global variable values from TAE */
getpar(&vblock,taein,hosin,taeout,hosout,bands,&btot,&msgtim,minmax,
       method,&bckgnd,&alpha,&maxlim,&minlim,tabhos,corhos);

/* Get the ancillary information stored in the associated correction file */
get_info(&lindis,&pixdis,mir,&nomalt,&nomwid,boff,sdelay,
         &erthrot,linlen,&nomlen,corhos,&lat);

/* Process the image data */
apply_cor (&vblock,lindis,pixdis,mir,nomalt,nomwid,boff,sdelay,erthrot,linlen,
           nomlen,msgtim,minmax,bands,btot,method,hosin,hosout,
           bckgnd,alpha,maxlim,minlim,tabhos);

/* Create the history file for the output image      */
status = c_hstout(&vblock,hosout[0],hosin,&one," ");
if (status != E_SUCC) 
   c_errmsg("Nonfatal error encountered--Image data saved","correct-warn"
		,&nonfatal);

/* Write a completion message to the user            */
c_complt("CORRECT",taeout,&one);

return EXIT_SUCCESS;
}		
