/*******************************************************************************
NAME        	    	           GET_INFO	

PURPOSE     GET_INFO gets the ancillary information stored in the input image's
            associated corrections file.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
K. Zanter	 10/1985        PRFs #3319 and #3334 
   				for systematic corrections
K. Zanter        10/1986        LAS updates
K. Zanter	 12/1986	Users did not want the image deleted
			        when a fatal error was encountered.
R. McKinney      12/1988        Converted to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
     GET_INFO must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM:
This file is then opened and the values are retrieved.  Next, the weight
table to be used in resampling is generated when using the -CUBIC or
-BILINEAR subcommands.  If the -TABLE subcommand is specified, the weight
table is read from INFILE, the disk file containing the table of user-
entered resampling weights.
 

ALGORITHM REFERENCES:
1.   Leo Nady of the Technique Development Section of the Technique Development
     and Applications Branch of EROS Data Center for correction algorithm

2.   Appendix G (Algorithmic Processes For Conversion of CCT-A Data to CCT-P
     Data) of the TRW contract for EDIPS (dated 6/19/80)

3.   Dan Steinwand of the Scientific Software Section of Computer Services 
     Branch of EROS Data Center for resampling methods
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"


/* The number of lines in an uncorrected MSS image from EDIPS 0.0 is 2400    */
#define  MAXLINE        2400

void get_info
(
    double *lindis,	    /* nominal scale of input inter-line distance        */
    double *pixdis,     /* nominal scale of input inter-pixel distance       */
    double mir[],      /* MSS mirror model coefficients  	  	             */
    double *nomalt,	    /* nominal spacecraft altitude                       */
    double *nomwid,	    /* nominal input image width		             */
    float boff[],     /* adjusted array of band-to-band offsets              */
    float sdelay[][7], /* ajusted array of MSS sampling delay constants      */
    float *erthrot,    /* earth rotation parameter                           */
    long linlen[],     /* actual line lengths			             */
    long *nomlen,       /* nominal line length      		    	     */
    char *corhos,      /* pointer to correction file host name descriptor */
    double *lat        /* scene center latitude				     */
)

{

FILE *fp;      	      /* corrections file pointer	                     */			
float  adelay [4] [6];/* all MSS sampling delay constants     		     */
float  alloff [5];    /* all MSS band-to-band offset values                  */
float  eighta;	      /* band 8A value -- not presently used		     */
float  eightb;	      /* band 8B value -- not presently used  		     */

long  blist [5];      /* band specification from corrections file            */
long  i;              /* loop index					     */
long  j;              /* loop index					     */
long  fatal = -1;     /* fatal error flag		   	             */
long  temp;	      /* temporary variable				     */

  
/* Open associated corrections file */
fp = fopen(corhos,"r");
if (fp == NULL)
   c_errmsg("Error opening correction file","correct-openerr",&fatal);

/* Retrieve Landsat mission number */
/* Not currently used	           */
fscanf(fp,"%d",&temp);

/* Retrieve nominal line length */
fscanf(fp,"%d",nomlen);

/* Retrieve nominal scale of input inter-pixel   */
/* and inter-line distance in meters per pixel */
fscanf(fp,"%lf",pixdis);
fscanf(fp,"%lf",lindis);

/* Retrieve nominal altitude and nominal width */
fscanf(fp,"%lf",nomalt);
fscanf(fp,"%lf",nomwid);

/* Retrieve the 4 mirror model coefficients */
for (i=0; i<4; i++)
	fscanf(fp,"%lf",&mir[i]);
	
/* Retrieve the 24 sampling delay constanst */
/* Bands 1 through 4, with 6 dectors for each band */
for (i=0; i<4; i++)
	for (j = 0; j < 6; j++)
		fscanf(fp,"%f",&adelay[i][j]);

/* Skip over the delay constants for 8A and 8B */
fscanf(fp,"%f",&eighta);
fscanf(fp,"%f",&eightb);

/* Retrieve band-to-band offsets */
for (i=0; i < 5; i++)
	fscanf(fp,"%f",&alloff[i]);

/* Retrieve scene center latitude */
fscanf(fp,"%lf",lat);

/* Retrieve earth rotation parameter */
fscanf(fp,"%f",erthrot);

/*
printf("pixdis %G\n",*pixdis);
printf("lindis %G\n",*lindis);
printf("nomalt %G\n",*nomalt);
printf("nomwid %G\n",*nomwid);
printf("lat %G\n",*lat);
printf("erthrot %G\n",*erthrot);
*/

/* Retrieve actual line lengths */
for (i=0; i< MAXLINE; i++)
	{
	fscanf(fp,"%d %d",&temp,&linlen[i]);
        if (i+1 != temp)
	    {
	    fclose(fp);
	    c_errmsg("Error reading correction file","correct-rderr",&fatal);
	    }
      	}

/* Retrieve band specification used in EDIPSIN */
for (i=0; i<5; i++)
	fscanf(fp,"%d",&blist[i]);

/* Since the input image may have been sub-banded when it was created in      */
/* EDIPSIN, the band offsets and sampling delay constants must be adjusted.   */
/* Example:  The image on the EDIPS tape had four bands, but the user only    */
/* specified bands 2 and 4 in EDIPSIN, so the output disk image has two bands.*/
/* Now, when this two-band image is input to CORRECT, the calculation made    */
/* for systematic corrections must use band offset 2 for band 1 of the image  */
/* and band offset 4 for band 2 of the image.			              */

/* Note: When "boff" and "sdelay" are ajusted, they are purposely begun       */
/* at subscript 1, rather than 0, so that indices will not have to be         */
/* ajusted repeatedly in later use.  					      */

/* Adjust band-to-band offset array */
/* The band-to-band offsets are in respect to Band 1 */
/* Therefore, the offset for Band 1 must be set to 0 */
i = 0;
if ( blist[0] == 1)
   {
   boff[1] = 0;
   i++;
   }

while (blist[i] != 0)
	{
	boff[i+1] = alloff[blist[i] - 2];
	i++;
	}

/* Adjust sampling delay constants array */
i = 0;
while (blist[i] != 0)
      {
      for (j = 0; j < 6; j++)
	  sdelay[i+1][j+1] = adelay[blist[i] - 1][j];
      i++;
      }

fclose(fp);
return;
}
