/****************************************************************************
NAME:                           accept

PURPOSE:  Prints correlation results and prompts user if confirm is set

PROGRAM HISTORY:
VERSION         DATE    AUTHOR
-------         ----    ------  
  1.0           8/92    D. Steinwand

RETURN VALUES:  TRUE      the point has been accepted
                FALSE     the point has been rejected

CHANGES:

$Log: accept.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.4  2001/06/13 15:36:33  ldjohn
CR854 Code review: Longs to ints.  Passed structures to pointers.  Message
      length 80 -> 120. #defines for active flag values.

Revision 1.1.2.3  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 1.1.2.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <string.h>
#include "las.h"
#include "correlate.h"
#include "geompak.h"


int accept
(
    const int confirm,         /* I: Boolean--confirm mode                 */
    const struct TPLDATA *data /* I: TPL data record                       */
)
{
  TEXT mess[120];         /* TAE message buffer                           */
  CODE term;              /* Terminator code from TAE terminal I/O        */
  char answer[2];         /* Buffer containing answer to prompt           */

  
  /* If no confirm mode, return TRUE
     -------------------------------*/
  if (!confirm)
      return TRUE;

  /* Display results to the terminal
     -------------------------------*/
  t_write(" ", T_STDCC);
  t_write("Correlation Results:", T_STDCC);
  t_write(" ", T_STDCC);
  sprintf(mess,
          "  Tie point reference image coords (line, sample):  %9.2f %9.2f",
          data->ref_coord[0], data->ref_coord[1]);
  t_write(mess, T_STDCC);
  sprintf(mess,
          "  Tie point search image coords (line, sample):     %9.2f %9.2f",
          data->sea_coord[0], data->sea_coord[1]);
  t_write(mess, T_STDCC);
  if (data->active == TPL_STATE_OK_AUTO)
  {
      sprintf(mess, 
            "  Estimated errors in search coordinates:           %9.2f %9.2f",
            data->rms_error[0], data->rms_error[1]);
      t_write(mess, T_STDCC);
      sprintf(mess,
              "  Correlation strength:                             %9.2f",
              data->strength);
      t_write(mess, T_STDCC);
      sprintf(mess,
              "  Displacement from nominal:                        %9.2f",
              data->displacement);
      t_write(mess, T_STDCC);
  }
  t_write(" ", T_STDCC);
  t_write("Search Parameters:", T_STDCC);
  t_write(" ", T_STDCC);
  sprintf(mess,
          "  Nominal search coordinates (line, sample):        %9.2f %9.2f", 
          data->nominal[0], data->nominal[1]);
  t_write(mess, T_STDCC);
  sprintf(mess,
       "  Requested reference subimage size (height, width):       %ld   %ld",
       data->req_size[0], data->req_size[1]);
  t_write(mess, T_STDCC);
  sprintf(mess, 
       "  Requested search offsets (above/below, left/right):      %ld   %ld",
       data->offsets[0], data->offsets[1]);
  t_write(mess, T_STDCC);
  sprintf(mess, "  Actual reference subimage window:  %9ld %9ld %4ld %4ld",
          data->location[0], data->location[1], data->ref_size[0],
          data->ref_size[1]);
  t_write(mess, T_STDCC);
  sprintf(mess, "  Actual search subimage window:     %9ld %9ld %4ld %4ld",
          data->chip_loc[0], data->chip_loc[1], data->sea_size[0],
          data->sea_size[1]);
  t_write(mess, T_STDCC);
  t_write(" ", T_STDCC);
  t_write("Record this tie point in the tie point location file [Y]/N ? ", 
          T_NULL);

  /* Loop until satisfactory input is given
     --------------------------------------*/
  for (;;)
  {
      t_read(mess, &term);
      if (strlen(mess) == 0) 
      {
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          return TRUE;
      }
      sscanf(mess, "%s", answer);
      if ((strcmp(answer, "Y") == 0) || (strcmp(answer, "y") == 0)) 
      {
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          return TRUE;
      }
      if ((strcmp(answer, "N") == 0) || (strcmp(answer, "n") == 0)) 
      {
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          t_write(" ", T_STDCC);
          return FALSE;
      }
      t_write("Please enter [Y] or N >  ", T_NULL);
  }
}
