/****************************************************************************
NAME:       ADJUST_CHIP_COORDINATES

PURPOSE:    Adjust coordinates if chips were used.

REFERENCES:

CHANGES:

$Log: adjust_chip_coordinates.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.3  2001/06/13 15:43:58  ldjohn
R854 Code review: Passed structures to pointers.  #defines for active flag
     values.

Revision 1.1.2.2  2001/04/23 18:02:01  ldjohn
CR854 Typo correction.

Revision 1.1.2.1  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "correlate.h"


void adjust_chip_coordinates
(
    const struct CORRIMG *ref,       /* I: Reference image file parameters  */
    const struct CORRIMG *srch,      /* I: Search image parameters          */
    const struct MTPDATA *mtp_data,  /* I: Merged tie point data record     */
    const struct TPLHEAD *tpl_header,/* I: TPL file header record           */
    struct TPLDATA *tpl_data         /* O: Tie point location data record   */
)
{
  const long one = 1;              /* Constant of 1                         */

  double chipcenter0, chipcenter1; /* Temporary storage for chip center     */


  if (tpl_data->active != TPL_STATE_BAD_PARAM)
  {
      if (strcmp(tpl_header->ref_mode, "CHIP-TEM") == 0)
      {
          /* Image-to-image registration chips used (i.e. AVHRR or
             Landsat GCPLIB).  mtp_data.geo_coord is ignored and the
             nominal coordinates are equal to the full-scene reference
             coordinates.  The computed search image coordinates are
             identical to full scene correlations...
             ---------------------------------------------------------*/
          chipcenter0 = tpl_data->ref_coord[0];
          chipcenter1 = tpl_data->ref_coord[1];
          tpl_data->ref_coord[0] = tpl_data->nominal[0];
          tpl_data->ref_coord[1] = tpl_data->nominal[1];
          tpl_data->nominal[0] = chipcenter0;
          tpl_data->nominal[1] = chipcenter1;
      }

      if (strcmp(tpl_header->ref_mode, "CHIP-FUL") == 0)
      {
          /* Reference chips establish a geographic registration.
             mtp_data.geo_coord contains a projection coordinate of the
             ref tie point in the projection space defined by the mtp
             header record.  This projection coordinate must be
             converted to output space image coordinates.  The search
             chip, if used, is treated as if the chip mode was
             "CHIP-TEM", using the master coordinate field.
             ----------------------------------------------------------*/
          tpl_data->ref_coord[0] = (tpl_header->corners[0] -
                             mtp_data->geo_coord[0])/tpl_header->pdist[0] + 1;
          tpl_data->ref_coord[1] = (mtp_data->geo_coord[1] -
                             tpl_header->corners[1])/tpl_header->pdist[1] + 1;
          tpl_data->sea_coord[0] += srch->master_line - 1;
          tpl_data->sea_coord[1] += srch->master_samp - 1;
      }

      if (strcmp(tpl_header->ref_mode, "CHIP-PRE") == 0)
      {
          /* Reference chips are treated as they are when the mode is
             "CHIP-FUL" if valid projection and framing information
             exists in the mtp/tpl file header (geo_coord is also
             used...); or like "CHIP-TEM" if the proj & frame info is
             not valid in the mtp/tpl file header.  The search chip
             must have a user-defined coordinate system (in pixels),
             defining the rotation and scaling of the chip.
             --------------------------------------------------------*/
          if ((tpl_header->proj_valid == GEO_VALID) && (ref->chip_flag))
          {
              tpl_data->ref_coord[0] = (tpl_header->corners[0] -
                             mtp_data->geo_coord[0])/tpl_header->pdist[0] + 1;
              tpl_data->ref_coord[1] = (mtp_data->geo_coord[1] -
                             tpl_header->corners[1])/tpl_header->pdist[1] + 1;
          }
          else
          {
              tpl_data->ref_coord[0] += ref->master_line - 1;
              tpl_data->ref_coord[1] += ref->master_samp - 1;
          }

          tpl_data->sea_coord[0] = c_eval(&one, srch->coef_y,
                        &(tpl_data->sea_coord[1]), &(tpl_data->sea_coord[0]));
          tpl_data->sea_coord[0] *= -1.0;
          tpl_data->sea_coord[1] = c_eval(&one, srch->coef_x,
                        &(tpl_data->sea_coord[1]), &(tpl_data->sea_coord[0]));
      }
  }
}
