/****************************************************************************
NAME:                           COPY_POINTS

PURPOSE:    Open input tie point file and copy the points to a specified
            output file.

REFERENCES:

RETURN VALUES:  E_SUCC  successful completion
                E_FAIL  unsuccessful completion

                Note: If this routine returns successfully, the file
                      out_tpl must be closed by the caller.
                
CHANGES:

$Log: copy_points.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.4  2001/06/13 15:46:30  ldjohn
CR854 Code review: "== E_FAIL" -> "!= E_SUCC".  Improved error messages.

Revision 1.1.2.3  2001/04/23 18:13:01  ldjohn
CR854 Modified to handle grey-edge correlation.

Revision 1.1.2.2  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 1.1.2.1  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "correlate.h"


int copy_points
(
    char *tpl,                    /* I: Input Tie Point Location file name  */
    char *out,                    /* I: Output Tie Point Location file name */
    const int corrmeth,           /* I: Method of correlation used          */
    struct PTDESC **out_tpl,      /* O: Output tiepoint location file desc  */
    struct TPLHEAD *tpl_header,   /* O: TPL file header record              */
    struct TPLDATA *tpl_data,     /* O: Tie point location data record      */
    struct CORRPAR *corr          /* O: Correlation parameters              */
)
{
  struct PTDESC *in_tpl;         /* Input tie point location file desc      */

  long acc;                      /* File access mode                        */
  long ptyp;                     /* Type of tie point file                  */
  long status;                   /* Return status variable                  */

  char corrstr[7];               /* Correlation method; similar to
                                    tpl_header->corr_type                   */

  
  /* Open the input tie point file and read the header. */
  acc = IREAD;
  ptyp = TPL;
  if (c_ptopen(&in_tpl, tpl, &ptyp, &acc, tpl_header) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-open", "Unable to open %s for reading", tpl);
      return E_FAIL;
  }

  /* Open the output tie point file and write the header. */
  acc = IWRITE;
  ptyp = TPL;
  if (c_ptopen(out_tpl, out, &ptyp, &acc, tpl_header) != E_SUCC)
  {
      c_ptclse(&in_tpl);
      c_lasmsg(FALSE, "correlate-open", "Unable to open %s for writing", out);
      return E_FAIL;
  }

  /* Copy the tie points from the input file to the output file.
     -----------------------------------------------------------*/
  for (;;)
  {
      /* Read a tie point. */
      status = c_ptio(&in_tpl, tpl_data);
      if (status == E_EOF)
          break;
      else if (status != E_SUCC)
      {
          c_ptclse(&in_tpl);
          c_ptclse(out_tpl);
          c_lasmsg(FALSE, "correlate-ptio", "Error reading tie point from %s",
                   tpl);
          return E_FAIL;
      }

      /* Check the tie point ID. */
      if (!c_ckptid(tpl_data->pt_id))
      {
          c_ptclse(&in_tpl);
          c_ptclse(out_tpl);
          c_lasmsg(FALSE, "correlate-ptid",
                   "Duplicate point identifiers exist in INTL");
          return E_FAIL;
      }

      /* Write the tie point to the output file. */
      if (c_ptio(out_tpl, tpl_data) != E_SUCC)
      {
          c_ptclse(&in_tpl);
          c_ptclse(out_tpl);
          c_lasmsg(FALSE, "correlate-ptio", "Error writing tie point to %s",
                   out);
          return E_FAIL;
      }
  }

  /* Close the input tie point file. */
  c_ptclse(&in_tpl);

  corr->mincorr = tpl_header->min_str;
  corr->maxdiff = tpl_header->max_dis;
  corr->edgeval = tpl_header->edge_thres;

  if (strcmp(tpl_header->peak_method, "PARABOLOID") == 0)
      corr->fitmeth = PARAB;
  else if (strcmp(tpl_header->peak_method, "GAUSSIAN") == 0)
      corr->fitmeth = GAUSS;
  else if (strcmp(tpl_header->peak_method, "RECIPROCAL") == 0)
      corr->fitmeth = RECIP;
  else if (strcmp(tpl_header->peak_method, "NEAR_INT") == 0)
      corr->fitmeth = INTEGER;
  else
  {
      c_ptclse(out_tpl);
      c_lasmsg(FALSE, "correlate-header",
               "TPL file header peak method %s is invalid",
               tpl_header->peak_method);
      return E_FAIL;
  }

  /* The correlation method to be applied to new tie points must agree
     with the method specified in the input TPL file header. */
  if ((strcmp(tpl_header->corr_type, "EDGE") == 0 && corrmeth != EDGE_CORR) ||
      (strcmp(tpl_header->corr_type, "GREY") == 0 && corrmeth != GREY_CORR) ||
      (strcmp(tpl_header->corr_type, "GREDGE") == 0 &&
                                                 corrmeth != GREYEDGE_CORR) ||
      (strcmp(tpl_header->corr_type, "PHASE") == 0 && corrmeth != PHASE_CORR))
  {
      c_ptclse(out_tpl);

      /* Convert correlation method to string. */
      switch (corrmeth)
      {
          case EDGE_CORR:
              strcpy(corrstr, "EDGE");
              break;
          case GREY_CORR:
              strcpy(corrstr, "GREY");
              break;
          case PHASE_CORR:
              strcpy(corrstr, "PHASE");
              break;
          case GREYEDGE_CORR:
              strcpy(corrstr, "GREDGE");
              break;
          default:
              strcpy(corrstr, "NOCORR");
      }
      c_lasmsg(FALSE, "correlate-header",
               "Correlation methods do not match:  Current = %s, INTL = %s",
               corrstr, tpl_header->corr_type);
      return E_FAIL;
  }

  return E_SUCC;
}
