/****************************************************************************
NAME:                           CORRELATE

PURPOSE:  Performs image-to-image correlation

VERSION  DATE   AUTHOR   
-------  ----   ------    
  1.0    8/92   D. Steinwand           Conversion to C
  2.0    2/93   T. Mittan              Released code
  3.0    7/94   T. Mittan              Released ADAPS version to LAS
  4.0    1/95   T. Mittan              Modified to use any number of tiepoints 
  5.0    10/99  T. Ratliff             Removed the 2gig and prototyped the
                                       the functions.

REFERENCES:

CHANGES:

$Log: correlate.c,v $
Revision 6.9  2010/09/23 15:35:16  tvaughn
Adding correlate changes

Revision 6.8  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 6.6.4.10  2001/06/19 19:00:41  ldjohn
CR854 Passing residual storage into get_transform().

Revision 6.6.4.9  2001/06/13 15:57:41  ldjohn
CR854 Code review:  Improved the cleanup macro.  Longs to ints.  Passed
      structures as pointers.  Replaced CORRIMG.nbytes with dtype element.
      Corrected potential datatype overflow when determining histogram size.
      Added handling of unsupported data types.  Added #defines for active
      flag values.

Revision 6.6.4.8  2001/05/14 14:43:38  ldjohn
CR854 Changed least squares matrix handling to correct behavior.

Revision 6.6.4.7  2001/05/09 15:15:42  ldjohn
CR854 Changes to incorporate use of the GSL (Gnu Scientific Library).

Revision 6.6.4.6  2001/04/26 16:25:04  ldjohn
CR854 Initializing the search master line/sample for cases where the search
      chip is not extracted, but the master line/sample values need to be set.

Revision 6.6.4.5  2001/04/23 18:16:46  ldjohn
CR854 Removed scale factor.
      Moved support arrays for least squares computations into
      get_transform().

Revision 6.6.4.4  2001/03/15 19:11:18  ldjohn
CR854 Created cleanup macro to handle all memory freeing and file closing.
        This makes the code more readable.
      Added edge filtering option for phase correlation.  edge_filter.c
        taken from picmatch/edge.c.
      Corrected error in order for opening the output TPL file.
      Moved binary edge buffers back out of edge_corr into main since the
        same buffers can be used repeatedly rather than reallocating for
        each correlation point.
      Moved the affine transformation process into get_transform().
      Changed edge_corr and grey_corr into type void (were int) and passing
        success/failure info through tpl.active flag.

Revision 6.6.4.3  2001/03/13 19:37:40  ldjohn
CR854 Moved computation of the reference-search space transformation
        coefficients into the correlation point loop for transformation
        refinement.
      Added GREYEDGE correlation method.
      Added RETRY capability.
      Moved results reporting and storage into process_results().

Revision 6.6.4.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "las.h"
#include "util.h"
#include "typlim.h"
#include "worgen.h"
#include "correlate.h"

/* Free memory and close files prior to exiting. */
#define CLEANUP() \
{ \
    if (!ref.chip_flag)          \
        c_eclose(&rfdsc);        \
    if (!srch.chip_flag)         \
        c_eclose(&sfdsc);        \
    if (in_mtp != NULL)          \
        c_ptclse(&in_mtp);       \
    else if (in_cutline != NULL) \
        c_ptclse(&in_cutline);   \
    if (out_tpl != NULL)         \
        c_ptclse(&out_tpl);      \
    if (ref_chip != NULL)        \
        free (ref_chip);         \
    if (srch_chip != NULL)       \
        free (srch_chip);        \
    if (bref != NULL)            \
        free(bref);              \
    if (bsrch != NULL)           \
        free(bsrch);             \
}

                          
int main(void)
{
  struct PARBLK vblk;            /* TAE parameter block                     */
  struct TPLHEAD tpl_header;     /* TPL file header record                  */
  struct TPLDATA ec_data;        /* Temporary storage for edge corr. results*/
  struct TPLDATA tpl_data;       /* Tie point location data record          */
  struct MTPHEAD mtp_header;     /* MTP file header record                  */
  struct MTPDATA mtp_data;       /* Merged tie point data record            */
  struct TPSHEAD cut_header;     /* Cutline TPS file header record          */
  struct CORRPAR corr;           /* Correlation parameters                  */
  struct CORRGRID grid;          /* Tie point grid parameters               */
  struct CORRIMG ref;            /* Reference image file parameters         */
  struct CORRIMG srch;           /* Search image parameters                 */
  struct RETRY retry;            /* Retry specification                     */
  struct FDESC *rfdsc;           /* Full reference image file desc          */
  struct FDESC *sfdsc;           /* Full search image file desc             */
  struct PTDESC *in_mtp = NULL;  /* Input merged tie point file desc        */
  struct PTDESC *in_cutline = NULL; /* Input cutline tie point file desc    */
  struct PTDESC *out_tpl = NULL; /* Output tiepoint location file desc      */
  
  short *bref = NULL;            /* Binary edge reference image buffer      */
  short *bsrch = NULL;           /* Binary edge search image buffer         */
      
  int chop;                      /* number of grid points outside the image */
  int confirm;                   /* Boolean--confirm flag                   */
  int corrmeth;                  /* Method of correlation used              */
  int count;                     /* Loop counter                            */
  int cutinc;                    /* cutline point increment or step size    */
  int hpf;                       /* High-pass filter flag                   */
  int ii;                        /* Loop counter                            */
  int inc;                       /* Step size--phase corr                   */
  int inmeth;                    /* Input correlation point method          */
  int nscale[2];                 /* Scale parameter                         */
  int numtie;                    /* number of tie points used for least-
                                    squares fit of transform coefficients   */
  int phase_edge;                /* Edge enhancement flag for phase corr.   */
  int pts_correlated = 0;        /* Number of points that correlate         */
  int pts_input = 0;             /* Number of points to check               */
  int srch_min, srch_max;        /* Min and max size for search chips       */
  int ssize;                     /* Size of search image--phase corr        */
  int status;                    /* Return status variable                  */
  int update_transform = TRUE;   /* Boolean flag representing whether or not
                                    the affine transformation should be
                                    recomputed                              */

  long ptyp;                     /* Type of tie point file                  */
  long opt[2];                   /* Option for image I/O                    */
  long nbands;                   /* Number of bands                         */
  long nl, ns;                   /* Number of lines and samples             */
  long acc = IREAD;              /* File access mode (always read-only)     */
  const long one = 1;            /* Constant of 1                           */
  const long four = 4;           /* Constant of 4                           */
  long std_out;                  /* Dummy variable to standard output       */

  float diff;                    /* max range of data                       */
  float *ref_chip = NULL;        /* Reference chip buffer                   */
  float *srch_chip = NULL;       /* Search chip buffer                      */
  float alpha;                   /* Exponential factor for phase corr.      */
  float threshold = 0.0;         /* Cloud cover threshold value             */
  float irtie[2*MAX_LSQ_TIE],    /* input reference tie points              */
        istie[2*MAX_LSQ_TIE];    /* input search tie points                 */
  float xg[6] = {1.0, 0.0, 0.0,  /* affine transformation coefficients      */
                 0.0, 1.0, 0.0};
  float xid[6] = {1.0, 0.0, 0.0, /* identity transformation coeff.          */
                  0.0, 1.0, 0.0};

  double fltmax;                 /* temp value used to determine the data
                                    scaling factor for the histogram bins.
                                    Max value is 4 times max range of data. */
  double worka[3*MAX_LSQ_TIE];   /* Work space to hold N x 3 coefficient
                                    matrix                                  */
  double workb[2*MAX_LSQ_TIE];   /* Work space to hold N x 2 right hand side
                                    matrix                                  */
  double workr[MAX_LSQ_TIE];     /* Work space for the least squares
                                    residual vector                         */
  double input_threshold;        /* User input correlation threshold        */
  double nominal[2];             /* Nominal search coordinates              */

  char mtp[CMLEN];               /* Input Merged Tie Point file name        */
  char cutline[CMLEN];           /* Name of the input cutline file          */
  char tpl[CMLEN];               /* Input Tie Point Location file name      */
  char out[CMLEN];               /* Output Tie Point Location file name     */


  /* Initialize LAS interface. */
  c_inlas(&vblk, &std_out);

  /* Initialize chip flags. */
  ref.chip_flag = TRUE;
  srch.chip_flag = TRUE;

  /* Get processing parameters from the user.
     ----------------------------------------*/
  if (getpar(&vblk, &ref, &srch, mtp, cutline, &cutinc, &grid, &inmeth,
             tpl, out, &corr, &retry, &ssize, &inc, &alpha, &hpf,
             &phase_edge, &corrmeth, &confirm, &threshold, irtie, istie,
             &numtie) != E_SUCC)
      c_lasmsg(TRUE, "correlate-getpar",
               "Error encountered while retrieving parameters");

  /* Store the input threshold for use with correlation retries. */
  input_threshold = corr.mincorr;

  /* Determine reference image mode--full image or chips?
     ----------------------------------------------------*/
  if (ref.chip_flag == FALSE)
  {
      /* Open the reference image. */
      opt[0] = opt[1] = 0;
      nbands = 0; nl = ns = 0;
      if (c_eopenr(&rfdsc, ref.name, &nbands, &one, &one, &nl, &ns, &acc,
                   &four, opt, &one) != E_SUCC)
          c_lasmsg(TRUE, "correlate-open", "Unable to open %s for reading",
                   ref.name);
      ref.nl = nl;
      ref.ns = ns;
  }

  /* Determine search image mode--full image or chips?
     -------------------------------------------------*/
  if (srch.chip_flag == FALSE)
  {
      /* Open the search image. */
      opt[0] = opt[1] = 0;
      nbands = 0; nl = ns = 0;
      if (c_eopenr(&sfdsc, srch.name, &nbands, &one, &one, &nl, &ns, &acc,
                   &four, opt, &one) != E_SUCC)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-open", "Unable to open %s for reading",
                   srch.name);
      }
      srch.nl = nl;
      srch.ns = ns;
  }

  /* Open any relevant correlation tie point file.
     ---------------------------------------------*/
  if (inmeth == MTPMETH)
  {
      /* Open the merged tie point file containing the points to be
         correlated. */
      ptyp = MTP;
      if (c_ptopen(&in_mtp, mtp, &ptyp, &acc, &mtp_header) != E_SUCC)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-open", "Unable to open %s for reading",
                   mtp);
      }
  }
  else if (inmeth == CUTMETH)
  {
      /* Open the cutline tie point file containing the points to be
         correlated. */
      ptyp = TPS;
      if (c_ptopen(&in_cutline, cutline, &ptyp, &acc, &cut_header) != E_SUCC)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-open", "Unable to open %s for reading",
                   cutline);
      }
  }

  /* If a file containing a list of tie points to be grown is specified,
     open the file and write its contents to the output tie point file.
     Otherwise fill the tie point header structure with correlation data
     and information from the merged tie point file and write to the
     output tie point file.
     -------------------------------------------------------------------*/
  if (strcmp(tpl, "") != 0)
      status = copy_points(tpl, out, corrmeth, &out_tpl, &tpl_header,
                           &tpl_data, &corr);
      
  else
      status = fill_header(&ref, &srch, &corr, out, corrmeth, &mtp_header,
                           &cut_header, inmeth, &out_tpl, &tpl_header);
  if (status != E_SUCC)
  {
      CLEANUP();
      c_lasmsg(TRUE, "correlate-copy",
               "Error occurred while initializing %s", out);
  }          
  
  if (corrmeth == EDGE_CORR || corrmeth == GREYEDGE_CORR)
  {
      /* Determine scaling factor needed, if any, to ensure at most 2**15
         histogram bins.
         ----------------------------------------------------------------*/
      if (srch.minval == 0 && srch.maxval == 0) 
      {
          if (srch.dtype == EBYTE)
              diff = MAXUTINY;
          else if (srch.dtype == EWORD)
              diff = MAXWORD;
          else if (srch.dtype == ELONG)
              diff = MAXLONG;
          else if (srch.dtype == EREAL)
              diff = MAXREAL;
          else
          {
              CLEANUP();
              c_lasmsg(TRUE, "correlate-dtype",
                       "Search image data type not supported."
                       "  Only BYTE, I*2, I*4, and R*4 supported.");
          }          
      }
      else
          diff = (float)(srch.maxval - srch.minval);
      for (fltmax=4.0*diff,count=0; fltmax>=HISTOGRAMSIZE; count++)
          fltmax /= 2.0;
      nscale[0] = count;

      if (ref.minval == 0 && ref.maxval == 0) 
      {
          if (ref.dtype == EBYTE)
              diff = MAXUTINY;
          else if (ref.dtype == EWORD)
              diff = MAXWORD;
          else if (ref.dtype == ELONG)
              diff = MAXLONG;
          else if (ref.dtype == EREAL)
              diff = MAXREAL;
          else
          {
              CLEANUP();
              c_lasmsg(TRUE, "correlate-dtype",
                       "Reference image data type not supported."
                       "  Only BYTE, I*2, I*4, and R*4 supported.");
          }          
      }
      else
          diff = (float)(ref.maxval - ref.minval);
      for (fltmax=4.0*diff, count=0; fltmax>=HISTOGRAMSIZE; count++)
          fltmax /= 2.0;
      nscale[1] = count;
  } /* if edge corr. */

  /* Dynamically allocate buffers.
     -----------------------------*/
  if ((ref_chip = (float *)malloc(REF_BUFSIZE*sizeof(float))) == NULL)
  {
      CLEANUP();
      c_lasmsg(TRUE, "correlate-alloc", "Unable to allocate dynamic memory");
  }
  if ((srch_chip = (float *)malloc(SRCH_BUFSIZE*sizeof(float))) == NULL)
  {
      CLEANUP();
      c_lasmsg(TRUE, "correlate-alloc", "Unable to allocate dynamic memory");
  }

  /* If edge correlation, allocate buffers for binary edge images.
     -------------------------------------------------------------*/
  if (corrmeth == EDGE_CORR || corrmeth == GREYEDGE_CORR)
  {
      if ((bref = (short *)malloc(REF_BUFSIZE*sizeof(short))) == NULL)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-alloc",
                   "Unable to allocate dynamic memory");
      }
      if ((bsrch = (short *)malloc(SRCH_BUFSIZE*sizeof(short))) == NULL)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-alloc",
                   "Unable to allocate dynamic memory");
      }
  }

  /* ----------------------------
     Process all tie point pairs.
     ----------------------------*/
  for (;;)
  {
      /* If the transformation tie point set has been updated,
         process tie points to set up the affine transform.
         If either the reference image or the search image is
         a chip, the transformation is not necessary.
         ----------------------------------------------------*/
      if (!ref.chip_flag  && !srch.chip_flag && update_transform)
      {
          if ( xg[0] != xg[0] ) { /* XXX */
              printf("xg[o] Nan before call to get_transform\n");
              exit(1);
          }
          
          /*printf("pts_input = %d\n", pts_input );  XXX 
          printf("numtie = %d\n", numtie ); */
          
          if (get_transform(numtie, irtie, istie, worka, workb, workr, xg)
                                                                    != E_SUCC)
          {
              CLEANUP();
              c_lasmsg(TRUE, "correlate-trans",
                       "Error generating transform coefficients");
          }
          
                    
          /* Set flag to show that the affine transform is up to date. */
          update_transform = FALSE;

          if ( xg[0] != xg[0] ) { /* XXX */
              /* printf("xg[o] NaN after call to get_transform\n"); */
              c_lasmsg( FALSE, "correlate-transNaN", 
                        "xg[o] NaN after call to get_transform");
              status = BAD_CHIP_DDR;
              /* exit(1); */
              goto BAD_END;
          }
      }     

      /* Retrieve the tie point pair.
         ----------------------------*/
      status = get_tp(corrmeth, inmeth, in_mtp, in_cutline, cutinc, &grid,
                      xg, &retry, &pts_input, &mtp_data);
      
      if ( xg[0] != xg[0] ) { /* XXX */
          printf("xg[o] Nan after call to get_tp\n");
          exit(1);
      }
      
      if (status == E_EOF)
          break;
      else if (status != E_SUCC)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-ptio", "Error reading tie point");
      }

      /* Initially set tpl active flag to "accept automatically extracted
         point".
         ----------------------------------------------------------------*/
      tpl_data.active = TPL_STATE_OK_AUTO;

      /* Extract reference window.
         -------------------------*/
      if ( xid[0] != xid[0] ) { /* XXX */
          printf("xid[o] Nan before call to extract\n");
          exit(1);
      }
      
      status = extract(&rfdsc, mtp_data.ref_coord, mtp_data.chip_size,
                       mtp_data.ref_chip, REF_MIN, REF_MAX, xid,
                       &ref, &chop, ref_chip);
      
      if ( xid[0] != xid[0] ) { /* XXX */
          printf("xid[o] Nan after call to extract\n");
          if ( status == E_FAIL ) {
              printf( "returned status E_FAIL\n" );
          }
          exit(1);
      }
      if ( status == E_FAIL ) { /* XXX */
          printf("xid[0] = %f\n", xid[0] );
      }

BAD_END:
      if (status == BAD_CHIP_DDR)
          tpl_data.active = TPL_STATE_BAD_PARAM;
      else if (status == BAD_SIZE)
          tpl_data.active = TPL_STATE_BAD_USER;
      else if (status == OUTSIDE_IMAGE)
          tpl_data.active = TPL_STATE_BAD_MODEL;
      else if (status == READ_ERROR)
          tpl_data.active = TPL_STATE_BAD_USER_MODEL;
      else if (status == E_FAIL)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-corr",
                   "Error encountered during correlation.");
      }

      /* Extract search window.
         ----------------------*/

      /* Initialize the master line/sample in case an error occurred
         during reference chip extraction.  These values are used later
         to adjust the image location even when nonfatal errors occur. */
      srch.master_line = srch.master_samp = 1;
      
      if (tpl_data.active == TPL_STATE_OK_AUTO)
      {
          if (corrmeth == PHASE_CORR)
          {
              tpl_data.sea_size[0] = ssize;
              tpl_data.sea_size[1] = ssize;
              srch_min = 32;
              srch_max = 128;
          }
          else
          {
              tpl_data.sea_size[0] = mtp_data.chip_size[0] +
                  2*mtp_data.offsets[0];
              tpl_data.sea_size[1] = mtp_data.chip_size[1] +
                  2*mtp_data.offsets[1];
              srch_min = SRCH_MIN;
              srch_max = SRCH_MAX;
          }
          
          if ( xg[0] != xg[0] ) { /* XXX */
              printf("xg[o] Nan before call to extract 2\n");
              exit(1);
          }
          
          status = extract(&sfdsc, mtp_data.nominal, tpl_data.sea_size,
                           mtp_data.sea_chip, srch_min, srch_max, xg,
                           &srch, &chop, srch_chip);
          
          if ( xg[0] != xg[0] ) { /* XXX */
              printf("xg[o] Nan after call to extract 2\n");
              if ( status == E_FAIL ) {
                  printf( "returned status E_FAIL\n" );
              }
              exit(1);
          }
          if ( status == E_FAIL ) { /* XXX */
              printf("xg[0] = %f\n", xg[0] );
          }
          
          if ((status == BAD_COEFS) &&
              (strcmp(tpl_header.ref_mode, "CHIP-PRE") == 0))
          {
              tpl_data.active = TPL_STATE_BAD_PARAM;
              c_lasmsg(FALSE, " ", "Error reading chip DDR information");
          }
          else if (status == BAD_CHIP_DDR)
              tpl_data.active = TPL_STATE_BAD_PARAM;
          else if (status == BAD_SIZE)
              tpl_data.active = TPL_STATE_BAD_USER;
          else if (status == OUTSIDE_IMAGE)
              tpl_data.active = TPL_STATE_BAD_MODEL;
          else if (status == READ_ERROR)
              tpl_data.active = TPL_STATE_BAD_USER_MODEL;
          else if (status != E_SUCC)
          {
              CLEANUP();
              c_lasmsg(TRUE, "correlate-corr",
                       "Error encountered during correlation.");
          }
      } /* tpl_data.active = TPL_STATE_OK_AUTO */


      /* Initialize the TPL data record with values from the
         MTP data record.
         ---------------------------------------------------*/
      fill_tpl_data(&ref, &srch, &mtp_data, &tpl_data);

      for (ii = 0; ii < 2; ii++)
          nominal[ii] = mtp_data.offsets[ii];

      /* If both chips have been extracted successfully,
         perform correlation.
         -----------------------------------------------*/
      if (tpl_data.active == TPL_STATE_OK_AUTO)
      {
          if (corrmeth == PHASE_CORR)
          {
              /* If requested, apply edge enhancement to the chips. */
              if (phase_edge)
              {
                  if (edge_filter(mtp_data.chip_size[0], ref_chip) != E_SUCC)
                  {
                      CLEANUP();
                      c_lasmsg(TRUE, "correlate-corr",
                               "Error encountered during correlation.");
                  }
                  if (edge_filter(tpl_data.sea_size[0], srch_chip) != E_SUCC)
                  {
                      CLEANUP();
                      c_lasmsg(TRUE, "correlate-corr",
                               "Error encountered during correlation.");
                  }
              }  
              phase_corr(&corr, ref_chip, srch_chip, inc, alpha, hpf,
                         xg, &tpl_data);
              
              if ( xid[0] != xid[0] ) { /* XXX */
                  printf("xid[o] Nan after call to phase_corr\n");
                  exit(1);
              }
          }
          
          if (corrmeth == EDGE_CORR || corrmeth == GREYEDGE_CORR)
              edge_corr(&corr, ref_chip, srch_chip, bref, bsrch, nominal,
                        nscale, threshold, xg, &tpl_data);

          if ( xid[0] != xid[0] ) { /* XXX */
              printf("xid[o] Nan after call to edge_corr\n");
              exit(1);
          }
          
          /* If gray-edge correlation is being applied, check the edge
             correlation results, and store data if necessary before
             applying gray correlation. */
          if (corrmeth == GREYEDGE_CORR)
          {
              ec_data.active = tpl_data.active;
              ec_data.displacement = tpl_data.displacement;
              ec_data.strength = tpl_data.strength;
              ec_data.sea_coord[0] = tpl_data.sea_coord[0];
              ec_data.sea_coord[1] = tpl_data.sea_coord[1];
              ec_data.rms_error[0] = tpl_data.rms_error[0];
              ec_data.rms_error[1] = tpl_data.rms_error[1];
          }
          
          if (corrmeth == GREY_CORR || corrmeth == GREYEDGE_CORR)
              grey_corr(&corr, ref_chip, srch_chip, nominal, xg,
                        &tpl_data);
          
          if ( xid[0] != xid[0] ) { /* XXX */
              printf("xid[o] Nan after call to grey_corr\n");
              exit(1);
          }
          
          if (corrmeth == GREYEDGE_CORR)
          {
              /* If edge corr. better than gray corr., copy stored
                 edge results into tpl_data structure. */
              if (ec_data.strength > tpl_data.strength)
              {
                  tpl_data.active = ec_data.active;
                  tpl_data.displacement = ec_data.displacement;
                  tpl_data.strength = ec_data.strength;
                  tpl_data.sea_coord[0] = ec_data.sea_coord[0];
                  tpl_data.sea_coord[1] = ec_data.sea_coord[1];
                  tpl_data.rms_error[0] = ec_data.rms_error[0];
                  tpl_data.rms_error[1] = ec_data.rms_error[1];
              }
          }

      } /* tpl_data.active = TPL_STATE_OK_AUTO */

      /* If retries have been specified, check the correlation results
         and prepare for another attempt if necessary.
         -------------------------------------------------------------*/
      if ((retry.count < retry.total) &&
          (tpl_data.active != TPL_STATE_OK_AUTO))
      {
          corr.mincorr = retry.threshold;
          retry.count++;
          continue;
      }
      else
      {
          corr.mincorr = input_threshold;
          retry.count = 0;
      }
      
      /* Adjust coordinates if chips were used.
         --------------------------------------*/
      adjust_chip_coordinates(&ref, &srch, &mtp_data, &tpl_header, &tpl_data);

      /* Report results to user and write to tie point location file
         -----------------------------------------------------------*/
      if (process_results(&tpl_data, confirm, out_tpl, &pts_correlated,
                          &numtie, irtie, istie, &update_transform) != E_SUCC)
      {
          CLEANUP();
          c_lasmsg(TRUE, "correlate-proc",
                   "Error encountered while processing correlation results.");
      }
      
  } /* tie point loop */

  /* Free memory and close files. */
  CLEANUP();

  /* Write completion message and exit
     ---------------------------------*/
  c_lasmsg(FALSE, " ",
           "%d of %d points were successfully correlated (%5.2f%c)",
          pts_correlated, pts_input, 100.0*pts_correlated/pts_input, '%');
  c_complt("CORRELATE", &out, &one);
  return 0;
}
