#ifndef _CORRELATE_H_
#define _CORRELATE_H_

#include "las.h"
#include "geompak.h"


/* Correlation methods */
#define GREY_CORR       1
#define EDGE_CORR       2
#define PHASE_CORR      3
#define GREYEDGE_CORR   4

/* Input correlation point methods */
#define MANMETH  0     /* manual input */
#define MTPMETH  1     /* MTP file */
#define CUTMETH  2     /* cutline file */
#define GRIDMETH 3     /* grid of points */

/* Surface fitting method */
#define PARAB   1
#define GAUSS   2
#define RECIP   3
#define INTEGER 4

/* Reference and search chip sizes */
#define REF_MIN         32
#define REF_MAX        128
#define REF_BUFSIZE  16384
#define SRCH_MIN        48 
#define SRCH_MAX       256      
#define SRCH_BUFSIZE 65536 

/* Histogram size */
#define HISTOGRAMSIZE 32768

/* Error messages.  Additional error messages may be found in geompak.h
  --------------------------------------------------------------------*/
#define BAD_SIZE        204
#define OUTSIDE_IMAGE   205
#define READ_ERROR      206
#define BAD_COEFS       207
#define BAD_CHIP_DDR    208

#define MAX_LSQ_TIE 1000 /* max. number of tiepoints used for least-squares
                            computations */

/* Structure holding correlation parameters
  ----------------------------------------*/
struct CORRPAR
{
   double mincorr;              /* Minimum correlate value */
   double maxdiff;              /* Maximum diff between nominal & refined */
   double edgeval;              /* Fraction of pixels to be classified edges*/
   long fitmeth;                /* Method of peak fitting */
}; 

/* Structure holding image specific parameters
  -------------------------------------------*/
struct CORRIMG
{
   double coef_y[3], coef_x[3]; /* Coefs for pre-scaled/rotated srch chips */
   long nl;                     /* Number of image lines */
   long ns;                     /* Number of image samples */
   long band;                   /* Band number to correlate on */
   long master_line;            /* Offset of coord to master image, if any */
   long master_samp;            /* Offset of coord to master image, if any */
   long starting_line;
   long starting_sample;
   long minval;                 /* Mininim pixel value */
   long maxval;                 /* Maximum pixel value */
   long dtype;                  /* Image data type */
   long chip_flag;              /* Boolean--are chip images used? */
   char name[CMLEN];            /* Image host name */
};

/* Structure holding correlation retry data
   ----------------------------------------*/
struct RETRY
{
    int total;                    /* Max number of retries */
    int count;                    /* Current count of number of retries */
    float distance;               /* Line/sample offset distance */
    float direction[10];          /* 5 retry vector directions.  Should be
                                     initialized to
                                     {0.0, 1.0, -1.0, 1.0, -1.0,
                                      0.0, 1.0, -1.0, -1.0, 1.0} */
    float threshold;              /* Correlation threshold */
};

/* Structure holding tie point grid data
   -------------------------------------*/
struct CORRGRID
{
    float uly, ulx, lry, lrx;  /* upper-left and lower-right coordinates */
    int nrows, ncols;          /* number of rows and columns */
};
  

/* Prototypes
   ----------*/
int accept
(
    const int confirm,         /* I: Boolean--confirm mode                 */
    const struct TPLDATA *data /* I: TPL data record                       */
);

void adjust_chip_coordinates
(
    const struct CORRIMG *ref,       /* I: Reference image file parameters  */
    const struct CORRIMG *srch,      /* I: Search image parameters          */
    const struct MTPDATA *mtp_data,  /* I: Merged tie point data record     */
    const struct TPLHEAD *tpl_header,/* I: TPL file header record           */
    struct TPLDATA *tpl_data         /* O: Tie point location data record   */
);

int copy_points
(
    char *tpl,                    /* I: Input Tie Point Location file name  */
    char *out,                    /* I: Output Tie Point Location file name */
    const int corrmeth,           /* I: Method of correlation used          */
    struct PTDESC **out_tpl,      /* O: Output tiepoint location file desc  */
    struct TPLHEAD *tpl_header,   /* O: TPL file header record              */
    struct TPLDATA *tpl_data,     /* O: Tie point location data record      */
    struct CORRPAR *corr          /* O: Correlation parameters              */
);

void edge_corr
(
    struct CORRPAR *corr, /* I: Correlation parameters                      */
    float *ref,           /* I: Reference subimage                          */
    float *srch,          /* I: Search subimage                             */
    short *bref,          /* I: Binary edge reference image buffer          */
    short *bsrch,         /* I: Binary edge search image buffer             */
    double *nominal,      /* I: Nominal search image coordinates            */
    int *nscale,          /* I: Scale parameter                             */
    const float threshold,/* I: Cloud cover threshold value                 */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct TPLDATA *tpl         /* I/O: Tie point location data record      */
);

int edge_filter
(
    const int size,  /* I: Size (N) of the array a  */
    float *aa        /* I/O: 2D array of size N x N */
);

void eval
(
    const int ncol,      /* I: Number of columns in coincidence-count array */
    const int nrow,      /* I: Number of rows in coincidence-count array    */
    const float *ccnorm, /* I: Array of norm xcorr coeffs                   */
    const float *pkval,  /* I: Table of top 32 normalized values            */
    const int *ipkcol,   /* I: Table of column numbers for top 32 values    */
    const int *ipkrow,   /* I: Table of row numbers for top 32 values       */
    const float csmin,   /* I: Minimum acceptable correlation strength      */
    float *sums,         /* O: Sum of all normalized values and sum of
                               squares                                      */
    float *streng,       /* O: Strength of correlation                      */
    int *iacrej,         /* O: Accept/Reject code (see geompak.h)           */
    float *cpval         /* O: 5 by 5 array of xcorr vals in std dev        */
);

int extract
(
    struct FDESC **ptr,       /* I: File pointer to image file              */
    const double *tp,         /* I: Tie point (l,s)                         */
    const int *size,          /* I: Size of chip--nl,ns                     */
    char *name,               /* I: Name of chip image file                 */
    const int minval,         /* I: Minimum chip size                       */
    const int maxval,         /* I: Maximum chip size                       */
    float *trans,        /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct CORRIMG *image,    /* I/O: Image file parameters                 */
    int *chop,                /* O: Number of grid points outside the image.
                                    Corresponding DN values in the "grid" are
                                    set to zero.                            */
    float *buffer             /* O: Chip buffer                             */
);

int fill_header
(
    const struct CORRIMG *ref,       /* I: Reference image file parameters  */
    const struct CORRIMG *srch,      /* I: Search image parameters          */
    const struct CORRPAR *corr,      /* I: Correlation parameters           */
    char *out,                    /* I: Output Tie Point Location file name */
    const int corrmeth,              /* I: Method of correlation used       */
    const struct MTPHEAD *mtp_header,/* I: MTP file header record           */
    const struct TPSHEAD *cut_header,/* I: Cutline TPS file header record   */
    const int inmeth,                /* I: Correlation point input method   */
    struct PTDESC **out_tpl,       /* O: Output tiepoint location file desc */
    struct TPLHEAD *tpl_header       /* O: TPL file header record           */
);

void fill_tpl_data
(
    const struct CORRIMG *ref,       /* I: Reference image file parameters  */
    const struct CORRIMG *srch,      /* I: Search image parameters          */
    const struct MTPDATA *mtp_data,  /* I: MTP file data structure          */
    struct TPLDATA *tpl_data         /* O: TPL file data structure          */
);

int fitreg
(
    const float *cpval, /* I: 5 by 5 array of xcorr vals, in units of standard 
                           dev above background, centered on corr peak      */
    const int mfit,     /* I: Method of surface fit: 1 - Elliptical paraboloid
                                                  2 - Elliptical Gaussian
                                                  3 - Reciprocal Paraboloid */
    float *pkoffs,      /* O: Best-fit horiz and vertical offsets of
                              correlation peak relative to center of 5 by 5
                              array                                         */
    float *tlerrs       /* O: Estimated horizontal error, vertical error, and
                              h-v cross term in best-fit offsets            */
);

void get_chip_corners
(
    const double *tp,         /* I: Tie point (l,s)                         */
    const int *size,          /* I: Size of chip--nl,ns                     */
    const float *trans,  /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct CORRIMG *image,    /* I/O: Image file parameters                 */
    int *line_sign,           /* O: line direction of 1st line in transform
                                    space: + = down, - = up                 */
    int *samp_sign,           /* O: sample direction of 1st line in transform
                                    space: + = right, - = left              */
    float *basey,             /* O: transform-space line of upper-left chip
                                    corner                                  */
    float *basex,             /* O: transform-space sample of upper-left chip
                                    corner                                  */
    float *ystart,            /* O: first (top) y value for transformed chip*/
    int *line_count,          /* O: num. of transform-space lines in chip   */
    int *samp_start,          /* O: leftmost transform-space sample in chip */
    int *samp_count           /* O: num. of transform-space samples in chip */
);

int get_tp
(
    const int corrmeth,       /* I: Method of correlation used              */
    const int inmeth,         /* I: Input correlation point method          */
    struct PTDESC *in_mtp,    /* I: Input merged tie point file desc        */
    struct PTDESC *in_cutline,/* I: Input cutline tie point file desc       */
    const int cutinc,         /* I: cutline point increment or step size    */
    const struct CORRGRID *grid, /* I: Tie point grid parameters            */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    const struct RETRY *retry,/* I: Retry specification                     */
    int *pt_num,              /* I/O: Current point number (counter)        */
    struct MTPDATA *mtp_data  /* I/O: One Merged Tie Point data record      */
);

int get_transform
(
    const int numtie,         /* I: number of tie points used for least-
                                    squares fit of transform coefficients   */
    float irtie[],            /* I: input reference tie points              */
    float istie[],            /* I: input search tie points                 */
    double aa[],              /* I: Work space to hold N x 3 coefficient
                                    matrix                                  */
    double bb[],              /* I: Work space to hold N x 2 right hand side
                                    matrix                                  */
    double residual[],        /* I: Work space for the residual vector
                                    (N elements)                            */
    float xg[6]               /* O: 2 x 3 solution matrix for llsq call     */
);

int get_tvals
(
    struct PARBLK *vblk,      /* I: TAE variable block                      */
    char *mtp,                /* I: Name of the input merged tie point file */
    const int inmeth,         /* I: Input correlation point method          */
    float irtie[],            /* O: Tie points for first input image (maximum
                                    of MAX_IN_TIE tiepoint pairs)           */
    float istie[],            /* O: Tie points for second input image (maximum
                                    of MAX_IN_TIE tiepoint pairs)           */
    int *numtie               /* O: Number of tie points used for least-
                                    squares fit of transform coefficients   */
);

int getpar
(
    struct PARBLK *vblk,      /* I: TAE variable block                      */
    struct CORRIMG *ref,      /* O: Name of the reference image             */
    struct CORRIMG *srch,     /* O: Name of the search image                */
    char *mtp,                /* O: Name of the input merged tie point file */
    char *cutline,            /* O: Name of the input cutline file          */
    int *cutinc,              /* O: Cutline point increment or step size    */
    struct CORRGRID *grid,    /* O: Tie point grid parameters               */
    int *inmeth,              /* O: Input correlation point method          */
    char *tpl,              /* O: Name of the input tie point location file */
    char *out,              /* O: Name of the output tie point location file*/
    struct CORRPAR *corr,     /* O: Correlation parameters                  */
    struct RETRY *retry,      /* O: Retry specification                     */
    int *ss,                  /* O: Search chip size for phase corr.        */
    int *inc,                 /* O: Search chip step size for phase corr.   */
    float *alpha,             /* O: Exponential factor for phase corr.      */
    int *hpf,                 /* O: High-pass filter flag                   */
    int *phase_edge,          /* O: Edge enhancement flag for phase corr.   */
    int *corrmeth,            /* O: Correlation method                      */
    int *confirm,             /* O: Confirm mode--boolean                   */
    float *threshold,         /* O: Cloud cover threshold value             */
    float ritie[],            /* O: Tie points for first input image (maximum
                                    of 10 tiepoint pairs)                   */
    float rotie[],            /* O: Tie points for second input image (maximum
                                    of 10 tiepoint pairs)                   */
    int *numtie               /* O: Number of tie points used for least-
                                    squares fit of transform coefficients   */
);

void grey_corr
(
    struct CORRPAR *corr,      /* I: Correlation parameters                 */
    float *ref,                /* I: Reference subimage                     */
    float *srch,               /* I: Search subimage                        */
    double *nominal,           /* I: Nominal coordinates                    */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct TPLDATA *tpl        /* I/O: Tie point location data record       */
);

void phase_corr
(
    const struct CORRPAR *corr, /* I: Correlation parameters             */
    float *ref,                 /* I: Reference subimage                 */
    float *srch,                /* I: Search subimage                    */
    const int inc,              /* I: Step size                          */
    const float alpha,          /* I: Exponential factor                 */
    const int hpf,              /* I: High-pass filter flag              */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct TPLDATA *tpl         /* I/O: Tie point location data record   */
);

int process_results
(
    const struct TPLDATA *tpl_data, /* I: Tie point location data record    */
    const int confirm,        /* I: Boolean--confirm flag                   */
    struct PTDESC *out_tpl,   /* I: Output tiepoint location file desc      */
    int *pts_correlated,      /* I/O: Number of points that correlate       */
    int *numtie,              /* I/O: number of tie points used for least-
                                      squares fit of transform coefficients */
    float irtie[],            /* O: input reference tie points              */
    float istie[],            /* O: input search tie points                 */
    int *update_transform     /* O: Boolean flag representing whether or not
                                    the affine transformation should be
                                    recomputed                              */
);

int read_chip
(
    char *name,               /* I: Name of chip image file                 */
    const int *size,          /* I: Size of chip--nl,ns                     */
    struct CORRIMG *image,    /* I/O: Image file parameters                 */
    float *buffer             /* O: Chip buffer                             */
);

int read_image_line
(
    float *bufin,      /* O: 2D image line return buffer                  */
    struct FDESC *fdesc, /* I: Image file control block                   */
    long *record,      /* I/O: Record number to read, if 0 then read the
                               next record (one-based)                    */
    const int skpbyt,  /* I: Number of bytes to skip before reading       */
    const int nbytes,  /* I: Total number of bytes to read                */
    const int ns,      /* I: Number of samples in the line                */
    const int ilow,    /* I: Row to be read (zero-based)                  */
    const long band    /* I: Band to be read                              */
);

int resample
(
    struct FDESC **ptr,       /* I: File pointer to image file              */
    const int *size,          /* I: Size of chip--nl,ns                     */
    const float *trans,  /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    const struct CORRIMG *image, /* I: Image file parameters                */
    const int line_sign,      /* I: line direction of 1st line in transform
                                    space: + = down/right, - = up/left      */
    const int samp_sign,      /* I: sample direction of 1st line in transform
                                    space: + = down/right, - = up/left      */
    const float ystart,       /* I: first (top) y value for transformed chip*/
    const float basey,        /* I: transform-space line of upper-left chip
                                    corner                                  */
    const float basex,        /* I: transform-space sample of upper-left chip
                                    corner                                  */
    int *chop,                /* O: Number of grid points outside the image.
                                    Corresponding DN values in the "grid" are
                                    set to zero.                            */
    float *buffer             /* O: Chip buffer                             */
);

#endif /* _CORRELATE_H_ */
