/****************************************************************************
NAME:                           edge_corr

PURPOSE:  Performs edge correlation of a tie point 

VERSION         DATE    AUTHOR
-------         ----    ------
  1.0           8/92    D. Steinwand

CHANGES:

$Log: edge.c,v $
Revision 6.5  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 6.4.4.6  2001/06/13 16:23:21  ldjohn
CR854 Code review:  Longs to ints.  Using #define for buffer sizes.  Added
      #defines for active flag values.

Revision 6.4.4.5  2001/05/09 15:15:42  ldjohn
CR854 Changes to incorporate use of the GSL (Gnu Scientific Library).

Revision 6.4.4.4  2001/04/23 18:19:50  ldjohn
CR854 Rearranged functions to avoid need for prototypes of internal functions.

Revision 6.4.4.3  2001/03/15 19:14:52  ldjohn
CR854 Changed edge_corr and grey_corr to type int and passing success/
        failure status through tpl->active flag.
      Documented return values.

Revision 6.4.4.2  2001/03/13 19:42:31  ldjohn
CR854 Rearranged edge_corr() parameters to align closer with grey_corr().

Revision 6.4.4.1  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <math.h>
#include <malloc.h>
#include "geompak.h"
#include "correlate.h"
#include "gsl/gsl_complex_math.h"
#include "gsl/gsl_fft_complex.h"


/****************************************************************************
NAME:                           COINC

PURPOSE:  Compute the unnormalized (raw) number of edge coincidences between 
          the reference and search images for every combination of horizontal 
          and vertical offsets of the reference realtive to the search image

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1. LAS 4.0 EDGECORR by R. White & G. Neal 6/83
*****************************************************************************/
static void coinc
(
    short *imgbns,          /* I: Binary edge image from search subimage    */
    short *imgbnr,          /* I: Binary edge image from reference subimage */
    float *npls,            /* I: Actual size of search chip: samps,lines   */
    float *nplr,            /* I: Actual size of ref chip: samps,lines      */
    const float nshrnk,     /* I: Number of cols in coincidence-count array */
    int *ncol,              /* O: Number of cols in coincidence-count array */
    int *nrow,              /* O: Number of rows in coincidence-count array */
    float *unormc           /* O: Array of unnormalized (raw) counts of edge
                                  coincidences for each alignment of ref
                                  image relative to search image            */
)
{
  int membfr[SRCH_BUFSIZE];    /* Zero-extended binary edge reference image*/
  int membfs[SRCH_BUFSIZE];    /* Zero-extended binary edge search image */
  int imgptr;                  /* Pointer into binary edge image arrays */
  int line;                    /* Loop index:  current buffer line */
  int lncont;                  /* Pointer to start of next segment of line */
  int lnstrt;                  /* Pointer to first segment of line */
  int memptr;                  /* Loop index */
  int ndxout;                  /* Pointer into array for correlation output*/
  int nlrbin;                  /* Number of lines in binary reference image*/
  int nlsbin;                  /* Number of lines in binary search image */
  int nprbin;                  /* # of pixels per line in binary ref image */
  int npsbin;                  /* Number of pixels per line in search image*/
  int memdim[2];               /* Power-of-2 dimensions for AP arrays */
  int nplsbn[2];               /* Dimensions of binary search image */
  int ii,jj;                   /* Loop counters */

  gsl_complex cser[SRCH_BUFSIZE]; /* Complex-valued search subimage */
  gsl_complex cref[SRCH_BUFSIZE]; /* Complex-valued reference subimage */

  
  /* Zero extend binary search image to next higher power of 2
     ---------------------------------------------------------*/
  nplsbn[0]=npsbin = (int)((npls[0] - nshrnk) + 0.5);
  nplsbn[1]=nlsbin = (int)((npls[1] - nshrnk) + 0.5);
  for(ii=0;ii<2;ii++)
  {
      memdim[ii]=64;
      for(;;)
      {
          if (nplsbn[ii] <= memdim[ii])
              break;
          memdim[ii] *= 2;
      }
  }

  /* Fix for incorrect handling of non-square arrays
     -----------------------------------------------*/
  memdim[0] = memdim[0] > memdim[1] ? memdim[0] : memdim[1];
  memdim[1] = memdim[0];
  for (lnstrt=0, imgptr=0, line=0; line<memdim[1]; line++)
  {
      if (line < nlsbin)
      {
          lncont = lnstrt + npsbin;
          for (memptr=lnstrt; memptr<lncont; imgptr++, memptr++)
              membfs[memptr] = imgbns[imgptr];
      }
      else 
          lncont = lnstrt;
      lnstrt += memdim[0];
      if (lncont < lnstrt)
          for (memptr=lncont; memptr<lnstrt; memptr++)
              membfs[memptr] = 0;
  }

  /* Make complex 2-d of search data
     -------------------------------*/
  for(memptr=0, ii=0; ii<memdim[1]; ii++)
      for(jj=0; jj<memdim[0]; jj++, memptr++)
          GSL_SET_COMPLEX(&cser[jj*memdim[1]+ii], (double)membfs[memptr], 0.);

  /* Now zero extend binary reference image
     --------------------------------------*/
  nprbin = (int)((nplr[0] - nshrnk) + 0.5);
  nlrbin = (int)((nplr[1] - nshrnk) + 0.5);
  for (lnstrt=0, imgptr=0, line=0; line<memdim[1]; line++)
  {
      if (line < nlrbin) 
      {
          lncont = lnstrt + nprbin;
          for (memptr=lnstrt; memptr<lncont; imgptr++, memptr++)
              membfr[memptr] = imgbnr[imgptr];
      }
      else 
          lncont = lnstrt;
      lnstrt += memdim[0];
      if (lncont < lnstrt)
          for (memptr=lncont; memptr<lnstrt; memptr++)
              membfr[memptr] = 0;
  }

  /* Make complex 2-d of reference data
     ----------------------------------*/
  for(memptr=0, ii=0; ii<memdim[1]; ii++)
      for(jj=0; jj<memdim[0]; jj++, memptr++)
          GSL_SET_COMPLEX(&cref[jj*memdim[1]+ii], (double)membfr[memptr], 0.);

  /* Take FFT of search and reference data
     -------------------------------------*/
  for (ii=0; ii<memdim[0]; ii++) /* row FFTs */
  {
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cser[ii*memdim[1]], 1, memdim[1]);
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cref[ii*memdim[1]], 1, memdim[1]);
  }
  for (ii=0; ii<memdim[1]; ii++) /* column FFTs */
  {
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cser[ii], memdim[1], memdim[0]);
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cref[ii], memdim[1], memdim[0]);
  }

  /* Make point by multiplication of search chip FT with conjugate of
     reference chip FT
     ----------------------------------------------------------------*/
  for(ii=0; ii<memdim[0]*memdim[1]; ii++)
      cser[ii] = gsl_complex_mul(cser[ii], gsl_complex_conjugate(cref[ii]));

  /* Take inverse FFT of cser
     ------------------------*/
  for (ii=0; ii<memdim[0]; ii++) /* row FFTs */
      gsl_fft_complex_radix2_inverse(
          (gsl_complex_packed_array)&cser[ii*memdim[1]], 1, memdim[1]);
  for (ii=0; ii<memdim[1]; ii++) /* column FFTs */
      gsl_fft_complex_radix2_inverse(
          (gsl_complex_packed_array)&cser[ii], memdim[1], memdim[0]);

  /* Extract part of correlation array which is valid
     ------------------------------------------------*/
  *ncol = (int)(npls[0] - nplr[0] + 1.5);
  *nrow = (int)(npls[1] - nplr[1] + 1.5);
  ndxout = 0;
  for(ii=0;ii<*nrow;ii++)
      for(jj=0;jj<*ncol;jj++,ndxout++)
          unormc[ndxout] = GSL_REAL(cser[jj*memdim[1]+ii]);
}


/****************************************************************************
NAME:                           ENORM

PURPOSE:  Convert raw edge-coincidence counts to normalized cross-correlation 
          coefficients, while tabulating statistics needed for subsequent 
          evaluation

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.  LAS 4.0 EDGECORR by R. White 6/83
*****************************************************************************/
static void enorm
(
    const int nrtot,     /* I: Total number of pixels in binary reference
                               image                                        */
    const int ncol,      /* I: Number of columns in coincidence-count array */
    const int nrow,      /* I: Number of rows in coincindence-count array   */
    float *unormc,       /* I: Array of unnormalized (raw) edge coincidences 
                               for each alignment of reference image relative
                               to the search image                          */
    const float edenr,   /* I: Density of edges in reference image          */
    float *edens,        /* I: Density of search-image edges for each relative 
                               alignment of the reference image             */
    float *ccnorm,       /* O: Array of normalized cross-corr coefficients for 
                               each alignment of the reference edge image
                               relative to the search edge image            */
    float *pkval,        /* O: Table of top 32 normalized values            */
    int *ipkcol,         /* O: Table of column numbers for top 32 values    */
    int *ipkrow,         /* O: Table of row numbers for top 32 values       */
    float *sums          /* O: Sum of all normalized values, and sum of
                               squares                                      */
)
{
  int iptr[32];        /* Pointers to value and location arrays             */
  int ixcol[32];       /* Col locations of 32 largest values--seq by IPTR   */
  int ixrow[32];       /* Row locations of 32 largest values                */
  int ii;              /* Loop index:  pointer to current row of alignments */
  int ipfree;          /* Index to elements in value and loc arrays which are 
                          freed by deleting 32-nd smallest                  */
  int jj;              /* Loop index to element in cur row of alignments    */
  int jstart;          /* Pointer to first element in current row           */
  int jstop;           /* Pointer to last element in current row            */
  int kk;              /* Loop index:  pointer into table of peak values    */

  float xval[32];      /* Temp storage for largest values--seq in IPTR      */
  float rho;           /* Normalized correlation coefficient                */
  float temp;          /* Temporary storage                                 */
  float aa;            /* Coefficient ...                                   */
  float redget;        /* Total edge pixels in reference image              */
  float sigma0;        /* 1st approx to std dev of search chip overlain by
                          reference image in current alignment              */
  float sigma1;        /* Second approximation to search-subarea std dev    */

  
  /* Clear sums and compute constants
     --------------------------------*/
  for (sums[0]=sums[1]=0.0, ii=0; ii<32; ii++)
  {
      xval[ii] = -1.0;
      iptr[ii] = ii;
  }
  redget = (float)nrtot*edenr;
  aa = 1.0/(nrtot*sqrt(edenr*(1.0 - edenr)));
  sigma0 = sqrt(0.67*edenr*(1.0 - 0.67*edenr));

  /* Compute normalized coincidence values
     -------------------------------------*/
  for( jstart=0, jstop=ncol, ii=0; ii<nrow; ii++)
  {
      for(jj=jstart; jj<jstop; jj++)
      {
          temp = edens[jj]*(1.0 - edens[jj]);
          sigma1 = (temp + sigma0*sigma0)/(2.0*sigma0);
          rho = 2.0*aa*sigma1/(temp + sigma1*sigma1)*
                (unormc[jj] - redget*edens[jj]);
          ccnorm[jj] = rho;
          sums[0] += rho;
          sums[1] += rho*rho;

          /* Check whether value among top 32
             --------------------------------*/
          if (rho>xval[iptr[31]])
          {
              kk = 31;
              ipfree = iptr[31];
              for(;;)
              {
                  if((kk <= 0) || (rho <= xval[iptr[kk-1]]))
                      break;
                  iptr[kk]=iptr[kk-1];
                  kk--;
              }

              iptr[kk] = ipfree;
              xval[ipfree] = rho;
              ixcol[ipfree] = jj - jstart;
              ixrow[ipfree] = ii;
          }
      }
      jstart += ncol;
      jstop += ncol;
  }

  /* Copy peak values and coordinates in correct sequence
     ----------------------------------------------------*/
  for(ii=0; ii<32; ii++)
  {
      pkval[ii] = xval[iptr[ii]];
      ipkcol[ii] = ixcol[iptr[ii]];
      ipkrow[ii] = ixrow[iptr[ii]];
  }
}


/****************************************************************************
NAME:                           ECORR

PURPOSE:  Correlate the edge images and evaluate the results

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand

REFERENCES:

1.  LAS 4.0 EDGECORR by R. White 6/83

RETURN VALUES:  E_SUCC  successful completion
                E_FAIL  unsuccessful completion
*****************************************************************************/
static int ecorr
(
    short *imgbns,       /* I: Binary edge image from search subimage       */
    short *imgbnr,       /* I: Binary edge image from reference image       */
    float *npls,         /* I: Actual size of search subimage:  samps,lines */
    float *nplr,         /* I: Actual size of ref. subimage: samps,lines    */
    const float nshrnk,  /* I: #lines and columns by which filtering "shrinks"
                               the subimages; one less than dim. of filter  */
    const float edenr,   /* I: Density of edges in reference image          */
    float *edens,        /* I: Density of search-image edges for each relative 
                               alignment of reference image                 */
    const float csmin,   /* I: Minimum acceptable correlation strength      */
    const int mfit,      /* I: Method of fitting surface:
                                                    1-Elliptical paraboloid 
                                                    2-Elliptical Gaussian 
                                                    3-Reciprocal Paraboloid 
                                                    4-Round to nearest int  */
    float ddmx,          /* I/O: Maximum allowed diagonal displacement from
                                 nominal tiepoint location to location found
                                 by corr                                    */
    float *ioffrq,       /* I: Requested max horiz. and vert. search offsets*/
    float *nomoff,       /* I: Nominal horiz. and vert. offsets of upper left 
                               corner of ref. chip relative to search chip  */
    int *iacrej,         /* O: Accept/Reject code                           */
    float *streng,       /* O: Strength of correlation                      */
    float *bfoffs,       /* O: Best-fit horiz. (pixel) and vert. (line)
                               offsets of corr. peak relative to nominal
                               location                                     */
    float *tlerrs,       /* O: Estimated horiz. error, vert. error, and h-v 
                               cross term in best-fit offsets               */
    float *ddact         /* O: Actual diag. displacement from nominal tiepoint
                               location to location found by correlation    */
)
{
  int ipkcol[32];       /* Col number for each of top 32 correlation values */
  int ipkrow[32];       /* Row number for each of top 32 correlation values */
  int ncol;             /* Number of columns of search alignments           */
  int nrow;             /* Number of rows of search alignments              */
  int nrtot;            /* Total number of pixels in binary reference image */

  float ccnorm[129*129];/* Normalized xcorr coef for each search alignment  */
  float cpval[25];      /* 5 by 5 array of xcorr values, in std dev         */
  float pkoffs[2];      /* Fractional horiz and vert offsets of best-fit 
                           peak location from nearest integer location      */
  float pkval[32];      /* Largest 32 values of normalized xcorr coefs      */
  float sums[2];        /* Sum and sum of squares of all xcorr values       */
  float unormc[129*129];/* Unnormalized edge-coincidence count              */

  
  /* Compute raw edge-coincidence counts
     -----------------------------------*/
  coinc(imgbns, imgbnr, npls, nplr, nshrnk, &ncol, &nrow, unormc);

  /* Compute normalized cross-correlation values and compile statistics
     ------------------------------------------------------------------*/
  nrtot = (int)(((nplr[0] - nshrnk)*(nplr[1] - nshrnk)) + 0.5);
  enorm(nrtot, ncol, nrow, unormc, edenr, edens, ccnorm, pkval, ipkcol,
        ipkrow, sums);

  /* Evaluate strength of correlation peak
     -------------------------------------*/
  eval(ncol, nrow, ccnorm, pkval, ipkcol, ipkrow, csmin, sums, streng,
       iacrej, cpval);

  /* Determine offsets of peak relative to nominal location
     ------------------------------------------------------*/
  if (*iacrej==1)
  {
      if (mfit!=4)
      {
          if (fitreg(cpval, mfit, pkoffs, tlerrs) != E_SUCC)
          {
              c_lasmsg(FALSE, "correlate-edge",
                       "Error encountered while computing subpixel "
                       "correlation peak location.");
              
              return E_FAIL;
          }
          bfoffs[0] = (float)ipkcol[0] - nomoff[0] + pkoffs[0];
          bfoffs[1] = (float)ipkrow[0] - nomoff[1] + pkoffs[1];
      }
      else
      {
          bfoffs[0] = (float)ipkcol[0] - nomoff[0];
          bfoffs[1] = (float)ipkrow[0] - nomoff[1];
          tlerrs[0] = 0.5;
          tlerrs[1] = 0.5;
      }

  /* Determine diagonal displacement from nominal and check against maximum 
     acceptable value
     ----------------------------------------------------------------------*/
      *ddact = sqrt(bfoffs[0]*bfoffs[0] + bfoffs[1]*bfoffs[1]);
      if (ddmx <= 0.0)
          ddmx = ioffrq[0] < ioffrq[1] ? ioffrq[0] - 2.0 : ioffrq[1] - 2.0;
      if (*ddact > ddmx)
          *iacrej = 5;
  }

  return E_SUCC;
}


/****************************************************************************
NAME:                           EDGFIL

PURPOSE:  Applies edge filter to grey-level image array and compile histogram 
          of edge values

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand

REFERENCES:

1.  LAS 4.0 EDGECORR by R. White 6/83 and S. Kondal 7/84

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion
*****************************************************************************/
static int edgfil
(
    float *image,       /* I: Grey-level image                             */
    const float npixel, /* I: Number of pixels per image line              */
    const float nline,  /* I: Number of lines in image                     */
    const int nshift,   /* I: # bits to right-shift to get edge value      */
    const float threshold, /* I: cloud cover threshold value               */
    short *iedge,       /* O: Array of edge values                         */
    float *nshrnk,      /* O: Num of lines/samps that filtering "shrinks"
                              chips                                        */
    int *ihist          /* O: Histogram of edginess values                 */
)
{
  int ii,jj;            /* Loop counters                                    */
  int iedgpt;           /* Index to next location in edginess array         */
  int ifilt;            /* Filter output for one location                   */
  int imgpt;            /* Index to start of next image-array line          */
  int jpt;              /* Index to UL pixel for computing 1 filtered value */
  int lshift;           /* Left shift count (negative)                      */
  int nl;               /* Number of lines in edge array                    */
  int np;               /* Number of samples in edge array                  */
  int npix2;            /* Twice number of samples                          */
  int nround;           /* Rounding term to add before right shift          */
  int pixnew;           /* Integer form of npixel                           */

  
/* Initialize array dimensions and histogram
   ------------------------------------------*/
  *nshrnk = 2.0;
  np = (int)(npixel - *nshrnk + 0.5);
  nl = (int)(nline - *nshrnk + 0.5);
  npix2 = (int)(2.0*npixel);
  pixnew=(int)npixel;

  for(ii=0; ii<HISTOGRAMSIZE; ii++)
      ihist[ii] = 0;
  imgpt = 0;
  iedgpt = 0;

  if (nshift == 0)
  {
      /* Compute edginess values without scaling
         ---------------------------------------*/
      for(ii=0; ii<nl; ii++)
      {
          jpt = imgpt;
          for(jj=0; jj<np; jj++, ihist[ifilt]++, jpt++, iedgpt++)
          {
              if ((image[jpt+npix2+2] > threshold)  ||
                  (image[jpt] > threshold)          ||
                  (image[jpt+npix2+1] > threshold)  ||
                  (image[jpt+1] > threshold)        ||
                  (image[jpt+npix2] > threshold)    ||
                  (image[jpt+2] > threshold)        ||
                  (image[jpt+pixnew+2] > threshold) ||
                  (image[jpt+pixnew] > threshold))
                  ifilt = 0;
              else
                  ifilt = (int)(fabs(image[jpt+npix2+2] - image[jpt]) +
                               fabs(image[jpt+npix2+1] - image[jpt+1]) +
                               fabs(image[jpt+npix2] - image[jpt+2]) +
                               fabs(image[jpt+pixnew+2] - image[jpt+pixnew]));
              iedge[iedgpt] = (short)ifilt;
          }
          imgpt += pixnew;
      }
  }
  else
  {
      /* Compute edginess with scaling
         ------------------------------*/
      nround = (int)pow(2.0, (nshift - 1));
      lshift = nshift;
      for(ii=0; ii<nl; ii++)
      {
          jpt = imgpt;
          for(jj=0; jj<np; jj++, ihist[ifilt]++, jpt++, iedgpt++)
          {
              if ((image[jpt+npix2+2] > threshold)  ||
                  (image[jpt] > threshold)          ||
                  (image[jpt+npix2+1] > threshold)  ||
                  (image[jpt+1] > threshold)        ||
                  (image[jpt+npix2] > threshold)    ||
                  (image[jpt+2] > threshold)        ||
                  (image[jpt+pixnew+2] > threshold) ||
                  (image[jpt+pixnew] > threshold))
                  ifilt = 0; 
              else 
                  ifilt = (int)(fabs(image[jpt+npix2+2] - image[jpt]) +
                               fabs(image[jpt+npix2+1] - image[jpt+1]) +
                               fabs(image[jpt+npix2] - image[jpt+2]) +
                               fabs(image[jpt+pixnew+2] - image[jpt+pixnew]));
              ifilt = (ifilt + nround) >> lshift;
              if (ifilt > 32769)
              {
                  c_lasmsg(FALSE, "correlate-bins", "Too many bins");
                  return E_FAIL;
              }
              iedge[iedgpt] = (short)ifilt;
          } 
          imgpt += pixnew;
      } 
  }
  return E_SUCC;
}


/****************************************************************************
NAME:                           EDGE

PURPOSE:  Determine the edginess value which will yield a percentage of edge 
          pixels not more than, but as close as possible to, the user
          specified edge fraction

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.  LAS 4.0 EDGECORR by R. White 6/83 and S. Kondal 7/84
*****************************************************************************/
static void edge
(
    int *ihist,         /* I: Histogram of edginess values                 */
    const float ntotal, /* I: Total number of values in histogram          */
    const float efract, /* I: Fraction of pixels to be classified as edges */
    int *ithrsh,        /* O: Threshold value                              */
    float *edgden       /* O: Density of edges in image                    */
)
{
  int kbin;               /* Current histogram bin                        */
  int mincnt;             /* Target minimum number of non-edge pixels     */
  int nonedg;             /* Actual number of non-edge pixels             */

  float edgcnt;           /* Number of pixels with edges !< threshold     */

 
  /* Compute minimum number of non-edge pixels
     ------------------------------------------*/
  mincnt = (int)(ntotal*(1.0 - efract));

  /* Find threshold histogram bin
     -----------------------------*/
  nonedg = 0;
  kbin = 0;

  for(;;)
  {
      if (nonedg >= mincnt) break;
      nonedg += ihist[kbin];
      kbin++;
  }
  if ((nonedg - ihist[kbin-1]/2) >= mincnt) 
  {
      kbin--;
      nonedg -= ihist[kbin];
  }

  /* Compute values to be returned to caller
     ---------------------------------------*/
  *ithrsh = kbin;
  edgcnt = ntotal - (float)nonedg;
  *edgden = edgcnt/ntotal;
}


/****************************************************************************
NAME:                           FLGEDG

PURPOSE:  Classify each pixel in edginess array as edge or non-edge

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.  LAS 4.0 EDGECORR by R. White 6/83 and S. KONDAL 7/84
*****************************************************************************/
static void flgedg
(
    short *iedge,               /* I: Array of edge values              */
    const float npixel,         /* I: Number of samples                 */
    const float nline,          /* I: Number of lines                   */
    const int ithrsh,           /* I: Edge threshold value              */
    short *ibin                 /* O: Array of edginess values          */
)
{
  int ii;                /* Loop counter                         */
  int stop;              /* Number of pixels to check            */

 
  /* Compare one pixel at a time with threshold.
     -------------------------------------------*/
  stop = (int)(nline*npixel + 0.5);
  for (ii=0; ii<stop; ii++)
      if (iedge[ii] >= ithrsh) 
          ibin[ii] = 1;
      else 
          ibin[ii] = 0;
}


/****************************************************************************
NAME:                           DENSE

PURPOSE:  Computes the density of edge images of the search image chip 
          overlayed by the reference image chip for each combination of 
          translational offsets

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

 1.  LAS 4.0 EDGECORR by R. White 6/83 and S. KONDAL 7/84
******************************************************************************/
static void dense
(
    short *imgbns,       /* I: Binary edge image from search subimage       */
    float *npls,         /* I: Size of search chip--#samps, #lines          */
    float *nplr,         /* I: Size of reference chip--#samps, #lines       */ 
    const float nshrnk,  /* I: Number of lines/samps which filtering shrinks
                               chips                                        */
    float *edens         /* O: Density of search edges per rel. alignment of
                               ref                                          */
)
{
  int imax;            /* Max index of bin chip that contributes to col sum */
  int index;           /* Loop index into binary search chip                */
  int iroff;           /* Index offset for tops of columns                  */
  int kcol;            /* Loop index--current column of search image        */
  int khalgn;          /* Loop index--current horizontal search alignment   */
  int kvalgn;          /* Loop index--current vertical search alignment     */
  int lroff;           /* Index offset for bottoms of columns               */
  int ncolr;           /* Number of columns in binary reference image       */
  int ncols;           /* Number of columns in binary search image          */
  int ndxden;          /* Index into edge-density array                     */
  int nedge;           /* Total # of edge pixels for particular alignment   */
  int nhalgn;          /* Number of horizontal search alignments            */
  int nrowr;           /* Number of rows in binary reference image          */
  int nrows;           /* Number of rows in binary search image             */
  int nvalgn;          /* Number of vertical search alignments              */
  int kolsum[256];     /* Edge pixel sums in partial cols of binary chip    */
  
  float totrpx;        /* Total pixels in binary reference image            */

 
  /* Compute sizes of arrays
     ------------------------*/
  ncolr = (int)(nplr[0] - nshrnk + 0.5);
  nrowr = (int)(nplr[1] - nshrnk + 0.5);
  ncols = (int)(npls[0] - nshrnk + 0.5);
  nrows = (int)(npls[1] - nshrnk + 0.5);
  totrpx = (float)(ncolr*nrowr);
  nhalgn = ncols - ncolr + 1;
  nvalgn = nrows - nrowr + 1;

  /* Clear column sums
     ------------------*/
  for (kcol=0; kcol<ncols; kcol++)
      kolsum[kcol] = 0;

  imax = ncols*nrowr;
  iroff = -ncols;
  lroff = iroff + imax;
  ndxden = 0;

  for (kvalgn=0; kvalgn<nvalgn; kvalgn++)
  {
      /* Get number of edge pixels in each partial column of search image
         ----------------------------------------------------------------*/
      for (kcol=0; kcol<ncols; kcol++)
      {
          if (kvalgn == 0) 
              for (index = kcol; index < imax ; index += ncols) 
                  kolsum[kcol] += imgbns[index];
          else 
              kolsum[kcol] = kolsum[kcol] - imgbns[iroff+kcol] +
                             imgbns[lroff+kcol];
      }

      iroff += ncols;
      lroff += ncols;

      /* Add up column sums for each alignment and compute density
         ---------------------------------------------------------*/
      nedge = 0;
      for (khalgn=0; khalgn<nhalgn; khalgn++)
      {
          if (khalgn == 0)
              for (kcol=0; kcol<ncolr; kcol++)
                  nedge += kolsum[kcol];
          else
              nedge = nedge - kolsum[khalgn-1] + kolsum[khalgn+ncolr-1];
          edens[ndxden] = (float)nedge/totrpx;
          ndxden++; 
      }
  }
}


/****************************************************************************
NAME:                           EXTBIN

PURPOSE:  Extract binary edge images from the grey-level search and reference 
          subimages

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand

REFERENCES:

1.0  LAS 4.0 EDGECORR by R. White 6/83 and S. Kondal 7/84

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion
*****************************************************************************/
static int extbin
(
    float *imggls,       /* I: Grey-level subimage array from search image  */
    float *imgglr,       /* I: Grey-level subimage array from ref. image    */
    float *npls,         /* I: Actual size of search chip                   */
    float *nplr,         /* I: Actual size of reference chip                */
    int *nscale,         /* I: #bits to scale (right shift) edginess filter */
    const float efract,  /* I: Fraction of pixels to be classified as edges */
    const float threshold, /* I: Cloud cover threshold value                */
    float *nshrnk,       /* O: #lines/samps by which filtering "shrinks"
                               chips                                        */
    short *imgbns,       /* O: Binary edge image from search subimage       */
    short *imgbnr,       /* O: Binary edge image from reference image       */
    float *edens,        /* O: Density of search edges for rel align of ref */
    float *edenr         /* O: Density of edges in reference image          */
)
{
  short iedge[SRCH_BUFSIZE]; /* Edginess values produced by filter          */

  int ihist[HISTOGRAMSIZE];  /* Histogram of edginess values                */
  int ithrsh;           /* Threshold at which a pixel is classified as edge */

  
  /* Filter search subimage
     -----------------------*/
  if (edgfil(imggls, npls[0], npls[1], nscale[0], threshold, iedge,
                  nshrnk, ihist) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-edge",
               "Error encountered while filtering the search subimage.");
      return E_FAIL;
  }
  edge(ihist, ((npls[0]-*nshrnk)*(npls[1]-*nshrnk)), efract, &ithrsh,
       &edens[0]);
  flgedg(iedge, npls[0]-*nshrnk, npls[1]-*nshrnk, ithrsh, imgbns);

  /* Filter reference subimage
     --------------------------*/
  if (edgfil(imgglr, nplr[0], nplr[1], nscale[1], threshold, iedge,
                  nshrnk, ihist) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-edge",
               "Error encountered while filtering the reference subimage.");
      return E_FAIL;
  }
  edge(ihist, ((nplr[0]-*nshrnk)*(nplr[1]-*nshrnk)), efract, &ithrsh, edenr);
  flgedg(iedge, nplr[0]-*nshrnk, nplr[1]-*nshrnk, ithrsh, imgbnr);

  /* Compute search-subarea edge densities for each alignment
     ---------------------------------------------------------*/
  dense(imgbns, npls, nplr, *nshrnk, edens);

  return E_SUCC;
}


/****************************************************************************
NAME:                           edge_corr

PURPOSE:  Performs edge correlation of a tie point 

*****************************************************************************/
void edge_corr
(
    struct CORRPAR *corr, /* I: Correlation parameters                      */
    float *ref,           /* I: Reference subimage                          */
    float *srch,          /* I: Search subimage                             */
    short *bref,          /* I: Binary edge reference image buffer          */
    short *bsrch,         /* I: Binary edge search image buffer             */
    double *nominal,      /* I: Nominal search image coordinates            */
    int *nscale,          /* I: Scale parameter                             */
    const float threshold,/* I: Cloud cover threshold value                 */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct TPLDATA *tpl         /* I/O: Tie point location data record      */
)
{
  int active;                  /* int version of (long)tpl->active          */
  
  float edens[16641];          /* Array of edge density values              */
  float edenr;                 /* Density of edges in reference subimage    */
  float shrink;                /* # of lines & samps that filtering shrinks */
  float efract;                /* Fraction of pixels classified as edges    */
  float mincorr;               /* Minimum acceptable correlation strength   */
  float maxdis;                /* Maximum displacement from nominal allowed */
  float disp = 0.;             /* Displacement from nominal coordinate      */
  float est_err[3];            /* Estimated error of fit                    */
  float best_fit[2];           /* Best fit offsets of correlation peak      */
  float npls[2];               /* Size of search subimage                   */
  float nplr[2];               /* Size of reference subimage                */
  float ioffrq[2];             /* Requested max horiz & vert search offsets */
  float nomoff[2];             /* Nominal offsets                           */
  float strength = 0.;         /* Strength of correlation                   */

  
  /* Assign values of C structure to individual variables
     ----------------------------------------------------*/
  npls[0] = (float)tpl->sea_size[0];
  npls[1] = (float)tpl->sea_size[1];
  nplr[0] = (float)tpl->ref_size[0];
  nplr[1] = (float)tpl->ref_size[1];
  mincorr = (float)corr->mincorr;
  maxdis = (float)corr->maxdiff;
  ioffrq[0] = (float)tpl->offsets[0];
  ioffrq[1] = (float)tpl->offsets[1];
  nomoff[0] = (float)nominal[1];
  nomoff[1] = (float)nominal[0];
  efract = (float)corr->edgeval;

  /* Extract binary edge from subimage
     ---------------------------------*/
  if (extbin(srch, ref, npls, nplr, nscale, efract, threshold, &shrink,
                  bsrch, bref, edens, &edenr) != E_SUCC)
      tpl->active = TPL_STATE_BAD_PARAM;
  else
  {
      /* Perform correlation
         -------------------*/
      if (ecorr(bsrch, bref, npls, nplr, shrink, edenr, edens, mincorr,
                corr->fitmeth, maxdis, ioffrq, nomoff, &active,
                &strength, best_fit, est_err, &disp) != E_SUCC)
          tpl->active = TPL_STATE_BAD_PARAM;
      else
          tpl->active = active;
  }

  tpl->displacement = (double)disp;
  tpl->strength = (double)strength;

  /* If the correlation was successful, compute the new search tie point.
     Otherwise, set the search tie point to the nominal value.
     --------------------------------------------------------------------*/
  if (tpl->active == TPL_STATE_OK_AUTO)
  {
      tpl->sea_coord[0] = tpl->nominal[0] +
                          best_fit[1]*trans[0] + best_fit[0]*trans[1];
      tpl->sea_coord[1] = tpl->nominal[1] +
                          best_fit[1]*trans[3] + best_fit[0]*trans[4];
      tpl->rms_error[0] = est_err[0];
      tpl->rms_error[1] = est_err[1];
  }
  else
  {
      tpl->sea_coord[0] = tpl->nominal[0];
      tpl->sea_coord[1] = tpl->nominal[1];
      tpl->rms_error[0] = 0.0;
      tpl->rms_error[1] = 0.0;
  }
}
