/****************************************************************************
   NAME:                          edge_filter

   FUNCTION: Apply edge enhancement on the input 2D array.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      1/00    G. Schmidt        CSB       FORTRAN to C
                         L. Johnson        CSB       Modified from Picmatch
                                                     edge.c

   RETURN VALUES:  E_SUCC   successful completion
                   E_FAIL   unsuccessful completion

   CHANGES:

$Log: edge_filter.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.2  2001/06/13 16:03:04  ldjohn
CR854 Code review: Reordered passed parameters.

Revision 1.1.2.1  2001/03/15 18:56:40  ldjohn
CR854 Added edge_filter.c and get_transform.c.

****************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "correlate.h"


int edge_filter
(
    const int size,  /* I: Size (N) of the array a  */
    float *aa        /* I/O: 2D array of size N x N */
)
{
  float **bb = NULL;         /* temp storage for 3 columns of array     */
  int fac[3] = {13, 10, 8};  /* self-term multiplier based on proximity
                                to array edge                           */
  int sizep1, sizep2;        /* size + 1, size + 2                      */
  int icol, jrow, kk, ll;    /* loop counters                           */
  int iq;                    /* index for first bb array value          */
  int liq;                   /* last (previous) iq value                */
  int icolm1;                /* current icol value minus 1              */
  int ifac, ifac1;           /* indexes into the fac array              */
  int jrowm1;                /* current jrow value minus 1              */
  int sum;                   /* sum of the bb values                    */
  int kint;                  /* filter result for pixel                 */

  
  /* Initialize sizep1 and sizep2 from size. */
  sizep1 = size + 1;
  sizep2 = size + 2;

  /* Allocate space for bb to be bb[3][sizep2]. */
  if ((bb = (float **)malloc(3*sizeof(float *))) == NULL)
  {
      c_lasmsg (FALSE, "correlate-alloc", "Error allocating memory");
      return E_FAIL;
  }
  for (icol=0; icol<3; icol++)
  {
      /* Initialize to 0. */
      if ((bb[icol] = (float *)calloc(sizep2, sizeof(float))) == NULL)
      {
          for (kk=icol-1; kk>=0; kk--)
              free(bb[kk]);
          free(bb);
          c_lasmsg (FALSE, "correlate-alloc", "Error allocating memory");
          return E_FAIL;
      }
  }

  /* Loop over all columns (plus one). */
  iq = 2;
  for (icol=0; icol<sizep1; icol++)
  {
      /* Set the values for the bb array indices.
         iq  = 0, 1, 2, 0, 1, 2, ...
         liq =    0, 1, 2, 0, 1, ... */
      liq = iq;
      iq++;
      if (iq > 2)
          iq = 0;

      /* If not at last column (plus one), copy the column into the
         temporary buffer.  Otherwise fill the temporary buffer column
         with zeros. */
      if (icol != size)
      {
          for (jrow=0; jrow<size; jrow++)
              bb[iq][jrow+1] = aa[jrow*size+icol];
      }
      else
      {
          for (jrow=1; jrow<sizep1; jrow++)
              bb[iq][jrow] = 0.0;
      }

      /* If this is the first column of aa, we need to read the next
         column before we can apply the filter. */
      if (icol == 0)
          continue;
      
      icolm1 = icol - 1;

      /* Set the self-term multiplier array (fac) index to assume a
         full 3x3 kernel. */
      ifac1 = 0;

      /* If this is the second or last (plus one) column (remember that
         we read ahead one column from the center column), only two
         array columns are used, so the self-term multiplier must be
         reduced appropriately. */
      if (icol == 1 || icol == size)
          ifac1 = 1;

      /* Loop over all rows for the three columns. */
      for (jrow=0; jrow<size; jrow++)
      {
          ifac = ifac1;

          /* If this is the first or last row, only two array
             rows are used, so the self-term multiplier must be reduced
             appropriately. */
          if (jrow == 0 || jrow == size - 1)
              ifac++;

          /* Initialize jrowm1 to jrow-1 and the sum to 0. */
          jrowm1 = jrow - 1;
          sum = 0;

          /* Sum the 3x3 kernel elements. */
          for (kk=0; kk<3; kk++)
              for (ll=0; ll<3; ll++)
                  sum += bb[kk][jrowm1+ll+1];

          /* Apply the filter.  If it is less than zero then set it
             to zero. */
          kint = ((bb[liq][jrow+1]*fac[ifac] - sum)/4.0 > 0.0) ?
                  (bb[liq][jrow+1]*fac[ifac] - sum)/4.0 : 0.0;
          aa[jrow*size+icolm1] = kint;
      }  /* for jrow */
  }  /* for icol */

  /* Free bb pointers. */
  for (icol=0; icol<3; icol++)
  {
      free(bb[icol]);
      bb[icol] = NULL;
  }
  free(bb);

  return E_SUCC;
}
