/****************************************************************************
NAME:                           EVAL

PURPOSE:  Evaluate various measures of correlation validity and extract a 
          subarea of the cross correlation array centered on the peak

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.0  LAS 4.0 GREYCORR and EDGECORR by R. White 6/83

CHANGES:

$Log: eval.c,v $
Revision 6.2  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 6.1.6.2  2001/06/13 16:03:41  ldjohn
CR854 Code review:  Longs to ints.

Revision 6.1.6.1  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <math.h>
#include "correlate.h"


void eval
(
    const int ncol,      /* I: Number of columns in coincidence-count array */
    const int nrow,      /* I: Number of rows in coincidence-count array    */
    const float *ccnorm, /* I: Array of norm xcorr coeffs                   */
    const float *pkval,  /* I: Table of top 32 normalized values            */
    const int *ipkcol,   /* I: Table of column numbers for top 32 values    */
    const int *ipkrow,   /* I: Table of row numbers for top 32 values       */
    const float csmin,   /* I: Minimum acceptable correlation strength      */
    float *sums,         /* O: Sum of all normalized values and sum of
                               squares                                      */
    float *streng,       /* O: Strength of correlation                      */
    int *iacrej,         /* O: Accept/Reject code (see geompak.h)           */
    float *cpval         /* O: 5 by 5 array of xcorr vals in std dev        */
)
{
  int ipt5[2];         /* Highest 2 vals > 2 cols or rows from peak value   */
  int ii, jj;          /* Loop counters                                     */
  int icol;            /* Initial col in 9 by 9 neighborhood of corr peak   */
  int idist;           /* Max horiz or vert distance from array peak        */
  int iptr;            /* Index to 5 by 5 array of output values            */
  int ipt7;            /* Index of largest val > 3 cols or rows from peak   */
  int irow;            /* Initial row in 9 by 9 neighborhood                */
  int krbase;          /* Offset to beginning of current row in array       */
  int lcol;            /* Last col in 9 by 9 area centered on peak          */
  int lrow;            /* Last row in 9 by 9 neighborhood                   */
  int npts;            /* #array values outside 9 by 9 neighborhood         */
  int n5;              /* #large values outside 5 by 5 neighborhood of peak */
  int n7;              /* #large values outside 7 x 7 neighborhood of peak  */
  int tempa,tempb;     /* Temporary storage                                 */
  
  float bmean;         /* Mean of background cross-correlation values       */
  float bsigma;        /* Standard deviation of background about mean       */

  
  /* Initialize accept/reject code, strength, and outlier pointers
     -------------------------------------------------------------*/
  *iacrej = 1;
  *streng = 0.0;
  ipt5[0] = 31;
  ipt5[1] = 31;
  ipt7 = 0;

  /* Check for peak value within two rows or columns of edge
     -------------------------------------------------------*/
  if ((ipkcol[0] <= 1) || (ipkcol[0] >= ncol - 2) ||
      (ipkrow[0] <= 1) || (ipkrow[0] >= nrow - 2))
  {
      *iacrej = 2;
      return;
  }

  /* Find largest values outside 5 by 5 and 7 by 7 neighborhoods of peak
     -------------------------------------------------------------------*/
  n5 = 0;
  n7 = 0;
  ii = 1;

  for(;;)
  {
      if ((n5 > 1) || (ii > 31))
          break;
      tempa = labs(ipkcol[0] - ipkcol[ii]);
      tempb = labs(ipkrow[0] - ipkrow[ii]);
      idist = tempa > tempb ? tempa : tempb;
      if (idist > 2)
      {
          ipt5[n5] = ii;
          n5++;
          if ((n7 == 0) && (idist > 3))
          {
              ipt7 = ii;
              n7 = 1;
          }
      }
      ii++;
  }
  if ((ipt5[0] <= 2) || (ipt5[1] <= 4))
  {
      *iacrej = 3;
      return;
  }

  /* Find edges of 9 by 9 array centered on peak
     -------------------------------------------*/
  icol = 0 > (ipkcol[0] - 4) ? 0 : ipkcol[0] - 4;
  irow = 0 > (ipkrow[0] - 4) ? 0 : ipkrow[0] - 4;
  lcol = (ncol - 1) < (ipkcol[0] + 4) ? ncol - 1 : ipkcol[0] + 4;
  lrow = (nrow - 1) < (ipkrow[0] + 4) ? nrow - 1 : ipkrow[0] + 4;

  /* Eliminate points within 9 by 9 array from background statistics
     ---------------------------------------------------------------*/
  krbase = ncol*irow;
  for(ii=irow; ii<=lrow; ii++)
  {
      for(jj=icol; jj<=lcol; jj++)
      {
          sums[0] -= ccnorm[krbase+jj];
          sums[1] -= ccnorm[krbase+jj]*ccnorm[krbase+jj];
      }
      krbase += ncol;
  }

  /* Compute background mean and standard deviation
     ----------------------------------------------*/
  npts = ncol*nrow - (lcol - icol + 1)*(lrow - irow + 1);
  bmean = sums[0]/(float)npts;
  bsigma = sqrt(sums[1]/(float)npts - bmean*bmean);

  /* Compute correlation strength and check against minimum.  Note that
     the LAS 4.x version of this code contains an error...it often tries
     to access pkval array element zero when there are no signifcant
     subsidiary peaks, resulting in unpredictable results
     -------------------------------------------------------------------*/
  if (n7 == 0)
      *streng = 2*((pkval[0] - bmean)/bsigma) - 0.2;
  else 
      *streng = (pkval[0] - bmean)/bsigma + (pkval[0] - pkval[ipt7])/bsigma
                + 0.2*(n7 - 1.0);

  if (*streng < csmin)
  {
      *iacrej = 4;
      return;
  }

  /* Convert 5 by 5 neighborhood of peak to standard deviations above mean
     ---------------------------------------------------------------------*/
  krbase = ncol*(ipkrow[0] - 2);
  for(iptr=0, ii=0; ii<5; ii++, krbase+=ncol)
      for(jj=ipkcol[0]-2; jj<=ipkcol[0]+2; jj++, iptr++)
          cpval[iptr] = (ccnorm[krbase+jj] - bmean)/bsigma;
}
