/****************************************************************************
NAME:                           extract

PURPOSE:  Extracts a subimage from a full image or reads a chip image

VERSION         DATE    AUTHOR
-------         ----    ------  
  1.0           8/92    D. Steinwand

RETURN VALUES:  E_SUCC          successful completion
                E_FAIL          unsuccessful completion
                READ_ERROR      error opening/reading the chip file
                BAD_SIZE        chip size not as expected
                BAD_CHIP_DDR    unable to retrieve chip DDR
                BAD_COEFS       projection coefficients are invalid
                OUTSIDE_IMAGE   chip extends outside image bounds

CHANGES:

$Log: extract.c,v $
Revision 6.2  2010/09/23 15:35:16  tvaughn
Adding correlate changes

Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.10  2001/06/13 16:04:44  ldjohn
CR854 Code review:  Longs to ints.

Revision 1.1.2.9  2001/05/16 19:33:24  ldjohn
CR854 Corrected error that was not completing the resampling.

Revision 1.1.2.8  2001/05/10 17:33:17  ldjohn
CR854 Removed unused parameter.

Revision 1.1.2.7  2001/05/09 15:34:01  ldjohn
CR854 Removed unused variables.

Revision 1.1.2.6  2001/04/23 21:19:46  ldjohn
CR854 Added freeing of buffer.

Revision 1.1.2.5  2001/04/23 18:21:51  ldjohn
CR854 Removed scale factor stuff.
      Moved bilinear resampling into resample().

Revision 1.1.2.4  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 1.1.2.3  2001/03/13 19:44:16  ldjohn
CR854 Corrected error with get_chip_corners parameters.

Revision 1.1.2.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "correlate.h"


int extract
(
    struct FDESC **ptr,       /* I: File pointer to image file              */
    const double *tp,         /* I: Tie point (l,s)                         */
    const int *size,          /* I: Size of chip--nl,ns                     */
    char *name,               /* I: Name of chip image file                 */
    const int minval,         /* I: Minimum chip size                       */
    const int maxval,         /* I: Maximum chip size                       */
    float *trans,        /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct CORRIMG *image,    /* I/O: Image file parameters                 */
    int *chop,                /* O: Number of grid points outside the image.
                                    Corresponding DN values in the "grid" are
                                    set to zero.                            */
    float *buffer             /* O: Chip buffer                             */
)
{
  float basey, basex;        /* transform-space line/sample of upper-left chip
                                corner */
  const float eps = 0.001;   /* arbitrarily small number */
  float *temp_buf;           /* Temporary buffer for image file read */
  float ystart;              /* first (top) y value for transformed chip */
  
  int fill_line;             /* line counter in fill space */
  int line_sign, samp_sign;  /* line and sample directions of 1st line in
                                transform space: + = down/right,
                                                 - = up/left */
  int status;                /* Return status code */
  int samp_count, line_count;/* maximum number of samples and lines in
                                transform-space chip  */
  int samp_start;            /* leftmost corner of the grid */
  int sl, ss;                /* Starting line and sample of chip */
  
  long line;                 /* Number of line being read from file */
  long one = 1;              /* constant one value */
  long int_y_val;
  
  

  if ( trans[0] != trans[0] ) { /* XXX */
      printf("trans[o] Nan upon entry to extract routine\n");
      return E_FAIL;
  }
  
  /* If processing chip subimages...
     --------------------------------*/
  if (image->chip_flag)
  {
      status = read_chip(name, size, image, buffer);
      if (status != E_SUCC)
          c_lasmsg(FALSE, "correlate-chip", "Error reading %s chip file.",
                   name);
      return status;
  }

  /* ... otherwise extract chips from full image.
     --------------------------------------------*/

  /* Check validity of chip size. */
  if ((size[0] < minval) || (size[0] > maxval) ||
      (size[1] < minval) || (size[1] > maxval))
      return BAD_SIZE;

  image->master_line = image->master_samp = 1;

  /* if ( trans[0] != trans[0] ) { XXX  NaN 
      printf("trans[0] Nan\n");
      return OUTSIDE_IMAGE;
  }
  */

  /* If there is no resampling required, do a quick extraction of the
     image chip.  Otherwise, compute the transformed chip parameters
     and resample the image chip data.  If the transformation
     coefficients are approximately those of the identity
     transformation (1 0 y  0 1 x), no resampling is required.

     Assuming no scaling difference, a difference of 0.001 corresponds
     to a rotation of approx. 2 degrees, which, in turn, corresponds
     to a change of roughly 1 pixel over a 32-pixel span.
     -----------------------------------------------------------------*/
  if ((fabs(1.0 - trans[0]*trans[0]) < eps) &&
      (fabs(trans[1]*trans[1]) < eps) &&
      (fabs(1.0 - trans[4]*trans[4]) < eps) &&
      (fabs(trans[3]*trans[3]) < eps))
  {
      /* Determine image coordinates at which to extract chips. */
      sl = (int)(tp[0] - (size[0]/2.0) + 1.0);
      ss = (int)(tp[1] - (size[1]/2.0) + 1.0);
      
      image->starting_line = sl;
      image->starting_sample = ss;
      
      /* Check window coordinates against image bounds. */
      if((sl < 1) || (ss < 1) || ((sl + size[0]) > image->nl) ||
         ((ss + size[1]) > image->ns))
          return OUTSIDE_IMAGE;
      
      /* Allocate space for temp_buf[#samps]. */
      temp_buf = (float *)malloc(image->ns*sizeof(float));
      if (temp_buf == NULL)
      {
          c_lasmsg (FALSE, "correlate-alloc", "Error allocating memory");
          return E_FAIL;
      }

      ss--;
      for (fill_line=0,line=sl; fill_line<size[0]; fill_line++,line++)
      {
          if (c_eread(ptr, &image->band, &line, (unsigned char *)temp_buf,
                      &one) == E_FAIL)
          {
              free(temp_buf);
              c_lasmsg(FALSE, "correlate-extract",
                       "Error reading image file for chip extraction.");
              return READ_ERROR;
          }
          memcpy(&buffer[fill_line*size[1]], &temp_buf[ss],
                 size[1]*sizeof(float));
      }

      free(temp_buf);
      
      return(E_SUCC);
  }
 
  /* Compute the transformed chip corners.
   ---------------------------------------*/
  if ( trans[0] != trans[0] ) { /* XXX */
      printf("xid[o] Nan before call to get_chip_corners from extract\n");
      exit(1);
  }
  
  get_chip_corners(tp, size, trans, image, &line_sign, &samp_sign, &basey,
                   &basex, &ystart, &line_count, &samp_start, &samp_count);

  if ( trans[0] != trans[0] ) { /* XXX */
      printf("xid[o] Nan after call to get_chip_corners from extract\n");
      exit(1);
  }
  
  /* Set the starting line and sample to zero as notification that this
     location is useless since the chip has been rotated. */
  image->starting_line = 0;
  image->starting_sample = 0;

  /* Check window coordinates against image bounds. */
  /* The problem with the original check is that get_chip_corners can 
   * set line_count to 71 for a 72 line chip.  The bilinear resampler
   * will use 72 for a line count and wants 74 lines for a 72-line chip.
   * It seems logical to guess the sample check has the same problem,
   * but it is not being changed at this time. ~tev 05/2010
   */
  if((ystart < 1) || (samp_start < 1) ||
     /* ((ystart + line_count) > image->nl) || */
     ((ystart + line_count + 1) >= image->nl) ||
     ((samp_start + samp_count) > image->ns))
      return OUTSIDE_IMAGE;

  if( ystart != ystart ) {
      c_lasmsg(FALSE, "correlate-resamp_ystart",
               "ystart Nan after get_chip_corners.");
      return OUTSIDE_IMAGE;
  }
  
  int_y_val = (long)ystart;
  if( (int_y_val < 1) || (int_y_val >= image->nl) ) {
      /*c_lasmsg(FALSE, "correlate-resam_yv",
               "y value bad after get_chip_corners."); */
      return OUTSIDE_IMAGE;
  }
  
  /* Apply bilinear resampling to fill the image chip.
   ---------------------------------------------------*/
  status = resample(ptr, size, trans, image, line_sign, samp_sign, ystart,
                    basey, basex, chop, buffer);
                    
  if ( trans[0] != trans[0] ) { /* XXX */
      printf("xid[o] Nan after call to resample from extract\n");
      exit(1);
  }
  
  if (status != E_SUCC)
  {
      
      /*
      printf( "ystart: %f, basey: %f, basex: %f\n", ystart, basey, basex );
      printf( "line_count: %d, samp_count: %d\n", line_count, samp_count );
      printf( "nl: %d, ns: %d\n", image->nl, image->ns );
      */
      
      c_lasmsg(FALSE, "correlate-resample",
               "Error occurred while resampling the image chip.");
      return status;
  }      

  return E_SUCC;
}
