/****************************************************************************
NAME:       FILL_HEADER

PURPOSE:    Fill the TPL header structure.

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion

                Note: If this routine returns successfully, the file
                      out_tpl must be closed by the caller.
                
REFERENCES:

$Log: fill_tpl_data.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.4  2001/06/13 16:06:07  ldjohn
CR854 Code review:  Passed structures as pointers.

Revision 1.1.2.3  2001/03/15 19:16:18  ldjohn
CR854 Documented return values.
      Changed fill_tpl_data() to type int from void, since adding opening
        of the output TPL file.

Revision 1.1.2.2  2001/03/13 19:45:02  ldjohn
CR854 Added GREYEDGE correlation method.

Revision 1.1.2.1  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "correlate.h"


int fill_header
(
    const struct CORRIMG *ref,       /* I: Reference image file parameters  */
    const struct CORRIMG *srch,      /* I: Search image parameters          */
    const struct CORRPAR *corr,      /* I: Correlation parameters           */
    char *out,                    /* I: Output Tie Point Location file name */
    const int corrmeth,              /* I: Method of correlation used       */
    const struct MTPHEAD *mtp_header,/* I: MTP file header record           */
    const struct TPSHEAD *cut_header,/* I: Cutline TPS file header record   */
    const int inmeth,                /* I: Correlation point input method   */
    struct PTDESC **out_tpl,       /* O: Output tiepoint location file desc */
    struct TPLHEAD *tpl_header       /* O: TPL file header record           */
)
{
  int ii;                         /* Loop counter                           */

  long acc;                       /* File access mode                       */
  long ptyp;                      /* Type of tie point file                 */

  
  tpl_header->edge_thres = corr->edgeval;
  tpl_header->min_str = corr->mincorr;
  tpl_header->max_dis = corr->maxdiff;
  if (corr->fitmeth == PARAB)
      strcpy(tpl_header->peak_method, "PARABOLOID");
  else if (corr->fitmeth == GAUSS)
      strcpy(tpl_header->peak_method, "GAUSSIAN");
  else if (corr->fitmeth == RECIP)
      strcpy(tpl_header->peak_method, "RECIPROCAL");
  else /* corr->fitmeth == INTEGER */
      strcpy(tpl_header->peak_method, "NEAR_INT");
  if (corrmeth == EDGE_CORR)
      strcpy(tpl_header->corr_type, "EDGE");
  else if (corrmeth == GREY_CORR)
      strcpy(tpl_header->corr_type, "GREY");
  else if (corrmeth == GREYEDGE_CORR)
      strcpy(tpl_header->corr_type, "GREYEDGE");
  else /* corrmeth == PHASE_CORR */
      strcpy(tpl_header->corr_type,"PHASE");

  if (inmeth == MTPMETH)
  {
      for (ii = 0; ii < 8; ii++)
          tpl_header->corners[ii] = mtp_header->corners[ii];
      tpl_header->pdist[0] = mtp_header->pdist[0];
      tpl_header->pdist[1] = mtp_header->pdist[1];
      for (ii = 0; ii < 15; ii++)
          tpl_header->projprms[ii] = mtp_header->projprms[ii];
      tpl_header->proj_valid = mtp_header->proj_valid;
      tpl_header->code = mtp_header->code;
      tpl_header->zone = mtp_header->zone;
      tpl_header->datum = mtp_header->datum;
      strcpy(tpl_header->ref_mode, mtp_header->chip_mode);
      strcpy(tpl_header->units, mtp_header->units);
  }
  else if (inmeth == CUTMETH)
  {
      for (ii = 0; ii < 8; ii++)
          tpl_header->corners[ii] = cut_header->corners[ii];
      tpl_header->pdist[0] = cut_header->pdist[0];
      tpl_header->pdist[1] = cut_header->pdist[1];
      for (ii = 0; ii < 15; ii++)
          tpl_header->projprms[ii] = cut_header->projprms[ii];
      tpl_header->proj_valid = cut_header->proj_valid;
      tpl_header->code = cut_header->code;
      tpl_header->zone = cut_header->zone;
      tpl_header->datum = cut_header->datum;
      strcpy(tpl_header->ref_mode, cut_header->mode);
      strcpy(tpl_header->units, cut_header->units);
  }
  else /* no header data available */
  {
      for (ii = 0; ii < 8; ii++)
          tpl_header->corners[ii] = 0.;
      tpl_header->pdist[0] = 0.;
      tpl_header->pdist[1] = 0.;
      for (ii = 0; ii < 15; ii++)
          tpl_header->projprms[ii] = 0.;
      tpl_header->proj_valid = 0;
      tpl_header->code = 0;
      tpl_header->zone = 0;
      tpl_header->datum = 0;
      strcpy(tpl_header->ref_mode, "");
      strcpy(tpl_header->units, "");
  }

  strcpy(tpl_header->ref_name, ref->name);
  strcpy(tpl_header->sea_name, srch->name);
  
  /* Open the output tie point file and write the header. */
  acc = IWRITE;
  ptyp = TPL;
  if (c_ptopen(out_tpl, out, &ptyp, &acc, tpl_header) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-open", "Unable to open %s for writing", out);
      return E_FAIL;
  }

  return E_SUCC;
}


/****************************************************************************
NAME:       FILL_TPL_DATA

PURPOSE:    Fill the TPL data structure.

*****************************************************************************/
void fill_tpl_data
(
    const struct CORRIMG *ref,       /* I: Reference image file parameters  */
    const struct CORRIMG *srch,      /* I: Search image parameters          */
    const struct MTPDATA *mtp_data,  /* I: MTP file data structure          */
    struct TPLDATA *tpl_data         /* O: TPL file data structure          */
)
{
  tpl_data->geo_coord[0] = mtp_data->geo_coord[0];
  tpl_data->geo_coord[1] = mtp_data->geo_coord[1];
  tpl_data->ref_coord[0] = mtp_data->ref_coord[0];
  tpl_data->ref_coord[1] = mtp_data->ref_coord[1];
  tpl_data->sea_coord[0] = mtp_data->nominal[0];
  tpl_data->sea_coord[1] = mtp_data->nominal[1];
  tpl_data->nominal[0] = mtp_data->nominal[0];
  tpl_data->nominal[1] = mtp_data->nominal[1];
  tpl_data->elevation = mtp_data->elevation;
  tpl_data->ref_size[0] = mtp_data->chip_size[0];
  tpl_data->ref_size[1] = mtp_data->chip_size[1];
  tpl_data->req_size[0] = mtp_data->chip_size[0];
  tpl_data->req_size[1] = mtp_data->chip_size[1];
  tpl_data->offsets[0] = mtp_data->offsets[0];
  tpl_data->offsets[1] = mtp_data->offsets[1];
  tpl_data->zoom_factor = mtp_data->zoom_factor;
  strcpy(tpl_data->pt_id, mtp_data->pt_id);
  
  tpl_data->chip_loc[0] = srch->starting_line;
  tpl_data->chip_loc[1] = srch->starting_sample;
  tpl_data->location[0] = ref->starting_line;
  tpl_data->location[1] = ref->starting_sample;
  strcpy(tpl_data->ref_chip, "");
}
