/****************************************************************************
NAME:                           FITREG

PURPOSE:  Fits a quadratic surface to the neighborhood of the correlation peak 
          and from it determine the best-fit registration offsets and their 
          estimated errors

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.0  LAS 4.0 GREYCORR & EDGECORR by R. White 6/83

CHANGES:

$Log: fitreg.c,v $
Revision 6.2  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 6.1.6.6  2001/06/13 16:06:56  ldjohn
CR854 Code review:  Longs to ints.

Revision 6.1.6.5  2001/05/09 15:15:42  ldjohn
CR854 Changes to incorporate use of the GSL (Gnu Scientific Library).

Revision 6.1.6.4  2001/04/23 18:23:12  ldjohn
CR854 Rearranged functions to remove need for internal function prototypes.

Revision 6.1.6.3  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 6.1.6.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "correlate.h"
#include "gsl/gsl_blas.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_permutation.h"


#define GMAX(A, B)     ((A) > (B) ? (A) : (B)) /* assign maximum of a and b */


/****************************************************************************
NAME:                           FILL_MATRIX

PURPOSE:  Compute the terms for the matrix elements and column vector 
          used for determining the best-fit surface to the cross-correlation 
          values in the neighborhood of the peak

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

*****************************************************************************/
static void fill_matrix
(
    const float *cpval, /* I: 5 by 5 array of xcorr values, in std dev     */
    const int mfit,     /* I: Method of fitting surface                    */
    float *zz,          /* O: Function of xcorr to which quadratic is fit  */
    float *wghts,       /* O: Weights to be assigned to points in 5 by 5
                              array                                        */
    double *aa,         /* O: Matrix of polynomial terms                   */
    double *vector      /* O: Col vector that is an approp. function of
                              xcorr value                                  */
)
{
  int iterm,ic,ir;      /* Loop counters                                   */
  int ivalpt;           /* Index of xcorr vals and weights in peak area    */
  
  float term[6];        /* Terms in approximating quadratic polynomial     */
  float val;            /* Value of one element in CPVAL array             */

  
  /* Compute function of correlation coefficient and assign weight for each 
     location in neighborhood of peak
     --------------------------------*/
  term[0] = 1.0;
  for(ivalpt=0,ir=0; ir<5; ir++)
  {
      term[2] = (float)ir - 2.;
      term[5] = term[2]*term[2];      
      for(ic=0; ic<5; ic++,ivalpt++)
      {
          val = GMAX(cpval[ivalpt], 1.0);
          if (mfit == 1)
          {
              zz[ivalpt] = val;
              wghts[ivalpt] = 1.0;
          }
          else if (mfit == 2)
          {
              zz[ivalpt] = -(log(val));
              wghts[ivalpt] = val*val;
          }
          else
          {
              zz[ivalpt] = 1.0/val;
              wghts[ivalpt] = val*val*val*val;
          }
 
         /* Compute matrix and vector elements in linear equations for
            best-fit 
            ----------------------------------------------------------*/
          term[1] = (float)ic - 2.;
          term[3] = term[1]*term[1];
          term[4] = term[1]*term[2];
          vector[ivalpt] = wghts[ivalpt]*zz[ivalpt];
          for(iterm=0; iterm<6; iterm++)
              aa[6*ivalpt+iterm] = wghts[ivalpt]*term[iterm];
      }
  }
}


/****************************************************************************
NAME:                           ESTERR

PURPOSE:  Estimate the probable errors in the best-fit location of the 
          correlation peak

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

*****************************************************************************/
static void esterr
(
    const float *zz,    /* I: Function of xcorr val to which quadratic is
                              fit                                           */
    const float *wghts, /* I: Weights for each point in 5 by 5 array of
                              values                                        */
    const gsl_matrix *mat_anvrs, /* I: Inverse of matrix B                  */
    const double *coeffs,/* I: Coefs of best-fit quad surface to values of Z*/
    const float *pkoffs,/* I: Best-fit offset of peak to center of 5 by 5
                              array                                         */
    float *tlerrs       /* O: Est horiz & vert error & h-v cross term       */
)
{
  int xx, yy;           /* Loop counters                                    */
  int ivalpt;           /* Index to values det from measured corr coeffs    */
  
  double du[6], dv[6],  /* Data storage for vec_du, vec_dv, vec_adu, vec_adv*/
         adu[6], adv[6];
  float cc;             /* Constant for computing weights or errors         */
  float denom;          /* Denominator used for partial derivatives         */
  float ff;              /* Value of function of corr coefficient computed
                           using best-fit coefficients                      */
  double usum;          /* Accum. for sum of terms to determine est errors in 
                           horizontal offset                                */
  float var;            /* Residual variance of fit                         */
  double vsum;          /* Sum of terms to estimate vertical offset error   */
  double xsum;          /* Accum. for sum of cross terms to est errors when
                           search chip was rotated                          */

  gsl_vector vec_du;    /* Partial derivatives of horiz best-fit peak offset
                           with respect to coefs of polynomial surface      */
  gsl_vector vec_dv;    /* Partial derivatives of vertical offset           */
  gsl_vector vec_adu,   /* [A]^{-1}[du] and [A]^{-1}[dv]                    */
             vec_adv;


  /* Define the vectors. */                             
  vec_du.size = 6;
  vec_du.stride = 1;
  vec_du.data = du;
  vec_du.block = NULL;
  vec_du.owner = 0;
  
  vec_dv.size = 6;
  vec_dv.stride = 1;
  vec_dv.data = dv;
  vec_dv.block = NULL;
  vec_dv.owner = 0;
  
  vec_adu.size = 6;
  vec_adu.stride = 1;
  vec_adu.data = adu;
  vec_adu.block = NULL;
  vec_adu.owner = 0;
  
  vec_adv.size = 6;
  vec_adv.stride = 1;
  vec_adv.data = adv;
  vec_adv.block = NULL;
  vec_adv.owner = 0;
  
  /* Compute residual variance for elements of peak array.
     -----------------------------------------------------*/
  ivalpt = 0;
  var = 0.0;
  for(yy= -2;yy<=2;yy++)
      for(xx= -2;xx<=2;xx++)
      {
          ff = coeffs[0] + coeffs[1]*(float)xx + coeffs[2]*(float)yy +
              coeffs[3]*(float)(xx*xx) + coeffs[4]*(float)(xx*yy) +
              coeffs[5]*(float)(yy*yy);
          var += wghts[ivalpt]*(ff - zz[ivalpt])*(ff - zz[ivalpt]);
          ivalpt++;
      }

  /* Compute constant to use with weights.
     -------------------------------------*/
  cc = var/19.0;

  /* Compute partial derivatives of peak offsets with respect to poly coefs.
     -----------------------------------------------------------------------*/
  denom = 4.0*coeffs[3]*coeffs[5] - coeffs[4]*coeffs[4];
  du[0] = 0.0;
  du[1] = -2.0*coeffs[5]/denom;
  du[2] = coeffs[4]/denom;
  du[3] = -4.0*coeffs[5]*pkoffs[0]/denom;
  du[4] = (coeffs[2] + 2.0*coeffs[4]*pkoffs[0])/denom;
  du[5] = (-2.0*coeffs[1] - 4.0*coeffs[3]*pkoffs[0])/denom;
  dv[0] = 0.0;
  dv[1] = du[2];
  dv[2] = -2.0*coeffs[3]/denom;
  dv[3] = (-2.0*coeffs[2] - 4.0*coeffs[5]*pkoffs[1])/denom;
  dv[4] = (coeffs[1] + 2.0*coeffs[4]*pkoffs[1])/denom;
  dv[5] = -4.0*coeffs[3]*pkoffs[1]/denom;

  /* Compute estimated errors in offsets.  [du]'[A]^{-1}[du]
                                           [dv]'[A]^{-1}[dv]
                                           [du]'[A]^{-1}[dv]
     -------------------------------------------------------*/
  gsl_blas_dgemv(CblasNoTrans, 1, mat_anvrs, &vec_du, 0, &vec_adu);
  gsl_blas_dgemv(CblasNoTrans, 1, mat_anvrs, &vec_dv, 0, &vec_adv);
  gsl_blas_ddot(&vec_du, &vec_adu, &usum);
  gsl_blas_ddot(&vec_dv, &vec_adv, &vsum);
  gsl_blas_ddot(&vec_du, &vec_adv, &xsum);
  
  tlerrs[0] = sqrt(fabs(cc*usum));
  tlerrs[1] = sqrt(fabs(cc*vsum));
  tlerrs[2] = cc*xsum;
}


/****************************************************************************
NAME:                           FITREG

PURPOSE:  Fits a quadratic surface to the neighborhood of the correlation peak 
          and from it determine the best-fit registration offsets and their 
          estimated errors

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.0  LAS 4.0 GREYCORR & EDGECORR by R. White 6/83

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion
*****************************************************************************/
int fitreg
(
    const float *cpval, /* I: 5 by 5 array of xcorr vals, in units of standard 
                           dev above background, centered on corr peak      */
    const int mfit,     /* I: Method of surface fit: 1 - Elliptical paraboloid
                                                  2 - Elliptical Gaussian
                                                  3 - Reciprocal Paraboloid */
    float *pkoffs,      /* O: Best-fit horiz and vertical offsets of
                              correlation peak relative to center of 5 by 5
                              array                                         */
    float *tlerrs       /* O: Estimated horizontal error, vertical error, and
                              h-v cross term in best-fit offsets            */
)
{
  int info;            /* Return status for matrix computations            */
  int ipiv[6];         /* Pivot vector for matrix computations             */

  float wghts[25];      /* Weight assigned to each value in area of peak    */
  float zz[25];         /* Xcorr coefficients to which surface is to be fit */
  float denom;          /* Denominator of equation for best-fit offsets     */
  
  double aa[25*6];      /* data array of polynomial terms                   */
  double anorm[36];     /* data array for normal matrix                     */
  double anvrs[36];     /* data array for inverse of normal matrix          */
  double bb[25];        /* data array for right hand side col vector to
                           solve for coefs                                  */
  double bnorm[6];      /* data array for normal RHS vector                 */

  int signum;
  
  gsl_matrix mat_aa;    /* Matrix of polynomial terms                       */
  gsl_matrix mat_anorm; /* Normal matrix                                    */
  gsl_matrix mat_anvrs; /* Inverse of aa                                    */
  gsl_vector vec_bb;    /* Right side column vector to solve for coefs      */
  gsl_vector vec_bnorm; /* Normal RHS vector                                */
  gsl_permutation perm_ipiv; /* Pivot vector                                */


  /* Define matrices and vectors. */
  mat_aa.size1 = 25;
  mat_aa.size2 = 6;
  mat_aa.tda = 6;
  mat_aa.data = aa;
  mat_aa.block = NULL;
  mat_aa.owner = 0;
                             
  mat_anorm.size1 = 6;
  mat_anorm.size2 = 6;
  mat_anorm.tda = 6;
  mat_anorm.data = anorm;
  mat_anorm.block = NULL;
  mat_anorm.owner = 0;
                             
  mat_anvrs.size1 = 6;
  mat_anvrs.size2 = 6;
  mat_anvrs.tda = 6;
  mat_anvrs.data = anvrs;
  mat_anvrs.block = NULL;
  mat_anvrs.owner = 0;
                             
  vec_bb.size = 25;
  vec_bb.stride = 1;
  vec_bb.data = bb;
  vec_bb.block = NULL;
  vec_bb.owner = 0;
  
  vec_bnorm.size = 6;
  vec_bnorm.stride = 1;
  vec_bnorm.data = bnorm;
  vec_bnorm.block = NULL;
  vec_bnorm.owner = 0;
  
  perm_ipiv.size = 6;
  perm_ipiv.data = ipiv;
  
  /* Compute elements of matrix and column vector.
     ---------------------------------------------*/
  fill_matrix(cpval, mfit, zz, wghts, mat_aa.data, vec_bb.data);

  /* Compute the matrix and RHS of the normal equation (A'A)x = (A'b).
     -----------------------------------------------------------------*/
  gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1, &mat_aa, &mat_aa, 0,
                 &mat_anorm);
  gsl_blas_dgemv(CblasTrans, 1, &mat_aa, &vec_bb, 0, &vec_bnorm);

  /* Invert matrix to get best-fit peak offsets
     -------------------------------------------*/
  info = gsl_linalg_LU_decomp(&mat_anorm, &perm_ipiv, &signum);
  if (info != GSL_SUCCESS)
  {
      c_lasmsg(FALSE, "correlate-fit",
               "Error encountered while solving linear equations.");
      return E_FAIL;
  }
  gsl_linalg_LU_svx(&mat_anorm, &perm_ipiv, &vec_bnorm);
  info = gsl_linalg_LU_invert(&mat_anorm, &perm_ipiv, &mat_anvrs);
  if (info != GSL_SUCCESS)
  {
      c_lasmsg(FALSE, "correlate-fit",
               "Error encountered while inverting matrix.");
      return E_FAIL;
  }
  denom = 4.0*bnorm[3]*bnorm[5] - bnorm[4]*bnorm[4];
  pkoffs[0] = (bnorm[2]*bnorm[4] - 2.0*bnorm[1]*bnorm[5])/denom;
  pkoffs[1] = (bnorm[1]*bnorm[4] - 2.0*bnorm[2]*bnorm[3])/denom;

  /* Estimate errors in best-fit offsets
     ------------------------------------*/
  esterr(zz, wghts, &mat_anvrs, bnorm, pkoffs, tlerrs);

  return E_SUCC;
}
