/****************************************************************************
NAME:       GET_CHIP_CORNERS

PURPOSE:    Compute the chip corners in the transform space.

REFERENCES:

CHANGES:

$Log: get_chip_corners.c,v $
Revision 6.2  2010/09/23 15:35:16  tvaughn
Adding correlate changes

Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.5  2001/06/13 16:07:34  ldjohn
CR854 Code review:  Longs to ints.

Revision 1.1.2.4  2001/04/23 18:25:29  ldjohn
CR854 Removed scale-related stuff.
      Cleaned up algorithm to match design document.

Revision 1.1.2.3  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 1.1.2.2  2001/03/13 19:46:10  ldjohn
CR854 Corrected error involving imin parameter.

Revision 1.1.2.1  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "correlate.h"


void get_chip_corners
(
    const double *tp,         /* I: Tie point (l,s)                         */
    const int *size,          /* I: Size of chip--nl,ns                     */
    const float *trans,  /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct CORRIMG *image,    /* I/O: Image file parameters                 */
    int *line_sign,           /* O: line direction of 1st line in transform
                                    space: + = down, - = up                 */
    int *samp_sign,           /* O: sample direction of 1st line in transform
                                    space: + = right, - = left              */
    float *basey,             /* O: transform-space line of upper-left chip
                                    corner                                  */
    float *basex,             /* O: transform-space sample of upper-left chip
                                    corner                                  */
    float *ystart,            /* O: first (top) y value for transformed chip*/
    int *line_count,          /* O: num. of transform-space lines in chip   */
    int *samp_start,          /* O: leftmost transform-space sample in chip */
    int *samp_count           /* O: num. of transform-space samples in chip */
)
{
  int ii;                    /* Loop counter */
  int imin;                  /* uppermost corner of the transformed quad */
  int line_end;              /* lowest line of the chip */
  int samp_end;              /* rightmost sample of the chip */
  const int  line_signs[4] = {1, 1, -1, -1}, samp_signs[4] = {1, -1, 1, -1};
                             /* the four possible directions for the
                                1st line of the transformed chip:
                                 1,  1: down and to the right
                                 1, -1: down and to the left
                                -1,  1: up and to the right
                                -1, -1: up and to the left                 */

  float shift[2];            /* Vector to add to the transformed
                                grid to bring its center to "tp",
                                [0] = y, [1] = x                           */
  float xx[4];               /* transform-space sample coordinates of four
                                chip corners */
  float yy[4];               /* line coordinates of four chip corners
                                in transform space */


  /* Compute the location of the upper left corner of the transformed chip. */
  shift[0] = tp[0] - 0.5*(size[0]*trans[0] + size[1]*trans[1]);
  shift[1] = tp[1] - 0.5*(size[1]*trans[3] + size[1]*trans[4]);

  /* Find the uppermost corner of the transformed quad and the
     line, sample directions (in transform space) of the (non-transformed)
     sample direction. */
  yy[0] = shift[0];
  yy[1] = shift[0] + trans[0]*size[0];
  yy[2] = shift[0] + trans[1]*size[1];
  yy[3] = shift[0] + trans[0]*size[0] + trans[1]*size[1];
  imin = 0;
  line_end = yy[0];
  for (ii = 1; ii < 4; ii++)
  {
      if (yy[ii] < yy[imin])
          imin = ii;
      if (yy[ii] > line_end)
          line_end = yy[ii];
  }
  *samp_sign = samp_signs[imin];
  *line_sign = line_signs[imin];
  *ystart = yy[imin];
  /* XXX
  if ( *ystart != *ystart ) {
      printf("ystart NaN\n" );
      printf("yy[imin] = %f\n", yy[imin] );
      printf("imin = %d\n", imin );
      printf("line_end = %d\n", line_end );
      printf("tp[0] = %f\n", tp[0] );
      printf("tp[1] = %f\n", tp[1] );
      printf("trans[0] = %f\n", trans[0] );
      printf("trans[1] = %f\n", trans[1] );
      exit(1);
  }
  */
  *line_count = line_end - *ystart;

  /* Find the sample coordinates of the leftmost and rightmost corners
     of the grid. */
  xx[0] = shift[1];
  xx[1] = shift[1] + trans[3]*size[0];
  xx[2] = shift[1] + trans[4]*size[1];
  xx[3] = shift[1] + trans[3]*size[0] + trans[4]*size[1];
  *samp_start = xx[0];
  samp_end = xx[0];
  for (ii = 1; ii < 4; ii++)
  {
      if (xx[ii] < *samp_start)
          *samp_start = xx[ii];
      if (xx[ii] > samp_end)
          samp_end = xx[ii];
  }
  
  /* Give us a little breathing room by reading a few extra samples
     on each end of the line. */
  *samp_start = (*samp_start - 10) > 0 ? (*samp_start - 10) : 0;
  *samp_count = samp_end - *samp_start + 10;

  /* Set up parameters for the loop. */
  *basey = shift[0];
  *basex = shift[1];
}
