/****************************************************************************
NAME:                           get_tp

PURPOSE:  Get the next tie point coordinate pair.

CHANGES:

$Log: get_tp.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.6  2001/06/13 16:08:43  ldjohn
CR854 Code review:  Improved error messages.  Passed structures as pointers.
      Longs to ints.

Revision 1.1.2.5  2001/04/23 18:34:34  ldjohn
CR854 Rearranged functions to remove prototypes of internal functions.
      Moved chip size setting for phase corr. from main().

Revision 1.1.2.4  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 1.1.2.3  2001/03/13 19:48:13  ldjohn
CR854 Fixed format for generation of new tie point IDs.
      Fixed handling of cutline files and grid specifications.

Revision 1.1.2.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <string.h>
#include <math.h>
#include "las.h"
#include "correlate.h"

#define MAX_INTERMEDIATE_CUTLINE_PTS 1000


/****************************************************************************
NAME:                           get_man_tp

PURPOSE:  Gets a tie point coordinate from the user

VERSION         DATE    AUTHOR
-------         ----    ------  
  1.0           8/92    D. Steinwand

RETURN VALUES:  E_SUCC   successful completion
                E_EOF    end of user input
*****************************************************************************/
static int get_man_tp
(
    const int corrmeth,   /* I: Correlation method                          */
    const int pt_num,     /* I: Prompt number (counter)                     */
    struct MTPDATA *data  /* I/O: One Merged Tie Point data record
                                  (user input)                              */
)
{
  TEXT mess[80];          /* Buffer for user messages and input           */
  CODE term;              /* Terminal flag for t_io                       */
  

  /* Do until a valid point id is entered
     ------------------------------------*/
  for (;;)
  {
      t_write(" ",T_STDCC);
      sprintf(mess," %3d.  Point identifier >  ",pt_num+1);
      t_write(mess,T_NULL);
      t_read(mess, &term);
      
      /* Check for end of entry
         ----------------------*/
      if (strlen(mess) == 0)
          return E_EOF;

      /* Check that a unique point id was entered
         ----------------------------------------*/
      sscanf(mess, "%s", data->pt_id);
      if (c_ckptid(data->pt_id))
          break;
      t_write(" Point id is not unique; re-enter!", T_STDCC);
  }

  /* Get the reference coordinates
     -----------------------------*/
  for (;;)
  {
      sprintf(mess,"       Reference line, sample >                        ");
      t_write(mess,T_NULL);
      t_read(mess, &term);
      if (sscanf(mess, "%lf %lf", &(data->ref_coord[0]),
                 &(data->ref_coord[1])) == 2)
          break;
      t_write(" Invalid entry; enter 2 space-separated numbers.", T_STDCC);
  }

  /* Get the nominal search coordinates
     ----------------------------------*/
  for (;;)
  {
      sprintf(mess,"       Nominal search line, sample >                   ");
      t_write(mess,T_NULL);
      t_read(mess, &term);
      if (sscanf(mess, "%lf %lf", &(data->nominal[0]),
                 &(data->nominal[1])) == 2)
          break;
      t_write(" Invalid entry; enter 2 space-separated numbers.", T_STDCC);
  }

  /* Get the ref chip size
     ----------------------*/
  for (;;)
  {
      sprintf(mess,
              "       Sub-image width, height [64,64] >               ");
      t_write(mess,T_NULL);
      t_read(mess, &term);
      if (strlen(mess) == 0) 
      {
          data->chip_size[0] = data->chip_size[1] = 64;
          break;
      }
      else 
      {
          if (sscanf(mess, "%ld %ld", &(data->chip_size[0]),
                     &(data->chip_size[1])) == 2)
              break;
      }
      t_write(" Invalid entry; enter 2 space-separated numbers", T_STDCC);
      t_write(" (or simply hit return to use the default values).", T_STDCC);
  }

  /* Get the search offsets
     -----------------------*/
  for (;;)
  {
      sprintf(mess,
              "       Horizontal, vertical search offsets [16,16] >   ");
      t_write(mess,T_NULL);
      t_read(mess, &term);
      if (strlen(mess) == 0) 
      {
          data->offsets[0] = data->offsets[1] = 16;
          break;
      }
      else 
      {
          if (sscanf(mess, "%ld %ld", &(data->offsets[0]), 
                     &(data->offsets[1])) == 2)
              break;
      }
      t_write(" Invalid entry; enter 2 space-separated numbers", T_STDCC);
      t_write(" (or simply hit return to use the default values).", T_STDCC);
  }

  data->elevation = 0.0;
  data->zoom_factor = 0.0;
  data->geo_coord[0] = data->geo_coord[1] = 0.0;
  strcpy(data->ref_chip, "");
  strcpy(data->sea_chip, "");

  return E_SUCC;
}


/****************************************************************************
NAME:                           set_tp

PURPOSE:  Sets tie point data defaults.

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion
*****************************************************************************/
static int set_tp
(
    const int corrmeth,   /* I: Correlation method                          */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct MTPDATA *data  /* I/O: One Merged Tie Point data record          */
)
{
  int ii = 0;             /* Loop counter                                 */
  
  char c_ii[5];           /* Character form of ii                         */
  char pt_id[20];         /* Locally generated point ID.                  */


  /* Copy structure element point ID to local variable. */
  strcpy(pt_id, data->pt_id);
      
  /* Get the point ID.  Check that it is unique, incrementing the ID until
     it is unique.
     ---------------------------------------------------------------------*/
  while (!c_ckptid(pt_id) && ii < MAX_INTERMEDIATE_CUTLINE_PTS)
  {
      ii++;
      /* Append counter to point ID. */
      sprintf(c_ii, "_%d", ii);
      strcpy(pt_id, data->pt_id);
      strcat(pt_id, c_ii);
  }
  if (ii == MAX_INTERMEDIATE_CUTLINE_PTS)
  {
      c_lasmsg(FALSE, "correlate-ptid",
               "Couldn't generate a unique point ID");
      return E_FAIL;
  }

  /* Copy local point ID back to structure element. */
  strcpy(data->pt_id, pt_id);

  /* Compute the nominal search coordinates.
     ---------------------------------------*/
  data->nominal[0] = trans[0]*data->ref_coord[0] +
                     trans[1]*data->ref_coord[1] + trans[2];
  data->nominal[1] = trans[3]*data->ref_coord[0] +
                     trans[4]*data->ref_coord[1] + trans[5];

  /* Get the ref chip size
     ----------------------*/
  data->chip_size[0] = data->chip_size[1] = 64;

  /* Get the search offsets
     -----------------------*/
  data->offsets[0] = data->offsets[1] = 16;

  data->elevation = 0.0;
  data->zoom_factor = 0.0;
  data->geo_coord[0] = data->geo_coord[1] = 0.0;
  strcpy(data->ref_chip, "");
  strcpy(data->sea_chip, "");

  return E_SUCC;
}


/****************************************************************************
NAME:                           get_tp

PURPOSE:  Get the next tie point coordinate pair.

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion
                E_EOF    end of file
*****************************************************************************/
int get_tp
(
    const int corrmeth,       /* I: Method of correlation used              */
    const int inmeth,         /* I: Input correlation point method          */
    struct PTDESC *in_mtp,    /* I: Input merged tie point file desc        */
    struct PTDESC *in_cutline,/* I: Input cutline tie point file desc       */
    const int cutinc,         /* I: cutline point increment or step size    */
    const struct CORRGRID *grid, /* I: Tie point grid parameters            */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    const struct RETRY *retry,/* I: Retry specification                     */
    int *pt_num,              /* I/O: Current point number (counter)        */
    struct MTPDATA *mtp_data  /* I/O: One Merged Tie Point data record      */
)
{
  struct TPSDATA cut_data;       /* Cutline tie point selection data rec.   */
      
  static int first_pass = TRUE;  /* Boolean flag for whether this is the
                                    time reading a point.                   */
  static int icut = 0;           /* cutline point step counter              */
  static int last_point = FALSE; /* Boolean flag for whether or not we have
                                    already read the last point in the file */
  static int next_point = TRUE;  /* Boolean flag for whether or not to read
                                    the next cutline point from file        */
  static int num_cut_steps;      /* Number of step to take between cutline
                                    file points                             */
  int row, column;               /* Grid row and column location            */
  int status;                    /* Return status variable                  */

  static double base_ref_coord[2];/* base reference coor. for retry offsets */
  static double prev_ref_coord[2],/* previous and next input reference      */
                next_ref_coord[2];/* coordinates for cutline computations   */
  double rdist;                  /* distance between to cutline file points */
  static double ry, rx;          /* line/sample differences between two
                                    cutline file points                     */

  static float grid_row_step,    /* step sizes (in pixels) for grid         */
      grid_col_step;

  char c_pt_num[5];              /* character form of the point number      */
  static char prev_pt_id[20],    /* previous and next input point IDs       */
              next_pt_id[20];


  /* If this is a retry, simply modify the reference point and get a new
     search point using the affine transformation. */
  if (retry->count != 0)
  {
      mtp_data->ref_coord[0] = base_ref_coord[0] +
                               retry->distance*retry->direction[retry->count];
      mtp_data->ref_coord[1] = base_ref_coord[1] +
                             retry->distance*retry->direction[retry->count+5];
      mtp_data->nominal[0] = trans[0]*mtp_data->ref_coord[0] +
                             trans[1]*mtp_data->ref_coord[1] + trans[2];
      mtp_data->nominal[1] = trans[3]*mtp_data->ref_coord[0] +
                             trans[4]*mtp_data->ref_coord[1] + trans[5];
      return E_SUCC;
  }

  /* Read or compute the next tiepoint location, based on the input method.
   ------------------------------------------------------------------------*/
  switch (inmeth)
  {
      case MANMETH:
          status = get_man_tp(corrmeth, *pt_num, mtp_data);
          break;
          
      case MTPMETH:
          status = c_ptio(&in_mtp, mtp_data);
          break;
          
      case CUTMETH:
          
          /* If needed, read the next cutline point from file. */
          if (next_point)
          {
              status = c_ptio(&in_cutline, &cut_data);
              if (status == E_EOF)
              {
                  /* We have reached the end of the file, so we must handle
                     the last point before exiting. */
                  if (last_point)
                      return E_EOF;
                  last_point = TRUE;
                  status = E_SUCC;
                  next_ref_coord[0] = prev_ref_coord[0];
                  next_ref_coord[1] = prev_ref_coord[1];
                  strcpy(next_pt_id, prev_pt_id);
              }
              else if (status != E_SUCC)
                  return E_FAIL;
              else
              {
                  next_ref_coord[0] = cut_data.image_coord[0];
                  next_ref_coord[1] = cut_data.image_coord[1];
                  strcpy(next_pt_id, cut_data.pt_id);
              }
              
              /* If we are just starting, we need to get the 2nd point
                 for computation of any intermediate tie points along
                 the cut line. */
              if (first_pass)
              {
                  prev_ref_coord[0] = next_ref_coord[0];
                  prev_ref_coord[1] = next_ref_coord[1];
                  strcpy(prev_pt_id, next_pt_id);
                  status = c_ptio(&in_cutline, &cut_data);
                  if (status == E_FAIL)
                      return E_FAIL;
                  next_ref_coord[0] = cut_data.image_coord[0];
                  next_ref_coord[1] = cut_data.image_coord[1];
                  strcpy(next_pt_id, cut_data.pt_id);
                  first_pass = FALSE;
              }

              /* Compute the number of steps required between the two
                 file points, and reset the step counter. */
              ry = next_ref_coord[0] - prev_ref_coord[0];
              rx = next_ref_coord[1] - prev_ref_coord[1];
              rdist = sqrt(rx*rx + ry*ry);
              num_cut_steps = (int)(rdist/cutinc) + 1;
              icut = 0;
              next_point = FALSE;
          } /* next_point */
          else
              status = E_SUCC;
          
          /* Compute the next cutline location. */
          mtp_data->ref_coord[0] = prev_ref_coord[0] + icut*ry/num_cut_steps;
          mtp_data->ref_coord[1] = prev_ref_coord[1] + icut*rx/num_cut_steps;
          
          /* Get the point ID. */
          strcpy(mtp_data->pt_id, prev_pt_id);
      
          icut++;

          if (icut >= num_cut_steps)
          {
              prev_ref_coord[0] = next_ref_coord[0];
              prev_ref_coord[1] = next_ref_coord[1];
              strcpy(prev_pt_id, next_pt_id);
              next_point = TRUE;
          }

          /* Set generic point data in the MTP data structure. */
          if (set_tp(corrmeth, trans, mtp_data) != E_SUCC)
          {
              c_lasmsg(FALSE, "correlate-settp",
                       "Error setting tie point data");
              return E_FAIL;
          }
          break;
          
      case GRIDMETH:
          
          /* If this is the first time computing a point, set up a few
             static variables. */
          if (first_pass)
          {
              grid_row_step = (grid->lry - grid->uly)/(grid->nrows - 1);
              grid_col_step = (grid->lrx - grid->ulx)/(grid->ncols - 1);
              first_pass = FALSE;
          }

          /* Compute the grid row and column number for this point.
             If the row number exceeds the grid specification, return the
             status flag stating that we are done. */
          row = *pt_num/grid->ncols;
          column = *pt_num % grid->ncols;
          if (row > grid->nrows)
              return E_EOF;

          /* Compute the reference pixel location. */
          mtp_data->ref_coord[0] = grid->uly + row*grid_row_step;
          mtp_data->ref_coord[1] = grid->ulx + column*grid_col_step;
      
          /* Set the point ID. */
          sprintf(c_pt_num, "%d", *pt_num+1);
          strcpy(mtp_data->pt_id, "cor_");
          strcat(mtp_data->pt_id, c_pt_num);
      
          /* Set generic point data in the MTP data structure. */
          if (set_tp(corrmeth, trans, mtp_data) != E_SUCC)
          {
              c_lasmsg(FALSE, "correlate-settp",
                       "Error setting tie point data");
              return E_FAIL;
          }

          status = E_SUCC;
          
          break;
          
      default:
          c_lasmsg(FALSE, "correlate-inmeth",
                   "Invalid tie point input method.");
          return E_FAIL;
  }

  /* Set the chip size for phase correlation. */
  if (corrmeth == PHASE_CORR)
  {
      mtp_data->chip_size[0] = 32;
      mtp_data->chip_size[1] = 32;
  }
  
  /* Store the reference point as the base reference coordinate for
     any future retries.
     --------------------------------------------------------------*/
  base_ref_coord[0] = mtp_data->ref_coord[0];
  base_ref_coord[1] = mtp_data->ref_coord[1];
  
  /* Increment the point counter.
     ----------------------------*/
  if (status == E_SUCC)
      (*pt_num)++;

  return status;
}
