/****************************************************************************
NAME:                           get_transform

PURPOSE:    Generate the affine transformation coefficients based on the
            least-squares solution.

REFERENCES:

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion

CHANGES:

$Log: get_transform.c,v $
Revision 6.2  2010/09/23 15:35:16  tvaughn
Adding correlate changes

Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.8  2001/12/06 13:44:51  ldjohn
CR854 Changed variable type to reflect change in GSL definition.

Revision 1.1.2.7  2001/06/19 19:00:41  ldjohn
CR854 Passing residual storage into get_transform().

Revision 1.1.2.6  2001/06/15 20:34:39  ldjohn
CR4098 Updated to use GSL 0.8.

Revision 1.1.2.5  2001/06/13 16:09:15  ldjohn
CR854 Code review:  Longs to ints.

Revision 1.1.2.4  2001/05/14 14:43:38  ldjohn
CR854 Changed least squares matrix handling to correct behavior.

Revision 1.1.2.3  2001/05/09 15:15:42  ldjohn
CR854 Changes to incorporate use of the GSL (Gnu Scientific Library).

Revision 1.1.2.2  2001/04/23 18:36:59  ldjohn
CR854 Moved support arrays out of function to improve flexibility.

Revision 1.1.2.1  2001/03/15 18:56:40  ldjohn
CR854 Added edge_filter.c and get_transform.c.

*****************************************************************************/
#include "las.h"
#include "correlate.h"
#include "gsl/gsl_linalg.h"


int get_transform
(
    const int numtie,         /* I: number of tie points used for least-
                                    squares fit of transform coefficients   */
    float irtie[],            /* I: input reference tie points              */
    float istie[],            /* I: input search tie points                 */
    double aa[],              /* I: Work space to hold N x 3 coefficient
                                    matrix                                  */
    double bb[],              /* I: Work space to hold N x 2 right hand side
                                    matrix                                  */
    double residual[],        /* I: Work space for the residual vector
                                    (N elements)                            */
    float xg[6]               /* O: 2 x 3 least squares solution matrix     */
)
{
  int ii;                 /* Loop counter */

  double tau[3];          /* Storage associated with the tau vector */
  double xx[3];           /* Storage associated with the least squares
                             solution vector */

  gsl_matrix mat_aa;      /* Least squares input matrix */
  gsl_matrix mat_bb;      /* Least squares input right hand sides */
  gsl_vector_view vec_bb;      /* Single column of mat_bb */
  gsl_vector vec_residual;/* Least squares residuals */
  gsl_vector vec_tau;     /* Householder coefficients from QR factorization */
  gsl_vector vec_xx;      /* Lesat squares solution vector */
  

  /* Define the vectors and matrices. */
  mat_aa.size1 = numtie;
  mat_aa.size2 = 3;
  mat_aa.tda = 3;
  mat_aa.data = aa;
  mat_aa.block = NULL;
  mat_aa.owner = 0;
  
  mat_bb.size1 = numtie;
  mat_bb.size2 = 2;
  mat_bb.tda = 2;
  mat_bb.data = bb;
  mat_bb.block = NULL;
  mat_bb.owner = 0;
  
  vec_residual.size = numtie;
  vec_residual.stride = 1;
  vec_residual.data = residual;
  vec_residual.block = NULL;
  vec_residual.owner = 0;
  
  vec_tau.size = 3;
  vec_tau.stride = 1;
  vec_tau.data = tau;
  vec_tau.block = NULL;
  vec_tau.owner = 0;
  
  vec_xx.size = 3;
  vec_xx.stride = 1;
  vec_xx.data = xx;
  vec_xx.block = NULL;
  vec_xx.owner = 0;
  
  /* Process tie points to set up the affine transform.
     {x_n}' = xg[0]*x_n + xg[1]*y_n + xg[2]
     {y_n}' = xg[3]*x_n + xg[4]*y_n + xg[5]

     Solve AX = B, where

                            +-        -+
                            | a_1  a_2 |
     A = [[x] [y] [1]], X = | b_1  b_2 |, B = [[x'] [y']]
                            | c_1  c_2 |
                            +-        -+
     ----------------------------------------------------*/

  /* Copy the input reference-search transformation tiepoint values
     to the least squares input matrices.
     --------------------------------------------------------------*/
  for (ii=0; ii<numtie; ii++)
  {
      aa[3*ii] = irtie[2*ii];
      aa[3*ii+1] = irtie[2*ii+1];
      aa[3*ii+2] = 1.0;
      bb[2*ii] = istie[2*ii];
      bb[2*ii+1] = istie[2*ii+1];
  }

  /* Perform the least squares fit on the tie points.
     --------------------------------------------------*/
  if (gsl_linalg_QR_decomp(&mat_aa, &vec_tau) != GSL_SUCCESS)
  {
      c_lasmsg(FALSE, "correlate-lsq",
               "Error performing least squares fit.");
      return E_FAIL;
  }
  vec_bb = gsl_matrix_column(&mat_bb, 0); /* column 1 of bb */
  if (gsl_linalg_QR_lssolve(&mat_aa, &vec_tau, &vec_bb, &vec_xx,
                            &vec_residual) != GSL_SUCCESS)
  {
      c_lasmsg(FALSE, "correlate-lsq",
               "Error performing least squares fit.");
      return E_FAIL;
  }
  if ( xx[0] != xx[0] ) { /* NaN XXX */
      /* printf( "NaN in get_transform 1\n"); */
      /* Short-circuit this now, and return so the point can be skipped */
      return E_SUCC;
  }
  for (ii=0; ii<3; ii++)
      xg[ii] = xx[ii];
  
  vec_bb = gsl_matrix_column(&mat_bb, 1); /* column 2 of bb */
  if (gsl_linalg_QR_lssolve(&mat_aa, &vec_tau, &vec_bb, &vec_xx,
                            &vec_residual) != GSL_SUCCESS)
  {
      c_lasmsg(FALSE, "correlate-lsq",
               "Error performing least squares fit.");
      return E_FAIL;
  }
  if ( xx[0] != xx[0] ) { /* NaN XXX */
      printf( "NaN in get_transform 2\n");
      exit(1);
  }
  for (ii=0; ii<3; ii++)
      xg[3+ii] = xx[ii];
  if ( xg[0] != xg[0] ) { /* NaN XXX */
      printf( "NaN in get_transform 2a\n");
      exit(1);
  }
  
  return E_SUCC;
}
