/****************************************************************************
NAME:                           get_tvals

PURPOSE:  Retrieve the tie point pairs to use for computation of the initial
          reference-search space affine transformation coefficients.

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion

CHANGES:

$Log: get_tvals.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.8  2001/06/19 19:00:41  ldjohn
CR854 Passing residual storage into get_transform().

Revision 1.1.2.7  2001/06/13 16:11:17  ldjohn
CR854 Code review:  Longs to ints. Error checking both TVAL1 and TVAL2.

Revision 1.1.2.6  2001/05/14 14:43:38  ldjohn
CR854 Changed least squares matrix handling to correct behavior.

Revision 1.1.2.5  2001/05/09 21:02:21  ldjohn
CR854 Removed erroneous sizeof() from row size of matrices.

Revision 1.1.2.4  2001/05/09 15:15:42  ldjohn
CR854 Changes to incorporate use of the GSL (Gnu Scientific Library).

Revision 1.1.2.3  2001/04/23 18:40:01  ldjohn
CR854 Changed TVAL computations for MTP data to use all tie points, compute
      the least squares transformation, and apply to three reference points.
      Added printed warning for assumed identity reference/search
      relationship.

Revision 1.1.2.2  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 1.1.2.1  2001/03/13 19:49:13  ldjohn
CR854 Extracted (and extended) from existing code to simplify layout.

*****************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "correlate.h"

#define MAX_IN_TIE 10 /* max. number of input tiepoints used to define the
                         initial reference-search space transformation */


int get_tvals
(
    struct PARBLK *vblk,      /* I: TAE variable block                      */
    char *mtp,                /* I: Name of the input merged tie point file */
    const int inmeth,         /* I: Input correlation point method          */
    float irtie[],            /* O: Tie points for first input image (maximum
                                    of MAX_IN_TIE tiepoint pairs)           */
    float istie[],            /* O: Tie points for second input image (maximum
                                    of MAX_IN_TIE tiepoint pairs)           */
    int *numtie               /* O: Number of tie points used for least-
                                    squares fit of transform coefficients   */
)
{
  struct VARIABLE *varref,      /* pointers to a TAE variable in vblk */
                  *varsrch;     /* for reference and search variables */
  struct MTPHEAD mtp_header;    /* MTP file header record */
  struct MTPDATA mtp_data;      /* Merged Tie Point data record */
  struct PTDESC *in_mtp;        /* Input merged tie point file desc */

  int citie;                    /* number of tie point values in TVAL1 */
  int cotie;                    /* number of tie point values in TVAL2 */
  int ii;                       /* loop counter */
  int mtp_total;                /* number of tie points in the INMT file */
  int status = E_SUCC;          /* Return status variable */
  
  long acc;                     /* File access mode */
  long ptyp;                    /* Type of tie point file */

  float *mtp_ref, *mtp_srch;    /* reference and search tie point data
                                   from INMT file */
  float trans[6];           /* affine transformation coefficients:
                               y_s = trans[0]*y_r + trans[1]*x_r + trans[2]
                               x_s = trans[3]*y_r + trans[4]*x_r + trans[5] */
  double *worka;            /* Work space to hold N x 3 coefficient matrix */
  double *workb;            /* Work space to hold N x 2 right hand side
                               matrix */
  double *workr;            /* Work space for the least squares residual
                               vector */

  
  /* Determine the method for generating the initial transformation tie
     points.
     ------------------------------------------------------------------*/
  varref = p_find(vblk, "TVAL1");
  varsrch = p_find(vblk, "TVAL2");
  citie = varref->v_count;
  cotie = varsrch->v_count;
  if (citie != 0 || cotie != 0)
  {
      /* TVAL1 and/or TVAL2 have been specified, so use them. */

      /* Error check input values. */
      if (citie > 2*MAX_IN_TIE)
      {
          c_lasmsg(FALSE, "correlate-toolarge",
                   "The number of values in TVAL1 exceeds the "
                   "valid %d tiepoint pairs", MAX_IN_TIE);
          status = E_FAIL;
      }
      else if (citie < 6)
      {
          c_lasmsg(FALSE, "correlate-toofew",
                   "A minimum of 3 tiepoint pairs is required for TVAL1");
          status = E_FAIL;
      }
      if (citie % 2 != 0)
      {
          c_lasmsg(FALSE, "correlate-multiple",
                   "The number of values in TVAL1 must be a multiple of 2");
          status = E_FAIL;
      }
      
      if (cotie > 2*MAX_IN_TIE)
      {
          c_lasmsg(FALSE, "correlate-toolarge",
                   "The number of values in TVAL2 exceeds the "
                   "valid %d tiepoint pairs", MAX_IN_TIE);
          status = E_FAIL;
      }
      else if (cotie < 6)
      {
          c_lasmsg(FALSE, "correlate-toofew",
                   "A minimum of 3 tiepoint pairs is required for TVAL2");
          status = E_FAIL;
      }
      if (cotie % 2 != 0)
      {
          c_lasmsg(FALSE, "correlate-multiple",
                   "The number of values in TVAL2 must be a multiple of 2");
          status = E_FAIL;
      }
      
      /* See if the same number of tiepoints are used in both reference
         space and search space.
         --------------------------------------------------------------*/
      if (citie != cotie)
      {
          c_lasmsg(FALSE, "correlate-unequal",
                   "The number of values in TVAL1 and TVAL2 are not equal");
          status = E_FAIL;
      }

      /* If an input error has been detected exit. */
      if (status != E_SUCC)
          return E_FAIL;

      /* Read the tiepoint pair values. */
      for (ii = 0; ii < citie; ii++)
      {
          irtie[ii] = RVAL(*varref, ii);
          istie[ii] = RVAL(*varsrch, ii);
      }

      *numtie = citie/2;   /* actual number of tiepoint pairs */
  } /* TVAL1/TVAL2 specified */
  else if (inmeth == MTPMETH)
  {
      /* No tvals have been specified, so extract the tie point
         pairs from INMT.
         ------------------------------------------------------*/

      /* Open the merged tie point file containing the points to be
         correlated, retrieve the number of points, and close the file.
         --------------------------------------------------------------*/
      ptyp = MTP;
      acc = IREAD;
      if (c_ptopen(&in_mtp, mtp, &ptyp, &acc, &mtp_header) != E_SUCC)
      {
          c_lasmsg(FALSE, "correlate-open", "Unable to open %s for reading",
                   mtp);
          return E_FAIL;
      }
      mtp_total = 0;
      for (;;) /* Loop until all tiepoints have been read. */
      {
          status = c_ptio(&in_mtp, &mtp_data);
          if (status == E_EOF)
              break;
          else if (status != E_SUCC)
          {
              c_ptclse(&in_mtp);
              c_lasmsg(FALSE, "correlate-mtp",
                      "Error encountered while retrieving merged tie point.");
              return E_FAIL;
          }
          mtp_total++;
      }
      c_ptclse(&in_mtp);

      if (mtp_total > 2)
      {
          /* Allocate storage for the MTP reference and search tie points. */
          if ((mtp_ref = (float *)malloc(2*mtp_total*sizeof(float))) == NULL)
          {
              c_lasmsg(TRUE, "correlate-alloc",
                       "Unable to allocate dynamic memory");
              return E_FAIL;
          }
          if ((mtp_srch = (float *)malloc(2*mtp_total*sizeof(float))) == NULL)
          {
              free(mtp_ref);
              c_lasmsg(TRUE, "correlate-alloc",
                       "Unable to allocate dynamic memory");
              return E_FAIL;
          }

          /* Reread the MTP file and compute the affine transformation
             coefficients.
             ---------------------------------------------------------*/
          if (c_ptopen(&in_mtp, mtp, &ptyp, &acc, &mtp_header) != E_SUCC)
          {
              free(mtp_ref);
              free(mtp_srch);
              c_lasmsg(FALSE, "correlate-open",
                       "Unable to open %s for reading", mtp);
              return E_FAIL;
          }
          for (ii=0; ii<mtp_total; ii++)
          {
              if (c_ptio(&in_mtp, &mtp_data) != E_SUCC)
              {
                  free(mtp_ref);
                  free(mtp_srch);
                  c_ptclse(&in_mtp);
                  c_lasmsg(FALSE, "correlate-mtp",
                      "Error encountered while retrieving merged tie point.");
                  return E_FAIL;
              }

              mtp_ref[2*ii] = mtp_data.ref_coord[0];
              mtp_ref[2*ii+1] = mtp_data.ref_coord[1];
              mtp_srch[2*ii] = mtp_data.nominal[0];
              mtp_srch[2*ii+1] = mtp_data.nominal[1];
          }
          c_ptclse(&in_mtp);

          /* Allocate storage for the work arrays. */
          if ((worka = (double *)malloc(3*mtp_total*sizeof(double))) == NULL)
          {
              free(mtp_ref);
              free(mtp_srch);
              c_lasmsg(TRUE, "correlate-alloc",
                       "Unable to allocate dynamic memory");
              return E_FAIL;
          }
          if ((workb = (double *)malloc(2*mtp_total*sizeof(double))) == NULL)
          {
              free(mtp_ref);
              free(mtp_srch);
              free(worka);
              c_lasmsg(TRUE, "correlate-alloc",
                       "Unable to allocate dynamic memory");
              return E_FAIL;
          }
          if ((workr = (double *)malloc(mtp_total*sizeof(double))) == NULL)
          {
              free(mtp_ref);
              free(mtp_srch);
              free(worka);
              free(workb);
              c_lasmsg(TRUE, "correlate-alloc",
                       "Unable to allocate dynamic memory");
              return E_FAIL;
          }

          /* Process the merged tie points to set up the affine transform. */
          status = get_transform(mtp_total, mtp_ref, mtp_srch, worka,
                                 workb, workr, trans);

          /* Free used memory. */
          free(mtp_ref);
          free(mtp_srch);
          free(worka);
          free(workb);
          free(workr);
          
          if (status != E_SUCC)
          {
              c_lasmsg(FALSE, "correlate-trans",
                       "Error generating transform coefficients");
              return E_FAIL;
          }
      
          /* Create 3 points that represent the affine transformation. */
          irtie[0] = 0.;
          irtie[1] = 0.;
          irtie[2] = 0.;
          irtie[3] = 1.;
          irtie[4] = 1.;
          irtie[5] = 1.;
          istie[0] = trans[0]*irtie[0] + trans[1]*irtie[1] + trans[2];
          istie[1] = trans[3]*irtie[0] + trans[4]*irtie[1] + trans[5];
          istie[2] = trans[0]*irtie[2] + trans[1]*irtie[3] + trans[2];
          istie[3] = trans[3]*irtie[2] + trans[4]*irtie[3] + trans[5];
          istie[4] = trans[0]*irtie[4] + trans[1]*irtie[5] + trans[2];
          istie[5] = trans[3]*irtie[4] + trans[4]*irtie[5] + trans[5];
      }
      else
      {
          /* MTP file contains less than 3 points, so must revert
             to assuming identity transformation. */
          c_lasmsg(FALSE, "correlate-tvals",
                   "Warning:  No input transformation points from "
                   "either TVAL1/2 or INMT.  Assuming images are "
                   "in same rotation and scale.");
          irtie[0] = 0.;
          irtie[1] = 0.;
          irtie[2] = 0.;
          irtie[3] = 1.;
          irtie[4] = 1.;
          irtie[5] = 1.;
          for (ii=0; ii<6; ii++)
              istie[ii] = irtie[ii];
      }
      *numtie = 3;
  } /* using MTP data */
  else
  {
      /* No tvals or INMT specified, so assume the reference/search images
         are coregistered and set the tvals equal.
      --------------------------------------------------------------------*/

      /* Set up 3 default affine tvals. (0,0), (0,1), (1,1) */
      c_lasmsg(FALSE, "correlate-tvals",
               "Warning:  No input transformation points from either "
               "TVAL1/2 or INMT.  Assuming images are in same rotation "
               "and scale.");
      irtie[0] = 0.;
      irtie[1] = 0.;
      irtie[2] = 0.;
      irtie[3] = 1.;
      irtie[4] = 1.;
      irtie[5] = 1.;
      for (ii=0; ii<6; ii++)
          istie[ii] = irtie[ii];
      *numtie = 3;
  }

  return E_SUCC;
}
