/****************************************************************************
NAME:                           getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

VERSION         DATE    AUTHOR
-------         ----    ------  
  1.0           8/92    D. Steinwand

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion

CHANGES:

$Log: getpar.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.7  2001/06/13 16:12:39  ldjohn
CR854 Code review:  Longs to ints. Replaced CORRIMG.nbytes with dtype element.

Revision 1.1.2.6  2001/04/26 16:26:28  ldjohn
CR854 Set the chip flags to false for full image cases.

Revision 1.1.2.5  2001/04/23 18:41:22  ldjohn
CR854 Removed scaling references.
      Added catch to ensure that the retry option is not applied to chips.

Revision 1.1.2.4  2001/03/15 19:20:37  ldjohn
CR854 Documented return values.
      Rearranged parameter retrieval to agree with design document.
      Added edge enhancement flag for phase correlation.

Revision 1.1.2.3  2001/03/13 20:44:08  ldjohn
CR854 Reverted back to subcommands (removing METHOD parameter), and added
        GREYEDGE correlation method.
      Created get_tvals() by passing off code section for retrieving the
        values for TVAL1 and TVAL2.
      Added to the RETRY data initialization.

Revision 1.1.2.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "typlim.h"
#include "worgen.h"
#include "correlate.h"


int getpar
(
    struct PARBLK *vblk,      /* I: TAE variable block                      */
    struct CORRIMG *ref,      /* O: Name of the reference image             */
    struct CORRIMG *srch,     /* O: Name of the search image                */
    char *mtp,                /* O: Name of the input merged tie point file */
    char *cutline,            /* O: Name of the input cutline file          */
    int *cutinc,              /* O: Cutline point increment or step size    */
    struct CORRGRID *grid,    /* O: Tie point grid parameters               */
    int *inmeth,              /* O: Input correlation point method          */
    char *tpl,              /* O: Name of the input tie point location file */
    char *out,              /* O: Name of the output tie point location file*/
    struct CORRPAR *corr,     /* O: Correlation parameters                  */
    struct RETRY *retry,      /* O: Retry specification                     */
    int *ss,                  /* O: Search chip size for phase corr.        */
    int *inc,                 /* O: Search chip step size for phase corr.   */
    float *alpha,             /* O: Exponential factor for phase corr.      */
    int *hpf,                 /* O: High-pass filter flag for phase corr.   */
    int *phase_edge,          /* O: Edge enhancement flag for phase corr.   */
    int *corrmeth,            /* O: Correlation method                      */
    int *confirm,             /* O: Confirm mode--boolean                   */
    float *threshold,         /* O: Cloud cover threshold value             */
    float irtie[],            /* O: Tie points for first input image (maximum
                                    of MAX_IN_TIE tiepoint pairs)           */
    float istie[],            /* O: Tie points for second input image (maximum
                                    of MAX_IN_TIE tiepoint pairs)           */
    int *numtie               /* O: Number of tie points used for least-
                                    squares fit of transform coefficients   */
)
{
  struct VARIABLE *var;         /* pointer to a TAE variable in vblk */
  
  long acc;                     /* access flag for c_getnam */
  const long zero = 0;          /* Constant of zero */
  const long one = 1;           /* Constant of one */
  long num_img;                 /* Number of images entered */
  long wind[1][4];              /* Window entered */
  long bands[1][MAXBND + 1];    /* Band numbers entered */
  long nbands;                  /* Number of bands */
  long totbands;                /* Total number of bands entered */
  long same;                    /* Same window flag for c_getin call */
  long qflag[1][2];             /* Option flag for c_getin call */
  
  char tdum[1][CMLEN];          /* Dummy variable for c_getin call */
  
  double minval, maxval;        /* min/max pixel values for input images */


  /* Initialize the correlation point input method to manual.
     --------------------------------------------------------*/
  *inmeth = MANMETH;
  
  /* Get the correlation method.
     ---------------------------*/
  var = p_find(vblk,"_SUBCMD");
  if (strncmp("EDGE", SVAL(*var, 0), 4) == 0)
      *corrmeth = EDGE_CORR;
  else if (strncmp("GREYEDGE", SVAL(*var, 0), 8) == 0)
      *corrmeth = GREYEDGE_CORR;
  else if (strncmp("PHASE", SVAL(*var, 0),5) == 0)
      *corrmeth = PHASE_CORR;
  else
      *corrmeth = GREY_CORR;

  /* Get the name of the reference image
     -----------------------------------*/
  ref->minval = ref->maxval = 0;
  ref->dtype = EBYTE;
  var = p_find(vblk, "IN");
  if (strcmp("",SVAL(*var,0)) != 0)
  {
      if (c_getin(vblk, "IN", &zero, tdum, ref->name, &num_img, wind,bands,
                  &nbands, &totbands, &one, &same, qflag) != E_SUCC)
          return E_FAIL;
      ref->band = bands[0][0];

      /* find minmax value for EDGE subcommand
         --------------------------------------*/
      if (*corrmeth == EDGE_CORR || *corrmeth == GREYEDGE_CORR)
      {
          c_minmax(ref->name, &one, wind, bands, &one, &minval, &maxval);
   
          if (minval > MAXLONG)
              ref->minval = MAXLONG;
          else if (minval < MINLONG)
              ref->minval = MINLONG;
          else
              ref->minval = (long)minval;

          if (maxval > MAXLONG)
              ref->maxval = MAXLONG;
          else if (maxval < MINLONG)
              ref->maxval = MINLONG;
          else
              ref->maxval = (long)maxval;
      }

      c_gettyp(ref->name, &one, &ref->dtype);
      
      ref->chip_flag = FALSE;
  }
  else
  {
      strcpy(ref->name, "");
      ref->chip_flag = TRUE;
  }

  /* Get the name of the search image
     --------------------------------*/
  srch->minval = srch->maxval = 0;
  srch->dtype = EBYTE;
  var = p_find(vblk, "IN");
  if (strcmp("",SVAL(*var,1)) != 0)
  {
      if (c_getin(vblk, "IN", &one, tdum, srch->name, &num_img, wind,
                  bands, &nbands, &totbands, &one, &same, qflag) != E_SUCC)
          return E_FAIL;
      srch->band = bands[0][0];

      /* find minmax value for EDGE subcommand
         --------------------------------------*/
      if (*corrmeth == EDGE_CORR || *corrmeth == GREYEDGE_CORR)
      {
          c_minmax(srch->name, &one, wind, bands, &one, &minval, &maxval);

          if (minval > MAXLONG)
              srch->minval = MAXLONG;
          else if (minval < MINLONG)
              srch->minval = MINLONG;
          else
              srch->minval = (long)minval;

          if (maxval > MAXLONG)
              srch->maxval = MAXLONG;
          else if (maxval < MINLONG)
              srch->maxval = MINLONG;
          else
              srch->maxval = (long)maxval;
      }

      c_gettyp(srch->name, &one, &srch->dtype);
      
      srch->chip_flag = FALSE;
  }
  else
  {
      strcpy(srch->name,"");
      srch->chip_flag = TRUE;
  }

  /* Get the output tie point location file name
     -------------------------------------------*/
  acc = IWRITE;
  if (c_getnam(vblk, "OUTTL", &zero, out, "tpl", &acc) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-infile", "Error retrieving the OUTTL name");
      return E_FAIL;
  }

  /* Get the input tie point location file name
     ------------------------------------------*/
  acc = IREAD;
  if (c_getnam(vblk, "INTL", &zero, tpl, "tpl", &acc) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-infile", "Error retrieving the INTL name");
      return E_FAIL;
  }

  /* Get the input merged tie point file name
     ----------------------------------------*/
  if (c_getnam(vblk, "INMT", &zero, mtp, "mtp", &acc) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-infile", "Error retrieving the INMT name");
      return E_FAIL;
  }
  
  /* If an input MTP file is given, set the input correlation point method.
     If an input MTP file wasnt't given, srch & ref images must be.
     ----------------------------------------------------------------------*/
  if (strcmp(mtp, "") != 0)
      *inmeth = MTPMETH;
  else if (srch->chip_flag || ref->chip_flag)
  {
      c_lasmsg(FALSE, "correlate-chips",
               "Must input image names if INMT is not used");
      return E_FAIL;
  }

  /* Get the input cutline file name.
     --------------------------------*/
  if (c_getnam(vblk, "CUTLINE", &zero, cutline, "tps", &acc) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-infile",
               "Error retrieving the CUTLINE name");
      return E_FAIL;
  }
  if (strcmp(cutline, "") != 0)
  {
      /* Set the input correlation point method. */
      if (*inmeth != MANMETH)
      {
          c_lasmsg(FALSE, "correlate-incnt",
                  "Too many input methods specified for correlation points.");
          return E_FAIL;
      }
      *inmeth = CUTMETH;

      /* Get the point increment or step size for the cutline. */
      var = p_find(vblk, "PNTINC");
      *cutinc = IVAL(*var, 0);
  }

  /* Get the grid specification.
     ---------------------------*/
  var = p_find(vblk, "GRIDSPEC");
  if (var->v_count != 0)
  {
      /* Set the input correlation point method. */
      if (*inmeth != MANMETH)
      {
          c_lasmsg(FALSE, "correlate-incnt",
                  "Too many input methods specified for correlation points.");
          return E_FAIL;
      }
      *inmeth = GRIDMETH;

      /* Get the corner coordinates. */
      if (var->v_count != 4)  /* must have 4 points (ul, ur, ll, lr) */
      {
          c_lasmsg(FALSE, "correlate-grdspec",
                   "Missing grid specification.  Four GRIDSPEC values need "
                   "to be specified.");
          return E_FAIL;
      }
      grid->uly = RVAL(*var, 0);
      grid->ulx = RVAL(*var, 1);
      grid->lry = RVAL(*var, 2);
      grid->lrx = RVAL(*var, 3);

      /* Get the number of columns in the grid. */
      var = p_find(vblk, "NCOLS");
      if (var->v_count == 0)
      {
          c_lasmsg(FALSE, "correlate-ncols",
                   "Missing grid number of columns");
          return E_FAIL;
      }
      grid->ncols = IVAL(*var, 0);

      /* Get the number of rows in the grid. */
      var = p_find(vblk, "NROWS");
      if (var->v_count == 0)
      {
          c_lasmsg(FALSE, "correlate-nrows",
                   "Missing grid number of rows");
          return E_FAIL;
      }
      grid->nrows = IVAL(*var, 0);
  }

  /* Check the runtype. If runtype is batch and the input method is
     manual, abort.
     --------------------------------------------------------------*/
  var = p_find(vblk, "$RUNTYPE");
  if ((strcmp(SVAL(*var, 0), "INTERACTIVE") != 0) && (*inmeth == MANMETH))
  {
      c_lasmsg(FALSE, "correlate-batch",
               "Cannot run in batch with manual point input");
      return E_FAIL;
  }

  /* Get correlation parameters
     --------------------------*/
  var = p_find(vblk, "MINCORR");
  corr->mincorr = RVAL(*var, 0);

  var = p_find(vblk, "MAXDIFF");
  corr->maxdiff = RVAL(*var, 0);

  if (*corrmeth != PHASE_CORR)
  {
      var = p_find(vblk, "FITMETH");
      if (strcmp(SVAL(*var,  0), "PARAB") == 0)
          corr->fitmeth = PARAB;
      else if (strcmp(SVAL(*var,  0), "GAUSS") == 0)
          corr->fitmeth = GAUSS;
      else if (strcmp(SVAL(*var,  0), "RECIP") == 0)
          corr->fitmeth = RECIP;
      else /* strcmp(SVAL(*var,  0), "INTEGER") == 0 */
          corr->fitmeth = INTEGER;
  }
  else
  {
      var = p_find(vblk, "SIZE");
      *ss = IVAL(*var, 0);
      var = p_find(vblk, "STEP");
      *inc = IVAL(*var, 0);
      var = p_find(vblk, "EXPVAL");
      *alpha = RVAL(*var, 0);
  
      /* Get high-pass filter flag. */
      var = p_find (vblk, "HPFFLG");
      if (strcmp(SVAL(*var, 0), "YES") == 0)
          *hpf = TRUE;
      else
          *hpf = FALSE;

      /* Get the edge enhancement flag. */
      var = p_find (vblk, "EDGEFLG");
      if (strcmp(SVAL(*var, 0), "YES") == 0)
          *phase_edge = TRUE;
      else
          *phase_edge = FALSE;
  }

  if (*corrmeth == EDGE_CORR || *corrmeth == GREYEDGE_CORR)
  {
      var = p_find(vblk, "EDGEVAL");
      corr->edgeval = RVAL(*var, 0);
      var = p_find(vblk, "THRVAL");
      *threshold = RVAL(*var, 0);
  }
  else
      corr->edgeval=0.0;

  /* Get the confirm mode
     --------------------*/
  *confirm = FALSE;
  var = p_find(vblk, "CONFLG");
  if (strcmp(SVAL(*var, 0), "YES") == 0)
      *confirm = TRUE;

  /* Get the tie points in TVAL1, TVAL2.
   -------------------------------------*/
  if (get_tvals(vblk, mtp, *inmeth, irtie, istie, numtie) != E_SUCC)
  {
      c_lasmsg(FALSE, "correlate-tvals",
               "Error encountered while retrieving TVAL1/TVAL2");
      return E_FAIL;
  }
  
  /* Get the retry option parameters.
   ----------------------------------*/
  var = p_find (vblk, "RETSPEC");
  if (var->v_count == 3)
  {
    retry->total = IVAL(*var, 0);
    if (retry->total > 4)
      retry->total = 4;
    retry->threshold = (float)IVAL(*var, 1);
    retry->distance = (float)IVAL(*var, 2);

    /* The retry option cannot be applied if image chips are being used. */
    if (srch->chip_flag || ref->chip_flag)
    {
        c_lasmsg(FALSE, "correlate-chips",
                 "RETRY cannot be applied with image chips");
        return E_FAIL;
    }
  }
  else
  {
    retry->total = 0;
    retry->threshold = corr->mincorr;
    retry->distance = 100.;
  }
  retry->count = 0;
  /* The retry directions are the 5 different offsets for the reference
     chip: ( 0, 0) = nominal location,
           ( 1, 1) = down and right, (-1,-1) = up and left,
           ( 1,-1) = up and right,   (-1, 1) = down and left */
  retry->direction[0] = retry->direction[5] = 0.;
  retry->direction[1] = retry->direction[3] = retry->direction[6] =
      retry->direction[9] = 1.;
  retry->direction[2] = retry->direction[4] = retry->direction[7] =
      retry->direction[8] = -1;
  
  return E_SUCC;
}
