/****************************************************************************
NAME:                           grey_corr

PURPOSE:  Performs grey (cross) correlation for a tie point

VERSION         DATE    AUTHOR
-------         ----    ------  
  1.0           8/92    D. Steinwand

CHANGES:

$Log: grey.c,v $
Revision 6.3  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 6.2.6.7  2001/06/13 16:23:21  ldjohn
CR854 Code review:  Longs to ints.  Using #define for buffer sizes.  Added
      #defines for active flag values.

Revision 6.2.6.6  2001/05/09 15:15:42  ldjohn
CR854 Changes to incorporate use of the GSL (Gnu Scientific Library).

Revision 6.2.6.5  2001/04/23 18:42:31  ldjohn
CR854 Rearranged functions to remove internal function prototypes.

Revision 6.2.6.4  2001/03/15 19:14:52  ldjohn
CR854 Changed edge_corr and grey_corr to type int and passing success/
        failure status through tpl->active flag.
      Documented return values.

Revision 6.2.6.3  2001/03/13 20:39:57  ldjohn
CR854 Aesthetic changes.

Revision 6.2.6.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <math.h>
#include "geompak.h"
#include "correlate.h"
#include "gsl/gsl_complex_math.h"
#include "gsl/gsl_fft_complex.h"


/****************************************************************************
NAME:                           CROSS

PURPOSE:  Compute the unnormalized (raw) sum of pixel-by-pixel cross products 
          between the reference and search images for every combination of 
          horizontal and vertical offsets of the reference relative to the 
          search image

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.  LAS 4.0 GREYCORR by R. White & G. Neal 8/83
*****************************************************************************/
static void cross
(
    float *images,     /* I: Search subimage                                */
    float *imager,     /* I: Reference subimage                             */
    float *npls,       /* I: Actual size of search subimage:  samps/lines   */
    float *nplr,       /* I: Actual size of reference subimage: samps/lines */
    float *unormc      /* O: Array of unnormalized (raw) counts of edge 
                             coincidences for each alignment of reference
                             image relative to search image                 */
)
{
  int imgptr;             /* Pointer into subimage arrays                   */
  int line;               /* Loop index:  current buffer line               */
  int lncont;             /* Pointer to start of next segment of line       */
  int lnstrt;             /* Pointer to first segment of line               */
  int memptr;             /* Loop index:  pointer into AP transfer buffers  */
  int ncol;               /* Number of columns in cross-product sums array  */
  int ndxout;             /* Pointer into array for correlation output      */
  int nrow;               /* Number of rows in cross-product sums array     */
  int nsnew[2];           /* integer version of npls                        */
  int nrnew[2];           /* integer version of nplr                        */
  int memdim[2];          /* Power-of-2 dimensions for AP arrays            */
  int ii,jj;              /* Loop counters                                  */
  
  float membfr[SRCH_BUFSIZE]; /* Buffer in which reference subimage is
                                 properly aligned and zero padded           */
  float membfs[SRCH_BUFSIZE]; /* Buffer for aligning and padding search
                                 subimage                                   */
  
  gsl_complex cser[SRCH_BUFSIZE]; /* Complex of search subimage             */
  gsl_complex cref[SRCH_BUFSIZE]; /* Complex of reference subimage          */

  
  nsnew[0] = (int)npls[0];
  nsnew[1] = (int)npls[1];
  nrnew[0] = (int)nplr[0];
  nrnew[1] = (int)nplr[1];

  /* Zero extend search image to next higher power of 2
     --------------------------------------------------*/
  for (ii=0; ii<2; ii++)
  {
      memdim[ii] = 64;
      while (nsnew[ii] > memdim[ii])
          memdim[ii] *= 2;
  }

  /* Fix for incorrect handling of non-square arrays
     -----------------------------------------------*/
  memdim[0] = memdim[0] > memdim[1] ? memdim[0] : memdim[1];
  memdim[1] = memdim[0];
  for (lnstrt=0,imgptr=0,line=0; line<memdim[1]; line++)
  {
      if (line < nsnew[1])
      {
          lncont = lnstrt + nsnew[0];
          for (memptr=lnstrt; memptr<lncont; imgptr++,memptr++)
              membfs[memptr] = images[imgptr];
      }
      else
          lncont = lnstrt;
      lnstrt += memdim[0];
      if (lncont < lnstrt)
          for (memptr=lncont; memptr<lnstrt; memptr++)
              membfs[memptr] = 0.0;
  }

  /* Make complex 2-d of search data
     -------------------------------*/
  for (memptr=0,ii=0; ii<memdim[1]; ii++)
      for (jj=0; jj<memdim[0]; jj++,memptr++)
          GSL_SET_COMPLEX(&cser[jj*memdim[1]+ii], (double)membfs[memptr], 0.);

  /* Now zero extend reference image
     -------------------------------*/
  for (lnstrt=0,imgptr=0,line=0; line<memdim[1]; line++)
  {
      if (line < nrnew[1]) 
      {
          lncont = lnstrt + nrnew[0];
          for (memptr=lnstrt; memptr<lncont; imgptr++,memptr++)
              membfr[memptr] = imager[imgptr];
      }
      else
          lncont = lnstrt;
      lnstrt += memdim[0];
      if (lncont < lnstrt) 
          for (memptr=lncont; memptr<lnstrt; memptr++)
              membfr[memptr] = 0.0;
  }

  /* Make complex 2-d of reference data
     ----------------------------------*/
  for (memptr=0, ii=0; ii<memdim[1]; ii++)
      for (jj=0; jj<memdim[0]; jj++, memptr++)
          GSL_SET_COMPLEX(&cref[jj*memdim[1]+ii], (double)membfr[memptr], 0.);

  /* Take FFT of search and reference data
     -------------------------------------*/
  for (ii=0; ii<memdim[0]; ii++) /* row FFTs */
  {
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cser[ii*memdim[1]], 1, memdim[1]);
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cref[ii*memdim[1]], 1, memdim[1]);
  }
  for (ii=0; ii<memdim[1]; ii++) /* column FFTs */
  {
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cser[ii], memdim[1], memdim[0]);
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cref[ii], memdim[1], memdim[0]);
  }

  /* Make point by multiplication of search subimage ft with conjugate of 
     reference subimage FT
     --------------------------------------------------------------------*/
  for(ii=0; ii<memdim[0]*memdim[1]; ii++)
      cser[ii] = gsl_complex_mul(cser[ii], gsl_complex_conjugate(cref[ii]));

  /* Take inverse FFT of cser
     ------------------------*/
  for (ii=0; ii<memdim[0]; ii++) /* row FFTs */
      gsl_fft_complex_radix2_inverse(
          (gsl_complex_packed_array)&cser[ii*memdim[1]], 1, memdim[1]);
  for (ii=0; ii<memdim[1]; ii++) /* column FFTs */
      gsl_fft_complex_radix2_inverse(
          (gsl_complex_packed_array)&cser[ii], memdim[1], memdim[0]);

  /* Extract part of correlation array which is valid
     ------------------------------------------------*/
  ncol = (int)(nsnew[0] - nrnew[0] + 1.5);
  nrow = (int)(nsnew[1] - nrnew[1] + 1.5);
  ndxout = 0;
  for (ii=0; ii<nrow; ii++)
      for (jj=0; jj<ncol; jj++,ndxout++)
          unormc[ndxout] = GSL_REAL(cser[jj*memdim[1]+ii]);
}


/****************************************************************************
NAME:                           GNORM

PURPOSE:  Convert raw cross-product sums to normalized cross-correlation 
          coefficients, while tabulating statistics needed for subsequent 
          evaluation

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand 

REFERENCES:

1.  LAS 4.0 GREYCORR by R. White 8/83 and Y. Jun 7/84
*****************************************************************************/
static void gnorm
(
    float *imager,   /* I: Reference subimage                               */
    float *images,   /* I: Search subimage                                  */
    float *nplr,     /* I: Actual size of reference subimage--samps/lines   */
    float *npls,     /* I: Actual size of search subimage--samps/lines      */
    float *unormc,   /* I: Array of raw cross product sums                  */
    float *ccnorm,   /* O: Array of norm xcorr coeffs                       */
    float *pkval,    /* O: Table of top 32 normalized values                */
    int *ipkcol,     /* O: Table of column numbers for top 32 values        */
    int *ipkrow,     /* O: Table of row numbers for top 32 values           */
    float *sums      /* O; Sum of all normalized values, and sum of squares */
)
{
  int iptr[32];        /* Index to values in location arrays                */
  int ixcol[32];       /* Col locals of 32 largest values                   */
  int nsnew[2];        /* integer version of npls                           */
  int nrnew[2];        /* integer version of nplr                           */
  int ixrow[32];       /* Row locations of 32 largest values                */
  int ii, jj, kk;      /* Loop counters                                     */
  int ipfree;          /* Index to element freed by deleting 32-nd smallest */
  int ipt;             /* Index to image array                              */
  int jstart;          /* Pointer to first element in current row           */
  int jstop;           /* Pointer to last element in current row            */
  int kol;             /* Loop counter                                      */
  int koladd;          /* #search chip col added                            */
  int kolsub;          /* #search chip col subtracted                       */
  int line;            /* Loop counter                                      */
  int lnadd;           /* #search chip line added                           */
  int lnsub;           /* #search chip line subtracted                      */
  int ncol;            /* #cols in cross-product sum array                  */
  int nrow;            /* #rows in cross-product sum array                  */
  int nrtot;           /* Number of pixels in reference image chip          */
  
  float colsqr[256];   /* Sum of squares of search (current overlain area)  */
  float colsum[256];   /* Col sum of search chip (currently overlain area)  */
  float xval[32];      /* Temp storage array                                */
  float refsum;        /* Sum of all reference-subimage pixel values        */
  float refsqr;        /* Sum of squares of reference-subimage pixels       */
  float rho;           /* Temp storage for xcorr coeffs                     */
  float rmean;         /* Mean of reference-subimage pixel values           */
  float rsigma;        /* Std Dev about mean of ref chip pixel values       */
  float rtotal;        /* Number of pixels in reference subimage            */
  float sigma1;        /* Val after 1st iter of Newton's method to app sqrt */
  float sigmas;        /* STD DEV mult by rtotal--inital est for next row   */
  float srchsm;        /* Sum of pixel vals for search subarea              */
  float srchsq;        /* Sum of squares of search subarea pixels           */
  float temp;          /* Temp var for search subarea STD DEV calc          */
  float tempmn;        /* Min val of temp--avoids divide by small values    */
  

  /* Compute pixel-value statistics for reference subimage
     -----------------------------------------------------*/
  nsnew[0] = (int)(npls[0] + 0.5);
  nsnew[1] = (int)(npls[1] + 0.5);
  nrnew[0] = (int)(nplr[0] + 0.5);
  nrnew[1] = (int)(nplr[1] + 0.5);
  refsum = refsqr = 0;
  nrtot = nrnew[0]*nrnew[1];
  for(ipt=0; ipt<nrtot; ipt++)
  {
      refsum += imager[ipt];
      refsqr += imager[ipt]*imager[ipt];
  }
  rtotal = (float)nrtot;
  tempmn = 0.01*rtotal*rtotal;
  rmean = refsum/rtotal;
  rsigma = refsqr/rtotal - rmean*rmean;
  rsigma = rsigma > 0.01 ? rsigma : 0.01;
  rsigma = sqrt(rsigma);

  /* Clear sums and sums of squares of normalized correlation values
     ---------------------------------------------------------------*/
  sums[0] = sums[1] = 0.0;
  for (kk=0; kk<32; kk++)
  {
      xval[kk] = -1.0;
      iptr[kk] = kk;
  }
  ncol = nsnew[0] - nrnew[0] + 1;
  nrow = nsnew[1] - nrnew[1] + 1;

  /* Compute normalized cross-corr values for one row of alignments
     at a time.
     --------------------------------------------------------------*/
  
  /* Get column sums and sums of squares for portion of search subimage
     overlaid by reference in current row of alignments.
   --------------------------------------------------------------------*/
  for (jstart=0,jstop=ncol,ii=0; ii<nrow; ii++,jstart+=ncol,jstop+=ncol)
  {
      if (ii == 0) 
      {
          for (kol=0; kol<nsnew[0]; kol++)
              colsum[kol] = colsqr[kol] = 0.0;
          for (ipt=0,line=0; line<nrnew[1]; line++)
              for (kol=0; kol<nsnew[0]; kol++, ipt++)
              {
                  colsum[kol] += images[ipt];
                  colsqr[kol] += images[ipt]*images[ipt];
              }
      }
      else
      {
          lnsub = (ii - 1)*nsnew[0];
          lnadd = lnsub + nsnew[0]*nrnew[1];
          for (kol=0; kol<nsnew[0]; kol++)
          {
              colsum[kol] = colsum[kol] + images[lnadd+kol] -
                            images[lnsub+kol];
              colsqr[kol] = colsqr[kol] +
                            images[lnadd+kol]*images[lnadd+kol] -
                            images[lnsub+kol]*images[lnsub+kol];
          }
      }

      /* Complete computation of search-subarea pixel statistics
         -------------------------------------------------------*/
      for (jj=jstart; jj<jstop; jj++)
      {
          if (jj == jstart) 
          {
              srchsm = srchsq = 0.0;
              for (kol=0; kol<nrnew[0]; kol++)
              {
                  srchsm += colsum[kol];
                  srchsq += colsqr[kol];
              }
              temp = rtotal*srchsq - srchsm*srchsm;
              temp = tempmn > temp ? tempmn : temp;
              sigmas = sqrt(temp);
          }
          else
          {
              kolsub = (jj - jstart) - 1;
              koladd = kolsub + nrnew[0];
              srchsm = srchsm + colsum[koladd] - colsum[kolsub];
              srchsq = srchsq + colsqr[koladd] - colsqr[kolsub];
              temp = rtotal*srchsq - srchsm*srchsm;
              temp = tempmn > temp ? tempmn : temp;
              sigma1 = 0.5*(sigmas + temp/sigmas);
              sigmas = 0.5*(sigma1 + temp/sigma1);
          }

          /* Compute normalized cross-correlation value
             ------------------------------------------*/
          rho = (unormc[jj] - rmean*srchsm)/(rsigma*sigmas);
          ccnorm[jj] = rho;
          sums[0] += rho;
          sums[1] += rho*rho;

          /* Check whether value among top 32
             --------------------------------*/
          if (rho > xval[iptr[31]]) 
          {
              kk = 31;
              ipfree = iptr[31];
              while ((kk > 0) && (rho > xval[iptr[kk-1]]))
              {
                  iptr[kk] = iptr[kk-1];
                  kk--;
              }
              iptr[kk] = ipfree;
              xval[ipfree] = rho;
              ixcol[ipfree] = jj - jstart;
              ixrow[ipfree] = ii;
          }
      }
  }

  /* Copy peak values and coordinates in correct sequence
     ----------------------------------------------------*/
  for (kk=0; kk<32; kk++)
  {
      pkval[kk] = xval[iptr[kk]];
      ipkcol[kk] = ixcol[iptr[kk]];
      ipkrow[kk] = ixrow[iptr[kk]];
  }
}


/****************************************************************************
NAME:                           GCORR

PURPOSE:  Correlate a reference subimage with a search subimage and evaluate 
          the results

VERSION  DATE   AUTHOR    
-------  ----   ------     
  1.0    8/92   D. Steinwand

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion

REFERENCES:

1.  LAS 4.0 GREYCORR by R. White 6/83
*****************************************************************************/
static int gcorr
(
    float *images,     /* I: Search subimage                                */
    float *imager,     /* I: Reference subimage                             */
    float *npls,       /* I: Actual size of search subimage:  samps,lines   */
    float *nplr,       /* I: Actual size of reference subimage: samps,lines */
    const float csmin, /* I: Minimum acceptable correlation strength        */
    const int mfit,    /* I: Surface Fit Method:  1 - Elliptical paraboloid
                                                  2 - Elliptical Gaussian 
                                                  3 - Reciprocal Paraboloid 
                                                  4 - Round to nearest int  */
    const float ddmx,  /* I: Maximum allowed diagonal displacement from
                             nominal tiepoint loc to location found by
                             correlation                                    */
    float *ioffrq,     /* I: Requested maximum horiz & vert search offsets  */
    float *nomoff,     /* I: Nominal horiz & vert offsets of UL corner of
                             reference subimage relative to search subimage */
    int *iacrej,       /* O: Accept/Reject code                             */
    float *streng,     /* O: Strength of correlation                        */
    float *bfoffs,     /* O: Best-fit horiz & vert offsets of corr. peak    */
    float *tlerrs,     /* O: Est horiz error, vert error, and h-v cross term
                             in best-fit offsets                            */
    float *ddact       /* O: Actual diagonal displacement from nominal
                             tiepoint location to location found by corr.   */
)
{
  int ipkcol[32];      /* Col number for each of top 32 correlation values  */
  int ipkrow[32];      /* Row number for each of top 32 correlation values  */
  int ncol;            /* Number of columns of search alignments            */
  int nrow;            /* Number of rows of search alignments               */
  
  float ccnorm[16641]; /* Normalized xcorr coefs for each search alignment  */
  float cpval[25];     /* 5 by 5 array of xcorr values, in std dev          */
  float ddmx_actual;   /* Since ddmx may be negative (implying n pixels less
                          than the chip size), use this variable to denote
                          the actual acceptable diagonal distance.          */
  float pkoffs[2];     /* Fractional horiz & vert offsets of best-fit peak  */
  float pkval[32];     /* Largest 32 values of normalized xcorr coefs       */
  float sums[2];       /* Sum and sum of squares of all xcorr values        */
  float unormc[16641]; /* Unnormalized cross-product sum for each alignment */


  /* Compute raw cross-product sums
     ------------------------------*/
  cross(images, imager, npls, nplr, unormc);

  /* Compute normalized cross-correlation values and compile statistics
     ------------------------------------------------------------------*/
  gnorm(imager, images, nplr, npls, unormc, ccnorm, pkval, ipkcol, ipkrow,
        sums);

  /* Evaluate strength of correlation peak
     -------------------------------------*/
  ncol = (int)(npls[0] + 0.5) - (int)(nplr[0] + 0.5) + 1;
  nrow = (int)(npls[1] + 0.5) - (int)(nplr[1] + 0.5) + 1;
  eval(ncol, nrow, ccnorm, pkval, ipkcol, ipkrow, csmin, sums, streng,
       iacrej, cpval);

  /* Determine offsets of peak relative to nominal location
     ------------------------------------------------------*/
  if (*iacrej == 1)
  {
      if (mfit != 4)
      {
          if (fitreg(cpval, mfit, pkoffs, tlerrs) != E_SUCC)
          {
              c_lasmsg(FALSE, "correlate-edge",
                       "Error encountered while computing subpixel "
                       "correlation peak location.");
              
              return E_FAIL;
          }
          bfoffs[0] = (float)ipkcol[0] - nomoff[0] + pkoffs[0];
          bfoffs[1] = (float)ipkrow[0] - nomoff[1] + pkoffs[1];
      }
      else
      {
          bfoffs[0] = (float)ipkcol[0] - nomoff[0];
          bfoffs[1] = (float)ipkrow[0] - nomoff[1];
          tlerrs[0] = 0.5;
          tlerrs[1] = 0.5;
      }

      /* Determine diagonal displacement from nominal and check against
         maximum acceptable value
         --------------------------------------------------------------*/
      *ddact = sqrt(bfoffs[0]*bfoffs[0] + bfoffs[1]*bfoffs[1]);
      if (ddmx <= 0.0)
          ddmx_actual = ioffrq[0] < ioffrq[1] ?
                        ioffrq[0] - 2.0 : ioffrq[1] - 2.0;
      else
          ddmx_actual = ddmx;
      if (*ddact > ddmx_actual)
          *iacrej = 5;
  }

  return E_SUCC;
}


/****************************************************************************
NAME:                           grey_corr

PURPOSE:  Performs grey (cross) correlation for a tie point

*****************************************************************************/
void grey_corr
(
    struct CORRPAR *corr,      /* I: Correlation parameters                 */
    float *ref,                /* I: Reference subimage                     */
    float *srch,               /* I: Search subimage                        */
    double *nominal,           /* I: Nominal coordinates                    */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct TPLDATA *tpl        /* I/O: Tie point location data record       */
)
{
  int active;                  /* int version of (long)tpl->active          */
  
  float mincorr;               /* Minimum acceptable correlation strength   */
  float maxdis;                /* Maximum displacement from nominal allowed */
  float disp = 0.;             /* Actual displacement from nominal          */
  float est_err[3];            /* Estimated correlation errors              */
  float best_fit[2];           /* Best fit offsets from correlation peak    */
  float npls[2];               /* Size of search subimage                   */
  float nplr[2];               /* Size of reference subimage                */
  float ioffrq[2];             /* Requested max horiz & vert search offsets */
  float nomoff[2];             /* Nominal offsets                           */
  float strength = 0.;         /* Strength of correlation                   */
  

  /* Assign values of C structure to individual variables 
     ----------------------------------------------------*/
  npls[0] = (float)tpl->sea_size[0];
  npls[1] = (float)tpl->sea_size[1];
  nplr[0] = (float)tpl->ref_size[0];
  nplr[1] = (float)tpl->ref_size[1];
  mincorr = (float)corr->mincorr;
  maxdis = (float)corr->maxdiff;
  ioffrq[0] = (float)tpl->offsets[0];
  ioffrq[1] = (float)tpl->offsets[1];
  nomoff[0] = (float)nominal[1];
  nomoff[1] = (float)nominal[0];

  /* Perform grey level (cross) correlation
     --------------------------------------*/
  if (gcorr(srch, ref, npls, nplr, mincorr, corr->fitmeth, maxdis,
            ioffrq, nomoff, &active, &strength, best_fit, est_err,
            &disp) != E_SUCC)
      tpl->active = TPL_STATE_BAD_PARAM;
  else
      tpl->active = active;

  tpl->displacement = (double)disp;
  tpl->strength = (double)strength;

  /* If the correlation was successful, compute the new search tie point.
     Otherwise, set the search tie point to the nominal value.
     --------------------------------------------------------------------*/
  if (tpl->active == TPL_STATE_OK_AUTO)
  {
      tpl->sea_coord[0] = tpl->nominal[0] +
                          best_fit[1]*trans[0] + best_fit[0]*trans[1];
      tpl->sea_coord[1] = tpl->nominal[1] +
                          best_fit[1]*trans[3] + best_fit[0]*trans[4];
      tpl->rms_error[0] = est_err[0];
      tpl->rms_error[1] = est_err[1];
  }
  else
  {
      tpl->sea_coord[0] = tpl->nominal[0];
      tpl->sea_coord[1] = tpl->nominal[1];
      tpl->rms_error[0] = 0.0;
      tpl->rms_error[1] = 0.0;
  }
}
