/*****************************************************************************
NAME:                           PHASE_CORR

PURPOSE:  Subroutine to match a 32 x 32 reference sub-image to a (up to)
          128 x 128 search sub-image.  Phase correlation is used.

VERSION         DATE    AUTHOR          
-------         ----    ------  
  1.0           8/92    D. Steinwand

CHANGES:

$Log: phase.c,v $
Revision 6.3  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 6.2.6.8  2001/06/15 20:34:39  ldjohn
CR4098 Updated to use GSL 0.8.

Revision 6.2.6.7  2001/06/13 16:14:10  ldjohn
CR854 Code review:  Longs to ints.  Added #defines for active flag values.

Revision 6.2.6.6  2001/05/09 19:14:19  ldjohn
CR854 Fixed indexing error that overstepped array bounds.

Revision 6.2.6.5  2001/05/09 15:15:42  ldjohn
CR854 Changes to incorporate use of the GSL (Gnu Scientific Library).

Revision 6.2.6.4  2001/04/23 18:44:03  ldjohn
CR854 Rearranged functions to remove internal function prototypes.

Revision 6.2.6.3  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 6.2.6.2  2001/03/07 21:09:40  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "geompak.h"
#include "correlate.h"
#include "gsl/gsl_complex_math.h"
#include "gsl/gsl_fft_complex.h"
#include "gsl/gsl_linalg.h"


/*****************************************************************************
NAME:                           PHASE

PURPOSE:  Subroutine to match a 32 x 32 reference sub-image to a (up to)
          128 x 128 search sub-image.  Phase correlation is used.

VERSION         DATE    AUTHOR
-------         ----    ------
  1.0           8/92    D. Steinwand

RETURN VALUES:  E_SUCC   successful completion
                E_FAIL   unsuccessful completion
*****************************************************************************/
static int phase
(
    float *ref,             /* I: 32 x 32 reference chip, stored by rows    */
    float *search,          /* I: Search subimage, stored by rows           */
    const int size,         /* I: Size of search subimage                   */
    const int inc,          /* I: Stepping increment                        */
    const float alpha,      /* I: Exponential factor                        */
    const int hpf,          /* I: High-pass filter flag                     */
    float *peak,            /* O: Correlation peak                          */
    float *line,            /* O: Line in srch subimage corr. to ref center */
    float *samp             /* O: Samp in srch subimage corr. to ref center */
)
{
  gsl_complex cref[1024];      /* Complex buffer for reference subimage     */
  gsl_complex cser[1024];      /* Complex buffer for search subimage        */
  gsl_complex sr[1024];        /* Complex buffer for correlation results    */
  
  int ii, jj, kk, ll, mm;      /* Loop counters                             */
  int kl, kl16;                /* Misc indeces                              */
  int iloc, jloc;              /* Correlation peak locations                */
  int kmax, lmax;              /* Location of max peak                      */
  int sl;                      /* Pointer to start of search sub-chip       */
  int stop;                    /* Upper increment                           */
  
  float maxpeak;               /* Maximum correlation peak                  */
  float rv, srmax;             /* Max peak location variables               */
  float temp;                  /* Temporary variable in re-arrangement      */
  
  double aa[54], corr[9],      /* Data blocks for mat_aa, vec_corr, vec_tau,*/
         tau[6], residual[9],  /* vec_residual, vec_xx                      */
         xx[6];
  double hh;                   /* temporary storage: mag^pow                */
  double hmag;                 /* mag/hh                                    */
  double magnitude;            /* Phase of point--temp variable             */
  double x0,y0,b2a;            /* Sub-pixel offsets & work variable         */
  
  gsl_matrix mat_aa;           /* Matrix for sub-pixel calculation          */
  gsl_vector vec_corr;         /* Vector for correlation peak & 8 neighbors */
  gsl_vector vec_residual;     /* Residual values of QR factorization       */
  gsl_vector vec_tau;          /* Householder coeff. from QR factorization  */
  gsl_vector vec_xx;           /* Least square solution                     */


  /* Define matrices and vectors. */
  mat_aa.size1 = 9;
  mat_aa.size2 = 6;
  mat_aa.tda = 6;
  mat_aa.data = aa;
  mat_aa.block = NULL;
  mat_aa.owner = 0;
                             
  vec_corr.size = 9;
  vec_corr.stride = 1;
  vec_corr.data = corr;
  vec_corr.block = NULL;
  vec_corr.owner = 0;
                             
  vec_residual.size = 9;
  vec_residual.stride = 1;
  vec_residual.data = residual;
  vec_residual.block = NULL;
  vec_residual.owner = 0;
  
  vec_tau.size = 6;
  vec_tau.stride = 1;
  vec_tau.data = tau;
  vec_tau.block = NULL;
  vec_tau.owner = 0;
  
  vec_xx.size = 6;
  vec_xx.stride = 1;
  vec_xx.data = xx;
  vec_xx.block = NULL;
  vec_xx.owner = 0;
  
/* Take FFT of the reference window
   --------------------------------*/
  for (ii=0; ii<1024; ii++)
      GSL_SET_COMPLEX(&cref[ii], ref[ii], 0.);
  for (ii=0; ii<32; ii++) /* row FFTs */
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cref[32*ii], 1, 32);
  for (ii=0; ii<32; ii++) /* column FFTs */
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cref[ii], 32, 32);

  /* Removed DC component. */
  GSL_SET_COMPLEX(&cref[0], 0., 0.);
  
  /* If the user has requested a high-pass filter be applied,
     do so by zeroing the first row and column of the transform array.
     -----------------------------------------------------------------*/
  if (hpf)
  {
      /* Make the top row all 0.0s */
      for (ii=0; ii<32; ii++)
          GSL_SET_COMPLEX(&cref[ii], 0., 0.);

      /* Make the first column all 0.0s */
      for (ii=0; ii<32; ii++)
          GSL_SET_COMPLEX(&cref[32*ii], 0., 0.);
  }

  /* Step through search window extracting 32 x 32 sub-chips and
     taking the FFT.
     -----------------------------------------------------------*/
  stop = size - 32 + 1;
  maxpeak = -1e20;
  for (ii=0; ii<stop; ii+=inc)
  {
      for (jj=0; jj<stop; jj+=inc)
      {
          sl = ii*size + jj;
          /* Take FFT of the search subwindow
             --------------------------------*/
          for(mm=0, kk=0; kk<32; kk++, sl+=size)
              for(ll=0; ll<32; ll++,mm++)
                  GSL_SET_COMPLEX(&cser[mm], search[sl+ll], 0.);
          for (kk=0; kk<32; kk++) /* row FFTs */
              gsl_fft_complex_radix2_forward(
                  (gsl_complex_packed_array)&cser[32*kk], 1, 32);
          for (kk=0; kk<32; kk++) /* column FFTs */
              gsl_fft_complex_radix2_forward(
                  (gsl_complex_packed_array)&cser[kk], 32, 32);

          /* Take complex conjugate of two chips
          --------------------------------------*/
          for (kk=0; kk<1024; kk++)
          {
              sr[kk] = gsl_complex_mul(cser[kk],
                                       gsl_complex_conjugate(cref[kk]));

              /* Calculate "phase" correlation
                 -----------------------------*/
              magnitude = gsl_complex_abs(sr[kk]);
              if (magnitude < 1.0e-6)
                  magnitude = 1.0e-6;
              hh = pow(magnitude,alpha);
              hmag = hh/magnitude;
              sr[kk] = gsl_complex_mul_real(sr[kk], hmag);
          }

          /* Take inverse transform of correlation windows
             ---------------------------------------------*/
          for (kk=0; kk<32; kk++) /* row FFTs */
              gsl_fft_complex_radix2_inverse(
                  (gsl_complex_packed_array)&sr[32*kk], 1, 32);
          for (kk=0; kk<32; kk++) /* column FFTs */
              gsl_fft_complex_radix2_inverse(
                  (gsl_complex_packed_array)&sr[kk], 32, 32);

          /* Rearrange chips to center
             -------------------------*/
          for (kk=0; kk<16; kk++)
              for (ll=0; ll<16; ll++)
              {
                  kl = kk*32 + ll;
                  kl16 = 32*(kk + 16) + ll + 16;
                  temp = GSL_REAL(sr[kl]);
                  GSL_SET_REAL(&sr[kl], GSL_REAL(sr[kl16]));
                  GSL_SET_REAL(&sr[kl16], temp);
                  kl =32*(kk + 16) + ll;
                  kl16 = 32*kk + ll + 16;
                  temp = GSL_REAL(sr[kl]);
                  GSL_SET_REAL(&sr[kl], GSL_REAL(sr[kl16]));
                  GSL_SET_REAL(&sr[kl16], temp);
              }

          /* Find peaks
             ----------*/
          srmax = -1e20;
          for (kk=7; kk<25; kk++)
              for (ll=7; ll<25; ll++)
              {
                  rv = GSL_REAL(sr[ll*32+kk]);
                  if (rv > srmax)
                  {
                      srmax=rv;
                      kmax=kk;
                      lmax=ll;
                  }
              }

          /* Check correlation strength against other windows.
             Store the "strongest."
             -------------------------------------------------*/
          if (srmax > maxpeak)
          {
              maxpeak = srmax;
              iloc = lmax - 16 + ii;
              jloc = kmax - 16 + jj;
          }
      } /* jj (sample) loop */
  } /* ii (line) loop */

  /* Refine at max
     -------------*/
  if (iloc < 0)
      iloc = 0;
  if (jloc < 0)
      jloc = 0;
  if (iloc > 95)
      iloc = 95;
  if (jloc > 95)
      jloc = 95;

  sl = iloc*size + jloc;
  for (mm=0,kk=0; kk<32; kk++,sl+=size)
      for(ll=0; ll<32; ll++,mm++)
          GSL_SET_COMPLEX(&cser[mm], search[sl+ll], 0.);
  for (ii=0; ii<32; ii++) /* row FFTs */
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cser[32*ii], 1, 32);
  for (ii=0; ii<32; ii++) /* column FFTs */
      gsl_fft_complex_radix2_forward(
          (gsl_complex_packed_array)&cser[ii], 32, 32);

  /* Take complex conjugate of two chips
     -----------------------------------*/
  for (kk=0; kk<1024; kk++)
  {
      sr[kk] = gsl_complex_mul(cser[kk], gsl_complex_conjugate(cref[kk]));

      /* Calculate "phase" correlation
         -----------------------------*/
      magnitude = gsl_complex_abs(sr[kk]);
      if (magnitude < 1.0e-6)
          magnitude = 1.0e-6;
      hh = pow(magnitude, alpha);
      hmag = hh/magnitude;
      sr[kk] = gsl_complex_mul_real(sr[kk], hmag);
  }
  
  /* Take inverse transform of correlation windows
     ---------------------------------------------*/
  for (ii=0; ii<32; ii++) /* row FFTs */
      gsl_fft_complex_radix2_inverse(
          (gsl_complex_packed_array)&sr[32*ii], 1, 32);
  for (ii=0; ii<32; ii++) /* column FFTs */
      gsl_fft_complex_radix2_inverse(
          (gsl_complex_packed_array)&sr[ii], 32, 32);

  /* Rearrange chips to center
     -------------------------*/
  for (kk=0; kk<16; kk++)
      for (ll=0; ll<16; ll++)
      {
          kl = 32*kk + ll;
          kl16 = 32*(kk + 16) + ll + 16;
          temp = GSL_REAL(sr[kl]);
          GSL_SET_REAL(&sr[kl], GSL_REAL(sr[kl16]));
          GSL_SET_REAL(&sr[kl16], temp);
          kl =32*(kk + 16) + ll;
          kl16 = 32*kk + ll + 16;
          temp = GSL_REAL(sr[kl]);
          GSL_SET_REAL(&sr[kl], GSL_REAL(sr[kl16]));
          GSL_SET_REAL(&sr[kl16], temp);
      }

  /* Find peak
     ---------*/
  *peak = -1e20;
  for (kk=7; kk<25; kk++)
      for (ll=7; ll<25; ll++)
      {
          rv = GSL_REAL(sr[32*ll+kk]);
          if (rv > *peak)
          {
              *peak = rv;
              kmax = kk;
              lmax = ll;
          }
      }

  /* Store peak & 8 neighbors
     ------------------------*/
  for (mm=0,kk=0; kk<3; kk++)
      for (ll=0; ll<3; ll++,mm++)
          corr[mm] = GSL_REAL(sr[32*(lmax+ll-1)+(kmax+kk-1)]);

  /* Fit quadratic to peak and its neighbors to determine sub-pixel location
     -----------------------------------------------------------------------*/
  for (kk=0,ii=0; ii<3; ii++)
      for (jj=0; jj<3; jj++)
      {
          aa[kk++] = (double)((jj + 1)*(jj + 1));
          aa[kk++] = (double)((ii + 1)*(jj + 1));
          aa[kk++] = (double)((ii + 1)*(ii + 1));
          aa[kk++] = (double)(jj + 1);
          aa[kk++] = (double)(ii + 1);
          aa[kk++] = 1.;
      }

  /* Perform least squares fit. */
  if (gsl_linalg_QR_decomp(&mat_aa, &vec_tau) != GSL_SUCCESS)
  {
      c_lasmsg(FALSE, "correlate-lsq",
               "Error performing least squares fit.");
      return E_FAIL;
  }
  if (gsl_linalg_QR_lssolve(&mat_aa, &vec_tau, &vec_corr, &vec_xx,
                            &vec_residual) != GSL_SUCCESS)
  {
      c_lasmsg(FALSE, "correlate-lsq",
               "Error performing least squares fit.");
      return E_FAIL;
  }
  for (ii=0; ii<6; ii++)
      corr[ii] = xx[ii];

  if (fabs(corr[0]) < 1.0e-7)
      y0 = x0 = 0.0;
  else
  {
      b2a = corr[1]/(corr[0]*2.0);
      y0 = (b2a*corr[3] - corr[4])/(2.0*corr[2] - b2a*corr[1]) - 2.0;
      x0 = -b2a*(y0 + 2.0) - corr[3]/(corr[0]*2.0) - 2.0;
      if (y0*y0 + x0*x0 > 4.0)
          y0 = x0 = 0.0;
  }

  /* Calculate search image coordinate corresponding to reference chip
     center
     -----------------------------------------------------------------*/
  *line = iloc + lmax + y0;
  *samp = jloc + kmax + x0;

  /* "Unnormalize" peak value to agree with legacy phase correlation
     results. */
  *peak *= 32*32;
  
  return E_SUCC;
}


/*****************************************************************************
NAME:                           PHASE_CORR

PURPOSE:  Subroutine to match a 32 x 32 reference sub-image to a (up to)
          128 x 128 search sub-image.  Phase correlation is used.

VERSION         DATE    AUTHOR          
-------         ----    ------  
  1.0           8/92    D. Steinwand

*****************************************************************************/
void phase_corr
(
    const struct CORRPAR *corr, /* I: Correlation parameters             */
    float *ref,                 /* I: Reference subimage                 */
    float *srch,                /* I: Search subimage                    */
    const int inc,              /* I: Step size                          */
    const float alpha,          /* I: Exponential factor                 */
    const int hpf,              /* I: High-pass filter flag              */
    float *trans,     /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    struct TPLDATA *tpl         /* I/O: Tie point location data record   */
)
{
  float line,samp;
  float peak;
  float diffx,diffy;
  float maxdis;           /* Maximum displacement from nominal allowed */
 

  /* Compute all correlation coefficients. If the correlation process
     fails, set the active code to TPL_STATE_BAD_EDGE (Reject--correlation
     peak too near edge of search image).*/
  if (phase(ref, srch, tpl->sea_size[0], inc, alpha, hpf, &peak, &line,
            &samp) != E_SUCC)
      tpl->active = TPL_STATE_BAD_EDGE;

  /* Reorient line/sample pair to chip center. */
  line = line - tpl->sea_size[0]/2;
  samp = samp - tpl->sea_size[1]/2;

  /* Set additional tie point structure elements. */
  tpl->strength = peak;
  tpl->rms_error[0] = 0.0;
  tpl->rms_error[1] = 0.0;

  /* If the correlation was successful, compute and store the correlation
     point and additional information. */
  if (tpl->active == TPL_STATE_OK_AUTO)
  {
      tpl->sea_coord[0] = tpl->nominal[0] + line*trans[0] + samp*trans[1];
      tpl->sea_coord[1] = tpl->nominal[1] + line*trans[3] + samp*trans[4];
      diffy = tpl->sea_coord[0] - tpl->nominal[0];
      diffx = tpl->sea_coord[1] - tpl->nominal[1];
      tpl->displacement = sqrt(diffx*diffx + diffy*diffy);
      maxdis = (float)corr->maxdiff;
      if (maxdis <= 0.0)
          maxdis = tpl->sea_size[0] < tpl->sea_size[1] ?
              tpl->sea_size[0] - 2.0 : tpl->sea_size[1] - 2.0; 
      if (tpl->displacement > maxdis)
          tpl->active = TPL_STATE_BAD_DISP;
                            /* Reject--diagonal displacement from nominal
                               location exceeds the maximum specified by the
                               user */
      if (corr->mincorr > tpl->strength)
          tpl->active = TPL_STATE_BAD_PEAK;
                            /* Reject--strength of peak below minimum
                               specified by user */
  }
  else
  {
      tpl->displacement = 0.0;
      tpl->sea_coord[0] = tpl->nominal[0];
      tpl->sea_coord[1] = tpl->nominal[1];
  }
}
