/****************************************************************************
NAME:                           process_results

PURPOSE:  Report the correlation results to the user and write to file if
          so directed.

CHANGES:

$Log: process_results.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.2  2001/06/13 16:15:24  ldjohn
CR854 Code review:  Longs to ints.  Passed structures as pointers.  Added
      #defines for active flag values.

Revision 1.1.2.1  2001/03/13 19:49:13  ldjohn
CR854 Extracted (and extended) from existing code to simplify layout.

*****************************************************************************/
#include "las.h"
#include "geompak.h"
#include "correlate.h"


int process_results
(
    const struct TPLDATA *tpl_data, /* I: Tie point location data record    */
    const int confirm,        /* I: Boolean--confirm flag                   */
    struct PTDESC *out_tpl,   /* I: Output tiepoint location file desc      */
    int *pts_correlated,      /* I/O: Number of points that correlate       */
    int *numtie,              /* I/O: number of tie points used for least-
                                      squares fit of transform coefficients */
    float irtie[],            /* O: input reference tie points              */
    float istie[],            /* O: input search tie points                 */
    int *update_transform     /* O: Boolean flag representing whether or not
                                    the affine transformation should be
                                    recomputed                              */
)
{
  switch(tpl_data->active)
  {
      case TPL_STATE_OK_AUTO:
          c_lasmsg(FALSE, " ", "%s [1] Successfully correlated",
                   tpl_data->pt_id);
          break;
      case TPL_STATE_BAD_EDGE:
          c_lasmsg(FALSE, " ",
                   "%s [2] Correlation peak too near edge of search image",
                   tpl_data->pt_id);
          break;
      case TPL_STATE_BAD_SUBPEAK:
          c_lasmsg(FALSE, " ",
                 "%s [3] Subsidiary peak comparable in strength to main peak",
                   tpl_data->pt_id);
          break;
      case TPL_STATE_BAD_PEAK:
          c_lasmsg(FALSE, " ",
                   "%s [4] Strength of peak below minimum specified by user",
                   tpl_data->pt_id);
          break;
      case TPL_STATE_BAD_DISP:
          c_lasmsg(FALSE, " ",
                   "%s [5] Diagonal displacement exceeds maximum specified",
                   tpl_data->pt_id);
          break;
      case TPL_STATE_BAD_USER:
          c_lasmsg(FALSE, " ", "%s [7] Size of image and chip do not match",
                   tpl_data->pt_id);
          break;
      case TPL_STATE_BAD_MODEL:
          c_lasmsg(FALSE, " ", "%s [8] Tie point outside image",
                   tpl_data->pt_id);
          break;
      case TPL_STATE_BAD_PARAM:
      case TPL_STATE_BAD_USER_MODEL:
          c_lasmsg(FALSE, "correlate-corr",
                   "Error encountered during correlation.");
          return E_FAIL;
      default:
          c_lasmsg(FALSE, "correlate-corr",
                   "Unsupported correlation state flag of %d.",
                   tpl_data->active);
          return E_FAIL;          
              
  } /* tpl_data->active switch */

  /* If the user requested confirmation of each tie point, do so now.
     Otherwise, just accept the point.
     ----------------------------------------------------------------*/
  if (accept(confirm, tpl_data)) 
  {
      /* Write tie point to output file. */
      if (c_ptio(&out_tpl, tpl_data) != E_SUCC)
      {
          c_lasmsg(FALSE, "correlate-ptio", "Error writing tie point");
          return E_FAIL;
      }

      /* If the point was successfully correlated increment
         the counter and add the reference and search points
         to the least-squares fit data. */
      if (tpl_data->active == TPL_STATE_OK_AUTO)
      {
          (*pts_correlated)++;
          if (*numtie < MAX_LSQ_TIE)
          {
              irtie[2*(*numtie)] = tpl_data->ref_coord[0];
              irtie[2*(*numtie)+1] = tpl_data->ref_coord[1];
              istie[2*(*numtie)] = tpl_data->sea_coord[0];
              istie[2*(*numtie)+1] = tpl_data->sea_coord[1];
              (*numtie)++;
              *update_transform = TRUE;
          }
      }
  } /* if correlation point accepted */

  return E_SUCC;
}
