/****************************************************************************
NAME:       READ_CHIP

PURPOSE:    Read image chip file.

REFERENCES:

RETURN VALUES:  E_SUCC         successful completion
                READ_ERROR     error opening/reading the chip file
                BAD_SIZE       chip size not as expected
                BAD_CHIP_DDR   unable to retrieve chip DDR
                BAD_COEFS      projection coefficients are invalid

CHANGES:

$Log: read_chip.c,v $
Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.4  2001/06/13 16:16:26  ldjohn
CR854 Code review:  Longs to ints.

Revision 1.1.2.3  2001/05/14 14:39:41  ldjohn
CR854 Closed chip file before returning on error.

Revision 1.1.2.2  2001/03/15 19:03:39  ldjohn
CR854 Added return values.

Revision 1.1.2.1  2001/03/07 21:09:41  ldjohn
CR854 Intermediate changes, including several additions from PICMATCH.

*****************************************************************************/
#include "las.h"
#include "correlate.h"


int read_chip
(
    char *name,               /* I: Name of chip image file                 */
    const int *size,          /* I: Size of chip--nl,ns                     */
    struct CORRIMG *image,    /* I/O: Image file parameters                 */
    float *buffer             /* O: Chip buffer                             */
)
{
  struct FDESC *chip_fdsc;   /* Chip file descriptor */
  struct DDR ddr;            /* DDR record */

  const long acc = IREAD;    /* File access mode */
  const long four = 4;       /* Constant of four */
  long image_coor[8];        /* Image coordinates are all four corners */
  long nbands;               /* Number of bands */
  long nl, ns;               /* Number of lines and samples */
  const long one = 1;        /* Constant of one */
  const long opt[2] = {0, 0};/* Option flags for image I/O */
  const long prt_flag = FALSE; /* Flag to print error messages in c_rtcoef */
  const long sizel = size[0]; /* Long version of (int)size[0] */

  double corners[8];         /* Corner projection/user-defined coordinates */
  double icoef[6];           /* Inverse of temporary coefficients */

  char host[CMLEN];          /* Host file name */

  
  /* Get the chip file host name, and open chip file.
     ------------------------------------------------*/
  c_cmhost(name, host);
  nbands = 0; nl = ns = 0;
  if (c_eopenr(&chip_fdsc,host,&nbands,&one,&one,&nl,&ns,&acc,&four,opt,
               &sizel) != E_SUCC)
      return READ_ERROR;
  if ((nl != size[0]) || (ns != size[1]))
  {
      c_eclose(&chip_fdsc);
      return BAD_SIZE;
  }
  image->nl = nl;
  image->ns = ns;

  /* Read the chip image.
     --------------------*/
  if (c_eread(&chip_fdsc, &one, &one, (unsigned char *)buffer, &sizel)
      == E_FAIL)
  {
      c_eclose(&chip_fdsc);
      return READ_ERROR;
  }
  image->starting_line = image->starting_sample = 1;

  /* Close the chip image.
     ---------------------*/
  c_eclose(&chip_fdsc);

  /* Get DDR info to track the chip back to a full image space.
     ----------------------------------------------------------*/
  if (c_getddr(host, &ddr) != E_SUCC)
      return BAD_CHIP_DDR;
  else
  {
      image->master_line = ddr.master_line;
      image->master_samp = ddr.master_sample;
   
      if (ddr.valid[6] == INVAL)
          return BAD_COEFS;
      else
      {
          /* Calculate coefficients mapping from image coordinates to
             projection/user-defined space.
             --------------------------------------------------------*/
          image_coor[0] = image_coor[1] = 1;
          image_coor[2] = ddr.nl;
          image_coor[3] = 1;
          image_coor[4] = 1;
          image_coor[5] = ddr.ns;
          image_coor[6] = ddr.nl;
          image_coor[7] = ddr.ns;
          corners[0] = ddr.upleft[0];
          corners[1] = ddr.upleft[1];
          corners[2] = ddr.loleft[0];
          corners[3] = ddr.loleft[1];
          corners[4] = ddr.upright[0];
          corners[5] = ddr.upright[1];
          corners[6] = ddr.loright[0];
          corners[7] = ddr.loright[1];
          image->coef_y[0] = 0.0;
          image->coef_y[1] = 0.0;
          image->coef_y[2] = 1.0;
          image->coef_x[0] = 0.0;
          image->coef_x[1] = 1.0;
          image->coef_x[2] = 0.0;
          if(c_rtcoef(corners, image_coor, icoef, &prt_flag) != E_SUCC)
              return BAD_COEFS;
          image->coef_y[0] = icoef[2];
          image->coef_y[1] = icoef[1];
          image->coef_y[2] = icoef[0];
          image->coef_x[0] = icoef[5];
          image->coef_x[1] = icoef[4];
          image->coef_x[2] = icoef[3];
      }
  }
  return E_SUCC;
}
