/****************************************************************************
NAME:                           resample

PURPOSE:  Apply bilinear resampling to fill an image chip.

RETURN VALUES:  E_SUCC          successful completion
                E_FAIL          unsuccessful completion
                READ_ERROR      error opening/reading the chip file

CHANGES:

$Log: resample.c,v $
Revision 6.2  2010/09/23 15:35:16  tvaughn
Adding correlate changes

Revision 6.1  2001/12/06 14:15:25  ldjohn
CR854 Added GREDGE subcommand to combine GREY and EDGE correlation methods.
      Merged PICMATCH capabilities for handling tiepoint grids and chip
      extraction from arbitrary (non-coregistered) reference and search
      images.
      Modified to use GSL (Gnu Scientific Libary) for mathematical routines.
      Added automated regression tests.

Revision 1.1.2.7  2001/06/19 19:01:37  ldjohn
CR854 Corrected starting line, sample error for certain rotation angle sets.

Revision 1.1.2.6  2001/06/13 16:19:19  ldjohn
CR854 Code review:  Longs to ints.  Moved bottom read new line section to
      allow for removal of another.

Revision 1.1.2.5  2001/05/16 19:33:24  ldjohn
CR854 Corrected error that was not completing the resampling.

Revision 1.1.2.4  2001/05/10 17:33:17  ldjohn
CR854 Removed unused parameter.

Revision 1.1.2.3  2001/05/09 15:34:01  ldjohn
CR854 Removed unused variables.

Revision 1.1.2.2  2001/04/23 21:18:58  ldjohn
CR854 Removed read_image_line.c.  Reading image line directly in resample().

Revision 1.1.2.1  2001/04/23 18:45:02  ldjohn
CR854 Moved bilinear resampling out of extract().


*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "correlate.h"


int resample
(
    struct FDESC **ptr,       /* I: File pointer to image file              */
    const int *size,          /* I: Size of chip--nl,ns                     */
    const float *trans,  /* I: Array of the grid transformation coefficients:
                            ynew = trans[0]*yold + trans[1]*xold + trans[2]
                            xnew = trans[3]*yold + trans[4]*xold + trans[5] */
    const struct CORRIMG *image, /* I: Image file parameters                */
    const int line_sign,      /* I: line direction of 1st line in transform
                                    space: + = down/right, - = up/left      */
    const int samp_sign,      /* I: sample direction of 1st line in transform
                                    space: + = down/right, - = up/left      */
    const float ystart,       /* I: first (top) y value for transformed chip*/
    const float basey,        /* I: transform-space line of upper-left chip
                                    corner                                  */
    const float basex,        /* I: transform-space sample of upper-left chip
                                    corner                                  */
    int *chop,                /* O: Number of grid points outside the image.
                                    Corresponding DN values in the "grid" are
                                    set to zero.                            */
    float *buffer             /* O: Chip buffer                             */
)
{
  int fill_line, fill_samp;  /* line/sample counters in fill space */
  int fill_line_0, fill_samp_0; /* starting vales for line/sample counters */
  int fill_samp_index, fill_line_index; /* chip line/sample indices */
  int first_samp_in_line;    /* 1st sample in current line for current pass */
  int ix, iy;                /* integer forms of x0, x0 + 1, and y0 */
  int ihi, ilow;             /* counters for two image lines for bilinear
                                resampling */
  int ihi_x_samp, ilow_x_samp; /* values of ihi and ilow muliplied by the
                                  number of samples */
                             /* value of ilow*samp */
  int line_start_index;      /* current starting line (all previous have
                                been filled) */
  int need_newline;          /* Boolean flag indicating whether a new image
                                line needs to be read */
  int resampled_dn;          /* temporary storage for resampled DN */
  int samp_start_index[128]; /* current starting sample for each chip line */
  int itmp;                  /* temporary value  */
  int num_reads = 0;
  
  long line_rec;             /* image line counter */
  long beg_line_rec;
  const long one = 1;        /* constant one value */
  
  float dn1 = 0.0,           /* digital (pixel) numbers for four pixels */
        dn2 = 0.0,           /* used during resampling                  */
        dn3 = 0.0,
        dn4 = 0.0;
  float dn12, dn34;          /* linear resampling values */
  float *temp_buf;           /* Temporary buffer for image file read */
  float x0, y0;              /* transform space line/sample location */
  float xt_x[128], xt_y[128],/* incremental transform terms */
        yt_x[128], yt_y[128];/* xt_x = x component of the x transform
                                xt_y = y component of the x transform
                                similar for yt_x and yt_y */

  
  /* Allocate space for temp_buf[2][#samps]. */
  temp_buf = (float *)malloc(2*image->ns*sizeof(float));
  if (temp_buf == NULL)
  {
      c_lasmsg (FALSE, "resample-alloc", "Error allocating memory");
      return E_FAIL;
  }

  /* Set up parameters for the resampling loop. */
  *chop = 0;
  ihi = 1;
  ilow = 0;
  /* printf( "line_sign = %d\n",  line_sign );  XXX */
  fill_samp_0 = (size[1] + 1)*(1 - line_sign)/2 - 1;
  fill_line_0 = (size[0] + 1)*(1 - samp_sign)/2 - 1;
  for (fill_samp_index=0; fill_samp_index<size[1]; fill_samp_index++)
  {
      yt_y[fill_samp_index] = trans[0]*(fill_samp_index + 1);
      yt_x[fill_samp_index] = trans[1]*(fill_samp_index + 1);
      xt_y[fill_samp_index] = trans[3]*(fill_samp_index + 1);
      xt_x[fill_samp_index] = trans[4]*(fill_samp_index + 1);
  }
  for (fill_line_index=0; fill_line_index<size[0]; fill_line_index++)
      samp_start_index[fill_line_index] = 0;
  line_start_index = 0;
  
  line_rec = (long)ystart;
  /* printf( "ystart = %f\n", ystart );  XXX */

  /* If the current line is within the image, read the line section that
     is relevant to the chip resampling. */
  if (line_rec > 0 && line_rec <= image->nl)
  {
      if (c_eread(ptr, &image->band, &line_rec, (unsigned char *)&temp_buf[0],
                  &one) == E_FAIL)
      {
          c_lasmsg(FALSE, "resample-read", "Error reading input file");
          free(temp_buf);
          temp_buf = NULL;
          return READ_ERROR;
      }
      num_reads++;
  }
  else
  {
      c_lasmsg(FALSE, "resample-read(1)", "Line %d outside image", line_rec);
      printf( "ystart = %f\n", ystart ); /* XXX */
      free(temp_buf);
      temp_buf = NULL;
      return READ_ERROR;
  }

  /* Begin the loop to fill the chip buffer.
     This is an infinite loop that is left when either the entire
     image chip has been filled or a read error occurs while retrieving
     a new transform-space line.
  ---------------------------------------------------------------------*/
  beg_line_rec = line_rec;
  while (TRUE)
  {
      /* Read the next line. */
      itmp = ihi;
      ihi = ilow;
      ilow = itmp;
      line_rec++;
      if (line_rec > 0 && line_rec <= image->nl)
      {
          if (c_eread(ptr, &image->band, &line_rec,
                      (unsigned char *)&temp_buf[ilow*image->ns], &one)
              == E_FAIL)
          {
              c_lasmsg(FALSE, "resample-read", "Error reading input file");
              free(temp_buf);
              temp_buf = NULL;
              return READ_ERROR;
          }
          num_reads++;
      }
      else
      {
          /*
          printf( "beginning line_rec = %d\n", beg_line_rec );
          printf( "line_rec = %d\n", line_rec );
	  printf( "size array: %d, %d\n", size[0], size[1] );
	  printf( "starting count for fill line, samp: %d, %d\n", fill_line_0, fill_samp_0);
	  printf( "line/samp signs: %d, %d\n", line_sign, samp_sign );
          printf( "fill_line_index = %d\n", fill_line_index );
          printf( "line_start_index = %d\n", line_start_index );
          printf( "num_reads = %d\n", num_reads );
          */
	  c_lasmsg(FALSE, "resample-read(2)", "Line %d outside image", line_rec);
          free(temp_buf);
          temp_buf = NULL;
          return READ_ERROR;
      }

      /* Reset the flag indicating that we need to read a new image line. */
      need_newline = FALSE;
      
      /* Skip the appropriate number of samples to get to the current line. */
      ihi_x_samp = ihi*image->ns;
      ilow_x_samp = ilow*image->ns;

      /* Loop over all lines in the chip. */
      for (fill_line_index=line_start_index; fill_line_index<size[0];
           fill_line_index++)
      {
          /* Determine the next line. */
          itmp = labs(fill_line_index+1);
          if (samp_sign < 0)
              itmp = -itmp;
          fill_line = fill_line_0 + itmp;

          first_samp_in_line = samp_start_index[fill_line];
          
          /* Loop over all samples in the chip line, picking up where
             we left off for this line. */
          for (fill_samp_index=first_samp_in_line;
               fill_samp_index<size[1]; fill_samp_index++)
          {
              /* Determine the next sample location. */
              itmp = labs(fill_samp_index+1);
              if (line_sign < 0)
                  itmp = -itmp;
              fill_samp = fill_samp_0 + itmp;

              /* Calculate the y and x location of the kernel grid point. */
              y0 = basey + yt_y[fill_line] + yt_x[fill_samp];
              x0 = basex + xt_y[fill_line] + xt_x[fill_samp];

              /* Update the first-sample index for this line. */
              samp_start_index[fill_line] = fill_samp_index;

              /* If no more samples can be resampled for this line value
                 and the current transform space lines, break out of the
                 sample loop. */
              if (y0 >= (float) line_rec)
              {
                  /* If this is the first sample of this fill line or
                     this is the last fill line, then we are done
                     with this line and need to grab a new one. */
                  if (fill_samp_index == first_samp_in_line ||
                      fill_line_index == size[0] - 1)
                      need_newline = TRUE;
                  
                  break;
              }

              /* If the transform-space location is within the image,
                 resample the four adjacent pixels.  Otherwise, set the
                 pixel value to zero and increment the number of failed
                 resampling attempts. */
              if (x0 >= 1.0 && x0 <= (float)image->ns && y0 >= 1.0 &&
                  y0 <= (float)image->nl)
              {
                  ix = (int)x0;
                  iy = (int)y0;

                  /* Interpolate the four points around the kernel grid
                     point. */
                  dn1 = temp_buf[ihi_x_samp+ix-1];
                  dn2 = temp_buf[ihi_x_samp+ix];
                  dn3 = temp_buf[ilow_x_samp+ix-1];
                  dn4 = temp_buf[ilow_x_samp+ix];
                  dn12 = dn1 + (dn2 - dn1)*(x0 - ix);
                  dn34 = dn3 + (dn4 - dn3)*(x0 - ix);
                  resampled_dn = (int)(dn12 + (dn34 - dn12)*(y0 - iy) + 0.5);
              }
              else
              {
                  resampled_dn = 0;
                  (*chop)++;
                  if (*chop > 5000)
                  {
                      c_lasmsg(FALSE, "resample-point",
                          "Too many grid points lie outside of image bounds");
                      free(temp_buf);
                      temp_buf = NULL;
                      return E_FAIL;
                  }
              }
              
              /* Store the resampled pixel value. */
              buffer[fill_line*size[1]+fill_samp] = (float)resampled_dn;
              
          }  /* for fill_samp_index , sample loop */

          /* If a new line needs to be read, break out of the line loop.
             Otherwise, see if we have completed this line so we can
             update the first line index. */
          if (need_newline)
              break;
          else if (fill_samp_index == size[1])
          {
              /* Make sure that the lines are being filled sequentially
                 and no lines are being left unfilled. */
              if (line_start_index != fill_line_index)
              {
                  c_lasmsg(FALSE, "resample-fill",
                          "Resampled lines are not being filled sequentially."
                          "  This should not happen.  Contact LAS support." );
                  free(temp_buf);
                  temp_buf = NULL;
                  return E_FAIL;
              }   
              line_start_index++;
          }
          
      }  /* for fill_line_index, line loop */

      /* If the chip has been filled, return to the calling routine.
         The chip has been filled if we get to this point without
         needing another image line. */
      if (line_start_index == size[0] && ! need_newline)
      {
          free(temp_buf);
          temp_buf = NULL;
          /* XXX
          printf( "*** line_start_index at %d, nreads at %d\n", line_start_index, num_reads );
          */
          return E_SUCC;
      }
      
  }  /* while (TRUE) */
  
}
