/******************************************************************************

NAME: CORRPLOT

PURPOSE: Generate a correlation residual plot.

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----       ------           -------
    1.0         10/92      D.Steinwand      Initial Development
    2.0         12/92      K.Sheffield      Baseline release
    3.0		 9/93	   T.Mittan	    Modified to plot inactive and
					    active points with different 
					    symbols
    4.0          9/96      B.Olson          Modified to allow choice of
                                            residual display direction and
                                            label display.  Removed legend
                                            when only one point type.
    5.0         11/99      D.Hames          Modify the variables xfrom, xto,
					    yfrom, yto to the actual x/y values
					    instead of initializing them to 
				            equal MAXNS. This is to support
					    images > 2gig. Add prototypes.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
    Runs under LAS/TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

Call GETPAR to get the input parameters from TAE.
Open the residual data file and the gnuplot data and command files.
Determine the output terminal type for gnuplot.
Setup the plotter.
Read in each point and write it out to gnuplot's data file.
Finish up and exit.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define TRUE 1
#define FALSE 0
#define TERM 0
#define LP   1
#define FIL  2

struct PARBLK block;

void main
(void)
{
   FILE *cmd_fp;                        /* The GNUPlot command file          */
   FILE *dataa_fp;                       /* The GNUPlot data file             */
   FILE *datai_fp;                       /* The GNUPlot data file             */
   FILE *in_fp;                         /* Input residual file               */
   FILE *pr_fp;                         /* Printer/terminal file             */
   
   char cmdfil[CMLEN];                  /* name of the GNUPlot command file  */
   char datfil[CMLEN];                  /* name of the GNUPlot data file     */
   char datifil[CMLEN];                 /* name of the GNUPlot inactive data 
								file         */
   char *display;                       /* pointer to the DISPLAY environment
					   variable                          */
   
   char infil[CMLEN];                   /* residual filename                 */
   char prtfil[CMLEN];                  /* name of the printer/terminal file */
   char ptype[20];                      /* printer type                      */
   char title[CMLEN];                   /* title of the plot                 */
   char dspflg[3];                      /* display point labels flag         */
   char plotopt[10];                    /* plot options for residuals        */

   long label;                          /* label points flag                 */
 
   long act;				/* variable to read in active flag   */
   long act_flag = FALSE;		/* active flag			     */
   long inact_flag = FALSE;		/* inactive flag		     */
   long dpi;                            /* printer resolution  dots per inch */
   long fatal=-1;                       /* fatal error flag                  */
   long len;				/* string length		     */
   long i;                              /* working index                     */
   long npts;                           /* the number of points to plot      */
   long print[3];                       /* terminal flags                    */
   long scale;                          /* residual exaggeration factor      */
   long zero = 0;

   long line = 0;                       /* flag for line residuals           */
   long sample = 0;                     /* flag for sample residuals         */
 
   float x,y;                           /* point coordinates                 */
   float dx,dy;                         /* point residuals                   */
   char ptid[20];                       /* point id/name                     */
   float xfrom,yfrom,xto,yto;           /* plot bounding box                 */
   float rmse;                          /* residual mean                     */
   
   /* get the parameters from TAE 
      ---------------------------*/

   getpar(cmdfil,infil,datfil,prtfil,&scale,ptype,dspflg,plotopt,&dpi,title, print);
    
   /* open the files needed
      ---------------------*/

   if((in_fp=fopen(infil,"r")) == NULL)
     c_errmsg("Error opening input plot file","corrplot-fopen",&fatal);

   if((cmd_fp=fopen(cmdfil,"w")) == NULL)
     c_errmsg("Error opening plot command file","corrplot-fopen",&fatal);

   if((dataa_fp=fopen(datfil,"w")) == NULL)
     c_errmsg("Error opening output data file","corrplot-fopen",&fatal);

   strcpy(datifil,datfil);
   len = (long)strlen (datifil);
   datifil[len-4] = 'i';
   datifil[len-3] = '.';
   datifil[len-2] = 'd';
   datifil[len-1] = 'a';
   datifil[len]   = 't';    
   datifil[len+1] = '\0';

   if((datai_fp=fopen(datifil,"w")) == NULL)
     c_errmsg("Error opening output data file","corrplot-fopen",&fatal);

   /* kludge to work around termout errors
      ------------------------------------*/
   if((pr_fp=fopen(prtfil,"w")) == NULL)
     c_errmsg("Error opening printer file","corrplot-fopen",&fatal);
   fclose(pr_fp);
  
   if (strcmp(dspflg,"YES")==0)
     label=1;
   else
     label=0;
 
   if (strcmp(plotopt,"BOTH")==0 || strcmp(plotopt,"LINE")==0)
     line = 1;
   if (strcmp(plotopt,"BOTH")==0 || strcmp(plotopt,"SAMPLE")==0)
     sample = 1; 

   /* Write GNUPLOT command file
      --------------------------*/
   if(print[LP] || print[FIL])
   {
     if (strncmp(ptype,"ps",2) == 0)
     {
       if(EOF == fprintf(cmd_fp,"set term postscript \"Courier\" 12\n"))
	 c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
	 
     }
     else if (strncmp(ptype,"hplj",4) == 0)
     { 
       if(EOF == fprintf(cmd_fp,"set term hpljii %d\n",dpi))
         c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
     }
     else if (strncmp(ptype,"ascii",5) == 0)
     { 
       if(EOF == fprintf(cmd_fp,"set term dumb 80 60\n"))
	 c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
     }
     if( EOF == fprintf(cmd_fp,"set output \"%s\"\n",prtfil))
       c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
   }
   else if(print[TERM])
   {
     if((display = getenv("DISPLAY")) == NULL)
     {
       if( EOF == fprintf(cmd_fp,"set term dumb \n"))
	 c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

       if(EOF == fprintf(cmd_fp,"set output \"%s\"\n",prtfil))
	 c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

     }
     else
     {
       if( EOF == fprintf(cmd_fp,"set term x11 \n"))
	 c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

     }
      
   }
   
   
   if(EOF == fprintf(cmd_fp,"set ylabel \"LINES\"\n"))
     c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

   if(strcmp(title,""))
     if(EOF == fprintf(cmd_fp,"set title \"%s\"\n",title))
       c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

   if(EOF == fprintf(cmd_fp,"set tics\n"))
     c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
   
   /* Read residual data file and create the gnuplot data file
      -------------------------------------------------------*/

   if(1 != fscanf(in_fp,"%d",&npts))
     c_errmsg("Error reading file","corrplot-fscanf",&fatal);

   for(i=0;i<npts;i++) 
   {
     /* read the point and it's residuals
	 --------------------------------*/

     if(6 != fscanf(in_fp,"%s %f %f %f %f %d",ptid,&x,&y,&dx,&dy,&act))
       c_errmsg("Error reading file","corrplot-fscanf",&fatal);

     /* set flags
     ----------*/
     if (act == 0)
       {
       act_flag = TRUE;
       }
     else
       {
       inact_flag = TRUE;
       }

     /* exaggerate the residual for visualization purposes
        --------------------------------------------------*/

     dx = x + (dx * (float)scale);
     dy = y + (dy * (float)scale);

     if(label)
       {
       if(EOF == fprintf(cmd_fp,"set label %d \"%s\" at %f,%f right\n",
	         i+1,ptid,x-20,y-20))
         c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
       }

     if (!line)
       {
       if(EOF == fprintf(cmd_fp,"set arrow %d from %f,%f to %f,%f\n",
                 i+1,x,y,dx,y))
         c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
       }
     else if (!sample)
       {
       if(EOF == fprintf(cmd_fp,"set arrow %d from %f,%f to %f,%f\n",
	         i+1,x,y,x,dy))
         c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
       }
     else
       if(EOF == fprintf(cmd_fp,"set arrow %d from %f,%f to %f,%f\n",
                 i+1,x,y,dx,dy))
         c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

     if (act == 0)
       {
       if(EOF == fprintf(dataa_fp,"%f %f\n",x,y))
         c_errmsg("Error writing data file","corrplot-fprintf",&fatal);
       }
     else
       {
       if(EOF == fprintf(datai_fp,"%f %f\n",x,y))
         c_errmsg("Error writing data file","corrplot-fprintf",&fatal);
       }
      
     /* initialize the plot bounding box
        -------------------------------*/
     if (i==0)
        {
        xfrom = x;
        yfrom = y;
        xto = x;
        yto = y;
        }
 

     /* recalculate the plot bounding box
        ---------------------------------*/

     if (x < xfrom) xfrom=x;
     if (dx < xfrom) xfrom=dx;
     if (y < yfrom) yfrom=y;
     if (dy < yfrom) yfrom=dy;
     if (x > xto) xto = x;
     if (dx > xto) xto = dx;
     if (y > yto) yto = y;
     if (dx > xto) xto = dx;
     if (dy > yto) yto = dy;
   }

   yto += 100.0;  
   xto += 100.0;
   yfrom -= 100.0; 
   xfrom -= 100.0;


   /* finish up the plotting parameters
      --------------------------------*/

   if(1 != fscanf(in_fp,"%f",&rmse))
     c_errmsg("Error reading file","corrplot-fscanf",&fatal);
   
   if(EOF == fprintf(cmd_fp,"set yrange [%d:%d]\n",(int)yto,(int)yfrom))
     c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

   if(EOF == fprintf(cmd_fp,"set xrange [%d:%d]\n",(int)xfrom,(int)xto))
     c_errmsg("Error writing command file","corrplot-fprintf",&fatal);

   if(EOF == fprintf(cmd_fp,
		     "set xlabel \"SAMPLES     (SCALE = %d   RMSE = %f)\"\n",
		     scale,rmse))
     c_errmsg("Error writing command file","corrplot-fprintf",&fatal);


   if ((act_flag) && (!inact_flag))
     {
     if(EOF == fprintf(cmd_fp,"plot \"%s\" notitle\n",datfil))
       c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
     }


   else
   if ((!act_flag) && (inact_flag))
     {
     if(EOF == fprintf(cmd_fp,"plot \"%s\" notitle with points 1 3\n",datifil))
       c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
     }
   else
   if ((act_flag) && (inact_flag))
     {
     if(EOF == fprintf(cmd_fp,
               "plot \"%s\" title \"active\", \"%s\" title \"inactive\" with points 1 3 \n",
               datfil,datifil))
       c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
     }

   if(print[TERM] && display != NULL )
     if(EOF == fprintf(cmd_fp,
                       "pause 10\npause -1 \"Hit return to continue\"\n"))
       c_errmsg("Error writing command file","corrplot-fprintf",&fatal);
   
   
   /* Close files and exit
      --------------------*/
   fclose(in_fp);
   fclose(cmd_fp);
   fclose(dataa_fp);
   fclose(datai_fp);

   c_complt ("CORRPLOT",NULL , &zero); 
}
