/**************************************************************************
NAME:  				enlarge

PURPOSE:  Enlarges a window using an interpolated zoom.  Resampling 
          techniques include nearest neighbor, bilinear interpolation,
          parametric cubic convolution, and a user entered table of
          resampling weights.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		3/89	D. Steinwand	   CSB      Original development

ALGORITHM DESCRIPTION:
	Perform enlargement & resampling in the horizontal direction
	Perform enlargement & resampling in the vertical direction
	Return

ALGORITHM REFERENCES:
  1.  D.E. Friedmann, "Two-Dimensional Resampling of Line Scan Imagery
       by One-Dimensional Processing," Photogrammetric Engineering and
       Remote Sensing, Vol. 47, No. 10, pp. 1459-1467, October 1981.
  2.  D.E. Friedmann, "Operational Resampling for Correcting Images
       to a Geocoded Format," MacDonald, Dettwiler & Associates Ltd.,
       Richmond, B.C., Canada, May 1981.
  3.  S.K. Park and R.A. Schowengerdt, "Image Reconstruction by
       Parametric Cubic Convolution," Computer Vision, Graphics,
       and Image Processing 23, Academic Press, Inc. 1983. pp. 258-272.
*****************************************************************************/
#include "las.h"
#include "tiepts.h"

void enlarge
(
    struct ENLARGE *zoom      /* I/O: Enlargement parameters & buffers (I/O) */
)
{
horiz_zoom(zoom);
vert_zoom(zoom);
return;
}

/**************************************************************************

NAME:  				horiz_zoom

PURPOSE:  Performs interpolated enlargement in horizontal direction.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		3/89	D. Steinwand	   CSB      Original development

ALGORITHM DESCRIPTION:
	Assign an offset value (pad) so that extra pixels needed for
	  resampling are skipped when going to the next line of data
	Assign "working" pointers to the input and hybrid space buffers
	Loop on the number of lines in input space
	  Loop on the number of samples in input space
	    Loop on the enlargement factor
	       Loop on the dimension of the sampling kernel
		 Hybrid space pixel = hybrid space pixel + pixel * weight
	Return
*****************************************************************************/
void horiz_zoom
(
    struct ENLARGE *zoom      /* I/O: Enlargement parameters & buffers (I/O) */
)
{
register long *hybrid;		/* Pointer to current pixel in hybrid space */
register long *weight;		/* Pointer to current weight table location */
register unsigned char *in_buf;	/* Pointer to current input pixel */
register unsigned char *ptr;	/* Pointer to current input pixel */
register long h, i, j, k;	/* Loop counters */
long pad;			/* Pad so extract_lines = in_lines - pad */

/* Initialize offsets and pointers to buffers
  ------------------------------------------*/
pad = zoom->extract_lines - zoom->in_lines;
hybrid = zoom->work;
in_buf = zoom->input;

/* Perform the horizontal resampling.  The resampling type is determined
   in init_zoom() and is placed in a table.  This routine applies the table,
   regardless of the resampling method chosen (yes, even nearest neightbor).
   The table is scaled up by 8 bits (multiplied by 256) to keep some 
   precision beyond the radix point and to allow the elimination of all
   floating point calculations.  The removal of the floating point calculations
   sped this algorithm up significantly.  The original range of the data
   in the hybrid space is not restored until the completion of the vertical
   pass.
  ------*/
for (h = zoom->extract_lines; h--; in_buf += pad)
  for (i = zoom->in_samps; i--; in_buf++)
    for (weight = zoom->horiz_table, j = zoom->factor; j--; hybrid++)
      for (*hybrid = 0, ptr = in_buf, k = zoom->horiz_dim; k--;
					*hybrid += *(ptr++) * *weight++ );
/*
      for (*hybrid = 0, k = 0; k < zoom->horiz_dim; k++)
	*hybrid += *(in_buf + k) * *weight++;
*/
return;
}

/**************************************************************************

NAME:  				vert_zoom

PURPOSE:  Performs interpolated enlargement in vertical direction.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		3/89	D. Steinwand	   CSB      Original development

ALGORITHM DESCRIPTION:
	Loop on the number of samples in hybrid space
	  Assign "working" pointers to the output and hybrid space buffers
	  Loop on the number of lines in input space
	    Loop on the enlargement factor
	       Loop on the dimension of the sampling kernel
		 Output space pixel = Output space pixel + pixel * weight
               If the resulting pixel is < 1, clip to 0
	       If the resulting pixel is > 255.0, clip to 255
	       Assign the output space pixel to the output buffer
	Return
*****************************************************************************/
void vert_zoom
(
    struct ENLARGE *zoom      /* I/O: Enlargement parameters & buffers (I/O) */
)
{
register long temp;		/* Temporary floating point output result */
register long *hybrid;		/* Pointer to current pixel in hybrid space */
register long *weight;		/* Pointer to current weight table location */
unsigned char *out_buf;		/* Pointer to current output pixel */
long inc;
register long h, i, j, k;	/* Loop counters */


/* The vertical resampling pass is very similar to the horizontal resampling
   pass, however, the added complexity in the code comes from accessing the
   data in columns instead of rows (lines).  As in the horizontal pass, the
   weight table is scale by 8 bits to remove floating point calculations.
   Since this pass and the horizontal pass involve multiplication of scaled
   values, the final result must be adjusted by performing an integer division
   by 16 bits (8 bits applied in the horizontal pass and 8 bits applied in
   the vertical pass).  A bit shift would be more efficient, but probably not
   portable, so it is avoided.
  ---------------------------*/
for (h = 0; h < zoom->hybrid_samps; h++)
  {
  out_buf = zoom->output + h;
  hybrid = zoom->work + h;
  for (i = zoom->in_lines; i--; hybrid += zoom->hybrid_samps)
    for(weight=zoom->vert_table,j=zoom->factor;j--;out_buf+=zoom->hybrid_samps)
      {
      for (temp = 0,inc = 0, k = zoom->vert_dim; k--; 
		temp += *(hybrid + inc) * *weight++, inc += zoom->hybrid_samps);
      if (temp < 65536) *out_buf = 0;
      else if (temp > 16711680) *out_buf = 255;
      else *out_buf = (unsigned char)(temp / 65536);
      }
  }
return;
}
