/**************************************************************************
NAME:  				init_zoom

PURPOSE:  Initializes the resampling tables & allocates buffer space for
          the interpolated enlargement process.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		3/89	D. Steinwand	   CSB      Original development
  5.1 		5/89    D. Steinwand	   CSB      Added nearest neighbor 
						    and table-based resampling
						    methods; general cleanup

ALGORITHM DESCRIPTION:
	If not the first time through this routine, free buffer space
	Calculate 1 / zoom factor
	If resampling = Nearest Neighbor,
	   Allocate memory for the resampling table
	   Calculate resampling table
	If resampling = Parametric Cubic Convolution,
	   Allocate memory for the resampling table
	   Calculate resampling table
	If resampling = Bilinear Interpolation,
	   Allocate memory for the resampling table
	   Calculate resampling table
	If resampling = Table-based,
	   Read the resampling table from disk
	   Set up table dimensions
	   Allocate memory for the resampling table
	   Assign elements from disk-based table to local table
	Allocate buffer space for input data
	Allocate buffer space for the working area
	Allocate buffer space for the output data
	Return

ALGORITHM REFERENCES:
  1.  D.E. Friedmann, "Two-Dimensional Resampling of Line Scan Imagery
       by One-Dimensional Processing," Photogrammetric Engineering and
       Remote Sensing, Vol. 47, No. 10, pp. 1459-1467, October 1981.
  2.  D.E. Friedmann, "Operational Resampling for Correcting Images
       to a Geocoded Format," MacDonald, Dettwiler & Associates Ltd.,
       Richmond, B.C., Canada, May 1981.
  3.  S.K. Park and R.A. Schowengerdt, "Image Reconstruction by
       Parametric Cubic Convolution," Computer Vision, Graphics,
       and Image Processing 23, Academic Press, Inc. 1983. pp. 258-272.
*****************************************************************************/
#include "las.h"
#include "tiepts.h"

int init_zoom
(
    struct ENLARGE *zoom	/* Enlargement parameters & buffers (I/O) */
)
{
struct RWTAB user_rwt;		/* User entered resampling weight table */
static int first = TRUE;	/* Flag--has init_zoom been called before? */
register long *index;		/* Weight table index */
register float	tmpdel;		/* Temporary delta value */
register float	delta;		/* Delta value; I (loop counter) * FACTOR */
float inc;			/* Table increment for table-based resampling */
float inv;			/* 1.0 / zoom->factor */
float offset;			/* Offset for even zoom factors */
long count;			/* Index incrementer for nn table calc */
long pass = 1;			/* Nonfatal error flag for c_errmsg */
register long i,j,k;		/* Loop index */
unsigned long size;		/* Size of weight table buffer */

/* If this is not the first time this routine has been called, free any
   previously allocated buffer space.
  ----------------------------------*/
if (!first)
    {
    free(zoom->horiz_table);
    free(zoom->work);
    free(zoom->input);
    free(zoom->output);
    }

/* Calculate 1/zoom->factor
  ------------------------*/
inv = 1.0 / (float) zoom->factor;

/* Nearest neighbor table initialization
  -------------------------------------*/
if (strcmp(zoom->method, "NN") == 0)
   {
   size = 2 * zoom->factor * sizeof(long);
   zoom->horiz_table = zoom->vert_table = (long *)malloc(size);
   if (zoom->horiz_table == NULL) 
	{
	c_errmsg("Error allocating memory","corrview-alloc",&pass);
	return(E_FAIL);
	}
   count = zoom->factor / 2;
   if ((float)count < ((float)zoom->factor / 2.0)) count++;
   for (index = zoom->horiz_table, i = count; i--;)
   	{
   	*index++ = 256;
   	*index++ = 0;
   	}
   for (i = zoom->factor - count; i--;)
   	{
   	*index++ = 0;
   	*index++ = 256;
   	}
   zoom->horiz_dim = zoom->vert_dim = 2;
   zoom->extract_lines = zoom->in_lines + 1;
   zoom->extract_samps = zoom->in_samps + 1;
   }

/* Parametric cubic convolution table initialization
  -------------------------------------------------*/
if (strcmp(zoom->method, "CC") == 0)
   {
   size = 4 * zoom->factor * sizeof(long);
   zoom->horiz_table = zoom->vert_table = (long *)malloc(size);
   if (zoom->horiz_table == NULL) 
	{
	c_errmsg("Error allocating memory","corrview-alloc",&pass);
	return(E_FAIL);
	}

   count = zoom->factor / 2;
   if ((float)count < ((float)zoom->factor / 2.0)) offset = 0;
   else offset = 1.0 / (2.0 * zoom->factor);
   for(index = zoom->horiz_table, i = 0; i < zoom->factor; i++)
	{
	delta = ((float)i * inv) + offset;
	tmpdel = 1 - delta;
	*index++ = (long)(256 * zoom->alpha * delta * SQUARE(tmpdel));
	*index++ = (long)(256 * (1.0 - ((zoom->alpha+3.0) * SQUARE(delta)) +
	   ((zoom->alpha + 2.0) * CUBE(delta))));
	*index++ = (long)(256 * (1.0 - ((zoom->alpha+3.0) * SQUARE(tmpdel)) +
	   ((zoom->alpha+2.0) * CUBE(tmpdel))));
	*index++ = (long)(256 * zoom->alpha * tmpdel * SQUARE(delta));
	}
   zoom->horiz_dim = zoom->vert_dim = 4;
   zoom->extract_lines = zoom->in_lines + 3;
   zoom->extract_samps = zoom->in_samps + 3;
   }

/* Bilinear interpolation table initialization
  -------------------------------------------*/
if (strcmp(zoom->method, "BI") == 0)
   {
   size = 2 * zoom->factor * sizeof(long);
   zoom->horiz_table = zoom->vert_table = (long *)malloc(size);
   if (zoom->horiz_table == NULL) 
	{
	c_errmsg("Error allocating memory","corrview-alloc",&pass);
	return(E_FAIL);
	}

   count = zoom->factor / 2;
   if ((float)count < ((float)zoom->factor / 2.0)) offset = 0;
   else offset = 1.0 / (2.0 * zoom->factor);
   for (index = zoom->horiz_table, i = 0; i < zoom->factor; i++)
   	{
   	delta = ((float)i * inv) + offset;
   	*index++ = (long)(256 * (1 - delta));
   	*index++ = (long)(256 * delta);
   	}
   zoom->horiz_dim = zoom->vert_dim = 2;
   zoom->extract_lines = zoom->in_lines + 1;
   zoom->extract_samps = zoom->in_samps + 1;
   }

/* User entered table-based resampling table read & initialization
  ---------------------------------------------------------------*/
if (strcmp(zoom->method, "TABLE") == 0)
	{
        if(c_getrwt(zoom->table_file, &user_rwt) != E_SUCC) return(E_FAIL);
        zoom->horiz_dim = user_rwt.dim[0];
        zoom->vert_dim = user_rwt.dim[1];
       	zoom->extract_lines = zoom->in_lines + (zoom->vert_dim - 1);
   	zoom->extract_samps = zoom->in_samps + (zoom->horiz_dim - 1);
    	size=(zoom->horiz_dim * zoom->factor * sizeof(long)) +
             (zoom->vert_dim * zoom->factor * sizeof(long));
        zoom->horiz_table = (long *)malloc(size);
 	if (zoom->horiz_table == NULL)
	   {
	   c_errmsg("Error allocating memory","corrview-alloc",&pass);
	   return(E_FAIL);
	   }
	zoom->vert_table = zoom->horiz_table + (zoom->horiz_dim * zoom->factor);

   	count = zoom->factor / 2;
   	if ((float)count < ((float)zoom->factor / 2.0)) offset = 0;
   	else offset = 32.0 / (2.0 * zoom->factor);
 
	index = zoom->horiz_table;
        inc = 32.0 / (float)zoom->factor;
	for (k = 0; k < zoom->factor; k++)
	   for (j=0,i=(long)(((float)k * inc)+offset); j < zoom->horiz_dim; j++)
		*index++ = (long)(256 * user_rwt.table1[i][j]);

	index = zoom->vert_table;
	for (k = 0; k < zoom->factor; k++)
	   for (j=0,i=(long)(((float)k * inc)+offset); j < zoom->vert_dim; j++)
		*index++ = (long)(256 * user_rwt.table2[i][j]);
	}

/* Allocate input, hybrid, and output space buffers
  ------------------------------------------------*/
zoom->hybrid_samps = zoom->in_samps * zoom->factor;

size = zoom->extract_lines * zoom->extract_samps * sizeof(unsigned char);
zoom->input = (unsigned char *)malloc(size);
if (zoom->input == NULL) 
	{
	c_errmsg("Error allocating memory","corrview-alloc",&pass);
	return(E_FAIL);
	}

size = zoom->extract_lines * zoom->hybrid_samps * sizeof(long);
zoom->work = (long*)malloc(size);
if (zoom->work == NULL) 
	{
	c_errmsg("Error allocating memory","corrview-alloc",&pass);
	return(E_FAIL);
	}

size = zoom->in_lines * zoom->factor * zoom->in_samps * zoom->factor * 
       sizeof(unsigned char);
zoom->output = (unsigned char *)malloc(size);
if (zoom->output == NULL) 
	{
	c_errmsg("Error allocating memory","corrview-alloc",&pass);
	return(E_FAIL);
	}

first = FALSE;
return(E_SUCC);
}
