/***************************************************************************
NAME: DDRCHECK

PURPOSE:  Checks the DDR file for validity

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     07/95 D.Steinwand	 C    Original utility in release
 2.0     12/97 B.Olson           C    Several changes. Added verification
				      for image attributes, compared file
				      size to reported, compared geographic
				      coords to zone, checked coordinates 
				      for reasonableness.
 2.1     04/98 B.Olson           C    Clarified meaning of valid flag,
                                      corrected handling of sample size 
                                      equal to zero and corrected how 
                                      coordinates in degrees are handled.
 2.2     06/98 B.Olson           C    Corrected how coordinates in DMS
                                      are handled.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
  Import the names of the DDR and image files.
  Read in the DDR file and the stats from the image file.
  Compare the size of the image file to the number of lines times the 
    number of samples times the number of bands times the pixel size and
    report the results.
  Compare the number of samples reported in the DDR against either the 
    corner coordinates and sample size (if they are in the DDR) or against
    the image size divided by the product of the number of bands times the
    number of lines times the pixel size and report the results.
  Compare the number of lines and bands in the same manner.
  If the image is UTM, convert the coordinates to DMS, determine the zone, and 
    compare to the zone in the DDR.  Report the results.
  If there is a discrepency between the zones, report the corner coordinates.
  Report the projection code from the DDR.
  Report the zone code from the DDR only if the projection code 
    is UTM or State Plane.
  Report the datum code from the DDR.
  Report the projection parameters based on the projection code in the DDR.
  Report the projection units from the DDR.
  Report the projection distance from the DDR and the total projection
    distance from number of samples/lines times the projection distance.
  Report the corner coordinates from the DDR.
  Using the corner coordinates from the DDR, calculate the distance along
    each side of the image.
  Compare the lengths of opposite sides and report the results if there
    is a difference.
  Using the corner coordinates from the DDR, calculate the tilt of each
    side of the image.  If the opposite sides do not have the same angle 
    report the results.
  Compare the angles for right angles at the corners.  If the angles are
    not exactly 90, report the results.
  Report the line and sample increments from the DDR.

**************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "util.h"
#include "protos.h"

#define TERM     0
#define PFILE    1
#define LP       2
#define NOTVALID 0
#define VALID    1
#define UNKNOWN  2

FILE *fp = NULL;                 /* file pointer to print file             */
long print[3];                   /* output destination                     */

int main
(void)
{
    struct PARBLK parblk;          /* TAE parameter block                    */


    struct DDR ddr;                /* data in records 1 and 2 of DDR         */
    struct DDR outddr;             /* translated data in rcds 1 & 2 of DDR   */
    struct stat stats;             /* file information from system           */
    struct COORDS
    {
        long degree;                 /* degree portion of coordinate         */
        long minute;                 /* minute portion of coordinate         */
        double sec;                  /* second portion of coordinate         */
    };
    struct COORDS lat;             /* expanded data for latitude             */
    struct COORDS lon;             /* expanded data for longitude            */

    long degree = 0;               /* degree portion of coordinate           */
    long fatal = -1;               /* fatal error code                       */
    long i = 0;                    /* loop counter                           */
    long minute = 0;               /* minute portion of coordinate           */
    long mode = 0;                 /* file access mode                       */
    long one = 1;                  /* one                                    */
    long out_units = 0;            /* unit of measure to convert into        */
    long prtprm[2];                /* print parms for c_transinit            */
    long status = 0;               /* function return status code            */
    long std_out = 0;              /* logical unit number for standard output*/
    long temp_size = 0;            /* temporary storage for calculated sizes */
    long temp_units = 2;           /* unit of measure used                   */
    long tmpnum=0;                 /* temporary hold for pixel size          */
    long zero = 0;                 /* zero                                   */

    double bot_dist = 0.0;         /* calculated distance across image bottom*/
    double botX = 0.0;             /* angle of image bottom from horizontal  */
    double left_dist = 0.0;        /* calculated distance along image side   */
    double leftY = 0.0;            /* angle of image side from vertical      */
    double outcoord[2];            /* converted coordinates                  */
    double right_dist = 0.0;       /* calculated distance along image side   */
    double rightY = 0.0;           /* angle of image side from vertical      */
    double sec = 0.0;              /* second portion of coordinate           */
    double tmpdbl = 0.0;           /* temporary for difference in angles     */
    double tmpdbl2 = 0.0;          /* temporary for difference in angles     */
    double top_dist = 0.0;         /* calculated distance across image top   */
    double topX = 0.0;             /* angle of image top from horizontal     */

    char hname[CMLEN];             /* host name of input image               */
    char prtfile[CMLEN];           /* print file name                        */
    char filename[1][CMLEN];       /* filename specified by the user         */
    char tmpstr[CMLEN];            /* temporary string                       */
    char tmpstr22[CMLEN];          /* temporary string                       */
    char tmpstr3[1][CMLEN];        /* temporary string                       */

    /* initialization 
       -------------- */
    c_inlas(&parblk,&std_out);

    /* Get the ddr name
       ------------------- */
    mode = IREAD;
    if (c_getnam(&parblk, "IN", &zero, hname, "ddr", &mode) != E_SUCC)
        c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

    /* Get the image name
       --------------------- */
    if (c_getnam(&parblk, "IN", &zero, tmpstr, "img", &mode) != E_SUCC)
        c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

    mode = IUPDATE;

    strcpy(prtfile,"ddrcheck");
    status = c_setprt(&parblk,&print[TERM],&print[PFILE],&print[LP],prtfile,
            filename[0]);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

    /* open the print file (if necessary)
       ----------------------------------*/
    if ( (print[PFILE]) || (print[LP]) )
        fp = fopen(prtfile,"w");

    /* get records 1 and 2 of DDR file
       -------------------------------*/
    status = c_getddr(hname,&ddr);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

    temp_units = c_string2unit(ddr.proj_units);
    if (temp_units == E_FAIL)
    {
        c_lasmsg(FALSE, "ddrcheck-warn", "Projection units %s are invalid",
                 ddr.proj_units);
    }

    /* translate coordinates into usable coordinates (switch from DMS)
       ---------------------------------------------------------------*/
    if (ddr.valid[4] == 1 && ddr.valid[6] == 1 && temp_units == 5)
    {
        c_decdeg(&ddr.upleft[1], "DMS", "LON");
        c_decdeg(&ddr.upleft[0], "DMS", "LAT");
        c_decdeg(&ddr.loleft[1], "DMS", "LON");
        c_decdeg(&ddr.loleft[0], "DMS", "LAT");
        c_decdeg(&ddr.upright[1], "DMS", "LON");
        c_decdeg(&ddr.upright[0], "DMS", "LAT");
        c_decdeg(&ddr.loright[1], "DMS", "LON");
        c_decdeg(&ddr.loright[0], "DMS", "LAT");
        c_decdeg(&ddr.pdist_x,"DMS","LON");
        c_decdeg(&ddr.pdist_y,"DMS","LAT");
    }

    /* read stats from image file
       --------------------------*/
    stat(tmpstr,&stats);

    /* check the size of the actual image file against calculated size
       ---------------------------------------------------------------*/
    sprintf(tmpstr,"Checking image file size...");
    switch (ddr.dtype)
    {
        case 1:     /* byte */ 
            tmpnum = 1;
            break;
        case 2:     /* i*2  */
            tmpnum = 2;
            break;
        case 3:     /* long */
        case 4:     /* real */
            tmpnum = 4;
            break;
    }
    if (stats.st_size == (long long)ddr.ns*ddr.nl*ddr.nbands*tmpnum)
        strcat(tmpstr,"...match\n");
    else
    {
        strcat(tmpstr,
             "...no match\n***  Computed and actual image sizes differ. ***\n");
        sprintf(tmpstr22,"Computed : %lld, ",
                (long long)ddr.ns*ddr.nl*ddr.nbands*tmpnum);
        strcat(tmpstr,tmpstr22);
        sprintf(tmpstr22,"Actual : %lld\n",(long long) stats.st_size);
        strcat(tmpstr,tmpstr22);
    }
    printout(tmpstr,print,fp);

    /* check sample size for reasonableness
       ------------------------------------*/
    if ((ddr.valid[5] == NOTVALID) || (ddr.pdist_x == 0) || (ddr.pdist_y == 0))
    {
        ddr.valid[5] = 0;
        sprintf(tmpstr,
                "*** Pixel size is either invalid or set to zero. ***\n");
        printout(tmpstr,print,fp);
    }

    /* check number of samples for reasonableness
       ------------------------------------------*/
    sprintf(tmpstr,"Checking number of samples...");
    if ((ddr.valid[6]) && (ddr.valid[5]))
    {
        /* using corner coordinates and sample size
           ----------------------------------------*/
        calcdist(&ddr.upleft[1],&ddr.upleft[0],
                &ddr.upright[1],&ddr.upright[0],&top_dist);

        top_dist += ddr.pdist_x;

        temp_size = (long)(top_dist / ddr.pdist_x + 0.5);
    }
    else
    {
        /* using image size against number of lines, and pixel size from ddr
           -----------------------------------------------------------------*/
        temp_size = (long)(stats.st_size/((long long)ddr.nl*ddr.nbands*tmpnum));
    }
    if (ddr.ns == temp_size)
        strcat(tmpstr,".match\n");
    else
    { 
        strcat(tmpstr,".no match\n");
        strcat(tmpstr,
               "*** Computed and actual number of samples differ. ***\n");
        sprintf(tmpstr22,"Reported : %d, ", ddr.ns);
        strcat(tmpstr,tmpstr22);
        sprintf(tmpstr22,"Computed : %d\n", temp_size);
        strcat(tmpstr,tmpstr22);
    }
    printout(tmpstr,print,fp);

    /* check number of lines for reasonability
       ----------------------------------------*/
    sprintf(tmpstr,"Checking number of lines.....");
    if ((ddr.valid[6]) && (ddr.valid[5]))
    {
        /* using corner coordinates and line size
           --------------------------------------*/
        calcdist(&ddr.upleft[1],&ddr.upleft[0],
                &ddr.loleft[1],&ddr.loleft[0], &right_dist);
        right_dist += ddr.pdist_y;
        temp_size = (long)(right_dist / ddr.pdist_y + 0.5);
    }
    else
    {
        /* using image size against number of samples, and pixel size from ddr
           -------------------------------------------------------------------*/
        temp_size = (long)(stats.st_size/((long long)ddr.ns*ddr.nbands*tmpnum));
    }
    if (ddr.nl == temp_size)
        strcat(tmpstr,".match\n");
    else
    {
        strcat(tmpstr,".no match\n");
        strcat(tmpstr,"*** Computed and actual number of lines differ. ***\n");
        sprintf(tmpstr22,"Reported : %d, Computed : %d\n", ddr.nl, temp_size);
        strcat(tmpstr,tmpstr22);
    }
    printout(tmpstr,print,fp); 

    /* check number of bands for reasonability
       ---------------------------------------*/
    sprintf(tmpstr,"Checking number of bands...");
    stat(hname,&stats);

    /* the ddr contains a record for each band (199 bytes long) and
       also records one and two which, combined, are 399 bytes long.
       ------------------------------------------------------------ */
    if (ddr.nbands == (stats.st_size-399)/199)
        strcat(tmpstr,"...match\n");
    else
    {
        strcat(tmpstr,
            "...no match\n***  Reported and computed # of bands differ. ***\n");
        sprintf(tmpstr22,"Reported : %d, Computed (from ddr): %lld\n", 
                ddr.nbands, (long long) ((stats.st_size-399)/199));
        strcat(tmpstr,tmpstr22);

    }
    printout(tmpstr,print,fp);

    tmpnum = 11;

    if (ddr.valid[1] == 1)
    {
        /* Convert the projection units to a number for use with c_trans
           ------------------------------------------------------------ */
        temp_units = c_string2unit(ddr.proj_units);
        if (strcmp(ddr.proj_units,"dms") == 0)
            temp_units --;

        /* Set up the rest of the parameters for c_trans.
           --------------------------------------------- */
        outddr.proj_code = GEO;
        outddr.zone_code = ddr.zone_code;
        out_units = 5;
        outddr.datum_code = ddr.datum_code;
        prtprm[GEO_TERM] = FALSE;
        prtprm[GEO_FILE] = FALSE;
        for (i = 0;i<15;i++)
        {
            outddr.proj_coef[i] = ddr.proj_coef[i];
        }
        /* Determine the approximate scene center
           -------------------------------------- */
        outcoord[1] = (ddr.upleft[1]+ddr.loright[1]) / 2;
        outcoord[0] = (ddr.upleft[0]+ddr.loright[0]) / 2;
        status =  c_transinit(&ddr.proj_code, &temp_units, &ddr.zone_code,
                &ddr.datum_code, ddr.proj_coef, &outddr.proj_code,
                &out_units, &outddr.zone_code, &outddr.datum_code,
                outddr.proj_coef, prtprm, "");
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);
        if (c_trans(&ddr.proj_code,&temp_units,
                &outddr.proj_code,&out_units, 
                &outcoord[1], &outcoord[0],
                &outddr.upleft[1], &outddr.upleft[0])
                != E_SUCC)
            c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

        /* Convert the packed degree/minute/second coordinates to
           degrees, minutes, and seconds 
           ------------------------------------------------------*/
        pdms2dms(&outddr.upleft[1],&lon.degree,&lon.minute,&lon.sec);
        pdms2dms(&outddr.upleft[0],&lat.degree,&lat.minute,&lat.sec);

        if (c_trans(&ddr.proj_code,&temp_units,
                &outddr.proj_code,&out_units, 
                &ddr.upleft[1], &ddr.upleft[0],
                &outddr.upleft[1], &outddr.upleft[0]) != E_SUCC)
            c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

        if (c_trans(&ddr.proj_code,&temp_units,
                &outddr.proj_code,&out_units, 
                &ddr.upright[1], &ddr.upright[0],
                &outddr.upright[1], &outddr.upright[0]) != E_SUCC)
            c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

        if (c_trans(&ddr.proj_code,&temp_units,
                &outddr.proj_code,&out_units, 
                &ddr.loleft[1], &ddr.loleft[0],
                &outddr.loleft[1], &outddr.loleft[0]) != E_SUCC)
            c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

        if (c_trans(&ddr.proj_code,&temp_units,
                &outddr.proj_code,&out_units, 
                &ddr.loright[1], &ddr.loright[0],
                &outddr.loright[1], &outddr.loright[0]) != E_SUCC)
            c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

        /* Compute the zone
           ---------------- */
        outddr.zone_code = (lon.degree + 180) / 6 + 1;
        if (lat.degree < 0)
            outddr.zone_code *= -1; 
        if (ddr.proj_code == UTM)
        {
            sprintf(tmpstr,"Checking zone..............");

            if (ddr.zone_code == outddr.zone_code)
                strcat(tmpstr,"...match\n");
            else
            {
                strcat(tmpstr,
                 "...no match\n***  Reported and computed zones differ. ***\n");

                /* display the reported and computed zones
                   --------------------------------------- */
                sprintf(tmpstr22,"Reported : %d, Computed : %d\n", 
                        ddr.zone_code, outddr.zone_code);
                strcat(tmpstr,tmpstr22);
                printout(tmpstr,print,fp);

                /* Also report the coordinates of the center point and all of
                   the corners
                   ---------------------------------------------------------- */
                sprintf(tmpstr,
                        "    Center point     : %+04d %02d %013.10f, "
                        "%+04d %02d %013.10f\n", lat.degree, lat.minute, 
                        lat.sec, lon.degree, lon.minute, lon.sec);

                pdms2dms(&outddr.upleft[1],&lon.degree,&lon.minute,&lon.sec);
                pdms2dms(&outddr.upleft[0],&lat.degree,&lat.minute,&lat.sec);
                sprintf(tmpstr22,
                        "    Upper left point : %+04d %02d %013.10f, "
                        "%+04d %02d %013.10f\n", lat.degree, lat.minute, 
                        lat.sec, lon.degree, lon.minute, lon.sec);
                strcat(tmpstr,tmpstr22);
                pdms2dms(&outddr.upright[1],&lon.degree,&lon.minute,&lon.sec);
                pdms2dms(&outddr.upright[0],&lat.degree,&lat.minute,&lat.sec);
                sprintf(tmpstr22,
                        "    Upper right point: %+04d %02d %013.10f, "
                        "%+04d %02d %013.10f\n",
                        lat.degree, lat.minute, lat.sec, lon.degree, 
                        lon.minute, lon.sec); 
                strcat(tmpstr,tmpstr22);
                pdms2dms(&outddr.loleft[1],&lon.degree,&lon.minute,&lon.sec);
                pdms2dms(&outddr.loleft[0],&lat.degree,&lat.minute,&lat.sec);
                sprintf(tmpstr22,
                        "    Lower left point : %+04d %02d %013.10f, "
                        "%+04d %02d %013.10f\n", lat.degree, lat.minute, 
                        lat.sec, lon.degree, lon.minute, lon.sec);
                strcat(tmpstr,tmpstr22);
                pdms2dms(&outddr.loright[1],&lon.degree,&lon.minute,&lon.sec);
                pdms2dms(&outddr.loright[0],&lat.degree,&lat.minute,&lat.sec);
                sprintf(tmpstr22,
                        "    Lower right point: %+04d %02d %013.10f, "
                        "%+04d %02d %013.10f\n", lat.degree, lat.minute, 
                        lat.sec, lon.degree, lon.minute, lon.sec);
                strcat(tmpstr,tmpstr22);
            }
            printout(tmpstr,print,fp);
        } 
    }

    printout("Reporting valid flag status:\n",print,fp); 

    sprintf(tmpstr,"   Projection code.........flag set to "); 
    switch  (ddr.valid[0])
    {
        case NOTVALID: 
            strcat(tmpstr, "NOT VALID\n"); 
            break;
        case VALID   : 
        case UNKNOWN : 
            if (ddr.valid[0] == 1)
                strcat(tmpstr, "VALID\n");
            else
                strcat(tmpstr, "UNKNOWN\n");
            printout(tmpstr,print,fp);

            /* Report the projection code from the DDR
               --------------------------------------- */
            sprintf(tmpstr, "      [%2d] ",ddr.proj_code);
            getproj(&ddr.proj_code, tmpstr22, ddr.proj_coef);
            strcat(tmpstr,tmpstr22);
            break;
    }
    printout(tmpstr,print,fp);

    sprintf(tmpstr,"   Zone code...............flag set to ");
    switch  (ddr.valid[1])
    {
        case NOTVALID: strcat(tmpstr,"NOT VALID\n"); break;
        case VALID   :
        case UNKNOWN : if (ddr.valid[1] == 1)
                           strcat(tmpstr,"VALID\n");
                       else
                           strcat(tmpstr,"UNKNOWN\n");
                       printout(tmpstr,print,fp); 

                       /* Report the zone code only if the projection
                          is Universal Transverse Mercator or 
                          State Plane Coordinate Systems 
                          ------------------------------------------- */
                       if ((ddr.proj_code == UTM) || (ddr.proj_code == SPCS))
                           sprintf(tmpstr,"      %2d\n",ddr.zone_code);
                       else
                           sprintf(tmpstr,
                                   "      (not used for this projection)\n");
                       break;
    }
    printout(tmpstr,print,fp);

    sprintf(tmpstr,"   Datum code..............flag set to ");
    switch  (ddr.valid[2])
    {
        case NOTVALID: strcat(tmpstr,"NOT VALID\n"); break;
        case VALID   :
        case UNKNOWN : if (ddr.valid[2] == 1)
                           strcat(tmpstr,"VALID\n");
                       else
                           strcat(tmpstr,"UNKNOWN\n");
                       printout(tmpstr,print,fp);

                       /* Report the datum code from the DDR
                          ---------------------------------- */
                       sprintf(tmpstr,"      [%2d] ",ddr.datum_code);
                       getdatum(&ddr.datum_code, tmpstr22);
                       strcat(tmpstr,tmpstr22);
                       break;
    }
    printout(tmpstr,print,fp);

    sprintf(tmpstr,"   Projection parameters...flag set to ");
    switch  (ddr.valid[3])
    {
        case NOTVALID: strcat(tmpstr,"NOT VALID\n"); 
                       printout(tmpstr,print,fp);
                       break;
        case VALID   :
        case UNKNOWN : if (ddr.valid[3] == 1)
                           strcat(tmpstr,"VALID\n");
                       else
                           strcat(tmpstr,"UNKNOWN\n");
                       printout(tmpstr,print,fp);

                       /* Report the appropriate projection parameters
                          -------------------------------------------- */
                       getprojparm(&ddr.proj_code,&ddr.zone_code,tmpstr22,
                               ddr.proj_coef,print, fp);
                       strcat(tmpstr,tmpstr22);
    } 

    sprintf(tmpstr,"   Projection units........flag set to ");
    switch  (ddr.valid[4])
    {
        case NOTVALID: 
            strcat(tmpstr,"NOT VALID\n"); 
            printout(tmpstr,print,fp);
            break;
        case VALID   : 
        case UNKNOWN : 
            if (ddr.valid[4] == 1)
                strcat(tmpstr,"VALID\n");
            else
                strcat(tmpstr,"UNKNOWN\n");
            printout(tmpstr,print,fp);

            /* Report the projection units from the DDR
               ---------------------------------------- */
            sprintf(tmpstr,"      %s\n",ddr.proj_units);
            printout(tmpstr,print,fp);
    }

    sprintf(tmpstr,"   Projection distance.....flag set to ");
    switch  (ddr.valid[5])
    {
        case NOTVALID:
            strcat(tmpstr,"NOT VALID\n");
            printout(tmpstr,print,fp);
            break;
        case VALID   :
        case UNKNOWN :
            if (ddr.valid[5] == 1)
                strcat(tmpstr,"VALID\n");
            else
                strcat(tmpstr,"UNKNOWN\n");
            printout(tmpstr,print,fp);

            /* Report the number of samples and lines
               -------------------------------------- */
            sprintf(tmpstr,"      %-30s: %10d\n",
                    "Number of lines", ddr.nl);
            printout(tmpstr,print,fp);
            sprintf(tmpstr,"      %-30s: %10d\n",
                    "Number of samples", ddr.ns);
            printout(tmpstr,print,fp);

            /* Report the projection distance from the DDR
               ------------------------------------------- */
            sprintf(tmpstr,"      %-30s: %18.7f\n",
                    "Projection distance/pixel (y)",ddr.pdist_y);
            printout(tmpstr,print,fp);
            sprintf(tmpstr,"      %-30s: %18.7f\n",
                    "Projection distance/pixel (x)",ddr.pdist_x);
            printout(tmpstr,print,fp);

            /* Calculate and report the total projection distance
               -------------------------------------------------- */
            sprintf(tmpstr,"      %-30s: %18.7f\n",
                    "Total projection distance (y)", ddr.pdist_y*ddr.nl);
            printout(tmpstr,print,fp);
            sprintf(tmpstr,"      %-30s: %18.7f\n",
                    "Total projection distance (x)", ddr.pdist_x*ddr.ns);
            printout(tmpstr,print,fp);
            break;
    }
    sprintf(tmpstr,"   Corner coordinates......flag set to ");
    switch  (ddr.valid[6])
    {
        case NOTVALID:
            strcat(tmpstr,"NOT VALID\n");
            printout(tmpstr,print,fp);
            break;
        case VALID   :
        case UNKNOWN :
            if (ddr.valid[6] == 1)
                strcat(tmpstr,"VALID\n");
            else
                strcat(tmpstr,"UNKNOWN\n");
            printout(tmpstr,print,fp);

            /* Display the corner coordinates
               ------------------------------ */
            if (ddr.valid[4] == VALID)
            {
                if (strcmp(ddr.proj_units,"meters") == 0)
                {
                    sprintf(tmpstr,"      %-30s: %18.7f  %18.7f\n",
                            "Upper left", ddr.upleft[0],ddr.upleft[1]);
                    printout(tmpstr,print,fp);

                    sprintf(tmpstr,"      %-30s: %18.7f  %18.7f\n",
                            "Lower left ", ddr.loleft[0], ddr.loleft[1]);
                    printout(tmpstr,print,fp);

                    sprintf(tmpstr,"      %-30s: %18.7f  %18.7f\n",
                            "Upper right ", ddr.upright[0],ddr.upright[1]);
                    printout(tmpstr,print,fp);

                    sprintf(tmpstr,"      %-30s: %18.7f  %18.7f\n",
                            "Lower right ", ddr.loright[0],ddr.loright[1]);
                    printout(tmpstr,print,fp);
                }
                else if ((strcmp(ddr.proj_units,"degrees") == 0) || 
                        (strcmp(ddr.proj_units,"dms") == 0))
                {
                    pdms2dms(&outddr.upleft[0],&degree,&minute,&sec);
                    sprintf(tmpstr,"      %-30s: %+04d %02d %010.7f, ",
                            "Upper left", degree,minute,sec);
                    pdms2dms(&outddr.upleft[1],&degree,&minute,&sec);
                    sprintf(tmpstr22,"%+04d %02d %010.7f\n",degree,minute,sec);
                    strcat(tmpstr,tmpstr22);
                    printout(tmpstr,print,fp);

                    pdms2dms(&outddr.loleft[0],&degree,&minute,&sec);
                    sprintf(tmpstr,"      %-30s: %+04d %02d %010.7f, ",
                            "Lower left", degree,minute,sec);
                    pdms2dms(&outddr.loleft[1],&degree,&minute,&sec);
                    sprintf(tmpstr22,"%+04d %02d %010.7f\n",degree,minute,sec);
                    strcat(tmpstr,tmpstr22);
                    printout(tmpstr,print,fp);

                    pdms2dms(&outddr.upright[0],&degree,&minute,&sec);
                    sprintf(tmpstr,"      %-30s: %+04d %02d %010.7f, ",
                            "Upper Right", degree,minute,sec);
                    pdms2dms(&outddr.upright[1],&degree,&minute,&sec);
                    sprintf(tmpstr22,"%+04d %02d %010.7f\n",degree,minute,sec);
                    strcat(tmpstr,tmpstr22);
                    printout(tmpstr,print,fp);

                    pdms2dms(&outddr.loright[0],&degree,&minute,&sec);
                    sprintf(tmpstr,"      %-30s: %+04d %02d %010.7f, ",
                            "Lower Right", degree,minute,sec);
                    pdms2dms(&outddr.loright[1],&degree,&minute,&sec);
                    sprintf(tmpstr22,"%+04d %02d %010.7f\n",degree,minute,sec);
                    strcat(tmpstr,tmpstr22);
                    printout(tmpstr,print,fp);
                }
            } 
            /* Calculate the distance for each side of the image
               ------------------------------------------------- */
            out_units = c_string2unit("meters");
            if (temp_units == out_units)
            {
                outddr = ddr;
            }
            else 
            {
                out_units = c_string2unit("dms");
                if (temp_units == out_units)
                {
                    outddr = ddr;
                    for (i=0;i<2;i++)
                    {
                        if (outddr.upleft[i] < 0)
                        {
                            c_decdeg(&outddr.upleft[i],"DMS","CIR");
                            c_decdeg(&outddr.loleft[i],"DMS","CIR");
                            c_decdeg(&outddr.upright[i],"DMS","CIR");
                            c_decdeg(&outddr.loright[i],"DMS","CIR");
                        }
                    }
                }
                else
                {
                    out_units = c_string2unit("degrees");
                    if (temp_units == out_units)
                    {
                        outddr = ddr;
                    }
                }
            }
            calcdist(&outddr.upleft[0],&outddr.upleft[1],
                    &outddr.loleft[0],&outddr.loleft[1],&left_dist);
            calcdist(&outddr.upright[0],&outddr.upright[1],
                    &outddr.loright[0],&outddr.loright[1],&right_dist);

            /* Calculate the distance for the top and bottom of the image
               ---------------------------------------------------------- */
            calcdist(&outddr.upleft[0],&outddr.upleft[1],
                    &outddr.upright[0],&outddr.upright[1],&top_dist);
            calcdist(&outddr.loleft[0],&outddr.loleft[1],
                    &outddr.loright[0],&outddr.loright[1],&bot_dist);

            if (ddr.valid[5] == VALID)
            {
                left_dist += ddr.pdist_y;
                right_dist += ddr.pdist_y;
                top_dist += ddr.pdist_x;
                bot_dist += ddr.pdist_x;
            }

            /* Opposite sides of the image should be the same size
               --------------------------------------------------- */
            if (left_dist != right_dist)
            {
                /* if they are not the same size, report to the user
                   ------------------------------------------------- */
                sprintf(tmpstr,
                        "*** Calculated left and right lengths differ by "
                        "%- 1.10E. ***\n",
                        fabs(left_dist - right_dist));
                printout(tmpstr,print,fp);
            }
            sprintf(tmpstr,"      %-30s: %18.7f\n",
                    "Total distance (left)",left_dist);
            printout(tmpstr,print,fp);
            sprintf(tmpstr,"      %-30s: %18.7f\n",
                    "Total distance (right)",right_dist);
            printout(tmpstr,print,fp);

            /* Opposite sides should be the same size
               -------------------------------------- */
            if (top_dist != bot_dist)
            {
                /* if they are not the same size, report to the user
                   ------------------------------------------------- */
                sprintf(tmpstr,
                        "*** Calculated top and bottom lengths differ by "
                        "%- 1.10E. ***\n", 
                        fabs(top_dist - bot_dist));
                printout(tmpstr,print,fp);
            }
            sprintf(tmpstr,
                   "      %-30s: %18.7f\n","Total distance (top) ",top_dist);
            printout(tmpstr,print,fp);
            sprintf(tmpstr,
                   "      %-30s: %18.7f\n","Total distance (bottom) ",bot_dist);
            printout(tmpstr,print,fp);

            if (ddr.valid[5] == VALID)
            {

                left_dist -= ddr.pdist_y;
                right_dist -= ddr.pdist_y;
                top_dist -= ddr.pdist_x;
                bot_dist -= ddr.pdist_x;
            }

            /* Calculate the angles of the sides from vertical
               ----------------------------------------------- */
            tmpdbl = (outddr.upleft[0]-outddr.loleft[0])/
                (left_dist);
            if (tmpdbl >= 1)
                tmpdbl2 = 0;
            else
                tmpdbl2 = (acos(tmpdbl) * R2D);
            if (c_degdms(&tmpdbl2,&leftY,"DEG","LON") != E_SUCC)
                c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);
            tmpdbl = (outddr.upright[0]-outddr.loright[0])/
                (right_dist);
            if (tmpdbl >= 1)
                tmpdbl2 = 0;
            else
                tmpdbl2 = acos(tmpdbl) * R2D;
            if (c_degdms(&tmpdbl2,&rightY,"DEG","LON") != E_SUCC)
                c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);


            /* Both sides should be at the same angle from vertical
               ---------------------------------------------------- */
            if (leftY != rightY)
            {
                /* if they are not the same, report to the user
                   -------------------------------------------- */
                c_decdeg (&leftY,"DMS","LAT");
                c_decdeg (&rightY,"DMS","LAT");
                tmpdbl = leftY - rightY;
                if (tmpdbl < 0)
                    tmpdbl *= -1;

                if (c_degdms(&tmpdbl,&tmpdbl2,"DEG","LON") != E_SUCC)
                    c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);
                pdms2dms(&tmpdbl2,&degree,&minute,&sec);
                sprintf(tmpstr,"%s %+04d %02d %013.10f. ***\n",
                        "*** Calculated left and right angles differ by",
                        degree,minute,sec);
                printout(tmpstr,print,fp);
            }
            pdms2dms(&leftY,&degree,&minute,&sec);
            sprintf(tmpstr,"      %-30s: %+04d %02d %013.10f\n",
                    "Left angle (from vertical)",degree, minute, sec);
            printout(tmpstr,print,fp);
            pdms2dms(&rightY,&degree,&minute,&sec);
            sprintf(tmpstr,"      %-30s: %+04d %02d %013.10f\n",
                    "Right angle (from vertical)",degree, minute, sec);
            printout(tmpstr,print,fp);

            /* Calculate the angles of the top and bottom from horizontal
               ---------------------------------------------------------- */
            tmpdbl = (outddr.upright[0]-outddr.upleft[0])/
                (top_dist);
            if (tmpdbl >= 1)
                tmpdbl2 = 0;
            else
                tmpdbl2 = (acos(tmpdbl) * R2D);
            if (c_degdms(&tmpdbl,&topX,"DEG","LON") != E_SUCC)
                c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);
            tmpdbl = (outddr.loright[0]-outddr.loleft[0])/
                (top_dist);
            if (tmpdbl >= 1)
                tmpdbl2 = 0;
            else
                tmpdbl2 = (acos(tmpdbl) * R2D);
            if (c_degdms(&tmpdbl,&botX,"DEG","LON") != E_SUCC)
                c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);

            /* Both top and bottom should be at the same angle from horizontal
               -------------------------------------------------------------- */
            if (topX != botX)
            {
                /* if they are not the same, report to the user
                   -------------------------------------------- */
                c_decdeg(&topX,"DMS","LON");
                c_decdeg(&botX,"DMS","LON");
                tmpdbl = topX - botX;
                if (tmpdbl < 0)
                    tmpdbl *= -1.0;
                if (c_degdms(&tmpdbl,&tmpdbl2,"DEG","LON") != E_SUCC)
                    c_errmsg("Fatal error encountered","ddrcheck-fatal",&fatal);
                pdms2dms(&tmpdbl2,&degree,&minute,&sec);
                sprintf(tmpstr, "*** %s %+04d %02d %013.10f. ***\n",
                        "Calculated top and bottom angles differ by",
                        degree, minute, sec);
                printout(tmpstr,print,fp);
            }
            pdms2dms(&topX,&degree,&minute,&sec);
            sprintf(tmpstr,"      %-30s: %+04d %02d %013.10f\n",
                    "Top angle (from horizontal)",degree, minute, sec);
            printout(tmpstr,print,fp);
            pdms2dms(&botX,&degree,&minute,&sec);
            sprintf(tmpstr,"      %-30s: %+04d %02d %013.10f\n",
                    "Bottom angle (from horizontal)",degree, minute, sec);
            printout(tmpstr,print,fp);

            /* Each of the corners should be a right angle
               ------------------------------------------- */
            if ((leftY != topX) || (leftY != botX) || 
                    (rightY != topX) || (rightY != botX))
            {
                /* At least one of the corners is not exactly a right angle
                   -------------------------------------------------------- */
                sprintf(tmpstr,"*** Corners are not square! ***\n");
                printout(tmpstr,print,fp);
            }
            break;
    }
    sprintf(tmpstr,"   Line/sample increments..flag set to ");
    switch  (ddr.valid[7])
    {
        case NOTVALID:
            strcat(tmpstr,"NOT VALID\n");
            printout(tmpstr,print,fp);
            break;
        case VALID   :
        case UNKNOWN :
            if (ddr.valid[7] == 1)
                strcat(tmpstr,"VALID\n");
            else
                strcat(tmpstr,"UNKNOWN\n");

            /* Simply report the line and sample increments from the DDR
               --------------------------------------------------------- */
            printout(tmpstr,print,fp);
            sprintf(tmpstr,"      %-30s: %18.7f\n","Line increment",
                    ddr.line_inc);
            printout(tmpstr,print,fp);
            sprintf(tmpstr,"      %-30s: %18.7f\n","Sample increment",
                    ddr.sample_inc);
            printout(tmpstr,print,fp);
            break;
    }

    /* close the print file (if necessary)
       --------------------------------------*/
    if ( (print[PFILE]) || (print[LP]) )
        fclose(fp);

    /* if the user requested a file, copy the temporary file to that file
       ------------------------------------------------------------------ */ 
    if (print[PFILE])
    {
        sprintf(tmpstr,"cp %s %s",prtfile,filename[0]);
        system(tmpstr);
    } 

    /* inform user of successful completion
       ---------------------------------------*/
    if (print[PFILE])
        c_complt("DDRCHECK",filename,&one);
    else
    {
        strcpy(tmpstr3[0], "\0");
        c_complt("DDRCHECK",tmpstr3,&zero);
    }

    return EXIT_SUCCESS;

}
