/*****************************************************************************
NAME: GETPROJPARM 

PURPOSE:  Determines the correct interpretation of the projection parameters
          based on the projection used.  Outputs the information to the proper
          device(s).

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     12/97 B.Olson           C    Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/
#include <string.h>
#include "proj.h"
#include "protos.h"

void getprojparm
(
    long *proj_code,                 /* Projection code from DDR             */
    long *zone_code,                 /* Zone code from DDR                   */
    char *string,                    /* Formatted data to be sent back       */
    double proj_coef[15],            /* Projection coefficients from DDR     */
    long print[3],                   /* output destination                   */
    FILE *fp                         /* file pointer to print file           */
)

{
    struct COORDS
    {
        long degree;                 /* degree portion of coordinate         */
        long minute;                 /* minute portion of coordinate         */
        double sec;                   /* second portion of coordinate        */
    };

    struct COORDS lon;             /* expanded data for longitude            */

    long degree = 0;               /* degree portion of coordinate           */
    long minute = 0;               /* minute portion of coordinate           */

    double sec = 0.0;              /* second portion of coordinate           */
    double def_rad = 6370997.0;    /* Default radius of projection           */
    double tmp_axis = 6378206.4;   /* Default axis                           */
    char string22[255];            /* temporary string                       */

    string = "";

    /* Determine and display the first projection parameter
       ---------------------------------------------------- */
    switch (*proj_code)
    {
        case UTM   :
            if ((proj_coef[0] == 0.0) && (proj_coef[1] == 0.0))
            {
                /* There is no longitude and latitude for this image
                   ------------------------------------------------- */ 
                sprintf(string,"      Using zone code for location\n");
            }
            else
            {
                /* Convert longitude from packed dms to degrees, minutes,
                   and seconds
                   -------------------------------------------------------- */ 
                pdms2dms(&proj_coef[0],&lon.degree,&lon.minute, &lon.sec);
                sprintf(string,
                        "      [ 1] %-40s: %+04d %02d %013.10f\n","Longitude",
                        lon.degree,lon.minute,lon.sec);
            }
            printout(string,print,fp);
            break;
        case ALBERS:
        case LAMCC :
        case MERCAT:
        case PS    :
        case POLYC :
        case EQUIDC:
        case TM    :
        case HOM   :
        case SOM   :
        case ALASKA:
            if (proj_coef[0] != 0)
            {
                /* There is a value for the axis, so use it.
                   Otherwise leave tmp_axis at the original value.
                   ----------------------------------------------- */
                tmp_axis = proj_coef[0];
            }
            sprintf(string,
                    "      [ 1] %-40s: % 18.7f meters\n",
                    "Semi-major axis of ellipsoid",tmp_axis);
            printout(string,print,fp);
            break;
        case STEREO:
        case LAMAZ :
        case AZMEQD:
        case GNOMON:
        case ORTHO :
        case GVNSP :
        case SNSOID:
        case EQRECT:
        case MILLER:
        case VGRINT:
        case ROBIN :
        case GOOD  :
        case MOLL  :
        case IMOLL :
        case HAMMER:
        case WAGIV :
        case WAGVII:
        case OBEQA :
            sprintf(string,"      [ 1] %-40s: ",
                    "Radius of reference sphere");
            if (proj_coef[0] == 0)
            {
                /* No Radius given, so use the default value
                   ----------------------------------------- */
                sprintf(string22,"%18.7f",def_rad);
            }
            else
                sprintf(string22,"%18.7f",proj_coef[0]);
            strcat(string,string22);
            strcat(string," meters\n");
            printout(string,print,fp);
            break;
    }

    switch (*proj_code)
    {
        case UTM   :
            if (proj_coef[1] == 0.0)
            {
                /* There is no longitude and latitude for this image
                   ------------------------------------------------- */
                if (zone_code == 0)
                    printout("***Error-require either zone or latitude.\n",
                            print,fp);
            }
            else
            {
                pdms2dms(&proj_coef[1],&degree,&minute,&sec);
                sprintf(string,"      [ 2] %-40s: %+04d %02d %013.10f\n",
                        "Latitude",degree,minute,sec);
                printout(string,print,fp);
            }
            break;
        case ALBERS:
        case LAMCC :
        case MERCAT:
        case PS    :
        case POLYC :
        case EQUIDC:
        case TM    :
        case HOM   :
        case SOM   :
        case ALASKA:
            if (proj_coef[1] > 1)
                sprintf(string,
                        "      [ 2] %-40s: ","Semi-minor axis of ellipsoid");
            else if (proj_coef[1] > 0)
                sprintf(string,
                        "      [ 2] %-40s: ",
                        "Eccentricity squared of ellipsoid");
            else if (proj_coef[1] == 0)
                sprintf(string,"      [ 2] %-40s","Spherical form");
            if (proj_coef[1] == 0)
            {
                /* In spherical form there are no numbers for ellipsoid
                   ---------------------------------------------------- */
                sprintf(string22,"\n");
            }
            else
                sprintf(string22,"% 18.7f meters\n",proj_coef[1]);
            strcat(string,string22);
            printout(string,print,fp);
            break;
    }
    switch (*proj_code)
    {
        case ALBERS:
        case LAMCC :
            pdms2dms(&proj_coef[2],&degree,&minute,&sec);
            sprintf(string,"      [ 3] %-40s: %+04d %02d %013.10f\n",
                    "Latitude of 1st std parallel", degree,minute,sec);
            printout(string,print,fp);
            break;
        case EQUIDC:
            pdms2dms(&proj_coef[2],&degree,&minute,&sec);
            if (proj_coef[8] == 0)
            {
                /* Equidistant Conic A
                   ------------------- */
                sprintf(string,"      [ 3] %-40s: %+04d %02d %013.10f\n",
                        "Latitude of standard parallel", degree,minute,sec);
            }
            else
            {
                /*  Equidistant Conic B
                    ------------------- */
                sprintf(string,"      [ 3] %-40s: %+04d %02d %013.10f\n",
                        "Latitude of 1st std parallel", degree,minute,sec);
            }
            printout(string,print,fp);
            break;
        case TM    :
            sprintf(string,"      [ 3] %-40s: %18.7f\n",
                    "Scale factor at central meridian", proj_coef[2]);
            printout(string,print,fp);
            break;
        case GVNSP :
            sprintf(string,"      [ 3] %-40s: %18.7f\n",
                    "Height of perspective point",proj_coef[2]);
            printout(string,print,fp);
            break;
        case HOM   :
            sprintf(string,"      [ 3] %-40s: %18.7f\n",
                    "Scale factor at center of projection",proj_coef[2]);
            printout(string,print,fp);
            break;
        case SOM   :
            if (proj_coef[12] == 1)
            {
                sprintf(string,"      [ 3] %-40s: %18.7f\n",
                        "Landsat satellite number", proj_coef[2]);
                printout(string,print,fp);
            }
            break;
        case OBEQA :
            sprintf(string,"      [ 3] %-40s: %18.7f\n",
                    "Oblated Equal Area Shape parm m", proj_coef[2]);
            printout(string,print,fp);
            break;
    }
    switch (*proj_code)
    {
        case ALBERS:
        case LAMCC :
            pdms2dms(&proj_coef[3],&degree,&minute,&sec);
            sprintf(string,"      [ 4] %-40s: %+04d %02d %013.10f\n",
                    "Latitude of 2nd std parallel",degree,minute,sec);
            printout(string,print,fp);
            break;
        case EQUIDC:
            if (proj_coef[8] == 1)
            {
                /* Equidistant Conic B.  Equidistant Conic A 
                   has nothing for this projection parameter.
                   ------------------------------------------ */
                pdms2dms(&proj_coef[3],&degree,&minute,&sec);
                sprintf(string,"      [ 4] %-40s: %+04d %02d %013.10f\n",
                        "Latitude of 2nd std parallel",degree,minute,sec);
                printout(string,print,fp);
            }
            break;
        case HOM   :
            if (proj_coef[12] == 1)
            {
                /* Oblique Mercator B (Hotine).
                   ---------------------------- */
                pdms2dms(&proj_coef[3],&degree,&minute,&sec);
                sprintf(string,"      [ 4] %-40s: ",
                        "Azimuth angle east of north of center");
                sprintf(string22,"%+04d %02d %013.10f\n",degree,minute,sec);
                strcat(string,string22);
            }
            else
            {
                /* Oblique Mercator A (Hotine) has no projection parameter
                   ------------------------------------------------------- */
                string="";
            }
            printout(string,print,fp);
            break;
        case SOM   :
            if (proj_coef[12] == 0)
            {
                /* Space Oblique Mercator A
                   ------------------------ */
                pdms2dms(&proj_coef[3],&degree,&minute,&sec);
                sprintf(string,"      [ 4] %-40s: %+04d %02d %013.10f\n",
                        "Inclination of orbit",degree,minute,sec);
            }
            else
            {
                /* Space Oblique Mercator B
                   ------------------------ */
                sprintf(string,"      [ 4] %-40s: %18.7f\n",
                        "Landsat path number",proj_coef[3]);
            }
            printout(string,print,fp);
            break;
        case OBEQA :
            sprintf(string,"      [ 4] %-40s: %18.7f\n",
                    "Oblated Equal Area Shape parm n", proj_coef[3]);
            printout(string,print,fp);
            break;
    }
    switch (*proj_code)
    {
        case ALBERS:
        case LAMCC :
        case MERCAT:
        case POLYC :
        case EQUIDC:
        case TM    :
        case SNSOID:
        case EQRECT:
        case MILLER:
        case VGRINT:
        case ROBIN :
        case MOLL  :
        case HAMMER:
        case WAGIV :
        case WAGVII:
            pdms2dms(&proj_coef[4],&degree,&minute,&sec);
            sprintf(string,"      [ 5] %-40s: %+04d %02d %013.10f\n",
                    "Longitude of central meridian",degree,minute,sec);
            printout(string,print,fp);
            break;
        case PS    :
            pdms2dms(&proj_coef[4],&degree,&minute,&sec);
            sprintf(string,"      [ 5] %-40s: %+04d %02d %013.10f\n",
                    "Longitude below pole of map", degree,minute,sec);
            printout(string,print,fp);
            break;
        case STEREO:
        case LAMAZ :
        case AZMEQD:
        case GNOMON:
        case ORTHO :
        case GVNSP :
        case OBEQA :
            pdms2dms(&proj_coef[4],&degree,&minute,&sec);
            sprintf(string,"      [ 5] %-40s: %+04d %02d %013.10f\n",
                    "Longitude of center of projection", degree,minute,sec);
            printout(string,print,fp);
            break;
        case HOM   :
            if (proj_coef[12] == 1)
            {
                /* Oblique Mercator B (Hotine) 
                   --------------------------- */
                pdms2dms(&proj_coef[4],&degree,&minute,&sec);
                sprintf(string,"      [ 5] %-40s: ",
                        "Longitude of azimuth point");
                sprintf(string22,"%+04d %02d %013.10f\n",degree,minute,sec);
                strcat(string,string22);
                printout(string,print,fp);
            }
            break;
        case SOM   :
            if (proj_coef[12] == 0)
            {
                /* Space Oblique Mercator A 
                   ------------------------ */
                pdms2dms(&proj_coef[4],&degree,&minute,&sec);
                sprintf(string,"      [ 5] %-40s: ",
                        "Longitude of ascending orbit");
                sprintf(string22,"%+04d %02d %013.10f\n",degree,minute,sec);
                strcat(string,string22);
                printout(string,print,fp);
            }
            break;
    }
    switch (*proj_code)
    {
        case ALBERS:
        case LAMCC :
        case POLYC :
        case EQUIDC:
        case TM    :
        case VGRINT:
        case HOM   :
            pdms2dms(&proj_coef[5],&degree,&minute,&sec);
            sprintf(string,"      [ 6] %-40s: %+04d %02d %013.10f\n",
                    "Latitude of projection origin",degree,minute,sec);
            printout(string,print,fp);
            break;
        case MERCAT:
        case PS    :
        case EQRECT:
            pdms2dms(&proj_coef[5],&degree,&minute,&sec);
            sprintf(string,"      [ 6] %-40s: %+04d %02d %013.10f\n",
                    "Latitude of true scale", degree,minute,sec);
            printout(string,print,fp);
            break;
        case STEREO:
        case LAMAZ :
        case AZMEQD:
        case GNOMON:
        case ORTHO :
        case GVNSP :
        case OBEQA :
            pdms2dms(&proj_coef[5],&degree,&minute,&sec);
            sprintf(string,"      [ 6] %-40s: %+04d %02d %013.10f\n",
                    "Latitude of center of projection", degree,minute,sec);
            printout(string,print,fp);
            break;
    }
    switch (*proj_code)
    {
        case GEO   :
        case UTM   :
        case SPCS  :
        case GOOD  :
        case IMOLL :
            break;
        default:
            sprintf(string,
                    "      [ 7] %-40s: %18.7f\n","False easting",proj_coef[6]);
            printout(string,print,fp);
            break;
    }
    switch (*proj_code)
    {
        case GEO   :
        case UTM   :
        case SPCS  :
        case GOOD  :
        case IMOLL :
            break;
        default:
            sprintf(string,"      [ 8] %-40s: %18.7f\n",
                    "False northing",proj_coef[7]);
            printout(string,print,fp);
            break;
    }
    if ((*proj_code == HOM) && (proj_coef[12] == 0))
    {
        /* Oblique Mercator A (Hotine) 
           --------------------------- */
        pdms2dms(&proj_coef[8],&degree,&minute,&sec);
        sprintf(string,"      [ 9] %-40s: %+04d %02d %013.10f\n",
                "Long. of 1st pt on center", degree,minute,sec);
        printout(string,print,fp);
        pdms2dms(&proj_coef[9],&degree,&minute,&sec);
        sprintf(string,"      [10] %-40s: %+04d %02d %013.10f\n",
                "Lat. of 1st pt on center",degree,minute,sec);
        printout(string,print,fp);
        pdms2dms(&proj_coef[10],&degree,&minute,&sec);
        sprintf(string,"      [11] %-40s: %+04d %02d %013.10f\n",
                "Long. of 2nd pt on center",degree,minute,sec);
        printout(string,print,fp);
        pdms2dms(&proj_coef[11],&degree,&minute,&sec);
        sprintf(string,"      [12] %-40s: %+04d %02d %013.10f\n",
                "Lat. of 2nd pt on center",degree,minute,sec);
        printout(string,print,fp);
    }
    if ((*proj_code == SOM) && (proj_coef[12] == 0))
    {
        /* Space Oblique Mercator A 
           ------------------------ */
        sprintf(string,"      [ 9] %-40s: %18.7f\n",
                "Period of satellite revolution",proj_coef[8]);
        printout(string,print,fp);
        sprintf(string,"      [10] %-40s: %18.7f\n",
                "Landsat compensation ratio",proj_coef[9]);
        printout(string,print,fp);
        sprintf(string,"      [11] %-40s: %18.7f\n",
                "End of path flag",proj_coef[10]);
        printout(string,print,fp);
    }
    if (*proj_code == OBEQA)
    {
        pdms2dms(&proj_coef[8],&degree,&minute,&sec);
        sprintf(string,"      [ 9] %-40s: %+04d %02d %013.10f\n",
                "Oval rotation angle",degree, minute, sec);
        printout(string,print,fp);
    }
}
