/***************************************************************************

NAME: DEBAND

PURPOSE:	Performs filtering of an image to reduce or eliminate the 
		swathing or banding found in Landsat TM images. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1.0           5/90     T. Drackley        Original development (DESWATH2)
    2.0           1/91     D. Steinwand       Driver program for DEBAND
    3.0           10/96    C. Engebretson     Rewrite for Baseline standards

PROJECT: 	LAS

ALGORITHM DESCRIPTION:

***************************************************************************/

#include <stdlib.h>		   /* required for call to malloc()	   */
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAX_BAND 7                 /* maximum number of bands in TM scene  */


int main
(void) 
{
struct PARBLK   parblk; 	     /* TAE parameter block	            */

double		line_inc = 1;        /* line interval for upddr             */
static double	maxval[MAX_BAND+1];  /* output image maximum value          */
static double	minval[MAX_BAND+1];  /* output image minimum value          */
double		samp_inc = 1;        /* sample interval for upddr           */
long		acc;		     /* access flag for tape I/O            */
static long	bands[MAXIMG][MAXBND+1];      /* image band numbers         */
long		bndout;              /* output band number                  */
long		comb_flag = NOCOMB;  /* for upddr                           */
long		curimg;              /* the current image counter           */
long		i;                   /* a counter                           */
long		intype;              /* input data type                     */ 
struct FDESC *  infd[2];             /* input file descriptor               */
struct FDESC *  outfd[2];            /* output file descriptor              */
long            minmax;              /* minmax option flag                  */
long            msgtim;              /* message display interval            */
long		nl;                  /* number of lines in image            */
long		nmrimg;              /* number of input images              */ 
long		ns;                  /* number of samples/line in image     */
long		onecat = ONECAT;     /* for inittm                          */
long		opt[2];              /* for eopenr                          */
long		otype;               /* output image data type              */
long		status;              /* function return status code         */ 
long		totbnd;              /* total number of bands               */
long            std_out;	     /* standard output                     */
long            zero = 0;	     /* constant of zero                    */
long            one = 1;             /* constant of one                     */

static long     nbands[MAXIMG];      /* number of bands in each image       */
static long	window[MAXIMG][4];   /* window for each input image         */

extern long     curent;              /* for processing message              */
extern long     curcat;              /* for processing message              */

static char	hosin[MAXIMG][CMLEN];/* host names of input images          */
char		taeout[1][CMLEN];	     /* TAE name of output image            */
char		hosout[1][CMLEN];	     /* host name of output image           */

unsigned char *outbuf;		     /* line of output image data	    */
static unsigned char *buf;	     /* input image data buffer             */

long numbands;			     /* number of bands in the image        */
long up;			     /* offset to top of filter window      */
long low;		  	     /* offset to bottom of filter window   */
long in;			     /* offset to center of filter window   */
long lineno;			     /* current line number in image        */
long height;			     /* height of filter window             */
float tol;			     /* user-specified tolerance value      */

/* Initialize LAS
   --------------*/
c_inlas(&parblk, &std_out);

/* Get user-specified parameters and tae global variables
   ------------------------------------------------------*/
getpar(&parblk, hosin, taeout, hosout, &totbnd, bands, nbands, &nl, &ns,
        window, &nmrimg, &tol, &height, &msgtim, &minmax);

/* Allocate memory for the input and output buffers
   ------------------------------------------------*/
init_filter(ns);
outbuf = (unsigned char *)malloc(ns);
if (!outbuf)
    c_lasmsg(TRUE, "deband-alloc", "Error allocating dynamic memory");
buf = (unsigned char *)malloc(ns * (height * 2 + 1));
if (!buf)
    c_lasmsg(TRUE, "deband-alloc", "Error allocating dynamic memory");

/* Force the input image data to byte.  This will have to change if one wants
   to process images in other data types.
   --------------------------------------*/
intype = otype = EBYTE;

/* Initialize processing message and min/max arrays
   ------------------------------------------------*/
c_inittm("DEBAND","Processed","line","band"," ",&nl,&totbnd,&msgtim,&onecat);
c_inmnmx (&otype, &totbnd, minval, maxval);

/* Open output image
   -----------------*/
opt[0] = 0;
opt[1] = 0;
acc = IWRITE;
status=c_eopenr(&outfd[0],hosout[0],&totbnd,&one,&one,&nl,&ns,&acc,&otype,
                opt,&one);
if (status!=E_SUCC)
    c_lasmsg(TRUE, "deband-fatal", "Fatal error encountered");

/* Initialize output band counter
   -----------------------------*/
bndout = 0;
curent = curcat = 1;
c_startm ();
for (curimg = 0; curimg < nmrimg; curimg++)
   {

/* Open input image
   ----------------*/
   numbands = 0;
   acc = IREAD;
   status = c_eopenr(&infd[0],hosin[curimg],&numbands,&(window[curimg][SL]),
	&(window[curimg][SS]),&(window[curimg][NL]),&(window[curimg][NS]),
	&acc,&intype,&zero,&one);
   if (status != E_SUCC)
       c_lasmsg(TRUE, "deband-fatal", "Fatal error encountered");

/* Process each band
   -----------------*/
   for (i = 0; i < nbands[curimg]; i++, curcat++)
      {
      init_get_line();
      bndout++;

/* Process each line
   -----------------*/
      curent = 1;
      for (lineno = 1; lineno <= nl; lineno++, curent++)
	{
	get_line(bands[curimg][i], nl, ns, height, buf, &up, &in,
		 &low, infd, outfd);
        filter(&(buf[up]), &(buf[in]), &(buf[low]),
	       tol, nl, ns, height, lineno, outbuf);

/* Write completed line to output image
   ------------------------------------*/
        status = c_ewrite (&outfd[0], &bndout, &lineno, outbuf, &one);
        if (status != ns) 
            {
            c_eclose(&outfd[0]);
	    c_eclose(&infd[0]);
	    c_lasmsg(TRUE, "deband-fatal", "Fatal error encountered");
            }
	}
    }

c_stoptm();

/* Close input and output images
   -----------------------------*/
   status = c_eclose(&infd[0]);
   if (status != E_SUCC) 
      c_lasmsg (FALSE, "deband-warn", "Nonfatal error encountered");
   status = c_eclose(&outfd[0]);
   if (status != E_SUCC) 
      c_lasmsg (FALSE, "deband-warn", "Nonfatal error encountered");
   }

/* Update ddr for output image
   --------------------------*/
c_upddr(&parblk, hosin, hosout[0], &nmrimg, window, bands,
        nbands, &line_inc, &samp_inc, &totbnd, &comb_flag);

/* Create associated history file for output image
   ----------------------------------------------*/
status = c_hstout(&parblk, hosout[0], hosin, &nmrimg, " ");

/* Output completion message, free allocated memory
   ------------------------------------------------*/
end_filter();
free(outbuf);
free(buf);
c_complt ("DEBAND", taeout, &one);

return EXIT_SUCCESS;

}
