/**************************************************************************

NAME: FILTER 

PURPOSE: applies the debanding (deswathing) filter to an image band.
         
PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1.0         1/91      T. Drackley        Original Development 
    1.1         1/91      D. Steinwand &     ???
			  J. Hood
    2.0		10/96	  C. Engebretson     Rewrite to replace lost
					     source code and update to
					     Baseline standards.

PROJECT: LAS

ALGORITHM REFERENCES:  Designed by Dr. Dennis Helder, South Dakota State
		       University Electrical Engineering Department.

***************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "protos.h"

/* The C compiler under DG/UX chokes with "signed char"; get around this with
   a typedef "schar" that represents the appropriate type.  This is an error
   in the compiler, by the way; char can be either signed or unsigned by
   default, depending on the particular implementation.  When using them as
   an arithmetic type, the type qualifier should be present.
   -------------------------------------------------------------------------*/
#if defined(DGUX) || defined(sun)
typedef char schar;
#else
typedef signed char schar;
#endif

#define MAXSAMPS 7000		/* Maximum allowed number of samples. */
#define NULL_CORRECT 100.0	/* Flag to represent no correction.   */

/* Macro to round floating-point numbers to the nearest integral value.
   -------------------------------------------------------------------- */
#define ROUND(x) ((schar)((x < 0.0) ? x - 0.5 : x + 0.5))

/* Function prototypes for forthcoming processing functions.
   --------------------------------------------------------- */
static float upperLowerDataPoint (unsigned char sample,
				  int index,
				  unsigned char *inbuf,
				  float tolrn,
				  int ns);

static float initialCorrectionFactor (unsigned char sample,
				      float udp,
				      float ldp);


/* These two static pointers are to dynamic arrays that the algorithm uses.
   A function below (init_filter) allocates memory for them, and another
   (end_filter) frees that memory.  This requires the caller of filter() to
   ensure that these two housekeeping functions are called.  This kind of
   complicates things, but without this, filter() would have to call malloc()
   each time it is called, which causes some execution time overhead. */

static float *ybuf;		   /* intermediate correction factors      */
static schar *correctbuf;	   /* final correction factors             */

int filter
(
    unsigned char *topbuf,  /* line at the top of the filter swath  */
    unsigned char *inbuf,   /* current line, middle of filter swath */
    unsigned char *botbuf,  /* line at bottom of the filter swath   */
    float tolrn,            /* tolerance value for rejection        */
    long nl,                /* number of lines to be processed      */
    long ns,                /* number samples per line to process   */
    long vertwidth,         /* lines in a swath (band) width        */
    long in_line,           /* current input line number            */
    unsigned char *outbuf  /* output buffer for corrected data     */
)

{
  float sum;		/* summing variable                        */
  float cf;		/* value of computed correction factor	   */
  long j;		/* loop counter				   */
  long n;		/* number of valid correction factors	   */

  float icf;			/* initial correction factor               */
  float ldp;			/* lower data point                        */
  float udp;			/* upper data point                        */

  unsigned char *optr; /* pointer to output data line             */
  unsigned char *iptr; /* pointer to input data line              */
  schar *cptr;         /* pointer to final correction factors     */
  long temp;


/* FILTER ALGORITHM PASS ONE: Find correction data line by line 
   ------------------------------------------------------------ */
  if (in_line > (nl - vertwidth))

    /* If we're near the bottom of the image, the lower data point is zero.
       We only worry about finding the upper data point.
       --------------------------------------------------------------------*/
    for (j = 0; j < ns; j++)	/* j = sample index */
      {

	/* Find UDP and initial correction factor, and store it in an
           intermediate array.
	   ----------------------------------------------------------*/
	udp = upperLowerDataPoint (inbuf[j], j, topbuf, tolrn, ns);
	ldp = 0.0;
	icf = initialCorrectionFactor (inbuf[j], udp, ldp);
	ybuf[j] = icf;
      }

  else if (in_line >= vertwidth + 1)

    /* We're in the middle of the image, so both the UDP and the LDP must
       be calculated here.
       ------------------------------------------------------------------*/
    for (j = 0; j < ns; j++)	/* j = sample index */
      {

	/* Find both the upper and lower data points, use them to calculate the
	   initial correction factor, and store it in an intermediate buffer.
	   ---------------------------------------------------------------- */
	udp = upperLowerDataPoint (inbuf[j], j, topbuf, tolrn, ns);
	ldp = upperLowerDataPoint (inbuf[j], j, botbuf, tolrn, ns);
	icf = initialCorrectionFactor (inbuf[j], udp, ldp);
	ybuf[j] = icf;
      }

  else
    for (j = 0; j < ns; j++)	/* j = sample index */

      /* Top of the image.
	 -----------------*/
      {

	/* Calculate the lower data point and then find the initial correction
	   factor, storing it in an intermediate buffer.
	   -----------------------------------------------------------------*/
	udp = 0.0;
	ldp = upperLowerDataPoint (inbuf[j], j, botbuf, tolrn, ns);
	icf = initialCorrectionFactor (inbuf[j], udp, ldp);
	ybuf[j] = icf;
      }


/* FILTER ALGORITHM PASS TWO: Do horizontal smoothing of the correction
   data.  Use a 35-pixel wide window, starting at left edge of the image
   ---------------------------------------------------------------------*/
  sum = 0.0;
  n = 0;

  /* Find the correction factor at the left edge of the image
     --------------------------------------------------------*/
  for (j = 0; j <= 17; j++)
    {
      if (ybuf[j] != NULL_CORRECT)
	{
	  sum += ybuf[j];
	  n++;
	}
    }
  if (n != 0)
    cf = sum / (float) n;
  else
    cf = 0.0;
  correctbuf[0] = ROUND (cf);

  /* Find the correction factors near the left edge of the image
     -----------------------------------------------------------*/
  for (j = 1; j <= 17; j++)
    {
      if (ybuf[j + 17] != NULL_CORRECT)
	{
	  sum += ybuf[j + 17];
	  n++;
	}
      if (n != 0)
	cf = sum / (float) n;
      else
	cf = 0.0;
      correctbuf[j] = ROUND (cf);
    }

/* Find the correction factors for the rest of the line.
   ----------------------------------------------------- */
  for (j = 18; j < ns - 17; j++)
    {
      if (ybuf[j + 17] != NULL_CORRECT)
	{
	  sum += ybuf[j + 17];
	  n++;
	}

      if (ybuf[j - 17] != NULL_CORRECT)
	{
	  sum -= ybuf[j - 17];
	  n--;
	}

      if (n != 0)
	cf = sum / (float) n;
      else
        cf = 0.0;

      if (ybuf[j] != NULL_CORRECT)
	correctbuf[j] = ROUND (cf);
      else
	correctbuf[j] = (schar) 0;
    }

/* Find the correction factors for the right-hand edge of the image.
   ----------------------------------------------------------------- */
  for (j = ns - 17; j < ns; j++)
    {
      if (ybuf[j - 17] != NULL_CORRECT)
	{
	  sum -= ybuf[j - 17];
	  n--;
	}

      if (n != 0)
	cf = sum / (float) n;
      else
	cf = 0.0;

      if (ybuf[j] != NULL_CORRECT)
	correctbuf[j] = ROUND (cf);
      else
	correctbuf[j] = (schar) 0;
    }

/* FINISHED: Apply the correction factors to the entire line.
   ---------------------------------------------------------- */
  optr = &(outbuf[0]);
  iptr = &(inbuf[0]);
  cptr = &(correctbuf[0]);

  for (j = 0; j < ns; j++)
    {
      temp = *iptr++ - *cptr++;
      temp = (temp < 0) ? 0 : temp;
      *optr++ = (unsigned char) ((temp > 255) ? 255 : temp);
    }
  return 0;
}


/* Find the upper or lower data point for the filter window.
   --------------------------------------------------------- */
static float 
upperLowerDataPoint (unsigned char sample,
		     int index,
		     unsigned char *inbuf,
		     float tolrn,
		     int ns)

{
  int k, n = 0;
  float sum = 0.0;
  float uldp = 0.0;

  /* If the difference between the current pixel and the compared pixel
     is within the tolerance value, then the value of the compared pixel
     becomes our upper/lower data point.
     -----------------------------------*/
  if ((float)abs ((int)(sample - inbuf[index])) <= tolrn)
    uldp = (float) inbuf[index];
  else
    {
     
      /* Otherwise, we average all of the pixel values that lie within
	 the tolerance in 10-pixel increments along the compared line.
	 -------------------------------------------------------------*/
      for (k = -20; k <= 20; k += 10)
	{
	  if (index + k >= 0 && index + k < ns &&
	      (float)abs ((int)(sample - inbuf[index + k])) <= tolrn)
	    {
	      n++;
	      sum += (float) inbuf[index + k];
	    }
	}
      if (n != 0)
	uldp = sum / (float) n;
    }
  return uldp;
}


/* Given the value of the current pixel, the upper data point, and the lower
   data point, calculate the value of the initial correction factor.  Details
   are given in the user's guide.
   -------------------------------------------------------------------------*/
static float initialCorrectionFactor
(
    unsigned char sample,
    float udp,
    float ldp
)

{
  float icf;

  /* The UDP/LDP is 0.0 if none was found.
     -------------------------------------*/
  if (udp != 0.0 && ldp != 0.0)
    icf = 0.5 * (sample - 0.5 * (udp + ldp));
  else if (udp != 0.0 && ldp == 0.0)
    icf = 0.5 * (sample - udp);
  else if (udp == 0.0 && ldp != 0.0)
    icf = 0.5 * (sample - ldp);
  else
    icf = NULL_CORRECT;

  return (icf);
}


/* These two simple functions allocate and free memory for the two dynamic
   arrays that the algorithm uses.  The first should be called before the
   first call to filter(); the second should be called after the last.
   -----------------------------------------------------------------------*/
void init_filter
(
    long ns
)
{
    ybuf = malloc(ns * sizeof(float));
    correctbuf = malloc(ns);
}

void
end_filter(void)
{
    free(ybuf);
    free(correctbuf);
}
