/*****************************************************************************

FUNCTION:       GET_LINE

PURPOSE:        Increments pointers into the image data.  These pointers
	        include a pointer to the line at the top of the filter
                window, a line at the center of the filter window, and a
	        line at the bottom of the filter window.  These three
                lines are needed by the debanding filter algorithm.  

ALGORITHM REFERENCES:   None.

*****************************************************************************/

#include "las.h"
#include "protos.h"

/* This macro increments a pointer to the input buffer.  It steps through
   the input image data one line at a time.
  ----------------------------------------------------------------------*/
#define INC(x)    x = (x >= ns * height * 2 ? 0 : x + ns)

static int first;	/* Flag if get_line() has been called for each band */
static long line;	/* Line in image currently being processed          */

void init_get_line
(void)
{

/* Initialize static data members for each input band.
 ---------------------------------------------------*/
first = 1;
line = 1;
return;
}     

void get_line
(
    long bndin,	   /* Current band of input image          */
    long nl,             /* Number of lines in input image       */
    long ns,             /* Number of samples in each line       */
    long height,         /* Height of filter window (swath)      */
    unsigned char *buf,  /* Pointer to image data buffer         */
    long *top,           /* Pointer to line at top of swath      */
    long *in,            /* Pointer to line at center of swath   */
    long *bottom,        /* Pointer to line at bottom of swath   */
    struct FDESC * infd[2],         /* Input image file descriptor          */
    struct FDESC * outfd[2]        /* Output image file descriptor         */
)

{        
long status;		          /* value returned from imageio functions */
long one = 1;		          /* constant of one                       */	
long i;		                  /* loop counter 	         	   */

/* If this is the first call for this band, we must prime the input
   image buffer with the first HEIGHT lines of image data.
   ----------------------------------------------------------------*/
if (first)
   {
   *in = *bottom = -ns;
   *top = 0;

   /* Read in the first HEIGHT lines of image data.  Since we're at the top
      of the image, there is no top line in the filter window until we read
      past the first HEIGHT lines.
      ---------------------------------------------------------------------*/
   for (i = 0; i < height; line++, i++)
      {
      INC(*bottom);
      status = c_eread (&infd[0], &bndin, &line, &(buf[*bottom]), &one);
      if (status != ns)
        {
        c_eclose(outfd);
        c_eclose(infd);
        c_lasmsg(TRUE, "deband-fatal", "Fatal error encountered");
        }
      }
   first = 0;
   }

/* If we are at the bottom of the image (the last HEIGHT lines of it), then
   we only need to increase the pointers to the top and center of the filter
   swath, since the bottom of the window no longer lies in the image data.
   ------------------------------------------------------------------------*/
if (line > nl)
    {
    INC(*top);
    INC(*in);
    }

else
   {
   /* Otherwise, we increment the pointer to the center and bottom of the 
      filter window (and if necessary, the top.  Read one line of image
      data at a time, as needed.
      -------------------------------------------------------------------*/
   INC(*in);
   INC(*bottom);
   if (line > 2*height+1) INC(*top);
   status = c_eread (&infd[0], &bndin, &line, &(buf[*bottom]), &one);
   if (status  !=  ns)
       {
       c_eclose(outfd);
       c_lasmsg(TRUE, "deband-fatal", "Fatal error encountered");
       }
   line++;
   }

return;
}
