/*****************************************************************************
PROGRAM:	DELDLTF

PURPOSE:	Delete a single entry from the image associated Display and
		Look-up Table File (DLTF).  If the specified entry is the
		only entry within the DLTF then the DLTF as a whole will
		also be deleted.

PROGRAM HISTORY:
VERSION	     DATE	PROGRAMMER   CODE/CONT	REQUEST
-------	     ----	----------   ---------	-------
  1.0	     7/85	D. Gordon    USGS/EDC	Initial development
  1.1	    11/88	D. Akkerman  USGS/EDC	DMS v3_0 retrocode.
						Rewritten for the LAS 5.0
						DLTF format.
  1.2	    12/89	L. Huewe     USGS/EDC	Modified so an error message is
						output when the DLTF entry to
						be deleted cannot be found.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	DELDLTF must be run under TAE.

PROJECT:	LAS

ALGORITHM:
	Get the user specified parameters.
	Create the DLTF host file name.
	Create the DLTF entry name.
	Open the DLTF for read/write access.
	Read the first type of record from the specified entry.
	Rewind the DLTF.
	Mark the first type of record for deletion.
	Mark the second type of DLTF record (one per band).
	Close the DLTF specifying that marked records be deleted.
	Issue a standard completion message and termiante processing.

*****************************************************************************/
#define UTIL 1
#define DMSLEN 12
#include <stdlib.h>
#include "dltf.h"
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
long	i,j,		/* Working loop indices.			*/
	numbnd,		/* Total number of bands in the DLTF entry.	*/
	access = 3,	/* Specifies read/write file access.		*/
	action = 1,	/* Specifies that marked DLTF records be 
			   deleted.					*/
	zero = 0,	/* Constant zero; also specifies that no files
			   were created.				*/
	fatal = -1,	/* fatal error flag				*/
	fd,		/* File descriptor for the DLTF file. 		*/
	slen,		/* Number of characters in the DLTF entry name
			   to be converted to upper case.		*/
	clen = MXDLTF,	/* Length of the character portion of the DLTF
			   entry.					*/
	dlen = 0,	/* Length of the data portion of the DLTF entry.*/
	one = 1,	/* Constant one.				*/
	status;		/* Status indicator.				*/

char	hostnm[CMLEN],	/* Host file name for the DLTF.			*/
	dltfnam[DMSLEN+1],
			/* DLTF entry name.				*/
	entry[DMSLEN+1],/* Working DLTF entry name.			*/
	dtype[DTYPL];	/* Data type of the DLTF entry.			*/

char tmpstr[1][CMLEN];  /* temporary string */

unsigned char *lut = NULL;	/* Dummy pointer used to contain the data portion
			   of the DLTF entry.				*/

struct CDLTF cdltf;	/* Structure to contain the character portion of
			   the DLTF entry.				*/

/* Get the user specified parameters.
  -----------------------------------*/
getpar(dltfnam,hostnm);

/* Create the DLTF entry name by converting the input string to upper
   case and then appending an "@".  The "@" is used to prevent possible 
   collisions within the first type of DLTF records.
  -----------------------------------------------------------*/
slen = (long) strlen(dltfnam);
c_low2up(dltfnam,&slen);
sprintf(entry,"%s@",dltfnam);

/* Open the DLTF for read/write access.
  --------------------------------------*/
status = c_lsopen(&fd,hostnm,&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","lstdltf-fatal",&fatal);

/* Read the first type of DLTF record to individual band numbers.  These
   numbers will be used to delete the individual band(s) from the DLTF
   which are stored as the second type of DLTF records (one for each
   band).
  -----------------------------------------------------------*/
status = c_lsread(&fd,entry,&clen,&dlen,(char *) &cdltf,lut,dtype);
if ((status != E_SUCC) && (status != E_SMAL))
    {
    c_lsclos(&fd,hostnm,&zero);
    c_errmsg("Unable to delete DLTF entry","deldltf-delete",&one);
    c_errmsg("Fatal error encountered","deldltf-fatal",&fatal);
    }

/* Rewind the DLTF.
  -----------------*/
status = c_lsrew(&fd);
if (status != E_SUCC)
   {
   c_lsclos(&fd,hostnm,&zero);
   c_errmsg("Fatal error encountered","deldltf-fatal",&fatal);
   }

/* Mark the first type of DLTF record for deletion.
  ------------------------------------------------*/
status = c_lsdel(&fd,entry);
if (status != E_SUCC)
   {
   c_lsclos(&fd,hostnm,&zero);
   c_errmsg("Fatal error encountered","deldltf-fatal",&fatal);
   }

/* Now mark the second type of DLTF record (one per band) for deletion.
  --------------------------------------------------------------------*/
sscanf(cdltf.numbnd,"%d",&numbnd);
for (i = 0, j = 1; i < numbnd; i++, j++)
  {
   sprintf(entry,"%s%d",dltfnam,j);
   c_lsdel(&fd,entry);
   if (status != E_SUCC)
      {
      c_lsclos(&fd,hostnm,&zero);
      c_errmsg("Fatal error encountered","deldltf-fatal",&fatal);
      }
  }

/* Close the DLTF specifying that marked record be deleted.
  --------------------------------------------------------*/
c_lsclos(&fd,hostnm,&action);

/* Issue a standard completion message and terminate processing.
  -------------------------------------------------------------*/
strcpy(tmpstr[0],"\0"); 
c_complt("DELDLTF",tmpstr,&zero);

return EXIT_SUCCESS;

}
