/******************************************************************************
SUBROUTINE ADEM
 
PURPOSE:
 	Subroutine ADEM process DEM type 'A' record data.
      The subroutine calls either ADEMA or ADEMB to do the actual 
      processing, then computes the number of lines and samples in 
      the image by dividing the distance between the corner coordinates
      by the spatial resolution value for the image.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
 
DESCRIPTION/ALGORITHM:
 
Read the first block of data
Test for ASCII data

If dem Ascii data then
  Call ADEMA for processing
Elseif dem Binary data then
  Call ADEMB for processing
Endif
	
Set YMAX as the maximum of the y-axis corner coordinates
Set YMIN as the minimum of the y-axis corner coordinates
 
The number of input lines is equal to the number of profiles on tape
Compute number of input samples as the difference between the y coordinates
   divided by the y spatial resolution value.

Write message to terminal showing output image size
 
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dementer.h"

void adem
(
  short *buff2,                 /* I: Input buffer */
  long *unit,                   /* I: Tape unit number */
  long *bufsiz,                 /* O: Current size of the buffer */
  long *bufref,                 /* O: Reference pointer to the input buffer */
  long *lines,                  /* O: Number of input lines */
  long *samps,                  /* O: Number of input samples */
  short *form,                  /* O: Format of the data */
  float *deltay,                /* O: Resolution in the Y direction */
  float *deltaz,                /* O: Resolution in the Z direction */
  short *cdata,
  double *ymin,                 /* O: Minimum Y location */
  struct VLL_DEM *vll_dem,      /* I: DEM header struct */
  long *outsys,                 /* I: Output system */
  FILE *infp                    /* I: Input data file pointer */
)
{  /*  adem  */

    static long fatal = -1;      /*  fatal error code                    */
    static long handle = HANDLE; /* handle flag of tapeio rountines      */
    static long nonfatal = 1;    /*  nonfatal error code                 */
    static long rewopt = REWOFFL;/* rewind option                        */
    static long zero = 0;        /*  value of 0                          */
    double ymax;                 /*  temporary maximum Y value           */
    double yfirst;               /*  first Y corrdinate of the corners   */
    double ylast;                /*  last Y corrdinate of the corners    */

    long nbytes;                 /*  number of bytes to read from the tape */
    long status;                 /*  status of function calls            */

    char msg[ERRLEN];            /*  character buffer of errmsgs         */

    /*  Read the first tape record
        ------------------------------*/
    nbytes = 32768;

    if(infp == NULL)
    {
        status = c_tpread(unit,(unsigned char *)buff2,&nbytes,&handle);
        if (status < TAPE_SUCC)
        {
            c_errmsg("Error encountered reading tape",
                    "dementer-tpread",&nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        else if (status > TAPE_SUCC) 
            *bufsiz = status;
    }
    else
    {
        if ((status = (long)fread(buff2,1,nbytes,infp)) < 0)
        {
            c_errmsg("Error encountered reading disk",
                    "dementer-fread",&nonfatal);
            fclose(infp);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        else
            *bufsiz = status;
    }

    /*	Test for ASCII or BINARY data: ASCII data will have the bit
        pattern '00110000' in byte 2 of a numeric value, where BINARY
        data would have the pattern '11110000'. Input data is assumed
        to be BINARY, and set to ASCII if the bit pattern matches.
        ---------------------------------------------------------------------*/
    *form = 22;
#ifndef vms
    if ((buff2[77] & 1) != 1) 
        *form = 0;
    if ((buff2[74] & 48) == 48) 
        *form = 23;
#else
    if ((buff2[77] & 256) != 256) 
        *form = 0;
    if ((buff2[74] & 12288) == 12288) 
        *form = 23;
#endif

    /*  Write error messages if needed
        ----------------------------------*/
    if (*form == 0)
    {
        c_errmsg("Data cannot be interpreted in dem format", "dementer-format",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);

        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    if (((*bufsiz % 1024) != 0) && (*form == 23))
    {
        c_errmsg("Blocksize error while reading ascii data",
                "dementer-block",&nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Get the Ascii or Binary header data
        ---------------------------------------*/
    if(*form == 22)
        ademb(vll_dem,buff2,bufsiz,bufref,deltay,deltaz,cdata,unit,outsys,infp);
    else if(*form == 23)
        adema(vll_dem,(char *)buff2,deltay,deltaz,bufref,cdata,unit,infp);

    /*  Abort if the input data is random
        -------------------------------------*/
    if (vll_dem->epat == 2)
    {
        c_errmsg("Random data patterns are not supported",
                "dementer-random",&nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Calculate the number of samples in the image
        ------------------------------------------------*/
    ymax = vll_dem->corner[0][1];
    ymax = DEMMAX(ymax,vll_dem->corner[1][1]);
    ymax = DEMMAX(ymax,vll_dem->corner[2][1]);
    ymax = DEMMAX(ymax,vll_dem->corner[3][1]);
    *ymin = vll_dem->corner[0][1];
    *ymin = DEMMIN(*ymin,vll_dem->corner[1][1]);
    *ymin = DEMMIN(*ymin,vll_dem->corner[2][1]);
    *ymin = DEMMIN(*ymin,vll_dem->corner[3][1]);

    /*  Calculate the coordinates of the datum nearest the min and max Y
        --------------------------------------------------------------------*/
    if (((long)*ymin % (long)vll_dem->yres) == 0)
        yfirst = *ymin;
    else
        yfirst = ((long) (*ymin / (double)vll_dem->yres) + 1) * vll_dem->yres;

    ylast  = ymax - ((long)ymax % (long)vll_dem->yres);

    *lines = (long) vll_dem->pcol;
    *samps = (long)(((ylast - yfirst) / (double)vll_dem->yres) + 1.0);

    sprintf (msg,"Output image will be %d lines by %d samples",*samps,*lines);
    c_errmsg (msg,"informational",&nonfatal);

    if (*deltaz == 1.0)
        c_errmsg("Output image will contain I*2 data","informational",
                  &nonfatal);
    else
        c_errmsg("Output image will contain R*4 data","informational",
                  &nonfatal);
}
