/******************************************************************************
SUBROUTINE ADEMA
 
PURPOSE:
 	Subroutine ADEMA processes DEM type A records in Ascii
      format. The routine reads and converts the data elements in 
      the record and places them into a structure which will be 
      displayed to the user and written to a labled table.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
DESCRIPTION/ALGORITHM:
Initialize the local variables
 
Read the first physical record from the tape
If error occurs then
   Output an error message and close the tape drive
endif
 
Place each value of the A record into the appropriate member of the DEM struct

If the data cannot be interpreted THEN
   Output an errmsg and close the tape

Set the buffer reference pointer (BUFREF)
******************************************************************************/
#include <string.h>
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "dementer.h"

void adema
(
  struct VLL_DEM *vll_dem,      /* I: Struct defining record A of DEM data */
  char *record,                 /* I: Input ASCII record */
  float *deltay,                /* O: Resolution in the Y direction */
  float *deltaz,                /* O: Resolution in the Z direction */
  long *bufref,                 /* O: Buffer reference pointer */
  short *cdat,     
  long *unit,                   /* I: Tape unit */
  FILE *infp                    /* I: Input data file pointer */
)
{  /*  adema  */

    static long fatal = -1;      /*  fatal error code                   */
    static long handle = HANDLE; /* handle flag of tapeio rountines     */
    static long nonfatal = 1;    /*  nonfatal error code                */
    static long rewopt = REWOFFL;/* rewind option                       */
    static long zero = 0;        /*  value of 0                         */
    long dtype;	/* data type of the value to be converted */
    long i;                     /*  counter --                          */
    long j;                     /*  counter --                          */
    long k;                     /*  counter --                          */
    long temp;                  /*  number of bytes to be copied        */

    char chrtemp[24];           /*  temporary character buffer*/
    char *chrptr;               /*  character pointer                   */

    /*  Read the header record values from the Ascii string
        When the ASCII string represents a double precision number the character
        'D' needs to be replaced with an 'E'.  Sscanf only recognizes the 'E'.
        ---------------------------------------------------------------------*/
    temp = 144;
    dtype = EBYTE;
    c_pxcopy((unsigned char *)record,(unsigned char *)vll_dem->crec,
             &dtype,&temp);
    sscanf(&record[144],"%hd",&(vll_dem->lvl));
    sscanf(&record[150],"%hd",&(vll_dem->epat));
    sscanf(&record[156],"%hd",&(vll_dem->gref));
    sscanf(&record[162],"%hd",&(vll_dem->zref));
    for (i = 0,j = 168; i < 15; i++, j += 24)
    {
        strncpy(chrtemp,&record[j],24);
        chrptr = strchr(chrtemp,'D');
        if (chrptr != NULL)
            *chrptr = 'E';
        sscanf(chrtemp,"%lE",&(vll_dem->ppar[i]));
    }
    sscanf(&record[528],"%hd",&(vll_dem->puni));
    sscanf(&record[534],"%hd",&(vll_dem->euni));
    sscanf(&record[540],"%hd",&(vll_dem->side));
    j = 546;
    for (i = 0; i < 4; i++)
        for (k = 0; k < 2; k++)
        {
            strncpy(chrtemp,&record[j],24);
            chrptr = strchr(chrtemp,'D');
            if (chrptr != NULL)
                *chrptr = 'E';
            sscanf(chrtemp,"%lE",&(vll_dem->corner[i][k]));
            j += 24;
        }
    strncpy(chrtemp,&record[738],24);
    chrptr = strchr(chrtemp,'D');
    if (chrptr != NULL)
        *chrptr = 'E';
    sscanf(chrtemp,"%lE",&(vll_dem->mine));
    strncpy(chrtemp,&record[762],24);
    chrptr = strchr(chrtemp,'D');
    if (chrptr != NULL)
        *chrptr = 'E';
    sscanf(chrtemp,"%lE",&(vll_dem->maxe));
    strncpy(chrtemp,&record[786],24);
    chrptr = strchr(chrtemp,'D');
    if (chrptr != NULL)
        *chrptr = 'E';
    sscanf(chrtemp,"%lE",&(vll_dem->ccw));
    strncpy(chrtemp,&record[810],6);
    chrtemp[6] = ' ';
    sscanf(chrtemp,"%hd",&(vll_dem->cdata));

    /*  Each of the three next parts of the character buffer is copied into a 
        temporary buffer in order that the sscanf works properly. The original 
        character buffer does not have any delimiters between these values.  
        ---------------------------------------------------------------------*/
    strncpy(chrtemp,&record[816],12);
    chrtemp[12] = ' ';
    sscanf(chrtemp,"%e",&vll_dem->xres);
    strncpy(chrtemp,&record[828],12);
    chrtemp[12] = ' ';
    sscanf(chrtemp,"%e",&vll_dem->yres);
    strncpy(chrtemp,&record[840],12);
    chrtemp[12] = ' ';
    sscanf(chrtemp,"%e",&vll_dem->zres);

    sscanf(&record[852],"%hd",&(vll_dem->prow));
    sscanf(&record[858],"%hd",&(vll_dem->pcol));

    /*  Test for valid DEM data
        ---------------------------*/
    if (((vll_dem->epat != 1) && (vll_dem->epat != 2)) &&
            ((vll_dem->euni != 1) && (vll_dem->euni != 2)))
    {
        c_errmsg("Data cannot be interpreted in dem format", "dementer-format",
                &nonfatal);
        if(infp == NULL)
            c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);

        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    *bufref += 432;
    *deltay = vll_dem->yres;
    *deltaz = vll_dem->zres;
    *cdat = vll_dem->cdata;
}  /* adema  */
