/******************************************************************************
SUBROUTINE ADEMB
 
PURPOSE:
 	Subroutine ADEMB processes DEM type A records in Binary
      format. The routine reads, and converts the 
      data elements in the record and places them into a buffer
      which will be displayed to the user and written to a
      labeled table file.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
DESCRIPTION/ALGORITHM:
INITIALIZE the local variables
 
READ the first physical record from the tape
IF error occurs THEN
   WRITE an error message and STOP
END IF
 
CALCULATE the buffer size (BUFSIZ) 
CALCULATE the buffer reference pointer (BUFREF)
 
Call PXSYS to convert values to the host machine
 
IF the data cannot be interpreted THEN
 CALL ERRMSG and STOP processing
 
SET the projection parameters to zero - this is done because 
  they are supposed to be zero but sometimes have unintelligible data
  in them. This data continuously fails when attempts are made
  to convert this data to DEC doubleprecision format. Only
  binary data seems to have this problem.
 
 CONVERT the corner coordinates to host doubleprecision format
 CONVERT the minimum elevation to host doubleprecision format
 CONVERT the maximum elevation to host doubleprecision format
 CONVERT the counterclockwise rotation angle to doubleprecision
 CONVERT the X spatial resolution value to doubleprecision format
 CONVERT the Y spatial resolution value to doubleprecision format
 CONVERT the Z spatial resolution value to doubleprecision format

 return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "util.h"
#include "dementer.h"

void ademb
(
  struct VLL_DEM *vll_dem,      /* I: Struct holding record A data */
  short *buff2,                 /* I: Buffer */
  long *bufsiz,                 /* O: Current size of the buffer */
  long *bufref,                 /* O: Buffer reference pointer */
  float *deltay,                /* O: Resolution in the Y direction */
  float *deltaz,                /* O: Resolution in the Z direction */
  short *cdat,
  long *unit,                   /* I: Tape unit number */
  long *outsys,                 /* I: Output system */
  FILE *infp                    /* I: Input data file pointer */
)
{ /*  ademb  */

    static double dover=MAXREAL;  /*  value if over flow                  */
    static double dunder=MINREAL; /*  value if under flow                 */
    static float fover=MAXREAL;   /*  value used if overflow occurs       */
    static float funder=MINREAL;  /*  value used if underflow occurs      */
    static long fatal = -1;       /*  fatal error code                    */
    static long flag = SYS_USE;   /*  values provided r over/under flow   */
    static long handle = HANDLE;  /*  handle flag of tapeio rountines     */
    static long insys = SYS_IBM_MVS; /* input system                      */
    static long nonfatal = 1;     /*  nonfatal error code                 */
    static long rewopt = REWOFFL; /*  rewind option                       */
    static long zero = 0;         /*  value of 0                          */
    static short sover = MAXWORD; /*  value if over flow                  */
    static short sunder = MINWORD;/*  value if under flow                 */
    long dtype;                   /*  data type of the values to be converted */
    long i;                       /*  counter --                          */
    long i2bytes;                 /*  number of words to copy             */
    long j;                       /*  counter --                          */
    long k;                       /*  counter --                          */
    long nbytes;                  /*  number of bytes needed              */
    long size;                    /*  size of the data type               */
    long status;                  /*  status of called routines           */

    /*  Copy he first 384 bytes into the correct members in the structure
        ---------------------------------------------------------------------*/
    i2bytes = 76;
    dtype = EWORD;
    c_pxcopy((unsigned char *)&buff2[4],(unsigned char *)vll_dem,&dtype,
            &i2bytes);

    i2bytes = 15;
    dtype = EDOUBLE;
    c_pxcopy((unsigned char *)&buff2[80],(unsigned char *)vll_dem->ppar,&dtype,
            &i2bytes);

    i2bytes = 3;
    dtype = EWORD;
    c_pxcopy((unsigned char *)&buff2[140],(unsigned char *)&vll_dem->puni,
            &dtype,&i2bytes);

    i2bytes = 1;
    dtype = EDOUBLE;
    for (i = 0,j = 143; i < 4; i++)
        for (k = 0; k < 2; k++,j += 4)
            c_pxcopy((unsigned char *)&buff2[j],
                    (unsigned char *)&vll_dem->corner[i][k],&dtype, &i2bytes);

    i2bytes = 3;
    c_pxcopy((unsigned char *)&buff2[175],(unsigned char *)&vll_dem->mine,
            &dtype,&i2bytes);

    i2bytes = 1;
    dtype = EWORD;
    c_pxcopy((unsigned char *)&buff2[187],(unsigned char *)&vll_dem->cdata,
            &dtype,&i2bytes);

    i2bytes = 3;
    dtype = EREAL;
    c_pxcopy((unsigned char *)&buff2[188],(unsigned char *)&vll_dem->xres,
            &dtype,&i2bytes);

    i2bytes = 2;
    dtype = EWORD;
    c_pxcopy((unsigned char *)&buff2[194],(unsigned char *)&vll_dem->prow,
            &dtype,&i2bytes);

    /*  Reset bufsiz and bufref  
        ---------------------------*/
    i2bytes = 192;
    *bufsiz = (*bufsiz / 2) - i2bytes - 4;
    *bufref = *bufref + i2bytes + 4;

    /*  Convert character data from EBCIDIC to ASCII
        ------------------------------------------------*/
    nbytes = 144;
    c_eb2asc(vll_dem->crec,&nbytes);

    /*  If needed, convert all the variables in the header record, from IBM 
        to host and set the projection parameters to zero.
        ---------------------------------------------------------------------*/
    dtype = EWORD;
    size = 4;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dem->lvl,&dtype,
            &size,&sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);

        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    for (j = 0; j < 15; j ++)
        vll_dem->ppar[j] = 0.0;

    size = 3;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dem->puni,&dtype,
            &size,&sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Test for valid DEM data
        ---------------------------*/
    if (((vll_dem->epat != 1) && (vll_dem->epat != 2)) &&
            ((vll_dem->euni != 1) && (vll_dem->euni != 2)))
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Data cannot be interpreted in dem format",
                "dementer-format",&nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    } 

    /*  If needed, convert all the variables in the header record from IBM 
        to host.
        ---------------------------------------------------------------------*/
    size = 1;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dem->cdata,&dtype,
            &size,&sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    size = 2;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dem->prow,&dtype,&size,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    dtype = EDOUBLE;
    size = 1;
    for (i = 0; i < 4; i++)
        for (k = 0; k < 2; k++)
            status = c_pxsys(&insys,outsys,
                    (unsigned char *)&vll_dem->corner[i][k],
                    &dtype,&size,&dunder,&dover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    size = 3;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dem->mine,&dtype,&size,
            &dunder,&dover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    dtype = EREAL;
    size = 3;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dem->xres,&dtype,
            &size,&funder,&fover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    *deltay = vll_dem->yres;
    *deltaz = vll_dem->zres;
    *cdat = vll_dem->cdata;
}  /*  demb  */
