/******************************************************************************
SUBROUTINE ADTED
 
PURPOSE:
 	Subroutine ADTT processes DTED type A data. Data elements
      within the record are read, and are placed to the members
      of a DTED structure. The DTED information is displayed to the user.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 5.1 03/03/93  D. Etrheim	CSB           Allow for Mexican DTED PRF #6232
 5.2 08/23/93  K. Sheffield	CSB           Read DTED from disk PRF #5717
 
DESCRIPTION/ALGORITHM:

Initialize variables
Read the next physical record from the tape
If an error occurs then
  Output an error message and close tape drive
end if

Read the header data from the tape
 
If the data cannot be interpreted then
  Output errmsg and close tape drive

Output message to terminal showing output image size
 
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "dementer.h"

int adted
(
  long *unit,                   /* I: Tape unit number */
  long *lines,                  /* O: Number of input lines */
  long *samps,                  /* O: Number of input samples */
  struct VLL_DTED *vll_dted,    /* O: DTED header struct */
  FILE *infp                    /* I: Input data file pointer */
)
{  /* adted  */

    static long fatal = -1;      /*  fatal error code                    */
    static long handle = HANDLE; /*  handle flag of tapeio rountines     */
    static long nonfatal = 1;    /*  nonfatal error code                 */
    static long rewopt = REWOFFL; /* rewind option                       */
    static long zero = 0;        /*  value of 0                          */
    long bufsiz;                 /*  current buffer size                 */
    long bytes_read = 0;         /*  Counts the # of bytes read          */
    long hdrflg;
    long status;                 /*  status of function calls            */

    char buff[80];               /*  buffer                         	 */
    char datum[10];              /*  Holds the datum name                */
    char chrtemp[5];             /*  temporary character buffer          */
    char msg[ERRLEN];            /*  message buffer                      */
    char *cptr;
    extern char hosin[CMLEN];    /* input data file host name            */

    /*  Skip over the record(s) that designate the tape volume(s)
        -------------------------------------------------------------*/
    do
    {
        bufsiz = DTED_HDR_LEN;
        if(infp == NULL)
        {
            status = c_tpread(unit,(unsigned char *)buff,&bufsiz,&handle);
            if (status < TAPE_SUCC)
            {
                c_errmsg("Error encountered reading tape","dementer-tpread",
                        &nonfatal);
                status = c_tpclos(unit,&handle,&rewopt,&zero);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }
            else if (status > TAPE_SUCC) bufsiz = status;
        }
        else
        {
            status = (long)fread(buff,1,bufsiz,infp);
            if (status < TAPE_SUCC)
            {
                c_errmsg("Error encountered reading disk","dementer-fread",
                        &nonfatal);
                fclose(infp);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }
            else if (status > TAPE_SUCC) bufsiz = status;
            bytes_read += status;
        }
    }
    while (strncmp(buff,"VOL",3) == 0);

    /* one spec. document indicates that only the UHL, DSI, ACC, & elevation
       data are present on CD's.  Check to see if an ANSI header record is 
       present */

    if (strncmp(buff,"HDR",3) == 0)
        hdrflg = TRUE;
    else
        hdrflg = FALSE;

    if (!hdrflg) 
    {
        /* HDR record not present, fill in with bogus data */
#ifdef vms
        cptr = strrchr(hosin,']');
#else
        cptr = strrchr(hosin,'/');
#endif
        if (cptr == NULL)
            cptr = hosin;
        else
            cptr++;

        strncpy(vll_dted->fname,cptr,17);
        vll_dted->fname[17] = '\0';
        strncpy(vll_dted->machin,"UNIVAC",6);
        vll_dted->machin[6] = '\0';
        strncpy(vll_dted->cdate,"",6);
        strncpy(vll_dted->edate,"",6);

        vll_dted->reel = 0;
        vll_dted->file = 0;
        vll_dted->access = 0;
    }
    else
    {
        /*  Copy the strings into the correct members of the DTED structure
            ----------------------------------------------------------------*/
        strncpy(vll_dted->fname,&buff[4],17);
        vll_dted->fname[17] = '\0';
        strncpy(vll_dted->machin,&buff[21],6);
        vll_dted->machin[6] = '\0';
        strncpy(vll_dted->cdate,&buff[41],6);
        vll_dted->cdate[6] = '\0';
        strncpy(vll_dted->edate,&buff[47],6);
        vll_dted->edate[6] = '\0';


        /*  Test for valid DTED data
            ----------------------------*/
        if (bufsiz != DTED_HDR_LEN) 
        {
            c_errmsg("Data cannot be interpreted in dted format",
                    "dementer-format",&nonfatal);
            if(infp == NULL)
                status = c_tpclos(unit,&handle,&rewopt,&zero);
            else
                fclose(infp);

            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        /* GEOGRAFIA is what is on the Mexican dted tapes
           -------------------------------------------------*/
        if ((strncmp(&buff[21],"UNIVAC",6) != 0) && 
                (strncmp(&buff[10],"GEOGRAFIA",9) != 0))
        {
            c_errmsg("Data cannot be interpreted in dted format",
                    "dementer-format",&nonfatal);
            if(infp == NULL)
                status = c_tpclos(unit,&handle,&rewopt,&zero);
            else
                fclose(infp);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }

        /*  Each of the three next parts of the character buffer is copied into 
            a temporary buffer in order that the sscanf works properly. The 
            original character buffer does not have any delimiters between 
            these values.  
            ----------------------------------------------------------------*/
        strncpy(chrtemp,&buff[27],4);
        chrtemp[4] = ' ';
        sscanf(chrtemp,"%d",&vll_dted->reel);
        strncpy(chrtemp,&buff[31],4);
        chrtemp[4] = ' ';
        sscanf(chrtemp,"%d",&vll_dted->file);
        strncpy(chrtemp,&buff[53],4);
        chrtemp[4] = ' ';
        sscanf(chrtemp,"%d",&vll_dted->access);

        bufsiz = DTED_UHL_LEN;
        if(infp == NULL)
        {
            status = c_tpread(unit,(unsigned char *)buff,&bufsiz,&handle);
            if (status < TAPE_SUCC)
            {
                c_errmsg("Error encountered reading tape","dementer-tpread",
                        &nonfatal);
                status = c_tpclos(unit,&handle,&rewopt,&zero);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }
            else if (status > TAPE_SUCC) bufsiz = status;
        }
        else
        {
            if(DTED_UHL_LEN != (bufsiz = (long)fread(buff,1,bufsiz,infp)))
            {
                c_errmsg("Error encountered reading disk","dementer-fread", 
                        &nonfatal);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }
            bytes_read += bufsiz;
        }
    }

    /*  Copy the strings of the second record into the correct members of the
        DTED structure
        --------------------------------------------------------------------*/
    strncpy(vll_dted->swlong,&buff[4],8);
    vll_dted->swlong[8] = '\0';
    strncpy(vll_dted->swlat,&buff[12],8);
    vll_dted->swlat[8] = '\0';

    /*  Each of the three next parts of the character buffer is copied into a 
        temporary buffer in order that the sscanf works properly. The original 
        character buffer does not have any delimiters between these values.  
        -------------------------------------------------------------------*/
    strncpy(chrtemp,&buff[20],4);
    chrtemp[4] = ' ';
    sscanf(chrtemp,"%d",&vll_dted->longs);
    strncpy(chrtemp,&buff[24],4);
    chrtemp[4] = ' ';
    sscanf(chrtemp,"%d",&vll_dted->lats);
    strncpy(chrtemp,&buff[28],4);
    chrtemp[4] = ' ';
    sscanf(chrtemp,"%d",&vll_dted->acode);

    strncpy(vll_dted->scode,&buff[32],3);
    vll_dted->scode[3] = '\0';

    /*  Each of the two next parts of the character buffer is copied into a 
        temporary buffer in order that the sscanf works properly. The original 
        character buffer does not have any delimiters between these values.  
        -------------------------------------------------------------------*/
    strncpy(chrtemp,&buff[47],4);
    chrtemp[4] = ' ';
    sscanf(chrtemp,"%d",&vll_dted->patchs);
    strncpy(chrtemp,&buff[51],4);
    chrtemp[4] = ' ';
    sscanf(chrtemp,"%d",&vll_dted->points);

    /*  Output the number of lines and samples in the output image.  The number
        of input sample is the number of output lines and the number of input
        lines is the number of output samples because of the rotation of the 
        image as it is read from a tape
        ----------------------------------------------------------------------*/
    *lines = vll_dted->patchs;
    *samps = vll_dted->points;
    sprintf(msg,"Output image will be %d lines by %d samples",*samps,*lines);
    c_errmsg(msg,"informational",&nonfatal);
    c_errmsg("Output image will contain I*2 data","informational",&nonfatal);

    if (infp != NULL)
    {
        /* Perform 2 calls to fread() to get the datum name into
           the buffer
           --------------------------------------------------------*/
        status = (long)fread(buff,1,bufsiz,infp);
        status = (long)fread(buff,1,bufsiz,infp);

        /* Because of the above calls to fread(), the file
           pointer is now ahead, the fseek() gets the
           pointer to the correct location so the rest of
           the processing can take place.
           --------------------------------------------------*/
        fseek(infp,bytes_read,0);

        /* Get the datum name.  If it is not WGS84,
           then issue an error.
           -------------------------------------------*/ 
        strncpy(datum,&buff[64],5);
        datum[5] = '\0';
        if (strcmp(datum,"WGS84") != 0)
        {
            c_lasmsg(FALSE,"dementer-datum",
                     "Datum not WGS84, check the header");
            return(E_FAIL);
        }
        else
            strcpy(vll_dted->datum,datum);
    }

    /*  Read the End-of-File
        ------------------------*/
    if(infp == NULL)
    {
        bufsiz = 80;
        status = c_tpread(unit,(unsigned char *)buff,&bufsiz,&handle);
        if (status != TAPE_EOF)
        {
            c_errmsg("Error encountered reading tape","dementer-tpread",
                    &nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
    }

    return(E_SUCC);
}
