/******************************************************************************
SUBROUTINE ADTT
 
PURPOSE:
 	Subroutine ADTT processes DTT type A data. Data elements
      within the record are read, converted to host format,
      and are placed into the members of DTT structure. The
      DTT information is displayed to the user and written to 
      a labeled table file.
 
PROJECT:   LAS 5.0
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITHM/DESCRIPTION:

Initialize variables
Read the next physical record from the tape
If an error occurs then
  Output an error message and close tape drive
End if
 
Set the buffer size counter (BUFSIZ)
Set the buffer reference pointer (BUFREF)
 
Convert each of the variables defined in the A record to the host system

If the data cannot be interpreted then
  Output an error message and close tape drive
 
Calculate (XMAX) as the maximum of the X corner coordinates
Calculate (YMAX) as the maximum of the Y corner coordinates
Calculate (XMIN) as the minimum of the X corner coordinates
Calculate (YMIN) as the minimum of the X corner coordinates
 
Calculate (LINES) as the difference of (XMAX) and (XMIN)
Calculate (SAMPS) as the difference of (YMAX) and (YMIN)
 	
Output message to terminal showing output image size
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "util.h"
#include "dementer.h"

void adtt
(
  long *unit,                  /* I: Tape unit number */
  short *buff2,                /* I: Buffer */
  long *bufsiz,                /* O: Current buffer size */
  long *bufref,                /* O: Reference buffer pointer */
  long *lines,                 /* O: Number of input lines */
  long *samps,                 /* O: Number of input samples */
  float *deltay,               /* O: Resolution in the Y direction */
  float *deltaz,               /* O: Resolution in the Z direction */
  double *ymin,                /* O: Minimum value in Y direction */
  struct VLL_DTT *vll_dtt,     /* I: DTT header struct */
  long *outsys,                /* I: Output system */
  FILE *infp                   /* I: Input data file pointer */
)
{ /* adtt  */

    static long fatal = -1;      /*  fatal error code                    */
    static long flag = SYS_USE;  /*  values provided r over/under flow   */
    static long handle = HANDLE; /*  handle flag of tapeio rountines     */
    static long insys = SYS_IBM_MVS; /* input system                     */
    static long lover = MAXLONG; /*  value used if overflow occurs       */
    static long lunder = MINLONG;/*  value used if underflow occurs      */
    static long nonfatal = 1;    /*  nonfatal error code                 */
    static long rewopt = REWOFFL;/*  rewind option                       */
    static long zero = 0;        /*  value of 0                          */
    static short sover = MAXWORD;/*  value if over flow                  */
    static short sunder = MINWORD;/* value if under flow                 */

    double xmax;                 /*  max in X direction                  */
    double xmin;                 /*  min in X direction                  */
    double ymax;                 /*  max in Y direction                  */

    long dtype;                  /*  dtype of values to be converted     */
    long i2byts;                 /*  number of words to be copied        */
    long nbytes;                 /*  number of bytes to be read	         */
    long size;                   /*  size of data type                   */
    long status;                 /*  status of function calls            */

    char msg[ERRLEN];            /*  error message buffer                */

    /*	Read the first record from the tape and copy the first 78 bytes
        into the DTT structure
        ----------------------------------------------------------------------*/
    nbytes = 32768;   /* read an integer multiple of 1024 */
    if (infp == NULL)
    {
        status = c_tpread(unit,(unsigned char *)buff2,&nbytes,&handle);
        if (status < TAPE_SUCC)
        {
            c_errmsg("Error encountered reading tape","dementer-tpread",
                     &nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        else if (status > TAPE_SUCC) 
            *bufsiz = status;
    }
    else
    {
        status = (long)fread(buff2,1,nbytes,infp);
        if (status < 0)
        {
            fclose(infp);
            c_errmsg("Error encountered reading disk", "dementer-fread",
                     &nonfatal);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        else if (status > 0) 
            *bufsiz = status;
    }

    i2byts = 24;
    dtype = EBYTE;
    c_pxcopy((unsigned char *)&buff2[4],(unsigned char *)vll_dtt->sht,&dtype,
            &i2byts);

    /*  Add a null terminator at the end of each of the following strings
        ---------------------------------------------------------------------*/
    vll_dtt->sht[12] = '\0';
    vll_dtt->mps[6] = '\0';
    vll_dtt->mpe[6] = '\0';

    i2byts = 11;
    dtype = EWORD;
    c_pxcopy((unsigned char *)&buff2[16],(unsigned char *)&vll_dtt->mpc,&dtype,
            &i2byts);
    i2byts = 8;
    dtype = ELONG;
    c_pxcopy((unsigned char *)&buff2[27],(unsigned char *)&vll_dtt->losw,&dtype,
            &i2byts);
    *bufsiz = (*bufsiz / 2) - 39 - 4;
    *bufref = *bufref + 39 + 4;

    /*  Convert the values from IBM to host format
        ----------------------------------------------*/
    dtype = EWORD;
    size = 11;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dtt->mpc,&dtype,
            &size,&sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);

        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    dtype = ELONG;
    size = 8;
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dtt->losw,&dtype,
            &size,&lunder,&lover,&flag);
    if (status != E_SUCC)
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Test for valid DTT data
        ---------------------------*/
    if (((vll_dtt->evc != 0) && (vll_dtt->evc != 1)) && (vll_dtt->mpc != 1))
    {
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Data cannot be interpreted in dtt format","dementer-format",
                &nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Convert EBCIDIC strings to ASCII
        -----------------------------------*/
    nbytes = 12;
    c_eb2asc(vll_dtt->sht,&nbytes);
    nbytes = 6;
    c_eb2asc(vll_dtt->mps,&nbytes);
    c_eb2asc(vll_dtt->mpe,&nbytes);

    /*  Calculate the min and max X and Y values of the corners
        -----------------------------------------------------------*/
    xmin = xmax = vll_dtt->xsw;
    *ymin = ymax = vll_dtt->ysw;

    xmax = DEMMAX(xmax,vll_dtt->xse);
    ymax = DEMMAX(ymax,vll_dtt->yse);
    xmin = DEMMIN(xmin,vll_dtt->xse);
    *ymin = DEMMIN(*ymin,vll_dtt->yse);
    xmax = DEMMAX(xmax,vll_dtt->xnw);
    ymax = DEMMAX(ymax,vll_dtt->ynw);
    xmin = DEMMIN(xmin,vll_dtt->xnw);
    *ymin = DEMMIN(*ymin,vll_dtt->ynw);
    xmax = DEMMAX(xmax,vll_dtt->xne);
    ymax = DEMMAX(ymax,vll_dtt->yne);
    xmin = DEMMIN(xmin,vll_dtt->xne);
    *ymin = DEMMIN(*ymin,vll_dtt->yne);

    /*  Set the DTT default values
        ------------------------------*/
    *deltay = 1;
    *deltaz = 1;

    /*  Output the number of lines and samples in the output image.  The number
        of input sample is the number of output lines and the number of input
        lines is the number of output samples because of the rotation of the 
        image as it is read from a tape
        ---------------------------------------------------------------------*/
    *lines = (long)(xmax - xmin) + 1;
    *samps = (long)(ymax - *ymin) + 1;
    sprintf(msg,"Output image will be %d lines by %d samples",*samps, *lines);
    c_errmsg (msg,"informational",&nonfatal);
    c_errmsg ("Output image will contain I*2 data","informational",&nonfatal);
}
