/******************************************************************************
SUBROUTINE BDEMA

PURPOSE:
	Subroutine BDEMA processes DEM type B records in ASCII
     format. The routine reads the data elements in the record 
     and places them into a patch buffer to be passed back to 
     BRECS. The patch buffer holds one 'patch' of data - the data 
     is positioned in the patch buffer in the same manner as it 
     appears in the image.

PROJECT:   LAS

PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITHM/DESCRIPTION:
INITIALIZE the local variables

Initialize the array (PATCH) to the backround value

If the buffer reference pointer (BUFREF) is at the end of a block then
   Reset it to the beginning of the next block.
end if

Read the variable (ROWID) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

Read the variable (COLID) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

If the column identification number from the header information does not 
 match the current line number then
   Print an error message to the user and close tape drive
endif

Read the variable (ROWS) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

Read the variable (COLS) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

Read the variable (Y0) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

Read the variable (DATUM) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

Read the variable (MINE) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

Read the variable (MAXE) from the Ascii buffer
Set the buffer reference pointer (BUFREF)

If the number of rows of data in this patch does not match the 
  number of samples in the image THEN
  Set (INSET) to position the data in the data buffer (PATCH)
end if

For each row in the patch

   If the buffer reference pointer (BUFREF) is at the end of a block then 
      reset it to the beginning of the next block.
   end if

   Read the data element from the Ascii buffer into the data array (PATCH)

   If the buffer reference pointer (BUFREF) is at the end of a block then
      Read the next physical record.
   end if

end for

return
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "pixman.h"
#include "dementer.h"

void bdema
(
  unsigned char *patch,   /* I: Temp buffer */
  long *lin,              /* I: Current line */
  char *record,           /* O: Input buffer */
  long *bufsiz,           /* O: Current buffer size */
  long *bufref,           /* O: Buffer reference pointer */
  long *unit,             /* I: Unit number of tape drive */
  long *samps,            /* I: Total number of samples */
  float *deltay,          /* I: Resolution in the Y direction */
  float *deltaz,          /* I: Resolution in the Z direction */
  double *ymin,           /* I: Minimum corner in y direction */
  long *dtype,            /* I: Data type of input data */
  long *bckgnd,           /* I: Background value */
  FILE *infp              /* I: Input data file pointer */
)
{  /*  bdema  */

    static long fatal = -1;        /*  fatal error code                   */
    static long handle = HANDLE;   /*  handle flag of tapeio rountines    */
    static long nonfatal = 1;      /* nonfatal error code                 */
    static long rewopt = REWOFFL;  /* rewind option                       */
    static long zero = 0;          /*  value of 0                         */

    double y0;		
    double datum;

    float rbckgnd;                  /*  FLOAT background value            */
    float *rpatch;	                /*  float pointer to buffer           */

    long inc;
    long j;                         /* counter                            */
    long k;                         /* counter -- current buffer increment*/

    short colid;                    /*  column id number                  */
    short cols;                     /*  number of columns                 */
    short inset = 0;
    short nbuf;                     /* new buffer logical                 */
    short rowid;                    /* row id number                      */
    short rows;                     /*  number of rows                    */
    short wbckgnd;                  /*  short background value            */
    short *wpatch;                  /*  word pointer to data buffer       */

    char chrtemp[25];               /*  temporary character buffer        */
    char *chrptr;                   /*  temporary character pointer       */

    /*  Initialize the buffer to the backround value
        ------------------------------------------------*/
    inc = (long) (*bufsiz / 1024. + .5);
    if (*deltaz == 1.)
    {
        wpatch = (short *)patch;
        wbckgnd = (short) *bckgnd;
        c_pxfill((unsigned char *)wpatch,dtype,samps,(unsigned char *)&wbckgnd);
    }
    else
    {
        rpatch = (float *)patch;
        rbckgnd = (float) *bckgnd;
        c_pxfill((unsigned char *)rpatch,dtype,samps,(unsigned char *)&rbckgnd);
    }

    /*  Ascii data varies in length but is always in increments of 1024 bytes.
        Set the buffer reference pointer and read the next record
        ---------------------------------------------------------------------*/
    nbuf = TRUE;
    for (j = 1; j <= inc - 1; j++)
    {
        if((*bufref > (1024*(j-1))) && (*bufref <= (1024*j) - 1))
        {
            *bufref = 1024 * j;
            nbuf = FALSE;
            break;
        }
    }

    if (nbuf)
    {
        newbuf(record,bufref,bufsiz,unit,infp);
    }

    /*  Read the header information for this patch
        ----------------------------------------------*/
    sscanf(&record[*bufref],"%hd",&rowid);
    *bufref += 6;

    /*  Abort if the input data is two-dimensional
        ----------------------------------------------*/
    if (rowid != 1)
    {
        c_errmsg("Two dimensional data is not supported","dementer-twodim",
                &nonfatal);
        if(infp == NULL)
            c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Abort if the current line identification number from the tape
        does not match the program's current line counter. If this occurs
        then all further data will be placed in the wrong lines, so abort.
        No LAS I/O has occurred at this point so no back-out is needed.
        ---------------------------------------------------------------------*/
    sscanf(&record[*bufref],"%hd",&colid);
    *bufref += 6;
    if (colid != *lin)
    {
        c_errmsg("Error reading data","dementer-read",&nonfatal);
        if(infp == NULL)
            c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    sscanf(&record[*bufref],"%hd",&rows);
    *bufref += 6;
    sscanf(&record[*bufref],"%hd",&cols);
    *bufref += 6;
    *bufref += 24;

    /*  When the ASCII string represents a double precision number the character
        'D' needs to be replaced with an 'E'.  Sscanf only recognizes the 'E'.
        ----------------------------------------------------------------------*/
    strncpy(chrtemp,&record[*bufref],24);
    chrtemp[24] = ' ';
    chrptr = strchr(chrtemp,'D');
    if (chrptr != NULL)
        *chrptr = 'E';
    sscanf(chrtemp,"%le",&y0);

    *bufref += 24;
    strncpy(chrtemp,&record[*bufref],24);
    chrtemp[24] = ' ';
    chrptr = strchr(chrtemp,'D');
    if (chrptr != NULL)
        *chrptr = 'E';
    sscanf(chrtemp,"%le",&datum);

    /*  Skip over the other header data not needed by this routine
        --------------------------------------------------------------*/
    *bufref += 72;

    if(rows != *samps)
        inset = (short)((y0 - *ymin) / (double)(*deltay));

    /*  Process each element in the data patch
        ------------------------------------------*/
    for (j = 0; j < rows; j++)
    {
        for (k = 1; k <= inc - 1; k++)
            if(*bufref == ((k*1024) - 4))
                *bufref += 4;

        if(*bufref == (*bufsiz - 4))
            newbuf(record,bufref,bufsiz,unit,infp);

        if (*deltaz == 1.)
        {
            wpatch[inset + j] = (short) atol(&record[*bufref]);
            /*
               sscanf(&record[*bufref],"%hd",&wpatch[inset + j]);
             */
            wpatch[inset + j] = (short)(wpatch[inset+ j] + datum);
        }
        else
        {
            sscanf(&record[*bufref],"%f",&rpatch[inset + j]);
            rpatch[inset + j] = rpatch[inset+ j] * *deltaz + datum;
        }
        *bufref += 6;
    }  /*  for j < rows  */
}  /* bdema  */
