/******************************************************************************
SUBROUTINE BDEMB
 
PURPOSE:
 	Subroutine BDEMB processes DEM type B records in Binary
     format. The routine reads the data elements in the record 
     and places them into a patch buffer to be passed back to 
     BRECS. The patch buffer holds one 'patch' of data - the data 
     is positioned in the patch buffer in the same manner as it 
     appears in the image.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITHM DESCRIPTION:
Initialize the local variables

For the number of integer*2 words in the header record
  Read the next record block if needed
  Set each local variable as appropriate
  Increment the buffer position pointer
  Decrement the buffer size counter
end for
 
Convert each of the variables to the host system

If the column identification number from the header information
 does not match the current line number then
   Output an error message to the user and close the tape
endif

For each data element in the record
  Read the next record block if needed
  Read and convert the bytes in the element
       and place it into the patch buffer
  Increment the buffer position pointer
  Decrement the buffer size counter
end for
return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "dementer.h"

void bdemb
(
  unsigned char *patch,   /* I: Temp buffer */
  long *lin,              /* I: Current line */
  short *buff2,           /* I: Input buffer */
  long *bufsiz,           /* O: Current buffer size */
  long *bufref,           /* O: Buffer reference pointer */
  long *unit,             /* I: Unit number of tape drive */
  long *samps,            /* I: Total number of samples */
  float *deltay,          /* I: Resolution in the Y direction */
  float *deltaz,          /* I: Resolution in the Z direction */
  double *ymin,           /* I: Minimum corner in Y direction */
  long *dtype,            /* I: Data type of input data */
  long *size,             /* I: Size of data type */
  long *outsys,           /* I: Output system */
  long *bckgnd,           /* I: Background value */
  FILE *infp              /* I: Input data file pointer */
)
{  /*  bdemb  */

    static long fatal = -1;       /*  fatal error code                  */
    static long flag = SYS_USE;   /*  values provided r over/under flow */
    static long handle = HANDLE;  /*  handle flag of tapeio rountines   */
    static long insys = SYS_IBM_MVS; /* input system                    */
    static long nonfatal = 1;     /*  nonfatal error code               */
    static long rewopt = REWOFFL; /* rewind option                      */
    static long zero = 0;         /*  value of 0                        */
    static short sover = MAXWORD; /*  value if over flow                */
    static short sunder = MINWORD;/* value if under flow                */

    double y0;		
    double datum;

    float rbckgnd;	              /*  FLOAT backround value             */

    long j;		                  /* counter                            */
    long nbytes = 32768;	      /* nbytes to read                     */
    long pxdtype;                 /* data type of variables to be converted*/
    long elsize;                  /* element size                       */
    long status;                  /* status of function calls           */

    short colid;                  /*  column id number                  */
    short inset = 0;
    short rowid;                  /* row id number                      */
    short rows;                   /*  number of rows                    */
    short *ty0;                   /* pointer to variable y0             */
    short *tdatum;                /* pointer to variable datum          */
    short wbckgnd;                /* short background value             */

    /* Initialization of variables
       ------------------------------*/
    ty0 = (short *)&y0;
    tdatum = (short *)&datum;

    /*  Fill in temporary buffer  
        ----------------------------*/
    if (*deltaz == 1.)
    {
        wbckgnd = (short) *bckgnd;
        c_pxfill(patch,dtype,samps,(unsigned char *)&wbckgnd);
    }
    else
    {
        rbckgnd = (float) *bckgnd;
        c_pxfill(patch,dtype,samps,(unsigned char *)&rbckgnd);
    }

    /*  Read the header information for this patch
        ----------------------------------------------*/
    for (j = 1; j <= 24; j++)
    {
        if(*bufsiz == 0)
        {
            if(infp == NULL)
            {
                status = c_tpread(unit,(unsigned char *)buff2,&nbytes,&handle);
                if (status < 0)
                {
                    c_errmsg("Error encountered reading tape","dementer-tpread",
                            &nonfatal);
                    status = c_tpclos(unit,&handle,&rewopt,&zero);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                else
                    if (status > 0) 
                        *bufsiz = status;
            }
            else
            {
                status = (long)fread(buff2,1,nbytes,infp);
                if (status < 0)
                {
                    fclose(infp);
                    c_errmsg("Error encountered reading disk","dementer-fread",
                            &nonfatal);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                else
                    if (status > 0) 
                        *bufsiz = status;
            }

            *bufsiz = (*bufsiz / 2) - 4;
            *bufref = 4;
        }

        if(j == 1) rowid     = buff2[*bufref];
        if(j == 2) colid     = buff2[*bufref];
        if(j == 3) rows      = buff2[*bufref];
        if(j == 9)  *ty0 = buff2[*bufref];
        if(j == 10) *(ty0 + 1) = buff2[*bufref];
        if(j == 11) *(ty0 + 2) = buff2[*bufref];
        if(j == 12) *(ty0 + 3) = buff2[*bufref];
        if(j == 13) *tdatum = buff2[*bufref];
        if(j == 14) *(tdatum + 1) = buff2[*bufref];
        if(j == 15) *(tdatum + 2) = buff2[*bufref];
        if(j == 16) *(tdatum + 3) = buff2[*bufref];
        *bufref = *bufref + 1;
        *bufsiz = *bufsiz - 1;
    }  /*  for j < 24  */

    /*  Convert the values if needed
        --------------------------------*/
    elsize = 1;
    pxdtype = EWORD;
    status = c_pxsys(&insys,outsys,(unsigned char *)&rowid,&pxdtype,&elsize,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
    status = c_pxsys(&insys,outsys,(unsigned char *)&colid,&pxdtype,&elsize,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
    status = c_pxsys(&insys,outsys,(unsigned char *)&rows,&pxdtype,&elsize,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
    pxdtype = EDOUBLE;
    status = c_pxsys(&insys,outsys,(unsigned char *)&y0,&pxdtype,&elsize,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
    status = c_pxsys(&insys,outsys,(unsigned char *)&datum,&pxdtype,&elsize,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Abort if the input data is two-dimensional
        ----------------------------------------------*/
    if (rowid != 1)
    {
        c_errmsg("Two dimensional data is not supported","dementer-twodim",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Abort if the current line identification number from the tape
        does not match the program's current line counter. If this occurs
        then all further data will be placed in the wrong lines, so abort.
        No LAS I/O has occurred at this point so no clean-up is needed.
        -------------------------------------------------------------------*/
    if (colid != *lin)
    {
        c_errmsg("Error reading data","dementer-read",&nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    if(rows != *samps) 
        inset = (short)((y0 - *ymin) / (double) *deltay); 

    /*  Process each element in the patch
        -------------------------------------*/
    for (j = 0; j < rows; j++)
    {
        if (*bufsiz == 0)
        {
            nbytes = 32768;
            if(infp == NULL)
            {
                status = c_tpread(unit,(unsigned char *)buff2,&nbytes,&handle);
                if (status < 0)
                {
                    c_errmsg("Error encountered reading tape","dementer-tpread",
                            &nonfatal);
                    status = c_tpclos(unit,&handle,&rewopt,&zero);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                else
                    if (status > 0)
                        *bufsiz = status;
            }
            else
            {
                status = (long)fread(buff2,1,nbytes,infp);
                if (status < 0)
                {
                    fclose(infp);
                    c_errmsg("Error encountered reading disk","dementer-fread",
                            &nonfatal);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                else
                    if (status > 0)
                        *bufsiz = status;
            }
            *bufsiz = (*bufsiz / 2) - 4;
            *bufref = 4;
        }

        /* Convert each pixel to the correct output system format
           ---------------------------------------------------------*/
        pxdtype = EWORD;
        status = c_pxsys(&insys,outsys,(unsigned char *)&buff2[*bufref],
                &pxdtype,&elsize,&sunder,&sover, &flag);
        if (status != E_SUCC)
        {
            c_errmsg("Error encountered converting data","dementer-convert",
                    &nonfatal);
            if(infp == NULL)
                status = c_tpclos(unit,&handle,&rewopt,&zero);
            else
                fclose(infp);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }

        if (*deltaz == 1.)
            (*(short *)(patch + (inset + j) * *size)) = 
                (short)(buff2[*bufref] + datum);
        else
            (*(float *)(patch + (inset + j)  * *size)) = 
                (float)(buff2[*bufref] * *deltaz + datum);

        *bufref = *bufref + 1;
        *bufsiz = *bufsiz - 1;
    }  /*  for j < rows  */
}  /*  bdemb  */
