/******************************************************************************
SUBROUTINE BDTED
 
PURPOSE:
 	Subroutine BDTED processes DTED type B records in Binary
      format. The routine reads and converts the data elements in 
      the record and places them into a patch buffer to be passed 
      back to BRECS. The patch buffer holds one 'patch' of data - 
      the data is positioned in the patch buffer in the same manner 
      as it appears in the image.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 5.1 07/11/90  D. Etrheim	CSB           Change to read in multiple files
					      of the "new" tape format
 
ALGORITHM/DESCRIPTION
Initialize the local variables

Read the next record block 
Set each local variable as appropriate
Increment the buffer position pointer

For each data element in the record
   Read and convert the data and place it into the patch buffer
   Increment the buffer position pointer
end for

return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "dementer.h"

void bdted
(
  short *patch,        /* O: Patch buffer */
  short *buff2,        /* I: Input buffer */
  long *samps,         /* I: Number of input samples */
  long *unit,          /* I: Tape unit number */
  long *bufsz,         /* O: Current buffer size */
  long *bufref,        /* O: Reference buffer pointer */
  long *dtype,         /* I: Data type of input data */
  long *outsys,        /* I: Output system */
  long *bakgnd,        /* I: Background value */
  FILE *infp,          /* I: Input data file pointer */
  long curpatch        /* I: Current patch being processed */
)
{  /*  bdted  */

    static long fatal = -1;       /*  fatal error code                    */
    static long handle = HANDLE;  /*  handle flag of tapeio rountines     */
    static long flag = SYS_USE;   /*  values provided r over/under flow   */
    static long insys = SYS_IBM_MVS; /* input system                      */
    static long nonfatal = 1;     /*  nonfatal error code                 */
    static long rewopt = REWOFFL; /*  rewind option                       */
    static long zero = 0;         /*  value of 0                          */
    static short sover = MAXWORD; /*  value if over flow                  */
    static short sunder = MINWORD;/*  value if under flow                 */

    long j;                       /*  counter --                          */
    long ns;                      /*  number of samples                   */
    long size = 30744;            /*  size of tape read                   */
    long status;                  /*  status of function calls            */

    /*  Fill patch with background value
        ------------------------------------*/
    ns = 1201;
    c_pxfill((unsigned char *)patch,dtype,&ns,(unsigned char *)bakgnd);

    if(infp == NULL)
    {
        status = c_tpread(unit,(unsigned char *)buff2,&size,&handle);
        if(status < 0)
        {
            c_errmsg("Error encountered reading tape","dementer-tpread",
                      &nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
    }
    else
    {
        /* read an amount equal to one data record */
        if(curpatch == 0)
            size = DTED_DSI_LEN;
        else 
            size = (long)(12 + sizeof(short) * (*samps));
        if(size != (status = (long)fread(buff2,1,size,infp)))
        {
            c_errmsg("Error encountered reading disk","dementer-fread", 
                    &nonfatal);
            fclose(infp);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
    }

    /* If first record was the Data Set Identification Record, read the 
       Accuracy Description Record and then read the first data record. 
       ------------------------------------------------------------------*/
    if (strncmp((const char *)buff2,"DSI",3) == 0 && curpatch == 0)
    {
        if(infp == NULL)
        {
            status = c_tpread(unit,(unsigned char *)buff2,&size,&handle);
            status = c_tpread(unit,(unsigned char *)buff2,&size,&handle);
            if(status < 0)
            {
                c_errmsg("Error encountered reading tape","dementer-tpread",
                        &nonfatal);
                status = c_tpclos(unit,&handle,&rewopt,&zero);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }
        }
        else
        {
            if(DTED_ACC_LEN!=(status = (long)fread(buff2,1,DTED_ACC_LEN,infp)))
            {
                c_errmsg("Error encountered reading disk","dementer-fread", 
                        &nonfatal);
                fclose(infp);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }
            size = (long)(12 + sizeof(short) * (*samps));
            if(size != (status = (long)fread(buff2,1,size,infp)))
            {
                c_errmsg("Error encountered reading disk","dementer-fread", 
                        &nonfatal);
                fclose(infp);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }

        }
    }

    if(status >= 0)
        *bufsz = status;

    *bufref = 4;

    size = 1;

    /*  for each sample convert and place in the patch buffer
        ---------------------------------------------------------*/
    for (j = 0; j < *samps; j++)
    {
        status = c_pxsys(&insys,outsys,(unsigned char *)&buff2[*bufref],dtype,
                &size,&sunder,&sover,&flag);
        if (status != E_SUCC)
        {
            c_errmsg("Error encountered converting data", "dementer-convert",
                    &nonfatal);
            if(infp == NULL)
                status = c_tpclos(unit,&handle,&rewopt,&zero);
            else
                fclose(infp);

            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        patch[j] = buff2[*bufref]; 
        *bufref += 1;
    }
}  /*  bdted  */
