/******************************************************************************
SUBROUTINE BDTT
 
PURPOSE:
 	Subroutine BDTT processes DTT type B records in Binary
      format. The routine reads, converts, and converts the 
      data elements in the record and places them into a patch
      buffer to be passed back to BRECS. The patch buffer holds
      one 'patch' of data - the data is positioned in the patch
      buffer in the same manner as it appears in the image.

PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITHM/DESCRIPTION:

Initialize the local variables

For the number of integer*2 words in the header record
  Read the next record block if needed
  Place each local variable as appropriate
  Increment the buffer position pointer
  Decrement the buffer size counter
end for
 
For each data element in the record
  Read the next record block if needed
  Convert the bytes in the element and place it into the patch buffer
  Increment the buffer position pointer
  Decrement the buffer size counter
end for
 
return
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "dementer.h"

void bdtt
(
  unsigned char *patch,    /* O: Patch buffer */
  short *buff2,            /* I: Input buffer */
  long *bufsiz,            /* O: Current buffer size */
  long *bufref,            /* O: Reference buffer pointer */
  long *unit,              /* I: Tape unit number */
  long *samps,             /* I: Number of samples */
  float *deltay,           /* I: Resolution in the Y direction */
  double *ymin,            /* I: Minimum Y value */
  long *outsys,            /* I: Output system */
  FILE *infp               /* I: Input data file pointer */
)
{  /*  bdtt  */

    static long fatal = -1;       /*  fatal error code                   */
    static long flag = SYS_USE;   /*  values provided r over/under flow  */
    static long handle = HANDLE;  /* handle flag of tapeio rountines     */
    static long insys = SYS_IBM_MVS; /* input system                     */
    static long nonfatal = 1;     /*  nonfatal error code                */
    static long rewopt = REWOFFL; /* rewind option                       */
    static long zero = 0;         /*  value of 0                         */
    static short sover = MAXWORD; /*  value if over flow                 */
    static short sunder = MINWORD;/* value if under flow                 */

    long dtype = EWORD;             /*  data type of input data          */
    long j;                         /*  counter -- current               */
    long k;                         /*  counter -- current               */
    long size = 1;                  /*  size of input array              */
    long status;                    /*  status of function calls         */
    long tsize = 32768;             /*  size of tape read                */

    short inset;
    short rows;                     /*  number of rows in patch          */
    short *wpatch;                  /*  SHORT pointer to patch buffer    */
    short y0;

    /*  Initialize the variables
        ----------------------------*/
    wpatch = (short *) patch;
    inset = 0; 
    c_pxfill((unsigned char *)wpatch,&dtype,samps,(unsigned char *)&inset);

    /*  Read the header information
        -------------------------------*/
    for (j = 1; j <= 5; j++)
    {
        if(*bufsiz == 0)
        {
            if(infp == NULL)
            {
                status = c_tpread(unit,(unsigned char *)buff2,&tsize,&handle);
                if(status < 0)
                {
                    c_errmsg("Error encountered reading tape","dementer-tpread",
                            &nonfatal);
                    status = c_tpclos(unit,&handle,&rewopt,&zero);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                if(status >= 0)
                    *bufsiz = status;
            }
            else
            {
                status = (long)fread(buff2,1,tsize,infp);
                if(status < 0)
                {
                    fclose(infp);
                    c_errmsg("Error encountered reading disk","dementer-fread",
                            &nonfatal);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                if(status >= 0)
                    *bufsiz = status;
            }

            *bufsiz = (*bufsiz / 2) - 4;
            *bufref = 4;
        }  /*  if *bufsiz == 0  */

        if (j == 2)  y0 =    buff2[*bufref];
        if (j == 5)  rows  = buff2[*bufref];
        *bufref = *bufref + 1;
        *bufsiz = *bufsiz - 1;

    }  /*  for j < 5  */

    /*  Convert the values from IBM to host format
        ----------------------------------------------*/
    dtype = EWORD;
    status = c_pxsys(&insys,outsys,(unsigned char *)&y0,&dtype,&size,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
    status = c_pxsys(&insys,outsys,(unsigned char *)&rows,&dtype,&size,
            &sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if(infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    if (rows != *samps)
        inset = (long)(((double)(y0) - *ymin) / (double)(*deltay));

    /*  For each element in the patch, convert, and update the 
        minimum and maximum pixel values
        -----------------------------------------------------------*/
    for (k = 0; k < rows; k++)
    {
        if (*bufsiz == 0)
        {
            if(infp == NULL)
            {
                status = c_tpread(unit,(unsigned char *)buff2,&tsize,&handle);
                if(status < 0)
                {
                    c_errmsg("Error encountered reading tape","dementer-tpread",
                            &nonfatal);
                    status = c_tpclos(unit,&handle,&rewopt,&zero);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                else if(status >= 0)
                    *bufsiz = status;
            }
            else
            {
                status = (long)fread(buff2,1,tsize,infp);
                if(status < 0)
                {
                    fclose(infp);
                    c_errmsg("Error encountered reading disk","dementer-fread",
                            &nonfatal);
                    c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
                }
                else if(status >= 0)
                    *bufsiz = status;
            }

            *bufsiz = (*bufsiz / 2) - 4;
            *bufref = 4;
        }  /*  if *bufsiz ==0  */

        status = c_pxsys(&insys,outsys,(unsigned char *)&buff2[*bufref],
                &dtype,&size,&sunder,&sover,&flag);
        if (status != E_SUCC)
        {
            c_errmsg("Error encountered converting data","dementer-convert",
                    &nonfatal);
            if(infp == NULL)
                status = c_tpclos(unit,&handle,&rewopt,&zero);
            else
                fclose(infp);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        wpatch[inset + k] = buff2[*bufref]; 

        *bufref += 1;
        *bufsiz -= 1;

    }  /*  for k < rows */
}  /*  bdtt  */
