/******************************************************************************
SUBROUTINE BRECS
 
PURPOSE:
        Subroutine BRECS process the type B records from the
      Elevation Model tapes. The B records consist of a header
      section containing information about the data in the record,
      and also holds the actual data records. The records are read,
      converted (if binary), and read into the LAS output image. 
      Any data record that exceeds the bounds of integer*2 data is 
      set to MAXWORD - no error or condition message is printed.
 
 	Physical I/O reads are performed as needed throughout the
      processing of the type B records. Each read command reads a
      physical block of data from the tape. Since the input data is
      variably blocked, a new READ can occur anywhere. This program 
      tests the need for a new block at two-byte intervals because the
      data is integer*2. It is conceivable that the program would fail
      if the block read occured on a single byte interval - since 
      testing for this rare condition limits the already slow processing
      speed of the program, the test is omitted.
 
PROJECT:   LAS

PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 5.1 8-NOV-89  P. Pease         GSFC          changed call to c_startm from
                                              startm and c_stoptm from stoptm.
 
ALGORITHM DESCRIPTION:
 
for each line in the image
 
   (For Binary DEM or DTT data)
   If (bufsiz) is not zero then
      Increment the buffer position counter
      Decrement the buffer size counter
   Endif
 
   If (FORM) specifies DTT data then
      Call BDTT to process this data line
   Elseif (FORM) specifies DEM Acsii data Then
      Call BDEMA to process this data line
   Elseif (FORM) specifies DEM Binary data Then
      Call BDEMB to process this data line
   Elseif (FORM) specifies DTED data Then
      Call BDTED to process this data line
   Endif
 
   For each sample in the line
       Set the element in TEMP equal to the next element
       Update the minimum and maximum pixel values
   End for
 
 end for
 
Call SETIN to write the data into the LAS image

return
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "util.h"
#include "dementer.h"

void brecs
(
  short *buff2,         /* I: Input buffer */
  long *unit,           /* I: Tape unit number */
  short *form,          /* I: Form of the DEM data */
  long *lines,          /* I: Total number of input lines */
  long *samps,          /* I: Total number of input samples */
  long *bufsiz,         /* I/O: Current buffer size */
  long *bufref,         /* O: Buffer reference pointer */
  char *hosout,         /* O: Output host file name */
  float *deltay,        /* I: Resolution in the Y direction */
  float *deltaz,        /* I: Resolution in the Z direction */
  double *ymin,         /* I: Minimum corner in Y direction */
  long *msgtim,         /* I: Time between display messages */
  long *bckgnd,         /* I: Background value */
  long *outsys,         /* I: Output system */
  char *mmflag,         /* I: Min/Max flag */
  FILE *infp            /* I: Input data file pointer */
)
{  /*  brecs  */
    static long one = 1; /*  value of 1                                 */

    double minele;       /* minimum element value                       */
    double maxele;       /* maximum element value                       */

    float fbckgnd;       /*  float background value                     */
    float *fdata;        /* float pointer                               */
    float *ftemp;        /* float pointer to temp buffer                */

    long acc = GEN2;     /* type of access                              */
    extern long curent;  /* current line                                */
    long dtype;          /* data type of input data                     */
    long j;              /*  counter -- current line                    */
    long k;              /*  counter -- current sample                  */
    long size;           /* size of datatype in bytes                   */

    float wbckgnd;       /* word background value                       */
    short *wdata;        /* word pointer                                */
    short *wtemp;        /* word pointer to temp buffer                 */

    unsigned char *data; /* data buffer before rotation                 */
    unsigned char  *temp;/* final data buffer                           */

    char *record;	/*  */

    /*  Set data type and size
        --------------------------*/
    if (*deltaz == 1.)
    {
        dtype = EWORD;
        size = 2;
        minele = MAXWORD;
        maxele = MINWORD;
    }
    else
    {
        dtype = EREAL;
        size = 4;
        minele = MAXREAL;
        maxele = MINREAL;
    }

    /*  Allocate buffer space
        -------------------------*/
    temp = (unsigned char *)malloc(*lines * *samps * size);
    data = (unsigned char *)malloc(*samps * size);

    /*  fill the buffer with the background value
        ---------------------------------------------*/
    if (*deltaz == 1.)
    {
        wtemp = (short *)temp;
        wdata = (short *)data;
        wbckgnd = (short) *bckgnd;
        for (j = 0; j < *lines; j++)
            c_pxfill((wtemp + (j * *samps)),&dtype,samps,&wbckgnd);
    }
    else
    {
        ftemp = (float *)temp;
        fdata = (float *)data;
        fbckgnd = (float) *bckgnd;
        for (j = 0; j < *lines; j++)
            c_pxfill((ftemp + (j * *samps)),&dtype,samps,&fbckgnd);
    }

    /*  For each patch of data in the image
        ---------------------------------------*/
    c_inittm("DEMENTER","Processed","profile"," "," ",lines,&one,msgtim,&acc);
    curent = 1;
    c_startm();
    for (j = 0,curent = 1; j < *lines; j++,curent++)
    {
        if((*bufsiz != 0) && ((*form == 1) || (*form == 22)))
        {
            *bufref = *bufref + 2;
            *bufsiz = *bufsiz - 2;
        }

        /*  Get the next 'B' record for DTT, DEM Binary, or DEM Ascii data
            ------------------------------------------------------------------*/
        if (*form == 1)
            bdtt(data,buff2,bufsiz,bufref,unit,samps,deltay,ymin,outsys,infp);
        else if (*form == 22)
            bdemb(data,&curent,buff2,bufsiz,bufref,unit,
                    samps,deltay,deltaz,ymin,&dtype,&size,outsys,bckgnd,infp);
        else if (*form == 23)
        {
            record = (char *)buff2;
            bdema(data,&curent,record,bufsiz,bufref,unit,
                    samps,deltay,deltaz,ymin,&dtype,bckgnd,infp);
        }
        else
            bdted(wdata,buff2,samps,unit,bufsiz,bufref,&dtype,outsys,
                  bckgnd,infp,j);

        /*  Place the input values into the correct array 
            Calculate the min and max values if specified
            -------------------------------------------------*/
        if (strncmp(mmflag,"YES",3) != 0)
        {
            /*  if 2 byte data type cast to short else type cast to real
                ------------------------------------------------------------*/
            if (*deltaz == 1.)
                for (k = 0; k < *samps; k++)
                    *(wtemp + ((curent * *samps) - k - 1)) = *(wdata + k); 
            else
                for (k = 0; k < *samps; k++)
                    *(ftemp + ((curent * *samps) - k - 1)) = *(fdata + k); 
        }
        else
        { /*  calculate the min and max values  */
            /*  if 2 byte data type cast to short else type cast to real
                ------------------------------------------------------------*/
            if (*deltaz == 1.)
                for (k = 0; k < *samps; k++)
                {
                    *(wtemp + ((curent * *samps) - k - 1)) = *(wdata + k); 
                    if ((double) *(wdata + k) < minele) 
                        minele = (double) *(wdata + k); 
                    if ((double) *(wdata + k) > maxele) 
                        maxele = (double) *(wdata + k); 
                }
            else
                for (k = 0; k < *samps; k++)
                {
                    *(ftemp + ((curent * *samps) - k - 1)) = *(fdata + k); 
                    if ((double) *(fdata + k) < minele) 
                        minele = (double) *(fdata + k); 
                    if ((double) *(fdata + k) > maxele) 
                        maxele = (double) *(fdata + k); 
                }
        }  /*  else calculate the min and max values  */
    }
    c_stoptm();

    /*  Create a LAS image from the input data
        ------------------------------------------*/
    setin(hosout,&minele,&maxele,lines,samps,temp,mmflag,msgtim,&dtype);
}  /*  brecs  */
