/******************************************************************************
SUBROUTINE DDRDEM
 
PURPOSE:
		To fill the individual fields of the DDR for DEM data
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
 5.0 05/31/89  B. Ailts		CSB	      Original development
 5.1 07/11/90  D. Etrheim	CSB           ADP #5489 Not packing the ddr
					      correctly.  Changed proj_units 
					      to "seconds"
 5.2 01/17/90  D. Etrheim	CSB           Make zone and datum codes valid
 
ALGORITHM DESCRIPTION:
Retrieve the DDR information
Retrieve the DDR band information

Place  the values from the DEM structure into the DDR
If Data is 1:24000
   Calculate the corner coordinates
else
   Copy the corner coordinates from the DEM structure

Fill in the DDR band information

Write the DDR information
Write the DDR band information
return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "cproj.h"
#include "util.h"
#include "geompak.h"
#include "dementer.h"

void ddrdem
(
  char *hosout,               /* I: Host output name */
  struct VLL_DEM *vll_dem,    /* I: DEM header struct */
  long *lines,                /* I: Number of lines in input image */
  long *samps,                /* I: Number of samples in input image */
  long *ddrvflg               /* I: DDR validity flag */
)
{  /* ddrdem  */

    static long one = 1;            /*  value of 1                    */
    extern struct PARBLK vblock;	/*  TAE parameter block           */
    struct DDR *ddr;                /*  DDR structure                 */
    struct BDDR *bddr;              /*  BDDR structure                */ 

    double ymax;                    /*  maximum Y corner              */
    double xmin;                    /*  maximum X corner              */

    long i;                         /*  counter --                    */
    long status;                    /*  status of function calls      */

    /*  Allocate space for the DDR structures
        -----------------------------------------*/
    ddr = (struct DDR *) malloc(sizeof(struct DDR));
    bddr = (struct BDDR *) malloc(sizeof(struct BDDR));

    /*  Retrieve DDR information
        ----------------------------*/
    status = c_getddr(hosout,ddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    status = c_getbdr(hosout,bddr,&one);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    ddr->master_line = 1;
    ddr->master_sample = 1;

    /*  Set projection units
        ------------------------*/
    ddr->valid[DDPUV] = VALID;
    if (c_unit2string(vll_dem->puni,ddr->proj_units) != E_SUCC)
    {
        /* c_unit2string displays the error message */
        c_lasmsg(FALSE,"dementer-warn", "Setting units to invalid");
        ddr->valid[DDPUV] = *ddrvflg;
        strcpy(ddr->proj_units," ");
    }

    ddr->valid[DDPCV] = VALID;
    ddr->proj_code = vll_dem->gref;

    ddr->valid[DDDCV] = VALID;
    ddr->datum_code = 0;

    ddr->valid[DDZCV] = VALID;
    ddr->zone_code = vll_dem->zref;

    ddr->valid[DDPDV] = VALID;
    ddr->pdist_y = (double) vll_dem->yres;
    ddr->pdist_x = (double) vll_dem->xres;

    ddr->valid[DDPPV] = VALID;
    for (i = 0; i < 15; i ++)
        ddr->proj_coef[i] = vll_dem->ppar[i];

    /*  If the data is 1:24000 7.5 min quad the corners of the output image must
        be calculated from the max y coordinate and the min x coordinate.
        If the data is 1:125000 quad the corners of the output image are exactly
        the same as the corners in the A type record.
        It still needs to fiquired out if the projection coordinates are in the 
        center of the pixel or not.
        ----------------------------------------------------------------------*/
    ddr->valid[DDCCV] = VALID;
    if ((vll_dem->lvl == 1) || (vll_dem->lvl == 2))
    {
        ymax = vll_dem->corner[0][1];
        ymax = DEMMAX(ymax,vll_dem->corner[1][1]);
        ymax = DEMMAX(ymax,vll_dem->corner[2][1]);
        ymax = DEMMAX(ymax,vll_dem->corner[3][1]);
        xmin = vll_dem->corner[0][0];
        xmin = DEMMIN(xmin,vll_dem->corner[1][0]);
        xmin = DEMMIN(xmin,vll_dem->corner[2][0]);
        xmin = DEMMIN(xmin,vll_dem->corner[3][0]);

        /*  The actual corners of the data start at even increment of the x
            and y resolution
            -----------------------------------------------------------------*/
        ymax = ((long)(ymax / (double)vll_dem->yres)) * (double)vll_dem->yres;
        xmin = ((long)(xmin / (double)vll_dem->xres) + 1)*(double)vll_dem->xres;

        ddr->upleft[0] = ymax;
        ddr->upleft[1] = xmin;
        ddr->upright[0] = ymax;
        ddr->upright[1] =(vll_dem->xres * (float)(*lines - 1)) + ddr->upleft[1];
        ddr->loleft[0] = ddr->upleft[0] - (vll_dem->yres * (float)(*samps - 1));
        ddr->loleft[1] = xmin;
        ddr->loright[0] = ddr->loleft[0];
        ddr->loright[1] = ddr->upright[1];
    }
    else
    {
        ddr->upleft[0] = vll_dem->corner[1][1];
        ddr->upleft[1] = vll_dem->corner[1][0];
        ddr->loleft[0] = vll_dem->corner[0][1];
        ddr->loleft[1] = vll_dem->corner[0][0];
        ddr->upright[0] = vll_dem->corner[2][1];
        ddr->upright[1] = vll_dem->corner[2][0];
        ddr->loright[0] = vll_dem->corner[3][1];
        ddr->loright[1] = vll_dem->corner[3][0];
    }

    ddr->valid[DDINCV] = VALID;
    ddr->line_inc = 1.0;
    ddr->sample_inc = 1.0;

    if ((vll_dem->lvl == 1) || (vll_dem->lvl == 2))
        strcpy(bddr->source,"DEM 1:24000");
    else if (vll_dem->lvl == 3)
        strcpy(bddr->source,"DEM 1:250000");

    status = c_putddr(hosout,ddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    status = c_putbdr(hosout,bddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    free(ddr);
    free(bddr);
}  /*  ddrdem  */
