/******************************************************************************
PROGRAM DEMENTER
 
PURPOSE:
 	The program DEMENTER reads Digital Elevation Model data
      from magnetic tapes in either IBM variable block or variable
      block spanned formats that contain either Binary or Ascii data.
      Currently the data elements in the file are converted to host format, 
      which includes byte swapping within the data elements, and converting
      the data from IBM fixed binary(15) and fixed binary(31) into
      the host system REAL*4 and doubleprecision formats respectively. 
 
        Since all data produced to date consists of a regular elevation
      pattern where the array of patches is one row high and each patch
      is one sample wide, only this type of data is supported by DEMENTER.
      Since no variance from this input format is anticipated, random 
      elevation patterns and two-dimensional patches are not supported.
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 5.1 07/11/90  D. Etrheim	CSB	      ADP #5489 Not packing the ddr
					      correctly. ADP #5492  Increase
					      limit on number of files to read.
					      Trap status of c_setprt. Change
					      proj_units to "seconds". Changed
					      to read in multiple files of the
					      "new" DTED tape format.
 5.2 01/17/90  D. Etrheim	CSB           Make zone and datum codes valid	
 5.3 12/13/91  D. Etrheim       CSB           Correct the corner coordinates for
					      DTED tape.
 5.4 08/26/92  D. Etrheim	CSB           Write module name to history file
 5.5 01/08/93  D. Etrheim	CSB           Correct problem in tapeset where
					      device was returning TAPE_BOT 
					      after c_tpbsr.
 5.6 03/03/93  D. Etrheim       CSB           Allow for Mexican DTED PRF #6232
 5.7 08/23/93  K. Sheffield     CSB           Added option to read data from
                                              a disk file.

 PROJECT:   LAS
 
 ALGORITHIM DESCRIPTION:

Call GETPAR to receive the input parameter
Call TAPESET to determine the tape format
 
If (FORM) specifies 'DTT' data then
  Call ADTT to process the DTT type A record
Elseif (FORM) specifies 'DEM' data then
  Call ADEM to process the DEM type A record
Else
  Call ADTED to process the DTED type A record
end if
 
Call BRECS to process type B records
 
If (FORM) specifies 'DTT' data then
  Call CDTT to process the DTT type C record
Else If type C record exists then
    If (FORM) specifies 'DEM' Binary then
      Call CDEMB to process type C record in Binary format
    Else if (FORM) specifies 'DEM' Ascii then
      Call CDEMA to process type C record in Ascii format
    end if
END IF

If LAST file to be read then
    Call TPCLOS to close the input tape
else
    Call TPREAD to read to tape eof

If DTT data
   Display the DTT header data
   Update the DDR fields from the DTT data
   If specified
      Write out a DTT labeled table file
Else if DEM data
   Display the DEM header data
   Update the DDR fields from the DEM data
   If specified
      Write out a DEM labeled table file
Else
   Display the DTED header data
   Update the DDR fields from the DTED data
   If specified
      Write out a DTED labeled table file
Endif

Call HSTOUT to generate a history label for the image

Call COMPLT to write a standard completion message to the user

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "dementer.h"

long fdesc[2];		/* file descriptor			*/
long gdesc;		/* group descriptor			*/
long offset[2];		/* offsets into buffer array		*/
extern long curent;	/* curent line				*/
struct PARBLK vblock;	/* TAE parameter block			*/
char hosin[CMLEN];	/*  input host file name		*/

void main
(
  void
)
{  /* dementer  */


    FILE *optr;                     /* file descriptor of the print file*/
    FILE *infp=NULL;                /* input data file descriptor*/

    struct VLL_DEM *vll_dem;        /* pointer to DEM structure	*/
    struct VLL_DTED *vll_dted;      /* pointer to DTED structure*/
    struct VLL_DTT *vll_dtt;        /* pointer to DTT structure	*/

    double ymin;                    /* minimum Y corrdinate	*/

    float deltaz;                   /* resolution in vertical direction	*/
    float deltay;                   /* resolution in Y direction*/

    long bckgnd;                    /* background value	                */
    long bufref;                    /* reference buffer pointer	        */
    long bufsiz;                    /*  current buffer size	            */
    long ddrvflg;                   /*  DDR validity flag               */
    long diskflg;                   /*  data is on disk flag            */
    long file;                      /*  flag if file is specified       */
    long fnum;                      /*  file number to be read from tape*/
    long ibytes;                    /*  number of WORDs to be copied    */
    long lines;                     /*  number of input lines           */
    long lp;                        /*  flag if line printer is specified*/
    long msgtim;                    /*  time between display messages   */
    long nfile = 1;                 /*  number of output files	        */
    long outltf;                    /*  flag if labeled table is generated */
    long outsys;                    /*  output system                   */
    long samps;                     /*  number of input samples         */
    long status;                    /*  status of function calls        */
    long std_out;                   /*  Standard output	                */
    long term;                      /*  flag if terminal is specified   */
    long unit;                      /*  tape unit number                */
    static long fatal = -1;         /*  fatal error code                */
    static long handle = HANDLE;    /*  handle flag of tapeio rountines */
    static long nonfatal = 1;       /*  nonfatal error code             */
    static long rewopt = REWOFFL;   /*  rewind option                   */
    static long zero = 0;           /*  value of 0                      */
    short *buff2;                   /*  input buffer			        */
    short cdata;
    short form;                     /*  type of input format            */
    short first;                    /*  first file on tape              */
    short last;                     /*  last file on tape               */

    char drive[CMLEN];              /*  drive name                      */
    char hosout[CMLEN];             /*  output host file name           */
    char mess[CMLEN];               /*  message to operator             */
    char mmflag[4];	                /*  minmax flag                     */
    char outlt[CMLEN];              /*  output host labeled table name  */
    char prtfile[CMLEN];            /*  ouptut host print file name     */
    char taein[CMLEN];              /*  input TAE file names            */
    char taeout[3][CMLEN];          /*  ouput TAE file names            */
    char *record;                   /*  ASCII record buffer             */

    /*  Initialize TAE
        ------------------*/
    c_inlas(&vblock,&std_out);

    /* Allocate dynamic memory
       --------------------------*/
    buff2 = (short *) malloc(32768);
    if (buff2 == NULL)
    {
        c_errmsg("Error allocating dynamic memory","dementer-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    /*  Re-use the same memory for binary or ASCII data -- only one will be
        used at a time
        -----------------------------------------------------------------*/
    record = (char *) buff2;

    /*  Get the input parameters
        ----------------------------*/
    getpar(taeout,hosout,mess,&bckgnd,&form,&fnum,&msgtim,outlt,&term,
            &file,&lp,&first,&last,&unit,drive,&outltf,prtfile,&outsys,mmflag,
            &ddrvflg,taein,hosin,&diskflg);

    if (diskflg == 0)
    {
        /*  Position the tape and determine the file size
            -------------------------------------------------*/
        tapeset(&form,&fnum,mess,&unit,(void *)buff2,&first,drive,&bufsiz);
    }
    else
    {
        if(NULL == (infp = fopen(hosin,"r")))
        {
            sprintf(mess,"Error opening file <%s>",taein);
            c_errmsg(mess,"dementer-fopen",&nonfatal);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
    }
    bufref = 0;

    /*  Process the 'A' header record data
        NOTE - the number of LINES returned by these routines is actually the
        number of patches to be processed, which will also be the number
        of samples in the output LAS image.
        -------------------------------------------------------------------*/
    if(form == DTT)
    {
        vll_dtt = (struct VLL_DTT *)calloc(1,sizeof(struct VLL_DTT));
        if (vll_dtt == NULL)
        {
            c_errmsg("Error allocating dynamic memory","dementer-alloc",
                      &nonfatal);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        adtt(&unit,buff2,&bufsiz,&bufref,&lines,&samps,&deltay,&deltaz,&ymin,
                vll_dtt,&outsys,infp);
    }
    else if (form == DEM)
    {
        vll_dem = (struct VLL_DEM *)calloc(1,sizeof(struct VLL_DEM));
        if (vll_dem == NULL)
        {
            c_errmsg("Error allocating dynamic memory","dementer-alloc",
                     &nonfatal);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        adem(buff2,&unit,&bufsiz,&bufref,&lines,&samps,&form,&deltay,&deltaz,
                &cdata,&ymin,vll_dem,&outsys,infp);
    }
    else if (form == DTED)
    {
        vll_dted = (struct VLL_DTED *)calloc(1,sizeof(struct VLL_DTED));
        if (vll_dted == NULL)
        {
            c_errmsg("Error allocating dynamic memory","dementer-alloc",
                      &nonfatal);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        status = adted(&unit,&lines,&samps,vll_dted,infp);
        if (status != E_SUCC)
            c_lasmsg(TRUE,"dementer-fatal","Fatal error encountered");
    }

    /*  Always assume DTED data is INTEGER*2 format only
        ----------------------------------------------------*/
    if (form == DTED) 
        deltaz = 1;

    /*  Process the 'B' data records
        -------------------------------*/
    brecs (buff2,&unit,&form,&lines,&samps,&bufsiz,&bufref,hosout,
            &deltay,&deltaz,&ymin,&msgtim,&bckgnd,&outsys,mmflag,infp); 

    /*  Process the 'C' trailer record
        ----------------------------------*/
    if (form == 1)
        cdtt (&unit,buff2,&bufref,vll_dtt,&outsys,infp);
    else
    {
        if (cdata == 1)
        {
            if (form == 22)
                cdemb (&unit,buff2,&bufref,vll_dem,&outsys,infp);
            else if (form == 23)
                cdema (&unit,record,&bufref,&bufsiz,vll_dem,infp);
        }
    }


    if(infp == NULL)
    {
        /*  Close the tape only if this is the final run.
            -------------------------------------------------*/
        if (last)
            status = c_tpclos(&unit,&handle,&rewopt,&zero);

        /*  Otherwise force eof on the tape.
            ------------------------------------*/
        else
        {
            ibytes = 32760;
            status = c_tpread(&unit,(void *)buff2,&ibytes,&handle);
        }

        if ((status < TAPE_SUCC) && (status != TAPE_EOF))
            c_errmsg("Error closing tape","dementer-tpclose ",&nonfatal);
    }
    else
        fclose(infp);

    /*  Display the header information and update the output DDR
        ------------------------------------------------------------*/
    optr = fopen(prtfile,"w");
    if(form == DTT)
    {
        dspdtt(optr,vll_dtt,taeout[0],&lines,&samps);
        ddrdtt(hosout,vll_dtt,&ddrvflg);

        /* Write the header data into a Variable Length Label  
           and output the label table if desired, finally display
           the label table to the user.
           --------------------------------------------------------*/
        if (outltf)
            putdtt(taeout[0],outlt,vll_dtt);

        /*  Free buffer space
            ---------------------*/
        free(vll_dtt);
    }

    else if(form > 20)
    {
        dspdem(optr,vll_dem,taeout[0],&lines,&samps);
        ddrdem(hosout,vll_dem,&lines,&samps,&ddrvflg);

        /* Write the header data into a Variable Length Label  
           and output the label table if desired, finally display
           the label table to the user.
           --------------------------------------------------------*/
        if (outltf)
            putdem(taeout[0],outlt,vll_dem);

        /*  Free buffer space
            ---------------------*/
        free(vll_dem);
    }

    else if(form == DTED)
    {
        dspdted(optr,vll_dted,taeout[0],&lines,&samps);
        ddrdted(hosout,vll_dted);

        /* Write the header data into a Variable Length Label  
           and output the label table if desired, finally display
           the label table to the user.
           --------------------------------------------------------*/
        if (outltf)
            putdted(taeout[0],outlt,vll_dted);

        /*  Free buffer space
            ---------------------*/
        free(vll_dted);
    }

    fclose(optr);

    status = c_hstout(&vblock,hosout,(char (*)[256])" ",&zero,"DEMENTER");

    /*  Free up the buffer space
        ----------------------------*/
    free(buff2);
    free(record);

    /*  Calculate the number of files that the completion message will output
        --------------------------------------------------------------------*/
    if (file)
        nfile++;

    if (outltf)
    {
        strcpy(taeout[nfile], outlt);
        nfile++;
    }

    /*  Write a standard completion message
        ---------------------------------------*/
    c_complt ("DEMENTER",taeout,&nfile);

}  /*  dementer  */
