/******************************************************************************
NAME:   		      PROCES 

PURPOSE   To perform vertical deswathing. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R. McKinney     12/88	  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.0       R. McKinney    04/89          EDC		Added the threshold
							option
  5.0       T. Sturman     01/91          EDC		Improve	execution time
							for all data types.  
  6.0       D. Bridges     05/94          EDC  		Added opt array for 
							opening files

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCES must be run under TAE.

PROJECT			LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   Apply the deswathing to the input image
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "imageio.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC *  fdesc[MAXIMG+1];	
                                /* image file descriptors used for image I/O */
extern struct GDESC *  gdesc;	/* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND+1];/* offsets into "thebuf" for input bands*/
extern long offset2[MAXBND+1];  /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void proces
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,			/* output data type			     */
    long kernel,			/* number of samples to process              */
    long thresh	 		/* smoothing threshold value		     */
)


{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;


long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long    i,k;			/* counter for number of columns	     */
long	image;			/* loop index counting input images          */
long    place1;			/* counter for loops			     */
long 	place2;			/* counter for loops		   	     */
long 	place3;			/* counter for loops		   	     */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long    n2;			/* half of the kernel size		     */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    ntime = 0;		/* number of values used		     */
long    one = 1;		/* one					     */
long	size = 0;		/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */
long    opt[2];			/* options for opening files		     */

float   fmin = 0.0;		/* minimum value to use			     */
float   fmax = 0.0;	 	/* maximum value to use			     */
float   nvalue;		    /* real value 				     */
float   oldval;         /* real value 				     */
float   realval = 0;	/* real value 				     */
float   tempval;		/* real value 				     */

unsigned char    *thebuf;	/* image input buffer                        */
unsigned char    *buf1; 	/* image input buffer for calculation        */
unsigned char    *buf2;		/* image input buffer for calculation        */

/* Open input image(s)
----------------------*/
opt[0] = opt[1] = 0;
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","deswath-open",&fatal);
   band += nbands[image];

   }

/* Open output image
--------------------*/
access = IWRITE;	
band = totbnd;
status = c_eopens(&fdesc[nmrimg],hosout[0],&band,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_clean(vblock,"deswath",hosout,&one);
fdesc[nmrimg+1] = 0;

/* Determine size of output image
--------------------------------*/
switch (odtype)
  {
  case EBYTE: size = 1; break;
  case EWORD: size = 2; break;
  case ELONG: size = 4; break;
  case EREAL: size = 4; break;
  }

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","deswath-alloc",&nonfatal);
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","deswath-close",&one);
   c_clean(vblock,"deswath",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","deswath-close",&one);
   c_clean(vblock,"deswath",hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("DESWATH","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
/*
   processing for PROCES 
*/
n2 = kernel / 2;
place2 = ns-n2;

switch(odtype)
{
 case EBYTE:
for (curlin=0;curlin<nl;curlin++)/* deswath input image to output image  */
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","deswath-close",&one);
      c_clean(vblock,"deswath",hosout,&one);
      }
   for (band=0;band<totbnd;band++)
      {
        buf1 = thebuf + (*(offset1+band) * size);
	buf2 = thebuf + (*(offset2+band) * size);

/*
 check for the first kernel/2 samples
*/
    oldval = -1;
    for(i = 0; i < n2; i++)
      {
       tempval = *(buf1+ (i * size));
       if (oldval == tempval)
   	{
	  nvalue = *(buf1+ ((n2+i) * size));
	  if ((nvalue >= fmin) && (nvalue <= fmax))
 	    {
  		ntime += 1;
		realval += nvalue;
	    }
	}
       else
	{
          fmin = tempval - (float) thresh;
          fmax = tempval + (float) thresh;
          realval = 0.0;
          ntime = 0;
          place1 = n2+i+1;
          for (k=0;k<place1; k++)
	    {
	      nvalue = *(buf1+ (k * size));
	      if (nvalue < fmin) continue;
	      if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	  *(buf2+(i * size)) = realval/(float) ntime;
	else
	  *(buf2+(i * size)) = tempval;
	oldval = tempval;
      }
/*
 check for the middle samples
*/
     oldval = -1;
     for (i=n2; i<place2;i++)
     { 
       tempval = *(buf1+(i));
       if (oldval == tempval)
   	{
	  nvalue = *(buf1+((i-n2-1) * size));
	  if (nvalue >= fmin) if (nvalue <= fmax)
 	    {
  		ntime -= 1;
		realval -= nvalue;
	    }
	  nvalue = *(buf1+((i+n2) * size));
	  if (nvalue >= fmin) if (nvalue <= fmax)
	   {
	     ntime += 1;
	     realval += nvalue;
	   }
	}
      else
        {
          realval = 0.0;
          ntime = 0;
	  fmin = tempval - (float) thresh;
	  fmax = tempval + (float) thresh;
	  place3 = i + n2 + 1;
          for (k=i-n2; k<place3;k++)
            {
              nvalue = *(buf1 + (k * size));
	      if (nvalue < fmin) continue;
              if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	     *(buf2 +(i * size)) = realval/(float) ntime;
	else
	     *(buf2+ (i * size)) = tempval;
	oldval = tempval;
      }
	    
/*
 check for the last kernel/2 number of samples
*/
	oldval = -1;
	for(i = place2; i < ns; i++)
	{
	   tempval = *(buf1+ (i * size));
           if (oldval == tempval)
   	    {
	      nvalue = *(buf1 + ((i-n2-1) * size));
	      if (nvalue >= fmin) if (nvalue <= fmax)
 	        {
  	    	   ntime -= 1;
		   realval -= nvalue;
	        }
	    }
          else
            {
              realval = 0.0;
              ntime = 0;
	      fmin = tempval - (float) thresh;
	      fmax = tempval + (float) thresh;
	      for (k=i-n2; k<ns; k++)
	         {
	            nvalue = *(buf1+ (k * size));
	            if (nvalue < fmin) continue;
	            if (nvalue > fmax) continue;
   	            ntime += 1;
	            realval += nvalue;
	         }
	     }
	if (ntime != 0)
	     *(buf2 + (i * size)) = realval/(float) ntime;
	else
	     *(buf2 + (i * size)) = tempval;
	oldval = tempval;
     }

	
      if (strcmp(minmax,"yes") == 0)
         status = c_pxmnmx(buf2,&ns,
                         minval+(band),maxval+(band),&odtype); 
      }
    curent ++;
   }
  break;

  case EWORD:
for (curlin=0;curlin<nl;curlin++)/* deswath input image to output image  */
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","deswath-close",&one);
      c_clean(vblock,"deswath",hosout,&one);
      }
   for (band=0;band<totbnd;band++)
      {
        buf1 = thebuf + (*(offset1+band) * size);
	buf2 = thebuf + (*(offset2+band) * size);

/*
 check for the first kernel/2 samples
*/
    oldval = -1;
    for(i = 0; i < n2; i++)
      {
       tempval = (*(short *)(buf1+ (i * size)));
       if (oldval == tempval)
   	{
	  nvalue = (*(short *)(buf1+ ((n2+i) * size)));
	  if ((nvalue >= fmin) && (nvalue <= fmax))
 	    {
  		ntime += 1;
		realval += nvalue;
	    }
	}
       else
	{
          fmin = tempval - (float) thresh;
          fmax = tempval + (float) thresh;
          realval = 0.0;
          ntime = 0;
          place1 = n2+i+1;
          for (k=0;k<place1; k++)
	    {
	      nvalue = (*(short *)(buf1+ (k * size)));
	      if (nvalue < fmin) continue;
	      if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	  (*(short *)(buf2+(i * size))) = realval/(float) ntime;
	else
	  (*(short *)(buf2+(i * size))) = tempval;
	oldval = tempval;
      }
/*
 check for the middle samples
*/
     oldval = -1;
     for (i=n2; i<place2;i++)
     { 
       tempval = (*(short *)(buf1+(i *size)));
       if (oldval == tempval)
   	{
	  nvalue = (*(short *)(buf1+((i-n2-1) * size)));
	  if (nvalue >= fmin) if (nvalue <= fmax)
 	    {
  		ntime -= 1;
		realval -= nvalue;
	    }
	  nvalue = (*(short *)(buf1+((i+n2) * size)));
	  if (nvalue >= fmin) if (nvalue <= fmax)
	   {
	     ntime += 1;
	     realval += nvalue;
	   }
	}
      else
        {
          realval = 0.0;
          ntime = 0;
	  fmin = tempval - (float) thresh;
	  fmax = tempval + (float) thresh;
	  place3 = i + n2 + 1;
          for (k=i-n2; k<place3;k++)
            {
              nvalue = (*(short *)(buf1 + (k * size)));
	      if (nvalue < fmin) continue;
              if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	     (*(short *)(buf2 +(i * size))) = realval/(float) ntime;
	else
	     (*(short *)(buf2+ (i * size))) = tempval;
	oldval = tempval;
      }
	    
/*
 check for the last kernel/2 number of samples
*/
	oldval = -1;
	for(i = place2; i < ns; i++)
	{
	   tempval = (*(short *)(buf1+ (i * size)));
           if (oldval == tempval)
   	    {
	      nvalue = (*(short *)(buf1 + ((i-n2-1) * size)));
	      if (nvalue >= fmin) if (nvalue <= fmax)
 	        {
  	    	   ntime -= 1;
		   realval -= nvalue;
	        }
	    }
          else
            {
              realval = 0.0;
              ntime = 0;
	      fmin = tempval - (float) thresh;
	      fmax = tempval + (float) thresh;
	      for (k=i-n2; k<ns; k++)
	         {
	            nvalue = (*(short *)(buf1+ (k * size)));
	            if (nvalue < fmin) continue;
	            if (nvalue > fmax) continue;
   	            ntime += 1;
	            realval += nvalue;
	         }
	     }
	if (ntime != 0)
	     (*(short *)(buf2 + (i * size))) = realval/(float) ntime;
	else
	     (*(short *)(buf2 + (i * size))) = tempval;
	oldval = tempval;
     }

	
      if (strcmp(minmax,"yes") == 0)
         status = c_pxmnmx(buf2,&ns,
                         minval+(band),maxval+(band),&odtype); 
      }
    curent ++;
   }
  break;

 case ELONG:
for (curlin=0;curlin<nl;curlin++)/* deswath input image to output image  */
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","deswath-close",&one);
      c_clean(vblock,"deswath",hosout,&one);
      }
   for (band=0;band<totbnd;band++)
      {
        buf1 = thebuf + (*(offset1+band) * size);
	buf2 = thebuf + (*(offset2+band) * size);

/*
 check for the first kernel/2 samples
*/
    oldval = -1;
    for(i = 0; i < n2; i++)
      {
       tempval = (float) (*(long *)(buf1+ (i * size)));
       if (oldval == tempval)
   	{
	  nvalue = (float) (*(long *)(buf1+ ((n2+i) * size)));
	  if ((nvalue >= fmin) && (nvalue <= fmax))
 	    {
  		ntime += 1;
		realval += nvalue;
	    }
	}
       else
	{
          fmin = tempval - (float) thresh;
          fmax = tempval + (float) thresh;
          realval = 0.0;
          ntime = 0;
          place1 = n2+i+1;
          for (k=0;k<place1; k++)
	    {
	      nvalue = (float) (*(long *)(buf1+ (k * size)));
	      if (nvalue < fmin) continue;
	      if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	  (*(long *)(buf2+(i * size))) = (long)(realval/(float)ntime);
	else
	  (*(long *)(buf2+(i * size))) = (long) tempval;
	oldval = tempval;
      }
/*
 check for the middle samples
*/
     oldval = -1;
     for (i=n2; i<place2;i++)
     { 
       tempval = (float) (*(long *)(buf1+(i *size)));
       if (oldval == tempval)
   	{
	  nvalue = (float) (*(long *)(buf1+((i-n2-1) * size)));
	  if (nvalue >= fmin) if (nvalue <= fmax)
 	    {
  		ntime -= 1;
		realval -= nvalue;
	    }
	  nvalue = (float) (*(long *)(buf1+((i+n2) * size)));
	  if (nvalue >= fmin) if (nvalue <= fmax)
	   {
	     ntime += 1;
	     realval += nvalue;
	   }
	}
      else
        {
          realval = 0.0;
          ntime = 0;
	  fmin = tempval - (float) thresh;
	  fmax = tempval + (float) thresh;
	  place3 = i + n2 + 1;
          for (k=i-n2; k<place3;k++)
            {
              nvalue = (float) (*(long *)(buf1 + (k * size)));
	      if (nvalue < fmin) continue;
              if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	     (*(long *)(buf2 +(i * size))) = (long) (realval/(float) ntime);
	else
	     (*(long *)(buf2+ (i * size))) = (long) tempval;
	oldval = tempval;
      }
	    
/*
 check for the last kernel/2 number of samples
*/
	oldval = -1;
	for(i = place2; i < ns; i++)
	{
	   tempval = (float) (*(long *)(buf1+ (i * size)));
           if (oldval == tempval)
   	    {
	      nvalue = (float) (*(long *)(buf1 + ((i-n2-1) * size)));
	      if (nvalue >= fmin) if (nvalue <= fmax)
 	        {
  	    	   ntime -= 1;
		   realval -= nvalue;
	        }
	    }
          else
            {
              realval = 0.0;
              ntime = 0;
	      fmin = tempval - (float) thresh;
	      fmax = tempval + (float) thresh;
	      for (k=i-n2; k<ns; k++)
	         {
	            nvalue = (float) (*(long *)(buf1+ (k * size)));
	            if (nvalue < fmin) continue;
	            if (nvalue > fmax) continue;
   	            ntime += 1;
	            realval += nvalue;
	         }
	     }
	if (ntime != 0)
	     (*(long *)(buf2 + (i * size))) = (long) (realval/(float)ntime);
	else
	     (*(long *)(buf2 + (i * size))) = (long) tempval;
	oldval = tempval;
     }
	
      if (strcmp(minmax,"yes") == 0)
         status = c_pxmnmx(buf2,&ns,
                         minval+(band),maxval+(band),&odtype); 
      }
    curent ++;
   }
  break;

case EREAL:
for (curlin=0;curlin<nl;curlin++)/* deswath input image to output image  */
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","deswath-close",&one);
      c_clean(vblock,"deswath",hosout,&one);
      }
   for (band=0;band<totbnd;band++)
      {
        buf1 = thebuf + (*(offset1+band) * size);
	buf2 = thebuf + (*(offset2+band) * size);

/*
 check for the first kernel/2 samples
*/
    oldval = -1;
    for(i = 0; i < n2; i++)
      {
       tempval = (*(float *)(buf1+ (i * size)));
       if (oldval == tempval)
   	{
	  nvalue = (*(float *)(buf1+ ((n2+i) * size)));
	  if ((nvalue >= fmin) && (nvalue <= fmax))
 	    {
  		ntime += 1;
		realval += nvalue;
	    }
	}
       else
	{
          fmin = tempval - (float) thresh;
          fmax = tempval + (float) thresh;
          realval = 0.0;
          ntime = 0;
          place1 = n2+i+1;
          for (k=0;k<place1; k++)
	    {
	      nvalue = (*(float *)(buf1+ (k * size)));
	      if (nvalue < fmin) continue;
	      if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	  (*(float *)(buf2+(i * size))) = realval/(float) ntime;
	else
	  (*(float *)(buf2+(i * size))) = tempval;
	oldval = tempval;
      }
/*
 check for the middle samples
*/
     oldval = -1;
     for (i=n2; i<place2;i++)
     { 
       tempval = (*(float *)(buf1+(i *size)));
       if (oldval == tempval)
   	{
	  nvalue = (*(float *)(buf1+((i-n2-1) * size)));
	  if (nvalue >= fmin) if (nvalue <= fmax)
 	    {
  		ntime -= 1;
		realval -= nvalue;
	    }
	  nvalue = (*(float *)(buf1+((i+n2) * size)));
	  if (nvalue >= fmin) if (nvalue <= fmax)
	   {
	     ntime += 1;
	     realval += nvalue;
	   }
	}
      else
        {
          realval = 0.0;
          ntime = 0;
	  fmin = tempval - (float) thresh;
	  fmax = tempval + (float) thresh;
	  place3 = i + n2 + 1;
          for (k=i-n2; k<place3;k++)
            {
              nvalue = (*(float *)(buf1 + (k * size)));
	      if (nvalue < fmin) continue;
              if (nvalue > fmax) continue;
   	      ntime += 1;
	      realval += nvalue;
	    }
	}
	if (ntime != 0)
	     (*(float *)(buf2 +(i * size))) = realval/(float) ntime;
	else
	     (*(float *)(buf2+ (i * size))) = tempval;
	oldval = tempval;
      }
	    
/*
 check for the last kernel/2 number of samples
*/
	oldval = -1;
	for(i = place2; i < ns; i++)
	{
	   tempval = (*(float *)(buf1+ (i * size)));
           if (oldval == tempval)
   	    {
	      nvalue = (*(float *)(buf1 + ((i-n2-1) * size)));
	      if (nvalue >= fmin) if (nvalue <= fmax)
 	        {
  	    	   ntime -= 1;
		   realval -= nvalue;
	        }
	    }
          else
            {
              realval = 0.0;
              ntime = 0;
	      fmin = tempval - (float) thresh;
	      fmax = tempval + (float) thresh;
	      for (k=i-n2; k<ns; k++)
	         {
	            nvalue = (*(float *)(buf1+ (k * size)));
	            if (nvalue < fmin) continue;
	            if (nvalue > fmax) continue;
   	            ntime += 1;
	            realval += nvalue;
	         }
	     }
	if (ntime != 0)
	     (*(float *)(buf2 + (i * size))) = realval/(float) ntime;
	else
	     (*(float *)(buf2 + (i * size))) = tempval;
	oldval = tempval;
     }
	
      if (strcmp(minmax,"yes") == 0)
         status = c_pxmnmx(buf2,&ns,
                         minval+(band),maxval+(band),&odtype); 
      }
    curent ++;
   }
  break;
}

/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","deswath-close",&one);
      c_clean(vblock,"deswath",hosout,&one);
   }
/* Turn off display messages
----------------------------*/
c_stoptm();
     
/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","deswath-close",&one);
      c_clean(vblock,"deswath",hosout,&one);
   }

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Error updating the minmax information","deswath-minmax",&one);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing images","deswath-close",&one);

/* Free buffer
--------------*/
free(thebuf);
return;
}
