/*****************************************************************************
NAME: compband

PURPOSE:
  This function will compare the bands of the two input images. A message
  will be output if there is a conflict between two bands, or if one image
  has more bands than the other.

DEVELOPMENT HISTORY:
VERSION      DATE    AUTHOR         CODE/CONT     REASON
---------------------------------------------------------
 1.0         7/00    B. Misterek      CSB         New Development


RETURN VALUE:
  int, status of the function(E_SUCC, E_FAIL)


******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "diffddr.h"

static void min_max_valid
(
    long input,             /* I: Input validity flag numeric value */
    char str[]              /* O: Validity flag's character value   */
);

int compband
(
    char input_files[][CMLEN], /* I: Array of input filenames                 */
    int  first,                /* I: Index of first input file in the array   */
    int  second,               /* I: Index of second input file in the array  */
    int  nbands_one,           /* I: Number of bands in the first input image */
    int  nbands_two            /* I: Number of bands in the second input image*/
)
{
    struct BDDR bddr1;   /* BDDR structure to hold the band information */
    struct BDDR bddr2;   /* BDDR structure to hold the band information */
   
    int i;              /* Loop variable  */
    int small_bands;    /* Smallest number of bands between the two images */

    int diff;      /* Variable to see if the bands were different*/

    char tmpstr[CMLEN];   /* Temp string for use in calling output */
    char tmpstr2[CMLEN];  /* Temp string for use in output and sprintf */
    char outstr[CMLEN];   /* Temp string for use in output */

    /*  Set small_bands the smaller number of bands between the two images.
    -----------------------------------------------------------------------*/
    if( nbands_one < nbands_two )
        small_bands = nbands_one;
    else
        small_bands = nbands_two;


    /*  Loop over all the band numbers that the two images have in common.
    ----------------------------------------------------------------------*/
    for( i=1 ; i <= small_bands ; i++)
    {
    
        diff = 0;
    
        /*  Call c_getbdr to read in the band fields of the first ddr. 
        ----------------------------------------------------------------*/
        if( c_getbdr(input_files[first], &bddr1,(long *) &i) != E_SUCC)
        {
            c_lasmsg(FALSE,"diffddr-compband","Error opening Band DDR for %s",
                     input_files[first]);
            return E_FAIL;
        }


        /*  Call c_getbdr to read in the band fields of the second ddr.
        ----------------------------------------------------------------*/
        if( c_getbdr(input_files[second],&bddr2, (long *) &i) != E_SUCC)
        {
            c_lasmsg(FALSE,"diffddr-compband","Error opening Band DDR for %s",
                     input_files[second]);
            return E_FAIL;
        }


        /*  Compare the valid fields of both bands.
        -------------------------------------------*/
        if( bddr1.valid != bddr2.valid)
        {
            sprintf( outstr, "Band %d Validity", i);
            min_max_valid(bddr1.valid, tmpstr2);
            sprintf( tmpstr, "\nFILE1: %s", tmpstr2);
            strcat(outstr, tmpstr);
            min_max_valid(bddr2.valid, tmpstr2);
            sprintf( tmpstr, "\nFILE2: %s", tmpstr2);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
                return E_FAIL;
            diff = 1;
        }
        

        /*  Compare the minimum values of the two bands. (with a tolerance)
        -------------------------------------------------------------------*/
        if( fabs(bddr1.minval - bddr2.minval) > TOLERANCE ) 
        {
            sprintf( outstr, "Band %d Minimum", i);
            sprintf( tmpstr, "\nFILE1: %f", bddr1.minval);
            strcat(outstr, tmpstr);
            sprintf( tmpstr, "\nFILE2: %f", bddr2.minval);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
                return E_FAIL;
            diff = 1;
        }


        /*  Compare the maximum values of the two bands. (with a tolerance)
        -------------------------------------------------------------------*/
        if( fabs(bddr1.maxval - bddr2.maxval) > TOLERANCE )
        {
            sprintf (outstr, "Band %d Maximum", i);
            sprintf( tmpstr, "\nFILE1: %f", bddr1.maxval);
            strcat(outstr, tmpstr);
            sprintf( tmpstr, "\nFILE2: %f", bddr2.maxval);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
                return E_FAIL;
            diff = 1;
        }
  
        
        /*  Compare the source of the two bands.  Case will be ignored
            when comparing, so use c_up2low to convert to lowercase.
        --------------------------------------------------------------*/
        if( strcasecmp(bddr1.source, bddr2.source) )
        {
            sprintf(outstr, "Band %d Source", i);
            sprintf(tmpstr, "\nFILE1: %s", bddr1.source);
            strcat(outstr, tmpstr);
            sprintf(tmpstr, "\nFILE2: %s", bddr2.source);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC ) 
                return E_FAIL;
            diff = 1;
        }


        /*  Compare the instrument of the two bands.  Case will be ignored
            when comparing, so use c_up2low to convert to lowercase.
        ------------------------------------------------------------------*/
        if( strcasecmp(bddr1.instrument, bddr2.instrument) )
        {
            sprintf(outstr, "Band %d Instrument", i);
            sprintf(tmpstr, "\nFILE1: %s", bddr1.instrument);
            strcat(outstr, tmpstr);
            sprintf(tmpstr, "\nFILE2: %s", bddr2.instrument);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
                return E_FAIL;
            diff = 1;
        }

        
        /*  Compare the direction of the two bands.  Case will be ignored
            when comparing, so use c_up2low to convert to lowercase.
        -----------------------------------------------------------------*/
        if( strcasecmp(bddr1.direction, bddr2.direction) )
        {
            sprintf(outstr, "Band %d Direction", i);
            sprintf(tmpstr, "\nFILE1: %s", bddr1.direction);
            strcat(outstr, tmpstr);
            sprintf(tmpstr, "\nFILE2: %s", bddr2.direction);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
               return E_FAIL;
            diff = 1;
        }


        /*  Compare the date of the two bands.  Case will be ignored when
            comparing, so use c_up2low to convert to lowercase.
        ----------------------------------------------------------------*/
        if( strcasecmp(bddr1.date, bddr2.date) )
        {
            sprintf(outstr, "Band %d Date", i);
            sprintf(tmpstr, "\nFILE1: %s", bddr1.date);
            strcat(outstr, tmpstr);
            sprintf(tmpstr, "\nFILE2: %s", bddr2.date);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
                return E_FAIL;
            diff = 1;
        }

        
        /*  Compare the time of the two bands.  Case will be ignored when
            comparing, so use c_up2low to convert to lowercase.
        -----------------------------------------------------------------*/
        if( strcasecmp(bddr1.time, bddr2.time) )
        {
            sprintf(outstr, "Band %d Time", i);
            sprintf(tmpstr, "\nFILE1: %s", bddr1.time);
            strcat(outstr, tmpstr);
            sprintf(tmpstr, "\nFILE2: %s", bddr2.time);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
                return E_FAIL;
            diff = 1;
        }
        
        /* If the bands were identical, output a message.
        -------------------------------------------------*/
        if( !diff )
        {
            sprintf(tmpstr, "Band %d was identical in both images", i);
            output(tmpstr, STATUS_MSG);

        }

 
    }


    /*  Check to see if one image had more bands than the other. If so, 
        output a message for each extra band in the image   
    -------------------------------------------------------------------*/
    if( nbands_one > small_bands)
    {
        for(i = small_bands + 1 ; i < nbands_one + 1 ; i++)
        {
            sprintf(tmpstr, "Band %d does not exist in FILE2", i);
            output(tmpstr, DIFF_MSG);
         }    
    }
    else if(nbands_two > small_bands)
    {
        for(i = small_bands + 1 ; i < nbands_two + 1 ; i++)
        {
            sprintf(tmpstr, "Band %d does not exist in FILE1", i);
            output(tmpstr, DIFF_MSG);
        }
    }

    return E_SUCC;
            
        

    
}


/******************************************************************************
NAME:  min_max_valid

PURPOSE:
  To create the correct string according to the value of the minmax flag.

RETURN VALUE:
  void
  
NOTES:
  Taken from dstbnd.c in dspddr.
  
******************************************************************************/

static void min_max_valid
(
    long input,             /* I: input validity flag numeric value  */
    char str[]              /* O: validity flag's character value    */
)
{
    switch (input)
    {
        case INVAL: sprintf (str,"INVALID");
                break;
        case VALID: sprintf (str,"VALID");
                break;
        case BOUND: sprintf (str,"BOUNDED");
                break;
        default: sprintf (str,"UNACCEPTABLE");
    }
}




