/******************************************************************************
NAME:  compddr

PURPOSE:
  This function compares the two DDR files.  A message will be output if
  a conflict in the validity occurs, or if the two values are different.

DEVELOPMENT HISTORY:
VERSION      DATE    AUTHOR         CODE/CONT     REASON
---------------------------------------------------------
 1.0         7/00    B. Misterek      CSB         New Development


RETURN VALUE:
  int, status of the function(E_SUCC, E_FAIL)

******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "diffddr.h"

static void print_valid
(
    long input,             /* I: Input validity flag numeric value  */
    char str[]              /* O: Validity flag's character value    */
);
        


int compddr
(
    char input_files[][CMLEN], /* I: Array of input filenames  */
    int  first,                /* I: Index of first input filename in the 
                                     array  */
    int  second,               /* I: Index of second input filename in the 
                                     array */
    int  bandddr               /* I: Flag to indicate whether to compare bands*/
)
{
    struct DDR ddr1;  /* DDR struct to hold the information from the DDR file */
    struct DDR ddr2;  /* DDR struct to hold the information from the DDR file */

    char tmpstr[CMLEN];   /* Temp string used with strcpy and sprintf */
    char tmpstr2[CMLEN];  /* Temp string used with strcpy and sprintf */
    char outstr[CMLEN];   /* Temp string to use with output */

    
    int i;             /* Loop variable  */

    /*  Open the first DDR file with c_getddr
    -----------------------------------------*/
    if( c_getddr(input_files[first], &ddr1) != E_SUCC)
    {
        c_lasmsg(FALSE,"diffddr-compddr","Error opening DDR file %s", 
                 input_files[first]);
        return E_FAIL;
    }

    /*  Open the second DDR file with c_getddr
    ------------------------------------------*/
    if( c_getddr(input_files[second], &ddr2) != E_SUCC)
    {
        c_lasmsg(FALSE,"diffddr-compddr","Error opening DDR file %s",
                 input_files[second]);
        return E_FAIL;
    }

    /*  Compare the number of lines in both DDR's.
    ---------------------------------------------*/
    if( ddr1.nl != ddr2.nl)
    {
        strcpy(outstr, "Number of Lines");
        sprintf(tmpstr, "\nFILE1: %d", ddr1.nl);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %d", ddr2.nl);
        strcat(outstr, tmpstr);
        if( output(outstr,DIFF_MSG) != E_SUCC)
            return E_FAIL;
    }


    /*  Compare the number of samples in both DDR's.
    -----------------------------------------------*/
    if( ddr1.ns != ddr2.ns)
    {
        strcpy(outstr, "Number of Samples");
        sprintf(tmpstr, "\nFILE1: %d", ddr1.ns);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %d", ddr2.ns);
        strcat(outstr, tmpstr);
        if( output(outstr,DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the number of bands in both DDR's.
    ----------------------------------------------*/
    if( ddr1.nbands != ddr2.nbands)
    {
        strcpy(outstr, "Number of Bands");
        sprintf(tmpstr, "\nFILE1: %d", ddr1.nbands);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %d", ddr2.nbands);
        strcat(outstr, tmpstr);
        if( output(outstr,DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the data type of the DDR's.
    ---------------------------------------*/
    if( ddr1.dtype != ddr2.dtype)
    {
        strcpy(outstr, "Data Type");
        
        /* Use a switch to output the name of the data type instead
           of the number.
        -----------------------------------------------------------*/
        switch(ddr1.dtype)
        {
            case EBYTE:  sprintf(tmpstr, "\nFILE1: BYTE");
                         break;
            case EWORD:  sprintf(tmpstr, "\nFILE1: INTEGER*2");
                         break;
            case ELONG:  sprintf(tmpstr, "\nFILE1: INTEGER*4");
                         break;
            case EREAL:  sprintf(tmpstr, "\nFILE1: REAL*4");
                         break;
            default:     sprintf(tmpstr, "\nFILE1: UNKNOWN");
                         break;
        }
        strcat(outstr, tmpstr);
        switch(ddr2.dtype)
        {
            case EBYTE:  sprintf(tmpstr, "\nFILE2: BYTE");
                         break;
            case EWORD:  sprintf(tmpstr, "\nFILE2: INTEGER*2");
                         break;
            case ELONG:  sprintf(tmpstr, "\nFILE2: INTEGER*4");
                         break;
            case EREAL:  sprintf(tmpstr, "\nFILE2: REAL*4");
                         break;
            default:     sprintf(tmpstr, "\nFILE2: UNKNOWN");
                         break;
        }
        strcat(outstr, tmpstr);
        if( output(outstr,DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

    
    /*  Compare the projection code valid field. 
    --------------------------------------------*/
    if( ddr1.valid[DDPCV] != ddr2.valid[DDPCV])
    {
        strcpy(outstr, "Projection Code");
        print_valid(ddr1.valid[0], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[0], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr,DIFF_MSG) != E_SUCC)
            return E_FAIL;
    }


    /*  Compare the projection codes.
    --------------------------------*/
    if( ddr1.proj_code != ddr2.proj_code )
    {
        strcpy(outstr, "Projection Code");
        sprintf(tmpstr, "\nFILE1: %d", ddr1.proj_code);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %d", ddr2.proj_code);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

    
    /*  Compare the Zone Code Valid fields.
    ---------------------------------------*/
    if( ddr1.valid[DDZCV] != ddr2.valid[DDZCV])
    {
        strcpy(outstr, "Zone Code");
        print_valid(ddr1.valid[1], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[1], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the Zone Code field.  
    --------------------------------*/
    if( ddr1.zone_code != ddr2.zone_code )
    {
        strcpy(outstr, "Zone Code");
        sprintf(tmpstr, "\nFILE1: %d", ddr1.zone_code);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %d", ddr2.zone_code);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;

    }

    
    /* Compare the Datum Code Valid fields.
    ---------------------------------------*/
    if( ddr1.valid[DDDCV] != ddr2.valid[DDDCV] )
    {
        strcpy(outstr, "Datum Code");
        print_valid(ddr1.valid[2], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[2], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

    
    /*  Compare the Datum Code fields.
    ----------------------------------*/
    if( ddr1.datum_code != ddr2.datum_code)
    {
        strcpy(outstr, "Datum Code");
        sprintf(tmpstr, "\nFILE1: %d", ddr1.datum_code);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %d", ddr2.datum_code);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

    
    /*  Compare the Projection Coefficient valid fields
    ---------------------------------------------------*/
    if( ddr1.valid[DDPPV] != ddr2.valid[DDPPV])
    {
        strcpy(outstr, "Projection Coefficients");
        print_valid(ddr1.valid[3], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[3], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Loop over the 15 projection coefficient fields.
    ---------------------------------------------------*/
    for( i = 0 ; i < 15 ; i ++)
    {
        
        /*  Compare the ith projection coefficient field.
        ------------------------------------------------*/
        if( ddr1.proj_coef[i] != ddr2.proj_coef[i] )
        {
            sprintf(outstr, "Projection Coefficient %d", i);
            sprintf(tmpstr, "\nFILE1: %f", ddr1.proj_coef[i]);
            strcat(outstr, tmpstr);
            sprintf(tmpstr, "\nFILE2: %f", ddr2.proj_coef[i]);
            strcat(outstr, tmpstr);
            if( output(outstr, DIFF_MSG) != E_SUCC )
                return E_FAIL;
        }

    }

    
    /*  Compare the Corner Coordinates valid field.
    -----------------------------------------------*/
    if( ddr1.valid[DDCCV] != ddr2.valid[DDCCV] )
    {
        strcpy(outstr, "Corner Coordinates");
        print_valid(ddr1.valid[6], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[6], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the upper left corner coordinates. (with a tolerance)
    -----------------------------------------------------------------*/
    if( (fabs(ddr1.upleft[0] - ddr2.upleft[0]) > TOLERANCE) ||
        (fabs(ddr1.upleft[1] - ddr2.upleft[1]) > TOLERANCE)) 
    {
        strcpy(outstr, "Upper Left Corner");
        sprintf(tmpstr, "\nFILE1: %f , %f", ddr1.upleft[0], ddr1.upleft[1]);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %f , %f", ddr2.upleft[0], ddr2.upleft[1]);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

  
    /*  Compare the lower left corner coordinates. (with a tolerance)
    -----------------------------------------------------------------*/
    if( (fabs(ddr1.loleft[0] - ddr2.loleft[0]) > TOLERANCE) ||
        (fabs(ddr1.loleft[1] - ddr2.loleft[1]) > TOLERANCE)) 
    {
        strcpy(outstr, "Lower Left Corner");
        sprintf(tmpstr, "\nFILE1: %f , %f", ddr1.loleft[0], ddr1.loleft[1]);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %f , %f", ddr2.loleft[0], ddr2.loleft[1]);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC ) 
            return E_FAIL;
    }


    /*  Compare the upper right corner coordinates. (with a tolerance)
    ------------------------------------------------------------------*/
    if( (fabs(ddr1.upright[0] - ddr2.upright[0]) > TOLERANCE) ||
        (fabs(ddr1.upright[1] - ddr2.upright[1]) > TOLERANCE)) 
    {
        strcpy(outstr, "Upper Right Corner");
        sprintf(tmpstr, "\nFILE1: %f , %f", ddr1.upright[0], ddr1.upright[1]);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %f , %f", ddr2.upright[0], ddr2.upright[1]);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the lower right corner coordinates. (with a tolerance)
    ------------------------------------------------------------------*/
    if( (fabs(ddr1.loright[0] - ddr2.loright[0]) > TOLERANCE) ||
        (fabs(ddr1.loright[1] - ddr2.loright[1]) > TOLERANCE))
    {
        strcpy(outstr, "Lower Right Corner");
        sprintf(tmpstr, "\nFILE1: %f, %f", ddr1.loright[0], ddr1.loright[1]);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %f, %f", ddr2.loright[0], ddr2.loright[1]);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }
   
    
    /*  Compare the projection distance valid fields.
    ------------------------------------------------*/
    if( ddr1.valid[DDPDV] != ddr2.valid[DDPDV])
    {
        strcpy(outstr, "Projection Distance");
        print_valid(ddr1.valid[5], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[5], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

 
    /*  Compare the x and y projection distances. (with a tolerance)
    ----------------------------------------------------------------*/
    if( (fabs(ddr1.pdist_x - ddr2.pdist_x) > TOLERANCE) ||
        (fabs(ddr1.pdist_y - ddr2.pdist_y) > TOLERANCE))
    {
        strcpy(outstr, "Projection Distance");
        sprintf(tmpstr, "\nFILE1: %f, %f", ddr1.pdist_x, ddr1.pdist_y);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %f, %f", ddr2.pdist_x, ddr2.pdist_y);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

    
    /*  Compare the projection units valid field.
    ---------------------------------------------*/
    if( ddr1.valid[DDPUV] != ddr2.valid[DDPUV])
    {
        strcpy(outstr, "Projection Units");
        print_valid(ddr1.valid[4], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[4], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the projection units field.  Case will be ignored when 
        comparing, so use c_up2low to convert to lowercase.
    ------------------------------------------------------------------*/
    if( strcasecmp(ddr1.proj_units, ddr2.proj_units) )
    {
        strcpy(outstr, "Projection Units");
        sprintf(tmpstr, "\nFILE1: %s", ddr1.proj_units);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %s", ddr2.proj_units);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC)
            return E_FAIL;
    }

    
    /*  Compare the increment valid field
    -------------------------------------*/
    if( ddr1.valid[DDINCV] != ddr2.valid[DDINCV])
    {
        strcpy(outstr, "Increment");
        print_valid(ddr1.valid[7], tmpstr2);
        sprintf(tmpstr, "\nFILE1: %s", tmpstr2);
        strcat(outstr, tmpstr);
        print_valid(ddr2.valid[7], tmpstr2);
        sprintf(tmpstr, "\nFILE2: %s", tmpstr2);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the line and sample increments.  (with a tolerance)
    ---------------------------------------------------------------*/
    if( (fabs(ddr1.line_inc - ddr2.line_inc) > TOLERANCE) ||
        (fabs(ddr1.sample_inc - ddr2.sample_inc) > TOLERANCE))
    {
        strcpy(outstr, "Increment");
        sprintf(tmpstr, "\nFILE1: %f, %f", ddr1.line_inc, ddr1.sample_inc);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %f, %f", ddr2.line_inc, ddr2.sample_inc);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }


    /*  Compare the master line and master sample. (with a tolerance)
    -----------------------------------------------------------------*/
    if( (fabs(ddr1.master_line - ddr2.master_line) > TOLERANCE) ||
        (fabs(ddr1.master_sample - ddr2.master_sample) > TOLERANCE))
    {
        strcpy(outstr, "Master Coordinates");
        sprintf(tmpstr, "\nFILE1: %d, %d", ddr1.master_line, 
                                         ddr1.master_sample);
        strcat(outstr, tmpstr);
        sprintf(tmpstr, "\nFILE2: %d, %d", ddr2.master_line,
                                         ddr2.master_sample);
        strcat(outstr, tmpstr);
        if( output(outstr, DIFF_MSG) != E_SUCC )
            return E_FAIL;
    }

    /*  If the user specified to compare the bands, call
        compband.
    ----------------------------------------------------*/
    if( bandddr )
    {
        if( compband(input_files, first, second, 
            ddr1.nbands, ddr2.nbands) != E_SUCC)
        {
            c_lasmsg(FALSE,"diffddr-compddr",
                           "Error comparing bands");
            return E_FAIL;
        }
    }

    if( get_filediff() == 0 )
    {
        strcpy(tmpstr, "There were no differences detected in the DDR's.");
        if( output(tmpstr, STATUS_MSG) != E_SUCC )
            return E_FAIL;
    }

    return E_SUCC;

    
}


/******************************************************************************
NAME:  print_valid

PURPOSE:
  This function will take the integer value of the valid flag and put
  "INVALID", "VALID", or "UNKNOWN" into str.

REUTURN VALUE:
  void

NOTES:
  Taken from dstimg.c of dspddr

******************************************************************************/

static void print_valid
(
    long input,             /* I: Input validity flag numeric value  */
    char str[]              /* O: Validity flag's character value    */
)
{
    switch (input)
    {
        case INVAL: sprintf (str,"INVALID");
                    break;
        case VALID: sprintf (str,"VALID");
                    break;
        case UNKNOW: sprintf (str,"UNKNOWN");
                     break;
        default: sprintf (str,"UNACCEPTABLE");
    }
}

