/*****************************************************************************
NAME:  output.c

PURPOSE:
  This module takes care of all of the output that needs to be done.  It
  consists of open_output, is_different, update_output, output, do_output, and
  close_output.  

DEVELOPMENT HISTORY:
VERSION      DATE    AUTHOR         CODE/CONT     REASON
---------------------------------------------------------
 1.0         7/00    B. Misterek      CSB         New Development

*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "diffddr.h"

/*  Some static variables local to this module
----------------------------------------------*/
static int diff;      /* Flag to indicate differences in the files       */
static int filediff;  /* Flag to indicate differences in these two files */
static int no_output; /* Flag to indicate if the user wants no output    */
static int print[3];  /* Flags to indicate where to output               */
static int filenames_output;   /* Flag to indicate if the filenames have been
                                  output */

static char outputfile[CMLEN]; /* Static character array to hold the output
                                  filename */
static char infile[2][CMLEN];  /* Static character array to hold the input
                                  filenames */

static FILE *fp;               /* File pointer to open the output file */

static int do_output(char *);


/******************************************************************************
NAME:          open_output

PURPOSE: 
  This function will initialize whatever output the user has specified.
  If an output file is specified, it will be opened here.

RETURN VALUE:
  int, status of the function(E_FAIL, E_SUCC)

ALGORITHM:

******************************************************************************/

int open_output
(
    int prnt[],         /* I: Array specifing where to output  */
    char outfile[]      /* I: Output file to open if specified */

)
{
    diff = 0;           /* Initialize diff to 0     */
    filediff = 0;       /* Initialize filediff to 0 */

    print[TERM] = prnt[TERM];
    print[PFILE] = prnt[PFILE];
    print[LP] = prnt[LP];

    strcpy(outputfile, outfile);

    /*  If the user selected to print to a file or line printer, open
        the file with the static file pointer fp
    -----------------------------------------------------------------*/
    if( print[PFILE] || print[LP] )
    {
        fp = fopen(outfile, "w");
        if( fp == NULL)
        {
            c_lasmsg(FALSE, "diffddr-open", "Error opening %s", outfile);
            return E_FAIL;
        }
    
    }
    
    /*  Set the no_output flag.  If the user didn't select any output options, 
        then set the no_output flag to 1.  Otherwise set it to 0;
    -------------------------------------------------------------------------*/
    if( !print[TERM] && !print[PFILE] && !print[LP] )
        no_output = 1;
    else
        no_output = 0;

    return E_SUCC;

}


/******************************************************************************
NAME:  is_different 

PURPOSE:
  This function will return the diff variable which is static to this
  module

RETURN VALUE:
  int, the value of diff

*******************************************************************************/

int is_different( void )
{
    return diff;
}

/******************************************************************************
NAME:  get_filediff

PURPOSE:
  This function will return the filediff variable which is static to this
  module

RETURN VALUE:
  int, the value of filediff

*******************************************************************************/

int get_filediff( void )
{
    return filediff;
}


/******************************************************************************
NAME:  update_output

PURPOSE:
  This function will update the filenames in the event that more than two 
  DDR's are specified.

RETURN VALUE:
  None 

ALGORITHM:

******************************************************************************/

void update_output
(
    char input_files[][CMLEN],/* I: Array of input filenames */
    int  first,               /* I: Index of first input file in the array */
    int  second               /* I: Index of second input file in the arrray */
)
{
    /* Set the static variable infile to the two files which were passed in.
    ------------------------------------------------------------------------*/
    strcpy( infile[0], input_files[first] );
    strcpy( infile[1], input_files[second] );

    filenames_output = 0;
    filediff = 0;
}


/******************************************************************************
NAME: output

PURPOSE:
  This function is called when a difference is found in the files.  It will 
  accept a difference flag to indicate whether the message to output is a
  difference message or not.  If the difference flag is 1, and it is the first
  time that it has been called, the function will output the filenames message
  also.

RETURN VALUE:
  int, status of the function(E_SUCC, E_FAIL)

ALGORITHM:

******************************************************************************/

int output
(
    char *output_string,     /* I: String to be output  */
    int   difference_flag    /* I: Flag to indicate if this is a difference 
                                   message */
)
{
    
    
    char tmpstr[CMLEN * 2 + 18];   /* Temp string with enough room for two
                                      command lines and two file headers */
    

    /* Check to see if the difference_flag is set.  If so, this message is
       in reference to a difference that was found in the files.
    ----------------------------------------------------------------------*/
    if( difference_flag == DIFF_MSG )
    {
        /* Set the diff flag to indicate a difference was found 
        -------------------------------------------------------*/
        diff = 1;
        filediff = 1;


        /* Check to see if the filenames have been output yet.  If not, 
           output them.
        ---------------------------------------------------------------*/
        if( filenames_output == 0 )
        {

            /* Output a message stating what FILE1 and FILE2 are. 
            ----------------------------------------------------*/
            sprintf(tmpstr,"FILE1: %s\nFILE2: %s", infile[0], infile[1]); 

            if( do_output(tmpstr) != E_SUCC)
            {
               c_lasmsg(FALSE,"diffddr-output","Error writing to %s",
                        outputfile);
               return E_FAIL;
            }
               
            filenames_output = 1;
        }
    }

    /*  Output the string which was passed in.
    ------------------------------------------*/
    if( do_output(output_string) != E_SUCC)
    {
        c_lasmsg(FALSE,"diffddr-output","Error writing to %s",
                 outputfile);
        return E_FAIL;
    }

    return E_SUCC;
        
}


/******************************************************************************
NAME:  do_output

PURPOSE:
  This function will output the string to the specified destinations.  This
  function will only be called from this module.

RETURN VALUE:
  int, status of the function(E_SUCC, E_FAIL)

ALGORITHM:

******************************************************************************/

static int do_output
(
    char *string       /* String to output */
)
{
#define BUFSIZE (3*CMLEN)  /* local buffer size */
    char output_string[BUFSIZE]; /* local buffer for copying string for 
                                    tokenizing */
    long zero = 0;
    
    if ( no_output )
        return E_SUCC;

    strncpy(output_string,string,BUFSIZE);
    if( print[PFILE] || print[LP] )
    {
      if( fprintf(fp,"%s\n",output_string) < 0)
      {
          fclose(fp);
          c_lasmsg(FALSE,"diffddr-do_output","Error writing to %s",
                   outputfile);
          return E_FAIL;
      }
    }
    if( print[TERM] )
    {
        char *line_ptr = strtok(output_string, "\n");
        
        /* Output the string to the terminal, sending one line at a time 
           to c_pterm so it properly counts the lines.
        ----------------------------------------------------------------*/
        while (line_ptr)
        {
            c_pterm(line_ptr, &zero);
            line_ptr = strtok(NULL,"\n");
        }
    }

    return E_SUCC;
}


/******************************************************************************
NAME: close_output

PURPOSE:
  This function will close any files which have been opened in open_output.

DEVELOPMENT HISTORY:

RETURN VALUE:
  int, status of the function(E_SUCC, E_FAIL)

ALGORITHM:

******************************************************************************/

int close_output
(
    void
)
{
    /* If the user has selected to output to a file or to the line printer, 
       fp was opened, so close fp.
    -----------------------------------------------------------------------*/
    if( print[PFILE] || print[LP] )
    {
       if( fp != NULL )
       {
           if( fclose(fp) != 0)
           {
               c_lasmsg(FALSE,"diffddr-close","Error closing output file %s",
                        outputfile);
               return E_FAIL;
           }
       }
    }
    
    return E_SUCC;
}



