/******************************************************************************
PROGRAM:                   DSPGOF

PURPOSE:        Displays graphics record information to the terminal, line
                printer, or file.  The graphics records or attribute 
                definitions of the graphics overlay file (GOF) are displayed.  
                In the case of graphics records, the user may list all of 
                the records, specific records  and/or records with certain 
                attribute values. The user also has the option of whether 
                or not to list all of the point pairs for a line or polygon. 

PROGRAM HISTORY:

VERSION DATE    AUTHOR          REASON
------- -----   ------          ---------------------------
 ---    01/86   D. Gordon       Initial code
 ---    10/88   L. Huewe        DMS v3_o retrocode
 1.0    07/94   S. Nelson       Conversion from DMS to LAS
 1.0    12/94   D. Eichacker    Conversion from DMS to LAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  must be run under TAE.

PROJECT:  LAS

ALGORITHM:

*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "gof.h"
#include "protos.h"

#define HEADR   0       /* Header record */

int main
(void)
{
    struct PARBLK vblock; /* TAE variable block */

    union GOF gofrec;     /* GOF record structure */
    union GOF gofhdr;     /* GOF header record structure */

    FILE *lfp;            /* Pointer to output file */

    /* GOF type names */
    static char goftxt[MAXGOF][6] = {"POINT","LINE","POLY","ANNOT"};

    long fatal = -1;      /* Indicates fatal error message */
    long gtype;           /* GOF type (used for calls to open & read) */
    long i;               /* Loop counter */
    long nonfatal=1;      /* Indicates nonfatal error message */
    long numattr;         /* The number of search attributes */
    long numrec;          /* Number of valid entries in RECNO */
    long one = 1;         /* Constant one */
    long outpt=0;         /* Number output printed */
    long rec;             /* Record number */
    long recno[100];      /* Records specified by user in -REC */
    long status;          /* Captures return status from subroutine */

    int obyte;            /* The number of bytes in output from read */
    int length;           /* Length of string in output from read */
    int goffd;            /* The file descriptor for the GOF file */
    int goftyp;           /* GOFtype (used as indices into arrays) */ 
    int goftype;          /* GOF type for read access */
    int cflag = 0;        /* Flag indicating whether an ARL or GOF
                             was created */ 
    int sflag = TRUE;     /* Search or Append flag */
    int slist[MAXATT];    /* Index into ATTR in the order they appear */
    int mode;             /* Mode for accessing file */
    int hdrtyp=HEADR;     /* Type of record to read */

    char attr[MAXATT][ATTVAL]; /* Values for attributes returned from ATTCHK */
    char attrval[MAXATT][ATTIN]; /* Values for attributes input by the user */
    char hname[CMLEN];    /* Fully qualified host GOF name */
    char ingof[CMLEN];    /* Source file image name */
    char lname[CMLEN];    /* Fully qualified LAS GOF name */
    char pfile[1][CMLEN]; /* Name of the temporary file to write to */
    char subcmd[7];       /* User input subcommand */
    char ufile[CMLEN];    /* Name of the user specified output file */

    long found=FALSE;     /* Used for -REC matching #s with RECNUM list */
    long printed = FALSE; /* Whether anything was printed in the list */
    long prtfil=FALSE;    /* Output destination(s) */
    long prtpts;          /* Whether or not to list out point pairs */
    long ptd = FALSE;     /* Whether anything was printed in the list */
    long ptdatt = FALSE;  /* Weather an attribute printed the record */

    /* Retrieve user specified parameters
       ----------------------------------*/
    getpar(subcmd,&goftyp,&prtfil,ingof,attrval,&prtpts,&numrec,recno,
            &numattr,pfile[0],ufile,&vblock);

    /* Open the output file for writing
       --------------------------------*/
    lfp = fopen(pfile[0],"w");

    /* Print opening information from getpar
       -------------------------------------*/
    printparms(ingof,lfp);

    /* Set the reading type for the gof types
       --------------------------------------*/
    if (goftyp == 0) 
        gtype = PNT;
    else if (goftyp == 3) 
        gtype = ANOT;
    else 
        gtype = FPOLIN;


    /* C_GOFOPN opens the GOF
       ----------------------*/
    mode = READ;
    c_cmhost(ingof,hname);
    status = c_gofopn(&goffd,hname,lname,goftxt[goftyp],&mode,&cflag);
    if (status != E_SUCC) 
        c_errmsg("Fatal error encountered","dspgof-fatal",&fatal);
    strcpy(ingof,hname);

    /* Read header of GOF
       ------------------*/
    status = c_gofrd(&goffd,&obyte,&length,&hdrtyp,(struct DDR *)NULL,&gofhdr);
    if (status != E_SUCC)
    {
        fclose (lfp);
        c_clean(&vblock,"dspgof",pfile,&one);
    }

    /* Print out information contained in the header
       ---------------------------------------------*/
    fprintf(lfp,"\f");
    ptd = printheader(subcmd,goftxt[goftyp],&gofhdr.hdrec,lfp);
    if (ptd) 
        printed = TRUE;

    /* PRINTHEADER takes care of all of the output 
       for the -DEF subcommand
       -------------------------------------------*/
    if (*subcmd == 'G')
    {
        /* Verify attribute names and values specified
           -------------------------------------------*/
        status = c_attchk(&gofhdr.hdrec,attrval,attr,slist,&sflag);
        if (status != E_SUCC)
        {
            fclose(lfp);
            c_clean(&vblock,"dspgof",pfile,&one);
        }

        /* For each record of interest
           ---------------------------*/
        for (rec = 1; ; rec++)
        {

            /* Read an entire GOF record
               -------------------------*/

            /* If gtype is fpolin then run two reads, one with 
               hpolin and xypairs otherwise read like normal 
               -----------------------------------------------*/
            if (gtype == FPOLIN)
            {
                goftype = HPOLIN;
                status = c_gofrd(&goffd,&obyte,&length,&goftype,
                                 (struct DDR *)NULL,&gofrec);
                goftype = XYPAIRS;
                status = c_gofrd(&goffd,&obyte,&length,&goftype,
                                 (struct DDR *)NULL,&gofrec);
            }
            else
            {
                goftype = gtype;
                status = c_gofrd(&goffd,&obyte,&length,&goftype,
                                 (struct DDR *)NULL,&gofrec);
            }


            /* If end of file, you've already listed the last one. Break out
               of loop.
               -------------------------------------------------------------*/
            if (status == E_EOF)
                break;
            else if (status != E_SUCC)
            {
                c_errmsg("Error reading GOF file","dspgof-read",&nonfatal);
                fclose(lfp);
                c_clean(&vblock,"dspgof",pfile,&one);
            }

            /* If record isn't deleted and the attributes either match or
               were not specified, then list out the record.
               ----------------------------------------------------------*/
            if (strcmp(gofrec.plrec.label,DELREC) != 0)
            {
                switch (gtype)
                {
                    case PNT:
                    case ANOT:
                        for (i = 0; i < numattr; i++)
                        {
                            if((strcmp(gofrec.ptrec.attrb[slist[i]],
                                            attr[i])==0) || 
                               ((strcmp(attr[i],ATHAT)==0) &&
                                (strcmp(gofrec.ptrec.attrb[slist[i]],"")!=0)))
                            {
                                ptd = printrec(rec,gtype,&gofrec,
                                        &gofhdr.hdrec,prtpts,lfp);
                                ptdatt = TRUE;
                            }
                        }
                        break;
                    case FPOLIN:
                        for (i = 0; i < numattr; i++)
                        {
                            if((strcmp(gofrec.plrec.attrb[slist[i]],
                                       attr[i])==0) || 
                               ((strcmp(attr[i],ATHAT)==0) &&
                                (strcmp(gofrec.plrec.attrb[slist[i]],"")!=0)))
                            {
                                ptd = printrec(rec,gtype,&gofrec,
                                        &gofhdr.hdrec,prtpts,lfp);
                                ptdatt = TRUE;
                            }
                        }
                        break;
                }   /* Switch gtype */

                /* If record not already printed, then the current record
                   needs to be searched in RECNUM list. If record is matched
                   (found) then list, otherwise goto top of loop.
                   ---------------------------------------------------------*/
                if (!ptdatt)
                {
                    if (recno[0] != 0)                  /* Record */
                    {
                        found = FALSE;
                        for ( i = 0; i < numrec; i++)
                            if (rec == recno[i])
                            {
                                found = TRUE;
                                break;
                            }
                        if (!found) continue;
                    }

                    if ((found) || ((numattr == 0) && (numrec == 0)))
                        ptd = printrec(rec,gtype,&gofrec,&gofhdr.hdrec,prtpts,
                                       lfp);
                }

                /* If one of the print routines generated output,
                   printed = TRUE.
                   ----------------------------------------------*/
                if (ptd) 
                    printed = TRUE;

                ptdatt = FALSE;
            }  /* If record is not deleted */
        }  /* For each record */
    } /* If sbcommand G */
    /* If the output list is empty, output a message to the user
       ---------------------------------------------------------*/
    if (!printed)
    {
        fclose(lfp);
        c_delfil(pfile[0]);
        c_errmsg("Search found no records or attributes","dspgof-nolist",
                 &nonfatal);
        c_errmsg("Fatal error encountered","dspgof-fatal",&fatal);
    }
    fprintf(lfp,"\nEnd of listing.\n");

    /* Output completion message
       -------------------------*/
    fclose(lfp);
    if (prtfil)
    {
        outpt = 1;
        strcpy (pfile[0],ufile);
    }
    c_complt("DSPGOF",pfile,&outpt);

    return EXIT_SUCCESS;
}
