/******************************************************************************
PROGRAM:	DSPLOG

PURPOSE: 	Displays a TAE session log.  Three subcommands allow the user 
		different processing options.  The -ALL subcommand writes the 
		session log from beginning to end.  The -TIME subcommand writes
		records from the session log based on a starting date and time
		to an ending date and time.  The -DIR subcommand displays a 
		directory listing of session logs.  

PROGRAM HISTORY:
	Created by F. Irani, SAR, 3/27/86
	Modified by R. Mckinney, USGS/EROS Data Center, 10/10/86
	Modified by L. Huewe, USGS/EROS Data Center, 10/90
	    - LAS v5_0 retrocode.
        Modified by T. Mittan USGS/EROS Data Center, 9/92 -Ival to Sval

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	DSPLOG must be run under TAE.

PROJECT:	LAS

ALGORITHM:

	Get the user specified parameters.
	Open the output print file and the input TAE session log.
	If the subcommand chosen was -ALL or the subcommand chosen was -TIME
	  and no dates or times were specified
	    Print the entire session log.
	Else
	    Print the selected records from the session log based on the
	      user specified dates and times.
	Close the session log and print files.
	Output a completion message.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
struct	PARBLK	vblock;		/* TAE variable block. 			      */

    FILE	*fp,		/* Pointer to session log file.		      */
		*lfp;		/* Pointer to output file.  	      	      */

	char	infile[CMLEN],	/* Session log file name.		      */
		pfile[1][CMLEN], 	/* Name of the temporary file to write to.    */
		subcmd[5],	/* Subcommand chosen.			      */
		ufile[1][CMLEN];	/* Name of the user specified output file.    */

	long	bdate[3],	/* Beginning month, day and year.	      */
		btime[3],	/* Beginning hours, minutes and seconds.      */
		edate[3],	/* Ending month, day and year.		      */
		etime[3],	/* Ending hours, minutes and seconds.         */
		fatal = -1, /* Indicates a fatal error (long)            */
        nonfatal = 1, /* Indicates a nonfatal error (long)        */
        numout = 0,	/* Number of output files created.     	      */
		one = 1,	/* Constant one.			      */
		prtfil=FALSE,	/* Flag indicating whether output has been
				   routed to a file.			      */
		status;		/* Status indicator.			      */


/*
   Get the user specified parameters.
*/
getpar(subcmd,bdate,edate,btime,etime,infile,&prtfil,pfile[0],ufile[0],&vblock);
    
/*
   Open the output print file for writing.
*/
lfp = fopen(pfile[0],"w");
if (lfp == NULL)
    {
    c_errmsg("Error opening print file","dsplog-opnprt",&nonfatal);
    c_errmsg("Fatal error encountered","dsplog-fatal",&fatal);
    }

/*
   Open the session log.
*/
fp = fopen(infile,"r");
if (fp == NULL)
    {
    fclose(lfp);
    c_errmsg("Error opening session log file","dsplog-opnses",&nonfatal);
    c_clean(&vblock,"dsplog",pfile,&one);
    }

/*
   Process the session log file.
*/
if (((*subcmd == 'T') && (bdate[0] == 999) && (edate[0] == 999) && 
  (btime[0] == 999) && (etime[0] == 999)) || (*subcmd == 'A'))
    prtall(lfp,fp);
else
    {
    status = prtsub(lfp,fp,bdate,edate,btime,etime);
    if (status != E_SUCC)
	{
        fclose(fp);
        fclose(lfp);
        c_errmsg("No records found within the specified date and time range",
	    "dsplog-range",&nonfatal);
        c_clean(&vblock,"dsplog",pfile,&one);
	}
    }

/*
   Close files and output completion message.
*/
status = fclose(fp);
if (status < 0)
    {
    fclose(lfp);
    c_errmsg("Error closing session log file","dsplog-clsfil",&nonfatal);
    c_clean(&vblock,"dsplog",pfile,&one);
    }
status = fclose(lfp);
if (status < 0)
    {
    c_errmsg("Error closing print file","dsplog-clsfil",&nonfatal);
    c_errmsg("Fatal error encountered","dsplog-fatal",&fatal);
    }
if (prtfil)
    numout = 1;
c_complt("DSPLOG",ufile,&numout);

return EXIT_SUCCESS;

}
/******************************************************************************
FUNCTION:	GETPAR

PURPOSE: 	Get the user specified parameters.

ALGORITHM:

	Initialize TAE and LAS.
	Determine the subcommand chosen.
	Retrieve the name of the session log file to process.
	If no file was specified
	   Set the session log file name to session.tsl.
	Get the print destinations.
	If the subcommand chosen was -TIME
	    Retrieve the beginning date, convert the values to integer and
	      perform some basic error checking.
	    Retrieve the ending date, convert the values to integer and
	      perform some basic error checking.
	    Retrieve the beginning time, convert the values to integer and
	      perform some basic error checking.
	    Retrieve the ending time, convert the values to integer and
	      perform some basic error checking.
	Return to the main routine.

*******************************************************************************/


void getpar
(
    char *subcmd,		/* The subcommand.  (O) 		      */
    long bdate[],		/* Beginning month,day and year. (O)          */
    long edate[],		/* Ending month, day and year (O).	      */
    long btime[],		/* Beginning hours, minutes and seconds.  (O) */
    long etime[],		/* Ending hours, minutes and seconds  (O).    */
    char *infile,		/* The input session log file.  (O)           */
    long *prtfil,		/* Indicates whether session log information 
				   is to be output to a file.  (O)	      */
    char *pfile,			/* Temporary file to write to for TERM, LP or
				   user specified file name.  (O)	      */
    char *ufile,			/* User specified file name.  (O)	      */
    struct PARBLK *vblock	/* TAE variable block.  (O) 		      */
)
{
struct VARIABLE	 *var;		/* Pointer to variable in parameter block.    */

int	status;			/* Status indicator.			      */

long term;          /* Indicates whether log infomation is to be
                       output to the terminal  */
long length;        /* String length                  */
long std_out;       /* Standard output.                 */
long lp;            /* Indicates whether log information is to be output
                       to the line printer  */

long fatal = -1;  /* Indicates fatal error message (long)  */
long nonfatal = 1; /* Indicates nonfatal error message (long) */
/*
   Initialize TAE and LAS.
*/
c_inlas(vblock,&std_out);

/*
   Get the subcommand.
*/
var = p_find(vblock,"_SUBCMD");
strcpy(subcmd,SVAL(*var,0));
length = (long) strlen(subcmd);
c_low2up(subcmd,&length);

/*
   Get the session log file name.
*/
var = p_find(vblock,"INFILE");
if ((*var).v_count == 0)
    strcpy(infile,"session.tsl");
else
    strcpy(infile,SVAL(*var,0));

/*
   Get the print destination.
*/
strcpy (pfile,"dsplog");
status = c_setprt(vblock,&term,prtfil,&lp,pfile,ufile);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","dsplog-fatal",&fatal);

if (*subcmd == 'T')
    {
/*
   Get the beginning and ending dates and convert them to integer values. 
*/
    var = p_find(vblock,"BEGDATE");
    if ((*var).v_count == 0)
	bdate[0] = bdate[1] = bdate[2] = 999;
    else
	{
	bdate[0] = bdate[1] = bdate[2] = 0;
	status = sscanf(SVAL(*var,0),"%d/%d/%d",&bdate[0],&bdate[1],&bdate[2]);
	if ((status <= 0) || (bdate[0] > 12) || (bdate[0] < 1) || 
	    (bdate[1] > 31) || (bdate[1] < 1) || (bdate[2] > 99) || 
	    (bdate[2] < 0))
	    {
	    c_errmsg("Invalid date format","dsplog-invdate",&nonfatal);
	    c_errmsg("Fatal error encountered","dsplog-fatal",&fatal);
	    }
	}

    var = p_find(vblock,"ENDDATE");
    if ((*var).v_count == 0)
	edate[0] = edate[1] = edate[2] = 999;
    else
	{
	edate[0] = edate[1] = edate[2] = 0;
	status = sscanf(SVAL(*var,0),"%d/%d/%d",&edate[0],&edate[1],&edate[2]);
	if ((status <= 0) || (edate[0] > 12) || (edate[0] < 1) || 
	    (edate[1] > 31) || (edate[1] < 1) || (edate[2] > 99) ||
	    (edate[2] < 0)) 
	    {
	    c_errmsg("Invalid date format","dsplog-invdate",&nonfatal);
	    c_errmsg("Fatal error encountered","dsplog-fatal",&fatal);
	    }
	}
/*
   Get the beginning and ending times and convert them to integer values. 
*/
    var = p_find(vblock,"BEGTIME");
    if ((*var).v_count == 0)
	btime[0] = btime[1] = btime[2] = 999;
    else
	{
	btime[0] = btime[1] = btime[2] = 0;
	status = sscanf(SVAL(*var,0),"%d:%d:%d",&btime[0],&btime[1],&btime[2]);
	if ((status <= 0) || (btime[0] > 24) || (btime[0] < 0) || 
	  (btime[1] > 60) || (btime[1] < 0) || (btime[2] > 60) ||
	  (btime[2] < 0))
	    {
	    c_errmsg("Invalid time format","dsplog-invtime",&nonfatal);
	    c_errmsg("Fatal error encountered","dsplog-fatal",&fatal);
	    }
	}
	
    var = p_find(vblock,"ENDTIME");
    if ((*var).v_count == 0)
	etime[0] = etime[1] = etime[2] = 999;
    else
	{
	etime[0] = etime[1] = etime[2] = 0;
	status = sscanf(SVAL(*var,0),"%d:%d:%d",&etime[0],&etime[1],&etime[2]);
	if ((status <= 0) || (etime[0] > 24) || (etime[0] < 0) || 
	  (etime[1] > 60) || (etime[1] < 0) || (etime[2] > 60) ||
	  (etime[2] < 0))
	    {
	    c_errmsg("Invalid time format","dsplog-invtime",&nonfatal);
	    c_errmsg("Fatal error encountered","dsplog-fatal",&fatal);
	    }
	}
    }
return;
}
