/*****************************************************************************
NAME				 DSPTIE 

PURPOSE    Reads a tie point file (TPS, MTP, or TPL) and displays it 
           on the user's terminal, the line printer, or to a file


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    2/89        EDC      Original development
  5.01     D. Steinwand   12/89        EDC      Reworked report format,
						 added options
  5.1      D. Etrheim      2/93        EDC	Removed the limit on the number
						of points. 
PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
	Get the point report options
	Get the name of the map identifier file, if given
	Get the output device parameter
	Get the subcommand chosen
	Open file for output
	Get the tie point file name for the subcommand chosen
	Write the tie point to the appropriate output device
	Print completion message & exit
*****************************************************************************/
#include <stdlib.h>
#include "dsptie.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK  vblk;		/* TAE variable block */
struct VARIABLE *var;		/* TAE variable value... */
FILE *filep;			/* File pointer to output file */
long  i;			/* Loop counter */
long  all;			/* Boolean--report all tie points? */
long  num_pts;			/* Number of point id's entered */
long  std_out;			/* Standard output LUN for inlas */
long  mode = IREAD;		/* IREAD flag for getnam routine */
long  fatal = -1;		/* Fatal error flag for c_errmsg */
long  zero = 0;			/* Constant of zero */
long  access = IUPDATE;		/* Access code for output host file */
long  num_out;			/* Number of output files 0 or 1 */
long  term, file, ln_pr;	/* Print option flags */
long  flags[FLAG_DIM];		/* Boolean flags */
char  fname[CMLEN];		/* Host name of tie point file */
char  temp_file[CMLEN];		/* Temporary file name for output */
char  filename[CMLEN];		/* Partial file name (wo/ext) for complt */
char  out_file[CMLEN];		/* Name of output file for complt */
char  mapfile[CMLEN];		/* Name of file containing map identifiers */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the point identifers to report.  Up to 10 pt_id's may be entered.  If
   the user enters the NULL value, all tie points in the file will be reported
  ---------------------------------------------------------------------------*/
var = p_find(&vblk, "PTID");
num_pts = (*var).v_count;
all = FALSE;
if (num_pts == 0) all = TRUE;
else for (i = 0; i < num_pts; i++) add_to_list(SVAL(*var,i));

/* Get the name of the map identifier file
  ---------------------------------------*/
if(c_getnam(&vblk, "INLT", &zero, mapfile, "lt", &mode) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);
flags[MAP_IDENT] = (strcmp(mapfile, "") == 0) ? OFF: ON;

/* Get the print option.  Make a file name (if neccessary) and open the
   temporary file
  --------------*/
strcpy(temp_file, "dsptie");
if(c_setprt(&vblk, &term, &file, &ln_pr, temp_file, filename) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);
if (file == 1)
   {
   c_mknam(filename, out_file, "prt", &access);
   num_out = 1;
   }
else
   {
   strcpy(out_file, " ");
   num_out= 0;
   }

filep = fopen(temp_file, "w");

/* Get the subcommand chosen, the name of the tie point file, and print the
   report to the specified destination for all three subcommands.
  --------------------------------------------------------------*/
var = p_find(&vblk, "_SUBCMD");
if (strcmp(SVAL(*var, 0), "TPS") == 0)
   {
   if(c_getnam(&vblk, "INTS", &zero, fname, "tps", &mode) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);

   var = p_find(&vblk, "LISTOPT");
   flags[IMG_COORD] = (strcmp(SVAL(*var, 0), "ON") == 0) ? ON: OFF;
   flags[GEO_COORD] = (strcmp(SVAL(*var, 1), "ON") == 0) ? ON: OFF;
   flags[ELEVATION] = (strcmp(SVAL(*var, 2), "ON") == 0) ? ON: OFF;
   flags[ZOOM_FACT] = (strcmp(SVAL(*var, 3), "ON") == 0) ? ON: OFF;
   flags[HEADERREC] = (strcmp(SVAL(*var, 4), "ON") == 0) ? ON: OFF;
   flags[CORR_RECS] = (strcmp(SVAL(*var, 5), "ON") == 0) ? ON: OFF;

   if(show_tps(fname, filep, mapfile, all, flags) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);
   }
if (strcmp(SVAL(*var, 0), "MTP") == 0)
   {
   if(c_getnam(&vblk, "INMT", &zero, fname, "mtp", &mode) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);

   var = p_find(&vblk, "LISTOPT");
   flags[REF_COORD] = (strcmp(SVAL(*var, 0), "ON") == 0) ? ON: OFF;
   flags[SEA_COORD] = (strcmp(SVAL(*var, 1), "ON") == 0) ? ON: OFF;
   flags[GEO_COORD] = (strcmp(SVAL(*var, 2), "ON") == 0) ? ON: OFF;
   flags[ELEVATION] = (strcmp(SVAL(*var, 3), "ON") == 0) ? ON: OFF;
   flags[ZOOM_FACT] = (strcmp(SVAL(*var, 4), "ON") == 0) ? ON: OFF;
   flags[HEADERREC] = (strcmp(SVAL(*var, 5), "ON") == 0) ? ON: OFF;
   flags[CORR_RECS] = (strcmp(SVAL(*var, 6), "ON") == 0) ? ON: OFF;

   if(show_mtp(fname, filep, mapfile, all, flags) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);
   }
if (strcmp(SVAL(*var, 0), "TPL") == 0)
   {
   if(c_getnam(&vblk, "INTL", &zero, fname, "tpl", &mode) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);

   var = p_find(&vblk, "LISTOPT");
   flags[REF_COORD] = (strcmp(SVAL(*var, 0), "ON") == 0) ? ON: OFF;
   flags[SEA_COORD] = (strcmp(SVAL(*var, 1), "ON") == 0) ? ON: OFF;
   flags[GEO_COORD] = (strcmp(SVAL(*var, 2), "ON") == 0) ? ON: OFF;
   flags[ELEVATION] = (strcmp(SVAL(*var, 3), "ON") == 0) ? ON: OFF;
   flags[ZOOM_FACT] = (strcmp(SVAL(*var, 4), "ON") == 0) ? ON: OFF;
   flags[AFLAGCODE] = (strcmp(SVAL(*var, 5), "ON") == 0) ? ON: OFF;
   flags[HEADERREC] = (strcmp(SVAL(*var, 6), "ON") == 0) ? ON: OFF;
   flags[CORR_RECS] = (strcmp(SVAL(*var, 7), "ON") == 0) ? ON: OFF;

   if(show_tpl(fname, filep, mapfile, all, flags) != E_SUCC)
       c_errmsg("Fatal error encountered","dsptie-fatal",&fatal);
   }

/* Write completion message and exit program
  -----------------------------------------*/
fclose(filep);
c_complt("DSPTIE", out_file, &num_out);

return EXIT_SUCCESS;

}
