/****************************************************************************
NAME:				map_rep

PURPOSE:  Gets a map parameter record from file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Read the map file--search for mapid key; Return(not found) if not found
	Report map parameters to the user
	Return success
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "geompak.h"
#include "ltable.h"
#include "dsptie.h"
#include "labserv.h"
#include "protos.h"


struct MAP_PARMS    /* Structure containing mapid file parameters */
   {
      double ul_x;     /* Upper left map corner X coordinate */
      double ul_y;     /* Upper left map corner Y coordinate */
      double lr_x;     /* Lower right map corner X coordinate */
      double lr_y;     /* Lower right map corner Y coordinate */
      double inc_x;    /* Increment in X */
      double inc_y;    /* Increment in Y */
      double verify_x; /* Verify point X coordinate */
      double verify_y; /* Verify point Y coordinate */
      long scale;      /* Annotation field--scale of map */
      long npts_x;     /* Number  of coordinates in X */
      long npts_y;     /* Number  of coordinates in Y */
      char datum[80];  /* Annotation field--map datum */
      char maptype[80];    /* Annotation field--type of map */
      char mapdate[10];    /* Annotation field--date of map */
      char mapdesc[80];    /* Annotation field--map description & notes */
   };

static int read_mapfile
(
    char *mapfile,          /* Map file name */
    char *mapid,            /* Map identifier -- used as key */
    struct MAP_PARMS *map_params    /* Map registration parameters */
);



int map_rep
(
    char *mapid,			/* Map identifier */
    char *mapfile,			/* Map parameter file name */
    FILE *ptr			/* Print file pointer */
)
{
struct MAP_PARMS map_params;	/* Map registration parameters */

/* Read map parameter record.  Use mapid as key.  Return if not found
  ------------------------------------------------------------------*/
if(read_mapfile(mapfile, mapid, &map_params) != E_SUCC) return(E_FAIL);

/* Report map parameters to the user
  ---------------------------------*/
fprintf(ptr, "\n\nMap identifier:  %s\n\n", mapid);
fprintf(ptr, "Upper left latitude, longitude:  %d %d %f  %d %d %f\n",
   find_deg(map_params.ul_y), find_min(map_params.ul_y),
   find_sec(map_params.ul_y), find_deg(map_params.ul_x), 
   find_min(map_params.ul_x), find_sec(map_params.ul_x));
fprintf(ptr, "Lower right latitude, longitude:  %d %d %f  %d %d %f\n",
   find_deg(map_params.lr_y), find_min(map_params.lr_y),
   find_sec(map_params.lr_y), find_deg(map_params.lr_x), 
   find_min(map_params.lr_x), find_sec(map_params.lr_x));
fprintf(ptr, "Increment in X:  %f degrees  (%d points)\n",
   fabs(map_params.inc_x), map_params.npts_x);
fprintf(ptr, "Increment in Y:  %f degrees  (%d points)\n\n",
   fabs(map_params.inc_y), map_params.npts_y);
fprintf(ptr, "Verify point latitude, longitude:  %d %d %f  %d %d %f\n\n",
   find_deg(map_params.verify_y), find_min(map_params.verify_y), 
   find_sec(map_params.verify_y), find_deg(map_params.verify_x), 
   find_min(map_params.verify_x), find_sec(map_params.verify_x));
fprintf(ptr, "Map type:   %s\n", map_params.maptype);
fprintf(ptr, "Map scale:  1:%d\n", map_params.scale);
fprintf(ptr, "Map datum:  %s\n", map_params.datum);
fprintf(ptr, "Map date:   %s\n", map_params.mapdate);
fprintf(ptr, "Map description:  %s\n\n", map_params.mapdesc);
return(E_SUCC);
}

/****************************************************************************
NAME:				read_mapfile

PURPOSE:   Reads a labeled table file of map parameters 

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	If the file exists, open it.  Otherwise return fail
	Retrieve file label
	Set up pointers to file fields
	Initialize boolean values
	Loop until key (mapid) is found or until EOF
	   Get a record
	   If found, pack map parameter fields and break out of loop
	Close the file
	Return success
*****************************************************************************/
static int read_mapfile
(
    char *mapfile,			/* Map file name */
    char *mapid,			/* Map identifier -- used as key */
    struct MAP_PARMS *map_params	/* Map registration parameters */
)
{
struct TAB_DEF tab;		/* Tabular structure of label table */
struct VECTOR *mapidptr;	/* Pointer to mapid field */
struct VECTOR *unitptr;		/* Pointer to units field */
struct VECTOR *ul_yptr;		/* Pointer to ul corner y field */
struct VECTOR *ul_xptr;		/* Pointer to ul corner x field */
struct VECTOR *lr_yptr;		/* Pointer to lr corner y field */
struct VECTOR *lr_xptr;		/* Pointer to lr corner x field */
struct VECTOR *ver_yptr;	/* Pointer to verify pt y field */
struct VECTOR *ver_xptr;	/* Pointer to verify pt x field */
struct VECTOR *npts_yptr;	/* Pointer to npts in y field */
struct VECTOR *npts_xptr;	/* Pointer to npts in x field */
struct VECTOR *scaleptr;	/* Pointer to scale field */
struct VECTOR *datumptr;	/* Pointer to datum field */
struct VECTOR *typeptr;		/* Pointer to map type field */
struct VECTOR *dateptr;		/* Pointer to map date field */
struct VECTOR *descptr;		/* Pointer to map description field */
long status;			/* Function return status */
long mode = 0;			/* Assess mode for lsstat */
long count = 0;			/* Count of records read */
static long rec_num = 1;	/* Count of records found */

/* Open the labeled table file & retrieve the label
  ------------------------------------------------*/
if (c_lsstat(mapfile, &mode) != E_SUCC) return(E_FAIL);
open_tab(&tab, mapfile, IREAD);
get_vector(&tab);

/* Determine pointers to fields in the file
  ----------------------------------------*/
mapidptr = get_field_ptr(&tab, "ID");
if (mapidptr == NULL) { close_tab(&tab); return(E_FAIL); }
unitptr = get_field_ptr(&tab, "UNITS");
if (unitptr == NULL) { close_tab(&tab); return(E_FAIL); }
ul_yptr = get_field_ptr(&tab, "UL_Y");
if (ul_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
ul_xptr = get_field_ptr(&tab, "UL_X");
if (ul_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
lr_yptr = get_field_ptr(&tab, "LR_Y");
if (lr_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
lr_xptr = get_field_ptr(&tab, "LR_X");
if (lr_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
ver_yptr = get_field_ptr(&tab, "VER_Y");
if (ver_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
ver_xptr = get_field_ptr(&tab, "VER_X");
if (ver_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
npts_yptr = get_field_ptr(&tab, "NPTS_Y");
if (npts_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
npts_xptr = get_field_ptr(&tab, "NPTS_X");
if (npts_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
scaleptr = get_field_ptr(&tab, "SCALE");
if (scaleptr == NULL) { close_tab(&tab); return(E_FAIL); }
datumptr = get_field_ptr(&tab, "DATUM");
if (datumptr == NULL) { close_tab(&tab); return(E_FAIL); }
typeptr = get_field_ptr(&tab, "TYPE");
if (typeptr == NULL) { close_tab(&tab); return(E_FAIL); }
dateptr = get_field_ptr(&tab, "DATE");
if (dateptr == NULL) { close_tab(&tab); return(E_FAIL); }
descptr = get_field_ptr(&tab, "DESC");
if (descptr == NULL) { close_tab(&tab); return(E_FAIL); }

/* Initialize BOOLEAN variables
  ----------------------------*/
rec_num = 1;

/* Loop until key is found or EOF
 -------------------------------*/
for (;;)
   {
   status = get_record(&tab);
   if (status == E_FAIL) { close_tab(&tab); return(E_FAIL); }
   if (status == E_EOF) { rec_num = 1; close_tab(&tab); return(E_FAIL); }
   count++;
   if(strcmp(mapidptr->ptr.c, mapid) == 0) 
	{
	rec_num++;
	map_params->ul_y = *ul_yptr->ptr.r8;
	map_params->ul_x = *ul_xptr->ptr.r8;
	map_params->lr_y = *lr_yptr->ptr.r8;
	map_params->lr_x = *lr_xptr->ptr.r8;
	map_params->verify_y = *ver_yptr->ptr.r8;
	map_params->verify_x = *ver_xptr->ptr.r8;
	map_params->npts_y = *npts_yptr->ptr.i4;
	map_params->npts_x = *npts_xptr->ptr.i4;
        map_params->inc_y = -fabs(map_params->ul_y - map_params->lr_y) /
		(map_params->npts_y - 1);
        map_params->inc_x = fabs(map_params->ul_x - map_params->lr_x) / 
		(map_params->npts_x - 1);
	map_params->scale = *scaleptr->ptr.i4;
	strcpy(map_params->datum, datumptr->ptr.c);
	strcpy(map_params->maptype, typeptr->ptr.c);
	strcpy(map_params->mapdate, dateptr->ptr.c);
	strcpy(map_params->mapdesc, descptr->ptr.c);
        break;
	}
   }

/* Close the file & return
  -----------------------*/
close_tab(&tab);
return(E_SUCC);
}

/****************************************************************************
NAME:			find_deg, find_min, find_sec	

PURPOSE:  Extracts deg, min, or sec portions of an angle

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Extract portion of angle
	Return
*****************************************************************************/
int find_deg
(
    double angle			/* Angle in total degrees */
)
{
long sign;			/* Sign of angle */
long deg;			/* Degrees portion of angle */
long minute;			/* Minutes portion of angle */
double sec;			/* Seconds portion of angle */
deg = (long)angle;
sign = 1;
if (deg < 0) sign = -1;
deg = (long) (fabs(angle));
minute =  (long) ((fabs(angle) - deg) * 60.0);
sec =  (((fabs(angle) - deg) * 60.0) - minute) * 60.0;
if (sec >= 59.999) minute++;
if (minute >= 60) deg++;
deg *= sign;
return(deg);
}

int find_min
(
    double angle			/* Angle in total degrees */
)
{
double sec;			/* Seconds portion of angle */
long minute;			/* Minutes portion of angle */
angle = fabs(angle);
angle -= (long)angle;
minute = (long) (angle * 60.0);
sec = ((angle * 60.0) - minute) * 60.0;
if (sec > 59.999) minute++;
return(minute);
}

double find_sec
(
    double angle			/* Angle in total degrees */
)
{
angle = fabs(angle);
angle -= (long)angle;
angle *= 60.0;
angle -= (long)angle;
angle *= 60.0;
if (angle > 59.999) angle -= 60.0;
return(angle);
}
