/*****************************************************************************
NAME				 show_mtp

PURPOSE    Writes a merged tie point file to a user defined output device.
	   User's terminal, an ASCII file, or the line printer are valid
	   output devices.

PROJECT			LAS

ALGORITHM DESCRIPTION
	Print the name of the tie point file to the report device
	Open the tie point file
	Print misc header information
	Print projection information, if available (Only if user requested)
	Set up point report headings
	Print the point report in tabular form
	Close the tie point file
 	If the user requests, print the correlation report 
	If the user requests, print the map parameters for the listed tie points
	Return

ALGORITHM REFERENCES		none
*****************************************************************************/
#include "dsptie.h"
#include "protos.h"

int show_mtp
(
    char *fname,                   /* Name of TPL file */
    FILE *filep,                   /* File pointer for output */
    char *mapfile,                 /* Name of map identifer file */
    long all,                      /* Boolean--all points processed? */
    long *flags                   /* Report flags */
)
{
struct MTPDATA data;		/* Merged tie point data records */
struct MTPHEAD header;		/* Merged tie point header records */
struct PTDESC *tie;			/* Pointer to tie point file descriptor */
long count;             	/* Loop counters */
long acc = IREAD;		/* File access mode */
long ptyp = MTP;		/* Tie point type is MTP */
long status;			/* Return status flag */
char mapid[20];                 /* Map identifier */
char proj_str[80];		/* Character version of projection code */
char dtm_str[46];		/* Character version of datum code */

 
/* Output MTP file name
  --------------------*/
fprintf(filep,"Merged Tie Point File Name:\n");
fprintf(filep,"  %s\n\n", fname);

/* Open MTP file
  -------------*/
if (c_ptopen(&tie, fname, &ptyp, &acc, &header) != E_SUCC) return(E_FAIL);

/* Print misc. header information
  ------------------------------*/
if (strcmp(header.chip_mode,"NO-CHIPS") == 0)
   fprintf(filep,"Subimage chips are not being used\n\n");
if (strcmp(header.chip_mode,"CHIP-TEM") == 0)
   fprintf(filep,"Subimage chips used to establish temporal registration\n\n");
if (strcmp(header.chip_mode,"CHIP-FUL") == 0)
  fprintf(filep,"Subimage chips used to establish geographic registration\n\n");
if (strcmp(header.chip_mode,"CHIP-PRE") == 0)
   fprintf(filep,"Subimage chips used are prescaled and/or rotated\n\n");
if (strcmp(header.ref_name,"") != 0)
   {
   fprintf(filep,"Reference image points were obtained from:\n");
   fprintf(filep,"  %s\n\n",header.ref_name);
   }

if (strcmp(header.sea_name,"") != 0)
   {
   fprintf(filep,"Search image points were obtained from:\n");
   fprintf(filep,"  %s\n\n",header.sea_name);
   }

/* Print projection information, if available
  ------------------------------------------*/
if (flags[HEADERREC])
 {
 if (header.proj_valid == GEO_VALID)
   {
   fprintf(filep,"\nProjection information:\n\n");
   c_prostr(&(header.code), proj_str);
   fprintf(filep,"  Projection code:   %s\n",proj_str);
   fprintf(filep,"  Projection zone:   %d\n", header.zone);
   c_dtmstr(&(header.datum),dtm_str);
   fprintf(filep,"  Datum code:        %s\n", dtm_str);
   fprintf(filep,"  Coordinate units:  %s\n", header.units);
   fprintf(filep,"  Projection Distance along lines:    %f\n",header.pdist[0]);
   fprintf(filep,"  Projection Distance along samples:  %f\n",header.pdist[1]);
   fprintf(filep,"\n  Projection parameters:\n");
   fprintf(filep,"    1:  %16.5f   2:  %16.5f   3:  %16.5f\n",
     header.projprms[0], header.projprms[1],header.projprms[2]);
   fprintf(filep,"    4:  %16.5f   5:  %16.5f   6:  %16.5f\n",
     header.projprms[3], header.projprms[4],header.projprms[5]);
   fprintf(filep,"    7:  %16.5f   8:  %16.5f   9:  %16.5f\n",
     header.projprms[6], header.projprms[7],header.projprms[8]);
   fprintf(filep,"   10:  %16.5f  11:  %16.5f  12:  %16.5f\n",
     header.projprms[9], header.projprms[10],header.projprms[11]);
   fprintf(filep,"   13:  %16.5f  14:  %16.5f  15:  %16.5f\n",
     header.projprms[12], header.projprms[13],header.projprms[14]);
   fprintf(filep,"\n  Projection coordinates of output space corners:\n\n");
   fprintf(filep,"  Upper left corner  (l/s):  %18.5f  %18.5f\n", 
		header.corners[0], header.corners[1]);
   fprintf(filep,"  Lower left corner  (l/s):  %18.5f  %18.5f\n", 
		header.corners[2], header.corners[3]);
   fprintf(filep,"  Upper right corner (l/s):  %18.5f  %18.5f\n", 
		header.corners[4], header.corners[5]);
   fprintf(filep,"  Lower right corner (l/s):  %18.5f  %18.5f\n\n", 
		header.corners[6], header.corners[7]);
   }
 else fprintf(filep,
   "\nProjection information not contained in header record\n\n");
 }

/* Set up Point report headers
  ---------------------------*/
if (flags[REF_COORD]) fprintf(filep,"     Reference      ");
if (flags[SEA_COORD]) fprintf(filep,"   Nominal Search   ");
if (flags[GEO_COORD]) fprintf(filep," Geographic/Projection/User-Def");
if (flags[ELEVATION]) fprintf(filep,"          ");
if (flags[ZOOM_FACT]) fprintf(filep,"      ");
if (flags[MAP_IDENT]) fprintf(filep,"                     ");
fprintf(filep,"\n");
if (flags[REF_COORD]) fprintf(filep," Image Coordinates  ");
if (flags[SEA_COORD]) fprintf(filep," Image Coordinates  ");
if (flags[GEO_COORD]) fprintf(filep,"         Coordinates            ");
if (flags[ELEVATION]) fprintf(filep,"          ");
if (flags[ZOOM_FACT]) fprintf(filep," Zoom ");
if (flags[MAP_IDENT]) fprintf(filep,"        Map          ");
fprintf(filep,"      Point\n");
if (flags[REF_COORD]) fprintf(filep,"   Line     Sample  ");
if (flags[SEA_COORD]) fprintf(filep,"   Line     Sample  ");
if (flags[GEO_COORD]) fprintf(filep," Latitude (y)   Longitude (x)   ");
if (flags[ELEVATION]) fprintf(filep,"Elevation ");
if (flags[ZOOM_FACT]) fprintf(filep," Fact ");
if (flags[MAP_IDENT]) fprintf(filep,"    Identifier       ");
fprintf(filep,"   Identifier\n");
if (flags[REF_COORD]) fprintf(filep,"------------------- ");
if (flags[SEA_COORD]) fprintf(filep,"------------------- ");
if (flags[GEO_COORD]) fprintf(filep,"------------------------------- ");
if (flags[ELEVATION]) fprintf(filep,"--------- ");
if (flags[ZOOM_FACT]) fprintf(filep,"----- ");
if (flags[MAP_IDENT]) fprintf(filep,"-------------------- ");
fprintf(filep,"----------------\n");

/* Do point report
  ---------------*/
for(;;)
   {
   status = c_ptio(&tie, &data);
   if (status == E_EOF) break;
   if (status == E_FAIL) return(E_FAIL);

   if ((all) || (in_list(data.pt_id)))
        {
        if (flags[REF_COORD])
          fprintf(filep,"%9.2f %9.2f ", data.ref_coord[0], data.ref_coord[1]);
        if (flags[SEA_COORD])
          fprintf(filep,"%9.2f %9.2f ", data.nominal[0], data.nominal[1]);
        if (flags[GEO_COORD])
          fprintf(filep,"%15.3f %15.3f ", data.geo_coord[0], data.geo_coord[1]);
        if (flags[ELEVATION]) fprintf(filep,"%9.2f ", data.elevation);
        if (flags[ZOOM_FACT]) fprintf(filep,"%5.2f ", data.zoom_factor);
        if (flags[MAP_IDENT])
           {
           fprintf(filep,"%-20s ", data.ref_chip);
           list_map(data.ref_chip);
           }
        fprintf(filep,"%s\n", data.pt_id);
        }
   }
c_ptclse(&tie);

/* Do Correlation report
  ---------------------*/
if(flags[CORR_RECS])
   {
   fprintf(filep,"\f");
   fprintf(filep,"Correlation Report\n");

/* Reopen MTP file
  ---------------*/
   if (c_ptopen(&tie, fname, &ptyp, &acc, &header) != E_SUCC) return(E_FAIL);

   for(count = 1;;count++)
       {
       status = c_ptio(&tie, &data);
       if (status == E_EOF) break;
       if (status == E_FAIL) return(E_FAIL);

       if ((all) || (in_list(data.pt_id)))
        {
        if (count > 9)
           {
           count = 1;
           fprintf(filep, "\f");
           }
        fprintf(filep,"\n%s\n", data.pt_id);
        fprintf(filep,"   Width & height of reference subimage:      %6d %6d\n",
                     data.chip_size[0], data.chip_size[1]);
        fprintf(filep,"   Req horizontal & vertical search offsets:  %6d %6d\n",
                    data.offsets[0], data.offsets[1]);
        fprintf(filep,"   Reference chip file name:  %s\n", data.ref_chip);
        fprintf(filep,"   Search chip file name:     %s\n", data.sea_chip);
        }
      }
   c_ptclse(&tie);
   }

/* print map report
  ----------------*/
if (flags[MAP_IDENT])
   {
   fprintf(filep, "\f");
   for(count = 1;;count++)
        {
        get_map(mapid);
        if (strcmp(mapid,"") == 0) break;
        if (count > 3)
           {
           count = 1;
           fprintf(filep, "\f");
           }
        map_rep(mapid, mapfile, filep);
        }
   }
return(E_SUCC);
}
