/******************************************************************************
 NAME:   			PROCES 

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   Loop on bands
     Initialize histogram
     Loop on lines
       Perform gradient calcutaltion, three lines at a time
       Update histogram
     Find egdge threshold
     Extract binary image and write to output file
   Update the output ddr
   Save the minmax values
   Close the input and output image files
ALGORITHM REFERENCES:		none
******************************************************************************/

#include "las.h"
#include "pixman.h"
#include "util.h"
#include "typlim.h"
#include "protos.h"

int proces
(
    struct PARBLK *vblock,	/* TAE parameter block			     */
    char hosin[][CMLEN],	/* host names of input images		     */
    char hosout[][CMLEN],   /* host name of output image		     */
    long nmrimg,		/* number of input images                    */
    long (*window)[4],		/* window specification of input images      */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long totbnd,		/* total number of bands to be processed     */
    long odtype,		/* data type of output image                 */
    long threshold,		/* Upper edge limit			     */
    double efract,               /* fraction of pixels classifed as edges     */
    long pixval,		/* Edge pixel value			     */
    long msgtim		/* elapsed time between display messages     */
)
{
extern long curcat;		/* current processing band		*/
extern long curent;		/* current processing line		*/

static double  maxval[MAXBND];	/* max pixel value array                */
static double  minval[MAXBND];	/* min pixel value array                */
double  line_inc;		/* line increment 			*/
double  samp_inc;		/* sample increment			*/

struct FDESC  *inptr;                 	/* Input image pointer 			*/
struct FDESC  *outptr;                	/* Ouput image pointer 			*/

unsigned char *edge;            /* Enhanced edge line			*/
unsigned char *line1 = NULL, 		/* image I/O lines			*/
              *line2 = NULL,
              *line3 = NULL;
unsigned char *outbuf = NULL;		/* image I/O buffer                     */
unsigned char *temp;		/* Temporary image I/O line		*/

char	 minmax[4];	/* min max flag                         */


static  long  opt[3]={0,0,0};	/* option array				*/
long	access;			/* image I/O access type                */
long	band;			/* loop index counting the bands        */
long 	count;			/* pxhist variable			*/
long	dtype = 0;		/* input data type			*/
long    fatal = -1;		/* fatal error flag			*/
long 	Gp5;			/* Enhanced pixel value			*/
long 	histogram[MAXUTINY+1];	/* Histogram of enhanced edge values	*/
long 	i,j;                    /* Loop counters 			*/
long	io_line = 1;		/* I/O line indicator           	*/
long	out_line = 1;		/* I/O line indicator           	*/
long 	lcnt;			/* line counter			        */
long	maxdat = MAXUTINY;  	/* Max data type limit (255)		*/
long 	mincnt;			/* Target minimum number of non-edge pixels  */
long	mindat = MINUTINY;	/* Min data type limit (0)		*/
long	msg_flag=ONECAT;	/* flag for porcessing message          */
long 	nl;			/* number of image lines                */
long 	nonedge;		/* Actual nuber of non-edge pixels	*/
long	nonfatal = 1;		/* nonfatal error flag                  */
long 	ns;			/* number of image samples              */
long    one = 1;		/* one					*/
long 	p1,p2,p3,	        /* Image pixel values			*/
   	p4,p6,
 	p7,p8,p9;	
long	sl;			/* starting image line                  */
long	ss;			/* starting image sample                */
long	status;			/* function return status code          */
long    upddr_flag;	        /* flag for update DDR			*/
unsigned char   zero = 0;		/* zero					*/

/* Open input image(s)
----------------------*/
access = IREAD;
sl = window[0][0];
ss = window[0][1];
nl = window[0][2];
ns = window[0][3];
status = c_eopenr(&inptr,hosin[0],&nbands[0],&sl,&ss,&nl,&ns,
                  &access,&dtype,opt,&one);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","edgex-fatal",&fatal);
if (dtype != EBYTE)
   c_errmsg("Only BTYE images allowed","edgex-type",&fatal);

/* Open output image
--------------------*/
access = IWRITE;	
status = c_eopenr(&outptr,hosout[0],&totbnd,&one,&one,&nl,&ns,
                  &access,&odtype,opt,&one);
if (status != E_SUCC)
   c_clean(vblock,"edgex",hosout,&one);

/* Allocate I/O buffers
----------------------*/
if ((edge = (unsigned char *)malloc(ns*dtype)) == NULL ||
    (outbuf= (unsigned char *)malloc(ns*odtype)) == NULL ||
    (line1 = (unsigned char *)malloc(ns*dtype)) == NULL ||
    (line2 = (unsigned char *)malloc(ns*dtype)) == NULL ||
    (line3 = (unsigned char *)malloc(ns*dtype)) == NULL ) 
    {
    c_errmsg("Error allocating dynamic memory","edgex-alloc",&nonfatal);
    c_clean(vblock,"edgex",hosout,&one);
    }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("EDGEX","Processed","line","band"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
curcat = 1;
c_startm();

/* Main processing loop
-----------------------*/
for (band = 1;band<=totbnd;band++)
    {
    curent=1;
    for(i=0;i<=maxdat;i++)
        histogram[i]=0;
    maxval[band-1]=maxdat;
    minval[band-1]=mindat;

    /* read fist two lines  
    ---------------------- */ 
    io_line=one;
    if (c_eread(&inptr,&band,&io_line,line2,&one) == E_FAIL)
       c_clean(vblock,"edgex",hosout,&one);
    io_line++;
    if (c_eread(&inptr,&band,&io_line,line3,&one) == E_FAIL)
       c_clean(vblock,"edgex",hosout,&one);

    /* Zero first output line
    -------------------------*/
    out_line=one;
    c_pxfill(edge,&dtype,&ns,&zero);
    c_pxhist(&ns,edge,&dtype,histogram,&count);
    if (c_ewrite (&outptr, &band, &out_line, edge, &one) != ns)
       c_clean(vblock,"edgex",hosout,&one);

    /* First create output img of enhanced gadient pixel values
    ----------------------------------------------------------*/
    for (lcnt = 1; lcnt < nl-1; lcnt++)
       {
       temp = line1;
       line1= line2;
       line2= line3;
       line3= temp;
       io_line++;
       if (c_eread(&inptr,&band,&io_line,line3,&one) == E_FAIL)
          c_clean(vblock,"edgex",hosout,&one);
       edge[0]=0;
       edge[ns-1]=0;
       for(j=1;j<ns-1;j++)
          {
          p1 = line1[j-1];
          p2 = line1[j];
          p3 = line1[j+1];
          p4 = line2[j-1];
          p6 = line2[j+1];
          p7 = line3[j-1];
          p8 = line3[j];
          p9 = line3[j+1];
          if ((p1 > threshold) || (p2 > threshold) || (p3 > threshold) ||
              (p4 > threshold) || (p6 > threshold) ||
              (p7 > threshold) || (p8 > threshold) || (p9 > threshold))
              Gp5=mindat;
          else
              Gp5 = abs(p1-p9)+abs(p2-p8)+abs(p3-p7)+abs(p4-p6);
          if(Gp5>maxdat) 
              Gp5=maxdat;
          if(Gp5<mindat) 
              Gp5=mindat;
          edge[j] = (unsigned char) Gp5; 
          } /* Sample Loop */
       c_pxhist(&ns,edge,&dtype,histogram,&count);
       out_line++;
       if (c_ewrite (&outptr, &band, &out_line, edge, &one) != ns)
          c_clean(vblock,"edgex",hosout,&one);
       curent++;
       } /* Line Loop */
    curent++;

    /* Final edge write 
    -------------------*/
    c_pxfill(edge,&dtype,&ns,&zero);
    c_pxhist(&ns,edge,&dtype,histogram,&count);
    out_line++;
    if (c_ewrite (&outptr, &band, &out_line, edge, &one) != ns)
       c_clean(vblock,"edgex",hosout,&one);

    /* Find edge threshold histogram bin
    ------------------------------------*/
    mincnt = (long)((nl*ns)*(1.0-efract));
    for(nonedge=0,i=0;i<=maxdat;i++) 
       { 
       if(nonedge>=mincnt) break; 
       nonedge+=histogram[i];
       }
    if((nonedge-histogram[i-1]/2) >= mincnt) i--;
    nonedge = i;

    /* Close outptr and open as update
    ---------------------------------*/
    if (c_eclose(&outptr) != E_SUCC)
       c_errmsg("Nonfatal error encountered","edgex-warn",&one);
    access = UPDATE;	
    status = c_eopenr(&outptr,hosout[0],&totbnd,&one,&one,&nl,&ns,
                      &access,&dtype,opt,&one);
    if (status != E_SUCC)
       c_clean(vblock,"edgex",hosout,&one);

    /* Convert output gradient pixel values to edge values
    -----------------------------------------------------*/
    for(i=1;i<=nl;i++)
       {
       if (c_eread(&outptr,&band,&i,edge,&one) == E_FAIL)
          c_clean(vblock,"edgex",hosout,&one);
       for(j=0;j<ns;j++) 
          if(edge[j] < nonedge) 
             outbuf[j] = (unsigned char) mindat; 
          else 
             outbuf[j] = (unsigned char) pixval;
       if (c_ewrite(&outptr,&band,&i,outbuf,&one) != ns)
          c_clean(vblock,"edgex",hosout,&one);
       }
    curcat++;
    } /* Band Loop */

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,
		&line_inc,&samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
   c_clean(vblock,"edgex",hosout,&one);

/* Save associated min/max 
--------------------------*/
strcpy(minmax,"YES");
if (c_svmnmx(hosout[0],&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","edgex-warn",&one);

/* Close all image files in the group
-------------------------------------*/
if (c_eclose(&inptr) != E_SUCC)
   c_errmsg("Nonfatal error encountered","edgex-warn",&one);
if (c_eclose(&outptr) != E_SUCC)
   c_errmsg("Nonfatal error encountered","edgex-warn",&one);

/* Free buffers
--------------*/
free(edge); 
free(outbuf);
free(line1);
free(line2);
free(line3);

return(E_SUCC);
}
