C     *************************************************************************
C     NAME	EDIPSIN	
C
C     PURPOSE:	Reads an EDIPS 0.0 format tape and creates a LAS image.
C		Optionally, the header, annotation, ancillary, and trailer
C		records are dumped to disk files for later reference.  ASCII
C		files containing the ancillary information necessary for
C		systematic corrections and platform modeling are automatically
C		created for uncorrected MSS tapes.
C
C   
C	PROGRAMMER	  DATE		REASON
C	----------	  ----		------
C	K. Zanter	 8/l985         1.  CIPS  conversion to UNIX
C				        2.  PRF for extraction of necessary
C					    ancillary information needed
C					    for systematic corrections
C					    and platform modeling
C     				         NOTE:  The VMS version for this
C				                PRF was not implemented at
C						this time.
C	R. McKinney     12/1988	       Converted to LAS 5.0
C	B. Ailts	09/91		Retreived and placed the projection and
C					corner information in the DDR file.  
C					Since not all the projection information
C					is stored in the header records of the
C					tape,  certain assumptions were made
C					in order to fill in all the projection
C					parameters.  Using assumptions instead
C					of acual parameters means that the 
C					corner coordinates are not very accurate
C					The corner coordinates although not very
C					accurate are still placed in the DDR
C					because they are accurate enough for
C					rough transformations in the geometric
C					registration proces.
C	D.Etrheim	03/92		Made change to density to run on vax 
C	B.Davis		04/92		I believe the conversion to newlas of
C 8/85 added tcomp calls after all of the pxcopy calls that convert tape record
C fields to real values, before writing them to the output files.  The
C conversion occurred before c_pxsys.c existed.  This logic now exists in pxsys.
C These tcomp calls were doing a twos compliment on negative values only.  It
C is believed this was the only difference between the way pn9050 utx fortran
C stores reals, and the way they existed on tape.  The hardware making the
C tapes at GSFC was a SEL, similar to those we had, and precursor to our
C pn9050.  It is unclear whether the SEL hardware, or some GSFC software is
C creating IBM type reals for tape fields.  The way that real numbers are
C stored in the FL fields of the MSS tape records (see document referenced
C below) is by coincidence the exact same way that the pn9050 at EDC stores
C fortran reals, except for negatives.  This is the machine on which newlas
C code was developed, so a straight copy of bytes from the tape read buffer to
C the real variable luckily produced the intended result.  To be able to read
C these real fields onto any unix system, I am now employing pxsys for this
C conversion.  I followed each pxcopy call by a call to pxsys.  I told pxsys to
C convert reals (both r*4 and r*8) from SYSIBMMVS to outsys format.  This
C produces the same values on all systems for real fields in the ancillary
C files that are later used for geometric correction.  The exception to this
C is that the r*4 values are only exact to about 6 significant digits, but this
C is to be expected because that is all the precision gauranteed to be
C contained in 4 bytes.
C
C I commented out all of the tcomp calls, and therefore took it off the list of
C objects in Makefile and make.com.  tcomp.c also had a define for EDOUBLE to
C be 8.  The define of EDOUBLE in sysdef.h is 5.  This was potentially
C dangerous, though tcomp.c was using its own value correctly.  I changed the
C EDOUBLE in tcomp.c to EDUBBLE.  Although it is no longer being used, if
C someone used it for an example, it could cause trouble.  I also got rid of
C the warning disclamer in the Makefile about ignoring the redefine of EDOUBLE.
C I do not think we should have ignored this warning.
C
C In order to test this program, the ancillary output files need to be verified.
C They should be compared against those created by an earlier verison of this
C program, but also, the output image of edipsin should be run through the
C program correct, to verify that geometric corrections that happen are
C accurate.  Get someone like Dan Steinwand involved if necessary.
C
C I also added the ifdef dgux to the parblk declaration of getpar subroutine.
C
C
C     D. Etrheim      10/92    Modfied the calcualation of corner coordinates
C                              to use a rotation angle (radrang).  This gives
C                              a better aproximation of corners.
C     D. Etrheim      11/92    Corrected a problem with capture time in the BDDR
C                              Corrected a runtime error on the DG that occurred
C                              in the prtdat subroutine.  There was an error
C                              trying to write the apostrophe in GCP'S.
C     S. Nelson	      12/92    Corrected a problem in GETPAR to allow all three
C			       options under WRTDSK.  Changed unit number
C			       (lplun) from 5 to 25 in PRTDAT to avoid
C				duplicate.   Corrected a buffer array value
C				in ANCDSK2 which reads ancillary file (changed
C				buffer[62] to buffer[63]).
C     D. Etrheim      9/95     Corrected a call to XRSTR which was causing
C				an error when two input tapes were specified on
C				an SG.
C     D. Etrheim      11/96     Negate the UTM zone code before the center
C                               point is converted to projection coords.
C
C     					    
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C             EDIPSIN must be run under TAE.
C
C     PROJECT	                LAS 
C
C     ALGORITHM:
C       The program begins by getting the user-specified parameters and the
C	global variable values.  The input tape is opened, and the tape
C       directory is read to determine the tape characteristics.  Next, the
C       specified associated files are opened.  Then, depending on the type
C       of the input tape, the appropriate processing subroutine is called.
C	If the user requested a printout of information from the header and
C       annotation records, the data is written to a file which is printed
C       and deleted after program completion.  Finally, associated files and
C       the tape are closed, and history is created for the output image.
C
C     ALGORITHM REFERENCES: 
C	Tape format and locations of individual fields were taken from
C	"manual on Characteristics of Landsat Computer-compatible Tapes
C	produced by the EROS Data Center Digital Image Processing System"
C	version 0.1 USGS by Patrick F. Holkenbrink, 1978
C     *****************************************************************
      program edipsin
      
	include 'las.fin'

	include 'tapeio.fin'

      
      integer*4          unit
C					tape unit number
      integer*4	         unithd
C					unit number of header file
      integer*4	         unitan
C   					unit number of annotation file
      integer*4		 unitac
C					unit number of ancillary file
      integer*4	         unittr
C					unit number of trailer file
      integer*4          status
C					status indicator
      integer*4		 tstat
C					tapeio status indicator
      integer*4		 tpopen
C					tape open function declaration
      integer*4		 tpclos
C					tape close function declaration
      integer*4	         msgtim
C					how often to display processing message
      integer*4		 smpinc
C					sample increment
      integer*4		 lininc
C					line increment
      integer*4		 window(5)
C                    			window specification
      integer*4          files(5)
C					user-specified list of bands from 
C					MSS tape to be output
      integer*4       	 nband
C					number of bands that user specified
      integer*4 	 nvol
C  					number of tape volumes in set
      integer*4		 volnum
C 				        current volume number
      integer*4		 mnum
C 		   			Landsat mission number

      integer*4 parblk(XPRDIM)

C					parameter block
      integer*4          stdout
C					standard output
      integer*4		 hstout
C					history function declaration
      integer*4		 clist
C					counter for number of entries
C					in the completion message list
      integer*4		 ddrvflag
C					DDR validity flag
      character*(CMLEN)  msg
C					first mount message sent to operator
      character*(CMLEN)  outary(7)
C					array of files created--used
C					in completion message
      character*(CMLEN)  out
C				        output image TAE name	
      character*(CMLEN)  tlabel(2)
C					tape label(s)
      character*(CMLEN)  temp
C					temporary variable
      character*(255)  hdhos
C					header file host name
      character*(255)  achos
C					ancillary file host name
      character*(255)  anhos    
C					annotation file host name
      character*(255)  trhos
C					trailer file host name
      character*(255)  corhos
C					correction file host name
      character*(255)  modhos
C					modeling file host name
      character*(CMLEN)  outh
C					output image host name
      character*16       minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band
      character*3        stype
C					sensor type (MSS or RBV)
      character*3	 bform
C				        band format of tape 
      character*3	 handsk
C					indicator for dumping header
C					and annotation records to
C					disk files
      character*3	 tradsk
C					indicator for dumping trailer
C					records to a disk file
      character*3	 ancdsk
C					indicator for dumping ancillary
C					records to a disk file
C
      character*1 	 density        
C					tape density
      character*(CMLEN)  tapeid	
C					tape id
      logical            corect
C					indicator for whether the tape
C					was geometrically corrected
      logical		 all
C					indicator for whether the user
C					requested that all bands from
C					the input tape be read in
      logical		 phead
C					indicator for whether the user
C					requested the predefined
C					header/annotation information
C					to be printed out

      integer*4		path
C					path
      integer*4         row
C					row
      real*8		wrsll(2)
C					WRS format center latitude/longitude
      real*8		wrsls(2)
C					WRS format center line/sample
      real*8		angle
C					Image orientaion angle
      character*9	edate
C					Date of exposure
      character*16	scenid
C					scene id
      character*2	projec
C					type of projection
      character*2	sentyp
C					sensor type
      character*2	direction
C					Direction of the pass

      common/outhos/outh
      common/hosts/hdhos, achos, anhos, trhos, corhos, modhos
      common/flags/handsk, ancdsk, tradsk, stype
      common/type/corect
      common/ddrfld/path, row, wrsll, wrsls, angle, edate, projec,
     +              scenid, sentyp, direction

      density = char(0) 
C     initialization
C     --------------
      call inlas(parblk,stdout)

C     get user specified parameters and global variables
C     --------------------------------------------------
      call getpar(parblk,files,nband,all,window,phead,out,
     +            smpinc,lininc,tlabel,minmax,msgtim,ddrvflag)

C     open the input tape
C     -------------------
      temp = tlabel(1)
      msg = temp//char(0)
      tstat = tpopen(unit,0,HANDLE,TNOREW,density,TREAD,msg,tapeid)
      if (tstat .ne. TSUCC) then
        call errmsg('Fatal error encountered','edipsin-fatal',-1)
      endif

C     read the tape directory to determine tape characterisitics
C     ----------------------------------------------------------
      call rddir (unit,nvol,bform,volnum,mnum)

C     open the associated files
C     -------------------------
      call openr(unithd,unitan,unitac,unittr)

C     call proper processing routine
C     ------------------------------
      if (corect) then

        if ((bform.eq.'BSQ').and.(stype.eq.'MSS')) then
           call cormss(files,window,nband,unithd,unitan,
     +	   	       unittr,unit,smpinc,lininc,all,
     +		       tlabel,nvol,volnum,minmax,msgtim)

        elseif ((bform.eq.'BIL').and.(stype.eq.'MSS')) then
 	   call corbil(unit,unithd,unitan,unittr,
     +                 files,nband,all,window,smpinc,lininc,
     +  	       nvol,volnum,tlabel,minmax,msgtim)

        elseif (stype.eq.'RBV') then
   	   call corrbv (unit,unithd,unitan,unittr,window,
     +			smpinc,lininc,minmax,msgtim)
        endif

      else

        if ((bform.eq.'BSQ').and.(stype.eq.'MSS')) then
           call uncmss(files,window,nband,unithd,unitac,
     +		       unitan,unittr,unit,smpinc,lininc,all,tlabel,
     +		       nvol,mnum,volnum,minmax,msgtim)

        elseif ((bform.eq.'BIL').and.(stype.eq.'MSS')) then
           call uncbil(unit,unithd,unitan,unitac,unittr,window,
     +		       smpinc,lininc,nvol,tlabel,files,
     +		       nband,all,volnum,mnum,minmax,msgtim)

        elseif (stype.eq.'RBV') then
           call uncrbv(unit,unithd,unitan,unitac,unittr,window,
     +		       smpinc,lininc,minmax,msgtim)
        endif

      endif
	
C     create the file for printing 
C     ----------------------------
      if (phead) call prtdat
 
C     close the associated files
C     --------------------------
      call closer(unithd,unitac,unitan,unittr)

C     build array for completion message
C     ----------------------------------
      call build(outary,out,clist)

C     close the input tape
C     --------------------
      tstat = tpclos(unit,HANDLE,TRWOFL,1)
      if (tstat .ne. TSUCC) then
        call errmsg('Nonfatal error encountered','edipsin-warn',1)
      endif

C
C  Update the DDR
C
          call updateddr(outh,window,nband,lininc,smpinc,sentyp,
     +                   direction,path,row,wrsll,wrsls,angle,edate,
     +                   projec,scenid,ddrvflag,corect)

C     create the history 
C     ------------------
      status = hstout(parblk,outh,' ',0,'EDIPSIN')
      if (status .ne. ESUCC) then
         call errmsg('Nonfatal error encountered','edipsin-warn',1)
      endif

C     write a completion message 
C     --------------------------
      call complt('EDIPSIN',outary,clist)

      end

C     **************************************************************************
C
C     BUILD builds an array containing the list of output files created.  This
C     array is needed for the completion message.
C     
C     **************************************************************************
      subroutine build(outary,out,clist)
   
	include 'las.fin'

      integer*4		 clist
C					counter for number of entries
C					in the completion message list
      integer*4          status
C				        function return status code	

      character*(CMLEN)  outary(7)
C					array of files created--used
C					in completion message
      character*(CMLEN)  out
C					output image TAE name
      character*(255)  hdhos
C					header file host name
      character*(255)  achos
C					ancillary file host name
      character*(255)  anhos    
C					annotation file host name
      character*(255)  trhos
C					trailer file host name
      character*(255)  corhos
C					correction file host name
      character*(255)  modhos
C					modeling file host name
      character*3        stype
C					sensor type (MSS or RBV)
      character*3	 handsk
C					indicator for dumping header
C					and annotation records to
C					disk files
      character*3	 tradsk
C					indicator for dumping trailer
C					records to a disk file
      character*3	 ancdsk
C					indicator for dumping ancillary
C					records to a disk file
      logical            corect
C					indicator for whether the tape
C					was geometrically corrected
      common/hosts/hdhos, achos, anhos, trhos, corhos, modhos
      common/flags/handsk, ancdsk, tradsk, stype
      common/type/corect

      clist = 1

C     output image
C     ------------
      outary(clist) = out
      clist = clist + 1

C     correction and platform modeling files
C     --------------------------------------
      if ((.not. corect) .and. (stype .eq. 'MSS')) then
	    outary(clist) = corhos
 	    clist = clist + 1

	    outary(clist) = modhos
 	    clist = clist + 1
      endif

C     header and annotation files
C     ---------------------------
      if (handsk .eq. 'YES') then
	    outary(clist) = hdhos
	    clist = clist + 1
      
	    outary(clist) = anhos
	    clist = clist + 1
      endif

C     trailer file
C     ------------
      if (tradsk .eq. 'YES') then
	    outary(clist) = trhos
	    clist = clist + 1
      endif

C     ancillary file
C     --------------
      if ((.not. corect) .and. (ancdsk .eq. 'YES')) then
	    outary(clist) = achos
            clist = clist + 1
      endif
    
      clist = clist - 1

      return
      end

C     **************************************************************************
C
C     GETPAR receives the user-specified parameters and TAE global variables.
C
C     **************************************************************************
      subroutine getpar (parblk,files,nband,all,window,phead,out,
     +			 smpinc,lininc,tlabel,minmax,msgtim,ddrvflag)

	include 'las.fin'

      
      integer*4		getout
C					getout function declaration
      integer*4		status
C					status indicator
      integer*4		length
C					length of string received
      integer*4		lngth(3)
C					length of string received
      integer*4		leng2(2)
C					length of string received
      integer*4		count
C					number of values received
      integer*4		msgtim
C					elapsed time between display messages
      integer*4		smpinc
C					sample increment
      integer*4		lininc
C					line increment
      integer*4		window(5)
C                    			window specification
      integer*4		files(5)
C					user-specified list of bands from MSS
C					input tape to be output
      integer*4		nband
C					number of bands that user specified
      integer*4		ddrvflag
C					ddr validity flags

      integer*4 parblk(XPRDIM)

C					parameter block
      integer*4		temp
C					temporary variable
      integer*4	      	pairs
C					loop counter
      integer*4		i
C					loop index
      integer*4		j
C					loop index
      character*(CMLEN)	out
C				        output image TAE name	
      character*(CMLEN)	subnam
C           				current subcommand specified
      character*(CMLEN)	outh
C					output image host name
      character*(CMLEN)	tlabel(2)
C					tape label(s)
      character*16	minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band

      character*3	ancdsk
C					indicator for dumping ancillary
C					records to disk file
      character*3	handsk
C					indicator for dumping header
C					and annotation records to
C					disk files
      character*3	prthd
C       			        print indicator
      character*3	stype
C					sensor type (MSS or RBV)
      character*3	tradsk
C					indicator for dumping trailer
C					records to a disk file
      character*3	wlist(3)
C					write to disk option list
      character*3	ddrvflg
C					DDR validity flag

      logical		all
C					indicator for whether the user
C					requested that all bands from
C					the input tape be read in
      logical		phead
C					indicator for whether the user
C					requested the predefined
C					header/annotation information
C					to be printed out
      logical		sflag
C					sort flag

      common/outhos/outh
      common/flags/handsk, ancdsk, tradsk, stype

C     get TAE global variables 
C     ------------------------
      call xrstr(parblk,'$MINMAX',1,minmax,length,count,status)
      call xrintg(parblk,'$MSGTIM',1,msgtim,count,status)
      call xrstr(parblk,'$DDRVFLG',1,ddrvflg,length,count,status)
      if (ddrvflg .eq. 'IN') then
          ddrvflag = INVAL
      else
          ddrvflag = UNKNOW
      endif
   
C     get description of tape (COMMENT)
C     ---------------------------------
      call xrstr(parblk,'COMMENT',2,tlabel,leng2,count,status) 

C     get output image name (OUT)
C     ---------------------------
      status = getout(parblk,'OUT',out,outh)
      if (status .ne. ESUCC) then
         call errmsg ('Fatal error encountered','edipsin-fatal',-1)
      endif

C     get input window specification (WINDOW)
C     ---------------------------------------
      call xrintg(parblk,'WINDOW',4,window,count,status)
      window(5) = count 

C     get line increment (LINEINC)
C     ----------------------------
      call xrintg(parblk,'LINEINC',1,lininc,count,status)

C     get sample increment (SAMPINC)
C     ------------------------------
      call xrintg(parblk,'SAMPINC',1,smpinc,count,status)

C     get write to disk option (WRTDSK)
C     ---------------------------------
      call xrstr(parblk,'WRTDSK',3,wlist,lngth,count,status)
      if (count .eq. 0) then
	 handsk = 'NO'
	 ancdsk = 'NO'
	 tradsk = 'NO'
      else
   	 do 100 i = 1,3
	    if (wlist(i) (1:1) .eq. 'H')   handsk = 'YES'
	    if (wlist(i) (1:1) .eq. 'A')   ancdsk = 'YES'
	    if (wlist(i) (1:1) .eq. 'T')   tradsk = 'YES'
100	 continue
      endif

C     get print option (PRINTFLG)
C     ---------------------------
      call xrstr(parblk,'PRINTFLG',1,prthd,length,count,status)
      phead = prthd .eq. 'YES'
      
C     determine subcommand being executed
C     -----------------------------------
      call xrstr(parblk,'_SUBCMD',1,subnam,length,count,status)

C     if subcommand is MSS, get band numbers (BANDS)
C     ----------------------------------------------
      if (subnam .eq. 'MSS') then
	 stype = 'MSS'
	 call xrintg(parblk,'BANDS',5,files,nband,status)
      else
	 stype = 'RBV'
	 nband = 1
      endif

C     if file list was allowed to default then process all bands
C     ----------------------------------------------------------
      if (nband .eq. 0) then
        do 300 i = 1,5
          files(i) = i 
300     continue
        nband = 5
        all = .true.
      endif
 
C     sort the file list into ascending order
C     ---------------------------------------
      do 500 pairs = nband-1, 1, -1
         sflag = .false.
         do 400 j = 1, pairs
            if (files(j) .gt. files(j+1)) then
   		temp = files(j)
		files(j) = files(j+1)
		files(j+1) = temp
		sflag = .true.
	    endif
400      continue

         if (.not. sflag) goto 600
500   continue

600   return
      end

C     **************************************************************************
C
C     RDDIR reads the tape directory to determine tape characteristics.
C
C     **************************************************************************
      subroutine rddir (unit,nvol,bform,volnum,mnum)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		size
C					buffer size
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpclos
C					tape close function declaration
      integer*4		tpread
C					tape read function declaration
      integer*4		nvol
C  					number of tape volumes in set
      integer*4		volnum
C 				        current volume number
      integer*4		mnum
C 		   			Landsat mission number
      character*3       bform
C				        band format of tape
      character*3       temp

      character*1	numtap
C					number of volumes in tape set
      character*1	intyp
C					interleaving type indicator
      character*1	npatrw(9)
C					nominal path row
      character*1	clatlo(17)
C					center latitude and longitude
      character*1	corapp
C					corrections applied
      character*1       nlatlo(17)
C					nominal latitude and longitude
      character*1	bpres(5)
C					spectral bands present
      character*1	imgsca
C					image scale
      character*1	resamp
C					resampling algorithm
      character*1	msstra
C					MSS transmission type
      character*1 	suninf(14)
C					sun information
      character*1	edipce
C					contrast enhancement indicator
      character*1	edipas
C					atmospheric scatter compensation
C					indicator
      character*1       edipeh
C					edge enhancement indicator
      character*1	bndgai(5)
C					sensor gain options
      character*1	tdate(3)
C					tape generation date
      character*1       buffer(360)
C					input buffer

      integer*4		path
C					path
      integer*4         row
C					row
      integer*4         mission
C					Landsat mission number
      real*8		wrsll(2)
C					WRS format center latitude/longitude
      real*8		wrsls(2)
C					WRS format center line/sample
      real*8		angle
C					Image orientaion angle
      character*9	edate
C					Date of exposure
      character*16	scenid
C					scene id
      character*2	projec
C					type of projection
      character*2	sentyp
C					sensor type
      character*2	direction
C					Direction of the pass

      logical   corect
C					indicator for whether the tape
C					was geometrically corrected

      common/ddrfld/path, row, wrsll, wrsls, angle, edate, projec,
     +              scenid, sentyp, direction
      common/type/corect
      common/prtfld/numtap, intyp, npatrw, clatlo, corapp, nlatlo,
     +		    bpres, imgsca, resamp, msstra, suninf, mission,
     +		    edipce, edipas, edipeh, bndgai, tdate

C     read the tape directory
C     -----------------------
      size = 360 
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. 360) then
	 tstat = tpclos(unit,HANDLE,TRWOFL,1)
        call errmsg('Fatal error encountered','edipsin-fatal',-1)
      endif 
      
C     get sentyp, numtap, intyp and direction of pass
C     -----------------------------------------------
      numtap = buffer(20)
      intyp  = buffer(31)

      sentyp(1:1) = buffer(9)
      sentyp(2:2) = char(0)
      direction(1:1) = buffer(46)
      direction(2:2) = char(0)

C     get mission number
C     ------------------

      read(buffer(8),'(I1)') mnum

C     get the current volume number
C     -----------------------------

      read(buffer(19),'(I1)') volnum

       

C     get number of volumes in the tape set
C     -------------------------------------

      read(buffer(20),'(I1)') nvol

C     determine if band sequential or band interleaved by line
C     --------------------------------------------------------

      if (ichar(buffer(31)) .eq. 0) then
        bform = 'BSQ'
      else
        bform = 'BIL'
      endif

C     determine if geometrically corrected or uncorrected
C     ---------------------------------------------------

      corect = ichar(buffer(11)) .eq. 80

C     get WRS frame number (path)
C     ---------------------------
      call pxcopy(buffer(47),temp,1,3)
      read (temp,'(I3)') path

C     get WRS orbit number (row)
C     ---------------------------
      call pxcopy(buffer(50),temp,1,3)
      read (temp,'(I3)') row

      
C     read the end of file mark after the tape directory
C     --------------------------------------------------
      size = 5388
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. TEEOF) then 
	 tstat = tpclos(unit,HANDLE,TRWOFL,1)
        call errmsg('Fatal error encountered','edipsin-fatal',-1)
      endif

      return 
      end

C     **************************************************************************
C
C     OPENR opens the header, ancillary, annotation and trailer files
C     if the user requested these records dumped to disk files.
C
C     **************************************************************************
      subroutine openr(unithd,unitan,unitac,unittr)
 
	include 'las.fin'

      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unitac
C					unit number of ancillary file
      integer*4		unittr
C					unit number of trailer file
      integer*4		status
C					status indicator
      integer*4		len
C					record length
      character*(CMLEN)	outh
C				        output image host name	
      character*(255) hdhos
C					header file host name
      character*(255) achos
C					ancillary file host name
      character*(255) anhos
C					annotation file host name
      character*(255)	trhos
C					trailer file host name
      character*(255) corhos 
C					correction file host name
      character*(255) modhos
C					modeling file host name
      character*3       stype
C					sensor type (MSS or RBV)
      character*3       handsk
C					indicator for dumping header
C					and annotation records to
C					disk files
      character*3	tradsk
C					indicator for dumping trailer
C					records to a disk file
      character*3	ancdsk
C					indicator for dumping ancillary
C					records to disk file
      logical           corect
C					indicator for whether the tape
C					was geometrically corrected

      common/outhos/outh
      common/hosts/hdhos, achos, anhos, trhos, corhos, modhos
      common/flags/handsk, ancdsk, tradsk, stype
      common/type/corect

C     Set unit numbers
C     ----------------
      unithd = 1
      unitan = 2
      unitac = 3
      unittr = 4

C     Set record length
C     -----------------
C     VMS wants longwords for record length.
C     1 uses bytes for record length.

      if (stype .eq. 'MSS') then
        len = 3596
      else
        len = 5388
      endif

      if (handsk .eq. 'YES') then
C	 create host name for associated header file
C        -------------------------------------------
         call lsmknm(outh,'edips_headr',hdhos)

C        open the header disk file
C        -------------------------
	 open (unit=unithd,file=hdhos,status='new',access='sequential',
     +         form = 'unformatted',iostat=status) 
         if (status .ne. 0) then
	    call errmsg 
     +          ('Error opening header file','edipsin-openerr',1)
	 endif
        
C        create host name for associated annotation file 
C        -----------------------------------------------
	 call lsmknm(outh,'edips_annot',anhos)
        
C        open the annotation disk file
C        -----------------------------
	 open (unit=unitan,file=anhos,status='new',access='sequential',
     +         form = 'unformatted',iostat=status)
         if (status .ne. 0) then
            call errmsg 
     +         ('Error opening annotation file','edipsin-openerr',1)
	 endif
      
      endif  

      if (tradsk .eq. 'YES') then
C        create host name for associated trailer file 
C        --------------------------------------------
	 call lsmknm(outh,'edips_trail',trhos)
        
C        open the trailer disk file
C        --------------------------
	 open (unit=unittr,file=trhos,status='new',access='direct',
     +         form='unformatted', recl=len,iostat=status)
         if (status .ne. 0) then
	    call errmsg 
     +         ('Error opening trailer file','edipsin-openerr',1)
	 endif
        
      endif

      if ((.not. corect) .and. (ancdsk .eq. 'YES')) then

C        create host name for associated ancillary file
C        ----------------------------------------------
         call lsmknm(outh,'edips_ancil',achos)
        
C        open the ancillary disk file
C        ----------------------------
	 open (unit=unitac,file=achos,status='new',access='sequential',
     +         form='unformatted', iostat=status)
         if (status .ne. 0) then
	    call errmsg 
     +        ('Error opening ancillary file','edipsin-openerr',1)
	 endif
        
      endif

      return
      end

C     **************************************************************************
C
C     OPENS opens the image file for sequential output.
C
C     **************************************************************************
      subroutine opens(tnl,tnp,nband,unit,gdesc,offset)
      
	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		estat
C					imageio status indicator
      integer*4		eopens
C					imageio function declaration
      integer*4		egroup
C					imageio function declaration
      integer*4		gdesc
C 					group descriptor
      integer*4		fdesc(2)
C					file descriptor array
      integer*4		offset(5)
C					array of offsets into "thebuf"
      integer*4		nband
C					number of bands that user specified
      integer*4		tnl
C					total number of output lines
      integer*4		tnp
C					total number of output pixels

      character*(CMLEN) outh
C					output image host name

      character*1      thebuf(BSIZE)
C					output buffer

      common/outhos/outh
      common/thebuf/thebuf
      common/imgio/fdesc

C     open the output image for sequential access
C     -------------------------------------------
      estat = eopens(fdesc(1),outh,nband,1,1,tnl,tnp,offset,
     +	  IWRITE,EBYTE,0,0)
      if (estat .ne. ESUCC) call fclean(unit)

C     form the group for sequential processing
C     ----------------------------------------
      fdesc(2) = 0
      estat = egroup(fdesc,gdesc,thebuf,BSIZE) 
      if (estat .ne. ESUCC) call fclean(unit)
      
      return
      end
     

C     **************************************************************************
C
C     CORMSS processes geometrically corrected MSS data in band 
C     sequential (BSQ) format.
C
C     **************************************************************************
      subroutine cormss(files,window,nband,unithd,unitan,
     +		        unittr,unit,smpinc,lininc,all,tlabel,nvol,
     +			volnum,minmax,msgtim)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unittr
C					unit number of trailer file
      integer*4		status
C					status inidicator
      integer*4		size
C					buffer size
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpopen
C					tape open function declaration
      integer*4		tpclos
C					tape close function declaration
      integer*4		tpread
C					tape read function declaration
      integer*4		tpfsf
C					tapeio function declaration
      integer*4		tpfsr
C					tapeio function declaration
      integer*4		lskip
C					lines to skip to get to starting line
      integer*4		lafter
C					lines remaining after skipping
      integer*4		mstat
C					mount status from nxtvol call
      integer*4		estat
C					imageio status indicator
      integer*4		eopenr
C					imageio function declaration
      integer*4	   	ewrite
C					imageio function declaration
      integer*4		eclose
C		  			imageio function declaration
      integer*4		rfdesc
C					file descriptor
      integer*4		msgtim
C					elapsed time between display messages
      integer*4		smpinc
C					sample increment
      integer*4		lininc
C					line increment
      integer*4		window(5)
C                    			window specification
      integer*4		files(5)
C					user-specified list of bands
C					from MSS input tape to be output
      integer*4  	trlist(5)
C					actual list of bands that
C					are output
      integer*4		file
C					current band being processed
      integer*4		nxt
C					next band in list
      integer*4		nband
C					number of bands that user specified
      integer*4		btot 
C					actual number of bands output
      integer*4	 	outbnd
C					current output band number--for
C					display messsage
      integer*4		numimg
C					total number of bands on tape
      integer*4         inumb
C				        band number -- needed by "hddat"
      integer*4		nvol
C  					number of tape volumes in set
      integer*4   	volnum
C 				        current volume number
      integer*4    	mnum
C 		   			Landsat mission number
      integer*4		rtype
C					record type
      integer*4		tnp
C					total number of output pixels
      integer*4		tnl
C				        total number of output lines
      integer*4		nol
C					number of lines in input image
      integer*4		nop
C					number of pixels in input image
      integer*4		nl
C					nl from window specification
      integer*4		np
C					np from window specification
      integer*4      	outlin
C					output line counter
      integer*4       	ibegin, ito, ifrom
C					array indices
      integer*4		i, j, jpix, ipix
C					loop counters/indices

      character*305     newmsg
C					new mount message
      character*(CMLEN) tlabel(2)
C		  			tape label(s)
      character*(CMLEN)	outh
C					output image host name
      character*35      msg
C					warning message
      character*16      minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band
      character*45      str1
      character*45	str2
      character*45      str3
C					switched volumes message for operator
      character*(CMLEN) tapeid         
C					 tape id
      character*1       density         
C					tape desity
      character*3	stype
C					sensor type (MSS or RBV)
      character*3	handsk
C					indicator for dumping header
C					and annotation records to
C					disk files
      character*3	ancdsk
C					indicator for dumping ancillary
C					records to a disk file
      character*3	tradsk
C					indicator for dumping trailer
C					records to a disk file
      character*3	bform 
C					band format of tape
      real*8		omin(5)
C					array of output min values
      real*8		omax(5)
C					array of output max values

      character*1       headr(3596)
C				        header buffer
      character*1       buffer(3596)
C					input buffer
      character*1	bufout(3596)
C					output buffer
      character*1	gain(5)
C					sensor gain options
      character*1	trans(5)
C					MSS transmission type

      logical           flag
C					flag for whether current band
C					is to be processed
      logical		corect
C					indicator for whether the tape
C					was geometrically corrected
      logical		all
C					indicator for whether the user
C					requested that all bands from
C					the input tape be read in
      common/outhos/outh
      common/curent/outlin
      common/curcat/outbnd
      common/flags/handsk, ancdsk, tradsk,stype
      common/type/corect

      equivalence (headr(3587),gain(1))
      equivalence (headr(3592),trans(1))

      data nop/3548/
      data nol/2983/
      
C     initialize the array of min and max values
C     ------------------------------------------
      call inmnmx(EBYTE,5,omin,omax)

C     validate the window
C     -------------------
50    call adjwin(window,nop,nol,np,nl,unit)

C     determine the number of lines and pixels in the output image
C     ------------------------------------------------------------
      tnl = (nl-1)/lininc+1
      tnp = (np-1)/smpinc+1
      
      file = 1
      nxt = 1
      outbnd = 0

      do 60 i = 1,5
	 trlist(i) = 0
60    continue

C     read the header record
C     ----------------------
      size = 3596
      tstat = tpread(unit,headr,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the needed header information
C     ---------------------------------
      call hddat(headr,.true.,.false.,inumb)

C     check that all the specified bands are present on the tape
C     ----------------------------------------------------------
      btot = 0

      numimg = ichar(headr(135))

      if (all) then
         btot = numimg
         do 70 i = 1,btot
	    trlist(i) = i
70	 continue
      else
         do 75 i = 1,nband
  	    if (files(i) .le. numimg) then
               btot = btot + 1
  	       trlist(btot) = files(i)
            else
               msg = 'There is no band   on the tape'
               write (msg(18:18),'(I1)') files(i)
               call errmsg(msg,'edipsin-fewbands',1)
            endif
75        continue
      endif

C     Reset the number of bands to the actual number of bands being processed
      nband = btot
      
C     check that the proper volume is mounted first
C     ---------------------------------------------
      if (nvol .eq. 2) then
        str1='Second volume of the set was mounted first.'
        str2='Mount the other volume of the set instead.'
        str3='The two volumes in the set are: '
        newmsg=str1//char(13)//char(10)//str2//char(13)//
     +       char(10)//str3//char(13)//char(10)//tlabel(1)//
     +       char(13)//char(10)//tlabel(2)//char(0)
      endif

      if (numimg .eq. 4) then  
        if ((volnum.gt.1).and.(files(1).lt.3)) then
          call errmsg('Volumes were switched', 'edipsin-volflag',1)
          call errmsg
     +      ('Second volume was mounted first','edipsin-volflag',1)
          call errmsg
     +      ('Operator was told to mount first volume',
     +       'edipsin-volflag',1)

          tstat = tpclos(unit,HANDLE,TRWOFL,1)
	  if (tstat .ne. TSUCC) then
  	     call errmsg('Nonfatal error encountered',
     +			 'edipsin-warn',1)
          endif

          density =  char(0)
          tstat = tpopen(unit,0,HANDLE,TNOREW,density,TREAD,newmsg,
     + tapeid)
          if (tstat .ne. TSUCC) call fclean(unit)

          call rddir(unit, nvol, bform, volnum, mnum)
          goto 50
        endif
      else if (numimg .eq. 5) then
        if ((volnum.gt.1).and.(files(1).lt.4)) then
          call errmsg('Volumes were switched','edipsin-volflag',1)
          call errmsg
     +       ('Second volume was mounted first','edipsin-volflag',1)
          call errmsg
     +       ('Operator was told to mount first volume',
     +        'edipsin-volflag',1)

           tstat = tpclos(unit,HANDLE,TRWOFL,1)
           if (tstat .ne. TSUCC) then
	      call errmsg('Nonfatal error encountered',
     +                    'edipsin-warn',1)
  	   endif
           density = char(0)
           tstat = tpopen(unit,0,HANDLE,TNOREW,density,TREAD,newmsg,
     + tapeid)
           if (tstat .ne. TSUCC) call fclean(unit)

           call rddir(unit, nvol, bform, volnum, mnum)
           goto 50
        endif
      endif
 
C     read the annotation record
C     --------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the needed annotation information
C     -------------------------------------
      call anndat(buffer)

      if (files(1) .eq. 1) then
        gain(file) = buffer(91)
        trans(file) = buffer(92)
      endif

C     dump the annotation record out to the related file
C     --------------------------------------------------
      if (handsk .eq. 'YES') then
       write  (unit=unitan,iostat=status) buffer
       if (status .ne. 0) then
	  call errmsg('Error writing to annotation file',
     +	              'edispin-writerr',1)
       endif
      endif
    
C     read the end of file mark
C     -------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. TEEOF) call fclean(unit)

C     initialize the display messages
C     -------------------------------
      call inittm('EDIPSIN','Processed','line','band',' ',
     + 	          tnl,btot,msgtim,ONECAT)

C     read the first image line
C     -------------------------
      size = 3596 
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     open the output image 
C     ---------------------
      estat = eopenr(rfdesc,outh,btot,1,1,tnl,tnp,IWRITE,EBYTE,0,1)
      if (estat .ne. ESUCC) call fclean(unit)

100   continue
C       determine if the current band is to be processed
C       ------------------------------------------------
      if (files(nxt) .eq. file) then
          flag = .true.
      else
          flag = .false.
         call errmsg('Skipping over input band','informational',1)
      endif

125     continue
C         read the next image record
C         --------------------------

          rtype = ichar(buffer(6))

	  if ((flag) .and.(rtype .eq. 219)) then
            gain(file) = buffer(91)
            trans(file) = buffer(92)
          endif
          
	  if ( rtype.eq.18 ) call hddat( buffer,.true.,.true.,inumb)

          if ((rtype .eq. 237).and.(tstat .ne. TEEOF)) goto 150
          
	  size = 3596
          tstat = tpread(unit,buffer,size,HANDLE)
          if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call fclean(unit)
          endif

	  goto 125

C       process the image data
C       ----------------------
150     continue
        if (flag) outbnd = outbnd + 1
        ibegin = window(2) + 12 

C       if necessary, skip to the starting line
C       ---------------------------------------
        if (window(1) .gt. 1) then
	   lskip = window(1) - 2
           if (flag) 
     +     call errmsg('Skipping to starting line','informational',1)
	   tstat = tpfsr(unit,lskip,HANDLE)
	   if (tstat .ne. TSUCC) call fclean(unit)

           tstat = tpread(unit,buffer,size,HANDLE)
 	   if (tstat .ne. size) call fclean(unit)
        endif

C       ------------- start the display messages -------------------
C       (This assignment had to made just so that the processing 
C       message didn't start with "Processed line 0" -- standards!!)
C       ------------------------------------------------------------
        outlin = 1
        if (flag)  call startm
        outlin = 0 
	
        do 300 j = 1,nl
          if (flag) then
            if ( mod(j-1,lininc) .eq. 0 )then
               outlin = outlin + 1

               if ( smpinc.gt.1 )then
                  jpix= 0
                  do 200 ipix=1,np,smpinc
                     jpix = jpix + 1 
                     ifrom = ibegin + ipix - 1
		     ito = jpix
		     bufout(ito) = buffer(ifrom)
200               continue
               else
 		  call pxcopy(buffer(ibegin),bufout,1,np)
	       endif

               if (minmax .eq. 'YES')
     +           call pxmnmx(bufout,tnp,omin(nxt),omax(nxt),EBYTE)

	       estat = ewrite(rfdesc,nxt,outlin,bufout,1)
	       if (estat .lt. 0) then
		  call stoptm
		  call fclean(unit)
               endif
            
            endif
          endif
          
C         read the next image line
C         ------------------------
	  tstat = tpread(unit,buffer,size,HANDLE)
          if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call stoptm
	     call fclean(unit)
          endif  

300     continue

C       stop the display messages
C	-------------------------
        if (flag) call stoptm

        if (flag) nxt = nxt + 1
        file = file + 1
    
C       if necessary, skip to the trailer record
C       ----------------------------------------
        lafter = (nol - window(1)) + 1
	if (lafter .ne. nl) then
	   tstat = tpfsf(unit,1,HANDLE)
	   if (tstat .ne. TSUCC) call fclean(unit)
	endif
	
C       read trailer record
C       -------------------
	size = 3596	
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

C       dump trailer record information out to the related file
C       -------------------------------------------------------
	if (flag) then
         if (tradsk .eq. 'YES') then
          write (unit=unittr,rec=nxt-1,iostat=status) buffer
          if (status .ne. 0) then
             call errmsg('Error writing to trailer file',    
     +	                 'edipsin-writerr',1) 
	  endif
         endif
        endif

C       return if processed all requested bands
C       ---------------------------------------
        if (nxt .gt. btot) then
           if (handsk .eq. 'YES') then
              write (unit=unithd,iostat=status) headr
              if (status .ne. 0) then
               call errmsg('Error writing to header file',
     +	                     'edipsin-writerr',1) 
	      endif
           endif
	 goto 600
        endif
        
C       check if need to mount the next volume
C       otherwise read the end of file mark
C       --------------------------------------
        if ((nvol.eq.2).and.(numimg.eq.4).and.(file.eq.3)) then
            call nxtvol(unit,tlabel,mstat)
            if (mstat .ne. 0) call fclean(unit) 
        elseif ((nvol.eq.2).and.(numimg.eq.5).and.(file.eq.4)) then
            call nxtvol(unit,tlabel,mstat)
            if (mstat .ne. 0) call fclean(unit)
        else
 	    tstat = tpread(unit,buffer,size,HANDLE)
            if (tstat .eq. TEOV) then
               if (handsk .eq. 'YES') then
                 write (unit=unithd,iostat=status) headr
                 if (status .ne. 0) then
		  call errmsg('Error writing to header file', 
     +                          'edipsin-writerr', 1)
                 endif
               endif
  
               if (.not. all) then
		  call errmsg
     +               ('Processed fewer bands than specified',
     +                'edipsin-fewbands',1)
               endif
               goto 600
            endif
        endif	  
	  
C       read the next record
C       --------------------
        size = 3596
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

	goto 100

600   continue

C     close the output image
C     ----------------------
      estat = eclose(rfdesc)
C
C  If no min/max value has been calculated use bounded because output
C  image is byte.
C
	if (minmax .eq. 'NO') then
           minmax = 'BOUNDED'
	do 101 i = 1, btot
	    omax(i) = 255.0
	    omin(i) = 0.0
 101    continue
	endif

C     save the min and max values
C     ---------------------------
      call svmnmx(outh,btot,omin,omax,minmax)

      return
      end

C     **************************************************************************
C
C     CORRBV processes geometrically corrected RBV data.
C
C     **************************************************************************
      subroutine corrbv (unit,unithd,unitan,unittr,window,
     +			 smpinc,lininc,minmax,msgtim)
      
	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unittr
C					unit number of trailer file
      integer*4         status 
C					status inidicator
      integer*4		size
C					buffer size
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpread
C					tapeio function declaration
      integer*4		tpfsr
C					tapeio function declaration
      integer*4		tpfsf
C					tapeio function declaration
      integer*4		estat
C					imageio status indicator
      integer*4		estep
C					imageio function declaration
      integer*4		egclse
C		  			imageio function declaration
      integer*4		gdesc
C					group descriptor
      integer*4         fdesc(2)
C					file descriptor array
      integer*4		offset(5)
C					array of offsets into "thebuf"
      integer*4		msgtim
C   					elapsed time between display messages
      integer*4		smpinc
C					sample increment
      integer*4		lininc
C					line increment
      integer*4		window(5)
C                    			window specification
      integer*4         inumb
C				        band number -- needed by "hddat"
      integer*4		lskip
C					lines to skip to get to starting line
      integer*4		lafter
C					lines remaining after skipping
      integer*4		tnp
C					total number of output pixels
      integer*4		tnl
C				        total number of output lines
      integer*4		nol
C					number of lines in input image
      integer*4       	nop
C					number of pixels in input image
      integer*4		nl
C					nl from window specification
      integer*4		np
C					np from window specification
      integer*4		outlin
C					output line counter
      integer*4		ibegin, ito, ifrom
C					array indices
      integer*4		j, jpix, ipix
C					loop counters/indices

      character*(CMLEN) outh
C					output image host name
      character*16      minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band
      character*3       handsk
C					indicator for dumping header
C					and annotation records to
C					disk files
      character*3	ancdsk
C					indicator for dumping ancillary
C					records to a disk file
      character*3	tradsk
C					indicator for dumping trailer
C					records to a disk file
      character*3	stype
C					sensor type (MSS or RBV)
      real*8		omin(2)
C					output min value
      real*8		omax(2)
C					output max value

      character*1       buffer(5388)
C					input buffer
      character*1	thebuf(BSIZE)
C					output buffer

      common/thebuf/thebuf
      common/imgio/fdesc
      common/curent/outlin
      common/outhos/outh
      common/flags/handsk, ancdsk, tradsk, stype
      
      data nop/5322/
      data nol/5322/

C     initialize min and max 
C     ----------------------
      call inmnmx(EBYTE,1,omin,omax)

C     validate the window
C     -------------------
      call adjwin(window,nop,nol,np,nl,unit)
      
C     determine the number of lines and pixels in the output image
C     ------------------------------------------------------------
      tnl = ( nl-1 ) / lininc  + 1
      tnp = ( np-1 ) / smpinc + 1

C     read the header record
C     ----------------------
      size = 5388
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the needed header information
C     ---------------------------------
      call hddat(buffer,.false.,.false.,inumb)

C     dump the header information to the related file
C     -----------------------------------------------
      if (handsk .eq. 'YES') then
         write (unit=unithd,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg
     +     ('Error writing to header file','edipsin-writerr',1)
	 endif
      endif

C     read the annotation record
C     --------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)
      
C     get the needed annotation information
C     -------------------------------------
      call anndat(buffer)
      
C     dump the annotation information to the related file 
C     ---------------------------------------------------
      if (handsk .eq. 'YES') then
         write (unit=unitan,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg  ('Error writing to annotation file',
     +                    'edipsin-writerr',1)
	 endif
      endif

C     read the end of file mark
C     -------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. TEEOF) call fclean(unit)

C     if necessary, skip to the starting line
C     ---------------------------------------
      if (window(1) .gt. 1) then
       call errmsg('Skipping to starting line','informational',1)
         lskip = window(1) - 1
         tstat = tpfsr(unit,lskip,HANDLE)
         if (tstat .ne. TSUCC) call fclean(unit)
      endif

C     read the starting line
C     ----------------------
      size = 5388
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     open the image file
C     -------------------
      call opens(tnl,tnp,1,unit,gdesc,offset)
 
C     initialize the display messages
C     -------------------------------
      call inittm('EDIPSIN','Processed','line','band',' ',
     + 	    	  tnl,1,msgtim,ALLCAT)

C     ------------- start the display messages -------------------
C     (This assignment had to made just so that the processing 
C     message didn't start with "Processed line 0" -- standards!!)
C     ------------------------------------------------------------
      outlin = 1
      call startm
      outlin = 0

C     process the image data
C     ----------------------
      ibegin = window(2) + 12
      
      do 300 j = 1,nl
          if ( mod(j-1,lininc) .eq. 0 ) then
            outlin = outlin + 1
	    estat = estep(gdesc)
            if (estat .ne. ESUCC) then
	       call stoptm
	       call fclean(unit)
            endif
	     
	    if (smpinc .gt. 1) then
                jpix = 0
                do 200 ipix = 1,np,smpinc
                   jpix = jpix + 1
                   ifrom = ibegin + ipix - 1 
                   ito = offset(1) + jpix 
 		   thebuf(ito) = buffer(ifrom)
200		continue
             else
                call pxcopy(buffer(ibegin),thebuf(offset(1)+1),1,np)
             endif
          
             if (minmax.eq.'YES')
     +         call pxmnmx(thebuf(offset(1)+1),tnp,omin,omax,EBYTE)
        
	  endif

	  tstat = tpread(unit,buffer,size,HANDLE)
          if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call stoptm
	     call fclean(unit)
          endif

300   continue

C     disable the display messages
C     ----------------------------
      call stoptm
   
C     flush the buffer
C     ----------------
      estat = estep(gdesc)
      if (estat .ne. ESUCC) call fclean(unit)

C     if necessary, process the trailer record
C     ----------------------------------------
      if (tradsk.eq.'YES') then

C        if necessary, skip forward to the trailer record
C        ------------------------------------------------
         lafter = (nol - window(1)) + 1
         if (lafter .ne. nl) then
            tstat = tpfsf(unit,1,HANDLE)
            if (tstat .ne. TSUCC) call fclean(unit)
         endif

C        read the trailer record
C        -----------------------
         size = 5388
         tstat = tpread(unit,buffer,size,HANDLE)
         if (tstat .ne. size) call fclean(unit)

C        dump the trailer record information to the related file
C        -------------------------------------------------------
         write (unit=unittr,rec=1,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg 
     +      ('Error writing to trailer file','edipsin-writerr',1)
	 endif

      endif
C
C  If no min/max value has been calculated use bounded because output
C  image is byte.
C
	if (minmax .eq. 'NO') then
            minmax = 'BOUNDED'
	    omax(1) = 255.0
	    omin(1) = 0.0
	endif

C     save the min and max
C     --------------------
      call svmnmx(outh,1,omin,omax,minmax)

C     close the output image
C     ----------------------
      estat = egclse(gdesc)

      return 
      end

C     **************************************************************************
C
C     CORBIL processes geometrically corrected MSS data in band
C     interleaved by line (BIL) format.
C
C     **************************************************************************
      subroutine corbil (unit,unithd,unitan,unittr,files,nband,
     +			 all,window,smpinc,lininc,nvol,volnum,
     +		 	 tlabel,minmax,msgtim)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unittr
C					unit number of trailer file
      integer*4		status
C					status inidicator
      integer*4		mstat
C					mount status from nxtvol call
      integer*4		size
C					buffer size
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpopen
C					tapeio function declaration
      integer*4		tpclos
C					tapeio function declaration
      integer*4		tpread
C					tapeio function declaration
      integer*4		tpfsr
C					tapeio function declaration
      integer*4		tpfsf
C					tapeio function declaration
      integer*4		lskip
C					lines to skip to get to starting line
      integer*4		lafter
C					lines remaining after skipping
      integer*4		outlin
C					output line counter--for display msg.
      integer*4   	estat
C					imageio status indicator
      integer*4		estep
C					imageio function declaration
      integer*4		egclse
C		  			imageio function declaration
      integer*4		gdesc
C					group descriptor
      integer*4	   	fdesc(2)
C					file descriptor array
      integer*4		offset(5)
C					array of offsets into "thebuf"
      integer*4		msgtim
C					how often to display processing message
      integer*4		smpinc
C					sample increment
      integer*4		lininc
C					line increment
      integer*4		window(5)
C                    			window specification
      integer*4		files(5)
C					user-specified list of bands
C					from MSS input tape to be output
      integer*4		trlist(5)
C					actual list of bands
C					that are output
      integer*4         inumb
C				        band number -- needed by "hddat"
      integer*4		nxt
C					next band in list
      integer*4		nband
C   				        number of bands that user specified	
      integer*4		btot
C					actual number of bands output
      integer*4		nvol
C  					number of tape volumes in set
      integer*4		volnum
C 				        current volume number
      integer*4		mnum
C 		   			Landsat mission number
      integer*4  	tnp
C					total number of output pixels
      integer*4		tnl
C				        total number of output lines
      integer*4		nol
C					number of lines in input image
      integer*4		nop
C					number of pixels in input image
      integer*4		nl
C					nl from window specification
      integer*4		np
C					np from window specification
      integer*4		line
C					line counter
      integer*4		ibegin, ito, ifrom
C					array indices
      integer*4		i, j, jpix, ipix
C					loop counters/indices

      integer*2		mask(5)
C					mask to determine the image bands
C					that are present
      integer*2		word
C					indication of data present by band
      integer*2		val
C					return value from pxbtan function

      character*305     newmsg
C					new mount message

      character*(CMLEN) outh
C					output image host name
      character*(CMLEN)	tlabel(2)
C					tape label(s)
      character*35      msg
C					warning message
      character*16      minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band
      character*45      str1
      character*45	str2
      character*45	str3
C					switched volumes message for operator
      character*(CMLEN) tapeid	
C					tape id
      character*1 	density		
C					tape density	

      character*3       handsk
C					indicator for dumping header and
C					annotation records to disk files
      character*3	ancdsk
C					indicator for dumping ancillary
C					records to a disk file
      character*3	tradsk
C					indicator for dumping trailer 
C					records to a disk file
      character*3	stype
C					sensor type (MSS or RBV)
      character*3	bform
C					band format of tape
      real*8		omin(5)
C					array of output min values
      real*8		omax(5)
C					array of output max values

      character*1       buffer(3596)
C					input buffer
      character*1	thebuf(BSIZE)
C					output buffer
      character*1	bilcnt
C				        number of bands interleaved	

      logical      	flag
C					flag for whether current band
C					is to be processed
      logical		corect
C					indicator for whether the tape
C					was geometrically corrected
      logical		all
C					indicator for whether the user
C					requested that all bands from
C					the input tape be read in
      common/thebuf/thebuf
      common/imgio/fdesc
      common/curent/outlin
      common/outhos/outh
      common/flags/handsk, ancdsk, tradsk, stype
      common/type/corect
       
      data nop/3548/
      data nol/2983/
      data nxt/1/
      data line/0/
     
      mask(1) = 16
      mask(2) = 8
      mask(3) = 4
      mask(4) = 2
      mask(5) = 1
      
C     check that the proper volume is mounted first
C     ---------------------------------------------
      if ((nvol .gt. 1) .and. (volnum .gt. 1)) then
        str1='Second volume of the set was mounted first.'
        str2='Mount the other volume of the set instead.'
        str3='The two volumes in the set are: '
        newmsg=str1//char(13)//char(10)//str2//char(13)//
     +       char(10)//str3//char(13)//char(10)//tlabel(1)//
     +       char(13)//char(10)//tlabel(2)//char(0)
      call errmsg('Volumes were switched','edipsin-volflag',1)
        call errmsg
     +    ('Second volume was mounted first','edipsin-volflag',1)
        call errmsg
     +    ('Operator was told to mount first volume',
     +     'edipsin-volflag',1)

        tstat = tpclos(unit,HANDLE,TRWOFL,1)
        if (tstat .ne. TSUCC) then
    	   call errmsg ('Nonfatal error encountered','edipsin-warn',1)
        endif
        density = char(0)
        tstat = tpopen(unit,0,HANDLE,TNOREW,density,TREAD,newmsg,
     + tapeid)
        if (tstat .ne. TSUCC) call fclean(unit)

        call rddir(unit, nvol, bform, volnum, mnum)
      endif

C     initialize the array of min and max values 
C     ------------------------------------------
      call inmnmx(EBYTE,5,omin,omax)

C     validate the window
C     -------------------
      call adjwin(window,nop,nol,np,nl,unit)

C     determine the number of lines and pixels in the output image
C     ------------------------------------------------------------
      tnl = ( nl-1 ) / lininc  + 1
      tnp = ( np-1 ) / smpinc + 1

C     read the header record
C     ----------------------
      size = 3596 
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the needed header information
C     ---------------------------------
      call hddat( buffer,.true.,.false.,inumb )

C     determine the number of lines interleaved
C     -----------------------------------------
      bilcnt = buffer(121)

C     ----------------------------------------
C     |if all bands were wanted              |
C     |   build the file list		     |
C     |else				     |
C     |   check that the bands which the user|
C     |   specified in the file list are     |
C     |   actually present on the tape       |
C     |endif 				     |
C     ----------------------------------------

      word = ichar(buffer(3586))

      btot = 0
      do 10 i = 1,5
         trlist(i) = 0
10     continue

      if (all) then

	btot = 1 

	do 025 i = 1,5
 	  call pxbtan(word,mask(i),val,1,EWORD)
          if (val .ne. 0) then
            trlist(btot) = i
            btot = btot + 1
          endif
025     continue

        btot = btot - 1

      else

        do 030 i = 1,nband
	  call pxbtan(word,mask(files(i)),val,1,EWORD)
          if (val .eq. 0) then
            flag = .true.
            msg = 'There is no band   on the tape'
            write (msg(18:18),'(I1)') files(i)
            call errmsg(msg,'edipsin-fewbands',1)
          else
	    trlist(i) = files(i)
	    btot = btot + 1
          endif
030     continue

      endif

C     Reset the number of bands to the actual number of bands being processed
      nband = btot

C     dump the header information to the related file (if necessary)
C     --------------------------------------------------------------
      if (handsk .eq. 'YES') then
         write (unit=unithd,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg 
     +  ('Error writing to header file','edipsin-writerr',1)
         endif
      endif

C     read the annotation record
C     --------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the necessary annotation information
C     ----------------------------------------
      call anndat( buffer )
        
C     dump the annotation information out to associated file (if necessary)
C     ---------------------------------------------------------------------
      if (handsk .eq. 'YES') then
         write (unit=unitan,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg('Error writing to annotation file', 
     + 	                'edipsin-writerr',1)
         endif
      endif

C     read the end of file mark
C     -------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. TEEOF) call fclean(unit)

C     if necessary, skip to the starting line
C     ---------------------------------------
      if (window(1) .gt. 1) then
   	 call errmsg
     +    ('Skipping to starting line','informational',1)

         lskip = (window(1)-1) * ichar(bilcnt)

         line = lskip
	 tstat = tpfsr(unit,lskip,HANDLE)
         if (tstat .ne. TSUCC) call fclean(unit)
      endif

C     read the starting line
C     ----------------------
      size = 3596
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     open the image file
C     -------------------
      call opens(tnl,tnp,btot,unit,gdesc,offset)

C     ------------- start the display messages ------------------
C     This assignment is made just so that the processing message
C     doesn't start with "Processed line 0" -- for standards!!)  
C     -----------------------------------------------------------
      call inittm('EDIPSIN','Processed','line','bands',' ',
     +		  tnl,btot,msgtim,ALLCAT)
      outlin = 1
      call startm
      outlin = 0

C     process the image data
C     ----------------------
      ibegin = window(2) + 12
      
      do 300 i = 1,nl
        if( mod(i-1,lininc) .eq. 0 )then
   	     outlin = outlin + 1
             estat =  estep(gdesc)
             if (estat .ne. ESUCC) then
		call stoptm
		call fclean(unit)
             endif
        endif
        

	do 200 j = 1,ichar(bilcnt)

          if (files(nxt) .eq. j) then
             if( mod(i-1,lininc) .eq. 0 )then
                if (smpinc .gt. 1) then
                   jpix = 0
                   do 100 ipix = 1,np,smpinc
                      jpix = jpix + 1
                      ifrom = ibegin + ipix -1
		      ito = offset(nxt) + jpix 
		      thebuf(ito) = buffer(ifrom)
100                continue
                else
                  call pxcopy(buffer(ibegin),thebuf(offset(nxt)+1),
     +       			  1,np)
                endif

                if (minmax.eq.'YES')
     +		   call pxmnmx(thebuf(offset(nxt)+1),tnp,
     +         		       omin(nxt),omax(nxt),EBYTE)
             endif
             nxt = nxt + 1
          endif

          line = line + 1

C         determine if the next volume needs to be mounted
C         ------------------------------------------------
          if (nvol.eq.2) then

             if ((ichar(bilcnt).eq.5).and.(line.eq.7455)) then

	        call stoptm
 	 	call nxtvol(unit,tlabel,mstat)
                if (mstat .ne. 0) then
		   call stoptm
		   call fclean(unit)
        	endif
	        call startm

             elseif ((ichar(bilcnt).eq.4).and.(line.eq.5964)) then

		call stoptm
 		call nxtvol(unit,tlabel,mstat)
                if (mstat .ne. 0) then
	 	   call stoptm
		   call fclean(unit)
        	endif
	        call startm

             endif
          endif

C         read the next image line
C         ------------------------
          tstat = tpread(unit,buffer,size,HANDLE)
          if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call stoptm
	     call fclean(unit)
          endif

200     continue

        nxt = 1
300   continue

C     disable the display messages
C     ----------------------------
      call stoptm

C     flush the buffer
C     ----------------
      estat = estep(gdesc)
      if (estat .ne. ESUCC) call fclean(unit)

C     if necessary, process the trailer records
C     -----------------------------------------
      if (tradsk .eq. 'YES') then

C       if necessary, skip to trailer records
C       -------------------------------------
        lafter = (nol - window(1)) + 1
        if (lafter .ne. nl) then
	   tstat = tpfsf(unit,1,HANDLE)
	   if (tstat .ne. TSUCC) call fclean(unit)
        endif
     
C       read the trailer records
C       ------------------------

        do 400 i = 1,ichar(bilcnt)

          size = 3596
          tstat = tpread(unit,buffer,size,HANDLE)
          if (tstat .ne. size) call fclean(unit)
          
	  if (files(nxt) .eq. i) then
            write (unit=unittr,rec=nxt,iostat=status) buffer 
            if (status .ne. 0) then
	       call errmsg 
     +          ('Error writing to trailer file','edipsin-writerr',1)
            endif
            nxt = nxt + 1 
	  endif

400     continue
      endif

C     send an error message if the warning flag was set      
C     -------------------------------------------------
      if (flag) then
	 call errmsg('Processed fewer bands than specified', 
     +               'edipsin-fewbands',1)
      endif
C
C  If no min/max value has been calculated use bounded because output
C  image is byte.
C
	if (minmax .eq. 'NO') then
           minmax = 'BOUNDED'
	do 101 i = 1, btot
	    omax(i) = 255.0
	    omin(i) = 0.0
 101    continue
	endif

C     save the min and max values
C     ---------------------------
      call svmnmx(outh,btot,omin,omax,minmax)

C     close the output image
C     ----------------------
      estat = egclse(gdesc)

      return
      end

C     **************************************************************************
C
C     UNCMSS processes geometrically uncorrected MSS data in band
C     sequential (BSQ) format.
C
C     **************************************************************************
      subroutine uncmss (files,window,nband,unithd,unitac,
     +			 unitan,unittr,unit,smpinc,lininc,all,tlabel,
     +			 nvol,mnum,volnum,minmax,msgtim)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unitac
C					unit number of ancillary file
      integer*4		unittr
C					unit number of trailer file
      integer*4		unitcr
C					unit number of correction file
      integer*4		unitmd
C					unit number of modeling file
      integer*4		status
C					status inidicator
      integer*4		size
C					buffer size
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpopen
C					tapeio function declaration
      integer*4		tpclos
C					tapeio function declaration
      integer*4		tpread
C					tapeio function declaration
      integer*4		tpfsf
C					tapeio function declaration
      integer*4		tpfsr
C					tapeio function declaration
      integer*4		lskip
C					lines to skip to get to starting line
      integer*4		lafter
C					lines remaining after skipping
      integer*4		mstat
C					mount status from nxtvol call
      integer*4		estat
C					imageio status indicator
      integer*4		eopenr
C					imageio function declaration
      integer*4		ewrite
C					imageio function declaration
      integer*4		eclose
C		  			imageio function declaration
      integer*4		rfdesc
C					file descriptor
      integer*4		msgtim
C					how often to display processing message
      integer*4		smpinc
C					sample increment
      integer*4		lininc
C					line increment
      integer*4		window(5)
C                    			window specification
      integer*4		files(5)
C					user-specified list of bands from
C					MSS input tape to be output
      integer*4		trlist(5)
C					actual list of bands that
C					are output
      integer*4         inumb
C				        band number -- needed by "hddat"
      integer*4		file
C					current band being processed
      integer*4		nxt
C					next band in list
      integer*4		nband
C      					number of bands that user specified
      integer*4		btot
C					actual number of bands output
      integer*4		numimg
C					total number of bands on tape
      integer*4         nvol
C  					number of tape volumes in set
      integer*4		volnum
C 				        current volume number
      integer*4		mnum
C 		   			Landsat mission number
      integer*4		rtype
C					record type
      integer*4		tnp
C					total number of output pixels
      integer*4		tnl
C				        total number of output lines
      integer*4		nol
C					number of lines in input image
      integer*4		nop
C					number of pixels in input image
      integer*4		nl
C					nl from window specification
      integer*4		np
C					np from window specification
      integer*4		outlin
C					output line counter--for display message
      integer*4	     	outbnd
C					current output band--for display message
      integer*4		ibegin, ito, ifrom
C					array indices
      integer*4 	i, j, jpix, ipix
C					loop counters/indices

      integer*2 	decode
C					decode function declaration--
C					used for decoding the encryped
C					binary actual line length
      integer*2		actlen  
C					actual line length

      character*305     newmsg
C					new mount message

      character*(CMLEN) tlabel(2)
C					tape label(s)
      character*(CMLEN) outh
C					output image host name
      character*(255) hdhos
C					header file host name
      character*(255)	achos
C					ancillary file host name
      character*(255)	anhos
C					annotation file host name
      character*(255) trhos
C					trailer file host name
      character*(255)	corhos
C					correction file host name
      character*(255)	modhos
C					modeling file host name

      character*16      minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band
      character*35      msg
C					warning message
      character*45      str1
      character*45	str2
      character*45	str3
C					switched volumes message for operator
C
      character*(CMLEN) tapeid	
C						tape id
      character*1	density		
C					tape density

      character*3       handsk	
C					indicator for dumping header and
C					annotation records to disk files
      character*3	ancdsk
C					indicator for dumping ancillary
C					records to a disk file
      character*3	tradsk
C					indicator for dumping trailer
C					records to a disk file
      character*3	stype 
C					sensor type (MSS or RBV)
      character*3	bform
C					band format of tape
      real*8		omin(5)
C					array of output min values
      real*8       	omax(5)
C					array of output max values

      character*1       headr(3596)
C					header buffer
      character*1	buffer(3596)
C					input buffer
      character*1	bufout(3596)
C					output buffer
      character*1	gain(5)
C					sensor gain options
      character*1	trans(5)
C					MSS transmission type

      logical           flag
C					flag for whether current band
C					is to be processed
      logical		corect
C					indicator for whether the tape
C					was geometrically corrected
      logical		all
C					indicator for whether the user
C					requested that all bands from
C					the input tape be read in
      logical		first 
C					flag indicating whether current
C					band is the first band
 
      common/outhos/outh
      common/hosts/hdhos, achos, anhos, trhos, corhos, modhos
      common/flags/handsk, ancdsk, tradsk, stype
      common/type/corect
      common/curent/outlin
      common/curcat/outbnd

      equivalence (headr(3587),gain(1))
      equivalence (headr(3592),trans(1))

      data nop/3548/
      data nol/2400/
      data unitcr/7/
      data unitmd/9/

      file = 1
      nxt = 1
      outbnd = 0
      do 5 i = 1,5
         trlist(i) = 0
5     continue
      first = .true.

C     initialize the array of min and max values 
C     ------------------------------------------
      call inmnmx(EBYTE,5,omin,omax)

C     create the host name for the associated correction file
C     -------------------------------------------------------
      call lsmknm(outh,'edips_cor',corhos)

C     open the systematic correction file
C     -----------------------------------
      open(unit=unitcr,file=corhos,status='new',
     +     access='sequential',form='formatted',iostat=status)
      if (status .ne. 0) then
	  call errmsg
     +     ('Error opening correction file','edipsin-openerr',1)
      endif

C     create the host name for the associated modeling file
C     -----------------------------------------------------
      call lsmknm(outh,'edips_mod',modhos)

C     open the platform modeling file
C     -------------------------------
      open(unit=unitmd,file=modhos,status='new',
     +     access='sequential',form='formatted',iostat=status)
      if (status .ne. 0) then
	  call errmsg
     +     ('Error opening modeling file','edipsin-openerr',1)
      endif

C     validate the window
C     -------------------
      call adjwin(window,nop,nol,np,nl,unit)
      
C     determine the number of lines and pixels in the output image
C     ------------------------------------------------------------
      tnp = ( np-1 ) / smpinc + 1
      tnl = ( nl-1 ) / lininc + 1

C     read the header record
C     ----------------------
25    size = 3596 
      tstat = tpread(unit,headr,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the needed header information
C     ---------------------------------
      call hddat( headr,.true.,.false.,inumb )
  
C     check that all the specified bands are present on the tape
C     ----------------------------------------------------------

      numimg = ichar(headr(135))

      btot = 0
      if (all) then
         btot = numimg
         do 30 i = 1,btot
            trlist(i) = i
30       continue
      else
         do 35 i = 1,nband
	    if (files(i).le.numimg) then
	       btot = btot + 1
               trlist(btot) = files(i)
            else
               msg = 'There is no band   on the tape'
               write (msg(18:18),'(I1)') files(i)
               call errmsg(msg,'edipsin-fewbands',1)
            endif
35       continue
      endif
   
C     Reset the number of bands to the actual number of bands being processed
      nband = btot
      
C     check that the proper volume is mounted first
C     ---------------------------------------------
      if (nvol .eq. 2) then
        str1='Second volume of the set was mounted first.'
        str2='Mount the other volume of the set instead.'
        str3='The two volumes in the set are: '
        newmsg=str1//char(13)//char(10)//str2//char(13)//
     +       char(10)//str3//char(13)//char(10)//tlabel(1)//
     +       char(13)//char(10)//tlabel(2)//char(0)
      endif

      if (numimg .eq. 4) then  
        if ((volnum.gt.1).and.(files(1).lt.3)) then
 	   call errmsg
     +   ('Volumes were switched','edipsin-volflag',1)
	   call errmsg
     +	    ('Second volume was mounted first','edipsin-volflag',1)
	   call errmsg
     +      ('Operator was told to mount first volume',
     +       'edipsin-volflag',1)

          tstat = tpclos(unit,HANDLE,TRWOFL,1)
          if (tstat .ne. TSUCC) then
	     call errmsg
     +      ('Nonfatal error encountered','edipsin-warn',1)
	  endif
          density = char(0)
          tstat = tpopen(unit,0,HANDLE,TNOREW,density,TREAD,newmsg,
     + tapeid)
          if (tstat .ne. TSUCC) call fclean(unit)

          call rddir(unit, nvol, bform, volnum, mnum)
          goto 25
        endif
      elseif (numimg.eq.5) then
        if ((volnum.gt.1).and.(files(1).lt.4)) then
	   call errmsg
     +     ('Volumes were switched','edipsin-volflag',1)
	   call errmsg
     +     ('Second volume was mounted first','edipsin-volflag',1)
	   call errmsg
     +     ('Operator was told to mount first volume',
     +      'edipsin-volflag',1)

          tstat = tpclos(unit,HANDLE,TRWOFL,1)
 	  if (tstat .ne. TSUCC) then
	     call errmsg
     +       ('Nonfatal error encountered','edipsin-warn',1)
	  endif
          density = char(0)
          tstat = tpopen(unit,0,HANDLE,TNOREW,density,TREAD,newmsg,
     + tapeid)
          if (tstat .ne. TSUCC) call fclean(unit)

          call rddir(unit, nvol, bform, volnum, mnum)
          goto 25
        endif
      endif

C     read ancillary record #1 
C     ------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     retrieve, decode and write the systematic correction and
C     platform modeling data found in ancillary record #1
C     --------------------------------------------------------
      call ancdt1(buffer,unitcr,unitmd,mnum)
        
C     dump ancillary record #1 to the related file (if necessary)
C     -----------------------------------------------------------
      if (ancdsk .eq. 'YES') then
         write (unit=unitac,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg('Error writing to ancillary file', 
     +	                'edipsin-writerr',1)
         endif
      endif

C     read ancillary record #2
C     ------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     retrieve, decode and write the systematic correction and
C     platform modeling data found in ancillary record #2
C     --------------------------------------------------------
      call ancdt2(buffer,unitcr,unitmd)

C     dump ancillary record #2 out to the related file (if necessary)
C     ---------------------------------------------------------------
      if (ancdsk .eq. 'YES') then
          write (unit=unitac,iostat=status) buffer 
          if (status .ne. 0) then
	     call errmsg 
     +      ('Error writing to ancillary file','edipsin-writerr',1)
          endif
      endif

C     close the platform modeling file
C     --------------------------------
      close (unitmd)

C     read and dump the remaining ancillary records 
C     these contain the projection dependent data
C     ---------------------------------------------
      do 050 i = 1,24
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

	if (ancdsk .eq. 'YES') then
           write (unit=unitac,iostat=status) buffer 
           if (status .ne. 0) then
	      call errmsg('Error writing to ancillary file', 
     +	                  'edipsin-writerr',1)
          endif
        endif
050   continue

C     read and dump the two annotation records
C     ----------------------------------------
      do 075 i = 1,2
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

C	get the needed annotation information
C	-------------------------------------
	call anndat( buffer )
        
	if (files(1) .eq. 1) then
          gain(file) = buffer(91)
          trans(file) = buffer(92)
        endif
        
	if (handsk .eq. 'YES') then
           write (unit=unitan,iostat=status) buffer 
           if (status .ne. 0) then
	      call errmsg('Error writing to annotation file', 
     +		          'edipsin-writerr',1)
           endif
        endif
075   continue

C     read the end of file mark
C     -------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. TEEOF) call fclean(unit)

C     initialize the display messages
C     -------------------------------
      call inittm('EDIPSIN','Processed','line','band',' ',tnl,
     +		  btot,msgtim,ONECAT)

C     read the first image line
C     -------------------------
      size = 3596
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     open the output image 
C     ---------------------
      estat = eopenr(rfdesc,outh,btot,1,1,tnl,tnp,IWRITE,EBYTE,0,1)
      if (estat .ne. ESUCC) call fclean(unit)

100   continue

C     determine if current band is to be processed
C     --------------------------------------------
        if (files(nxt) .eq. file) then
           flag = .true.
        else
           flag = .false.
          call errmsg('Skipping over input band','informational',1)
        endif

125     continue

C       read until find an image record
C       -------------------------------

          rtype = ichar(buffer(6))

          if ((flag) .and. (rtype .eq. 219)) then
            gain(file) = buffer(91)
            trans(file) = buffer(92)
          endif
          
	  if ( rtype.eq.18 ) call hddat( buffer,.true.,.true.,inumb )
          
	  if ((rtype .eq. 237) .and. (tstat .ne. TEEOF)) goto 150

          size = 3596 
          tstat = tpread(unit,buffer,size,HANDLE)
          if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call fclean(unit)
          endif

          goto 125

150     continue
	if (flag) outbnd = outbnd + 1
	ibegin = window(2) + 12

C       if necessary, skip to the starting line
C       ---------------------------------------
        if (window(1).gt.1) then
           lskip = window(1) - 2
           if (flag) 
     +	    call errmsg('Skipping to starting line','informational',1)

           tstat = tpfsr(unit,lskip,HANDLE)
           if (tstat .ne. TSUCC) call fclean(unit)

           tstat = tpread(unit,buffer,size,HANDLE)
           if (tstat .ne. size) call fclean(unit)

        endif

C       ------------- start the display messages -------------------
C       (This assignment had to made just so that the processing 
C       message didn't start with "Processed line 0" -- standards!!)
C       ------------------------------------------------------------
	outlin = 1
	if (flag) call startm
        outlin = 0 
       
	do 300 j = 1,nl
          if (flag) then
            if ( mod(j-1,lininc) .eq. 0 ) then
   	 	outlin = outlin + 1

  		if (first) then
		 actlen = decode(buffer(3565),buffer(3566))
		 write (unit=unitcr,fmt=*,iostat=status) outlin,actlen
		 if (status .ne. 0) then
                     call xtwrit(' ',XCCSTD)
		     call errmsg
     +		         ('Error writing to correction file',
     +     	          'edipsin-writerr',1)
                 endif
                endif

                 if (smpinc .gt. 1) then
                   jpix= 0
                   do 200 ipix = 1,np,smpinc
                      jpix = jpix + 1
                      ifrom = ibegin + ipix - 1 
                      ito = jpix 
                      bufout(ito) = buffer(ifrom) 
200                continue
                 else
                   call pxcopy(buffer(ibegin),bufout,1,np)
                 endif

 	         if (minmax.eq.'YES')
     +		   call pxmnmx(bufout,tnp,omin(nxt),omax(nxt),EBYTE)

	         estat = ewrite(rfdesc,nxt,outlin,bufout,1)
	         if (estat .lt. 0) then
	 	    call stoptm
		    call fclean(unit)
                 endif

	    endif
          endif
          
C         read the next image line
C         ------------------------
	  tstat = tpread(unit,buffer,size,HANDLE)
 	  if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call stoptm
	     call fclean(unit)
          endif

300     continue

	if (flag) then
	   call stoptm
	   nxt = nxt + 1
        endif

        if ((first) .and. (flag)) first = .false.
        file = file + 1

C       if necessary, skip to the trailer record
C       ----------------------------------------
        lafter = (nol - window(1)) + 1
        if (lafter.ne.nl) then
	   tstat = tpfsf(unit,1,HANDLE)
           if (tstat .ne. TSUCC) call fclean(unit)
	endif

C       read the trailer record
C       -----------------------
	size = 3596 
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)
        
C       dump trailer record out to the related file (if necessary)
C       ----------------------------------------------------------
	if (flag) then
           if (tradsk .eq. 'YES') then
              write (unit=unittr,rec=nxt-1,iostat=status) buffer 
              if (status .ne. 0) then
	        call errmsg('Error writing to trailer file', 
     +		             'edipsin-writerr',1)
              endif
           endif
        endif

C       return if all bands have been processed
C       ---------------------------------------
        if (nxt .gt. btot) then
          if (handsk .eq. 'YES') then
             write (unit=unithd,iostat=status) headr
             if (status .ne. 0) then
                call errmsg('Error writing to header file', 
     +		            'edipsin-writerr',1)
            endif
          endif
	  goto 600
        endif

C       check if need to mount the next volume
C       otherwise read the end of file mark
C       --------------------------------------
        if ((nvol.eq.2).and.(file.eq.4)) then
	    call nxtvol(unit,tlabel,mstat)
            if (mstat .ne. 0) call fclean(unit)

        else
            tstat = tpread(unit,buffer,size,HANDLE)
            if (tstat .eq. TEOV) then
	      if (handsk.eq.'YES') then 
		 write(unit=unithd,iostat=status) headr
		 if (status .ne. 0) then
		    call errmsg('Error writing to header file',
     +    	                'edipsin-writerr',1)
		 endif
              endif

    	      if (.not. all) then
		 call errmsg
     +             ('Processed fewer bands than specified',
     +              'edipsin-fewbands',1)
     	      endif
              goto 600
            endif
        endif

C       read the next record
C       --------------------
        size = 3596 
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

        goto 100

600   continue
C
C  If no min/max value has been calculated use bounded because output
C  image is byte.
C
	if (minmax .eq. 'NO') then
           minmax = 'BOUNDED'
	do 101 i = 1, btot
	    omax(i) = 255.0
	    omin(i) = 0.0
 101    continue
	endif

C     save the min and max values
C     ---------------------------
      call svmnmx(outh,btot,omin,omax,minmax)

C     output the bands selected to the correction file
C     ------------------------------------------------
      do 620 i = 1,5 
         write (unit=unitcr,fmt=*,iostat=status) trlist(i)
         if (status .ne. 0) then
	    call errmsg('Error writing to correction file',
     +	                'edipsin-writerr',1)
         endif
620   continue

C     close the systematic correction file
C     ------------------------------------
      close (unitcr)

C     close the output image
C     ----------------------
      estat = eclose(rfdesc)

      return
      end

C     **************************************************************************
C
C     UNCBIL processes geometrically uncorrected MSS data in band
C     interleaved by line (BIL) format.
C
C     **************************************************************************
      subroutine uncbil(unit,unithd,unitan,unitac,unittr,window,
     +	  	 	smpinc,lininc,nvol,tlabel,files,
     +			nband,all,volnum,mnum,minmax,msgtim)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unitac
C					unit number of ancillary file
      integer*4		unittr
C					unit number of trailer file
      integer*4		unitcr
C					unit number of correction file
      integer*4		unitmd
C					unit number of modeling file
      integer*4		status
C					status inidicator
      integer*4		size
C					buffer size
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpopen
C					tapeio function declaration
      integer*4		tpclos
C					tapeio function declaration
      integer*4		tpread
C					tapeio function declaration
      integer*4		tpfsf
C					tapeio function declaration
      integer*4		tpfsr
C					tapeio function declaration
      integer*4		mstat
C					mount status from nxtvol call
      integer*4		lskip
C				        lines to skip to get to starting line
      integer*4		lafter
C					lines remaining after skipping
      integer*4         outlin
C					output line counter
      integer*4    	estat
C					imageio status indicator
      integer*4		estep
C					imageio function declaration
      integer*4		egclse
C		  			imageio function declaration
      integer*4		gdesc
C					group descriptor
      integer*4         fdesc(2)
C					file descriptor array
      integer*4		offset(5)
C					array of offsets into "thebuf"
      integer*4		msgtim
C					how often to display processing message
      integer*4		smpinc
C					sample increment
      integer*4		lininc
C					line increment
      integer*4		window(5)
C                    			window specification
      integer*4		files(5)
C                                       user-specified list of bands from
C					MSS input tape to be output
      integer*4		trlist(5)
C					actual list of bands 
C 					that are output
      integer*4         inumb
C				        band number -- needed by "hddat"
      integer*4		nxt
C					next band in list
      integer*4		nband
C   				        number of bands that user specified	
      integer*4		btot
C					actual number of bands output
      integer*4		nvol
C  					number of tape volumes in set
      integer*4		volnum
C 				        current volume number
      integer*4		mnum
C 		   			Landsat mission number
      integer*4		tnp
C					total number of output pixels
      integer*4		tnl
C				        total number of output lines
      integer*4		nol
C					number of lines in input image
      integer*4		nop
C					number of pixels in input image
      integer*4		nl  
C					nl from window specification
      integer*4		np
C					np from window specification
      integer*4		line
C					line counter
      integer*4		ibegin, ito, ifrom 
C					array indices
      integer*4		i, j, jpix, ipix
C					loop counters/indices

      integer*2		mask(5)
C					mask to determine the image
C					bands present
      integer*2		word
C					indication of data present
C					by band
      integer*2		val
C					return value from pxbtan function
      integer*2		decode
C					decode function declaration--
C					used for decoding the encrypted
C					binary actual line length
      integer*2		actlen
C					actual line length

      character*305     newmsg
C					new mount message

      character*(CMLEN) tlabel(2)
C					tape label(s)
      character*(CMLEN)	outh
C					output image host name
      character*(255)	hdhos
C					header file host name
      character*(255) achos
C					ancillary file host name
      character*(255)	anhos
C					annotation file host name
      character*(255)	trhos
C					trailer file host name
      character*(255)	corhos
C					correction file host name
      character*(255)	modhos
C					modeling file host name
      character*45      str1
      character*45	str2
      character*45	str3
C					switched volume message for operator
      character*(CMLEN) tapeid	
C						tape id
      character*1	density		
C					tape density	
      character*35      msg
C					warning message
      character*16      minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band
      character*3       handsk
C					indicator for dumping header and
C					annotation records to disk files
      character*3	ancdsk
C					indicator for dumping ancillary 
C					records to a disk file
      character*3	tradsk
C					indicator for dumping trailer 
C					records to a disk file
      character*3	stype
C					sensor type (MSS or RBV)
      character*3	bform
C					band format of tape
      real*8		omin(5)
C					array of output min values
      real*8		omax(5)
C					array of output max values

      character*1       buffer(3596)
C					input buffer
      character*1	thebuf(BSIZE)
C					output buffer
      character*1	bilcnt
C				        number of bands interleaved	

      logical      	flag
C					flag for whether current band
C					is to be processed
      logical           corect
C					indicator for whether the tape
C					was geometrically corrected
      logical		all
C					indicator for whether the user
C					requested that all bands from
C					the input tape be read in

      common/thebuf/thebuf
      common/imgio/fdesc
      common/curent/outlin
      common/outhos/outh
      common/hosts/hdhos, achos, anhos, trhos, corhos, modhos
      common/flags/handsk, ancdsk, tradsk, stype  
      common/type/corect

      data nop/3548/
      data nol/2400/
      data nxt/1/ 
      data line/0/
      data unitcr/7/
      data unitmd/9/
     
      mask(1) = 16
      mask(2) = 8
      mask(3) = 4
      mask(4) = 2
      mask(5) = 1
      
C     check that the proper volume is mounted first
C     ---------------------------------------------
      if ((nvol.gt.1).and.(volnum.gt.1)) then
        str1='Second volume of the set was mounted first.'
        str2='Mount the other volume of the set instead.'
        str3='The two volumes in the set are: '
        newmsg=str1//char(13)//char(10)//str2//char(13)//
     +       char(10)//str3//char(13)//char(10)//tlabel(1)//
     +       char(13)//char(10)//tlabel(2)//char(0)
        call errmsg('Volumes were switched',
     +                'edipsin-volflag',1)
        call errmsg('Second volume was mounted first',
     +               'edipsin-volflag',1)
        call errmsg
     +  ('Operator was told to mount fisrt volume',
     +   'edipsin-volflag',1)

        tstat = tpclos(unit,HANDLE,TRWOFL,1)
 	if (tstat .ne. TSUCC) then
	    call errmsg('Nonfatal error encountered',
     +			'edipsin-warn',1)
        endif
        density = char(0)
        tstat = tpopen(unit,0,HANDLE,TNOREW,density,TREAD,newmsg,
     + tapeid)
        if (tstat .ne. TSUCC) call fclean(unit)

        call rddir(unit, nvol, bform, volnum, mnum)
      endif

C     initialize the array of min and max values 
C     ------------------------------------------
      call inmnmx(EBYTE,5,omin,omax)

C     validate the window
C     -------------------
      call adjwin(window,nop,nol,np,nl,unit)

C     create the host name for the associated correction file
C     -------------------------------------------------------
      call lsmknm(outh,'edips_cor',corhos)

C     open the systematic correction file
C     -----------------------------------
      open(unit=unitcr,file=corhos,status='new',
     +     access='sequential',form='formatted',iostat=status)
      if (status .ne. 0) then
	  call errmsg('Error opening correction file',
     +                'edipsin-openerr',1)
      endif

C     create the host name for the associated modeling file
C     -----------------------------------------------------
      call lsmknm(outh,'edips_mod',modhos)

C     open the platform modeling file
C     -------------------------------
      open(unit=unitmd,file=modhos,status='new',
     +     access='sequential',form='formatted',iostat=status)
      if (status .ne. 0) then
	  call errmsg('Error opening modeling file',
     +                'edipsin-openerr',1)
      endif

C     determine the number of lines and pixels in the output image
C     ------------------------------------------------------------
      tnl = ( nl-1 ) / lininc  + 1
      tnp = ( np-1 ) / smpinc + 1

C     read the header record
C     ----------------------
      size = 3596 
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the needed header information
C     ---------------------------------
      call hddat( buffer,.true.,.false.,inumb )

C     determine the number of lines interleaved
C     -----------------------------------------
      bilcnt = buffer(121)

C     --------------------------------------
C     if all bands were wanted 
C        build the file list
C     else
C        check that the bands which the user 
C        specified in the file list are
C        actully present on the tape 
C     endif
C     --------------------------------------

      word = ichar(buffer(3586))

      btot = 0
      do 10 i = 1,5
         trlist (i) = 0
10    continue

      if (all) then

	btot = 1 

	do 025 i = 1,5
	  call pxbtan(word,mask(i),val,1,EWORD)
          if (val .ne. 0) then
            trlist(btot) = i
            btot = btot + 1
          endif
025     continue

        btot = btot - 1

      else

        do 030 i = 1,nband
	  call pxbtan(word,mask(files(i)),val,1,EWORD)
          if (val .eq. 0) then
             flag = .true.
             msg = 'There is no band   on the tape'
             write (msg(18:18),'(I1)') files(i)
             call errmsg(msg,'edipsin-fewbands',1)
          else
             trlist(i) = files(i)
 	     btot = btot + 1
          endif
030     continue

      endif

C     Reset the number of bands to the actual number of bands being processed
      nband = btot
      
C     dump the header information to the related file (if necessary)
C     --------------------------------------------------------------
      if (handsk .eq. 'YES') then
         write (unit=unithd,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg('Error writing to header file',
     +                  'edipsin-writerr',1)
        endif
      endif

C     read ancillary record #1
C     ------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     retrieve, decode and write the systematic correction and
C     platform modeling data found in ancillary record #1
C     --------------------------------------------------------
      call ancdt1(buffer,unitcr,unitmd,mnum)

C     dump ancillary record #1 to the related file (if necessary)
C     -----------------------------------------------------------
      if (ancdsk .eq. 'YES') then
         write (unit=unitac,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg ('Error writing to ancillary file',
     +                   'edipsin-writerr',1)
         endif
      endif

C     read ancillary record #2
C     ------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     retrieve, decode and write the systematic correction and
C     platform modeling data found in ancillary record #2
C     --------------------------------------------------------
      call ancdt2(buffer,unitcr,unitmd)

C     dump ancillary record #2 out to the related file (if necessary)
C     ---------------------------------------------------------------
      if (ancdsk .eq. 'YES') then
         write (unit=unitac,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg('Error writing to ancillary file', 
     +                  'edipsin-writerr',1)
         endif
      endif

C     close the platform modeling file
C     --------------------------------
      close (unitmd)

C     read and dump the remaining ancillary records
C     these contain projection dependent data
C     ---------------------------------------------

      do 050 i = 1,24
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

	if (ancdsk .eq. 'YES') then
           write (unit=unitac,iostat=status) buffer 
           if (status .ne. 0) then
	      call errmsg('Error writing to ancillary file', 
     +                    'edipsin-writerr',1)
          endif
        endif
050   continue

C     read and dump the two annotation records
C     ----------------------------------------
      do 075 i = 1,2
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

	call anndat( buffer )
        
	if (handsk .eq. 'YES') then
          write (unit=unitan,iostat=status) buffer 
          if (status .ne. 0) then
	    call errmsg('Error writing to annotation file', 
     + 	                'edipsin-writerr',1)
          endif
        endif
075   continue

C     read the end of file mark
C     -------------------------
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. TEEOF) call fclean(unit)

C     if necessary, skip to the starting line
C     ---------------------------------------
      if (window(1).gt.1) then

        lskip = (window(1)-1) *  ichar(bilcnt)

        call errmsg ('Skipping to starting line','informational',1)
        line = lskip
        tstat = tpfsr(unit,lskip,HANDLE)
        if (tstat .ne. TSUCC) call fclean(unit)
      endif

C     read the starting line
C     ----------------------
      size = 3596
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     open the image file
C     -------------------
      call opens(tnl,tnp,btot,unit,gdesc,offset)

C     ------------- start the display messages -------------------
C     (This assignment had to made just so that the processing 
C     message didn't start with "Processed line 0" -- standards!!)
C     ------------------------------------------------------------
      outlin = 1
      call inittm('EDIPSIN','Processed','line','bands',' ',
     +	          tnl,btot,msgtim,ALLCAT)
      call startm
      outlin = 0

C     process the image data
C     ----------------------
      ibegin = window(2) + 12
      
      do 300 i = 1,nl
        if( mod(i-1,lininc) .eq. 0 )then
   	     outlin = outlin + 1
	     actlen = decode(buffer(3565),buffer(3566))
	     write (unit=unitcr,fmt=*,iostat=status) outlin, actlen
             if (status .ne. 0) then
               call xtwrit(' ',XCCSTD)
	       call errmsg('Error writing to correction file',
     + 	        'edipsin-writerr',1)
             endif

             estat =  estep(gdesc)
             if (estat .ne. ESUCC) then
		call stoptm
		call fclean(unit)
             endif
        endif

	do 200 j = 1,ichar(bilcnt)

          if (trlist(nxt) .eq. j) then
             if( mod(i-1,lininc) .eq. 0 )then
                if( smpinc.gt.1 )then
                   jpix = 0
                   do 100 ipix = 1,np,smpinc
                      jpix = jpix + 1
                      ifrom = ibegin + ipix -1
		      ito = offset(nxt) + jpix 
		      thebuf(ito) = buffer(ifrom)
100                continue
                else
                   call pxcopy(buffer(ibegin),thebuf(offset(nxt)+1),
     +       			  1,np)
                endif

                if (minmax.eq.'YES')
     +             call pxmnmx(thebuf(offset(nxt)+1),tnp,
     +			       omin(nxt),omax(nxt),EBYTE)
             endif
             nxt = nxt + 1
          endif

          line = line + 1

C         determine if the next volume needs to be mounted
C         ------------------------------------------------
          if (nvol.eq.2) then

             if ((ichar(bilcnt).eq.5).and.(line.eq.6000)) then

		call stoptm
 	 	call nxtvol(unit,tlabel,mstat)
                if (mstat .ne. 0) then
		   call stoptm
		   call fclean(unit)
        	endif
	 	call startm

             endif
          endif

C         read next image line
C         --------------------
          tstat = tpread(unit,buffer,size,HANDLE)
	  if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call stoptm
	     call fclean(unit)
          endif
200     continue

        nxt = 1

300   continue

C     disable the display messages
C     ----------------------------
      call stoptm

C     flush the buffer
C     ----------------
      estat = estep(gdesc)
      if (estat .ne. ESUCC) call fclean(unit)

C     if necessary, process the trailer records
C     -----------------------------------------
      if (tradsk .eq. 'YES') then

C       if necessary, skip to trailer records
C       -------------------------------------
        lafter = (nol - window(1)) + 1
        if (lafter.ne.nl) then
           tstat = tpfsf(unit,1,HANDLE)
           if (tstat .ne. TSUCC) call fclean(unit)
        endif

C       read trailer records
C       --------------------

        do 400 i = 1,ichar(bilcnt)

          size = 3596
          tstat = tpread(unit,buffer,size,HANDLE)
          if (tstat .ne. size) call fclean(unit)
          
	  if (trlist(nxt) .eq. i) then
             write (unit=unittr,rec=nxt,iostat=status) buffer 
             if (status .ne. 0) then
	        call errmsg('Error writing to trailer file', 
     +		            'edipsin-writerr',1)
             endif
             nxt = nxt + 1 
	  endif
400     continue
      endif

C     send an error message if the warning flag was set      
C     -------------------------------------------------
      if (flag) then
	 call errmsg('Processed fewer bands than specified', 
     +    	     'edipsin-fewbands',1)
      endif

C     output the bands selected to the correction file     
C     ------------------------------------------------
      do 440 i = 1,5
         write (unit=unitcr,fmt=*,iostat=status) trlist(i)
         if (status .ne. 0) then
            call errmsg('Error writing to correction file',
     +                  'edipsin-writerr',1)
         endif
440    continue
C
C  If no min/max value has been calculated use bounded because output
C  image is byte.
C
	if (minmax .eq. 'NO') then
           minmax = 'BOUNDED'
	do 101 i = 1, btot
	    omax(i) = 255.0
	    omin(i) = 0.0
 101    continue
	endif
		
C     save min and max values
C     -----------------------
      call svmnmx(outh,btot,omin,omax,minmax)

C     close the systematic correction file
C     ------------------------------------
      close (unitcr)

C     close the output image
C     ----------------------
      estat = egclse(gdesc)

      return
      end

C     **************************************************************************
C
C     UNCRBV processes geometrically uncorrected RBV data.
C
C     **************************************************************************
      subroutine uncrbv (unit,unithd,unitan,unitac,unittr,window,
     +			 smpinc,lininc,minmax,msgtim)
      
	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unitac
C					unit number of ancillary file
      integer*4		unittr
C					unit number of trailer file
      integer*4		status
C					status indicator
      integer*4		size
C					buffer size
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpread
C					tapeio function declaration
      integer*4		tpfsr
C					tapeio function declaration
      integer*4		tpfsf
C					tapeio function declaration
      integer*4		estat
C					imageio status indicator
      integer*4		estep
C					imageio function declaration
      integer*4		egclse
C		  			imageio function declaration
      integer*4		gdesc
C					group descriptor
      integer*4		fdesc(2)
C					file descriptor array
      integer*4		offset(5)
C					array of offsets into "thebuf"
      integer*4		msgtim
C					elapsed time between display messages
      integer*4         inumb
C				        band number -- needed by "hddat"
      integer*4         smpinc
C					sample increment
      integer*4		lininc
C					line increment
      integer*4		window(5)
C                    			window specification
      integer*4		lskip
C					lines to skip to get to starting line
      integer*4		lafter
C					lines remaining after skipping
      integer*4		tnp
C					total number of output pixels
      integer*4		tnl
C				        total number of output lines
      integer*4		nol
C					number of lines in input image
      integer*4		nop
C					number of pixels in input image
      integer*4		nl
C					nl from window specification
      integer*4		np
C					np from window specification
      integer*4		outlin
C					output line counter
      integer*4		ibegin, ito, ifrom
C					array indices
      integer*4		i, j, jpix, ipix
C					loop counters/indices

      character*(CMLEN) outh
C					output image host name
      character*16      minmax
C					indicator for whether the user
C					wants the min and max determined
C					for each band
      character*3       handsk
C					indicator for dumping header
C					and annotation records to
C					disk files
      character*3	tradsk    
C					indicator for dumping trailer
C					records to a disk file
      character*3	ancdsk
C					indicator for dumping ancillary
C					records to a disk file
      character*3	stype
C					sensor type (MSS or RBV)
      real*8		omin
C					output min value
      real*8		omax
C					output max value

      character*1       buffer(5388)
C					input buffer
      character*1	thebuf(BSIZE)
C					output buffer

      common/thebuf/thebuf
      common/imgio/fdesc
      common/curent/outlin
      common/outhos/outh
      common/flags/handsk, ancdsk, tradsk, stype

      data nop/5375/
      data nol/4125/

C     initialize min and max
C     ----------------------
      omin = 255.0
      omax = 0.0

C     validate the window
C     -------------------
      call adjwin(window,nop,nol,np,nl,unit)
      
C     determine the number of lines and pixels in the output image
C     ------------------------------------------------------------
      tnl = ( nl-1 ) / lininc  + 1
      tnp = ( np-1 ) / smpinc + 1

C     read the header record
C     ----------------------
      size = 5388 
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     get the needed header information
C     ---------------------------------
      call hddat( buffer,.false.,.false.,inumb )

C     dump the header information to the related file
C     -----------------------------------------------
      if (handsk .eq. 'YES') then
         write (unit=unithd,iostat=status) buffer 
         if (status .ne. 0) then
	    call errmsg('Error writing to header file', 
     +	                'edipsin-writerr',1)
         endif
      endif

C     read the ancillary records
C     --------------------------
      do 050 i = 1,30
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)
	
	if (ancdsk .eq. 'YES') then
           write (unit=unitac,iostat=status) buffer 
           if (status .ne. 0) then
	      call errmsg('Error writing to ancillary file',
     +     		  'edipsin-writerr',1) 
           endif
        endif
050   continue

C     process the annotation records
C     ------------------------------
      do 075 i = 1,2

C       read the annotation record
C       --------------------------
        tstat = tpread(unit,buffer,size,HANDLE)
        if (tstat .ne. size) call fclean(unit)

C	get the needed annotation information
C	-------------------------------------
	call anndat( buffer )
        
C       dump the annotation record out to the related file
C       --------------------------------------------------
	if (handsk .eq. 'YES') then
           write (unit=unitan,iostat=status) buffer 
           if (status .ne. 0) then
     	      call errmsg('Error writing to annotation file',
     +			  'edipsin-writerr',1) 
           endif
        endif
075   continue

C     read the end of file mark
C     -------------------------
      tstat = tpread(unit,buffer,size,HANDLE) 
      if (tstat .ne. TEEOF) call fclean(unit)

C     if necessary, skip to the starting line
C     ---------------------------------------
      if (window(1) .gt. 1) then
         call errmsg('Skipping to starting line','informational',1)
         lskip = window(1) - 1
         tstat = tpfsr(unit,lskip,HANDLE)
         if (tstat .ne. TSUCC) call fclean(unit)
      endif

C     read the starting line
C     ----------------------
      size = 5388
      tstat = tpread(unit,buffer,size,HANDLE)
      if (tstat .ne. size) call fclean(unit)

C     open the image file
C     -------------------
      call opens(tnl,tnp,1,unit,gdesc,offset)

C     ------------- start the display messages -------------------
C     (This assignment had to made just so that the processing 
C     message didn't start with "Processed line 0" -- standards!!)
C     ------------------------------------------------------------
      call inittm('EDIPSIN','Processed','line','bands',' ',
     +  	  tnl,1,msgtim,ALLCAT)
      outlin = 1
      call startm
      outlin = 0

C     process the image data
C     ----------------------
      ibegin = window(2) + 12
      
      do 300 j = 1,nl
          if ( mod(j-1,lininc) .eq. 0 ) then
	     estat = estep(gdesc)
	     if (estat .ne. ESUCC) then
		 call stoptm
	   	 call fclean(unit)
      	     endif

             if (smpinc .gt. 1) then
                jpix = 0
                do 200 ipix = 1,np,smpinc
                   jpix = jpix + 1 
                   ifrom = ibegin + ipix - 1 
                   ito = offset(1) + jpix
                   thebuf(ito) = buffer(ifrom) 
200             continue
             else
                call pxcopy(buffer(ibegin),thebuf(offset(1)+1),1,np)
             endif

             outlin = outlin + 1

             if (minmax.eq.'YES') 
     +          call pxmnmx(thebuf(offset(1)+1),tnp,omin,omax,EBYTE)

	  endif
      
          tstat = tpread(unit,buffer,size,HANDLE)
          if ((tstat .ne. size) .and. (tstat .ne. TEEOF)) then
	     call stoptm
	     call fclean(unit)
          endif

300   continue

C     disable the display messages
C     ----------------------------
      call stoptm

C     flush the buffer
C     ----------------
      estat = estep(gdesc)
      if (estat .ne. ESUCC) call fclean(unit)

C     if necessary, process the trailer file
C     --------------------------------------
      if (tradsk .eq. 'YES') then
 
C         if necessary, skip forward to the trailer file
C         ----------------------------------------------
          lafter = (nol - window(1)) + 1
          if (lafter .ne. nl) then
             tstat = tpfsf(unit,1,HANDLE)
	     if (tstat .ne. TSUCC) call fclean(unit)
          endif

C         read the trailer record
C         -----------------------
          size = 5388 
          tstat = tpread(unit,buffer,size,HANDLE)
          if (tstat .ne. size) call fclean(unit)

C         dump the trailer record information to the related file
C         -------------------------------------------------------
          write (unit=unittr,rec=1,iostat=status) buffer 
          if (status .ne. 0) then
	     call errmsg('Error writing to trailer file', 
     +     		 'edipsin-writerr',1)
          endif

      endif
C
C  If no min/max value has been calculated use bounded because output
C  image is byte.
C
	if (minmax .eq. 'NO') then
            minmax = 'BOUNDED'
	    omax = 255.0
	    omin = 0.0
	endif

C     save the min and max
C     --------------------
      call svmnmx(outh,1,omin,omax,minmax)

C     close the output image
C     ----------------------
      estat = egclse(gdesc)

      return 
      end

C     **************************************************************************
C
C     CLOSER closes the image related (associated) files.
C
C     **************************************************************************
      subroutine closer (unithd,unitac,unitan,unittr)
      

	include 'las.fin'

      integer*4		unithd
C					unit number of header file
      integer*4		unitan
C   					unit number of annotation file
      integer*4		unitac
C					unit number of ancillary file
      integer*4		unittr
C					unit number of trailer file
      logical           opn 
C					open file indicator

C     close header file
C     -----------------
      inquire (unit=unithd,opened=opn)
      if (opn) close (unit=unithd,err=100)

C     close ancillary file
C     --------------------
      inquire (unit=unitac,opened=opn)
      if (opn) close(unit=unitac,err=100)

C     close annotation file
C     ---------------------
      inquire (unit=unitan,opened=opn)
      if (opn) close(unit=unitan,err=100)

C     close trailer file
C     ------------------
      inquire (unit=unittr,opened=opn)
      if (opn) close(unit=unittr,err=100)

100   continue 

      return
      end

C     **************************************************************************
C
C     ADJWIN validates the window specification.
C
C     **************************************************************************
      subroutine adjwin(window,nop,nol,np,nl,unit)

	include 'las.fin'

      integer*4		unit
C					tape unit number
      integer*4	 	window(5) 
C                    			window specification
      integer*4		nol
C					number of lines in input image
      integer*4		nop
C					number of pixels in input image
      integer*4		nl 
C					nl from window specification
      integer*4		np
C					np from window specification

      character*(CMLEN) msg
C					window error message

      if (window(5) .eq. 0) then
C          set the window to the entire image if no window was specified
C          -------------------------------------------------------------
           window(1) = 1
           window(2) = 1
	   window(3) = nol
	   window(4) = nop
      else

C          print an error message if the starting sample is greater than nop
C          -----------------------------------------------------------------
           if (window(2) .gt. nop) then
	      call errmsg
     +        ('Specified window is outside the input image', 
     +         'edipsin-window',1)
	      call fclean(unit)

C          adjust the number of samples if the number specified exceeds
C          the size of the image
C          -------------------------------------------------------------
           elseif ( (window(2)+window(4)-1) .gt. nop ) then
             window(4) = nop - window(2) + 1
	     msg = 'Window exceeds image size, NS adjusted to     '
	     write (msg(44:47),'(I4)') window(4)
	     call errmsg (msg,'edipsin-adjsamp',1)
           endif

C          print an error message if the starting line is greater than nol
C          ---------------------------------------------------------------
           if (window(1) .gt. nol ) then
	      call errmsg 
     +        ('Specified window is outside the input image', 
     +         'edipsin-window',1) 
	      call fclean(unit)

C           adjust the number of lines if the number specified exceeds the
C           size of the image
C           --------------------------------------------------------------
           elseif ( (window(1)+window(3)-1) .gt. nol ) then
             window(3) = nol - window(1) + 1
	     msg = 'Window exceeds image size, NL adjusted to     '
	     write (msg(44:47),'(I4)') window(3)
	     call errmsg (msg,'edipsin-adjline',1)
           endif
      endif
      
      nl = window(3)
      np = window(4)
      
      return
      end

C     **************************************************************************
C
C     FCLEAN closes the tape and aborts the program after a fatal error
C     is encountered during processing.
C
C     **************************************************************************
      subroutine fclean(unit)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		status
C					tapeio status indicator
      integer*4		tpclos  
C					tape close function declaration

C     close the tape
C     --------------
      status = tpclos(unit,HANDLE,TRWOFL,1)

C     print the terminating message and abort
C     ---------------------------------------
      call errmsg('Fatal error encountered','edipsin-fatal',-1)
      
      return 
      end

C     **************************************************************************
C
C     NXTVOL mounts the next tape volume in a two-volume set.
C
C     **************************************************************************
      subroutine nxtvol (unit,tlabel,mstat)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4		unit
C					tape unit number
      integer*4		tstat
C					tapeio status indicator
      integer*4		tpopen
C					tape open function declaration
      integer*4		tpclos
C					tape close function declaration
      integer*4		mstat
C				        returned mount status indicator	
      character*(CMLEN) opmesg
C					mount message sent to operator
      character*(CMLEN)	tlabel(2)
C					tape label(s)
      character*(CMLEN)	temp
C					temporary variable
      character*(CMLEN) tapeid	
C						tape id

      character*1  	density		
C					tape tape density 

 
C     close the current volume 
C     ------------------------
      tstat = tpclos(unit,HANDLE,TRWOFL,1)
      if (tstat .ne. TSUCC)
     + call errmsg('Nonfatal error encountered','edipsin-warn',1)

C     inform user that next tape volume is waiting to be mounted      
C     ----------------------------------------------------------
      call errmsg('Second tape volume is being mounted',
     +	          'informational',1)

C     build the null terminated string containing the tape label for 
C     the second volume of the set
C     --------------------------------------------------------------
      temp = tlabel(2)
      opmesg = temp//char(0)

C     mount and open next tape volume
C     set return status indicator
C     -------------------------------
      density = char(0)
      tstat = tpopen(unit,1,HANDLE,TNOREW,density,TREAD,opmesg,
     + tapeid)
      if (tstat .ne. TSUCC) then
         mstat = -1
      else 
         mstat = 0
      endif

      return
      end

C     **************************************************************************
C
C     ANNDAT gets the annotation information needed for the printout.
C
C     **************************************************************************
      subroutine anndat(buffer)

	include 'las.fin'

     
      integer*4		i
C					loop index

      character*1	buffer(5322)
C					input buffer
      character*1	numtap
C					number of volumes in tape set
      character*1	intyp
C					interleaving type indicator
      character*1	npatrw(9)
C					nominal path row
      character*1	clatlo(17)
C					center latitude and longitude
      character*1	corapp
C					corrections applied
      character*1	nlatlo(17)
C					nominal latitude and longitude
      character*1	bpres(5)
C					spectral bands present
      character*1	imgsca
C					image scale
      character*1	resamp
C					resampling algorithm
      character*1	msstra
C					MSS transmission type
      character*1	suninf(14)
C					sun information
      character*1	edipce
C					contrast enhancement indicator
      character*1	edipas
C					atmospheric scatter compensation
C					indicator
      character*1	edipeh
C					edge enhancement indicator
      character*1	bndgai(5)
C					MSS sensor gain options
      character*1	tdate(3)
C					tape generation date

      integer*4		path
C					path
      integer*4         row
C					row
      integer*4         mission
C					Landsat mission number
      real*8		wrsll(2)
C					WRS format center latitude/longitude
      real*8		wrsls(2)
C					WRS format center line/sample
      real*8		angle
C					Image orientaion angle
      character*16	scenid
C					scene id
      character*2	projec
C					type of projection

      character*2	sentyp
C					sensor type
      character*2	direction
C					Direction of the pass
      character*9   edate
C					Date of exposure

      common/ddrfld/path, row, wrsll, wrsls, angle, edate, projec,
     +              scenid, sentyp, direction
      common/prtfld/numtap, intyp, npatrw, clatlo, corapp, nlatlo,
     +		    bpres, imgsca, resamp, msstra, suninf, mission,
     +		    edipce, edipas, edipeh, bndgai, tdate

C     get the date of exposure
C     ------------------------

      do 50 i = 1,8
         edate(i:i) = buffer(i+6)
50    continue
      edate(9:9) = char(0)

C     get the nominal path row
C     ------------------------
      do 100 i = 32,40
         npatrw(i-31) = buffer(i)
100   continue

C     get the center latitude and longitude
C     -------------------------------------
      do 200 i = 15,31
         clatlo(i-14) = buffer(i)
200   continue

C     get the corrections applied
C     ---------------------------
      corapp = buffer(82)

C     get the nominal latitude and longitude
C     -------------------------------------
      do 300 i = 41,57
         nlatlo(i-40) = buffer(i)
300   continue
      
      call wrscll(nlatlo,wrsll)

C     get the image scale
C     -------------------
      imgsca = buffer(83)

C     get the type of projection
C     --------------------------

      projec(1:1) = buffer(84)
      projec(2:2) = char(0)

C     get the resampling algorithm
C     ----------------------------
      resamp = buffer(86)

C     get the MSS transmission type
C     -----------------------------
      msstra = buffer(92)

C     get the scene id
C     ----------------

      do 500 i = 1,15
         scenid(i:i) = buffer(i+106)
500   continue
      scenid(16:16) = char(0)

C     get the sun information
C     -----------------------
      do 600 i = 68,81
         suninf(i-67) = buffer(i)
600   continue

      return
      end

C     **************************************************************************
C
C     HDDAT gets the header information needed for the printout.
C
C     **************************************************************************
      subroutine hddat( buffer,fmss,second,inumb)

	include 'las.fin'

	include 'sysdef.fin'

      real*4		pi
C					value of PI
      integer*4		inumb
C					array index
      integer*4		iptr
C					array index
      integer*4		i
C					loop index
      integer*2		iword
C					indication of data
C					present by band
      integer*2     	imask
C					mask to determine
C					the image bands present
      integer*2		val
C					return value from pxbtan 
      logical		fmss
C					flag that a MSS tape is
C					being processed
      logical		second
C					flag if the routine has
C				        already been called at
C					least once

      character*1       buffer(3596)
C					input buffer
      character*1	numtap
C					number of volumes in tape set
      character*1	intyp
C		    			interleaving type indicator
      character*1	npatrw(9)
C					nominal path row
      character*1	clatlo(17)
C					center latitude and longitude
      character*1	corapp
C					corrections applied
      character*1	nlatlo(17)
C					nominal latitude and longitude
      character*1	bpres(5)
C					spectral bands present
      character*1	imgsca
C					image scale
      character*1	resamp
C					resampling algorithm
      character*1	msstra
C					MSS transmission type
      character*1	suninf(14)
C					sun information
      character*1	edipce
C					contrast enhancement indicator
      character*1	edipas
C					atmospheric scatter compensation
C					indicator
      character*1	edipeh
C					edge enhancement indicator
      character*1	bndgai(5)
C					MSS sensor gain options
      character*1	tdate(3)
C					tape generation date

      integer*4		path
C					path
      integer*4         row
C					row
      real*8		wrsll(2)
C					WRS format center latitude/longitude
      real*8		wrsls(2)
C					WRS format center line/sample
      real*8		angle
C					Image orientaion angle
      integer*4		mission
C					Landsat mission number
      character*9	edate
C					Date of exposure
      character*16	scenid
C					scene id
      character*2	projec
C					type of projection
      character*2	sentyp
C					sensor type
      character*2	direction
C					Direction of the pass

      common/ddrfld/path, row, wrsll, wrsls, angle, edate, projec,
     +              scenid, sentyp, direction
      common/prtfld/numtap, intyp, npatrw, clatlo, corapp, nlatlo,
     +		    bpres, imgsca, resamp, msstra, suninf, mission,
     +		    edipce, edipas, edipeh, bndgai, tdate

      pi = 3.141592653589

C     get the spectral bands present
C     ------------------------------
C      (this section is done for subsequent bands of BSQ tapes)
C        --------------------------------------------------------
      if (second) then
         inumb = inumb + 1
         bpres(inumb) = buffer(136)
         return
      endif

C      (this section is done only for the first band of BSQ or for BIL tapes)
C      ----------------------------------------------------------------------
      do 100 i=1,5
         bpres(i)= ' '
100   continue

      iptr = 0
      inumb = 1

      if ( ichar(intyp) .eq. 0 ) then

         bpres(inumb) = buffer(136)
      else

	 iword = ichar(buffer(3586))

         imask = 16
         do 200 i=1,5
	    call pxbtan(iword,imask,val,1,2)
	    if (val.ne.0) then
               iptr = iptr + 1

               bpres(iptr) = char(i)

            endif
            imask = imask/2
200      continue
      endif

C     get the flags indicating whether or not EDIPS perfomed contrast 
C     enhancement, atmospheric scatter compensation, and edge enhancement
C     -------------------------------------------------------------------
      if (fmss) then
         edipce = buffer(3583)
         edipas = buffer(3584)
         edipeh = buffer(3585)
      else
         edipce = buffer(3575)
         edipas = buffer(3576)
         edipeh = buffer(3577)
      endif

C     get the MSS sensor gain options
C     -------------------------------
      do 300 i = 3587,3591
         bndgai(i-3586) = buffer(i)
300   continue

C     get the tape generation date
C     ----------------------------
      do 400 i=27,29
         tdate(i-26) = buffer(i)
400   continue

	wrsls(1) = 1492
	wrsls(2) = 1774

        read(buffer(8),'(I1)') mission

      return
      end

C     **************************************************************************
C
C     PRTDAT creates the print file. 
C
C     **************************************************************************
      subroutine prtdat

	include 'las.fin'

      integer*4		lplun
C					logical unit number for print file
      integer*4		i
C				        loop index
      integer*4		jj
C					loop index
      integer*4		temp(3)
C					temporary array
      character*8       hilow
C					high/low gain indicator

      character*1	numtap
C					number of volumes in tape set
      character*1	intyp
C					interleaving type indicator
      character*1	npatrw(9)
C					nominal path row
      character*1	clatlo(17)
C					center latitude and longitude
      character*1	corapp
C					corrections applied
      character*1	nlatlo(17)
C					nominal latitude and longitude
      character*1	bpres(5)
C					spectral bands present
      character*1	imgsca
C					image scale
      character*1	resamp
C					resampling algorithm
      character*1	msstra
C					MSS transmission type
      character*1	suninf(14)
C					sun information
      character*1	edipce
C					contrast enhancement indicator
      character*1	edipas
C					atmospheric scatter compensation
C					indicator
      character*1	edipeh
C					edge enhancement indicator
      character*1	bndgai(5)
C					MSS sensor gain options
      character*1	tdate(3)
C					tape generation date

      integer*4		path
C					path
      integer*4         row
C					row
      integer*4         mission
C					Landsat mission number
      real*8		wrsll(2)
C					WRS format center latitude/longitude
      real*8		wrsls(2)
C					WRS format center line/sample
      real*8		angle
C					Image orientaion angle
      character*9	edate
C					Date of exposure
      character*16	scenid
C					scene id
      character*2	projec
C					type of projection

      character*2	sentyp
C					sensor type
      character*2	direction
C					Direction of the pass

      common/ddrfld/path, row, wrsll, wrsls, angle, edate, projec,
     +              scenid, sentyp, direction
      common/prtfld/numtap, intyp, npatrw, clatlo, corapp, nlatlo,
     +		    bpres, imgsca, resamp, msstra, suninf,mission,
     +		    edipce, edipas, edipeh, bndgai, tdate

C     print the selected information
C     ------------------------------
      lplun = 25
      open(unit=lplun,status='UNKNOWN',file='edipsin.pr')

      if (sentyp(1:1) .eq. 'M' ) then
         write(lplun,'(21X,'' SENSOR TYPE: MSS'')')
      elseif( sentyp(1:1) .eq. 'R' ) then
         write(lplun,'(21X,'' SENSOR TYPE: RBV'')')
      else
         write(lplun,'(21X,'' SENSOR TYPE: NO VALUE'')')
      endif

      write(lplun,'(10X,'' NUMBER OF TAPES IN SET: '',A1)') numtap

      if( ichar(intyp) .eq. 0 ) then

         write(lplun,'(15X,'' INTERLEAVING TYPE: BSQ'')')

      elseif (ichar(intyp) .eq. 255 ) then

         write(lplun,'(15X,'' INTERLEAVING TYPE: BIL'')')
      else
         write(lplun,'(15X,'' INTERLEAVING TYPE: NO VALUE'')')
      endif

      write(lplun,'(10X,'' SPECTRAL BANDS PRESENT:'',5(1X,A1))')
     +   bpres

      temp(1) = ichar(tdate(1))
      temp(2) = ichar(tdate(2))
      temp(3) = ichar(tdate(3))

      write(lplun,'(13X,''TAPE GENERATION DATE: '',2(I2.2,1H-),I2.2)')
     +    temp

      write(lplun,'(/'' *** GENERAL INFORMATION ***''/)')

      write(lplun,'(28X,'' PATH: '',3A1/29X,'' ROW: '',3A1)')
     +   (npatrw(jj),jj=2,4),(npatrw(jj),jj=6,8)

      write(lplun,'(18X,'' CENTER LAT,LON: '',15A1)')
     +   (clatlo(jj),jj=3,17)

      write(lplun,'(17X,'' NOMINAL LAT,LON: '',15A1)')
     +   (nlatlo(jj),jj=3,17)

      if (msstra .eq. '1' ) then
         write(lplun,'(11X,'' MSS TRANSMISSION TYPE: LINEAR'')')
      elseif( msstra .eq. '2' ) then
         write(lplun,'(11X,'' MSS TRANSMISSION TYPE: COMPRESSED'')')
      else
         write(lplun,'(11X,'' MSS TRANSMISSION TYPE: NO VALUE'')')
      endif

      write(lplun,'(24X,'' SCENE ID: LANDSAT '',A1,1X,A15)')
     +    scenid(3:3),scenid

      write(lplun,'(17X,'' SUN INFORMATION: '',14A1)') suninf

      do 100 i=1,5
         if( bndgai(i) .eq. 'H' ) then
            hilow= 'HIGH    '
         elseif( bndgai(i) .eq. 'L' ) then
            hilow= 'LOW     '
         else
            hilow= 'NO VALUE'
         endif
         write(lplun,'(21X,'' GAIN BAND '',I1,1H:,1X,A8)')i,hilow
100   continue

      write(lplun,'(/'' *** DATA PROCESSING INFORMATION ***''/)')
      if( corapp .eq. 'U' ) then
         write(lplun,'(11X,'' CORRECTION(S) APPLIED: UNCORRECTED'')')
      elseif( corapp .eq. 'S' ) then
         write(lplun,'(11X,'' CORRECTION(S) APPLIED: SYSTEM'')')
      elseif( corapp .eq. 'G' ) then
         write(lplun,'(11X,'' CORRECTION(S) APPLIED: GEOMETRIC BASED''
     +   ,'' ON GEOMETRIC GCPS'')')
C Does not work on DG
C         write(lplun,'(11X,'' CORRECTION(S) APPLIED: GEOMETRIC BASED''
C     +   ,'' ON GEOMETRIC GCP''''S'')')
      elseif( corapp .eq. 'R' ) then
         write(lplun,'(11X,'' CORRECTION(S) APPLIED: GEOMETRIC BASED''
     +   ,'' ON RELATIVE GCPS'')')
C Does not work on DG
C         write(lplun,'(11X,'' CORRECTION(S) APPLIED: GEOMETRIC BASED''
C     +   ,'' ON RELATIVE GCP''''S'')')
      elseif (corapp.eq.'T') then
         write (lplun,'(11X,'' CORRECTION(S) APPLIED: TEMPORAL''
     +   ,'' REGISTRATION USING GEODETIC INFORMATION'')')
      else
         write(lplun,'(11X,'' CORRECTION(S) APPLIED: NO VALUE'')')
      endif
      if( imgsca .eq. '1' ) then
         write(lplun,'(21X,'' IMAGE SCALE: 185 KM X 185 KM'')')
      elseif( imgsca .eq. '2' ) then
         write(lplun,'(21X,'' IMAGE SCALE: 99 KM X 99 KM'')')
      elseif( imgsca .eq. '3' ) then
         write(lplun,'(21X,'' IMAGE SCALE: 185 KM X 170 KM'')')
      else
         write(lplun,'(21X,'' IMAGE SCALE: NO VALUE'')')
      endif

      if( projec(1:1) .eq. 'L' ) then
         write(lplun,'(22X,'' PROJECTION: LAMBERT PROJECTION'')')
      elseif( projec(1:1) .eq. 'P' ) then
         write(lplun,'(22X,'' PROJECTION: POLAR STEREOGRAPHIC '',
     .   ''PROJECTION'')')
      elseif( projec(1:1) .eq. 'S' ) then
         write(lplun,'(22X,'' PROJECTION: SPACE OBLIQUE MERCATOR '',
     .   ''PROJECTION'')')
      elseif( projec(1:1) .eq. 'U' ) then
        write(lplun,'(22X,'' PROJECTION: UNIVERSAL TRANSVERSE '',
     .   ''MERCATOR PROJECTION'')')
      elseif( projec(1:1) .eq. 'H' ) then
         write(lplun,'(22X,'' PROJECTION: HOTINE OBLIQUE MERCATOR '',
     .   ''PROJECTION'')')
      else
         write(lplun,'(22X,'' PROJECTION: NO VALUE'')')
      endif

      if( resamp .eq. 'C' ) then
         write(lplun,'(22X,'' RESAMPLING: CUBIC CONVOLUTION'')')
      elseif( resamp .eq. 'N' ) then
         write(lplun,'(22X,'' RESAMPLING: NEAREST NEIGHBOR'')')
      else
         write(lplun,'(22X,'' RESAMPLING: NO VALUE'')')
      endif

      if( ichar(edipce) .eq. 0 ) then

         write(lplun,'(12X,'' CONTRAST ENHANCEMENT: NO'')')

      elseif( ichar(edipce) .eq. 255 ) then

         write(lplun,'(12X,'' CONTRAST ENHANCEMENT: YES'')')
      else
         write(lplun,'(12X,'' CONTRAST ENHANCEMENT: NO VALUE'')')
      endif

      if( ichar(edipas) .eq. 0 ) then

         write(lplun,'('' ATMOSPHERIC SCATTER COMPENSATION: NO'')')

      elseif( ichar(edipas) .eq. 255 ) then

         write(lplun,'('' ATMOSPHERIC SCATTER COMPENSATION: YES'')')
      else
         write(lplun,'('' ATMOSPHERIC SCATTER COMPENSATION: '',
     +    ''NO VALUE'')')
      endif

      if( ichar(edipeh) .eq. 0 ) then

         write(lplun,'(16X,'' EDGE ENHANCEMENT: NO'')')

      elseif( ichar(edipeh) .eq. 255 ) then

         write(lplun,'(16X,'' EDGE ENHANCEMENT: YES'')')
      else
         write(lplun,'(16X,'' EDGE ENHANCEMENT: NO VALUE'')')
      endif

      close (unit=lplun)

      return
      end

C     **************************************************************************
C
C     ANCDT1 retrieves, decodes, and writes out information from
C     ancillary record #1 of uncorrected MSS tapes. 
C
C     **************************************************************************

      subroutine ancdt1(buffer,unitcr,unitmd,mnum)

C     ******************
C     * PN9050 version *
C     ******************

	include 'las.fin'

	include 'sysdef.fin'

      real*8		pixdis
C					nominal scale of input 
C					inter-pixel distance
      real*8		lindis
C					nominal scale of input
C					inter-line distance
      real*8		nomalt
C					nominal spacecraft altitude
      real*8		nomwid
C					nominal input image width
      real*8	   	mirror(4)
C					MSS mirror model coefficients
      real*8		maxang
C					MSS maximum mirror angle
      real*8		scskew
C					scan skew constant
      real*8		swpint
C					sweep interval
      real*8		swptim
C					sweep time
      real*8		mjaxis
C					major semi-axis of earth ellipsoid
      real*8		mnaxis
C					minor semi-axis of earth ellipsoid
      real*8		ecurv
C					earth curvature constant

      real*4            btb(5)
C					MSS band-to-band offsets
      real*4		sdelay(26)
C					MSS sampling delay constants

      integer*4         x
C					buffer index
      integer*4		i
C					loop counter
      integer*4		status
C					status indicator
      integer*4		nomlen
C					nominal number of pixels
      integer*4		unitcr
C					unit number for correction file
      integer*4		unitmd
C					unit number for modeling file
      integer*4		mnum
C					Landsat mission number
      character*1       buffer(3596)
C					input buffer
      character*(DDSYLN) cursys
      integer*4 outsys,getsys,pxsys,sysset

C     output mission number to systematic correction 
C     and platform modeling files
C     ----------------------------------------------
      write (unit=unitcr,fmt=*,iostat=status) mnum
      if (status .ne. 0) then
	 call errmsg('Error writing to correction file',
     +	             'edipsin-writerr',1)
      endif

      write (unit=unitmd,fmt=*,iostat=status) mnum
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get nominal line length
C     -----------------------
      call pxcopy(buffer(7),nomlen,1,4)
      write (unit=unitcr,fmt=*,iostat=status) nomlen
      if (status .ne. 0) then
	 call errmsg('Error writing to correction file',
     +	             'edipsin-writerr',1)
      endif

C     get nominal scale of input inter-pixel distance (in meters per pixel)
C     ---------------------------------------------------------------------
      call pxcopy(buffer(15),pixdis,1,8)
      status = getsys(cursys)
      status = sysset(cursys,outsys,SETN)
      status = pxsys(SYSIBMMVS,outsys,pixdis,EDBLE,1,0,0,SYSET)
C      call tcomp(pixdis,8)
      write (unit=unitcr,fmt='(E25.15)',iostat=status) pixdis
      if (status.ne.0) then
	 call errmsg('Error writing to correction file',
     +	             'edipsin-writerr',1)
      endif

C     get nominal scale of input inter-line distance (in meters per pixel)
C     --------------------------------------------------------------------
      call pxcopy(buffer(23),lindis,1,8)
      status = pxsys(SYSIBMMVS,outsys,lindis,EDBLE,1,0,0,SYSET)
C      call tcomp(lindis,8)
      write (unit=unitcr,fmt='(E25.15)',iostat=status) lindis
      if (status.ne.0) then
         call errmsg('Error writing to correction file',
     +	             'edipsin-writerr',1)
      endif

C     get nominal spacecraft altitude in meters
C     -----------------------------------------
      call pxcopy(buffer(55),nomalt,1,8)
      status = pxsys(SYSIBMMVS,outsys,nomalt,EDBLE,1,0,0,SYSET)
C      call tcomp(nomalt,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) nomalt
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif
      write (unit=unitcr,fmt='(E25.15)',iostat=status) nomalt
      if (status .ne. 0) then
         call errmsg('Error writing to correction file',
     +               'edipsin-writerr',1)
      endif		

C     get nominal input image width in meters
C     ---------------------------------------
      call pxcopy(buffer(63),nomwid,1,8)
      status = pxsys(SYSIBMMVS,outsys,nomwid,EDBLE,1,0,0,SYSET)
C      call tcomp(nomwid,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) nomwid
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif
      write (unit=unitcr,fmt='(E25.15)',iostat=status) nomwid
      if (status.ne.0) then
         call errmsg('Error writing to correction file',
     +	             'edipsin-writerr',1)
      endif

C     get mirror scan model coefficients
C     ----------------------------------
      x = 71
         do 10 i = 1,4
            call pxcopy(buffer(x),mirror(i),1,8)
            status = pxsys(SYSIBMMVS,outsys,mirror(i),EDBLE,1,0,0,SYSET)
C            call tcomp(mirror(i),8)
            write (unit=unitcr,fmt='(E25.15)',iostat=status) mirror(i)
            if (status .ne. 0) then
               call errmsg('Error writing to correction file',
     +                     'edipsin-writerr',1)
            endif
	    x = x + 8
10       continue

C     get MSS maximum mirror angle 
C     (in radians)
C     ----------------------------
      call pxcopy(buffer(103),maxang,1,8)
      status = pxsys(SYSIBMMVS,outsys,maxang,EDBLE,1,0,0,SYSET)
C      call tcomp(maxang,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) maxang
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get scan skew constant
C     ----------------------
      call pxcopy(buffer(111),scskew,1,8)
      status = pxsys(SYSIBMMVS,outsys,scskew,EDBLE,1,0,0,SYSET)
C      call tcomp(scskew,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) scskew
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get time betweeen successive MSS mirror sweeps 
C     (in seconds)
C     ----------------------------------------------
      call pxcopy(buffer(119),swpint,1,8)
      status = pxsys(SYSIBMMVS,outsys,swpint,EDBLE,1,0,0,SYSET)
C      call tcomp(swpint,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) swpint
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get time for the active portion of an MSS mirror sweep
C     (in seconds)
C     ------------------------------------------------------
      call pxcopy(buffer(127),swptim,1,8)
      status = pxsys(SYSIBMMVS,outsys,swptim,EDBLE,1,0,0,SYSET)
C      call tcomp(swptim,8)
      write (unit=unitmd, fmt='(E25.15)',iostat=status) swptim
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get major semi-axis of Earth ellipsoid
C     --------------------------------------
      call pxcopy(buffer(135),mjaxis,1,8)
      status = pxsys(SYSIBMMVS,outsys,mjaxis,EDBLE,1,0,0,SYSET)
C      call tcomp(mjaxis,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) mjaxis
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get minor semi-axis of Earth ellipsoid
C     --------------------------------------
      call pxcopy(buffer(143),mnaxis,1,8)
      status = pxsys(SYSIBMMVS,outsys,mnaxis,EDBLE,1,0,0,SYSET)
C      call tcomp(mnaxis,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) mnaxis
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get Earth curvature constant
C     ----------------------------
      call pxcopy(buffer(151),ecurv,1,8)
      status = pxsys(SYSIBMMVS,outsys,ecurv,EDBLE,1,0,0,SYSET)
C      call tcomp(ecurv,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) ecurv
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get sampling delay 
C     ------------------
         x = 159
         do 20 i = 1,26
	    call pxcopy(buffer(x),sdelay(i),1,4)
            status = pxsys(SYSIBMMVS,outsys,sdelay(i),EREAL,1,0,0,SYSET)
C            call tcomp(sdelay(i),4)
            write (unit=unitcr,fmt='(F22.12)',iostat=status) sdelay(i)
            if (status .ne. 0) then
               call errmsg('Error writing to correction file',
     +                     'edipsin-writerr',1)
            endif
	    x = x + 4
20       continue

C     get band-to-band offsets
C     ------------------------
         x = 263
         do 30 i = 1,5
	   call pxcopy(buffer(x),btb(i),1,4)
C           call tcomp(btb(i),4)
           status = pxsys(SYSIBMMVS,outsys,btb(i),EREAL,1,0,0,SYSET)
           write (unit=unitcr,fmt='(F22.12)',iostat=status) btb(i)
           if (status .ne. 0) then
      	       call errmsg('Error writing to correction file',
     +                     'edipsin-writerr',1)
           endif
	   x = x+4
30       continue

C       ***************
C	* VAX version *
C       ***************

	return
	end

C     **************************************************************************
C
C     ANCDT2 retrieves, decodes, and writes out information from
C     ancillary record #2 of uncorrected MSS tapes. 
C
C     **************************************************************************

      subroutine ancdt2(buffer,unitcr,unitmd)

C     ******************
C     * PN9050 version *
C     ******************

	include 'las.fin'

	include 'sysdef.fin'

      integer*4		path
C					path
      integer*4         row
C					row
      real*8		wrsll(2)
C					WRS format center latitude/longitude
      real*8		wrsls(2)
C					WRS format center line/sample
      real*8		angle
C					Image orientaion angle
      character*2	sentyp
C					sensor type
      character*2	direction
C					direction of pass
      character*16	scenid
C					scene id
      character*2	projec
C					type of projection
      character*9	edate
C					date of exposure
      real*8		nlat
C					nadir latitude
      real*8		nlong
C					nadir longitude
      real*8		nadir(3) 
C					nadir coordinates
      real*8		hdang
C					spacecraft heading angle
      real*8		nline
C					input image line coordinate of nadir
      real*8		npixel
C					input image pixel coordinate
C					of nadir
      real*8		vlcity
C					normalize spacecraft velocity
C					at nadir
      real*8	       	aacoef(14)
C					attitude/altitude coefficients

      real*4  		earth
C					earth rotation parameter
      real*4		eint
C					time interval between successive
C					sets of ephemeris entries
      real*4		aint
C					time interval between successive
C					sets of attitude entries
      real*4		eentry(6,16)
C					ephemeris entries
      real*4		aentry(3,60)
C					attitude entries

      integer*4   	unitcr
C					unit number of correction file
      integer*4		unitmd 
C					unit number of modeling file
      integer*4		eset
C					number of sets of ephemeris entries
      integer*4		aset
C					number of sets of attitude entries
      integer*4		edq(16)
C					array of ephemeris data quality
C					indicators
      integer*4		adq(60)
C					array of attitude data quality
C					indicators
      integer*4		status
C					status indicator
      integer*4		x
C					buffer index
      integer*4		i, j
C					loop counters

      character*14      ctime
C					spacecraft time of frame center
      character*14	etime
C					time of first set of ephemeris entries
      character*14      atime
C					time of first set of attitude entries

      character*3       pathtp
C					temporary used for decoding path
      character*3	rowtmp
C					temporary used for decoding row
      character*1  	buffer(3596)
C					input buffer
      character*1	pathbf(3)
C					buffer containing bytes for path	
      character*1	rowbuf(3)
C					buffer containing bytes for row
      character*1	cbuf(14)
C					buffer containing bytes for ctime
      character*1	ebuf(14)
C					buffer containing bytes for etime
      character*1	abuf(14)
C					buffer containing bytes for atime
      character*(DDSYLN) cursys
      integer*4 outsys,getsys,pxsys,sysset

      common/ddrfld/path, row, wrsll, wrsls, angle, edate, projec,
     +              scenid, sentyp, direction

      equivalence(pathtp,pathbf(1))
      equivalence (rowtmp,rowbuf(1))
      equivalence (ctime,cbuf(1))
      equivalence (etime,ebuf(1))
      equivalence (atime,abuf(1))

C     get WRS frame number (path)
C     ---------------------------
      call pxcopy(buffer(8),pathbf(1),1,3)
      read (pathtp,'(I3)') path
      write (unit=unitmd,fmt=*,iostat=status) path
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get WRS orbit number (row)
C     --------------------------
      call pxcopy(buffer(12),rowbuf(1),1,3)
      read (rowtmp,'(I3)') row
      write (unit=unitmd,fmt=*,iostat=status) row
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get WRS format center latitude
C     (in radians)
C     ------------------------------
      call pxcopy(buffer(15),wrsll(1),1,8)
      status = getsys(cursys)
      status = sysset(cursys,outsys,SETN)
      status = pxsys(SYSIBMMVS,outsys,wrsll(1),EDBLE,1,0,0,SYSET)
C      call tcomp(wrsll(1),8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) wrsll(1)
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get WRS format center longitude
C     (in radians)
C     -------------------------------
      call pxcopy(buffer(23),wrsll(2),1,8)
      status = pxsys(SYSIBMMVS,outsys,wrsll(2),EDBLE,1,0,0,SYSET)
C      call tcomp(wrsll(2),8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) wrsll(2)
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get spacecraft time of frame center (in ASCII)
C     ----------------------------------------------
      call pxcopy(buffer(31),cbuf(1),1,14)
      write (unit=unitmd,fmt=*,iostat=status) ctime
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get Nadir latitude (in radians)
C     -------------------------------
      call pxcopy(buffer(55),nlat,1,8)
      status = pxsys(SYSIBMMVS,outsys,nlat,EDBLE,1,0,0,SYSET)
C      call tcomp(nlat,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) nlat
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif
      write (unit=unitcr,fmt='(E25.15)',iostat=status) nlat
      if (status .ne. 0) then 
         call errmsg('Error writing to correction file',
     +	     	     'edipsin-writerr',1)
      endif

C     get Nadir longitude (in radians)
C     --------------------------------
      call pxcopy(buffer(63),nlong,1,8)
      status = pxsys(SYSIBMMVS,outsys,nlong,EDBLE,1,0,0,SYSET)
C      call tcomp(nlong,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) nlong
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get Nadir Earth-centered coordinates (in meters)
C     ------------------------------------------------
      x = 71
      do 10 i = 1,3
         call pxcopy(buffer(x),nadir(i),1,8)
         status = pxsys(SYSIBMMVS,outsys,nadir(i),EDBLE,1,0,0,SYSET)
C         call tcomp(nadir(i),8)
         write (unit=unitmd,fmt='(E25.15)',iostat=status) nadir(i)
         if (status .ne. 0) then
            call errmsg('Error writing to modeling file',
     +                  'edipsin-writerr',1)
         endif
         x = x + 8
10    continue

C     get spacecraft heading angle at nadir (beta) in radians
C     -------------------------------------------------------
      call pxcopy(buffer(95),hdang,1,8)
      status = pxsys(SYSIBMMVS,outsys,hdang,EDBLE,1,0,0,SYSET)
C      call tcomp(hdang,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) hdang
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get input image line coordinate of nadir 
C     in partially-processed output
C     ----------------------------------------
      call pxcopy(buffer(103),nline,1,8)
      status = pxsys(SYSIBMMVS,outsys,nline,EDBLE,1,0,0,SYSET)
C      call tcomp(nline,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) nline
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get image pixel coordinate of nadir 
C     in partially-processed output
C     -----------------------------------
      call pxcopy(buffer(111),npixel,1,8)
      status = pxsys(SYSIBMMVS,outsys,npixel,EDBLE,1,0,0,SYSET)
C      call tcomp(npixel,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) npixel
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get Earth rotation velocity at nadir (in meters per second)
C     -----------------------------------------------------------
      call pxcopy(buffer(119),vlcity,1,8)
      status = pxsys(SYSIBMMVS,outsys,vlcity,EDBLE,1,0,0,SYSET)
C      call tcomp(vlcity,8)
      write (unit=unitmd,fmt='(E25.15)',iostat=status) vlcity
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get the earth rotation parameter (image skew)
C     ---------------------------------------------
      call pxcopy(buffer(135),earth,1,4)
      status = pxsys(SYSIBMMVS,outsys,earth,EREAL,1,0,0,SYSET)
C      call tcomp(earth,4)
      write (unit=unitcr,fmt='(F22.12)',iostat=status) earth
      if (status .ne. 0) then
	 call errmsg('Error writing to correction file',
     +	             'edipsin-writerr',1)
      endif
    
C     get attitude/altitude coefficients
C     ----------------------------------
      x = 139
      do 50 i = 1,14
         call pxcopy(buffer(x),aacoef(i),1,8)
         status = pxsys(SYSIBMMVS,outsys,aacoef(i),EDBLE,1,0,0,SYSET)
C         call tcomp(aacoef(i),8)
         write (unit=unitmd,fmt='(E25.15)',iostat=status) aacoef(i)
         if (status .ne. 0) then
            call errmsg('Error writing to modeling file',
     +                  'edipsin-writerr',1)
         endif
         x = x + 8
50    continue

C     get time of the first set of ephemeris entries (in ASCII)
C     ---------------------------------------------------------
      call pxcopy(buffer(667),ebuf(1),1,14)
      write (unit=unitmd,fmt=*,iostat=status) etime
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get time interval between successive sets of
C     ephemeris entries (in seconds)
C     ---------------------------------------------
      call pxcopy(buffer(681),eint,1,4)
      status = pxsys(SYSIBMMVS,outsys,eint,EREAL,1,0,0,SYSET)
C      call tcomp(eint,4)
      write (unit=unitmd,fmt='(F22.12)',iostat=status) eint
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get number of sets of ephemeris entries
C     ---------------------------------------
      call pxcopy(buffer(685),eset,1,4)
      write (unit=unitmd,fmt=*,iostat=status) eset
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get sets of ephemeris entries
C     -----------------------------
      x = 689
      do 75 i = 1,16
         do 70 j = 1,6
	   call pxcopy(buffer(x),eentry(j,i),1,4)
           status = pxsys(SYSIBMMVS,outsys,
     +                    eentry(j,i),EREAL,1,0,0,SYSET)
C   	   call tcomp(eentry(j,i),4)
 	   write (unit=unitmd,fmt='(F22.12)',iostat=status) eentry(j,i)
           if (status .ne. 0) then
   	      call errmsg('Error writing to modeling file',
     +         'edipsin-writerr',1)
           endif
           x = x + 4
70       continue
       
	 call pxcopy(buffer(x),edq(i),1,4)
	 write (unit=unitmd,fmt=*,iostat=status) edq(i) 
         if (status .ne. 0) then
  	    call errmsg('Error writing to modeling file',
     +                  'edipsin-writerr',1)
         endif
	 x = x + 4

75    continue

C     get time of first set of attitude entries (in ASCII)
C     ----------------------------------------------------
      call pxcopy(buffer(1137),abuf(1),1,14)
      write (unit=unitmd,fmt=*,iostat=status) atime
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get time interval between successive sets of
C     attitude entries (in seconds)
C     --------------------------------------------
      call pxcopy(buffer(1151),aint,1,4)
      status = pxsys(SYSIBMMVS,outsys,aint,EREAL,1,0,0,SYSET)
C      call tcomp(aint,4)
      write (unit=unitmd,fmt='(F22.12)',iostat=status) aint
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get number of sets of attitude entries
C     --------------------------------------
      call pxcopy(buffer(1155),aset,1,4)
      write (unit=unitmd,fmt=*,iostat=status) aset
      if (status .ne. 0) then
	 call errmsg('Error writing to modeling file',
     +	             'edipsin-writerr',1)
      endif

C     get sets of attitude entries
C     ----------------------------
      x = 1159
        do 85 i = 1,60
           do 80 j = 1,3
             call pxcopy(buffer(x),aentry(j,i),1,4)
             status = pxsys(SYSIBMMVS,outsys,aentry(j,i),
     +                      EREAL,1,0,0,SYSET)
C	     call tcomp(aentry(j,i),4)
	     write(unit=unitmd,fmt='(F22.12)',iostat=status) aentry(j,i)
             if (status .ne. 0) then
	 	call errmsg('Error writing to modeling file',
     +                      'edipsin-writerr',1)
              endif
	      x = x + 4
80         continue
 
	   call pxcopy(buffer(x),adq(i),1,4)
	   write (unit=unitmd,fmt=*,iostat=status) adq(i)
           if (status .ne. 0) then
  	      call errmsg('Error writing to modeling file',
     +                    'edipsin-writerr',1)
           endif
	   x = x + 4

85    continue

	return
	end
