/******************************************************************************
FUNCTION:	UPDATE_DDR

PURPOSE:	Updates the DDR information for an image


PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	04/04/91	add & to ddr.zone_code in call to
				utmzon() and utmprm().
B. Ailts	09/91		Calculated and updated the corner points for
				the DDR.  Some of this data is an approximation
				because not all the projection information
				needed can be pulled from the header
				records on the tape.  The approximate corner
				coordinates are close enough for a rough
				transformation in the geometric registration
				process
D. Etrheim	10/92		Modfied the calcualation of corner coordinates 
				to use a rotation angle (radrang).  This gives
				a better aproximation of corners.
D. Etrheim	11/92           Corrected problem with capture time
D. Etrheim	11/96		Negate the UTM zone code before the center
				point is converted to projection coords.
A. Weissenberger06/01           Changed default ddr.datum_code from 0 to 4.
                                (Clark 1866 to International 1909)
                                Added DATUMDEF struct so that c_getdatum can be
                                used to retrieve the semi major and semi minor
                                axis.
                                
ALGORITHM 

ALGORITHM REFERENCES:
Dan Steinwand helped fiquire out close approximations for some of the projectio
parameters.
Charles Wivell helped to figure out the angle to true north for the Polar
Stereographic projection
The parameters for Universal Polar stereographic are taken from the following
******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"	/* LAS include */
#include "proj.h"
#include "geompak.h"
#include "tm.h"
#include "protos.h"

/*  Macro for getting a substring */
#define substr(in,out,num) strncpy((out),(in),(num)); out[(num)] = '\0'

int update_ddr
(
    char *hname,		/* Host file name			*/
    long window[],		/* Input window				*/
    long *totbnd,		/* Total number of bands to process	*/
    long *linc, 		/* Line increment factor		*/
    long *sinc,	        	/* Sample increment factor		*/
    char *stype,		/* sensor type (MSS or RBV)		*/
    char *direction,	        /* Direction of satellite "D" or "A"	*/
    long *path,		        /* WRS path				*/
    long *row,		        /* WRS row				*/
    double wrsll[],             /* WRS center latitude/longitude	*/
    double wrsls[],		/* WRS center line/sample		*/
    char *edate,		/* Date of exposure			*/
    char *projec,		/* Projection type			*/
    char *scene,		/* Scene id (contains time of exposure) */
    long *ddrvflag		/* DDR validity flag			*/
)
{
    struct DDR ddr;		/* DDR structure			*/
    struct BDDR bddr[5];        /* Band DDR structure			*/
    struct DATUMDEF datum;      /* DATUM structure                      */
  
    double wrspj[2];	        /* WRS projection coordinates		*/
    double inparm[COEFCT];	/* Input projection parameters		*/
    double lp;		        /* Line value after coordinate to be
                                   converted is subtracted from center
                                   line and multiplied by pixel size.   */
  
    double sp;                  /* Sample value after center sample is
                                   subtracted from sample to be converted
                                   and multiplied by pixel size         */
    double xp, yp;		/* Intermediate value after translation
                                   is applied                           */
 
    double radrang;		/* rotation angle in radians	       	*/
    double radiang;		/* inclination angle in radians	       	*/
    double incang = 0.0;        /* inclination angle in degrees		*/
  
    long i;			/* Looping variable			*/
    long band;		        /* Band looping variable		*/
    long status;                /* Status variable			*/
    long zone;		        /* Zone code				*/
    long fatal = -1;	        /* Fatal error code			*/
    long inproj = GEO;	        /* Input lat/long projection		*/
    long inunit = DEGREE;	/* Input Unit code			*/
    long inzone = 62;
    long nonfatal = 1;
    long outunit = METER;       /* Output Unit code			*/
    long indatum = 0;	        /* Datum code				*/
    long prtprm[2];		/* Print flag options			*/
  
    short projsupp = TRUE;
  
    char satid[2];                      /* Satellite id			*/
    char day[3];		        /* Day of exposure		*/
    char month[4];                      /* Month of exposure		*/
    char year[3];		        /* Year of exposure		*/
    char hrmn[6];		        /* Hour and minute of exposure 	*/
    char osec[2];		        /* Seconds of exposure	       	*/ 
    char tmpstr[6];
    char errtxt[ERRLEN];
    double a1,a2;

    /* Strip the satellite id number from the scene id */
    substr(&scene[2],satid,1);
    
    /* Strip the time of exposure from the scene id
       The scene id E-21619-16183-4 did not work for the old way.
       Unable to determine at this point if tapes exists that 
       would work here, and if so, what field to key on to know
       the difference between the two formats.  This check should
       work for now until more is known about the tape formats. */
    /* Old way.   
       substr(&scene[6],hrmn,4)
       substr(&scene[10],osec,1); */
  
    substr(&scene[6],tmpstr,5);
    if (strchr(tmpstr,'-') != NULL)
    {
        substr(&scene[6],hrmn,4);
        substr(&scene[10],osec,1);
    }
    else
    {
        substr(&scene[8],hrmn,4);
        substr(&scene[12],osec,1);
    }
    
    /* Get the DDR information from the image file
       ----------------------------------------------*/
    status = c_getddr(hname,&ddr);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);
    
    /* Initialize the validity flags
       --------------------------------*/
    for (i = 0; i < 8; i++)
        ddr.valid[i] = VALID;
  
    /* Update the values that are known
     -----------------------------------*/
    ddr.master_line   = window[0];
    ddr.master_sample = window[1];
    ddr.line_inc   = (double)*linc;
    ddr.sample_inc = (double)*sinc;

    /* Set the datum code to international 1909 as default */
    ddr.datum_code = 4;
    datum.datumnum = ddr.datum_code;

    /* Load the datum array with the correct values */
    status = c_getdatum( &datum );
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);

    if (strcmp(projec,"U") == 0)
    {     
        utmzon(&wrsll[1],&ddr.zone_code);
        utmprm(&ddr.zone_code,&wrsll[0],ddr.proj_coef);
        ddr.proj_code = UTM;
    }
    else if (strcmp(projec,"H") == 0)
    {
        homzon(row,&zone);
        status = homprm(path,&zone,ddr.proj_coef);
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);
        ddr.zone_code = 62;
        ddr.proj_code = HOM;

        /* Set the major and minor axis */
        ddr.proj_coef[0] = datum.smajor;
        ddr.proj_coef[1] = datum.sminor;
    }
    else if (strcmp(projec,"S") == 0)
    {
        somprm(path,ddr.proj_coef);
        ddr.zone_code = 62;
        ddr.proj_code = SOM;

        /* Set the major and minor axis, however, somprm already sets both
           of these variables */

        ddr.proj_coef[0] = datum.smajor;
        ddr.proj_coef[1] = datum.sminor;     
    }
    else if (strcmp(projec,"P") == 0)
    {
        /* The projection parameters below were pulled from page 157 of Map 
        Projectins -- A working manual by John Sydner
        ________________________________________________________________*/
        ddr.zone_code = 62;
        ddr.proj_code = PS;
        for (i = 0;i < COEFCT;i++)
        {
            ddr.proj_coef[i] = 0.0;
        }

        /* Set the major and minor axis */
        ddr.proj_coef[0] = datum.smajor;
        ddr.proj_coef[1] = datum.sminor;
      
        /* if the latitude is above the 84 degrees N or below 80 degrees S then
           use the parameters for the Universal Polar Stereographic
           ------------------------------------------------------------------*/
        if ((wrsll[0] >= 84.0) || (wrsll[0] <= -80.0))
        {
            /*  Check to see which hemispere the input image is located
                -----------------------------------------------------------*/
            if (wrsll[0] < 0.0)
                ddr.proj_coef[5] = -81006052.3;
            else
                ddr.proj_coef[5] = 81006052.3;
            ddr.proj_coef[6] = 2000000.0;
            ddr.proj_coef[7] = 2000000.0;
        }
        else
        {
            /*  Check to see which hemispere the input image is located
                -----------------------------------------------------------*/
            ddr.proj_coef[4] = 0.0;
            if (wrsll[0] < 0.0)
                ddr.proj_coef[5] = -81006052.3;
            else
                ddr.proj_coef[5] = 81006052.3;
            ddr.proj_coef[6] = 2000000.0;
            ddr.proj_coef[7] = 2000000.0;
        }
        
        /*  Set the inclination angle based on the satellite #
            Angle is set to approximately 98 degrees for the satellites that
            are not known until the inclination angle of all the satellites
            can be calculated. The inclination angle is used to calculate the
            angle of rotation of the image to true North
            ----------------------------------------------------------------*/
        if (satid[0] == '1')
            incang = 99.906;
        else if (satid[0] == '2')
            incang = 99.210;
        else if (satid[0] == '3')
            incang = 99.117;
        else if (satid[0] == '4')
            incang = 98.22;
        else
        {
            sprintf(errtxt,"Inclination angle of Landsat %s is not known",
                    satid);
            c_errmsg(errtxt,"edipsin-incang",&nonfatal);
            c_errmsg("Inclination angle is set to 98 degrees",
                     "edipsin-incang",&nonfatal);
            incang = 98.00;
        }
    }
    else
    {
        sprintf(errtxt,"The projection %s is not supported",projec);
        c_errmsg(errtxt,"edipsin-projtype",&nonfatal);
        c_errmsg("DDR projection information will be invalid",
                 "edipsin-invalid",&nonfatal);
        projsupp = FALSE;
        
        /* Initialize the validity flags
           --------------------------------*/
        ddr.valid[0] = *ddrvflag;
        ddr.valid[1] = *ddrvflag;
        ddr.valid[2] = *ddrvflag;
        ddr.valid[3] = *ddrvflag;
        ddr.valid[4] = VALID;
        ddr.valid[5] = VALID;
        ddr.valid[6] = *ddrvflag;
        ddr.valid[7] = VALID;
    }
    
    strcpy(ddr.proj_units,"meters");
    ddr.pdist_y = 57.0;
    ddr.pdist_x = 57.0;
    
    if ((wrsll[0] == 0.0) && (wrsll[1] == 0.0))
        projsupp = FALSE;
    
    if (projsupp)
    {
        /* Update the corner coordinates
           First calculate the center projection coordinates and then calculate
           each of the corner coordinates based on the center coordinate and
           the angle that the projection coordinate system is rotated from
           the image.
           -----------------------------------------------------------------*/
        for (i = 0;i < COEFCT;i++)
            inparm[i] = 0.0;
        prtprm[0] = 0;
        prtprm[1] = 0;
        
        /* Reset zone code for images below equator*/
        if (ddr.proj_code == UTM)
            if (wrsll[0] < 0.0)
                ddr.zone_code *= -1;
        
        if ( c_transinit
             (&inproj,&inunit,&inzone,&indatum,inparm,&ddr.proj_code,
              &outunit,&ddr.zone_code,&ddr.datum_code,ddr.proj_coef,
              prtprm,"") != E_SUCC )
            c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);
        
        if ( c_trans(&inproj,&inunit,&ddr.proj_code,&outunit,
                     &wrsll[1],&wrsll[0],&wrspj[1],
                     &wrspj[0]) != E_SUCC)
            c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);
        
        /* If the projection is Polar Stereographic then the rotation angle is
           the longitude of scene center minus the calculated rotation angle.
           This is done because the Polar stereographic projection's origin is
           Greenwich.
           ------------------------------------------------------------------*/
        if (strcmp(projec,"P") == 0)
        {
            /* Calculate the rotation angle based on the inclination angle and
               the latitude of the scene.  
              --------------------------------------------------------------*/
            radiang = ((180.0 - incang) / 180.0) * PI;
            radrang = asin(cos(radiang) / cos((wrsll[0] / 180.0) * PI));
            radrang = (double) ((wrsll[1] / 180.0) * PI) - radrang;
        }
        else
        {
            /*  The angle stored in the header record is not useful for the
                Polar Stereographic projection but seems to be useful for the
                SOM projection.  If the angle information is correct for HOM
                and UTM is yet to be determined but is assumed to be correct
                ------------------------------------------------------------*/
            
            /* The angle for UTM doesn't seem to be correct, and it doesn't
               look like radrang is set anywhere for any projections.  More
               time should be spent to determine if more info is available on
               tape. Calculate the rotation angle.  The angle is given by
               sin(a) = cos(180 - incang)/cos(lat). 
               First convert things to radians. */
            a1 = (180.0 - 98.22) /  (180.0 / PI);
            a2 = wrsll[0] / (180.0 / PI);
            radrang = cos(a1) / cos(a2);
        }
        
        /*  retreive the absolute value of the rotation angle
            -----------------------------------------------------*/
        if (radrang < 0.0)
            radrang *= -1;
        
        /* Reapply center value and pixel size
           -------------------------------------- */
        sp = ((double)window[SS] - wrsls[1]) * ddr.pdist_x;
        lp = (wrsls[0] - (double)window[SL]) * ddr.pdist_y;
        
        /* Solve for Xp and Yp using the following transformation equations:
          ------------------------------------------------------------------ */
        xp =  (sp * cos(radrang)) + (lp * sin(radrang));
        yp = (-sp * sin(radrang)) + (lp * cos(radrang));
        
        /* Calculate projection coordinates based on full scene center
           --------------------------------------------------------------*/
        ddr.upleft[0] = yp + wrspj[0];
        ddr.upleft[1] = xp + wrspj[1];
        
        /* Reapply center value and pixel size
           -------------------------------------- */
        sp = ((double)(window[SS] + window[NS] - 1) - wrsls[1]) * ddr.pdist_x;
        lp = (wrsls[0] - (double)window[SL]) * ddr.pdist_y;
        
        /* Solve for Xp and Yp using the following transformation equations:
           ---------------------------------------------------------------- */
        xp =  sp * cos(radrang) + lp * sin(radrang);
        yp = -sp * sin(radrang) + lp * cos(radrang);
        
        /* Calculate projection coordinates based on full scene center
           --------------------------------------------------------------*/
        ddr.upright[0] = yp + wrspj[0];
        ddr.upright[1] = xp + wrspj[1];
        
        /* Reapply center value and pixel size
           -------------------------------------- */
        sp = ((double)(window[SS] + window[NS] -1) - wrsls[1]) * ddr.pdist_x;
        lp = (wrsls[0] - (double)(window[SL] + window[NL] - 1)) * ddr.pdist_y;
        
        /* Solve for Xp and Yp using the following transformation equations:
           ---------------------------------------------------------------- */
        xp =  sp * cos(radrang) + lp * sin(radrang);
        yp = -sp * sin(radrang) + lp * cos(radrang);
        
        /* Calculate projection coordinates based on full scene center
           --------------------------------------------------------------*/
        ddr.loright[0] = yp + wrspj[0];
        ddr.loright[1] = xp + wrspj[1];
        
        /* Reapply center value and pixel size 
           --------------------------------------*/
        sp = ((double)window[SS] - wrsls[1]) * ddr.pdist_x;
        lp = (wrsls[0] - (double)(window[SL] + window[NL] - 1)) * ddr.pdist_y;
        
        /* Solve for Xp and Yp using the following transformation equations: 
           -----------------------------------------------------------------*/
        xp =  sp * cos(radrang) + lp * sin(radrang);
        yp = -sp * sin(radrang) + lp * cos(radrang);
        
        /* Calculate projection coordinates based on full scene center
           --------------------------------------------------------------*/
        ddr.loleft[0] = yp + wrspj[0];
        ddr.loleft[1] = xp + wrspj[1];
    }
    else
    {
        ddr.loright[0] =0.0;
        ddr.loright[1] =0.0;
        ddr.upright[0] = 0.0;
        ddr.upright[1] = 0.0;
        ddr.upleft[0] = 0.0;
        ddr.upleft[1] = 0.0;
        ddr.loleft[0] = 0.0;
        ddr.loleft[1] = 0.0;
    }
    
    /* Update the Band DDR information
       ----------------------------------*/
    for (band = 1; band <= *totbnd; band++)
    {
        /* Get the Band DDR information from the image
           ---------------------------------------------- */
        c_getbdr(hname,&bddr[band-1],&band);
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);
        
        /* Update the Band DDR information
           ----------------------------------*/
        sprintf(bddr[band-1].source,"landsat %s",satid);
        
        if (strcmp(stype,"M") == 0)
            strcpy(bddr[band-1].instrument,"mss");
        else
            strcpy(bddr[band-1].instrument,"rbv");
        
        /* Update the direction of the pass
           -----------------------------------*/
        if (strcmp(direction,"D") == 0)
            strcpy(bddr[band-1].direction,"descending");
        else
            strcpy(bddr[band-1].direction,"ascending");
        
        /* Update the date and time
           ---------------------------*/
        substr(&edate[0],day,2);
        substr(&edate[2],month,3);
        substr(&edate[5],year,2);
        sprintf(bddr[band-1].date,"%s-%s-%s",day,month,year);
        sprintf(bddr[band-1].time,"%s:%s",hrmn,osec);
    }
    
    /* Put the updated DDR information to the image
       ---------------------------------------------- */
    status = c_putddr(hname,&ddr);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);
    
    /* Put the updated Band DDR information to the image
       ----------------------------------------------------*/
    for (band = 0; band < *totbnd; band++)
        c_putbdr(hname,&bddr[band]);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","edipsin-fatal",&fatal);
    
    return E_SUCC;
}
