
C     ******************************************************************
C     PURPOSE: BILMSS processes MSS image data in BIL format.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     bform	       band format indicator
C     bndnum(4)	       output band numbers
C     btot	       total number of output bands
C     dens	       output tape density
C     fill	       fill value
C     msgtim	       elapsed time between display messages
C     nl	       number of lines in input image(s)
C     ns	       number of samples per line in input image(s)
C     path	       nominal path number
C     prefix	       first portion of associated file names
C     row	       nominal row number
C     stype	       sensor type
C     unit	       logical unit number of the tape drive
C     volnum	       current output volume number
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     count	       record counter
C     curr	       current band 
C     egclse	       imageio function to close all image files in a group
C     estep	       imageio function to read/write image lines
C     i		       loop index
C     image(3596)      output image buffer
C     j		       loop index
C     lin	       indicates where to begin extracting lines when 
C		       the image is larger than MSS size
C     mstat	       mount status indicator
C     one	       value of one
C     pix	       indicates where to begin extracting pixels when
C		       the image is larger than MSS size
C     pxcopy	       function to copy from one buffer to another
C     pxswap	       byte swap a data buffer
C     reclen	       record length
C     record	       current record number
C     status	       function return status code
C     stlin	       indicates how many fill lines need to be inserted
C     stpix	       indicates how many fill pixels need to be inserted
C     tpweof           tapeio function to write EOF mark(s) to tape
C     tpwrit	       tapeio function to write to tape
C
C	Algorithm:  BILMSS first fills the image with zeros.
C		    Next the processing message is initalized.
C		    Image records are then written to tape.
C		    Processing continues until all images have
C		    been written in a BIL format.
C     *****************************************************************

      subroutine bilmss (btot,nl,ns,unit,dens,bform,
     +			stype,volnum,bndnum,path,row,fill,prefix,msgtim)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            bndnum(4),
     .			   btot,
     .			   count,
     .                     curr,
     .	 		   egclse,
     .			   estep,
     .   		   fdesc(MAXIMG + 1),
     .                     fill,
     .                     gdesc,
     .                     i,
     .                     j,
     .                     lin,
     .			   mstat,
     .			   msgtim,
     .			   nl,
     .			   ns,
     .	        	   offset(MAXBND+1),
     .			   one
      integer*4
     .                     pix,
     .			   pxcopy,
     .                     reclen,
     .                     record,
     .                     status,
     .                     stlin,
     .                     stpix,
     .                     tpweof,
     .                     tpwrit,
     .			   unit,
     .                     volnum

      character*(CMLEN)  	   prefix

      character*11	   bform

      character*5          dens
 
      character*3          
     .			   path,
     .			   row,
     .			   stype

      character*1          fillvl,
     .			   image(3596),
     .			   thebuf(BSIZE),
     .			   v127,
     .			   zero

      common/seqio/ fdesc,gdesc,offset
      common/thebuf/thebuf
      common/curent/i
      

      data reclen/3596/

      zero = char(0)
      v127 = char(127)
      fillvl = char(fill)

C     Initialization 
C     --------------
      call pxfill(image,EBYTE,3596,zero)
      lin = 0
      pix = 0
      stlin = 0
      stpix = 0
   
      if (nl .gt. 2983) then
        lin = (nl - 2983)/2
        nl = 2983
      else
        stlin = (2983 - nl)/2
      endif

      if (ns .gt. 3548) then
        pix = (ns - 3548)/2
        ns = 3548
      else
        stpix = (3548 - ns)/2
      endif

C     Set the record type to 237
C     --------------------------

      image(6) = char(237)

C     Set the record number to 1
C     --------------------------
      count = 1

C     Fill the image portion of the record with the fill value
C     --------------------------------------------------------
      call pxfill(image(13),EBYTE,3548,fillvl)

C     Initialize and start the display message utility
C     ------------------------------------------------
      i = 1
      call inittm('EDIPSOUT','Processed','line','bands',' ',2983,
     +		  btot,msgtim,ALLCAT)
      call startm

C     Write the proper number of fill lines to center the image
C     ---------------------------------------------------------

      do 200 i = 1,stlin
C       DO FOR four output bands
C       ------------------------
        do 100 j = 1,4

C         Set the record number
C         ---------------------
          record = count

          call btcpy(record,image(1),4)

C         Write the record to tape
C         ------------------------
          status = tpwrit (unit,image,reclen,HANDLE)
   	  if (status .ne. reclen) then
  	     status = egclse(gdesc)
	     call fclean(unit)
 	  endif
          count = count + 1
100     continue
200   continue

C     Skip over unused lines
C     ----------------------
      do 300 i = 1,lin
        status = estep (gdesc)
        if (status .ne. ESUCC) then
	   status = egclse(gdesc)
  	   call fclean(unit)
        endif
300   continue

C     Process the image lines
C     -----------------------
      do 600 i = stlin + 1,nl + stlin

C       Read in the current image line for each input band
C       --------------------------------------------------
        status = estep (gdesc)
        if (status .ne. ESUCC) then
           status = egclse(gdesc)
	   call fclean(unit)
        endif
        curr = 1

C       DO FOR four output bands
C       ------------------------
        do 500 j = 1,4

          if ((j .ne. bndnum(curr)) .or. (curr .gt. btot)) then
C         Process a fill line since current output band wasn't specified
C         --------------------------------------------------------------

C           Initialize the image record
C           ---------------------------
	    call pxfill(image,EBYTE,3596,zero)

C           Set the record number
C           ---------------------
            record = count

            call btcpy (record,image(1),4)

C           Set the record type to 237
C           --------------------------

            image(6) = char(237)

C           Fill the image portion of the record with the fill value
C           --------------------------------------------------------
            call pxfill(image(13),EBYTE,3548,fillvl)

C           Write the fill record to tape
C           -----------------------------
            status = tpwrit (unit,image,reclen,HANDLE)
  	    if (status .ne. reclen) then
	       status = egclse(gdesc)
	       call fclean(unit)
	    endif

C           Increment record counter
C           ------------------------
            count = count + 1

C           Check if next volume needs to be mounted
C           ----------------------------------------
            if ((count .eq. 5965) .and. (dens .eq. '1600')) then
	      call stoptm
              call nxtvol (dens,unit,volnum,mstat)
	      call startm
   	      if (mstat .ne. 0) then
	         status = egclse(gdesc)
	         call errmsg('Error mounting second volume',
     +			     'edipsout-mnterr ',1)
	 	 call fclean(unit)
	      endif
              call wrtdir (stype,bform,btot,path,row,unit,volnum,prefix)
            endif

          else

C         Process the existing image line
C         -------------------------------

C           Set the record number
C           ---------------------
            record = count

            call btcpy(record,image(1),4)

C	    Copy image data from input buffer to output buffer
C           --------------------------------------------------
 	    status = pxcopy(thebuf(offset(curr) + 1 + pix),
     +			    image(13 + stpix), EBYTE, NS)

C           Fill the end of the record with 127
C           -----------------------------------
            call pxfill(image(3561),EBYTE,36,v127) 

C           Write the image record to tape
C           ------------------------------
            status = tpwrit (unit,image,reclen,HANDLE)
	    if (status .ne. reclen) then
	       status = egclse(gdesc)
	       call fclean(unit)
	    endif

C           Increment counters
C           ------------------
            curr = curr + 1
            count = count + 1

C           Check if next volume needs to be mounted
C           ----------------------------------------
            if ((count .eq. 5965) .and. (dens .eq. '1600')) then
	      call stoptm
              call nxtvol (dens,unit,volnum,mstat)
	      call startm
   	      if (mstat .ne. 0) then
	         status = egclse(gdesc)
	         call errmsg('Error mounting second volume',
     +			     'edipsout-mnterr ',1)
	         call fclean(unit)
	      endif
              call wrtdir (stype,bform,btot,path,row,unit,volnum,prefix)
            endif
          endif

500     continue
600   continue

C     Initialize the image record
C     ---------------------------
      call pxfill(image,EBYTE,3596,zero)

C     Set the record type to 237
C     --------------------------

      image(6) = char(237)

C     Fill the image portion of the record with the fill value
C     --------------------------------------------------------
      call pxfill(image(13),EBYTE,3548,fillvl)

C     Write the remaining fill lines
C     ------------------------------

      do 800 i = stlin + nl + 1,2983
        do 700 j = 1,4
C           Set the record number
C           ---------------------
            record = count

            call btcpy (record,image(1),4)

C           Write the fill record to tape
C           -----------------------------
            status = tpwrit (unit,image,reclen,HANDLE)
	    if (status .ne. reclen) then
	       status = egclse(gdesc)
	       call fclean(unit)
            endif

C           Increment the record counter
C           ----------------------------
            count = count + 1

C           Check if next volume needs mounting
C           -----------------------------------
            if ((count .eq. 5965) .and. (dens .eq. '1600')) then
	      call stoptm
              call nxtvol (dens,unit,volnum,mstat)
	      call startm
              if (mstat .ne. 0) then
	         status = egclse(gdesc)
 	         call errmsg('Error mounting second volume',
     +			     'edipsout-mnterr',1)
 	         call fclean(unit)
              endif
              call wrtdir (stype,bform,btot,path,row,unit,volnum,prefix)
            endif
700     continue
800   continue

C     Disable the display message utility
C     -----------------------------------
      call stoptm

C     Write the EOF mark
C     ------------------
      one  = 1
      status = tpweof (unit,one,HANDLE)
      if (status .ne. TSUCC) then
	 status = egclse(gdesc)
	 call fclean(unit)
      endif

      return
      end
