C     **************************************************************************
C     NAME        	      EDIPSOUT	
C
C     PURPOSE	 	     		
C        EDIPSOUT is a routine used to copy a LAS image from disk
C        to tape in EDIPS 0.0 format.  The user specifies the number of
C        bands to be copied and the format to be used (either band
C        sequential or band interleaved by line).
C
C     COMMON BLOCK REFERENCES
C        Name	     Field             Type      Description
C        ----	     -----             ----      -----------
C        randio      rfdesc            I*4       file descriptor 
C						 (used with random I/O)
C        seqio       fdesc(MAXIMG+1)   I*4       file descriptor
C						 (used with seq. I/O)
C        seqio       gdesc	       I*4       group descriptor
C						 (used with seq. I/O)
C        seqio       offset(MAXBND+1)  I*4       offsets into "thebuf"
C						 (used with seq. I/0)
C        thebuf      thebuf(BSIZE)     C*1/B     IMAGEIO buffer
C	 
C     SUBPROGRAMS CALLED
C
C        Name          Description
C        ----          -----------
C        btcpy         copies bytes of an integer into a character buffer
C        bilmss        processes MSS image data in BIL format
C        bsqmss	       processes a band with MSS image data in BSQ format
C        fclean         fcleans up after a fatal error and aborts processing
C        getpar        gets the parameters from the user
C        inter         controls processing in BIL format
C        nxtvol        mounts the second tape in a two-volume set
C        rbv	       processes a band with RBV image data
C        seq           controls processing in band sequential format
C        wrtant        sets up and writes the annotation record to tape
C        wrtdir        sets up and writes the directory to tape
C        wrthd         sets up and writes the header record to tape
C        wrttra	       sets up and writes the trailer record to tape
C
C        (TAE routines)
C        xrintg         get an integer parameter
C        xrstr  	get a string parameter
C  
C        (Image I/O Routines)
C        eclose	        close a LAS image file
C        egclse		close all files in a LAS image file group
C        egroup		group several LAS image files
C        eopenr	        open a LAS image file for random access
C        eopens		open a LAS image file for sequential access
C        eread	        random read from a LAS image file  
C        estep		sequentially steps through LAS image file groups
C
C        (Tapeio Routines)
C        tpclos	        close a tape device
C        tpopen		open a tape device
C        tpweof	        write EOF mark(s) to tape
C        tpwrit		write to tape
C
C        (LAS Utilities)
C        complt      	write a completion message to the terminal
C	 errmsg		output error message
C        getin		parse the input image specification
C	 inittm		initialize display message utility
C	 inlas		initialize a LAS program
C	 stoptm		output final display mesage and terminate the
C			alarm interrupt
C  	 startm		start displaying messages based on elapsed time
C  
C        (Label Services Routines)
C        lsmknm		make an associated file name
C	 lsstat		check whether associated file exists
C    
C        (Pixel Manipulation Routines)
C        pxcopy    	copy pixels from one buffer to another
C        pxfill	        fill a buffer with a specified value
C        pxswap 	byte swap a data buffer
C
C     INFORMATIONAL MESSAGES AND
C     NON-FATAL ERROR CONDITIONS	
C     1.  [edipso-extra]      More than four bands were specified
C			      Only the first four are being processed
C   
C     2.  [edipso-note]       Image is being processed in band sequential format
C   
C     3.  [edipso-tpcloserr]  Warning:  Error closing tape
C
C
C     FATAL ERROR CONDITIONS
C     1.  [edipso-fatal]     Fatal error encountered--output tape is not valid
C
C     2.  [edipso-notbyte]   Input image is not BYTE
C	 	             Only BYTE data is currently supported
C
C     3.  [edipso-window]    Input images do not have the same window size
C
C     4.  [edipso-mnterr]    Error mounting tape
C
C     5.  [edipso-openerr]   Error opening associated <header,
C			     annotation, or trailer> file
C
C     PROGRAM HISTORY
C	PROGRAMMER	  DATE		REASON
C	----------	  ----		------
C	K. Zanter	Nov. 1985     CIPS conversion to 1
C       K. Zanter       Oct. 1986     LAS updates
C	R. McKinney     Dec. 1988     LAS 5.0 conversion
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C       EDIPSOUT must be run under TAE.
C
C     PROJECT		        LAS		
C
C     LANGUAGE			FORTRAN-77
C
C     ALGORITHM (FLOW)
C       Retrieve user specified parameters and global variables.
C       Open the output tape.
C       Call the respective routines for either BIL or BSQ format.
C       Close the output tape.
C       Send a completion message to standard output.
C       
C     ALGORITHM REFERENCES 		none
C
C     ******************************************************************

      program edipsout

C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     annot 	       user-specified annotation record
C     bands(MAXBND+1,MAXIMG)
C	               input band specification
C     bform	       band format indicator
C     bndnum(4)	       output band numbers
C     btot	       total number of output bands
C     dens	       output tape density
C     fill	       fill value
C     hosin(MAXIMG)    input host names
C     msg	       mount message sent to operator
C     msgtim	       elapsed time between display messages
C     nband(MAXIMG)    number of bands in each input image
C     nmrimg	       number of input images
C     parblk(XPRDIM)   parameter block	
C     path	       nominal path number
C     prefix	       first portion of associated file names
C     row	       nominal row number
C     skip 	       number of files to skip
C     status	       function return status code
C     stdout	       standard output 
C     stype	       sensor type
C     tlabel	       mount message for first volume
C     tpclos           tapeio function to close a tape device
C     tpopen	       tapeio function to open a tape device
C     three	       value of three
C     unit	       logical unit number of the tape drive
C     window(4,MAXIMG) input window specification
C     zero	       value of zero
C     ******************************************************************

	include 'las.fin'

	include 'tapeio.fin'

      integer*4
     .     	          bands(MAXBND+1,MAXIMG),
     .		  	  bndnum(4),
     .			  btot,
     .                    fill,
     .      	          msgtim,
     .			  nband (MAXIMG),
     .		          nmrimg,
     .			  skip,
     .                    status,
     .			  stdout,
     .                    tpclos,
     .                    tpopen,
     .	      	          three,
     .                    unit,
     .		          window(4,MAXIMG),
     .			  zero

      integer*4 parblk(XPRDIM)

      character*112       annot

      character*(CMLEN)       hosin(MAXIMG), 
     .			  msg,
     .			  prefix

      character*80        tlabel

      character*11	  bform
      character*(CMLEN)   tapeid

      character*5	  dens
      character*3	  
     .			  path,
     .			  row,
     .			  stype

      zero = 0

C     Initialization
C     --------------
      call inlas(parblk,stdout)

C     Get user-specified parameters
C     -----------------------------
      call getpar(parblk,msgtim,prefix,hosin,nmrimg,btot,window,
     +	          bands,nband,dens,path,row,annot,bndnum,fill,
     +		  bform,stype,tlabel)

C     Open the output tape volume
C     ---------------------------
      msg = tlabel//char(0)
      skip = 0
      status = tpopen(unit,skip,HANDLE,TNOREW,dens,TWRITE,msg,tapeid)
      if (status .ne. TSUCC) then
	 call errmsg
     +	 ('Fatal error encountered--Output tape is not valid',
     +    'edipsout-fatal',-1)
      endif

C     Call proper processing routine
C     ------------------------------
      if (bform .eq. 'INTERLEAVED') then
        call inter (hosin,prefix,msgtim,dens,unit,nmrimg,btot,window,
     +		    bands,fill,bndnum,bform,stype,nband,path,row,annot)
      else
        call seq (hosin,prefix,msgtim,dens,unit,nmrimg,btot,window,
     +		  bands,fill,bndnum,bform,stype,nband,path,row,annot)
      endif

C     Close the output tape
C     ---------------------
      three = 3
      status = tpclos(unit,HANDLE,TRWOFL,three)
      if (status .ne. TSUCC) then
	 call errmsg('Warning:  Error closing tape',
     +	 	     'edipsout-tpcloserr',1)
      endif

C     Print a completion message
C     --------------------------
      call complt ('EDIPSOUT',' ',zero)

      end
