
C     ******************************************************************
C     PURPOSE:  INTER is used to control the processing of an image out 
C               to tape in band interleaved by line (BIL) format.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     antrec           user-specified annotation record
C     bands(MAXBND+1,MAXIMG)
C                      input band specification
C     bform 	       band format indicator
C     bndnum(4)        output band numbers
C     btot	       total number of output bands
C     dens             output tape density
C     fill  	       fill value
C     hosin(MAXIMG)    input host names
C     msgtim           elapsed time between display messages
C     nband(MAXIMG)    number of bands in each input image
C     nmrimg           number of input images
C     path	       nominal path number
C     prefix	       first portion of associated file names
C     row	       nominal row number
C     stype            sensor type
C     unit	       logical unit number of the tape drive
C     window(4,MAXIMG)
C                      input window specification
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     band	       band counter
C     curbnd	       current band number
C     dtype	       data type
C     egclse	       imageio function to close all image files in a group
C     egroup	       imageio function to group a set of image files
C     eopens	       imageio function to open an image for sequential access
C     four             value of four
C     gain(4)	       MSS sensor gain options
C     i		       loop index
C     nl	       number of lines in input image(s)
C     ns	       number of samples per line in input image(s)
C     one	       value of one
C     status	       function return status code
C     tpweof	       tapeio function to write EOF mark(s) to tape
C     trans(4)	       transmission mode
C     unittr	       logical unit number of associated trailer file
C     volnum	       current output volume number
C     zero	       value of zero
C
C	ALGORITHM: Write the tape directory
C                  Open the LAS image
C                  Group images with header and annotation routines
C                  Write in BIL format with trailer information last
C                  Close the files
C           
C     ******************************************************************

      subroutine inter(hosin,prefix,msgtim,dens,unit,nmrimg,btot,
     +		       window,bands,fill,bndnum,bform,stype,nband,
     +		       path,row,antrec)
     
	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .			   band,
     .     	           bands(MAXBND+1,MAXIMG),
     .			   bndnum(4),
     .		  	   btot,
     .  		   dtype,
     .  	           egclse,
     .	         	   egroup,
     .			   eopens,
     .			   fill,
     .   		   fdesc(MAXIMG + 1),
     .			   four,
     .			   gdesc,
     .			   i,
     .			   msgtim,
     .		           nl,
     .			   ns,
     .	                   nband (MAXIMG),
     .		           nmrimg
      integer*4
     .			   offset(MAXBND+1),
     .			   one,
     .                     status,
     .                     tpweof,
     .                     unit,
     .                     unittr,
     .                     volnum,
     .		           window(4,MAXIMG),
     .			   zero

      character*112        antrec

      character*(CMLEN)        hosin(MAXIMG),
     .                     prefix
      character*5          dens

      character*11         bform

      character*3 	   
     .			   path,
     .			   row,
     .			   stype

      character*1          
     .                     curbnd,
     .  	 	   gain(4),
     . 	  		   thebuf(BSIZE),
     .			   trans(4)

      common/seqio/ fdesc,gdesc,offset
      common/thebuf/thebuf

      zero = 0
      one = 1

      volnum = 1
      nl = window(3,1)
      ns = window(4,1)

C     Create the tape directory
C     -------------------------
      call wrtdir (stype,bform,btot,path,row,unit,volnum,prefix)

C     Open the LAS image(s)
C     ---------------------
      band = 1
      do 10 i = 1,nmrimg
         dtype = 0
         status = eopens(fdesc(i),hosin(i),bands(1,i),window(1,i),
     +        	  window(2,i),window(3,i),window(4,i),offset(band),
     +		  IREAD,dtype,zero,zero)
         if (status .ne. ESUCC) call fclean(unit)
         if (dtype. ne. EBYTE) then
	    call errmsg('Input image data is not byte',
     +                  'edipsout-notbyte',1)
            call errmsg('Only byte data is currently supported',
     +                  'edipsout-notbyte',1)
            call fclean(unit)
          endif
          band = band + nband(i)
10    continue

C     Group the images for sequential processing
C     ------------------------------------------
      fdesc(nmrimg + 1) = 0
      status = egroup(fdesc,gdesc,thebuf,BSIZE)
      if (status .ne. ESUCC) then
         status = egclse(gdesc)
         call fclean(unit)
      endif

C     Create the header record
C     ------------------------

      curbnd = char(0)

      call wrthd (stype,bform,btot,path,row,unit,prefix,
     +		  gain,trans,curbnd,bndnum)

C     Create the annotation records
C     -----------------------------

      curbnd = char(1)

      call wrtant (stype,path,row,unit,prefix,gain,trans,
     +		   curbnd,antrec)

C     Write the EOF mark
C     ------------------
      status = tpweof (unit,one,HANDLE)
      if (status .ne. TSUCC) then
         status = egclse(gdesc)
         call fclean(unit)
      endif

C     Process the image lines
C     -----------------------
      call bilmss (btot,nl,ns,unit,dens,
     + 	           bform,stype,volnum,bndnum,path,row,
     +	  	   fill,prefix,msgtim)

C     Create the trailer records
C     --------------------------
      four = 4
      call wrttra (stype,bform,prefix,unit,unittr,bndnum,four)

C     Close the LAS image(s)
C     ----------------------
      status = egclse(gdesc)

      close (unit = unittr)

      return
      end
