
C     ******************************************************************
C     PURPOSE:  SEQ is used to control the processing of an image to
C               tape in band sequential (BSQ) format.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     antrec	       user-specified annotation record
C     bands(MAXBND+1,MAXIMG)
C		       input band specification
C     bform	       band format indicator
C     bndnum(4)	       output band numbers
C     btot	       total number of output bands
C     dens	       output tape density
C     fill	       fill value
C     hosin(MAXIMG)    input host names
C     msgtim	       elapsed time between display messages
C     nband(MAXIMG)    number of bands in each input image
C     nmrimg	       number of input images
C     path	       nominal path number
C     prefix	       first portion of associated file names
C     row	       nominal row number
C     stype	       sensor type
C     unit	       logical unit number of the tape drive
C     window(4,MAXIMG)
C		       input window specification     
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     bndout	       current output band number
C     curbnd	       current band number
C     dtype	       data type
C     eclose	       imageio function to close an image file
C     eopenr	       imageio function to open an image for random access
C     gain(4)	       MSS sensor gain options
C     i	 	       loop index
C     j		       loop index
C     last	       flag indicating whether current band is the
C		       last band to be processed
C     mstat	       mount status indicator
C     nl	       number of lines in input image(s)
C     ns	       number of samples per line in input image(s)
C     nxt	       flag indicating whether next volume needs to be mounted
C     one	       value of one
C     status	       function return status code
C     tpweof	       tapeio function to write EOF mark(s) to tape
C     trans(4)	       transmission mode
C     unittr	       logical unit number of associated trailer file
C     volnum	       current output volume number
C     zero	       value of zero
C
C     ALGORITHM:
C                Write tape directory
C                Open LAS image
C                Group images with header and annotation routines
C                Write in BSQ format with trailer information last
C                Close files
C       
C     ******************************************************************

      subroutine seq  (hosin,prefix,msgtim,dens,unit,nmrimg,btot,
     +		       window,bands,fill,bndnum,bform,stype,nband,
     +		       path,row,antrec)

	include 'las.fin'

	include 'tapeio.fin'

       integer*4           
     .			   bands(MAXBND + 1,MAXIMG),
     .			   bndnum(4),
     .			   bndout,
     .			   btot,
     .			   dtype,
     .		           eclose,
     .			   eopenr,
     .                     i,
     .                     fill,
     .			   j,
     .			   msgtim,
     .			   mstat,
     .			   nband (MAXIMG),
     .			   nl,
     .			   nmrimg,
     .			   ns,
     .			   one
      integer*4
     .			   rfdesc,
     .                     status,
     .                     tpweof,
     .			   unit,
     .                     unittr,
     .                     volnum,
     .			   window(4,MAXIMG),
     .			   zero

      character*112        antrec

      character*(CMLEN)        
     .                     hosin(MAXIMG),
     .                     prefix

      character*11	   bform
      character*5          dens

      character*3	   
     .			   path,
     .			   row,
     .			   stype

      character*1       
     .			   gain(4),
     .			   trans(4),
     .			   curbnd

      logical              
     .			   last,
     .                     nxt

      common/randio/rfdesc
      common/curcat/bndout

      zero = 0
      one = 1

      volnum = 1
      bndout = 0
      last = .false.
      nxt = .false.
      nl = window(3,1)
      ns = window(4,1)

C     Create the tape directory
C     -------------------------
      call wrtdir (stype,bform,btot,path,row,unit,volnum,prefix)

      do 100 i = 1,nmrimg

C        Open the LAS image
C        ------------------
    	 dtype = 0
  	 status = eopenr(rfdesc,hosin(i),zero,window(1,i),window(2,i),
     +			 window(3,i),window(4,i),IREAD,dtype,zero,one)
	 if (status .ne. ESUCC) call fclean(unit)
         if (dtype .ne. EBYTE) then
 	   call errmsg('Input image data is not byte',
     +			'edipsout-notbyte',1)
	   call errmsg('Only byte data is currently supported',
     +			'edipsout-notbyte',1)
	    call fclean(unit)
         endif
	    
         do 50 j = 1, nband(i)
 	    bndout = bndout + 1
  	    last = bndout .eq. btot

	    curbnd = char(bndnum(bndout))

	  
C	    Determine if the next volume needs to be mounted
C  	    ------------------------------------------------
    	    if (dens .eq. '1600') then
	       if (nxt) then
		   call nxtvol(dens,unit,volnum,mstat)
	    	   if (mstat .ne. 0) then
	 	      status = eclose(rfdesc)
		      call errmsg('Error mounting second volume',
     +		                  'edipsout-mnterr ',1)
		      call fclean(unit)
	 	   endif
		   call wrtdir(stype,bform,btot,path,row,unit,
     +			       volnum,prefix)
		   nxt = .false.
	       endif
            endif

C	    Create the header record
C           ------------------------
            call wrthd(stype,bform,btot,path,row,unit,prefix,
     +		       gain,trans,curbnd,bndnum)

C 	    Create the annotation record
C    	    ----------------------------
 	    call wrtant(stype,path,row,unit,prefix,gain,trans,
     +			curbnd,antrec)

C	    Write the EOF mark
C  	    ------------------
	    status = tpweof(unit,one,HANDLE)
	    if (status .ne. TSUCC) then
	       status = eclose(rfdesc)
	       call fclean(unit)
	    endif

C	    Process the image lines
C	    -----------------------
	    if (stype .eq. 'MSS') then
	        call bsqmss (btot,nl,ns,unit,fill,msgtim,bands(j,i))
	    else
		call rbv (nl,ns,unit,fill,msgtim,bands(j,i))
	    endif

C	    Create the trailer records
C 	    --------------------------
 	    call wrttra(stype,bform,prefix,unit,unittr,bndnum,1)
	 
	    nxt = (bndout .eq. 2) .and. (btot .eq. 4) 
	    
 	    if ( (.not. nxt) .and. (.not. last)) then
	       status = tpweof (unit,one,HANDLE)
	       if (status .ne. TSUCC) then
		  status = eclose(rfdesc)
		  call fclean(unit)
	       endif
	    endif

50       continue

C        Close the LAS image file
C        ------------------------
         status = eclose(rfdesc)
	 if (status .ne. ESUCC) call fclean(unit)

100   continue

C     Close the trailer file
C     ----------------------
      close (unit=unittr)

      return
      end
