
C     ******************************************************************
C     PURPOSE: WRTDIR sets up the tape directory and writes it to tape.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     bform            band format indicator
C     btot	       total number of output bands
C     path	       nominal path number
C     prefix	       first portion of associated file name
C     row	       nominal row number
C     stype	       sensor type
C     unit	       logical unit number of the tape drive
C     volnum	       current output volume number
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     day	       current day
C     flag	       indicator for whether associated header file exists
C     filenm	       host name of associated header file
C     head1(5388)      header buffer (RBV)
C     head2(3596)      header buffer (MSS)
C     i	               loop index
C     id	       image id (from header record)
C     imgid	       image id (for tape directory)
C     j 	       loop index
C     lsmknm	       function to make an associated file name
C     lsstat	       function to check whether associated file exists
C     mon	       current month
C     pthbuf	       nominal path number (for tape directory buffer)
C     reclen	       record length
C     recnum	       record number
C     rowbuf	       nominal row number (for tape directory buffer)
C     status 	       function return status code
C     tapdir(360)      tape directory buffer
C     tpweof	       tapeio function to write EOF mark(s) to tape
C     tpwrit	       tapeio function to write to tape
C     unithd	       logical unit number of associated header file
C     yr	       current year (last two digits)
C
C	ALGORITHM:
C                  Initialize the tape directory buffer
C                  Fill the tape directory array
C                  Retrieve and use the header information if available
C                  Write tape directory to tape with an end of file marker
C                  Close files
C                   
C     ******************************************************************

      subroutine wrtdir (stype,bform,btot,path,row,unit,volnum,prefix)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .                     btot,
     .                     day,
     .                     i,
     .                     j,
     .			   lsmknm,
     .			   lsstat,
     .                     mon,
     .			   reclen,
     .                     recnum,
     .                     status,
     .                     tpweof,
     .                     tpwrit,
     .                     unit,
     . 			   unithd,
     .                     volnum,
     .                     yr

      character*(CMLEN)        
     .			   filenm,
     .			   prefix

      character*11         
     .			   bform,
     .			   imgid

      character*10 	   id 

      character*3	   
     .			   path,
     .			   pthbuf,
     .			   row,
     .			   rowbuf,
     .			   stype

      character*1          tapdir(360),
     .		           head1(5388),
     .			   head2(3596),
     .			   zero

      logical              flag

      equivalence (imgid,tapdir(35))
      equivalence (pthbuf,tapdir(47))
      equivalence (rowbuf,tapdir(50))
      equivalence (id,head1(8))
      equivalence (head1,head2)

      zero = char(0)

C     Initialize the tape directory record to zeroes
C     ----------------------------------------------
      call pxfill(tapdir,EBYTE,360,zero)

C     Set the record number
C     ---------------------
      recnum = 1
      call btcpy (recnum,tapdir(1),4)

C     Set the record type to 9
C     ------------------------
      tapdir(6) = char(9)

C     Set bytes 7-29 to blanks
C     ------------------------
      do 100 i = 7,29
        tapdir(i) = char(32)
100   continue

C     Set mission designator to 'L'
C     -----------------------------
      tapdir(7) = char(76)

C     Set the sensor type
C     -------------------
      if (stype .eq. 'MSS') then
        tapdir(9) = char(77)
      else
        tapdir(9) = char(82)
      endif

C     Set the tape type to CP
C     -----------------------
      tapdir(10) = char(67)
      tapdir(11) = char(80)

C     Set the volume number 
C     ---------------------
      tapdir(19) = char(volnum + 48)

C     Set the number of volumes in the set
C     ------------------------------------
      if (bform .eq. 'SEQUENTIAL') then
        if (btot .eq. 4) then
          tapdir(20) = char(50)
        else
          tapdir(20) = char(49)
        endif
      else
        tapdir(20) = char(50)
      endif

C     Get the current date 
C     --------------------
      call sysdat (mon,day,yr)

C     Set tape generation date
C     ------------------------
      tapdir(27) = char(day)
      tapdir(28) = char(mon)
      tapdir(29) = char(yr)

C     Set site of production
C     ----------------------
      tapdir(30) = char(237)

C     Set the interleaving type indicator
C     -----------------------------------
      if (bform .eq. 'INTERLEAVED')  tapdir(31) = char(255)

C     Set the record length
C     ---------------------
      if (stype .eq. 'MSS') then
        reclen = 3596
      else
        reclen = 5388
      endif

      call btcpy(reclen,tapdir(32),2)

C     Set indicator for geometrically corrected
C     -----------------------------------------
      tapdir(34) = char(67)

C     Set bytes 35-52 to blanks
C     -------------------------
      do 200 j = 35,52
        tapdir(j) = char(32)
200   continue

C     Determine if the associated header file exists 
C     ----------------------------------------------
      status = lsmknm(prefix,'edips_headr',filenm)
      status = lsstat(filenm,0)
      if (status .eq. 1) then
         flag = .true.
      else
         flag = .false.
      endif

      if (flag) then
C       get the data from the associated header file 
C       --------------------------------------------
        unithd = 2
        open (unit=unithd,status='old',file=filenm,form='unformatted',
     +    iostat=status)
       
        rewind (unit=unithd)

        if (stype .eq. 'MSS') then
          read (unit=unithd,end=300,err=300) head2
        else 
          read (unit=unithd,end=300,err=300) head1
        endif

        imgid = id

        if (stype .eq. 'MSS') then
          tapdir(45) = ' '
        else
          tapdir(45) = 'A'
        endif

      else

C       Insert a dummy image ID
C       -----------------------

300     continue
        if (stype .eq. 'MSS') then
          imgid = '4999999999 '
        else
          imgid = '3999999999A'
        endif

      endif

C     Insert user specified path row indicator
C     ----------------------------------------
      if (path .ne. ' ') pthbuf = path
      if (row .ne. ' ') rowbuf = row

C     Write the tape directory to the output tape
C     -------------------------------------------
      status = tpwrit (unit,tapdir,360,HANDLE)  
      if (status .ne. 360) call fclean(unit)

C     Write the EOF mark
C     ------------------
      status = tpweof (unit,1,HANDLE)
      if (status .ne. TSUCC) call fclean(unit)

C     If necessary, close the associated header file
C     ----------------------------------------------
      if (flag) close (unit = unithd)

      return
      end

