
C     ******************************************************************
C     PURPOSE: WRTHD sets up and writes the header record to tape.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     bform	       band format indicator
C     bndnum(4)	       output band numbers
C     btot	       total number of output bands
C     curbnd	       current band number
C     gain(4)	       MSS sensor gain options
C     path	       nominal path number
C     prefix	       first portion of associated file names
C     row	       nominal row number
C     stype	       sensor type
C     trans(4)	       transmission mode     
C     unit	       logical unit number of the tape drive
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     bnd1             number of input bands stored in a byte
C     bnd2	       number of input bands stored in two bytes
C     day	       current day
C     filenm	       host name of associated header file
C     flag             indicator for whether associated header file exists
C     header(5388)     header buffer (RBV)
C     headm(3596)      header buffer (MSS)
C     i	   	       loop index
C     imgid	       image ID
C     j		       loop index
C     k	   	       loop index
C     l	   	       loop index
C     lsmknm	       function to make an associated file name
C     lsstat	       function to check whether associated file exists
C     mon	       current month
C     number
C     opn	       indicator for whether associated header file is open
C     pres(4)          data present masks
C     pix	       number of pixels per scan line
C     ptbuf1	       nominal path number (for first place in header buffer)
C     ptbuf2	       nominal path number (for second place in header buffer)
C     reclen	       record length
C     recnum	       record number
C     rwbuf1 	       nominal row number (for first place in header buffer)
C     rwbuf2	       nominal row number (for second place in header buffer)
C     status	       function return status code
C     tpwrit	       tapeio function to write to tape
C     unithd	       logical unit number of associated header file
C     yr	       current year (last two digits)
C
C	ALGORITHM:  WRTHD initializes the header buffer with zeros.
C		    Information is taken from the header file.
C		    The header buffer is filled with known values
C		      if the header file  doesn't exist.
C		    The header file is written to tape.
C		    The files are closed.
C     ******************************************************************

      subroutine wrthd (stype,bform,btot,path,row,unit,prefix,
     +			gain,trans,curbnd,bndnum)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .		  	   bndnum(4),
     .			   btot,
     .                     day,
     .                     i,
     .                     j,
     .                     k,
     .                     l,
     .			   lsmknm,
     .			   lsstat,
     .                     mon,
     .		           number,
     .			   pix,
     .                     reclen,
     .                     recnum
      integer*4
     .                     status,
     .                     tpwrit,
     .                     unit,
     .			   unithd,
     .                     yr

      integer*2            bnd2

      character*(CMLEN)	   filenm,
     .			   prefix
    
      character*11         bform

      character*10 	   imgid

      character*3	
     .			   path,
     .			   ptbuf1,
     .			   ptbuf2,
     .			   row,
     .			   rwbuf1,
     .			   rwbuf2,
     .			   stype

      character*1 	   header(5388),
     .			   headm(3596),
     .		           bnd1,
     .			   gain(4),
     .			   trans(4),
     .			   curbnd,
     .			   pres(4),
     .			   zero

      logical              flag,
     .                     opn

      equivalence (header,headm)
      equivalence (ptbuf1,header(21)),  (ptbuf2,header(177))
      equivalence (rwbuf1,header(24)),  (rwbuf2,header(180))
      equivalence (imgid,header(8))

      pres(1) = char(16)
      pres(2) = char(8)
      pres(3) = char(4)
      pres(4) = char(2)

      zero = char(0)

C     Set the record length
C     ---------------------
      if (stype .eq. 'MSS') then
        reclen = 3596
      else
        reclen = 5388
      endif

C     Initialize the header record to zeroes
C     --------------------------------------
      call pxfill(header,EBYTE,reclen,zero)

C     Determine if associated header file exists
C     ------------------------------------------
      status = lsmknm(prefix,'edips_headr',filenm)
      status = lsstat(filenm,0)
      if (status .eq. 1) then
	 flag = .true.
      else
	 flag = .false.
      endif

      if (flag) inquire(file=filenm,opened=opn)

C     Open the associated header file if necessary
C     --------------------------------------------
      if ((flag) .and. (.not. opn)) then
        unithd = 2
        open (unit=unithd,status='old',file=filenm,form='unformatted',
     +        iostat=status)
        if (status .ne. 0) then
          call errmsg ('Error opening associated header file',
     +     	       'edipsout-openerr',1)
          call fclean(unit)
        endif

        rewind (unit=unithd)
      endif

      if (flag) then
C       Read a record from the file if it exists
C       ----------------------------------------
        if (stype .eq. 'RBV') then
          read (unit=unithd,end=50,err=50) header
        else
          read (unit=unithd,end=50,err=50) headm
        endif

        if (stype .eq. 'MSS') then
          do 10 i = 1,4
            gain(i) = header(3586 + i)
            trans(i) = header(3591 + i)
10        continue          
        endif

        if ((bform .eq. 'SEQUENTIAL') .and. (stype .eq. 'MSS')) then
          header(136) = char(ichar(curbnd) + 48)
        endif

      else
50      continue
C 	        -------------------------------------
C               |Associated file does not exist     |
C               |Write known fields to header record|
C               -------------------------------------

C       Set record number
C       -----------------
        recnum = 1
        call btcpy (recnum,header(1),4)

C       Set record type to 18
C       ---------------------       
        header(6) = char(18)

C       Set bytes 7-29 to blanks
C       ------------------------
        do 100 i = 7,29
          header(i) = char(32)
100     continue

C       Insert a dummy image ID
C       -----------------------
        if (stype .eq. 'MSS') then
          imgid = '4999999999'
        else
          imgid = '3999999999'
        endif

C       Set band indicator
C       ------------------
        if ((stype .eq. 'MSS') .and. (bform .eq. 'SEQUENTIAL')) then
          header(18) = char(ichar(curbnd) + 48)
        elseif (stype .eq. 'RBV') then
          header(18) = char(65)
        else
          header(18) = char(48)
        endif

C       Get the current date
C       --------------------
        call sysdat (mon,day,yr)

C       Set tape generation date
C       ------------------------
        header(27) = char(day)
        header(28) = char(mon)
        header(29) = char(yr)

C       Set the sensor type
C       -------------------
        if (stype .eq. 'MSS') then
          header(37) = char(77)
          header(38) = char(83)
          header(39) = char(83)
        else
          header(37) = char(82)
          header(38) = char(66)
          header(39) = char(86)
        endif

C       Set record lengths of header, annotation, ancillary, and image records
C       ----------------------------------------------------------------------
        call btcpy (reclen,header(93),2)
        call btcpy (reclen,header(97),2)
        call btcpy (reclen,header(99),2)
        call btcpy (reclen,header(103),2)
        call btcpy (reclen,header(111),2)
        call btcpy (reclen,header(142),2)
         
C       Set bytes 40-44 to a blank
C       --------------------------
        do 200 j = 40,44
          header(j) = char(32)
200     continue

C       Set mission number
C       ------------------
        if (stype .eq. 'MSS') then
          header(46) = char(4)
        else
          header(46) = char(3)
        endif
 
C       Set bytes 77-92 to a blank
C       --------------------------
        do 300 k = 77,92
          header(k) = char(32)
300     continue

C       Set flags indicating data is geometrically and
C       radiometrically corrected
C       ----------------------------------------------
        header(107) = char(255)
        header(109) = char(255)

C       Set flags indicating no resampling or map projections have been applied
C       -----------------------------------------------------------------------
        header(123) = char(192)
        header(124) = char(192)

C       Set interleaving type indicator and BIL line interleaving count
C       ---------------------------------------------------------------
        if (bform .eq. 'INTERLEAVED') then
          header(120) = char(255)
          header(121) = char(4)
        endif

C       Set number of bits per pixel 
C       ----------------------------
        header(122) = char(8)
  
C       Set image justification flag
C       ----------------------------
        header (129) = char(255)

C       Set MSS band number
C       -------------------
        if ((bform .eq. 'SEQUENTIAL') .and. (stype .eq. 'MSS')) then
          header(136) = char(ichar(curbnd) + 48)
        elseif (stype .eq. 'RBV') then
          header(136) = char(65)
        else
          header(136) = char(48)
        endif
   
C       Set bytes 163-182 to a blank
C       ----------------------------
        do 400 l = 163,182
          header(l) = char(32)
400     continue

C       Insert user specified path row indicator into the header record
C       ---------------------------------------------------------------
        if (path .ne. ' ') then
           ptbuf1 = path
           ptbuf2 = path
        endif
      
        if (row .ne. ' ') then
   	   rwbuf1 = row
  	   rwbuf2 = row
        endif

C       Set the number of header, annotation and trailer records
C       --------------------------------------------------------
        number = 1
        call btcpy (number,header(95),2)
        call btcpy (number,header(101),2)

        if (bform .eq. 'INTERLEAVED') then
          bnd2 = btot
          number = bnd2
        else
          number = 1
        endif

        call btcpy (number,header(144),2)

C       Set number of pixels per scan line
C       ----------------------------------
        if (stype .eq. 'MSS') then
          pix = 3548
        else
          pix = 5322
        endif

        call btcpy (pix,header(131),2)

C       Set number of usable images
C       ---------------------------
        if (bform .eq. 'INTERLEAVED') then
          header(135) = char(4)
        else
          bnd1 = char(btot)
          header(135) = bnd1
        endif

C       Set data present indicator if band format is BIL
C       ------------------------------------------------
        if (bform .eq. 'INTERLEAVED') then
          do 450 i = 1,4
            if ((bndnum(i) .ne. 0) .and. (i .le. btot)) then
              header(3586) = char(ichar(header(3586)) 
     +				 + ichar(pres(bndnum(i))))
            endif
450       continue
        endif

C       Set bytes 3587-3596 to a blank
C       ------------------------------
        do 500 i = 3587,3596
          header(i) = char(32)
500     continue

      endif

C     Write the header record to tape
C     -------------------------------
      if (stype .eq. 'RBV') then
        status = tpwrit (unit,header,reclen,HANDLE)
        if (status .ne. reclen) call fclean(unit)
      else
        status = tpwrit (unit,headm,reclen,HANDLE)
        if (status .ne. reclen) call fclean(unit)
      endif

C     Close associated file 
C     ---------------------
      if (flag) close (unit = unithd)

      return
      end

