/******************************************************************************

NAME:	EDTBAT

FUNCTION: EDTBAT is used to edit the contents of the DDR file using TAE 
	  parameters instead of interactively on the terminal.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         10/87       K.Zanter    initial development
    1.1 	07/88	    B.Ailts	LAS 5.0 standards
    1.2         10/89       B.Ailts     Changed last used to last modified
    1.3		08/90       D.Etrheim	Changed to accept upper case projection
					types for -TUTOR.
    1.3		05/92	    S.Nelson	Changed proj. types to be excepted by
					the proper proj. code instead of the
					first part of the name.
					Added map projections Hammer and
					Robinson.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
The existance of the DDR file is checked
If it does exist 
   The DDR file is read into the DDR structures
else
   The DDR structures are initialized
The value of each of the parameters are read from the TAE parameter block
If there was a value specified by the user 
   The new value replaces the old value
else
   The current value left in place
The DDR structures are then written to the DDR file

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "editddr.h"

void edtbat
(
  struct PARBLK *parblk,           /* I: Pointer to TAE parameter block */
  char *hostddr                    /* I: DDR file host name */
)
{ /* edtbatch */
struct DDR ddr;			/* integer data in record 1 		      */
struct BDDR bddr[MAXBND];	/* buffer storage for bddr		      */

struct VARIABLE *var;	        /* pointer to a TAE variable		      */

long    bands[MAXBND];		/* bands array				      */
long    bnd_cnt;		/* number of bands			      */
long	count1;			/* parameter count			      */
long	count2;			/* parameter count			      */
long	ddrvflg;		/* ddr valid flag			      */
long	exist_flag;		/* flag if file exists or not		      */
long	fatal = 0;		/* fatal error code			      */
long	i;			/* loop index				      */
long	len;			/* string length			      */
long    lm_flag = FALSE;	/* last used flag			      */
long	mode;			/* file access mode			      */
long    nbands;			/* no. of input bands			      */
long	nonfatal = 1;		/* nonfatal error code			      */
long	status;			/* function return status code		      */
long    temp;                   /* temporary storage for long variable        */

char    dtype[5];		/* data type of output ddr		      */
char    ddrvf_str[3];		/*  ddr valid flag buffer		      */
char    echo[80];               /* temp. buffer to hold output value          */
char    mmval[4];		/* buffer for the min/max valid flag	      */
char    proj_str[9];            /* project code string                        */

long valid(char *);

/*  Check for the existance of the ddr file
-------------------------------------------*/
exist_flag = TRUE;
mode = 0;
status = c_lsstat(hostddr,&mode);
if (status != E_SUCC)
   {
   c_errmsg("Input file could not be found","editddr-input",&nonfatal);
   c_errmsg("A DDR file will be created","editddr-input",&nonfatal);
   exist_flag = FALSE;
   }

/* Retreive data for records 1 and 2
------------------------------------*/
if (exist_flag)
   {
   status = c_getddr(hostddr,&ddr);  
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }
else
   c_intddr(&ddr);

/* Get number of bands
----------------------*/
var = p_find(parblk,"nbands");
if ((*var).v_count != 0)
   {
   if (!exist_flag)
      ddr.nbands = IVAL(*var,0);  /*  set the output ddr.nbands to NBANDS */
   else
      {
      nbands = ddr.nbands;  /*  set the nbands to the input no. of bands  */
      ddr.nbands = IVAL(*var,0);  /* set the output nbands to NBANDS */
      }
   }
else  /* if nbands is -- null then output nbands is equal to input nbands  */
   nbands = ddr.nbands;

if ((!exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg("No. of bands is required when a DDR is being created","ddr-create",
	    &nonfatal);
   c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }

/*  Initialize the band ddr fields
----------------------------------*/
if (!exist_flag)
   {
   for (i = 0; i < ddr.nbands; i++)
      c_intbdr(&bddr[i]);
   }
else
   {
   for (i = 0; i < nbands; i++)
      {
      bands[i] = i + 1;
      status = c_getbdr(hostddr,&bddr[i],&bands[i]);
      if (status != E_SUCC)
         c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   /*  if output nbands is greater than input nbands, bddr.bandno of new bands
       must be initialized to the correct band numbers
   ---------------------------------------------------------------------------*/
   for (i = nbands; i < ddr.nbands; i++)
      bddr[i].bandno = i + 1;
   }

/* Get global parameter $DDRVFLG
--------------------------------*/
var = p_find(parblk,"$DDRVFLG");
strcpy (ddrvf_str,SVAL(*var,0));
if (strcmp(ddrvf_str,"IN") == 0)
   ddrvflg = 0;
else if (strcmp(ddrvf_str,"VAL") == 0)
   ddrvflg = 1;
else 
   ddrvflg = 2;

/* Get parameters (NL,NS,DTYPE)
-------------------------------*/
var = p_find(parblk,"nl");
if ((*var).v_count != 0)
   ddr.nl = IVAL(*var,0);
if ((!exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg("No. of lines is required when a DDR is being created","ddr-create",
	    &nonfatal);
   c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }

var = p_find(parblk,"ns");
if ((*var).v_count != 0)
   ddr.ns = IVAL(*var,0);

if ((!exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg("No. of samples is required when a DDR is being created",
	    "ddr-create", &nonfatal);
   c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }

var = p_find(parblk,"odtype");
if ((*var).v_count != 0)
   {
   strcpy(dtype,SVAL(*var,0));
   if (strcmp(dtype,"BYTE") == 0)
      ddr.dtype = EBYTE;
   else if (strcmp(dtype,"I*2") == 0)
      ddr.dtype = EWORD;
   else if (strcmp(dtype,"I*4") == 0)
      ddr.dtype = ELONG;
   else if (strcmp(dtype,"R*4") == 0)
      ddr.dtype = EREAL;
   }

if ((!exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg("Data type is required when a DDR is being created","ddr-create",
	    &nonfatal);
   c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }

/* Get master line and sample parameters and set corresponding validity 
   flag if needed
-----------------------------------------------------------------------*/ 
var = p_find(parblk,"mline");
count1 = (*var).v_count;
if (count1 != 0)
   ddr.master_line = IVAL(*var,0);

var = p_find(parblk,"msamp");
count2 = (*var).v_count;
if (count2 != 0)
   ddr.master_sample = IVAL(*var,0);

/* Get projection type and corresponding validity flag
------------------------------------------------------*/
var = p_find(parblk,"projtype");
if ((*var).v_count != 0)
   {
   strcpy (proj_str,SVAL(*var,0));
   len = (long)strlen(proj_str);
   c_up2low(proj_str,&len);
   if (strncmp(proj_str,"geo",3) == 0)
      ddr.proj_code=0;
   else if (strncmp(proj_str,"u",1) == 0)
      ddr.proj_code = 1;
   else if (strncmp(proj_str,"spc",3)==0) 
      ddr.proj_code = 2;
   else if (strncmp(proj_str,"alber",5) == 0) 
      ddr.proj_code = 3;
   else if ((strncmp(proj_str,"lambert c",9) == 0) || 
            (strncmp(proj_str,"lamc",4) == 0))
      ddr.proj_code = 4;
   else if (strncmp(proj_str,"me",2) == 0) 
      ddr.proj_code = 5;
   else if (strncmp(proj_str,"ps",2)==0)
      ddr.proj_code = 6;
   else if (strncmp(proj_str,"poly",4) == 0) 
      ddr.proj_code = 7;
   else if (strncmp(proj_str,"equid",5) == 0) 
      ddr.proj_code = 8;
   else if (strncmp(proj_str,"t",1) == 0) 
      ddr.proj_code = 9;
   else if (strncmp(proj_str,"ste",3) == 0) 
      ddr.proj_code = 10;
   else if ((strncmp(proj_str,"lambert a",9) == 0) || 
            (strncmp(proj_str,"lama",4) == 0))
      ddr.proj_code = 11;
   else if (strncmp(proj_str,"az",2) == 0)
      ddr.proj_code = 12;
   else if (strncmp(proj_str,"gn",2) == 0)
      ddr.proj_code = 13;
   else if (strncmp(proj_str,"or",2) == 0)
      ddr.proj_code = 14;
   else if (strncmp(proj_str,"gv",2) == 0) 
      ddr.proj_code = 15;
   else if (strncmp(proj_str,"sn",2) == 0) 
      ddr.proj_code = 16;
   else if (strncmp(proj_str,"eqr",3) == 0)
      ddr.proj_code = 17;
   else if (strncmp(proj_str,"mi",2) == 0)
      ddr.proj_code = 18;
   else if (strncmp(proj_str,"v",1) == 0) 
      ddr.proj_code = 19;
   else if (strncmp(proj_str,"hom",3) == 0)
      ddr.proj_code = 20;
   else if (strncmp(proj_str,"ro",2) == 0)
      ddr.proj_code = 21;
   else if (strncmp(proj_str,"so",2) == 0)
      ddr.proj_code = 22;
   else if (strncmp(proj_str,"alask",5) == 0)
      ddr.proj_code = 23;
   else if (strncmp(proj_str,"good",4) == 0)
      ddr.proj_code = 24;
   else if (strncmp(proj_str,"moll",4) == 0)
      ddr.proj_code = 25;
   else if (strncmp(proj_str,"imoll",5) == 0)
      ddr.proj_code = 26;
   else if (strncmp(proj_str,"ha",2) == 0)
      ddr.proj_code = 27;
   else if (strncmp(proj_str,"wagiv",5) == 0)
      ddr.proj_code = 28;
   else if (strncmp(proj_str,"wagvii",6) == 0)
      ddr.proj_code = 29;
   else if (strncmp(proj_str,"obeqa",5) == 0)
      ddr.proj_code = 30;
   else
     {
     sscanf(proj_str,"%2d",&temp);
     if ((temp < 0) || (temp > 99))
         {
         c_errmsg("Invalid projection code","editddr-projcode",&fatal);
         status = E_FAIL;
         }
      else
	 {
         strcpy(echo,proj_str);
         ddr.proj_code = temp;
         }
      }
     }
var = p_find(parblk,"ptval");
if ((*var).v_count != 0)
   ddr.valid[DDPCV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDPCV] = ddrvflg;

/* Get zone code and corresponding validity flag
------------------------------------------------*/
var = p_find(parblk,"zonenum");
if ( (*var).v_count != 0)
   ddr.zone_code = IVAL(*var,0);

var = p_find(parblk,"znval");
if ((*var).v_count != 0)
   ddr.valid[DDZCV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDZCV] = ddrvflg;

/* Get datum code and corresponding validity flag
-------------------------------------------------*/
var = p_find(parblk,"dcode");
if ( (*var).v_count != 0)
   ddr.datum_code = IVAL(*var,0);

var = p_find(parblk,"dcval");
if ( (*var).v_count != 0)
   ddr.valid[DDDCV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDDCV] = ddrvflg;

/* Get projection coefficients and corresponding validity flag
--------------------------------------------------------------*/
var = p_find(parblk,"projparm");
if ( (*var).v_count != 0)
   {
   for (i = 0; i < (*var).v_count ; i++)
       sscanf(SVAL(*var,i),"%lf",ddr.proj_coef + i);
   }

var = p_find(parblk,"ppval");
if ( (*var).v_count != 0)
   ddr.valid[DDPPV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDPPV] = ddrvflg;

/* Get four corner coordinates and corner validity flag if needed
-----------------------------------------------------------------*/
var = p_find(parblk,"upleft");
count1 = (*var).v_count;
if (count1 != 0)
   for (i = 0; i < 2; i++)
       sscanf(SVAL(*var,i),"%lf",ddr.upleft+i);

var = p_find(parblk,"loleft");
if ( (*var).v_count != 0)
   for (i = 0; i < 2; i++)
       sscanf(SVAL(*var,i),"%lf",ddr.loleft+i);

var = p_find(parblk,"upright");
if ( (*var).v_count != 0)
   for (i = 0; i < 2; i++)
       sscanf(SVAL(*var,i),"%lf",ddr.upright+i);

var = p_find(parblk,"loright");
if ( (*var).v_count != 0)
   for (i = 0; i < 2; i++)
       sscanf(SVAL(*var,i),"%lf",ddr.loright+i);

var = p_find(parblk,"ccval");
if ( (*var).v_count != 0)
   ddr.valid[DDCCV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDCCV] = ddrvflg;

/* Get ground distance in line and sample direction and get corresponding 
   validity flag if necessary
-------------------------------------------------------------------------*/
var = p_find(parblk,"pdline");
if ((*var).v_count != 0)
   sscanf(SVAL(*var,0),"%lf",&(ddr.pdist_y));   

var = p_find(parblk,"pdsamp");
if ((*var).v_count != 0)
   sscanf(SVAL(*var,0),"%lf",&(ddr.pdist_x));

var = p_find(parblk,"pdval");
if ( (*var).v_count != 0)
   ddr.valid[DDPDV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDPDV] = ddrvflg;

/* Get ground measurement units and set corresponding validity flag
-------------------------------------------------------------------*/
var = p_find(parblk,"punits");
if ( (*var).v_count != 0)
   strcpy(ddr.proj_units,SVAL(*var,0));

var = p_find(parblk,"puval");
if ( (*var).v_count != 0)
   ddr.valid[DDPUV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDPUV] = ddrvflg;

/* Get line and sample increments and set corresponding validity flag if needed
------------------------------------------------------------------------------*/
var = p_find(parblk,"lineinc");
if ((*var).v_count != 0)
   sscanf(SVAL(*var,0),"%lf",&(ddr.line_inc));

var = p_find(parblk,"sampinc");
if ((*var).v_count != 0)
   sscanf(SVAL(*var,0),"%lf",&(ddr.sample_inc));

var = p_find(parblk,"incval");
if ( (*var).v_count != 0)
   ddr.valid[DDINCV] = valid(SVAL(*var,0));
else
   if (!exist_flag)
      ddr.valid[DDINCV] = ddrvflg;

/* Get last used date
---------------------*/
var = p_find(parblk,"lmdate");
if ((*var).v_count != 0)
   strcpy(ddr.last_used_date,SVAL(*var,0));
else
   if (!exist_flag)
      lm_flag = TRUE;

/* Get last used time
---------------------*/
var = p_find(parblk,"lmtime");
if ((*var).v_count != 0)
   strcpy(ddr.last_used_time,SVAL(*var,0));
else
   if (!exist_flag)
      if (lm_flag)
         lm_flag = TRUE;

/* Get system data is on
------------------------*/
var = p_find(parblk,"system");
if ((*var).v_count != 0)
   strcpy(ddr.system,SVAL(*var,0));
else
   if (!exist_flag)
      c_getsys(ddr.system);

/* *************************Band Dependent Data ***************************** */
   
/* Place the band no in the bddr structure
------------------------------------------*/
var = p_find(parblk,"bands");
bnd_cnt = (*var).v_count;
if (bnd_cnt != 0)
   if (!exist_flag)
      {
      for (i = 0; i < bnd_cnt; i++)
         {
         bands[i] = IVAL(*var,i);
	 if (bands[i] > ddr.nbands)
	    {
	    c_errmsg("Invalid band number","editddr-band",&nonfatal);
	    c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
	    }
	 }
      /*  Fill in the bddr.bandno array with band number not specified in BANDS
      ------------------------------------------------------------------------*/
      for (i = 0; i < ddr.nbands; i++)
         {
         bddr[i].bandno = i + 1;
         }
      } /*  if not exist  */
   else
      for (i = 0; i < bnd_cnt; i++)
         bands[i] = IVAL(*var,i);

else
   if (!exist_flag)
      for (i = 0; i < ddr.nbands; i++)
         {
         bddr[i].bandno = i + 1;
         bands[i] = i + 1;
	 }

/* Get minimum values
---------------------*/
var = p_find(parblk,"minval");
count1 = (*var).v_count;
if (count1 != 0)
   {
   if ((count1 == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
 	  sscanf(SVAL(*var,0),"%lf",&(bddr[i].minval));
   else if (count1 == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
          sscanf(SVAL(*var,i),"%lf",&(bddr[bands[i] - 1].minval));
   else
      {
      c_errmsg("Invalid number of values entered for MINVAL",
              "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }

/* Get maximum values
---------------------*/
var = p_find(parblk,"maxval");
count2 = (*var).v_count;
if (count2 != 0)
   {
   if ((count2 == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
          sscanf(SVAL(*var,0),"%lf",&(bddr[i].maxval));
   else if (count2 == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
          sscanf(SVAL(*var,i),"%lf",&(bddr[bands[i] - 1].maxval));
   else
      {
      c_errmsg("Invalid number of values entered for MAXVAL",
 	      "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }
      
/* Get min/max valid values
---------------------------*/
var = p_find(parblk,"mmval");
count1 = (*var).v_count;
if (count1 != 0)
   {
   if ((count1 == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
          {
 	  strcpy(mmval,SVAL(*var,0));
	  if (strcmp(mmval,"IN") == 0)
	     bddr[i].valid = INVAL;
	  else if (strcmp(mmval,"VAL") == 0)
	     bddr[i].valid = VALID;
 	  else
	     bddr[i].valid = BOUND;
          }
   else if (count1 == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
          {
 	  strcpy(mmval,SVAL(*var,0));
	  if (strcmp(mmval,"IN") == 0)
	     bddr[bands[i] - 1].valid = 0;
	  else if (strcmp(mmval,"VAL") == 0)
	     bddr[bands[i] - 1].valid = 1;
 	  else
	     bddr[bands[i] - 1].valid = 2;
          }
   else
      {
      c_errmsg("Invalid number of values entered for MINVAL",
              "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }

/* Get data source
------------------*/
var = p_find(parblk,"dsource");
if ( (*var).v_count != 0)
   {
   if (((*var).v_count == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
          strcpy(bddr[i].source,SVAL(*var,0));
   else if ( (*var).v_count == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
   	  strcpy(bddr[bands[i] - 1].source,SVAL(*var,i));
   else
      {
      c_errmsg("Invalid number of values entered for DSOURCE",
	      "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }

/* Get sensor type
------------------*/
var = p_find(parblk,"sensor");
if ( (*var).v_count != 0)
   {
   if (((*var).v_count == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
          strcpy(bddr[i].instrument,SVAL(*var,0));
   else if ( (*var).v_count == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
          strcpy(bddr[bands[i] - 1].instrument,SVAL(*var,i));
   else
      {
      c_errmsg("Invalid number of values entered for SENSOR",
	      "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }

/* Get capture process
----------------------*/
var = p_find(parblk,"capdir");
if ( (*var).v_count != 0)
   {
   if (((*var).v_count == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
          strcpy(bddr[i].direction,SVAL(*var,0));
   else if ( (*var).v_count == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
	  strcpy(bddr[bands[i] - 1].direction,SVAL(*var,i));
   else
      {
      c_errmsg("Invalid number of values entered for CAPDIR",
	      "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }

/* Get capture date
-------------------*/
var = p_find(parblk,"cdate");
if ( (*var).v_count != 0)
   {  
   if (((*var).v_count == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
          strcpy(bddr[i].date,SVAL(*var,0));
   else if ( (*var).v_count == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
          strcpy(bddr[bands[i] - 1].date,SVAL(*var,i));
   else
      {
      c_errmsg("Invalid number of values entered for CDATE",
	      "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }

/* Get capture time
-------------------*/
var = p_find(parblk,"ctime");
if ( (*var).v_count != 0)
   {
   if (((*var).v_count == 1) && (bnd_cnt == 0))
      for (i = 0; i < ddr.nbands; i++)
          strcpy(bddr[i].time,SVAL(*var,0));
   else if ( (*var).v_count == bnd_cnt)
      for (i = 0; i < bnd_cnt; i++)
          strcpy(bddr[bands[i] - 1].time,SVAL(*var,i));
   else
      {
      c_errmsg("Invalid number of values entered for CTIME",
	      "editddr-count",&nonfatal);
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
      }
   }

/* output info to file
----------------------*/
status = writeddr(hostddr,&ddr,bddr);
if (status != E_SUCC)
   {
   c_errmsg("Error writing to DDR file","editddr-write",&nonfatal);
   c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }

if (lm_flag)
   c_lused(hostddr);

}

/******************************************************************************
NAME: VALID

PURPOSE: To return the correct valid flag based on the input str.

ALGORITHM:
 If val_str = IN
     return INVALID
 Else if val_str = VAL
     return VALID
 Else
     return UNKNOWN
******************************************************************************/
long valid
(
  char *val_str           /* Input valid string */
)
{
  long val_int;

  if (strcmp(val_str,"IN") == 0)
     val_int = INVAL;
  else if (strcmp(val_str,"VAL") == 0)
     val_int = VALID;
  else
     val_int = UNKNOW;

  return(val_int);
}

