/******************************************************************************

NAME:	EDTDDR

FUNCTION: EDTDDR is used to edit the contents of the DDR file interactively on 
          the terminal.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         10/87      B.Ailts      initial development
    1.1		07/88	   B.Ailts	LAS 5.0 standards
    1.2		12/88      B.Ailts	fixed parsing of production codes
    1.3         10/89      B.Ailts	Changed last used to last modified
    1.4		08/90      D.Etrheim	ADP #5512  Changed to accept the
					projection name "utm" 
    1.5		05/92	   S.Nelson	Changed the proj. type to except the
					appropriate key name.
    1.6		06/92	   S.Nelson	Added map projections Hammer and
					Robinson.
    1.7		10/09      D.Etrheim    Modified to allow negative zone codes.
					Needed for UTM below the equator.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
This function compares the edit command code with the codes of each of 
 the ddr fields
When the correct code is found
   the correct field of the DDR is updated with the new value 
   the new value is output to the correct place on the terminal
   Validity checking is done when possible.  If an incorrect code is input, 
     an error message is output.

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "geompak.h"
#include "editddr.h"

int edtddr
(
  char *edans_buf,                 /* I: Pointer to edit command code */
  char *taeout,                    /* O: TAE output image name */
  char *hosout,                    /* I: Host ddr name */
  long *line,                      /* I: Number of lines on the screen */
  struct DDR *ddr,                 /* I: Integer portion of ddr */
  struct BDDR *bddr,               /* O: BDDR of current band */
  long *edit_flag                  /* O: Edit flag */
)
{  /* edtddr */

long status;				/* status of called routines	*/
long temp;				/* temporary storage for long var*/
long zero = 0;				/* value of 0			  */
long i;                                 /* Loop index counter             */

char *eqptr;				/* temporary pointer to command code*/
char blank5[6];				/* buffer to blank out 5 spaces	*/
char blank10[11];			/* buffer to blank out 10 spaces*/
char blank12[13];			/* buffer to blank out 12 spaces*/
char blank23[24];			/* buffer to blank out 23 spaces*/
char blank40[41];			/* buffer to blank out 40 spaces*/
char blank45[46];			/* buffer to blank out 45 spaces*/
char blank60[61];			/* buffer to blank out 60 spaces*/
char blank70[71];			/* buffer to blank out 70 spaces*/
char echo[80];				/* temp. buffer to hold output value*/
char *tae;				/* temp. buffer of tae image name  */
char *hname;				/* temp. buffer of host image name */

void validate(char *,char *,long *,long *);
void valmm(char *,char *,long *,long *);
int start_of_line(long *,char **,char *);

/*  Initialize the blank strings 
--------------------------------*/
strcpy(blank5,"     ");
strcpy(blank10,"          ");
strcpy(blank12,"            ");
strcpy(blank23,"                       ");
strcpy(blank40,"                                        ");
strcpy(blank45,"                                             ");
strcpy(blank60,"                                                            ");
strcpy(blank70,
"                                                                      ");

/*  Initialize variables
------------------------*/
status = E_SUCC;

/*  Set temporary pointer to edit command buffer 
------------------------------------------------*/
eqptr = edans_buf;

/*  Find the start of the new value (1st non-space letter past the equal sign)
------------------------------------------------------------------------------*/
status = start_of_line(line,&eqptr,edans_buf);
if (status != E_SUCC)
   return(E_FAIL);

/*  Compare the edit command code with each of the known codes of the ddr 
fields.  When the correct code is found change the value of that field
and output the new value to the correct place on the terminal.  Make
sure that the place on the terminal is blanked out first to remove any
artifacts of the previous value.
-------------------------------------------------------------------------*/
if (strncmp(edans_buf,"in",2) == 0)
   {
   strcpy(echo,eqptr);
   tae = (char *)malloc(CMLEN);
   hname = (char *)malloc(CMLEN);
   status = c_mkimg(echo,"",tae,hname,&zero);
   if (status != E_SUCC)
      t_output(*line - 2,2,"[editddr-image] Invalid image name");
   else
      {
      c_lsmknm(hname,"ddr",hosout);
      t_output(1,15,blank70);
      t_output(1,15,echo);
      strcpy(taeout,eqptr);
      }
   free(tae);
   free(hname);
   }  /* if 'in' or 'IN' */

else if (strncmp(edans_buf,"nl",2)== 0)
   {
   temp = ddr->nl;
   sscanf(eqptr,"%d",&(ddr->nl));
   if (ddr->nl < 1)
      {
      ddr->nl = temp;
      t_output(*line - 2,2,"[editddr-nline] Invalid number of lines");
      status = E_FAIL;
      }
   else
      {
      strcpy(echo,eqptr);
      t_output(2,15,blank5);
      t_output(2,15,echo);
      }
   }  /* if 'nl' */
else if (strncmp(edans_buf,"ns",2)== 0)
   {
   temp = ddr->ns;
   sscanf(eqptr,"%d",&(ddr->ns));
   if (ddr->ns < 1)
      {
      ddr->ns = temp;
      t_output(*line -2,2,"[editddr-nsamp] Invalid number of samples");
      status = E_FAIL;
      }
   else
      {
      strcpy(echo,eqptr);
      t_output(2,32,blank5);
      t_output(2,32,echo);
      }
   }  /* if 'ns' */
else if (strncmp(edans_buf,"nb",2)== 0)
   {
   temp = ddr->nbands;
   sscanf(eqptr,"%d",&(ddr->nbands));

   /* If input nbands is < output nbands, bddr.bandno of new
      bands must be initialized to the correct band numbers
   ---------------------------------------------------------*/
   for (i = temp; i < ddr->nbands; i++)
     bddr[i].bandno = i + 1;

   if ((ddr->nbands < 1) || (ddr->nbands > MAXBND))/* valid check */
      {
      ddr->nbands = temp;
      t_output(*line - 2,2,"[editddr-nband] Invalid number of bands");
      status = E_FAIL;
      }
   else
      {
      strcpy(echo,eqptr);
      t_output(2,50,blank5);
      t_output(2,50,echo);
      } /* if valid */
   }  /* if 'nb' */

else if (strncmp(edans_buf,"dt",2)== 0)
   {
   /*  Output the correct string based on the new data type value
   --------------------------------------------------------------*/
   if (strncmp(eqptr,"b",1) == 0)
      {
      ddr->dtype = 1;
      strcpy(echo,"BYTE");
      t_output(2,67,blank10);
      t_output(2,67,echo);
      }
   else if (strncmp(eqptr,"i*2",9) == 0)
      {
      ddr->dtype = 2;
      strcpy(echo,"INTEGER*2");
      t_output(2,67,blank10);
      t_output(2,67,echo);
      }
   else if (strncmp(eqptr,"i*4",9) == 0)
      {
      ddr->dtype = 3;
      strcpy(echo,"INTEGER*4");
      t_output(2,67,blank10);
      t_output(2,67,echo);
      }
   else if (strncmp(eqptr,"r",1) == 0)
      {
      ddr->dtype = 4;
      strcpy(echo,"REAL*4");
      t_output(2,67,blank10);
      t_output(2,67,echo);
      }
   else 
      {
      t_output(*line - 2,2,"[editddr-type] Invalid data type specified");
      status = E_FAIL;
      }
   }  /* if 'dt' */
else if (strncmp(edans_buf,"lmd",3) == 0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 11)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(3,32,blank10);
      t_output(3,32,echo);
      strcpy(ddr->last_used_date,eqptr);
      }
   }  /* if 'lud' */

else if (strncmp(edans_buf,"lmt",3) == 0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 11)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(3,50,blank10);
      t_output(3,50,echo);
      strcpy(ddr->last_used_time,eqptr);
      }
   }  /* if 'lut'  */

else if (strncmp(edans_buf,"sy",2)==0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 11)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(3,68,blank10);
      t_output(3,68,echo);
      strcpy(ddr->system,eqptr);
      }
   }  /* if 'sy'  */

else if (strncmp(edans_buf,"pcv",3)== 0)
   {
   validate(eqptr,echo,&(ddr->valid[DDPCV]),line);
   t_output(4,67,blank12);
   t_output(4,67,echo);
   }  /* if 'pcv'  */

else if (strncmp(edans_buf,"pc",2)== 0)
   {
   t_output(4,15,blank40);

   /*  output the correct projection based on the new projection code
   ------------------------------------------------------------------*/
   if ((strncmp(eqptr,"geo",3) == 0) || (strncmp(eqptr,"0",1) == 0))
      {
      strcpy(echo,"(0)GEOGRAPHIC");  
      ddr->proj_code=0;
      }
   else if ((strncmp(eqptr,"u",1) == 0) || (strcmp(eqptr,"1") == 0))
      {
      strcpy(echo,"(1)UTM");  
      ddr->proj_code = 1;
      }
   else if ((strncmp(eqptr,"spc",3)==0) || (strcmp(eqptr,"2") ==0))
      {
      strcpy(echo,"(2)STATE PLANE");  
      ddr->proj_code = 2;
      }
   else if ((strncmp(eqptr,"alber",4) == 0) || (strcmp(eqptr,"3") ==0))
      {
      strcpy(echo,"(3)ALBERS CONICAL EQUAL AREA");  
      ddr->proj_code = 3;
      }
   else if ((strncmp(eqptr,"lambert c",9) == 0) || 
            (strncmp(eqptr,"lamc",4) == 0) || (strcmp(eqptr,"4") == 0))
      {
      strcpy(echo,"(4)LAMBERT CONFORMAL CONIC");  
      ddr->proj_code = 4;
      }
   else if ((strncmp(eqptr,"me",2) == 0) || (strcmp(eqptr,"5") == 0))
      {
      strcpy(echo,"(5)MERCATOR");  
      ddr->proj_code = 5;
      }
   else if ((strncmp(eqptr,"ps",2) == 0) || (strcmp(eqptr,"6") == 0))
      {
      strcpy(echo,"(6)POLAR STEREOGRAPHIC");  
      ddr->proj_code = 6;
      }
   else if ((strncmp(eqptr,"poly",4) == 0) || (strcmp(eqptr,"7") == 0))
      {
      strcpy(echo,"(7)POLYCONIC");  
      ddr->proj_code = 7;
      }
   else if ((strncmp(eqptr,"equid",5) == 0) || (strcmp(eqptr,"8") == 0))
      {
      strcpy(echo,"(8)EQUIDISTANT CONIC");  
      ddr->proj_code = 8;
      }
   else if ((strncmp(eqptr,"t",1) == 0) || (strcmp(eqptr,"9") == 0))
      {
      strcpy(echo,"(9)TRANSVERSE MERCATOR");  
      ddr->proj_code = 9;
      }
   else if ((strncmp(eqptr,"ste",3)==0) || (strncmp(eqptr,"10",2) == 0))
      {
      strcpy(echo,"(10)STEREOGRAPHIC");  
      ddr->proj_code = 10;
      }
   else if ((strncmp(eqptr,"lambert a",9) == 0) || 
            (strncmp(eqptr,"lama",4) == 0) || (strncmp(eqptr,"11",2) == 0))
      {
      strcpy(echo,"(11)LAMBERT AZIMUTHAL EQUAL AREA");  
      ddr->proj_code = 11;
      }
   else if ((strncmp(eqptr,"az",2) == 0) || (strncmp(eqptr,"12",2) == 0))
      {
      strcpy(echo,"(12)AZIMUTHAL EQUIDISTANT");  
      ddr->proj_code = 12;
      }
   else if ((strncmp(eqptr,"gn",2) == 0) || (strncmp(eqptr,"13",2) == 0))
      {
      strcpy(echo,"(13)GNOMONIC");  
      ddr->proj_code = 13;
      }
   else if ((strncmp(eqptr,"or",2) == 0) || (strncmp(eqptr,"14",2) == 0))
      {
      strcpy(echo,"(14)ORTHOGRAPHIC");  
      ddr->proj_code = 14;
      }
   else if ((strncmp(eqptr,"gvn",3) == 0) || (strncmp(eqptr,"15",2) == 0))
      {
      strcpy(echo,"(15)GENERAL VERTICAL NEAR-SIDE PERSPECTIVE"); 
      ddr->proj_code = 15;
      }
   else if ((strncmp(eqptr,"sn",2) == 0) || (strncmp(eqptr,"16",2) == 0))
      {
      strcpy(echo,"(16)SINUSOIDAL");  
      ddr->proj_code = 16;
      }
   else if ((strncmp(eqptr,"eqr",3) == 0) || (strncmp(eqptr,"17",2) == 0))
      {
      strcpy(echo,"(17)EQUIRECTANGULAR");  
      ddr->proj_code = 17;
      }
   else if ((strncmp(eqptr,"mi",2)==0) || (strncmp(eqptr,"18",2) == 0))
      {
      strcpy(echo,"(18)MILLER CYLINDRICAL");  
      ddr->proj_code = 18;
      }
   else if ((strncmp(eqptr,"v",1)==0) || (strncmp(eqptr,"19",2) == 0))
      {
      strcpy(echo,"(19)VAN DER GRINTEN");  
      ddr->proj_code = 19;
      }
   else if ((strncmp(eqptr,"ho",2)==0) || (strncmp(eqptr,"20",2) == 0))
      {
      strcpy(echo,"(20)OBLIQUE MERCATOR (Hotine)");  
      ddr->proj_code = 20;
      }
   else if ((strncmp(eqptr,"ro",2)==0) || (strncmp(eqptr,"21",2) == 0))
      {
      strcpy(echo,"(21)ROBINSON");  
      ddr->proj_code = 21;
      }
   else if ((strncmp(eqptr,"so",2)==0) || (strncmp(eqptr,"22",2) == 0))
      {
      strcpy(echo,"(22)SOM");  
      ddr->proj_code = 22;
      }
   else if ((strncmp(eqptr,"alaska",5)==0) || (strncmp(eqptr,"23",2) == 0))
      {
      strcpy(echo,"(23)ALASKA CONFORMAL");  
      ddr->proj_code = 23;
      }
   else if ((strncmp(eqptr,"good",4)==0) || (strncmp(eqptr,"24",2) == 0))
      {
      strcpy(echo,"(24)INTERRUPTED GOODE HOMOLOSINE");  
      ddr->proj_code = 24;
      }
   else if ((strncmp(eqptr,"moll",4)==0) || (strncmp(eqptr,"25",2) == 0))
      {
      strcpy(echo,"(25)MOLLWEIDE");  
      ddr->proj_code = 25;
      }
   else if ((strncmp(eqptr,"imoll",5)==0) || (strncmp(eqptr,"26",2) == 0))
      {
      strcpy(echo,"(26)INTERRUPTED MOLLWEIDE");  
      ddr->proj_code = 26;
      }
   else if ((strncmp(eqptr,"ro",2)==0) || (strncmp(eqptr,"27",2) == 0))
      {
      strcpy(echo,"(27)HAMMER");  
      ddr->proj_code = 27;
      }
   else if ((strncmp(eqptr,"wagiv",5)==0) || (strncmp(eqptr,"28",2) == 0))
      {
      strcpy(echo,"(28)WAGNER IV");  
      ddr->proj_code = 28;
      }
   else if ((strncmp(eqptr,"wagvii",6)==0) || (strncmp(eqptr,"29",2) == 0))
      {
      strcpy(echo,"(29)WAGNER VII");  
      ddr->proj_code = 29;
      }
   else if ((strncmp(eqptr,"obeqa",5)==0) || (strncmp(eqptr,"30",2) == 0))
      {
      strcpy(echo,"(30)OBLATED EQUAL AREA");  
      ddr->proj_code = 30;
      }
   else
      {
      sscanf(eqptr,"%2d",&temp);
      if ((temp < 0) || (temp > 99))
         {
         t_output(*line - 2,2,"[editddr-projcode] Invalid projection code");
         status = E_FAIL;
         }
      else
	 {
         strcpy(echo,eqptr);
         ddr->proj_code = temp;
         }
      }
   t_output(4,15,echo);
   }  /* if 'pc' */

else if (strncmp(edans_buf,"zcv",3) == 0)
   {
   validate(eqptr,echo,&(ddr->valid[DDZCV]),line);
   t_output(5,67,blank12);
   t_output(5,67,echo);
   }  /* if 'zcv'  */

else if (strncmp(edans_buf,"zc",2)== 0)
   {
   temp = ddr->zone_code;
   strcpy(echo,eqptr);
   sscanf(eqptr,"%d",&(ddr->zone_code));
   if ((ddr->zone_code < -60) || (ddr->zone_code > 5400))
      { /* Invalid zone code */
      t_output(*line - 2,2,"[editddr-zone] Invalid zone code");
      ddr->zone_code = temp;
      status = E_FAIL;
      } /* Invalid zone code */
   else
      { /* valid zone code */
      t_output(5,15,blank5);
      t_output(5,15,echo);
      }
   }  /* if 'zc' */

else if (strncmp(edans_buf,"dcv",3)== 0)
   {  /* if 'dcv' or 'DCV' */
   validate(eqptr,echo,&(ddr->valid[DDDCV]),line);
   t_output(6,67,blank12);
   t_output(6,67,echo);
   }  /* if 'dcv' */

else if (strncmp(edans_buf,"dc",2)== 0)
   {
   temp = ddr->datum_code;
   strcpy(echo,eqptr);
   sscanf(eqptr,"%d",&(ddr->datum_code));
   if ((ddr->datum_code < 0) || (ddr->datum_code > MAX_DATUM))
      {
      t_output(*line - 2,2,"[editddr-datum] Invalid datum code");
      ddr->datum_code = temp;
      status = E_FAIL;
      }
   else
      { /* valid datum code */
      c_dtmstr(&(ddr->datum_code),echo);
      t_output(6,15,blank45);
      t_output(6,15,echo);
      } /* valid datum code */
   }  /* if 'dc' */

else if (strncmp(edans_buf,"ppv",3)== 0)
   {
   validate(eqptr,echo,&(ddr->valid[DDPPV]),line);
   t_output(7,67,blank12);
   t_output(7,67,echo);
   }  /* if 'ppv'  */

else if (strncmp(edans_buf,"ppa",3)==0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(8,9,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[0]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[0]);
      t_output(8,9,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(8,32,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[1]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[1]);
      t_output(8,32,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(8,55,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[2]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[2]);
      t_output(8,55,echo);
      }
   else
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      {
      t_output(8,9,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(ddr->proj_coef[0]),
             &(ddr->proj_coef[1]),&(ddr->proj_coef[2]));
      sprintf(echo,"%-1.14lE   %-1.14lE   %-1.14lE",ddr->proj_coef[0],
              ddr->proj_coef[1],ddr->proj_coef[2]);
      t_output(8,9,echo);
      } 
   }  /* if 'ppa' */

else if (strncmp(edans_buf,"ppb",3)==0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(9,9,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[3]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[3]);
      t_output(9,9,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(9,32,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[4]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[4]);
      t_output(9,32,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(9,55,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[5]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[5]);
      t_output(9,55,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(9,9,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(ddr->proj_coef[3]),
             &(ddr->proj_coef[4]),&(ddr->proj_coef[5]));
      sprintf(echo,"%-1.14lE   %-1.14lE   %-1.14lE",ddr->proj_coef[3],
              ddr->proj_coef[4],ddr->proj_coef[5]);
      t_output(9,9,echo);
      }
   }  /* if 'ppb' */

else if (strncmp(edans_buf,"ppc",3)==0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(10,9,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[6]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[6]);
      t_output(10,9,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(10,32,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[7]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[7]);
      t_output(10,32,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(10,55,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[8]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[8]);
      t_output(10,55,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(10,9,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(ddr->proj_coef[6]),
             &(ddr->proj_coef[7]),&(ddr->proj_coef[8]));
      sprintf(echo,"%-1.14lE   %-1.14lE   %-1.14lE",ddr->proj_coef[6],
              ddr->proj_coef[7],ddr->proj_coef[8]);
      t_output(10,9,echo);
      }
   }  /* if 'ppc' */

else if (strncmp(edans_buf,"ppd",3)==0)
   {

   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(11,9,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[9]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[9]);
      t_output(11,9,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(11,32,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[10]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[10]);
      t_output(11,32,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(11,55,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[11]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[11]);
      t_output(11,55,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(11,9,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(ddr->proj_coef[9]),
             &(ddr->proj_coef[10]),&(ddr->proj_coef[11]));
      sprintf(echo,"%-1.14lE   %-1.14lE   %-1.14lE",ddr->proj_coef[9],
              ddr->proj_coef[10],ddr->proj_coef[11]);
      t_output(11,9,echo);
      }
   }  /* if 'ppd' */

else if (strncmp(edans_buf,"ppe",3)==0)
   {

   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(12,9,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[12]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[12]);
      t_output(12,9,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(12,32,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[13]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[13]);
      t_output(12,32,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(12,55,blank23);
      sscanf(eqptr,"%lf",&(ddr->proj_coef[14]));
      sprintf(echo,"%-1.14lE",ddr->proj_coef[14]);
      t_output(12,55,echo);
      }
   else

      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      {
      t_output(12,9,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(ddr->proj_coef[12]),
      &(ddr->proj_coef[13]),&(ddr->proj_coef[14]));
      sprintf(echo,"%-1.14lE   %-1.14lE   %-1.14lE",ddr->proj_coef[12],
      ddr->proj_coef[13],ddr->proj_coef[14]);
      t_output(12,9,echo);
      }
   }  /* if 'ppe' */

else if (strncmp(edans_buf,"ccv",3)== 0)
   {
   validate(eqptr,echo,&(ddr->valid[DDCCV]),line);
   t_output(13,67,blank12);
   t_output(13,67,echo);
   }  /* if 'ccv' */

else if ((strncmp(edans_buf,"ul",2) == 0) || (strncmp(edans_buf,"ccul",4) == 0))
   {
   t_output(14,15,blank60);
   sscanf(eqptr,"%lf %lf",&(ddr->upleft[0]),&(ddr->upleft[1]));
   sprintf(echo,"%-1.14lE   %-1.14lE",ddr->upleft[0],
 	   ddr->upleft[1]);
   t_output(14,15,echo);
   }  /* if 'ul' */

else if ((strncmp(edans_buf,"ur",2)==0) || 
         (strncmp(edans_buf,"ccur",4) == 0))
   {
   t_output(15,15,blank60);
   sscanf(eqptr,"%lf %lf",&(ddr->upright[0]),&(ddr->upright[1]));
   sprintf(echo,"%-1.14lE   %-1.14lE",ddr->upright[0],
     ddr->upright[1]);
   t_output(15,15,echo);
   }  /* if 'ur' */

else if ((strncmp(edans_buf,"ll",2) == 0) || 
         (strncmp(edans_buf,"ccll",4) == 0))
   {
   t_output(16,15,blank60);
   sscanf(eqptr,"%lf %lf",&(ddr->loleft[0]),&(ddr->loleft[1]));
   sprintf(echo,"%-1.14lE   %-1.14lE",ddr->loleft[0], ddr->loleft[1]);
   t_output(16,15,echo);
   }  /* if 'll' */

else if ((strncmp(edans_buf,"lr",2) == 0) || 
	 (strncmp(edans_buf,"cclr",4) == 0))
   {
   t_output(17,15,blank60);
   sscanf(eqptr,"%lf %lf",&(ddr->loright[0]),&(ddr->loright[1]));
   sprintf(echo,"%-1.14lE   %-1.14lE",ddr->loright[0], ddr->loright[1]);
   t_output(17,15,echo);
   }  /* if 'lr' */

else if (strncmp(edans_buf,"pdv",3)== 0)
   {
   validate(eqptr,echo,&(ddr->valid[DDPDV]),line);
   t_output(18,67,blank12);
   t_output(18,67,echo);
   }  /* if 'gdv' */

else if (strncmp(edans_buf,"pd",2)==0)
   {
   t_output(18,15,blank40);
   sscanf(eqptr,"%lf %lf",&(ddr->pdist_y), &(ddr->pdist_x));
   sprintf(echo,"%-1.14lE   %-1.14lE",ddr->pdist_y, ddr->pdist_x);
   t_output(18,15,echo);
   }  /* if 'gd' */

else if (strncmp(edans_buf,"puv",3)== 0)
   {
   validate(eqptr,echo,&(ddr->valid[DDPUV]),line);
   t_output(19,67,blank12);
   t_output(19,67,echo);
   }  /* if 'puv' */

else if (strncmp(edans_buf,"pu",2)==0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 11)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(19,15,blank40);
      t_output(19,15,echo);
      strcpy(ddr->proj_units,eqptr);
      }
   }  /* if 'pu' */

else if (strncmp(edans_buf,"icv",3)== 0)
   {
   validate(eqptr,echo,&(ddr->valid[DDINCV]),line);
   t_output(20,67,blank12);
   t_output(20,67,echo);
   }  /* if 'icv'  */

else if (strncmp(edans_buf,"ic",2)==0)
   {
   t_output(20,15,blank40);
   sscanf(eqptr,"%lf %lf",&(ddr->line_inc),&(ddr->sample_inc));
   sprintf(echo,"%-1.14lE   %-1.14lE",ddr->line_inc,ddr->sample_inc);
   t_output(20,15,echo);
   }  /* if 'IC' */

else if (strncmp(edans_buf,"mc",2)== 0)
   {
   sscanf(eqptr,"%d %d",&(ddr->master_line),&(ddr->master_sample));
   sprintf(echo,"%d     %d",ddr->master_line,ddr->master_sample);
   t_output(21,15,blank40);
   t_output(21,15,echo);
   }  /* if 'mc' */

else if (strncmp(edans_buf,"minv",4)== 0) 
   {
   valmm(eqptr,echo,&(bddr->valid),line);
   t_output(4,65,blank12);
   t_output(4,65,echo);
   t_output(5,65,blank12);
   t_output(5,65,echo);
   }  /* if 'minv' */

else if (strncmp(edans_buf,"min",3)==0)
   {
   t_output(4,18,blank12);
   sscanf(eqptr,"%lf",&(bddr->minval));
   sprintf(echo,"%-1.14lE",bddr->minval);
   t_output(4,18,echo);
   }  /* if 'min' */

else if (strncmp(edans_buf,"maxv",4)== 0) 
   {
   valmm(eqptr,echo,&(bddr->valid),line);
   t_output(5,65,blank12);
   t_output(5,65,echo);
   t_output(4,65,blank12);
   t_output(4,65,echo);
   }  /* if 'maxv' */

else if (strncmp(edans_buf,"max",3)==0)
   {
   t_output(5,18,blank12);
   sscanf(eqptr,"%lf",&(bddr->maxval));
   sprintf(echo,"%-1.14lE",bddr->maxval);
   t_output(5,18,echo);
   }  /* if 'max' */

else if (strncmp(edans_buf,"ds",2)==0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 31)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(7,18,blank40);
      t_output(7,18,echo);
      strcpy(bddr->source,eqptr);
      }
   }  /* if 'ds' */

else if (strncmp(edans_buf,"st",2)==0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 31)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(8,18,blank40);
      t_output(8,18,echo);
      strcpy(bddr->instrument,eqptr);
      }
   }  /* if 'st' */

else if (strncmp(edans_buf,"cd",2)==0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 63)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(9,18,blank60);
      t_output(9,18,echo);
      strcpy(bddr->direction,eqptr);
      }
   }  /* if 'cp' */

else if (strncmp(edans_buf,"da",2)==0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 9)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(10,18,blank10);
      t_output(10,18,echo);
      strcpy(bddr->date,eqptr);
      }
   }  /* if 'da' */

else if (strncmp(edans_buf,"ti",2)==0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 7)
      {
      t_output(*line-2,2,"[editddr-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(11,18,blank10);
      t_output(11,18,echo);
      strcpy(bddr->time,eqptr);
      }
   }  /* if 'ti' */
else
   { /* invalid code */
   t_output(*line - 2,2,"[editddr-code] Invalid edit command code");
   t_output(*line,2,blank60);
   status = E_FAIL;
   } /* invalid code */

/*  Set edit flag to TRUE if no error had occured and a field was edited
------------------------------------------------------------------------*/
if (status == E_SUCC)
   *edit_flag = TRUE;

return (E_SUCC);
}


/******************************************************************************
NAME: START_OF_LINE

PROGRAM HISTORY:
Version	Date       Author       Request
-------	----       ------       -------
1.0     11/87      B.Ailts      Initial development
1.1     07/88      B.Ailts      LAS 5.0 standards

ALGORITHM DESCRIPTION:
  START_OF_LINE parses the edit command string 
  Returns the starting position of the character string which is to the
  right of the equal sign(=)
  A non-fatal error is output if the command string does not contain an
  equal sign
******************************************************************************/
int start_of_line
(
  long *line,                 /* Number of lines on the terminal */
  char **eqptr,               /* Address of the str. right of = */
  char *string                /* Address of command string */
)
{
  /* Locate the address of the equal
     sign in the command string
  ----------------------------------*/
  *eqptr = strchr(string,'=');

  /* Output non-fatal error if equal sign is not found
  ----------------------------------------------------*/
  if (*eqptr == 0)
  {
    t_output(*line - 2,2,"[editddr-syntax] Invalid syntax of an edit command");
    return(E_FAIL);
  }

  /* Increment address to position just to the
     right of the equal sign
  --------------------------------------------*/
  (*eqptr)++;		

  /* Skip over the white space between the
     equal sign and start of string
  ----------------------------------------*/
  while (strncmp(*eqptr," ",1) == 0)
    (*eqptr) ++;
 
  return(E_SUCC);
}


/******************************************************************************
NAME: VALIDATE

PURPOSE: Returns the correct string depending on the valid string

ALGORITHM DESCRIPTION:
  VALIDATE outputs the correct string 
  Updates the vld argument based on the input command string
  A non-fatal error is output if the command string is not valid.
******************************************************************************/
void validate
(
  char *eqptr,                 /* Point to command string */
  char *echo,                  /* Pointer to output string */
  long *vld,                   /* Pointer to valid code	*/
  long *line                   /* Number of lines on terminal */
)
{
  /* Output the correct string and correct valid
     code based on the input command code
  ----------------------------------------------*/
  if (strncmp(eqptr,"in",2)==0)
  {
    *vld = 0;
    strcpy(echo,"INVALID");
  }
  else if (strncmp(eqptr,"v",1)==0)
  {
    *vld = 1;
    strcpy(echo,"VALID");
  }
  else if (strncmp(eqptr,"u",1)==0)
  {
    *vld = 2;
    strcpy(echo,"UNKNOWN");
  }
  else
  {
    /* If invalid command code was input, output a
       non-fatal error message and output the correct
       string for the current valid code
    -------------------------------------------------*/
    if (*vld == 0)
      strcpy(echo,"NOT VALID");
    else if (*vld == 1)
      strcpy(echo,"VALID");
    else if (*vld == 2)
      strcpy(echo,"UNKNOWN");
    t_output(*line-2,2,"[editddr-value] Unacceptable valid value");
  }
}


/******************************************************************************
NAME: VALMM

PURPOSE: Returns the correct string depending upon the valid code 

ALGORITHM DESCRIPTION:
  VALMM outputs the correct string 
  Updates the vld argument based on the input command string
  A non-fatal error is output if the command string is not valid.
******************************************************************************/
void valmm
(
  char *eqptr,              /* Point to command string */
  char *echo,               /* Pointer to output string */
  long *vld,                /* Pointer to valid code */
  long *line                /* Number of lines on terminal */
)
{
  /*  Output the correct string and correct valid
      code based on the input command code
  -----------------------------------------------*/
  if (strncmp(eqptr,"i",1)==0)
  {
    *vld = 0;
    strcpy(echo,"INVALID");
  }
  else if (strncmp(eqptr,"v",1)==0)
  {
    *vld = 1;
    strcpy(echo,"VALID");
  }
  else if (strncmp(eqptr,"b",1)==0)
  {
    *vld = 2;
    strcpy(echo,"BOUNDED");
  }
  else
  {
    /* If invalid command code was input, output
       a non-fatal error message and output the
       correct string for the current valid code
    --------------------------------------------*/
    if (*vld == 0)
      strcpy(echo,"INVALID");
    else if (*vld == 1)
      strcpy(echo,"VALID");
    else if (*vld == 2)
      strcpy(echo,"BOUNDED");
    t_output(*line-2,2,"[editddr-value] Unacceptable valid value");
  }
}
