/******************************************************************************* 
NAME:  EDITGRID

PURPOSE:  To edit a GRID file.

PROGRAM HISTORY:
Version		Date	Author		Code/contr	Request
-------		----	------		----------	-------
  1.0		5/89	D.Etrheim	   CSB		initial development
  1.1		????	?????????	   GSFC		Minor changes for
							command line parsing	
  1.2		6/92	S.Nelson	   CSB		Corrected error 
							entering proj. type in
							tutor and screen mode.
							Added map projections
							Hammer and Robinson.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
TAE is initialized 
Input grid file, output grid file, and subcommand are retrieved from the 
      parameter block
Read in the grid file or create a new grid file
Depending on the subcommand chosen the following procedures are followed:

-SCREEN
   The contents of grid file are displayed to the terminal.  
   1. The user can edit any of the fields by using the codes shown on the 
      screen in upper case.  
   2. The user can also receive on-line help on any of the fields by 
      using the codes shown in upper case. 
   3. The user can page through all the grid information.  
   4. The user may quit without writing the grid information or write the
      grid information and exit this routine.  The completion message will be 
      displayed with an output file name only when a grid file is written to.

-TUTOR
   Each of the parameters are retrieved from the parameter block
   If the parameter has a value of NULL then the GRID field that pertains to 
    that parameter will not be changed
   If the parameter has a value, then the GRID field that pertains to that 
    parameter will be changed to the new input value
   After all the parameters are retrieved, the fields are written to the GRID
    file and the completion message is displayed.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int main
(void)

{  /* main */

struct PARBLK parb;			/*  TAE parameter block		*/
struct GEOGRID grid;			/*  Geometric mapping grid	*/

long one = 1;				/*  one				*/
long std;				/*  standard output		*/
long write_flag = FALSE;		/*  write flag			*/
long fatal = -1;			/*  fatal error flag for c_errmsg */
long zero = 0;				/*  zero			*/
long exist_flag = TRUE;			/*  flag for file existence	*/
long status;				/*  function status		*/
char hosin[CMLEN];			/*  input host grid file name	*/
char hosout[CMLEN];			/*  output host grid file name	*/
char subcmd[10];			/*  subcommand parameter	*/
char tempstr[1][CMLEN];    

/*  Initialize TAE and parameter block
--------------------------------------*/
c_inlas(&parb,&std);

/*  Retrieve the parameters
---------------------------*/
getpar(&parb,hosin,hosout,subcmd,&exist_flag);

/*  Read the grid file or initialize a new file
-----------------------------------------------*/
if (!exist_flag)
   status = clrgrd(&grid); 
else
   status = c_getgrd(hosin,&grid); 

if (status != E_SUCC)
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
 
/*  Branch on the basis of the subcommand
-----------------------------------------*/
if (strcmp(subcmd,"TUTOR") == 0)
   {
   edtbat(&parb,&grid,hosout,&exist_flag);
   write_flag = TRUE;
   }
else
   {
   dspcom(&grid,hosin,hosout,&write_flag,&exist_flag);
   }

/* Output the completion message.  Output file name if a write was completed
----------------------------------------------------------------------------*/
if (!write_flag)
   {
   strcpy(tempstr[0], "\0");
   c_complt("EDITGRID",tempstr,&zero);
   }
else
   c_complt("EDITGRID",hosout,&one);

return EXIT_SUCCESS;

} /* main */
/*****************************************************************************
*
*  NAME:	CLRGRD
*
*  FUNCTION:	To initialize the fields of a grid file
*
*  ALGORITHM DESCRIPTION:	
*  Initialize each of the grid fields to be edited.  Also initial other fields
*  for a new grid file.
*
******************************************************************************/

int clrgrd
(
    struct GEOGRID *grid			/* geometric mapping grid	*/
)

{ /* clrgrd */
long i;					/* counter			*/
long nonfatal = 1;			/* nonfatal error		*/

/*Fields to be edited
---------------------*/
grid->proj_valid = GEO_NOTVALID;
grid->ncoeff = 0;
grid->code = 0;
grid->zone = 0;
grid->datum = 0;
for (i = 0; i < 8; i++)
    grid->corners[i] = 0.0;
grid->pdist[0] = grid->pdist[1] = 0.0;
for (i = 0; i < 15; i++)
    grid->projprms[i] = 0.0;
grid->coeffs = (double *)malloc(30 *  sizeof(double));
if (grid->coeffs == NULL)
   {
   c_errmsg("Error allocating memory for geometric mapping grid",
            "editgrid-alloc",&nonfatal);
   return(E_FAIL);
   }
for (i = 0; i < 30; i++)
    grid->coeffs[i] = 0.0; 

grid->model[0] = (char)NULL;
grid->units[0] = (char)NULL;

/*Other fields
--------------*/
grid->fiterr_valid = GEO_NOTVALID;
grid->griderr_valid = GEO_NOTVALID;
grid->max_grid_err[0] = grid->max_grid_err[1] = 0.0;
grid->ave_grid_err[0] = grid->ave_grid_err[1] = 0.0;
grid->rms_grid_err = 0.0;
for (i = 0; i < 32; i++)
    grid->act_grid_err[i] = 0.0; 
grid->grid_tol = 0.0;
grid->max_fit_err[0] = grid->max_fit_err[1] = 0.0;
grid->ave_fit_err[0] = grid->ave_fit_err[1] = 0.0;
grid->rms_fit_err = 0.0;

return(E_SUCC);
} /* clrgrd */
