/******************************************************************************

NAME:	EDTBAT

FUNCTION: EDTBAT is used to edit the contents of the grid file using TAE 
	  parameters.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         05/89	   D.Etrheim	initial development
    1.2		06/92	   S.Nelson	Changed proj types to be excepted by
					the proper proj code instead of the
					first part of the name.
					Added map projections Hammer and
					Robinson.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
If a value was entered
   the value is entered into the grid structure
   If the grid file does not exist and no value was entered for a required field
      issue a fatal error message 
else
   The current value is left in place
The grid values are written to the grid file

******************************************************************************/
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

void edtbat
(
    struct PARBLK *parblk,		/* pointer to TAE parameter block	*/
    struct GEOGRID *grid,		/* Geometric mapping grid 		*/
    char *hosout,			/* input grid file host name		*/
    long *exist_flag		/* flag if file exists or not		*/
)

{ /* edtbat */
struct VARIABLE *var;	        /* pointer to a TAE variable		*/

long	fatal = -1;		/* fatal error code			*/
long	i;			/* loop index				*/
long	nonfatal = 1;		/* nonfatal error code			*/
long	flag1 = FALSE;		/* flag if linecoef were entered	*/
long    flag2 = FALSE;		/* flag if sampcoef were entered	*/
long	length;			/* length of string			*/
long	temp;			/* temp storage for long variable	*/

char    proj_str[9];            /* project code string			*/

/* Get parameters NCOEF,LINECOEF,SAMPCOEF, and MODEL.  These are required if
creating a new grid file.
----------------------------------------------------------------------------*/
var = p_find(parblk,"ncoef");
if ((*var).v_count != 0)
   {
   grid->ncoeff = IVAL(*var,0);
   for (i = 0; i < grid->ncoeff * 2; i++)
    grid->coeffs[i] = 0.0;
   }
if ((!*exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg("No. of coefficients is required when a grid is being created",
            "editgrid-create",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }

var = p_find(parblk,"linecoef");
if ((*var).v_count > grid->ncoeff)
   {
   c_errmsg("Invalid polynomial entry","editgrid-poly",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }
for (i=0; i < (*var).v_count; i++)
    grid->coeffs[i] = RVAL(*var,i);
if ((*var).v_count != 0) 
   flag1 = TRUE; 
if ((!*exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg
     ("Transformation coefficients are required when a grid is being created",
      "editgrid-create",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }

var = p_find(parblk,"sampcoef");
if ((*var).v_count > grid->ncoeff)
   {
   c_errmsg("Invalid polynomial entry","editgrid-poly",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }
for (i=0; i < (*var).v_count; i++)
    grid->coeffs[i + grid->ncoeff] = RVAL(*var,i);
if ((*var).v_count != 0) 
   flag2 = TRUE; 
if ((!*exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg
     ("Transformation coefficients are required when a grid is being created",
      "editgrid-create",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }

/*If transformation coefficients were 
entered, clear out some grid fields
-------------------------------------*/
if (flag1 || flag2 )
    grid->nrows = grid->ncols = grid->lines = grid->samples = 0;

var = p_find(parblk,"model");
if ((*var).v_count != 0)
   {
   strcpy(grid->model,SVAL(*var,0)); 
   length = (long) strlen(grid->model);
   c_low2up(grid->model,&length);
   }
if ((!*exist_flag) && ((*var).v_count == 0))
   {
   c_errmsg("Model method is required when a grid is being created",
            "editgrid-create",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }
    
/* Get projection type 
----------------------*/
var = p_find(parblk,"projtype");
if ((*var).v_count != 0)
   {
   strcpy (proj_str,SVAL(*var,0));
   if (strncmp(proj_str,"geo",3) == 0)
      grid->code=0;
   else if (strncmp(proj_str,"u",1) == 0)
      grid->code = 1;
   else if (strncmp(proj_str,"spc",3)==0) 
      grid->code = 2;
   else if (strncmp(proj_str,"alber",5) == 0) 
      grid->code = 3;
   else if ((strncmp(proj_str,"lambert c",9) == 0) || 
            (strncmp(proj_str,"lamc",4) == 0))
      grid->code = 4;
   else if (strncmp(proj_str,"me",2) == 0) 
      grid->code = 5;
   else if (strncmp(proj_str,"ps",2) == 0)
      grid->code = 6;
   else if (strncmp(proj_str,"poly",4) == 0) 
      grid->code = 7;
   else if (strncmp(proj_str,"equid",5) == 0) 
      grid->code = 8;
   else if (strncmp(proj_str,"t",1) == 0) 
      grid->code = 9;
   else if (strncmp(proj_str,"ste",3)==0) 
      grid->code = 10;
   else if ((strncmp(proj_str,"lambert a",9) == 0) || 
            (strncmp(proj_str,"lama",4) == 0))
      grid->code = 11;
   else if (strncmp(proj_str,"az",2) == 0)
      grid->code = 12;
   else if (strncmp(proj_str,"gn",2) == 0) 
      grid->code = 13;
   else if (strncmp(proj_str,"or",2) == 0)
      grid->code = 14;
   else if (strncmp(proj_str,"gv",2) == 0) 
      grid->code = 15;
   else if (strncmp(proj_str,"sn",2) == 0) 
      grid->code = 16;
   else if (strncmp(proj_str,"eqr",3) == 0)
      grid->code = 17;
   else if (strncmp(proj_str,"mi",2)==0)
      grid->code = 18;
   else if (strncmp(proj_str,"v",1)==0) 
      grid->code = 19;
   else if (strncmp(proj_str,"hom",3)==0) 
      grid->code = 20;
   else if (strncmp(proj_str,"rob",3) == 0)
      grid->code = 21;
   else if (strncmp(proj_str,"so",2) == 0)
      grid->code = 22;
   else if (strncmp(proj_str,"alask",5) == 0)
      grid->code = 23;
   else if (strncmp(proj_str,"good",4) == 0)
      grid->code = 24;
   else if (strncmp(proj_str,"moll",4) == 0)
      grid->code = 25;
   else if (strncmp(proj_str,"imoll",5) == 0)
      grid->code = 26;
   else if (strncmp(proj_str,"ha",2) == 0)
      grid->code = 27;
   else if (strncmp(proj_str,"wagiv",5) == 0)
      grid->code = 28;
   else if (strncmp(proj_str,"wagvii",6) == 0)
      grid->code = 29;
   else if (strncmp(proj_str,"obeqa",5) == 0)
      grid->code = 30;
   else
      {
      sscanf(proj_str,"%d",&temp);
      if ((temp < 0) || (temp > 99))
         {
         c_errmsg("Invalid projection code","editgrid-projcode",&nonfatal);
         c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
         }
      else
         grid->code = temp;
      }
   }

/* Get projection validity 
--------------------------*/
var = p_find(parblk,"ppval");
if ((*var).v_count != 0)
   grid->proj_valid = valid(SVAL(*var,0));
 
/* Get zone number 
-----------------*/
var = p_find(parblk,"zonenum");
if ((*var).v_count != 0)
   grid->zone = IVAL(*var,0);

/* Get datum code 
-----------------*/
var = p_find(parblk,"dcode");
if ((*var).v_count != 0)
   grid->datum = IVAL(*var,0);


/* Get projection coefficients 
------------------------------*/
var = p_find(parblk,"projparm");
if ((*var).v_count != 0)
   {
   for (i = 0; i < (*var).v_count ; i++)
       grid->projprms[i] = RVAL(*var,i);
   }

/* Get the four corner coordinates 
---------------------------------*/
var = p_find(parblk,"upleft");
if ((*var).v_count != 0)
   {
   grid->corners[0] = RVAL(*var,0);
   grid->corners[1] = RVAL(*var,1);
   }
var = p_find(parblk,"loleft");
if ((*var).v_count != 0)
   {
   grid->corners[2] = RVAL(*var,0);
   grid->corners[3] = RVAL(*var,1);
   }
var = p_find(parblk,"upright");
if ((*var).v_count != 0)
   {
   grid->corners[4] = RVAL(*var,0);
   grid->corners[5] = RVAL(*var,1);
   }
var = p_find(parblk,"loright");
if ((*var).v_count != 0)
   {
   grid->corners[6] = RVAL(*var,0);
   grid->corners[7] = RVAL(*var,1);
   }

/* Get ground distance in line and sample direction 
---------------------------------------------------*/
var = p_find(parblk,"pdline");
if ((*var).v_count != 0)
   grid->pdist[0] = RVAL(*var,0);

var = p_find(parblk,"pdsamp");
if ((*var).v_count != 0)
   grid->pdist[1] = RVAL(*var,0);

/* Get ground measurement units 
-------------------------------*/
var = p_find(parblk,"punits");
if ((*var).v_count != 0)
   {
   strcpy(grid->units,SVAL(*var,0));
   length = (long) strlen(grid->units);
   c_up2low(grid->units,&length);
   }

/* write the grid information to the file
-----------------------------------------*/
if (c_putgrd(hosout,grid) != E_SUCC)
   {
   c_errmsg("Error writing to grid file","editgrid-write",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }

return;
}  /* edit batch */
/******************************************************************************
*
*  NAME:  		VALID
*
*  FUNCTION:		To return the correct valid flag based on the input str.
*
*  ALGORITHM:
*  If val_str = IN
*     return INVALID
*  Else 
*     return VALID
*
******************************************************************************/

int valid
(
    char *val_str		/*  input valid string				*/
)

{  /*  valid  */
long val_int;

if (strcmp(val_str,"IN") == 0)
   val_int = GEO_NOTVALID;
else 
   val_int = GEO_VALID;

return(val_int);
}  /*  valid  */

