/******************************************************************************

NAME:	EDTGRID

FUNCTION: EDTGRID is used to edit the contents of the GRID file interactively
          on the terminal.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         05/89      D.Etrheim    initial development
    1.1		06/92	   S.Nelson	Changed the proj type to except the
					appropriate code name.
					Added map projections Hammer and
					Robinson.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
Compare the edit command code with the codes of each of the grid fields
When the correct code is found
   the correct field of the grid is updated with the new value 
   the new value is output to the correct place on the terminal
   Validity checking is done when possible.  If an incorrect code is input, 
     an error message is output.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "datum.h"
#include "util.h"
#include "protos.h"

void edtgrid
(
    char *edans_buf,			/* pointer to edit command code	*/
    long *line,				/* number of lines on the screen*/
    struct GEOGRID *grid,			/* Geometric mapping grid	*/
    long *edit_flag			/* edit flag			*/
)

{  /* edtgrid */

long status;				/* status of called routines	*/
long temp;				/* temporary storage for long var*/
long length;				/* length of string		*/
long i;					/* loop index			*/

char *eqptr;				/* temporary pointer to command code*/
char blank5[6];				/* buffer to blank out 5 spaces	*/
char blank12[13];			/* buffer to blank out 12 spaces*/
char blank23[24];			/* buffer to blank out 23 spaces*/
char blank40[41];			/* buffer to blank out 40 spaces*/
char blank60[61];			/* buffer to blank out 60 spaces*/
char blank70[71];			/* buffer to blank out 70 spaces*/
char echo[80];				/* temp. buffer to hold output value*/
char ctemp[3];

/*  Initialize the blank strings 
--------------------------------*/
strcpy(blank5,"     ");
strcpy(blank12,"            ");
strcpy(blank23,"                       ");
strcpy(blank40,"                                        ");
strcpy(blank60,"                                                            ");
strcpy(blank70,
"                                                                      ");
/*  Initialize variables
------------------------*/
status = E_SUCC;

/*  Set temporary pointer to edit command buffer 
------------------------------------------------*/
eqptr = edans_buf;

/*  Find the start of the new value (1st non-space letter past the equal sign)
------------------------------------------------------------------------------*/
status = start_of_line(line,&eqptr,edans_buf);
if (status != E_SUCC)
   return;

/*  Compare the edit command code with each of the known codes of the grid 
fields.  When the correct code is found change the value of that field
and output the new value to the correct place on the terminal.  Make
sure that the place on the terminal is blanked out first to remove any
artifacts of the previous value.
-------------------------------------------------------------------------*/
if (strncmp(edans_buf,"pc",2)== 0)
   {
   /*  output the correct projection based on the new projection code
   ------------------------------------------------------------------*/
   if ((strncmp(eqptr,"geo",3) == 0) || (strncmp(eqptr,"0",1) == 0))
      {
      strcpy(echo,"(0)GEOGRAPHIC");  
      grid->code=0;
      }
   else if ((strncmp(eqptr,"u",1) == 0) || (strcmp(eqptr,"1") == 0))
      {
      strcpy(echo,"(1)UTM");  
      grid->code = 1;
      }
   else if ((strncmp(eqptr,"spc",3) == 0) || (strcmp(eqptr,"2") == 0))
      {
      strcpy(echo,"(2)STATE PLANE");  
      grid->code = 2;
      }
   else if ((strncmp(eqptr,"al",2) == 0) || (strcmp(eqptr,"3") == 0))
      {
      strcpy(echo,"(3)ALBERS CONICAL EQUAL AREA");  
      grid->code = 3;
      }
   else if ((strncmp(eqptr,"lambert c",9) == 0) || 
            (strncmp(eqptr,"lamc",4) == 0) || (strcmp(eqptr,"4") == 0))
      {
      strcpy(echo,"(4)LAMBERT CONFORMAL CONIC");  
      grid->code = 4;
      }
   else if ((strncmp(eqptr,"me",2) == 0) || (strcmp(eqptr,"5") == 0))
      {
      strcpy(echo,"(5)MERCATOR");  
      grid->code = 5;
      }
   else if ((strncmp(eqptr,"ps",2) == 0) || (strcmp(eqptr,"6") == 0))
      {
      strcpy(echo,"(6)POLAR STEREOGRAPHIC");  
      grid->code = 6;
      }
   else if ((strncmp(eqptr,"poly",4) == 0) || (strcmp(eqptr,"7") == 0))
      {
      strcpy(echo,"(7)POLYCONIC");  
      grid->code = 7;
      }
   else if ((strncmp(eqptr,"equid",5) == 0) || (strcmp(eqptr,"8") == 0))
      {
      strcpy(echo,"(8)EQUIDISTANT CONIC");  
      grid->code = 8;
      }
   else if ((strncmp(eqptr,"t",1) == 0) || (strcmp(eqptr,"9") == 0))
      {
      strcpy(echo,"(9)TRANSVERSE MERCATOR");  
      grid->code = 9;
      }
   else if ((strncmp(eqptr,"ste",3) == 0) || (strncmp(eqptr,"10",2) == 0))
      {
      strcpy(echo,"(10)STEREOGRAPHIC");  
      grid->code = 10;
      }
   else if ((strncmp(eqptr,"lambert a",9) == 0) || 
            (strncmp(eqptr,"lama",4) == 0) || (strncmp(eqptr,"11",2) == 0))
      {
      strcpy(echo,"(11)LAMBERT AZIMUTHAL EQUAL AREA");  
      grid->code = 11;
      }
   else if ((strncmp(eqptr,"az",2) == 0) || (strncmp(eqptr,"12",2) == 0))
      {
      strcpy(echo,"(12)AZIMUTHAL EQUIDISTANT");  
      grid->code = 12;
      }
   else if ((strncmp(eqptr,"gn",2) == 0) || (strncmp(eqptr,"13",2) == 0))
      {
      strcpy(echo,"(13)GNOMONIC");  
      grid->code = 13;
      }
   else if ((strncmp(eqptr,"or",2) == 0) || (strncmp(eqptr,"14",2) == 0))
      {
      strcpy(echo,"(14)ORTHOGRAPHIC");  
      grid->code = 14;
      }
   else if ((strncmp(eqptr,"gvn",3) == 0) || (strncmp(eqptr,"15",2) == 0))
      {
      strcpy(echo,"(15)GENERAL VERTICAL NEAR-SIDE PERSPECTIVE"); 
      grid->code = 15;
      }
   else if ((strncmp(eqptr,"sn",2) == 0) || (strncmp(eqptr,"16",2) == 0))
      {
      strcpy(echo,"(16)SINUSIODAL");  
      grid->code = 16;
      }
   else if ((strncmp(eqptr,"eqr",3) == 0) || (strncmp(eqptr,"17",2) == 0))
      {
      strcpy(echo,"(17)EQUIRECTANGULAR");  
      grid->code = 17;
      }
   else if ((strncmp(eqptr,"mi",2) == 0) || (strncmp(eqptr,"18",2) == 0))
      {
      strcpy(echo,"(18)MILLER CYLINDRICAL");  
      grid->code = 18;
      }
   else if ((strncmp(eqptr,"v",1) == 0) || (strncmp(eqptr,"19",2) == 0))
      {
      strcpy(echo,"(19)VAN DER GRINTEN");  
      grid->code = 19;
      }
   else if ((strncmp(eqptr,"som",2) == 0) || (strncmp(eqptr,"20",2) == 0))
      {
      strcpy(echo,"(20)OBLIQUE MERCATOR (Hotine)");  
      grid->code = 20;
      }
   else if ((strncmp(eqptr,"rob",2)==0) || (strncmp(eqptr,"21",2) == 0))
      {
      strcpy(echo,"(21)ROBINSON");
      grid->code = 21;
      }
   else if ((strncmp(eqptr,"so",2)==0) || (strncmp(eqptr,"22",2) == 0))
      {
      strcpy(echo,"(22)SOM");
      grid->code = 22;
      }
   else if ((strncmp(eqptr,"alaska",5)==0) || (strncmp(eqptr,"23",2) == 0))
      {
      strcpy(echo,"(23)ALASKA CONFORMAL");
      grid->code = 23;
      }
   else if ((strncmp(eqptr,"good",4)==0) || (strncmp(eqptr,"24",2) == 0))
      {
      strcpy(echo,"(24)INTERRUPTED GOODE HOMOLOSINE");
      grid->code = 24;
      }
   else if ((strncmp(eqptr,"moll",4)==0) || (strncmp(eqptr,"25",2) == 0))
      {
      strcpy(echo,"(25)MOLLWEIDE");
      grid->code = 25;
      }
   else if ((strncmp(eqptr,"imoll",5)==0) || (strncmp(eqptr,"26",2) == 0))
      {
      strcpy(echo,"(26)INTERRUPTED MOLLWEIDE");
      grid->code = 26;
      }
   else if ((strncmp(eqptr,"ro",2)==0) || (strncmp(eqptr,"27",2) == 0))
      {
      strcpy(echo,"(27)HAMMER");
      grid->code = 27;
      }
   else if ((strncmp(eqptr,"wagiv",5)==0) || (strncmp(eqptr,"28",2) == 0))
      {
      strcpy(echo,"(28)WAGNER IV");
      grid->code = 28;
      }
   else if ((strncmp(eqptr,"wagvii",6)==0) || (strncmp(eqptr,"29",2) == 0))
      {
      strcpy(echo,"(29)WAGNER VII");
      grid->code = 29;
      }
   else if ((strncmp(eqptr,"obeqa",5)==0) || (strncmp(eqptr,"30",2) == 0))
      {
      strcpy(echo,"(30)OBLATED EQUAL AREA");
      grid->code = 30;
       }

   else
      {
      sscanf(eqptr,"%d",&temp);
      if ((temp < 0) || (temp > 99))
         {
         t_output(*line - 2,2,"[editgrid-projcode] Invalid projection code");
         status = E_FAIL;
         }
      else
	 {
         strcpy(echo,eqptr);
         grid->code = temp;
         }
      }
   if (status == E_SUCC)
      {
      t_output(3,14,blank60);
      t_output(3,14,echo);
      }
   }  /* if 'pc' */

else if (strncmp(edans_buf,"pz",2) == 0)
   {
   sscanf(eqptr,"%d",&temp);
   if ((temp < 0) || (temp > 5400))
      { /* Invalid zone code */
      t_output(*line - 2,2,"[editgrid-zone] Invalid zone code");
      status = E_FAIL;
      } 
   else
      { /* valid zone code */
      strcpy(echo,eqptr);
      grid->zone = temp;
      t_output(5,14,blank5);
      t_output(5,14,echo);
      }
   }  /* if 'pz' */

else if (strncmp(edans_buf,"dc",2) == 0)
   {
   sscanf(eqptr,"%d",&temp);
   if ((temp < 0) || (temp > MAX_DATUM)) 
      {
      t_output(*line - 2,2,"[editgrid-datum] Invalid datum code");
      status = E_FAIL;
      }
   else
      { /* valid datum code */
      c_dtmstr(&temp,echo);
      grid->datum = temp;
      t_output(4,14,blank60);
      t_output(4,14,echo);
      } 
   }  /* if 'dc' */

else if (strncmp(edans_buf,"pv",2) == 0)
   {
   validate(eqptr,echo,&(grid->proj_valid),line);
   t_output(6,18,blank12);
   t_output(6,18,echo);
   }  /* if 'pv'  */

else if (strncmp(edans_buf,"ppa",3) == 0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(8,7,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[0]));
      sprintf(echo,"%-1.14E",grid->projprms[0]);
      t_output(8,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(8,30,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[1]));
      sprintf(echo,"%-1.14E",grid->projprms[1]);
      t_output(8,30,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(8,53,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[2]));
      sprintf(echo,"%-1.14E",grid->projprms[2]);
      t_output(8,53,echo);
      }
   else
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      {
      t_output(8,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(grid->projprms[0]),
             &(grid->projprms[1]),&(grid->projprms[2]));
      sprintf(echo,"%-1.14E   %-1.14E   %-1.14E",grid->projprms[0],
              grid->projprms[1],grid->projprms[2]);
      t_output(8,7,echo);
      } 
   }  /* if 'ppa' */

else if (strncmp(edans_buf,"ppb",3) == 0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(9,7,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[3]));
      sprintf(echo,"%-1.14E",grid->projprms[3]);
      t_output(9,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(9,30,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[4]));
      sprintf(echo,"%-1.14E",grid->projprms[4]);
      t_output(9,30,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(9,53,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[5]));
      sprintf(echo,"%-1.14E",grid->projprms[5]);
      t_output(9,53,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(9,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(grid->projprms[3]),
             &(grid->projprms[4]),&(grid->projprms[5]));
      sprintf(echo,"%-1.14E   %-1.14E   %-1.14E",grid->projprms[3],
              grid->projprms[4],grid->projprms[5]);
      t_output(9,7,echo);
      }
   }  /* if 'ppb' */

else if (strncmp(edans_buf,"ppc",3) == 0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(10,7,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[6]));
      sprintf(echo,"%-1.14E",grid->projprms[6]);
      t_output(10,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(10,30,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[7]));
      sprintf(echo,"%-1.14E",grid->projprms[7]);
      t_output(10,30,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(10,53,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[8]));
      sprintf(echo,"%-1.14E",grid->projprms[8]);
      t_output(10,53,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(10,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(grid->projprms[6]),
             &(grid->projprms[7]),&(grid->projprms[8]));
      sprintf(echo,"%-1.14E   %-1.14E   %-1.14E",grid->projprms[6],
              grid->projprms[7],grid->projprms[8]);
      t_output(10,7,echo);
      }
   }  /* if 'ppc' */

else if (strncmp(edans_buf,"ppd",3) == 0)
   {

   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(11,7,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[9]));
      sprintf(echo,"%-1.14E",grid->projprms[9]);
      t_output(11,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(11,30,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[10]));
      sprintf(echo,"%-1.14E",grid->projprms[10]);
      t_output(11,30,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(11,53,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[11]));
      sprintf(echo,"%-1.14E",grid->projprms[11]);
      t_output(11,53,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(11,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(grid->projprms[9]),
             &(grid->projprms[10]),&(grid->projprms[11]));
      sprintf(echo,"%-1.14E   %-1.14E   %-1.14E",grid->projprms[9],
              grid->projprms[10],grid->projprms[11]);
      t_output(11,7,echo);
      }
   }  /* if 'ppd' */

else if (strncmp(edans_buf,"ppe",3) == 0)
   {

   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(12,7,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[12]));
      sprintf(echo,"%-1.14E",grid->projprms[12]);
      t_output(12,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(12,30,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[13]));
      sprintf(echo,"%-1.14E",grid->projprms[13]);
      t_output(12,30,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(12,53,blank23);
      sscanf(eqptr,"%lf",&(grid->projprms[14]));
      sprintf(echo,"%-1.14E",grid->projprms[14]);
      t_output(12,53,echo);
      }
   else

      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      {
      t_output(12,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(grid->projprms[12]),
      &(grid->projprms[13]),&(grid->projprms[14]));
      sprintf(echo,"%-1.14E   %-1.14E   %-1.14E",grid->projprms[12],
      grid->projprms[13],grid->projprms[14]);
      t_output(12,7,echo);
      }
   }  /* if 'ppe' */

else if ((strncmp(edans_buf,"ul",2) == 0) || (strncmp(edans_buf,"ccul",4) == 0))
   {
   t_output(15,14,blank60);
   sscanf(eqptr,"%lf %lf",&(grid->corners[0]),&(grid->corners[1]));
   sprintf(echo,"%-1.14E   %-1.14E",grid->corners[0],
 	   grid->corners[1]);
   t_output(15,14,echo);
   }  /* if 'ul' */

else if ((strncmp(edans_buf,"ur",2)==0) || (strncmp(edans_buf,"ccur",4) == 0))
   {
   t_output(16,14,blank60);
   sscanf(eqptr,"%lf %lf",&(grid->corners[4]),&(grid->corners[5]));
   sprintf(echo,"%-1.14E   %-1.14E",grid->corners[4],
     grid->corners[5]);
   t_output(16,14,echo);
   }  /* if 'ur' */

else if ((strncmp(edans_buf,"ll",2) == 0) || (strncmp(edans_buf,"ccll",4) == 0))
   {
   t_output(17,14,blank60);
   sscanf(eqptr,"%lf %lf",&(grid->corners[2]),&(grid->corners[3]));
   sprintf(echo,"%-1.14E   %-1.14E",grid->corners[2], grid->corners[3]);
   t_output(17,14,echo);
   }  /* if 'll' */

else if ((strncmp(edans_buf,"lr",2) == 0) || (strncmp(edans_buf,"cclr",4) == 0))
   {
   t_output(18,14,blank60);
   sscanf(eqptr,"%lf %lf",&(grid->corners[6]),&(grid->corners[7]));
   sprintf(echo,"%-1.14E   %-1.14E",grid->corners[6], grid->corners[7]);
   t_output(18,14,echo);
   }  /* if 'lr' */

else if (strncmp(edans_buf,"pd",2) == 0)
   {
   t_output(20,14,blank40);
   sscanf(eqptr,"%lf %lf",&(grid->pdist[0]), &(grid->pdist[1]));
   sprintf(echo,"%-1.14E   %-1.14E",grid->pdist[0], grid->pdist[1]);
   t_output(20,14,echo);
   }  /* if 'pd' */

else if (strncmp(edans_buf,"pu",2) == 0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 12)
      {
      t_output(*line-2,2,"[editgrid-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(21,15,blank40);
      t_output(21,15,echo);
      strcpy(grid->units,eqptr);
      }
   }  /* if 'pu' */

else if (strncmp(edans_buf,"mm",2) == 0)
   {
   strcpy(echo,eqptr); 
   length = (long) strlen(echo);
   if (length > 15)
      {
      t_output(*line-2,2,"[editgrid-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      c_low2up(echo,&length);
      strcpy(grid->model,echo);
      t_output(2,16,blank23);
      t_output(2,16,echo);
      if (strcmp(grid->model,"POLYNOMIAL") != 0)
         {
         grid->ncoeff = 0;
         sprintf(echo,"0");
         t_output(3,26,blank5);
         t_output(3,26,echo);
         }
      dstcoef(grid);
      }
   }  /* if 'mm' */

else if (strncmp(edans_buf,"nc",2) == 0)
   {
   sscanf(eqptr,"%d",&temp);
   if ((temp == 3) || (temp == 6) || (temp == 10) || (temp == 15))
      {
      grid->ncoeff = temp;
      strcpy(echo,eqptr);
      t_output(3,26,blank5);
      t_output(3,26,echo);
      for (i = 0; i < grid->ncoeff * 2; i++)
         grid->coeffs[i] = 0.0;
      dstcoef(grid);
      }
   else
      {
      t_output(*line - 2,2,"[editgrid-ncoef] Invalid number of coefficients");
      status = E_FAIL;
      }
   } /* if 'nc' */

else if(strncmp(edans_buf,"fpa",3) == 0) 
   {
   sscanf(edans_buf,"%3c%2d",ctemp,&temp);
   if (temp >= grid->ncoeff)
      {
      t_output(*line - 2,2,"[editgrid-poly] Invalid polynomial entry");
      status = E_FAIL;
      }
   else
      { 
      sscanf(eqptr,"%lf",&(grid->coeffs[temp]));
      sprintf(echo,"%20.12e",grid->coeffs[temp]);
      t_output(6+temp,9,blank23);
      t_output(6+temp,9,echo);
      grid->nrows = grid->ncols = grid ->lines = grid->samples = 0;
      }
   } /* if 'fpa' */ 

else if(strncmp(edans_buf,"a",1) == 0) 
   {
   sscanf(edans_buf,"%1c%2d",ctemp,&temp);
   if (temp >= grid->ncoeff)
      {
      t_output(*line - 2,2,"[editgrid-poly] Invalid polynomial entry");
      status = E_FAIL;
      }
   else
      { 
      sscanf(eqptr,"%lf",&(grid->coeffs[temp]));
      sprintf(echo,"%20.12e",grid->coeffs[temp]);
      t_output(6+temp,9,blank23);
      t_output(6+temp,9,echo);
      grid->nrows = grid->ncols = grid ->lines = grid->samples = 0;
      }
   } /* if 'a' */ 

else if(strncmp(edans_buf,"spb",3) == 0) 
   {
   sscanf(edans_buf,"%3c%2d",ctemp,&temp);
   if (temp >= grid->ncoeff)
      {
      t_output(*line - 2,2,"[editgrid-poly] Invalid polynomial entry");
      status = E_FAIL;
      }
   else
      { 
      sscanf(eqptr,"%lf",&(grid->coeffs[temp + grid->ncoeff]));
      sprintf(echo,"%20.12e",grid->coeffs[temp + grid->ncoeff]);
      t_output(6+temp,56,blank23);
      t_output(6+temp,56,echo);
      grid->nrows = grid->ncols = grid ->lines = grid->samples = 0;
      }
   } /* if 'spb' */ 

else if(strncmp(edans_buf,"b",1) == 0) 
   {
   sscanf(edans_buf,"%1c%2d",ctemp,&temp);
   if (temp >= grid->ncoeff)
      {
      t_output(*line - 2,2,"[editgrid-poly] Invalid polynomial entry");
      status = E_FAIL;
      }
   else
      { 
      sscanf(eqptr,"%lf",&(grid->coeffs[temp + grid->ncoeff]));
      sprintf(echo,"%20.12e",grid->coeffs[temp + grid->ncoeff]);
      t_output(6+temp,56,blank23);
      t_output(6+temp,56,echo);
      grid->nrows = grid->ncols = grid ->lines = grid->samples = 0;
      }
   } /* if 'b' */ 

else
   {
   t_output(*line - 2,2,"[editgrid-code] Invalid edit command code");
   t_output(*line,2,blank60);
   status = E_FAIL;
   } /* invalid code */

/*  Set edit flag to TRUE if no error had occured and a field was edited
------------------------------------------------------------------------*/
if (status == E_SUCC)
   *edit_flag = TRUE;

return;
}  /* edtgrid */

/******************************************************************************

NAME:	START_OF_LINE

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         05/89      D.Etrheim    initial development

ALGORITHM DESCRIPTION:
START_OF_LINE parses the edit command string 
Returns the starting position of the character string which is to the right of 
the equal sign(=)
A non-fatal error is output if the command string does not contain an equal sign

******************************************************************************/

int start_of_line
(
    long *line,				/*  number of lines on the terminal*/
    char **eqptr,				/*  agridess of the str. right of = */
    char *string				/*  agridess of command string	*/
)

{  /*  start_of_line  */

/*  Locate the agridess of the equal sign in the command string
--------------------------------------------------------------*/
*eqptr = strchr(string,'=');

/*  output non-fatal error if equal sign is not found
-----------------------------------------------------*/
if (*eqptr == 0)
   {
   t_output(*line - 2,2,"[editgrid-syntax] Invalid syntax of an edit command");
   return(E_FAIL);
   }

/* increment agridess to position just to the right of the equal sign
--------------------------------------------------------------------*/
(*eqptr) ++;		

/*  Skip over the white space between the equal sign and start of string
------------------------------------------------------------------------*/
while (strncmp(*eqptr," ",1) == 0)
   (*eqptr) ++;
 
return(E_SUCC);
}  /* start_of_line */

/******************************************************************************

NAME:	VALIDATE

FUNCTION: Returns the correct string depending on the valid string

ALGORITHM DESCRIPTION:
VALIDATE outputs the correct string 
Updates the vld argument based on the input command string
A non-fatal error is output if the command string is not valid.

******************************************************************************/
void validate
(
    char *eqptr,				/* point to command string	*/
    char *echo,				/* pointer to output string	*/
    long *vld,				/* pointer to valid code	*/
    long *line				/* number of lines on terminal	*/
)

{  /* validate */

/*  Output the correct string and correct valid code based on the 
    input command code
-----------------------------------------------------------------*/
if (strncmp(eqptr,"i",1) == 0)
   {
   *vld = GEO_NOTVALID;
   strcpy(echo,"INVALID");
   }
else if (strncmp(eqptr,"v",1) == 0)
   {
   *vld = GEO_VALID;
   strcpy(echo,"VALID");
   }
else
   {
   /*  If invalid command code was input, output a non-fatal error message
       and output the correct string for the current valid code
   -----------------------------------------------------------------------*/
   if (*vld == GEO_NOTVALID)
      strcpy(echo,"INVALID");
   else 
      strcpy(echo,"VALID");
   t_output(*line-2,2,"[editgrid-value] Unacceptable valid value");
   }

return;
}  /* validate */
