/****************************************************************************

  NAME:		EDITTIE

  FUNCION: Edit a Tie Point Selection file, Tie Point Location file,
	   or a Merged Tie Point File.  The editing may be done
	   interactively or in a parameter mode. 

  PROGRAM HISTORY:
  VERSION	DATE	PROGRAMMER	CODE/CONT	REQUEST
  -------	----	----------	---------	-------
    1.0		6/89	D. Etrheim	 EDC/CSB	Initial development
    1.1		7/90	D. Etrheim	 EDC/CSB	ADP #5779 - Add width/
							precision to sprintf
							statements to allow 
							character fields to be 
							printed.
    1.2	        7/92	S. Nelson	 EDC/CSB	Added map projections
							Hammer and Robinson.
							Changed proj types to be
							excepted by the proper
							projection code.
							Changed parameters
							CORNCOOR to CORNVALS
							and PVAL to PPVALS.
    1.3		4/94	D. Etrheim	 EDC/CSB	Allow projection units
							to be in upper or lower
							case.
    1.4		1/95	T. Mittan	 EDC/CSB	Modified to use
							realloc to handle any
							number of tie points. 
    1.5        10/96    B. Olson         EDC/CSB        Added subcommand to
                                                        remove coincident 
                                                        tiepoints based on user
                                                        specified distance
                                                        between points.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	EDITTIE must be run under TAE.

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  LAS is initialized
  The input and output file names, file type and subcommand are retrieved from
      the parameter block
  Depending on the subcommand chosen and/or the file type, the following 
      procedures are followed:
  
  -TPS, -TPL, -MTP
  Read in the Tie Point files and combine the data records if necessary
  Each of the parameters are retrieve from the parameter block
  If the parameter count is zero, then the Tie Point field that pertains to
      that field will not be updated
  Else the tie point filed will be updated
  After all the parameters are retrieve, the fields are written to the 
      Tie Point file and the completion message is displayed

  -SCREEN
  Read in the Tie Point files and combine the data records if necessary
  The contents of the Tie Point header record are displayed to the terminal
  1. The user can edit any of the fields by using the codes shown on the
     terminal in upper case.
  2. The user can receive on-line help on any of the fields by using the 
     codes shown in upper case.
  3. The user can page through all the tie point header or data records.
  4. The user may quit without writing the tie point information or write
     the tie point file and exit and the appropriate completion message is
     displayed.

  -RMDUPS
  Read in the Tie Point files and combine the data records if necessary
  The distance between each pair of tie points is calculated and sorted.
  Based on the minimum separation specified select tie point with shortest
      overall path length to all other active tie points and mark it inactive.
  Tie points are written to the Tie Point file and the completion message is
      displayed.
  
  ALGORITHM REFERENCES: None
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int main(void)  
{ /* edittie */

struct PARBLK vblock;				/* TAE variable block	*/

char hosin[2][CMLEN];				/* Input host file names*/
char hosout[CMLEN];				/* Output host file name*/
char outtie[2][CMLEN];				/* Output TAE tie point name*/
char subcmd[8];					/* Subcommand used	*/
char type[4];					/* type of tie point file*/
char prtfile[CMLEN];
double mindist;                          /* minimum separation allowed*/
long combin = FALSE;				/* input files to be combined*/
long fatal = -1;				/* Fatal error message	*/
long new_file = FALSE;				/* creating a new file flag*/
long one = 1;					/* Constant of one	*/
long status;					/* Function status	*/
long std_out;					/* Standard output	*/
long write_flag = FALSE;			/* logical write flag	*/
long zero = 0;					/* Constant of zero	*/
long two =2;

/* initialize LAS processing
----------------------------*/
c_inlas(&vblock,&std_out);

/* get input tie point file names and file type
----------------------------------------------*/
if (getpar(&vblock,subcmd,hosout,hosin,type,&mindist,&combin,&new_file) 
      != E_SUCC)
   c_errmsg("Fatal error encountered","edittie-fatal",&fatal);

strcpy(prtfile,"");

/* branch on the subcommand and file type
----------------------------------------*/
if (strcmp(subcmd,"SCREEN") == 0)
   {
   if (strcmp(type,"tps") == 0)
      status = dsp_tps(hosin[0],hosout,&combin,&write_flag,&new_file,subcmd);
   else if (strcmp(type,"mtp") == 0)
      status = dsp_mtp(hosin[0],hosout,&combin,&write_flag,&new_file,subcmd);
   else
      status = dsp_tpl(hosin[0],hosout,&combin,&write_flag,&new_file,subcmd);
   }
else if (strcmp(subcmd, "RMDUPS") == 0)
   {
   status = rmdups(&vblock,hosin[0],hosout,&combin,&new_file,&mindist,
            subcmd,prtfile);
   write_flag = TRUE;
   }
else
   {
   if (strcmp(type,"tps") == 0)
      status = bat_tps(&vblock,hosin[0],hosout,&combin,&new_file,subcmd);
   else if (strcmp(type,"mtp") == 0)
      status = bat_mtp(&vblock,hosin[0],hosout,&combin,&new_file,subcmd);
   else
      status = bat_tpl(&vblock,hosin[0],hosout,&combin,&new_file,subcmd);
   write_flag = TRUE;
   }
if (status != E_SUCC)
   c_clean(&vblock,"edittie",hosout,&one);

/* Issue completion message
---------------------------*/
if (write_flag)
   {
   strcpy(outtie[0], hosout);
   if (strlen(prtfile)!=0)
      {
      strcpy(outtie[1], prtfile);
      c_complt("EDITTIE",outtie,&two);
      }
   else
      c_complt("EDITTIE",outtie,&one);

   }
else
   c_complt("EDITTIE"," ",&zero);

} /* edittie */
