/******************************************************************************

   NAME:         RMDUPS

   FUNCTION:     Delete nearly coincident tie points

   PROJECT:      LAS

   ALGORITHM DESCRIPTION:
   Calculate distance between all active tie points
   For every pair with distance less than minimum allowed make the point with
      shortest total distance to all other tiepoints inactive.

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"
#define TERM 0
#define PFILE 1
#define LP 2

int rmdups
(
    struct PARBLK *vblock,
    char hosin[][CMLEN],                 /*  input host TPL file names    */
    char *hosout,                        /*  output host TPL file name    */
    long *combin,                        /*  combine two input files      */
    long *new_file,                      /*  TRUE if no input files       */
    double *mindist,                     /*  minimum distance between points */
    char *subcmd,                        /*  subcommand                   */
    char *prtfile                       /*  output print file            */
)

{ /* rmdups */

struct PTDESC *tie;                  /*  File descriptor              */

struct TPLHEAD head1;                /*  TPL header record            */
struct TPLDATA *data;                /*  TPL data records             */

struct actpt {
   long tpid;                        /*  tie point record number      */
   double totdist;                   /*  distance to all other tie pts */
   struct actpt *next;               /*  pointer to next actpt        */ 
};

struct distance {
   double dist;                      /*  distance between tiept pair  */
   struct actpt *tpid[2];            /*  tiept pair                   */
   struct distance *next;            /*  pointer to next longest pair */
   struct distance *prev;            /*  pointer to next shortest pair */
};

struct distance *current,            /*  pointer to current pair      */
     *top,                           /*  pointer to binary tree top   */
     *tmpptr;                        /*  temporary pointer            */

struct actpt *active,                /*  pointer to tie point         */
     *firstact,                      /*  pointer to tie point         */
     *pointone,                      /*  pointer to tie point         */
     *pointtwo;                      /*  pointer to tie point         */


double x,y;                          /*  distance in respective direction */
long acc =IWRITE;                    /*  file mode access             */
long count=0;                        /*  total number of tie point    */
long nonfatal=1;                     /*  nonfatal error messages      */
long maxtie;                         /*  maximum amount of space allocated */
long pptype=TPL;                     /*  file type                    */
long status;                         /*  function status              */
long line;                           /*  fatal error messages         */
long fatal = -1;                     /*  fatal flag for errmsg        */
long i,j;                            /*  loop counters                */
long done;                           /*  done flag                    */
long activefound;                    /*  number of active points found */
char  tmpfile[CMLEN];                /*  print file name              */
long   print[3];                     /*  print option                 */

FILE *fp;

/* Get print options
   ----------------- */
strcpy(tmpfile,"edittie");
if (c_setprt(vblock,&print[TERM],&print[PFILE],&print[LP],
             tmpfile,prtfile) != E_SUCC)
   c_errmsg("Fatal error encountered","edittie-fatal",&fatal);
fp=fopen(tmpfile,"w");
if (fp==NULL)
   {
   c_errmsg("Unable to create temporary file","edittie-rmdups",&nonfatal);
   return(E_FAIL);
   }

/* Allocate buffer space for data records
   -------------------------------------- */
data = (struct TPLDATA *) calloc(MAX_TIE_POINTS, (sizeof(struct TPLDATA)));
if (data == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","edittie-alloc", &nonfatal);
   return(E_FAIL);
   }
maxtie=MAX_TIE_POINTS;

/* Read the input files
   -------------------- */
if (load_tpl(hosin,combin,new_file,&line,subcmd,&head1,&data,&count,&maxtie)
      != E_SUCC)
   return(E_FAIL);

i=0;
activefound=0;

/* find the first active tie point
   ------------------------------- */

while ((!activefound) && (i<count))
   {
   if((data[i].active<2)||(data[i].active>9))
      {
      active=malloc(sizeof(struct actpt)); 
      active->tpid=i;
      active->totdist=0;
      active->next=NULL;
      firstact=active;
      activefound++;
      }
   i++;
   }

/* find the rest of the active tie points
   -------------------------------------- */
 
for (j=i;j<count;j++)
   {
   if((data[j].active<2)||(data[j].active>9))
      {
      active->next=malloc(sizeof(struct actpt));
      active=active->next;
      active->next=NULL;
      active->tpid=j; 
      active->totdist=0;
      activefound++;
      }
   }
fprintf(fp,
   "Found %d active points out of %d total. Minimum distance is %f pixels.\n\n",
    activefound,count,*mindist);

/* build tree of all pairs of points of MINDIST or less
   ---------------------------------------------------- */

pointone=firstact;
top=malloc(sizeof(struct distance));
top->tpid[0]=NULL;
top->tpid[1]=NULL;
top->dist=*mindist;
top->next=NULL;
top->prev=NULL;
/* check each point ...
   ----------------------- */
while (pointone->next!=NULL)
   {
   pointtwo=pointone->next;
   /* ... against all points that come after that point
      ------------------------------------------------- */
   while(pointtwo!=NULL)
      {
      /* calculate the distance between points using the distance formula.
         -------------------------------------------------- */
      y=data[pointone->tpid].ref_coord[0]-data[pointtwo->tpid].ref_coord[0];
      y*=y;
      x=data[pointone->tpid].ref_coord[1]-data[pointtwo->tpid].ref_coord[1];
      x*=x;
      current=malloc(sizeof(struct distance));
      current->dist=sqrt(y+x);
      pointone->totdist+=current->dist;
      pointtwo->totdist+=current->dist;
      current->tpid[0] = pointone;
      current->tpid[1] = pointtwo;
      current->next=NULL;
      current->prev=NULL;
      if (current->dist>top->dist)
         {
         /* release memory since distance between points is okay
            ---------------------------------------------------- */
         free(current);
         }
      else
         {
         tmpptr=top;
         done=0;
         /* find where it goes in the binary tree
            ------------------------------------- */
         while (!done)
            {
            /* do we need to go shorter?
               ------------------------- */ 
            if (current->dist<=tmpptr->dist)
               {
               /* YES...are there any shorter below this?
                  --------------------------------------- */
               if (tmpptr->prev!=NULL)
                  {
                  /* YES...move down one level
                     ------------------------- */
                  tmpptr=tmpptr->prev;
                  }
               else
                  {
                  /* NO...we have reached the end...stick it here
                     -------------------------------------------- */
                  tmpptr->prev=current;
                  done=1;
                  }
               }
            else
               {
               /* NO...are there any longer below this?
                  ------------------------------------- */
               if (tmpptr->next!=NULL)
                  {
                  /* YES...move down one level
                     ------------------------- */
                  tmpptr=tmpptr->next;
                  }
               else
                  {
                  /* NO...we have reached the end...stick it here
                     -------------------------------------------- */
                  tmpptr->next=current;
                  done=1;
                  }
               }
            }
         }
      pointtwo=pointtwo->next;
      }
   pointone=pointone->next;
   }  

/* starting with the most coincident pair,  make one point inactive
   ---------------------------------------------------------------- */

while (top->prev!=NULL)
   {
   /* starting at the top of the tree
      ------------------------------- */
   current=top;

   /* while there are shorter distances move down the tree
      ---------------------------------------------------- */
   while (current->prev!=NULL)
      {
      tmpptr=current;
      current=current->prev;
      }

   /* remove the current pair from the tree
      ------------------------------------- */
   tmpptr->prev=current->next;

   /* check to see that both points involved are still active
      ------------------------------------------------------- */
   if ((current->tpid[0]->totdist!=0)&&
       (current->tpid[1]->totdist!=0))
      {

      /* update active point count
         ------------------------- */
      activefound--;

      /* find which of the points is more centrally located and
         deactivate that point
         ------------------------------------------------------ */
      if (current->tpid[0]->totdist<current->tpid[1]->totdist)
         {
         active=current->tpid[0]; 
         }
      else
         {
         active=current->tpid[1];  
         }
      data[active->tpid].active=7;
      fprintf(fp,"Distance between %-10s and %-10s is %f     Deactivated %s\n", 
              data[current->tpid[0]->tpid].pt_id,
              data[current->tpid[1]->tpid].pt_id,
              current->dist,data[active->tpid].pt_id);
      pointone=firstact;

      /* go through the list of points and reduce total distance
         by distance from each point to freshly deactivated point
         -------------------------------------------------------- */
      while (pointone!=NULL)
         {
         if ((pointone->tpid!=active->tpid) && (pointone->totdist!=0))
            {
            y=data[pointone->tpid].ref_coord[0]-
                   data[active->tpid].ref_coord[0];
            y*=y;
            x=data[pointone->tpid].ref_coord[1]-
                   data[active->tpid].ref_coord[1];
            x*=x;
            pointone->totdist-=sqrt(y+x);
            }
         pointone=pointone->next;
         } 
      active->totdist=0;
      }

   /* release memory used by newly pruned pair
      ---------------------------------------- */
   free(current); 
   }   

fprintf(fp,"\n%d points remain active.\n", activefound);

/* Free memory
   ----------- */
while (firstact!=NULL)
   {
   active=firstact->next;
   free(firstact);
   firstact=active;
   }
 
/* Write the file
   -------------- */
status = c_ptopen(&tie,hosout,&pptype,&acc,&head1);
if (status!=E_SUCC)
   {
   c_ptclse(&tie);
   return(E_FAIL);
   }
for (i=0;i<count;i++)
   {
   status=c_ptio(&tie, &(data[i]));
   if (status!=E_SUCC)
      {
      c_ptclse(&tie);
      return(E_FAIL);
      }
   }
c_ptclse(&tie);
fclose(fp);
return(E_SUCC);
}
