/******************************************************************************
FUNCTION:	SHEADC

PURPOSE:	SHEADC prints a standard five-line heading in a file which
		will be printed.

  		NOTE:   The print file must be opened prior to calling SHEADC.
         		If the print file was opened with "open" or "creat"
			(which return a file descriptor instead of a file
			pointer), then the program must first call "fdopen"
			to get the file pointer to be used by SHEADC.
   
PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      01/86    K. Zanter		initail development
	   01/86    K. Zanter           PRF #3381 	 
  5.0      06/88    D. Hollaren		

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
The page width of the printer must be between 80 and 132 characters.

PROJECT			LAS

ALGORITHM 
  Check the validity of page width and page number
  Determine string lengths of input arguments
  The current date and time are retrieved
  Each line is then printed according to the standard format

ALGORITHM REFERENCES	none
  NOTE:  There should be five blank spaces at the beginnng of each line.
	 This margin was left in the event that the report would be bound.

  The following is a sample output heading (width = 80):

     Tue. July 7 1988          EROS Data Center            Report No. EXAMPLE
     Time 12:24        Sioux Falls, South Dakota  57198    Page 0001
                              Software Inventory
                                  LAS Modules

*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"

static int header_num = 0;

int sheadc
(
    FILE *fp,		     /* pointer to print file   	       */
    long width,		     /* page width of printer   	       */
    char *name,             /* facility's name			       */
    char *address,          /* facility's address                     */
    long pgnum,	             /* page number		               */
    char *repnum,           /* report number/program name 	       */
    char *title1,           /* first user-supplied title line         */
    char *title2           /* second user-supplied title line        */
)

{
    char wday[6];	     /* current weekday, Sun. - Sat.	       */
    char mon[5];	     /* current mont, Jan. - Dec.	       */

    long addr_len;           /* length of "address"		       */
    long name_len;           /* length of "name"		       */
    long title1_len;         /* length of "title1"		       */
    long title2_len;	     /* length of "title2"		       */

    long day;                /* current day of month, 1 - 31           */
    long yr;		     /* current year 		               */
    long hour;               /* current hour of 24 hour clock          */
    long minute;  	     /* current minute of 24 hour clock        */

    long last_tab_stop;      /* position for last tab stop	       */
    long mid;                /* position for middle of the page        */
    long mid_start_pos;      /* starting position for middle words     */
    long skip;		     /* number of positions to skip over       */
    long start_pos;          /* starting position            	       */
    long word_end;	     /* position for end of middle word        */

    long i;		     /* loop counter		               */
    long n;                  /* number of characters to be written     */
    long nonfatal = 1;       /* nonfatal error code		       */

    char fmt[10];            /* format specification		       */
  
/*
  Check validity of width and page numer
*/
    if (width < 80 )
	{
	c_errmsg("Page width is less than the minimum of 80\n",
		 "fileutil-width",&nonfatal);
	return(E_FAIL);
	}

    if (width > 132)
	{
	c_errmsg("Page width exceeds the upper limit of 132\n",
		 "fileutil-width",&nonfatal);
	return(E_FAIL);
	}
    
    if (pgnum > 9999)
	{
	c_errmsg("Page number exceeds the upper limit of 9999\n",
		 "fileutil-page",&nonfatal);
	return(E_FAIL);
	}

/*
  Determine string lengths of input arguments
*/
    name_len = (long) strlen(name);
    addr_len = (long) strlen(address);
    title1_len = (long) strlen(title1);
    title2_len = (long) strlen(title2);

/*
  Get the current date and time
*/
    getdate_and_time(wday,mon,&day,&yr,&hour,&minute);

/*
  Compute middle of page
*/
    mid = width / 2;

/*
  Compute position of last tab
*/
    last_tab_stop = width - 20;

/*
  NOTE:  There should be five blank spaces at the beginnng of each line.
	 This margin was left in the event that the report would be bound.
*/

/*
  NOTE:  The initial form feed character for the FIRST header being produced
	 caused some problems with LAS applications (such as DITTO) because
	 it was forcing blank pages to be printed before any actual data. 
	 Thus, there should only be a form feed inserted before the header 
	 if it is NOT the first one being printed.
*/
    fprintf (fp,"%s     %s%s", ((header_num++ == 0) ? "" : "\f"), wday, mon);
    if (day < 10)
       fprintf (fp," ");
    fprintf (fp,"%d %d",day,yr);

    mid_start_pos = mid - (name_len / 2);
    if (mid_start_pos < 23) 
	mid_start_pos = 23;

    skip = mid_start_pos - 21;
    for (i = 1; i < skip; i++)
        fprintf(fp," ");

    n = (last_tab_stop - 1) - mid_start_pos;
    sprintf (fmt,"%%-.%ds",n);
    fprintf (fp,fmt,name);

    word_end = mid_start_pos + (name_len - 1);
    if (word_end >= (last_tab_stop - 1))
        word_end = last_tab_stop - 2;

    skip = last_tab_stop - word_end;
    for (i = 1; i < skip; i++)
        fprintf (fp," ");

    fprintf (fp,"Report No. %-.10s\n",repnum);

/*
  Print the second line
*/
    fprintf (fp,"     Time ");
    if (hour < 10)
       fprintf (fp," ");

    fprintf (fp,"%d:%02d",hour,minute);

    mid_start_pos = mid - (addr_len / 2);
    if (mid_start_pos < 17) 
	mid_start_pos = 17;

    skip = mid_start_pos - 15;
    for (i = 1; i < skip; i++)
        fprintf(fp," ");

    n = (last_tab_stop - 1) - mid_start_pos;
    sprintf (fmt,"%%-.%ds",n);
    fprintf (fp,fmt,address);

    word_end = mid_start_pos + (addr_len - 1);
    if (word_end >= (last_tab_stop - 1))
        word_end = last_tab_stop - 2;

    skip = last_tab_stop - word_end;
    for (i = 1; i < skip; i++)
        fprintf (fp," ");

    fprintf (fp,"Page %04d\n",pgnum);

/*
  Print the third line
*/
   start_pos = mid - (title1_len / 2);
   if (start_pos < 6)
       start_pos = 6;

   for (i = 1; i < start_pos; i++)
       fprintf (fp," ");

   n = (width - start_pos) + 1;
   sprintf (fmt,"%%-.%ds\n",n);
   fprintf (fp,fmt,title1);
 
/*
  Print the fourth line
*/
   start_pos = mid - (title2_len / 2);
   if (start_pos < 6)
       start_pos = 6;

   for (i = 1; i < start_pos; i++)
       fprintf(fp," ");

   n = (width - start_pos) + 1;
   sprintf(fmt,"%%-.%ds\n",n);
   fprintf(fp,fmt,title2);
 
/*
  Print a blank line
*/
    fprintf(fp ,"\n");

    return(E_SUCC);
}
