/*
*************************************************************************

NAME: FILTER

PURPOSE: Performs a low-pass, high-pass, or median filter on an
         input image.  Low-pass filters and median filters are
  	 used most often for noise suppression or smoothing, while 
         high-pass filters are typically used for image enhancement. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    4	         8/89   J. Marmon     Changes to work on PN9050
    5	         9/89   B. Davis      Adjusted iamgeio and looping to correctly
				      handle multiple input image specification.
    6	        11/89   T. Baltzer    add threshold parameter 'THRVAL'
				      to -VLOW, -ILOW, -HIGH.
				      (rewrite entire filter coding to
				      accomplish this!)
    7		 4/92	S. Nelson     Added output min/max options, HIGHVAL
				      and LOWVAL, for subcommand -HIGH.
    8		 8/92	D. Etrheim    Corrected bug in med filter for I*2 data
    9		 9/92	T. Mittan     Added extern to curent to eliminate 
				      warnings on link
   10           11/98   L. Johnson    Modified thresholded high pass filter to
                                      use an adaptive threshold based on the
                                      kernel standard deviation.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
       Call GETPAR to get the user-specified parameters and tae 
       global variables
       Open output image 
       For each input band
          Call appropriate processing subroutines to perform
   	  the desired filter
       Update the output image DDR, min/max values, and history

*****************************************************************
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "imageio.h"
#include "filt.h"
#include "util.h"
#include "protos.h"

/*
global declarations that will be declared in individual subroutines by filter.h
*/
struct PARBLK parblk;                  	/* TAE parameter block     */

long	min_max;                       	/* minmax option flag      */
long 	msgtim;                        	/* message interval        */
extern	long	curent;			/* current buffer */
struct FDESC *  infd[MAXIMG + 1];              	/* file descriptors        */
struct FDESC *	outfd;                         	/* file descriptors        */
char	prog_name[7];			/* program name            */
 

int main
(void) 
{
double		line_inc = 1;        /* line interval for upddr        */
static double	maxbnd[MAXBND+1];    /* output image maximum value     */
static double	minbnd[MAXBND+1];    /* output image minimum value     */
double		samp_inc = 1;        /* sample interval for upddr      */
float		thrperc;             /* minimum fraction of valid      */
                                     /* samples that must be in kernel */
static long	bands[MAXIMG][MAXBND+1];  /* image band numbers        */
long	comb_flag = NOCOMB;          /* for upddr                      */
long	dtype;                       /* input image data type          */
long	intype;			     /* data type in RAM	       */
static long	nbands[MAXIMG];      /* no. bands in each image        */
long	nl;                          /* no. lines in image             */
long	nmrimg;                      /* no. images                     */
long	ns;                          /* no. samples/line in image      */
long	otype;                       /* output image data type         */
long	out_flag[3];		     /* output min/max flag            */
long	stat = 0;                    /* status                         */
long	totbnd;                      /* total no. bands                */
float	highval;		     /* maximum value for output       */
float	minflt;			     /* float version of minpix	       */
float	lowval;			     /* minimum value for output       */
static	long	one = 1;	     /* constant		       */
static long	window[MAXIMG][4];   /* image windows                  */
char	prog_name[7];	    	     /* program name		       */
static char	hosin[MAXIMG][CMLEN];
char	hosout[1][CMLEN];
char	taeout[1][CMLEN];
long	zero = 0;		     /* constant		       */
long	totsam;			     /* total samples per band 	       */
struct	FSTR	fstr;		     /* filter parameter structure     */
long	nonfatal = 1;		     /* non-fatal error status	       */


strcpy(prog_name,"FILTER");
/*
	get user-specified parameters and tae global variables
	------------------------------------------------------
*/
getpar (hosin, taeout, hosout, &totbnd, bands, nbands, &nl, &ns,
	window, &nmrimg, fstr.subcmd, &(fstr.nsk), &(fstr.nlk),
	&thrperc, &(fstr.addback), &(fstr.scalf), &(fstr.minval),
	&(fstr.maxval), &(fstr.thresh), &otype, &(fstr.offset),
	&(fstr.invld), &dtype, &(fstr.tolrn),&lowval,&highval,out_flag);


/*
	set filter processing parameters
	--------------------------------
*/
fstr.minpix = 1;
if (thrperc  !=  0.0)
   {
   minflt = thrperc * (float) ( fstr.nlk * fstr.nsk);
   fstr.minpix = (long) minflt;
   if ( (float)((long) minflt) != minflt )  fstr.minpix++;
   if (fstr.minpix < 1) fstr.minpix = 1;
   }


if ((fstr.subcmd[0] == 'M') ||		/* median filter, or */
    (((fstr.subcmd[0] == 'H') ||	/* threshold filter  */
      (fstr.subcmd[0] == 'V') ||
      (fstr.subcmd[0] == 'I')) &&
      (fstr.thresh != 0.0)))
   intype = dtype;
else
   intype = EREAL;

if (intype == EREAL)
   {
   filtold(hosin,taeout,hosout,totbnd,bands,nbands,nl,ns,window,nmrimg,
	   thrperc,&fstr,otype,lowval,highval,out_flag);
   return EXIT_SUCCESS;
   }

/*
	initialize display message
	--------------------------
*/
totsam = nl * ns;
c_inittm ("FILTER", "Processed", "sample", "bands", " ", &totsam,
          &totbnd, &msgtim, &zero);

/*
	initialize min and max arrays
	-----------------------------
*/
if ( c_inmnmx(&intype, &totbnd, minbnd, maxbnd) != E_SUCC )
   {
   c_errmsg("Fatal error encountered","Filter-fatal",&nonfatal);
   return EXIT_FAILURE;
   }


/*
	process the image(s)
	--------------------
*/

switch (intype)
   {
   case EREAL:
   	       break;
   case ELONG: 
   	       break;
   case EWORD: stat = pro_wrd(nmrimg,window,bands,nbands,totbnd,hosin,
			      hosout[0],otype,min_max,maxbnd,minbnd,&fstr,
			      lowval,highval,out_flag);
   	       break;
   case EBYTE: stat = pro_byt(nmrimg,window,bands,nbands,totbnd,hosin,
			      hosout[0],otype,min_max,maxbnd,minbnd,&fstr,
			      lowval,highval,out_flag);
   	       break;
   }
if (stat == E_FAIL)  return EXIT_FAILURE;


/*
	update ddr and minmax and for output image
	------------------------------------------
*/
if ( c_upddr(&parblk, hosin, hosout[0], &nmrimg, window, bands,
             nbands, &line_inc, &samp_inc, &totbnd, &comb_flag) == E_FAIL )
   {
   c_errmsg("Nonfatal error encountered","Filter-warn",&nonfatal);
   }

if (min_max)
   {
   if ( c_svmnmx(hosout[0], &totbnd, minbnd, maxbnd, "YES") == E_FAIL )
      {
      c_errmsg("Nonfatal error encountered","Filter-warn",&nonfatal);
      }
   }

/*
	create associated history file for output image
	-----------------------------------------------
*/
if ( c_hstout(&parblk, hosout[0], hosin, &nmrimg, " ") == E_FAIL )
   {
   c_errmsg("Nonfatal error encountered","Filter-warn",&nonfatal);
   }

/*
	output completion message
	-------------------------
*/
c_complt ("FILTER", taeout, &one);

return EXIT_SUCCESS;

}    /* end of filter()  */
