/*
*************************************************************************
NAME: filtold

PURPOSE: Performs a low-pass, high-pass, filter on an
         input image.  Low-pass filters are
  	 used most often for noise suppression or smoothing, while 
         high-pass filters are typically used for image enhancement. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    4	         8/89   J. Marmon     Changes to work on PN9050
    5	         9/89   B. Davis      Adjusted iamgeio and looping to correctly
				      handle multiple input image specification.
    6		 4/92	S. Nelson     Added min and max values for output.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

*****************************************************************

	arguments:
	name		description
	----		-----------
	hosin		input host names
	taeout
	hosout		output host name
	totbnd		total bands in output image
	bands		image band numbers
	nbands		number of bands per image
	nl		number of lines in image window
	ns 		number of samples in image window
	window		image window dimensions
	nmrimg		number of images
	thrperc		threshold percentage
	fstr		filter parameter structure
	otype		output image data type
	lowval          minimum value for output
	highval		maximum value for output
	out_flag	out min/max flag

*****************************************************************
*/


#include <stdio.h>
#include "filter.h"
#include "filt.h"
#include "util.h"
#include "protos.h"

float	*sum;		/* pointer to the middle of sum array */

int filtold
(
    char hosin[MAXIMG][CMLEN],
    char taeout[][CMLEN],
    char hosout[][CMLEN],
    long totbnd,
    long bands[MAXIMG][MAXBND+1],
    long nbands[MAXIMG],
    long nl,
    long ns,
    long window[MAXIMG][4],
    long nmrimg,
    double thrperc,
    struct FSTR *fstr,
    long otype,
    double lowval,
    double highval,
    long out_flag[3]
)

{
long    one = 1;
long    fatal_err = -1; 


double		line_inc = 1;        /* line interval for upddr        */
static double		maxval[MAXBND+1];    /* output image maximum value     */
static double		minval[MAXBND+1];    /* output image minimum value     */
double		samp_inc = 1;        /* sample interval for upddr      */

float		kersum;              /* sum of data in the kernel      */

long	access;
long	bndout;                      /* output band number             */
long	comb_flag = NOCOMB;          /* for upddr                      */
long	curimg;                    
long	i;
long	intype=EREAL;                      
long	onecat = ONECAT;             /* for inittm                     */
long	opt[2];                      /* for eopenr                     */
long	status;                      /* status                         */
long	valsum;                      /* no. elements in the kernel     */
long	xhalf;                       /* half kernel x dimension        */
long	yhalf;                       /* half kernel y dimension        */
long	frac;                        /* threshold flag                 */
long	nonfatal = 1;		     /* non-fatal error status	       */



/*
	set "threshold" logical
	-----------------------
*/
if (thrperc  !=  0.0) 
   frac = TRUE;
else
   frac = FALSE;

/*
	set filter processing parameters
	--------------------------------
*/
xhalf = (fstr->nsk - 1) / 2;
yhalf = (fstr->nlk - 1) / 2;


/*
	initialize display message
	--------------------------
*/
c_inittm ("FILTER", "Processed", "line", "band", " ", &nl, 
          &totbnd, &msgtim, &onecat);

/*
	initialize min and max arrays
	-----------------------------
*/
if ( c_inmnmx(&intype, &totbnd, minval, maxval) != E_SUCC )
   {
   c_errmsg("Fatal error encountered","Filter-fatal",&nonfatal);
   return(E_FAIL);
   }

/*
	open output image
	-----------------
*/
opt[0] = otype;
opt[1] = 0;
access = IWRITE;
status = c_eopenr(&outfd, hosout[0], &totbnd, &one, &one, &nl, &ns, &access, 
                  &intype, opt, &one);
if (status  !=  E_SUCC) 
   c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);

/*
	initialize output band counter
	------------------------------
*/
bndout = 0;
curcat = 1;
for (curimg = 0; curimg < nmrimg; curimg++)
   {
/*
	   open input image
	   ----------------
*/
   initio (hosin[curimg], window[curimg], &intype, &curimg);

   }
/*
	   process each band
	   -----------------
*/
for (curimg = 0; curimg < nmrimg; curimg++)
   {
   for (i = 0; i < nbands[curimg]; i++, curcat++)
      {
      bndout++;
      switch (fstr->subcmd[0])
         {
         case 'H': prehi(hosout,xhalf,yhalf,&ns,&curimg,
			 &bands[curimg][i],
			 fstr->maxval,fstr->minval,&kersum,&valsum);
         	   status = sldhi(hosout,nl,&ns,&curimg,&bands[curimg][i],
			          &bndout,xhalf,yhalf,
			          fstr->nsk,fstr->maxval,fstr->minval,frac,
			          &kersum,&valsum,fstr->invld,fstr->addback,
			          fstr->scalf,fstr->minpix,fstr->offset,minval,
			          maxval,lowval,highval,out_flag);
	           break;
         case 'V': prehi(hosout,xhalf,yhalf,&ns,&curimg,
			 &bands[curimg][i],
			 fstr->maxval,fstr->minval,&kersum,&valsum);
         	   status = sldvl(hosout,nl,&ns,&curimg,&bands[curimg][i],
			          &bndout,xhalf,yhalf,
			          fstr->nsk,fstr->maxval,fstr->minval,frac,
			          &kersum,&valsum,fstr->invld,fstr->scalf,
			          fstr->minpix,minval,maxval);
	           break;
         case 'I': prehi(hosout,xhalf,yhalf,&ns,&curimg,
			 &bands[curimg][i],
			 fstr->maxval,fstr->minval,&kersum,&valsum);
         	   status = sldil(hosout,nl,&ns,&curimg,&bands[curimg][i],
			          &bndout,xhalf,yhalf,
			          fstr->nsk,fstr->maxval,fstr->minval,frac,
			          &kersum,&valsum,fstr->scalf,
			          fstr->minpix,minval,maxval);
	           break;
         case 'L': prelo(hosout,xhalf,yhalf,&ns,&curimg,
			 &bands[curimg][i],&kersum);
         	   status = sldlo(hosout,nl,&ns,&curimg,&bands[curimg][i],
			           &bndout,xhalf,yhalf,
			           fstr->nsk,&kersum,minval,maxval);
	           break;
         case 'D': prelo(hosout,xhalf,yhalf,&ns,&curimg,
			 &bands[curimg][i],&kersum);
         	   status = slddp(hosout,nl,&ns,&curimg,&bands[curimg][i],
			          &bndout,xhalf,yhalf,
			          fstr->nsk,&kersum,minval,maxval,fstr->tolrn);
	           break;
         }
      }
   }

/*
	   close input and output images
	   -----------------------------
*/
   clos_img(&infd[0]);
   clos_img(&outfd);

/*
	   quit if problem encountered processing data
	   -------------------------------------------
*/
   if (status == E_FAIL)   return(E_FAIL);

/*
	update ddr and minmax and for output image
	------------------------------------------
*/
if ( c_upddr(&parblk, hosin, hosout[0], &nmrimg, window, bands,
             nbands, &line_inc, &samp_inc, &totbnd, &comb_flag) == E_FAIL )
   {
   c_errmsg("Nonfatal error encountered","Filter-warn",&nonfatal);
   }

if (min_max)
   {
   if ( c_svmnmx(hosout[0], &totbnd, minval, maxval, "YES") == E_FAIL )
      {
      c_errmsg("Nonfatal error encountered","Filter-warn",&nonfatal);
      }
   }

/*
	create associated history file for output image
	-----------------------------------------------
*/
if ( c_hstout(&parblk, hosout[0], hosin, &nmrimg, " ") == E_FAIL )
   {
   c_errmsg("Nonfatal error encountered","Filter-warn",&nonfatal);
   }

/*
	output completion message
	-------------------------
*/
c_complt ("FILTER", taeout, &one);

return(E_SUCC);
}
