/*
*************************************************************************
NAME: GETPAR

PURPOSE: gets the user-specified parameters and tae globals

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    4           11/89   T. Baltzer    add parameter 'THRVAL'
    5           11/98   L. Johnson    removed kernel limitation on thresholded
                                      filters

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name			description
	----			-----------
	bands                   band specification
	dtype			data type of input image
	high			high end of valid data range
	highval			maximum value of output image
	hosin  		        host names of input images
	hosout			host name of output image
	invld			option to use for invalid samples
	ksamps			number of samples per line in kernel
	klines			number of lines in kernel
	low			low end of valid data range
	lowvalue		minimum value of output image
	minmax			indicator for whether the user wants the
				min and max values determined for each band
	msgtim			elapsed time in seconds between display messages
	nbands 			number of bands in each image
	nl			number of lines to be processed
	nmrimg			number of input images
	norm			value from offset parameter
	ns			number of samples per line to be processed
	otype			output data type
	out_flag		output min/max flag
	subcmd			specified subcommand
	taeout			tae name of output image
	thresh			threshold range for valid data
	totbnd			total number of output bands
	window			window specification
	xadb			percentage of center kernel value to add back
	xfrc			minimum percentage of valid samples
				that must be in a kernel
	xmlt			gain factor
	
	kernel			kernel dimensions
	length			length specification
	qflag			flag indicating if a subwindow was specified
	same			flag indicating if images have same window size
	status			function return status code
	stdout			logical unit number for standard output
	taein			tae names of input images
	xfrac			value of threshold percentage
************************************************************************
*/
#include "filter.h"
#include "util.h"
#include "protos.h"
 
void getpar
(
    char hosin[][CMLEN],
    char taeout[][CMLEN],
    char hosout[][CMLEN],
    long *totbnd,
    long bands[][MAXBND+1],
    long nbands[],
    long *nl,
    long *ns,
    long window[][4],
    long *nmrimg,
    char subcmd[],
    long *ksamps,
    long *klines,
    float *xfrc,
    float *xadb,
    float *xmlt,
    float *low,
    float *high,
    float *thresh,
    long *otype,
    float *norm,
    long *invld,
    long *dtype,
    float *tolrn,
    float *lowval,
    float *highval,
    long out_flag[3]
)

{

long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;
long    one = 1;

struct VARIABLE *var;

long	ftype = EREAL;
long	itype = ELONG;
long	kernel[2];
long	length;
long	maximg = MAXIMG;
long	qflag[MAXIMG][2];
long	same;
long	status;
long	std_out;
long	two = 2;
long	xfrac;

char	minmax[4];
char	odtype[5];
char	taein[MAXIMG][CMLEN];

out_flag[0] = TRUE;
out_flag[1] = TRUE;
out_flag[2] = TRUE;
/*
	initialization
	--------------
*/
c_inlas (&parblk, &std_out);

/*
	get globals                     
	-----------
*/
var = p_find( &parblk, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &parblk, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

/*
	get subcommand
	--------------
*/
c_getstr (&parblk, "_SUBCMD", &one, &length, subcmd);

/*
	get input file information
	--------------------------
*/
status = c_getin(&parblk, "IN", &zero, taein, hosin, nmrimg, window, bands, 
                 nbands, totbnd, &maximg, &same, qflag);
if (status  !=  E_SUCC) 
   c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);

if (same  !=  1) 
   c_errmsg ("Input images do not have the same window size","filter-windsiz",
	     &fatal_err);

if (subcmd[0] ==  'M') 
   {
   status = c_gettyp(hosin, nmrimg, dtype);
   if (status  !=  E_SUCC) 
      {
      c_errmsg ("Input images are not of same data type", "filter-diftyp",
		 &inf_err);
      c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);
      } 

   if ((*dtype  ==  ELONG)  ||  (*dtype  ==  EREAL)) 
       c_errmsg ("Median filters allow only BYTE or INTEGER*2 input", 
          "filter-mdtype", &fatal_err);
   } 

*nl = window[0][NL];
*ns = window[0][NS];

/*
	get output file name
	--------------------
*/
status = c_getout (&parblk, "OUT", taeout, hosout);
if (status  !=  E_SUCC) 
   c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);

/*
	get kernel dimensions
	---------------------
*/
c_getnum (&parblk, "KERNDIM", &two, &itype, kernel);
*klines = kernel[0];
*ksamps = kernel[1];

/*
	validate kernel dimensions
	--------------------------
*/
if ((*klines % 2)  !=  1  ||  (*ksamps % 2)  !=  1)
   c_errmsg ("Invalid kernel size -- Kernel values must be odd", 
       "filter-odd", &fatal_err);
else if(*ksamps + *klines  <  4) 
   c_errmsg ("Invalid kernel size -- Too small", "filter-small", &fatal_err);
else if((*ksamps  >  *ns)  ||  (*klines  >  *nl)) 
   c_errmsg ("Invalid kernel size -- Too large", "filter-large", &fatal_err); 
else if ( (subcmd [0] ==  'M')  &&  
   ((*klines  >  51)  ||  (*ksamps  >  51)) ) 
   c_errmsg ("Invalid kernel size -- Median filters limited to 51 by 51", 
      "filter-medlim", &fatal_err);

/*
	get output data type
	--------------------
*/
c_getstr (&parblk, "ODTYPE", &one, &length, odtype);
if (strcmp (odtype, "SAME") == 0) *otype = 0;
else if (strcmp (odtype, "BYTE") == 0) *otype = 1;
else if (strcmp (odtype, "I*2") == 0) *otype = 2;
else if (strcmp (odtype, "I*4") == 0) *otype = 3;
else if (strcmp (odtype, "R*4") == 0) *otype = 4;
if (*otype  ==  0) 
   {
   status = c_gettyp(hosin, nmrimg, otype);
   if (status  !=  E_SUCC) 
      {
      c_errmsg ("Input images are not of same data type", "filter-diftyp",
		 &inf_err);
      c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);
      } 
   } 

/*
	 get remaining parameters
	 ------------------------
*/
if (strcmp (subcmd, "DESPIKE")  == 0)
   c_getnum (&parblk, "TOLVAL", &one, &ftype, tolrn);
else if (strcmp (subcmd, "VLOW") == 0)
   {
   c_getnum (&parblk, "MINVAL", &one, &ftype, low);
   c_getnum (&parblk, "MAXVAL", &one, &ftype, high);
   if (*high  <=  *low) 
      c_errmsg ("Invalid values for MIN and MAX", "filter-minmax", &fatal_err);
   if (c_getnum (&parblk, "THRVAL", &one, &ftype, thresh) == 0)
      *thresh = 0.0;    /* THRVAL is null , set thresh to zero */
   else                 /* THRVAL is an actual value, check it  */
      {
      if (*thresh <= 0.0)
         c_errmsg ("Invalid value for THRVAL", "filter-thrval", &fatal_err);
      }
   c_getnum (&parblk, "SCALFACT", &one, &ftype, xmlt);
   if (*xmlt  ==  0.0) *xmlt = 1.0;
   c_getnum (&parblk, "INVALOPT", &one, &itype, invld);
   }
else if (strcmp (subcmd, "ILOW") == 0)
   {
   c_getnum (&parblk, "MINVAL", &one, &ftype, low);
   c_getnum (&parblk, "MAXVAL", &one, &ftype, high);
   if (*high  <=  *low) 
      c_errmsg ("Invalid values for MIN and MAX", "filter-minmax", &fatal_err);
   if (c_getnum (&parblk, "THRVAL", &one, &ftype, thresh) == 0)
      *thresh = 0.0;    /* THRVAL is null , set thresh to zero */
   else                 /* THRVAL is an actual value, check it  */
      {
      if (*thresh <= 0.0)
         c_errmsg ("Invalid value for THRVAL", "filter-thrval", &fatal_err);
      }
   c_getnum (&parblk, "SCALFACT", &one, &ftype, xmlt);
   if (*xmlt  ==  0.0) *xmlt = 1.0;
   }
else if (strcmp (subcmd, "HIGH")  == 0)
   {
   c_getnum (&parblk, "MINVAL", &one, &ftype, low);
   c_getnum (&parblk, "MAXVAL", &one, &ftype, high);
   if (*high  <=  *low) 
      c_errmsg ("Invalid values for MIN and MAX", "filter-minmax", &fatal_err);
   if (c_getnum (&parblk, "THRVAL", &one, &ftype, thresh) == 0)
      *thresh = 0.0;    /* THRVAL is null , set thresh to zero */
   else                 /* THRVAL is an actual value, check it  */
      {
      if (*thresh <= 0.0)
         c_errmsg ("Invalid value for THRVAL", "filter-thrval", &fatal_err);
      }
   c_getnum (&parblk, "THRPERC", &one, &itype, &xfrac);
   *xfrc = xfrac/100.;
   c_getnum (&parblk, "SCALFACT", &one, &ftype, xmlt);
   if (*xmlt  ==  0.0) *xmlt = 1.0;
   c_getnum (&parblk, "ADDBACK", &one, &ftype, xadb);
   c_getnum (&parblk, "OFFSET", &one, &ftype, norm);
   c_getnum (&parblk, "INVALOPT", &one, &itype, invld);

   /* Out_flag dimensioned for three values.  When assigning output, 
      out_flag[2] eliminates one test when LOWVAL and HIGHVAL are both null
     ---------------------------------------------------------------------- */
   if (c_getnum (&parblk, "LOWVAL", &one, &ftype, lowval) == 0)
      out_flag[0] = FALSE;
   if (c_getnum (&parblk, "HIGHVAL", &one, &ftype, highval) == 0)
      out_flag[1] = FALSE;
   if (!out_flag[0] && !out_flag[1])
      out_flag[2] = FALSE;
   } 

if (((strcmp (subcmd, "HIGH") == 0) ||
    (strcmp (subcmd, "VLOW") == 0) ||
    (strcmp (subcmd, "ILOW") == 0)) &&
    (*thresh != 0.0))
   {
   status = c_gettyp(hosin, nmrimg, dtype);
   if (status  !=  E_SUCC) 
      {
      c_errmsg ("Input images are not of same data type", "filter-diftyp",
		 &inf_err);
      c_errmsg ("Fatal error encountered", "filter-fatal", &fatal_err);
      } 

   if ((*dtype  ==  ELONG)  ||  (*dtype  ==  EREAL)) 
       c_errmsg ("THRVAL filters allow only BYTE or INTEGER*2 input", 
          "filter-tdtype", &fatal_err);
   } 
}
