/*
************************************************************************
NAME: pro_wrd

PURPOSE:  process an image window (all specified bands, i*2 data)

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           02/90   T. Baltzer   
    2		04/92	S. Nelson	Added output min/max options, HIGHVAL
					and LOWVAL, for subcommand -HIGH.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

	Loop thru reading/processing/writing blocks of data within
	a specified image window, for all of the specified bands.  Quit
	when all of the appropriate data has been processed, or an error
	occurs.  The inner loop loops thru bands to minimize the image
	i/o calls.  All specified images will be processed by one
	call to this routine.
	The output bands will be written to the output image in 
	sequential order.

*****************************************************************
	arguments:
	name		description
	----		-----------
	nmrimg		number of images (input)
	window		window of image (input)
	bands		band numbers to be processed (input)
	nbands		number of bands in each image (input)
	totbnd		total number of bands (input)
	hosin		input image host name (input)
	hosout		output image host name (input)
	otype		output data type (input)
	min_max		min, max flag from TAE land (input)
	maxbnd		array of max pixel values (output)
	minbnd		array of min pixel values (output)
	fstr		filter parameter structure (input)
	lowval		minimum value for output
	highval		maximum value for output
	out_flag	output min/max flag
	
*****************************************************************
*/


#include <stdio.h>
#include "worgen.h"
#include "imageio.h"
#include "filter.h"
#include "filt.h"
#include "util.h"
#include "protos.h"


int pro_wrd
(
    long nmrimg,
    long window[MAXIMG][4],
    long bands[MAXIMG][MAXBND+1],
    long nbands[MAXIMG],
    long totbnd,
    char hosin[MAXIMG][CMLEN],
    char hosout[CMLEN],
    long otype,
    long min_max,
    double maxbnd[MAXBND+1],
    double minbnd[MAXBND+1],
    struct FSTR *fstr,
    double lowval,
    double highval,
    long out_flag[3]
)

{

long    inf_err = 1;

short	*buffer;	/* data buffer to read */
short	*obuff;		/* data buffer to write */
long	*histgm;	/* histogram array */
long	numh = HIST_SIZE; /* number of elements in 'histgm' */
long	dtype = EWORD;	/* data type of 'buffer' */
long	nlbuf;		/* number of lines in 'buffer' */
long	nsbuf;		/* number of samples in 'buffer' */
long	nlav;		/* number of actual valid lines in 'buffer' */
long	nsav;		/* number of actual valid samples in 'buffer' */
long	nlavo;		/* number of actual valid lines in 'obuff' */
long	nsavo;		/* number of actual valid samples in 'obuff' */
long	nlob;		/* number of lines in the output buffer */
long	nsob;		/* number of samples in the output buffer */
long	nlb;		/* number of lines in the boundary */
long	nsb;		/* number of samples in the boundary */
long	sldat[MAXIMG];	/* starting line of input image */
long	ssdat[MAXIMG];	/* starting sample of input image */
long	sldato;		/* starting line of output image */
long	ssdato;		/* starting sample of output image */
long	nextflg;	/* next flag for input image */
long	nextflp;	/* next flag for output image */
long	fill = MIRR;	/* boundary fill technique */
struct FDESC *	fdesc[MAXIMG];	/* input image file descriptor */
struct FDESC *  odesc;		/* output image file descriptor */
long	stat;		/* error return status */
long	maxram;		/* maximum available bytes in RAM */
long	oband;		/* output band number */
long	i, im;		/* counter */
long	nli;		/* number of lines in the image */
long	nsi;		/* number of samples in the image */
long	ftthru = 1;	/* first time thru flag	*/


maxram = MAXRAM;

nli = window[0][2];
nsi = window[0][3];
  
if ( size_bnd(&(fstr->nlk),&(fstr->nsk),&nlb,&nsb) == E_FAIL )
   return(E_FAIL);

if ( size_buf(&nli,&nsi,&(fstr->nlk),&(fstr->nsk),&nlb,&nsb,&dtype,&maxram,
              &nlbuf,&nsbuf,&nlob,&nsob) == E_FAIL )
   return(E_FAIL);

if ( (buffer = aloc_buf(nlbuf,nsbuf,dtype)) == NULL )  return(E_FAIL);
if ( (obuff = aloc_buf(nlob,nsob,dtype)) == NULL )	    return(E_FAIL);
if ( (histgm = aloc_buf(1,numh,ELONG)) == NULL )	    return(E_FAIL);

nextflg = INIT;		/* first call to get_buf() */
nextflp = CREAT;	/* first call to put_buf() */

sldato = 1;		/* initialize output starting origins */
ssdato = 1;

for (im = 0; im < nmrimg; im++)  /* init input starting origins */
   {
   sldat[im] = window[im][0];
   ssdat[im] = window[im][1];
   }

oband = 1;		/* initialize output band number */

c_startm();

for (  ; ;  )
   {
   for (im = 0; im < nmrimg; im++)
      {
      for (i = 0; i < nbands[im]; i++)
         {
         stat = get_buf(buffer,&nlbuf,&nsbuf,&nlav,&nsav,&window[im][0],
			&window[im][1],&nli,&nsi,&nlb,&nsb,&sldat[im],
			&ssdat[im],&nextflg,&bands[im][i],&fill,
			&fdesc[im],&hosin[im][0],&dtype);
         if (stat  !=  E_SUCC) 
            {
            if (stat == E_EOIW)
	       stat = E_SUCC;
	    if ( free_buf(buffer) == E_FAIL )  return(E_FAIL);
	    if ( free_buf(obuff)  == E_FAIL )  return(E_FAIL);
	    if ( free_buf(histgm) == E_FAIL )  return(E_FAIL);
	    for (i = 0; i <= im; i++)
	       if ( c_eclose(&fdesc[i]) == E_FAIL )
		 c_errmsg("Nonfatal error encountered","filter-warn",&inf_err);
	    if ( c_eclose(&odesc) == E_FAIL )
	       c_errmsg("Nonfatal error encountered","filter-warn",&inf_err);
	    c_stoptm();
            return(stat);		/* done processing */
            }

         /* apply the filter here */
	 stat = app_wrd(buffer, obuff, histgm, numh, nsbuf, nlav, nsav,
	     		nlb, nsb, fstr, lowval, highval, out_flag);
         if (stat == E_FAIL)
            {
	    if ( free_buf(buffer) == E_FAIL )  return(E_FAIL);
	    if ( free_buf(obuff)  == E_FAIL )  return(E_FAIL);
	    if ( free_buf(histgm) == E_FAIL )  return(E_FAIL);
	    for (i = 0; i <= im; i++)
	       if ( c_eclose(&fdesc[i]) == E_FAIL )
		 c_errmsg("Nonfatal error encountered","filter-warn",&inf_err);
	    if ( c_eclose(&odesc) == E_FAIL )
	       c_errmsg("Nonfatal error encountered","filter-warn",&inf_err);
	    c_stoptm();
            return(E_FAIL);
            }

         nlavo = nlav - (2 * nlb);
         nsavo = nsav - (2 * nsb);
         stat = put_buf(obuff,&nlob,&nsob,&nlavo,&nsavo,&sldato,&ssdato,
		        &nli,&nsi,&nextflp,&oband,&totbnd,&odesc,hosout,
		        &dtype,&otype);
         if (stat  !=  E_SUCC) 
            {
	    if ( free_buf(buffer) == E_FAIL )  return(E_FAIL);
	    if ( free_buf(obuff)  == E_FAIL )  return(E_FAIL);
	    if ( free_buf(histgm) == E_FAIL )  return(E_FAIL);
	    for (i = 0; i <= im; i++)
	       if ( c_eclose(&fdesc[i]) == E_FAIL )
		 c_errmsg("Nonfatal error encountered","filter-warn",&inf_err);
	    if ( c_eclose(&odesc) == E_FAIL )
	       c_errmsg("Nonfatal error encountered","filter-warn",&inf_err);
	    c_stoptm();
            return(E_FAIL);
            }
         /* calculate band min, max value for the ddr */
         if (min_max)
            extr_wrd(obuff,nsob,nlavo,nsavo,oband,maxbnd,minbnd);

         oband++;		/* increment output band number    */
         nextflg = CURNT;	/* read, write all bands before    */
         nextflp = CURNT;	/* moving to next block		   */

         }   /* bottom of for i =  construct */

      if (ftthru == 1)
         nextflg = INIT;	/* first open on input image	   */
      else
         nextflg = NEXT;

      }    /* bottom of for im =  construct */

   ftthru = 0;		/* no longer first time thru		   */
   oband = 1;		/* reset starting output band number	   */
   nextflg = NEXT;	/* move to next block before reading image */
   nextflp = NEXT;	/* move to next block before writing image */
   curent = curent + (nlavo * nsavo); /* increment sample counter  */

   }    /* bottom of infinite loop */


}    /* end of pro_wrd()   */
