/*
************************************************************************
NAME: set_h_wrd

PURPOSE: Using the HIGH filter, calculate the value of a pixel within
	 the output buffer (take into account the threshold parameter,
	 INTEGER*2 data).

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           03/90   T. Baltzer   
    2		04/92	S. Nelson     Added output min/max options, HIGHVAL
				      and LOWVAL, for subcommand -HIGH.
    3           11/98   L. Johnson    Modified thresholded high pass filter to
                                      use an adaptive threshold based on the
                                      kernel standard deviation.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

*****************************************************************
	arguments:
	name		description
	----		-----------
	minval		minimum valid value (input)
	maxval		maximum valid value (input)
	thresh		threshold value (input)
	histgm		histogram array (input)
	tmp1		temporary variable (input)
	tmp2		temporary variable (input)
	scalf		scale factor (input)
	invld		invalid pixel option (input)
	minpix		minimum number of pixels (input)
	iadd		input buffer address (input)
	oadd		output buffer address (output)
	lowval		minimum value for output
	highval		maximum value for output
	out_flag	output min/max flag

************************************************************************
*/
#include "las.h"
#include "typlim.h"
#include <math.h>
#include "protos.h"

int set_h_wrd
(
    double minval,
    double maxval,
    double thresh,
    long *histgm,
    double tmp1,
    double tmp2,
    double scalf,
    long invld,
    long minpix,
    short *iadd,
    short *oadd,
    double lowval,
    double highval,
    long out_flag[3]
)

{

float	outflt;		/* floating point output */
long	kersum;		/* kernel summation */
long	nkers;		/* number of pixels in sum 'kersum' */
long	i;		/* counter */
float   mean;           /* kernel mean */
float   stddev;         /* kernel standard deviation */


if ((float)(*iadd) < minval)		/* less than minimum value */
   {
   switch (invld)
      {
      case 1:   outflt = minval - 1.0;
		if (outflt < MINWORD)  outflt = MINWORD;
		else if (outflt > MAXWORD)  outflt = MAXWORD;
		*oadd = (short) outflt;
		break;
      case 2:	*oadd = 0;
		break;
      case 3:	*oadd = *iadd;
		break;
      }
   }
else if ((float)(*iadd) > maxval)	/* more than maximum value */
   {
   switch (invld)
      {
      case 1:   outflt = maxval + 1.0;
		if (outflt < MINWORD)  outflt = MINWORD;
		else if (outflt > MAXWORD)  outflt = MAXWORD;
		*oadd = (short) outflt;
		break;
      case 2:	*oadd = MAXUTINY;
		break;
      case 3:	*oadd = *iadd;
		break;
      }
   }
else					/* within bounds */
   {
   kersum = 0;				/* sum the appropriate range */
   nkers = 0;				/*  of the histogram 	     */
   for ( i = (long) minval; i <= maxval; i++)
      {
      kersum += (i * histgm[i]);
      nkers += histgm[i];
      }

   mean = (float)kersum/(float)nkers;    /* the kernel mean */

   /* Compute the kernel standard deviation.
   -----------------------------------------*/
   stddev = 0.;
   mean -= (float)minval;
   for (i = (long) minval; i<=maxval; i++, mean-=1.)
      stddev += mean*mean*(float)histgm[i];
   stddev = (float)sqrt((double)stddev/(double)(nkers - 1));

   /* Find the new min and max.
   ----------------------------*/
   if ((float)(*iadd) - thresh*stddev > minval)
      minval = (float)(*iadd) - thresh*stddev;
   if ((float)(*iadd) + thresh*stddev < maxval)
      maxval = (float)(*iadd) + thresh*stddev;

   kersum = 0;
   nkers = 0;
   for (i = (long) minval; i<=maxval; i++)
      {
      kersum += i*histgm[i];
      nkers += histgm[i];
      }

   mean = (float)kersum/(float)nkers;

   if (nkers < minpix)
      {
      *oadd = 0;	/* not enough valid pixels */
      }
   else
      {
      outflt = tmp1 + (float)(*iadd)*tmp2 - mean*scalf;
      if (outflt < MINWORD)  outflt = MINWORD;
      else if (outflt > MAXWORD)  outflt = MAXWORD;
      *oadd = (short) outflt;
      }
   }

/* Check for output min and max.  First check if both values are null.
  -------------------------------------------------------------------*/
if (out_flag[2])
   {
   if (out_flag[0])
      if (*oadd < lowval)
	 *oadd = lowval;
   if (out_flag[1])
      if (*oadd > highval)
	 *oadd = highval;
   }

return(E_SUCC);

}    /*  end of set_h_wrd()  */
