/*
************************************************************************
NAME: SIZE_BUF

PURPOSE: determines the size of the input and output buffer required
by the sliding window.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/89   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
	The following function determines the maximum allowable size
	for the input and output buffers.  The following assumptions
	are made:
	  (1) The sum of the input buffer and the output buffer should
	      not exceed maxram bytes.
	  (2) The output buffer size will be that of the input buffer
	      less the boundary.  Since the output buffer size is a 
	      function of the input buffer size, both buffer sizes must
	      be calculated simultaneously.
	  (3) An approximately square buffer is the most efficient
	      buffer shape for general processing.
*****************************************************************
	arguments:
	name		description
	----		-----------
	 nli		number of lines in the input image (input)
	 nsi		number of samples in the input image (input)
	 nlk		number of lines in the kernal (input)
	 nsk		number of samples in the kernal (input)
	 nlb		number of lines in the boundary (input)
	 nsb		number of samples in the boundary (input)
	 dtype		data type (input)
	 maxram		maximum buffer memory size in bytes (input)
	 nlib		number of lines in the input buffer (output)
	 nsib		number of samples in the input buffer (output)
	 nlob		number of lines in the output buffer (output)
	 nsob		number of samples in the output buffer (output)

************************************************************************
*/

#include "worgen.h"
#include "protos.h"

int size_buf
(
    long *nli,
    long *nsi,
    long *nlk,
    long *nsk,
    long *nlb,
    long *nsb,
    long *dtype,
    long *maxram,
    long *nlib,
    long *nsib,
    long *nlob,
    long *nsob
)
{

long bsize = 1; /* number of bytes per data element       */
long status;	/* return status flag			  */
long nonfatal = 1; /* non-fatal error status           */
long maxsam;	/* number of maximum samples */
long total;	/* total buffer size */
long lors;	/* line or sample flag */


/* Error check the input parameters.				*/

status = err_sbuf(*nli,*nsi,*nlk,*nsk,*nlb,*nsb,*dtype,*maxram);
if (status == E_FAIL)
   return(E_FAIL);

switch (*dtype)
   {
   case EREAL:  bsize = 4;  break;
   case ELONG:  bsize = 4;  break;
   case EWORD:  bsize = 2;  break;
   case EBYTE:  bsize = 1;  break;
   }

maxsam = *maxram / bsize;

*nlib = *nlk;
*nsib = *nsk;
*nlob = *nlib - ( *nlb * 2 );
*nsob = *nsib - ( *nsb * 2 );
total = *nlib * *nsib + *nlob * *nsob;

if ( total > maxsam )
   {
   c_errmsg("Buffer too small for full kernel","filter-smallb",&nonfatal);
   c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
   return(E_FAIL);	/* Not enough memory to fit in one kernel */
   }

for (  ;  ;  )
   {
   if ( *nlob < *nli )	/* max lines not reached */
      {
      if ( *nsob < *nsi )	/* max samples not reached */
         {
         if ( *nsib > *nlib )	/* more samples than lines */
	    {
	    lors = 0;		/* increment lines */ 
	    }
	 else
	    {
	    lors = 1;		/* increment samples */ 
	    }
         }
      else			/* max samples reached */
         {
	 lors = 0;		/* increment lines */
	 }
      }
   else				/* max lines reached */
      {
      if ( *nsob < *nsi )	/* max samples not reached */
         {
	 lors = 1;		/* increment samples */
	 }
      else
	 {
	 lors = 2;		/* max lines and samples reached */
	 }
      }

   switch (lors)
      {
      case  0: 			/* increment lines */
         (*nlib)++;
         (*nlob)++;
         total += *nsib + *nsob;
         if (total > maxsam)
            {
            (*nlib)--;
            (*nlob)--;
	    return(E_SUCC);
            }
	 break;
      case 1:			/* increment samples */
         (*nsib)++;
         (*nsob)++;
         total += *nlib + *nlob;
         if (total > maxsam)
            {
            (*nsib)--;
            (*nsob)--;
	    return(E_SUCC);
            }
	 break;
      case 2:			/* all done */
	 return(E_SUCC);

      }	/* end of switch construct */

   }	/* end of infinite loop */


}     /*  end of size_buf()   */
