/*
*************************************************************************
NAME: slddp 

PURPOSE: applies a despike filter to an image band.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    3.1          4/89   D. Hollaren   Dynamically allocated large arrays in
				      order to compile on PN9050
    3.2		 2/91   T. Baltzer    reduce unneccessarily large arrays

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name		description
	----		-----------
	curbnd		current input band number
	curimg		current input image number
	hosout		out image host name
	kersum		data sum of the kernel
	ksamps		number of samples per line in kernel
	maxval		maximum values for each band
	minval		minimum values for each band
	nl		number of lines to be processed
	ns		number of samples per line to be processed
	otype		output data type
	subcmd		specified subcommand
	tolrn		tolerance value
	xhalf		half of the number of samples in the kernel
	yhalf		half of the number of lines in the kernel

	internal variables:
	name		description
	----		-----------
	addlin		counter for lines to be added sequentially
			to bottom of kernel
	bindex		the index into the "bottom" array
	bottom		the lines to read to mirror at the bottom 
	dellin		counter for lines to be deleted sequentially
			from the top of the kernel
	lpos		position for left edge
	status		function return status code
	tindex		the index into the "top" array
	totpix		total number of pixels in the kernel
	top		the lines to read to mirror at the top
************************************************************************
*/
#include "filter.h"
#include "filt.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

int slddp
(
    char hosout[][CMLEN],
    long nl,
    long *ns,
    long *curimg,
    long *curbnd,
    long *bndout,
    long xhalf,
    long yhalf,
    long ksamps,
    float *kersum,
    double minval[],
    double maxval[],
    double tolrn
)

{
extern	float	*sum;

long    one = 1;
long    inf_err = 1;

float	mean;
float	totpix;

long	addlin;
long	bindex;
long	*bottom;
long	curlin;
long	dellin;
long	ereal = EREAL;
long	j;
long	k;
long	l;
long	lpos;
long	nsk;
long	status;
long	tindex;
long	nbytes;			/* Number of bytes to allocate memory for */
long	*top;


float *addbuf;
float *delbuf;
float *inbuf;
float *outbuf;

nsk = (xhalf * 2) + 1;
nbytes = (long) ((*ns + nsk) * sizeof(float));

totpix = (float) ((xhalf * 2 + 1) * (yhalf * 2 + 1));

/*
  Allocate memory for the add buffer
*/
top = (long *) malloc(nsk * sizeof(long));
if (top == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

addbuf = (float *) malloc(nbytes);
if (addbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the delete buffer
*/
delbuf = (float *) malloc(nbytes);
if (delbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the input buffer
*/
inbuf = (float *) malloc(nbytes);
if (inbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the output buffer
*/
outbuf = (float *) malloc(nbytes);
if (outbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the 'bottom' buffer
*/
bottom = (long *) malloc(nsk * sizeof(long));
if (bottom == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
	initialization
	--------------
*/
for (j = 0; j < yhalf; j++)
   {
   top[j] = yhalf + 1 - j;
   bottom[j]= nl - j - 1;
   }

tindex = 0;
bindex = 0;
dellin = 1;
addlin = yhalf + 2;

curlin = curent = 1;
c_startm ();

for (curlin = 1; curlin <= nl; curlin++, curent++)
   {

/*
	   get the input line 
	   ------------------
*/
   status = c_eread (&infd[*curimg], curbnd, &curlin, inbuf, &one);
   if (status  !=  *ns) 
      {
      clos_img(&outfd);
      c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
               prog_name, hosout, &one);
      }

   lpos = 1 - xhalf;

   for (k = 0; k < *ns; k++)
      {

      mean = *kersum / totpix;

      if ( abs (inbuf[k] - mean) > tolrn )
         {
         outbuf[k] = mean;
         }
      else
         {
         outbuf[k] =  inbuf[k];
         }

/*
	      reset kernel sum and sum valid values
	      -------------------------------------
*/
      if (k  <  *ns - 1) 
         {
         *kersum -= sum[lpos];
         *kersum += sum[lpos + ksamps];
         lpos++;
         } 
      }

/*
	   determine min and max values of output line
	   -------------------------------------------
*/
   if (min_max)
      {
      status = c_pxmnmx(outbuf,ns,&minval[*bndout-1],&maxval[*bndout-1],&ereal);
      if (status  !=  E_SUCC) 
         {
         clos_img(&outfd);
         c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                  prog_name, hosout, &one);
         }
      }

/*
	   write completed line to output image
	   ------------------------------------
*/
   status = c_ewrite (&outfd, bndout, &curlin, outbuf, &one);
   if (status  !=  *ns) 
      {
      clos_img(&outfd);
      c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
               prog_name, hosout, &one);
      }
   
   if (curlin  <  nl) 
      {
/*
	      get next line to be deleted from the top of the kernel
	      ------------------------------------------------------
*/
      if (curlin  >=  yhalf + 1) 
         {
         status = c_eread (&infd[*curimg], curbnd, &dellin, delbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         dellin++;
         }
      else
         {
         status = c_eread (&infd[*curimg], curbnd, &top[tindex], delbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         tindex++;
         } 

/*
	      get next line to be added to the bottom of the kernel
	      -----------------------------------------------------
*/
      if ( (curlin + 1)  <=  (nl - yhalf) ) 
         {
         status = c_eread (&infd[*curimg], curbnd, &addlin, addbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         addlin++;
         }
      else
         {
         status = c_eread (&infd[*curimg], curbnd, &bottom[bindex],
			   addbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         bindex++;
         } 

/*
	      reset the sum array and kernel sum value
	      ----------------------------------------
*/
      *kersum = 0;
      for (k = 1 - xhalf; k <= *ns + xhalf; k++)
         {
         set (k, &l, *ns);

/*
	         determine the value to be added,
	         determine the value to be subtracted
	         ------------------------------------
*/

         sum[k] = sum[k] - delbuf[l] + addbuf[l];
         if (k  <=  xhalf + 1) 
            {
            *kersum += sum[k];
            } 
         }
      } 
   }

c_stoptm ();

if ( free_buf(addbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(delbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(inbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(outbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(top) == E_FAIL )  return(E_FAIL);
if ( free_buf(bottom) == E_FAIL )  return(E_FAIL);

return(E_SUCC);
}
