/*
************************************************************************
NAME: sldhi 

PURPOSE: applies a high pass filter to an image band.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    3.1          4/89   D. Hollaren   Dynamically allocated large arrays in
				      order to compile on PN9050
    3.2		 2/91   T. Baltzer    reduce unneccessarily large arrays
    3.3		 4/92	S. Nelson     Added min and max values for output.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name		description
	----		-----------
	curbnd		current input band number
	curimg		current input image number
	frac		threshold flag
	high		high end of valid data range
	highval		maximum value for output
	hosout		out image host name
	invld		option to use for invalid samples
	kersum		data sum of the kernel
	ksamps		number of samples per line in kernel
	low		low end of valid data range
	lowval		minumum value for output
	maxval		maximum values for each band
	minval		minimum values for each band
	nl		number of lines to be processed
	norm		value from offset parameter
	ns		number of samples per line to be processed
	valsum		the valid number of samples which contributed
			to the kernel sum
	xadb		percentage of center kernel value to add back
	xhalf		half of the number of samples in the kernel
	xmin		minimum number of valid samples that must
			be in a kernel
	xmlt		gain factor
	yhalf		half of the number of lines in the kernel

	internal variables:
	name		description
	----		-----------
	addlin		counter for lines to be added sequentially
			to bottom of kernel
	bindex		the index into the "bottom" array
	bottom		the lines to read to mirror at the bottom 
	dellin		counter for lines to be deleted sequentially
			from the top of the kernel
	lpos		position for left edge
	status		function return status code
	tindex		the index into the "top" array
	top		the lines to read to mirror at the top
************************************************************************
*/

#include "filter.h"
#include "typlim.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

int sldhi
(
    char hosout[][CMLEN],
    long nl,
    long *ns,
    long *curimg,
    long *curbnd,
    long *bndout,
    long xhalf,
    long yhalf,
    long ksamps,
    double high,
    double low,
    long frac,
    float *kersum,
    long *valsum,
    long invld,
    double xadb,
    double xmlt,
    long xmin,
    double norm,
    double minval[],
    double maxval[],
    double lowval,
    double highval,
    long out_flag[3]
)

{
extern	float	*sum;
extern	short	*vsum;

long    inf_err = 1;
long    one = 1;

float		x1;
float		x2;

long	addlin;
long	bindex;
long	*bottom;
long	curlin;
long	dellin;
long	ereal = EREAL;
long	j;
long	k;
long	l;
long	lpos;
long	nsk;
long	status;
long	tindex;
long	nbytes;			/* Number of bytes to allocate memory for */
long	*top;

float *addbuf;
float *delbuf;
float *inbuf;
float *outbuf;

nsk = (xhalf * 2) + 1;
nbytes = (long) ((*ns + nsk) * sizeof(float));

/*
  Allocate memory for the add buffer
*/
top = (long *) malloc(nsk * sizeof(long));
if (top == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

addbuf = (float *) malloc(nbytes);
if (addbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the delete buffer
*/
delbuf = (float *) malloc(nbytes);
if (delbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the input buffer
*/
inbuf = (float *) malloc(nbytes);
if (inbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the output buffer
*/
outbuf = (float *) malloc(nbytes);
if (outbuf == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
  Allocate memory for the bottom buffer
*/
bottom = (long *) malloc(nsk * sizeof(long));
if (bottom == NULL)
    {
    clos_img(&outfd);
    c_errmsg("Error allocating dynamic memory.","filter-alloc",&inf_err);
    c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
    }

/*
	initialization
	--------------
*/
for (j = 0; j < yhalf; j++)
   {
   top[j] = yhalf + 1 - j;
   bottom[j]= nl - j - 1;
   }

tindex = 0;
bindex = 0;
dellin = 1;
addlin = yhalf + 2;

x1 = norm * (1.0 - xadb);
x2 = xmlt * (1.0 + xadb);

curlin = curent = 1;
c_startm ();

for (curlin = 1; curlin <= nl; curlin++, curent++)
   {

/*
	   get the input line 
	   ------------------
*/
   status = c_eread (&infd[*curimg], curbnd, &curlin, inbuf, &one);
   if (status  !=  *ns) 
      {
      clos_img(&outfd);
      c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
               prog_name, hosout, &one);
      }

   lpos = 1 - xhalf;

   for (k = 0; k < *ns; k++)
      {
      if ((frac)  &&  (*valsum  <  xmin)) 
         {
         outbuf[k] = 0;
         } 
      else if (inbuf[k] <  low) 
         {
	 switch (invld)
	    {
	    case 1: outbuf[k] = low - 1;
		    break;
	    case 2: outbuf[k] = 0;
		    break;
	    case 3: outbuf[k] = inbuf[k];
		    break;
	    }
         }
      else if (inbuf[k] >  high) 
         {
	 switch (invld)
	    {
	    case 1: outbuf[k] = high + 1;
		    break;
	    case 2: outbuf[k] = MAXUTINY;
		    break;
	    case 3: outbuf[k] = inbuf[k];
		    break;
	    }
         } 
      else
         { 
         if (*valsum  ==  0) 
	    {
            outbuf[k] = x1 + inbuf[k] * x2;
	    }
         else
	    {
            outbuf[k] = x1 + inbuf[k] * x2 -
			*kersum / (float) *valsum * xmlt;
	    }
         } 


/*
	      reset kernel sum and sum valid values
	      -------------------------------------
*/
      if (k  <  *ns - 1) 
         {
         *kersum -= sum[lpos];
         *kersum += sum[lpos + ksamps];
         *valsum -= vsum[lpos];
         *valsum += vsum[lpos + ksamps];
         lpos++;
         } 

      /* if min/max values for output were set, check and reassign
        ----------------------------------------------------------- */
      if (out_flag[2])
	 {
	 if (out_flag[0])
	    if (outbuf[k] < lowval)
	 	outbuf[k] = lowval;
	 if (out_flag[1])
	    if (outbuf[k] > highval)
	    	outbuf[k] = highval;
	 }
      }

/*
	   determine min and max values of output line
	   -------------------------------------------
*/
   if (min_max)
      {
      status = c_pxmnmx(outbuf,ns,&minval[*bndout-1],&maxval[*bndout-1],&ereal);
      if (status  !=  E_SUCC) 
         {
         clos_img(&outfd);
         c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                  prog_name, hosout, &one);
         }
      }

/*
	   write completed line to output image
	   ------------------------------------
*/
   status = c_ewrite (&outfd, bndout, &curlin, outbuf, &one);
   if (status  !=  *ns) 
      {
      clos_img(&outfd);
      c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
               prog_name, hosout, &one);
      }
   
   if (curlin  <  nl) 
      {
/*
	      get next line to be deleted from the top of the kernel
	      ------------------------------------------------------
*/
      if (curlin  >=  yhalf + 1) 
         {
         status = c_eread (&infd[*curimg], curbnd, &dellin, delbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         dellin++;
         }
      else
         {
         status = c_eread (&infd[*curimg], curbnd, &top[tindex], delbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         tindex++;
         } 

/*
	      get next line to be added to the bottom of the kernel
	      -----------------------------------------------------
*/
      if ( (curlin + 1)  <=  (nl - yhalf) ) 
         {
         status = c_eread (&infd[*curimg], curbnd, &addlin, addbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         addlin++;
         }
      else
         {
         status = c_eread (&infd[*curimg], curbnd, &bottom[bindex],
			   addbuf, &one);
         if (status  !=  *ns) 
            {
            clos_img(&outfd);
            c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
                     prog_name, hosout, &one);
            }
         bindex++;
         } 

/*
	      reset the sum array and kernel sum value
	      ----------------------------------------
*/
      *kersum = 0;
      *valsum = 0;
      for (k = 1 - xhalf; k <= *ns + xhalf; k++)
         {
         set (k, &l, *ns);

/*
	         determine the value to be added,
	         determine the value to be subtracted
	         ------------------------------------
*/
	 if ((addbuf[l] >= low) && (addbuf[l] <= high))
	    {
	    sum[k] += addbuf[l];
	    vsum[k]++;
	    }
	 if ((delbuf[l] >= low) && (delbuf[l] <= high))
	    {
	    sum[k] -= delbuf[l];
	    vsum[k]--;
	    }

         if (k  <=  xhalf + 1) 
            {
            *kersum += sum[k];
            *valsum += vsum[k];
            } 
         }
      } 
   }

c_stoptm ();

if ( free_buf(addbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(delbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(inbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(outbuf) == E_FAIL )  return(E_FAIL);
if ( free_buf(top) == E_FAIL )  return(E_FAIL);
if ( free_buf(bottom) == E_FAIL )  return(E_FAIL);

return(E_SUCC);
}
