/*====================================================================
   NAME:                          CHKWIN

   FUNCTION:
      CHKWIN verifies the correctness of the window being specified.

  PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions
====================================================================*/
#include "las.h"
#include "fixlin.h"

void chkwin(
   long  totblk,       /*  I: number of blocks to be fixed           */
   long  iwind[10][4], /*  I: bad block definition(sl,ss,nl,ns)      */
   long  rwind[10][2], /*  I: replacement definition(sl,ss)          */
   long  wind1[4],     /*  I: window specification of IN1            */
   long  wind2[4],     /*  I: window specification of IN2            */
   long  nmrhis[10]    /*  I: number of histograms to be built       */
   )

   {
   long  fatal = -1;   /*  fatal error flag constant                 */
   long  h1;           /*  temporary variable                        */
   long  i;            /*  counter--current block                    */
   long  j;            /*  counter--current block                    */
   long  k;            /*  counter--current value                    */
   long  ln;           /*  temporary variable                        */
   long  one = 1;      /*  constant one                              */
   long  px;           /*  temporary variable                        */
   long  t1;           /*  temporary variable                        */
   long  t2;           /*  temporary variable                        */
   long  temp[4];      /*  temporary buffer                          */

   /*----------------------------------------------------------------
   Sort the WINDOW1 block specifications so that the bad image blocks 
   are in ascending line order. Any change in the WINDOW1 elements will 
   require the same change in the replacement blocks defined in WINDOW2
   ----------------------------------------------------------------*/

   for(i = 0; i < (totblk - 1); i++)
      {
      for(k = 0; k < 4; k++)
         temp[k] = iwind[i][k];

      for(j = (i+1); j < totblk; j++)
         {
         if (temp[0] > iwind[j][0])
            {
            for(k = 0; k < 4; k++)
               {
               iwind[i][k] = iwind[j][k];
               iwind[j][k] = temp[k];
               temp[k] = iwind[i][k];
               }
            
            t1 = rwind[i][0];
            t2 = rwind[i][1];
            rwind[i][0] = rwind[j][0];
            rwind[i][1] = rwind[j][1];
            rwind[j][0] = t1;
            rwind[j][1] = t2;
            h1 = nmrhis[i];
            nmrhis[i] = nmrhis[j];
            nmrhis[j] = h1;
            }
         }
      }

   /*----------------------------------------------------------------
      Check to see if two distinct bad image blocks have the same line 
      range. If so an error will result.
   ----------------------------------------------------------------*/
   for(i = 0; i < (totblk - 1); i++)
      if ((iwind[i][0] + iwind[i][2]) >= iwind[i+1][0])
      {
         c_errmsg("Blocks defined by INWIND overlap","fixlin-overlap",
                                &one);
         c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   /*----------------------------------------------------------------
      For each bad block defined by WINDOW1 the following steps are 
      taken.
         1. If elements of WINDOW2 are the default values THEN set the 
            WINDOW2 elements equal to starting pixel and line of WINDOW1
         2. If the replacement block is NOT within the window of IN2 
            then write an error message. Likewise if the bad image block 
            defined by WINDOW1 is NOT within the window of IN2 then 
            write an error message.
         3. Redefine the starting pixel and starting line of WINDOW1 to 
            correspond to the window specification of IN1.
   ----------------------------------------------------------------*/
   for(i = 0; i < totblk; i++)
      {
      px = iwind[i][3];
      ln = iwind[i][2];
      if ((rwind[i][1] == 0) && (rwind[i][0] == 0))
         {
         rwind[i][1] = iwind[i][1];
         rwind[i][0] = iwind[i][0];
         }
      
      if ((iwind[i][1] < wind1[1]) || (iwind[i][0] < wind1[0]) ||
           (iwind[i][1] + px > wind1[1] + wind1[3]) ||
           (iwind[i][0] + ln > wind1[0] + wind1[2]))
         {
         c_errmsg("Window specification error in INWIND","fixlin-badwin",
                                &one);
         c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
         }

      if ((rwind[i][1] < wind2[1]) || (rwind[i][0] < wind2[0]) ||
           (rwind[i][1] + px > wind2[1] + wind2[3]) ||
           (rwind[i][0] + ln > wind2[0] + wind2[2]))
         {
         c_errmsg("Window specification error in SUWIND","fixlin-badwin",
                                &one);
         c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
         }
 
      iwind[i][1] += (- wind1[1] + 1);
      iwind[i][0] += (- wind1[0] + 1);
      rwind[i][1] += (- wind2[1] + 1);
      rwind[i][0] += (- wind2[0] + 1);
      }
   }
