/*====================================================================
   NAME:                          FIXLIN

   FUNCTION:
      FIXLIN is a module which fixes block(s) of data within an image 
      that are corrupted.  Up to five blocks of data may be fixed at 
      one time.  The input images can be sub-windowed and only one 
      band of an image may be fixed at one time.  The input images may 
      be BYTE,INTEGER*2, INTEGER*4, or REAL*4. The output image will 
      be the same data type as the first input image.

      FIXLIN optionally computes the minimum and maximum pixel values 
      of the output image.  It also optionally copies all history 
      records of the input images into its history file.  These options 
      are defined by the TAE global variables of "minmax" and "histry".

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        3.1      5/91    D.Etrheim          CSB      Renamed some common
                                                     blocks be unique.
        3.2      1/99    L.Johnson          CSB      Changed vblock to
                                                     real*8 for ALL
                                                     platforms due to
                                                     apparent alignment
                                                     problems between
                                                     Fortran and C.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions

   ALGRITHM(FLOW):
      The TAE input paramters are retreived.  These parameters consist of:
          1. IN(1)--the input image to be fixed
          2. IN(2)--the input image which is to be used to fix the first
          3. OUT--the fixed output image to be created
          4. INWIND#--the block definition of the area(s) to be fixed
              (sl,ss,nl,ns)
          5. SUWIND#--the block definition of the area(s) that will be
              used to fix the bad data block(s) (sl,ss)
          6. NHIST--the number of sub-histograms to be created
      The global variables consist of:
          1. MINMAX--the flag to create the min max file or not
          2. MSGTIM--the elapsed time between display messages
      The min max values of the two input images are fiquired, if the
   min max values can not be found within the images' minmax file.  These
   values will be used later on to fiquire the histograms.  Error checking 
   is done on the two input images to be sure that they do not contain more 
   than one band each.  Then the INWIND# and SUWIND# parameters are checked 
   to be sure that they do not overlap and that they are within the window 
   specifications of IN(1) and IN(2) respectively.

      Once the TAE parameters are inputted and checked for errors, the 
   input and output images are opened and control is passed to one of the 
   five proccessing subroutines depending on the data type of the first 
   input image.  The processing subroutine reads the data one line at a 
   time from IN(1) up to the first band data block and writes this good 
   data to the output image.  The histogram subroutine is called to create 
   histogram(s) of the the proximity of the bad data in IN(1) and the 
   proximity and bad data block in IN(2).

      The number of histograms created is specified by the paramter NHIST.
   These histograms are then used to create a lookup table that contains 
   the transformed pixel values that will replace the bad data.

      For every line of bad data the good samples just before and just 
   after the bad data block are read and copied into the output buffer.  
   Then one sample at a time the values fo the bad data are replaced by a 
   value found in look up table that corresponds to the value of the 
   replacement data in IN(2).  When all the samples for that line are 
   fixed the line is written to the output image file.

      If the minmax flag is set to "yes" then the minmax values of the 
   output image are fiquired and saved in the minmax file.  If the histry 
   flag is set to "yes" then the history file is created and updated.  All 
   the files are closed and cataloged and the complete message is displayed.
====================================================================*/
#include "las.h"
#include "imageio.h"    /* FDESC and GDESC structures         */
#include "fixlin.h"
#include "util.h"       /* c_clean, c_hstout, c_complt protos */

long offset[4];          /* offsets into 'bufout' for each band   */
struct FDESC *fdesc1;     /*  file descriptor of IN(1)             */
struct FDESC *fdesc2;     /*  file descriptor of IN(2)             */
struct FDESC *fdesc3[2];  /*  file descriptor for OUT              */
struct GDESC *gdesc;      /*  group descriptor for OUT             */


void main()
   {
   char minmx[4];      /*  minmx flag                                */
   char hosin1[CMLEN]; /*  host input file name 1                    */
   char hosin2[CMLEN]; /*  host input file name 2                    */
   char hosout[CMLEN]; /*  host output file name                     */
   char taein1[CMLEN]; /*  TAE input file name 1                     */
   char taein2[CMLEN]; /*  TAE input file name 2                     */
   char taeout[CMLEN]; /*  TAE output file name                      */

   double linein;      /*  Line increment for updating the DDR       */
   double sampin;      /*  Sample increment for updating the DDR     */

   long band1[MAXBND+1]; /*  band number of IN(1)                    */
   long band2[MAXBND+1]; /*  band number of IN(2)                    */
   long dtype;         /*  data type of IN(1) and OUT images         */
   long fatal = -1;    /*  fatal error flag                          */
   long flag;          /*  Flag for function call                    */
   long iwind[10][4];  /*  bad data block definition(sl,ss,nl,ns)    */
   long msgtim;        /*  time between display messages             */
   long nbands;        /*  number of bands per input image           */
   long nimg;          /*  number of images inputted                 */
   long imglimit = 1;  /*  number of images allowed for input        */
   long nmrhis[10];    /*  number of histograms to be built          */
   long odtype;        /*  output image data type                    */
   long one = 1;       /*  constant one                              */
   long rwind[10][2];  /*  replacement block definition(sl,ss)       */
   long status;        /*  error status                              */
   long totblk;        /*  total number of blocks to be fixed        */
   long vmax;          /*  maximum pixel value                       */
   long vmin;          /*  minimum pixel value                       */
   long wind1[4];      /*  window specification of IN(1)             */
   long wind2[4];      /*  window specification of IN(2)             */

   struct PARBLK vblock;    /*  TAE parameter block                  */


   /*  Input the parameters and check for errors.
   ------------------------------------------------------------------*/
   getpar(&vblock, taein1, hosin1, taein2, hosin2, taeout, hosout,
          &totblk, iwind, rwind, wind1, nmrhis, minmx, &msgtim, band1,
          band2, wind2, &nimg, imglimit, &nbands);

   init_imgs(hosin1, hosin2, &dtype, band1, band2, wind1, wind2, &vmin, 
             &vmax, totblk, iwind, rwind, nmrhis, nimg, nbands, &odtype);

   if (dtype == EBYTE)
      prbyte(&vblock, totblk, iwind, rwind, dtype, wind1, wind2, 
             nmrhis, hosout, minmx, band1, band2, vmin, vmax, msgtim, 
             odtype, nbands);
   else if (dtype == EWORD)
      prword(&vblock, totblk, iwind, rwind, dtype, wind1, wind2, 
             nmrhis, hosout, minmx, band1, band2, vmin, vmax, msgtim, 
             odtype, nbands);
   else if (dtype == ELONG)
      {
      if ((vmax-vmin) <  NMRLEV)
         prlong(&vblock, totblk, iwind, rwind, dtype, wind1, wind2,
                nmrhis, hosout, minmx, band1, band2, vmin, vmax, msgtim, 
                odtype, nbands);
      else
         {
         dtype = EREAL;
         prreal(&vblock, totblk, iwind, rwind, dtype, wind1, wind2,
                nmrhis, hosout, minmx, band1, band2, msgtim, odtype, nbands);
         dtype = ELONG;
         }
      }
   else if (dtype == EREAL)
      prreal(&vblock, totblk, iwind, rwind, dtype, wind1, wind2, 
             nmrhis, hosout, minmx, band1, band2, msgtim, odtype, nbands);


   /*  Update the DDR file
   ------------------------------------------------------------------*/
   linein = 1.0;
   sampin = 1.0;
   flag = NOCOMB;

   status = c_upddr(&vblock, &hosin1, hosout, &nimg, &wind1, band1, &nbands, 
                    &linein, &sampin, &nbands, &flag);
   if (status != E_SUCC)
      {
      c_errmsg("DDR file not updated", "fixlin-ddr", &one);
      c_clean(&vblock, "FIXLIN", hosout, &one);
      }

   /*  Update the history file
   ------------------------------------------------------------------*/
   status = c_hstout(&vblock, hosout, &hosin1, &nimg, "FIXLIN");
   if (status != E_SUCC)
      {
      c_errmsg("History not created", "fixlin-history", &one);
      c_clean(&vblock, "FIXLIN", hosout, &one);
      }

   /*  Close both the input and output files
   ------------------------------------------------------------------*/
   status = c_eclose(&fdesc1);
   if (status != E_SUCC)
   {
      c_errmsg("Error closing the file descriptor for IN(1)", "fixlin-desc",
         &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
   }

   status = c_eclose(&fdesc2);
   if (status != E_SUCC)
   {
      c_errmsg("Error closing the file descriptor for IN(2)", "fixlin-desc",
         &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
   }

   status = c_egclse(&gdesc);
   if (status != E_SUCC)
   {
      c_errmsg("Error closing the group descriptor", "fixlin-desc", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
   }

   /*  Call complt to end the routine
   ------------------------------------------------------------------*/
   c_complt("FIXLIN", taeout, &one);
   }
