/*====================================================================
   NAME:                          GETPAR

   FUNCTION:
      GETPAR retrieves the parameters.
  
   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions
====================================================================*/
#include <string.h>
#include "las.h"
#include "fixlin.h"
#include "util.h"     /* c_inlas, c_getin, c_getout proto */
#include "tae.h"      /* p_find proto */

void getpar (
   struct PARBLK *vblock, /*  O: TAE parameter block                 */
   char  taein1[CMLEN],   /*  O: TAE input file name 1               */
   char  hosin1[CMLEN],   /*  O: host input file name 1              */
   char  taein2[CMLEN],   /*  O: TAE input file name 2               */
   char  hosin2[CMLEN],   /*  O: host input file name 2              */
   char  taeout[CMLEN],   /*  O: TAE output file name                */
   char  hosout[CMLEN],   /*  O: host output file name               */
   long  *totblk,      /*  O: total number of blocks to fix          */
   long  iwind[10][4], /*  O: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  O: replacement block (sl,ss)              */
   long  wind1[4],     /*  O: window specification of IN(1)          */
   long  nmrhis[10],   /*  O: number of histograms to build          */
   char  minmx[4],     /*  O: minmax flag                            */
   long  *msgtim,      /*  O: time between display messages          */
   long  band1[MAXBND+1], /*  O: band number of IN(1)                */
   long  band2[MAXBND+1], /*  O: band number of IN(2)                */
   long  wind2[4],     /*  O: window specification of IN(2)          */
   long  *nimg,        /*  O: number of images inputted              */
   long  imglimit,     /*  I: maximum number of images allowed       */
   long  *nbands       /*  O: number of bands per input image        */
   )

   {
   long  count;        /* # of values passed to FIXLIN               */
   long  count2;       /*      */
   long  fatal = -1;   /* fatal error flag                           */
   long  i;            /* counter-number of histograms               */
   long  one = 1;      /* constant one                               */
   long  qflag[MAXIMG][2];     /* window and/or band flag                    */
   long  same;         /* 1 if all windows are the same, 0 if not    */
   long  status;       /* error status                               */
   long  std_out = 0;  /* standard output                            */
   long  totbl2;       /* total number of blocks to fix              */
   long  totbnds;      /* total number of bands                      */
   long  zero = 0;     /* constant zero                              */
   long  nbands1;      /* number of bands for hosin1                 */
   long  nbands2;      /* number of bands for hosin2                 */

   struct VARIABLE *in;  /* pointer to TAE par block var             */

   c_inlas(vblock, &std_out);

   /*-----------------------------------------------------------------
      Receive the two input images. The number of bands within each must 
      be one else an error results.
   ------------------------------------------------------------------*/
   status = c_getin(vblock, "IN", &zero, taein1, hosin1, nimg, wind1, band1,
                    &nbands1, &totbnds, &imglimit, &same, qflag);
   if (status == E_FAIL)
      {
      c_errmsg("Error getting the IN(1) parameter", "fixlin-getin", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   if (totbnds > 1)
      {
      c_errmsg("Too many bands specified in the input image", 
               "fixlin-band", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   status = c_getin(vblock, "IN", &one, taein2, hosin2, nimg, wind2, band2,
                    &nbands2, &totbnds, &imglimit, &same, qflag);
   if (status == E_FAIL)
      {
      c_errmsg("Error getting the IN(2) parameter", "fixlin-getin", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   if (nbands1 != nbands2)
      {
      c_errmsg("Number of bands in both images is not equal", "fixlin-band",
        &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }
   else
      *nbands = nbands1;

   if (totbnds > 1)
      {
      c_errmsg("Too many bands specified in the input image", 
               "fixlin-band", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   /*  Receive the output image name.
   ------------------------------------------------------------------*/
   status = c_getout(vblock, "OUT", taeout, hosout);
   if (status == E_FAIL)
      {
      c_errmsg("Error getting the OUT parameter", "fixlin-getout", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   /*  Receive the bad image data block definitions.
   ------------------------------------------------------------------*/
   *totblk = 0;
   totbl2 = 0;

   in = p_find(vblock, "INWIND1");
   count = in->v_count;
   for( i = 0; i < count; i++)
   {
      iwind[0][i] = IVAL(*in,i);
   }
   if (iwind[0][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND1 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND2");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[1][i] = IVAL(*in,i);
   if (iwind[1][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND2 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND3");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[2][i] = IVAL(*in,i);
   if (iwind[2][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND3 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND4");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[3][i] = IVAL(*in,i);
   if (iwind[3][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND4 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND5");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[4][i] = IVAL(*in,i);
   if (iwind[4][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND5 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND6");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[5][i] = IVAL(*in,i);
   if (iwind[5][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND6 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND7");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[6][i] = IVAL(*in,i);
   if (iwind[6][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND7 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND8");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[7][i] = IVAL(*in,i);
   if (iwind[7][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND8 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   in = p_find(vblock, "INWIND9");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[8][i] = IVAL(*in,i);
   if (iwind[8][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND9 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }
 
   in = p_find(vblock, "INWIND10");
   count = in->v_count;
   for( i = 0; i < count; i++)
      iwind[9][i] = IVAL(*in,i);
   if (iwind[9][0] > 0)
      {
      *totblk = count / 4;
      totbl2 += *totblk;
      if ((count % 4) != 0)
         {
         c_errmsg("INWIND10 is not a multiple of four", "fixlin-inwind", 
                  &one);
         c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
         }
      }

   /*   Receive the replacement block definitions.
   ------------------------------------------------------------------*/
   count2 = 0;

   in = p_find(vblock, "SUWIND1");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[0][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND1 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   in = p_find(vblock, "SUWIND2");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[1][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND2 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   in = p_find(vblock, "SUWIND3");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[2][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND3 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }
 
   in = p_find(vblock, "SUWIND4");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[3][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND4 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }
 
   in = p_find(vblock, "SUWIND5");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[4][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND5 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }
 
   in = p_find(vblock, "SUWIND6");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[5][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND6 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }
 
   in = p_find(vblock, "SUWIND7");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[6][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND7 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }
 
   in = p_find(vblock, "SUWIND8");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[7][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND8 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }
 
   in = p_find(vblock, "SUWIND9");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[8][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND9 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   in = p_find(vblock, "SUWIND10");
   count = in->v_count;
   for( i = 0; i < count; i++)
      rwind[9][i] = IVAL(*in,i);
   if (count == 2) count2 += count;
   if ((count % 2) != 0)
      {
      c_errmsg("SUWIND10 is not a multiple of two", "fixlin-suwind", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   /*--------------------------------------------------------------------
      Check to see if the default values for SUWIND was used. If not, 
      the number of blocks defined by parameters INWIND and SUWIND must 
      be equal or an error results.
   --------------------------------------------------------------------*/
   if ((count2 != 0) && (totbl2 != count2 / 2))
      {
      c_errmsg("INWIND and SUWIND are not equal", "fixlin-blocks", &one);
      c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
      }

   /*--------------------------------------------------------------------
      Determine the number of histograms to be built for each bad block 
      of image data.
   --------------------------------------------------------------------*/
   for( i = 0; i < 10; i++)
      nmrhis[i] = 0;

   in = p_find(vblock, "NHIST");
   count = in->v_count;
   for( i = 0; i < count; i++)
      nmrhis[i] = IVAL(*in,i);

   *totblk = totbl2;
   for( i = 0; i < *totblk; i++)
      if (nmrhis[i] == 0)
         nmrhis[i] = 1;

   in = p_find(vblock, "$MINMAX");
   strcpy(minmx, SVAL(*in, 0));

   in = p_find(vblock, "$MSGTIM");
   *msgtim = IVAL(*in, 0);
   }
