/*====================================================================
   NAME:                          HIREAL

   FUNCTION:
      HIREAL creates the histograms called 'ihist' and 'rhist'.
      HIREAL creates the proximity and replacement histograms.


   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions
====================================================================*/
#include <math.h>
#include "las.h"
#include "fixlin.h"
#include "util.h"     /* c_clean proto */
#include "pixman.h"   /* c_px* protos */

extern struct FDESC *fdesc1; /*  I: file descriptor of IN(1)        */
extern struct FDESC *fdesc2; /*  I: file descriptor of IN(2)        */


void hireal(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  *max1,        /*  O:          */
   long  *max2,        /*  O:          */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   float pixdiv,       /*  I: dividing point of sub-histograms       */
   long  nmrhis,       /*  I: number of histograms to build          */
   long  bn,           /*  I: current block                          */
   char  hosout[CMLEN],/*  I: host output file name                  */
   long  dtype,        /*  I: data type images                       */
   float ihist[MAXHIS][2][NMRLEV], /* O: histogram array             */
   float rhist[MAXHIS][2][NMRLEV]  /* O: replacement histogram array */
   )

   {
   float *bufin1=NULL; /*  storage buffer for IN(1)                  */
   float *bufin2=NULL; /*  storage buffer for IN(2)                  */

   float ipix;         /*  pixels histogrammed in proximity hist.    */
   float j;            /*  counter--current sample                   */
   float rpix;         /*  pixels in the replacement histogram       */

   long  count;        /*  # of values c_pxhist read into histogram  */
   long  fatal = -1;   /*  fatal error flag                          */
   long  hisin;        /*  counter--current histogram                */
   long  i;            /*  counter--current line                     */
   long  ji;           /*  counter--current sample of input buffers  */
   long  iafter;       /*  lines processed after the bad data block  */
   long  ibefor;       /*  lines processed before the bad data block */
   long  line;         /*  line to read                              */
   long  nelem;        /*  number elements to set in array           */
   long  one = 1;      /*  constant one                              */
   long  rafter;       /*  lines processed after relacement block    */
   long  rbefor;       /*  lines processed before relacement block   */
   long  status;       /*  error status                              */
   long  t_dtype;      /*  temporary data type to fill array         */
   long  temp;         /*  number of pixels to be histogramed        */
   long  zero = 0;     /*  constant zero                             */

   /*   Determine the image area that is included in the proximity histogram
   -----------------------------------------------------------------------*/
   defwin(&ibefor, &iafter, &rbefor, &rafter, iwind, rwind, wind1, wind2,
          bn);

   ipix = (float)(ibefor + iafter) * pixdiv;
   rpix = (float)(rbefor + rafter + iwind[bn][2]) * pixdiv;

   /*   Zero out the two histograms.
   -----------------------------------------------------------------------*/
   nelem = NMRLEV;
   t_dtype = EREAL;
   for( i = 0; i < nmrhis; i++)
      {
      c_pxfill((unsigned char *)ihist[i][0], &t_dtype, &nelem,
          (unsigned char *)&zero);
      c_pxfill((unsigned char *)rhist[i][0], &t_dtype, &nelem,
          (unsigned char *)&zero);
      c_pxfill((unsigned char *)ihist[i][1], &t_dtype, &nelem,
          (unsigned char *)&zero);
      c_pxfill((unsigned char *)rhist[i][1], &t_dtype, &nelem,
          (unsigned char *)&zero);
      }

   /*  dynamically allocate the arrays for bufin1 and bufin2
   -----------------------------------------------------------------------*/
   if((bufin1 = (float *)malloc(EREALSIZE * wind1[3])) == NULL)
      {
      c_errmsg("Error allocating memory", "fixlin-alloc", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }
   if((bufin2 = (float *)malloc(EREALSIZE * wind2[3])) == NULL)
      {
      free(bufin1);
      c_errmsg("Error allocating memory", "fixlin-alloc", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   /*-----------------------------------------------------------------------
      Build the proximity histogram called 'ihist' by reading the image data
      lines before the bad image block using pxhist.
   -----------------------------------------------------------------------*/
   count = 0;
   for( i = 1; i <= ibefor; i++)
      {
      line = iwind[bn][0] - ibefor + i - 1;
      status = c_eread(&fdesc1, band1, &line, (unsigned char *)bufin1, 
                       &one);
      if (status == E_FAIL)
         c_clean(vblock, "FIXLIN", hosout, &one);

      for( j = 1; j <= (float) iwind[bn][3]; j += pixdiv)
         {
         /*  find the nearest integer
         -----------------------------------------------------------------*/
         ji = (long)(j + 0.5);
         temp = (long)(j + pixdiv + 0.5) - ji;
         hisin = (long)((j-1)/pixdiv + 1);
         c_pxhist(&temp, (long *)&bufin1[iwind[bn][1] + ji - 2], &dtype,
                  (long *)&ihist[hisin-1][0][0], &count);
         }
      }

   /*-----------------------------------------------------------------------
      Add to the proximity histogram called 'ihist' by reading the image
      data lines after the bad image block using pxhist.
   -----------------------------------------------------------------------*/
   for( i = 1; i <= iafter; i++)
      {
      line = iwind[bn][2] + iwind[bn][0] + i - 1;
      status = c_eread(&fdesc1, band1, &line, (unsigned char *)bufin1, 
                       &one);
      if (status == E_FAIL)
         c_clean(vblock, "FIXLIN", hosout, &one);

      for( j = 1; j <= (float) iwind[bn][3]; j += pixdiv)
         {
         /*  find the nearest integer
         -----------------------------------------------------------------*/
         ji = (long)(j + 0.5);
         temp = (long)(j + pixdiv + 0.5) - ji;
         hisin = (long)((j-1)/pixdiv + 1);
         c_pxhist(&temp, (long *)&bufin1[iwind[bn][1] + ji - 2], &dtype,
                  (long *)&ihist[hisin-1][0][0], &count);
         }
      }

   /*-----------------------------------------------------------------------
      Build the replacement histogram called 'rhist' by reading the
      replacement block using pxhist.
   -----------------------------------------------------------------------*/
   count = 0;
   for( i = 1; i <= rbefor+rafter+iwind[bn][2]; i++)
      {
      line = rwind[bn][0] - rbefor + i - 1;
      status = c_eread(&fdesc2, band2, &line, (unsigned char *)bufin2, 
                       &one);
      if (status == E_FAIL)
         c_clean(vblock, "FIXLIN", hosout, &one);

      for( j = 1; j <= (float) iwind[bn][3]; j += pixdiv)
         {
         /*  find the nearest integer
         -----------------------------------------------------------------*/
         ji = (long)(j + 0.5);
         temp = (long)(j + pixdiv + 0.5) - ji;
         hisin = (long)((j-1)/pixdiv + 1);
         c_pxhist(&temp, (long *)&bufin2[rwind[bn][1] + ji - 2], &dtype,
                  (long *)&rhist[hisin-1][0][0], &count);
         }
      }

   /*-----------------------------------------------------------------------
      Standardize the two histograms so the the sum of all elements in each
      histogram will equal one.  This will allow the two histograms to be
      compared with each other even though the number of pixels used to make
      each of the histograms are different.  The two histograms will contain
      the percentages.
   -----------------------------------------------------------------------*/
   for( ji = 0; ji < nmrhis; ji++)
      {
      max1[ji] = 0;
      max2[ji] = 0;
      for( i = 0; i < NMRLEV; i++)
         {
         if (ihist[ji][1][i] > 0.0)
            {
            max1[ji]++;
            ihist[ji][1][i] /= ipix;
            }
         if (rhist[ji][1][i] > 0.0)
            {
            max2[ji]++;
            rhist[ji][1][i] /= rpix;
            }
         }
      }

   /*   Free up dynamically allocated memory
   -----------------------------------------------------------------------*/
   free(bufin1);
   free(bufin2);
   }
