/*====================================================================
   NAME:                          INIT_IMGS
 
   FUNCTION:
      INIT_IMGS opens the input image files.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions
====================================================================*/
#include "las.h"
#include "fixlin.h"
#include "util.h"      /* c_gettyp proto */

extern struct FDESC *fdesc1; /*  O: file descriptor for IN(1)        */
extern struct FDESC *fdesc2; /*  O: file descriptor for IN(2)        */


void init_imgs (
   char  hosin1[CMLEN], /*  I: host input file name 1                */
   char  hosin2[CMLEN], /*  I: host input file name 2                */
   long  *dtype,        /*  O: data type images                      */ 
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   long  wind1[4],     /*  I: IN(1) window specification             */
   long  wind2[4],     /*  I: IN(2) window specification             */
   long  *vmin,        /*  O: minimum pixel value                    */ 
   long  *vmax,        /*  O: maximum pixel value                    */ 
   long  totblk,       /*  I: number of blocks to fix                */
   long  iwind[10][4], /*  I: bad data block (sl,ss,nl,ns)           */
   long  rwind[10][2], /*  I: replacement block definition(sl,ss)    */
   long  nmrhis[10],   /*  I: number of histograms to be built       */
   long  nimg,         /*  I: number of input images                 */
   long  nbands,       /*  I: number of bands per input image        */
   long  *odtype       /*  O: output image data type                 */
   )

   {
   double omax;         /*  max value array for hosin1               */
   double omin;         /*  min value array for hosin1               */
   double rmax;         /*  max value array for hosin2               */
   double rmin;         /*  min value array for hosin2               */

   long  access;        /*  file access type                         */
   long  dtype2;        /*             */
   long  fatal = -1;    /*  error message flag                       */
   long  one = 1;       /*  constant one                             */
   long  opt[2];        /* option array for c_eopenr call            */
   long  status;        /*  error status                             */
   long  zero = 0;      /*  constant zero                            */


   /*  INIT_IMGS initializes
   ------------------------------------------------------------------*/
   *dtype = 0;
   dtype2 = 0;
 
   status = c_gettyp(hosin1, &nimg, dtype);
   if (status != E_SUCC)
      {
      c_errmsg("Error getting the type of hosin1", "fixlin-gettyp", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   status = c_gettyp(hosin2, &nimg, &dtype2);
   if (status != E_SUCC)
      {
      c_errmsg("Error getting the type of hosin2", "fixlin-gettyp", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   /*------------------------------------------------------------------
      CHKWIN is a routine to check for errors within the block 
      definitions for the bad image data and replacement image data.
   ------------------------------------------------------------------*/
   chkwin(totblk, iwind, rwind, wind1, wind2, nmrhis);

   status = c_minmax(hosin1, &nimg, wind1, band1, &nbands,
      &omin, &omax);
   if (status == E_FAIL)
      {
      c_errmsg("Error getting the minmax for hosin1", "fixlin-mnmx", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   status = c_minmax(hosin2, &nimg, wind2, band2, &nbands,
      &rmin, &rmax);
   if (status == E_FAIL)
      {
      c_errmsg("Error getting the minmax for hosin2", "fixlin-mnmx", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   /*------------------------------------------------------------------
      Determine the MIN and MAX pixel values within the images of IN(1) 
      and IN(2) combined.  This will give the range of pixel values 
      within the two images, which will be used to build and transform 
      the histograms.  The reason for computing the MIN and MAX is to 
      scale the pixel values into the histograms.
   ------------------------------------------------------------------*/
   if (omin <= rmin)
      *vmin = (long)omin;
   else
      *vmin = (long)rmin;

   if (omax >= rmax)
      *vmax = (long)omax;
   else
      *vmax = (long)rmax;

   /*   If no errors encountered then the image files may be opened.
   ------------------------------------------------------------------*/
   *odtype = *dtype;
   access = IREAD;

   if ((vmax-vmin) > NMRLEV)
      *dtype = EREAL;
   
   opt[0] = zero;
   opt[1] = 0;  /* use the COMPRES1 environment variable */
   status = c_eopenr(&fdesc1, hosin1, &nbands, &wind1[0], &wind1[1],
      &wind1[2], &wind1[3], &access, dtype, opt, &one); 
   if (status != E_SUCC)
      {
      c_errmsg("Cannot open IN(1) image file","fixlin-open", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   status = c_eopenr(&fdesc2, hosin2, &nbands, &wind2[0], &wind2[1],
      &wind2[2], &wind2[3], &access, dtype, opt, &one); 
   if (status != E_SUCC)
      {
      c_errmsg("Cannot open IN(2) image file","fixlin-open", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }
   }
