/*====================================================================
   NAME:                          PRREAL

   FUNCTION:
      PRREAL corrects the bad blocks for real data
      PRREAL fixes the original image by replacing the bad image block(s)
          with pixel values of the transformed replacement block.
          Procedure to do this is: read the pixel value from the
          replacement block and determine which sub-histogram this pixel
          value was put into and use this value to determine the location
          of the transformed pixel value in the look up table.  Then
          assign this value to the output image.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions

   ALGRITHM(FLOW):
      PRREAL is the processing routine for real data.
      For each block of bad data:
         Read image lines of IN(1) and assign the pixel values to the
            output image until the first line of bad data is reached.
            Copy the pixel values into the output image because this
            is good image data.
         Call "hireal" to obtain the histogram of the adjacent lines
            to the bad image data, and the histogram of the
            replacement lines
         Call "trreal" to equalize the two histograms and create the
            look up table
         Create a new file group to read the replacement lines.  A new
            file group is created because the replacement blocks may
            not be in line ascending order.
      For each line of data within this bad block of image data:
         Read a line of image IN(1)
         For each pixel up to the starting pixel of the bad block
            Copy the pixel values of IN(1) into the output image,
               because these pixel values are still part of good image
               data.
         For each pixel after the bad block to the end of the image
            Copy the pixel values of IN(1) into the output image,
               because these pixel values are still part of good image
               data.
         Read a line of image of the replacement block
         For each pixel within the bad block
            Obtain the pixel value from IN(2) and use this value as
               the location in the look up table of the transformed
               pixel value.
      After the last block of bad data has been fixed, read each
         remaining line of image IN(1) and copy the pixel values into
         the output image.
====================================================================*/
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "fixlin.h"
#include "util.h"   /* c_clean,c_startm,c_stoptm,c_inittm,c_inmnmx protos */
#include "pixman.h" /* c_px* protos */

extern long offset[4];    /*  O: offsets into 'bufout' for each band    */
extern struct FDESC *fdesc1;    /*  I: file descriptor of IN(1)      */
extern struct FDESC *fdesc2;    /*  I: file descriptor of IN(2)      */
extern struct FDESC *fdesc3[2]; /*  I: file descriptor for OUT       */
extern struct GDESC *gdesc;     /*  I: group descriptor for OUT      */
extern long curent; /*  O: current entity being processed         */


void prreal(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  totblk,       /*  I: total number of blocks                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  dtype,        /*  I: data type images                       */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  tothis[10],   /*  I: number of sub-histograms per block     */
   char  hosout[CMLEN],/*  I: host output file name                  */
   char  minmx[4],     /*  I: min max flag                           */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   long  msgtim,       /*  I: elapsed time between display mesaages  */
   long  odtype,       /*  I: output image data type                 */
   long  nbands        /*  I: number of bands in the output image    */
   )

   {
   double maxv;        /*  maximum pixel value for output image      */
   double minv;        /*  minimum pixel value for output image      */

   float *bufin1=NULL; /*  storage buffer for IN(1)                  */
   float *bufin2=NULL; /*  storage buffer for IN(2)                  */
   float *bufout=NULL; /*  storage buffer for OUT                    */
   float cur;          /*  temporary variable                        */
   long temptype;      /*  temporary data type                       */
   float pixdiv;       /*  dividing point of sub-histograms          */
   float lkup[MAXHIS][2][NMRLEV]; /*  look up table                  */
   float val;          /*  histogram value returned from lkup        */
   float ihist[MAXHIS][2][NMRLEV]; /*  histogram array               */
   float rhist[MAXHIS][2][NMRLEV]; /*  replacement histogram array   */

   long  access;       /*  file access type                          */
   long  count1;       /*  counter--number of samples                */
   long  fatal = -1;   /*  fatal error flag                          */
   long  hisin;        /*  counter--current sub-histogram            */
   long  i;            /*  counter--current block                    */
   long  j;            /*  counter--current line                     */
   long  k;            /*  counter--current line                     */
   long  lin;          /*  starting line                             */
   long  max1[MAXHIS]; /*              */
   long  max2[MAXHIS]; /*              */
   long  nmrhis;       /*  number of sub-histograms                  */
   long  one = 1;      /*  constant one                              */
   long  opt[2];       /* option array for c_eopens call             */
   long  rline;        /*  current line to be processed              */
   long  status;       /*  error status                              */
   long  temp;         /*  temporary variable                        */
   long  zero = 0;     /*  constant zero                             */
   long  bsize;        /*  buffer size                               */

   /*  Initialize the minmx arrays
   -----------------------------------------------------------------------*/
   c_inmnmx(&dtype, &one, &minv, &maxv);

   /*  initialize variables
   -----------------------------------------------------------------------*/
   lin = 1;
   curent = 0;
   if ((bufout = (float *) malloc (EREALSIZE * wind1[3] * nbands)) == NULL)
   {
     c_errmsg("Error allocating memory", "fixlin-alloc", &one);
     c_errmsg("Fatal error encountered", "fixlin-fatal", &fatal);
   }

   /*  Open the ouput image.
   -----------------------------------------------------------------------*/
   access = IWRITE;
   opt[0] = odtype;
   opt[1] = 0;  /* use the COMPRES1 environment variable */
   status = c_eopens(&fdesc3[0], hosout, &one, &one, &one, &wind1[2],
                     &wind1[3], offset, &access, &dtype, &zero, opt);
   if (status != E_SUCC)
      {
      c_errmsg("Cannot open OUT image file", "fixlin-open", &one);
      c_clean(vblock, "FIXLIN", hosout, &one);
      }

   fdesc3[1] = 0;
   temp = (long) EREALSIZE * wind1[3] * nbands;
   status = c_egroup(fdesc3, &gdesc, (unsigned char *)bufout, &temp);
   if (status != E_SUCC)
      c_clean(vblock, "FIXLIN", hosout, &one);

   c_errmsg("Bad data block(s) are being fixed", "informational", &one);

   /*  dynamically allocate the arrays for bufin1 and bufin2
   ------------------------------------------------------------------------*/
   if((bufin1 = (float *)malloc(EREALSIZE * wind1[3])) == NULL)
      {
      c_errmsg("Error allocating memory", "fixlin-alloc", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }
   if((bufin2 = (float *)malloc(EREALSIZE * wind2[3])) == NULL)
      {
      free(bufin1);
      c_errmsg("Error allocating memory", "fixlin-alloc", &one);
      c_errmsg("Fatal error encountered","fixlin-fatal",&fatal);
      }

   /*  set temporary variable to output print style
   ------------------------------------------------------------------------*/
   temptype = GEN1;
   c_inittm("FIXLIN", "Processed", "line", "band", "of output image",
            &wind1[2], &one, &msgtim, &temptype);
   c_startm();

   for( i = 0; i < totblk; i++)
      {
      /*  Read in the good data before the bad block.
      ---------------------------------------------------------------------*/
      for( j = lin; j <= iwind[i][0]-1; j++)
         {
         status = c_estep(&gdesc);
         if (status != E_SUCC)
            c_clean(vblock, "FIXLIN", hosout, &one);

         status = c_eread(&fdesc1, &band1[0], &j, (unsigned char *)bufin1, 
                          &one);
         if (status == E_FAIL)
            c_clean(vblock, "FIXLIN", hosout, &one);

         c_pxcopy((unsigned char *)bufin1,
            (unsigned char *)&bufout[offset[0]], &dtype, &wind1[3]);

         /* convert to lower case for only one test condition.
         ------------------------------------------------------------------*/
         bsize = (long) strlen (minmx);
         c_up2low (minmx, &bsize);
         if (strcmp(minmx,"yes") == 0)
            c_pxmnmx((long *)&bufout[offset[0]], &wind1[3], &minv, &maxv,
               &dtype);
         curent++;
         }

      lin = iwind[i][0];
      nmrhis = tothis[i];
      pixdiv = (float)iwind[i][3] / (float)nmrhis;

      /* Calculate the histograms needed.
      ------------------------------------------------------------------*/
      hireal(vblock, iwind, rwind, wind1, wind2, max1, max2, band1, band2,
             pixdiv, nmrhis, i, hosout, dtype, ihist, rhist);

      /* Transform the pixel values and store in the look up table.
      ------------------------------------------------------------------*/
      trreal(max1, max2, nmrhis, ihist, rhist, lkup);
      rline = rwind[i][0] - 1;

      /* Read in the bad data block.
      ------------------------------------------------------------------*/
      for (j = lin; j <= (lin + iwind[i][2] - 1); j++)
         {
         rline++;
         status = c_estep(&gdesc);
         if (status != E_SUCC)
            c_clean(vblock, "FIXLIN", hosout, &one);

         status = c_eread(&fdesc1, &band1[0], &j, (unsigned char *)bufin1, 
                          &one);
         if (status == E_FAIL)
            c_clean(vblock, "FIXLIN", hosout, &one);

         status = c_eread(&fdesc2, &band2[0], &rline, (unsigned char *)bufin2,
            &one);
         if (status == E_FAIL)
            c_clean(vblock, "FIXLIN", hosout, &one);

         /*  Copy the good samples before and after the bad data.
         ---------------------------------------------------------------*/
         c_pxcopy((unsigned char *)bufin1,
            (unsigned char *)&bufout[offset[0]], &dtype, &iwind[i][1]);

         temptype = wind1[3] - (iwind[i][1] + iwind[i][3] - 1);
         c_pxcopy((unsigned char *)&bufin1[iwind[i][1] + iwind[i][3] - 1],
            (unsigned char *)&bufout[offset[0]+iwind[i][1]+iwind[i][3]-1],
            &dtype, &temptype);

         /*  Fix the original image by replacing the bad image block
         ---------------------------------------------------------------*/
         for(k = iwind[i][1]-1, count1 = 0; k < (iwind[i][1]+iwind[i][3]-1);
             k++)
            {
            cur = bufin2[rwind[i][1] + count1 - 1];
            hisin = (long)((float) count1 / pixdiv + 1);
            bsrch(max2[hisin-1], cur, hisin-1, &val, lkup);
            bufout[offset[0] + k] = val;
            count1++;
            }

         /* convert to lower case for only one test condition.
         ------------------------------------------------------------------*/
         bsize = (long) strlen (minmx);
         c_up2low (minmx, &bsize);
         if (strcmp(minmx,"yes") == 0)
            c_pxmnmx((long *)&bufout[offset[0]], &wind1[3], &minv, &maxv,
               &dtype);
         curent++;
         }
      lin += iwind[i][2];
      }

   /*  Read in the data after the bad block.
   ---------------------------------------------------------------------*/
   for(j = lin; j <= wind1[2]; j++)
      {
      status = c_eread(&fdesc1, &band1[0], &j, (unsigned char *)bufin1, 
                       &one);
      if (status == E_FAIL)
         c_clean(vblock, "FIXLIN", hosout, &one);

      status = c_estep(&gdesc);
      if (status != E_SUCC)
         c_clean(vblock, "FIXLIN", hosout, &one);

      c_pxcopy((unsigned char *)bufin1, (unsigned char *)&bufout[offset[0]],
         &dtype, &wind1[3]);

      /* convert to lower case for only one test condition.
      ------------------------------------------------------------------*/
      bsize = (long) strlen (minmx);
      c_up2low (minmx, &bsize);
      if (strcmp(minmx,"yes") == 0)
         c_pxmnmx((long *)&bufout[offset[0]], &wind1[3], &minv, &maxv, &dtype);
      curent++;
      }

   status = c_estep(&gdesc);
   if (status != E_SUCC)
      c_clean(vblock, "FIXLIN", hosout, &one);

   c_stoptm();

   /*  Save the minimum and maximum values to the min max file.
   ---------------------------------------------------------------------*/
   status = c_svmnmx(hosout, &one, &minv, &maxv, minmx);
   if (status != E_SUCC)
      c_errmsg("Error creating minmax file", "fixlin-minmax", &one);

   /*   Free up dynamically allocated memory
   -----------------------------------------------------------------------*/
   free(bufin1);
   free(bufin2);
   }
