/*====================================================================
   NAME:                          TRANS

   FUNCTION:
      TRANS transforms the histograms into a look up table

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions

   ALGRITHM:
      The routine loops for the number of histograms.
         Using values from the replacement histogram and from the
         proximity histogram the pixel values of the replacement block
         are transformed into pixel values corresponding to the bad image
         data, and are stored into the look up table.
====================================================================*/
#include <stdlib.h>
#include "math.h"
#include "las.h"
#include "fixlin.h"


void trans(
   long  vmin,         /*  I: minimum value of IN(1) and IN(2)       */
   long  vmax,         /*  I: maximum value of IN(1) and IN(2)       */
   long  nmrhis,       /*  I: number of histograms to build          */
   float ihst[MAXHIS][NMRLEV], /* I: percentages of histogram values */
   float rhst[MAXHIS][NMRLEV], /* I: percentages of histogram values */
   long  lkup[MAXHIS][NMRLEV]  /* O: look up table                   */
   )

   {
   float div;          /*  sum of histogram values                   */
   float sum1;         /*  sum of proximity histogram values         */
   float sum2;         /*  sum of replacement histogram values       */
   float tot;          /*  sum of no. of values times value          */
   float val;          /*  value placed in the lookup table          */

   long  hisin;        /*  counter--current histogram                */
   long  i;            /*  counter--current line                     */
   long  index1;       /*  counter-current place in proximity histo  */
   long  index2;       /*  counter-current place in replace. histo.  */
   long  k;            /*  counter--current value                    */
   long  lb;           /*  place in look up table to place val       */
   long  max1;         /*  maximum value of ihst                     */
   long  max2;         /*  maximum value of rhst                     */
   long  min1;         /*  minimum value of ihst                     */
   long  min2;         /*  minimum value of rhst                     */
   long  next1;        /*  next place in proximity histogram         */
   long  next2;        /*  next place in replacement histogram       */

   /*-----------------------------------------------------------------------
     for each sub-histogram -- equalize the two corresponding sub-
     histograms of the proximity and replacement blocks.
   -----------------------------------------------------------------------*/
   for( hisin = 0; hisin < nmrhis; hisin++)
      {
      /*--------------------------------------------------------------------
         Initialization of variables
            INDEX1 is index into the proximity histogram
            INDEX2 is index into the replacement histogram
            SUM1 and SUM2 are the sum of the elements within the histograms
      --------------------------------------------------------------------*/
      min1 = vmin;
      max1 = vmax;
      min2 = vmin;
      max2 = vmax;
      index1 = min1;
      index2 = min2;
      sum1 = 0.0;
      sum2 = 0.0;

      /*--------------------------------------------------------------------
         Using values from the replacement histogram (RHST), and from the 
         proximity histogram (IHST) the pixel values of the replacement 
         block are transformed into pixel values corresponding to the bad 
         image data, and are stored into the look up table.  The result will 
         be that the histogram of the replacement block when applied to the 
         look up table will be approximately the same as the histogram of 
         the proximity histogram.
      --------------------------------------------------------------------*/
      do
         {
         /*  check for no occurences of this value
         -----------------------------------------------------------------*/
         while((ihst[hisin][index1] == 0.0) && (index1 <= max1))
            index1++;

         /*  check for no occurences of this value
         -----------------------------------------------------------------*/
         while((rhst[hisin][index2] == 0.0) && (index2 <= max2))
            index2++;

         if ((index1 <= max1) && (index2 <= max2))
            {
            lb = index2;
            next1 = index1 + 1;
            next2 = index2 + 1;
            sum1 += ihst[hisin][index1];
            sum2 += rhst[hisin][index2];
            tot = (float)(index1+1) * ihst[hisin][index1];
            div = ihst[hisin][index1];

            if (sum1 < sum2)
               {
               while(1)
                  {
                  /*  check for no occurences of this value
                  --------------------------------------------------------*/
                  if ((ihst[hisin][next1] == 0.0) && (next1 < max1))
                     {   
                     next1++;
                     continue;
                     }

                  if ((fabs(sum1 + ihst[hisin][next1] - sum2) <
                       fabs(sum1 - sum2)) && (next1 <= max1))
                     {
                     index1 = next1;
                     sum1 += ihst[hisin][index1];
                     tot += (float)(index1+1) * ihst[hisin][index1];
                     div += ihst[hisin][index1];
                     next1++;
                     }
                  else
                     break;
                  }
               }
            else
               {
               while(1)
                  {
                  /*  check for no occurences of this value
                  --------------------------------------------------------*/
                  if ((rhst[hisin][next2] == 0.0) && (next2 < max2))
                     {
                     next2++;
                     continue;
                     }

                  if ((fabs(sum1 - (sum2 + rhst[hisin][next2])) <
                       fabs(sum1 - sum2)) && (next2 <= max2))
                     {
                     index2 = next2;
                     sum2 += rhst[hisin][index2];
                     next2++;
                     }
                  else
                     break;
                  }
               }

            if ((next1 > max1) || (next2 > max2))
               {
               index2 = max2;
               for(i = next1; i <= max1; i++)
                  if (ihst[hisin][i] != 0.0)
                     {
                     tot += (float)(i+1) * ihst[hisin][i];
                     div += ihst[hisin][i];
                     }
               }

            /*--------------------------------------------------------------
               .00002 is needed to compensate for the lack of precision of 
               digits on the PN9050.
            --------------------------------------------------------------*/
            val = (tot / div) - .5 + .00002;

            for(k = lb; k <= index2; k++)
               lkup[hisin][k] = (long)val;

            index1 = next1;
            index2 = next2;
            }
         else
            break;
         } while((index1 <= max1) || (index2 <= max2));
      }
   }
