/*====================================================================
   NAME:                          TRREAL

   FUNCTION:
      TRREAL transforms the histograms into a look up table

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions

   ALGRITHM:
      The routine loops for the number of histograms.
         Using values from the replacement histogram and from the
         proximity histogram the pixel values of the replacement block
         are transformed into pixel values corresponding to the bad image
         data, and are stored into the look up table.
====================================================================*/
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "fixlin.h"


void trreal(
   long  *max1,        /*  I:  maximum value in ihist                */
   long  *max2,        /*  I:  maximum value in rhist                */
   long  nmrhis,       /*  I:  number of histograms                  */
   float ihist[MAXHIS][2][NMRLEV], /*  I:  histogram array           */
   float rhist[MAXHIS][2][NMRLEV], /* I:  replacement hist. array    */
   float lkup[MAXHIS][2][NMRLEV] /*  O: look up table                */
   )

   {
   float div;          /*  sum of histogram values                   */
   float sum1;         /*  sum of proximity histogram values         */
   float sum2;         /*  sum of replacement histogram values       */
   float tot;          /*  sum of no. of values times value          */
   float val;          /*  value placed in the lookup table          */

   long  hisin;        /*  counter--current histogram                */
   long  i;            /*  counter--current line                     */
   long  index1;       /*  counter-current place in proximity histo  */
   long  index2;       /*  counter-current place in replace. histo.  */
   long  k;            /*  counter--current value                    */
   long  lb;           /*  place in look up table to place val       */
   long  lkin;         /*  ending place in lookup table to place val */


   /*-----------------------------------------------------------------------
     for each sub-histogram -- equalize the two corresponding sub-
     histograms of the proximity and replacement blocks.
   -----------------------------------------------------------------------*/
   for( hisin = 0; hisin < nmrhis; hisin++)
      {
      /*--------------------------------------------------------------------
         Initialization of variables
            INDEX1 is index into the proximity histogram
            INDEX2 is index into the replacement histogram
            SUM1 and SUM2 are the sum of the elements within the histograms
      --------------------------------------------------------------------*/
      index1 = 1;
      index2 = 1;
      sum1 = 0.0;
      sum2 = 0.0;
      lkin = 0;

      /*--------------------------------------------------------------------
         Using values from the replacement histogram (RHIST), and from the 
         proximity histogram (IHIST) the pixel values of the replacement 
         block are transformed into pixel values corresponding to the bad 
         image data, and are stored into the look up table.  The result will 
         be that the histogram of the replacement block when applied to the 
         look up table will be approximately the same as the histogram of 
         the proximity histogram.
      --------------------------------------------------------------------*/
      while((index1 <= max1[hisin]) && (index2 <= max2[hisin]))
         {
         lb = lkin;
         lkup[hisin][0][lkin]  = rhist[hisin][0][index2-1];
         sum1 += ihist[hisin][1][index1-1];
         sum2 += (float)rhist[hisin][1][index2-1];
         tot = ihist[hisin][0][index1-1] * ihist[hisin][1][index1-1];
         div = ihist[hisin][1][index1-1];

         if (sum1 < sum2)
            {
            while(fabs(sum1 + ihist[hisin][1][index1] - sum2) <
                fabs(sum1 - sum2))
               {
               index1++;
               sum1 += ihist[hisin][1][index1-1];
               tot += ihist[hisin][0][index1-1] * ihist[hisin][1][index1-1];
               div += ihist[hisin][1][index1-1];
               }
            }
         else
            {
            while(fabs(sum1 - (sum2 + rhist[hisin][1][index2])) <
                fabs(sum1 - sum2))
               {
               index2++;
               sum2 += rhist[hisin][1][index2-1];
               lkin++;
               lkup[hisin][0][lkin] = rhist[hisin][0][index2-1];
               }
            }

         if ((index1 > max1[hisin]) || (index2 > max2[hisin]))
            {
            for (i = index1 + 1; i <= max1[hisin]; i++)
               {
               tot += ihist[hisin][0][i] * ihist[hisin][1][i];
               div += ihist[hisin][1][i];
               }
            for (i = index2 + 1; i <= max2[hisin]; i++)
               {
               lkin++;
               lkup[hisin][0][lkin]  = rhist[hisin][0][i];
               }
            }

         /*--------------------------------------------------------------
            .00002 is needed to compensate for the lack of precision of
            digits on the PN9050.
         --------------------------------------------------------------*/
         val = (tot / div) + .00002;

         for(k = lb; k <= lkin; k++)
            lkup[hisin][1][k] = val;

         lkin++;
         index1++;
         index2++;
         }  /* end while */
      }  /* end for */
   }
