/*******************************************************************************
NAME:		SETUP	

PURPOSE:    	SETUP  	Determines which, if any, regions are to be flagged
			with negative class values.
			
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	10/90	Re-retrocode (All new C version!)
5.1     D.Etrheim       11/91   Changed to call bsearch only if a threshold
                                table is being used.
5.2	S.Nelson	 5/92	Updated error messages.

PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/

#include <stdlib.h>
#include "las.h"
#include "ltable.h"
#include "typlim.h"
#include "protos.h"

struct threshold        /* threshold values structure */
   {
       long class;          /* class value */
       long thresh;         /* area threshold */
   };

static int s_compar
(       /* compare routine used by qsort */
    struct threshold *a,
    struct threshold *b     /* threshold entry pointers */
);

static int compar
(       /* compare routine used by bsearch */
    struct threshold *a,
    struct threshold *b     /* threshold entry pointers */
);



void setup
(
    char *regionlt,		/* region description labeled table */
    char *threshlt,		/* threshold values labeled table */
    long mskcls,			/* mask class value */
    long d_thresh,		/* default threshold value */
    long *regcls[],		/* class identifier for each region */
    long *regcnt[],		/* count of pixels in each region */
    long *mincls,		/* minimum class value */
    long *maxcls,		/* maximum class value */
    long *numregs		/* number of regions in image */
)
{

long	i;			/* loop counter */
long	k;			/* loop counter */
long	access = IREAD;		/* read access */
long	status;			/* status flag value */
long	fatal = -1;		/* fatal error code */
long	nonfatal = 1;		/* nonfatal error code */
long	mxnum;			/* maximum number of entries in arrays */
long	numthresh;		/* number of threshold values in thresh_vals */
long	region;			/* region number read from lt */
long	flgcnt;			/* count of regions to be flagged */

struct 	TAB_DEF	reglt;		/* region labeled table */	
struct 	TAB_DEF	thrlt;		/* threshold labeled table */	
struct	VECTOR	*v_regnum;	/* region number vector */		
struct	VECTOR	*v_regclass = NULL;	/* region class vector */		
struct	VECTOR	*v_regcount = NULL;	/* region count vector */		
struct  VECTOR  *v_class;	/* class value vector */
struct  VECTOR	*v_thresh = NULL;	/* threshold value vector */

struct 	threshold *thresh_vals;	/* threshold values array */ 
struct 	threshold *thresh_ptr = NULL;	/* threshold pointer */

char	msg[ERRLEN+1];		/* error message buffer */
/*****************************************************************************/

/* Read in region descriptions from labeled table
-------------------------------------------------*/
open_tab(&reglt,regionlt,access);
get_vector(&reglt);
if (((v_regnum = get_field_ptr(&reglt,"REGNUM")) == NULL) ||
   ((v_regclass = get_field_ptr(&reglt,"CLASS")) == NULL) ||
   ((v_regcount = get_field_ptr(&reglt,"COUNT")) == NULL))
      c_errmsg("Error getting field pointer from region labeled table",
		"flag-getfield",&fatal); 

/* Allocate initial arrays for reading data from lt
---------------------------------------------------*/
mxnum = 1000;
if (((*regcls = (long *) malloc((mxnum+1)*4)) == NULL) || 
   ((*regcnt = (long *) malloc((mxnum+1)*4)) == NULL)) 
   c_errmsg("Error allocating dynamic memory","flag-alloc",&fatal);
for (i = 0; i <= mxnum; i++) 
   (*regcls)[i] = (*regcnt)[i] = 0;

/* Read all records in labeled table 
------------------------------------*/
for (i = 0;;)
   {
   status = get_record(&reglt);
   if (status == E_FAIL)
      c_errmsg("Fatal error encountered","flag-fatal",&fatal);
   else if (status == E_EOF)
      break;
   else
      {
      i++;
      if (i > mxnum)
         {
	 /* Allocate more memory when needed
         -----------------------------------*/
         mxnum += 1000;
         if (((*regcls = (long *) realloc(*regcls,(mxnum+1)*4)) == NULL) || 
             ((*regcnt = (long *) realloc(*regcnt,(mxnum+1)*4)) == NULL))
            c_errmsg("Error allocating dynamic memory","flag-alloc",&fatal);
         for (k = i; k <= mxnum; k++) (*regcls)[k] = (*regcnt)[k] = 0;
         }
      /* Extract field values for region, class and count
      ---------------------------------------------------*/
      get_matrix(v_regnum,I4,(unsigned char *)&region);
      if (region != i)
         c_errmsg("Region labeled table out of sequence or missing data",
			"flag-region",&fatal);
      get_matrix(v_regclass,I4,(unsigned char *)&(*regcls)[i]);
      get_matrix(v_regcount,I4,(unsigned char *)&(*regcnt)[i]);
      }
   }
*numregs = i;
sprintf(msg,"Number of regions read from labeled table: %d",i);
c_errmsg(msg," ",&nonfatal);

/* Close labeled table 
----------------------*/
close_tab(&reglt);

/* Check whether threshold table was entered by user
----------------------------------------------------*/
if (*threshlt == NULL)  
   /* No table entered 
   -------------------*/
   {
   numthresh = 0;
   thresh_vals = NULL;
   thresh_ptr = NULL;
   }
else
   {
   /* Read in threshold values from labeled table
   ----------------------------------------------*/
   open_tab(&thrlt,threshlt,access);
   get_vector(&thrlt);
   if (((v_class = get_field_ptr(&thrlt,"CLASS")) == NULL) ||
      ((v_thresh = get_field_ptr(&thrlt,"THRESH")) == NULL)) 
         c_errmsg("Error getting field pointer from threshold labeled table",
		"flag-getfield",&fatal); 

   /* Allocate initial arrays for reading data from lt
   ---------------------------------------------------*/
   mxnum = 1000;
   if ((thresh_vals = (struct threshold *) malloc((mxnum)*8)) == NULL)  
      c_errmsg("Error allocating dynamic memory","flag-alloc",&fatal);
   for (i = 0; i < mxnum; i++) 
      thresh_vals[i].class = thresh_vals[i].thresh = 0;
   
   /* Read all records in labeled table 
   ------------------------------------*/
   for (i = 0;;)
      {
      status = get_record(&thrlt);
      if (status == E_FAIL)
         c_errmsg("Fatal error encountered","flag-fatal",&fatal);
      else if (status == E_EOF)
         break;
      else
         {
         i++;
         if (i >= mxnum)
            {
	    /* Allocate more memory when needed
            -----------------------------------*/
            mxnum += 1000;
            if ((thresh_vals = 
		(struct threshold *) realloc(thresh_vals,(mxnum)*8)) == NULL) 
               c_errmsg("Error allocating dynamic memory",
					"flag-alloc",&fatal);
            for (k = i; k < mxnum; k++) 
               thresh_vals[k].class = thresh_vals[k].thresh = 0;
            }
 	 /* Extract field values for class and threshold
         -----------------------------------------------*/
         get_matrix(v_class,I4,(unsigned char *)&(thresh_vals[i-1].class));
         get_matrix(v_thresh,I4,(unsigned char *)&(thresh_vals[i-1].thresh));
         }
      }
 
   numthresh = i;
   sprintf(msg,"Number of classes & thresholds read from labeled table: %d",i);
   c_errmsg(msg," ",&nonfatal);
   
   /* Close labeled table 
   ----------------------*/
   close_tab(&thrlt);

   /* Sort the class numbers and thresholds
   ----------------------------------------*/
   qsort ((char *)thresh_vals, numthresh, 8, s_compar);
   }

/* Initialize min and max values
--------------------------------*/
*mincls = MAXLONG;
*maxcls = MINLONG;
flgcnt = 0;

/* Compare region sizes to threshold values to decide when to flag
------------------------------------------------------------------*/
for (i = 1; i <= *numregs; i++)
   {
   /* Check for mask class
   -----------------------*/
   if ((*regcls)[i] == mskcls) 
      (*regcls)[i] = 0;
   else
      {
      /* Look for class value in threshold table 
      ------------------------------------------*/
      if (*threshlt != NULL)
         thresh_ptr = (struct threshold *) bsearch((char *)&((*regcls)[i]),
			(char *)thresh_vals,numthresh,8,compar); 
      if (thresh_ptr == NULL)
         {
	 /* Class not found in table--use default threshold
         --------------------------------------------------*/
         if ((*regcnt)[i] < d_thresh)
            {
	    (*regcls)[i] *= -1;
	    flgcnt++;
 	    }
 	 }  
      else
         /* Check area of region against threshold value
         -----------------------------------------------*/
         if ((*regcnt)[i] < thresh_ptr->thresh)
            {
	    (*regcls)[i] *= -1;
	    flgcnt++;
 	    }
      }
   /* Track min and max values 
   ---------------------------*/
   if ((*regcls)[i] < *mincls) *mincls = (*regcls)[i];
   if ((*regcls)[i] > *maxcls) *maxcls = (*regcls)[i];
   }

/* Free memory
--------------*/
if (*threshlt != NULL) 
   free (thresh_vals);

/* Put out informational message
--------------------------------*/
sprintf (msg,"Number of regions to be flagged: %d",flgcnt);
c_errmsg (msg," ",&nonfatal);

}

/*****************************************************************************/

static int s_compar
(		/* compare routine used by qsort */
    struct threshold *a,
    struct threshold *b		/* threshold entry pointers */
)
{
char	msg[ERRLEN+1];			/* buffer for error message */
long	nonfatal = 1;			/* nonfatal flag */

if (a->class == b->class) 
   {
   sprintf(msg,"Class %d occurs more than once in threshold table.",
	a->class);
   c_errmsg(msg,"flag-thresh",&nonfatal);
   c_errmsg("Threshold value used is unpredictable."," ",&nonfatal);
   }
 
/* Return (0) when two entries are equal; (-) when second entry is greater 
   than first; and (+) when first entry is greater than second  
-----------------------------------------------------------------------*/
return ((int)(a->class - b->class));
}

/*****************************************************************************/

static int compar
(		/* compare routine used by bsearch */
    struct threshold *a,
    struct threshold *b		/* threshold entry pointers */
)
{
 
/* Return (0) when two entries are equal; (-) when second entry is greater 
   than first; and (+) when first entry is greater than second  
-----------------------------------------------------------------------*/
return ((int)(a->class - b->class));
}
