/******************************************************************************
NAME:	FLIP

PURPOSE:
	FLIP is a module which copies a LAS image into another LAS image
	that is rotated (90, 180, or 270 clockwise degrees), flipped top to 
	bottom, or mirrorred left to right.  Only one input size is allowed,
	that is, multiple images combined as input must all have the same 
        window.  The input images may sub-windowed and/or sub-banded, and
        may be of data type BYTE, INTEGER*2, INTEGER*4, or REAL*4. The
        output image is always the same data type as input images.

PROGRAM HISTORY:
Version   Date    Author     Code/Contr.   Change Request
-------   ----   ---------   -----------   --------------
1         5/88   B. Davis      	CSB	   New Development
5.0       3/89   B. Davis      	CSB	   LAS 5.0 Conversion
5.1       8/89   B. Davis      	CSB	   Changed proces.c to correctly handle
					   multiple input images as the input
					   bands, used a substitute variable for
					   curcat.
5.2       9/89   B. Davis      	CSB	   Fixed bug in buffered 270 alogrithm
				  	   of proces.c.
5.3      11/89   P. Pease       GSFC       Changed call to c_stoptm from stoptm
5.4       8/90   D. Etrheim	CSB        ADP #5772 Corrected calling sequence 
                                           to putbdr 
5.5      03/91   W. Xia         STX        modified the codes to handel multi-
                                           images & multi-bands input
5.6      09/92   T. Mittan	CSB	   Added extern to curent and curcat
					   to eliminate warnings on link
6.0      05/94   D. Bridges     CSB        Added option array for opening image
					   
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	MUST BE RUN UNDER TAE WITH LAS 5.0.

PROJECT:   LAS 5.0

LANGUAGE:   C

ALGORITHM:
	Get the user specified parameters:
		IN,OUT,SUBCMD,$MSGTIM,$MINMAX.
	Define and get memory reserved for the output image.
	According to the subcommand specification:
		perform flip operation and write the result to the output image.
	Update the ddr and history records.
	Close files, free up memory.
	Complt the application process.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "las.h"
#include "util.h"
#include "protos.h"

#define IN 0			/* index for INput window specs		      */
#define BUFSZ 1024		/* buffer size for stepping through 90 	      */
				/* and 270 images			      */
#define INVALID 0		/* for setting valid flags of ddr	      */
/*
The following variables must be declared global for image I/0 routines
*/
struct FDESC * fdescin[MAXIMG+1];  
                               /* image file descriptors used for image I/O   */
struct FDESC * fdescot;            
                               /* image file descriptors used for image I/O   */
extern long curent;            /* current entity for processing message       */
extern long curcat;            /* current category for processing message     */
       
int main
(void)
{
struct	PARBLK	vblock;		/* TAE parameter block 			      */

struct  DDR 	ddr;		/* ddr structure for updating cornerpoints    */

struct  BDDR 	*bdr;		/*band ddr structure for updating cornerpoints*/

double maxval[MAXBND];	/* max pixel value array                      */
double minval[MAXBND];	/* min pixel value array                      */
double	line_inc = 1;		/* ddr field				      */
double	samp_inc = 1;		/* ddr field				      */

static long bands[MAXIMG][MAXBND+1];    /* array of bands for input image  */
            
long   	bndcnt;			/* loop index band counter		      */
long    bufflg;			/* flag to indicate io buffering is needed    */
long	ddrflg = NOCOMB;	/* flag for ddr field for combined input bands*/
long	dtype;			/* data type of images                        */
long	fatal = -1;		/* fatal error message flag	 	      */
long    imgln;			/* storage for number of lines in output img  */
long    imgsmp;			/* storage for number of samples, output img  */
long	msg_flag=ONECAT;	/* flag for processing message                */
long 	msgtim;			/* elapsed time between display messages      */
long    nbands[MAXIMG+1];	/* number of bands per input image            */
long 	nl;			/* number of image lines                      */
long	nl2sav;			/* number of input lines to read & save / buf */
long	nmrimg;			/* number of input images-1,need for getin    */
long	non = 1;		/* non-fatal error message flag	 	      */
long 	ns;			/* number of image samples                    */
long	ns2sav;			/* number of input samples to read & save /ln */
long	numout = 1;		/* number of output files		      */
long    one = 1;		/* constant				      */
long	rot;			/* rotation for updating ddr cornerpoints     */
long	sabfct;			/* counter for stepping through 1K samples    */
long    size;       		/* number of bytes per pixel                  */
long	status;			/* return status for cxdputf                  */
long    std_out;		/* fortran LUN to standard output             */
long	totbnd;			/* total number of bands		      */
long    window[MAXIMG][4];	/* window specifications for input image      */
long    itemp;
char    cat[6];			/* category of display message.    	      */
char    hosin[MAXIMG][CMLEN]; 	/* input image name(s)		      	      */
char    hosout[1][CMLEN];		/* output image name(s)		      	      */
char    mnmx[4];		/* min max flag        		     	      */
char    obj[5];			/* object of display message   		      */
char 	pname[5];		/* program name for complt		      */
char	subcmd[7];		/* subcommand from T.A.E.                     */
char    taein[MAXIMG][CMLEN];   /* buffer storage for "taein" 		      */
char    taeout[1][CMLEN];		/* buffer storage for "taeout"		      */
char    text[2];		/* text of display message        	      */
char    verb[10];		/* verb of display message     		      */

static unsigned char inbuf[BUFSZ * BUFSZ];/* input image line I/O buffer      */
static unsigned char outbuf[BUFSZ * BUFSZ];/* output image line I/O buffer    */
static unsigned char *in92buf;   	/* input image I/O buffer             */
static unsigned char *out92buf;  	/* output image I/O buffer            */
long  	out_img_bands[1][MAXBND+1];
long    n;

/******************************************************************************/

c_inlas(&vblock,&std_out);	/* initialize TAE                             */

/* Get the user specified parameters */
status = getpar (&vblock,subcmd,&rot,taein,hosin,taeout,hosout,&nmrimg,
		 window,bands,nbands,&totbnd,&msgtim,mnmx);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","flip-fatal",&fatal);

/* Initialize the input and output images, read write buffers, and arrays */
status = initio(subcmd,hosin,hosout[0],&nmrimg,window,&totbnd,
		&imgln,&imgsmp,&nl,&ns,&nl2sav,&ns2sav,&size,
		&dtype,&bufflg,&in92buf,&out92buf);
                
if (status != E_SUCC)
   clenup(&vblock,&nmrimg,hosout,subcmd,&in92buf,&out92buf,TRUE);

strcpy(pname,"FLIP");				/* start processing message */
strcpy(verb,"Processed");
strcpy(obj,"line");
strcpy(cat,"bands");
strcpy(text," ");
curent = 1;
curcat = 1;
if ((subcmd[0] == '9') || (subcmd[0] == '2'))
   c_inittm(pname,verb,obj,cat,text,&ns,&totbnd,&msgtim,&msg_flag);
else
   c_inittm(pname,verb,obj,cat,text,&imgln,&totbnd,&msgtim,&msg_flag);
c_startm();

/* Process the image by changing the location of pixels according to subcmd */
status = proces(subcmd,&nmrimg,nbands,&nl,&ns,
		&nl2sav,&ns2sav,&sabfct,&size,&dtype,inbuf,outbuf,
	 	in92buf,out92buf,hosout[0],&totbnd,
	 	bands);
/*
status = proces(subcmd,&nmrimg,nbands,&nl,&ns,
		&nl2sav,&ns2sav,&sabfct,&size,&dtype,inbuf,outbuf,
	 	in92buf,out92buf,maxval,minval,hosout[0],&totbnd,
	 	bands);
*/
if (status != E_SUCC)
   clenup(&vblock,&nmrimg,hosout,subcmd,&in92buf,&out92buf,TRUE);

c_stoptm();

/* update the ddr file */
status = c_upddr(&vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&totbnd,&ddrflg);
if (status != E_SUCC)
   clenup(&vblock,&nmrimg,hosout,subcmd,&in92buf,&out92buf,TRUE);

bdr = (struct BDDR *)calloc(totbnd,sizeof(struct BDDR));
status = c_getddr(hosout[0],&ddr);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered -- Image data saved","flip-warn",&non);

for (bndcnt = 1; bndcnt < (totbnd + 1); bndcnt++)
   {
   status = c_getbdr(hosout[0],&(bdr[bndcnt-1]),&bndcnt);
   if (status != E_SUCC)
      c_errmsg("Nonfatal error encountered -- Image data saved",
	       "flip-warn",&non);
   }

/* update the cornerpoints of the ddr */
status = c_rtiddr(&ddr,&rot);
if (status != E_SUCC)
   {
   ddr.valid[6] = INVALID;
   c_errmsg("DDR contains invalid corner point information","flip-corner",&non);
   c_errmsg("Nonfatal error encountered -- Image data saved","flip-warn",&non);
   }

status = c_putddr(hosout[0],&ddr);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered -- Image data saved","flip-warn",&non);

for (bndcnt = 1; bndcnt < (totbnd + 1); bndcnt++)
   {
   status = c_putbdr(hosout[0],&(bdr[bndcnt - 1]));
   if (status != E_SUCC)
      c_errmsg("Nonfatal error encountered -- Image data saved",
	       "flip-warn",&non);
   }

free(bdr);

/* update the last used date of the ddr */
status = c_lused(hosout[0]);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered -- Image data saved","flip-warn",&non);

/* close files, deallocate buffer space      */
clenup(&vblock,&nmrimg,hosout,subcmd,&in92buf,&out92buf,FALSE);

/* update the min max of the ddr */

for (n = 0; n < totbnd; n++)  
  { 
  out_img_bands[0][n] = n + 1;
  }
out_img_bands[0][n]=0;
 
if(mnmx[0]=='Y')
{     
    window[0][0]=1;
    window[0][1]=1; 
    if ((subcmd[0] == '9') || (subcmd[0] == '2'))
        {        
        itemp= window[0][3];
        window[0][3]=window[0][2]; 
        window[0][2]=itemp; 
        }               
    c_minmax (hosout,&one,window,out_img_bands,&totbnd,minval,maxval);
    c_svmnmx (hosout[0],&totbnd,minval,maxval, "YES");
    if (status != E_SUCC)
      {
      c_errmsg("Nonfatal error encountered -- Image data saved",
               "flip-warn",&non);
      }
} 

/* update history file        */
status =  c_hstout(&vblock,hosout[0],hosin,&nmrimg," ");
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered -- Image data saved","flip-warn",&non);

/* Write a completion message to the user    */
c_complt(pname,taeout,&numout);

return EXIT_SUCCESS;

}
/*************************** END OF PROGRAM FLIP ******************************/
