/****************************************************************************

NAME:   CONVERT_WINDOW

PURPOSE: This routine will read the input header file, create a dummy DDR,
         and convert the window values to line/sample from their input units
         value.


PROGRAM HISTORY:
  Version       Date       Author        Request
  -------       ----       ------        -------
    7.2         06/99    G. Schmidt      Original development
                10/01    A.Weissenberger CR#5057. Formatting and redundant
                                         c_lasmsg calls removed.
                                         
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT:    LAS

NOTES: Since the Fast-L7A and SIC Format headers are 3-1536 byte subheaders,
the header buffers need to be able to read 3*HEADER_SIZE information.

We will create a dummy DDR with only one band.

If this routine is called and units are LS then we will check the window,
but a window conversion is not necessary.  Currently this routine is not
called for units of LS, since we really don't need to create a DDR to
check that the window is valid.

****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "fhdr.h"
#include "worgen.h"
#include "diskio.h"
#include "util.h"
#include "fstfmtin.h"

void main () 
{
    /* file names
       -------------*/
    char inhdr[CMLEN];            /* input fast format header name          */
    char hdr_extension[CMLEN];    /* header extension string (fhdr or fst)  */

    /* image information
       --------------------*/
    struct  PARBLK  vblk;         /* TAE parameter block                    */
    char fheader[3*HEADER_SIZE+1];/* buffer for header file                 */
    char htype[5];                /* fast format header type
                                     (A,B,C,L7A,SIC) */
    char tmpstr[HEADER_SIZE+1];   /* temporary string                       */
    long resolution = 0;          /* L7 resolution type (VISIBLE, THERMAL,
                                     PAN) not used for Fast-ABC or SIC      */
    long same_res;                /* 1=bands are of same resolution, 0=not
                                     same not used for Fast-ABC or SIC      */

    /* data retrieved from fast format header
       -----------------------------------------*/
    char source[32];             /* source of image                         */
    char instrument[32];         /* instrument                              */
    char ac_date[10];            /* acquistion date                         */
    char ac_time[8];             /* acquistion time                         */
    long datum = 0;              /* GCTP datum code                         */
    long origsize[2];            /* original size of the image on tape      */
    long proj_code;              /* GCTP projection code                    */
    long zone_code;              /* UTM or State Plane code                 */
    double corners[8];           /* image corner points                     */
    double pixsize;              /* pixel size                              */
    double proj_coef[15];        /* GCTP projection parameters              */

    /* misc. variables
       ------------------*/
    long tmp_long;               /* temp long value                        */
    long std_out = 0;            /* standard out                           */
    int index;
    int junk_flags[4];           /* need these to pass to process_hdr but
                                    they are not every used                */
    long mode;
    long status;
    char *env_ptr;
    struct DDR ddr;              /* DDR to output                           */

    /* window paramters
       -------------------*/
    char units[4];               /* units of the input window(DEG, DMS, LS,
                                    PRO)*/
    long lngunits;               /* long value of the units DEG=4, DMS=5,
                                    PRO=2 (meters)              */
    double window[4];            /* image window                           */
    long valid_window[4];        /* long values of the converted and validated
                                window in sl, ss, nl, ns                  */

    /* get las parameters
       --------------------- */
    c_inlas (&vblk, &std_out);
    cwgetpar(&vblk, inhdr, htype, window, units, &resolution, &same_res);

    /* if we are not processing an image with all the same resolutions,
       then each different header for each resolution may have an _vis,
       _thm, or _pan on it.  this is only for Fast-L7A.
       ----------------------------------------------------------------*/
    if (!same_res)
    {
        if (!strcmp (htype, "L7A"))
        {
            switch (resolution)
            {
                case 30:
                    strcat (inhdr, "_vis");
                    break;
                case 60:
                    strcat (inhdr, "_thm");
                    break;
                case 15:
                    strcat (inhdr, "_pan");
                    break;
            }
        }
    }

    /* open and read the header
       --------------------------- */
    mode = IREAD;
    strcpy (hdr_extension, "fhdr");  /* for A, B, C, and SIC hdr types */
    if (!strcmp (htype, "A"))
        status = c_open_header_fst(tmpstr,inhdr,hdr_extension,&mode,
                                   HEADER_SIZE);
    else if (!strcmp (htype, "B") || !strcmp (htype, "C"))
        status = c_open_header_fst(fheader,inhdr,hdr_extension,&mode,
                                   HEADER_SIZE);
    else if (!strcmp (htype, "SIC"))
        status = c_open_header_fst(fheader,inhdr,hdr_extension,&mode,
                                   3*HEADER_SIZE);
    else if (!strcmp (htype, "L7A"))
    {
        strcpy (hdr_extension, "fst");
        status = c_open_header_fst(fheader,inhdr,hdr_extension,&mode,
                                   3*HEADER_SIZE);
    }
    else
    {
        c_lasmsg(TRUE,"fstfmtin-header","Unknown header type.");
    }

    if(status == E_FAIL)
    {
        c_lasmsg(TRUE,"fstfmtin-header","Unable to open specified header");
    }

    /* header A
       ----------- */
    if(!strcmp (htype, "A"))
    {
        tmpstr[HEADER_SIZE] = '\0';
    
        /* get the acquisition time -- only version A has this info
           --------------------------------------------------------*/
        strncpy(ac_time,&tmpstr[15],4);
        ac_time[4] = ':';
        ac_time[5] = tmpstr[19];
        ac_time[6] = '0';
        ac_time[7] = 0;
    
        /* convert the header to B format
           ------------------------------------*/
        c_convertab(tmpstr,fheader);
    }

    /* header B, C, SIC, or L7A
       --------------------------- */
    else
    {
        /* ver. B, C, L7A, SIC don't have the time so make up one
           --------------------------------------------------------- */
        strcpy(ac_time,"0000:00"); 
    }

    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
        fheader[3*HEADER_SIZE] = '\0';
    else
        fheader[HEADER_SIZE] = '\0';

    /* read the parameters for creating the ddr
       -------------------------------------------*/
    if (process_hdr(fheader, htype, origsize, &pixsize, proj_coef,
                    corners, &proj_code, &zone_code, &datum, source,
                    instrument, ac_date, junk_flags) == E_FAIL)
    {
        c_lasmsg(TRUE, "fstfmtin-convertwindow",
                 "Error processing the header");
    }

    /* if the datum is not available (set to 99), then a DDR cannot be
       created, thus the line/sample values of the window cannot be calculated
       ---------------------------------------------------------------------*/
    if ((datum == 99) && (strncmp (units, "LS", 2)))
    {
        c_lasmsg (TRUE,"fstfmtin-datum","Datum value is not available");
    }

    /* Create the dummy DDR
       -----------------------*/
    ddr.nl = origsize[0];
    ddr.ns = origsize[1];
    ddr.dtype = EBYTE;         /* simply set a dummy value */
    ddr.master_line = 1;
    ddr.master_sample = 1;
    ddr.proj_code = proj_code;
    ddr.zone_code = zone_code;
    ddr.datum_code = datum;

    /* get the system from the DATASYS environment variable */
    env_ptr = getenv("DATASYS");
    if (env_ptr != NULL)
        strncpy(ddr.system,env_ptr,12); /* 12 length */
    else
    {
        c_lasmsg(TRUE,"fstfmtin-datasys",
                 "DATASYS environment variable not set");
    }

    strcpy(ddr.proj_units,"meters");
    strcpy (ddr.last_used_date,"");
    strcpy (ddr.last_used_time,"");
    for (index = 0; index < 15; index++)
        ddr.proj_coef[index] = proj_coef[index];

    /* set the lat/long in the DDR */
    ddr.upleft[0] = corners[1];
    ddr.upleft[1] = corners[0];
    ddr.loleft[0] = corners[5];
    ddr.loleft[1] = corners[4];
    ddr.upright[0] = corners[3];
    ddr.upright[1] = corners[2];
    ddr.loright[0] = corners[7];
    ddr.loright[1] = corners[6];

    /* Set the pixel distance */
    ddr.pdist_y = ddr.pdist_x = pixsize;

    /* Set the line/sample increments to 1 */
    ddr.line_inc = ddr.sample_inc = 1.0;

    /* set the valid flags */
    ddr.valid[0] = 1; /* projection code */
    ddr.valid[1] = 1; /* zone code */
    ddr.valid[2] = 1; /* datum code */
    ddr.valid[3] = 1; /* proj_coef */
    ddr.valid[4] = 1; /* ground units */
    ddr.valid[5] = 1; /* ground distance */
    ddr.valid[6] = 1; /* corner coordinates */
    ddr.valid[7] = 1; /* line/sample increments */

    /* for each of the  output images to create with the available bands,
       write a DDR */
    ddr.nbands = 1;

    /* Convert the string units value to a long value
       DEG = 4, DMS = 5, PRO = 2 (meters).  If units
       are LS then we don't need the lngunits because
       c_window_proj_to_ls will not be called.
       -------------------------------------------------*/
    if (!strncmp (units, "DEG", 3))
        lngunits = 4;
    else if (!strncmp (units, "DMS", 3))
        lngunits = 5;
    else if (!strncmp (units, "PRO", 3))
        lngunits = 2;

    /* Convert the window parameters from their units to line/sample.
       This will not be done for units of LS.
       -----------------------------------------------------------------*/
    if (strncmp (units, "LS", 2))
    {
        if (c_window_proj_to_ls (&ddr, window, &lngunits) != E_SUCC)
        {
            c_lasmsg(TRUE,"fstfmtin-wndconv",
                     "Error converting window coordinates");
        }
    }

    /* Check the new window
       -----------------------*/
    if (c_check_window (&ddr, window, valid_window) != E_SUCC)
    {
        c_lasmsg(TRUE,"fstfmtin-wndchk",
                 "Error checking the window coordinates");
    }

    /* Return the valid window back to PDF.
       ---------------------------------------*/
    if (E_SUCC != lng2tae(&vblk,valid_window,1,4,"OUTWINDOW",&tmp_long))
    {
        c_lasmsg(TRUE,"fstfmtin-lng2tae","Error outputting OUTWINDOW");
    }

    if (q_out(&vblk) != SUCCESS)
    {
        c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
    }
}
