/*****************************************************************************

NAME:   FHDR2DDR

PURPOSE: Reads a Fast-A/B/C Format, Fast-L7A Format, or Space Imaging Fast-C
         header and fills in the ddr.


PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    0.0         04/90    T. Baltzer     ??? 
    0.1         12/91    J. Fenno       ???
    1.0         12/92    K. Sheffield   Initial development/rewrite
    1.1         07/93    D. Etrheim     Change to change SOM for new GCTP
                                        and use projection constants
    1.2         11/93    D. Etrheim     Made correction for Version A in
                                        SOM projection.  Also, undo blanking
                                        out of fields.
    1.3         08/94    D. Etrheim     Corrected so header file on disk is
                                        not updated when it is read.  Also
                                        modified so the tapes created by 
                                        FSTFMTOUT can be read (not all fields
                                        in the header get filled and this
                                        routine was trying to use these
                                        fields.) 
    1.4         03/95    S. Nelson      Modified code so that it does not
                                        fatal error out when certain
                                        projection information is not found
                                        in the header.
                                        Added a nonfatal message that the
                                        DDR is not updated.
                                        Added an assignment to update_ddr.c
                                        to assign datum to the datum code
                                        field of the ddr structure.
    7.2         04/99    G. Schmidt     Updated to support Fast-L7A Format. 
    7.2         06/99    G. Schmidt     Created the source in the DDR to be
                                        the L7 band number
    7.2         07/99    G.Schmidt      Updated to support Space Imaging
                                        Fast-C
    7.3         12/00    A.Shimitz      Added ddr_valid flags to indicate the
                                        appropriate setting of the ddr flags
                                        in update_ddr.  Now if any of the
                                        projection parameters are invalid
                                        update_ddr will still be called but it
                                        will be passed the ddr_valid flags so
                                        the projection parameters will be
                                        marked invalid.
                10/01    A.Weissenberger Formatting and removal of redundant
                                         c_lasmsg calls.
                                         
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT:    LAS

ALGORITHM DESCRIPTION:
   Call getpar to get TAE parameters
   Read in the fast format header, converting from version A to version B if
   needed.
   Call process_hdr to extract the projection, corner point data, and other 
   information from the header.
   If a fast format associated file needs to be created, blank out tape
   related fields, and then write out the header to disk.
   Call recalc_corners to calculate new corner points if the a subwindow of 
   the original image was specified.
   Call update_ddr to update the DDR of the image with the new corner points, 
   projection info, and other information extracted from the fast format 
   header.
   Copy and update image history, and issue completion message 

NOTES: Since the Fast-L7A Format headers are 3-1536 byte subheaders, the
header buffers need to be able to read 3*HEADER_SIZE information.

Space Imaging Fast-C Format is the same as Fast-L7A, in most cases.  If we
are dealing with a Space Imaging Fast-C format and the ellipsoid is Walbeck,
then this program will exit with an error.  Walbeck ellipsoids are not
supported by the LAS package.

Space Imaging Fast-C Format for IRS 1C specifies the bands as 2,3,4,5,P,
where P represents the Pan band.  Internally, the code will represent this
band as band 8.
****************************************************************************/

/* include files, global parameters, and definitions for FDHR2DDR    */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "fhdr.h"
#include "util.h"
#include "labserv.h"
#include "fstfmtin.h"

void main () 
{
    /* file names
       -------------*/
    struct PARBLK vblk;           /* TAE parameter block                   */
    char hostimg[CMLEN];          /* image host name                       */
    char hosthdr[CMLEN];          /* input fast format header host name    */
    char hhdr[CMLEN];             /* output fast format header host name   */
    char hdr_extension[CMLEN];    /* header extension string (fhdr or fst) */

    /* image information
       --------------------*/
    char bndlist[MAXBND][6];      /* list of bands being read from tape (each
                                     band may be up to 5 characters/digits
                                     long)                                  */
    int nbands;                   /* the number of bands in the bndlist
                                     array */
    char fheader[3*HEADER_SIZE+1];/* buffer for header file                 */
    char htype[5];                /* fast format header type (A,B,C,L7A,SIC)
                                   */
    char tmpstr[HEADER_SIZE+1];   /* temporary string                       */

    long hflag;                   /* blank out fast format header fields    */
    long window[4];               /* image window                           */

    /* data retrieved from fast format header
       -----------------------------------------*/
    char source[32];             /* source of image                        */
    char instrument[32];         /* instrument                             */
    char ac_date[10];            /* acquistion date                        */
    char ac_time[8];             /* acquistion time                        */
    long datum = 0;              /* GCTP datum code                        */
    long origsize[2];            /* original size of the image on tape     */
    long proj_code;              /* GCTP projection code                   */
    long zone_code;              /* UTM or State Plane code                */

    double corners[8];           /* image corner points (x,y) UL, UR, LL, LR*/
    double pixsize;              /* pixel size                             */
    double proj_coef[15];        /* GCTP projection parameters             */

    /* misc. variables
       ---------------*/
    long std_out = 0;            /* logical unit for standard output       */
    long one = 1;
    long mode;
    long status;
    long zero = 0;
    int  i;                      /* loop counter                            */
    int  ddr_valid_flags[8];     /* values for the ddr.valid flags to be
                                    passed to update_ddr                    */

    /* get las parameters
       --------------------- */
    c_inlas (&vblk, &std_out);
    getpar(&vblk, hostimg, hosthdr, window, bndlist, &nbands, htype, &hflag);

    /* initialize the ddr valid flags to all TRUE
       ---------------------------------------------*/
    for (i = 0; i < 8; i++)
        ddr_valid_flags[i] = TRUE;

    /* open and read the header
       --------------------------- */
    mode = IREAD;
    strcpy (hdr_extension, "fhdr");  /* for A, B, C, SIC hdr types */
    if (!strcmp (htype, "A"))
        status = c_open_header_fst(tmpstr,hosthdr,hdr_extension,&mode,
                                   HEADER_SIZE);
    else if (!strcmp (htype, "B") || !strcmp (htype, "C"))
        status = c_open_header_fst(fheader,hosthdr,hdr_extension,&mode,
                                   HEADER_SIZE);
    else if (!strcmp (htype, "SIC"))
        status = c_open_header_fst(fheader,hosthdr,hdr_extension,&mode,
                                   3*HEADER_SIZE);
    else if (!strcmp (htype, "L7A"))
    {
        strcpy (hdr_extension, "fst");
        status = c_open_header_fst(fheader,hosthdr,hdr_extension,&mode,
                                   3*HEADER_SIZE);
    }
    else
    {
        c_lasmsg(TRUE,"fstfmtin-header","Unknown header type.");
    }

    if(status == E_FAIL)
    {
        c_lasmsg(TRUE,"fstfmtin-header","Unable to open specified header");
    }

/* header A
----------- */
    if(!strcmp (htype, "A"))
    {
        tmpstr[HEADER_SIZE] = '\0';
    
        /* get the acquisition time -- only version A has this info
           --------------------------------------------------------*/
        strncpy(ac_time,&tmpstr[15],4);
        ac_time[4] = ':';
        ac_time[5] = tmpstr[19];
        ac_time[6] = '0';
        ac_time[7] = 0;
    
        /* convert the header to B format
           ------------------------------------*/
        c_convertab(tmpstr,fheader);
    }

    /* header B, C, L7A, or SIC
       ---------------------------*/
    else
    {
        /* ver. B, C, L7A, and SIC don't have the time so make up one
           -------------------------------------------------------------*/
        strcpy(ac_time,"0000:00"); 
    }

    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        fheader[3*HEADER_SIZE] = '\0';
    }
    else
    {
        fheader[HEADER_SIZE] = '\0';
    }

    /* read the parameters for updating the ddr
       -------------------------------------------*/
    status = process_hdr(fheader, htype, origsize, &pixsize, proj_coef,
                         corners, &proj_code, &zone_code, &datum, source,
                         instrument, ac_date, ddr_valid_flags);

    /* write out a fast format associated file if it is desired
       -----------------------------------------------------------*/
    if(hflag)
    {
        /* Open the new file and write out the header contents.
           -------------------------------------------------------*/
        mode = IWRITE;
    
        if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
        {
            if (!strcmp (htype, "L7A"))
                strcpy (hdr_extension, "fst");
            if (c_open_header_fst(fheader,hostimg,hdr_extension,&mode,
                                  3*HEADER_SIZE) == E_FAIL )
            {
                c_lasmsg(TRUE,"fstfmtin-header",
                         "Unable to open specified header");
            }
        }
        else
        {
            if (c_open_header_fst(fheader,hostimg,hdr_extension,&mode,
                                  HEADER_SIZE) == E_FAIL )
            {
                c_lasmsg(TRUE,"fstfmtin-header",
                         "Unable to open specified header");
            }
        }
    }

    /* Calculate new corner points for the subwindow if necessary
       (cannot recalc if the corners were not processed, which is
       the case if the datum is not available (99))
       -------------------------------------------------------------*/
    if (status == E_SUCC) /* process_hdr had no problem retrieving orig
                             window */
    {
        if ((origsize[0] != window[NL] || origsize[1] != window[NS]) &&
            (datum != 99))
        {
            recalc_corners(corners,origsize,window);
        }
    }
    
    /* set the datum code flag to false if the datum code is 99
       -----------------------------------------------------------*/
    if (datum == 99)
        ddr_valid_flags[2] = FALSE;

    /* update the ddr
       ---------------*/
    update_ddr(hostimg, window, pixsize, proj_coef, corners, proj_code,
               zone_code, datum, source, bndlist, nbands, instrument,
               ac_date, ac_time, htype, ddr_valid_flags);

    if(hflag)
    {
        c_lsmknm(hostimg,hdr_extension,hhdr);
        c_complt ("FHDR2DDR", hhdr, &one);
    }
    else
        c_complt ("FHDR2DDR", NULL, &zero);
}
