/****************************************************************************
NAME: GET_ZONENUM

PURPOSE: To retrieve the appropriate zone code number from the tape's header
     file, convert it if necessary, and place it in the project parameter
     file.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    7.2         04/99    G. Schmidt   Updated to support Fast-L7A Format
    7.2         07/99    G. Schmidt   Updated to support Space Imaging Fast-C
    7.3         10/00    A.Shimitz    Modified method of determining when to
                                      negate the zone code with Fast-A
                                      imagery.
                                      Since the center coordinates are not
                                      available, estimate the center latitude
                                      by calculating the average of the UL
                                      lat and the LR lat. (CR #1009)
                10/01    A.Weissenberger CR5057. Formatting and removal of
                                         redundant calls to c_lasmsg
                                         
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "proj.h"
#include "fhdr.h"
#include "fstfmtin.h"

void get_zonenum (
    char header[],     /* ptr into the tape's header file for the zone
                          number and center latitude                        */
    char *ff_version,  /* character designation of the Fast Format version  */
    char proj_type[],  /* string holding the projection type of the image   */
    long proj_number,  /* projection number of the image                    */
    double *projcoef,  /* GCTP projection parameters (15)                   */
    long *zone_number) /* zone number of the image                          */

{
    long status;            /* function return status                       */
    long double centerlat;  /* scene center latitude                        */
    long double ul_lat;     /* latitude of UL corner of the image           */
    long double lr_lat;     /* latitude of LR corner of the image           */

    /* If the projection number implies Universal Transverse Mercator or State
       Plane Grid, or the projection type is Gauss-Krueger, Swedish National
       Grid, or State Plane Coordinate System then retrieve the zone number
       from the tape's header file.  In all other cases, force the zone
       number to 62.
       --------------------------------------------------------------------*/
    if ((proj_number == 1)              || (proj_number == 2)              ||
        (strcmp(proj_type,"gk")   == 0) || (strcmp(proj_type,"sng")  == 0) ||
        (strcmp(proj_type,"spcs") == 0))
    {
        /* Note: For Fast-B/C and Fast-L7A the field number is the same, but
         * for Fast-L7A the geometric header is needed.  For Space Imaging
         * Fast-C the zone number is the 3rd projection parameter.
         */
        if (!strcmp (ff_version, "L7A"))
            status = c_fhdrrd_fst(header,(long)(GEOM+47),(char *)zone_number,
                                  ff_version);
        else if (!strcmp (ff_version, "SIC"))
            *zone_number = (long) projcoef[2];
        else
            status = c_fhdrrd_fst(header,47,(char *)zone_number,ff_version);

        if(E_FAIL == status)
        {
            c_lasmsg(TRUE,"fstfmtin-fhdrrd","Error reading the zone number");
        }
    }
    else
        *zone_number = 62;

    /* If the projection number implies State Plane Grid or the projection
       type is State Plane Grid or State Plane Coordinate System and the Fast
       Format version is A, then convert from the old zone code number to the
       new zone code number.  This switch statement was taken from
       vaxa::pkg:[spotpcs.sourceapp.util.haat2ddr]c_fillddr.c
       --------------------------------------------------------------------*/
    if ((!strcmp (ff_version, "A")) && ((proj_number == 2)
        || (!strcmp(proj_type, "stpl")) || (!strcmp(proj_type, "spcs")) ) )
    {
        switch (*zone_number)
        {
            case 3101: *zone_number = 101;  break;
            case 3126: *zone_number = 102;  break;
            case 6101: *zone_number = 5001; break;
            case 6126: *zone_number = 5002; break;
            case 6151: *zone_number = 5003; break;
            case 6176: *zone_number = 5004; break;
            case 6201: *zone_number = 5005; break;
            case 6226: *zone_number = 5006; break;
            case 6251: *zone_number = 5007; break;
            case 6276: *zone_number = 5008; break;
            case 6301: *zone_number = 5009; break;
            case 6326: *zone_number = 5010; break;
            case 3151: *zone_number = 201;  break;
            case 3176: *zone_number = 202;  break;
            case 3201: *zone_number = 203;  break;
            case 3226: *zone_number = 301;  break;
            case 3251: *zone_number = 302;  break;
            case 3276: *zone_number = 401;  break;
            case 3301: *zone_number = 402;  break;
            case 3326: *zone_number = 403;  break;
            case 3351: *zone_number = 404;  break;
            case 3376: *zone_number = 405;  break;
            case 3401: *zone_number = 406;  break;
            case 3426: *zone_number = 407;  break;
            case 3451: *zone_number = 501;  break;
            case 3476: *zone_number = 502;  break;
            case 3501: *zone_number = 503;  break;
            case 3526: *zone_number = 600;  break;
            case 3551: *zone_number = 700;  break;
            case 3601: *zone_number = 901;  break;
            case 3626: *zone_number = 902;  break;
            case 3576: *zone_number = 903;  break;
            case 3651: *zone_number = 1001; break;
            case 3676: *zone_number = 1002; break;
            case 5876: *zone_number = 5101; break;
            case 5901: *zone_number = 5102; break;
            case 5926: *zone_number = 5103; break;
            case 5951: *zone_number = 5104; break;
            case 5976: *zone_number = 5105; break;
            case 3701: *zone_number = 1101; break;
            case 3726: *zone_number = 1102; break;
            case 3751: *zone_number = 1103; break;
            case 3776: *zone_number = 1201; break;
            case 3801: *zone_number = 1202; break;
            case 3826: *zone_number = 1301; break;
            case 3851: *zone_number = 1302; break;
            case 3876: *zone_number = 1401; break;
            case 3901: *zone_number = 1402; break;
            case 3926: *zone_number = 1501; break;
            case 3951: *zone_number = 1502; break;
            case 3976: *zone_number = 1601; break;
            case 4001: *zone_number = 1602; break;
            case 4026: *zone_number = 1701; break;
            case 4051: *zone_number = 1702; break;
            case 4076: *zone_number = 1801; break;
            case 4101: *zone_number = 1802; break;
            case 4126: *zone_number = 1900; break;
            case 4151: *zone_number = 2001; break;
            case 4176: *zone_number = 2002; break;
            case 4201: *zone_number = 2101; break;
            case 4226: *zone_number = 2102; break;
            case 4251: *zone_number = 2103; break;
            case 6351: *zone_number = 2111; break;
            case 6376: *zone_number = 2112; break;
            case 6401: *zone_number = 2113; break;
            case 4276: *zone_number = 2201; break;
            case 4301: *zone_number = 2202; break;
            case 4326: *zone_number = 2203; break;
            case 4351: *zone_number = 2301; break;
            case 4376: *zone_number = 2302; break;
            case 4401: *zone_number = 2401; break;
            case 4426: *zone_number = 2402; break;
            case 4451: *zone_number = 2403; break;
            case 4476: *zone_number = 2501; break;
            case 4501: *zone_number = 2502; break;
            case 4526: *zone_number = 2503; break;
            case 4551: *zone_number = 2601; break;
            case 4576: *zone_number = 2602; break;
            case 4601: *zone_number = 2701; break;
            case 4626: *zone_number = 2702; break;
            case 4651: *zone_number = 2703; break;
            case 4676: *zone_number = 2800; break;
            case 4701: *zone_number = 2900; break;
            case 4726: *zone_number = 3001; break;
            case 4751: *zone_number = 3002; break;
            case 4776: *zone_number = 3003; break;
            case 4801: *zone_number = 3101; break;
            case 4826: *zone_number = 3102; break;
            case 4851: *zone_number = 3103; break;
            case 4876: *zone_number = 3104; break;
            case 4901: *zone_number = 3200; break;
            case 4926: *zone_number = 3301; break;
            case 4951: *zone_number = 3302; break;
            case 4976: *zone_number = 3401; break;
            case 5001: *zone_number = 3402; break;
            case 5026: *zone_number = 3501; break;
            case 5051: *zone_number = 3502; break;
            case 5076: *zone_number = 3601; break;
            case 5101: *zone_number = 3602; break;
            case 5126: *zone_number = 3701; break;
            case 5151: *zone_number = 3702; break;
            case 5176: *zone_number = 3800; break;
            case 5201: *zone_number = 3901; break;
            case 5226: *zone_number = 3902; break;
            case 5251: *zone_number = 4001; break;
            case 5276: *zone_number = 4002; break;
            case 5301: *zone_number = 4100; break;
            case 5326: *zone_number = 4201; break;
            case 5351: *zone_number = 4202; break;
            case 5376: *zone_number = 4203; break;
            case 5401: *zone_number = 4204; break;
            case 5426: *zone_number = 4205; break;
            case 5451: *zone_number = 4301; break;
            case 5476: *zone_number = 4302; break;
            case 5501: *zone_number = 4303; break;
            case 5526: *zone_number = 4400; break;
            case 5551: *zone_number = 4501; break;
            case 5576: *zone_number = 4502; break;
            case 5601: *zone_number = 4601; break;
            case 5626: *zone_number = 4602; break;
            case 5651: *zone_number = 4701; break;
            case 5676: *zone_number = 4702; break;
            case 5701: *zone_number = 4801; break;
            case 5726: *zone_number = 4802; break;
            case 5751: *zone_number = 4803; break;
            case 5776: *zone_number = 4901; break;
            case 5801: *zone_number = 4902; break;
            case 5826: *zone_number = 4903; break;
            case 5851: *zone_number = 4904; break;
            case 6001: *zone_number = 5201; break;
            case 6051: *zone_number = 5202; break;
            default:
                c_lasmsg(TRUE,"fstfmtin-zone",
                         "Illegal zone number %1d, for projection %s",
                         *zone_number, proj_type);
        }
    } /* if */

    /* retrieve the scene center latitude and negate the zone code if
       necessary
       ---------------------------------------------------------------------*/
    if (!strcmp (ff_version, "L7A") || !strcmp (ff_version, "SIC"))
    {
        /* Retrieve the value from the header file buffer. c_fhdrrd_fst
           converts the string to a double using the dms2deg routine.
           -----------------------------------------------------------------*/
        if(E_FAIL == c_fhdrrd_fst(header,(long)(GEOM+98),(char *)&centerlat,
                                  ff_version))
        {
            c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                     "Error reading the center latitude");
        }
    }
    else if(!strcmp (ff_version, "A"))
    {
        /* Retrieve the values from the header file buffer
           -------------------------------------------------*/
        if(E_FAIL == c_fhdrrd_fst(header,65,(char *)&ul_lat,ff_version))
        {
            c_lasmsg(TRUE,"fstfmtin-fhdrrd","Error reading the UL latitude");
        }
        if(E_FAIL == c_fhdrrd_fst(header,81,(char *)&lr_lat,ff_version))
        {
            c_lasmsg(TRUE,"fstfmtin-fhdrrd","Error reading the LR latitude");
        }
        centerlat = (ul_lat + lr_lat) / 2.0;
    }
    else
    {
        /* Retrieve the value from the header file buffer. c_fhdrrd_fst
           converts the string to a double using the dms2deg routine.
           ----------------------------------------------------------------*/
        if(E_FAIL == c_fhdrrd_fst(header,107,(char *)&centerlat, ff_version))
        {
            c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                     "Error reading the center latitude");
        }
    }

    if((centerlat < 0.0) && (proj_number == UTM) && (*zone_number > 0))
        *zone_number = -*zone_number;
}
