/******************************************************************************

NAME: LOCATE_CD_HDR

PURPOSE: Opens or reopens the CD being read from to locate imagery data and
         return the path to the directory the data was found in as well as
         the header file basename.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.3  09/00  A.Shimitz        EDC   Initial development - CR#3519
    7.3  08/01  A.Weissenberger  EDC   CR#5057 - Removed hardcoded image
                                       extensions
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
   mount and open or just reopen the CD
   call search_4_hdrs to locate image header file(s)
   pass the basename and path determined by search_4_hdrs back to TAE

NOTES:
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "fhdr.h"
#include "util.h"
#include "fstfmtin.h"

#define MAXDIR 10
#define BASNAMLEN 120

void main(void)
{
    struct PARBLK vblk;        /* TAE parameter block                        */
   
    char basename[MAXIMG][BASNAMLEN];
                               /* basename supplied by the user in the INFILE
                                  param - if NULL it will reflect the
                                  basename(s) of the header file(s) found    */
    char density[CMLEN];       /* tape density returned from tapeio routines */
    char drivename[CMLEN];     /* string descriptor for CD-ROM drive         */
    char lhost[CMLEN];         /* local host name                            */
    char opr_drive[CMLEN];     /* string descriptor for CD-ROM drive         */
    char oprmsg[CMLEN];        /* message for the operator for CD mounting   */
    char *path = NULL;         /* path to directory where imagery is found
                                  (format: dir1/dir2/dir3/)                  */
    char *out_path = NULL;     /* path to directory where imagery is found
                                  (format: dir1_dir2_dir3)                   */
    char srchname[CMLEN];      /* string descriptor for CD-ROM drive         */
    char *string_ptrs[MAXIMG]; /* used to return variables to tae            */
    char *strptr = NULL;       /* pointer for path string conversion         */
    char tape_id[CMLEN];       /* tape id returned from tapeio routines      */
   
    long hdr_count;            /* number of header files found - if more than
                                  one resolution header file is found for one
                                  L7A image is found, it is just counted as 1*/
    long i;                    /* loop index                                 */
    size_t length;             /* string length                              */
    long mounted;              /* CD mounted flag                            */
    long num_usrbasnam;        /* flag to indicate if user specified a
                                  basename with the INFILE parameter
                                  (1 - they did, 0 - they didn't)            */
    long norewind = 0;         /* no rewind flag                             */
    long one = 1;              /* constnat one                               */
    long status;               /* return status                              */
    long std_out = 0;          /* standard out                               */
    long unit;                 /* tape unit                                  */
    long zero = 0;             /* constant zero                              */

    c_inlas(&vblk, &std_out);
    
    /* Get the variables from the TAE variable block
       ------------------------------------------------*/
    lcdgetpar(&vblk, oprmsg, drivename, basename, &mounted, opr_drive,
              &num_usrbasnam);
   
    /* Open or reopen the disk and get the drive name.  The oprmsg is the
       message returned from the operator.  This is the drive name.
       ---------------------------------------------------------------------*/
    strcpy(density,"0");
    if (mounted == FALSE)
    {
        status = c_tpopen(&unit, &zero, &one, &zero, density, &zero, oprmsg,
                          tape_id);
        if (status != TAPE_SUCC)
        {
            c_lasmsg(TRUE,"fstfmtin-tpopen", "Error opening the CD file");
        }
      
        if (SUCCESS != q_intg(&vblk,"MOUNTED",1,(int *)&one,P_UPDATE))
        {
            c_lasmsg(TRUE,"fstfmtin-integer","Error passing MOUNTED to TAE");
        }
      
        /* Return the operator drive name
           ---------------------------------*/
        length = strlen(oprmsg);
        if (E_SUCC != str2tae(&vblk,oprmsg,(long) length,"%s",1,"OPR_DRIVE"))
        {
            c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting OPR_DRIVE");
        }
      
        /* If the DRIVE doesn't have an @ sign in it then this is a local
           CD drive.  If the DRIVE does have an @ sign then this is a
           remote drive.  If this is a local DRIVE, then we need to get the
           hostname and compute the DRIVE from hostname + @ + DRIVE.
           If the DRIVE has !! as the first two characters, then this
           is a local drive and the user doesn't want to prefix it with the
           hostname.  We should strip off the first !!s and process this
           as the specified drive name.
           ------------------------------------------------------------------*/
        if (strchr(oprmsg,'@') == NULL)
        {  /* This is a local tape drive */
            if (gethostname(lhost,CMLEN) != 0)
            {
                c_lasmsg(TRUE,"fstfmtin-hostname",
                         "Error reading the hostname");
            }
            length = strlen(lhost);
            c_up2low(lhost,(long *) &length);
            sprintf (drivename, "/%s@%s/", lhost, oprmsg);
            sprintf (srchname, "[%s@%s]", lhost, oprmsg);
        }
        else if (strncmp (oprmsg, "!!", 2) == 0)
        {
            sprintf (drivename, "/%s/", &oprmsg[2]);
            sprintf (srchname, "[%s]", &oprmsg[2]);
        }
        else
        {
            /* This is a remote tape drive */
            sprintf (drivename, "/%s/", oprmsg);
            sprintf (srchname, "[%s]", oprmsg);
        }
      
        /* Return the full drive name
           -----------------------------*/
        length = strlen(drivename);
        if (E_SUCC != str2tae(&vblk,drivename,(long) length,"%s",1,"DRIVE"))
        {
            c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting DRIVE");
        }
      
        /* Return the TAE search drive name
           -----------------------------------*/
        length = strlen(srchname);
        if (E_SUCC != str2tae(&vblk,srchname,(long) length,"%s",1,
                              "SRCH_DRIVE"))
        {
            c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting SRCH_DRIVE");
        }
      
        /* Immediately send the drive name back to TAE.  This way
           if we error out it will know which drive to read or dismount.
           -------------------------------------------------------------*/
        if (q_out(&vblk) != SUCCESS)
        {
            c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
        }
    }
    else  /* MOUNTED = TRUE */
    {
        status = c_tpreop(&unit, &zero, &one, &zero, density, &zero,
                          opr_drive);
        if (status != TAPE_SUCC)
        {
            c_tpclos(&unit, &one, &norewind, &zero);
            c_lasmsg(TRUE,"fstfmtin-tpreop", "Error reopening the CD file");
        }
    }
   
    /* Use search_4_hdrs to find the image header file(s) on the CD.  Malloc
       path just so the strcpy(cur_path, path) in search_4_hdrs will work
       the first time through.
       ---------------------------------------------------------------------*/
    path = (char *)malloc(1 * sizeof(char));
    if(path == NULL)
    {
        c_lasmsg(TRUE, "fstfmtin-alloc", "Error allocating dynamic memory");
    }
    strcpy(path, "");
    hdr_count = search_4_hdrs(drivename, num_usrbasnam, basename, &path);
   
    /* Convert the path to out_path for use with the OUT filename.
       path is of the form dir1/dir2/dir3/ and out_path needs to be
       of the form dir1_dir2_dir3.
       ---------------------------------------------------------------*/
    if(strlen(path) > 0)
    {
        out_path = (char *)malloc (strlen(path) * sizeof(char));
        if(out_path == NULL)
        {
            free(path);
            c_lasmsg(TRUE, "fstfmtin-alloc",
                     "Error allocating dynamic memory");
        }
        strcpy(out_path, path);
      
        /* Remove the tail / so dir1/dir2/ will become dir1/dir2
           --------------------------------------------------------*/
        strptr = strrchr(out_path, '/');
        strptr = '\0';
      
        /* Now replace the remaining / with _ so dir1/dir2 becomes dir1_dir2
           ------------------------------------------------------------------*/
        strptr = strchr(out_path, '/');
        while(strptr != NULL)
        {
            *strptr = '_';
            strptr = strchr(out_path, '/');
        }
    }
    
    /* Pass basename and path to TAE variable block
       -----------------------------------------------*/
    for (i = 0; i < hdr_count; i++)
    {
        string_ptrs[i] = basename[i];
    }
   
    if(q_string(&vblk,"BASENAME",hdr_count,string_ptrs,P_UPDATE) != SUCCESS)
    {
        free(path);
        free(out_path);
        c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting BASENAME");
    }
    
    if(strlen(path) > 0)
    {
        if (E_SUCC != str2tae(&vblk,path,(long)strlen(path),"%s",1,"CDPATH"))
        {
            free(path);
            free(out_path);
            c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting CDPATH");
        }
        if (E_SUCC != str2tae(&vblk,path,(long)strlen(out_path),"%s",1,
                              "OUT_CDPATH"))
        {
            free(path);
            free(out_path);
            c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting OUT_CDPATH");
        }
    }
    if (q_out(&vblk) != SUCCESS)
    {
        free(path);
        free(out_path);
        c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
    }
   
    /* Free resourses
       -----------------*/
    free(path);
    path = NULL;
    free(out_path);
    out_path = NULL;
   
    /* Note that we purposely do not close the CD since read_header
       will use it */
}

/******************************************************************************

NAME: search_4_hdrs

PURPOSE: To locate imagery data on a CD and modify the 'path' to the directory
         the data was found in as well as the header file 'basename' for the
         calling routine locate_cd_hdr.


PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.3  09/00  A.Shimitz        EDC   Initial development - CR#3519

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
   set hdr_found to FALSE
   set cur_path to path (path is "" when intially called to start in root dir)
   create complete Fast-A,B,C header filename to search CD for
   check current directory for this header file (c_getfiles will set num_hdrs)
   set hdr_found flag if header file was found
   if hdr_found flag is not set
      create complete Fast-L7A visible header
      check current directory for this header file
      set hdr_found flag if header file was found
   if hdr_found flag is not set
      create complete Fast-L7A thermal header
      check current directory for this header file
      set hdr_found flag if header file was found
   if hdr_found flag is not set
      create complete Fast-L7A pan header
      check current directory for this header file
      set hdr_found flag if header file was found
   if hdr_found flag is not set
      call getdirs to determine if there are any subdirectories to search
      if there are any subdirectories
         for each subdirectory
            set the new path to cur_path appended with the current subdirectory
            call search_4_hdrs with the new path to subdirectory to search
            set num_hdrs to the return value of search_4_hdrs
            if num_hdrs is > 0
               return num_hdrs
         return 0 (no header files in any subdirectory so allow the process to
                   continue searching the subdirectories of the parent dir)
      else
         return 0 (no header files in current dir and there are no subdirs)
   else (a header file was found so we need to set basename and path)
      if basename was not specified by the user
         determine the basename(s) from the file(s) returned by c_getfiles
      return num_hdrs

NOTES:

RETURNS:
   Returns the number of images found.  Passes back through the parameters
   the path to the directory on the CD where the image(s) was found and the
   basename for the image(s).

******************************************************************************/

int search_4_hdrs(
    char *drivename,          /* I: string descriptor for CD-ROM drive       */
    long num_usrbasnam,       /* I: indicates whether user supplied a basename
                                 with the INFILE parameter (1 - they did,
                                 0 - they didn't)           */
    char basename[][BASNAMLEN],
    /* I/O: basename supplied by the user in the
       INFILE param - if NULL it will reflect
       the basename(s) of the header file(s)
       found                                   */
    char **path)               /* I/O: path to current directory we are
                                  searching                               */
{
    char cur_path[CMLEN];      /* path to current directory to search on CD  */
    char found_dirs[MAXDIR][CMLEN];
    /* array of subdirectories in the current
                                 directory                                  */
    char found_files[MAXIMG][CMLEN];
    /* array of filenames that matched the search */
    char hdr_file[CMLEN];      /* header file found in the search            */
    char srch_filename[CMLEN]; /* full host file name (including drive
                                 specification) to search for               */
    char *str_ptr = NULL;      /* string pointer for string manipulation     */
    char tmp_basename[CMLEN];  /* temp basename for manipulation             */
    char tmp_str[CMLEN];       /* temp string                                */
   
    int ndirs;                 /* number of directories found in the current
                                 directory                                  */
   
    long hdr_found = FALSE;    /* flag indicating whether a header file was
                                  found or not                               */
    long i;                    /* looping index                              */
    long num_hdrs;             /* number of files found in the search        */
   
    size_t length;             /* length of a string                         */
   
    char *fast_abc_suffix = "_[Hh][Ee][Aa][Dd][Ee][Rr].[Dd][Aa][Tt]";
    char *fast_l7_suffix = ".[Ff][Ss][Tt]";
    char *fast_l7_visible = "_[Hh][Rr][Ff]";
    char *fast_l7_thermal = "_[Hh][Tt][Mm]";
    char *fast_l7_pan = "_[Hh][Pp][Nn]";
   
   /* Initialize variables
   -----------------------*/
    strcpy(cur_path, *path);
    num_hdrs = ndirs = 0;
    length = strlen(drivename);
   
    /* Copy the user-supplied basename to tmp_basename
   --------------------------------------------------*/
    if(num_usrbasnam < 1)
        strcpy(tmp_basename, "*");
    else
        strcpy(tmp_basename, basename[0]);
   
   /* Look for Fast-ABC or SIC header file
   ---------------------------------------*/
    sprintf(srch_filename, "%s%s%s%s", drivename, cur_path, tmp_basename,
            fast_abc_suffix );
   
    /* Search given directory on CD for a header file.  If there are
      multiple header files (and hence multiple images) we will save
      each basename and pass back the number of header files found.
   ----------------------------------------------------------------*/
    if (c_getfiles(srch_filename, found_files, &num_hdrs, MAXIMG, 1, 0)
        != E_SUCC)
    {
        c_lasmsg(TRUE, "fstfmtin-c_getfiles","Error searching for header "
                 "file.");
    }
    
   
    /* Set flag if we found a matching header file
   ----------------------------------------------*/
    if(num_hdrs > 0)
        hdr_found = TRUE;
   
   /* If a Fast-A,B,C or SIC header file was not found, look for a
      L7A header file. We will check for header files of each
      resolution separately so it is easier to determine if we
      found multiple images
   ----------------------------------------------------------------*/
    if(!hdr_found)
    {
        /* Look for Fast-L7A visible header file
           ----------------------------------------*/
        sprintf(srch_filename, "%s%s%s%s%s", drivename, cur_path, tmp_basename,
                fast_l7_visible, fast_l7_suffix);
      
        /* Search given directory on CD for a header file.  If there are
         multiple header files (and hence multiple images) we will save
         each basename and pass back the number of header files found.
      ----------------------------------------------------------------*/
        if (c_getfiles(srch_filename, found_files, &num_hdrs, MAXIMG, 1, 0)
            != E_SUCC)
        {
            c_lasmsg(TRUE, "fstfmtin-c_getfiles", "Error searching for header "
                     "file.");
        }
        
      
        /* Set flag if we found a matching header file
      ----------------------------------------------*/
        if(num_hdrs > 0)
            hdr_found = TRUE;
    }
    if(!hdr_found)
    {
        /* Look for Fast-L7A thermal header file
           ----------------------------------------*/
        sprintf(srch_filename, "%s%s%s%s%s", drivename, cur_path, tmp_basename,
                fast_l7_thermal, fast_l7_suffix);
      
        /* Search given directory on CD for a header file.  If there are
         multiple header files (and hence multiple images) we will save
         each basename and pass back the number of header files found.
      ----------------------------------------------------------------*/
        if (c_getfiles(srch_filename, found_files, &num_hdrs, MAXIMG, 1, 0)
            != E_SUCC)
        {
            c_lasmsg(TRUE, "fstfmtin-c_getfiles","Error searching for header "
                     "file.");
        }       
        /* Set flag if we found a matching header file
               ----------------------------------------------*/
        if(num_hdrs > 0)
            hdr_found = TRUE;
    }
    if(!hdr_found)
    {
        /* Look for Fast-L7A pan header file
           ----------------------------------------*/
        sprintf(srch_filename, "%s%s%s%s%s", drivename, cur_path,
                tmp_basename, fast_l7_pan, fast_l7_suffix);
      
        /* Search given directory on CD for a header file.  If there are
               multiple header files (and hence multiple images) we will save
               each basename and pass back the number of header files found.
               -------------------------------------------------------------*/
        if (c_getfiles(srch_filename, found_files, &num_hdrs, MAXIMG, 1, 0)
            != E_SUCC)
        {
            c_lasmsg(TRUE, "fstfmtin-c_getfiles","Error searching for header ",
                     "file.");
        }
        
        /* Set flag if we found a matching header file
               ----------------------------------------------*/
        if(num_hdrs > 0)
            hdr_found = TRUE;
    }
   
    /* If we did not find any image header files in the current directory,
           check each subdirectory, if there are any.
           ------------------------------------------------------------------*/
    if(!hdr_found)
    {
        /* Create wildcard filename to search the directory for subdirs
           ---------------------------------------------------------------*/
        sprintf(tmp_str, "%s%s%s", drivename, cur_path, "*");
        getdirs(tmp_str, found_dirs, &ndirs, MAXDIR, 0);
        if(ndirs > 0)
        {
            for(i = 0; i < ndirs; i++)
            {
                /* Call search_4_hdrs for each subdirectory found.
                   Modify path each time to reflect new subdirectory
                   (/drive/dir1/, /drive/dir2/...)
                   -----------------------------------------------------*/
                str_ptr = strrchr(found_dirs[i], '/');
                str_ptr++;
                length = strlen(cur_path);
                length += strlen(str_ptr);
                *path = (char *)realloc(*path, (length + 2) * sizeof(char));
                if(*path == NULL)
                {
                    c_lasmsg(TRUE, "fstfmtin-alloc",
                             "Error allocating dynamic memory");
                }
                else
                {
                    c_lasmsg(FALSE, "informational",
                             "Memory reallocated successfully");
                }
                sprintf(*path, "%s%s%s", cur_path, str_ptr, "/");
                num_hdrs = search_4_hdrs(drivename, num_usrbasnam, basename,
                                         path);
                if(num_hdrs > 0)
                {
                    /* Return number of header files found
                       --------------------------------------*/
                    return num_hdrs;
                }
            }
         
            /* If we get here then we have not found any header files
               in any of the subdirectories so we need to return the
               searching back up to the parent directory to search its
               next subdirectory.
               ---------------------------------------------------------*/
            return 0;
        }
        else
        {
            /* We did not find any header files in the current directory and
               there are no subdirectories, so return to the parent
               directory so it can search its next subdirectory.
               -------------------------------------------------------------*/
            return 0;
        }
    }
      
    /* If a header file was found set the flag and copy the basename
       if necessary.  Return the number of header files found.
       ----------------------------------------------------------------*/
    else
    {
        if(num_usrbasnam < 1)
        {
            for(i = 0; i < num_hdrs; i++)
            {
                /* Remove the drive specification that preceeds the file
                   name when returned from c_getfiles
                   -----------------------------------------------------*/
                str_ptr = strrchr(found_files[i], '/');
                str_ptr++;
                strcpy(hdr_file, str_ptr);
            
                /* Now we need to remove the header extension to get the
                   basename by itself
                   -----------------------------------------------------*/
                    
                length =(strlen(hdr_file) - strlen(strrchr(hdr_file, '_')));
                strncpy(tmp_basename, hdr_file, length);
                tmp_basename[(long)length] = '\0';
                strcpy(basename[i], tmp_basename);
            }
        }
        /* Return number of header files found
           --------------------------------------*/
        return num_hdrs;
    }
}
