/*****************************************************************************

NAME: MERGE_HDR_BANDS

PURPOSE: To read Fast-ABC, Fast-L7A, and Space Imaging Fast-C Format tape
     headers from disk, retrieve the bands from all the headers, and merge
     all these bands into the first header.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  03/00  G.Schmidt        EDC   Original development.
         10/01  A.Weissenberger  EDC   CR 5057. Formatting and removal of
                                       redundant c_lasmsg calls.
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES: Since the Fast-L7A Format headers are 3-1536 byte subheaders, the
header buffers need to be able to read 3*HEADER_SIZE information.

Space Imaging Fast-C Format is almost the same as Fast-L7A.  To distinguish
the Space Imaging Fast-C Format from the EDC Fast-C Format (both have a 'C'
in byte 1536 of their header) we must look at byte 1521.  Byte 1521 for the
Space Imaging Fast-C is "REV", whereas for EDC Fast-C it is "OFFSET".

Space Imaging Fast-C Format for IRS 1C specifies the bands as 2,3,4,5,P,
where P represents the panchromatic band.
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tapeio.h"
#include "util.h"
#include "fhdr.h"
#include "fstfmtin.h"

void main () 
{
    struct PARBLK vblk;        /* TAE parameter block                       */
    FILE *infp = NULL;         /* input file, for cd and disk images only   */

    char bndlist[256];         /* merged list of bands                      */
    char ff_version[5];        /* Fast Format version A,B,EDC C,
                                  Space Imaging C, or L7A                   */
    char gains_biases[256][17];/* gains/biases for each band                */
    char hdrs[256][CMLEN];     /* array of header filenames w/o extension   */
    char hdr_extension[CMLEN]; /* extension for image header                */
    char hdrfile[CMLEN];       /* full name of the header to be read        */
    char header[3*HEADER_SIZE+2];/* buffer to hold header file              */
    char *ptr;                 /* temp pointer to point to chars in the
                                  filenames                                 */
    char tape_span[5];         /* tape spanning flag                        */
    char thisgainbias[17];     /* the current gain and bias                 */
    char tmp_str[HEADER_SIZE+1]; /* dummy string to obtain str value        */

    int curhdr;                /* current header to be processed            */
    int i, j;                  /* looping variables                         */
    int nbands;                /* number of bands in the merged array       */
    int ngainsbiases;          /* number of gains/biases in the merged array*/
    int nhdrs;                 /* number of headers to merge                */
    int thisnbands;            /* number of bands in the current header     */

    long bias_fields[] = {4, 9, 14, 19, 24, 29, 34, 39};
    /* bias fields for Fast-L7A and SIC headers  */
    long gain_fields[] = {6, 11, 16, 21, 26, 31, 36, 41};
    /* gain fields for Fast-L7A and SIC headers  */
    long gain_bias_fields[] = {21, 23, 25, 27, 29, 31, 33};
    /* gain and bias fields for Fast-BC headers  */
    long hfields[5];           /* fast format field information             */
    long hlen;                 /* length of header                          */
    long hsize;                /* size of the header read from the file     */
    long mode;                 /* file i/o mode                             */
    long num_tapes;            /* number of tapes for tape spanning field   */
    long std_out = 0;          /* standard out                              */
    long vol_in_set;           /* volume number in tape spanning field      */

    double biases[256];        /* array of biases for each band             */
    double dvalue;             /* temporary double value                    */
    double gains[256];         /* array of gains for each band              */

    c_inlas (&vblk, &std_out);

    /* Get the variables from the TAE variable block
       ------------------------------------------------*/
    mhbgetpar (&vblk, &nhdrs, hdrs, hdr_extension);

    /* If there is only one header then don't process it.  The bands are
       technically already merged.
       --------------------------------------------------------------------*/
    if (nhdrs <= 1)
        return;

    /* Change the LAS-based filenames to filenames that fopen will recognize.
       ---------------------------------------------------------------------*/
    for (curhdr = 0; curhdr < nhdrs; curhdr++)
    {
        /* Change the first '[' to '/'
           ------------------------------*/
        ptr = (char *) strchr (hdrs[curhdr], '[');
        if (ptr != NULL)
            ptr[0] = '/';

        /* Change the last ']' to '/'
           -----------------------------*/
        ptr = (char *) strrchr (hdrs[curhdr], ']');
        if (ptr != NULL)
            ptr[0] = '/';

        /* Loop through the filename changing all leftover '.'s to '/'s
           ---------------------------------------------------------------*/
        for (j = 0; j < strlen(hdrs[curhdr]); j++)
            if (hdrs[curhdr][j] == '.')
                hdrs[curhdr][j] = '/';

        /* Change the last ';' to '.'
           -----------------------------*/
        ptr = (char *) strrchr (hdrs[curhdr], ';');
        if (ptr != NULL)
            ptr[0] = '.';
    }

    /* Initialize the merged band list and the number of merged bands
       -----------------------------------------------------------------*/
    strcpy (bndlist, "");
    nbands = 0;
    ngainsbiases = 0;

    /* Loop through the headers, read the bands, and merge the bands
       ----------------------------------------------------------------*/
    for (curhdr = 0; curhdr < nhdrs; curhdr++)
    {
        /* The image is to be read from disk.  Read the header file and then
           close it.
           ----------------------------------------------------------------*/
        sprintf (hdrfile, "%s.%s", hdrs[curhdr], hdr_extension);
        infp = fopen (hdrfile, "r");
        if (infp == NULL)
        {
            c_lasmsg (FALSE, "fstfmtin-header", "Error reading the header: %s",
                      hdrfile);
            c_lasmsg (TRUE, "fstfmtin-fatal", "Fatal error encountered");
        }

        /* Open and try to read three 1536-byte records.  For Fast-ABC, we will
           only need to read 1536 bytes, but for Fast-L7A and Space Imaging
           Fast-C we need three 1536-byte structures.
           ------------------------------------------------------------------*/
        hlen = 3*HEADER_SIZE;
        hsize = (long) fread (header, sizeof (unsigned char), hlen, infp);
        if ((hsize != hlen) && (hsize != HEADER_SIZE))
        { /* accept headers for Fast-ABC or SIC/Fast-L7A */
            fclose (infp);
            c_lasmsg (TRUE, "fstfmtin-header",
                      "Error in size of the header: %s",hdrfile);
        }
        fclose (infp);

        /* Define the size of the header files according to which Fast-Format
           the file is.  Fast-L7A and Space Imaging Fast-C headers have
           three 1536-byte structures, and Fast-ABC headers are 1536-bytes.
           -----------------------------------------------------------------*/
        if (!strncmp (&header[1532], "L7A", 3))
        { /* Fast-L7A headers must be 3*HEADER_SIZE bytes long */
            if (hsize != 3*HEADER_SIZE)
            {
                c_lasmsg (TRUE, "fstfmtin-header",
                "Error reading the correct header size of a Fast-L7A header");
            }
        }
        else if (header[1535] == 'C' && !strncmp (&header[1520], "REV", 3))
        { /* SIC headers must be 3*HEADER_SIZE bytes long */
            if (hsize != 3*HEADER_SIZE)
            {
                c_lasmsg (TRUE, "fstfmtin-header",
                          "Error reading the correct header size of a Space "
                          "Imaging Fast-C header");
            }
        }
        else
        { /* Fast-ABC headers must be 3*HEADER_SIZE bytes long */
            if (hsize != HEADER_SIZE)
            {
                c_lasmsg (TRUE, "fstfmtin-header",
                "Error reading the correct header size of a Fast-ABC header");
            }
        }

        /* Retrieve the Fast Format version.  For version A there will be a
           space in this position and for version B, C, and L7A there will be a
           B, C, and L7A (respectively) in this position.  For Space Imaging
           Fast C there will also be a C, thus another check must be done on
           the header to verify which version of Fast-C this data is.
           -----------------------------------------------------------------*/
        if (!strncmp (&header[1532], "L7A", 3))
            strcpy (ff_version, "L7A");
        else if(header[1535] == 'A' || header[1535] == ' ')
            strcpy (ff_version, "A");
        else if (header[1535] == 'B')
            strcpy (ff_version, "B");
        else if (header[1535] == 'C')
        {
            /* There are two versions of Fast-C.  One is the EDC Fast-C which
             * is similar to Fast-A and Fast-B.  The other is Space Imaging
             * Fast-C. For Space Imaging Fast-C, byte 1521 will be "REV". Byte
             * 1521 for EDC Fast-C is "OFFSET".
             ---------------------------------------------------------------*/
            if (!strncmp (&header[1520], "REV", 3))
                strcpy (ff_version, "SIC");
            else
                strcpy (ff_version, "C");
        }
        else
        {
            c_lasmsg (TRUE, "fstfmtin-version",
                      "Fast format version \"%c\" %s",header[1535], "unknown");
        }

        /* Fast-L7A headers have the same bands listed on each volume
         * of the header.  Thus we do not need to merge all the headers
         * from each volume.  We only need to pick on header and use it.
         * Note: the tape spanning field will need to be reset to 1/1.
         ----------------------------------------------------------------*/
        if (!strcmp (ff_version, "SIC"))
        {
            /* Space Imaging Fast-C header.  Read the bands present.
               --------------------------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst (header, 106, tmp_str, ff_version))
            {
                c_lasmsg (TRUE, "fstfmtin-fhdrrd",
                          "Error reading bands present");
            }
            if (E_FAIL == c_fhdfld_fst (106, hfields, ff_version))
            {
                c_lasmsg (TRUE, "fstfmtin-fhdfld",
                          "Error reading bands present");
            }
            tmp_str[hfields[3]] = '\0';
        }
        else if (!strcmp (ff_version, "L7A"))
        {  /* don't process Fast-L7A bands */
            hfields[3] = 0;  /* don't loop through the bands */
        }
        else
        {
            /* Fast-ABC header.  Read the bands present.
               --------------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst (header, 95, tmp_str, ff_version))
            {
                c_lasmsg (TRUE, "fstfmtin-fhdrrd",
                          "Error reading bands present");
            }
            if (E_FAIL == c_fhdfld_fst (95, hfields, ff_version))
            {
                c_lasmsg (TRUE, "fstfmtin-fhdfld",
                          "Error reading bands present");
            }
            tmp_str[hfields[3]] = '\0';
        }

        /* Determine the number of bands and add them to the band list
           --------------------------------------------------------------*/
        thisnbands = 0;
        for (i = 0; i < hfields[3]; i++, nbands++, thisnbands++)
        {
            if (tmp_str[i] == ' ')
            {
                /* no band value -- set to end of string and break out */
                tmp_str[i] = '\0';
                break;
            }
        }
        strcat (bndlist, tmp_str);

        if (!strcmp (ff_version, "SIC"))
        { /* Space Imaging Fast-C */
            /* Read the gains and biases for each band in this header
               and place them in the merged array.
               ---------------------------------------------------------*/
            for (i = 0; i < thisnbands; i++, ngainsbiases++)
            {
                /* Read the bias for the current band and place it in the
                   merged biases array.
                   ---------------------------------------------------------*/
                if (E_FAIL == c_fhdrrd_fst (header, bias_fields[i],
                                            (char *)&dvalue, ff_version))
                {
                    c_lasmsg (TRUE, "fstfmtin-fhdrrd", "Error reading bias");
                }
                biases[ngainsbiases] = dvalue;

                /* Read the gain for the current band and place it in the
                   merged gains array.
                   ---------------------------------------------------------*/
                if (E_FAIL == c_fhdrrd_fst (header, gain_fields[i],
                                            (char *)&dvalue, ff_version))
                {
                    c_lasmsg (TRUE, "fstfmtin-fhdrrd", "Error reading gain");
                }
                gains[ngainsbiases] = dvalue;
            }
        }
        else if (strcmp (ff_version, "L7A"))
        {
            /* Fast-ABC header.  Read the gains and biases for each band in
               this header and place them in the merged array.
               -------------------------------------------------------------*/
            for (i = 0; i < thisnbands; i++, ngainsbiases++)
            {
                /* Initialize the current gains and biases string to NULL.
                   ----------------------------------------------------------*/
                strcpy (gains_biases[ngainsbiases], "");

                /* Read the bias for the current band and place it in the
                   merged biases array.
                   ---------------------------------------------------------*/
                if (E_FAIL == c_fhdrrd_fst (header, gain_bias_fields[i],
                                            thisgainbias, ff_version))
                {
                    c_lasmsg (TRUE, "fstfmtin-fhdrrd",
                              "Error reading gain/bias");
                }
                if (E_FAIL == c_fhdfld_fst (gain_bias_fields[i],
                                            hfields, ff_version))
                {
                    c_lasmsg (TRUE, "fstfmtin-fhdfld",
                              "Error reading gain/bias");
                }
                thisgainbias[hfields[3]] = '\0';
                strcpy (gains_biases[ngainsbiases], thisgainbias);
            }
        }
    }  /* Header loop */

    /* Update the header file for the merged bands, the tape spanning
       field, and the merged gains/biases.
       ----------------------------------------------------------------*/
    if (!strcmp (ff_version, "L7A") || !strcmp (ff_version, "SIC"))
    {
        /* Don't process the bands or the gains/biases for Fast-L7A.
           ------------------------------------------------------------*/
        if (!strcmp (ff_version, "SIC"))
        {
            /* Update the bands present with the merged bands list.
               -------------------------------------------------------*/
            if (c_fhdrup_fst (header, 106, bndlist, ff_version) == E_FAIL)
            {
                c_lasmsg (TRUE, "fstfmtin-fhdrup",
                          "Error updating bands present");
            }

            /* Update the gains and biases for all the ngainsbiases bands.
               --------------------------------------------------------------*/
            for (i = 0; i < ngainsbiases; i++)
            {
                if (c_fhdrup_fst (header, bias_fields[i], (char *)&biases[i],
                                  ff_version) == E_FAIL)
                {
                    c_lasmsg (TRUE, "fstfmtin-fhdrup", "Error updating bias");
                }

                if (c_fhdrup_fst (header, gain_fields[i], (char *)&gains[i],
                                  ff_version) == E_FAIL)
                {
                    c_lasmsg (TRUE, "fstfmtin-fhdrup", "Error updating gain");
                }
            }
        }

        /* Update the tape spanning field to 1/1.  It is stored as two longs.
           -----------------------------------------------------------------*/
        vol_in_set = 1;
        num_tapes = 1;
        if (c_fhdrup_fst (header, 79, (char *)&vol_in_set, ff_version)
            == E_FAIL)
        {
            c_lasmsg (TRUE, "fstfmtin-fhdrup",
                      "Error updating tape spanning field");
        }
        if (c_fhdrup_fst (header, 81, (char *)&num_tapes, ff_version)
            == E_FAIL)
        {
            c_lasmsg (TRUE, "fstfmtin-fhdrup",
                      "Error updating tape spanning field");
        }
    }
    else  /* Fast ABC */
    {
        /* Update the bands present with the merged bands list.
           -------------------------------------------------------*/
        if (c_fhdrup_fst (header, 95, bndlist, ff_version) == E_FAIL)
        {
            c_lasmsg (TRUE, "fstfmtin-fhdrup", "Error updating bands present");
        }

        /* Tape spanning field and radiometric gains/biases are not
           available in the Fast-A header, so don't update them.
           -----------------------------------------------------------*/
        if (strcmp (ff_version, "A"))
        {
            /* Update the tape spanning field to 1/1.  It is stored as a
               string.
               --------------------------------------------------------*/
            strcpy (tape_span, "1/1");
            if (c_fhdrup_fst (header, 35, tape_span, ff_version) == E_FAIL)
            {
                c_lasmsg (TRUE, "fstfmtin-fhdrup",
                          "Error updating tape spanning field");
            }

            /* Update the gains and biases for all the ngainsbiases bands.
               --------------------------------------------------------------*/
            for (i = 0; i < ngainsbiases; i++)
            {
                if (c_fhdrup_fst (header, gain_bias_fields[i], gains_biases[i],
                                  ff_version) == E_FAIL)
                {
                    c_lasmsg (TRUE, "fstfmtin-fhdrup",
                              "Error updating gains/biases");
                }
            }
        }
    }

    /* Write out the header to the first header file
       ------------------------------------------------*/
    mode = IWRITE;
    if (E_FAIL == c_open_header_fst (header, hdrs[0], hdr_extension, &mode,
                                     hlen))
    {
        c_lasmsg (TRUE, "fstfmtin-header", "Error writing the header");
    }
}

