/******************************************************************************

NAME: MOUNT_MEDIA

PURPOSE: Opens the CD or tape being read from to locate imagery data 
         and return a flag to indicate if a tape or CD was mounted.

RETURN VALUE:
void

PROJECT: LAS
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "fhdr.h"
#include "util.h"
#include "fstfmtin.h"

void main(void)
{
    struct PARBLK vblk;        /* TAE parameter block                        */
    struct VARIABLE *var;      /* pointer to a variable in the vblk          */
    struct TAPEDESC *drive;    /* tape descriptor structure                  */
   
    char density[CMLEN];       /* tape density returned from tapeio routines */
    char drivename[CMLEN];     /* string descriptor for CD-ROM drive         */
    char lhost[CMLEN];         /* local host name                            */
    char oprmsg[CMLEN];        /* message for the operator for CD mounting   */
    char srchname[CMLEN];      /* string descriptor for CD-ROM drive         */
    char tape_id[CMLEN];       /* tape id returned from tapeio routines      */

    int media_type;            /* type of media tape = 0, cd = 1             */
   
    size_t length;             /* string length                              */
    long one = 1;              /* constant one                               */
    long status;               /* return status                              */
    long std_out = 0;          /* standard out                               */
    long unit;                 /* tape unit                                  */
    long zero = 0;             /* constant zero                              */

    long option;               /* rewind option: no rewind = 0, rewind = 1   */ 

    c_inlas(&vblk, &std_out);
    
    /* Get the operator message from the TAE variable block
       --------------------------------------------------*/
    var = p_find(&vblk, "OPERATOR_MSG");
    strcpy(oprmsg, SVAL(*var, 0));
    
    /* Open the disk and get the drive name.  The oprmsg is the
       message returned from the operator.  This is the drive name.
       -----------------------------------------------------------*/
    strcpy(density,"0");

    /* Since we don't know if this is a CD or tape, set option to rewind.
    If it is a CD this option will be reset in c_tpopen to no rewind */
    option = REWIND;
    status = c_tpopen(&unit, &zero, &one, &option, density, &zero, oprmsg,
                      tape_id);
    if (status != TAPE_SUCC)
    {
       c_lasmsg(TRUE,"fstfmtin-tpopen", "Error opening the CD file");
    } 

    drive = (struct TAPEDESC *)unit;

    if (las_is_tape(drive) == TRUE)
        media_type = 0;
    else
        media_type = 1;

    if (SUCCESS != q_intg(&vblk,"MEDIAFLAG",1,(int *)&media_type,P_UPDATE))
    {
        c_lasmsg(TRUE,"fstfmtin-integer","Error passing MEDIAFLAG to TAE");
    }

    if (SUCCESS != q_intg(&vblk,"MOUNTED",1,(int *)&one,P_UPDATE))
    {
        c_lasmsg(TRUE,"fstfmtin-integer","Error passing MOUNTED to TAE");
    }
      
    /* Return the operator drive name
       ------------------------------*/
    length = strlen(oprmsg);
    if (E_SUCC != str2tae(&vblk,oprmsg,(long) length,"%s",1,"OPR_DRIVE"))
    {
        c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting OPR_DRIVE");
    }
      
    /* If the DRIVE doesn't have an @ sign in it then this is a local
       CD drive.  If the DRIVE does have an @ sign then this is a
       remote drive.  If this is a local DRIVE, then we need to get the
       hostname and compute the DRIVE from hostname + @ + DRIVE.
       If the DRIVE has !! as the first two characters, then this
       is a local drive and the user doesn't want to prefix it with the
       hostname.  We should strip off the first !!s and process this
       as the specified drive name.
       ------------------------------------------------------------------*/
    if (strchr(oprmsg,'@') == NULL)
    {  /* This is a local tape drive */
        if (gethostname(lhost,CMLEN) != 0)
        {
            c_lasmsg(TRUE,"fstfmtin-hostname",
                     "Error reading the hostname");
        }
        length = strlen(lhost);
        c_up2low(lhost,(long *) &length);
        sprintf (drivename, "/%s@%s/", lhost, oprmsg);
        sprintf (srchname, "[%s@%s]", lhost, oprmsg);
    }
    else if (strncmp (oprmsg, "!!", 2) == 0)
    {
        sprintf (drivename, "/%s/", &oprmsg[2]);
        sprintf (srchname, "[%s]", &oprmsg[2]);
    }
    else
    {
        /* This is a remote tape drive */
        sprintf (drivename, "/%s/", oprmsg);
        sprintf (srchname, "[%s]", oprmsg);
    }

    /* If a tape close it without rewinding it  
       --------------------------------------*/
    option = NOREW;
    status = c_tpclos(&unit, &one, &option, &zero);
    if (status != TAPE_SUCC)
    {
        c_lasmsg(TRUE,"fstfmtin-tape","Error closing the tape");
    }

    /* Return the full drive name
       -----------------------------*/
    length = strlen(drivename);
    if (E_SUCC != str2tae(&vblk,drivename,(long) length,"%s",1,"DRIVE"))
    {
        c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting DRIVE");
    }
      
    /* Return the TAE search drive name
       -----------------------------------*/
    length = strlen(srchname);
    if (E_SUCC != str2tae(&vblk,srchname,(long) length,"%s",1,
                          "SRCH_DRIVE"))
    {
        c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting SRCH_DRIVE");
    }
  
    /* Send variables back to TAE
       --------------------------*/
    if (q_out(&vblk) != SUCCESS)
    {
        c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
    }
}

