/******************************************************************************

NAME: process_hdr

PURPOSE: To read the fast format header and extract the geometric and 
         scene acquistion data.

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
  7.2    04/99  G.Schmidt        EDC   Updated to support Fast-L7A
  7.2    07/99  G.Schmidt        EDC   Updated to support Space Imaging Fast-C
  7.2    07/99  G.Schmidt        EDC   Acquisition date for Fast-L7A has
                                       changed from yyyyddmm to yyyymmdd
         10/01  A.Weissenberger  EDC   CR 5057 Formatting and removal of
                                       redundant c_lasmsg calls.
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

ALGORITHM DESCRIPTION:
   Read the projection information stored in the header.
   Call convert_dms to adjust the projection coefficients to the proper 
   DMS specification needed.
   Read in the projection coefficients.
   Get the lat/long for the corner points.
   Convert the lat/long pts into northings/eastings.
   Get the ground pixel size.
   Get the instrument and acquistion date.

NOTES:

******************************************************************************/
#include <string.h>
#include "fhdr2ddr.h"
#include "proj.h"
#include "geompak.h"
#include "util.h"
#include "fstfmtin.h"

static void get_datum (char *hdr, char *htype, long *datum);
/* Since get_datum is static and only used in process_hdr, this declaration
   is in this file instead of the header file. */


int process_hdr (
    char *hdr,           /* fast format header                               */
    char *htype,         /* header version, A, B, C, L7A, or
                            Space Imaging Fast-C                             */
    long *isize,         /* image size (lines, samples)                      */
    double *pixsize,     /* ground pixel size                                */
    double *projcoef,    /* GCTP projection parameters (15)                  */
    double *corners,     /* image corner points (8)                          */
    long *pcode,         /* GCTP projection code                             */
    long *zcode,         /* UTM or State Plane code                          */
    long *datum,         /* GCTP datum code                                  */
    char *source,        /* source of image                                  */
    char *inst,          /* instrument                                       */
    char *date,          /* date acquired                                    */
    int ddr_valid_flags[8])
/* validity flags for the ddr - set appropriate
                          flag to FALSE if an error is encountered
                          retrieving certain header information            */
{
    char *coefptrs[15];      /* pointers to projection coefficients        */
    char pname[360];         /* abreviated name of the projection          */
    char projstr[360];       /* temporary storage for proj. coef. string   */

    long hfields[5];         /* array containing information about a field in
                                the fast format header                     */
    double inparm[15];       /* input projection coefficients              */
    double latlong[8];       /* array for storing the lat/long for each corner
                                point                                      */

    long inproj;             /* input projection code                      */
    long inunit;             /* input projection units                     */
    long outunit;            /* output projection units                    */
    long prtprm[2];          /* report destination flags                    */
    long psize;              /* size of each proj. coef. value in characters */
    int ppfields[15] = {11, 13, 16, 18, 20, 23, 25, 27, 30, 32, 34, 37, 39,
                        41, 44};  /* field numbers of the proj parms */
    int retval = E_SUCC;     /* return status of the routine - only changed to
                                E_FAIL if the image size is in error since
                                fhdr2ddr uses that image size when
                                recalculating the corner coordinates when
                                subwindowing */

    /* misc variables */
    char *cptr = NULL;       /* working character pointer                   */
    char tmpstr[CMLEN];      /* temporary character string buffer           */
    char tmp_char1, tmp_char2; /* temporary char variables                  */
    long i;                  /* working index variable                      */
    long status;             /* function return status                      */

    /* get the projection name
       -------------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        if (E_FAIL == c_fhdfld_fst((long)(GEOM+3),hfields,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdfld",
                     "Error reading the projection name");
            ddr_valid_flags[0] = FALSE;
        }
        if ( c_fhdrrd_fst(hdr,(long)(GEOM+3),pname,htype) == E_FAIL )
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd",
                     "Error reading the projection name");
            if (ddr_valid_flags[0])
            {
                ddr_valid_flags[0] = FALSE;
            }
        }
        if (ddr_valid_flags[0])
            pname[hfields[3]] = 0;
    }
    else /* Fast A, B, C Formats */
    {
        if (E_FAIL == c_fhdfld_fst(43,hfields,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdfld",
                     "Error reading the projection name");
            ddr_valid_flags[0] = FALSE;
        }
        if ( c_fhdrrd_fst(hdr,43,pname,htype) == E_FAIL )
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd",
                     "Error reading the projection name");
            ddr_valid_flags[0] = FALSE;
        }
        if (ddr_valid_flags[0])
            pname[hfields[3]] = 0;
    }

    /* If there was an error retrieving the projection
       name no longer do anything with it
       --------------------------------------------------*/
    if (ddr_valid_flags[0])
    {
        i = 4;
        c_up2low(pname,&i);
        pname[4] = 0;

        /* get the projection code
           -------------------------*/
        if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
        {
            /* No projection code is available in the Fast-L7A or SIC formats
               so we must assign the proj code based on the proj name.
             */
            *pcode = (long) c_projstr2num (pname);
            if (*pcode == -1)
            {
                c_lasmsg(FALSE,"fstfmtin-pname","Invalid projection name.");
                c_lasmsg(FALSE,"fstfmtin-ddr","Unable to update DDR");
                ddr_valid_flags[0] = FALSE;
            }
        }
        else /* Fast A, B, C Format */
        {
            /*  UTM, GK, SNG are all special cases of TM.  The GCTP package
                keys off the TM values.  The projection number is the most
                reliable field to retrieve from the tape, however there
                isn't a projection number assigned to SNG.
                ------------------------------------------------------------*/
            if(strcmp(pname,"sng "))
            {
                if ( c_fhdrrd_fst(hdr,45,(char *)pcode,htype) == E_FAIL)
                {
                    c_lasmsg(FALSE,"fstfmtin-fhdrrd",
                             "Error reading the projection number");
                    ddr_valid_flags[0] = FALSE;
                }
    
                if (ddr_valid_flags[0] && (*pcode == ((long) HUGE_VAL)))
                {
                    c_lasmsg(FALSE,"fstfmtin-hfield",
                             "Missing projection number");
                    c_lasmsg(FALSE,"fstfmtin-ddr","Unable to update DDR");
                    ddr_valid_flags[0] = FALSE;
                }
    
                if(ddr_valid_flags[0] && (*pcode < 0 || *pcode > 21))
                {
                    c_lasmsg(FALSE,"fstfmtin-process",
                             "Map projection number %d unknown",*pcode);
                    ddr_valid_flags[0] = FALSE;
                }
    
                /* Change the projection number for the new GCTP.
                   SOM changed from 21 to 22.
                   ------------------------------------------------*/
                if (ddr_valid_flags[0] && (*pcode == 21))
                    *pcode = SOM;
            }
            else
                *pcode = TM;

            if (ddr_valid_flags[0] && (strcmp(pname,"stpl") == 0))
                strcpy(pname,"spcs");
        }
    }

    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        /* get information about the projection parameter field.
           we will assume all parameters have the same format so only read the
           format of the first parameter.
           ---------------------------------------------------------------*/
        if(E_FAIL == c_fhdfld_fst((long)(GEOM+11),hfields,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdfld",
                     "Error reading projection parameters");
            ddr_valid_flags[3] = FALSE;
        }

        /* read the parameters
           ----------------------*/
        if (ddr_valid_flags[3])
        {
            for(i=0;i<15;i++)
            {
                if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+ppfields[i]),
                                          (char *)&projcoef[i], htype))
                {
                    c_lasmsg(FALSE,"fstfmtin-fhdrrd",
                             "Error reading projection parameters");
                    ddr_valid_flags[3] = FALSE;
                }
            }
        }
    }
    else /* Fast-A,B,C Format */
    {
        /* ok, now we have to convert the projection coefficients to the form
           LAS uses but first, get information about the projection parameter
           field
           ---------------------------------------------------------------*/
        if(E_FAIL == c_fhdfld_fst(49,hfields,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdfld",
                     "Error reading projection parameters");
            ddr_valid_flags[3] = FALSE;
        }

        if (ddr_valid_flags[3])
        {
            /* get the starting byte of the projection coefficients
               ----------------------------------------------------*/
            cptr = hdr + hfields[0];

            /* save the original projection parameters for later (field size of
               24 times 15 parameters)
               ------------------------------------------------------------ */
            strncpy(projstr,cptr,360);

            /* get the character width of each projection coefficient
               ---------------------------------------------------------*/
            psize = hfields[3];

            for(i=0;i<15;i++)
            {
                coefptrs[i] = cptr;
                cptr += psize;
            }

            /* convert the projection coefs from the new GCTP format to the
               old format that LAS still uses.
               ----------------------------------------------------------- */
            convert_dms(coefptrs,*pcode,1);

            /* ok, now read 'em */
            if(E_FAIL == c_fhdrrd_fst(hdr,49,(char *)projcoef,htype))
            {
                c_lasmsg(FALSE,"fstfmtin-fhdrrd",
                         "Error reading projection parameters");
                ddr_valid_flags[3] = FALSE;
            }

            /* reset them to their original format
               (field size of 24 times 15 params)
               --------------------------------------*/
            if (ddr_valid_flags[3])
                strncpy(coefptrs[0],projstr,360);
        }
    }

    /* ok, now check to make sure that they are valid
       ----------------------------------------------- */
    if (ddr_valid_flags[3])
    {
        for(i=0;i<15;i++)
        {
            if(projcoef[i] == HUGE_VAL)
            {
                c_lasmsg(FALSE,"fstfmtin-hfield",
                         "Missing projection coefficients in Fast Format "
                         "header");
                c_lasmsg(FALSE,"fstfmtin-ddr","Unable to update DDR");      
                ddr_valid_flags[3] = FALSE;
            }
        }
    }

    /* get the zone code (the projection parameters are needed first) - don't
       call the routine if we could not successfully retrieve the projection
       name and/or code or if we could not successfully retrieve the projection
       coefficients and we have a Fast Format SIC image (since get_zonenum
       retrieves the zonenum from the projection parameters in this case).
       ---------------------------------------------------------------------*/
    if (ddr_valid_flags[0] && !(!ddr_valid_flags[3]
                                && (!strcmp (htype, "SIC"))))
        get_zonenum(hdr, htype, pname, *pcode, projcoef, zcode);
    else
        *zcode = 62;

    /* now read the corner points (degree lat/long )
       --------------------------------------------- */
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        /* UL longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+52),(char *)&latlong[0],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UL longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* UL latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+54),(char *)&latlong[1],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UL latitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* UR longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+63),(char *)&latlong[2],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UR longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* UR latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+65),(char *)&latlong[3],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UR latitude");
            ddr_valid_flags[6] = FALSE;
        }


        /* LL longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+85),(char *)&latlong[4],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LL longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* LL latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+87),(char *)&latlong[5],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LL latitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* LR longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+74),(char *)&latlong[6],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LR longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* LR latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,(long)(GEOM+76),(char *)&latlong[7],
                                  htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LR latitude");
            ddr_valid_flags[6] = FALSE;
        }
    }
    else /* Fast-A,B,C Format */
    {
        /* UL longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,63,(char *)&latlong[0],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UL longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* UL latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,65,(char *)&latlong[1],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UL latitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* UR longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,71,(char *)&latlong[2],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UR longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* UR latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,73,(char *)&latlong[3],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading UR latitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* LL longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,87,(char *)&latlong[4],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LL longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* LL latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,89,(char *)&latlong[5],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LL latitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* LR longitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,79,(char *)&latlong[6],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LR longitude");
            ddr_valid_flags[6] = FALSE;
        }

        /* LR latitude */
        if(E_FAIL == c_fhdrrd_fst(hdr,81,(char *)&latlong[7],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading LR latitude");
            ddr_valid_flags[6] = FALSE;
        }
    }

    /* check to make sure they are valid
       ---------------------------------*/
    if (ddr_valid_flags[6])
    {
        for(i=0;i<8;i++)
        {
            if(latlong[i] == HUGE_VAL)
            {
                c_lasmsg(FALSE,"fstfmtin-process",
                         "Invalid Latitude/Longitude in Fast Format header");
                ddr_valid_flags[6] = FALSE;
            }
        }
    }

    /* get the datum code (if it is invalid (99) then we cannot determine the
       corners)
       --------------------------------------------------------------------- */
    get_datum(hdr,htype,datum);

    /* Initialize variables for Geometric Registration
       (only if there is a valid datum)
       --------------------------------------------------*/
    if (*datum != 99)
    {
        inproj    = 0;             /* convert from geographic */
        inunit    = 4;             /* input is in degrees */
        outunit   = 2;             /* output is in degrees */
        prtprm[0] = 0;             /* print to terminal = FALSE           */
        prtprm[1] = 0;             /* print to file     = FALSE           */

        for (i = 0; i < 15; i++)
            inparm[i] = 0;

        /* Initialize the Geometric Registration routines
           -------------------------------------------------*/
        if (ddr_valid_flags[3])
        {
            status = c_transinit(&inproj, &inunit, zcode, datum, inparm, pcode,
                                 &outunit, zcode, datum, projcoef, prtprm,
                                 NULL);
            if (status != E_SUCC)
            {
                c_lasmsg(FALSE,"fstfmtin-transinit",
                         "Error initializing the geometric registration "
                         "routines");
                ddr_valid_flags[3] = FALSE;
            }
        }

        if (ddr_valid_flags[6] && ddr_valid_flags[0] && ddr_valid_flags[3])
        {
            /* upper left
               -------------*/
            if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
                                  &latlong[0], &latlong[1], &corners[0],
                                  &corners[1]))
            {
                c_lasmsg(FALSE,"fstfmtin-trans",
                         "Error converting the UL coordinates");
                ddr_valid_flags[6] = FALSE;
            }

            /* upper right
               --------------*/
            if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
                                  &latlong[2], &latlong[3], &corners[2],
                                  &corners[3]))
            {
                c_lasmsg(FALSE,"fstfmtin-trans",
                         "Error converting the UR coordinates");
                ddr_valid_flags[6] = FALSE;
            }

            /* lower left
               -------------*/
            if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
                                  &latlong[4], &latlong[5], &corners[4],
                                  &corners[5]))
            {
                c_lasmsg(FALSE,"fstfmtin-trans",
                         "Error converting the LL coordinates");
                ddr_valid_flags[6] = FALSE;
            }

            /* lower right
               --------------*/
            if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
                                  &latlong[6], &latlong[7], &corners[6],
                                  &corners[7]))
            {
                c_lasmsg(FALSE,"fstfmtin-trans",
                         "Error converting the LR coordinates");
                ddr_valid_flags[6] = FALSE;
            }
        }
    }

    /* get the original image size
       ----------------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,87,(char *)&isize[0],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the image size");
            retval = E_FAIL;
        }

        if(E_FAIL == c_fhdrrd_fst(hdr,83,(char *)&isize[1],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the image size");
            retval = E_FAIL;
        }
    }
    else /* Fast-A,B,C Format */
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,61,(char *)&isize[0],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the image size");
            retval = E_FAIL;
        }

        if(E_FAIL == c_fhdrrd_fst(hdr,59,(char *)&isize[1],htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the image size");
            retval = E_FAIL;
        }
    }

    if(isize[0] == ((long) HUGE_VAL) || isize[1] == ((long) HUGE_VAL) )
    {
        c_lasmsg(FALSE,"fstfmtin-process",
                 "Invalid image size in Fast Format header");
        retval = E_FAIL;
    }


    /* read the pixel size
       -------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,97,(char *)pixsize,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the pixel size");
            ddr_valid_flags[5] = FALSE;
        }
    }
    else /* Fast-A,B,C Format */
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,57,(char *)pixsize,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the pixel size");
            ddr_valid_flags[5] = FALSE;
        }
    }

    if(ddr_valid_flags[5] && (*pixsize == HUGE_VAL))
    {
        c_lasmsg(FALSE,"fstfmtin-process",
                 "Invalid pixel size in Fast Format header");
        ddr_valid_flags[5] = FALSE;
    }


    /* get the instrument (sensor)
       -------------------------------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,12,inst,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the instrument");
        }
    }
    else /* Fast-A,B,C Format */
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,10,inst,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the instrument");
        }
    }


    /* get the source (satellite)
       -----------------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,10,source,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the source");
        }
    }
    else /* Fast-A,B,C Format */
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,8,tmpstr,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd", "Error reading the source");
        }

        switch (tmpstr[1])
        {
            case '4':
                strcpy(source,"LANDSAT-4");
                break;
            case '5':
                strcpy(source,"LANDSAT-5");
                break;
            case '6':
                strcpy(source,"LANDSAT-6");
                break;
            default:
                strcpy(source,"unknown");
                break;
        }
    }

    /* now get the acquisition date
       ------------------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,6,date,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd",
                     "Error reading the acquisition date");
        }

        /* Fast-L7A format acquisition date is in yyyymmdd (version 3), but
           the Space Imaging Fast-C is still in yyyyddmm
           -----------------------------------------------------------------*/
        if (!strcmp (htype, "SIC"))
        {
            /* change the yyyyddmm format to yyyymmdd format for datecon
               ------------------------------------------------------------ */
            tmp_char1 = date[4];  /* first d */
            tmp_char2 = date[5];  /* second d */
            date[4] = date[6];    /* move first m to first d */
            date[5] = date[7];    /* move second m to second d */
            date[6] = tmp_char1;  /* move first d to first m */
            date[7] = tmp_char2;  /* move second d to second m */
        }
    }
    else /* Fast-A,B,C Format */
    {
        if(E_FAIL == c_fhdrrd_fst(hdr,6,date,htype))
        {
            c_lasmsg(FALSE,"fstfmtin-fhdrrd",
                     "Error reading the acquisition date");
        }
    }

    /* convert it to what LAS uses.
       ---------------------------*/
    datecon(date);

    return(retval);
}

/******************************************************************************

NAME: get_datum

PURPOSE: Given a fast format header, determine the projection datum.  The 
     fast format header documentation did not agree with the header
     files concerning the abreviations for the datum names.  Because of
     that, the datum is determined from the ellipsoid parameters.  If
     a match cannot be found, the datum is set to N/A (99) so that the DDR
     datum flag can be set to INVALID.

     Note - For Fast-L7A and SIC, we will read the datum from the header.

     Note - If the datum is Walbeck, then notify the user that Walbeck is no
     longer supported.  Set to N/A (99).

ALGORITHM DESCRIPTION:
    Read the ellipsoid parameters from the fast format header.  If an error
    occurs, set the datum to N/A (99), reset the header values and return.
    Compare the ellipsoid parameters from the header files with those that
    LAS uses.  If a match is found, set the datum to it's LAS ID and return.
    If a match cannot be found, set the datum to N/A(99), reset the header
    values and return.
    
NOTES: The geometric header is required for the Fast-L7A and SIC Formats.
******************************************************************************/

static void get_datum (
    char *hdr,           /* header (geometric header for Fast-L7A and SIC)  */
    char *htype,         /* header type (A, B, C, L7A, SIC)                 */
    long *datum)         /* datum value                                     */
{
    long datum_valid=0;    /* flag to specify if the datum is valid        */
    long hfields[5];       /* fast format field information                */
    double major;          /* semi-major axis of the ellipsoid from header */
    double minor;          /* semi-minor axis of the ellipsoid from header */
    long status;           /* function return status                       */
    char tmp_str[200];     /* for L7A and SIC the semi-major and semi-minor
                              axis are stored as ascii for reading the proj
                              parms thus they need to be read in as ascii
                              their format is 24.15                        */

    /* read the semi-major axis (stored as ASCII in the proj_coefs)
       ---------------------------------------------------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        status = c_fhdrrd_fst(hdr,(long)(GEOM+11),(char *)&major,htype);
        if(status == E_FAIL || major == HUGE_VAL)
            /* something went wrong, so set to 0.0
               --------------------------------------*/
            major = 0.0;

        status = c_fhdrrd_fst(hdr,(long)(GEOM+13),(char *)&minor,htype);
        if(status == E_FAIL || minor == HUGE_VAL)
            /* something went wrong, so set to 0.0
               --------------------------------------*/
            minor = 0.0;
    }
    else  /* Fast-ABC */
    {
        status = c_fhdrrd_fst(hdr,53,(char *)&major,htype);
        if(status == E_FAIL || major == HUGE_VAL)
            /* something went wrong, so set to 0.0
               --------------------------------------*/
            major = 0.0;

        status = c_fhdrrd_fst(hdr,55,(char *)&minor,htype);
        if(status == E_FAIL || minor == HUGE_VAL)
            /* something went wrong, so set to 0.0
               --------------------------------------*/
            minor = 0.0;
    }

    /* Read the datum (only for Fast-L7A and Space Imaging Fast-C)
       --------------------------------------------------------------*/
    if (!strcmp (htype, "L7A") || !strcmp (htype, "SIC"))
    {
        status = c_fhdrrd_fst(hdr,(long)(GEOM+5),tmp_str,htype);
        if(status == E_FAIL || !strcmp (tmp_str, ""))
        {
            /* If the datum cannot be read or the datum is not available, then
               try to calculate the datum using the semi-major and semi-minor
               axis. (If the semi-major or semi-minor axis are 0.0, then they
               are invalid.
               _________-----------------------------------------------------*/
            if (minor != 0.0 && major != 0.0)
            {
                if ((c_get_spheroid (major, minor, datum, &datum_valid)
                     != E_SUCC) || (datum_valid != 1))
                    *datum = 99;
                return;
            }
            else
            {
                *datum = 99;
                return;
            }
        }
        else
        {
            /* set up the datum code */
            if(E_FAIL == c_fhdfld_fst((long)(GEOM+5),hfields,htype))
            {
                c_lasmsg(FALSE,"fstfmtin-datum",
                         "Datum field is not available in the header");
                *datum = 99;
                return;
            }
            tmp_str[hfields[3]] = '\0';

            if (c_dtmnum (tmp_str, datum) != E_SUCC)
            {
                if (!strncmp (tmp_str, "WALBECK", 7))
                {
                    c_lasmsg(FALSE,"fstfmtin-datum",
                             "Walbeck ellipsoid is not supported");
                    *datum=99;
                    return;
                }
                else
                {
                    c_lasmsg(FALSE,"fstfmtin-datum","Unknown ellipsoid value");
                    *datum=99;
                    return;
                }
            }
        }
    }
    else
    { /* Fast-ABC */
        /* If the datum cannot be read or the datum is not available, then
           try to calculate the datum using the semi-major and semi-minor axis.
           (If the semi-major or semi-minor axis are 0.0, then they are
           invalid.
           --------------------------------------------------------------*/
        if (minor != 0.0 && major != 0.0)
        {
            if ((c_get_spheroid (major, minor, datum, &datum_valid) != E_SUCC)
                ||(datum_valid != 1))
                *datum = 99;
        }
        else
            *datum = 99;
    }
}
