/*****************************************************************************

NAME: READ_HEADER

PURPOSE: To read Fast-ABC, Fast-L7A, and Space Imaging Fast-C Format tape
     headers and retrieve the required information.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    1    04/90  T.Baltzer        EDC   Origination
    2    06/90  B.Davis          EDC   Created crn2mtr and dbl2tae.
                                       We now use the lat/longs to calculate 
                                       our own corner point values.
                                       Forced zone code to 62 when proj type is
                                       not TM or State Plane.
    3    07/91  T.Baltzer        EDC   Fix pipe size limit in EDITDDR call in
                                       fstfmtin.pdf.
                                       Clean-up error handling and messages in
                                       c routines.
    4    12/91  J.Fenno          EDC   Modified to read both versions of the
                                       EOSAT Fast Format tape headers.
                                       Modified so that only the parameters 
                                       that are required for reading the 
                                       image from tape are send back through 
                                       the TAE variable block.  All others 
                                       are written to a project parameter file.
    5   08/92   D.Etrheim        EDC   Make the ppf file name unique
    6   12/92   K.Sheffield      EDC   Create a .fhdr file for FHDR2DDR to 
                                       process instead of a ppf file.
                                       Removed the code to pass parameters 
                                       to TAE via a .ppf file.  FHDR2DDR 
                                       eliminates the need for this file and
                                       most of the processing done in 
                                       READ_HEADER.
    7    1/95   S.Nelson         EDC   Added version C.
    7.1  10/98  J.Janssen        EDC   Added capability to read header 
                                       from disk or CD.
    7.2  04/99  G.Schmidt        EDC   Updated to support Fast-L7A.
    7.2  07/99  G.Schmidt        EDC   Updated to support Space Imaging Fast-C
    7.3  09/00  A.Shimitz        EDC   Removed section that mounted the CD
                                       because locate_cd_hdr now does it
                                       (CR #3519)
    7.3  08/01  A.Weissenberger  EDC   Modified to be case insensitive when
                                       searching for files
                                       Removed redundant calls to c_lasmsg
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES: Since the Fast-L7A Format headers are 3-1536 byte subheaders, the
header buffers need to be able to read 3*HEADER_SIZE information.

Space Imaging Fast-C Format is almost the same as Fast-L7A.  To distinguish
the Space Imaging Fast-C Format from the EDC Fast-C Format (both have a 'C'
in byte 1536 of their header) we must look at byte 1521.  Byte 1521 for the
Space Imaging Fast-C is "REV", whereas for EDC Fast-C it is "OFFSET".

Space Imaging Fast-C Format for IRS 1C specifies the bands as 2,3,4,5,P,
where P represents the panchromatic band.

The output bits per pixel (input data type) is available for Fast-L7A and
Space Imaging Fast-C.  In the case of 32 bits per pixel, this can be either
I*4 or R*4.  We will represent it as I*4.  The idtype for Fast-ABC is set
to BYTE.

The bands from the headers are sorted before being passed back.  Since some
formats have filenames that are not linked to their band number, we also
need to pass back the original order of the bands so that we can correspond
them to the filenames.

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tapeio.h"
#include "util.h"
#include "fhdr.h"
#include "fstfmtin.h"

#define TAPEMEDIA 0
#define CDMEDIA   1
#define DISKMEDIA 2

static void get_suffix (char *file, char *file_ext, char *basename_suffix);

void main () 
{
    struct PARBLK vblk;        /* TAE parameter block                      */
    static long one  = 1;      /* constant                                 */
    FILE *infp = NULL;         /* input file, for cd and disk images only  */
   
    char cdpath[CMLEN];        /* path to the header file on CD            */
    char density[CMLEN];       /* tape density returned from tapeio routines*/
    char drive[CMLEN];         /* string descriptor for tape drive          */
    char opr_drive[CMLEN];     /* string descriptor for tape drive          */
    char dummy[3*HEADER_SIZE+1]; /* dummy buffer                            */
    char ff_version[5];        /* Fast Format version A,B,EDC C,
                                  Space Imaging C, or L7A                   */
    char hdrfile[CMLEN];       /* name of header file (on disk)             */
    char header[3*HEADER_SIZE+1];/* buffer to hold header file              */
    char hpath[CMLEN];         /* host path to image data on CD or disk     */
    char basename[CMLEN];      /* basename of the image data on CD          */
    char oprmsg[CMLEN];        /* operator message                          */
    char outfhdr[CMLEN];       /* fhdr file name                            */
    char tape_id[CMLEN];       /* tape id returned from tapeio routines     */
    char tmp_str[HEADER_SIZE+1]; /* dummy string to obtain str value        */
    char hdr_extension[CMLEN]; /* extension for image header                */
    char tmp_str1[10];         /* temporary string values                   */
    char temp;                 /* used for the bubble sort                  */
    char idtype[5];            /* input data type (BYTE, I*2, I*4, R*4)     */
    /* set to BYTE for Fast-ABC                  */
   
    int i,j;                   /* looping variable                          */
    int nbands;                /* number of bands in the array              */
    int swapped;               /* used for the bubble sort                  */
   
    long bytes_xfr;            /* bytes transferred                         */
    long mediaflag;            /* 0 = tape, 1 = cd, 2 = disk                */
    long hfields[5];           /* fast format field information             */
    long hlen;                 /* length of header                          */
    long hsize;                /* size of the header read from the file     */
    long length;               /* string length                             */
    long lvalue;               /* long value retrieved from the header      */
    long mode;                 /* file i/o mode                             */
    long mounted;              /* tape mounted flag                         */
    long norewind   = 0;       /* no rewind flag                            */
    long status;               /* return status                             */
    long std_out = 0;          /* standard out                              */
    long tmp_long;             /* dummy long to obtain lng value            */
    long unit;                 /* tape unit                                 */
    long zero = 0;             /* constant                                  */
    long resolution = 0;       /* L7 resolution type (VISIBLE, THERMAL, PAN)
                                  not used for Fast-ABC or SIC              */
    long same_res;             /* 1=bands are of same resolution, 0=not same
                                  not used for Fast-ABC or SIC              */
    long found = 0;            /* flag to specify if the header file was
                                  found */
    char found_files[MAXIMG][CMLEN];
    long num_hdrs;
    char basename_suffix[CMLEN];  /* String of basename suffix  i.e. '_B'   */
    char file_ext[CMLEN];         /* String of the file_extension, ie '.fst'*/
   
    c_inlas (&vblk, &std_out);
   
    /* Get the variables from the TAE variable block
       ------------------------------------------------*/
    rhgetpar (&vblk, oprmsg, drive, opr_drive, &mounted, outfhdr, &mediaflag,
              hpath, &resolution, &same_res, cdpath);
   
    /* If image is to be read from CD or disk, prepare the header file 
       for reading, read the header file, and close the file.
       ------------------------------------------------------------------*/
    if (mediaflag == CDMEDIA || mediaflag == DISKMEDIA)
    {
        if (mediaflag == CDMEDIA)
        {
            /* No longer responsible for mounting the CD since locate_cd_hdr
               does it now.
               Copy the hpath to basename so that we can use it later.
               -------------------------------------------------------------*/
            strcpy (basename, hpath);
         
            /* Reopen the disk and get the drive name.  The oprmsg is the
             * message returned from the operator.  This is the drive name.
             ---------------------------------------------------------------*/
            strcpy(density,"0");
            status = c_tpreop(&unit, &zero, &one, &zero, density, &zero,
                              opr_drive);
            if (status != TAPE_SUCC)
            {
                c_tpclos(&unit, &one, &norewind, &zero);
                c_lasmsg(TRUE,"fstfmtin-tpreop",
                         "Error reopening the CD file");
            }
         
            /* Use drive as the hpath if cd.
               ------------------------------------*/
            sprintf (hpath, "%s%s%s", drive, cdpath, basename);
        }  /* if mediaflag == CDMEDIA */
      
        /* Look for the Fast-A,B,C or SIC header file first.  (header.dat)
           When reading from a CD, _header.dat is appended to the input
           filename.
           ---------------------------------------------------------------*/
        strcpy(hdrfile,hpath);
        strcat(hdrfile,"_[Hh][Ee][Aa][Dd][Ee][Rr].[Dd][Aa][Tt]");
        if (c_getfiles (hdrfile, found_files, &num_hdrs, MAXIMG, 1, 0)
            != E_SUCC)
        {
            c_lasmsg(TRUE, "fstfmtin-c_getfiles","Error searching for "
                     ".dat header file.");
        }   
        if (num_hdrs > 0)
        {
            /* read in the first filename from c_getfiles */
            infp = fopen(found_files[0], "rt");
         
            /* Look for a band file */
            strcpy (hdrfile, hpath);
            strcat (hdrfile, "_[Bb][Aa][Nn][Dd]*.[Dd][Aa][Tt]");
            if (c_getfiles (hdrfile, found_files, &num_hdrs, MAXIMG, 1, 0)
                != E_SUCC)
            {
                c_lasmsg(TRUE, "fstfmtin-c_getfiles", "Error searching for "
                         ".dat band file.");
            }     
            if (num_hdrs > 0)
            {
                get_suffix (found_files [0], file_ext, basename_suffix);
            
                /* Copy the basename_suffix and file_extension to TAE */
                if (E_SUCC != str2tae (&vblk, basename_suffix,
                                       (long) strlen (basename_suffix),
                                       "%s", 1, "BASENAME_SUFFIX"))
                {
                    c_lasmsg (TRUE, "fstfmtin-str2tae",
                              "Error outputting BASENAME_SUFFIX");
                }
            
                if (E_SUCC != str2tae (&vblk, file_ext,
                                       (long) strlen (file_ext),
                                       "%s", 1, "FILE_EXTENSION"))
                {
                    c_lasmsg (TRUE, "fstfmtin-str2tae",
                              "Error outputting FILE_EXTENSION");
                } 
            }
            else
            {
                c_lasmsg(TRUE, "fstfmtin-c_getfiles", "Error searching for "
                         ".dat band file.");
            } 
        }
        else 
        {
            /* If not a Fast-A,B,C or Space Imaging Fast-C header file then
               check for the Fast-L7A header file.  (_hrf, _htm, _hpn)
               ------------------------------------------------------------*/
            strcpy (hdrfile, hpath);
            switch (resolution)
            {
                case (VISIBLE):
                    strcat (hdrfile, "_[Hh][Rr][Ff].[Ff][Ss][Tt]");
                    break;
                case (THERMAL):
                    strcat (hdrfile, "_[Hh][Tt][Mm].[Ff][Ss][Tt]");
                    break;
                case (PAN):
                    strcat (hdrfile, "_[Hh][Pp][Nn].[Ff][Ss][Tt]");
                    break;
                default:
                    c_lasmsg(TRUE, "fstfmtin-resolution",
                             "Invalid resolution entered");
                    break;
            }
         
            if (c_getfiles( hdrfile, found_files, &num_hdrs, MAXIMG, 1, 0)
                != E_SUCC)
            {
                c_lasmsg(TRUE, "fstfmtin-c_getfiles", "Error searching for "
                         ".fst header file.");
            }
            if (num_hdrs > 0)
            {
                /* read in the first filename returned from c_getfiles */
                infp = fopen(found_files[0], "rt");
            
                /* get the file extension and the band suffix */
                /* Look for a band file */
                strcpy (hdrfile, hpath);
                strcat (hdrfile, "_[Bb]*.[Ff][Ss][Tt]");
                if (c_getfiles (hdrfile, found_files, &num_hdrs, MAXIMG, 1, 0)
                    != E_SUCC)
                {
                    c_lasmsg(TRUE, "fstfmtin-c_getfiles", "Error searching "
                             "for .fst band file.");
                }
            
                /* If a band file was found */
                if (num_hdrs > 0)
                {
                    get_suffix (found_files [0], file_ext, basename_suffix);
               
                    /* Copy the basename_suffix and file_extension to TAE */
                    if (E_SUCC != str2tae (&vblk, basename_suffix,
                                           (long) strlen (basename_suffix),
                                           "%s", 1, "BASENAME_SUFFIX"))
                    {
                        c_lasmsg (TRUE, "fstfmtin-str2tae",
                                  "Error outputting BASENAME_SUFFIX");
                    }
               
                    if (E_SUCC != str2tae (&vblk, file_ext,
                                           (long) strlen (file_ext),
                                           "%s", 1, "FILE_EXTENSION"))
                    {
                        c_lasmsg (TRUE, "fstfmtin-str2tae",
                                  "Error outputting FILE_EXTENSION");
                    }
                }
               
                else
                {
                    c_lasmsg (TRUE, "fstfmtin-c_getfiles", "Error searching "
                              "for .fst band file.");
                    
                }
            }
        }
        if (infp != NULL)
        {
            /* A header file was found, now we can read it.
               -----------------------------------------------*/
            found = 1;
            if (E_SUCC != lng2tae(&vblk,&found,1,1,"FOUND",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae","Error outputting FOUND");   
            }
         
            /* Open and try to read three 1536-byte records.  For Fast-ABC,
               we will only need to read 1536 bytes, but for Fast-L7A and
               Space Imaging Fast-C we need three 1536-byte structures.
               ------------------------------------------------------------*/
            hlen = 3*HEADER_SIZE;
            hsize = (long) fread (header, sizeof (unsigned char), hlen, infp);
            if ((hsize != hlen) && (hsize != HEADER_SIZE))
            { /* accept headers for Fast-ABC or SIC/Fast-L7A */
                c_lasmsg(TRUE,"fstfmtin-header","Error reading the header");
            }
            fclose(infp);
         
            /* Define the size of the header files according to which
               Fast-Format the file is.  Fast-L7A and Space Imaging Fast-C
               headers have three 1536-byte structures, and Fast-ABC headers
               are 1536-bytes.
               -------------------------------------------------------------*/
            if (!strncmp (&header[1532], "L7A", 3))
            { /* Fast-L7A headers must be 3*HEADER_SIZE bytes long */
                if (hsize != 3*HEADER_SIZE)
                {
                    c_lasmsg(TRUE,"fstfmtin-header", "Error reading the "
                             "correct header size of a Fast-L7A header");
                }
            }
            else if (header[1535] == 'C' && !strncmp (&header[1520], "REV", 3))
            { /* SIC headers must be 3*HEADER_SIZE bytes long */
                if (hsize != 3*HEADER_SIZE)
                {
                    c_lasmsg(TRUE,"fstfmtin-header", "Error reading the "
                             "correct header size of a Space Imaging Fast-C "
                             "header");
                }
            }
            else
            { /* Fast-ABC headers must be 3*HEADER_SIZE bytes long */
                if (hsize != HEADER_SIZE)
                {
                    c_lasmsg(TRUE,"fstfmtin-header", "Error reading the "
                             "correct header size of a Fast-ABC header");
                }
            }
         
            /* Close the CD.
               ----------------*/
            if (mediaflag == CDMEDIA)
            {
                status = c_tpclos(&unit, &one, &norewind, &zero);
                if (status != TAPE_SUCC)
                {
                    c_lasmsg(TRUE,"fstfmtin-tpclos","Error closing the CD");
                }
            }
        }
        else
        {
            found = 0;

            /* If a header file was not found on disk issue an error
               ----------------------------------------------------*/
            if (mediaflag == DISKMEDIA)
            {
                c_lasmsg(FALSE,"fstfmtin-infile","Error locating a header "
                          "file for %s" ,hpath);
                c_lasmsg(TRUE, "fstfmtin-fatal", "Verify a correct and complete"
                         " host path and basename was entered in INFILE");
            }


            /* Immediately send found back to TAE and exit.
               -----------------------------------------------*/
            if (E_SUCC != lng2tae(&vblk,&found,1,1,"FOUND",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae","Error outputting FOUND");
            }
         
            if (q_out(&vblk) != SUCCESS)
            {
                c_lasmsg(TRUE,"fstfmtin-qout",
                         "Error passing variables to TAE");  
            }
        }
    }
    else
    {
        /* Open or reopen the tape and read the tape header file into a buffer
           Fast-L7A and SIC formats are not supported on tape.
           -----------------------------------------------------------------*/
        hlen = HEADER_SIZE;
        strcpy(density,"0");
        if (mounted == FALSE)
        {
            status = c_tpopen(&unit, &zero, &one, &one, density, &zero, oprmsg,
                              tape_id);
            if (status != TAPE_SUCC)
            {
                c_lasmsg(TRUE,"fstfmtin-tpopen", "Error opening the tape"); 
            }
         
            if (SUCCESS != q_intg(&vblk,"MOUNTED",1,(int *)&one,P_UPDATE))
            {
                c_lasmsg(TRUE,"fstfmtin-integer",
                         "Error passing MOUNTED to TAE");
            }
         
            /* Return the operator drive name
               ---------------------------------*/
            length = (long) strlen(oprmsg);
            if (E_SUCC != str2tae(&vblk,oprmsg,length,"%s",1,"OPR_DRIVE"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae",
                         "Error outputting OPR_DRIVE");
            }
         
            /* Return the drive name (same as the operator drive name).
               -----------------------------------------------------------*/
            length = (long) strlen(oprmsg);
            if (E_SUCC != str2tae(&vblk,oprmsg,length,"%s",1,"DRIVE"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae", "Error outputting DRIVE");
            }
        }
        else
        {
            status = c_tpreop(&unit, &zero, &one, &zero, density, &zero,
                              opr_drive);

            if (status != TAPE_SUCC)
            {
                c_tpclos(&unit, &one, &norewind, &zero);
                c_lasmsg(TRUE,"fstfmtin-tpclos", "Error closing the tape"); 
            }

            /* Return the DRIVE parameter as the opr_drive instead of drive
               ----------------------------------------------------------*/
            length = (long) strlen(opr_drive);
            if (E_SUCC != str2tae(&vblk,opr_drive,length,"%s",1,"DRIVE"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae", "Error outputting DRIVE");
            }
        }

        /* Immediately send the drive name back to TAE.  This way
           if we error out it will know which drive to dismount.
           ----------------------------------------------------*/
        if (q_out(&vblk) != SUCCESS)
        { 
            c_lasmsg(TRUE,"fstfmtin-qout", "Error passing variables to TAE");
        }
      
        bytes_xfr = c_tpread(&unit, (unsigned char *)header, &hlen, &one);
        if (bytes_xfr != hlen)
        {
            c_tpclos(&unit, &one, &norewind, &zero);
            c_lasmsg(TRUE,"fstfmtin-header",
                     "Header file does not contain 1,536 bytes");
        }
      
        found = 1;
        if (E_SUCC != lng2tae(&vblk,&found,1,1,"FOUND",&tmp_long))
        {
            c_lasmsg(TRUE,"fstfmtin-lng2tae","Error outputting FOUND");
        }
      
        /* Read past the end of file marker and close.
           WARNING: This may error out if the buffer is smaller than the tape
           block size. GLS  (Can't test this until we have tapes to test with.)
           -----------------------------------------------------------------*/
        bytes_xfr = c_tpread(&unit, (unsigned char *)dummy, &hlen, &one);
        if (bytes_xfr != TAPE_EOF)
        {
            c_tpclos(&unit, &one, &norewind, &zero);
            c_lasmsg(TRUE,"fstfmtin-header",
                     "Header file does not contain 1,536 bytes");
        }
        status = c_tpclos(&unit, &one, &norewind, &zero);
        if (status != TAPE_SUCC)
        {
            c_lasmsg(TRUE,"fstfmtin-tape","Error closing the tape");
        }
    }
   
    /* Read the tape/CD if we found the header.
       ---------------------------------------------------------*/
    if (found)
    {
        /* Retrieve the Fast Format version.  For version A there will be a
           space in this position and for version B, C, and L7A there will be
           a B, C, and L7A (respectively) in this position.  For Space Imaging
           Fast C there will also be a C, thus another check must be done on
           the header to verify which version of Fast-C this data is.
           -----------------------------------------------------------------*/
        strcpy (hdr_extension, "fhdr");
        if (!strncmp (&header[1532], "L7A", 3))
        {
            strcpy (ff_version, "L7A");
            strcpy (hdr_extension, "fst");
        }
        else if(header[1535] == 'A' || header[1535] == ' ')
            strcpy (ff_version, "A");
        else if (header[1535] == 'B')
            strcpy (ff_version, "B");
        else if (header[1535] == 'C')
        {
            /* There are two versions of Fast-C.  One is the EDC Fast-C which
               is similar to Fast-A and Fast-B.  The other is Space Imaging
               Fast-C. For Space Imaging Fast-C, byte 1521 will be "REV".
               Byte 1521 for EDC Fast-C is "OFFSET".
               ---------------------------------------------------------------*/
            if (!strncmp (&header[1520], "REV", 3))
                strcpy (ff_version, "SIC");
            else
                strcpy (ff_version, "C");
        }
        else
        {
            c_lasmsg(TRUE,"fstfmtin-version","Fast format version \"%c\" %s",
                     header[1535],"unknown");
        }
      
        /* Write out the header to a temporary header file
           For Fast-L7A each different header for each resolution may
           have an _vis, _thm, or _pan on it.
           -------------------------------------------------------------*/
        mode = IWRITE;
        if (!same_res)
        {
            if (!strcmp (ff_version, "L7A"))
            {
                strcpy (tmp_str, outfhdr);
                switch (resolution)
                {
                    case 30:
                        sprintf (outfhdr, "%s_vis", tmp_str);
                        break;
                    case 60:
                        sprintf (outfhdr, "%s_thm", tmp_str);
                        break;
                    case 15:
                        sprintf (outfhdr, "%s_pan", tmp_str);
                        break;
                }
            }
        }
      
        if(E_FAIL == c_open_header_fst(header, outfhdr, hdr_extension, &mode,
                                       hlen))
        {
            c_lasmsg(TRUE,"fstfmtin-header", "Error opening the header");
        }
      
        /* If ff_version is A then convert the header to B.
           --------------------------------------------------- */
        if (!strcmp (ff_version, "A"))
        {
            strncpy(tmp_str,header,hlen);
            c_convertab(tmp_str,header);
        }
      
        /* Output the FF_VERSION
           ------------------------ */
        strcpy (tmp_str, ff_version);
        if (E_SUCC != str2tae(&vblk,tmp_str,5,"%5s",1,"FF_VERSION"))
        {
            c_lasmsg(TRUE,"fstfmtin-str2tae", "Error outputting FF_VERSION");
        }
      
        /* Fast-L7A and Space Imaging Fast-C can be read the same way.
           -------------------------------------------------------------- */
        if (!strcmp (ff_version, "L7A") || !strcmp (ff_version, "SIC"))
        {
            /* get the tape spanning field
               ---------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,79,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading tape spanning field");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"VOL_IN_SET",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting VOL_IN_SET");
            }
         
            if (E_FAIL == c_fhdrrd_fst(header,81,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading tape spanning field");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"NUM_CDS",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting NUM_CDS");
            }
         
            /* get the starting line number on this tape
               ------------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,91,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading the starting line");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"STARTING_LINE",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting STARTING_LINE");
            }
         
            /* get the number of lines in this volume
               --------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,85,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading number of lines in this volume");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"NUMBER_LINES",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting NUMBER_LINES");
            }
         
            /* get the total number of samples per line
               ----------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,83,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading total number of samples per line");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"TOTAL_SAMPLES",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting TOTAL_SAMPLES");
            }
         
            /* get the total number of lines per image
               ----------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,87,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading total number of lines per image");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"TOTAL_LINES",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting TOTAL_LINES");
            }
         
            /* get the output bits per pixel for the input data type
               (this is only valid in Fast-L7A and SIC)
               I*4 and R*4 are both a value of 32.  We will use I*4.
               --------------------------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,100,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading output bits per pixel");
            }
            switch (lvalue)
            {
                case 8:
                    strcpy (idtype, "BYTE");
                    break;
                case 16:
                    strcpy (idtype, "I*2");
                    break;
                case 32:
                    strcpy (idtype, "I*4");
                    break;
            }
            if (E_SUCC != str2tae(&vblk,idtype,sizeof (idtype),"%s",1,
                                  "IDTYPE"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting IDTYPE");
            }
         
            /* get the bands present on the tape
               ----------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,106,tmp_str,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd","Error reading bands present");
            }
            if (E_FAIL == c_fhdfld_fst(106,hfields,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdfld","Error reading bands present");
            }
            tmp_str[hfields[3]] = 0;
         
            if (!strcmp (ff_version, "L7A"))
            {
                /* convert the L7A bands (1, 2, 3, 4, 5, 6L, 7, 8, 6H) to LAS
                   bands (1, 2, 3, 4, 5, 6, 7, 8, 9) respectively
                   ------------------------------------------- */
                nbands = 0;
                for (i = 0; i < hfields[3]; i++)
                {
                    switch (tmp_str[i])
                    {
                        case ('1'):
                        case ('2'):
                        case ('3'):
                        case ('4'):
                        case ('5'):
                        case ('7'):
                        case ('8'):
                            /* remain the same */
                            break;
                        case ('L'):
                            /* band 6L */
                            tmp_str[i] = '6';
                            break;
                        case ('H'):
                            /* band 6H */
                            tmp_str[i] = '9';
                            break;
                        case (' '):
                            /* no band value -- set to end of string */
                            tmp_str[i] = '\0';
                            break;
                        default:
                            c_lasmsg (FALSE,"fstfmtin-warning",
                                      "Illegal band found for Fast-L7A");
                            break;
                    }
               
                    /* break out after the first ' ' because the bands are done
                       -----------------------------------------------------*/
                    if (tmp_str[i] == '\0') 
                        break;
                }
                nbands = i;
            }
            else
            {
                /* Processing Space Imaging Fast-C.
                   -----------------------------------*/
                nbands = 0;
                for (i = 0; i < hfields[3]; i++)
                {
                    if (tmp_str[i] == ' ')
                    {
                        /* no band value -- set to end of string and break
                           out */
                        tmp_str[i] = '\0';
                        break;
                    }
                }
                nbands = i;
            }
         
            /* Pass the current band list back before it is sorted
               ------------------------------------------------------*/
            if (E_SUCC != str2tae(&vblk,tmp_str,1,"%1c",nbands,"BANDS_ORIG"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae",
                         "Error outputting BANDS_ORIG");
            }
         
            /* Sort the bands_avail array (only for the nbands available)
               -------------------------------------------------------------*/
            swapped = TRUE;
            for (i = 0; (i < nbands-1) && swapped; i++)
            {
                swapped = FALSE;
                for (j = nbands-1; i < j; j--)
                {
                    if (tmp_str[j] < tmp_str[j-1])
                    {
                        temp = tmp_str[j];
                        tmp_str[j] = tmp_str[j-1];
                        tmp_str[j-1] = temp;
                        swapped = TRUE;
                    }
                }
            }
         
            if (E_SUCC != str2tae(&vblk,tmp_str,1,"%1c",nbands,
                                  "BANDS_PRESENT"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae",
                         "Error outputting BANDS_PRESENT");
            }
         
            /* get the tape blocking factor
               NOTE: Tapes are not supported with Fast-L7A or SIC so the
               BLOCKING_FACTOR is just set to 1. -gs
               ---------------------------------------------------------*/
            /*  if (E_FAIL == c_fhdrrd_fst(header,93,(char *)&lvalue,
                ff_version))
                {
                c_lasmsg(TRUE,"fstfmtin-fhddrd",
                "Error reading tape blocking factor");
                } */
            lvalue = 1;  /* no blocking factor is available for Fast-L7A or
                            SIC*/
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"BLOCK_FACTOR",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting BLOCK_FACTOR");
            }
        }
        else
        {
            /* Retrieve the fields that are independent between the B and C
               versions
               ------------------------------------------------------------*/
            if (!strcmp (ff_version, "B") || !strcmp (ff_version, "C"))
            {
                /* get the tape spanning field  (n/m)
                   -------------------------------------*/
                if (E_FAIL == c_fhdrrd_fst(header,35,tmp_str,ff_version))
                {
                    c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                             "Error reading tape spanning field");
                }
                if (E_FAIL == c_fhdfld_fst(35,hfields,ff_version))
                {
                    c_lasmsg(TRUE,"fstfmtin-fhdfld",
                             "Error reading tape spanning field");
                }
                tmp_str[hfields[3]] = 0;
            
                strncpy (tmp_str1, tmp_str, 1);
                lvalue = atoi (tmp_str1);
                if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"VOL_IN_SET",
                                      &tmp_long))
                {
                    c_lasmsg(TRUE,"fstfmtin-lng2tae",
                             "Error outputting VOL_IN_SET");
                }
                strncpy (tmp_str1, &tmp_str[2], 1);
                lvalue = atoi (tmp_str1);
                if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"NUM_CDS",&tmp_long))
                {
                    c_lasmsg(TRUE,"fstfmtin-lng2tae",
                             "Error outputting NUM_CDS");
                }
            
                /* get the starting line number on this tape
                   ------------------------------------------*/
                if (E_FAIL == c_fhdrrd_fst(header,37,(char *)&lvalue,
                                           ff_version))
                {
                    c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                             "Error reading the starting line");
                }
                if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"STARTING_LINE",
                                      &tmp_long))
                {
                    c_lasmsg(TRUE,"fstfmtin-lng2tae",
                             "Error outputting STARTING_LINE");
                }
            
                /* get the number of lines in this volume
                   --------------------------------------*/
                if (E_FAIL == c_fhdrrd_fst(header,39,(char *)&lvalue,
                                           ff_version))
                {
                    c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                             "Error reading number of lines in this volume");
                }
                if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"NUMBER_LINES",
                                      &tmp_long))
                {
                    c_lasmsg(TRUE,"fstfmtin-lng2tae",
                             "Error outputting NUMBER_LINES");
                }
            }
         
            /* get the total number of samples per line
               ----------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,59,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading total number of samples per line");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"TOTAL_SAMPLES",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting TOTAL_SAMPLES");
            }
         
            /* get the total number of lines per image
               ----------------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,61,(char *)&lvalue,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd",
                         "Error reading total number of lines per image");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"TOTAL_LINES",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting TOTAL_LINES");
            }
         
            /* default the output bits per pixel (input data type) to BYTE
               since it is not in the header
               --------------------------------------------------------------*/
            strcpy (idtype, "BYTE");
            if (E_SUCC != str2tae(&vblk,idtype,sizeof(idtype),"%s",1,"IDTYPE"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting IDTYPE");
            }
         
            /* get the bands present on the tape
               ----------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,95,tmp_str,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdrrd","Error reading bands present");
            }
            if (E_FAIL == c_fhdfld_fst(95,hfields,ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhdfld","Error reading bands present");
            }
            tmp_str[hfields[3]] = '\0';
         
            /* cut the string off at the first blank in the string
               ------------------------------------------------------*/
            nbands = 0;
            for (i = 0; i < hfields[3]; i++)
            {
                if (tmp_str[i] == ' ')
                {
                    tmp_str[i] = '\0';
                    break;
                }
            }
            nbands = i;
         
            /* Pass the current band list back before it is sorted
               ------------------------------------------------------*/
            if (E_SUCC != str2tae(&vblk,tmp_str,1,"%1c",nbands,"BANDS_ORIG"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae",
                         "Error outputting BANDS_ORIG");
            }
         
            /* Sort the bands_avail array (only for the nbands)
               ---------------------------------------------------*/
            swapped = TRUE;
            for (i = 0; (i < nbands-1) && swapped; i++)
            {
                swapped = FALSE;
                for (j = nbands-1; i < j; j--)
                {
                    if (tmp_str[j] < tmp_str[j-1])
                    {
                        temp = tmp_str[j];
                        tmp_str[j] = tmp_str[j-1];
                        tmp_str[j-1] = temp;
                        swapped = TRUE;
                    }
                }
            }
         
            if (E_SUCC != str2tae(&vblk,tmp_str,1,"%1c",nbands,
                                  "BANDS_PRESENT"))
            {
                c_lasmsg(TRUE,"fstfmtin-str2tae",
                         "Error outputting BANDS_PRESENT");
            }
         
            /* get the tape blocking factor
               ------------------------------*/
            if (E_FAIL == c_fhdrrd_fst(header,97,(char *)&lvalue,
                                       ff_version))
            {
                c_lasmsg(TRUE,"fstfmtin-fhddrd",
                         "Error reading tape blocking factor");
            }
            if (E_SUCC != lng2tae(&vblk,&lvalue,1,1,"BLOCK_FACTOR",&tmp_long))
            {
                c_lasmsg(TRUE,"fstfmtin-lng2tae",
                         "Error outputting BLOCK_FACTOR");
            }
        }
      
        if (q_out(&vblk) != SUCCESS)
        {
            c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
        }
    }   
}

void get_suffix (char *file, char *file_ext, char *basename_suffix)
{
    char *suffix_ptr = NULL;     /* pointer to temporary variable for suffix */
    int j;                       /* Loop index */
    int length;
    
    /* Get the location of the last occurrence of a '.' from 'file'
       Copy the contents into 'file_ext', from the '.' to the end of the array,
       inclusive.
       Current file naming convention appends '.dat' or '.fst' to the end of
       each file.
       suffix_ptr will be null if the file naming convention is changed. 
    */
       
    suffix_ptr = strrchr (file, '.');
    if (suffix_ptr == NULL)
    {
        c_lasmsg (TRUE, "fstfmtin-ext", "Programming error when calling "
                  "get_suffix");
    } 
    strcpy (file_ext, suffix_ptr);
      
    /* Get the location of the last occurrence of a '_' from 'file'
       Copy the contents into 'basename_suffix', from the '_' to the end of the
       array, inclusive.
       Current file naming convention appends one of the following onto the
       basename of the data file, depending on the file type:
       '_band##'
       '_header'
       '_hrf'
       '_htm'
       '_hpn'
       '_b##'

       suffix_ptr will be null if the file naming convention is changed
    */
    suffix_ptr = strrchr (file, (int)'_');
    if (suffix_ptr == NULL)
    {
        c_lasmsg (TRUE, "fstfmtin-ext", "Programming error when calling "
                  "get_suffix");
    }
    length = (long) (strlen (suffix_ptr) - strlen (file_ext));
    strncpy (basename_suffix, suffix_ptr, length);
    basename_suffix[length] = '\0';
      
    /* Loop through the basename_suffix string and stop when the first
       occurrence of a digit is found. Terminate the string with a null at that
       position and break out of the loop.
       Applications using 'read_header' will fail if the band numbers are
       included.
    */
    for (j = 0; j < length; j++)
    {
        if (isdigit (basename_suffix[j]) )
        {
            basename_suffix[j] = '\0';
            break;
        }
    }
}
