/******************************************************************************

NAME: SRCH_DIR

PURPOSE: To search the specified filenames and determine which of the BANDS
         are listed in this file.  Sort the BANDS before they are returned.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  05/99  G.Schmidt        EDC   Initial development
    7.2  06/99	G.Schmidt        EDC   Changed to use c_getfiles
         08/01  A.Weissenberger  EDC   Removed redundant c_lasmsg calls
                                       Made searching for bands case
                                       insensitive
         
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES: Fast-L7A headers specify all the bands for each individual resolution
in the header even though these bands may span across several CDs.

For Fast-L7A, band 9 is really band 6H.  Normally band 6H would need to be
less than band 7 and band 8.  But this routine will only be called with a
list of bands from a certain header file and therefore we will only be
listing and sorting according to visible bands, thermal bands, and pan bands.
Thus band 6H will never be compared to any other band than 6L, since these
bands will only appear in the thermal header.  Given this, the bubble sort
will still work correctly, since 6(6L) < 9(6H).
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tapeio.h"
#include "fhdr.h"
#include "util.h"
#include "fstfmtin.h"

void main () 
{
    struct PARBLK vblk;        /* TAE parameter block                       */
    long std_out = 0;          /* standard out                              */
    char srchfile[CMLEN];      /* string to search for                      */
    char found_files[MAXBND][CMLEN];      
    /* array of filenames that matched the search   */
    long nfiles;               /* number of files found in the search       */
    long curr_file;            /* loop variable for the current file        */
    char curr_line[CMLEN];     /* current line to process                   */
    char bands[MAXBND][6];     /* array of the bands to search for  (each band
                                  may be up to 5 characters/digits long)    */
    char bands_avail[MAXBND][6];/* array of the bands that are available    */
    int nbands;                /* number of bands in bands array            */
    char band_str[10];         /* string to hold _b##.fst                   */
    char band_val[3];          /* string to hold ##                         */
    int band;                  /* integer band value                        */
    int avail_count;           /* number of bands in bands_avail            */
    int i,j;                   /* loop variable                             */
    long tmp_long;             /* temp long value                           */
    long multi_vol = 0;        /* specifies if this is a multi-volume set
                                  if bands == bands_avail then it is NOT a
                                  multi-volume set                          */
    int swapped;               /* used for the bubble sort                  */
    char temp[6];              /* used for the bubble sort                  */
    char tmp_str[MAXBND];
    long found = FALSE;        /* were any bands found?                      */
   
   
    c_inlas (&vblk, &std_out);
   
    /* Get the variables from the TAE variable block
       ------------------------------------------------*/
    sdgetpar (&vblk, srchfile, bands, &nbands);
   
    /* Get the filenames that match the srchfile string
       ---------------------------------------------------*/
    if (c_getfiles (srchfile, found_files, &nfiles, MAXBND, 1, 0) != E_SUCC)
    {
        c_lasmsg (TRUE, "fstfmtin-c_getfiles", "Error searching for header "
                  "file.");
    }    
   
    /* Read each line of the file and search for _b##.fst (filenames are output
       in tae format)
       -------------------------------------------------------------------*/
    for (i = 0; i < MAXBND; i++)
        strcpy (bands_avail[i], "");
    avail_count = 0;
    for (curr_file = 0; curr_file < nfiles; curr_file++)
    {
        strcpy (curr_line, found_files[curr_file]);
      
        /* Find the last occurrence of '_'
           ----------------------------------*/
        if (strrchr (curr_line, '_') != NULL)
        {
            strcpy (band_str, strrchr (curr_line, '_'));
            if ( !strncmp (band_str, "_b", 2) ||
                 !strncmp (band_str, "_B", 2) )
            { /* Found a band file */
                strncpy (band_val, &band_str[2], 2);   /* ## in _b##.fst */
                band = atoi (band_val);
            
                /* Check to see if the band is in the bands array.
                   --------------------------------------------------*/
                switch (band)
                {
                    case 10:
                    case 20:
                    case 30:
                    case 40:
                    case 50:
                    case 70:
                    case 80:
                        /* Look for the same band number in bands
                           -----------------------------------------*/
                        sprintf (band_val, "%d", band);
                        for (i = 0; i < nbands; i++)
                            if (band_val[0] == bands[i][0])
                            {
                                strcpy (bands_avail[avail_count++], bands[i]);
                                found = TRUE;
                            }
                        break;
               
                    case 61:
                        /* Look for '6' in the bands array
                           ----------------------------------*/
                        for (i = 0; i < nbands; i++)
                            if (bands[i][0] == '6')
                            {
                                strcpy (bands_avail[avail_count++], bands[i]);
                                found = TRUE;
                            }
                        break;
               
                    case 62:
                        /* Look for '9' in the bands array
                           ----------------------------------*/
                        for (i = 0; i < nbands; i++)
                            if (bands[i][0] == '9')
                            {
                                strcpy (bands_avail[avail_count++], bands[i]);
                                found = TRUE;
                            }
                        break;
               
                    default:
                        c_lasmsg(TRUE,"fstfmtin-band",
                                 "Invalid band in bands array");
                }
            }
        }
    }
   
    /* Sort the bands_avail array
       -----------------------------*/
    swapped = TRUE;
    for (i = 0; (i < avail_count-1) && swapped; i++)
    {
        swapped = FALSE;
        for (j = avail_count-1; i < j; j--)
        {
            if (bands_avail[j] < bands_avail[j-1])
            {
                strcpy (temp, bands_avail[j]);
                strcpy (bands_avail[j], bands_avail[j-1]);
                strcpy (bands_avail[j-1], temp);
                swapped = TRUE;
            }
        }
    }
   
    /* Since only bands in the array BANDS are put in BANDS_AVAIL, then
       if the size of BANDS and the size of BANDS_AVAIL are the same the
       two arrays must also be the same.
       --------------------------------------------------------------------*/
    if (nbands != avail_count)
        multi_vol = 1;
    else
        multi_vol = 0;
   
    if (found)
    {
        /* Pass the available bands back to TAE
           ---------------------------------------*/
        for (i = 0; i < avail_count; i++)
            tmp_str[i] = bands_avail[i][0];
        tmp_str[avail_count] = '\0';
        if (E_SUCC != str2tae(&vblk,tmp_str,1,"%1c",avail_count,"BANDS_AVAIL"))
        {
            c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting BANDS_AVAIL");
        }
      
        /* Pass the multi-volume flag back to TAE
           -----------------------------------------*/
        if (E_SUCC != lng2tae(&vblk,&multi_vol,1,1,"MULTI_VOL",&tmp_long))
        {
            c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting MULTI_VOL");
        }
    }
   
    /* Immediately send found back to TAE and exit.
       -----------------------------------------------*/
    if (E_SUCC != lng2tae(&vblk,&found,1,1,"SRCH_FOUND",&tmp_long))
    {
        c_lasmsg(TRUE,"fstfmtin-lng2tae","Error outputting SRCH_FOUND");
    }
   
    if (q_out(&vblk) != SUCCESS)
    {
        c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
    }
}
