/*****************************************************************************

NAME:update_ddr

PURPOSE: Update the information in the ddr obtained from the fast format 
         header.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
         10/01  A.Weissenberger   EDC  CR 5057 Formatting and removal of
                                       redundant c_lasmsg calls.
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
    Get the DDR record from disk.
    Allocate memory for all the band dependant records and then retrieve 
    them.
    Set the master line and sample parameters in the ddr.
    Set all of the valid flags to true.
    Update the projection, zone, and datum codes.
    Update the projection parameters and projection units.
    Update the corner points.
    Update the pixel size.
    Write the ddr to disk.
    For every band dependant record, copy the source, instrument, acquistion
    date and time.  Then write it out to disk.

NOTES:

******************************************************************************/
#include <string.h>
#include "fhdr2ddr.h"
#include "fstfmtin.h"

void update_ddr (
    char hostddr[CMLEN],   /* name of the host ddr to update */
    long window[4],        /* image window */
    double pixsize,        /* ground pixel size */
    double proj_coef[15],  /* GCTP projection parameters */
    double corners[8],     /* image projection corner points 
                              (x,y) UL, UR, LL, LR */
    long proj_code,        /* GCTP projection code */
    long zone_code,        /* UTM or State Plane code */
    long datum,            /* GCTP datum code */
    char source[32],       /* source of image */
    char band_list[][6],   /* list of bands to process (if NULL then
                              don't append the band to the DDR source) */
    int nbands,            /* number of bands in the band_list */
    char instrument[32],   /* instrument */
    char ac_date[10],      /* acquistion date */
    char ac_time[8],       /* acquistion time */
    char htype[5],         /* header type */
    int ddr_valid_flags[8]) /* ddr validity flags from fhdr2ddr */
{
    long i;                  /* working index value */
    int last_char_pos;       /* position of the last character in the source */
    char orig_source[32];    /* keep a copy of the original source */

    struct BDDR *bddrs;      /* pointer to the array of band dependantrecords*/
    struct BDDR *bptr;       /* pointer to the current BDDR record */
    struct DDR ddr;          /* local ddr structure */

    /* get the DDR
       ---------------*/
    if(E_FAIL == c_getddr(hostddr,&ddr))
    {
        c_lasmsg(TRUE,"fstfmtin-getddr","Error retrieving the DDR");
    }

    /* allocate memory for the band dependant records
       --------------------------------------------------*/
    if((bddrs = (struct BDDR *) malloc(ddr.nbands*sizeof(struct BDDR)))
       == NULL)
    {
        c_lasmsg(TRUE,"fstfmtin-alloc","Error allocating dynamic memory");
    }

    /* check to make sure there are ddr.nbands bands in the band_list
       -----------------------------------------------------------------*/
    if ((band_list != NULL) && (ddr.nbands != nbands))
    {
        c_lasmsg(TRUE,"fstfmtin-bands",
                 "Error in bands specified for the DDR");
    }

/* copy the source value (stripping any blanks from the source at the end)
   --------------------------------------------------------------------------*/
    last_char_pos = 0;
    for (i=(long)strlen(source)-1; i>=0; i--)
    {
        if (source[i] != ' ' || last_char_pos != 0)
        {
            orig_source[i] = source[i];
            if (last_char_pos == 0)
                last_char_pos = i;   /* keep track of the last character */
        }
    }
    orig_source[last_char_pos+1] = '\0';

    /* get the band dependant records
       ----------------------------------*/
    bptr = bddrs;

    for(i=1;i<=ddr.nbands;i++)
    {
        if(E_FAIL == c_getbdr(hostddr,bptr,&i))
        {
            c_intbdr(bptr);
            bptr->bandno = i;
        }
        bptr++;
    }

    /* update the DDR record
       -------------------------*/
    ddr.master_line = window[SL];
    ddr.master_sample = window[SS];

    /* set the valid flags
       ----------------------*/
    for (i = 0; i < 8; i++)
        ddr.valid[i] = ddr_valid_flags[i];

    /* set the datum code
       ---------------------*/
    ddr.datum_code = datum;

    /* set the projection and zone codes
       -------------------------------------*/
    ddr.proj_code = proj_code;
    ddr.zone_code = zone_code;

    strcpy(ddr.proj_units,"meters");

    /* copy the projection coefficients
       ------------------------------------ */
    i = 0;
    do
    {
        ddr.proj_coef[i] = proj_coef[i];
        i++;
    } while(i<15);

    /* update the corner points (if the datum is invalid then the corners
       were not able to be calculated)
       -------------------------------------------------------------------- */
    if (datum != 99)
    {
        ddr.upleft[0] = corners[1];
        ddr.upleft[1] = corners[0];

        ddr.loleft[0] = corners[5];
        ddr.loleft[1] = corners[4];

        ddr.upright[0] = corners[3];
        ddr.upright[1] = corners[2];

        ddr.loright[0] = corners[7];
        ddr.loright[1] = corners[6];
    }
    else
    {
        ddr.valid[6] = 0;         /* corners valid flag */

        ddr.upleft[0] = 0.0;
        ddr.upleft[1] = 0.0;

        ddr.loleft[0] = 0.0;
        ddr.loleft[1] = 0.0;

        ddr.upright[0] = 0.0;
        ddr.upright[1] = 0.0;

        ddr.loright[0] = 0.0;
        ddr.loright[1] = 0.0;
    }

    /* set the ground distance
       ---------------------------- */
    ddr.pdist_y = ddr.pdist_x = pixsize;

    /* Set the line/sample increments to 1 */
    ddr.line_inc = ddr.sample_inc = 1.0;

    /* now write the ddr to disk
       -------------------------------*/
    if(E_FAIL == c_putddr(hostddr,&ddr))
    {
        c_lasmsg(TRUE,"fstfmtin-putddr","Error writing DDR");
    }

    /* now update the band dependent stuff
       ---------------------------------------*/
    bptr = bddrs;
    for(i=0;i<ddr.nbands;i++)
    {
        /* add the band_channel to the source (if the band_list is not NULL)
           ----------------------------------------------------------------*/
        if (band_list != NULL)
        {
            if (!strcmp (band_list[i], "6"))
            {
                if (!strcmp (htype, "L7A"))
                    sprintf (source, "%s_BAND_6L", orig_source);
                else
                    sprintf (source, "%s_BAND_6", orig_source);
            }
            else if (!strcmp (band_list[i], "9"))
            {
                if (!strcmp (htype, "L7A"))
                    sprintf (source, "%s_BAND_6H", orig_source);
                else
                    sprintf (source, "%s_BAND_9", orig_source);
            }
            else
            {
                sprintf (source, "%s_BAND_%s", orig_source, band_list[i]);
            }
        }

        strncpy(bptr->source,source,32);
        strncpy(bptr->instrument,instrument,32);
        strncpy(bptr->date,ac_date,10);
        strncpy(bptr->time,ac_time,8);
    
        if(E_FAIL == c_putbdr(hostddr,bptr))
        {
            c_lasmsg(TRUE,"fstfmtin-putbdr","Error writing BDDR");
        }
        bptr++;
    }

    free(bddrs);
}
