/*******************************************************************************
NAME: convert_dms

PURPOSE: Convert all of the projection parameters from older format to
     the newer GCTP DMS format.  This is only for exponetial forms. 
     The new format only uses 2 ascii characters for the minutes 
     and seconds fields, while the older format uses 3.

PROJECT: LAS

ASSUMPTIONS:
   
   All character strings begin with the form:
      
      "  -0.dddmmmsssssssssE+09"   => remove 0 before minutes and seconds
      "  -0.ddmmmssssssssssE+08"   => remove 0 before minutes and seconds
      "  -0.dmmmsssssssssssE+07"   => remove 0 before minutes and seconds
      "  -0.mmmssssssssssssE+06"   => remove 0 before minutes and seconds
      "  -0.mmsssssssssssssE+05"   => remove 0 before seconds
      "  -0.mssssssssssssssE+04"   => remove 0 before seconds 
      "  -0.sssssssssssssssE+03"   => remove 0 before seconds
      "  -0.sssssssssssssssE+02"   => do nothing
      "  -0.sssssssssssssssE+01"   => do nothing
      "  -0.sssssssssssssssE+00"   => do nothing
   
   where:
   
      1. The leading sign can be either negative, positive or absent
      2. The character to the left of the decimal point is '0'
      3. The fifth character is '.'
      4. d, m, and s are characters from '0' to '9'
      5. E is always in the fourth position from the right
      6. The third position from the right is either '+' or '-'
      7. The exponent is '00' to '09' only.  All others will be trapped

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

static void convert1
(
    char *key,  /* key used in error messages */
    char *str   /* in/out - string containing a DDDMMSS longitude or latitude */
);

void local_convert_dms
(
    char **strings,       /* pointer to strings     */
    long proj_number,     /* USGS projection number */
    char *key             /* key used in error messages */
)
{
    char temp_str[CMLEN]; /* temporary string */
    char tempkey[30];     /* temporary string used in creating msg keys. */

    if (proj_number == 1)
    {
        /* UTM parameters 1 & 2 are DMS fields */
        convert1(key,*(strings + 0));
        convert1(key,*(strings + 1));
    }
    else if (proj_number == 2)
    {
        /* SPCS has no parameters that are in DMS format */
    }
    else if ((proj_number == 3) || (proj_number == 4) || (proj_number == 8))
    {
        /* ACEA, LCC, & EC parms 3, 4, 5, & 6 are DMS fields */
        convert1(key,*(strings + 2));
        convert1(key,*(strings + 3));
        convert1(key,*(strings + 4));
        convert1(key,*(strings + 5));
    }
    else if ((proj_number ==  5) || (proj_number ==  6) ||
            (proj_number ==  7) || (proj_number ==  9) ||
            (proj_number == 10) || (proj_number == 11) ||
            (proj_number == 12) || (proj_number == 13) ||
            (proj_number == 14) || (proj_number == 15) ||
            (proj_number == 17) || (proj_number == 19))
    {
        /* MER,PS,PC,TM,SG,LAEA,AE,GNO,OG,GVNP,ER, & VDG parms 5 & 6 are DMS 
           fields*/
        convert1(key,*(strings + 4));
        convert1(key,*(strings + 5));
    }
    else if ((proj_number == 16) || (proj_number == 18))
    {
        /* SIN & MC parameter 5 is a DMS field */
        convert1(key,*(strings + 4));
    }
    else if (proj_number == 21)
    {
        /* SOM parameters 4 & 5 are DMS fields */
        convert1(key,*(strings + 3));
        convert1(key,*(strings + 4));
    }
    else if (proj_number == 20)
    {
        /* OM parms 4, 5, 6, 9, 10, 11, & 12 are DMS fields */
        convert1(key,*(strings +  3));
        convert1(key,*(strings +  4));
        convert1(key,*(strings +  5));
        convert1(key,*(strings +  8));
        convert1(key,*(strings +  9));
        convert1(key,*(strings + 10));
        convert1(key,*(strings + 11));
    }
    else
    {
        sprintf(temp_str,"Projection number %d unknown",proj_number);
        strcpy(tempkey,key);
        strcat(tempkey,"-invalid");
        c_lasmsg(TRUE,tempkey,temp_str);
    }
}


/*****************************************************************************
NAME: convert1

PURPOSE: Convert a single string from the older format to the  GCTP DMS 
     format.  This is only for the exponetial form.  The new format 
     only uses 2 ascii characters for the minutes and seconds fields, 
     while the older format uses 3.

PROJECT: LAS

ASSUMPTIONS:  See those for convert_dms

*******************************************************************************/

static void convert1
(
    char *key,  /* key used in error messages */
    char *str   /* in/out - string containing a DDDMMSS longitude or latitude */
)

{
    char degree_str[4];    /* the degrees portion of the input and output */
    char dms_str[16];      /* the dms portion of the input */
    char E_str[3];         /* the E+ or E- portion of the input and output */
    char exponent_str[3];  /* the exponent portion of the input and output */
    char minute_str[4];    /* the minutes portion of the input and output */
    char second_str[16];   /* the seconds portion of the input and output */
    char temp_str[CMLEN];  /* temporary string */
    char tempkey[30];      /* temporary string used in creating msg keys */


    /* Extract the dms, E+ or E-, and the exponent portions of the input string
       ----------------------------------------------------------------------*/
    strncpy(dms_str,      (str +  5), 15);  dms_str[15]     = '\0';
    strncpy(E_str,        (str + 20),  2);  E_str[2]        = '\0';
    strncpy(exponent_str, (str + 22),  2);  exponent_str[2] = '\0';
    *(str + 5) = '\0';

    if ((strcmp(exponent_str,"09") == 0) ||
            (strcmp(exponent_str,"08") == 0) ||
            (strcmp(exponent_str,"07") == 0) ||
            (strcmp(exponent_str,"06") == 0))
    {
        if (strcmp(exponent_str,"09") == 0)
        {
            strncpy(degree_str, &dms_str[0], 3);  degree_str[3] = '\0';
            strncpy(minute_str, &dms_str[4], 2);  minute_str[2] = '\0';
            strncpy(second_str, &dms_str[7], 8);
            strncpy(&second_str[8], "00",2); second_str[10] = '\0';
            strcpy (exponent_str, "07");
        }
        else if (strcmp(exponent_str,"08") == 0)
        {
            strncpy(degree_str, &dms_str[0], 2);  degree_str[2] = '\0';
            strncpy(minute_str, &dms_str[3], 2);  minute_str[2] = '\0';
            strncpy(second_str, &dms_str[6], 9);
            strncpy(&second_str[9], "00",2); second_str[11] = '\0';
            strcpy (exponent_str, "06");
        }
        else if (strcmp(exponent_str,"07") == 0)
        {
            strncpy(degree_str, &dms_str[0],  1);  degree_str[1]  = '\0';
            strncpy(minute_str, &dms_str[2],  2);  minute_str[2]  = '\0';
            strncpy(second_str, &dms_str[5], 10);
            strncpy(&second_str[10], "00",2); second_str[12] = '\0';
            strcpy (exponent_str, "05");
        }
        else if (strcmp(exponent_str,"06") == 0)
        {
            strcpy (degree_str, "");
            strncpy(minute_str, &dms_str[1],  2);  minute_str[2]  = '\0';
            strncpy(second_str, &dms_str[4], 11);
            strncpy(&second_str[11], "00",2); second_str[13] = '\0';
            strcpy (exponent_str, "04");
        }
    }
    else if ((strcmp(exponent_str,"05") == 0) ||
            (strcmp(exponent_str,"04") == 0) ||   
            (strcmp(exponent_str,"03") == 0))
    {
        strcpy (degree_str, "");
        if (strcmp(exponent_str,"05") == 0)
        {
            strncpy(minute_str, &dms_str[0],  2);  minute_str[2]  = '\0';
            strncpy(second_str, &dms_str[3], 12);
            strncpy(&second_str[12], "00",1); second_str[13] = '\0';
            strcpy (exponent_str, "04");
        }
        else if (strcmp(exponent_str,"04") == 0)
        {
            strncpy(minute_str, &dms_str[0],  1);  minute_str[1]  = '\0';
            strncpy(second_str, &dms_str[2], 13);
            strncpy(&second_str[13], "00",1); second_str[14] = '\0';
            strcpy (exponent_str, "03");
        }
        else if (strcmp(exponent_str,"03") == 0)
        {
            strcpy(minute_str, "");
            strncpy(second_str, &dms_str[1], 13);
            strncpy(&second_str[14], "00",1); second_str[15] = '\0';
            strcpy (exponent_str, "02");
        }
    }
    else if ((strcmp(exponent_str,"02") == 0) ||
            (strcmp(exponent_str,"01") == 0) ||
            (strcmp(exponent_str,"00") == 0))
    {
        strcpy(degree_str, "");
        strcpy(minute_str, "");
        strcpy(second_str, dms_str);
    }
    else
    {
        sprintf(temp_str,"Unexpected exponent %s found in DMS field", 
                exponent_str);
        strcpy(tempkey,key);
        strcat(tempkey,"-dms");
        c_lasmsg(TRUE,tempkey,temp_str);
    }

    strcat(str, degree_str);
    strcat(str, minute_str);
    strcat(str, second_str);
    strcat(str, E_str);
    strcat(str, exponent_str);
    str[24] = ' ';
}

