/******************************************************************************
FUNCTION:   CP_BSQ

PURPOSE:    Copies a LAS image to tape in BSQ format.

ALGORITHM 
  Calculate the output number of lines and samples
  Allocate input buffer space
  Calculate the number of bytes for the output buffer
  Initialize the processing message
  Process each image
      Process each band
      Start the processing message
      Process each line
          Set the output pointer to the desired output block location
          Only write the data to tape when the block size has been reached
  Stop the processing message
  Free allocated buffer memory

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "tapeio.h"
#include "pixman.h"
#include "util.h"
#include "sysdef.h"
#include "fhdr.h"
#include "protos.h"

void cp_bsq
(
    long *unit,             /* tape unit number                           */
    long *nmrimg,           /* number of input images                     */
    long window[][4],       /* input window (sl,ss,nl,ns)                 */
    long bands[][MAXBND+1], /* list of input bands to be copied           */
    long nbands[],          /* # bands in each input image                */
    long *totbnd,           /* total number of bands to be copied         */
    long *odtype,           /* output image data type                     */
    long *blksiz,           /* number of image lines in output block      */
    long *msgtim,           /* elapsed time between messages              */
    long *bands_tape,
    char *header,           /* header string                              */
    char append_file[],     /* the append file name                       */
    long *append_flag       /* flag  TRUE if file is to be append to tape */
)
{
    extern struct FDESC *fdesc[];/* image file descriptors used for image I/O */
    extern long  curent;        /* output line being processed                */
    extern long  curcat;        /* output band being processed                */
    extern long  datasize[];    /* data size of the output data               */
    extern short ddr_flag;      /* DDR header flag                            */
    extern struct PARBLK vblock;/* TAE parameter block                        */
    extern char label_file[];   /* File containing the tape label info        */

    long image;         /* loop index for current input image             */
    long inbnd;         /* current input image band number                */
    long inlin;         /* current input image line                       */
    long curblk;        /* current line count for blocking                */
    long out_byte;      /* # of bytes per output image line               */
    long out_nl;        /* output number of lines                         */
    long out_ns;        /* output number of samples                       */
    long nelem;         /* number of output element                       */
    long status;        /* function return status code                    */
    long one = 1;       /* value of one                                   */
    long zero = 0;      /* value of zero                                  */
    long msgflg = ONECAT;   /* Processing message format flag             */
    long band_count;    /* current band number                            */
    long header_count = HEADER_SIZE; /* number of bytes in header         */
    long mode = HANDLE; /* Tapeio mode for error handeling                */
    long lng_val[1];    /* Array of long values used for call to c_putparm */
    long bytes_to_write;/* bytes to write to tape                         */
    char ops[1][CMLEN]; /* Operators initials field of tape label         */
    char temp[1][CMLEN];
    FILE *fptr;         /* file pointer to append file                    */
    struct stat status_buf; /* structure for file statistics              */

    double dbl_val[1];  /* Array of double values used for call to c_putparm */

    char *tempbuf;          /* temporary buffer to hold appended file         */
    char message[80];       /* append flag message                            */
    unsigned char *in_buf;  /* buffer to read input image line                */
    unsigned char *out_buf; /* buffer to read input image line                */
    unsigned char *out_ptr; /* pointer into output buffer block               */

    /* Calculate the output number of lines and samples
       ---------------------------------------------------*/
    out_nl = window[0][NL];
    out_ns = window[0][NS];

    /* Allocate input buffer space
       ------------------------------*/
    in_buf = (unsigned char *) malloc((window[0][NS] + 1) * datasize[*odtype]);
    if (in_buf == NULL)
    {
        c_lasmsg(TRUE,"fstfmtout-malloc",
                 "Error allocating dynamic buffer space");
    }     

    /* Calculate the number of bytes for the output buffer
       ------------------------------------------------------*/
    out_byte = *blksiz * out_ns * datasize[*odtype];

    /* Allocate output buffer space
       -------------------------------*/
    out_buf = (unsigned char *) malloc(out_byte);
    if (out_buf == NULL)
    {
        c_lasmsg(TRUE,"fstfmtout-malloc",
                 "Error allocating dynamic buffer space");
    }

    /* Initialize the processing message
       ------------------------------------*/
    c_inittm("FSTFMTOUT","Processed","line","band","",&out_nl,totbnd,msgtim,
            &msgflg);

    /* Initialize the current block
       -------------------------------*/
    curblk = 0;
    band_count = *bands_tape;

    /* Process each image
       ---------------------*/
    for (image = 0, curcat = 0; image < *nmrimg; image++)
    {
        /* Process each band
           --------------------*/
        for (inbnd = 0; inbnd < nbands[image]; inbnd++)
        {
            /* Start the processing message
               -------------------------------*/
            ++curcat;
            curent = 1;
            c_startm();

            /* do not overfill tape
               ---------------------*/
            if (curcat > band_count)
            {
                if (curblk != 0)
                    write_tape(unit,out_buf,&out_byte,&zero);
                /* Close the tape device
                   ------------------------*/
                c_stoptm();
                close_tape(unit,REWOFFL,2);
                /* If label_file isn't an empty string then print a label for  
                   tape
                   ----------------------------------------------------------*/
                if (strlen(label_file) > 0)
                {
                    strcpy(ops[0], "SUC");
                    sprintf(temp[0],"%d/%d",(curcat - 1)/(*bands_tape),
                            (long)((float)*totbnd/((float)*bands_tape) + .9));
                    c_putparm(label_file, "OPS", ops, "S", dbl_val, 1, lng_val);
                    c_putparm(label_file,"SEQUENCE",temp, "S", dbl_val, 1,
                              lng_val);
                    print_label(label_file);
                }

                mount_tape(unit,TWRITE,FALSE);
                if ((curcat -1 + *bands_tape) > *totbnd)
                    *bands_tape = *totbnd - curcat + 1;
                update_header(header,window[0][NL],window[0][NS],curcat,
                        *bands_tape,blksiz);

                write_tape(unit,(unsigned char *)header,&header_count,&zero);
                if ((status = c_tpweof(unit,&one,&mode)) != TAPE_SUCC)
                    c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
                band_count += *bands_tape;
                c_startm();
            }


            /* Process each line
               --------------------*/
            for (inlin = 1, curent = 0; curent < out_nl; inlin++, curent++)
            {
                /* read in data
                   -------------*/
                status = c_eread(&fdesc[image],&bands[image][inbnd],&inlin,
                        in_buf, &one);
                if (status == E_FAIL)
                {
                    bksp_file(unit);
                    c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
                }

                nelem = out_ns;

                /* Set the output pointer to the desired output block location
                   ----------------------------------------------------------*/
                out_ptr = out_buf + (curblk * nelem * datasize[*odtype]);

                /* Copy the converted input data to the output buffer
                   -----------------------------------------------------*/
                c_pxcopy(in_buf,out_ptr,odtype,&nelem);

                /* Only write the data to tape when the block size has 
                   been reached
                   ----------------------------------------------------*/
                if (curblk == *blksiz - 1)
                {
                    write_tape(unit,out_buf,&out_byte,&zero);
                    curblk = 0;
                }
                else
                    curblk++;
            }
            /* Insure that all the data is written
               --------------------------------------*/
            if (curblk != 0)
            {
                write_tape(unit,out_buf,&out_byte,&zero);
                curblk = 0;
            }

            /* Stop the processing message
               ------------------------------*/
            c_stoptm();
            if ((status = c_tpweof(unit,&one,&mode)) != TAPE_SUCC)
                c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        }


        /* Close the input image
           ------------------------*/
        status = c_eclose(&fdesc[image]);
        if (status != E_SUCC)
            c_lasmsg(FALSE,"fstfmtout-warn","Nonfatal error encountered");
    }

    if (*append_flag)
    {
        sprintf(message,"Appending file %s to end of tape",append_file);
        c_lasmsg(FALSE," ",message);

        fptr = fopen(append_file,"r");

        /* find file size and allocate buffer
           -----------------------------------*/
        stat(append_file,&status_buf);
        tempbuf = (char *) malloc(status_buf.st_size);
        if (tempbuf == NULL)
        {
            c_lasmsg(TRUE,"fstfmtout-malloc",
                     "Error allocating dynamic buffer space");
        }

        /* read file in and write to tape
           --------------------------------*/
        fread(tempbuf,sizeof(char),status_buf.st_size,fptr);
        bytes_to_write = (long)status_buf.st_size;
        write_tape(unit,(unsigned char *)tempbuf,&bytes_to_write,&zero);
        if ((status = c_tpweof(unit,&one,&mode)) != TAPE_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        fclose(fptr);
    }

    /* Free allocated buffer memory
       -------------------------------*/
    free(in_buf);
    free(out_buf);

}

