/*******************************************************************************
FUNCTION:   MOUNT_TAPE

PURPOSE:    Physically mounts a tape.  The mount message is determined by
        the value of the global variables "comment" and "curvol",
        where "curvol" is an index into array "comment".  After the
        tape is mounted the tape drive name and "curvol" are saved
        in the TAE parameter block.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "protos.h"

void mount_tape
(
    long *unit,               /* Tape unit number                           */
    long access,              /* Open tape for specified  access            */
    int flag                  /* Flag to disable display messages           */
)
{
    extern struct PARBLK vblock;    /* TAE parameter block                  */
    extern char   comment[][CMLEN]; /* Tape mount message to the operator   */
    extern char   density[5];       /* Tape density in BPI                  */
    extern char   *drive[1];        /* Physical tape drive name             */
    extern char   label_file[];     /* File containing tape label info      */
    extern long   curvol;           /* Current tape volume being process    */

    char   tmp_str[CMLEN];          /* Temporary string variable            */

    long   i;                       /* Temporary variable                   */
    long   mode     = HANDLE;       /* Tape I/O outputs error messages      */
    long   rewopt   = REWIND;       /* Rewind option                        */
    long   skip     = 0;            /* Skip zero files on open              */
    long   status;                  /* Function return status code          */

    /* Variables for tape label generation
       --------------------------------------*/
    char   date    [1][CMLEN];      /* Date the tape is created             */
    char   ops     [1][CMLEN];      /* Operators initials field of tape label */
    char   sequence[1][CMLEN];      /* Sequence number of this tape         */
    char   tapeid  [1][CMLEN];      /* ID of the tape that is mounted       */
    char   time    [1][CMLEN];      /* Time the tape is created             */
    double dbl_val[1];              /* Temp array for calls to c_putparm    */
    long   lng_val[1];              /* Temp array for calls to c_putparm    */


    /* If disable display message flag is set, stop the processing message
       ----------------------------------------------------------------------*/
    if (flag)
        c_stoptm();

    /* Mount the current tape volume
       --------------------------------*/
    curvol++;
    status = c_tpopen(unit, &skip, &mode, &rewopt, density, &access,
            comment[curvol], tapeid[0]);
    drive[0] = comment[curvol];
    if (status != TAPE_SUCC)
        c_lasmsg(TRUE,"fstfmtout-faatl","Fatal error encountered");

    /* Send the tape drive and current volume back to TAE
       -----------------------------------------------------*/
    if (q_string(&vblock, "DRIVE", 1, drive, P_UPDATE) != SUCCESS)
        c_lasmsg(TRUE,"fstfmtout-string","Error passing string to TAE");
    if (q_intg(&vblock, "CURVOL", 1, (int*)&curvol, P_UPDATE) != SUCCESS)
        c_lasmsg(TRUE,"fstfmtout-intg","Error passing integer to TAE");
    if (q_out(&vblock) != SUCCESS)
        c_lasmsg(TRUE,"fstfmtout-out","Error sending variable block to TAE");

    /* Notify the user of the tape ID mounted by the operator
       ---------------------------------------------------------*/
    i = (long)strlen(tapeid[0]);
    if (i < 1)
    {
        sprintf(tmp_str,"Tape %1d: operator did not enter tape ID",curvol + 1);
        c_lasmsg(FALSE," ",tmp_str);
        strcpy(tapeid[0], "??????");
    }
    else
    {
        c_up2low(tapeid[0], &i);
        sprintf(tmp_str,"Tape %1d: operator mounted %s", curvol + 1,tapeid[0]);
        c_lasmsg(FALSE," ",tmp_str);
    }

    /* If label_file is a non empty string then output info for tape labels
       -----------------------------------------------------------------------*/
    if (strlen(label_file) > 0)
    {

        /* Place the labels flag in the vblock
           --------------------------------------*/
        i = TRUE;
        if (q_intg(&vblock, "LABELS", 1, (int*)&i, P_UPDATE) != SUCCESS)
            c_lasmsg(TRUE,"fstfmtout-intg","Error passing integer to TAE");
        if (q_out(&vblock) != SUCCESS)
            c_lasmsg(TRUE,"fstfmtout-out",
                     "Error sending variable block to TAE");

        /* Write the tape information to label_file.  Write the operators 
           initials as ERR, (ie. assume an error will occur), and if the tape 
           is successfully completed then this can be changed to SUC.
           -----------------------------------------------------------------*/
        c_date_time(date[0], time[0]);
        strcpy(ops[0], "ERR");
        sprintf(sequence[0], "%1d/", curvol + 1);
        if (c_putparm(label_file,"TAPEID",tapeid,"S",dbl_val,1,lng_val)!=E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        if (c_putparm(label_file,"DENSITY",(char (*)[256])density,"S",dbl_val,1,
                     lng_val) != E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        if (c_putparm(label_file,"DRIVE",(char (*)[256])comment[curvol],
                "S",dbl_val,1,lng_val) != E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        if (c_putparm(label_file,"OPS",ops,"S",dbl_val,1,lng_val) != E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        if (c_putparm(label_file,"SEQUENCE",sequence,"S",dbl_val,1,lng_val)
                !=E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        if (c_putparm(label_file,"TIME",time,"S",dbl_val,1,lng_val) != E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        if (c_putparm(label_file,"CREATE",date,"S",dbl_val,1,lng_val) != E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
    }
    else
    {
        /* Place the labels flag in the vblock
           --------------------------------------*/
        i = FALSE;
        if (q_intg(&vblock, "LABELS", 1, (int*)&i, P_UPDATE) != SUCCESS)
            c_lasmsg(TRUE,"fstfmtout-intg","Error passing integer to TAE");
        if (q_out(&vblock) != SUCCESS)
            c_lasmsg(TRUE,"fstfmtout-out",
                    "Error sending variable block to TAE");
    }

    /* If processing message was stopped, restart it
       -----------------------------------------------*/
    if (flag)
        c_startm();

}

/*******************************************************************************
FUNCTION:   REOPEN_TAPE

PURPOSE:    Reopens a tape device.  The tape drive name is stored in the
        global "drive".  The drive name is saved in the TAE parameter
        block.

ALGORITHM REFERENCES    none
*******************************************************************************/
void reopen_tape
(
    long *unit,                 /* tape unit number                           */
    long access                 /* open tape for read access                  */
)
{
    extern char *drive[1];      /* string descriptor for tape drive           */
    extern char density[5];     /* tape density in BPI                        */
    extern struct PARBLK vblock;/* TAE parameter block                        */

    long rewopt = NOREW;        /* rewind option                              */
    long mode = HANDLE;         /* tape I/O outputs error messages            */
    long skip = 0;              /* skip zero files on open                    */
    long status;                /* function return status code                */


    /* Reopen the tape on the specified drive
       -----------------------------------------*/
    status = c_tpreop(unit,&skip,&mode,&rewopt,density,&access,drive[0]);
    if (status != TAPE_SUCC)
        c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");

    /* Update the parameter block with the tape drive
       -------------------------------------------------*/
    status = q_string(&vblock,"DRIVE",1,drive,P_UPDATE);
    status = q_out(&vblock);

}

/*******************************************************************************
FUNCTION:   WRITE_TAPE

PURPOSE:    Writes a record onto tape and checks the status of a tape 
        write.  If the end of tape is encountered, the next tape
        volume is mounted.  Note that if the multi-volume flag is
        set to "NO" the current image being processed is erased
        and written to the next tape volume.

ALGORITHM REFERENCES    none
*******************************************************************************/
int write_tape
(
    long *unit,          /* tape unit number                               */
    unsigned char *buf,  /* buffer to be written to tape                   */
    long *out_byte,      /* number of bytes to be written                  */
    long *bckrec         /* number of records to back space to get back to 
                            the beginning of the bands (BIL only)          */
)
{
    long mode = HANDLE; /* tape I/O outputs error msg.                    */
    long status;        /* function return status code                    */

    /* Write the buffer to tape
       ---------------------------*/
    status = c_tpwrit(unit,buf,out_byte,&mode);
    if (status == *out_byte)
        return(E_SUCC);

    /* Check the return status
       --------------------------*/
    switch (status)
    {
        case TAPE_EOT:
            /* Stop the processing message
               ------------------------------*/
            c_stoptm();
            c_lasmsg(FALSE," ","EOT encountered");

            /* Close the current tape
               -------------------------*/
            close_tape(unit,REWOFFL,2);

            /* Display a fatal error message and terminate
               ----------------------------------------------*/
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");

            break;

        case TPWRIT_SIZE:
            /* Stop the processing message
               ------------------------------*/
            c_stoptm();

            /* Back space to the end of the last file processed to erase it.
               ---------------------------------------------------------------*/
            bksp_file(unit);

            /* Display a fatal error message and terminate
               ----------------------------------------------*/
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
            break;

        case TPWRIT_WRITE:
            /* Stop the processing message
               ------------------------------*/
            c_stoptm();

            /* Back space to the end of the last file processed to erase it.
               ---------------------------------------------------------------*/
            bksp_file(unit);

            /* Display a fatal error message and terminate
               ----------------------------------------------*/
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
            break;

        default:
            /* Stop the processing message
               ------------------------------*/
            c_stoptm();

            /* Back space to the end of the last file processed to erase it.
               --------------------------------------------------------------*/
            bksp_file(unit);

            /* Display a fatal error message and terminate
               ----------------------------------------------*/
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
            break;
    }

    return(status);
}

/******************************************************************************
FUNCTION:   CLOSE_TAPE

PURPOSE:    Closes the tape device.

ALGORITHM 
  Close the tape

ALGORITHM REFERENCES    none
*******************************************************************************/
void close_tape
(
    long *unit,         /* tape unit number                                   */
    long rewopt,        /* rewind option                                      */
    long eofs           /* Number of EOF's to write to tape                   */
)
{
    long mode = NO_HANDLE;  /* tape I/O outputs error messages                */

    /* Close the tape
       -----------------*/
    if (c_tpclos(unit, &mode, &rewopt, &eofs) != TAPE_SUCC)
        c_lasmsg(FALSE,"fstfmtout-warn","Nonfatal error encountered");

}

