/******************************************************************************

NAME:	 GAUSTRAN	

PURPOSE: Calculates gauss transfer function of an images. 

PROGRAM HISTORY:

  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Mar-85    B. Dealy
   2         16-Oct-89    K. Kunska          conversion to 5.0   
   3         02-Mar-90    K. Kunska          fixed information msg display in 
                                             GETPAR.C; corrected PDF file
   4         03-Apr-90    K. Kunska          Fixed a min_max problem 
   5	     09-Sep-92	  T. Mittan	     Modified include file to 
					     eliminate warnings on link
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get the user supplied parameters.
   Call FILSPEC to open the image files.
   Call COMCALC to apply the Gauss transfer function to the input image.
   Close the images.
   Update the history records and print the completion message.

******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

struct  PARBLK  block;                 /* TAE parameter block     */
struct  DDR     ddr;                   /* ddr                     */

float   *inbuf;                        /* image input buffer      */
float   *outbuf;                       /* image output buffer     */

char    out_files[2][CMLEN];           /* output file names        */
struct GDESC *  gdescin;               /* input group descriptor   */
struct GDESC *  gdescout;                      /* output group descriptor  */

long    min_max;                       /* minmax option flag       */
long    msgtim;                        /* message interval         */
long    nclass;                        /* number of classes in the */
                                       /* statistics file          */
extern long    curent;                 /* current entity           */
extern long    curcat;                 /* current category         */

static  long    inf_err = 1;           /* info error return status  */

static  long    one = 1;	       /* constant of one	    */


int main
(void) /* gaustran */
{

char	hosin[MAXIMG][CMLEN],		/* input image host name            */ 
	hosout[1][CMLEN],			/* output image host name           */ 
	taein[MAXIMG][CMLEN],		/* input image tae name             */ 
	taeout[1][CMLEN];			/* output image tae name            */ 
static
double	amax[MAXBND],			/* max pixel value for each band     */
	amin[MAXBND];			/* min pixel value for each band     */
static
float	expsf[50],			/* scale factors for the exponent    */ 
	offset[50],			/* offset to be subtracted from the  */
					/* input pixels                      */
	scalfact[50];			/* scale factors for the result of   */
					/* the exponentiation                */ 
static
long	*bands_per_img,			/* number of bands for each image    */
	dtype,				/* data type of the input image      */
	img_flag[MAXIMG][2],		/* flags for window/band options     */
	img_limit,			/* maximum number of images          */
	*ipoint,
	nbytes,				/* dynamic memory size allocation    */
	num_inimg,			/* number of input images            */
	num_lines,			/* number of lines in the image      */
	num_pixels,			/* number of pixels per line         */
	*opoint,
	out_bands,			/* number of output image bands      */
	out_img_bands[MAXIMG][MAXBND+1],/* band numbers for output image     */
	outwin[1][4],			/* output image window specifications*/
	same_wnd,			/* same window flag                  */
	tot_inband,			/* total number of bands             */
	window[MAXIMG][4],		/* input image dimensions SL SS NL NS*/
	num_img_bands[MAXIMG][MAXBND+1];/* band numbers for input image      */

/*
Dynamic Memory Allocation
-------------------------
*/
nbytes = (MAXIMG * MAXBND +1) * sizeof(long);
ipoint = (long *) malloc(nbytes);
if (ipoint == NULL)
   {
   c_errmsg("Error allocating dynamic memory","gaustran-alloc",&inf_err);
   }

nbytes = MAXBND * sizeof(long);
opoint = (long *) malloc(nbytes);
if (opoint == NULL)
   {
   c_errmsg("Error allocating dynamic memory","gaustran-alloc",&inf_err);
   }

nbytes = MAXIMG * sizeof(long);
bands_per_img = (long *) malloc(nbytes);
if (bands_per_img == NULL)
   {
   c_errmsg("Error allocating dynamic memory","gaustran-alloc",&inf_err);
   }

/*
Call GETPAR to get the user supplied parameters.
-------------------------------------------------
*/
getpar (taein, hosin, taeout, hosout, &dtype, bands_per_img, img_flag,
        img_limit, num_img_bands, &num_inimg, &same_wnd, &tot_inband,
        scalfact, expsf, offset, window);

/*
Call FILSPEC to open the image files.
-------------------------------------
*/
out_bands = tot_inband;
filspec (hosin, hosout[0], num_inimg, num_img_bands, out_img_bands,
         out_bands, bands_per_img, ipoint, opoint, window);
 
num_pixels = window[0][NS];
num_lines = window[0][NL]; 

/*
Call COMCALC to apply the Gauss transfer function to the input image.
-----------------------------------------------------------------------
*/
comcalc (num_pixels, num_lines, tot_inband, scalfact, expsf, offset, ipoint,
	 opoint);

/*
Close the images.
-------------------
*/
c_egclse (&gdescin);
c_egclse (&gdescout);

/* 
Determine the minimum and maximum values of the output image.
Write the min, max and the validity flag values to the ddr file.
-----------------------------------------------------------------
*/
if (min_max)
   {
   outwin[0][SL] = outwin[0][SS] = 1;
   outwin[0][NL] = num_lines;
   outwin[0][NS] = num_pixels;
   c_minmax (hosout, &one, outwin, out_img_bands, &out_bands, amin, amax);
   c_svmnmx (hosout[0], &out_bands, amin, amax, "YES");
   }

/*
Update the history records and print the completion message.
-------------------------------------------------------------
*/
c_hstout (&block, hosout[0], hosin, &num_inimg, "");
c_complt ("GAUSTRAN", out_files, &one); 

free (bands_per_img);
free (ipoint);
free (opoint);
return EXIT_SUCCESS;

}

