/******************************************************************************
NAME: GET_COORDINATES

PURPOSE:
     Retrieve the corner coordinates or the transformation from the 
     geoTIFF keys.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      01/01  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

NOTE: A typical geoTIFF file should have both of the tags ModelTiepointTag
      (which lists the upper left corner coordinates) and ModelPixelScaleTag 
      (which lists the pixel size). OR contain only the ModelTransformationTag
      (which lists the upper left corner coordinates and the pixel size).
      Unfortunately there have been some geoTIFF images that have a mixture
      of a couple (for example, ModelPixelScaleTag and ModelTransformationTag)
      or lists all three tags. This routine is written to first use the tags
      ModelPixelScaleTag and ModelTiepointTag. Then use ModelTransformationTag.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "geotiffin.h"

int get_coordinates 
(
    TIFF *in,               	/* I: Input tiff file pointer     	   */
    double pixel,	 	/* I: resolution of image		   */	
    long *coordinate_valid_flag,/* I/0: corner coordinate flag value 
                                        (VALID or INVAL)                   */
    double tiepoints[],         /* O: corner coordinates values            */
    double geotrans[]           /* O: model transformation corner 
                                      coordinates                          */
)
{
    double *temp;               /* used to extract geoTIFF tag              */
    double *transformation_tag_ptr; /* pointer to transformation tag values */
    double *tiepoints_tag_ptr;  /* pointer to tiepoints tag values          */
    int index;                  /* looping variable                         */
    long status = 0;            /* Function return status. If 0, the tag was 
                                   not found otherwise the tag was found    */ 


    /* Get tagged keys (corner coordinates)
    --------------------------------------*/
    /* If pixel has a value, then ModelPixelScaleTag exists and was retrieved
       earlier in this application */
    if (pixel != 0.0)
    {
        /* Look for ModelTiepointTag */
        status = TIFFGetField(in, TIFFTAG_GEOTIEPOINTS, &temp, 
                              &tiepoints_tag_ptr);
        if (status == 0)
        { 
            /* Since the ModelTiepointTag was not found then look for 
               the ModelTransformationTag */
            status = TIFFGetField(in, TIFFTAG_GEOTRANSMATRIX, &temp,
                                  &transformation_tag_ptr);
            if (status == 0)
            {
                /* Since neither the ModelTransformationTag or ModelTiepointTag
                   were found then set the corner coordinates flag to invalid*/
                c_lasmsg(NONFATAL,"coordinates-warn","Neither ModelTiepointTag"
                         " or ModelTransformationTag exist in geoTIFF file."
                         " Coordinates are set to invalid");
                *coordinate_valid_flag = INVAL; 
            }
         
            /* The ModelTransformationTag (which has 16 values) was found
               so copy it to the geotrans array */
            for (index = 0; index < 16; index++) 
            {
                geotrans[index] = transformation_tag_ptr[index];
            }
        }
        else
        {
            /* The ModelTiepointTag was found (which has 6 values) so copy it
               the tiepoints array */
            for (index = 0; index < 6; index++)
            {
                tiepoints[index] = tiepoints_tag_ptr[index];
            }  
        }
    }

    /* Since pixel does not have a value, ModelPixelScaleTag does not exist*/
    else 
    { 
        /* Look for ModelTransformationTag, this must exist for processing to
           continue because the pixel size is required */
        status = TIFFGetField(in, TIFFTAG_GEOTRANSMATRIX, &temp,
                              &transformation_tag_ptr);
        if (status == 0)
        {
            /* Since ModelTransformationTag does not exist, then the
               pixel size and corner coordinates could not be retrieved.
               Issue a fatal error.  */
            c_lasmsg(NONFATAL, "coordinates-fatal", "ModelTransformationTag " 
                    "does not exist in geoTIFF file. Unable to retrieve the "
                    "corner coordinates and projection distance");
            return (E_FAIL);
        }
        else
        { 
            /* The ModelTransformationTag (which has 16 values) was found so 
               copy it to the geotrans array */
            for (index = 0; index < 16; index++) 
            {
                geotrans[index] = transformation_tag_ptr[index];
            }
        }
    }
    return(E_SUCC);
}
