/*************************************************************************
NAME: GTIF2DDR

PURPOSE:
     Update the the current ddr (already ingested through TRANSLAS-IN) with 
     the geographic information which will be obtained from geoTIFF.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

NOTES:
  1. This routine will also be called if a window was specified and its
     units are not in LS. If the units are either DEG, DMS or PRO then 
     convert them to LS. 
  2. The variable rasterType indicates whether "pixel" is defined as a
     point or an area.  Point references the center of a pixel and area 
     references the upper left corner of a pixel.
**************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "geotiffin.h"

void main
(void)  
{
    struct PARBLK vblk;  /* TAE parameter block                              */
    TIFF *in;            /* Input file pointer (tiff tags)                   */ 
    GTIF *gtif;          /* Input file pointer (gtif keys)                   */ 
    char cd_dpath[CMLEN]; /* path to CD                                      */
    char *citation;      /* Ascii key used for the projection name           */
    char coorunits[4];   /* Units of the window                              */
    char infile[MAX_FILE][CMLEN];/* Input TIFF file                          */
    char outf[MAX_FILE][CMLEN];  /* Output DDR file                          */
    char sceneid[CMLEN]; /* sceneid of geoTIFF image                         */
    char tiff_file_ary[MAX_FILE][CMLEN]; /* files retrieved from directory   */
    double pixel[2];     /* pixel size (also the resolution) of the image    */
    int mounted;         /* flag indicating if path is to a CD mounted by ops*/
    int one = 1;         /* constant                                         */
    int zero = 0;        /* constant                                         */
    int size;            /* size of key value                                */
    int status;          /* return value from key retrieval call             */
    long bands[MAXBND];  /* array of sorted bands                            */
    long cit_length;     /* length of string in citation key                 */
    long convert_window_flag;/* flag indicating a window was specified and
                                to convert its units to LS (TRUE or FALSE)   */
    long lan7;           /* flag indicating landsat 7 data                   */
    long n_files=0;      /* number of files retrieved from directory         */
    long nl;             /* num of lines in the image (no windowing applied) */
    long ns;             /* num of samples in the image (no windowing applied)*/
    long numouts;        /* number of output images                          */
    long std_out;        /* Logical unit number for std output               */
    double window[4];    /* windowed area of image, its units can be DEG, 
                            DMS, LS or PRO                                   */
    long valid_window[4];/* window values in line/sample                     */
    short coordTrans=0;  /* Projection name                                  */
    short geoType=0;     /* GeoTIFF datum key                                */ 
    short modelType=0;   /* key showing the tiff is a geographic image       */
    short projType=0;    /* key containing datum and zone for ddr            */
    short rasterType=0;  /* indicates whether pixel is defined as a point  
                            or an area                                       */
    tagtype_t type;      /* type of key                                      */

    /*  Initialize TAE interface
    -----------------------------*/
    c_inlas(&vblk,&std_out);   

    /* Retrieve the input parameters
    --------------------------------*/
    g2d_getpar(&vblk,infile,outf,bands,window,&numouts,&convert_window_flag,
            &nl,&ns,pixel,&mounted,&lan7,cd_dpath,sceneid,coorunits);  

    /* if CD, retrieve file names on the CD that is currently mounted 
       The first geoTIFF file is used to get the common geographic information 
    -------------------------------------------------------------------------*/
    if (mounted == 1) 
    {
        if (c_getfiles(cd_dpath,tiff_file_ary,&n_files,MAX_FILE,1,0) != E_SUCC)
        {
            c_lasmsg(FATAL, "gtif2ddr-getfile", 
                    "Error retrieving list of files from CD");
        }
        strcpy(infile[0],tiff_file_ary[0]);
    }

    /* Open the TIFF file
    ----------------------*/
    in = XTIFFOpen((const char *)infile, "r");
    if (in == NULL)
        c_lasmsg(FATAL,"gtif2ddr-open","Error opening input image %s", infile); 


    /* Open GTIF Key parser; keys will be read at this time.
    -------------------------------------------------------*/
    gtif = GTIFNew(in);
    if(in == NULL)
        c_lasmsg(FATAL, "gtif2ddr-open", "Error opening input image");

    /* Retrieve keys that help define the projection
    ----------------------------------------------*/
    GTIFKeyGet(gtif, ProjCoordTransGeoKey, &coordTrans, zero, one);
    GTIFKeyGet(gtif, GeographicTypeGeoKey, &geoType, zero, one);
    GTIFKeyGet(gtif, ProjectedCSTypeGeoKey, &projType, zero, one);
    GTIFKeyGet(gtif, GTModelTypeGeoKey, &modelType, zero,one);

    cit_length = GTIFKeyInfo(gtif,PCSCitationGeoKey,&size,&type);
    if (cit_length > 0)
    {
        citation = malloc(size*cit_length);
        GTIFKeyGet(gtif, PCSCitationGeoKey, citation, zero, cit_length);
    }

    /* 32767 is key for User-Defined */
    if ((coordTrans == 32767) && (cit_length > 0))
    {
        c_up2low(citation, &cit_length);
        if ((strncmp(citation,"somb",4)==NULL) ||
                (strncmp(citation,"space oblique mercator b",24) == NULL))
            coordTrans = 50;
    }

    status = GTIFKeyGet(gtif, GTRasterTypeGeoKey, &rasterType, zero, one);
    if (status == 0)
        c_lasmsg(FATAL, "gtif2ddr-key", 
                "Error retrieving key GTRasterTypeGeoKey"); 

    /* Now that all the tags common to any geoTIFF have been retieved,
       get the projection specified tags and keys
    -------------------------------------------------------------------*/
    if ((coordTrans == 0) && (modelType == 2)) /* 2 = geographic */
    {
        geographic(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
    } 

    else if (((coordTrans == 0) && (geoType != 0))  
            || ((projType <= 32760) && (projType >= 32601)) 
            || ((projType <= 32360) && (projType >= 32201)) 
            || ((projType <= 26923) && (projType >= 26903))
            || ((projType <= 26722) && (projType >= 26703))
			//||((projType == 26700) || (projType == 26900) 
			//|| (projType == 32200) || (projType == 32600))
			)
        utm(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);

    else if ((coordTrans == 0) && (geoType == 0) 
            || ((projType <= 32060) && (projType >= 32001))
            || ((projType <= 26798) && (projType >= 26729))
            || ((projType <= 32161) && (projType >= 26998))
            || ((projType <= 32100) && (projType >= 26929))) 
        stpl(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);

    else if ((coordTrans == 0) && (projType != 0))
        c_lasmsg(FATAL,"geotiffin-datum", 
                "GeoTIFF does not contain valid datum code");
    else
    { 
        switch (coordTrans)
        {
            case 1: 
                tm(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break; 
            case 2:
                alaska(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 3:
                om(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 7:
                mercat(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits); 
                break; 
            case 8:
                lamcc(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 10:
                lamaz(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;   
            case 11:
                albers(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 12:
                azmuth(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break; 
            case 13:
                equidc(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 14:
                stereo(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 15:
                ps(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 17:
                eqrect(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 19:
                gnomon(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 20:
                miller(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 21:
                ortho(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 22:
                poly(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 23:
                robin(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 24:
                snsoid(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;
            case 25:
                vgrint(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,&rasterType,valid_window,coorunits);
                break;  
            case 50:
                somb(outf,gtif,in,&numouts,window,bands,&convert_window_flag,
                        &nl,&ns,pixel,&lan7,sceneid,&rasterType,valid_window,
                        coorunits);
                break;
            default: 
                c_lasmsg(NONFATAL,"geotiffin-proj",
                        "GeoTIFF image does not contain valid projection code");
        } 
    } 

    /* Close the input file
    ------------------------*/
    free(citation);
    TIFFClose(in);
    GTIFFree(gtif);


    /* Return the valid window back to PDF if convert_window_flag is TRUE
    --------------------------------------------------------------------*/
    if (convert_window_flag == TRUE)
    {
        if (q_intg(&vblk,"OUTWINDOW",4,(int*)valid_window,P_UPDATE) != SUCCESS)
        {
            c_lasmsg(FATAL,"geotiffin-lng2tae","Error passing variable "
                    "OUTWINDOW to TAE");
        }

        if (q_out(&vblk) != SUCCESS)
        {
            c_lasmsg(FATAL,"geotiffin-qout","Error passing variables to TAE");
        }
    }
}
