/*************************************************************************
NAME: POLY

PURPOSE:
	Retrieve the Polyconic geographic information geoTIFF keys. 

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "geompak.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "datum.h"
#include "geotiffin.h"

int poly

(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was 
                                      specified and to convert its 
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],		/* I: resolution of image		*/
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
)
 
{
struct DATUMDEF datuminfo;      /* structure for datum information      */

/* variables needed for geoTIFF keys */
double centMer = 0.0;           /* longitude of central meridian        */
double falseEast = 0.0;         /* false easting                        */
double falseNorth = 0.0;        /* false northing                       */
double geotrans[16];		/* model transformation corner coordinates */
double originLat = 0.0;         /* latitude of projection origin        */
double recipFlat = 0.0;         /* recipricol flattening                */
double semiMajor = 0.0;         /* semi major axis                      */
double semiMinor = 0.0;         /* semi minor axis                      */
double tiepoints[6];              /* corner coordinates values            */
short angularUnits=0;           /* angular unit of measurement          */
short geoType = 0;		/* geoTIFF key for datum		*/
unsigned short datumGeoKey = 0; /* geoTiff key for datum */
short ellipsType = 0;           /* key for ellipsoid (set by MRT)       */
short linearUnits=0;            /* linear unit of measurement           */

/* geoTIFF variables, will stay zero */
double parallel1 = 0.0;         /* latitude of 1st standard parallel    */
double parallel2 = 0.0;         /* latitude of 2nd standard parallel    */

/* for output to ddr */
long datum;			/* GCTP datum code                       */
long proj_code = 7;		/* GCTP projection code                     */
long zone_code = 62;		/* UTM or State Plane code, default is 62   */

/* for Oblique Merc. A only, set equal to zero */
double Lat = 0.0;
double Lon = 0.0;
double falseLat = 0.0;
double falseLon = 0.0;
double num = 0.0;

/* other variables   */
double dms = 0.0;               /* packed degrees/minutes/seconds       */
double lat = 0.0;               /* temporary number for latitude        */
double lon = 0.0;               /* temporary number of longitude        */
int  index;                     /* variable used for looping            */
int one = 1;                    /* constant                             */
int zero = 0;                   /* constant                             */
long datumvalid = 0;            /* flag indicating a valid datum or not */
long ddr_valids[8];             /* array containing valids or invalids  */
long status = 0;                /* Function return status               */

/* Set all the valid flags to VAILD
----------------------------------*/
for (index = 0; index < 8; index++)
    ddr_valids[index] = VALID;

/* Get corner coordinates
------------------------*/
if (get_coordinates(in, pixel[0],&ddr_valids[6], tiepoints, geotrans) != E_SUCC)
   c_lasmsg(FATAL, "poly-coords", "Error retrieving corner coordinates");

/* Get other keys
-----------------*/
GTIFKeyGet(gtif, GeographicTypeGeoKey, &geoType, zero, one);
GTIFKeyGet(gtif, GeogGeodeticDatumGeoKey, &datumGeoKey, zero, one);
GTIFKeyGet(gtif, GeogEllipsoidGeoKey, &ellipsType, zero, one);
GTIFKeyGet(gtif, GeogSemiMajorAxisGeoKey, &semiMajor, zero, one);
GTIFKeyGet(gtif, GeogSemiMinorAxisGeoKey, &semiMinor, zero, one); 
GTIFKeyGet(gtif, GeogInvFlatteningGeoKey, &recipFlat, zero, one); 

/* If the GeographicTypeGeoKey is zero (undefined); or the TypeGeoKey,
   DatumGeoKey, and EllipsoidGeoKey are user defined (meaning it is likely a
   ERDAS product without a datum);  or the datum from GeographicTypeGeoKey is
   not recognized, use the semiMajor and semiMinor axes */
if ((geoType == 0) || ((geoType == KvUserDefined ) 
        && (datumGeoKey == KvUserDefined) && (ellipsType == KvUserDefined))
    || (get_datum(&geoType,&datum) != E_SUCC))
{
	/* If this is a sphere and the semi-minor axis was not provided,
	   set its value to the semi-major axis (7035 indicates sphere)
	----------------------------------------------------------------*/
	if (semiMajor != 0.0 && semiMinor == 0.0 && ellipsType == 7035)
	   semiMinor = semiMajor;
  
	/* Check if the semi-major, semi-minor or recipFlat is defined.
	If not, grab this information from supplied from datum.txt
	------------------------------------------------------------*/
	if ((semiMinor == 0.0) && (semiMajor == 0.0) && (recipFlat == 0.0))
	   {
		if (get_datum(&geoType,&datum) != E_SUCC)
		   c_lasmsg(FATAL,"geotiffin-datum","Fatal error encountered");
		datuminfo.datumnum = datum;
		if (c_getdatum(&datuminfo) != E_SUCC)
		   c_lasmsg(FATAL,"geotiffin-datum",
			"Datum number %ld does not exist in LASTABLES",datum);
		semiMajor = datuminfo.smajor;
		semiMinor = datuminfo.sminor;
	   }

	/* Check if the semi-minor axis is defined or not
	-----------------------------------------------*/
	else if (semiMinor == 0.0)
	   {
	   /* Whether recipricol flattening is defined or not compute
	   the minor axis, if semi minor is 0.0 then it may be a sphere
	   ---------------------------------------------------------------*/
	   c_recip2minor(&recipFlat,&semiMajor,&semiMinor);
	   if (c_get_spheroid(semiMajor,semiMinor,&datum,&datumvalid) != E_SUCC)
		  {
		  c_lasmsg(FATAL,"geotiffin-datum","Fatal error encountered"); 
		  }
	   if (!datumvalid)
		  {
		  c_lasmsg(NONFATAL,"geotiffin-datum","Warning: datum is invalid");
		  ddr_valids[2] = INVAL;  /* set datum code to invalid */
		  }
	   }
	else
	   {
	   if (c_get_spheroid(semiMajor,semiMinor,&datum,&datumvalid) != E_SUCC)
		  {
		  c_lasmsg(FATAL,"geotiffin-datum","Fatal error encountered"); 
		  }
	   if (!datumvalid)
		  {
		  c_lasmsg(NONFATAL,"geotiffin-datum","Warning: datum is invalid");
		  ddr_valids[2] = INVAL;  /* set datum code to invalid */
		  }
	   }
}
/* Get projection information
-----------------------------*/
status = GTIFKeyGet(gtif, ProjNatOriginLongGeoKey, &centMer, zero, one);
if (status == 0) /* module las2geotiff uses this key instead */
    {
    status = GTIFKeyGet(gtif, ProjCenterLongGeoKey, &centMer, zero, one);
    if (status == 0)
       {
       c_lasmsg(NONFATAL, "poly-warn",
                "ProjNatOriginLongGeoKey does not exist in geoTIFF file");
       ddr_valids[3] = INVAL;
       }
    }
 
status = GTIFKeyGet(gtif, ProjNatOriginLatGeoKey, &originLat, zero, one);
if (status == 0)
    {
    status = GTIFKeyGet(gtif, ProjFalseOriginLatGeoKey, &originLat, zero, one);
    if (status == 0)
        {
        c_lasmsg(NONFATAL, "poly-warn", "Neither ProjNatOriginLatGeoKey or "
                 "ProjFalseOriginLatGeoKey exist in geoTIFF file");
        ddr_valids[3] = INVAL;
	}
    }

status = GTIFKeyGet(gtif, ProjFalseEastingGeoKey, &falseEast, zero, one);
if (status == 0)
    {
    c_lasmsg(NONFATAL, "poly-warn",
             "ProjFalseEastingGeoKey does not exist in geoTIFF file");
    ddr_valids[3] = INVAL;
    }

status = GTIFKeyGet(gtif, ProjFalseNorthingGeoKey, &falseNorth, zero, one);  
if (status == 0)
    {
    c_lasmsg(NONFATAL, "poly-warn",
             "ProjFalseNorthingGeoKey does not exist in geoTIFF file");
    ddr_valids[3] = INVAL;
    }

status = GTIFKeyGet(gtif, ProjLinearUnitsGeoKey, &linearUnits, zero, one);
if (status == 0)
   {
   status = GTIFKeyGet(gtif, GeogLinearUnitsGeoKey, &linearUnits, zero, one);
   if (status == 0)
      {
      c_lasmsg(NONFATAL, "poly-warn",
              "LinearUnits does not exist in geoTIFF file");
      ddr_valids[4] = INVAL;   /* set ground units to invalid */
      }
   }

status = GTIFKeyGet(gtif, GeogAngularUnitsGeoKey, &angularUnits, zero, one);
if (status == 0)
    {
    c_lasmsg(NONFATAL, "poly-warn",
             "GeogAngularUnitsGeoKey does not exist in geoTIFF file");
    c_lasmsg(NONFATAL, "poly-warn", "Assuming angular units to be degrees");
    c_lasmsg(NONFATAL, "poly-warn", "Projection parameters may be invalid");

    /* Assuming degrees so change the value and flag proj. params VALID */
    angularUnits = 9102;
    }

/* if angularUnits equals angular degree then convert the degree angle to DMS 
---------------------------------------------------------------------------*/
if (angularUnits == 9102) 		/* 9102 equals Angular_Degree */
   {
   /* set longitude of the central meridian
   ---------------------------------------*/
   lon = centMer;
   status = c_degdms(&lon,&dms,"DEG","LON");
   if (status != E_SUCC)
      {
      c_lasmsg(NONFATAL, "poly-warn",
               "Error converting degrees to DMS for ProjNatOriginLongGeoKey");
      ddr_valids[3] = INVAL;
      }
   else
      centMer = dms;

   /* set latitude of the projection origin
   --------------------------------------*/
   lat = originLat;
   status = c_degdms(&lat,&dms,"DEG","LAT");
   if (status != E_SUCC)
      {
      c_lasmsg(NONFATAL, "poly-warn",
               "Error converting degrees to DMS for ProjNatOriginLatGeoKey");
      ddr_valids[3] = INVAL;
      }
   else
      originLat = dms;
   }

/* write to ddr
--------------*/
if(update_ddr(outf,proj_code,zone_code,datum,semiMajor,semiMinor,parallel1,
           parallel2,centMer,originLat,falseEast,falseNorth,
           Lon,Lat,falseLon,falseLat,num,linearUnits,tiepoints,numouts,window,
           bands,convert_window_flag,nl,ns,pixel,lan7,geotrans,ddr_valids,
           rasterType,valid_window,coorunits) != E_SUCC)
   c_lasmsg(FATAL,"geotiffin-ddr","Fatal error encountered");

return(E_SUCC);
}
