/*************************************************************************
NAME: PROCESS_BAND

PURPOSE:
     Read in image files from CDs or disk that are in geoTIFF format. 
     Convert the geoTIFF(strip/tile) files to a flat file suitable to 
     ingest through TRANSLAS-IN.  

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS


**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "geotiffin.h"
 
/* Macro used to quickly compute the number of an (x,y) tile
   given the number of tiles across (a).
------------------------------------------------------------*/
#define GetTileNum(x,y,a) (long)((long)(x)+(long)(y)*(long)(a))

int process_band
(
    char infile[],	   /* I: tiff file to be processed		    */ 
    char outfile[],	   /* O: Output BIL directory path and file         */
    double pixel[],	   /* O: resolution of the image		    */ 
    uint16 *bitspersample, /* O: input data type			    */
    tsize_t *image_width,  /* O: number of samples			    */
    tsize_t *image_length, /* O: number of lines			    */
    tsample_t *nbands,     /* O: number of bands in the image	            */ 
    long *real_flag,       /* O: Flag for floating-point data 1=true        */
    char outpath[] 	   /* I: directory path of the output image used
				 for the output path of the  BIL file	    */ 
)

{
   TIFF *in; 		/* Input file pointer                               */
   FILE *out;		/* Output file pointer                              */
   char directory[CMLEN];/* Directory portion of infile			    */
   char extension[CMLEN];/* Extension portion of file			    */
   char root[CMLEN];    /* Root portion of file				    */
   double *geopixel;    /* pixel size					    */ 
   double *geotrans;    /* model transformation corner coordinates          */
   double *temp;        /* used to extract geoTIFF tag                      */
   double *tiepoints;   /* corner coordinates values                        */
   long bnd;		/* Band counter					    */
   long i;		/* Looping variable				    */
   long j;		/* Looping variable			            */
   long k;              /* Looping variable                                 */
   long line;		/* Line counter                                     */
   long result;         /* Return result from TIFFReadEncodedTile(...)      */
   long status;		/* function return status			    */
   tsample_t config=0;  /* The planar configuration in the image (bip, bil) */ 
   tsize_t last_tile_length_used;  /* The number of lines in the last tile  */
   tsize_t last_tile_width_used;   /* The number of samples in the last tile*/
   tsize_t tile_length; /* The number of rows in a tile                     */
   tsize_t tile_rowsize;/* The number of bytes/samples per row in a tile    */
   tsize_t tile_size;   /* The number of bytes in a tile                    */
   tsize_t tiles_across;/* The number of tiles across for the image         */
   tsize_t tiles_down;  /* The number of tiles down for the image           */
   tsample_t rdata;     /* Datatype check for real (FP) data                */
   unsigned char *inbuf=NULL;/* Processing buffer for strip geoTIFF images  */
   unsigned char *bip2bil_buf=NULL;  /* buffer for bip2bil to use           */  
   unsigned char **tile_data; /* Used for allocating the tiles across  */

	/* Open the TIFF file   
	--------------------*/
   in = XTIFFOpen(infile, "r");  
   if(in == NULL)
      c_lasmsg(FATAL, "geotiffin-open", "Error opening input image %s",infile);

	/* Check the data type
	-----------------------*/
   TIFFGetField(in,TIFFTAG_BITSPERSAMPLE,bitspersample);

   if ((*bitspersample!=8) && (*bitspersample!=16) && (*bitspersample!=32))
   {
      c_lasmsg(FATAL,"geotiffin-bit","Only handle 8, 16, and 32-bit images"); 
   }

	/* Get pixel size found in the ModelPixelScaleTag (TIFFTAG_GEOPIXELSCALE),
	if geopixel does not exist then check the ModelTransformationTag
	(TIFFTAG_GEOTRANSMATRIX). ModelTransformationTag contains both pixel and
	corner coordinate information
	---------------------------------------------------------------------------*/
   *pixel = 0.0;
   status = TIFFGetField(in, TIFFTAG_GEOPIXELSCALE, &temp, &geopixel);
   if (status != 0) 
   {  
      pixel[0] = geopixel[0];
      pixel[1] = geopixel[1];
   } 
   else 
   {
      status = TIFFGetField(in, TIFFTAG_GEOTRANSMATRIX, &temp, &geotrans);
      {
         if (status == 0)
         {
            c_lasmsg(FATAL,"geotiffin-pixel","Pixel size does not exist"); 
         }
      }
   }

	/* if ModelPixelScaleTag exists then ModelTiepointTag (TIFFTAG_GEOTIEPOINTS)
	must exist in order to retrieve the corner coordinates 
	----------------------------------------------------------------------*/
   if (pixel[0] != 0.0)
   { 
      status = TIFFGetField(in, TIFFTAG_GEOTIEPOINTS, &temp, &tiepoints);
      if (status == 0)
      {
         c_lasmsg(FATAL,"geotiffin-coors","Corner coordinates do not exist"); 
      } 
   }
 
	/* Extract the buffer length from TIFF file
	-------------------------------------------*/
   status = TIFFGetField(in, TIFFTAG_IMAGELENGTH, image_length);
   if (status == 0) 
   {
      c_lasmsg(FATAL,"geotiffin-lines","Error retrieving number of lines");
   }

   TIFFGetField(in, TIFFTAG_PLANARCONFIG, &config);
   if (status == 0)
   {
      c_lasmsg(FATAL,"geotiffin-config",
              "Error retrieving Planar Configuration type");
   }

   TIFFGetField(in, TIFFTAG_SAMPLEFORMAT, &rdata);
   if (status == 0)
   {
      c_lasmsg(FATAL,"geotiffin-dtype",
              "Error retrieving Sample Format");
   }
   else
   {
       if( rdata == SAMPLEFORMAT_IEEEFP )
       {
           *real_flag = 1;
       }
       c_lasmsg(NONFATAL,"Informational","Input data format: %d", rdata);
   }

   TIFFGetField(in, TIFFTAG_IMAGEWIDTH,image_width);
   if (status == 0)
   {
      c_lasmsg(FATAL,"geotiffin-samples","Error retrieving number of samples");
   }

   TIFFGetFieldDefaulted(in, TIFFTAG_SAMPLESPERPIXEL,nbands);
   if (status == 0)
      {
      c_lasmsg(FATAL,"geotiffin-bands",
                 "Error retrieving total number of bands");
      }

	/* Create the output bil file name
	---------------------------------*/
   c_splitname(infile,directory,root,extension);

   strcpy(outfile,outpath);
   strcat(outfile,root);
   strcat(outfile,".bil");

	/* Let user know which "bil" file is being processed
	---------------------------------------------------*/
   c_lasmsg(NONFATAL,"Informational","Creating output file: %s", outfile);

	/* Open the output bil file
	--------------------------*/
   out = fopen(outfile, "wb");
   if(out == NULL)
      c_lasmsg(FATAL,"geotiffin-open","Error opening output image %s", 
               outfile);

/* Check to see if the geoTIFF image is tiled or not
-------------------------------------------------*/
   if(!TIFFIsTiled(in))
   {
      int scanlinesize;  /* value returned by TIFFScanlineSize */
      scanlinesize = TIFFScanlineSize(in);

	 /* Copy the strip image data to the output flat file
	 ----------------------------------------------------*/		

	 /* Allocate the memory needed for a line
	 ----------------------------------------*/
      inbuf = malloc(scanlinesize * sizeof(char));
      if(inbuf == NULL)
         c_lasmsg(FATAL, "geotiffin-mem", "Error allocating memory for inbuf");


   /* Process the strip image
   --------------------------*/

      /* TIFF image is BIL
      --------------------*/
      if (config == PLANARCONFIG_SEPARATE)
      {
         for(line = 0; line < *image_length; line++)
         {
            for (bnd = 0; bnd < *nbands; bnd++ )
            {
               if(TIFFReadScanline(in, inbuf, line, (unsigned char)bnd) < 0)
               {
				  free(inbuf);
                  c_lasmsg(FATAL,"geotiffin-read",
                           "Error reading geoTIFF image %s", infile);
               }
               if (fwrite(inbuf,scanlinesize,1,out) != 1) 
               {
                  fclose(out);
                  c_lasmsg(FATAL,"geotiffin-file","Error writing to %s",
                           outfile);
               }
           }
        }
     }
      /* TIFF image is BIP
      --------------------*/
      else if (config == PLANARCONFIG_CONTIG)
      {
         bip2bil_buf = malloc(scanlinesize * sizeof(char));
         if (bip2bil_buf == NULL)
         {
            c_lasmsg(FATAL, "geotiffin-mem", 
                     "Error allocating memory for bip2bil_buf");            
         }
         for(line = 0; line < *image_length; line++)
         {
            if (TIFFReadScanline(in, inbuf, line,0) < 0)
            {
               free(inbuf);
               c_lasmsg(FATAL,"geotiffin-read",
                          "Error reading geoTIFF image %s",infile);
            }
            bip2bil(inbuf, bip2bil_buf, scanlinesize, *nbands);
            if (fwrite(inbuf, 1, scanlinesize, out) != scanlinesize)
            {
               fclose(out);
               c_lasmsg(FATAL,"geotiffin-file","Error writing to %s",
                        outfile);
            }
         }
         free (bip2bil_buf);
         bip2bil_buf = NULL;
      }
     
   /* Free the memory buffer
   -------------------------*/
      free(inbuf);
   }
   else
   {
      int image_width_all_bands; /* width of all bands together */

      /* Copy the single band tiled image data to the output flat file
      ----------------------------------------------------------------*/

      /* Do some initialization
      -------------------------*/
      tile_rowsize = TIFFTileRowSize(in);
      tile_size = TIFFTileSize(in);
      tile_length = tile_size / tile_rowsize;
      tiles_down = (*image_length + (tile_length - 1)) / tile_length;
      last_tile_length_used = *image_length % tile_length;
      image_width_all_bands = *image_width * (tsize_t)*nbands;

      if (config == PLANARCONFIG_CONTIG)
      {
         /* multiple bands in contigous layout, so multiply the image_width
            of all bands to get the size of each row */
         tiles_across = (image_width_all_bands + (tile_rowsize - 1)) /
                            tile_rowsize;
         last_tile_width_used = image_width_all_bands % tile_rowsize;
         bip2bil_buf = malloc(sizeof(char) * image_width_all_bands);
         if (bip2bil_buf == NULL)
            c_lasmsg(FATAL, "geotiffin-mem", 
                          "Error allocating memory for bip2bil_buf");
      } else
      {
         if (*nbands > 1)
         {
            /* The tiff spec does not give information on how to handle
               PLANARCONFIG_SEPARATE for multiple bands, so until we encounter
               an image with it, we'll just error out */
            c_lasmsg(FATAL,"geotiffin-notsup","The image is a tiled, multiband "
                     "image which is not supported");
         }
         tiles_across = (*image_width + (tile_rowsize - 1)) / tile_rowsize;
         last_tile_width_used = *image_width % tile_rowsize;
      }


      /* Allocate memory for the number of tile buffers needed
      --------------------------------------------------------*/
      tile_data = malloc(sizeof(unsigned char *) * tiles_across);
      if (tile_data == NULL)
      {
         c_lasmsg(FATAL, "geotiffin-mem", 
                     "Error allocating memory for tile_data");
      }
      for(i = 0; i < tiles_across; i++)
      {
         tile_data[i]=(unsigned char *) malloc(sizeof(char)*tile_size);
         if(tile_data[i] == NULL)
         {
            c_lasmsg(FATAL, "geotiffin-mem", 
                       "Error allocating memory for tile_data");
         }
      }
      inbuf = malloc(sizeof(char) * tile_rowsize * tiles_across);
      if (inbuf == NULL)
         {
         c_lasmsg(FATAL, "geotiffin-mem", "Error allocating memory for inbuf");
         }
      memset (inbuf,0,image_width_all_bands);
 
      /* For all but the bottom row of tiles
      --------------------------------------*/
      for(j = 0; j < tiles_down-1; j++)
      {
         /* Read in the tiles needed to fill a row
         -----------------------------------------*/
         for(i = 0; i < tiles_across; i++)
         {
            result = TIFFReadEncodedTile(in,GetTileNum(i,j,tiles_across),
                     tile_data[i],tile_size);
            if(result == -1)
               c_lasmsg(FATAL, "geotiffin-read", 
                        "Error reading geoTIFF image %s", infile);
         }
         /* Process the lines in the current row of tiles
         ------------------------------------------------*/
         if (config == PLANARCONFIG_CONTIG)
         {
            for (line = 0; line < tile_length; line++)
            {
               for (i = 0; i < tiles_across; i++)
               {
                  memcpy(&inbuf[i * tile_rowsize],
                         &tile_data[i][line * tile_rowsize],tile_rowsize);
               }
               bip2bil(inbuf,bip2bil_buf,image_width_all_bands,*nbands);
               if (fwrite(inbuf,1,image_width_all_bands,out) !=
                           image_width_all_bands)
               {
                  c_lasmsg(FATAL,"geotiffin-write", 
                                 "Error writing output image %s", outfile);
               }
            }
         }
         else
         {
            for(k = 0; k < tile_length; k++)
            {
               for(i = 0; i < tiles_across-1; i++)
               {
                  if (fwrite(&tile_data[i][k*tile_rowsize],1,tile_rowsize,out)
                        != tile_rowsize)
                  {
                     fclose(out);
                     c_lasmsg(FATAL,"geotiffin-write", 
                                "Error writing output image %s", outfile);
                  }
               }
               if (fwrite(&tile_data[i][k*tile_rowsize],1,last_tile_width_used,
                           out) != last_tile_width_used)
               {   
                  c_lasmsg(FATAL,"geotiffin-file","Error writing to %s",
                           outfile);
               }
            }
         }
      }

      /* Read in the bottom row of tiles
      ----------------------------------*/
      for(i = 0; i < tiles_across; i++)
      {
         result = TIFFReadEncodedTile(in,GetTileNum(i,j,tiles_across),
                  tile_data[i],tile_size);
         if(result == -1)
         {
            c_lasmsg(FATAL, "geotiffin-read", 
                           "Error reading geoTIFF image %s", infile);
         }
      }
      /* Process the lines used in the bottom row of tiles
      ----------------------------------------------------*/ 
      if (config == PLANARCONFIG_CONTIG)
      {
         for (line = 0; line < last_tile_length_used; line++)
         {
            for (i = 0; i < tiles_across; i++)
            { 
               memcpy(&inbuf[i * tile_rowsize],
                      &tile_data[i][line * tile_rowsize],tile_rowsize);
            }
            bip2bil(inbuf,bip2bil_buf,image_width_all_bands, *nbands);
            if (fwrite(inbuf,1,image_width_all_bands,out) !=
                      image_width_all_bands)
            {
               c_lasmsg(FATAL,"geotiffin-write", 
                         "Error writing to %s", outfile);
            }
         }
      } 
      else 
      {
         for(k = 0; k < last_tile_length_used; k++)
         {
            for(i = 0; i < tiles_across-1; i++)
            {
               if (fwrite(&tile_data[i][k*tile_rowsize],1,tile_rowsize,out) !=
                            tile_rowsize)
               {
                  fclose(out);
                  c_lasmsg(FATAL,"geotiffin-file","Error writing to %s",
                           outfile);
               }
	       }
            if (fwrite(&tile_data[i][k*tile_rowsize],1,last_tile_width_used,out)
                      != last_tile_width_used) 
            {
               fclose(out);
	           c_lasmsg(FATAL,"geotiffin-file",
                          "Error writing to %s", outfile);
			} 
         }
     
	  }

      /* Free the tile memory buffers
      -------------------------------*/
      for(i=0; i < tiles_across; i++)
         free(tile_data[i]);
      free(tile_data);
      free(inbuf);
      free(bip2bil_buf);
   }

 
   /* Close the input and output files
   -----------------------------------*/
   TIFFClose(in); 
   fclose(out);  

   return(E_SUCC);
}
