/*************************************************************************
NAME: STPL

PURPOSE:
	Retrieve the State Plane geographic information geoTIFF keys. 

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "geompak.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "geotiffin.h"

int stpl

(
    char outf[][CMLEN],         /* I: output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer 	*/
    TIFF *in,                   /* I: Input tiff file pointer     	*/
    long *numouts,		/* I: number of output images    	*/
    double window[],		/* I: windowed area of image    	*/
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was 
                                      specified and to convert its 
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],		/* I: resolution of image		*/
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
) 

{
/* variables needed for geoTIFF keys */
double tiepoints[6];            /* corner coordinates values            */
short linearUnits=0;            /* linear unit of measurement           */

/* variables needed for proj parms, stpl does not have any, default to 0*/
double Lat = 0.0;
double Lon = 0.0;
double centerLon = 0.0;
double falseEast = 0.0;
double falseLat = 0.0;
double falseLon = 0.0;
double falseNorth = 0.0;
double geotrans[16];
double natOrigin = 0.0;
double num = 0.0;
double parallel1 = 0.0;
double parallel2 = 0.0;
double semiMajor = 0.0;
double semiMinor = 0.0;

/* for output to ddr */
long datum;			/* GCTP datum code                     */
long proj_code = 2;		/* GCTP projection code                */
long zone_code;			/* State Plane zone code	       */

/* other geoTIFF key stuff */
short projType = 0;		/* key containing datum and zone       */

/* other variables */
int  index;                     /* variable used for looping            */
int one = 1;                    /* constant                             */
int zero = 0;                   /* constant                             */
long ddr_valids[8];             /* array containing valids or invalids  */
long status;                    /* function return status               */

/* Set all the valid flags to VAILD
----------------------------------*/
for (index = 0; index < 8; index++)
    ddr_valids[index] = VALID;

/* Get corner coordinates
------------------------*/
if (get_coordinates(in, pixel[0],&ddr_valids[6], tiepoints, geotrans) != E_SUCC)
   c_lasmsg(FATAL, "stpl-coords", "Error retrieving corner coordinates");

/* Get other keys
-----------------*/
status = GTIFKeyGet(gtif, ProjLinearUnitsGeoKey, &linearUnits, zero, one);
if (status == 0)
{
   status = GTIFKeyGet(gtif, GeogLinearUnitsGeoKey, &linearUnits, zero, one);
   if (status == 0)
   {
      c_lasmsg(NONFATAL, "stpl-warn",
               "LinearUnits does not exist in geoTIFF file");
      ddr_valids[4] = INVAL;   /* set ground units to invalid */
   }
}


GTIFKeyGet(gtif, ProjectedCSTypeGeoKey, &projType, zero, one); 

/* Get zone, datum and units. Get units only if linearUnits key was not 
   specified
-----------------------------------------------------------------------*/

/* keys beginning with 267 or 320 represent NAD27*/
if(((projType >= 26729) && (projType <= 26798)) || 
  ((projType >= 32001) && (projType <= 32060)))
{
   datum = 225;
   get_stateplanezone(&projType,&zone_code,&linearUnits,ddr_valids);
}

/* keys beginning with 269 or 321 represent NAD83*/
else if (((projType >= 26929) && (projType <= 26998)) ||
((projType >= 32100) && (projType <= 32161)))
{
   datum = 219;     
   get_stateplanezone(&projType,&zone_code,&linearUnits,ddr_valids);      
}
else
   c_lasmsg(FATAL,"geotiffin-stpl",
		"geoTIFF does not contain a vaild zone code");


/* WRITE TO DDR HERE
---------------------*/ 
if(update_ddr(outf,proj_code,zone_code,datum,semiMajor,semiMinor,parallel1,
           parallel2,centerLon,natOrigin,falseEast,falseNorth,
           Lon,Lat,falseLon,falseLat,num,linearUnits,tiepoints,numouts,window,
           bands,convert_window_flag,nl,ns,pixel,lan7,geotrans,ddr_valids,
           rasterType,valid_window,coorunits) != E_SUCC)
   c_lasmsg(FATAL,"geotiffin-ddr","Fatal error encountered");

return(E_SUCC);
}
