/******************************************************************************

NAME: UPDATE_DDR

PURPOSE: Update the information in the ddr obtained from geoTIFF tags.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
    Check if the convert window flag is TRUE. 
        If so then set the total numbers and samples
    Get the DDR record from disk.
    Update the projection, zone, and datum codes.
    Update the projection parameters.
    Update projection units.
    Update the projetion distance/resolution.
    Update master coordinates.
    Update line and sample increments.
    Calculate corner coordinates using upper left corner.
    Set projection distance to invalid if it's less than or equal zero.
    If the convert window flag is FALSE and there is windowing
        recalculate the corners
    Set all flags to valid or invalid.
    Write the ddr to disk.
    Allocate memory for the band dependant records.
    Get bddr and write to disk.    

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "proj.h"
#include "ddr.h"
#include "geompak.h"
#include "geotiffin.h"

int update_ddr
(
    char hostddr[][CMLEN],/* I: name of the host ddr to update          */
    long proj_code,       /* I: GCTP projection code                    */
    long zone_code,       /* I: UTM or State Plane code                 */
    long datum,           /* I: GCTP datum code                         */
    double semiMajor,     /* I: semi major axis, sphere or lon/z        */
    double semiMinor,     /* I: semi minor axis                         */
    double parallel1,     /* I: standard parallel1,factor,height,satnum 
                                or shapem*/
    double parallel2,     /* I: standard parallel2,aziand,incang,path 
                                or shapen */
    double centerLon,     /* I: central long,central merc,azmthpt or asclong */
    double natOrigin,     /* I: origin latitude, true scale, center latitude */
    double falseEast,     /* I: false easting                           */
    double falseNorth,    /* I: false northing                          */
    double Lon,           /* I: long1,PSRev,angle                       */
    double Lat,           /* I: lat1,LRat                               */
    double falseLon,      /* I: long2,PFlag                             */
    double falseLat,      /* I: lat2                                    */
    double num,           /* I: value of 0 or 1                         */
    int units,            /* I: projection units                        */
    double tiepoints[6],  /* I: corner coordinates                      */
    long *numouts,        /* I: number of output images                 */
    double window[],      /* I: windowed area of image                  */
    long bands[],         /* I: array of sorted used to update BDDR     */
    long *convert_window_flag,  /* I: flag indicating a window was 
                                      specified and to convert its 
                                      units to LS (TRUE or FALSE)       */
    long *nl,             /* I: number of lines in nonwindowed image    */
    long *ns,             /* I: number of samples in nonwindowed image  */
    double pixel[],       /* I: resolution of image                     */
    long *lan7,           /* I: flag indicating landsat 7 data          */
    double geotrans[16],  /* I: model transformation corner coordinates */
    long ddr_valids[8],   /* I: array containing valids or invalids     */
    short *rasterType,    /* I: pixel definition as point or area       */
    long valid_window[],  /* O: window values in line/sample            */
    char coorunits[]      /* I: window units (DEG,DMS,PRO,LS)           */
)
 
{
    struct BDDR *bddrs;       /* local bddr structure                       */
    struct BDDR *bptr;        /* pointer to the current BDDR record         */
    struct DDR ddr;           /* local ddr structure                        */
    char *env_ptr;            /* pointer to system environment              */
    int unitcode;             /* linear units code                          */
    double corners[8];        /* image corner points (x,y) UL, UR, LL, LR   */
    double I;                 /* 1st model transformation coefficient       */ 
    double J;                 /* 2nd model transformation coefficient       */ 
    double pixel_reference;   /* reference point of the pixel               */
    float float_pixelx;       /* float value of pixel in x direction        */
    float float_pixely;       /* float value of pixel in y direction        */
    int offset;               /* term used to calculate corner coordinates  */
    long i;                   /* loop counter                               */
    long j;                   /* loop counter                               */      
    long one=1;               /* constant                                   */      
    long origsize[2];         /* original size of the image                 */

    if(*convert_window_flag == TRUE) 
    { 
        ddr.dtype = 1;     /* simply set a dummy value 1=BYTE */
        ddr.nbands = 1;    /* set a dummy value */
        ddr.nl = *nl;      /* total number of lines in the nonwindowed image */ 
        ddr.ns = *ns;      /* total number of samples in the nonwindowed image*/

        /* get the system from the DATASYS environment variable */
        env_ptr = getenv("DATASYS");
        if (env_ptr != NULL)
            strncpy(ddr.system,env_ptr,12); /* 12 length */
        else
        {
            c_lasmsg(FATAL,"geotiffin-datasys","DATASYS environment variable "
                    "not set");
        }
    } 
    else /* get the DDR */
    {
        if(E_FAIL == c_getddr((char *)hostddr,&ddr))
            c_lasmsg(FATAL,"geotiffin-fatal","DDR does not exist"); 
    }

    /* set the projection, zone and datum codes
       -----------------------------------------*/
    ddr.proj_code = proj_code;
    ddr.zone_code = zone_code;
    ddr.datum_code = datum;

    /* set projection parameters 
       --------------------------*/
    ddr.proj_coef[0] = semiMajor;
    ddr.proj_coef[1] = semiMinor;
    ddr.proj_coef[2] = parallel1;
    ddr.proj_coef[3] = parallel2;
    ddr.proj_coef[4] = centerLon;
    ddr.proj_coef[5] = natOrigin;
    ddr.proj_coef[6] = falseEast;
    ddr.proj_coef[7] = falseNorth;
    ddr.proj_coef[8] = Lon;
    ddr.proj_coef[9] = Lat;
    ddr.proj_coef[10]= falseLon;
    ddr.proj_coef[11]= falseLat;
    ddr.proj_coef[12]= num;  
    ddr.proj_coef[13]= 0.0;
    ddr.proj_coef[14]= 0.0;

    /* set projection units
       ---------------------*/
    if (units == 9001)
        unitcode = METER;
    else if (units == 9003)
        unitcode = FEET;
    else if (units == 9102)
        unitcode = DEGREE;
	else if (units == 9104)
		unitcode = SECOND;
    else
    { 
        c_lasmsg(NONFATAL,"ddr-warn",
                "GEOTIFFIN does not support the unit number "
                "%ld found in the geoTIFF file. Units set to invalid.", units); 
        ddr_valids[4] = INVAL;
    }  

    if (ddr_valids[4] == VALID)
    {
        if (c_unit2string(unitcode,ddr.proj_units) != E_SUCC)
        {
            /* c_unit2string displays the error message */
            c_lasmsg(NONFATAL,"ddr-warn", "Setting units to invalid");
            ddr_valids[4] = INVAL;
        }
    }

    /* line and sample relative to master image
       -----------------------------------------*/
    ddr.master_line = one;
    ddr.master_sample = one;

    /* line and sample increments
       ----------------------------*/
    ddr.line_inc = one;
    ddr.sample_inc = one;

    /* if pixel does not exist then use ModelTransformationTag information
       ----------------------------------------------------------------------*/
    if ((pixel[0] == 0.0) && (ddr_valids[6] == VALID)) 
    {
        /* TIFF images are referenced at either the upper left corner or the 
           center of the pixel.  The LAS ddr needs to references the center of 
           the pixel.  "pixel_reference" is a reference term in the geotrans
           equations below.  
           -------------------------------------------------------------------*/
        /* RasterPixelIsArea represents the upper left corner of a pixel.  Set
           pixel_reference to 0.5 in order to calculate the center of the pixel.
           Set offset to 0 because we do not need it for our calculations */

        if (*rasterType == RasterPixelIsArea)
        {
            pixel_reference = 0.5; 
            offset = 0;    
        }

        /* RasterPixelIsPoint represents the center of a pixel. No adjustments 
           are needed to reference the center of the pixel. Set pixel_reference
           to 0.0.  Set offset to 1 because 1 line and 1 sample needs to be 
           subtracted from the total lines and total samples */

        else /* *rasterType == RasterPixelIsPoint */
        {
            pixel_reference = 0.0;  
            offset = 1; 
        }

        I = pixel_reference;     
        J = pixel_reference;

        /*ULX*/
        ddr.upleft[1] = (I*(geotrans[0])) + (J*(geotrans[1])) + geotrans[3];
        /*ULY*/
        ddr.upleft[0] = (I*(geotrans[4])) + (J*(geotrans[5])) + geotrans[7]; 

        I = (*ns - offset) - pixel_reference;
        J = pixel_reference;

        /*URX*/
        ddr.upright[1] = (I*(geotrans[0])) + (J*(geotrans[1])) + geotrans[3];
        /*URY*/
        ddr.upright[0] = (I*(geotrans[4])) + (J*(geotrans[5])) + geotrans[7]; 

        I = pixel_reference;
        J = (*nl- offset) - pixel_reference;

        /*LLX*/
        ddr.loleft[1] = (I*(geotrans[0])) + (J*(geotrans[1])) + geotrans[3];
        /*LLY*/
        ddr.loleft[0] = (I*(geotrans[4])) + (J*(geotrans[5])) + geotrans[7]; 

        I = (*ns - offset) - pixel_reference;
        J = (*nl - offset) - pixel_reference;

        /*LRX*/
        ddr.loright[1] = (I*(geotrans[0])) + (J*(geotrans[1])) + geotrans[3];
        /*LRY*/
        ddr.loright[0] = (I*(geotrans[4])) + (J*(geotrans[5])) + geotrans[7]; 

        /* The ModelTransformationTag is used for images that require rotation 
           or shearing to place it into the standard model space. The DDR 
           requires that the projection distance/resolution be the size of the 
           pixel before it was sheared or rotated. Recalculate the pixel
           distance.
           ------------------------------------------------------------------*/
        if (c_recalc_pixel_size(&pixel[1], &pixel[0], ddr.upleft[0], 
                    ddr.loleft[0], ddr.loright[0], ddr.upleft[1],
                    ddr.loleft[1], ddr.loright[1], *nl, *ns) != E_SUCC)
        {
            c_lasmsg(FATAL, "geotiffin-pixel","Error recalculating pixel size");
        }
    }

    else /* Use the ModelTiepointTag information */
    {
        if (ddr_valids[6] == VALID)
        { 

            /* Set corner coordinates, reference the center of the corner pixel,
               not the actual corner. If coordinates are invalid set to 0.0.
               If the tiepoint does not specify the point (0,0) in raster
               space, include the offset in the calculation of the corners.
               --------------------------------------------------------------*/
            if (*rasterType == RasterPixelIsArea)
            {
                ddr.upleft[0] = tiepoints[4] + (tiepoints[1] * pixel[1]) -
                                 (pixel[1]/2);
                ddr.upleft[1] = tiepoints[3] - (tiepoints[0] * pixel[0]) +
                                 (pixel[0]/2);
                ddr.upright[0] = tiepoints[4] + (tiepoints[1] * pixel[1]) -
                                  (pixel[1]/2);
                ddr.loleft[1] = tiepoints[3] - (tiepoints[0] * pixel[0]) +
                                 (pixel[0]/2);
            }
            else /* *rasterType == RasterPixelIsPoint */
            {
                ddr.upleft[0] = tiepoints[4] + (tiepoints[1] * pixel[1]);
                ddr.upleft[1] = tiepoints[3] - (tiepoints[0] * pixel[0]);
                ddr.upright[0] = tiepoints[4] + (tiepoints[1] * pixel[1]);
                ddr.loleft[1] = tiepoints[3] - (tiepoints[0] * pixel[0]);
            }
            ddr.upright[1] = (ddr.upleft[1] +((*ns - 1)*pixel[0]));
            ddr.loleft[0] = (ddr.upleft[0] -((*nl - 1)*pixel[1]));
            ddr.loright[0] = (ddr.upleft[0] -((*nl - 1)*pixel[1]));
            ddr.loright[1] = (ddr.upleft[1] +((*ns - 1)*pixel[0])); 
        }
        else 
        {
            ddr.upleft[0] = 0.0;
            ddr.upleft[1] = 0.0;
            ddr.upright[0] = 0.0;
            ddr.upright[1] = 0.0;
            ddr.loleft[0] = 0.0;
            ddr.loleft[1] = 0.0;
            ddr.loright[0] = 0.0;
            ddr.loright[1] = 0.0;
        } 
    }


    /* Sometimes the double pixel needs to be rounded up. To help with this
       use a float value
       ---------------------------------------------------------------------*/
    float_pixely = pixel[1];
    float_pixelx = pixel[0];

    /* set projection distance/resolution
       ------------------------------------*/
    ddr.pdist_y = fabs (float_pixely);
    ddr.pdist_x = fabs (float_pixelx);

    /* If the window exists but it does not need to be converted to LS, then the
       corners need to be recalculated. This is to update the DDR for final 
       output LAS image.  This will reflect the new windowed image.
       ----------------------------------------------------------------------*/
    if ((window[0] != 0.0) && (*convert_window_flag == FALSE))
    {
        corners[0] = ddr.upleft[1];
        corners[1] = ddr.upleft[0];
        corners[2] = ddr.upright[1];
        corners[3] = ddr.upright[0];
        corners[4] = ddr.loleft[1];
        corners[5] = ddr.loleft[0];
        corners[6] = ddr.loright[1];
        corners[7] = ddr.loright[0];

        origsize[0] = *nl;  
        origsize[1] = *ns;

        recalc_corners(corners,origsize,window);

        /*copy new corner points to ddr structure */
        ddr.upleft[1] = corners[0];
        ddr.upleft[0] = corners[1];
        ddr.upright[1] = corners[2];
        ddr.upright[0] = corners[3];
        ddr.loleft[1] = corners[4];
        ddr.loleft[0] = corners[5];
        ddr.loright[1] = corners[6];
        ddr.loright[0] = corners[7];

        /* copy the lines and samples reflecting the windowed image*/
        ddr.master_line = (long)window[0];
        ddr.master_sample = (long)window[1];
        ddr.nl = (long)window[2];
        ddr.ns = (long)window[3];
    } 

    /* set all the flags to VALID or INVAL 
       --------------------------------------*/
    for (i = 0; i < 8; i++)
        ddr.valid[i] = ddr_valids[i];


    /* If convert_window_flag is TRUE, then call convert_window. Otherwise 
       write the structure to a ddr  
       --------------------------------------------------------------------*/
    if (*convert_window_flag == TRUE)
    {
        if (convert_window(&ddr, window, valid_window,coorunits) != E_SUCC)
            c_lasmsg(FATAL,"geotiffin-wind", 
                    "Error converting window coordinates");
    }
    else
    {
        /* now write the ddr to disk only if it is not a temporary ddr
           -------------------------------------------------------------*/
        for(i=0; i<*numouts; i++)
        {
            if(E_FAIL == c_putddr(hostddr[i],&ddr))
                c_lasmsg(FATAL,"geotiffin-putddr","Error writing DDR"); 
        }

        /* now allocate memory for the band dependant records
           ---------------------------------------------------*/
        if((bddrs = (struct BDDR *) malloc((*numouts)*ddr.nbands*
                        sizeof(struct BDDR))) == NULL)
            c_lasmsg(FATAL,"geotiffin-mem","Error allocating dynamic memory.");

        /* get bddr and write to disk
           ----------------------------*/
        bptr = bddrs;

        for (j=0; j<*numouts; j++)
        {
            for (i=1; i<=ddr.nbands; i++)
            {
                /*initializes band data record w/ zeros & blanks */
                c_intbdr(bptr); 
                bptr->bandno = i;
                bptr++;
            }
        }

        bptr = bddrs;

        for (j=0; j<*numouts; j++)
        {
            for (i=0; i<ddr.nbands; i++)
            {
                if (*lan7 == 1)
                    strcpy(bptr->source, "Landsat 7");
                if (bands[i] != 0)
                    sprintf(bptr->instrument, "BAND_%ld", bands[i]);
                if(E_FAIL == c_putbdr(hostddr[j],bptr))
                    c_lasmsg(FATAL,"geotiffin-putbdr","Error writing BDDR");
                bptr++;
            }
        }
    }
    free(bddrs);
    return(E_SUCC);
}
