/*************************************************************************
NAME: UTM

PURPOSE:
	Retrieve the Universal Transverse Mercator geographic information 
	geoTIFF keys. 

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "geotiffin.h"

int utm

(
    char outf[][CMLEN],         /* I: output DDR file          		*/
    GTIF *gtif,                 /* I: Input geoTIFF file pointer 	*/
    TIFF *in,                   /* I: Input tiff file pointer     	*/
    long *numouts,		/* I: number of output images           */
    double window[],		/* I: windowed area of image      	*/
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was 
                                      specified and to convert its 
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],		/* I: resolution of image		*/
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
)
 
{
	/* variables needed for geoTIFF keys */
	double semiMajor = 0.0;         /* semi major axis                      */
	double semiMinor = 0.0;		/* semi minor axis			*/
	double tiepoints[6];            /* corner coordinates values            */
	short linearUnits=0;            /* linear unit of measurement           */

	/* variables needed for proj parms, utm does not have any, default to 0 */
	double parallel1 = 0.0;
	double Lat = 0.0;
	double Lon = 0.0;
	double centerLon = 0.0;
	double falseEast = 0.0;
	double falseLat = 0.0;
	double falseLon = 0.0;
	double falseNorth = 0.0;
	double geotrans[16];
	double natOrigin = 0.0;
	double num = 0.0;
	double parallel2 = 0.0;

	/* for output to ddr */
	long datum;			/* datum code                               */
	long proj_code = 1;		/* GCTP projection code                     */
	long zone_code;			/* UTM zone code			    */

	/* other geoTIFF key stuff */
	short geoType = 0;		/* geoTIFF key for datum	            */ 
	short projGeo = 0;		/* generic geoTIFF key for zone		    */ 
	short projType = 0;		/* contains datum, zone and units for ddr   */
	int  index;             /* variable used for looping            */
	int one = 1;            /* constant                                 */
	int zero = 0;           /* constant                                 */
	long ddr_valids[8];     /* array containing valids or invalids      */
	long status;            /* function return status                   */

	/* Set all the valid flags to VAILD
	----------------------------------*/
	for (index = 0; index < 8; index++)
		ddr_valids[index] = VALID;

	/* Get corner coordinates
	------------------------*/
	if (get_coordinates(in, pixel[0],&ddr_valids[6], tiepoints, geotrans) 
        != E_SUCC)
    {
	   c_lasmsg(FATAL, "utm-coordinates", 
                "Error retrieving corner coordinates");
    }

	/* Get other keys
	-----------------*/
	status = GTIFKeyGet(gtif, GeogLinearUnitsGeoKey, &linearUnits, zero, one);

	/* I found some geoTIFF files use projection linear units instead of 
	geographic linear units  */
	if (status == 0)
	{
	   status = GTIFKeyGet(gtif, ProjLinearUnitsGeoKey, &linearUnits, zero, 
                           one);
	   if (status == 0)
	   {
		  c_lasmsg(NONFATAL, "utm-warn",
				   "LinearUnits does not exist in geoTIFF file");
		  ddr_valids[4] = INVAL;   /* set ground units to invalid */
	   }
	}

	/* This is the standard key to use for landsat 7. It contains both
	   zone code and datum code. This is required for UTM because it uses  
	   zone codes instead of specific projection parameters
	----------------------------------------------------------------------*/
	GTIFKeyGet(gtif, ProjectedCSTypeGeoKey, &projType, zero, one); 


	/* GeographicTypeGeoKey key is used to get the datum  
	----------------------------------------------------------------------*/
	GTIFKeyGet(gtif, GeographicTypeGeoKey, &geoType, zero, one);


	/* ProjectionGeoKey key will use the generic key number to get the 
	   zone number. 
	---------------------------------------------------------------------*/
	GTIFKeyGet(gtif, ProjectionGeoKey, &projGeo, zero, one);

	/* Now get zone code and datum code to use for LAS
	-------------------------------------------------*/
	/* if projType does not exist or is user defined look for geoType and 
       projGeo*/
	if ((projType == 0) || (projType == 32767))
	{
	   if (geoType != 0)
	   {
		  if (get_datum(&geoType,&datum) != E_SUCC)
			 c_lasmsg(FATAL,"geotiffin-datum","Fatal error encountered");
	   }
	   else
		 c_lasmsg(FATAL,"geotiffin-utm",
                  "geoTIFF does not contain a datum code");
 
	   if (projGeo != 0)
	   {
		  get_utmzonegen(&projGeo,&zone_code);
	   }
	   else
		 c_lasmsg(FATAL,"geotiffin-utm","geoTIFF does not contain a zone code");   
	}
	else 
    {
		/* projType exists so get zone, datum and units. Get units only if
		   linearUnits key was not specified*/
		get_zone_datum(&projType,&zone_code,&datum,&linearUnits,ddr_valids);
    }

	/* write to ddr here
	---------------------*/ 
	if(update_ddr(outf,proj_code,zone_code,datum,semiMajor,semiMinor,parallel1,
			   parallel2,centerLon,natOrigin,falseEast,falseNorth,
			   Lon,Lat,falseLon,falseLat,num,linearUnits,tiepoints,numouts,
               window, bands,convert_window_flag,nl,ns,pixel,lan7,geotrans,
               ddr_valids, rasterType,valid_window,coorunits) != E_SUCC)
	   c_lasmsg(FATAL,"geotiffin-ddr","Fatal error encountered");


	return(E_SUCC);
}
