/****************************************************************************
NAME:                           elevation

PURPOSE:   Gets an elevation of a tie point from a DEM image file

ALGORITHM DESCRIPTION:
        Open the report file
        Initialize the processing message
        Open the input tie point location file
        Read input projection info from the TPL header record & process
        Read projection info from the DEM image DDR
        Initialize the projection transformation package
        Open the output tie point location file
        Open the input DEM image
        Allocate buffer space
        Calculate coefficients mapping projection coordinates to line and
          sample (image) coordinates in the DEM image.  Coefficients are
          used to handle a rotated projection coordinate system.
        Process all tie points in the file:
           If the elevation field is not zero, and replace is false 
             tell the user that an elevation already exists
           Else, calculate the line and sample coordinate of the projection
             coordinate in the tie point record.  Check the line, sample for
             image bounds.   If it is out of the image, warn the user and set
             the elevation field to zero.  If inside the image, read the
             appropriate DEM image line, and pack the appropriate sample into
             the tie point location file's elevation record.
           Write the resulting tie point location record to the output file
        Stop the processing message and close all files
        Return
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "getelev.h"
#include "util.h"
#include "protos.h"

int elevation
(
    char *in,                   /* Input Tie Point Location file name */
    char *in_tpl,               /* Input Tie Point Location file name */
    char *out_tpl,              /* Output Tie Point Location file name */
    long msgtim,                /* Message timer */
    long band,                  /* Band to read elevations from */
    char *printer,              /* Print file name */
    long cooropt,               /* Coordinate option */
    long replace                /* Option flag to replace elevation */
)
{
    FILE *fptr;                  /* File pointer to print file */
    struct TPLHEAD header;       /* Tie Point Location file header data */
    struct TPLDATA tpldata;      /* Tie Point Location file data records */
    struct DDR ddr;              /* DDR structure */
    double outx, outy;           /* X & Y coordinates input to transformation */
    double c[6];                 /* Coefficients mapping from l,s to proj x,y */
    double cc[6];                /* Coefficients mapping from proj x,y to l,s */
    double pc[8];                /* Projection coordinates of 4 image corners */
    float *buffer;               /* Floating point buffer for image I/O */
    long ic[8];                  /* Image coordinates of 4 image corners */
    long status;                 /* Function return status */
    long prtprm[2];              /* Print flags for c_transinit */
    long line;                   /* Line coordinate in DEM file */
    long sample;                 /* Sample coordinate in DEM file */
    long in_units;               /* Input units code */
    long out_units;              /* Output units code */
    long nbands;                 /* Number of bands in DEM image */
    long nl, ns;                 /* Number of lines and samples in DEM image */
    long buffer_size;            /* Number of bytes to allocate for DEM read */
    long previous;               /* Previous line read (currently in buffer) */
    struct FDESC *infdsc;        /* File descriptor for DEM image */
    struct PTDESC *indesc;       /* File descriptor for input TPS */
    struct PTDESC *outdesc;      /* File descriptor for output TPS */
    long flag = TRUE;            /* Flag--> print errors from coef calc */
    long ptyp = TPL;             /* Type of tie point file (TPL) */
    long acc;                    /* Access mode */
    long one = 1;                /* Constant of one */
    long two = 2;                /* Constant of two */
    long four = 4;               /* Constant of four */
    long opt[3];                 /* Option buffer for data-conversion */

    extern long curent;

    /* Open report/print file
       ----------------------*/
    fptr = fopen(printer, "w");

    fprintf(fptr,"Input Tie Point Location file:   %s\n", in_tpl);
    fprintf(fptr,"Output Tie Point Location file:  %s\n", out_tpl);
    fprintf(fptr,"DEM image file:                  %s\n\n", in);

    /* Initialize the processing message
       ---------------------------------*/
    c_inittm("GETELEV","Processed","point number","","",&two,&two,&msgtim,&two);

    /* Open the input Tie Point Location file
       ---------------------------------------*/
    acc = IREAD;
    if(c_ptopen(&indesc, in_tpl, &ptyp, &acc, (long*)&header) != E_SUCC) 
        return(E_FAIL);

    /* Read input projection information from header, if available
       -----------------------------------------------------------*/
    if (cooropt != IMAGE_COORS) 
    {
        if (header.proj_valid != GEO_VALID)
        {
            c_errmsg("Tie point location file must have valid projection "
                     "information", "getelev-tplproj",&one);
            return(E_FAIL);
        }

        if ((in_units = c_string2unit(header.units)) == E_FAIL)
        {
            c_lasmsg(FALSE, "getelev-units",
                    "The projection units; %s are not supported by LAS",
                    ddr.proj_units);
        }

        /* Read projection information from the image
           ------------------------------------------*/
        if (c_getddr(in, &ddr) != E_SUCC) 
            return(E_FAIL);

        if((ddr.valid[0]==INVAL)||(ddr.valid[3]==INVAL)||(ddr.valid[4]==INVAL) 
                || (ddr.valid[5] == INVAL) || (ddr.valid[6] == INVAL))
        {
            c_errmsg("Projection information not available from DEM DDR",
                    "getelev-noproj",&one);
            return(E_FAIL);
        }
        if (ddr.valid[1] == INVAL) ddr.zone_code = 62;
        if (ddr.valid[2] == INVAL) ddr.datum_code= 0;

        if ((out_units = c_string2unit(ddr.proj_units)) == E_FAIL)
        {
            c_lasmsg(FALSE, "getelev-units", 
                    "The projection units %s are not supported by LAS", 
                    ddr.proj_units);
        }

        /* Initialize the Projection Transformation Package
           ------------------------------------------------*/
        prtprm[GEO_TERM] = FALSE;
        prtprm[GEO_FILE] = FALSE;

        if(c_transinit(&header.code, &in_units, &header.zone, &header.datum,
                header.projprms, &ddr.proj_code, &out_units, &ddr.zone_code,
                &ddr.datum_code, ddr.proj_coef, prtprm, "") != E_SUCC) 
        {
            return(E_FAIL);
        }             
    }

    /* Open output Tie Point Location file
       ------------------------------------*/
    acc = IWRITE;
    if(c_ptopen(&outdesc, out_tpl, &ptyp, &acc, (long *)&header) != E_SUCC)
        return(E_FAIL);

    /* Open input image
       ----------------*/
    opt[0] = opt[1] = opt[2] = 0;
    nbands = 0; nl = ns = 0;
    acc = IREAD;
    if(c_eopenr(&infdsc,in,&nbands,&one,&one,&nl,&ns,&acc,&four,opt,
                &one)!=E_SUCC)
    {
        return(E_FAIL);
    }

    /* Allocate buffer space
       ---------------------*/
    buffer_size = (long) ( ns * sizeof(float));
    buffer = (float *)malloc(buffer_size);
    if (buffer == NULL) 
    {
        c_errmsg("Error allocating memory","getelev-alloc",&one);
        return(E_FAIL);
    }

    /* Setup coefficients mapping projection coordiantes to line, sample
       coordinates.  This is used just in case the projection coordinate system
       is rotated from the image coordinate system in the DEM image.
       -------------------------------------------------------------*/
    if (cooropt != IMAGE_COORS) 
    {
        pc[0] = ddr.upleft[0];
        pc[1] = ddr.upleft[1];
        pc[2] = ddr.loleft[0];
        pc[3] = ddr.loleft[1];
        pc[4] = ddr.upright[0];
        pc[5] = ddr.upright[1];
        pc[6] = ddr.loright[0];
        pc[7] = ddr.loright[1];
        ic[0] = ic[1] = 1;
        ic[2] = nl;
        ic[3] = 1;
        ic[4] = 1;
        ic[5] = ns;
        ic[6] = nl;
        ic[7] = ns;
        if (c_rtcoef(pc, ic, c, &flag) != E_SUCC) return(E_FAIL);
        if (c_invert(c,cc) != E_SUCC) return(E_FAIL);
    }

    /* Process all points in the file (Loop until EOF)
       -----------------------------------------------*/
    curent = 1;
    c_startm();
    curent = 0;
    previous = 0;

    for (;;)
    {
        status = c_ptio(&indesc, (long *)&tpldata);
        if (status == E_EOF) break;
        if (status == E_FAIL) return(E_FAIL);
        curent++;

        if (tpldata.elevation > 0.0001 && replace == FALSE)
            fprintf(fptr,"%s already contained an elevation of %f\n",
                    tpldata.pt_id, tpldata.elevation);
        else 
        {
            if (cooropt == IMAGE_COORS) 
            {
                line =   (long) tpldata.ref_coord[0];
                sample = (long) tpldata.ref_coord[1];
            }
            else 
            {
                if (c_trans(&header.code, &in_units, 
                        &ddr.proj_code, &out_units, &tpldata.geo_coord[1], 
                        &tpldata.geo_coord[0], &outx, &outy) 
                        != E_SUCC) 
                {  
                    return(E_FAIL);
                }
                /* Convert to line, sample coordinates
                   -----------------------------------*/
                line = (long) (((cc[0] * outy) + (cc[1] * outx) + cc[2]) + 0.5);
                sample = (long) (((cc[3] * outy) + (cc[4] * outx) + cc[5])+0.5);
            }

            /* Get the elevation
               -----------------*/
            if ((sample > ns) || (sample < 1) || (line > nl) || (line < 1))
            {
                tpldata.elevation = 0.0;
                fprintf(fptr, "%s not within image; elevation set to zero\n",
                        tpldata.pt_id);
            }
            else
            {
                if (line != previous)
                {
                    if (c_eread(&infdsc,&band,&line,(unsigned char *)buffer,
                            &one) == E_FAIL)
                        return(E_FAIL);
                    previous = line;
                }
                tpldata.elevation = buffer[sample - 1];
                if (replace == TRUE)
                    fprintf(fptr,"%s replaced with an elevation of %f\n", 
                            tpldata.pt_id, tpldata.elevation);
                else
                    fprintf(fptr,"%s assigned an elevation of %f\n", 
                            tpldata.pt_id, tpldata.elevation);
            }
        }

        /* Write to output file 
           --------------------*/
        if (c_ptio(&outdesc, (long *)&tpldata) != E_SUCC) return(E_FAIL);
    }

    c_stoptm();
    fclose(fptr);
    c_ptclse(&indesc);
    c_ptclse(&outdesc);
    c_eclose(&infdsc);
    free(buffer);

    return(E_SUCC);
}
