/****************************************************************************
NAME:                           GETELEV

PURPOSE:  Gets the elevation of a tie point from a DEM image file.

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  5.0    11/89  D. Steinwand  CSB      LAS 5.0 Original development
  5.1     1/94  D. Steinwand  SAB      Added Line/Sample option
  5.2     2/94  D. Etrheim    CSB      Cleanup, Document, and release above
                                       change PRF# 6445
  6.0    04/95  D. Eichacker  CSB      added the replace option

ALGORITHM DESCRIPTION:
        Initialize LAS processing
        Get all parameters 
        Get elevations of the tie points
        Write the standard completion message & exit

*****************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "getelev.h"
#include "util.h"
#include "protos.h"

int main
(void)          
{
    struct PARBLK vblk;             /* TAE parameter block */
    struct VARIABLE *var;           /* pointer to a TAE variable in vblk */
    long acc;                       /* access flag for c_getnam */
    long zero = 0;                  /* Constant of zero */
    long msgtim;                    /* Message timer */
    long std_out;                   /* Standard output LUN */
    long one = 1;                   /* Constant of 1 */
    long num_out;                   /* Number of output files */
    long fatal = -1;                /* Fatal flag for errmsg */
    long num_img;                   /* The number of output images */
    long in_wind[1][4];             /* Input window spec */
    long totbands;                  /* Total bands */
    long bands[1][MAXBND + 1];      /* Array for band numbers */
    long nbands;                    /* The number of bands */
    long band;                      /* Band number of band used for elevations*/
    long same;                      /* Same window flag for getin */
    long qflag[1][2];               /* Flags for getin */
    long term, file, ln_pr;         /* PRINT flags */
    long cooropt;                   /* coordinate option */
    long replace;                   /* Option flag to replace elevation */
    char tdum[1][CMLEN];            /* Tae name dummy buffer */
    char tae_name[2][CMLEN];        /* TAE names of output TPL file */
    char in[1][CMLEN];              /* Name of DEM image file */
    char in_tpl[CMLEN];             /* Input tie point location file name */
    char out_tpl[CMLEN];            /* Output tie point location file name */
    char name[CMLEN];               /* Name for PRINT parameter */
    char filename[CMLEN];           /* Name for PRINT parameter */
    char host_name[CMLEN];          /* Name for PRINT parameter */

    /* Initialize LAS processing
       -------------------------*/
    c_inlas(&vblk, &std_out);

    /* Get the print option.  Make a file name (if neccessary) and open the
       temporary file
       --------------*/
    strcpy(host_name, "getelev");
    if (c_setprt(&vblk, &term, &file, &ln_pr, host_name, filename) != E_SUCC)
        c_errmsg("Fatal error encountered","getelev-fatal",&fatal);

    acc = IUPDATE;
    if (file == 1) c_mknam(filename, name, "prt", &acc);
    else strcpy(name, " ");

    /* Get the input tie point location file name
       ------------------------------------------*/
    acc = IREAD;
    if (c_getnam(&vblk, "INTL", &zero, in_tpl, "tpl", &acc) != E_SUCC)
        c_errmsg("Fatal error encountered","getelev-fatal",&fatal);

    /* Get the output tie point selection file name
       -------------------------------------------*/
    acc = IWRITE;
    if(c_getnam(&vblk, "OUTTL", &zero, out_tpl, "tpl", &acc) != E_SUCC)
        c_errmsg("Fatal error encountered","getelev-fatal",&fatal);

    /* Get the name of the DEM image & other misc info
       -----------------------------------------------*/
    if(c_getin(&vblk,"IN",&zero,tdum,in,&num_img,in_wind,bands,&nbands,
            &totbands,&one,&same,qflag) != E_SUCC)
        c_errmsg("Fatal error encountered","getelev-fatal",&fatal);
    band = bands[0][0];

    /* Get the message timer
       ---------------------*/
    var = p_find(&vblk, "$msgtim");
    msgtim = IVAL(*var, 0);

    /* Get the coordinate mode
       -----------------------*/
    var = p_find(&vblk, "COOROPT");
    if (strcmp(SVAL(*var, 0), "YX") == 0) 
        cooropt = COORS;
    else 
        cooropt = IMAGE_COORS;

    /* Get the replace flag 
       -----------------------*/
    var = p_find(&vblk, "REPFLG");
    if (strcmp(SVAL(*var, 0), "YES") == 0) 
        replace = TRUE;
    else 
        replace = FALSE;

    /* Get elevations for each tie point record
       ----------------------------------------*/
    if (elevation(in[0], in_tpl, out_tpl, msgtim, band, host_name, cooropt, 
            replace) != E_SUCC)
        c_errmsg("Fatal error encountered","getelev-fatal",&fatal);

    /* Set up file names for completion message
       ----------------------------------------*/
    num_out = 1;
    strcpy(tae_name[0], out_tpl);
    if(strcmp(name, " ") != 0) 
    {
        strcpy(tae_name[num_out], name);
        num_out++;
    }

    /* Write the standard completion message and exit
       ----------------------------------------------*/
    c_complt("GETELEV", tae_name, &num_out);

    return EXIT_SUCCESS;

}
