/******************************************************************************

NAME:   DESC_HDR

FUNCTION:
        DESC_HDR writes the header information of the Attribute Labeled Table 
        file.  This information includes definitions for the unique number
        identifying each record, the label and any attributes defined by
        the input GOF.

ALGORITHM:

        Count the number of attributes defined in the GOF.
        If processing a polygon GOF
            Write out the REGNUM (which is a unique number identifying each
              polygon) vector to the LT.
        Else if processing a line GOF
            Write out the ARCNUM (which is a unique number identifying each
              line) vector to the LT.
        Else if processing a point GOF
            Write out the PNTNUM (which is a unique number identifying each
              point) vector to the LT.
        Else if processing a annotation GOF
            Write out the ANNOTNUM (which is a unique number identifying each
              annotation record) vector to the LT.
        Write out the LABEL vector to the LT (which defines a user specified 
          label for each graphics element).
        For each attribute defined in the GOF header record
            If the data type of the attribute is character
                Write out a LT vector defining a character attribute.
            Else if the data type of the attribute is integer
                Write out a LT vector defining an integer attribute.
            Else if the data type of the attribute is real
                Write out a LT vector defining a real attribute.
            Else
                Print an error message indicating an unrecognized data type
                  was found in the GOF.
        Return to the calling routine.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "gof.h"
#include "ltable.h"
#include "protos.h"

int desc_hdr
(
    struct TAB_DEF *tabl,               /* Labeled table definition structure (I)     */
    union GOF *gofrec,  /* GOF header record (I)                      */
    long *attnmr,       /* Number of attributes defined by the GOF (O)*/
    long attyp[]        /* Data type of each attribute (O)            */
)
{
long    dtype;                  /* Data type of current attribute             */
long    index;                  /* Looping index                              */
long    noabort = 1;            /* Report and continue flag for error messages*/

char    sep;                    /* Type of separator for the LT vector being
                                   processed                                  */

/*
Count the number of attributes defined in the GOF
*/
for (*attnmr=0; gofrec->hdrec.mask[*attnmr]!=0; (*attnmr)++)
   ;

/*
Write the LT vectors for the unique number defining each record in the LT
*/
if (strcmp(tabl->ftype,"REGATT") == 0)
   put_vector(tabl,"REGNUM",I4,"unique region number",1,1,FIELD_SEP2);
else if (strcmp(tabl->ftype,"ARCATT") == 0)
   put_vector(tabl,"ARCNUM",I4,"unique region number",1,1,FIELD_SEP2);
else if (strcmp(tabl->ftype,"PNTATT") == 0)
   put_vector(tabl,"PNTNUM",I4,"unique region number",1,1,FIELD_SEP2);
else if (strcmp(tabl->ftype,"ANNOTATT") == 0)
   put_vector(tabl,"ANNOTNUM",I4,"unique region number",1,1,FIELD_SEP2);

/*
Write the LT vector for the label of the graphics element
*/
if (*attnmr == 0)
   sep = RECORD_SEP;
else
   sep = FIELD_SEP2;
put_vector(tabl,"LABEL",C,"user defined label description",1,LABLEN,sep);

for (index=0; index<*attnmr; index++)
   {
   /*
   Write the LT vector for each attribute in the GOF 
   */
   if (index+1 == *attnmr)
      sep = RECORD_SEP;
   else
      sep = FIELD_SEP2;
   switch (gofrec->hdrec.attyp[index][0])
      {
      case 'c':
      case 'C': dtype = attyp[index] = C;
                put_vector(tabl,gofrec->hdrec.attrb[index],dtype,"",1,ATTVAL,
                   sep);
                break;
      case 'i':
      case 'I': dtype = attyp[index] = I4;
                put_vector(tabl,gofrec->hdrec.attrb[index],dtype,"",1,1,sep);
                break;
      case 'r':
      case 'R': dtype = attyp[index] = R4;
                put_vector(tabl,gofrec->hdrec.attrb[index],dtype,"",1,1,sep);
                break;
      default:  c_errmsg ("Unknown attribute type in GOF","gof2tab-attyp",
                   &noabort);
                return (E_FAIL);
      }
   }
return(E_SUCC);
}
