/******************************************************************************

NAME:   PROCESS_POINT

FUNCTION:
        PROCESS_POINT creates 2 output LT files (the attribute and the
        data files) and writes the appropriate header information to each.
        The header information for the attribute LT is partially defined by
        the GOF header record.  The specified GOF records are then read with
        the label and attribute values being written to the attribute LT and 
        the X,Y location of point being written to the data LT.

ALGORITHM:

        Open the GOF and read the header record.
        Verify that the user specified attributes are valid for this GOF.
        Open both the attribute and data LT files.
        Write the appropriate header information to the attribute and data LTs.
        For each point record from the input GOF
            Read the GOF record, break from the loop if end of file was 
              reached and print an error if the read failed.
            Verify whether this record is to be processed based on the
              user specified attribute values.
            Write the point number, X location and Y location to the data LT.
            Write the label to the attribute LT.
            For each attribute defined for the GOF
                Write the attribute value for this record to the attribute LT.
        Close the GOF and LT files.
        Return to the calling routine.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "gof.h"
#include "ltable.h"
#include "protos.h"

#define rec curent              /* Global for processing message              */

void process_point
(
    char *host_gof,     /* host name of input GOF (I)                         */
    char *las_gof,      /* LAS name of input GOF (I)                          */
    char *host_tab,     /* Host name of output Labeled Table file (I)         */
    char *ftype,                /* Input GOF type (I)                         */
    char inatr[][ATTIN],        /* User specified GOF attributes (I)          */
    long numatr,                /* Number of user specified GOF attributes (I)*/
    char *desc,         /* Labeled Table file description (I)                 */
    long msgtim         /* Number of seconds between processing
                                   messages (I)                               */
)
{
    extern  struct  PARBLK  vblock; /* TAE parameter block                    */
    extern  char    outlt[2][CMLEN];/* Output Labeled Table Host names        */
    extern  long    rec;            /* Current record being proccessed        */

    struct  TAB_INFO tab;           /* Labeled Table definition structure     */

    union   GOF     gofrec;         /* GOF definition structure               */
        
    long    abort = -1;             /* Abort flag for error messages          */
    long    attnmr;                 /* Number of GOF attributes               */
    long    attyp[MAXATT];          /* GOF attribute type array               */
    long    flag = 3;               /* Type of processing message             */
    long    index;                  /* Loop index counter                     */
    long    nrec;                   /* Total number of GOF records to be read */
    long    status;                 /* Function return status code            */
    long    total_recs;             /* Running total of number of points      */
    long    two = 2;                /* Constant two                           */
    long    zero = 0;               /* Constant zero                          */

    int access = READ;              /* Read access for GOF                    */
    int atrindex[MAXATT];           /* GOF attribute index array              */
    int cflag;                      /* Flag indicating whether GOF was created*/  
    int gof_fd;                     /* File descriptor for GOF                */
    int gof_type;                   /* Type of GOF record to be read          */
    int length;                     /* Length to next GOF record              */
    int obyte;                      /* Number of bytes read from GOF          */
    int sflag = TRUE;               /* Flag to sort GOF attributes            */


    char    atrval[50][ATTVAL];     /* GOF attribute value array              */
    char    object[20];             /* Object of processing message           */
    char    sep;                    /* Field or record Labeled Table separator*/

    /* Open the GOF file, read the header record, and check the user
       specified attribute specifications with the GOF */
    if (c_gofopn(&gof_fd,host_gof,las_gof,ftype,&access,&cflag) != E_SUCC)
        c_errmsg("Fatal error encountered","gof2tab-fatal",&abort);

    gof_type = HDR;
    if (c_gofrd(&gof_fd,&obyte,&length,&gof_type,(struct DDR *)NULL,&gofrec) != E_SUCC)
        c_errmsg("Fatal error encountered","gof2tab-fatal",&abort);
    nrec = gofrec.hdrec.nrec;
    if (attck(&gofrec.hdrec,inatr,atrval,atrindex,&sflag) != E_SUCC)
        c_errmsg("Fatal error encountered","gof2tab-fatal",&abort);

    /* Open the Labeled Table files and write header information to the
       Labeled table files */
    open_lt(&tab,las_gof,host_tab,ftype,desc);
    status = desc_hdr(&tab.desc,&gofrec,&attnmr,attyp);
    if (status != E_SUCC)
    {
        close_tab(&tab.desc);
        close_tab(&tab.data);
        c_clean(&vblock,"gof2tab",outlt,&two);
    }
    data_xy_hdr(&tab.data);

    sprintf(object,"%s record",ftype);
    c_inittm("GOF2TAB","Processed",object,"","",&nrec,&zero,&msgtim,&flag);
    rec = 1;
    c_startm();
    gof_type = PNT;
    for (total_recs=0; ; rec++)
    {
        /* Read each GOF record */
        if ((status = c_gofrd(&gof_fd,&obyte,&length,&gof_type,(struct DDR *)NULL, &gofrec)) 
                   == E_EOF)
            break;
        if (status != E_SUCC)
        {
            close_tab(&tab.desc);
            close_tab(&tab.data);
            c_clean(&vblock,"gof2tab",outlt,&two);
        }
        /* If the record has been deleted, read the next record */
        if (strcmp(gofrec.ptrec.label,DELREC) == 0)
            continue;
        /* Determine whether this record is to be processed based on the
           user specified attributes */
        if (check_attrb(gofrec.ptrec.attrb,atrval,atrindex,numatr) == FALSE)
            continue;
        ++total_recs;
        /* Write the point to the Data Labeled Table file in the format:
           current record;x point;y point<CR> */
        put_i4_field (&tab.data,total_recs,FIELD_SEP1);
        put_r4_field (&tab.data,gofrec.ptrec.x,FIELD_SEP1);
        put_r4_field (&tab.data,gofrec.ptrec.y,RECORD_SEP);

        /* Write the attribute values to the Description Labeled Table 
           file in the format: current record;label field;attribute 
           field 1; ... ; attribute field n<CR> */
        put_i4_field (&tab.desc,total_recs,FIELD_SEP1);
        if (attnmr == 0)
            sep = RECORD_SEP;
        else
            sep = FIELD_SEP1;
        put_string_field(&tab.desc,gofrec.ptrec.label,sep,TRUE);
        for (index=0; index<attnmr; index++)
        {
            if (index+1 == attnmr)
                sep = RECORD_SEP;
            else
                sep = FIELD_SEP1;
            switch (attyp[index])
            {
                case C:  
                    put_string_field (&tab.desc,gofrec.ptrec.attrb[index],
                                      sep,TRUE);
                    break;
                default: 
                    put_string_field (&tab.desc,gofrec.ptrec.attrb[index],
                                      sep,FALSE);
                    break;
            }
        }
    }
    c_stoptm();
    /* Close the GOF and LT files */
    if (c_gofcls(&gof_fd) != E_SUCC)
    {
        close_tab(&tab.desc);
        close_tab(&tab.data);
        c_clean(&vblock,"gof2tab",outlt,&two);
    }
    close_tab(&tab.data);
    close_tab(&tab.desc);
    return;
}
