/*****************************************************************************
NAME:               LAS_CODE

PURPOSE:  Contains TAE & LAS specific code for simplifing IO sequences

PROGRAM HISTORY:
VERSION     DATE    AUTHOR      CODE/CONT
-------     ----    ------      ---------
  1.0       7/91    D. Steinwand     AB/ISS
*****************************************************************************/
#include "ezio.h"


char FUNC_NAME1[25];            /* Function name (as passed) */
char func_name[25];             /* Function name (lower case) */
char name[1][CMLEN];            /* Output image TAE name */
char minmax[10];                /* Minmax flag */
long msgtim;                    /* Message timer */
struct FDESC *outptr;
struct FDESC *inptr;

long inimage;
long outimage;

long in_block_nl;
long in_block_ns;

char hosin[MAXIMG][CMLEN];
char hosout[1][CMLEN];

long nmrimg;
long nbands[MAXIMG];

double minval[MAXBND];
double maxval[MAXBND];

long out_nl;
long in_image_ns;

long out_image_ns;

struct PARBLK vblk;             /* TAE parameter block */


/***************** Initialize LAS & TAE processing **************************/

void start
(
    char name[25]               /* Name of LAS function */
)
{
    struct VARIABLE *var;       /* Pointer to a TAE variable in vblk */
    long std_out;               /* Standard output (is this ever used?) */
    long twentyfive=25;         /* Constant of 25 */

    /* Initialize LAS processing
       -------------------------*/
    c_inlas(&vblk, &std_out);

    /* Copy function name to global for later use
       ------------------------------------------*/
    strcpy(FUNC_NAME1, name);
    strcpy(func_name, name);
    c_up2low(func_name, &twentyfive);

    /* Get TAE/LAS parameters and store in globals for later use
       ---------------------------------------------------------*/
    var = p_find(&vblk, "$msgtim");
    msgtim = IVAL(*var, 0);

    var = p_find(&vblk, "$minmax");
    strcpy(minmax, SVAL(*var, 0));

    outimage = FALSE;
    inimage = FALSE;
}

/***************** Get a floating point variable ****************************/

float get_float
(
    char parm[25]
)
{
    struct VARIABLE *var;       /* Pointer to a TAE variable in vblk */

    var = p_find(&vblk, parm);
    return(RVAL(*var, 0));
}

/***************** Close down processing & exit *****************************/

void finish
(
    int reduce_flag           /* flag to indicate the output image was reduced
                                 from the input image (1=reduced, 0=enlarged) */
)
{
    long num_out = 0;         /* Number of output images */
    struct DDR ddr;
    struct BDDR *bptr;        /* pointer to current BDDR record */
    struct BDDR bddr[MAXBND];
    struct DDR new_ddr;
    double scale;
    char temp_str[CMLEN];
    int cur_band;             /* looping variable */

    c_stoptm();

    if (inimage)
        c_eclose(&inptr);
    if (outimage)
    {
        c_eclose(&outptr);
        num_out++;
    }

    /* Update ddr and minmax and for output image
       ---------------------------------------------*/
    /* read the original DDR and the new DDR */
    if (c_getddr(hosin[0],&ddr) != E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-upddr");
        c_lasmsg(FALSE,temp_str,"Error reading DDR");
        strcat(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }
    
    for (cur_band = 1; cur_band < nbands[0]+1; cur_band++)
    {
        if (c_getbdr(hosin[0],&bddr[cur_band-1],&cur_band) != E_SUCC)
        {
            strcpy(temp_str, func_name);
            strcat(temp_str, "-upddr");
            c_lasmsg(FALSE,temp_str,"Error reading BDDR");
            strcat(temp_str, func_name);
            strcat(temp_str, "-fatal");
            c_lasmsg(TRUE,temp_str,"Fatal error encountered");
        }
    }

    if (c_getddr(hosout[0],&new_ddr) != E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-upddr");
        c_lasmsg(FALSE,temp_str,"Error reading DDR");
        strcat(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }

    /* copy the NL, NS, NB, dtype, last modified info, and system from the new
       DDR to the original ddr */
    ddr.nl = new_ddr.nl;
    ddr.ns = new_ddr.ns;
    ddr.nbands = new_ddr.nbands;
    ddr.dtype = new_ddr.dtype;
    strcpy(ddr.system,new_ddr.system);
    strcpy(ddr.last_used_date,new_ddr.last_used_date);
    strcpy(ddr.last_used_time,new_ddr.last_used_time);

    /* set the master coordinate line/sample based on the window - since no 
       windowing is supported right now, always 1,1 */
    ddr.master_line = 1;
    ddr.master_sample = 1;

    /* set the projection distance and increments correctly, depending on 
       whether the image was reduced or enlarged and set the valid flags */
    if (reduce_flag)
        scale = 2.0;
    else
        scale = 0.5;
    ddr.line_inc = scale;
    ddr.sample_inc = scale;
    ddr.pdist_y *= scale;
    ddr.pdist_x *= scale;
    ddr.valid[7] = 1;

    /* update the corners if they are valid */
    if (ddr.valid[6] == 1)
    {
        double pix_y, pix_x; /* pixel size in y and x directions */
        double odd_sample_adjust = 0.0;
        double odd_line_adjust = 0.0;

        pix_y = (ddr.loright[0] - ddr.upleft[0])/(in_block_nl - 1);
        pix_x = (ddr.loright[1] - ddr.upleft[1])/(in_block_ns - 1);

        if (reduce_flag)
        {
            /* adjust for odd number of pixels in the source image since the
               last pixel is just dropped */
            if (in_block_nl & 0x01)
                odd_line_adjust = pix_y;
            if (in_block_ns & 0x01)
                odd_sample_adjust = pix_x;

            /* reduced image, so offset corners by 0.5 pixels */
            pix_y /= 2.0;
            pix_x /= 2.0;
        }
        else
        {
            /* enlarged image, so offset corners by 1/4 a pixel in the opposite
               direction */
            pix_y = -pix_y/4.0;
            pix_x = -pix_x/4.0;
        }

        /* adjust the corners */
        ddr.upleft[0]  += pix_y;
        ddr.upright[0] += pix_y;
        ddr.loleft[0]  = ddr.loleft[0] - pix_y - odd_line_adjust;
        ddr.loright[0] = ddr.loright[0] - pix_y - odd_line_adjust;
        ddr.upleft[1]  += pix_x;
        ddr.upright[1] = ddr.upright[1] - pix_x - odd_sample_adjust;
        ddr.loleft[1]  += pix_x; 
        ddr.loright[1] = ddr.loright[1] - pix_x - odd_sample_adjust;
    }

    /* write the ddr for the new file */
    if (c_putddr(hosout[0],&ddr) != E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-upddr");
        c_lasmsg(FALSE,temp_str,"Error writing DDR");
        strcat(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }


    // Fill in the BDDR structure and output to BDDR
   
    bptr = bddr;
  
    for (cur_band = 1; cur_band <= nbands[0]; cur_band ++)
    {
        bptr->bandno = cur_band;
        bptr->valid = INVAL;     /* Since min max is invalid  */
        bptr->minval = 0.0;      /* Since it is not available */
        bptr->maxval = 0.0;      /* Since it is not available */

        if (c_putbdr(hosout[0],&bddr[cur_band-1]) != E_SUCC)
        {
            strcpy(temp_str, func_name);
            strcat(temp_str, "-upddr");
            c_lasmsg(FALSE,temp_str,"Error writing BDDR");
            strcat(temp_str, func_name);
            strcat(temp_str, "-fatal");
            c_lasmsg(TRUE,temp_str,"Fatal error encountered");
        }
    }

    /* Create associated history file for output image
       --------------------------------------------------*/
    if(c_hstout(&vblk, hosout[0], hosin, &nmrimg, " ") != E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-upddr");
        c_lasmsg(FALSE,temp_str,"Error updating history");
        strcat(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }

    /* Write completion message
       ------------------------*/
    c_complt(FUNC_NAME1, name, &num_out);
}

/***************** Allocate buffer space ***********************************/

unsigned char* init_buf
(
    long length,
    long dtype,
    long nbands
)
{
    long size;
    unsigned char *buffer;
    char temp_str[80];

    if (dtype == SHORT)
        size = length * (long)sizeof(short) * nbands;
    else if (dtype == LONG)
        size = length * (long)sizeof(long) * nbands;
    else if (dtype == FLOAT)
        size = length * (long)sizeof(float) * nbands;
    else if (dtype == DOUBLE)
        size = length * (long)sizeof(double) * nbands;
    else if (dtype == COMPLEX)
        size = length * (long)sizeof(float) * 2 * nbands;
    else if (dtype == DCOMPLEX)
        size = length * (long)sizeof(double) * 2 * nbands;
    else size = length * nbands; /* dtype == BYTE */

    buffer = (unsigned char *)malloc(size);
    if (buffer == NULL)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-alloc");
        c_lasmsg(FALSE,temp_str,"Error allocating memory");
        strcat(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }
    return(buffer);
}

/***************** Get a block of image lines ******************************/

void proc_mess
(
    long num_bands
)
{
    long one = 1;

    c_inittm(FUNC_NAME1,"Processed","line","band"," ",&out_nl,
             &num_bands,&msgtim,&one);
    c_startm();
}

/***************** Get an image line ***************************************/

void get_line
(
    unsigned char *buf,
    int current_band,
    int in_image_line 
)
{
    long status;
    long one = 1;
    char temp_str[80];

    status = c_eread (&inptr, &current_band, &in_image_line, buf, &one);
    if (status != in_image_ns)
    {
        c_eclose(&outptr);
        strcpy(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_abort(&vblk, "Fatal error encountered",temp_str,func_name,
                hosout,&one);
    }  
}

/***************** Put an image line ***************************************/

void put_line
(
    unsigned char *buf,
    int output_band,
    int out_image_line
)
{
    long status;
    long one = 1;
    char temp_str[80];

    status = c_ewrite (&outptr, &output_band, &out_image_line, buf, &one);
    if (status  !=  out_image_ns)
    {
        c_eclose (&outptr);
        strcpy(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_abort(&vblk,"Fatal Error encountered",temp_str,func_name,hosout,&one);
    }
}

/***************** Open the output image ************************************/

void init_outimg
(
    long nl,
    long ns,
    long nb,
    long odtype
)
{
    char temp_str[80];
    long one = 1;
    long acc;
    long otype;
    long status;
    long opt[2];

    /* Set otype
       ---------*/
    if (odtype == SHORT)
        otype = EWORD;
    else if (odtype == LONG)
        otype = ELONG;
    else if (odtype == FLOAT)
        otype = EREAL;
    else if (odtype == DOUBLE)
    {
        otype = EREAL;
        ns *= 2;
    }
    else if (odtype == COMPLEX)
    {
        otype = EREAL;
        ns *= 2;
    }
    else if (odtype == DCOMPLEX)
    {
        otype = EREAL;
        ns *= 4;
    }
    else otype = EBYTE; /* odtype == BYTE */


    /* Get output file name
       -----------------------*/
    status = c_getout (&vblk, "OUT", name, hosout);
    if (status  !=  E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }

    /* Open the output image
       ---------------------*/
    opt[0] = 0;
    opt[1] = 0;
    acc = IWRITE;
    status=c_eopenr(&outptr,hosout[0],&nb,&one,&one,&nl,&ns,&acc,&otype,
                    opt,&one);
    if(status != E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }

    outimage = TRUE;
    out_image_ns = ns;
    out_nl = nl;

    /* Initialize min and max arrays
       --------------------------------*/
    c_inmnmx (&otype, &nb, minval, maxval);

}
/***************** Open the input image ************************************/

void init_inimg
(
    long *nl,
    long *ns,
    long *nb,
    long dtype,
    int bands[]
)
{
    char taein[MAXIMG][CMLEN];
    long same;
    long qflag[MAXIMG][2];
    long maximg = MAXIMG;
    long zero = 0;
    long one = 1;
    long status;
    char temp_str[80];
    long acc = IREAD;
    long var_num = 0;
    long in_block_sl;
    long in_block_ss;
    long window[MAXIMG][4];
    long totbnd;
    long in_type;

    /* Get input file information
       -----------------------------*/
    status = c_getin(&vblk, "IN", &var_num, taein, hosin, &nmrimg, window, 
                     bands, nbands, &totbnd, &maximg, &same, qflag);
    if(status != E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }

    if (same != 1)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-samewind");
        c_lasmsg(FALSE,temp_str,"Input images must have same window sizes");
        strcat(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }

    status = c_gettyp(hosin, &nmrimg, &dtype);
    if (status  !=  E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-sametype");
        c_lasmsg(FALSE,temp_str,"Input images must be the same data type");
        strcat(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }

    in_block_sl = 1;
    in_block_ss = 1;
    in_block_nl = 0;
    in_block_ns = 0;

    if (dtype == SHORT) in_type = EWORD;
    else if (dtype == LONG) in_type = ELONG;
    else if (dtype == FLOAT) in_type = EREAL;
    else in_type = EBYTE; /* dtype == BYTE */

    status = c_eopenr(&inptr, &(hosin[0][0]), &totbnd, &in_block_sl, 
              &in_block_ss, &in_block_nl, &in_block_ns, &acc, &in_type,
              &zero, &one);

    if (status != E_SUCC)
    {
        strcpy(temp_str, func_name);
        strcat(temp_str, "-fatal");
        c_lasmsg(TRUE,temp_str,"Fatal error encountered");
    }
    inimage= TRUE;
    *nl = in_block_nl;
    *ns = in_block_ns;
    *nb = totbnd;

    in_image_ns = *ns;

    return;
}
