/****************************************************************************
NAME:				conv_grid

PURPOSE:  Convolves two geometric mapping grids and places the results
	  back into the second geometric mapping grid structure

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		7/88	D. Steinwand	   CSB      Original development--
							ditty for ADAPS newtem
  5.0		10/89	D. Steinwand	   CSB      Conversion to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Call complete & exit

ALGORITHM REFERENCES:
*****************************************************************************/

#include "las.h"
#include "geompak.h"
#include "protos.h"

int conv_grid
(
    struct GEOGRID *grid1,
    struct GEOGRID *grid2
)
{
double a0, a1, a2, a3;
double b0, b1, b2, b3;
double a[MAX_COEF];
double b[MAX_COEF];
double line,samp;
double tmp;
double newlin, newsam;
double xtl, xtr, xbl, xbr;
double ytl, ytr, ybl, ybr;
double *new_line_coord;
double *new_samp_coord;

long i, j, k, n;
long row, col;
long dx, dy;
long order;
long nel;
long nonfatal = 1;
long fatal = -1;
long loc;

/* Get polynomial coefficients from first geometric mapping grid
  -------------------------------------------------------------*/
if (grid1->ncoeff == 3) order = 1;
if (grid1->ncoeff == 6) order = 2;
if (grid1->ncoeff == 10) order = 3;
if (grid1->ncoeff == 15) order = 4;
for (i = 0; i < grid1->ncoeff; i++)
    {
    b[i] = grid1->coeffs[i];
    a[i] = grid1->coeffs[i + grid1->ncoeff];
    }

/* Allocate temporary memory for the new input image locations
  -----------------------------------------------------------*/
nel = grid2->nrows * grid2->ncols;
new_line_coord = (double *)malloc(nel * sizeof(double));
new_samp_coord = (double *)malloc(nel * sizeof(double));
if ((new_line_coord == NULL) || (new_samp_coord == NULL))
    {
    c_errmsg("Error allocating memory","gridconv-alloc",&nonfatal);
    c_errmsg("Fatal error occured","gridconv-fatal",&fatal);
    }

/* Apply polynomial to create new grid file
  ----------------------------------------*/
for (k = 0, i = 0; i < grid2->nrows; i++)
    for (j = 0; j < grid2->ncols; k++, j++)
	{
	samp = (double) grid2->out_samps[j];
	line = (double) grid2->out_lines[i];
	newsam = c_eval(&order, a, &samp, &line);
	newlin = c_eval(&order, b, &samp, &line);

	for (n = 0, row = 0; n < (grid2->nrows - 1); n++)
	    if (newlin > grid2->out_lines[n]) row = n;

	for (n = 0, col = 0; n < (grid2->ncols - 1); n++)
	    if (newsam > grid2->out_samps[n]) col = n;

/* Find the corners of the output space grid cell in the first grid which 
   contains newsam and newlin
  --------------------------*/
   	loc = (row * grid2->ncols) + col;
   	xtl = *(grid2->in_samps + loc);
   	ytl = *(grid2->in_lines + loc++);
   	xtr = *(grid2->in_samps + loc);
   	ytr = *(grid2->in_lines + loc);
   	loc += grid2->ncols;
   	xbr = *(grid2->in_samps + loc);
   	ybr = *(grid2->in_lines + loc--);
   	xbl = *(grid2->in_samps + loc);
   	ybl = *(grid2->in_lines + loc);

/* Calculate coefficients
  ----------------------*/
	dx = grid2->out_samps[1] - grid2->out_samps[0];
	dy = grid2->out_lines[1] - grid2->out_lines[0];
	coef(xtl,xtr,xbl,xbr,&a0,&a1,&a2,&a3,dx,dy);
	coef(ytl,ytr,ybl,ybr,&b0,&b1,&b2,&b3,dx,dy);

/* Adjust coordinates to be zero relative to the current grid cell
  ---------------------------------------------------------------*/
	newsam = newsam - grid2->out_samps[col];
	newlin = newlin - grid2->out_lines[row];

/* Find raw image location for the reference image output space
  ------------------------------------------------------------*/
	tmp = newsam * newlin;
	new_samp_coord[k] = a0 + (a1 * newsam) + (a2 * newlin) + (a3 * tmp);
	new_line_coord[k] = b0 + (b1 * newsam) + (b2 * newlin) + (b3 * tmp);
	}

/* Place new grid records in second grid record
  --------------------------------------------*/
grid2->in_lines = new_line_coord;
grid2->in_samps = new_samp_coord;
strcpy(grid2->model,"CONVOLVED GRID");
grid2->ncoeff = 0;
grid2->griderr_valid = GEO_NOTVALID;

return(E_SUCC);
}

/****************************************************************************
NAME:

PURPOSE:  Calculates bilinear equation coefficients for an input grid cell.  

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		7/88	D. Steinwand	   CSB      Original development--
							ditty for ADAPS newtem
  5.0		10/89	D. Steinwand	   CSB      Conversion to LAS 5.0

ALGORITHM DESCRIPTION:
	Calculate coefficients
	Return
*****************************************************************************/
void coef
(
    double tl,
    double tr,
    double bl,
    double br,			/* Corners of grid cell in one dimension */
    double *c0,
    double *c1,
    double *c2,
    double *c3,	/* Coefficients */
    long deltax,
    long deltay			/* Size of grid cell in X and Y */
)
{
*c0 = tl;
*c1 = (tr - tl) / deltax;
*c2 = (bl - tl) / deltay;
*c3 = (br - tr - bl + tl) / (deltax * deltay);
return;
}
