/****************************************************************************
NAME:				GRIDCONV

PURPOSE:  This function convolves two grids--it reads the polynomial from 
	  the first grid entered and applies it to the second to create an 
	  output grid to be used to avoid double resampling of an image.
	  This technique is especially useful when doing image-to-image
 	  products in a system where the raw search image must first be
	  corrected to a map projection, then warped to fit the reference
	  space.  In that case, instead of again resampling the nominally
	  corrected search image, one would run this function on the
	  raw search image using the geometric mapping grid to warp from
	  nominally (systematic) corrected imagary to reference space as 
	  the first input grid and the inital raw-to-nominal(systematic) 
	  grid as the second input grid.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		7/88	D. Steinwand	   CSB      Original development--
							ditty for ADAPS system
  5.0		10/89	D. Steinwand	   CSB      Conversion to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Call complete & exit

ALGORITHM REFERENCES:
*****************************************************************************/

#include <stdlib.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

#define GABORT c_errmsg("Fatal error encountered","gridconv-fatal",&fatal)

int main
(void)
{
struct GEOGRID grid1;		/* First Geometric mapping grid */
struct GEOGRID grid2;		/* Second Geometric mapping grid */
char fname[CMLEN];		/* Grid file name */
struct PARBLK vblk;		/* TAE variable block */
long std_out;			/* Standard output for c_inlas */
long acc = IREAD;		/* Access flag for c_getnam */
long zero = 0;			/* Constant of zero */
long one = 1;			/* Constant of one (1) */
long fatal = -1;		/* Failure condition for GABORT define */

/* Initialize tae parameter blocks, etc
  ------------------------------------*/
c_inlas(&vblk, &std_out);

/* Get the first input geometric mapping grid file name
  ----------------------------------------------------*/
if (c_getnam(&vblk,"INGRID", &zero, fname,"grid",&acc) != E_SUCC) GABORT;
if (c_getgrd(fname, &grid1) != E_SUCC) GABORT;

/* Get the input geometric mapping grid file name
  ----------------------------------------------*/
if (c_getnam(&vblk,"INGRID", &one, fname,"grid",&acc) != E_SUCC) GABORT;
if (c_getgrd(fname, &grid2) != E_SUCC) GABORT;

/* Get the output geometric mapping grid file name
  -----------------------------------------------*/
acc = IWRITE;
if (c_getnam(&vblk,"OUTGRID", &zero, fname,"grid",&acc) != E_SUCC) GABORT;

/* Convolve the two geometric mapping grid files
  ---------------------------------------------*/
if (conv_grid(&grid1, &grid2) != E_SUCC) GABORT;

/* Write the new geometric mapping grid file to disk
  -------------------------------------------------*/
if (c_putgrd(fname, &grid2) != E_SUCC) GABORT;

/* Print completion message and exit
  ---------------------------------*/
c_complt("GRIDCONV", fname, &one);

return EXIT_SUCCESS;

}
