/*****************************************************************************
NAME				oldstd 

PURPOSE    Reads a grid in the old EDC standard format

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand   10/88        EDC      Original development

PROJECT			LAS

ALGORITHM DESCRIPTION
	Open the input file
	Read the number of rows and columns from the grid
	Read the number of output lines and samples
	Check the grid dimensions
	Allocate memory for the rest of the grid
	Read in the output space intersections
	Read in the input space coordinate pairs
	Pack misc geometric mapping grid fields
	Close the input file
	Return

ALGORITHM REFERENCES		none
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

int oldstd
(
    char *name,		/* Name of input grid file */
    struct GEOGRID *grid	/* LAS geometric mapping grid structure */
)
{
FILE	*fp;		/* File pointer for grid file */
long i;			/* Loop counter */
long pass = 1;		/* Nonfatal error flag */
long nel;		/* Number of elements in input space buffers */

/* Open the input grid file
  ------------------------*/
fp = fopen(name,"r");
if (fp == NULL)
   {
   c_errmsg("Error opening input grid file","gridform-open",&pass);
   return(E_FAIL);
   }

/* Read the number of rows and columns from the grid
  -------------------------------------------------*/
if(fscanf(fp, "%d%d%d%d", &(grid->nrows), &(grid->ncols), &(grid->lines),
    &(grid->samples)) != 4)
   {
   c_errmsg("Error reading grid file","gridform-read",&pass);
   return(E_FAIL);
   }
if ((grid->nrows < 2) || (grid->nrows > 127) || (grid->ncols < 2)
   || (grid->ncols > 127))   
   {
   c_errmsg("Illegal number of rows or columns in grid","gridform-dim",&pass);
   return(E_FAIL);
   }


/* Calculate the amount of buffer space needed for the output grid intersections
   and the input grid coordinate pairs.  Then, allocate memory.
  ------------------------------------------------------------*/
nel = grid->nrows * grid->ncols;
grid->out_lines = (long *)malloc(grid->nrows * sizeof(long));
grid->out_samps = (long *)malloc(grid->ncols * sizeof(long));
grid->in_lines = (double *)malloc(nel * sizeof(double));
grid->in_samps = (double *)malloc(nel * sizeof(double));
if ((grid->out_lines == NULL) || (grid->out_samps == NULL) ||
    (grid->in_lines == NULL) || (grid->in_samps == NULL))
   {
   c_errmsg("Error allocating memory","gridform-alloc",&pass);
   return(E_FAIL);
   }

/* Read output space intersections
  -------------------------------*/
for (i = 0; i < grid->nrows; i++)
   if(fscanf(fp, "%d", &(grid->out_lines[i])) != 1)
      {
      c_errmsg("Error reading grid file","gridform-read",&pass);
      return(E_FAIL);
      }

for (i = 0; i < grid->ncols; i++)
   if(fscanf(fp, "%d", &(grid->out_samps[i])) != 1)
      {
      c_errmsg("Error reading grid file","gridform-read",&pass);
      return(E_FAIL);
      }

/* Read the input space line and sample coordinates
  ------------------------------------------------*/
for (i = 0; i < nel; i++)
   if(fscanf(fp, "%lf", &(grid->in_lines[i])) != 1)
      {
      c_errmsg("Error reading grid file","gridform-read",&pass);
      return(E_FAIL);
      }

for (i = 0; i < nel; i++)
   if(fscanf(fp, "%lf", &(grid->in_samps[i])) != 1)
      {
      c_errmsg("Error reading grid file","gridform-read",&pass);
      return(E_FAIL);
      }

/* Pack misc grid fields
  ---------------------*/
grid->proj_valid = GEO_NOTVALID;
grid->fiterr_valid = GEO_NOTVALID;
grid->griderr_valid = GEO_NOTVALID;
grid->ncoeff = 0;
grid->grid_tol = 0;
strcpy (grid->model,"FOREIGN");

fclose(fp);
return(E_SUCC);
}
