/****************************************************************************
NAME:				getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		10/88	D. Steinwand	   CSB      New development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize tae parameter blocks, etc
	Get the input geometric mapping grid file name
	Get the output geometric mapping grid file name
	Get the window specification.  This determines the output area of the
  	   output image described by the mapping grid 
	Get the number of lines and samples desired in each grid cell in the
	   output space.  These can have values or can be null.
	Get the tolerance value for grid reduction
	Get the message timer

ALGORITHM REFERENCES:
*****************************************************************************/
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "local_protos.h"

int getpar
(
    char *in_name,			/* Input grid file name */
    char *out_name,			/* Output grid file name */
    int  *window,			/* Mapping grid window (size) */
    double *tolerance,		/* Tolerance for linear approximation */
    long *nglines,          /* Number of lines in each cell */
    long *ngsamps,          /* Number of samples in each cell */
    long *msgtim 			/* Message time variable */
)
{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk */
struct PARBLK vblk;		/* TAE variable block */
long std_out;			/* standard output for c_inlas */
long acc;			/* access flag for c_getnam */
long count;			/* Loop counter */
long zero = 0;			/* Constant of zero */
long nonfatal = 1;


/* Initialize tae parameter blocks, etc
  ------------------------------------*/
c_inlas(&vblk, &std_out);

/* Get the input geometric mapping grid file name
  ----------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk,"INGRID", &zero, in_name,"grid",&acc) != E_SUCC)
	return(NO_INGRID);

/* Get the output geometric mapping grid file name
  -----------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk,"OUTGRID", &zero, out_name,"grid",&acc) != E_SUCC)
	return(BAD_OUTGRID);

/* Get the window specification.  This determines the output area of the
   output image described by the mapping grid 
  ------------------------------------------*/
var = p_find(&vblk, "WINDOW");
for (count = 0; count < 4; count++)
{
   window[count] = IVAL(*var, count);
   if (window[count] < 1)
   {
      c_errmsg("WINDOW coordinates must be greater than zero","gridgen-invalid",
               &nonfatal);
      return (E_FAIL);
   }
}

/* Get the number of lines and samples desired in each grid cell in the
   output space.  These can have values or can be null.
  ---------------------------------------------------*/
var = p_find(&vblk, "NLGRID");
if ((*var).v_count > 0)
{
   *nglines = IVAL(*var,0);
   if (*nglines < 1)
   {
      c_errmsg("NLGRID must be greater than zero","gridgen-invalid", &nonfatal);
      return (E_FAIL);
   }
}
else
   *nglines = 0;

var = p_find(&vblk, "NSGRID");
if ((*var).v_count > 0)
{
   *ngsamps = IVAL(*var,0);
   if (*ngsamps < 1)
   {
      c_errmsg("NSGRID must be greater than zero","gridgen-invalid", &nonfatal);
      return (E_FAIL);
   }
}
else
   *ngsamps = 0;


/* Get the tolerance value for grid reduction
  ------------------------------------------*/
var = p_find(&vblk, "TOLVAL");
*tolerance = RVAL(*var, 0);

/* Get the message timer
  ---------------------*/
var = p_find(&vblk, "$msgtim");
*msgtim = IVAL(*var, 0);

return(E_SUCC);
}
