/****************************************************************************
NAME:			      MAKE_GRID	

PURPOSE:  Tests one-to-one property over the user specified window
	  by computing the Jacobian and then, if one-to-one, compute
	  input & output grid values

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		 5/84	M. Nishihama	   SAR	    Original Development
  5.0		10/88	D. Steinwand	   CSB      Conversion to C

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Find polynomial degree & unpack coefficients
	Create the grid
	Compute the Jacobian of transformation at the initial point
	Get the sign of the Jacobian
	If the Jacobian is zero
	   Return error (transformation is not one-to-one)
	Loop on NL
	   Get Y coordinate
	   Loop on NS
		Get X coordinate
		Compute Jacobian of transformation at (X,Y)
		Get the sign of the Jacobian
		If the Jacobian is not equal to initial Jacobain
		   Return (transformation is not one-to-one)
	Return(success)
*****************************************************************************/
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "local_protos.h"

typedef struct
{
    long degree;  /* degree of polynomial for coefficient arrays */
    double *aa;   /* output line/sample to input sample coefficients */
    double *bb;   /* output line/sample to input line coefficients */
} OLS2ILS_PARAMS;

static int ols2ils
(
    void *data,     /* I: pointer to a OLS2ILS_PARAMS structure for the 
                          parameters specific to this gridding application */
    int oline,      /* I: output space line */
    int osamp,      /* I: output space sample */
    double *iline,  /* O: input space line */
    double *isamp   /* O: input space sample */
)
{
    OLS2ILS_PARAMS *ols2ils_data = (OLS2ILS_PARAMS *)data;
    long degree = ols2ils_data->degree;
    double xval = (double)osamp;
    double yval = (double)oline;

    *isamp = c_eval(&degree, ols2ils_data->aa, &xval, &yval);
	*iline = c_eval(&degree, ols2ils_data->bb, &xval, &yval);

    return (E_SUCC);
}


int make_grid
(
    struct GEOGRID *grid,		/* Geometric mapping grid */
    int  *wind, 			    /* User specified window */
    int  nl,	    		    /* Number of lines per grid cell */
    int  ns,	    		    /* Number of samples per grid cell */
    double tolerance,           /* Error tolerance (in pixels) for grid 
                                   generation */
    long msgtim		        	/* Processing message timer */
)
{
double aa[15],bb[15];		/* Polynomial coefficients */
double dx,dy;			/* x & y values */
double jacobian;		/* Jacobian of transformation */
double p11,p12,p21,p22;		/* Temp vars for Jacobian test */
long degree;			/* Degree of polynomial */
long cnt, count;		/* Loop counters */
long init_sign, jac_sign;	/* Sign of Jacobian; inital val, working val */
long one = 1;			/* Constant of one */
long three = 3;			/* Constant of three */
long nel;			/* Number of grid elements */
double old_corners[8];		/* Input mapping grid corner proj coords */
double dwind[4];		/* Window spec; double */
long image[8];			/* Line, sample locations of old_corners */
OLS2ILS_PARAMS ols2ils_data; /* parameters for the ols2ils routine */

extern long curent;		/* Current "line" number for processing mess */

/* Save old grid framing for corners update
  ----------------------------------------*/
image[0] = grid->out_lines[0];
image[1] = grid->out_samps[0];
image[2] = (grid->out_lines[0] + grid->lines) - 1;
image[3] = grid->out_samps[0];
image[4] = grid->out_lines[0];
image[5] = (grid->out_samps[0] + grid->samples) - 1;
image[6] = (grid->out_lines[0] + grid->lines) - 1;
image[7] = (grid->out_samps[0] + grid->samples) - 1;

/* If the grid is modeled on a polynomial, find it's degree (1, 2, 3, or 4)
   and unpack coefficients
  -----------------------*/
if (strcmp(grid->model,"POLYNOMIAL") == 0)
   {
   if (grid->ncoeff == 3) degree = 1;
   else if (grid->ncoeff == 6) degree = 2;
   else if (grid->ncoeff == 10) degree = 3;
   else if (grid->ncoeff == 15) degree = 4;
   else
	return(BAD_DEGREE);

   for (count = 0; count < grid->ncoeff; count++)
	{
	bb[count] = grid->coeffs[count];
	aa[count] = grid->coeffs[count + grid->ncoeff];
	}
   }
else
   return(NOT_POLY);

/* setup the parameters for the ols2ils call */
ols2ils_data.degree = degree;
ols2ils_data.aa = aa;
ols2ils_data.bb = bb;

/* build the grid */
if (c_makegrid(wind, nl, ns, ols2ils, &ols2ils_data, tolerance, "GRIDGEN",
               msgtim, grid) != E_SUCC)
{
    c_lasmsg(FALSE,"makegrid-fail","Error returned from c_makegrid");
    return (E_FAIL);
}

/* Compute the Jacobian of transformation at inital point
  ------------------------------------------------------*/
dx = (double)wind[1];
dy = (double)wind[0];

p11 = derivative(1, degree, aa, dx, dy);
p12 = derivative(2, degree, aa, dx, dy);
p21 = derivative(1, degree, bb, dx, dy);
p22 = derivative(2, degree, bb, dx, dy);
jacobian = (p11 * p22) - (p12 * p21);
init_sign = (jacobian < 0.0) ? -1 : 1;

/* If the Jacobian is zero, return since transform is not one-to-one
  -----------------------------------------------------------------*/
if (jacobian == 0.0) return(DEGENERATE);

/* Check Jacobian on rest of grid
  ------------------------------*/
nel = grid->nrows * grid->ncols;
curent = 1;
c_inittm("GRIDGEN","Performed one-to-one test at","grid element","","",&nel,
         &one, &msgtim, &three);
c_startm();
for (count = 0; count < grid->nrows; count++)
  for (dy = grid->out_lines[count], cnt = 0; cnt < grid->ncols; cnt++, curent++)
	{
	dx = grid->out_samps[cnt];
	p11 = derivative(1, degree, aa, dx, dy);
	p12 = derivative(2, degree, aa, dx, dy);
	p21 = derivative(1, degree, bb, dx, dy);
	p22 = derivative(2, degree, bb, dx, dy);
	jacobian = (p11 * p22) - (p12 * p21);
	jac_sign = (jacobian < 0.0) ? -1 : 1;

	if ((jacobian == 0) || (jac_sign != init_sign)) return(DEGENERATE);
	}
c_stoptm();

/* Update corner projection coordinates to reflect new grid
  --------------------------------------------------------*/
if (grid->proj_valid == GEO_VALID)
   {
   for (count = 0; count < 8; count++)
      old_corners[count] = grid->corners[count];
   for (count = 0; count < 4; count++)
      dwind[count] = (double)wind[count];
   if (c_rotcor(old_corners, image, dwind, grid->corners) != E_SUCC)
	grid->proj_valid = GEO_NOTVALID;
   }

return(E_SUCC);
}
