#include "hindu.h"
#include "protos.h"

void classify
(
    char taeout[],		/* output image file tae name                 */
    long npix,			/* number of pixels per line                  */
    long nlin,			/* number of lines in the input image         */
    long ncells,			/* number of occupied histogram bins          */
    long signum,			/* number of significant clusters             */
    float *hist[MAXBND],		/* multi-dimensional histogram values         */
    float *xcentr[MAXBND],	/* cluster centroids                          */
    long nbands,			/* number of bands                            */
    float scale[6001],		/* cluster significance values                */
    long print[]                /* print options                              */
)
/*
*****************************************************************************

NAME: CLASSIFY

PURPOSE:  Reads the temporary image file and reassigns each pixel to the
          cluster which is his nearest neighbor; writes the final assignement
          to the output image file and a summary of the classification results
          to the line printer.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         08-Jul-86   N. Deodhar/SAR #212
   3.0         19-May-89   K. Kunska/STX	conversion to 5.0
   5.1          8-NOV-89   P. Pease/GSFC        Changed call from errmsg to
                                                c_errmsg.
   5.1          6-Aug-93   D. Bridges           Added Print paramenter

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Allocate dynamic memory for work buffer.
   Build a lookup table for pixel reassignments to nearest significant 
   neighbour.
   For each line 
      Read the temporary image and assign the pixel values 
      according to the lookup table.
      Create a new histogram.
   End-for
   Write the summary of the classification results.
   Free the dynamic memory.

*****************************************************************************
*/

{
char	text[130];		/* text buffer for line printer output        */

double	dmin,			/* minimum difference between two histogram   */
				/* values (current and significant)           */
	exp,			/* exponenet for math. operations             */
	hdiff,			/* difference of the histogram values         */
	sumsq;			/* square of the differences between two      */
				/* histogram values                           */

long	band,			/* current band index                         */
	cell;			/* current histogram cell                     */
static	long 	chist[6001];	/* histogram based on the classified output   */
				/* image                                      */
long	count,			/* number of values in he histogram array     */
	dtype,			/* image data type                            */
	i,			/* loop counter and array index               */
	line,			/* current line                               */
	lmin,			/* cluster corresponding to the smallest      */
				/* difference between two histogram bins      */
	ltype,			/* lookup table entry type                    */
	sig,			/* current significant cluster                */
	work_size;		/* size of the work buffer                    */

long fatal_err = -1;     /* fatal error flag                      */
static	short	lut[6001];	/* lookup table                               */
short	*work;			/* work buffer for translated pixels          */


/*
Allocate dynamic memory for work buffer.
------------------------------------------
*/
work_size = (long) (npix * sizeof(short));
if ((work = (short *) malloc (work_size)) == NULL)
   c_errmsg ("Cannot allocate dynamic memory for work buffer", "hindu-alloc",
            &fatal_err);
/*
Build a lookup table for pixel re-assignments to nearest significant neighbour.
-------------------------------------------------------------------------------
*/
exp = 2.0;

for (i = 0; i < 6001; i++)
   lut[i] = 0;

for (cell = 0; cell < ncells; cell++)
   {
   dmin = 1.0e10;
   for (sig = 0; sig < signum; sig++)
      {
      sumsq = 0.0;
      for (band = 0; band < nbands; band++)
         {
         hdiff = hist[band][cell] - xcentr[band][sig];
         sumsq = sumsq + pow (hdiff, exp);
         }  
      if (sumsq < dmin)
         {
         dmin = sumsq;
         lmin = sig;
         }  
      }  
   lut[cell] = (short) (lmin+1);
   }  /* do */

/*
For each line, read the temporary image and assign the pixel values 
according to the lookup table.
Create a new histogram.
---------------------------------------------------------------------
*/
count = 0;
dtype = EBYTE;
ltype = EWORD;
for (i = 0; i < 6001; i++)
   chist[i] = 0;
for (line = 0; line < nlin; line++)
   {
   c_estep (&gdesctmp);
   c_estep (&gdescout);
   c_pxlkup (&tempbuf[tpoint[0]], work, &npix, &dtype, lut, &ltype);
   c_pxhist (&npix, work, &ltype, chist, &count);
   c_pxconv (&ltype, &dtype, work, &outbuf[opoint[0]], &npix);
   }
c_estep (&gdesctmp);
c_estep (&gdescout);

/*
Write the summary of the classification results.
------------------------------------------------
*/

if (print[LP] || print[FIL] || print[TERM])
{
fprintf (fd1,"SUMMARY OF HINDU CLASSIFICATION RESULTS.\n");
fprintf  (fd1, "OUTPUT FILE NAME: %s\n", taeout);
fprintf  (fd1, "\nCLUSTER #     POPULATION   SIGNIFICANCE         CENTROID\n");
sprintf  (text, "                                              ");
for (band = 1; band <= nbands; band++)
   sprintf (&text[strlen(text)], "%3d    ", band);
fprintf (fd1,"%s\n",text);
for (sig = 0; sig < signum; sig++)
   {
   sprintf (text, "%9d     %10d   %11.7f       ", sig+1, chist[sig+1], 
            scale[sig]);
   for (band = 0; band < nbands; band++)
      sprintf (&text[strlen(text)], "%6.2f ", xcentr[band][sig]);
   fprintf (fd1,"%s\n",text);
   }
}
/*
Free the dynamic memory.
--------------------------
*/
free(work);
}
