#include "hindu.h"
#include "protos.h"

void cluster
(
    long npix,			/* number of pixels per line                  */
    long nlin,			/* number of lines in the input image         */
    long ncells,			/* number of occupied histogram bins          */
    long ktab[6001], 		/* locations of sorted hnum values            */
    long hnum[6001], 		/* number of pixels per histogram bin         */
    float *hist[MAXBND], 		/* multi-dimensional histogram gray levels   */
    long *repeat,		/* indicates if the number of clusters is     */
    long nbands,			/* number of bands in the input image         */
    long *signum,		/* number of significant clusters             */
    float *xcentr[MAXBND],	/* cluster centroids                         */
    float scale[6001]		/* array of significance values              */
)

/*
*****************************************************************************

NAME: CLUSTER

PURPOSE:  Finds sets of siginficant bins (clusters) and their significance
          values and centroids.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         12-Jan-84   K. McGee		SCN#20-145
   3.0         19-May-89   K. Kunska		conversion to 5.0
   4.0	       09-Sep-89   K. Kunska		corrected parameter names in 
                                                error messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   For all occupied bins, 
      Find cells with greater than average density.
      For all significance levels 
         For all bands, 
            Find the maximum and minimum differences between histogram 
            values of current and siginificant bins.
         End-for
      End-for
      Compute the significance of each cluster and store it in the 
      scale array.
   End-for
   Call SORT to sort the significance values from high to low.
   Check to see if the number of clusters found falls within the user
   specified range.
   If too many clusters were found, exit to the main program to recalculate
   the histogram or, if that was already done, discard the clusters with the 
   lowest significance values.
   If too few clusters were found, exit to the main program to recalculate
   the histogram, or, if that was already done, ask the user to restart HINDU
   with the new values for CLUSTER and GRANGE parameters.
   If the number of clusters found falls within the user specified range, 
   allocate the dynamic memory for the centroid array and calculate the 
   cluster centroids.

*****************************************************************************
*/

{

long    one = 1;
char    *prog_name = "HINDU"; /* program name                 */


char	bfill,			/* blank fill value                           */
	text[80];		/* text buffer for the error messages         */

double	exp,			/* exponent for math. calculations            */
	hdiff;			/* difference between two histogram values    */

float	sumsq,			/* sum of squares of differences between two  */
				/* histogram values                           */
	xmax,			/* maximum difference between histogram values*/
	xmin;			/* minimum difference between histogram values*/

long	band,			/* current band index                         */
	cell,			/* current histogram cell                     */
	cnums[6001],		/* bin number for each significant cluster    */
	density,		/* average density of pixels in bin           */
	ftype,			/* fill value data type                       */
	i, j, 			/* loop counters and array indeces            */
	loc,			/* location of the current hnum value         */
				/* maximum acceptable number                  */
	sbin,			/* bin number for the current signif. cluster */
	sig,			/* current significance                       */
	sloc,			/* location of the density value for the      */
				/* significant cluster                        */
	txtsize,		/* text buffer size                           */
	xbuf_size;		/* buffer size for the centroid array         */

long fatal_err = -1; /* fatal error return status */
long inf_err = 1;   /* info error return status                   */

/*
Initialize variables.
----------------------
*/
exp = 2.0;
*signum = 0;
bfill = ' ';
ftype = EBYTE;
txtsize = 80;
density = npix * nlin / ncells;
for (i = 0; i < 6001; i++)
   {
   cnums[i] = 0;
   scale[i] = 0;
   }

/*
For all occupied bins, find cells with greater than average density.
--------------------------------------------------------------------
*/
for (cell = 0; cell < ncells; cell++)
   {
   if (hnum[cell] >= density)
      {
/*
For all significance levels and all bands, find the maximum and minimum
differences between histogram values of current and siginificant bins.
Compute the significance of each cluster and store it in the scale array.
---------------------------------------------------------------------------
*/
      loc = ktab[cell];
      cnums[*signum] = cell;
      scale[*signum] = (float)hnum[cell] / (float)hnum[0];
      xmax = 0.0;
      xmin = 1.0e10;
      if (*signum > 0)
         {
         for (sig = 0; sig <= *signum-1; sig++)
            {
            sumsq = 0.0;
            sbin = cnums[sig];
            sloc = ktab[sbin];
            for (band = 0; band < nbands; band++)
               {
               hdiff = hist[band][loc] - hist[band][sloc];
               sumsq = sumsq + pow (hdiff, exp);
               }
            if (sumsq > xmax)
               xmax = sumsq;
            if (sumsq < xmin)
               xmin = sumsq;
            }  
         if (xmax != 0.0)
            scale[*signum] = scale[*signum] * xmin / xmax;
         }  /* if */
      *signum = *signum + 1;
      }  /* if */
   }  /* do */

/*
Call SORT to sort the significance values from high to low.
------------------------------------------------------------
*/
sort (scale, cnums, *signum);

/*
Check to see if the number of clusters found falls within the user
specified range.
If too many clusters were found, exit to the main program to recalculate
the histogram or, if that was already done, discard the clusters with the 
lowest significance values.
If too few clusters were found, exit to the main program to recalculate
the histogram, or, if that was already done, ask the user to restart HINDU
with the new values for CLUSTER and GRANGE parameters.
---------------------------------------------------------------------------
*/
if (*signum > maxclus)
   {
   c_pxfill (text, &ftype, &txtsize, &bfill);
   sprintf (text, "%4d clusters found. Exceeds maximum specified.", *signum);
   c_errmsg (text, "hindu-maxclus", &inf_err); 
   if ((init_grid == max_grid) || *repeat)
      {
      c_pxfill (text, &ftype, &txtsize, &bfill);
      sprintf (text, "Reducing number of clusters to maximum %4d.", maxclus);
      c_errmsg (text, "hindu-reduce", &inf_err); 
      *signum = maxclus;
      }
   else
      {
      c_pxfill (text, &ftype, &txtsize, &bfill);
      sprintf (text, "Recalculating histogram with maximum value for NLEVBIN");
      c_errmsg (text, "hindu-compmax", &inf_err); 
      init_grid = max_grid;
      *repeat = TRUE;
      return;
      }  
   }
else if (*signum < minclus)
   {
   c_pxfill (text, &ftype, &txtsize, &bfill);
   sprintf (text, "%4d clusters found. Less than minimum specified.", 
            *signum);
   c_errmsg (text, "hindu-minclus", &inf_err); 
   if ((init_grid == min_grid) || *repeat)
      {
      c_pxfill (text, &ftype, &txtsize, &bfill);
      sprintf (text, "Restart with new values for NLEVBIN and MINMAXCL.");
      c_errmsg (text, "hindu-cluster", &inf_err); 
      c_clean (&block, prog_name, tmp_files, &one);
      }
   else
      {
      c_pxfill (text, &ftype, &txtsize, &bfill);
      sprintf (text, "Recalculating histogram with minimum value for NLEVBIN.");
      c_errmsg (text, "hindu-compmin", &inf_err); 
      init_grid = min_grid;
      *repeat = TRUE;
      return;
      }  /* if */
   }  /* if */

/*
If the number of clusters falls within the user specified range, 
allocate the dynamic memory for the centroid array and calculate the 
cluster centroids.
---------------------------------------------------------------------
*/
xbuf_size =(long) ( MAXBND * *signum * sizeof(float) );
if ((cbuffer = (float *) malloc (xbuf_size)) == NULL)
   c_errmsg ("Cannot allocate dynamic memory for centroids", "hindu-alloc",
              &fatal_err);
for (i = 0, j = 0; i < MAXBND; i++, j += *signum)
   xcentr[i] = &cbuffer[j];

for (j = 0; j < nbands; j++)
   for (i = 0; i < *signum; i++)
      xcentr[j][i] = 0.0;

for (j= 0; j < *signum; j++)
   {
   cell = ktab[cnums[j]];
   for (i = 0; i < nbands; i++)
      xcentr[i][j] = hist[i][cell];
   }  

printf ("\nTotal number of classes found = %4d.\n", *signum);
 

}
