#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"


#define NR      0                       /* number of lines in local window    */
#define NP      1                       /* number of pixels in local window   */

#define TERM    0                       /* directs output to terminal         */
#define FIL     1                       /* directs output to file             */
#define LP      2                       /* directs ouput to line printer      */

struct  PARBLK  block;          /* TAE parameter block                */
struct  DDR     ddr;            /* ddr                                */
FILE    *fd1;                   /* print file descriptor              */

unsigned char   *inbuf;         /* image input buffer                 */
unsigned char   *outbuf;        /* image output buffer                */
unsigned char   *tempbuf;       /* temporary image buffer             */

static  char    *prog_name = "HINDU"; /* program name                 */

char    out_files[2][CMLEN];    /* output file names                  */
char    tmp_files[2][CMLEN];    /* temporary file names               */
struct FDESC *  fdescin[MAXIMG+1];      /* input file descriptors             */
struct FDESC *  fdescout[2];            /* output file descriptor             */
struct FDESC *  fdesctmp[2];            /* temporary file descriptor          */

struct GDESC *  gdescin;                /* input group descriptor             */
struct GDESC *  gdescout;               /* output group descriptor            */
struct GDESC *  gdesctmp;               /* temprary group descriptor          */
long    ipoint[MAXIMG*MAXBND+1],/* offsets into input image buffer    */
                opoint[MAXBND],         /* offsets into output image buffer   */
                tpoint[MAXBND];         /* offsets into temporary image buffer*/

long	print[3];		/* print option : TERM LP FIL         */
long    min_max;               	/* minmax option flag                 */
long    msgtim;                 /* message interval                   */
long    num_outfiles;      	/* number of output files             */
long    num_tmpfiles;           /* number of temporary files          */

extern long     curent;                 /* current entity                     */
extern long     curcat;                 /* current category                   */

static  long    one = 1;
long    window[MAXIMG][4];    /* input image dimensions (SL SS NL NS) */

long    minclus,                /* minimum number of clusters         */
                maxclus,                /* maximum number of clusters         */
                init_grid,              /* initial gray level value for       */
                                        /* histogram bins                     */
                max_grid,               /* maximum gray level value for       */
                                        /* histogram bins                     */
                min_grid;               /* minimum gray level value for       */
                                        /* histogram bins                     */

float   *cbuffer;               /* pointer to the centroid array      */


int main
(void)

/*
*****************************************************************************

NAME: HINDU

PURPOSE:  Performs an unsupervised classification of an image based on its
          multidimensional histogram.

PROGRAM HISTORY:

  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         08-Jul-86   N. Deodhar/SAR #212
   3.0         19-May-89   K. Kunska/STX	conversion to 5.0
   4.0	       09-Sep-89   K. Kunska/STX	corrected parameter names in
                                                error messages in CLUSTER.C
   5.0	       26-Oct-89   T. Baltzer/EDC       dynamic allocation of large 
						arrays, declare line_inc and
						samp_inc as double for call to
						c_upddr()
   5.1          8-NOV-89   P. Pease/GSFC        changed call to c_errmsg from
                                                errmsg.
   5.1          3-Apr-90   K. Kunska/STX        Fixed a min-max problem
   5.1          6-Aug-93   D. Bridges           Added print parameter
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Get tae parameters.
   Call TEMPFILE to set up the temporary output file.
   Allocate dynamic memory for the histogram array.
   Initialize the histogram to 0.
   While iterate flag is true
      Call INSPEC to open the input image.
      Call TEMPSPEC to open the temporary output image.
      Call MULTHIST TO create a multi-dimensional histogram.
      Close the input and temprary output images.
      Call MERGE to merge histogram cells with their higher density neighbours.
      Call CLUSTER to find sets of significant cells.
      If the number of significant cells is outside of the user specified 
      range (minclus - maxclus), then repeat the above process with a 
      different histogram grid size, provided that the histogram was not 
      already recalculated in MULTHIST due to insufficient storage space.
   End-while
   Call OUTSPEC to open the output image.
   Call TEMPSPEC to open the temporary image as input image.
   Call CLASSIFY to classify the input image by assigning each pixel 
   to the cluster which is its nearest significant neighbour.
   Close the images and delete the temprorary image file.
   Free the dynamic memory.
   Determine the minimum and maximum pixel values of the output image.
   Write the min, max, and the validity flag values to the ddr file.
   Generate the history file and print the completion message.

*****************************************************************************
*/

{

char	hostin[MAXIMG][CMLEN];	  		/* input image host name              */
char 	hostemp[CMLEN];			/* temporary image host name          */
char	hostout[1][CMLEN];			/* output image host name             */
char	print_file[CMLEN];			/* print file name                    */
char	taein[MAXIMG][CMLEN];   /* input image tae name               */
char 	taeout[1][CMLEN]; 			/* output image tae name              */

double	amax[MAXBND];				/* maximum pixel value                */
double	amin[MAXBND];				/* minimum pixel value                */
double  histloc[6001];			/* location of each occupied histogram*/
 					/* bin in histogram space             */

float	*hbuffer;			/* buffer for histogram array         */
float   *hist[MAXBND];			/* multi-dimesional histogram image   */
float	scale[6001];
float 	*xcentr[MAXBND];		/* centroids corresponding to each    */
					/* histogram bin                      */

long	bands_per_img[MAXIMG];		/* number of bands for each image     */
long    buf_size;			/* size of cbuffer and hbuffer        */
long	hnum[6001]; 				/* number of pixels in each histogram */
      					/* bin                                */
long    i, j;				/* loop counters and array indeces    */
long	img_flag[MAXIMG][2];		/* flag for window/band option        */
long	img_limit;			/* maximum number of images           */
long	iterate;			/* flag set to true if the space      */
					/* allocated for the histogram is full*/
long	ktab[6001];				/* original locations of hnum values  */
                                        /* before sorting                     */
long	loop_flag;			/* controls execution of main while   */
long	ncells;				/* number of occupied histogram bins  */
long	num_img_bands[MAXIMG][MAXBND+1];/* band numbers for each input image  */
long	num_inimg;			/* number of input images             */
long	num_lines;			/* number of input image lines        */
long	num_pixels;			/* number of pixels per line          */
long	out_bands;			/* number of output image bands       */
long	out_img_bands[MAXIMG][MAXBND+1];/* band numbers for each output image */
long	outwin[1][4];			/* output window specifications       */
long	repcnt;				/* repetition count for recalculating */
					/* of the histogram (only 1 recalcula-*/
					/* tion allowed)                      */
long	repeat; 			/* flag to abort when recalculating   */
					/* histogram when max value for grange*/
					/* fails to provide enough space      */
long	same_wnd;			/* same window size: 1=YES, 0=NO      */
long	signum;				/* number of significant clusters     */
long	temp_bands;			/* number of bands in the temp image  */
long	tot_inband;			/* total number of input bands        */


long	fatal_err = -1;			/* fatal error flag	              */


/*
Initialize variables.
----------------------
*/
img_limit = MAXIMG;
iterate = FALSE;
repeat = FALSE;
strcpy(tmp_files[0], "temp_dat.img");
num_tmpfiles = 1;

/*
Get tae parameters.
--------------------
*/
getpar (taein, hostin, taeout, hostout, hostemp, &num_inimg, 
        num_img_bands, bands_per_img, &tot_inband, img_limit, &same_wnd,
        img_flag, print, print_file);

/*
Call TEMPFILE to set up the temporary output file.
---------------------------------------------------
*/
temp_bands = 1;
tempfile (hostemp, &temp_bands, hostin, num_inimg, num_img_bands, 
          bands_per_img);

/*
Allocate dynamic memory for the histogram array.
Initialize the histogram to 0.
--------------------------------------------------
*/
buf_size = MAXBND * 6001 * sizeof(float);
if ((hbuffer = (float *) malloc (buf_size)) == NULL)
     c_errmsg ("Cannot allocate dynamic memory for histogram buffer",
               "hindu-alloc", &fatal_err);

for (i = 0, j = 0; i < MAXBND; i++, j += 6000)
   hist[i] = &hbuffer[j];

for (j = 0; j < tot_inband; j++)
   for (i = 0; i < 6001; i++)
      hist[j][i] = 0.0;

/*
Call INSPEC to open the input image.
Call TEMPSPEC to open the temporary output image.
Call MULTHIST TO create a multi-dimensional histogram.
Close the input and temprary output images.
------------------------------------------------------
*/
repcnt = 0;
loop_flag = TRUE;
num_lines = window[0][NL];
num_pixels = window[0][NS];

while (loop_flag)
   {
   iterate = TRUE;
   while (iterate)
      {
      iterate = FALSE;
      inspec (hostin, num_inimg, num_img_bands, bands_per_img);
      tempspec (hostemp, temp_bands);
      multhist (hist, histloc, hnum, &ncells, tot_inband, num_pixels, 
                num_lines, repeat, &iterate);
      c_egclse (&gdesctmp);
      c_egclse (&gdescin);
      }

/*
Call MERGE to merge histogram cells with their higher density neighbours.
--------------------------------------------------------------------------
*/
   merge (hist, histloc, hnum, tot_inband, ncells, ktab);

/*
Call CLUSTER to find sets of significant cells.
-----------------------------------------------
*/
   cluster (num_pixels, num_lines, ncells, ktab, hnum, hist, &repeat, 
            tot_inband, &signum, xcentr, scale);

/*
If the number of significant cells is outside of the user specified 
range (minclus - maxclus), then repeat the above process with a 
different histogram grid size, provided that the histogram was not 
already recalculated in MULTHIST due to insufficient storage space.
----------------------------------------------------------------------
*/
   if (repeat && repcnt == 0)
      {
      iterate = TRUE;
      repcnt = 1;
      }  
   else
      loop_flag = FALSE;
   }

/*
Call OUTSPEC to open the output image.
Call TEMPSPEC to open the temporary image as input image.
Call CLASSIFY to classify the input image by assigning each pixel 
to the cluster which is its nearest significant neighbour.
-----------------------------------------------------------------
*/
out_bands = 1;
outspec (hostin,hostout[0], num_inimg, num_img_bands, 
         out_img_bands, out_bands, bands_per_img);
tempspec (hostemp, temp_bands);
classify (taeout[0], num_pixels, num_lines, ncells, signum, hist, xcentr, 
          tot_inband, scale, print);

/*
Close the images and delete the temprorary image file.
----------------------------------------------------------
*/
c_egclse (&gdescout);
c_egclse (&gdesctmp);
c_delfil (hostemp);

/*
Free the dynamic memory.
------------------------
*/
free (cbuffer);
free (hbuffer);
   
/* 
Determine the minimum and maximum pixel values of the output image.
Write the min, max, and the validity flag values to the ddr file.
--------------------------------------------------------------------
*/
if (min_max)
   {
   outwin[0][SL] = outwin[0][SS] = 1;
   outwin[0][NL] = num_lines;
   outwin[0][NS] = num_pixels;
   c_minmax (hostout, &one, outwin, out_img_bands, &out_bands, amin, amax);
   c_svmnmx (hostout[0], &one, amin, amax, "YES");
   }

/*
Generate the history file and print the completion message.
------------------------------------------------------------
*/
c_hstout (&block, hostout[0], hostin, &num_inimg, " ");
c_complt (prog_name, out_files, &num_outfiles);   

return EXIT_SUCCESS;

}
