/*******************************************************************************
FUNCTION:	process

PURPOSE:	Compare the input images

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"	
#include "util.h"
#include "protos.h"


/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc1[];	 /* image file descriptors used for image I/O */
extern struct FDESC * fdesc2[];	 /* image file descriptors used for image I/O */
extern struct GDESC * gdesc1;	 /* group descriptor used for image I/O       */
extern struct GDESC * gdesc2;	 /* group descriptor used for image I/O       */
extern long offset1[];           /* offsets into "thebuf" for input bands     */
extern long offset2[];           /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


int process
(
    char hosin1[][CMLEN],	/* Host file names for IN(1)                  */
    long nmrimg1,		/* Number of input images in IN(1)            */
    long window1[][4],		/* Windows for all image in IN(1)             */
    long bands1[][MAXBND+1],	/* Bands specified for each image in IN(1)    */
    long *nbands1,		/* Number of bands for each image in IN(1)    */
    long totbnd,		/* Total number of bands to be compared       */
    char hosin2[][CMLEN],	/* Host file names for IN(1)                  */
    long nmrimg2,		/* Number of input images in IN(2)            */
    long window2[][4],		/* Windows for all image in IN(2)             */
    long bands2[][MAXBND+1],	/* Bands specified for each image in IN(2)    */
    long *nbands2,		/* Number of bands for each image in IN(2)    */
    long msgtim,		/* Elapsed time between processing messages   */
    double tolval		/* Tolerance value                            */
)
{
char str[CMLEN];		/* Error message string                       */

double diff;			/* Difference between the current pixels      */

float pixel1;			/* Pixel value from the first image           */
float pixel2;			/* Pixel value from the second image          */
float *thebuf1;			/* Buffer for the first image                 */
float *thebuf2;			/* Buffer for the second image                */

long access	= IREAD;	/* Access mode                                */
long band;			/* Loop control                               */
long bufsiz1;			/* Size of the buffer for the first image     */
long bufsiz2;			/* Size of the buffer for the second image    */
long dtype	= EREAL;	/* Process all images as float images         */
long fatal	= -1;		/* Fatal error flag                           */
long image;			/* Loop control                               */
long msgflag	= ALLCAT;	/* Flag for processing message                */
long nl;			/* Number of lines                            */
long nonfatal	= 1;		/* Nonfatal error flag                        */
long ns;			/* Number of samples                          */
long sample;			/* Loop control                               */
long sl;			/* Starting line                              */
long ss;			/* Starting sample                            */
long status;			/* Return status from functions               */
long zero	= 0;		/* Value of zero                              */



/*
  Open the first set of images
*/
for (band = 0, image = 0; image < nmrimg1; image++)
    {
    sl = window1[image][SL];
    ss = window1[image][SS];
    nl = window1[image][NL];
    ns = window1[image][NS];
    status = c_eopens(&fdesc1[image], hosin1[image], bands1[image], &sl, &ss,
	&nl, &ns, &offset1[band], &access, &dtype, &zero, &zero);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    band += nbands1[image];
    }
fdesc1[nmrimg1] = 0;


/*
  Open the second set
*/
for (band = 0, image = 0; image < nmrimg2; image++)
    {
    sl = window2[image][SL];
    ss = window2[image][SS];
    nl = window2[image][NL];
    ns = window2[image][NS];
    status = c_eopens(&fdesc2[image], hosin2[image], bands2[image], &sl, &ss,
	&nl, &ns, &offset2[band], &access, &dtype, &zero, &zero);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    band += nbands2[image];
    }
fdesc2[nmrimg2] = 0;


/*
  Allocate I/O buffers
*/
if ((thebuf1 = (float *)c_ealloc(fdesc1, &bufsiz1)) == NULL)
    {
    c_errmsg("Error allocating dynamic memory", "imgcmp-alloc", &nonfatal);
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    }
if ((thebuf2 = (float *)c_ealloc(fdesc2, &bufsiz2)) == NULL)
   {
   free(thebuf1);
   c_errmsg("Error allocating dynamic memory", "imgcmp-alloc", &nonfatal);
   c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
   }


/*
  Group the images
*/
if (c_egroup(fdesc1, &gdesc1, thebuf1, &bufsiz1) != E_SUCC)
    {
    free(thebuf1);
    free(thebuf2);
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    }
if (c_egroup(fdesc2, &gdesc2, thebuf2, &bufsiz2) != E_SUCC)
    {
    free(thebuf1);
    free(thebuf2);
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    }


/*
  Set up processing message
*/
c_inittm("IMGCMP", "Processed", "line", "bands", " ",
    &nl, &totbnd, &msgtim, &msgflag);
curent = 1;    
curcat = 1;
c_startm();


/*
  Compare the images
*/
for (; curent <= nl; curent++)
    {
    /*
      Step each of the image groups
    */
    if (c_estep(&gdesc1) != E_SUCC)
	{
	free(thebuf1);
	free(thebuf2);
	c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
	}
    if (c_estep(&gdesc2) != E_SUCC)
	{
	free(thebuf1);
	free(thebuf2);
	c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
	}


    /*
      Compare each band
    */
    for (band = 0; band < totbnd; band++)
	{
	for (sample = 0; sample < ns; sample++)
	    {
	    pixel1 = *(float *)(thebuf1 + offset1[band] + sample);
	    pixel2 = *(float *)(thebuf2 + offset2[band] + sample);

	    diff = pixel1 - pixel2;

	    if (fabs(diff) > tolval)
		{
		c_stoptm();
		c_egclse(&gdesc1);
		c_egclse(&gdesc2);
		free(thebuf1);
		free(thebuf2);
		sprintf(str, "Pixels differ in band %d at line %d, sample %d",
		    band+1, curent, sample+1);
		c_errmsg(str, "imgcmp-diff", &fatal);
		}
	    }
	}
    }


/*
  Stop the processing messages
*/
c_stoptm();


/*
  Close all images
*/
c_egclse(&gdesc1);
c_egclse(&gdesc2);


/*
  Free the buffers
*/
free(thebuf1);
free(thebuf2);


/*
  Issue an informational message
*/
c_lasmsg(FALSE, " ", "The images are identical.  The tolerance value was %lf",
    tolval);

return(E_SUCC);
}
