/******************************************************************************
NAME: GET_ZONENUM

PURPOSE: To retrieve the appropriate zone code number from the tape's header
	 file, convert it if necessary, and place it in the project parameter
	 file.  If zone is not given, it has to be calculated and checked.
	 That is why the corners are calculated in this function.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

******************************************************************************/
#define ZONETOL 5

#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "header.h"
#include "fhdr2ddr.h"
#include "protos.h"

static int stpln_zone
(
    long *zcode      /* zone number of the image                          */
);

int get_zonenum 
(
    char header[],     /* ptr into the tape's header file for the zone number */
    char proj_type[],  /* string holding the projection type of the image     */
    long *zcode,       /* zone number of the image                  	*/
    double latlong[8], /* array for storing the lat/long for corner points  */
    double *centlat,   /* center latitude				     */
    long *estmzone,	   /* flag for estimating zone			     */
    double *corners,   /* corner for northing and easting computed	     */
    double *corneryx,  /* corner for northing and easting from header
                                                       0:uln, 1:ule
                                                       2:urn, 3:ure
                                                       4:lln, 5:lle
                                                       6:lrn, 6:lre         */
    long datum,		   /* GCTP datum code			  	*/
    long *pcode,	   /* GCTP projection code				*/
    double *projcoef   /* GCTP projection parameters			     */
)
{
   long i;		/* loop control variable			*/
   long inproj;		/* input projection code			*/
   long inunit;		/* input projection units			*/
   long outunit;	/* output projection units			*/
   long prtprm[2];	/* report destination flags			*/
   long status;         /* function return status      			*/
   long zoneflg;	/* flag indicating incorrect zone		*/

   double centlong;	/* center longitude				*/
   double centmer;	/* temporary central meridian			*/
   double inparm[15];	/* input projection coefficients		*/
   
   long fatal = -1;     /* fatal error flag				*/
   long nonfatal = 1;	/* nonfatal error flag				*/
   
   *estmzone = FALSE;

   /* If the projection number implies Universal Transverse Mercator or State
      Plane Grid, or the projection type is Gauss-Krueger, Swedish National
      Grid, or State Plane Coordinate System then retrieve the zone number from
      the tape's header file.  In all other cases, force the zone number to 62.
      NOTE:  	The INPE tapes currently (2/2/95) only support UTM projection,
		however, code has been preserved to add other projections.
      ----------------------------------------------------------------------*/
   if ((*pcode == UTM)              || (*pcode == SPCS)              ||
       (strcmp(proj_type,"gk")   == 0) || (strcmp(proj_type,"sng")  == 0) ||
       (strcmp(proj_type,"spcs") == 0))
      {
      status = c_fhdrrd(header,47,(char*)zcode);
      if(E_FAIL == status)
	return(status);

      /* find the center latitude to determine the zone if not given
	 and the sign of the zone otherwise.
        -----------------------------------------------------------*/
      *centlat = (latlong[1]+latlong[3]+latlong[5]+latlong[7])/4.0;

      /* calculate zone code from the corner cordinates if necessary
	 ----------------------------------------------------------*/
      if ((*zcode > 60) && (*pcode == UTM))
	{
        *estmzone = TRUE;
	centlong = (latlong[0]+latlong[2]+latlong[4]+latlong[6])/4.0;
	*zcode = (long) (((centlong + 180.0) / 6.0) + 1.0);
	c_errmsg("The zone code has been calculated","inpeddr-calczone",
								&nonfatal);
	}

      /* if the center latitude is south of the equator, zone code is neg.
	----------------------------------------------------------------*/
      if (*centlat <= 0.0)
	   *zcode = -*zcode;
      }
   else
     *zcode = 62;
   
   /* If the projection number implies State Plane Grid or the projection type
      is State Plane Grid or State Plane Coordinate System and the Fast Format
      version is A, then convert from the old zone code number to the new zone
      code number.  This switch statement was taken from
      vaxa::pkg:[spotpcs.sourceapp.util.haat2ddr]c_fillddr.c
	NOTE:  	This section does not apply to INPE tapes but is preserved
		for possible expansion.
      ----------------------------------------------------------------------*/
/*
   if ((ff_version == 'A') && ((*pcode == 2) || 
			       (strcmp(proj_type, "stpl") == 0) || 
			       (strcmp(proj_type, "spcs") == 0) ) )
      {
      if (stpln_zone(zcode) == E_FAIL);
	 sprintf(tmp_str,"Illegal zone number %1d, for projection %s",
		 *zcode, proj_type);
	 c_errmsg(tmp_str,"inpeddr-zone",&fatal);
      }
*/

   /* Change projection from UTM to TM -- UTM supports Clarke 1866 spheroid.
    ----------------------------------------------------------------------*/

   if (*pcode == UTM)
      {
      projcoef[2] = .9996;
      centmer = 6.0 * abs(*zcode) - 183.0;
      if (c_degdms (&centmer, &projcoef[4], "DEG", "LON") != E_SUCC)
	 {
	 c_errmsg("Fatal error encountered","readinpe-fatal",&fatal);
	 }
      *pcode = 9;
      }
   
   /* Initialize variables for Geometric Registration
      --------------------------------------------------*/
   inproj    = 0;             /* convert form geographic */
   inunit    = 4;             /* input is in degrees */
   outunit   = 2;             /* output is in degrees */
   prtprm[0] = 0;             /* print to terminal = FALSE           */
   prtprm[1] = 0;             /* print to file     = FALSE           */
   for (i = 0; i < 15; i++)
     inparm[i] = 0;

   /* get projection corners,  check to see that the zone is valid, if not,
      change the zone (one above and one below estimate) and try again
     ---------------------------------------------------------------------*/
   zoneflg = TRUE;
   for (i = 0; i < 3; i++)
      {
      /* Second attempt--decrease zone by one.  If the zone was given in the
 	 header than no second attempt should be necessary--error out.
        ------------------------------------------------------------------*/
      if (i == 1)
	 {
         (*zcode)--;
	 if (*estmzone == FALSE)
	    {
            c_errmsg("Zone does not match corners","readinpe-zone",&nonfatal);
	    c_errmsg("Fatal error encountered","readinpe-fatal",&fatal);
	    }
	 }

      /* Third attempt--increment to one above original zone
        ---------------------------------------------------*/
      else if (i == 2)
         *zcode = *zcode + 2;

      /* Initialize the Geometric Registration routines
         ---------------------------------------------*/
      status = c_transinit(&inproj,  &inunit,  zcode,   &datum, inparm, pcode,
		&outunit, zcode, &datum, projcoef,  prtprm, NULL);
      if (status != E_SUCC)
        c_errmsg ("Fatal error encountered", "inpeddr-fatal", &fatal);

      /* upper left
        ----------*/
      if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
			&latlong[0],&latlong[1],
			&corners[0],&corners[1]))
         c_errmsg ("Fatal error encountered", "inpeddr-fatal", &fatal);

      if (fabs(corners[0]-corneryx[0]) > ZONETOL)
	 zoneflg = FALSE;
      if (fabs(corners[1]-corneryx[1]) > ZONETOL)
	 zoneflg = FALSE;

      /* upper right
         -----------*/
      if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
			&latlong[2], &latlong[3],
			&corners[2], &corners[3]))
         c_errmsg ("Fatal error encountered", "inpeddr-fatal", &fatal);
      if (fabs(corners[2]-corneryx[2]) > ZONETOL)
	 zoneflg = FALSE;
      if (fabs(corners[3]-corneryx[3]) > ZONETOL)
	 zoneflg = FALSE;

      /* lower left
         -------------*/
      if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
			&latlong[4], &latlong[5],
			&corners[4], &corners[5]))
         c_errmsg ("Fatal error encountered", "inpeddr-fatal", &fatal);
      if (fabs(corners[4]-corneryx[4]) > ZONETOL)
	 zoneflg = FALSE;
      if (fabs(corners[5]-corneryx[5]) > ZONETOL)
	 zoneflg = FALSE;

      /* lower right
         --------------*/
      if (E_SUCC != c_trans(&inproj,  &inunit, pcode, &outunit,
			&latlong[6], &latlong[7],
			&corners[6], &corners[7]))
         c_errmsg ("Fatal error encountered", "inpeddr-fatal", &fatal);
      if (fabs(corners[6]-corneryx[6]) > ZONETOL)
	 zoneflg = FALSE;
      if (fabs(corners[7]-corneryx[7]) > ZONETOL)
	 zoneflg = FALSE;

      if (zoneflg)
	break;
      }


   /* set false northings and eastings
    ---------------------------------*/
   projcoef[6] = 500000.0; 
   if (*zcode < 0)
	{
        projcoef[7] = 10000000.0;
	}

   /* adjust corners with appropriate false northings and eastings
   --------------------------------------------------------------*/
   corners[1] += 10000000;
   corners[3] += 10000000;
   corners[5] += 10000000;
   corners[7] += 10000000;
   if (*zcode < 0)
      {
      corners[0] += 500000;
      corners[2] += 500000;
      corners[4] += 500000;
      corners[6] += 500000;
      }
   return(E_SUCC);
   
}

/**************************************************************************
NAME:				STPLN_ZONE

**************************************************************************/

static int stpln_zone
(
    long *zcode      /* zone number of the image                          */
)
{

      switch (*zcode) 
      {
       case 3101: *zcode = 101;  break;
       case 3126: *zcode = 102;  break;
       case 6101: *zcode = 5001; break;
       case 6126: *zcode = 5002; break;
       case 6151: *zcode = 5003; break;
       case 6176: *zcode = 5004; break;
       case 6201: *zcode = 5005; break;
       case 6226: *zcode = 5006; break;
       case 6251: *zcode = 5007; break;
       case 6276: *zcode = 5008; break;
       case 6301: *zcode = 5009; break;
       case 6326: *zcode = 5010; break;
       case 3151: *zcode = 201;  break;
       case 3176: *zcode = 202;  break;
       case 3201: *zcode = 203;  break;
       case 3226: *zcode = 301;  break;
       case 3251: *zcode = 302;  break;
       case 3276: *zcode = 401;  break;
       case 3301: *zcode = 402;  break;
       case 3326: *zcode = 403;  break;
       case 3351: *zcode = 404;  break;
       case 3376: *zcode = 405;  break;
       case 3401: *zcode = 406;  break;
       case 3426: *zcode = 407;  break;
       case 3451: *zcode = 501;  break;
       case 3476: *zcode = 502;  break;
       case 3501: *zcode = 503;  break;
       case 3526: *zcode = 600;  break;
       case 3551: *zcode = 700;  break;
       case 3601: *zcode = 901;  break;
       case 3626: *zcode = 902;  break;
       case 3576: *zcode = 903;  break;
       case 3651: *zcode = 1001; break;
       case 3676: *zcode = 1002; break;
       case 5876: *zcode = 5101; break;
       case 5901: *zcode = 5102; break;
       case 5926: *zcode = 5103; break;
       case 5951: *zcode = 5104; break;
       case 5976: *zcode = 5105; break;
       case 3701: *zcode = 1101; break;
       case 3726: *zcode = 1102; break;
       case 3751: *zcode = 1103; break;
       case 3776: *zcode = 1201; break;
       case 3801: *zcode = 1202; break;
       case 3826: *zcode = 1301; break;
       case 3851: *zcode = 1302; break;
       case 3876: *zcode = 1401; break;
       case 3901: *zcode = 1402; break;
       case 3926: *zcode = 1501; break;
       case 3951: *zcode = 1502; break;
       case 3976: *zcode = 1601; break;
       case 4001: *zcode = 1602; break;
       case 4026: *zcode = 1701; break;
       case 4051: *zcode = 1702; break;
       case 4076: *zcode = 1801; break;
       case 4101: *zcode = 1802; break;
       case 4126: *zcode = 1900; break;
       case 4151: *zcode = 2001; break;
       case 4176: *zcode = 2002; break;
       case 4201: *zcode = 2101; break;
       case 4226: *zcode = 2102; break;
       case 4251: *zcode = 2103; break;
       case 6351: *zcode = 2111; break;
       case 6376: *zcode = 2112; break;
       case 6401: *zcode = 2113; break;
       case 4276: *zcode = 2201; break;
       case 4301: *zcode = 2202; break;
       case 4326: *zcode = 2203; break;
       case 4351: *zcode = 2301; break;
       case 4376: *zcode = 2302; break;
       case 4401: *zcode = 2401; break;
       case 4426: *zcode = 2402; break;
       case 4451: *zcode = 2403; break;
       case 4476: *zcode = 2501; break;
       case 4501: *zcode = 2502; break;
       case 4526: *zcode = 2503; break;
       case 4551: *zcode = 2601; break;
       case 4576: *zcode = 2602; break;
       case 4601: *zcode = 2701; break;
       case 4626: *zcode = 2702; break;
       case 4651: *zcode = 2703; break;
       case 4676: *zcode = 2800; break;
       case 4701: *zcode = 2900; break;
       case 4726: *zcode = 3001; break;
       case 4751: *zcode = 3002; break;
       case 4776: *zcode = 3003; break;
       case 4801: *zcode = 3101; break;
       case 4826: *zcode = 3102; break;
       case 4851: *zcode = 3103; break;
       case 4876: *zcode = 3104; break;
       case 4901: *zcode = 3200; break;
       case 4926: *zcode = 3301; break;
       case 4951: *zcode = 3302; break;
       case 4976: *zcode = 3401; break;
       case 5001: *zcode = 3402; break;
       case 5026: *zcode = 3501; break;
       case 5051: *zcode = 3502; break;
       case 5076: *zcode = 3601; break;
       case 5101: *zcode = 3602; break;
       case 5126: *zcode = 3701; break;
       case 5151: *zcode = 3702; break;
       case 5176: *zcode = 3800; break;
       case 5201: *zcode = 3901; break;
       case 5226: *zcode = 3902; break;
       case 5251: *zcode = 4001; break;
       case 5276: *zcode = 4002; break;
       case 5301: *zcode = 4100; break;
       case 5326: *zcode = 4201; break;
       case 5351: *zcode = 4202; break;
       case 5376: *zcode = 4203; break;
       case 5401: *zcode = 4204; break;
       case 5426: *zcode = 4205; break;
       case 5451: *zcode = 4301; break;
       case 5476: *zcode = 4302; break;
       case 5501: *zcode = 4303; break;
       case 5526: *zcode = 4400; break;
       case 5551: *zcode = 4501; break;
       case 5576: *zcode = 4502; break;
       case 5601: *zcode = 4601; break;
       case 5626: *zcode = 4602; break;
       case 5651: *zcode = 4701; break;
       case 5676: *zcode = 4702; break;
       case 5701: *zcode = 4801; break;
       case 5726: *zcode = 4802; break;
       case 5751: *zcode = 4803; break;
       case 5776: *zcode = 4901; break;
       case 5801: *zcode = 4902; break;
       case 5826: *zcode = 4903; break;
       case 5851: *zcode = 4904; break;
       case 6001: *zcode = 5201; break;
       case 6051: *zcode = 5202; break;
       default:
	 return(E_FAIL);

      }
      return(E_SUCC);
}
