/******************************************************************************
NAME:	INPEDDR

PURPOSE: Uses the ;FHDR file from FSTFMTIN to determine the projection
	information necessary to update the DDR for INPE data.  The
	file to be processed should be on disk.


PROGRAM HISTORY:
  Version       Date    Author          Request
  -------       ----    ------          -------
    1.0		 3/95	S. Nelson	INPEDDR was stolen from the FHDR2DDR
					portion of FSTFMTIN.  It was modified
					to read INPE tapes.   All INPE tapes
					are in the UTM projection.  Some of
					the sections relating to other
					projections have been preserved for
					possible future development.

    1.1		6/96    M.Choate 	Modified process_hdr.c and 
			and		pcorners.c.  Pcorners.c was
		        A.VanderVorste	modified to take in the orientation
					angle of the image and the pixel size
					in order to calculate the lat/lon pairs
					for the image. If the orientation 
					angle was not given in the fast format
					header (as in INPE's first version of
					their headers) then calculate it.  
					See algorithm reference 1. 
			

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call getpar to get TAE parameters
   Call process_hdr to extract the projection, corner point data, and other 
   information from the header.
   Call update_ddr to update the DDR of the image with the new corner points, 
   projection info, and other information extracted from the fast format 
   header.
   Copy and update image history, and issue completion message 

ALGORITHM REFERENCES:
        1. EOSAT FAST FORMAT DOCUMENT for TM Digital Products Version B,
           Effective December 1, 1993
******************************************************************************/

/* include files, global parameters, and definitions */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "fhdr.h"
#include "header.h"
#include "protos.h"

struct	PARBLK	block;                 /* TAE parameter block     */

long 	msgtim;                        /* message interval        */
 
int main
(void) 
{
   /* file names 
      ----------*/
   
   char hostimg[CMLEN];          /* image host name                          */
   char hhdr[CMLEN];             /* output fast format header host name      */
   char lasimg[CMLEN];           /* image las name                           */
   char fheader_ext[CMLEN];      /* header extension */
   
   /* image information 
      ------------------*/
   
   char fheader[HEADER_SIZE+1];  /* buffer for header file                   */
   char htype[2]="\0\0";         /* fast format header type                  */
   char tmpstr[HEADER_SIZE+1];   /* temporary string                         */

   long hflag;                   /* blank out fast format header fields      */
   long nb;                      /* number of bands for this image           */
   long window[4];               /* image window                             */
   
   /* data retrieved from fast format header 
      --------------------------------------*/
   char source[32];             /* source of image                          */
   char instrument[32];         /* instrument                               */
   char ac_date[10];            /* acquistion date                          */
   char ac_time[8];             /* acquistion time                          */
   
   long datum = 0;		/* GCTP datum code                          */
   long origsize[2];            /* original size of the image on tape       */
   long proj_code;              /* GCTP projection code                     */
   long zone_code;              /* UTM or State Plane code                  */

   double corners[8];           /* image corner points  0:ul x, 1:ul y
                                                        2:ur x, 3:ur y
                                                        4:ll x, 5:ll y
                                                        6:lr x, 7:lr y       */ 

   double corneryx[8];          /* array for the corner northing and easting
                                                        0:uln, 1:ule
                                                        2:urn, 3:ure
                                                        4:lln, 5:lle
                                                        6:lrn, 6:lre      */


   double pixsize;              /* pixel size                               */
   double orientation;          /* orientation angle of scene (systematic)  */
   double proj_coef[15];        /* GCTP projection parameters               */
   
   /* misc. variables 
      ---------------*/
   
   long fatal_err = -1;          /* fatal error flag for errmsg              */
   long one = 1;
   long mode;
   long status;
   long zero = 0;
   strcpy(fheader_ext,"fhdr");
   
   /* get las parameters
      --------------------- */
   
   getpar(lasimg, hostimg, window , &nb, htype, &hflag);
   
   mode = IREAD;
   if(htype[0] == 'A')
     status = c_open_header(tmpstr,hostimg,&mode);

   else			/* header type is B or C */
     status = c_open_header(fheader,hostimg,&mode);
   
   if(status == E_FAIL)
     c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);
     
   /* header A
     --------*/
   if(htype[0] == 'A')
   {
      tmpstr[HEADER_SIZE] = '\0';
      
      /* get the acquisition time -- only version A has this info 
	 --------------------------------------------------------*/

      strncpy(ac_time,&tmpstr[15],4);
      ac_time[4] = ':';
      ac_time[5] = tmpstr[19];
      ac_time[6] = '0';
      ac_time[7] = 0;

      /* convert the header to B format 
	 ------------------------------------*/
      c_convertab(tmpstr,fheader);
   }

   /* header B and C
     --------------*/
   else
   {
      /* ver. B and C don't have the time so make up one 
	 -------------------------------------------*/

      strcpy(ac_time,"0000:00");  
   }
   
   fheader[HEADER_SIZE] = '\0';
   
   /* read the parameters for updating the ddr 
      -------------------------------------------*/
   process_hdr(fheader, origsize, &pixsize, &orientation, 
               proj_coef, corners, corneryx, &proj_code, &zone_code, 
	       &datum, source, instrument, ac_date );

   /* Find the file corners from the header corners--in this image, the
      corners in the header are given.
      --------------------------------------------*/
   pcorners(pixsize, orientation, corners, corneryx, window);

   /* write out a fast format associated file if it is desired 
      -----------------------------------------------------------*/
   
   if(hflag)
   {
      /* open the new file and write out the header contents 
	 --------------------------------------------------*/

      mode = IWRITE;
      
      if( E_FAIL==c_open_header(fheader,hostimg,&mode))
	c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   }
   
   /* Calculate new corner points for the subwindow if necessary 
      ----------------------------------------------------------*/

   if(origsize[0] != window[NL] || origsize[1] != window[NS])
   {
     c_errmsg("Error rountine has not been checked for subwindows",
              "inpeddr-fatal", &fatal_err);
     recalc_corners(corners,origsize,window);
   }
   
   /* update the ddr 
      -----------------*/
   
   update_ddr(hostimg, window, pixsize, proj_coef, corners, proj_code, 
	      zone_code, datum, source, instrument, ac_date, ac_time);
   
   /*
     update output image history and terminate processing
     ----------------------------------------------------- */

   if(E_FAIL == c_hstout( &block, hostimg, "", &zero, "")) 
     c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);
   
   if(hflag)
   {
      c_lsmknm(hostimg,"fhdr",hhdr);
      c_complt ("INPEDDR", hhdr, &one);
   }
   else
     c_complt ("INPEDDR", NULL, &zero);
   
}
