/******************************************************************************

NAME:				PCORNERS

PURPOSE: Uses a polynomial to find the file corners of an image in
         projection.  Reads a label table file to get the line sample
	 corners of the image data.


PROGRAM HISTORY:
  Version       Date	Author		Request
  -------       ----	------		-------
   1.0		3/95	S. Nelson	Original Development

   1.1          6/96    M.Choate        Pcorners.c was modified to take in 
		        and             the orientation angle of the image 
		        A.VanderVorste	and the pixel size in order to 
					calculate the lat/lon pairs 
					for the image. If orientation angle 
					was not given in header, calculate it 
					- see Algorithm reference 1.

ALGORITHM:
If orientation angle was not given in header, calculate it - see Algorithm
	reference 1.
Find UTM coordinate centers of image:
	Find slope and intercept of line between UL and LR corners
	Then find the slope and intercept of the line between LL and UR corners
	Find coordinate center for scene
Calculate projection corners of image

ALGORITHM REFERENCES:
	1. EOSAT FAST FORMAT DOCUMENT for TM Digital Products Version B, 
	   Effective December 1, 1993
******************************************************************************/

/* include files, global parameters, and definitions for FDHR2DDR    */

#include <math.h>
#include "las.h"
#include "ltable.h"
#include "protos.h"


#ifndef DEG
#define DEG(p)    (p) * 180 / PI
#endif


#define PI            3.141592653589793238 

void pcorners
(
    double pixsize,		/* size of image pixel */
    double orientation,		/* Orientation angle of scene (systematic)*/
    double *corners,		/* image data corners-- 0:ul x, 1:ul y
							2:ur x, 3:ur y
							4:ll x, 5:ll y
							6:lr x, 7:lr y    */
    double *corneryx,            /* array for the corner northing and easting  
                                                        0:uln, 1:ule
                                                        2:urn, 3:ure
                                                        4:lln, 5:lle
                                                        6:lrn, 6:lre      */
    long *window		/* image window */
)

{
/* file names 
  ----------*/
long fatal = -1;		/* fatal error message */
long nonfatal = 1;		/* nonfatal error message */
double sample;
double line;
double m1,m2;			/* the slopes */
double b1,b2;			/* the intercepts */
double pixsize_x, pixsize_y;	/* pixel sizes */
double line_center, sample_center;  /* image centers in the line and
					sample direction  */
double utmxc, utmyc;		/* the UTM coordinate centers */	
double x,y;		
double cos_angle,sin_angle;     /* cosine and size of orientation
				   angle of scene */
double tmp;			/* temporary vairable */


/* seeing if the orientation angle was in the fast format header
   (INPE's old version of the header doesn't contain the orientation angle)
   if if was not in the header, then calculate it 
   the formula is given in the "EOSAT FAST FORMAT DOCUMENT for TM Digital
   Products Version B, Effective December 1, 1993". The formula, with the
   field number listed from the fast format header, is as follows

   	orientation_angle = (UR_northing_field77 - UL_northing_field69)
			   -------------------------------------------
			   (UR_easting_field74 - UL_easting_field67)     */
if (orientation >= HUGE_VAL)
    {
    /* seeing if the orientation angle can be calculated */
    if (corneryx[2] - corneryx[0] < .0001)
        c_errmsg("Error estimating center proj coord",
	"inpeddr-getcor",&fatal);


    /* below is how the orientation angle in the EOSAT document is
       defined */
 
   c_errmsg("The orientation angle has been calculated",
		"inpeddr-calcangle",&nonfatal);
   tmp = (corneryx[3]-corneryx[1])/(corneryx[2]-corneryx[0]);
   orientation = DEG(atan(tmp));

   /* the orientation angle given by the EOSAT document, does not produce
      good results but it's negative does (this is also consistent
      with when INPE does include orientation angles*/
   orientation = -orientation;

   }

pixsize_x = pixsize;
pixsize_y = pixsize;

/* calculating the slope and y intercept of the 
	line between the UL and LR corners */
if( corners[0]==corners[6] )
   {
   c_errmsg("Error estimating center proj coord","inpeddr-getcor",&fatal);
   }
else
   {
   m1 = (corners[1] - corners[7]) / (corners[0] - corners[6]);
   b1 = corners[1] - m1 * corners[0];
   }

/* calculating the slope and y intercept of the 
	line between the LL and UR corners */
if( corners[4]==corners[2] )
   {
   c_errmsg("Error estimating center proj coord","inpeddr-getcor",&fatal);
   }
else
   {
   m2 = (corners[5] - corners[3]) / (corners[4] - corners[2]);
   b2 = corners[5] - m2 * corners[4];
   }

/* finding the UTM coordinate center of the image */
if( m1==m2 )
   {
   c_errmsg("Error estimating center proj coord","inpeddr-getcor",&fatal);
   }
else
   {
   utmxc = (b2-b1) / (m1-m2);
   utmyc = m1 * utmxc + b1;
   }

sample_center = window[3] / 2.0;
line_center = window[2] / 2.0;

/* change orientation angle from degrees to radians
  ------------------------------------------------*/

orientation = (orientation * PI) / 180.0;
cos_angle = cos(orientation);
sin_angle = sin(orientation);

/* find the ul corner projection x/y value
  ---------------------------------------*/
line = window[0];
sample = window[1];
x = ( sample - sample_center ) * cos_angle * pixsize_x +
    ( line_center - line ) * sin_angle * pixsize_y;
y = -( sample - sample_center ) * sin_angle * pixsize_x +
    ( line_center - line ) * cos_angle * pixsize_y;
corners[0] = utmxc + x;
corners[1] = utmyc + y;

/* find the ur corner projection x/y value
  ---------------------------------------*/
line = window[0];
sample = window[3];
x = ( sample - sample_center ) * cos_angle * pixsize_x +
    ( line_center - line ) * sin_angle * pixsize_y;
y = -( sample - sample_center ) * sin_angle * pixsize_x +
    ( line_center - line ) * cos_angle * pixsize_y;
corners[2] = utmxc + x;
corners[3] = utmyc + y;

/* find the lr corner projection x/y value
  ---------------------------------------*/
line = window[2];
sample = window[3];
x = ( sample - sample_center ) * cos_angle * pixsize_x +
    ( line_center - line ) * sin_angle * pixsize_y;
y = -( sample - sample_center ) * sin_angle * pixsize_x +
    ( line_center - line ) * cos_angle * pixsize_y;
corners[6] = utmxc + x;
corners[7] = utmyc + y;

/* find the ll corner projection x/y value
  ---------------------------------------*/
line = window[2];
sample = window[1];
x = ( sample - sample_center ) * cos_angle * pixsize_x +
    ( line_center - line ) * sin_angle * pixsize_y;
y = -( sample - sample_center ) * sin_angle * pixsize_x +
    ( line_center - line ) * cos_angle * pixsize_y;
corners[4] = utmxc + x;
corners[5] = utmyc + y;

}
