/******************************************************************************

NAME: process_hdr

PURPOSE: To read the fast format header and extract the geometric and 
         scene acquistion data.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
    Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Read the projection information stored in the header.
   Call convert_dms to adjust the projection coefficients to the proper 
   DMS specification needed.
   Read in the projection coefficients.
   Get the lat/long for the corner points.
   Convert the lat/long pts into northings/eastings.
   Get the ground pixel size.
   Get the instrument and acquistion date.

******************************************************************************/
#define D2R     0.0174532925199

#include "las.h"
#include "util.h"
#include "header.h"
#include "fhdr2ddr.h"
#include "proj.h"
#include "protos.h"

static void get_datum
(
    char *hdr,
    long *datum
);


int process_hdr
(
    char *hdr,            /* fast format header                               */
    long isize[2],        /* image size (on tape)                             */
    double *pixsize,      /* ground pixel size                                */
    double *orientation,  /* orientation angle of the scene		     */
    double projcoef[15],  /* GCTP projection parameters                       */
    double corners[8],    /* image corner points         0:ul x, 1:ul y
                                                         2:ur x, 3:ur y
                                                         4:ll x, 5:ll y
                                                         6:lr x, 7:lr y       */
    double corneryx[8],   /* array for the corner northing and easting 
                                                         0:uln, 1:ule
                                                         2:urn, 3:ure
                                                         4:lln, 5:lle
                                                         6:lrn, 6:lre      */
    long *pcode,          /* GCTP projection code                             */
    long *zcode,          /* UTM or State Plane code                          */
    long *datum,          /* GCTP datum code                                  */
    char source[32],      /* source of image                                  */
    char inst[32],        /* instrument                                       */
    char date[10]         /* date acquired                                    */
)
{

   char pname[5];           /* abreviated name of the projection             */
   char projstr[360];       /* temporary storage for proj. coef. string      */
   
   long hfields[5];         /* array containing information about a field in
			       the fast format header                        */
   double latlong[8];       /* array for storing the lat/long for each corner
			       point                                         */
   double centlat;	    /* center latitude				     */

   long psize;              /* size of each proj. coef. value in characters  */
   
   /* misc variables */

   char *cptr;              /* working character pointer                     */
   char tmpstr[CMLEN];      /* temporary character string buffer             */

   long estmzone;	    /* flag indicating that the zone was estimated   */
   long fatal_err = -1;     /* fatal error flag for c_errmsg                 */
   long i;                  /* working index variable                        */
   long one = 1;

   
   /* get the projection name 
    -------------------------*/
   if ( c_fhdrrd(hdr,43,pname) == E_FAIL )
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);
   
   i = 4;
   c_up2low(pname,&i);
   pname[4] = 0;

   /* get the projection code 
    -------------------------*/

   /*  UTM is the only projection currently supported for INPE tapes.
       UTM is a special case of TM.  The GCTP package keys off 
       the TM values.
       --------------------------------------------------------------------*/
   
   if(strcmp(pname,"utm "))
      {
      c_errmsg("Projection in Fast Format header is not UTM",
						"readinpe-proj", &one);
      return(E_FAIL);
      }
      
   /* get the projection code, 45 is the field to read it from */
   if ( c_fhdrrd(hdr,45,(char *)pcode) == E_FAIL)
	c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   if (*pcode == ((long) HUGE_VAL))
      {
      *pcode=UTM;
      }
   if (*pcode != UTM)
      {
      c_errmsg("Projection in Fast Format header is not UTM",
						"readinpe-proj", &one);
      return(E_FAIL);
      }
      
   /* get the orientation angle of the scene (in field 41) */
   if ( c_fhdrrd(hdr,41,(char *)orientation) == E_FAIL)
	c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* ok, now we have to convert the projection coefficients to the form LAS
      uses
      ----------------------------------------------------------------------*/

   /* but first, get information about the projection parameter field
      ---------------------------------------------------------------*/

   if(E_FAIL == c_fhdfld(49,hfields))
     c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* get the starting byte of the projection coefficients 
      ----------------------------------------------------*/

   cptr = hdr + hfields[0];
 
   /* save the original projection parameters for later 
      -------------------------------------------------*/

   strncpy(projstr,cptr,360);
   
   /* get the character width of each projection coefficient 
      ------------------------------------------------------*/

   psize = hfields[3];
   
   for(i=0;i<15;i++)
      {
      cptr += psize;
      }
   
   /* Since INPE tapes do not specify projection coefficients, just assign
      zeros to all elements of the projection array
     --------------------------------------------------------------------*/
      for(i=0;i<15;i++)
           projcoef[i] = 0;
     
   /* now read the corner points (degree lat/long )
      --------------------------------------------- */

   /* UL longitude */

   if(E_FAIL == c_fhdrrd(hdr,63,(char *)&latlong[0]))
     c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal_err);

   /* UL latitude */

   if(E_FAIL == c_fhdrrd(hdr,65,(char *)&latlong[1]))
     c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal_err);

   /* UL easting */

  if(E_FAIL == c_fhdrrd(hdr,67,(char *)&corneryx[0]))
     c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal_err);

   /* UL northing */

   if(E_FAIL == c_fhdrrd(hdr,69,(char *)&corneryx[1]))
     c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal_err);

   /* UR longitude */

   if(E_FAIL == c_fhdrrd(hdr,71,(char *)&latlong[2]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* UR latitude */

   if(E_FAIL == c_fhdrrd(hdr,73,(char *)&latlong[3]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* UR easting */

   if(E_FAIL == c_fhdrrd(hdr,75,(char *)&corneryx[2]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* UR northing */

   if(E_FAIL == c_fhdrrd(hdr,77,(char *)&corneryx[3]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* LL longitude */

   if(E_FAIL == c_fhdrrd(hdr,87,(char *)&latlong[4]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* LL latitude */

   if(E_FAIL == c_fhdrrd(hdr,89,(char *)&latlong[5]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* LL easting */

   if(E_FAIL == c_fhdrrd(hdr,91,(char *)&corneryx[4]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* LL northing */

   if(E_FAIL == c_fhdrrd(hdr,93,(char *)&corneryx[5]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* LR longitude */

   if(E_FAIL == c_fhdrrd(hdr,79,(char *)&latlong[6]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* LR latitude */

   if(E_FAIL == c_fhdrrd(hdr,81,(char *)&latlong[7]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);
   
   /* LR easting */

   if(E_FAIL == c_fhdrrd(hdr,83,(char *)&corneryx[6]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* LR northing */

   if(E_FAIL == c_fhdrrd(hdr,85,(char *)&corneryx[7]))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);
   
   /* check to make sure they are valid 
      ---------------------------------*/

   for(i=0;i<8;i++)
   {
      if(latlong[i] == HUGE_VAL)
      {
	 c_errmsg("Invalid Latitude/Longitude in Fast Format header",
		 "inpeddr-process", &fatal_err);
      }
      if(corneryx[i] == HUGE_VAL)
      {
	 c_errmsg("Invalid northing/easting in Fast Format header",
		 "inpeddr-process", &fatal_err);
      }
      
   }
   
   /* get the datum code 
      ----------------- */
   get_datum(hdr,datum);
   
   /* get the zone code and the image corner coordinates
      -------------------------------------------------*/
   if( E_FAIL == get_zonenum(hdr, pname, zcode,latlong, &centlat,
		&estmzone, corners, corneryx,*datum,pcode, projcoef))
     c_errmsg("Fatal error encountered","inpein-fatal", &fatal_err);
   

   /* get the original image size 
      ----------------------------*/

   if(E_FAIL == c_fhdrrd(hdr,61,(char *)&isize[0]))
      c_errmsg("Fatal error encountered","inpein-fatal", &fatal_err);

   if(E_FAIL == c_fhdrrd(hdr,59,(char *)&isize[1]))
      c_errmsg("Fatal error encountered","inpein-fatal", &fatal_err);

   if(isize[0] == ((long) HUGE_VAL) || isize[1] == ((long) HUGE_VAL) )
   {
      c_errmsg("Invalid image size in Fast Format header",
		"inpeddr-process", &fatal_err);
   }
   
   /* read the pixel size 
      -------------------*/

   if(E_FAIL == c_fhdrrd(hdr,57,(char *)pixsize))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   if(*pixsize == HUGE_VAL)
   {
      c_errmsg("Invalid pixel size in Fast Format header",
		"inpeddr-process", &fatal_err);
   }
   
   /* get the instrument 
      ------------------*/

   if(E_FAIL == c_fhdrrd(hdr,10,inst))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   /* get the source 
      ------------------*/

   if(E_FAIL == c_fhdrrd(hdr,8,tmpstr))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);

   switch (tmpstr[1])
   {
    case '4':
      strcpy(source,"landsat-4");
      break;
    case '5':
      strcpy(source,"landsat-5");
      break;
    case '7':
      strcpy(source,"landsat-7");
      break;
    default:
      strcpy(source,"unknown");
      break;
   }
   
   /* now get the acquisition date 
      ------------------------------*/

   if(E_FAIL == c_fhdrrd(hdr,6,date))
      c_errmsg("Fatal error encountered","inpeddr-fatal", &fatal_err);
   
   /* convert it to what LAS uses.
      ---------------------------*/
   datecon(date);
   
   return(E_SUCC);
  
}

/******************************************************************************

NAME: get_datum

PURPOSE: Given a fast format header, determine the projection datum.  The 
	 fast format header documentation did not agree with the header
	 files concerning the abreviations for the datum names.  Because of
	 that, the datum is determined from the ellipsoid parameters.  If
	 a match cannot be found, the datum is set to the value for 
	 Clarke-1866 and the header is updated.

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----      ------           -------
    1.0        12/92      K.Sheffield      Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
    Runs under TAE

PROJECT: LAS 

ALGORITHM DESCRIPTION:
    Read the ellipsoid parameters from the fast format header.  If an error
    occurs, set the default datum to Clarke-1866, reset the header values and
    return.
    Compare the ellipsoid parameters from the header files with those that
    LAS uses.  If a match is found, set the datum to it's LAS ID and return.
    If a match cannot be found, set the default datum to Clarke-1866, 
    reset the header values and return.
    

******************************************************************************/


static double ellipsoids[20][2] = {
{6378206.4, 6356583.8},        /* Clarke 1866*/
{6378249.145 , 6356514.87},    /* Clarke 1880 */
{6377397.155 , 6356078.9628},  /* Bessel            */
{6378157.5 , 6356772.2},       /* Internat. 1967    */
{6378388.0 , 6356911.94613},   /* Internat. 1909    */
{6378135.0 , 6356750.519915},  /* WGS 72            */
{6377276.3452 , 6356075.4133}, /* Everest           */
{6378145.0 , 6356759.769356},  /* WGS 66            */
{6378137.0 , 6356752.31414},   /* GRS 1980          */
{6377563.396 , 6356256.91},    /* Airy              */
{6377304.063 , 6356103.039},   /* Modified Everest  */
{6377341.89 , 6356036.143},    /* Modified Airy     */
{6376896.0 , 6355834.8467},    /* Walbeck           */
{6378155.0 , 6356773.3205},    /* Southeast Asia    */
{6378160.0 , 6356774.719},     /* Australian Nat.   */
{6378245.0 , 6356863.0188},    /* Krassovsky        */
{6378270.0 , 6356794.343479},  /* Hough             */
{6378166.0 , 6356784.283666},  /* Mercury 1960      */
{6378150.0 , 6356768.337303},  /* Mod. Mercury 1968 */
{6370997.0 , 6370997.0}        /* Sphere: 6370997   */
};


static void get_datum
(
    char *hdr,
    long *datum
)
{
   double e1;               /* semi-major axis of the ellipsoid from header */
   double e2;               /* semi-minor axis of the ellipsoid from header */
   double *eptr;            /* pointer to the current ellipsoid value       */
   long i;                  /* working index variable                       */
   long status;             /* function return status                       */
   

   /* read the semi-major axis 
      ------------------------*/

   status = c_fhdrrd(hdr,53,(char*)&e1);
   if(status == E_FAIL || e1 == HUGE_VAL)
   {
      /* something went wrong, so default to Clark-1866
	 ----------------------------------------------*/

     *datum = 0;
     e1 = 6378206.4;
     e2 = 6356583.8;
     
     return;
   }
   
   /* read the semi-minor axis
      ------------------------*/

   status = c_fhdrrd(hdr,55,(char *)&e2);
   if(status == E_FAIL || e2 == HUGE_VAL)
   {
     *datum = 0;
     e1 = 6378206.4;
     e2 = 6356583.8;
     
     return;
   }


   /* search for the ellipsoid that we found in the header 
      ----------------------------------------------------*/

   eptr = &ellipsoids[0][0];
   
   i = 0;
   do 
   {
      if(fabs(e1-eptr[0]) < 0.01 && fabs(e2-eptr[1]) < 0.01)
      {
	 /* found it, so break out of the loop and then return 
	    --------------------------------------------------*/
	 *datum = i;
	 break;
      }
      eptr += 2;
      i++;
   } while(i<20);

   /* we must have went through the entire list and didn't find it, so
      set the ellipsoid to the default.
      ---------------------------------------------------------------*/

   if(i == 20)
   {
     *datum = 0;
     e1 = 6378206.4;
     e2 = 6356583.8;
     
   }
}
