/******************************************************************************

NAME:update_ddr

PURPOSE: Update the information in the ddr obtained from the fast format 
         header.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
       Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
    Get the DDR record from disk.
    Allocate memory for all the band dependant records and then retrieve 
    them.
    Set the master line and sample parameters in the ddr.
    Set all of the valid flags to true.
    Update the projection, zone, and datum codes.
    Update the projection parameters and projection units.
    Update the corner points.
    Update the pixel size.
    Write the ddr to disk.
    For every band dependant record, copy the source, instrument, acquistion
    date and time.  Then write it out to disk.

******************************************************************************/
#include "fhdr2ddr.h"
#include "protos.h"

void update_ddr
(
    char hostddr[CMLEN],            /* name of the host ddr to update */
    long window[4],                 /* image window */
    double pixsize,                 /* ground pixel size */
    double proj_coef[15],           /* GCTP projection parameters */
    double corners[8],              /* image projection corner points 
				   (x,y) UL, UR, LL, LR */
    long proj_code,                 /* GCTP projection code */
    long zone_code,                 /* UTM or State Plane code */
    long datum,                     /* GCTP datum code */
    char source[32],                /* source of image */
    char instrument[32],            /* instrument */
    char ac_date[10],               /* acquistion date */
    char ac_time[8]                /* acquistion time */
)
{
   long fatal = -1;             /* fatal error value for c_errmsg */
   long i;                      /* working index value */

   struct BDDR *bddrs;          /* pointer to the array of band dependant
				   records */
   struct BDDR *bptr;           /* pointer to the current BDDR record */
   struct DDR ddr;              /* local ddr structure */
   
   
   /* get the DDR 
      -----------*/

   if(E_FAIL == c_getddr(hostddr,&ddr))
     c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal);

   /* allocate memory for the band dependant records
      ---------------------------------------------*/
   if((bddrs = (struct BDDR *) malloc(ddr.nbands*sizeof(struct BDDR))) == NULL)
     c_errmsg("Error allocating dynamic memory.","inpeddr-alloc",
	      &fatal);
   
   /* get the band dependant records
      ------------------------------*/
   bptr = bddrs;
   
   for(i=1;i<=ddr.nbands;i++)
   {
      if(E_FAIL == c_getbdr(hostddr,bptr,&i))
      {
	 c_intbdr(bptr);
	 bptr->bandno = i;
      }
      bptr++;
   }

   /* update the DDR record
      ---------------------*/

   ddr.master_line = window[SL];
   ddr.master_sample = window[SS];

   /* set all the valid flags to VALID 
      --------------------------------*/

   ddr.valid[0] = ddr.valid[1] = ddr.valid[2] = ddr.valid[3] = VALID;
   ddr.valid[4] = ddr.valid[5] = ddr.valid[6] = ddr.valid[7] = VALID;
   
   /* set the projection and zone codes 
      ---------------------------------*/

   ddr.proj_code = proj_code;
   ddr.zone_code = zone_code;
   ddr.datum_code = datum;


   strcpy(ddr.proj_units,"meters");
   
   /* copy the projection coefficients 
      --------------------------------- */

   i = 0;
   do 
   {
      ddr.proj_coef[i] = proj_coef[i];
      i++;
   } while(i<15);
   

   /* update the corner points 
      ------------------------ */

   ddr.upleft[0] = corners[1];
   ddr.upleft[1] = corners[0];

   ddr.loleft[0] = corners[5];
   ddr.loleft[1] = corners[4];

   ddr.upright[0] = corners[3];
   ddr.upright[1] = corners[2];

   ddr.loright[0] = corners[7];
   ddr.loright[1] = corners[6];

   
   /* set the ground distance 
      ----------------------- */

   ddr.pdist_y = ddr.pdist_x = pixsize;
   
   ddr.line_inc = ddr.sample_inc = 1.0;
   
   /* now write the ddr to disk
      -------------------------*/


   if(E_FAIL == c_putddr(hostddr,&ddr))
     c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal);

   /* now update the band dependant stuff 
      -----------------------------------*/

   bptr = bddrs;
   for(i=1;i<=ddr.nbands;i++)
   {
      strncpy(bptr->source,source,32);
      strncpy(bptr->instrument,instrument,32);
      strncpy(bptr->date,ac_date,10);
      strncpy(bptr->time,ac_time,8);

      if(E_FAIL == c_putbdr(hostddr,bptr))
	c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal);
      bptr++;
   }

   free(bddrs);
   
}
