/*****************************************************************************
NAME:	GETPTS

PURPOSE: Gets polygon coordinates from statistics file
         and define EFFECTIVE FILL REGION where polygons exist,
         to minimize execution time.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        6-Jan-83   J. Hwang
     2        9-May-85   Y. Sheu 
     3       13-Jan-89   K. Narayanan        Conversion to 5.0   
     5          May-90   D. VanderZee        Fixed numerous bugs

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
     open statistics file
     get master coordinates of the first input image if valid 
     else set them to 1 
     for each class obtain
        the number of polygons, the number of points per polygon,
        and the vertices of the polygon
        convert vertices relative to the image window 
        check that total of vertices does not exceed 10000    
     get the minimum and maximum values of the vertex coordinates
        obtained to determine the size of effective fill window
     get the intersection between the specified image window and
        the fill window
     adjust polygon vertices relative to the fill window

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "insert.h"
#include "statio.h"

void getpts
(
  char hos_in[][CMLEN],         /* I: Input images host names */
  char hos_sfin[],              /* I: Stats file host name */
  char class[],                 /* I: Class name */
  long *num_polygons,           /* O: Total number of polygons */
  long num_points[],            /* O: No. vertices/polygon */
  long wind[][4],               /* I: Input image windows */
  long fill_window[4],          /* I/O: Effective fill area */
  int table[],                /* O: Polygon vertices */
  long in_flag[]                /* I: Flags for using input images */
)
{
struct	DDR	ddr;
long	access,
	class_start, class_stop, 
	corner[4], 
	fd_sfin,
	i, ic, ipoly, ivout, ix, j, k, 
	nclass,
	npoints,
	nsite,
	points[2000],         /* polygon coordinates                      */
	shape[9],             /* shape vector                             */
	total, 
	xinc, xmaster,     /* x and y coordinates, master coordinates  */
	ymaster, yinc,     /* and sampling increments                  */
	window[4];

char	buf[ERRLEN];

*num_polygons = 0;

for (i = 0; i < 4; i++) fill_window[i] = wind[0][i];
/*
open statistics file
--------------------
*/
access = 0;
if (c_stopen( &fd_sfin, hos_sfin, &access, &nclass) !=  E_OPOLD) 
   c_errmsg ("Error opening statistics file", "insert-stopen", 
            &fatal_err);
if (nclass == 0) 
   c_errmsg ("No class found in the statistics file", "insert_noclass",
             &fatal_err); 

/*
set up class user specified or for all classes 
----------------------------------------------
*/
if (class[0] != '*') 
   {
   nclass = 1;

   if (c_stfdcl (&fd_sfin, class, &ic) != E_SUCC) 
      c_errmsg ("Specified class does not exist", "insert_noclass",
                 &fatal_err); 
   class_start = ic;
   class_stop = ic;
   }
else
   {
   class_start = 1;
   class_stop = nclass;
   }  

total = 0;
ix = 0;
corner[0] = 99999;
corner[1] = 99999;
corner[2] = 0;
corner[3] = 0;

/*
in1 image window bounds
-----------------------
*/
window[0] = fill_window[0];
window[1] = fill_window[1];
window[2] = fill_window[0]+fill_window[2]-1;
window[3] = fill_window[1]+fill_window[3]-1;

/*
initialize master coordinates and line/sample increments to 1
-------------------------------------------------------------
*/
xmaster = 1;
ymaster = 1;
xinc = 1;
yinc = 1;

/*
reset them if master coordinates are valid for input image 1
---------------------------------------------------------------
*/
if (in_flag[0])
   {
   c_getddr (hos_in[0], &ddr);
   if (ddr.master_line != 0 && ddr.master_sample != 0)
      {
      xmaster = ddr.master_sample;
      ymaster = ddr.master_line;
      }  
   if ((ddr.valid[DDINCV] == VALID) && 
       (ddr.sample_inc != 0 && ddr.line_inc != 0))
      {
      xinc = ddr.sample_inc;
      yinc = ddr.line_inc;
      }
   }  

/*
for each class k obtain vertex coordinates of all polygons
----------------------------------------------------------
*/
ipoly = 0;

for (k = class_start; k <= class_stop; k++)
   {
   if (c_stget (&fd_sfin, "NSITES", &k, &zero, shape, &nsite) != E_SUCC || 
       nsite == 0)
      {
      c_errmsg ("No sites in this class", "insert_nosite", &inf_err); 
      nsite = 0;
      }  

/*
   get coordinates of polygons for each site in a class
   ----------------------------------------------------
*/
   for (i = 1; i <= nsite; i++)
      {
      if (c_stget (&fd_sfin, "POLYGON", &k, &i, shape, points) != E_SUCC)
/*
         skip if site is not a polygon
         -----------------------------
*/
         {
         c_errmsg ("Site was not a polygon", "insert_poly", &inf_err); 
         continue;
         }  

/*
      obtain number of points per polygon & total number of points
      ------------------------------------------------------------
*/
      npoints = shape[2];
      num_points[(*num_polygons)++] = npoints;
      total += npoints;
      if (total > 10000) 
          c_errmsg ("Number of polygon vertices exceeds 10000",
                   "insert_numpoints", &fatal_err); 
      ivout = 0;

/*
      get coordinates(vertices)
      -------------------------
*/
      for (j = ix; j < ix+npoints; j++)
         {
         table[j*2] = (float) (points[2*(j-ix)]-xmaster)/xinc+1;
         if (corner[1] > table[j*2]) corner[1] = table[j*2];
         if (corner[3] < table[j*2]) corner[3] = table[j*2];

         table[j*2+1] = (float) (points[2*(j-ix)+1]-ymaster)/yinc+1;
         if (corner[0] > table[j*2+1]) corner[0] = table[j*2+1];
         if (corner[2] < table[j*2+1]) corner[2] = table[j*2+1];

/*
         check if polygons inside image in1
         ---------------------------------- 
*/
         if (table[j*2]  <  window[1]  ||  
             table[j*2]  >  window[3]  ||  
             table[j*2+1]  <  window[0]  ||  
             table[j*2+1]  >  window[2]) 
            ivout++;
         }  
      if (ivout  >  0) 
         {
         sprintf (buf, 
            "Class %d site %d polygon vertices outside of first image",
             k, i);
         c_errmsg (buf, "insert-vert", &inf_err);
         if (ivout  ==  npoints) ipoly++;
         }  
      ix += npoints;
      }  
   }  

if (*num_polygons == 0) 
   c_errmsg ("No polygon found in this statistics file", "insert_nopolygon",
              &fatal_err); 
if (ipoly  ==  *num_polygons) 
   c_errmsg ("All polygons are outside of first image", "insert_polyout1",
              &fatal_err); 

/*
get effective fill region of the first input image
--------------------------------------------------
*/
corner[2] = corner[2]-corner[0]+1;
corner[3] = corner[3]-corner[1]+1;

intsct (fill_window, corner, 0);

if (fill_window[1]  >  wind[1][3] || fill_window[0] >  wind[1][2]) 
   c_errmsg ("All polygons are outside of second image", "insert_polyout2",
             &fatal_err);
for (i = 0; i < total; i++)
   {
   table[2*i] = table[2*i]-fill_window[1]+1;
   table[2*i+1] = table[2*i+1]-fill_window[0]+1;
   }  

/*
close statistics file
---------------------
*/
c_stclse (&fd_sfin);

/*
get interior fill window with respect to the output image
---------------------------------------------------------
*/
fill_window[0] = fill_window[0]-wind[0][0]+1;
fill_window[1] = fill_window[1]-wind[0][1]+1;
}
