/*****************************************************************************
NAME:	PROCESS

PURPOSE: Inserts polygons from first image into the second image, with
         specified options for EXTER, INTER, and BACKGRND values.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        6-Jan-83   J. Hwang
     2        9-May-85   Y. Sheu 
     3       13-Jan-89   K. Narayanan        Conversion to 5.0   
     5.         May-90   D. VanderZee        Fixed numerous bugs

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   for the number of lines in the output image
      initialize output line with EXTER values if EXTER is specified
      else copy line from the second input image 
      if line is within the fill region 
         call POLYGON to determine if a pixel within the fill region
         is within or outside any polygon, and map its value accordingly
      write the line to the output 
      update max and min values
   close all files
   write max and min values to the ddr if requested

****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "util.h"
#include "insert.h"

void process
(
  char hos_out[],              /* I: Output host file name */
  long fill_window[4],         /* I: Effective polygonal region */
  long in_flag[2],             /* I: Flags for using images */
  long wind[][4],              /* I: Input image windows */
  long *nbands,                /* I: No. bands in the image */
  float *inter,                /* I: Grey level for interior region */
  float *exter,                /* I: Grey level for exterior region */
  float *bound,                /* I: Grey level for boundry */
  long inter_flag,             /* I: Option for interior region */
  long bound_flag,             /* I: Option for boundry */
  long num_polygons,           /* I: No. polygons */
  int table[],               /* I: Polygon vertices */
  long num_points[]            /* I: No. points/polygon */
)
{
double maxval[MAXBND]; 	          /* max pixel value array            */
double minval[MAXBND];	          /* min pixel value array            */

int	*fill, *iflag, *ispt; 	/* working arrays for polygon () */

long	i, ix,j,
	msg_flag = ALLCAT,
	type = EREAL;

/* Allocate working arrays for use in  polygon()
-----------------------------------------------*/
if ((fill = (int *) malloc(wind[1][NS] * sizeof (int))) == NULL)
      c_errmsg ("Error allocating dynamic memory", "insert-alloc", &fatal_err);
if ((iflag = (int *) malloc(wind[1][NS] * sizeof (int))) == NULL)
      c_errmsg ("Error allocating dynamic memory", "insert-alloc", &fatal_err);
if ((ispt = (int *) malloc(wind[1][NS] * sizeof (int))) == NULL)
      c_errmsg ("Error allocating dynamic memory", "insert-alloc", &fatal_err);



/*
initialize and enable processing message utility
and initialize min max arrays
------------------------------------------------
*/
c_inittm ("INSERT", "Processed", "line", "bands", " ", &wind[1][NL],
         nbands, &msgtim, &msg_flag);
curent = 1;
c_startm();

if (min_max) c_inmnmx (&type, nbands, minval, maxval);
/*
process line by line
--------------------
*/
for (i = 1; i <= wind[1][NL]; i++)
   {
   curent = i;
   if (!in_flag[1])
/*
      initialize output line with EXTER if specified 
      ----------------------------------------------
*/
      for (j = 0; j < *nbands; j++)
         c_pxfill (&outbuf[opoint[j]], &type, &wind[1][NS], exter); 

   else
/*
      else copy values from second image
      ----------------------------------
*/
      {
      c_estep (&gdescin[1]);
      for (j = 0; j < *nbands; j++)
         c_pxcopy (&inbuf2[ipoint2[j]], &outbuf[opoint[j]], &type, 
                   &wind[1][NS]);
      }  

   if (i >= fill_window[SL] && i < fill_window[SL]+fill_window[NL])
      {
/*
      if line is within fill region - determine if a pixel inside the 
      fill region is within or outside any polygon, and assign
      a value from first input image (or INTER) or second input image
      (or EXTER), respectively
      ---------------------------------------------------------------
*/
      if (in_flag[0]) c_estep (&gdescin[0]);

      ix = 0;
      for (j = 0; j < num_polygons; j++)
         {
         polygon(i-fill_window[SL]+1,&table[ix],num_points[j],fill_window[SS], 
                  fill_window[NS], *inter, *bound, inter_flag, bound_flag,
                  *nbands,fill,iflag,ispt);
         ix = ix+2*num_points[j];
         }  
      }
/*
   write the line to the output; update max and min arrays
   -------------------------------------------------------
*/
   c_estep (&gdescout);

   if (min_max)
      for (j = 0; j < *nbands; j++)
         c_pxmnmx (&outbuf[opoint[j]], &wind[1][NS], &minval[j], &maxval[j], 
                   &type); 

   }  

/*
stop processing message, close all files, and free memory
---------------------------------------------------------
*/
c_stoptm ();

c_egclse (&gdescin[0]);
c_egclse (&gdescin[1]);
c_egclse (&gdescout);


free (inbuf1);
free (inbuf2);

free (fill);
free (iflag);
free (ispt);

/*
write max and min values to the ddr
-----------------------------------
*/
if (min_max)
   c_svmnmx (hos_out, nbands, minval, maxval, "YES");

}
