/*****************************************************************************
*
*   NAME:  GETPAR 
*
*   FUNCTION: Receive the parameters from the parameter block. 
*
*   PROGRAM HISTORY:
*
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     1.0       2/84	K.A.Narayanan   Goddard 
*     4.0      10/87    D. Akkerman	CSB	PRF #4329
*     5.0      10/89	D. Etrheim	CSB	LAS 5.0 conversion
*     5.1      05/90	D. Etrheim	CSB	ADP #5506  Allow different
*					sl,ss for each image band.  ADP #5196
*					Add LTTYPE and ATRNAME parameters.
*     5.2      06/90	D. Etrheim	CSB	ADP #5196 Increase performance
*     5.3      10/91    T. Mittan       CSB     Check data type for improved
*						performance.
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*	Get the input image names and validate the windows and data types.
*   Get the Labeled Table name and the output image name if needed.  Get
*   the mask values and verify that there is the correct amount. Retrieve
*   the needed global values.
*
******************************************************************************/
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,		/* TAE parameter block		*/
    char hosin[][CMLEN],		/* Input host file names	*/
    char hosout[][CMLEN],			/* Output host file image name 	*/
    char *lthos,			/* Output LT host file name	*/
    long *nbands,			/* number of bands		*/
    long (*wind)[4],		/* window specification	for each image */
    long *mask,			/* mask values			*/
    long (*bands)[MAXBND+1],	/* band specifications for each input image */
    long *totbnds,			/* total number of bands	*/
    long *nmrimg,			/* number of images being processed */
    long *mskflg,			/* masking flag			*/
    long *imgflg,			/* FASLE if no output image is requested */
    long *msgtim,			/* message timing 		*/
    char atrnam[][10],		/* attribute names		*/
    char *lttype,			/* labeled tabel type		*/
    long *dataflag			 /* data type flag               */
)

{
struct VARIABLE *var;		/* pointer to TAE parameter */

char taeout[1][CMLEN],		/* output TAE name		*/
     taein[MAXIMG][CMLEN];	/* input TAE names		*/

long atrcnt = 0,		/* attribute count		*/ 
     count,			/* parameter count		*/
     dtype,			/* data type of each image	*/
     element = 0,		/* element for getin		*/
     i,j,			/* loop counters		*/
     imglimit = MAXIMG,		/* maximum number of images to process */ 
     len,			/* length of string		*/
     maxcnt = 0,		/* total count of mask values	*/
     non_fatal = 1,		/* for fatal errors		*/
     mode = IWRITE,		/* File access mode		*/
     one = 1,			/* constant of 1		*/
     qflag[MAXIMG][2],		/* flag indicating if subwindows specificed */
     same,			/* flag indicating if images have save window */
     zero = 0;			/* zero constant		*/

/* Get the input image names 
----------------------------*/
if (c_getin(vblock,"IN",&element,taein,hosin,nmrimg,wind,bands,nbands,
            totbnds,&imglimit,&same,qflag) != E_SUCC)
   return (E_FAIL);

if (!same)
   {
   c_errmsg("Input images do not have same window sizes",
            "intersect-size",&non_fatal);
   return (E_FAIL);
   }

/* Check the data type of each image
------------------------------------*/
for (i = 0; i < *nmrimg; i++)
   {
   if (c_gettyp(hosin[i],&one,&dtype) != E_SUCC)
      return (E_FAIL);
   if (dtype == EREAL)
      {
      c_errmsg("Real images are not allowed","intersect-real",&non_fatal);
      return (E_FAIL);
      }
   if (dtype != EBYTE)
      *dataflag = TRUE;
   }

/* Check the windows
--------------------*/
/*for (i = 1; i < *nmrimg; i++)
   {
   if ((wind[i][0] != wind[0][0]) || (wind[i][1] != wind[0][1]) ||
       (wind[i][2] != wind[0][2]) || (wind[i][3] != wind[0][3]))
      {
      c_errmsg("Input images must use the same processing window",
               "intersect-window",&non_fatal);
      return(E_FAIL);
      }
   }*/ 

/* Get LT file name
-------------------*/
if (c_getnam(vblock,"OUTLT",&zero,lthos,"lt",&mode) != E_SUCC)
   return(E_FAIL); 

/* Get output image name
------------------------*/
if (c_getcnt(vblock,"OUT",&count) != E_SUCC)
    return(E_FAIL);
if (count != 0)
   {
   if (c_getout(vblock,"OUT",taeout,hosout) != E_SUCC)
      return(E_FAIL);
   }
else
   *imgflg = FALSE;

/* Get attribute names 
----------------------*/
   if (c_getcnt(vblock,"ATRNAME",&count) != E_SUCC)
       return(E_FAIL);
   if (count != 0)
      {
      var = p_find(vblock,"ATRNAME");
      for (j = 0; j < count; j++)
          strcpy(atrnam[j+atrcnt],SVAL(*var,j));
      atrcnt += count;
      }
if (atrcnt != *totbnds)
   {
   c_errmsg("Number of attribute names must equal the number of image bands",
            "intersect-atrname",&non_fatal);
   return(E_FAIL);
   }

/* Make attribute names upper case
---------------------------------*/
for (i = 0; i < atrcnt; i++)
   {
   len = (long) strlen(atrnam[i]);
   c_low2up(atrnam[i],&len);
   }

/* Get labeled tabel type 
-------------------------*/
var = p_find(vblock,"LTTYPE");
strcpy(lttype,SVAL(*var,0));
len = (long) strlen(lttype);
c_low2up(lttype,&len);

/* Get mask value
-----------------*/
   if(c_getcnt(vblock,"MASKVAL",&count) != E_SUCC)
      return(E_FAIL);
   if (count != 0)
      {
      var = p_find(vblock,"MASKVAL");
      for (j = 0; j < count; j++)
          mask[j+maxcnt] = IVAL(*var,j);
      maxcnt += count;
      *mskflg = TRUE;
      }
if ((*mskflg) && (maxcnt != *totbnds))
   {
   c_errmsg("Number of mask values must equal the number of image bands",
            "intersect-mask",&non_fatal);
   return(E_FAIL);
   }
   
/* Retrieve globals
-------------------*/
var = p_find(vblock,"$msgtim");
*msgtim = IVAL(*var,0); 

return(E_SUCC);
}
