/*****************************************************************************
*
*   NAME: GETPRNM 
*
*   FUNCTION:  Set the permutation number for this set of attributes.  Search
*          the permutation table for a permutation with matching attributes.
*          If such a permutation is found, update the permutation counter and
*          use this permutation number.  Otherwise, log the new permutation in
*          the table and assign it the next sequential permutation number.
*
*   FILE REFERENCES:
*       SORT.DAT - Unsorted permutation file
*
*   PROGRAM HISTORY:
*   Version     Date    Programmer      Code    Request
*   -------     ----    ----------      ----    -------
*     1.0       2/84    K.A.Narayanan   Goddard
*     4.0      10/87    D. Akkerman     CSB     PRF #4329
*     5.0      10/89    D. Etrheim      CSB     LAS 5.0 conversion
*     5.2      06/90    D. Etrheim      CSB     ADP #5196 Increase performance
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*    Set THIS to the header for the linked list
*    Loop until the desired permutation number is found
*       Set this to point to the current node in the list
*       If this node is the list header Then
*         Add this permutation at the beginning of the linked list
*          If the linked list is full Then
*             Write the last node to the permutation file
*	      Reset the links for the new permutation
*          Else 
*             Load the new permutation 
*             Set the links for the new permutation 
*          Set the permutation value 
*          Increment the next permutation number 
*       Else 
*          Check for this permutation in the current node
*          For each input image band
*             If the attribute values don't match
*                Branch back to retrieve the next node
*          This is the matching permutation
*          Set the permutation valued 
*          Increment the count of this permutation
*          Set the links for this permutation 
*
******************************************************************************/
#include <stdio.h>
#include "intersect.h"
#include "protos.h"

void intersect_getprnm
(
    long *forlnk,			/* Permutation record forward link */
    long *bcklnk, 			/* Permutation record previous link */
    FILE *fp,			/* file descriptor to permutation file */
    long *totbnds, 			/* total no. of bands to be processed */
    long (*attr)[TBLSIZ + 1],	/* Attributes defining the permutation */
    long *pcount,			/* Permutation occurrence counter */
    long *prmnum,			/* List of permutation numbers */
    long *curatt,			/* Attributes defining the current permutation*/
    long *mincnt,			/* Shortest current run length code */
    long *prmval,			/* Permutation value */
    long *next,			/* Next permutation number */
    long *addoff			/* offset to be added */
)
{
long this,		/* Index to the current permutation table cell */
     last;		/* Index to the last permutation table cell */
long i;			/* loop counter */

/* Set this to the header for the linked list
---------------------------------------------*/
this = 0;

/* Traverse the linked list until a matching permutation is found
----------------------------------------------------------------*/
loop: this = forlnk[this];
if (this == 0) 
   {
   /* Add this permutation at the beginning of the linked list
   ----------------------------------------------------------*/
   if ( *next > TBLSIZ) 
      {
      /* The table is full.  Write the last permutation to the permutation file
      ------------------------------------------------------------------------*/
      last = bcklnk[0];
      fprintf(fp,"\n%11d%11d",prmnum[last],pcount[last]);
      for (i = 0; i < *totbnds; i++)
           fprintf(fp,"%11d",(attr[i][last] + *addoff));
      /* Load the new permutation into last
      ------------------------------------*/
      for (i = 0; i < *totbnds; i++) 
           attr[i][last] = curatt[i];
      pcount[last] = *mincnt;
      prmnum[last] = *next;
      /* Extract last from the end of the linked list
      ----------------------------------------------*/
      bcklnk[0] = bcklnk[last];
      forlnk[bcklnk[last]] = 0;
      /* Set the links in last for the first node of the linked list
      -------------------------------------------------------------*/
      forlnk[last] = forlnk[0];
      bcklnk[last] = 0;
      /* Place last at the beginning of the linked list
      ------------------------------------------------*/
      bcklnk[forlnk[0]] = last;
      forlnk[0] = last;
      }
  else
      {
      /* The table is not full
      ------------------------*/
      for (i = 0; i < *totbnds; i++)
          attr[i][*next] = curatt[i];
      pcount[*next] = *mincnt;
      prmnum[*next] = *next;
      /* Set the links in next for the first node of the linked list
      -------------------------------------------------------------*/
      forlnk[*next] = forlnk[0];
      bcklnk[*next] = 0;
      /* Place next at the beginning of the linked list
      -------------------------------------------------*/
      bcklnk[forlnk[0]] = *next;
      forlnk[0] = *next;
      } 
   *prmval = *next;
   *next += 1;
   }
else
   {
   /* Check for this permutation in the current node.  Get 
   the next permutation if the current is not a match. 
   ------------------------------------------------------*/
   for (i = 0; i < *totbnds; i++)
       if (curatt[i] != attr[i][this]) 
            goto loop; 
   /* This is the matching permutation 
   -----------------------------------*/
   *prmval = prmnum[this];
   pcount[this] += *mincnt;
   if (bcklnk[this] != 0)
      {
      /* Extract this from the linked list
      -----------------------------------*/
      bcklnk[forlnk[this]] = bcklnk[this];
      forlnk[bcklnk[this]] = forlnk[this];
      /* Set the links in this for the first node of the linked list
      -------------------------------------------------------------*/
      forlnk[this] = forlnk[0];
      bcklnk[this] = 0;
      /*  Place this at the beginning of the linked list
      -------------------------------------------------*/
      bcklnk[forlnk[0]] = this;
      forlnk[0] = this;
      }
   }
}
