/*****************************************************************************
*
*   NAME:  INTERSECT
*
*   FUNCTION:  Compare corresponding pixels from multiple input images and
*              generate a Labeled Table and an optional single-band output
*              image.  Each entry in the Labeled Table stores a permutation 
*              code, a count of the number of pixels that have been assigned
*              that permutation value, and a list of the input image pixel 
*              pixel values that compose the permutation.  Each pixel in
*              the output image stores the permutation code assigned to the
*              collection of pixel values from the input images.
*
*   PROGRAM HISTORY:
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     1.0       2/84	K.A.Narayanan   Goddard 
*     4.0	10/87	D. Akkerman     CSB	PRF #4329
*     5.0       10/89	D. Etrheim	CSB	LAS 5.0 conversion
*     5.1       05/90	D. Etrheim	CSB	ADP #5506  Allow different
*					sl,ss for each image band.  ADP #5196
*					Add LTTYPE and ATRNAME parameters.
*     5.2       06/90	D. Etrheim	CSB	ADP #5196 Increase performance
*     5.3       10/91   T. Mittan       CSB     Checked data type befor minmax
*						on vms.
*     5.4	09/92	T. Mittan	CSB	Added extern to curent to 
*						eliminate warnings on link.
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*
*	Initial the processing session.  Get the input parameters and do
*   validity checking. Process the images by overlaying and creating 
*   permutation codes based on the input pixel values.  Sort the permutation 
*   file and create a Labeled Table and optionally a translation table and
*   an output image after the duplicate permutations have been removed.  
*   Conditionally update the image history and issue a completion message. 
*
******************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
struct FDESC * fdesc[MAXIMG+1];                                
                            /* image file descriptors used for image I/O	*/
struct GDESC * gdesc;		/* group descriptor used for image I/O		*/
long offset1[MAXIMG*MAXBND];/* offsets for input bands			*/
long offset2[MAXBND];	/* offsets for output 				*/
extern long curent;	/* current line being processed			*/

int main
(void)
{ 
struct PARBLK vblock;			/* TAE parameter block		*/

char  atrnam[MAXBND][10],		/* attribute names		*/ 
      hosin[MAXIMG][CMLEN],		/* input host file names	*/
      hosout[1][CMLEN],			/* output host file name	*/
      lthos[CMLEN],			/* output LT host file name	*/
      lttype[CMLEN],			/* labeld tabel type		*/
      taeout[2][CMLEN];			/* output TAE names		*/

static long bands[MAXIMG][MAXBND+1];	/* bands to be processed	*/

long  fatal = -1,			/* for fatal error messages	*/
      dataflag = FALSE,			/* data type flag               */
      imgflg = TRUE,			/* flag if creating an output image */ 
      mask[MAXBND],			/* mask values for each band	*/ 
      msgtim,				/* message timing value		*/
      mskflg = FALSE,			/* flag if masking is to take place */
      nbands[MAXIMG],			/* number of bands for each image */
      nmrimg,				/* number of images being processed */
      non_fatal = 1,			/* for non_fatal errors		*/
      nout = 1,				/* number of output images/files */
      std_out,				/* standard output		*/
      totbnds,				/* total number of bands	*/
      wind[MAXIMG][4];			/* window specification for each image*/
      
/*Initialize LAS processing
---------------------------*/
c_inlas(&vblock,&std_out);

/*Get the input parameters
-------------------------*/
if (getpar(&vblock,hosin,hosout,lthos,nbands,wind,mask,bands,&totbnds,
           &nmrimg,&mskflg,&imgflg,&msgtim,atrnam,lttype,&dataflag) != E_SUCC)
   c_errmsg("Fatal error encountered","intersect-fatal",&fatal);

/* Process the images
--------------------*/
proces(&vblock,hosin,hosout[0],lthos,wind,bands,nbands,&totbnds,&nmrimg,mask,
       &mskflg, &imgflg,&msgtim,atrnam,lttype,&dataflag);

/* Conditionally update the history record
-----------------------------------------*/
if (imgflg)
   if (c_hstout(&vblock,hosout[0],hosin,&nmrimg," ") != E_SUCC) 
      c_errmsg("Nonfatal error encountered","intersect-warn",&non_fatal);

/* Set up TAE names for completion message
------------------------------------------*/
strcpy(taeout[0], lthos);
if (imgflg)
   {
   strcpy(taeout[1], hosout[0]);
   nout += 1;
   }
c_complt("INTERSECT",taeout,&nout);

return EXIT_SUCCESS;

} 
