/*****************************************************************************
*
*   NAME: OPNLAS 
*
*   FUNCTION:  Open input and output images 
*
*   PROGRAM HISTORY:
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     1.0       2/84	K.A.Narayanan   Goddard 
*     4.0      10/87	D. Akkerman	CSB	PRF #4329	
*     5.0      10/89	D. Etrheim	CSB	LAS 5.0 conversion
*     5.1      05/90	D. Etrheim	CSB	ADP #5506  Allow different
*					sl,ss for each image band.  ADP #5196
*					Add LTTYPE and ATRNAME parameters.
*     5.2      10/91    T. Mittan       CSB      Check data type before
*                    				 performing minmax.  If byte
*						 image don't minmax.
*     6.1      05/94    D. Eichacker            Added opt[].
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*	For the first pass, open the input image(s) and optionally the
*   temporary output image.  If a VMS machine, calculate the offset value.
*   For the second pass, open the temporary image and the output image with 
*   the smallest possible data type.
*   NOTE:  An offset value is added to the attribute values in the permutation
*   file.  The offset value is zero if run on a UNIX machine or the minimum of
*   (zero,lowest pixel value in input images) on a VMS machine.  The VMS 
*   sort being used will not handle negative data when represented as
*   characters; therefore, all values are made positive before the system
*   sort and then the offset is subtracted before creating the Labeled Table.
*
******************************************************************************/
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "protos.h"

int opnlas
(
    char hosin[][CMLEN],		/* input host file names	*/
    char *hosout,			/* output host file name	*/
    long (*wind)[4],		/* window specifications for each input image */
    long (*bands)[MAXBND + 1],	/* bands to be processed for all images */
    long *nbands,			/* number of bands for input images*/
    long *nmrimg,			/* number of input images	*/
    long *imgflg,			/* flag if creating an output image */
    char *hostmp,			/* temp host file name		*/
    long *pass,			/* flag to determine which pass	*/
    long *maxout,			/* maximum output permutation value */
    long *nl,			/* number of lines		*/
    long *ns,			/* number of samples		*/
    long *addoff,			/* offset to be added		*/
    long *range,			/* TRUE if unable to handle the data range */
    long *dataflag		        /* data flag                    */
)
{
/* The following variables have been declared global for image I/O routines
--------------------------------------------------------------------------*/
extern struct FDESC * fdesc[MAXIMG+1]; 
                                /* image file descriptors used for image I/O*/
extern long offset1[MAXIMG*MAXBND];/* offsets for input bands		*/
extern long offset2[MAXBND];/* offsets for output 			*/

long  access,			/* file access			*/
      band,			/* band counter			*/ 
      image,			/* image counter		*/
      odtype = ELONG,		/* output data type		*/
      open = ELONG,		/* data type			*/
      one = 1,			/* constant of one		*/
      sl,			/* starting line in input image window	*/
      ss,			/* starting sample in input image window */
      status,			/* function return status	*/
      zero = 0;			/* zero constant		*/

#ifdef vms
 
long maxv = 0,   /* maximum pixel value  */
     i;          /* loop counter         */

double minval[MAXBND],    /* minimum pixel values */ 
       maxval[MAXBND];    /* maximum pixel values */


#endif

static	long	opt[3]={0,0,0}; /* for c_eopens			*/

char  cbuf[CMLEN],		/* temp character buffer	*/
      taetmp[CMLEN];		/* temp TAE image name		*/

/* Open the input images
------------------------*/
if (*pass == 1)
   {
   access = IREAD;

   for (band = 0, image = 0; image < *nmrimg; image++)
      {
      sl = wind[image][0];
      ss = wind[image][1];
      *nl = wind[image][2];
      *ns = wind[image][3];
      status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,nl,ns,
                        &offset1[band],&access,&odtype,&zero,opt);
      if (status != E_SUCC)
         return(E_FAIL);
      band += nbands[image];
      }

#ifdef vms
if (*dataflag)
 {
   /* Find the minimum and maximum pixel value.  If the minimum +
   maximum is greater then the value we can store in I*4, set the offset
   value to zero and flag it to force a bubble sort.
   ----------------------------------------------------------------*/
   if (c_minmax(hosin,nmrimg,wind,bands,nbands,minval,maxval) != E_SUCC)
      return(E_FAIL);
   for (i = 0; i < band; i++)
      *addoff = min(*addoff,minval[i]);
   *addoff = abs(*addoff);
   for (i = 0; i < band; i++)
      maxv = max(maxv,maxval[i]);
   if (maxv > (MAXLONG - *addoff))
       {
       c_errmsg("Unable to handle the data range efficiently",
                "intersect-range",&one);
       *range = TRUE;
       *addoff = 0;
       }
 }
 else
 {
     *range = FALSE;
     *addoff = 0;
 }
#endif

   /* Conditionally open the output image
   -------------------------------------*/
   if (*imgflg)
      {
      strcpy(cbuf,"inter.tmp");
      status = c_mkimg(cbuf,"",taetmp,hostmp,&one);
      if (status != E_SUCC)
         {
         c_errmsg("Temporary image name already exists","intersect-temp",
                   &one);
         c_delfil(hostmp);
         }

      access = IWRITE;
      status = c_eopens(&fdesc[*nmrimg],hostmp,&one,&one,&one,nl,ns,offset2,
                        &access,&odtype,&zero,opt);

      if (status != E_SUCC)
         return(E_FAIL);
      fdesc[*nmrimg+1] = 0;
      }
   }

/* Open the input and output images for the second pass
------------------------------------------------------*/
else
   {
   access = IREAD;
   odtype = ELONG;
   status = c_eopens(&fdesc[0],hostmp,&zero,&one,&one,nl,ns,
                     &offset1[0],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      return(E_FAIL); 

   if (*maxout < MAXUTINY)
      opt[0] = EBYTE;
   else if (*maxout < MAXWORD)
      opt[0] = EWORD;
   else
      opt[0] = ELONG;
   access = IWRITE;
   status = c_eopens(&fdesc[1],hosout,&one,&one,&one,nl,ns,
                     offset2,&access,&open,&zero,opt);
   opt[0] = 0;
   if (status != E_SUCC)
      return(E_FAIL);
   fdesc[2] = 0;
   }

return(E_SUCC);
}
