/*****************************************************************************
*
*   NAME:  OVRLAY
*
*   FUNCTION:  Overlay the input images to detect and log permutations of the
*          run-length code intersections.
*
*   FILE REFERENCES:
*	SORT.DAT - Unsorted permutation file
*       
*   PROGRAM HISTORY:
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     1.0       2/84	K.A.Narayanan   Goddard 
*     4.0      10/87	D. Akkerman	CSB	PRF #4329	
*     5.0      10/89	D. Etrheim	CSB	LAS 5.0 conversion
*     5.2      06/90	D. Etrheim	CSB	ADP #5196 Increase performance
*     5.3      10/91    T. Mittan       CSB     Time stamped temporary file.
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*    Open the permutation file
*    Initialize values for overlay processing
*    For each line of the input images
*       For each sector in these input image lines
*           Set sectsz to the minimum of 512  and the remaining samples
*           Get the run length codes of this sector
*           Do (while there are more pixels in this sector)
*              Scan for the shortest current run length code
*              If mask values have been specified during this run 
*                  For each band of the input images
*                     check to see if pixel value matches mask value
*              If the Permutation has not been located
*                 For each band of the input images
*                   Load this band of attributes from runval 
*                   Get the needed permutation number
*              If an image is being created 
*                Load this segment of the temporary image
*   Write the contents of the permutation table to the permutation file
*   
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "intersect.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void ovrlay
(
    struct PARBLK *vblock,		/* TAE paramater block		*/
    long *totbnds,			/* total no. of bands to be processed */
    long *mskflg,			/* masking flag			*/
    long *msgtim,			/* for message display		*/
    long *nl,			/* number of lines		*/
    long *ns,			/* number of samples		*/
    long *thebuf,			/* Image I/O buffer		*/
    long *mask,			/* mask values 			*/
    long *npum,			/* number of permutations	*/
    long *imgflg,			/* flag if image is to be created */
    long *mskcnt,			/* number of mask values	*/
    char hostmp[][CMLEN],			/* temp host file name		*/
    long *addoff, 			/* offset to be added */
    char *selfile                  /* sort file name               */
)

{
/* The following variables have been declared global for image I/O routines
--------------------------------------------------------------------------*/
extern struct GDESC * gdesc;	/* group descriptor used for image I/O 		*/
extern long offset2[MAXBND];/* offsets for output 			*/
extern long curent;	/* current line being processed			*/

FILE *fp;			/* File descriptor		*/

long  curatt[MAXBND],		/* attributes for a permutation */
      i,j,			/* loop counters		*/
      msgflg = ALLCAT,		/* message processing		*/
      non_fatal = 1,		/* for nonfatal error messages	*/
      one = 1;			/* constant of one		*/

long zero = 0,			/* constant of zero		*/
     elong= ELONG;		
long bcklnk[TBLSIZ + 1],	/* Permutation record forward link */
     forlnk[TBLSIZ + 1];	/* Permutation record previous link */
long bufndx;			/* Image buffer index to the current sample */
static long runcnt[SCTSIZ][MAXBND],	/* Run length code count table */
     runval[SCTSIZ][MAXBND];	/* Run length code value table */
long runptr[MAXBND];		/* Pointer to next run length code per band */
long sectsz,			/* Size of the current line sector */
     sector,			/* loop index	*/
     mincnt,			/* Shortest current run length code */
     fndprm,			/* flag if permutation was found */
     prmval,			/* permutation value */
     this;			/* Index to the current permutation table cell*/

long next = 1;			/* next permutation value */
static long attr[MAXBND][TBLSIZ + 1]; /* Attributes defining the permutation */
static long pcount[TBLSIZ + 1],	/* Permutation occurrence counter */
     prmnum[TBLSIZ + 1];	/* List of permutation numbers */

/* Open the temp sort file
--------------------------*/
strcpy(selfile,"intersect");
c_creat_name(selfile,"dat");
if ((fp = fopen(selfile,"w")) == NULL)
     { 
     c_errmsg("Error opening temporary file","intersect-open",&non_fatal);
     c_clean(vblock,"intersect",hostmp,&one);
     }
bcklnk[0] = forlnk[0] = 0;

/* Start process timing message
-------------------------------*/
c_inittm("INTERSECT","Processed","line","bands"," ",nl,totbnds,msgtim,&msgflg);
curent = 1;
c_startm();

for (curent = 0; curent < *nl; curent++)
    {
    if (c_estep(&gdesc) != E_SUCC)
       {
       fclose(fp);
       c_delfil("sort.dat");
       c_clean(vblock,"intersect",hostmp,&one);
       }
    bufndx = 0;
    /* Do for each sector in the input image lines
    ----------------------------------------------*/
    for (sector = 0; sector < *ns; sector += SCTSIZ)
       {
       sectsz = min(SCTSIZ,(*ns - bufndx));
       /* Get the run length codes for this sector
       -------------------------------------------*/
       getrun(totbnds,&bufndx,&sectsz,runcnt,runval,thebuf);
       c_pxfill(runptr,&elong,totbnds,&zero);
       do 
          {
          mincnt = sectsz;
          /*  Scan for the shortest current run length code
          -------------------------------------------------*/
          for (i = 0; i < *totbnds; i++)
             mincnt = min(mincnt,runcnt[(runptr[i])][i]);
          sectsz -= mincnt;
          /* Determine the permutation number for this 
          intersection.  Check for mask values if necessary.
          --------------------------------------------------*/
          fndprm = TRUE;
          if (*mskflg)
             {
             for (i = 0; i < *totbnds; i++)
                {
                if (runval[(runptr[i])][i] == mask[i])
                   {
                   /* A run code value matches its mask value
                   ------------------------------------------*/
                   if (fndprm)
                      {
                      fndprm = FALSE;
                      prmval = 0;
                      *mskcnt += mincnt;
                      /* Decrement mincnt form each curent runcnt
                      -------------------------------------------*/
                      for (j = 0; j < *totbnds; j++)
                          {
                          runcnt[(runptr[j])][j] -= mincnt; 
                          if(runcnt[(runptr[j])][j] == 0)
                             runptr[j] +=1;
                          }
                      }
                   }
                }
             }
          if (fndprm)
             {
             for (i = 0; i < *totbnds; i++)
                {
                curatt[i] = runval[(runptr[i])][i];
                runcnt[(runptr[i])][i] -= mincnt;
                if (runcnt[(runptr[i])][i] == 0)
                   runptr[i] += 1;
                }
             /* Get the permutation value 
             ---------------------------*/
             intersect_getprnm(forlnk,bcklnk,fp,totbnds,attr,pcount,prmnum,
                               curatt,&mincnt,&prmval,&next,addoff);
             }
          /* If an output image is requested, fill the buffer
          ---------------------------------------------------*/
          if (*imgflg)
             c_pxfill(&thebuf[offset2[0] + bufndx],&elong,&mincnt,&prmval);
          bufndx += mincnt;
          }
       while (sectsz > 0);
       }
    }

c_stoptm();

/* Final step 
-------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   c_delfil("sort.dat");
   c_clean(vblock,"intersect",hostmp,&one);
   }

/* Set the number of permutations found
--------------------------------------*/
*npum = next - 1;

/* Write the permutations to the file 
-------------------------------------*/
this = 0;
this = forlnk[this];
do 
   {
   fprintf(fp,"\n%11d%11d",prmnum[this],pcount[this]);
   for (i = 0; i < *totbnds; i++)
      fprintf(fp,"%11d",(attr[i][this] + *addoff));
   this = forlnk[this];
   }
while (this != 0);
fprintf(fp,"\n");
fclose(fp);
}
