/*****************************************************************************
*
*   NAME:  PROCES 
*
*   FUNCTION: Proces the images 
*
*   FILE REFERENCES:
*	FILE1 - Sorted permutation file
*       
*   PROGRAM HISTORY:
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     1.0       2/84	K.A.Narayanan   Goddard 
*     4.0      10/87	D. Akkerman	CSB	PRF #4329	
*     5.0      10/89	D. Etrheim	CSB	LAS 5.0 conversion
*     5.1      05/90	D. Etrheim	CSB	ADP #5506  Allow different
*					sl,ss for each image band.  ADP #5196
*					Add LTTYPE and ATRNAME parameters.
*     5.2      06/90	D. Etrheim	CSB	ADP #5196 Increase performance
*     5.3      10/91    T. Mitan        CSB     Time stamp sort files.
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*	Open the images and allocate space for Image I/0 and the translation
*   table.  Process the images by overlaying and creating permutation codes 
*   based on the input pixel values.  Sort the permutation file.  Initialize 
*   the Labeled Table, write the values to the LT, and create the translation 
*   table if needed.  If and output image is requested, translate the image, 
*   update the DDR, and delete the temporary image.  Free the allocated space.
*   NOTE:  An offset value is added to the attribute values in the permutation
*   file.  The offset value is zero if run on a UNIX machine or the minimum of 
*   (zero,lowest pixel value in input images) on a VAX/VMS machine.  The 
*   VMS sort being used will not handle negative data when represented as
*   characters; therefore, all values are made positive before the system
*   sort and then the offset is subtracted before creating the Labeled Table. 
*   
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "intersect.h"
#include "util.h"
#include "protos.h"

void proces
(
    struct PARBLK *vblock,		/* TAE parameter block		*/
    char hosin[][CMLEN],		/* input host file names	*/
    char *hosout,			/* output host file name	*/
    char *lthos,			/* output LT host file name	*/
    long (*wind)[4],		/* window specifications for each input image */
    long (*bands)[MAXBND + 1],	/* bands to be processed for all images */
    long *nbands,			/* number of bands for input images*/
    long *totbnds,			/* total no. of bands to be processed */
    long *nmrimg,			/* number of input images	*/
    long *mask,			/* mask values for each band	*/
    long *mskflg,			/* masking flag			*/
    long *imgflg,			/* FALSE if no output image	*/
    long *msgtim,			/* for message display		*/
    char atrnam[][10],		/* attribute names		*/
    char *lttype,			/* labeled tabel type		*/
    long *dataflag                 /* datatype flag                */
)

{
/* The following variables have been declared global for image I/O routines
--------------------------------------------------------------------------*/
extern struct FDESC * fdesc[MAXIMG+1]; 
                                /* image file descriptors used for image I/O*/
extern struct GDESC * gdesc;	/* group descriptor used for image I/O 		*/

struct TAB_DEF tab;		/* Labeled Table descriptor	*/

double line_inc,		/* line increment for ddr update */
       samp_inc;		/* sample increment for ddr update */
double minval[MAXBND],		/* minimum pixel value in the output image */	
       maxval[MAXBND];		/* maximum pixel value in the output image */

long  addoff = 0,		/* offset to be added		*/
      bufsiz,			/* size of the image I/O buffer */
      bufsiz1,			/* size of the image I/O buffer */
      maxout = 1,		/* max number of permutations 
				   after duplicates removed	*/
      mskcnt= 0,		/* mask value count		*/
      ndel,			/* number of files to delete on failure */
      non_fatal = 1,		/* for nonfatal error messages	*/
      nl,			/* number of lines		*/
      ns,			/* number of samples		*/
      npum = 0,			/* number of permutations	*/
      one = 1,			/* constant of one		*/
      pass = 1,			/* flag to determine which pass */
      range = FALSE,		/* TRUE if unable to handle the data range */
      status;			/* function return status	*/
static long trntbl[MAXTBL];	/* translation table		*/
long   up_flg;			/* update flag for ddr		*/

long  *thebuf;			/* Image I/O buffer		*/
long  *thebuf1;			/* Image I/O buffer		*/

char  file1[CMLEN];		/* sort file name		*/
char  file3[CMLEN];		/* sort file name		*/
char  hostmp[1][CMLEN];		/* temp host image name		*/
char  mnmx[8];			/* mimnax flag			*/ 
char  tbuf[3][CMLEN];		/* temp buffer used for clean up */
char  selfile[CMLEN];           /* temp sort file                */
                     

/* Open the input images
------------------------*/
if (opnlas(hosin,hosout,wind,bands,nbands,nmrimg,imgflg,
           hostmp[0],&pass,&maxout,&nl,&ns,&addoff,&range,dataflag) != E_SUCC)
   c_clean(vblock,"intersect",hostmp,&one);

/* Allocate the I/O buffer
--------------------------*/
if ((thebuf = (long *) c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","intersect-alloc",&non_fatal);
   c_clean(vblock,"intersect",hostmp,&one);
   } 

/* Group the images
-------------------*/
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   c_clean(vblock,"intersect",hostmp,&one);

/* Overlay the input images
---------------------------*/
ovrlay(vblock,totbnds,mskflg,msgtim,&nl,&ns,thebuf,mask,&npum,
       imgflg,&mskcnt,hostmp,&addoff,selfile);

/* Close all image files
-----------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","intersect-warn",&non_fatal);

/* Sort the permutation file
----------------------------*/
if (sortbl(&npum,totbnds,file1,&range,selfile) != E_SUCC)
      c_clean(vblock,"intersect",hostmp,&one);

/* Open and write the label vectors to the LT file
--------------------------------------------------*/
initlt(&tab,hosin,lthos,bands,nbands,totbnds,mskflg,&mskcnt,atrnam,lttype);

/* Create translation table and the LT file
-------------------------------------------*/
if (crttbl(&tab,trntbl,&npum,totbnds,&maxout,file1,imgflg,&addoff,
    file3) != E_SUCC)
    {
    close_tab(&tab);
    strcpy(tbuf[0],lthos);
    strcpy(tbuf[1],hostmp[0]);
    ndel = 2;
    c_clean(vblock,"intersect",tbuf,&ndel);
    }

close_tab(&tab);

/* Conditionally open the input and output images
------------------------------------------------*/
if (*imgflg)
   {
   pass = 2;
   if (opnlas(hosin,hosout,wind,bands,nbands,nmrimg,imgflg,
             hostmp[0],&pass,&maxout,&nl,&ns,&addoff,&range,dataflag) != E_SUCC)
       {
       c_delfil(file3);
       strcpy(tbuf[0],lthos);
       strcpy(tbuf[1],hostmp[0]);
       strcpy(tbuf[2],hosout);
       ndel = 3;
       c_clean(vblock,"intersect",tbuf,&ndel);
       }

   if ((thebuf1 = (long *) c_ealloc(fdesc,&bufsiz1)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","intersect-alloc",&non_fatal);
      c_clean(vblock,"intersect",hostmp,&one);
      } 

   /* Group the images
   -------------------*/
   if (c_egroup(fdesc,&gdesc,thebuf1,&bufsiz1) != E_SUCC)
       {
       c_delfil(file3);
       strcpy(tbuf[0],lthos);
       strcpy(tbuf[1],hostmp[0]);
       strcpy(tbuf[2],hosout);
       ndel = 3;
       c_clean(vblock,"intersect",tbuf,&ndel);
       }

   /* Translate the image
   ----------------------*/
   if (trnimg(&nl,&ns,thebuf1,&npum,trntbl,msgtim,file3) != E_SUCC)
       {
       strcpy(tbuf[0],lthos);
       strcpy(tbuf[1],hostmp[0]);
       strcpy(tbuf[2],hosout);
       ndel = 3;
       c_clean(vblock,"intersect",tbuf,&ndel);
       }

   /* Update the DDR
   ----------------*/
   line_inc = (double)one;
   samp_inc = (double)one;
   up_flg = COMB;
   status = c_upddr(vblock,hosin,hosout,nmrimg,wind,bands,nbands,&line_inc,
                    &samp_inc,&one,&up_flg);
   if (status != E_SUCC)
       {
       strcpy(tbuf[0],lthos);
       strcpy(tbuf[1],hostmp[0]);
       strcpy(tbuf[2],hosout);
       ndel = 3;
       c_clean(vblock,"intersect",tbuf,&ndel);
       }

   /* Update the min/max
   ---------------------*/
   if (mskcnt != 0)
      minval[0] = 0;
   else
      minval[0] = 1;
   maxval[0] = maxout;
   strcpy(mnmx,"YES");
   if (c_svmnmx(hosout,&one,minval,maxval,mnmx) != E_SUCC)
      c_errmsg("Nonfatal error encountered","intersect-warn",&non_fatal);

   /* Close all image files
   -----------------------*/
   if (c_egclse(&gdesc) != E_SUCC)
      c_errmsg("Nonfatal error encountered","intersect-warn",&non_fatal);

   /* Delete the temp host file
   ----------------------------*/
   c_delfil(hostmp[0]);

   /* Free the buffer
   -----------------*/
   free(thebuf1);
   }

/* Free the buffer
-----------------*/
free(thebuf);

return;
}
