#include "isoclass.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"


/*****************************************************************************
 *
 * NAME		do_last_iteration
 *
 * PURPOSE	Handles the last iteration of assigning pixels to clusters.
 *		This includes determining minimum and maximum cluster numbers,
 *		determining the sum and squared sum of the pixels in each
 *		cluster, and determining the covariance matrix for each cluster.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *	Bani Allam / STX 	24-Jul-89       Freed allocated buffers
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clsumx [MAXCLS]
 *	       [MAXBND]	double		O	Sum of pixels in cluster
 *	clsumsq [MAXCLS]
 *	       [MAXBND]	double		O	Squared sum of pixels in cluster
 *	mean [MAXCLS]
 *	     [MAXBND]	double		I/O	Cluster mean for each band
 *	scov [MAXCLS]
 *	     [LOW_HALF]	double		O	Each cluster's covariance matrix
 *	*min_clust	double		I/O	Minimum cluster #
 *	*max_clust	double		I/O	Maximum cluster #
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	totbnd		short		I	Total # bands
 *
 * ALGORITHM DESCRIPTION
 *
 *	for each sample on current line
 *		if this sample's cluster is greater than max cluster
 *			set max cluster to this sample's cluster
 *		end if
 *		if this sample's cluster is less than min cluster
 *			set min cluster to this sample's cluster
 *		end if
 *		for each band
 *			use pixel value of sample and band to update
 *				sum, squared sum, mean, and covariance matrix
 *	    	end for
 *	end for
 *
 *****************************************************************************/
void do_last_iteration
(
    double *clsumx[],
    double *clsumsq[],
    float *mean[],
    double *scov[],
    double *min_clust,
    double *max_clust
)
{                                         
	short	*cluster_ptr,	/* pointer into cluster array */
		ic,		/* cluster of a particular sample */
		j,		/* pixel counter */
		k,		/* band counter */
		m;		/* pixel counter */
	float	xval,		/* value of a pixel */
	     	yval,		/* value of a pixel */
		*mean_k_ptr,	/* pointer into mean array offset by k */
		*mean_m_ptr,	/* pointer into mean array offset by m */
		xn;		/* intermediate covariance value */
	double	*clsumx_ptr,	/* pointer into clsumx array */
		*clsumsq_ptr,	/* pointer into clsumsq array */
		*scov_ptr,	/* pointer into scov array */
		prod;		/* intermediate covariance value */
	long	npx_val;	/* particular value in npx array */


	cluster_ptr = cluster;
	for (j = 0;  j < nsamples;  j++)  {
		/*
		 * get cluster assignment and
		 * increment it for file output
		 */
 		ic = (*cluster_ptr)++;
		if (ic+1 > *max_clust)
			*max_clust = ic+1;
		if (ic+1 < *min_clust)
			*min_clust = ic+1;
		npx_val = ++npx [ic];
		if (npx_val > 1) 
			xn = (npx_val - 1.0) / npx_val;
		else
			xn = 0.0;

		clsumx_ptr = clsumx [ic];
		clsumsq_ptr = clsumsq [ic];
		mean_k_ptr = mean [ic];
		scov_ptr = scov [ic];
		for (k = 0;  k < totbnd;  k++)  {
			xval = pixel_get (k, j);
			*clsumx_ptr += (double) xval;
			*clsumsq_ptr += (double) (xval) * xval;
			*mean_k_ptr = *clsumx_ptr / npx_val;

		     	/*
			 * update sums of products for covariance matrix
			 * (lower left triangle)
			 */
			mean_m_ptr = mean [ic];
			for (m = 0;  m <= k;  m++)  {
				yval = pixel_get (m, j);
				prod = (yval - *mean_m_ptr++) *
  				       (xval - *mean_k_ptr);
				*scov_ptr++ += prod * xn;
			}

			/* increment pointers into arrays */
			clsumx_ptr++;
			clsumsq_ptr++;
			mean_k_ptr++;
		}				/* 0 <= k < totbnd */
		cluster_ptr++;			/* point to next value */
	}					/* 0 <= j < np */
}


/*****************************************************************************
 *
 * NAME		do_iteration
 *
 * PURPOSE	Handles all but the last iteration of collecting stats on
 *		the pixels just recently assigned to clusters.  This includes
 *		determining the sum and squared sum for each cluster.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clsumx [MAXCLS]
 *	       [MAXBND]	double		O	Sum of pixel vals in each clust
 *	clsumsq [MAXCLS]
 *	       [MAXBND]	double		O	Squared sum - pix vals in clusts
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	totbnd		short		I	Total # bands
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	for each sample on current line
 *		figure out which cluster it belongs to
 *		for each band
 *			get pixel value
 *			update sum for this cluster and band
 *			update squared sum for this cluster and band
 *	       	end for
 *	end for
 *
 *****************************************************************************/
void do_iteration
(
    double *clsumx[],
    double *clsumsq[]
)
{
	short	*cluster_ptr,		/* pointer into cluster array */
		ic,			/* cluster that pixel belongs to */
		j,			/* pixel counter */
		k;			/* band counter */
	float	this_pix;		/* value of a particular pixel */
	double	*clsumx_ptr,		/* pointer into clsumx array */
		*clsumsq_ptr;		/* pointer into clsumsq array */


	cluster_ptr = cluster;
 	for (j = 0;  j < nsamples;  j++)  {
		/*
		 * get cluster assignment and
		 * increment it for file output
		 */
 		ic = (*cluster_ptr)++;
		clsumx_ptr = clsumx [ic];
		clsumsq_ptr = clsumsq [ic];
 		for (k = 0;  k < totbnd;  k++)  {
			this_pix = pixel_get (k, j);
			*clsumx_ptr++ += (double) this_pix;
 			*clsumsq_ptr++ += (double) (this_pix * this_pix);
 		}
		cluster_ptr++;			/* point to next element */
 	}
}                                                           
                                                         
                                 
/*****************************************************************************
 *
 * NAME		assign
 *
 * PURPOSE	To assign each sample (pixel) to one of the clusters based
 *		on minimum distance (city street).
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *	Blaine Ailts / EDC      28-jul-89       Removed some unused variables
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	iter		short		I	Current iteration #
 *	clsumx [MAXCLS]
 *	       [MAXBND]	double		O	Sum of pixel vals in each clust
 *	clsumsq [MAXCLS]
 *	       [MAXBND]	double		O	Squared sum - pix vals in clusts
 *	scov [MAXCLS]
 *	     [LOW_HALF]	double		O	Each cluster's covariance matrix
 *	*min_clust	double		I/O	Minimum cluster #
 *	*max_clust	double		I/O	Maximum cluster #
 *              
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clknt [MAXCLS]	long		O	# pixels in each cluster
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I	Means of clusters on bands
 *	gd_in		long		I	Group descriptor - input images
 *	gd_out		long		I	Group descriptor - output images
 *	maxit		short		I	Maximum # of iterations
 *	msgtime		long		I	Interval between messages
 *	nclust		short		I	Total # of clusters
 *	offset_out [1]	long		I	Offset into output image buffer
 *	window [4]	long		I	Dimensions of image window
 *
 * RETURN VALUES
 *
 *	Value		Meaning
 *	-----		-------
 *
 * ALGORITHM
 *
 *	if this is first iteration
 *		clear out pixel counting array
 *	clear out sum, sqauared sum, and pixel count arrays
 *	initialize informational messages to come at certain intervals
 *	set maximum cluster # to 0
 *	set minimum cluster # to total # of clusters
 *	for each line in image
 *		step into the line
 *		process (assign all of the pixels on) the line
 *		transfer cluster assignments to output image
 *		for each cluster
 *			add lines cluster counts to grand total cluster counts
 *		end for
 *		if this is the last iteration
 *		      	update sums, squared sums, means, covariance matrix,
 *				minimum cluster #, and maximum cluster #
 *		else
 *		    	update sums and squared sums
 *		end if
 *	end for
 *	step through images to update last line
 *	stop timing messages
 *	close input and output image files
 *
 *****************************************************************************/
void assign
(
    int iter,		/* iteration # */
    double *clsumx[],	/* sum pixel vals */
    double *clsumsq[],	/* square pixel vals */
    double *scov[],	/* covariance matrix */
    double *min_clust,			/* min cluster value */
    double *max_clust			/* max cluster value */
)
{
   	long		numlines, in,	/* scratch counter */
                        i, j, m, c_size,
		     	from_type,		/* conversion type for pxconv */
			in_clust [MAXCLS],	/* # pixels in each cluster */
			max_entity,		/* maximum entity (tm mesg) */
			max_category,		/* maximum category (tm mesg) */
			status,			/* return val of ext modules */
			to_type;		/* conversion type for pxconv */
	float		*mean [MAXCLS] ,	/* mean of all clusters */
                        *mean_matrix;
	extern long	curcat,			/* current category (tm mesg) */
			curent;			/* current entity (tm mesg) */


	nlines = window[0][NL];
	nsamples = window[0][NS];

        c_size = (long)(maxcls * totbnd * sizeof (float));
	mean_matrix = (float *) malloc (c_size);
        if (mean_matrix == NULL)
	  c_errmsg("Error allocating dynamic memory","isoclass-alloc",&one);
        for (i=0, m=0; i < maxcls; i++, m+= totbnd)
           mean[i] = &mean_matrix[m];

	/* clear npx array first time through */
	if (iter == 1)  {
		for (i=0; i < maxcls; i++)
			npx[i] = 0;
	}
		
	/*
	 * clear out cluster sums, initialize count of assignment change,
	 * initialize cluster counts
	 */
        for (i = 0; i < maxcls; i++) {
	  for(j = 0; j < totbnd; j++)  {
		clsumx[i][j]= (double) 0.0;
		clsumsq[i][j]= (double) 0.0;
          }
	}
	for (i=0; i < maxcls; i++)
		clknt[i] = 0;

	/* set up automatic messages */
	max_entity = nlines;
	max_category = maxit;
	curent = 1;
	curcat = iter;
	c_inittm ("ISOCLASS", "Assigned", "line", "iteration", " ",
		&max_entity, &max_category, &msgtime, &one);

	/* start processing the image */
	*max_clust = 0;
	*min_clust = nclust;

	/* start time interval messages */
	c_startm ();

	/* assign a line at a time */
        numlines = 1;
	while (numlines<= nlines)  {
		/* step through image */
		status = c_estep ((struct GDESC **)&gd_in); /* collect errors 
							       in status */
		status |= c_estep ((struct GDESC **)&gd_out);
		if (status != E_SUCC)  
			iso_error ("$", ""); 	/* fatal error */
		

		/* process one multi-image line */
		doline (clmean, cluster, in_clust);

		/* transfer cluster assignments to output image buffer */
		from_type = (sizeof (cluster [0]) <= 2) ? 2 : 3;
		to_type = 1;				/* 1 byte int */

		   /* status is not trapped from c_pxconv because the cluster
		      assignments can never exceed byte values */ 
		c_pxconv (&from_type, &to_type, (unsigned char *)cluster,
			buffer_out +offset_out[0], &nsamples);

                for (in=0; in < nsamples; in++)
                  buffer_out[offset_out[0]+in] = buffer_out[offset_out[0]+in]
                                                 + 1;

		/*
		 * update cluster counts by adding in_clust values to
		 * corresponding clknt values
		 */
		for (i=0; i < nclust; i++)
			clknt[i] += in_clust[i];

		/* compute CLUSTER SUms based on current assignments */
		if (iter == maxit)
			do_last_iteration (clsumx, clsumsq, mean, scov,
					   min_clust, max_clust);
		else
			do_iteration (clsumx, clsumsq);
                curent++; numlines++;
       	}

 	/* step through image one last time to finish writing */
	status = c_estep ((struct GDESC **)&gd_out);
	if (status != E_SUCC)  
		iso_error ("$", ""); 	/* fatal error */

	/* stop time interval messages */
	c_stoptm ();

	status = c_egclse ((struct GDESC **)&gd_in);  /* close input images */
	status |= c_egclse ((struct GDESC **)&gd_out);/* close output image */
	if (status != E_SUCC)
		iso_error ("$", ""); 	/* fatal error */
	free(mean_matrix);
}
