#include <math.h>
#include "isoclass.h"
#include "protos.h"

void clstats
(
    double *clsumx[],
    double *clsumsq[],
    int iter,
    double *scov[]
)
/*****************************************************************************
 *
 * NAME		clstats
 *
 * PURPOSE	Computes new cluster means and standard deviations and prints
 *		tables every krn'th iteration and the last iteration.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        08-31-88        Retrocode
 *      A.Shimitz / CSB         12-8-00         Modified so that if krn is 0,
 *                                              only print the tables on the
 *                                              first and last iteration
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clsumx [MAXCLS]
 *	       [MAXBND]	double		I	Sum of pixel values in clusters
 *	clsumsq [MAXCLS]
 *	       [MAXBND]	double		I	Squared sum of pixel values
 *	scov [MAXCLS]
 *	     [LOW_HALF]	double		I/O	Covariance matrix.
 *	iter		short		I	Current iteration #
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----	       	----		---	-----------
 *	clknt [MAXCLS]	long		I	# pixels in each cluster
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		O	Clusters means for each band
 *	clstdv [MAXCLS]
 *	       [MAXBND]	float		O	Clusters std devs for each band
 *	dtype_in	long		I	Data type of input image
 *	krn		short		I	Frequency to print tables at
 *	maxit		short		I	Maximum # of iterations
 *	nclust		short		I	# clusters currently in image
 *	print_file	FILE		I	Handle for the report file
 *	totbnd		short		I	# bands in input image
 *
 * ALGORITHM DESCRIPTION
 *
 *	for each cluster
 *		if it has more than one pixel in it
 *			for each band
 *				divide sum by pixel count to get mean
 *				figure out std dev based on sum, squared sum,
 *					and cluster count
 *			end for
 *		end if
 *	end for
 *	finish covariance matrix computations
 *	if this is a krn'th iteration or the last iteration then
 *		print a table giving the # of pixels in each cluster
 *		print a table giving the band means for each cluster
 *		print a table giving the band standard devs for each cluster
 *	end if
 *
 *****************************************************************************/
{
	char 	*print_string1,		/* scratch string for print control */
		*print_string2;		/* scratch string for print control */
	short	i,			/* cluster counter */
		j,			/* band counter */
		k,			/* band counter */
		upper;			/* top limit of band counter on line */
	double	x;			/* intermediate standard dev value */


	/* compute new cluster means and standard deviations */
	for (i = 0;  i < nclust;  i++)  {
		if (clknt [i] > 1)  {
			for (j = 0;  j < totbnd;  j++)  {
/* SLOW */		    	clmean [i] [j] = clsumx [i] [j] / clknt [i];
				x = clsumx [i] [j] * clsumx [i] [j];
				x = fabs (clknt [i] * clsumsq [i] [j] - x);
	      			x /= clknt [i] * (clknt [i] - 1.0);
	  			clstdv [i] [j] = sqrt (x);
			}
		} 
	}

	/* finish up covariance matrix computations */
	if (iter == maxit)
		finish_cov (scov);

	/* print tables every krn'th iteration and on last iteration - if krn
           is zero, just print tables the first and last iterations */
	if (((krn == 0) && (iter == 1)) ||
            ((krn != 0) && ((iter % krn) == 0)) ||
            (iter == maxit))
        {
		fprintf (print_file,
			 "\n\n CLUSTER   #PIXELS IN CLUSTER\n -------   ------------------\n");

		for (i = 0;  i < nclust;  i++)
			fprintf (print_file, "   %3d    %12d\n",
				 i + 1, clknt [i]);

		/* variable attributes of table */
		print_string1 = (dtype_in < 3) ? "  " : "    ";
	   	print_string2 = (dtype_in < 3) ? "%8.2f" : "%10.3e";

	 	/* print means */
		fprintf (print_file, "\n\nMEANS\n=====\n\n");

		for (k = 0;  k < totbnd;  k += PER_LINE)  {
			upper = (short)min(k + PER_LINE, totbnd);
			fprintf (print_file, " CLUSTER");
			for (j = k + 1;  j <= upper;  j++)
				fprintf (print_file, "%sBAND%2d",
					 print_string1, j);
			fprintf (print_file, "\n -------");
			for (j = k;  j < upper;  j++)
				fprintf (print_file, "%s------", print_string1);
			for (i = 0;  i < nclust;  i++)  {
				fprintf (print_file, "\n   %3d  ", i + 1);
 				for (j = k;  j < upper;  j++)
				fprintf (print_file, print_string2,
					 clmean [i] [j]);
			}
		        fprintf (print_file, "\n\n");
		}

		/* print standard deviations */
		fprintf (print_file,
			 "\n\n\nSTANDARD DEVIATION\n==================\n\n");

		for (k = 0;  k < totbnd;  k += PER_LINE)  {
			upper = (short)min(k + PER_LINE, totbnd);
			fprintf (print_file, " CLUSTER");
			for (j = k + 1;  j <= upper;  j++)
				fprintf (print_file, "%sBAND%2d",
					 print_string1, j);
			fprintf (print_file, "\n -------");
			for (j = k;  j < upper;  j++)
				fprintf (print_file, "%s------", print_string1);
			for (i = 0;  i < nclust;  i++)  {
				fprintf (print_file, "\n   %3d  ", i + 1);
				for (j = k;  j < upper;  j++)
				fprintf (print_file, print_string2,
					 clstdv [i] [j]);
			}
		fprintf (print_file, "\n\n\n");
		}
	}
}
