#include <math.h>
#include "isoclass.h"
#include "protos.h"

short	nc;			/* # clusters */
float	cdist [MAXCLS];		/* distance from one cluster to all others */


/*****************************************************************************
 *
 * NAME		clear_dist
 *
 * PURPOSE  	To clear the array dist so all element have the value 0.0 .
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *	Bani Allam / STX 	24-Jul-89       Freed allocated buffers
 *
 * ALGORITHM DESCRIPTION
 *	for each element in array
 *		set element to 0.0
 *	end for
 *
 *****************************************************************************/
void clear_dist
(void)
{
	short	i = MAXCLS;		/* cluster counter */
	float	*cursor = cdist;	/* cursor into cdist array */

	while (i--)
		*cursor++ = 0.0;
}


/*****************************************************************************
 *
 * NAME		fig_dist
 *
 * PURPOSE	Figures out the `distance' between clusters i and j and places
 *		the result in dist [j - 1].
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	i		short		I	Cluster #
 *	j		short		I	Cluster #
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I	Cluster means for each band
 *	clstdv [MAXCLS]
 *	       [MAXBND]	float		I	Cluster std devs for each band
 *	sep		float		I	Split separation value
 *	totbnd		short		I	Total # of bands in image
 *
 * ALGORITHM DESCRIPTION
 *
 *	for each band
 *		set numerator to square of differences of the cluster means
 *		if sep is 0.0 then
 *			set denominator to the product of the cluster std devs
 *			if the denominator is 0.0 then
 *				set the denominator to 0.0001
 *			end if
 *		else
 *			set denominator to SSEPVAL squared
 *		end if
 *		add the square root of the numerator divided by the denominator
 *		    to the accumulated distance
 *	end for
 *
 *****************************************************************************/
void fig_dist
(
    int i,
    int j
)
{
	short	k;		/* band counter */
	float	*cdist_ptr,	/* pointer into dist array */
		*clmean_cur_i,	/* cursor equivalent to clmean [i] */
		*clmean_cur_j,	/* cursor equivalent to clmean [j] */
		*clstdv_cur_i,	/* cursor equivalent to clstdv [i] */
		*clstdv_cur_j,	/* cursor equivalent to clstdv [j] */
		denom,		/* denominator in distance calculation */
		num;		/* numerator in distance calculation */


	cdist_ptr = &(cdist [j - 1]);
	clmean_cur_i = clmean [i];
	clmean_cur_j = clmean [j];
	clstdv_cur_i = clstdv [i];
	clstdv_cur_j = clstdv [j];
                               
	for (k = 0;  k < totbnd;  k++)  {
		num = *clmean_cur_i++ - *clmean_cur_j++;
		num *= num;				/* squared */
		if (sep == 0.0)  {
			denom = *clstdv_cur_i * *clstdv_cur_j;
			if (denom == 0.0)
				denom = 0.0001;
		}  else
			denom = sep * sep;
		*cdist_ptr += (num / denom);
		clstdv_cur_i++;
	 	clstdv_cur_j++;
	}
	*cdist_ptr = sqrt (*cdist_ptr);
}


/*****************************************************************************
 *
 * NAME		close_enuf
 *
 * PURPOSE	Determine if requirements for combination of clusters are met.
 *		This means that the distance between i and j is less than the
 *		distance between any other cluster k, where j < k < nc.  Also,
 *		the distance must be less than dlmin.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	j		short		I	???
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clknt [MAXCLS]	long		I	# pixels in each cluster
 *	dlmin		float		I	Distance to combine clusters at
 *
 * RETURN VALUES
 *
 *	Whether the two clusters are close enough (boolean).
 *
 * ALGORITHM DESCRIPTION
 *
 *	if the distance is less than dlmin
 *		for each cluster beyond j
 *			if this clusters distance is less than j's and
 *			    this cluster has some members
 *				return FALSE
 *		       	end if
 *		end for
 *		return TRUE
 *	else
 *		return FALSE
 *	end if
 *
 *****************************************************************************/
short close_enuf
(
    int j
)
{
	short	k;				 	/* cluster index */
	long	*clknt_cur	= &clknt [j + 1];	/* cursor into clknt */
	float	*dist_cur	= &cdist [j],		/* cursor into cdist */
		dist_val	= cdist [j - 1];	/* a value in cdist */


	if (dist_val < dlmin)  {
		for (k = j;  k < nc - 1;  k++)
			if ((*dist_cur++ < dist_val) && (*clknt_cur++ != 0))
			       	return (FALSE);
		return (TRUE);
	}  else
		return (FALSE);
}
                               

/*****************************************************************************
 *
 * NAME	    	combine
 *
 * PURPOSE	To combine clusters that are close enough in terms of
 *		'distance'.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *      Blaine Ailts / EDC	28-jul-89       Fixed pointers -- now works
 *						correctly up to MAXBND and
 *						MAXCLASS
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----  		----		---	-----------
 *	clknt [MAXCLS]	long		I/O	# pixels in each cluster
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I/O	Cluster means for each band
 *	nclust		short		I/O	Total # of clusters so far
 *	totbnd		short		I	Total # bands in input images
 *
 * ALGORITHM DESCRIPTION
 *
 *	for each cluster except last
 *		clear distance
 *		if this cluster has some members
 *			for each cluster after this one
 *				figure distance between these two clusters
 *			end for
 *			for each cluster after this one
 *				if these clusters are close enough then
 *					save information
 *					break out of for loop
 *			       	end if
 *			if mergeable clusters found then
 *				merge clusters' means to create new mean
 *				copy this new mean into buffer
 *				mark both original clusters as having no members
 *			else
 *				copy cluster's mean into buffer
 *	if last cluster has not been merged
 *		copy its mean into buffer
 *	copy buffer into means array
 *
 *****************************************************************************/
void combine
(void)
{
	float	*hclmean [MAXCLS],	/* holding array - new clmean */
		*hclmean_matrix;
	long	c_size;
	short	i,			/* cluster index */
		ic,			/* cluster to combine w/ i */
		index = 0,		/* index into hclmean (new means) */
		j,			/* cluster index (above i) */
		k,			/* image index */
		minimum;		/* bool : w/i min distance to combine */


   c_size = (long)(maxcls * totbnd * sizeof (float));
   if ((hclmean_matrix = (float *) malloc( c_size)) == NULL) 
      iso_error ("!Error allocating dynamic memory", "isoclass-alloc");
   
   for (i = 0, j = 0; i < maxcls; i++, j += totbnd)
      hclmean[i] = &hclmean_matrix[j];

	nc = nclust;

	/* compute distances of all clusters to all other clusters */
	for (i = 0;  i < (nc - 1);  i++)  {
		clear_dist ();				/* clear dist array */

/* SLOW!!! */  	if (clknt [i] != 0)  {
			/* find distance from cluster i to clusters j (j=i+1,nclust) */
			for (j = i + 1;  j < nc;  j++)
/* SLOW!!! */			if (clknt [j] != 0)
					fig_dist (i, j);

			/*
			 * at this point have distances from cluster i to
			 * clusters j where (i < j < nclust)
			 */
			for (j = i + 1;  j < nc;  j++)
/* SLOW!!! */		    	if (clknt [j] != 0)  {
					minimum = close_enuf (j);
					if (minimum)  {
						ic = j;
				 		break;
					}
				}

			/*
			 * If combinations requirements are met then create a
			 * new mean for the new cluster in the holding area.
			 * Indicate that cluster j has zero members.
			 * If combination requirements are not met then simply
			 * copy the the statistics for cluster i to the
			 * holding area.
			 */
			if (minimum)  {
				nclust--;
				for (k = 0;  k < totbnd;  k++)  {
/* SLOW!!! */				hclmean [index] [k] =
					   ((float)clknt [i] * clmean [i] [k] +
					   (float)clknt [ic] * clmean [ic] [k])/
				           ((float)clknt[i] + (float)clknt[ic]);
				}
				clknt [i] = 0;
				clknt [ic] = 0;
			}  else
				buff_copy ((char *)clmean [i],
					   (char *)hclmean [index],
					   totbnd,
					   (short) sizeof (float));
			index++;
		}				/* if (clknt(i) != 0) */
	}					/* 0 <= i < nc - 1 */

	/*
	 * Copy last cluster data if cluster has not been combined already
	 */
	if (clknt [nc - 1] != 0)  {
		buff_copy ((char *)clmean [nc - 1], (char *)hclmean [index],
			   totbnd, (short) sizeof (float));
		index++;
	}

	/*
	 * Copy updated holding area back to original means
	 */
	buff_copy ((char *)hclmean[0], (char *)clmean[0], totbnd * maxcls,
		   (short) sizeof (float));
	free (hclmean_matrix);
}
