#include "isoclass.h"
#include "protos.h"


/*****************************************************************************
 *
 * NAME		abs_sum
 *
 * PURPOSE	To sum up all of the components (band-wise) of a vector.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *	
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	*vector		float		I	First element of vector
 *	length		short		I	# of components in vector
 *	increment	short		I	Distance between components
 *	
 *
 * RETURN VALUES
 *
 *	sum (float) of the absolute value of the components
 *
 * ALGORITHM
 *
 *	for each component
 *		add the absolute value of the component to a running sum
 *		point to the next component
 *	end for
 *
 *****************************************************************************/
float abs_sum
(
    float *vector,
    int length,
    int increment
)
{
	float	sum = 0.0;	/* sum of all vectors */

	while (length--)  {
		sum += (*vector < 0) ? -*vector : *vector;	/* abs */
		vector += increment;
	}

	return (sum);
}


/*****************************************************************************
 *
 * NAME		offset_pix
 *
 * PURPOSE	Offsets all of the pixels in one band by a certain amount.
 *		Used to add a constant vector to all of the pixels in a line,
 *		band by band.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	count		short		I	# samples to offset
 *	*destination	float		I/O	Destination array
 *	offset		float		I	Offset for each pixel
 *
 * ALGORITHM
 *
 *	for each sample
 *		set destination to offset added to the value of the sample
 *	end for
 *
 *****************************************************************************/
void offset_pix
(
    float *destination,
    double offset,
    int count
)
{
	short	sample = 0;		/* pixel counter */

	while (sample++ < count)
		*destination++ = offset + pixel_next ();
}
                                   
              
/*****************************************************************************
 *
 * NAME		doline
 *
 * PURPOSE	Assigns each pixel in a line to the cluster with the minimum
 *		city street distance.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *	B. Ailts/EDC  		11-30-90	Fixed using images that had
 *						nbands * ns >  32767
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I	Cluster means for each band
 *	*cluster 	short		O	Cluster # each pixel is assigned
 *	in_clust
 *	       [MAXCLS]	long		O	# pixels assigned to each clust
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----	      	----		---	-----------
 *	*difference	float		I	Points array of vect differences
 *	*dist		float		I	Points array of clust distances
 *	nclust		short		I	# of clusters currently in image
 *	totbnd		short		I	Total # bands in input image
 *	window[][4]	long		I	Dimension of images
 *
 * ALGORITHM
 *
 *	clear out in_clust array
 *	for each cluster
 *		for each band
 *		       	offset each pixel by the cluster mean
 *		end for
 *		add up all of the offsets to get distance
 *	end for
 *	for each sample
 *		start with first cluster as minimum
 *		for each cluster after first
 *			if distance is less than current minimum
 *				set this cluster as current minimum
 *			end if
 *		end for
 *		show this pixel belonging to this minimum cluster
 *		increment pixel count for minimum cluster
 *	end for
 *
 *****************************************************************************/
void doline
(
    float *clmean[],
    short cluster[],
    long in_clust[MAXCLS]
)
{
	short		*cluster_ptr;	/* pointer into cluster array */
	long		i,		/* general counter */
			imin, 		/* cluster w/ minimum distance */
	 		into_diff,	/* offset into difference */
			j, 		/* band counter */
			k,		/* cluster counter */
			nsamples = window[0][NS];	/* # samples per line */
	long 		*icursor;    	/* scratch int cursor */
	float		*clmean_ptr,	/* pointer into clmean array */
			*dist_ptr,	/* pointer into dist */
			*fcursor,	/* scratch float cursor */
			mindist,	/* min distance from cluster to a pix */
			offset;		/* offset for pxadd */
	extern float	*difference,	/* difference vectors 'array' */
			*dist;		/* distances 'array' */


	/* clear out in_clust array */
	icursor = in_clust;
	i = MAXCLS;
	while (i--)
		*icursor++ = 0;

	for (k = 0;  k < nclust;  k++)  {
		/* compute difference vectors */
		clmean_ptr = clmean [k];

		/* reset pixel pointer to beginning of line */
		pixel_line_reset ();

		for (j = 0;  j < totbnd;  j++)  {
			/* amount to offset each pixel value */
                        offset = *clmean_ptr;
                        clmean_ptr++;
			offset = - offset;

			/* offset into difference */
			into_diff = nsamples * j;

			/* update band */
			pixel_band_next ();

			/* add them all up */
			offset_pix (difference + into_diff, offset,
				    nsamples);
		}

		/* compute sum of absolute values */
		dist_ptr = dist + (k * nsamples);
		fcursor = difference;
		i = nsamples;
		while (i--)
			*dist_ptr++ = abs_sum (fcursor++, totbnd, nsamples);
	}

	/* find the class with minimum value */
	cluster_ptr = cluster;
	for (j = 0;  j < nsamples;  j++)  {
		mindist = *(dist + j);
		imin = 0;
		for (k = 1;  k < nclust;  k++)  {
			dist_ptr = dist + nsamples * k + j;
			if (*dist_ptr < mindist)  {
 				mindist = *dist_ptr;
			 	imin = k;
		 	}
		}
	 	in_clust [imin]++;
	 	*cluster_ptr++ = (short)imin;
 	}
}
