#include "isoclass.h"
#include "util.h"
#include "protos.h"    

/*****************************************************************************
 *
 * NAME		iosetup
 *
 * PURPOSE	To open and group the files for the input and output images.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *      Bani Allam / STX 	24-Jul-89	Status of C_EALLOC checked
 *						against NULL
 *      D. VanderZee 		May-90		Cleaned up error messages
 *	B. Ailts/EDC  		11-30-90	Fixed inputing multiple images
 *	
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	iter		short		I	Current iteration #
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name	       	Type		I/O	Description
 *	----		----		---	-----------
 *	bands [MAXIMG]
 *	      [MAXBND]	long		I	Selected band #'s
 *	*buffer_in	unsigned char	I	Holds lines of input images
 *	*buffer_out	unsigned char	I	Holds lines of output image
 *	dtype_in	long		O	Data type of input images
 *	dtype_in_size			O	Size of input image in bytes
 *	dtype_out	long		O	Data type of output image
 *	fd_in [MAXIMG]	long		O	File descriptors - input images
 *	fd_out [2]	long		O	File descriptor - output image
 *	gd_in	 	long		O	Group descriptor - input images
 *	gd_out	 	long		O	Group descriptor - output image
 *	host_in		char    	I	Host version - input filenames
 *	host_out	char    	I	Host version - output filename
 *	nimages		long		I	# of input image files
 *	offset_in
 *	       [MAXIMG]	long		I	Line offsets into input images
 *	offset_out[1] 	long   		I	Line offsets into output image
 *	window[][4]	long		I	Dimensions of images
 *
 * ALGORITHM
 *
 *	if this isn't the setup iteration (no input statistics file)
 *		open the output image file
 *		allocate buffer space for line data
 *		put the single file into a group
 *	end if
 *	for each input image
 *		open the input image file
 *		step through to the next filename in the descriptor
 *	end for
 *	terminate the file descriptor array with a zero
 *	allocate buffer space for line data
 *	put all of the input files into a group
 *	get the data type of the input data
 *
 *****************************************************************************/
void iosetup
(
    int iter,
    long bands[MAXIMG][MAXBND+1]	/* selected band #'s */
)
{
    	short		i;		/* image counter */
	long		access,		/* file access */
			band = 0,
			buff_size,	/* size of buffer allocated */
			no_opt [3],	/* optional parameters */
			one_band [2],	/* output image is only 1 band */
			status,		/* return value of external modules */
			zero_nbuf = 0;	/* default # lines to remember */


	/* general to opening input and output images */
	no_opt[0] = no_opt[2] = 0;
	no_opt[1] = NO_COMPRES;

	/*
	 * Don't open output image on setup iteration, only on assign,
	 * split, and combine iterations.
	 */
	if (iter > 0)  {
		one_band[0] = 1;
		one_band[1] = 0;
		access = (iter == 1) ? IWRITE : IUPDATE;
		dtype_out = EBYTE;
		status = c_eopens ((struct FDESC **)&fd_out[0], host_out, 
				 one_band, &one, &one, &window[0][NL],
				 &window[0][NS],offset_out, &access,
				 &dtype_out, &zero_nbuf, no_opt);
		if (status != E_SUCC)
	 	  iso_error("!Error opening output image file","isoclass-open");
		fd_out [1] = 0;				/* terminate array */
                if (iter == 1){
                  strcpy(out[count], host_out);
                  count = count +1;
                }

		/* figure out buffer size that is needed */
		buffer_out = (unsigned char *)c_ealloc ((struct FDESC **)fd_out,
                               &buff_size);
		if (buffer_out == (UTINY *) NULL)
	            iso_error("$",""); 	/* fatal error */

		/*
		 * Group all input image files together to be treated as one
		 * unit.
		 */
		status = c_egroup ((struct FDESC **)fd_out, 
				    (struct GDESC **)&gd_out, buffer_out,
                                     &buff_size);
		if (status != E_SUCC)
			iso_error ("$", ""); 	/* fatal error */
	}

	/* open all input image files and place file descriptors in array */
	access = IREAD;
	for (i = 0;  i < nimages;  i++)  {
	   status = c_eopens ((struct FDESC **)&fd_in[i], host_in[i], bands[i],
			 &window[i][SL], &window[i][SS], &window[i][NL], 
			 &window[i][NS], &offset_in[band], &access, &dtype_in,
 			 &zero_nbuf, no_opt);
    	   if (status != E_SUCC)  
	 	iso_error("!Error opening input image file","isoclass-open");
           band += nbands[i];
		
	}
	fd_in [i] = 0;					/* terminate array */

	/* figure out buffer size that is needed */
	buffer_in=(unsigned char *)c_ealloc((struct FDESC **)fd_in,&buff_size);
	if (buffer_in == (UTINY *) NULL)
	   iso_error("$",""); 	/* fatal error */

	/* group all input image files together to be treated as one unit */
	status=c_egroup((struct FDESC **)fd_in,(struct GDESC **)&gd_in, 
			buffer_in, &buff_size);
	if (status != E_SUCC)
		iso_error ("$", ""); 	/* fatal error */

	/* get data type of input images */
	status = c_gettyp (host_in, &nimages, &dtype_in);
	if (status != E_SUCC)
   	   iso_error ("$",""); 	/* fatal error */
	dtype_in_size = bytes_per_pixel (dtype_in);
}
