#include <ctype.h>
#include "las.h"
#include "statio.h"
#include "vars.h"			/* declare all global variables */
#include "util.h"
#include "protos.h"

/* 
 * Global to doline.c
 */
float     *difference,
          *dist;

/*****************************************************************************
 *
 * NAME		isoclass (main)
 *
 * PURPOSE	To control the process of determining the clusters of an
 *		image.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *      K. Narayanan            21-Mar-89       Modified split.c 
 *      Bani Allam / STX        24-Jul-89       Freed allocated buffers
 *	Blaine Ailts/EDC	27-Jul-89	Fixed a problem with pointers --
 *						Now will handle images up to
 *						MAXBND with MAXCLASS clusters
 *						correctly
 *						Freed imageio buffers between
 *						interations
 *	D. VanderZee/EDC	2-14-90	 	Since timelines did not permit
 *						a complete overhaul of this 
 *						module, only the following
 *						blatant	bugs were fixed:
 * 	  					--distance calculations for 
 *						  both chaining and combining
 *						  clusters were corrected
 *						--minmax values for classified 
 *						  image ddr were corrected
 *						--initialized covariance arrays
 *						  so results on suns are ok
 *						--changed print file so that
 *						  parameter names match pdf
 *						--print option for "terminal"
 * 						  was fixed to open print file
 *						--initial statistics can now 
 *						  be read from an input stats
 *						  file
 *	B. Ailts/EDC  		11-30-90	Added error checks for malloc 
 *						fixed inputing multiple images
 *						fixed using images that had
 *						nbands * ns >  32767
 *	R. Kruse/EDC,CSB	06-15-92	--MINNUM changed to MINCLUST.
 *						Place correct module name in 
 *						.hst file.
 *						--Print file contains host 
 *						name - should be LAS name.
 *						--Output all files created when
 *						finished.
 *						--Modify isoclass.h to have 
 *						extern declarations of global
 *						vars. declared in isoclass.c
 *	D. Hames/EDC		12-22-99	Remove MAXNS from variable
 * 					        cluster and allocate using 
 *						window[0][NS]. Modified to 
 *						support images > 2gig. 
 *						Add prototypes. SRF #893.
 *	
 *
 * ALGORITHM DESCRIPTION
 *
 *	initialize LAS
 *	read in the LAS parameters
 *	if input statistics come from a file
 *		read the input statistics
 *	else
 *		set up the image files
 *	      	determine the statistics for the whole image
 *	end if
 *	for each iteration
 *		set up the image files
 *		output the current # of clusters
 *		recompute the cluster means and standard deviations
 *		if interupt flag is set then
 *			break out of iteration loop
 *		end if
 *		if this is a view'th iteration and this is not the last one then
 *			repeat
 *				prompt user if they would like to quit
 *				read in answer
 *				convert answer to upper case
 *				if first character of answer is 'Y' then
 *					set interupt flag
 *				else if answer is not 'N' then
 *		      			inform user of incorrect response
 *				end if
 *			until user answers properly
 *		end if
 *		if this is not the last iteration
 *			delete any clusters too close to one another
 *			decide whether to split or combine
 *			if decided to split
 *				split
 *			else
 *				combine
 *			end if
 *		end if
 *	end for
 *	chain clusters that are close enough
 *	write the output statistics file
 *	save min/max information if flag set
 *	update history
 *	close the report file
 *	free up allocated memory
 *
 *****************************************************************************/

void main
(void)
{
	char		itertyp = NO_ITER;	/* split or combine tag */
        double          *scov[MAXCLS];
	unsigned char	answer;		/* 1st character of user's response */
	short 		alter = FALSE,	/* bool : alternating split/combine? */
			interupt = FALSE,	/* bool : interupt processing */
			iter = 0;	/* iteration count */
        long            dtype = ELONG, 
			one = 1,	/* integer one value */
			status,		/* return value of external modules */
			stdlun,		/* logical unit # */
                        lines, col, type;
        static long    	bands [MAXIMG][MAXBND+1];	/* selected band #'s */
	double		*clsumsq [MAXCLS],	/* sum pixel^2 values */
			*clsumx [MAXCLS],	/* sum pixel values */
                        y_inc,
                        x_inc,
			max_clust,	/* maximum cluster value */
      	   		min_clust;	/* minimum cluster value */
	char		outbuf [76],	/* temp buff - screen output */
	                print_filename[CMLEN],  /* output file name  */
			temp_str [76];	/* holds response */
	long		las_status,	/* status of las call */	
			termination;	/* termination char of user input */

	/* initialize LAS */
	c_inlas (&vblock, &stdlun);
        t_init (&lines, &col, &type);

	/* get and error check parameters */
	getpar (print_filename, bands);

        /* allocate memory for cluster */  
        if ((cluster = (short *) malloc (window[0][NS] * sizeof(short)))==NULL)
              iso_error ("!Error allocating dynamic memory", "isoclass-alloc");

        /*allocate dynamic storage for covariance matrices 
          and get pointers to their rows*/
        dyn_alloc( scov, clsumx, clsumsq);

	/* allocate arrays for doline */
	difference = (float *) malloc (window[0][NS] * totbnd * sizeof (float));
	if (difference == NULL)
	  c_errmsg("error allocating dynamic memory","isoclass-alloc",&one);
	dist = (float *) malloc (maxcls * window[0][NS] * sizeof (float));
	if (dist == NULL)
	  c_errmsg("error allocating dynamic memory","isoclass-alloc",&one);

	/* read statistics file if available /else/ compute initial means */
	if (srcstat [0] == 'F')
		rdstats ();
	else  {
		/* set up file specifications */
		iosetup (0, bands);

		/*  compute initial cluster means */
		setup (clmean);
 
	}

        /* initialize min and max values */
        status = c_inmnmx(&dtype, &one, &min_clust, &max_clust);
	if (status != E_SUCC)
		iso_error ("$", "");  /* fatal error */

	/* start iterations */
	fprintf (print_file, "\n");
	for (iter = 1;  iter <= maxit;  iter++)  {
		iosetup (iter, bands);
		assign (iter, clsumx, clsumsq, scov, &min_clust, &max_clust);

		/* recompute cluster means and standard deviations */
		fprintf (print_file,
			 "Number of clusters after iteration #%3d: %4d\n",
			 iter, nclust);
		clstats (clsumx, clsumsq, iter, scov);
		sprintf (outbuf, "Iteration # %d : Clusters = %d\n",
			 iter, nclust);
		t_write (outbuf, T_STDCC);
		free (buffer_out);

		free (buffer_in);
		/*
		 * if viewth iteration then
		 * ask if user would like to stop here
		 */
		if (((iter % view) == 0) && (iter != maxit))
		   do  {
		        /* prompt user */
 t_write ("Would you like to stop iterations at this point (Y/N) ?", T_PROMPT);
			/* read in response */
			las_status = t_read (temp_str, (int *)&termination);
			if (las_status != SUCCESS)
			   iso_error ("?Error reading keyboard response",
				"isoclass-keybd"); 
			answer = temp_str [0];
			if (islower (answer))
				answer = toupper (answer);
			t_write ("", T_STDCC);

			/* analyze response */
			if (answer == 'Y')
				interupt = TRUE;
			else if (answer != 'N')
				t_write ("Incorrect response, try again.",
					 T_DOUBLE);
		   }  while ((answer != 'Y') && (answer != 'N'));

		/* if interupt then break out of for loop */
		if (interupt)
			break;

		/*
		 * If not on maximum iteration, prepare for next iteration
		 * by deleting any clusters with fewer than required members,
		 * decide on what type of iteration is required, then perform
		 * either a split or combine operation on the cluster means.
		 */
		if (iter != maxit)  {
			isodel ();
			decide (&itertyp, iter, &alter);
			if (itertyp == SPLIT)
				split ();
			else
				combine ();
		}
	}						/* 1 <= i <= maxit */

	/* examine clusters to see if any may be chained */
  	chain ();

	/* write statistics file */
	wrstats (scov, clmean);


	status = c_hstout (&vblock, host_out, host_in, &nimages, "ISOCLASS");
	if (status != E_SUCC)
	  iso_error ("?Error writing to output history file", "isoclass-write");

        /*
	update DDR
	----------
	*/
	y_inc = 1;
	x_inc = 1;
	status = c_upddr(&vblock, host_in, host_out, &nimages, window, bands, 
			nbands, &y_inc, &x_inc, &one, &one);
	if (status != E_SUCC)
		iso_error ("$",""); 	/* fatal error */

	/* update minimum and maximum statistics */
        if (do_minmax) {
	   status = c_svmnmx (host_out, &one, &min_clust, &max_clust, "YES");
  	   if (status != E_SUCC)
	     iso_error ("$", "");  	/* fatal error */
        };

	/* close printing file */
 	fclose (print_file);

	/* free up allocated space */
        free (scov_matrix);
        free (clsumx_matrix);
        free (clsumsq_matrix);
        free (clstdv_matrix);
        free (clmean_matrix);
	free (difference);
	free (dist);
        free (cluster);

	/* output completion message */
	c_complt ("ISOCLASS", out_las, &count_las);

}
