#include "isoclass.h"
#include "protos.h"

static char	*pix_ptr;	/* points into buffer_in to current sample*/
static short	pix_band;	/* current band # being pointed to */


/*****************************************************************************
 *
 * NAME		pixel_line_reset
 *
 * PURPOSE	To reset all static data when processing of a new line begins.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	None
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	set current band to -1
 *	set pixel pointer to NULL
 *
 *****************************************************************************/
void pixel_line_reset
(void)
{
	/* update band and pointer */
	pix_band = -1;
	pix_ptr = NULL;
}


/*****************************************************************************
 *
 * NAME	    	pixel_band_next
 *
 * PURPOSE	To start processing the next band of a line.  This must
 *		be called before the very first line is processes too.
 *		Pixel_line_reset should be called at the appropriate time to
 *		set up this call.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----	     	----		---	-----------
 *	*buffer_in	unsigned char	I	Holds lines of input images
 *	dtype_in_size	short		I	# bytes per pixel
 * 	offset_in
 *	       [MAXIMG]	long		I	Offsets into buffer_in
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	increment the current band number
 *	calculate the offset into the buffer
 *	set the pointer to the address of the offset into the buffer
 *
 *****************************************************************************/
void pixel_band_next
(void)
{
	long	pix_offset;			/* offset into buffer_in */


	/* update band and pointer */
	pix_band++;
	pix_offset = offset_in [pix_band];
	pix_ptr = (char *)buffer_in + pix_offset * dtype_in_size;
}


/*****************************************************************************
 *
 * NAME		pixel_next
 *
 * PURPOSE	Returns the value of the next pixel.  This must be called to
 *		process the very first pixel of a line or band too.
 *		Pixel_band_next and pixel_line_reset should be called at the
 *		appropriate times to set up this call.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	dtype_in	long		I	Data type of input images
 *
 * RETURN VALUES
 *
 *	Value of the pixel (float)
 *
 * ALGORITHM
 *
 *	depending on the type of data
 *		set pointer of proper type to value of general pointer
 *		increment general pointer by size of data
 *		return what is pointed to in the form of a float
 *	end depend
 *
 *****************************************************************************/
float pixel_next
(void)
{
 	unsigned char	*byte_ptr;		/* pointer to byte data type */
	short		*word_ptr;		/* pointer to word data type */
	long		*long_ptr;		/* pointer to long data type */
	float  		*real_ptr;     		/* pointer to real data type */


	/* depending on type and size of data, update pointer and return data */
	switch (dtype_in)  {
		case EBYTE:
			byte_ptr = (unsigned char *) pix_ptr;
			pix_ptr += BYTE_SIZE;
	 		return ((float) *byte_ptr);
		case EWORD:
		   	word_ptr = (short *) pix_ptr;
			pix_ptr += WORD_SIZE;
	 		return ((float) *word_ptr);
		case ELONG:
			long_ptr = (long *) pix_ptr;
			pix_ptr += LONG_SIZE;
	 		return ((float) *long_ptr);
		case EREAL:
			real_ptr = (float *) pix_ptr;
			pix_ptr += REAL_SIZE;
	 		return ((float) *real_ptr);
	}
	return(-1); /* This return was put in to avoid a compiler error.
		       It it is not used as an error code. */
}


/*****************************************************************************
 *
 * NAME		pixel_get
 *
 * PURPOSE	Returns the value of a given pixel (sample and band) on the
 *		current line.  Does not require any prior setup. 
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	band		short		I	Band on which pixel lies
 *	sample		short		I	Which pixel to get value of
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----	 	----		---	-----------
 *	*buffer_in	unsigned char	I	Holds lines of input images
 *	dtype_in	long		I	Data type of input images
 * 	offset_in
 *	       [MAXIMG]	long		I	Offsets into buffer_in
 *
 * RETURN VALUES
 *
 *	Value of pixel (float) at the given band and sample.
 *
 * ALGORITHM
 *
 *	calculate index based on band and sample
 *	depending on data type
 *		set a pointer equal to the address of buffer offset by the index
 *		return the float value of what the pointer points at
 *	end depend
 *
 *****************************************************************************/
float pixel_get
(
    int band,
    int sample
)
{
 	unsigned char	*byte_ptr;		/* pointer to byte data type */
	short		*word_ptr;		/* pointer to word data type */
	long		index,			/* index into image buffer */
			*long_ptr;		/* pointer to long data type */
	float  		*real_ptr;		/* pointer to real data type */


	/* figure index into buffer */                     
	index = offset_in [band] + sample;

	/* depending on type and size of data, figure out address of data */
	switch (dtype_in)  {
		case EBYTE:
			byte_ptr = &(buffer_in [index * BYTE_SIZE]);
			return ((float) *byte_ptr);
		case EWORD:
			word_ptr = (short *)&(buffer_in [index * WORD_SIZE]);
			return ((float) *word_ptr);
		case ELONG:
			long_ptr = (long *)&(buffer_in [index * LONG_SIZE]);
			return ((float) *long_ptr);
		case EREAL:
			real_ptr = (float *)&(buffer_in [index * REAL_SIZE]);
			return ((float) *real_ptr);
	}
	return(-1); /* This return was put in to avoid a compiler error.
		       It it is not used as an error code. */
}
