#include "isoclass.h"
#include "protos.h"

/*****************************************************************************
 *
 * NAME		rdstats
 *
 * PURPOSE	If input statitistics come from a file, then this routine
 *		reads them in and outputs the relevant data to the report file.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam              31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----  	       	----		---	-----------
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I/O	Cluster means for each band
 *	host_in		char    	I	Host version - input filenames
 *	nclust		short		I/O	# clusters defined in stats file
 *	print_file	long		I	Handle to the report file
 *	stat_host_in	char    	I	Host version - input stat file
 *	totbnd		short		I	Total # of bands in input images
 *
 * RETURN VALUES
 *                                                                             
 *	None
 *
 * ALGORITHM
 *
 *	get size of input image pixels
 *	open input statistics file
 *	for each cluster
 *		get the cluster name
 *		output the cluster name to the report file
 *		get the cluster mean vector
 *		check for consistency
 *		copy means read from file to program variable
 *		output the cluster mean vector to the report file
 *	end for
 *	close the input statistics file
 *
 *****************************************************************************/
void rdstats
(void)
{
	char		*print_string;	/* string controls dat file output */
	unsigned char	cname [MAXCLS] [80];	/* buffer for cluster names */
	short		dtype_in,	/* data type of input image */
	     		i;		/* general counter */
	long		access = 0,	/* want to read existing stat file */
			tmp_nclust,     /* temp var for calling stopen */
			ic,		/* class index */
			is = 0,		/* site index */
			file_ic,	/* class index for file (1 + ic) */
			nmshape [9],	/* shape vector for cluster names */
  			shape [9],	/* shape vector for cluster means */
			status;		/* return value of external modules */
static 	double		filmean [MAXCLS] [MAXBND];	/* buff - file means */
	long		stat_fd;	/* file descriptor of stats file */


	/* get input data type */
	status = c_getddr ((char *)host_in, &ddr);
	if (status != E_SUCC) 
	  iso_error ("$", ""); 	/* fatal error */
       

	/* figure out the number of bytes per pixel */
	dtype_in = bytes_per_pixel (ddr.dtype);

	/* open stats file and check for errors */
	status = c_stopen ((FILE **)&stat_fd,stat_host_in,&access,&tmp_nclust);
	if (status == E_GNLIO) 
	   iso_error ("!Error opening input statistics file", "isoclass-open");                               

	nclust = (short)tmp_nclust;
 	if (nclust < 1) {
	   iso_error ("!Error: Input statistics file contains no classes", 
		"isoclass-nocls");
         }

	/* get class (cluster) names and mean vectors */
	fprintf (print_file, "\n\nINITIAL CLUSTER CENTROIDS AS READ FROM STATISTICS FILE \n");
	for (ic = 0;  ic < nclust;  ic++)  {
		file_ic = ic + 1;		/* file starts @ 1, array @ 0 */

		status = c_stget ((FILE **)&stat_fd, "NAME", &file_ic, &is,
				   nmshape, cname [ic]);
		if (status != E_SUCC) 
	 	   iso_error ("!Error reading name from input statistics file", 
			"isoclass-read");
                
		fprintf (print_file, "\n CLASS :  %s\n", cname [ic]);

		status = c_stget ((FILE **)&stat_fd, "MEAN_VECTOR", &file_ic, 
				&is, shape, (unsigned char *)filmean [ic]);
		if (status != E_SUCC) 
	 	   iso_error ("!Error reading mean from input statistics file", 
			"isoclass-read");
                

		/* check for consistency */
		if (shape [2] != totbnd) 
	   	  iso_error(
		     "!Error: Incorrect number of bands in input mean vectors", 
		      "isoclass-bands");
                

		/* copy file means to program variable */
		for (i = 0;  i < totbnd;  i++)
			clmean [ic] [i] = filmean [ic] [i];

		/*
		 * output means to print file; different formats depending on
		 * size of data
		 */
		print_string = (dtype_in < 3) ? " %8.2f" : " %12.3e";
		for (i = 0;  i < shape [2];  i++)  {
			if (!(i % PER_LINE))	/* new line every 12 */
				fprintf (print_file, "\n MEAN : ");
			fprintf (print_file, print_string, clmean [ic] [i]);
		}
		fprintf (print_file, "\n");
	}					/* for 0 <= ic < nclust */

	/* close statistics file */
	status = c_stclse ((FILE **)&stat_fd);
	if (status != E_SUCC)
	   iso_error ("!Error closing input statistics file","isoclass-close");
}
