#include <math.h>
#include "isoclass.h"
#include "protos.h"
                      
/*****************************************************************************
 *
 * NAME		setup
 *
 * PURPOSE	If no input statistics file was given, this routine goes
 *		through the image and figures out the sum, squared sum,
 *		overall mean, and overall standard deviation of the image,
 *		and outputs the information to the report file.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----  	     	----		---	-----------
 *	clknt [MAXCLS]	long		O	# pixels in each cluster
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I/O	Cluster means for each band
 *	clstdv [MAXCLS]
 *	       [MAXBND]	float		I/O	Cluster std devs for each band
 *	dtype_in	long		I	Data type of input images
 *	gd_in		long		I	Group descriptor - input images
 *	nclust		short		I/O	# clusters currently in image
 *	print_file	FILE		I	Handle to report file
 *	totbnd		short		I	Total # bands in input images
 *	window[][4]	long		I	Dimensions of images
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	clear out sum and squared sum arrays
 *	for each line
 *		for each band
 *			for each sample
 *				get pixel value
 *				add it to sum array
 *				add square of it to squared sum array
 *			end for
 *		end for
 *	end for
 *	for each band
 *		figure out mean and standard deviation for the image
 *	end for
 *	close the input files in input group
 *	print mean and standard deviation for each band in input image files
 *	try to split the single cluster
 *	if it doesn't split
 *		exit with error
 *	end if
 *	print initial cluster means
 *
 *****************************************************************************/
void setup
(
    float *clmean[]
)
{                                                 
	char		*print_string1,		/* for variable printf's */
			*print_string2;		/* for variable printf's */
	short		band,			/* band loop index */
			i,			/* band counter */
			j,			/* band counter (always >= i) */
			k,			/* cluster counter */
			line,			/* line loop index */
			nlines,			/* # of lines in window */
			nsamples,		/* # of samples in window */
			sample,			/* sample loop index */
			upper;			/* upper limit on loops */
	long   		npix,			/* # of pixels in window */
			status;			/* return val of ext modules */
	float		this_pix,		/* particular pixel value */
			x,			/* scratch for std dev calc */
			xnum;			/* # of pixels in window */
	double	 	sum1 [MAXBND],		/* summing array */
			*sum1_ptr,		/* pointer into sum1 */
			sum2 [MAXBND],	     	/* summing array of squares */
			*sum2_ptr;		/* pointer into sum2 */


	/* clear out summing arrays */
	sum1_ptr = sum1;
	sum2_ptr = sum2;
	i = MAXBND;
	while (i--)  {
		*sum1_ptr++ = 0.0;
		*sum2_ptr++ = 0.0;
	}

	nlines = (short)window[0][NL];
	nsamples = (short)window[0][NS];
	npix = nlines * nsamples;

	for (line = 0;  line < nlines;  line++)  {
		/* update offset array */
		status = c_estep ((struct GDESC **)&gd_in);
		if (status != E_SUCC)
		   iso_error  ("$", "");	/* fatal error */

		/* reset pixel routines to start of line */
		pixel_line_reset ();

	       	/* compute intermediate sums */
		sum1_ptr = sum1;
		sum2_ptr = sum2;
		for (band = 0;  band < totbnd;  band++)  {
			pixel_band_next ();
			for (sample = 0;  sample < nsamples;  sample++)  {
				this_pix = pixel_next ();
				*sum1_ptr += this_pix;
				*sum2_ptr += this_pix * this_pix;
			}
			sum1_ptr++;
			sum2_ptr++;
		}
	}

	/* compute overall means and standard deviations */
	xnum = (float) npix;
	for (i = 0;  i < totbnd;  i++)  {
	 	clmean [0] [i] = sum1 [i] / xnum;
		x = sum1 [i] * sum1 [i];
		x = (float) fabs ((double) (xnum * sum2 [i] - x));
		x /= xnum * (xnum - 1);
		clstdv [0] [i] = (float) sqrt ((double) x);
	}

	status = c_egclse ((struct GDESC **)&gd_in);
	if (status != E_SUCC)
		iso_error ("$","");	/* fatal error */
	
	/* Place info in print file */
	fprintf (print_file,
		 "\n\nOVERALL STATISTICS OF INPUT DATA \n\n");
	print_string1 = (dtype_in < 3) ? "%8.2f" : "%10.3e";
	print_string2 = (dtype_in < 3) ? "" : "  ";

	for (i = 0;  i < totbnd;  i += PER_LINE)  {
		upper = (short)min(i + PER_LINE, totbnd);

		/* print header */
      		fprintf (print_file, "        ");
		for (j = i + 1;  j <= upper;  j++)
		  	fprintf (print_file, "%s  BAND%2d",
				 print_string2, j);
		fprintf (print_file, "\n        ");
		for (j = i + 1;  j <= upper;  j++)
			fprintf (print_file, "%s  ------", print_string2);

		/* print means */
		fprintf (print_file, "\n MEAN   ");
		for (j = i;  j < upper;  j++)
		    	fprintf (print_file, print_string1,
				 clmean [0] [j]);

		/* print standard deviations */
		fprintf (print_file, "\n STDEV  ");
		for (j = i;  j < upper;  j++)
	       		fprintf (print_file, print_string1,
				 clstdv [0] [j]);
		fprintf (print_file, "\n\n");
	}

	/* compute initial cluster means */              
	clknt [0] = npix;
	nclust = 1;
	split ();
	if (nclust != 2)
           iso_error(
    "!Error: Unable to split initial cluster, maxclstd and/or minclust too large"
    ,"isoclass-split");

	fprintf (print_file, "\n\n\nINITIAL CLUSTER MEANS \n\n");
	for (i = 0;  i < totbnd;  i += PER_LINE)  {
		upper = (short)min(i + PER_LINE, totbnd);

		/* print out header */
		fprintf (print_file, " CLUSTER");
		for (j = i + 1;  j <= upper;  j++)
			fprintf (print_file, "%s  BAND%2d",
				 print_string2, j);
		fprintf (print_file, "\n -------");
		for (j = i;  j < upper;  j++)
			fprintf (print_file, "%s  ------", print_string2);

		/* print array of mean values */
		for (k = 0;  k < nclust;  k++)  {
			fprintf (print_file, "\n    %2d  ", k + 1);
		 	for (j = i;  j < upper;  j++)
				fprintf (print_file, print_string1,
					 clmean [k] [j]);
		}
		fprintf (print_file, "\n\n");
	}
}
