#include "isoclass.h"
#include "protos.h"

/*****************************************************************************
 *
 * NAME		split
 *
 * PURPOSE	To split all clusters that have too few members and a large
 *		standard deviation.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *	K. Narayanan            21-Mar-89       Changed buff_copy call
 *	Bani Allam / STX 	24-Jul-89       Freed allocated buffers
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clknt [MAXCLS]	long		I	# pixels in each cluster
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I	Cluster means for each band
 *	clstdv [MAXCLS]
 *	       [MAXBND]	float		I	Cluster std dev for each band
 *	maxcls		short		I	Maximum # of clusters allowed
 *	nclust		short		I/O	# clusters in currently defined
 *	nmin		long		I	Min # pixels allowed in cluster
 *	sep		float		I	Split separation value
 *	stdmax		float		I	Split determiner
 *	totbnd		short		I	Total # bands in input images
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	for each cluster
 *		if it has too many members
 *			set current maximum standard deviation to first band
 *			for each band other than first
 *				if its standard deviation is larger then
 *				    	make its std dev the current maximum
 *				end if
 *			end for
 *			if the maximum is greater than MAXCLSTD and the number 
 *			    of clusters is less than MAXCLUST then 
 *			    	increment the cluster count
 *				copy over means of all bands other than max to
 *				    temporary buffer
 *				split mean with max std dev and place in buf
 *			else
 *				copy the cluster's means to temporary buffer
 *			end if
 *		else
 *			copy the cluster's means to temporary buffer
 *		end if
 *	end for
 *	copy the temporary buffer back over to the means array
 *
 *****************************************************************************/
void split
(void)
{
	short	i,			/* cluster index (w/o splits) */
		ii,			/* cluster index (w/ splits) */
		imax,			/* index of maximum std. deviation */
		j,			/* image index */
		nc;			/* # clusters before any changes */
	long	c_size;
	float	*hclmean [MAXCLS],	/* holding array - new clmean */
		*hclmean_matrix,
		maximum;		/* maximum standard deviation */

   c_size = (long)(maxcls * totbnd * sizeof (float));
   if ((hclmean_matrix = (float *) malloc( c_size)) == NULL)
      iso_error ("!Error allocating dynamic memory", "isoclass-alloc");
   for (i = 0, j = 0; i < maxcls; i++, j += totbnd)
      hclmean[i] = &hclmean_matrix[j];

	nc = nclust;
	ii = 0;
	for (i = 0;  i < nc;  i++)  {
		/*
		 * Requirements for splitting a cluster are that the number of 
		 * members be > 2 * (nmin + 1)
		 * The cluster is split on the vector coordinate that has the 
		 * maximum standard deviation for the vector, and if that 
		 * standard deviation is > stdmax
		 */
		if (clknt [i] > (2 * (nmin + 1)))  {
			maximum = clstdv [i] [0];
			imax = 0;

			/*
			 * find maximum standard deviation along the
			 * vector mean
			 */
			for (j = 1;  j < totbnd;  j++)
				if (clstdv [i] [j] > maximum)  {
					maximum = clstdv [i] [j];
					imax = j;
				}

			/*
			 * if maximum is > stdmax and we haven't exceeded
			 * the maximum number of clusters allowed then split
			 * the mean
			 */
			if ((maximum > stdmax) && (nclust < maxcls))  {
			    	nclust++;

				/* copy prior means over */
				for (j = 0;  j < imax;  j++)  {
					hclmean [ii] [j] = clmean [i] [j];
					hclmean [ii + 1] [j] = clmean [i] [j];
				}
				j = imax;

				/*
				 * split the mean by either +/- standard
				 * deviation, or use the value sep (supplied
				 * by the user)
				 */
				if (sep == 0.0)  {
					hclmean [ii] [j] = clmean [i] [j] -
							   clstdv [i] [j];
					hclmean [ii + 1] [j] = clmean [i] [j] +
   							       clstdv [i] [j];
				}  else  {                     
					hclmean [ii] [j] = clmean [i] [j] - sep;
					hclmean [ii + 1] [j] = clmean [i] [j] +
							       sep;
				}
				/* copy subsequent means over */
				for (j = imax + 1;  j < totbnd;  j++)  {
					hclmean [ii] [j] = clmean [i] [j];
					hclmean [ii + 1] [j] = clmean [i] [j];
				}
				ii += 2;		/* used two clusters */
			}  else  {
				/* no split, just copy means over */
				for (j = 0;  j < totbnd;  j++)
			 		hclmean [ii] [j] = clmean [i] [j];
			 	ii++;			/* used one cluster */
			}		
		}  else  {
			/* no split, just copy means over (same as above) */
			for (j = 0;  j < totbnd;  j++)
		 	  	hclmean [ii] [j] = clmean [i] [j];
			ii++;				/* used one cluster */
		}
	}					/* 0 <= i < nc */

	/* copy adjusted means in holding area back to original means */
	buff_copy ((char *)hclmean[0], (char *)clmean[0], (long)totbnd * maxcls,
		   (short) sizeof (float));
	free (hclmean_matrix);
}                                                      
