#include "isoclass.h"
#include "protos.h"


long	cshape [9],	/* covariance matrix */
	mshape [9],	/* number of pixels in each mean */
	nshape [9],	/* shape vector for cluster means */
    	nmshape [9];	/* shape vector for cluster names */


/*****************************************************************************
 *
 * NAME		init_shapes
 *
 * PURPOSE	To initialize the shape variables.  Since they are almost all
 *		identical, do one and copy it to the others.  
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	totbnd		short		I	Total # bands in input images
 *
 * RETURN VALUES
 *
 *	None
 *
 *****************************************************************************/
void init_shapes
(void)
{
	short	i;	/* general counter */



	/* initialize all */
	for (i = 2;  i < 9;  i++)
		nmshape [i] = 0;
	nmshape [1] = 1;
	buff_copy ((char *)nmshape,(char *)mshape, (long)9, (short)sizeof(int));
	buff_copy ((char *)nmshape,(char *)cshape, (long)9, (short)sizeof(int));
	buff_copy ((char *)nmshape,(char *)nshape, (long)9, (short)sizeof(int));

	nmshape [0] = EBYTE;
	nmshape [2] = 7;

	mshape [0] = ELONG;
	mshape [2] = 1;

	nshape [0] = EDOUBLE;
	nshape [2] = totbnd;

	cshape [0] = EDOUBLE;
	cshape [2] = (float) totbnd * (float) (totbnd + 1) / 2;
}


/*****************************************************************************
 *
 * NAME		wrstats
 *
 * PURPOSE	To output class names, pixel counts of clusters, cluster mean
 *		vectors, and image covariance matrix to the output statistics
 *		file.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *	Bani Allam / STX 	24-Jul-89       Freed allocated buffers
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	scov [MAXCLS]
 *	     [LOW_HALF]	double		I	Covariance matrix for image
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----  	       	----		---	-----------
 *	clknt [MAXCLS]	float		I	# pixels in each cluster
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I	Cluster means for each band
 *	nclust		short		I	Total # clusters found
 *	stat_host_out	char    	I	Host vers. - out stats filename
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	initialize shapes
 *	open output statistics file
 *	for each cluster
 *		output cluster name to file
 *		output # pixels in cluster to file
 *		output cluster mean vector to file
 *		output covariance matrix
 *	end for
 *	close statistic file
 *
 *****************************************************************************/
void wrstats
(
    double *scov[],
    float *clmean[]
)
{
	char	cname [MAXCLS] [80];	/* names of clusters */
 	short		i, n;		       	/* image index */
	long		access,			/* stats file access */
                        c_size,
			file_ic,		/* file class index */
			ic,			/* actual class index */
			is = 0,			/* site index */
			nclass,			/* # classes in file */
			status;			/* return val of ext modules */
        float           *cl_ptr;
        double          *f_ptr;
	double		*filmean_matrix,
                        *filmean[MAXCLS];	/* (double) clmean */
	long		stat_fd;		/* file descriptor */


	/* initialize shape variables */
	init_shapes ();
	
        /* allocate dynamic storage */
        c_size = (long)(maxcls * totbnd * sizeof (double));
        filmean_matrix = (double *) malloc (c_size);
	if (filmean_matrix == NULL)
	  c_errmsg("error allocating dynamic memory","isoclass-alloc",&one);
        for (i=0, n=0; i < maxcls; i++, n += totbnd)
           filmean[i] = &filmean_matrix[n];

	/* open output stats file */
	access = IUNKNOWN;				/* write new file */
	status = c_stopen ((FILE **)&stat_fd, stat_host_out, &access, &nclass);
	if (status == E_GNLIO) 
           iso_error ("!Error opening output statistics file", "isoclass-open");
	

	/* put class names, these names will be clust01, clust02, ... */
	for (ic = 0;  ic < nclust;  ic++)  {
		file_ic = ic + 1;

		sprintf (cname [ic], "CLUST%02d", file_ic);
		status = c_stput ((FILE **)&stat_fd, "NAME", &file_ic, &is,
				nmshape, (unsigned char *)cname [ic]);
		if (status != E_DTNEW && status != E_NOCL && status != E_NOSI)
		   iso_error ("!Error writing name to output statistics file", 
			"isoclass-write");


		/* put npoints */
		status = c_stput ((FILE **)&stat_fd, "NPOINTS", &file_ic, &is, 
				  mshape, (unsigned char *)&clknt [ic]);
		if (status != E_DTNEW && status != E_NOCL && status != E_NOSI)
	   iso_error("!Error writing number of points to output statistics file"
 			,"isoclass-write");


		/* put cluster mean vectors */
                cl_ptr = clmean[ic];
                f_ptr = filmean[ic];
		for (i = 0;  i < totbnd;  i++)
                   {
			*f_ptr = (double) *cl_ptr;
                        f_ptr++; cl_ptr++;
                   }
		status = c_stput ((FILE **)&stat_fd, "MEAN_VECTOR", &file_ic, 
				&is, nshape, (unsigned char *)filmean [ic]);
		if (status != E_DTNEW && status != E_NOCL && status != E_NOSI)
		   iso_error ("!Error writing mean to output statistics file", 
			"isoclass-write");


		/* put covariance matrices */
		status = c_stput ((FILE **)&stat_fd, "COVARIANCE_MATRIX", 
				&file_ic, &is, cshape,
				(unsigned char *)scov [ic]);
		if (status != E_DTNEW && status != E_NOCL && status != E_NOSI)
       iso_error ("!Error writing covariance matrix to output statistics file", 
			"isoclass-write");
	}						/* 0 <= ic < nclust */

	/* close output statistics file */
	status = c_stclse ((FILE **)&stat_fd);
	if (status != E_SUCC)
	   iso_error ("!Error closing output statistics file","isoclass-close");
	free (filmean_matrix);
}
