/****************************************************************************
NAME:	JSCGRID

PURPOSE:  Creates a Geometric Mapping Grid file mapping from systematicly
          corrected MSS 4/5 data to the UTM map projection at 57 meter pixels.

VERSION		DATE	AUTHOR		CHANGE	
-------		----	------		------
  1.0		11/92	D. Steinwand

  2.0		2/93    D. Etrheim	Release to baseline.  This routine
				does not conform to all LAS standards.  The
				main effort was put into creating the standard
				LAS interface, User Guide, and error 
				checking/trapping.  The supports can be looked
				at later after the C version of GCTP is
				finialized.

  3.0		8/95	C. Engebretson  Fix problems with routine's
					interface to GCTP functions.
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "jsc.h"
#include "geompak.h"
#include "util.h"
#include "header.h"
#include "proj.h"
#include "cproj.h"
#include "protos.h"

extern long curent;		/* Current "line" element for processing mess */
FILE *fp;			/* File pointer for output print file */
int main
(void)
{
struct PARBLK vblock;           /* TAE parameter block */
struct GEOGRID grid;		/* Geometric mapping grid */
double *iline_ptr;		/* Pointer to input space grid coordinate */
double *isamp_ptr;		/* Pointer to input space grid coordinate */
double proj_y, proj_x;		/* Projection coordinates in Y & X */
long msgtim;			/* Message timing parameter */
long *xptr,*yptr;		/* x & y values for polynomial evaluation */
long one = 1;			/* Constant of one */
long three = 3;			/* Constant of three */
long j, i;		        /* Loop counters */
long nel;			/* Number of input grid coordinates */
double lat_center;		/* latitude center from header file */
double lon_center;		/* longitude center from header file */
double xval,yval;
float line_center;		/* center of full scene */
float samp_center;		/* center of full scene */
double x,y;
double out_x, out_y;
long path;			/* scene path number from landsat header */
long zone;			/* zone number for scene */
double angle,sin_angle,cos_angle;
double sp,lp;
double ulxp,ulyp,urxp,uryp,llxp,llyp,lrxp,lryp;
double lat,lon = 0.0;
double A[12]; 			/* Matrix A (used for LSQ fit) */
double inbx[4]; 		/* Vector containing X coordinates--input */
double inby[4]; 		/* Vector containing Y coordinates--input */
double v[4]; 			/* Work buffer for LSQ fit */
double outbx[4]; 		/* Vector containing X coordinates--output */
double outby[4]; 		/* Vector containing Y coordinates--output */
double r_major;
double r_minor;
double scale_fact;
double false_east = 0;
double false_north = 0;
double alf_in = 0.0;
double time = 0.0;
char header_file[CMLEN];	/* host name of LANDSAT header file */
char gridname[CMLEN];		/* host nmae of output grid file */
char taename[2][CMLEN];		/* output TAE names */
long print[3];			/* print utility options */
long fatal = -1;
char tmpfil[1][CMLEN];
char prtfile[CMLEN];		/* TAE name of output print file */
long numout = 1;		/* number of output file names */
long temp, temp1,temp2;
long satnum;			/* sat number */	
long start1 = 0;
long flag;

struct JSCLH lhdr;

/* Initialize LAS processing and get parameters
----------------------------------------------*/
getpar(&vblock,header_file,gridname,print,tmpfil[0],prtfile,&msgtim);

/* Read the Landsat Header file
  ----------------------------*/
if (c_jsc_lhdr(header_file,&lhdr) != E_SUCC)
   c_errmsg("Fatal error encountered","jscgrid-fatal",&fatal);
temp = atol(lhdr.frame_centre_lat);
temp1 = temp/100;
temp2 = temp - temp1*100;
lat_center = temp1 + (double)temp2/60.0;

temp = atol(lhdr.frame_centre_long);
temp1 = temp/100;
temp2 = temp - temp1*100;
lon_center = temp1 + (double)temp2/60.0;
if (lon_center > 180.0)
   lon_center -= 360.0;

path = atol(lhdr.path_number);

/* Open the temporary print file
--------------------------------*/
fp = fopen(tmpfil[0],"w");
if (fp == NULL)
   {
   c_errmsg("Error opening temporary file","jscgrid-open",&one);
   c_errmsg("Fatal error encountered","jscgrid-fatal",&fatal);
   }

/* Initalize projection routines
-------------------------------*/
satnum = atol(lhdr.mission_id);

flag = 0;
r_major = 6378388.0;
r_minor = 6356911.94613;
if (somforint(r_major,r_minor,satnum,path,alf_in,lon,false_east,false_north,
              time, start1,flag) != E_SUCC)
   c_clean(&vblock,"jscgrid",tmpfil,&one);

if (sominvint(r_major,r_minor,satnum,path,alf_in,lon,false_east,false_north,
 	      time,start1,flag) != E_SUCC)
   c_clean(&vblock,"jscgrid",tmpfil,&one);

zone=calc_utm_zone(lon_center);
if (lat_center < 0)
    zone = -zone;

scale_fact = .9996;
r_major = 6378206.4;
r_minor = 6356583.8;
utmforint(r_major,r_minor,scale_fact,zone);
utminvint(r_major,r_minor,scale_fact,zone);

/* Calculate the corner coordinates of the input image.  First calculate the 
   center projection coordinate and then calculate each of the corner 
   coordinates based on the center coordinate and the angle that the 
   projection coordinate system is rotated from the image.
---------------------------------------------------------------------------*/
line_center = 1143.0;
samp_center = 1800.0;
lat_center *= D2R;
lon_center *= D2R;
if (somfor(lon_center,lat_center,&y,&x) != E_SUCC)
   c_clean(&vblock,"jscgrid",tmpfil,&one);
y = -y;
fprintf(fp,"\n\nCenter SOM coordinates:  %f %f\n",y,x);

/* Calculate rotation angle from true north
------------------------------------------*/
sin_angle = 0.142974363/cos(lat_center);
angle = asin(sin_angle);
cos_angle = cos(angle);
fprintf(fp,"Rotation angle (degrees):  %f\n", angle*R2D);
if (angle < 0.0) angle *= -1;

fprintf(fp,"\n\nInput SOM coordinates:");
/* Calculate Upper Left coordinates
----------------------------------*/
sp = (1.0 - samp_center) * 57.0;
lp = (line_center - 1.0) * 82.0;
ulxp =  (sp * cos_angle + lp * sin_angle) + x;
ulyp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"\nUL = %f %f\n", ulyp,ulxp);

/* Calculate Upper Right Coordinates
-----------------------------------*/
sp = (3600 - samp_center) * 57.0;
lp = (line_center - 1.0) * 82.0;
urxp =  (sp * cos_angle + lp * sin_angle) + x;
uryp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"UR = %f %f\n", uryp,urxp);

/* Calculate Lower Right Coordinates
-----------------------------------*/
sp = (3600 - samp_center) * 57.0;
lp = (line_center - 2286) * 82.0;
lrxp =  (sp * cos_angle + lp * sin_angle) + x;
lryp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"LR = %f %f\n", lryp,lrxp);

/* Calculate Lower Left Coordinates
----------------------------------*/
sp = (1.0 - samp_center) * 57.0;
lp = (line_center - 2286) * 82.0;
llxp =  (sp * cos_angle + lp * sin_angle) + x;
llyp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"LL = %f %f\n", llyp,llxp);

/* Calculate a set of first order polynomial coefficients which map input
   space SOM coordinates to input space line,sample coordinates
-----------------------------------------------------------------------*/
for(i = 0; i < 4; i++) 
    A[i] = 1.0;
A[4]=ulxp;
A[5]=urxp;
A[6]=lrxp;
A[7]=llxp;
A[8]=ulyp;
A[9]=uryp;
A[10]=lryp;
A[11]=llyp;

inbx[0]= 1.0;
inbx[1]= 3600.0;
inbx[2]= 3600.0;
inbx[3]= 1.0;
inby[0]= 1.0;
inby[1]= 1.0;
inby[2]= 2286.0;
inby[3]= 2286.0;

QR(A, 4, 3, v);
QR_solve(A, 4, 3, v, inbx, 0);
QR_solve(A, 4, 3, v, inby, 0);

/* Now calculate the corner coordinates of the output space.  First 
   calculate in SOM coorinates and convert to UTM coordinates
---------------------------------------------------------------------*/
line_center = 1491.5;
samp_center = 1774.0;
fprintf(fp,"\n\nOutput SOM coordinates:");
 
/* Calculate Upper Left coordinates
----------------------------------*/
sp = (1.0 - samp_center) * 57.0; 
lp = (line_center - 1.0) * 57.0;
ulxp =  (sp * cos_angle + lp * sin_angle) + x;
ulyp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"\nUL = %f %f\n", ulyp,ulxp);
 
/* Calculate Upper Right Coordinates
-----------------------------------*/
sp = (3548 - samp_center) * 57.0;
lp = (line_center - 1.0) * 57.0;
urxp =  (sp * cos_angle + lp * sin_angle) + x;
uryp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"UR = %f %f\n", uryp,urxp);
 
/* Calculate Lower Right Coordinates
-----------------------------------*/
sp = (3548 - samp_center) * 57.0;
lp = (line_center - 2983) * 57.0;
lrxp =  (sp * cos_angle + lp * sin_angle) + x;
lryp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"LR = %f %f\n", lryp,lrxp);
 
/* Calculate Lower Left Coordinates
----------------------------------*/
sp = (1.0 - samp_center) * 57.0;
lp = (line_center - 2983) * 57.0;
llxp =  (sp * cos_angle + lp * sin_angle) + x;
llyp = (-sp * sin_angle + lp * cos_angle) + y;
fprintf(fp,"LL = %f %f\n", llyp,llxp);

/* Calculate UTM corners of output space
---------------------------------------*/
if (sominv(-ulyp,ulxp,&lon,&lat) != E_SUCC)
   c_clean(&vblock,"jscgrid",tmpfil,&one);
utmfor(lon,lat,&ulxp,&ulyp);
sominv(-llyp,llxp,&lon,&lat);
utmfor(lon,lat,&llxp,&llyp);
sominv(-uryp,urxp,&lon,&lat);
utmfor(lon,lat,&urxp,&uryp);
sominv(-lryp,lrxp,&lon,&lat);
utmfor(lon,lat,&lrxp,&lryp);
fprintf(fp,"\n\nOutput UTM coordinates:\n");
fprintf(fp,"\nUL = %f %f\n", ulyp,ulxp);
fprintf(fp,"UR = %f %f\n", uryp,urxp);
fprintf(fp,"LR = %f %f\n", lryp,lrxp);
fprintf(fp,"LL = %f %f\n", llyp,llxp);

fclose(fp);
/* Calculate first order polynomial coefficients which map output line,sample
   coordinates to output projection coordinates
---------------------------------------------------------------------------*/
for(i = 0; i < 4; i++) 
    A[i] = 1.0;
A[4]=1.0;
A[5]=3548.0;
A[6]=3548.0;
A[7]=1.0;
A[8]=1.0;
A[9]=1.0;
A[10]=2983.0;
A[11]=2983.0;

grid.corners[1] = outbx[0] = ulxp;
grid.corners[5] = outbx[1] = urxp;
grid.corners[7] = outbx[2] = lrxp;
grid.corners[3] = outbx[3] = llxp;
grid.corners[0] = outby[0] = ulyp;
grid.corners[4] = outby[1] = uryp;
grid.corners[6] = outby[2] = lryp;
grid.corners[2] = outby[3] = llyp;

QR(A, 4, 3, v);
QR_solve(A, 4, 3, v, outbx, 0);
QR_solve(A, 4, 3, v, outby, 0);

/* Place the number of output lines and samples into the grid record
 ------------------------------------------------------------------*/
grid.nrows = grid.ncols = 30;
grid.lines = 2983;
grid.samples = 3548;

/* Calculate the amount of buffer space needed for the output grid
   intersections and the input grid coordinate pairs, then allocate memory.
-------------------------------------------------------------------------*/
nel = 900;
grid.out_lines = (long *)malloc(30 * sizeof(long));
grid.out_samps = (long *)malloc(30 * sizeof(long));
grid.in_lines = (double *)malloc(900 * sizeof(double));
grid.in_samps = (double *)malloc(900 * sizeof(double));
if((grid.out_lines == NULL) || (grid.out_samps == NULL) ||
    (grid.in_lines == NULL) || (grid.in_samps == NULL)) 
    {
    c_errmsg("Error allocating dynamic memory","jscgrid-alloc",&one); 
    c_clean(&vblock,"jscgrid",tmpfil,&one);
    }

/* Calculate the output grid intersections and place in geometric mapping
   grid record.
------------------------------------------------------------------------*/
for (i = 0; i < 30; i++) 
    grid.out_lines[i] = 1 + (i * 103);
for (i = 0; i < 30; i++) 
    grid.out_samps[i] = 1 + (i * 123);

/* Compute input grid coordinates
--------------------------------*/
curent = 1;
c_inittm("JSCGRID","Generated","grid element","","",&nel,&one,&msgtim,&three);
c_startm();
iline_ptr = grid.in_lines;
isamp_ptr = grid.in_samps;

for (yptr = grid.out_lines, i= grid.nrows; i--; yptr++)
  for (xptr = grid.out_samps, j = grid.ncols; j--; xptr++, curent++)
    {
    xval=(double)(*xptr);
    yval=(double)(*yptr);
    proj_y = c_eval(&one,outby,&xval,&yval);
    proj_x = c_eval(&one,outbx,&xval,&yval);
    if (utminv(proj_x,proj_y,&lon,&lat) != E_SUCC)
       c_clean(&vblock,"jscgrid",tmpfil,&one);
    if (somfor(lon,lat,&out_y,&out_x) != E_SUCC)
       c_clean(&vblock,"jscgrid",tmpfil,&one);
    out_y = (-out_y);
    *iline_ptr++ = c_eval(&one,inby,&out_x,&out_y);
    *isamp_ptr++ = c_eval(&one,inbx,&out_x,&out_y);
    }

c_stoptm();

/* Pack projection related fields in the grid file
-------------------------------------------------*/
grid.proj_valid = GEO_VALID;
grid.code = 1;
grid.zone = zone;
grid.datum = 0;
grid.pdist[0] = grid.pdist[1] = 57.0;
for (i = 0; i < 15; i++) 
    grid.projprms[i] = 0.0;
strcpy(grid.units, "meters");

/* Pack modeling related fields in the grid file
-----------------------------------------------*/
strcpy(grid.model, "JSCGRID");
grid.ncoeff = 0;
grid.coeffs = NULL;

/* Pack statistics related fields in the grid file 
-------------------------------------------------*/
grid.fiterr_valid = GEO_NOTVALID;
grid.max_fit_err[0] = grid.max_fit_err[1] = 0;
grid.ave_fit_err[0] = grid.ave_fit_err[1] = 0;
grid.rms_fit_err = 0;
grid.griderr_valid = GEO_NOTVALID;
grid.max_grid_err[0] = grid.max_grid_err[1] = 0;
grid.ave_grid_err[0] = grid.ave_grid_err[1] = 0;
grid.rms_grid_err = 0;

/* Write the geometric mapping grid to disk
------------------------------------------*/
if(c_putgrd(gridname, &grid) != E_SUCC)
  c_clean(&vblock,"jscgrid",tmpfil,&one);

/* Issue completion message
--------------------------*/
strcpy(taename[0], gridname);
if (print[1])
   {
   strcpy(taename[1],prtfile);
   numout = 2;
   }
c_complt("JSCGRID",taename,&numout);

return EXIT_SUCCESS;

}
