/******************************************************************************
FUNCTION:	GETHDR

PURPOSE:	Read the universal header from tape and write it to disk if
		specified.  Also optionally read and write the the LANDSAT 
		header to disk.	

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "jscin.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"

int gethdr
(
    long *unit,			/* I: tape unit number			*/
    long *wrtdsk,         	/* I: ancillary files to read in 	*/
    char hosout[][CMLEN],       /* I: Output host names                	*/
    long *num_out, 		/* I/O: number of output files         	*/
    unsigned char **hdr_buf	/* O: header buffer read from tape   	*/
)

{
FILE *fp;		/* file pointer					*/

unsigned char *in_buf;	/* buffer read from data on tape           	*/

char sysid[33];         /* computing system identification              */

long count;		/* number of files to skip			*/
long i;			/* loop counter					*/
long mode = HANDLE;	/* tape I/O error flag                        	*/
long one = 1;		/* constant of one				*/
long numbytes;		/* number of bytes to read			*/
long status;		/* status flag 					*/
long temp;		/* temp value					*/


/* Allocate space
-----------------*/
in_buf = (unsigned char *) malloc(GT_THAI);
if (in_buf == NULL)
    {
    c_errmsg("Error allocating dynamic memory","jscin-alloc",&one);
    return(E_FAIL);
    }

*hdr_buf = (unsigned char *) malloc(UHDR_SIZ);
if (*hdr_buf == NULL)
    {
    c_errmsg("Error allocating dynamic memory","jscin-alloc",&one);
    return(E_FAIL);
    }

/* Read the universal header
---------------------------*/
numbytes=UHDR_SIZ;
status = c_tpread(unit,*hdr_buf,&numbytes,&mode);
if (status != UHDR_SIZ)
    {
    c_errmsg("Error encountered reading tape","jscin-tape",&one);
    return(E_FAIL);
    }

/* Read the EOF tape mark
------------------------*/
status = c_tpread(unit,in_buf,&numbytes,&mode);
if (status != TAPE_EOF)
    {
    c_errmsg("Error encountered reading tape","jscin-tape",&one);
    return(E_FAIL);
    }

/* If specified, write the universal header and LANDSAT header to disk,
otherwise, skip the record.
---------------------------------------------------------------------*/
if (wrtdsk[1])
    {
    /* Get the computing system id first
    ------------------------------------*/
    temp = 32;
    strncpy(sysid,(const char *)*hdr_buf,temp);
    c_eb2asc(sysid,&temp);
    
    /* Create name for universal header and write
    ---------------------------------------------*/
    c_lsmknm(hosout[0],"uhdr",hosout[*num_out]);
    fp = fopen(hosout[*num_out],"w");
    *num_out +=1;
    fwrite(*hdr_buf,1,UHDR_SIZ,fp);
    fclose(fp);
    
    /* Create name for LANDSAT header, read and write it
    ----------------------------------------------------*/
    c_lsmknm(hosout[0],"lhdr",hosout[*num_out]);
    fp = fopen(hosout[*num_out],"w");
    *num_out +=1;
    
    numbytes = LHDR_SIZ;
    status = c_tpread(unit,in_buf,&numbytes,&mode);
    if (status != LHDR_SIZ)
        {
        c_errmsg("Error encountered reading tape","jscin-tape",&one);
        return(E_FAIL);
        }
    fwrite(in_buf,1,LHDR_SIZ,fp);
    
    /* Determine number of bytes to read for geometric transformation record
    -----------------------------------------------------------------------*/
    if (strncmp(sysid,"ELS",3) == 0)
        numbytes = GT_ESA;			/* ESA tapes */
        else if (strncmp(sysid,"TELES",5) == 0)
            numbytes = GT_ITA;			/* Italy tapes */
	else
	    numbytes = GT_THAI;			/* Thailand and other tapes */
                
	status = c_tpread(unit,in_buf,&numbytes,&mode);
    if (status != numbytes)
        {
        c_errmsg("Error encountered reading tape","jscin-tape",&one);
        return(E_FAIL);
        }
    fwrite(in_buf,1,numbytes,fp);
    
    /* Read the radiometric transformation records
    ----------------------------------------------*/
    numbytes = RT_SIZ;
    for (i = 0; i < 5; i++)
        {
        status = c_tpread(unit,in_buf,&numbytes,&mode);
        if (status != RT_SIZ)
            {
            c_errmsg("Error encountered reading tape","jscin-tape",&one);
            return(E_FAIL);
            }
        fwrite(in_buf,1,RT_SIZ,fp);
        }
    
    fclose(fp);
    
    status = c_tpread(unit,in_buf,&numbytes,&mode);
    if (status != TAPE_EOF)
        {
        c_errmsg("Error encountered reading tape","jscin-tape",&one);
        return(E_FAIL);
        }
    }
else
    {
    count = 1;
    status = c_tpfsf(unit,&count,&mode);
    if (status != TAPE_SUCC)
        return(E_FAIL);
    }

free(in_buf);

return(E_SUCC);
}

