/******************************************************************************
FUNCTION:	JSCIN


PURPOSE:	Reads in image files from tapes that are in the Johnson Space 
		Center (JSC) format and and optionally reads in the ancillary
		data

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  -----------------------------------------------
1.0	 01/93	D. Etrheim	 Initial development
2.0	 12/99  D. Hames         Remove Valid=(1:50000) from WINDOW parameter in
				 jscin.pdf. Add an error check to getpar to 
				 ensure the value for WINDOW is greater than or
				 equal to 1.  Add prototypes. SRF #893.
3.0	 01/00	D. Hames	 Modify the error check in getpar to check for a
				 positive value for WINDOW only if the value of
				 WINDOW is not null. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "jscin.h"
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
int    fdesc[2];		/* image file descriptors used for image I/O */
long   gdesc;			/* group descriptor used for image I/O       */
long   offset[MAXBND + 1];	/* offsets into "thebuf" for output bands    */

void main
(void)
{
struct PARBLK vblock;	/* TAE parameter block                        */

unsigned char *hdr_buf;	/* header buffer read from tape               */

long bands[MAX_BAND]; 	/* bands to be processed within image         */
long i;			/* counter				      */
long msgtim;		/* elapsed time between display msg           */
long num_out = 1;	/* number of output files 		      */
long one = 1;		/* value of one			              */
long unit;             	/* tape unit number                           */
long totbnd = 0;	/* total number of bands to be copied         */
long wrtdsk[2];        	/* ancillary files to read in 		      */
long window[4];		/* window of input image                      */
long zero = 0;		/* value of zero                              */

long minmax;		/* min max flag                               */

char comment[CMLEN];	/* tape mount message to the operator         */
char hosout[5][CMLEN];	/* Output host file name                      */
char taeout[5][CMLEN];	/* Output TAE file names	              */

/* Retrieve the input parameters
--------------------------------*/
getpar(&vblock, bands, &msgtim, window, &minmax, hosout, taeout,
comment, wrtdsk);

/* Open the tape
---------------*/
mount_tape(&unit, comment);

/* Read the header data and write to disk if specified
------------------------------------------------------*/
if (gethdr(&unit, wrtdsk, hosout, &num_out, &hdr_buf) != E_SUCC)
    {
    close_tape(&unit);
    c_clean(&vblock,"jscin",hosout,&num_out);
    }

/* Process the image data
------------------------*/
if (proces(hosout, window, &totbnd, &unit, wrtdsk, &num_out, bands, hdr_buf,
    &msgtim, &minmax) != E_SUCC)
    {
    close_tape(&unit);
    c_clean(&vblock,"jscin",hosout,&num_out);
    }

/* Dismount the tape
--------------------*/
close_tape(&unit);

/* Fill in the DDR with info from the header file
-------------------------------------------------*/
if (update_ddr(hosout, window, hdr_buf, &totbnd) != E_SUCC)
    c_errmsg("Nonfatal error encountered","jscin-nonfatal",&one);

/* Free allocated space
----------------------*/
free(hdr_buf);

/* Output the history information
---------------------------------*/
if (c_hstout(&vblock,(char *)hosout,hosout,&zero," ") != E_SUCC)
    c_errmsg("Nonfatal error encountered","jscin-nonfatal",&one);

/* Issue the completion message
-------------------------------*/
for (i = 1; i < num_out; i++)
    strcpy(taeout[i], hosout[i]); 
c_complt("JSCIN",taeout,&num_out);
}

