/******************************************************************************
FUNCTION:	PROCES

PURPOSE:	Read image data from tape and write it to disk.  The image
                data is BIL.  Each image line contains the record number
                (2 bytes), ancillary data, and image data.  The ancillary 
                data is before the image data for the first band and after 
                the image data on the remaining bands.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "jscin.h"
#include "util.h"
#include "pixman.h"
#include "labserv.h"
#include "protos.h"

int proces
(
    char hosout[][CMLEN],  /* I: output host names  		      	   */
    long *window,	   /* I: window area of image to be copied 	   */
    long *totbnd,	   /* I: total number of bands to be copied	   */
    long *unit,            /* I: tape unit number                          */
    long *wrtdsk,          /* I: ancillary files to read in                */
    long *num_out,         /* I/O: number of output files 		   */
    long *bands,           /* I: bands to be processed within image        */
    unsigned char *hdr_buf,/* I: header buffer read from tape              */
    long *msgtim,	   /* I: elapsed time between display msg  	   */
    long *minmax	   /* I: min max flag                      	   */
)

{
extern long curent;     /* output line being processed               */
extern long gdesc;	/* group descriptor used for image I/O       */
extern long offset[];	/* offsets into "thebuf" for output bands    */
extern int  fdesc[];	/* image file descriptors used for image I/O */

FILE *fp;		/* file descriptor			     */

long  access = IWRITE;  /* write access to output image      	     */
long  ancsize;		/* Size of ancillary record on each line     */
long  bandoff[MAX_BAND];/* offset into each band to get to image data */
long  bufsiz;		/* image I/O buffer size             	     */
long  cb;               /* current output disk band being processed  */
long  curbnd;           /* curent tape band being processed          */
long  i;		/* loop counter				     */
long  idtype;		/* data type of input image          	     */
long  mode = HANDLE;    /* tape I/O error flag			     */
long  msgflg;		/* Processing message format flag	     */
long  nrec;		/* number of records to skip                 */
long  numbands;		/* number of bands taken from header record  */
long  one = 1;		/* constant one			             */
long  option[3];	/* output option array		             */
long  outspc[3];	/* output image file specifications  	     */
long  recsize;		/* bytes per record			     */
long  status;		/* return status                             */
long  temp;		/* temp value                                */
long  zero = 0;		/* value of zero                     	     */

static long datasize[4] = {0,1,2,4};
/* defines number bytes for each data type */

unsigned char *in_buf;          /* buffer read from image on tape    */
unsigned char *in_ptr;          /* pointer to input buffer window    */
unsigned char *out_ptr;         /* pointer to output buffer window   */
unsigned char *thebuf;          /* declare image I/O buffer          */

double outmax[MAXBND];          /* max pixel values of output image  */
double outmin[MAXBND];          /* min pixel values of output image  */

/* Set offsets into each line to get to image data
--------------------------------------------------*/
for (i = 0; i < MAX_BAND; i++)
    bandoff[i] = 2;

/* Pull out needed info from the header record
----------------------------------------------*/
outspc[0] =  ((long)hdr_buf[2756] * 256 + (long)hdr_buf[2757]);
outspc[1] =  ((long)hdr_buf[95] * 256 + (long)hdr_buf[96]);
numbands = (long)hdr_buf[89];
recsize =  ((long)hdr_buf[99] * 256 + (long)hdr_buf[100]);
ancsize =  ((long)hdr_buf[104] * 256 + (long)hdr_buf[105]);

/* Get data type
----------------*/
temp = (long)hdr_buf[90];
switch (temp)
    {
    case 8:  idtype = EBYTE; break;
    case 16: idtype = EWORD; break;
    default: c_errmsg("Data type not handled","jscin-dtype",&one);
        return(E_FAIL);
    }

/* For the first band, add the size of the ancillary
record to get to start of the image data.
----------------------------------------------------*/
bandoff[0] += ancsize;

/* Check the window
-------------------*/
chk_window(outspc,window);

/* Check the bands
-------------------*/
chk_bands(bands,&numbands,totbnd);
if (*totbnd == 0)
    {
    c_errmsg("Zero total bands specified","jscin-zero",&one);
    return(E_FAIL);
    }

/* Open the output image file for sequential write access
---------------------------------------------------------*/
option[0] = 0;
option[1] = 0;
option[2] = 0;
if (c_eopens((struct FDESC **)&fdesc[0],hosout[0],totbnd,&one,&one,&window[NL],
    &window[NS],offset,&access,&idtype,&zero,option) != E_SUCC)
    return(E_FAIL);
fdesc[1] = 0;

/* Allocate I/O buffer
---------------------*/
if ((thebuf =(unsigned char *)c_ealloc((struct FDESC **)fdesc,&bufsiz)) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","jscin-alloc",&one);
    return(E_FAIL);
    }

/* Group the file descriptors, so that only one estep is needed
---------------------------------------------------------------*/
if (c_egroup((struct FDESC **)fdesc,(struct GDESC **)&gdesc,thebuf,
		&bufsiz) != E_SUCC)
    return(E_FAIL);

/* Allocate memory for the input buffer
---------------------------------------*/
in_buf = (unsigned char *) malloc(recsize);
if (in_buf == NULL)
    {
    c_errmsg("Error allocating dynamic memory","jscin-alloc",&one);
    return(E_FAIL);
    }

/* If min max values are to be computed, call inmnmx to
initialize the min max array for the output data type.
--------------------------------------------------------*/
if (*minmax)
    c_inmnmx(&idtype,totbnd,outmin,outmax);

/* If specified, open the ancillary file
----------------------------------------*/
if (wrtdsk[0])
    {
    c_lsmknm(hosout[0],"anc",hosout[*num_out]);
    fp = fopen(hosout[*num_out],"w");
    *num_out +=1;
    }

/* Initialize the processing message
------------------------------------*/
msgflg = ALLCAT;
c_inittm("JSCIN","Processed","line","band(s)","",&window[NL],totbnd,
msgtim,&msgflg);

/* Skip records on tape to starting line of the window
------------------------------------------------------*/
if (window[SL] > 1)
    {
    c_errmsg("Skipping to starting postion on tape","informational",&one);
    nrec = (window[SL] - 1) * numbands;
    if (c_tpfsr(unit,&nrec,&mode) != TAPE_SUCC)
        return(E_FAIL);
    }

/* Start processing message
---------------------------*/
curent = 1;
c_startm();

/* Initial step to set up offset into the output image.
------------------------------------------------------*/
if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
    return(E_FAIL);

/* Process each line
--------------------*/
for (curent=0; curent < window[NL]; curent++)
    {
    cb = 0;
    /* Process each band from tape
    ------------------------------*/
    for (curbnd = 0; curbnd < numbands; curbnd++)
        {
        status = c_tpread(unit,in_buf,&recsize,&mode);
        if (status != recsize)
            {
            c_errmsg("Error encountered reading tape","jscin-tape",&one);
            return(E_FAIL);
            }
        
        if (bands[curbnd])
            {
            /* Position the output buffer pointer to
            the beginning of the desired band.
            Position the input buffer to the start
            of the image data.
            ----------------------------------------*/
            in_ptr = &in_buf[bandoff[curbnd] +
            ((window[SS] - 1) * datasize[idtype])];
            out_ptr = thebuf + (offset[cb] * datasize[idtype]);
            
            /* Copy the data to the output buffer
            -------------------------------------*/
            c_pxcopy(in_ptr,out_ptr,&idtype,&window[NS]);
            
            /* Compute the min/max values if required
            -----------------------------------------*/
            if (*minmax)
                status = c_pxmnmx((long *)out_ptr,&window[NS],&outmin[cb],
		&outmax[cb], &idtype);
            cb +=1;
            }
        
        /* Write the ancillary record to disk
        -------------------------------------*/
        if (wrtdsk[0] && (curbnd == 0))
            {
            in_ptr = &in_buf[2];
            fwrite(in_ptr,1,ancsize,fp);
            }
        }
    
    if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
        return(E_FAIL);
    }

/* Stop the processing message
-----------------------------*/
c_stoptm();

/* Close the image file
-----------------------*/
if (c_egclse((struct GDESC **)&gdesc) != E_SUCC)
    return(E_FAIL);

/* Update the min max file for the output image
-----------------------------------------------*/
if (*minmax)
    if (c_svmnmx((char *)hosout,totbnd,outmin,outmax,"YES") != E_SUCC)
        c_errmsg("Nonfatal error encountered","jscin-nonfatal",&one);

/* Close ancillary file
----------------------*/
if (wrtdsk[0])
    fclose(fp);

/* Free allocated space
-----------------------*/
free(thebuf);
free(in_buf);

return(E_SUCC);
}

