#include "karlov.h"

void getstat
(
    char hosst[],                 /* stas file host name descriptor   */
    long mode,                    /* processing mode                  */
    char class[],                 /* class name                       */
    long num_inbands,             /* number of image bands            */
    float smean[MAXBND],           /* mean vector                      */
    double *cov[]                  /* covariance matrix                */
)
/*
******************************************************************************
NAME: GETSTAT

PURPOSE: Get mean vector and covariance matrix from statistics file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   2.0         16-dec-88   k. a. narayanan    conversion to 5.0
   1.0          7-dec-82   k. a. narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Open statistics file and get number of classes
   If error in open inform user and abort
   Compare stats file dimensions with input image, if specified.
   Allocate dynamic storage for covariance vectors
   If mode is not image 
      If number of classes <= 0  or > 255 inform user and abort
   If mode is total class 
         Get apriori probability for all classes
   If class apriori probabilities are not available 
      Inform user and set class probabilities equal
   Get the mean and covariance of the image, or
      of the specified class, or their average for all classes 
      according to mode
   Expand the covariance vector to full matrix
   If all elements of covariance matrix are zero inform user and abort
   Free dynamic storage for covariance vectors
   Close statistics file

******************************************************************************
*/
{
double	*scov;                           /* covariance matrix in vector form */
double	*tcov;                           /* temp cov vector                  */
static	double	tmean[MAXBND];           /* temp mean vector                 */

float	aprior[MAXCLS];                  /* apriori probabilities            */
float	pk;                       
float	sum;

long	access;                          /* stats file access type           */
long	cov_siz;
long	cshape[9];                       /* covariance shape vector          */
FILE *	fd_sfin;                         /* stats file descriptor            */
long	i, j ,k, k1, k2;                 
long	mshape[9];                       /* mean vector shape vector         */
long	nclass;                  /* number of classes                */
long	nval;
long	shape[9];                        /* probability shape vector         */
long	stat;                            /* return status                    */
long	zero = 0;                        /* constant zero                    */
long    inf_err = 1;

/*
open statistics file
--------------------
*/
access = 0;
if (c_stopen( &fd_sfin, hosst, &access, &nclass) !=  E_OPOLD) 
   {
   if (fd) fclose (fd);
   if (gdesc > 0) c_egclse (&gdesc);
   c_abort (&block, "Error opening statistics file", "karlov-stopen", 
            "karlov", out_files, &num_outfiles);
   }

/*
allocate storage for covariance vectors
---------------------------------------
*/
cov_siz = (long) ((num_inbands * (num_inbands + 1)) * sizeof( double) / 2);
scov = (double *) malloc(cov_siz);
tcov = (double *) malloc(cov_siz);

if (mode  !=  1) 
   if (nclass  ==  0)
      {
      if (fd) fclose (fd);
      if (gdesc > 0) c_egclse (&gdesc);
      c_abort (&block, "No classes found in statistics file", 
               "karlov-nclass", "karlov", out_files, &num_outfiles);
      }

if (mode  ==  3) 
   {
/*
   if all class mode get apriori probabilities from statistics file
   ----------------------------------------------------------------
*/
   k = 1;
   
   while ( (stat = c_stget( &fd_sfin, "PRIOR_PROBABILITY", &k, &zero, shape, 
                            &aprior[k-1])) 
           == E_SUCC  && k <= nclass)  k++;

   if (stat  !=  E_SUCC) 
      {
      c_errmsg( "No apriori probs in stats file; equal values assumed",
               "karlov-apriori", &inf_err);
/*
      if not available set default apriori probabilities
      --------------------------------------------------
*/
      for (k = 0; k < nclass; k++) aprior[k] = 1.0 / nclass;
      }  

   }  

/*
get mean and covariance - check no. of elements
-----------------------------------------------
*/
nval = (num_inbands * (num_inbands + 1))/2;

for (k = 0; k < num_inbands; k++) smean[k] = 0;
for (k = 0; k < nval; k++) scov[k] = 0;

if (mode == 1) 
   {
   k1 = 0; k2 = 0;
   }
else if (mode  ==  2) 
   {
   if ( c_stfdcl( &fd_sfin, class, &k) !=  E_SUCC) 
      {
      if (fd) fclose (fd);
      if (gdesc > 0) c_egclse (&gdesc);
      c_abort (&block, "Class not found in the statistics file", 
               "karlov-noclass", "karlov", out_files, &num_outfiles);
      }

   k1 = k; k2 = k;
   }
else
   {
   k1 = 1; k2 = nclass;
   }  

for (k = k1; k <= k2; k++)
   {
   if (c_stget( &fd_sfin, "MEAN_VECTOR", &k, &zero, mshape, tmean) !=  E_SUCC) 
      {
      if (fd) fclose (fd);
      if (gdesc > 0) c_egclse (&gdesc);
      c_abort (&block, "Mean vector not found in statistics file", 
               "karlov-nomean", "karlov", out_files, &num_outfiles);
      }

   if (k == k1 && mshape[2] != num_inbands) 
      {
      if (fd) fclose (fd);
      if (gdesc > 0) c_egclse (&gdesc);
      c_abort (&block, "Statistics file dimensions do not match with the image",
               "karlov-statdim", "karlov", out_files, &num_outfiles);
      }

   if (mode  ==  3) 
      pk = aprior[k-1];
   else
      pk = 1;

   for (i = 0; i < num_inbands; i++)
      smean[i] += pk * tmean[i];

   if (c_stget (&fd_sfin, "COVARIANCE_MATRIX", &k, &zero, cshape, tcov) != 
                E_SUCC) 
      {
      if (fd) fclose (fd);
      if (gdesc > 0) c_egclse (&gdesc);
      c_abort (&block, "Covariance matrix not found in statistics file", 
               "karlov-nocov", "karlov", out_files, &num_outfiles);
      }

   for (i = 0; i < nval; i++)
      scov[i] += pk * tcov[i];

   }  

/*
expand covariance vector to full matrix
---------------------------------------
*/

for (i = 0, k = 0, sum = 0.0; i < num_inbands; i++)
   {
   for (j = 0; j <= i; j++, k++)
      {
      cov[i][j] = scov[k];
      cov[j][i] = scov[k];
      sum += scov[k];
      }  
   }  

/*
check for all zero values
-------------------------
*/
if (sum  ==  0.0)
   {
   if (fd) fclose (fd);
   if (gdesc > 0) c_egclse (&gdesc);
   c_abort (&block, "Covariance matrix contains all zeros", "karlov-zerocov",
            "karlov", out_files, &num_outfiles);
   }


/*
deallocate storage and close stats file
---------------------------------------
*/
free (scov);
free (tcov);
c_stclse( &fd_sfin); 

}
