/******************************************************************************
NAME:	KARLOV

PURPOSE:
   Performs Karhunen-Loeve transformation on a multi-band image to 
   get its principal components

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   3.4         23-sep-93   D. Bridges         Modified to call calc_scale to 
 					      run the process on the image to 
                                              calc scale and offset if out type
					      is BYTE. 
   3.3         26-mar-93   D. Etrheim         Modified to use the jacobi method
                                              to calculate the eigenvectors
   3.2         23-jul-92   D. Etrheim         Set comb_flag correctly on call
					      to c_upddr.  Write correct proc
					      name to history file.  Change
					      parm name for standards.
   3.1         16-may-91   D. Etrheim         General program cleanup.  Change
                                              to run on the VAX.
   3.0         23-nov-88   k. a. narayanan    conversion to 5.0
   2.0         14-jan-87   d.sullivan         
   1.0          7-dec-82   k. a. narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get TAE parameters
   Call FILSPEC to set up I/O file specifications
   Call DYN_ALLOC to allocate storage for covariance and eigen matrices
   If user wants to enter tranformation matrix and (optionally)
      the mean vector through terminal
         Call USERVEC to prompt the user for the values
   Else
      Call GETSTAT to get the mean vector and covariance matrix
      Call EIGVEC to compute the eigen vectors of the covariance matrix and
         print the information on the specified output device
   If output image is requested
      Call PRCOMP to compute the principal components and output them
   Call DYN_FREE to free dynamic storage
   Call TERMINATE to free storage, copy and update image history, and
      issue completion message 

*****************************************************************************
*/

#define GLOBALVARS /* set this to get appropriate globals from include file */

#include <stdlib.h>
#include "karlov.h"

int main
(void) 
{

double   *cov[MAXBND];            /* pointers to covariance matrix    */
float	*eig_vec[MAXBND],        /* pointers to eigen vectors        */
	*smean,           /* mean vector                      */
        scale,                   /* scale factor for BYTE output    */
        offset;                  /* offset for BYTE output          */

long	components[30];          /* components for out image         */
static	long	image_band_num[MAXIMG][MAXBND + 1];
                                 /* band numbers in each input image */
long    fatal = -1;
long    nonfatal = 1;

long	mode,                    /* processing mode: image, single,  */
                                 /* class, all class                 */
	*num_image_bands, 	/* number of bands in each input image  */
	num_inbands,             /* total number of input image bands*/
	num_inimages,            /* number of input images           */
	num_outbands,            /* number of bands in output image  */
	out_type,                /* output image data type           */
	print[3],                /* print parameter option           */
	sub,                     /* subtract parameter               */
	termtran,                /* transformation file input param  */
	wind[MAXIMG][4];         /* input image windows              */

char	class[CMLEN];            /* class name parameter             */
static	char	hos_in[MAXIMG][CMLEN];
                                 /* input images host names          */
char	hos_out[1][CMLEN];          /* output image host name           */
char	hos_stat[CMLEN];         /* stats file host name             */
static	char	las_in[MAXIMG][CMLEN];   
                                 /* input images las names           */
char	las_out[1][CMLEN];          /* output image las name            */
char	print_file[CMLEN];       /* output print file name           */

/*  Allocate memory
-------------------*/
if ((num_image_bands = (long *) malloc(MAXIMG)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","karlov-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","karlov-fatal",&fatal);
   }
if ((smean = (float *) malloc(MAXBND)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","karlov-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","karlov-fatal",&fatal);
   }

/*
get las parameters
------------------
*/
getpar (las_in, hos_stat, hos_in, las_out, hos_out, &mode, class, 
	&num_inimages, num_image_bands, image_band_num, &num_inbands, 
        &num_outbands, wind, components, &sub, &out_type, 
        &termtran, print, print_file);

/*
allocate dynamic storage for covariance matrix and eigen vectors and
get pointers to their rows
--------------------------------------------------------------------
*/
dyn_alloc( num_inbands, cov, eig_vec);

/*
get transformation (eigen vector) matrix through terminal input if specified
----------------------------------------------------------------------------
*/
if (termtran)                        
   {
   uservec (num_inbands, num_outbands, sub, eig_vec, smean);
   }
else
   {
/*
   otherwise get statistics from statistics file
   ---------------------------------------------
*/
   getstat (hos_stat, mode, class, num_inbands, smean, cov);

/*
   and compute eigen vectors of covariance matrix
   ----------------------------------------------
*/
   eigvec (las_in, mode, smean, cov, num_inbands, eig_vec, class, print);

   }  

/* 
if output data type is byte, run through processing of input image once
to calculate the minimum and maximum pixel values in the output image.
From these minimum and maximum values, calculate the scale and offset
for converting to BYTE output.  The reprocess input image and scale
the output to BYTE using the scale and offset found in the previous step.
A better approach would be to have the PDF run KARLOV with the output
type set to R*4, then run UNARY_SCA to convert the R*4 image to BYTE.
However, this method would use too much drive space.
------------------------------------------------------------------------
*/ 
if (out_type == 1)
   calc_scale( hos_in, &num_inimages, num_image_bands, image_band_num, wind, 
             &num_inbands, &num_outbands, eig_vec, components,
             sub, smean, &scale, &offset);

/*
set up files
------------
*/
if (num_outbands != 0) 
   filspec (hos_in, hos_out[0], &num_inimages, num_image_bands, 
            image_band_num, &num_outbands, wind,out_type);

/*
if requested compute principal components and output them
---------------------------------------------------------
*/
if (num_outbands != 0) 
   prcomp (hos_out[0], &num_inbands, &num_outbands, components, wind, 
           eig_vec,sub, out_type, smean, print, &scale, 
           &offset);

if (fd) 
    fclose (fd);
/*
free storage, update output image history and terminate processing
------------------------------------------------------------------
*/
terminate (las_out[0], hos_out[0], hos_in, &num_inimages, num_outbands, print, 
           print_file);

return EXIT_SUCCESS;

}
