#include "kmeans.h"
#include "protos.h"

void classify
(
    char hos_in[][CMLEN],         /* I: input images host names          */
    char hos_out[],               /* I: output image host name           */
    long *num_inimages,           /* I: number of input images           */
    long num_image_bands[],       /* I: number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* I: band numbers in each        */
    long num_inbands,             /* I: total number of input image bands*/
    float *clmean[],              /* I: class means                      */
    float *clstd[],               /* I: class means                      */
    double *clsumx[], 		  /* I: sums of pixels                   */
    double *clsumsq[],            /* I: sums of squares of pixels        */
    long nclust,                  /* I: no. clusters                     */
    long *maxit,                  /* I: maximum no. iterations           */
    long iview,                   /* I: iteration view interval          */
    double pctcng,                /* I: percentage change threshold      */
    long wind[][4]                /* I: input image windows              */
)
/*
**********************************************************************

NAME: CLASSIFY

PURPOSE:
   Assigns pixels to clusters using the minimum Euclidean distance 
   from cluster means and produces a classified image

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0
    4         19-Oct-99  D. Hames          Add variable num_samps
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call FILSPEC to open input image
   Call SETUP to compute initial image means and initial cluster means
   For maximum number of iterations (MAXIT)
      Call FILSPEC to open input and output images
      Call ASSIGN to assign all pixels in input images to a cluster
      If last iteration or if requested print cluster statistics
      If number of pixels changing cluster assignments is 
         below THRPERC*total number of pixels in one band of input image
         stop iterations and report convergence
   Call SEPAR to compute separability information

**********************************************************************
*/
{
double	out_max, out_min;       /* output image max/min values      */

float	clknt[MAXCLS];          /* count of pixels in each cluster  */
float	pctcng_actual;          /* % pixels changing assignments    */
char	*messg, outbuf[ERRLEN];

long	bands[2];
long	i, idtype, iter, mincng, nchnge, nl, ns, printit = 1;
long    num_samps;		/* total num of samples in input image */
long	wind_out[4];
	
nl = wind[0][NL]; 
ns = wind[0][NS];

/*
**************************************************************
set up file specifications
**************************************************************
*/
iter = 0;
filspec( hos_in, hos_out, num_inimages, num_image_bands,
         image_band_num, wind, &idtype, iter, &num_samps);
/*
**********************************************************************
 compute initial cluster means
**********************************************************************
*/
printf ("\n   ... Computing initial cluster means ...\n");
setup (num_inbands, &nl, &ns, nclust, clmean);
mincng = (long)pctcng * ns *nl;
/*
**********************************************************************
 start iterations
**********************************************************************
*/
printf ("\n   ... Starting iterations ...\n");

for (i = 1; i <= *maxit; i++)
   {
   iter++;
   filspec( hos_in, hos_out, num_inimages, num_image_bands,
            image_band_num, wind, &idtype, iter, &num_samps);

/*
*********************************************************************
  assign pixels to appropriate clusters and compute % changing 
  cluster assignments
*********************************************************************
*/
   assign (i, maxit, clmean, &nchnge, clknt, clsumx, clsumsq, &wind[0][NL],
           &wind[0][NS], nclust, num_inbands, num_samps);

   pctcng_actual = ((float)nchnge/(nl*ns))*100.0;
/*
**********************************************************************
 recompute cluster means and standard deviations.
 determine if stats should be printed - convergence reached, multiple
 of specified iterations reached or maxit reached.
**********************************************************************
*/
   if (iview != 0) printit = iter%iview;
   if (nchnge  <=  mincng) printit = 0;
   if ((iter == *maxit) || (printit == 0)) 
      {
      fprintf (fd, "\f\n\n ***  CLUSTER STATISTICS AFTER ITERATION # %d\n", i);
      }
   else
      {
      fprintf (fd, "\n*** ITERATION # %d\n", i);
      } 
   messg = "Percentage of pixels changing cluster assignments = ";
   fprintf (fd, "%s %7.3f\n\n", messg, pctcng_actual);
   clstats (nclust, clknt, num_inbands, clsumx, clsumsq, clmean, clstd, iter, 
            printit, *maxit);
   sprintf (outbuf, "Iteration # %4d  completed ...", iter);
   c_errmsg (outbuf, " ", &inf_err); 
   sprintf (outbuf, "%s %7.3f\n", messg, pctcng_actual);
   c_errmsg (outbuf, " ", &inf_err); 

/*
**********************************************************************
 check for convergence
**********************************************************************
*/
   if (nchnge  <=  mincng) 
      {
      sprintf (outbuf, "*** Cluster convergence after %4d iterations ***\n",
                i);
      c_errmsg (outbuf, " ", &inf_err);
      fprintf (fd,"%s\n", outbuf);
      break;
      }  
   }  
/*
***************************************************************
  compute separability information
**************************************************************
*/
separ (clmean, clstd, num_inbands, nclust, clknt);

/*
compute and update output image min/max values if needed
--------------------------------------------------------
*/
if (min_max)
   {
   wind_out[SS] = wind_out[SL] = 1;
   wind_out[NS] = ns;
   wind_out[NL] = nl;
   bands[0] = 1;
   bands[1] = 0;
   c_minmax ((char (*)[256])hos_out,&one,(long(*)[4])wind_out,
             (long(*)[257])bands, &one, &out_min, &out_max);
   c_svmnmx (hos_out, &one, &out_min, &out_max, "YES");
   }

}
