/****************************************************************************

NAME: cleanup_files

PURPOSE: Cleanup/delete all the files that were created by this application,
         if the $DELFLG was set.  If $DELFLG was not set then do nothing.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  7.2    10/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:
  This routine loops from the end of the procd_files list when deleting the
  files.  When a file is deleted, the num_procd_files counter gets
  decremented and the procd_files[i] filename is also removed.  This allows
  the updated information to be used if an interrupt occurs while deleting
  these files.

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "l7hdf2las.h"

int cleanup_files
(
  long *num_procd_files,     /* I/O: number of processed files.  This counter
                                     gets decremented when a file has been
                                     deleted. */
  char procd_files[][CMLEN], /* I/O: names of the files that have been created
                                     by this application.  These filenames get
                                     set to NULL after being deleted. */
  int delflg                 /* I: flag to represent the global DELFLG flag
                                   (TRUE or FALSE) */
)

{
  int error_occurred;   /* error flag */
  long i;               /* looping variable */
  long num_files;       /* number of files to be deleted */

  /* If $DELFLG was not set then don't do anything
     ---------------------------------------------*/
  if (!delflg)
    return (E_SUCC);

  /* Delete the files that were created by this application, only if
     $DELFLG was set.  If an error occurs, then set a flag and continue
     deleting the rest of the files.  Loop from the end of the list and
     decrement the num_procd_files counter in case an interrupt occurs
     in the middle of deleting these files.  The filename should be set
     to NULL after the file is deleted.
     ------------------------------------------------------------------*/
  c_lasmsg (FALSE, "l7hdf2las-informational",
    "$DELFLG is set, therefore deleting all output files created by this "
    "application");
  error_occurred = FALSE;
  num_files = *num_procd_files;  /* num_procd_files is decremented in loop */
  for (i = num_files-1; i >= 0; i--)
  {
    if (delete_image (procd_files[i]) != E_SUCC)
    {
      /* delete_image outputs its own error message */
      error_occurred = TRUE;
    }

    /* Set the filename to NULL and decrement the num_procd_files counter
       ------------------------------------------------------------------*/
    strcpy (procd_files[i], "");
    (*num_procd_files)--;
  }

  if (error_occurred)
    return (E_FAIL);
  else
    return (E_SUCC);
}
