/******************************************************************************

NAME: convert_verify_window

PURPOSE: Convert the window from the specified units to line/sample units and
  verify that the window is a valid window.  If no window has been specified
  then set the window to 1,1,nl,ns.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    06/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
  Must be run under TAE.

PROJECT:    LAS

NOTES:
Only the window_units, window, window_specd, nl, and ns will be used for
  windows of L0R and L1R products or with windows in line/sample (LS)
  coordinates.

******************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "imageio.h"   /* SL, SS, NL, NS definitions */
#include "ddr.h"       /* DDR definition */
#include "util.h"      /* c_window_proj_to_ls definition */
#include "cproj.h"     /* DEGREE, DMS, METER definitions */

int convert_verify_window
(
  int window_units,        /* I: Units of input window (L7_DEG, L7_DMS, L7_LS,
                                 L7_PRO) */
  double window[],         /* I: Image window (If window_units is L7_LS then
                                 sl, ss, nl, ns.  If window_units is L7_DMS or
                                 L7_DEG then UL_lat, UL_long, LR_lat, LR_long.
                                 If window_units is L7_PRO then UL_Y, UL_X,
                                 LR_Y, LR_X.) */
  int num_window,          /* I: number of window values */
  int window_specd,        /* I: flag to specify if the window was specified */
  int product_type,        /* I: type of HDF file this is (L0R, L1R, L1G, L1T)*/
  long proj_code,          /* I: integer value to specify the projection */
  long zone_code,          /* I: the UTM zone, for UTM projections */
  long datum_code,         /* I: integer value to specify the datum */
  double pixel_size,       /* I: pixel size for current band */
  double proj_parms[15],   /* I: 15 projection parameters for GCTP */
  double ul_corner[2],     /* I: projection coordinates of upper left corner
                                 in the image (x,y) in units of proj_units */
  double ur_corner[2],     /* I: projection coordinates of upper right corner
                                 in the image (x,y) in units of proj_units */
  double ll_corner[2],     /* I: projection coordinates of lower left corner
                                 in the image (x,y) in units of proj_units */
  double lr_corner[2],     /* I: projection coordinates of lower right corner
                                 in the image (x,y) in units of proj_units */
  char proj_units[],       /* I: units of image projection corners */
  long nl,                 /* I: number of lines in the image for this band */
  long ns,                 /* I: number of samples in the image for this band */
  long valid_window[4]     /* O: array of valid window values in line/sample
                                 coordinates (SL, SS, NL, NS) */
)

{
  char *datasys = NULL;     /* pointer to the DATASYS environment variable */
  int i;                    /* looping variable */
  int intunits;            /* int value of the units
                               DEG=4, DMS=5, PRO=2 (meters) */
  double temp_window[4];    /* temporary window storage */
  struct DDR ddr;           /* DDR to output */

  /* If the window was not specified then set the valid window to 1, 1, nl, ns
     -------------------------------------------------------------------------*/
  if (!window_specd)
  {
    valid_window[SL] = 1;
    valid_window[SS] = 1;
    valid_window[NL] = nl;
    valid_window[NS] = ns;
  }
  else
  {
    /* Copy the window values so that they don't get overwritten
       ---------------------------------------------------------*/
    for (i = 0; i < 4; i++)
      temp_window[i] = window[i];

    /* Don't process the window if the units are L7_LS or if this is an L0R
       or L1R product.
       --------------------------------------------------------------------*/
    if (window_units != L7_LS &&
        ((product_type == L1G) || (product_type == L1T)))
    {
      /* Create a dummy DDR for the current band.  This DDR will be used to
         call c_window_proj_to_ls.
         ------------------------------------------------------------------*/
      strcpy (ddr.proj_units, proj_units);
      ddr.proj_code = proj_code;
      ddr.zone_code = zone_code;
      for (i = 0; i < 15; i++)
        ddr.proj_coef[i] = proj_parms[i];
      ddr.datum_code = datum_code;
      ddr.master_line = 1;
      ddr.master_sample = 1;
      ddr.nl = nl;
      ddr.ns = ns;
      ddr.dtype = EBYTE;  /* unsigned char */
      ddr.upleft[0] = ul_corner[0];
      ddr.upleft[1] = ul_corner[1];
      ddr.upright[0] = ur_corner[0];
      ddr.upright[1] = ur_corner[1];
      ddr.loleft[0] = ll_corner[0];
      ddr.loleft[1] = ll_corner[1];
      ddr.loright[0] = lr_corner[0];
      ddr.loright[1] = lr_corner[1];
      ddr.pdist_x = pixel_size;
      ddr.pdist_y = pixel_size;
      ddr.nbands = 1;  /* only creating info for one band */

      ddr.valid[0] = VALID;  /* projection code */
      ddr.valid[1] = VALID;  /* zone code */
      ddr.valid[2] = VALID;  /* datum code */
      ddr.valid[3] = VALID;  /* proj_coef */
      ddr.valid[4] = VALID;  /* ground units */
      ddr.valid[5] = VALID;  /* ground distance */
      ddr.valid[6] = VALID;  /* corner coordinates */

      /* Get the system from the DATASYS environment variable
         ----------------------------------------------------*/
      datasys = getenv ("DATASYS");
      if (datasys != NULL)
        strncpy (ddr.system, datasys, 12); /* 12 length */
      else
      {
        c_lasmsg (FALSE, "l7hdf2las-datasys",
          "Warning: DATASYS environment variable not set");
        strcpy (ddr.system, "UNKNOWN");
      }

      /* Convert the string window_units value to a long value
         -----------------------------------------------------*/
      if (window_units == L7_DEG)
        intunits = DEGREE;
      else if (window_units == L7_DMS)
        intunits = DMS;
      else if (window_units == L7_PRO)
        intunits = METER;

      /* Convert the window parameters from their units to line/sample.
         --------------------------------------------------------------*/
      if (c_window_proj_to_ls (&ddr, temp_window, &intunits) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-windowprojtols",
          "Error converting window coordinates");
        return (E_FAIL);
      }
    }
    else if ((window_units != L7_LS) && (product_type != L1G) &&
             (product_type != L1T))
    {
      /* window_units != L7_LS and product_type = L0R or L1R then this is an
         invalid option.  Only L1G & L1T products can have window units of
         other than L7_LS.
         -----------------------------------------------------------------*/
        c_lasmsg (FALSE, "l7hdf2las-window_units",
          "L0R and L1R products can only have window units of LS");
        return (E_FAIL);
    }

    /* Copy the temp_window to valid_window
       ------------------------------------*/
    for (i = 0; i < 4; i++)
      valid_window[i] = (long) temp_window[i];

    /* Check the new window
       --------------------*/
    if (window_units == L7_LS && num_window == 2)
    {
      /* If the window_units are L7_LS and the number of window values is 2
         then calculate the number of lines and samples for the window.
         --------------------------------------------------------------------*/
      valid_window[NL] = nl - valid_window[SL];
      valid_window[NS] = ns - valid_window[SS];
    }
    else
    {
      /* If sl or ss are greater than the nl or ns then fatal error
         ----------------------------------------------------------*/
      if (valid_window[SL] > nl)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "The starting line %ld cannot be larger than the number of lines "
          "in the image %ld", valid_window[SL], nl);
        return (E_FAIL);
      }
      if (valid_window[SS] > ns)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "The starting sample %ld cannot be larger than the number of "
          "samples in the image %ld", valid_window[SS], ns);
        return (E_FAIL);
      }

      /* If sl or ss are less than 1 then reset to 1
         -------------------------------------------*/
      if (valid_window[SL] < 1)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Starting line %ld is less than 1 so resetting to 1",
          valid_window[SL]);
        valid_window[SL] = 1;
      }
      if (valid_window[SS] < 1)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Starting sample %ld is less than 1 so resetting to 1",
          valid_window[SS]);
        valid_window[SS] = 1;
      }

      /* If nl or ns extends past the LR corner then reset the nl and ns
         ---------------------------------------------------------------*/
      if (valid_window[SL] + valid_window[NL] > nl)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Window extends past image bounds so the number of lines will be "
          "reset to stop at the LR corner of the image");
        valid_window[NL] = nl - valid_window[SL];
      }
      if (valid_window[SS] + valid_window[NS] > ns)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Window extends past image bounds so the number of samples will "
          "be reset to stop at the LR corner of the image");
        valid_window[NS] = ns - valid_window[SS];
      }
    }
  }

  return (E_SUCC);
}
