/******************************************************************************

NAME: convert_window_for_each_res

PURPOSE: Convert the window from the specified units to line/sample units and
  verify that the window is a valid window.  Do this for each resolution.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
  Must be run under TAE.

PROJECT:    LAS

NOTES:
Only the window_units, window, window_specd, nl, and ns will be used for
  windows of L0R and L1R products or with windows in line/sample (LS)
  coordinates.

******************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "imageio.h"   /* SL, SS, NL, NS definitions */
#include "ddr.h"       /* DDR definition */
#include "diskio.h"    /* c_dkdel definition */

int convert_window_for_each_res
(
  int window_units,       /* I: Units of input window (L7_DEG, L7_DMS, L7_LS,
                                L7_PRO) */
  double window[],        /* I: Image window (If window_units is L7_LS then
                                sl, ss, nl, ns.  If window_units is L7_DMS or
                                L7_DEG then UL_lat, UL_long, LR_lat, LR_long.
                                If window_units is L7_PRO then UL_Y, UL_X,
                                LR_Y, LR_X.) */
  int num_window,         /* I: number of window values */
  int window_specd,       /* I: flag to specify if the window was specified */
  int product_type,       /* I: type of HDF file this is (L0R, L1R, L1G, L1T) */
  long proj_code,         /* I: integer value to specify the projection */
  long zone_code,         /* I: the UTM zone, for UTM projections */
  long datum_code,        /* I: integer value to specify the datum */
  double vis_pixel_size,  /* I: pixel size for visible bands */
  double thm_pixel_size,  /* I: pixel size for thermal bands */
  double pan_pixel_size,  /* I: pixel size for pan bands */
  double proj_parms[15],  /* I: 15 projection parameters for GCTP */
  double ul_corner[2],    /* I: projection coordinates of upper left corner
                                in the image (x,y) in units of proj_units */
  double ur_corner[2],    /* I: projection coordinates of upper right corner
                                in the image (x,y) in units of proj_units */
  double ll_corner[2],    /* I: projection coordinates of lower left corner
                                in the image (x,y) in units of proj_units */
  double lr_corner[2],    /* I: projection coordinates of lower right corner
                                in the image (x,y) in units of proj_units */
  char proj_units[],      /* I: units of image projection corners */
  long vis_nl,            /* I: number of lines in image for visible bands */
  long thm_nl,            /* I: number of lines in image for thermal bands */
  long pan_nl,            /* I: number of lines in image for pan bands */
  long vis_ns,            /* I: number of samples in image for visible bands */
  long thm_ns,            /* I: number of samples in image for thermal bands */
  long pan_ns,            /* I: number of samples in image for pan bands */
  int process_vis,        /* I: flag to specify that visible bands are present
                                (TRUE or FALSE) */
  int process_thm,        /* I: flag to specify that thermal bands are present
                                (TRUE or FALSE) */
  int process_pan,        /* I: flag to specify that pan bands are present
                                (TRUE or FALSE) */
  long vis_valid_window[4], /* O: array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for visible bands */
  long thm_valid_window[4], /* O: array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for thermal bands */
  long pan_valid_window[4]  /* O: array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for pan bands */
)

{
  int i;       /* looping variable */

  if (window_units != L7_LS)
  {
    /* If the window units are not L7_LS, then they need to be converted to
       line/sample coordinates for the existing resolutions.
       --------------------------------------------------------------------*/
    if (process_vis)
    {
      if (convert_verify_window (window_units, window, num_window,
        window_specd, product_type, proj_code, zone_code, datum_code,
        vis_pixel_size, proj_parms, ul_corner, ur_corner, ll_corner, lr_corner,
        proj_units, vis_nl, vis_ns, vis_valid_window) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Error converting/verifying visible band window coordinates");
        return (E_FAIL);
      }
    }

    if (process_thm)
    {
      if (convert_verify_window (window_units, window, num_window,
        window_specd, product_type, proj_code, zone_code, datum_code,
        thm_pixel_size, proj_parms, ul_corner, ur_corner, ll_corner, lr_corner,
        proj_units, thm_nl, thm_ns, thm_valid_window) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Error converting/verifying thermal band window coordinates");
        return (E_FAIL);
      }
    }

    if (process_pan)
    {
      if (convert_verify_window (window_units, window, num_window,
        window_specd, product_type, proj_code, zone_code, datum_code,
        pan_pixel_size, proj_parms, ul_corner, ur_corner, ll_corner, lr_corner,
        proj_units, pan_nl, pan_ns, pan_valid_window) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Error converting/verifying pan band window coordinates");
        return (E_FAIL);
      }
    }
  }  /* if (window_units != L7_LS) */
  else if (window_specd)
  {
    /* The window was specified by the user and it was specified in L7_LS
       coordinates.
       ---------------------------------------------------------------*/
    if (process_vis || (process_thm && process_pan))
    {
      /* The window values were specified for the visible bands.  Verify
         the window values are valid and calculate the window for the
         other resolutions from the values input for the visible bands.
         ---------------------------------------------------------------*/
      if (convert_verify_window (window_units, window, num_window,
        window_specd, product_type, proj_code, zone_code, datum_code,
        vis_pixel_size, proj_parms, ul_corner, ur_corner, ll_corner, lr_corner,
        proj_units, vis_nl, vis_ns, vis_valid_window) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Error converting/verifying visible band window coordinates");
        return (E_FAIL);
      }

      if (product_type == L0R || product_type == L1R)
      {
        /* The resolutions for L0R and L1R are all 30m, 60m, and 15m so
           calculate the other windows using the relationship
           (2N-1, N, 4N-3) for (30m, 60m, 15m).
           ------------------------------------------------------------*/

        /* Calculate the thermal band window from the visible band window
           --------------------------------------------------------------*/
        if (process_thm)
          for (i = 0; i < 4; i++)
            thm_valid_window[i] = (vis_valid_window[i] + 1) / 2;

        /* Calculate the pan band window from the visible band window
           ----------------------------------------------------------*/
        if (process_pan)
          for (i = 0; i < 4; i++)
            pan_valid_window[i] = (2 * vis_valid_window[i]) - 1;
      }
      else
      {
        /* The pixel size for each resolution will have to be used to
           calculate the window for each resolution.  The equation for
           this is window2 = (window1 - 1) * (pixel_size1 / pixel_size2) + 1
           -----------------------------------------------------------------*/

        /* Calculate the thermal band window from the visible band window
           --------------------------------------------------------------*/
        if (process_thm)
          for (i = 0; i < 4; i++)
            thm_valid_window[i] = (long) ((float) (vis_valid_window[i] - 1) *
              (vis_pixel_size / thm_pixel_size) + 1.0);

        /* Calculate the pan band window from the visible band window
           --------------------------------------------------------------*/
        if (process_pan)
          for (i = 0; i < 4; i++)
            pan_valid_window[i] = (long) ((float) (vis_valid_window[i] - 1) *
              (vis_pixel_size / pan_pixel_size) + 1.0);
      }
    }  /* if (process_vis || (process_thm && process_pan)) */
    else if (process_thm)
    {
      /* The window values were specified for the thermal bands.  Verify the
         window values are valid.  Only the thermal bands will be processed.
         -------------------------------------------------------------------*/
      if (convert_verify_window (window_units, window, num_window,
        window_specd, product_type, proj_code, zone_code, datum_code,
        thm_pixel_size, proj_parms, ul_corner, ur_corner, ll_corner, lr_corner,
        proj_units, thm_nl, thm_ns, thm_valid_window) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Error converting/verifying thermal band window coordinates");
        return (E_FAIL);
      }
    }
    else
    {
      /* The window values were specified for the pan bands.  Verify the
         window values are valid.  Only the pan bands will be processed.
         ---------------------------------------------------------------*/
      if (convert_verify_window (window_units, window, num_window,
        window_specd, product_type, proj_code, zone_code, datum_code,
        pan_pixel_size, proj_parms, ul_corner, ur_corner, ll_corner, lr_corner,
        proj_units, pan_nl, pan_ns, pan_valid_window) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-window",
          "Error converting/verifying pan band window coordinates");
        return (E_FAIL);
      }
    }
  }  /* else if (window_specd) */
  else
  {
    /* The window was not specified so fill in the window with the entire
       image
       ------------------------------------------------------------------*/
    if (process_vis)
    {
      vis_valid_window[SL] = 1;
      vis_valid_window[SS] = 1;
      vis_valid_window[NL] = vis_nl;
      vis_valid_window[NS] = vis_ns;
    }

    if (process_thm)
    {
      thm_valid_window[SL] = 1;
      thm_valid_window[SS] = 1;
      thm_valid_window[NL] = thm_nl;
      thm_valid_window[NS] = thm_ns;
    }

    if (process_pan)
    {
      pan_valid_window[SL] = 1;
      pan_valid_window[SS] = 1;
      pan_valid_window[NL] = pan_nl;
      pan_valid_window[NS] = pan_ns;
    }
  }

  return (E_SUCC);
}
