/****************************************************************************

NAME: create_links

PURPOSE: Create an upper-case filename link to the HDF and band filenames on
  the current CD.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    11/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include <unistd.h>
#include "l7hdf2las.h"
#include "cm.h"    /* c_getfiles definition */
#include "util.h"  /* c_splitname definition */

int create_links
(
  char infile[],             /* I/O: HDF file name (Unix format)
                                Note: infile gets renamed to point to the
                                linked HDF file name */
  char output_dir[],         /* I: directory name of the output file */
  int product_type,        /* I: type of HDF file this is (L0R, L1R, L1G, L1T)*/
  char link_files[][CMLEN],  /* O: array of filenames created for links */
  int *num_link_files        /* O: number of link files created */
)

{
  char directory[CMLEN];  /* directory portion of input HDF filename */
  char extension[CMLEN];  /* extension portion of input HDF file name */
  char found_files[MAXFILE][CMLEN];
                          /* list of filenames found in the wildcard search */
  char root[CMLEN];       /* root portion of input HDF file name */ 
  char root_tmp[CMLEN];   /* root portion of input HDF file name (temp) */
  char srchfile[CMLEN];   /* wildcard file to search for */
  char *tmp_ptr;          /* pointer to the last '_' in the HDF filename */

  int i;                  /* looping variables */
  int root_len;           /* string length of the root */
  int nfiles;             /* number of files found in the wildcard search */
  long strsize;           /* string length of the string */

  /* Split input HDF filename into directory, root, and extension
     ------------------------------------------------------------*/
  c_splitname (infile, directory, root, extension);

  /* If this is an L0R product then insert "[12]" at the 7th character of
     the root so that we search for both format 1 and format 2 filenames.
     L7XsssfnYYDOYHHuuv_xxx has 22 characters.
     --------------------------------------------------------------------*/
  if (product_type == L0R)
  {
    root_len = strlen (root);
    if (root_len != 22)
    {
      c_lasmsg (FALSE, "l7hdf2las-l0rname",
        "L0R root name %s must be of the form L7XsssfnYYDOYHHuuv_xxx", root);
      return (E_FAIL);
    }

    /* Copy the first 6 characters
       ---------------------------*/
    strncpy (root_tmp, root, 6);
    root_tmp[6] = '\0';

    /* Put in the format 1 and 2 wildcard.  The next digit can change, so
       use the ? for any character.
       ------------------------------------------------------------------*/
    strcat (root_tmp, "[12]?");

    /* Copy the rest of root (except the 7th and 8th characters) into
       root_tmp.  Recall we have added 3 extra characters to the root.
       ---------------------------------------------------------------*/
    for (i = 8; i < root_len; i++)
      root_tmp[i+3] = root[i];
    root_tmp[root_len+3] = '\0';

    /* Put root_tmp back into root
       ---------------------------*/
    strcpy (root, root_tmp);
  }

  /* If this is an L1R, L1G, or L1T product then insert "[12]" at the 3rd
     character of the root so that we search for both format 1 and format 2
     filenames.  L7fppprrr_rrrYYYYMMDD_AAA has 25 characters.
     ----------------------------------------------------------------------*/
  else if ((product_type == L1R) || (product_type == L1G) ||
           (product_type == L1T))
  {
    root_len = strlen (root);
    if (root_len != 25)
    {
      c_lasmsg (FALSE, "l7hdf2las-l1rgname",
       "L1R/L1G/L1T root name %s must be of the form L7fppprrr_rrrYYYYMMDD_AAA",
        root);
      return (E_FAIL);
    }

    /* Copy the first 2 characters
       ---------------------------*/
    strncpy (root_tmp, root, 2);
    root_tmp[2] = '\0';

    /* Put in the format 1 and 2 wildcard
       ----------------------------------*/
    strcat (root_tmp, "[12]");

    /* Copy the rest of root (except the 3rd character) into root_tmp.
       Recall we have added 3 extra characters to the root.
       ---------------------------------------------------------------*/
    for (i = 3; i < root_len; i++)
      root_tmp[i+3] = root[i];
    root_tmp[root_len+3] = '\0';

    /* Put root_tmp back into root
       ---------------------------*/
    strcpy (root, root_tmp);
  }

  /* Set up the search wildcard.  Strip any characters after the last
     occurrence of '_' in srchfile and append a '*' after the last occurrence
     of the '_' in srchfile to find all the files for this product.
     Then concatenate the directory and root.  The search for '_' should be
     done before the root and directory are concatinated since an '_' may
     not exist in the root but may be incorrectly found in the directory.
     -----------------------------------------------------------------------*/
  tmp_ptr = strrchr (root, '_');
  if (tmp_ptr == NULL)
  {
    c_lasmsg (FALSE, "l7hdf2las-HDFfilename", "Invalid HDF filename format");
    return (E_FAIL);
  }
  tmp_ptr[0] = '*';    /* Replace '_' with '*' */
  tmp_ptr[1] = '\0';   /* End the string */
  strcpy (srchfile, directory);
  strcat (srchfile, root);

  /* Get the filenames that match the srchfile string
     ------------------------------------------------*/
  if (c_getfiles (srchfile, found_files, &nfiles, MAXFILES, 1 /*first call*/,
      0 /*return list of all files found*/) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-getfiles", "Error searching for _vol files");
    return (E_FAIL);
  }

  /* Loop through all the found files and search for _b##/_B## or _HDF/_hdf
     ----------------------------------------------------------------------*/
  *num_link_files = 0;
  for (i = 0; i < nfiles; i++)
  {
    /* Split input HDF filename into directory, root, and extension
       ------------------------------------------------------------*/
    c_splitname (found_files[i], directory, root, extension);

    /* Find the last occurrence of '_' in the current file
       ---------------------------------------------------*/
    tmp_ptr = strrchr (root, '_');
    if (tmp_ptr != NULL)
    {
      /* Make upper-case link files in the current directory for all the
         band files (or HDF file) that were found on this CD.
         ---------------------------------------------------------------*/
      if (!strncasecmp (tmp_ptr, "_b", 2) || !strncasecmp (tmp_ptr, "_hdf", 4))
      {
        /* Convert the root and extension to upper case
           --------------------------------------------*/
        strsize = (long) strlen (root);
        c_low2up (root, &strsize);
        strsize = (long) strlen (extension);
        c_low2up (extension, &strsize);

        /* Create the link filename in the output directory
           ------------------------------------------------*/
        sprintf (link_files[*num_link_files], "%s%s.%s", output_dir, root,
          extension);

        /* Link the link filename to the same file on the CD
           -------------------------------------------------*/
        if (symlink (found_files[i], link_files[*num_link_files]) != 0)
        {
          c_lasmsg (FALSE, "l7hdf2las-link", "Error linking %s to %s",
            found_files[i], link_files[*num_link_files]);
          return (E_FAIL);
        }

        /* If this is the hdf file, then copy the link filename to infile
           --------------------------------------------------------------*/
        if (!strncasecmp (tmp_ptr, "_hdf", 4))
          strcpy (infile, link_files[*num_link_files]);

        (*num_link_files)++;
      }  /* end if */
    }  /* end if */
  }  /* end for */

  return (E_SUCC);
}
