/****************************************************************************

NAME: find_infilename

PURPOSE: When the user is processing imagery from the CD and does not
         specify an input filename to process, we need to search the CD
         for a valid HDF filename.  This could be an L0R, L1R, or L1G
         product name.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

NOTES:
  If more than one valid HDF filename is found on the CD, only the first
  HDF filename will be processed.  The other filenames will be printed in
  a warning message to the user.
  It is assumed that the CD is already mounted.

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "las.h"        /* CMLEN for tapeio routines */
#include "tapeio.h"     /* tape routines and defines */
#include "util.h"       /* c_up2low definition */

int find_infilename
(
  char drivename[],       /* I: name of the CD drive in Unix format */
  char filename[],        /* I: file name or partial file name to look for */
  char hdfname[]          /* O: name of the HDF file found on the CD */
)

{
  char found_files[MAXFILE][CMLEN];
                         /* array of HDF filenames found on this CD */
  char srchname[CMLEN];  /* wildcard name of the HDF file to search for */
  int drive_length;      /* length of the drivename string */
  int index;             /* looping variable */
  int nfiles;            /* number of HDF files found on this CD */
  char local_filename[CMLEN]; /* local version of the file name, possibly 
                                 modified from the filename passed in */

  /* determine whether the user entered a partial file name */
  if (filename[0] != '\0')
  {
    char dir[CMLEN],root[CMLEN],ext[CMLEN];

    /* the filename was not empty, so at least a partial file name was
       specified.  Now figure out how much was specified. */
    c_splitname(filename,dir,root,ext);
    if (ext[0] == '\0')
    {
      char *ptr;
      /* no extension specified, so partial filename specified.  Strip off 
         any part of the _hdf or _HDF that is on the file name to allow the
         search to add it cleanly. */
      ptr = strstr(root,"_h");
      if (ptr == NULL)
        ptr = strstr(root,"_H");
      if (ptr != NULL)
      {
        /* start of _hdf or _HDF found, so strip it off */
        *ptr = '\0';
      }
      sprintf(local_filename,"%s%s",dir,root);
    }
    else
    {
      /* full filename specified, so no search required. */
      sprintf(hdfname,"%s%s",drivename,filename);
      if ((access (hdfname, F_OK) != 0))
      {
        c_lasmsg (FALSE,"l7hdf2las-infile", 
                  "Error: Image file %s does not exist", hdfname);
        return (E_FAIL);
      }
      return (E_SUCC);
    }
  }
  else
  {
    /* no filename entered, so default the local filename to be a pattern 
       match for L7 or l7 */
    strcpy(local_filename,"[Ll]7");
  }

  /* Search for the L1R and L1G HDF file first
     -----------------------------------------*/
  sprintf (srchname, "%s%s*_[Hh][Dd][Ff].[Ll]1[GgRr]", drivename, 
           local_filename);
  if (c_getfiles (srchname, found_files, &nfiles, MAXFILE, 1 /* first call
    to this function */, 0 /* return list of all files found */) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-getfiles",
      "Error finding the L1R/L1G HDF files at %s%s",drivename,filename);
    return (E_FAIL);
  }

  if (nfiles >= 1)
  {
    /* An L1R or L1G HDF file was found.  Strip off the drivename when
       returning the found HDF filename.  In the case of multiple HDF
       files found, only the first filename will be returned.
       ---------------------------------------------------------------*/
    drive_length = strlen (drivename);
    strcpy (hdfname, &found_files[0][drive_length]);
  }
  else
  {
    /* Search for the L0R HDF file
       ---------------------------*/
    sprintf (srchname, "%s%s*_[Hh][Dd][Ff].*", drivename,local_filename);
    if (c_getfiles (srchname, found_files, &nfiles, MAXFILE, 1 /* first call
      to this function */, 0 /* return list of all files found */) != E_SUCC)
    {
      c_lasmsg (FALSE, "l7hdf2las-getfiles",
        "Error finding the L0R HDF files at %s%s",drivename,filename);
      return (E_FAIL);
    }

    if (nfiles >= 1)
    {
      /* An L0R HDF file was found.  Strip off the drivename when
         returning the found HDF filename.  In the case of multiple HDF
         files found, only the first filename will be returned.
         ---------------------------------------------------------------*/
      drive_length = strlen (drivename);
      strcpy (hdfname, &found_files[0][drive_length]);
    }
    else
    {
      /* No L0R, L1R, or L1G HDF file was found
         --------------------------------------*/
      c_lasmsg (FALSE, "l7hdf2las-find_infilename",
        "Error: No L0R, L1R, or L1G HDF file was found at %s%s",drivename,
        filename);
      return (E_FAIL);
    }
  }

  /* If more than one HDF file was found on this CD, then we will only
     process the first HDF file found.  A warning message will be output
     to the user in the case of multiple HDF files on the current CD and
     the filenames of the unprocessed files will also be output.
     -------------------------------------------------------------------*/
  if (nfiles > 1)
  {
    c_lasmsg (FALSE, "l7hdf2las-find_infilename",
      "Warning: More than one matching HDF file was found.  The following "
      "files will not be processed:");
    for (index = 1 /*skip the first file*/; index < nfiles; index++)
      c_lasmsg (FALSE, "l7hdf2las-find_infilename", found_files[index]);
  }

  return (E_SUCC);
}
