/****************************************************************************

NAME: get_lcc_projparms

PURPOSE: Fill in the necessary 15 projection parameters from the LPGS
         metadata for the Lambert Conformal Conic projection.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "geompak.h"     /* c_getdatum function definition */

int get_lcc_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
)

{
  char units[10];       /* false easting/northing units */
  double centmer;       /* longitude of center of meridian */
  double false_east;    /* false easting value from metadata */
  double false_north;   /* false northing value from metadata */
  double originlat;     /* latitude of projection origin */
  double stdpr1;        /* latitude of first standard parallel */
  double stdpr2;        /* latitude of second standard parallel */
  struct DATUMDEF dtm_info;
                        /* structure for the datum information */

  /* Get necessary values from the LPGS Metadata
     -------------------------------------------*/
  strncpy (units, LPGSMetaDataFile->Parameters.u.lcc.FalseEastingNorthingUnits,
    sizeof (units) - 1);
  units[sizeof (units) - 1] = '\0';
  stdpr1 = LPGSMetaDataFile->Parameters.u.lcc.LatitudeOfFirstStandardParallel;
  stdpr2 = LPGSMetaDataFile->Parameters.u.lcc.LatitudeOfSecondStandardParallel;
  centmer = LPGSMetaDataFile->Parameters.u.lcc.LongitudeOfCentrolMeridian;
  originlat = LPGSMetaDataFile->Parameters.u.lcc.LatitudeOfProjectionOrigin;
  false_east = LPGSMetaDataFile->Parameters.u.lcc.FalseEasting;
  false_north = LPGSMetaDataFile->Parameters.u.lcc.FalseNorthing;

  /* Convert the units to meters since that is what the semi major and semi
     minor axis will be calculated in.  false_east, false_north, semi_major,
     and semi_minor must all be of the same units
     -----------------------------------------------------------------------*/
  if (!strcmp (units, "feet"))
  {
    /* Convert false_east from feet to meters
       --------------------------------------*/
    false_east *= FEET2MTRS;

    /* Convert false_north from feet to meters
       ---------------------------------------*/
    false_north *= FEET2MTRS;
  }

  /* Fill in the projection paramters
     --------------------------------*/
  proj_parms[0] = 0.0;
  proj_parms[1] = 0.0;
  if (c_degdms(&stdpr1,&proj_parms[2],"DEG","LAT") != E_SUCC)
  {
    c_lasmsg(FALSE,"l7hdf2las-projparms",
             "Error parsing the first standard parallel projection parameter");
    return (E_FAIL);
  }
  if (c_degdms(&stdpr2,&proj_parms[3],"DEG","LAT") != E_SUCC)
  {
    c_lasmsg(FALSE,"l7hdf2las-projparms",
             "Error parsing the second standard parallel projection parameter");
    return (E_FAIL);
  }
  if (c_degdms(&centmer,&proj_parms[4],"DEG","LON") != E_SUCC)
  {
    c_lasmsg(FALSE,"l7hdf2las-projparms",
             "Error parsing the central meridian projection parameter");
    return (E_FAIL);
  }
  if (c_degdms(&originlat,&proj_parms[5],"DEG","LAT") != E_SUCC)
  {
    c_lasmsg(FALSE,"l7hdf2las-projparms",
             "Error parsing the latitude of origin projection parameter");
    return (E_FAIL);
  }
  proj_parms[6] = false_east;
  proj_parms[7] = false_north;
  proj_parms[8] = 0.0;
  proj_parms[9] = 0.0;
  proj_parms[10] = 0.0;
  proj_parms[11] = 0.0;
  proj_parms[12] = 0.0;
  proj_parms[13] = 0.0;
  proj_parms[14] = 0.0;

  /* Put the semi-major and semi-minor axis in proj_parms
     ----------------------------------------------------*/
  dtm_info.datumnum = datum_code;
  if (c_getdatum (&dtm_info) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-getdatum", "Error getting the datum info");
    return (E_FAIL);
  }
  proj_parms[0] = dtm_info.smajor;
  proj_parms[1] = dtm_info.sminor;

  return (E_SUCC);
}
