/****************************************************************************

NAME: get_om_projparms

PURPOSE: Fill in the necessary 15 projection parameters from the LPGS
         metadata for the Oblique Mercator projection.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "geompak.h"     /* c_getdatum function definition */

int get_om_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
)

{
  char om_type[2];      /* OM type (A or B) */
  char units[10];       /* false easting/northing units */
  double aziang;        /* angle of azimuth */
  double azmthpt;       /* longitude of along projection */
  double factor;        /* scale factor at center of meridian */
  double false_east;    /* false easting value from metadata */
  double false_north;   /* false northing value from metadata */
  double lat1;          /* latitude of first geodetic point */
  double lat2;          /* latitude of second geodetic point */
  double long1;         /* longitude of first geodetic point */
  double long2;         /* longitude of second geodetic point */
  double originlat;     /* latitude of projection origin */
  struct DATUMDEF dtm_info;
                        /* structure for the datum information */

  /* Get necessary values from the LPGS Metadata
     -------------------------------------------*/
  strncpy (units, LPGSMetaDataFile->Parameters.u.om.FalseEastingNorthingUnits,
    sizeof (units) - 1);
  units[sizeof (units) - 1] = '\0';
  factor = LPGSMetaDataFile->Parameters.u.om.ScaleFactorAtCenterOfProjection;
  originlat = LPGSMetaDataFile->Parameters.u.om.LatitudeOfProjectionOrigin;
  false_east = LPGSMetaDataFile->Parameters.u.om.FalseEasting;
  false_north = LPGSMetaDataFile->Parameters.u.om.FalseNorthing;
  strncpy (om_type, LPGSMetaDataFile->Parameters.u.om.OM_Type,
    sizeof (om_type) - 1);
  om_type[sizeof (om_type) - 1] = '\0';


  /* Convert the units to meters since that is what the semi major and semi
     minor axis will be calculated in.  false_east, false_north, semi_major,
     and semi_minor must all be of the same units
     -----------------------------------------------------------------------*/
  if (!strcmp (units, "feet"))
  {
    /* Convert false_east from feet to meters
       --------------------------------------*/
    false_east *= FEET2MTRS;

    /* Convert false_north from feet to meters
       ---------------------------------------*/
    false_north *= FEET2MTRS;
  }

  /* Fill in the projection paramters
     --------------------------------*/
  proj_parms[0] = 0.0;
  proj_parms[1] = 0.0;
  proj_parms[2] = factor;
  proj_parms[3] = 0.0;
  proj_parms[4] = 0.0;
  if (c_degdms(&originlat,&proj_parms[5],"DEG","LAT") != E_SUCC)
  {
    c_lasmsg(FALSE,"l7hdf2las-projparms",
             "Error parsing latitude of origin projection parameter");
    return (E_FAIL);
  }
  proj_parms[6] = false_east;
  proj_parms[7] = false_north;
  proj_parms[8] = 0.0;
  proj_parms[9] = 0.0;
  proj_parms[10] = 0.0;
  proj_parms[11] = 0.0;
  proj_parms[12] = 0.0;
  proj_parms[13] = 0.0;
  proj_parms[14] = 0.0;

  /* Put the semi-major and semi-minor axis in proj_parms
     ----------------------------------------------------*/
  dtm_info.datumnum = datum_code;
  if (c_getdatum (&dtm_info) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-getdatum", "Error getting the datum info");
    return (E_FAIL);
  }
  proj_parms[0] = dtm_info.smajor;
  proj_parms[1] = dtm_info.sminor;

  if (!strcmp (om_type, "A") || !strcmp (om_type, "A"))
  { /* om_type = "A" */
    long1 = LPGSMetaDataFile->Parameters.u.om.s.oma.LongitudeFirstPointGeodetic;
    lat1 = LPGSMetaDataFile->Parameters.u.om.s.oma.LatitudeFirstPointGeodetic;
    long2 =LPGSMetaDataFile->Parameters.u.om.s.oma.LongitudeSecondPointGeodetic;
    lat2 = LPGSMetaDataFile->Parameters.u.om.s.oma.LatitudeSecondPointGeodetic;

    if (c_degdms(&long1,&proj_parms[8],"DEG","LON") != E_SUCC)
    {
      c_lasmsg(FALSE,"l7hdf2las-projparms","Error parsing the longitude of "
               "first center line point projection parameter");
      return (E_FAIL);
    }
    if (c_degdms(&lat1,&proj_parms[9],"DEG","LAT") != E_SUCC)
    {
      c_lasmsg(FALSE,"l7hdf2las-projparms","Error parsing the latitude of "
               "first center line point projection parameter");
      return (E_FAIL);
    }
    if (c_degdms(&long2,&proj_parms[10],"DEG","LON") != E_SUCC)
    {
      c_lasmsg(FALSE,"l7hdf2las-projparms","Error parsing the longitude of "
               "second center line point projection parameter");
      return (E_FAIL);
    }
    if (c_degdms(&lat2,&proj_parms[11],"DEG","LAT") != E_SUCC)
    {
      c_lasmsg(FALSE,"l7hdf2las-projparms","Error parsing the latitude of "
               "second center line point projection parameter");
      return (E_FAIL);
    }
  }
  else
  { /* om_type = "B" */
    aziang = LPGSMetaDataFile->Parameters.u.om.s.omb.AngleOfAzimuth;
    azmthpt = LPGSMetaDataFile->Parameters.u.om.s.omb.LongitudeAlongProjection;

    /* note that aziang is between 0 and 360 degrees */
    if (c_degdms(&aziang,&proj_parms[3],"DEG","DEGREES") != E_SUCC)
    {
      c_lasmsg(FALSE,"l7hdf2las-projparms",
               "Error parsing the azimuth angle projection parameter");
      return (E_FAIL);
    }
    if (c_degdms(&azmthpt,&proj_parms[4],"DEG","LON") != E_SUCC)
    {
      c_lasmsg(FALSE,"l7hdf2las-projparms",
               "Error parsing the azimuth point projection parameter");
      return (E_FAIL);
    }
    proj_parms[12] = 1.0;
  }

  return (E_SUCC);
}
