/****************************************************************************

NAME: get_parms

PURPOSE: Input the user parameters and verify that they are valid.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include <unistd.h>      /* access definition */
#include "l7hdf2las.h"
#include "imageio.h"     /* SL, SS, NL, NS definitions */
#include "las.h"
#include "util.h"        /* c_splitname and c_get_multiple_out definitions */

int get_parms
(
  struct PARBLK *vblock,  /* I: pointer to TAE parameter block */
  int max_out,            /* I: maximum number of output files allowed in
                                outfile */
  char comment[][CMLEN],  /* O: array of user comments for the operator */
  int *mediaflag,         /* O: flag to specify DISK or CD */
  int *num_vols,          /* O: number of CD volumes to process */
  char infile[CMLEN],     /* O: input file name (UNIX format) */
  int bands[],            /* O: array of bands to process */
  int *num_bands,         /* O: number of values in bands */
  int *default_bands,     /* O: flag to specify if this application will
                                process the default set of bands (TRUE =
                                process the default bands, FALSE = bands were
                                specified) */
  int nbands[],           /* O: array of NBANDS values */
  int *num_nbands,        /* O: number of values in nbands */
  int *window_units,      /* O: units of input window (L7_DEG, L7_DMS, L7_LS,
                                L7_PRO) */
  double window[],        /* O: Image window (If window_units is L7_LS then
                                sl, ss, nl, ns.  If window_units is L7_DMS or
                                L7_DEG then UL_lat, UL_long, LR_lat, LR_long.
                                If window_units is L7_PRO then UL_Y, UL_X,
                                LR_Y, LR_X.) */
  int *num_window,        /* O: number of window values */
  int *window_specd,      /* O: flag to specify if the window was specified */
  char outfile[][CMLEN],  /* O: array of output file names from OUT (Unix
                                format) */
  int *num_out,           /* O: number of output file names */
  int *many_to_one,       /* O: flag to specify if all bands will be written
                                to one output file */
  long *odtype,           /* O: data type of output image
                                (0=SAME, EBYTE=BYTE, EWORD=I*2, ELONG=I*4,
                                 EREAL=R*4) */
  int nbands_same_res[],  /* O: array of same_res values, one for each nbands
                                value, each to specify if all the bands in the
                                nbands group are of the same resolution (TRUE,
                                FALSE) */
  int *process_vis,       /* O: flag specifying visible bands will be
                                processed (TRUE or FALSE) */
  int *process_thm,       /* O: flag specifying thermal bands will be
                                processed (TRUE or FALSE) */
  int *process_pan,       /* O: flag specifying pan bands will be processed
                                (TRUE or FALSE) */
  int *delflg,            /* O: flag to represent the global DELFLG flag
                                (TRUE or FALSE) */
  long *msgtim,           /* O: time between display messages */
  char minmax[],          /* O: minmax flag */
  char processing_system[]/* O: processing system for input files, IAS, LPGS
                                NLAPS or NONE provided by user */
)

{
  char directory[CMLEN];     /* directory portion of input HDF filename */
  char extension[CMLEN];     /* extension portion of input HDF file name */
  char err_text[CMLEN];      /* text string for error messages */
  char file_exts[3][5] = {".img", ".ddr", ".his"}; /* file extensions */
  char multi_res_combs[8][13] = {"", "_vis", "_thm", "_pan", "_vis_thm",
    "_vis_pan", "_thm_pan", "_vis_thm_pan"};
                             /* combinations for multiple resolution imagery */
  char outname[CMLEN];       /* output name with resolution extension */
  char root[CMLEN];          /* root portion of input HDF file name */
  char tmp_str[CMLEN];       /* temporary string holder */

  int band_count;            /* band counter for looping */
  int bands_present[9];      /* array of bands present in this nbands group */
  int group_start;           /* index for the start of the next band group */
  int index_nb, index_b;     /* index for nbands and bands */
  int index,index2,index3;   /* looping variables */
  int num_file_exts = 3;     /* number of file_exts values */
  int num_multi_res_combs = 8; /* number of multi_res_combs values */
  int sum;                   /* sum of the nbands values */
  int tmp_same_res;          /* temporary same_res flag */

  struct VARIABLE *var;      /* pointer to a variable in TAE block */

  /* Get the COMMENTs
     ----------------*/
  var = p_find (vblock, "COMMENT");
  if (var->v_count == 0)
  {
    /* Processing from a disk
       ----------------------*/
    *mediaflag = DISKMEDIA;
    *num_vols = 1;
  }
  else
  {
    /* Processing from a CD
       --------------------*/
    *mediaflag = CDMEDIA;
    *num_vols = var->v_count;
  }
  for (index = 0; index < var->v_count; index++)
    strcpy (comment[index], SVAL (*var, index));

  /* Get the input image (INFILE)
     ----------------------------*/
  var = p_find (vblock, "INFILE");
  if (var->v_count > 0)
  {
    /* The input filename was specified
       --------------------------------*/
    strcpy (infile, SVAL (*var, 0));

    /* if media is not an ops mounted CD and the infile name is a directory, 
       append a '/' at the end to force a search of the directory */
    if (*mediaflag != CDMEDIA)
    {
      if (isdir(infile))
        strcat(infile,"/");
    }
  }

  /* Get the image bands to be processed (BANDS)
     -------------------------------------------*/
  var = p_find (vblock, "BANDS");
  *num_bands = var->v_count;
  if (*num_bands == 0)
  {
    /* Use the default bands for this image
       ------------------------------------*/
    *default_bands = TRUE;
    *process_vis = TRUE;
    *process_thm = TRUE;
    *process_pan = TRUE;
  }
  else
  {
    /* The user specified bands for this image
       ---------------------------------------*/
    *default_bands = FALSE;

    /* Verify that the bands are valid bands (1, 2, 3, 4, 5, 61, 62, 7, 8)
       Also determine which resolutions will be processed
       -------------------------------------------------------------------*/
    *process_vis = FALSE;
    *process_thm = FALSE;
    *process_pan = FALSE;
    for (index = 0; index < *num_bands; index++)
    {
      bands[index] = IVAL (*var, index);
      switch (bands[index])
      {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 7:
          *process_vis = TRUE;
          break;

        case 6:
          *process_thm = TRUE;
          /* band 6 is the same as 61 */
          bands[index] = 61;
          break;

        case 61:
        case 62:
          *process_thm = TRUE;
          break;

        case 9:
          *process_thm = TRUE;
          /* band 9 is the same as 62 */
          bands[index] = 62;
          break;

        case 8:
          *process_pan = TRUE;
          break;

        default:
          sprintf (err_text, "Error: Band %d is invalid", bands[index]);
          c_lasmsg (FALSE, "l7hdf2las-bands", err_text);
          return (E_FAIL);
      }
    }
  }

  /* Only input the NBANDS field if BANDS was specified
     --------------------------------------------------*/
  if (!(*default_bands))
  {
    var = p_find (vblock, "NBANDS");
    *num_nbands = var->v_count;
    if (*num_nbands > 0)
    {
      /* Verify the sum of the NBANDS values adds up to the number of BANDS
         ------------------------------------------------------------------*/
      sum = 0;
      for (index = 0; index < *num_nbands; index++)
      {
        nbands[index] = IVAL (*var, index);
        sum += nbands[index];
      }

      if (sum != *num_bands)
      {
        c_lasmsg (FALSE, "l7hdf2las-nbands",
         "The nbands values must sum to the total number of bands");
        return (E_FAIL);
      }
    }
  }
  else
  {
    /* Set num_nbands to 0 since we won't be reading them
       --------------------------------------------------*/
    *num_nbands = 0;
  }

  /* Get the image window (WINDOW)
     -----------------------------*/
  var = p_find (vblock, "WINDOW");
  *num_window = var->v_count;

  if (*num_window == 0)
  {
    /* The window was not specified.  We should process all lines and samples
       in the image, for each different resolution.  Also, if the window was
       not specified, then the UNITS value must be set to LS regardless of
       what the user specified for this value.
       ----------------------------------------------------------------------*/
    *window_specd = FALSE;
    *window_units = L7_LS;
  }
  else
  {
    *window_specd = TRUE;
    for (index = 0; index < *num_window; index++)
      window[index] = RVAL (*var, index);

    /* Get the window units (UNITS)
       ----------------------------*/
    var = p_find (vblock, "UNITS");
    strcpy (tmp_str, SVAL(*var, 0));
    if (!strncmp (tmp_str, "DEG", 3))
      *window_units = L7_DEG;
    else if (!strncmp (tmp_str, "DMS", 3))
      *window_units = L7_DMS;
    else if (!strncmp (tmp_str, "LS", 2))
      *window_units = L7_LS;
    else if (!strncmp (tmp_str, "PRO", 3))
      *window_units = L7_PRO;
    else
    {
      sprintf (err_text,
        "Invalid units value: %s.  Only DEG, DMS, LS, and PRO are allowed",
        tmp_str);
      c_lasmsg (FALSE, "l7hdf2las-units", err_text);
      return (E_FAIL);
    }

    if (*window_units == L7_LS && (*num_window != 2 && *num_window != 4))
    {
      c_lasmsg (FALSE, "l7hdf2las-units",
        "Error: Invalid window specification for the specified UNITS of LS");
      return (E_FAIL);
    }
    else if (*window_units != L7_LS && *num_window != 4)
    {
      c_lasmsg (FALSE, "l7hdf2las-units",
        "Error: Invalid window specification for the specified UNITS of DEG, "
        "DMS, or PRO");
      return (E_FAIL);
    }
  }

  if (c_get_multiple_outs (vblock, max_out, "OUT", outfile, num_out) != E_SUCC)
  {
  }

  /* Get the output image names (OUT)
     --------------------------------*/
  if (c_get_multiple_outs (vblock, max_out, "OUT", outfile, num_out) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-out",
      "Error reading the OUT parameter.  A maximum of %d filenames can be "
      "specified for this parameter.", max_out);
    return (E_FAIL);

  }

  if (outfile[0] == "\0")
  {
    c_lasmsg (FALSE, "l7hdf2las-out",
      "Error: The OUT parameter must be specified");
    return (E_FAIL);
  }

  /* Check the OUT values with the NBANDS and BANDS values
     -----------------------------------------------------*/
  if ((*num_nbands > 0) && (*num_nbands != *num_out))
  {
    c_lasmsg (FALSE, "l7hdf2las-out",
      "Error: Number of NBANDS values must match the number of OUT values");
    return (E_FAIL);
  }
  else if ((*num_nbands == 0) && (*num_out > 1) && (*num_out != *num_bands))
  {
    c_lasmsg (FALSE, "l7hdf2las-out",
      "Error: Number of BANDS values must match the number of OUT values "
      "since NBANDS was not specified");
    return (E_FAIL);
  }

  /* Set up the default nbands values and the many_to_one flag
     ---------------------------------------------------------*/
  if (*num_out == 1)
  {
    /* All bands go to one image.  This will be reset later for
       multi-resolution imagery
       --------------------------------------------------------*/
    *many_to_one = TRUE;
    nbands[0] = *num_bands;
    *num_nbands = 1;
  }
  else if (*num_out == *num_bands)
  {
    /* Each image has one band
       -----------------------*/
    *many_to_one = FALSE;
    for (index = 0; index < *num_bands; index++)
      nbands[index] = 1;
    *num_nbands = *num_bands;
  }
  else
  {
    /* NBANDS was already set by the user
       ----------------------------------*/
    *many_to_one = FALSE;
  }

  /* Check if the file names specified in OUT currently exist
     --------------------------------------------------------*/
  for (index = 0; index < *num_out; index++)
    /* Loop through the output filenames */
  {
    /* Split HDF filename into directory, root, and extension
       ------------------------------------------------------*/
    c_splitname (outfile[index], directory, root, extension);

    for (index2 = 0; index2 < num_multi_res_combs; index2++)
      /* Loop through the possible multi-res combinations */
    {
      for (index3 = 0; index3 < num_file_exts; index3++)
        /* Loop through all the file extensions */
      {
        sprintf (outname, "%s%s%s%s", directory, root, multi_res_combs[index2],
          file_exts[index3]);

        /* Check if the current filename combination already exists
           --------------------------------------------------------*/
        if (access (outname, F_OK) == 0)
        {
          sprintf (err_text, "Error: Image file %s already exists", outname);
          c_lasmsg (FALSE, "l7hdf2las-out", err_text);
          return (E_FAIL);
        }
      }
    }
  }

  /* Get the output data type (ODTYPE)
     ---------------------------------*/
  var = p_find (vblock, "ODTYPE");
  strcpy (tmp_str, SVAL (*var, 0));
  if (!strncmp (tmp_str, "SAME", 4) || !strncmp (tmp_str, "same", 4))
    *odtype = 0;
  else if (!strncmp (tmp_str, "BYTE", 4) || !strncmp (tmp_str, "byte", 4))
    *odtype = EBYTE;
  else if (!strncmp (tmp_str, "I*2", 4) || !strncmp (tmp_str, "i*2", 4))
    *odtype = EWORD;
  else if (!strncmp (tmp_str, "I*4", 4) || !strncmp (tmp_str, "i*4", 4))
    *odtype = ELONG;
  else if (!strncmp (tmp_str, "R*4", 4) || !strncmp (tmp_str, "r*4", 4))
    *odtype = EREAL;
  else
  {
    sprintf (err_text, "Error: An output data type of %s is invalid", tmp_str);
    c_lasmsg (FALSE, "l7hdf2las-odtype", err_text);
    return (E_FAIL);
  }

  /* Get product type (PTYPE)
     -------------------------*/
  var = p_find (vblock, "PTYPE");

  if (var->v_count == 0)
    strcpy (processing_system, "NONE");
  else
    strcpy (processing_system, SVAL (*var, 0));

  /* Sort the BANDS within each NBANDS group as long as the user specified
     the BANDS.  NOTE: the bands array is overwritten with the sorted bands
     array.
     ----------------------------------------------------------------------*/
  if (!(*default_bands))
    sort_bands_in_nbands (nbands, *num_nbands, bands, *num_bands);

  /* If the user specified the BANDS and NBANDS, check if all BANDS within
     the NBANDS sets are of the same resolution.
     ---------------------------------------------------------------------*/
  if (*num_nbands > 0 && !(*many_to_one) && !(*default_bands))
  {
    /* If the user specified NBANDS then check to see if the resolution is
       the same for all bands in each NBAND set
       -------------------------------------------------------------------*/
    group_start = 0;

    /* Loop through NBANDS
       -------------------*/
    for (index_nb = 0; index_nb < *num_nbands; index_nb++)
    {
      /* Loop through the BANDS in the current NBANDS group
         --------------------------------------------------*/
      for (index_b = group_start, band_count = 0;
           index_b < group_start + nbands[index_nb];
           index_b++, band_count++)
      {
        bands_present[band_count] = bands[index_b];
      }

      check_res (bands_present, band_count, &tmp_same_res);
      nbands_same_res[index_nb] = tmp_same_res;
      group_start += nbands[index_nb];
    }
  }
  else if (!(*default_bands))
  {
    check_res (bands, *num_bands, &tmp_same_res);
    nbands_same_res[0] = tmp_same_res;
  }
  else
  {
    /* Default_bands is TRUE.  We don't know which bands we are processing
       so we will assume they are not all of the same resolution, until we
       read the metadata
       -------------------------------------------------------------------*/
    nbands_same_res[0] = FALSE;
  }

  /* Get the $MSGTIM
     ---------------*/
  var = p_find (vblock, "$MSGTIM");
  *msgtim = IVAL (*var, 0);

  /* Get the $MINMAX
     ---------------*/
  var = p_find (vblock, "$MINMAX");
  strcpy (minmax, SVAL (*var, 0));

  /* Get the $DELFLG
     ---------------*/
  var = p_find (vblock, "$DELFLG");
  strcpy (tmp_str, SVAL (*var, 0));
  if (!strncmp (tmp_str, "YES", 3))
    *delflg = TRUE;
  else
    *delflg = FALSE;

  return (E_SUCC);
}
