/****************************************************************************

NAME: get_som_projparms

PURPOSE: Fill in the necessary 15 projection parameters from the LPGS
         metadata for the Space Oblique Mercator B projection.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "geompak.h"     /* c_getdatum function definition */

int get_som_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
)

{
  char units[10];       /* false easting/northing units */
  long wrs_path;        /* WRS path from metadata */
  double false_east;    /* false easting value from metadata */
  double false_north;   /* false northing value from metadata */
  struct DATUMDEF dtm_info;
                        /* structure for the datum information */

  /* Get necessary values from the LPGS Metadata
     -------------------------------------------*/
  strncpy (units, LPGSMetaDataFile->Parameters.u.om.FalseEastingNorthingUnits,
    sizeof (units) - 1);
  units[sizeof (units) - 1] = '\0';
  false_east = LPGSMetaDataFile->Parameters.u.om.FalseEasting;
  false_north = LPGSMetaDataFile->Parameters.u.om.FalseNorthing;
  wrs_path = LPGSMetaDataFile->ProductMetaData.WRSPath;

  /* Convert the units to meters since that is what the semi major and semi
     minor axis will be calculated in.  false_east, false_north, semi_major,
     and semi_minor must all be of the same units
     -----------------------------------------------------------------------*/
  if (!strcmp (units, "feet"))
  {
    /* Convert false_east from feet to meters
       --------------------------------------*/
    false_east *= FEET2MTRS;

    /* Convert false_north from feet to meters
       ---------------------------------------*/
    false_north *= FEET2MTRS;
  }

  /* The third projection paramter is satellite number.  This should be a
     '7' for Landsat-7
     --------------------------------------------------------------------*/
  proj_parms[0] = 0.0;
  proj_parms[1] = 0.0;
  proj_parms[2] = 7.0;
  proj_parms[3] = (double) wrs_path;
  proj_parms[4] = 0.0;
  proj_parms[5] = 0.0;
  proj_parms[6] = false_east;
  proj_parms[7] = false_north;
  proj_parms[8] = 0.0;
  proj_parms[9] = 0.0;
  proj_parms[10] = 0.0;
  proj_parms[11] = 0.0;
  proj_parms[12] = 1.0;
  proj_parms[13] = 0.0;
  proj_parms[14] = 0.0;

  /* Put the semi-major and semi-minor axis in proj_parms
     ----------------------------------------------------*/
  dtm_info.datumnum = datum_code;
  if (c_getdatum (&dtm_info) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-getdatum", "Error getting the datum info");
    return (E_FAIL);
  }
  proj_parms[0] = dtm_info.smajor;
  proj_parms[1] = dtm_info.sminor;

  return (E_SUCC);
}
