/******************************************************************************
NAME:		gxx_create_1g_band

PURPOSE:	Create a SDS for the specified 1G band.

DEVELOPMENT HISTORY:
$Log: gxx_create_1g_band.c,v $
Revision 1.4  2001/09/28 13:34:51  tbeckman
Removed redundant prototypes for some LAS support routines to prevent breaking the build when those prototypes are changed.  Other minor cleanups.

Revision 1.3  2001/01/19 20:36:35  gschmidt
Finished making code review changes.  In addition, added signal handlers
to handle the Ctrl-C introduced by the user.

Revision 1.2  2000/11/08 14:53:20  gschmidt
The gxx and xxx routines had to be updated to compile with the LAS code
and within the LAS system.  Several minor changes were made, but nothing
has been changed to affect the overall flow/working of the original code.

 * 
 *    Rev 1.7   Thu Mar 05 15:24:22 1998   tbeckman
 * Changed the longs to ints to remove warnings for the -64 compile.
 * 
 *    Rev 1.6   Mon Aug 11 11:06:46 1997   rockvam
 * post-walkthrough changes:
 *  cleaned up Log
 *  fixed comment regarding name of external element files
 *  changed fill values to use HDF types
 * 

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
XXXSUCCESS	Successfully created band SDS
ERROR    	Error occurred; band SDS not created

NOTES:		The 1G image is a VGroup containing a SDS for each band.  The 
		data for each SDS are stored in an external element.

		The external files are named image_filename root + SDS band tag
		(SDS_BAND1, etc.) + image_filename extension. e.g., img_B10.l1g

		The band numbers specified in bandno are assumed to be in
		the range b1..b8.

		The HDFIO structure on input must have the hdf_dir_name
		field filled in.
		The HDFIO structure will have the sds_name and sds_id fields for
		the requested band [bandno] filled in.

*******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "xxx_Types.h"
#include "gxx_hdf_io.h"
#include "wrapper.h"

int gxx_create_1g_band (
	HDFIO *hdfid,		/* I/O-SDS id for the band added to structure */
	BandNumber bandno,	/* I-Enumerated band number	*/
	DataType dtype,		/* I-Data type of the band	*/
	int  nl,		/* I-Number of lines for the band*/
	int  ns)		/* I-Number of samples for the band*/

{
intn	hdf_status;		/* HDF function return status	*/
int32	ref,			/* vgroup element reference	*/
	tag,			/* vgroup element tag		*/
	data_type,		/* data type for 1G images	*/
	rank=2,			/* 2-d SDS (one band)		*/
	dim_sizes[MAX_VAR_DIMS];/* dimension sizes (nl, ns)	*/

int16 	fill_i16=0;		/* integer fill value		*/
int32	fill_i32=0;		/* long fill value		*/
float32 fill_f32=0.0;		/* float fill value		*/
uint8 	fill_ui8=0;		/* byte fill value		*/
void 	*fill_val;		/* pointer to fill value	*/

char 	SDname[STRLEN];	  	/* SDS name			*/
char	msg[STRLEN];		/* Error message string		*/
char	ext_filename[STRLEN];	/* External file name		*/

/* 
 * Set up SDS name using name constructed from hdf dir file name and band
 * This will also be the filename of the external element
 */
gxx_create_band_name(hdfid->hdf_dir_name, "SDS", bandno, SDname);
strcpy(hdfid->sds_name[(int)bandno],SDname);

dim_sizes[0] = nl;
dim_sizes[1] = ns;
if (dtype == I2) 
    {
    data_type = DFNT_INT16;
    fill_val = (void *)&fill_i16;
    }
else if (dtype == I4) 
    {
    data_type = DFNT_INT32;
    fill_val = (void *)&fill_i32;
    }
else if (dtype == R4) 
    {
    data_type = DFNT_FLOAT32;
    fill_val = (void *)&fill_f32;
    }
else if (dtype == BYTE)
    {
    data_type = DFNT_UINT8;
    fill_val = (void *)&fill_ui8;
    }
else
    {
    print_message(FALSE, "gxx_HDFIO", "Invalid image data type");
    return(ERROR);
    }

hdfid->sds_id[(int)bandno] = SDcreate(hdfid->sd_id, SDname, data_type, 
    rank, dim_sizes);
if (hdfid->sds_id[(int)bandno] == FAIL)
    {
    sprintf(msg,"Error creating SDS %s", SDname);
    print_message(FALSE, "gxx_HDFIO", msg);
    return(ERROR);
    }

/*
 * Set the fill value
 */
hdf_status = SDsetfillvalue(hdfid->sds_id[(int)bandno], fill_val);
if (hdf_status == FAIL)
    {
    HEprint(stderr,0);
    print_message(FALSE, "gxx_HDFIO", "Error setting fill value");
    }

/*
 * Set the SDS into an external file
 * Use the SDS name as the exernal file name
 */
gxx_create_band_name(hdfid->hdf_dir_name, "FILE", bandno, ext_filename);

/* remove the file in case it already exists */
unlink(ext_filename);

hdf_status=SDsetexternalfile(hdfid->sds_id[(int)bandno], ext_filename, 0);
if (hdf_status == FAIL)
    {
    sprintf(msg,"Error creating external SDS %s", SDname);
    print_message(FALSE, "gxx_HDFIO", msg);
    return(ERROR);
    }

/*
 * Add the SDS to the group
 */
ref = SDidtoref(hdfid->sds_id[(int)bandno]);
tag = DFTAG_SD;
hdf_status = Vaddtagref(hdfid->group_id, tag, ref);
if (hdf_status == FAIL)
    {
    sprintf(msg,"Error creating SDS %s", SDname);
    print_message(FALSE, "gxx_HDFIO", msg);
    return(ERROR);
    }


return(XXXSUCCESS);
}
