/******************************************************************************
NAME:           gxx_create_band_name

PURPOSE:
gxx_create_band_name forms a band file name or band SDS name given the 
HDF file name.
The "_HDF" at the end of the root HDF file name is replaced with the appropriate
band tag (e.g., "_B10").  If no "_HDF" is at the end of the file name, the band
tag is appended to the end of the root HDF file name.

DEVELOPMENT HISTORY:
$Log: gxx_create_band_name.c,v $
Revision 1.3  2001/01/19 20:36:35  gschmidt
Finished making code review changes.  In addition, added signal handlers
to handle the Ctrl-C introduced by the user.

Revision 1.2  2000/11/08 14:53:20  gschmidt
The gxx and xxx routines had to be updated to compile with the LAS code
and within the LAS system.  Several minor changes were made, but nothing
has been changed to affect the overall flow/working of the original code.

 * 
 *    Rev 1.2   Mon Aug 11 11:07:34 1997   rockvam
 * post-walkthrough changes:
 *  updated documentation
 *  made input parms const

RETURN VALUE: 
No return value

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include "gxx_hdf_io.h"	 /* HDF i/o function prototypes & SDS band tags */
#include "gxx_utility.h" /* createname and splitname prototypes */

void gxx_create_band_name (
    const char *HDF_name,/* I: HDF name to form band file name from 	*/
    const char *type,	/* I: SDS or FILE name ("SDS" or "FILE")	*/
    BandNumber bandno,	/* I: enumerated band number			*/
    char *band_name)  	/* O: constructed band file name 		*/
{
char directory[STRLEN]; /* directory name */
char root[STRLEN];      /* root file name */
char extension[STRLEN]; /* file extension */
char *temp_char_ptr;    /* temporary character pointer */
static char band_tag[NBANDS][4] = {SDS_BAND1, SDS_BAND2, SDS_BAND3, SDS_BAND4, 
    SDS_BAND5, SDS_BAND6L, SDS_BAND6H, SDS_BAND7, SDS_BAND8};
    
/* form the bandfile file name from the HDF name */
gxx_splitname(HDF_name, directory, root, extension);

/* find the "_HDF" part of the file name if it exists */
temp_char_ptr = strrchr(root,'_');
if (temp_char_ptr == NULL)
    {
    /* no "_HDF" so just append the appropriate band tag */
    /*  for SDS  name it is root.tag */
    /*  for file name it is root_tag.ext */
    if (strcmp(type,"SDS") == 0)
        strcpy(extension, band_tag[(int)bandno]);
    else
	{
        strcat(root, "_");
        strcat(root, band_tag[(int)bandno]);
        }
    }
else
    {
    if (strcmp (temp_char_ptr, "_HDF") == 0)
        {
        /* replace the "_HDF" with the band substring */
        /*  for SDS  name it is root.tag so set a NULL where the _HDF is 
	    and set the extension to the tag */
        /*  for file name it is root_tag.ext so replace the HDF with the tag */
        if (strcmp(type,"SDS") == 0)
	    {
            *temp_char_ptr = '\0';
            strcpy(extension, band_tag[(int)bandno]);
	    }
        else
            strcpy(temp_char_ptr+1, band_tag[(int)bandno]);
        }
    else
        {
        /* doesn't end with "_HDF" so just append the band tag */
        /*  for SDS  name it is root.tag */
        /*  for file name it is root_tag.ext */
        if (strcmp(type,"SDS") == 0)
            strcpy(extension, band_tag[(int)bandno]);
        else
	    {
            strcat(root, "_");
            strcat(root, band_tag[(int)bandno]);
	    }
        }
    }

/* if dealing with a format 2 file (B6H, B7, or B8) then change the third
   character of the root to a '2'  (-gls 10/18/00) */
if (bandno == b6h || bandno == b7 || bandno == b8)
  root[2] = '2';

/* create band file name */
gxx_createname(directory, root, extension, band_name);

return;
}
