/******************************************************************************
NAME:		gxx_open_1g_band

PURPOSE:	Open a band of the 1G image.

DEVELOPMENT HISTORY:
$Log: gxx_open_1g_band.c,v $
Revision 1.4  2001/09/28 13:34:51  tbeckman
Removed redundant prototypes for some LAS support routines to prevent breaking the build when those prototypes are changed.  Other minor cleanups.

Revision 1.3  2001/01/19 20:36:35  gschmidt
Finished making code review changes.  In addition, added signal handlers
to handle the Ctrl-C introduced by the user.

Revision 1.2  2000/11/08 14:53:20  gschmidt
The gxx and xxx routines had to be updated to compile with the LAS code
and within the LAS system.  Several minor changes were made, but nothing
has been changed to affect the overall flow/working of the original code.

 * 
 *    Rev 1.6   Thu Nov 06 15:13:42 1997   tbeckman
 * Corrected a format in a sprintf statement
 * 
 *    Rev 1.5   Mon Aug 11 11:11:06 1997   rockvam
 * post-walkthrough changes:
 *  cleaned up Log and updated documentation

RETURN VALUE: 
Type = int32
Value           Description
-----           -----------
band id		HDF SDS id used to access the band
FAIL     	HDF failure code--Band not opened (not found)

NOTES:		THIS ROUTINE SHOULD ONLY BE CALLED BY OTHER GPS 1G I/O ROUTINES
		
		The 1G image is a VGroup containing a SDS for each band.  The 
		data for each SDS are stored in an external element.

		The band number specified in bandno is assumed to be in
		the range b1..b8.

		The HDFIO structure will fill in the sds_name field for the 
		requested band [bandno] and return the sds_id.  

		***This routine assumes the band SDS is already created.
		The band is simply selected (opened) so that other routines
		may access it.

		***This routine does not issue any error messages, because
		a non-present band may not necessarily be an error (ie
		if the caller is checking to see which of the bands are
		present).

*******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "gxx_hdf_io.h"
#include "xxx_Types.h"
#include "wrapper.h"

int32 gxx_open_1g_band (
	HDFIO *hdfid,		/* I-HDF info structure for the image	*/
	BandNumber bandno)	/* I-enumerated band number		*/

{
int32	sds_index,		/* HDF SDS index number		 */
	sds_id;			/* HDF SDS id			 */
char	msg[STRLEN];		/* error message		 */

/*
 * Initialize the sds id
 */
sds_id = -1;

/*
 * Construct the name of the SDS for this band
 */
gxx_create_band_name(hdfid->hdf_dir_name, "SDS", bandno, 
    hdfid->sds_name[(int)bandno]);

/*
 * Select the user-specified band
 */
sds_index = SDnametoindex(hdfid->sd_id, hdfid->sds_name[(int)bandno]);
if (sds_index == FAIL)
    {
    print_message(FALSE, "gxx_HDFIO", "Band not found");
    return(FAIL); /* band not found */
    }

sds_id = SDselect(hdfid->sd_id, sds_index);
if (sds_id == FAIL)
    {
    sprintf(msg,"Band %s not found", hdfid->sds_name[(int)bandno]);
    print_message(FALSE, "gxx_HDFIO", msg);
    return(FAIL);
    }

return (sds_id);
}
