/******************************************************************************
NAME:		gxx_open_1g_image

PURPOSE:	Open the specified 1G image.

DEVELOPMENT HISTORY:
$Log: gxx_open_1g_image.c,v $
Revision 1.4  2001/09/28 13:34:51  tbeckman
Removed redundant prototypes for some LAS support routines to prevent breaking the build when those prototypes are changed.  Other minor cleanups.

Revision 1.3  2001/01/19 20:36:35  gschmidt
Finished making code review changes.  In addition, added signal handlers
to handle the Ctrl-C introduced by the user.

Revision 1.2  2000/11/08 14:53:20  gschmidt
The gxx and xxx routines had to be updated to compile with the LAS code
and within the LAS system.  Several minor changes were made, but nothing
has been changed to affect the overall flow/working of the original code.

 * 
 *    Rev 1.10   Mon Jul 20 10:28:26 1998   vanderv
 * added checks to sprintfs that were not 
 * in error and just returned or in a
 * debug statement.
 * 
 *    Rev 1.9   Thu Mar 05 15:25:12 1998   tbeckman
 * Changed the longs to ints to remove warnings for the -64 compile.
 * 
 *    Rev 1.8   Thu Nov 06 15:18:24 1997   tbeckman
 * Corrected a format in a sprintf statement
 * 
 *    Rev 1.7   Mon Aug 11 11:12:26 1997   rockvam
 * post-walkthrough changes:
 *  cleaned up Log and updated documentation
 *  fixed used of group_access_mode (string instead of a single character)
 *  added prolog for cleanup function

RETURN VALUE: 
Type = HDFIO *
Value           Description
-----           -----------
hdfid		HDF structure used to access the image
NULL     	Error occurred; image not open

NOTES:		The 1G naming convention is as follows:
		The HDF directory file name is specified by the user,
		  convention is root_HDF.xxx
		Each band file will be named using the root and extension of the
	 	  directory, replacing the "HDF" with the correct band keyword:
		  root_B10.xxx 		root_B61.xxx
		  root_B10.xxx 		root_B62.xxx
		  root_B20.xxx 		root_B70.xxx
		  root_B30.xxx 		root_B80.xxx
		  root_B40.xxx
		The 1G image is a VGroup containing a SDS for each band.  The 
		data for each SDS are stored in an external element.
		The external files are named image_filename root + SDS band tag
		(SDS_BAND1, etc.) + image_filename extension. e.g., img_B10.l1g
		The band numbers specified in bandno are assumed to be in
		the range b1..b8.
		For READ access, the available band and sizes are returned in 
		bandno, nl, ns, and dtype if nbands=0, or if nbands>0, the bands
		specified in bandno are opened, and the sizes are returned in 
		nl, ns, and dtype (no subwindows may be specified for reading).
		For WRITE access the bands and sizes must be specified on input 
		in bandno, nl, ns, and dtype so the image can be created.
		If any of the files for the product already exist, they
		are overwritten.
		For UPDATE access the available bands and sizes are returned in
		bandno, nl, ns, and dtype if nbands is 0; if nbands>0, the bands
		specified in bandno are opened and the sizes are returned in nl,
		ns, and dtype.  If the file does not exist it is created.

*******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>		/* for malloc */
#include "xxx_Types.h"		/* ERROR/XXXSUCCESS */
#include "gxx_hdf_io.h"		/* HDF i/o defines and prototypes */
#include "gxx_utility.h"	/* for gxx_splitname */
#include "wrapper.h"

void cleanup_1g_image(HDFIO *hdfid, char *msg);

HDFIO *gxx_open_1g_image (
	const char *image_filename,/* I-Path & name of the 1G image file*/
	AccessMode access_mode,	/* I-Requested access mode of image*/
	int  *nbands,		/* I/O-Number of bands		*/
	BandNumber bandno[NBANDS], /* I/O-enumerated band numbers	*/
	DataType dtype[NBANDS],	/* I/O-Data type of each band	*/
	int  nl[NBANDS],	/* I/O-Number of lines per band	*/
	int  ns[NBANDS])	/* I/O-Number of samples per band*/

{
HDFIO 	*hdfid=NULL;		/* HDF I/O data			*/

intn	hdf_access_mode;	/* HDF access mode code		*/
intn	hdf_status;		/* HDF function return status	*/
int32	group_ref,		/* HDF group reference number	*/
	data_type,		/* data type for 1G images	*/
	rank=2,			/* 2-d SDS (one band)		*/
	dim_sizes[MAX_VAR_DIMS],/* dimension sizes (nl, ns)	*/
	nattrs;			/* number of attributes		*/

BandNumber band;		/* enumerated band loop counter	*/

int	curband,		/* integer loop counter for specified bands */
	return_info,		/* flag to return info to the user */
	status,			/* return of status from function */
	sds;			/* sds counter			*/

char	group_access_mode[2];	/* access mode code for Vgroup	*/
char	str_mode[8];		/* string of file access mode	*/
char	msg[STRLEN];		/* error message string		*/
char    dir[STRLEN];		/* directory of the 1G image file*/
char    root[STRLEN];		/* root name of the 1G image file*/
char    ext[STRLEN];		/* extension of the 1G image file*/
char    dirspec[STRLEN];	/* search directories for external files*/
char    sds_name[STRLEN];       /* temporary SDS name            */

/*
 * Check the number of bands
 */
if ( (*nbands < 0) || ((access_mode == WRITE) && (*nbands == 0)) )
    {
    sprintf(msg,"Invalid number of bands %ld",*nbands);
    cleanup_1g_image(hdfid, msg);
    return(NULL);
    }
if (*nbands > NBANDS) 
    {
    *nbands = NBANDS;
    status = sprintf(msg,"Only the first %d bands will be written to the 1G image",
        NBANDS);
	if (status < 1)
		{
                print_message(FALSE, "gxx_HDFIO", "Error in sprintf");
		return(NULL);
		}
    print_message(FALSE, "gxx_HDFIO", msg);
    }

/*
 * Create the structure to track HDF info
 * The structure contains an array of band SDS id's which are indexed by
 * the enumerated band numbers
 */
hdfid = malloc(sizeof(HDFIO));
if (hdfid == NULL)
    {
    cleanup_1g_image(hdfid, "Error allocating memory");
    return(NULL);
    }

/*
 * Initialize the sds id's
 * Get the root filename for the hdf_dir_name
 */
gxx_splitname(image_filename, dir, root, ext);
status = sprintf(hdfid->hdf_dir_name,"%s.%s",root,ext);
if (status < 1)
	{
   	print_message(FALSE, "gxx_HDFIO", "Error in sprintf");
	return(NULL);
	}
hdfid->file_id = -1;
hdfid->group_id = -1;
hdfid->sd_id = -1;
for (band=b1; band<=b8; band++)
    {
    hdfid->sds_id[(int)band] = -1;
    strcpy(hdfid->sds_name[(int)band],"");
    }

/*
 * Do different types of opens for different access modes
 */
if ((access_mode == READ) || (access_mode == UPDATE))
    {
   /*
    * Initialize info returned to the user 
    */
    return_info = FALSE;
    if (*nbands == 0)
	{
	return_info = TRUE;
	for (sds=0; sds<NBANDS; sds++)
	    {
            bandno[sds] = berror;
            nl[sds] = 0;
            ns[sds] = 0;
	    }
	}

   /*
    * Open the HDF file
    */
    if (access_mode == READ)
	{
        hdf_access_mode = DFACC_READ;
        strcpy(group_access_mode,"r");
        strcpy(str_mode,"READ");
	}
    else /* UPDATE */
	{
        hdf_access_mode = DFACC_RDWR;
        strcpy(group_access_mode,"w");
        strcpy(str_mode,"UPDATE");
	}
    hdfid->file_id = Hopen(image_filename, hdf_access_mode, 0);
    if (hdfid->file_id == FAIL)
        {
        sprintf(msg,"Error opening file %s for requested mode %s",
	    image_filename,str_mode);
	cleanup_1g_image(hdfid, msg);
    	return(NULL);
        }

   /*
    * Set up the search path for external elements  (HXsetdir) and 
    * path for creating new external elements (HXsetcreatedir)
    */
    gxx_splitname(image_filename, dir, root, ext);
    if (dir[0] != 0)
	{
	/* directory specified */
        if (access_mode == UPDATE)
	    HXsetcreatedir(dir);
        status = sprintf(dirspec,".|%s",dir);
		if (status < 1)
			{
   			print_message(FALSE, "gxx_HDFIO", "Error in sprintf");
			return(NULL);
			}
	}
    else
	{
	/* no directory specified; just use CWD  */
        if (access_mode == UPDATE)
	    HXsetcreatedir(".");
        status = sprintf(dirspec,".");
		if (status < 1)
			{
   			print_message(FALSE, "gxx_HDFIO", "Error in sprintf");
			return(NULL);
			}
	}
    /* Save the external file directory so the read function can 
       search it using HXsetdir */
    strcpy(hdfid->extdir, dirspec);


   /*
    * Initialize the V interface
    */
    hdf_status = Vstart (hdfid->file_id);
    if (hdf_status == FAIL)
        {
        sprintf(msg,"xxxError initializing HDF for file %s",image_filename);
	cleanup_1g_image(hdfid, msg);
	return(NULL);
        }

   /*
    * Locate & attach to the 1G image group
    */
    group_ref = Vfindclass(hdfid->file_id, IMAGE_CLASS);
    if (group_ref != 0)
	{
	hdfid->group_id = Vattach(hdfid->file_id, group_ref, 
				     group_access_mode);
	if (hdfid->group_id == FAIL)
	    {
	    cleanup_1g_image(hdfid,"Error accessing image group");
	    return(NULL);
	    }
	}
    else
	{
       /* Try the old IMAGE_CLASS definition  (-gls 10/18/00)
        */
       group_ref = Vfindclass(hdfid->file_id, IMAGE_CLASS_OLD);
       if (group_ref != 0)
           {
           hdfid->group_id = Vattach(hdfid->file_id, group_ref,
                                     group_access_mode);
           if (hdfid->group_id == FAIL)
               {
               cleanup_1g_image(hdfid,"Error accessing image group");
               return(NULL);
               }
           }
       else
           {

          /* 
   	   * If update mode, create the group and set the class if group not
           * found.  If read mode, non-existent group is an error.
   	   */
   	   if (access_mode == UPDATE)
	       {
    	       group_ref = -1;
    	       hdfid->group_id = Vattach(hdfid->file_id, group_ref, 
				 group_access_mode);
    	       if (hdfid->group_id == FAIL)
	           {
	           cleanup_1g_image(hdfid, "Error creating image group");
	           return(NULL);
	           }

    	       hdf_status = Vsetclass(hdfid->group_id, IMAGE_CLASS);
    	       if (hdf_status == FAIL)
		   {
		   cleanup_1g_image(hdfid, "Error creating image group");
		   return(NULL);
		   }
	       }
	   else
	       {
	       cleanup_1g_image(hdfid, "Error accessing image group");
	       return(NULL);
	       }
           }
        }

   /*
    * Get the SDS interface id
    */
    hdfid->sd_id = SDstart(image_filename, hdf_access_mode);
    if (hdfid->sd_id == FAIL)
	{
	cleanup_1g_image(hdfid, "Error initializing HDF");
	return(NULL);
	}

   /*
    * Now open the bands 
    */
    if (return_info)  
	{
       /*
	* Opening whichever bands are there 
	*/
	for (band = b1; band <= b8; band++)
	    {
	    hdfid->sds_id[(int)band]=gxx_open_1g_band(hdfid, band);
	    if (hdfid->sds_id[(int)band] != FAIL)
		{
		hdf_status = SDgetinfo(hdfid->sds_id[(int)band], 
		    hdfid->sds_name[(int)band], 
		    &rank, dim_sizes, &data_type, &nattrs);
		if (hdf_status == FAIL)
	    	    {
            	    sprintf(msg,"Error getting %s info", 
		        hdfid->sds_name[(int)band]);
		    cleanup_1g_image(hdfid, msg);
		    return(NULL);
            	    }

	    	bandno[*nbands] = band;
	    	nl[*nbands] = dim_sizes[0];
	    	ns[*nbands] = dim_sizes[1];

		if (data_type == DFNT_UINT8) dtype[*nbands] = BYTE;
		else if (data_type == DFNT_INT16) dtype[*nbands] = I2;
		else if (data_type == DFNT_UINT16) dtype[*nbands] = UI2;
		else if (data_type == DFNT_INT32) dtype[*nbands] = I4;
		else if (data_type == DFNT_UINT32) dtype[*nbands] = UI4;
		else if (data_type == DFNT_FLOAT32) dtype[*nbands] = R4;
		else if (data_type == DFNT_FLOAT64) dtype[*nbands] = R8;
		else dtype[*nbands] = type_error;

		(*nbands)++;

            	} /* found a valid band */
	    } /* loop on possible bands */
	} /* if returning whatever is there */
    else
	{
       /*
	* Opening user-specified bands
	*/
	for (curband = 0; curband < *nbands; curband++)
	    {
	    hdfid->sds_id[(int)bandno[curband]] = 
		 gxx_open_1g_band(hdfid, bandno[curband]);

	    if (hdfid->sds_id[(int)bandno[curband]] == FAIL)
	     	{
	       /*
		* On update if the band isn't there, create it
		* On read it is an error if the band isn't there
		*/
		if (access_mode == READ)
		    {
            	    sprintf(msg,"Error locating %s",
                      hdfid->sds_name[(int)bandno[curband]]);
		    cleanup_1g_image(hdfid, msg);
		    return(NULL);
		    }
		else
		    {
	           /* 
		    * Create the band if it's not there 
		    */
		    if (gxx_create_1g_band(hdfid, 
			bandno[curband], dtype[curband], nl[curband], 
			ns[curband]) != XXXSUCCESS)
                        {
                        sprintf(msg,"Error creating SDS %s",
			    hdfid->sds_name[(int)bandno[curband]]);
                        cleanup_1g_image(hdfid, msg);
                        return(NULL);
                        }

		    } /* adding band SDS on UPDATE */
		} /* band not found */
	    else
		{
                /* Don't keep the sds_name from SDgetinfo since we already
                   created the sds_name in gxx_open_1g_band.
                   (-gls 10/19/00) */
		hdf_status = SDgetinfo(hdfid->sds_id[(int)bandno[curband]], 
		    sds_name, &rank, dim_sizes, &data_type, &nattrs);
		if (hdf_status == FAIL)
	    	    {
            	    sprintf(msg,"Error getting %s info",
                      hdfid->sds_name[(int)bandno[curband]]);
		    cleanup_1g_image(hdfid, msg);
		    return(NULL);
            	    }

	    	nl[curband] = dim_sizes[0];
	    	ns[curband] = dim_sizes[1];

		if (data_type == DFNT_UINT8) dtype[curband] = BYTE;
		else if (data_type == DFNT_INT16) dtype[curband] = I2;
		else if (data_type == DFNT_UINT16) dtype[curband] = UI2;
		else if (data_type == DFNT_INT32) dtype[curband] = I4;
		else if (data_type == DFNT_UINT32) dtype[curband] = UI4;
		else if (data_type == DFNT_FLOAT32) dtype[curband] = R4;
		else if (data_type == DFNT_FLOAT64) dtype[curband] = R8;
		else dtype[curband] = type_error;

		} /* band found */
	    } /* loop on user-specified bands */
	} /* opening user-specified bands */
    } /* read/update */

else if (access_mode == WRITE)
    {
   /*
    * Open the HDF file
    */
    hdf_access_mode = DFACC_CREATE;
    strcpy(str_mode,"WRITE");
    hdfid->file_id = Hopen(image_filename, hdf_access_mode, 0);
    if (hdfid->file_id == FAIL)
        {
        sprintf(msg,"Error opening file %s for requested mode %s",
	    image_filename,str_mode);
	cleanup_1g_image(hdfid, msg);
	return(NULL);
        }

   /*
    * Set up the external directory path
    * and set up a band file name without directory so it will
    * be created in HXsetcreatedir
    */
    gxx_splitname(image_filename, dir, root, ext);
    if (dir[0] != 0)
       HXsetcreatedir(dir);
    else
       HXsetcreatedir(".");

   /*
    * Initialize the V interface
    */
    hdf_status = Vstart(hdfid->file_id);
    if (hdf_status == FAIL)
        {
        sprintf(msg,"Error initializing HDF for file %s",image_filename);
	cleanup_1g_image(hdfid, msg);
	return(NULL);
        }

   /*
    * Create the Vgroup containing the bands
    */
    group_ref = -1;
    hdfid->group_id = Vattach(hdfid->file_id, group_ref, "w");
    if (hdfid->group_id == FAIL)
	{
	cleanup_1g_image(hdfid, "Error creating image group");
	return(NULL);
	}

   /* 
    * Set the group class name
    */
    hdf_status = Vsetclass(hdfid->group_id, IMAGE_CLASS);
    if (hdf_status == FAIL)
	{
	cleanup_1g_image(hdfid, "Error creating image group");
	return(NULL);
	}

   /*
    * Get the SDS interface id
    * Change the access to read/write because the file is now created
    */
    hdf_access_mode = DFACC_RDWR;
    hdfid->sd_id = SDstart(image_filename, hdf_access_mode);
    if (hdfid->sd_id == FAIL)
	{
	cleanup_1g_image(hdfid, "Error initializing HDF");
	return(NULL);
	}

   /*
    * Create a SDS for each band
    */
    for (curband = 0; curband < *nbands; curband++)
	{
	if (gxx_create_1g_band(hdfid, bandno[curband], 
	    dtype[curband], nl[curband], ns[curband]) != XXXSUCCESS)
	    {
	    sprintf(msg,"Error creating SDS %s", 
		hdfid->sds_name[(int)bandno[curband]]);
	    cleanup_1g_image(hdfid, msg);
	    return(NULL);
	    }
	}

    } /* write */

else
    {
    print_message(FALSE, "gxx_HDFIO", "Invalid image access mode");
    return(NULL);
    }

return(hdfid);
}


/******************************************************************************
NAME:		cleanup_1g_image

PURPOSE:	close the 1g image, shutting down all HDF accesses and
		interfaces

RETURN VALUE: 
No return value
*******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

void cleanup_1g_image(HDFIO *hdfid, char *msg)
{
BandNumber band;

if (hdfid)
    {
    for (band=b1; band<=b8; band++)
	{
        if (hdfid->sds_id[band] != -1)
	    SDendaccess(hdfid->sds_id[band]);
	}

    if (hdfid->sd_id != -1) SDend(hdfid->sd_id);
    if (hdfid->group_id != -1) Vdetach(hdfid->group_id);
    if (hdfid->file_id != -1) 
        {
        Vend(hdfid->file_id);
        Hclose(hdfid->file_id);
        }

    free(hdfid);
    }

print_message(FALSE, "gxx_HDFIO", msg);
}
