/******************************************************************************
NAME:		gxx_read_1g_image

PURPOSE:	Read the specified 1G image.

DEVELOPMENT HISTORY:
$Log: gxx_read_1g_image.c,v $
Revision 1.4  2001/09/28 13:34:51  tbeckman
Removed redundant prototypes for some LAS support routines to prevent breaking the build when those prototypes are changed.  Other minor cleanups.

Revision 1.3  2001/01/19 20:36:35  gschmidt
Finished making code review changes.  In addition, added signal handlers
to handle the Ctrl-C introduced by the user.

Revision 1.2  2000/11/08 14:53:20  gschmidt
The gxx and xxx routines had to be updated to compile with the LAS code
and within the LAS system.  Several minor changes were made, but nothing
has been changed to affect the overall flow/working of the original code.

 * 
 *    Rev 1.6   Thu Mar 05 15:25:16 1998   tbeckman
 * Changed the longs to ints to remove warnings for the -64 compile.
 * 
 *    Rev 1.5   Mon Aug 11 11:13:36 1997   rockvam
 * post-walkthrough changes:
 *  cleaned up Log
 *  made input parms const
 *  added a comment regarding assumption that the SDS is 2 dimensions.
 * 

RETURN VALUE: 
Type = int
Value           Description
-----           -----------
XXXSUCCESS     	Successfully read image
ERROR     	Error occurred; image not read

NOTES:		The 1G image is a VGroup containing a SDS for each band.  The 
		data for each SDS are stored in an external element.
		The subsampling factor if set to 0 or 1 will result in reading
		the full resolution data.  Set to n will result in reading every
		n'th data point in both dimensions, starting with sl, ss.
		**This routine assumes the band is a 2-dimensional SDS.  If it 
		is of higher dimension, results will be undefined.

*******************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "gxx_hdf_io.h"
#include "wrapper.h"

int gxx_read_1g_image (
	const HDFIO *hdfid,	/* I-HDF IO structure			*/
	BandNumber band,	/* I-ETM+ (enumerated) band number to read*/
	int  sl,		/* I-Line to start reading (0-rel)	*/
	int  ss,		/* I-Sample to start reading (0-rel)	*/
	int  nl,		/* I-Number of lines to read		*/
	int  ns,		/* I-Number of samples to read		*/
	int  subsamp,		/* I-Subsampling factor			*/
	void *data)		/* O-Data buffer			*/
{
intn	hdf_status;		/* HDF function return status	*/
int32	start[2],		/* starting data location	*/
	edge[2],		/* size of data buffer		*/
	stride[2];		/* subsampling factor		*/
char	msg[STRLEN];		/* Error message string		*/


/*
 * Check the structure
 */
if (hdfid == NULL)
    {
    sprintf(msg,"Error reading from 1G image");
    print_message(FALSE, "gxx_HDFIO", msg);
    return(ERROR);
    }

/*
 * Set the search path 
 */
HXsetdir(hdfid->extdir);

/*
 * Set up specification for block to read
 */
start[0] = sl; /* start is 0-relative */
start[1] = ss;
edge[0] = nl;
edge[1] = ns;

if ((subsamp == 0) || (subsamp == 1))
    {
    /* use NULL for stride */
    hdf_status = SDreaddata(hdfid->sds_id[(int)band], start, NULL, edge, 
		 (VOIDP)data);
    }
else
    {
    stride[0] = subsamp;
    stride[1] = subsamp;
    hdf_status = SDreaddata(hdfid->sds_id[(int)band], start, stride, edge, 
		 (VOIDP)data);
    }

if (hdf_status == FAIL)
    {
    sprintf(msg,"Error reading from SDS %s", hdfid->sds_name[(int)band]);
    print_message(FALSE, "gxx_HDFIO", msg);
    return(ERROR);
    }

return(XXXSUCCESS);
}
