/****************************************************************************

NAME: l7hdf2las

PURPOSE: L7HDF2LAS will ingest a Landsat7 (L7) HDF image and output this
         image in the LAS format. This system will support L0R, L1R, L1G,
         and L1T (NLAPS) Landsat7 HDF formats. Documentation for the formats
         are defined in the NOTES section. This system will support spatial
         subsetting, multi-volume input, and multi-resolution input. This
         system will be written specifically for the LAS system. It is
         sponsored by the Data Services Branch/Digital Data Production at EDC.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
EXIT_SUCCESS    Successful completion
EXIT_FAILURE    An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    09/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

REFERENCE DOCUMENTS:
NASA Goddard Space Flight Center, Earth Science Data and Information System
  (ESDIS) Level 1 Product Output Files Data Format Control Book, Volume 5,
  Book 2, Revision 4, January 2000.
  (available at http://edcwww.cr.usgs.gov/l7dhf/L7MMO/document.htm#DFCB_vol)

NASA Goddard Space Flight Center, Landsat 7 System Zero-R Distribution Product
  Data Format Control Book, Volume 5, Book 1, Revision 3, July 1999.
  (available at http://edcwww.cr.usgs.gov/l7dhf/L7MMO/document.htm#DFCB_vol)

NCSA HDF Home Page, http://hdf.ncsa.uiuc.edu/

Landsat7 Home Page, http://landsat7.usgs.gov/

NOTES:
  1) If $DELFLG is set and an error occurs during processing of this
     application, then all files created by this application will be deleted.
  2) When processing CDs, some CD readers read the input filenames as
     lower-case.  Lower-case names will not work when reading these L7 HDF
     files, since the internal HDF SDSs are based on the upper-case names.
     As a work-around when processing CDs, upper-case links are made to the
     lower-case filenames.  The links are created in the output directory
     determined by the first output filename.  Any links that are created get
     unlinked when processing has been completed.  The files that are linked
     include the HDF filename (_HDF), the metadata (_MTL or _MTP), and the
     band files (_B##).
     This is not an issue on disk, since the L7 HDF files are really written
     as upper-case filenames.  The problem with CDs is really a CD reader
     problem, which allows the CD writer/reader to read these upper-case files
     as lower-case.

******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include "gxx_const.h"
#define L7HDF2LAS_C_
#include "l7hdf2las.h"
#include "util.h"

/* Define the global constant arrays for L7HDF2LAS
   -----------------------------------------------*/
const int lines_per_scan[NUM_L7_RES] = {16, 8, 32};
const long number_of_samples[NUM_L7_RES] = {6600, 3300, 13200};
const int resolutions[NUM_L7_RES] = {30, 60, 15};
const char res_suffixes[NUM_L7_RES][5] = {"_vis", "_thm", "_pan"};

/* Define the global variables that are used by the cleanup routine when
   the SIGKILL or SIGTERM signals are caught
   ---------------------------------------------------------------------*/
long g_num_procd_files = 0;  /* number of files processed for output */
char g_procd_files[MAX_L7_BANDS * NUM_L7_RES * MAX_L7_VOLS][CMLEN];
                             /* array of filenames that have been created for
                                output from L7HDF2LAS */
int g_delflg = FALSE;        /* flag to represent the global DELFLG flag
                                (TRUE or FALSE) */
int g_mounted = FALSE;       /* flag to specify if a CD is mounted */
long g_unit;                 /* CD unit number */
int g_mediaflag;             /* flag to specify DISK or CD */
int g_num_link_files = 0;    /* number of link files that were created */
char g_link_files[MAXFILE][CMLEN];
                             /* name of the link files that were created for
                                CD processing */
int g_links_created;         /* links were created for file names */

/* Define the variables to allow a periodic message to be issued during 
   processing.
   --------------------------------------------------------------------*/
extern long curcat;                 /* current processing band */


/* Helper routine to return TRUE (non-zero) if a band is a particular
   resolution type.
   ------------------------------------------------------------------*/
static int band_is_res_type
(
    int band,                 /* I: band number */
    enum res_type res_index   /* I: resolution type index (VIS, THM, PAN) */
)
{
    if (res_index == L7_VIS)
    {
        /* check if the band is a visible band */
        if ((band == 1) || (band == 2) || (band == 3) || (band == 4) ||
            (band == 5) || (band == 7))
        {
            return 1;
        }
    }
    else if (res_index == L7_THM)
    {
        /* check if the band is a thermal band */
        if ((band == 61) || (band == 62))
            return 1;
    }
    else if (res_index == L7_PAN)
    {
        /* check if the band is the pan band */
        if (band == 8)
            return 1;
    }

    /* made it here, so the band is not part of the group */
    return 0;
}

/* Helper routine to count the number of bands with the given resolution
   and in the current set of bands requested in "bands".
   ---------------------------------------------------------------------*/
static int count_bands_in_group
(
    enum res_type res_index,/* I: resolution type index (VIS, THM, PAN) */
    const int *bands_avail, /* I: array of bands available for processing */
    int num_bands_avail,    /* I: number of bands in bands_avail */
    const int *bands,       /* I: list of bands in the order requested */
    int start_band,         /* I: index in bands for first band of group*/
    int end_band            /* I: index in bands for last band of group */
)
{
    int band_count = 0;
    int band_index;

    /* loop through the list of available bands, counting the ones that are
       in the desired group */
    for (band_index = 0; band_index < num_bands_avail; band_index++)
    {
        if (band_is_res_type(bands_avail[band_index],res_index))
        {
            /* band is the requested resolution, so see if it is 
               in the requested output file */
            int band;
            for (band = start_band; band < end_band; band++)
            {
                if (bands[band] == bands_avail[band_index])
                {
                    /* band is one requested for this output file */
                    band_count++;
                    break;
                }
            }
        }
    }

    return band_count;
}


/* main routine for the l7hdf2las application 
  -------------------------------------------*/
int main ()
{
  char acquisition_date[9]; /* acquisition date listed in the metadata
                               (yyyymmdd) */
  char append_str[CMLEN];   /* string to be appended to the filename */
  char avail_file_list[MAXFILE][CMLEN];
                            /* list of filenames for the bands_avail list */
  char comment[MAX_L7_VOLS][CMLEN];
                            /* array of user comments for the operator */
  char directory[CMLEN];    /* directory portion of filespec */
  char drivename[CMLEN]=""; /* name of the CD drive (initialize to no drive) */
  char errmsg[CMLEN];       /* error message string */
  char extension[CMLEN];    /* extension portion of the file name */
  char hdfname[CMLEN];      /* name of the HDF file found on the CD (without
                               the drivename) */
  char infile[CMLEN];       /* input file name (UNIX format) */
  char infomsg[CMLEN];      /* informational message string */
  char infomsg2[CMLEN];     /* second informational message string */
  char minmax[4];           /* minmax flag */
  char out_filename[CMLEN]; /* output filename */
  char outfile[MAX_L7_BANDS][CMLEN];
                            /* array of output file names from OUT (Unix
                               format) */
  char output_dir[CMLEN];   /* output directory used for creating links when
                               processing CDs */
  char processing_system[6];/* processing system IAS, LPGS, NLAPS or NONE */
  char proj_units[12];      /* units of image projection corners */
  char root[CMLEN];         /* root portion of the file name */
  char saved_infile[CMLEN]; /* saved input file name (UNIX format) */
  char sensor_id[15];       /* sensor id listed in the metadata */
  char tmpstr[CMLEN];       /* temporary string holder */

  int bands[MAX_L7_BANDS];  /* array of bands to process */
  int bandsavail_index;     /* index for the bands_avail array */
  int bands_avail[MAX_L7_BANDS];
                            /* array of bands from the bands array that are
                               available (in ascending order) */
  int bands_index;          /* keep track of current band in the bands list */
  int bands_index_save;     /* keep track of the bands_index starting point
                               for this nbands group */
  int band_in_output_image; /* keep track of the number of bands output
                               to the current image */
  int bnd_count;            /* keep track of the number of bands to be output
                               for this resolution in the nbands group */
  int bnd_found;            /* specifies if a band for the current resolution
                               in nbands group has been found (TRUE, FALSE) */
  int bnd_indx;             /* index for the current band in the array of 9
                               possible bands (b1=0, b2=1, b3=2, b4=3, b5=4,
                               b61=5, b62=6, b7=7, b8=8) */
  int curr_bnds[MAX_L7_BANDS];
                            /* keep track of the list of bands that will be
                               processed for this group */
  int currbndinnbands_index;/* the current band in nbands set */
  int default_bands;        /* flag to specify if this application will
                               process the default set of bands (TRUE =
                               process the default bands, FALSE = bands were
                               specified) */
  int found_lpgs_meta=FALSE;/* flag to specify that the LPGS metadata was
                               found for this file (TRUE, FALSE) */
  int found_product_meta=FALSE;
                            /* flag to specify that the product metadata was
                               found for this file (TRUE, FALSE) */
  int i, j;                 /* looping variables */
  int idtype;               /* input data type of the product (EBYTE=1,
                               EWORD=2, ELONG=3, EREAL=4) */
  int many_to_one;          /* flag to specify if all bands will be written
                               to one output file */
  int meta_bands[9];        /* bands listed for this product in the metadata
                               (this list is sorted in ascending order) */
  int nbands[MAX_L7_BANDS]; /* array of NBANDS values */
  int nbands_bnd_count;     /* keep track of the number of bands processed
                               for this nbands group */
  int nbands_index;         /* index for looping through nbands */
  int nbands_same_res[MAX_L7_BANDS];
                            /* array of same_res values, one for each nbands
                               value, each to specify if all the bands in the
                               nbands group are of the same resolution (TRUE,
                               FALSE) */
  int nbands_1g;            /* number of bands in the 1G image */
  int nl_1g[NBANDS];        /* number of lines in the 1G image for each band */
  int ns_1g[NBANDS];        /* number of samples in 1G image for each band */
  int num_bands;            /* number of values in bands */
  int num_bands_avail;      /* number of bands in bands_avail */
  int num_meta_bands;       /* number of bands in the metadata */
  int num_nbands;           /* number of values in nbands */
  int num_out;              /* number of output file names */
  int num_vols;             /* number of CD volumes to process */
  int num_window;           /* number of window values */
  int process_pan;          /* flag specifying pan bands will be processed
                               (TRUE or FALSE) */
  int process_thm;          /* flag specifying thermal bands will be
                               processed (TRUE or FALSE) */
  int process_vis;          /* flag specifying visible bands will be
                               processed (TRUE or FALSE) */
  int product_type;         /* type of HDF file this is (L0R, L1R, L1G, L1T) */
  int proj_code;            /* integer value to specify the projection */
  enum res_type res_index;  /* index for the resolutions (VIS, THM, PAN) */
  int srch_found;           /* specifies if any bands were found on this CD */
  int bands_output=0;       /* bands output from the current CD */
  int display_updating=0;   /* flag to indicate the display update is running */
  int ttl_bnd_count;        /* keep track of the total number of bands
                               processed overall */
  int ttl_bnds_procd[MAX_L7_BANDS];
                            /* keep track of all the bands processed overall */
  int vol_index;            /* current volume being processed */
  int window_specd;         /* flag to specify if the window was specified */
  int window_units;         /* units of input window (L7_DEG, L7_DMS, L7_LS,
                               L7_PRO) */

  long acc;                 /* image file open access */
  long datum_code;          /* integer value to specify the datum */
  long max_nlines_write;    /* maximum number of lines to write to the LAS
                               image */
  long msgtim;              /* time between display messages */
  long msgflag=ONECAT;      /* display message flag */
  long nl;                  /* LAS image file number of lines */
  long ns;                  /* LAS image file number of samples */
  long num_total_lines;     /* total number of lines in the original image
                               for the current resolution */
  long num_total_samps;     /* total number of samples in the original image
                               for the current resolution */
  long odtype;              /* data type of output image
                               (0=SAME, EBYTE=BYTE, EWORD=I*2, ELONG=I*4,
                                EREAL=R*4) */
  long opt[2];              /* image file open option */
  long outwindow[4];        /* array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for the current
                               resolution */
  long pan_nl;              /* number of lines in image for pan bands */
  long pan_ns;              /* number of samples in image for pan bands */
  long pan_valid_window[4]; /* array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for pan bands */
  long sl;                  /* LAS image file starting line */
  long ss;                  /* LAS image file starting sample */
  long std_out = 0;         /* standard output */
  long thm_nl;              /* number of lines in image for thermal bands */
  long thm_ns;              /* number of samples in image for thermal bands */
  long thm_valid_window[4]; /* array of valid window values in line/sample
                               coordinates (SL,SS,NL,NS) for thermal bands */
  long vis_nl;              /* number of lines in image for visible bands */
  long vis_ns;              /* number of samples in image for visible bands */
  long vis_valid_window[4]; /* array of valid window values in line/sample
                               coordinates (SL,SS,NL,NS) for visible bands */
  long zero = 0;            /* zero constant */
  long zone_code;           /* UTM zone, for UTM projections */

  double pixel_size;        /* pixel size for the current resolution */
  double proj_parms[15];    /* 15 projection parameters for GCTP */
  double ul_corner[2];      /* projection coordinates of upper left corner
                               in the image (y,x) in units of proj_units */
  double ur_corner[2];      /* projection coordinates of upper right corner
                               in the image (y,x) in units of proj_units */
  double ll_corner[2];      /* projection coordinates of lower left corner
                               in the image (y,x) in units of proj_units */
  double lr_corner[2];      /* projection coordinates of lower right corner
                               in the image (y,x) in units of proj_units */
  double pan_pixel_size;    /* pixel size for pan bands */
  double thm_pixel_size;    /* pixel size for thermal bands */
  double vis_pixel_size;    /* pixel size for visible bands */
  double window[4];         /* image window (If window_units is L7_LS then
                               sl, ss, nl, ns.  If window_units is L7_DMS or
                               L7_DEG then UL_lat, UL_long, LR_lat, LR_long.
                               If window_units is L7_PRO then UL_Y, UL_X,
                               LR_Y, LR_X.) */

  AccessMode file_access;   /* type of access for the file open */
  BandNumber bandno_1g[NBANDS];
                            /* enumerated band numbers in the 1G image */
  DataType dtype_1g[NBANDS];/* data type in the 1G image for each band */

  xxx_ImageryInfo_TYPE l0r_fileptr;
                            /* HDF file pointer for L0R product */
  HDFIO *l1rg_fileptr=NULL; /* HDF file pointer for L1R, L1G, or L1T product */
  struct PARBLK vblock;     /* TAE parameter block */
  struct FDESC *out_fileptr;/* LAS file descriptor for the output file */

  char *band_names[NBANDS] = {XXX_BAND1, XXX_BAND2, XXX_BAND3, XXX_BAND4,
    XXX_BAND5, XXX_BAND6L, XXX_BAND6H, XXX_BAND7, XXX_BAND81};
                            /* band names for the xxx_OpenImagery function */
  int links_needed = 0;     /* flag to indicate links need to be created to
                               handle lower case file names */
  char *tempptr=NULL;       /* temporary pointer to a character */
  double software_version;  /* IAS or LPGS processing software version */
  char processing_software[5]; /* IAS or LPGS software */

  /* Get the parameter block from LAS
     --------------------------------*/
  c_inlas (&vblock, &std_out);

  /* Input the user parameters from TAE
     ----------------------------------*/
  if (get_parms (&vblock, MAX_L7_BANDS, comment, &g_mediaflag, &num_vols,
      infile, bands, &num_bands, &default_bands, nbands,
      &num_nbands, &window_units, window, &num_window, &window_specd, outfile,
      &num_out, &many_to_one, &odtype, nbands_same_res, &process_vis,
      &process_thm, &process_pan, &g_delflg, &msgtim, minmax, 
      processing_system) != E_SUCC)
  {
    c_lasmsg (TRUE, "l7hdf2las-get_parms",
      "Error parsing the TAE parameter block for L7HDF2LAS parameters");
  }

  /* Set the signal handler for the standard TAE signals so that they are
     caught and we cleanup all the files that have been created
     --------------------------------------------------------------------*/
  if (c_install_cleanup_handler (signal_cleanup) != E_SUCC)
  {
    c_lasmsg (TRUE, "l7hdf2las-signal",
      "Error setting the signal handlers for the TAE standard signals");
  }

  /* If we are processing a CD, then mount the first CD.
     ---------------------------------------------------*/
  if (g_mediaflag == CDMEDIA)
  {
    if (mount_cd (comment, 0 /* mount first volume */, &g_mounted, drivename,
        &g_unit) != E_SUCC)
    {
      c_lasmsg (TRUE, "l7hdf2las-mount_cd", "Error mounting the CD");
    }
  }

  /* find the input file name 
     ------------------------*/
  if (find_infilename (drivename, infile, hdfname) != E_SUCC)
  {
    unmount_cd (&g_mounted, &g_unit);
    c_lasmsg (TRUE, "l7hdf2las-find_infilename",
        "Error reading filenames from the CD");
  }

  /* Set the infile name for processing
     ----------------------------------*/
  strcpy (infile, hdfname);

  /* Determine if links need to be created to compensate for lower case in 
     the file names.
     -----------------------------------------------------------------------*/
  tempptr = strrchr(infile,'/');
  if (tempptr == NULL)
    tempptr = infile;
  else
    tempptr++;
  /* if any part of the base name is lower case, links are needed */
  while (((*tempptr != '\0') && (*tempptr != '.')) && !links_needed)
  {
    if (islower(*tempptr))
      links_needed = 1;
    tempptr++;
  }

  /* If processing CDs, then the drivename must be attached to the filename.
     -----------------------------------------------------------------------*/
  if (g_mediaflag == CDMEDIA)
  {
    sprintf (tmpstr, "%s%s", drivename, infile);
    strcpy (infile, tmpstr);
  }

  /* Get the output directory for creating links in CD processing.
     -------------------------------------------------------------*/
  c_splitname (outfile[0], output_dir, root, extension);

  /* Read the appropriate metadata file to determine the bands in this
     product. First try the L1R, L1G, or L1T product, then try the L0R
     product. Bands for L1R, L1G, and L1T are stored in the LPGS metadata
     file (.MTL) This read will also provide some of the information needed
     for the LAS DDR.
     ----------------------------------------------------------------------*/
  if (read_lpgs_meta (infile, drivename, output_dir, links_needed, meta_bands,
      &num_meta_bands, sensor_id, &product_type, &idtype, acquisition_date,
      ul_corner, ur_corner, ll_corner, lr_corner, proj_units, &proj_code,
      &zone_code, &datum_code, proj_parms, &vis_pixel_size, &thm_pixel_size,
      &pan_pixel_size, &vis_nl, &thm_nl, &pan_nl, &vis_ns, &thm_ns, &pan_ns,
      &found_lpgs_meta,&software_version,processing_software,
      processing_system) != E_SUCC)
  {
    /* If the LPGS metadata was not found then check for the L0R product
       metadata
       -----------------------------------------------------------------*/
    if (!found_lpgs_meta)
    {
      /* Try reading the L0R product.  Bands for L0R are stored in the
         product metadata file (_MTP)
         -------------------------------------------------------------*/
      if (read_product_meta (infile, drivename, output_dir, links_needed,
          meta_bands, &num_meta_bands, sensor_id, &product_type, &idtype, 
          acquisition_date, proj_parms, &vis_pixel_size, &thm_pixel_size,
          &pan_pixel_size, &vis_nl, &thm_nl, &pan_nl, &vis_ns, &thm_ns, &pan_ns,
          &found_product_meta) != E_SUCC)
      {
        if (!found_product_meta)
        {
          unmount_cd (&g_mounted, &g_unit);
          sprintf (errmsg,
            "Unable to find the L0R, L1R, L1G, or L1T metadata for file: %s",
            infile);
          c_lasmsg (TRUE, "l7hdf2las-read_meta", errmsg);
        }
        else
        {
          unmount_cd (&g_mounted, &g_unit);
          c_lasmsg (TRUE, "l7hdf2las-read_product",
            "Error reading the product metadata");
        }
      }
    }
    else
    {
      unmount_cd (&g_mounted, &g_unit);
      c_lasmsg (TRUE, "l7hdf2las-read_lpgs",
        "Error reading the LPGS metadata");
    }
  }

  /* Set the output data type if it is specified as SAME
     ---------------------------------------------------*/
  if (odtype == SAME)
    odtype = idtype;

  /* If the product type is not L1G or L1T, then the user cannot specify
     a window in any other units than line/sample.
     -------------------------------------------------------------------*/
  if ((product_type != L1G) && (product_type != L1T) && (window_units != L7_LS))
  {
    unmount_cd (&g_mounted, &g_unit);
    c_lasmsg (TRUE, "l7hdf2las-window",
      "Product is not L1G or L1T thus the window must be in line/sample units");
  }

  /* If the user did not specify any bands, then set the bands to be the
     bands listed in the metadata
     -------------------------------------------------------------------*/
  if (default_bands)
  {
    if (set_bands_from_meta (meta_bands, num_meta_bands, bands, &num_bands,
        nbands, &num_nbands, nbands_same_res, &process_vis, &process_thm,
        &process_pan) != E_SUCC)
    {
      unmount_cd (&g_mounted, &g_unit);
      c_lasmsg (TRUE, "l7hdf2las-metabands",
        "Error setting bands from the metadata");
    }
  }

  /* Convert the window and verify that the window coordinates are valid.
     This needs to be done for each resolution to be processed if the units
     are not LS.
     ----------------------------------------------------------------------*/
  if (convert_window_for_each_res (window_units, window, num_window,
      window_specd, product_type, proj_code, zone_code, datum_code,
      vis_pixel_size, thm_pixel_size, pan_pixel_size, proj_parms, ul_corner,
      ur_corner, ll_corner, lr_corner, proj_units, vis_nl, thm_nl, pan_nl,
      vis_ns, thm_ns, pan_ns, process_vis, process_thm, process_pan,
      vis_valid_window, thm_valid_window, pan_valid_window) != E_SUCC)
  {
    unmount_cd (&g_mounted, &g_unit);
    c_lasmsg (TRUE, "l7hdf2las-convert_window_res",
      "Error converting the user-specified window to line/sample");
  }

  /* Provide some informational messages before processing
     -----------------------------------------------------*/
  output_info_msg (product_type, g_mediaflag, infile, process_vis, process_thm,
    process_pan, vis_nl, vis_ns, thm_nl, thm_ns, pan_nl, pan_ns,
    vis_pixel_size, thm_pixel_size, pan_pixel_size);

  /* Loop through each CD volume (only one volume for files on disk) looking
     for every band listed in BANDS
     NOTE: At this point the first CD should still be mounted.
     -----------------------------------------------------------------------*/ 
  ttl_bnd_count = 0;  /* keep track of ttl number of bands processed overall */
  g_num_procd_files = 0;  /* keep track of number of files opened for output */
  for (vol_index = 0; vol_index < num_vols; vol_index++)
  {
    /* If dealing with CDs, then set up links to the files on the CD.  Some of
       the Landsat-7 CD media has been released with lower-case filenames on
       CDs.  These lower-case filenames cause the HDF libraries to not read
       the band imagery correctly.  Thus a link of the files on the CD to
       upper-case versions of these files in the current directory will be
       made temporarily for processing the band imagery.  Note that infile is
       now changed to have a directory name of the current directory and the
       filename will be in upper case (i.e. infile is now the link file).
       ----------------------------------------------------------------------*/
    if (!g_links_created && 
        ((g_mediaflag == CDMEDIA) || links_needed))
    {
      strcpy (saved_infile, infile);
      if (create_links (infile, output_dir, product_type, g_link_files,
          &g_num_link_files) != E_SUCC)
      {
        cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
        unmount_cd (&g_mounted, &g_unit);
        c_lasmsg (TRUE, "l7hdf2las-createlinks",
          "Error creating the upper-case symbolic links for CD processing");
      }
      g_links_created = 1;
    }

    /* Determine which of the BANDS are available on this CD or disk
       -------------------------------------------------------------*/
    if (srch_dir (infile, bands, num_bands, product_type, bands_avail,
        &num_bands_avail, avail_file_list, &srch_found) != E_SUCC)
    {
      cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
      if (g_links_created)
        delete_links (g_link_files, &g_num_link_files);
      unmount_cd (&g_mounted, &g_unit);
      c_lasmsg (TRUE, "l7hdf2las-srch_dir",
        "Error searching for band files on CD or disk");
    }

    /* Process the available bands if any were found.  Provide some
       informational messages for this volume and set of bands.
       ------------------------------------------------------------*/
    if (g_mediaflag == CDMEDIA)
    {
      sprintf (infomsg, "Processing volume number %d", vol_index + 1);
      c_lasmsg (FALSE, "informational", infomsg);
    }

    sprintf (infomsg, "%d of the requested band(s) were found: ",
      num_bands_avail);
    for (i = 0; i < num_bands_avail; i++)
    {
      sprintf (infomsg2, "%d ", bands_avail[i]);
      strcat (infomsg, infomsg2);
    }
    c_lasmsg (FALSE, "informational", infomsg);

    /* Loop through the NBANDS band combinations
       -----------------------------------------*/
    bands_index = 0;  /* keep track of the current band in the bands list */
    for (nbands_index = 0; nbands_index < num_nbands; nbands_index++)
    {
      nbands_bnd_count = 0;  /* keep track of the number of bands processed
                                for this nbands group */
      bands_index_save = bands_index;  /* keep track of bands_index starting
                                          point for this nbands group */
      band_in_output_image = 1; /* keep track of the number of bands output
                                   to this image (one-based) */

      /* Loop through the possible resolutions in this nbands set
         --------------------------------------------------------*/
      for (res_index = L7_VIS; res_index <= L7_PAN; res_index++)
      {
        /* Only process the resolutions that the user specified
           ----------------------------------------------------*/
        if (res_index == L7_VIS && !process_vis)
          continue;
        else if (res_index == L7_THM && !process_thm)
          continue;
        else if (res_index == L7_PAN && !process_pan)
          continue;

        bands_index = bands_index_save;  /* reset it for this resolution */
        bnd_count = 0;  /* keep track of the number of bands to be output for
                           this resolution in the nbands group */
        bnd_found = FALSE;  /* specify that a band for the current resolution
                               in the nbands group has not yet been found */

        /* Determine the output filename.  Split up the filename so that
           base name suffixes can be added for multi-resolution and
           multi-volume imagery, if necessary.
           -------------------------------------------------------------*/
        c_splitname (outfile[nbands_index], directory, root, extension);

        /* Determine the pixel size and the output window for this resolution
           ------------------------------------------------------------------*/
        if (res_index == L7_VIS)
        {
          pixel_size = vis_pixel_size;
          num_total_lines = vis_nl;
          num_total_samps = vis_ns;
          for (i = 0; i < 4; i++)
            outwindow[i] = vis_valid_window[i];
        }
        else if (res_index == L7_THM)
        {
          pixel_size = thm_pixel_size;
          num_total_lines = thm_nl;
          num_total_samps = thm_ns;
          for (i = 0; i < 4; i++)
            outwindow[i] = thm_valid_window[i];
        }
        else if (res_index == L7_PAN)
        {
          pixel_size = pan_pixel_size;
          num_total_lines = pan_nl;
          num_total_samps = pan_ns;
          for (i = 0; i < 4; i++)
            outwindow[i] = pan_valid_window[i];
        }

        /* If the bands in this nbands group are not of the same resolution
           then _vis, _thm, and _pan must be appended to the base filename.
           ----------------------------------------------------------------*/
        if (!nbands_same_res[nbands_index])
        {
          strcpy (append_str, res_suffixes[res_index]);

          /* Also reset the band_in_output_image counter, since we are
             outputting to a new file.
             ---------------------------------------------------------*/
          band_in_output_image = 1;  /* one-based */
        }
        else
          strcpy (append_str, "");

        /* If this is a multi-volume image then the current volume number
           must be appended to the base filename.
           --------------------------------------------------------------*/
        if (num_vols > 1)
          sprintf (append_str, "%s_vol%d", append_str, vol_index + 1);

        sprintf (out_filename, "%s%s%s.%s", directory, root, append_str,
          extension);

        /* Loop through the bands in the current nbands group
           --------------------------------------------------*/
        for (currbndinnbands_index = 0;
             currbndinnbands_index < nbands[nbands_index];
             currbndinnbands_index++)
        {
          /* if the band at band_index isn't part of the group, skip to the 
             next band */
          if (!band_is_res_type(bands[bands_index],res_index))
          {
            bands_index++;
            continue;
          }

          /* Loop through the list of available bands
             ----------------------------------------*/
          for (bandsavail_index = 0; bandsavail_index < num_bands_avail;
               bandsavail_index++)
          {
            if (bands[bands_index] == bands_avail[bandsavail_index])
            {
              /* If no previous bands for this resolution in the nbands
                 group have been found, then open the output file.
                 ------------------------------------------------------*/
              if (!bnd_found)
              {
                long band_count = count_bands_in_group(res_index,bands_avail,
                                   num_bands_avail,bands,bands_index_save,
                                   bands_index_save + nbands[nbands_index]);

                acc = IWRITE;
                opt[0] = odtype;  /* convert the data from idtype to odtype
                                     on output to the file */
                opt[1] = 0;
                sl = ss = 1;
                nl = outwindow[NL];
                ns = outwindow[NS];
                max_nlines_write = 32 * 10;  /* maximum of 32 lines per scan
                                                times 10 scans */
                if (c_eopenr (&out_fileptr, out_filename, &band_count, &sl, &ss,
                  &nl, &ns, &acc, (long *) &idtype, opt, &max_nlines_write)
                  != E_SUCC)
                {
                  cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
                  if (g_links_created)
                    delete_links (g_link_files, &g_num_link_files);
                  unmount_cd (&g_mounted, &g_unit);
                  sprintf (errmsg,
                    "Error opening the output LAS image: %s", out_filename);
                  c_lasmsg (TRUE, "l7hdf2las-openLASimagery", errmsg);
                }

                /* Add the output filename to our list of files that have
                   been created for this application
                   ------------------------------------------------------*/
                strcpy (g_procd_files[g_num_procd_files++], out_filename);
              }

              /* start the processing display message if not already running 
                 -----------------------------------------------------------*/
              if (!display_updating)
              {
                curcat = bands_output + 1;
                curent = 1;
                c_inittm("L7HDF2LAS","Processed","line","bands"," ",
                          &outwindow[NL], (long *)&num_bands_avail,
                          &msgtim,&msgflag);
                c_startm();
                display_updating = 1;
              }

              /* The band was found so add it to the list of bands to process
                 ------------------------------------------------------------*/
              bnd_found = TRUE;
              nbands_bnd_count++;
              curr_bnds[bnd_count] = bands[bands_index];  /* keep track of
                the list of bands that will be processed for this group */
              bnd_count++;
              ttl_bnds_procd[ttl_bnd_count] = bands[bands_index];  /* keep
                track of all the bands processed overall */
              ttl_bnd_count++;

              /* Figure out which band we are processing
                 ---------------------------------------*/
              switch (bands[bands_index])
              {
                case 1:
                case 2:
                case 3:
                case 4:
                case 5:
                  bnd_indx = bands[bands_index] - 1;
                  break;

                case 61:
                  bnd_indx = 5;
                  break;

                case 62:
                  bnd_indx = 6;
                  break;

                case 7:
                case 8:
                  bnd_indx = bands[bands_index];
                  break;
              }

              /* If dealing with an L1R, L1G, or L1T image, then the band
                 has to be an enumerated band.
                 --------------------------------------------------------*/
              if ((product_type == L1R) || (product_type == L1G) ||
                  (product_type == L1T))
              {
                switch (bands[bands_index])
                {
                  case 1: bandno_1g[0] = b1; break;
                  case 2: bandno_1g[0] = b2; break;
                  case 3: bandno_1g[0] = b3; break;
                  case 4: bandno_1g[0] = b4; break;
                  case 5: bandno_1g[0] = b5; break;
                  case 61: bandno_1g[0] = b6l; break;
                  case 62: bandno_1g[0] = b6h; break;
                  case 7: bandno_1g[0] = b7; break;
                  case 8: bandno_1g[0] = b8; break;
                }
              }

              /* Open the file avail_file_list(bandsavail_index) for reading.
                 Each product type has a different HDF format and therefore
                 will require separate handling.
                 ------------------------------------------------------------*/
              if (product_type == L0R)
              {
                /* The HDF filename should have access to each of the
                   individual bands, thus use the HDF filename instead of
                   the _B## filename for opening.
                   ------------------------------------------------------*/
                if (xxx_OpenImagery (infile, DFACC_RDONLY,
                    band_names[bnd_indx], 0 /* write position - not used */,
                    NULL /* external file pointer */, &l0r_fileptr, errmsg)
                    == ERROR)
                {
                  c_eclose (&out_fileptr);
                  cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
                  if (g_links_created)
                    delete_links (g_link_files, &g_num_link_files);
                  unmount_cd (&g_mounted, &g_unit);
                  c_lasmsg (FALSE, "l7hdf2las-openimagery", errmsg);
                  sprintf (errmsg,
                    "Error opening the input L7 HDF image (L0R): %s", infile);
                  c_lasmsg (TRUE, "l7hdf2las-openimagery", errmsg);
                }
              }
              else
              {
                /* L1R, L1G, and L1T products
                   --------------------------*/
                nbands_1g = 1;
                file_access = (AccessMode) READ;
                l1rg_fileptr = gxx_open_1g_image ((const char *) infile,
                  file_access, &nbands_1g, bandno_1g, dtype_1g, nl_1g, ns_1g);
                if (l1rg_fileptr == NULL)
                {
                  c_eclose (&out_fileptr);
                  cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
                  if (g_links_created)
                    delete_links (g_link_files, &g_num_link_files);
                  unmount_cd (&g_mounted, &g_unit);
                  sprintf (errmsg,
                    "Error opening the input L7 HDF image (L1R/L1G/L1T): %s",
                    infile);
                  c_lasmsg (TRUE, "l7hdf2las-openimagery", errmsg);
                }
              }

              /* update the current band being ingested */
              curcat = bands_output + 1;

              /* Read the HDF file using the outwindow window coords and
                 output the imagery to the LAS output file.  The input data
                 type needs to be converted to the output data type.
                 ----------------------------------------------------------*/
              if (readHDF_writeLAS_image (&l0r_fileptr, l1rg_fileptr,
                product_type, idtype, bands[bands_index], band_in_output_image,
                outwindow, num_total_samps, out_fileptr) != E_SUCC)
              {
                c_eclose (&out_fileptr);
                if (product_type == L0R)
                  xxx_CloseImagery (&l0r_fileptr, errmsg);
                else
                {
                  gxx_close_1g_image (l1rg_fileptr);
                  free (l1rg_fileptr);
                }
                cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
                if (g_links_created)
                  delete_links (g_link_files, &g_num_link_files);
                unmount_cd (&g_mounted, &g_unit);
                c_lasmsg (TRUE, "l7hdf2las-readHDF_writeLAS",
                  "Error reading the HDF file and writing to the LAS image");
              }
              
              /* Increment the band_in_output_image counter so it is ready
                 for the next band in the image.
                 ---------------------------------------------------------*/
              band_in_output_image++;

              /* update the number of bands output for the current data set */
              bands_output++;

              /* Close the image file pointer
                 ----------------------------*/
              if (product_type == L0R)
              {
                if (xxx_CloseImagery (&l0r_fileptr, errmsg) != XXXSUCCESS)
                {
                  c_eclose (&out_fileptr);
                  cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
                  if (g_links_created)
                    delete_links (g_link_files, &g_num_link_files);
                  unmount_cd (&g_mounted, &g_unit);
                  c_lasmsg (FALSE, "l7hdf2las-closeimagery", errmsg);
                  c_lasmsg (TRUE, "l7hdf2las-closeimagery",
                    "Error closing the input L7 HDF image (L0R)");
                }
              }
              else
              {
                /* Close the image and free the memory for the file pointer
                   --------------------------------------------------------*/
                if (gxx_close_1g_image (l1rg_fileptr) != XXXSUCCESS)
                {
                  c_eclose (&out_fileptr);
                  cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
                  if (g_links_created)
                    delete_links (g_link_files, &g_num_link_files);
                  unmount_cd (&g_mounted, &g_unit);
                  c_lasmsg (TRUE, "l7hdf2las-closeimagery",
                    "Error closing the input L7 HDF image (L1R/L1G/L1T)");
                }
              }

              break;  /* we have found our band and processed it */
            }
          }  /* for bandsavail_index */

          bands_index++;
        }  /* for currbndinnbands_index */

        /* stop the display message if it is running
           -----------------------------------------*/
        if (display_updating)
        {
          c_stoptm();
          display_updating = 0;
        }

        /* We have looped through the bands_avail list for each of the bands
           in the current nbands combination for the current resolution.
           If any bands were found for this resolution in the current nbands
           set, then close the output file and create a DDR for the bands
           that were found.
           -----------------------------------------------------------------*/
        if (bnd_found)
        {
          if (c_eclose (&out_fileptr) != E_SUCC)
          {
            cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
            if (g_links_created)
              delete_links (g_link_files, &g_num_link_files);
            unmount_cd (&g_mounted, &g_unit);
            c_lasmsg (TRUE, "l7hdf2las-closeLASimage",
              "Error closing the output LAS image");
          }

          if (write_ddr (out_filename, curr_bnds, bnd_count, odtype,
            proj_units, proj_code, zone_code, datum_code, pixel_size,
            proj_parms, ul_corner, ur_corner, ll_corner, lr_corner,
            acquisition_date, sensor_id, outwindow, num_total_lines,
            num_total_samps, product_type, minmax,software_version,
            processing_software) != E_SUCC)
          {
            cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
            if (g_links_created)
              delete_links (g_link_files, &g_num_link_files);
            unmount_cd (&g_mounted, &g_unit);
            c_lasmsg (TRUE, "l7hdf2las-write_ddr",
              "Error writing the DDR for %s", out_filename);
          }

          if (c_hstout (&vblock, out_filename, "", &zero, "") != E_SUCC)
          {
            cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
            if (g_links_created)
              delete_links (g_link_files, &g_num_link_files);
            unmount_cd (&g_mounted, &g_unit);
            c_lasmsg (TRUE, "l7hdf2las-hstout",
              "Error writing the history file for %s", out_filename);
          }
        }

        if (nbands_bnd_count == nbands[nbands_index])
          break;  /* we are done processing all the bands in the current
                     nbands group */

        if (ttl_bnd_count == num_bands)
          break;  /* we are done processing all the bands */
      }  /* for res_index */

      /* Increment the bands_index by the number of bands in the current
         nbands group so that we are ready to process the next nbands group
         ------------------------------------------------------------------*/
      bands_index = bands_index_save + nbands[nbands_index];

      if (ttl_bnd_count == num_bands)
        break;  /* we are done processing all the bands */
    }  /* for nbands_index */

    /* If processing a CD, then the links need to be deleted and infile needs
       to be reset
       ----------------------------------------------------------------------*/
    if (g_links_created)
    {
      strcpy (infile, saved_infile);
      if (delete_links (g_link_files, &g_num_link_files))
      {
        cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
        unmount_cd (&g_mounted, &g_unit);
        c_lasmsg (TRUE, "l7hdf2las-deletelinks",
          "Error deleting the link files created for CD processing");
      }
      g_links_created = 0;
    }

    /* Unmount if processing a CD
       --------------------------*/
    if (g_mediaflag == CDMEDIA && g_mounted)
    {
      if (unmount_cd (&g_mounted, &g_unit) != E_SUCC)
      {
        cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
        c_lasmsg (TRUE, "l7hdf2las-unmount_cd", "Error unmounting the CD");
      }
    }

    if (ttl_bnd_count == num_bands)
      break;  /* we are done processing all the bands */

    /* Mount the next CD as long as we aren't on the last volume
       ---------------------------------------------------------*/
    if ((g_mediaflag == CDMEDIA) && (vol_index != num_vols - 1))
    {
      if (mount_cd (comment, vol_index+1 /* mount next volume */, &g_mounted,
        drivename, &g_unit) != E_SUCC)
      {
        cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
        c_lasmsg (TRUE, "l7hdf2las-mount_cd", "Error mounting the CD");
      }

      /* clear the count of bands output since each volume is counted 
         separately */
      bands_output = 0;
    }
  }  /* for vol_index */

  /* If this is a multi-volume product then we need to merge the _vol# files
     together for each of the output files
     -----------------------------------------------------------------------*/
  if (num_vols > 1)
  {
    if (merge_vol_files (&vblock, outfile, num_nbands, nbands_same_res,
      msgtim, minmax, g_delflg, &g_num_procd_files, g_procd_files) != E_SUCC)
    {
      cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
      c_lasmsg (TRUE, "l7hdf2las-merge_vol_files",
        "Error merging the _vol files");
    }
  }

  /* If this is a L1G or L1T product, it is possible that the resolution
     (pixel size) for the visible, thermal, and/or pan bands might be the
     same. If any of these products have the same resolution, then they need
     to be merged together into one product. The suffixes will be based on
     which bands are in the product, but will be in the order of _vis_thm_pan.
     -------------------------------------------------------------------------*/
  if ((product_type == L1G) || (product_type == L1T))
  {
    if (merge_res_files (&vblock, outfile, num_nbands, vis_pixel_size,
      thm_pixel_size, pan_pixel_size, process_vis, process_thm, process_pan,
      msgtim, minmax, g_delflg, &g_num_procd_files, g_procd_files) != E_SUCC)
    {
      cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);
      c_lasmsg (TRUE, "l7hdf2las-merge_res_files",
        "Error merging the _res files");
    }
  }

  /* Check the bands processed with the bands that were requested by the
     user.  Output a warning message for any bands that were requested but
     not processed.
     ---------------------------------------------------------------------*/
  for (i = 0; i < num_bands; i++)
  {
    bnd_found = FALSE;
    for (j = 0; j < ttl_bnd_count; j++)
    {
      if (bands[i] == ttl_bnds_procd[j])
      {
        bnd_found = TRUE;
        break;
      }
    }

    if (!bnd_found)
    {
      sprintf (errmsg, "Band %d was not processed for Landsat 7 HDF image: %s",
        bands[i], infile);
      c_lasmsg (FALSE, "l7hdf2las-notprocessed",
        errmsg);
    }
  }

  /* Remove our user-defined cleanup handlers
     ----------------------------------------*/
  if (c_remove_cleanup_handler () != E_SUCC)
    return (EXIT_FAILURE);

  /* Output a message for successful completion
     ------------------------------------------*/
  c_complt ("L7HDF2LAS", g_procd_files, &g_num_procd_files);

  return (EXIT_SUCCESS);
}
