#ifndef L7HDF2LAS_H
#define L7HDF2LAS_H

#include "xxx_EDCMeta.h"
#include "xxx_LPGSMeta.h"
#include "xxx_Imagery.h"
#include "gxx_hdf_io.h"
#include "worgen.h"
#include "imageio.h"
#include "tae.h"
#include "display.h"
#include "signal.h"

/* Define the number of resolutions in a Landsat-7 HDF image */
#define NUM_L7_RES 3

/* Define the maximum number of volumes allowed by this application */
#define MAX_L7_VOLS 15

/* Define feet to meters conversion.  1 meter = 3.281 feet */
#define FEET2MTRS 0.304785126

/* Most arrays of info are stored (VIS, THM, PAN) */
enum res_type {L7_VIS, L7_THM, L7_PAN};

/* Define the window units values */
#define L7_DEG 0
#define L7_DMS 1
#define L7_LS  2
#define L7_PRO 3

/* Define the valid product types */
#define L0R 0
#define L1R 1
#define L1G 2
#define L1T 3

/* Define the maximum number of files that can be returned in a
   c_getfiles search */
#define MAXFILE 50

/* Define the maximum number of bands that can be processed for
   Landsat-7.  (1, 2, 3, 4, 5, 61, 62, 7, 8) */
#define MAX_L7_BANDS 9

/* Define the media types */
#define DISKMEDIA 0
#define CDMEDIA   1

#ifndef L7HDF2LAS_C_
/* constant variables for L7HDF2LAS handling */
extern const int lines_per_scan[NUM_L7_RES];
extern const long number_of_samples[NUM_L7_RES];
extern const int resolutions[NUM_L7_RES];
extern const char res_suffixes[NUM_L7_RES][5];

/* information needed for the signal handler to clean up after a SIGKILL
   or SIGTERM signal */
extern long g_num_procd_files;      /* number of files processed for output */
extern char g_procd_files[MAX_L7_BANDS * NUM_L7_RES * MAX_L7_VOLS][CMLEN];
                                    /* names of the files that have been
                                       created by L7HDF2LAS */
extern int g_delflg;                /* flag to represent the global DELFLG flag
                                       (TRUE or FALSE) */
extern int g_mounted;               /* flag to specify if a CD is mounted */
extern long g_unit;                 /* CD unit number */
extern int g_mediaflag;             /* flag to specify DISK or CD */
extern char g_link_files[MAXFILE][CMLEN];
                                    /* name of the link files that were created
                                       for CD processing */
extern int g_num_link_files;        /* number of link files to be deleted */
extern int g_links_created;         /* links were created for file names */

#endif

/* Define the variables to allow a periodic message to be issued */
extern long curent;                 /* current processing line */

/* Function prototypes */
void check_res
(
  int bands[],     /* I: array of bands to search */
  int num_bands,   /* I: number of bands in bands */
  int *same_res    /* O: flag to specify if the bands are of the same
                         resolution (TRUE, FALSE) */
);

int cleanup_files
(
  long *num_procd_files,     /* I/O: number of processed files.  This counter
                                     gets decremented when a file has been
                                     deleted */
  char procd_files[][CMLEN], /* I/O: names of the files that have been created
                                     by this application.  These filenames get
                                     set to NULL after being deleted. */
  int delflg                 /* I: flag to represent the global DELFLG flag
                                   (TRUE or FALSE) */
);

int convert_verify_window
(
  int window_units,        /* I: Units of input window (DEG, DMS, LS, PRO) */
  double window[],         /* I: Image window (If window_units is LS then
                                 sl, ss, nl, ns.  If window_units is DMS or
                                 DEG then UL_lat, UL_long, LR_lat, LR_long.
                                 If window_units is PRO then UL_Y, UL_X,
                                 LR_Y, LR_X.) */
  int num_window,          /* I: number of window values */
  int window_specd,        /* I: flag to specify if the window was specified */
  int product_type,        /* I: type of HDF file this is (L0R, L1R, L1G) */
  long proj_code,          /* I: integer value to specify the projection */
  long zone_code,          /* I: the UTM zone, for UTM projections */
  long datum_code,         /* I: integer value to specify the datum */
  double pixel_size,       /* I: pixel size for current band */
  double proj_parms[15],   /* I: 15 projection parameters for GCTP */
  double ul_corner[2],     /* I: projection coordinates of upper left corner
                                 in the image (x,y) in units of proj_units */
  double ur_corner[2],     /* I: projection coordinates of upper right corner
                                 in the image (x,y) in units of proj_units */
  double ll_corner[2],     /* I: projection coordinates of lower left corner
                                 in the image (x,y) in units of proj_units */
  double lr_corner[2],     /* I: projection coordinates of lower right corner
                                 in the image (x,y) in units of proj_units */
  char proj_units[],       /* I: units of image projection corners */
  long nl,                 /* I: number of lines in the image for this band */
  long ns,                 /* I: number of samples in the image for this band */  long valid_window[4]     /* O: array of valid window values in line/sample
                                 coordinates (SL, SS, NL, NS) */
);

int convert_window_for_each_res
(
  int window_units,       /* I: Units of input window (DEG, DMS, LS, PRO) */
  double window[],        /* I: Image window (If window_units is LS then
                                sl, ss, nl, ns.  If window_units is DMS or
                                DEG then UL_lat, UL_long, LR_lat, LR_long.
                                If window_units is PRO then UL_Y, UL_X,
                                LR_Y, LR_X.) */
  int num_window,         /* I: number of window values */
  int window_specd,       /* I: flag to specify if the window was specified */
  int product_type,       /* I: type of HDF file this is (L0R, L1R, L1G) */
  long proj_code,         /* I: integer value to specify the projection */
  long zone_code,         /* I: the UTM zone, for UTM projections */
  long datum_code,        /* I: integer value to specify the datum */
  double vis_pixel_size,  /* I: pixel size for visible bands */
  double thm_pixel_size,  /* I: pixel size for thermal bands */
  double pan_pixel_size,  /* I: pixel size for pan bands */
  double proj_parms[15],  /* I: 15 projection parameters for GCTP */
  double ul_corner[2],    /* I: projection coordinates of upper left corner
                                in the image (x,y) in units of proj_units */
  double ur_corner[2],    /* I: projection coordinates of upper right corner
                                in the image (x,y) in units of proj_units */
  double ll_corner[2],    /* I: projection coordinates of lower left corner
                                in the image (x,y) in units of proj_units */
  double lr_corner[2],    /* I: projection coordinates of lower right corner
                                in the image (x,y) in units of proj_units */
  char proj_units[],      /* I: units of image projection corners */
  long vis_nl,            /* I: number of lines in image for visible bands */
  long thm_nl,            /* I: number of lines in image for thermal bands */
  long pan_nl,            /* I: number of lines in image for pan bands */
  long vis_ns,            /* I: number of samples in image for visible bands */
  long thm_ns,            /* I: number of samples in image for thermal bands */
  long pan_ns,            /* I: number of samples in image for pan bands */
  int process_vis,        /* I: flag to specify that visible bands are present
                                (TRUE or FALSE) */
  int process_thm,        /* I: flag to specify that thermal bands are present
                                (TRUE or FALSE) */
  int process_pan,        /* I: flag to specify that pan bands are present
                                (TRUE or FALSE) */
  long vis_valid_window[4], /* O: array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for visible bands */  long thm_valid_window[4], /* O: array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for thermal bands */  long pan_valid_window[4]  /* O: array of valid window values in line/sample
                               coordinates (SL, SS, NL, NS) for pan bands */
);

int copy_vol_files
(
  struct PARBLK *vblock,   /* I: TAE parameter block */
  char vol_files[][CMLEN], /* I: list of filenames that matched the _vol file
                              search (Unix format) */
  int nvol_files,          /* I: number of _vol files found in the search */
  char ofile_name[],       /* I: output filename for the merged files */
  long msgtim,             /* I: Elapsed time between display msg for TAE */
  char minmax[4]           /* I: Min max flag for TAE */
);

int create_links
(
  char infile[],             /* I/O: HDF file name (Unix format)
                                Note: infile gets renamed to point to the
                                linked HDF file name */
  char output_dir[],         /* I: directory name of the output file */
  int product_type,          /* I: type of HDF file this is (L0R, L1R, L1G) */
  char link_files[][CMLEN],  /* O: array of filenames created for links */
  int *num_link_files        /* O: number of link files created */
);

int delete_image
(
  char img_file[CMLEN]  /* I: image file to be deleted (with .img extension) */
);

int delete_links
(
  char link_files[][CMLEN],  /* I/O: array of links to be deleted.  These
                                     filenames get set to NULL after being
                                     unlinked. */
  int *num_link_files        /* I/O: number of link files to be deleted.  This
                                     counter gets decremented when a file has
                                     been unlinked. */
);

int find_infilename
(
  char drivename[],       /* I: name of the CD drive in Unix format */
  char filename[],        /* I: file name or partial file name to look for */
  char hdfname[]          /* O: name of the HDF file found on the CD */
);

int get_lcc_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
);

int get_om_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
);

int get_parms
(
  struct PARBLK *vblock,  /* I: pointer to TAE parameter block */
  int max_out,            /* I: maximum number of output files allowed in
                                outfile */
  char comment[][CMLEN],  /* O: array of user comments for the operator */
  int *mediaflag,         /* O: flag to specify DISK or CD */
  int *num_vols,          /* O: number of CD volumes to process */
  char infile[CMLEN],     /* O: input file name (UNIX format) */
  int bands[],            /* O: array of bands to process */
  int *num_bands,         /* O: number of values in bands */
  int *default_bands,     /* O: flag to specify if this application will
                                process the default set of bands (TRUE =
                                process the default bands, FALSE = bands were
                                specified) */
  int nbands[],           /* O: array of NBANDS values */
  int *num_nbands,        /* O: number of values in nbands */
  int *window_units,      /* O: units of input window (DEG, DMS, LS, PRO) */
  double window[],        /* O: Image window (If window_units is LS then
                                sl, ss, nl, ns.  If window_units is DMS or
                                DEG then UL_lat, UL_long, LR_lat, LR_long.
                                If window_units is PRO then UL_Y, UL_X,
                                LR_Y, LR_X.) */
  int *num_window,        /* O: number of window values */
  int *window_specd,      /* O: flag to specify if the window was specified */
  char outfile[][CMLEN],  /* O: array of output file names from OUT (Unix
                                format in lower case) */
  int *num_out,           /* O: number of output file names */
  int *many_to_one,       /* O: flag to specify if all bands will be written
                                to one output file */
  long *odtype,           /* O: data type of output image
                                (0=SAME, EBYTE=BYTE, EWORD=I*2, ELONG=I*4,
                                 EREAL=R*4) */
  int nbands_same_res[],  /* O: array of same_res values, one for each nbands
                                value, each to specify if all the bands in the
                                nbands group are of the same resolution (TRUE,
                                FALSE) */
  int *process_vis,       /* O: flag specifying visible bands will be
                                processed (TRUE or FALSE) */
  int *process_thm,       /* O: flag specifying thermal bands will be
                                processed (TRUE or FALSE) */
  int *process_pan,       /* O: flag specifying pan bands will be processed
                                (TRUE or FALSE) */
  int *delflg,            /* O: flag to represent the global DELFLG flag
                                (TRUE or FALSE) */
  long *msgtim,           /* O: time between display messages */
  char minmax[],          /* O: minmax flag */
  char processing_system[]/* O: processing system for input files, IAS, LPGS
                                NLAPS or NONE provided by user */
);

int get_pc_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
);

int get_ps_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
);

int get_som_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
);

int get_tm_projparms
(
  const xxx_LPGSMetaDataFile_TYPE *LPGSMetaDataFile,
                         /* I: pointer to LPGS metadata file structure */
  long datum_code,       /* I: integer code for the reference datum */
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
);

void get_utm_projparms
(
  double proj_parms[15]  /* O: 15 projection parameters for GCTP */
);

int merge_res_files
(
  struct PARBLK *vblock,    /* I: TAE parameter block */
  char outfile[][CMLEN],    /* I: array of output file names from OUT */
  int num_files,            /* I: number of output files */
  double vis_pixel_size,    /* I: pixel size for visible bands */
  double thm_pixel_size,    /* I: pixel size for thermal bands */
  double pan_pixel_size,    /* I: pixel size for pan bands */
  int process_vis,          /* I: flag to specify that visible bands are
                                  present (TRUE or FALSE) */
  int process_thm,          /* I: flag to specify that thermal bands are
                                  present (TRUE or FALSE) */
  int process_pan,          /* I: flag to specify that pan bands are present
                                  (TRUE or FALSE) */
  long msgtim,              /* I: Elapsed time between display msg for TAE */
  char minmax[4],           /* I: Min max flag for TAE */
  int delflg,               /* I: flag to represent the global DELFLG flag
                                  (TRUE or FALSE) */
  long *num_procd_files,    /* I/O: number of processed files */
  char procd_files[][CMLEN] /* I/O: names of the files that have been created
                                    by this application */
);

int merge_vol_files
(
  struct PARBLK *vblock,    /* I: TAE parameter block */
  char outfile[][CMLEN],    /* I: array of output file names from OUT */
  int num_files,            /* I: number of output files */
  int nbands_same_res[],    /* I: array specifying if each nbands group is
                                  of the same resolution (TRUE, FALSE) */
  long msgtim,              /* I: Elapsed time between display msg for TAE */
  char minmax[4],           /* I: Min max flag for TAE */
  int delflg,               /* I: flag to represent the global DELFLG flag
                                  (TRUE or FALSE) */
  long *num_procd_files,    /* I/O: number of processed files */
  char procd_files[][CMLEN] /* I/O: names of the files that have been created
                                    by this application */
);

int mount_cd
(
  char comment[][CMLEN],  /* I: array of user comments for the operator */
  int vol_num,            /* I: volume number of CD to mount (zero-based) */
  int *mounted,           /* O: flag to specify if a CD is mounted */
  char drivename[],       /* O: name of the CD drive in Unix format */
  long *unit              /* O: tape unit number */
);

void output_info_msg
(
  int product_type,       /* I: type of HDF file this is (L0R, L1R, L1G) */
  int mediaflag,          /* I: flag to specify DISK or CD */
  char infile[],          /* I: input file name (UNIX format) */
  int process_vis,        /* I: flag specifying visible bands will be
                                processed (TRUE or FALSE) */
  int process_thm,        /* I: flag specifying thermal bands will be
                                processed (TRUE or FALSE) */
  int process_pan,        /* I: flag specifying pan bands will be processed
                                (TRUE or FALSE) */
  long vis_nl,            /* I: number of lines in image for visible bands */
  long vis_ns,            /* I: number of samples in image for visible bands */
  long thm_nl,            /* I: number of lines in image for thermal bands */
  long thm_ns,            /* I: number of samples in image for thermal bands */
  long pan_nl,            /* I: number of lines in image for pan bands */
  long pan_ns,            /* I: number of samples in image for pan bands */
  double vis_pixel_size,  /* I: pixel size for visible bands */
  double thm_pixel_size,  /* I: pixel size for thermal bands */
  double pan_pixel_size   /* I: pixel size for pan bands */
);

int readHDF_writeLAS_image
(
  xxx_ImageryInfo_TYPE *l0r_fileptr,
                       /* I: HDF file pointer for L0R product (NULL if not
                             processing L0R imagery) */
  HDFIO *l1rg_fileptr, /* I: HDF file pointer for L1R or L1G product (NULL if
                             not processing L1R or L1G imagery) */
  int product_type,    /* I: type of HDF file this is (L0R, L1R, L1G) */
  int idtype,          /* I: input data type (EBYTE=BYTE, EWORD=I*2) */
  int band,            /* I: band number being read (1,2,3,4,5,61,62,7,8) */
  int las_band,        /* I: current band being written to the LAS image
                             (first band is 1, second band is 2, etc) */
  long window[4],      /* I: window coordinates for spatial subsetting of
                             the input image (SL, SS, NL, NS) */
  long num_total_samps,/* I: total number of samples in the original image */
  struct FDESC *outfileptr
                       /* O: LAS file pointer for output */
);

int read_lpgs_meta
(
  char infile[],           /* I: full path of HDF file name */
  char drivename[],        /* I: CD drive name, if processing from CD */
  char output_dir[],       /* I: directory name of the output file */
  int links_needed,        /* I: flag to indicate links are needed for 
                                 accessing files */
  int meta_bands[9],       /* O: bands listed for this product in the metadata
                                 (this list is sorted in ascending order) */
  int *num_meta_bands,     /* O: number of bands in the metadata */
  char sensor_id[15],      /* O: sensor id listed in the metadata */
  int *product_type,       /* O: type of HDF file this is (L1R, L1G) */
  int *data_type,          /* O: data type of the product */
  char acquisition_date[9], /* O: acquisition date listed in the metadata
                                  (yyyymmdd) */
  double ul_corner[2],     /* O: projection coordinates of upper left corner
                                 in the image (y,x) in units of proj_units */
  double ur_corner[2],     /* O: projection coordinates of upper right corner
                                 in the image (y,x) in units of proj_units */
  double ll_corner[2],     /* O: projection coordinates of lower left corner
                                 in the image (y,x) in units of proj_units */
  double lr_corner[2],     /* O: projection coordinates of lower right corner
                                 in the image (y,x) in units of proj_units */
  char proj_units[],       /* O: units of image projection corners */
  int *proj_code,          /* O: integer value to specify the projection */
  long *zone_code,         /* O: the UTM zone, for UTM projections */
  long *datum_code,        /* O: integer value to specify the datum */
  double proj_parms[15],   /* O: 15 projection parameters for GCTP */
  double *vis_pixel_size,  /* O: pixel size for visible bands */
  double *thm_pixel_size,  /* O: pixel size for thermal bands */
  double *pan_pixel_size,  /* O: pixel size for pan bands */
  long *vis_nl,            /* O: number of lines in image for visible bands */
  long *thm_nl,            /* O: number of lines in image for thermal bands */
  long *pan_nl,            /* O: number of lines in image for pan bands */
  long *vis_ns,            /* O: number of samples in image for visible bands */
  long *thm_ns,            /* O: number of samples in image for thermal bands */
  long *pan_ns,            /* O: number of samples in image for pan bands */
  int *found_lpgs_meta,    /* O: flag to specify that the LPGS metadata was
                                 found for this file (TRUE, FALSE) */
  double *software_version, /* O: IAS or LPGS processing software version */
  char processing_software[],/* O: IAS or LPGS software */
  char processing_system[]  /* O: processing system IAS, LPGS, NLAPS or NONE */
);

int read_product_meta
(
  char infile[],           /* I: full path of HDF file name */
  char drivename[],        /* I: CD drive name, if processing from CD */
  char output_dir[],       /* I: directory name of the output file */
  int links_needed,        /* I: flag to indicate links are needed for 
                                 accessing files */
  int meta_bands[9],       /* O: bands listed for this product in the metadata
                                 (this list is sorted in ascending order) */
  int *num_meta_bands,     /* O: number of bands in the metadata */
  char sensor_id[15],      /* O: sensor id listed in the metadata */
  int *product_type,       /* O: type of HDF file this is (L0R) */
  int *data_type,          /* O: data type of the product */
  char acquisition_date[9], /* O: acquisition date listed in the metadata
                                  (yyyymmdd) */
  double proj_parms[15],   /* O: 15 projection parameters for GCTP */
  double *vis_pixel_size,  /* O: pixel size for visible bands */
  double *thm_pixel_size,  /* O: pixel size for thermal bands */
  double *pan_pixel_size,  /* O: pixel size for pan bands */
  long *vis_nl,            /* O: number of lines in image for visible bands */
  long *thm_nl,            /* O: number of lines in image for thermal bands */
  long *pan_nl,            /* O: number of lines in image for pan bands */
  long *vis_ns,            /* O: number of samples in image for visible bands */
  long *thm_ns,            /* O: number of samples in image for thermal bands */
  long *pan_ns,            /* O: number of samples in image for pan bands */
  int *found_product_meta  /* O: flag to specify that the product metadata was
                                 found for this file (TRUE, FALSE) */
);

int rename_image
(
  char infile[CMLEN],   /* I: filename of file to be renamed (with .img
                              extension) */
  char outfile[CMLEN]   /* I: name of the new file (with .img extension) */
);

int set_bands_from_meta
(
  int meta_bands[],       /* I: array of bands specified in the metadata */
  int num_meta_bands,     /* I: number of bands in meta_bands */
  int bands[],            /* O: array of bands that has been filled */
  int *num_bands,         /* O: number of bands in bands */
  int nbands[],           /* O: array of NBANDS values */
  int *num_nbands,        /* O: number of values in nbands */
  int nbands_same_res[],  /* O: array of same_res values, one for each nbands
                                value, each to specify if all the bands in the
                                nbands group are of the same resolution (TRUE,
                                FALSE) */
  int *process_vis,       /* O: flag specifying visible bands will be
                                processed (TRUE or FALSE) */
  int *process_thm,       /* O: flag specifying thermal bands will be
                                processed (TRUE or FALSE) */
  int *process_pan        /* O: flag specifying pan bands will be processed
                                (TRUE or FALSE) */
);

void signal_cleanup ();

void sort_bands_in_nbands
(
  int nbands[],    /* I: array of NBANDS values */
  int num_nbands,  /* I: number of values in nbands */
  int bands[],     /* I/O: on input, array of bands to sort; on output,
                      array of sorted bands */
  int num_bands    /* I: number of bands in bands */
);

int srch_dir
(
  char infile[],        /* I: HDF file name (Unix format) */
  int bands[],          /* I: array of bands to search for on this CD */
  int num_bands,        /* I: number of bands in bands */
  int product_type,     /* I: type of HDF file this is (L0R, L1R, L1G) */
  int bands_avail[],    /* O: array of bands from the bands array that are
                              available on this CD (in ascending order) */
  int *avail_count,     /* O: number of bands in bands_avail */
  char avail_file_list[MAXFILE][CMLEN],
                        /* O: list of filenames for the bands_avail list */
  int *found            /* O: specifies if any bands were found on this CD */
);

int srch_vol
(
  char srchfile[],    /* I: wildcard filename to search for (Unix format) */
  char vol_files[MAXFILE][CMLEN],
                      /* O: list of filenames that matched the srchfile
                            search (Unix format) */
  int *nvol_files     /* O: number of files in vol_files */
);

int strip_vol_from_string
(
  char ifile_name[],   /* I: input filename with _vol# */
  char ofile_name[]    /* O: output filename without _vol# */
);

int unmount_cd
(
  int *mounted,    /* I/O: flag to specify if a CD is mounted */
  long *unit       /* I/O: CD unit number */
);

int write_ddr
(
  char imgname[CMLEN],   /* I: filename of the output image (Unix format) */
  int bands[],           /* I: array of bands to process */
  int num_bands,         /* I: number of values in bands */
  long data_type,        /* I: data type of the product
                               (0=SAME, EBYTE=BYTE, EWORD=I*2, ELONG=I*4,
                                EREAL=R*4) */
  char proj_units[],     /* I: units of image projection corners */
  long proj_code,        /* I: integer value to specify the projection */
  long zone_code,        /* I: the UTM zone, for UTM projections */
  long datum_code,       /* I: integer value to specify the datum */
  double pixel_size,     /* I: pixel size for current band */
  double proj_parms[15], /* I: 15 projection parameters for GCTP */
  double ul_corner[2],   /* I: projection coordinates of upper left corner
                               in the image (x,y) in units of proj_units */
  double ur_corner[2],   /* I: projection coordinates of upper right corner
                               in the image (y,x) in units of proj_units */
  double ll_corner[2],   /* I: projection coordinates of lower left corner
                               in the image (y,x) in units of proj_units */
  double lr_corner[2],   /* I: projection coordinates of lower right corner
                               in the image (y,x) in units of proj_units */
  char acquisition_date[10],
                         /* I: acquisition date listed in the metadata */
  char sensor_id[8],     /* I: sensor id listed in the metadata */
  long window[4],        /* I: array of window values in line/sample
                               coordinates (SL, SS, NL, NS) */
  long orig_nl,          /* I: original number of lines in the image */
  long orig_ns,          /* I: original number of samples in the image */
  int product_type,      /* I: type of HDF file this is (L0R, L1R, L1G) */
  char minmax[],         /* I: minmax tae global */
  double software_version,/* I: IAS or LPGS processing software version */
  char processing_software[] /* I: IAS or LPGS software */
);

#endif
