/****************************************************************************

NAME: merge_res_files

PURPOSE: Merge all the _<res> files together if the resolutions of these
         files are the same.  The suffixes will be based on which bands are
         in the product, but will be in the order of _vis_thm_pan.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "l7hdf2las.h"
#include "util.h"  /* c_splitname definition */

int merge_res_files
(
  struct PARBLK *vblock,    /* I: TAE parameter block */
  char outfile[][CMLEN],    /* I: array of output file names from OUT */
  int num_files,            /* I: number of output files */
  double vis_pixel_size,    /* I: pixel size for visible bands */
  double thm_pixel_size,    /* I: pixel size for thermal bands */
  double pan_pixel_size,    /* I: pixel size for pan bands */
  int process_vis,          /* I: flag to specify that visible bands are
                                  present (TRUE or FALSE) */
  int process_thm,          /* I: flag to specify that thermal bands are
                                  present (TRUE or FALSE) */
  int process_pan,          /* I: flag to specify that pan bands are present
                                  (TRUE or FALSE) */
  long msgtim,              /* I: Elapsed time between display msg for TAE */
  char minmax[4],           /* I: Min max flag for TAE */
  int delflg,               /* I: flag to represent the global DELFLG flag
                                  (TRUE or FALSE) */
  long *num_procd_files,    /* I/O: number of processed files */
  char procd_files[][CMLEN] /* I/O: names of the files that have been created
                                    by this application */
)

{
  char directory[CMLEN];   /* directory portion of input HDF filename */
  char extension[CMLEN];   /* extension portion of input HDF file name */
  char ifile_name[3][CMLEN]; /* array of filenames to be merged */
  char ofile_name[CMLEN];  /* output filename for the merged files */
  char panfile[CMLEN];     /* pan image filename */
  char local_procd_files[MAX_L7_BANDS * NUM_L7_RES * MAX_L7_VOLS][CMLEN];
                           /* array of filenames that have been created for
                              output from this routine */
  char thmfile[CMLEN];     /* thermal image filename */
  char visfile[CMLEN];     /* visible image filename */
  char root[CMLEN];        /* root portion of input HDF file name */

  int index, index2;       /* looping variables */
  int ofile_index;         /* looping index for the output files */

  int bands[MAX_L7_BANDS][MAXBND+1];
                           /* bands to process - all images.  If all 0s then
                              all bands in all the images will be processed.
                              nbands also needs to be set to 0 for the
                              default. */
  int i;                   /* looping variable */
  int nbands[MAX_L7_BANDS];
                           /* number of bands per input image.  If bands are
                              all 0s and nbands[i] is 0 then all bands in all
                              the images will be processed */
  long num_local_procd_files;
                           /* number of local files processed for output */
  int window[MAX_L7_BANDS][4];
                           /* window for each image (SL, SS, NL, NS)
                              If all values are 0s then the entire
                              image will be processed.  If the
                              images are not of the same size then
                              processing will be stopped. */

  /* Initialized the local processed files
     -------------------------------------*/
  num_local_procd_files = 0;

  /* Initialize the windows, bands, and nbands values to process the
     entire window and all the bands for each image to be copied.
     There will be a maximum of 3 images copied at once.
     ---------------------------------------------------------------*/
  for (index = 0; index < 3; index++)
  {
    /* Set the window values to 0s so that the entire image is copied
       --------------------------------------------------------------*/
    window[index][SL] = 0;
    window[index][SS] = 0;
    window[index][NL] = 0;
    window[index][NS] = 0;

    /* Set the band values to 0s so that all the bands are copied
       ----------------------------------------------------------*/
    for (index2 = 0; index2 < MAXBND; index2++)
    {
      bands[index][index2] = 0;
    }
    nbands[index] = 0;
  }

  /* Loop through all the output filenames
     -------------------------------------*/
  for (ofile_index = 0; ofile_index < num_files; ofile_index++)
  {
    /* Split LAS filename into directory, root, and extension
       ------------------------------------------------------*/
    c_splitname (outfile[ofile_index], directory, root, extension);

    if (process_vis && process_thm && process_pan &&
        (vis_pixel_size == thm_pixel_size) &&
        (vis_pixel_size == pan_pixel_size))
    {
      /* All three resolutions are present and are the same
         --------------------------------------------------*/
      sprintf (visfile, "%s%s_vis.%s", directory, root, extension);
      sprintf (thmfile, "%s%s_thm.%s", directory, root, extension);
      sprintf (panfile, "%s%s_pan.%s", directory, root, extension);
      sprintf (ofile_name, "%s%s_vis_thm_pan.%s", directory, root, extension);

      /* Copy the three images into one
         ------------------------------*/
      strcpy (ifile_name[0], visfile);
      strcpy (ifile_name[1], thmfile);
      strcpy (ifile_name[2], panfile);
      if (c_copy_image (vblock, ifile_name, ofile_name, EXT_COMP, 3,
          window, bands, nbands, 0 /* process all bands */,
          0 /* output to the same data type */, TRUE /* Round the input
          data */, msgtim, minmax) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-copy_image",
          "Error copying the _vol files to one output image");
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      /* Keep track of the new filenames so they can be deleted if an
         error occurs
         ------------------------------------------------------------*/
      strcpy (local_procd_files[num_local_procd_files], ofile_name);
      num_local_procd_files++;

      /* Delete the visfile, thmfile, and panfile
         ----------------------------------------*/
      if (delete_image (visfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the visfile: %s", visfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      if (delete_image (thmfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the thmfile: %s", thmfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      if (delete_image (panfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the panfile: %s", panfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }
    }
    else if (process_vis && process_thm && (vis_pixel_size == thm_pixel_size))
    {
      /* The visible and thermal resolutions are the same
         ------------------------------------------------*/
      sprintf (visfile, "%s%s_vis.%s", directory, root, extension);
      sprintf (thmfile, "%s%s_thm.%s", directory, root, extension);
      sprintf (ofile_name, "%s%s_vis_thm.%s", directory, root, extension);

      /* Copy the two images into one
         ----------------------------*/
      strcpy (ifile_name[0], visfile);
      strcpy (ifile_name[1], thmfile);
      if (c_copy_image (vblock, ifile_name, ofile_name, EXT_COMP, 2,
          window, bands, nbands, 0 /* process all bands */,
          0 /* output to the same data type */, TRUE /* Round the input
          data */, msgtim, minmax) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-copy_image",
          "Error copying the _vol files to one output image");
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      /* Keep track of the new filenames so they can be deleted if an
         error occurs
         ------------------------------------------------------------*/
      strcpy (local_procd_files[num_local_procd_files], ofile_name);
      num_local_procd_files++;

      /* Delete the visfile and thmfile
         ------------------------------*/
      if (delete_image (visfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the visfile: %s", visfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      if (delete_image (thmfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the thmfile: %s", thmfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }
    }
    else if (process_vis && process_pan && (vis_pixel_size == pan_pixel_size))
    {
      /* The visible and pan resolutions are the same
         --------------------------------------------*/
      sprintf (visfile, "%s%s_vis.%s", directory, root, extension);
      sprintf (panfile, "%s%s_pan.%s", directory, root, extension);
      sprintf (ofile_name, "%s%s_vis_pan.%s", directory, root, extension);

      /* Copy the two images into one
         ----------------------------*/
      strcpy (ifile_name[0], visfile);
      strcpy (ifile_name[1], panfile);
      if (c_copy_image (vblock, ifile_name, ofile_name, EXT_COMP, 2,
          window, bands, nbands, 0 /* process all bands */,
          0 /* output to the same data type */, TRUE /* Round the input
          data */, msgtim, minmax) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-copy_image",
          "Error copying the _vol files to one output image");
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      /* Keep track of the new filenames so they can be deleted if an
         error occurs
         ------------------------------------------------------------*/
      strcpy (local_procd_files[num_local_procd_files], ofile_name);
      num_local_procd_files++;

      /* Delete the visfile and panfile
         ------------------------------*/
      if (delete_image (visfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the visfile: %s", visfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      if (delete_image (panfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the panfile: %s", panfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }
    }
    else if (process_thm && process_pan && (thm_pixel_size == pan_pixel_size))
    {
      /* The thermal and pan resolutions are the same
         --------------------------------------------*/
      sprintf (thmfile, "%s%s_thm.%s", directory, root, extension);
      sprintf (panfile, "%s%s_pan.%s", directory, root, extension);
      sprintf (ofile_name, "%s%s_thm_pan.%s", directory, root, extension);

      /* Copy the two images into one
         ----------------------------*/
      strcpy (ifile_name[0], thmfile);
      strcpy (ifile_name[1], panfile);
      if (c_copy_image (vblock, ifile_name, ofile_name, EXT_COMP, 2,
          window, bands, nbands, 0 /* process all bands */,
          0 /* output to the same data type */, TRUE /* Round the input
          data */, msgtim, minmax) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-copy_image",
          "Error copying the _vol files to one output image");
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      /* Keep track of the new filenames so they can be deleted if an
         error occurs
         ------------------------------------------------------------*/
      strcpy (local_procd_files[num_local_procd_files], ofile_name);
      num_local_procd_files++;

      /* Delete the thmfile and panfile
         ------------------------------*/
      if (delete_image (thmfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the thmfile: %s", thmfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      if (delete_image (panfile) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-delete_image",
          "Error deleting the panfile: %s", panfile);
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }
    }
  }  /* for ofile_index */

  /* If files were merged, then return the names of the merged files
     ---------------------------------------------------------------*/
  if (num_local_procd_files > 0)
  {
    /* Replace the procd_files with the local_procd_files since the
       procd_files have been deleted
       ------------------------------------------------------------*/
    for (i = 0; i < num_local_procd_files; i++)
      strcpy (procd_files[i], local_procd_files[i]);

    /* Cleanup the rest of the procd_files
       -----------------------------------*/
    for (; i < *num_procd_files; i++)
      strcpy (procd_files[i], "");
    *num_procd_files = num_local_procd_files;
  }

  return (E_SUCC);
}
