/****************************************************************************

NAME: merge_vol_files

PURPOSE: Merge all the _vol# files together for each of the output files.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "l7hdf2las.h"
#include "util.h"  /* c_splitname definition */

int merge_vol_files
(
  struct PARBLK *vblock,    /* I: TAE parameter block */
  char outfile[][CMLEN],    /* I: array of output file names from OUT */
  int num_files,            /* I: number of output files */
  int nbands_same_res[],    /* I: array specifying if each nbands group is
                                  of the same resolution (TRUE, FALSE) */
  long msgtim,              /* I: Elapsed time between display msg for TAE */
  char minmax[4],           /* I: Min max flag for TAE */
  int delflg,               /* I: flag to represent the global DELFLG flag
                                  (TRUE or FALSE) */
  long *num_procd_files,    /* I/O: number of processed files */
  char procd_files[][CMLEN] /* I/O: names of the files that have been created
                                    by this application */
)

{
  char append_str[5];      /* string to hold the resolution suffixes */
  char directory[CMLEN];   /* directory portion of input HDF filename */
  char extension[CMLEN];   /* extension portion of input HDF file name */
  char local_procd_files[MAX_L7_BANDS * NUM_L7_RES * MAX_L7_VOLS][CMLEN];
                           /* array of filenames that have been created for
                              output from this routine */
  char newname[CMLEN];     /* name of new filename */
  char ofile_name[CMLEN];  /* output filename for the merged files */
  char oldname[CMLEN];     /* name of old filename */
  char root[CMLEN];        /* root portion of input HDF file name */
  char srch_file[CMLEN];   /* base name of files to search for */
  char vol_files[MAXFILE][CMLEN];
                           /* list of filenames that matched the _vol file
                              search (Unix format) */

  long i;                  /* looping variable */
  long num_local_procd_files;
                           /* number of local files processed for output */
  int nvol_files;          /* number of _vol files found in the search */
  int ofile_index;         /* looping index for the output files */
  enum res_type res_index; /* looping index for each possible resolution */

  /* Initialized the local processed files
     -------------------------------------*/
  num_local_procd_files = 0;

  /* Loop through all the output filenames
     -------------------------------------*/
  for (ofile_index = 0; ofile_index < num_files; ofile_index++)
  {
    /* Split LAS filename into directory, root, and extension
       ------------------------------------------------------*/
    c_splitname (outfile[ofile_index], directory, root, extension);

    if (!nbands_same_res[ofile_index])
    {
      /* This nbands group has multiple resolutions so check each possible
         resolution
         -----------------------------------------------------------------*/
      for (res_index = L7_VIS; res_index <= L7_PAN; res_index++)
      {
        /* Search for _vol* files for the current resolution
           -------------------------------------------------*/
        strcpy (append_str, res_suffixes[res_index]);
        sprintf (srch_file, "%s%s%s_vol*.%s", directory, root, append_str,
          extension);
        if (srch_vol (srch_file, vol_files, &nvol_files) != E_SUCC)
        {
          c_lasmsg (FALSE, "l7hdf2las-srch_vol",
            "Error searching for _vol files");
          cleanup_files (&num_local_procd_files, local_procd_files, delflg);
          return (E_FAIL);
        }

        /* Strip the _vol# from the first image in vol_files to come up
           with the output filename
           ------------------------------------------------------------*/
        if (strip_vol_from_string (vol_files[0], ofile_name) != E_SUCC)
        {
          c_lasmsg (FALSE, "l7hdf2las-strip_vol_from_string",
            "Error stripping _vol# from the first vol_file.");
          cleanup_files (&num_local_procd_files, local_procd_files, delflg);
          return (E_FAIL);
        }

        /* If there is only one _vol file then simply rename the image and
           DDR files to the new output filename.  If there is more than one
           _vol file, then all the files must be copied together.
           ----------------------------------------------------------------*/
        if (nvol_files == 1)
        {
          strcpy (oldname, vol_files[0]);
          strcpy (newname, ofile_name);
          if (rename_image (oldname, newname) != E_SUCC)
          {
            c_lasmsg (FALSE, "l7hdf2las-rename_image",
              "Error renaming the _vol file");
            cleanup_files (&num_local_procd_files, local_procd_files, delflg);
            return (E_FAIL);
          }

          /* Keep track of the new filenames so they can be deleted if an
             error occurs
             ------------------------------------------------------------*/
          strcpy (local_procd_files[num_local_procd_files], newname);
          num_local_procd_files++;
        }
        else if (nvol_files > 1)
        {
          /* Copy all of the vol_files to hosout using the entire image
             window and all the bands
             ----------------------------------------------------------*/
          if (copy_vol_files (vblock, vol_files, nvol_files, ofile_name,
            msgtim, minmax) != E_SUCC)
          {
            c_lasmsg (FALSE, "l7hdf2las-copy_vol_files",
              "Error copying the _vol files to one output image");
            cleanup_files (&num_local_procd_files, local_procd_files, delflg);
            return (E_FAIL);
          }

          /* Keep track of the new filenames so they can be deleted if an
             error occurs
             ------------------------------------------------------------*/
          strcpy (local_procd_files[num_local_procd_files], ofile_name);
          num_local_procd_files++;
        }
      }  /* for res_index */
    }  /* if !nbands_same_res[] */
    else
    {
      /* This nbands group does not have multiple resolutions therefore the
         resolution suffixes won't be attached
         ------------------------------------------------------------------*/
      sprintf (srch_file, "%s%s_vol*.img", directory, root);
      if (srch_vol (srch_file, vol_files, &nvol_files) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-srch_vol",
          "Error searching for _vol files");
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      /* Strip the _vol# from the first image in vol_files to come up
         with the output filename
         ------------------------------------------------------------*/
      if (strip_vol_from_string (vol_files[0], ofile_name) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-strip_vol_from_string",
          "Error stripping _vol# from the first vol_file.");
        cleanup_files (&num_local_procd_files, local_procd_files, delflg);
        return (E_FAIL);
      }

      /* If there is only one _vol file then simply rename the image and
         DDR files to the new output filename.  If there is more than one
         _vol file, then all the files must be copied together.
         ----------------------------------------------------------------*/
      if (nvol_files == 1)
      {
        strcpy (oldname, vol_files[0]);
        strcpy (newname, ofile_name);
        if (rename_image (oldname, newname) != E_SUCC)
        {
          c_lasmsg (FALSE, "l7hdf2las-rename_image",
            "Error renaming the _vol file");
          cleanup_files (&num_local_procd_files, local_procd_files, delflg);
          return (E_FAIL);
        }

        /* Keep track of the new filenames so they can be deleted if an
           error occurs
           ------------------------------------------------------------*/
        strcpy (local_procd_files[num_local_procd_files], newname);
        num_local_procd_files++;
      }
      else if (nvol_files > 1)
      {
        /* Copy all of the vol_files to hosout using the entire image
           window and all the bands
           ----------------------------------------------------------*/
        if (copy_vol_files (vblock, vol_files, nvol_files, ofile_name,
          msgtim, minmax) != E_SUCC)
        {
          c_lasmsg (FALSE, "l7hdf2las-copy_vol_files",
            "Error copying the _vol files to one output image");
          cleanup_files (&num_local_procd_files, local_procd_files, delflg);
          return (E_FAIL);
        }

        /* Keep track of the new filenames so they can be deleted if an
           error occurs
           ------------------------------------------------------------*/
        strcpy (local_procd_files[num_local_procd_files], ofile_name);
        num_local_procd_files++;
      }
    }  /* else */
  }  /* for ofile_index */

  /* If files were merged, then return the names of the merged files
     ---------------------------------------------------------------*/
  if (num_local_procd_files > 0)
  {
    /* Replace the procd_files with the local_procd_files since the
       procd_files have been deleted
       ------------------------------------------------------------*/
    for (i = 0; i < num_local_procd_files; i++)
      strcpy (procd_files[i], local_procd_files[i]);

    /* Cleanup the rest of the procd_files
       -----------------------------------*/
    for (; i < *num_procd_files; i++)
      strcpy (procd_files[i], "");
    *num_procd_files = num_local_procd_files;
  }

  return (E_SUCC);
}
