/****************************************************************************

NAME: output_info_msg

PURPOSE: Output informational messages about the type of imagery being
  processed, the media type, the number of lines/samples, and the pixel size.

RETURN VALUE:
None

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    12/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"

void output_info_msg
(
  int product_type,       /* I: type of HDF file this is (L0R, L1R, L1G, L1T) */
  int mediaflag,          /* I: flag to specify DISK or CD */
  char infile[],          /* I: input file name (UNIX format) */
  int process_vis,        /* I: flag specifying visible bands will be
                                processed (TRUE or FALSE) */
  int process_thm,        /* I: flag specifying thermal bands will be
                                processed (TRUE or FALSE) */
  int process_pan,        /* I: flag specifying pan bands will be processed
                                (TRUE or FALSE) */
  long vis_nl,            /* I: number of lines in image for visible bands */
  long vis_ns,            /* I: number of samples in image for visible bands */
  long thm_nl,            /* I: number of lines in image for thermal bands */
  long thm_ns,            /* I: number of samples in image for thermal bands */
  long pan_nl,            /* I: number of lines in image for pan bands */
  long pan_ns,            /* I: number of samples in image for pan bands */
  double vis_pixel_size,  /* I: pixel size for visible bands */
  double thm_pixel_size,  /* I: pixel size for thermal bands */
  double pan_pixel_size   /* I: pixel size for pan bands */
)

{
  char infomsg[CMLEN];      /* informational message string */
  char infomsg2[CMLEN];     /* second informational message string */

  /* Provide some informational messages before processing
     -----------------------------------------------------*/
  switch (product_type)
  {
    case L0R:
      sprintf (infomsg, "Processing L7 HDF L0R imagery on ");
      break;

    case L1R:
      sprintf (infomsg, "Processing L7 HDF L1R imagery on ");
      break;

    case L1G:
      sprintf (infomsg, "Processing L7 HDF L1G imagery on ");
      break;

    case L1T:
      sprintf (infomsg, "Processing L7 HDF L1T imagery on ");
      break;
  }

  if (mediaflag == CDMEDIA)
    sprintf (infomsg2, "CD: %s", infile);
  else
    sprintf (infomsg2, "DISK: %s", infile);

  strcat (infomsg, infomsg2);
  c_lasmsg (FALSE, "informational", infomsg);

  /* Print the total lines
     ---------------------*/
  sprintf (infomsg, "Total lines (vis, thm, pan): ");
  if (process_vis)
    sprintf (infomsg2, "%ld, ", vis_nl);
  else
    sprintf (infomsg2, "N/A, ");
  strcat (infomsg, infomsg2);

  if (process_thm)
    sprintf (infomsg2, "%ld, ", thm_nl);
  else
    sprintf (infomsg2, "N/A, ");
  strcat (infomsg, infomsg2);

  if (process_pan)
    sprintf (infomsg2, "%ld", pan_nl);
  else
    sprintf (infomsg2, "N/A");
  strcat (infomsg, infomsg2);
  c_lasmsg (FALSE, "informational", infomsg);

  /* Print the total samples
     -----------------------*/
  sprintf (infomsg, "Total samples (vis, thm, pan): ");
  if (process_vis)
    sprintf (infomsg2, "%ld, ", vis_ns);
  else
    sprintf (infomsg2, "N/A, ");
  strcat (infomsg, infomsg2);

  if (process_thm)
    sprintf (infomsg2, "%ld, ", thm_ns);
  else
    sprintf (infomsg2, "N/A, ");
  strcat (infomsg, infomsg2);

  if (process_pan)
    sprintf (infomsg2, "%ld", pan_ns);
  else
    sprintf (infomsg2, "N/A");
  strcat (infomsg, infomsg2);
  c_lasmsg (FALSE, "informational", infomsg);

  /* Print the pixel sizes
     ---------------------*/
  sprintf (infomsg, "Pixel sizes (vis, thm, pan): ");
  if (process_vis)
    sprintf (infomsg2, "%lf, ", vis_pixel_size);
  else
    sprintf (infomsg2, "N/A, ");
  strcat (infomsg, infomsg2);

  if (process_thm)
    sprintf (infomsg2, "%lf, ", thm_pixel_size);
  else
    sprintf (infomsg2, "N/A, ");
  strcat (infomsg, infomsg2);

  if (process_pan)
    sprintf (infomsg2, "%lf", pan_pixel_size);
  else
    sprintf (infomsg2, "N/A");
  strcat (infomsg, infomsg2);
  c_lasmsg (FALSE, "informational", infomsg);
}
