/******************************************************************************

NAME: readHDF_writeLAS_image

PURPOSE: Read the input HDF image and output the image contents to the LAS
         image.  An input window will be used for spatial subsetting.  The
         input data type will be converted to the output data type.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT:    LAS

NOTES:
  The LAS output file must have been opened with the data conversion option,
  so that the data is automatically converted from idtype to odtype when
  outputting the data.

******************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "imageio.h"   /* SL, SS, NL, NS definitions */

/* Define the number of scans to be read at one time */
#define NUM_SCANS 10

int readHDF_writeLAS_image
(
  xxx_ImageryInfo_TYPE *l0r_fileptr,
                       /* I: HDF file pointer for L0R product (NULL if not
                             processing L0R imagery) */
  HDFIO *l1rg_fileptr, /* I: HDF file pointer for L1R or L1G product (NULL if
                             not processing L1R or L1G imagery) */
  int product_type,    /* I: type of HDF file this is (L0R, L1R, L1G, L1T) */
  int idtype,          /* I: input data type (EBYTE=BYTE, EWORD=I*2) */
  int band,            /* I: band number being read (1,2,3,4,5,61,62,7,8) */
  int las_band,        /* I: current band being written to the LAS image
                             (first band is 1, second band is 2, etc) */
  long window[4],      /* I: window coordinates for spatial subsetting of
                             the input image (SL, SS, NL, NS) */
  long num_total_samps,/* I: total number of samples in the original image */
  struct FDESC *outfileptr
                       /* IO: LAS file pointer for output */
)

{
  char errmsg[CMLEN];        /* error message */
  int input_size;            /* size of the input data type */
  long long_las_band;        /* long copy of the las_band */
  int lines_per_scan;        /* number of lines per scan for this band */
  long line_index;           /* index for the current line in the image */
  long num_lines;            /* number of lines to process */
  long num_samps;            /* number of samples to process */
  int32 read_offset;         /* offset of the next read position */
  long start_line;           /* starting line to process */
  long start_sample;         /* starting sample to process */
  long sub_line_index;       /* subwindow line index */
  void *scan_buffer = NULL;  /* scan buffer for all input data types */
  unsigned char *byte_scan_buffer_ptr = NULL;
                             /* scan buffer pointer for subwindowing */
  short *word_scan_buffer_ptr = NULL;
                             /* scan buffer pointer for subwindowing */
  BandNumber enumband=berror;/* enumerated band number for gxx_read calls */ 

  /* Define the number of lines per scan for this band
     -------------------------------------------------*/
  if ((band >= 1 && band <= 5) || band == 7)
    lines_per_scan = 16;
  else if (band == 61 || band == 62)
    lines_per_scan = 8;
  else if (band == 8)
    lines_per_scan = 32;
  else
  {
    c_lasmsg (FALSE, "l7hdf2las-band", "Invalid band number");
    return (E_FAIL);
  }

  /* Set up the starting sample and number of samples since these
     will be constant for every scan
     ------------------------------------------------------------*/
  start_sample = window[SS] - 1;  /* zero-based */
  num_samps = window[NS];

  /* Allocate space for the scan buffer
     ----------------------------------*/
  switch (idtype)
  {
    case EBYTE: input_size = EBYTESIZE; break;
    case EWORD: input_size = EWORDSIZE; break;
    case ELONG: input_size = ELONGSIZE; break;
    case EREAL: input_size = EREALSIZE; break;
  }

  scan_buffer = malloc (input_size * num_samps * (NUM_SCANS*lines_per_scan));
  if (scan_buffer == NULL)
  {
    c_lasmsg (FALSE, "l7hdf2las-alloc",
      "Error allocating space for scan buffer");
    return (E_FAIL);
  }

  /* If dealing with an L1R, L1G, or L1T image, then the band has
     to be an enumerated band.
     ------------------------------------------------------------*/
  if ((product_type == L1R) || (product_type == L1G) || (product_type == L1T))
  {
    switch (band)
    {
      case 1: enumband = b1; break;
      case 2: enumband = b2; break;
      case 3: enumband = b3; break;
      case 4: enumband = b4; break;
      case 5: enumband = b5; break;
      case 61: enumband = b6l; break;
      case 62: enumband = b6h; break;
      case 7: enumband = b7; break;
      case 8: enumband = b8; break;
    }
  }

  /* Process NUM_SCANS scans at a time
     --------------------------*/
  long_las_band = (long) las_band;
  read_offset = window[SL] - 1;  /* zero-based */
  for (line_index = 0; line_index < window[NL];
       line_index += (NUM_SCANS * lines_per_scan))
  {
    /* update the current line displayed */
    curent = line_index;

    /* Determine the number of lines to be read.  We will process NUM_SCANS
       scans at a time, except for the cleanup loop.
       -------------------------------------------------------------------*/
    start_line = line_index + 1;  /* one-based */
    if (window[NL] - line_index < NUM_SCANS * lines_per_scan)
    {
      /* Cleanup loop.  Processing fewer than NUM_SCANS scans
         ---------------------------------------------*/
      num_lines = window[NL] - line_index;
    }
    else
    {
      /* Process NUM_SCANS scans
         ----------------*/
      num_lines = NUM_SCANS * lines_per_scan;
    }

    /* If a subwindow has been specified, then the imagery will need to be
       read one line at a time.  Otherwise, the whole chunk of NUM_SCANS scans
       can be read.  Neither xxx_ReadImagery nor gxx_read_1g_image support
       subwindowing.
       --------------------------------------------------------------------*/
    if (window[NS] == num_total_samps)
    { /* Not subwindowing so read the whole chunk of data */
      /* Read the input data.  L0R, L1G, and L1T will be BYTE data and L1R will
         be I*2 data.  Each product type has a different HDF format and
         therefore will require separate handling.
         ---------------------------------------------------------------------*/
      if (product_type == L0R)
      {
        if (xxx_ReadImagery (l0r_fileptr, scan_buffer, num_lines,
            &read_offset, start_sample, num_samps, errmsg) != XXXSUCCESS)
        {
          c_lasmsg (FALSE, "l7hdf2las-readimagery",
            "Error reading the input L7 HDF image (L0R)");
          free (scan_buffer);
          return (E_FAIL);
        } 
      }
      else
      {
        /* gxx_read_1g_image will handle L1R, L1G, and L1T imagery
           -------------------------------------------------------*/
        if (gxx_read_1g_image (l1rg_fileptr, enumband, line_index, 0,
            num_lines, num_samps, 0, scan_buffer) == ERROR)
        {
          c_lasmsg (FALSE, "l7hdf2las-readimagery",
            "Error reading the input L7 HDF image (L1R, L1G, or L1T)");
          free (scan_buffer);
          return (E_FAIL);
        }
      }
    }
    else
    { /* Subwindowing so read one line at a time */
      byte_scan_buffer_ptr = scan_buffer;
      word_scan_buffer_ptr = scan_buffer;
      for (sub_line_index = 0; sub_line_index < num_lines; sub_line_index++)
      {
        /* Read the input data. L0R, L1G, and L1T will be BYTE data and
           L1R will be I*2 data. Each product type has a different HDF
           format and therefore will require separate handling.
           ------------------------------------------------------------*/
        if (product_type == L0R)
        {
          if (xxx_ReadImagery (l0r_fileptr, byte_scan_buffer_ptr, 1,
              &read_offset, start_sample, num_samps, errmsg) != XXXSUCCESS)
          {
            c_lasmsg (FALSE, "l7hdf2las-readimagery",
              "Error reading the input L7 HDF image (L0R)");
            free (scan_buffer);
            return (E_FAIL);
          }

          /* Increment the scan buffer pointer by the number of samples
             just read
             ----------------------------------------------------------*/
          byte_scan_buffer_ptr += num_samps;
        }
        else if (product_type == L1R)
        {
          /* gxx_read_1g_image will handle L0R, L1G, and L1T
             imagery but L1R products are I*2 data
             -----------------------------------------------*/
          if (gxx_read_1g_image (l1rg_fileptr, enumband,
              window[SL] + line_index + sub_line_index, start_sample, 1,
              num_samps, 0, word_scan_buffer_ptr) == ERROR)
          {
            c_lasmsg (FALSE, "l7hdf2las-readimagery",
              "Error reading the input L7 HDF image (L1R)");
            free (scan_buffer);
            return (E_FAIL);
          }

          /* Increment the scan buffer pointer by the number of samples
             just read
             ----------------------------------------------------------*/
          word_scan_buffer_ptr += num_samps;
        }
        else /* L1G and L1T */
        {
          /* gxx_read_1g_image will handle L0R, L1G, and L1T
             imagery but L1R products are I*2 data
             -----------------------------------------------*/
          if (gxx_read_1g_image (l1rg_fileptr, enumband,
              window[SL] + line_index + sub_line_index, start_sample, 1,
              num_samps, 0, byte_scan_buffer_ptr) == ERROR)
          {
            c_lasmsg (FALSE, "l7hdf2las-readimagery",
              "Error reading the input L7 HDF image (L1G/L1T)");
            free (scan_buffer);
            return (E_FAIL);
          }

          /* Increment the scan buffer pointer by the number of samples
             just read
             ----------------------------------------------------------*/
          byte_scan_buffer_ptr += num_samps;
        }
      }  /* end for sub_line_index */
    }

    /* Output the data to the LAS format.  (Note: data conversion is done
       automatically since the eopenr opened the file with automatic data
       conversion.)
       ------------------------------------------------------------------*/
    if (c_ewrite (&outfileptr, &long_las_band, &start_line, scan_buffer,
        &num_lines) == E_FAIL)
    {
      c_lasmsg (FALSE, "l7hdf2las-ewrite", "Error writing to the LAS image");
      free (scan_buffer);
      return (E_FAIL);
    }
  }  /* end for */

  /* update the current line displayed */
  curent = window[NL];

  free (scan_buffer);
  return (E_SUCC);
}
