/******************************************************************************

NAME: read_lpgs_meta

PURPOSE: Read the LPGS metadata for an L1R or L1G product.  The bands,
         projection code, zone code, datum code, projection parameter specific
         fields, pixel size, sensor type, and capture date need to be read
         from this file.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT:    LAS

NOTES:
  For an L1R product the projection information will not be available.
******************************************************************************/

#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include "l7hdf2las.h"
#include "geompak.h"
#include "util.h"  /* c_splitname definition */

int read_lpgs_meta
(
  char infile[],           /* I: full path of HDF file name */
  char drivename[],        /* I: CD drive name, if processing from CD */
  char output_dir[],       /* I: directory name of the output file */
  int links_needed,        /* I: flag to indicate links are needed for 
                                 accessing files */
  int meta_bands[9],       /* O: bands listed for this product in the metadata
                                 (this list is sorted in ascending order) */
  int *num_meta_bands,     /* O: number of bands in the metadata */
  char sensor_id[15],      /* O: sensor id listed in the metadata */
  int *product_type,       /* O: type of HDF file this is (L1R, L1G, L1T) */
  int *data_type,          /* O: data type of the product */
  char acquisition_date[9], /* O: acquisition date listed in the metadata
                                  (yyyymmdd) */
  double ul_corner[2],     /* O: projection coordinates of upper left corner
                                 in the image (y,x) in units of proj_units */
  double ur_corner[2],     /* O: projection coordinates of upper right corner
                                 in the image (y,x) in units of proj_units */
  double ll_corner[2],     /* O: projection coordinates of lower left corner
                                 in the image (y,x) in units of proj_units */
  double lr_corner[2],     /* O: projection coordinates of lower right corner
                                 in the image (y,x) in units of proj_units */
  char proj_units[],       /* O: units of image projection corners */
  int *proj_code,          /* O: integer value to specify the projection */
  long *zone_code,         /* O: the UTM zone, for UTM projections */
  long *datum_code,        /* O: integer value to specify the datum */
  double proj_parms[15],   /* O: 15 projection parameters for GCTP */
  double *vis_pixel_size,  /* O: pixel size for visible bands */
  double *thm_pixel_size,  /* O: pixel size for thermal bands */
  double *pan_pixel_size,  /* O: pixel size for pan bands */
  long *vis_nl,            /* O: number of lines in image for visible bands */
  long *thm_nl,            /* O: number of lines in image for thermal bands */
  long *pan_nl,            /* O: number of lines in image for pan bands */
  long *vis_ns,            /* O: number of samples in image for visible bands */
  long *thm_ns,            /* O: number of samples in image for thermal bands */
  long *pan_ns,            /* O: number of samples in image for pan bands */
  int *found_lpgs_meta,    /* O: flag to specify that the LPGS metadata was
                                 found for this file (TRUE, FALSE) */
  double *software_version, /* O: IAS or LPGS processing software version */
  char processing_software[],/* O: IAS or LPGS software */
  char processing_system[]  /* O: processing system IAS, LPGS, NLAPS or NONE */
)

{
    char acq_date[11];      /* acquisition date listed in the metadata
                               (YYYY-MM-DD) */
    char bands[10];         /* string to represent the bands present
                               (They are listed as 123456678.  A '-' is used for
                               a position holder for absent bands.) */
    char cd_file[CMLEN];    /* full path of metadata filename on CD */
    char datum_type[8];     /* datum string type */
    char directory[CMLEN];  /* directory portion of input HDF filename */
    char errmsg[CMLEN];     /* error message */
    char extension[CMLEN];  /* extension portion of input HDF file name */
    char hdf_file[CMLEN];   /* full path of HDF filename */
    char hdfname[CMLEN];    /* name of HDF file */
    char local_file[CMLEN]; /* local path of metadata link filename */
    char metaname[CMLEN];   /* metadata filename */
    char proj_name[4];      /* map projection */
    char prod_type[4];      /* product type (L0R, L1R, L1G) */
    char root[CMLEN];       /* root portion of input HDF file name */
    char software_ver_string[30];/* IAS or LPGS processing software version */
    char software_ver_tmp[6];/* IAS or LPGS numeric version number */ 
    char *tmp_ptr;          /* pointer to the last '.' in the HDF filename */
    char tmpstr[2];         /* string to hold the current band number */
    int i;                  /* looping variable */
    int pos;                /* position of '-' character in a string */
    long strsize;           /* string length of the string */

    xxx_LPGSMetaDataFile_TYPE *metadata_struct = NULL;
    /* LPGS metadata file structure */
    xxx_LPGSProductMetaData_TYPE *product_metadata = NULL;
    /* LPGS metadata structure */
    xxx_ProjectionParameters_TYPE *projparms_struct = NULL;
    char local_drivename[CMLEN]; /* local drive name to support CDs mounted 
                                    without a mount request */
    char *ptr = NULL;       /* pointer to first occurance of "-" substring */

    strcpy(local_drivename,drivename);
    strcpy(local_file,"");

    if (strcmp (local_drivename, "") || links_needed)
    {
        /* Split input HDF filename into directory, root, and extension
           ------------------------------------------------------------*/
        c_splitname (infile, directory, root, extension);

        /* handle CD's that are local to the machine */
        if (strcmp(local_drivename,"")==0)
            strcpy(local_drivename,directory);

        /* For CD processing we need to make sure we are dealing with upper-case
           filenames when processing the metadata.  Thus, an upper-case link to
           the metadata file on CD needs to be made. 
           -------------------------------------------------------------------*/
        sprintf (hdfname, "%s.%s", root, extension);
        strsize = (long) strlen (hdfname);
        c_low2up (hdfname, &strsize);

        /* Replace the _HDF with _MTL for the linked metadata filename
           -----------------------------------------------------------*/
        tmp_ptr = strrchr (hdfname, '_');  
        if (tmp_ptr != NULL)
        {
            if (!strncmp (tmp_ptr, "_HDF", 4))
                strncpy (tmp_ptr, "_MTL", 4);
        }
        else
        {
            c_lasmsg (FALSE, "l7hdf2las-lpgsmeta", "Error finding _HDF");
            *found_lpgs_meta = FALSE;
            return (E_FAIL);
        }

        /* Replace the _HDF or _hdf with _MTL or _mtl for the true metadata
           filename.
           ----------------------------------------------------------------*/
        sprintf (metaname, "%s.%s", root, extension);
        tmp_ptr = strrchr (metaname, '_');
        if (tmp_ptr != NULL)
        {
            if (!strncmp (tmp_ptr, "_HDF", 4))
                strncpy (tmp_ptr, "_MTL", 4);
            else if (!strncmp (tmp_ptr, "_hdf", 4))
                strncpy (tmp_ptr, "_mtl", 4);
        }
        else
        {
            c_lasmsg (FALSE, "l7hdf2las-lpgsmeta",
                    "Error finding _HDF/_hdf in metadata filename");
            *found_lpgs_meta = FALSE;
            return (E_FAIL);
        }

        /* Does the CD file that we plan to link to exist?  This may be an L0R
           product instead of an L1R/L1G product.
           -------------------------------------------------------------------*/
        sprintf (local_file, "%s%s", output_dir, hdfname);
        sprintf (cd_file, "%s%s", local_drivename, metaname);
        if (access (cd_file, R_OK /* read permissions */) != 0)
        {
            /* cd_file does not exist so exit this read metadata and look for 
               the L0R metadata
               ---------------------------------------------------------------*/
            *found_lpgs_meta = FALSE;
            return (E_FAIL);
        }

        /* Make the link (EEXIST is returned in errno if the file already 
           exists)
           -------------------------------------------------------------------*/
        if (symlink (cd_file, local_file) != 0)
        {
            c_lasmsg (FALSE, "l7hdf2las-link", "Error linking %s to %s", 
                    cd_file, local_file);
            if (errno == EEXIST)
                c_lasmsg (FALSE, "l7hdf2las-exists", "%s already exists", 
                          local_file);
            *found_lpgs_meta = FALSE;
            return (E_FAIL);
        }

        /* Copy the local filename to hdf_file
           -----------------------------------*/
        strcpy (hdf_file, local_file);

        /* Turn it back to an HDF filename
           -------------------------------*/
        tmp_ptr = strrchr (hdf_file, '_');
        if (tmp_ptr != NULL)
        {
            if (!strncmp (tmp_ptr, "_MTL", 4))
                strncpy (tmp_ptr, "_HDF", 4);
        }
        else
        {
            c_lasmsg (FALSE, "l7hdf2las-lpgsmeta",
                    "Error finding _MTL in metadata filename");
            *found_lpgs_meta = FALSE;
            if (unlink (local_file) != 0)
                c_lasmsg (FALSE, "l7hdf2las-unlink", 
                         "Error unlinking %s", local_file);
            return (E_FAIL);
        }
    }
    else
    {
        /* Just use the infile name
           ------------------------*/
        strcpy (hdf_file, infile);
    }

    /* Read the LPGS metadata file (allocates space for the
       xxx_LPGSMetaDataFile_TYPE structure)
       ----------------------------------------------------*/
    metadata_struct = xxx_ReadLPGSMetadata (hdf_file, errmsg);
    if (metadata_struct == NULL)
    {
        *found_lpgs_meta = FALSE;
        if (strcmp (local_file, ""))
        {
            if (unlink (local_file) != 0)
                c_lasmsg (FALSE, "l7hdf2las-unlink", 
                          "Error unlinking %s", local_file);
        }

        /* Use this message for tracking problems with reading the metadata.
           It has been left commented out since the L1R/L1G metadata may not be
           here and we are instead looking for L0R metadata.
         */
        c_lasmsg (FALSE, "l7hdf2las-ReadLPGSMetadata", errmsg);
        return (E_FAIL);
    }
    *found_lpgs_meta = TRUE;
    product_metadata = &metadata_struct->ProductMetaData;
    projparms_struct = &metadata_struct->ProjectionParameters;

    /* Unlink the local file
       ---------------------*/
    if (strcmp (local_file, ""))
    {
        if (unlink (local_file) != 0)
        {
            c_lasmsg (FALSE, "l7hdf2las-unlink", "Error unlinking %s",
                    local_file);
            *found_lpgs_meta = FALSE;
            free (metadata_struct);
            return (E_FAIL);
        }
    }

    /* Get the bands.  They are listed as 123456678.  A '-' is used for a
       position holder for absent bands.  The first 6 is 61 and the second 6
       is 62.  123---678 represents bands 1,2,3,62,7,8.
       ---------------------------------------------------------------------*/
    strncpy (bands, product_metadata->Band_combination, 9);
    bands[9] = '\0';
    *num_meta_bands = 0;
    for (i = 0; i < 9; i++)
    {
        if (isdigit (bands[i]))
        {
            if (i == 5)
                meta_bands[*num_meta_bands] = 61;
            else if (i == 6)
                meta_bands[*num_meta_bands] = 62;
            else
            {
                strncpy (tmpstr, &bands[i], 1);
                tmpstr[1] = '\0';
                meta_bands[*num_meta_bands] = atoi (tmpstr);
            }
            (*num_meta_bands)++;
        }
    }

    /* Get the sensor ID and product type
       ----------------------------------*/
    strncpy (sensor_id, product_metadata->SensorId, 14);
    sensor_id[14] = '\0';
    strncpy (prod_type, product_metadata->ProductType, 3);
    prod_type[3] = '\0';
    if (!strncasecmp (prod_type, "L1R", 3))
        *product_type = L1R;
    else if (!strncasecmp (prod_type, "L1G", 3))
        *product_type = L1G;
    else if (!strncasecmp (prod_type, "L1T", 3))
        *product_type = L1T;
    else
    {
        c_lasmsg (FALSE, "l7hdf2las-lpgsmeta",
                "Product type can only be L1R or L1G for this image");
        free (metadata_struct);
        return (E_FAIL);
    }

    /* The product acquisition date is in the form of YYYY-MM-DD.  The format
       for this in the LAS DDR is yyyymmdd, so the '-'s need to be stripped.
       ----------------------------------------------------------------------*/
    strncpy (acq_date, product_metadata->AcquisitionDate, 10);
    acq_date[10] = '\0';
    strncpy (acquisition_date, acq_date, 4);
    acquisition_date[4] = '\0';
    strncat (acquisition_date, &acq_date[5], 2);
    strncat (acquisition_date, &acq_date[8], 2);

    /* Read the number of scans and determine the number of lines in each
       resolution.  16 lines/scan for visible, 8 lines/scan for thermal, and
       32 lines/scan for pan bands.
       ---------------------------------------------------------------------*/
    *vis_nl = product_metadata->ProductLinesREF;
    *thm_nl = product_metadata->ProductLinesTHM;
    *pan_nl = product_metadata->ProductLinesPAN;

    /* Set the number of samples in each resolution according to the L1R DFCB
       ----------------------------------------------------------------------*/
    *vis_ns = product_metadata->ProductSamplesREF;
    *thm_ns = product_metadata->ProductSamplesTHM;
    *pan_ns = product_metadata->ProductSamplesPAN;

    /* Get the projection information (only available for L1G & L1T products)
       ----------------------------------------------------------------------*/
    if (*product_type == L1G || *product_type == L1T)
    {
        strncpy (proj_name, projparms_struct->MapProjection, 3);
        proj_name[3] = '\0';
        *proj_code = c_projstr2num (proj_name);
        if (*proj_code == -1)
        {
            c_lasmsg (FALSE, "l7hdf2las-projname", "Invalid projection name");
            free (metadata_struct);
            return (E_FAIL);
        } 

        if (!strcasecmp (proj_name, "UTM"))
            *zone_code = metadata_struct->Parameters.u.utm.zone_number;

        strncpy (datum_type, projparms_struct->ReferenceDatum, 7);
        datum_type[7] = '\0';
        if (c_dtmnum (datum_type, datum_code) != E_SUCC)
        {
            c_lasmsg (FALSE, "l7hdf2las-datumname", "Invalid datum name");
            free (metadata_struct);
            return (E_FAIL);
        }

        /* Get the pixel size for each resolution
           --------------------------------------*/
        *vis_pixel_size = projparms_struct->GridCellSizeRef;
        *thm_pixel_size = projparms_struct->GridCellSizeThm;
        *pan_pixel_size = projparms_struct->GridCellSizePan;

        /* The data type for L1G products is always BYTE
           ---------------------------------------------*/
        *data_type = EBYTE;

        /* Read the projection parameter fields based on the current projection
           -------------------------------------------------------------------*/
        if (!strcasecmp (proj_name, "SOM"))
      {
            if (get_som_projparms (metadata_struct, *datum_code, proj_parms)
                    != E_SUCC)
            {
                c_lasmsg (FALSE, "l7hdf2las-projparms",
                        "Error getting SOM projection parameters");
                free (metadata_struct);
                return (E_FAIL);
            }

            /* Until release 6.1 LPGS and IAS SOM projection metadata corner 
               coordinates were modified to be in the same orientation as all 
               other projections (Y increasing upward and X increasing to the
               right.) NLAPS products were not modified which correctly follows
               the "Map Projections: A Working Manual".  Version 6.1 and on 
               will no longer be modified. In order to support these corner 
               differences we need to get the processing software and version.
               CR 6868. 
               --------------------------------------------------------------*/
            strncpy (software_ver_string,
                    product_metadata->ProcessingSoftware, 30);

            if ((pos = strcspn(software_ver_string, "_")) == NULL)
            {
                if ((strcmp (processing_system, "NONE")) == 0)
                {
                    /* No '_' and PTYPE was not specified. We don't 
                       know if this is a IAS, LPGS or NLAPS product. The user 
                       is required to enter the product type parameter.  */
                    c_lasmsg (FALSE, "l7hdf2las-product", 
                            "Unable to detect which system produced this SOM "
                            "product. This is required in order to calculate "
                            "the corner coordinates correctly.  Please "
                            "specify PTYPE parameter in the pdf.");
                    free (metadata_struct);
                    return (E_FAIL);
                }
                else
                {
                    strcpy(processing_software, processing_system);
                    *software_version = 0.0;
                }
            }
            else
            {
                strncpy (processing_software, software_ver_string, pos);
                /*Know ptr won't be NULL since the previous strcspn succeeded*/
                ptr = strchr(software_ver_string, '_');
                strcpy(software_ver_tmp, (ptr+1));
                *software_version = (atof(software_ver_tmp));
            }
        }
        else if (!strcasecmp (proj_name, "UTM"))
        {
            get_utm_projparms (proj_parms);
        }
        else if (!strcasecmp (proj_name, "LCC"))
        {
            if (get_lcc_projparms (metadata_struct, *datum_code, proj_parms)
                    != E_SUCC)
            {
                c_lasmsg (FALSE, "l7hdf2las-projparms",
                        "Error getting LCC projection parameters");
                free (metadata_struct);
                return (E_FAIL);
            }
        }
        else if (!strcasecmp (proj_name, "TM"))
        {
            if (get_tm_projparms (metadata_struct, *datum_code, proj_parms)
                    != E_SUCC)
            {
                c_lasmsg (FALSE, "l7hdf2las-projparms",
                        "Error getting TM projection parameters");
                free (metadata_struct);
                return (E_FAIL);
            }
        }
        else if (!strcasecmp (proj_name, "OM"))
        {
            if (get_om_projparms (metadata_struct, *datum_code, proj_parms)
                    != E_SUCC)
            {
                c_lasmsg (FALSE, "l7hdf2las-projparms",
                        "Error getting OM projection parameters");
                free (metadata_struct);
                return (E_FAIL);
            }
        }
        else if (!strcasecmp (proj_name, "PC"))
        {
            if (get_pc_projparms (metadata_struct, *datum_code, proj_parms)
                    != E_SUCC)
            {
                c_lasmsg (FALSE, "l7hdf2las-projparms",
                        "Error getting PC projection parameters");
                free (metadata_struct);
                return (E_FAIL);
            }
        }
        else if (!strcasecmp (proj_name, "PS"))
        {
            if (get_ps_projparms (metadata_struct, *datum_code, proj_parms)
                    != E_SUCC)
            {
                c_lasmsg (FALSE, "l7hdf2las-projparms",
                        "Error getting PC projection parameters");
                free (metadata_struct);
                return (E_FAIL);
            }
        }
        else
        {
            c_lasmsg (FALSE, "l7hdf2las-projname", "Invalid projection name");
            free (metadata_struct);
            return (E_FAIL);
        }

        /* Read the corners in projection coordinates and set the projection
           units to meters
           -----------------------------------------------------------------*/
        ul_corner[0] = product_metadata->ProductULCornerMapY;
        ul_corner[1] = product_metadata->ProductULCornerMapX;
        ur_corner[0] = product_metadata->ProductURCornerMapY;
        ur_corner[1] = product_metadata->ProductURCornerMapX;
        ll_corner[0] = product_metadata->ProductLLCornerMapY;
        ll_corner[1] = product_metadata->ProductLLCornerMapX;
        lr_corner[0] = product_metadata->ProductLRCornerMapY;
        lr_corner[1] = product_metadata->ProductLRCornerMapX;
        strcpy (proj_units, "meters");
    }
    else
    {
        /* Set the pixel sizes according to the L0R DFCB
           ---------------------------------------------*/
        *vis_pixel_size = resolutions[L7_VIS];
        *thm_pixel_size = resolutions[L7_THM];
        *pan_pixel_size = resolutions[L7_PAN];

        /* The data type for L1R products is always 16-bit integers
           --------------------------------------------------------*/
        *data_type = EWORD;

        /* Initialize the projection parameters to zeros
           ---------------------------------------------*/
        for (i = 0; i < 15; i++)
            proj_parms[i] = 0.0;
    }

    free (metadata_struct);
    return (E_SUCC);
}
