/******************************************************************************

NAME: read_product_meta

PURPOSE: Read the product metadata for an L0R product.  The bands, sensor id,
         and acquisition date need to be read from this file.  Also set up
         the pixel size and initialize the projection parameters.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT:    LAS

NOTES:
  Projection information is not available in the L0R product.

******************************************************************************/

#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include "l7hdf2las.h"
#include "util.h"  /* c_splitname definition */

int read_product_meta
(
  char infile[],           /* I: full path of HDF file name */
  char drivename[],        /* I: CD drive name, if processing from CD */
  char output_dir[],       /* I: directory name of the output file */
  int links_needed,        /* I: flag to indicate links are needed for 
                                 accessing files */
  int meta_bands[9],       /* O: bands listed for this product in the metadata
                                 (this list is sorted in ascending order) */
  int *num_meta_bands,     /* O: number of bands in the metadata */
  char sensor_id[15],      /* O: sensor id listed in the metadata */
  int *product_type,       /* O: type of HDF file this is (L0R) */
  int *data_type,          /* O: data type of the product */
  char acquisition_date[9], /* O: acquisition date listed in the metadata
                                  (yyyymmdd) */
  double proj_parms[15],   /* O: 15 projection parameters for GCTP */
  double *vis_pixel_size,  /* O: pixel size for visible bands */
  double *thm_pixel_size,  /* O: pixel size for thermal bands */
  double *pan_pixel_size,  /* O: pixel size for pan bands */
  long *vis_nl,            /* O: number of lines in image for visible bands */
  long *thm_nl,            /* O: number of lines in image for thermal bands */
  long *pan_nl,            /* O: number of lines in image for pan bands */
  long *vis_ns,            /* O: number of samples in image for visible bands */
  long *thm_ns,            /* O: number of samples in image for thermal bands */
  long *pan_ns,            /* O: number of samples in image for pan bands */
  int *found_product_meta  /* O: flag to specify that the product metadata was
                                 found for this file (TRUE, FALSE) */
)

{
  char acq_date[11];      /* acquisition date listed in the metadata
                             (YYYY-MM-DD) */
  char bands[10];         /* string to represent the bands present
                             (They are listed as 123456678.  A '-' is used for
                             a position holder for absent bands.) */
  char cd_file[CMLEN];    /* full path of metadata filename on CD */
  char directory[CMLEN];  /* directory portion of input HDF filename */
  char errmsg[CMLEN];     /* error message */
  char extension[CMLEN];  /* extension portion of input HDF file name */
  char hdf_file[CMLEN];   /* full path of HDF filename */
  char hdfname[CMLEN];    /* name of HDF file */
  char local_file[CMLEN]; /* local path of metadata link filename */
  char metaname[CMLEN];   /* metadata filename */
  char prod_type[4];      /* type of HDF file this is (L0R) */
  char root[CMLEN];       /* root portion of input HDF file name */
  char *tmp_ptr;          /* pointer to the last '.' in the HDF filename */
  char tmpstr[2];         /* string to hold the current band number */
  int i;                  /* looping variable */
  int nscans;             /* number of scans in this image */
  long strsize;           /* string length of the string */

  xxx_EDCMetaDataFile_TYPE *metadata_struct = NULL;
                          /* EDC metadata file structure */
  xxx_ProductMetaData_TYPE *product_metadata = NULL;
                          /* product metadata structure */
  char local_drivename[CMLEN]; /* local drive name to support CDs mounted 
                                  without a mount request */

  strcpy(local_drivename,drivename);
  strcpy(local_file,"");

  if (strcmp (local_drivename, "") || links_needed)
  {
    /* Split input HDF filename into directory, root, and extension
       ------------------------------------------------------------*/
    c_splitname (infile, directory, root, extension);

    /* handle CD's that are local to the machine */
    if (strcmp(local_drivename,"")==0)
      strcpy(local_drivename,directory);
      
    /* For CD processing we need to make sure we are dealing with upper-case
       filenames when processing the metadata.  Thus, an upper-case link to
       the metadata file on CD needs to be made.
       ---------------------------------------------------------------------*/
    sprintf (hdfname, "%s.%s", root, extension);
    strsize = (long) strlen (hdfname);
    c_low2up (hdfname, &strsize);

    /* Replace the _HDF with _MTP for the linked metadata filename
       -----------------------------------------------------------*/
    tmp_ptr = strrchr (hdfname, '_');
    if (tmp_ptr != NULL)
    {
      if (!strncmp (tmp_ptr, "_HDF", 4))
        strncpy (tmp_ptr, "_MTP", 4);
    }
    else
    {
      c_lasmsg (FALSE, "l7hdf2las-productmeta", "Error finding _HDF");
      *found_product_meta = FALSE;
      return (E_FAIL);
    }

    /* Replace the _HDF or _hdf with _MTP or _mtp for the true metadata
       filename
       ----------------------------------------------------------------*/
    sprintf (metaname, "%s.%s", root, extension);
    tmp_ptr = strrchr (metaname, '_');
    if (tmp_ptr != NULL)
    {
      if (!strncmp (tmp_ptr, "_HDF", 4))
        strncpy (tmp_ptr, "_MTP", 4);
      else if (!strncmp (tmp_ptr, "_hdf", 4))
        strncpy (tmp_ptr, "_mtp", 4);
    }
    else
    {
      c_lasmsg (FALSE, "l7hdf2las-productmeta",
        "Error finding _HDF/_hdf in metadata filename");
      *found_product_meta = FALSE;
      return (E_FAIL);
    }

    /* Does the CD file that we plan to link to exist?
       -----------------------------------------------*/
    sprintf (local_file, "%s%s", output_dir, hdfname);
    sprintf (cd_file, "%s%s", local_drivename, metaname);
    if (access (cd_file, R_OK /* read permissions */) != 0)
    {
      /* cd_file does not exist
         ----------------------*/
      *found_product_meta = FALSE;
      return (E_FAIL);
    }

    /* Make the link (EEXIST is returned in errno if the file already exists)
       ----------------------------------------------------------------------*/
    if (symlink (cd_file, local_file) != 0)
    {
      c_lasmsg (FALSE, "l7hdf2las-link", "Error linking %s to %s", cd_file,
        local_file);
      if (errno == EEXIST)
        c_lasmsg (FALSE, "l7hdf2las-exists", "%s already exists", local_file);
      *found_product_meta = FALSE;
      return (E_FAIL);
    }

    /* Copy the local filename to hdf_file
       -----------------------------------*/
    strcpy (hdf_file, local_file);

    /* Turn it back to an HDF filename
       -------------------------------*/
    tmp_ptr = strrchr (hdf_file, '_');
    if (tmp_ptr != NULL)
    {
      if (!strncmp (tmp_ptr, "_MTP", 4))
        strncpy (tmp_ptr, "_HDF", 4);
    }
    else
    {
      c_lasmsg (FALSE, "l7hdf2las-productmeta",
        "Error finding _MTP in metadata filename");
      *found_product_meta = FALSE;
      if (unlink (local_file) != 0)
        c_lasmsg (FALSE, "l7hdf2las-unlink", "Error unlinking %s", local_file);
      return (E_FAIL);
    }
  }
  else
  {
    /* Just use the infile name
       ------------------------*/
    strcpy (hdf_file, infile);
  }

  /* Read the product metadata file (allocates space for the
     xxx_EDCMetaDataFile_TYPE structure)
     -------------------------------------------------------*/
  metadata_struct = xxx_ReadEDCMetadata (hdf_file, errmsg);
  if (metadata_struct == NULL)
  {
    *found_product_meta = FALSE;
    if (strcmp (local_file, ""))
    {
      if (unlink (local_file) != 0)
        c_lasmsg (FALSE, "l7hdf2las-unlink", "Error unlinking %s", local_file);
    }
    /* Use this message for tracking problems with reading the metadata.
       It has been left commented out since the L0R metadata may not be
       here and we are instead looking for L1R/L1G metadata.
    c_lasmsg (FALSE, "l7hdf2las-ReadEDCMetadata", errmsg);
    */
    return (E_FAIL);
  }
  *found_product_meta = TRUE;
  product_metadata = &metadata_struct->ProductMetaData;

  /* Unlink the local file
     ---------------------*/
  if (strcmp (local_file, ""))
  {
    if (unlink (local_file) != 0)
    {
      c_lasmsg (FALSE, "l7hdf2las-unlink", "Error unlinking %s", local_file);
      *found_product_meta = FALSE;
      free (metadata_struct);
      return (E_FAIL);
    }
  }

  /* Get the bands.  They are listed as 123456678.  A '-' is used for a
     position holder for absent bands.  The first 6 is 61 and the second 6
     is 62.  123---678 represents bands 1,2,3,62,7,8.
     ---------------------------------------------------------------------*/
  strncpy (bands, product_metadata->Band_combination, 9);
  bands[9] = '\0';
  *num_meta_bands = 0;
  for (i = 0; i < 9; i++)
  {
    if (isdigit (bands[i]))
    {
      if (i == 5)
        meta_bands[*num_meta_bands] = 61;
      else if (i == 6)
        meta_bands[*num_meta_bands] = 62;
      else
      {
        strncpy (tmpstr, &bands[i], 1);
        tmpstr[1] = '\0';
        meta_bands[*num_meta_bands] = atoi (tmpstr);
      }
      (*num_meta_bands)++;
    }
  }

  /* Get the sensor ID and product type
     ----------------------------------*/
  strncpy (sensor_id, product_metadata->SensorId, 14);
  sensor_id[14] = '\0';
  strncpy (prod_type, product_metadata->ProductType, 3);
  prod_type[3] = '\0';
  if (!strncasecmp (prod_type, "L0R", 3))
    *product_type = L0R;
  else
  {
    c_lasmsg (FALSE, "l7hdf2las-productmeta",
      "Product type can only be L0R for this image");
    free (metadata_struct);
    return (E_FAIL);
  }

  /* The product acquisition date is in the form of YYYY-MM-DD.  The format
     for this in the LAS DDR is yyyymmdd, so the '-'s need to be stripped.
     ----------------------------------------------------------------------*/
  strncpy (acq_date, product_metadata->AcquisitionDate, 10);
  acq_date[10] = '\0';
  strncpy (acquisition_date, acq_date, 4);
  acquisition_date[4] = '\0';
  strncat (acquisition_date, &acq_date[5], 2);
  strncat (acquisition_date, &acq_date[8], 2);

  /* Read the number of scans and determine the number of lines in each
     resolution.  16 lines/scan for visible, 8 lines/scan for thermal, and
     32 lines/scan for pan bands.
     ---------------------------------------------------------------------*/
  nscans = product_metadata->NumberOfScans;
  *vis_nl = nscans * lines_per_scan[L7_VIS];
  *thm_nl = nscans * lines_per_scan[L7_THM];
  *pan_nl = nscans * lines_per_scan[L7_PAN];

  /* Set the number of samples in each resolution according to the L0R DFCB
     ----------------------------------------------------------------------*/
  *vis_ns = number_of_samples[L7_VIS];
  *thm_ns = number_of_samples[L7_THM];
  *pan_ns = number_of_samples[L7_PAN];

  /* Set the pixel sizes according to the L0R DFCB
     ---------------------------------------------*/
  *vis_pixel_size = resolutions[L7_VIS];
  *thm_pixel_size = resolutions[L7_THM];
  *pan_pixel_size = resolutions[L7_PAN];

  /* Initialize the projection parameters to zeros
     ---------------------------------------------*/
  for (i = 0; i < 15; i++)
    proj_parms[i] = 0.0;

  /* The data type for L0R products is always BYTE
     ---------------------------------------------*/
  *data_type = EBYTE;

  free (metadata_struct);
  return (E_SUCC);
}
