/****************************************************************************

NAME: set_bands_from_meta

PURPOSE: Fill in the bands array to the bands specified in the metadata.
         Also determine if these bands are of the same resolution.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "las.h"           /* CMLEN definition */

int set_bands_from_meta
(
  int meta_bands[],       /* I: array of bands specified in the metadata */
  int num_meta_bands,     /* I: number of bands in meta_bands */
  int bands[],            /* O: array of bands that has been filled */
  int *num_bands,         /* O: number of bands in bands */
  int nbands[],           /* O: array of NBANDS values */
  int *num_nbands,        /* O: number of values in nbands */
  int nbands_same_res[],  /* O: array of same_res values, one for each nbands
                                value, each to specify if all the bands in the
                                nbands group are of the same resolution (TRUE,
                                FALSE) */
  int *process_vis,       /* O: flag specifying visible bands will be
                                processed (TRUE or FALSE) */
  int *process_thm,       /* O: flag specifying thermal bands will be
                                processed (TRUE or FALSE) */
  int *process_pan        /* O: flag specifying pan bands will be processed
                                (TRUE or FALSE) */
)

{
  char err_text[CMLEN];   /* text string for error messages */
  int i;                  /* looping variable */
  int tmp_same_res;       /* temporary same_res flag */

  /* Copy the metadata bands into bands
     ----------------------------------*/
  *num_bands = num_meta_bands;
  for (i = 0; i < num_meta_bands; i++)
    bands[i] = meta_bands[i];

  /* Initialize nbands and the number of nbands to 1
     -----------------------------------------------*/
  nbands[0] = *num_bands;
  *num_nbands = 1;

  /* Determine if all the meta_bands are of the same resolution
     ----------------------------------------------------------*/
  check_res (bands, *num_bands, &tmp_same_res);
  nbands_same_res[0] = tmp_same_res;

  /* Verify that the bands are valid bands (1, 2, 3, 4, 5, 61, 62, 7, 8).
     Also determine which resolutions will be processed
     --------------------------------------------------------------------*/
  *process_vis = FALSE;
  *process_thm = FALSE;
  *process_pan = FALSE;
  for (i = 0; i < *num_bands; i++)
  {
    switch (bands[i])
    {
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 7:
        *process_vis = TRUE;
        break;

      case 61:
      case 62:
        *process_thm = TRUE;
        break;

      case 8:
        *process_pan = TRUE;
        break;

      default:
        sprintf (err_text, "Error: Band %d is invalid", bands[i]);
        c_lasmsg (FALSE, "l7hdf2las-metabands", err_text);
        return (E_FAIL);
    }
  }

  return (E_SUCC);
}
