/****************************************************************************

NAME: srch_dir

PURPOSE: Search the specified filename and determine which of the bands
  are listed on this CD or disk.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    06/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "cm.h"    /* c_getfiles definition */
#include "util.h"  /* c_splitname definition */

int srch_dir
(
  char infile[],        /* I: HDF file name (Unix format) */
  int bands[],          /* I: array of bands to search for on this CD or disk */
  int num_bands,        /* I: number of bands in bands */
  int product_type,     /* I: type of HDF file this is (L0R, L1R, L1G, L1T) */
  int bands_avail[],    /* O: array of bands from the bands array that are
                              available on this CD or disk (in ascending
                              order) */
  int *avail_count,     /* O: number of bands in bands_avail */
  char avail_file_list[MAXFILE][CMLEN],
                        /* O: list of filenames for the bands_avail list */
  int *found            /* O: specifies if any bands were found on this CD
                              or disk */
)

{
  char band_str[3];       /* string to hold ## */
  char directory[CMLEN];  /* directory portion of input HDF filename */
  char extension[CMLEN];  /* extension portion of input HDF file name */
  char found_files[MAXFILE][CMLEN];
                          /* list of filenames found in the wildcard search */
  char root[CMLEN];       /* root portion of input HDF file name */ 
  char root_tmp[CMLEN];   /* root portion of input HDF file name (temp) */ 
  char srchfile[CMLEN];   /* wildcard file to search for */
  char *tmp_ptr;          /* pointer to the last '_' in the HDF filename */

  int band_val;           /* integer band value */
  int i, index;           /* looping variables */
  int nfiles;             /* number of files found in the wildcard search */
  int root_len;           /* string length of the root */

  /* Split input HDF filename into directory, root, and extension
     ------------------------------------------------------------*/
  c_splitname (infile, directory, root, extension);

  /* If this is an L0R product then insert "[12]" at the 7th character of
     the root so that we search for both format 1 and format 2 filenames.
     L7XsssfnYYDOYHHuuv_xxx has 22 characters.
     --------------------------------------------------------------------*/
  if (product_type == L0R)
  {
    root_len = strlen (root);
    if (root_len != 22)
    {
      c_lasmsg (FALSE, "l7hdf2las-l0rname",
        "L0R root name %s must be of the form L7XsssfnYYDOYHHuuv_xxx", root);
      return (E_FAIL);
    }

    /* Copy the first 6 characters
       ---------------------------*/
    for (i = 0; i < 6; i++)
      root_tmp[i] = root[i];
    root_tmp[i] = '\0';

    /* Put in the format 1 and 2 wildcard.  The next digit can change, so
       use the ? for any digit.
       ------------------------------------------------------------------*/
    strcat (root_tmp, "[12]?");

    /* Copy the rest of root (except the 7th and 8th characters) into
       root_tmp.  Recall we have added 3 extra characters to the root.
       ---------------------------------------------------------------*/
    for (i = 8; i < root_len; i++)
      root_tmp[i+3] = root[i];
    root_tmp[root_len+3] = '\0';

    /* Put root_tmp back into root
       ---------------------------*/
    strcpy (root, root_tmp);
  }

  /* If this is an L1R, L1G, or L1T product then insert "[12]" at the 3rd
     character of the root so that we search for both format 1 and format 2
     filenames.  L7fppprrr_rrrYYYYMMDD_AAA has 25 characters.
     ----------------------------------------------------------------------*/
  else if ((product_type == L1R) || (product_type == L1G) ||
           (product_type == L1T))
  {
    root_len = strlen (root);
    if (root_len != 25)
    {
      c_lasmsg (FALSE, "l7hdf2las-l1rgname",
       "L1R/L1G/L1T root name %s must be of the form L7fppprrr_rrrYYYYMMDD_AAA",
        root);
      return (E_FAIL);
    }

    /* Copy the first 2 characters
       ---------------------------*/
    for (i = 0; i < 2; i++)
      root_tmp[i] = root[i];
    root_tmp[i] = '\0';

    /* Put in the format 1 and 2 wildcard
       ----------------------------------*/
    strcat (root_tmp, "[12]");

    /* Copy the rest of root (except the 3rd character) into root_tmp.
       Recall we have added 3 extra characters to the root.
       ---------------------------------------------------------------*/
    for (i = 3; i < root_len; i++)
      root_tmp[i+3] = root[i];
    root_tmp[root_len+3] = '\0';

    /* Put root_tmp back into root
       ---------------------------*/
    strcpy (root, root_tmp);
  }

  /* Set up the search wildcard.  Strip any characters after the last
     occurrence of '_' in srchfile and append a '*' after the last occurrence
     of the '_' in srchfile to find all the files for this product.
     Then concatenate the directory and root.  The search for '_' should be
     done before the root and directory are concatinated since an '_' may
     not exist in the root but may be incorrectly found in the directory.
     ----------------------------------------------------------------------*/
  tmp_ptr = strrchr (root, '_');
  if (tmp_ptr == NULL)
  {
    c_lasmsg (FALSE, "l7hdf2las-HDFfilename", "Invalid HDF filename format");
    *found = FALSE;
    return (E_FAIL);
  }
  tmp_ptr[0] = '*';    /* Replace '_' with '*' */
  tmp_ptr[1] = '\0';   /* End the string */
  strcpy (srchfile, directory);
  strcat (srchfile, root);

  /* Get the filenames that match the srchfile string
     ------------------------------------------------*/
  if (c_getfiles (srchfile, found_files, &nfiles, MAXFILES, 1 /*first call*/,
    0 /*return list of all files found*/) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-getfiles", "Error searching for files");
    *found = FALSE;
    return (E_FAIL);
  }

  /* Loop through all the found files and search for _b## or _B##
     ------------------------------------------------------------*/
  *avail_count = 0;
  *found = FALSE;
  for (i = 0; i < nfiles; i++)
  {
    /* Split input HDF filename into directory, root, and extension
       ------------------------------------------------------------*/
    c_splitname (found_files[i], directory, root, extension);

    /* Find the last occurrence of '_' in the current file
       ---------------------------------------------------*/
    tmp_ptr = strrchr (root, '_');
    if (tmp_ptr != NULL)
    {
      if (!strncasecmp (tmp_ptr, "_b", 2))
      { /* Found a band file */
        strncpy (band_str, &tmp_ptr[2], 2);   /* ## in _b## */
        band_str[2] = '\0';
        band_val = atoi (band_str);

        /* Determine the true band number for each band_val.
           Bands 61 and 62 are already correct band numbers.
           -------------------------------------------------*/
        switch (band_val)
        {
          case 10:
          case 20:
          case 30:
          case 40:
          case 50:
          case 70:
          case 80:
          case 81:
            /* Integer-divide band_val by 10 to get the true band
               number.  For example, 10 / 10 = band 1.
               --------------------------------------------------*/
            band_val /= 10;
            break;

          case 60:
            /* This will only occur in a L0R product.  The 7th character in
               the base filename will provide the format number (L72EDC#...).
               If this format number is 1 then look for band 61.  If it is a
               2 then look for band 62.
               -------------------------------------------------------------*/
            if (root[6] == '1')
              band_val = 61;
            else if (root[6] == '2')
              band_val = 62;
            else
            {
              c_lasmsg (FALSE, "l7hdf2las-HDFfilename",
                "HDF filename is not format 1 or 2");
              *found = FALSE;
              return (E_FAIL);
            }

            break;
        }  /* end switch */

        /*  Look for band_val in the bands array
            ------------------------------------*/
        for (index = 0; index < num_bands; index++)
        {
          if (bands[index] == band_val)
          {
            bands_avail[*avail_count] = band_val;
            strcpy (avail_file_list[*avail_count], found_files[i]);
            (*avail_count)++;
            *found = TRUE;
            break;
          }
        }
      }  /* end if */
    }  /* end if */
  }  /* end for */

  return (E_SUCC);
}
