/****************************************************************************

NAME: unmount_cd

PURPOSE: Close and unmount the specified CD.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

NOTES: The SIGTERM signal will be held during an unmount and handled after
  the unmount has finished.

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "util.h"       /* c_block_cleanup and c_unblock_cleanup routines */
#include "las.h"        /* CMLEN for tapeio routines */
#include "tapeio.h"     /* tape routines and defines */

int unmount_cd
(
  int *mounted,    /* I/O: flag to specify if a CD is mounted */
  long *unit       /* I/O: CD unit number */
)

{
  int blocked;                /* was the SIGTERM signal blocked? */
  long err_mode = HANDLE;     /* print the error messages */
  long rewind_flag = NOREW;   /* don't rewind the CD */
  long zero = 0;              /* constant zero */
  sigset_t oldmask;           /* status of signal mask before SIGTERM was
                                 masked to be blocked */

  if (*mounted)
  {
    /* Block the SIGTERM signal during the unmount
       -------------------------------------------*/
    if (c_block_cleanup (&oldmask) != E_SUCC)
    {
      c_lasmsg (FALSE, "l7hdf2las-block",
        "Warning: Cannot block the SIGTERM signal while unmounting the CD");
      blocked = FALSE;
    }
    blocked = TRUE;

    /* Take CD offline
       ---------------*/
    rewind_flag = REWOFFL;
    if (c_tpclos (unit, &err_mode, &rewind_flag, &zero /*don't write EOF marks
      at the end of the CD*/) != TAPE_SUCC)
    {
      c_lasmsg (FALSE, "l7hdf2las-tpclos", "Error encountered closing the CD");
      return (E_FAIL);
    }
    *mounted = FALSE;

    /* Reset the SIGTERM signal mask if it was blocked
       -----------------------------------------------*/
    if (blocked)
    {
      if (c_unblock_cleanup (&oldmask) != E_SUCC)
      {
        c_lasmsg (FALSE, "l7hdf2las-reset",
          "Error: Cannot reset the SIGTERM signal after unmounting the CD");
        return (E_FAIL);
      }
    }
  }

  return (E_SUCC);
}
