/******************************************************************************

NAME: write_ddr

PURPOSE: Create and write the DDR for the specified image.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT:    LAS

******************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "l7hdf2las.h"
#include "imageio.h"   /* SL, SS, NL, NS definitions */
#include "ddr.h"       /* DDR, VALID, INVAL definitions */
#include "util.h"      /* c_splitname definition */
#include "proj.h"      /* SOM definition */

int write_ddr
(
  char imgname[CMLEN],   /* I: filename of the output image (Unix format) */
  int bands[],           /* I: array of bands to process */
  int num_bands,         /* I: number of values in bands */
  long data_type,        /* I: data type of the product
                               (0=SAME, EBYTE=BYTE, EWORD=I*2, ELONG=I*4,
                                EREAL=R*4) */
  char proj_units[],     /* I: units of image projection corners */
  long proj_code,        /* I: integer value to specify the projection */
  long zone_code,        /* I: the UTM zone, for UTM projections */
  long datum_code,       /* I: integer value to specify the datum */
  double pixel_size,     /* I: pixel size for current band */
  double proj_parms[15], /* I: 15 projection parameters for GCTP */
  double ul_corner[2],   /* I: projection coordinates of upper left corner
                               in the image (x,y) in units of proj_units */
  double ur_corner[2],   /* I: projection coordinates of upper right corner
                               in the image (y,x) in units of proj_units */
  double ll_corner[2],   /* I: projection coordinates of lower left corner
                               in the image (y,x) in units of proj_units */
  double lr_corner[2],   /* I: projection coordinates of lower right corner
                               in the image (y,x) in units of proj_units */
  char acquisition_date[10],
                         /* I: acquisition date listed in the metadata
                               (dd-mmm-yy) in the LAS DDR format */
  char sensor_id[8],     /* I: sensor id listed in the metadata */
  long window[4],        /* I: array of window values in line/sample
                               coordinates (SL, SS, NL, NS) */
  long orig_nl,          /* I: original number of lines in the image */
  long orig_ns,          /* I: original number of samples in the image */
  int product_type,      /* I: type of HDF file this is (L0R, L1R, L1G, L1T) */
  char minmax[],         /* I: minmax tae global */
  double software_version, /* I: IAS or LPGS processing software version */
  char processing_software[]/* I: IAS or LPGS software */
)

{
    int i;                  /* looping variable */

    char *datasys = NULL;   /* pointer to the DATASYS environment variable */
    char directory[CMLEN];  /* directory portion of input HDF filename */
    char errmsg[CMLEN];     /* error message */
    char extension[CMLEN];  /* extension portion of input HDF file name */
    char hostimg[CMLEN];    /* host image name without the extension */
    char product_type_str[4];/* string for the product type (L0R,L1R,L1G,L1T) */
    char root[CMLEN];       /* root portion of input HDF file name */

    double save_ul_corner_y; /* temp place holders for SOM */
    double save_ul_corner_x; /* corner coordinates         */
    double save_ur_corner_y; /*          ''                */
    double save_ur_corner_x; /*          ''                */
    double save_ll_corner_y; /*          ''                */
    double save_ll_corner_x; /*          ''                */
    double save_lr_corner_y; /*          ''                */
    double save_lr_corner_x; /*          ''                */

    int save_corners;       /* flag indicating corner coordinates were saved */

    struct BDDR *bddr=NULL; /* pointer to the array of band dependant records */
    struct DDR ddr;         /* local ddr structure */


    /* Strip the extension from the image filename and assign the DDR name
       to the basename of the image
       -------------------------------------------------------------------*/
    c_splitname (imgname, directory, root, extension);
    sprintf (hostimg, "%s%s", directory, root);

    /* Declare num_bands BDDR structures
       ---------------------------------*/
    if ((bddr = (struct BDDR *) malloc (num_bands * sizeof (struct BDDR)))
            == NULL)
    {
        c_lasmsg (FALSE,"l7hdf2las-alloc", "Error allocating memory for BDDRs");
        return (E_FAIL);
    }

    /* Fill in the DDR structure
       -------------------------*/
    ddr.nl = window[NL];
    ddr.ns = window[NS];
    ddr.dtype = data_type;
    ddr.master_line = 1;
    ddr.master_sample = 1;
    ddr.pdist_x = pixel_size;
    ddr.pdist_y = pixel_size;
    ddr.line_inc = 1.0;
    ddr.sample_inc = 1.0;
    ddr.nbands = num_bands;

    /* Get the system from the DATASYS environment variable
       ----------------------------------------------------*/
    datasys = getenv ("DATASYS");
    if (datasys != NULL)
        strncpy (ddr.system, datasys, 12); /* 12 length */
    else
    {
        c_lasmsg (FALSE, "l7hdf2las-datasys",
                "Warning: DATASYS environment variable not set");
        strcpy (ddr.system, "UNKNOWN");
    }

    /* Set the valid flags for ground units and distance to VALID since
       these values will be set.  
       ----------------------------------------------------------------*/
    ddr.valid[4] = VALID;   /* ground units */
    ddr.valid[5] = VALID;   /* ground distance */
    ddr.valid[7] = VALID;   /* line/sample increments */

    /* Fill in the projection information in the DDR structure. This
       information is only available for L1G and L1T imagery.
       -------------------------------------------------------------*/
    if ((product_type == L1G) || (product_type == L1T))
    {
        save_corners = FALSE;

        if (proj_code == SOM)
        {
            /* If SOM projection, swap corners for NLAPS products and for 
               IAS/LPGS products with the software version of 6.1 and greater.
               LAS expects the SOM corner coordinates to be in the same
               orientation as all other projections (Y increading upward and X
               increasing to the right) and NLAPS, IAS/LPGS version 6.1 and up
               do not reflect this in the MTL file. 
               ---------------------------------------------------------------*/
            if ((((strcasecmp(processing_software, "ias") == 0) ||
                            (strcasecmp(processing_software, "lpgs") == 0)) &&
                   (software_version > 6.0)) ||
                    (strcasecmp(processing_software, "nlaps") == 0))

            {
                /* Save original corners. These corners are saved becasue
                   write_ddr is called every time a new file is output.
                   This happens when for each resolution. We don't want
                   to overwrite the original corners found in the MTL file */
                save_ul_corner_y = ul_corner[0];
                save_ul_corner_x = ul_corner[1];
                save_ur_corner_y = ur_corner[0];
                save_ur_corner_x = ur_corner[1];
                save_ll_corner_y = ll_corner[0];
                save_ll_corner_x = ll_corner[1];
                save_lr_corner_y = lr_corner[0];
                save_lr_corner_x = lr_corner[1];

                save_corners = TRUE;

                /* Swap corner coordinates; x=y, y=-x */
                ul_corner[0] = (ul_corner[1]) * -1;
                ul_corner[1] = save_ul_corner_y;

                ur_corner[0] = (ur_corner[1]) * -1;
                ur_corner[1] = save_ur_corner_y;

                ll_corner[0] = (ll_corner[1]) * -1;
                ll_corner[1] = save_ll_corner_y;

                lr_corner[0] = (lr_corner[1]) * -1;
                lr_corner[1] = save_lr_corner_y;
            }
        }                   

        /* Calculate the corner points in projection y/x
           ---------------------------------------------*/
        /* UL */
        ddr.upleft[0] = ul_corner[0] -
            (double) (window[SL] - 1) * pixel_size;
        ddr.upleft[1] = ul_corner[1] +
            (double) (window[SS] - 1) * pixel_size;

        /* UR */
        ddr.upright[0] = ur_corner[0] -
            (double) (window[SL] - 1) * pixel_size;
        ddr.upright[1] = ur_corner[1] -
            (double) (orig_ns - (window[SS] + window[NS] - 1)) * pixel_size;

        /* LL */
        ddr.loleft[0] = ll_corner[0] +
            (double) (orig_nl - (window[SL] + window[NL] - 1)) * pixel_size;
        ddr.loleft[1] = ll_corner[1] +
            (double) (window[SS] - 1) * pixel_size;

        /* LR */
        ddr.loright[0] = lr_corner[0] +
            (double) (orig_nl - (window[SL] + window[NL] - 1)) * pixel_size;
        ddr.loright[1] = lr_corner[1] -
            (double) (orig_ns - (window[SS] + window[NS] - 1)) * pixel_size;

        /* Restore original corner coordinates for SOM projection just in case
           write_ddr routine is called again due to writing out another
           output file (one example is for a new resolution) */
        if (save_corners)
        {
            ul_corner[0] = save_ul_corner_y;
            ul_corner[1] = save_ul_corner_x;
            ur_corner[0] = save_ur_corner_y;
            ur_corner[1] = save_ur_corner_x;
            ll_corner[0] = save_ll_corner_y;
            ll_corner[1] = save_ll_corner_x;
            lr_corner[0] = save_lr_corner_y;
            lr_corner[1] = save_lr_corner_x;
        }

        strcpy (ddr.proj_units, proj_units);
        ddr.proj_code = proj_code;
        ddr.zone_code = zone_code;
        ddr.datum_code = datum_code;
        for (i = 0; i < 15; i++)
            ddr.proj_coef[i] = proj_parms[i];

        ddr.valid[0] = VALID;   /* projection code */
        ddr.valid[1] = VALID;   /* zone code */
        ddr.valid[2] = VALID;   /* datum code */
        ddr.valid[3] = VALID;   /* projection coefficients */
        ddr.valid[6] = VALID;   /* corner coordinates */
    }
    else
    {
        /* Projection information is not available for L0R and L1R products
           ----------------------------------------------------------------*/
        ddr.upleft[0] = 0;
        ddr.upleft[1] = 0;
        ddr.upright[0] = 0;
        ddr.upright[1] = 0;
        ddr.loleft[0] = 0;
        ddr.loleft[1] = 0;
        ddr.loright[0] = 0;
        ddr.loright[1] = 0;
        strcpy (ddr.proj_units, "meters");  /* the pixel sizes are in meters */
        ddr.proj_code = 0;
        ddr.zone_code = 0;
        ddr.datum_code = 0;
        for (i = 0; i < 15; i++)
            ddr.proj_coef[i] = 0.0;

        ddr.valid[0] = INVAL;   /* projection code */
        ddr.valid[1] = INVAL;   /* zone code */
        ddr.valid[2] = INVAL;   /* datum code */
        ddr.valid[3] = INVAL;   /* projection coefficients */
        ddr.valid[6] = INVAL;   /* corner coordinates */
    }

    /* Output the DDR
       --------------*/
    if (c_putddr (hostimg, &ddr) != E_SUCC)
    {
        c_lasmsg (FALSE, "l7hdf2las-ddr", "Error outputting the DDR");
        free (bddr);
        return (E_FAIL);
    }

    /* Fill in the BDDR structures for each band
       -----------------------------------------*/
    for (i = 0; i < num_bands; i++)
    {
        bddr[i].bandno = i + 1;
        bddr[i].valid = INVAL;  /* min/max is not valid */
        bddr[i].minval = 0.0;   /* since it is not available */
        bddr[i].maxval = 0.0;   /* since it is not available */

        if (product_type == L0R)
            strcpy (product_type_str, "L0R");
        else if (product_type == L1R)
            strcpy (product_type_str, "L1R");
        else if (product_type == L1G)
            strcpy (product_type_str, "L1G");
        else if (product_type == L1T)
            strcpy (product_type_str, "L1T");
        sprintf (bddr[i].source, "LANDSAT7_%s_BAND_%d", product_type_str,
                bands[i]);

        strcpy (bddr[i].direction, "");  /* since it is not available */
        strncpy (bddr[i].instrument, sensor_id, 7);
        bddr[i].instrument[7] = '\0';
        strncpy (bddr[i].date, acquisition_date, 9);
        bddr[i].date[9] ='\0';
        strcpy (bddr[i].time, "");  /* since it is not available */

        /* Output this BDDR record
           -----------------------*/
        if (c_putbdr (hostimg, &bddr[i]) != E_SUCC)
        {
            sprintf (errmsg,"Error outputting the %d BDDR record for %s\n", i+1,
                    imgname);
            c_lasmsg (FALSE, "l7hdf2las-bddr", errmsg);
            free (bddr);
            return (E_FAIL);
        }
    }

    /* Update the minmax fields if requested 
       -------------------------------------*/
    if (strcmp(minmax,"YES") == 0)
    {
        long nimg = 1; 
        char filename[1][CMLEN];
        double minval[MAXBND];
        double maxval[MAXBND];
        long imgwindow[1][4];
        long imgbands[1][MAXBND+1];
        long img_numbands = num_bands;
        int i;

        /* setup for call to c_minmax (mainly to eliminate compiler warnings
           due to type mismatches) 
           -----------------------------------------------------------------*/
        strcpy(filename[0],imgname);
        for (i = 0; i < 4; i++)
            imgwindow[0][i] = window[i];
        for (i = 0; i < num_bands; i++)
            imgbands[0][i] = i + 1;
        imgbands[0][num_bands] = 0;

        /* fill in the min/max fields in the DDR */
        if (c_minmax(filename,&nimg,imgwindow, imgbands, &img_numbands, minval, 
                    maxval) != E_SUCC)
        {
            /* issue a warning message to warn user that minmax update failed */
            c_lasmsg(FALSE, "l7hdf2las-minmax", 
                    "Warning: Unable to update min/max field");
        }
    }

    /* Set the last used date and time
       -------------------------------*/
    if (c_lused (hostimg) != E_SUCC)
    {
        c_lasmsg (FALSE, "l7hdf2las-lused",
                "Error retrieving the current date and time");
        free (bddr);
        return (E_FAIL);
    }

    /* Free the BDDR space
       -------------------*/
    free (bddr);

    return (E_SUCC);
}
