/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_ChkForSDS.c

PURPOSE:  Find and attach to the SDS name in the file..

INVOCATION METHOD:
	status = xxx_ChkForSDS(p_ImageryInfo, ProductType, p_ErrorMessage);

RETURN VALUE:
Type = int

Value           Description
--------------  ----------------------------------------------------------------
ERROR           Fail to find SDS name in file
XXXSUCCESS      Found SDS name in file

ARGUMENT LIST:
Name             Type           	Use	Description
---------------- --------------		---	---------------------------------------
p_ImageryInfo    xxx_ImageryInfo_TYPE * I       HDF imagery information
ProductType	 int		       	I	Indicates L0R or L1 product type
p_ErrorMessage	 char *			O	Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
HDF file                        I       HDF file

EXTERNAL VARIABLES:
Source	               	Name		       	Type		Use	Description
---------------	        ----------------      	---------------	---	-----------------------
herr.h                  error_top               int32           I       Top to HDF error stack

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                   Description
---------------        -------------------------------------------------
HEstring	       Returns error message for the error number
HEvalue                Returns the current error number
SDendaccess            Detach from SDS name
SDgetinfo              Get information on data set
SDnametoindex          Get the index to the SDS name
SDselect               Select the SDS name
xxx_CompareObject2     Compare for match

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
HDF errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               N/A             R1.0    07/02/97        Original Implementation
J. kerich               IASI990538      R3.2    11/23/98        Add ProductType flag

NOTES:
none

Unit xxx_ChkForSDS PDL:

        Call SDnametoindex() to get the index to the SDS name
1	IF SDS name does not exist THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage

	        Call SDfileinfo() to retrieve SDS name count
2               IF error retrieving count  THEN
                        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                        RETURN ERROR 
3               ENDIF

4		DOFOR SDS name found
                       Call SDselect() to select the SDS name
5                      IF error THEN
                                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                                RETURN ERROR 
6                      ENDIF
 
		       Call SDgetinfo() to get information on data set and store it in p_ImageryInfo
7		       IF error THEN
			        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
			      	Call SDendaccess to detach from SDS name
			        RETURN ERROR 
8		       ENDIF

9		       IF xxx_CompareObject2() using ProductType says object doesn't match SDSName THEN
			        Call SDendaccess() to detach from SDS name
10			     	IF error THEN
					Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
				        RETURN ERROR 
11				ENDIF
12		       ELSE
		                Leave the loop
		       ENDIF
13		ENDDO

14	       	IF xxx_CompareObject2() using ProductType says p_ObjectName was not found THEN
  		       Build error message in p_ErrorMessage
 	  	       RETURN ERROR
15   		ENDIF
16	ELSE
	        Call SDselect() to select the SDS name
17	        IF error THEN
	               Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
		       RETURN ERROR
18	        ENDIF
  	ENDIF

	RETURN XXXSUCCESS

End Unit xxx_ChkForSDS
******************************************************************************/

/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Local headers */
#include <hdf.h>
#include <mfhdf.h>
#include "xxx_HDFDirectory.h"
#include "xxx_Imagery.h"
#include "xxx_Types.h"

extern int32 error_top;

int xxx_ChkForSDS(xxx_ImageryInfo_TYPE *p_ImageryInfo, int ProductType, char *p_ErrorMessage)
{
	const char *p_HEstring;

	int32   sds_index;
        char    ObjectName[VSNAMELENMAX+1];
        int32   n_datasets;    /* number of data sets */
        int32   nattrs;        /* number of attributes */
        int32   DataType;
        int32   Rank;
        int32   DimensionSizes[MAX_VAR_DIMS];

	if ((sds_index = SDnametoindex(p_ImageryInfo->SDId, p_ImageryInfo->SDSName)) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Can't find SDS name: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);

		if (SDfileinfo(p_ImageryInfo->SDId, &n_datasets, &nattrs) == FAIL)
		{
			(void)strcpy(p_ErrorMessage, "Can't get file information: ");
			p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
			(void)strcat(p_ErrorMessage, p_HEstring);
			return (ERROR);
		}

		ObjectName[0] = '\0';
		for (sds_index = 0; sds_index < n_datasets; sds_index++)
		{
			if ((p_ImageryInfo->SDS_Id = SDselect(p_ImageryInfo->SDId, sds_index)) == FAIL)
			{
				(void)strcpy(p_ErrorMessage, "SDselect failed: ");
				p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
				(void)strcat(p_ErrorMessage, p_HEstring);
				return (ERROR);
			}

			if (SDgetinfo(p_ImageryInfo->SDS_Id, ObjectName, &Rank, DimensionSizes, &DataType, &nattrs) == FAIL)
			{
				(void)strcpy(p_ErrorMessage, "Can't get file information: ");
				p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
				(void)strcat(p_ErrorMessage, p_HEstring);
				(void)SDendaccess(p_ImageryInfo->SDS_Id);
				return (ERROR);
			}

			if (xxx_CompareObject2(p_ImageryInfo->SDSName, ProductType, ObjectName))
			{
				if (SDendaccess(p_ImageryInfo->SDS_Id) == FAIL)
				{
					(void)strcpy(p_ErrorMessage, "Can't end access: ");
					p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
					(void)strcat(p_ErrorMessage, p_HEstring);
					return (ERROR);
				}
			}
			else
			{
				break;
			}
		}

		if (xxx_CompareObject2(p_ImageryInfo->SDSName, ProductType, ObjectName))
		{
			return (ERROR);
		}
	}
	else
	{
		if ((p_ImageryInfo->SDS_Id = SDselect(p_ImageryInfo->SDId, sds_index)) == FAIL)
		{
			(void)strcpy(p_ErrorMessage, "SDselect failed: ");
			p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
			(void)strcat(p_ErrorMessage, p_HEstring);
			return (ERROR);
		}
	}

	return XXXSUCCESS;
}
