/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_ChkForSDSInHDF.c

PURPOSE:  Find and attach to the SDS name in the HDF directory..

INVOCATION METHOD:
	p_HDFDirectory = xxx_ChkForSDSInHDF(p_ImageryInfo, ProductType, p_ErrorMessage);

RETURN VALUE:
Type = xxx_HDFDirectoryControl_TYPE *

Value           Description
--------------  ----------------------------------------------------------------
XXXSUCCESS      Found SDS name in a vgroup
ERROR           SDS name not found or HDF error

ARGUMENT LIST:
Name             Type           	Use	Description
---------------- --------------		---	---------------------------------------
p_ImageryInfo   xxx_ImageryInfo_TYPE *  I       HDF imagery infor
ProductType	int		       	I	Indicates L0R or L1 product type
p_ErrorMessage	char *			O	Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
HDF file                        I       HDF file

EXTERNAL VARIABLES:
Source	               	Name		       	Type		Use	Description
---------------	        ----------------      	---------------	---	-----------------------
hdf.h                   FAIL                   	int             I       HDF failure status
herr.h                  error_top               int32           I       Top to HDF error stack
hlimits.h               VSNAMELENMAX            int             I       Max vdata name
htags.h                 DFTAG_NDG               int             I       Scientific Data real tag
htags.h                 DFTAG_SD                int             I       Scientific Data int tag
xxx_HDFDirectory.h      XXX_INDEX_LISTING      	char *          I       HDF directory listing
xxx_HDFDirectory.h      XXX_VGROUP1      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP2      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP3      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP4      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP5      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP6      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP7      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP8      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP9      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP10      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP11      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP12      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP13      	char *          I       Defined EDC DAAC vgroup
xxx_HDFDirectory.h      XXX_VGROUP1_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP2_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP3_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP4_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP5_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP6_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP7_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP8_L1      	char *          I       Defined LPGS vgroup
xxx_HDFDirectory.h      XXX_VGROUP9_L1      	char *          I       Defined LPGS vgroup

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                   Description
---------------        -------------------------------------------------
HEstring	       Returns error message for the error number
HEvalue                Returns the current error number
SDendaccess            Detach from SDS name
SDgetinfo              Get information on data set
SDreftoindex           Get the SDS index for given SDS reference
Vattach	               Establish access to a specified vgroup
Vdetach	               Detach from vgroup
Vfind		       Get vgroup reference id
Vgettagref             Get the objects tag and reference
Vinquire	       Get information on vgroup
XXX_ARRAY_SIZE         Defined Macro
xxx_CompareObject2     Compare for match

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
HDF errors
memory errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               N/A             R1.0    04/16/97        Original Implementation
J. Kerich	        IAS970051       R3.0    03/11/98	Test tag for  DFTAG_NDG setting
J. kerich               IASI990538      R3.2    11/23/98        Add ProductType flag

NOTES:
when p_OjbectName is set to "IndexListing" the unit returns a information on the vgroups.

Unit xxx_ChkForSDSInHDF PDL:

1	DOFOR all vgroups AND ObjectName not found
	        Call Vfind() to get currect vgroup reference id
2               IF vgroup not found  THEN
                        Continue 
3               ENDIF

	        Call Vattach() to attach to the vgroup
4               IF error attaching to vgroup THEN
                        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                        ABORT to ERROR_PROC 
                ENDIF

	        Call Vinquire() to get information on vgroup
5               IF error getting vgroup information THEN
                        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                        ABORT to ERROR_PROC 
6               ENDIF

7		DOFOR all Object in the ProductType Vgroups AND xxx_CompareObject2() says ObjectName not found
	                Call Vgettagref() to get the object's tag and reference
8			IF error getting object information  THEN
                                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                                ABORT to ERROR_PROC 
9			ENDIF

10			IF tag is scientific data  THEN
	                        Call SDreftoindex() to get the SDS index
11			        IF error getting SDS index  THEN
			                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
					ABORT to ERROR_PROC 
12				ENDIF

				Call SDselect() to select the SDS name
13				IF error THEN
				        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
					ABORT to ERROR_PROC 
14				ENDIF

				Call SDgetinfo() to get information on data set and store it in p_ImageryInfo
15				IF error THEN
				        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
					Call SDendaccess to detach from SDS name
					ABORT to ERROR_PROC 
16				ENDIF

17				IF xxx_CompareObject2() says object doesn't match p_ObjectName THEN
				        Call SDendaccess() to detach from SDS name
18					IF error THEN
					        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
						ABORT to ERROR_PROC 
19					ENDIF
20				ENDIF

21			ENDIF
22,23		ENDDO

		Call Vdetach() to detach from vgroup
24,25   ENDDO

26	IF xxx_CompareObject2() says p_ObjectName was not found THEN
                Build error message in p_ErrorMessage
 	      	ABORT to ERROR_PROC
27      ENDIF

	RETURN xxx_HDFDirectoryControl_TYPE structure

ERROR_PROC:
	Call Vdetach() to detach from vgroup

	RETURN NULL

End Unit xxx_ChkForSDSInHDF
******************************************************************************/

/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Local headers */
#include <hdf.h>
#include <mfhdf.h>
#include "xxx_Imagery.h"
#include "xxx_HDFDirectory.h"
#include "xxx_Types.h"

extern int32 error_top;

int xxx_ChkForSDSInHDF(xxx_ImageryInfo_TYPE *p_ImageryInfo, int ProductType, char *p_ErrorMessage)
{
	xxx_HDFDirInfo_TYPE VgroupInfo[2][XXX_NUMBER_OF_VGROUPS] = {
	{
		{XXX_VGROUP1, ""},
		{XXX_VGROUP2, ""},
		{XXX_VGROUP3, ""},
		{XXX_VGROUP4, ""},
		{XXX_VGROUP5, ""},
		{XXX_VGROUP6, ""},
		{XXX_VGROUP7, ""},
		{XXX_VGROUP8, ""},
		{XXX_VGROUP9, ""},
		{XXX_VGROUP10, ""},
		{XXX_VGROUP11, ""},
		{XXX_VGROUP12, ""},
		{XXX_VGROUP13, ""}
	},
	{
		{XXX_VGROUP1_L1,  ""},
		{XXX_VGROUP2_L1,  ""},
		{XXX_VGROUP3_L1,  ""},
		{XXX_VGROUP4_L1,  ""},
		{XXX_VGROUP5_L1,  ""},
		{XXX_VGROUP6_L1,  ""},
		{XXX_VGROUP7_L1,  ""},
		{XXX_VGROUP8_L1,  ""},
		{XXX_VGROUP9_L1,  ""},
		{XXX_VGROUP10, ""},
		{XXX_VGROUP11, ""},
		{XXX_VGROUP12, ""},
		{XXX_VGROUP13, ""}
	}
	};

	const char *p_HEstring;

	int     i,k;
	int32	vgroup_id;
	int32	vgroup_ref = -1;
	char    ObjectName[VSNAMELENMAX+1];
	int32   tag;
	int32   ref;
	int32   sds_index;
	int32   DataType;
	int32   Rank;
	int32   DimensionSizes[MAX_VAR_DIMS];
	int32   nattrs;    /* number of attributes */
	char    VgroupName[VGNAMELENMAX+1];
	int32   NumberEntries;

	ObjectName[0] = '\0';
	for (k=0; k < XXX_ARRAY_SIZE(VgroupInfo[ProductType]) && xxx_CompareObject2(p_ImageryInfo->SDSName, ProductType, ObjectName); k++)
	{

		if ((vgroup_ref = Vfind(p_ImageryInfo->FileId, VgroupInfo[ProductType][k].VgroupName)) == 0)
		{
			continue;
		}

		if ((vgroup_id = Vattach(p_ImageryInfo->FileId, vgroup_ref, "r")) == FAIL)
		{
			(void)strcpy(p_ErrorMessage, "Error attaching to HDFDirectory file: ");
			p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
			(void)strcat(p_ErrorMessage, p_HEstring);
			goto ERROR_PROC;
		}

		if (Vinquire(vgroup_id, &NumberEntries, VgroupName) == FAIL)
		{
			(void)strcpy(p_ErrorMessage, "Error inquiring HDFDirectory file: ");
			p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
			(void)strcat(p_ErrorMessage, p_HEstring);
			goto ERROR_PROC;
		}

		for (i=0; i < NumberEntries && xxx_CompareObject2(p_ImageryInfo->SDSName, ProductType, ObjectName); i++)
		{
			if (Vgettagref(vgroup_id, i, &tag, &ref) == FAIL)
			{
				(void)strcpy(p_ErrorMessage, "Error get tag/ref in HDFDirectory file: ");
				p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
				(void)strcat(p_ErrorMessage, p_HEstring);
				goto ERROR_PROC;
			}

			if (tag == DFTAG_SD || tag == DFTAG_NDG)
			{
				if ((sds_index = SDreftoindex(p_ImageryInfo->SDId, ref)) == FAIL)
				{
					(void)strcpy(p_ErrorMessage, "Can't convert ref to index in HDFDirectory file: ");
					p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
					(void)strcat(p_ErrorMessage, p_HEstring);
					goto ERROR_PROC;
				}

				if ((p_ImageryInfo->SDS_Id = SDselect(p_ImageryInfo->SDId, sds_index)) == FAIL)
				{
					(void)strcpy(p_ErrorMessage, "SDselect failed in HDFDirectory file: ");
					p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
					(void)strcat(p_ErrorMessage, p_HEstring);
					goto ERROR_PROC;
				}

				if (SDgetinfo(p_ImageryInfo->SDS_Id, ObjectName, &Rank, DimensionSizes, &DataType,
				    &nattrs) ==
				    FAIL)
				{
					(void)strcpy(p_ErrorMessage, "Can't get file information: ");
					p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
					(void)strcat(p_ErrorMessage, p_HEstring);
					(void)SDendaccess(p_ImageryInfo->SDS_Id);
					goto ERROR_PROC;
				}

				if (xxx_CompareObject2(p_ImageryInfo->SDSName, ProductType, ObjectName))
				{
					if (SDendaccess(p_ImageryInfo->SDS_Id) == FAIL)
					{
						(void)strcpy(p_ErrorMessage, "Can't end access: ");
						p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
						(void)strcat(p_ErrorMessage, p_HEstring);
						goto ERROR_PROC;
					}
				}
			}
		}

		Vdetach(vgroup_id);
	}

	if (xxx_CompareObject2(p_ImageryInfo->SDSName, ProductType, ObjectName))
	{
		(void)sprintf(p_ErrorMessage, "Object not found in any vgroup: %s", p_ImageryInfo->SDSName);
		goto ERROR_PROC;
	}

	return XXXSUCCESS;

ERROR_PROC:
	Vdetach(vgroup_id);
	return (ERROR);
}
