/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:
	xxx_CloseImagery.c

PURPOSE:
	Close the HDF SDS file.

INVOCATION METHOD:
	status = xxx_CloseImagery(p_ImageryInfo, p_ErrorMessage)

RETURN VALUE:
Type = int

Value         	Description
--------------	----------------------------------------------------------------
0		Success
non-zero	Error status returned by HDF

ARGUMENT LIST:
Name            Type               	Use 	Description
--------------- ------------------ 	--- 	---------------------------------------
p_ImageryInfo	xxx_ImageryInfo_TYPE *	I	HDF imagery information
p_ErrorMessage	char *			O	Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                            Use     Description
-------------------------       ---     -------------------------------------------------
HDF file                        I       HDF file

EXTERNAL VARIABLES:
Source          Name            Type  Use Description
--------------- -------------   ----- --- -----------------------
hdf.h           FAIL            int   I   HDF failure status
herr.h          error_top       int32 I   Top to HDF error stack
xxx_Types.h     ERROR           int   I   Error creating time stamp.
xxx_Types.h     XXXSUCCESS      int   I   Indicates successful completion

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name         		Description
----------------	-------------------------------------------------
SDend			Terminate access to the SD interface and close the file
SDendaccess		Detach from SDS name
Hclose                  Close HDF file
HEstring                Returns error message for the error number
HEvalue                 Returns the current error number
Vend                    Close the vgroup interface

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
Close error

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               N/A             R1.0    03/19/97        Original Implementation

NOTES:
See HDF Users Guide for detailed field/value setting.

Unit xxx_CloseImagery PDL:

        Initialize status to XXXSUCCESS

	Call SDendaccess() to detach from SDS name
1	IF error detaching THEN
	        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
	        Free memory
	        RETURN ERROR
2	ENDIF

3       IF this is an HDF directory file THEN
                Call Vend() to close the vdata interface
                Call Hclose() to close imagery file
4               IF error closing HDF file THEN
                        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                        Set status to ERROR
5               ENDIF
6       ENDIF

	Call SDend() to terminate access to the SD interface and close the file
7	IF error closing SD file THEN
		Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                Set status to ERROR
8	ENDIF

	RETURN status

End Unit xxx_CloseImagery
******************************************************************************/

/* System headers */
#include <stdio.h>

/* Local headers */
#include <hdf.h>
#include <mfhdf.h>
#include "xxx_Imagery.h"
#include "xxx_Types.h"

extern int32 error_top;

int xxx_CloseImagery(xxx_ImageryInfo_TYPE *p_ImageryInfo, char *p_ErrorMessage)
{
        const char *p_HEstring;
	int status = XXXSUCCESS;

	if (SDendaccess(p_ImageryInfo->SDS_Id) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Can't end access: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
                status = ERROR;
	}

	if (p_ImageryInfo->EDC_DAAC_HDF)
	{
		Vend(p_ImageryInfo->FileId);
                if (Hclose(p_ImageryInfo->FileId) == FAIL)
                {
                        (void)strcpy(p_ErrorMessage, "Can't close imagery file: ");
                        p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
                        (void)strcat(p_ErrorMessage, p_HEstring);
                        status = ERROR;
                }
	}

        if (SDend(p_ImageryInfo->SDId) == FAIL)
        {
                (void)strcpy(p_ErrorMessage, "Can't close SD file: ");
                p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
                (void)strcat(p_ErrorMessage, p_HEstring);
                status = ERROR;
        }

        return(status);
}
