/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_CompareObject.c

PURPOSE:  Check to see if the strings match or are a subset with L0R product type 
	    option set.

INVOCATION METHOD:
	status = xxx_CompareObject(p_ObjectName, p_CurrentObject);

RETURN VALUE:
Type = int

Value           Description
--------------  ----------------------------------------------------------------
0	        Strings match
1		String does not a match

ARGUMENT LIST:
Name             Type           Use	Description
---------------- -------------- ---	---------------------------------------
p_ObjectName	char *		I	Object vdata name/SDS name you want to attach to
p_CurrentObject	char *		I	Current object vdata name/SDS name

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source	               	Name		       	Type		Use	Description
---------------	        ----------------      	---------------	---	-----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name            	Description
--------------- 	-------------------------------------------------
xxx_CompareObject2	Compare object names

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               IASI990538      R3.2    11/23/98        Original Implementation

NOTES:
none

Unit xxx_CompareObject PDL:

1	Call xxx_CompareObject2() with p_ObjectName and p_CurrentObject with the L0R product type option set

End Unit xxx_CompareObject
******************************************************************************/

/* System headers */
#include <string.h>

/* Local headers */
#include "xxx_HDFDirectory.h"
#include "xxx_Types.h"

int xxx_CompareObject(char *p_ObjectName, char *p_CurrentObject)
{
	return (xxx_CompareObject2(p_ObjectName, XXX_L0R_PRODUCT, p_CurrentObject));
}

/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_CompareObject.c

PURPOSE:  Check to see if the strings match or are a subset.

INVOCATION METHOD:
	status = xxx_CompareObject2(p_ObjectName, ProductType, p_CurrentObject);

RETURN VALUE:
Type = int

Value           Description
--------------  ----------------------------------------------------------------
0	        Strings match
1		String does not a match

ARGUMENT LIST:
Name             Type           Use	Description
---------------- -------------- ---	---------------------------------------
p_ObjectName	char *		I	Object vdata name/SDS name you want to attach to
ProductType	int		I	Indicates L0R or L1 product type
p_CurrentObject	char *		I	Current object vdata name/SDS name

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source	               	Name		       	Type		Use	Description
---------------	        ----------------      	---------------	---	-----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name            Description
--------------- -------------------------------------------------
none

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
none

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               N/A             R1.0    04/16/97        Original Implementation
J. kerich               IAS970051       R3.0    03/04/98        Add XXX_BAND8 for work order files
J. kerich               IAS980093       R3.1    07/22/98        Replace XXX_CAL_BAND8 with XXX_CAL_BAND81, 
                                                                XXX_CAL_BAND82, XXX_CAL_BAND83
J. kerich		IASI990538     	R3.2	11/23/98	Add ProductType flag and rename unit

NOTES:
none

Unit xxx_CompareObject2 PDL:

1       IF p_ObjectName and p_CurrentObject match THEN
	        RETURN 0
2	ENDIF

3	IF p_ObjectName is a subset2 ProductType of p_CurrentObject THEN
      	        RETURN 0
4     	ENDIF

5       IF p_ObjectName is a subset of p_CurrentObject THEN
      	        RETURN 0
6      	ENDIF

	RETURN 1

End Unit xxx_CompareObject2
******************************************************************************/

/* Macros */
#define XXX_SUBSET   40
#define XXX_SUBSET2  6
#define XXX_FORMAT(format) ((format)  == 0 ? '1' : '2')

int xxx_CompareObject2(char *p_ObjectName, int ProductType, char *p_CurrentObject)
{
	char Subsets[XXX_SUBSET][8] = {
		XXX_BAND1, 
		XXX_BAND2, 
		XXX_BAND3,
		XXX_BAND4,
		XXX_BAND5,
		XXX_WO_BAND6L,
		XXX_WO_BAND6H,
		XXX_BAND7,
		XXX_BAND8,
		XXX_BAND81,
		XXX_BAND82,
		XXX_BAND83,
		XXX_CAL_BAND1, 
		XXX_CAL_BAND2, 
		XXX_CAL_BAND3,
		XXX_CAL_BAND4,
		XXX_CAL_BAND5,
		XXX_WO_CAL_BAND6L,
		XXX_WO_CAL_BAND6H,
		XXX_CAL_BAND7,
		XXX_CAL_BAND8,
		XXX_CAL_BAND81,
		XXX_CAL_BAND82,
		XXX_CAL_BAND83,
		XXX_SLO_BAND1, 
		XXX_SLO_BAND2, 
		XXX_SLO_BAND3,
		XXX_SLO_BAND4,
		XXX_SLO_BAND5,
		XXX_SLO_BAND7,
		XXX_SLO_BAND8,
		XXX_SLO_BAND81,
		XXX_SLO_BAND82,
		XXX_SLO_BAND83,
		XXX_GEO,
		XXX_MSCD,
		XXX_MTL,
		XXX_PCD,
		XXX_MTP,
		XXX_CPF
	};

	typedef struct xxx_SubSet2Info_TYPE {
		char Subset[8];
		char ObjectExtension[8];
	} xxx_SubSet2Info_TYPE;

	xxx_SubSet2Info_TYPE Subsets2[2][XXX_SUBSET2] = {
		{{XXX_BAND6L,     ".B60"},
		 {XXX_CAL_BAND6L, ".C60"},
		 {XXX_SLO_BAND6L, ".O60"},
		 {XXX_MT1,        ".MTA"},
		 {XXX_MSCD1,      ".MSD"},
		 {XXX_PCD1,       ".PCD"}
		},
		{{XXX_BAND6H,     ".B60"},
		 {XXX_CAL_BAND6H, ".C60"},
		 {XXX_SLO_BAND6H, ".O60"},
		 {XXX_MT2,        ".MTA"},
		 {XXX_MSCD2,      ".MSD"},
		 {XXX_PCD2,       ".PCD"}
		}
	};

	int FormatOffset[2] = {6, 2};
	int i,k;

	if (!strcmp(p_ObjectName, p_CurrentObject))
	{
		return (0);
	}


	for (k=0; k < 2; k++)
	{
		for (i=0; i < XXX_SUBSET2 && strcmp(p_ObjectName, Subsets2[k][i].Subset); i++);
		if (i != XXX_SUBSET2  && 
		    strstr(p_CurrentObject, Subsets2[k][i].ObjectExtension) && 
		    p_CurrentObject[FormatOffset[ProductType]] == XXX_FORMAT(k))
		{
			return (0);
		}
	}

	for (i=0; i < XXX_SUBSET && strcmp(p_ObjectName, Subsets[i]); i++);
	if (i != XXX_SUBSET && strstr(p_CurrentObject, p_ObjectName))
	{
		return (0);
	}

	return (1);
}
