/******************************************************************************
*		Landsat 7 Image Assessment System (IAS) Software
*			Property of the U.S. Government
*			    NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME: xxx_ConvertString.c

PURPOSE: To convert the string to the new type using the memory address given by
         caller

INVOCATION METHOD:
    status = xxx_ConvertString(p_destination, parm_type, kvalue, p_ErrorMessage)

RETURN VALUE:
    Type = int

Value	         		Description
---------------  		----------------------------------------------------------------
XXXSUCCESS	 		Converted the String in to requested type
ERROR			 	Allocation error or unknown parm_type
XXX_E_ODL_INVALID_DATA_TYPE	Invalid data type

ARGUMENT LIST:
Name 	    	  Type              Use       Description
---------------- ------------------ --- ---------------------------------------
p_destination    void *              O        Location where to copy converted values
parm_type        xxx_ValueType_TYPE  I        What type the p_destination is, enum field
kvalue		 char *              I        String to be converted 
p_ErrorMessage   char *              O        Error Message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name 		 	  Use Description
------------------------- --- -------------------------------------------------
none

EXTERNAL VARIABLES:
Source            Name                Type            Use Description
---------------   ------------------  --------------- --- -----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                          Description
----------------------------- -------------------------------------------------
none

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
- Convertion errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
 none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
T. Raheja         N/A         R1.0    03/97      Original Implementation
J. Kerich         IAS970011   R2.0    11/12/97   Add XXX_Long for 64 bit compile
J. kerich         IASI990549  R3.2    02/18/99   Use '\0' instead of NULL to check for string end

NOTES:
  If call to strtol or strtod failed, return XXX_E_ODL_INVALID_DATA_TYPE.

Unit xxx_ConvertString PDL:

    DOCASE parm_type
1       CASE XXX_Long:
            Convert kvalue to long and store at p_destination

2       CASE XXX_Int:
            Convert kvalue to int and store at p_destination

3       CASE XXX_Float
            Convert kvalue to float and store at p_destination

4       CASE XXX_Double
            Convert kvalue to double and store at p_destination

5       CASE XXX_ArrayOfString
            Allocate memory for length+1 of kvalue
6           IF allocation error THEN
                Build error message in p_ErrorMessage
                RETURN ERROR
7           ENDIF
            Copy kvalue to p_destination

8       CASE XXX_Sci_Not
            Convert kvalue to double and store at p_destination

9       CASE default
            Build error message in p_ErrorMessage
            RETURN ERROR
    ENDDO

    RETURN XXXSUCCESS

End Unit xxx_ConvertString PDL
******************************************************************************/

/* System Headers */
#include <stdio.h>
#include <string.h>
#include <lablib3.h>                 /* prototypes for the ODL functions */
/* Third party control */
#include "xxx_ODL.h"                 /* prototype for the xxx ODL functions */

int xxx_ConvertString( void *p_destination, xxx_ValType_TYPE parm_type, char *kvalue, char *p_ErrorMessage)
{
	char *p_endptr;
	char *p_a;

	switch(parm_type)
	{
	case XXX_Long:
		(*(long *)p_destination) = strtol(kvalue,&p_endptr,10);
		if (*p_endptr != '\0')
		{
                    return XXX_E_ODL_INVALID_DATA_TYPE;
		}
		break;

	case XXX_Int:
		(*(int *)p_destination) = (int)strtol(kvalue,&p_endptr,10);
		if (*p_endptr != '\0')
		{
                    return XXX_E_ODL_INVALID_DATA_TYPE;
		}
		break;

	case XXX_Float:
		(*(float *)p_destination) = (float)strtod(kvalue,&p_endptr);
		if (*p_endptr != '\0')
		{
		    return XXX_E_ODL_INVALID_DATA_TYPE;
                }
		break;

	case XXX_Sci_Not:
	case XXX_Double:
		(*(double *)p_destination) = strtod(kvalue,&p_endptr);
		if (*p_endptr != '\0')
	        {
		    return XXX_E_ODL_INVALID_DATA_TYPE;
	        }
	        break;

	case XXX_ArrayOfString:
		p_a = malloc (strlen(kvalue)+1);
		if (p_a == '\0')
		{
			(void)strcpy(p_ErrorMessage, "Error : Allocating Memory ... ");
			return (ERROR);
		}
		(void)strcpy(p_a,kvalue);
		(*(char **)p_destination) = p_a;
		break;

	default:
		(void)strcpy(p_ErrorMessage, "Invalid Conversion type ");
		(void)strcat(p_ErrorMessage, kvalue);
		(void)strcat(p_ErrorMessage, " ...");
		return (ERROR);
	}

	return (XXXSUCCESS);
}
