/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME: xxx_GetODLField.c

PURPOSE: Get the requested ODL field and convert it, if necessary.

INVOCATION METHOD:
    status = xxx_GetODLField(p_MemoryAddr, MemorySize, ValueType, p_ODLTree, 
             p_ClassName, p_LabelName, p_ErrorMessage,p_Count)

RETURN VALUE:
    Type = int

Value                                   Description
--------------                          ---------------------------------------------------------------
XXXSUCCESS                              Found and converted field(s) into requested type
XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED        Not enough memory passed in
XXX_E_ODL_NOT_FOUND		        Group/label not found
XXX_E_ODL_INVALID_DATA_TYPE             Data type mismatch
ERROR                                   Fatal error                       

ARGUMENT LIST:
Name            Type                 Use Description
--------------- -------------------  --- ---------------------------------------
p_MemoryAddr    void *               I   p_Memory to copy fields into
MemorySize      int                  I   Size in bytes of p_Memory
ValueType       xxx_ValueType_TYPE   I   What type the P_FieldName is, enum field
p_ODLTree       OBJDESC *            I   ODL tree
p_ClassName     char *               I   Group/Object name, optional
p_LabelName     char *               I   Field to retrieve
p_ErrorMessage  char *               O   Error Message
p_Count         int *                O   Count the number of values in a array

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use Description
------------------------- --- -------------------------------------------------
xxx_Errno.c               O   return error messages

EXTERNAL VARIABLES:
Source             Name               Type            Use Description
---------------    ------------------ --------------- --- -----------------------
OdlPrintMessage.c  ODLErrorMessage    Char *          I   ODL error message
toolbox.h          TB_MAXLINE         int             I   Maximum keyword string length

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                    Description
----------------------- -------------------------------------------------
OdlFindObjDesc          Find field in the ODL tree
OdlGetKwdValue          Get the label value
OdlGetKwdValueType      What type is the label value
OdlFindKwd              To find the label in the class
xxx_ConvertString       Convert string to "real" value 

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
- ODL errors
- Convertion errors
- memory errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
- ODL file is opened
- Can't have embedded Quotes and Commas in the ODL file

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
T.Raheja                      R1.0    03/97      Original Implementation
J. Kerich                     R1.0    05/23/97   Fixed string size logic error
J. Kerich         IASI970057  R2.0    08/18/97   Add sequence logic, fix multi-line logic 
                                                 and find group logic
						 Add XXX_Long for 64 bit compile
J. kerich         IASI980424  R3.1    08/13/98   Fix lint errors
J. kerich         IASI990549  R3.2    02/18/99   Add p_MemoryAddr to prologue

NOTES:
- Caller must pass in a pointer to a memory location large enough to contain the converted
  field value(s).  The unit will then convert the ascii values to the specified value types.
- if the caller is expecting an array of strings, the the p_Memory array is an array of char *.
  The calles will have to free the malloc memory.
- if any of the ODL calls fails the system will build the error in the p_ErrorMessage, free memory 
  (if any), and return error to the calling function.

Unit xxx_GetODLField PDL:
     
     Check for p_LabelName
1    IF p_LabelName is Null THEN
	Build error message in p_ErrorMessage
	RETURN ERROR 
2    ENDIF

     Call OdlFindObjDesc() to find field using the p_ODLTree, p_ClassName and p_LabelName
     Call OdlFindKwd() to find the label in class
     Call OdlGetKwdValue() to get the field value

     Allocate memory for keyword

3    IF OldGetKwdValueType() is ODL_SET THEN 
4        DOFOR all values in label
             Get a token
5	     DOUNTIL token string is not NULL
6  	         IF MemorySize overflow THEN
                     Build error message in p_ErrorMessage
	             RETURN ERROR
7                ENDIF
                 Call xxx_ConvertString() to convert string to new type using p_MemoryAddr, ValueType, and p_ErrorMessage
8                IF converting error THEN
9                    IF parm_type is array of string THEN
                         Free all string memory
10                   ENDIF
                     RETURN ERROR
11               ENDIF
                 Increment p_MemoryAddr by size of Array Element
                 Increment p_Count
12           ENDDO
13       ENDDO

14    ELSE IF OldGetKwdValueType() is ODL_SEQUENCE THEN 
15       DOFOR all values in label
             Get a token
16	     DOUNTIL token string is not NULL
17 	         IF MemorySize overflow THEN
                     Build error message in p_ErrorMessage
	             RETURN ERROR
18               ENDIF
                 Call xxx_ConvertString() to convert string to new type using p_MemoryAddr, ValueType, and p_ErrorMessage
19               IF converting error THEN
                     Free all string memory
                     RETURN ERROR
20               ENDIF
                 Increment p_MemoryAddr by size of Array Element
                 Increment p_Count
21           ENDDO
22       ENDDO

23   ELSE
24       IF ValueType is a String THEN
25           IF MemorySize overflow THEN
                 Build error message in p_ErrorMessage
                 RETURN ERROR
26	     ENDIF
             Copy kvalue to p_MemoryAddr
27       ELSE
	     Check for MemorySize 
28	     IF MemorySize overflow THEN
                 Build error message in p_ErrorMessage
                 RETURN ERROR
29	     ENDIF
             Call xxx_ConvertString() to convert string to new type using p_MemoryAddr, ValueType, and p_ErrorMessage
30           IF converting error THEN
                RETURN ERROR
31           ENDIF
         ENDIF
         Increment p_Count
     ENDIF

     RETURN XXXSUCCESS

End Unit xxx_GetODLField
******************************************************************************/

/* System Headers */
#include <stdio.h>
#include <string.h>
#include "xxx_ODL.h"                 /* prototype for the xxx ODL functions */
#include <toolbox.h>

extern char ODLErrorMessage[];       /* External Variables */

int xxx_GetODLField( void *p_MemoryAddr, int MemorySize, xxx_ValType_TYPE ValueType, OBJDESC *p_ODLTree, char *p_ClassName,
char *p_LabelName, char *p_ErrorMessage, int *p_Count)
{

    OBJDESC *p_lp;         /* Object Descriptor */
    KEYWORD *p_kw;         /* Keyword Name */
    char *p_kwv;           /* Keyword Value */
    char *p_keyword;       /* Copy of the Keyword Value */
    int i;
    char *p_word;
    char temp_str[XXXCMLEN];     /* temporary string */
    char string[TB_MAXLINE];
    int ret_code = 0;
    int ignore_field = FALSE;

    *p_Count = 0;

    /* p_ClassName may be NULL coming in.  If so, then the sprintfs
       core dump when printing p_ClassName.  If p_ClassName is NULL,
       then set a temporary string to "".
       -------------------------------------------------------------*/
    if (p_ClassName == NULL)
        strcpy (temp_str, "");
    else
        strcpy (temp_str, p_ClassName);

    if ( (p_LabelName == NULL) || (strlen(p_LabelName) == 0) )
    {
        (void)strcpy(p_ErrorMessage,"Error : Invalid Keyword");
        return ERROR;
    }

    /* Don't error if can't find PROCESSING_SOFTWARE. This field did not exist in older metadata files */
    if ((p_lp = OdlFindObjDesc(p_ODLTree, p_ClassName, p_LabelName, NULL, 1, ODL_RECURSIVE_DOWN)) == NULL)
    {
        if ((strcasecmp (p_LabelName, "PROCESSING_SOFTWARE")) != NULL)
        {
            (void)strcpy(p_ErrorMessage, ODLErrorMessage);
            if ((long)strlen(p_ErrorMessage) <= 1 )
            {
                (void)sprintf(p_ErrorMessage,"Error : %s Object not found with %s keyword", temp_str, p_LabelName);
            }
            return XXX_E_ODL_NOT_FOUND;
        }
        else
            ignore_field = TRUE;
    }

    if (!ignore_field)
    {
        if ((p_kw = OdlFindKwd(p_lp, p_LabelName, NULL, 1 ,ODL_RECURSIVE_DOWN)) == NULL)
        {
            (void)strcpy(p_ErrorMessage, ODLErrorMessage);
            if ((long)strlen(ODLErrorMessage) <= 1 )
            {
                (void)sprintf(p_ErrorMessage," Error : %s Keyword not found",p_LabelName);
            }
            return XXX_E_ODL_NOT_FOUND;
        }

        if ((p_kwv = OdlGetKwdValue(p_kw)) == NULL)
        {
            (void)strcpy(p_ErrorMessage, ODLErrorMessage);
            if ((long)strlen(p_ErrorMessage) <= 1 )
            {
                (void)sprintf(p_ErrorMessage," Error : %s Keyword not found",p_LabelName);
            }
            return XXX_E_ODL_NOT_FOUND;
        }
        if ((p_keyword= malloc(strlen(p_kwv)+1)) == NULL)
        {
            (void)strcpy(p_ErrorMessage,"Malloc error");
            return (ERROR);
        }
        (void)strcpy(p_keyword,p_kwv);

        if (OdlGetKwdValueType (p_kw) == ODL_SET)
        {
            p_word = strtok(p_keyword,"(,) \"\n");
            while(p_word != NULL)
            {
                if (strlen(p_word))
                {
                    switch (ValueType)
                    {
                        case XXX_Long :
                            MemorySize -= sizeof(long);
                            if (MemorySize < 0 )
                            {
                                (void)strcpy(p_ErrorMessage, " Input ODL value overflows allocated space ");
                                free(p_keyword);
                                return XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED;
                            }
                            if ( ( ret_code = xxx_ConvertString(p_MemoryAddr,ValueType,p_word, p_ErrorMessage) ) !=
                                    XXXSUCCESS )
                            {
                                (void)sprintf(p_ErrorMessage,"Error : Converting %s keyword's value %s",p_LabelName,
                                              p_word);
                                free(p_keyword);
                                return ret_code;
                            }
                            p_MemoryAddr = (long *)p_MemoryAddr + 1;
                            break;

                        case XXX_Int :
                            MemorySize -= sizeof(int);
                            if (MemorySize < 0 )
                            {
                                (void)strcpy(p_ErrorMessage, " Input ODL value overflows allocated space ");
                                free(p_keyword);
                                return XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED;
                            }
                            if ( ( ret_code = xxx_ConvertString(p_MemoryAddr,ValueType,p_word, p_ErrorMessage) ) !=
                                    XXXSUCCESS )
                            {
                                (void)sprintf(p_ErrorMessage,"Error : Converting %s keyword's value %s",p_LabelName,
                                              p_word);
                                free(p_keyword);
                                return ret_code;
                            }
                            p_MemoryAddr = (int *)p_MemoryAddr + 1;
                            break;

                        case XXX_Float :
                            MemorySize -= sizeof(float);
                            if (MemorySize < 0 )
                            {
                                (void)strcpy(p_ErrorMessage, " Input ODL value overflows allocated space ");
                                free(p_keyword);
                                return XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED;
                            }
                            if ( ( ret_code = xxx_ConvertString(p_MemoryAddr,ValueType,p_word, p_ErrorMessage) ) !=
                                    XXXSUCCESS )
                            {
                                (void)sprintf(p_ErrorMessage,"Error : Converting %s keyword's value %s",p_LabelName,
                                              p_word);
                                free(p_keyword);
                                return ret_code;
                            }
                            p_MemoryAddr = (float *)p_MemoryAddr + 1;
                            break;

                        case XXX_Double :
                        case XXX_Sci_Not :
                            MemorySize -= sizeof(double);
                            if (MemorySize < 0 )
                            {
                                (void)strcpy(p_ErrorMessage, " Input ODL value overflows allocated space ");
                                free(p_keyword);
                                return XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED;
                            }
                            if ( ( ret_code = xxx_ConvertString(p_MemoryAddr,ValueType,p_word, p_ErrorMessage) ) !=
                                    XXXSUCCESS )
                            {
                                (void)sprintf(p_ErrorMessage,"Error : Converting %s keyword's value %s",p_LabelName,
                                              p_word);
                                free(p_keyword);
                                return ret_code;
                            }
                            p_MemoryAddr = (double *)p_MemoryAddr + 1;
                            break;

                        case XXX_ArrayOfString :
                            MemorySize -= sizeof(char *);
                            if (MemorySize < 0 )
                            {
                                (void)strcpy(p_ErrorMessage, " Input ODL value overflows allocated space ");
                                free(p_keyword);
                                return  XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED;
                            }
                            if ( ( ret_code = xxx_ConvertString(p_MemoryAddr,ValueType,p_word, p_ErrorMessage) ) !=
                                    XXXSUCCESS )
                            {
                                for( i=0;i<*p_Count;i++)
                                    free((char *)p_MemoryAddr + i);
                                (void)sprintf(p_ErrorMessage,"Error : Converting %s keyword's value %s",p_LabelName,
                                              p_word);
                                free(p_keyword);
                                return ret_code;
                            }
                            p_MemoryAddr = (char *)p_MemoryAddr + sizeof(char *);
                            break;

                        default:
                            (void)strncpy( p_MemoryAddr, p_word, MemorySize);
                            (void)sprintf(p_ErrorMessage,"Type XXX_String is not valid for arrays of strings");
                            free(p_keyword);
                            return ERROR;
                    }
                    *p_Count += 1;
                    p_word = strtok(NULL,",() \"\n");
                }
            }
        }

        else if (OdlGetKwdValueType (p_kw) == ODL_SEQUENCE)
        {
            p_word = strtok(p_keyword,"{,} \"\n");
            while(p_word != NULL)
            {
                if (strlen(p_word))
                {
                    switch (ValueType)
                    {
                        case XXX_ArrayOfString :
                            MemorySize -= sizeof(char *);
                            if (MemorySize < 0 )
                            {
                                (void)strcpy(p_ErrorMessage, " Input ODL value overflows allocated space ");
                                free(p_keyword);
                                return  XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED;
                            }
                            if ( ( ret_code = xxx_ConvertString(p_MemoryAddr,ValueType,p_word, p_ErrorMessage) ) !=
                                    XXXSUCCESS )
                            {
                                for( i=0;i<*p_Count;i++)
                                    free((char *)p_MemoryAddr + i);
                                (void)sprintf(p_ErrorMessage,"Error : Converting %s keyword's value %s", p_LabelName,
                                              p_word);
                                free(p_keyword);
                                return ret_code;
                            }
                            p_MemoryAddr = (char *)p_MemoryAddr + sizeof(char *);
                            break;

                        default:
                            (void)strncpy( p_MemoryAddr, p_word, MemorySize);
                            (void)sprintf(p_ErrorMessage,"An ODL sequence is of type XXX_ArrayOfString only");
                            free(p_keyword);
                            return ERROR;
                    }
                    *p_Count += 1;
                    p_word = strtok(NULL,",{} \"\n");
                }
            }
        }
        else
        {
            if (ValueType == XXX_String)
            {
                if (p_kwv[0] == '\"')
                {
                    (void)strcpy(string,  &p_kwv[1]);
                    string[strlen(string)-1] = '\0';
                }
                else /* date */
                {
                    (void)strcpy(string, p_kwv);
                }

                if ((int)(MemorySize - (strlen(string) + 1)) < 0 )
                {
                    (void)strcpy(p_ErrorMessage, " Input ODL value overflows allocated space ");
                    (void)strncpy(p_MemoryAddr, string, MemorySize);
                    ((char *)p_MemoryAddr)[MemorySize-1] = '\0';
                    free(p_keyword);
                    return XXX_E_NOT_ENOUGH_MEMORY_SUPPLIED;
                }
                else
                {
                    (void)strcpy(p_MemoryAddr, string);
                }
            }

            else
            {
                switch(ValueType)
                {
                    case XXX_Long:
                        MemorySize -= sizeof(long);
                        break;

                    case XXX_Int:
                        MemorySize -= sizeof(int);
                        break;

                    case XXX_Float:
                        MemorySize -= sizeof(float);
                        break;

                    case XXX_Double:
                    case XXX_Sci_Not:
                        MemorySize -= sizeof(double);
                        break;

                    default:
                        (void)sprintf(p_ErrorMessage,"Invalid Type specified");
                        free(p_keyword);
                        return ERROR;
                }

                if (MemorySize < 0 )
                {
                    (void)strcpy(p_ErrorMessage,"Input ODL value overflows allocated space ");
                    free(p_keyword);
                    return ERROR;
                }

                if ( ( ret_code = xxx_ConvertString(p_MemoryAddr,ValueType,p_kwv, p_ErrorMessage) ) != XXXSUCCESS )
                {
                    (void)sprintf(p_ErrorMessage,"Error : Converting %s keyword's value %s",p_LabelName,p_kwv);
                    free(p_keyword);
                    return ret_code;
                }
            }
            *p_Count += 1;
        }
        free(p_keyword);
    }
    return XXXSUCCESS;
}
