/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_OpenImagery.c

PURPOSE:  Open/create the SDS HDF file and create and/or and attach to the requested
          SDS name.  If external file option is choosen the unit will set up for an external
          write.  This unit will only work on one SDS name which has one field.


INVOCATION METHOD:
	Status = xxx_OpenImagery(p_PathName, AccessMode, p_SDSName, offset, p_ExternalFile, 
	                         p_ImageryInfo, p_ErrorMessage)

RETURN VALUE:
Type = int

Value         	Description
--------------	----------------------------------------------------------------
XXXSUCCESS	Wrote data to file
ERROR		Failed to write data to file

ARGUMENT LIST:
Name            Type              	Use	Description
--------------- ------------------	---	---------------------------------------
p_PathName	char *			I	Path name to SDS HDF file
AccessMode	int32			I	HDF access mode
p_SDSName	char *			I	SDS name to attach to or create
offset          int32                   I       Where to start writing in external file
p_ExternalFile  char *                  I       External file path
p_ImageryInfo	xxx_ImageryInfo_TYPE *	I/O	HDF imagery information
p_ErrorMessage	char *			O	Error message 

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use     Description
------------------------- ---     -------------------------------------------------
HDF file                  I       HDF file

EXTERNAL VARIABLES:
Source          Name                  Type  Use Description
--------------- -------------         ----- --- -----------------------
hdf.h           FAIL                  int   I   HDF failure status
herr.h          error_top             int32 I   Top to HDF error stack
xxx_Types.h     ERROR                 int   I   Error creating time stamp.
xxx_Types.h     XXXSUCCESS            int   I   Indicates successful completion
xxx_Types.h     TRUE                  int   I   Indicates true
xxx_Types.h     FALSE                 int   I   Indicates false
xxx_Types.h     XXX_HDF_DIR_EXTENTION int   I   HDF directory file extension
xxx_Imagery.h   XXX_MAX_RANK          int   I   Maximum rank allowed

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name           		Description
--------------- 	-------------------------------------------------
HEclear                 Clear the HDF error stack
HEstring        	Returns error message for the error number
HEvalue         	Returns the current error number
Hclose                  Close HDF file
Hopen                   Open HDF file
HXsetdir                Sets the external directory path
SDnametoindex           Get the index to the SDS name
SDgetinfo               Get information on data set
SDselect                Select the SDS name
SDstart                 Open/create HDF file
Vend                    Close the vgroup interface
Vfind                   Find the Vgroup
Vstart                  Initialize the vgroup interface
SDstart			Open/create HDF file
xxx_ChkForSDS           Find the SDS name in HDF file
xxx_ChkForSDSInHDF      Find the SDS name in HDF directory file
xxx_CloseImagery        Close the HDF file
xxx_FindDirectory       Get the directory path from the path name

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
HDF errors

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. Kerich			R1.0  02/18/97   Original Implementation
J. Kerich	  IASI970058    R2.0  09/30/97	 Save directory path in  DirectoryPath
J. Kerich	  IASI970070    R2.0  10/28/97	 Add path to open error message
J. Kerich	  IASI980294    R3.0  01/15/98	 Add getinfo logic for existing SDS names 
                                                 on none RDONLY options
J. Kerich	  IAS970051     R3.0  02/19/98	 Check to see if the HDF directory file has any 
                                                 vgroups in it.  If not it is not a real L0R HDF 
                                                 directory file
J. Kerich	  IASI990538    R3.2  11/23/98	 Add L1 logic

NOTES:
- See HDF Users Guide for detailed field/value setting.
- If external write is not wanted then set the p_ExternalFile variable to NULL.

Unit xxx_OpenImagery PDL:

        Initialize key p_ImageryInfo fields

        Call xxx_FindDirectory() to get the directory path from the path name
	Call HXsetdir() to set the external directory path
1       IF error setting the external directory path THEN
		Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
		RETURN ERROR
2       ENDIF

3       IF this is an HDF directory file THEN
		Set p_ImageryInfo's EDC_DAAC_HDF to TRUE

        	Call Hopen() to open HDFDirectory
4       	IF error opening file THEN
                	Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                	RETURN ERROR 
5       	ENDIF

        	Call Vstart() to initialize the vgroup interface

		Call Vfind() to find the product metadata in the directory
6		IF PCD Vgroup doesn't exist THEN
		        Call Vend() to close the vdata interface 
		        Call Hclose() to close imagery file
			Set p_ImageryInfo's EDC_DAAC_HDF to FALSE
7		ENDIF
8	ELSE
		Set p_ImageryInfo's EDC_DAAC_HDF to FALSE
        ENDIF

	Call SDstart() to open/create SD p_PathName
9	IF error opening/creating SD file THEN
		Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
		ABORT to PROC_ERROR
10      ENDIF

	Call HEclear() to clear the HDF error stack

	Copy p_SDSName to p_ImageryInfo
11 	IF AccessMode is DFACC_READ THEN
12		IF this is a HDF directory file THEN
                        Call xxx_ChkForSDSInHDF() to find the SDS name in either L0R or L1 product
13		        IF error THEN
		                Build error message in p_ErrorMessage
			        ABORT to PROC_ERROR
14		        ENDIF
15              ELSE
                        Call xxx_ChkForSDS() to find the SDS name in either L0R or L1 product
16		        IF error THEN
			        ABORT to PROC_ERROR
17		        ENDIF
                ENDIF

	        Call SDgetinfo() to get information on data set and store it in p_ImageryInfo
18	        IF error THEN
	                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
		        ABORT to PROC_ERROR
19	        ENDIF
20	ELSE
	        Call SDnametoindex() to get the index to the SDS name
21	        IF SDS name does not exist THEN
		        Call SDcreate() to create the the SDS name
22                      IF error THEN
                                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                                ABORT to PROC_ERROR
23                      ENDIF

24                      IF p_ExternalFile is set THEN
25                              IF offset is negitive THEN
                                       Build error message in p_ErrorMessage
                                       ABORT to PROC_ERROR                                      
26                              ENDIF

                                Call SDsetexternalfile() to create the the external file using p_ExternalFile and p_offset
27                              IF error THEN
                                        Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                                        ABORT to PROC_ERROR
28                              ENDIF
29                      ENDIF
30	        ELSE
		        Call SDselect() to select the SDS name
31                      IF error THEN
                                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                                ABORT to PROC_ERROR
32                      ENDIF

                        Call SDgetinfo() to get information on data set
33                      IF error THEN
                                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
                                ABORT to PROC_ERROR
34                      ENDIF

35                      IF Rank is not match XXX_MAX_RANK THEN
	                        Build error message in p_ErrorMessage
	                        ABORT to PROC_ERROR
36		        ENDIF
	        ENDIF
	ENDIF

	RETURN XXXSUCCESS

PROC_ERROR:
        Call xxx_CloseImagery() to close the HDF file
	RETURN ERROR

End Unit xxx_OpenImagery
******************************************************************************/

/* System headers */
#include <string.h>
#include <sys/param.h>

/* External functions */
#include <hdf.h>
#include <mfhdf.h>

/* Local headers */
#include "xxx_Types.h"
#include "xxx_HDFDirectory.h"
#include "xxx_Imagery.h"
#include "xxx_Types.h"

extern int32 error_top;

int xxx_OpenImagery(char *p_PathName, int32 AccessMode, char *p_SDSName, int32 offset, 
	char *p_ExternalFile, xxx_ImageryInfo_TYPE *p_ImageryInfo, char *p_ErrorMessage)
{
	const char *p_HEstring;
	char ErrorMessage[XXXCMLEN];
	int32 sds_index;
	int32 nattrs;    /* number of attributes */
	int32 dimension[XXX_MAX_RANK];

	p_ImageryInfo->FileId = 0;
	p_ImageryInfo->SDS_Id = 0;
	p_ImageryInfo->SDId = 0;

	xxx_FindDirectory(p_ImageryInfo->DirectoryPath, p_PathName);
	if (HXsetdir(p_ImageryInfo->DirectoryPath) == FAIL)
	{
	       	(void)strcpy(p_ErrorMessage, "Can't set external file path: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		return (ERROR);
	}

	if (strstr(p_PathName, XXX_HDF_DIR_EXTENTION))
	{
		p_ImageryInfo->EDC_DAAC_HDF = TRUE;

		if ((p_ImageryInfo->FileId = Hopen(p_PathName, AccessMode, 0)) == FAIL)
		{
			(void)sprintf(p_ErrorMessage, "Can't open HDFDirectory file %s: ", p_PathName);
			p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
			(void)strcat(p_ErrorMessage, p_HEstring);
		        return (ERROR);
		}

		Vstart(p_ImageryInfo->FileId);

		/* Check to see if this is really an HDF directory file with Vgroups */
                if (Vfind(p_ImageryInfo->FileId, XXX_VGROUP12) == 0)
		{
			Vend(p_ImageryInfo->FileId);
			(void)Hclose(p_ImageryInfo->FileId);
			p_ImageryInfo->FileId = 0;
			p_ImageryInfo->EDC_DAAC_HDF = FALSE;
		}
	}
	else
	{
		p_ImageryInfo->EDC_DAAC_HDF = FALSE;
	}

	if ((p_ImageryInfo->SDId = SDstart(p_PathName, AccessMode)) == FAIL)
	{
		(void)sprintf(p_ErrorMessage, "Can't open SD file %s: ", p_PathName);
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		goto PROC_ERROR;
	}

	HEclear();

	if (!strcmp(p_SDSName, ""))
	{
	        (void)strcpy(p_ErrorMessage, "SDSName is not set");
		goto PROC_ERROR;
	}

	(void)strcpy(p_ImageryInfo->SDSName, p_SDSName);

	if (AccessMode == DFACC_READ)
	{
		if (p_ImageryInfo->EDC_DAAC_HDF)
		{
			if (xxx_ChkForSDSInHDF(p_ImageryInfo, XXX_L0R_PRODUCT, ErrorMessage))
			{
				if (xxx_ChkForSDSInHDF(p_ImageryInfo, XXX_L1_PRODUCT, ErrorMessage))
				{
					(void)sprintf(p_ErrorMessage, "Can't read in HDFDirectory files:  %s", ErrorMessage);
					goto PROC_ERROR;
				}
			}
		}
		else
		{
			if (xxx_ChkForSDS(p_ImageryInfo, XXX_L0R_PRODUCT, p_ErrorMessage))
			{
				if (xxx_ChkForSDS(p_ImageryInfo, XXX_L1_PRODUCT, p_ErrorMessage))
				{
					goto PROC_ERROR;
				}
			}
		}

		if (SDgetinfo(p_ImageryInfo->SDS_Id, p_ImageryInfo->SDSName, &p_ImageryInfo->Rank, p_ImageryInfo->DimensionSizes,

		    &p_ImageryInfo->DataType, &nattrs) == FAIL)
		{
			(void)strcpy(p_ErrorMessage, "Can't get file information: ");
			p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
			(void)strcat(p_ErrorMessage, p_HEstring);
			goto PROC_ERROR;
		}
	}
	else
	{
		if ((sds_index = SDnametoindex(p_ImageryInfo->SDId, p_ImageryInfo->SDSName)) == FAIL)
		{
			if (HEvalue(error_top) != DFE_NONE)
			{
				(void)strcpy(p_ErrorMessage, "SDnametoindex failed: ");
				p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
				(void)strcat(p_ErrorMessage, p_HEstring);
				p_ImageryInfo->SDS_Id = FAIL;
				goto PROC_ERROR;
			}

			dimension[0] = SD_UNLIMITED;
			dimension[1] = p_ImageryInfo->DimensionSizes[1];
			p_ImageryInfo->Rank = XXX_MAX_RANK;
			if ((p_ImageryInfo->SDS_Id = SDcreate(p_ImageryInfo->SDId, p_ImageryInfo->SDSName, p_ImageryInfo->DataType,

			    p_ImageryInfo->Rank, dimension)) == FAIL)
			{
				(void)strcpy(p_ErrorMessage, "SDcreate failed: ");
				p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
				(void)strcat(p_ErrorMessage, p_HEstring);
				goto PROC_ERROR;
			}

			if (p_ExternalFile != NULL)
			{

			        if (offset < 0)
				{
				   	(void)strcpy(p_ErrorMessage, "offset must be >= 0");
					goto PROC_ERROR;
				}
				if (SDsetexternalfile(p_ImageryInfo->SDS_Id, p_ExternalFile, offset) == FAIL)
				{
                        		(void)strcpy(p_ErrorMessage, "SDsetexternalfiles failed: ");
                        		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
                        		(void)strcat(p_ErrorMessage, p_HEstring);
					goto PROC_ERROR;
				}
			}
		}
		else
		{
			if ((p_ImageryInfo->SDS_Id = SDselect(p_ImageryInfo->SDId, sds_index)) == FAIL)
			{
				(void)strcpy(p_ErrorMessage, "SDselect failed: ");
				p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
				(void)strcat(p_ErrorMessage, p_HEstring);
				goto PROC_ERROR;
			}

			if (SDgetinfo(p_ImageryInfo->SDS_Id, p_ImageryInfo->SDSName, &p_ImageryInfo->Rank, 
			        p_ImageryInfo->DimensionSizes, &p_ImageryInfo->DataType, &nattrs) == FAIL)
			{
				(void)strcpy(p_ErrorMessage, "Can't get file information: ");
				p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
				(void)strcat(p_ErrorMessage, p_HEstring);
				goto PROC_ERROR;
			}

			if (XXX_MAX_RANK != p_ImageryInfo->Rank)
			{
				(void)sprintf(p_ErrorMessage, "Rank missmatch, Input rank was %d while file rank was %d",
				    p_ImageryInfo->Rank, XXX_MAX_RANK);
				goto PROC_ERROR;
			}
		}
	}

	return(XXXSUCCESS);

PROC_ERROR:
        (void)xxx_CloseImagery(p_ImageryInfo, ErrorMessage);
	return (ERROR);
}
