/******************************************************************************
*                              PACOR II SOFTWARE
*                      Property of the U.S. Government
*                              NASA/GSFC/Code 560
*******************************************************************************
*
*    UNIT NAME:
*	xxx_OpenMap.c
*
*
*    PURPOSE:
*	To open a file, get its length, and optionally memory map the file.
*
*
*    INVOCATION METHOD:
*	status = xxx_OpenMap(p_file_name, p_fd, file_options, p_pa, file_start, 
*		p_file_len, p_remainder, mapflag, lockflag, p_ErrorMessage)
*
*
*    RETURNED VALUE:
*
*       TYPE = int
*
*       VALUE			DESCRIPTION
*       ------------		----------------------------
*	XXX_OPEN_ERR		File open error
*	XXX_STAT_ERR		File status error
*	XXX_MMAP_ERR		Memory map error
*	XXX_MMAP_ZERO_ERR	Indicates zero file size for memory map error
*       XXXSUCCESS		Terminated successfully
*
*
*    ARGUMENT LIST:
*
*       NAME            TYPE		USE	DESCRIPTION
*       ------------    ---------------	---	----------------------------------
*       p_file_name	char *		I	Name of file to open
*	p_fd		int *		O	File descriptor
*	file_options	int		I	File options for opening
*	p_pa		void **		O	Memory map address
*	file_start	off_t 		O	Start offset of file or 
*						  -1 for map enter file
*	p_file_len	long *		O	File length
*	p_remainder	long *		O	Byte offset into the page,
*						  used only if not defaulting
*	mapflag		boolean		I	Mapping indicator
*	lockflag	boolean		I	locking indicator
*	p_ErrorMessage	char *		O	Error Message
*
*
*    EXTERNAL FILE/RECORD REFERENCES:
*
*       NAME			USE     DESCRIPTION
*       ----			---     -----------------------------------------------
*	fcntl.h			I	Header file needed for file processing
*	string.h		I	Header file needed for errno error processing
*	unistd.h		I	Defines SEEK_SET
*	sys/mman.h		I	Header file needed for memory mapping support
*	sys/stat.h		I	Header file needed for file status processing
*	xxx_CloseUnmap.h	I	Defines function prototypes and contants
*	xxx_FileLock.h		I	Defines prototype and macros for lock and 
*					  unlock files
*	xxx_OpenMap.h		I 	Defines function prototypes and contants 
*       xxx_Types.h		I       Header file needed for PDP type definitions
*	p_file_name		I	Any file that may be passed into the routine
*
* 
*    EXTERNAL VARIABLES:
*
*       SOURCE          NAME			TYPE	USE	DESCRIPTION
*       ------          ---			----	---	-------------------------------
*	xxx_OpenMap.h	XXX_OPEN_ERR		int	I	Indicates file open error
*	xxx_OpenMap.h	XXX_STAT_ERR		int	I	Indicates file status error
*	xxx_OpenMap.h	XXX_MMAP_ERR		int	I	Indicates memory map error
*	xxx_OpenMap.h	XXX_MMAP_ZERO_ERR	int	I	Indicates zero file size for memory map error
*	xxx_Types.h	XXXSUCCESS		int	I	Indicates successful completion
*	xxx_Types.h	FALSE			int	I	Defined parameter
*
*
*    EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
*
*       NAME			PURPOSE
*       ------------		-------------------------------------------------------
*	xxx_FileLock		Locks and unlock files
*	XXX_WRITE_WAIT_LOCK	Macro to call xxx_FileLock to lock file in wait mode
*
*
*    ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
*	Can't open and/or create file
*	Can't get file statistics
*	Can't Memory map file
*	Tried to memory map an empty (0 size) file
*
*
*    NON-PORTABLE CODE:
*
*       DESCRIPTION     REASON FOR USE          SUGGESTED PORTING PROCEDURES
*       -----------     ---------------         ---------------------------------------
*	mmap		Speed up I/O to file
*
*
*    ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
*	The file_start and p_file_len values are correct if the user is not defaulting.
*
*
*    CHANGE HISTORY:
*
*                                  BUILD/
*       AUTHOR          CHANGE-ID  RELEASE    DATE	DESCRIPTION OF CHANGE
*       -----------     ---------  ---------- --------	-----------------------
*       J. Kerich	PAC2940631 R2.2       12/28/94	Original Implementation
*	J. Kerich       PAC2961624 R3.2       06/19/96	Check file options if locking file
*	A. Taylor	TRACE0116  TRACE R2.0 02/07/97  Set p_fd to error on failure and don't
*							 overwrite p_ErrorMessage when closing
*	J. kerich                  R1.0       03/24/97  Ported to IAS
*
*
*    NOTES:
*	File options normally are:
*       	O_RDWR | O_CREAT
*       	O_RDWR
*       	O_RDONLY
*
*	Warning:  The map option does not work for O_WRONLY or O_RDONLY.  This unit threrefore 
*	does not support O_WRONLY option for mapping!  You can still open and lock
*	the the file however.
*
*	Unit was originally pxx_open_map.c
*
******************************************************************************/
/******************************************************************************
UNIT xxx_OpenMap PDL
-------------------------------------------------------------------------------

1	IF lockflag is TRUE THEN
2		IF file_options is not O_RDWR THEN
3			IF file_options is O_WRONLY THEN
				Mask out O_WRONLY option
4			ENDIF
			Change file_options to O_RDWR 
5		ENDIF
6	ENDIF

	Open p_file_name using file_options and assign descriptor to p_fd
7	IF open error occured THEN
		Build error message in p_ErrorMessage
		RETURN XXX_OPEN_ERR
8	ENDIF

9	IF lockflag is TRUE THEN
		Call XXX_WRITE_WAIT_LOCK() to lock the file in wait mode
10	ENDIF	

	Call fstat() to get file length
11	IF file status error occured THEN
		Build error message in p_ErrorMessage
		Call xxx_CloseUnmap() to close file with lockfile option
		RETURN XXX_STAT_ERR
12	ENDIF

13	IF mapflag is TRUE THEN
14		IF file is zero size THEN
			Build error message in p_ErrorMessage
			Call xxx_CloseUnmap() to close file with lockfile option
			RETURN XXX_MMAP_ZERO_ERR
15		ENDIF
		
16		IF file_start indicates map all of file THEN
			Set file_start to zero
			Set p_file_len to file size
		ELSE
			Calculate file_start a multiple of a page size
			Calcutale the offset into the last page size and save in p_remainder
			Increment p_file_len by p_remainder
17		ENDIF

		Call mmap() to memory map file from file_start to p_file_len and assign 
		  address to p_pa
18		IF memory map error occurs THEN 
			Call xxx_CloseUnmap() to close file with lockfile option
			Build error message in p_ErrorMessage
			RETURN XXX_MMAP_ERR
19		ENDIF
20	ELSE
		Set p_file_len to file size
 	ENDIF

	RETURN XXXSUCCESS

-------------------------------------------------------------------------------
END UNIT xxx_OpenMap PDL
******************************************************************************/

/* Feature test switches */
#undef _POSIX_SOURCE

/* System headers */
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <errno.h>

/* Local headers */
#include "xxx_FileLock.h"
#include "xxx_Types.h"

/* External Functions */
#include "xxx_CloseUnmap.h"
#include "xxx_OpenMap.h"

int xxx_OpenMap(char *p_filename, int *p_fd, int file_options, void **p_pa, off_t file_start, 
	long *p_file_len, long *p_remainder, boolean mapflag, boolean lockflag, char *p_ErrorMessage)
{
/* local variables */

	struct	stat buf;
	int	flags;
	int	org_file_options = file_options;
	char	ErrorMessage[XXXCMLEN];
	
/* open file and return error if one occured */

	if (lockflag)
	{
		if (!((file_options & O_RDWR) == O_RDWR))
		{
			if ((file_options & O_WRONLY) == O_WRONLY)
			{
				file_options = (file_options ^ O_WRONLY); /* clear out O_WRONLY option */
			}
			file_options = file_options | O_RDWR; /* set to O_RDWR */
		}
	}

	if( (*p_fd = open(p_filename, file_options, 0664)) < 0){
		(void) strcpy(p_ErrorMessage, strerror(errno));
		(void) strcat(p_ErrorMessage, ": ");
		(void) strcat(p_ErrorMessage, p_filename);
		return(XXX_OPEN_ERR);
	}

	if (lockflag)
	{
		(void)XXX_WRITE_WAIT_LOCK(*p_fd, 0, SEEK_SET, 0);
	}

	/* call fstat to get file status for file size. Return error if one occured */

	if( (fstat(*p_fd,&buf)) < 0){
		(void) strcpy(p_ErrorMessage, strerror(errno));
		(void) strcat(p_ErrorMessage, ": ");
		(void) strcat(p_ErrorMessage, p_filename);
		(void) xxx_CloseUnmap(*p_fd, (caddr_t)NULL, (long)NULL, FALSE, lockflag, ErrorMessage);
		*p_fd = -1;
		return(XXX_STAT_ERR);
	}

/* memory map the file if requested. Return error if one occured */

	if(mapflag)
	{
		if (buf.st_size == 0)
		{
			(void) strcpy(p_ErrorMessage, "Can't map file because its file size is zero: ");
			(void) strcat(p_ErrorMessage, p_filename);
			(void) xxx_CloseUnmap(*p_fd, (caddr_t)NULL, (long)NULL, FALSE, lockflag, ErrorMessage);
			*p_fd = -1;
			return(XXX_MMAP_ZERO_ERR);
		}

		if (file_start == -1)  /* map enter file */
		{
			file_start = 0;
			*p_file_len = (long) buf.st_size;
			if (p_remainder != NULL)
			{
				*p_remainder = 0;
			}
		}
		else
		{
			/* turn into multiple of page size */
			*p_remainder =  (long) (file_start % (off_t) sysconf(_SC_PAGESIZE));
			*p_file_len += *p_remainder;
			 file_start = (file_start / (off_t) sysconf(_SC_PAGESIZE)) * 
				(off_t) sysconf(_SC_PAGESIZE);
		}

		/* Memory map file as it was openned */
		if ((org_file_options & O_RDWR) == O_RDWR)
		{
			flags = PROT_READ|PROT_WRITE;
		}
		else if ((org_file_options & O_WRONLY) == O_WRONLY)
		{
			flags = PROT_WRITE;
		}
		else
		{
			flags = PROT_READ;
		}

		if ((*p_pa=mmap((void *)NULL, (size_t)*p_file_len, flags,
			MAP_SHARED, *p_fd, file_start)) == MAP_FAILED)
		{
			(void) strcpy(p_ErrorMessage, strerror(errno));
			(void) strcat(p_ErrorMessage, ": ");
			(void) strcat(p_ErrorMessage, p_filename);
			(void) xxx_CloseUnmap(*p_fd, (caddr_t)NULL, (long)NULL, FALSE, lockflag, ErrorMessage);
			*p_fd = -1;
			return(XXX_MMAP_ERR);
		}
	}
	else
	{
		*p_file_len = (long) buf.st_size;
	}

	return(XXXSUCCESS);
}

