/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME: xxx_OpenODL.c

PURPOSE: Open and parse the ODL file.

INVOCATION METHOD:
    Pointer = xxx_OpenODL(p_ODLFile, ODLType, p_ErrorMessage)

RETURN VALUE:
    Type = OBJDESC *

Value           Description
--------------- ----------------------------------------------------------------
positive        Pointer to ODL tree 
NULL            Failure

ARGUMENT LIST:
Name            Type            Use Description
--------------- --------------  --- ---------------------------------------
p_ODLFile       char *           I  If a normal odl file is to be opened, the path is 
                                    to the ODL file otherwise the path is to the 
				    L0R directory file.
ODLType         xxx_EDCOdl_TYPE  I  Indicates if a ODL file to read or 
                                    a normal odl file (ie.xxx_NoEDCMeta). ..
p_ErrorMessage  char *           O  Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use         Description
----------------------- ---  -------------------------------------------------
none

EXTERNAL VARIABLES:
Source            Name                  Type            Use Description
---------------   -------------------   --------------- --- --------------------
OdlPrintMessage   ODLErrorMessage       char *          I   ODL error message
xxx_Errno.c       xxx_errno             long            O   Error number 

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                    Description
----------------------- -------------------------------------------------
ODLParseLabelFile       Read in and parse ODL file
xxx_FindDirectory       Find the directory for the file path
xxx_GetDirFiles         List the directory
XXX_ARRAY_SIZE          Return array size
xxx_GetTempName         Generate temp name
xxx_RecursiveDeletion   Delete the file/directory recursively
OdlFreeTree             To free the ODL tree

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
ODL error
Can't find product meta data file

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- -----------------------------
T. Raheja         N/A         R1.0    03/97      Original Implementation
j. kerich         N/A         R1.0    05/01/97   Add EDC Meta data logic
j. kerich         IAS970011   R2.0    11/10/97   Change L7*_CPF* to L7CPF*
j. kerich         IASI970057  R2.0    12/05/97   Delete ODL error file if ODL file 
						 was successfully opened
j. kerich         IASI980424  R3.1    08/11/98   Add Errno logic 
j. kerich         IASI990538  R3.2    11/23/98   Add L1 file logic and ODL_NOEXPAND instead of NULL

Unit xxx_OpenODL PDL:

1       IF ODLType is not xxx_NoEDCMeta THEN
	        Call xxx_FindDirectory() to find the product meta data directory
		Find the product metadata extention
		Call xxx_GetDirFiles() to get the product meta data path
2	        IF product meta data file was not found THEN
			Build error message in p_ErrorMessage
			Set xxx_errno to XXX_E_FILE_NOT_FOUND
		        RETURN NULL
3		ENDIF
4	ELSE
	        Use p_ODLFile as ODL path
	ENDIF

        Call xxx_GetTempName() to generate ODL error file name
5       IF error THEN
	        Build error message in p_ErrorMessage
		Set xxx_errno to XXX_E_NO_MEMORY
	        RETURN NULL
6       ENDIF

	Call OdlParseLabelFile() to open and parse the ODL file
7       IF error THEN
                Build error message in p_ErrorMessage
		Set xxx_errno to XXX_E_ODL_SYNTAX
                RETURN NULL
8       ENDIF
      
9	IF warning is set in ODLErrorMessage THEN 
                Build error message in p_ErrorMessage
		Call OdlFreeTree() to free the tree
		Set xxx_errno to XXX_E_ODL_SYNTAX
                RETURN NULL
10	ENDIF

        Call xxx_RecursiveDeletion() to delete the ODL error file

        RETURN ODL pointer

End Unit xxx_OpenODL
******************************************************************************/

/* System Headers */
#include <stdio.h>
#include <string.h>
#include <sys/param.h>

/* Local Headers */
#include <lablib3.h>                 /* prototypes for the ODL functions */
#include "xxx_Errno.h"
#include "xxx_GetDirFiles.h"
#include "xxx_Imagery.h"
#include "xxx_ODL.h"                 /* prototype for the xxx ODL functions */
#include "xxx_Types.h"               
#include "xxx_GetTempName.h"
#include "xxx_RecursiveDeletion.h"

extern char ODLErrorMessage[];       /* External Variables */

OBJDESC *xxx_OpenODL (char *p_ODLFile, xxx_EDCOdl_TYPE ODLType, char *p_ErrorMessage)
{
	OBJDESC *p_lp= NULL;
        char ODLTempFile[MAXPATHLEN];
	char ODLFileName[MAXPATHLEN];
	char ODLPathName[MAXPATHLEN];
	char ODLErrorFile[MAXPATHLEN];
        char *p_TempFile;
        char *p_FileType;
	char *p_TempName;
	int Count;
	int i;
	xxx_DirList_TYPE *p_DirectoryList;

	typedef struct xxx_MetaIndex_TYPE {
		xxx_EDCOdl_TYPE  Want;
		char ProductMeta[30];
	} xxx_MetaIndex_TYPE;

	xxx_MetaIndex_TYPE ProductMetaData[5];

        /* Set the metadata file strings, based on the input ODL file name */
        /* Get a pointer to filename portion of ODL File string */
        strcpy(ODLTempFile, p_ODLFile);
        p_TempFile = strrchr(ODLTempFile, '/');
        if(p_TempFile != NULL) /* ignore directory path */
            p_TempFile++;
        else /* no directory path in file name */
            p_TempFile = ODLTempFile;

        /* Use input file name string up to file type to create metadata
           file name string to match with */
        p_FileType = strrchr(p_TempFile, '_');
        if(p_FileType == NULL)
        {
            /* File name does not follow format defined by DFCB */
            (void)strcpy(p_ErrorMessage, "Invalid L7 product file name\n");
            xxx_errno = XXX_E_FILE_NOT_FOUND;
            return(NULL);
        }
        p_FileType++;
        *p_FileType = '\0'; /* crop file name beginning at file type */

        /* Some elements of the HDF file name may differ among files of
           the same product. To locate the correct metadata file, use a
           wild card for those elements that are known to vary. */
        /* L0R product metadata file */
        sprintf(ODLFileName, "%sMTP*", p_TempFile);
        ODLFileName[6] = '?';
        ODLFileName[7] = '?';
        ProductMetaData[0].Want = xxx_EDCMeta;
        strcpy(ProductMetaData[0].ProductMeta, ODLFileName);

        /* L0R/L1R/L1G LPS metadata file format 1 */
        sprintf(ODLFileName, "%sMTA.*", p_TempFile);
        ODLFileName[6] = '?';
        ODLFileName[7] = '?';
        ProductMetaData[1].Want = xxx_LPSMeta1;
        strcpy(ProductMetaData[1].ProductMeta, ODLFileName);

        /* L0R/L1R/L1G LPS metadata file format 2 */
        sprintf(ODLFileName, "%sMTA.*", p_TempFile);
        ODLFileName[2] = '?';
        ProductMetaData[2].Want = xxx_LPSMeta2;
        strcpy(ProductMetaData[2].ProductMeta, ODLFileName);

        /* L0R calibration parameter file */
        sprintf(ODLFileName, "L7CPF*");
        ProductMetaData[3].Want = xxx_CPFMeta;
        strcpy(ProductMetaData[3].ProductMeta, ODLFileName);

        /* L1T/L1R/L1G LPGS metadata file */
        sprintf(ODLFileName, "%sMTL*", p_TempFile);
        ODLFileName[2] = '?';
        ProductMetaData[4].Want = xxx_LPGSMeta;
        strcpy(ProductMetaData[4].ProductMeta, ODLFileName);

	ODLErrorMessage[0] = '\0';

	if (ODLType != xxx_NoEDCMeta)
	{
		xxx_FindDirectory(ODLPathName, p_ODLFile);
		for(i = 0;  i < XXX_ARRAY_SIZE(ProductMetaData) && ProductMetaData[i].Want != ODLType;  i++);

		p_DirectoryList = xxx_GetDirFiles(&Count, ODLPathName, ProductMetaData[i].ProductMeta,  p_ErrorMessage);

		if (p_DirectoryList == NULL)
		{
			xxx_errno = XXX_E_FILE_NOT_FOUND;
			return NULL;
		}
		(void)strcpy(ODLPathName, p_DirectoryList[0].FilePathName);
		free (p_DirectoryList);
	}
	else
	{
		(void)strcpy(ODLPathName, p_ODLFile);
	}

	/* Build ODL error file name */
	if ( (p_TempName =  xxx_GetTempName("/usr/tmp", "ODLEr")) == NULL)
	{
		(void)strcpy(p_ErrorMessage, "Can't generate temporary path name\n");
		xxx_errno = XXX_E_NO_MEMORY;
		return(NULL);
	}
	(void)sprintf(ODLErrorFile,"%s.dat", p_TempName);
	free(p_TempName);

	/*   open and parse ODL file - error messages to ODLPathName */
	if ((p_lp = OdlParseLabelFile(ODLPathName, ODLErrorFile, ODL_NOEXPAND, FALSE)) == NULL)
	{
		(void)sprintf(p_ErrorMessage, "ODL Error on File: %s:  %s", ODLPathName, ODLErrorMessage);
		xxx_errno = XXX_E_ODL_SYNTAX;
		return NULL;
	}

	/* Check of ODL syntax errors */
	if (ODLErrorMessage[0] != '\0')
	{
		(void)sprintf(p_ErrorMessage,"%s: check out %s", ODLErrorMessage, ODLErrorFile);
		OdlFreeTree(p_lp);
		xxx_errno = XXX_E_ODL_SYNTAX;
		return NULL;
	}

	/* Delete the ODL error log file */
        (void)xxx_RecursiveDeletion(ODLErrorFile, p_ErrorMessage);

	return p_lp;
}
