/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_ReadEDCMetadata.c

PURPOSE:  Read in the EDC metadata and store it in a structure.

INVOCATION METHOD:
        Pointer - xxx_ReadEDCMetadata(p_Pathname, p_ErrorMessage)

RETURN VALUE:
Type = xxx_ECDMetaDataFile_TYPE

Value           Description
--------------  ----------------------------------------------------------------
positive        EDC meta data read in and converted successfully
NULL            Failed to read in the MSCD files.

ARGUMENT LIST:
Name            Type            Use     Description
--------------- --------------- ---    ---------------------------------------
p_PathName      char *          I       Pathname to L0R HDF directory file
p_ErrorMessage  char *          O       Error message, if any

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use Description
------------------------- --- -------------------------------------------------
EDC metadata file         I   ODL file

EXTERNAL VARIABLES:
Source          Name                Type            Use Description
--------------- ------------------- --------------- --- -----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name               Description
------------------ -------------------------------------------------
xxx_OpenODL        Open the ODL
xxx_GetODLField    Return the converted field
xxx_CloseODL       Close ODL file
XXX_ARRAY_SIZE     Macro returns size of array

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
memory error
ODL error

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. kerich         N/A         R1.0    03/19/97   Original Implementation
J. kerich         IASI970103  R2.0    09/16/97   Change xxxYDHMSFTime to xxxISOYDHMSFTime
J. kerich         IAS970011   R2.0    11/12/97   Change XXX_Int to XXX_Long
J. kerich         IASI980295  R3.0    03/24/97   Calculate TotalScenes using StartingRow and EndingRow
J. kerich         IAS980096   R3.1    07/20/98   Add FORMAT_SCAN_OFFSET field

NOTES:
none

Unit xxx_ReadEDCMetadata PDL:

      Call xxx_OpenODL() to open the p_PathName
1     IF error openning file THEN
              RETURN NULL
2     ENDIF

      Allocate memory for xxx_ECSMetaDataFile_TYPE
3     IF allocation error THEN
              Build error message in p_ErrorMessage
	      ABORT to ERROR_PROC
4     ENDIF

5     DOFOR all fields in the EDC metadata
              Call xxx_GetODLField() to get the field and save it in xxx_ECDMetaDataFile_TYPE
6	      IF error getting field THEN
	              ABORT to ERROR_PROC
7	      ENDIF
8     ENDDO

      Call xxx_CloseODL() to close EDC metadata file
9     IF error closing file THEN
              ABORT to ERROR_PROC:
10    ENDIF

      RETURN xxx_ECDMetaDataFile_TYPE

ERROR_PROC:
      Free memory
      Call xxx_CloseODL() to close EDC metadata file

      RETURN NULL	      

End Unit xxx_ReadEDCMetadata
******************************************************************************/

/* System headers */
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

/* Local headers */
#include "xxx_EDCMeta.h"
#include "xxx_ODL.h"
#include "xxx_Types.h"

xxx_EDCMetaDataFile_TYPE *xxx_ReadEDCMetadata(char *p_PathName, char *p_ErrorMessage)
{
	typedef struct labelLocations {
		char		  LabelName[100];
		boolean           Optionial;
		int  		  Memorysize;
		xxx_ValType_TYPE  ValType;
		long		  offset;
	} LabelLocations;

	static LabelLocations table[] = {
		{"PRODUCT_CREATION_DATE_TIME", 0, sizeof(xxxISO8601Time),   XXX_String, offsetof(xxx_EDCMetaDataFileInfo_TYPE,
		    ProductCreationDateTime)},
		{"STATION_ID",                 0, 20,                       XXX_String, offsetof(xxx_EDCMetaDataFileInfo_TYPE,
		    StationId)},
	};

	static LabelLocations table2[] = {
		{"PRODUCT_TYPE",              0, 10,                    XXX_String, offsetof(xxx_ProductMetaData_TYPE, ProductType)},
		{"SPACECRAFT_ID",             0, 15,                    XXX_String, offsetof(xxx_ProductMetaData_TYPE, SpacecraftId)},
		{"SENSOR_ID",                 0, 15,                    XXX_String, offsetof(xxx_ProductMetaData_TYPE, SensorId)},
		{"ACQUISITION_DATE",          0, sizeof(xxxYMDTime),    XXX_String, offsetof(xxx_ProductMetaData_TYPE, AcquisitionDate)},
		{"STARTING_PATH",             0, sizeof(long),          XXX_Long,   offsetof(xxx_ProductMetaData_TYPE, WRSPath)},
		{"STARTING_ROW",              0, sizeof(long),          XXX_Long,   offsetof(xxx_ProductMetaData_TYPE, StartingRow)},
		{"ENDING_ROW",                0, sizeof(long),          XXX_Long,   offsetof(xxx_ProductMetaData_TYPE, EndingRow)},
		{"TOTAL_WRS_SCENES",          0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, TotalWRSScenes)},
		{"NUMBER_OF_SCANS",           0, sizeof(long),          XXX_Long,   offsetof(xxx_ProductMetaData_TYPE, NumberOfScans)},
		{"STARTING_SUBINTERVAL_SCAN", 0, sizeof(long),          XXX_Long,   offsetof(xxx_ProductMetaData_TYPE, StartingSubintervalScan)},
		{"ENDING_SUBINTERVAL_SCAN",   0, sizeof(long),          XXX_Long,   offsetof(xxx_ProductMetaData_TYPE, EndingSubintervalScan)},
		{"BAND_COMBINATION",          0, 14,                    XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band_combination)},
		{"PRODUCT_UL_CORNER_LAT",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductULCornerLat)},
		{"PRODUCT_UL_CORNER_LON",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductULCornerLon)},
		{"PRODUCT_UR_CORNER_LAT",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductURCornerLat)},
		{"PRODUCT_UR_CORNER_LON",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductURCornerLon)},
		{"PRODUCT_LL_CORNER_LAT",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductLLCornerLat)},
		{"PRODUCT_LL_CORNER_LON",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductLLCornerLon)},
		{"PRODUCT_LR_CORNER_LAT",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductLRCornerLat)},
		{"PRODUCT_LR_CORNER_LON",     0, sizeof(float),         XXX_Float,  offsetof(xxx_ProductMetaData_TYPE, ProductLRCornerLon)},
		{"BAND1_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band1_FileName)},
		{"BAND2_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band2_FileName)},
		{"BAND3_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band3_FileName)},
		{"BAND4_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band4_FileName)},
		{"BAND5_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band5_FileName)},
		{"BAND6_FILE_NAME_F1",        1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band6_F1_FileName)},
		{"BAND6_FILE_NAME_F2",        1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band6_F2_FileName)},
		{"BAND7_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band7_FileName)},
		{"BAND8_FILE1_NAME",          1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band8_F1_FileName)},
		{"BAND8_FILE2_NAME",          1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band8_F2_FileName)},
		{"BAND8_FILE3_NAME",          1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Band8_F3_FileName)},
		{"IC_DATA_FILE_NAME_F1",      1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, IC_Data_F1_FileName)},
		{"IC_DATA_FILE_NAME_F2",      1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, IC_Data_F2_FileName)},
		{"SCAN_OFFSETS_FILE_NAME_F1", 1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, ScanOffsetsF1Name)},
		{"SCAN_OFFSETS_FILE_NAME_F2", 1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, ScanOffsetsF2Name)},
		{"MSCD_FILE_NAME_F1",         1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, MSCD_Data_F1_FileName)},
		{"MSCD_FILE_NAME_F2",         1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, MSCD_Data_F2_FileName)},
		{"PCD_FILE_NAME_F1",          1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, PCD_Data_F1_FileName)},
		{"PCD_FILE_NAME_F2",          1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, PCD_Data_F2_FileName)},
		{"METADATA_FILE_NAME_F1",     1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, MetaData_F1_FileName)},
		{"METADATA_FILE_NAME_F2",     1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, MetaData_F2_FileName)},
		{"METADATA_PS_FILE_NAME",     0, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, MetaData_PS_FileName)},
		{"CPF_FILE_NAME",             0, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, CPF_FileName)},
		{"GEOLOCATION_FILE_NAME",     0, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, Geolocation_FileName)},
		{"HDF_DIR_FILE_NAME",         0, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_ProductMetaData_TYPE, HDF_FileName)},
	};

	OBJDESC  *Pointer;
	char      ErrorMessage[XXXCMLEN];
	xxx_EDCMetaDataFile_TYPE *p_EDCMetaDataFile;
	int       Count;
	int       i;

	if ((Pointer = xxx_OpenODL(p_PathName, xxx_EDCMeta, p_ErrorMessage)) == NULL)
	{
		return (NULL);
	}

	if ((p_EDCMetaDataFile = (xxx_EDCMetaDataFile_TYPE *)calloc(1, sizeof(xxx_EDCMetaDataFile_TYPE))) == NULL)
	{
		(void)sprintf(p_ErrorMessage, "Calloc error");
		goto ERROR_PROC;
	}

	for(i = 0; i < XXX_ARRAY_SIZE(table); i++)
	{
		if (xxx_GetODLField(((char *)&p_EDCMetaDataFile->MetadataFileInfo+table[i].offset), table[i].Memorysize,
		    table[i].ValType, Pointer, NULL, table[i].LabelName, p_ErrorMessage, &Count))
		{
			if (table[i].Optionial)
			{
				continue;
			}
			else
			{
				goto ERROR_PROC;
			}
		}
	}

	for(i = 0; i < XXX_ARRAY_SIZE(table2); i++)
	{

		if (xxx_GetODLField(((char *)&p_EDCMetaDataFile->ProductMetaData+table2[i].offset), table2[i].Memorysize,
		    table2[i].ValType, Pointer, NULL, table2[i].LabelName, p_ErrorMessage, &Count))
		{
			if (table2[i].Optionial)
			{
				continue;
			}
			else
			{
				goto ERROR_PROC;
			}
		}
	}

	if (p_EDCMetaDataFile->ProductMetaData.StartingRow <= p_EDCMetaDataFile->ProductMetaData.EndingRow)
	{
		p_EDCMetaDataFile->ProductMetaData.TotalScenes = 
		    p_EDCMetaDataFile->ProductMetaData.EndingRow - p_EDCMetaDataFile->ProductMetaData.StartingRow + 1;
	}
	else
	{
		p_EDCMetaDataFile->ProductMetaData.TotalScenes = 
		    (p_EDCMetaDataFile->ProductMetaData.EndingRow + 248) - p_EDCMetaDataFile->ProductMetaData.StartingRow + 1;
	}

	if (xxx_CloseODL(Pointer, p_ErrorMessage))
	{
	        Pointer = NULL;
		goto ERROR_PROC;
	}

	return (p_EDCMetaDataFile);

ERROR_PROC:
	free(p_EDCMetaDataFile);
	(void)xxx_CloseODL(Pointer, ErrorMessage);
	return (NULL);
}
