/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_ReadImagery.c

PURPOSE:  To read the data field from the SDS HDF file, starting at a given row in
	  the HDF file.  This unit will only work on one SDS name which has one field.

INVOCATION METHOD:
	status = xxx_ReadImagery(p_ImageryInfo, p_Imagery, RowstoRead, p_StartOffset, 
			FirstCol, ColsToRead, p_ErrorMessage);

RETURN VALUE:
Type = int

Value         	Description
--------------	----------------------------------------------------------------
XXXSUCCESS	Read data from file
ERROR		Failed to read data from file

ARGUMENT LIST:
Name             Type              	Use	Description
---------------- ------------------	---	---------------------------------------
p_ImageryInfo	xxx_ImageryInfo_TYPE *	I/O	HDF imagery information
p_Imagery	void *			I/O	Array of data read in from the file
RowsToRead	int32			I	Number of rows to read
p_StartOffset	int32 *			I	Position were the read will start,
                                                the p_StartOffset is increment by RowsToRead
FirstCol	int32			I	Position were the read will start for column,
                                                if not subsetting set to -1
ColsToRead	int32			I	Number of columns to read
                                                if not subsetting set to -1
p_ErrorMessage	char *			O	Error message 

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use Description
------------------------- --- -------------------------------------------------
HDF file                  I   HDF file

EXTERNAL VARIABLES:
Source          Name            Type  Use Description
--------------- --------------  ----- --- -----------------------
hdf.h           FAIL            int   I   HDF failure status
herr.h          error_top       int32 I   Top to HDF error stack
xxx_Imagery.h   XXX_MAX_RANK    int   I   Maximum rank allowed
xxx_Types.h     ERROR           int   I   Error creating time stamp.
xxx_Types.h     XXXSUCCESS      int   I   Indicates successful completion

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                    Description
----------------------- -------------------------------------------------
HEstring                Returns error message for the error number
HEvalue                 Returns the current error number
HXsetdir                Sets the external directory path
SDreaddata		Read in the data

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
HDF errors
Invalide arguments

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
That rows are one based.

DEVELOPMENT HISTORY:
Author                  Change-ID       Release Date            Description of Change
-----------------       -----------     ------- --------        ------------------------------
J. kerich               N/A             R1.0    03/19/97        Original Implementation
J. Kerich	        IASI970058      R2.0    09/30/97	Reset DirectoryPath for HDF files
J. Kerich	        IAS970051       R3.0    02/19/98	Reset DirectoryPath all the time

NOTES:
- The p_Imagery array will be an unpadded array.
- See HDF Users Guide for detailed field/value setting.
- The p_StartOffset is zero base.
- The FirstCol is zero base

Unit xxx_ReadImagery PDL:

	 Check the rank, RowsToRead, and p_StartOffset against the SD file information for 
	   valid inputs
1	 IF error THEN
	         Build error message in p_ErrorMessage
		 RETURN ERROR
2	 ENDIF

	 Check the FirstCol and ColsToRead against the SD file information for 
	   valid inputs
3	 IF error THEN
	         Build error message in p_ErrorMessage
		 RETURN ERROR
4	 ENDIF

         Call HXsetdir() to set the external directory path
5        IF error setting the external directory path THEN
                Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
	        RETURN ERROR
6        ENDIF

	 Call SDreaddata() to read in the data from the p_StartOffset into p_Imagery
7	 IF error THEN
	         Call HEstring(HEvalue(error_top)) to build error message in p_ErrorMessage
		 RETURN ERROR
8	 ENDIF

	 RETURN XXXSUCCESS

End Unit xxx_ReadImagery
******************************************************************************/

/* System headers */
#include <string.h>

/* Local headers */
#include <hdf.h>
#include <mfhdf.h>
#include "xxx_Imagery.h"
#include "xxx_HDFDirectory.h"
#include "xxx_Types.h"

extern int32 error_top;

int xxx_ReadImagery(xxx_ImageryInfo_TYPE *p_ImageryInfo, void *p_Imagery, int32 RowsToRead, int32 *p_StartOffset, 
	int32 FirstCol, int32 ColsToRead, char *p_ErrorMessage)
{
	const char *p_HEstring;

	int32 start[2];
	int32 stride[2] = {
		1, 1	};
	int32 edge[2];

	if (RowsToRead > p_ImageryInfo->DimensionSizes[0] || RowsToRead < 1 ||  
	    *p_StartOffset < 0 || *p_StartOffset >= p_ImageryInfo->DimensionSizes[0] ||
	    (*p_StartOffset +  RowsToRead) > p_ImageryInfo->DimensionSizes[0])
	{
		(void)sprintf(p_ErrorMessage, "Invalid range, StartOffset=%ld RowsToRead=%ld where number of rows are 0->%ld",
		    *p_StartOffset, RowsToRead, p_ImageryInfo->DimensionSizes[0]-1);
		return(ERROR);
	}

	if (p_ImageryInfo->Rank != XXX_MAX_RANK)
	{
		(void)sprintf(p_ErrorMessage, "Data sets rank %ld is not equal to %d", p_ImageryInfo->Rank, XXX_MAX_RANK);
		return(ERROR);
	}

	if (FirstCol >= 0 || ColsToRead >= 0)
	{
		if (FirstCol < 0 || ColsToRead < 0 || 
		    FirstCol > p_ImageryInfo->DimensionSizes[1] || ColsToRead > p_ImageryInfo->DimensionSizes[1])
		{
			(void)sprintf(p_ErrorMessage, "FirstCol: %ld or ColsToRead: %ld is out of range for dimension 0->%ld",

			    FirstCol, ColsToRead, p_ImageryInfo->DimensionSizes[1]);
			return(ERROR);
		}
	}

       	if (HXsetdir(p_ImageryInfo->DirectoryPath) == FAIL)
       	{
       		(void)strcpy(p_ErrorMessage, "Can't set external file path: ");
       		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
       		(void)strcat(p_ErrorMessage, p_HEstring);
       		return (ERROR);
       	}

	if (FirstCol >= 0 || ColsToRead >= 0)
	{
		start[0] = *p_StartOffset;
		start[1] = FirstCol;
		edge[0] = RowsToRead;
		edge[1] = ColsToRead;
	}
	else
	{
		start[0] = *p_StartOffset;
		start[1] = 0;
		edge[0] = RowsToRead;
		edge[1] = p_ImageryInfo->DimensionSizes[1];
	}

	if (SDreaddata(p_ImageryInfo->SDS_Id, start, stride, edge, p_Imagery) == FAIL)
	{
		(void)strcpy(p_ErrorMessage, "Can't read SD file: ");
		p_HEstring = HEstring((hdf_err_code_t)HEvalue(error_top));
		(void)strcat(p_ErrorMessage, p_HEstring);
		return(ERROR);
	}

	*p_StartOffset += RowsToRead;

	return (XXXSUCCESS);
}
