/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_ReadLPGSMetadata.c

PURPOSE:  Read in the LPGS metadata and store it in a structure.

INVOCATION METHOD:
        Pointer - xxx_ReadLPGSMetadata(p_Pathname, p_ErrorMessage)

RETURN VALUE:
Type = xxx_ECDMetaDataFile_TYPE

Value           Description
--------------  ----------------------------------------------------------------
positive        LPGS meta data read in and converted successfully
NULL            Failed to read in the MSCD files.

ARGUMENT LIST:
Name            Type            Use     Description
--------------- --------------- ---    ---------------------------------------
p_PathName      char *          I       Pathname to L0R HDF directory file
p_ErrorMessage  char *          O       Error message, if any

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use Description
------------------------- --- -------------------------------------------------
LPGS metadata file         I   ODL file

EXTERNAL VARIABLES:
Source          Name                Type            Use Description
--------------- ------------------- --------------- --- -----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name               Description
------------------ -------------------------------------------------
xxx_OpenODL        Open the ODL
xxx_GetODLField    Return the converted field
xxx_CloseODL       Close ODL file
XXX_ARRAY_SIZE     Macro returns size of array

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
memory error
ODL error

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. kerich         IASI990538  R3.2    11/23/98   Original Implementation
J. kerich      	  IASI990550  R3.2    02/17/99	 Add new metadata fields

NOTES:
none

Unit xxx_ReadLPGSMetadata PDL:

      Call xxx_OpenODL() to open the p_PathName
1     IF error openning file THEN
              RETURN NULL
2     ENDIF

      Allocate memory for xxx_ECSMetaDataFile_TYPE
3     IF allocation error THEN
              Build error message in p_ErrorMessage
	      ABORT to ERROR_PROC
4     ENDIF

5     DOFOR all fields in the LPGS metadata
              Call xxx_GetODLField() to get the field and save it in xxx_ECDMetaDataFile_TYPE
6	      IF error getting field THEN
	              ABORT to ERROR_PROC
7	      ENDIF
8     ENDDO

      Call xxx_CloseODL() to close LPGS metadata file
9     IF error closing file THEN
              ABORT to ERROR_PROC:
10    ENDIF

      Save the rounded up TotalWRSScenes in TotalScenes

      RETURN xxx_ECDMetaDataFile_TYPE

ERROR_PROC:
      Free memory
      Call xxx_CloseODL() to close LPGS metadata file

      RETURN NULL	      

End Unit xxx_ReadLPGSMetadata
******************************************************************************/

/* System headers */
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

/* Local headers */
#include "xxx_LPGSMeta.h"
#include "xxx_ODL.h"
#include "xxx_Types.h"

xxx_LPGSMetaDataFile_TYPE *xxx_ReadLPGSMetadata(char *p_PathName, char *p_ErrorMessage)
{
	typedef struct LabelLocations_TYPE {
		char		  LabelName[100];
		boolean           Optionial;
		int 		  Memorysize;
		xxx_ValType_TYPE  ValType;
		long		  offset;
	} LabelLocations_TYPE;

	static LabelLocations_TYPE table1[] = {
		{"REQUEST_ID",                 0, 24,                       XXX_String, offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    RequestId)},
		{"PRODUCT_CREATION_TIME",      0, sizeof(xxxISO8601Time),   XXX_String, offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    ProductCreationTime)},
		{"STATION_ID",                 0, 4,                        XXX_String, offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    StationId)},
		{"LANDSAT7_XBAND",             0, 2,                        XXX_String, offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    Landsat7Xband)},
		{"GROUND_STATION",             0, 4,                        XXX_String, offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    GroundStation)},
		{"LPS_PROCESSOR_NUMBER",       0, sizeof(long),             XXX_Long,   offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    LPS_ProcessorNumber)},
		{"DATEHOUR_CONTACT_PERIOD",    0, 8,                        XXX_String, offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    DateHourContactPeriod)},
		{"SUBINTERVAL_NUMBER",         0, 4,                        XXX_String, offsetof(xxx_LPGSMetadataFileInfo_TYPE,
		    SubintervalNumber)},
	};

	static LabelLocations_TYPE table2[] = {
		{"PRODUCT_TYPE",              0, 4,                     XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, ProductType)},
        {"PROCESSING_SOFTWARE",       0, 30,                    XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, ProcessingSoftware)},
		{"SPACECRAFT_ID",             0, 12,                    XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, SpacecraftId)},
		{"SENSOR_ID",                 0, 8,                     XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, SensorId)},
		{"ACQUISITION_DATE",          0, sizeof(xxxYMDTime),    XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, AcquisitionDate)},
		{"WRS_PATH",                  0, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, WRSPath)},
		{"STARTING_ROW",              0, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, StartingRow)},
		{"ENDING_ROW",                0, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, EndingRow)},
		{"BAND_COMBINATION",          0, 12,                    XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band_combination)},
		{"PRODUCT_UL_CORNER_LAT",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductULCornerLat)},
		{"PRODUCT_UL_CORNER_LON",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductULCornerLon)},
		{"PRODUCT_UR_CORNER_LAT",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductURCornerLat)},
		{"PRODUCT_UR_CORNER_LON",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductURCornerLon)},
		{"PRODUCT_LL_CORNER_LAT",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLLCornerLat)},
		{"PRODUCT_LL_CORNER_LON",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLLCornerLon)},
		{"PRODUCT_LR_CORNER_LAT",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLRCornerLat)},
		{"PRODUCT_LR_CORNER_LON",     0, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLRCornerLon)},
		{"PRODUCT_UL_CORNER_MAPX",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductULCornerMapX)},
		{"PRODUCT_UL_CORNER_MAPY",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductULCornerMapY)},
		{"PRODUCT_UR_CORNER_MAPX",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductURCornerMapX)},
		{"PRODUCT_UR_CORNER_MAPY",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductURCornerMapY)},
		{"PRODUCT_LL_CORNER_MAPX",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLLCornerMapX)},
		{"PRODUCT_LL_CORNER_MAPY",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLLCornerMapY)},
		{"PRODUCT_LR_CORNER_MAPX",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLRCornerMapX)},
		{"PRODUCT_LR_CORNER_MAPY",    1, sizeof(double),        XXX_Double, offsetof(xxx_LPGSProductMetaData_TYPE, ProductLRCornerMapY)},
		{"PRODUCT_SAMPLES_PAN",       1, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, ProductSamplesPAN)},
		{"PRODUCT_LINES_PAN",         1, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, ProductLinesPAN)},
		{"PRODUCT_SAMPLES_REF",       1, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, ProductSamplesREF)},
		{"PRODUCT_LINES_REF",         1, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, ProductLinesREF)},
		{"PRODUCT_SAMPLES_THM",       1, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, ProductSamplesTHM)},
		{"PRODUCT_LINES_THM",         1, sizeof(long),          XXX_Long,   offsetof(xxx_LPGSProductMetaData_TYPE, ProductLinesTHM)},
		{"BAND1_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band1_FileName)},
		{"BAND2_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band2_FileName)},
		{"BAND3_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band3_FileName)},
		{"BAND4_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band4_FileName)},
		{"BAND5_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band5_FileName)},
		{"BAND6L_FILE_NAME",          1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band6L_FileName)},
		{"BAND6H_FILE_NAME",          1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band6H_FileName)},
		{"BAND7_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band7_FileName)},
		{"BAND8_FILE_NAME",           1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Band8_FileName)},
		{"IC_DATA_F1_FILE_NAME",      1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, IC_Data_F1_FileName)},
		{"IC_DATA_F2_FILE_NAME",      1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, IC_Data_F2_FileName)},
		{"SCAN_SHIFTS_F1_FILE_NAME",  1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, ScanShifts_F1_FileName)},
		{"SCAN_SHIFTS_F2_FILE_NAME",  1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, ScanShifts_F2_FileName)},
		{"MSCD_FILE_NAME",            1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, MSCD_FileName)},
		{"PCD_FILE_NAME",             1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, PCD_FileName)},
		{"METADATA_LPS1_FILE_NAME",   1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, MetaData_LPS1_FileName)},
		{"METADATA_LPS2_FILE_NAME",   1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, MetaData_LPS2_FileName)},
		{"METADATA_LPGS_FILE_NAME",   1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, MetaData_LPGS_FileName)},
		{"CPF_FILE_NAME",             1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, CPF_FileName)},
		{"GEOLOCATION_FILE_NAME",     1, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, Geolocation_FileName)},
		{"HDF_DIR_FILE_NAME",         0, XXX_MAXFILE_SIZE,      XXX_String, offsetof(xxx_LPGSProductMetaData_TYPE, HDF_Directory_FileName)},
	};

	static LabelLocations_TYPE table7[] = {
		{"REFERENCE_DATUM",         1, 8,              XXX_String, offsetof(xxx_ProjectionParameters_TYPE, ReferenceDatum)},
		{"REFERENCE_ELLIPSOID",     1, 8,              XXX_String, offsetof(xxx_ProjectionParameters_TYPE, ReferenceEllipsoid)},
		{"GRID_CELL_SIZE_PAN",      1, sizeof(double), XXX_Double, offsetof(xxx_ProjectionParameters_TYPE, GridCellSizePan)},
		{"GRID_CELL_SIZE_THM",      1, sizeof(double), XXX_Double, offsetof(xxx_ProjectionParameters_TYPE, GridCellSizeThm)},
		{"GRID_CELL_SIZE_REF",      1, sizeof(double), XXX_Double, offsetof(xxx_ProjectionParameters_TYPE, GridCellSizeRef)},
		{"ORIENTATION",             1, 4,              XXX_String, offsetof(xxx_ProjectionParameters_TYPE, Orientation)},
		{"RESAMPLING_OPTION",       1, 4,              XXX_String, offsetof(xxx_ProjectionParameters_TYPE, ResamplingOption)},
		{"MAP_PROJECTION",          1, 4,              XXX_String, offsetof(xxx_ProjectionParameters_TYPE, MapProjection)},
	};

	static LabelLocations_TYPE table8[7] = {
		{"LATITUDE_OF_FIRST_STANDARD_PARALLEL",   0, sizeof(double), XXX_Double,  offsetof(xxx_LCC_Parameters_TYPE,
		    LatitudeOfFirstStandardParallel)},
		{"LATITUDE_OF_SECOND_STANDARD_PARALLEL",  0, sizeof(double), XXX_Double,  offsetof(xxx_LCC_Parameters_TYPE,
		    LatitudeOfSecondStandardParallel)},
		{"LONGITUDE_OF_CENTRAL_MERIDIAN",         0, sizeof(double), XXX_Double,  offsetof(xxx_LCC_Parameters_TYPE,
		    LongitudeOfCentrolMeridian)},
		{"LATITUDE_OF_PROJECTION_ORIGIN",         0, sizeof(double), XXX_Double,  offsetof(xxx_LCC_Parameters_TYPE,
		    LatitudeOfProjectionOrigin)},
		{"FALSE_EASTING",                         0, sizeof(double), XXX_Double, offsetof(xxx_LCC_Parameters_TYPE,
		    FalseEasting)},
		{"FALSE_NORTHING",                        0, sizeof(double), XXX_Double, offsetof(xxx_LCC_Parameters_TYPE,
		    FalseNorthing)},
		{"FALSE_EASTING_NORTHING_UNITS",          0, 8,              XXX_String, offsetof(xxx_LCC_Parameters_TYPE,
		    FalseEastingNorthingUnits)}
	};	

	static LabelLocations_TYPE table9[6] = {
		{"SCALE_FACTOR_AT_CENTRAL_MERIDIAN",      0, sizeof(double), XXX_Double,  offsetof(xxx_TM_Parameters_TYPE,
		    ScaleFactorAtCentrolMeridian)},
		{"LONGITUDE_OF_CENTRAL_MERIDIAN",         0, sizeof(double), XXX_Double,  offsetof(xxx_TM_Parameters_TYPE,
		    LongitudeOfCentrolMeridian)},
		{"LATITUDE_OF_PROJECTION_ORIGIN",         0, sizeof(double), XXX_Double,  offsetof(xxx_TM_Parameters_TYPE,
		    LatitudeOfProjectionOrigin)},
		{"FALSE_EASTING",                         0, sizeof(double), XXX_Double, offsetof(xxx_TM_Parameters_TYPE,
		    FalseEasting)},
		{"FALSE_NORTHING",                        0, sizeof(double), XXX_Double, offsetof(xxx_TM_Parameters_TYPE,
		    FalseNorthing)},
		{"FALSE_EASTING_NORTHING_UNITS",          0, 8,              XXX_String, offsetof(xxx_TM_Parameters_TYPE,
		    FalseEastingNorthingUnits)}
	};	

	static LabelLocations_TYPE table10[5] = {
		{"LONGITUDE_OF_CENTRAL_MERIDIAN",         0, sizeof(double), XXX_Double,  offsetof(xxx_PC_Parameters_TYPE,
		    LongitudeOfCentrolMeridian)},
		{"LATITUDE_OF_PROJECTION_ORIGIN",         0, sizeof(double), XXX_Double,  offsetof(xxx_PC_Parameters_TYPE,
		    LatitudeOfProjectionOrigin)},
		{"FALSE_EASTING",                         0, sizeof(double), XXX_Double, offsetof(xxx_PC_Parameters_TYPE,
		    FalseEasting)},
		{"FALSE_NORTHING",                        0, sizeof(double), XXX_Double, offsetof(xxx_PC_Parameters_TYPE,
		    FalseNorthing)},
		{"FALSE_EASTING_NORTHING_UNITS",          0, 8,              XXX_String, offsetof(xxx_PC_Parameters_TYPE,
		    FalseEastingNorthingUnits)}
	};	

	static LabelLocations_TYPE table11[5] = {
		{"VERTICAL_LONGITUDE_FROM_POLE",         0, sizeof(double), XXX_Double,  offsetof(xxx_PS_Parameters_TYPE,
		    VerticalLongitudeFromPole)},
		{"LATITUDE_OF_TRUE_SCALE",               0, sizeof(double), XXX_Double,  offsetof(xxx_PS_Parameters_TYPE,
		    LatitudeOfTrueScale)},
		{"FALSE_EASTING",                        0, sizeof(double), XXX_Double, offsetof(xxx_PS_Parameters_TYPE,
		    FalseEasting)},
		{"FALSE_NORTHING",                       0, sizeof(double), XXX_Double, offsetof(xxx_PS_Parameters_TYPE,
		    FalseNorthing)},
		{"FALSE_EASTING_NORTHING_UNITS",         0, 8,              XXX_String, offsetof(xxx_PS_Parameters_TYPE,
		    FalseEastingNorthingUnits)}
	};	

	static LabelLocations_TYPE table12[12] = {
		{"SCALE_FACTOR_AT_CENTER_OF_PROJECTION", 0, sizeof(double),  XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    ScaleFactorAtCenterOfProjection)},
		{"LATITUDE_OF_PROJECTION_ORIGIN",        0, sizeof(double),  XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    LatitudeOfProjectionOrigin)},
		{"FALSE_EASTING",                        0, sizeof(double),  XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    FalseEasting)},
		{"FALSE_NORTHING",                       0, sizeof(double),  XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    FalseNorthing)},
		{"FALSE_EASTING_NORTHING_UNITS",         0, 8,               XXX_String, offsetof(xxx_OM_Parameters_TYPE,
		    FalseEastingNorthingUnits)},
		{"OM_TYPE",                              0, 2,               XXX_String, offsetof(xxx_OM_Parameters_TYPE,
		    OM_Type)},
		{"LONGITUDE_FIRST_POINT_GEODETIC",       1,  sizeof(double), XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    s.oma.LongitudeFirstPointGeodetic)},
		{"LATITUDE_FIRST_POINT_GEODETIC",        1,  sizeof(double), XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    s.oma.LatitudeFirstPointGeodetic)},
		{"LONGITUDE_SECOND_POINT_GEODETIC",      1,  sizeof(double), XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    s.oma.LongitudeSecondPointGeodetic)},
		{"LATITUDE_SECOND_POINT_GEODETIC",       1,  sizeof(double), XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    s.oma.LatitudeSecondPointGeodetic)},
		{"ANGLE_OF_AZIMUTH",                     1,  sizeof(double), XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    s.omb.AngleOfAzimuth)},
		{"LONGITUDE_ALONG_PROJECTION",           1,  sizeof(double), XXX_Double, offsetof(xxx_OM_Parameters_TYPE,
		    s.omb.LongitudeAlongProjection)}
	};

	static LabelLocations_TYPE table13[1] = {
		{"ZONE_NUMBER",                         0, sizeof(long),  XXX_Long,  offsetof(xxx_UTM_Parameters_TYPE, zone_number)}
	};	

	typedef struct UnionGroups_TYPE {
		char		  	GroupName[16];
	  	void *		  	p_Address;
		LabelLocations_TYPE    	*p_Table;
		long  		 	 Entries;
	} UnionGroups_TYPE;

	UnionGroups_TYPE OtherGroups[] = {
		{"", NULL,	table1, 	XXX_ARRAY_SIZE(table1)},
		{"", NULL,	table2, 	XXX_ARRAY_SIZE(table2)},
		{"", NULL,	table7,	        XXX_ARRAY_SIZE(table7)},
	}; 

	UnionGroups_TYPE UnionGroups[] = {
		{"LCC_PARAMETERS", NULL,	table8, 	7}, /* 0 */
		{"TM_PARAMETERS",  NULL,	table9, 	6}, /* 1 */
		{"PC_PARAMETERS",  NULL,	table10, 	5}, /* 2 */
		{"PS_PARAMETERS",  NULL,	table11,  	5}, /* 3 */
		{"OM*_PARAMETERS", NULL,	table12,       12}, /* 4 */
		{"UTM_PARAMETERS", NULL,	table13,	1}, /* 5 */
	}; 						            /* 6 = SOM */

	OBJDESC  *p_Pointer;
	char      ErrorMessage[XXXCMLEN];
	xxx_LPGSMetaDataFile_TYPE *p_LPGSMetaDataFile;
	int       Count;
	int       h,i;

	if ((p_Pointer = xxx_OpenODL(p_PathName, xxx_LPGSMeta, p_ErrorMessage)) == NULL)
	{
		return (NULL);
	}

	if ((p_LPGSMetaDataFile = (xxx_LPGSMetaDataFile_TYPE *)calloc(1, sizeof(xxx_LPGSMetaDataFile_TYPE))) == NULL)
	{
		(void)sprintf(p_ErrorMessage, "Calloc error");
		goto ERROR_PROC;
	}

	/* load up on group addresses */
	OtherGroups[0].p_Address = &p_LPGSMetaDataFile->MetadataFileInfo;
	OtherGroups[1].p_Address = &p_LPGSMetaDataFile->ProductMetaData;
	OtherGroups[2].p_Address = &p_LPGSMetaDataFile->ProjectionParameters;
	UnionGroups[0].p_Address = &p_LPGSMetaDataFile->Parameters.u.lcc.LatitudeOfFirstStandardParallel;

	for (h=0; h < XXX_ARRAY_SIZE(OtherGroups); h++)
        {
		for(i = 0; i < OtherGroups[h].Entries; i++)
		{
			if (xxx_GetODLField(((char *)OtherGroups[h].p_Address + OtherGroups[h].p_Table[i].offset), 
			    OtherGroups[h].p_Table[i].Memorysize, OtherGroups[h].p_Table[i].ValType, p_Pointer, 
			    NULL, OtherGroups[h].p_Table[i].LabelName, p_ErrorMessage, &Count))
			{
				if (OtherGroups[h].p_Table[i].Optionial)
				{
					continue;
				}
				else
				{
					goto ERROR_PROC;
				}
			}
		}
	}

	/* find which Geometric group is in the LPGS metadata file */
	for(p_LPGSMetaDataFile->Parameters.GroupFound = 0; p_LPGSMetaDataFile->Parameters.GroupFound < XXX_ARRAY_SIZE(UnionGroups) &&
	    !OdlFindObjDesc(p_Pointer, UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].GroupName, NULL, NULL, 1, ODL_RECURSIVE_DOWN);
	    p_LPGSMetaDataFile->Parameters.GroupFound++);
	if (p_LPGSMetaDataFile->Parameters.GroupFound != XXX_ARRAY_SIZE(UnionGroups))
	{
		for(i = 0; i < UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].Entries; i++)
		{
			if (xxx_GetODLField(((char *)UnionGroups[0].p_Address +
			    UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].p_Table[i].offset), 
			    UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].p_Table[i].Memorysize,
			    UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].p_Table[i].ValType, p_Pointer, 
			    UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].GroupName, 
			    UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].p_Table[i].LabelName, p_ErrorMessage, &Count))
			{
				if (UnionGroups[p_LPGSMetaDataFile->Parameters.GroupFound].p_Table[i].Optionial)
				{
					continue;
				}
				else
				{
					goto ERROR_PROC;
				}
			}
		}
	}

	if (xxx_CloseODL(p_Pointer, p_ErrorMessage))
	{
		p_Pointer = NULL;
		goto ERROR_PROC;
	}

	/* how many scenes in LPGS metadata? */
	if (p_LPGSMetaDataFile->ProductMetaData.StartingRow <= p_LPGSMetaDataFile->ProductMetaData.EndingRow)
	{
		p_LPGSMetaDataFile->ProductMetaData.TotalScenes = 
		    p_LPGSMetaDataFile->ProductMetaData.EndingRow - p_LPGSMetaDataFile->ProductMetaData.StartingRow + 1;
	}
	else
	{
		p_LPGSMetaDataFile->ProductMetaData.TotalScenes = 
		    (p_LPGSMetaDataFile->ProductMetaData.EndingRow + 248) - p_LPGSMetaDataFile->ProductMetaData.StartingRow + 1;
	}

	return (p_LPGSMetaDataFile);

ERROR_PROC:
	free(p_LPGSMetaDataFile);
	(void)xxx_CloseODL(p_Pointer, ErrorMessage);
	return (NULL);
}
