/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

FILE NAME: 
	xxx_TimeConvert.h

PURPOSE: 
	Include file for time formats.

INVOCATION METHOD:
	#include <xxx_TimeConvert.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use 	Description
----------------------- --- 	-------------------------------------------------
none

CHANGE HISTORY:
Author		Change ID	Build/Release	Date		Description of Change
------		---------	-------------	---------	---------------------
J. Kerich	N/A		R1.0		02/11/97	Original Implementation
J. Kerich	IASI970103     	R2.0		09/16/97	Add ISO YDHMSF & YDHMS format


NOTES:
See xxx_TimeConvert.h for a function prototype for xxx_TimeConvert(), a function
that can convert between these time formats. The table below gives a summary of
the supported formats.

xxxTimeFormat		typedef name	size	format	epoch		year range	rollover	granularity
-------------		-------------	----	------	-----------	----------	--------	-----------
XXX_TIME_DMYHMS         xxxDMYHMSTime   20      ASCII   n/a             1901-2130       ~136 years      1 msec
XXX_TIME_DYHMSM         xxxDYHMSMTime   21      ASCII   n/a             1901-2130       ~136 years      1 msec
XXX_TIME_ISO8601        xxxISO8601Time  20      ASCII   n/a             1901-2130       ~136 years      1 second
XXX_TIME_ISOYDHMS	xxxISOYDHMSTime	18	ASCII	n/a		1901-2130	~136 years	1 second	
XXX_TIME_ISOYDHMSF	xxxISOYDHMSFTime26	ASCII	n/a		1901-2130       ~136 years	15/16 millisconds
XXX_TIME_JDHMS		xxxJDHMSTime	12	ASCII	n/a		n/a		1 year		1 second
XXX_TIME_MJDHMS 	xxxMJDHMSTime	14	ASCII	01 Jan 5713 B.C.n/a		n/a		1 second
XXX_TIME_ORACLE		xxxOracleTime	14	ASCII	n/a		0000-9999	10,000 years	1 second
XXX_TIME_T		time_t		4	binary	01 Jan 1970	1901-2038	~136 years	1 second
XXX_TIME_TF		double		8	binary	01 Jan 1993	1993-2130	~136 years	15/16 millisconds
XXX_TIME_YDHMS		xxxYDHMSTime	17	ASCII	n/a		1901-2130	~136 years	1 second	
XXX_TIME_YDHMSF		xxxYDHMSFTime	25	ASCII	n/a		1901-2130	~136 years	15/16 millisconds
XXX_TIME_YMD            xxxYMDTime      10      ASCII   n/a             1901-2130       ~136 years      1 msec

******************************************************************************/

#ifndef XXX_TIMECONVERT_H
#define XXX_TIMECONVERT_H

#include "xxx_Types.h"

/******************************************************************************
These define sizes and data types for time formats used on IAS. Note that for
ASCII formats, the defined size does NOT include a terminating '\0' char.
******************************************************************************/
#define XXX_DMYHMS_TIME_SIZE            20
#define XXX_DYHMSM_TIME_SIZE            21
#define XXX_ISO8601_TIME_SIZE  		20
#define XXX_ISOYDHMS_TIME_SIZE     	18
#define XXX_ISOYDHMSF_TIME_SIZE    	26
#define XXX_JDHMS_TIME_SIZE	        12
#define XXX_MJDHMS_TIME_SIZE        	14
#define XXX_ORACLE_TIME_SIZE       	14
#define XXX_TIME_T_TIME_SIZE    	4
#define XXX_TF_TIME_SIZE        	8			
#define XXX_YDHMS_TIME_SIZE     	17
#define XXX_YDHMSF_TIME_SIZE    	25
#define XXX_YMD_TIME_SIZE               10

/* xxxDMYHMSTime - ASCII format " DD-MMM-YYYY HH:MM:SS.MMMM", example "27-May-1997 23:59:59" */
typedef char xxxDMYHMSTime[XXX_DMYHMS_TIME_SIZE + 1];

/* xxxDYHMSMTime - ASCII format " DDD/YYYY HH:MM:SS.MMM", example "365/1993 23:59:59.999" */
typedef char xxxDYHMSMTime[XXX_DYHMSM_TIME_SIZE + 1];

/* xxxISO8601Time - ASCII format "YYYY-MM-DDTHH:MM:SSZ", example "1993-12-31T12:59:59Z" */
typedef char xxxISO8601Time[XXX_ISO8601_TIME_SIZE + 1];

/* xxxISOYDHMSTime - ASCII format "YYYY-DDDTHH:MM:SSZ", example "1993-365T23:59:59Z" */
typedef char xxxISOYDHMSTime[XXX_ISOYDHMS_TIME_SIZE + 1];

/* xxxISOYDHMSFTime - ASCII format "YYYY-DDDTHH:MM:SS.FFFFFFFZ", example "1997-365T23:59:59.9999375Z" */
typedef char xxxISOYDHMSFTime[XXX_ISOYDHMSF_TIME_SIZE + 1];

/* xxxJDHMSTime - ASCII format "DDD HH:MM:SS", example "365 23:59:59" */
typedef char xxxJDHMSTime[XXX_JDHMS_TIME_SIZE + 1];

/* xxxMJDHMSTime - ASCII format "DDDDD HH:MM:SS", example "50234 23:59:59" */
typedef char xxxMJDHMSTime[XXX_MJDHMS_TIME_SIZE + 1];

/* xxxOracleTime - ASCII format "YYYYMMDDHHMMSS", example "19930101123456" */
typedef char xxxOracleTime[XXX_ORACLE_TIME_SIZE + 1];

/* time_t - binary UNIX time, seconds since 00:00:00 on 1 Jan 1970, example 855619199 */
/* time_t's typedef is in <time.h> */

/* double - binary UNIX time,  seconds since 00:00:00 on 1 Jan 1993 and fractional seconds (0-9999375), example 98236799.99999375 */

/* xxxYDHMSTime - ASCII format "YYYY:DDD:HH:MM:SS", example "1993:365:23:59:59" */
typedef char xxxYDHMSTime[XXX_YDHMS_TIME_SIZE + 1];

/* xxxYDHMSFTime - ASCII format "YYYY:DDD:HH:MM:SS.FFFFFFF", example "1997:365:23:59:59.9999375" */
typedef char xxxYDHMSFTime[XXX_YDHMSF_TIME_SIZE + 1];

/* xxxYMDTime - ASCII format "YYYY-MM-DD", example "1993-12-31" */
typedef char xxxYMDTime[XXX_YMD_TIME_SIZE + 1];

/******************************************************************************
This defines the time formats supported by xxx_TimeConvert(). If
this typedef is changed, be sure to update xxx_TimeConvert.c to match.
******************************************************************************/
typedef enum xxxTimeFormat {
        XXX_TIME_DMYHMS,        /* xxxDMYHMSTime */
        XXX_TIME_DYHMSM,        /* xxxDYHMSMTime */
	XXX_TIME_ISO8601,       /* xxxISO8601Time */
	XXX_TIME_ISOYDHMS,	/* xxxISOYDHMSTime */
	XXX_TIME_ISOYDHMSF, 	/* xxxISOYDHMSFTime */
	XXX_TIME_JDHMS,		/* xxxJDHMSTime */
	XXX_TIME_MJDHMS,	/* xxxMJDHMSTime */
	XXX_TIME_ORACLE,	/* xxxOracleTime */
	XXX_TIME_T,		/* time_t */
	XXX_TIME_TF,		/* double */
	XXX_TIME_YDHMS,		/* xxxYDHMSTime */
	XXX_TIME_YDHMSF, 	/* xxxYDHMSFTime */
	XXX_TIME_YMD 		/* xxxYMDTime */
}
xxxTimeFormat;

extern boolean xxx_TimeConvert(void *old_time, xxxTimeFormat old_format, void *new_time, xxxTimeFormat new_format);

#endif
