/******************************************************************************
NAME: LAS2ARC -- Convert LAS image to Arc/Info Grid format

NAME:
    LAS2ARC

PURPOSE:
    Process LAS2ARC is invoked by procedure LAS2ARC to generate header,
    projection, and location files needed by the Arc/Info IMAGEGRID
    module for converting TRANSLAS-OUT output into Grid format.

ARGUMENTS:
    IN      (IN)    Name of input LAS image.
    OUTGRID (OUT)   Name of Arc/Info directory to receive
            single-band output grid file or stack file
            listing directories for each band of a
            multi-band output.
    NBAND   (OUT)   Number of bands to be output.

FILE REFERENCES:
    <IN>.ddr        READ to get image size, projection, and
            geolocation data.
    <OUTGRID>.hdr   CREATEd to record image size and data type.
    <OUTGRID>.prj   CREATEd to record projection data.
    <OUTGRID>       Directory CREATEd by Arc/Info GRIDIMAGE to
     or     contain single-band image and supporting files.
    <OUTGRID>.stk   CREATEd by Arc/Info GRIDIMAGE to list
            directories for separate bands of multi-band
    and     image stack.
    <OUTGRID>.cN    Directory CREATEd by A/I for band N of
            multi-band input image, one directory per band.

PROGRAM HISTORY:                                  
    07-92   Original development, K.Larson, EDC.
    02-94   Modification for UNIX/LAS6.0, K. Lowell, EDC.
    09-94   Rewritten to support going from LAS to Arc Grid format
        in a single step, create files expected by Arc/Info
        version 6.x, and use table-driven conversion of LAS
        projection coefficients to Arc/Info projection
        parameters.  VAX/VMS no longer supported.  R. White,
        PSU/ESSC.
    10-94       Updated for ARC7.0/EDC standards
    03-98   Modified output format for floats in the header file to use
            the full precision.  If valid, pixel sizes in DDR are used
            for XDIM and YDIM in header file.  Added warnings for nonvalid
            corner points and pixel sizes in DDR.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    This program runs under the TAE/LAS/UNIX environment.

PROJECT:
    LAS 

INFORMATIONAL AND ERROR MESSAGES:

    These are described in LAS2ARC.PDF.

ALGORITHM DESCRIPTION:
    Initialize TAE argument passing interface
    Get program arguments
    Get the DDR for the input file
    Open .hdr file for output
    Create entries for .hdr file
    If DDR contains valid corner coordinates
    add entries to .hdr file
    Compute coefficients of mapping from image to projection coords
    Write coefficients to .hdr file
    Close .hdr file
    Else
    Inform user that postioning data not available
    Check whether projection coefficents are present for projections
    which need them
    Look for coefficient mapping record for projection
    If DDR contains valid data for projection known to Arc/Info
    Open .prj file for output
    Write text records specifying projection and zone
    Interpret spheroid code and write spheroid name if not defaulted
    Write datum name if unambiguous
    Convert units name to form accepted by Arc/Info and write out
    Create parameter (projection coefficient) entries
    Close .prj file
    Else
    Inform user that projection data not available or unusable
    Output value of NBAND as TAE argument
    Write completion message

ALGORITHM REFERENCES:
    The Arc/Info projection and georeferencing procedures are discussed
    in the ESRI publications "Map Projections and Coordinate
    Management" and "Image Integration".
                                        
*******************************************************************/

#include <stdio.h>  /* Standard I/O definitions     */
#include <stdlib.h>
#include <ctype.h>  /* isupper prototype */
#include "las.h"        /* Standard LAS include files       */
#include "proj.h"   /* Map projection definitions       */
#include "util.h"
#include "labserv.h"
#include "protos.h"

struct PARBLK vblk;             /* TAE parameter block          */


int main
(void)
{

char  gridtik[CMLEN];        /* Name of the .tik file. */
char  gridstx[CMLEN];        /* Name of the .stx file. */
char  gridhdr[CMLEN];        /* Name of the .hdr file. */
char  gridprj[CMLEN];        /* Name of the .prj file. */

long    tmp_long;
long    lvalue;
long    ep3flag;       /* 8.3 file naming convention flag       */

struct PROJCOEFMAP *cmptr;     /* Mapping record for input projection */    
long    delflg;         /* flag on wether or not to create stx file */
struct DDR  ddr;        /* Input image data descriptor record */
char    outgrid[CMLEN]; /* Name of output Arc/Info grid */
long    pc;             /* Projection code */
long    nbands;	        /* Number of actual bands in input image */
char    projok;         /* Flag set if proj'n known to Arc/Info */
double  projtol;        /* Projection tolerance                 */
long    projvers;       /* Version of proj. (if alternatives) */
long    stdlun;         /* Logical unit number for std output */
char    hosin[CMLEN];   /* Host name of input image */
long    window[1][4];   /* Input window specification */
long    zero = 0;       /* Constant zero */
long    msgtim;
char    tmpstr[1][CMLEN]; /* Temp String */
int     j;              /* Counter */


/*  Initialize TAE interface
-----------------------------*/
c_inlas (&vblk,&stdlun);


/*  Get the variables from the TAE variable block
--------------------------------------------------  */
getpar(&vblk,hosin,outgrid,window,&msgtim,&ep3flag,&delflg,&nbands,&projtol);

/* Make sure OUTGRID field is lower case since 
   ARC does not support upper case file names
----------------------------------------------*/
for (j=0; outgrid[j]!='\0'; j++)
{
    if (isupper(outgrid[j]))
        c_lasmsg(TRUE,"las2arc-upcase",
                "OUTGRID name cannot contain upper case characters");
}

/* Set up the intermediate file names
--------------------------------------*/
c_lsmknm(outgrid,"hdr",gridhdr);
c_lsmknm(outgrid,"stx",gridstx);
c_lsmknm(outgrid,"tik",gridtik);
c_lsmknm(outgrid,"prj",gridprj);

/*  Get the DDR for the input file
-----------------------------------*/
if (c_getddr (hosin, &ddr) != E_SUCC)
    c_lasmsg(TRUE,"las2arc-fatal","Fatal error encountered");  
    
/* Create the .hdr file
------------------------*/
makehdr(gridhdr, nbands, ddr, window, projtol);


/*  Determine whether projection data are usable
-------------------------------------------------- */
checkproj(&projok, &cmptr, &projvers, &ddr, &pc);


/*  Open .prj file for output
--------------------------------------*/
if ((! projok) || makeprj(gridprj, cmptr, projvers, &ddr, pc) != E_SUCC)
    c_lasmsg(FALSE,"las2arc-noprj",
            "Projection or datum/spheroid information not available");


/*  If DELFLG=="NO", create the .stx file
---------------------------------------------------*/
if (!delflg)
    makestx(gridstx, hosin, ddr, window);


/*  Output value of NBAND as TAE argument
-----------------------------------------*/
lvalue = nbands;
if (E_SUCC != lng2tae(&lvalue,1,1,"NBANDS",&tmp_long))
    c_lasmsg(TRUE,"las2arc-qout","Error passing variables to TAE");

lvalue = ep3flag;
if (E_SUCC != lng2tae(&lvalue,1,1,"EP3FLAG",&tmp_long))
    c_lasmsg(TRUE,"las2arc-qout","Error passing variables to TAE");


if (q_out(&vblk) != SUCCESS)
    c_lasmsg(TRUE,"las2arc-qout","Error passing variables to TAE");


/*  Write completion message
----------------------------*/
strcpy(tmpstr[0],"\0");
c_complt ("LAS2ARC", tmpstr, &zero);


return EXIT_SUCCESS;

}
