/*******************************************************************************

NAME: MAKEHDR

PURPOSE:  To make a .hdr file with given information

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT:    LAS

*******************************************************************************/

#include "las.h"
#include "proj.h"
#include <stdio.h>
#include <math.h>
#include "protos.h"


int is_image_rotated
(
    const struct DDR *ddr,   /* I: image data descriptor record         */
    double           projtol /* I: Projection tolerance                 */
);


void makehdr
(
    char *gridhdr,       /* Name of .hdr file            */
    long nbands,         /* Number of bands              */
    struct DDR ddr,
    long window[1][4],   /* Window specification         */
    double projtol       /* Image projection tolerance   */
)

{
FILE *fp;               /* File pointer                 */
double  world[6];   /* Coeffs A, B, C, D, E, F of mapping
			from image to projections coords:
			x' = Ax + By + C
			y' = Dx + Ey + F     */

long nbits = 0;             /* Number of bits per sample    */
long ii;                /* loop counter                 */

/*  Open .hdr file for writing
-------------------------------*/
if ((fp = fopen (gridhdr, "w")) == NULL)
    c_lasmsg(TRUE,"las2arc-openhdr","Error opening .hdr file");

/*  Create entries for .hdr file
-----------------------------------*/
fprintf (fp, "BYTEORDER      M\n");
fprintf (fp, "LAYOUT       BIL\n");
fprintf (fp, "NROWS         %d\n", window[0][NL]);
fprintf (fp, "NCOLS         %d\n", window[0][NS]);
fprintf (fp, "NBANDS        %d\n", nbands);
switch (ddr.dtype)
    {
    case EBYTE:
        nbits = 8;
        break;
    case EWORD:
        nbits = 16;
        break;
    case ELONG:
        nbits = 32;
        break;
    default:
        c_lasmsg(TRUE,"las2arc-dtype","Cannot process data");
    }
fprintf (fp, "NBITS         %d\n", nbits);
fprintf (fp, "BANDROWBYTES  %d\n", window[0][3] * nbits / 8);
fprintf (fp, "TOTALROWBYTES %d\n", window[0][3] * nbands * nbits / 8);

/*  Check validity of corner points AND pixel size
--------------------------------------------------*/
if (ddr.valid[DDCCV] != VALID)
    c_lasmsg(FALSE,"las2arc-cornerpts", "DDR corner points not valid."
             "  Arc grid will not contain projection information.");
if (ddr.valid[DDPDV] != VALID)
    c_lasmsg(FALSE,"las2arc-pixelsz","DDR pixel size not valid."
             "  Arc grid will not contain projection information.");

/*  Exit if image is rotated
----------------------------*/
if (is_image_rotated(&ddr, projtol))
    c_lasmsg(TRUE,"las2arc-rot",
       "Input image is rotated.  Rotated images are not supported by Arc");

/*  Compute coefficients of mapping from image to projection coords
-------------------------------------------------------------------- */
for (ii = 0; ii < 2; ii++)
    {
    world[3*ii]   = (ddr.upright[1 - ii] - ddr.upleft[1 - ii])/(ddr.ns - 1);
    world[3*ii+1] = ( ddr.loleft[1 - ii] - ddr.upleft[1 - ii])/(ddr.nl - 1);
    world[3*ii+2] = ddr.upleft[1 - ii] + (window[0][1] - 1) * world[3 * ii]
    + (window[0][0] - 1) * world[3 * ii + 1];
    }

/*  Both pixel size and corner coordinates must be VALID in order to
    place XY dimensions and upper left coordinates into Arc header file.
------------------------------------------------------------------------*/
if (ddr.valid[DDPDV] == VALID && ddr.valid[DDCCV] == VALID)
    {
    if (ddr.proj_code == GEO)
        {
        fprintf (fp, "XDIM          %.15lf\n", ddr.pdist_x);
        fprintf (fp, "YDIM          %.15lf\n", ddr.pdist_y);
        fprintf (fp, "ULXMAP        %.15lf\n", world[2]);
        fprintf (fp, "ULYMAP        %.15lf\n", world[5]);
        }
    else
        {
        fprintf (fp, "XDIM          %f\n", ddr.pdist_x);
        fprintf (fp, "YDIM          %f\n", ddr.pdist_y);
        fprintf (fp, "ULXMAP        %f\n", world[2]);
        fprintf (fp, "ULYMAP        %f\n", world[5]);
        }
    }

fclose (fp);
return;
}


/****************************************************************************
FUNCTION:    is_image_rotated

PURPOSE:     Determines whether the input DDR describes a rotated image.

INPUT:       ddr     -  Image data descriptor record
             projtol -  Projection tolerance

OUTPUT:      none

RETURNS:     1 if image is rotated, else 0
*****************************************************************************/
int is_image_rotated
(
    const struct DDR *ddr,   /* I: image data descriptor record         */
    double           projtol /* I: Projection tolerance                 */
)
{
double      ytol,            /* Y direction tolerance                   */
            xtol;            /* X direction tolerance                   */

/* IF invalid corner coords OR pixel ground distance,
   cannot conclude that the image is rotated
   -------------------------------------------------- */
if ( (ddr->valid[DDCCV] != VALID) || (ddr->valid[DDPDV] != VALID) )
  return 0;

/* Compute the image tolerance in both directions
   ---------------------------------------------- */
ytol  = projtol * ddr->pdist_y;
xtol  = projtol * ddr->pdist_x;

/* Check for any corner cordinate value out of tolerance
   ----------------------------------------------------- */
if(ddr->upleft[0] + ytol < ddr->upright[0] ||
   ddr->upleft[0] - ytol > ddr->upright[0] ||
   ddr->loleft[0] + ytol < ddr->loright[0] ||
   ddr->loleft[0] - ytol > ddr->loright[0]  )
     return 1;
if(ddr->upleft[1] + xtol < ddr->loleft[1] ||
   ddr->upleft[1] - xtol > ddr->loleft[1] ||
   ddr->upright[1]+ xtol < ddr->loright[1]||
   ddr->upright[1]- xtol > ddr->loright[1] )
     return 1;
return 0;
} /* is_image_rotated */
