/*******************************************************************************
NAME: MAKEPRJ

PURPOSE:  To create .prj file from the given input specifications

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT:    LAS

*******************************************************************************/
#include <stdio.h>
#include <math.h>

#include "las.h"
#include "geompak.h"
#include "datum.h"
#include "protos.h"

SETPROJKN (projkn);             /* Projection keywords and names */


int makeprj
(
    char *gridprj,
    struct PROJCOEFMAP *cmptr,
    long projvers,
    struct DDR *ddr,
    long pc
)

{
    char    *cptr;      /* Pointer into map record */
    FILE *fp;
    long ii, jj;
    double  angle;      /* Angle value as dddmmmsss.sss */
    double  fract;      /* Fractional part of angle seconds */
    double  intang;     /* Interger part of angle */
    long    minn;       /* Minutes of angle */
    long    sec;        /* Seconds of angle */
    char    units[12];  /* String for horizontal units */
    char    c;          /* Temporary storage for character */
    int     rv = E_SUCC; /* return value */

    /*  Open .prj file for output
        --------------------------------------*/
    if ((fp = fopen (gridprj, "w")) == NULL)
        c_lasmsg(TRUE,"las2arc-open","Error opening .prj file");

    /*  Write text records specifying projection and zone  */

    fprintf (fp, "Projection    %s\n", projkn[pc].akey);
    if (ddr->valid[DDZCV] == VALID)
        fprintf (fp, "Zone          %d\n", ddr->zone_code);

    /*  Write Arc datum or spheroid name */

    if (ddr->valid[DDDCV] == VALID)
    {
        if ((ddr->datum_code <= MAX_DATUM) && (ddr->datum_code > MINDTM) &&
                (ddr->datum_code < (num_arc_datums() + 100)))
        {
            if (!strcmp(arc_datums[ddr->datum_code - 100], "NOTDEFINED"))
            {
                c_lasmsg(FALSE,"las2arc-datarc", 
                        "Datum %d not supported by Arc", 
                        ddr->datum_code);
                rv = E_FAIL;
            }
            fprintf( fp, "Datum         %s\n", 
                    arc_datums[ddr->datum_code - 100]);
        }
        else if (ddr->datum_code >= 0 && 
                ddr->datum_code < num_arc_spheroids())
        {
            if (!strcmp(arc_spheroids[ddr->datum_code], "NOTDEFINED"))
            {
                c_lasmsg(FALSE,"las2arc-spharc", 
                        "Spheroid %d not supported by Arc", 
                        ddr->datum_code);
                rv = E_FAIL;
            }
            fprintf( fp, "Datum         NONE\nSpheroid      %s\n", 
                    arc_spheroids[ddr->datum_code]); 
        }
        else
        {
            c_lasmsg(FALSE,"las2arc-datlas","Datum %d not supported by LAS2ARC",
                    ddr->datum_code);
            rv = E_FAIL;
        }
    }

    /*  Convert units name to form accepted by Arc/Info and write out  */

    if (ddr->valid[DDPUV] == VALID)
    {
        if (c_string2unit(ddr->proj_units) == E_FAIL)
        {
            c_lasmsg(FALSE, "las2arc-units",
                    "The projection units; %s are invalid. Setting Units to "
                    "empty.", ddr->proj_units);
            strcpy (units, " ");
        }
        else
        {
            strcpy (units, ddr->proj_units);
            if (strcmp (units, "degrees") == 0)
                strcpy (units, "dd");
            else if (strcmp (units, "seconds") == 0)
                strcpy (units, "ds");
        }
        fprintf (fp, "Units         %s\n", units);
    }

    /*  Output projection coefficients in format expected by Arc/Info  */

    fprintf (fp, "Parameters\n");
    ii = 1;
    cptr = cmptr->map;

    /*  First output projection version number, if any  */

    if (projvers > 0)
    {
        ii = 2;
        fprintf (fp, "%d\n", projvers);
    }

    /*  Find which coefficient to output next  */

    for (; ii <= COEFCT; ii++)
    {
        for (jj = 0; jj < COEFCT; jj++)
        {
            c = cptr[2 * jj + 1];
            if (((c >= '0') && (c <= '9') && ((c - '0') == ii))
                    || ((c >= 'a') && (c < 'g') && ((c - 'a' + 10) == ii)))
                break;
        }
        if (jj < COEFCT)
        {

            /*  Convert to Arc parameter as specified by conversion code  */

            switch (cptr[2 * jj])
            {
                case 'd':
                    fprintf (fp, "%.6f\n", ddr->proj_coef[jj]);
                    break;
                case 'a':
                    angle = ddr->proj_coef[jj];
                    if (angle < 0.0)
                    {
                        fprintf (fp, "-");
                        angle *= -1.0;
                    }
                    fract = modf (angle, &intang);
                    sec = (long )intang % 1000;
                    intang = (intang - sec) / 1000.;
                    minn = (long )intang % 1000;
                    intang = (intang - minn) / 1000.;
                    fprintf (fp, "%02.0f %02d %07.4f\n", intang, minn, 
                             fract+sec);
                    break;
                default:
                    c_lasmsg(FALSE,"las2arc-illconv",
                            "Illegal conversion %s %d %s",
                            "code for projection coefficient", jj,
                            " -- value not written to PRJ file");
                    break;
            }
        }
        else
            ii = COEFCT;
    }

    /*  Close  projection file  */

    fclose (fp);
    return rv;
}
