/*******************************************************************************
NAME: MAKESTX

PURPOSE:    To create a .stx file for the given input parameters

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT:    LAS

*******************************************************************************/
#include "las.h"
#include <stdio.h>
#include <math.h>
#include "protos.h"

#define FROM_BEGIN   0  /* flag for fseek */
#define FROM_CURRENT 1  /* flag for fseek */

void makestx
(
    char *gridstx,       /* Name of .stx file                        */
    char *hosin,         /* Name of image (BSQ) file                 */
    struct DDR ddr,            /* (used for .nbands .dtype .nl .ns)        */
    long window[1][4]   /* Window specification                     */
)
{
FILE    *ofp;           /* output (.stx) file pointer               */
FILE    *ifp;           /* input (.img) file pointer                */
char    *tline = NULL;  /* file buffer for reading in (.img)        */

long    bandnumber;
long    linenumber;
long    sample;         /* sample number - 1 currently being used   */
long    themax = 0;
long    themin = 0;
long    nbytes = 0;     /* Number of bytes for each element in buffer*/
long    skipfirst;      /* # of skip-bytes needed to read first line in window*/
long    skipnext;       /* # of skip-bytes needed to start window in next band*/
long    skipline;       /* # of skip-bytes needed between lines */
long    tempval = 0;    /* Each sample dereferenced from buffer is placed here*/

double  sum;            /* Used for calculating mean and std.dev    */
double  *mean;          /* Array of mean by band number             */
double  stddev;         /* Std Dev = sqrt( (x-mean)^2 / n-1 )       */


/* Allocate an array of averages for each band
---------------------------------------------------------*/
mean = (double*) calloc (ddr.nbands+1, sizeof(double));
if (mean == NULL)
    c_lasmsg(TRUE,"las2arc-alloc", "Memory allocation error");


/* Open input *.img and output *.stx file
-------------------------------------------------------*/
if ((ofp = fopen (gridstx, "w")) == NULL)
    c_lasmsg(TRUE,"las2arc-open","Error opening .stx file");
if ((ifp = fopen (hosin, "rb")) == NULL)
    c_lasmsg(TRUE, "las2arc-open", "Error opening: %s",hosin);


/* Allocate a memory buffer for reading in image file
-------------------------------------------------------*/
switch (ddr.dtype)
    {
    case EBYTE:
        nbytes = 1;
        tline = (char *)calloc(window[0][NS], nbytes);
        break;
    case EWORD:
        nbytes = 2;
        tline = (char *)calloc(window[0][NS], nbytes);
        break;
    case ELONG:
        nbytes = 4;
        tline = (char *)calloc(window[0][NS], nbytes);
        break;
    default:
        c_lasmsg(TRUE,"las2arc-dtype","Cannot process data");
    }
if (tline==NULL)
    c_lasmsg(TRUE, "las2arc-alloc", "Memory allocation error");


/* Calculate offsets for manuevering around image file
-----------------------------------------------------------*/
skipfirst = ((window[0][SL] - 1)*ddr.ns + window[0][SS] - 1)      * nbytes;
skipnext  = ((ddr.nl - window[0][NL] + 1)*ddr.ns - window[0][NS]) * nbytes;
skipline  = (ddr.ns - window[0][NS]) * nbytes;


/*  CALCULATE THE MEAN FOR ALL BANDS
---------------------------------------------------------------*/
for (bandnumber = 1; bandnumber <= ddr.nbands; bandnumber++)
    {
    if (bandnumber == 1)
        fseek(ifp, skipfirst, FROM_BEGIN);
    else
        fseek(ifp, skipnext, FROM_CURRENT);
    
    sum = 0.0;
    for (linenumber = 1; linenumber <= window[0][NL]; linenumber++)
        {
        fread(tline, nbytes, window[0][NS], ifp);
        if (linenumber < window[0][NL])   /* IF MORE LINES IN WINDOW TO READ */
            fseek(ifp, skipline, FROM_CURRENT);
        
        for (sample = 0; sample < window[0][NS]; sample++)
            {
            if (ddr.dtype==EBYTE)
                tempval = (long)(*((char  *)&tline[sample * nbytes]));
            else if (ddr.dtype==EWORD)
                tempval = (long)(*((short *)&tline[sample * nbytes]));
            else if (ddr.dtype==ELONG)
                tempval = (long)(*((long  *)&tline[sample * nbytes]));
            
            sum += tempval;
            }
        }
    mean[bandnumber] = sum / (double)(window[0][NS] * window[0][NL]);
    }


/*  CALCULATE THE MIN, MAX, & STANDARD DEVIATION FOR ALL BANDS
-----------------------------------------------------------------*/
for (bandnumber = 1; bandnumber <= ddr.nbands; bandnumber++)
    {
    if (bandnumber == 1)
        fseek(ifp, skipfirst, FROM_BEGIN);
    else
        fseek(ifp, skipnext, FROM_CURRENT);
    
    sum = 0.0;
    for (linenumber = 1; linenumber <= window[0][NL]; linenumber++)
        {
        fread(tline, nbytes, window[0][NS], ifp);
        
        if (linenumber < window[0][NL])   /* IF MORE LINES IN WINDOW TO READ */
            fseek(ifp, skipline, FROM_CURRENT);
        
        for (sample = 0; sample < window[0][NS]; sample++)
            {
            if (ddr.dtype==EBYTE)
                tempval = (long)(*((char  *)&tline[sample * nbytes]));
            else if (ddr.dtype==EWORD)
                tempval = (long)(*((short *)&tline[sample * nbytes]));
            else if (ddr.dtype==ELONG)
                tempval = (long)(*((long  *)&tline[sample * nbytes]));
            
            
            /* DETERMINE MAX and MIN
            --------------------------------------*/
            if ((linenumber == 1)&&(sample == 0))
                {
                themin = tempval;
                themax = tempval;
                }  
            if (tempval<themin)
                themin = tempval;
            else if (tempval>themax)
                themax = tempval;
            
            
            /* SUMMATION of (X - mean)^2
            ---------------------------------------------*/
            sum += ((double)tempval - mean[bandnumber])
            * ((double)tempval - mean[bandnumber]);
            }
        }
    stddev = sqrt( sum / ((double)((window[0][NL]*window[0][NS])-1)) );
    
    fprintf(ofp, "%d %d %d %f %f\n",
    bandnumber, themin, themax, mean[bandnumber], stddev);
    }


free (mean);
free (tline);
fclose (ofp);
fclose (ifp);

return;
}
