/***************************************************************************
NAME:   conv_coord.c

PURPOSE:  Converts the corner coordinates in a ddr to meters and dms.

PROGRAM HISTORY:

Version  Date  Programmer        Code  Reason
-------  ----- ----------        ----  ------
  1.0    01/98 T.Ratliff          C    Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
      Must be run under TAE.

PROJECT:  LAS

ALGORITHM DESCRIPTION:
  Check the ddr.proj_units to see what conversions we need to do
  Convert the corner coordinates into meters and dms.
  Pass the new corner coordinates back to ddr2nhdr

****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "proj.h"
#include "protos.h"

#define F2M  .3048006096012192

int conv_coord
(
    struct DDR *ddr,                      /* data in recs 1 and 2 of ddr      */
    double nept[],                        /* array of northing/easting points */
    char llpt[][15]                       /* array of lat/long points         */
)
{
    double tmpdeg[8];                   /* holds corner points in degrees   */
    double tmpdms[8];                   /* holds corner points in pdms      */
    long eleven = 11;                   /* Constant of eleven               */
    long i = 0;                         /* loop counter                     */

    /* Converting proj_units to upper case
       --------------------------------------*/
    c_low2up(ddr->proj_units, &eleven);

    /* Determine what units are, then
       convert accordingly.
       -------------------------------------*/
    if (strcmp(ddr->proj_units, "RADIANS") == 0)
    {
        tmpdeg[0] = ddr->upleft[1] * R2D;
        tmpdeg[1] = ddr->upleft[0] * R2D;
        tmpdeg[2] = ddr->upright[1] * R2D;
        tmpdeg[3] = ddr->upright[0] * R2D;
        tmpdeg[4] = ddr->loright[1] * R2D;
        tmpdeg[5] = ddr->loright[0] * R2D;
        tmpdeg[6] = ddr->loleft[1] * R2D;
        tmpdeg[7] = ddr->loleft[0] * R2D;

        if ((ddr->proj_code != 1) && (ddr->proj_code != 2))
            ddr->zone_code = calc_utm_zone(tmpdeg[0]);

        /* Converting from degrees to dms
           ---------------------------------*/
        for (i = 0; i < 8; i=i+2)
            c_degdms(&tmpdeg[i],&tmpdms[i],"DEG","LON");
        for (i = 1; i < 8; i=i+2)
            c_degdms(&tmpdeg[i],&tmpdms[i],"DEG","LAT");

        dms2met(tmpdms,llpt,nept,ddr);
    }
    else if (strcmp(ddr->proj_units, "FEET") == 0)
    {
        /* Convert from feet to meters
           ------------------------------*/
        nept[0] = ddr->upleft[1] * F2M;
        nept[1] = ddr->upleft[0] * F2M;
        nept[2] = ddr->upright[1] * F2M;
        nept[3] = ddr->upright[0] * F2M;
        nept[4] = ddr->loright[1] * F2M;
        nept[5] = ddr->loright[0] * F2M;
        nept[6] = ddr->loleft[1] * F2M;
        nept[7] = ddr->loleft[0] * F2M;
        met2dms(nept,llpt,ddr);
    }
    else if (strcmp(ddr->proj_units, "METERS") == 0)
    {
        nept[0] = ddr->upleft[1];
        nept[1] = ddr->upleft[0];
        nept[2] = ddr->upright[1];
        nept[3] = ddr->upright[0];
        nept[4] = ddr->loright[1];
        nept[5] = ddr->loright[0];
        nept[6] = ddr->loleft[1];
        nept[7] = ddr->loleft[0];
        met2dms(nept,llpt,ddr);
    }
    else if (strcmp(ddr->proj_units, "SECONDS") == 0)
    {
        /* Convert from sec. of arc to degrees
           --------------------------------------*/
        tmpdeg[0] = ddr->upleft[1] * S2D;
        tmpdeg[1] = ddr->upleft[0] * S2D;
        tmpdeg[2] = ddr->upright[1] * S2D;
        tmpdeg[3] = ddr->upright[0] * S2D;
        tmpdeg[4] = ddr->loright[1] * S2D;
        tmpdeg[5] = ddr->loright[0] * S2D;
        tmpdeg[6] = ddr->loleft[1] * S2D;
        tmpdeg[7] = ddr->loleft[0] * S2D;

        if ((ddr->proj_code != 1) && (ddr->proj_code != 2))
            ddr->zone_code = calc_utm_zone(tmpdeg[0]);

        /* Converting from degrees to dms
           ---------------------------------*/
        for (i = 0; i < 8; i=i+2)
            c_degdms(&tmpdeg[i],&tmpdms[i],"DEG","LON");
        for (i = 1; i < 8; i=i+2)
            c_degdms(&tmpdeg[i],&tmpdms[i],"DEG","LAT");

        dms2met(tmpdms,llpt,nept,ddr);
    }
    else if (strcmp(ddr->proj_units, "DEGREES") == 0)
    {
        tmpdeg[0] = ddr->upleft[1];
        tmpdeg[1] = ddr->upleft[0];
        tmpdeg[2] = ddr->upright[1];
        tmpdeg[3] = ddr->upright[0];
        tmpdeg[4] = ddr->loright[1];
        tmpdeg[5] = ddr->loright[0];
        tmpdeg[6] = ddr->loleft[1];
        tmpdeg[7] = ddr->loleft[0];

        if ((ddr->proj_code != 1) && (ddr->proj_code != 2))
            ddr->zone_code = calc_utm_zone(tmpdeg[0]);

        /* Converting from degrees to dms
           ---------------------------------*/
        for (i = 0; i < 8; i=i+2)
            c_degdms(&tmpdeg[i],&tmpdms[i],"DEG","LON");
        for (i = 1; i < 8; i=i+2)
            c_degdms(&tmpdeg[i],&tmpdms[i],"DEG","LAT");

        dms2met(tmpdms,llpt,nept,ddr);
    }  
    else if (strcmp(ddr->proj_units, "DMS") == 0)
    {
        tmpdms[0] = ddr->upleft[1];
        tmpdms[1] = ddr->upleft[0];
        tmpdms[2] = ddr->upright[1];
        tmpdms[3] = ddr->upright[0];
        tmpdms[4] = ddr->loright[1];
        tmpdms[5] = ddr->loright[0];
        tmpdms[6] = ddr->loleft[1];
        tmpdms[7] = ddr->loleft[0];

        if ((ddr->proj_code != 1) && (ddr->proj_code != 2))
        {
            tmpdeg[0] = ddr->upleft[1]/1000000;
            ddr->zone_code = calc_utm_zone(tmpdeg[0]);
        }

        dms2met(tmpdms,llpt,nept,ddr);
    }  

    return(E_SUCC);
}
