/******************************************************************************
NAME: DDR2NHDR

PURPOSE: Generates NDF format header file from DDR. 
         Part of the LAS2NDF program.

PROGRAM HISTORY:
Version  Date     Programmer        Code  Reason
-------  -----    ----------        ----  -------
  1.0    06/94    K.Lowell            C   Initial development - NLAPS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

PROJECT: LAS

ALGORITHM/DESCRIPTION
   This function is going to read the DDR file and create an NDF
   format header file.  The function is going to look for the
   following information:
      NDF Revision Number
      Data Type
      Pixels per Line
      Lines per Data File
      Data File Interleaving
      Corner Coordinate Information
      Map Projection Information
      Band Information
      Satellite Information
      Semi-Major & Semi-Minor Axis
      Horizontal Datum Code

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"

int ddr2nhdr
(
    char *hostout,                   /* UNIX format of output name   */
    char *outddr,                    /* name of temp. ddr file       */
    long *nb,                        /* number of bands              */
    char *bform                     /* data format of files         */
)

{
    struct DDR ddr;	                /* local ddr structure          */
    struct BDDR bptr;               /* ptr to current BDDR record   */

    FILE *fp;                       /* file pointer                 */

    char corner[20];                /* image corner                 */
    char date[10];                  /* holds the date information   */
    char ftemp[30];              	/* float string temp            */
    char buff[CMLEN];               /* output buffer                */
    char nlines[10];                /* number of image lines        */
    char nsamps[10];                /* number of samples            */
    char outheader[CMLEN];          /* output NDF header file name  */
    char temp[CMLEN];               /* temp long                    */
    char time[8];                   /* time                         */
    char time2[8];                  /* time                         */
    char sat[32];                   /* satellite                    */
    char sensor[3];                 /* satellite sensor             */
    char llpt[8][15];               /* long/lat points              */
    char proj_name[CMLEN];          /* map projection name          */
    char corstr[20];                /* corner we're working with    */
    char errmsg[100];               /* error message string         */
    char *p;                        /* pointer to characters        */

    long dtype = 0;                 /* ddr data type                */
    long fatal = -1;                /* fatal error flag             */
    long hlong = 0;                 /* length of buff               */
    long i = 0;                     /* counter                      */
    long key = 0;                   /* BDDR key value               */
    long nonfatal = 1;              /* nonfatal error flag          */
    long status = 0;                /* return status                */
    long senstat = 0;               /* return status of sat. info   */
    long errflg = 1;                /* flag for sat./sensor/date/time   */

    double nept[8];                 /* northing/easting points      */
    double minor = 0.0;             /* semi-minor axis              */
    double major = 0.0;             /* semi-major axis              */


    /* Get the DDR record and 
       create the NDF header file name 
       ----------------------------------*/
    status = c_getddr(outddr,&ddr);
    if (status != E_SUCC)
        c_errmsg("Error reading DDR file","las2ndf-read",&fatal);

    c_lsmknm(hostout,"hd",outheader);
    fp = fopen(outheader,"w");
    if (fp == NULL)
        c_errmsg("Error opening NDF header file","las2ndf-open",&fatal);

    /* Initalize the BDDR structure
       -------------------------------*/
    i = 1;
    status = c_getbdr(outddr,&bptr,&i);
    if (status == E_FAIL)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);

    /* Write fields into NDF header file
       ------------------------------------*/

    /* NDF revision number
       ----------------------*/
    strcpy(buff,"NDF_REVISION=0.00;\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the NDF_REVISION to header file","las2ndf-writ",
                &nonfatal);

    /* Data File Interleaving 
       ---------------------------*/
    sprintf(buff,"DATA_FILE_INTERLEAVING=%s;\n",bform);
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the DATA_INTERLEAVING to header file",
                 "las2ndf-writ", &nonfatal);

    /* Tape_Spanning flag
       ---------------------*/
    strcpy(buff,"TAPE_SPANNING_FLAG=1/1\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the TAPE_SPANNING_FLAG to header file",
                "las2ndf-writ", &nonfatal);

    /* Start Line Number
       --------------------*/
    strcpy(buff,"START_LINE_NUMBER=1;\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the START_LINE_NUMBER to header file",
                "las2ndf-writ", &nonfatal);

    /* Start Data File
       ------------------*/
    strcpy(buff,"START_DATA_FILE=1;\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the START_DATA_FILE to header file",
                "las2ndf-writ", &nonfatal);

    /* Blocking Factor
       ------------------*/
    strcpy(buff,"BLOCKING_FACTOR=1;\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the BLOCKING_FACTOR to header file",
                "las2ndf-writ", &nonfatal);

    /* Map projection information
       -----------------------------------*/
    if (ddr.valid[0] == 1)
    {
        strcpy(buff,"MAP_PROJECTION_NAME=");
        c_prostr(&ddr.proj_code,proj_name);
        sprintf(temp,"%s",proj_name);
        strcat(buff,temp);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the USGS_PROJ_NAME to header file",
                     "las2ndf-writ", &nonfatal);

        strcpy(buff,"USGS_PROJECTION_NUMBER=");
        sprintf(temp,"%d",ddr.proj_code);
        strcat(buff,temp);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the USGS_PROJ_NUM to header file",
                    "las2ndf-writ", &nonfatal);
    }

    if (ddr.valid[1] == 1)
    {
        strcpy(buff,"USGS_MAP_ZONE=");
        sprintf(temp,"%d",ddr.zone_code);
        strcat(buff,temp);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the USGS_MAP_ZONE to header file",
                    "las2ndf-writ", &nonfatal);
    }

    if (ddr.valid[3] == 1)
    {
        strcpy(buff,"USGS_PROJECTION_PARAMETERS=");
        for (i = 0; i < 15; i++)
        {
            sprintf(ftemp,"%.15lf",ddr.proj_coef[i]);
            strcat(buff,ftemp);
            if (i < 14)
                strcat(buff,",");
        }
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the USGS_PROJ_PARMS to header file",
                    "las2ndf-writ", &nonfatal);
    }  

    /* Horizontal Datum Code
       ------------------------*/
    if (ddr.valid[2] == 1)
    {
        strcpy(buff,"HORIZONTAL_DATUM=");
        if (ddr.proj_coef[14] == 2)
            sprintf(ftemp,"NAD27");
        else if (ddr.proj_coef[14] == 3)
            sprintf(ftemp,"NAD83");
        else
        {
            sprintf(ftemp,"UNKNOWN");
        }
        strcat(buff,ftemp);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the HORIZONTAL_DATUM to header file",
                    "las2ndf-writ", &nonfatal);
    }

    /* Semi-Major Axis
       ------------------*/
    get_semiaxis(&major,&minor,&ddr);
    strcpy(buff,"EARTH_ELLIPSOID_SEMI-MAJOR_AXIS=");
    sprintf(ftemp,"%.3lf",major);
    strcat(buff,ftemp);
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the SEMI-MAJOR_AXIS to header file",
                 "las2ndf-writ", &nonfatal);

    /* Semi-Minor Axis
       ------------------*/
    strcpy(buff,"EARTH_ELLIPSOID_SEMI-MINOR_AXIS=");
    sprintf(ftemp,"%.3lf",minor);
    strcat(buff,ftemp);
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the SEMI-MINOR_AXIS to header file",
                 "las2ndf-writ", &nonfatal);

    /* Satellite sensor
       -------------------*/
    key = DDINST;
    senstat = c_getbdf(&bptr,&key,(unsigned char *)sensor);
    if (senstat == E_FAIL)
        c_errmsg("Nonfatal error encountered","las2ndf-nonfatal",&nonfatal);

    if ((strlen(sensor) <= 1) || (strncmp(sensor,"  ",2) == 0))
        errflg = 0;

    strcpy(buff,"DATA_SET_TYPE=");
    if ((errflg) && (senstat != E_FAIL))
    {
        strcat(buff,"EDC_");
        strcat(buff,sensor);
    }
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the DATA_SET_TYPE to header file",
                 "las2ndf-writ", &nonfatal);

    /* Data type
       ------------*/
    dtype = ddr.dtype;
    if (dtype == 1)
        strcpy(buff,"PIXEL_FORMAT=BYTE;\n");
    else if (dtype == 2)
        strcpy(buff,"PIXEL_FORMAT=2BYTEINT;\n");
    else if (dtype == 3)
        strcpy(buff,"PIXEL_FORMAT=4BYTEINT;\n");
    else if (dtype == 4)
        strcpy(buff,"PIXEL_FORMAT=REAL;\n");
    else
    {
        fclose(fp);
        c_errmsg("Invalid data type","las2ndf-dtype",&fatal);
    }

    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the PIXEL FORMAT to header file","las2ndf-writ",
                &nonfatal);

    /* Pixels per line
       ------------------*/
    strcpy(buff,"PIXELS_PER_LINE=");
    sprintf(nlines,"%d",ddr.ns);
    strcat(buff,nlines);
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the PIXELS_PER_LINE to header file",
                 "las2ndf-writ", &nonfatal);

    /* lines per data file
       ----------------------*/
    strcpy(buff,"LINES_PER_DATA_FILE=");
    sprintf(nsamps,"%d",ddr.nl);
    strcat(buff,nsamps);
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing the LINES_PER_FILE to header file",
                "las2ndf-writ", &nonfatal);

    /* Number of data files
       -----------------------*/
    strcpy(buff,"NUMBER_OF_DATA_FILES=");
    sprintf(temp,"%d",ddr.nbands);
    strcat(buff,temp);
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing NUMBER_OF_FILES field to header file",
                "las2ndf-writ", &nonfatal);

    /* Lines per volume
       -------------------*/
    strcpy(buff,"LINES_PER_VOLUME=");
    sprintf(temp,"%d",ddr.nbands*ddr.nl);
    strcat(buff,temp);
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing LINES_PER_VOLUME to header file","las2ndf-writ",
                &nonfatal);

    /* Corner coordinate information
       --------------------------------*/
    if ((ddr.valid[4] == 1) && (ddr.valid[6] == 1))
    {
        /* Get the corner coordinates in
           Northing/Easting, Lat/Long
           --------------------------------*/
        conv_coord(&ddr,nept,llpt);

        for (i = 0; i < 7; i=i+2)
        {
            switch(i)
            {
                case 0:
                    strcpy(corstr,"UPPER_LEFT_CORNER");
                    break;
                case 2:
                    strcpy(corstr,"UPPER_RIGHT_CORNER");
                    break;
                case 4:
                    strcpy(corstr,"LOWER_RIGHT_CORNER");
                    break;
                case 6:
                    strcpy(corstr,"LOWER_LEFT_CORNER");
                    break;
            }
            strcat(corstr,"=");
            strcpy(buff,corstr);

            /* Longitude value
               ------------------*/
            sprintf(corner,"%s",llpt[i]);
            strcat(buff,corner);
            strcat(buff,",");

            /* Latitude value
               -----------------*/
            sprintf(corner,"%s",llpt[i+1]);
            strcat(buff,corner);
            strcat(buff,",");

            /* Easting value
               ----------------*/
            sprintf(corner,"%.3lf",nept[i]);
            strcat(buff,corner);
            strcat(buff,",");

            /* Northing value
               -----------------*/
            sprintf(corner,"%.3lf",nept[i+1]);
            strcat(buff,corner);
            strcat(buff,";\n");
            hlong=(long)strlen(buff);
            if (fwrite(buff,1,hlong,fp) != hlong)
            {
                sprintf(errmsg,"Error writing the %s to header file",corstr);
                c_errmsg(errmsg,"las2ndf-writ",&nonfatal);
            }
        }
    }

    /* Acquisition_date/time
       ------------------------*/
    key = DDCDAT;
    status = c_getbdf(&bptr,&key,(unsigned char *)date);
    if (status == E_FAIL)
        c_errmsg("Nonfatal error encountered","las2ndf-nonfatal",&nonfatal);

    key = DDCTIM;
    status = c_getbdf(&bptr,&key,(unsigned char *)time);
    if (status == E_FAIL)
        c_errmsg("Nonfatal error encountered","las2ndf-nonfatal",&nonfatal);

    if ((strncmp(date,"  ",2) == 0) || (strlen(date) <= 1))
        errflg = 0;
    if ((strncmp(time,"  ",2) == 0) || (strlen(time) <= 1))
        errflg = 0;

    if ((errflg) && (status != E_FAIL))
    {
        status=(dateconr(date));
        strncpy(time2,time,4);
        time2[4]='\0';
        p = (char *)strchr(time,':');
        strcpy(buff,"ACQUISITION_DATE/TIME=");
        strcat(buff,date);
        strcat(buff,"/");
        strcat(buff,time2);
        if (p != NULL)
        {
            p++;
            strcat(buff,p);
        } 
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the ACQUIS_DATE/TIME to header file",
                    "las2ndf-writ", &nonfatal);
    }

    /* Satellite name and instrument
       --------------------------------*/
    key = DDSRC;
    status = c_getbdf(&bptr,&key,(unsigned char *)sat);
    if (status == E_FAIL)
        c_errmsg("Nonfatal error encountered","las2ndf-nonfatal",&nonfatal);

    if ((strncmp(sat,"  ",2) == 0) || (strlen(sat) <= 1))
        errflg = 0;

    if ((errflg) && (status != E_FAIL))
    {
        strcpy(buff,"SATELLITE=");
        strcat(buff,sat);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the SATELLITE to header file",
                    "las2ndf-writ", &nonfatal);
    } 

    if ((strncmp(sensor,"  ",2) == 0) || (strlen(sensor) <= 1))
        errflg = 0;

    if ((errflg) && (senstat != E_FAIL))
    {
        strcpy(buff,"SATELLITE_INSTRUMENT=");
        hlong=(long)strlen(sensor);
        c_low2up(sensor,&hlong);
        strcat(buff,sensor);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the SAT._INSTRUMENT to header file",
                     "las2ndf-writ", &nonfatal);
    }

    if (ddr.valid[5] == 1)
    {
        strcpy(buff,"PIXEL_SPACING=");
        sprintf(ftemp,"%.4lf",ddr.pdist_x);
        strcat(buff,ftemp);
        strcat(buff,",");
        sprintf(ftemp,"%.4lf",ddr.pdist_y);
        strcat(buff,ftemp);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the PIXEL_SPACING to header file",
                    "las2ndf-writ", &nonfatal);
    }

    if (ddr.valid[6] == 1)
    {
        if (c_string2unit(ddr.proj_units) == E_FAIL)
        { 
            c_lasmsg(FALSE,"las2ndf-warn","Projection units %s are invalid", 
                    ddr.proj_units);
        } 
        strcpy(buff,"PIXEL_SPACING_UNITS=");
        strcat(buff,ddr.proj_units);
        strcat(buff,";\n");
        hlong=(long)strlen(buff);
        if (fwrite(buff,1,hlong,fp) != hlong)
            c_errmsg("Error writing the PIXEL_SPACING_UNITS to header file",
                    "las2ndf-writ",&nonfatal);
    }

    /* Number of bands in volume
       ----------------------------*/
    strcpy(buff,"NUMBER_OF_BANDS_IN_VOLUME=");
    sprintf(temp,"%d",ddr.nbands);
    *nb = ddr.nbands;
    strcat(buff,temp);
    strcat(buff,";\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing NUMBER_OF_BANDS to header file","las2ndf-writ",
                &nonfatal);

    /* Close files
       -----------------------------------*/
    strcpy(buff,"END_OF_HDR;\n");
    hlong=(long)strlen(buff);
    if (fwrite(buff,1,hlong,fp) != hlong)
        c_errmsg("Error writing END_OF_HDR marker to header file",
                "las2ndf-writ", &nonfatal);

    fclose(fp);

    return(E_SUCC);
}
