/*****************************************************************************
NAME: GET_SEMIAXIS

PURPOSE:  This will return the semi-major and minor axis based on the
          projection code and if needed the datum code.

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     01/98 T.Ratliff         C    Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"

int get_semiaxis
(
    double *maj,                      /* semi-major axis value        */
    double *min,                      /* semi-minor axis value        */
    struct DDR *ddr                   /* DDR record structure         */
)

{
    double flat = 0.0;              /* used to find Semi-minor axis */
    long pcode = 0;                 /* ddr->proj_code               */
    long dcode = 0;                 /* ddr->datum_code              */
    long zeroflg = 0;               /* set if ddr->proj_coef[0]==0  */
    long nonfatal = 1;              /* nonfatal error flag          */

    /* Initializing pcode and dcode
       -------------------------------*/
    pcode = ddr->proj_code;
    dcode = ddr->datum_code;

    /* If projection code is not valid, or if the projection
       code is either Geographic or State Plane
       --------------------------------------------------------*/
    if ((ddr->valid[3] != VALID) || (pcode == 0) || (pcode == 1) ||
            (pcode == 2))
        zeroflg = 1;
    else if (((pcode >= 3) && (pcode <= 9)) || (pcode == 20) || (pcode == 22) ||
            (pcode == 23)) 
    {
        if (ddr->proj_coef[0] == 0)
            zeroflg = 1;
        else
        {
            *maj = ddr->proj_coef[0];
            if (ddr->proj_coef[1] < 1)
            {
                flat = (1 - sqrt(1 - ddr->proj_coef[1]));
                *min = ddr->proj_coef[0] * (1 - flat);
            }
            else if (ddr->proj_coef[1] == 1)
            {
                *min = ddr->proj_coef[0];
            }
            else if (ddr->proj_coef[1] > 1)
            {
                *min = ddr->proj_coef[1];
            }
        }
    }
    else if (((pcode >= 10) && (pcode <= 19)) || (pcode == 21) ||
            ((pcode >= 24) && (pcode <= 30)))
    {
        if (ddr->proj_coef[0] == 0)
            zeroflg = 1;
        else
        {
            *maj = ddr->proj_coef[0];
            *min = ddr->proj_coef[0];
        }
    }

    if (zeroflg)
    {
        switch(dcode)
        {
            case 0:
                *maj = 6378206.4;
                *min = 6356583.8;
                break;
            case 1:
                *maj = 6378249.145;
                *min = 6356514.86955;
                break;
            case 2:
                *maj = 6377397.155;
                *min = 6356078.96284;
                break;
            case 3:
                *maj = 6378157.5;
                *min = 6356772.2;
                break;
            case 4:
                *maj = 6378388.0;
                *min = 6356911.94613;
                break;
            case 5:
                *maj = 6378135.0;
                *min = 6356750.519915;
                break;
            case 6:
                *maj = 6377276.3452;
                *min = 6356075.4133;
                break;
            case 7:
                *maj = 6378145.0;
                *min = 6356759.769356;
                break;
            case 8:
                *maj = 6378137.0;
                *min = 6356752.31414;
                break;
            case 9:
                *maj = 6377563.396;
                *min = 6356256.91;
                break;
            case 10:
                *maj = 6377304.063;
                *min = 6356103.039;
                break;
            case 11:
                *maj = 6377340.189;
                *min = 6356034.448;
                break;
            case 12:
                *maj = 6376896.0;
                *min = 6355834.8467;
                break;
            case 13:
                *maj = 6378155.0;
                *min = 6356773.3205;
                break;
            case 14:
                *maj = 6378160.0;
                *min = 6356774.719;
                break;
            case 15:
                *maj = 6378245.0;
                *min = 6356863.0188;
                break;
            case 16:
                *maj = 6378270;
                *min = 6356794.343479;
                break;
            case 17:
                *maj = 6378166.0;
                *min = 6356784.283666;
                break;
            case 18:
                *maj = 6378150.0;
                *min = 6356768.337303;
                break;
            case 19:
                *maj = 6370997.0;
                *min = 6370997.0;
                break;
            default:
                c_errmsg("Invalid datum code","las2ndf-datum",&nonfatal);
                break;
        }
    }

    return(E_SUCC);
}
