#!/bin/csh
#
# FUNCTION:	lasbench.csh
#
# PURPOSE:	Generate a temporary PDF that will allow a user to run a LAS
#		function as a cron job.  The temporary PDF will be created in
#		the specified directory and the LAS process will be run from
#		that directory as well.


#---------------------
# Check the arguments.
#---------------------
if (($#argv != 1) && ($#argv != 2)) then
    echo 'usage: lasbench.csh  workingdir [rundir]'
    echo ''
    echo '       Create a temporary PDF that will allow a user to run the LAS'
    echo '       benchmark as a cron job.  The temporary PDF will be created in'
    echo '       workingdir and the LAS process will be run from that directory'
    echo '       as well.'
    echo ''
    echo '       If [rundir] is specified, a new environment pointing to that'
    echo '       run directory will be created.  If [rundir] is not specified,'
    echo '       the environment will not be altered prior to executing the'
    echo '       benchmark.'
    echo ''
    echo '       The temporary PDF will be removed as soon as the LAS process'
    echo '       begins.'
    exit 1
endif

#------------------------------------------------------------------------
# Set up the production LAS environment if it hasn't already been set up.
#------------------------------------------------------------------------
if ((! $?EDCSOFT) && (-r /etc/login) && ($#argv == 1)) then
    source /etc/login
    eval `$masterlog response comfile las72`
    unset masterlog
    unset response
    unset comfile
endif


#----------------------------------------------------------------------------
# If the user has specified an optional environment to run from, reconfigure
# the production LAS environment to point to the new environment.
#----------------------------------------------------------------------------
if ($#argv == 2) then
    setenv EDCSOFT $2
    source $2/env/las/assign/lasenv.csh
endif

#-------------------------------
# Create a unique log file name.
#-------------------------------
set tstamp = `date '+%H%M%S'`


#--------------------------------------------------------------------
# Change to the working directory, copy lasbench to local, and enter.
#--------------------------------------------------------------------
cd $1
rm -rf lasbench
cp -r $EDCSOFT/src/las/base/app/lasbench .
cd lasbench/testpdf

#--------------------------
# Generate a temporary PDF.
#--------------------------
cat << EOF > bmark_$tstamp.pdf
procedure bmark_$tstamp

body
!-----------------------------------------------------------------
! Submit into batch queue.  The first "batch" means a batch job.
! The second occurance indicates the name of the queue to be used.
!-----------------------------------------------------------------
testpdf |runtype=batch stdout="bmark_$tstamp.log"|
exit
end-proc
EOF


#---------------------------------------------
# Start LAS and submit the PDF as a batch job.
#---------------------------------------------
cat << EOJ | tae >& bmark_$tstamp.stdout
bmark_$tstamp
EOJ


#--------------------------
# Remove the temporary PDF.
#--------------------------
rm -f bmark_$tstamp.pdf

exit 0
