#!/bin/csh -f
#
# FUNCTION:     runtest.csh
#
# PURPOSE:      Create a temporary PDF to run the test suite for a single
#               LAS module.


#---------------------
# Check the arguments.
#---------------------
if ($#argv < 1) then
   echo ''
   echo 'usage: runtest.csh <module> [subdirectories]'
   echo ''
   echo '       Create a temporary PDF to run the test suite for a single'
   echo '       LAS module. Any subdirectories that have been specified'
   echo '       will be traversed and their test modules will be run for'
   echo '       this test. This script needs to be run from the application'
   echo '       directory. A test directory should reside in this'
   echo '       application directory. Any subdirectories should reside in'
   echo '       the test directory.'
   echo ''
   echo '       Any orig* files that are used for comparisons, are run'
   echo '       through the make_locals.pl script to change the xxxxx or'
   echo '       vvvvv substitutions into local directory paths for the'
   echo '       comparisons. These new files are output to local*. Any'
   echo '       application tests should provide the orig* files with xxxxx'
   echo '       (replaced with Unix path) or vvvvv (replaced with VAX path)'
   echo '       in place of any pathname.  Their tests should be written to'
   echo '       compare against the local* files instead of the orig* files,'
   echo '       since this script will run the orig* files through the'
   echo '       make_locals.pl script to create the local* files.'
   echo ''
   echo '       <module> is the name of module being tested. This is only'
   echo '       used in a message stating which module is being tested. This'
   echo '       is a required parameter.'
   echo ''
   echo '       Example: runtest.csh wavemerge'
   echo '       This runs test/test.pdf from the current directory.'
   echo ''
   echo '       [subdirectories] is the list of subdirectories to be tested'
   echo '       as a part of this test. The subdirectory names only need to'
   echo '       be a path from the test directory. These cannot be the full'
   echo '       pathname. The subdirectories will be traversed and the tests'
   echo '       in each subdirectory will be run. Each subdirectory needs to'
   echo '       have its own test.pdf. It is assumed that the main directory'
   echo '       does NOT have a test.pdf that needs to be run. This is an'
   echo '       optional parameter.'
   echo ''
   echo '       Example: runtest.csh l7hdf2las l0r l1r l1g l1g/old'
   echo '       This runs test/l0r/test.pdf, test/l1r/test.pdf,'
   echo '       test/l1g/test.pdf, and test/l1g/old/test.pdf from the'
   echo '       current directory.'
   echo ''
   echo '       The temporary PDF, local* files are removed upon completion.'
   exit 1
endif

echo ''

#-----------------------------------------------------------------------
# If not dealing with subdirectories ...
# Check for existence of test subdirectory and test/test.pdf.  If either
# does not exist, the current module has not been prepared for automated
# testing, so print a notice and exit.
#-----------------------------------------------------------------------
if ($#argv == 1) then
   # Set subdirs to contain the local subdir for later looping.  We will
   # change directories to the test subdir, thus the directory that we
   # want at that time is the local dirctory.
   set subdirs = (./)

   if (! -d test) then
      echo "Error: $1 not prepared for automatic testing. The test subdirectory"
      echo "       does not exist."
      exit 0
   else if (! -e test/test.pdf) then
      echo "Error: $1 not prepared for automatic testing. test.pdf does not"
      echo "       exist in the test subdirectory."
      exit 0
   endif

#-------------------------------------------------------------------------
# If dealing with subdirectories ...
# Check for the existence of a test subdirectory, then check for existence
# of each subdirectory under the test subdirectory and
# test/subdir/test.pdf.  If a subdirectory or its test.pdf file does
# not exist, the current module has not been prepared for automated
# testing, so print a notice and exit.
#-------------------------------------------------------------------------
else
   # Set subdirs to all the command-line arguments except the module argument
   set subdirs = ($argv[2-])
   set num_subdirs = $#subdirs

   if (! -d test) then
      echo "Error: $1 not prepared for automatic testing. The test subdirectory"
      echo "       does not exist."
      exit 0
   endif

   # Check each of the subdirectories
   foreach dir ($subdirs)
      if (! -d test/$dir) then
         echo "Error: $1 not prepared for automatic testing. The test/$dir"
         echo "       subdirectory does not exist."
         exit 0
      else if (! -e test/$dir/test.pdf) then
         echo "Error: $1 not prepared for automatic testing. test/$dir/test.pdf"
         echo "       does not exist."
         exit 0
      endif
   end
endif

#------------------
# Start testing ...
#------------------
echo "$1 being tested."
set curdir = `pwd`
cd test
setenv SHELL '/bin/csh -f'

#--------------------------------------------------------------------
# Loop through all the subdirectories and run test.pdf in each subdir
#--------------------------------------------------------------------
foreach dir ($subdirs)

   #-------------------------------------------------------------------
   # Change into the subdirectory, but save the current directory so we
   # can return.
   #-------------------------------------------------------------------
   set testdir = `pwd`
   cd $dir

   if ($#argv > 1) then
      echo "   Testing $dir ..."
   endif

   #-------------------------------
   # Create a unique log file name.
   #-------------------------------
   set tstamp = `date '+%H%M%S'`

   #---------------------------------------------------------------------------
   # Create local* files from orig* files using the current directory for
   # the path.  'xxxxx' in the orig* files is replaced with the current
   # unix directory path.
   # 'vvvvv' in the orig* file is replaced with the current
   # vax directory path.  Also change origdiffs file to localdiffs.
   #---------------------------------------------------------------------------
   echo '   Creating local* files with current path.'
   set currdir = `pwd`
   foreach origfile (orig*.diff orig*.prt orig*.ppf origdiffs*)
      make_local.pl $origfile $currdir $HOME
   end

   #-------------------------------------------------------------------------
   # Generate a temporary PDF.  If the TESTING_LAS_BUILD environment variable
   # is not defined, do a setlib to the current directory to allow for use
   # during development.
   #-------------------------------------------------------------------------
   if ($?TESTING_LAS_BUILD) then
# The following must be left-justified ...
cat << EOF > bmark_$tstamp.pdf
procedure bmark_$tstamp
body
test |stdout="bmark_$tstamp.log"|
exit
end-proc
EOF
   else
# The following must be left-justified ...
cat << EOF > bmark_$tstamp.pdf
procedure bmark_$tstamp
body
setlib ($curdir,*)
test |stdout="bmark_$tstamp.log"|
exit
end-proc
EOF
   endif

   #---------------------------------------------
   # Start LAS and submit the PDF as a batch job.
   #---------------------------------------------
# The following must be left-justified ...
cat << EOJ | tae >& bmark_$tstamp.stdout
bmark_$tstamp
EOJ

   #--------------------------
   # Remove the temporary PDF.
   #--------------------------
   rm -f bmark_$tstamp.pdf

   #-------------------------------------------------------------------------
   # If a "results" file exists, it contains a list of the discrepancies that
   # occurred during testing.  Otherwise, the module passed testing.
   #-------------------------------------------------------------------------
   if (-e results) then
      echo "$1 testing FAILED.  See the files 'results' and 'bmark*' in "
      echo "     $PWD."
      exit 1
   else
      if ($#argv > 1) then
         echo "   $1 testing PASSED for $dir."
      endif
      echo ''
   endif

   #----------------------------------------------------------------------
   # Remove the temporary bmark*, local*.diff, local*.prt, local*.ppf,
   # and localdiffs files that were created from the orig*.diff, orig*.prt,
   # orig*.ppf, and origdiffs files.
   #----------------------------------------------------------------------
   # first make sure a file matches every pattern to avoid annoying "no match"
   # messages from showing up
   echo junk > local$tstamp.diff
   echo junk > local$tstamp.prt
   echo junk > local$tstamp.ppf
   echo " " >> localdiffs
   foreach tempfile (bmark_$tstamp.log bmark_$tstamp.stdout local*.diff \
                     local*.prt local*.ppf localdiffs)
      rm -f $tempfile
   end

   #---------------------------------------
   # Change back to the previous directory.
   #---------------------------------------
   cd $testdir
end

echo "$1 testing completed successfully and PASSED for all tests."
exit 0
