#!/usr/local/bin/perl
#
# diff_ignore_lines
#
# This program will diff two input files, ignoring the set of differences
# passed in as arguments.  The list of lines must be in ascending order and
# in standard diff format (e.g., "2c2", "5,6d4", "11a10,11").
#
# Usage:   diff_ignore_lines.pl [<diff args>] <file1> <file2> <first_diff> \
#                               [... <last_diff>]
#
# Example: diff_ignore_lines.pl old_results new_results 2c2 5,6d4 13,14c13,14

# Check for valid input arguments.  Must have the two input files
# and at least one difference to ignore.  Pass any arguments beginning with
# "-" to the actual diff command.
my $num_args = 0;
my @diff_arg;
my $quiet_flag = "";
while ($ARGV[$num_args] =~ /^-/) # While the argument begins with "-"...
{
    # If the argument is "-q", we don't want to pass this to diff.  So
    # just note that this argument was input so we can generate the
    # appropriate output.  Otherwise, add the argument to the list to be
    # passed to diff.
    if ($ARGV[$num_args] eq "-q")
    {
        $quiet_flag = "-q";
    }
    elsif ($ARGV[$num_args] =~ /q/)
    {
        $quiet_flag = "-q";
        $ARGV[$num_args] =~ s/q//;
        push @diff_arg, $ARGV[$num_args];
    }
    else
    {
        push @diff_arg, $ARGV[$num_args];
    }
    $num_args++;
}
if ($#ARGV - $num_args < 2)
{
    print "Usage: diff_ignore_diffs.pl [<diff args>] <file1> <file2> ";
    print "<first_diff> [... <last_diff>]\n";
    exit 1;
}

# Verify the existence of the two input files.
my $file1 = $ARGV[$num_args];
my $file2 = $ARGV[$num_args+1];
if (!(-e $file1) || !(-e $file2))
{
    if (!(-e $file1))
    {
        print "Error: File $file1 does not exist.\n";
    }
    if (!(-e $file2))
    {
        print "Error: File $file2 does not exist.\n";
    }
    exit 1;
}

# The number of differences to ignore is the number of arguments, not
# counting the filenames and arguments for diff.
my $numdiffs = $#ARGV - $num_args - 1;

# Load the differences to ignore into an array.
my @in_file1_first_line;
my @ignore_diff = @ARGV[$num_args+2..$#ARGV];
my $num_lines;
for (my $next_arg = 0; $next_arg <= $#ignore_diff; $next_arg++)
{
    # Extract line numbers from string argument.
    ($in_file1_first_line[$next_arg], $num_lines) =
                                   extract_lines($ignore_diff[$next_arg]);
    if ($num_lines == -1)
    {
        print "Error: Argument $ignore_diff[$next_arg] not a valid format.\n";
        exit 1;
    }

    # Verify that differences are in ascending order.
    if (($next_arg > 0) && ($in_file1_first_line[$next_arg] <=
                            $in_file1_first_line[$next_arg-1]))
    {
        print "Error: Arguments not in strictly ascending order.\n";
        exit 1;
    }
}

# Diff the input files and store the results.
open diff_results, "diff @diff_arg $file1 $file2 |";

my $i;
my $current_diff;
my $in_diff_number = 0;
my $file1_first_line;
my $status = 0;
while (<diff_results>)          # Loop over all lines in diff_results.
{
    chomp;

    # Check that the diff results are good.  The first character of the
    # first line should be a number.  If not, print out entire results
    # (error diagnostics from diff).
    if (! /^[0-9]/)
    {
        print "$_\n";
        while (<diff_results>)
        {
            print;
        }
        $status = 1;
        last;
    }

    $current_diff = $_;

    # Extract line numbers from string argument.
    ($file1_first_line, $num_lines) = extract_lines($current_diff);

    # If this difference covers line numbers greater than the current
    # ignore describes, then move to the next ignore.
    while (($in_diff_number <= $#ignore_diff) &&
           ($file1_first_line > $in_file1_first_line[$in_diff_number]))
    {
        $in_diff_number++;
    }

    # If we still have diff lines to ignore, compare the current diff
    # with next input diff to ignore.
    if (($in_diff_number <= $#ignore_diff) &&
        ($current_diff eq $ignore_diff[$in_diff_number]))
    {
        # Ignore this diff, so swallow the associated diff lines and
        # prepare for next diff to ignore.
        for ($i=0; $i<$num_lines; $i++)
        {
            <diff_results>;
        }
        $in_diff_number++;
    }
    else
    {
        # This difference hasn't been ignored, so print out (if -q not input)
        # and set the return status to 1 (failure).
        if (! $quiet_flag)
        {
            print "$current_diff\n";
            for ($i=0; $i<$num_lines; $i++)
            {
                $next_line = <diff_results>;
                print $next_line;
            }
        }
        else
        {
            for ($i=0; $i<$num_lines; $i++)
            {
                <diff_results>;
            }
        }
        $status = 1;
    }
}

# Close pipe containing diff results.
close diff_results;
#$status = $? >> 8;

# Print diff response for -q argument, if applicable.
if ($status && $quiet_flag)
{
    print "Files $file1 and $file2 differ\n";
}

# Return the error status (0 = success, 1 = failure).
exit $status;


# extract_lines
#
# Determine the starting line for the first input file and the number of
# lines in the diff output based on the standard diff format (e.g., "2c2",
# "5,6d4", "11a10,11").
#
#  inputs:  diff format string
# outputs:  first line in first file
#           number of lines in diff printout (not counting format line)
#
#           Upon error, -1, -1 is returned.
#
# Example:
# 13,14c13,14
# < [informational] Program DSPDDR required 00:00:01 of CPU.; +
# < [informational] Routing file to user specified destination; +
# ---
# > [informational] Program DSPDDR required 00:00:00 of CPU.; +
# > [informational] Routing file to user specifieed destination; +
#
# Input:
# 13,14c13,14
#
# Output:
# 13, 5
#
sub extract_lines
{
    my $in_string = $_[0];
    my $file1_first_line;
    my $file1_last_line;
    my $file2_first_line;
    my $file2_last_line;

    my @line_number = split /([,adc])/, $in_string;

    # Extract lines while verifying proper diff format.
    # Must have at least 3 items (e.g., "3c4") and no more than 7
    # (e.g., "10,11c15,18").
    # First and last items must be integers (line numbers).
    if (($#line_number < 2) || ($#line_number > 6) ||
        ($line_number[0] !~ /^[0-9]+$/) ||
        ($line_number[$#line_number] !~ /^[0-9]+$/))
    {
        return (-1, -1);
    }

    $file1_first_line = $line_number[0];
    $file2_last_line = $line_number[$#line_number];

    # If the second item is a comma...
    if ($line_number[1] eq ",")
    {
        # Must have at least 5 items with an integer for the third item,
        # "c" or "d" for the fourth, and in integer for the fifth
        # (e.g., "3,4d4").
        if (($#line_number < 4) || ($line_number[2] !~ /^[0-9]+$/) ||
            ($line_number[3] !~ /^[cd]$/) || ($line_number[4] !~ /^[0-9]+$/))
        {
            return (-1, -1);
        }

        $file1_last_line = $line_number[2];
        $file2_first_line = $line_number[4];

        # If there are 7 items, the 6th must be a comma.
        if (($#line_number == 6) && ($line_number[5] ne ","))
        {
            return (-1, -1);
        }
    }
    else  # Second item not comma.
    {
        $file1_last_line = $file1_first_line;
        $file2_first_line = $line_number[2];

        # Second item must be "a" or "c" and third must be an integer
        # (e.g., "9a10,11").
        if (($line_number[1] !~ /^[ac]$/) || ($line_number[2] !~ /^[0-9]+$/))
        {
            return (-1, -1);
        }
        # If there are 5 items, the 4th must be a comma.
        if (($#line_number == 4) && ($line_number[3] ne ","))
        {
            return (-1, -1);
        }
    }

    # Compute the number of lines associated with this difference.  This
    # number does not include the format line itself.
    my $num_lines = 1;  # Always at least one line.
    if ($in_string =~ /c/)
    {
        $num_lines += 2; # One line for "change separator" ("---") and
                         # one line for file 2 change.
    }
    # Additional lines for each additional file 1 and file 2 line.
    $num_lines += $file1_last_line - $file1_first_line +
                  $file2_last_line - $file2_first_line;
    
    return ($file1_first_line, $num_lines);
}
