/******************************************************************************

NAME:	 LINEREPR	

PURPOSE: Repairs bad TM or MSS detector (line) of LANDSAT-D one detector
         at a time for an entire image.  

PROGRAM HISTORY:

  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          03-Nov-81    E. Harris     
   2          25-Nov-86    R. McKinney        EDC       
   3          01-Mar-90    K. Kunska          Conversion to 5.0   
   4          15-Mar-90    K. Kunska          Corrected a parameter name
   5	      09-Sep-92	   T. Mittan	      Modified include file to 
					      eliminate warnings on link
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get the user supplied parameters.
   Call FILSPEC to open the image files.
   Initialize variables.
   Allocate dynamic memory for the work buffers.
   Initialize and enable the processing message utility.
   Save the start line number for each 3-line segment needed for repairs.
   For each image and each band, 
      Perform line repairs.
      If at the beginning of 3-line segment, 
         Save first line in pbuf, and 
         Write it to the output image.
         If at the next to last image line, 
            Read the last line and 
            Write it to the output image.
         Else
            Skip over the bad line to read the next line.
            Save the next line in nbuf.
            Find the average the lines adjacent to the bad line (now located in 
            pbuf and nbuf). 
            Store the average in the output buffer.
            Write out the average from the output buffer.
            Write out the next line from the nbuf.
            If number of detectors = lineinc = 2, 
               the 3-line segments will overlap so that the next line of the 
               first segment becomes the first line of the following segment.
         End-if
      Else-If not at the beginning of a 3-line segment, 
         Copy the input line to the output image.
      End-If
   End-For
   Close the image files.
   Free the dynamic memory.
   Update the output image ddr.
   Update the history record.
   Print the completion message.

*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

struct  PARBLK  block;                 /* TAE parameter block     */
struct  DDR     ddr;                   /* ddr                     */

short   *inbuf;                        /* image input buffer      */
short   *outbuf;                       /* image output buffer     */

static  char    *prog_name = "LINEREPR";  /* program name            */
char    out_files[2][CMLEN];           /* output file names        */
struct FDESC *  fdescin[MAXIMG+1];             /* input file descriptors   */   struct FDESC *  fdescout;                      /* output file descriptor   */
long    min_max;                       /* minmax option flag       */
long    msgtim;                        /* message interval         */
long    num_outfiles;                  /* number of output files   */

extern  long    curent;                /* current entity           */
extern  long    curcat;                /* current category         */

static  long    one = 1;

long    window[MAXIMG][4];    /* input image dimensions (SL SS NL NS) */

int main
(void) /* linerepr */
{


char	hosin[MAXIMG][CMLEN], 		/* host name of the input image     */
	hosout[1][CMLEN],			/* host name of the output image    */
	taein[MAXIMG][CMLEN],		/* tae name the input image         */
	taeout[1][CMLEN];			/* tae name of the output image     */

static
double	amax[MAXBND],			/* maximum pixel value for band      */
	amin[MAXBND];			/* minimum pixel value for band      */

static
float	offset,				/* offset value                      */
	scale;				/* scaling factor                    */
static	
long	badlin,				/* starting bad line relative to the */
                                        /* size of the entire input image    */
	band,				/* current band                      */
	bands_per_img[MAXIMG],		/* number of bands for each image    */
	
	bline[4198], 			/* indexes for the beginnings of     */
					/* 3-line segments                   */
	curlin,				/* current line looping index        */
	detect2_flg,			/* detects lineinc = 2               */
	dtype,				/* data type of the input image      */
	
	first_line,			/* first line to plot                */
	i, j,				/* loop counters and array indexes   */
	img,				/* current image                     */
	img_flag[MAXIMG][2],		/* flags for window/band options     */
	img_limit,			/* maximum number of images          */
	inband,				/* current band number for eread     */
	iinline,			/* current line number for eread     */
	lineinc,			/* bad line increment (number of     */
					/* detectors                         */
	lindex, 			/* line index for repairs            */
	msg_flag,			/* flag for the processing message   */
	num_img_band[MAXIMG][MAXBND+1], /* band numbers for input image      */
	num_inimg,			/* number of input images            */
 	num_lines,			/* number of lines in the image      */
	num_pixels,			/* number of pixels per line         */
	outband,			/* current band number for ewrite    */
	outline,			/* current line number for ewrite    */
	outwin[1][4],			/* output image window specifications*/
	out_img_band[MAXIMG][MAXBND+1], /* band numbers for the output image */
	same_wnd,			/* same window flag                  */
	tot_inband,			/* total number of bands             */
	work_size;			/* size of the i/o and work buffers  */



short	*nbuf = NULL,				/* buffer holding the next line      */
	*pbuf = NULL;				/* buffer holding the previous line  */

/*
Call GETPAR to get the user supplied parameters.
------------------------------------------------
*/
getpar (taein, hosin, taeout, hosout, &dtype, bands_per_img, img_flag, 
        img_limit, num_img_band, &num_inimg, &same_wnd, &tot_inband, &badlin, 
        &lineinc);

/*
Call FILSPEC to open the image files.
-------------------------------------
*/
filspec (hosin, hosout[0], num_inimg, num_img_band, bands_per_img, dtype, 
         out_img_band, tot_inband); 

/*
Initialize variables.
----------------------
*/
num_lines = window[0][NL];
num_pixels = window[0][NS];
first_line = window[0][SL];
msg_flag = 1;
outband = 0;
curlin = 1;
curent = 1;
curcat = 1;
dtype = EWORD;

/*
Allocate dynamic memory for the work buffers.
---------------------------------------------
*/
work_size = (long) (num_pixels * sizeof (short));
if ((inbuf = (short *) malloc (work_size)) == NULL ||
    (outbuf = (short *) malloc (work_size)) == NULL ||
    (pbuf = (short *) malloc (work_size)) == NULL ||
    (nbuf = (short *) malloc (work_size)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "linerepr-alloc",
            prog_name, out_files, &num_outfiles);

/*
Initialize and enable the processing message utility.
-----------------------------------------------------
*/
c_inittm ("LINEREPR", "Processed", "line", "band", "", &num_lines, &tot_inband,
           &msgtim, &msg_flag);
c_startm (); 

/*
Save the start line number for each 3-line segment needed for repairs.
-----------------------------------------------------------------------
*/
j = 0;
for (i = badlin; i <= first_line + num_lines - 1; i+=lineinc)
   {
   if ((i > first_line) && (i != first_line + num_lines- 1)) 
      {
      bline[j] = i - first_line;
      j++;
      }   
   }  

/*
For each image and each band, perform line repairs.
---------------------------------------------------
*/

for (img = 0; img < num_inimg; img++)
   {
   for (band = 0; band < bands_per_img[img]; band++)
      {
      curent = 1;
      curlin = 1;
      iinline = 1;
      lindex = 0;
      outband++;
      outline = 1;
      inband = num_img_band[img][band];
      detect2_flg = FALSE;
      while (curlin <= num_lines)
         {
         if (! detect2_flg)
            {
            curent++;
            c_eread (&fdescin[img], &inband, &iinline, inbuf, &one);
            iinline++;
            }
/*
If at the beginnin of 3-line segment, save first line in pbuf, 
and write it to the output image.
-------------------------------------------------------------------
*/
         if ((curlin == bline[lindex]) || (detect2_flg))
            {
            if (! detect2_flg)
               {
               c_pxcopy (inbuf, pbuf, &dtype, &num_pixels); 
               c_pxcopy (pbuf, outbuf, &dtype, &num_pixels);
               c_ewrite (&fdescout, &outband, &outline, outbuf, &one);
               outline++;
               }
/*
If at the next to last image line, read the last line and write it to the
output image.
--------------------------------------------------------------------------
*/
            if (iinline == num_lines) 
               {
               c_eread (&fdescin[img], &inband, &iinline, inbuf, &one);
               iinline++;
               c_pxcopy (inbuf, outbuf, &dtype, &num_pixels); 
               c_ewrite (&fdescout, &outband, &outline, outbuf, &one);
               outline++;
               break;
               }  
/*
Otherwise, skip over the bad line to read the next line.
---------------------------------------------------------
*/
            iinline++;
            curent = curent + 2;
            curlin = curlin + 2;
            c_eread (&fdescin[img], &inband, &iinline, inbuf, &one);
            iinline++;
/*
Save the next line in nbuf.
---------------------------
*/
            c_pxcopy (inbuf, nbuf, &dtype, &num_pixels);
/*
Find the average the lines adjacent to the bad line (now located in pbuf and
nbuf). Store the average in the output buffer.
------------------------------------------------------------------------------
*/       
            scale = offset = 0.5;
            c_pxadd (&num_pixels, pbuf, nbuf, outbuf, &scale, &scale,&offset, 
                     &dtype); 
/*
Write out the average from the output buffer.
-----------------------------------------------
*/
            c_ewrite (&fdescout, &outband, &outline, outbuf, &one);
            outline++;
/*
Write out the next line from the nbuf.
--------------------------------------
*/
            c_pxcopy (nbuf, outbuf, &dtype, &num_pixels); 
            c_ewrite (&fdescout, &outband, &outline, outbuf, &one);
            outline++;
            lindex++;
/*
If number of detectors = lineinc = 2, the 3-line segments will overlap.
so that the next line of the first segment becomes the first line of the
following segment.
---------------------------------------------------------------------------
*/
            if (lineinc == 2)        
               {
               c_pxcopy (nbuf, pbuf, &dtype, &num_pixels);
               if (curlin < num_lines) 
                  detect2_flg = TRUE;
               }  
            }
/*
If not at the beginning of a 3-line segment, copy the input line to the
output image.
------------------------------------------------------------------------
*/
         else                                     
            {
            c_pxcopy (inbuf, outbuf, &dtype, &num_pixels); 
            c_ewrite (&fdescout, &outband, &outline, outbuf, &one);
            outline++;
            }  
         if (! detect2_flg)
            curlin++;
         }  /* end while */

      curcat++;
      } /* end loop in bands */
   } /* end loop on images */

c_stoptm (); 

/* 
Close the image files.
----------------------
*/
for (i = 0; i < num_inimg; i++)
   c_eclose (&fdescin[i]);
c_eclose (&fdescout);

/*
Free the dynamic memory.
------------------------
*/
free (inbuf);
free (outbuf);
free (pbuf);
free (nbuf);

/*
Update the output image ddr.
-----------------------------
*/
if (min_max)
   {
   outwin[0][SL] = outwin[0][SS] = 1;
   outwin[0][NL] = num_lines;
   outwin[0][NS] = num_pixels;
   c_minmax (hosout, &num_inimg, outwin, out_img_band, &tot_inband, amin, 
             amax);
   c_svmnmx (hosout[0], &tot_inband, amin, amax, "YES");  
   }

/*
Update the history record.
---------------------------
*/
c_hstout (&block, hosout[0], hosin, &num_inimg, ""); 

/*
Print the completion message.
------------------------------
*/
c_complt (prog_name, out_files, &num_outfiles);

return EXIT_SUCCESS;
}
