/******************************************************************************
NAME    		        LIST    	    	            

PURPOSE    
  LIST generates either a printer or terminal listing of the pixel
  intensities of a LAS image.

FILE REFERENCES		        
  If printer output is requested, LIST creates an intermediate disk 
  file (list_<time stamp>.pr) to hold the output listing.  After 
  LIST finishes, this file is sent to the line printer and then is 
  automatically deleted. 
   
ERROR CONDITIONS
                terminal type is not supported by TAE
                Error opening temporary file for fileutil
                Error freeing dynamic memory
                Error closing temporary file for fileutil
		Error retrieving user name from the host file

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
S. Ross		Feb. 1984	Version 1.0
N. Deodhar/SAR	Sep. 1987	Version 3.1 - added loginit.for

K. Zanter	Jan. 1986       CIPS conversion to UNIX
K. Gacke        Sept 1986	LAS updates -- processing message utility.
				Modified calls to inlas, getin, and gettyp.  
				Removed calls to CM routines.
B. Davis	Jun. 1988	Conversion to LAS 5.0.
B. Davis	Jan. 1989	Fixed bug in processing message handling.
B. Davis	Mar. 1989	Removed reference to fileutil in error message
				of cleanup.c.
T. Mittan       Feb. 1992       Changed MAXLINES_PER_PAGE so that printer
                                does not print only one line on a page
T. Mittan	Sept 1992	Added extern to curent to eliminate warnings
				on link
S. Nelson	Aug. 1995	Initialized the variables PGTOT and DTYPE in
				LIST.  Initialized the variables REM in
				subroutines TERM_BYTE, TERM_LONG, TERM_REAL,
				and TERM_WORD.  The old operating system
				initialized these variables to zero.  This
				did not occur with the new system.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   LIST must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM (FLOW)
  initialize LAS and the terminal
  call getpar to obtain TAE and user parameters
  if print or disk file has been specified
	call calc_pages
	open a temporary work file
	call inittm and starttm
  for all images
  	for all bands
		call subroutine to write specific datatype to disk
		else
		call subroutine to write specific datatype to terminal
  close files and call stoptm and complt
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "protos.h"

#define MAXLINES_PER_PAGE 52
/* The following variable must be declared global for the processing message  */
extern long    curent;
#define pgnum curent

/* The following variable must be declared global for image I/O routines      */

struct FDESC * fdesc[MAXIMG+1];  /* image file descriptors used for image I/O */

int main
(void)
{

struct  PARBLK    vblock;           /* TAE parameter block                    */

FILE    *fp;			    /* file pointer to print file             */

static long bands[MAXIMG][MAXBND+1];/* band specification		      */
long    columns;	            /* number of columns for the CRT terminal */
long    dtype[MAXIMG];	    	    /* data type of image		      */
long 	flag = GEN2;		    /* general display message two            */
long    fyle;			    /* flag for output to file         	      */
long    i;		            /* loop index		 	      */
long    j;		            /* loop index			      */
long    k;			    /* loop index			      */
long    lines;	 		    /* number of lines for the CRT terminal   */
long    lnpr;			    /* flag for output to line printer        */
long    max_pplin[MAXIMG];	    /* max number of pixels per line          */
long    msgtim;			    /* elapsed time between display messages  */
long    nbands[MAXIMG];             /* number of bands for each input image   */
long    nfiles;			    /* number of output files passed to complt*/
long    nmrimg;			    /* number of input images		      */
long	one = 1;		    /* value of one                           */
long    pages_in_set[MAXIMG];	    /* number of pages in a set               */
long    pages_per_band[MAXIMG];	    /* total number of pages for each band    */
long    pgtot = 0;                  /* total number of pages to be printed    */
long    status;		            /* function return status code	      */
long	stdlun;			    /* fortran LUN to standard output         */
long    term;			    /* flag for output to terminal	      */
long    totbnd[MAXIMG];	            /* total number of bands to be processed  */
long    type;		            /* terminal type		              */
long    window[MAXIMG][4];          /* window specification		      */
long    zero = 0;		    /* value of zero			      */

char	filnm[CMLEN];		    /* output file name for FILE option       */
char    *hname;		            /* pointer to a single host name          */
char    hosin[MAXIMG][CMLEN];       /* descriptor for input host names        */
char    object[CMLEN];	    	    /* descriptor for object being processed  */
char    outfil[1][CMLEN];		    /* output file name                       */
char	outmsg[CMLEN];	    	    /* buffer to be outputted to terminal     */
char    pname[5];      		    /* program name for processing mesages    */
char    prognm[5];     		    /* program name for error messages        */
char    runtype[12];		    /* flag indicating current session type   */
char    taein[MAXIMG][CMLEN];       /* descriptor for input TAE names         */
char	text[CMLEN];		    /* text of processing message             */
char    *tname;	    	            /* pointer to a single TAE name           */
char    verb[CMLEN];	            /* descriptor for verb in processing msg  */


c_inlas(&vblock,&stdlun);	    /* initialize TAE                         */
t_init(&lines,&columns,&type);
if (type == T_NOTCRT)
   {
   c_errmsg("This terminal type is not supported by TAE","list-warn",&one);
   c_errmsg("Fatal error encountered","list-fatal",&zero);
   }
strcpy(verb,"Processed");
strcpy(object,"page");
strcpy(text," ");
strcpy(pname,"LIST");
strcpy(prognm,"list");

/* get user specified parameters and global variable values from TAE
  -----------------------------------------------------------------*/
getpar(&vblock,taein,hosin,&nmrimg,bands,nbands,totbnd,window,
       &term,&fyle,&lnpr,filnm,outfil[0],&msgtim,runtype);

/* Initialize data type
  --------------------*/
for (i=0; i<nmrimg; i++)
      dtype[i]=0;

/* calculate the total number of output pages and if the printer option
   is chosen, open the print file
   -----------------------------*/

calc_pages(hosin,&nmrimg,window,dtype,nbands,max_pplin,
	   pages_in_set,pages_per_band,&pgtot,&lnpr,runtype);
if ((lnpr == 1) || (fyle == 1))
   {
   fp = fopen(filnm,"w");
   if (fp == NULL)
      {
      c_errmsg("Error opening temporary file for fileutil","list-warn",&one);
      c_errmsg("Fatal error encountered","list-fatal",&zero);
      }
   pgnum = 1;
   }
if (term == 0)
   {
   c_inittm(pname,verb,object,text,text,&pgtot,&zero,&msgtim,&flag);
   c_startm();
   }

hname = hosin[0];
tname = taein[0];

/* loop through images, then bands, choosing processing subroutine by data    */
/* type and print option						      */

for (i = 0; i < nmrimg; i++)
    {
    for (j = 0; j < nbands[i]; j++)
        {
        if ((lnpr == 1) || (fyle == 1))
           {
           switch (dtype[i])
              {
	      case EBYTE:
		   print_byte(&vblock,fp,tname,&pages_per_band[i],
			      &pgtot,&max_pplin[i],&pages_in_set[i],window,
			      &i,&bands[i][j],filnm);
       	   	   break;

              case EWORD:
    		   print_word(&vblock,fp,tname,&pages_per_band[i],
			      &pgtot,&max_pplin[i],&pages_in_set[i],window,
			      &i,&bands[i][j],filnm);
		   break;

	      case ELONG:
		   print_long(&vblock,fp,tname,&pages_per_band[i],
			      &pgtot,&max_pplin[i],&pages_in_set[i],window,
			      &i,&bands[i][j],filnm);
		   break;

  	      case EREAL:
		   print_real(&vblock,fp,tname,&pages_per_band[i],
			      &pgtot,&max_pplin[i],&pages_in_set[i],window,
			      &i,&bands[i][j],filnm);
		   break;
              }
           }
        if (term == 1)
  	   {
           /* output the 3 heading lines */
	   sprintf(outmsg,"Image: \"%s\"  Band Number: %d",
                   tname,bands[i][j]);
	   c_pterm(outmsg,&zero);
           for (k = 0; k < columns; k++)
	       *(outmsg + k) = '-';
	   *(outmsg + columns) = '\0';
	   c_pterm(outmsg,&zero);
	   strcpy(outmsg," Line   Pixels");
	   c_pterm(outmsg,&zero);

           switch (dtype[i])
  	      {
	      case EBYTE:
		   term_byte(&vblock,fp,window,&i,&bands[i][j],
			     &nmrimg,&nbands[i],&lnpr,&fyle,filnm,&lines);
   		   break;

    	      case EWORD:
		   term_word(&vblock,fp,window,&i,&bands[i][j],
			     &nmrimg,&nbands[i],&lnpr,&fyle,filnm,&lines);
 		   break;

	      case ELONG:
		   term_long(&vblock,fp,window,&i,&bands[i][j],
			     &nmrimg,&nbands[i],&lnpr,&fyle,filnm,&lines);
		   break;

   	      case EREAL:
		   term_real(&vblock,fp,window,&i,&bands[i][j],
			     &nmrimg,&nbands[i],&lnpr,&fyle,filnm,&lines);
		   break;
	      }
           }
        }
    status = c_eclose(&fdesc[i]);
    if (status != E_SUCC)
       c_errmsg("Nonfatal error encountered","list-warn",&one);

    /* increment input image name array */
    hname += CMLEN;
    tname += CMLEN;

    } 

/* close the print and/or disk file */
if ((lnpr == 1) || (fyle == 1))
   {
   status = fclose(fp);
   if (status == EOF)
      {
      c_errmsg("Error closing temporary file for fileutil","list-warn",&one);
      c_errmsg("Fatal error encountered","list-fatal",&zero);
      }
   }

if (term == 0)
   c_stoptm();

if (fyle == 1)
   {
   nfiles = 1;
   }
else
   {
   nfiles = 0;
   strcpy(outfil[0]," ");
   }

/* write a completion message to the user */
c_complt(pname,outfil,&nfiles);

return EXIT_SUCCESS;
}
