/*******************************************************************************
NAME    		        PRINT_REAL    	    	            

PURPOSE    
  PRINT_REAL processes a band of a real image.  The output listing is written 
  to a file which is later routed to print and/or disk by fileutil.

FILE REFERENCES		        
  If printer output is requested, LIST creates an intermediate disk 
  file (list_<time stamp>.pr) to hold the output listing.  After 
  LIST finishes, this file is sent to the line printer and then is 
  automatically deleted. 
   
PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
S. Ross		Feb. 1984	Version 1.0
N. Deodhar/SAR	Sep. 1987	Version 3.1 - added loginit.for

K. Zanter	Jan. 1986       CIPS conversion to UNIX
K. Gacke        Sept 1986	LAS updates -- processing message utility.
				Modified calls to inlas, getin, and gettyp.  
				Removed calls to CM routines.
B. Davis	Jun. 1988	Conversion to LAS 5.0
T. Mittan       Feb. 1992       Changed MAXLINES_PER_PAGE so that printer
                                does not print only one line on a page

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   LIST must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM (FLOW)
  get window information
  allocate buffer space
  write headers
  for all pages of current band
     while line is less than MAX_LINES_PER_PAGE and less than number of lines
        read a line
	while sample is less than max samples per line (from calc_pages)
	   write current sample on output line and increment
	write newline character
     write headers for each new page
  free buffer space
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"

#define pgnum curent
#define MAXLINES_PER_PAGE 52
extern struct FDESC * fdesc[MAXIMG  + 1];

void print_real
(
    struct PARBLK *vblock,       /* pointer to TAE block	 	      */
    FILE *fp,		      /* pointer to print file 		              */
    char *taein,	      	      /* pointer to TAE name descriptor		      */
    long *pages_per_band,	  /* total number of pages for each band	      */
    long *pgtot,		      /* total number of output pages		      */
    long *max_pplin,	      /* max number of pixels per line		      */
    long *pages_in_set,	      /* number of pages in a set		      */
    long (*window)[4],	      /* window specification		 	      */
    long *curimg,	              /* index to current image 		      */
    long *bndnum,		      /* current band number		   	      */
    char *filnm	      	      /* pointer to temporary print file name	      */
)

{
extern long curent;

register long   curpage;      /* current page number of set		      */
register long   curr_samp;    /* current sample number			      */
long            i;	      /* loop index 				      */
long 	        in_line;      /* line to read from input image	     	      */
register long   in_samp_loc;  /* sample location from input image	      */
long            k;	      /* loop index				      */
long            last_samp;    /* position of last sample 		      */
register long   line_num;     /* current line number			      */
long            nl;	      /* number of lines			      */
long            ns;	      /* number of samples			      */
long            one = 1;      /* value of one				      */
long            out_lines;    /* number of lines written to the output page   */
register long   out_samps;    /* number of samples written to the output line */
long            sl;	      /* starting line				      */
long            ss;	      /* starting sample			      */
long            status;	      /* function return status code		      */
long            tmp;	      /* temporary used for output of annotation line */

register float  *bufptr;      /* pointer to input buffer		      */
register float  *ptr;	      /* pointer to sample to be processed	      */
		
char prognm[5]; 	      /* program name for error messages              */

strcpy(prognm,"list");
sl = window[*curimg][0];
ss = window[*curimg][1];
nl = window[*curimg][2];
ns = window[*curimg][3];
last_samp = ss + ns - 1;

/* allocate buffer space for image lines */
bufptr = (float *) malloc(ns * 4);
if (bufptr == NULL)
   {
   c_errmsg("Unable to allocate enough buffer space","list-nospace",&one);
   cleanup(vblock,fp,filnm,&one,&one);
   }

/* initialize local variables */
in_line = 1;
line_num = sl;
in_samp_loc = 0;
curpage = 1;

/* output standard header */
fprintf (fp,"\fImage: \"%s\"  Band Number: %d\n",taein,*bndnum);

/* output first annotation line */
tmp = ss;
out_samps = 1;
fprintf (fp,"          ");
while ((out_samps <= (*max_pplin)) && (tmp <= last_samp))
      {
      fprintf (fp, "       %5d ",tmp);
      tmp++;
      out_samps++;
      }
fprintf (fp,"\n");

/* process each output page for this band */
for (i = 0; i < (*pages_per_band); i++)
    {
    out_lines = 0;
    while ((out_lines < MAXLINES_PER_PAGE) && (in_line <= nl))
          {
          status = c_eread(&fdesc[*curimg],bndnum,&in_line,bufptr,&one);
	  if (status != ns)
	     {
	     c_errmsg("Error reading input file","list-read",&one);
	     cleanup(vblock,fp,filnm,&one,&one);
	     }

          ptr = bufptr + in_samp_loc;
          curr_samp = in_samp_loc + 1;
          out_samps = 1;
 
          fprintf (fp,"  %5d   ",line_num);
  	  while ((out_samps <= (*max_pplin)) && (curr_samp <= ns))
	      {
	      fprintf (fp,"%12.5e ",*ptr);
              ptr++;
   	      curr_samp++;
              out_samps++;
              }
          fprintf (fp,"\n");
          out_lines++;
          in_line++;
          line_num++;
	  }
  
    if (in_line > nl ) 
       {
       in_line = 1;
       line_num = sl;
       }

    if (pgnum < *pgtot)
       pgnum++;
    curpage++;
    if (curpage > (*pages_in_set))
       {
       curpage = 1;
       in_samp_loc = in_samp_loc + (*max_pplin);
       }

/*  output header and annotation for next page */
    if (i+1 < (*pages_per_band))
       {
       fprintf (fp,"\fImage: \"%s\"  Band Number: %d\n",taein,*bndnum);
       tmp = in_samp_loc + ss;
       k = 1;
       fprintf (fp,"          ");
       while ((k <= (*max_pplin)) && (tmp <= last_samp))
             {
    	     fprintf (fp,"       %5d ",tmp);
  	     tmp++;
             k++;
             }
        fprintf (fp,"\n");
        }
    }
free (bufptr);
return;
}
