/*****************************************************************************
NAME:   		       TERMINATE

PURPOSE   To update the ddr, close the image files and free the buffer 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
1.0	   K. Kimball      8/91		EDC             Create smaller modules
                                                        for routine proces by
                                                        separting the closing
							tasks.
2.0 	   T. Mittan	   9/92		EDC		Modified so that a blank
							line is no longer output
							with the completion
							message in update mode.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TERMINATE must be run as part of LNFILL.

PROJECT			LAS

ALGORITHM:


ALGORITHM REFERENCES		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[];   /* Image file descriptors used for image I/O */
extern struct GDESC * gdesc;     /* Group descriptor used for image I/O       */

extern long curcat;		/* Current processing band		     */
extern long curent;		/* Current processing line		     */
extern long offset1[];          /* Offsets into "thebuf" for input bands     */
extern long offset2[];          /* Offsets into "thebuf" for output bands    */

void terminate
(
    struct PARBLK *vblock,	/* TAE parameter block			     */
    char hosin[][CMLEN],	/* Host names of input images		     */
    char hosout[][CMLEN],	/* Host name of output image		     */
    char *minmax,	/* Min max flag                              */
    char taeout[][CMLEN],	/* LAS name of output image                  */
    double *maxval,  	/* Max pixel value array                     */
    double *minval,  	/* Min pixel value array                     */
    long (*bands)[MAXBND+1],
    long *nbands,	/* Number of bands per input image           */
    long *nmrimg,	/* Number of input images                    */
    long *totbnd,	/* Total number of bands to be processed     */
    long (*window)[4],	/* Window specification of input images      */
    short *update	/* True when the input image is to be updated*/
)


{

double  line_inc;		/* Number of pixels of the input image that
				   an output pixel represents in the line
				   direction				     */
double  samp_inc;		/* Number of pixels of the input image that
				   an output pixel represents in the sample
				   direction				     */

long    band;			/* Iterates through the bands in the input 
                                   images                                    */
long    image;			/* Iterates through all the input images     */
long    num_image = 1;          /* Number of images to close when not in 
                                   update                                    */
long    one = 1;		/* Address containing value of one           */
long	status;			/* Function return status code               */
long    tmp_window[1][4];	/* Window specification of the output image  */
long    tmp_bands[2][MAXBND+1]; /* actual # of each band in output image     */
long    tmp_nbands[1];          /* # of bands in the output image            */

long    upddr_flag;	        /* Flag for update DDR			     */
long 	zero = 0;		/* constant of zero			     */


/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/

line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;


/********************************************************************
 * Important Note about calculations of Min and Max in this routine *
 ********************************************************************/

/* Due to the capability of updating input images, this code does
   some strange things to ensure the min and max are calculated
   correctly.  It is possible that one or more of the images to update
   already has a min and max calculated for it.  When the calculations
   are made to the image or images, the min and max will need to be
   calculated again.  

   However, there is no routine to calculate it automatically.  
   MINMAX is available, but it retrieves the min and max first if it 
   exists.  In the case of update, it may exist and it may be
   wrong.  In the case where a new image is being created, the
   min and max do not exist so they will be calculated.

   The case of update then needs a work around.  The work around
   developed is to call SVMNMX with the m_flag set to "NO".  This
   invalidates the min and max for a ddr.  Then MINMAX can be called
   for that image and the calculations will be performed rather
   than retrieved.  The call to MINMAX will automatically update the ddr.

   The other work around, considered but not used, is to call
   PXMNMX for each band of each image.  This would require more 
   coding, so it was not used.  The calculations are performed
   when the images are read a line at a time.  PXMNMX needs the 
   images read a band at a time to perform the min max calculations
   correctly.  

                                                                              */

   

if (*update)

   {
   /* Need  to call svmnmx separately for each image
      regardless of the value of minmax 
      ----------------------------------------------*/

   for (band = 0, image = 0; image < *nmrimg; image++)
      {
      c_svmnmx(hosin[image], &nbands[image], &minval[band], &maxval[band],
	       "NO");
      band += nbands[image];
      }

   /* Calculate the new min and max for each of the images if
      minmax is set to yes
      ----------------------------------------------------*/

   if (strcmp(minmax,"YES") == 0)
      c_minmax(hosin, nmrimg, window, bands, nbands, minval, maxval);

   
   /* Close all image files in the group
   -------------------------------------*/
   close_output(nmrimg);
   }
else
   {
   /* Update the ddr
      --------------*/

   status = c_upddr(vblock,hosin,hosout[0],nmrimg,window,bands,nbands,
		 &line_inc, &samp_inc,totbnd,&upddr_flag);

   if (status != E_SUCC)
      {
      close_output(nmrimg);
      c_clean(vblock,"lnfill",hosout,&one);
      }


   /* Determine if the min and max need to be calculated
      --------------------------------------------------*/

   if (strcmp(minmax,"YES") == 0)
      {
      /* c_minmax is written to update a set of input images, but we want
         to update only the output image.  Set up the necessary temporary
         data structures for the call.
      --------------------------------------------------------------------*/
      for (band = 0; band < *totbnd; band++)
         tmp_bands[0][band] = band + 1;
      tmp_bands[1][0]   = 0;                 /* bands must be zero terminated */
      tmp_nbands[0]     = *totbnd;
      tmp_window[0][SL] = 1;
      tmp_window[0][SS] = 1;
      tmp_window[0][NL] = window[0][NL];
      tmp_window[0][NS] = window[0][NS];
      c_minmax(hosout,&one,tmp_window,tmp_bands,tmp_nbands,minval,maxval);

      /* Save the minmax values
      -------------------------*/
      c_svmnmx(hosout[0],totbnd,minval,maxval,minmax);
      }

   /* Create associated history file
   ---------------------------------*/
   c_hstout(vblock, hosout[0], hosin, nmrimg, "");

   /* Close the output image
   -------------------------*/
   close_output(&num_image);
   }

/* Inform user of successful completion
---------------------------------------*/
if (*update)
   c_complt("LNFILL", taeout, &zero);
else
   c_complt("LNFILL", taeout, &one);

return;
}
