#include "lowcal.h"
#include "util.h"
#include "pixman.h"

void knear
(
    long iter,                  /* current iteration                          */
    long niter,     		/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of output images                    */
    long num_img_bands[][MAXBND+1],	/* band numbers for the input images  */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands		/* total number of bands                      */
)
/*
*****************************************************************************

NAME:	KNEAR

PURPOSE: Performs the k-nearest-neighbor window operation on the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-85    Y. Sheu,           non-ap conversion
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          corrected display of processing
                                             msg and selection of input band
   4	     16-Nov-89    K. Kunska          Fixed dynamic memory allocation
   5         28-Nov-89    K. Kunska          Allocated i/o buffers dynamically

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables.
   Allocate dynamic memory for input, output and work buffers.
   For input images
      Initialize and enable the processing message utility.
      Store the first two input lines in the buffer and and compute their sum.
      Replace the top lines of the output image with either the oirginal input 
      lines or 0's, depending on the iteration number.
      Process the rest of the image by reading the remaining input lines. 
      Copy the sliding window pixels values to array.
      Compute the absolute differences between the center pixel and the 
      neighbor pixels.
      Store the results in count array.
      Call sort2 routine to sort the count and array buffers in parallel. 
      Find the mean of the kn values having the lowest absolute differences 
      while excluding the center pixel itself.
      Store the final results in the work buffer.
      Copy the work buffer to the output image .
      Replace the leftmost and rightmost pixel values by the original input 
      values or 0's depending on the iteration number.
      Replace the bottom lines of the output image with either the oirginal 
      input lines or 0's, depending on the iteration number.
   Free the dynamic memory.

*****************************************************************************
*/


{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


float	array[226], 		/* box pixels in one dimension                */
	*buffer,		/* I/O buffer for input image                 */
	*bufrow[MXLSIZ],	/* pointers to buffer rows                    */
	count[226],		/* abs differences between the central pixel  */
				/* and other pixels in the box                */
	offset,			
	scale = 1.0,		/* scale factor for arithmetic operations     */
	tsum,                   /* sum of kkn values of "array"               */
	*work = NULL,		/* I/O work data buffer                       */
	zfill = 0.0;  		/* fill value                                 */

long	buf1_siz,		/* buffer size for 1-dimensional arrays       */
        buf2_siz,		/* buffer size for 2-dimensional arrays       */
	cpix,			/* center pixel of the first window in row    */
	cur_band,		/* current band number                        */
	img,			/* current image being processed              */
	in_band,		/* input band for eread                       */
	in_line,		/* starting line for eread                    */
	i, in, ip, ir,		/* loop counter and array index               */
	intyp,			/* input data type                            */
	ipend,			/* icend + 1                                  */
	irow,			/* current line pointer for circular buffer   */
	jj,			/* loop counter and array index               */
	k,			/* index into array "array"                   */
	kkn,			/* kn + 1                                     */
	line, m,		/* line counters for the processing loops     */
	maxpix,			/* maximum number of pixels in a buffer line  */
	mid_col,		/* middle of columns (local window)           */
	mid_row, 		/* middle of rows (local window)              */
	msg_flag = 1,           /* flag for the processing message            */
	nbox,			/* column difference between the local window */
				/* and the input image                        */
	nsize,			/* size of the local window                   */
	num_cols,		/* number of columns in the local window      */
	num_lines,		/* number of lines                            */
	num_pixels,		/* number of pixels                           */
	num_rows,		/* number of rows in the local window         */
	out_band,		/* band number for the output image           */
	out_line,               /* line number for the output image           */
	rsave;			/* pointer to mid_row                         */
	

/*
Initialize the local variables
------------------------------
*/
num_pixels = inwnd[0][NS];
num_lines = inwnd[0][NL];
num_cols = iwsize[NP];
num_rows = iwsize[NR];
mid_col = num_cols / 2;
mid_row = num_rows / 2;
cpix = mid_col;
ipend = num_pixels - mid_col;
nbox = num_pixels - num_cols + 1;
kkn = kn + 1;
nsize = num_cols * num_rows;
out_band = 0;
intyp = EREAL;

/*
Allocate dynamic memory for input, output and work buffers.
------------------------------------------------------------
*/
buf1_siz = (long) ((num_pixels + 1) * sizeof(float));
buf2_siz = (long) (MXLSIZ * (num_pixels + 1) * sizeof(float));
if ((buffer = (float *) malloc(buf2_siz)) == NULL ||
   (work = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "knear-alloc",
               &fatal_err);

maxpix = num_pixels + 1;
for (i = 0, m = 0; i < MXLSIZ; i++, m += maxpix)
   bufrow[i] = &buffer[m];

buf1_siz = (long) (num_pixels * sizeof (float));
if ((inbuf = (float *) malloc(buf1_siz)) == NULL ||
   (outbuf = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);


/*
Initialize and enable the processing message utility
-----------------------------------------------------
*/
printf("\nLOWCAL   Iteration %3d of %3d\n", iter, niter);
c_inittm ("LOWCAL", "Processed", "line", "band", " ", &num_lines,
           &tot_inbands, &msgtim, &msg_flag);
curcat = 0;

/* 
Perform the k-nearest-neighbor window operations on all input images
---------------------------------------------------------------------
*/

for (img = 0; img < num_img; img++)
   {
   for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {
      curent = 1;
      curcat++;
      out_band++;
      out_line = 1;
      if (iter == 1)
         in_band = num_img_bands[img][cur_band];
      else
         in_band = out_img_bands[img][cur_band];
      c_pxfill (bufrow[zero], &intyp, &num_pixels, &zfill);

      c_startm();

/*
Store the first few input lines in the buffer and and compute their sum.
-------------------------------------------------------------------
*/
      irow = 0;
      for (line = 1; line < num_rows; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);
         irow++;
         }


/*
Replace the top lines of the output image with either the oirginal input 
lines or 0's, depending on the iteration number.
---------------------------------------------------
*/
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[line], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         }

/*
Process the rest of the image by reading the remaining input lines. 
--------------------------------------------------------------------
*/
      rsave = mid_row;
      for (line = num_rows; line <= num_lines; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);

/*
Copy the sliding window pixels values to array.
-----------------------------------------------
*/
         for (ip = 0; ip < nbox; ip++)
            {
            k = 0;
            for (ir = 0; ir < num_rows; ir++)
               {
               c_pxcopy (&bufrow[ir][ip], &array[k], &intyp, &num_cols); 
               k = k + num_cols;
               }

/*
Compute the absolute differences between the center pixel and the neighbor 
pixels.
Store the results in count array.
--------------------------------------------------------------------------
*/
            offset = -1.0 * bufrow[rsave][ip+mid_col];
            c_pxline (array, count, &nsize, &intyp, &scale, &offset); 
            for (jj = 0; jj < nsize; jj++)
               count[jj] = abs(count[jj]);

/*
Call sort2 routine to sort the count and array buffers in parallel. 
--------------------------------------------------------------------- 
*/
            sort2 (count, array, nsize);

/*
Find the mean of the kn values having the lowest absolute differences while 
excluding the center pixel itself.
Store the final results in the work buffer.
----------------------------------------------------------------------------
*/
            tsum = 0.0;
            for (in = 1; in < kkn; in++)
               tsum = tsum + array[in];
            work[ip] = tsum / (float) kn;
            }

/*
Copy the work buffer to the output image .
------------------------------------------
*/
            c_pxcopy (work, &outbuf[cpix], &intyp, &nbox); 

/*
Replace the leftmost and rightmost pixel values by the original input 
values or 0's depending on the iteration number.
---------------------------------------------------------------------
*/
            if (iter  <  niter) 
               {
               c_pxcopy (bufrow[rsave], outbuf, &intyp, &mid_col);
               c_pxcopy (&bufrow[rsave][ipend], &outbuf[ipend], &intyp, 
                         &mid_col);
               }
            else
               {
               c_pxfill (outbuf, &intyp, &mid_col, &zfill); 
               c_pxfill (&outbuf[ipend], &intyp, &mid_col, &zfill); 
               }
            c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
            out_line++;
            irow = (irow+1) % num_rows;
            rsave = (rsave+1) % num_rows;
      }

/*
Replace the bottom lines of the output image with either the oirginal 
input lines or 0's, depending on the iteration number.
---------------------------------------------------------------------
*/
      irow = rsave;
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[irow], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill); 
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;   
         irow = (irow+1) % num_rows;
         }

      c_stoptm();
      } /* bands loop */
   } /* image loop */

/*
Free the dynamic memory.
------------------------
*/
free(buffer);
free(work);
}
