/*******************************************************************************

NAME:	LOWCAL

PURPOSE: Performs a selected sliding-window operation on an input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          1-Jun-84    J. Bay, SAR
   2         23-Sep-85    Y. Sheu            non-ap conversion    
   3         13-Jan-89    K. Kunska          conversion to 5.0
   4         17-Apr-89    K. Kunska          fixed minmax option, list of 
                                             output images, phrasing of 
                                             error messages, selection of
                                             input bands in all subcommands
   5         10-Aug-89    J. Marmon          Changes for the Pn9050
   6         16-Nov-89    K. Kunska          Fixed the dynamic memory
                                             allocation for all subcommands
   7         05-Dec-89    K. Kunska          removed redundant params passed to
                                             subcommand routines
   8         03-Apr-90    K. Kunska          Fixed min_max problem
   9	     23-Sep-92	  T. Mittan	     Modified include file to 
					     eliminate warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to initialize tae and retrieve user-supplied parameter values.
   For the first and last iterations, input and output files are the original
        user-specified input and output files.
   When more than one iteration is performed, the intermediate results from one
        iteration are placed in a temporary file that then serves as input to
        the next iteration.
   Call FILSPEC to open the input and output files for processing.
   Process box filters.
   Close the files used in this iteration.
   If not the first iteration, then delete the temporary file which served as
        the input to the iteration just completed.
   Generate the history file and print the completion message.

******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

#define NR      0                       /* number of lines in local window  */
#define NP      1                       /* number of pixels in local window */

#define MXLSIZ  15                      /* max number of lines for I/O buffer */
#define MHALF   MXLSIZ/2                /* half of max number of lines        */

struct  PARBLK  block;                 /* TAE parameter block     */
struct  DDR     ddr;                   /* ddr                     */
FILE    *fd;                           /* output file descriptor  */

float   *inbuf;                        /* image input buffer      */
float   *outbuf;                       /* image output buffer     */
char    out_files[2][CMLEN];           /* output file names       */

struct FDESC *  fdescin[MAXIMG+1];             /* input file descriptors  */
struct FDESC *  fdescout;                      /* output file descriptor  */

long    min_max;                       /* minmax option flag      */
long    msgtim;                        /* message interval        */
long    num_outfiles;                  /* number of output files  */

extern long     curent;                        /* current entity           */
extern long     curcat;                        /* current category         */

static  long    fatal_err = -1;

static  long    zero = 0;
static  long    one = 1;
static  char    *prog_name = "LOWCAL"; /* program name            */

float   ccol[MXLSIZ];           /* column weights for separable convolution
*/
float   crow[MXLSIZ];           /* row weights for separable convolution
*/

long    iopt;                   /* option numbers (1 - 13)            */
long    inwnd[MAXIMG][4];       /* input image dimensions (SL SS NL NS)*/
long    iwsize[2];              /* dimensions of local window (NR, NP) */
long    jns;                    /* number of neighbors for selective average
*/
long    kn;                     /* number of neighbors for KNN option
*/
long    mthre;                  /* treshold value for quantized mode option
*/
long    nbins;                  /* number of bins for quantized mode option
*/
float   thresh;                 /* treshold value for selective average option
*/


int main
(void) /* lowcal */
{

char 
	hos_his[MAXIMG][CMLEN], /* holds the original host name for hstout   */
	hos_in[MAXIMG][CMLEN], 	/* input image host name                     */
	tae_in[MAXIMG][CMLEN];  /* input image tae name                      */
char 
	hos_out[1][CMLEN], 	/* output image host name                    */
	(*inhos)[CMLEN], 	/* temporary input image file host name	     */
	(*intae)[CMLEN], 	/* temporary input image file tae name       */
	othos[CMLEN],		/* temporary output image file host name     */
	ottae[CMLEN],           /* temporary output image file tae name      */
	out_type[1][5],            /* output data type; valid values:           */
				/* (SAME, BYTE, I*2, I*4, R*4)               */
	tae_out[1][CMLEN], 	/* output image tae name                     */
	temp_in[CMLEN],         /* temporary input file name holder          */
	temp_out[CMLEN];    	/* temporary output file name holder         */

double	amax[MAXBND],           /* maximum pixel values                      */
	amin[MAXBND];           /* minimum pixel values                      */

static  long 
	num_img_band[MAXIMG][MAXBND+1], /* band numbers for each image       */
	img_flag[MAXIMG][2];	/* specifies image/band options              */

long	bands_per_img[MAXIMG],	/* number of bands per image   		     */
        dtype,			/* output image data type - numeric value    */
	i, ii, j,		/* loop counter and array index              */
	img_limit,		/* maximum number of images                  */
	iter,                   /* current iteration                         */
	niter,			/* total number of iterations                */
	num_inimg,		/* number of input images                    */
/*	num_rows,l */		/* number of rows in the local window        */
	out_bands[2],		/* zero terminated array with the number of  */
                                /* bands for the output image                */
	out_img_band[1][MAXBND+1],	/* band numbers of the output image  */
	outwin[1][4],		/* output window specifications              */
	same_wnd,		/* same window size: 1=Y, 0=N                */
	stat,			/* return status value                       */
	temp_bands[MAXIMG],	/* number of bands for temporary image       */
	tot_inband;		/* total number of bands to be processed     */

img_limit = MAXIMG;

/*
Initialize tae and retrieve user-supplied parameter values
-----------------------------------------------------------
*/
getpar (tae_in, hos_in, tae_out, hos_out, &num_inimg, num_img_band, 
        bands_per_img, &tot_inband, img_limit, &same_wnd, img_flag, &niter, 
        amin, amax, out_type);

for (i = 0; i < MAXIMG; i++)
   for (j = 0; j < CMLEN; j++)
      hos_his[i][j] = hos_in[i][j];

/*
Loop on iterations
-------------------------------
*/
for (iter = 1; iter <=  niter; iter++)
   {

/*
For the first iteration, the input file is the original, user-specified 
input file. If not the first iteration, then the input file is a temporary 
file created as the output of the previous iteration.
--------------------------------------------------------------------------
*/
   if (iter == 1) 
      {
      intae = tae_in;
      inhos = hos_in;
      }
   else
      {
      strcpy (temp_in, "LOCITER0_IMG");
      if ((iter - 1) % 2)
         temp_in[7] = '1';
      else
         temp_in[7] = '0';
      stat = c_mkimg (temp_in, "", intae[0], inhos[0], &zero);
      if (stat != E_SUCC)
         c_errmsg ("Problem in creating temporary image file", 
		    "lowcal-mkimg", &fatal_err);
      } 

/*
For the last iteration, the output file is the user-specified output file.
If not the last iteration, then the output file is a temporary file which will
serve as input to the next iteration.
------------------------------------------------------------------------------
*/
   if (iter == niter) 
      {
      strcpy(ottae, tae_out[0]);
      strcpy(othos, hos_out[0]);
      }
   else
      {
      strcpy (temp_out, "LOCITER0_IMG");
      if (iter % 2)
         temp_out[7] = '1';
      else
         temp_out[7] = '0';
      stat = c_mkimg (temp_out, "", ottae, othos, &zero);
      if (stat != E_SUCC)
         c_errmsg ("Problem in creating temporary image file", 
		    "lowcal-mkimg", &fatal_err);
      }

/* 
Open the input and output files for processing.
------------------------------------------------
*/
if (iter == 1)
   filspec (inhos, othos, num_inimg, num_img_band, out_img_band, 
            bands_per_img, &tot_inband, out_type[0], &dtype);
else
   {
   temp_bands[0] = tot_inband;
   filspec (inhos, othos, one, out_img_band, out_img_band, temp_bands, 
            &tot_inband, out_type[0], &dtype);
   }

out_bands[0] = tot_inband;
out_bands[1] = 0;

/*
Process box filters
--------------------
*/
   if (iopt  ==  1  ||  iopt  ==  12) 
      {
      xmean (iter, niter, bands_per_img, num_inimg, num_img_band, out_img_band,
             tot_inband);
      }
   else if (iopt  ==  2  ||  iopt  ==  3) 
      {
      varian (iter, niter, bands_per_img, num_inimg, num_img_band, out_img_band,
              tot_inband);
      }
   else if (iopt  ==  4) 
      {
      quan (iter, niter, bands_per_img, num_inimg, num_img_band, out_img_band,
            amin, amax, tot_inband);
      }
   else if (iopt  ==  5  ||  iopt  ==  6) 
      {
      xminmax (iter, niter, bands_per_img, num_inimg, num_img_band, 
               out_img_band, tot_inband);
      }
   else if (iopt  ==  7  ||  iopt  ==  8) 
      {
      median (iter, niter, bands_per_img, num_inimg, num_img_band, 
              out_img_band, tot_inband);
      }
   else if (iopt  ==  9) 
      {
      sharp (iter, niter, bands_per_img, num_inimg, num_img_band, 
             out_img_band, tot_inband);
      }
   else if (iopt  ==  10) 
      {
      knear (iter, niter, bands_per_img, num_inimg, num_img_band, 
             out_img_band, tot_inband);
      }
   else if (iopt  ==  11) 
      {
      selavg (iter, niter, bands_per_img, num_inimg, num_img_band, 
              out_img_band, tot_inband);
      }
   else if (iopt  ==  13) 
      {
      convol (iter, niter, bands_per_img, num_inimg, num_img_band,
              out_img_band, tot_inband);
      }

/*
Close the files used in this iteration
---------------------------------------
*/
   for (ii = 0; ii < num_inimg; ii++)
      c_eclose (&fdescin[ii]);
   c_eclose (&fdescout);

/*
If not the first iteration, then delete the temporary file which served as
the input to the iteration just completed
--------------------------------------------------------------------------
*/
   if (iter > 1) 
      c_delfil (inhos[0]);

   }  /* end of loop on iterations */

if (min_max)
   {
   outwin[0][SL] = outwin[0][SS] = 1;
   outwin[0][NL] = inwnd[0][NL];
   outwin[0][NS] = inwnd[0][NS];
   c_minmax (hos_out, &one, outwin, out_img_band, out_bands, amin, amax);
   c_svmnmx (hos_out[0], out_bands, amin, amax, "YES");
   }


/*
Generate the history file and print the completion message.
-----------------------------------------------------------
*/
c_hstout (&block, othos, hos_his, &num_inimg, " ");
c_complt (prog_name, out_files, &num_outfiles);

return EXIT_SUCCESS;

}
