#include "lowcal.h"
#include "util.h"
#include "pixman.h"

void median
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of input images                     */
    long num_img_bands[][MAXBND+1],	/* band numbers for the input images  */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands		/* total number of bands to be processed      */
)

/*
*****************************************************************************

NAME:	MEDIAN

PURPOSE: Performs the mean window operation on the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-85    Y. Sheu,           non-ap conversion
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          corrected display of processing
                                             msg and selection of input band
   4         16-Nov-89    K. Kunska          Fixed dynamic memory alocation
   5         28-Nov-89    K. Kunska          Allocated i/o buffers dynamically

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables.
   Allocate the dynamic memory for the input, output and work buffers.
   For input images
      Initialize and enable the processing message utility.
      Store the first few input lines in the buffer.
      Replace the top lines of the output image with either the oirginal input 
      lines or 0's, depending on the iteration number.
      Process the rest of the image by reading the remaining input lines, and 
      copying the current window pixel values to a buffer called array.
      Store the pixels from the first window in array.
      Continue with the rest of the input image.
      Call xmed to get the median for the current window.
      If the option is "median absolute deviation ", subtract the median value 
      from the array values. Store the results in array called diff. 
      Find the absolute values of diff elements.
      Call xmed to get the median of the differeces.
      Copy the work buffer to the output image .
      Replace the leftmost and rightmost pixel values by the original input 
      values or 0's depending on the iteration number.
      Replace the bottom lines of the output image with either the oirginal 
      input lines or 0's, depending on the iteration number.
   Free the dynamic memory.

*****************************************************************************
*/


{

long    fatal_err = -1;
long    one = 1;


float	*array = NULL, 	/* stores image data for one box              */
	*buffer,		/* I/O buffer for input image                 */
	*bufrow[MXLSIZ],	/* pointers to buffer rows                    */
	*diff = NULL,		/* difference array for MABS option           */
	offset,             /* offset value for arithmetic operations     */
	scale = 1.0,		/* scaling factor for arithmetic operations   */
	*work = NULL,		/* I/O work data buffer                       */
	zfill = 0.0;		/* fill value                                 */

long	buf1_siz,		/* buffer size for 1-dimensional arrays       */
	buf2_siz,		/* buffer size for 2-dimensional arrays       */
	cpix,			/* center pixel of the first window in row    */
	cur_band,		/* current band being processed               */
	i,			/* index into the "difference" array          */
	ic,			/* loop counter and array index               */
	in_band,                /* input band for eread                       */
	in_line,                /* starting line for eread                    */
	img,			/* current image being processed              */
	intyp,			/* input image data type                      */
	ip,			/* pixel loop counter                         */
	ipend,			/* icend + 1                                  */
	ir, m,			/* loop counters and array indexes            */
	irow,			/* current line pointer for circular buffer   */
	jj,      		/* loop counter and array index               */
	k, 			/* index into the "array" array               */
	line,			/* line counter for processing loops          */
	mid_col,		/* middle of columns (local window)           */
	mid_row,                /* middle of rows (local window)              */
	msg_flag = 1,		/* flag for the processing message            */
	nbox,			/* column difference between local window     */
				/* and the input image                        */
	ncc,			/* num_col - 1                                */
	nhalf,			/* half of local window size                  */
	nsize,			/* local window size                          */
	num_cols,		/* number of columns in the local window      */
	num_lines,		/* number of lines                            */
	num_pixels,		/* number of pixels                           */
	num_rows,		/* number of rows in the local window         */
	out_band,		/* band number for the output image           */
	out_line,		/* line number for the output image           */
	rsave;		 	/* saves the mid_row pointer                  */


/*
Initialize the local varibles
-----------------------------
*/
num_pixels = inwnd[0][NS];
num_lines = inwnd[0][NL];
num_cols = iwsize[NP];
num_rows = iwsize[NR];
mid_col = num_cols / 2;
mid_row = num_rows / 2;
cpix = mid_col;
ipend = num_pixels - mid_col;
ncc = num_cols - 1;
nsize = num_cols * num_rows;
nhalf = (nsize-1) / 2;
nbox = num_pixels - num_cols + 1;
out_band = 0;
intyp = EREAL;

/*
Allocate the dynamic memory for the input, output and work buffers.
--------------------------------------------------------------------
*/
buf1_siz = (long) ((num_pixels + 1) * sizeof(float));
buf2_siz = (long) (MXLSIZ * (num_pixels + 1) * sizeof(float));
if ((buffer = (float *) malloc(buf2_siz)) == NULL ||
   (array = (float *) malloc(buf1_siz)) == NULL ||
   (diff = (float *) malloc(buf1_siz)) == NULL ||
   (work = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "median-alloc",
              &fatal_err);

for (i = 0, m = 0; i < MXLSIZ; i++, m += num_pixels)
   bufrow[i] = &buffer[m];

buf1_siz = (long) (num_pixels * sizeof (float));
if ((inbuf = (float *) malloc(buf1_siz)) == NULL ||
   (outbuf = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);

/*
Initialize and enable the processing message utility
-------------------------------------------------------
*/
printf("\nLOWCAL   Iteration %3d of %3d\n", iter, niter);
c_inittm ("LOWCAL", "Processed", "line", "band", " ", &num_lines,
           &tot_inbands, &msgtim, &msg_flag);
curcat = 0;

/*
For each input image, perform the median-window operations
----------------------------------------------------------
*/
for (img = 0; img < num_img; img++)
   {
   for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {
      curcat++;
      curent = 1;
      out_band++;
      out_line = 1;
      if (iter == 1)
         in_band = num_img_bands[img][cur_band];
      else
         in_band = out_img_bands[img][cur_band];

      c_startm();

/*
Store the first few input lines in the buffer.
-----------------------------------------------
*/
   irow = 1;
   for (line = 1; line <= num_rows-1; line++)
     {
     curent++;
     in_line = line;
     c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
     c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);
     irow++;
     }


/*
Replace the top lines of the output image with either the oirginal input 
lines or 0's, depending on the iteration number.
------------------------------------------------------------------------
*/
   for (line = 1; line <= mid_row; line++)
      {
      if (iter < niter)
         c_pxcopy (bufrow[line], outbuf, &intyp, &num_pixels);
      else
         c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
      c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
      out_line++;
      }

/*
Process the rest of the image by reading the remaining input lines, and 
copying current window pixel values to a buffer called array.
------------------------------------------------------------------------
*/
   rsave = mid_row + 1;
   irow = 0;
   c_startm(); 
   for (line = num_rows; line <= num_lines; line++)
      {
      curent++;
      in_line = line;
      c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
      c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);

/*
Store the pixels from the first window in array.
------------------------------------------------
*/
      k = 0;
      for (ic = 0; ic < ncc; ic++)
         {
         for (ir = 0; ir < num_rows; ir++)
            {
            array[k] = bufrow[ir][ic];
            k++;
            }
         }
/*
Continue with the rest of the input image
-----------------------------------------
*/
      for (ip = ncc; ip < num_pixels; ip++)
         {
         for (ir = 0; ir < num_rows; ir++)
            {
            array[k] = bufrow[ir][ip];
            k++;
            }

/*
Call xmed to get the median for the current window.
----------------------------------------------------
*/

         work[ip] = xmed(array, nsize, nhalf);

/*
If the option is "median absolute deviation ", subtract the median value from 
the array values. Store the results in array called diff. 
Find the absolute values of diff elements.
Call xmed to get the median of the differeces.
-------------------------------------------------------------------------------
*/
         if (iopt == 8)
            {
            offset = -1.0 * work[ip];
            c_pxline (array, diff, &nsize, &intyp, &scale, &offset);
            for (jj = 0; jj < nsize; jj++)
               diff[jj] = abs(diff[jj]);
            work[ip] = xmed(diff, nsize, nhalf);
            }

         k = k % nsize;
         }

/*
Copy the work buffer to the output image .
------------------------------------------
*/
      c_pxcopy (&work[ncc], &outbuf[cpix], &intyp, &nbox);
      c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);

/*
Replace the leftmost and rightmost pixel values by the original input 
values or 0's depending on the iteration number.
-----------------------------------------------------------------------
*/
      if (iter  <  niter) 
         {
         c_pxcopy (bufrow[rsave], outbuf, &intyp, &mid_col);
         c_pxcopy (&bufrow[rsave][ipend], &outbuf[ipend], &intyp, &mid_col);
         }
      else
         {
         c_pxfill (outbuf, &intyp, &mid_col, &zfill);
         c_pxfill (&outbuf[ipend], &intyp, &mid_col, &zfill);
         }
      c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
      out_line++;
      rsave = (rsave+1) % num_rows;
      irow = (irow+1) % num_rows;
   }

/*
Replace the bottom lines of the output image with either the oirginal 
input lines or 0's, depending on the iteration number.
----------------------------------------------------------------------
*/
   irow = rsave;
   for (line = 0; line < mid_row; line++)
      {
      if (iter  <  niter)  
         c_pxcopy (bufrow[irow], outbuf, &intyp, &num_pixels);
      else
         c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
      c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
      out_line++;      
      irow = (irow+1) % num_rows;
      }

   c_stoptm(); 
      } /* band loop */
   } /* image loop */

/*
Free the dynamic memory
-----------------------
*/
free (buffer);
free (array);
free (diff);
free (work);
}
