#include "lowcal.h"
#include "util.h"
#include "pixman.h"
#include "lasmath.h"

void quan
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of images                           */
    long num_img_bands[][MAXBND+1],	/* band numbers for input images      */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    double omin[],			/* array of minimum pixel values             */
    double omax[],			/* array of maximum pixel values             */
    long tot_inbands
)

/*
*****************************************************************************

NAME:	QUAN

PURPOSE: Performms the quantized mode window operations on the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-85    Y. Sheu,           non-ap conversion
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          corrected display of processing
                                             msg and selection of input band
   4         16-Nov-89    K. Kunska          Fixed dynamic memory allocation
   5         28-Nov-89    K. Kunska          Allocated i/o buffers dynamically
   6         07-Dec-89    K. Kunska          Adjusted calculation of PVAL

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables.
   Allocate dynamic memory for input, output and work buffers.
   For input images
      Initialize and enable the processing message utility.
      Store the first few input lines in the buffer and scale them to histogram 
      the bins.
      Replace the top lines of the output image with either the oirginal input 
      lines or 0's, depending on the iteration number.
      Copy the leftmost and the rightmost pixels of the current line to save 
      buffer.
      Process the rest of the image by reading the remaining input lines, and 
      finding the quantized mode of the pixels in the sliding window.
      Copy the leftmost and the rightmost pixels of the current line to save 
      buffer.
      Scale the current line to the histogram bins.
      Initialize the histogram counts for the first sliding window.
      Compute the histogram counts for the rest of the image.
      Call the vabmxf routine to find the maximum value of counts (bin 
      occupancy).
      Verify that the mode is unique.
      Verify that the bin occupancy for the mode is grater than or equal to the 
      treshold.
      If the threshold test or the unique mode test fails, use the center of 
      the bins, else use the mode itself.
      Depending on the iteration number, replace the leftmost and rightmost 
      pixel values of the output image by the original input values.
      Copy the last row of the window to a save area, so that it  can be 
      copied to the output image 
      Replace the bottom lines of the output image with 0's if necessary.
      Copy the processing results to the output image.
      Replace the bottom lines of the output image with either the oirginal 
      input lines or 0's, depending on the iteration number.
   Free the dynamic memory.

*****************************************************************************
*/


{

long    fatal_err = -1;
long    one = 1;


float	count[1025],		/* holds number of elements in each bin      */
	offset,			/* offset value for arithmetic oper/transf   */
	pval[1026], 		/* bin values for quantization               */
	save[MXLSIZ][MXLSIZ-1], /* saves values at the edges                 */
	*rowsave = NULL,  		/* saves the last row of boxes               */
	*rowrow[MHALF],		/* pointers to rowsave rows                  */
	treshold,		/* treshold value                            */
	vmax,                   /* maximum of count values                   */
	wbin,			/* bin width ?                               */
	*work = NULL,	/* I/O work data buffer                      */
	zfill = 0.0;		/* fill value                                */

long	buf1_siz,		/* buffer size for 1-dimensional arrays      */
	buf2_siz,		/* buffer size for 2-dimnesional long array  */
	buf3_siz,		/* buffer size for 2-dimensional float array */
	*buffer,		/* I/O buffer for input image                */
	*bufrow[MXLSIZ],	/* pointers to buffer rows                   */
	cpix,			/* conter pixel of the first window in row   */	
	cur_band,		/* current band number                       */
	i,			/* loop counter                              */
	ii,			/* loop counter                              */
	img,			/* current image being processed             */
	in_band,		/* input band for eread                      */
	in_line,		/* starting line for eread                   */
	ind,			/* index into count array (histogram init.)  */
	ind1,			/* index into count array (histogram proces.)*/
	ind2,			/* index into count array (histogram proces.)*/
	index,			/* index of max value in count array         */
	indout,			/* index into the output buffer              */
	intyp,			/* input data type                           */
	ipend,			/* num_pixels - mid_col + 1                  */
	ir,			/* loop counter for buffer initialization    */
	irow,			/* current line pointer for circular buffer  */
	irsave,			/* index into rowsave array                  */
	kk,			/* loop counter                              */
	jj,			/* loop counter                              */
	line, m,		/* line counter for the processing loop      */
	maxpix,			/* maximum number of pixels in a buffer      */
	mid_col,		/* middle of columns (local window)          */
	mid_row,		/* middle of rows (local window)             */
	msg_flag = 1,  		/* flag for the processing message           */
	nb2,			/* nbins + 1                                 */
	nbox,			/* column difference between local window    */
				/* input image                               */
	ncc,			/* num_col - 1                               */
	nrbox,			/* row difference between local window and   */
				/* input image                               */
	num_cols,		/* number of columns in the local window     */
	num_lines,		/* number of lines (input image)             */
	num_pixels,      	/* number of pixels (input image             */
	num_rows,		/* number of rows in the local window        */
	out_band,		/* band number for the output image          */
	out_line,		/* starting line for ewrite                  */
	skip_flag,		/* flag for skipping over part of the code   */
				/* in a for loop                             */
	rsave,			/* pointer to mid_row                        */
	totyp;			/* data type for pxconv to convert to        */

/*
Initialize the local variables
------------------------------
*/
totyp = ELONG;
num_pixels = inwnd[0][NS];
num_lines = inwnd[0][NL];
num_cols = iwsize[NP];
num_rows = iwsize[NR];
ncc = num_cols - 1;
mid_col = num_cols / 2;
mid_row = num_rows / 2;
cpix = mid_col;
nbox = num_pixels - num_cols + 1;
nrbox = num_lines - mid_row + 1;
ipend = num_pixels - mid_col;
nb2 = nbins + 1;
treshold = (float)mthre;
out_band = 0;
intyp = EREAL;


/*
Allocate dynamic memory for input, output and work buffers.
------------------------------------------------------------
*/
buf1_siz = (long) ((num_pixels + 1) * sizeof(float));
buf2_siz = (long) (MXLSIZ * (num_pixels + 1) * sizeof(long));
buf3_siz = (long) (MHALF * (num_pixels + 1) * sizeof (float));
if ((buffer = (long *) malloc(buf2_siz)) == NULL ||
   (rowsave = (float *) malloc(buf3_siz)) == NULL ||
   (work = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);

maxpix = num_pixels + 1;
for (i = 0, m = 0; i < MXLSIZ; i++, m += maxpix)
   bufrow[i] = &buffer[m];

for (i = 0, m = 0; i < MHALF; i++, m += maxpix)
   rowrow[i] = &rowsave[m];

buf1_siz = (long) (num_pixels * sizeof (float));
if ((inbuf = (float *) malloc(buf1_siz)) == NULL ||
   (outbuf = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);

/*
Initialize and enable the processing message utility
-------------------------------------------------------
*/
printf("\nLOWCAL   Iteration %3d of %3d\n", iter, niter);
c_inittm ("LOWCAL", "Processed", "line", "band", "", &num_lines,
          &tot_inbands, &msgtim, &msg_flag);
curcat = 0;

/* 
Perform quantized mode window operations on all input images
-------------------------------------------------------------
*/

for (img = 0; img < num_img; img++)
   {
  for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {
      curent = 1;
      curcat++;
      out_line = 1;
      if (iter == 1)
         in_band = num_img_bands[img][cur_band];
      else
         in_band = out_img_bands[img][cur_band];

      for (ir = 1; ir <= num_rows; ir++)
         bufrow[ir][0] = 0;

      wbin = (nbins) / (omax[out_band] - omin[out_band]);
      offset = -omin[out_band] * wbin + 1.0;
      for (jj = 1; jj <= nbins; jj++)
         {
         i = jj;
/*         pval[jj] = omin[out_band] + (((float)i) - 0.5) / wbin;  */
         pval[jj] = omin[out_band] + ((float)i) / wbin; 
         }
      out_band++;
      c_startm();

/*
Store the first few input lines in the buffer and scale them to histogram 
the bins
-------------------------------------------------------------------------
*/
      irow = 0;
      for (line = 1; line <= num_rows-1; line++)
         {
         curent++;
         irow++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxline (inbuf, work, &num_pixels, &intyp, &wbin, &offset);
         c_pxconv (&intyp, &totyp, work, &bufrow[irow][one], &num_pixels);

/*
Replace the top lines of the output image with either the oirginal input 
lines or 0's, depending on the iteration number.
------------------------------------------------------------------------
*/
         if (line  <=  mid_row) 
               {
               if (iter  <  niter)  
                  c_pxcopy (inbuf, outbuf, &intyp, &num_pixels); 
               else
                  c_pxfill (outbuf, &intyp, &num_pixels, &zfill); 
               c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
               out_line++;
               } 

/*
Copy the leftmost and the rightmost pixels of the current line to save buffer.
------------------------------------------------------------------------------
*/
         else
            {
            c_pxcopy (inbuf, save[irow], &intyp, &mid_col); 
            c_pxcopy (&inbuf[ipend], &save[irow][cpix], &intyp, &mid_col); 
            }
         }

/*
Process the rest of the image by reading the remaining input lines, and 
finding the quantized mode of the pixels in the sliding window.
-----------------------------------------------------------------------
*/
      rsave = mid_row;
      irsave = 0;
      c_startm();
      for (line = num_rows; line <= num_lines; line++)
         {
         curent++;
         irow++;
         rsave++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);

/*
Copy the leftmost and the rightmost pixels of the current line to save buffer.
------------------------------------------------------------------------------
*/
         c_pxcopy (inbuf, save[irow], &intyp, &mid_col); 
         c_pxcopy (&inbuf[ipend], &save[irow][cpix], &intyp, &mid_col);

/*
Scale the current line to the histogram bins.
---------------------------------------------
*/
         c_pxline (inbuf, work, &num_pixels, &intyp, &wbin, &offset);
         c_pxconv (&intyp, &totyp, work, &bufrow[irow][one], &num_pixels);

/*
Initialize the histogram counts for the first sliding window.
----------------------------------------------------------------
*/
         maxpix = 1025;
         c_pxfill (count, &intyp, &maxpix, &zfill);
         for (ii = 1; ii <= num_rows; ii++)
            {
            for (jj = 1; jj <= ncc; jj++)
               {
               ind = bufrow[ii][jj];
               count[ind] = count[ind] + 1;                  
               }
            }

/*
Compute the histogram counts for the rest of the image.
--------------------------------------------------------
*/
         for (jj = 1; jj <= nbox; jj++)
            {
            skip_flag = FALSE;
      
            for (kk = 1; kk <= num_rows; kk++)
               {
               ind1 = bufrow[kk][jj-1];
               count[ind1] = count[ind1] - 1;
               ind2 = bufrow[kk][jj+ncc];
               count[ind2] = count[ind2] + 1;                    
               }

            if (count[nb2]  !=  0) 
               {
               count[nbins] = count[nbins] + count[nb2];
               count[nb2] = 0;
               } 

/*
Call the vabmxf routine to find the maximum value of counts (bin occupancy).
----------------------------------------------------------------------------
*/
            vabmxf (&count[one], &nbins, &one, &index, &vmax);

/*
Verify that the mode is unique.
--------------------------------
*/
            for (i = 1; i <= nbins; i++)
               {
               if (i != index) 
                  if (count[i] == count[index]) 
                     {
                     skip_flag = TRUE;
                     indout = jj + mid_col - 1;
                     outbuf[indout] = pval[bufrow[rsave][indout]];
                     break;
                     }
               }
   
/*
Verify that the bin occupancy for the mode is grater than or equal to the 
treshold.
If the threshold test or the unique mode test fails, use the center of 
the bins, else use the mode itself.
--------------------------------------------------------------------------
*/            
            if (skip_flag == FALSE)
               {
               indout = jj + mid_col - 1;
               if (vmax >= treshold) 
                  outbuf[indout] = pval[index];
               else
                  outbuf[indout] = pval[bufrow[rsave][indout]];
               }
            }

/*
Replace the leftmost and rightmost pixel values of the output image by the 
original input values.
-----------------------------------------------------------------------
*/
         if (iter < niter) 
            {
            c_pxcopy (save[rsave], outbuf, &intyp, &mid_col);
            c_pxcopy (&save[rsave][cpix], &outbuf[ipend], &intyp, &mid_col);

/*
Copy the last row of the window to a save area, so that it  can be copied 
to the output image 
----------------------------------------------------------------------------
*/
            if (line >= nrbox) 
               {
               irsave++;
               c_pxcopy (inbuf, rowrow[irsave], &intyp, &num_pixels);
               }
            }

/*
Replace the bottom lines of the output image with 0's if necessary.
--------------------------------------------------------------------
*/
          else
            {
            c_pxfill (outbuf, &intyp, &mid_col, &zfill);
            c_pxfill (&outbuf[ipend], &intyp, &mid_col, &zfill); 
            }

/*
Copy the processing results to the output image.
--------------------------------------------------
*/
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         rsave = rsave % num_rows;
         irow = irow % num_rows;
         }

/*
Replace the bottom lines of the output image with either the oirginal 
input lines or 0's, depending on the iteration number.
---------------------------------------------------------------------
*/
      for (irsave = 1; irsave <= mid_row; irsave++)
         {
         if (iter  <  niter)  
            c_pxcopy (rowrow[irsave], outbuf, &intyp, &num_pixels); 
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         }

      c_stoptm();
      } /* bands loop */
   } /* image loop */

/*
Free the dynamic memory.
------------------------
*/
free(buffer);
free(rowsave);
free(work);
}
