#include "lowcal.h"
#include "util.h"
#include "pixman.h"

void selavg
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of images                           */
    long num_img_bands[][MAXBND+1],	/* band numbers for input images      */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands		/* total number of bands                      */
)


/*
*****************************************************************************

NAME:	SELAVG

PURPOSE: Performs the selective average window operations on the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-85    Y. Sheu,           non-ap conversion
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          corrected display of processing
                                             msg and selection of input band;
                                             fixed memory allocation problem
   4	     16-Nov-89    K. Kunska          Fixed dynamic memory allocation
   5         28-Nov-89    K. Kunska          Allocated i/o buffers dynamically

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables.
   Allocate dynamic memory for the input, output and work buffers.
   For input images
      Initialize and enable the processing message utility.
      Store the first two input lines in the buffer and and compute their sum.
      Replace the top lines of the output image with either the oirginal input 
      lines or 0's, depending on the iteration number.
      Process the rest of the image by reading the remaining input lines.
      Store the pixel values of the first window in the array called array.
      Continue for the rest of the image.
      Compute the differences between the center pixel and its neighbors and 
      store it in the count array.
      Count the number of elements of the count array with the absolute values
      greater than or equant to the threshold value.
      If the number of pixels with absolute difference values is greater than or
      equal the user specified number of pixels, find the mean of all the pixels 
      excluding the center pixel. 
      Copy the work buffer to the output image .
      Replace the leftmost and rightmost pixel values by the original input 
      values or 0's depending on the iteration number.
      Replace the bottom lines of the output image with either the oirginal 
      input lines or 0's, depending on the iteration number.
   Free the dynamic memory.

*****************************************************************************
*/

{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


float	*array = NULL,  /* box pixels in one dimension                */
	*buffer, 		/* I/O buffer for input image                 */
	*bufrow[MXLSIZ],	/* pointers to buffer rows                    */
	*count = NULL,   	/* abs differences between the center pixel   */
                                /* and other pixels in the box                */
	scale = 1.0,		/* scaling factor for arithmetic operations   */
	tsum,			/* sum of "array" values                      */
	*work = NULL,	/* I/O work data buffer                       */
	xcenter,		/* center pixel in the box (not used for      */
				/* computation of the average)                */
	zfill = 0.0;		/* fill value                                 */

long	buf1_siz,		/* buffer size for 1-dimensional arrays       */
	buf2_siz,		/* buffer size for 2-dimensional arrays       */
	buf3_siz,		/* buffer of array and count buffers          */
	cpix,			/* center pixel of the first window in row    */
	cur_band,		/* current band number                        */
	i, ic,			/* loop counter and array index               */
/*	icc,  */		/* right neighbor of the center pixel         */
	icnt,			/* number of pixels with abs difference value */
				/* greater than the treshold value            */
	img,			/* current image being processed              */
	in,			/* loop counter and array index               */
	in_band,		/* input band for eread                       */
	in_line,		/* starting line for eread                    */
	intyp,			/* input data type                            */
	ip,			/* pixel loop counter                         */
	ipend,			/* icend + 1                                  */
	ir,			/* loop counter and array index               */
	irow,			/* current line pointer for circular buffer   */
	k,			/* index into array "array"                   */
	l,			/* loop counter and array index               */
	line, m,		/* line counter for the processing loops      */
	maxpix,			/* maximum number of pixels per buffer line   */
	mid_col,		/* middle of columns (local window)           */
	mid_row,		/* middle of rows (local window)              */
	msg_flag = 1, 		/* flag for the processing message            */
	nbox,			/* column difference between the local window */
				/* and the input image                        */
	ncc,			/* num_col - 1                                */
	nsize,			/* local window size                          */
	num_cols,		/* number of columns in the local window      */
	num_lines,		/* number of lines (input window)             */
	num_pixels,		/* number of pixels (input window)            */
	num_rows,    		/* number of rows in the local window         */
	out_band,		/* band number for the output image           */
	out_line,		/* starting line for ewrite                   */
	rsave;			/* pointer to mid_row                         */

/*
Initialize the local variables
------------------------------
*/
num_pixels = inwnd[0][NS];
num_lines = inwnd[0][NL];
num_cols = iwsize[NP];
num_rows = iwsize[NR];
mid_col = num_cols / 2;
mid_row = num_rows / 2;
cpix = mid_col;
ipend = num_pixels - mid_col;
nbox = num_pixels - num_cols + 1;
ncc = num_cols - 1;
nsize = num_cols * num_rows;
out_band = 0;
intyp = EREAL;

/*
Allocate dynamic memory for the input, output and work buffers.
----------------------------------------------------------------
*/
buf1_siz = (long) ((num_pixels + 1) * sizeof(float));
buf2_siz = (long) (MXLSIZ * (num_pixels + 1) * sizeof(float));
buf3_siz = (long) (nsize * sizeof(float));
if ((buffer = (float *) malloc(buf2_siz)) == NULL ||
   (array = (float *) malloc(buf3_siz)) == NULL ||
   (count = (float *) malloc(buf3_siz)) == NULL ||
   (work = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "selavg-alloc",
               &fatal_err);

maxpix = num_pixels + 1;
for (i = 0, m = 0; i < MXLSIZ; i++, m += maxpix)
   bufrow[i] = &buffer[m];

buf1_siz = (long) (num_pixels * sizeof (float));
if ((inbuf = (float *) malloc(buf1_siz)) == NULL ||
   (outbuf = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);

/*
Initialize and enable the processing message utility
----------------------------------------------------
*/
printf("\nLOWCAL   Iteration %3d of %3d\n", iter, niter);
c_inittm ("LOWCAL", "Processed", "line", "band", " ", &num_lines,
           &tot_inbands, &msgtim, &msg_flag);
curcat = 0;

/* 
Perform the selective-average window operations on all input images
---------------------------------------------------------------------
*/

for (img = 0; img < num_img; img++)
   {
   for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {
      curent = 1;
      curcat++;
      out_band++;
      out_line = 1;
      if (iter == 1)
         in_band = num_img_bands[img][cur_band];
      else
         in_band = out_img_bands[img][cur_band];
      c_pxfill (bufrow[zero], &intyp, &num_pixels, &zfill);

      c_startm();

/*
Store the first few input lines in the buffer.
------------------------------------------------
*/
      irow = 1;
      for (line = 1; line <= num_rows-1; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels); 
         irow++;
         }


/*
Replace the top lines of the output image with either the oirginal input 
lines or 0's, depending on the iteration number.
------------------------------------------------------------------------
*/
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
               c_pxcopy (bufrow[line], outbuf, &intyp, &num_pixels); 
         else
               c_pxfill (outbuf, &intyp, &num_pixels, &zfill); 
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         }

/*
Process the rest of the image by reading the remaining input lines.
--------------------------------------------------------------------
*/
      rsave = mid_row + 1;
      irow = 0;
      for (line = num_rows; line <= num_lines; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels); 

/*
Store the pixel values of the first window in the array called array.
-----------------------------------------------------------------------
*/
         k = 0;
         for (ic = 0; ic < ncc; ic++)
            {
            for (ir = 0; ir < num_rows; ir++)
               {
               array[k] = bufrow[ir][ic];
               k++;
               }
            }

/*
Continue for the rest of the image.
------------------------------------
*/
          for (ip = ncc; ip <= num_pixels; ip++)
             {
             for (ir = 0; ir < num_rows; ir++)
                {
                array[k] = bufrow[ir][ip];
                k++;
                }

/*
Compute the differences between the center pixel and its neighbors and store
it in the count array.
-----------------------------------------------------------------------------
*/
                xcenter = -bufrow[rsave][ip-mid_col];
                c_pxline (array, count, &nsize, &intyp, &scale, &xcenter); 

/*
Count the number of elements of the count array with the absolute values
greater than or equant to the threshold value.
---------------------------------------------------------------------------
*/
                 icnt = 0;
                 for (l = 0; l < nsize; l++)
                    {
                    if (abs(count[l]) >= thresh) 
                    icnt++;
                 }

/*
If the number of pixels with absolute difference values is greater than or
equal the user specified number of pixels, find the mean of all the pixels 
excluding the center pixel. 
----------------------------------------------------------------
*/
                 if (icnt >= jns) 
                    {
                    tsum = 0.0;
                    for (in = 0; in < nsize; in++)
                       tsum = tsum + array[in];
                    work[ip] = (tsum + xcenter) / ((float) nsize - 1);
                    }
                 else
                    work[ip] = -xcenter;
                 k = k % nsize;
                 }

/*
Copy the work buffer to the output image.
-----------------------------------------
*/
              c_pxcopy (&work[ncc], &outbuf[cpix], &intyp, &nbox); 

/*
Replace the leftmost and rightmost pixel values by the original input 
values or 0's depending on the iteration number.
-----------------------------------------------------------------------
*/
              if (iter  <  niter) 
                 {
                 c_pxcopy (bufrow[rsave], outbuf, &intyp, &mid_col); 
                 c_pxcopy (&bufrow[rsave][ipend], &outbuf[ipend], &intyp, 
                           &mid_col);
                 }
              else
                 {
                 c_pxfill (outbuf, &intyp, &mid_col, &zfill); 
                 c_pxfill (&outbuf[ipend], &intyp, &mid_col, &zfill); 
                 } 
              c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
              out_line++;
              irow = (irow+1) % num_rows;
              rsave = (rsave+1) % num_rows;
        }

/*
Replace the bottom lines of the output image with either the oirginal 
input lines or 0's, depending on the iteration number.
----------------------------------------------------------------------
*/
      irow = rsave;
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[irow], outbuf, &intyp, &num_pixels); 
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill); 
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         irow = (irow+1) % num_rows;
         }

      c_stoptm();
      } /* band loop */
   } /* image loop */

/*
Free the dynamic memory.
------------------------
*/
free(buffer);
free(array);
free(count);
free(work);
}
