#include "lowcal.h"
#include "util.h"
#include "pixman.h"

void sharp
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of output images                    */
    long num_img_bands[][MAXBND+1],	/* band numbers for the input images  */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands			/* total number of bands              */
)

/*
*****************************************************************************

NAME:	SHARP

PURPOSE: Performs the mean window operation on the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-85    Y. Sheu,           non-ap conversion
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          corrected display of processing
                                             msg and selection of input band
   4         16-Nov-89    K. Kunska          Fixed dynamic memory allocation
   5         28-Nov-89    K. Kunska          Allocated i/o buffers dymamically

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables.
   Allocate dynamic memory for the input, output and work buffers.
   For input images
      Initialize and enable the processing message utility.
      Store the first two input lines in the buffer and and compute their sum.
      Replace the top lines of the output image with either the oirginal input 
      lines or 0's, depending on the iteration number.
      Process the rest of the image by reading the remaining input lines, and 
      finding the differences between the center pixel and the highest and 
      lowest valued pixel in the sliding window.
      Find the maximum and the minimum pixel values for the columns of the 
      first window. Store the minimum value in work and the maximum value 
      in work2.
      Continue the above step for the rest of the input image.
      Compare the pixel values in work with the corresponding values in work2.
      If necessary, switch the values so that work has the lower values and
      work2 the higher values.
      Compute the differences between the center pixel in the window and the
      elements of work (miminum values) and work2 (maximum values). 
      Store the first set of differences in the array called mini and the second
      set of differences in the array called maxi.
      Store the larger of the absolute values in mini and maxi in the array 
      called buf2.
      Copy buf2 to the output image.
      Replace the leftmost and rightmost pixel values by the original input 
      values or 0's depending on the iteration number.
      Replace the bottom lines of the output image with either the oirginal 
      input lines or 0's, depending on the iteration number.
   Free the dynamic memory.

*****************************************************************************
*/

{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


float	*buf2 = NULL, 
	*buffer,		/* I/O buffer for input image                 */
	*bufrow[MXLSIZ],	/* pointers to buffer rows                    */
	*maxi = NULL,		/* maximum pixel values of the box            */
	*mini = NULL, 		/* minimum pixel values of the box            */
	offset = 0.0,		/* offset value for arithmetic operations     */
	scale =  1.0,		/* scale factor for arithmetic operations     */
	temp1,      		/* the current minimum pixel value            */
	temp2,                  /* the current maximun pixel value            */
	tnew,                   /* pixel to be compared with temp1 and temp2  */
	tnew1,			/* temporary holder of min values for each box*/
	tnew2,			/* temporary holder of max values for each box*/
	*work = NULL,   /* I/O work data buffer - min box values      */
	*work2 = NULL,	/* maximum box values                         */
	zfill =  0.0;		/* fill value                                 */

long	buf1_siz,		/* buffer size for 1-dimensional arrays       */
	buf2_siz,		/* buffer size for 2-dimensional arrays       */
	cpix,			/* center pixel of the first window in row    */
	cur_band,		/* current band number                        */
/*	data_type,	*/	/* data type of function parameters           */
	ic,			/* loop counter and array index               */
	icol,			/* loop counter and array index               */
	img,			/* current image being processed              */
	in_band,		/* input band for eread                       */
	in_line,		/* starting line for eread                    */
	i, ip, m,		/* loop counters and array indexes            */
	intyp,			/* input data type                            */
	ipend,			/* icend + 1                                  */
	ipp,			/* array index                                */
	ir,			/* loop counter and array index               */
	irow,			/* current line pointer for circular buffer   */
	line,			/* line counter for the processing loops      */
	maxpix,			/* maximum number of pixels per buffer line   */
	mid_col,		/* middle of columns (local window)           */
	mid_row,		/* middle of rows (local window)              */
	msg_flag = 1,		/* flag for the processing message            */
	nbox,			/* column difference between the local window */
				/* and the input image                        */
	ncc,			/* num_col - 1                                */
	num_cols,		/* number of columns in the local window      */
	num_lines,		/* number of lines                            */
	num_pixels,		/* number of pixels                           */
	num_rows,		/* number of rows in the local window         */
	out_band, 		/* band number for the output image           */
	out_line, 		/* line number for the output image           */
	rsave;			/* center line pointer                        */

/*
Initialize the local variables
------------------------------
*/
num_pixels = inwnd[0][NS];
num_lines = inwnd[0][NL];
num_cols = iwsize[NP];
num_rows = iwsize[NR];
mid_col = num_cols/2;
mid_row = num_rows/2;
cpix = mid_col;
ipend = num_pixels - mid_col;
nbox = num_pixels - num_cols + 1;
ncc = num_cols - 1;
out_band = 0;
intyp = EREAL;

/*
Allocate dynamic memory for the input, output and work buffers.
----------------------------------------------------------------
*/
buf1_siz = (long) ((num_pixels + 1) * sizeof(float));
buf2_siz = (long) (MXLSIZ * (num_pixels + 1) * sizeof(float));

if ((buffer = (float *) malloc(buf2_siz)) == NULL ||
   (buf2 = (float *) malloc(buf1_siz)) == NULL ||
   (maxi = (float *) malloc(buf1_siz)) == NULL ||
   (mini = (float *) malloc(buf1_siz)) == NULL ||
   (work = (float *) malloc(buf1_siz)) == NULL ||
   (work2 = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "sharp-alloc",
               &fatal_err);

maxpix = num_pixels + 1;
for (i = 0, m = 0; i < MXLSIZ; i++, m += maxpix)
   bufrow[i] = &buffer[m];

buf1_siz = (long) (num_pixels * sizeof (float));
if ((inbuf = (float *) malloc(buf1_siz)) == NULL ||
   (outbuf = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);

/*
Initialize and enable the processing message utility
----------------------------------------------------
*/
printf("\nLOWCAL   Iteration %3d of %3d\n", iter, niter);
c_inittm ("LOWCAL", "Processed", "line", "band", " ", &num_lines,
           &tot_inbands, &msgtim, &msg_flag);
curcat = 0;

/* 
Perform sharp-window operations on all input images
---------------------------------------------------
*/

for (img = 0; img < num_img; img++)
   {
   for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {

      curent = 1;
      curcat++;
      out_band++;
      out_line = 1;
      if (iter == 1)
         in_band = num_img_bands[img][cur_band];
      else
         in_band = out_img_bands[img][cur_band];
      c_pxfill (bufrow[zero], &intyp, &num_pixels, &zfill);

      c_startm();

/*
Store the first few input lines in the buffer.
------------------------------------------------
*/
      irow = 1;
      for (line = 1; line <= num_rows-1; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);
         irow++;
         }


/*
Replace the top lines of the output image with either the oirginal input 
lines or 0's, depending on the iteration number.
------------------------------------------------------------------------
*/
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[line], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         }

/*
Process the rest of the image by reading the remaining input lines, and 
finding the differences between the center pixel and the highest and lowest
valued pixel in the sliding window.
---------------------------------------------------------------------------
*/
      rsave = mid_row + 1;
      irow = 0;
      for (line = num_rows; line <= num_lines; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);

/*
Find the maximum and the minimum pixel values for the columns of the first 
window. Store the minimum value in work and the maximum value in work2.
--------------------------------------------------------------------------
*/
         for (ic = 0; ic < ncc; ic++)
            {
            temp1 = bufrow[0][ic];
            temp2 = temp1;
            for (ir = 1; ir < num_rows; ir++)
               {
               tnew = bufrow[ir][ic];
               if (tnew  <  temp1) 
                  temp1 = tnew;
               if (tnew  >  temp2) 
                  temp2 = tnew;
               }
            work[ic] = temp1;
            work2[ic] = temp2;
            }

/*
Continue the above step for the rest of the input image.
--------------------------------------------------------
*/
            for (ip = 0; ip < nbox; ip++)
               {
               ipp = ip + ncc;
               temp1 = bufrow[0][ipp];
               temp2 = temp1;
               for (ir = 1; ir < num_rows; ir++)
                  {
                  tnew = bufrow[ir][ipp];
                  if (tnew  <  temp1) 
                     temp1 = tnew;
                  if (tnew  >  temp2) 
                     temp2 =tnew;
                  }
               work[ipp] = temp1;
               work2[ipp] = temp2;

/*
Compare the pixel values in work with the corresponding values in work2.
If necessary, switch the values so that work has the smaller values and
work2 the higher values.
------------------------------------------------------------------------
*/
               temp1 = work[ip];
               temp2 = work2[ip];
               for (icol = ip+1; icol <= ip+ncc; icol++)
                  {
                  tnew1 = work[icol];
                  tnew2 = work2[icol];
                  if (tnew1  <  temp1) 
                     temp1 = tnew1;
                  if (tnew2  >  temp2) 
                     temp2 = tnew2;
                  }
               work[ip] = temp1;
               work2[ip] = temp2;

/*
Compute the differences between the center pixel in the window and the
elements of work (miminum values) and work2 (maximum values). 
Store the first set of differences in the array called mini and the second
set of differences in the array called maxi.
---------------------------------------------------------------------------
*/
             c_pxsub (&nbox, &bufrow[rsave][cpix], work, mini, &scale, &scale, 
                      &offset, &intyp);
             c_pxsub (&nbox, &bufrow[rsave][cpix], work2, maxi, &scale, &scale, 
                     &offset, &intyp);

/*
Store the larger of the absolute values in mini and maxi in the array called
buf2.
-----------------------------------------------------------------------------
*/
            for (ic = 0; ic < nbox; ic++)
               {
               if (abs(mini[ic]) > abs(maxi[ic])) 
                  buf2[ic] = work2[ic];
               else
                  buf2[ic] = work[ic];
               }
            }

/*
Copy buf2 to the output image.
------------------------------
*/
            c_pxcopy (buf2, &outbuf[cpix], &intyp, &nbox);
            c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);

/*
Replace the leftmost and rightmost pixel values by the original input 
values or 0's depending on the iteration number.
----------------------------------------------------------------------
*/
            if (iter  <  niter) 
               {
               c_pxcopy (bufrow[rsave], outbuf, &intyp, &mid_col);
               c_pxcopy (&bufrow[rsave][ipend], &outbuf[ipend], &intyp, 
                         &mid_col);
               }
            else
               {
               c_pxfill (outbuf, &intyp, &mid_col, &zfill);
               c_pxfill (&outbuf[ipend], &intyp, &mid_col, &zfill);
               } 
            c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
            out_line++;
            irow = (irow+1) % num_rows;
            rsave = (rsave+1) % num_rows;
            } 

/*
Replace the bottom lines of the output image with either the oirginal 
input lines or 0's, depending on the iteration number.
-----------------------------------------------------------------------
*/
      irow = rsave;
      for (line = 0; line < mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[irow], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill );
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         irow = (irow+1) % num_rows;
         }

      c_stoptm();
      } /* bands loop */
   }    /* image loop */

/*
Free the dynamic memory.
------------------------
*/
free(buffer);
free(buf2);
free(maxi);
free(mini);
free(work);
free(work2);
}
