#include "lowcal.h"
#include "util.h"
#include "pixman.h"

void varian
(
    long iter,			/* current iteration		              */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,           /* number of images                           */
    long num_img_bands[][MAXBND+1],/* band numbers for input images           */
    long out_img_bands[][MAXBND+1],/* band numbers for the output image       */
    long tot_inbands		/* total number of bands                      */
)

/*
*****************************************************************************

NAME:	VARIAN

PURPOSE: Performs the variance and standard deviation window operations on 
         the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-85    Y. Sheu,           non-ap conversion
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          corrected display of processing
                                             msg, selection of input bands and
                                             output values from stdev option
   4         16-Nov-89    K. Kunska          Fixed dynamic memory allocation
   5         28-Nov-89    K. Kunska          Allocated i/o buffers dynamically

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables.
   Allocate dynamic memory for the input, output and work buffers.
   For input images
      Initialize and enable the processing message utility.
      Store the first few input lines in the buffer and and compute their sums 
      and squares.
      Replace the top lines of the output image with either the oirginal input 
      lines or 0's, depending on the iteration number.
      Process the rest of the image by reading the remaining input lines, and 
      updating the sum array.
      Compute the first variance-window, then, update the subsequent sliding 
      windows by "recursion relation" storing the results in the work buffer.
      If the option is "standard deviation", take the square root of the work 
      buffer.
      Copy the work buffer to the output image .
      Replace the leftmost and rightmost pixel values by the original input 
      values or 0's depending on the iteration number.
      Replace the bottom lines of the output image with either the oirginal 
      input lines or 0's, depending on the iteration number.
   Free the dynamic memory.

*****************************************************************************
*/


{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


float	*buffer,		/* I/O buffer for input image                 */
	*bufrow[MXLSIZ],	/* pointers to buffer rows                    */
	*bufsqr = NULL,		/* holds squares of input image data          */
				/* divided by N-1 for one row of boxes        */
	*bufsqrow[MXLSIZ],	/* pointers to bufsqr rows                    */
	offset = 0.0, 		/* offset for arithmetic operations           */
	scale = 1.0,		/* scale factor for arithmetic operations     */
	scl1,			/* another scale factor for arith operations  */
	size,			/* local window size in floating form         */
	ss,			/* scaled down size                           */
	*sum = NULL,	/* column sums of buffer divided by N-1       */
	sump,                   /* accumulated column sum of pixels           */
	sump2,			/* accumulated column sum of pixel squares    */
	*sumsqr = NULL,	/* column sums of bufsqr array                */
	temp,			/* ensures correct precision in computations  */
	*work = NULL,	/* I/O work data buffer for column sums       */
	zfill = 0.0;		/* fill value                                 */

long	buf1_siz,		/* buffer size for 1-dimensional arrays       */
	buf2_siz,		/* buffer size for 2_dimensional arrays       */
	cpix,		 	/* center pixel of the first window in row    */
	cur_band,		/* current band number                        */
	icend,			/* diff between pixels in input image and     */
				/* middle of columns in local window          */
	img,			/* current image being processed              */
	in_band,		/* input band for eread                       */
	in_line, 		/* starting line for eread                    */
	i, ip, m,               /* loop counters and array indexes            */
	intyp,			/* input data type                            */
	ipc,			/* index into sum and work array              */
	ipend,			/* icend + 1                                  */
	irow,                   /* current line pointer for circular buffer   */
	line,			/* line counter for processing loops          */
	maxpix,			/* maximum number of pixels per buffer line   */
	mid_col,		/* middle of columns (local window)           */
	mid_row,		/* middle of rows (local window)              */
	msg_flag = 1,		/* flag for the processing message            */
	nbox,			/* column difference between local window     */
				/* and input image                            */
	next,			/* index into work array                      */
	nsize,			/* local window size                          */
	num_cols,             	/* number of columns in the local window      */
	num_lines,		/* number of lines (input image)	      */
	num_pixels,		/* number of pixels (input image)             */
	num_rows, 		/* number of rows in the local window         */
	out_band,		/* band number for the output image           */
	out_line,		/* starting line for ewrite                   */
	rsave;			/* saves the mid_row pointer                  */

/* 
Initialize local varianbles.
---------------------------
*/
num_pixels = inwnd[0][NS];
num_lines = inwnd[0][NL];
num_cols = iwsize[NP];
num_rows = iwsize[NR];
mid_col = num_cols / 2;
mid_row = num_rows / 2;
cpix = mid_col;
icend = num_pixels - mid_col - 1;
ipend = num_pixels - mid_col;
nbox = num_pixels - num_cols;
nsize = num_cols * num_rows;
size = (float)nsize;
ss = (size - 1.0 ) / size;
scl1 = 1.0 / (size - 1.0);
out_band = 0;
intyp = EREAL;

/* 
Allocate dynamic memory for work buffers
----------------------------------------
*/
buf1_siz = (long) ((num_pixels + 1) * sizeof(float));
buf2_siz = (long) (MXLSIZ * (num_pixels + 1) * sizeof(float));
if ((buffer = (float *) malloc(buf2_siz)) == NULL ||
   (bufsqr = (float *) malloc(buf2_siz)) == NULL ||
   (sum = (float *) malloc(buf1_siz)) == NULL ||
   (sumsqr = (float *) malloc(buf1_siz)) == NULL ||
   (work = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "varian-alloc",
             &fatal_err);

maxpix = num_pixels + 1;
for (i = 0, m = 0; i < MXLSIZ; i++, m += maxpix)
   {
   bufrow[i] = &buffer[m];
   bufsqrow[i] = &bufsqr[m];
   }

buf1_siz = (long) (num_pixels * sizeof (float));
if ((inbuf = (float *) malloc(buf1_siz)) == NULL ||
   (outbuf = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);

/*
Initialize and enable the processing message utility
-----------------------------------------------------
*/
printf("\nLOWCAL   Iteration %3d of %3d\n", iter, niter);
c_inittm ("LOWCAL", "Processed", "line", "band", " ", &num_lines,
           &tot_inbands, &msgtim, &msg_flag);
curcat = 0;
 
/*
Perform the variance/standard deviation window operations on all input images
-----------------------------------------------------------------------------
*/
for (img = 0; img < num_img; img++)
   {
   for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {
      curent = 1;
      curcat++;
      out_band++;
      out_line = 1;
      if (iter == 1)
         in_band = num_img_bands[img][cur_band];
      else
         in_band = out_img_bands[img][cur_band];
      c_pxfill (sum, &intyp, &num_pixels, &zfill);
      c_pxfill (sumsqr, &intyp, &num_pixels, &zfill);
      c_pxfill (bufrow[zero], &intyp, &num_pixels, &zfill);
      c_pxfill (bufsqrow[zero], &intyp, &num_pixels, &zfill);

      c_startm();

/*
Store the first few input lines in the buffer and and compute their sums and
squares.
----------------------------------------------------------------------------
*/
      irow = 1;
      for (line = 1; line <= num_rows-1; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);
         c_pxadd (&num_pixels, sum, bufrow[irow], sum, &scale, &scl1, &offset, 
                  &intyp);
         c_pxmul (&num_pixels, bufrow[irow], bufrow[irow], bufsqrow[irow],
                  &scl1, &offset, &intyp);
         c_pxadd (&num_pixels, sumsqr, bufsqrow[irow], sumsqr, &scale, &scale, 
                  &offset, &intyp);
         irow++;
         }


/*
Replace the top lines of the output image with either the oirginal input 
lines or 0's, depending on the iteration number.
-------------------------------------------------------------------------
*/
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[line], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         }

/*
Process the rest of the image by reading the remaining input lines, and 
updating the sum and the sumsqr arrays.
------------------------------------------------------------------------
*/
      rsave = mid_row + 1;
      irow = 0;
      for (line = num_rows; line <= num_lines; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxsub (&num_pixels, sum, bufrow[irow], sum, &scale, &scl1, &offset,
                  &intyp);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);
         c_pxadd (&num_pixels, sum, bufrow[irow], sum, &scale, &scl1, &offset, 
                  &intyp);
         c_pxsub (&num_pixels, sumsqr, bufsqrow[irow], sumsqr, &scale, &scale, 
                  &offset, &intyp);
         c_pxmul (&num_pixels, bufrow[irow], bufrow[irow], bufsqrow[irow], 
                  &scl1, &offset, &intyp);
         c_pxadd (&num_pixels, sumsqr, bufsqrow[irow], sumsqr, &scale, &scale, 
                  &offset, &intyp);

/*
Compute the first variance-window, then, update the subsequent sliding 
windows by "recursion relation" storing the results in the work buffer.
-----------------------------------------------------------------------
*/
         sump = 0.0;
         sump2 = 0.0;
         for (ip = 0; ip < num_cols; ip++)
            {
            sump = sump + sum[ip];
            sump2 = sump2 + sumsqr[ip];
            }
         temp = ss * sump * sump;
         next = cpix;
         work[next] = sump2 - temp;
         for (ip = 0; ip < nbox; ip++)
            {
            next++;
            ipc = ip + num_cols;
            sump = sump - sum[ip] + sum[ipc];
            sump2 = sump2 - sumsqr[ip] + sumsqr[ipc];
            temp = ss * sump * sump;
            work[next] = sump2 - temp;
            }

/*
If the option is "standard deviation", take the square root of the work buffer
------------------------------------------------------------------------------
*/
            if (iopt  ==  2) 
               for (ip = cpix; ip <=  icend; ip++)
                  work[ip] = sqrt((double)work[ip]);

/*
Copy the work buffer to the output image .
------------------------------------------
*/
            c_pxcopy (&work[cpix], &outbuf[cpix], &intyp, &icend);
            c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);

/*
Replace the leftmost and rightmost pixel values by the original input 
values or 0's depending on the iteration number.
---------------------------------------------------------------------
*/
            if (iter  <  niter) 
               {
               c_pxcopy (bufrow[rsave], outbuf, &intyp, &mid_col);
               c_pxcopy (&bufrow[rsave][ipend], &outbuf[ipend], &intyp, 
                         &mid_col);
               }
            else
               {
               c_pxfill (outbuf, &intyp, &mid_col, &zfill);
               c_pxfill (&outbuf[ipend], &intyp, &mid_col, &zfill);
               }
            c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
            out_line++;
            irow = (irow+1) % num_rows;
            rsave = (rsave+1) % num_rows;
         }

/*
Replace the bottom lines of the output image with either the oirginal 
input lines or 0's, depending on the iteration number.
---------------------------------------------------------------------
*/
      irow = rsave;
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[irow], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         irow = (irow+1) % num_rows;
         }

      c_stoptm();
      } /* band loop */
   } /* image loop */

/*
Free the dynamic memory
-----------------------
*/
free (buffer);
free (bufsqr);
free (sum);
free (sumsqr);
free (work);
}
