#include "lowcal.h"
#include "util.h"
#include "pixman.h"


void xmean
(
    long iter,			/* current iteration			   */
    long niter,			/* total number of iterations		   */
    long bands_per_image[],	/* number of bands to be processed         */
    long num_img,		/* number of images                        */
    long num_img_bands[][MAXBND+1], /* band numbers for input images        */
    long out_img_bands[][MAXBND+1], /* band numbers for the output image    */
    long tot_inbands		/* total number of bands                   */
)
/*
*****************************************************************************

NAME:	XMEAN

PURPOSE: Performs the mean window operation on the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Sep-85    Y. Sheu,           non-ap conversion
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          corrected display of processing 
                                             msg and selection of input band
   4         16-Nov-89    K. Kunska          Fixed dynamic memory allocation
   5         28-Nov-89    K. Kunska          Allocated i/o buffers dynamically

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables.
   Allocate dynamic memory for the input, output and work buffers.
   For input images
      Initialize and enable the processing message utility.
      Store the first two input lines in the buffer and and compute their sum.
      Replace the top lines of the output image with either the oirginal input 
      lines or 0's, depending upon the iteration number.
      Process the rest of the image by reading the remaining input lines, and 
      updating the sum array.
      Compute the first mean-window, then, update the subsequent sliding 
      windows by "recursion relation".
      If the option is "mean subtraction ", subtract the mean from the central 
      pixel.
      Copy the work buffer to the output image .
      Replace the leftmost and rightmost pixel values by the original input 
      values or 0's depending upon the iteration number.
      Replace the bottom lines of the output image with either the oirginal 
      input lines or 0's, depending upon the iteration number.
   Free the dynamic memory.

*****************************************************************************
*/

{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


float	*buffer, 		/* I/O data buffer for input image 	   */
	*bufrow[MXLSIZ],	/* pointers to buffer rows                 */
	diff,			/* difference between mean and center pixel*/
	offset = 0.0,		/* offset for addition & subtraction       */
	scale = 1.0,		/* scale factor for addition & subtraction */
	*sum = NULL,    	/* work data buffer for column sums        */
	sum_sum,		/* sum of array sum elements               */
	*work = NULL,		/* buffer for intermediate calculations    */
	zfill = 0.0;		/* fill value                              */
      
long	buf1_siz,		/* buffer size for 1-dimensional arrays    */
	buf2_siz,		/* buffer size for 2-dimensional arrays    */
	cpix,			/* center pixel of the first window in row */
	cur_band,		/* current band being processed            */
	icc,			/* right neighbor of the local center pixel*/
	icend,			/* diff between pixels in input image and  */
				/* middle of columns in local window       */
	img,			/* current image being processed           */
        in_band,		/* input band for eread                    */
        in_line,		/* starting line for eread                 */
	i, ip, m,		/* loop counters and array indexes         */
	ipend,			/* icend + 1                               */
	irow, 			/* current line pointer for circular buffer*/
	intyp,                  /* data type                               */
	line,			/* line counter for processing loops       */
	maxpix,			/* maximum number of pixels per buffer line*/
	mid_col,		/* middle of colums (local window)         */
	mid_row,                /* middle of rows (local window)	   */
	msg_flag = 1,		/* flag for the processing message         */
	nbox,			/* column difference between the local     */
				/* window and the input image              */
	nsize,			/* local window size                       */
	num_cols,		/* number of columns in the local window   */
	num_lines,		/* number of lines in the input image      */
	num_pixels,		/* number of pixels in the input image     */
	num_rows,		/* number of rows in the local window      */
	out_band,		/* band number for the output image        */
	out_line,		/* starting line for ewrite                */
	rsave;			/* saves the mid_row pointer               */

/*
Initialize local variables.
--------------------------
*/
num_pixels = inwnd[0][NS];
num_lines = inwnd[0][NL];
num_cols = iwsize[NP];
num_rows = iwsize[NR];
mid_col = num_cols / 2;
mid_row = num_rows / 2;
cpix = mid_col;
icc =cpix + 1;
icend = num_pixels - mid_col - 1;
ipend = num_pixels - mid_col;
nbox = num_pixels - num_cols + 1;
nsize = num_rows * num_cols;
out_band = 0;
intyp = EREAL;

/*
Allocate dynamic memory for the input, output and work buffers.
----------------------------------------------------------------
*/
buf1_siz = (long) ((num_pixels + 1) * sizeof(float));
buf2_siz = (long) (MXLSIZ * (num_pixels + 1) * sizeof(float));
if ((buffer = (float *) malloc(buf2_siz)) == NULL ||
   (sum = (float *) malloc(buf1_siz)) == NULL ||
   (work = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "xmean-alloc",
               &fatal_err);

maxpix = num_pixels +1;
for (i = 0, m = 0; i < MXLSIZ; i++, m += maxpix)
   bufrow[i] = &buffer[m];

buf1_siz = (long) (num_pixels * sizeof (float));
if ((inbuf = (float *) malloc(buf1_siz)) == NULL ||
   (outbuf = (float *) malloc(buf1_siz)) == NULL)
   c_errmsg ("Error: cannot allocate dynamic memory", "quan-alloc",
               &fatal_err);

/*
Initialize and enable processing message utility
------------------------------------------------
*/
printf("\nLOWCAL   Iteration %3d of %3d\n", iter, niter);
c_inittm ("LOWCAL", "Processed", "line", "band", " ", &num_lines,
           &tot_inbands, &msgtim, &msg_flag);
curcat = 0;

/* 
Perform mean window operations on all input images
--------------------------------------------------
*/
for (img = 0; img < num_img; img++)
   {
   for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {
      curent = 1;
      curcat++;
      out_band++;
      out_line = 1;
      if (iter == 1)
         in_band = num_img_bands[img][cur_band];
      else
         in_band = out_img_bands[img][cur_band];
      c_pxfill (sum, &intyp, &num_pixels, &zfill);
      c_pxfill (bufrow[zero], &intyp, &num_pixels, &zfill);

      c_startm();


/*
Store the first two input lines in the buffer and and compute their sum.
------------------------------------------------------------------------
*/
      irow = 1;
      for (line = 1; line <= num_rows-1; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);
         c_pxadd (&num_pixels, sum, bufrow[irow], sum, &scale, &scale, &offset,
                  &intyp);
         irow++;
         }

/*
Replace the top lines of the output image with either the oirginal input lines 
or 0's, depending upon the iteration number.
------------------------------------------------------------------------------
*/
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)
            c_pxcopy (bufrow[line], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         }

/*
Process the rest of the image by reading the remaining input lines, and updating
the sum array.
--------------------------------------------------------------------------------
*/
      rsave = mid_row + 1;
      irow = 0;
      for (line = num_rows; line <= num_lines; line++)
         {
         curent++;
         in_line = line;
         c_eread (&fdescin[img], &in_band, &in_line, inbuf, &one);
         c_pxsub (&num_pixels, sum, bufrow[irow], sum, &scale, &scale, 
                  &offset, &intyp);
         c_pxcopy (inbuf, bufrow[irow], &intyp, &num_pixels);
         c_pxadd (&num_pixels, sum, bufrow[irow], sum, &scale, &scale, 
                  &offset, &intyp);

/*
Compute the first mean-window
------------------------------
*/
         sum_sum = 0.0;
         for (ip = 0; ip < num_cols; ip++)
            sum_sum = sum_sum + sum[ip];
   
         work[cpix] = sum_sum / (float) nsize;

/*
 then, update the subsequent sliding windows by "recursion relation"
---------------------------------------------------------------------
*/
         for (ip = icc; ip <= icend; ip++)
            {
            sum_sum = sum_sum - sum[ip-cpix-1] + sum[ip+mid_col];
            work[ip] = sum_sum / (float) nsize;
            }

/*
If the option is "mean subtraction ", subtract the mean from the central pixel
-------------------------------------------------------------------------------
*/
         if (iopt  ==  12) 
            {
            for (ip = cpix; ip <= icend; ip++)
               {
               diff = bufrow[rsave][ip] - work[ip];
               if (diff  >=  0.0) 
                  work[ip] = diff;
               else
                  work[ip] = 0.0;
               }
            }

/*
Copy the work buffer to the output image 
------------------------------------------
*/
         c_pxcopy (&work[cpix], &outbuf[cpix], &intyp, &nbox);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);

/*
Replace the leftmost and rightmost pixel values by the original input values
or 0's depending upon the iteration number.
-----------------------------------------------------------------------------
*/
         if (iter  <  niter) 
            {
            c_pxcopy (bufrow[rsave], outbuf, &intyp, &mid_col);
            c_pxcopy (&bufrow[rsave][ipend], &outbuf[ipend], &intyp, &mid_col);
            }
         else
            {
            c_pxfill (outbuf, &intyp, &mid_col, &zfill);
            c_pxfill (&outbuf[ipend], &intyp, &mid_col, &zfill);
            }
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         irow = (irow+1) % num_rows;
         rsave = (rsave+1) % num_rows;
      }

/*
Replace the bottom lines of the output image with either the oirginal input 
lines or 0's, depending upon the iteration number.
-----------------------------------------------------------------------------
*/
      irow = rsave;
      for (line = 1; line <= mid_row; line++)
         {
         if (iter  <  niter)  
            c_pxcopy (bufrow[irow], outbuf, &intyp, &num_pixels);
         else
            c_pxfill (outbuf, &intyp, &num_pixels, &zfill);
         c_ewrite (&fdescout, &out_band, &out_line, outbuf, &one);
         out_line++;
         irow = (irow+1) % num_rows;
         }

      c_stoptm();
      } /* band loop */
   } /* image loop */

/*
Free the dynamic memory.
------------------------
*/
free(buffer);
free(sum);
free(work);
} 
