/****************************************************************************
NAME               LS2TPS

PURPOSE  This function will take in as input either a file containing a list
         of lines and samples, or lines and samples input in TAE.  It will
         use the DDR of the input image and these line/sample pairs to write
         a Tie Point Selection file specified by the user.
         

DEVELOPMENT HISTORY:

VERSION    PROGRAMMER      DATE         REASON
-------    ----------      ----         ------
 1.0       B. Misterek     7/00         Added ability to output a TPS file

RETURN VALUE:  int, status of the function (E_SUCC, E_FAIL)


****************************************************************************/

#include "las.h"
#include "geompak.h"
#include "ls2pc.h"

static int write_point
(
    struct TPSDATA * data,   /* I: Holds the data for the data point */
    struct PTDESC  * ptdesc, /* I: Descriptor for the TPS file */
    double line,             /* I: Line Number */
    double sample,           /* I: Sample Number */
    double * coef,           /* I: Coefficients used to compute proj coord */
    long num_lines,          /* I: Number of Lines in the ddr */
    long num_samples,        /* I: Number of Samples in the ddr */
    long i                   /* I: Number to assign the point id */
);


int ls2tps
(
  struct PARBLK *vblk,      /* I:  TAE parameter block                        */
  struct DDR *ddr,          /* I:  DDR Structure                              */
  long n_ls,                /* I:  Number of line/sample pairs                */
  char *infile,             /* I:  File containing the line/sample pairs      */
  char *out_file,           /* I:  User name of output file                   */
  double *coef              /* I:  Coefficients used to compute proj coord    */
)
{

  struct VARIABLE *var;      /*  Pointer to a TAE parameter                   */
  struct VARIABLE *var1;     /*  Pointer to a TAE parameter                   */

  struct TPSHEAD head;       /*  Structure to hold the TPS header             */
  struct TPSDATA data;       /*  Structure to hold the TPS data points        */

  struct PTDESC * ptdesc;    /*  Structure to hold the pt descriptor          */

  long acc = IWRITE;         /*  Access flag                                  */
  long pttype  = TPS;        /*  Variable to identify the type of file        */
  long status;               /*  Status flag for use with function calls      */

  double line;               /*  Variable to hold the current line            */
  double sample;             /*  Variable to hold the current sample          */

  FILE *fp = NULL;           /*  File pointer to point to the input file      */

  int i;                     /*  General loop variable                        */


  /* Initialize the TPL header.
  -----------------------------*/
  head.corners[0] = ddr->upleft[0];
  head.corners[1] = ddr->upleft[1];
  head.corners[2] = ddr->loleft[0];
  head.corners[3] = ddr->loleft[1];
  head.corners[4] = ddr->upright[0];
  head.corners[5] = ddr->upright[1];
  head.corners[6] = ddr->loright[0];
  head.corners[7] = ddr->loright[1];

  head.pdist[0] = ddr->pdist_y;
  head.pdist[1] = ddr->pdist_x;

  for(i = 0 ; i < 15 ; i++)
      head.projprms[i] = ddr->proj_coef[i];

  head.proj_valid = ddr->valid[3];
  head.code = ddr->proj_code;
  head.zone = ddr->zone_code;
  head.datum = ddr->datum_code;
  strcpy(head.units, ddr->proj_units);
  strcpy(head.name, infile); 
  
  if( ddr->proj_code == 0 )
      strcpy(head.mode, "GEO");
  else
      strcpy(head.mode, "PRO");

  /* Open the tie point selection output file.
  --------------------------------------------*/
  if(c_ptopen(&ptdesc, out_file, &pttype, &acc, &head) != E_SUCC)
  {
      c_lasmsg(FALSE,"ls2pc-open","Error opening %s file", out_file);
      return E_FAIL;
  }

  /* Initialize each field of the data record.
  --------------------------------------------*/
  data.coord[0] = data.coord[1] = data.image_coord[0] = data.image_coord[1] = 
      data.elevation = data.zoom_factor = 0.0;
  data.chip_size[0] = data.chip_size[1] = data.offsets[0] = data.offsets[1] = 0;
  data.pt_id[0] = data.chip_name[0] = '\0';
  

  i = 1;
  /*  If an INFILE was provided, then the user wants
      the points to be pulled out of the INFILE.
  --------------------------------------------------*/
  if(strlen(infile) != 0)
  {
      /*  Open the input line/sample file. 
      ------------------------------------*/
      fp = fopen(infile,"r");
      if(fp == NULL)
      {
          c_lasmsg(FALSE,"ls2pc-open","Error opening %s file", infile);
          return(E_FAIL);
      }

      /*  Read line/samples as long as the end of file is not reached.
      ----------------------------------------------------------------*/
      status = fscanf(fp, "%lf %lf",&line, &sample); 
      while(status != EOF)
      {
          /*  Make sure that there was a line/sample pair.
          ------------------------------------------------*/
          if(status != 2)
          {
              c_lasmsg(FALSE, "ls2pc-pairs",
                       "The LINE and SAMPLE values in INFILE must be in pairs"
                       "  Invalid pair at line %i", i);
              fclose(fp);
              c_ptclse(&ptdesc);
              return E_FAIL;
          }

          /*  Copy the relevant parts to the TPSDATA structure and
              write it out.
          --------------------------------------------------------*/
          if( write_point(&data, ptdesc, line, sample, coef, 
                          ddr->nl, ddr->ns, i) != E_SUCC)
          {
              fclose(fp);
              c_ptclse(&ptdesc);
              return E_FAIL;
          }

          status = fscanf(fp, "%lf %lf",&line, &sample);
          i++;
      }
      if(i == 1)
      {
          c_lasmsg(FALSE, "ls2pc-info", "Specified line/sample file, %s, "
                   "contained no line/sample pairs", infile);
      }
      fclose(fp);
  }
  else /* The user input line/samples through TAE */
  {
    var = p_find(vblk,"LINES");
    var1 = p_find(vblk, "SAMPS");
    for (i = 0 ; i < n_ls ; i++)
    {
        line = RVAL(*var,i);
        sample = RVAL(*var1,i);

        /*  Copy the relevant parts to the TPSDATA structure and
            write it out.
        --------------------------------------------------------*/
        if( write_point(&data, ptdesc, line, sample, coef, 
                        ddr->nl, ddr->ns, i+1) != E_SUCC)
        {
            c_ptclse(&ptdesc);
            return E_FAIL;
        }

    }
  }

  /*  Close the TPS file.
  -----------------------*/
  if( c_ptclse(&ptdesc) != E_SUCC)
  {
      c_lasmsg(FALSE, "ls2pc-close","Error closing TPS file");
      return E_FAIL;
  }
 
  return E_SUCC;
}


/*******************************************************************************
 NAME:  write_point

 PURPOSE:  This function will fill in the relevant fields in the TPSDATA
           structure, and then write it out to the TPS file specified by
           ptdesc 

 RETURN VALUE:  int, status of the function (E_SUCC, E_FAIL);

*******************************************************************************/
static int write_point
(
    struct TPSDATA * data,    /*  I: holds the data for the data point */
    struct PTDESC  * ptdesc,  /*  I: Descriptor for the TPS file */ 
    double line,              /*  I: Line Number */
    double sample,            /*  I: Sample number */
    double * coef,            /*  I: coefficients used to compute proj coord */
    long num_lines,           /*  I: Number of lines in the ddr */
    long num_samples,         /*  I: Number of samples in the ddr */
    long i                    /*  I: Number to assign the point id */
)
{
    
    /*  Copy the relevant parts over to data and assign a point id.
    ---------------------------------------------------------------*/
    data->image_coord[0] = line;
    data->image_coord[1] = sample;
    sprintf(data->pt_id, "pt_%i",i);
    if( calc_pc(coef, line, sample, &data->coord[0], &data->coord[1])
        != E_SUCC)
    {
        return E_FAIL;
    }

    /*  Write out the data point to the TPS file if the line/sample
        pair is in bounds.
    ---------------------------------------------------------------*/
    if(  line > 0 &&   line <= num_lines &&
       sample > 0 && sample <= num_samples)
    {
        if( c_ptio(&ptdesc, data) != E_SUCC)
        {
            c_lasmsg(FALSE, "ls2pc-write", "Error writing to TPS file");
            return E_FAIL;
        }
    }
    else
    {
        c_lasmsg(FALSE, "ls2pc-info",
                 "Out of bounds data point: %13f %13f", line, sample);
    }
    
    return E_SUCC;
    
}



