/*****************************************************************************
NAME: PROCESS

PURPOSE: Processes the points from either the user entered parameters, or
         from the input file.  Stores results in 2 files.

PROGRAM HISTORY:
  Version    Programmer    Date      Reason
  -------    ----------    ----      ------
    1.0      T.Ratliff     10/99     Initial Development

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "ls2pc.h"

int process
(
  struct PARBLK *vblk,         /* I: TAE parameter block */
  struct DDR *ddr,              /* I: DDR structure */
  long n_ls,                   /* I: Number of line/sample pairs */
  double *coef,                /* I: Coefficients used to compute proj coord */
  char *infile,                /* I: File containing line/sample coords */
  char *tmpfil,                /* I: Name of file for PRINT parameter output */
  char *outfile                /* I: User name of output file */
)
{
  struct VARIABLE *var;        /* Pointer to a TAE parameter */
  struct VARIABLE *var1;       /* Pointer to a TAE parameter */
  long i;                      /* Loop control variable */
  long status;                 /* Function return status */

  double line;                 /* Line number read from file */
  double samp;                 /* Sample number read from file */
  double x;                    /* Computed X projection coord */
  double y;                    /* Computed Y projection coord */

  FILE *fp_out[2];             /* Pointers to the 2 output files */
  FILE *fp_in;                 /* Pointer to the input file */

  /* Open output file, print header to file
  -----------------------------------------*/
  fp_out[0] = fopen(tmpfil,"w");
  fp_out[1] = fopen(outfile,"w");
  if ((fp_out[0] == NULL) || (fp_out[1] == NULL))
  {
    c_lasmsg(FALSE,"ls2pc-open","Error opening output file");
    return(E_FAIL);
  }

  fprintf(fp_out[0],"LINE/SAMPLE                                PROJ (Y, X)\n");
  fprintf(fp_out[0],"-----------------------------------------------------------\n");

  /* Read the points that are
     in the input file
  ---------------------------*/
  if (strlen(infile) != 0)
  {
    i = 0;
    /* Open the input line/sample file
    ----------------------------------*/
    fp_in = fopen(infile,"r");
    if (fp_in == NULL)
    {
      c_lasmsg(FALSE,"ls2pc-open","Error opening input file");
      fclose(fp_out[0]);
      fclose(fp_out[1]);
      return(E_FAIL);
    }

    while ((status=fscanf(fp_in,"%lf %lf",&line,&samp)) != EOF)
    {
      i++;
      if (status != 2)
      {
        c_lasmsg(FALSE,"ls2pc-pairs",
                 "The LINE and SAMPLE values in INFILE must be in pairs"
                 "  Invalid pair at line %i",i);
        fclose(fp_in);
        fclose(fp_out[0]);
        fclose(fp_out[1]);
        return(E_FAIL);
      }

      /* Compute the projection coordinates
         from the line/sample coordinates
      -------------------------------------*/
      status = calc_pc(coef,line,samp,&y,&x);
      if (status != E_SUCC)
        return(E_FAIL);

      /* Write the values to the outfile
      ----------------------------------*/
      fprintf(fp_out[1],"%13f %13f\n",y,x);

      /* Check for boundary conditions and
         write results to the print file
      ------------------------------------*/
      if ((line < 1) || (line > ddr->nl) || (samp < 1) || (samp > ddr->ns))
        fprintf(fp_out[0],"(%13f, %13f)          Out of bounds data point.\n",
                line,samp);
      else
        fprintf(fp_out[0],"(%13f, %13f)        (%lf, %lf)\n",line,samp,y,x);
    }
    fclose(fp_in);
  }
  else
  {
    var = p_find(vblk,"LINES");
    var1 = p_find(vblk,"SAMPS");
    for (i = 0; i < n_ls; i++) 
    {
      line = RVAL(*var,i);
      samp = RVAL(*var1,i);

      /* Compute the projection coordinates
         from the line/sample coordinates
      -------------------------------------*/
      status = calc_pc(coef,line,samp,&y,&x);
      if (status != E_SUCC)
        return(E_FAIL);

      /* Write the values to the outfile
      ----------------------------------*/
      fprintf(fp_out[1],"%13f %13f\n",y,x);

      /* Check for boundary conditions and
         write results to the print file
      ------------------------------------*/
      if ((line < 1) || (line > ddr->nl) ||
          (samp < 1) || (samp > ddr->ns))
        fprintf(fp_out[0],"(%13f, %13f)          Out of bounds data point.\n",
                line,samp);
      else
        fprintf(fp_out[0],"(%13lf, %13lf)        (%lf, %lf)\n",
                line,samp,y,x);
    }
  }

  /* Add new line to file and close output file pointer
  -----------------------------------------------------*/
  fprintf(fp_out[0],"\n");
  fclose(fp_out[0]);
  fclose(fp_out[1]);

  return(E_SUCC);
}
