/*****************************************************************************
NAME:		LSTDLTF

PURPOSE:	Function to list out either all of the fields of an entry from
		the Display and Look-up Table File (DLTF) or all of the entries
		in a DLTF and their descriptions.

PROGRAM HISTORY:
VERSION	     DATE	PROGRAMMER   CODE/CONT	REQUEST
-------	     ----  	----------   ---------	-------
  1.0	     7/85	D. Gordon    USGS/EDC	Initial development.
  1.1	    10/88	D. Akkerman  USGS/EDC	Rewritten for the LAS DLTF
						format; DMS v3_0 retrocode.
  1.2	     3/89	D. Akkerman  USGS/EDC	Changed the value of slope and 
						offset to double.
  1.3	    10/92	S. Nelson    USGS/EDC	Changed from DMS application to
						LAS application to use for XID.
						User must enter INFILE.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	LSTDLTF must be run under TAE.

PROJECT:	LAS

ALGORITHM:

	Get the user specified parameters.
	Set the DLTF extension.
	If the user did not specifiy a DLTF entry name then output a summary
	  report of all entries in the DLTF.
	Else output a detailed listing for the specified entry.
	Free the dynamically allocated memory.
	Close the print file if one exists.
	Issue a standard completion message based on whether an output file was
	  created or not.
	Terminate processing.
	
*****************************************************************************/
#define UTIL 1
#define DMSLEN 12
#define LUTSIZ 256
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "dltf.h"
#include "protos.h"

int main
(void)
{
long	i,j,k,m,n,	/* Loop indices; working variables.		*/
	bnd,		/* Current band number to be printed.		*/
	status,		/* Return status indicator. 			*/
	zero = 0,	/* Constant zero; specifies read access; specifies
			   that pterm is to output a page of information
			   before prompting the user.			*/
	fatal = -1,	/* Specifies a fatal error.			*/
	one = 1,	/* Specifies a non-fatal error.			*/
	start,		/* Starting point of the LUT.			*/
	end,		/* Ending point of the LUT.			*/
	fd,		/* File descriptor for the DLTF file. 		*/
	clen,		/* Length of the character portion of the DLTF. */
	dlen,		/* Length of the data portion of the DLTF.	*/
	temp,		/* Maximum number of entries evenly divisible by
			   five; this is used in the formatting of I*2
			   LUT lines.					*/
	nument,		/* Total number of entries to be output; used in
			   the formatting of I*2 LUT data.		*/
	extra,		/* Number of extra entries after the number of
			   entries that is evenly divisible by five is
			   calculated (temp above).			*/
	prtflg = FALSE,	/* Flag that indicates whether a print file was
			   created or not.				*/
	term,		/* Flag to indicate if output is destined for the
			   terminal.					*/
	fyle,		/* Flag to indicate if output is destined for a
			   user specified file.				*/
	lnptr;		/* Flag to indicate if output is destined for the
			   line printer.				*/

double	slope,		/* Slope of the LUT; this is used if DMS saves
			   a non-byte LUT (see SAVDLTF).		*/
	offset;		/* Offset of the LUT; this is used if DMS saves
			   a non-byte LUT (see SAVDLTF).		*/

FILE	*pfp = NULL;	/* Pointer to a print file. 			*/

char	hostnm[CMLEN],  /* Host name of the DLTF assoiciated file.	*/
	dltfnam[DMSLEN+1],/* Name of the DLTF entry to be read. 	*/
	*type1,		/* Working variable used to ascertain the DLTF
			   record type.					*/
	key[KEYTL],	/* DLTF entry name; key value for Label
			   Services.					*/
	dtype[DTYPL],	/* Data type of the DLTF.			*/
	line[12][80],	/* Working line format buffer.			*/
	desc[DESLEN],	/* Description associated with the entry. 	*/
	prtfil[1][CMLEN],	/* Print file name.				*/
	uoutfil[1][CMLEN];	/* Working print file name used in the completion
			   message.					*/

char tmpstr[1][CMLEN];  /* Temp string */

unsigned char  *lut = NULL,	/* Lookup table data. 				*/
	*byteptr;	/* Working BYTE pointer into the LUT buffer.	*/

short	*shrtptr;	/* Working I*2 pointer into the LUT data buffer
			   for type casting.				*/

struct CDLTF cdltf;	/* Structure to contain the character portion of
			   the first type of DLTF record; used in a
			   summary report.				*/

struct ASDLTF asdltf;	/* Structure to contain DLTF information for a
			   detailed listing of a DLTF entry.		*/

struct PARBLK vblock;	/* TAE variable block.				*/

/* Get the user specified parameters.
  -----------------------------------*/
getpar(&vblock,dltfnam,&term,&fyle,&lnptr,prtfil[0],uoutfil[0],hostnm);

/* If the user specified nothing for the DLTF entry name then output a
   summary report.
  ------------------------------------------------------*/
if ((strcmp(dltfnam,"")) == 0)
  {
	/* Create the report explanation line.
          -----------------------------------*/
	strcpy(line[0],"                   ");
	strcat(line[0],"- Display and Look-up Table File -");

	strcpy(line[1],"                   ");
	strcat(line[1],"==================================");

	sprintf(line[2],"     Source file:\t%s", hostnm);

	if ((lnptr) || (fyle))
  	  {
    		pfp = fopen(prtfil[0],"w");
		prtflg = TRUE;
    		fprintf(pfp,"%s\n",line[0]);
    		fprintf(pfp,"%s\n\n",line[1]);
    		fprintf(pfp,"%s\n\n",line[2]);
  	  }

	if (term)
  	  {
    		c_pterm(line[0],&zero);
    		c_pterm(line[1],&zero);
	  	c_pterm("",&zero);
    		c_pterm(line[2],&zero);
	  	c_pterm("",&zero);
  	  }

	/* Format the DLTF entry explanation line for the summary report
	   of each entry.
 	  ------------------------------------------------*/
	strcpy(line[3],"Entry Name      Bands    Dtype   ");
	strcat(line[3],"SL      SS      NL      NS     ");

	strcpy(line[4],"==========      =====    =====   ");
	strcat(line[4],"==      ==      ==      ==     ");

	if ((lnptr) || (fyle))
	  {
	   fprintf(pfp,"%s\n",line[3]);
	   fprintf(pfp,"%s\n",line[4]);
	  }

	if (term)
	  {
	   c_pterm(line[3],&zero);
	   c_pterm(line[4],&zero);
	  }

	/* Get the DLTF information for each entry and output it.
	  ------------------------------------------------------*/
	status = c_lsopen(&fd,hostnm,&zero);
	if (status != E_SUCC)
	  {
	   if (prtflg)
	    fclose(pfp);
	   c_clean(&vblock,"lstdltf",prtfil,&one);
	  }

	/* Set the key field (entry name) to null so that the key of the
	   current record read will be returned.  Also, assign the length
	   of data portion of the DLTF to zero so that Label Services will 
	   return its true length.
	   ------------------------------------------------------------*/
	strcpy(key,"");
	clen = MXDLTF;
	dlen = 0;

	status = c_lsread(&fd,key,&clen,&dlen,(char *) &cdltf,lut,dtype);
	if ((status != E_EOF) && (status != E_SUCC))
	  {
	   if (prtflg)
	    fclose(pfp);
	   c_clean(&vblock,"lstdltf",prtfil,&one);
	  }

	while (status != E_EOF)
 	  {
		/* If the return pointer does not equal null (found	
		   occurence of "@") then it must be the first DLTF 
		   record type.
		  -----------------------------------------------*/
		type1 = strchr(key,'@');
		if (type1 != 0)
		  {
			/* Strip off the "@" appended to the entry name to
			   prevent possible collisions.
			  ---------------------------------*/
			*type1 = '\0';

			/* Format the output information from the DLTF.
			  --------------------------------------------*/
			sprintf(line[5],
		  	"%-16.16s  %-5s    %-4.4s  %-7s %-7s %-7s %-7s",
		         key,cdltf.numbnd,dtype,cdltf.window[0],
			 cdltf.window[1],cdltf.window[2],cdltf.window[3]);

			/* Output the information.
			  -----------------------*/
			if ((lnptr) || (fyle))
			 {
			  fprintf(pfp,"%s\n",line[5]);
			  fprintf(pfp,"%s\n\n",cdltf.descr);
			 }

			if (term)
			 {
			  c_pterm(line[5],&zero);
			  c_pterm(cdltf.descr,&zero);	
			  c_pterm("",&zero);
			 }
		  }

		/* Reassign the key (DLTF entry name) for another call 
		   to lsread.
		  -------------------------------------------*/
		strcpy(key,"");
		clen = MXDLTF;

		status = c_lsread(&fd,key,&clen,&dlen,(char *) &cdltf,lut,dtype);

		/* If the return status indicates "buffers too small" then
		   allocate sufficient memory to contain the LUT data for
		   the second type of DLTF record.  This information is
		   not used but must be read past.
		  ---------------------------------------------*/
		if (status == E_SMAL)
		  {
		   if ((lut = (unsigned char *)malloc(dlen)) == 0)
		    {
		     c_errmsg("Error allocating buffer space for DLTF record",
		              "lstdltf-nomem",&one);
	             if (prtflg)
		      fclose(pfp);
		     c_clean(&vblock,"lstdltf",prtfil,&one);
		    }
		  }
		else if ((status != E_EOF) && (status != E_SUCC))
		  {
	           if (prtflg)
		    fclose(pfp);
		   c_clean(&vblock,"lstdltf",prtfil,&one);
		  }
	}

	/* Close the DLTF file and possibly the output print file.
	  ------------------------------------------------------*/
	status = c_cldltf(&fd,hostnm);
	if (status != E_SUCC)
	 {
	  if (prtflg)
	   fclose(pfp);
	  c_clean(&vblock,"lstdltf",prtfil,&one);
	 }

	if ((lnptr) || (fyle))
	  fclose(pfp);
  }
else 
  {
/* Output a detailed listing of the fields for a single DLTF entry.
  ----------------------------------------------------------------*/
	/* Read the first record of the DLTF.
	  ----------------------------------*/
	if (c_r1dltf(&fd,hostnm,dltfnam,desc,&asdltf,&dlen,dtype) != E_SUCC)
	   c_errmsg("Fatal error encountered","lstdltf-fatal",&fatal);

	/* If successful then allocate memory to contain the actual LUT(s)
	   from the second type of DLTF record.
	  -----------------------------------------------*/
	if ((lut = (unsigned char *)malloc(dlen)) == 0)
	  {
	   c_errmsg("Error allocating buffer space for DLTF record",
	            "lstdltf-nomem",&one);
	   if (prtflg)
	    fclose(pfp);
	   c_clean(&vblock,"lstdltf",prtfil,&one);
	  }

	/* Output information from the first DLTF record type before 
	   processing the second type.
	   ------------------------------------------*/
	/* Create the report explanation lines.
           ------------------------------------*/

	strcpy(line[0],"                   ");
	strcat(line[0],"- Display and Look-up Table File -");

	strcpy(line[1],"                   ");
	strcat(line[1],"==================================");

	sprintf(line[2],"     Source file:\t%s", hostnm);

	/* Format the first explanation and data lines.			*/

	strcpy(line[3],"Entry Name      Bands    Dtype   ");
	strcat(line[3],"SL      SS      NL      NS     ");

	strcpy(line[4],"==========      =====    =====   ");
	strcat(line[4],"==      ==      ==      ==     ");

	sprintf(line[5],
	          "%-16.16s  %-5d    %-4.4s  %-7d %-7d %-7d %-7d",
	          dltfnam,asdltf.numbnd,dtype,asdltf.window[0],
		  asdltf.window[1],asdltf.window[2],asdltf.window[3]);

	/* Format the second explanation and data lines.		*/

	sprintf(line[6],"Izoomx  Izoomy  Gzoomx  Gzoomy");
	sprintf(line[7],"======  ======  ======  ======");

	sprintf(line[8],
		  "%-7d %-7d %-7d %-7d",asdltf.izoom[0],
		  asdltf.izoom[1],asdltf.gzoom[0],asdltf.gzoom[1]);

	/* Format the output for the third explanation line.
	  --------------------------------------------------*/
	sprintf(line[9],"Ishftx  Ishfty  Gshftx  Gshfty");
	sprintf(line[10],"======  ======  ======  ======");

	sprintf(line[11],
		  "%-7d %-7d %-7d %-7d",asdltf.ishift[0],
		  asdltf.ishift[1],asdltf.gshift[0],asdltf.gshift[1]);

	/* Output the information from the first type of DLTF record
	   (except the actual band numbers).
	  ------------------------------------------------*/
	if ((lnptr) || (fyle))
	 {
	  pfp = fopen(prtfil[0],"w");
	  prtflg = TRUE;
	  fprintf(pfp,"%s\n",line[0]);
	  fprintf(pfp,"%s\n\n",line[1]);
	  fprintf(pfp,"%s\n\n",line[2]);
	  fprintf(pfp,"%s\n\n",desc);
	  fprintf(pfp,"%s\n",line[3]);
	  fprintf(pfp,"%s\n",line[4]);
	  fprintf(pfp,"%s\n\n",line[5]);
	  fprintf(pfp,"%s\n",line[6]);
	  fprintf(pfp,"%s\n",line[7]);
	  fprintf(pfp,"%s\n\n",line[8]);
	  fprintf(pfp,"%s\n",line[9]);
	  fprintf(pfp,"%s\n",line[10]);
	  fprintf(pfp,"%s\n\n",line[11]);
	 }

	if (term)
	 {
	  c_pterm(line[0],&zero);
	  c_pterm(line[1],&zero);
	  c_pterm("",&zero);
	  c_pterm(line[2],&zero);
	  c_pterm("",&zero);
	  c_pterm(desc,&zero);	
	  c_pterm("",&zero);
	  c_pterm(line[3],&zero);
	  c_pterm(line[4],&zero);
	  c_pterm(line[5],&zero);
	  c_pterm("",&zero);
	  c_pterm(line[6],&zero);
	  c_pterm(line[7],&zero);
	  c_pterm(line[8],&zero);
	  c_pterm("",&zero);
	  c_pterm(line[9],&zero);
	  c_pterm(line[10],&zero);
	  c_pterm(line[11],&zero);
	  c_pterm("",&zero);
	 }

	/* Read each of the second type of records (one for each LUT)
	   and output the information accordingly.  Note that we also
	   output the band numbers from the first type of DLTF record,
	   they serve as the identifier for each LUT since we do not
	   have a strictly RGB relationship anymore.
	  ---------------------------------------------------------*/
	for (i = 0, bnd = 1; i < asdltf.numbnd; i++, bnd++)
	  {
	   status = c_r2dltf(&fd,dltfnam,&bnd,&start,&end,&dlen,lut,
			      &slope,&offset);

	   if (status != E_SUCC)
	    {
	     if (prtflg)
	      fclose(pfp);
	     c_clean(&vblock,"lstdltf",prtfil,&one);
	    }

	   /* Format the explanation line for the data from the second
	      record.
	     ----------------------------------------------------*/
	   strcpy(line[0],
	       "Band   Startpt      Endpt        Slope          Offset");

	   strcpy(line[1],
	       "====   =======      =====        =====          ======");

	   /* Format the data line for the second record.
	    --------------------------------------------*/
	   sprintf(line[2],"%-4d   %-12d %-12d %-.10f   %-.10f",
		   asdltf.band[i],start,end,slope,offset);

	   /* Output the information from the second record.
	     -----------------------------------------------*/
	   if ((lnptr) || (fyle))
	    {
		fprintf(pfp,"\n%s\n",line[0]);
		fprintf(pfp,"%s\n",line[1]);
		fprintf(pfp,"%s\n\n",line[2]);
	    }

	   if (term)
	    {
		c_pterm("",&zero);
		c_pterm(line[0],&zero);
		c_pterm(line[1],&zero);
		c_pterm(line[2],&zero);
		c_pterm("",&zero);
	    }

	/* Output the LUT information based upon the data type of
	   the DLTF.
	  ---------------------------------------------*/
	if ((strcmp(dtype,"B") == 0))
	  {
	   byteptr = lut;
	   for (j = 0; j < LUTSIZ; j+=32)
	     {
		/* Format the BYTE LUT for output.  For efficiency output 
		   eight lines at a time.
		  ------------------------------------------*/
		for (k = 0, m = j; k < 8; k++, m+=4, byteptr+=4)
		   sprintf(line[k],
		"[%3d]: %-4d  %-4d  %-4d  %-4d",
		   m,*byteptr,*(byteptr+1),*(byteptr+2),*(byteptr+3));

		/* Output the LUT information.
		  ---------------------------*/
		if ((lnptr) || (fyle))
		   for (k = 0; k < 8; k++)
			fprintf(pfp,"%s\n",line[k]);

		if (term)
		   for (k = 0; k < 8; k++)
			c_pterm(line[k],&zero);
	     }
	  }
	else
	  {
		/* Format the I*2 LUT for output.  The algorithm below
		   calculates the number of times five lines may be output
		   at one time along with any extra.  The alogorithm takes
		   into account such possibilities as a negative range and
		   a range less than five.
		  -------------------------------------------------------*/
		temp = nument = (end - start) + 1;

		if ((nument % 5) != 0)
		  while (((temp % 5) != 0) && (temp > 0))
			temp--;

		extra = nument - temp;
		shrtptr = (short *)lut;

		if (temp != 0)
		  for (m = start; m < (temp + start); m+=5)
		    {
		     sprintf(line[0],
			"[%7d] : %-7d  %-7d  %-7d  %-7d %-7d",
			m,*shrtptr,*(shrtptr+1),*(shrtptr+2),*(shrtptr+3),
			*(shrtptr+4));

		     if ((lnptr) || (fyle))
			fprintf(pfp,"%s\n",line[0]);

		     if (term)
			c_pterm(line[0],&zero);

		     shrtptr+=5;
		    }

		if (extra != 0)
		  {
		   if (temp != 0)
			n = temp + start;
		   else
			n = start;

		   switch (extra)
		    {
			case 1 : sprintf(line[0],
				     "[%7d] : %-7d",
				     n,*shrtptr);
				 break;
			case 2 : sprintf(line[0],
				     "[%7d] : %-7d  %-7d",
				     n,*shrtptr,*(shrtptr+1));
				 break;
			case 3 : sprintf(line[0],
				     "[%7d] : %-7d  %-7d  %-7d",
				     n,*shrtptr,*(shrtptr+1),*(shrtptr+2));
				 break;
			case 4 : sprintf(line[0],
				     "[%7d] : %-7d  %-7d  %-7d  %-7d",
				     n,*shrtptr,*(shrtptr+1),*(shrtptr+2),
				     *(shrtptr+3));
				 break;
		    }

		   if ((lnptr) || (fyle))
			fprintf(pfp,"%s\n",line[0]);

		   if (term)
			c_pterm(line[0],&zero);
		  }
	  }
	}
  }

	/* Free the dynamically allocated memory.
	  --------------------------------------*/
	free(lut);

	/* Issue a standard completion message based on whether an
	   output file was created.
	  ---------------------------------------------*/
	if (fyle)
	  {
	  fclose(pfp);
          c_complt("LSTDLTF",uoutfil,&one);
	  }
	else
	  {
      strcpy(tmpstr[0],"\0");
      c_complt("LSTDLTF",tmpstr,&zero);
      }

return EXIT_SUCCESS;

}
