/******************************************************************************
FUNCTION:	GETLEAD

PURPOSE:	Read the leader file from tape and write it to disk if
		specified.  Extract needed information.

ALGORITHM REFERENCES	none
*******************************************************************************/
#define PI    3.141592653589793238 /* PI */

#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "ltwg2in.h"
#include "labserv.h"
#include "protos.h"

int getlead
(
    long *unit,		   /* I: tape unit number			*/
    long *wrtdsk,	   /* I: ancillary files to read in		*/
    char hosout[][CMLEN],  /* I/O : Output host names                  	*/
    long *num_out, 	   /* I/O: number of output files             	*/
    long *numlead,	   /* I: number of leader records		*/
    long *leadsize,	   /* I: size of leader records			*/
    double fcenll[],       /* O: Full scene center latitude/longitude degrees */
    double fcenls[],       /* O: Full scene center line/sample          */
    long *path,		   /* O: scene path				*/
    char *direction,	   /* O: satellite direction			*/
    char *maprj,           /* O: map projection				*/
    double pixsiz[],	   /* O: image pixel size			*/
    long *satid,           /* O: satellite id.				*/
    char *sensor,	   /* O: satellite sensor			*/
    double *angle,	   /* O: angle of rotation in radians		*/
    long *utmzone,	   /* O: UTM zone code				*/
    char *satnam,          /* O: satellite name                         */
    char cornlat[][17],	   /* O: corner latitudes                       */
    char cornlong[][17]	   /* O: corner longitudes                      */
)

{
FILE *fp;		/* file pointer					*/

unsigned char *in_buf;	/* buffer read from data on tape           	*/
char tempbuf[17];	/* temp work buffer				*/

long i=0;		/* loop counter					*/
long mode = HANDLE;	/* tape I/O error flag                        	*/
long one = 1;		/* constant of one				*/
long status=0;		/* status flag 					*/
double tzone=0.0;	/* temp double value				*/

/* Create the leader file name
------------------------------*/
if (*wrtdsk)
   {
   c_lsmknm(hosout[0],"lead",hosout[*num_out]);
   fp = fopen(hosout[(*num_out)++],"w");
   }

/* Allocate space
-----------------*/
in_buf = (unsigned char *) malloc(*leadsize);
if (in_buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
   return(E_FAIL);
   }

/* Read and optionally write to disk.  Also extract needed information
---------------------------------------------------------------------*/
for (i = 0; i < *numlead; i++)
    {
    status = c_tpread(unit,in_buf,leadsize,&mode);
    if (status != *leadsize) 
       {
       c_errmsg("Error reading from tape","ltwg2in-read",&one);
       return(E_FAIL);
       }

    if (i == 0)
       {
       /* Get the satellite name, number and sensor
       --------------------------------------------*/
       strncpy(satnam,(const char *)&in_buf[48],2);
       strncpy(tempbuf,(const char *)&in_buf[50],1);
       tempbuf[1] = '\0';
       *satid = atol(tempbuf);
       if (strncmp(satnam,"LS",2) == 0)
          strncpy(sensor,(const char *)&in_buf[52],2);
       else if (strncmp(satnam,"SP",2) == 0)
          strncpy(sensor,(const char *)&in_buf[52],3);
       }
    else if (i == 1)
       {
       if (strncmp(satnam,"LS",2) == 0)
          {
          /* Retrieve the full scene center lat/long 
          ------------------------------------------*/
          strncpy(tempbuf,(const char *)&in_buf[52],16);
          tempbuf[16] = '\0';
          sscanf(tempbuf,"%lf",&fcenll[0]);

          strncpy(tempbuf,(const char *)&in_buf[68],16);
          tempbuf[16] = '\0';
          sscanf(tempbuf,"%lf",&fcenll[1]);

          /* Get the full scene center line/sample 
          ----------------------------------------*/
          strncpy(tempbuf,(const char *)&in_buf[84],16);
          tempbuf[16] = '\0';
          sscanf(tempbuf,"%lf",&fcenls[0]);
          strncpy(tempbuf,(const char *)&in_buf[100],16);
          tempbuf[16] = '\0';
          sscanf(tempbuf,"%lf",&fcenls[1]);

          /* Get the WRS path number 
          -------------------------*/
          strncpy(tempbuf,(const char *)&in_buf[165],3);
          tempbuf[3] = '\0';
          sscanf(tempbuf,"%d",path);

          /* Get the orbital direction 
          ----------------------------*/
          strncpy(direction,(const char *)&in_buf[356],10);
          direction[9] = '\0';

          /* Get the map projection 
          -------------------------*/
          strncpy(maprj,(const char *)&in_buf[1556],3);
          maprj[3] = '\0';
          }
       else if (strncmp(satnam,"SP",2) == 0)
          {
          /* Get the four corners 
          -----------------------*/
          strncpy(cornlat[0],(const char *)&in_buf[148],16);
          strncpy(cornlong[0],(const char *)&in_buf[164],16);

          strncpy(cornlat[1],(const char *)&in_buf[212],16);
          strncpy(cornlong[1],(const char *)&in_buf[228],16);
          
          strncpy(cornlat[2],(const char *)&in_buf[276],16);
          strncpy(cornlong[2],(const char *)&in_buf[292],16);

          strncpy(cornlat[3],(const char *)&in_buf[340],16);
          strncpy(cornlong[3],(const char *)&in_buf[356],16);
          }
       }
    else if (i == 2)
       {
       /* Retrieve the pixel size and convert from km to meters
       -------------------------------------------------------*/
       strncpy(tempbuf,(const char *)&in_buf[364],16);
       tempbuf[16] = '\0';
       sscanf(tempbuf,"%lf",&pixsiz[1]);
       pixsiz[1] *= 1000;

       strncpy(tempbuf,(const char *)&in_buf[380],16);
       tempbuf[16] = '\0';
       sscanf(tempbuf,"%lf",&pixsiz[0]);
       pixsiz[0] *= 1000;

       /* Get the rotation angle and convert to radians
       ------------------------------------------------*/
       strncpy(tempbuf,(const char *)&in_buf[204],16);
       tempbuf[16] = '\0';
       sscanf(tempbuf,"%lf",angle);
       *angle = *angle * PI/180.0;

       /* Get the UTM zone 
       -------------------*/
       strncpy(tempbuf,(const char *)&in_buf[396],16);
       tempbuf[16] = '\0';
       sscanf(tempbuf,"%lf",&tzone);
       *utmzone = (long)tzone;
       }

    /* Write to disk
    ----------------*/
    if (*wrtdsk)
       fwrite(in_buf,1,*leadsize,fp);
    }

/* Close the file
-----------------*/
if (*wrtdsk)
   fclose(fp);

/* One more read for EOF
------------------------*/
status = c_tpread(unit,in_buf,leadsize,&mode);
if (status != TAPE_EOF) 
   {
   c_errmsg("Error reading from tape","ltwg2in-read",&one);
   return(E_FAIL);
   }

/* Free allocated space
----------------------*/
free(in_buf);

return(E_SUCC);
}
