/******************************************************************************
FUNCTION:	LTWG2IN

PURPOSE:	Read in image files from tapes that are in the LTWG2 format 
                and optionally read the ancillary data

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
   1.0	 04/95  K. Lowell	 Original development/NLAPS
   2.0   12/99  D. Hames         Remove Valid=(1:50000) from WINDOW parameter
				 in ltwg2in.pdf.  Add error check in getpar to 	
				 ensure values of WINDOW are >= 1. Modified to 
			         Modified to support images > 2 gig. Add
		         	 prototypes. SRF #893.
				 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltwg2in.h"
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
int    fdesc[2];		/* image file descriptors used for image I/O */
long   gdesc;			/* group descriptor used for image I/O       */
long   offset[MAXBND + 1];	/* offsets into "thebuf" for output bands    */

void main
(void)
{
struct PARBLK vblock;	/* TAE parameter block                        */

long bands[MAX_BAND]; 	/* bands to be processed within image         */
long msgtim=0;		/* elapsed time between display msg           */
long num_out = 1;	/* number of output files 		      */ 
long one = 1;		/* value of one			              */
long path=0;		/* satellite path			      */
long unit=0;           	/* tape unit number                           */
long totbnd = 0;	/* total number of bands to be copied         */
long wrtdsk[8];        	/* ancillary files to read in 		      */
long window[4];		/* window of input image                      */
long zero = 0;		/* value of zero                              */
long leadsize=0;	/* size of leader records		      */
long fatal = -1;	/* fatal error 				      */
long minmax=0;		/* min max flag                               */
long no_lead = FALSE;   /* Flag if successive tapes will have leader files */
long numimg_recs=0;	/* number of image records on current tape    */
long numtapes=0;	/* number of tapes specified                  */
long num_trailrec=0;	/* number of trailer records                  */
long numlead=0;		/* number of leader records		      */
long recsize=0;		/* image record length                        */
long tapenum = 1;	/* tape number currently mounted              */
long trailsiz=0;	/* max size of trailer records		      */
long satid=0;		/* satellite id				      */
long utmzone=0;		/* UTM zone code			      */
char bandnum[7][1];	/* Band numbers to write to disk	      */
long bandrecsiz[7];	/* Band record sizes			      */
long bandnumrec[7];	/* Number of records per band		      */
long mode = 1;		/* Satellite mode			      */

char comment[3][CMLEN];	/* tape mount message to the operator         */
char hosout[8][CMLEN];	/* Output host file name                      */
char taeout[8][CMLEN];	/* Output TAE file names	              */
char order[4];		/* band order on tape                         */
char maprj[5];		/* map projection			      */
char direction[10];	/* satellite direction			      */
char sensor[5];		/* sensor type				      */
char satnam[5];		/* satellite name			      */
char cornlat[4][17];	/* corner latitudes			      */
char cornlong[4][17];	/* corner longitudes			      */
char ttype[CMLEN];	/* input tape type			      */

double fcenll[2];       /* Full scene center latitude/longitude degrees */
double fcenls[2];       /* Full scene center line/sample              */
double pixsiz[2];	/* image pixel size			      */
double angle;		/* rotation angle 			      */

/* Retrieve the input parameters
--------------------------------*/
getpar(&vblock, bands, &msgtim, window, &minmax, hosout, taeout, 
       comment, wrtdsk, &numtapes);

/* Open the tape
---------------*/
if (mount_tape(&unit, comment, one, &numtapes,ttype) != E_SUCC)
   c_errmsg("Fatal error encountered","ltwg2in-fatal",&fatal); 

/* Read the volume directory file and write to disk if specified
---------------------------------------------------------------*/
if (getvdf(&unit, &wrtdsk[VDF], hosout, &num_out, &recsize, &numimg_recs, 
           &num_trailrec, order, &numlead, &leadsize, &trailsiz,
	   bandnum, bandrecsiz,bandnumrec,&mode) != E_SUCC)
   {
   close_tape(&unit);
   c_clean(&vblock,"ltwg2in",hosout,&num_out);
   }

/* Read the leader file and write to disk if specified
-----------------------------------------------------*/
if (getlead(&unit, &wrtdsk[LEAD], hosout, &num_out, &numlead, &leadsize, fcenll,
            fcenls, &path, direction, maprj, pixsiz, &satid, sensor, &angle, 
            &utmzone, satnam, cornlat, cornlong) != E_SUCC)
   {
   close_tape(&unit);
   c_clean(&vblock,"ltwg2in",hosout,&num_out);
   }

/* Process the image data
------------------------*/
  if (mode == 1)
   {
   if (proces_bsq_m1(hosout, window, &totbnd, &unit, wrtdsk, &num_out, bands, 
                  &msgtim, &minmax, comment, &recsize, &tapenum,&numtapes,
                  &no_lead,ttype, bandrecsiz,bandnumrec) != E_SUCC)
      {
      close_tape(&unit);
      c_clean(&vblock,"ltwg2in",hosout,&num_out);
      }
    }
  

/* Get the trailer and supplemental files if requested
------------------------------------------------------*/
if (wrtdsk[TRAI] || wrtdsk[SUPP] || wrtdsk[CAL] || wrtdsk[WOR] || wrtdsk[HST])
   if (getenf(hosout, &unit, wrtdsk, &num_out, comment, &num_trailrec, &tapenum,
              &trailsiz, &numtapes, &no_lead, satnam) != E_SUCC)
      {
      close_tape(&unit);
      c_clean(&vblock,"ltwg2in",hosout,&num_out);
      }

/* Dismount the tape
--------------------*/
close_tape(&unit);

/* Update the DDR file 
---------------------*/
if (update_ddr(hosout, window, &totbnd, fcenll, fcenls, &path, direction, maprj,
               pixsiz, &satid, sensor, &angle, &utmzone, satnam, cornlat,
               cornlong) != E_SUCC)
   c_errmsg("Nonfatal error encountered","ltwg2in-nonfatal",&one);

/* Output the history information
---------------------------------*/
if (c_hstout(&vblock,(char *)hosout,hosout,&zero," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","ltwg2in-nonfatal",&one);

c_complt("LTWG2IN",hosout,&num_out);
}
