/******************************************************************************
FUNCTION:	PROCES_BSQ

PURPOSE:	Read image data from tape and write it to disk.  The image
                data is BSQ.  Each image line contains CEOS standard 
                12 bytes, suffix and prefix data, and image data.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "ltwg2in.h"
#include "string.h"
#include "labserv.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

int proces_bsq_m1
(
    char hosout[][CMLEN],   /* I/O: output host names  		      	     */
    long *window,	    /* I: window area of image to be copied 	     */
    long *totbnd,	    /* O: total number of bands to be copied	     */
    long *unit,             /* I/O: tape unit number                         */
    long *wrtdsk,           /* I: ancillary files to read in                 */
    long *num_out,          /* I/O: number of output files 		     */
    long *bands,            /* I: bands to be processed within image         */
    long *msgtim,	    /* I: elapsed time between display msg  	     */
    long *minmax,	    /* I: min max flag                      	     */
    char comment[][CMLEN],  /* I: tape mount message to the operator         */
    long *recsize,          /* I: image record length                        */
    long *tapenum,	    /* I/O: tape number mounted 		     */
    long *numtapes,	    /* I: number of tapes specified 		     */
    long *no_lead,	    /* I/O: flag if successive tapes will have leader 
			            file */
    char ttype[],	    /* I/O: tape type				     */
    long *bandrecsiz,	    /* I: Band record sizes                          */ 
    long *bandnumrec	    /* I: Number of records per band                 */
)

{
extern long curcat;     /* curent tape band being processed          */
extern long curent;     /* output line being processed               */
extern int  fdesc[];	/* image file descriptors used for image I/O */

FILE *fp;		/* file descriptor			     */
   
long  access = IWRITE;  /* write access to output image      	     */
long  bandoff[MAX_BAND];/* offset into each band to get to image data */
long  cb=0;               /* current tape band being processed  */
long  idtype=0;		/* data type of input image          	     */
long  mode = HANDLE;    /* tape I/O error flag			     */
long  msgflg=0;		/* Processing message format flag	     */
long  one = 1;		/* constant one			             */
long  option[2];	/* output option array		             */
long  outspc[2];	/* output image file specifications  	     */
long  status=0;		/* return status                             */
long  temp=0;		/* temp value                                */
long  dumlong=0;	/* dummy value				     */
long  dum2 = FALSE;	/* dummy value				     */
long  prefix,suffix;	/* number of prefix and suffix pixles        */
long  curblk=0;		/* current block within the blocked data     */
long  recleng=0;	/* record length of image descriptor record  */
long  skip=0;		/* number of files to skip		     */
long  dmod=0;		/* part of curblk that is needed for the window */
long  ptrband = 1;	/* pointer to which band the tape is currently at */
long  swreel = FALSE;	/* flag if need to switch reels		     */
long  blkfct=0;		/* blocking factor			     */
long  bndcnt = 0;	/* number of bands requested		     */
long  lrecsiz = 0;      /* numeric record size			     */
long  ii,jj,kk;		/* process counters			     */
long  cnt;		/* Offset loop counter for band 6	     */
long lastband=0;	/* last band processed				*/

static long datasize[4] = {0,1,2,4};
			/* defines number bytes for each data type */

unsigned char *in_buf;          /* buffer read from image on tape    */
unsigned char *in_ptr;          /* pointer to input buffer window    */
unsigned char *out_buf;		/* buffer write for thermal expansion */
unsigned char *out_ptr;		/* thermal expansion pointer	     */
unsigned char *out_ptr2;
char dumbstr[7][1];		/* dummy value			     */
char tempbuf[10];		/* temp work buffer 		     */

double outmax[MAX_BAND];        /* max pixel values of output image  */
double outmin[MAX_BAND];        /* min pixel values of output image  */

/* Allocate memory for the input buffer
---------------------------------------*/
in_buf = (unsigned char *) malloc(*recsize);
if (in_buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
   return(E_FAIL);
   }

/* Allocate memory for the thermal expansion buffer.
   The thermal band must be multiplied by 4x in line
   and the out_buf written four times due to sensor
   detector differences
----------------------------------------------------*/
out_buf = (unsigned char *) malloc(*recsize);
if (in_buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
   return(E_FAIL);
   }

      
/* Read first line.  This is the file descriptor record in the image file. 
   Get the record length.  This is different then the record size if the 
   image data is blocked.
-------------------------------------------------------------------------*/
status = c_tpread(unit,in_buf,recsize,&mode);
recleng = atol((const char *)&in_buf[186]);
if (status != recleng)
   return(E_FAIL);

/* If specified, open the image data logical record file and write this
   record.  This file contains the file descriptor record from the
   image file and the prefix and suffix data from the image line.
---------------------------------------------------------------*/
if (wrtdsk[IDR])
   {
   c_lsmknm(hosout[0],"idr",hosout[*num_out]);
   fp = fopen(hosout[(*num_out)++],"w");
   fwrite(in_buf,1,*recsize,fp);
   }

/* Set offsets into each line to get to image data. 
   xx bytes of prefix data and xx bytes of suffix data.
   12 bytes are for standard CEOS record
------------------------------------------------------*/
prefix = atol((const char *)&in_buf[276]);
suffix = atol((const char *)&in_buf[288]);
for (ii = 0; ii < MAX_BAND; ii++)
    bandoff[ii] = prefix + 12;

/* Get data type (bits per pixel)
---------------------------------*/
temp = atol((const char *)&in_buf[216]);
switch (temp)
    {
    case 8:  idtype = EBYTE; break;
    default: c_errmsg("Data type not currently handled","ltwg2in-dtype",&one);
             return(E_FAIL);
    }

/* Get blocking factor.  Blocking factor was not correctly specified within
   the file descriptor record.  Calculating the blocking factor instead
   should work in all instances.
---------------------------------------------------------------------------*/
/*blkfct = atol(&in_buf[272]);*/
blkfct = *recsize/recleng;

/* Get the image size on tape
----------------------------*/
outspc[0]= atol((const char *)&in_buf[236]);
strncpy(tempbuf,(const char *)&in_buf[248],8);
outspc[1]= atol(tempbuf);

/* Count number of bands the user wants 
---------------------------------------*/
for (ii = 0; ii < MAX_BAND; ii++)
    if (bands[ii])
       bndcnt++;
*totbnd = bndcnt;

/* Check the window
-------------------*/
chk_window(outspc,window);

/* Open the output image file for random write access
-----------------------------------------------------*/
option[0] = 0;
option[1] = 0;
if (c_eopenr((struct FDESC **)&fdesc[0],hosout[0],totbnd,&one,&one,&window[NL],
	     &window[NS],&access,&idtype,option,&one) != E_SUCC)
   return(E_FAIL);
fdesc[1] = 0;
      
/* If min max values are to be computed, call inmnmx to
   initialize the min max array for the output data type.
--------------------------------------------------------*/
if (*minmax)
   c_inmnmx(&idtype,totbnd,outmin,outmax);

/* Initialize the processing message
------------------------------------*/
msgflg = ONECAT;
c_inittm("LTWG2IN","Processed","line","band","",&window[NL],totbnd,
	 msgtim,&msgflg);

/* Start processing message
---------------------------*/
kk = 1;
curent = 1;
curcat = 1;
c_startm();

/* Reset for processing
-----------------------*/
curcat = 0;
lrecsiz = *recsize;

/* Process each requested band  
------------------------------*/
for (cb = 1; cb <= MAX_BAND; cb++)
    {
    /* Assuming bands are on three tapes - 3,3,1 - 9-track
                             two tapes   - 4,3   - 3480
                             one tape    - 7     - 8mm
    ---------------------------------------------------------------------*/
    if ((((cb == 4) || (cb == 7)) && (strcmp(ttype,"9-track") == NULL)) || 
       ((cb == 5)  && (strcmp(ttype,"3480") == NULL)))
       swreel = TRUE;
    if (strcmp(ttype,"8mm") == NULL)
       swreel = FALSE; 

    if (bands[cb-1])
       {
       lastband=cb;
       curcat++;
       curent = 1;

       /* Skip to correct band or window as needed
       -------------------------------------------*/
       if (skip_bsq(window, unit, comment, tapenum, numtapes, &cb,
                   &dmod, &blkfct, &ptrband, &swreel, bandrecsiz)  != E_SUCC)
          {
          c_errmsg("Error skipping records","ltwg2in-skip",&one);
          return(E_FAIL);
          }

       /* Process each line
       --------------------*/
       for (curent=0; curent < window[NL];) 
           {
           status = c_tpread(unit,in_buf,&lrecsiz,&mode);
           if (status == TAPE_EOT)
              {
              /* Close current tape, mount next tape, and read volume 
                 directory file.
              ---------------------------------------------------------------*/
              c_stoptm();
              close_tape(unit);
              if (mount_tape(unit, comment, ++(*tapenum), numtapes, 
		             ttype) != E_SUCC)
                  return(E_FAIL);

              if (getvdf(unit, &dum2, hosout, &dumlong, &lrecsiz, &dumlong, 
                          &dumlong,tempbuf,&dumlong,&dumlong,&dumlong,
			  dumbstr,&dumlong,&dumlong,&dumlong) != E_SUCC)
                   return(E_FAIL);

              *no_lead = TRUE;
              status = c_tpread(unit,in_buf,&lrecsiz,&mode);
              c_startm();
              }

	   /*if (status != lrecsiz)*/
	   if (status != bandrecsiz[cb-1])
              {
              c_errmsg("Error reading from tape","ltwg2in-read",&one);
              return(E_FAIL);
              }
           /* Position the input pointer to the start of the image data.
           ------------------------------------------------------------*/
	   if (window[SS] == 1)
                   cnt=0;
                 else
                    cnt = window[SS]/4;
           for (curblk=dmod; (curblk<blkfct) && (curent < window[NL]); curblk++)
               {
               if (cb!=6)
                 in_ptr = &in_buf[(bandrecsiz[cb-1] * curblk * 
			  datasize[idtype]) +
                          bandoff[cb-1] + 
                          ((window[SS] - 1) * datasize[idtype])];
               else
                  in_ptr = &in_buf[(bandrecsiz[cb-1] * curblk * 
                           datasize[idtype]) +
                           bandoff[cb-1] +
                           ((cnt - 1) * datasize[idtype])];

               /* Write the data to the output file
               -------------------------------------*/
               if (cb!=6)
                 {
                 curent++;
                 if(c_ewrite((struct FDESC **)&fdesc[0],&curcat,&curent,in_ptr,
			&one) == E_FAIL)
                   {
                   c_errmsg("Error writing output image","ltwg2in-write",&one);
                   return(E_FAIL);
                   }
                 }
               else
                 {
                 out_ptr = out_buf;
                 /*for (jj=cnt;jj<(lrecsiz-270);jj++)*/
                 for (jj=cnt;jj<(bandrecsiz[cb-1]-270);jj++)
                   {
                   for (ii=0;ii<4;ii++)
                     {
		     *(out_ptr++) = *in_ptr;
		     }
		   in_ptr++;
                   }
                 out_ptr2 = &out_buf[0];

		 for (ii=0;ii<4;ii++)
                   {
		   curent++;
                   if((status=c_ewrite((struct FDESC **)&fdesc[0],&curcat,
				&curent,out_ptr2,&one)) == E_FAIL)
                     {
                     c_errmsg("Error writing output image","ltwg2in-write",&one);
                     return(E_FAIL);
                     }
                   }
                 }
               }
           dmod = 0;

           /* Compute the min/max values if required
           -----------------------------------------*/
           if (*minmax)
              status = c_pxmnmx((long *)in_ptr,&window[NS],&outmin[curcat-1],
                                &outmax[curcat-1], &idtype);

        /* Write the image data logical record to disk
        ---------------------------------------------*/
        if (wrtdsk[IDR] && (cb == 1))
           {
           in_ptr = &in_buf[12];
           fwrite(in_ptr,1,prefix,fp);
           in_ptr = &in_buf[(lrecsiz - suffix)];
           fwrite(in_ptr,1,suffix,fp);
           }
        }

       /* Skip to the next image file.  If this is the last tape,
          skip to the end of the file as needed. 
          Assuming bands are on three tapes - 3,3,1
       ----------------------------------------------------------*/
	if (((cb != 3) && (cb != 6) && (strcmp(ttype,"9-track")==NULL)) ||
           ((cb != 4) && (strcmp(ttype,"3480") == NULL)) ||
           ((cb != 13) && (cb != 13) && (strcmp(ttype,"8mm") == NULL)))
          {
          status = c_tpread(unit,in_buf,&lrecsiz,&mode);
          if (status != TAPE_EOF)
             skip = 3;
          else 
             skip = 2;

          /* If this is the last tape, set skip as needed
          -----------------------------------------------*/
            if ((wrtdsk[TRAI] || wrtdsk[SUPP] || wrtdsk[CAL] || wrtdsk[WOR] || wrtdsk[HST]) 
                 && (*tapenum == *numtapes) &&
                (kk == *totbnd))
              {
              if (cb == 7) skip -=2;
              else /*all other bands*/
                {
                skip -=2;
                skip=(skip+((7-lastband)*3));
                }
              }
            status = c_tpfsf(unit,&skip,&mode);
            if (status != TAPE_SUCC)
              return(E_FAIL);
          }

       /* At next band 
       ---------------*/
       ptrband = cb + 1;
       kk++;
       lrecsiz = bandrecsiz[ptrband-1];
       }
     else
       {
       lrecsiz = bandrecsiz[ptrband];
       } 
    }

/* Close idr file
-----------------*/
if (wrtdsk[IDR])
   fclose(fp);

/* Stop the processing message
-----------------------------*/
c_stoptm();

/* Skip forward if need trailer files
--------------------------------------*/
/*if (wrtdsk[TRAI] || wrtdsk[SUPP] || wrtdsk[CAL] || wrtdsk[WOR] || wrtdsk[HST])
  {
  if (window[0] + window[2] != bandnumrec[0])
    skip=2;
  else
    skip=1;
  skip = ((skip + ((7-lastband)*3))-2);
  status=c_tpfsf(unit,&skip,&mode);
  if (status != TAPE_SUCC)
    {
    c_errmsg("failure skipping to trailer files","ltwg2in-skip",&one);
    return(E_FAIL);
    }
  }
*/
/* Close the image file
-----------------------*/
if (c_eclose((struct FDESC **)&fdesc[0]) != E_SUCC)
   return(E_FAIL);

/* Update the min max file for the output image
-----------------------------------------------*/
if (*minmax)
   if (c_svmnmx((char *)hosout,totbnd,outmin,outmax,"YES") != E_SUCC)
      c_errmsg("Nonfatal error encountered","ltwg2in-nonfatal",&one);

/* Free allocated space
-----------------------*/
free(in_buf);
free(out_buf);

return(E_SUCC);
}
