/******************************************************************************
FUNCTION:       CALCORN

PURPOSE:        The purpose of this program is to calculate corner
                coordinates for landsat scenes given the recording
                technique and center coordinates.

ALGORITHM 

ALGORITHM REFERENCES    
        C. Wivell developed this algorithm to approximate the corner points
        for Landsat scenes.  This algorithm was implemented in GLIS by
        B. Radspinner and Wivell.  Refer to memo RE: OA5B-65 
******************************************************************************/
#include <string.h>
#include <math.h>
#include "las.h"
#include "ltwgin.h"

#define REQ 6378.135            /* Radius of the earth at the equator */
#define E .006694541854858764   /* Eccentricity of the earth */
#define PI 3.14159265358979     /* Ratio of circumference to diameter */
#define RAD 57.295779505601047  /* (Circumference / 2) / PI */
#define WE 0.00007292115642     /* Rotation of the earth */

void calcorn
(
  double clat_l,                /* Center point latitude */
  double clon_l,                /* Center point longitude */
  char *rectec,                 /* Recording technique used */
  double *rtnlat,               /* Calculated corner point latitudes */
  double *rtnlon,               /* Calculated corner point longitudes */
  long sat                      /* Satellite number */
)
{
    int    x;           /* counter for loop */

    double A1;          /* Heading of beginning of a scene */
    double A2;          /* Heading of ending of a scene */
    double B1;          /* Scan azimuth of corner 1 */
    double B2;          /* Scan azimuth of corner 2 */
    double B3;          /* Scan azimuth of corner 3 */
    double B4;          /* Scan azimuth of corner 4 */
    double ANODE;       /* Longitude of the Ascending node */
    double L;           /* Length of scene (earth central angles) */
    double M;           /* Mean anomaly past the equator */
    double M1;          /* Mean anomaly for the beginning of the scene */
    double M2;          /* Mean anomaly for the ending of the scene */
    double MDOT;        /* Mean motion */
    double P;           /* Orbital Period */
    double Re;          /* Radius of the earth */
    double W;           /* Width of scene (earth central angles) */
    double clats;       /* cosine of lats (latitude of scene center */
    double clats1;      /* cosine of lats (latitude of beginning of scene */
    double clats2;      /* cosine of lats (latitude of ending of scene */
    double ck;          /* cosine of k */
    double cW2;         /* cosine of W divided by 2 */
    double dlam;        /* rotation of earth * dt */
    double dlong;       /* longitude at beginning of the scene */
    double dlong1;      /* longitude at beginning of the scene */
    double dlong2;      /* longitude at beginning of the scene */
    double dM;          /* MDOT * dt */
    double dt;          /* time required to create half of the scene */
    double i;           /* inclanation angle */
    double k;           /* PI - i */
    double l;           /* length of scene (km) */
    double lats;        /* latitude of center of scene */
    double lats1;       /* latitude of beginning of scene */
    double lats2;       /* latitude of ending of scene */
    double longs;       /* longitude of center of scene */
    double longs1;      /* longitude of beginning of scene */
    double longs2;      /* longitude of ending of scene */
    double newlat[4];   /* latitudes calulated for corner points in radians */
    double newlon[4];   /* longitudes calulated for corner points in radians */
    double q;           /* Temp calculation to be used later */
    double sinA;        /* sin of Angle A */
    double sk;          /* sin of k */
    double slats;       /* sin of lats */
    double slats1;      /* sin of lats1 */
    double slats2;      /* sin of lats2 */
    double sW2;         /* sin of (W divided by 2) */
    double tmplat[4];   /* calculated latitudes before order is decided */
    double tmplon[4];   /* calculated longitudes before order is decided */
    double w = 185.0;   /* width of scene (km) */
    double xc;          /* Temp calculation to be used later */
    double zc;          /* Temp calculation to be used later */

    switch (sat)
    {
        case 1:
            i = 99.906;         /* inclanation angle */
            l = 179.0;          /* length of scene */
            P = 103.143;        /* Orbital Period */
            break;
        case 2:
            i = 99.210;         /* inclanation angle */
            l = 179.0;          /* length of scene */
            P = 103.155;        /* Orbital Period */
            break;
        case 3:
            i = 99.117;         /* inclanation angle */
            l = 179.0;          /* length of scene */
            P = 103.150;        /* Orbital Period */
            break;
        case 4:
            i = 98.22;          /* inclanation angle */
            l = 185.0;          /* length of scene */
            P = 98.9;           /* Orbital Period */
            break;
        case 5:
        default:
            i = 98.22;          /* inclanation angle */
            l = 185.0;          /* length of scene */
            P = 98.9;           /* Orbital Period */
            break;
    }
    i = i / RAD;        /* inclanation angle  in radians */

    /* convert center latitude to radians */
    lats = clat_l / RAD;

    /* convert center longitude to radians */
    longs = clon_l / RAD;
    if (longs < 0)
        longs = (2 * PI) + longs;

    /* Calculate the radius of the earth at scene center */

    slats = sin(lats);
    q = sqrt(1 - E * E * slats * slats);

    /* REQ is the radius of the earth at the equator
     * E is the eccentricity of the earth
     */

    clats = cos (lats);
    xc = (REQ * clats) / q;

    zc = (REQ * (1 - E * E) * slats) / q;

    Re = sqrt(xc * xc + zc * zc);

    /* Calculate the mean anomaly past the equator (M),
       the longitude of the ascending node (ANODE),
       mean motion (MDOT) and change the swath lengths
       to earth central angles (W,L) */

    k = PI - i;
    ck = cos(k);
    sk = sin(k);

    if (fabs(lats) > k)
        lats = (fabs(lats)/lats) * k;
    M = asin(slats/sk);

    if (strcmp(rectec,"D") == 0)
        /*if (rectec != 42 && rectec != 39)*/       /* Descending */
        M = PI - M;
    if (M < 0)
        M = (2 * PI) + M;

    dlong = acos(cos(M) / clats);

    if (lats < 0)
        dlong = (-dlong);

    ANODE = longs + dlong;

    if (ANODE > (2 * PI))
        ANODE = ANODE - (2 * PI);

    if (ANODE < 0)
        ANODE = (2 * PI) + ANODE;

    MDOT = ((2 * PI) / (P * 60));

    W = w / Re;

    L = l / Re;

    /* Calculate the time required to create half of the swath (dt).
     * First calculate the sine of the angle A
     */

    sinA = ck / clats;
    dt = sqrt((1 - cos(L / 2)) / ((MDOT * MDOT) / 2 +
            MDOT * WE * sinA + (WE * WE) / 2));
    dM = MDOT * dt;
    M1 = M - dM;
    M2 = M + dM;
    lats1 = asin(sk * sin(M1));
    lats2 = asin(sk * sin(M2));
    clats1 = cos(lats1);
    clats2 = cos(lats2);
    dlam = WE * dt;
    dlong1 =acos(cos(M1) / clats1);
    if (lats1 < 0)
        dlong1 = (2 * PI) - dlong1;
    dlong2 =acos(cos(M2) / clats2);
    if (lats2 < 0)
        dlong2 = (2 * PI) - dlong2;
    longs1 = ANODE + dlam - dlong1;
    longs2 = ANODE - dlam - dlong2;
    A1 = asin (ck/clats1);
    A2 = asin (ck/clats2);
    B1 = PI/2 - A1;
    B2 = PI/2 + A1;
    B3 = PI/2 - A2;
    B4 = PI/2 + A2;
    slats1 = sin (lats1);
    slats2 = sin (lats2);
    cW2 = cos (W/2);
    sW2 = sin (W/2);
    tmplat[0] = asin(slats1 * cW2 + clats1 * sW2 * cos(B1));
    tmplat[1] = asin(slats1 * cW2 + clats1 * sW2 * cos(B2));
    tmplat[2] = asin(slats2 * cW2 + clats2 * sW2 * cos(B3));
    tmplat[3] = asin(slats2 * cW2 + clats2 * sW2 * cos(B4));

    tmplon[0]=acos((cW2 - slats1 * sin(tmplat[0])) / (clats1 * cos(tmplat[0])));
    tmplon[1]=acos((cW2 - slats1 * sin(tmplat[1])) / (clats1 * cos(tmplat[1])));
    tmplon[2]=acos((cW2 - slats2 * sin(tmplat[2])) / (clats2 * cos(tmplat[2])));
    tmplon[3]=acos((cW2 - slats2 * sin(tmplat[3])) / (clats2 * cos(tmplat[3])));

    if (strcmp(rectec,"D") == 0)
        /*if (rectec != 42 && rectec != 39)*/       /* Descending */
    {
        newlat[0] = tmplat[0];                  /* North West Corner */
        newlat[3] = tmplat[1];                  /* North East Corner */
        newlat[1] = tmplat[2];                  /* South West Corner */
        newlat[2] = tmplat[3];                  /* South East Corner */

        newlon[0] = longs1 - tmplon[0];         /* North West Corner */
        newlon[3] = longs1 + tmplon[1];         /* North East Corner */
        newlon[1] = longs2 - tmplon[2];         /* South West Corner */
        newlon[2] = longs2 + tmplon[3];         /* South East Corner */
    }
    else                                /* Ascending */
    {
        newlat[2] = tmplat[0];                  /* South East Corner */
        newlat[1] = tmplat[1];                  /* South West Corner */
        newlat[3] = tmplat[2];                  /* North East Corner */
        newlat[0] = tmplat[3];                  /* North West Corner */

        newlon[2] = longs1 + tmplon[0];         /* South East Corner */
        newlon[1] = longs1 - tmplon[1];         /* South West Corner */
        newlon[3] = longs2 + tmplon[2];         /* North East Corner */
        newlon[0] = longs2 - tmplon[3];         /* North West Corner */
    }

    for (x = 0; x < 4; x++)
    {
        rtnlat[x] = newlat[x] * RAD;
        if (newlon[x] > PI)
            newlon[x] = newlon[x] - (2 * PI);
        if (newlon[x] < -PI)
            newlon[x] = (2 * PI) + newlon[x];
        rtnlon[x] = newlon[x] * RAD; 
    }
}
