/******************************************************************************

FUNCTION:       det_ds

PURPOSE:        Determines the directory structure of image data on a CD.

ALGORITHM REFERENCES:   none

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "cm.h"
#include "ltwgin.h"

int det_ds
(
  char hpath[CMLEN],            /* I: Qualified path to image on CD */
  int *ds,                      /* O: Denotes type of structure */
  char sceneid[SCENELEN],       /* O: Naming convention for NLAPS files */
  char filename[MAX_FILES][CMLEN], /* O: Array of filenames extracted */
  int *nfiles                   /* O: Number of filenames returned */
)
{
    char *file_ptr;      /* points to the actual filename portion of the
                            filename strings returned from c_getfiles */
    char filename_up[MAX_FILES][CMLEN];
                         /* names of files on disk or CD (upper-case) */
    char path[CMLEN];    /* Path specification for call to c_getfiles()  */
    char tmp_file[CMLEN]; /* Temporary string */
    long i, j;           /* Loop counters                                */
    long filename_len;   /* length of filename string */
    long one = 1;        /* constant of one */

    /* Set default format to SAFRICAMSS.
    ------------------------------------*/
    *ds = SAFRICAMSS;

    strcpy(path, hpath);
    strcat(path, "*");

    if(c_getfiles(path, filename, nfiles, MAX_FILES, 1, 0) != E_SUCC)
    {
        c_errmsg("Error retrieving filenames from disk","ltwgin-getfiles",&one);
        return(E_FAIL);
    }

    /* Strip the path off the filenames
    -----------------------------------*/
    for(i = 0; i < *nfiles; i++)
    {
        file_ptr = strrchr(filename[i], '/');
        if(file_ptr == NULL)
            continue;
        file_ptr++;
        strcpy(tmp_file, file_ptr);
        strcpy(filename[i], tmp_file);
    }

    /* Make copy of filenames and make them upper-case for comparisons
    ------------------------------------------------------------------*/
    for(i = 0; i < *nfiles; i++)
    {
        strcpy(filename_up[i], filename[i]);
        filename_len = (long)strlen(filename_up[i]);
        c_low2up(filename_up[i], &filename_len);
    }

    /*-------------------------------------------------------------------------
      Check for South Africa MSS data.  This data is stored using the directory
      structure as follows:

      scene1/img_01.001
      img_02.001
      img_03.001
      vdf_dat.001
      tra_01.001
      lea_01.001
      .
      .
      .

      In this case, the function will key off the presence or absence of a
      file which begins with "vdf_dat" and "img_0" to prevent the Radarsat
      data cds from matching here since their imagery data files are "dat_0*".
   ---------------------------------------------------------------------------*/

    for (i = 0; i < *nfiles; i++)
    {
        if (strncmp(filename_up[i], "VDF_DAT", 7) == 0)
        {
            for(j = 0; j < *nfiles; j++)
            {
                if (strncmp(filename_up[j], "IMG_0", 5) == 0)
                {
                    *ds = SAFRICAMSS; 
                    return (E_SUCC);
                }
            }
        }
    }

    /*-------------------------------------------------------------------------
      Check for SPOT BIL data.  This data is found in the form:

      scene01/imag_01.dat
      lead_01.dat
      trai_01.dat
      vold_01.dat
      .
      .
      .

      The filename "vold*" will be used as the key.
   ---------------------------------------------------------------------------*/

    for (i = 0; i < *nfiles; i++)
    {
        if (strncmp(filename_up[i], "VOLD", 4) == 0)
        {
            *ds = SPOTBIL;
            return (E_SUCC);
        }
    }

    /*-------------------------------------------------------------------------
      Check for NLAPS data.  This data is found in the form:

      sceneid.i1
      sceneid.l1
      sceneid.t1
      sceneid.sv
      sceneid.sf
      .
      .
      .

      The filename extension ".i" will be used as the key.
   ---------------------------------------------------------------------------*/
    for (i = 0; i < *nfiles; i++)
    {
        if (strncmp(filename_up[i] + SCENELEN, ".I", 2) == 0)
        {
            *ds = NLAPS;
            strncpy(sceneid,filename[i],SCENELEN);
            filename[i][SCENELEN] = '\0';
            return (E_SUCC);
        }  
    }

    /*-------------------------------------------------------------------------
      Check for ESA TM data. This data is stored using the directory structure
      as follows:

      SCENE1/VDF_DAT.001 ..................image volume directory file
      LEA_0n.001 with n = 1,...,7 ..leader file relative to band n
      DAT_0n.001 with n = 1,...,7 ..data file relative to band n
      TRA_0n.001 with n = 1,...,7 ..trailer file relative to band n
      VDF_SUP.001 ..................supplementary volume directory file
      DAT_SUP.001 ..................supplementary information file
      VDF_CAL.001 ..................calibration volume directory file
      DAT_CAL.001 ..................calibration information file
      NUL_VDF.001 ..................null volume directory file

      In this case, the function will key off the presence or absence of a
      file which begins with "VDF_DAT" and "VDF_CAL" to prevent the Radarsat
      data cds from matching here since they don't have a calibration file.
   ---------------------------------------------------------------------------*/

    for (i = 0; i < *nfiles; i++)
    {
        if (strncmp(filename_up[i], "VDF_DAT", 7) == 0)
        {
            for(j = 0; j < *nfiles; j++)
            {
                if (strncmp(filename_up[j], "VDF_CAL", 7) == 0)
                {
                    *ds = ESATM; 
                    return (E_SUCC);
                }
            }
        }
    }

    /*-------------------------------------------------------------------------
      Check for Radarsat data. This data is stored using the directory
      structure as follows:

      scene01/vdf_dat.001 ..................image volume directory file
      lea_0n.001 with n = 1,...,7 ..leader file relative to band n
      dat_0n.001 with n = 1,...,7 ..data file relative to band n
      tra_0n.001 with n = 1,...,7 ..trailer file relative to band n
      nul_vdf.001 ..................null volume directory file

      In this case, the function will key off the presence or absence of a
      file which begins with "vdf_dat".
   ---------------------------------------------------------------------------*/

    for (i = 0; i < *nfiles; i++)
    {
        if (strncmp(filename_up[i], "VDF_DAT", 7) == 0)
        {
            *ds = SAR; 
            return (E_SUCC);
        }
    }

    return(E_SUCC);
}
