/******************************************************************************

FUNCTION:       DET_SCENE

PURPOSE:        Determines which scene to ingest from CD

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ltwgin.h"

int det_scene
(
   char drivenam[],              /* I: drive media mounted on */
   int dirnum,                   /* I: scene number user wishes to ingest */
   char hpath[]                  /* O: Host name of input file/directory */
)
{
    char dir_path[CMLEN];            /* host path to scene directories on CD */
    char dirnames[MAX_FILES][CMLEN]; /* names of directories on disk or CD */
    char dirnames_up[MAX_FILES][CMLEN]; /* names of directories on disk or CD
                                           forced to upper-case */
    char lhost[CMLEN];               /* local host name */
    char dirname[8];                 /* scene directory name */
    char *name_ptr;                  /* ptr used when removing drive portion
                                        of directory/file names on CD */
    char tmp_str[CMLEN];             /* temporary string value */

    int num_scenes;                  /* number of scene directories on the CD */
    int i, j;                        /* Looping counters */
    int found = FALSE;               /* flag to indicate the scene directory
                                        was found on the CD */

    size_t length;                   /* string lengths */

    /* Create a filename for the location of the data on CD so processing
       may then continue identical as to processing from disk. If the DRIVE
       doesn't have an @ sign in it then this is a local CD drive. If the
       DRIVE does have an @ sign then this is a remote drive. If this is a
       local DRIVE, then we need to get the hostname and compute the DRIVE
       from hostname + @ + DRIVE. If the DRIVE has !! as the first two
       characters, then this is a local drive and the user doesn't want to
       prefix it with the hostname. We should strip off the first !!s and
       process this as the specified drive name. */
    if(strchr(drivenam, '@') == NULL)
    {
        /* This is a local tape drive */
        if(gethostname(lhost, CMLEN) != 0)
        {
            c_lasmsg(FALSE, "ltwgin-hostnam", "Error reading the hostname");
            return E_FAIL;
        }
        length = strlen(lhost);
        c_up2low(lhost, (long *)&length);
        sprintf(dir_path, "/%s@%s/", lhost, drivenam);
    }
    else if(strncmp(drivenam, "!!", 2) == 0)
        sprintf(dir_path, "/%s/", &drivenam[2]);
    else /* remote drive */
        sprintf(dir_path, "/%s/", drivenam);

    /* Save the path at this point so once the appropriate directory
       name is established, that can just be appended to this string. */
    strcpy(hpath, dir_path);

    /* Retrieve the directory names from the CD */
    strcat(dir_path, "SCENE*");
    if(c_getfiles(dir_path, dirnames, &num_scenes, MAX_FILES, 1, 0)
                  != E_SUCC)
    {
        c_lasmsg(FALSE, "ltwgin-getfiles",
                 "Error retrieving directory names from CD");
        return E_FAIL;
    }

    /* If no directories were found, try lower case before giving up */
    if(num_scenes == 0)
    {
        /* Reset the dir_path to the value before "SCENE*" was appended,
           the value saved in hpath, then append "scene*" */
        strcpy(dir_path, hpath);
        strcat(dir_path, "scene*");
        if(c_getfiles(dir_path, dirnames, &num_scenes, MAX_FILES, 1, 0)
                      != E_SUCC)
        {
            c_lasmsg(FALSE, "ltwgin-getfiles",
                     "Error retrieving directory names from CD");
            return E_FAIL;
        }

        /* If we still didn't match any directories, error */
        if(num_scenes == 0)
        {
            c_lasmsg(FALSE, "ltwgin-nodirs",
                     "No valid scene directories could be located");
            return E_FAIL;
        }
    }

    /* Look at each file matched by c_getfiles and remove those that
       aren't valid scene directories (have a '.' in the name) */
    for(i = 0; i < num_scenes; i++)
    {
        /* If there is a '.' in the name, it is not a valid scene directory */
        if(strchr(dirnames[i], '.') != NULL)
        {
            /* Move remaining file names in the dirnames array up one
               to remove this one. This shouldn't take too much time since
               the number of scenes that can exist on a single CD is not
               that large */
            for(j = i + 1; j < num_scenes; j++)
            {
                strcpy(dirnames[i], dirnames[j]);

                /* At the last file name move, decrement counter i so
                   the first file name that was moved into the current
                   index will be evaluated next */
                if(j == (num_scenes - 1))
                    i--;
            }

            /* Decrement num_scenes since we removed one of the file names */
            num_scenes--;
        }
    }

    /* If there are no directories left now that the
       invalid ones removed, error */
    if(num_scenes == 0)
    {
        c_lasmsg(FALSE, "ltwgin-nodirs",
                 "No valid scene directories could be located");
        return E_FAIL;
    }

    /* Remove drive portion of directory names */
    for(i = 0; i < num_scenes; i++)
    {
        strcpy(tmp_str, dirnames[i]);
        name_ptr = strrchr(tmp_str, '/');

        if(name_ptr != NULL)
        {
            name_ptr++;
            strcpy(dirnames[i], name_ptr);
        }
    }

    /* Make copy of directory names and make them upper-case for comparisons */
    for(i = 0; i < num_scenes; i++)
    {
        strcpy(dirnames_up[i], dirnames[i]);
        length = (long)strlen(dirnames_up[i]);
        c_low2up(dirnames_up[i], (long *)&length);
    }

    /* Now append the appropriate scene directory name to the hpath string */
    if(dirnum != -1) /* user specified scene */
        sprintf(dirname, "SCENE%d", dirnum);
    else /* if scene not specified, ingest scene 1 by default */
    {
        if(num_scenes > 1)
        {
            c_lasmsg(FALSE, "ltwgin-info", "CD contains multiple scenes"
                     " and the scene to ingest was not specified."
                     " Ingesting scene 1. See User Note 9.");
        }
        strcpy(dirname, "SCENE1");
    }

    /* Verify the directory for the specified scene exists. If not,
       check for it with a 0 in the number before bailing out. */
    for(i = 0; i < num_scenes; i++)
    {
        if(strcmp(dirnames_up[i], dirname) == 0)
        {
            strcat(hpath, dirnames[i]);
            strcat(hpath, "/");
            found = TRUE;
            break;
        }
    }

    if(!found)
    {
        /* The directory "SCENEx" did not exist, so check for
           directory "SCENE0x" */
        if(dirnum != -1) /* user specified scene */
            sprintf(dirname, "SCENE0%d", dirnum);
        else /* ingesting scene 1 by default since no scene specified */
            strcpy(dirname, "SCENE01");

        for(i = 0; i < num_scenes; i++)
        {
            if(strcmp(dirnames_up[i], dirname) == 0)
            {
                strcat(hpath, dirnames[i]);
                strcat(hpath, "/");
                found = TRUE;
                break;
            }
        }

        if(!found)
        {
            c_lasmsg(FALSE, "ltwgin-scene", "Could not locate scene %d on CD",
                     dirnum);
            return E_FAIL;
        }
    }
    return E_SUCC;
}
