/******************************************************************************
FUNCTION:       GETLEAD

PURPOSE:        Read the leader file from media and write it to disk if
                specified.  Extract needed information.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "labserv.h"
#include "cm.h"
#include "tapeio.h"
#include "ltwgin.h"

#define PI    3.141592653589793238 /* PI */

int getlead
(
   long *unit,                    /* Tape unit number */
   long *wrtdsk,                  /* Ancillary files to read in */
   char filename[][CMLEN],        /* Filenames on disk or CD */
   int nfiles,                    /* Number of filenames on disk or CD */
   int tapeflag,                  /* I: TRUE if processing from tape */
   int dir_struct,                /* I: flag indicating which directory
                                        structure is used for data */
   char hosout[][CMLEN],          /* Output host names */
   long *num_out,                 /* Number of output files */
   long *numlead,                 /* Number of leader records */
   long *leadsize,                /* Size of leader records */
   double fcenll[],               /* Full scene center lat/long degrees */
   double fcenls[],               /* Full scene center line/sample */
   long *path,                    /* Scene path */
   char *direction,               /* Satellite direction */
   char *maprj,                   /* Map projection */
   double pixsiz[],               /* Image pixel size */
   long *satid,                   /* Satellite id */
   char *sensor,                  /* Sateelite sensor */
   double *angle,                 /* Angle of rotation in radians */
   long *datum_code,              /* Datum of the dataset */
   long *utmzone,                 /* UTM zone code */
   char *satnam,                  /* Satellite name */
   char *data_type,               /* Type of data (corrected or uncorrected) */
   struct DDR_FLAGS *ddr_flags,   /* flags to indicate if sscanf was successful
                                     in retrieving numerical values from
                                     strings retrieved from leader file. */
   char cornlat[][17],            /* Corner latitudes */
   char cornlong[][17],           /* Corner longitudes */
   char hpath[CMLEN],             /* Path to image data on disk */
   long ceosid,                   /* Id number of receiving station */
   double proj_corners[8],        /* Projection corners */
   double ls_corners[8],          /* Line/sample corners */
   int *data_set_summ_rec_flg,    /* indicates if the Data Set Summary
                                     record in the SAR format is present */
   int *map_proj_data_rec_flg     /* indicates if the Map Projection Data
                                     record in the SAR format is present */
)
{
    FILE *fp=NULL;          /* file pointer                                 */
    FILE *infp=NULL;        /* LEA file pointer (for CD input only)         */

    char filename_up[MAX_FILES][CMLEN];
                            /* names of files on disk or CD (upper-case) */
    char *in_buf = NULL;    /* buffer read from LGSOWG image */
    char tempbuf[33];       /* temp work buffer                             */
    char leafile[CMLEN];    /* name of leader file on disk                  */
    char scenenum[3];       /* string representation of scene number        */
    char datum[13];         /* the datum of the data                        */
    char deg[4];            /* degrees for fcenll                           */
    char min[3];            /* minutes for fcenll                           */
    char nlaps_filename[10][CMLEN]; /* array of filenames extracted */
    char lpath[CMLEN];      /* temporary host path for leader file          */

    int rec_num;            /* leader record currently retrieving */
    int rec_sizes[17];      /* leader record sizes - these are variable in
                               the SAR format */
    int nlaps_nfiles;       /* number of filenames returned                 */

    long filename_len;      /* length of the filename string */
    long i;                 /* loop counter                                 */
    long mode = HANDLE;     /* tape I/O error flag                          */
    long status;            /* status flag                                  */
    long tmp_len;           /* length of scene number string */
    long tzone=0;           /* temp double value                            */
    long location;          /* offset into input buffer for extraction      */
    long location2;         /* see above                                    */
    long one = 1;           /* constant value 1 */
    double temp;            /* temporary holding                            */
    double degrees;         /* degrees for fcenll                           */
    double minutes;         /* minutes for fcenll                           */

    /* Initialize flags
    -------------------*/
    ddr_flags->path = TRUE;
    ddr_flags->fcenll = TRUE;
    ddr_flags->fcenls = TRUE;
    ddr_flags->utmzone = TRUE;
    ddr_flags->pixsiz = TRUE;
    ddr_flags->proj_corners = TRUE;
    ddr_flags->ls_corners = TRUE;
    ddr_flags->angle = TRUE;

    /* Create the leader file name
    ------------------------------*/
    if (*wrtdsk)
    {
        c_lsmknm(hosout[0],"lead",hosout[*num_out]);
        fp = fopen(hosout[(*num_out)++],"w");
        if(fp == NULL)
        {
            c_lasmsg(FALSE,"ltwgin-diskread",
                    "Error reading leader file from disk");
            return(E_FAIL);
        }
    }

    /* If image is to be read from CD or disk, prepare
       the LEA file for reading
    --------------------------------------------------*/
    if (!tapeflag)
    {
        /* Convert filenames on disk or CD to all upper-case for comparisons
        --------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            strcpy(filename_up[i], filename[i]);
            filename_len = (long)strlen(filename_up[i]);
            c_low2up(filename_up[i], &filename_len);
        }

        strcpy(leafile, hpath);

        /* Append the leader filename to the leafile string. Look through
           the array with the filenames in upper-case, then append the
           filename at the same array index from the case-sensitive array.
        ------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            if (dir_struct == SAFRICAMSS)
            {
                if(strncmp(filename_up[i], "LEA_01.001", 10) == 0)
                {
                    strcat(leafile, filename[i]);
                    break;
                }
            }
            else if (dir_struct == SPOTBIL)
            {
                if(strncmp(filename_up[i], "LEAD_", 5) == 0)
                {
                    if(imgnum(hpath, scenenum) != E_SUCC)
                    {
                        c_lasmsg(FALSE, "ltwgin-imgnum",
                                "Error retrieving the image number from CD.");
                        return(E_FAIL);
                    }
                    strncat(leafile, filename[i], 5); /* add "lead_" */
                    strcat(leafile, scenenum); /* add the scene number */
                    tmp_len = (long)strlen(scenenum);
                    strncat(leafile, filename[i] + 5 + tmp_len, 4);
                                                               /* add ".dat" */
                    break;
                }
            }
            else if (dir_struct == NLAPS)
            {
                strcpy(lpath,hpath);
                strcat(lpath,"*.l*");
                if(c_getfiles(lpath,nlaps_filename,&nlaps_nfiles,10,1,10)
                   != E_SUCC)
                {
                    c_errmsg("Error retrieving filenames from disk",
                            "ltwgin-getfiles", &one);
                    return(E_FAIL);
                }
                if (nlaps_nfiles > 0)
                {
                    strcpy(leafile, nlaps_filename[0]); 
                    break;
                }
                else /* try upper-case */
                {
                    strcpy(lpath,hpath);
                    strcat(lpath,"*.L*");
                    if(c_getfiles(lpath,nlaps_filename,&nlaps_nfiles,10,1,10)
                            != E_SUCC)
                    {
                        c_errmsg("Error retrieving filenames from disk",
                                "ltwgin-getfiles", &one);
                        return(E_FAIL);
                    }
                    if (nlaps_nfiles > 0)
                    {
                        strcpy(leafile, nlaps_filename[0]); 
                        break;
                    }
                }
            }
            else if((dir_struct == ESATM) || (dir_struct == SAR))
            {
                if(strncmp(filename_up[i], "LEA_01.001", 10) == 0)
                {
                    strcat(leafile, filename[i]);
                    break;
                }
            }
        }

        infp = fopen(leafile, "r"); /* Standard leader filename for CD */
        if (infp == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-diskread",
                    "Error reading leader file from disk");
            return(E_FAIL);
        }
    }

    /* Read and optionally write to disk.  Also extract needed information
       present in the first three records.
    ----------------------------------------------------------------------*/
    for (i = 0; i < *numlead; i++)
    {
        /* Allocate space
        -----------------*/
        if(in_buf != NULL)
        {
            free(in_buf);
            in_buf = NULL;
        }
        in_buf = (char *) malloc(*leadsize);
        if (in_buf == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-alloc",
                    "Error allocating dynamic memory");
            return(E_FAIL);
        }

        if (!tapeflag)
        {
            if (fread(in_buf, 1, *leadsize, infp) != *leadsize)
            {
                c_lasmsg(FALSE, "ltwgin-diskread",
                        "Error reading leader file from disk");  
                free(in_buf);
                return(E_FAIL);
            }
        }
        else
        {
            if(c_tpread(unit,(unsigned char *)in_buf,leadsize,&mode) !=
               *leadsize)

            {
                c_lasmsg(FALSE, "ltwgin-read",
                        "Error reading leader file from tape");
                free(in_buf);
                return(E_FAIL);
            }
        }

        if (i == 0)
        {
            /* Get the satellite name, number, sensor and data type
            -------------------------------------------------------*/
            strncpy(satnam,&in_buf[48],2);
            satnam[2] = '\0';

            /* If data is from Sweden (or France) receiving station, but this
               is not SPOT data, we do not know how to process it */
            if ((ceosid == SWEDEN) && (strncmp(satnam,"SP",2) != 0))
            {
                c_lasmsg(FALSE, "ltwgin-notsup",
                         "Non-SPOT data from this receiving station "
                         "not supported");
                free(in_buf);
                return(E_FAIL);
            }
            if (ceosid == RSAT)
            {
                strncpy(tempbuf,&in_buf[53],1);
                tempbuf[1] = '\0';
                *satid = atol(tempbuf);
            }
            else
            {
                strncpy(tempbuf,&in_buf[50],1);
                tempbuf[1] = '\0';
                *satid = atol(tempbuf);
            }
            if (ceosid == RSAT)
                strncpy(sensor,&in_buf[55],3);
            else if (ceosid == CAN)
                strncpy(sensor,&in_buf[51],3);
            else
                strncpy(sensor,&in_buf[52],3);
            sensor[3] = '\0';
            if (ceosid == RSAT)
                *data_type = in_buf[55];

            /* Get leader record sizes if SAR format since they are of
               variable length. First check if the record is present in the
               leader file (in_buf[xxx] != '0'), and if it is present,
               retrieve the record length from the appropriate location in
               the file descriptor record
            ---------------------------------------------------------------*/
            if(ceosid == RSAT)
            {
                rec_num = 1; /* retrieving record lengths for records
                                1...numlead */
                /* Retrieve size of Data Set Summary record, if present */
                if(in_buf[185] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[186]);
                    rec_num++;
                    /* Set flag to indicate the Data Set Summary
                       record is present */
                    *data_set_summ_rec_flg = TRUE;
                }
                else
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "The Data Set Summary record was not present; some"
                            " DDR information will not be available");
                    *data_set_summ_rec_flg = FALSE;
                }
                /* Retrieve size of Map Projection Data record, if present */
                if(in_buf[197] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[198]);
                    rec_num++;
                    /* Set flag to indicate the Map Projection Data record
                       is present */
                    *map_proj_data_rec_flg = TRUE;
                }
                else
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "The Map Projection Data record was not present;"
                            " some DDR information will not be available");
                    *map_proj_data_rec_flg = FALSE;
                }
                /* Retrieve size of Platform Position Data record, if present */
                if(in_buf[209] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[210]);
                    rec_num++;
                }
                /* Retrieve size of Attitude Data record, if present */
                if(in_buf[221] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[222]);
                    rec_num++;
                }
                /* Retrieve size of Radiometric Data record, if present */
                if(in_buf[233] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[234]);
                    rec_num++;
                }
                /* Retrieve size of Radiometric Compensation
                   record, if present */
                if(in_buf[245] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[246]);
                    rec_num++;
                }
                /* Retrieve size of Data Quality Summary record, if present */
                if(in_buf[257] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[258]);
                    rec_num++;
                }
                /* Retrieve size of Data Histograms record, if present.
                   If not 0, then there are 2 records. */
                if(in_buf[269] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[270]);
                    rec_num++;
                    /* Do again for second data histogram record */
                    rec_sizes[rec_num] = atol((char *)&in_buf[270]);
                    rec_num++;
                }
                /* Retrieve size of Range Spectra record, if present */
                if(in_buf[281] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[282]);
                    rec_num++;
                }
                /* Retrieve size of Digital Elevation Model Descriptor record,
                   if present */
                if(in_buf[293] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[294]);
                    rec_num++;
                }
                /* Retrieve size of Radar Parameter Data Update record,
                   if present */
                if(in_buf[305] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[306]);
                    rec_num++;
                }
                /* Retrieve size of Annotation Data record, if present */
                if(in_buf[317] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[318]);
                    rec_num++;
                }
                /* Retrieve size of Detailed Processing Parameters record,
                   if present */
                if(in_buf[329] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[330]);
                    rec_num++;
                }
                /* Retrieve size of Calibration Data record, if present */
                if(in_buf[341] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[342]);
                    rec_num++;
                }
                /* Retrieve size of Ground Control Points record, if present */
                if(in_buf[353] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[354]);
                    rec_num++;
                }
                /* Retrieve size of Facility Related Data record, if present */
                if(in_buf[425] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[426]);
                    rec_num++;
                }

                /* Make sure the number of records found to be present
                   here matches the number of records as reported by the
                   VDF file leader file pointer record
                --------------------------------------------------------*/
                if(rec_num != *numlead)
                {
                    c_lasmsg(FALSE, "ltwgin-numlead",
                            "Number of records doesn't match that reported by"
                            " the VDF file");
                    free(in_buf);
                    return(E_FAIL);
                }

                /* Reset the record number to 1 */
                rec_num = 1;
            }
        }
        else if (((i == 1) && (ceosid != RSAT)) ||
                ((i == 1) && (ceosid == RSAT) &&
                 (*data_set_summ_rec_flg == TRUE)))
        {
            if (strncmp(satnam,"LS",2) == 0) /* Landsat */
            {
                if ((JAPAN) && (strcmp(sensor, "MSS") == 0))
                {
                    /* Get the full scene center line/sample
                    ----------------------------------------*/
                    strncpy(tempbuf, &in_buf[100], 4);
                    tempbuf[4] = '\0';
                    fcenls[0] = atol(tempbuf);
                    strncpy(tempbuf, &in_buf[104], 4);
                    tempbuf[4] = '\0';
                    fcenls[1] = atol(tempbuf);

                    /* Get the WRS path
                    -------------------*/
                    if(ceosid != RSAT)
                    {
                        strncpy(tempbuf, &in_buf[26], 3);
                        tempbuf[3] = '\0';
                        if(sscanf(tempbuf, "%ld", path) != 1)
                        {
                            c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                    "Could not retrieve the WRS path");
                            ddr_flags->path = FALSE;
                        }
                    }

                    /* Get the orbital direction
                    ----------------------------*/
                    strncpy(direction, &in_buf[25], 1);
                    direction[1] = '\0';

                    /* Get the map projection
                    -------------------------*/
                    strncpy(maprj, &in_buf[180], 3);
                    maprj[3] = '\0';
                }
                else
                {
                    /* Retrieve the full scene center lat/long 
                    ------------------------------------------*/
                    /* Japan (corrected) & Thai store lat/long in
                       different location */
                    if ((JAPAN && (*data_type == 'P')) || (ceosid == THAI))
                    {
                        location = 212;
                        location2 = 228;
                    }
                    else 
                    {
                        location = 52;
                        location2 = 68; 
                    }
                    strncpy(tempbuf,&in_buf[location],16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf,"%lf",&fcenll[0]) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the full scene center"
                                " latitude");
                        ddr_flags->fcenll = FALSE;
                    }

                    strncpy(tempbuf,&in_buf[location2],16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf,"%lf",&fcenll[1]) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the full scene center"
                                " longitude");
                        ddr_flags->fcenll = FALSE;
                    }

                    /* Get the full scene center line/sample 
                    ----------------------------------------*/
                    if ((JAPAN && (*data_type == 'P')) || (ceosid == THAI))
                    {
                        location = 244;
                        location2 = 260;
                    }
                    else
                    {
                        location = 84;
                        location2 = 100;
                    }
                    strncpy(tempbuf,&in_buf[location],16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf,"%lf",&fcenls[0]) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the full scene center"
                                " line");
                        ddr_flags->fcenls = FALSE;
                    }
                    strncpy(tempbuf,&in_buf[location2],16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf,"%lf",&fcenls[1]) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the full scene center"
                                " sample");
                        ddr_flags->fcenls = FALSE;
                    }

                    /* Get the WRS path number 
                    -------------------------*/
                    strncpy(tempbuf,&in_buf[165],3);
                    tempbuf[3] = '\0';
                    if(sscanf(tempbuf,"%ld",path) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the WRS path number");
                        ddr_flags->path = FALSE;
                    }

                    /* Get the orbital direction 
                    ----------------------------*/
                    if ((ceosid == HBK) || (ceosid == CAN))
                    {

                        /* South Africa and Canada data stores the orbital
                           direction as a single character immediately 
                           preceding the WRS path number. */

                        direction [0] = in_buf[164];
                        direction [1] = '\0';
                    }
                    else
                    {
                        strncpy(direction,&in_buf[356],10);
                        direction[9] = '\0';
                    }

                    /* Get the map projection 
                    -------------------------*/
                    strncpy(maprj,&in_buf[1556],3);
                    maprj[3] = '\0';
                    if (ceosid == HBK)

                        /* South Africa UTM data stores the projection and
                           zone in a slightly different form: whereas the
                           'standard' is to store the projection as "UTM"
                           and the zone in the next record, this data stores
                           them both together. For example, UTM zone 39
                           would be stored as 'UT39'. */
                    {
                        if (maprj[0] == 'U')
                        {
                            maprj[2] = 'M';
                            maprj[3] = '\0';
                        }
                        else 
                            maprj[2] = '\0';
                    }
                    else if (JAPAN)
                    {
                        if (strcmp(maprj, "YN ") == 0)
                        {
                            strncpy(maprj, "UTM", 3);
                            maprj[3] = '\0';
                        }
                        else if (strcmp(maprj, "NY ") == 0)
                        {
                            strncpy(maprj, "SOM", 3);
                            maprj[3] = '\0';
                        }
                    }
                    else if (ceosid == SAUDI)
                    {
                        if (maprj[1] == 'Y')
                        {
                            strncpy(maprj, "UTM", 3);
                            maprj[3] = '\0';
                        }
                    }
                    else if (ceosid == THAI)
                    {
                        if (maprj[1] == 'Y')
                        {
                            strncpy(maprj, "UTM", 3);
                            maprj[3] = '\0';
                        }
                        else if ((maprj[1] == 'N') && (maprj[2] == 'Y'))
                        {
                            strncpy(maprj, "UTM", 3);
                            maprj[3] = '\0';
                        }
                    }

                    /* If this is HBK data, get the UTM zone
                    ---------------------------------------*/
                    if (ceosid == HBK)
                    {
                        strncpy(tempbuf, &in_buf[1558],2);
                        tempbuf[2] = '\0';
                        if(sscanf(tempbuf, "%lf", &temp) != 1)
                        {
                            c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                    "Could not retrieve the UTM zone");
                            ddr_flags->utmzone = FALSE;
                        }
                        else
                        {
                            tzone = (long)temp;
                            *utmzone = (long)tzone;
                        }
                    }
                }
            }
            else if (strncmp(satnam,"SP",2) == 0) /* SPOT */
            {
                /* Get the four corners 
                -----------------------*/
                strncpy(cornlat[0],&in_buf[148],16);
                cornlat[0][16] = '\0';
                strncpy(cornlong[0],&in_buf[164],16);
                cornlong[0][16] = '\0';

                strncpy(cornlat[1],&in_buf[212],16);
                cornlat[1][16] = '\0';
                strncpy(cornlong[1],&in_buf[228],16);
                cornlong[1][16] = '\0';

                strncpy(cornlat[2],&in_buf[276],16);
                cornlat[2][16] = '\0';
                strncpy(cornlong[2],&in_buf[292],16);
                cornlong[2][16] = '\0';

                strncpy(cornlat[3],&in_buf[340],16);
                cornlat[3][16] = '\0';
                strncpy(cornlong[3],&in_buf[356],16);
                cornlong[3][16] = '\0';
            }
            else if (ceosid == RSAT)
            {
                /* Get the orbital direction
                ----------------------------*/
                direction [0] = in_buf[100];
                direction [1] = '\0';

                /* Retrieve the full scene center lat/long 
                ------------------------------------------*/
                strncpy(tempbuf,&in_buf[116],16); /* lat */
                tempbuf[16] = '\0';
                if(sscanf(tempbuf,"%lf",&fcenll[0]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the full scene center"
                            " latitude");
                    ddr_flags->fcenll = FALSE;
                }

                strncpy(tempbuf,&in_buf[132],16); /* long */
                tempbuf[16] = '\0';
                if(sscanf(tempbuf,"%lf",&fcenll[1]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the full scene center"
                            " longitude");
                    ddr_flags->fcenll = FALSE;
                }
            }
        }
        else if (((i == 2) && (ceosid != RSAT)) ||
                ((i == 2) && (ceosid == RSAT) &&
                 (*map_proj_data_rec_flg == TRUE)))
        {
            if(ceosid == RSAT)
            {
                /* Get the four corners 
                -----------------------*/
                strncpy(cornlat[0],&in_buf[1072],16);
                cornlat[0][16] = '\0';
                strncpy(cornlong[0],&in_buf[1088],16);
                cornlong[0][16] = '\0';

                strncpy(cornlat[1],&in_buf[1104],16);
                cornlat[1][16] = '\0';
                strncpy(cornlong[1],&in_buf[1120],16);
                cornlong[1][16] = '\0';

                strncpy(cornlat[2],&in_buf[1136],16);
                cornlat[2][16] = '\0';
                strncpy(cornlong[2],&in_buf[1152],16);
                cornlong[2][16] = '\0';

                strncpy(cornlat[3],&in_buf[1168],16);
                cornlat[3][16] = '\0';
                strncpy(cornlong[3],&in_buf[1184],16);
                cornlong[3][16] = '\0';

                /* Get the map projection
                -------------------------*/
                strncpy(tempbuf,&in_buf[444],32);
                tempbuf[32] = '\0';
                if(strstr(tempbuf, "TRANSVERSE") != NULL)
                    strcpy(maprj, "UTM");
                else
                {
                    strncpy(tempbuf,&in_buf[592],32);
                    tempbuf[32] = '\0';
                    if(strstr(tempbuf, "POLAR") != NULL)
                    {
                        /* Universal Polar Stereographic not supported by LAS */
                        c_lasmsg(FALSE, "ltwgin-project",
                                "Map projection not currently supported");
                        free(in_buf);
                        return(E_FAIL);
                    }
                    else
                    {
                        strncpy(tempbuf,&in_buf[672],32);
                        tempbuf[32] = '\0';
                        if(strstr(tempbuf, "ALBERS") != NULL)
                            strcpy(maprj, "ALBERS");
                        else if(strstr(tempbuf, "AZIMUTHAL EQUIDISTANT")
                                != NULL)
                            strcpy(maprj, "AZMEQD");
                        else if(strstr(tempbuf, "EQUIDISTANT CONIC TYPE A")
                                != NULL)
                        {
                            /* Tangential Equidistant Conic not supported by
                               LAS */
                            c_lasmsg(FALSE, "ltwgin-project",
                                    "Map projection not currently supported");
                            free(in_buf);
                            return(E_FAIL);
                        }
                        else if(strstr(tempbuf, "EQUIDISTANT CONIC TYPE B")
                                != NULL)
                            strcpy(maprj, "EQUIDC");
                        else if(strstr(tempbuf, "EQUIRECTANGULAR") != NULL)
                            strcpy(maprj, "EQRECT");
                        else if(strstr(tempbuf, "GENERAL") != NULL)
                            strcpy(maprj, "GVNSP");
                        else if(strstr(tempbuf, "GNOMONIC") != NULL)
                            strcpy(maprj, "GNOMON");
                        else if(strstr(tempbuf, "LAMBERT AZIMUTHAL") != NULL)
                            strcpy(maprj, "LAMAZ");
                        else if(strstr(tempbuf, "LAMBERT CONFORMAL") != NULL)
                            strcpy(maprj, "LAMCC");
                        else if(strstr(tempbuf,"HOTINE OBLIQUE MERCATOR TYPE A")
                                != NULL)
                        {
                            /* Hotine Oblique Mercator Type A (two points on the
                               central line) not supported by LAS */
                            c_lasmsg(FALSE, "ltwgin-project",
                                    "Map projection not currently supported");
                            free(in_buf);
                            return(E_FAIL);
                        }
                        else if(strstr(tempbuf, "HOTINE OBLIQUE MERCATOR TYPE B")
                                != NULL)
                            strcpy(maprj, "HOM");
                        else if(strstr(tempbuf, "TRANSVERSE") != NULL)
                            strcpy(maprj, "TM");
                        else if(strstr(tempbuf, "MERCATOR") != NULL)
                            strcpy(maprj, "MERCAT");
                        else if(strstr(tempbuf, "CYLINDRICAL") != NULL)
                            strcpy(maprj, "MILLER");
                        else if(strstr(tempbuf, "ORTHOGRAPHIC") != NULL)
                            strcpy(maprj, "ORTHO");
                        else if(strstr(tempbuf, "POLAR") != NULL)
                            strcpy(maprj, "PS");
                        else if(strstr(tempbuf, "POLYCONIC") != NULL)
                            strcpy(maprj, "POLYC");
                        else if(strstr(tempbuf, "SINUSOIDAL") != NULL)
                            strcpy(maprj, "SNSOID");
                        else if(strstr(tempbuf, "STATE PLANE") != NULL)
                            strcpy(maprj, "SPCS");
                        else if(strstr(tempbuf, "STEREOGRAPHIC") != NULL)
                            strcpy(maprj, "STEREO");
                        else if(strstr(tempbuf, "VAN DER") != NULL)
                            strcpy(maprj, "VGRINT");
                        else
                        {
                            /* unknown map projection not supported by LAS */
                            c_lasmsg(FALSE, "ltwgin-project",
                                    "Map projection not currently supported");
                            free(in_buf);
                            return(E_FAIL);
                        }
                    }
                }

                /* Get the UTM zone code
                ------------------------*/
                if(strcpy(maprj, "UTM") == 0)
                {
                    strncpy(tempbuf, &in_buf[476],4);
                    tempbuf[4] = '\0';
                    if(sscanf(tempbuf, "%lf", &temp) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the UTM zone");
                        ddr_flags->utmzone = FALSE;
                    }
                    else
                    {
                        tzone = (long)temp;
                        *utmzone = (long)tzone;
                    }
                }

                /* Retrieve the pixel size
                --------------------------*/
                strncpy(tempbuf, &in_buf[92], 16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf, "%lf", &pixsiz[1]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the pixel size");
                    ddr_flags->pixsiz = FALSE;
                }

                strncpy(tempbuf, &in_buf[108], 16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf, "%lf", &pixsiz[0]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the pixel size");
                    ddr_flags->pixsiz = FALSE;
                }
            }

            /* If this is HBK (South Africa) data, get the image projection
               and line/sample corners to be later converted into their
               corresponding file corners.
            ---------------------------------------------------------------*/
            if (ceosid == HBK)
            {
                /* flag the scene center lat/long as unavailable */
                ddr_flags->fcenll = FALSE;

                for (i = 0; i < 8; i++)
                {
                    strncpy(tempbuf, &in_buf[(580 + (i * 16))], 16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf, "%lf", &proj_corners[i]) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the image projection"
                                " corners");
                        ddr_flags->proj_corners = FALSE;
                    }
                }
                for (i = 0; i < 4; i++)
                {
                    strncpy(tempbuf, &in_buf[(836 + (i * 32) + 16)], 16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf, "%lf", &ls_corners[i * 2]) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the line/sample corners");
                        ddr_flags->ls_corners = FALSE;
                    }
                    strncpy(tempbuf, &in_buf[(836 + (i * 32))], 16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf, "%lf", &ls_corners[(i * 2) + 1]) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the line/sample corners");
                        ddr_flags->ls_corners = FALSE;
                    }
                }

                /* Since the corners in the leader file are not stored "neatly"
                   in the order expected by the DDR, swap the out-of-order ones
                   to rectify things.  This is much easier to do than reading
                   in all 16 corners one at a time.  
                --------------------------------------------------------------*/

                temp = proj_corners[4];
                proj_corners[4] = proj_corners[6]; 
                proj_corners[6] = temp;
                temp = proj_corners[5];
                proj_corners[5] = proj_corners[7];
                proj_corners[7] = temp;
                temp = ls_corners[4];
                ls_corners[4] = ls_corners[6];
                ls_corners[6] = temp;
                temp = ls_corners[5]; 
                ls_corners[5] = ls_corners[7];
                ls_corners[7] = temp;
            }

            /* Retrieve the full scene center lat/lon for Japan MSS 
            -------------------------------------------------------*/
            if (JAPAN && (strcmp(sensor, "MSS") == 0))
            {
                strncpy(deg, &in_buf[23], 2);
                deg[2] = '\0';
                degrees = (double)atol(deg); 
                strncpy(min, &in_buf[26], 2);
                min[2] = '\0';
                minutes = (double)atol(min);
                fcenll[0] = degrees + (minutes/60);

                strncpy(deg, &in_buf[30], 3);
                deg[3] = '\0';
                degrees = (double)atol(deg);
                strncpy(min, &in_buf[34], 2);
                min[2] = '\0';
                minutes = (double)atol(min);
                fcenll[1] = degrees + (minutes/60); 
            }


            /* Retrieve the pixel size and convert from km to meters
            -------------------------------------------------------*/
            if (ceosid == HBK)
            {
                /* South Africa pixel sizes are found in a different location.
                   Also, they are already in meters rather than kilometers, so
                   there is no need to multiply them by 1000. */

                strncpy(tempbuf, &in_buf[180], 16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf, "%lf", &pixsiz[1]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the pixel size");
                    ddr_flags->pixsiz = FALSE;
                }

                strncpy(tempbuf, &in_buf[196], 16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf, "%lf", &pixsiz[0]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the pixel size");
                    ddr_flags->pixsiz = FALSE;
                }
            }
            else if ((JAPAN) && (strcmp(sensor,"MSS") == 0))
            {
                /* MSS data from Japan does not contain the pixel size for
                   corrected data.  These hard-coded values are "best guess"
                   based on the LTWG Production Description Chart entry on
                   Japan MSS data. */

                if (*data_type == 'A')
                {
                    pixsiz[0] = 57.0;
                    pixsiz[1] = 59.0;
                }
                else
                {
                    pixsiz[0] = 57.0;
                    pixsiz[1] = 57.0;
                }
            }
            else if (ceosid != RSAT)
            {
                if (ceosid == CAN)
                    strncpy(tempbuf,&in_buf[180],16);
                else
                    strncpy(tempbuf,&in_buf[364],16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf,"%lf",&pixsiz[1]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the pixel size");
                    ddr_flags->pixsiz = FALSE;
                }
                /* Thailand, Saudi Arabia, Japan, and Canada data is
                   already in meters */
                if ((ceosid != THAI) && (ceosid != SAUDI) && (!(JAPAN)) && 
                        (ceosid != CAN))
                    pixsiz[1] *= 1000;

                if (ceosid == CAN)
                    strncpy(tempbuf,&in_buf[196],16);
                else
                    strncpy(tempbuf,&in_buf[380],16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf,"%lf",&pixsiz[0]) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the pixel size");
                    ddr_flags->pixsiz = FALSE;
                }
                if ((ceosid != THAI) && (ceosid != SAUDI) && (!(JAPAN)) && 
                        (ceosid != CAN))
                    pixsiz[0] *= 1000;
            }

            /* Get the rotation angle and convert to radians
            ------------------------------------------------*/
            if (ceosid == HBK || ceosid == CAN)
            {
                strncpy(tempbuf,&in_buf[132], 16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf, "%lf", angle) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the rotation angle");
                    ddr_flags->angle = FALSE;
                }
            }
            else if (ceosid == THAI)
            {
                strncpy(tempbuf,&in_buf[444], 16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf, "%lf", angle) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the rotation angle");
                    ddr_flags->angle = FALSE;
                }
            }
            else if (JAPAN && (strcmp(sensor, "MSS") == 0))
            {
                strncpy(tempbuf,&in_buf[128], 16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf, "%lf", angle) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the rotation angle");
                    ddr_flags->angle = FALSE;
                }
            }
            else if(ceosid != RSAT)
            {
                strncpy(tempbuf,&in_buf[204],16);
                tempbuf[16] = '\0';
                if(sscanf(tempbuf,"%lf",angle) != 1)
                {
                    c_lasmsg(FALSE, "ltwgin-ddrinfo",
                            "Could not retrieve the rotation angle");
                    ddr_flags->angle = FALSE;
                }
            }

            if (((ceosid == THAI) || (ceosid == SAUDI)) && (*angle > 180))
            {
                *angle = (*angle - 180);
                *angle = *angle * PI/180.0;
            }
            else if (!(JAPAN) && (ceosid != RSAT))
            {
                *angle = *angle * PI/180.0;
            }

            /* Get the UTM zone and datum 
            -----------------------------*/
            if ((ceosid != HBK) && (ceosid != RSAT))
            {
                /* This was in previous record for South Africa */
                if (((ceosid == CAN) && (strncmp(satnam,"SP",2) != 0)) ||
                        (JAPAN && (strcmp(sensor, "MSS") == 0)))
                {
                    tzone = (long)(((fcenll[1] + 180.0) / 6.0) + 1.0);
                    if (fcenll[0] < 0.0)
                        tzone = -tzone;
                    datum[0] = '\0';
                    *utmzone = tzone;
                }
                else if ((ceosid == THAI) || (ceosid == SAUDI))
                {
                    strncpy(datum,&in_buf[396],13);
                    datum[12] = '\0';
                    strncpy(tempbuf,&in_buf[410],2);
                    tempbuf[2] = '\0';
                    if(sscanf(tempbuf,"%ld",&tzone) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the UTM zone");
                        ddr_flags->utmzone = FALSE;
                    }
                    else
                        *utmzone = tzone;
                }
                else
                {
                    strncpy(datum,&in_buf[92],13);
                    datum[12] = '\0'; 
                    strncpy(tempbuf,&in_buf[396],16);
                    tempbuf[16] = '\0';
                    if(sscanf(tempbuf,"%ld",&tzone) != 1)
                    {
                        c_lasmsg(FALSE, "ltwgin-ddrinfo",
                                "Could not retrieve the UTM zone");
                        ddr_flags->utmzone = FALSE;
                    }
                    else
                        *utmzone = tzone;
                }
                /* Japan data doesn't contain datum information.  According
                   to the receiving station, the spheroid used is Bessel(2).
                ------------------------------------------------------------*/ 
                if (JAPAN)
                    strcpy(datum,"BESSEL");
                /* Assign the datum code.  Use the default datum of 0 
                   for NAD 27 and 8 for NAD 83 until datums are fully 
                   implemented.  The datum_code for NAD 27 will be 225.
                   The datum_code for NAD 83 will be 219.
                ------------------------------------------------------*/
                if (strncmp(datum,"NAD 27",6) == 0)
                    *datum_code = 0;          
                else if (strncmp(datum,"NAD 83",6) == 0)
                    *datum_code = 8;
                else
                    /* If c_dtmnum is not successful, the programmer can
                       set the datum code using the semimajor and semiminor
                       axes if they are available in the leader file. This
                       assignment will be done on a case by case basis.
                    -------------------------------------------------------*/
                    c_dtmnum(datum, datum_code);      
            }
        }

        /* Write to disk
        ----------------*/
        if (*wrtdsk)
        {
            if(fwrite(in_buf,1,*leadsize,fp) != *leadsize)
            {
                c_lasmsg(FALSE,"ltwgin-write","Error writing to leader file");
                free(in_buf);
                return(E_FAIL);
            }
        }

        /* Update the leader record size if SAR format
        ----------------------------------------------*/
        if((i < *numlead) && (ceosid == RSAT))
        {
            *leadsize = rec_sizes[rec_num];
            rec_num++;
        }
    }

    /* Close the file
    -----------------*/
    if (*wrtdsk)
    {
        if(fclose(fp) != 0)
        {
            c_lasmsg(FALSE,"ltwgin-write","Error closing the leader file");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* One more read for EOF
    ------------------------*/
    if (tapeflag)
    {
        status = c_tpread(unit,(unsigned char *)in_buf,leadsize,&mode);
        if (status != TAPE_EOF) 
        {
            c_lasmsg(FALSE, "ltwgin-read",
                    "Error reading leader file from tape");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Free allocated space
    ----------------------*/
    free(in_buf);
    if (!tapeflag)
        fclose (infp);

    return(E_SUCC);
}
