/******************************************************************************
FUNCTION:       GETVDF

PURPOSE:        Read the volume directory file and write it to disk if
                specified.  Extract needed information

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "tapeio.h"
#include "ltwgin.h"

int getvdf
(
  long *unit,                     /* I: Tape unit number */
  long *wrtdsk,                   /* I: Ancillary files to read in */
  char filename[][CMLEN],         /* I: Names of files on disk or CD */
  int nfiles,                     /* I: Number of filenames on disk or CD */
  int tapeflag,                   /* I: Indicates processing media type
                                          TRUE: tape
                                          FALSE: other (including CD, disk) */
  int dir_struct,                 /* I: flag indicating which directory
                                        structure is used for data */
  char hpath[CMLEN],              /* I: Host path to image data */
  char sceneid[],                 /* I: Naming convention of NLAPS files */
  char hosout[][CMLEN],           /* O: Output host names */
  long *num_out,                  /* O: Number of output files */
  long *recsize,                  /* O: Image record length */
  long *numimg_recs,              /* O: Number of image records on this tape */
  long *num_trailrec,             /* O: Number of trailer records */
  char *order,                    /* O: Band order */
  long *numlead,                  /* O: Number of leader files */
  long *leadsize,                 /* O: Size of leader records */
  long *trailsiz,                 /* O: Max size of trailer records */
  long *ceosid                    /* O: Code for receiving ground station */
)
{
    FILE *fp=NULL;          /* file pointer                                 */
    FILE *infp=NULL;        /* input file, for CD images only               */

    char filename_up[MAX_FILES][CMLEN];
                            /* names of files on disk or CD (upper-case) */
    char *in_buf;           /* buffer read from LGSOWG image */
    char tempbuf[20];       /* temp work buffer                             */
    char vdffile[CMLEN];    /* name of vdf file (on disk)                   */
    char scenenum[3];       /* string representation of scene number        */
    char idstring[32];      /* string extracted to identify ground station  */

    int  temp_val;          /* temporary integer value                      */

    long filename_len;      /* length of the filename string */
    long i;                 /* loop counter                                 */
    long mode = HANDLE;     /* tape I/O error flag                          */
    long numbytes = 360;    /* number of bytes in a volume directory record */
    long numrecs;           /* number of records in the file                */
    long status;            /* status flag                                  */
    long tmp_len;           /* length of scene number string */
    long japan_mss=FALSE;   /* flag for Japan MSS data                      */
    long rec;               /* record number being processed                */ 

    /* Allocate space
    -----------------*/
    in_buf = (char *) malloc(numbytes);
    if (in_buf == NULL)
    {
        c_lasmsg(FALSE, "ltwgin-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }

    /* Create name for volume directory file 
    ----------------------------------------*/
    if (*wrtdsk)
    {
        c_lsmknm(hosout[0],"vdf",hosout[*num_out]);
        fp = fopen(hosout[(*num_out)++],"w");
        if(fp == NULL)
        {
            c_lasmsg(FALSE,"ltwgin-diskread",
                    "Error reading volume directory file from disk");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* If image is to be read from CD, prepare the VDF file for reading
    -------------------------------------------------------------------*/
    if (!tapeflag)
    {
        /* Convert filenames on disk or CD to all upper-case for comparisons
        --------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            strcpy(filename_up[i], filename[i]);
            filename_len = (long)strlen(filename_up[i]);
            c_low2up(filename_up[i], &filename_len);
        }

        strcpy(vdffile, hpath);

        /* Append the VDF filename to the vdffile string. Look through
           the array with the filenames in upper-case, then append the
           filename at the same array index from the case-sensitive
           array.
        --------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            if (dir_struct == SAFRICAMSS)
            {
                if(strncmp(filename_up[i], "VDF_DAT", 7) == 0)
                {
                    strcat(vdffile, filename[i]);
                    break;
                }
            }
            else if (dir_struct == SPOTBIL)
            {
                if(strncmp(filename_up[i], "VOLD", 4) == 0)
                {
                    if(imgnum(hpath, scenenum) != E_SUCC)
                    {
                        c_lasmsg(FALSE,"ltwgin-imgnum",
                                "Error retrieving the image number from CD.");
                        free(in_buf);
                        return(E_FAIL);
                    }
                    strncat(vdffile, filename[i], 5); /* add "vold_" */
                    strcat(vdffile, scenenum); /* add the scene number */
                    tmp_len = (long)strlen(scenenum);
                    strncat(vdffile, filename[i] + 5 + tmp_len, 4);
                                                               /* add ".dat" */
                    break;
                }
            }
            else if((dir_struct == ESATM) || (dir_struct == SAR))
            {
                if(strncmp(filename_up[i], "VDF_DAT", 7) == 0)
                {
                    strcat(vdffile, filename[i]);
                    break;
                }
            }
            else if (dir_struct == NLAPS)
            {
                if(strncmp(filename_up[i] + SCENELEN, ".IV", 3) == 0)
                {
                    strncat(vdffile, sceneid, SCENELEN); 
                    strncat(vdffile, filename[i] + SCENELEN, 3); /* add ".iv" */
                    break;
                }
            }
        }

        infp = fopen(vdffile, "r"); /* Standard name for VDF file on CD */
        if (infp == 0)
        {
            c_lasmsg(FALSE, "ltwgin-diskread",
                    "Error reading VDF file from disk");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Read and optionally write the volume directory file.
       Get the number of records in this file from the first
       records  Extract other needed information. 
    --------------------------------------------------------*/
    numrecs = -1;
    for (i = 0; ; i++)
    {
        if (i == numrecs)
            break;
        if (!tapeflag)
        {
            if(fread(in_buf, 1, numbytes, infp) != numbytes)
            {
                c_lasmsg(FALSE,"ltwgin-diskread",
                        "Error reading volume directory file from disk");
                free(in_buf);
                return(E_FAIL);
            }
        }
        else
        {
            if(c_tpread(unit,(unsigned char *)in_buf,&numbytes,&mode)
               != numbytes)
            {
                c_lasmsg(FALSE, "ltwgin-read",
                        "Error reading VDF file from tape");
                free(in_buf);
                return(E_FAIL);
            }
        }

        /* The second record in the vdf for Japan MSS data is the text 
           record, not a file pointer record.  If we have mss data from
           Japan, skip the second record and process records 3, 4, and 5.
        -----------------------------------------------------------------*/ 
        if ((i == 1) && (japan_mss))
            rec = 4;
        else if ((i > 1) && (japan_mss))
            rec = i - 1;
        else
            rec = i;

        switch (rec)
        {
            /* Get the number of records in volume directory file
            ----------------------------------------------------*/
            case 0: strncpy(tempbuf,&in_buf[164],4);
                    tempbuf[4] = '\0';
                    numrecs = atol(tempbuf);

                    /* Identify the ground station this data is coming from
                    -------------------------------------------------------*/
                    strncpy(idstring, &in_buf[128], 32);
                    idstring[31] = '\0';
                    *ceosid = get_ceosid(idstring);

                    /* If this is MSS data from Japan, set flag
                    -------------------------------------------*/
                    if ((*ceosid == 3) && (in_buf[57] == 'M'))
                        japan_mss = TRUE; 
                    break;

                    /* Get the number of records in leader file and
                       size of records
                    -----------------------------------------------*/
            case 1: strncpy(tempbuf,&in_buf[100],8);
                    tempbuf[8] = '\0';
                    *numlead = atol(tempbuf);

                    if(*ceosid == RSAT)
                    {
                        strncpy(tempbuf,&in_buf[108],8);
                        tempbuf[8] = '\0';
                        *leadsize = atol(tempbuf);
                    }
                    else
                    {
                        strncpy(tempbuf,&in_buf[116],8); 
                        tempbuf[8] = '\0';
                        *leadsize = atol(tempbuf);
                    }

                    /* Need to determine the band order from the record
                       sequence number if data uses the SAR format - this
                       is not present in the imagery file pointer record
                       in the new format. A '2' indicates BIL, all other
                       values indiate BSQ.
                    ------------------------------------------------------*/
                    if(*ceosid == RSAT)
                    {
                        strncpy(tempbuf, &in_buf[0], 4);
                        tempbuf[4] = '\0';
                        temp_val = atol(tempbuf);
                        if(temp_val == 2)
                            strcpy(order, "BIL");
                        else
                            strcpy(order, "BSQ");
                    }
                    break;

                    /* Get the record length and band order. Also get the
                       number of image records on this tape for BIL.
                    -----------------------------------------------------*/
            case 2: strncpy(tempbuf,&in_buf[116],8);
                    tempbuf[8] = '\0';
                    *recsize = atol(tempbuf);
                    if(*ceosid != RSAT)
                    {
                        strncpy(order,&in_buf[32],3);
                        order[3] = '\0';
                    }
                    strncpy(tempbuf,&in_buf[100],8);
                    tempbuf[8] = '\0';
                    /* Subtract the one image file descriptor record */
                    *numimg_recs = atol(tempbuf) - 1;
                    break;

                    /* Get the number and size of the trailer records
                    ------------------------------------------------*/
            case 3: strncpy(tempbuf,&in_buf[100],8);
                    tempbuf[8] = '\0';
                    *num_trailrec = atol(tempbuf);

                    /* If SAR, we want the actual size of the first record
                       instead of the maximum record size, since records are
                       of variable length with this format
                    --------------------------------------------------------*/
                    if(*ceosid == RSAT)
                    {
                        strncpy(tempbuf,&in_buf[108],8); 
                        tempbuf[8] = '\0';
                    }
                    else
                    {
                        strncpy(tempbuf,&in_buf[116],8); 
                        tempbuf[8] = '\0';
                    }
                    *trailsiz = atol(tempbuf);
                    break;

            default: break;
        }

        /* Write to disk
        ----------------*/
        if (*wrtdsk)
        {
            if(fwrite(in_buf,1,numbytes,fp) != numbytes)
            {
                c_lasmsg(FALSE,"ltwgin-write",
                        "Error writing to volume directory file");
                free(in_buf);
                return(E_FAIL);
            }
        }
    }

    /* Close the file
    ----------------*/
    if (*wrtdsk)
    {
        if(fclose(fp) != 0)
        {
            c_lasmsg(FALSE,"ltwgin-write",
                    "Error closing the volume directory file");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* One more read for EOF
    -----------------------*/
    if (tapeflag)
    {
        status = c_tpread(unit,(unsigned char *)in_buf,&numbytes,&mode);
        if (status != TAPE_EOF) 
        {
            c_lasmsg(FALSE, "ltwgin-read",
                    "Error reading VDF file from tape");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /*Free allocated space 
   ----------------------*/
    free(in_buf);
    if (!tapeflag)
        fclose(infp);

    return(E_SUCC);
}
