/******************************************************************************
FUNCTION:       LS_PROJ

PURPOSE:        This routine does basically the reverse process of LINSAM to
                convert from line/sample to projection coordinates.

ALGORITHM 
  Initialize variables
  Reapply center value and pixel size
  Solve for Xp and Yp using the ing transformation equations
  Calculate projection coordinates based on full scene center

ALGORITHM REFERENCES
This routine uses the algorithm described in the paper entitled
"Geodetic Accuracy of LANDSAT 4 Multispectral Scanner and Thematic Mapper Data"
written by J. M. Thormodsgard and D. J. DeVries.
*******************************************************************************/
#include <math.h>
#include "las.h"
#include "proj.h"
#include "ltwgin.h"

void ls_proj
(
  double *angle,                 /* Display rotation angle */
  double fcproj[],               /* Full scene center projection coords */
  double fcenls[],               /* Full scene center line/sample coords */
  double proj[],                 /* Projection coordinates calcuated */
  long linsamp[],                /* Line/sample values calculated */
  double *pixsiz                 /* Pixel size in meters */
)
{
    double lp;         /* Line value after coordinate to be converted is     */
    /* subtracted from center line and multiplied by pixel*/
    /* size.                                              */

    double sp;         /* Sample value after center sample is subtracted from*/
    /* sample to be converted and multiplied by pixel size*/

    double xp, yp;     /* Intermediate value after translation is applied    */

    /* Initialize variables
    -----------------------*/
    *angle = fabs(*angle);

    /* Reapply center value and pixel size
    --------------------------------------*/
    sp = ((double)linsamp[1] - fcenls[1]) * pixsiz[1];
    lp = (fcenls[0] - (double)linsamp[0]) * pixsiz[0];

    /* Solve for Xp and Yp using the following transformation equations:
    --------------------------------------------------------------------*/
    xp =  sp * cos(*angle) + lp * sin(*angle);
    yp = -sp * sin(*angle) + lp * cos(*angle);

    proj[0] = yp + fcproj[0];
    proj[1] = xp + fcproj[1];
}
