/******************************************************************************
FUNCTION:       LTWGIN

PURPOSE:        Read in image files from tape(s), CD(s), or disk that are
                in the LTWG format and optionally read the ancillary data

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      11/90  T. Baltzer       Original development
2.0      03/94  D. Etrheim       Rewrote to be more generic to allow LTWG tapes
                                 from different receiving stations
2.1      09/94  K. Lowell        Modified to allow reading of entire scene from
                                 a single 8mm tape
2.2      05/95  C. Engebretson   Added routines to allow for 1) data to be read
                                 directly from CDs and disk files 2) more
                                 general types of input data 3) ingesting 
                                 non-standard foreign LTWG-format data.
7.0      07/98  J. Janssen       Modified to ingest SAUDI data
7.1      07/98  J. Janssen       Modified to allow users to write all the bands
                                 to one image or to write each band to its own
                                 image. 
7.2      08/98  J. Janssen       Modified to ingest Japan data
7.3      03/99  D. Hames         Modified to ingest NLAPS data as produced 
                                 by PDS
7.4      05/01  A. Shimitz       Modified to ingest ESA TM data

$Log: ltwgin.c,v $
Revision 6.16  2002/10/16 11:51:48  shimitz
Merging changes for CR 5257 into trunk. Added support for ingesting from CD.

Revision 6.15.2.2  2002/05/23 11:57:55  shimitz
Made changes following code review. Mainly added more explanation to the
comments. CR 5257

Revision 6.15.2.1  2002/05/16 18:55:38  shimitz
Implemented CR 5257. This included enhancing ltwgin to support processing
from CDs and adding the DIRNUM parameter to allow users to specify which scene
they want to ingest from CDs that contain multiple scenes. I also made a few
other modifications (spelling errors, comments that needed updating, etc.)


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "cm.h"
#include "util.h"
#include "ltwgin.h"

int main
(
  void
)
{
    struct PARBLK vblock;   /* TAE parameter block                        */

    long bands[MAX_BAND];   /* bands to be processed within image         */
    long i;                 /* counter                                    */
    long msgtim;            /* elapsed time between display msg           */
    int nfiles;             /* number of filenames on disk or CD */
    long num_out;           /* number of output files                     */ 
    long one = 1;           /* value of one                               */
    long path;              /* satellite path                             */
    long unit;              /* media unit number                          */
    long totbnd = 0;        /* total number of bands to be copied         */
    long wrtdsk[5];         /* ancillary files to read in                 */
    long window[4];         /* window of input image                      */
    long zero = 0;          /* value of zero                              */
    long leadsize;          /* size of leader records                     */
    long fatal = -1;        /* fatal error                                */
    long minmax;            /* min max flag                               */
    long no_lead = FALSE;   /* Flag if successive tapes will have
                               leader files */
    long numimg_files;      /* number of output image files               */
    long numimg_recs;       /* number of image records on current media   */
    long media_cnt;         /* number of media specified                  */
    long num_trailrec;      /* number of trailer records                  */
    long numlead;           /* number of leader records                   */
    long recsize;           /* image record length                        */
    long tapenum = 1;       /* Tape volume currently mounted              */
    long trailsiz;          /* max size of trailer records                */
    long satid;             /* satellite id                               */
    long datum;             /* the datum code of the data                 */        
    long utmzone;           /* UTM zone code                              */
    long std_out;           /* Standard output                            */

    struct DDR_FLAGS ddr_flags;
    /* flags to indicate if sscanf was successful
       in retrieving numerical values from
       strings retrieved from leader file. */
    int  data_set_summ_rec_flg; /* indicates if the Data Set Summary record
                                   in the SAR format is present */
    int  map_proj_data_rec_flg; /* indicates if the Map Projection Data record
                                   in the SAR format is present */
    int tapeflag = FALSE;       /* indicates processing media type
                                     TRUE: tape
                                     FALSE: other (including CD, disk) */
    int cdflag = FALSE;         /* set to TRUE if processing from CD */
    int dir_struct = 0;         /* flag indicating which directory structure
                                   is used for data - based on receiving sta. */
    int dirnum;                 /* scene number user wishes to ingest */

    char dpath[CMLEN];      /* path to CD or disk image (TAE)             */
    char filename[MAX_FILES][CMLEN]; /* names of files on disk or CD */
    char hpath[CMLEN];      /* actual machine path to CD or disk image    */

    long ceosid;            /* integer ID number for ground station       */
    char comment[3][CMLEN]; /* media mount message to the operator        */
    char hosout[12][CMLEN]; /* Output host file name                      */
    char taeout[12][CMLEN]; /* Output TAE file names                      */
    char order[4];          /* band order                                 */
    char maprj[5];          /* map projection                             */
    char direction[10];     /* satellite direction                        */
    char sensor[5];         /* sensor type                                */
    char satnam[5];         /* satellite name                             */
    char cornlat[4][17];    /* corner latitudes                           */
    char cornlong[4][17];   /* corner longitudes                          */
    char ttype[CMLEN];      /* input tape type                            */
    char data_type;         /* type of data (corrected or uncorrected)    */
    char sceneid[SCENELEN]; /* naming convention for NLAPS files */

    double fcenll[2];       /* Full scene center latitude/longitude degrees */
    double fcenls[2];       /* Full scene center line/sample              */
    double pixsiz[2];       /* image pixel size                           */
    double angle;           /* rotation angle                             */
    double proj_corners[8]; /* Projection corners                         */
    double ls_corners[8];   /* Line/sample corners                        */

    /* Initialize LAS
    -----------------*/
    c_inlas(&vblock, &std_out);

    /* Retrieve the input parameters
    --------------------------------*/
    getpar(&vblock, bands, &msgtim, window, &minmax, hosout, taeout, 
            comment, wrtdsk, &media_cnt, &numimg_files, dpath, &dirnum);

    num_out = numimg_files;
    if (c_cmhost(dpath, hpath) != E_SUCC) /* Create actual host path */
        c_errmsg("Fatal error encountered","ltwgin-fatal", &fatal);
    strcat(hpath,"/");

    /* If the user specified neither a tape mount comment nor a disk pathname,
       error out. Note that if BOTH are specified, the comment will be 
       used to mount the media and INFILE will be ignored.
    -------------------------------------------------------------------------*/
    if ((strlen(dpath) == 0) && (strlen(comment[0]) == 0))
        c_lasmsg(TRUE, "ltwgin-comment",
                "Either COMMENT or INFILE must be specified");

    /* If mounting message(s) specified, mount first media and determine
       if tape or not. Since we don't know if this is a CD or tape, set
       option to rewind. If it is a CD, this option will be reset in
       c_tpopen to no rewind. When COMMENT is specified as well as INFILE,
       INFILE is ignored.
    ----------------------------------------------------------------------*/
    if(strlen(comment[0]) != 0)
    {
        if(mount_media(&unit, comment, one, &media_cnt, ttype, &tapeflag)
                        != E_SUCC)
            c_lasmsg(TRUE, "ltwgin-tpopen", "Error mounting the media");
        if(!tapeflag)
            cdflag = TRUE;
    }

    /* If the image is being read from CD, check to see if the user
       specified which scene they wish to ingest and add that
       scene name to the filename path.
    ---------------------------------------------------------------*/
    if(cdflag)
    {
        /* Retrieve the directories on the CD */
        if(det_scene(comment[0], dirnum, hpath) != E_SUCC)
        {
            c_lasmsg(FALSE, "ltwgin-scene", "Error determining which scene"
                     " to ingest.");
            close_media(&unit);
            return EXIT_FAILURE;
        }
    }

    /* If the image to be read is on CD or disk, determine the directory
       structure used by the receiving station the data was generated by
       and create a host path from the TAE path.
    --------------------------------------------------------------------*/
    if (!tapeflag) /* user specified a disk path */
    {
        if(det_ds(hpath, &dir_struct, sceneid, filename, &nfiles) != E_SUCC)
            c_errmsg("Fatal error encountered", "ltwgin-fatal", &fatal);
    }

    /* Read the volume directory file and write to disk if specified
    ---------------------------------------------------------------*/
    if (getvdf(&unit, &wrtdsk[VDF], filename, nfiles, tapeflag, dir_struct,
               hpath, sceneid, hosout, &num_out, &recsize, &numimg_recs,
               &num_trailrec, order, &numlead, &leadsize, &trailsiz, &ceosid)
               != E_SUCC)
    {
        if (tapeflag || cdflag)
            close_media(&unit);
        c_clean(&vblock,"ltwgin",hosout,&num_out);
    }

    /* Read the leader file and write to disk if specified
    -----------------------------------------------------*/
    if (getlead(&unit, &wrtdsk[LEAD], filename, nfiles, tapeflag, dir_struct,
                hosout, &num_out, &numlead, &leadsize, fcenll, fcenls, &path,
                direction, maprj, pixsiz, &satid, sensor, &angle, &datum,
                &utmzone, satnam, &data_type, &ddr_flags, cornlat, cornlong,
                hpath, ceosid, proj_corners, ls_corners,
                &data_set_summ_rec_flg, &map_proj_data_rec_flg) != E_SUCC)
    {
        if (tapeflag || cdflag)
            close_media(&unit);
        c_clean(&vblock,"ltwgin",hosout,&num_out);
    }

    /* Process the image data
    ------------------------*/
    if (strncmp(order,"BIL",3) == 0)
    {
        if (proces_bil(hosout, window, &totbnd, filename, sceneid, nfiles,
                       tapeflag, dir_struct, &unit, wrtdsk, num_out,
                       numimg_files, bands, &msgtim, &minmax, comment,
                       &recsize, &numimg_recs, &tapenum, &media_cnt, &no_lead,
                       ttype, hpath, sensor, ceosid) != E_SUCC)
        {
            if (tapeflag || cdflag)
                close_media(&unit);
            c_clean(&vblock,"ltwgin",hosout,&num_out);
        }
    }
    else
    { 
        if (proces_bsq(hosout, window, &totbnd, filename, sceneid, nfiles,
                       dir_struct, tapeflag, &unit, wrtdsk, &num_out,
                       &numimg_files, bands, &msgtim, &minmax, comment,
                       &recsize, &tapenum, &media_cnt, &no_lead, ttype, hpath,
                       sensor, ceosid) != E_SUCC)
        {
            if (tapeflag || cdflag)
                close_media(&unit);
            c_clean(&vblock,"ltwgin",hosout,&num_out);
        }
    }

    /* Get the trailer and supplemental files if requested
    ------------------------------------------------------*/
    if (wrtdsk[TRAI] || wrtdsk[SUPP])
    {
        if (getenf(hosout, filename, nfiles, dir_struct, tapeflag, &unit,
                   wrtdsk, &num_out, comment, &num_trailrec, &tapenum,
                   &trailsiz, &media_cnt, &no_lead, satnam, hpath, sceneid,
                   ceosid) != E_SUCC)
        {
            if (tapeflag || cdflag)
                close_media(&unit);
            c_clean(&vblock,"ltwgin",hosout,&num_out);
        }
    }

    /* Dismount the tape
    --------------------*/
    if (tapeflag || cdflag)
        close_media(&unit);

    /* Update the DDR file 
    ---------------------*/
    if (update_ddr(hosout, window, &totbnd, fcenll, fcenls, &path, direction,
                   maprj, pixsiz, &satid, sensor, data_type, &angle, &datum,
                   &utmzone, satnam, cornlat, cornlong, proj_corners,
                   ls_corners, &numimg_files, ceosid, data_set_summ_rec_flg,
                   map_proj_data_rec_flg, &ddr_flags) != E_SUCC)
        c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);

    /* Output the history information
    ---------------------------------*/
    for (i = 0; i < numimg_files; i++)
        if (c_hstout(&vblock,hosout[i],hosout,&zero," ") != E_SUCC)
            c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);

    for (i = 0; i < num_out; i++)
        strcpy(taeout[i], hosout[i]); 

    /* Issue the completion message
    -------------------------------*/
    c_complt("LTWGIN",taeout,&num_out);

    return EXIT_SUCCESS;
}
