/******************************************************************************
FUNCTION:       PROCES_BIL

PURPOSE:        Read image data from media and write it to disk.  The image
                data is BIL.  Each image line contains the LTWG standard 
                12 bytes, suffix and prefix data, and image data.  

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "pixman.h"
#include "tapeio.h"
#include "ltwgin.h"

int proces_bil
(
   char hosout[][CMLEN],         /* I/O: Output host names */
   long *window,                 /* I/O: Window of image to be copied */
   long *totbnd,                 /* O: Total number of bands to be copied */
   char filename[][CMLEN],       /* I: filenames on disk or CD */
   char sceneid[],               /* I: naming convention of NLAPS files */
   int nfiles,                   /* I: number of filenames on disk or CD */
   int tapeflag,                 /* I: indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD, disk) */
   int dir_struct,               /* I: flag indicating which directory
                                       structure is used for data */
   long *unit,                   /* I: Tape unit number */
   long *wrtdsk,                 /* I: Ancillary files to read in */
   long num_out,                 /* I: Number of output files */
   long numimg_files,            /* I: Number of output image files */
   long *bands,                  /* I/O: Bands to be processed within image */
   long *msgtim,                 /* I: Time between display messages */
   long *minmax,                 /* I: Min/Max flag */
   char comment[][CMLEN],        /* I: Mount message to the operator */
   long *recsize,                /* I: Image record length */
   long *numimg_recs,            /* I: Number of image records on this tape */
   long *tapenum,                /* I/O: Current tape number */
   long *media_cnt,              /* I: Number of media specified */
   long *no_lead,                /* I: Flag if successive tapes have leader
                                       file */
   char ttype[],                 /* I: */
   char hpath[CMLEN],            /* I: Path to image data on disk */
   char *sensor,                 /* I: Satellite sensor */
   long ceosid                   /* I: Integer ID of receiving station */
)
{
    extern long curent;          /* output line being processed               */
    struct FDESC *fdesc[MAXBND]; /* image file descriptors used for image I/O */

    FILE *fp=NULL;               /* file descriptor                           */
    FILE *infp=NULL;             /* input image file for disk/CD data         */

    long  access = IWRITE;  /* write access to output image                   */
    long  bandoff[MAX_BAND];/* offset into each band to get to image data */
    long  cb;               /* current output disk band being processed  */
    long  cl;               /* current output disk line begin processed  */
    long  curbnd;           /* curent tape band being processed          */
    long  filename_len;     /* length of filename string */
    long  i;                /* loop counter                              */
    long  idtype;           /* data type of input image                  */
    long  mode = HANDLE;    /* tape I/O error flag                            */
    long  msgflg;           /* Processing message format flag            */
    long  numbands;         /* number of bands taken from header record  */
    long  one = 1;          /* constant one                              */
    long  option[2];        /* output option array                       */
    long  outspc[2];        /* output image file specifications          */
    long  status;           /* return status                             */
    long  temp;             /* temp value                                */
    long  tmp_len;          /* length of scene number string */
    long  dumlong;          /* dummy value                               */
    long  dum2 = FALSE;     /* dummy value                               */
    long  prefix,suffix;    /* number of prefix and suffix pixles        */

    int tmp;                /* temp value to pass to mount_media so tapeflag
                               won't get changed during processing somehow */

    static long datasize[4] = {0,1,2,4};
                            /* defines number bytes for each data type */

    unsigned char *in_buf;          /* buffer read from LGSOWG image */
    unsigned char *in_ptr;          /* pointer to input buffer window    */
    char filename_up[MAX_FILES][CMLEN];
                            /* names of files on disk or CD
                               (upper-case) */
    char tempbuf[10];               /* temp work buffer                  */
    char scenenum[3];               /* scene number on CD, i.e., "04"    */
    char imgfile[CMLEN];            /* fully qualified name of disk file */

    double outmax[MAX_BAND];        /* max pixel values of output image  */
    double outmin[MAX_BAND];        /* min pixel values of output image  */

    /* Allocate memory for the input buffer
    ---------------------------------------*/
    in_buf = (unsigned char *) malloc(*recsize);
    if (in_buf == NULL)
    {
        c_lasmsg(FALSE, "ltwgin-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }

    /* If data is to be read from disk or CD, open the input file  
    -------------------------------------------------------------*/
    if (!tapeflag)
    {
        /* Convert filenames on disk or CD to all upper-case for comparisons
        --------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            strcpy(filename_up[i], filename[i]);
            filename_len = (long)strlen(filename_up[i]);
            c_low2up(filename_up[i], &filename_len);
        }

        strcpy(imgfile, hpath);

        /* Append the image filename to the imgfile string. Look through
           the array with the filenames in upper-case, then append the
           filename at the same array index from the case-sensitive array.
        ------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            if((dir_struct == ESATM) || (dir_struct == SAR))
            {
                if(strncmp(filename_up[i], "DAT_01.001", 10) == 0)
                {
                    strcat(imgfile, filename[i]);
                    break;
                }
            }
            else
            {
                if(strncmp(filename_up[i], "IMAG_", 5) == 0)
                {
                    if(imgnum(hpath, scenenum) != E_SUCC)
                    {
                        c_lasmsg(FALSE, "ltwgin-imgnum",
                                "Error retrieving the image number from CD.");
                        free(in_buf);
                        return(E_FAIL);
                    }
                    strncat(imgfile, filename[i], 5); /* add "imag_" */
                    strcat(imgfile, scenenum); /* add the scene number */
                    tmp_len = (long)strlen(scenenum);
                    strncat(imgfile, filename[i] + 5 + tmp_len, 4);
                                                               /* add ".dat" */
                    break;
                }
            }
        }
        infp = fopen(imgfile, "r");
        if (infp == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-diskread",
                    "Error reading BIL data file from disk"); 
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Read first line.  This is the file descriptor record in the image file 
    ------------------------------------------------------------------------*/
    if (!tapeflag)
        status = (long)fread(in_buf, 1, *recsize, infp);
    else
        status = c_tpread(unit,in_buf,recsize,&mode);

    if (status != *recsize)
    {
        if (tapeflag) 
        {
            c_lasmsg(FALSE, "ltwgin-read",
                    "Error reading file descriptor record from tape");
        }
        else
        {
            c_lasmsg(FALSE, "ltwgin-diskread",
                    "Error reading file descriptor record from disk");
        }
        free(in_buf);
        return(E_FAIL);
    }

    /* If specified, open the image data logical record file and write this
       record.  This file contains the file descriptor record from the 
       image file and the prefix and suffix data from the image line. 
    ---------------------------------------------------------------*/
    if (wrtdsk[IDR])
    {
        c_lsmknm(hosout[0],"idr",hosout[num_out]);
        fp = fopen(hosout[(num_out)++],"w");
        if (fp == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-diskread",
                    "Error opening IDR file for writing");
            free(in_buf);
            return(E_FAIL);
        }
        status = (long)fwrite(in_buf,1,*recsize,fp);
        if (status != *recsize)
        {
            c_lasmsg(FALSE, "ltwgin-write",
                    "Error writing to image data logical record file");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Set offsets into each line to get to image data. 
       xx bytes of prefix data and xx bytes of suffix data.
       12 bytes are for standard LTWG record
    ------------------------------------------------------*/
    if (JAPAN && (strncmp(sensor,"TM",2) == 0))
    {
        strncpy(tempbuf, (char *)&in_buf[280], 4);
        tempbuf[4] = '\0';
        prefix = atol(tempbuf);
    }
    else
        prefix = atol((char *)&in_buf[276]);

    suffix = atol((char *)&in_buf[288]);
    for (i = 0; i < MAX_BAND; i++)
        bandoff[i] = prefix + 12;

    /* Get data type (bits per pixel)
    ---------------------------------*/
    temp = atol((char *)&in_buf[216]);
    switch (temp)
    {
        case 8:  idtype = EBYTE;
                 break;
        default: c_lasmsg(FALSE, "ltwgin-dtype",
                         "Data type not currently handled");
                 free(in_buf);
                 return(E_FAIL);
    }

    /* Get the image size on media 
    ------------------------------*/
    numbands = atol((char *)&in_buf[232]);
    outspc[0]= atol((char *)&in_buf[236]);
    strncpy(tempbuf,(char *)&in_buf[248],8);
    tempbuf[8] = '\0';
    outspc[1]= atol(tempbuf);

    /* Check the bands
    ------------------*/
    chk_bands(bands,&numbands,totbnd);
    if (*totbnd == 0)
    {
        c_lasmsg(FALSE, "ltwgin-zero", "Zero total bands specified");
        free(in_buf);
        return(E_FAIL);
    }

    /* If the user wants 1 image file per band, check that the
       number of output image files specified is equal to totbnd
    ------------------------------------------------------------*/
    if ((numimg_files != 1) && (numimg_files != *totbnd))
    {
        c_errmsg("Number of output images must equal number of bands specified",
                "ltwgin-bndcnt",&one);
        free(in_buf);
        return(E_FAIL);
    }

    /* Check the window
    -------------------*/
    chk_window(outspc,window);

    /* Open the output image file for random write access
    -----------------------------------------------------*/
    option[0] = 0;
    option[1] = 0;
    if (numimg_files == 1)
    {
        if (c_eopenr(&fdesc[0],hosout[0],totbnd,&one,&one,&window[NL],
                &window[NS],&access,&idtype,option,&one) != E_SUCC)
        {
            free(in_buf);
            return(E_FAIL);
        }
    }
    else
    {
        for (i = 0; i < numimg_files; i++)
        {
            if (c_eopenr(&fdesc[i],hosout[i],&one,&one,&one,&window[NL],
                    &window[NS],&access,&idtype,option,&one) != E_SUCC)
            {
                free(in_buf);
                return(E_FAIL);
            }
        }
    }

    /* If min max values are to be computed, call inmnmx to
       initialize the min max array for the output data type.
    --------------------------------------------------------*/
    if (*minmax)
        c_inmnmx(&idtype,totbnd,outmin,outmax);

    /* Initialize the processing message
    ------------------------------------*/
    msgflg = ALLCAT;
    c_inittm("LTWGIN","Processed","line","band(s)","",&window[NL],totbnd,
            msgtim,&msgflg);

    /* Skip records on media to starting line of the window.
       Mount next tape as needed.
    ------------------------------------------------------*/
    if (!tapeflag)
        fseek(infp, ((window[SL] - 1) * numbands) * (*recsize), 1);
    else
    {
        if (skip_bil(window, unit, filename, nfiles, tapeflag, dir_struct,
                     comment, numimg_recs, tapenum, &numbands, media_cnt,
                     hpath, sceneid) != E_SUCC)
        {
            c_lasmsg(FALSE, "ltwgin-skip", "Error skipping records");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Start processing message
    ---------------------------*/
    curent = 1;
    c_startm();

    /* Process each line
    --------------------*/
    for (curent=0; curent < window[NL]; curent++)
    {
        cb = 0;
        cl = curent + 1;
        /* Process each band
        --------------------*/
        for (curbnd = 0; curbnd < numbands; curbnd++)
        {
            if (!tapeflag)
                status = (long)fread(in_buf, 1, *recsize, infp);
            else
            {
                status = c_tpread(unit,in_buf,recsize,&mode);
                /* EOV - end of physical volume with subsequent volumes
                   EOT - end of set - last tape. should not encounter this
                         if there is more bands to process yet, but
                         leave it here since that's how it was before */
                if((status == TAPE_EOV) || (status == TAPE_EOT))
                {
                    /* Close current tape, mount next tape, and read volume 
                       directory file
                    -------------------------------------------------------*/
                    c_stoptm();
                    close_media(unit);
                    status = mount_media(unit, comment,++(*tapenum), media_cnt,
                                         ttype, &tmp);
                    if(status != E_SUCC)
                    {
                        free(in_buf);
                        return(E_FAIL);
                    }

                    if (getvdf(unit, &dum2, filename, nfiles, tapeflag,
                               dir_struct, hpath, sceneid, hosout, &dumlong,
                               recsize, numimg_recs, &dumlong, tempbuf,
                               &dumlong, &dumlong, &dumlong, &ceosid) != E_SUCC)
                    {
                        free(in_buf);
                        return(E_FAIL);
                    }

                    /* Read first line.  This is the file descriptor record in
                       the image file. It is a REPEAT of the file descriptor
                       record on the first tape, so ignore it.
                    ----------------------------------------------------------*/
                    status = c_tpread(unit,in_buf,recsize,&mode);
                    if (status != *recsize)
                    {
                        if (!tapeflag)
                        {
                            c_lasmsg(FALSE, "ltwgin-diskread",
                                     "Error reading imagery file descriptor"
                                     " record");
                            free(in_buf);
                            return(E_FAIL);
                        }
                        else
                        {
                            c_lasmsg(FALSE, "ltwgin-read",
                                     "Error reading imagery file descriptor"
                                     " record");
                            free(in_buf);
                            return(E_FAIL);
                        }
                    }

                    /* Now read the next line of image data
                    ---------------------------------------*/
                    status = c_tpread(unit,in_buf,recsize,&mode);
                    c_startm();
                }
            }

            if (status != *recsize)
            {
                free(in_buf);
                c_stoptm();
                if (!tapeflag)
                {
                    c_lasmsg(FALSE, "ltwgin-diskread",
                            "Error reading line %d, band %d from disk",curent,
                            curbnd);
                    return(E_FAIL);
                }
                else
                {
                    c_lasmsg(FALSE, "ltwgin-read",
                            "Error reading line %d, band %d from tape",curent,
                            curbnd);
                    return(E_FAIL);
                }
            }

            /* If this band was specified
            --------------------------*/
            if (bands[curbnd])
            {
                cb++;

                /* Position the input buffer to the start of the image data.
                ------------------------------------------------------------*/
                in_ptr = &in_buf[bandoff[curbnd] + 
                    ((window[SS] - 1) * datasize[idtype])];

                /* Write the data to the output file(s)
                ---------------------------------------*/
                if (numimg_files == 1)
                {
                    if(c_ewrite(&fdesc[0],&cb,&cl,in_ptr,&one) == E_FAIL)
                    {
                        c_lasmsg(FALSE, "ltwgin-write",
                                 "Error writing output image");
                        free(in_buf);
                        c_stoptm();
                        return(E_FAIL);
                    }
                }
                else
                {
                    if(c_ewrite(&fdesc[cb-1],&one,&cl,in_ptr,&one) == E_FAIL)
                    {
                        c_lasmsg(FALSE, "ltwgin-write", 
                                 "Error writing output image");
                        free(in_buf);
                        c_stoptm();
                        return(E_FAIL);
                    }
                }

                /* Compute the min/max values if required
                -----------------------------------------*/
                if (*minmax)
                    status = c_pxmnmx((long *)in_ptr,&window[NS],&outmin[cb],
                            &outmax[cb],&idtype);
            }

            /* Write the image data logical record to disk 
            ---------------------------------------------*/
            if (wrtdsk[IDR] && (curbnd == 0))
            {
                in_ptr = &in_buf[12];
                status = (long)fwrite(in_ptr,1,prefix,fp);
                if(status != prefix)
                {
                    c_lasmsg(FALSE, "ltwgin-write",
                            "Error writing to image data logical record file");
                    free(in_buf);
                    c_stoptm();
                    return(E_FAIL);
                }
                in_ptr = &in_buf[(*recsize - suffix)];
                status = (long)fwrite(in_ptr,1,suffix,fp);
                if(status != suffix)
                {
                    c_lasmsg(FALSE, "ltwgin-write",
                            "Error writing to image data logical record file");
                    free(in_buf);
                    c_stoptm();
                    return(E_FAIL);
                }
            }
        }
    }

    /* Close idr file
    ----------------------*/
    if (wrtdsk[IDR])
    {
        status = fclose(fp);
        if(status != 0)
        {
            c_lasmsg(FALSE,"ltwgin-write",
                    "Error closing the image data logical record file");
            free(in_buf);
            c_stoptm();
            return(E_FAIL);
        }
    }

    /* Stop the processing message
    -----------------------------*/
    c_stoptm();

    /* Close the image file(s)
    --------------------------*/
    for (i = 0; i < numimg_files; i++)
    {
        if (c_eclose(&fdesc[i]) != E_SUCC)
        {
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* One more read
    ---------------*/
    if (tapeflag)
    {
        status = c_tpread(unit,in_buf,recsize,&mode);
        if (status != TAPE_EOF)
        {
            /* Skip the rest of the file if needed 
            ---------------------------------------*/
            if ((wrtdsk[TRAI] || wrtdsk[SUPP]) && (*tapenum == *media_cnt))
            {
                if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
                {
                    free(in_buf);
                    return(E_FAIL);
                }
            }
        }
    }

    /* Best guess is that BIL images span tapes and do not have
       have leader files.
    -----------------------------------------------------------*/
    *no_lead = TRUE;

    /* Update the min max file for the output image
    -----------------------------------------------*/
    if (*minmax)
    {
        if (c_svmnmx(hosout[0],totbnd,outmin,outmax,"YES") != E_SUCC)
            c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);
    }

    /* Close input file if necessary
    --------------------------------*/
    if (!tapeflag)
        fclose(infp);

    /* Free allocated space
    -----------------------*/
    free(in_buf);

    return(E_SUCC);
}
