/******************************************************************************
FUNCTION:       PROCES_BSQ

PURPOSE:        Read image data from media and write it to disk.  The image
                data is BSQ.  Each image line contains LTWG standard 
                12 bytes, suffix and prefix data, and image data.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "cm.h"
#include "labserv.h"
#include "pixman.h"
#include "tapeio.h"
#include "ltwgin.h"

int proces_bsq
(
   char hosout[][CMLEN],         /* I/O: Output host names */
   long *window,                 /* I/O: Window of image to be copied */
   long *totbnd,                 /* O: Total number of bands to be copied */
   char filename[][CMLEN],       /* I: filenames on disk or CD */
   char sceneid[],               /* I: naming convention for NLAPS files */
   int nfiles,                   /* I: number of filenames on disk or CD */
   int dir_struct,               /* I: flag indicating which directory
                                       structure is used for data */
   int tapeflag,                 /* I: indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD, disk) */
   long *unit,                   /* I: Tape unit number */
   long *wrtdsk,                 /* I: Ancillary files to read in */
   long *num_out,                /* I: Number of output files */
   long *numimg_files,           /* I: Number of output image files */
   long *bands,                  /* I: Bands to be processed within image */
   long *msgtim,                 /* I: Time between display messages */
   long *minmax,                 /* I: Min/Max flag */
   char comment[][CMLEN],        /* I: Mount message to the operator */
   long *recsize,                /* I: Image record length */
   long *tapenum,                /* I/O: Current tape number */
   long *media_cnt,              /* I: Number of media specified */
   long *no_lead,                /* I: Flag if successive tapes have leader
                                       file */
   char ttype[],                 /* I: */
   char hpath[CMLEN],            /* I: Path to image data on disk */
   char *sensor,                 /* I: Satellite sensor */
   long ceosid                   /* I: Integer ID of receiving station */
)
{
    extern long curcat;     /* curent tape band being processed          */
    extern long curent;     /* output line being processed               */
    struct FDESC *fdesc[MAXBND]; /* image file descriptors used for image I/O */

    FILE *fp=NULL;          /* file descriptor                           */
    FILE *infp=NULL;        /* image data file (on CD only)              */

    long  access = IWRITE;  /* write access to output image              */
    long  bandoff[MAX_BAND];/* offset into each band to get to image data */
    long  cb;               /* current tape band being processed (one-based) */
    long  filename_len;     /* length of filename string */
    long  i;                /* loop counter                              */
    long  idtype;           /* data type of input image                  */
    long  mode = HANDLE;    /* tape I/O error flag                       */
    long  msgflg;           /* Processing message format flag            */
    long  need_new_tape = FALSE;
                            /* indicates if file skipping can be done or
                               if it should not be done b/c a new tape
                               needs to be mounted                       */
    long  one = 1;          /* constant one                              */
    long  option[2];        /* output option array                       */
    long  outspc[2];        /* output image file specifications          */
    long  status;           /* return status                             */
    long  temp;             /* temp value                                */
    long  dumlong;          /* dummy value                               */
    long  dum2 = FALSE;     /* dummy value                               */
    long  prefix,suffix;    /* number of prefix and suffix pixles        */
    long  curblk;           /* current block within the blocked data     */
    long  recleng;          /* record length of image descriptor record  */
    long  skip;             /* number of files to skip                   */
    long  dmod;             /* part of curblk that is needed for the window */
    long  ptrband = 1;      /* pointer to which band the tape is currently at */
    long  swreel = FALSE;   /* flag if need to switch reels              */
    long  blkfct;           /* blocking factor                           */
    long  bndcnt = 0;       /* number of bands requested                 */
    long  maxbnd;           /* maximum number of bands on the tape/disk  */

    int nlaps_nfiles;       /* Number of filenames returned              */
    int tmp;                /* temp value to pass to mount_media so tapeflag
                               won't get changed during processing somehow */

    static long datasize[4] = {0,1,2,4};
                            /* defines number bytes for each data type */

    unsigned char *in_buf;          /* buffer read from LGSOWG image */
    unsigned char *in_ptr=NULL;     /* pointer to input buffer window    */
    char filename_up[MAX_FILES][CMLEN];
                            /* names of files on disk or CD
                               (upper-case) */
    char tempbuf[10];               /* temp work buffer                  */
    char imgfile[CMLEN];            /* name of image file on disk        */
    char ipath[CMLEN];              /* temporary host path for image     */
    char nlaps_filename[10][CMLEN]; /* Array of filenames extracted      */

    double outmax[MAX_BAND];        /* max pixel values of output image  */
    double outmin[MAX_BAND];        /* min pixel values of output image  */

    /* Allocate memory for the input buffer
    ---------------------------------------*/
    in_buf = (unsigned char *) malloc(*recsize);
    if (in_buf == NULL)
    {
        c_lasmsg(FALSE, "ltwgin-alloc", "Error allocating dynamic memory");
        return(E_FAIL);
    }

    /* If reading from CD, open the image data file 
    -----------------------------------------------*/
    if (!tapeflag)
    {
        /* Convert filenames on disk or CD to all upper-case for comparisons
        --------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            strcpy(filename_up[i], filename[i]);
            filename_len = (long)strlen(filename_up[i]);
            c_low2up(filename_up[i], &filename_len);
        }

        strcpy(imgfile, hpath);

        /* Append the image filename to the imgfile string. Look through the
           array with the filenames in upper-case, then append the filename
           at the same array index from the case-sensitive array.
        --------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            if (dir_struct == NLAPS)
            {
                strcpy(ipath,hpath);
                strcat(ipath,"*.i*");
                if(c_getfiles(ipath,nlaps_filename,&nlaps_nfiles,10,1,10)
                   != E_SUCC)
                {
                    c_errmsg("Error retrieving filenames from disk",
                            "ltwgin-getfiles", &one);
                    free(in_buf);
                    return(E_FAIL);
                }
                if (nlaps_nfiles > 0)
                {
                    strcpy(imgfile, nlaps_filename[0]);
                    break;
                }
                else /* try upper-case */
                {
                    strcpy(ipath,hpath);
                    strcat(ipath,"*.I*");
                    if(c_getfiles(ipath,nlaps_filename,&nlaps_nfiles,10,1,10)
                            != E_SUCC)
                    {
                        c_errmsg("Error retrieving filenames from disk",
                                "ltwgin-getfiles", &one);
                        free(in_buf);
                        return(E_FAIL);
                    }
                    if (nlaps_nfiles > 0)
                    {
                        strcpy(imgfile, nlaps_filename[0]);
                        break;
                    }
                }
            }
            else if((dir_struct == ESATM) || (dir_struct == SAR))
            {
                if(strncmp(filename_up[i], "DAT_01.001", 10) == 0)
                {
                    strcat(imgfile, filename[i]);
                    break;
                }
            }
            else
            {
                if(strncmp(filename_up[i], "IMG_01.001", 10) == 0)
                {
                    strcat(imgfile, filename[i]);
                    break;
                }
            }
        }
        infp = fopen(imgfile, "r");
        if (infp == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-diskread",
                    "Error reading initial image data from disk");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Read first line.  This is the file descriptor record in the image file. 
       Get the record length.  This is different from the record size if the 
       image data is blocked.
    -------------------------------------------------------------------------*/
    if (!tapeflag)
        status = (long)fread(in_buf, 1, *recsize, infp);
    else
        status = c_tpread(unit, in_buf, recsize, &mode);

    recleng = atol((char *)&in_buf[186]);
    if (status != recleng)
    {
        if(tapeflag)
        {
            c_lasmsg(FALSE,"ltwgin-diskread",
                    "Error reading file descriptor record from tape");
        }
        else
        {
            c_lasmsg(FALSE,"ltwgin-diskread",
                    "Error reading file descriptor record from disk");
        }
        free(in_buf);
        return(E_FAIL);
    }

    /* If specified, open the image data logical record file and write this
       record.  This file contains the file descriptor record from the
       image file and the prefix and suffix data from the image line.
    -----------------------------------------------------------------------*/
    if (wrtdsk[IDR])
    {
        c_lsmknm(hosout[0],"idr",hosout[*num_out]);
        fp = fopen(hosout[(*num_out)++],"w");
        if(fp == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-diskread",
                    "Error reading IDR file");
            free(in_buf);
            return(E_FAIL);
        }
        if(fwrite(in_buf,1,*recsize,fp) != *recsize)
        {
            c_lasmsg(FALSE,"ltwgin-write","Error writing to IDR file");
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Set offsets into each line to get to image data. 
       xx bytes of prefix data and xx bytes of suffix data.
       12 bytes are for standard LTWG record
    ------------------------------------------------------*/
    if (JAPAN && (strncmp(sensor,"TM",2) == 0))
    {
        strncpy(tempbuf, (char *)&in_buf[280], 4);
        tempbuf[4] = '\0';
        prefix = atol(tempbuf);
    }
    else
        prefix = atol((char *)&in_buf[276]);

    suffix = atol((char *)&in_buf[288]);
    for (i = 0; i < MAX_BAND; i++)
        bandoff[i] = prefix + 12;

    /* Get data type (bits per pixel)
    ---------------------------------*/
    temp = atol((char *)&in_buf[216]);
    switch (temp)
    {
        case 8:  idtype = EBYTE;
                 break;
        default: c_lasmsg(FALSE, "ltwgin-dtype",
                         "Data type not currently handled");
                 free(in_buf);
                 return(E_FAIL);
    }

    /* Get blocking factor.  Blocking factor was not correctly specified
       within the file descriptor record.  Calculating the blocking factor
       instead should work in all instances.
    ----------------------------------------------------------------------*/
    /*blkfct = atol(&in_buf[272]);*/
    blkfct = *recsize/recleng;

    /* Get the image size
    ---------------------*/
    outspc[0]= atol((char *)&in_buf[236]);
    strncpy(tempbuf,(char *)&in_buf[248],8);
    tempbuf[8] = '\0';
    outspc[1]= atol(tempbuf);

    /* Count number of bands the user wants 
    ---------------------------------------*/
    for (i = 0; i < MAX_BAND; i++)
    {
        if (bands[i])
            bndcnt++;
    }
    *totbnd = bndcnt;
    maxbnd = MAX_BAND;

    /* If we have a South African CD, Canadian MSS or Japan 
       MSS data, there are only four bands on the tape/disk.   
    ---------------------------------------------------------*/
    if ((dir_struct == SAFRICAMSS) || (ceosid == CAN) ||
            ((JAPAN) && (strcmp(sensor,"MSS") == 0))) 
    {
        maxbnd = 4;
        if (*totbnd > 4)
        {
            c_lasmsg(FALSE,"ltwgin-bands","Only four bands are available"); 
            *totbnd = 4;
        }
    }

    /* If we have SPOT data created by the Sweden or France receiving
       stations, there are only three bands in the scene. Have not
       encountered SPOT data on CD from these receiving stations, so
       not sure if it only has three bands too.
    ----------------------------------------------------------------*/
    if(ceosid == SWEDEN)
    {
        maxbnd = 3;
        if(*totbnd > 3)
        {
            c_lasmsg(FALSE,"ltwgin-bands","Only three bands are available");
            *totbnd = 3;
        }
    }

    /* If the user wants 1 image file per band, check that the 
       number of output image files specified is equal to totbnd
    ------------------------------------------------------------*/
    if ((*numimg_files != 1) && (*numimg_files != *totbnd))
    {
        c_errmsg("Number of output images must equal number of bands specified",
                "ltwgin-bndcnt",&one);
        free(in_buf);
        return(E_FAIL);
    }

    /* Check the window
    -------------------*/
    chk_window(outspc,window);

    /* Open the output image file(s) for random write access
    --------------------------------------------------------*/
    option[0] = 0;
    option[1] = 0;
    if (*numimg_files == 1)
    {
        if (c_eopenr(&fdesc[0],hosout[0],totbnd,&one,&one,&window[NL],
                     &window[NS],&access,&idtype,option,&one) != E_SUCC)
        {
            free(in_buf);
            return(E_FAIL);
        }
    }
    else
    {
        for (i = 0; i < *numimg_files; i++)
        {
            if (c_eopenr(&fdesc[i],hosout[i],&one,&one,&one,&window[NL],
                         &window[NS],&access,&idtype,option,&one) != E_SUCC)
            {
                free(in_buf);
                return(E_FAIL);
            }
        }
    }

    /* If min max values are to be computed, call inmnmx to
       initialize the min max array for the output data type.
    --------------------------------------------------------*/
    if (*minmax)
        c_inmnmx(&idtype,totbnd,outmin,outmax);

    /* Initialize the processing message
    ------------------------------------*/
    msgflg = ONECAT;
    c_inittm("LTWGIN","Processed","line","band","",&window[NL],totbnd,
            msgtim,&msgflg);

    /* Start processing message
    ---------------------------*/
    curent = 1;
    curcat = 1;
    c_startm();

    /* Reset for processing
    -----------------------*/
    curcat = 0;

    /* Process each requested band  
    ------------------------------*/
    for (cb = 1; cb <= maxbnd; cb++)
    {
        /* Assuming bands are on three tapes - 3,3,1;
           except for 8mm, which always has just 1 physical volume.
           All 3490 (and 3480) products have just 1 physical volume
           except Thailand and ESA TM data. Tapes we've received of SPOT
           data produced by Sweden and France receiving stations has one
           band per physical volume. We'll assume this is the norm for
           this data.
        ----------------------------------------------------------------*/
        if (tapeflag)
        {
            if((cb > 1) && (ceosid == SWEDEN))
                swreel = TRUE;
            if (((cb == 4) || (cb == 7)) && (strcmp(ttype, "8mm") != 0))
            {
                if(((strcmp(ttype, "3490") == 0) ||
                    (strcmp(ttype, "3480")) == 0) &&
                   ((ceosid == THAI) || (ceosid == FUCINO)))
                    swreel = TRUE;
                else if((strcmp(ttype, "3490") != 0) &&
                        (strcmp(ttype, "3480") != 0))
                    swreel = TRUE;
            }
            if(strcmp(ttype, "8mm") == 0)
                swreel = FALSE;
            if (ceosid == CAN)
                swreel = FALSE;
        }

        if (bands[cb-1])
        {
            curcat++;
            curent = 1;

            /* Skip to correct band or window as needed
            -------------------------------------------*/
            if (!tapeflag)
            {
                fclose(infp);
                if (cdskip_bsq(window, &cb, &dmod, &blkfct, infp, recleng,
                               recsize, hpath, filename, sceneid, nfiles,
                               dir_struct) != E_SUCC)
                {
                    c_lasmsg(FALSE, "ltwgin-skip", "Error skipping records");
                    free(in_buf);
                    return(E_FAIL);
                }
            } 
            else
            {
                if (skip_bsq(window, unit, comment, ceosid, tapenum, media_cnt,
                             &cb, &dmod, &blkfct, &ptrband, &swreel) != E_SUCC)
                {
                    c_lasmsg(FALSE, "ltwgin-skip", "Error skipping records");
                    free(in_buf);
                    return(E_FAIL);
                }
                need_new_tape = FALSE; /* if necessary, a new tape was mounted
                                          so file skipping should resume */
            } 

            /* Process each line
            --------------------*/
            for (curent=0; curent < window[NL];) 
            {
                if (!tapeflag)
                {
                    status = (long)fread(in_buf, 1, *recsize, infp);
                    if (status != *recsize)
                    {
                        c_lasmsg(FALSE, "ltwgin-diskread",
                                 "Error reading line %d, band %d from disk",
                                 curent, cb);
                        free(in_buf);
                        return(E_FAIL);
                    }
                }   
                else    /* if this is a tape  */
                {
                    status = c_tpread(unit,in_buf,recsize,&mode);
                    /* EOV - end of physical volume with subsequent volumes
                       EOT - end of set - last tape. should not encounter this
                             if there is more bands to process yet, but
                             leave it here since that's how it was before */
                    if((status == TAPE_EOV) || (status == TAPE_EOT))
                    {
                        /* Close current tape, mount next tape, and read
                           volume directory file.
                        ------------------------------------------------*/
                        c_stoptm();
                        close_media(unit);
                        if (mount_media(unit, comment, ++(*tapenum), media_cnt,
                                        ttype, &tmp) != E_SUCC)
                        {
                            free(in_buf);
                            return(E_FAIL);
                        }

                        if (getvdf(unit, &dum2, filename, nfiles, tapeflag,
                                   dir_struct, hpath, sceneid, hosout, &dumlong,
                                   recsize, &dumlong, &dumlong, tempbuf,
                                   &dumlong, &dumlong, &dumlong, &ceosid)
                                   != E_SUCC)
                        {
                            free(in_buf);
                            return(E_FAIL);
                        }

                        /* Read first line.  This is the file descriptor
                           record in the image file. It is a REPEAT of the
                           file descriptor record on the first tape, so
                           ignore it.
                        --------------------------------------------------*/
                        status = c_tpread(unit,in_buf,recsize,&mode);
                        if (status != *recsize)
                        {
                            if (!tapeflag)
                            {
                                c_lasmsg(FALSE, "ltwgin-diskread",
                                        "Error reading imagery file descriptor"
                                        " record");
                                free(in_buf);
                                return(E_FAIL);
                            }
                            else
                            {
                                c_lasmsg(FALSE, "ltwgin-read",
                                        "Error reading imagery file descriptor"
                                        " record");
                                free(in_buf);
                                return(E_FAIL);
                            }
                        }

                        /* Now read the next line of image data
                        ---------------------------------------*/
                        *no_lead = TRUE;
                        status = c_tpread(unit,in_buf,recsize,&mode);
                        c_startm();
                    }

                    if (status != *recsize)
                    {
                        c_lasmsg(FALSE, "ltwgin-read",
                                 "Error reading line %d, band %d from tape",
                                 curent, cb);
                        free(in_buf);
                        c_stoptm();
                        return(E_FAIL);
                    }
                } 

                /* Position the input pointer to the start of the image data.
                ------------------------------------------------------------*/
                for(curblk=dmod; (curblk<blkfct) && (curent < window[NL]);
                    curblk++)
                {
                    in_ptr = &in_buf[(recleng * curblk * datasize[idtype]) +
                        bandoff[cb-1] + 
                        ((window[SS] - 1) * datasize[idtype])];

                    /* Write the data to the output file
                    -------------------------------------*/
                    curent++;
                    if (*numimg_files == 1)
                    {
                        if (c_ewrite(&fdesc[0],&curcat,&curent,in_ptr,&one)
                            == E_FAIL)
                        {
                            c_lasmsg(FALSE, "ltwgin-write",
                                    "Error writing output image");
                            free(in_buf);
                            c_stoptm();
                            return(E_FAIL);
                        }
                    }
                    else
                    {
                        if (c_ewrite(&fdesc[curcat-1],&one,&curent,in_ptr,&one) 
                            == E_FAIL)
                        {
                            c_lasmsg(FALSE, "ltwgin-write",
                                    "Error writing output image");
                            free(in_buf);
                            c_stoptm();
                            return(E_FAIL);
                        }
                    }
                }
                dmod = 0;

                /* Compute the min/max values if required
                -----------------------------------------*/
                if (*minmax)
                    status = c_pxmnmx((long *)in_ptr,&window[NS],
                                      &outmin[curcat-1], &outmax[curcat-1],
                                      &idtype);

                /* Write the image data logical record to disk
                ---------------------------------------------*/
                if (wrtdsk[IDR] && (cb == 1))
                {
                    in_ptr = &in_buf[12];
                    if(fwrite(in_ptr,1,prefix,fp) != prefix)
                    {
                        c_lasmsg(FALSE,"ltwgin-write",
                                 "Error writing to IDR file");
                        free(in_buf);
                        c_stoptm();
                        return(E_FAIL);
                    }
                    in_ptr = &in_buf[(*recsize - suffix)];
                    if(fwrite(in_ptr,1,suffix,fp) != suffix)
                    {
                        c_lasmsg(FALSE,"ltwgin-write",
                                 "Error writing to IDR file");
                        free(in_buf);
                        c_stoptm();
                        return(E_FAIL);
                    }
                }
            }  
        }  
        /* Skip to the next image file.  If this is the last tape,
           skip to the end of the file as needed. 
           Assuming bands are on three tapes - 3,3,1
        ----------------------------------------------------------*/
        if (tapeflag)
        {
            skip = 0;

            /* If not at the last band yet, skip over trailer and leader
               files to position file pointer for read of next band's
               imagery file. If 8mm or 3490 or 3480 and Thailand or ESA TM
               data there is only one physical volume so skip the trailer
               and leader files between imagery files for every band
               except maxbnd. Assuming all SPOT data produced by the Sweden
               and France receiving stations has one band per physical
               volume, so there is no need to skip files.
            ---------------------------------------------------------------*/
            if((((strcmp(ttype, "8mm") == 0) && (cb != maxbnd)) ||
                (((strcmp(ttype, "3490") == 0) ||
                  (strcmp(ttype, "3480") == 0)) &&
                 ((ceosid != THAI) && (ceosid != FUCINO)) && (cb != maxbnd)) ||
                ((cb != maxbnd) && (cb != 3) && (cb != 6) && !need_new_tape &&
                 (strcmp(ttype, "8mm") != 0))) && (!(ceosid == SWEDEN)))
            {
                /* Do one more read to determine if we were at end of imagery
                   file or if we need to skip past the end of the file because
                   the user sub-windowed the output product
                --------------------------------------------------------------*/
                status = c_tpread(unit,in_buf,recsize,&mode);
                if (status != TAPE_EOF)
                {
                    /* skip rest of current imagery file, the current band's
                       trailer file, and the next band's leader file
                    --------------------------------------------------------*/
                    skip = 3;
                }
                else
                {
                    /* just skip the current band's trailer file and the
                       next band's leader file
                    ----------------------------------------------------*/
                    skip = 2;
                }

                /* Now do the actual file skipping
                ----------------------------------*/
                status = c_tpfsf(unit,&skip,&mode);
                if (status != TAPE_SUCC)
                {
                    free(in_buf);
                    c_stoptm();
                    c_lasmsg(FALSE, "ltwgin-fileskip",
                            "Error skipping files on tape");
                    return(E_FAIL);
                }
            }
            else
            {
                /* Indicate not to skip any more files until
                   the next tape gets mounted
                --------------------------------------------*/
                need_new_tape = TRUE;
            }

            /* Just got done with last band so position the file pointer
               for reading the trailer and/or supplementary file(s)
            ------------------------------------------------------------*/
            if(cb == maxbnd)
            {
                /* If user wants the trailer and/or supplementary file(s),
                   move the file pointer, but not if this isn't the last tape,
                   since getenf will dismount it and mount the last tape
                --------------------------------------------------------------*/
                if((wrtdsk[TRAI] || wrtdsk[SUPP]) && (*tapenum == *media_cnt))
                {
                    /* Do one more read to determine if we were at end of
                       imagery file or if we need to skip past the end of
                       the file because the user sub-windowed the output
                       product
                    ------------------------------------------------------*/
                    status = c_tpread(unit,in_buf,recsize,&mode);
                    if (status != TAPE_EOF)
                    {
                        /* skip rest of current imagery file and leave
                           file pointer here to read trailer and/or
                           supplementary file(s)
                        ----------------------------------------------*/
                        skip = 1;
                    }
                    else
                    {
                        /* already at end of imagery file - leave file pointer
                           here to read trailer and/or supplementary file(s)
                        ------------------------------------------------------*/
                        skip = 0;
                    }

                    /* Now do the actual file skipping
                    ----------------------------------*/
                    status = c_tpfsf(unit,&skip,&mode);
                    if (status != TAPE_SUCC)
                    {
                        free(in_buf);
                        c_stoptm();
                        c_lasmsg(FALSE, "ltwgin-fileskip",
                                "Error skipping files on tape");
                        return(E_FAIL);
                    }
                }
            }
        }

        /* At next band 
        ---------------*/
        ptrband = cb + 1;
    }

    /* Close idr file
    -----------------*/
    if (wrtdsk[IDR])
    {
        if(fclose(fp) != 0)
        {
            c_lasmsg(FALSE,"ltwgin-write","Error closing the IDR file");
            free(in_buf);
            c_stoptm();
            return(E_FAIL);
        }
    }

    /* Stop the processing message
    -----------------------------*/
    c_stoptm();

    /* Close the image file(s)
    --------------------------*/
    for (i = 0; i < *numimg_files; i++)
    {
        if (c_eclose(&fdesc[i]) != E_SUCC)
        {
            free(in_buf);
            return(E_FAIL);
        }
    }

    /* Update the min max file for the output image
    -----------------------------------------------*/
    if (*minmax)
    {
        if (c_svmnmx(hosout[0],totbnd,outmin,outmax,"YES") != E_SUCC)
            c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);
    }

    /* Free allocated space
    -----------------------*/
    free(in_buf);

    /* Close file opened for CD read
    ---------------------------------*/
    if (!tapeflag)
        fclose(infp);

    return(E_SUCC);
}
