/*******************************************************************************
FUNCTION:       UPDATE_DDR

PURPOSE:        Updates the DDR information for an image

ALGORITHM 

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "ddr.h"
#include "geompak.h"
#include "proj.h"
#include "tm.h"
#include "ltwgin.h"

int update_ddr
(
  char hosout[][CMLEN],          /* Host file name */
  long *window,                  /* Input window */
  long *totbnd,                  /* Total number of bands to process */
  double fcenll[],               /* Full scene center lat/long degrees */
  double fcenls[],               /* Full scene center line/sample */
  long *path,                    /* Path number */
  char *direction,               /* Sat direction */
  char *maprj,                   /* Map projection */
  double pixsiz[],               /* Pixel size */
  long *satid,                   /* Satellite id */
  char *sensor,                  /* Sensor type */
  char data_type,                /* Type of data (corrected/uncorrected) */
  double *angle,                 /* Rotation angle */
  long *datum,                   /* Datum of the data */
  long *utmzone,                 /* UTM zone code */
  char *satnam,                  /* Satellite name */
  char cornlat[][17],            /* Corner latitudes */
  char cornlong[][17],           /* Corner longitudes */
  double proj_corners[8],        /* Projection corners */
  double ls_corners[8],          /* Line/sample corners */
  long *numimg_files,            /* Number of output image files */
  long ceosid,                   /* ID number of ground station */
  int data_set_summ_rec_flg,     /* indicates if the Data Set Summary
                                    record in the SAR format is present */
  int map_proj_data_rec_flg,     /* indicates if the Map Projection Data
                                    record in the SAR format is present */
  struct DDR_FLAGS *ddr_flags    /* flags to indicate if sscanf was successful
                                    in retrieving numerical values from
                                    strings retrieved from leader file. */
)
{
    struct FDESC *filedesc;    /* image file descriptors used for image I/O */

    struct DDR ddr;      /* DDR structure                        */
    struct BDDR bddr[7]; /* Band DDR structure                   */
    double out_corners[8];       /* output corners from c_file_corner    */

    unsigned char *buff;    /* Temporary buffer for image file read */

    long access = IREAD;    /* read access to output image          */
    long idtype = EBYTE;  /* the only data type handled by process functions */
    long band = 1;               /* Band looping variable                */
    long line;           /* Line where center is located         */
    long option[2];         /* output option array                  */
    long inunit = 4;     /* input unit - deg                     */
    long outunit = 2;    /* output unit - meters                 */
    long inzone = 62;    /* input zone                           */
    long outzone = 62;   /* output zone                          */
    long indatum = 0;    /* input datum                          */
    long outdatum = 0;   /* output datum                         */
    long line_samp[2];   /* line/sample pair                     */
    long i;                      /* loop counter                         */
    long fill=0;         /* fill counter                         */
    long inproj;         /* input projection                     */
    long prtprm[2];         /* report to term and/or file           */
    long one = 1;                /* constant of one                      */
    long multfact;               /* sign value                           */
    long status;         /* return status of call                */

    double inparm[15];      /* input projection parameters          */
    double fcproj[2];       /* Full scene center degrees                 */
    double deglat[4];    /* corner corndinates in degrees        */
    double deglong[4];   /* corner corndinates in degrees        */
    double minutes;              /* minutes                              */
    double seconds;              /* seconds                              */

    char ctemp[5];          /* Temp buffer                          */

    buff = malloc(sizeof(unsigned char) * window[NS]);
    if (buff == NULL)
    {
        c_lasmsg(FALSE,"ltwgin-alloc","Error allocating dynamic memory");
        return(E_FAIL);
    }

    /* Get the DDR information from the image file
    ----------------------------------------------*/
    if (c_getddr(hosout[0],&ddr) != E_SUCC)
    {
        free(buff);
        return(E_FAIL);
    }

    /* Calculate the corner coordinates for these projections
    ---------------------------------------------------------*/
    if (((strncmp(maprj,"SOM",3) == 0) || (strncmp(maprj,"UTM",3) == 0)) &&
        (ceosid != RSAT))
    {
        inproj = 0;
        prtprm[0] = 0;
        prtprm[1] = 0;

        for (i = 0; i < 15; i++)
            inparm[i] = 0.0;

        /* Initialize the validity flags
        --------------------------------*/
        for (i = 0; i < 8; i++)
            ddr.valid[i] = VALID;

        if (strncmp(maprj,"SOM",3) == 0)
        {
            ddr.proj_code = SOM;
            ddr.zone_code = 62;
            if(ddr_flags->path)
            {
                if (somprm(path,ddr.proj_coef) != E_SUCC)
                {
                    free(buff);
                    return(E_FAIL);
                }
            }
            else
            {
                /* WRS path was not successfully retrieved from
                   the leader file */
                for(i = 0; i < 15; i++)
                    ddr.proj_coef[i] = 0.0;
                ddr.valid[DDPPV] = INVAL;
            }
        }
        else
        {
            ddr.proj_code = UTM;
            if(ddr_flags->utmzone)
                ddr.zone_code = *utmzone;
            else
            {
                /* UTM zone not successfully retrieved from the leader file */
                ddr.zone_code = 0;
                ddr.valid[DDZCV] = INVAL;
            }
            if(ddr_flags->fcenll)
            {
                ddr.proj_coef[0] = fcenll[1];
                ddr.proj_coef[1] = fcenll[0];
            }
            else
            {
                /* Scene center lat and long not successfully retrieved
                   from the leader file */
                ddr.proj_coef[0] = ddr.proj_coef[1] = 0.0;
                ddr.valid[DDPPV] = INVAL;
            }

            if (ceosid == HBK)
                *datum = 1;

            if(ddr_flags->utmzone) /* let them default to 62 if
                                      not in leader file */
            {
                inzone = *utmzone;
                outzone = *utmzone;
            }
        }
        if (ceosid == HBK)
        {
            if(ddr_flags->proj_corners && ddr_flags->ls_corners)
            {
                c_file_corner(proj_corners, ls_corners, window, out_corners);
                ddr.upleft[0] = out_corners[UPLEFT_Y];
                ddr.upleft[1] = out_corners[UPLEFT_X];
                ddr.upright[0] = out_corners[UPRIGHT_Y];
                ddr.upright[1] = out_corners[UPRIGHT_X];
                ddr.loright[0]= out_corners[LORIGHT_Y];
                ddr.loright[1] = out_corners[LORIGHT_X];
                ddr.loleft[0] = out_corners[LOLEFT_Y];
                ddr.loleft[1] = out_corners[LOLEFT_X];
                strcpy(ddr.proj_units,"meters");
            }
            else
            {
                /* projection corners or line/sample corners not successfully
                   retrieved from leader file */
                ddr.upleft[0] = ddr.upleft[1] = 0.0;
                ddr.upright[0] = ddr.upright[1] = 0.0;
                ddr.loright[0] = ddr.loright[1] = 0.0;
                ddr.loleft[0] = ddr.loleft[1] = 0.0;
                ddr.valid[DDCCV] = ddr.valid[DDPUV] = INVAL;
            }
        }
        else if ((JAPAN) && (strcmp(sensor,"MSS") == 0) && (data_type == 'A'))
        {
            c_errmsg("Corner coordinates will be approximate","ltwgin-corn",
                     &one);

            ddr.zone_code = inzone;

            if(ddr_flags->fcenll)
            {
                /* calcorn will give us the approximate corners
                -----------------------------------------------*/
                strncpy(ctemp,direction,1);
                ctemp[1] = '\0';
                calcorn (fcenll[0], fcenll[1], ctemp, deglat, deglong, *satid);

                ddr.upleft[0] = deglat[0];
                ddr.upleft[1] = deglong[0];
                ddr.upright[0] = deglat[3];
                ddr.upright[1] = deglong[3];
                ddr.loright[0] = deglat[2];
                ddr.loright[1] = deglong[2];
                ddr.loleft[0] = deglat[1];
                ddr.loleft[1] = deglong[1];
                strcpy(ddr.proj_units,"degrees");
            }
            else
            {
                /* Scene center lat and long not successfully
                   retrieved from leader file */
                ddr.upleft[0] = ddr.upleft[1] = 0.0;
                ddr.upright[0] = ddr.upright[1] = 0.0;
                ddr.loright[0] = ddr.loright[1] = 0.0;
                ddr.loleft[0] = ddr.loleft[1] = 0.0;
                ddr.valid[DDCCV] = ddr.valid[DDPUV] = INVAL;
            }
        }
        else
        {
            /* With SOM, if the WRS path was not successfully retrieved from
               the leader file, then the first two projection coefficients
               were set to 0.0. This would cause c_transinit to fail and,
               thus, update_ddr to return E_FAIL, which we don't want to
               happen. So, if the WRS path could not be retrieved, then
               just skip the PTP calls.
            ----------------------------------------------------------------*/
            if ((strncmp(maprj,"SOM",3 != 0)) ||
                    ((strncmp(maprj,"SOM",3) == 0) && (ddr_flags->path)))
            {
                /* Initialize the Projection Transformation Package
                --------------------------------------------------*/
                if (c_transinit(&inproj,&inunit,&inzone,&indatum,inparm,
                        &ddr.proj_code,&outunit,&outzone,&outdatum,
                        ddr.proj_coef,prtprm,"") != E_SUCC)
                {
                    free(buff);
                    return(E_FAIL);
                }
            }

            if (ceosid == THAI)
            {
                option[0] = 0;
                option[1] = 0; 
                if (*numimg_files == 1)
                {
                    status = c_eopenr(&filedesc,hosout[0],totbnd,&one,&one,
                            &window[NL],&window[NS],&access,&idtype,
                            option,&one);
                }
                else
                {
                    status = c_eopenr(&filedesc,hosout[0],&one,&one,&one,
                            &window[NL],&window[NS],&access,&idtype,
                            option,&one);
                }
                if (status != E_SUCC)
                {
                    free(buff);
                    return(E_FAIL);
                }

                if(ddr_flags->fcenls)
                {
                    line = (long)fcenls[0];
                    status = c_eread(&filedesc,&one,&line,buff,&one);
                    if (status == E_FAIL)
                    {
                        free(buff);
                        return(E_FAIL);
                    }
                }

                /* Close the image file
                -----------------------*/
                if (c_eclose(&filedesc) != E_SUCC)
                {
                    free(buff);
                    return(E_FAIL);
                }

                if(ddr_flags->fcenls)
                {
                    /* Interpret the values of buff as an integer
                    --------------------------------------------*/
                    while (buff[fill] == 0)
                        fill++;
                    fcenls[1] = fcenls[1] + fill;
                }
            }

            if((ddr_flags->fcenll && ddr_flags->fcenls && ddr_flags->pixsiz &&
                ddr_flags->angle) &&
               ((strncmp(maprj,"SOM",3 != 0)) ||
                ((strncmp(maprj,"SOM",3) == 0) && (ddr_flags->path))))
            {
                /* Calculate the center projection coordinates. With SOM, if
                   the WRS path was not successfully retrieved from the
                   leader file, then the first two projection coefficients
                   were set to 0.0. This would cause c_transinit to fail and,
                   thus, update_ddr to return E_FAIL, which we don't want to
                   happen. So, if the WRS path could not be retrieved, then
                   just skip the PTP calls.
                -------------------------------------------------------------*/
                if (c_trans(&inproj,&inunit,&ddr.proj_code,&outunit,
                        &fcenll[1],&fcenll[0],&fcproj[1],&fcproj[0]) != E_SUCC)
                {
                    free(buff);
                    return(E_FAIL);
                }

                /* Find the projection coordinates for the corners
                -------------------------------------------------*/
                line_samp[0] = window[SL];
                line_samp[1] = window[SS];
                ls_proj(angle,fcproj,fcenls,ddr.upleft,line_samp,pixsiz);

                line_samp[0] = window[SL] + window[NL] - 1;
                line_samp[1] = window[SS];
                ls_proj(angle,fcproj,fcenls,ddr.loleft,line_samp,pixsiz);

                line_samp[0] = window[SL];
                line_samp[1] = window[SS] + window[NS] - 1;
                ls_proj(angle,fcproj,fcenls,ddr.upright,line_samp,pixsiz);

                line_samp[0] = window[SL] + window[NL] - 1;
                line_samp[1] = window[SS] + window[NS] - 1;
                ls_proj(angle,fcproj,fcenls,ddr.loright,line_samp,pixsiz);

                strcpy(ddr.proj_units,"meters");
            }
            else
            {
                /* Scene center lat and long, scene center line/sample,
                   ground pixel size, or rotation angle not successfully
                   retrieved from leader file */
                ddr.upleft[0] = ddr.upleft[1] = 0.0;
                ddr.loleft[0] = ddr.loleft[1] = 0.0;
                ddr.upright[0] = ddr.upright[1] = 0.0;
                ddr.loright[0] = ddr.loright[1] = 0.0;
                ddr.valid[DDCCV] = ddr.valid[DDPUV] = INVAL;
            }
        }

        /* Update the values that are known
        -----------------------------------*/
        ddr.master_line   = window[SL] - 1;
        ddr.master_sample = window[SS] - 1;
        ddr.line_inc   = 1.0;
        ddr.sample_inc = 1.0;
        if(ddr_flags->pixsiz)
        {
            ddr.pdist_y = pixsiz[0];
            ddr.pdist_x = pixsiz[1];
        }
        else
        {
            /* Pixel size not successfully retrieved from leader file */
            ddr.pdist_y = ddr.pdist_x = 0.0;
            ddr.valid[DDPDV] = INVAL;
        }
        if (*datum < 0)
        {
            ddr.valid[DDDCV] = INVAL;
            if(ceosid == THAI)
            {
                c_lasmsg(FALSE, "ltwgin-proj", "This data set is from the "
                         "Thailand receiving station. The datum for this data "
                         "is unclear and is being marked invalid in the DDR. "
                         "It could be any of the datums numbered 183-189, "
                         "194, 195, or 308. If anyone ever confirms the "
                         "correct datum for this data, please let the "
                         "developers know.");
            }
        }
        else
            ddr.datum_code = *datum;
    }

    else if ((strncmp(maprj,"NNN",3) == 0) || (strncmp(maprj,"YN",2) == 0))
    {
        /* No map projection, not UTM, and not using the reserved field
        ---------------------------------------------------------------*/
        c_errmsg("Corner coordinates will be approximate","ltwgin-corn",&one);

        /* Initialize the validity flags
        --------------------------------*/
        for (i = 0; i < 8; i++)
            ddr.valid[i] = VALID;

        ddr.zone_code = inzone;

        if(ddr_flags->fcenll)
        {
            /* calcorn will give us the approximate corners
            -----------------------------------------------*/
            strncpy(ctemp,direction,1);
            ctemp[1] = '\0';
            calcorn (fcenll[0], fcenll[1], ctemp, deglat, deglong, *satid);

            ddr.upleft[0] = deglat[0];
            ddr.upleft[1] = deglong[0];
            ddr.upright[0] = deglat[3];
            ddr.upright[1] = deglong[3];
            ddr.loright[0] = deglat[2];
            ddr.loright[1] = deglong[2];
            ddr.loleft[0] = deglat[1];
            ddr.loleft[1] = deglong[1];
            strcpy(ddr.proj_units,"degrees");
        }
        else
        {
            /* Scene center lat and long not successfully
               retrieved from leader file */
            ddr.upleft[0] = ddr.upleft[1] = 0.0;
            ddr.loleft[0] = ddr.loleft[1] = 0.0;
            ddr.upright[0] = ddr.upright[1] = 0.0;
            ddr.loright[0] = ddr.loright[1] = 0.0;
            ddr.valid[DDCCV] = ddr.valid[DDPUV] = INVAL;
        }
        if (*datum < 0)
            ddr.valid[DDDCV] = INVAL;
        else
            ddr.datum_code = *datum;

        /* Update some values
        ---------------------*/
        ddr.master_line   = 0;
        ddr.master_sample = 0;
        ddr.line_inc   = 1.0;
        ddr.sample_inc = 1.0;

        /* Approximate pixel size in degrees
        ------------------------------------*/
        ddr.pdist_y = fabs(ddr.upleft[0] - ddr.loleft[0]) / window[NL];
        ddr.pdist_x = ddr.pdist_y;
    }
    else if (strncmp(satnam,"SP",2) == 0) /* SPOT */
    {
        c_errmsg("Corner coordinates will be approximate","ltwgin-corn",&one);

        /* Set the corners for SPOT Levels 1A and 1B. We have the four
           corners in a character string in DDDMMSS.  Level 2 is registered
           to a map but we don't have level2 data.  First convert
           coordinates to total degrees.
        --------------------------------------------------------------*/
        for (i = 0; i < 4; i++)
        {
            if (cornlat[i][0] == 'N')
                multfact = 1;
            else
                multfact = -1;

            strncpy(ctemp,&cornlat[i][1],3);
            ctemp[3] = '\0';
            deglat[i] = (double)atol(ctemp);
            strncpy(ctemp,&cornlat[i][4],2);
            ctemp[2] = '\0';
            minutes = (double)atol(ctemp);
            strncpy(ctemp,&cornlat[i][6],2);
            ctemp[2] = '\0';
            seconds = (double)atol(ctemp);

            deglat[i] = (deglat[i] + (minutes/60) + (seconds/3600)) * multfact;

            if (cornlong[i][0] == 'E')
                multfact = 1;
            else
                multfact = -1;

            strncpy(ctemp,&cornlong[i][1],3);
            ctemp[3] = '\0';
            deglong[i] = (double)atol(ctemp);
            strncpy(ctemp,&cornlong[i][4],2);
            ctemp[2] = '\0';
            minutes = (double)atol(ctemp);
            strncpy(ctemp,&cornlong[i][6],2);
            ctemp[2] = '\0';
            seconds = (double)atol(ctemp);

            deglong[i] = (deglong[i] + (minutes/60) + (seconds/3600)) *
                         multfact;
        }

        /* Initialize the validity flags
        --------------------------------*/
        for (i = 0; i < 8; i++)
            ddr.valid[i] = VALID;

        /* Fill in DDR fields
        ---------------------*/
        ddr.zone_code = inzone;
        ddr.upleft[0] = deglat[0];
        ddr.upleft[1] = deglong[0];
        ddr.upright[0] = deglat[1];
        ddr.upright[1] = deglong[1];
        ddr.loright[0] = deglat[3];
        ddr.loright[1] = deglong[3];
        ddr.loleft[0] = deglat[2];
        ddr.loleft[1] = deglong[2];
        strcpy(ddr.proj_units,"degrees");
        if (*datum < 0)
            ddr.valid[DDDCV] = INVAL;
        else
            ddr.datum_code = *datum;

        /* Update some values
        ---------------------*/
        ddr.master_line   = 0;
        ddr.master_sample = 0;
        ddr.line_inc   = 1.0;
        ddr.sample_inc = 1.0;

        /* Approximate pixel size in degrees
        ------------------------------------*/
        ddr.pdist_y = fabs(ddr.upleft[0] - ddr.loleft[0]) / window[NL];
        ddr.pdist_x = ddr.pdist_y;
    }
    else if (ceosid == RSAT)
    {
        /* Until the Radarsat data products are fully supported, fatal
           error if this somehow gets executed, since it should not be reached
           unless we get ahold of some 8-bit test data.
        ----------------------------------------------------------------------*/
        c_lasmsg(FALSE,"ltwgin-radar","Radarsat data not yet fully supported");
        return(E_FAIL);

        /* Initialize the validity flags
        --------------------------------*/
        for (i = 0; i < 8; i++)
            ddr.valid[i] = VALID;

        if(map_proj_data_rec_flg)
        {
            for (i = 0; i < 4; i++)
            {
                deglat[i] = (double)atol(cornlat[i]);
                deglong[i] = (double)atol(cornlong[i]);
            }

            if(strcmp(maprj, "UTM") == 0)
            {
                ddr.proj_code = UTM;
                if(ddr_flags->utmzone)
                    ddr.zone_code = *utmzone;
                else
                {
                    /* UTM zone not successfully retrieved from leader file */
                    ddr.zone_code = 0;
                    ddr.valid[DDZCV] = INVAL;
                }
            }
            else if(strcmp(maprj, "ALBERS") == 0)
                ddr.proj_code = ALBERS;
            else if(strcmp(maprj, "AZMEQD") == 0)
                ddr.proj_code = AZMEQD;
            else if(strcmp(maprj, "EQUIDC") == 0)
                ddr.proj_code = EQUIDC;
            else if(strcmp(maprj, "EQRECT") == 0)
                ddr.proj_code = EQRECT;
            else if(strcmp(maprj, "GVNSP") == 0)
                ddr.proj_code = GVNSP;
            else if(strcmp(maprj, "GNOMON") == 0)
                ddr.proj_code = GNOMON;
            else if(strcmp(maprj, "LAMAZ") == 0)
                ddr.proj_code = LAMAZ;
            else if(strcmp(maprj, "LAMCC") == 0)
                ddr.proj_code = LAMCC;
            else if(strcmp(maprj, "HOM") == 0)
                ddr.proj_code = HOM;
            else if(strcmp(maprj, "TM") == 0)
                ddr.proj_code = TM;
            else if(strcmp(maprj, "MERCAT") == 0)
                ddr.proj_code = MERCAT;
            else if(strcmp(maprj, "MILLER") == 0)
                ddr.proj_code = MILLER;
            else if(strcmp(maprj, "ORTHO") == 0)
                ddr.proj_code = ORTHO;
            else if(strcmp(maprj, "PS") == 0)
                ddr.proj_code = PS;
            else if(strcmp(maprj, "POLYC") == 0)
                ddr.proj_code = POLYC;
            else if(strcmp(maprj, "SNSOID") == 0)
                ddr.proj_code = SNSOID;
            else if(strcmp(maprj, "SPCS") == 0)
                ddr.proj_code = SPCS;
            else if(strcmp(maprj, "STEREO") == 0)
                ddr.proj_code = STEREO;
            else if(strcmp(maprj, "VGRINT") == 0)
                ddr.proj_code = VGRINT;
        }
        else
        {
            /* If the Map Projection Data record was not in the leader
               file, these values were not retrieved */
            for(i = 0; i < 4; i++)
                deglat[i] = deglong[i] = 0.0;
            ddr.proj_code = 0;
            ddr.valid[DDCCV] = ddr.valid[DDPCV] = INVAL;
        }

        if(data_set_summ_rec_flg && ddr_flags->fcenll)
        {
            ddr.proj_coef[0] = fcenll[1];
            ddr.proj_coef[1] = fcenll[0];
        }
        else
        {
            /* If the Data Set Summary record was not in the leader
               file or the scene center lat and long were not successfully
               retrieved, these values were not retrieved */
            ddr.proj_coef[0] = ddr.proj_coef[1] = 0.0;

            ddr.valid[DDPPV] = INVAL;
        }

        /* Initialize the validity flags
        --------------------------------*/
        for (i = 0; i < 8; i++)
            ddr.valid[i] = VALID;

        /* Fill in DDR fields
        ---------------------*/
        ddr.upleft[0] = deglat[0];
        ddr.upleft[1] = deglong[0];
        ddr.upright[0] = deglat[1];
        ddr.upright[1] = deglong[1];
        ddr.loright[0] = deglat[3];
        ddr.loright[1] = deglong[3];
        ddr.loleft[0] = deglat[2];
        ddr.loleft[1] = deglong[2];
        strcpy(ddr.proj_units,"degrees");

        if (*datum < 0)
            ddr.valid[DDDCV] = INVAL;
        else
            ddr.datum_code = *datum;

        /* Update some values
        ---------------------*/
        ddr.master_line = 0;
        ddr.master_sample = 0;
        ddr.line_inc = 1.0;
        ddr.sample_inc = 1.0;
        if(map_proj_data_rec_flg && ddr_flags->pixsiz)
        {
            ddr.pdist_y = pixsiz[0];
            ddr.pdist_x = pixsiz[1];
        }
        else
        {
            /* If the Map Projection Data record was not in the leader
               file or the pixel size was not successfully retrieved, these
               values were not retrieved */
            ddr.pdist_y = ddr.pdist_x = 0.0;
            ddr.valid[DDPDV] = INVAL;
        }
    }
    else
    {
        c_errmsg("Unable to determine projection information","ltwgin-proj",
                &one);
    }

    /* Update the Band DDR information
    ----------------------------------*/
    for (band = 1; band <= *totbnd; band++)
    {
        /* Get the Band DDR information from the image
        ----------------------------------------------- */
        if (*numimg_files == 1)
        {
            if (c_getbdr(hosout[0],&bddr[band-1],&band) != E_SUCC)
            {
                free(buff);
                return(E_FAIL);
            }
        }
        else
        {
            if (c_getbdr(hosout[band-1],&bddr[band-1],&one) != E_SUCC)
            {
                free(buff);
                return(E_FAIL);
            }
        }

        /* Update the Band DDR information
        ----------------------------------*/
        if (strncmp(satnam,"LS",2) == 0)
        {
            sprintf(bddr[band-1].source,"Landsat %ld",*satid);
            strcpy(bddr[band-1].instrument,sensor);

            /* Update the direction of the pass
            -----------------------------------*/
            if (strncmp(direction,"D",1) == 0)
                strcpy(bddr[band-1].direction,"Descending");
            else
                strcpy(bddr[band-1].direction,"Ascending");
        }
        else if (strncmp(satnam,"SP",2) == 0)
            sprintf(bddr[band-1].source,"SPOT %ld",*satid);
        else if (strncmp(satnam,"RS",2) == 0)
        {
            sprintf(bddr[band-1].source,"Radarsat-%ld",*satid);
            strcpy(bddr[band-1].instrument,sensor);

            /* Update the direction of the pass
            -----------------------------------*/
            if(data_set_summ_rec_flg)
            {
                if (strncmp(direction,"D",1) == 0)
                    strcpy(bddr[band-1].direction,"Descending");
                else
                    strcpy(bddr[band-1].direction,"Ascending");
            }
        }
    }

    /* Put the updated DDR information to the image
    -----------------------------------------------*/
    for (i = 0; i < *numimg_files; i++)
    {
        if (c_putddr(hosout[i],&ddr) != E_SUCC)
        {
            free(buff);
            return(E_FAIL);
        }
    }

    /* Put the updated Band DDR information to the image
    ----------------------------------------------------*/
    for (band = 0; band < *totbnd; band++)
    {
        if (*numimg_files == 1)
        {
            if (c_putbdr(hosout[0],&bddr[band]) != E_SUCC)
            {
                free(buff);
                return(E_FAIL);
            }
        }
        else
        {
            if (c_putbdr(hosout[band],&bddr[band]) != E_SUCC)
            {
                free(buff);
                return(E_FAIL);
            }
        }
    }

    free(buff);
    return(E_SUCC);
}
