/******************************************************************************
NAME:   			RDDLTF 

PURPOSE:    To read type 2 records from a dltf entry, and, if necessary, convert
	    it to the correct data type.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   B. Davis        01/89	  EDC		Original development

ALGORITHM:
	if the dltf type is B(byte)
		call r2dltf to read dltf into the look up table
		if the type of dltf required for processing is not Byte
			read the DMS saved slope and offset, or calculate them
			convert the look up table
	else the dltf type is I2(I*2)
		call r2dltf to read dltf into the look up table
		if the type of dltf required for processing is I2
			construct the full lut needed by pxlkup in applut
		else the type of dltf required for processing is not I2
			call dtcvrt to calculate slope and offset
			convert lut to byte

ALGORITHM REFERENCES:		dtcvrt is called to calculate the slope and
				offset required to convert a lut from i*2 to
				byte.  It is a dms support routine that is 
				incorporated to eliminate dms includes.  This
				algorithm was modified slightly and included
				in this source file to accommodate converting
				a lut from byte to i*2, because the i*2 luts
				range from image min to image max, not data
				type min to data type max, as would be the
				case if dtcvrt were used.  dtcvrt is used for
				converting from i*2 to byte because the full
				byte dltf is written to a dltf file when saved.
				When an i*2 dltf is saved, only the entries from
				startpt to endpt are written to the dltf file.
				Therefore when converting a byte dltf to i*2,
				startpt and endpt are obtained from the input
				image(s) min and max values.
******************************************************************************/

#include <stdio.h>
#include <sys/file.h>
#include "las.h"
#include "dltf.h"
#include "typlim.h"
#include "map.h"

#define LUTSIZ 256


int rddltf
(
    long *otype,           /* data type of output image                     */
    long *ltype,           /* data type of lut required                     */
    long *ldesc,           /* file descriptor for dltf file                 */
    char *inkey,           /* key field of input dltf entry to read         */
    long *bnd,             /* band number of dltf entry to be read          */
    char *indtype,         /* data type of input dltf entry to read         */
    long *indtlen,         /* data portion length of 2nd dltf rec           */
    unsigned char *blut,   /* pointer to byte look up table array           */
    short *lut,            /* pointer to I*2 look up table array            */
    double *mn,	           /* min pixel value of input image(s)             */
    double *mx,	           /* max pixel value of input image(s)             */
    long *instpt,          /* input dltf starpt                             */
    long *inenpt           /* input dltf endpt                              */
)
{
      	double 	slope;		/* slope of input dltf entry to be read */
	double	lslope,		/* Slope for a scaled LUT.		*/
		dslope;		/* Slope from the second type of DLTF
				   record; unused by DMS.		*/
	double	intr;		/* intermediate value for expanding byte
				   blut to word lut to enhance precision*/
	double	lowx, highx;	/* The x values used for interpolation
				   of the DLTF LUTs.			*/
	double	lowy, highy;	/* The y values used for interpolation
				      of the DLTF LUTs.			*/
	double	loffset,	/* Offset for a scaled LUT.		*/
		doffset;	/* Offset from the second type of DLTF
				   record; unused by DMS.		*/
      	double 	offset;		/* offset of input dltf entry to be read*/
	double	minn,maxx;	/* Working min/max of a LUT; used to
				      scale a non-byte LUT for DMS.	*/
	double	rndval = 0.5;	/* rounding value for slope and offset
				   calculations, see comment above	*/

	int one = 1;            /* value of one for errmsg and dltf type*/
	int two = 2;            /* Specifies I*2 (2 byte) data.		*/
 	int zero = 0;           /* value of zero for support calls	*/
        
	long	i,j,k, 		/* Loop indices. 			*/
		inter,		/* Intermediate value variable		*/
		rstrt,		/* LUT starting point after being scaled
				      using the slope and offset of the
				      displayed image.			*/
		rend,		/* LUT ending point after being scaled
				      using the slope and offset of the
				      displayed image.			*/
		target,		/* Target LUT entry to scale.		*/
		utarget,	/* LUT target entry to scale.		*/
		lutindx;	/* Index into the LUT data.		*/
      	long 	endpt;		/* end point of dltf data record	*/
	long 	index = 0;	/* index for converting dltf data types */
	long 	lutsiz; 	/* size of dltf, mx - mn		*/
	long 	maxfreq; 	/* size of dltf, mx - mn		*/
      	long 	startpt;	/* start point of dltf data record	*/
      	long 	status;		/* return status of support calls	*/

      	short 	*ptr;		/* temp pointer to convert dltf buffer  */
	short	*shrtptr;	/* Working I*2 pointer into the
			           lutdata buffer.			*/
	short	*sptr;	   	/* I2 pointer into the lut buffer    	*/
	short	*sfptr;	   	/* Twin pointer to sptr; use to keep the
			      	   original value of sptr when it is
			           assigned to lut.		 	*/

      	unsigned char *lutdata;	/* temp pointer to convert dltf types	*/
/************************************************************************/

if (strcmp(indtype,"B") == 0)
   {
   status = c_r2dltf(ldesc,inkey,bnd,&startpt,&endpt,indtlen,
		     blut,&slope,&offset);
   if (status != E_SUCC)
      return(E_FAIL);

   if (((startpt - offset) / slope) < 0)
      startpt = (long)(((double)startpt - offset)/slope - rndval);
   else
      startpt = (long)(((double)startpt - offset)/slope + rndval);
   *instpt = startpt;

   if (((endpt - offset) / slope) < 0)
      endpt = (long)(((double)endpt - offset)/slope - rndval);
   else
      endpt = (long)(((double)endpt - offset)/slope + rndval);
   *inenpt = endpt;

   if (*ltype != 1) 				/* I*2 input image, I*2 lut   */
      { 					/* needed, scale byte to I*2  */

      if ((slope == 1.0) && (offset == 0.0))	/* Calculate slope offset else*/
         {					/*use dltf's, they're from dms*/
    	 if ((*mx - *mn) <= (MAXUTINY - MINUTINY))
	    {
	    lslope = (MAXUTINY - MINUTINY)/(*mx - *mn);
	    loffset = MINUTINY - (lslope * (*mn));
	    }
    	 else
	    {
	    lslope = (MAXUTINY - MINUTINY)/(*mx - *mn);
	    loffset = MINUTINY - (lslope * (*mn));
	    }
         }
      else
         {
         lslope = slope;
	 loffset = offset;
         }

/* fill in bottom of lut from MINWORD up to mn */

      maxfreq = (long)(*mx - *mn) + 1;
      if (maxfreq > LUTSIZ)
	 lutsiz = maxfreq;
      else
	 lutsiz = LUTSIZ;
      sptr = sfptr = lut;
      for (i = MINWORD,j = 0; i < (startpt); i++,sfptr++)
         *sfptr = (short)startpt;


/* fill lut from mn to mx using linear interpolation */

      for (j = 0; j < (LUTSIZ - 1); j++)
	 {
	 if (((j - loffset) / lslope) < 0)
	    inter = (long)((j - loffset) / lslope - rndval);
	 else
	    inter = (long)((j - loffset) / lslope + rndval);
	 lowx = (double)inter;
	 if ((((j + 1) - loffset) / lslope) < 0)
	    inter = (long)(((j + 1) - loffset) / lslope - rndval);
	 else
	    inter = (long)(((j + 1) - loffset) / lslope + rndval);
	 highx = (double)inter;
	 lowy = (double)(blut[j]);
	 highy = (double)(blut[j  + 1]);
	 for (k = (long)lowx; k < (long)highx; k++,sfptr++)
	    {
	    intr = (lowy + ((highy - lowy) * (k - lowx)/(highx - lowx)));
	    if (((intr - loffset) / lslope) < 0)
	       *sfptr = (short)((intr - loffset) / lslope - rndval);
	    else
	       *sfptr = (short)((intr - loffset) / lslope + rndval);
	    index = index + 1;
	    }
	 if ((j + 1) == (LUTSIZ - 1))
	    {
	    if (((highy - loffset) / lslope) < 0)
	       *sfptr = (short)((highy - loffset) / lslope - rndval);
	    else
	       *sfptr = (short)((highy - loffset) / lslope + rndval);
	    sfptr++;
	    }
	 }

/* fill in top of lut from mx up to MAXWORD */

      for (i = (endpt); i < MAXWORD; i++,sfptr++)
         *sfptr = (short)endpt;
      }
   }
else	/* indtype equal to I2 */
   {
   if ((lutdata = (unsigned char *)malloc(65536 * 2)) == 0)
	 {
	 c_errmsg("Error allocating buffer space for DLTF record",
		  "map-alloc",(long *)&one);
         return(E_FAIL);
         }
   status = c_r2dltf(ldesc,inkey,bnd,&startpt,&endpt,indtlen,
   		     lutdata,&dslope,&doffset);
   if (status != E_SUCC)
      return(E_FAIL);

   if (((startpt - offset) / dslope) < 0)
      *instpt = (long)(((double)startpt - offset)/dslope - rndval);
   else
      *instpt = (long)(((double)startpt - offset)/dslope + rndval);

   if (((endpt - offset) / dslope) < 0)
      *inenpt = (long)(((double)endpt - offset)/dslope - rndval);
   else
      *inenpt = (long)(((double)endpt - offset)/dslope + rndval);

   /* lut entries less than startpt are set equal to startpt,
      lut entries greater than endpt are set equal to endpt,
      a full range lookup table is needed for applut call to pxlkup */

   if (*ltype == 2)
      {
      for (i = MINWORD,j = 0; i < startpt; i++,j++)
         lut[j] = (short)startpt;
      ptr = (short *)lutdata;
      for (i = startpt; i < endpt; i++,j++,ptr++)
         lut[j] = *ptr;
      for (i = endpt; i < MAXWORD; i++,j++)
         lut[j] = (short)endpt;
      }
   /* output image is byte, need to write byte lut, scale from I*2 to byte */
   if (*otype == 1)
      {
			/* Find the min/max of the LUT data.		*/

      minn = 32767;
      maxx = -32768;

      shrtptr = (short *)lutdata;

      for (k = startpt; k < endpt; k++, shrtptr++)
        if (*shrtptr < minn)
	      minn = (double)*shrtptr;
        else if (*shrtptr > maxx)
	      maxx = (double)*shrtptr;

			/* Calculate the slope and offset of the LUT.	*/

      status = dtcvrt(&two,&one,&minn,&maxx,&lslope,
		      &loffset,&zero);

      if (status != E_SUCC)
	      return(status);

			/* Apply the slope and offset
			   to the range limits of the LUT.		*/

      if (((startpt * lslope) + loffset) < 0)
         rstrt = (long)((startpt * lslope) + loffset - rndval);
      else
         rstrt = (long)((startpt * lslope) + loffset + rndval);

      if (((endpt * lslope) + loffset) < 0)
         rend  = (long)((endpt   * lslope) + loffset - rndval);
      else
         rend  = (long)((endpt   * lslope) + loffset + rndval);

			/* Scale the LUT data.				*/

      if ((rstrt < LUTSIZ) && (rend > 0))
        {
			    /* Clamp the range values if necessary.	*/

          if (rstrt < 0)
	      rstrt = 0;

          if (rend > 255)
	      rend = 255;

          for (j = 0; j < LUTSIZ; j++)
             {
	      target = j;
	      shrtptr = (short *)lutdata;

				/* Fill in range areas where necessary.	*/

	      if (j <= rstrt)
	         target = rstrt;

	      if (j >= rend)
	         target = rend;

				/* Find the target entry in the I*2 LUT.*/

	      if (((target - loffset) / lslope) < 0)
	         utarget = (long)((target - loffset) / lslope - rndval);
	      else
	         utarget = (long)((target - loffset) / lslope + rndval);

				/* Calculate the index necessary for the LUT
				   data.				*/

	      lutindx = (utarget - startpt);

				/* Increment the LUT pointer.		*/

	      shrtptr += lutindx;

				/* Scale the LUT data entry.		*/

	      if (((*shrtptr * lslope) + loffset) < 0)
	      blut[j] = (unsigned char)((*shrtptr * lslope) + loffset - rndval);
	      else
	      blut[j] = (unsigned char)((*shrtptr * lslope) + loffset + rndval);
             }
        }
      else
        {
          c_errmsg("DLTF data outside input image range","map-lutdata",
                   (long *)&one);
          return(E_FAIL);
        }
      }
   }

return(E_SUCC);
}


int rddltf_
(
    long *otype,
    long *ltype,
    long *ldesc,
    char *inkey,
    long *bnd,
    char *indtype,
    long *indtlen,
    unsigned char *blut,
    short *lut,    
    double *mn,
    double *mx,
    long *instpt,
    long *inenpt        
)
{
	long status;

	status = rddltf(otype,ltype,ldesc,inkey,bnd,indtype,indtlen,
			blut,lut,mn,mx,instpt,inenpt);
	return (status);
}
