/****************************************************************************
NAME:				mappts

PURPOSE:  A ground control point collection module which creates a tie
	  point selection file consisting of latitude, longitude
	  coordinates from a map.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		5/89	D. Steinwand	   CSB      Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize LAS processing
	Get parameters from TAE
	If an input tie point selection file, initialize values in the 
		header of the tie point selection file
	Else open the input TPS file
	Register the first map
	Initialize TPS file for output
	Open the output TPS file
	If an input TPS file, copy records to the output TPS file, checking
	   pt_id's along the way
	Get coodinates from the digitizer until done
	   Get point
	   If button is FAIL, report error and abort
	   If button is QUIT, break out of loop
	   If button is NEW_MAP, register a new map
	   If button is SELECT, 
	 	Get point id
		Get elevation
		Write to file
	Done.  Close file
	Write the standard completion message & exit
*****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "digitizer.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK vblk;		/* TAE variable block */
struct TPSHEAD header;		/* TPS header record */
struct TPSDATA tpsdata;		/* TPS data records */
long std_out;			/* Standard output */
long stat;			/* Function return status */
long one = 1;			/* Constant of one */
long elev_flag;			/* Elevation input flag (boolean) */
long pt_id_flag;		/* Point id input flag (boolean) */
long button;			/* Button value returned from digitizer */
long acc;			/* File access mode */
long ptyp = TPS;		/* Type of type point file */
long fatal = -1;		/* Fatal error flag for errmsg */
long i;				/* Loop counter */
long prompt_flag = TRUE;	/* Prompt flag for c_digpt */
long num_img;			/* Number of output files */
struct PTDESC * in_desc;		/* Pointer to tiepoint IO records (input)*/
struct PTDESC * out_desc;		/* Pointer to tiepoint IO records (output) */
char mapid[CMLEN];		/* Map id into map identifier file */
char in_tps[CMLEN];		/* Input tie point selection file */
char out_tps[CMLEN];		/* Output tie point selection file */
char mapfile[CMLEN];		/* Map identifier file name (TAE) */
char outtae[2][CMLEN];		/* Output file names (TAE) */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get parameters from the user
  ----------------------------*/
stat = getpar(&vblk, in_tps, out_tps, mapfile, &elev_flag, &pt_id_flag);
if (stat != E_SUCC) c_errmsg("Fatal error encountered","mappts-fatal",&fatal);

/* Register first map.  This will determine some values to be written to the
   header of the tie point selection file.
  ---------------------------------------*/
if (strcmp(in_tps,"\0") == 0) 
   {
   strcpy(header.units, " ");
   header.pdist[0] = header.pdist[1] = 0.0;
   for (i = 0; i < 15; i++)
      header.projprms[i] = 0.0;
   for (i = 0; i < 8; i++)
      header.corners[i] = 0.0;
   header.proj_valid = GEO_VALID;
   header.code = GEO;
   header.zone = 62;
   header.datum = 0;
   strcpy(header.mode,"GEO");
   strcpy(header.name,"Points were obtained from paper maps (or similar).");
   }
else
   {
   acc = IREAD;
   if(c_ptopen(&in_desc, in_tps, &ptyp, &acc, &header) != E_SUCC)
	c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
   }
stat = c_regmap(mapfile, mapid, header.units);
if (stat == E_FAIL) c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
if (stat == E_DONE) c_errmsg("Program aborted by user","mappts-quit",&fatal);

/* Initialize file for output
  --------------------------*/
if((strcmp(header.units,"meters")==0) || (strcmp(header.units,"feet")==0) ||
   (strcmp(header.units,"user_def")==0))
   {
   strcpy(header.mode,"USR");
   header.proj_valid = GEO_NOTVALID;
   }
else if (header.proj_valid == GEO_NOTVALID)
   {
   header.pdist[0] = header.pdist[1] = 0.0;
   for (i = 0; i < 15; i++)
      header.projprms[i] = 0.0;
   for (i = 0; i < 8; i++)
      header.corners[i] = 0.0;
   header.proj_valid = GEO_VALID;
   header.code = GEO;
   header.zone = 62;
   header.datum = 0;
   strcpy(header.mode,"GEO");
   strcpy(header.name,"Points were obtained from paper maps (or similar).");
   }
acc = IWRITE;
if(c_ptopen(&out_desc, out_tps, &ptyp, &acc, &header) != E_SUCC)
	c_errmsg("Fatal error encountered","mappts-fatal",&fatal);

/* If an input tie point selection file was given, copy records from that
   file into the output file.  Check for unique point id's in the process
  ----------------------------------------------------------------------*/
if (strcmp(in_tps,"\0") != 0) 
   for (;;)
	{
   	stat = c_ptio(&in_desc, &tpsdata);
   	if (stat == E_EOF) break;
   	if (stat == E_FAIL)
	   c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
        if (c_ckptid(tpsdata.pt_id))
	   {
   	   if (c_ptio(&out_desc, &tpsdata) != E_SUCC)
	       c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
	   }
	else 
	   {
	   c_errmsg("Duplicate point identifier found in input file",
			"mappts-duppt", &one);
	   c_errmsg("Point identifers must be unique!","mappts-duppt",&one);
	   c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
    	   }
  	}

/* Get coordinates from the digitizer
  ----------------------------------*/
for(;;)
   {
   button = c_digpt(tpsdata.coord, &prompt_flag);
   if(button == E_FAIL)
	c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
   if(button == QUIT_DIG) break;
   if(button == NEW_MAP)
	{
	stat = c_regmap(mapfile, mapid, header.units);
	if (stat == E_FAIL) 
		c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
	if (stat == E_DONE) break;
	}
   else if(button == SELECT_PT)
	{
   	get_pt_id(pt_id_flag, tpsdata.pt_id);
	tpsdata.elevation = get_elevation(elev_flag);
	strcpy(tpsdata.chip_name, mapid);
	if (c_ptio(&out_desc, &tpsdata) != E_SUCC)
		c_errmsg("Fatal error encountered","mappts-fatal",&fatal);
	}
   else
	c_errmsg("Non-digitized input detected.  Re-select point"," ",&one);
   }

c_ptclse(&out_desc);

/* Write completion message and exit
  ---------------------------------*/
strcpy(&(outtae[0][0]), out_tps);
strcpy(&(outtae[1][0]), mapfile);
num_img = 2;
if (strcmp(mapfile, "\0") == 0) num_img = 1;
c_complt("MAPPTS", outtae, &num_img);

return EXIT_SUCCESS;

}
