
/*******************************************************************************
NAME:     image.c 
PURPOSE:  Simplify image IO for application developers by providing a data
          structure driven image IO implementation.  Multiple processing modes
          are supported, no global variables are required, full LAS
          functionality is assured.  Application developers may select some of
          these IO routines;  a few applications require simple revisions to
          these IO routines.

SOURCE CODE HISTORY
--------------------------------------------------------------------------------
  Date       Programmer                          Changes 
---------  ---------------  ----------------------------------------------------
 Dec 1992  David Zokaites   Developed imageio application:  functions and data
                            structures for sequential IO.
 Feb 1993  David Zokaites   Enabled random IO by writing openrin(), openrout(),
			    rread(), rwrite(), rimage();  created global vars
			    for easier error messages;  wrote minmax().
 May 1993  David Zokaites   Enabled block IO by writing bread(), bwrite(),
                            bimage();  wrote make_ddr().
 Jul 1993  David Zokaites   Renamed application from imageio to template;
			    eliminated functions getin2() and getout2();  wrote
			    help for template.
 Aug 1993  David Zokaites   Allowed for data conversion upon read and write;
                            wrote convert[srb]().
 May 1994  David Zokaites   Wrote openbin(), openbout(), minmax[srb](),
			    parse_file();  expanded help in template.pdf;
			    removed make.com;  added tae parms iomode,
			    readtype, procmeth, writconv.
 Aug 1994  David Zokaites   Prototyped all las support functions via
			    makep.{c,csh};  changed variable names to keep with
			    las conventions;  put errclean() in a separate file
			    to minimize global vars;  wrote Lintfile to run
			    lint;  eliminated functions real_parm(),
			    string_parm(), integer_parm(), parse_file();
			    renamed global data structures for las images;
			    wrote functions open[sr]out2().
 Nov 1994  David Zokaites   Improved error messages;  renamed ddr_his() to
                            finish_outi2().
 Jan 1995  David Zokaites   Enabled errclean() to process a variable number of
			    arguments;  wrote function errmsg();  (errmsg later
			    became c_lasmsg which later became a support);
			    reduced # of global vars, deleted begin_las();
			    reconstructed program history;  updated prototypes
			    of las support functions.
 Feb 1995  David Zokaites   Updated Makefile format.
 Jun 1995  David Zokaites   Tweaked error messages, called c_lasmsg() instead 
                            of c_errmsg() or errmsg(). 
 Jul 1995  David Zokaites   For fatal errors, errclean() can now be called from 
			    any function;  added status flags to image data
			    structures;  pblock is no longer global;
			    open[sr]out() now just call open[sr]out2();  changed
			    parms to open[sr]out2();  changed indenting per las
			    conventions;  renamed process.c to proces.c;
			    deleted c_lasmsg.c as this function is now a
			    support;  changed data structure and function names;
			    wrote read_bil() and write_bil(). 
 Aug 1995  David Zokaites   Defined macros for image data sizes;  used random
			    supports instead of sequential for BIL processing;
			    combined functions to open and close images;
			    deleted tae parm writconv;  simplified error 
			    messages to keep with las convention;  changed 
			    indentation per las convention;  replaced
			    if "c_function()" with if "c_function()!=E_SUCC".
 Sep 1995  David Zokaites   Deleted functions not needed by MSSCOLOR 
*******************************************************************************/

#include <math.h>        /* for call to ceil() */ 
#include <string.h>      /* for call to strcmp() */ 
#include <stdlib.h>      /* for call to malloc(), free() */ 
#include "msscolor.h"    /* application header file */ 
#include "pixman.h"      /* for TOOBIG, TOOSMALL macros */ 
#include "util.h"


/*******************************************************************************
NAME:     open_bil_in 
PURPOSE:  open an input image for BIL IO and allocate data buffers
          Use this function instead of the more general open_inimage() 
*******************************************************************************/
void open_bil_in 
(
    struct PARBLK *pblock, /* input:  tae parameter block */ 
    char *parmname,        /* input:  tae parm, name of image to open */ 
    long dtypeap,          /* input:  convert image data to this type upon
			     reading.  pass NOTAPPLY if not converting*/
    struct INIMAGE *inimage /* output: opened input image */ 
)
{
    /* call open_inimage to do all the work */ 
    open_inimage (pblock, parmname, dtypeap, BILIO, inimage); 
}

/*******************************************************************************
NAME:     open_inimage 
PURPOSE:  general function to open an input image and allocate data buffers 
*******************************************************************************/

void open_inimage 
(
    struct PARBLK *pblock, /* input:  tae parameter block */ 
    char *parmname,        /* input:  tae parameter, name of image to open */ 
    long dtypeap,          /* input:  convert image data to this type upon
			             reading.  pass NOTAPPLY if not converting*/
    long iomode,           /* input:  mode of IO. = BILIO, BSQIO  */ 
    struct INIMAGE *inimage /* output: opened input image */ 
)
{

    /* declare variables */ 
    long accessm=IREAD,    /* image access mode, parm to c_eopenr() */ 
    bandnum,          /* loop index, band number */ 
    imagenum,         /* loop index, image number */ 
    maximg=MAXIMG,    /* max number of images allowed */ 
    one=1,            /* value of one */ 
    qflag[MAXIMG][2], /* flag, do image components have a band or
                         window specified */ 
    samewindow,       /* flag, do image components have the same 
                         window specified? */ 
    tband,            /* loop index, total band number */ 
    zero=0;           /* value of zero */ 

    /* set misc vars */ 
    inimage->iomode  = iomode;   
    inimage->dtypeap = dtypeap; 

    /* parse input image spec */ 
    /* call c_getin() */ 
    if (c_getin (pblock, parmname, &zero, inimage->taename, inimage->hostname,
                &inimage->nmrimg, inimage->window, inimage->bands, 
                inimage->nbands, &inimage->totalbands, &maximg, &samewindow, 
                qflag) !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 
    /* error check:  different window sizes */ 
    if (!samewindow)
        errclean ("msscolor-window", 
                "Images specified by %s do not have the same window size", 
                parmname);
    /* set nl, ns */ 
    inimage->nl = inimage->window[0][NL]; 
    inimage->ns = inimage->window[0][NS]; 

    /* update last used fields of input images */ 
    for (imagenum=0;  imagenum<inimage->nmrimg;  imagenum++)
        (void) c_lused (inimage->hostname[imagenum]); 

    /* open image files */ 
    for (imagenum=0;  imagenum<inimage->nmrimg;  imagenum++)
        if (c_eopenr (inimage->fdarray+imagenum, inimage->hostname[imagenum], 
                    &inimage->nbands[imagenum], &inimage->window[imagenum][SL], 
                    &inimage->window[imagenum][SS], 
                    &inimage->window[imagenum][NL], 
                    &inimage->window[imagenum][NS], 
                    &accessm, &inimage->dtypeap, &zero, &one) !=E_SUCC) 
            errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* set data size */ 
    switch (inimage->dtypeap)
    {
        case EBYTE:  inimage->dsizeap=EBYTESIZE;  break; 
        case EWORD:  inimage->dsizeap=EWORDSIZE;  break; 
        case ELONG:  inimage->dsizeap=ELONGSIZE;  break; 
        case EREAL:  inimage->dsizeap=EREALSIZE;  break; 
        default:  errclean ("msscolor-fatal", 
                          "Program error:  unknown image data type");
    }

    /* get input data type.  
       get datatype of 1st disk image, supporting routines convert other images
       to data type of first image */ 
    if (c_gettyp (inimage->hostname, &one, &inimage->dtypein) !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* build luts for later access by functions to read image data.  Due to 
       subbanding, the parm eband to c_eread() is relative to the bands in the 
       input -- not output -- image files.  This is accommodated by cblut. */
    for (tband=imagenum=0;  imagenum<inimage->nmrimg;  imagenum++)
    {
        for (bandnum=0;  bandnum<inimage->nbands[imagenum];  bandnum++, tband++)
        {
            inimage->cblut[tband] = inimage->bands[imagenum][bandnum];
            inimage->fdlut[tband] = imagenum; 
        } 
    }

    /* allocate memory for image buffers */ 
    switch (inimage->iomode)
    {
        case BSQIO:  
            /* buffer holds one line for one band */ 
            if (( inimage->buffer = (unsigned char *) malloc 
                        (inimage->ns * inimage->dsizeap) ) == NULL)
                errclean ("msscolor-memory", 
                        "Unable to allocate memory for input image buffer"); 
            break; 

        case BILIO:  
            /* linebuf holds one line for each band */ 
            for (bandnum=0;  bandnum<inimage->totalbands;  bandnum++)
                if (( inimage->linebuf[bandnum] = (unsigned char *) malloc 
                            (inimage->ns * inimage->dsizeap) ) == NULL)
                    errclean ("msscolor-memory", 
                            "Unable to allocate memory for input image buffer"); 
            break; 

        default:  
            errclean ("msscolor-fatal", 
                    "Program error:  iomode has a bad value in open_in()"); 
    }

    /* set flag, image was opened */ 
    inimage->opened = TRUE; 
}








/*******************************************************************************
NAME:     open_outimage
PURPOSE:  general function to open an output image and allocate data buffers,
          does not rely on an input image 
*******************************************************************************/

void open_outimage 
(
    struct PARBLK *pblock,   /* input:  tae parameter block */ 
    char *parmname,          /* input:  tae parm, name of image to open */ 
    long dtypein,   /* input:  data type of input image.  pass NOTAPPLY if there
		      is no input image */ 
    long dtypeap,   /* input:  output image data type as processed by the app, 
		      pass NOTAPPLY if the app handles data type conversion and 
		      conversion does not occur upon writing to disk */ 
    long dtypeout,  /* input:  image data type as output to disk image.  pass
		      NOTAPPLY if dtypeout should be based on parm odtype */ 
    long iomode,    /* input:  mode of io. = BSQIO, BILIO */ 
    long nl,                   /* input:  nl for output image */ 
    long ns,                   /* input:  ns for output image */ 
    long nbands,               /* input:  nbands for output image */ 
    struct OUTIMAGE *outimage  /* output: opened output image */
)
{
    /* declare variables */ 
    char *odtype;            /* tae parm odtype, output data type */ 
    long accessm=IWRITE,     /* image access mode, parm to c_eopenr() */ 
    bandnum,            /* loop index, image band */ 
    one=1,              /* value of one */ 
    opt[2];             /* optional parm to c_eopenr(), used for data 
                           type conversion */ 
    struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 

    /* get tae globals:  $delflg, $minmax, $msgtim */ 
    /* delflg */ 
    tparm = p_find (pblock, "$delflg");   
    if ( !strcmp (SVAL(*tparm,0), "YES"))
        outimage->delflg = TRUE;
    else
        outimage->delflg = FALSE;
    /* minmax */ 
    tparm = p_find (pblock, "$minmax");   
    if ( !strcmp (SVAL(*tparm,0), "YES"))
        outimage->minmax = TRUE;
    else 
        outimage->minmax = FALSE;
    /* msgtim */ 
    tparm = p_find (pblock, "$msgtim");   
    outimage->msgtim = IVAL (*tparm, 0); 

    /* get output image name */ 
    if (c_getout (pblock, parmname, &outimage->taename[0], 
                  &outimage->hostname[0]) !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* set dtypeout, output data type */ 
    if (dtypeout == NOTAPPLY)
    {
        /* get tae parm odtype */ 
        tparm = p_find (pblock, "odtype");    
        odtype = SVAL(*tparm, 0); 
        /* error check:  odtype=same and dtypein==notapply */ 
        if (!strcmp (odtype, "SAME") && dtypein == NOTAPPLY)
            errclean ("msscolor-fatal", 
                    "Program error:  ODTYPE=SAME but dtypein=NOTAPPLY"); 
        /* set dtypeout */ 
        if      (!strcmp (odtype, "SAME")) outimage->dtypeout = dtypein; 
        else if (!strcmp (odtype, "BYTE")) outimage->dtypeout = EBYTE; 
        else if (!strcmp (odtype, "I*2"))  outimage->dtypeout = EWORD; 
        else if (!strcmp (odtype, "I*4"))  outimage->dtypeout = ELONG; 
        else if (!strcmp (odtype, "R*4"))  outimage->dtypeout = EREAL; 
        else errclean ("msscolor-fatal", 
                "Program error:  parm ODTYPE has an illegal value"); 
    } 
    else
        outimage->dtypeout = dtypeout; 

    /* set dtypeap, data type as processed by the application */ 
    if (dtypeap == NOTAPPLY)
        outimage->dtypeap = outimage->dtypeout;
    else
        outimage->dtypeap = dtypeap; 

    /* set dsizeap, data size as processed by the application */ 
    switch (outimage->dtypeap)
    {
        case EBYTE:  outimage->dsizeap=EBYTESIZE;  break; 
        case EWORD:  outimage->dsizeap=EWORDSIZE;  break; 
        case ELONG:  outimage->dsizeap=ELONGSIZE;  break; 
        case EREAL:  outimage->dsizeap=EREALSIZE;  break; 
        default:  errclean ("msscolor-fatal", 
                          "Program error:  unknown image data type");
    }

    /* set variables in data structure */ 
    outimage->nl = nl; 
    outimage->ns = ns; 
    outimage->totalbands = nbands; 
    outimage->iomode = iomode; 

    /* open image files after setting opt to allow for data conversion on
       output */ 
    opt[0] = opt[1] = 0;  /* initialize array */ 
    if (outimage->dtypeap != outimage->dtypeout)
        opt[0] = outimage->dtypeout; 

    if (c_eopenr (outimage->fdarray, outimage->hostname[0], 
                &outimage->totalbands, &one, &one, &outimage->nl, &outimage->ns,
                &accessm, &outimage->dtypeap, opt, &one) !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* allocate memory for image buffers */ 
    switch (outimage->iomode)
    {
        case BSQIO:  
            /* buffer holds one line for one band */ 
            if (( outimage->buffer = (unsigned char *) malloc 
                        (outimage->ns * outimage->dsizeap) ) == NULL)
                errclean ("msscolor-memory", 
                        "Unable to allocate memory for input image buffer"); 
            break; 

        case BILIO:  
            /* linebuf holds one line for each band */ 
            for (bandnum=0;  bandnum<outimage->totalbands;  bandnum++)
                if (( outimage->linebuf[bandnum] = (unsigned char *) malloc 
                            (outimage->ns * outimage->dsizeap) ) == NULL)
                    errclean ("msscolor-memory", 
                            "Unable to allocate memory for input image buffer"); 
            break; 

        default:  
            errclean ("msscolor-fatal", 
                    "Program error:  iomode has a bad value in open_in()"); 
    }

    /* initialize min max arrays */ 
    if (outimage->minmax && 
            c_inmnmx (&outimage->dtypeap, &outimage->totalbands, 
                outimage->minval, outimage->maxval) !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* set flag, image was opened */ 
    outimage->opened = TRUE; 
}


/*******************************************************************************
NAME:     read_bil 
PURPOSE:  read data from input image files, using BIL IO  
*******************************************************************************/

void read_bil 
(
    long curline,            /* input:  line of data to read, 0 relative */ 
    struct INIMAGE *inimage  /* output: new image data */ 
)
{
    /* declare variables */ 
    long bandnum,    /* loop index, image band */ 
    curband,    /* current band relative to c_eread() */ 
    imagenum,   /* image number, index to fdarray[] */ 
    one=1;      /* n image lines to read at a time */ 

    /* convert line from 0 relative to 1 relative */ 
    curline++; 

    /* loop for all image bands */ 
    for (bandnum=0;  bandnum<inimage->totalbands;  bandnum++)
    {

        /* determine which image file to access then translate from 
           bandnum to curband relative to input image file */ 
        imagenum = inimage->fdlut[bandnum]; 
        curband  = inimage->cblut[bandnum]; 

        /* read data from disk image */ 
        if (c_eread (inimage->fdarray+imagenum, &curband, &curline, 
                    inimage->linebuf[bandnum], &one) != inimage->ns)
            errclean ("msscolor-fatal", "Fatal error encountered"); 

    }
}

/*******************************************************************************
NAME:     write_bil 
PURPOSE:  write data to disk image, using BIL IO  
*******************************************************************************/

void write_bil 
(
    long curline,           /* input:  starting line of block of data to write, 
			              0 relative */ 
    struct OUTIMAGE *outimage   /* input:  data structure of image to write */
)
{
    /* declare variable */ 
    long bandnum,  /* loop index, image band */ 
    curband,  /* current band relative to c_ewrite() */ 
    one=1;    /* n lines to write at a time */ 

    /* convert curline from 0 relative to 1 relative */ 
    curline++; 

    /* loop for all bands */ 
    for (bandnum=0;  bandnum<outimage->totalbands;  bandnum++)
    {

        /* convert bandnum from 0 relative to 1 relative */ 
        curband = bandnum + 1; 

        /* write data to disk */ 
        if (c_ewrite (outimage->fdarray, &curband, &curline, 
                    outimage->linebuf[bandnum], &one) != outimage->ns)
            errclean ("msscolor-fatal", "Fatal error encountered"); 
    }
}

/*******************************************************************************
NAME:     minmax_bil 
PURPOSE:  optionally update minmax for a line of image data for BIL IO
*******************************************************************************/

void minmax_bil 
(
    struct OUTIMAGE *outimage /* input and output:  structure of image data to 
			     update min/max for */ 
)
{
    /* declare variable */ 
    long band;              /* loop index to image bands */ 

    /* return if min/max does not need updated */ 
    if (!outimage->minmax)
        return; 

    /* update minmax */ 
    for (band=0;  band<outimage->totalbands;  band++)
        if (c_pxmnmx (outimage->linebuf[band], &outimage->ns, 
                    outimage->minval+band, outimage->maxval+band, 
                    &outimage->dtypeap) !=E_SUCC)
            errclean ("msscolor-fatal", "Fatal error encountered"); 
}

/*******************************************************************************
NAME:     close_inimage 
PURPOSE:  close input image files and free buffers  
*******************************************************************************/

void close_inimage
(
    struct INIMAGE *inimage  /* input:  close these files */ 
)
{
    /* declare variables */
    long bandnum,   /* loop index, band number */ 
    imagenum;  /* loop index, image number */ 

    /* close image files, close individual file descriptors */ 
    for (imagenum=0;  imagenum<inimage->nmrimg;  imagenum++)
        (void) c_eclose (inimage->fdarray+imagenum);

    /* free buffers */ 
    switch (inimage->iomode)
    {
        case BSQIO:    free (inimage->buffer); 
                       break;  
        case BILIO:    for (bandnum=0;  bandnum<inimage->totalbands;  bandnum++)
                           free (inimage->linebuf[bandnum]); 
                       break;
    }

    /* set status flag, image was closed */ 
    inimage->closed = TRUE; 
}

/*******************************************************************************
NAME:     close_outimage 
PURPOSE:  close output image files and free buffers  
*******************************************************************************/

void close_outimage
(
    struct OUTIMAGE *outimage    /* input:  close these files */ 
)
{
    /* declare variable */ 
    long bandnum;  /* loop index, image band */ 

    /* close image file, close file descriptor */ 
    (void) c_eclose (outimage->fdarray);

    /* free buffers */ 
    switch (outimage->iomode)
    {
        case BSQIO:    free (outimage->buffer); 
                       break;  
        case BILIO:    for (bandnum=0; bandnum<outimage->totalbands; bandnum++)
                           free (outimage->linebuf[bandnum]); 
                       break;
    }

    /* set status flag, image was closed */ 
    outimage->closed = TRUE; 
}

/*******************************************************************************
NAME:     finish_out 
PURPOSE:  finalize processing of output image:  create ddr and history file.  
	  If there are input images, call finish_out() instead of 
	  finish_out2().  The former puts valid projection information into 
	  the image ddr while the latter does not.

	  cflag was changed from COMB to NOCOMB for msscolor 
*******************************************************************************/

void finish_out
(
    struct PARBLK *pblock,    /* input:  tae parameter block */ 
    struct INIMAGE *inimage,  /* input:  image input to the application, used 
                                         to create ddr for output image */ 
    struct OUTIMAGE *outimage /* input:  image output from the application */ 
)
{
    /* variables for ddr */ 
    long cflag=COMB;   /* combine flag for c_upddr() */ 
    double one=1.0;    /* one as a double */ 

    /* create ddr for output image.  this call updates last used fields for
       input images;  these fields are also updated in the functions to open
       images */ 
    if (c_upddr (pblock, inimage->hostname, outimage->hostname[0], 
                &inimage->nmrimg, inimage->window, inimage->bands, 
                inimage->nbands, &one, &one, &outimage->totalbands, &cflag) 
            !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* update ddr with min max, must come after writing ddr to disk */ 
    if (outimage->minmax && 
        c_svmnmx (outimage->hostname[0], &outimage->totalbands, 
                  outimage->minval, outimage->maxval, "YES") !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* create history file */ 
    if (c_hstout (pblock, outimage->hostname[0], inimage->hostname, 
                &inimage->nmrimg, "") !=E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 
}



