
/*******************************************************************************
NAME:     image.h
PURPOSE:  header file for image IO 
	  includes data structures for LAS images, 
	  symbolic constants and function prototypes
*******************************************************************************/


/* macro:  this function parameter does not apply.  must be 0 for the supports*/
#define NOTAPPLY 0 


/* define macros for image data sizes, macros really belong in worgen.h */
#define EBYTESIZE (sizeof(unsigned char))
#define EWORDSIZE (sizeof(short))
#define ELONGSIZE (sizeof(long))
#define EREALSIZE (sizeof(float))



/* types of image IO allowed for, they all use LAS random IO.  random supports
   seem to work a little better than sequential supports */ 
enum iomodes 
{
    BSQIO,   /* BSQ processing */ 
    BILIO    /* BIL processing */ 
};



/* a note on data types: 
   An image can potentially have 3 different data types:
   dtypein:  data type as exists in input disk image
   dtypeap:  data type as processed internally by the application
   dtypeout:  data type as will exist in output disk image.  Data types
   can be EBYTE, EWORD, ELONG, EREAL. */ 



/* data structure for one LAS input image, used to facilitate IO.  Because LAS
   image specs allow for grouping image components, (e.g. in="image1 + image2")
   one input image may correspond to multiple disk image files.  The same
   structure is used for all types of IO, although all structure members are 
   not always used. */
struct INIMAGE
{
    char hostname[MAXIMG][CMLEN],/* host name for image files */ 
    taename[MAXIMG][CMLEN]; /* tae name for image files */ 
    struct FDESC* fdarray[MAXIMG+1]; /* file descriptor array */ 
    long bands[MAXIMG][MAXBND+1],/* band numbers for each image file */ 
    cblut[MAXIMG*MAXBND],   /* lut for current band */ 
    dsizeap,                /* # bytes per pixel as seen by the app */
    dtypeap,                /* image data type as seen by the app */ 
    dtypein,                /* image data type as read = EBYTE, ... */
    fdlut[MAXIMG*MAXBND],   /* lut for file descriptor */
    iomode,                 /* mode of io, = BILIO, BSQIO */ 
    nbands[MAXIMG],         /* number of bands in each image file */ 
    nmrimg,                 /* number of disk image files */ 
    nl, ns,                 /* size of input image = window[][NL], 
                               window[][NS] */ 
    totalbands,             /* total # of bands for all image files */
    window[MAXIMG][4];      /* window (sl ss nl ns) for each image file*/
    unsigned char 
        *buffer,            /* holds one image line for BSQIO */ 
    *linebuf[MAXBND];       /* holds one line for each band. for BILIO */
    /* status true/false flags used for error handling */ 
    long closed,                 /* has the image been closed */ 
    opened;                 /* has the image been opened */
}; 



/* data structure for one las output image, used to facilitate IO, 
   structure includes some tae globals */ 
struct OUTIMAGE
{
    char hostname[1][CMLEN],/* host name for image file, supporting
                               routines expect a 2-d array */ 
    taename[1][CMLEN];      /* tae name for image, supporting 
                               routines expect a 2-d array */
    struct FDESC* fdarray[1+1]; /* file descriptor array */ 
    long dsizeap,           /* # bytes per pixel as seen by the app */
    dtypeap,                /* image data type as seen by the app */ 
    dtypeout,               /* image data type to write = EBYTE, ... */
    iomode,                 /* mode of io, = BILIO, BSQIO */ 
    minmax,                 /* tae global $minmax, flag, determine 
                               min/max for ddr? = T,F */
    msgtim,                 /* tae global $msgtim, time interval for
                               processing message */ 
    nl, ns,                 /* size of output image */ 
    totalbands;             /* total number of bands */ 
    unsigned char *buffer,  /* holds one image line for BSQIO */ 
    *linebuf[MAXBND];       /* holds one line for each band. for BILIO */
    double minval[MAXBND],  /* min pixel value, used to update ddr */ 
    maxval[MAXBND];         /* max pixel value, used to update ddr */ 
    /* status flags used for error handing, = true/false */ 
    long closed,            /* has the image been closed */ 
    delflg,                 /* tae global $delflg, delete images if err*/
    opened;                 /* has the image been opened */
}; 



/* function in errclean.c */ 
void errclean (char *mkey, char *format, ...); 

/* functions in image.c */
void open_bil_in (struct PARBLK *pblock, char *parmname, long dtypeap, 
	   struct INIMAGE *inimage); 
void open_inimage (struct PARBLK *pblock, char *parmname, long dtypeap, 
	   long iomode, struct INIMAGE *inimage); 
void open_outimage (struct PARBLK *pblock, char *parmname, long dtypein, 
	   long dtypeap, long dtypeout, long iomode, long nl, long ns, 
	   long nbands, struct OUTIMAGE *outimage); 
void read_bil (long curline, struct INIMAGE *inimage);
void write_bil (long curline, struct OUTIMAGE *outimage);
void minmax_bil (struct OUTIMAGE *outimage);
void close_inimage (struct INIMAGE *inimage);
void close_outimage (struct OUTIMAGE *outimage);
void finish_out (struct PARBLK *pblock, struct INIMAGE *inimage,
           struct OUTIMAGE *outimage);
