#include "multiply.h"
#include "util.h"

void getpar
(
    char hos_in[][CMLEN],     /* input images host names          */
    char las_out[][CMLEN],           /* output image las name            */
    char hos_out[][CMLEN],           /* output image host name           */
    long *num_images,         /* number of images to be multiplied*/
    long *num_bands,          /* total number of input image bands*/
    long num_inimages[10],    /* number of images in INx          */
    long num_image_bands[MAXIMG],
    long image_band_num[][MAXBND + 1],
    long wind[MAXIMG][4],     /* input image windows              */
    long *nlout,              /* output image number of lines and */
    long *nsout,              /* number of samples per line       */
    long *complex,            /* real/complex image indicator     */
    long *odtype              /* output data type 		 */
)
  	
/*
**********************************************************************

NAME: GETPAR

PURPOSE: Retrieves parameter values 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          09-Jan-84  J. Bay, SAR
   2          25-Sep-87  D. Etrheim       
   3           5-Dec-88  K. Narayanan, SAR    Conversion to C
   4          21-Mar-89  K. Narayanan, STX    Changed parameter name

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Initialize the program
     Get complex image indicator
     Call get_in to get names of input images
     If number of bands does not agree with that for 1st image
        issue fatal error message
     If more than two images are specified
        Call get_in to get names of files in the image
        If number of bands does not match previous number
           issue fatal error message
     Call get_out to get output image name
     If images are complex adjust window SS and NS to reflect two times the
        specified number of complex pixels 
     Determine output image size from minimum of all input image sizes
     If complex and number of pixels per line not even, report error and abort
     Get LAS globals for processing message interval, minmax, and history
     
**********************************************************************
*/

{
struct VARIABLE *var;              /* pointer to a variable in TAE block */

long	flag[MAXIMG][2],            /* flag for subwindows                */
	i, j, 
	img,
	imglimit = MAXIMG,          /* maximum number of images           */
	nbands,
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

char	buf[ERRLEN],                /* buffer for error messages          */
	complex_flag[8],            /* real/complex flag                  */
	minmax[4],                  /* minmax option                      */
	dtype[10];		    /* data type 			  */
static	char las_in[MAXIMG][CMLEN]; /* input images las names             */

char tmpstr[1][CMLEN];     /* Temp String */

char *prog_name = "multiply";
long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;



/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get real/complex option
-----------------------
*/
var = p_find (&block, "MULTOPT");
strcpy (complex_flag, SVAL(*var, 0));
if (complex_flag[0] == 'C') *complex = TRUE;
else *complex = FALSE;

/*
get and validate input images parameters - at least 2 images must be specified
and all images must have same number of bands. If the images are complex and
if no window is specified then divide the size of the image in pixels by 2;
if window is specified it is in terms of complex pixels
------------------------------------------------------------------------------
*/
var = p_find (&block, "IN");
*num_images = (*var).v_count;

for (j = 0, img = 0; j < *num_images; j++)
   {
   stat = c_getin (&block, "IN", &j, las_in[img], hos_in[img], 
                   &num_inimages[j], wind[img], image_band_num[img], 
                   &num_image_bands[img], 
                   &nbands, &imglimit, &same_window, flag[img]);
   if (stat != E_SUCC) 
       c_errmsg ("Error accessing input image", "multiply-in", 
              &fatal_err);
   img += num_inimages[j];

   if (j == 0) 
      *num_bands = nbands;
   else
      if (nbands != *num_bands) 
         {
         strcpy(tmpstr[0],"\0");
         c_abort (&block, "All images must have the same number of bands", 
                   "multiply-nbands", prog_name, tmpstr, &zero);
         }
   }

/*
get and validate output image parameters
----------------------------------------
*/
stat = c_getout (&block, "OUT", las_out, hos_out);
if (stat != E_SUCC)
   {
   strcpy(tmpstr[0], "\0");
   c_abort (&block, "Error getting output image name", "multiply-out", 
            prog_name, tmpstr, &zero);
   }

/*
complex math is simulated by 2 r*4 pixels;if a window is specified it is 
in terms of complex pixels; multiply it by 2 to get actual number
------------------------------------------------------------------------
*/
for (i = 0, img = 0; i < *num_images; i++)
   {
/*
      getddr (hosin[i], ddr);
      if (ddr->dtype ==  EREAL) 
         {
         dreal = TRUE;
         } 
*/
   if (*complex)
      {
      for (j = 0; j < num_inimages[i]; j++, img++)
         {
         if (flag[img][0])
            {
            wind[img][SS] = 2*wind[img][SS]-1;
            wind[img][NS] = 2*wind[img][NS];
            }
         }
      }  
   }  

/* Retrieve output data type
----------------------------*/
var = p_find(&block,"odtype");
strcpy(dtype,SVAL(*var,0));
if (strcmp(dtype,"BYTE") == 0)
   *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
   *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
   *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
   *odtype = EREAL;
/*
Set the output image size as the minimum of all the input image sizes
---------------------------------------------------------------------
*/
*nlout = wind[0][NL];
*nsout = wind[0][NS];
for (i = 0; i <  img; i++)
   {
   if (*nlout  >  wind[i][NL]) *nlout = wind[i][NL];
   if (*nsout  >  wind[i][NS]) *nsout = wind[i][NS];
   }  
/*
Report the size of the output image
-----------------------------------
*/
if (*complex)
   {
   if ((*nsout/2)*2 != *nsout) 
      {
      strcpy(tmpstr[0], "\0");
      c_abort (&block, 
               "Number of input pixels must be even for complex multiply", 
               "multiply-evenpix", prog_name, tmpstr, &zero);
      }
      
   sprintf (buf, 
            "Output image will be COMPLEX and of size %5d lines %5d samples",
            *nlout, *nsout/2);      
   c_errmsg (buf, "multiply-outsize1", &inf_err); 
   }
else
   {
   sprintf (buf, "Output image size will be  %5d lines %5d samples",
            *nlout, *nsout);
   c_errmsg (buf, "multiply-outsize2", &inf_err); 
   }  
/*
Get globals
-----------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

}
