/*--------------------------------------------------------------------------
ROUTINE: find_header_files

PURPOSE:
        Search the current media for valid NDF header files.  Three media
        types are supported: tape, CD, and disk.  For tape and CD, 
        multiple volumes are supported and mount requests are issued
        at the proper times.  Any NDF headers found will be copied
        to the current working directory.

PROGRAM HISTORY:
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        OPERATOR_MSG - (mount message) if NULL, no mount required
        VOLUME - currently mounted volume number (index to current COMMENT) 
        INFILE - path to input file
        WORK_DIR - working directory
        MOUNTED - indicates a tape or CD is mounted
        DRIVE - drive name of previously mounted CD or tape
        WRTDSK - WOR, HST, and/or NHDR to keep report, history, and header
                 files respectively
        SKIP_FILES - number of files to skip on a tape
        OUTFILE - path to output file (for naming ndf header, report, and
                  history files)
        DEMOUTFILE - path out DEM output file (for naming DEM header file)
        RESOLUTION - current resolution number (used for naming header files
                     when reading revision 2 or later from tape)
        REMAIN_FLAGS - bit array indicating which outputs need to be created yet
                       bit 0:bands remain, 1:history file, 2:report file, 
                           3:DEM, 4:initialize

OUTPUTS:
        HEADER_FILES - List of header files found
        CLEANUP_FILES - List of files to be cleaned up
        DRIVE - drive name of CD or tape mounted
        DEVICE - mount point of CD or tape mounted
        VOLUME - currently mounted volume number
        MOUNTED - flag to indicate a tape or CD is mounted
        RESOLUTION - current resolution number (used for naming header files
                     when reading revision 2 or later from tape)
        FILES_CREATED - files created by this routine that do not fall under
                        the cleanup list or the header list (report and 
                        history files)
        REMAIN_FLAGS - updated remaining flags 

ALGORITHM:
        Retrieve parameters from TAE block
        Set the tape flag if reading from tape
        While HEADER_FILE_LIST is empty AND (more tapes or CD's available OR
            have not run through the loop once already)
            Set done at least once flag
            If reading from CD and the mounted flag is set
                Unmount the previous CD
                Increment the volume count
            If tape or CD and not mounted
                If no volumes remain to be mounted
                    Break out of the loop
                Issue the next mount request
                If CD
                    Add device path to the file name
            If tape
                Get the header file from the tape
            Else (CD or disk)
                Get the header files from the disk or CD
        End-while
        Set return values in the TAE parameter block

NOTES: 
- Both NDF versions 1 and 2 are supported for all media.

--------------------------------------------------------------------------*/

#include <string.h> /* string function prototypes */
#include "las.h"    /* las header file */
#include "util.h"   /* c_inlas prototype */
#include "media_control.h" /* mount_media and unmount_media prototypes */
#include "get_header_from_disk.h" /* get_header_from_disk prototype */
#include "get_header_from_tape.h" /* get_header_from_tape prototype */
#include "ndfin.h"  /* ndfin defines */



int getpar
(
    struct PARBLK *vblk, /* pointer to initial TAE parameter block */
    char oprmsg[][CMLEN],/* message for the operator when requesting tape */
    int  *num_oprmsg,    /* number of operator messages */ 
    int  *volume,        /* volume number (for generating multiple files) */
    char *infile,        /* host filename of image data on disk */ 
    int  *num_infiles,   /* number of input files */
    char *work_dir,      /* working directory */
    int  *mounted,       /* flag indicating if the tape is mounted or not */
    int  *cd,            /* flag indicating the media mounted is a CD */
    int  *tape,          /* flag indicating the media mounted is Tape */
    char *drive,         /* las drive name for a tape or CD */
    long *keep_nhdr,     /* keep headers flag */
    long *keep_wor,      /* keep work order report */
    long *keep_hst,      /* keep history file */
    long *skip_files,    /* number of files to skip */
    char *outfile,       /* output file name */
    char *demoutfile,    /* DEM output file name */
    int  *resolution,    /* current resolution */
    int  *remaining_flags/* remaining output to create */
)
{
    struct VARIABLE *var;  /* ptr to a variable in the vblk */
    int index;             /* loop counter */
       
    char tmpstr[CMLEN];

    /* default flags */
    *keep_nhdr = 0;
    *keep_wor = 0;
    *keep_hst = 0;
    
    /* Get the operator messages
    ----------------------------*/
    var = p_find(vblk, "OPERATOR_MSG");
    *num_oprmsg = var->v_count;
    for (index = 0; index < var->v_count; index++)
        strcpy(oprmsg[index],SVAL(*var,index));    

    /* Retrieve the tape mounted flag
    ---------------------------------*/
    var = p_find(vblk, "MOUNTED");
    *mounted = IVAL(*var, 0);

    /* Retrieve the cd flag
    ----------------------*/
    var = p_find(vblk, "CDFLAG");
    *cd = IVAL(*var, 0);     
    
    /* Retrieve the cd flag
    -----------------------*/
    var = p_find(vblk, "TAPEFLAG");
    *tape = IVAL(*var, 0); 

    /* Retrieve the volume number
    -----------------------------*/
    var = p_find(vblk, "VOLUME");
    *volume = IVAL(*var, 0);
       
    /* get ndf base name
    --------------------*/
    var = p_find(vblk,"INFILE");
    *num_infiles = var->v_count;
    if (*num_infiles > 0)
        strcpy(infile,SVAL(*var,0));
    else
        strcpy(infile,"");

    /* get working directory name
    -----------------------------*/
    var = p_find(vblk,"WORK_DIR");
    strcpy(work_dir,SVAL(*var,0));
    
    /* get the drive name
    ---------------------*/
    var = p_find(vblk,"DRIVE");
    strcpy(drive,SVAL(*var,0));

    /* Retrieve the number of files to skip
    ---------------------------------------*/
    var = p_find(vblk, "SKIP_FILES");
    *skip_files = IVAL(*var, 0);
    
    /* get output file name
    -----------------------*/
    var = p_find(vblk,"OUTFILE");
    if (var->v_count > 0)
        strcpy(outfile,SVAL(*var,0));
    else
        strcpy(outfile,"");
       
    /* get DEM output file name
    ---------------------------*/
    var = p_find(vblk,"DEMOUTFILE");
    if (var->v_count > 0)
        strcpy(demoutfile,SVAL(*var,0));
    else
        strcpy(demoutfile,"");
       
    /* get the WRTDSK entries 
    -------------------------*/
    var = p_find(vblk,"WRTDSK");
    for (index = 0; index < var->v_count; index++)
    {
        strcpy(tmpstr,SVAL(*var,index));
        if (strcmp(tmpstr,"WOR") == 0)
        {
            *keep_wor = 1;
        }
        else if (strcmp(tmpstr,"HST") == 0)
        {
            *keep_hst = 1;
        }
        else if (strcmp(tmpstr,"NHDR") == 0)
        {
            *keep_nhdr = 1;
        }
    }

    /* Retrieve the resolution
    --------------------------*/
    var = p_find(vblk, "RESOLUTION");
    *resolution = IVAL(*var, 0);

    /* Retrieve the remaining flags
    -------------------------------*/
    var = p_find(vblk,"REMAIN_FLAGS");
    *remaining_flags = IVAL(*var,0);
    if (*remaining_flags == INITIALIZE_NOT_DONE)
    {
        *remaining_flags = BANDS_NOT_DONE;
        if (*keep_hst)
            (*remaining_flags) |= HISTORY_NOT_DONE;
        if (*keep_wor)
            (*remaining_flags) |= WOR_NOT_DONE;
        if (strcmp(demoutfile,"") != 0)
            (*remaining_flags) |= DEM_NOT_DONE;
    }
   
    return(E_SUCC);
}


int main(void)
{
    /* LAS constants */
    long nonfatal = 1;  /* non-fatal message */
    long fatal = -1;    /* fatal error flag */
    long std_out = 0;   /* standard out */
    
    /* parameters read from tae block (some also written) */
    struct PARBLK vblk;  /* TAE parameter block */
    long keep_nhdr;      /* keep headers flag */
    long keep_wor;       /* keep work order report flag */
    long keep_hst;       /* keep history file flag */
    int  mounted;        /* mounted flag (1=tape or CD mounted */
    int  remaining_flags;/* remaining output to create bit array */
    int  resolution;     /* current resolution */
    long skip_files;     /* number of files to skip */
    int  volume;         /* current volume number */
    int  already_have_wor = 0; /* flag to indicate report file has been read */
    int  already_have_hst = 0; /* flag to indicate history file has been read */
    int  done_at_least_once=0; /* flag to run through loop at least once */
    int  index;          /* loop counter */
    int  tape = 0;       /* reading tape flag */
    int  cd   = 0;       /* reading cd flag */
    int  disk = 0;       /* reading from disk flag */
    char demoutfile[CMLEN]; /* output file name for the DEM */
    char device[CMLEN]="";  /* tape or CD device */
    char drive[CMLEN];   /* las drive name */
    char outfile[CMLEN]; /* output file name */
    char *string_ptrs[MAX_HEADER_FILES]; /* used to return variables to tae */
    char work_dir[CMLEN];/* working directory path */

    /* lists */
    char infile[CMLEN];  /* path name of input file */
    int  num_infiles;    /* number of input files */
    char oprmsg[MAX_MOUNT_MESSAGES][CMLEN]; /* operator mount message */
    int  num_oprmsg=0;   /* number of operator messages */
    char header_file_list[MAX_HEADER_FILES][CMLEN]; /* list of header files */
    long num_header_files=0;  /* number of header files in the list */
    char cleanup_file_list[MAX_CLEANUP_FILES][CMLEN]; 
                              /* list of files to cleanup */
    long num_cleanup_files=0; /* number of header files in the list */
    char files_created[MAX_CREATED_FILES][CMLEN]; /* files created that don't
                                fall in the header or cleanup list */
    long num_files_created=0;

    /* Initialize tae block
    -------------------------*/
    c_inlas (&vblk, &std_out);

    /* Get the parameters from the TAE variable block
    -------------------------------------------------*/
    if (getpar (&vblk, oprmsg, &num_oprmsg, &volume, infile, &num_infiles,
                work_dir, &mounted, &cd, &tape, drive, &keep_nhdr, 
                &keep_wor,&keep_hst, &skip_files, outfile, demoutfile,
                &resolution,&remaining_flags) != E_SUCC)
    {
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* Loop through the media looking for the first available header files.
       Loop while no header files have been found and there are more media sets
       to check (do it at least once in case the source is on disk) 
    ----------------------------------------------------------------------*/
    while ((num_header_files == 0) && 
           ((volume <= num_oprmsg) || !done_at_least_once))
    {
        done_at_least_once = 1;
       
        /* if working with CD's, clear the mount flag since we only need to
           look at a CD once to find all the header files */
        if ((cd) && (mounted))
        {
            if (unmount_media(&vblk, drive, &mounted) != E_SUCC)
            {
                /* fatal error occurred unmounting the media, return a
                   failure (no cleanup required here) */
                return (E_FAIL);
            }  

            /* increment the volume number anytime the mount flag is cleared */
            volume++;
        }
 
        /* issue a mount request if nothing is mounted
        ----------------------------------------------*/
        if ((num_oprmsg > 0) && !mounted)
        {
      
            /* if out of volumes, break out */
            if (volume > num_oprmsg)
                break;

            /* mount the media (no return code to check since any error is
               fatal in the routine and no cleanup needed) */
            if (mount_media(&vblk, oprmsg, volume, &tape, &cd, drive, device, 
                            &mounted) != E_SUCC)
            {
                c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
            }  

            /* if the source is a CD, prepend the mount path to the infile */
            if (cd)
            {
                char temp_string[CMLEN];
                sprintf(temp_string,"%s%s",device,infile);
                strcpy(infile,temp_string);
            }
        }

        /* look for header files
        ------------------------*/
        if (tape)
        {
            if (get_header_from_tape(&vblk,&volume,outfile,demoutfile,drive,
                                     &skip_files,&mounted,&already_have_wor,
                                     &already_have_hst,&resolution,
                                     keep_nhdr,keep_wor,keep_hst,
                                     header_file_list, &num_header_files,
                                     cleanup_file_list, &num_cleanup_files,
                                     files_created, &num_files_created,
                                     &remaining_flags)
                               != E_SUCC)
            {
                c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
            }
        }
        else
        {
            if (get_header_from_disk(cd,infile,outfile,demoutfile,volume,
                                     keep_nhdr,keep_wor,keep_hst,
                                     header_file_list, &num_header_files,
                                     cleanup_file_list, &num_cleanup_files,
                                     files_created, &num_files_created,
                                     &remaining_flags) != E_SUCC)
            {
                c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
            }
        }
    }

    /* Set disk flag
    --------------*/
    if ((!tape) && (!cd))
        disk = 1;

    /* Return the results to the TAE block
    --------------------------------------*/
    /* return the header file list */
    for (index = 0; index < num_header_files; index++)
        string_ptrs[index] = header_file_list[index];
    if (q_string(&vblk,"HEADER_FILES",num_header_files,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing HEADER_FILES to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }

    /* return the cleanup file list */
    for (index = 0; index < num_cleanup_files; index++)
        string_ptrs[index] = cleanup_file_list[index];
    if (q_string(&vblk,"CLEANUP_FILES",num_cleanup_files,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing CLEANUP_FILES to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }

    /* return the device designation */
    string_ptrs[0] = device;
    if (q_string(&vblk,"DEVICE",1,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing DEVICE to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
     
    /* return the possibly modified infile name */
    string_ptrs[0] = infile;
    if (q_string(&vblk,"INFILE_FOUND",1,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing INFILE to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
    
    /* return the drive */
    string_ptrs[0] = drive;
    if (q_string(&vblk,"DRIVE",1,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing DRIVE to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
 
    /* return the volume number */
    if (q_intg(&vblk,"VOLUME",1,&volume,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing VOLUME to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
    
    /* return the mounted flag number */
    if (q_intg(&vblk,"MOUNTED",1,&mounted,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing MOUNTED to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
   
    /* return the cd flag, this is needed if this routine is called again  */
    if (q_intg(&vblk,"CDFLAG",1,&cd,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing CDFLAG to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }

    /* return the tape flag */
    if (q_intg(&vblk,"TAPEFLAG",1,&tape,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing TAPEFLAG to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }

    /* return the disk flag */
    if (q_intg(&vblk,"DISKFLAG",1,&disk,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing DISKFLAG to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }

    /* return the resolution */
    if (q_intg(&vblk,"RESOLUTION",1,&resolution,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing RESOLUTION to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
    
    /* return the remaining output flags */
    if (q_intg(&vblk,"REMAIN_FLAGS",1,&remaining_flags,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing REMAIN_FLAGS to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }

    /* return the files created names */
    for (index = 0; index < num_files_created; index++)
        string_ptrs[index] = files_created[index];
    if (q_string(&vblk,"FILES_CREATED",num_files_created,string_ptrs,P_UPDATE) 
            != SUCCESS)
    {
        c_errmsg("Error passing FILES_CREATED to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
      
    if (q_out(&vblk) != SUCCESS)
    {
        c_errmsg("Error passing variables to TAE","ndfin-tae",&nonfatal);
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
    return (E_SUCC);
}
