/*--------------------------------------------------------------------------
ROUTINE: get_file_from_tape

PURPOSE:
        Read a single file from the tape drive specified.

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        drive - name of tape drive to read the file from
        dest_filename - name of the file to create on disk
        skip_files - number of files to skip before reading the file
        
OUTPUTS:
        bytes_read - number of bytes read from the tape
        returns - TPREAD_READ on fatal error (error - calling routine needs
                    to unmount media)
                  TAPE_EOF when end of file detected (successful)
                  TAPE_EOT when end of tape detected (successful)
                  TAPE_EOV when end of volume detected (successful)
                  
ALGORITHM:
        Reopen the tape drive, skipping the indicated number of files
        While not exceeding the maximum number of reads
            Read a buffer of data from the tape
            If a device error was encountered
                Issue an error message
                Set the error flag and break out of the loop
            Else if end of file, end of tape, or end of volume
                Break out of loop
            Else
                If destination file not opened yet
                    Open the destination file
                    If an error occurred opening the file
                        Issue an error message
                        Set the error flag and break out of the loop
                Write the data read from the tape to the destination file
                If an error occurred writing
                    Issue an error message
                    Set the error flag and break out of the loop
                Update the number of bytes transferred
        If too many reads occurred
            Issue an error message
            set the error flag
        If the destination file was opened
            Close the destination file
        Close the tape drive (but don't take the tape offline)
        If an error occurred
            If the destination file was opened
                Remove destination file
            Return error
        Return status of last tape read
        
NOTES:
none

--------------------------------------------------------------------------*/

#include <string.h>     /* strcpy prototype */
#include "las.h"        /* las header file */
#include "diskio.h"     /* c_dkdel prototype */
#include "tapeio.h"     /* tapeio header file */
#include "get_file_from_tape.h" /* prototype for this module */

/*
** MACRO definitions
*/

#define TAPE_BUF_SIZE 32000
/* size of buffer for tape reads.  Note: this must be bigger than the 
   blocking factor on the tape */
   
#define MAX_FILE_READS 100
/* maximum number of buffers of data to read.  So no more than 3.2 MB will
   be read from the tape (since only expecting a few KB of data). */

int get_file_from_tape
(
    char *drive,        /* I: drive name */
    char *dest_filename,/* I: destination file name on disk */
    long skip_files,    /* I: files to skip when opening the tape drive */
    int  *bytes_read    /* O: bytes read into the destination file */
)
{
    /* LAS constants */
    long nonfatal = 1;  /* non-fatal message */
    long zero = 0;      /* Constant of zero */
    long one = 0;       /* Constant of one */

    FILE *fp = NULL;       /* pointer to destination file */
    long buf_size=TAPE_BUF_SIZE;/* buffer size */
    long access = TREAD;   /* Indicates tape access mode */
    long bytes_xfr;        /* bytes transferred from tape */
    long mode = HANDLE;    /* handle own errors */
    long rewopt = NOREW;   /* Don't rewind on open */
    long unit;             /* tape drive unit */
    int  error = 0;        /* error encountered flag */
    int  index;            /* loop counter */
    int  status;           /* status returned by routines called */
    unsigned char buffer[TAPE_BUF_SIZE]; /* buffer for reading data from tape */
    char density[5] = "0"; /* Density of mounted tape */
    
    *bytes_read = 0;
   
    /* issue a message if skipping files */
    if (skip_files)
    {
        c_lasmsg(FALSE,"informational","Skipping %d files on the tape",
                 skip_files);
    }

    /* reopen the tape drive without rewinding the tape */
    status = c_tpreop(&unit, &skip_files, &mode, &rewopt, density, 
                      &access, drive);
    if (status != TAPE_SUCC)
    {
        c_lasmsg(FALSE,"ndfin-media","Error encountered opening media");
        return (TPREAD_READ); /* return an error */
    }
    
    for (index = 0; index < MAX_FILE_READS; index++)
    {
        bytes_xfr = c_tpread(&unit, buffer, &buf_size, &one);
        if (bytes_xfr == TPREAD_READ)
        {
            /* device error */
            c_errmsg("Error encountered reading meda","ndfin-media", &nonfatal);
            
            error = 1;
            break;
        }
        else if ((bytes_xfr == TAPE_EOF) || (bytes_xfr == TAPE_EOT) ||
                 (bytes_xfr == TAPE_EOV))
        {
            /* end of file reached */
            break;
        }
        else
        {
            /* open the destination file since we have data */
            if (fp == NULL)
            {
                
                fp = fopen(dest_filename,"wt");
                if (fp == NULL)
                {
                    c_lasmsg(FALSE,"ndfin-file","Unable to open %s",
                             dest_filename);

                    error = 1;
                    break;
                }
            }
            
            /* write the data to the file */
            if (fwrite(buffer,1,bytes_xfr,fp) != bytes_xfr)
            {
                /* error writing to disk */
                c_lasmsg(FALSE,"ndfin-file","Error writing %s", dest_filename);

                error = 1;
                break;
            }

            *bytes_read += bytes_xfr;
        }
    }
   
    /* check for reading an unexpected large file */
    if (index >= MAX_FILE_READS)
    {
        /* unexpected extremely large file found, consider it an error */
        c_lasmsg(FALSE,"ndfin-media","Unexpected file found on the tape");

        error = 1;
    }

    /* close the file */
    if (fp)
        fclose(fp);

    /* close the tape drive */
    rewopt = NOREW;
    status=c_tpclos(&unit, &mode, &rewopt, &zero);
    if (status != TAPE_SUCC)
    {
        c_lasmsg(FALSE,"ndfin-media","Error encountered closing media");
        return (TPREAD_READ);
    }
    
    /* if an error was encountered or a very small file and the end of tape or
       volume was found, remove the destination file */
    if ((error) || ((bytes_xfr != TAPE_EOF) && (*bytes_read < 3)))
    {
        /* remove the destination file if it was ever opened */
        if (fp)  /* note: fp is no longer pointing to a valid file */
            c_dkdel(dest_filename); /* don't bother error checking */

        *bytes_read = 0;
        /* if an error occurred, return an error indication */
        if (error)
            return (TPREAD_READ);
    }

    /* return the final tape read status */
    return (bytes_xfr);
}


