/*--------------------------------------------------------------------------
ROUTINE: get_header_from_disk

PURPOSE:
        Find valid header files in the current input directory for either 
        disk or CD and copy them to the working directory.  Optionally, the
        history and report files are copied to the working directory.

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        infile - input filename pattern
        outfile - output file name specified
        demoutfile - output DEM filename specified
        volume - current volume number
        keep_nhdr - keep headers flag (non-zero=keep)
        keep_wor - keep work order report flag (non-zero=keep)
        keep_hst - keep history flag (non-zero=keep)
        remaining_flags - bit array used to track remaining files needed before
                          we can stop looking for files
        
OUTPUTS:
        header_file_list - list of header files found
        num_header_files - number of files in header file list
        cleanup_file_list - list of temporary files that need cleaning up
        num_cleanup_files - number of files in cleanup list
        files_created_list - list of files created
        num_files_created - number of files in files created list
        remaining_flags - updated remaining data needed bit array
        returns - E_SUCC on success
                  E_FAIL when a failure occurs
                  
ALGORITHM:
        Determine the output file name to use for the files copied
        Add the header file pattern to the file specification list
        If keeping header files or a DEM is requested
            Add the DEM file name pattern to the file specification list
        If keeping the report file
            Add the report file pattern to the file specification list
        For each file specification in the list
            Look for files matching in the list
        For each file found
            If report file
                If keeping report files
                    Clear the report file bit in the remaining flags
                    Build the real file name for the report file
                    Add the file to the files created list
                    Set the copy flag
            Else if history file
                If keeping history files
                    Clear the history file bit in the remaining flags
                    Build the real file name for the history file
                    Add the file to the files created list
                    Set the copy flag
            Else if valid NDF header file
                Build the real file name
                Add the file to the header file list
                If not keeping header files
                    Add the file to the cleanup list
                Set the copy flag
            If the copy flag is set
                Copy the file to the correct destination file name
        
NOTES:
- Search for file names with upper or lower case (Mixed case is not expected)

--------------------------------------------------------------------------*/

#include <stdlib.h> /* atoi protottype */
#include <string.h> /* string function prototypes */
#include <ctype.h>  /* isdigit prototype */
#include "las.h"    /* las header file */
#include "cm.h"     /* c_getfiles prototype */
#include "util.h"   /* c_splitname prototype */
#include "get_header_from_disk.h" /* prototypes for this module */
#include "valid_ndf_header.h" /* valid_ndf_header prototype */
#include "ndfin.h"  /* unit defines */

/*
** MACRO DEFINITIONS
*/

#define MAX_SEARCH_FILES 30
/* maximum number of files to accept on a search */

#define MAX_FILE_SPECS 16
/* maximum number of file patterns to match */

int get_header_from_disk
(
    int  cd_flag,       /* I: flag to indicate the source is a CD */
    char *infile,       /* I: input file name */
    char *outfile,      /* I: output file name */
    char *demoutfile,   /* I: DEM output file name */
    long volume,        /* I: volume currently mounted */
    long keep_nhdr,     /* I: keep headers flag */
    long keep_wor,      /* I: keep work order report flag */
    long keep_hst,      /* I: keep history file flag */
    char header_file_list[][CMLEN], /*I/O: header file list */
    long *num_header_files,         /*I/O: number of files in header file list*/
    char cleanup_file_list[][CMLEN],/*I/O: cleanup file list */
    long *num_cleanup_files,        /*I/O: num of files in cleanup list */
    char files_created_list[][CMLEN],/*I/O: files created list */
    long *num_files_created,        /*I/O: num of files in files created list */
    int  *remaining_flags /* I/O: remaining outputs to create */
)
{
    /* LAS constants */
    long nonfatal = 1;  /* non-fatal message */

    int  index, i;       /* loop counters */
    int  revision;       /* ndf header revision number */
    char *base_filename = NULL; /* pointer to output base file name */
    char dest_filename[CMLEN]; /* file name for header file */
    char dir[CMLEN];     /* directory name */
    char ext[CMLEN];     /* extension of file */
    char filespec[CMLEN];   /* current file pattern matching */
    char root[CMLEN];    /* root of file name */
    char root_of_first_file[CMLEN]; /* root of the 1st file name */
    /* lists */
    char files[MAX_SEARCH_FILES][CMLEN]; /* files matching patterns found */
    long files_found = 0;   /* number of files found */
    char filespecs[MAX_FILE_SPECS][6]; /* file patterns to match */
    int  num_filespecs = 0; /* number of file patterns to match */
    int  infile_set = 0;    /* flag to indicate the infile has been set to the
                               full file spec (so DEM image files can be found
                               if only a partial file name is set) */
    int  local_volume;   /* volume header file being analyzed is on */

    /* Get the base_filename */
    if (strcmp(outfile,"") != 0)
        base_filename = outfile;
    else
        base_filename = demoutfile;

    /* Allow filespecs with either upper or lower case extension */

    /* If the files are to be read from CD then a partial infile name 
       is allowed. Otherwise the user is required to know the full 
       file name for files to be read from disk */

    /* Add header file extensions to the filespec list */
    if (cd_flag)
    {
        strcpy(filespecs[num_filespecs],"*.h*");
        num_filespecs++;
        strcpy(filespecs[num_filespecs],"*.H*");
    }
    else
    {
        strcpy(filespecs[num_filespecs],".h*");
        num_filespecs++;
        strcpy(filespecs[num_filespecs],".H*");
    }
    
    num_filespecs++;

    /* add the dem header file filespec if DEM requested or keeping headers */
    if (keep_nhdr || (strcmp(demoutfile,"") != 0))
    {
        if (cd_flag)
        {
            strcpy(filespecs[num_filespecs],"*.dh");
            num_filespecs++;
            strcpy(filespecs[num_filespecs],"*.DH");
        }
        else
        {
            strcpy(filespecs[num_filespecs],".dh");
            num_filespecs++;
            strcpy(filespecs[num_filespecs],".DH");
        }

        num_filespecs++;
    }

    if (keep_wor)
    {
        if (cd_flag)
        {
            /* add the work order report extension to the filespec list 
               since the user has requested it */
            strcpy(filespecs[num_filespecs],"*.wo");
            num_filespecs++;
            strcpy(filespecs[num_filespecs],"*.WO");
        }
        else 
        {
            strcpy(filespecs[num_filespecs],".wo");
            num_filespecs++;
            strcpy(filespecs[num_filespecs],".WO");
        }

        num_filespecs++;
    }

    /* Note: this code isn't needed since the file name pattern for a 
       history file is covered by the normal header file pattern.
       So it is commented out!
       if (keep_hst)
       {
       strcpy(filespecs[num_filespecs],"*.hi");
       num_filespecs++;
       }
     */

    /* reading a CD or disk, so search for any header files in the 
       input directory 
       --------------------------------------------------------------*/
    for (index = 0; index < num_filespecs; index++)
    {
        long temp_files_found = 0;

        /* build file pattern */
        strcpy (filespec,infile);
        strcat (filespec,filespecs[index]);

        if (c_getfiles(filespec, files[files_found],
                    &temp_files_found,
                    MAX_SEARCH_FILES - files_found, 1 /*first call*/,
                    0 /*return all files*/) != E_SUCC)
        {
            /* error encountered, let calling routine generate the message */
            return (E_FAIL);
        }

        /* generate an error message if too many files found to read
           in a single pass since it will leave the pipe open in 
           c_getfiles */
        if (temp_files_found >= MAX_SEARCH_FILES - files_found)
        {
            /* generate an error message */
            c_errmsg("Too many entries for files array","ndfin-size",&nonfatal);
            return (E_FAIL);
        }

        files_found += temp_files_found;
    }

    /* If a CD was mounted and infile was left blank, 
       find the root of the first file found on the CD 
       and use those header files only */ 
    if (cd_flag)
    {
        c_splitname (files[0], dir, root_of_first_file, ext);
    }

    /* determine which of the files found are really NDF header files */
    for (index = 0; index < files_found; index++)
    {
        int copy_flag = 0;
        local_volume = -1; /* reset to -1 to key off */

        /* split the file name to get the extension */
        c_splitname(files[index],dir,root,ext);

        if (((cd_flag) && (strcmp(root_of_first_file,root) == 0)) || (!cd_flag))
        {
            /* check if the file is a work order report */
            if ((strcmp(ext,"WO") == 0) || (strcmp(ext,"wo") == 0))
            {
                if ((keep_wor) && ((*remaining_flags) & WOR_NOT_DONE))
                {
                    *remaining_flags &= ~WOR_NOT_DONE;
                    sprintf(dest_filename,"%s.%s",base_filename,ext);
                    copy_flag = 1;

                    if (*num_files_created > MAX_CREATED_FILES)
                    {
                        c_lasmsg(FALSE,"ndfin-size",
                                "Too many entries for files created list");
                        return (E_FAIL);
                    }

                    strcpy(files_created_list[*num_files_created],
                            dest_filename);
                    (*num_files_created)++;
                }
            }
            /* check if the file is a history file */
            else if ((strcmp(ext,"HI") == 0) || (strcmp(ext,"hi") == 0))
            {
                if ((keep_hst) && ((*remaining_flags) & HISTORY_NOT_DONE))
                {
                    *remaining_flags &= ~HISTORY_NOT_DONE;
                    sprintf(dest_filename,"%s.%s",base_filename,ext);
                    copy_flag = 1;

                    if (*num_files_created > MAX_CREATED_FILES)
                    {
                        c_lasmsg(FALSE,"ndfin-size",
                                "Too many entries for files created list");
                        return (E_FAIL);
                    }

                    strcpy(files_created_list[*num_files_created],
                            dest_filename);
                    (*num_files_created)++;
                }
            }
            /* if the current file is a valid NDF header, add it to the
               header file list */
            else
            {
                int valid_header; /* valid header flag */

                if (valid_ndf_header(files[index],&revision,&valid_header,TRUE) 
                        != E_SUCC)
                {
                    return (E_FAIL); 
                }

                if (valid_header)
                {
                    /* set the infile to the full file name for the first valid 
                       header found */
                    if (!infile_set)
                    {
                        infile_set = 1;
                        sprintf(infile,"%s%s",dir,root);
                    }
                    /* create the name of the file to copy to by adding the
                       volume to the name (except for DEM headers) */
                    if ((strcmp(ext,"DH") != 0) && (strcmp(ext,"dh") != 0))
                    {
                        /* non-DEM header, create the destination name */
                        if (!cd_flag)
                        {
                            /* not reading from a CD, so check for the 
                               possibility that the "_vol#" is already 
                               on the header file */
                            char *vol_ptr;

                            vol_ptr = strrchr(root,'_');
                            if ((vol_ptr != NULL) &&
                                    (strncmp(vol_ptr, "_vol", 4) == 0) &&
                                    (strlen(vol_ptr) > 4))
                            {
                                /* save the volume number */
                                local_volume = atoi(&vol_ptr[4]);
                            }
                            /* If this image was created pre-PDS v2.0 release,
                               which was the first format that supported
                               multi-resolution imagery. Yet, the header files
                               were named in a way that, according to the new
                               format, indicates this is a multi-res image.
                               So, when copying the header files to the local
                               directory, rename them according to the new
                               format spec. so it is clear they are from
                               multiple volumes of a single resolution image
                               (xxxx_vol1.h1, xxxx_vol2.h2, etc) */
                            else if (revision < 2.0)
                            {
                                for (i = 1; i < strlen(ext); i++)
                                {
                                    if(isdigit(ext[i]))
                                    {
                                        local_volume = atoi(&ext[i]);
                                        break;
                                    }
                                }
                            }
                            /* Skip headers that have no volume number after
                               the first pass. Also skip headers that have a
                               volume number that isn't the current volume. */
                            if (((local_volume == -1) && (volume != 1)) ||
                                ((local_volume != -1) &&
                                 (local_volume != volume)))
                            {
                                continue;
                            }
                        }
                        sprintf(dest_filename,"%s_vol%d.%s",base_filename,
                                volume,ext);
                    }
                    else
                    {   
                        /* DEM header, so use dem out file name if available */
                        *remaining_flags &= ~DEM_NOT_DONE;
                        if (strcmp(demoutfile,"") != 0)
                            sprintf(dest_filename,"%s.%s",demoutfile,ext);
                        else
                            sprintf(dest_filename,"%s.%s",base_filename,ext);

                        /* if not reading from a disk and this is not the first 
                           pass, skip this file since the first volume pass 
                           already dealt with the DEM */
                        if ((!cd_flag) && (volume > 1))
                            continue;
                    }

                    /* make sure we haven't found too many files */
                    if (*num_header_files >= MAX_HEADER_FILES)
                    {
                        c_errmsg("Too many entries for header file list",
                                "ndfin-size", &nonfatal);
                        return (E_FAIL);
                    }

                    /* add the copied header file to the header list */
                    strcpy(header_file_list[*num_header_files],dest_filename);
                    (*num_header_files)++;

                    /* if not keeping the header files, add the copied header 
                       file to the cleanup list */
                    if (!keep_nhdr)
                    {
                        if (*num_cleanup_files >= MAX_CLEANUP_FILES)
                        {
                            c_errmsg("Too many entries for cleanup file list",
                                    "ndfin-size", &nonfatal);
                            return (E_FAIL);
                        }

                        strcpy(cleanup_file_list[*num_cleanup_files],
                                dest_filename);
                        (*num_cleanup_files)++;
                    }

                    copy_flag = 1;
                }
            }
        } 
        /* if the copy flag is set, copy the file to disk */
        if (copy_flag)
        {
            /* copy the header file to the working directory with
               the new name */
            if (copyfile(files[index],dest_filename, 
                        2 /*report errors using host filenames*/) != E_SUCC)
            {
                c_lasmsg(FALSE,"ndfin-file", "Error copying file %s to %s",
                        files[index],dest_filename);
                return (E_FAIL);
            }
        }
    }

    return (E_SUCC);
}
