/*--------------------------------------------------------------------------
ROUTINE: get_header_from_tape

PURPOSE:
        Find valid header files on the currently mounted tape and copy them 
        to the working directory.  Optionally, the history and report files 
        are copied to the working directory.

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        vblk - TAE parameter block
        volume - current volume number
        outfile - output file name specified
        demoutfile - output DEM filename specified
        drive - drive the tape is mounted on
        skip_files - number of files to skip when opening the tape drive
        already_have_wor - already have report file flag
        already_have_hst - already have history file flag
        resolution - resolution number
        keep_nhdr - keep headers flag (non-zero=keep)
        keep_wor - keep work order report flag (non-zero=keep)
        keep_hst - keep history flag (non-zero=keep)
        remaining_flags - bit array used to track remaining files needed before
                          we can stop looking for files
        
OUTPUTS:
        header_file_list - list of header files found
        num_header_files - number of files in header file list
        cleanup_file_list - list of temporary files that need cleaning up
        num_cleanup_files - number of files in cleanup list
        files_created_list - list of files created
        num_files_created - number of files in files created list
        remaining_flags - updated remaining data needed bit array
        returns - E_SUCC on success
                  E_FAIL when a failure occurs
                  
ALGORITHM:
        Construct the destination file name
        Read the first file from the tape
        If end-of-tape, end-of-volume, or device error
            Unmount the current tape
            If the unmount fails
                If a file was read
                    Delete the file
                Return an error
            If device error
                Return an error
            Increment the volume number
        Clear the count of files to skip
        If a file was read
            If the file is a valid header file
                If we have the history file and a DEM is wanted
                    Change the base output name to the DEM output name
                If we already have a history file
                    Use the DEM header extension ".dh"
                Else
                    If old NDF version
                        Use the ".hd" extension
                    Else
                        Get the starting data file from the header
                        If the starting data file is one
                            Increment the resolution number
                        Use the ".h#" extension where #=resolution
                Rename the file read from the tape
                Add the file to the header file list
                If not keeping header files
                    Add the file to the cleanup list
            Else if a header was found previously
                If a report file hasn't been found yet
                    Assume it is the report file
                    Clear the report file bit in the remaining flags
                    If keeping the report file
                        Rename the file to the correct name
                        Add the file to the files created list
                    Else
                        Remove the file
                Else if a history file hasn't been found yet
                    Assume it is the history file
                    Clear the history file bit inthe remaining flags
                    If keeping the history file
                        Rename the file to the correct name
                        Add the file to the files created list
                    Else
                        Remove the file
            Else
                Error since a header wasn't found first on the tape
        Return E_SUCC
        
NOTES:
- The assumption is made that a header file is found before any data files
  and that the order of files is:  header, data files, [header, data file...],
  report file, history file, [DEM header, DEM data files].  Where files in
  the [] are optional.

--------------------------------------------------------------------------*/

#include <string.h>        /* string function prototypes */
#include "las.h"           /* las header file */
#include "tapeio.h"        /* tapeio header file */
#include "diskio.h"        /* c_dkdel prototype */
#include "get_header_from_tape.h" /* prototype for this module */
#include "get_file_from_tape.h"   /* get_file_from_tape prototype */
#include "media_control.h"        /* mount_media and unmount_media prototypes */
#include "valid_ndf_header.h"     /* valid_ndf_header prototype */
#include "read_ndf_parameters.h"  /* read_ndf_parameters prototype */
#include "ndfin.h"         /* common defines */

int get_header_from_tape
(
    struct PARBLK *vblk,/* I: TAE parameter block */
    int  *volume,       /* I: volume to mount */
    char *outfile,      /* I: output file name */
    char *demoutfile,   /* I: DEM output file name */
    char *drive,        /* I: Drive tape is on */
    long *skip_files,   /* I/O: number of files to skip before reading */
    int  *mounted,      /* O: mounted flag */
    int  *already_have_wor,/* I/O:flag to indicate report file has been read */
    int  *already_have_hst,/* I/O:flag to indicate history file has been read*/
    int  *resolution,   /* I/O: current resolution number */
    long keep_nhdr,     /* I: keep headers flag */
    long keep_wor,      /* I: keep work order report flag */
    long keep_hst,      /* I: keep history file flag */
    char header_file_list[][CMLEN], /*I/O: header file list */
    long *num_header_files,         /*I/O: number of files in header file list*/
    char cleanup_file_list[][CMLEN],/*I/O: cleanup file list */
    long *num_cleanup_files,        /*I/O: num of files in cleanup list */
    char files_created_list[][CMLEN],/*I/O: files created list */
    long *num_files_created,        /*I/O: num of files in files created list */
    int  *remaining_flags /* I/O: remaining outputs to create */
)
{
    char *base_filename = NULL;/* pointer to output base file name */
    char dest_filename[CMLEN]; /* target file name */
    int bytes_read;            /* number of bytes read from the tape */
    int status;                /* return status */

    /* construct the destination file name */
    if (strcmp(outfile,"") != 0)
        base_filename = outfile;
    else
        base_filename = demoutfile;
    sprintf(dest_filename,"%s.temp",base_filename);
    
    /* copy the first file from the tape to disk */
    status = get_file_from_tape(drive,dest_filename,*skip_files,
                                &bytes_read);
    if ((status == TAPE_EOT) || (status == TAPE_EOV) || (status == TPREAD_READ))
    {
        /* unmount the tape */
        if (unmount_media(vblk, drive, mounted) != E_SUCC)
        {
            /* fatal error unmounting media, remove any file just
               read from the tape */
            if (bytes_read)
            {
                if (c_dkdel(dest_filename) == E_FAIL)
                {
                    c_lasmsg(FALSE,"informational",
                             "Error cleaning up temporary file %s",
                             dest_filename);
                }
            }

            return (E_FAIL);
        }
        
        /* if an error occurred reading the tape, error out */
        if (status == TPREAD_READ)
        {
            return (E_FAIL);
        }
        
        /* increment the volume number anytime the mount flag is 
           cleared */
        (*volume)++;
    }

    /* clear the count of files to skip */
    *skip_files = 0;

    if (bytes_read)
    {
        char new_name[CMLEN];/* new file name */
        int  revision;       /* ndf header revision number */
        int  valid_header;   /* valid header flag */
        
        /* if the file isn't a valid ndf header file, assume it is
           the report file and rename it as such.  Then read the 
           next file from the tape as the history file. */
        if (valid_ndf_header(dest_filename, &revision, &valid_header, FALSE)
                != E_SUCC)
        {
            return (E_FAIL);
        }

        if (valid_header)
        {
            /* valid ndf header, so rename it to the correct name */
            if ((*already_have_hst) && (strcmp(demoutfile,"") != 0))
            {
                /* override the base name to be the DEM header file */
                base_filename = demoutfile;
            }
                
            if (*already_have_hst)
            {
                /* Use the DEM extension since it is the only header that
                   can appear after a history file */
                sprintf(new_name,"%s.dh",base_filename);
                *remaining_flags &= ~DEM_NOT_DONE;
            }
            else
            {
                if (revision < 2)
                {
                    sprintf(new_name,"%s_vol%d.hd", base_filename, 
                            *volume);
                    *resolution = 1; /* flag header read */
                }
                else
                {
                     long data_files;
                     NDF_PARAMETER_DEF_TYPE param;
                     strcpy(param.name,"START_DATA_FILE");
                     param.param_type = LONG_PARAM;
                     param.dest = &data_files;
                     param.dest_size = sizeof(long);
                     param.num_items = 1;
                     param.valid_flag_ptr = NULL;
                     
                    /* read the NUMBER_OF_DATA_FILES from the header
                       to determine if a new resolution is present */
                    if (read_ndf_parameters(dest_filename,&param,1)
                            != E_SUCC)
                    {
                        c_lasmsg(TRUE,"ndfin-fatal",
                                 "Error reading NDF Header file %s",
                                 dest_filename);
                    }

                    if (data_files == 1)
                        (*resolution)++;  /* new resolution */
                   
                    sprintf(new_name,"%s_vol%d.h%d", base_filename, 
                            *volume, *resolution);
                }
            }
            if (renamefile(dest_filename,new_name,0L,0L,0L) != E_SUCC)
            {
                c_lasmsg(TRUE,"ndfin-fatal","Error renaming %s to %s",
                         dest_filename,new_name);
            }
            strcpy(dest_filename,new_name);
            
            /* add the header file to the header file list */
            strcpy(header_file_list[*num_header_files],dest_filename);
            (*num_header_files)++;

            /* if not keeping the header files, add the copied header 
               file to the cleanup list */
            if (!keep_nhdr)
            {
                if (*num_cleanup_files >= MAX_CLEANUP_FILES)
                {
                     c_lasmsg(TRUE,"ndfin-fatal",
                              "too many files in the cleanup list");
                }
                
                strcpy(cleanup_file_list[*num_cleanup_files],
                       dest_filename);
                (*num_cleanup_files)++;
            }
        }
        else if (*resolution > 0) /* header already found */
        {
            /* invalid ndf header, so assume it is either the report 
               file or the history file */
            if (!(*already_have_wor))
            {
                *already_have_wor = 1;
                *remaining_flags &= ~WOR_NOT_DONE;

                if (keep_wor)
                {
                    /* rename the file to the report file name */
                    sprintf(new_name,"%s.wo",base_filename);
                    if (renamefile(dest_filename,new_name,0L,0L,0L) !=
                        E_SUCC)
                    {
                        c_lasmsg(TRUE,"ndfin-fatal",
                                 "Error renaming %s to %s",
                                 dest_filename,new_name);
                    }

                    if (*num_files_created > MAX_CREATED_FILES)
                    {
                         c_lasmsg(TRUE,"ndfin-fatal",
                                  "too many files in the files created list");
                    }
                    
                    strcpy(files_created_list[*num_files_created],new_name);
                    (*num_files_created)++;
                }
                else 
                {
                    /* remove the file since not keeping the report 
                       file */
                    c_dkdel(dest_filename);
                }
            }
            else if (!(*already_have_hst))
            {
                *already_have_hst = 1;
                *remaining_flags &= ~HISTORY_NOT_DONE;

                /* must be history file */
                if (keep_hst)
                {
                    /* rename the file to the history file name */
                    sprintf(new_name,"%s.hi",base_filename);
                    if (renamefile(dest_filename,new_name,0L,0L,0L) 
                            != E_SUCC)
                    {
                        c_lasmsg(TRUE,"ndfin-fatal",
                                 "Error renaming %s to %s",
                                 dest_filename,new_name);
                    }

                    if (*num_files_created > MAX_CREATED_FILES)
                    {
                         c_lasmsg(TRUE,"ndfin-fatal",
                                  "too many files in the files created list");
                    }
                    
                    strcpy(files_created_list[*num_files_created],new_name);
                    (*num_files_created)++;
                }
                else
                {
                    /* remove the file since not keeping the history 
                       file */
                    c_dkdel(dest_filename);
                }
            }
            else
            {
                /* unexpected file */
                c_lasmsg(TRUE,"ndfin-fatal",
                         "Unexpected file read from tape");
            }
        }
        else
        {
            /* error since no headers found yet */
            c_lasmsg(TRUE,"ndfin-fatal",
                     "Unexpected file read from tape");
        }
    }

    return (E_SUCC);
}
