/*--------------------------------------------------------------------------
ROUTINES: mount_media, unmount_media

PURPOSE:
        Routines to mount and unmount the current media (tape and CD supported)

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

MOUNT_MEDIA:
INPUTS:
        vblk - TAE parameter block pointer
        oprmsg - operator messages for mounting media volumes
        volume - current volume number (starting from 1)

OUTPUTS:
        tape - tape flag (1=yes, 0=no)
        cd - cd flag (1=yes, 0=no)
        drive - Drive the media was mounted on
        device - Device the media was mounted on (mount point)
        mounted - mounted flag (1=mounted, 0=not mounted)
                  
ALGORITHM:
        Open the tape/CD drive
        Close the drive without taking it offline
        Set the mounted flag
        Extract the drive and device name
        Set the mounted flag, drive, and device names in the TAE block
            so the drive can be unmounted if a failure occurs
        

UNMOUNT_MEDIA:
INPUTS:
        vblk - TAE parameter block pointer
        drive - Drive to unmount
        
OUTPUTS:
        mounted - mounted flag (1=mounted, 0=not mounted)

ALGORITHM:
        Reopen the drive without rewinding to get the unit number
        Close the drive, taking it offline
        Clear the mounted flag and update the TAE block

NOTES:
none

--------------------------------------------------------------------------*/

#include <string.h> /* string function prototypes */
#include <unistd.h> /* gethostname prototype */
#include "las.h"    /* las header file */
#include "tapeio.h" /* tapeio header file */
#include "util.h"   /* c_up2low prototype */
#include "media_control.h" /* prototypes for this module */



int mount_media
(
    struct PARBLK *vblk,  /* I: TAE parameter block */
    char oprmsg[][CMLEN], /* I: mount messages */
    long volume,          /* I: volume to mount */
    int  *tape,           /* O: tape flag (1=yes, 0=no) */
    int  *cd,             /* O: cd flag (1=yes, 0=no) */
    char *drive,          /* O: drive media was mounted on */
    char *device,         /* O: device media was mounted on */
    int *mounted          /* O: mounted flag */
)
{
    /* LAS constants */
    long nonfatal = 1;  /* non-fatal message */
    long zero = 0;      /* Constant of zero */

    struct TAPEDESC *tdrive; /* tape descriptor structure */
    long access = TREAD;     /* Indicates tape access mode */
    long mode   = HANDLE;    /* Let tapeio handle tape errors*/
    long rewopt;             /* Rewind tape upon close */
    long skip = 0;           /* Files to skip upon tape open */
    long unit;               /* Tape unit number */
    int  status;             /* status returned by tapeio routines */
    char comment[CMLEN];     /* tape mount comment */
    char density[5] = "0";   /* Density of mounted tape */
    char *string_ptrs[1];    /* used to return variables to tae */
    char tape_id[CMLEN];     /* id of tape or CD mounted */
    
    /* issue a mount request */
    strcpy(comment,oprmsg[volume-1]);

    /* c_tpopen will accept option REWIND for tape or CD but will only
       verify this option if it is a tape */
    rewopt = REWIND; 

    status = c_tpopen(&unit, &skip, &mode, &rewopt, density, &access, 
                      comment, tape_id);
    if (status != TAPE_SUCC)
    {
        c_errmsg ("Error encountered mounting media", "ndfin-media", &nonfatal);
        return (E_FAIL);
    }

    /* retrieve the media type */
    tdrive = (struct TAPEDESC *)unit;

    if (las_is_tape(tdrive) == TRUE)
        *tape = 1;
    else
        *cd = 1; 
    
    /* close the drive without taking it offline so it can be 
       reopened later */
    rewopt = NOREW;
    if (c_tpclos(&unit, &mode, &rewopt, &zero) != TAPE_SUCC)
    {
        c_errmsg ("Error encountered closing media", "ndfin-media", &nonfatal);
        return (E_FAIL);
    } 
    
    /* set the mounted flag */
    *mounted = 1;
    
    /* get the drive name and device */
    strcpy(drive, comment);
    
    if (*tape)
    {
        /* tape, so get the device name from the unit structure */
        strcpy(device, tdrive->name);
    }
    else
    {
        /* If the DRIVE doesn't have an @ sign in it then this is a 
           local CD drive.  If the DRIVE does have an @ sign then this 
           is a remote drive.  If this is a local DRIVE, then we need 
           to get the hostname and compute the DRIVE from hostname + 
           @ + DRIVE.
        -------------------------------------------------------------*/
        /* create the device name for the CD drive */
        if (strchr(drive,'@') == NULL)
        {  /* This is a local tape drive */
            char lhost[CMLEN];
            long length;
            
            if (gethostname(lhost,CMLEN) != 0)
            {
                c_lasmsg(FALSE,"ndfin-hostname","Error reading the hostname");
                return (E_FAIL);
            }
            length = (long)strlen(lhost);
            c_up2low(lhost,&length);
            sprintf (device, "/%s@%s/", lhost, drive);
        }
        else
        {
            /* This is a remote tape drive */
            sprintf (device, "/%s/", drive);
        }
    }

/* debug code 
    c_lasmsg(FALSE,"mount-debug","drive = %s", drive);
    c_lasmsg(FALSE,"mount-debug","device = %s", device);
*/

    /* return the mounted flag, drive, and device right away so they
       can be unmounted in the pdf if needed */
    if (SUCCESS != q_intg(vblk,"MOUNTED",1,mounted,P_UPDATE))
    {
        c_errmsg("Error passing MOUNTED to TAE","ndfin-tae",&nonfatal);
        return (E_FAIL);
    }
    string_ptrs[0] = drive;
    if (q_string(vblk,"DRIVE",1,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing DRIVE to TAE","ndfin-tae",&nonfatal);
        return (E_FAIL);
    }
    string_ptrs[0] = device;
    if (q_string(vblk,"DEVICE",1,string_ptrs,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing DEVICE to TAE","ndfin-tae",&nonfatal);
        return (E_FAIL);
    }

    if (q_out(vblk) != SUCCESS)
    {
        c_errmsg("Error passing variables to TAE","ndfin-tae",&nonfatal);
        return (E_FAIL);
    }
        
    return (E_SUCC);
}


int unmount_media
(
    struct PARBLK *vblk, /* I: TAE parameter block */
    char *drive,         /* I: drive to unmount media */
    int *mounted        /* O: mounted flag */
)
{
    /* LAS constants */
    long nonfatal = 1;  /* non-fatal error flag */
    long zero = 0;      /* Constant of zero */
    
    long unit;             /* Tape unit number */
    long skip = 0;         /* Files to skip upon tape open */
    char density[5] = "0"; /* Density of mounted tape */
    long rewopt;           /* Rewind option */
    long mode   = HANDLE;  /* Let tapeio handle tape errors*/
    long access = TREAD;   /* Indicates tape access mode */
    
    /* reopen the drive to get the unit for the close call */
    rewopt = NOREW;
    if (c_tpreop(&unit, &skip, &mode, &rewopt, density, &access, drive) 
            != TAPE_SUCC) 
    {
        c_errmsg ("Error encountered reopening media", "ndfin-media",&nonfatal);
        return (E_FAIL);
    } 

    /* rewind the media and take it offline */
    rewopt = REWOFFL;
    if (c_tpclos(&unit, &mode, &rewopt, &zero) != TAPE_SUCC)
    {
        c_errmsg ("Error encountered closing media", "ndfin-media", &nonfatal);
        return (E_FAIL);
    } 

    /* update the state of the mounted flag and immediately write it to the
       tae block in case a failure is encountered later */
    *mounted = 0;
    if (SUCCESS != q_intg(vblk,"MOUNTED",1,mounted,P_UPDATE))
    {
        c_errmsg("Error passing MOUNTED to TAE","ndfin-tae",&nonfatal);
        return (E_FAIL);
    }
    if (q_out(vblk) != SUCCESS)
    {
        c_errmsg("Error passing variables to TAE","ndfin-tae",&nonfatal);
        return (E_FAIL);
    }

    return (E_SUCC);
}
