/*--------------------------------------------------------------------------
ROUTINE: read_ndf_header

PURPOSE:
        Read the header file from disk and return the required parameters
        to the calling routine.  At the same time, create a LAS image DDR
        and populate its fields with the data read from the NDF header.
        
INPUTS:
        INFILE - path to original input file
        HEADER_FILE - path to header file
        WINDOW and UNITS - parameters for the user specified window
        BANDS - bands to read
        NBANDS - band group definitions
        ODTYPE - output data type requested
        OUT - file names submitted by the user (indexed by NBANDS group)
        BAND_GROUP - group in NBANDS that a band belongs in
        OUT_FILES - list of output file names to use for the DDR, indexed by
                    band number
        DEM_OUT_FILE - output file for a DEM
        VOLUME - current volume number

OUTPUTS:
        DDR file with information from the header file
        OUT_WINDOW in line/sample space
        IDTYPE - Data type of the input image
        BANDS_AVAILABLE - List of bands available
        SOURCE_IMAGE_FILENAMES - file names of source bands
        LINES - lines in an input image band
        SAMPLES - samples across an input image band
        FORMAT - input image format (BSQ or BIL)
        BLOCK_FACTOR - blocking factor for input image
        RESOLUTION - current resolution number
        DATA_FILES - total number of data files on volume
        FILES_CREATED - files created by this routine
        TOTAL_BANDS - total number of bands in the input (for BIL images)

ALGORITHM:
        Read the TAE parameters
        Extract the resolution number from the header file name
        Read the common parameters from the NDF header file
        Convert the datum name into a datum number
        If not a DEM header
            Read the non-DEM parameters from the header file
            If no non-DEM bands requested
                Update the applicable TAE output parameters and return
            Read the name of each band from the header file
        Else
            Set the DEM flag
            Clear parameters not found in the DEM
            If DEM not requested
                Update the applicable TAE output parameters and return
        If NDF revision 2 and not a DEM
            Read the data file name for each band
            Extract the available band numbers from the data file names
        Else
            Set the bands available based on the start data file and the
                number of data files available
            If DEM
                Build the DEM data file name
            Else
                Build the non-DEM data file names
        If not a DEM
            If the user requested a specific set of bands
                Eliminate the bands not requested from the available bands list
                Sort the bands available to match the bands requested
            Else
                Sort the bands numerically
        If no bands remain in the bands available list
            Return the applicable parameters to the TAE block and exit
        If DEM
            Clear the acquisition date and time
        Else
            Extract the acquisition date and time from the header information
        Extract the input data type from the header file parameter
        If the user requested the SAME output data type
            Set the output data type to the input data type
        Extract the corners from the header information
        Setup the DDR structure with the proper data
        For each output file needed for the available bands
            Create the temporary output DDR name
            Add the DDR to the files created list
            Write the DDR
            If the window has not been calculated yet
                Clear the calc window flag
                If a window has been specified
                    Get the window units
                    If the units are not line/sample
                        Convert the window to line/sample
                    Else
                        If only the start line/sample of the window are spec'd
                            Calculate the number of lines/samples to the bottom
                                of the image
                        Convert the window to integers
                    Limit the window to the image extents
                    Recalculate the image corners using the new window
                    Write an updated DDR
                Else
                    Set the output window to the image size
                Write a Band DDR for each band in the output file
        Return the output parameters to the TAE block
        Return

NOTES:
none

--------------------------------------------------------------------------*/
#include <string.h> /* strcpy and strlen prototypes */
#include <ctype.h>  /* toupper, tolower, isupper, islower prototypes */
#include <stdlib.h> /* getenv prototype */
#include "las.h"    /* las header file */
#include "geompak.h"/* c_get_spheroid prototype */
#include "util.h"   /* c_inlas prototype */
#include "read_ndf_parameters.h" 
#include "recalc_corners.h"


/* Remove intermediate files created */
static void cleanup
(
    char files_to_remove[][CMLEN], /* I: list of names of files to delete */
    long num_files                 /* I: number of intermediate files */
)
{
    int count; /* counter */

    for (count=0; count<num_files; count++)
        unlink(files_to_remove[count]);
} /* cleanup() */


/* Remove the presence of "_vol#" from header filename, where '#' is one or
   more ASCII digits */
int cut_vol
(  
    char *root             /* I: root of output filename */
)
{
    char digs_only = 1;    /* flag (!0) indicates only digits found after 
                              "_vol" substring */
    int  i;                /* counter */
    char *ptr      = NULL; /* pointer to first occurrance of "_vol" 
                              substring */

    /* Validate input */
    if (root && *root)                  
    {
            /* IF found the "_vol" substring , AND root does not start 
               with substring */
            if ((ptr = strstr(root,"_vol")) && (ptr != root))             
            {
                /* Check each char to right of it to ensure it's a digit */
                /* There must be at least one digit to right */
                if (!ptr[4]) 
                    digs_only = 0;                 
                else
                {
                    for (i=4; ptr[i]; i++)                
                    {
                        if (!isdigit(ptr[i])) 
                        {
                            /* Flag first occurrance of non-digit */
                            digs_only = 0;                 
                            break;
                        }
                    }
                }

                /* IF "_vol" substring is followed by digits only */
                if (digs_only)                         

                    /* remove "_vol" substring along with digits */
                    *ptr = 0;                          
            }
    }
    return(E_SUCC);
} /* cut_vol() */


FUNCTION int getpar
(
    struct PARBLK *vblk, /* pointer to initial TAE parameter block */
    char *infile,        /* input file path name */
    char *header_file,   /* host filename of header file on disk */ 
    long *volume,        /* volume number (for generating multiple files) */
    long *od_type,       /* requested output type */
    double *window,      /* window parameter */
    long *num_window,    /* number of window parameters read */
    char *units,         /* units to use for the window */
    long *bands,         /* list of bands to look for */
    long *num_bands,     /* entries in bands */
    long *nbands,        /* nbands parameter */
    long *num_nbands,    /* entries in nbands */
    long *group_of_band, /* group in nbands a band is grouped with */
    char out_files[][CMLEN], /* output file names */
    char *dem_out_file   /* dem output file name */
)
{
    struct VARIABLE *var;  /* ptr to a variable in the vblk */
    float temp_float;      /* temporary float value */
    int index;             /* loop counter */
    char tmpstr[CMLEN];    /* temporary string */

    /* Get the input file name
    --------------------------*/
    var = p_find(vblk, "INFILE");
    strcpy(infile,SVAL(*var,0));    
    
    /* Get the header file name
    ---------------------------*/
    var = p_find(vblk, "HEADER_FILE");
    strcpy(header_file,SVAL(*var,0));    
    
    /* Retrieve the WINDOW coordinates
    ----------------------------------*/
    var = p_find(vblk, "WINDOW");
    *num_window = var->v_count;
    if (*num_window > 4)
        *num_window = 4;
    for (index = 0; index < *num_window; index++)
    {
        temp_float = RVAL(*var,index);
        window[index] = (double)temp_float;
    }

    /* Retrieve the UNITS setting
    -----------------------------*/
    var = p_find(vblk, "UNITS");
    strncpy(units,SVAL(*var,0),4);

    /* Retrieve the BANDS
    ---------------------*/
    var = p_find(vblk, "BANDS");
    *num_bands = var->v_count;
    for (index = 0; index < var->v_count; index++)
        bands[index] = IVAL(*var,index);

    /* Retrieve the NBANDS parameter
    --------------------------------*/
    var = p_find(vblk, "NBANDS");
    *num_nbands = var->v_count;
    for (index = 0; index < var->v_count; index++)
        nbands[index] = IVAL(*var,index);

    /* Retrieve the BAND_GROUP parameter
    ------------------------------------*/
    var = p_find(vblk, "BAND_GROUP");
    for (index = 0; index < var->v_count; index++)
        group_of_band[index] = IVAL(*var,index);

    /* Retrieve the OUT_FILES parameter (host names)
    ------------------------------------------------*/
    var = p_find(vblk, "OUT_FILES");
    for (index = 0; index < var->v_count; index++)
        strcpy(out_files[index],SVAL(*var,index));    

    /* Retrieve the DEM_OUT_FILE parameter (host name)
    ---------------------------------------------------*/
    var = p_find(vblk, "DEM_OUT_FILE");
    if (var->v_count > 0)
        strcpy(dem_out_file,SVAL(*var,0));    
    else
        strcpy(dem_out_file,"");

    /* Retrieve the volume number
    -----------------------------*/
    var = p_find(vblk, "VOLUME");
    *volume = IVAL(*var, 0);

    /* Retrieve the output data type
    --------------------------------*/
    var = p_find(vblk, "ODTYPE");
    strcpy(tmpstr,SVAL(*var,0));
    if (strcmp(tmpstr,"BYTE") == 0)
        *od_type = EBYTE;
    else if (strcmp(tmpstr,"I*2") == 0)
        *od_type = EWORD;
    else if (strcmp(tmpstr,"I*4") == 0)
        *od_type = ELONG;
    else if (strcmp(tmpstr,"R*4") == 0)
        *od_type = EREAL;
    else
    {
        /* default is the same */
        *od_type = 0;
    }
    
    return(E_SUCC);
}


/* routine to extract the corners */
int extract_corner
(
    char ascii_corner[][18],         /* ascii corner array 
                                        (long,lat,easting,northing) */
    double *northing_easting         /* returned northing/easting */
)
{
    if ((sscanf(ascii_corner[2],"%lf",&northing_easting[1]) != 1) ||
        (sscanf(ascii_corner[3],"%lf",&northing_easting[0]) != 1))
    {
        return (E_FAIL);
    }

    return (E_SUCC);
}

void return_early_bailout_values
(
    struct PARBLK *vblk, /* I: pointer to initial TAE parameter block */
    int data_files,      /* I: data files specified in this header */
    int total_bands,     /* I: total bands specified in the current header */
    char *interleave     /* I: type of data interleaving (BSQ or BIL) */
)
{
    int bands_available = 0; /* no bands available */
    long nonfatal = 1;  /* non-fatal message */
    long fatal = -1;    /* fatal error flag */
    char *string_ptrs[1];  /* points to tae string return parameters */

    /* Return the no bands in bands available output parameter to TAE */
    if (q_intg(vblk,"BANDS_AVAIL",0,&bands_available,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing BANDS_AVAIL to TAE","ndfin-tae",
                 &nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
    
    /* return the number of data files so tape files can be skipped */
    if (q_intg(vblk,"DATA_FILES",1,&data_files,P_UPDATE)
        != SUCCESS)
    {
        c_errmsg("Error passing DATA_FILES to TAE","ndfin-tae",
                 &nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* return the number of bands so they can be skipped */
    if (q_intg(vblk,"TOTAL_BANDS",1,&total_bands,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing TOTAL_BANDS to TAE","ndfin-tae",
                 &nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* return the interleave so we know whether to skip a single file or
       total_bands files */
    string_ptrs[0] = interleave;
    if (q_string(vblk,"DLEAVE",1,string_ptrs,P_UPDATE) != SUCCESS)
    {
        c_errmsg("Error passing DLEAVE to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }  

    if (q_out(vblk) != SUCCESS)
    {
        c_errmsg("Error passing variables to TAE","ndfin-tae",
                 &nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
}

int main(void)
{
    /* LAS constants */
    long nonfatal = 1;  /* non-fatal message */
    long fatal = -1;    /* fatal error flag */
    long std_out = 0;   /* standard out */

    /* variables for function */
    struct PARBLK vblk;         /* TAE parameter block */
    double window[4]={0.0,0.0,0.0,0.0}; /* user specified window */
    long volume;                /* volume number */
    long id_type;               /* input data type */
    long od_type;               /* output data type */
    long num_window;            /* number of entries in window array */
    long out_window[4];         /* calculated window */
    long bands[MAXBND];         /* band list */
    long num_bands;             /* number of entries in bands */
    long group_of_band[MAXBND]; /* NBANDS group for each band */
    int  position_on_tape[MAXBND]; /* position on tape of each band */
    long nbands[MAXBND];        /* number of bands in each group */
    long num_nbands;            /* number of entries in nbands */
    int index;                  /* loop counter */
    char units[4];              /* projection units */
    char out_files[MAXBND][CMLEN]; /* output file names */
    char infile[CMLEN] = "";    /* input file path */
    char header_file[CMLEN];    /* header file host name */
    char header_dir[CMLEN] = "";/* header file directory */
    char in_dir[CMLEN] = "";    /* infile directory */
    char dir[CMLEN] = "";       /* directory name */
    char root[CMLEN] = "";      /* root of file name */
    char ext[CMLEN] = "";       /* extension of file */    
    char ext_len;               /* string length of file extension */
    int  resolution;            /* resolution number of file */
    static const char months[12][4] =
        { "jan", "feb", "mar", "apr", "may", "jun", 
          "jul", "aug", "sep", "oct", "nov", "dec" }; /* month names */
    static float ndf_revision;  /* NDF header revision number */
    static char interleave[4];  /* data interleaving (BSQ or BIL) */
    static long start_data_file; /* starting data file number */
    static int  blocking_factor; /* blocking factor */
    static long projection_code; /* projection code */
    static long zone;            /* zone code */
    static int zone_valid;       /* zone code valid flag */
    static double proj_parms[15];/* projection parameters */
    static char ascii_datum[10]; /* datum name in ascii */
    static int  ascii_datum_valid; /* ascii datum valid flag */
    static double semi_major_axis; /* semi-major axis value */
    static int semi_major_valid;   /* semi-major axis valid flag */
    static double semi_minor_axis; /* semi-minor axis value */
    static int semi_minor_valid;   /* semi-minor axis valid flag */
    static char data_set_type[20]; /* data set type source */
    static char pixel_format[20];  /* pixel format (BYTE, 2BYTEINT, etc) */
    static int samples;            /* samples across image */
    static int lines;              /* lines in image */
    static int data_files;         /* number of data files in image */
    static long lines_per_volume;  /* lines of data per volume */
    static char ascii_upper_left_corner[4][18]; /* longitude,latitude,easting,*/
    static char ascii_upper_right_corner[4][18];/* northing of corners */
    static char ascii_lower_right_corner[4][18];
    static char ascii_lower_left_corner[4][18];
    static char acq_date_time[25]; /* acquisition date and time */
    static int acq_date_time_valid;/* acquisition date and time valid flag */
    static char satellite[32];     /* satellite data came from */
    static int satellite_valid;    /* satellite valid flag */
    static float pixel_size[2];    /* size of a pixel (x,y) */
    static char pixel_units[20];   /* pixel units */
    static long bands_in_volume;   /* number of bands in the volume */
    int  total_bands;              /* unmodified bands_in_volume for BIL files*/
    static long length;            /* string length */
    
    NDF_PARAMETER_DEF_TYPE common_params[] = 
    {
        {"NDF_REVISION",FLOAT_PARAM,&ndf_revision,sizeof(float),1,NULL,
          NDF_UNUSED},
        {"DATA_FILE_INTERLEAVING",STRING_PARAM,interleave,sizeof(interleave),1,
         NULL,NDF_UNUSED},
        {"START_DATA_FILE",LONG_PARAM,&start_data_file,sizeof(long),1,NULL, 
         NDF_UNUSED},
        {"BLOCKING_FACTOR",INT_PARAM,&blocking_factor,sizeof(int),1,NULL,
         NDF_UNUSED},
        {"USGS_PROJECTION_NUMBER",LONG_PARAM,&projection_code,sizeof(long),1,
         NULL,NDF_UNUSED},
        {"USGS_MAP_ZONE",LONG_PARAM,&zone,sizeof(long),1,&zone_valid,
         NDF_NO_WARNING},
        {"USGS_PROJECTION_PARAMETERS",DOUBLE_PARAM,proj_parms,sizeof(float),15,
         NULL,NDF_UNUSED},
        {"HORIZONTAL_DATUM",STRING_PARAM,ascii_datum,sizeof(ascii_datum),1,
         &ascii_datum_valid,NDF_ISSUE_WARNING},
        {"EARTH_ELLIPSOID_SEMI-MAJOR_AXIS",DOUBLE_PARAM,&semi_major_axis,
                sizeof(double),1,&semi_major_valid,NDF_ISSUE_WARNING},
        {"EARTH_ELLIPSOID_SEMI-MINOR_AXIS",DOUBLE_PARAM,&semi_minor_axis,
                sizeof(double),1,&semi_minor_valid,NDF_ISSUE_WARNING},
        {"DATA_SET_TYPE",STRING_PARAM,data_set_type,sizeof(data_set_type),1,
         NULL,NDF_UNUSED},
        {"PIXEL_FORMAT",STRING_PARAM,pixel_format,sizeof(pixel_format),1,NULL,
         NDF_UNUSED},
        {"PIXELS_PER_LINE",INT_PARAM,&samples,sizeof(int),1,NULL,
         NDF_UNUSED},
        {"LINES_PER_DATA_FILE",INT_PARAM,&lines,sizeof(int),1,NULL,
         NDF_UNUSED},
        {"LINES_PER_VOLUME",LONG_PARAM,&lines_per_volume,sizeof(long),1,NULL,
         NDF_UNUSED},
        {"UPPER_LEFT_CORNER",STRING_PARAM,ascii_upper_left_corner,18,4,NULL,
         NDF_UNUSED},
        {"UPPER_RIGHT_CORNER",STRING_PARAM,ascii_upper_right_corner,18,4,NULL,
         NDF_UNUSED},
        {"LOWER_RIGHT_CORNER",STRING_PARAM,ascii_lower_right_corner,18,4,NULL,
         NDF_UNUSED},
        {"LOWER_LEFT_CORNER",STRING_PARAM,ascii_lower_left_corner,18,4,NULL,
         NDF_UNUSED},
        {"PIXEL_SPACING",FLOAT_PARAM,pixel_size,sizeof(long),2,NULL,
         NDF_UNUSED},
        {"PIXEL_SPACING_UNITS",STRING_PARAM,pixel_units,sizeof(pixel_units),1,
         NULL,NDF_UNUSED},
        {"NUMBER_OF_BANDS_IN_VOLUME",LONG_PARAM,&bands_in_volume,sizeof(long),1,
         NULL,NDF_UNUSED}
    };
    NDF_PARAMETER_DEF_TYPE nondem_params[] =
    {
        {"NUMBER_OF_DATA_FILES",INT_PARAM,&data_files,sizeof(int),1,NULL,
         NDF_UNUSED},
        {"ACQUISITION_DATE/TIME",STRING_PARAM,acq_date_time,
                sizeof(acq_date_time),1,&acq_date_time_valid,NDF_ISSUE_WARNING},
        {"SATELLITE",STRING_PARAM,satellite,sizeof(satellite),1,
                &satellite_valid,NDF_ISSUE_WARNING}
    };
    double upper_left_corner[2];  /* northing/easting */
    double upper_right_corner[2];
    double lower_right_corner[2];
    double lower_left_corner[2];
    char band_names[MAXBND][CMLEN];   /* file name of each band */
    char source_names[MAXBND][CMLEN]; /* source name of each band */
    int source_names_valid[MAXBND];   /* source name valid flags */
    int param_count;                  /* count of parameters */
    NDF_PARAMETER_DEF_TYPE band_params[MAXBND]; 
                                      /* params for band specific info */
    int bands_available[MAXBND];
    long datum=0;               /* datum number */
    long datum_valid = 0;       /* datum number valid flag */
    struct DDR ddr;             /* DDR structure */
    struct BDDR bddr;           /* Band DDR structure */
    char image_name[CMLEN];     /* image file name */
    char *env_ptr;              /* pointer to environment variables */
    int band_index;             /* loop counter */
    int dem_flag = 0;           /* DEM header flag */
    char dem_out_file[CMLEN];   /* DEM output file name */
    char *string_ptrs[MAXBND];  /* points to tae string return parameters */
    int corners_valid;          /* corners valid flag */
    int calc_window = 1;        /* window needs calculating flag */
    char files_created[MAXBND][CMLEN]; /* list of files created */
    char *ext_ptr;              /* pointer to filename extension */
    char temp_in_path[CMLEN]="";/* Pathname created to check for existence */
    long num_files_created = 0; /* number of files in files created list */
    int bil_data = 0;           /* flag to indicate BIL data (1=BIL, 0=BSQ) */
    int reading_from_tape = 0;  /* flag to indicate if reading data from tape 
                                   (1=yes 0=no) */
   
    /* Initialize tae block
    -------------------------*/
    c_inlas (&vblk, &std_out);

    /* initialize the out_files array
    ---------------------------------*/
    memset(out_files, 0, MAXBND * CMLEN * sizeof(char));

    /* Get the parameters from the TAE variable block
    -------------------------------------------------*/
    if (getpar (&vblk, infile, header_file, &volume, &od_type, 
                window, &num_window, units, bands, &num_bands, nbands, 
                &num_nbands, group_of_band, out_files, dem_out_file)
                != E_SUCC)
    {
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* IF infile is NOT empty, we are reading from CD or disk, not tape
    -------------------------------------------------------------------*/
    if (*infile)
    {   
        /* Get the input directory 
        --------------------------*/
        c_splitname(infile,in_dir,root,ext);

        reading_from_tape = 0;
    }
    /* ELSE we are reading from tape
    --------------------------------*/
    else
        reading_from_tape = 1;

    /* read the common parameters from the NDF header file 
    ------------------------------------------------------*/
    param_count = sizeof(common_params)/sizeof(NDF_PARAMETER_DEF_TYPE);
    if (read_ndf_parameters(header_file, common_params, param_count) != E_SUCC)
    {
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
    }
    
    /* Get the current resolution from the header file name (for H#, 
       resolution is #, for HD and DH, resolution is zero), unless
       this is pre-PDS v2.0, before multi-resolution images were supported
       (resolution is always 0; the # in H# indicates the volume the
       header file is on).
    ----------------------------------------------------------------------*/
    c_splitname(header_file,header_dir,root,ext);
    ext_len = strlen(ext);
    if (  (strcmp(ext,"dh") == 0) || (strcmp(ext,"hd") == 0) ||
          (strcmp(ext,"DH") == 0) || (strcmp(ext,"HD") == 0) ||
          (ndf_revision < 2.0) )
    {
        resolution = 0;
    }
    else
        resolution = atoi(&ext[1]);

    /* set the bil_data flag */
    if (strcmp(interleave,"BIL") == 0)
    {
        bil_data = 1;
    }
   
    /* save the total bands in the volume */
    total_bands = bands_in_volume;
    
    /* Convert the parameters read into the needed representation
    -------------------------------------------------------------*/
    /* get the datum number */
    if (strcmp(ascii_datum,"ELLIPSOID") != 0)
    {
        if (ascii_datum_valid)
        {
            if (c_dtmnum(ascii_datum,&datum) == E_SUCC)
            {
                /* c_dtmnum returns -1 if datum not found */
                if (datum != -1)
                    datum_valid = 1;
                else
                    datum = 0; /* otherwise dspddr dies eventhough invalid*/
            }
        }
    }
    else
    {
        /* ELLIPSOID datum, so enter 0 for datum number */
        if (semi_major_valid && semi_minor_valid)
        {
            if (c_get_spheroid(semi_major_axis,semi_minor_axis,&datum,
                               &datum_valid) != E_SUCC)
            {
                c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
            }
        }
    }
    if (!datum_valid)
        c_lasmsg(FALSE,"ndfin-datum","Warning: datum in NDF header is invalid");
    
    /* get the non-DEM parameters if not a DEM 
    ------------------------------------------*/
    if (strcmp(data_set_type,"NLAPS_DEM") != 0)
    {
        /* read the non-DEM parameters from the NDF header file 
        -------------------------------------------------------*/
        param_count = sizeof(nondem_params)/sizeof(NDF_PARAMETER_DEF_TYPE);
        if (read_ndf_parameters(header_file, nondem_params, param_count) 
                != E_SUCC)
        {
            c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
        }
    
        /* if no output file defined, assume the data isn't wanted */
        if (strcmp(out_files[0],"") == 0)
        {
            return_early_bailout_values(&vblk,data_files, total_bands, 
                                        interleave);
            return (E_SUCC);
        }

        for (index = 0; index < 2; index++)
        {
            int band;
            int start_band_index;

            if (index == 0)
            {
                /* first attempt to read band names, so look for band names
                   starting with 0 (newer style PDS products) */
                start_band_index = 1;
            }
            else
            {
                /* on second pass, look for old style PDS (multivolumes 
                   continue using band numbers instead of starting over */
                start_band_index = start_data_file;
            }
            /* build the parameter list for the name of each band */
            for (band = 0; band < bands_in_volume; band++)
            {
                sprintf(band_params[band].name,"BAND%d_NAME",
                        band + start_band_index);
                band_params[band].param_type = STRING_PARAM;
                band_params[band].dest = source_names[band];
                band_params[band].dest_size = CMLEN;
                band_params[band].num_items = 1;
                band_params[band].valid_flag_ptr = &source_names_valid[band];
                band_params[band].issue_warning = NDF_ISSUE_WARNING;
            }
            
            /* read the band names from the header file */
            if (read_ndf_parameters(header_file,band_params,bands_in_volume)
                != E_SUCC)
            {
                c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
            }

            /* bail out if first name was found successfully */
            if (source_names_valid[0])
                break;
        }
    }
    else
    {
        /* flag the file as a DEM */
        dem_flag = 1;
        
        /* clear the parameters not available in the DEM header name */
        data_files = 1;
        strcpy(acq_date_time,"");
        strcpy(satellite,"");
        satellite_valid = 0;
        strcpy(source_names[0],"");
        source_names_valid[0] = 0;
        
        /* if no DEM output file defined, assume the DEM isn't wanted */
        if (strcmp(dem_out_file,"") == 0)
        {
            return_early_bailout_values(&vblk, data_files, total_bands, 
                                        interleave);
            return (E_SUCC);
        }
    }
    
    /* get the bands available and their data file names
    ----------------------------------------------------*/
    if ((ndf_revision >= 2.0) && (!dem_flag))
    {
        /* ndf version 2 or higher and not a DEM header */
        /* create the file names to look for */
        for (index = 0; index < bands_in_volume; index++)
        {
            sprintf(band_params[index].name,"BAND%d_FILENAME",
                    index + 1);
            band_params[index].param_type = STRING_PARAM;
            band_params[index].dest = band_names[index];
            band_params[index].dest_size = CMLEN;
            band_params[index].num_items = 1;
            band_params[index].valid_flag_ptr = NULL;
            band_params[index].issue_warning = NDF_UNUSED;
        }
    
        /* read the band file names from the header file */
        if (read_ndf_parameters(header_file,band_params,bands_in_volume)
            != E_SUCC)
        {
            c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
        }

        /* if BIL data interleave, set the bands available according to
           the bands_in_volume.  For BSQ, get the available bands from the 
           band file names */
        if (bil_data)
        {
            /* BIL - generate the bands available */
            for (index = 0; index < bands_in_volume; index++)
            {
                bands_available[index] = index+1;
                position_on_tape[index] = 1;
            }

            /* adjust the number of lines to be just one band instead
               of all the bands in the volume */
            lines = lines / bands_in_volume;
        }
        else
        {
            /* BSQ - extract the band numbers from the file names */
            for (index = 0; index < bands_in_volume; index++)
            {
                int loop;
                int bad_name = 0;

                c_splitname(band_names[index],dir,root,ext);

                ext_len = strlen(ext);
                if (ext_len < 2)
                    bad_name = 1;  /* not enough characters in extension */
                else
                {
                    for (loop = 1; loop < ext_len; loop++)
                    {
                        if (!isdigit(ext[loop]))
                        {
                            bad_name = 1;
                            break;
                        }
                    }
                }
                if (bad_name)
                {
                    /* file name extension is illegal */
                    c_lasmsg(FALSE,"ndfin-filename","Filename %s read from "
                        "%s is not legal",band_names[index],header_file);
                    c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
                }
                
                bands_available[index] = atoi(&ext[1]); 
                position_on_tape[index] = index + 1;
            }
        }
    }
    else   /* ELSE ( (ndf_revision < 2.0) || dem_flag) */
    {
        if (bil_data)
        {
            /* BIL - generate the bands available */
            for (index = 0; index < bands_in_volume; index++)
            {
                bands_available[index] = index+1;
                position_on_tape[index] = 1;
            }

            /* adjust the number of lines to be just one band instead
               of all the bands in the volume */
            lines = lines / bands_in_volume;
        }
        else
        {
            /* revision 1.0 or DEM data, so set the bands available according to
               the number of data files parameter */
            for (index = 0; index < bands_in_volume; index++)
            {
                bands_available[index] = index + start_data_file;
            }
        }
            
        /* if a DEM header, create the input file name by changing the 
           extension to .dd or .DD (depending on case of original name) */
        if (dem_flag)
        {
            c_splitname(infile,dir,root,ext);
            strcpy(band_names[0],root);
            c_splitname(header_file,dir,root,ext);
            if (ext[1] == 'H')
                ext[1] = 'D';
            else
                ext[1] = 'd';
            sprintf(band_names[0],"%s.%s",band_names[0],ext);
            position_on_tape[0] = 1;
        }
        else
        {
            /* set band file names to NDF revision 1 names */
            c_splitname(infile,dir,root,ext);

            /* Remove the presence of "_vol#" from header filename,  
               where '#' is one or more ASCII digits */
            cut_vol(root);

            for (index = 0; index < bands_in_volume; index++)
            {
                if (bil_data)
                {
                    sprintf(band_names[index],"%s.i1",root);
                }
                else
                {
                    sprintf(band_names[index],"%s.i%d",root,
                            bands_available[index]);
                    position_on_tape[index] = index + 1;
                }
            }
        }
    }

    /* if not a DEM, either order the bands available in the same order as 
       the requested bands, or if no specific bands were requested, put the
       bands in numerical order 
    -----------------------------------------------------------------------*/
    if (!dem_flag)
    {
        if (num_bands > 0)
        {
            int requested;  /* requested band loop counter */
            int next_index; /* next index to insert at */
            long temp;      /* temp long variable */
            char temp_name[CMLEN]; /* temp name string */
            
            /* eliminate the bands not requested from the available bands 
               list */
            for (index = 0; index < bands_in_volume; index++)
            {
                for (requested = 0; requested < num_bands; requested++)
                {
                    if (bands[requested] == bands_available[index])
                        break;
                }
                if (requested >= num_bands) /* band not wanted */
                {
                    bands_available[index] = 0;
                }
            }

            /* compress the bands_available and band_names lists to remove the 
               unwanted entries */
            next_index = 0;
            for (index = 0; index < bands_in_volume; index++)
            {
                if (bands_available[index] != 0)
                {
                    /* band to keep, so compress the list if needed */
                    if (next_index != index)
                    {
                        bands_available[next_index] = bands_available[index];
                        strcpy(band_names[next_index],band_names[index]);
                        strcpy(source_names[next_index],source_names[index]);
                        source_names_valid[next_index] = 
                                source_names_valid[index];
                        position_on_tape[next_index] = position_on_tape[index];
                    }
                   
                    /* update the next list compression index */
                    next_index++;
                }
                
            }

            /* update the number of bands in the volume */
            bands_in_volume = next_index;
        
            /* set the order of bands_available the same as the bands 
               requested */
            next_index = 0;
            for (requested = 0; requested < num_bands; requested++)
            {
                for (index = requested; index < bands_in_volume; index++)
                {
                    if (bands[requested] == bands_available[index])
                    {
                        if (next_index != index)
                        {
                            /* swap the fields since they're not in the correct
                               order right now */
                            temp = bands_available[next_index];
                            bands_available[next_index] =bands_available[index];
                            bands_available[index] = temp;
                            temp = position_on_tape[next_index];
                            position_on_tape[next_index] =
                                    position_on_tape[index];
                            position_on_tape[index] = temp;
                            strcpy(temp_name,band_names[next_index]);
                            strcpy(band_names[next_index],band_names[index]);
                            strcpy(band_names[index],temp_name);
                            strcpy(temp_name,source_names[next_index]);
                            strcpy(source_names[next_index],
                                   source_names[index]);
                            strcpy(source_names[index],temp_name);
                            temp = source_names_valid[next_index];
                            source_names_valid[next_index] = 
                                    source_names_valid[index];
                            source_names_valid[index] = temp;
                        }

                        next_index++;
                        break;
                    }
                }
            }
        }
        else
        {
            /* no bands requested, so take them all and put them in numerical
               order */
            long temp;
            char temp_name[CMLEN];
            int search;

            /* no bands requested by the user, so sort the bands numerically */
            for (index = 0; index < bands_in_volume-1; index++)
            {
                for (search = index + 1; search < bands_in_volume; search++)
                {
                    if (bands_available[search] < bands_available[index])
                    {
                        /* swap the bands since they are out of order */
                        temp = bands_available[search];
                        bands_available[search] =bands_available[index];
                        bands_available[index] = temp;
                        temp = position_on_tape[search];
                        position_on_tape[search] = position_on_tape[index];
                        position_on_tape[index] = temp;
                        strcpy(temp_name,band_names[search]);
                        strcpy(band_names[search],band_names[index]);
                        strcpy(band_names[index],temp_name);
                        strcpy(temp_name,source_names[search]);
                        strcpy(source_names[search], source_names[index]);
                        strcpy(source_names[index],temp_name);
                        temp = source_names_valid[search];
                        source_names_valid[search] = source_names_valid[index];
                        source_names_valid[index] = temp;
                    }
                }
            }
        }
    }

    /* if no bands available, exit 
    ------------------------------*/
    if (bands_in_volume <= 0)
    {
        return_early_bailout_values(&vblk, data_files, total_bands, interleave);
        return (E_SUCC);
    }

    /* extract the acquisition date and time using the correct format for
       the NDF revision 
    ---------------------------------------------------------------------*/
    if (dem_flag)
    {
        /* no acquisition date or time in DEM, so blank it out */
        strcpy(bddr.date,""); 
        strcpy(bddr.time,"");
    }
    else 
    {
        int month, day, year;
        int hour, minute, second;
        int valid_date = acq_date_time_valid;  /* assume valid date */
        
        if (valid_date)
        {
            if (ndf_revision >= 2.0)
            {
                /* revision 2 of NDF header, so date/time format is 
                   YYYY-MM-DDThh:mm:ssZ */
                if ((strlen(acq_date_time) == 20) && (acq_date_time[10] == 'T')
                    && (acq_date_time[19] == 'Z'))
                {
                    if (sscanf(acq_date_time,"%*c%*c%2d-%2d-%2dT%2d:%2d:%2d",
                               &year, &month, &day, &hour, &minute, &second) 
                               != 6)
                    {
                        valid_date = 0;
                    }
                }
                else
                    valid_date = 0;
            }
            else
            {
                /* revision 1 of NDF header, so date/time format is 
                   MMDDYY/hhmmssxx */
                if ((strlen(acq_date_time) == 15) && (acq_date_time[6] == '/'))
                {
                    if (sscanf(acq_date_time,"%2d%2d%2d/%2d%2d%2d",&month,&day,
                               &year, &hour, &minute, &second) != 6)
                    {
                        valid_date = 0;
                    }
                }
                else
                    valid_date = 0;
            }

            /* if any of the date/time values are illegal, clear the valid
               flag */
            if ((month < 1) || (month > 12) || (day < 1) || (day > 31) ||
                (hour < 0) || (hour > 23) || (minute < 0) || (minute > 59) ||
                (second < 0) || (second > 59))
            {
                valid_date = 0;
            }
        }
        
        if (valid_date)
        {
            /* valid date/time so add it to the band DDR */
            sprintf(bddr.date,"%02d-%3s-%02d", day, months[month-1], year);
            sprintf(bddr.time,"%02d%02d:%02d", hour, minute, second);
        }
        else
        {
            /* clear the date/time since they are invalid */
            strcpy(bddr.date,""); 
            strcpy(bddr.time,"");
        }
    }

    /* extract the IDTYPE (input data type) in the input file 
    ---------------------------------------------------------*/
    if (strcmp(pixel_format,"BYTE") == 0)
    {
        id_type = EBYTE;
    }
    else if (strcmp(pixel_format,"2BYTEINT") == 0)
    {
        strcpy(pixel_format,"I*2");
        id_type = EWORD;
    }
    else if (strcmp(pixel_format,"4BYTEINT") == 0)
    {
        strcpy(pixel_format,"I*4");
        id_type = ELONG;
    }
    else if (strcmp(pixel_format,"REAL") == 0)
    {
        strcpy(pixel_format,"R*4");
        id_type = EREAL;
    }
    else
    {
        c_lasmsg(FALSE,"ndfin-header","Error retrieving IDTYPE of %s from "
                 "header file", pixel_format);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* set the od_type to the same as the id_type if "SAME" was 
       requested by user 
    -----------------------------------------------------------*/
    if (od_type == 0)
        od_type = id_type;

    /* extract the corners from the header file 
    -------------------------------------------*/
    corners_valid = 1;
    if (extract_corner(ascii_upper_left_corner, upper_left_corner) != E_SUCC)
        corners_valid = 0;
        
    if (extract_corner(ascii_upper_right_corner,upper_right_corner) != E_SUCC)
        corners_valid = 0;
        
    if (extract_corner(ascii_lower_right_corner,lower_right_corner) != E_SUCC)
        corners_valid = 0;
        
    if (extract_corner(ascii_lower_left_corner, lower_left_corner) != E_SUCC)
        corners_valid = 0;

    /* patch the corner points for special conditions */
    if (projection_code == 1)
    {
        /* issue a message if the zone code is not valid with a UTM proj */
        if (!zone_valid)
        {
            c_lasmsg (FALSE,"ndfin-data","Unable to locate USGS_MAP_ZONE");
        }
        else if (upper_left_corner[0] < 0.0)
        {
           /* UTM projection with valid zone, so make sure the zone code is 
              negative if below the equator */
            if (zone > 0)
                zone = -zone;
        }

        /* if this is a valid UTM zone then the first two projection
           parameters should be zeros.  it has been the case where the
           semi-major and semi-minor axes have been incorrectly inserted 
           in the first two projection parameters. */
        if (zone_valid)
        {
           proj_parms[0] = 0.0;
           proj_parms[1] = 0.0;
        }
    }
    else if (projection_code == 22)
    {
        double temp;
        
        /* SOM projection, so swap the Y-X coordinates and negate the new Y
           coordinate */
        temp = -upper_left_corner[1];
        upper_left_corner[1] = upper_left_corner[0];
        upper_left_corner[0] = temp;
        temp = -upper_right_corner[1];
        upper_right_corner[1] = upper_right_corner[0];
        upper_right_corner[0] = temp;
        temp = -lower_left_corner[1];
        lower_left_corner[1] = lower_left_corner[0];
        lower_left_corner[0] = temp;
        temp = -lower_right_corner[1];
        lower_right_corner[1] = lower_right_corner[0];
        lower_right_corner[0] = temp;
    }
    
    /* Set up the DDR structure
    ---------------------------*/
    
    /* set up the values for the DDR that are constant for all the files
       created from this NDF header file */
    ddr.nl = lines;
    ddr.ns = samples;
    ddr.dtype = od_type;  /* set to real value by translas */
    ddr.master_line = 1;
    ddr.master_sample = 1;
    ddr.proj_code = projection_code;
    /* if the zone isn't valid, default it to 62 which is the invalid zone 
       number */
    if (!zone_valid)
        zone = 62;
    ddr.zone_code = zone;
    ddr.datum_code = datum;
    
    /* get the system from the DATASYS environment variable */
    env_ptr = getenv("DATASYS");
    if (env_ptr != NULL)
        strncpy(ddr.system,env_ptr,12); /* 12 length */
    else
    {
        c_errmsg("DATASYS not defined","ndfin-env",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* Set the projection units the same as the pixel units from the NDF
       header file */
    if (c_string2unit(pixel_units) == E_FAIL)
    {
        c_lasmsg(FALSE,"ndfin-warn",
               "PIXEL_SPACING_UNITS, %s, are not supported in LAS",pixel_units);
        c_lasmsg(FALSE, "ndfin-valid", "Setting projection units to invalid");
        ddr.valid[4] = 0;
    }
    else
    {
        strncpy(ddr.proj_units,pixel_units,12);
        ddr.valid[4] = 1;
    }

    /* clear the last date/time used field */
    strcpy (ddr.last_used_date,"");
    strcpy (ddr.last_used_time,"");

    /* copy the projection parameters */
    for (index = 0; index < 15; index++)
        ddr.proj_coef[index] = proj_parms[index];
        
    /* set the corners in the DDR */
    ddr.upleft[0] = upper_left_corner[0];
    ddr.upleft[1] = upper_left_corner[1];
    ddr.loleft[0] = lower_left_corner[0];
    ddr.loleft[1] = lower_left_corner[1];
    ddr.upright[0] = upper_right_corner[0];
    ddr.upright[1] = upper_right_corner[1];
    ddr.loright[0] = lower_right_corner[0];
    ddr.loright[1] = lower_right_corner[1];

    /* Set the pixel distance */
    ddr.pdist_y = pixel_size[1];
    ddr.pdist_x = pixel_size[0];

    /* set the line and sample increments */
    ddr.line_inc = 1.0;
    ddr.sample_inc = 1.0;
    
    /* set the valid flags */
    ddr.valid[0] = 1; /* projection code */
    ddr.valid[1] = 1; /* zone code */
    ddr.valid[2] = datum_valid; /* datum code */
    ddr.valid[3] = 1; /* proj_coef */
    ddr.valid[4] = 1; /* ground units */
    ddr.valid[5] = 1; /* ground distance */
    ddr.valid[6] = corners_valid; /* corner coordinates */
    ddr.valid[7] = 1; /* line/sample increments */

    /* set up the values for the band DDR that are constant for all the bands
       in the DDR */
    bddr.valid = 0; /* min/max not valid */
    bddr.minval = 0.0;
    bddr.maxval = 0.0;
    if (satellite_valid)
        strncpy(bddr.source,satellite,32); 
    else
        strcpy(bddr.source,"");
    strcpy(bddr.direction,""); /* max size 64 */ /* direction of capture */

    /* write a DDR for each of the output images to create with the available 
       bands
    -------------------------------------------------------------------------*/
    band_index = 0;
    while (band_index < bands_in_volume)
    {
        int bands_in_group;
        int start_band = band_index;
        int end_band;
        int current_group = group_of_band[bands_available[start_band]-1];
       
        /* find the bands in the current output file */
        for (end_band = start_band+1; end_band < bands_in_volume; end_band++)
        {
            if (current_group != group_of_band[bands_available[end_band]-1])
                break;
        }
        bands_in_group = end_band - start_band;
        
        /* set the number of bands in the DDR */
        ddr.nbands = bands_in_group;
       
        /* get the output file name for the current group */
        if (!dem_flag)
        {
            /* not a DEM, so use out_files */
            strcpy(image_name,out_files[current_group-1]);

            /* add a resolution to the file name if needed */
            if (resolution > 0)
            {
                sprintf(image_name,"%s_r%d",image_name,resolution);
            }

            /* add a volume number to the image name */
            sprintf(image_name,"%s_vol%d_temp",image_name,volume);
        }
        else
        {
            /* a DEM, so use the dem_out_file name */
            strcpy(image_name,dem_out_file);
            strcat(image_name,"_temp");
        }

        /* add the DDR to the list of files created */
        strcpy(files_created[num_files_created],image_name);
        strcat(files_created[num_files_created],".ddr");
        num_files_created++;
        
        /* write the DDR */
        if (c_putddr(image_name,&ddr) != E_SUCC)
        {
            cleanup(files_created, num_files_created);
            c_lasmsg(FALSE,"ndfin-file","Error writing DDR %s",image_name);
            c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
        }
        
        /* Calculate the window the first time through the loop 
           if one was specified (delayed until now since the 
           routines used required a DDR)
        -------------------------------------------------------*/
        if (calc_window)
        {
            double corners[8]; /* corner array UL(Y,X), LL, UR, LR */
            long origsize[2];  /* NL,NS */
            
            /* flag the window has been calculated */
            calc_window = 0;
            if (num_window != 0)
            {
                /* window parameters specified */
                long lngunits = 0; /* window units */
               
                /* get the window units */
                if (!strncmp (units, "DEG", 3))
                  lngunits = 4;
                else if (!strncmp (units, "DMS", 3))
                  lngunits = 5;
                else if (!strncmp (units, "PRO", 3))
                  lngunits = 2;

                /* if units are a projection, convert it to line/sample */
                if (lngunits != 0)
                {
                    if (c_wndconv (image_name, window, &lngunits) != E_SUCC)
                    {
                        cleanup(files_created, num_files_created);
                        c_errmsg("Fatal error encountered","ndfin-fatal",
                                 &fatal);
                    }        
                }
                else 
                {
                    /* window in line/sample, truncate the window to be
                       integers and if whole window wasn't specified
                       set the rest now */
                    window[0] = (long)window[0];
                    window[1] = (long)window[1];
                    if (num_window == 4)
                    {
                        /* whole window specified, so cast the rest to int */
                        window[2] = (long)window[2];
                        window[3] = (long)window[3];
                    }
                    else
                    {
                        /* using line sample and the whole window wasn't 
                           specified, so fill in the last parameters */
                        window[2] = lines - (long)window[0] + 1;
                        window[3] = samples - (long)window[1] + 1;
                        
                    }
                }
                
                /* check the window against the image extents and put the 
                   window in out_window */
                if (c_wndchk (image_name, window, out_window) != E_SUCC)
                {
                    cleanup(files_created, num_files_created);
                    c_errmsg("Fatal error encountered","ndfin-fatal",
                                 &fatal);
                }

                /* recalculate the corners for the output image and update the
                   ddr */
                corners[0] = upper_left_corner[0];
                corners[1] = upper_left_corner[1];
                corners[2] = lower_left_corner[0];
                corners[3] = lower_left_corner[1];
                corners[4] = upper_right_corner[0];
                corners[5] = upper_right_corner[1];
                corners[6] = lower_right_corner[0];
                corners[7] = lower_right_corner[1];
                origsize[0] = lines;
                origsize[1] = samples;
                recalc_corners(corners,origsize,out_window);
                ddr.upleft[0] = corners[0];
                ddr.upleft[1] = corners[1];
                ddr.loleft[0] = corners[2];
                ddr.loleft[1] = corners[3];
                ddr.upright[0] = corners[4];
                ddr.upright[1] = corners[5];
                ddr.loright[0] = corners[6];
                ddr.loright[1] = corners[7];

                /* update the master line/sample and NL, NS fields */
                ddr.master_line = out_window[0];
                ddr.master_sample = out_window[1];
                ddr.nl = out_window[2];
                ddr.ns = out_window[3];
                
                /* write the ddr out with the updated information */
                if (c_putddr(image_name,&ddr) != E_SUCC)
                {
                    cleanup(files_created, num_files_created);
                    c_lasmsg(FALSE,"ndfin-file","Error writing DDR %s",
                             image_name);
                    c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
                }
                
            }
            else
            {
                /* Set the window size to the image size */
                out_window[0] = 1;
                out_window[1] = 1;
                out_window[2] = lines;
                out_window[3] = samples;
            }
        }

        /* write a BDDR (band ddr) record for each band in the current 
           group
        --------------------------------------------------------------*/
        for (index = start_band; index < end_band; index++)
        {
            /* set the band number */
            bddr.bandno = index - start_band + 1;
            
            /* set the instrument name */
            if (source_names_valid[index])
                strncpy(bddr.instrument,source_names[index],32);
            else
                strcpy(bddr.instrument,"");
          
            /* write the bddr record */
            if (c_putbdr(image_name,&bddr) != E_SUCC)
            {
                cleanup(files_created, num_files_created);
                c_lasmsg(FALSE,"ndfin-file","Error writing band DDR %s", 
                         image_name);
                c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
            }
        }

        /* update the band index to the next group */
        band_index = end_band;

        /* set the last updated time in the ddr */
        if (c_lused(image_name) != E_SUCC)
        {
            cleanup(files_created, num_files_created);
            c_lasmsg(FALSE,
                     "ndfin-file","Error setting last used time in DDR %s", 
                     image_name);
            c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
        }
    }

    /* Return the output parameters to TAE
    ---------------------------------------*/
    /* return the bands available in the header */
    if (q_intg(&vblk,"BANDS_AVAIL",bands_in_volume,bands_available,P_UPDATE)
        != SUCCESS)
    {
        cleanup(files_created, num_files_created); 
        c_errmsg("Error passing BANDS_AVAIL to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* return the position on tape of each band */
    if (q_intg(&vblk,"TAPE_POS",bands_in_volume,position_on_tape,P_UPDATE)
        != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing TAPE_POS to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* return the IDTYPE */
    string_ptrs[0] = pixel_format;
    if (q_string(&vblk,"IDTYPE",1,string_ptrs,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing IDTYPE to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
    
    /* return the data interleave */
    if ((strcmp(interleave,"BSQ") != 0) && (strcmp(interleave,"BIL") != 0))
    {
        cleanup(files_created, num_files_created);
        c_lasmsg(FALSE,"ndfin-header","Error retrieving DLEAVE of %s from "
                 "header file", interleave);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
    string_ptrs[0] = interleave;
    if (q_string(&vblk,"DLEAVE",1,string_ptrs,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing DLEAVE to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }  

    /* return the resolution number available in the header */
    if (q_intg(&vblk,"RESOLUTION",1,&resolution,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing RESOLUTION to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
    
    /* return the lines in the image */
    if (q_intg(&vblk,"LINES",1,&lines,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing LINES to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
    
    /* return the samples in the image */
    if (q_intg(&vblk,"SAMPLES",1,&samples,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing SAMPLES to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
   
    /* return the band file names */
    for (index = 0; index < bands_in_volume; index++)
    {
        /* IF not reading from tape, construct full input pathname
               (Tape processing takes no input directory) */
        if (!reading_from_tape)
        {
            strcpy(temp_in_path, in_dir);
            strcat(temp_in_path, band_names[index]);
        }

        /* IF reading from tape OR input path found, use band name */
        if (reading_from_tape || (access(temp_in_path, F_OK) == 0) )
            string_ptrs[index] = band_names[index];

        /* ELSE change filename case to whatever extension is not,
           and then use this new band name, if found */
        else 
        {
            ext_ptr = strrchr(band_names[index], '.');
            if (ext_ptr == NULL) 
            {
                cleanup(files_created, num_files_created);

                /* file name extension is illegal */
                c_lasmsg(TRUE,"ndfin-filename","Filename %s read from "
                    "%s is not legal",band_names[index],header_file);
            }
            ext_ptr++;
            length = (long)strlen(band_names[index]);
            if (isupper(*ext_ptr))
            {
                c_up2low(band_names[index], &length);
            }         
            else if (islower(*ext_ptr))
            {
                c_low2up(band_names[index], &length);
            }         
            else /* file name extension is illegal */
            {
                cleanup(files_created, num_files_created);
                c_lasmsg(TRUE,"ndfin-filename","Filename %s read from "
                    "%s is not legal",band_names[index],header_file);
            }
            strcpy(temp_in_path, in_dir);
            strcat(temp_in_path, band_names[index]);
            if (access(temp_in_path, F_OK) == 0)
                string_ptrs[index] = band_names[index];
            else
            {
                cleanup(files_created, num_files_created);

                /* error opening the file */
                c_lasmsg(TRUE,"ndfin-file","Unable to open %s",temp_in_path);
            }
        } 
    }
    if (q_string(&vblk,"IN_BAND_NAMES",bands_in_volume,string_ptrs,P_UPDATE) 
            != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing IN_BAND_NAMES to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }  
    
    /* return the DEM_FLAG */
    if (q_intg(&vblk,"DEM_FLAG",1,&dem_flag,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing DEM_FLAG to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
  
    /* return the window in line/sample space */
    if (q_intg(&vblk,"OUT_WINDOW",4,(int *)out_window,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing BANDS_AVAIL to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
  
    /* return the blocking factor */
    if (q_intg(&vblk,"BLOCK_FACTOR",1,&blocking_factor,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing BLOCK_FACTOR to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }

    /* return the number of data files */
    if (q_intg(&vblk,"DATA_FILES",1,&data_files,P_UPDATE)
        != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing DATA_FILES to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
   
    /* return the files created names */
    for (index = 0; index < num_files_created; index++)
        string_ptrs[index] = files_created[index];
    if (q_string(&vblk,"FILES_CREATED",num_files_created,string_ptrs,P_UPDATE) 
            != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing FILES_CREATED to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
    
    /* return the number of data files */
    if (q_intg(&vblk,"TOTAL_BANDS",1,&total_bands,P_UPDATE) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing TOTAL_BANDS to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
   
    /* return the output block */
    if (q_out(&vblk) != SUCCESS)
    {
        cleanup(files_created, num_files_created);
        c_errmsg("Error passing variables to TAE","ndfin-tae",&nonfatal);
        c_errmsg("Fatal error encountered","ndfin-fatal",&fatal);
    }
    
    return (E_SUCC);
}

