/*--------------------------------------------------------------------------
ROUTINE: read_ndf_parameters

PURPOSE:
        Read an NDF header file and retrieve a list of parameters.

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        filename - name of NDF header file to read
        param_list - array of parameters to read from the header file
        num_parameters - number of entries in the param_list
        
OUTPUTS:
        param_list - the parameters read are returned to the addresses 
                     specified by the list
        returns - E_SUCC when a valid datum is found
                  E_FAIL when no valid datum is found
                  
ALGORITHM:
        Allocate a buffer for reading the entire NDF header file
        Read the header file into the buffer
        For each parameter in the list
            Search for the parameter and extract the value
        Free the buffer
        
NOTES:
none

--------------------------------------------------------------------------*/

#include <stdlib.h>      /* atof, atoi, atol protottypes */
#include <string.h>      /* string function prototypes */
#include "las.h"         /* las type defines */
#include "get_file_size.h" /* get_file_size prototype */
#include "read_ndf_parameters.h" /* prototype for this module */

int read_ndf_parameters
(
    const char *filename,     /* I: file to read parameters from */
    NDF_PARAMETER_DEF_TYPE *param_list, 
                              /* I/O: pointer to parameter definition list */
    int num_parameters        /* I: number of parameters in list */
)
{
    /* LAS constants */
    long nonfatal = 1;  /* non-fatal message */
    
    /* variables */
    FILE *file;       /* pointer to the header file */
    long buffer_size; /* size of buffer allocated for the header file */
    long file_size;   /* number of bytes in header file */
    int index;        /* loop index */
    char *buffer;     /* pointer to the buffer for the header file */
    
    /* Determine the size of the parameter file
    -------------------------------------------*/
    if (get_file_size(filename,&file_size) != E_SUCC)
    {
        c_lasmsg(FALSE,"ndfin-file","Error getting size of %s", filename);
        return (E_FAIL);
    }
    buffer_size = file_size + 1;
    
    /* Allocate a buffer large enough to store the parameter file
    -------------------------------------------------------------*/
    buffer = (char *)malloc (buffer_size * sizeof(char));
    if (buffer == NULL)
    {
        c_errmsg("Error allocating dynamic memory","ndfin-alloc", &nonfatal);
        return (E_FAIL);
    }
    
    /* Open the parameter file 
    --------------------------*/
    file = fopen(filename,"rt");
    if (file == NULL)
    {
        free(buffer);
        c_lasmsg(FALSE,"ndfin-file","Unable to open %s",filename);
        return (E_FAIL);
    }

    /* Read the entire parameter file into the buffer
    -------------------------------------------------*/
    if (fread(buffer, sizeof(char), file_size, file) != file_size)
    {
        free(buffer);
        fclose(file);
        c_lasmsg(FALSE,"ndfin-file","Error reading %s",filename);
        return (E_FAIL);
    }
    buffer[file_size] = '\0'; /* terminate the buffer for string searches */

    /* Close the parameter file
    ---------------------------*/
    fclose (file);
    
    /* Locate each parameter and parse it into the destination memory 
    -----------------------------------------------------------------*/
    for (index = 0; index < num_parameters; index++)
    {
        char *param_name_ptr;
        char *value_ptr;
        int count;
        char cur_char;
        int items;
        char temp_string[500];
        
        /* look for the current parameter in the buffer */
        param_name_ptr = strstr (buffer,param_list[index].name);
        if (param_name_ptr == NULL)
        {
            if (param_list[index].valid_flag_ptr == NULL)
            {
                /* issue a message that the item wasn't found */
                c_lasmsg (FALSE,"ndfin-data","Unable to locate %s",
                          param_list[index].name);
                /* Required parameter wasn't found, so fatal */
                return (E_FAIL);
            }
            else
            {
                if (param_list[index].issue_warning)
                {
                    /* issue a warning message that the item wasn't found */
                    c_lasmsg (FALSE,"ndfin-data","Unable to locate %s",
                              param_list[index].name);
                }
                /* clear the valid flag */
                *param_list[index].valid_flag_ptr = 0;
                continue;
            }
        }

        /* find the '=' sign following the parameter name and step past it */
        count = 0;
        cur_char = param_name_ptr[count];
        while ((cur_char != ';') &&  (cur_char != '\0'))
        {
            count++;
            cur_char = param_name_ptr[count];
        }

        /* extract the string up to and including the ending ';' */
        strncpy(temp_string,param_name_ptr,count+1);

        /* start the tokenizing process by finding the '=' */
        value_ptr = strtok(temp_string,"=");

        /* tokenize the string using ',' or ';' as token delimiters to 
           extract the values */
        count = 0;
        items = 0;
        while ((value_ptr=strtok(NULL,",;")) && 
               (items < param_list[index].num_items))
        {
            int len;
            /* convert the current value to the desired type */
            switch (param_list[index].param_type) 
            {
                case STRING_PARAM:
                    /* limit the length to the size available */
                    len = strlen(value_ptr);
                    if (len > param_list[index].dest_size - 1)
                        len = param_list[index].dest_size - 1;

                    strncpy(&((char *)param_list[index].dest)[items * 
                                param_list[index].dest_size], value_ptr, len);
                    items++;
                    break;
                case DOUBLE_PARAM:
                    ((double*)param_list[index].dest)[items] = atof(value_ptr);
                    items++;
                    break;
                case FLOAT_PARAM:
                    ((float*)param_list[index].dest)[items] = atof(value_ptr);
                    items++;
                    break;
                case LONG_PARAM:
                    ((long*)param_list[index].dest)[items] = atol(value_ptr);
                    items++;
                    break;
                case INT_PARAM:
                    ((int*)param_list[index].dest)[items] = atoi(value_ptr);
                    items++;
                    break;
            }
        }
        
        /* set the valid flag if one was passed in */
        if (param_list[index].valid_flag_ptr != NULL)
            *param_list[index].valid_flag_ptr = 1;
    }
    
    /* Free the buffer
    ------------------*/
    free (buffer);

    return (E_SUCC);
}

