/******************************************************************************

NAME: recalc_corners

PURPOSE: Recalculate the corner points for a subwindow of an image.

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        corners - projection corners in order of UL,LL,UR,LR in (Y,X) order
                  (i.e. UL-Y, UL-X, LL-Y,...)
        origsize - number of lines and samples in the image
        window - image window the corners are being recalculated for 
                 (SL, SS, NL, NS)

OUTPUTS:
        corners - projection corners updated with the corners of the window
                  specified
    
ALGORITHM DESCRIPTION:
    Setup the image coordinates
    Call c_rtcoef to get the projection transformation coefficients
    Compute the new corner points for that subwindow

******************************************************************************/
#include "las.h"
#include "recalc_corners.h"

void recalc_corners
(
    double corners[8], /* I:O corner points of the image - UL,LL,UR,LR 
                              with each corner specifed in (Y,X) */
    long origsize[2],  /* original size of the image NL,NS */
    long window[4]     /* image window specified */
)
{
    double coefs[6];              /* transformation coefficients              */
    long flag = TRUE;             /* c_rtcoef error flag                      */
    long img_coords[8];           /* image corner line,sample coordinates     */
    long line=0;                  /* current line                             */
    long samp=0;                  /* current sample                           */
    long status=0;                /* function return status                   */

    /* set up the image coordinates
    ---------------------------- */
    img_coords[0] = 1; img_coords[1] = 1;                     /* UL */
    img_coords[2] = origsize[0]; img_coords[3] = 1;           /* LL */
    img_coords[4] = 1; img_coords[5] = origsize[1];           /* UR */
    img_coords[6] = origsize[0]; img_coords[7] = origsize[1]; /* LR */

    /* compute the transformation coefficients
    ----------------------------------------*/
    status = c_rtcoef(corners, img_coords, coefs, &flag);
    if(status == E_FAIL)
        c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");

    /* now compute the new corner points
    ---------------------------------*/

    /* UL */
    line = window[SL];
    samp = window[SS];
    corners[0] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
    corners[1] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];

    /* UR */
    samp += window[NS] - 1;
    corners[4] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
    corners[5] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];

    /* LL */
    line = window[SL] + window[NL] - 1;
    samp = window[SS];
    corners[2] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
    corners[3] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];

    /* LR */
    samp += window[NS] - 1;
    corners[6] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
    corners[7] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
}

