/*--------------------------------------------------------------------------
ROUTINE: valid_ndf_header

PURPOSE:
        Determines if a file is a valid NDF header by looking for the 
        "NDF_REVISION" tag in the file.

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        filename - name of file to check
        check_extension - check the file extension for validity if not zero
        
OUTPUTS:
        revision - NDF header revision number read from the file
        valid_header - valid header flag (1=valid header file)
        returns - E_SUCC or E_FAIL
                  
ALGORITHM:
        If check_extension
            If the filename extension is not a valid NDF header extension
                Return false
        If the file contains "NDF_REVISION"
            Get the revision number
            Return true
        Return false
        
NOTES:
- A valid NDF file is defined as a file with an extension of .hd, .h#, or .dh
  that contains "NDF_REVISION" at the beginning of any line

--------------------------------------------------------------------------*/

#include <stdlib.h>           /* atof protottype */
#include <stdio.h>            /* file i/o routines */
#include <string.h>           /* string function prototypes */
#include <ctype.h>            /* toupper prototype */
#include "las.h"              /* las header file */
#include "util.h"             /* c_splitname prototype */
#include "valid_ndf_header.h" /* prototype for this module */


#define BUFFER_SIZE 200
/* size of buffer for reading lines from the NDF header file */

int valid_ndf_header
(
    char *filename,      /* I: pathname of file to check */
    int  *revision,      /* O: revision number found */
    int  *valid_header,  /* O: file is a valid NDF header (1=valid) */
    int  check_extension /* I: 1 = check for valid file extension */
)
{
    FILE *file;                 /* file pointer */
    int index;                  /* loop index */
    int length;                 /* length of tag */
    int valid_header_file = 0;  /* assume it is not a valid header file */
    char buffer[BUFFER_SIZE];   /* buffer for reading the file */
    char dir[CMLEN];            /* directory name */
    char ext[CMLEN];            /* extension of file */
    char ext_len;               /* extension length */
    char root[CMLEN];           /* root of file name */
    char signature[] = "NDF_REVISION"; /* tag to search for */
    
    length = strlen(signature); /* length of tag */
    
    if (check_extension)
    {
        /* verify the header has either a .DD, .H#, or .HD extension (either
           upper or lower case) */
        c_splitname(filename,dir,root,ext);
        
        ext_len = strlen(ext);
        if (ext_len < 2)
        {
            *valid_header = valid_header_file;
            return (E_SUCC);
        }
            
        for (index = 0; index < ext_len; index++)
        {
            ext[index]=toupper(ext[index]);
        }
        if ((strcmp(ext,"DH") != 0) &&
            (strcmp(ext,"HD") != 0))
        {
            if (ext[0] != 'H')
            {
                *valid_header = valid_header_file;
                return (E_SUCC);
            }
            
            for (index = 1; index < ext_len; index++)
            {
                if (!isdigit(ext[index]))
                {
                    *valid_header = valid_header_file;
                    return (E_SUCC);
                }
            }
        }
    }
    
    /* verify the file contains a line with NDF_REVISION at the beginning */
    file = fopen (filename, "rt");
    if (file == NULL)
    {
        /* error opening the file */
        c_lasmsg(FALSE,"ndfin-file","Unable to open %s",filename);
        return (E_FAIL);
    }

    while (!valid_header_file && fgets(buffer, BUFFER_SIZE, file))
    {
        if (strncmp(buffer,signature,length) == 0)
            valid_header_file = 1;
    }

    fclose (file);

    /* extract the floating point revision number and truncate it to 
       an int (since only version 1 and 2 exist) */
    if (valid_header_file)
        *revision = (int)atof(&buffer[length+1]);
        
    *valid_header = valid_header_file;

    return (E_SUCC);
}
