/****************************************************************************
NAME: CHECKTP 

PURPOSE: Determines if volume spanning is required and if so, creates the
         additional NDF header files

PROGRAM HISTORY:

Version   Date     Programmer      Code  Reason
-------   ----     --------------  ----- ------
  1.0     03/98    T.Ratliff        C    Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "protos.h"

#define D3SIZE 45000000               /* D-3 density tape size (kb)     */
#define D7SIZE 31500000               /* DLT-7000 density tape size (kb)*/
#define D4SIZE 18000000               /* DLT-4000 density tape size (kb)*/
#define EHSIZE 4500000                /* 8mm HI density tape size (kb)  */
#define ELSIZE 2300000                /* 8mm LO density tape size (kb)  */
#define NTSIZE 750000                 /* 3490 tape size (kb)            */
#define ETSIZE 250000                 /* 3480 tape size (kb)            */
#define NINTRK 180000                 /* 9-track tape size (kb)         */

void checktp
(
    char *tapetype,                         /* tape media type             */
    char *bform,                            /* data format of files        */
    char *hostout,                          /* NDF file names in UNIX form */
    char fnames[][CMLEN],                   /* filenames to write to tape  */
    double totsize,                         /* size of all order files     */
    long ns,                                /* number of samples           */
    long bytes_per_sample,                  /* bytes per sample */
    long filesize[300],                     /* size of target files(s)     */
    long bpv[100],                          /* bands per volume            */
    long recdsz[300],                       /* record size of output files */
    long fpv[100],                          /* files per volume            */
    long *nvol,                             /* number of volumes           */
    long *nfiles                           /* number of files found       */
)
{
  long tpsize = 0;                      /* tape media size (kb)        */
  long index = 0;                       /* index for recdsz array      */
  long fcnt = 0;                        /* file counter for loops      */
  long i = 0;                           /* loop counter                */
  long filenum = 0;                     /* file counter for loops      */
  long tmpsize = 0;                     /* tmp file size hold          */
  long numnhdr = 0;                     /* number of nhdr files        */
  long splitflg = 0;                    /* split header flag           */
  long temp = 0;                        /* holding counter             */
  long tmpnvol = 0;                     /* Temp number of volumes      */
  long tmpnfiles = 0;                   /* Temp number of files        */
  long j = 0;                           /* loop counter                */
  long fatal = -1;                      /* fatal error flag            */
  long tmprecdsz[300];                  /* temp recdsz                 */

  char tmpfnames[300][CMLEN];           /* temp fnames                 */
  char *p;                              /* pointer to string of char   */


  tmpnfiles = *nfiles;
  /* Determine what type of tape media is being used
     and assign the appropriate byte size
  --------------------------------------------------*/
  if (strcmp(tapetype,"8LO") == 0)
    tpsize = ELSIZE;
  else if (strcmp(tapetype,"8HI") == 0)
    tpsize = EHSIZE;
  else if (strcmp(tapetype,"3480") == 0)
    tpsize = ETSIZE;
  else if (strcmp(tapetype,"3490") == 0)
    tpsize = NTSIZE;
  else if (strcmp(tapetype,"D3") == 0)
    tpsize = D3SIZE;
  else if (strcmp(tapetype,"DLT7000") == 0)
    tpsize = D7SIZE;
  else if (strcmp(tapetype,"DLT4000") == 0)
    tpsize = D4SIZE;
  else if (strcmp(tapetype,"9-TRACK") == 0)
    tpsize = NINTRK;

  /* Making sure that an entire band can fit on 1 tape
  ----------------------------------------------------*/
  if (strcmp(bform,"BSQ") == 0)
    {
    if (filesize[1] > tpsize)
      c_errmsg("Band size too large for tape","ndfout-size",&fatal);
  else if (strcmp(bform,"BIL") == 0)
    if (totsize > tpsize)
      c_errmsg("Image too large for tape","ndfout-bilsiz",&fatal);
    }

  /* Check file(s) size and compare to specified media
     type for possible volume spanning
  ----------------------------------------------------*/
  if (totsize < tpsize)
    {
    tmpnvol = 1;
    fpv[0] = tmpnfiles;
    }
  else
    {
    /* Checking the size of the files
       against the tape media size
    ---------------------------------*/
    splitflg = 1;
    i = 0;
    tmpnvol = 1;
    tmpsize = 0;
    for (j = 0; j < tmpnfiles; j++)
      {
      tmpsize += filesize[j];
      if ((tpsize - tmpsize) > 0)
        fpv[i]++;
      else
        {
        tmpnvol++;
        i++;
        fpv[i]++;
        tmpsize = filesize[j];
        }
      }
    }
 
  /* Assign the record sizes and determine number of
     bands per volume
  --------------------------------------------------*/
  p = malloc(sizeof(char));
  for (i = 0; i < tmpnvol; i++)
    {
    fcnt += fpv[i];

    for (j = filenum; j < fcnt; j++)
      {
      p = strstr(fnames[j],".hd");
      if (p != NULL)
        recdsz[index] = 2600;
      else
        {
        recdsz[index] = ns * bytes_per_sample;
        bpv[i]++;
        }
      index++;
      }
    filenum += fpv[i];
    if (bpv[i] > 0)
      numnhdr++;
    }

  if (splitflg == 1)
    {
    /* Call mknhdr to create additional NDF header files
    ----------------------------------------------------*/
    mknhdr(hostout,numnhdr,bpv);

    /* Shift the file names and record sizes
       to make room for the new header files
    ----------------------------------------*/
    for (i = 0; i < fcnt; i++)
      {
      sprintf(tmpfnames[i],"%s",fnames[i]);
      tmprecdsz[i] = recdsz[i];
      }
    fcnt = 1;
    for (i = 0; i < tmpnvol; i++)
      {
      if (i == 0)
        sprintf(fnames[0],"%s%s",hostout,".h1");
      else if ((i > 0) && (i < numnhdr))
        {
        temp += fpv[i-1];
        sprintf(fnames[temp],"%s.h%d",hostout,i+1);
        recdsz[temp] = recdsz[0];
        for (j = temp+1; j <= tmpnfiles; j++)
          {
          strcpy(fnames[j],tmpfnames[j-fcnt]);
          recdsz[j] = tmprecdsz[j-fcnt];
          }
        fpv[i] = fpv[i]++;
        tmpnfiles++;
        fcnt++;
        }
      }
    }
  *nvol = tmpnvol;
  *nfiles = tmpnfiles;  
}
