/*******************************************************************************

NAME		NORMD

PURPOSE		Calcuate a normalized difference image.   

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
K.GACKE         July 1984       Original development
R.MCKINNEY      Oct. l987       Conversion to UNIX 
R.MCKINNEY      July 1988       Conversion to LAS 5.0
R.MCKINNEY	Dec  1988	Updated to close all files
D.ETRHEIM	Apr  1989	PRF# 5058  Corrected calculation when both 
				pixels are zero
T.MITTAN    	Nov  1991       PRF #5607  Corrected to convert real images
				on write 
D.ETRHEIM	Feb  1992	Convert all data types on write instead of read
T.MITTAN	Sept 1992	Added extern to curent and curcat to eliminate
				warnings on link
D. Hollaren      11/93		Added OFFSET and PIXVAL.  OFFSET is added to
				the normalized differance (this should include
				a rounding factor for the specified output data
				type.  Some of the images input to NORMD are
				scaled.  Usually the value of zero is set to
				zero in the output, PIXVAL allows the user
				to indicate what a zero value represents in
				a scaled input image.
J.Fenno         Dec 1994	Added parameters for the new global AVHRR
				scaling method.

ALGORITHM:
The normalized difference image is computed using the following equation:

    ND = (((BAND2 - BAND1) / (BAND2 + BAND1))  + OFFSET) * SCALE

ALGORITHM REFERENCES:     none

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
struct FDESC *  fdesc[MAXIMG]; 	/* image file descriptors used for image I/O */
struct GDESC *  gdesc;		    /* group descriptor used for image I/O       */
long offset1[2*MAXBND+1];	/* offsets into "thebuf" for input bands     */
long offset2[MAXBND+1];         /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		      */

float scale;			/* scale factor				      */
float offset;			/* offset to be added to the normd calculation*/
float rndval;                   /* rounding option			      */

static double minval[MAXBND];   /* min pixel value array		      */
static double maxval[MAXBND];   /* max pixel value array		      */
static long bands[2][MAXBND+1]; /* band specification for first image         */
double line_inc;
double samp_inc;

long access;			/* image I/O access type		      */
long band;			/* loop index counting the bands 	      */
long bandno;			/* loop index counting the bands 	      */
long normlim;			/* normalize difference limit		      */
long bufsiz;			/* size of image I/O buffer		      */
long curlin;			/* current line				      */
long fatal = -1;		/* fatal error flag			      */
long msg_flag;			/* flag for processing message		      */
long msgtim;			/* elpased time between display messages      */
long nmrimg;			/* number of input images for image           */
long nonfatal = 1;		/* nonfatal error flag			      */
long image;			/* loop index				      */
long nbands[2];			/* number of bands per input image            */
long odtype;			/* odtype for processing                      */
long idtype = 0;		/* input data type			      */
long one = 1;			/* number one				      */
long size = 0;			/* number of bytes per pixel		      */
long status = 0;			/* function return status code	              */
long std_out;			/* logical unit # for standard output         */
long totbnd = 0;		/* total number of bands in first image       */
long upddr_flag;		/* flag for update DDR			      */
long window[2][4];		/* window specification for first image       */
long opt[2];
long zero = 0;			/* integer value of zero		      */
long nmasks;			/* Number of input mask values                */

char hosin[2][CMLEN];		/* buffer storage        	   	      */
char hosout[1][CMLEN];	        /* buffer storage        		      */
char minmax[4];		        /* minmax flag buffer 			      */
char taein[2][CMLEN];		/* buffer storage       		      */
char taeout[1][CMLEN];		/* buffer storage       		      */

unsigned char *thebuf;	        /* image I/O buffer			      */

/*
  Initialize LAS and TAE
*/
c_inlas(&vblock, &std_out);

/*
  Get user-specified parameters and TAE globals
*/
getpar(&vblock, taein, hosin, taeout, hosout, &nmrimg, bands, nbands, window,
    &msgtim, minmax, &odtype, &totbnd, &scale, &offset, &rndval, &normlim,
    &nmasks);

/*
  Open input images
*/
access = IREAD;
for (band = 0, image = 0; image < nmrimg; image++)
    {
    status = c_eopens(&fdesc[image], hosin[image], bands[image],
	&window[image][SL], &window[image][SS], &window[image][NL],
	&window[image][NS], &offset1[band], &access, &idtype, &zero, &zero);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered", "normd-fatal",&fatal);

    band += nbands[image];
    }

if (nmrimg == 1)
    bandno = 1;
else
    bandno = totbnd / 2;

/*
  Open output image
*/
opt[0] = odtype;
opt[1] = 0;
access = IWRITE;
status = c_eopens(&fdesc[nmrimg],hosout[0], &bandno, &one, &one, &window[0][NL],
    &window[0][NS], offset2, &access, &idtype, &zero, opt);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","normd-fatal",&fatal);

fdesc[nmrimg+1] = 0;

/*
  Allocate space for image I/O buffer
*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc, &bufsiz)) == NULL)
    {
    c_errmsg("Error allocating dynamic memory", "normd-alloc",&nonfatal);
    c_eclose(&fdesc[nmrimg]);
    c_clean(&vblock,"normd",hosout,&one);
    }

/* 
  Set up file group
*/
if (c_egroup(fdesc, &gdesc, thebuf, &bufsiz) != E_SUCC)
     {
     c_eclose(&fdesc[nmrimg]);
     c_clean(&vblock,"normd",hosout,&one);
     }

/*
  Set the output data type if not specified
*/
if (odtype == 0)
    odtype = idtype;

/*
  Initialize min and max 
*/
if (strcmp(minmax,"YES") == 0) 
   c_inmnmx(&odtype,&bandno,minval,maxval);

/*
  Set up and start timed process message
*/
curlin = 1;
curcat = 1;
curent = 1;
msg_flag = ALLCAT;
c_inittm("NORMD", "Processed", "line", "bands", " ", &window[0][NL], &bandno,
    &msgtim, &msg_flag);
c_startm();

/*
  Determine the size of the data from the data type
*/
switch(idtype)
   {
   case EBYTE:
	size = 1;
	break;
   case EWORD:
   case EUWORD:
	size = 2;
	break;
   case ELONG:
   case EULONG:
   case EREAL:
	size = 4;
	break;
   }

/*
  Process all the lines
*/
for (curlin = 0; curlin < window[0][NL]; ++curlin)
    {
    /*
      Get the next line
    */
    if (c_estep(&gdesc) != E_SUCC)
       {
       c_egclse(&gdesc);
       c_clean(&vblock,"normd",hosout,&one);
       }

    /*
      Compute the normalized difference
    */
    for (band = 0; band < bandno; band++)
        {
	status = proces(&(thebuf[offset1[band] * size]),
	       &(thebuf[offset1[band+bandno] * size]),
	       &(thebuf[offset2[band] * size]),
	       &window[0][NS],&idtype,&scale,&offset,rndval,&normlim,
	       nmasks);
	if (status != E_SUCC)
	    c_clean(&vblock,"normd",hosout,&one);

	/*
	  Calcluate the min/max value if desired
	*/
	if (strcmp(minmax,"YES") == 0)
	    c_pxmnmx(&(thebuf[offset2[band] * size]),&window[0][NS],
		     &minval[band],&maxval[band],&idtype);

	}

    /*
      Increment the processing message line counter
    */
    curent++;
    }

/*
  Final buffer flush
*/
if (c_estep(&gdesc) != E_SUCC)
   {
   c_egclse(&gdesc);
   c_clean(&vblock,"normd",hosout,&one);
   }

/*
  Turn off process message
*/
c_stoptm();


/*
  Report the number of pixels zeroed due to NORMLIM
*/
report();


/* 
  Close the images
*/
if (c_egclse(&gdesc) != E_SUCC)
    c_clean(&vblock,"normd",hosout,&one);

/*
  Free the buffers
*/
free(thebuf);

/*
  Update the ddr file
*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = COMB;
status = c_upddr(&vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,
                 &line_inc,&samp_inc,&bandno,&upddr_flag);
if (status != E_SUCC)
   c_clean(&vblock,"normd",hosout,&one);

/*
  Created associated minmax file
*/
if (c_svmnmx(hosout[0],&bandno,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","normd-nonfatal",&nonfatal);

/*
  Create associated history file
*/
if (c_hstout(&vblock, hosout[0], hosin , &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","normd-nonfatal",&nonfatal);

/*
  Inform user of successful completion
*/
c_complt("NORMD", taeout, &one);
return EXIT_SUCCESS;
}
