/*******************************************************************************
NAME		PROCES	

PURPOSE		Calcuates a normalized difference.

ALGORITHM:
The normalized difference image is computed using the following equation:

    ND = (((BAND2 - BAND1) / (BAND2 + BAND1))  + OFFSET) * SCALE + RNDOPT

ALGORITHM REFERENCES:
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"


static long n_normlim = 0;	/* Number of pixels zeroed due to NORMLIM     */
static long n_mask    = 0;	/* Number of mask pixels found in the image   */
static long nonfatal  = 1;	/* Nonfatal error flag                        */


int report
(void)
{
char str[CMLEN];
long nonfatal = 1;

sprintf(str, "%d mask values were encountered", n_mask);
c_errmsg(str, "informational", &nonfatal);

sprintf(str, "%d pixels set to zero due to NORMLIM", n_normlim);
c_errmsg(str, "informational", &nonfatal);

return(E_SUCC);
}


int proces
(
    void *in,	  /* input array                         		*/
    void *map,	  /* second input array                  		*/
    void *out,	  /* output array                        		*/
    long *ns,      /* number of samples                   		*/
    long *dtype,   /* data type						*/
    float *scale,	  /* scale factor				      	*/
    float *offset,	  /* offset to be added to the normd calculation	*/
    float rndval,     /* rounding option					*/
    long *normlim, /* nomalize difference limit				*/
    long nmasks      /* Number of mask values                              */
)
{
long status = 0;
  
/*
  Call the appropriate function for the specified data type
*/
switch(*dtype)
   {
   case (EBYTE):
	status = bnormd(in,map,out,*ns,*scale,*offset,rndval,*normlim,nmasks);
	break;
   case (EWORD):
	status = wnormd(in,map,out,*ns,*scale,*offset,rndval,*normlim,nmasks);
	break;
   case (EUWORD):
	status = uwnormd(in,map,out,*ns,*scale,*offset,rndval,*normlim,nmasks);
	break;
   case (ELONG):
	status = lnormd(in,map,out,*ns,*scale,*offset,rndval,*normlim,nmasks);
	break;
   case (EULONG):
	status = ulnormd(in,map,out,*ns,*scale,*offset,rndval,*normlim,nmasks);
	break;
   case (EREAL):
	status = rnormd(in,map,out,*ns,*scale,*offset,*normlim,nmasks);
	break;
   }

return(status);
}

/****************************************************************
NAME		BNORMD	

PURPOSE		Calcuates a normalized difference for byte data.

****************************************************************/

int bnormd
(
    unsigned char *in,	/* input array                         		*/
    unsigned char *map,	/* second input array                  		*/
    unsigned char *out,	/* output array                        		*/
    long ns,		/* number of samples                   		*/
    float scale,		/* scale factor					*/
    float offset,		/* offset to be added to the normd calculation	*/
    float rndval,           /* rounding option 				*/
    long normlim,		/* normalized differance limit			*/
    long nmasks		/* Number of mask values                        */
)
{
float b1,b2,nd;
long i;

/*
  Process all the samples
*/
for (; ns; ns--, in++, map++, out++)
    {
    /*
      Extract the data values
    */
    b1 = (unsigned char)*in;
    b2 = (unsigned char)*map;


    /*
      Check for one of the mask values.  If there are 10 mask values, they are
      the values 0-9.
    */
    for (i = 0; i < nmasks; i++)
	{
	/*
	  If either pixel is a mask value
	*/
	if ((b1 == (float) i) || (b2 == (float) i))
	    {
	    /*
	      Mask value 0 takes precedence over everything else.  If this is
	      not mask value 0, both mask values must agree or something is
	      wrong.
	    */
	    if ((i != 0) && (b1 != b2))
		{
		c_errmsg("Mask values do not agree", "normd-msk", &nonfatal);
		return(E_FAIL);
		}

	    *out = (unsigned char)i;
	    break;
	    }
	}
    

    /*
      If the pixel was a mask value then move on to the next pixel
    */
    if (i < nmasks)
	{
	n_mask++;
	continue;
	}
    

    /*
      Subtract off the number of mask values from the input pixels.  If there
      are ten mask values, the offset needed to reserve the values 0-9 will be
      ten.
    */
    b1 -= (float) nmasks;
    b2 -= (float) nmasks;


    /*
      If the pixel value is zero in either band, set the true ND value to -1.
      In all other cases, compute the true ND.
    */
    if ((fabs(b1) <= 0.000001) || (fabs(b2) <= 0.000001))
	nd = (-1.0 + offset) * scale;
    else
	{
	nd = (((b2 - b1) / (b2 + b1)) + offset) * scale + rndval;

	/*
	  If ND is greater than normlim, set the true ND value to -1 and apply
	  the user specified offset and scale factors.  NOTE: don't forget to
	  test the nd value after converting it to the output data type!
	*/
	if ((unsigned char)nd > normlim)
	    {
	    nd = (-1.0 + offset) * scale;
	    n_normlim++;
	    }
	}

    /*
      Add the number of mask values back to the output ND value to reserve all
      of the input mask values and place the ND value in the output buffer
    */
    nd += (float) nmasks;
    *out = (unsigned char)nd;
    }

return(E_SUCC);
}

/****************************************************************
NAME		WNORMD	

PURPOSE		Calcuates a normalized difference for word data

****************************************************************/

int wnormd
(
    short *in,	/* input array                         		*/
    short *map,	/* second input array                  		*/
    short *out,	/* output array                        		*/
    long ns,	/* number of samples                   		*/
    float scale,	/* scale factor					*/
    float offset,	/* offset to be added to the normd calculation	*/
    float rndval,   /* rounding option				*/
    long normlim,	/* normalized differance limit			*/
    long nmasks      /* Number of mask values                              */
)
{
float b1,b2,nd;
long i;


/*
  Process all the samples
*/
for (; ns; ns--, in++, map++, out++)
    {
    /*
      Extract the data values
    */
    b1 = (short)*in;
    b2 = (short)*map;


    /*
      Check for one of the mask values.  If there are 10 mask values, they are
      the values 0-9.
    */
    for (i = 0; i < nmasks; i++)
	{
	if ((b1 == (float) i) || (b2 == (float) i))
	    {
	    /*
	      Mask value 0 takes precedence over everything else.  If this is
	      not mask value 0, both mask values must agree or something is
	      wrong.
	    */
	    if ((i != 0) && (b1 != b2))
		{
		c_errmsg("Mask values do not agree", "normd-msk", &nonfatal);
		return(E_FAIL);
		}

	    *out = (short)i;
	    break;
	    }
	}
    

    /*
      If the pixel was a mask value then move on to the next pixel
    */
    if (i < nmasks)
	{
	n_mask++;
	continue;
	}
    

    /*
      Subtract off the number of mask values from the input pixels.  If there
      are ten mask values, the offset needed to reserve the values 0-9 will be
      ten.
    */
    b1 -= (float) nmasks;
    b2 -= (float) nmasks;


    /*
      If the pixel value is zero in either band, set the true ND value to -1.
      In all other cases, compute the true ND.
    */
    if ((fabs(b1) <= 0.000001) || (fabs(b2) <= 0.000001))
	nd = (-1.0 + offset) * scale;
    else
	{
	nd = (((b2 - b1) / (b2 + b1)) + offset) * scale + rndval;

	/*
	  If ND is greater than normlim, set the true ND value to -1 and apply
	  the user specified offset and scale factors.  NOTE: don't forget to
	  test the nd value after converting it to the output data type!
	*/
	if ((short)nd > normlim)
	    {
	    nd = (-1.0 + offset) * scale;
	    n_normlim++;
	    }
	}

    /*
      Add the number of mask values back to the output ND value to reserve all
      of the input mask values and place the ND value in the output buffer
    */
    nd += (float) nmasks;
    *out = (short)nd;
    }

return(E_SUCC);
}

/***********************************************************************
NAME		UWNORMD	

PURPOSE		Calcuates a normalized difference for unsigned word data

************************************************************************/

int uwnormd
(
    unsigned short *in,	/* input array                         		*/
    unsigned short *map,	/* second input array                  		*/
    unsigned short *out,	/* output array                        		*/
    long ns,		/* number of samples                   		*/
    float scale,		/* scale factor					*/
    float offset,		/* offset to be added to the normd calculation	*/
    float rndval,   	/* rounding option				*/
    long normlim,		/* normalized differance limit			*/
    long nmasks		/* Number of mask values                        */
)
{
float b1,b2,nd;
long i;

/*
  Process all the samples
*/
for (; ns; ns--, in++, map++, out++)
    {
    /*
      Extract the data values
    */
    b1 = (unsigned short)*in;
    b2 = (unsigned short)*map;


    /*
      Check for one of the mask values.  If there are 10 mask values, they are
      the values 0-9.
    */
    for (i = 0; i < nmasks; i++)
	{
	if ((b1 == (float) i) || (b2 == (float) i))
	    {
	    /*
	      Mask value 0 takes precedence over everything else.  If this is
	      not mask value 0, both mask values must agree or something is
	      wrong.
	    */
	    if ((i != 0) && (b1 != b2))
		{
		c_errmsg("Mask values do not agree", "normd-msk", &nonfatal);
		return(E_FAIL);
		}

	    *out = (unsigned short)i;
	    break;
	    }
	}
    

    /*
      If the pixel was a mask value then move on to the next pixel
    */
    if (i < nmasks)
	{
	n_mask++;
	continue;
	}
    

    /*
      Subtract off the number of mask values from the input pixels.  If there
      are ten mask values, the offset needed to reserve the values 0-9 will be
      ten.
    */
    b1 -= (float) nmasks;
    b2 -= (float) nmasks;


    /*
      If the pixel value is zero in either band, set the true ND value to -1.
      In all other cases, compute the true ND.
    */
    if ((fabs(b1) <= 0.000001) || (fabs(b2) <= 0.000001))
	nd = (-1.0 + offset) * scale;
    else
	{
	nd = (((b2 - b1) / (b2 + b1)) + offset) * scale + rndval;

	/*
	  If ND is greater than normlim, set the true ND value to -1 and apply
	  the user specified offset and scale factors.  NOTE: don't forget to
	  test the nd value after converting it to the output data type!
	*/
	if ((unsigned short)nd > normlim)
	    {
	    nd = (-1.0 + offset) * scale;
	    n_normlim++;
	    }
	}

    /*
      Add the number of mask values back to the output ND value to reserve all
      of the input mask values and place the ND value in the output buffer
    */
    nd += (float) nmasks;
    *out = (unsigned short)nd;
    }

return(E_SUCC);
}

/****************************************************************
NAME		LNORMD	

PURPOSE		Calcuates a normalized difference for long data

****************************************************************/

int lnormd
(
    long *in,	/* input array                         		*/
    long *map,	/* second input array                  		*/
    long *out,	/* output array                        		*/
    long ns,	/* number of samples                   		*/
    float scale,	/* scale factor					*/
    float offset,	/* offset to be added to the normd calculation	*/
    float rndval,   /* rounding option				*/
    long normlim,	/* normalized differance limit			*/
    long nmasks      /* Number of mask values                              */
)
{
float b1,b2,nd;
long i;


/*
  Process all the samples
*/
for (; ns; ns--, in++, map++, out++)
    {
    /*
      Extract the data values
    */
    b1 = (float)*in;
    b2 = (float)*map;


    /*
      Check for one of the mask values.  If there are 10 mask values, they are
      the values 0-9.
    */
    for (i = 0; i < nmasks; i++)
	{
	if ((b1 == (float) i) || (b2 == (float) i))
	    {
	    /*
	      Mask value 0 takes precedence over everything else.  If this is
	      not mask value 0, both mask values must agree or something is
	      wrong.
	    */
	    if ((i != 0) && (b1 != b2))
		{
		c_errmsg("Mask values do not agree", "normd-msk", &nonfatal);
		return(E_FAIL);
		}

	    *out = (long)i;
	    break;
	    }
	}
    

    /*
      If the pixel was a mask value then move on to the next pixel
    */
    if (i < nmasks)
	{
	n_mask++;
	continue;
	}
    

    /*
      Subtract off the number of mask values from the input pixels.  If there
      are ten mask values, the offset needed to reserve the values 0-9 will be
      ten.
    */
    b1 -= (float) nmasks;
    b2 -= (float) nmasks;


    /*
      If the pixel value is zero in either band, set the true ND value to -1.
      In all other cases, compute the true ND.
    */
    if ((fabs(b1) <= 0.000001) || (fabs(b2) <= 0.000001))
	nd = (-1.0 + offset) * scale;
    else
	{
	nd = (((b2 - b1) / (b2 + b1)) + offset) * scale + rndval;

	/*
	  If ND is greater than normlim, set the true ND value to -1 and apply
	  the user specified offset and scale factors.  NOTE: don't forget to
	  test the nd value after converting it to the output data type!
	*/
	if ((long)nd > normlim)
	    {
	    nd = (-1.0 + offset) * scale;
	    n_normlim++;
	    }
	}

    /*
      Add the number of mask values back to the output ND value to reserve all
      of the input mask values and place the ND value in the output buffer
    */
    nd += (float) nmasks;
    *out = (long)nd;
    }

return(E_SUCC);
}

/***********************************************************************
NAME		ULNORMD	

PURPOSE		Calcuates a normalized difference for unsigned long data

************************************************************************/

int ulnormd
(
    unsigned long *in,	/* input array                         		*/
    unsigned long *map,	/* second input array                  		*/
    unsigned long *out,	/* output array                        		*/
    long ns,		/* number of samples                   		*/
    float scale,		/* scale factor					*/
    float offset,		/* offset to be added to the normd calculation	*/
    float rndval,		/* rounding option				*/
    long normlim,		/* normalized differance limit			*/
    long nmasks		/* Number of mask values                        */
)
{
float b1,b2,nd;
long i;

/*
  Process all the samples
*/
for (; ns; ns--, in++, map++, out++)
    {
    /*
      Extract the data values
    */
    b1 = (float)*in;
    b2 = (float)*map;


    /*
      Check for one of the mask values.  If there are 10 mask values, they are
      the values 0-9.
    */
    for (i = 0; i < nmasks; i++)
	{
	if ((b1 == (float) i) || (b2 == (float) i))
	    {
	    /*
	      Mask value 0 takes precedence over everything else.  If this is
	      not mask value 0, both mask values must agree or something is
	      wrong.
	    */
	    if ((i != 0) && (b1 != b2))
		{
		c_errmsg("Mask values do not agree", "normd-msk", &nonfatal);
		return(E_FAIL);
		}

	    *out = (unsigned long)i;
	    break;
	    }
	}
    

    /*
      If the pixel was a mask value then move on to the next pixel
    */
    if (i < nmasks)
	{
	n_mask++;
	continue;
	}
    

    /*
      Subtract off the number of mask values from the input pixels.  If there
      are ten mask values, the offset needed to reserve the values 0-9 will be
      ten.
    */
    b1 -= (float) nmasks;
    b2 -= (float) nmasks;


    /*
      If the pixel value is zero in either band, set the true ND value to -1.
      In all other cases, compute the true ND.
    */
    if ((fabs(b1) <= 0.000001) || (fabs(b2) <= 0.000001))
	nd = (-1.0 + offset) * scale;
    else
	{
	nd = (((b2 - b1) / (b2 + b1)) + offset) * scale + rndval;

	/*
	  If ND is greater than normlim, set the true ND value to -1 and apply
	  the user specified offset and scale factors.  NOTE: don't forget to
	  test the nd value after converting it to the output data type!
	*/
	if ((unsigned long)nd > normlim)
	    {
	    nd = (-1.0 + offset) * scale;
	    n_normlim++;
	    }
	}

    /*
      Add the number of mask values back to the output ND value to reserve all
      of the input mask values and place the ND value in the output buffer
    */
    nd += (float) nmasks;
    *out = (unsigned long)nd;
    }

return(E_SUCC);
}

/*************************************************************
NAME		RNORMD	

PURPOSE		Calcuates a normalized difference float data

*************************************************************/

int rnormd
(
    float *in,	/* input array                         		*/
    float *map,	/* second input array                  		*/
    float *out,	/* output array                        		*/
    long ns,	/* number of samples                   		*/
    float scale,	/* scale factor					*/
    float offset,	/* offset to be added to the normd calculation	*/
    long normlim,	/* normalized differance limit			*/
    long nmasks      /* Number of mask values                              */
)
{
float b1, b2, nd;
long i;

/*
  Process all the samples
*/
for (; ns; ns--, in++, map++, out++)
    {
    /*
      Extract the data values
    */
    b1 = *in;
    b2 = *map;


    /*
      Check for one of the mask values.  If there are 10 mask values, they are
      the values 0-9.
    */
    for (i = 0; i < nmasks; i++)
	{
	if ((b1 == (float) i) || (b2 == (float) i))
	    {
	    /*
	      Mask value 0 takes precedence over everything else.  If this is
	      not mask value 0, both mask values must agree or something is
	      wrong.
	    */
	    if ((i != 0) && (b1 != b2))
		{
		c_errmsg("Mask values do not agree", "normd-msk", &nonfatal);
		return(E_FAIL);
		}

	    *out = (float)i;
	    break;
	    }
	}
    

    /*
      If the pixel was a mask value then move on to the next pixel
    */
    if (i < nmasks)
	{
	n_mask++;
	continue;
	}
    

    /*
      Subtract off the number of mask values from the input pixels.  If there
      are ten mask values, the offset needed to reserve the values 0-9 will be
      ten.
    */
    b1 -= (float) nmasks;
    b2 -= (float) nmasks;


    /*
      If the pixel value is zero in either band, set the true ND value to -1.
      In all other cases, compute the true ND.
    */
    if ((fabs(b1) <= 0.000001) || (fabs(b2) <= 0.000001))
	nd = (-1.0 + offset) * scale;
    else
	{
	nd = (((b2 - b1) / (b2 + b1)) + offset) * scale;

	/*
	  If ND is greater than normlim, set the true ND value to
	  -1 and apply the user specified offset and scale factors.
	*/
	if (nd > (float) normlim)
	    {
	    nd = (-1.0 + offset) * scale;
	    n_normlim++;
	    }
	}

    /*
      Add the number of mask values back to the output ND value to reserve all
      of the input mask values and place the ND value in the output buffer
    */
    nd += (float) nmasks;
    *out = (float)nd;
    }

return(E_SUCC);
}
