/*******************************************************************************
NAME:      		   	GETPAR

PURPOSE:     	To retrieve the user parameters and check the validity of them

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey	  06/84		Original development
R. McKinney	  10/87		#4206
B. Ailts	  08/87		newLAS conversion
B. Ailts	  10/88		LAS 5.0 conversion + #4595

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PERSPEC must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Retrieve elevation input image specifications
Retrieve reflectence input image specifications
Retrieve output image name
Retrieve output window size
Retrieve output window secification
Check to see if the input images are the same size
Retrieve pixel size parameter
If pixel size parameter is NULL
   Retreive the size from the DDR
   If the size in the line and sample directions are equal
      Use the size from the DDR
   Else
      Output an error message
Else
   Use the size given in the parameter
Retrieve viewer location
Check to be sure viewer location is within the specified window
Retrieve viewer direction
Retrieve field of view
Retrieve vertical scale
Retrieve horizin line
Retrieve pixel value of the sky
Retrieve output data type
Retrieve history flag
Retrieve message time interval
Retrieve min/max flag

ALGORITHM REFERENCES:     none
*******************************************************************************/

#include "las.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblk,        /* I: TAE parameter block		      */
    char *hosin1,		/* O: host name of elevation image	      */
    char *hosin2,		/* O: host name of reflectence image	      */
    char *hosout,		/* O: host name of output image		      */
    long *window1,		/* O: window specified for elevation image    */
    long *bands1,		/* O: band specified for elevation image      */
    long *window2,		/* O: window specified for reflectence image  */
    long *bands2,		/* O: bands specified for reflectence image   */
    long *totbnd,    	        /* O: total bands specified for reflectence 
				      image*/
    long *out_wind,		/* O: output window specification	      */
    float *pixsiz,		/* O: pixel size 			      */
    long *viewloc,		/* O: viewer location			      */
    float *viewdir,		/* O: viewing direction			      */
    float *foview,		/* O: field of view parameter		      */
    float *vertscl,		/* O: vertical scaling factor		      */
    long *horizlin,		/* O: Line that will be the start of the hrizn*/
    long *skyval,		/* O: the pixel value of the sky	      */
    long *odtype,		/* O: output data type			      */
    long *msgtim,		/* O: display message interval time	      */
    char *minmax		/* O: minmax flag			      */
)

{  /*  getpar  */

struct DDR *ddr1;		/*  structure of DDR			*/
struct VARIABLE *var;		/*  holds the location of the parameter */

long nonfatal = 1;		/*  nonfatal error code			*/
long fatal = -1;		/*  fatal error code			*/
long i;				/*  counter--current pararmeter element */
long nbands;			/*  number of bands per image		*/
long nimg;			/*  number of images			*/
long one = 1;			/*  value of one			*/
long qflag[2];			/*  defualt flag			*/
long same;			/*  input images have same window	*/
long status;			/*  error status			*/
long zero = 0;			/*  value of zero			*/

char strdtype[5];		/*  temporary string value of odtype	*/
char taein1[CMLEN];		/*  LAS name of first input image	*/
char taein2[CMLEN];		/*  LAS name of second input image	*/
char taeout[CMLEN];	        	/*  temp tae name of output image	*/


/* Get the input Elevation and Reflectance images and the output image names.
-----------------------------------------------------------------------------*/
status = c_getin(vblk,"IN1",&zero,(char (*)[256])taein1,(char (*)[256])hosin1,
 	 	 &nimg,(long (*)[4])window1,(long (*)[257])bands1,&nbands,
		 totbnd,&one,&same,(long (*)[2])qflag);
if (status != E_SUCC) 
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);

if (*totbnd != 1)
   {
   c_errmsg ("In1 must be a single-band image","perspec-in1",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

status = c_getin (vblk,"IN2",&zero,(char (*)[256])taein2,(char (*)[256])hosin2,
		&nimg,(long (*)[4])window2,(long (*)[257])bands2,
		&nbands,totbnd,&one,&same,(long (*)[2])qflag);
if (status != E_SUCC) 
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);

if ((*totbnd != 1) && (*totbnd != 3))
   {
   c_errmsg("In2 must be a single- or three-band image",
	    "perspec-in2",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

if (nimg > 1)
   {
   c_errmsg("In2 must be a single image","perspec-image",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

/* Retrieve the output image name
---------------------------------*/
status = c_getout (vblk,"OUT",(char (*)[256])taeout,(char (*)[256])hosout);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);

/* Check the validity of the input windows.
-------------------------------------------*/
if ((window1[2] != window2[2]) || (window1[3] != window2[3]))
   {
   c_errmsg("Specified windows are not of equal sizes",
	    "perspec-size",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

/* Retreive the output image specification
------------------------------------------*/
var = p_find(vblk,"OUTWIND");
if (var->v_count == 0)
   {
   out_wind[0] = window1[2];
   out_wind[1] = window1[3];
   }
else
   {
   out_wind[0] = IVAL(*var,0);
   out_wind[1] = IVAL(*var,1);

/*  Below is not true any more because the output image size needs to be 
greater is the field of view is large
---------------------------------------------------------------------------*/
   /*  Output window specification has to be within the windows specified for 
       the input images
   --------------------------------------------------------------------------*/
/*   if ((out_wind[0] > window1[2]) || (out_wind[1] > window1[3]) ||
       (out_wind[0] > window2[2]) || (out_wind[1] > window2[3]))
      {
      c_errmsg ("Output window is not within the limits of the input window",
	        "perspec-outwind",&nonfatal);
      c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
      }
*/
   }

/* Retrieve the parameter values from V-block.
----------------------------------------------*/
var = p_find(vblk,"PIXSIZ");
if (var->v_count == 0)
   {  /* if NULL retreive the pixel size from the DDR  */
   ddr1 = (struct DDR *)malloc(sizeof(struct DDR));
   c_getddr(hosin1,ddr1);
   if (ddr1->valid[DDPDV] == INVAL)
      {
      c_errmsg("Projection distance of input image is not valid",
               "perspec-valid",&nonfatal);
      c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
      }
   if (ddr1->pdist_y == ddr1->pdist_x)
      *pixsiz = ddr1->pdist_y;
   else
      {
 c_errmsg("Projection distance in the line and sample directions are not equal",
          "perspec-pdist",&nonfatal);
      c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
      }
   free(ddr1);
   }
else
   *pixsiz = RVAL(*var,0);


var = p_find(vblk,"VIEWLOC");
viewloc[0] = IVAL(*var,0);
viewloc[1] = IVAL(*var,1);
viewloc[2] = IVAL(*var,2);

/*  the algorithm has changed to handle viewer location's that are outside of
    the input images
-----------------------------------------------------------------------------*/
/*  viewer location has to be within the windows specified for the input images
------------------------------------------------------------------------------*/
/*
if ((viewloc[0] < window1[0]) || 
    (viewloc[1] < window1[1]) ||
    (viewloc[0] > (window1[0] + window1[2] - 1)) ||
    (viewloc[1] > (window1[1] + window1[3] - 1)))
   {
   c_errmsg ("Viewer location is not within the limits of the input window",
	     "perspec-viewloc",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }
*/

/*
if ((viewloc[0] < window2[0]) || 
    (viewloc[1] < window2[1]) ||
    (viewloc[0] > (window2[0] + window2[2] - 1)) ||
    (viewloc[1] > (window2[1] + window2[3] - 1)))
   {
   c_errmsg ("Viewer location is not within the limits of the input window",
	     "perspec-viewloc",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }
*/

var = p_find(vblk,"VIEWDIR");
*viewdir = RVAL(*var,0);

var = p_find(vblk,"FOVIEW");
*foview = RVAL(*var,0);

var = p_find(vblk,"VERTSCAL");
*vertscl = RVAL(*var,0);

/* Retrieve HORIZLIN and verify that it is a positive number 
----------------------------------------------------------*/
var = p_find(vblk,"HORIZLIN");
*horizlin = IVAL(*var,0);
if (*horizlin < 1)
   {
   c_errmsg ("HORIZLIN must be greater than zero","perspec-horizlin",&nonfatal);
   c_errmsg ("Fatal error encountered","perspec-fatal",&fatal);
   }


var = p_find(vblk,"SKYVAL");
for (i = 0; i < var->v_count; i++)
   skyval[i] = IVAL(*var,i);

var = p_find(vblk,"ODTYPE");
strcpy(strdtype,SVAL(*var,0));

/*  Retrieve the data type of the input DDR when odtype == SAME
---------------------------------------------------------------*/
if (strcmp(strdtype,"SAME") == 0)
   {
   status = c_gettyp((char (*)[256])hosin2,&nimg,odtype);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }
else if (strcmp(strdtype,"BYTE") == 0)
   *odtype = EBYTE;
else if (strcmp(strdtype,"I*2") == 0)
   *odtype = EWORD;
else if (strcmp(strdtype,"I*4") == 0)
   *odtype = ELONG;
else if (strcmp(strdtype,"R*4") == 0)
   *odtype = EREAL;

var = p_find(vblk,"$MSGTIM");
*msgtim = IVAL(*var,0);

var = p_find(vblk,"$MINMAX");
strcpy (minmax,SVAL(*var,0));

return;
}  /*  getpar  */
