/******************************************************************************
NAME:      		   	PERSPEC

PURPOSE:    Create a 3_D output image by combining input elevation image data
	    with reflectence image data.

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey	  06/84		Original development
R. McKinney	  10/87		#4206
B. Ailts	  08/87		newLAS conversion
B. Ailts	  10/88		LAS 5.0 conversion + #4595
B. Ailts	  8/89 		Fixed problem with odd no. of samples on a large
				image
D. Etrheim        08/92         Write module name to history file
T. Mittan	  09/92		Added external to curent and curcat to eliminate
				warnings on link
D. Hames          12/99 	Remove 50000 line limitation for parameter
				HORIZLIN from pdfs. Add error check in getpar
				to verify that HORIZLIN is a positive number.
				Add prototypes. SRF #893
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PERSPEC must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Initialization of variables
Retrieve the user-specified parameters.  
If the image is small enough
   Use the processing routine that will hold the whole image in a buffer at 
   one time in order to increase processing speed
Else
   Use the processing routine that will proces the image in blocks
   Rotate the image
Endif
The ddr fields are updated
Update the DDR min/max fields
The associated history file are created for the output image 
The completion message is output.

ALGORITHM REFERENCES: 
The main processing algorithm was originally taken from:
  B. Fishman and B. Schachter, "Computer Display of Height Fields," 
  Computers and Graphics, Vol.  5, pp. 53-60, 1980.

The rotation algorithm was developed from conversations with
  Brian Davis   - EROS Data Center
  Doug Hollaren - EROS Data Center
  Rich Mckinney - EROS Data Center
  Dan Steinwand - EROS Data Center
  
******************************************************************************/


#include <time.h>
#include <stdio.h>

#include "las.h"
#include "protos.h"
#include "proj.h"
#include "util.h"

extern long curent;     /* current line                         */
extern long curcat;     /* currnet band                         */


/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
int fdesc3[2];	     	/* image file descriptors	      		*/
int gdesc;		/* group descriptor 	      			*/
long offset[3 + 1];	/* offsets into thebuf for input 		*/

/* The following globals are set the maximum block size of memory to be used
----------------------------------------------------------------------------*/
long blksiz = 768;	/* the size of the blocks that can be processed */
long buffersiz = 589824;/* the size of the maximum buffer(blksiz squared)*/

void main
(void)

{  /* perspec  */

struct PARBLK vblk;	/*  TAE parameter block				*/
struct DDR ddr;		/*  DDR structure				*/
struct BDDR *bddr;	/*  band DDR structure				*/

double line_inc;	/*  line increment value			*/
double samp_inc;	/*  sample increment value			*/
double maxval[3];	/*  maximum pixel values			*/
double minval[3];	/*  minimum pixel values			*/

float foview;		/*  field of view				*/
float pixsiz;		/*  pixel size of input image			*/
float vertscl;		/*  vertical scale				*/
float viewdir;		/*  viewer direction				*/

long tbands[2][MAXBND + 1];/*  temporary band specifications		*/
long bands1[2];		/*  band specification of first input image	*/
long bands2[3 + 1];	/*  band specification of second input image	*/
long c_flag;		/*  Combination flag needed by c_upddr		*/
long horizn;		/*  start of the horizin line			*/
long i;			/*  counter -- current band			*/
long image_count = 2;	/*  number of input images			*/
long msgtim;		/*  time between display messages		*/
long nbands[2];		/*  number of bands for each input image	*/
long nonfatal = 1;	/*  nonfatal error code				*/
long odtype;		/*  output data type				*/
long one = 1;		/*  value of one				*/
long out_wind[2];	/*  output window specification			*/
long skyval[3];		/*  the sky value for each band			*/
long status;		/*  error status of function calls		*/
long std_out;		/*  standard output				*/
long totbnd;		/*  total number of bands of second input image */
long viewloc[3];	/*  viewer location				*/
long window1[4];	/*  window of first input image			*/
long window2[4];	/*  window of second input image		*/

char minmax[4];		/*  minmax flag					*/
char hosin1[CMLEN];	/*  host name of first input image		*/
char hosin2[CMLEN];	/*  host name of second input image		*/
char tempout[CMLEN];	/*  host name of temporary image		*/
char thosin[2][CMLEN];	/*  host name of first and second input image	*/
char fast[4];		/*  flag to TAE for fast processing		*/
char *ptr[1];
char *p;

char hosout[CMLEN];	/*  LAS name of output image			*/
char taeout[CMLEN];     /*  LAS name of output image                    */
 
/* initialization of parameter block
------------------------------------*/
c_inlas(&vblk,&std_out);	 

/*  Retreive the parameters from the TAE parameter block
--------------------------------------------------------*/
getpar(&vblk,hosin1,hosin2,hosout,window1,bands1,window2,bands2,&totbnd,
       out_wind,&pixsiz,viewloc,&viewdir,&foview,&vertscl,
       &horizn,skyval,&odtype,&msgtim,minmax);

c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Create temporary file name in TAE format
-------------------------------------------
*/

/*ptr = hosout;
q_string(&vblk,"ofile",1,ptr,P_UPDATE);
q_out(&vblk);
*/
strcpy(tempout,"perspec_");
c_creat_name(tempout,"img");
strcpy(taeout, tempout);
p = (char *)strrchr(taeout, HOST_EXT);
*p = '\0'; 


/* Send temporary file name back to TAE
---------------------------------------*/
*ptr = taeout;
q_string(&vblk,"temp",1,ptr,P_UPDATE);
q_out(&vblk);

if ((totbnd * window1[2] * window1[3] <= buffersiz) && (odtype == EBYTE))
  {
   /*  Proces and rotate the image at the same time, send flag to TAE
   ------------------------------------------------------------------*/
   fast_proc(&vblk,hosin1,hosin2,tempout,window1,bands1,window2,bands2,&totbnd,
	     out_wind,&pixsiz,viewloc,&viewdir,&foview,&vertscl,&horizn,skyval,
	     &odtype,&msgtim,minmax,minval,maxval);

   strcpy(fast,"YES");
   *ptr = fast;
   q_string(&vblk,"fastflag",1,ptr,P_UPDATE);
   q_out(&vblk);
  }
else
   { /*  Proces the image in blocks*/

   switch (odtype)
      {
      /*  Proces the input images based on the data type
      --------------------------------------------------*/
      case EBYTE: proces(&vblk,hosin1,hosin2,tempout,window1,bands1,window2,
	                 bands2,&totbnd,out_wind,&pixsiz,viewloc,&viewdir,
			 &foview,&vertscl,&horizn,skyval,&odtype,&msgtim);
                  break;

      case EWORD: procesw(&vblk,hosin1,hosin2,tempout,window1,bands1,window2,
	                 bands2,&totbnd,out_wind,&pixsiz,viewloc,&viewdir,
			 &foview,&vertscl,&horizn,skyval,&odtype,&msgtim);
	          break;

      case ELONG: procesl(&vblk,hosin1,hosin2,tempout,window1,bands1,window2,
	                 bands2,&totbnd,out_wind,&pixsiz,viewloc,&viewdir,
			 &foview,&vertscl,&horizn,skyval,&odtype,&msgtim);
	          break;

      case EREAL: procesr(&vblk,hosin1,hosin2,tempout,window1,bands1,window2,
	                 bands2,&totbnd,out_wind,&pixsiz,viewloc,&viewdir,
			 &foview,&vertscl,&horizn,skyval,&msgtim);
	          break;
      } /*  switch odtype  */

   } /*  else of if totbnd * window1[2] * window1[3] > buffersiz  */

/* Update the DDR fields.  
-------------------------*/
strcpy(thosin[0],hosin1);
strcpy(thosin[1],hosin2);
nbands[0] = 1;
nbands[1] = totbnd;
tbands[0][0] = bands1[0];
for (i = 0; i < totbnd; i++)
   tbands[1][i] = bands2[i];
line_inc = 1.0;
samp_inc = 1.0;
c_flag = COMB;
status = c_upddr(&vblk,thosin,tempout,&image_count,(long (*)[4])window2,tbands,
		 nbands,&line_inc,&samp_inc,&totbnd,&c_flag);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);

/*  The projection information is lost so invalidate the projection information
    in the DDR file
------------------------------------------------------------------------------*/
bddr = (struct BDDR *)calloc(totbnd,sizeof(struct BDDR));
status = c_getddr(tempout,&ddr);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);

/*  Retreive the band records of the DDR so not to lose that information
------------------------------------------------------------------------*/
for (i = 1; i < totbnd + 1; i++)
   {
   status = c_getbdr(tempout,&(bddr[i - 1]),&i);
   if (status != E_SUCC)
      c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
   }

ddr.master_line = 1;
ddr.master_sample = 1;
ddr.proj_code = 0;
ddr.valid[DDPCV] = INVAL;
ddr.zone_code = 0;
ddr.valid[DDZCV] = INVAL;
ddr.datum_code = 0;
ddr.valid[DDPUV] = INVAL;
for (i = 0; i < COEFCT; i++)
   ddr.proj_coef[i] = 0;
ddr.valid[DDPPV] = INVAL;
ddr.upleft[0] = 0;
ddr.upleft[1] = 0;
ddr.loleft[0] = 0;
ddr.loleft[1] = 0;
ddr.upright[0] = 0;
ddr.upright[1] = 0;
ddr.loright[0] = 0;
ddr.loright[1] = 0;
ddr.valid[DDCCV] = INVAL;

/*  Write the first record of the DDR
-------------------------------------*/
status = c_putddr(tempout,&ddr);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);

/*  Write the band records of the DDR
-------------------------------------*/
for (i = 1; i < totbnd + 1; i++)
   {
   status = c_putbdr(tempout,&(bddr[i - 1]));
   if (status != E_SUCC)
      c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
   }

free(bddr);

/* update the min/max fields 
----------------------------*/
status = c_svmnmx(tempout,&totbnd,minval,maxval,minmax);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);

/* create associated history file
--------------------------------- */
status = c_hstout(&vblk,tempout,thosin,&image_count,"PERSPEC");
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);

/* inform user of successful completion 
---------------------------------------*/
c_complt("PERSPEC",(char (*)[256])taeout,&one);

}  /* perspec  */
