/* ----------------------------------------------------------------------------
NAME: GALPHA2

FUNCTION:   Calculates the alpha angle for the subcommand MODIFY 
            according to the method specified.  Two methods are available
            the 1x3 convolution method and the 3x3 convolution method.

PROGRAM HISTORY:
  Version   Date       Author           Request
  -------   ----       ------           -------
    1.0       06/27/86     J. Stansbery      
    2.0       08/1/89      B. Allam     conversion


ALGORITHM:

   Initialize alpha to zero.
   Set all of the mask values in the 3x3 matrix to the value of 
   the center pixel.
   If method is ONE, 
      Calculate the kernels (dx and dy) for the 1x3 convolution method.
   If method is THREE, 
      Calculate the kernels (dx and dy) for the 3x3 convolution method.
   Calculate the denom (denominator).
   Calculate numer1 (numerator generated using the sun direction constants 
                     calculated from sun1 and azmth1).
   Calculate numer2 (numerator generated using the sun direction constants 
                     calculatedfrom sun2 and azmth2).  
   Calculate alpha = (numer1 / denom)**oexp.
   Calculate alpha2 = (numer2 / denom)**iexp.
   If alpha is less than zero, 
      Set (alpha) to zero
   If alpha2 is less than zero, 
      Set (alpha2) to zero
   If the exponent (oexp) is equal to zero then
      Set (alpha) to one
   Else
      Set (alpha) to (alpha**oexp)
   End-if
   If the exponent (iexp) is equal to zero then
      Set (alpha2) to one
   Else
      Set (alpha2) to (alpha2**iexp)
   End-if
   Calculate the final output angle.
   If the output data type is byte, 
      Clip accordingly.
   Store the result in the output buffer.
   Update the minium and maximum pixel values.

---------------------------------------------------------------------------- */
#include "picshade.h"

void galpha2
(
    float matrix[][3],        /* 3x3 matrix from input image */
    long method,         /* Method of calculation */
    double mask,           /* Mask value for non-inclusive data */
    float a[],            /* Constants to describe sun direction */
    float b[],            /* Constants to describe sun direction */
    double dirx1,          /* Directional derivative in x-direction */
    double diry1,          /* Directional derivative in y-direction */
    double dirx2,          /* Directional derivative in x-direction */
    double diry2,          /* Directional derivative in y-direction */
    double oexp,           /* Exponent factor for the output scene */
    double iexp,           /* Exponent factor for input scene */
    double tnsity,         /* Output sun intensity factor */
    double ambien,         /* Ambient light factor */
    long odtype,         /* Output data type */
    long pix,            /* Current sample */
    float *workbuf2      /* Buffer with 2nd image data */
)
{
double  temp;           /* Intermediate result */

float   alpha, alpha2,      /* Angles between sun and surface */
dx, dy,         /* x and y kernels for convolution */
denom,          /* Denominator to alpha and alpha2 */
numer1,         /* Numerator to alpha */
numer2;         /* Numerator to alpha2 */

long    i, ix, iz;      /* Loop counters and array indices */

/* Initialize alpha to zero.
Set all of the mask values in the 3x3 matrix to the value of the center pixel.
----------------------------------------------------------------------------- */
alpha = 0.0;
dx = 0.0;
dy = 0.0;
if (matrix[1][1] != mask)
    {
    for (ix = 0;ix < 3; ix++)
        {
        for (iz = 0;iz < 3; iz++)
            {
            if (matrix[ix][iz] == mask)
                matrix[ix][iz] = matrix[1][1];
            }
        }
    } 

/* If method is ONE, calculate the kernels (dx and dy) for the 1x3
convolution method.
---------------------------------------------------------------- */
if (method == 1)
    {
    if (matrix[1][1] != mask)
        {
        dx = dirx1 * (matrix[1][2] - matrix[1][0]);
        dy = diry1 * (matrix[0][1] - matrix[2][1]);
        } 
    }
/* If method is THREE, calculate the kernels (dx and dy) for the 3x3
convolution method.
------------------------------------------------------------------ */
else
    {
    if (matrix[1][1] != mask)
        {
        for (i = 0;i < 3; i++)
            {
            dx = dx + matrix[i][2] - matrix[i][0];
            dy = dy + matrix[0][i] - matrix[2][i];
            }
        dx = dirx2 * dx;
        dy = diry2 * dy;
        } 
    }

/* Calculate the denom (denominator).
Calculate numer1 (numerator generated using the sun direction constants
calculated from sun1 and azmth1).
Calculate numer2 (numerator generated using the sun direction constants
calculatedfrom sun2 and azmth2). 
Calculate alpha = (numer1 / denom)**oexp.
Calculate alpha2 = (numer2 / denom)**iexp.
---------------------------------------------------------------------------- */
temp = dx * dx + dy * dy + 1.0;
denom = pow(temp, 0.5);

numer1 = -(a[0] * dx) - (a[1] * dy) + a[2];
numer2 = -(b[0] * dx) - (b[1] * dy) + b[2];

alpha = numer1 / denom;
alpha2 = numer2 / denom;
if (alpha < 0) alpha = 0.0;
if (alpha2 < 0) alpha2 = 0.0;
if (oexp == 0)
    alpha = 1.0;
else
    {
    temp = alpha;
    alpha = pow(temp, (double)oexp);
    } 
if (iexp == 0)
    alpha2 = 1.0;
else
    {
    temp = alpha2;
    alpha2 = pow(temp, (double)iexp);
    }

/* Calculate the final output angle.
If the output data type is byte, clip accordingly.
Store the result in the output buffer.
--------------------------------------------------- */
if (alpha2 == 0)
    temp = workbuf2[pix-1];
else
    {
    temp = tnsity * alpha + ambien;
    temp = (workbuf2[pix-1] * temp) / alpha2;
    }
if (odtype == 1)
    {
    if (temp < 0)
        temp = 0.0;
    else if (temp > 255)
        temp = 255.0;
    obuf[pix-1] = (float)(int)(temp + 0.5);
    }
else
    obuf[pix-1] = temp;
}

