/*------------------------------------------------------------------------------
NAME: GETPAR

FUNCTION: Retrieves the user-supplied parameters.

PROGRAM HISTORY:
  Version   Date       Author           Request
  -------   ----       ------           -------
    1.0       06/27/86     J. Stansbery      
    2.0       08/1/89      B. Allam     conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT:    LAS

ALGORITHM:

   Get the subcommand name.
   Get the parameters for the subcommand CREATE.
      Get the name of the input image.
      If specified windows differ in size, 
         Abort with an error message.
      If the specified image has more than one band, 
         Abort with an error message.
      Get the sun angle, azimuth angle, and exponent factor.
      Get the output data type.
   End-Create
   Get the parameters for the subcommand MODIFY.
      Get the names of the input images.
      If specified windows differ in size, 
         Abort with an error message.
      If the specified image has more than one band, 
         Abort with an error message.
      Get the sun angles, azimuth angles, and exponent factors.
   End-Modify
   Get the output image name.
   Get the type of the convolution to be performed.
   Get the ground dimensions of the pixels.
   Get the mask value, offset value, and the input and output scale factors.
   Get the LAS global parameters.

----------------------------------------------------------------------------- */
#include "picshade.h"

void getpar
(
    char subcmd[],       /* Subcommand */
    char lasin[][CMLEN],        /* LAS name for input elevation image */
    char hosin[][CMLEN],        /* Host name for input elevation image */
    char lasin2[][CMLEN],    /* LAS names for input composite image */
    char hosin2[][CMLEN],    /* Host names for input composite images */
    char lasout[][CMLEN],       /* Output image LAS name */
    char hosout[][CMLEN],       /* Output image host name */
    long window[MAXIMG][4],  /* Window specifications for 1st image */
    long window2[MAXIMG][4],     /* Window specifications for 2nd image */
    long *odtype,        /* Output data type */
    float *osunang,       /* Sun angle for the 1st image */
    float *oazmth,        /* Azimuth angle for the 1st image */
    float *oexp,          /* Exponent value for the 1st image */
    float *isunang,       /* Sun angle for the 2nd image */
    float *iazmth,        /* Azimuth angle for the 2nd image */
    float *iexp,          /* Exponent value for the 2nd image */
    float xygrd[2],       /* x and y ground scale factor */
    float *zscale,        /* Input scale factor */
    float *mask,          /* Mask value for non-inclusive data */
    long *method,        /* Method of calculation */
    float *tnsity,        /* Output sun intensity factor */
    float *ambien,        /* Ambient light factor */
    long *mnmx,          /* Min/max value */
    long *msgtim,        /* Time interval for the processing message */
    long *num_inimg,         /* Number of input images from IN(1) */
    long *num_inimg2,        /* Number of input images from IN(2) */
    long *totbnd,        /* Total number of bands in 1st image */
    long *totbnd2       /* Total number of bands in 2nd image */
)
{

long    zero = 0;
long    one = 1;

char    cmeth[6],       /* Convolution method */
minmax[3],      /* Min/max option */
ttype[5];       /* Output data type */

long    imgflag[MAXIMG][2], /* Image window and/or band option flag */
imglim,         /* Maximum number of images */
same,           /* Same size window flag */
status;         /* Return status */


struct  VARIABLE *var; /* Routine that finds variables */

*odtype = 0;
imglim = MAXIMG;

/* Get the subcommand name.
------------------------- */
var = p_find (&block, "_SUBCMD");
strcpy (subcmd, SVAL (*var, 0));

/* Get the parameters for the subcommand CREATE.
Get the name of the input image.
----------------------------------------------- */
if (!strcmp(subcmd, "CREATE"))
    {
    status = c_getin (&block, "IN", &zero, lasin, hosin, num_inimg, window,
    bands, bands_per_img, totbnd, &imglim, &same, imgflag);
    if (status != E_SUCC)
        c_lasmsg(TRUE,"picshade-in","Unable to retrieve input image parameter");
    /* If specified windows differ in size, abort with an error message.
    ----------------------------------------------------------------- */
    if (!same)
        c_lasmsg(TRUE,"picshade_window","Input image windows differ in size");
    
    /* If the specified image has more than one band, abort with error message.
    ------------------------------------------------------------------------- */
    if (*totbnd != 1) 
        c_lasmsg(TRUE,"picshade-cband","IN must be a single band image");
    
    /* Get the sun angle, azimuth angle, and exponent factor.
    ------------------------------------------------------ */
    var = p_find (&block, "SUNANG");
    *osunang = RVAL (*var, 0);
    
    var = p_find (&block, "AZIANG");
    *oazmth = RVAL (*var, 0);
    
    var = p_find (&block, "EXPVAL");
    *oexp = RVAL (*var, 0);
    
    /* Get the output data type.
    --------------------------- */
    var = p_find (&block, "ODTYPE");
    strcpy (ttype, SVAL(*var, 0));
    if (ttype[0] == 'B')
        *odtype = 1;
    else
        *odtype = 4;
    }

/* Get the parameters for the subcommand MODIFY.
Get the names of the input images.
--------------------------------------------- */
else
    {
    status = c_getin (&block, "IN", &zero, lasin, hosin, num_inimg, window,
    bands, bands_per_img, totbnd, &imglim, &same, imgflag);
    if (status != E_SUCC)
        c_lasmsg(TRUE,"picshade-in","Unable to retrieve input image parameter");
    /* If specified windows differ in size, abort with an error message.
    ----------------------------------------------------------------- */
    if (!same)
        c_lasmsg(TRUE,"picshade_window","Input image windows differ in size");
    
    /* If the specified image has more than one band, abort with error message.
    ------------------------------------------------------------------------- */
    if (*totbnd != 1) 
        c_lasmsg(TRUE,"picshade-mband","IN1 must be a single band image");
    
    status = c_getin (&block, "IN", &one, lasin2, hosin2, num_inimg2, window2,
    bands2, bands_per_img2, totbnd2, &imglim, &same, imgflag);
    if (status != E_SUCC)
        c_lasmsg(TRUE,"picshade-in","Unable to retrieve input image parameter");
    /* If specified windows differ in size, abort with an error message.
    ----------------------------------------------------------------- */
    if (!same)
        c_lasmsg(TRUE,"picshade_window","Input image windows differ in size");
    
    /* Get the sun angles, azimuth angles, and exponent factors.
    ---------------------------------------------------------- */
    var = p_find (&block, "SUNANG");
    *osunang = RVAL (*var, 0);
    *isunang = RVAL (*var, 1);
    
    var = p_find (&block, "AZIANG");
    *oazmth = RVAL (*var, 0);
    *iazmth = RVAL (*var, 1);
    
    var = p_find (&block, "EXPVAL");
    *oexp = RVAL (*var, 0);
    *iexp = RVAL (*var, 1);
    }  /* if */

/* Get the output image name.
------------------------------- */
status = c_getout (&block, "OUT", lasout, hosout);
if (status != E_SUCC)
    c_lasmsg(TRUE,"picshade-out","Unable to retrieve output image");

/* Get the type of the convolution to be performed.
------------------------------------------------ */
var = p_find (&block, "CONVMETH");
strcpy(cmeth, SVAL(*var, 0));
if (cmeth[0] == 'O')
    *method = 1;
else
    *method = 3;

/* Get the ground dimensions of the pixels.
----------------------------------------- */
var = p_find (&block, "PIXSIZ");
xygrd[0] = RVAL (*var, 0);
xygrd[1] = RVAL (*var, 1);

/* Get the mask value, offset value, and the input and output scale factors.
-------------------------------------------------------------------------- */
var = p_find (&block, "MASKVAL");
*mask = RVAL (*var, 0);

var = p_find (&block, "OFFSET");
*ambien = RVAL (*var, 0);

var = p_find (&block, "INSF");
*zscale = RVAL (*var, 0);

var = p_find (&block, "OUTSF");
*tnsity = RVAL (*var, 0);

/* Get the LAS global parameters.
---------------------------------- */
var = p_find (&block, "$MSGTIM");
*msgtim = IVAL (*var, 0);

var = p_find  (&block, "$MINMAX");
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y' || minmax[0] == 'y')
    *mnmx = TRUE;
else
    *mnmx = FALSE;
}

