/*------------------------------------------------------------------------------
NAME: OPTION1

FUNCTION:   Calculate the alpha angles.

PROGRAM HISTORY:
  Version   Date       Author           Request
  -------   ----       ------           -------
    1.0       06/27/86     J. Stansbery      
    2.0       08/1/89      B. Allam     conversion

ALGORITHM:

   Allocate dynamic memory for work buffers.
   Initialize the directional derivatives for the two convolution methods.
   Initialize and enable the processing message utility.
   Copy input image lines into work buffer.
   Initialize the matrix and work buffer pointers.
   For all lines:
      Set the first and last line flags.
      For all samples:
         Set upper and lower loop limits.
         If on the first image line, 
            Pad the 1st row in matrix with mask value.
         If on the last image line, 
            Pad the 3rd row in matrix with mask value.
         Fill the 3x3 matrix with pixels from the work buffer.
         If on the 1st sample of a line, 
            Pad 1st column with the mask value.
         If on the last sample of a line, 
            Pad 3rd column with the mask value.
         Call GALPHA1 to calculate the alpha angle according to the method 
         specified, and store the final result in the output buffer.
         Reinitialize the line address pointers.
      End-for
      Write out the output file.
      Reset input buffer pointers.
      Read the next line into the work buffer.
    End-for
    Free the dynamic memory.

------------------------------------------------------------------------------*/
#include "picshade.h"

void option1
(
    long method,         /* Method of calculation */
    float xygrd[2],       /* x and y ground scale factor */
    double mask,           /* Mask value for non-inclusive data */
    double zscale,         /* Input scale factor */
    long odtype,         /* Output data type */
    double oexp,           /* Exponent factor for the output scene */
    double tnsity,         /* Output sun intensity factor */
    double ambien,         /* Ambient light factor */
    long wind[][4],      /* Window specifications */
    float a[],            /* Constants to describe sun direction */
    long msgtim,         /* Time processing message time interval */
    long bnd,            /* Input image band to be read */
    long img            /* Image number */
)
{

long    one = 1;

float   alpha = 0.0,          /* Angle between sun and surface */
fill_val,       /* Fill value for work buffer */
matrix[3][3],       /* 3x3 matrix from input image */
dirx1,          /* Directional derivative in x-direction */
/* for 1x3 convolution */
dirx2,          /* Directional derivative in x-direction */
/* for 3x3 convolution */
diry1,          /* Directional derivative in y-direction */
/* for 1x3 convolution */
diry2;          /* Directional derivative in y-direction */
/* for 3x3 convolution */

long    first,          /* First line flag */
ia, ib, ic, id,     /* Upper and lower limits for looping */
/* through matrix */
icur[3],        /* Pointers to lines */
i, ii, ix, k,   /* Loop counters and array indices */
index,          /* Circular index into work buffer */
iinline,         /* Current line */
ipos, jpos,         /* Positions of pixels in buffer */
last,           /* Last line flag */
line,           /* Line number counter */
msg_flag,       /* Flag for formatting message */
num_lines,      /* Number of image lines */
num_pixels,     /* Number of image pixels */
pix,            /* Current line and sample */
type,           /* Pixel data type */
work_siz;       /* Size of work buffer */


static float    *workbuf[3],    /* 2-dimensional work buffer */
*wbuf;      /* Pointer to work area */

/* Allocate dynamic memory for work buffers.
------------------------------------------ */
num_lines = wind[0][NL];
num_pixels = wind[0][NS];
work_siz = (long) (3 * num_pixels * sizeof(float));
if ((wbuf = (float *) malloc(work_siz)) == NULL)
    msg_err ("!Error: cannot allocate dynamic memory", "option1-alloc");

if ((obuf = (float *) malloc(num_pixels * sizeof (float))) == NULL)
    msg_err ("!Error: cannot allocate dynamic memory", "option1-alloc");

for (i = 0, k = 0; i < 3; i++, k += num_pixels)
    workbuf[i] = &wbuf[k];

/* Initialize the directional derivatives for the two convolution methods.
------------------------------------------------------------------------ */
dirx1 = zscale / (2.0 * xygrd[0]);
diry1 = zscale / (2.0 * xygrd[1]);
dirx2 = zscale / (6.0 * xygrd[0]);
diry2 = zscale / (6.0 * xygrd[1]);
for (i = 0; i < 3; i++)
    for (k = 0; k < 3; k++)
        matrix[i][k] = 0.0;

/* Initialize and enable the processing message utility.
------------------------------------------------------ */
msg_flag = 1;
c_inittm ("PICSHADE", "Processed", "line", "band", "", &num_lines,
&one, &msgtim, &msg_flag);
curent = 1;
curcat = 1;
c_startm();

/* Copy input image lines into work buffer.
----------------------------------------- */
type = EREAL;
fill_val = 0.0;
c_pxfill (workbuf[0], &type, &num_pixels, &fill_val);
for (iinline = 1; iinline <= 2; iinline++)
    {
    c_eread (&fdesc[img], &bnd, &iinline, workbuf[iinline], &one);
    curent++;
    }

/* Initialize the matrix and work buffer pointers.
----------------------------------------------- */
index = -1;
for (i = 0; i < 3; i++)
    icur[i] = i;

/* For all lines:
		Set the first and last line flags.
------------------------------------------ */
for (line = 1; line <= num_lines; line++)
    {
    first = TRUE;
    last = FALSE;
    
    /* For all samples:
    		Set upper and lower loop limits.
    		If on first image line, pad the 1st row in matrix with mask value.
    		If on last image line, pad the 3rd row in matrix with mask value.
    ------------------------------------------------------------------------ */
    for (pix = 1; pix <= num_pixels; pix++)
        {
        if (pix == num_pixels) last = TRUE;
        ia = 1;
        ib = 3;
        ic = 1;
        if (first) ic = 2;
        id = 3;
        if (last) id = 2;
        if (line == 1) 
            {
            ia = 2;
            for (ii = 0; ii < 3; ii++)
                matrix[0][ii] = mask;
            }
        else if (line == num_lines) 
            {
            ib = 2;
            for (ii = 0; ii < 3; ii++)
                matrix[2][ii] = mask;
            }
        
        /* Fill the 3x3 matrix with pixels from the work buffer.
        If on the 1st sample of a line, pad 1st column with the mask value.
        If on the last sample of a line, pad 3rd column with the mask value.
        --------------------------------------------------------------------- */
        for (ix = ia; ix <= ib; ix++)
            {
            jpos = (pix - 1);
            for (ii = ic; ii <= id; ii++)
                {
                ipos = jpos + ii - 2;
                matrix[ix-1][ii-1] = workbuf[icur[ix-1]][ipos];
                }
            if (first)
                matrix[ix-1][0] = mask;
            if (last)
                matrix[ix-1][2] = mask;
            }
        
        /* Call GALPHA1 to calculate the alpha angle according to the method
        specified, and store the final result in the output buffer.
        ------------------------------------------------------------------ */
        galpha1 (matrix, alpha, mask, method, a, dirx1,  diry1, dirx2,
        diry2, oexp, tnsity, ambien, odtype, pix);
        
        /* Reinitialize the line address pointers.
        ----------------------------------------- */
        first = FALSE;
        }
    
    /* Write out the output file.
    --------------------------- */
    c_ewrite (&fdescout, &one, &line, obuf, &one);
    
    /* Reset input buffer pointers.
    Read the next line into the work buffer.
    ------------------------------------------ */
    for (i = 0; i < 3; i++)
        icur[i] = (icur[i] + 1) % 3;
    if (line <= num_lines-2)
        {
        index = (index + 1) % 3;
        c_eread (&fdesc[img], &bnd, &iinline, workbuf[index], &one);
        iinline++;
        curent++;
        }
    }

c_stoptm();

/* Free the dynamic memory.
------------------------ */
free (wbuf);
free (obuf);
}

