/* -----------------------------------------------------------------------------
NAME: OPTION2

FUNCTION:   Calculates the alpha angles.

PROGRAM HISTORY:
  Version   Date       Author           Request
  -------   ----       ------           -------
    1.0       06/27/86     J. Stansbery      
    2.0       08/1/89      B. Allam     Conversion
    3.0       08/8/89      B. Allam         Changed band specification for eread
    4.0       07/28/90     K. Narayanan     Changed band specification 
                                            for elevation image
    5.0       10/16/90     K. Kunska        Changed working buffer allocation
                                            from number_lines x number_samples
                                            to 3 x number_samples.

ALGORITHM:

   Initialize the directional derivatives for the two convolution methods.
   For all bands:
      Initialize the matrix and work buffer pointers.
      Copy the first two input image lines into work buffer.
      For all lines:
         Set the first and last line flags.
         For all samples:
            Set the upper and lower loop limits.
            If on the first line, 
               Pad the 1st row in the matrix with the mask value.
            If on the last line, 
               Pad the 3rd row in the matrix with the mask value.
            Fill the 3x3 matrix with pixels from the work buffer.
            If on the 1st sample of a line, 
               Pad 1st column with the mask value.
            In on the last sample of a line, 
               Pad 3rd column with the mask value.
            Call GALPHA2 to calculate the alpha angles for each input 
            band according to the method specified, and store the final 
            results in the output buffer.
         End-for
         Write out the output line.
         Reset input buffer pointers.
         Read the next line into the work buffer.
      End-for
   End-for
   Free the dynamic memory.
----------------------------------------------------------------------------- */
#include "picshade.h"

void option2
(
    long method,         /* Method of calculation */
    float xygrd[2],       /* x and y ground scale factor */
    double mask,           /* Mask value for non-inclusive data */
    double zscale,         /* Input scale factor */
    long odtype,         /* Output data type */
    double tnsity,         /* Output sun intensity factor */
    double oexp,           /* Exponent factor for the output scene */
    double iexp,           /* Exponent factor for the input scene */
    double ambien,         /* Ambient light factor */
    float a[],    /* Constants to describe sun direction for the output scene */
    float b[],    /* Constants to describe sun direction for the input scene */
    long num_lines,      /* Number of lines in the image */
    long num_pixels,     /* Number of samples in the image */
    long *outband,       /* Output band number */
    long img            /* Current image number */
)
{

long    one = 1;

float   
dirx1,     /* Directional derivative in x-direction */ /* for 1x3 convolution */
dirx2,     /* Directional derivative in x-direction */ /* for 3x3 convolution */
diry1,     /* Directional derivative in y-direction */ /* for 1x3 convolution */
diry2,     /* Directional derivative in y-direction */ /* for 3x3 convolution */
fill_val,       /* Fill value for work buffer */
matrix[3][3],       /* 3x3 matrix from input image */
*workbuf[3],        /* Pointer to 2-dimensional work area */
*workbuf2,      /* Work area for the 2nd image */
*wbuf;          /* Work buffer for IN */

long    band,           /* Current band number being processed */
first,          /* First pixel flag */
ia, ib, ic, id,     /* Upper and lower limits for looping */
/* through matrix */
icur[3],        /* Pointers to lines */
i, ii, ix, k,   /* Loop counters and array indices */
index,          /* Circular index into work buffer */
iinline,         /* Current line */
ipos, jpos,         /* Positions in line */
last,           /* Last pixel flag */
line,           /* Current line */
pix,            /* Current sample */
type,           /* Data type of work buffer */
work_siz;       /* Size of work buffer */

/* Allocate dynamic memory for work buffers.
---------------------------------------- */
work_siz = (long) (3 * num_pixels * sizeof(float));
if ((wbuf = (float *) malloc(work_siz)) == NULL)
    msg_err ("!Error: cannot allocate dynamic memory", "option2-alloc");
for (i = 0, k = 0; i < 3; i++, k += num_pixels)
    workbuf[i] = &wbuf[k];

if ((workbuf2 = (float *) malloc(num_pixels  * sizeof(float))) == NULL)
    msg_err ("!Error: cannot allocate dynamic memory", "option2-alloc");

if ((obuf = (float *) malloc(num_pixels * sizeof (float))) == NULL)
    msg_err ("!Error: cannot allocate dynamic memory", "option1-alloc");

/* Initialize the directional derivatives for the two convolution methods.
------------------------------------------------------------------------ */
dirx1 = zscale / (2.0 * xygrd[0]);
diry1 = zscale / (2.0 * xygrd[1]);
dirx2 = zscale / (6.0 * xygrd[0]);
diry2 = zscale / (6.0 * xygrd[1]);

/* For all bands:
Initialize the matrix and work buffer pointers.
Copy the first two input image lines into work buffer.
------------------------------------------------------- */
for (band = 1; band <= bands_per_img2[img]; band++)
    {
    index = -1;
    for (i = 0; i < 3; i++)
        for (k = 0; k < 3; k++)
            matrix[i][k] = 0.0;
    for (i = 0; i < 3; i++)
        icur[i] = i;
    type = EREAL;
    fill_val = 0.0;
    c_pxfill (workbuf[0], &type, &num_pixels, &fill_val);
    for (iinline = 1; iinline <= 2; iinline++)
        c_eread (&fdesc[0], &one, &iinline, workbuf[iinline], &one);
    
    /* For all lines:
    Set the first and last line flags.
    ----------------------------------- */
    for (line = 1; line <= num_lines; line++)
        {
        first = TRUE;
        last = FALSE;
        c_eread (&fdesc2[img], &bands2[img][band-1], &line, workbuf2, &one);
        curent = line;
        
        /* For all samples:
        Set the upper and lower loop limits.
        If on the first line, pad the 1st row in the matrix with the mask value.
        If on the last line, pad the 3rd row in the matrix with the mask value.
        --------------------------------------------------------------------- */  
        for (pix = 1; pix <= num_pixels; pix++)
            {
            if (pix == num_pixels) last = TRUE;
            ia = 1;
            ib = 3;
            ic = 1;
            if (first) ic = 2;
            id = 3;
            if (last) id = 2;
            if (line == 1) 
                {
                ia = 2;
                for (ii = 0; ii < 3; ii++)
                    matrix[0][ii] = mask;
                }
            else if (line == num_pixels) 
                {
                ib = 2;
                for (ii = 0; ii < 3; ii++)
                    matrix[2][ii] = mask;
                }
            
            /* Fill the 3x3 matrix with pixels from the work buffer.
            If on the 1st sample of a line, pad 1st column with the mask value.
            In on the last sample of a line, pad 3rd column with the mask value.
            ----------------------------------------------------------------- */
            for (ix = ia; ix <= ib; ix++)
                {
                jpos = (pix - 1);
                for (ii = ic; ii <= id; ii++)
                    {
                    ipos = jpos + ii - 2;
                    matrix[ix-1][ii-1] = workbuf[icur[ix-1]][ipos];
                    }
                if (first) matrix[ix-1][0] = mask;
                if (last) matrix[ix-1][2] = mask;
                }
            
            /* Call GALPHA2 to calculate the alpha angles for each input band 
				according to the method specified, and store the final results 
				in the output buffer.
            --------------------------------------------------------------- */
            galpha2 (matrix, method, mask, a, b, dirx1, diry1, dirx2, diry2,
            oexp, iexp, tnsity, ambien, odtype, pix, workbuf2);
            
            first = FALSE;
            }
        
        /* Write out the output line.
        -------------------------- */
        c_ewrite (&fdescout, outband, &line, obuf, &one);
        
        /* Reset input buffer pointers.
        Read the next line into the work buffer.
        ------------------------------------------ */
        for (i = 0; i < 3; i++)
            icur[i] = (icur[i] + 1) % 3;
        if (line <= num_lines-2)
            {
            index = (index + 1) % 3;
            c_eread (&fdesc[0], &one, &iinline, workbuf[index], &one);
            iinline++;
            curent++;
            }
        }
    curcat++;
    *outband = *outband + 1;
    }

/* Free the dynamic memory.
------------------------ */
free (wbuf);
free (workbuf2);
free (obuf);
}

