/* -----------------------------------------------------------------------------

NAME: PICSHADE

FUNCTION:   Creates a shaded terrain image to simulate the effects
            of the illumination of the sun by diffuse scattering or to
            correct an image for the effects of sun illumination.

PROGRAM HISTORY:
  Version   Date       Author           Request
  -------   ----       ------           -------
    1.0       06/27/86     J. Stansbery      
    1.1       07/25/86     J. Lavergne      Changed call to complt
                                            and added chkstat call
                                            following fgcre_group
    2.0       08/1/89      B. Allam     Conversion
    3.0       08/8/89      B. Allam         Changed band specification in 
                                            option2.c
    4.0       07/28/90     K. Narayanan     Changed band specification in
                                            option2.c, for elevation image
    5.0       10/16/90     K. Kunska        Changed working buffer allocation
                                            from number_lines x number_samples
                                            to 3 x number_samples.
    5.1       09/23/92     T. MIttan        Modified include file to eliminate
                        warnings on link.
ALGORITHM:
   Initialize LAS
   Call GETPAR to get the user-supplied parameters.
   Check the image window size and get the data type of the input image.
   If the subcommand chosen was MODIFY, 
      Initialize the output data type and 
   Check the image window size.
   Convert the given angles (osunang, oazmth, isunang, azmth2) from degrees 
   to radians, and calculate the constants that describe the sun direction.
   Call INITIO to open image files
   If the subcommand is CREATE, 
      Call OPTION1 to calculate the alpha angle for each pixel.
   If the subcommand is MODIFY, 
      Call OPTION2 to calculate the alpha angle for each pixel.
   Update the output file history record.
   Update output image ddr min max fields.
   Close the image files.
   Print the completion message.

----------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

static  long    one = 1;

extern long     curcat;         /* current category                  */
extern long     curent;         /* curent entitly                    */

struct PARBLK   block;          /* paramter block                    */
char    out_files[2][CMLEN];    /* output file names                 */

long    bands[MAXIMG][MAXBND+1];  /* band numbers for 1st image      */
long    bands2[MAXIMG][MAXBND+1]; /* band numbers for 2nd image      */
long    bands_per_img[MAXIMG];  /* number of bands in 1st image      */
long    bands_per_img2[MAXIMG]; /* number of bands in 2nd image      */
long    out_img_bands[1][MAXBND+1];  /* band numbers for the output image */
long    obands;                 /* number of output image bands      */
struct FDESC *  fdesc[MAXIMG+1];        /* 1st input file descriptors        */
struct FDESC *  fdesc2[MAXIMG+1];       /* 2nd input file descriptors        */
struct FDESC *  fdescout;               /* output file descriptor            */
float   *obuf;                  /* pointer to the output buffer      */
long    num_files;              /* number of output files            */


int main
(void) /* picshade */
{

char    hosin[1][CMLEN],       /* Host name for input elevation image */
hosin2[MAXIMG][CMLEN],  /* Host names for input composite images */
hosout[1][CMLEN],      /* Output image host name */
lasin[1][CMLEN],       /* LAS name for input elevation image */
lasin2[MAXIMG][CMLEN],  /* LAS names for input composite image */
lasout[1][CMLEN],      /* Output image LAS name */
subcmd[7];      /* Subcommand */

double  omin[MAXBND],       /* Minimum pixel values in the output image */
omax[MAXBND];       /* Maximum pixel values in the output image */

float   a[3],           /* Constants to describe sun direction for */
/* the output scene */
ambien,         /* Ambient light factor */
b[3],           /* Constants to describe sun direction for */
/* the input scene */
iazmth,         /* Azimuth angle for the input scene */
iexp,           /* Exponent factor for the input scene */
isunang,        /* Sun angle for the input scene */
mask,           /* Mask value for non-inclusive data */
oazmth,         /* Azimuth angle for the output scene */
oexp,           /* Exponent factor for the output scene */
osunang,        /* Sun angle for the output scene */
tnsity,             /* Output sun intensity factor */
xygrd[2],       /* x and y ground scale factor */
zscale;         /* Input scale factor */

long    img,            /* Current image being processed */
inband,         /* Input image band */
intype,         /* Data type of the 1st input image */
intype2,        /* Data type of the 2nd input image */
method,         /* Method of calculation */
mnmx,           /* Min/max value */
msgtim,         /* Time processing message time interval      */
msg_flag,       /* Time interval for processing msg utility */
num_inimg,      /* Number of input images from IN(1) */
num_inimg2,     /* Number of input images from IN(2) */
num_lines,      /* Number of lines in the image */
num_pixels = 0,     /* Number of samples in the image */
odtype,         /* Output data type */
outband,        /* Output image band */
std_out,        /* Logical unit for standard output */
totbnd,         /* Total number of bands in the 1st image */
totbnd2,        /* Total number of bands in the 2nd image */
wind[MAXIMG][4],    /* Window specifications for 1st image */
wind2[MAXIMG][4],   /* Window specifications for 2nd image */
windout[1][4];     /* Output image window specifications */

/* Initialize LAS.
---------------- */
num_files = 0;
c_inlas (&block, &std_out);

/* Call GETPAR to get the user-supplied parameters.
------------------------------------------------ */
getpar (subcmd, lasin, hosin, lasin2, hosin2, lasout, hosout, wind, wind2,
&odtype, &osunang, &oazmth, &oexp, &isunang, &iazmth, &iexp, xygrd,
&zscale, &mask, &method, &tnsity, &ambien, &mnmx, &msgtim, &num_inimg,
&num_inimg2, &totbnd, &totbnd2);

/* Check the image window size and get the data type of the input image.
----------------------------------------------------------------------- */
c_gettyp (hosin, &num_inimg, &intype);
if (!strcmp(subcmd, "CREATE"))
    {
    if ((wind[0][NL] < 3) || (wind[0][NS] < 3))
      c_lasmsg(TRUE,"picshade-cwind","IN must be at least 3 lines x 3 samples");
    }
else if ((wind2[0][NL] < 3) || (wind2[0][NS] < 3))
   c_lasmsg(TRUE,"picshade-mwind","IN(1) must be at least 3 lines x 3 samples");


/* If the subcommand chosen was MODIFY, initialize the output data type
and check the image window size.
----------------------------------------------------------------------- */
if (!strcmp(subcmd, "MODIFY"))
    {
    c_gettyp (hosin2, &num_inimg2, &intype2);
    if (odtype == 0)
        odtype = intype2;
    if ((wind[0][NL] != wind2[0][NL]) || (wind[0][NS] != wind2[0][NS]))
       c_lasmsg(TRUE,"picshade-dim","IN2 must have the same dimensions as IN1");
    } 

/* Convert the given angles (osunang, oazmth, isunang, iazmth) from degrees
to radians, and calculate the constants that describe the sun direction.
------------------------------------------------------------------------- */
osunang = 0.017453 * osunang;
oazmth = 0.017453 * oazmth;
a[0] = sin((double)oazmth) * cos((double)osunang);
a[1] = cos((double)oazmth) * cos((double)osunang);
a[2] = sin((double)osunang);

if (!strcmp(subcmd, "MODIFY"))
    {
    isunang = 0.017453 * isunang;
    iazmth = 0.017453 * iazmth;
    b[0] = sin((double)iazmth) * cos((double)isunang);
    b[1] = cos((double)iazmth) * cos((double)isunang);
    b[2] = sin((double)isunang);
    }

/* Call INITIO to open image files
----------------------------------- */
initio (hosin, hosin2, hosout[0], num_inimg, num_inimg2, wind, wind2,
totbnd, totbnd2, odtype, subcmd);

/* If the subcommand is CREATE, call OPTION1 to calculate the alpha angle for 
each pixel.
----------------------------------------------------------------------- */
if (!strcmp(subcmd, "CREATE"))
	{
	for (img = 0; img < num_inimg; img++)
		{
		inband = bands[img][0];
		option1 (method, xygrd, mask, zscale, odtype, oexp, tnsity, ambien,
		wind, a, msgtim, inband, img);
		}
	}

/* If the subcommand is MODIFY, initialize and enable the processing message
utility and call OPTION2 to calculate the alpha angle for each pixel.
------------------------------------------------------------------------- */
else
    {
    msg_flag = 1;
    num_lines = wind[0][NL];
    num_pixels = wind[0][NS];
    c_inittm ("PICSHADE", "Processed", "line", "band", " ", &num_lines,
    &totbnd2, &msgtim, &msg_flag);
    curent = 1;
    curcat = 1;
    outband = 1;
    c_startm();
    for (img = 0; img < num_inimg2; img++)
        option2 (method, xygrd, mask, zscale, odtype, tnsity, oexp,
        iexp, ambien, a, b, num_lines, num_pixels, &outband, img);
    c_stoptm();
    } 

/* Update the output file history record.
--------------------------------------- */
if (!strcmp(subcmd, "CREATE"))
    c_hstout(&block, hosout[0], hosin, &num_inimg, " ");
else
    c_hstout(&block, hosout[0], hosin2, &num_inimg2, " ");

/* Close the image files.
----------------------- */
for (img = 0; img < num_inimg; img++)
    c_eclose(&fdesc[img]);

if (!strcmp(subcmd, "MODIFY"))
    for (img = 0; img < num_inimg2; img++)
        c_eclose(&fdesc2[img]);

c_eclose(&fdescout);

/* Update output image ddr min max fields.
---------------------------------------- */
if (mnmx) 
    {
    windout[0][SL] = windout[0][SS] = 1;
    windout[0][NL] = num_lines;
    windout[0][NS] = num_pixels;
    if (!strcmp(subcmd, "CREATE"))
        {
        c_minmax (hosout, &one, windout, out_img_bands, &obands, omin, omax);
        c_svmnmx (hosout[0], &totbnd, omin, omax, "YES");
        }
    else
        {
        c_minmax (hosout, &one, windout, out_img_bands, &obands, omin, omax);
        c_svmnmx (hosout[0], &totbnd2, omin, omax, "YES");
        }
    }

/* Print the completion message.
------------------------------ */
c_complt("PICSHADE", lasout, &one);

return EXIT_SUCCESS;

}

