/*************************************************************************
   NAME:                        LOGPBIN2


   PURPOSE:  LOGPBIN2 prints the histogram and data for a specified number
             of bins for integer*2 processing in logrithmic format.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <string.h>
#include <math.h>
#include "las.h"
#include "pixcount.h"

void logpbin2 (
   long long *hist2,           /*  I: histogram array                    */
   long   minval,              /*  I: minimum pixel value                */
   long   maxval,              /*  I: maximum pixel value                */
   long   *window,             /*  I: current window specification       */
   long   nbins,               /*  I: number of specified bins           */
   double mean,                /*  I: mean                               */
   double stddev,              /*  I: standard deviation                 */
   float  mode,                /*  I: mode                               */
   float  med,                 /*  I: median                             */
   char   *tname,              /*  I: TAE name of input image            */
   long   bndnum,              /*  I: current band number                */
   char   *htype,              /*  I: histogram type                     */
   long   mskcnt,              /*  I: pixels not in histogram count      */
   FILE   *fp,                 /*  I/O: output file pointer              */
   long   sumry,               /*  I: summary report option              */
   long   prt,                 /*  I: print option                       */
   long   midpt                /*  I: middle point in histogram array    */
   )

   {
   char scale_string[CMLEN];   /* buffer used for scale line output       */
   char scale_string_top[CMLEN]; /* buffer used for scale #s above scale line */
   char scale_string_bottom[CMLEN]; /* used for scale #s below scale line */

   float aprcnt;               /* accumulative percentage of pixel values */
   float logvalue;             /* log10 value of the count of pixels #   */
   float intrvl;               /* interval size                          */
   float prcnt;                /* total percentage of pixel values       */
   float range;                /* range of the image                     */
   float stars_ratio;          /* value used to calculate the number of
                                  stars to output to the histogram       */
   int scale_spacing_flag = 0; /* flag indicating spacing for logpbin2   */   
   long flag;                  /* flag used for pterm                    */
   long long histm[MAXBUF];    /* new histogram array                    */
   long i;                     /* loop index                             */
   long j;                     /* loop index                             */
   long k;                     /* loop index                             */
   long max_nstars_per_page = MAXSTARS;
                               /* maximum number of stars that will fit
                                  on a page                              */
   long long maxpixelcount;    /* the largest total number of pixels
                                      with a unique pixel value          */
   long maxv[MAXBUF];          /* maximum pixel value for the histogram  */
   long minv[MAXBUF];          /* minimum pixel value for the histogram  */
   long nstars = 0;            /* number stars scaled for a pixel value  */
   long long numel;            /* number of pixels to be histogrammed    */
   long page;                  /* number of pages to print               */
   long long total_pixels = 0; /* running total of pixel occurences      */

   flag = 0;

   /*---------------------------------------------------------------------
      calculate the required number of pages needed with 60 lines per page.
      plus 1 page of extra information for no summary.
           2 pages extra pages with the summary information.
   ----------------------------------------------------------------------*/
   if (sumry)
      page = nbins / 60 + 2;
   else
      page = nbins / 60 + 1;

   if (prt)
      c_lasmsg(flag,""," %7d page(s) of data will be printed",page);

   /*  print the heading
   -------------------------------------------------------------------*/
   fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

   if (strcmp(htype,"DIST") == 0)
      fprintf(fp,"          DISTRIBUTION HISTOGRAM\n\n");
   else
      fprintf(fp,"          CUMULATIVE HISTOGRAM\n\n");

   fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",  
                 window[0], window[1]);
   fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                 window[2], window[3]);
   fprintf(fp,"   PIXEL RANGE\n");
   fprintf(fp,"    LOW   HIGH      NO.   TOT. %% ACC. %%");
   fprintf(fp,"                    HISTOGRAM\n");
   fprintf(fp,"   -----------      ---   ------ ------ ");
   fprintf(fp,"                    ---------\n");


   /*   initialize accumulative percentage
   -------------------------------------------------------------------*/
   aprcnt = 0;

   numel = (long long)window[2] * window[3] - mskcnt;

   /*-----------------------------------------------------------------   
        create new histogram with the specified number of bins
           --find the range of the pixel values in the image
           --calculate the appropriate intervals
           --accumulate pixel values and occurences accordingly
   -------------------------------------------------------------------*/
   k = minval + midpt;
   range = (float)(maxval - minval);

   minv[0] = minval;
   intrvl = (range) / (float)nbins;

   for(i = 0; i < nbins; i++)
      {
      histm[i] = 0;

      /*  0.5 was added for round off 
      ---------------------------------------------------------------*/
      maxv[i] = minv[0] + (long)(intrvl * ((float)i + 1.0) + 0.5);
      for(j =  minv[i]; j <= maxv[i]; j++)
         {
         histm[i] += hist2[k];
         k++;
         }
         
      minv[i+1] = maxv[i] + 1;
      }

   /*   find the largest number of the unique pixel value
   --------------------------------------------------------*/
   maxpixelcount = histm[0];
   for(i = 1; i < nbins; i++)
      {
      if (maxpixelcount < histm[i])
         maxpixelcount = histm[i];
      }


   /*   create the scale
   --------------------*/
   logscale(fp, maxpixelcount, scale_string, scale_string_top, 
            scale_string_bottom, &stars_ratio, scale_spacing_flag,
            max_nstars_per_page,numel,htype);

   /*   process each histogram value
   -------------------------------------------------------------------*/
   if (numel > 0)
      {
      for(i = 0; i < nbins; i++) 
         {
         /*   Calculate log of histogram value and figure the total number
              of stars to be output to the histogram.
         -------------------------------------------------------------------*/
         if (strcmp(htype,"DIST") == 0)
            {
           if (histm[i] > 0)
             /* add .5 to round up*/ 
             nstars = (long)(log10(histm[i])  * stars_ratio + .5);
           else
             nstars = 0;
            }

         else
            {
            total_pixels = histm[i] + total_pixels;

            if (total_pixels <= 0)
               logvalue = 0;
            else
               logvalue = log10(total_pixels);
  
            nstars = (long)(logvalue * stars_ratio + .5);
            }

         /*   calculate the total pixels (percent), and keep tract of
              accumulative percent
         ------------------------------------------------------------*/
         prcnt = (histm[i] / (double)numel) * 100; 
         aprcnt += prcnt;

         /*   print the histogram
         -------------------------------------------------------------------*/
         if ((aprcnt ==  100.00) && (i == 0))
            {
            fprintf(fp,"%5d %5d %11lld %6.2f %6.2f  :", minv[i], maxv[i],
                       histm[i], prcnt, aprcnt);
            for(j = 0; j < nstars; j++)
               fprintf(fp,"*");
            fprintf(fp,"\n");
            break;
            }
         fprintf(fp,"%5d %5d %11lld %6.2f %6.2f  :", minv[i], maxv[i],
                    histm[i], prcnt, aprcnt);
         for(j = 0; j < nstars; j++)
            fprintf(fp,"*");
         fprintf(fp,"\n");
         }
      }

   fprintf(fp, "%s\n", scale_string_top);
   fprintf(fp, "%s\n", scale_string);
   fprintf(fp, "%s\n", scale_string_bottom);

   /*   print blank lines
   -------------------------------------------------------------------*/
   fprintf(fp,"\n\n\n\n\n");


   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   fprintf(fp,"  MEAN= %10.3f          MODE= %#8.0f          MEDIAN= %#8.0f\n",
            mean, mode, med);
   fprintf(fp,"  STANDARD DEVIATION= %10.4f\n", stddev);
   }
