/*************************************************************************
   NAME:                        LOGPBIN4


   PURPOSE:  LOGPBIN4 prints the histogram and data for a specified number
             of bins (integer*4 and real*4 data) in logrithmic format.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <string.h>
#include <math.h>
#include "las.h"
#include "pixcount.h"

void logpbin4 (
   double histr[2][MAXBIN],    /*  I: histogram array                    */
   long   icount,              /*  I: number of different pixel values   */
   double dmean,               /*  I: mean                               */
   double dstddv,              /*  I: standard deviation                 */
   float  mode,                /*  I: mode                               */
   float  med,                 /*  I: median                             */
   char   *tname,              /*  I: TAE name of input image            */
   long   *window,             /*  I: current window specification       */
   long   bndnum,              /*  I: current band number                */
   long   dtype,               /*  I: data type                          */
   long   mskcnt,              /*  I: pixels not in histogram count      */
   char   *htype,              /*  I: histogram type                     */
   long   nbins,               /*  I: number of specified bins           */
   FILE   *fp,                 /*  I/O: output file pointer              */
   long   sumry,               /*  I: summary report option              */
   long   prt,                 /*  I: print option                       */
   long   sigdig               /*  I: number of significant digits       */
   )

   {
   char buff[DDLEN];           /* formatted string output                */
   char scale_string[CMLEN];   /* buffer used for scale line output       */
   char scale_string_top[CMLEN]; /* buffer used for scale #s above scale line */
   char scale_string_bottom[CMLEN]; /* used for scale #s below scale line */

   double total_pixels = 0.0;  /* running total of pixel occurences      */
   double histrm[2][MAXBIN];   /* new histogram array                    */

   float aprcnt;               /* accumulated percentage of pixel values */
   float intrvl;               /* interval size                          */
   float logvalue;             /* log10 value of the count of pixels #   */
   float maxv;                 /* maximum value                          */
   float nmoder;               /* R*4 new mode                           */
   float prcnt;                /* total percentage of pixel values       */
   float range;                /* range of the image                     */
   float stars_ratio;          /* value used to calculate the number of
                                  stars to output to the histogram       */
   int scale_spacing_flag = 1; /* flag indicating spacing for logpbin4   */
   long flag;                  /* flag used for pterm                    */
   long i;                     /* loop index                             */
   long j;                     /* loop index                             */
   long k;                     /* loop index                             */
   long max_nstars_per_page = MAXSTARS;
                               /* maximum number of stars that will fit
                                  on a page                              */
   long long maxpixelcount;    /* the largest total number of pixels
                                      with a unique pixel value          */
   long nstars = 0;            /* stars scaled for a pixel value         */
   long long numel;            /* number of pixels to be histogrammed    */
   long page;                  /* number of pages to print               */
   long long temp;             /* temporary variable                     */

   flag = 0; 

   /*---------------------------------------------------------------------
      calculate the required number of pages needed with 60 lines per page.
      plus 1 page of extra information for no summary.
           2 pages extra pages with the summary information.
   ----------------------------------------------------------------------*/
   if (sumry)
      page = nbins / 60 + 2;
   else
      page = nbins / 60 + 1;

   if (prt)
      c_lasmsg(flag,""," %7d page(s) of data will be printed",page);

   /*  print the heading
   -------------------------------------------------------------------*/
   fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

   if (strcmp(htype,"DIST") == 0)
      fprintf(fp,"          DISTRIBUTION HISTOGRAM\n\n");
   else
      fprintf(fp,"          CUMULATIVE HISTOGRAM\n\n");

   fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                 window[0], window[1]);
   fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                 window[2], window[3]);

   fprintf(fp,"  PIXEL RANGE         NO.    TOTAL %%    ACCUM. %%");
   fprintf(fp,"                    HISTOGRAM\n");
   fprintf(fp,"  -----------         ---    -------    --------");
   fprintf(fp,"                    ---------\n");

   /*-----------------------------------------------------------------
        create new histogram with the specified number of bins
           --find the range of the pixel values in the image
           --calculate the appropriate intervals
           --accumulate pixel values and occurences accordingly
           --find the current mode for the new histogram
   -------------------------------------------------------------------*/
   range = histr[0][icount-1] - histr[0][0];
   histrm[0][0] = histr[0][0];
   histrm[1][0] = 0.0;
   intrvl = range / (float)nbins;
   j = 0;
   k = 0;

   for(i = 0; i < nbins; i++)
      { 
      maxv = histrm[k][k] + (intrvl * ((float)i + 1.0));
      
      while ((j <= icount) && (histr[0][j] <= maxv))
         if (histr[0][j] <= maxv)
            {
            histrm[1][i] += histr[1][j];
            j++;
            }
      histrm[0][i+1] = histr[0][k] + (intrvl * ((float)i + 2.0));
      histrm[1][i+1] = histr[1][j];

      if ((i == 0) || (histrm[1][i] > nmoder))
         nmoder = histrm[1][i];
      j++;
      }

   /*   find the largest number of the unique pixel value
   --------------------------------------------------------*/
   maxpixelcount = (long long)histrm[1][0];
   for(i = 1; i <= nbins; i++)
      {
      if (maxpixelcount < (long long)histrm[1][i])
         maxpixelcount = (long long)histrm[1][i];
      }

   /*   initialize accumulative percentage
   -------------------------------------------------------------------*/
   aprcnt = 0.0;

   numel = (long long)window[2] * window[3] - mskcnt;

   /*   create the scale
   --------------------*/
   logscale(fp, maxpixelcount, scale_string, scale_string_top,
            scale_string_bottom, &stars_ratio, scale_spacing_flag,
            max_nstars_per_page,numel,htype);


   /*   process each histogram value
   -------------------------------------------------------------------*/
   if (numel > 0)
      {
      for(i = 0; i < nbins; i++)
         {
         /*   Calculate log of histogram value and figure the total number
              of stars to be output to the histogram.
         -------------------------------------------------------------------*/
         if (strcmp(htype,"DIST") == 0)
            {
           if (histrm[1][i] > 0)
             /* add .5 to round up */ 
             nstars = (long)(log10(histrm[1][i])  * stars_ratio + .5);
           else
             nstars = 0;
            }

         else
            {
            total_pixels = histrm[1][i] + total_pixels;

            if (total_pixels <= 0)
               logvalue = 0;
            else
               logvalue = log10(total_pixels);

            nstars = (long)(logvalue * stars_ratio + .5);
            }

         /*   calculate the total pixels (percent), and keep tract of
              accumulative percent
         ------------------------------------------------------------*/
         prcnt = (histrm[1][i] / (double)numel) * 100;
         aprcnt += prcnt;
         temp = (long long)histrm[1][i];

         /*   print the histogram
         -------------------------------------------------------------------*/
         if (dtype ==  ELONG)
            {
            fprintf(fp,"   %10lld  %10lld     %6.2f      %6.2f   :",
                       (long long)(histrm[0][i]-1), temp, prcnt, aprcnt);
            for(j = 0; j < nstars; j++)
               fprintf(fp,"*");
            fprintf(fp,"\n");
            }
 
         if (dtype == EREAL)
            {
            /*   build string with correct sig. digits, then print
            ----------------------------------------------------------------*/
            sprintf(buff," %c14.%df",'%',sigdig);
            fprintf(fp, buff, histrm[0][i]-1);

            fprintf(fp,"   %11lld %6.2f     %6.2f    :", temp,
                        prcnt, aprcnt);
            for(j = 0; j < nstars; j++)
               fprintf(fp,"*");
            fprintf(fp,"\n");
            }
         }
      }

   fprintf(fp, "%s\n", scale_string_top);
   fprintf(fp, "%s\n", scale_string);
   fprintf(fp, "%s\n", scale_string_bottom);

   /*   print blank lines
   -------------------------------------------------------------------*/
   fprintf(fp,"\n\n\n\n\n");


   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   fprintf(fp,"  MEAN= %11.3f          MODE= %#9.3f          MEDIAN= %#10.3f\n",
            dmean, mode, med);
   fprintf(fp,"  STANDARD DEVIATION= %11.4f", dstddv);
   }

