/*************************************************************************
   NAME:                        LOGPLOT4


   PURPOSE:  LOGPLOT4 creates the print file in logrithmic format with the
             histogram and data of each different pixel value for
             integer*4 / real*4 processing.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <string.h>
#include <math.h>
#include "las.h"
#include "pixcount.h"

void logplot4 (
   double histr[2][MAXBIN],    /*  I: histogram array                    */
   long   icount,              /*  I: number of different pixel values   */
   double dmean,               /*  I: mean                               */
   double dstddv,              /*  I: standard deviation                 */
   float  mode,                /*  I: mode                               */
   float  med,                 /*  I: median                             */
   long   *window,             /*  I: current window specification       */
   char   *tname,              /*  I: TAE name of input image            */
   long   bndnum,              /*  I: current band number                */
   long   dtype,               /*  I: data type                          */
   char   *htype,              /*  I: histogram type                     */
   long   mskcnt,              /*  I: pixels not in histogram count      */
   FILE   *fp,                 /*  I/O: output file pointer              */
   long   sumry,               /*  I: summary report option              */
   long   sigdig,              /*  I: number of significant digits       */
   long long maxpixelcount     /*  I: the largest total number of pixels
                                      with a unique pixel value          */

   )

   {
   char  buff[DDLEN];          /* formatted string output                */
   char scale_string[CMLEN];   /* buffer used for scale line output      */
   char scale_string_top[CMLEN]; /* buffer used for scale #s above scale line */
   char scale_string_bottom[CMLEN]; /* used for scale #s below scale line*/

   float aprcnt;               /* accumulated percentage of pixel values */
   float logvalue;             /* log10 value of the count of pixels #   */
   float prcnt;                /* total percentage of pixel values       */
   float stars_ratio;          /* value used to calculate the number of
                                  stars to output to the histogram       */
   
   double total_pixels = 0.0;  /* running total of pixels                */

   int scale_spacing_flag = 1; /* flag used for spacing for logplot4     */  
   long flag;                  /* flag used for pterm                    */
   long i;                     /* loop index                             */
   long j;                     /* loop index                             */
   long max_nstars_per_page = MAXSTARS;
                               /* maximum number of stars that will fit
                                  on a page                              */
   long nstars = 0;            /* stars scaled for a pixel value         */
   long long numel;                 /* number of pixels to be histogrammed    */
   long page;                  /* number of pages to print               */

   flag = 0;

   /*---------------------------------------------------------------------
      calculate the required number of pages needed with 60 lines per page.
      plus 1 page of extra information for no summary.
           2 pages extra pages with the summary information.
   ----------------------------------------------------------------------*/
   if (sumry)
      page = icount / 60 + 2;
   else
      page = icount / 60 + 1;

   c_lasmsg(flag,""," %7d page(s) of data will be printed",page);

   /*  print the heading
   -------------------------------------------------------------------*/
   fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

   if (strcmp(htype,"DIST") == 0)
      fprintf(fp,"          DISTRIBUTION HISTOGRAM\n\n");
   else
      fprintf(fp,"          CUMULATIVE HISTOGRAM\n\n");


   fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                 window[0], window[1]);
   fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                 window[2], window[3]);

   fprintf(fp,"    PIXEL VALUE         NO.   TOTAL %%   ACCUM. %%\n");
   fprintf(fp,"    -----------         ---   -------   --------\n");

   /*   initialize accumulative percentage
   -------------------------------------------------------------------*/
   aprcnt = 0.0;

   numel = (long long)window[2] * window[3] - mskcnt;

   /*   create the scale
   --------------------*/
   logscale(fp, maxpixelcount, scale_string, scale_string_top,
            scale_string_bottom, &stars_ratio, scale_spacing_flag,
            max_nstars_per_page,numel,htype);

   /*   process each histogram value
   -------------------------------------------------------------------*/
   if (numel > 0)
      {
      for(i = 0; i < icount; i++)
         {
         /*   Calculate log of histogram value and figure the total number
              of stars to be output to the histogram.
         -------------------------------------------------------------------*/
         if (strcmp(htype,"DIST") == 0)
            {
            if (histr[1][i] > 0.0)
               /* add .5 to round up */
               nstars = (long)(log10(histr[1][i])  * stars_ratio + .5);
            else
               nstars = 0;
            }
         else
            {
            total_pixels = histr[1][i] + total_pixels;

            if (total_pixels <= 0.0)
               logvalue = 0;
            else
               logvalue = log10(total_pixels);
            
            nstars = (long)(logvalue * stars_ratio + .5);
            }

         /*   calculate the total pixels (percent), and keep tract of
              accumulative percent
         ------------------------------------------------------------*/
         prcnt = (histr[1][i] / (double)numel) * 100;
         aprcnt += prcnt;

         /*   print the histogram if ELONG type.
         -------------------------------------------------------------------*/
         if (dtype ==  ELONG)
            {
            if (nstars < 1)
               fprintf(fp," %14d   %10lld   %6.2f     %6.2f   :\n",
                   (long)histr[0][i]-1,(long long)histr[1][i], prcnt, 
                    aprcnt);
            else
               {
               fprintf(fp," %14d   %10lld   %6.2f     %6.2f   :",
                      (long)histr[0][i]-1, (long long)histr[1][i], prcnt, 
                      aprcnt);
               for(j = 0; j < nstars; j++)
                  fprintf(fp,"*");
               fprintf(fp,"\n");
               }
            }

         /*   print the histogram if EREAL type
         -------------------------------------------------------------------*/
         if (dtype == EREAL)
            {
            /*--------------------------------------------------------------
                generate the format based upon the number of significant
                digits specified.
            --------------------------------------------------------------*/
            sprintf(buff," %c14.%df",'%',sigdig);
            fprintf(fp, buff, histr[0][i]-1);

            if (nstars < 1)
               fprintf(fp,"   %10lld   %6.2f     %6.2f   :\n", 
                           (long long)histr[1][i], prcnt, aprcnt);
            else
               {
               fprintf(fp,"   %10lld   %6.2f     %6.2f   :",  
                           (long long)histr[1][i], prcnt, aprcnt);
               for(j = 0; j < nstars; j++)
                  fprintf(fp,"*");
               fprintf(fp,"\n");
               }
            }
         }
      }

   /* print to file */
   fprintf(fp, "%s\n", scale_string_top);
   fprintf(fp, "%s\n", scale_string);
   fprintf(fp, "%s\n", scale_string_bottom);

   /*   print blank lines
   -------------------------------------------------------------------*/
   fprintf(fp,"\n\n\n\n\n");

   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   fprintf(fp,"  MEAN= %11.3f          MODE= %#9.3f          MEDIAN= %#9.3f\n",
            dmean, mode, med);
   fprintf(fp,"  STANDARD DEVIATION= %11.3f", dstddv);
   }
