/*************************************************************************
     NAME                      PIXCOUNT

     PURPOSE     PIXCOUNT generates a histogram or pixel count listing
                 to the terminal, printer, or a file.  The mean, mode,
                 median, and standard deviation are also calculated and
                 printed.  The input image data type may be BYTE, INTEGER*2,
                 INTEGER*4, or REAL*4 and may be a single- or multi-band
                 image.

     PROGRAM HISTORY
       PROGRAMMER        DATE          REASON
       ----------        ----          ------
       J. Peterson    Jan.  1983     original development
       K. Zanter      Sept. 1986     CIPS conversion to UNIX
       R. McKinney    Dec.  l986     Changed MASK value to allow for a range
       R. McKinney    March 1989     Converted to LAS 5.0 standards
       B. Ailts       May   1989     Nows reads the last line of data
       D. Etrheim     July  1989     PRF#4971 Corrected problem on -BCKGRD
                                     where mask value of zero was always
                                     being selected
       D. Etrheim     Aug.  1989     Only read one line at a time to correct
                                     allocation problems that occur when
                                     taking a small window of a large image
       T. Sturman     Oct.  1990     Automatically sets the min and max for
                                     EBYTE data to 0 and 255 without calling
                                     the minmax function to improve performance
       T. Mittan      Oct.  1991     Corrected median
       D. Etrheim     Dec.  1991     On parse of MASKVAL, error out nicely if
                                     unable to read correctly.
       D. Etrheim     Aug. 1994      Correct calling sequence to xrstr for
                                     MASKVAL.  Unpredictable results would
                                     occur if multiple values were used.
       K. Hoback      Jun.  1999     Converted from FORTRAN to C
       D. Hames       June  2000     Modify the logarithmic scale to use
                                     the actual log function to calculate   
                                     the asterisks instead of using an
                                     approximation of a scale . CR#2813        
                                     Also change the block size in getim2 and
				     getim4 from 1 line to 20 lines. CR#2818.

     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        PIXCOUNT must be run under TAE.

     PROJECT                   LAS

     LANGUAGE                  C

     ALGORITHM (FLOW)

     The program begins by getting the user-specified parameters and the
     necessary TAE global variables.  The data type and the minimum and
     maximum values of the input image are determined.  If the input image
     is BYTE, INTEGER*2, or INTEGER*4 in INTEGER*2 range (-32768 to 32767),
     the input image is opened as INTEGER*2, otherwise it is opened as
     REAL*4.  The following process is then repeated for each band:
     1)  get the pixel values and fill the histogram array,  2) determine
     the mean, mode, median, and standard deviation, and 3) call the
     appropriate subroutine(s) to format the desired output for the
     specified destination.  If the terminal is specified, the output is
     printed to the screen interactively during program execution.  Other-
     wise, if the line printer or a file is the destination, the data is
     printed to the file 'PIXCOUNT.PR' and handled by FILEUTIL after
     program completion.
******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "pixcount.h"

extern long curcat;        /* current category being processed             */


void main()
   {
   long    file;           /*  used to determine if disk file needed       */
   long    lp;             /*  used to determine if print file needed      */
   long    nomask;         /*  used of masking                             */
   long    term;           /*  used to determine if terminal output        */
   long    sumry;          /*  used for summary histogram                  */
   long    true = TRUE;    /*  logical value of true                       */
   long    fals = FALSE;   /*  logical value of false                      */

   char hosin[MAXIMG][CMLEN];  /* input image names                        */
   char taein[MAXIMG][CMLEN];  /* input tae image names                    */
   char subcmd[9];         /*  the subcommand used                         */ 
   char option[6];         /*  the list option                             */
   char htype[5];          /*  the type of histogram                       */
   char logopt[4];         /*  the histogram form (log or lin)             */
   char prtfil[1][CMLEN];  /*  the output type of file                     */
   char prtfile[CMLEN];    /*  print file name                             */

   double maxvl[MAXBND];   /*  maximum values for all bands                */
   double mean;            /*  the variable to house the mean              */
   double minvl[MAXBND];   /*  minimum values for all bands                */
   double stddev;          /*  the variable to house the standard dev.     */

   FILE *fp;               /*  output file pointer                         */

   double histr[2][MAXBIN];/*  array for real and i*4 data                  */
   float maskr[THOUS];     /*  array for mask values                       */
   float med;              /*  variable for median value                   */
   float mode;             /*  variable for mode value                     */
   float mnval;            /*  minimum value                               */
   float mxval;            /*  maximum value                               */
   float rrange;           /*  range of values                             */

   long access;            /*  access type for eopenr call                 */
   long astrik;            /*  number for asterisk representation          */
   long band;              /*  band specification for eopenr call          */
   long bands[MAXIMG][MAXBND+1];        /*  bands used for input           */
   long curbnd;            /*  current processing band                     */
   long curimg;            /*  current processing image                    */
   long dtype;             /*  data type of image                          */
   long fatal = -1;        /*  fatal error flag                            */
   long i;                 /*  loop counter                                */
   long icount;            /*  loop counter                                */
   long j;                 /*  loop counter                                */
   long mask[THOUS];       /*  array for mask values for non real data     */
   long maxval;            /*  maximum value                               */
   long long maxpixelcount;/*  the largest total number of pixels          */
   long midpt;             /*  middle point in histogram array             */
   long minval;            /*  minimum value                               */
   long long moccur;       /*  variable for number of occurrences          */
   long msgtim;            /*  global for message timing                   */
   long mskcnt;            /*  number of mask values                       */
   long nbands[MAXIMG];    /*  array of input bands                        */
   long nbins;             /*  number of output bins                       */
   long nlines;            /*  number of lines for each read               */
   long nmrimg;            /*  number of input images                      */
   long one = 1;           /*  value of one                                */
   long range;             /*  range of values for non real data           */
   long sigdig;            /*  number of significant digits                */
   long status;            /*  status of the output file                   */
   long std_out;           /*  standard output                             */
   long totbnd;            /*  total number of bands                       */
   long totcnt;            /*  total number of mask values * 2             */
   long window[MAXIMG][4]; /*  window specified on input                   */
   long smbins;            /*  number of bins                              */
   long zero = 0;          /*  constant zero                               */

   long long hist2[MAXBUF]; /*  array used for byte and i*2 data           */
   struct FDESC *fdesc;    /*  File descriptor                             */
   struct PARBLK parblk;   /*  parameter block structure                   */

   /*   initialize las and output band counter
   -----------------------------------------------------------------------*/
   c_inlas(&parblk,&std_out);

   smbins = MBINS;
   curcat = 1;
   midpt = MIDPT;

   /*   get user-specified parameters
   -----------------------------------------------------------------------*/
   status = getpar(&parblk, subcmd, &msgtim, taein, hosin, window, bands, 
                   &nmrimg, nbands, &totbnd, option, htype, &nbins, maskr, 
                   &nomask, &file, &lp, &term, logopt, &sumry, prtfile, 
                   prtfil, &astrik, &sigdig, &totcnt);
   if (status == E_FAIL)
      c_errmsg("Fatal error encountered", "pixcount-fatal", &fatal);

   /*   open the print file
   -----------------------------------------------------------------------*/
   if (lp || file)
      {
      fp = fopen(prtfile,"w");
      if (fp == NULL)
         c_errmsg("Error opening output file", "pixcount-open", &fatal);
      }

   /*    process the data
   -----------------------------------------------------------------------*/
   for(curimg = 0; curimg < nmrimg; curimg++)
      {
      /*   get the data type of the input image
      --------------------------------------------------------------------*/
      c_gettyp(&hosin[curimg],&one,&dtype);

      /*   get the minmax values
      --------------------------------------------------------------------*/
      if (dtype ==  EBYTE)
         {
         for(i = 0; i < nbands[curimg]; i++)
            {
            minvl[i] = 0;
            maxvl[i] = 255;
            }
         }
      else
         {
         status = c_minmax(&hosin[curimg], &one,
                           &window[curimg], 
                           &bands[curimg], &nbands[curimg], minvl, 
                           maxvl);
         if (dtype == EREAL && sigdig == 0)
            {
            sigdig = 1;
            c_errmsg("Data is EREAL and NSIG = 0", "pixcount-real", &one);
            c_errmsg("Resetting NSIG = 1","informational",&one);
            }
         }

      /*   set the processing data type and output informational message
      --------------------------------------------------------------------*/
      if (dtype == EBYTE || dtype == EWORD)
         dtype = EWORD;
      else if (dtype == ELONG)
         {
         dtype = EWORD;
         for(i = 0; i < nbands[curimg]; i++)
            {
            minval = (long)minvl[i];
            maxval = (long)maxvl[i];
            if ((minval < -MIDPT) || (maxval > MIDPT-1))
               {
               dtype = ELONG;
               c_errmsg("Data is not in the range -32768 < x < 32767", 
                        "informational", &one);
               c_errmsg("Processing may be slower","informational", &one);
               break;
               }
            }
         }
      else if (dtype == EREAL)
         {
         c_errmsg("Data is REAL*4 -- Processing may be slower", 
                  "informational", &one);
         }

      /*   open the image
      --------------------------------------------------------------------*/
      access = IREAD;
      nlines = MAX_LINES_IN_BLOCK;
      status = c_eopenr(&fdesc, hosin[curimg], &band, &window[curimg][0], 
                        &window[curimg][1], &window[curimg][2], 
                        &window[curimg][3], &access, &dtype, &zero, &nlines);
      if (status != E_SUCC)
         {
         c_errmsg("Fatal error encountered--No output created", 
                  "pixcount-fatal", &fatal);
         }


      /*   set the mask
      --------------------------------------------------------------------*/
      for(j = 0; j < totcnt; j++)
         mask[j] = (long)maskr[j];

      mskcnt = 0;

      for(curbnd = 0; curbnd < nbands[curimg]; curbnd++)
         {
         if (dtype == EWORD)
            {
            minval = (long)minvl[curbnd];
            maxval = (long)maxvl[curbnd];

            /*   create histogram array
            --------------------------------------------------------------*/
            status = getim2( bands[curimg][curbnd], window[curimg][2], 
                             window[curimg][3], hist2, subcmd, mask, &mskcnt, 
                             msgtim, totbnd, totcnt, midpt, &fdesc, maxval,
                             minval, &maxpixelcount);
            if (status == E_FAIL)
               c_errmsg("Fatal error encountered", "pixcount-fatal", &fatal);

            /*   calculate the range
            --------------------------------------------------------------*/
            if (strcmp(subcmd, "BCKGRD") == 0)
               range = maxval - minval;
            else
               range = maxval - minval + 1; 

            /*   calculate statistics
            --------------------------------------------------------------*/
            mmmsd2( hist2, minval, maxval, &mean, &stddev, &med, &mode,
                   window[curimg][2], window[curimg][3], mskcnt, midpt);

            if (strcmp(option, "BOTH") == 0)
               {
               if (term)
                  {
                  /*   print the histogram and data to the terminal
                  --------------------------------------------------------*/
                  tplot2( hist2, minval, maxval, mean, stddev, mode, med,
                         taein[curimg], bands[curimg][curbnd], htype, 
                         astrik, midpt);
                  }

               if (lp || file)
                  {
                  /*   print the histogram and data to a file
                  --------------------------------------------------------*/
                  if ((nbins ==  0) || (nbins > range))
                     {
                     if (strcmp(logopt, "LIN") == 0)
                        {
                        if ((sumry) && (range > smbins))
                           {
                           ppbin2( hist2, minval, maxval, window[curimg],
                                  smbins, mean, stddev, mode, med, 
                                  taein[curimg], bands[curimg][curbnd],
                                  htype, mskcnt, subcmd, fp, sumry, fals,
                                  astrik, midpt);
                           }

                        pplot2( hist2, minval, maxval, mean, stddev, mode,
                               med, window[curimg], taein[curimg], 
                               bands[curimg][curbnd], subcmd, htype, mskcnt,
                               fp, sumry, astrik, midpt);
                        }
                     else
                        {
                        if ((sumry) && (range > smbins))
                           {
                           logpbin2( hist2, minval, maxval, window[curimg],
                                    smbins, mean, stddev, mode, med, 
                                    taein[curimg], bands[curimg][curbnd], 
                                    htype, mskcnt, fp, sumry, fals, midpt);
                           }

                        logplot2( hist2, minval, maxval, mean, stddev, mode,
                                 med, window[curimg], taein[curimg], 
                                 bands[curimg][curbnd], subcmd, htype,
                                 mskcnt, fp, sumry, midpt, maxpixelcount);
                        }
                     }
                  else
                     {
                     if (strcmp(logopt, "LIN") == 0)
                        {
                        if ((sumry) && (range > smbins))
                           {
                           ppbin2( hist2, minval, maxval, window[curimg],
                                  smbins, mean, stddev, mode, med, 
                                  taein[curimg], bands[curimg][curbnd],
                                  htype, mskcnt, subcmd, fp, sumry, fals,
                                  astrik, midpt);
                           }

                        ppbin2( hist2, minval, maxval, window[curimg], nbins, 
                               mean, stddev, mode, med, taein[curimg], 
                               bands[curimg][curbnd], htype, mskcnt, 
                               subcmd, fp, sumry, true, astrik, midpt);
                        }  
                     else
                        {
                        if ((sumry) && (range > smbins))
                           {
                           logpbin2( hist2, minval, maxval, window[curimg],
                                    smbins, mean, stddev, mode, med, 
                                    taein[curimg], bands[curimg][curbnd], 
                                    htype, mskcnt, fp, sumry, fals, midpt);
                           }

                        logpbin2( hist2, minval, maxval, window[curimg],
                                 nbins, mean, stddev, mode, med, 
                                 taein[curimg], bands[curimg][curbnd], 
                                 htype, mskcnt, fp, sumry, true, midpt);
                        }
                     }
                  }
               }
            else
               {
               prpix2( hist2, minval, maxval, mean, stddev, mode, med,
                      taein[curimg], bands[curimg][curbnd], 
                      window[curimg], mskcnt, htype, subcmd, file, lp,
                      term, fp, midpt);
               }
            }

         if ((dtype == ELONG) || (dtype == EREAL))
            {
            mnval = (float)minvl[curbnd];
            mxval = (float)maxvl[curbnd];
            rrange = 0.0;

            /*   calculate the range
            --------------------------------------------------------------*/
            rrange = mxval - mnval + 1;

            /*   create histogram array
            --------------------------------------------------------------*/
            status = getim4( histr, &icount, window[curimg], dtype, subcmd, 
                             maskr, &mskcnt, bands[curimg][curbnd], totbnd, 
                             msgtim, mnval, mxval, rrange, totcnt, &fdesc, 
                             &maxpixelcount);
            if (status == E_FAIL)
               c_errmsg("Fatal error encountered", "pixcount-fatal", &fatal);

            /*   calculate statistics
            --------------------------------------------------------------*/
            mmmsd4( histr, icount, &mean, &stddev, &med, &mode, &moccur,
                   window[curimg], mskcnt);

            if (strcmp(option, "BOTH") == 0)
               {
               if (term)
                  {
                  /*   print the histogram and data to the terminal
                  -----------------------------------------------------*/
                  tplot4( histr, icount, mean, stddev, mode, med,
                         taein[curimg], bands[curimg][curbnd], dtype,
                         htype, astrik, mnval, mxval, sigdig);
                  }

               if (lp || file)
                  {
                  /*   print the histogram and data to a file
                  --------------------------------------------------*/
                  if ((nbins == 0) || (nbins > icount))
                     {
                     if (strcmp(logopt, "LIN") == 0)
                        {
                        if ((sumry) && (icount > smbins))
                           {
                           ppbin4( histr, icount, mean, stddev, mode, 
                                  med, taein[curimg], window[curimg], 
                                  bands[curimg][curbnd], dtype, mskcnt,
                                  htype, subcmd, smbins, fp, sumry,
                                  fals, astrik, sigdig);
                           }

                        pplot4( histr, icount, mean, stddev, mode, 
                               med, window[curimg], taein[curimg], 
                               bands[curimg][curbnd], dtype, subcmd, 
                               htype, moccur, mskcnt, fp, sumry, 
                               astrik, rrange, sigdig);
                        }
                     else
                        {
                        if ((sumry) && (icount > smbins))
                           {
                           logpbin4( histr, icount, mean, stddev, mode, 
                                    med, taein[curimg], window[curimg], 
                                    bands[curimg][curbnd], dtype, 
                                    mskcnt, htype, smbins, fp, sumry,
                                    fals, sigdig);
                           }

                        logplot4( histr, icount, mean, stddev, mode, 
                                 med, window[curimg], taein[curimg], 
                                 bands[curimg][curbnd], dtype, htype,
                                 mskcnt, fp, sumry, sigdig, maxpixelcount);
                        }
                     }
                  else
                     {
                     if (strcmp(logopt, "LIN") == 0)
                        {
                        if ((sumry) && (icount > smbins))
                           {
                           ppbin4( histr, icount, mean, stddev, mode, 
                                  med, taein[curimg], window[curimg], 
                                  bands[curimg][curbnd], dtype, mskcnt,
                                  htype, subcmd, smbins, fp, sumry,
                                  fals, astrik, sigdig);
                           }

                        ppbin4( histr, icount, mean, stddev, mode, 
                               med, taein[curimg], window[curimg], 
                               bands[curimg][curbnd], dtype, mskcnt,
                               htype, subcmd, nbins, fp, sumry,
                               true, astrik, sigdig);
                        } 
                     else
                        {
                        if ((sumry) && (icount > smbins))
                           {
                           logpbin4( histr, icount, mean, stddev, mode, 
                                    med, taein[curimg], window[curimg], 
                                    bands[curimg][curbnd], dtype, 
                                    mskcnt, htype, smbins, fp, sumry,
                                    fals, sigdig);
                           }

                        logpbin4( histr, icount, mean, stddev, mode, 
                                 med, taein[curimg], window[curimg], 
                                 bands[curimg][curbnd], dtype, mskcnt, 
                                 htype, nbins, fp, sumry, true, 
                                 sigdig);
                        }
                     }
                  }
               }
            else
               {
               if (dtype == ELONG)
                  {
                  prpix4( histr, icount, mean, stddev, mode, med, 
                         taein[curimg], window[curimg], 
                         bands[curimg][curbnd], mskcnt, htype,
                         subcmd, file, lp, term, fp);
                  }

               if (dtype == EREAL)
                  {
                  prpixr( histr, icount, mean, stddev, mode, med, 
                         taein[curimg], window[curimg], 
                         bands[curimg][curbnd], mskcnt, htype,
                         subcmd, file, lp, term, fp, sigdig);
                  }
               }
            }
         curcat++;
         }
      
      /*   close the image
      --------------------------------------------------------------------*/
      status = c_eclose(&fdesc);
      if (status != E_SUCC)
         c_errmsg("Nonfatal error encountered","pixcount-warn",&one);
      }

   /*   close the print file
   -----------------------------------------------------------------------*/
   icount = 0;
   if (lp || file)
      {
      fclose(fp);
      if (file) icount = 1;
      }

   /*   print a completion message
   -----------------------------------------------------------------------*/
   c_complt("PIXCOUNT",prtfil,&icount); 
   }
