/*************************************************************************
   NAME:                        PPBIN2


   PURPOSE:  PPBIN2 prints the histogram and data for a specified number
             of bins for integer*2 processing.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "pixcount.h"

void ppbin2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   long    *window,            /*  I: current window specification       */
   long    nbins,              /*  I: number of specified bins           */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   char    *htype,             /*  I: histogram type                     */
   long    mskcnt,             /*  I: pixels not included in histogram   */
   char    *subcmd,            /*  I: subcommand specified               */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    prt,                /*  I: print option                       */
   long    astrik,             /*  I: number of occurrences / asterisk   */
   long    midpt               /*  I: middle point in histogram array    */
   )

   {
   float   aprcnt;               /* accumulated % of pixel values        */
   float   intrvl;               /* interval size                        */
   float   prcnt;                /* total percentage of pixel values     */
   float   range;                /* range of the image                   */

   long    flag;                 /* flag used for pterm                  */
   long long histm[MAXBUF];      /* new histogram array                  */
   long    i;                    /* loop index                           */
   long    j;                    /* loop index                           */
   long    k;                    /* loop index                           */
   long    maxv[MAXBUF];         /* maximum pixel value in histogram     */
   long    minv[MAXBUF];         /* minimum pixel value in histogram     */
   long long nmode;              /* new mode                             */
   long long nstars;             /* stars scaled for a pixel value       */
   long long numel;              /* number of pixels to be histogrammed  */
   long    page;                 /* number of pages to print             */
   long long roccur;             /* running total of pixel occurences    */
   long long scale;              /* scale of * to number of occurences   */


   flag = 0;

   /*---------------------------------------------------------------------
      calculate the required number of pages needed with 60 lines per page.
      plus 1 page of extra information for no summary.
           2 pages extra pages with the summary information.
   ----------------------------------------------------------------------*/
   if (sumry)
      page = nbins / 60 + 2;
   else
      page = nbins / 60 + 1;

   if (prt)
      c_lasmsg(flag,""," %7d page(s) of data will be printed",page);

   /*  print the heading
   -------------------------------------------------------------------*/
   fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

   if (strcmp(htype,"DIST") == 0)
      fprintf(fp,"          DISTRIBUTION HISTOGRAM\n");
   else
      fprintf(fp,"          CUMULATIVE HISTOGRAM\n");

   if (strcmp(subcmd,"BCKGRD") == 0)
      fprintf(fp,"          NUMBER OF BACKGROUND VALUES = %10d\n\n",mskcnt);
   else
      fprintf(fp,"\n\n");

   fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                 window[0], window[1]);
   fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                 window[2], window[3]);
   fprintf(fp,"   PIXEL RANGE\n");
   fprintf(fp,"    LOW   HIGH      NO.   TOT. %% ACC. %%");
   fprintf(fp,"                    HISTOGRAM\n");
   fprintf(fp,"   -----------      ---   ------ ------");
   fprintf(fp,"                    ---------\n");
 
  /*-----------------------------------------------------------------
        create new histogram with the specified number of bins
           --find the range of the pixel values in the image
           --calculate the appropriate intervals
           --accumulate pixel values and occurences accordingly
   -------------------------------------------------------------------*/
   k = (minval + midpt);
   range = (float)(maxval - minval);

   minv[0] = minval;
   intrvl = range / (float)nbins;

   for(i = 0; i < nbins; i++)
      {
      histm[i] = 0;
      /*--------------------------------------------------------------   
          the 0.01 was added to account for round off error between 
          FORTRAN and C calculations.
      ---------------------------------------------------------------*/
      maxv[i] = minv[0] + (long)(intrvl * ((float)i + 1) + 0.01);
      for(j = minv[i]; j <= maxv[i]; j++)
         {
         histm[i] += hist2[k];
         k++;
         }

      /*   find the current mode of the new histogram
      -------------------------------------------------------------------*/
      if ((i == 0) || (histm[i] > nmode))
         nmode = histm[i];

      minv[i+1] = maxv[i] + 1;
      }

   /*   initialize accumulative percentage
   -------------------------------------------------------------------*/
   aprcnt = 0;

   /*   calculate the scale
   -------------------------------------------------------------------*/
   numel = (long long)window[2] * window[3] - mskcnt;
 
   if (astrik == 0)
      {
      if (strcmp(htype,"DIST") == 0)
         scale = (nmode / 85) + 1;
      else
         scale = (numel / 85) + 1;
      }
   else
      scale = astrik;

   roccur = 0;

   /*   process each histogram value
   -------------------------------------------------------------------*/
   if (numel > 0)
      {
      for(i = 0; i < nbins; i++)
         {
         /*---------------------------------------------------------------
           calculate the number of stars according to scale specifications,
           ratio of histogram value to number of total pixels (percent), and
           accumulative percent
         -------------------------------------------------------------------*/
         roccur += histm[i];
         if (strcmp(htype,"DIST") == 0)
            nstars = histm[i] / scale;
         else
            nstars = roccur / scale;

         prcnt = (histm[i] / (double)numel) * 100;
         aprcnt += prcnt;

         /*   print the histogram
         -------------------------------------------------------------------*/
         if ((aprcnt == 100.00) && (i == 0))
            {
            fprintf(fp,"%5d %5d  %10lld   %6.2f %6.2f  :", minv[i], maxv[i],
                       histm[i], prcnt, aprcnt);
            for(j = 0; j < nstars; j++)
               fprintf(fp,"*");
            fprintf(fp,"\n");
            break;
            }

         if (nstars < 1)
            fprintf(fp,"%5d %5d  %10lld   %6.2f %6.2f  :\n", minv[i], maxv[i],
                       histm[i], prcnt, aprcnt);
         else
            {
            if (nstars > 85) nstars = 85;
            fprintf(fp,"%5d %5d  %10lld   %6.2f %6.2f  :", minv[i], maxv[i],
                       histm[i], prcnt, aprcnt);
            for(j = 0; j < nstars; j++)
               fprintf(fp,"*");
            fprintf(fp,"\n");
            }
         }
      }

   /*   print blank lines
   -------------------------------------------------------------------*/
   fprintf(fp,"\n\n\n\n\n");

   /*   print scale specifications
   -------------------------------------------------------------------*/
   fprintf(fp,"  SCALE:   1 * = %7lld OCCURENCES\n\n",scale);

   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   fprintf(fp,"  MEAN= %10.3f          MODE= %#8.0f          MEDIAN= %#8.0f\n",
            mean, mode, med);
   fprintf(fp,"  STANDARD DEVIATION= %10.4f\n\n", stddev);
   }
