 /*************************************************************************
   NAME:                        PPBIN4


   PURPOSE:  PPBIN4 prints the histogram and data for a specified number of
          bins (integer*4 and real*4 data).

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "pixcount.h"

void ppbin4 (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    *window,            /*  I: current window specification       */
   long    bndnum,             /*  I: current band number                */
   long    dtype,              /*  I: data type                          */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   char    *htype,             /*  I: histogram type                     */
   char    *subcmd,            /*  I: subcommand specified               */
   long    nbins,              /*  I: number of specified bins           */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    prt,                /*  I: print option                       */
   long    astrik,             /*  I: number of occurrences per asterisk */
   long    sigdig              /*  I: number of significant digits       */
   )

   {
   char    buff[DDLEN];        /*  formatted string output               */

   float   aprcnt;             /*  accumulated percentage of pixel values */
   double  histrm[2][MAXBIN];  /*  new R*4 histogram array                */
   float   prcnt;              /*  total percentage of pixel values       */
   float   roccur;             /*  running total of pixel occurences      */
   float   temp;               /*  temporary variable                     */

   long    i;                  /*  loop index                             */
   long    indx;               /*  index counter                          */
   long    intrvl;             /*  interval size                          */
   long    itemp;              /*  temporary variable                     */
   long long itemp2;           /*  temporary variable                     */
   long    j;                  /*  loop index                             */
   long    moccur;             /*  number of occurrences                  */
   long    nstars;             /*  stars scaled for a pixel value         */
   long long numel;            /*  number of pixels to be histogrammed    */
   long    page;               /*  number of pages to print               */
   long long scale;            /*  scale of * to number of occurences     */
   long    soccur;             /*  number of occurrences                  */
   long    flag;               /*  flag used for pterm                    */

   flag = 0;

   /*---------------------------------------------------------------------
      calculate the required number of pages needed with 60 lines per page.
      plus 1 page of extra information for no summary.
           2 pages extra pages with the summary information.
   ----------------------------------------------------------------------*/
   if (sumry)
      page = nbins / 60 + 2;
   else
      page = nbins / 60 + 1;

   if (prt)
      c_lasmsg(flag,""," %7d page(s) of data will be printed",page);

   /*  print the heading
   -------------------------------------------------------------------*/
   fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

   if (strcmp(htype,"DIST") == 0)
      fprintf(fp,"          DISTRIBUTION HISTOGRAM\n");
   else
      fprintf(fp,"          CUMULATIVE HISTOGRAM\n");

   if (strcmp(subcmd,"BCKGRD") == 0)
      fprintf(fp,"          NUMBER OF BACKGROUND VALUES = %10d\n\n",mskcnt);
   else
      fprintf(fp,"\n\n");

   fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                 window[0], window[1]);
   fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                 window[2], window[3]);

   fprintf(fp,"  PIXEL RANGE         NO.    TOTAL %%    ACCUM. %%");
   fprintf(fp,"                    HISTOGRAM\n");
   fprintf(fp,"  -----------         ---    -------    --------");
   fprintf(fp,"                    ---------\n");

  /*-----------------------------------------------------------------
        create new histogram with the specified number of bins
           --calculate the appropriate intervals
           --accumulate pixel values and occurences accordingly
           --find the current mode for the new histogram
   -------------------------------------------------------------------*/

   /*   initialize local variables
   -------------------------------------------------------------------*/
   for(i = 0; i < nbins; i++)
      histrm[0][i] = histrm[1][i] = 0;

   roccur = 0;
   moccur = 0;

   /*   calculate interval size
   -------------------------------------------------------------------*/
   intrvl = (icount  / nbins) + 1;

   for(i = 0, indx = 0; i < icount; (i += intrvl), indx++)
      {
      /*   initialize occurences in an interval
      ----------------------------------------------------------------*/
      soccur = 0;
      histrm[0][indx] = histr[0][i];

      /*   calculate all occurences in given interval
      ----------------------------------------------------------------*/
      for(j = i; j < (i + intrvl); j++)
         soccur += histr[1][j];

      /*   place occurences in array
      ----------------------------------------------------------------*/
      histrm[1][indx] = soccur;
      roccur += (float)soccur;

      /*----------------------------------------------------------------
      calculate maximum number of pixels in
      intervals encountered so far
      ----------------------------------------------------------------*/
      if (moccur < soccur) moccur = soccur;
      }
 
   /*   initialize accumulative percentag
   -------------------------------------------------------------------*/
   aprcnt = 0;

   /*   calculate the scale
   -------------------------------------------------------------------*/
   numel = (long long)window[2] * window[3] - mskcnt;

   if (astrik == 0)
      {
      if (strcmp(htype,"DIST") == 0)
         scale = (long)(moccur / 80.0) + 1;
      else
         scale = (numel / 80) + 1;
      }
   else
      scale = astrik;

   roccur = 0.0;
 
   /*   process each histogram value
   -------------------------------------------------------------------*/
   if (numel > 0)
      {
      for(i = 0; i < indx; i++)
         {
         itemp = (long)histrm[0][i]; 
         histrm[0][i] -= itemp;
         temp = (histrm[0][i] * pow(10,sigdig)); 
         temp = temp / (float)pow(10,sigdig) + (float)itemp;
         itemp2= (long long)histrm[1][i];

         /*-------------------------------------------------------------------
            calculate the number of stars according to scale specifications,
            ratio of histogram value to number of total pixels (percent), and
            accumulative percent
         -------------------------------------------------------------------*/
         roccur += histrm[1][i];
         if (strcmp(htype,"DIST") == 0)
            nstars = (long)(histrm[1][i] / scale);
         else
            nstars = (long)(roccur / scale);
         prcnt = (histrm[1][i] / (double)numel) * 100.;
         aprcnt += prcnt;

         /*---------------------------------------------------------------
              print the histogram
              itemp is modified to make up for the nonzero bin value
              histogram count
         ---------------------------------------------------------------*/
         if (dtype ==  ELONG)
            {
            if (nstars < 1)
               fprintf(fp,"   %10d  %11lld   %6.2f      %6.2f   :\n",
                        (itemp - 1), itemp2, prcnt, aprcnt);
            else 
               {
               if (nstars > 80) nstars = 80;
               fprintf(fp,"   %10d  %11lld   %6.2f      %6.2f   :",
                        (itemp - 1), itemp2, prcnt, aprcnt);
               for(j = 0; j < nstars; j++)
                  fprintf(fp,"*");
               fprintf(fp,"\n");
               }
            }
         else
            {
            /*-------------------------------------------------------------
               generate the format based upon the number of significant
               digits specified.
            -------------------------------------------------------------*/
            sprintf(buff,"%c13.%df",'%',sigdig);
            fprintf(fp, buff, (temp - 1));

            if (nstars < 1)
               fprintf(fp,"  %11lld   %6.2f     %6.2f   :\n", itemp2,
                           prcnt, aprcnt);
            else
               {
               if (nstars > 80) nstars = 80;
               fprintf(fp,"  %11lld   %6.2f     %6.2f   :", itemp2,
                           prcnt, aprcnt);
               for(j = 0; j < nstars; j++)
                  fprintf(fp,"*");
               fprintf(fp,"\n");
               }
            }
         }
      }

   /*   print blank lines
   -------------------------------------------------------------------*/
   fprintf(fp,"\n\n\n\n\n");

   /*   print scale specifications
   -------------------------------------------------------------------*/
   fprintf(fp,"  SCALE:   1 * = %7lld OCCURENCES\n\n",scale);

   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   fprintf(fp,"  MEAN= %11.3f          MODE= %#9.3f          MEDIAN= %#10.3f\n",
            dmean, mode, med);
   fprintf(fp,"  STANDARD DEVIATION= %11.4f\n\n", dstddv);
   }

