/*************************************************************************
   NAME:                        PPLOT2


   PURPOSE:  PPLOT2 creates the print file with the histogram and data
             of each different pixel value for integer*2 processing.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "pixcount.h"

void pplot2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   long    *window,            /*  I: current window specification       */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   char    *subcmd,            /*  I: subcommand specified               */
   char    *htype,             /*  I: histogram type                     */
   long    mskcnt,             /*  I: pixels not included in count       */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    astrik,             /*  I: number of occurrences / asterisk   */
   long    midpt               /*  I: middle point in histogram array    */
   )

   {
   float   aprcnt;              /*  accumulated % of pixel values        */
   float   prcnt;               /*  total percentage of pixel values     */
   float   range;               /*  range of the image                   */

   long    i;                   /*  loop index                           */
   long    j;                   /*  loop index                           */
   long    nstars;              /*  stars scaled for a pixel value       */
   long long numel;             /*  number of pixels to be histogrammed  */
   long    page;                /*  number of pages to print             */
   long long roccur;            /*  running total of pixel occurences    */
   long long scale;             /*  scale of * to number of occurences   */
   long    flag;                /*  flag used for pterm                  */

   flag = 0;

   if (strcmp(subcmd,"BCKGRD") == 0)
      range = (float)(maxval - minval);
   else
      range = (float)(maxval - minval + 1);

   /*---------------------------------------------------------------------
      calculate the required number of pages needed with 60 lines per page.
      plus 1 page of extra information for no summary.
           2 pages extra pages with the summary information.
   ----------------------------------------------------------------------*/
   if (sumry)
      page = (long)range / 60 + 2;
   else
      page = (long)range / 60 + 1;

   c_lasmsg(flag,""," %7d page(s) of data will be printed",page);

   /*  print the heading
   -------------------------------------------------------------------*/
   fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

   if (strcmp(htype,"DIST") == 0)
      fprintf(fp,"          DISTRIBUTION HISTOGRAM\n");
   else
      fprintf(fp,"          CUMULATIVE HISTOGRAM\n");

   if (strcmp(subcmd,"BCKGRD") == 0)
      fprintf(fp,"          NUMBER OF BACKGROUND VALUES = %10d\n\n",mskcnt);
   else
      fprintf(fp,"\n\n");

   fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                 window[0], window[1]);
   fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                 window[2], window[3]);
   fprintf(fp,"    PIXEL VALUE         NO.   TOTAL %%   ACCUM. %%\n");
   fprintf(fp,"    -----------         ---   -------   --------\n");

   /*   initialize accumulative percentage
   -------------------------------------------------------------------*/
   aprcnt = 0;

   /*   calculate the scale
   -------------------------------------------------------------------*/
   numel = (long long)window[2] * window[3] - mskcnt;

   if (astrik == 0)
      {
      if (strcmp(htype,"DIST") == 0)
         scale = (hist2[(long long)mode + midpt] / 80) + 1;
      else
         scale = (numel / 80) + 1;
      }
   else
      scale = astrik;

   roccur = 0;

   /*   process each histogram value
   -------------------------------------------------------------------*/
   if (numel > 0)
      {
      for(i = (minval + midpt); i <= (maxval + midpt); i++)
         {
         /*---------------------------------------------------------------
           calculate the number of stars according to scale specifications,
           ratio of histogram value to number of total pixels (percent), and
           accumulative percent
         -------------------------------------------------------------------*/
         roccur = roccur + hist2[i];
         if (strcmp(htype,"DIST") == 0)
            nstars = (long)(hist2[i] / scale);
         else
            nstars = (long)(roccur / scale);

         prcnt = (hist2[i] / (double)numel) * 100;
         aprcnt = aprcnt + prcnt;

         /*   print the histogram
         -------------------------------------------------------------------*/
         if (nstars < 1)
            fprintf(fp," %14d   %10lld   %6.2f     %6.2f   :\n", (i - midpt), 
                        hist2[i], prcnt, aprcnt);
         else
            {
            if (nstars > 80) nstars = 80;
            fprintf(fp," %14d   %10lld   %6.2f     %6.2f   :", (i - midpt), 
                        hist2[i], prcnt, aprcnt);
            for(j = 0; j < nstars; j++)
               fprintf(fp,"*");
            fprintf(fp,"\n");
            }
         }
      }

   /*   print blank lines
   -------------------------------------------------------------------*/
   fprintf(fp,"\n\n\n\n\n");

   /*   print scale specifications
   -------------------------------------------------------------------*/
   fprintf(fp,"  SCALE:   1 * = %7lld OCCURENCES\n\n",scale);

   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   fprintf(fp,"  MEAN= %#10.3f          MODE= %#8.0f          MEDIAN= %#8.0f\n",
            mean, mode, med);
   fprintf(fp,"  STANDARD DEVIATION= %#10.3f", stddev);
   }
