/*************************************************************************
   NAME:                        PRPIX2


   PURPOSE:  PRPIX2 prints data for BYTE and INTEGER*2 input image data.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "pixcount.h"


void prpix2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   long    *window,            /*  I: current window specification       */
   long    mskcnt,             /*  I: pixels not included in count       */
   char    *htype,             /*  I: histogram type                     */
   char    *subcmd,            /*  I: subcommand specified               */
   long    file,               /*  I: file output flag                   */
   long    lp,                 /*  I: line printer output flag           */
   long    term,               /*  I: terminal output flag               */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    midpt               /*  I: middle point in histogram array    */
   )

   {
   char    buf[MXCHAR];         /*  message string                       */
   
   double  cum;                 /*  cumulative percentage                */
   double  frac;                /*  fractional percentage                */

   long    flag;                /*  flag for whether "pterm" query user  */
   long    i;                   /*  loop index                           */
   long long numel;               /*  number of pixels to be histogrammed  */

   flag = 0;

   /*  print the heading
   -------------------------------------------------------------------*/
   if (term)
      {
      sprintf(buf,"            IMAGE: %s   BAND: %3d", tname, bndnum);
      c_pterm(buf, &flag);

      if (strcmp(htype,"DIST") == 0)
         sprintf(buf,"            DISTRIBUTION HISTOGRAM");
      else
         sprintf(buf,"            CUMULATIVE HISTOGRAM");
      c_pterm(buf, &flag);

      if (strcmp(subcmd,"BCKGRD") == 0)
         sprintf(buf,"            NUMBER OF BACKGROUND VALUES = %10d",
                     mskcnt);
      else
         sprintf(buf,"\n");
      c_pterm(buf, &flag);

      sprintf(buf,"          STARTING LINE= %7d  STARTING SAMPLE= %7d",
                    window[0], window[1]);
      c_pterm(buf, &flag);

      sprintf(buf,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                    window[2], window[3]);
      c_pterm(buf, &flag);

      sprintf(buf,"         PIXEL VALUE          COUNT            FRACTION");
      c_pterm(buf, &flag);

      sprintf(buf,"         -----------          -----            --------");
      c_pterm(buf, &flag);
      }

   if (lp || file)
      {
      fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

      if (strcmp(htype,"DIST") == 0)
         fprintf(fp,"          DISTRIBUTION HISTOGRAM\n");
      else
         fprintf(fp,"          CUMULATIVE HISTOGRAM\n");

      if (strcmp(subcmd,"BCKGRD") == 0)
         fprintf(fp,"          NUMBER OF BACKGROUND VALUES = %10d\n\n",
                     mskcnt);
      else
         fprintf(fp,"\n\n");
 
      fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                    window[0], window[1]);
      fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                    window[2], window[3]);
      fprintf(fp,"         PIXEL VALUE          COUNT            FRACTION\n");
      fprintf(fp,"         -----------          -----            --------\n");
      }

   /*   calculate number of elements
   -------------------------------------------------------------------*/
   numel = (long long)window[2] * window[3] - mskcnt;

   /*   initialize cumulative percentage
   -------------------------------------------------------------------*/
   cum = 0;

   /*   process each histogram value
   -------------------------------------------------------------------*/
   if (numel > 0)
      for(i = (minval + midpt); i <= (maxval + midpt); i++)
         {
         frac = (hist2[i] / (double)numel);
         cum += frac;

         if (hist2[i] > 0)
            {
            if (term)
               {
               if (strcmp(htype,"DIST") == 0)
                  sprintf(buf,"          %10d       %8lld      %12.6f", 
                                 (i - midpt), hist2[i], frac);
               else
                  sprintf(buf,"          %10d       %8lld      %12.6f",
                                 (i - midpt), hist2[i], cum);
               c_pterm(buf, &flag);
               }

            if (lp || file)
               {
               if (strcmp(htype,"DIST") == 0)
                  fprintf(fp,"          %10d       %8lld      %12.6f\n",
                                 (i - midpt), hist2[i], frac);
               else
                  fprintf(fp,"          %10d       %8lld      %12.6f\n",
                                 (i - midpt), hist2[i], cum);
               }
            }  
         }

   /*   print blank lines
   -------------------------------------------------------------------*/
   if (term)
      {
      sprintf(buf,"\n\n");
      c_pterm(buf, &flag);
      }
   if (lp || file)
      fprintf(fp,"\n\n\n");

   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   if (term)
      {
      sprintf(buf," MEDIAN             = %#8.0f", med);
      c_pterm(buf, &flag);
      sprintf(buf," MODE               = %#8.0f", mode);
      c_pterm(buf, &flag);
      sprintf(buf," MEAN PIXEL VALUE   = %16.8f", mean);
      c_pterm(buf, &flag);
      sprintf(buf," STANDARD DEVIATION = %16.8f", stddev);
      c_pterm(buf, &flag);
      sprintf(buf,"\n");
      c_pterm(buf, &flag);
      }

   if (lp || file)
      {
      fprintf(fp," MEDIAN             = %#8.0f\n", med);
      fprintf(fp," MODE               = %#8.0f\n", mode);
      fprintf(fp," MEAN PIXEL VALUE   = %16.8f\n", mean);
      fprintf(fp," STANDARD DEVIATION = %16.8f\n", stddev);
      fprintf(fp,"\n\n");
      }
   }
