/******************************************************************************
NAME:   			GETPAR 

ALGORITHM:
   Retrieve the required parameters and globals from the TAE parameter block
   Parse the input image specification by calling GETIN which will validate 
    the window and band specifications  
   Checking to ensure that all the input images have equal window sizes  
   Get a maximum window which is a superset of all smaller windows
   Retreive the output image name
   Retreive the output data type
   Retreive the global parameters
   Return
******************************************************************************/
#define MAX_BAND 256

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,	/* I: pointer to TAE parameter block         */
    struct DDR *ddr,            /* I: DDR structure for image file           */
    char *hosin,       	        /* O: host file names of input images        */
    char *taeout,		/* O: TAE file name of output image          */
    char *hosout,		/* O: host file name of output imag          */
    long window[][4],		/* O: window specifications for ea input image*/
    long maxwin[],		/* O: overall window containing parameter
				      windows                                */
    long *totwin,		/* O: total number of windows                */
    long (*bands)[MAXBND+1],	/* O: band specifications for each input image*/
    long *totbnd,		/* O: total number of bands to be processed  */
    long *updatflg,		/* O: Flag to signal update   	             */
    long *inspc,		/* O: Total number lines, samples, and bands */
    float *pixval,		/* O: Specified pixel value   		     */
    long *msgtim,		/* O: elapsed time between display messages  */
    char *minmax		/* O: min max flag buffer                    */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long	element = 0;		/* parameter element number		     */
long    fatal = -1;		/* fatal error flag			     */
long	i;			/* counter -- current image		     */
long    nonfatal = 1;		/* nonfatal error flag			     */
long	status;			/* function return status code               */
long    access;			/* access flag                               */
long    temp;			/* temporary variable                        */
char    msg[ERRLEN];            /* buffer space for error message            */


/* Retrieve output image name
-----------------------------*/
status = c_getout(vblock,"OUT",(char (*)[256])taeout,(char (*)[256])hosout);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","pixfill-fatal",&fatal);
if (strlen(taeout) == 0)
   *updatflg = TRUE;
else 
   *updatflg = FALSE;

/* Retrieve input avhrr archive file name
-----------------------------------------*/
if (*updatflg == TRUE)
   access = IUPDATE;
else
   access = IREAD;
status = c_getnam(vblock,"IN",&element,hosin,"img",&access);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","pixfill-fatal",&fatal);

/* Retrieve windows and check for same size
-------------------------------------------*/
val = p_find(vblock,"SL");
   for (i = 0; i < val->v_count; i++)
     window[i][0] = IVAL(*val,i);
temp = val->v_count;

val = p_find(vblock,"SS");
   for (i = 0; i < val->v_count; i++)
     window[i][1] = IVAL(*val,i);
if (temp != val->v_count)
   {
   c_errmsg("Unequal number of windows","pixfill-uneqwin",&nonfatal);
   c_errmsg("Fatal error encountered","pixfill-fatal",&fatal);
   }

val = p_find(vblock,"NL");
   for (i = 0; i < val->v_count; i++)
     window[i][2] = IVAL(*val,i);
if (temp != val->v_count)
   {
   c_errmsg("Unequal number of windows","pixfill-uneqwin",&nonfatal);
   c_errmsg("Fatal error encountered","pixfill-fatal",&fatal);
   }

val = p_find(vblock,"NS");
   for (i = 0; i < val->v_count; i++)
     window[i][3] = IVAL(*val,i);
if (temp != val->v_count)
   {
   c_errmsg("Unequal number of windows","pixfill-uneqwin",&nonfatal);
   c_errmsg("Fatal error encountered","pixfill-fatal",&fatal);
   }
*totwin = temp;

/* Calculate the max and min lines and samples to open overall window
   ------------------------------------------------------------------ */
/* Calculate starting line
   ----------------------- */
temp = window[0][0];
for (i=0;i<val->v_count;i++)
   if (window[i][0] < temp)
      temp = window[i][0];
maxwin[0] = temp;

/* Calculate starting sample 
   ------------------------- */
temp = window[0][1];
for (i=0;i<val->v_count;i++)
   if (window[i][1] < temp)
      temp = window[i][1];
maxwin[1] = temp;

/* Calculate number lines 
   ---------------------- */
temp = window[0][0] + window[0][2];
for (i=0;i<val->v_count;i++)
   if (window[i][0] + window[i][2] > temp)
      temp = window[i][0] + window[i][2];
temp = temp - maxwin[0];
maxwin[2] = temp;

/* Calculate number samples 
   ------------------------ */
temp = window[0][1] + window[0][3];
for (i=0;i<val->v_count;i++)
   if (window[i][1] + window[i][3] > temp)
      temp = window[i][1] + window[i][3];
temp = temp - maxwin[1];
maxwin[3] = temp;

/* Get ddr image info
   ------------------*/
c_getddr (hosin, ddr);

/* Get the requested bands
-----------------------*/
for (i = 0; i < MAX_BAND+1;  i++)
    bands[0][i] = FALSE;
val = p_find(vblock, "BANDS");
if (val->v_count == 0)
   {
   for (i = 0; i < ddr->nbands; i++)
      bands[0][i] = i + 1;
   *totbnd = ddr->nbands;
   /* Get total lines, total samples and total bands to store in INSPC
   ---------------------------------------------------------------- */
   inspc[0] = maxwin[NL] + maxwin[SL];
   inspc[1] = maxwin[NS] + maxwin[SS];
   inspc[2] = ddr->nbands;
   }
else
   {
   for (i = 0; i < val->v_count; i++)
      {
      temp= IVAL(*val,i);
      if(temp > ddr->nbands)
         {
         sprintf(msg,"Band %d does not exist in image file",temp);
         c_errmsg(msg,"pixfill-nband",&fatal);
         }
      else
         bands[0][i] = temp;
      }
   *totbnd = val->v_count;
   /* Get total lines, total samples and total bands to store in INSPC
   ---------------------------------------------------------------- */
   inspc[0] = maxwin[NL] + maxwin[SL];
   inspc[1] = maxwin[NS] + maxwin[SS];
   inspc[2] = val->v_count;
   }

/* Retrieve pixel value
-----------------------*/
val = p_find(vblock,"PIXVAL");
*pixval = RVAL(*val,0);

/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return (E_SUCC);

}
