/*******************************************************************************

NAME:				PIXFILL

PURPOSE:    Fills specified pixel values into a window.  The input 
	    image may be sub-banded, in which case the 
	    specific bands will be changed according to the 
	    pixel value.  The input image will be updated if no output 
	    image is declared, otherwise the input image is copied to 
	    the output image with the specified window(s) and band(s) 
	    changed to the new pixel value.  The input image may be 
	    BYTE, INTEGER*2, INTEGER*4, or REAL*4.  If an output 
	    image is created, it will be the same data type as the 
	    input image.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE    CODE/CONT	REASON
-------	   ----------      ----	   ---------	------
  1.0	   J. Matzner 	   3/93	    EDC/CSB	Original Development for LAS
  1.1      D. Bridges     10/93     EDC/CSB     Modified so only specified 
                                                bands are filled.
  1.2      D. Bridges      2/94     EDC/CSB     Corrected problem so when 
						BANDS is NULL, all bands will
						be updated. 
  7.0	   D. Hames	  11/99     EDC/CSB	Remove 50000 limits of lines 
						and samples from pixfill.pdf.
						Add prototypes.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PIXFILL must be run under TAE.

PROJECT:			LAS

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
int fdesc[3];			/* image file descriptors used for image I/O */
int gdesc;			/* group descriptor used for image I/O       */
long offset1[MAXBND + 1];       /* offsets into thebuf for input bands 	     */
long offset2[MAXBND + 1];   	/* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

void main
(void)
{
struct PARBLK vblock;		/* TAE parameter block               */
struct DDR    ddr;		/* DDR structure for image file      */

static long bands[1][MAXBND+1]; /* bands to be processed - all images*/
float   pixval;			/* Specified pixel value	     */
long	updatflg;		/* Flag to signal update	     */
long	msgtim;			/* elapsed time between display msg  */
long	nbands[1];		/* number of bands per input image   */
long	one = 1;		/* value of one			     */
long	zero = 0;		/* value of zero		     */
long	std_out;		/* LUN to standard output    	     */
long	totbnd;			/* total number of bands             */
long    inspc[3];     		/* input file specifications         */
long	window[10][4];		/* window array for all windows      */
long    maxwin[4];		/* overall window containing parameter windows*/
long    totwin;			/* total number of windows                    */
long    i;			/* counter variable		     */

char	hosin[CMLEN];		/* host names of input images        */
char	hosout[CMLEN];		/* host name of output image 	     */
char	minmax[4];		/* min max flag                      */
char	taeout[CMLEN];		/* TAE name of output image	     */

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);

/* Get parameters and globals
-----------------------------*/
getpar(&vblock,&ddr,hosin,taeout,hosout,window,maxwin,&totwin,bands
	,&totbnd,&updatflg,inspc,&pixval,&msgtim,minmax);

/* Check the range of Pixval for different data types 
   -------------------------------------------------- */
if (ddr.dtype == 1)
   {
   if ((pixval < 0) || (pixval > 255))
      {
      c_errmsg("Pixval is out of range of data type","pixfill-byte",&one);
      c_errmsg("Fatal error encountered","pixfill-fatal",&zero);
      }
   }
else if (ddr.dtype == 2)
   {
   if ((pixval < -32768) || (pixval > 32767))
      {
      c_errmsg("Pixval is out of range of data type","pixfill-int2",&one);
      c_errmsg("Fatal error encountered","pixfill-fatal",&zero);
      }
   }
else if (ddr.dtype == 3)
   {
   if ((pixval < -2147483648.0) || (pixval > 2147483647.0))
      {
      c_errmsg("Pixval is out of range of data type","pixfill-int4",&one);
      c_errmsg("Fatal error encountered","pixfill-fatal",&zero);
      }
   }
else ;

/* Inspc is the number of total lines and total samples
   ---------------------------------------------------- */
inspc[0] = ddr.nl;
inspc[1] = ddr.ns;

/* Check the  max window
   --------------------- */
chk_window(inspc,maxwin);

/* Check individual windows
   ------------------------ */
for (i=0;i<totwin;i++)
   chk_window(inspc,window[i]);

/* Process image -- update image or create new image
----------------------------------------------------- */
nbands[0] = ddr.nbands;
proces(&vblock,&ddr,hosin,hosout,window,maxwin,&totwin,bands,nbands,totbnd,
         &pixval,&updatflg,msgtim,minmax);

/*  Update history file and output completion message 
----------------------------------------------------- */
if (updatflg == 1)
   {
   strcpy(hosout,"  ");
   if (c_hstout(&vblock,hosin,(char (*)[256])hosout,&zero," ") != E_SUCC)
      c_errmsg("Nonfatal error encountered","pixfill-warn",&one);
   c_complt("PIXFILL",(char (*)[256])taeout,&zero);
   }
else
   {
   if (c_hstout(&vblock,hosout,(char (*)[256])hosin,&one," ") != E_SUCC)
      c_errmsg("Nonfatal error encountered","pixfill-warn",&one);
   c_complt("PIXFILL",(char (*)[256])taeout,&one);
   }
}
