/******************************************************************************
NAME:   			PROCES 

PURPOSE:    To copy an input image into an output image with a specified
	    pixel value sub-banded to the output image.  A maximum of ten
	    windows may be specified.  If no output image is named, the 
	    input image will be updated.  Pixfill relates to copy.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   J.Matzner	   3/93		 EDC/CSB	Original Development
							for ADAPS.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROCES must be run under TAE.

PROJECT:			LAS

ALGORITHM REFERENCES:		none

******************************************************************************/
#define TRUE 1
#define SL   0
#define SS   1
#define NL   2
#define NS   3

#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "util.h"
#include "imageio.h"
#include "protos.h"


void proces
(
    struct PARBLK *vblock,	/* I: TAE parameter block		     */
    struct DDR *ddr,            /* I: DDR structure for image file           */
    char *hosin,		/* I: host names of input image		     */
    char *hosout,		/* I: host name of output image		     */
    long (*window)[4],		/* I: window specification of input images   */
    long *maxwin,		/* I: overall window containing parameter  
                                      windows                                */
    long *totwin,		/* I: total number of windows		     */
    long (*bands)[MAXBND+1],	/* I: bands to be processed for all images   */
    long nbands[],		/* I: number of bands of input image         */
    long totbnd,		/* I: total number of bands to be processed  */
    float *pixval,		/* I: Specified pixel value		     */
    long *updatflg, 		/* I: Flag for update mode                   */
    long msgtim,		/* I: elapsed time between display messages  */
    char *minmax		/* I: min max flag                           */
)

{

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern int fdesc[3];	        /* image file descriptors used for image I/O */
extern int gdesc;		/* group descriptor used for image I/O       */
extern long offset1[MAXBND+1];  /* offsets into "thebuf" for input bands     */
extern long offset2[MAXBND+1];  /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

static double  minval[MAXBND];	/* min pixel value array                     */
static double  maxval[MAXBND];	/* max pixel value array                     */
double  line_inc;		/* line increment 			     */
double  samp_inc;		/* sample increment			     */
float   tpixval;		/* pixel value for byte, short or long	     */

long	*lpixval;		/* pixval for long representation	     */
long    tempband[1][MAXBND+1];	/* Temporary band array			     */
long 	totempbnd;		/* total number of bands to be processed     */
long	access;			/* image I/O access type                     */
long	bandcnt;		/* loop index counting bands for new image   */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long    mmflag = FALSE;
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long	nonfatal = 1;		/* value of one                              */
long    one = 1;		/* one					     */
long	odtype=0;		/* data type of output image                 */
long	size;			/* number of bytes per pixel 	 	     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */
long    opt[3]; 
long    nl;
long    i,j;

short 	*spixval;		/* SHORT pixval value			     */
unsigned char  *bpixval;	/* BYTE pixval value			     */

char    *thebuf;		/* image I/O buffer                          */

/*  Initialize the pointer to the temporary mask value
------------------------------------------------------*/
bpixval = (unsigned char *)&tpixval;
spixval = (short *)&tpixval;
lpixval = (long *)&tpixval;

/* Open for update or open for read and write
   ------------------------------------------ */
opt[0] = 0;
opt[1] = NO_COMPRES;
opt[2] = 0;
if (*updatflg == TRUE)
   {
   access = IUPDATE;
   status = c_eopens((struct FDESC **)&fdesc[0],hosin,bands[0],&maxwin[SL],
                   &maxwin[SS] ,&maxwin[NL],&maxwin[NS],&offset1[0],&access,
		   &odtype,&zero,opt);
   totempbnd = totbnd;
   }
else
   {
   for (i=0; i<MAXBND+1; i++)
      tempband[0][i] = zero;
   for (i=0; i<*nbands; i++)
       tempband[0][i] = i + 1;
   totempbnd = i;
   /* Open image for read access
      -------------------------- */
   access = IREAD;
   opt[0]=opt[1]=opt[2]=0;
   status = c_eopens((struct FDESC **)&fdesc[0],hosin,tempband[0],&one,&one,
                   &zero,&ddr->ns,&offset1[0],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
       c_errmsg("Fatal error encountered","pixfill-fatal",&fatal);
             
   /* Open output image for write access
   ------------------------------------- */
   access = IWRITE;	
   opt[0] = 0;
   opt[1] = NO_COMPRES;
   opt[2] = 0;
   status = c_eopens((struct FDESC **)&fdesc[1],hosout,&totempbnd,&one,&one,
                   &zero,&ddr->ns ,offset2,&access,&odtype,&zero,opt);
   if (status != E_SUCC)
       {
       c_clean(vblock,"pixfill",(char (*)[256])hosout,&one);
       }
   fdesc[2] = 0;
   }

/* Allocate I/O buffer
----------------------*/
if ((thebuf = c_ealloc((struct FDESC **)fdesc,&bufsiz)) == NULL)
   {
   c_clean(vblock,"pixfill",(char (*)[256])hosout,&one);
   c_errmsg("Error allocating dynamic memory","pixfill-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixfill-fatal",&fatal);
   }

/* Setup for data type
----------------------*/
switch(odtype)
   {
   case EBYTE:  *bpixval= (unsigned char) *pixval;
		size = 1; 
		break;
   case EWORD:  *spixval = (short) *pixval;
		size = 2; 
		break;
   case ELONG:	*lpixval = (long) *pixval;
		size = 4; 
		break;
   case EREAL:  size = 4; 
		tpixval = *pixval;
		break;
   }

/* Group images
---------------*/ 
if (c_egroup((struct FDESC **)fdesc,(struct GDESC **)&gdesc,
              (unsigned char *)thebuf, &bufsiz) != E_SUCC)
   c_clean(vblock,"pixfill",(char (*)[256])hosout,&one);

/* Setup for processing message utility
---------------------------------------*/
if (*updatflg == TRUE)
   nl = maxwin[NL];
else
   nl = ddr->nl;
c_inittm("PIXFILL","Processed","line","bands"," "
	 ,&nl,&totempbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
   mmflag is set to be used within the processing loop -- quicker than strcmp
-----------------------------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   c_inmnmx(&odtype,&totempbnd,minval,maxval);
   mmflag = TRUE;
   }

/* Main processing loop, process all lines
------------------------------------------ */
for (curlin = 0;curlin < nl;curlin++)
   {
   if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
      c_clean(vblock,"pixfill",(char (*)[256])hosout,&one);
   /*  Process all specified bands
       --------------------------- */
   if (*updatflg == TRUE)
      {
      for (bandcnt=0; bandcnt<totempbnd; bandcnt++)
            {
            if (bands[0][bandcnt])
	       for (i=0;i<*totwin;i++)
                  {
	          if ((curlin >= (window[i][SL] - maxwin[SL])) && 
	                (curlin <= (window[i][SL] + window[i][NL] - 
                        maxwin[SL] - 1)))
 		     {
	    	     c_pxfill((unsigned char *)thebuf+(*(offset1+bandcnt)*size)+
                             ((window[i][SS] -maxwin[SS])* size),&odtype,
                              &window[i][NS],(unsigned char *)&tpixval);
                     }
                  }
            }
      }
   else 
      { 
      for (bandcnt=0;bandcnt<totempbnd;bandcnt++)
         {
         if ((curlin >= (maxwin[SL] - 1)) && 
	        (curlin <= (maxwin[SL] + maxwin[NL] - 2))) 
  	    {
	    for (j=0; j<totbnd; j++)
	      if (bands[0][j] == bandcnt + 1) 
	        for (i=0;i<*totwin;i++)
	          if ((curlin >= window[i][SL] - 1) && 
	   	         (curlin <= (window[i][SL] + window[i][NL] - 2)))
	       	     c_pxfill((unsigned char *)thebuf+(*(offset1+bandcnt)*size)+
		      	    ((window[i][SS] - 1) * size),&odtype,
                              &window[i][NS],(unsigned char *)&tpixval);
            }
         c_pxcopy((unsigned char *)thebuf + (*(offset1+bandcnt) * size),
                 (unsigned char *)thebuf + (*(offset2+bandcnt) * size),&odtype,
                  &ddr->ns);
         if (mmflag)
            status = c_pxmnmx((long *)thebuf + (*(offset2+bandcnt) * size),
	             &ddr->ns,&minval[bandcnt],&maxval[bandcnt],&odtype); 
	 }
      }
   curent++;
   }

/* Final write
--------------*/
if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
      c_clean(vblock,"pixfill",(char (*)[256])hosout,&one);

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
*nbands=0;
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
if (*updatflg != TRUE)
   {
   status = c_upddr(vblock,(char (*)[256])hosin,hosout,&one,window,tempband,nbands,
		 &line_inc,&samp_inc,&totempbnd,&upddr_flag);
   if (status != E_SUCC)
      {
      c_clean(vblock,"pixfill",(char (*)[256])hosout,&one);
      }
   }

/* Create associated min/max file
---------------------------------*/
if (*updatflg != TRUE)
    if (c_svmnmx(hosout,&totempbnd,minval,maxval,minmax) != E_SUCC)
       c_errmsg("Nonfatal error encountered","pixfill-warn",&one);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse((struct GDESC **)&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","pixfill-warn",&one);

/* Free buffer
--------------*/
free(thebuf);

return;
}
