 /*************************************************************************
NAME: CREATE_MASK

PURPOSE:  Create a mask of the region to be inserted into the output file.

Version  Date  Programmer      Code  Reason
-------  ----- ----------      ----  ------
 1.0     03/97 B.Olson          C    Added circular region to module

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

(Uses the midpoint circle algorithm from Computer Graphics, C version by
  Donald Hearn & M. Pauline Baker)

Initialize the mask to zero
If the region is not a circle
  For each line in the mask
    Set the number of pixels to the size of the mask
Otherwise if the region is a circle
  Determine the mask line containing 3/4 PI
  For each line to the line containng PI
    Calculate the starting pixel for this line
    If the line equal to the starting pixel is zero
      set the starting pixel of that line to line number of the current line
    Calculate the number of pixels within the circle for each line
    Reflect the starting pixel and number of pixels to the lower half
      of the circle
Return to the calling program

****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"

#define MAXREC 5000
#define COS45 0.707106781   /* cosine of 45 degrees           */

void create_mask
(
    long *radius,            /* I: Radius of circle                   */
    long blksiz[2],          /* I: Block size (NL, NS)                */
    long mask[MAXREC][2]     /* O: Block shape mask (SP, NP)          */
)

{
  long i;                /* loop counter                       */
  long j;                /* temporary number                   */
  long rad;              /* local copy of radius               */
  long tempnum;          /* temporary number                   */

  if (*radius == 0)
    {
    /* easy..this is a block and all lines are the same
       ------------------------------------------------ */
    for (i = 0; i < blksiz[0]; i++)
      {
      mask[i][0] = 0;
      mask[i][1] = blksiz[1];
      }
    }
  else
    {
    /* a little harder...this is a circle so must calculate the
       starting point for each line
       -------------------------------------------------------- */
    rad = *radius;

    /* find the line that starts at 135 degrees
       --------------------------------------- */
    j = (long)(rad - rad * COS45);

    for (i = j; i <= rad; i++)
      {
      /* find the starting pixel location in for the current line 
         -------------------------------------------------------- */
      tempnum = rad*rad - (rad - i) * (rad - i);

      /* save it in the mask
         ------------------- */
      mask[i][0] =(long)(rad - sqrt(tempnum));

      /* calculate the number of pixels within the circle
         ------------------------------------------------ */
      mask[i][1] = 2 * (rad - mask[i][0]) + 1;

      /* mirror these values down to the next eighth of the circle 
         --------------------------------------------------------- */
      mask[2*rad - i][0] = mask[i][0];
      mask[2*rad - i][1] = mask[i][1];

      /* check to see if the line of the starting sample value
         has been set
         ----------------------------------------------------- */
      if (mask[mask[i][0]][0] == 0)
        {
        /* it hasn't so mirror these values up to the previous eighth
           of the circle 
           ------------------------------------------- */
        mask[mask[i][0]][0] = i;
        mask[mask[i][0]][1] = 2 * (*radius - mask[mask[i][0]][0]) + 1;
        /* and mirror those down to the corresponding eighth on the lower
           portion of the circle
           -------------------------------------------------------------- */
        mask[2**radius - mask[i][0]][0] = i;
        mask[2**radius - mask[i][0]][1] = 2 * (*radius - mask[mask[i][0]][0])+1;
        }
      }
    }
}
