/*
***************************************************************************

NAME: INS_BLOCK

PURPOSE: Inserts specified pixel regions. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1          1/01/85    Y. Sheu
    2          7/28/89    K. A. Narayanan    conversion to 5.0
    3          7/29/91    K. Kimball         Removed unused local variables
					     endp, endl, and g.
    6.1        18/5/94    D. Eichacker       Added opt[] for eopen
    6.2        3/04/97    B. Olson           Converted to use a mask to
                                             allow shapes to be inserted

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
  Determine the size of a pixel
  Allocate memory based on the size
  Open output file for random access
  For each region in the array
    Adjust the region bounds based on the portion of the region in the image
    Read in the lines containing the region
    Change the gray values for the portion within the image
    Write the altered lines back to the output image 
  Free the allocated memory
  Return to the calling program

***************************************************************************
*/
#include "pixsert.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void ins_block
(
    char hosout[],       /* I: output image host name               */
    long wind[][4],      /* I: image window                         */
    long num_bands,      /* I: no. bands                            */
    long bands[],        /* I: bands to contain blocks              */
    long blksiz[2],      /* I: size of full size block              */
    long mask[][2],      /* I: shape of block to insert             */
    long *otype,         /* I: output image data type               */
    long array[][4],     /* I: block information                    */
    double gray[],       /* I: grey levels for blocks               */
    long *nblock         /* I: no. blocks                           */
)

{
float	r_fill_val;

static	long	first = TRUE;   /* flag for first time in routine       */

long	access,                 /* file access type                     */
        band,                   /* loop counter for band number         */
        line,                   /* loop counter for line number         */
        l_fill_val,             /* fill value storage for longs         */
	msg_flag = GEN2,        /* message type flag                    */
	n,                      /* loop counter                         */
        np,                     /* number of pixels                     */
        nl,                     /* number of lines                      */
        numpix,                 /* number of pixels                     */
        ounit,                  /* output image unit                    */
        size,                   /* pixel size                           */
        sp,                     /* starting pixel                       */
        sl,                     /* starting line                        */
        startpix;               /* starting pixel                       */
long    masklocation[3];        /* Info for current line in mask        */
                                /*  starting line in mask,              */
                                /*  starting pixel in mask,             */
                                /*  number of pixels                    */ 
long	stat;			/* Error status                         */
short	w_fill_val;             /* fill value storage for words         */
static	unsigned char b_fill_val, /* fill value storage for bytes       */
        *outbuf;                /* line buffer for image                */

long    fatal_err = -1;
long    inf_err = 1;
static	long	opt[3] = {0,0,0};
long    zero = 0;
long    one = 1;

if (first)
   {
   if (*otype == EBYTE) size = 1;
   else if (*otype == EWORD) size = sizeof (short);
   else if (*otype == ELONG) size = sizeof (long);
   else size = sizeof (float);

   if ((outbuf = (unsigned char *) malloc (wind[0][NS] * size)) == NULL)
      c_abort (&block, "Error allocating dynamic memory", "pixsert-alloc", 
               "pixsert", (char (*)[256])hosout, &one);
   first = FALSE;
   }
/*
open output image file for random access
----------------------------------------
*/
access = IUPDATE;
stat = c_eopenr ((struct FDESC **)&ounit, hosout, &num_bands, &one, &one,
                  &wind[0][NL], &wind[0][NS], &access, otype, opt, &one); 
if (stat != E_SUCC)
    c_errmsg("Fatal error encountered","pixsert-fatal",&fatal_err);

/*
initialize processing message
-----------------------------
*/
c_inittm ("PIXSERT", "Inserted", "block", "", "", nblock, &zero, &msgtim, 
          &msg_flag);
curent = 1;
c_startm ();

/*
fill image blocks with specified gray values
--------------------------------------------
*/
for (n = 0; n < *nblock; n++, curent++)
   {
   sp = array[n][SS] - 1;
   sl = array[n][SL];
   np = array[n][NS];
   nl = array[n][NL];

   /*
   if the block is partially off the top of the image, 
   need to relocate to the first line in the mask that is in the image
   -------------------------------------------------------------------
   */
   if ((blksiz[0] != nl) && (sl == 1))
     masklocation[0] = blksiz[0] - nl;
   else
     masklocation[0] = 0;

   /*
   if the block is partially off the left side of the image,
   need to relocate to the first pixel in the mask that in in the image
   --------------------------------------------------------------------
   */
   if (blksiz[1] != np)
     {
     if (sp == 0)
       {
       masklocation[1] = blksiz[1] - np;
       masklocation[2] = blksiz[1];
       }
     else
     /*
     if the block in partially off the right side of the image,
     need to readjust the number of pixels to use from the line in the mask
     ----------------------------------------------------------------------
     */
       {
       masklocation[1] = 0;
       masklocation[2] = np;
       }
     }
   else  
   /*
   the entire line from the mask in in the image so set the bounds to max
   ----------------------------------------------------------------------
   */
     {
     masklocation[1] = 0;
     masklocation[2] = blksiz[1];
     }

   for (line = sl; line <= sl + nl - 1 ; line++)
      for (band = 1; band <= num_bands; band++)
         if (bands[band-1])
            {
            /* 
            find the actual starting pixel for the current line
            ---------------------------------------------------
            */
            startpix = sp + mask[line-sl+masklocation[0]][0] - masklocation[1];

            /*
            calculate the actual number of pixels for the current line
            ----------------------------------------------------------
            */
            numpix = mask[line-sl+masklocation[0]][1];

            /*
            if the first pixel is left of the image reduce the number of
            pixels to be changed by the offset
            ------------------------------------------------------------
            */
            if (startpix < 0)
              {
              numpix += startpix;
              startpix = 0;
              }

            /*
            if the first pixel is right of the image then this line
            will not affect the output
            -------------------------------------------------------
            */
            if (startpix > wind[0][NS])
              numpix = 0;

            /*
            if the numpix is negative then none of the pixels in the
            output image for this line will be changed
            --------------------------------------------------------
            */
            if (numpix < 0)
              numpix = 0;

            /*
            this block could be partially off the right of the image
            so need to reduce the number of pixels to change
            --------------------------------------------------------
            */
           if (startpix + numpix > wind[0][NS])
              numpix = wind[0][NS] - startpix;
            c_eread ((struct FDESC **)&ounit, &band, &line, outbuf, &one); 
            switch (*otype)
               {
               case EBYTE:
                  b_fill_val = gray[n];
                  c_pxfill (&outbuf[startpix], otype, &numpix, &b_fill_val);
                  break;
               case EWORD:
                  w_fill_val = gray[n];
                  c_pxfill (&outbuf[startpix*sizeof (short)], otype, 
                            &numpix, (unsigned char *)&w_fill_val);
                  break;
               case ELONG:
                  l_fill_val =(long)gray[n];
                  c_pxfill (&outbuf[startpix*sizeof (long)], otype, 
                            &numpix,(unsigned char *)&l_fill_val);
                  break;
               case EREAL:
                  r_fill_val = gray[n];
                  c_pxfill (&outbuf[startpix*sizeof (float)], otype, 
                            &numpix,(unsigned char *)&r_fill_val);
                  break;
               }

            stat = c_ewrite ((struct FDESC **)&ounit, &band, &line, outbuf,
                              &one);
	    if (stat == E_FAIL)
 		c_errmsg("Fatal error encountered ","pixsert-fatal",&fatal_err);
            }
   }

/* 
stop processing message, free storage, and close image
------------------------------------------------------
*/
c_stoptm ();
free (outbuf);
stat = c_eclose ((struct FDESC **)&ounit);
if (stat != E_SUCC)
    c_errmsg("Nonfatal error encountered","pixsert-warn", &inf_err); 
}
