/*
**********************************************************************

NAME: PIXSERT

PURPOSE: Insert pixel regions of specified gray levels at specified 
         coordinates

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1          1/01/85    Y. Sheu
    2          9/04/87    B. Kovalick 
    3          7/28/89    K. A. Narayanan    conversion to 5.0
    4	       7/29/91    K. Kimball         Initialized fend before use and 
					     increased size of bands to MAXBND
    5          8/27/92    D. Etrheim         Check the status of c_getnam 
                                             in getpar
    6	       9/23/92	  T. Mittan	     Added extern to curent and curcat
					     to eliminate warnings on link
    6.1        3/04/97    B. Olson           Added ability to add regions
                                             that are circles
    7.0        11/5/99    D. Hames           Remove 50000 limits on number of
					     lines and samples. Add prototypes.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get and validate parameters 
   If subcommand is IMAGE
      Call COPY_IMAGE to copy input image to output
   Else
      Call GEN_IMAGE to generate constant valued image
   Call CREATE_MASK to create a mask of the shape of the region to insert
   Process MAXREC regions at a time until all regions are processed
      Read a maximum of MAXREC regions from coordinates file if specified
      or prompt user
      Call INS_BLOCK to insert the regions
   Issue successful completion message

**********************************************************************
*/

#include <stdio.h>
#include "las.h"
#include "protos.h"

#define	MAXREC	5000

struct PARBLK block;                   /* TAE parameter block     */

char	*iobuf;	 	    	       /* image i/o buffer	  */
long	gdesc;                         /* group descriptor        */
long	min_max;                       /* minmax option flag      */
long 	msgtim;			       /* message interval        */
 
char	buf[ERRLEN];

extern long	curcat;                /* parameters for processing */
extern long	curent;                /* message utility         */

int *fdesc; 
int *ipoint;
int *opoint;

void main
(void) 
{

float	aa[32]; 		/* transformation coefficients for x	*/
float	bb[32]; 		/* and y				*/
double	*gray;	 		/* grey levels for regions		*/
double   bkgrnd; 		/* image background value		*/

static long array[MAXREC][4];	/* array of regions			*/
long	bands[MAXBND]; 		/* bands to contain regions		*/
long	blksiz[2];		/* size of regions (NL, NS)		*/
long	fend = FALSE;		/* flag to continue processing		*/
long	grid; 			/* transformation flag			*/
static long image_band_num[MAXIMG][MAXBND + 1];  
				/* image band numbers			*/
long    fatal = -1;
long    nonfatal = 1;

long    mask [MAXREC][2];       /* shape of region to insert             */
long	nblock; 		/* no. regions 				*/
long	ndeg;			/* degree of transformation polynomial	*/
long *num_image_bands;          /* no. bands in each image		*/
long	num_inimages;		/* no. input images			*/
long	num_bands;              /* number of image bands      		*/
long	out_type;		/* output image data type		*/
long    radius;                 /* radius of circle                     */
long	tblock;			/* total number of regions		*/
static long wind[MAXIMG][4];    /* input image windows              	*/

char	coord_opt[9];		/* master/relative coordinates option	*/
char hos_in[MAXIMG][CMLEN];     /* input image host name           	*/
char *hos_out;                  /* output image host name          	*/
char *las_out;                  /* output image las name           	*/
char	subcmd[6]; 		/* input/constant image option		*/
char    coordfil[CMLEN];        /* coordinate file host name	        */

/* allocate dynamic memory */
if ((gray = (double *) malloc(MAXREC * sizeof(double))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","pixsert-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal);
   }
if ((num_image_bands = (long *) malloc(MAXREC * sizeof(long))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","pixsert-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal);
   }

if ((fdesc = (int *) malloc((MAXIMG + 1) * sizeof(int))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","pixsert-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal);
   }

if ((ipoint = (int *) malloc(MAXIMG * (MAXBND + 1) * sizeof(int))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","pixsert-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal);
   }

if ((opoint = (int *) malloc((MAXBND + 1) * sizeof(int))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","pixsert-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal);
   }

if ((hos_out = (char *) malloc(CMLEN * sizeof(char))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","pixsert-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal);
   }
if ((las_out = (char *) malloc(CMLEN * sizeof(char))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","pixsert-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal);
   }
/*
get user parameters 
-------------------
*/
getpar (hos_in, las_out, hos_out, &num_inimages, image_band_num, 
        num_image_bands, &num_bands, wind, coord_opt, subcmd, blksiz, 
        &radius, bands, &out_type, &bkgrnd, coordfil, &grid, &ndeg, 
        aa, bb);

/*
copy input image to output or generate a constant output image
--------------------------------------------------------------
*/
if (subcmd[0] == 'I')
   copy_image (hos_in, hos_out, num_inimages, num_image_bands, 
               image_band_num, &out_type, wind, &num_bands);
else
   gen_image (hos_in, hos_out, num_inimages, num_image_bands, 
                     image_band_num, &out_type, wind, &num_bands, &bkgrnd);

/*
create a mask for the shape to insert into the output image
-----------------------------------------------------------
*/
create_mask(&radius, blksiz, mask);

/*
process MAXREC regions at a time
-------------------------------
*/
tblock = 0;
while (!fend)      
   {
   nblock = 0;
/*
   obtain data regions either from file or from prompt
   --------------------------------------------------
*/
   if (strlen (coordfil) != 0)
      rdfile (coordfil, hos_in, wind, out_type, coord_opt, array, gray, tblock, 
              &nblock, &fend, grid, subcmd, blksiz, aa, bb, ndeg);
   else
      prompt (hos_in, wind, out_type, coord_opt, array, gray, tblock, 
              &nblock, &fend, grid, subcmd, blksiz, aa, bb, ndeg);

/*
   insert the regions
   -----------------
*/
   if (nblock  >  0) 
      {
      if (tblock != 0)
         t_write ("Processing next set of regions", T_STDCC);
      ins_block (hos_out, wind, num_bands, bands, blksiz, mask, &out_type, 
                 array, gray, &nblock);
      tblock += nblock;
      }
   }  
/*
update output image min/max and history and issue completion message
--------------------------------------------------------------------
*/
terminate (las_out, hos_out, hos_in, &num_inimages, &num_bands, wind);
}                                                                  
