/*****************************************************************************
*
*  NAME:  ALLOPT
*
*  FUNCTION: Create the Labeled Table
*
*	
*  PROGRAM HISTORY:
*  Version	Date	Programmer	Code	Request
*  -------	----	----------	----	-------
*   5.0	       11/89	R.McKinney 	DDPS	LAS 5.0 Conversion
*   5.1         7/90    T.Mittan        LAS     Convert to baseline
*                                               and improve performance.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
* 	Must be run under TAE
*
*  PROJECT:  LAS
*
*  ALOGRITHM DESCRIPTION:
*  Loop by class, and then by pixel value.  For each class find the mean,
*  standard deviation (using the sample formula), median, and mode.  Then 
*  write these with the minimum and maximum values to the LT file.  Loop
*  to next class.
*
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "ltable.h"
#include "pixstat.h"
#include "protos.h"

int allopt
(
    struct TAB_DEF *tab,		/* labeled table descriptor	*/
    long *bnd,			/* total number of bands	*/
    struct RECORD *perm,            /* permutation attributes       */
    long *class,                    /* number of classes            */
    long *range,                    /* range of values              */
    long *minv,                     /* minimum value offset         */
    long *pxvalue,                  /* pixel value                  */
    long *pxcnt,                    /* number of pixel values       */
    long *pxflg                    /* TRUE if pixvalue option used */
)

{

long *count,      		/* permutation counter		*/
     i,j,y,k,			/* loop counters		*/
     *mode,                     /* array for mode value         */
     temp,                      /* temporary value              */
     non_fatal = 1;             /* nonfatal flag                */

float *mean,	 		/* mean values per band         */
     *med;                       /* array for median value       */
float mid;                      /* middle value for median      */
float med_mad;

double stddev;			/* standard deviation holding   */
long *mad;

struct VECTOR *ptr;		/* pointer to current vector */
struct MATRIX size;		/* matrix size */


/* Initialize the needed variables
----------------------------------*/

size.nrow = 1;
size.ncol = 1;

if ((mean = (float *)calloc((*class),(sizeof(float)))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }
if ((mad = (long *)calloc((2 * (*range)),(sizeof(long)))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }
if ((med = (float *)calloc((*class),(sizeof(float)))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }
if ((mode = (long *)calloc((*class),(sizeof(long)))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }
if ((count = (long *)calloc((*class),(sizeof(long)))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }

/* find the needed values (mean, standard deviation, median, and mode.)
--------------------------------------------------------------------*/
if (*pxflg == 0)
  {
  for (j =0; j < *class; j++)
   {
   if (perm[j].count > 0)
    {
   /* Find mean and standard deviation using sample formula
   ------------------------------------------------------*/
   mean[j] = perm[j].sum / perm[j].count;
   stddev = (perm[j].std - (float) perm[j].count * (mean[j] * mean[j]));
   perm[j].std = sqrt(fabs(stddev / perm[j].count ));

   /* Find median and mode
   ---------------------*/
   mid = (perm[j].count / 2.0) ;
   for (i = 0, y = 0, count[j] = 0; i < *range; i++)
     {
     if (perm[j].list[i] > 0)
       {
       if ((float) y < mid)
         {
         med[j] = (float) i ;
         y += perm[j].list[i];    
         if (((float) y<=mid) && ((perm[j].count % 2) == 0)) 
            {
            for (k = i + 1; k < *range; k++)
               {
               if (perm[j].list[k] > 0 )
                 {
                 temp = k;
                 break;
                 }
               }
            }
         else
            temp = i ;
         }
       if (count[j] < perm[j].list[i])
         {
         count[j] = perm[j].list[i];
         mode[j] = i;
         }
       }
     }
     med[j] = ((med[j] + (float) temp)/2);


     /* Find the Median Absolute Deviation
     -----------------------------------*/
     for (i = 0; i < (2 * *range); i++)
         mad[i] = 0;
     for (i = 0; i < *range; i++)
       {
       if (perm[j].list[i] > 0)
          mad[abs((long)(2 * (med[j] - (float)i)))] += perm[j].list[i];
       }
     for (i = 0,y = 0; i < (2 *  *range); i++)
       {
       if (mad[i] > 0)
          {
          if ((float)y < mid)
            {
            med_mad =  (float) i ;
            y += mad[i];
            if (((float)y<=mid) && ((perm[j].count % 2) == 0))
               {
               for (k = i + 1; k < *range; k++)
                  {
                  if (mad[k] > 0 )
                    {
                    temp = k;
                    break;
                    }
                  }
               }
            else
               temp = i ;
            }
          }
       }

     med_mad = ((med_mad + (float)temp)/4);
     mode[j] += minv[1];
     med[j]  += (float) minv[1];
    /* Build the Labeled Table.
    -------------------------*/
		ptr = tab->vector;
                temp = j + minv[0];
    		put_matrix(ptr,I4,&size,&temp);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[j].count);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&mean[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,R8,&size,&perm[j].std);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[j].max1);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[j].min1);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&mode[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med_mad);
    		put_record(tab);
      }
   }
 }
else
  {
  for (j =0; j < *pxcnt; j++)
   {
   mean[pxvalue[j]] = perm[pxvalue[j]].sum / perm[pxvalue[j]].count;
   stddev = (perm[pxvalue[j]].std - (float) perm[pxvalue[j]].count * 
            (mean[pxvalue[j]] * mean[pxvalue[j]]));
   perm[pxvalue[j]].std = sqrt(fabs(stddev / perm[pxvalue[j]].count));
   mid = (perm[pxvalue[j]].count / 2.0);
   for (i = 0, y = 0, count[pxvalue[j]] = 0; i < *range; i++)
     {
     if (perm[pxvalue[j]].list[i] > 0)
       {
       if ((float) y < mid)
         {
         med[pxvalue[j]] = (float) i;
         y += perm[pxvalue[j]].list[i];    
         if (((float)y <= mid) && ((perm[pxvalue[j]].count % 2) == 0)) 
            {
            for (k = i + 1; k < *range; k++)
               {
               if (perm[pxvalue[j]].list[k] > 0 )
                 {
                 temp = k;
                 break;
                 }
               }
            }
         else
            temp = i ;
         }
       if (count[pxvalue[j]] < perm[pxvalue[j]].list[i])
         {
         count[pxvalue[j]] = perm[pxvalue[j]].list[i];
         mode[pxvalue[j]] = i ;
         }
       }
     }
     med[pxvalue[j]] = ((med[pxvalue[j]] + (float)temp)/2);


    /* Find Meadian Absolute Deviation
    --------------------------------*/
    for (i = 0; i <(2 *  *range); i++)
         mad[0] = 0;
     for (i = 0; i < *range; i++)
       {
       if (perm[pxvalue[j]].list[i] > 0)
          mad[ abs((long) (2 * (med[pxvalue[j]] - (float)i)))] += 
						perm[pxvalue[j]].list[i];
       }
     for (i = 0,y = 0; i < (2 * *range); i++)
       {
       if (mad[i] > 0)
          {
          if ((float)y < mid)
            {
            med_mad =  (float) i ;
            y += mad[i];
            if (((float) (y)<=mid) && ((perm[pxvalue[j]].count % 2) == 0))
               {
               for (k = i + 1; k < *range; k++)
                  {
                  if (mad[k] > 0 )
                    {
                    temp = k;
                    break;
                    }
                  }
               }
            else
               temp = i ;
            }
          }
       }

     med_mad = ((med_mad + (float) temp)/4);
     med[pxvalue[j]] += (float) minv[1];
     mode[pxvalue[j]] += minv[1];
     /* Build the Labeled Table.
     -------------------------*/
		ptr = tab->vector;
    		put_matrix(ptr,I4,&size,&pxvalue[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[pxvalue[j]].count);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&mean[pxvalue[j]]);
    		ptr = ptr->next;
    		put_matrix(ptr,R8,&size,&perm[pxvalue[j]].std);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[pxvalue[j]].max1);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[pxvalue[j]].min1);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&mode[pxvalue[j]]);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med[pxvalue[j]]);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med_mad);
    		put_record(tab);
      }
 }
free(mean);
free(med);
free(mad);
free(mode);
free(count);

return(E_SUCC);
}
