/*****************************************************************************
*
*   NAME:  GETPAR 
*
*   FUNCTION: Receive the parameters from the parameter block. 
*
*   PROGRAM HISTORY:
*
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     5.0      11/89	R.McKinney	DDPS	LAS 5.0 conversion
*     5.1       7/91    T.Mittan        LAS     convert to baseline code
*                                               and improve performance.
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRITPION:
*	Get the input image names and validate the windows and data types.
*   Get the Labeled Table name.  Get the mask values and verify that there
*   are the correct amount.  Retrieve the needed global values. Get the 
*   minimum and maximum pixel values.
*
******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "pixstat.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,  /* TAE parameter block		*/
    char hosin[][CMLEN],    /* Input host file names	*/
    char *hosout,           /* Output host file image name  */
    char *lthos,            /* Output LT host file name	*/
    long *nbands,           /* number of bands		*/
    long (*wind)[4],        /* window specification	for each image */
    long *maskval,          /* mask values			*/
    long (*bands)[MAXBND+1],/* band specifications for each input image */
    long *totbnds,          /* total number of bands	*/
    long *pixval,           /* pixel values to be selected  */
    char *statcalc,         /* statistical option		*/
    long *nmrimg,           /* number of images being processed */
    long *mskflg,           /* masking flag			*/
    long *pixflg,           /* pixel flag			*/
    long *imgflg,           /* flag for output image        */
    long *msgtim,           /* message timing 		*/
    long *pixcnt,           /* number of pixel values       */
    long *mskcnt,           /* number of mask values   	*/
    double *minval,         /* minimum value                */
    double *maxval,         /* maximum value                */
    long *ltflg             /* flag for output labeled table */
)

{
    struct VARIABLE *var;		/* pointer to TAE parameter */

    char taein[MAXIMG][CMLEN],	/* input TAE names		*/
         tmp_hosin[MAXIMG][CMLEN]; /* temporary storage for host file names */

    long dtype,	                /* data type of each image	*/
         count,                 /* parameter counter		*/
         count1,                /* parameter counter		*/
         element = 0,           /* element for getin		*/
         i,j,index,img_index,   /* loop counters		*/
         imglimit = MAXIMG,     /* maximum number of images to process */ 
         non_fatal = 1,         /* for fatal errors		*/
         mode = IWRITE,         /* File access mode		*/
         one = 1,               /* constant of 1		*/
         qflag[MAXIMG][2],      /* flag indicating if subwindows specificed */
         same,                  /* flag indicating if images have save window */
         tmp_bands[MAXIMG][MAXBND+1], /* temporary storage for band numbers */
         zero = 0;              /* zero constant		*/
         
    /* Get the input image names 
       ----------------------------*/
    if (c_getin(vblock,"IN",&element,taein,tmp_hosin,nmrimg,wind,tmp_bands,
                nbands,totbnds,&imglimit,&same,qflag) != E_SUCC)
        return (E_FAIL);

    if (!same)
    {
        c_errmsg("Input images do not have same window sizes",
                "pixstat-size",&non_fatal);
        return (E_FAIL);
    }

    if (*totbnds == 1)
    {
        c_errmsg("Input image must contain more than one band", "pixstat-band",
                &non_fatal);
        return (E_FAIL);
    }

    /* Separate the multiple band images into their own image files. This will
       make it easier to process requests for multiple band images.
       -----------------------------------------------------------------------*/
       index = 0;
       for (img_index =0; img_index < nmrimg[0]; img_index++)
       {
           for (i=0; i < nbands[img_index]; i++)
           {
               bands[index][0] = tmp_bands[img_index][i];

               /* Create an infile for each band specified */
               strcpy(hosin[index], tmp_hosin[img_index]);
               index++;
           }
       }
     
       /* There should be only 1 band per input image */
       for (index=0; index < totbnds[0]; index++)
           nbands[index] = 1;

       /* Number of images are the same as total bands; 1 band per image */
       nmrimg[0] = totbnds[0];

       /* Copy the input images will have the same window size, copy this
          window to all images */
       for (index = 1; index < nmrimg[0]; index++)
       {
           for (i = 0; i < 4; i++)
               wind[index][i] = wind[0][i];
       }

    /* Check the data type of each image
       ------------------------------------*/

    if (c_gettyp(hosin,&one,&dtype) != E_SUCC)
        return (E_FAIL);
    if (dtype == EREAL)
    {
        c_errmsg("Real images are not allowed","pixstat-real",&non_fatal);
        return (E_FAIL);
    }

    /* Get LT file name
       -------------------*/
    if (c_getcnt(vblock,"OUTLT",&count1) != E_SUCC)
        return(E_FAIL);
    if (count1 != 0)
    {
        *ltflg = TRUE;
        if (c_getnam(vblock,"OUTLT",&zero,lthos,"lt",&mode) != E_SUCC)
            return(E_FAIL); 
    }
    else
        *ltflg = FALSE;

    /* Get the output image name
       ----------------------------*/
    if (c_getcnt(vblock,"OUTFILE",&count) != E_SUCC)
        return(E_FAIL);
    if (count != 0)
    {
        *imgflg = TRUE;
        if (c_getnam(vblock,"OUTFILE",&zero,hosout,"txt",&mode) != E_SUCC)
            return(E_FAIL);
    }
    else
        *imgflg = FALSE;

    /* Check that at least one output was specified.
       ----------------------------------------------*/
    if (count1 == 0 && count == 0)
    {
        c_errmsg("No output specified","pixstat-out",&non_fatal);
        return(E_FAIL);
    }

    /* Retrieve pixel value
       -----------------------*/
    var = p_find(vblock,"CLASSVAL");
    i = (*var).v_count;
    for (j = 0; j < i; j++)
        pixval[j] = IVAL(*var,j); 
    *pixflg = TRUE;
    if (i == 0)
        *pixflg = FALSE;
    *pixcnt = i;

    /* Retrieve option
       ------------------*/
    var = p_find(vblock,"STATCALC");
    strcpy(statcalc, SVAL(*var,0));

    /* Get mask value
       -----------------*/
    var = p_find(vblock,"MASKVAL");
    i = (*var).v_count;
    for (j = 0; j < i; j++)
        maskval[j] = IVAL(*var,j);
    *mskflg = TRUE;
    if (i == 0)
        *mskflg = FALSE;
    *mskcnt = i;

    /* Retrieve globals
       -------------------*/
    var = p_find(vblock,"$MSGTIM");
    *msgtim = IVAL(*var,0); 


    /* Retrieve minimum and maximum values.
       ------------------------------------*/
    if (c_minmax(hosin,nmrimg,wind,bands,nbands,minval,maxval) != E_SUCC)
    {
        c_errmsg("Error finding min and max","pixstat-minmax",&non_fatal);
        return(E_FAIL);
    }
    return(E_SUCC);
}
