/*****************************************************************************
*
*   NAME:  PROCES 
*
*   FUNCTION: Process the images 
*
*   FILE REFERENCES:
*       
*   PROGRAM HISTORY:
*   Version     Date    Programmer      Code    Request
*   -------     ----    ----------      ----    -------
*     5.0      11/89    R.McKinney      DDPS    LAS 5.0 conversion
*     5.1       7/91    T.Mittan        LAS     Convert to baseline code
*                                               and improve performance.
*     5.2      11/92    T.Mittan        LAS     Corrected problem freeing
*                                               memory
*     5.3      12/92    D. Etrheim      EDC     Correct problem when using
*                                               windows
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*       Open the images and allocate space for Image I/0. For each sample
*   of each line, get a permutation number. Initialize the Labeled Table, and 
*   write the values to the LT. If an ouput listing of the data is requested
*   print it. Free the allocated space.
*   NOTE:  An offset value is added to the attribute values in the permutation
*   file.  The offset value is the minimum value in the image.  This is done
*   to better use memory. 
*   
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "util.h"
#include "pixstat.h"
#include "protos.h"


void freebufs              /* Free the dynamically allocated buffers */
(
    long  *thebuf,         /* I: Image I/O buffer                    */
    long  *pixflg,         /* I: masking flag                        */
    long  *pixcnt,         /* I: number of pixel values              */
    long  *class,          /* I: number of classes                   */
    struct RECORD *perm,   /* I: permutaion records                  */
    struct GDESC **gdesc   /* I: group descriptor used for image I/O */
)
{
    long j;

    c_egclse(gdesc);
    free(thebuf);
    if (*pixflg == 0)
        for (j = 0; j < *class; j++)
            free(perm[j].list);
    else
        for (j = 0; j < *pixcnt; j++)
            free(perm[j].list);
    free(perm);
}


int proces
(
    struct PARBLK *vblock,      /* TAE paramater block          */
    char hosin[][CMLEN],        /* input host file names        */
    char hosout[][CMLEN],       /* listing file name            */
    char *lthos,                /* output LT host file name     */
    long (*wind)[4],            /* window specifications for each input image */
    long (*bands)[MAXBND + 1],  /* bands to be processed for all images */
    long *nbands,               /* number of bands for input images*/
    long *totbnds,              /* total no. of bands to be processed */
    long *nmrimg,               /* number of input images       */
    long *maskval,              /* mask values for each band    */
    long *pixval,               /* pixel values for each band   */
    char *statcalc,             /* statistical option           */
    long *mskflg,               /* masking flag                 */
    long *pixflg,               /* masking flag                 */
    long *pixcnt,               /* number of pixel values       */
    double *minval,             /* minimum value                */
    double *maxval,             /* maximum value                */
    long *msgtim,               /* message timing value         */
    long *imgflg,               /* output list flag             */
    long *ltflg                 /* labled table flag            */
)
{
    /* The following variables have been declared global for image I/O routines
    --------------------------------------------------------------------------*/
    extern struct FDESC * fdesc[MAXIMG+1]; 
                                /* image file descriptors used for image I/O*/
    extern struct GDESC * gdesc;  /* group descriptor used for image I/O    */
    extern long offset1[MAXIMG*MAXBND]; /* offsets for input bands          */
    extern long curent;     /* current line being processed                 */
    extern long curcat;     /* current catagory being processed             */

    struct RECORD *perm;            /* permutaion records           */
    struct TAB_DEF tab;             /* Labeled Table descriptor     */

    FILE *fp;                       /* File descriptor              */

    long  band,                     /* band counter                 */ 
          bufsiz,                   /* size of the image I/O buffer */
          class,                    /* number of classes            */
          curatt[3],                /* attributes for a permutation */
          curlin,                   /* curent line                  */
          i,j,                      /* loop counters                */
          msgflg = ONECAT,          /* message processing flag: 
                                       processing one band at a time*/
          ndel,                     /* number of files to delete on failure */
          non_fatal = 1,            /* for nonfatal error messages  */
          offptr,                   /* offset pointer               */
          pixel_value,              /* value of current pixel       */
          nl,                       /* number of lines              */
          ns,                       /* number of samples            */
          sl,                       /* starting line                */
          ss,                       /* starting sample              */
          range,                    /* the data range */
          minv[2];                  /* minimum values               */
    long  *thebuf;                  /* Image I/O buffer             */
     
    char  tbuf[1][CMLEN];           /* tempary buffer for clean up  */

    int int_minval = (int)minval[0];/* Integerized min pixel value  */
    int int_maxval = (int)maxval[0];/* Integerized max pixel value  */

    minv[0] = (long )minval[0];
    class = (long) ((maxval[0] - minval[0]) + 1);

    /* Allocate space for permutation records
    -----------------------------------------*/
    perm = (struct RECORD *)calloc((class),(sizeof(struct RECORD)));
    if (perm == NULL)
    {
        c_errmsg("Error allocating dynamic memory","pixstat-alloc",&non_fatal);
        return(E_FAIL);
    }

    /* Open and write the label vectors to the LT file
    --------------------------------------------------*/
    initlt(&tab,lthos,statcalc);

    /* Start process timing message
    -------------------------------*/
    nl = wind[0][NL];
    curcat = 1;
    c_inittm("PIXSTAT","Processed","line","band"," ",&nl,totbnds,msgtim,
              &msgflg);
    c_startm();

    for (band=1; band < *totbnds; band++)
    {
        curent = 1;
        curcat = band;
        range = (long) ((maxval[band] - minval[band]) + 1);
        /* Open the input images
        ------------------------*/
        offptr = band;
        if (opnlas(hosin,wind,bands,nbands,nmrimg,&nl,&ns,&sl,&ss,&offptr) 
            != E_SUCC)
        {
            free(perm);
            return(E_FAIL);
        }

        /* Allocate the I/O buffer
        --------------------------*/
        if ((thebuf = (long *)c_ealloc(fdesc,&bufsiz)) == NULL)
        {
            free(perm);
            return(E_FAIL);
        } 
        /* Group the images 
        -------------------*/ 
        if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
        { 
            freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
            return(E_FAIL);
        }

        if (*pixflg == 0)
        {
           /* Allocate space for permutation records 
           -----------------------------------------*/ 
           for (i = 0; i < class; i++)
           {
               if ((perm[i].list = (long *) calloc(range,sizeof(long))) == NULL)
               {
                   c_lasmsg(FALSE,"pixstat-alloc","Error allocating dynamic "
                            "memory range %d  %d\n",range,i);
                   freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                   return(E_FAIL);
               }
               /*Fill buffers with a zero value
               ------------------------------*/
               perm[i].sum = 0;
               perm[i].std = 0;
               perm[i].min1 = (long) maxval[band];
               perm[i].max1 = (long) minval[band];
               perm[i].count = 0;
           }
        }
        else 
        {
            for (i = 0; i < *pixcnt; i++)
            {
                if ((perm[pixval[i]].list = (long *) calloc(range,sizeof(long))) 
                     == NULL)
                {
                    c_errmsg("Error allocating dynamic memory","pixstat-alloc",
                             &non_fatal);
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
                /*Fill buffers with a zero value
                ------------------------------*/
                perm[pixval[i]].sum = 0;
                perm[pixval[i]].std = 0;
                perm[pixval[i]].min1 = (long) maxval[band];
                perm[pixval[i]].max1 = (long) minval[band];
                perm[pixval[i]].count = 0;
            }
        }
        minv[1] = (long )minval[band];

        /* Process the images. 
        ---------------------------------------------------------------*/
        /* Process all of the values                        
        ---------------------------------------------------------------*/
        if ((*mskflg == 0) && (*pixflg == 0))
        {
            for (curlin = 0; curlin < nl; curlin++)
            {
                if (c_estep(&gdesc) != E_SUCC)
                {
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
                for (i = 0; i < ns; i++)
                {
                    /* Validate pixel value
                    -----------------------*/
                    pixel_value = thebuf[offset1[0] + i];
                    if (pixel_value < int_minval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixlo",
                                  "Pixel value: %d is less than"
                                  " reported minimum: %f\n",
                                  (int)pixel_value, minval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }
                    else if (pixel_value > int_maxval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixhi",
                                  "Pixel value: %d is greater than"
                                  " reported maximum: %f\n",
                                  (int)pixel_value, maxval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }

                    /* subtract class offset value */
                    curatt[0] = pixel_value - minv[0]; 

                    curatt[1] = thebuf[offset1[offptr] + i];
                    get_permutation_num(perm,curatt,minv); 
                 }
                 curent++;
            }
        }
        else if ((*mskflg != 0) && (*pixflg == 0))
        {
            for (curlin = 0; curlin < nl; curlin++)
            {
                if (c_estep(&gdesc) != E_SUCC)
                {
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
                for (i = 0; i < ns; i++)
                {
                    /* Validate pixel value
                    -----------------------*/
                    pixel_value = thebuf[offset1[0] + i];
                    if (pixel_value < int_minval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixlo",
                                  "Pixel value: %d is less than"
                                  " reported minimum: %f\n",
                                  (int)pixel_value, minval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }
                    else if (pixel_value > int_maxval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixhi",
                                  "Pixel value: %d is greater than"
                                  " reported maximum: %f\n",
                                  (int)pixel_value, maxval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }

                    /* subtract class offset value */
                    curatt[0] = pixel_value - minv[0]; 

                    curatt[1] = thebuf[offset1[offptr] + i];
                    if (curatt[1] != maskval[(band-1)])
                        get_permutation_num(perm,curatt,minv);
                }
                curent++;
            }
        }
        else if ((*mskflg == 0) && (*pixflg != 0))
        {
            for (curlin = 0; curlin < nl; curlin++)
            {
                if (c_estep(&gdesc) != E_SUCC)
                {
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
                for (i = 0; i < ns; i++)
                {
                    /* Validate pixel value
                    -----------------------*/
                    pixel_value = thebuf[offset1[0] + i];
                    if (pixel_value < int_minval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixlo",
                                  "Pixel value: %d is less than"
                                  " reported minimum: %f\n",
                                  (int)pixel_value, minval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }
                    else if (pixel_value > int_maxval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixhi",
                                  "Pixel value: %d is greater than"
                                  " reported maximum: %f\n",
                                  (int)pixel_value, maxval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }

                    curatt[0] = pixel_value;
                    curatt[1] = thebuf[offset1[offptr] + i];
                    for (j=0; j < *pixcnt; j++)
                    {
                        if (curatt[0] == pixval[j])
                            get_permutation_num(perm,curatt,minv);
                    }
                 }
                 curent++;
            }
        }
        else if ((*mskflg != 0) && (*pixflg != 0))
        {
            for (curlin = 0; curlin < nl; curlin ++)
            {
                if (c_estep(&gdesc) != E_SUCC)
                {
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
                for (i = 0; i < ns; i++)
                {
                    /* Validate pixel value
                    -----------------------*/
                    pixel_value = thebuf[offset1[0] + i];
                    if (pixel_value < int_minval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixlo",
                                  "Pixel value: %d is less than"
                                  " reported minimum: %f\n",
                                  (int)pixel_value, minval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }
                    else if (pixel_value > int_maxval)
                    {
                        c_lasmsg( FALSE, "pixstat-pixhi",
                                  "Pixel value: %d is greater than"
                                  " reported maximum: %f\n",
                                  (int)pixel_value, maxval[0]);
                        freebufs( thebuf, pixflg, pixcnt, &class,
                                  perm, &gdesc);
                        return(E_FAIL);
                    }

                    curatt[0] = pixel_value;
                    curatt[1] = thebuf[offset1[offptr] + i];
                    for (j=0; j < *pixcnt; j++)
                    {
                        if ((curatt[1] != maskval[(band-1)]) &&
                            (curatt[0] == pixval[j]))
                            get_permutation_num(perm,curatt,minv);
                    }
                }
                curent ++;
            }
        }
           
        if (*ltflg)
        {
            /* Create the LT file
            -------------------*/
            if (strcmp(statcalc,"MEAN") == 0)
            {
                if (menopt(&tab,&band,perm,&class,pixval,pixcnt,pixflg,minv) 
                     != E_SUCC)
                {
                    close_tab(&tab);
                    strcpy(tbuf[0],lthos);
                    ndel = 2;
                    c_clean(vblock,"pixstat",tbuf,&ndel);
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
            }
            else if (strcmp(statcalc,"MAX") == 0)
            {
                if (maxopt(&tab,&band,perm,&class,pixval,pixcnt,pixflg,minv) 
                     != E_SUCC)
                {
                    close_tab(&tab);
                    strcpy(tbuf[0],lthos);
                    ndel = 2;
                    c_clean(vblock,"pixstat",tbuf,&ndel);
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
            }
            else if (strcmp(statcalc,"MED") == 0)
            {
                if (medopt(&tab,&band,perm,&class,minv,pixval,
                           pixcnt,pixflg,&range) != E_SUCC)
                {
                    close_tab(&tab);
                    strcpy(tbuf[0],lthos);
                    ndel = 2;
                    c_clean(vblock,"pixstat",tbuf,&ndel);
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
            }
            else if (strcmp(statcalc,"MODE") == 0)
            {
                if (modopt(&tab,&band,perm,&class,&range,minv,pixval,
                        pixcnt,pixflg) != E_SUCC)
                {
                    close_tab(&tab);
                    strcpy(tbuf[0],lthos);
                    ndel = 2;
                    c_clean(vblock,"pixstat",tbuf,&ndel);
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
            }
            else if (strcmp(statcalc,"ALL") == 0)
            {
                if (allopt(&tab,&band,perm,&class,&range,minv,pixval,pixcnt,
                    pixflg) != E_SUCC) 
                {
                    close_tab(&tab);
                    strcpy(tbuf[0],lthos);
                    ndel = 2;
                    c_clean(vblock,"pixstat",tbuf,&ndel);
                    freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                    return(E_FAIL);
                }
            }
        }
        /* open output listing 
        -----------------------*/
        if (*imgflg)
        {
            if ((fp = fopen(hosout[0],"a")) == NULL)
            {
                c_errmsg("Error opening listing file","pixstat-open", 
                         &non_fatal);
                fclose(fp);
                ndel = 1;
                c_clean(vblock,"pixstat",hosout,&ndel);
                freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
                return (E_FAIL);
            }
            if (*pixflg == 0)
            {
                fprintf(fp,"      class   pixels/class    pixel value "
                        "count  pixel value");
                for (j = 0; j < class; j++)
                { 
                    for (i = 0; i < range; i++)
                    {
                        if (perm[j].list[i] > 0)
                            fprintf(fp,"\n%11d    %11d          %11d  %11d",
                                    j + minv[0], perm[j].count, perm[j].list[i],
                                    (i + minv[1]));  
                    } 
                }
                fprintf(fp,"\n");
                fclose(fp);
            }
            else
            {
                fprintf(fp,"      class   pixels/class    pixel value "
                        "count  pixel value");
                for (j = 0; j < *pixcnt; j++)
                { 
                    for (i = 0; i < range; i++)
                    {
                        if (perm[pixval[j]].list[i] > 0)
                        {
                            fprintf(fp,"\n%11d    %11d          %11d  %11d",
                                    pixval[j], perm[pixval[j]].count, 
                                    perm[pixval[j]].list[i],(i+minv[1]));
                        }
                    } 
                }
                fprintf(fp,"\n");
                fclose(fp);
            }
        }
        
        /* Close all image files
        -----------------------*/
        if (c_egclse(&gdesc) != E_SUCC)
        {
            c_errmsg("Nonfatal error encountered","pixstat-close",&non_fatal);
            freebufs( thebuf, pixflg, pixcnt, &class, perm, &gdesc);
            return (E_FAIL);
        }

        /* Free the buffers
        -----------------*/
        free(thebuf);

        if (*pixflg == 0)
        {
            for (j = 0; j < class; j++)
                free(perm[j].list);
        }
        else
        {
            for (j = 0; j < *pixcnt; j++)
                free(perm[j].list);
        }

        curcat = 1;
    }
    c_stoptm();

    free(perm);

    if (*ltflg)
      close_tab(&tab);
    return(E_SUCC);
}
