/******************************************************************************
FUNCTION:	DOSIAT

PURPOSE:	Reads the SIAT file from the tape and then writes it to a
		disk if it was requested by the user.  The data in these
		files are a combination of binary and EBCDIC data types.
		The EBCDIC data is converted into ASCII data.

ALGORITHM 
  Make an associated SIAT file name
  Open the associated SIAT file

  Read record 1 of SIAT file
  Write the record to the associated SIAT file

  Read record 2 of SIAT file
  Write the record to the associated SIAT file
		.
		.
		.
  Read record 7 of SIAT file
  Write the record to the associated SIAT file

  If the CCTX tape is not of the old format
	Read record 8 of SIAT file
	Write the record to the associated SIAT file

  Close the associated SIAT file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif

#include <fcntl.h>
#include "las.h"	/* LAS include				*/
#include "tapeio.h"	/* Tape I/O include			*/
#include "siat.h"   /* Siat include file 			*/
#include "util.h"
#include "header.h"
#include "labserv.h"
#include "protos.h"

void dosiat
(
    char *hname,		/* Host file name			*/
    long *old,		/* Old version of CCTX tapes		*/
    char *direction        /* Direction of the satellite "D" or "A" */
)

{
extern long uni;	   /* Tape drive unit number		*/
extern long outcnt;	   /* Number of output files to clean up*/
extern char out[][CMLEN]; /* Output files to clean up		*/

int  fd;		/* File descriptor			*/

long size;		/* Buffer size				*/
long sflag;		/* Siat flag				*/
long status;		/* Status return code			*/
long oflags;		/* Open flags				*/
long mode = HANDLE;	/* Tape I/O handles errors		*/
long nbyte;
long nonfatal = 1;	/* Non-fatal error code			*/
long pmode = 0755;	/* Protection mode on open		*/

char hsfil[CMLEN];	/* SIAT host file name			*/
char buffr[2050];	/* input buffer				*/

struct SIAT siat;		/* Siat structure			*/

/* Make an associated SIAT file name
  ---------------------------------*/
c_lsmknm(hname,"cctx_siat",hsfil);

/* Open the associated SIAT file
  ---------------------------------*/
oflags = O_WRONLY | O_TRUNC | O_APPEND | O_CREAT;
if ((fd = open(hsfil,oflags,pmode)) == NULL)
    {
    c_errmsg("Error opening siat file","plenter-open",&nonfatal);
    clean_up();
    }

/* Add the host SIAT file name to the output list
  ---------------------------------------------*/
strcpy(out[outcnt],hsfil);
outcnt++;

/* Read record 1 of SIAT file
  ---------------------------*/
size = 2048;
status = c_tpread(&uni,buffr,&size,&mode);
if (status < TAPE_SUCC)
    c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

/* Convert the EBCDIC stuff to ASCII
  ---------------------------------*/
if (*old)
    {
    nbyte = 18;
    c_eb2asc(&buffr[0],&nbyte);

    nbyte = 24;
    c_eb2asc(&buffr[28],&nbyte);

    nbyte = 1536;
    c_eb2asc(&buffr[68],&nbyte);
    }
else
    {
    nbyte = 24;
    c_eb2asc(&buffr[0],&nbyte);

    nbyte = 30;
    c_eb2asc(&buffr[34],&nbyte);

    nbyte = 1536;
   c_eb2asc(&buffr[78],&nbyte);
    }

/* Write the record to the associated SIAT file
  ---------------------------------------------*/
write(fd,buffr,size);

/* Read record 2 of SIAT file
  ---------------------------*/
size = 270;
status = c_tpread(&uni,buffr,&size,&mode);
if (status < TAPE_SUCC)
    c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

/* Convert the EBCDIC stuff to ASCII
  ---------------------------------*/
if (*old)
    {
    nbyte = 30;
    c_eb2asc(&buffr[12],&nbyte);

    nbyte = 136;
    c_eb2asc(&buffr[50],&nbyte);
    }


/* Write the record to the associated SIAT file
  ---------------------------------------------*/
write(fd,buffr,size);

/* Read record 3 of SIAT file
  ---------------------------*/
size = 225;
status = c_tpread(&uni,buffr,&size,&mode);

if (status < TAPE_SUCC)
    c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

/* assign siag flag, if record three has 216 bytes, siat version is 2
  ------------------------------------------------------------------*/
if (status == 216)
   {
   sflag = 2;
   size = 216;
   }
else if (*old)
   sflag = 1;
else
   sflag = 3;

/* Convert the EBCDIC stuff to ASCII
  ---------------------------------*/
if (sflag == 1)
    {
    nbyte = 96;
    c_eb2asc(&buffr[0],&nbyte);

    nbyte = 12;
    c_eb2asc(&buffr[100],&nbyte);

    nbyte = 8;
    c_eb2asc(&buffr[196],&nbyte);
    }
else if (sflag == 2)
    {
    nbyte = 30;
    c_eb2asc(&buffr[12],&nbyte);

    nbyte = 136;
    c_eb2asc(&buffr[50],&nbyte);
    }
else
    {
    nbyte = 12;
    c_eb2asc(&buffr[12],&nbyte);
    }

write(fd,buffr,size);

/* Read record 4 of SIAT file
  -------------------------*/
if (sflag == 2)
   size = 204;
else
   size = 165;
status = c_tpread(&uni,buffr,&size,&mode);
if (status < TAPE_SUCC)
    c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

if (sflag == 1)
    {
    nbyte = 144;
    c_eb2asc(&buffr[0],&nbyte);
    }
else if (sflag == 2)
    {
    nbyte = 96;
    c_eb2asc(&buffr[0],&nbyte);

    nbyte = 12;
    c_eb2asc(&buffr[100],&nbyte);

    nbyte = 8;
    c_eb2asc(&buffr[196],&nbyte);
    }
else
    {
    nbyte = 96;
    c_eb2asc(&buffr[0],&nbyte);

    nbyte = 12;
    c_eb2asc(&buffr[100],&nbyte);
    }

write(fd,buffr,size);

/* Read record 5 of SIAT file
  --------------------------*/
size = 145;
status = c_tpread(&uni,buffr,&size,&mode);
if (status < TAPE_SUCC)
    c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

if (sflag == 1)
    {
    nbyte = 18;
    c_eb2asc(&buffr[18],&nbyte);
    }
else
    {
    nbyte = 144;
    c_eb2asc(&buffr[0],&nbyte);
    }

write(fd,buffr,size);

/* Read record 6 of SIAT file
  --------------------------*/
size = 325;
status = c_tpread(&uni,buffr,&size,&mode);
if (status < TAPE_SUCC)
    c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

if (sflag != 1)
     {
     nbyte = 18;
     c_eb2asc(&buffr[18],&nbyte);
     }

write(fd,buffr,size);

/* Read record 7 of SIAT file
  --------------------------*/
size = 480;
status = c_tpread(&uni,buffr,&size,&mode);
if (status < TAPE_SUCC)
    c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

write(fd,buffr,size);

/* Read record 8 of SIAT file.  The older tape format does not contain
  an eighth record.
  ------------------------------------------------------------------*/
if (sflag != 1)
    {
    size = 480;
    status = c_tpread(&uni,buffr,&size,&mode);
    if (status < E_SUCC)
	c_errmsg("Error reading SIAT file","plenter-siat",&nonfatal);

    write(fd,buffr,size);
    }

/* Close the associated SIAT file
  -------------------------------*/
close(fd);


/* Get the direction.  siat.latitude[5] is the lat. at scene center.
 ------------------------------------------------------------------*/
c_read_siat(&siat,hsfil);
if (siat.mss_latitude[5] > siat.mss_latitude[6])
   strcpy(direction,"D");
else 
   strcpy(direction,"A");

return;
}
