/******************************************************************************
FUNCTION:	GETPAR


PURPOSE:	Retrieves the required parameters from the TAE parameter block

ALGORITHM 
  Initialize the array of bands wanted to process
  Get the parameter for the bands wanted
  From the number of bands set the band flags
  Set the band array
  Get the output image name
  Get the comments for each tape mount
  Retrieve the number of files to skip before processing
  Retrieve the flag indicating wether the ID ANNOTATION and SIAT files are
  to be saved in files associated with the image
  Retrieve the flag indicating that skew is to be corrected
  Retrieve the flag indicating if synthetic pixels are to be removed
  Retrieve the line increment.  Each Nth line will be written out
  Retrieve the sample increment.  Each Sth sample will be written out
  Retrieve the value that is to be added to each sample of the output
  Retrieve the scale factor for the fourth band
  Retrieve the min/max flag
  Retrieve the time between processing messages

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>              /* Standard I/O include                 */
#include "las.h"                /* LAS include file                     */
#include "util.h"
#include "protos.h"

void getpar
(
    char tname[][CMLEN],		/* Image file name				*/
    char hname[][CMLEN],		/* Host image file name				*/
    long inwin[],		/* Input window to process			*/
    long *band,		/* Total number of bands needed			*/
    long bands[],		/* Which bands are needed			*/
    long *b4fac,		/* Scaling factor for band 4			*/
    long *linc,		/* Line increment				*/
    long *sinc,		/* Sample increment				*/
    long *upscal,		/* Scaling factor added to each pixel		*/
    long *tapes,		/* Number of tapes needed			*/
    long *skip,		/* Number of files to skip			*/
    char comment[][CMLEN],	/* Tape numbers to be used			*/
    char *wrtdsk,		/* Write id, annotation, SIAT files to disk	*/
    char *earth,		/* Correct for earth rotation flag		*/
    char *synth,		/* Delete synthetic pixels flag			*/
    long bla[],		/* Bands wanted					*/
    char *minmax,		/* Min/Max flag					*/
    long *msgtim,		/* elapsed time between display messages	*/
    long *windflg		/* Window flag					*/
)
{
extern struct PARBLK vblock;	/* Variable block			*/

struct VARIABLE *val;		/* pointer to TAE variables		*/
long	status; 		/* Return status variable		*/
long	i;			/* Looping variable			*/
long	fatal = -1;		/* Fatal error code			*/

/*
  Initialize the input window and array of bands wanted to process
*/
for (i = 0; i < 4; i++)
    {
    inwin[i] = 0;
    bla[i] = FALSE;
    }
/*
  Get the input window to be processed
*/
val = p_find(&vblock,"INWIND");
for (i = 0; i < (*val).v_count; i++)
    {
    inwin[i] = IVAL(*val,i);
    *windflg = TRUE;
    }

/*
  Get the parameter for the bands wanted
*/
val = p_find(&vblock,"BANDS");
*band = (*val).v_count;

/*
  From the number of bands set the band flags
*/
if (*band > 0)
    {
    for (i = 0; i < *band; i++)
	{
	bla[(IVAL(*val,i)) - 1] = TRUE;
	bands[i] = i + 1;
	}
    }
else
    {
    *band = 4;
    for (i = 0; i < *band; i++)
	{
	bla[i] = TRUE;
	bands[i] = i + 1;
	}
    }

bands[*band] = 0;

/*
  Get the output image name
*/
status = c_getout(&vblock,"OUT",tname,hname);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","plenter-fatal",&fatal);

/*
  Get the comments for each tape mount
*/
val = p_find(&vblock,"COMMENT");
*tapes = val->v_count;
for (i = 0; i < *tapes; i++)
    strcpy(comment[i],SVAL(*val,i));

/*
  Retrieve the number of files to skip before processing
*/
val = p_find(&vblock,"SKIPNUM");
*skip = IVAL(*val,0);

/*
  Retrieve the flag indicating wether the ID ANNOTATION and SIAT files are
  to be saved in files associated with the image
*/
val = p_find(&vblock,"WRTDSK");
strcpy(wrtdsk,SVAL(*val,0));

/*
  Retrieve the flag indicating that skew is to be corrected
*/
val = p_find(&vblock,"SKEWFLG");
strcpy(earth,SVAL(*val,0));

/*
  Retrieve the flag indicating if synthetic pixels are to be removed
*/
val = p_find(&vblock,"SYNTHFLG");
strcpy(synth,SVAL(*val,0));

/*
  Retrieve the line increment.  Each Nth line will be written out
*/
val = p_find(&vblock,"LINEINC");
*linc = IVAL(*val,0);

/*
  Retrieve the sample increment.  Each Sth sample will be written out
*/
val = p_find(&vblock,"SAMPINC");
*sinc = IVAL(*val,0);

/*
  Retrieve the value that is to be added to each sample of the output
*/
val = p_find(&vblock,"OFFSET");
*upscal = IVAL(*val,0);

/*
  Retrieve the scale factor for the fourth band
*/
val = p_find(&vblock,"SCALFACT");
*b4fac = IVAL(*val,0);

/*
  Retrieve the min/max flag
*/
val = p_find(&vblock,"$MINMAX");
strcpy(minmax,SVAL(*val,0));

/*
  Retrieve the time between processing messages
*/
val = p_find(&vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);

}
