/******************************************************************************
FUNCTION:	HEADER

PURPOSE:	Writes out the ID and ANNOTATION files

ALGORITHM 
  Set the flags for the open of associated files
  Read the ID information from the tape
  Convert the tape sequence number and make sure that the first tape is
  being processed first
  If not first tape
	display an error message.
  Determine if the tape is of the old format
  If not old format
	set offsets into the buffer for conversion purposes

  If output is to go to the terminal
	write the ID information to the terminal

  If the ID or ANNOTAION files need to be printed on a line printer or a filei
	information is written to a file and the PDF sends the file to the
	file utility to print, save and/or delete the file

  If the user wishes to keep the ID information from the tape
        create an associated file for the ID information
	open/create an associated file 
	write the information to the file
	close the associated file

  Read the annotation record from the tape
  Convert the ANNOTATION information to ASCII
  If the output to the terminal was requested 
	print the ANNOTATION information to the screen

  If the output to a file or the line printer was requested
	print the ANNOTATION information to a file

  If the user requested the information to be saved
	 write the ANNOTATION information to an associated file

  If the user wishes to keep the ANNOTATION information from the tape
        create an associated file for the ANNOTATION information
	open/create an associated ANNOTATION file 
	write the information to the file
	close the associated file

  Set the image window size and calculate the maximum earth rotaional
  skew for the image on the initial pass.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include "las.h"		/* LAS include file			*/
#include "tapeio.h"		/* TAPE I/O include file		*/
#include "geompak.h"    /* Geometry include file                */
#include "util.h"
#include "labserv.h"
#include "protos.h"

/*
  Macro for getting a substring
  ----------------------------*/
#define substr(in,out,num) \
strncpy((out),(in),(num)); \
out[(num)] = '\0';	   \

void header
(
    char *hname,		/* TAE image file name				*/
    long *strip,		/* current strip number				*/
    long inwin[4],		/* sl,sp,el,ep					*/
    char *wrtdsk,		/* write ID,ANNOTATION,SIAT files to disk	*/
    long *term,		/* flag for terminal output			*/
    long *file,		/* flag for file output				*/
    long *lp,		/* flag for line printer output			*/
    long *initl,		/* initial pass					*/
    long *adjlin,		/* adjusted line length				*/
    float *skwcon,		/* earth skew variable				*/
    long *maxskw,		/* maximum skew					*/
    long *np,		/* number of samples in output image		*/
    long *nl,		/* number of lines in output image		*/
    long *old,		/* old version of CCTX tapes			*/
    char *earth,		/* correct for earth rotation flag		*/
    long *sinc,		/* sample increment				*/
    long *linc,		/* line increment				*/
    FILE *fp,		/* File pointer to print file			*/
    char *scene,		/* scene/frame id				*/
    char *day,		/* Date of exposure  (any day is fine)		*/
    char *month,		/* Month of exposure (Jan is to cold)		*/
    char *year,		/* Year of exposure  (I wouldn't tell you)	*/
    char *center		/* Center  Lat/Long				*/
)
{
extern long uni;	   /* tape drive unit number			*/
extern char out[][CMLEN]; /* Output files to clean up			*/
extern long outcnt;	   /* Number of output files to clean up	*/

double lat;		/* latitude					*/
double degree;		/* degree					*/
double minute;		/* minute					*/

long fd;		/* File descriptor				*/
long adj[2];		/* adjusted line length bytes			*/
long vlwin[4];		/* Valid window verification			*/
long size;		/* buffer size					*/
long i;			/* loop counter-current band number		*/
long ntape;		/* tape number					*/
long status;		/* function return status code			*/
long oflags;		/* Open flags					*/
long nbyte;		/* Number of bytes to convert			*/
long mode = HANDLE;	/* Tape I/O routines handle errors		*/
long flag = 0;		/* Paging flag for PTERM			*/
long one  = 1;		/* One						*/
long pmode = 0755;	/* protection mode				*/
long nonfatal = 1;	/* Non-fatal error code				*/
long j[4];		/* points to beginning of annotation fields	*/
char seq[5];		/* tape sequence number				*/
char hafil[CMLEN];	/* annotation host file name			*/
char hhfil[CMLEN];	/* id host file name				*/
char annot[145];	/* annotation file				*/
char pbuf[145];		/* print buffer					*/
char sdeg[3];		/* String for degree				*/
char smin[3];		/* String for minute				*/
char numtape[2];	/* Number of tapes				*/
char nadir[15];		/* Nadir Lat/Long				*/
char selev[3];		/* Sun elevation				*/
char sazm[4];		/* Sun azmuth					*/
char frame[9];		/* Frame Path/Row				*/

unsigned char idbuff[41];  /* input buffer				*/
unsigned char anbuff[625]; /* input buffer for annotation file		*/

/*
  Set the flags for the open of associated files
  ---------------------------------------------*/
oflags = O_WRONLY | O_TRUNC | O_APPEND | O_CREAT;

/*
  Read the ID information from the tape
  -------------------------------------*/
size = 40;
status = c_tpread(&uni,idbuff,&size,&mode);
if (status < TAPE_SUCC)
    clean_up();

/*
  Convert the tape sequence number and make sure that the
  first tape is being processed first.  If not display an
  error message.
  ---------------------------------------------*/
for (i = 0; i < 4; i++)
    seq[i] = idbuff[12 + i];
seq[4] = '\0';

for (i = 0; i < 12; i++)
    scene[i] = idbuff[i];
scene[12]='\0';

adj[0] = idbuff[38];
adj[1] = idbuff[39];
*adjlin = (adj[0] * 256) + adj[1];

nbyte = 4;
c_eb2asc(seq,&nbyte);

nbyte = 12;
c_eb2asc(scene,&nbyte);

substr(&seq[1],numtape,1);
sscanf(numtape,"%d",&ntape);

if ((*initl) && (ntape != 1))
    {
    c_errmsg("Mount tape one first ","plenter-tape",&nonfatal);
    clean_up();
    }

/*
  Determine if the tape is of the old format
  If it is not of the old format set offsets into the buffer for conversion
  purposes.
  ---------------------------------------------*/
*old = FALSE;
if (scene[5] != '-')
    {
    *old = TRUE;
    for (i = 0; i < 4; i++)
	j[i] = 0;
    }
else
    {
    j[0] = 9;
    j[1] = 7;
    j[2] = 6;
    j[3] = -44;
    }

/*
  If output is to go to the terminal write the header information to the
  terminal
  ---------------------------------------------*/
if (*term)
    {
    strcpy(pbuf," STRIP #    SCENE ID    TAPE SEQ #  LINE WIDTH");
    c_pterm(pbuf,&flag);

    sprintf(pbuf,"    %d     %s    %s         %d",*strip,scene,seq,*adjlin);
    c_pterm(pbuf,&flag);
    }

/*
  If the ID or ANNOTAION files need to be printed on a line printer or a file
  then the information is written to a file and the PDF sends the file to the
  file utility to print, save and/or delete the file
  ---------------------------------------------*/

if (*lp || *file)
    {
    fprintf(fp,"\n");
    fprintf(fp," STRIP #    SCENE ID    TAPE SEQ #  LINE WIDTH\n");
    fprintf(fp,"    %d      %s    %s         %d\n",*strip,scene,seq,*adjlin);
    }

/*
  If the user wishes to keep the ID information from the tape create an
  associated file and write the information to the file
  ---------------------------------------------*/
if ((*initl) && ((strcmp(wrtdsk,"ID") == 0) || (strcmp(wrtdsk,"IAS") == 0)))
    {
    /*
      Create an associated file for the ID information
      ---------------------------------------------*/
    c_lsmknm(hname,"cctx_id",hhfil);

    /*
      Open/Create the associated file
      ------------------------------*/
    if ((fd = open(hhfil,oflags,pmode)) == NULL)
	{
	c_errmsg("Error opening ID file","plenter-open",&nonfatal);
	clean_up();
	}

    /*
      Write the ID buffer to the associated file
      -----------------------------------------*/
    write(fd,idbuff,size);

    /*
      Close the associated file
      ------------------------*/
    close(fd);

    /*
      Update the output file counter
      -----------------------------*/
    strcpy(out[outcnt],hhfil);
    outcnt++;
  }

/*
  Read the annotation record from the tape
  ---------------------------------------*/
size = 624;
status = c_tpread(&uni,anbuff,&size,&mode);
if (status < TAPE_SUCC)
    clean_up();

/*
  Convert the ANNOTATION information to ASCII
  ------------------------------------------*/
for (i = 0; i < 144; i++)
    annot[i] = anbuff[i];

nbyte = 144;
c_eb2asc(annot,&nbyte);

/* Set values to adjust to tape format
   ----------------------------------*/
if (annot[29] == '-')
    {
    j[0] = 9;
    j[1] = 7;
    j[2] = 6;
    j[3] = -44;
    }

substr(&annot[0],day,2);
substr(&annot[2],month,3);
substr(&annot[5],year,2);
substr(&annot[10],center,14);
/* Not all annotation buffers have the direction in this location.
   We will calculate the direction from the siat file. */ 
/*substr(&annot[25],direction,1);*/
substr(&annot[27+j[0]],nadir,14);
substr(&annot[60+j[1]],selev,2);
substr(&annot[65+j[2]],sazm,3);
substr(&annot[69+j[3]],frame,8);

/*
  If requested print the ANNOTATION information to the screen
  ---------------------------------------------*/
if (*term)
    {
    c_pterm("\n",&flag);
    sprintf(pbuf," DATE OF EXPOSURE:  %s %s %s",day,month,year);
    c_pterm(pbuf,&flag);

    c_pterm("\n",&flag);
    sprintf(pbuf,"     CENTER          NADIR          SUN        FRAME");
    c_pterm(pbuf,&flag);

    sprintf(pbuf,"   LAT    LONG    LAT    LONG    ELEV AZMUTH  PATH/ROW");
    c_pterm(pbuf,&flag);

    sprintf(pbuf," %s  %s   %s   %s    %s",center,nadir,selev,sazm,frame);
    c_pterm(pbuf,&one);
    }

/*
  If the output to a file or the line printer was requested print the
  ANNOTATION information to a file
  ---------------------------------------------*/
if (*lp || *file)
    {
    fprintf(fp,"DATE OF EXPOSURE:  %s %s %s\n", day, month, year);
    fprintf(fp,"     CENTER          NADIR          SUN        FRAME\n");
    fprintf(fp,"   LAT    LONG    LAT    LONG    ELEV AZMUTH  PATH/ROW\n");
    fprintf(fp," %s  %s   %s   %s    %s\n",center,nadir,selev,sazm,frame);
    fprintf(fp,"\n");
    }

/*
  If the user requested the information to be saved write the ANNOTATION
  information to an associated file
  ---------------------------------------------*/
if ((*initl) && ((strcmp(wrtdsk,"ANN") == 0) || (strcmp(wrtdsk,"IAS") == 0)))
    {
    /*
      Create an associated ANNOTATION file
      -----------------------------------*/
    c_lsmknm(hname,"cctx_annot",hafil);

    /*
      Open/Create the associated ANNOTATION file
      -----------------------------------------*/
    if ((fd = open(hafil,oflags,pmode)) == NULL)
	{
	c_errmsg("Error opening ANNOTATION file","plenter-open",&nonfatal);
	clean_up();
	}

    /*
      Write the ANNOTATION buffer to the associated file
      ---------------------------------------------*/
    write(fd,anbuff,size);

    /*
      Close the file
      -------------*/
    close(fd);

    /*
      Update the output file counter
      -----------------------------*/
    strcpy(out[outcnt],hafil);
    outcnt++;
    }

/*
  Set the image window size and calculate the maximum earth rotaional
  skew for the image on the initial pass.
  ---------------------------------------------*/
if (*initl)
    {
    *nl = 2340;
    *np = *adjlin;
    status = chkwind(inwin,vlwin,nl,np);
    if (status != E_SUCC)
	clean_up();

    for (i = 0; i < 4; i++)
	inwin[i] = vlwin[i];

    substr(&annot[11],sdeg,2);
    substr(&annot[14],smin,2);
    sscanf(sdeg,"%lf",&degree);
    sscanf(smin,"%lf",&minute);

    lat = (degree + (minute / 60.0)) * (TWOPI / 360.0);

    /*
      What is this equation?  We should use more significant figures.
      ---------------------------------------------------------------*/
    *skwcon = .587 * sqrt(pow(cos(lat),2.0) - .02495);
    *maxskw = (long) ((float)((*nl - 1) / 6) * (*skwcon));
	
    if (strcmp(earth,"NO") == 0)
	 *maxskw = 0;

    /*
      Set the number of pixels and number of lines used for the
      output image file.
      ---------------------------------------------*/
    *nl = inwin[2] / *linc;
    *np = (inwin[3] + *maxskw) / *sinc;
    }

return;
}
