/******************************************************************************

FUNCTION:	PLENTER

PURPOSE:	Reads in a CCTX tape and creates a LAS image.  It allows the
		user to specify which bands are needed, if synthetic pixels
		should be deleted, if a skew for the earths rotation should
		be used, and if a file of ID,annotation,and SIAT information
		should be created.  The output image is in BYTE format.

		PLENTER optionally computes the minimun and maximum pixel values
		of each of the bands.  It also optionally creates a history file
		for the image.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
1.0		    D. Holleran		Initial develop
2.0	   02/86    B. Ailts	        UNIX conversion
5.0        06/88    D. Hollaren	        Conversion to 5.0
					Converted from fortran to C
5.1        01/92    D. Etrheim          ADP #6030 Modified to approximate the
                                        corner coordinates
5.2        02/92    D. Etrheim          Corected a divide by zero error in 
					process.c 
5.3        05/92    T. Jensen           Corrected routine to remove synthetic
                                        pixels correctly and added code to
					create a raw line length file
					(.cctx_rll).
5.4	   09/92    T. Mittan		Added extern to curent and curcat
					to eliminate warnings on link
5.5        10/92    D. Etrheim          Made correction for latitudes < 0 in
					calcorn.  Changed so that the direction
					is calculated from the SIAT file.  The
					direction field is not in one version
					of the annotation record.   Corrected
					problem of invalid corners on SUN.
5.6	    2/93    S. Nelson		Took out comments so that the
					";cct_siat" file will be in ASCII rather
					than EBCDIC.  In DOSIAT.C, Eliminated
					the function SIAT.C and instead call
					the support C_READ_SIAT.
7.0         3/98    L. Johnson		Fixed string parsing for lat and long
     					in update_ddr.c
                                        Changed "hardwired" 6.28... to TWOPI.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT			LAS

ALGORITHM 
  Initalize TAE and LAS
  Set the initial processing loop flag
  Get the input parameters
  Get the working output print file
  Open the tape drive
  Process the four strips of a CCTX tape
	Get the header ID and ANNOTATION information
	Calculate the left and right edges of the strip being processed
	within the output image.
	The initial time the output needes to be created.  There after
	it only needed to be updated with the new strips.
	Process the input data
	Reset the initial processing loop flag
	Close the output file
	Mount the next tape volume if needed
  Determine if the SIAT data at the end of the tape is to be
  written to a disk file
  Close the tape
  Update the min/max values within the DDR
  Output the history information to the image
  Output a standard completion message

ALGORITHM REFERENCES
  The data is arranged in band interleaved by pixel pair (BIPS), the dat is
  processed eight bytes at a time (4 bands * 2 pixels).  The bands needed
  are written to the image file with a scaling factor added to each pixel
  (upscal) to all bands and a scaling factor is multiplied to each
  pixel of band four (b4scal).
*******************************************************************************/
#include <stdio.h>		/* Standard I/O include			*/
#include <stdlib.h>
#include <string.h>
#include "las.h"		/* LAS include file			*/
#include "util.h"
#include "tapeio.h"		/* Tape I/O include			*/
#include "protos.h"

/*
  The following variables have been declared global for image I/O routines
*/
struct FDESC * fdesc[2];	/* image file descriptors used for image I/O	*/
struct GDESC * gdesc;		/* group descriptor used for image I/O		*/

long offset1[5];	/* offsets into "thebuf" for output bands	*/
extern long curent;	/* current processing line			*/
extern long curcat;	/* current processing band			*/
long lstcnt;		/* last counter					*/
long lstidx;		/* last index counter				*/
long lstsam;		/* last sample counter				*/
long skew;		/* earth's rotational skew			*/
long uni;		/* Unit number of tape drive			*/
long outcnt;		/* Number of output files			*/
long bufsiz;		/* Size of buffer area			        */
long tape;		/* Flag for tape open				*/
long las;		/* Flag for las open				*/
long prnt;		/* Flag for print file open			*/

FILE *fp;		/* File pointer to print file			*/
 
unsigned char *thebuf;	/* Image buffer					*/
char out[6][CMLEN];	/* Output files					*/

struct PARBLK vblock;	/* Variable block				*/

int main
(void)
{
static double maxv[5];	/* buffer for maximum pixel value/band		*/
static double minv[5];	/* buffer for minimum pixel value/band		*/

static float skwcon;	/* Earth skew variable				*/

static char tname[1][CMLEN];    /* Image file name				*/
static char hname[1][CMLEN];    /* Host image file name			*/
static char prtfile[CMLEN];  /* Printer working file			*/
static char filename[CMLEN]; /* Output print file name			*/
static char wrtdsk[5];	     /* Write id, annotation, SIAT files to disk*/
static char earth[4];	     /* Correct for earth rotation flag		*/
static char synth[4];	     /* Delete synthetic pixels flag		*/
static char comment[4][CMLEN];/* Tape numbers to be used		*/
static char minmax[8];	     /* Min/Max flag				*/
static char scene[13];	/* scene/frame id				*/
static char month[4];	/* Month of exposure (Jan is to cold)		*/
static char day[3];	/* Date of exposure  (any day is fine)		*/
static char year[3];	/* Year of exposure  (I wouldn't tell you)	*/
static char direction[2];/* Direction of the satellite "D" or "A"	*/
char   center[15];         /* Center  Lat/Long                          */


long tapes;		/* Number of tapes needed			*/
long skip;		/* Number of files to skip			*/
long band;		/* Total number of bands needed			*/
long bands[5];		/* Which bands are needed			*/
long upscal;		/* Scaling factor added to each pixel		*/
long linc;		/* Line increment				*/
long sinc;		/* Sample increment				*/
long b4fac;		/* Scaling factor for band 4			*/
long msgtim;		/* elapsed time between display messages	*/
long windflg = FALSE;	/* Window flag 					*/
long std_out;		/* Standard output device			*/
long i, j;		/* Looping variable				*/
long inwin[4];		/* Input window (sl,ss,el,es)			*/
long left;		/* Left side of strip				*/
long maxskw;		/* Maximum skew possible			*/
long nl;		/* Number of lines in output image		*/
long np;		/* Number of samples in output image		*/
long right;		/* Right side of the input strip		*/
long status;		/* Return status variable			*/
long strip;		/* Current strip number				*/
long adjlin;		/* adjusted input line length			*/
long bla[4];		/* Bands wanted					*/
long initl;		/* Initial pass flag				*/
long old;		/* Old format flag				*/
long term;		/* Flag for terminal output			*/
long lp;		/* Flag for line printer output			*/
long file;		/* Flag for file output				*/

static long nonfatal = 1;	/* Non-fatal error code			*/
static long fatal = -1;		/* Fatal error code			*/
static long mode   = HANDLE;	/* Flag to handle tape errors		*/
static long rewopt = REWOFFL;	/* Rewind option			*/
static long eofopt = 1;		/* Number of EOF marks to place on tape */
static long zero   = 0;		/* zero					*/
static long opt1   = REWIND;	/* Rewind option			*/
static long inout  = TREAD;	/* Tape read/write option		*/

char dens[5];			/* tape density				*/
char tapeid[CMLEN];		/* tape id				*/

char tmpstr[1][CMLEN];  /* Temp String */

strcpy(dens,"0");
outcnt = 0;

/*
  Initialize TAE and LAS
*/
c_inlas(&vblock,&std_out);

/*
  Set initial time for processing
*/
initl = TRUE;

/*
  Signals to clean_up function that the print file, las file and tape
  have not been opened yet.
*/
prnt = FALSE;
las  = FALSE;
tape = FALSE;

/*
  Get the input parameters
*/
getpar (tname,hname,inwin,&band,bands,&b4fac,&linc,&sinc,&upscal,
	&tapes,&skip,comment,wrtdsk,earth,synth,bla,minmax,&msgtim,&windflg);

/*
  Get the working output print file
*/
strcpy(prtfile,"plenter");
status = c_setprt(&vblock,&term,&file,&lp,prtfile,filename);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","plenter-fatal",&fatal);

/*
  Open the temporary print file if going to the line printer or a file
*/
if (lp || file)
    {
    if ((fp = fopen(prtfile,"a+")) == NULL)
	{
	c_errmsg("Error opening the print file","plenter-open",&nonfatal);
	clean_up();
	}
    prnt = TRUE;

    /*
      Hold onto the print host file name so clean up can be done when an
      error occurs
    */
    strcpy(out[outcnt],prtfile);
    outcnt++;
    }

/*
  Open the tape drive
*/
status = c_tpopen(&uni,&skip,&mode,&opt1,dens,&inout,comment[0],tapeid);
if (status != TAPE_SUCC)
    clean_up();
tape = TRUE;	/* Signals to clean_up that tape drive is open		      */

/*
  Save the LAS host output names for clean up
*/
strcpy(out[outcnt],hname[0]);
outcnt++;

/*
  Process each strip of the CCTX tape
*/
for (strip = 1; strip <= 4; strip++)
    {
    /*
      Get the header ID and ANNOTATION information
    */
    header(hname[0],&strip,inwin,wrtdsk,&term,&file,&lp,&initl,&adjlin,
	&skwcon,&maxskw,&np,&nl,&old,earth,&sinc,&linc,fp,scene,day,
	month,year,center);

    /*
      Calculate the left and right edges of the strip being processed
      within the output image.
    */
    left  = (strip - 1) * (adjlin / 4) + 1;
    right = strip * (adjlin / 4);

    if ((right >= inwin[1]) && (left <= (inwin[1] + inwin[3]) - 1))
	{
	/*
	  The initial time the output needes to be created.  There after
	  it only needed to be updated with the new strips.
	*/
	open_las(hname[0],&band,bands,&np,&nl,&initl);
	las = TRUE;	/* Signals clean_up that las file is open	      */

	/*
	  Process the input data
	*/
	process(hname[0],&strip,bla,inwin,&left,&right,&upscal,&b4fac,&linc,
		&sinc,&adjlin,&skwcon,&maxskw,&initl,earth,synth,&msgtim,
		minmax,&np,minv,maxv,&band);

	/*
	  Reset the initial flag
	*/
	initl = FALSE;

	/*
	  Close the output file
	*/
	status = close_las();

	/*
	  Signals clean_up that the LAS file has been closed
	*/
	las = FALSE;
	}

    /*
      Mount the next tape volume if needed
    */
    nxtstp(comment,&tapes,&strip,&adjlin);
    }

/*
  Close the print temporary print file
*/
if (lp || file)
    {
    fclose(fp);
    prnt = FALSE;	/* Signals to clean_up that print file is closed      */
    }

/*
  Determine if the SIAT data at the end of the tape is to be
  written to a disk file.  If the siat file is not written to disk,
  we will set the direction to be decending, which will probably be the
  majority of scenes read in.  If there is a doubt, the siat file 
  should be read in to get the correct direction, but for processing at 
  EDC, it always is.  */ 
if ((strcmp(wrtdsk,"IAS") == 0) || (strcmp(wrtdsk,"SIAT") == 0))
    {
    dosiat(hname[0],&old,direction);
    }
else
    strcpy(direction,"D");

/*
  Close the tape
*/
status = c_tpclos(&uni,&mode,&rewopt,&eofopt);
if (status != TAPE_SUCC)
    c_errmsg("Nonfatal error encountered","plenter-warn",&nonfatal);

/*
  Signals to clean_up that the tape is closed
*/
tape = FALSE;

/*
  If no min/max value has been calculated use bounded because output
  image is byte.
*/
if (strcmp(minmax,"NO") == 0)
    {
    strcpy(minmax,"BOUNDED");
    for (i = 0; i < band; i++)
	{
	minv[i] = 0.0;
	maxv[i] = 255.0;
	}
    }

/*
  Update the min/max values within the DDR
*/
status = c_svmnmx(hname[0],&band,minv,maxv,minmax);
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered","plenter-warn",&nonfatal);

/*
  Update the DDR
*/
update_ddr(hname[0],inwin,&band,&linc,&sinc,scene,day,month,year,direction,
	   &old,&windflg,center);

/*
  Output the history information to the image
*/
strcpy(tmpstr[0], "\0");
status = c_hstout(&vblock,hname[0],tmpstr,&zero," ");
if (status != E_SUCC)
    c_errmsg("Nonfatal error encountered","plenter-warn",&nonfatal);

/*
  Set the output LAS file name created
*/
j = outcnt;
outcnt = 0;
strcpy(out[outcnt],tname[0]);
outcnt++;

/*
  If a file is to be created for the user create a host file name and
  then get the LAS file name
*/
if (file)
    {
    /*
      Add it to the output file list created.
    */
    strcpy(out[outcnt],filename);
    outcnt++;
    }

for (i = outcnt; i < j; i++)
    {
    strcpy(tname[0], out[i]);

    /*
      Add it to the output file list created.
    */
    strcpy(out[outcnt],tname[0]);
    outcnt++;
    }

/*
  Standard completion message
*/
c_complt("PLENTER",out,&outcnt);

return EXIT_SUCCESS;

}
