/******************************************************************************
FUNCTION:	UPDATE_DDR

PURPOSE:	Updates the DDR information for an image

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
1.0                 ????		Initial develop
2.0        01/92    D. Etrheim          ADP #6030 Modified to approximate the
					corner coordinates

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"	/* LAS include				*/
#include "proj.h"
#include "geompak.h"
#include "plenter.h"
#include "protos.h"

/*
  Macro for getting a substring
*/
#define substr(in,out,num) \
strncpy((out),(in),(num)); \
out[(num)] = '\0';	   \

void update_ddr
(
    char *hname,		/* Host file name			*/
    long inwin[],		/* Input window				*/
    long *totbnd,		/* Total number of bands to process	*/
    long *linc,		/* Line increment factor		*/
    long *sinc,		/* Sample increment factor		*/
    char *scene,		/* Scene id (contains time of exposure) */
    char *day,		/* Day of exposure			*/
    char *month,		/* Month of exposure			*/
    char *year,		/* Year of exposure			*/
    char *direction,	/* Direction of satellite "D" or "A"	*/
    long *old,		/* old version of CCTX tapes		*/
    long *windflg,          /* Window flag                    	*/
    char *center           /* Center  Lat/Long                      */
)

{
struct DDR ddr;		/* DDR structure			*/
struct BDDR bddr[4];	/* Band DDR structure			*/

long band;		/* Band looping variable		*/
long east;		/* East/west flag			*/
long fatal = -1;	/* Fatal error code			*/
long i;			/* Looping variable			*/
long indatum = 0;       /* Datum code                           */
long inproj = GEO;      /* Input lat/long projection            */
long inunit = 4;      	/* Input Unit code                      */
long inzone = 62;
long mesgflg = FALSE;   /* Indicates error messages are not to be
                                   output by the routine        */
long north;		/* North/south flag			*/
long nonfatal = 1;	/* Nonfatal error code			*/
long outproj = GEO;     /* Output lat/long projection           */
long outunit = 3; 	/* Output Unit code                     */
long prtprm[2];         /* Print flag options                   */
long status;		/* Status variable			*/

char satid[2];		/* Satellite id				*/
char hrmn[6];		/* Hour and minute of exposure		*/
char osec[2];		/* Seconds of exposure			*/
char ctemp[5];		/* temp buffer				*/

int sat;		/* satellite id				*/

double lat,		/* Nadir latitude			*/
       lng;		/* Nadir longitude			*/
double deg;		/* degress				*/
double dtemp;		/* km/sec				*/
double minutes;		/* minutes				*/
double inparm[COEFCT];  /* Input projection parameters          */
double outparm[COEFCT];  /* Input projection parameters          */

double rtnlat[4],	/* corner latitudes returned from calcorn */
       rtnlon[4];	/* corner longitudes returned from calcorn */
double latsec[4],	/* corner latitudes in total seconds */
       lonsec[4];	/* corner latitudes in total seconds */
long   icoords[8];      /* image cooordinates */ 
double coef[6];  	/* Transformation coefficients */ 

/*
  Strip the satellite id number from the scene id
*/
substr(scene,satid,1);

/*
  Strip the time of exposure from the scene id
*/
if (*old)
    {
    substr(&scene[5],hrmn,4);
    substr(&scene[9],osec,1);
    }
else
    {
    substr(&scene[6],hrmn,4);
    substr(&scene[10],osec,1);
    }

/*
  Get the DDR information from the image file
*/
status = c_getddr(hname,&ddr);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","plenter-fatal",&fatal);

/*
  Initialize the validity flags
*/
for (i = 0; i < 8; i++)
    ddr.valid[i] = VALID;

/*
  Update the values that are known
*/
ddr.master_line   = inwin[0] - 1;
ddr.master_sample = inwin[1] - 1;
ddr.line_inc   = (double)*linc;
ddr.sample_inc = (double)*sinc;

/* 
  Find some corner coordinates.  They are only rough estimates.
  First convert the center to total degrees.
*/
if (center[0] == 'N')
   north = TRUE;   
else
   north = FALSE;
 
center++;
strncpy(ctemp,center,2);
ctemp[2] = (char)NULL;
deg = (double)atol(ctemp);
center+=3;
strncpy(ctemp,center,2);
ctemp[2] = (char)NULL;
minutes = (double)atol(ctemp);
center+=3;

if (north)
   lat = deg + (minutes/60);
else
   lat = (deg + (minutes/60)) * -1;

if (center[0] == 'E')
   east = TRUE;   
else
   east = FALSE;
 
center++;
strncpy(ctemp,center,3);
ctemp[3] = (char)NULL;
deg = (double)atol(ctemp);
center+=4;
strncpy(ctemp,center,2);
ctemp[2]= (char)NULL;
minutes = (double)atol(ctemp);

if (east)
   lng = deg + (minutes/60);
else
   lng = (deg + (minutes/60)) * -1;

/* change the sat. number to a long */
sat = atol(satid);

/* calcorn will give us the four corners 
----------------------------------------*/
calcorn (lat, lng, direction, rtnlat, rtnlon, sat);

/* Run the lat/longs thru the projetion package even though going from
   geographic to geographic.  A different output projection can be added
   quickly if and when it is known.
-----------------------------------------------------------------------*/
for (i = 0;i < COEFCT;i++)
   {
   inparm[i] = 0.0;
   outparm[i] = 0.0;
   }
prtprm[0] = 0;
prtprm[1] = 0;
if ( c_transinit(&inproj,&inunit,&inzone,&indatum,inparm,&outproj,
              &outunit,&inzone,&indatum,outparm, prtprm,"") != E_SUCC )
   c_errmsg("Fatal error encountered","plenter-fatal",&fatal);

for (i = 0;i < 4;i++)
   {
   if ( c_trans(&inproj,&inunit,&outproj,&outunit,
            &rtnlon[i],&rtnlat[i],&lonsec[i],
            &latsec[i]) != E_SUCC)
   c_errmsg("Fatal error encountered","plenter-fatal",&fatal);
   }

ddr.zone_code = inzone;
ddr.datum_code = indatum;
ddr.upleft[0] = latsec[0];
ddr.upleft[1] = lonsec[0];
ddr.upright[0] = latsec[3];
ddr.upright[1] = lonsec[3];
ddr.loright[0] = latsec[2];
ddr.loright[1] = lonsec[2];
ddr.loleft[0] = latsec[1];
ddr.loleft[1] = lonsec[1];
strcpy(ddr.proj_units,"seconds");

/* Approximate the pixel distance in seconds for a sphere.  dtemp is km/second 
at given latitude. Pixel distance is 79mx57m.
------------------------------------------------------------------------*/
dtemp = (((REQ * cos((lat/RAD))) * 2 * PI) / 360) / 3600;
ddr.pdist_x = .057 / dtemp;
/* pixel distance in longitude direction should be constant */
dtemp = ((REQ  * 2 * PI) / 360) / 3600;
ddr.pdist_y = .079 / dtemp;

/* If image was windowed from tape, calculate some corner points.
   These will probably be very rough since we are not realy working
   with a true geographic projection.
----------------------------------------------------------------*/
if (*windflg)
   {
/*
   Calculate the transformation coefficients mapping file coordinates to
   projection coordinates.
*/
   icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
   icoords[2] = icoords[6] = ddr.nl;
   icoords[5] = icoords[7] = ddr.ns;
   status = c_rtcoef(ddr.upleft,icoords,coef,&mesgflg);
   if (status != E_SUCC)
      {
      c_errmsg("Error calculating corner coordinates","plenter-corner",
               &nonfatal);       
      ddr.upleft[0] = 0.0;
      ddr.upleft[1] = 0.0;
      ddr.upright[0] = 0.0;
      ddr.upright[1] = 0.0;
      ddr.loright[0] = 0.0;
      ddr.loright[1] = 0.0;
      ddr.loleft[0] = 0.0;
      ddr.loleft[1] = 0.0;
      ddr.valid[DDCCV] = INVAL;
      }
  else
      {
      ddr.upleft[0] = (coef[0] * inwin[SL]) + (coef[1] * inwin[SS]) + coef[2];
      ddr.upleft[1] = (coef[3] * inwin[SL]) + (coef[4] * inwin[SS]) + coef[5];
      ddr.upright[0] = (coef[0] * inwin[SL]) + 
                       (coef[1] * (inwin[SS] + inwin[NS])) + coef[2];
      ddr.upright[1] = (coef[3] * inwin[SL]) + 
                       (coef[4] * (inwin[SS] + inwin[NS])) + coef[5];
      ddr.loright[0] = (coef[0] * (inwin[NL] + inwin[SL])) + 
                       (coef[1] * (inwin[SS] + inwin[NS])) + coef[2];
      ddr.loright[1] = (coef[3] * (inwin[NL] + inwin[SL])) + 
                       (coef[4] * (inwin[SS] + inwin[NS])) + coef[5];
      ddr.loleft[0] = (coef[0] * (inwin[NL] + inwin[SL])) + 
                      (coef[1] * inwin[SS]) + coef[2];
      ddr.loleft[1] = (coef[3] * (inwin[NL] + inwin[SL])) + 
                      (coef[4] * inwin[SS]) + coef[5];
      }
   }

/*
  Update the Band DDR information
*/
for (band = 1; band <= *totbnd; band++)
    {
    /*
      Get the Band DDR information from the image
    */
    status = c_getbdr(hname,&bddr[band-1],&band);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","plenter-fatal",&fatal);

    /*
      Update the Band DDR information
    */
    sprintf(bddr[band-1].source,"landsat %s",satid);
    strcpy(bddr[band-1].instrument,"mss");

    /*
      Update the direction of the pass
    */
    if (strcmp(direction,"D") == 0)
	strcpy(bddr[band-1].direction,"descending");
    else
	strcpy(bddr[band-1].direction,"ascending");

    /*
      Update the date and time
    */
    sprintf(bddr[band-1].date,"%s-%s-%s",day,month,year);
    sprintf(bddr[band-1].time,"%s:%s",hrmn,osec);
    }

/*
  Put the updated DDR information to the image
*/
status = c_putddr(hname,&ddr);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","plenter-fatal",&fatal);

/*
  Put the updated Band DDR information to the image
*/
for (band = 0; band < *totbnd; band++)
    {
    status = c_putbdr(hname,&bddr[band]);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","plenter-fatal",&fatal);
    }

return;
}
