/****************************************************************************
NAME:               find_corners

PURPOSE:  Update projection corners to correspond to output mapping grid

ALGORITHM DESCRIPTION:
    If the window is null, use the framed space as the window
       Calculate the number of lines and samples in the framed space.
    Else,
       Place the user entered window in a double buffer
       Calcluate frame space line,sample coordinates of corners
       Call c_rotcor to update corners for user entered window
    Return

SPECIAL CONSIDERATION:
	If the window not specified, the corner coords (header->corners) are simply 
	copied over to the grid (frame->corners).  If the window is specified the
	corner coordinates are calculated.  The corner coordinates will be 
	calculated from the UL, UR, and LL corners.  Thus, if the window specified
	is the entire image, the LR corner might be different than if the window
	was not specified.  The calculated corners will be a parallelogram where
	as the input coordinates might not lie on lines which are parallel with each
	other.

*****************************************************************************/
#include <math.h>
#include "polyfit.h"

int find_corners
(
    struct FRAME *frame,    /* Framing parameters */
    struct TPLHEAD *header /* Tie point location file header data */
)
{
double dwind[4];    /* Input window; double precision */
double a,b;     /* Temporary variables for nlines,nsamps calculation */
long ls[8];     /* Line, sample coordinates in frame space */
long count;     /* Loop counter */
long nlines;        /* Number of lines in input frame space */
long nsamps;        /* Number of samples in input frame space */


if ((frame->window[2] == 0) && (frame->window[3] == 0))
    {
    if (header->proj_valid == GEO_NOTVALID) return(NO_FRAME);


	/* The following code uses the Distance Formula where 'a' is dx
		and 'b' is dy.  The distance divided by the pixel distance is
		calculated because the number of lines in the original image
		is not immediately available.  The 1 is added to include the
		last pixel.  The .5 is added to round correctly when the double
		is converted to a long.
	-----------------------------------------------------------------*/
    a = (header->corners[0] - header->corners[2]) / header->pdist[0];
    b = (header->corners[1] - header->corners[3]) / header->pdist[1];
    nlines = (long)(sqrt((a * a) + (b * b)) + 1.5);
    a = (header->corners[0] - header->corners[4]) / header->pdist[0];
    b = (header->corners[5] - header->corners[1]) / header->pdist[1];
    nsamps = (long)(sqrt((a * a) + (b * b)) + 1.5);

    frame->window[0] = frame->window[1] = 1;
    frame->window[2] = nlines;
    frame->window[3] = nsamps;
	
	/* The corner coords are copied over */
    for (count = 0; count < 8; count++)
        frame->corners[count] = header->corners[count];
    }
else
    {
    if (header->proj_valid == GEO_NOTVALID)
        for (count = 0; count < 8; count++)
            frame->corners[count] = 0.0;
    else
        {
        a = (header->corners[0] - header->corners[2]) / header->pdist[0];
        b = (header->corners[1] - header->corners[3]) / header->pdist[1];
        nlines = (long)(sqrt((a * a) + (b * b)) + 1.5);
        a = (header->corners[0] - header->corners[4]) / header->pdist[0];
        b = (header->corners[5] - header->corners[1]) / header->pdist[1];
        nsamps = (long)(sqrt((a * a) + (b * b)) + 1.5);

        for (count = 0; count < 4; count++)
            dwind[count] = (double)frame->window[count];
        ls[0] = 1;
        ls[1] = 1;
        ls[2] = nlines;
        ls[3] = 1;
        ls[4] = 1;
        ls[5] = nsamps;
        ls[6] = nlines;
        ls[7] = nsamps;

        if (c_rotcor(header->corners, ls, dwind, frame->corners) != E_SUCC)
            {
            c_lasmsg(FALSE,"polyfit-warn","Nonfatal error encountered");
            for (count = 0; count < 8; count++)
                frame->corners[count] = 0.0;
            }
        }
    }
return(E_SUCC);
}

