/****************************************************************************
NAME:               getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

ALGORITHM DESCRIPTION:
    Get the input tie point location file name
    Get the output tie point location file name
    Get the output geometric mapping grid file name
    Get the wind specification.  This determines the output area of the
       output image described by the mapping grid 
    Get the number of lines and samples desired in each grid cell in the
       output space.  These can have values or can be null.
    Get the tolerance value for grid reduction
    Get the entry & exit tests (alpha values) for the stepwise regression
    Get the maximum degree of the polynomial
    Get the maximum residual value allowed.  If this is null, manual 
       editing is assumed
    Get the transformation option--forward or inverse
    Get the runtype from TAE.  
    If runtype == batch and the maximum residual value is null, 
        set edit_mode to NO_EDIT
    Get the print option parameters (flag & name)
    Get message timer
    Return
*****************************************************************************/
#include "polyfit.h"
#include "typlim.h"

int getpar
(
    struct PARBLK *vblk,    /* TAE variable block */
    char *tpl_name,     /* Input tie point location file name */
    char *out_tpl,      /* Output tie point location file name */
    struct FRAME *frame,    /* Output framing parameters */
    struct FIT *fit,    /* Model fit parameters */
    struct PRINT *print,    /* Print/report parameters */
    long *msgtim,       /* Message timer */
    char *plotnm,       /* plot file name */
    long *prtopt           /* Print option */
)
{
struct VARIABLE *var;       /* pointer to a TAE variable in vblk */
long acc;           /* access flag for c_getnam */
long count;         /* Loop counter */
long zero = 0;          /* Constant of zero */
long term, ln_pr, file;     /* Print option flags */
long status;            /* function status */
char tname[CMLEN];      /* Print file name as entered by user */

/* Get the input tie point location file name
------------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INTL", &zero, tpl_name, "tpl", &acc) != E_SUCC)
    return(NO_TPL);

/* Get the output tie point location file name
-------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTTL", &zero, out_tpl, "tpl", &acc) != E_SUCC)
    return(BAD_TPL);

/* Get the output geometric mapping grid file name
-----------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTGRID", &zero, frame->name, "grid", &acc) != E_SUCC)
    return(BAD_OUTGRID);

/* Get the wind specification.  This determines the output area of the
output image described by the mapping grid
------------------------------------------*/
var = p_find(vblk, "WINDOW");
if ((*var).v_count == 0)
    for (count = 4; count--; frame->window[count] = 0);

else for (count = 0; count < 4; count++)
    frame->window[count] = IVAL(*var, count);

/* Get the number of lines and samples desired in each grid cell in the
output space.  These can have values or can be null.
---------------------------------------------------*/
var = p_find(vblk, "NLGRID");
if ((*var).v_count > 0)
{
    frame->ngline = IVAL(*var, 0);
    if (frame->ngline < 1)
    {
        c_lasmsg(FALSE,"polyfit-invalid","NLGRID must be greater than zero");
        return (E_FAIL);
    }
}
else
    frame->ngline = 0;

var = p_find(vblk, "NSGRID");
if ((*var).v_count > 0)
{
    frame->ngsamp = IVAL(*var, 0);
    if (frame->ngsamp < 1)
    {
        c_lasmsg(FALSE,"polyfit-invalid","NSGRID must be greater than zero");
        return (E_FAIL);
    }
}
else
    frame->ngsamp = 0;

/* Get the tolerance value for grid reduction
------------------------------------------*/
var = p_find(vblk, "TOLVAL");
frame->tol = RVAL(*var, 0);

/* Get the entry & exit test (alpha values) for the stepwise regression
--------------------------------------------------------------------*/
var = p_find(vblk, "STATSIG");
fit->alpha[0] = RVAL(*var, 0);
fit->alpha[1] = RVAL(*var, 1);

/* Get the maximum allowable degree of polynomial for model
--------------------------------------------------------*/
var = p_find(vblk, "PDEGREE");
fit->degree = IVAL(*var, 0);

/* Get the maximum residual value allowed.  If this is null, manual editing
is assumed
----------*/
var = p_find(vblk, "MAXRES");
fit->edit_mode = ((*var).v_count > 0) ? AUTO: MANUAL;
fit->max_res = ((*var).v_count > 0) ? RVAL(*var, 0) : 0.0;

/* Get the print option
--------------------*/
var = p_find(vblk, "PRTOPT");
if (strcmp(SVAL(*var, 0), "ALL") == 0)
    *prtopt = TRUE;
else
    *prtopt = FALSE;

/* Get the tranformation option
----------------------------*/
var = p_find(vblk, "TRANOPT");
if (strcmp(SVAL(*var, 0), "INV") == 0) fit->xform_mode = INVERSE;
else fit->xform_mode = FORWARD;

/* Check the runtype.  If runtype is batch, set the edit_mode to NOEDIT
--------------------------------------------------------------------*/
var = p_find(vblk, "$RUNTYPE");
if ((strcmp(SVAL(*var, 0), "INTERACTIVE") != 0) && (fit->edit_mode == MANUAL))
    {
    fit->edit_mode = NOEDIT;
    fit->max_res = MAXREAL;
    }

/* Get the print option.  Make a file name (if neccessary) and open the
temporary file
--------------*/
strcpy(print->name, "polyfit");
status = c_setprt(vblk, &term, &file, &ln_pr, print->name, tname);
if (status != E_SUCC)
    c_lasmsg(TRUE,"polyfit-fatal","Fatal error encountered");

acc = IUPDATE;
if (file == 1) c_mknam(tname, print->host_name, "prt", &acc);
else strcpy(print->host_name, " ");

/* If editing mode is manual, output must go to the terminal, reguardless of
the PRINT parameter entered!
----------------------------*/
if (fit->edit_mode == MANUAL) term = 1;

/* Determine combination of output destinations
--------------------------------------------*/
if ((term == 1) && (ln_pr == 0) && (file == 0)) print->option = TERM;
if ((term == 1) && ((ln_pr == 1) || (file == 1))) print->option = ALL;
if (term == 0) print->option = PFILE;

if (print->option != TERM)
    {
    print->fptr = fopen(print->name, "w");
    fprintf(print->fptr, "Input Tie Point Location File name:   %s\n",
            tpl_name);
    fprintf(print->fptr, "Output Tie Point Location File name:  %s\n",
            out_tpl);
    fprintf(print->fptr, "Geometric Mapping Grid File name:     %s\n",
            frame->name);
    fprintf(print->fptr,"\nAlpha (enter test):  %f\n",fit->alpha[0]);
    fprintf(print->fptr,"Alpha (exit test):   %f\n\n",fit->alpha[1]);
    fprintf(print->fptr, "Explanation of Active Flag Codes:\n");
    fprintf(print->fptr, "   0:  Accept manually extracted tie point\n");
    fprintf(print->fptr, "   1:  Accept automatically extracted tie point\n");
    fprintf(print->fptr,
    "   2:  Reject--correlation peak too near edge of search image\n");
    fprintf(print->fptr,
    "   3:  Reject--subsidiary peak comparable in strength to main peak\n");
    fprintf(print->fptr,
    "   4:  Reject--strength of peak below minimum specified by user\n");
    fprintf(print->fptr,
    "   5:  Reject--diagonal displacement from nominal location exceeds\n");
    fprintf(print->fptr,
    "               the maximum specified by the user\n");
    fprintf(print->fptr,
    "   6:  Correlation not attempted--error in correlation parameters\n");
    fprintf(print->fptr, "   7:  Reject--manually by the user\n");
    fprintf(print->fptr,
    "   8:  Reject--automatically by the modeling process\n");
    fprintf(print->fptr,
    "   9:  Reject--manually by the user during the modeling process\n");
    fprintf(print->fptr,
    "  1X:  Previously rejected point re-accepted manually by user\n");
    fprintf(print->fptr,
    "         X indicates original active flag code before re-acceptance\n\n");
    }

/* Get the plot file name
--------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTPLOT", &zero, plotnm, "plot", &acc) != E_SUCC)
    return(E_FAIL);

/* Get the message timer
---------------------*/
var = p_find(vblk, "$msgtim");
*msgtim = IVAL(*var, 0);

return(E_SUCC);
}

