/****************************************************************************
NAME:                 MAKE_GRID 

PURPOSE:  Creates a Geometric Mapping Grid file from a set of transformation
      coefficeints.  Grid reduction techniques may be applied and a
      summary of gridding errors is calcualted and reported to the user.

ALGORITHM DESCRIPTION:
    Set up pointer-to-function for report destination
    Build the grid
    Pack projection related fields from the TPL header into the Geometric
       Mapping Grid file
    Pack modeling parameters in the Geometric Mapping Grid file
    Pack modeling residuals in the Geometric Mapping Grid file
    Calculate errors due to gridding & record in the grid file
    Report errors due to gridding to the user
    Write the Geometric Mapping Grid file to disk
    Return
*****************************************************************************/
#include <math.h>
#include "polyfit.h"

typedef struct
{
    struct COEF *coef;  /* output space to input space transform coefficients */
} OLS2ILS_PARAMS;

static int ols2ils
(
    void *data,     /* I: pointer to a OLS2ILS_PARAMS structure for the 
                          parameters specific to this gridding application */
    int oline,      /* I: output space line */
    int osamp,      /* I: output space sample */
    double *iline,  /* O: input space line */
    double *isamp   /* O: input space sample */
)
{
    double yval = (double)oline;
    double xval = (double)osamp;
    struct COEF *coef = ((OLS2ILS_PARAMS *)data)->coef;

    *iline = c_eval(&(coef->degree), coef->y_coef, &xval, &yval);
    *isamp = c_eval(&(coef->degree), coef->x_coef, &xval, &yval);
    
    return (E_SUCC);
}

int make_grid
(
    struct FRAME *frame,        /* Output framing parameters */
    struct COEF *coef,          /* Mapping model coefficients */
    struct RESIDUAL *residual,  /* Residuals of fit */
    struct TPLHEAD *header,     /* Header data from tie point location file */
    long xform,                 /* Transformation mode--forward or inverse */
    struct PRINT *print,        /* Print file parameters */
    long msgtim                 /* Message timing parameter */
)
{
struct GEOGRID grid;        /* Geometric mapping grid */
double *dblptr;         /* Temporary pointer to a double */
long count;        /* Loop counters */
char mess[80];          /* Message buffer for (*printit) */
OLS2ILS_PARAMS ols2ils_data; /* parameters to pass to ols2ils */

void (*printit)(char*, FILE*);      /* Pointer to print function */

/* Determine report output destination and set up a pointer to the appropriate
function
--------*/
if (print->option == TERM) printit = print_term;
if (print->option == PFILE) printit = print_file;
if (print->option == ALL) printit = print_both;

ols2ils_data.coef = coef;

/* build the grid
-----------------*/
if (c_makegrid(frame->window, frame->ngline, frame->ngsamp, ols2ils,
               &ols2ils_data, frame->tol, "POLYFIT", msgtim, &grid) != E_SUCC)
{
    c_lasmsg(FALSE,"makegrid-fail","Error returned from c_makegrid");
    return (E_FAIL);
}

/* Pack projection related fields in the grid file
-----------------------------------------------*/
if (xform == INVERSE)
    {
    grid.proj_valid = header->proj_valid;
    grid.code = header->code;
    grid.zone = header->zone;
    grid.datum = header->datum;
    for (count = 0; count < 8; count++)
        grid.corners[count] = frame->corners[count];
    grid.pdist[0] = header->pdist[0];
    grid.pdist[1] = header->pdist[1];
    for (count = 0; count < 15; count++)
        grid.projprms[count] = header->projprms[count];
    strcpy(grid.units, header->units);
    }
else  /* xform == FORWARD */
    grid.proj_valid = GEO_NOTVALID;

/* Pack modeling related fields in the grid file
---------------------------------------------*/
strcpy(grid.model, "POLYNOMIAL");
grid.ncoeff = (coef->degree + 1) * (coef->degree + 2) / 2;
grid.coeffs = (double *)malloc((grid.ncoeff * 2) * sizeof(double));
if (grid.coeffs == NULL) return(NO_SPACE);
for (count = 0, dblptr = grid.coeffs; count < grid.ncoeff; count++, dblptr++)
    *dblptr = coef->y_coef[count];
for (count = 0; count < grid.ncoeff; count++, dblptr++)
    *dblptr = coef->x_coef[count];

/* Pack statistics related fields in the grid file & report to the user the
"goodness" of the modeling and the gridding process
---------------------------------------------------*/
grid.fiterr_valid = GEO_VALID;
grid.max_fit_err[0] = residual->y_max_err;
grid.max_fit_err[1] = residual->x_max_err;
grid.ave_fit_err[0] = residual->y_ave_err;
grid.ave_fit_err[1] = residual->x_ave_err;
grid.rms_fit_err = residual->ave_rmse;

c_grderr(&grid, &(frame->tol));

(*printit)(" ", print->fptr);
sprintf(mess, "Geometric Mapping Grid file created with %d rows by %d columns",
grid.nrows, grid.ncols);
(*printit)(mess, print->fptr);
sprintf(mess, "Number of output lines:    %d",grid.lines);
(*printit)(mess, print->fptr);
sprintf(mess, "Number of output samples:  %d",grid.samples);
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
sprintf(mess, "Average Grid Interpolation Line Residual:    %lf",
grid.ave_grid_err[0]);
(*printit)(mess, print->fptr);
sprintf(mess, "                           Sample Residual:  %lf",
grid.ave_grid_err[1]);
(*printit)(mess, print->fptr);
sprintf(mess, "                           RMSE:             %lf",
grid.rms_grid_err);
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
sprintf(mess, "Max Grid Interpolation Line Residual:    %lf",
grid.max_grid_err[0]);
(*printit)(mess, print->fptr);
sprintf(mess, "                       Sample Residual:  %lf",
grid.max_grid_err[1]);
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);

/* Write the geometric mapping grid to disk
----------------------------------------*/
if(c_putgrd(frame->name, &grid) != E_SUCC) return(E_FAIL);

free(grid.out_lines);
free(grid.out_samps);
free(grid.in_lines);
free(grid.in_samps);
free(grid.coeffs);

return(E_SUCC);
}

