/****************************************************************************
NAME:               model

PURPOSE:  Driver for the modeling process:  Computes transformation 
      coefficients, computes residuals, reports to user, edits
      tie points, & writes a plot file until resulting model is
      satisfactory or the user aborts

ALGORITHM DESCRIPTION:
    Allocate memory for the RESIDUAL structure
    Set the degree of polynomial used to the maximum degree the user entered
    Let satisfied = FALSE
    Do while not satisfied:
       Calculate transformation coefficients
       Calculate residual errors
       Create a residual error plot file
       Report residual errors to the user
       Modify tie points & set satisfied TRUE if satisfied with fit
    Return
*****************************************************************************/
#include "polyfit.h"

int model
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,        /* Polynomial fit parameters */
    struct PRINT *print,        /* Print parameters */
    struct COEF *coef,      /* Results of model--coefficients & stats */
    struct RESIDUAL *residual,  /* Residuals from fit */
    char *plotnm,               /* plot file name */
    long *prtopt           /* print option */
)

{
long satisfied;         /* TRUE/FALSE flag for fit completion */
long num_bytes;         /* Number of bytes to allocate for residuals */
long dummy;         /* Dummy variable */
long status;            /* Return status flag */
long keepopt;

/* Allocate memory for the RESIDUAL structure
------------------------------------------*/
num_bytes = (long)(coef->npts * sizeof(double));
residual->x_residual = (double *)malloc(num_bytes * 3);
if (residual->x_residual == NULL) return(NO_SPACE);

residual->y_residual = residual->x_residual + coef->npts;
residual->rms_residual = residual->y_residual + coef->npts;

coef->degree = fit->degree;
find_max_degree(coef->npts, &(coef->degree), &dummy);

if (! *prtopt)
    {
    keepopt = print->option;
    print->option = TERM;
    }
satisfied = FALSE;
while (!satisfied)
    {
    status = calc_coef(tie_data, fit, coef);
    if (status != E_SUCC) return(status);
    calc_residuals(tie_data, coef, residual);
    do_plot(tie_data, coef, residual, plotnm);
    if (*prtopt || fit->edit_mode == MANUAL)
        report(tie_data, coef, residual, fit->xform_mode, print);
    satisfied = modify(residual, tie_data, fit, coef);
    }
if (! *prtopt && keepopt != TERM)
    {
    if (keepopt == ALL)
        print->option = PFILE;
    else
        print->option = keepopt;
    report(tie_data, coef, residual, fit->xform_mode, print);
    }
return(E_SUCC);
}

