/****************************************************************************
NAME:               modify

PURPOSE:  Allows user to edit tie points, change degree of polynomial, etc.

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  1.0    3/82   R. Wenger     CSC      N/A    (Old name:  TIEPOINTS)
  4.0    5/85   S. Kondal     SAR      20-472 (Old name:  TIEFIT)
  5.0    1/89   D. Steinwand  CSB      LAS 5.0 C conversion
  5.01   11/89  D. Steinwand  CSB      Modified poly_opt

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
    If using manual editing
       Ask user if fit is OK
       If OK, return (satisfied)
       If not OK, return (results from next prompt --poly_opt)
    Else return (auto_edit results)
*****************************************************************************/
#include "polyfit.h"
#include <math.h>

int modify
(
    struct RESIDUAL *residual,  /* Structure containing residual values */
    struct TPLDATA *tie_data,   /* Structure containing tie point records */
    struct FIT *fit,        /* Structure containing fit parameters */
    struct COEF *coef      /* Structure containing xform coef's & stats */
)
{
TEXT mess[80];          /* Message buffer */
FUNINT flag = T_PROMPT;     /* Prompt flag for t_ calls */
CODE term;          /* TERM code for t_ calls */

if (fit->edit_mode == MANUAL)
    {
    t_write("Is the Tie point fitting satisfactory (Y/N)?  ", flag);
    t_read(mess, &term);
    if ((strncmp(mess,"y",1) == 0) || (strncmp(mess,"Y",1) == 0))
        return(TRUE);
    if ((strncmp(mess,"n",1) == 0) || (strncmp(mess,"N",1) == 0))
        return(poly_opt(tie_data, fit, coef));
    
    /* If a correct response was not given, loop until the response is valid
    ---------------------------------------------------------------------*/
    for (;;)
        {
        t_write("Please enter Y or N:  ", flag);
        t_read(mess, &term);
        if (strlen(mess) == 0) return(TRUE);
        if ((strncmp(mess,"y",1) == 0) || (strncmp(mess,"Y",1) == 0))
            return(TRUE);
        if ((strncmp(mess,"n",1) == 0) || (strncmp(mess,"N",1) == 0))
            return(poly_opt(tie_data, fit, coef));
        }
    }
else return(auto_edit(residual, tie_data, fit, coef));
}

/****************************************************************************
NAME:               auto_edit

PURPOSE:  Automatically edits points based on the maximum residual value

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  1.0    3/82   R. Wenger     CSC      N/A    (Old name:  TIEPOINTS)
  4.0    5/85   S. Kondal     SAR      20-472 (Old name:  TIEFIT)
  5.0    1/89   D. Steinwand  CSB      LAS 5.0 C conversion

ALGORITHM DESCRIPTION:
    If the number of points used in the fit < 3, edit process done, return
    Find which is larger, the maximum x residual or the maximum y residual
    If the maximum or x or y found above is less than the maximum allowed
       residual, the editing process is complete, return
    Scan the tie point records for the point id of the tie point with the
             largest residual in x or y (found above)
       Set the active flag to "INACTIVE"
       Report to the user the point made inactive
    Adjust the degree of fit, if needed
    Return (not statisfied)
*****************************************************************************/
int auto_edit
(
    struct RESIDUAL *residual,  /* Residual errors for each tie point */
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,        /* User entered fitting parameters */
    struct COEF *coef      /* Coefficient & statistics records */
)
{
struct TPLDATA *tmp;    /* Temporary pointer to tie point data records */
long count;     /* Loop index */
char pt_id[20];     /* Point id of tie point with largest residual */

if (coef->npts_fit <= 3)
    {
    c_lasmsg(FALSE," ","No more points can be deleted from the model");
    return(TRUE);
    }

if (fabs(residual->x_max_err) > fabs(residual->y_max_err))
    {
    strcpy(pt_id, residual->x_max_ptid);
    if (fabs(residual->x_max_err) < fit->max_res) return(TRUE);
    }
else
    {
    strcpy(pt_id, residual->y_max_ptid);
    if (fabs(residual->y_max_err) < fit->max_res) return(TRUE);
    }
for (tmp = tie_data, count = 0; count < coef->npts; count++, tmp++)
    if (strcmp(pt_id, tmp->pt_id) == 0)
    {
    if ((tmp->active < 2) || (tmp->active > 9)) coef->npts_fit--;
    tmp->active = 8;
    c_lasmsg(FALSE," ","Point excluded:  %s",tmp->pt_id);
    find_max_degree(coef->npts_fit, &(coef->degree), &(fit->degree));
    }
return(FALSE);
}

/****************************************************************************
NAME:               poly_opt

PURPOSE:  Second prompt (editing prompt) parsing routine

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  1.0    3/82   R. Wenger     CSC      N/A    (Old name:  TIEPOINTS)
  4.0    5/85   S. Kondal     SAR      20-472 (Old name:  TIEFIT)
  5.0    1/89   D. Steinwand  CSB      LAS 5.0 C conversion
  5.01   11/89  D. Steinwand  CSB      Change--if too few points remain
                    to fit, polyfit warned the user but
                    let them attempt a fit, resulting in
                    a non-trapable fatal error.  This was
                    changed.

ALGORITHM DESCRIPTION:
    Report to the user the current polynomial degree
    Report to the user the number of tie points used in the fit
    Report to the user the maximum possible polynomial degree
    Loop until valid user input is given
       Write prompt to user's terminal
       Get user's response
       If response == exclude, call mod_pt(EXCLUDE)
       If response == include, call mod_pt(INCLUDE)
       If response == change degree & run,
        Change_degree
        Return (not statisfied)
       If response == run, return (not statisfied)
       If response == quit, quit
       If the current degree is now == 0,
        Print error condition & direct user to quit or include a point
*****************************************************************************/
int poly_opt
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,        /* Fit parameters as entered by user */
    struct COEF *coef      /* Transformation coefficients & statistics */
)
{
TEXT mess[80];      /* Message buffer for t_ calls */
CODE term;      /* Terminal code for t_ calls */

t_write(" ", T_STDCC);
sprintf(mess, "Current degree = %d", coef->degree);
t_write(mess, T_STDCC);
sprintf(mess, "Number of points = %d", coef->npts_fit);
t_write(mess, T_STDCC);
sprintf(mess, "Maximum possible degree = %d", fit->degree);
t_write(mess, T_DOUBLE);
for (;;)
    {
    t_write(
    "Enter (E)xclude, (I)nclude, (Q)uit, (R)un, (C)hange degree & run:  ",
    T_PROMPT);
    t_read(mess, &term);
    if ((strncmp(mess,"e",1) == 0) || (strncmp(mess,"E",1) == 0))
        mod_pt(tie_data, fit, coef, EXCLUDE);
    if ((strncmp(mess,"i",1) == 0) || (strncmp(mess,"I",1) == 0))
        mod_pt(tie_data, fit, coef, INCLUDE);
    if ((strncmp(mess,"c",1) == 0) || (strncmp(mess,"C",1) == 0))
        {
        change_degree(coef->npts_fit, &(coef->degree), &(fit->degree));
        if (coef->degree > 0) return(FALSE);
        }
    if ((strncmp(mess,"r",1) == 0) || (strncmp(mess,"R",1) == 0))
        return(FALSE);
    if ((strncmp(mess,"q",1) == 0) || (strncmp(mess,"Q",1) == 0))
        pabort(QUIT_IN_MOD);
    if (coef->degree == 0)
        {
        t_write(" ", T_STDCC);
        t_write("Warning:  Too few Tie points remain to produce a valid fit!",
        T_STDCC);
        t_write("          Either INCLUDE a removed point or QUIT", T_DOUBLE);
        for (;;)
            {
            t_write("Enter (I)nclude, (Q)uit:  ", T_PROMPT);
            t_read(mess, &term);
            if ((strncmp(mess,"i",1) == 0) || (strncmp(mess,"I",1) == 0))
                mod_pt(tie_data, fit, coef, INCLUDE);
            if ((strncmp(mess,"q",1) == 0) || (strncmp(mess,"Q",1) == 0))
                pabort(QUIT_IN_MOD);
            coef->degree = fit->degree;
            if (coef->degree != 0) break;
            }
        }
    }
}


/****************************************************************************
NAME:               mod_pt

PURPOSE:  Include or exclude a tie point from the fitting (modeling) process

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  1.0    3/82   R. Wenger     CSC      N/A    (Old name:  TIEPOINTS)
  4.0    5/85   S. Kondal     SAR      20-472 (Old name:  TIEFIT)
  5.0    1/89   D. Steinwand  CSB      LAS 5.0 C conversion

ALGORITHM DESCRIPTION:
    Loop until satisfactory user input is given
       Prompt the user to enter the tie point id of the point to be modifed
       Scan the tie point data records, until point_id matches
        If Excluding a point
           Set the point to inactive
           Report to the user which point is inactive
           Check degree & adjust if necessary
        Else if Including a point
           Set the point to active
           Report to the user that the point is now active
           Check degree & adjust if possible
        Return
       Print error message--Tie point id not found; re-enter
*****************************************************************************/
void mod_pt
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,        /* User entered fit parameters */
    struct COEF *coef,      /* Transformation coefficients & statistics */
    long action            /* Edit action--INCLUDE or EXCLUDE */
)
{
struct TPLDATA *tmp;    /* Temporary pointer to TPL data records */
TEXT mess[80];      /* Message buffer for t_ calls */
CODE term;      /* Terminal code for t_ calls */
long count;     /* Loop counter */

for (;;)
    {
    t_write("Enter Tie point name:  ", T_PROMPT);
    t_read(mess, &term);
    for (tmp = tie_data, count = 0; count < coef->npts; count++, tmp++)
        if (strcmp(mess, tmp->pt_id) == 0)
        {
        if (action == EXCLUDE)
            {
            if ((tmp->active < 2) || (tmp->active > 9)) coef->npts_fit--;
            tmp->active = 9;
            sprintf(mess, "Point excluded:  %s",tmp->pt_id);
            t_write(mess, T_STDCC);
            find_max_degree(coef->npts_fit, &(coef->degree), &(fit->degree));
            }
        else if ((tmp->active > 1) && (tmp->active < 10))
            {
            tmp->active += 10;
            sprintf(mess, "Point reinstated:  %s",tmp->pt_id);
            t_write(mess, T_STDCC);
            coef->npts_fit++;
            find_max_degree(coef->npts_fit, &(coef->degree), &(fit->degree));
            }
        return;
        }
    t_write("Requested Tie point name does not exist", T_STDCC);
    }
}

/****************************************************************************
NAME:               Change_degree

PURPOSE:  Allows user to change degree of polynomial used to model tie points

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  1.0    3/82   R. Wenger     CSC      N/A    (Old name:  TIEPOINTS)
  4.0    5/85   S. Kondal     SAR      20-472 (Old name:  TIEFIT)
  5.0    1/89   D. Steinwand  CSB      LAS 5.0 C conversion

ALGORITHM DESCRIPTION:
    Find the maximum allowable degree for the number of tie points used
    Loop until satisfactory user input is given:
       Report to the user the maximum permissible degree & enter
       Get user's response
       If response is less than or equal to max_degree, return
*****************************************************************************/
void change_degree
(
    long npts,      /* Number of tie points used in modeling process */
    long *degree,       /* New degree of polynomial */
    long *max_degree   /* Maximum allowable degree of polynomial */
)
{
TEXT mess[80];      /* Message buffer for t_ calls */
CODE term;      /* Terminal code for t_ calls */

find_max_degree(npts, degree, max_degree);
for (;;)
    {
    sprintf(mess, "Permissible degrees (1-%d).  Enter degree:  ", *max_degree);
    t_write(mess, T_PROMPT);
    t_read(mess, &term);
    if (strncmp(mess,"1",1) == 0) *degree = 1;
    if (strncmp(mess,"2",1) == 0) *degree = 2;
    if (strncmp(mess,"3",1) == 0) *degree = 3;
    if (strncmp(mess,"4",1) == 0) *degree = 4;
    if (*degree <= *max_degree) return;
    }
}

/****************************************************************************
NAME:               find_max_degree

PURPOSE:  Finds maximum allowable degree for current number of tie points

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  1.0    3/82   R. Wenger     CSC      N/A    (Old name:  TIEPOINTS)
  4.0    5/85   S. Kondal     SAR      20-472 (Old name:  TIEFIT)
  5.0    1/89   D. Steinwand  CSB      LAS 5.0 C conversion

ALGORITHM DESCRIPTION:
    If current number of active points is less than 3, max degree = 0
    If current number of active points  > 2 and < 6, max degree = 1
    If current number of active points  > 5 and < 10, max degree = 2
    If current number of active points  > 9 and < 15, max degree = 3
    If current number of active points is greater than 14, max degree = 4
    If degree greater than max degree, degree = max degree
    Return
*****************************************************************************/
void find_max_degree
(
    long npts,      /* Number of tie points used in modeling process */
    long *degree,       /* Current degree of polynomial used */
    long *max_degree   /* Maximum degree of polynomial allowable */
)
{
if (npts < 3) *max_degree = 0;
if ((npts > 2) && (npts < 6)) *max_degree = 1;
if ((npts > 5) && (npts < 10)) *max_degree = 2;
if ((npts > 9) && (npts < 15)) *max_degree = 3;
if (npts > 14) *max_degree = 4;
if (*degree > *max_degree) *degree = *max_degree;

}

