/****************************************************************************
NAME:               POLYFIT

PURPOSE:  POLYFIT uses a Tie Point Location file containing tie point pairs
          to derive two bivariate polynomials using least squares regression
      analysis in a forward stepping procedure.  This defines the trans-
      formation from one coordinate system to another and then creates a 
      geometric mapping grid that approximates the transformation defined 
      by the polynomials.  The mapping grid and the transformation coef-
      ficients are then written to the Geometric Mapping Grid file.

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  1.0    3/82   R. Wenger     CSC      N/A    (Old name:  TIEPOINTS)
  4.0    5/85   S. Kondal     SAR      20-472 (Old name:  TIEFIT)
  5.0    1/89   D. Steinwand  CSB      LAS 5.0 C conversion
  5.1   10/93   D. Bridges    CSB      Added PRTOPT to add option of printing
                       information only at end of iterations
  5.2    1/95   T. Mittan     CSB      Modified to handle any number of
                       tie points 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
    Initialize LAS processing
    Allocate buffer space for tie points (allocate MAX_TIE_POINTS records)
    Get parameters from the user
    Read in tie point records from a tie point location file
    If generating a forward transformation, swap tie point location search
    and reference coordinate values
    Generate a polynomial model from the tie points
    Create the geometric mapping grid, reduce (if possible), & write to disk
    If generating a forward transformation, swap tie point location search
    and reference coordinate values
    Write out tie point location records (with updated active flags)
    Call complete & exit
*****************************************************************************/
#include "polyfit.h"

int main(void)
{
struct COEF coef;       /* Polynomial coefficient records */
struct PRINT print;     /* Print parameters */
struct FRAME frame;     /* Framing parameters */
struct FIT fit;         /* Coefficient modeling parameters */
struct TPLHEAD tie_head;    /* Tie point location file header record */
struct PARBLK vblk;     /* TAE parameter block */
struct RESIDUAL residual;   /* Residual errors */
struct TPLDATA *tie_data;   /* Pointer to Tie point location data records */
long msgtim;            /* Message timer variable */
long num_out;           /* Number of output files */
long std_out;           /* Dummy variable to standard output */
long STAT_FLAG;         /* Return status variable */
long prtopt;            /* Print option */
char tpl_name[CMLEN];       /* Input Tie Point Location file name */
char out_tpl[CMLEN];        /* Output Tie Point Location file name */
char outbuf[4][CMLEN];      /* Tae name of output files for complete */
char plotnm[CMLEN];     /* Plot file name */

c_inlas(&vblk, &std_out);

/* Allocate buffer space for tie point records
-------------------------------------------*/
tie_data = (struct TPLDATA *) malloc((sizeof(struct TPLDATA) * MAX_TIE_POINTS));
if (tie_data == NULL) pabort(NO_SPACE);

CHECK(getpar(&vblk,tpl_name,out_tpl,&frame,&fit,&print,&msgtim,plotnm,&prtopt));
CHECK(get_tie_points(tpl_name, &tie_head, &tie_data, &(coef.npts)));
if (fit.xform_mode == FORWARD) swap_tpl(tie_data, coef.npts);
CHECK(find_corners(&frame, &tie_head));
CHECK(model(tie_data, &fit, &print, &coef, &residual,plotnm,&prtopt));
CHECK(make_grid(&frame,&coef,&residual,&tie_head,fit.xform_mode,&print,msgtim));
if (fit.xform_mode == FORWARD) swap_tpl(tie_data, coef.npts);
CHECK(put_tie_points(out_tpl, &tie_head, tie_data, coef.npts));
strcpy(outbuf[0], frame.name);
strcpy(outbuf[1], out_tpl);
num_out = 2;
if (print.option != TERM)
    {
    strcpy(outbuf[num_out], print.host_name);
    num_out++ ;
    }
if (strlen(plotnm) != 0)
    {
    strcpy(outbuf[num_out], plotnm);
    num_out++;
    }
c_complt("POLYFIT", outbuf, &num_out);
return (0);
}

