#ifndef POLYFIT_H
#define POLYFIT_H

#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"

#define STAT_FLAG status
#define CHECK(ARG) status = ARG; if(status != E_SUCC) pabort(status)

/* Edit modes
  ----------*/
#define AUTO	100
#define MANUAL	200
#define NOEDIT	300

/* Direction of transformation--INVERSE is normally used
  -----------------------------------------------------*/
#define FORWARD 100
#define INVERSE 200

/* Report destinations
  -------------------*/
#define ALL	100
#define TERM	200
#define PFILE	300

/* Tie point edit actions
  ----------------------*/
#define EXCLUDE	700
#define INCLUDE 800

/* Error flag--more error flags in geompak.h 
  -----------------------------------------*/
#define QUIT_IN_MOD  777
#define NO_FRAME     778

/* Structure holding transformation coefficients and related parameters 
   & statistics
  ------------*/
struct COEF 
   {
   long degree;			/* Degree of polynomial used for fit */
   double x_coef[MAX_COEF];	/* Coefficients in x */
   double y_coef[MAX_COEF];	/* Coefficients in y */
   long npts_fit;		/* Number of points used in fit */
   long npts;			/* Number of points in tie point file used */
   long x_ncoef;		/* Number of coefficients in x */
   long y_ncoef;		/* Number of coefficients in y */
   long x_freedom;		/* Number of degrees of freedom in x */
   long y_freedom;		/* Number of degrees of freedom in y */
   };

/* Structure containing report destination options & related parameters 
  --------------------------------------------------------------------*/
struct PRINT 
   {
   long option;		/* Print option:  TERM only, FILE only, or ALL (both) */
   FILE *fptr;		/* Print file pointer (NULL if print file not used) */
   char name[CMLEN];	/* Print file name (NULL if print file not used) */
   char host_name[CMLEN];/* Host name of print file */
   };

/* Structure containing framing parameters & related gridding information.
   These are all user-entered parameters
  -------------------------------------*/
struct FRAME 
   {
   double tol;		/* Tolerance for linear approximation--grid reduce */
   double corners[8];	/* Resulting grid projection corners */
   int  window[4];	/* Window parameters for output space */
   int  ngline;		/* Number of geometric mapping grid lines (rows) */
   int  ngsamp;		/* Number of geometric mapping grid samples (cols) */
   char name[CMLEN];	/* Name of the Geometric Mapping Grid file */
   };

/* Structure containing user-entered parameters related to the fitting & 
   tie point editing process
  -------------------------*/
struct FIT 
   {
   double alpha[2];	/* Entry & exit text alpha values for regression */
   double max_res;	/* Maximum allowable residual value for auto edit */
   long degree;		/* Maximum allowable degree of polynomial */
   long edit_mode;	/* Edit mode:  MANUAL, AUTO or NOEDIT */
   long xform_mode;	/* Transformation mode:  FORWARD or INVERSE */
   };

/* Structure containing residual values and related statistics
  -----------------------------------------------------------*/
struct RESIDUAL
   {
   double *x_residual;	/* Pointer to array of residual values in x */
   double *y_residual;	/* Pointer to array of residual values in y */
   double *rms_residual;/* Pointer to array of rms values */
   double ave_rmse;	/* Average rms error */
   double x_ave_err;	/* Average residual in x */
   double y_ave_err;	/* Average residual in y */
   double x_max_err;	/* Maximum residual in x */
   double x_max_rms;	/* Maximum residual in x RMS error */
   double y_max_err;	/* Maximum residual in y */
   double y_max_rms;	/* Maximum residual in y RMS error */
   char x_max_ptid[CMLEN]; /* Point id of tie point with largest x residual */
   char y_max_ptid[CMLEN]; /* Point id of tie point with largest y residual */
   };


int calc_coef
(
    struct TPLDATA *tie_data,   /* Tie Point Location records */
    struct FIT *fit,            /* Structure containing fitting parameters */
    struct COEF *coef           /* Structure to contain coefficients & stats */
);

void calc_residuals
(
    struct TPLDATA *tp,         /* Tie Point Location data records */
    struct COEF *coef,          /* Coefficient & stats from model */
    struct RESIDUAL *residual   /* Residual errors from fit */
);

void do_plot
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct COEF *coef,          /* Coefficients & modelling stats */
    struct RESIDUAL *residual,  /* Fit residuals */
    char *plotnm                /* plot file name */
);

int find_corners
(
    struct FRAME *frame,        /* Framing parameters */
    struct TPLHEAD *header      /* Tie point location file header data */
);

int get_tie_points
(
    char *name,                 /* Name of the Tie Point Location file */
    struct TPLHEAD *header,     /* Tie Point Location header record */
    struct TPLDATA **data,      /* Tie Point Location data records */
    long *npts                  /* Number of tie point data records read */
);

int getpar
(
    struct PARBLK *vblk,    /* TAE variable block */
    char *tpl_name,         /* Input tie point location file name */
    char *out_tpl,          /* Output tie point location file name */
    struct FRAME *frame,    /* Output framing parameters */
    struct FIT *fit,        /* Model fit parameters */
    struct PRINT *print,    /* Print/report parameters */
    long *msgtim,           /* Message timer */
    char *plotnm,           /* plot file name */
    long *prtopt            /* Print option */
);

int make_grid
(
    struct FRAME *frame,        /* Output framing parameters */
    struct COEF *coef,          /* Mapping model coefficients */
    struct RESIDUAL *residual,  /* Residuals of fit */
    struct TPLHEAD *header,     /* Header data from tie point location file */
    long xform,                 /* Transformation mode--forward or inverse */
    struct PRINT *print,        /* Print file parameters */
    long msgtim                 /* Message timing parameter */
);

int model
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,            /* Polynomial fit parameters */
    struct PRINT *print,        /* Print parameters */
    struct COEF *coef,          /* Results of model--coefficients & stats */
    struct RESIDUAL *residual,  /* Residuals from fit */
    char *plotnm,               /* plot file name */
    long *prtopt                /* print option */
);

int modify
(
    struct RESIDUAL *residual,  /* Structure containing residual values */
    struct TPLDATA *tie_data,   /* Structure containing tie point records */
    struct FIT *fit,            /* Structure containing fit parameters */
    struct COEF *coef           /* Structure containing xform coef's & stats */
);

int auto_edit
(
    struct RESIDUAL *residual,  /* Residual errors for each tie point */
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,            /* User entered fitting parameters */
    struct COEF *coef           /* Coefficient & statistics records */
);

int poly_opt
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,            /* Fit parameters as entered by user */
    struct COEF *coef           /* Transformation coefficients & statistics */
);

void mod_pt
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct FIT *fit,            /* User entered fit parameters */
    struct COEF *coef,          /* Transformation coefficients & statistics */
    long action                 /* Edit action--INCLUDE or EXCLUDE */
);

void change_degree
(
    long npts,          /* Number of tie points used in modeling process */
    long *degree,       /* New degree of polynomial */
    long *max_degree    /* Maximum allowable degree of polynomial */
);

void find_max_degree
(
    long npts,          /* Number of tie points used in modeling process */
    long *degree,       /* Current degree of polynomial used */
    long *max_degree    /* Maximum degree of polynomial allowable */
);

void pabort
(
    long status            /* Error condition flag */
);

void print_term
(
    char *mess,     /* Message to be printed to terminal */
    FILE *ptr       /* Pointer to output file -- not used here! */
);

void print_file
(
    char *mess,     /* Message to be printed to file */
    FILE *ptr       /* Pointer to output file */
);

void print_both
(
    char *mess,     /* Message to be printed to terminal and to file */
    FILE *ptr       /* Pointer to output file */
);

int put_tie_points
(
    char *name,                 /* Name of the Tie Point Location file */
    struct TPLHEAD *header,     /* Tie Point Location header record */
    struct TPLDATA *data,       /* Tie Point Location data records */
    long npts                   /* Number of tie point data records read */
);

void report
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct COEF *coef,          /* Coefficients & modelling stats */
    struct RESIDUAL *residual,  /* Fit residuals */
    long xform,                 /* Transformation mode */
    struct PRINT *print         /* Print/report parameters */
);

void swap_tpl
(
    struct TPLDATA *tie_data,   /* Tie point records */
    long npts                   /* Number of tie point records */
);

#endif
