/****************************************************************************
NAME:               report

PURPOSE:  Reports results of modeling process to the user

ALGORITHM DESCRIPTION:
    Initialize the terms-of-polynomial constant
    Set up pointer-to-function for report/print destination
    Inform the user the mode of transformation (forward or inverse)
    Inform the user the degree of polynomial used for this iteration
    Report to the user the coefficients obtained from the modeling process
    Report to the user the number of tie points used in the fit
    Report the degrees of freedom in X and in Y
    Report to the user the input & output line,sample pairs, residual
       values, RES errors, 7 characters of the point id, and active flag
    Print the average line residual
    Print the average sample residual
    Print the average RMS error
    Print the maximum residual in Y, that points RES error, & point id
    Print the maximum residual in X, that points RES error, & point id
    Return
*****************************************************************************/
#include "polyfit.h"

void report
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct COEF *coef,      /* Coefficients & modelling stats */
    struct RESIDUAL *residual,  /* Fit residuals */
    long xform,         /* Transformation mode */
    struct PRINT *print        /* Print/report parameters */
)
{
struct TPLDATA *tp; /* Temporary pointer to tie point data records */
long top;       /* Number of coefficients */
long count;     /* Loop counter */
long act;       /* Temporary variable for active flag */
char mess[135];     /* Buffer for messages given to c_pterm */
char mess2[30];     /* Buffer for messages given to c_pterm */
char term[15][20];  /* Character constants:  terms of polynomial */

void (*printit)(char*,FILE*);  /* Pointer to print function */

if (print->option != TERM)
    fprintf(print->fptr,"\f");
else
    print->fptr = NULL;

/* Initialize the "terms of polynomial" contsant
---------------------------------------------*/
strcpy(&(term[0][0]), "Constant");
strcpy(&(term[1][0]), "X");
strcpy(&(term[2][0]), "Y");
strcpy(&(term[3][0]), "X**2");
strcpy(&(term[4][0]), "XY");
strcpy(&(term[5][0]), "Y**2");
strcpy(&(term[6][0]), "X**3");
strcpy(&(term[7][0]), "(X**2)Y");
strcpy(&(term[8][0]), "X(Y**2)");
strcpy(&(term[9][0]), "Y**3");
strcpy(&(term[10][0]), "X**4");
strcpy(&(term[11][0]), "(X**3)Y");
strcpy(&(term[12][0]), "(X**2)(Y**2)");
strcpy(&(term[13][0]), "X(Y**3)");
strcpy(&(term[14][0]), "Y**4");

/* Determine report output destination and set up a pointer to the appropriate
function
--------*/
if (print->option == TERM) printit = print_term;
if (print->option == PFILE) printit = print_file;
if (print->option == ALL) printit = print_both;

(*printit)(" ", print->fptr);
if (xform == INVERSE) sprintf(mess,
    "INVERSE Transformation -- Coefficients map output to input space");
if (xform == FORWARD) sprintf(mess,
    "FORWARD Transformation -- Coefficients map input to output space");
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
sprintf(mess, "Maximum permitted degree for approximation:  %d", coef->degree);
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
sprintf(mess, "Coefficients:");
(*printit)(mess, print->fptr);
top = (long)((coef->degree + 1.0) * (coef->degree + 2.0) / 2.0);
sprintf(mess, "  F(X) = A(0)");
for (count = 1; count < top; count++)
    {
    if (coef->x_coef[count] != 0.0)
        {
        sprintf(mess2," + A(%d)%s",count, &(term[count][0]));
        strcat(mess, mess2);
        }
    if (strlen(mess) > 65)
        {
        (*printit)(mess, print->fptr);
        strcpy(mess, "        ");
        }
    }
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
sprintf(mess, "  F(Y) = B(0)");
for (count = 1; count < top; count++)
    {
    if (coef->y_coef[count] != 0.0)
        {
        sprintf(mess2," + B(%d)%s",count, &(term[count][0]));
        strcat(mess, mess2);
        }
    if (strlen(mess) > 65)
        {
        (*printit)(mess, print->fptr);
        strcpy(mess, "        ");
        }
    }
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
sprintf(mess, "   n        A(n)                 B(n)               term");
(*printit)(mess, print->fptr);

for (count = 0; count < top; count++)
    {
    if ((coef->x_coef[count] != 0.0) && (coef->y_coef[count] != 0.0))
        {
        sprintf(mess,"  %2d  %20.12e %20.12e      %s",count,coef->x_coef[count],
        coef->y_coef[count],&(term[count][0]));
        (*printit)(mess, print->fptr);
        }
    if ((coef->x_coef[count] != 0.0) && (coef->y_coef[count] == 0.0))
        {
        sprintf(mess, "  %2d  %20.12e                           %s",count,
        coef->x_coef[count], &(term[count][0]));
        (*printit)(mess, print->fptr);
        }
    if ((coef->x_coef[count] == 0.0) && (coef->y_coef[count] != 0.0))
        {
        sprintf(mess, "  %2d                       %20.12e      %s",count,
        coef->y_coef[count],&(term[count][0]));
        (*printit)(mess, print->fptr);
        }
    }
(*printit)(" ", print->fptr);
sprintf(mess, "Number of points used in fit:  %d",coef->npts_fit);
(*printit)(mess, print->fptr);
sprintf(mess, "Degrees of freedom in x:  %d",coef->x_freedom);
(*printit)(mess, print->fptr);
sprintf(mess, "Degrees of freedom in y:  %d",coef->y_freedom);
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
(*printit)(" ", print->fptr);

if (print->option != TERM)
    {
    fprintf(print->fptr, "Residual errors are given in pixels\n\n");
    fprintf(print->fptr, "    Output Space                                              Input Space          Residuals      Residual Code Point\n");
    fprintf(print->fptr, "   Line     Sample      Lat/Y       Long/X      Elevation   Line     Sample     Line     Sample   Magnitude     Id\n");
    fprintf(print->fptr, "--------- --------- --------------------------- --------- --------- --------- --------- --------- --------- -- ------\n");
    }
if (print->option != PFILE)
    {
    print_term ("Residual errors are given in pixels", print->fptr);
    print_term (" ", print->fptr);
    sprintf(mess, 
"    Output Space         Input Space          Residuals     Residual Code Point ");
    print_term(mess, print->fptr);
    sprintf(mess, "   Line     Sample     Line     Sample     Line     Sample  Magnitude     Id");
    print_term(mess, print->fptr);
    sprintf(mess, 
"--------- --------- --------- --------- --------- --------- --------- -- ------");
    print_term(mess, print->fptr);
    }
for (tp = tie_data, count = 0; count < coef->npts; count++, tp++)
    {
    act = tp->active;
    if ((act == 1) || (act == 0) || (act > 9))
        {
        if (print->option != TERM)
            {
            fprintf(print->fptr,
     "%9.2f %9.2f %13.3f %13.3f %9.2f %9.2f %9.2f %9.3f %9.3f %9.3f %2d %-7s\n",
            tp->ref_coord[0], tp->ref_coord[1], tp->geo_coord[0],
            tp->geo_coord[1], tp->elevation, tp->sea_coord[0],
            tp->sea_coord[1], residual->y_residual[count],
            residual->x_residual[count], residual->rms_residual[count],
            tp->active, tp->pt_id);
            }
        if (print->option != PFILE)
            {
            sprintf(mess, "%9.2f %9.2f %9.2f %9.2f %9.3f %9.3f %9.3f %2d %-7s",
            tp->ref_coord[0], tp->ref_coord[1], tp->sea_coord[0],
            tp->sea_coord[1], residual->y_residual[count],
            residual->x_residual[count], residual->rms_residual[count],
            tp->active, tp->pt_id);
            print_term(mess, print->fptr);
            }
        }
    else
        {
        if (print->option != TERM)
            {
            fprintf(print->fptr, 
"%9.2f %9.2f %13.3f %13.3f %9.2f %9.2f %9.2f     *** Not Included ***      %2d %-7s\n",
            tp->ref_coord[0], tp->ref_coord[1], tp->geo_coord[0],
            tp->geo_coord[1], tp->elevation, tp->sea_coord[0],
            tp->sea_coord[1], tp->active, tp->pt_id);
            }
        if (print->option != PFILE)
            {
            sprintf(mess,
            "%9.2f %9.2f %9.2f %9.2f     *** Not Included ***      %2d %-7s",
            tp->ref_coord[0], tp->ref_coord[1], tp->sea_coord[0],
            tp->sea_coord[1], tp->active, tp->pt_id);
            print_term(mess, print->fptr);
            }
        }
    }
(*printit)(" ", print->fptr);
sprintf(mess, "Average Line Residual:    %lf", residual->y_ave_err);
(*printit)(mess, print->fptr);
sprintf(mess, "        Sample Residual:  %lf", residual->x_ave_err);
(*printit)(mess, print->fptr);
sprintf(mess, "        RMSE:             %lf", residual->ave_rmse);
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
sprintf(mess, "Max Line Residual:    %lf Res Error:  %lf Point id:  %s",
residual->y_max_err, residual->y_max_rms, residual->y_max_ptid);
(*printit)(mess, print->fptr);
sprintf(mess, "    Sample Residual:  %lf Res Error:  %lf Point id:  %s",
residual->x_max_err, residual->x_max_rms, residual->x_max_ptid);
(*printit)(mess, print->fptr);
(*printit)(" ", print->fptr);
(*printit)(" ", print->fptr);

}

