/***************************************************************************
 PROGRAM                              PROCHECK


FUNCTION:
	 PROCHECK performs a syntax check on specified PDF files.	

PROGRAM HISTORY:
  Version	Date       Author      	Request
  -------	----	   ------       -------
    4.0         8/87       D. Steinwand  NewLAS development
    5.0        12/88       R. McKinney   Conversion to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS 5.0

ALGORITHM (FLOW):

   This program modifies a Procedure Proc so that its syntax can be checked
   without executing any procs called in the procedure.  To do this, PROCHECK
   copies the procedure from STDIN to STDOUT.  In this process of filtering,
   PROCHECK comments out certain lines in the procedure which should not be
   executed when checking the syntax (such as DCL lines or TUTOR commands).
   PROCHECK also inserts the command qualifier "runtype=norun" for all procs
   called from the procedure (but not for any TCL commands as this would
   generate an error).  PROCHECK also inserts the line 'let _onfail="continue"'
   before every proc initiation.  This insures that when the proc generated
   by PROCHECK is checked, all lines will be checked.  If this line were not
   included, it is very possible that the first error encountered would stop
   all further syntax checks.
****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#ifdef vms
#include <types.h>
#endif
/*
   The heart of PROCHECK is determining whether a line contains a TCL command
   or is a proc initiation.  The structure TCL_CMNDS contains all TCL commands
   which can possibly appear in a procedure proc.  Each entry contains three
   fields:  the first field is the name in upper case; the second, the length
   of the name (this speeds up processing by the function INTRINSIC); the
   third, whether a line containing this TCL command is to be commented out.
*/
#define		YES		1
#define		NO		0
#define		MAX_TCL_LENGTH	10	/* longest TCL cmnd is 1 less */
#define		TCL_CARD	49	/* cardinality of set of TCL cmnds */
	char	tcl_cmnds_ndx[MAX_TCL_LENGTH] = {
			TCL_CARD, TCL_CARD, 0, 1, 6, 20, 27, 41, 47, 48
						};
	struct	{
		char	command[10];
		char	size;
		char	comment_out;
		}	tcl_cmnds[TCL_CARD] =	{
				{"IF",		2,	NO},
				{"DCL",		3,	YES},
				{"END",		3,	NO},
				{"FOR",		3,	NO},
				{"LET",		3,	NO},
				{"USH",		3,	YES},
				{"BODY",	4,	NO},
				{"ELSE",	4,	NO},
				{"EMIT",	4,	YES},
				{"EXIT",	4,	YES},
				{"GOTO",	4,	NO},
				{"HELP",	4,	YES},
				{"LOOP",	4,	NO},
				{"MENU",	4,	YES},
				{"NEXT",	4,	NO},
				{"PARM",	4,	NO},
				{"SAVE",	4,	YES},
				{"SHOW",	4,	YES},
				{"STOP",	4,	YES},
				{"WAIT",	4,	YES},
				{"ABORT",	5,	YES},
				{"BATCH",	5,	YES},
				{"BREAK",	5,	NO},
				{"LOCAL",	5,	NO},
				{"REPLY",	5,	YES},
				{"TUTOR",	5,	YES},
				{"WRITE",	5,	YES},
				{"DEFCMD",	6,	NO},
				{"DELCMD",	6,	NO},
				{"DELETE",	6,	NO},
				{"ENABLE",	6,	YES},
				{"GETPAR",	6,	NO},
				{"GLOBAL",	6,	NO},
				{"LOGOFF",	6,	YES},
				{"PUTMSG",	6,	YES},
				{"REFGBL",	6,	NO},
				{"REMOVE",	6,	YES},
				{"RETURN",	6,	NO},
				{"SETLIB",	6,	NO},
				{"SUBCMD",	6,	NO},
				{"SWITCH",	6,	YES},
				{"COMPILE",	7,	NO},
				{"DISABLE",	7,	YES},
				{"DISPLAY",	7,	NO},
				{"PARMSET",	7,	NO},
				{"PROCESS",	7,	NO},
				{"RESTORE",	7,	NO},
				{"CONTINUE",	8,	NO},
				{"PROCEDURE",	9,	NO}	};
		
#define		line_sz		101	/* size of line storage */
					/* changed from 201 1/15/86 */
					/* to fix problem PA's had */
					/* with long (several continuations) */
					/* command lines. PJM */
#define		word_sz		41	/* size of word storage */
	/* Macro for Setting Continuation Line Flag:
	      The flag CMNT_OUT indicates whether the current line is to
	      be commented out.  Lines are to be commented out if they
	      are the first line in a TCL command that is to be commented
	      out (see the body of this program) or if they are a continuation
	      of a line which is to be commented out.  The flag CONTIN indi-
	      cates whether the next line is a continuation of the current
	      line. CONTIN is set if the last non-blank character on this
	      is a '+'.
	*/
#define		set_cont_cmnt	cmnt_out = (contin && cmnt_out); \
				contin = (*last_char == '+')
	/* Macro for Setting the Body and Header Flags
	      The BODY and HEADER flags are set when the first line in each
	      section is processed.  HEADER is set when PROCHECK begins
	      execution.  It is unset when PROCHECK processes the line
	      containing "body."  When HEADER is unset, BODY is set.  BODY
	      in unset when PROCHECK processes the line containing "end-proc."
	*/
#define		set_flag(flag,wordp)	\
		    { int len;	\
		      len = strlen(wordp); \
		      if((word_length==len)&&(0==strncmp(word,wordp,len))) \
			  flag = 0; }
				

main ()
{
	struct PARBLK parblk;		/*  TAE parmater block		*/
	struct VARIABLE *var;		/*  pointer to variable in pblock*/
	struct VARIABLE *p_find();	/*  function to find variable	*/

	FILE *fp;
	FILE *outfp;

	long std_out;			/* standard out			*/
	long status;			/* status of called functions	*/
        long access,			/* file access */
	     fatal = -1,		/* fatal error */
	     zero = 0;			/* constant */
          
	register long	line_length,	/* length of line read in */
			word_length;	/* length of first word on line */

	short	header,		/* flag for in header */
		body,		/* flag for in body */
		cmnt_out,	/* comment out line */
		contin;		/* continuation line flag */

	register int	tbl_pos;	/* return from intrinsic--position
					   in TCL commands table */

	char	line[line_sz],	/* line from stdin */
		tline[line_sz],	/* line for inserting runtype */
		word[word_sz],	/* first word on line */
		*fgets();	/* get string from stdout */

	register char	*first_word,	/* pointer to first non-blank in line.
					   either an intrinsic or a proc */
			*last_char,	/* pointer to last non-blank.  used
					   to check for a continuation line */
			*next_blank;	/* pointer to first blank after first
					   non-blank */
	char hname[100];		/* host input name		*/
	char oname[100];		/* host output name		*/

	/* get a line and the first word on that line,
	   while that word is not "body" write the line and any continuations,
	      repeat.
	*/
	header = 1;			/* in header */
	status = c_inlas(&parblk,&std_out);
        access = IREAD;
        status = c_getnam(&parblk,"INFILE",&zero,hname,"pdf",&access);
        if (status != E_SUCC)
           c_errmsg("Fatal error encountered","procheck-fatal",&fatal);
	fp = fopen(hname,"r");
	
	var = p_find(&parblk,"OUTFILE");
	strcpy(oname,SVAL(*var,0));
	outfp = fopen(oname,"w");

	do
	    {
	    if ( fgets(line,132,fp) == NULL )	/* bad proc */
		{ fprintf(outfp,"No body in proc.\n"); exit(-1); }
	    else
		{
		line_length = strlen(line);
		first_word = line + strspn(line," \t\n");
		if ( first_word == line+line_length )	/* blank line */
		    {	}
		else					/* non-blank line */
		    {
		    for (next_blank=first_word;	/* 1st space after 1st word */
			 (!isspace(*next_blank)) && (*next_blank!='\0');
			 next_blank++)
			;
		    word_length = next_blank - first_word;
		    strncpy_upper( word, first_word, word_length );
		    for (last_char=line+line_length-1; /* last non-space */
			 isspace(*last_char);
			 last_char--)
			;
		    set_flag(header,"BODY");
		    }
		fprintf(outfp,"%s\n",line);
		}
	    }
	while ( header );

	/* do
	      get a line and the first work on that line,
	      if the word is an intrinsic then write and any continuations,
	      otherwise write the word, "|runtype=norun|", and continuations,
	   while the first word is not "end-proc"
	*/
	body = 1;			/* in body */
	cmnt_out = NO;			/* don't comment out first line */
	contin = 0;			/* not continuing any lines */
	do
	    {
	    if ( fgets(line,132,fp) == NULL )	/* bad proc */
		{ fprintf(outfp,"No body in proc.\n"); exit(-1); }
	    else
		{
		line_length = strlen(line);
		first_word = line + strspn(line," \t\n");
		if ( first_word == line+line_length )	/* blank line */
		    { contin = 0; }
		else if ( *first_word == '!' )		/* comment line */
		    {
		    contin = 0;
		    cmnt_out = 0;
		    }
		else					/* non-blank line */
		    {
		    for (next_blank=first_word;	/* 1st space after 1st word */
			 (!isspace(*next_blank)) && (*next_blank!='\0');
			 next_blank++)
			;
		    word_length = next_blank - first_word;
		    strncpy_upper( word, first_word, word_length );
		    for (last_char=line+line_length-1; /* last non-space */
			 isspace(*last_char);
			 last_char--)
			;
		    if ( contin )		/* continuation, write as is */
			{ set_cont_cmnt; }
		    else
			{
			if ( TCL_CARD > (tbl_pos=intrinsic(word,word_length)))
			    {			/* TCL intrinsic */
			    set_flag(body,"END-PROC");
			    contin = (*last_char == '+');
			    cmnt_out = tcl_cmnds[tbl_pos].comment_out;
			    }
			else		/* proc:  insert "|runtype=norun|" */
			    {
			    fprintf(outfp,"\nlet _onfail = \"continue\"\n");
			    set_cont_cmnt;
			    strncpy(tline,line,next_blank-line);
			    *(tline+(next_blank-line)) = '\0';
			    strcat(tline," |runtype=norun|");
			    strcat(tline,next_blank);
			    strcpy(line,tline);
			    }
			}
		    }
		if ( cmnt_out )
		    { fprintf(outfp,"%c%s\n",'!',line); }
		else
		    { fprintf(outfp,"%s\n",line); }
		}
	    }
	while ( body );
	fclose(fp);
	fclose(outfp);
#ifdef vms
	exit (1);
#else
	exit (0);
#endif
}

/*
   Function STRNCPY_UPPER copies a string and converts it to uppercase
   at the same time.
*/
strncpy_upper( dest, source, length )
	register char	*dest, *source;
	register long	length;
{
	for( ; length>0; length--, dest++, source++)
  	    {
  	      if(islower(*source))
  		{
  		*dest = toupper(*source); 
  		}
  	      else
  		*dest = *source;
  	     }
	if ( *dest != '\0' ) *dest = '\0';
	return;
}

/*
   Function INTRINSIC determines if a word is a TCL command.
*/
intrinsic( word, length )
	register char	*word;
	register long	length;
{
		char	cmnd[word_sz];	/* command minus subcommand */
	register int	index;		/* index into table of commands */
	register short	dash;		/* index of '-' in word */
	register int	same;		/* result of comparing target command
					   with candidate command */

	/* Some TCL commands have subcommands so check only "main" cmnd */
	strncpy(cmnd,word, dash=strcspn(word,"-") );
	if ( dash < MAX_TCL_LENGTH )
	    {
	    /* Search through table of TCL commands until the size changes,
	       the string is found, or the table is exhausted (only happens
	       on longest command. */
	    for( index=tcl_cmnds_ndx[dash];
		 (dash=tcl_cmnds[index].size)
		 && (0!=(same=strncmp(cmnd,tcl_cmnds[index].command,dash)))
		 && (index<TCL_CARD);
		 index++ )
		;
	    if ( 0 != same ) { index = TCL_CARD; }	/* not same--not TCL */
	    }
	else				/* word too long--not TCL */
	    { index = TCL_CARD; }
	return index;
}
