
/***************************************************************************
 PROGRAM                              PROCLIB


FUNCTION:
   PROCLIB finds all the program calls in a specified PDF file and
   lists the library in which each program was found.

PROGRAM HISTORY:
  Version	Date       Author      	Request
  -------	----	   ------       -------
    1.0        03/94       J. LaVergne   Initial development - converted from 
                                         the PROCHECK program
    1.0        06/94       D. Eichacker  Initial development - finished
					 conversion. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE


ALGORITHM (FLOW):

   This program reads a Procedure Proc so its calls to other procs can
   be traced. PROCLIB comments out certain lines in the procedure which 
   are irrelevent (such as ush lines, TUTOR commands, and TAE commands).
   The resulting list of procedure calls (with any subcommands removed)
   is sent to the PROCLIB PDF for further processing.
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"
/*
   The heart of PROCLIB is determining whether a line contains a TCL command
   or is a proc initiation.  The structure TCL_CMNDS contains all TCL commands
   which can possibly appear in a procedure proc.  Each entry contains three
   fields:  the first field is the name in upper case; the second, the length
   of the name (this speeds up processing by the function INTRINSIC); the
   third, whether a line containing this TCL command is to be commented out.
*/
#define		YES		1
#define		NO		0
#define		MAX_TCL_LENGTH	10	/* longest TCL cmnd is 1 less */
#define		TCL_CARD	49	/* cardinality of set of TCL cmnds */
	char	tcl_cmnds_ndx[MAX_TCL_LENGTH] = {
			TCL_CARD, TCL_CARD, 0, 1, 6, 20, 27, 41, 47, 48
						};
	struct	{
		char	command[10];
		char	size;
		char	comment_out;
		}	tcl_cmnds[TCL_CARD] =	{
				{"IF",		2,	NO},
				{"DCL",		3,	YES},
				{"END",		3,	NO},
				{"FOR",		3,	NO},
				{"LET",		3,	NO},
				{"USH",		3,	YES},
				{"BODY",	4,	NO},
				{"ELSE",	4,	NO},
				{"EMIT",	4,	YES},
				{"EXIT",	4,	YES},
				{"GOTO",	4,	NO},
				{"HELP",	4,	YES},
				{"LOOP",	4,	NO},
				{"MENU",	4,	YES},
				{"NEXT",	4,	NO},
				{"PARM",	4,	NO},
				{"SAVE",	4,	YES},
				{"SHOW",	4,	YES},
				{"STOP",	4,	YES},
				{"WAIT",	4,	YES},
				{"ABORT",	5,	YES},
				{"BATCH",	5,	YES},
				{"BREAK",	5,	NO},
				{"LOCAL",	5,	NO},
				{"REPLY",	5,	YES},
				{"TUTOR",	5,	YES},
				{"WRITE",	5,	YES},
				{"DEFCMD",	6,	NO},
				{"DELCMD",	6,	NO},
				{"DELETE",	6,	NO},
				{"ENABLE",	6,	YES},
				{"GETPAR",	6,	NO},
				{"GLOBAL",	6,	NO},
				{"LOGOFF",	6,	YES},
				{"PUTMSG",	6,	YES},
				{"REFGBL",	6,	NO},
				{"REMOVE",	6,	YES},
				{"RETURN",	6,	NO},
				{"SETLIB",	6,	NO},
				{"SUBCMD",	6,	NO},
				{"SWITCH",	6,	YES},
				{"COMPILE",	7,	NO},
				{"DISABLE",	7,	YES},
				{"DISPLAY",	7,	NO},
				{"PARMSET",	7,	NO},
				{"PROCESS",	7,	NO},
				{"RESTORE",	7,	NO},
				{"CONTINUE",	8,	NO},
				{"PROCEDURE",	9,	NO}	};
		
#define		line_sz		101	/* size of line storage */
					/* changed from 201 1/15/86 */
					/* to fix problem PA's had */
					/* with long (several continuations) */
					/* command lines. PJM */
#define		word_sz		41	/* size of word storage */
	/* Macro for Setting Continuation Line Flag:
	      The flag CMNT_OUT indicates whether the current line is to
	      be commented out.  Lines are to be commented out if they
	      are the first line in a TCL command that is to be commented
	      out (see the body of this program) or if they are a continuation
	      of a line which is to be commented out.  The flag CONTIN indi-
	      cates whether the next line is a continuation of the current
	      line. CONTIN is set if the last non-blank character on this
	      is a '+'.
	*/
#define		set_cont_cmnt	cmnt_out = (contin && cmnt_out); \
				contin = (*last_char == '+')
	/* Macro for Setting the Body and Header Flags
	      The PROC and HEADER flags are set when the first line in each
	      section is processed.  HEADER is set when PROCLIB begins
	      execution.  It is unset when PROCLIB processes the line
	      containing "procedure."  When HEADER is unset, PROC is set. PROC 
	      in unset when PROCLIB processes the last "end-proc."
	*/
#define		flag_up(flag,wordp)	\
		    { long len;	\
		      len = (long) strlen(wordp); \
		      if((word_length==len)&&(0==strncmp(word,wordp,len))) \
			  flag++; }
#define		flag_down(flag,wordp)	\
		    { long len;	\
		      len = (long) strlen(wordp); \
		      if((word_length==len)&&(0==strncmp(word,wordp,len))) \
			  flag--; }
				
int main
(void)
{
	struct PARBLK parblk;		/*  TAE parameter block		*/
	struct VARIABLE *p_find();	/*  function to find variable	*/

	FILE *fp;

	char	*first_word,	/* pointer to first non-blank in line.
					   either an intrinsic or a proc */
			*last_char,	/* pointer to last non-blank.  used
					   to check for a continuation line */
			*next_blank;	/* pointer to first blank after first
					   non-blank */

	long	line_length,	/* length of line read in */
			tbl_pos,	/* return from intrinsic--position
					   in TCL commands table */
			word_length;	/* length of first word on line */

        short	add_wd=YES,	/* For adding a word to pglist  */
		cmnt_out,	/* comment out line */
		contin,		/* continuation line flag */
		header,		/* flag for in header */
		lp_var,		/* loop variable */
		proc;		/* flag for in procedure*/

        long 	access,			/* file access */
	     	fatal = -1,		/* fatal error */
        	jj,			/* temporary index variable */
	     	nonfatal = 1,		/* nonfatal error */
		pgcnt=0,		/* number of called programs */
		srt=0,			/* Start postion for duplicates */
		status,			/* status of called functions */
		std_out,		/* standard out	*/
	     	zero = 0;		/* constant */
         
char		hname[100],	/* host input name		*/
        	*ii,            /* temporary index variable */
		line[line_sz],	/* line from stdin */
  		*lpt,		/* pointer to char in LINE */
		*pglist[256],	/* called programs list	*/
		tline[line_sz],	/* line for inserting runtype */
                word[word_sz],  /* first word on line */
		word2[word_sz];	/* first word on line */


	/* get a line and the first word on that line,
	   while that word is not "procedure", just skip over it
	*/
	header = 1;			/* in header */
	c_inlas(&parblk,&std_out);
        access = IREAD;


        status = c_getnam(&parblk,"INFILE",&zero,hname,"pdf",&access);
        if (status != E_SUCC) 
           c_errmsg("Fatal error encountered","prolib-fatal",&fatal); 
	fp = fopen(hname,"r");
	

	do
	    {
	    if ( fgets(line,132,fp) == NULL )	/* bad proc */
		c_errmsg("No body in procedure","proclib-body",&fatal);
	    else
		{
		line_length = (long) strlen(line);
		first_word = line + strspn(line," \t\n");
		if ( first_word == line+line_length )	/* blank line */
		    {	}
		else					/* non-blank line */
		    {
		    for (next_blank=first_word;	/* 1st space after 1st word */
			 (!isspace(*next_blank)) && (*next_blank!='\0');
			 next_blank++)
			;
		    word_length = next_blank - first_word;
		    strncpy_upper( word, first_word, word_length );
		    for (last_char=line+line_length-1; /* last non-space */
			 isspace(*last_char);
			 last_char--)
			;
		    flag_down(header,"PROCEDURE");
		    }

		}
	    }
	while ( header );

	/* 
	      Get a line and the first word on that line. While the first 
              word is not "end-proc", see if the line is a program call.
              If so, remove any subcommands it may have, and send the
              program name back to PROCLIB.PDF.
	*/
	proc = 1;			/* in procedure */
	cmnt_out = NO;			/* don't comment out first line */
	contin = 0;			/* not continuing any lines */
	do
	    {
	    if ( fgets(line,132,fp) == NULL )	/* bad proc */
		c_errmsg("No body in procedure","proclib-body",&fatal);
	    else
		{
                line_length = (long) strlen(line);
		first_word = line + strspn(line," \t\n");
                if ( first_word == line+line_length )	/* blank line */
		    { contin = 0; }
		else if ( *first_word == '!' )		/* comment line */
		    {
		    contin = 0;
		    cmnt_out = 0;
		    }
		else if ( *first_word == '&' ) /* function stored in variable */
		    {
		    contin = 0;
		    cmnt_out = 0;
		    }
		else					/* non-blank line */
		    {
		    for (next_blank=first_word;	/* 1st space after 1st word */
			 (!isspace(*next_blank)) && (*next_blank!='\0');
			 next_blank++)
			;
		    word_length = next_blank - first_word;
		    strncpy_upper( word, first_word, word_length );
		    for (last_char=line+line_length-1; /* last non-space */
			 isspace(*last_char);
			 last_char--)
			;

		    /* Ignore the line if it is a label */
                    if ( *last_char == '>' ){
		          contin = YES;
		    }
		    /* remove comments after + */
		    lpt = strrchr(line,'+');
		    if (lpt++ != NULL){
		       while (isspace(*lpt)){
			   ++lpt;
			   if (*lpt=='!'){
		              last_char  = strrchr(line,'+');
			      last_char[1] = '\n';
			      last_char[2] = '\0';
			      line_length = (long) strlen(line);
			   }
		       }	
                    }
 
	     	    if ( contin )		/* continuation, write as is */
			{ set_cont_cmnt; }
		    else
			{
			if ( TCL_CARD > (tbl_pos=intrinsic(word))){
						/* TCL intrinsic */
			    if (0==strcmp(word,"SETLIB")){
			       pglist[pgcnt]=(char *) malloc(line_sz);
			       for (lp_var=0; lp_var<line_length-1; ++lp_var)
				   pglist[pgcnt][lp_var]=line[lp_var]; 
			       pgcnt++;
			       srt=pgcnt;
			    }
			    else{
			      flag_up(proc,"PROCESS");
			      flag_up(proc,"PROCEDURE");
			      flag_down(proc,"END-PROC");
			      contin = (*last_char == '+');
			      cmnt_out = tcl_cmnds[tbl_pos].comment_out;
			    }
			}
			else		/* proc */
			    {
			    set_cont_cmnt;
			    strncpy(tline,line,next_blank-line);
			    *(tline+(next_blank-line)) = '\0';
			    strcat(tline,next_blank);
			    strcpy(line,tline);

                            /* Remove any subcommands, and put back into */
                            /* lower case for the calls to GETLIB        */
                            ii = strchr(word,'-');
                            if (ii != NULL)
                                {
                                strncpy(word2,word,ii-word);
                                word2[ii-word] = '\0';
                                jj = (long) strlen(word2);
                                c_up2low(word2,&jj);
				/* 
				   Check to see if word2 is in pglist
				   after last SETLIB, if not add it
				*/
				add_wd=YES;
                              for (lp_var=(short)srt; lp_var < pgcnt; lp_var++)
				   {
				   if (strcmp(pglist[lp_var],word2)== 0)
				       add_wd=NO;
                                   }
				if (add_wd==YES)
				   {
				   pglist[pgcnt] = (char *) malloc(word_sz);
				   for (lp_var=0; lp_var<=jj; ++lp_var)
			              pglist[pgcnt][lp_var]=word2[lp_var];
				   ++pgcnt;
                                   }
				}
                             else
                                {
                                jj = (long) strlen(word);
                                c_up2low(word,&jj);
				/*
				   Check to see if word is in pglist
				   after last SETLIB, if not add it
				*/
				add_wd=YES;
    			        for (lp_var= (short) srt; lp_var < pgcnt; lp_var++)
				   {
				   if (strcmp(pglist[lp_var],word)== 0)
				      add_wd=NO;
			           }
                                if (add_wd==YES)
				   {
				   pglist[pgcnt] = (char *) malloc(word_sz);
				   for (lp_var=0; lp_var<=jj; ++lp_var)
			              pglist[pgcnt][lp_var]=word[lp_var];
				   ++pgcnt;
				   }
                                }
                            }
			}
		    }
		}
	    }
	while ( proc );

	if (q_string(&parblk,"PGLIST",pgcnt,pglist,P_UPDATE) != SUCCESS)
	   {
	   c_errmsg("Error returning parameter to TAE","proclib-rparm",&nonfatal);
	   c_errmsg("Fatal error encountered","proclib-fatal",&fatal);
	   }
	if (q_intg(&parblk,"PGCNT",1,&pgcnt,P_UPDATE)!= SUCCESS)
	   {
	   c_errmsg("Error returning parameter to TAE","proclib-rparm",&nonfatal);
	   c_errmsg("Fatal error encountered","proclib-fatal",&fatal);
	   }
	if (q_out(&parblk) != SUCCESS)
	   {
	   c_errmsg("Error returning parameter to TAE","proclib-rparm",&nonfatal);
	   c_errmsg("Fatal error encountered","proclib-fatal",&fatal);
	   }
	if (pgcnt == 0)
	   c_errmsg("No programs called","proclib-nopr",&nonfatal);
        for(lp_var = 0; lp_var<pgcnt; ++lp_var)
	   {
           free(pglist[lp_var]);
           }
	if (fclose(fp)== EOF)
	     c_errmsg("Error closing file", "proclib-close", &fatal);
	z_exit(1,"done");

    return EXIT_SUCCESS;
}
