/****************************************************************************
NAME:				 ISIN

PURPOSE:   Retrieves parameters for the Integerized Sinusoidal projection

PROGRAM HISTORY:
$Log: isin.c,v $
Revision 6.1  2001/05/08 16:25:39  tbeckman
Added support for the ISIN projection for CR #2640

Revision 1.1.2.2  2001/04/30 15:22:07  tbeckman
Verify the nzone parameter is not odd

Revision 1.1.2.1  2001/04/23 20:25:23  tbeckman
Adding isin module


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Get the sphere radius using the input datum
	Convert the parameters from strings to double precision values.  This
	   done because TAE does not allow the input of double precision
       parameters
	Convert the values to be placed in PRJPARMS to DMS format and check
	   to be sure they are valid. 
	Return

ALGORITHM REFERENCES		
None
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

void isin
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum,			/* projection datum */
    double *prjparms		/* projection parameters */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
double temcen;			/* center longitude */
long status;			/* status of called subroutines */
long fatal = -1;		/* Fatal error condition */
long one = 1;			/* for error messages */
char punits[4];			/* buffer for projection units */
char centmer[39];		/* center longitude parameter */
char faleast[39];		/* false east parameter */
char falnorth[39];		/* false north parameter */
int nzone;              /* number of zones */
int rflag;              /* justify flag */

/*  Set the zone and projection type.
-------------------------------------*/
*prjzone = 62;
*prjtype = 31;

/*  Retreive the needed parameters.
-----------------------------------*/
val = p_find(parb,"GEOUNITS");
strcpy (punits,SVAL(*val,0));

val = p_find(parb,"CENTMER");
strcpy (centmer,SVAL(*val,0));

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

val = p_find(parb,"FALEAST");
strcpy (faleast,SVAL(*val,0));

val = p_find(parb,"FALNORTH");
strcpy (falnorth,SVAL(*val,0));

val = p_find(parb,"NZONE");
nzone = IVAL(*val,0);
prjparms[8] = nzone;

val = p_find(parb,"RFLAG");
rflag = IVAL(*val,0);
prjparms[10] = rflag;

/* verify the nzone parameter is even */
if ((nzone % 2) != 0)
   {
   c_lasmsg(TRUE,"isin-nzone","NZONE parameter value of %d is not legal.  It "
            "can not be an odd number.",nzone);
   }
   
/*  Get the sphere radius using the input datum.
------------------------------------------------*/
if (getsphereval(&prjparms[0],&prjparms[1],prjdatum) != E_SUCC)
   {
   c_errmsg("Error returned from getsphereval","isin-call",&one);
   c_errmsg("Fatal error encountered","isin-fatal",&fatal);
   }

/*  Convert the parameters from strings to double precision values.  This
    done because TAE does not allow the input of double precision parameters.
-----------------------------------------------------------------------------*/
sscanf(centmer,"%lf",&temcen);
sscanf(faleast,"%lf",&prjparms[6]);
sscanf(falnorth,"%lf",&prjparms[7]);

/*  Convert the values to be placed in PRJPARMS to DMS format and check
    to be sure they are valid. 
-----------------------------------------------------------------------*/
status = c_degdms(&temcen,(prjparms + 4),punits,"LON");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
}
