/****************************************************************************
NAME:  LAMCC 

PURPOSE:  Retrieves parameters for the LAMBERT CONFORMAL CONIC projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Get the semimajor axis and eccentricity squared using the input datum
	Convert the parameters from strings to double precision values.  This
	   done because TAE does not allow the input of double precision 		   parameters
	Convert the values to be placed in PRJPARMS to DMS format and check
	   to be sure they are valid. 
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

void lamcc
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum,			/* projection datum */
    double *prjparms		/* projection parameters */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
double temspar1;		/* first standard parallel */
double temspar2;		/* second standard parrallel */
double temcen;			/* center longitude */
double temorg;			/* longitude of origin */
long status;			/* status of called subroutines */
long fatal = -1;		/* Fatal error condition */
long one = 1; 			/* for error messages */
char punits[4];			/* buffer for projection units */
char stdpar1[37];		/* first standard parallel parameter */
char stdpar2[37];		/* second standard parallel parameter */
char centmer[37];		/* center longitude parameter */
char origin[37];		/* latitude of origin */
char faleast[37];		/* false east parameter */
char falnorth[37];		/* false north parameter */

/*  Set the zone and projection type.
-------------------------------------*/
*prjzone = 62;
*prjtype = 4;

/*  Retreive the needed parameters.
-----------------------------------*/
val = p_find(parb,"GEOUNITS");
strcpy (punits,SVAL(*val,0));

val = p_find(parb,"STDPARL");
strcpy (stdpar1,SVAL(*val,0));
strcpy (stdpar2,SVAL(*val,1));

val = p_find(parb,"CENTMER");
strcpy (centmer,SVAL(*val,0));

val = p_find(parb,"ORIGIN");
strcpy (origin,SVAL(*val,0));

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

val = p_find(parb,"FALEAST");
strcpy (faleast,SVAL(*val,0));

val = p_find(parb,"FALNORTH");
strcpy (falnorth,SVAL(*val,0));

/*  Get the semimajor axis and eccentricity squared for the projection
    parameters using the input datum.
----------------------------------------------------------------------*/
if (getsphereval(prjparms,prjparms + 1,prjdatum) != E_SUCC)
   {
   c_errmsg("Error returned from getsphereval","lamcc-call",&one);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  Convert the parameters from strings to double precision values.  This
    done because TAE does not allow the input of double precision parameters.
-----------------------------------------------------------------------------*/
sscanf(stdpar1,"%lf",&temspar1);
sscanf(stdpar2,"%lf",&temspar2);
sscanf(centmer,"%lf",&temcen);
sscanf(origin,"%lf",&temorg);
sscanf(faleast,"%lf",(prjparms + 6));
sscanf(falnorth,"%lf",(prjparms + 7));

/*  Convert the values to be placed in PRJPARMS to DMS format and check
    to be sure they are valid. 
-----------------------------------------------------------------------*/
status = c_degdms(&temspar1,(prjparms + 2),punits,"LAT");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
status = c_degdms(&temspar2,(prjparms + 3),punits,"LAT");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
status = c_degdms(&temorg,(prjparms + 5),punits,"LAT");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);

status = c_degdms(&temcen,(prjparms + 4),punits,"LON");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
}
