/****************************************************************************
NAME: OBLMERCA 

PURPOSE:  Retrieves parameters for the OBLIQUE MERCATOR(HOTLINE) A projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Get the semimajor axis and eccentricity squared using the input datum
	Convert the parameters from strings to double precision values.  This
	   done because TAE does not allow the input of double precision 		   parameters
	Convert the values to be placed in PRJPARMS to DMS format and check
	   to be sure they are valid. 
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

void oblmerca
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum,			/* projection datum */
    double *prjparms		/* projection parameters */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
double tempnt1[2];
double tempnt2[2];
double temorg;			/* longitude of origin */
long status;			/* status of called subroutines */
long fatal = -1;		/* Fatal error condition */
long one = 1;			/* for error messages */
char punits[4];			/* buffer for projection units */
char sf[37];			/* scale factor */
char pntloc1[2][37];		/* first point location */
char pntloc2[2][37];		/* second point location */
char origin[37];		/* latitude of origin */
char faleast[37];		/* false east parameter */
char falnorth[37];		/* false north parameter */

/*  Set the zone and projection type.
-------------------------------------*/
*prjzone = 62;
*prjtype = 20;

/*  Retreive the needed parameters.
-----------------------------------*/
val = p_find(parb,"GEOUNITS");
strcpy (punits,SVAL(*val,0));

val = p_find(parb,"SCALFACT");
strcpy (sf,SVAL(*val,0));

val = p_find(parb,"PNTLOC");
strcpy (pntloc1[0],SVAL(*val,0));
strcpy (pntloc1[1],SVAL(*val,1));
strcpy (pntloc2[0],SVAL(*val,2));
strcpy (pntloc2[1],SVAL(*val,3));

val = p_find(parb,"ORIGIN");
strcpy (origin,SVAL(*val,0));

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

val = p_find(parb,"FALEAST");
strcpy (faleast,SVAL(*val,0));

val = p_find(parb,"FALNORTH");
strcpy (falnorth,SVAL(*val,0));

/*  Get the semimajor axis and eccentricity squared for the projection
    parameters using the input datum.
----------------------------------------------------------------------*/
if (getsphereval(prjparms,prjparms + 1,prjdatum) != E_SUCC)
   {
   c_errmsg("Error returned from getsphereval","oblmerca-call",&one);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  Convert the parameters from strings to double precision values.  This
    done because TAE does not allow the input of double precision parameters.
-----------------------------------------------------------------------------*/
sscanf(sf,"%lf",(prjparms + 2));
sscanf(pntloc1[0],"%lf",&tempnt1[0]);
sscanf(pntloc1[1],"%lf",&tempnt1[1]);
sscanf(pntloc2[0],"%lf",&tempnt2[0]);
sscanf(pntloc2[1],"%lf",&tempnt2[1]);
sscanf(origin,"%lf",&temorg);
sscanf(faleast,"%lf",(prjparms + 6));
sscanf(falnorth,"%lf",(prjparms + 7));

/*  Convert the values to be placed in PRJPARMS to DMS format and check
    to be sure they are valid. 
-----------------------------------------------------------------------*/
status = c_degdms(&temorg,(prjparms + 5),punits,"LAT");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
status = c_degdms(&tempnt1[0],(prjparms + 9),punits,"LAT");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
status = c_degdms(&tempnt2[0],(prjparms + 11),punits,"LAT");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);

status = c_degdms(&tempnt1[1],(prjparms + 8),punits,"LON");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
status = c_degdms(&tempnt2[1],(prjparms + 10),punits,"LON");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
}
