/***************************************************************************
NAME:  				PROJPRM 

PURPOSE:  To create a PROJ parameter labeled table file.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  5.1           8/89    D. Steinwand       CSB      Problem with Obl Mer B
  5.2		9/92	S. Nelson	   CSB	    Added map projections
						    Hammer and Robinson.
						    Changed parameters from
						    OPTION to PATHOPT and
						    PTHT to PNTHT.
  5.3	       10/92	D. Etrheim	   CSB	    If latitude is negitive,
						    make the UTM zone negative.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Parameters and subcommands are retrieved
	Parameters common to all subcommands are read
	Parameter values measured in angles are converted to degrees, minutes, 
	 seconds (DMS) as required by projection transformation package
	Parameters for the chosen subcommand are retrieved by calling the
	 appropriate routine
	Form a record of projection transformation package parameters
	Initialize the output labeled table file and write projection 
	 parameters to the file--creating a new file if needed, or writing 
	 to the end of an existing file.

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"


int main
(void)
{ 
struct PARBLK parb;		/* TAE parameter block */
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
double prjparms[15];		/* projection parameter array */
long access = IUPDATE;		/* access to output file */
long i;				/* counter -- current variable */
long zero = 0;			/* zero */
long one = 1;			/* one */
long prjsph;			/* projection sphere */
long prjtype;			/* projection type */	
long prjunits;			/* projection unit of measure */
long prjzone;			/* projection zone */
long std;			/* standard output */
long fatal = -1;		/* fatal error flag for c_errmsg */
char hosout[CMLEN];		/* buffer for host output file name */
char subcmd[20];		/* subcommand */
char prjkey[9];			/* projection key */
char units[5];			/* units of projection parameters */

/* Initialize TAE
-----------------*/
c_inlas (&parb,&std);

/*  Retreive subcommand specified
---------------------------------*/
val = p_find(&parb,"_SUBCMD");
strcpy (subcmd,SVAL(*val,0));

/*  Retreive the parameters needed by all the subcommands.
----------------------------------------------------------*/
if (c_getnam(&parb,"OUTPROJ",&zero,hosout,"proj",&access) != E_SUCC)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);

val = p_find(&parb,"PROJKEY");
strcpy (prjkey,SVAL(*val,0));

val = p_find(&parb,"PROJUNIT");
strcpy (units,SVAL(*val,0));
if (strncmp(units,"RAD",3) == 0)
   prjunits = 0;
else if (strncmp(units,"FEET",4) == 0)
   prjunits = 1;
else if (strncmp(units,"MET",3) == 0)
   prjunits = 2;
else if (strncmp(units,"SEC",3) == 0)
   prjunits = 3;
else if (strncmp(units,"DEG",3) == 0)
   prjunits = 4;
else if (strncmp(units,"DMS",3) == 0)
   prjunits = 5;
else
   {
   c_errmsg("Invalid unit of measure","projprm-unit",&one);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  Initailize the 15 projection parameters to zero.
----------------------------------------------------*/
for (i=0; i < 15; prjparms[i++] = 0.0);

/*  Call the correct subroutine based upon the subcommand that was called.
--------------------------------------------------------------------------*/
if (strcmp(subcmd,"GEOGRAPH") == 0)
   geograph(&parb,&prjzone,&prjtype,&prjsph);
else if (strcmp(subcmd,"UTM") == 0)
   utm(&parb,&prjzone,&prjtype,&prjsph);
else if (strcmp(subcmd,"STATEPLN") == 0)
   statepln(&parb,&prjzone,&prjtype,&prjsph);
else if (strcmp(subcmd,"ALBERS") == 0)
   albers(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"LAMCC") == 0)
   lamcc(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"MERCATOR") == 0)
   mercator(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"PLSTEREO") == 0)
   plstereo(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"POLYCON") == 0)
   polycon(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"EDISCONA") == 0)
   ediscona(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"EDISCONB") == 0)
   edisconb(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"TM") == 0)
   tm(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"STEREO") == 0)
   stereo(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"LAMAZEQA") == 0)
   lamazeqa(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"AZIMEQD") == 0)
   azimeqd(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"GNOMONIC") == 0)
   gnomonic(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"ORTHO") == 0)
   ortho(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"GVNSP") == 0)
   gvnsp(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"SNSOIDAL") == 0)
   snsoidal(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"EQUIRECT") == 0)
   equirect(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"MILLCYL") == 0)
   millcyl(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"GRINTEN") == 0)
   grinten(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"OBLMERCA") == 0)
   oblmerca(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"OBLMERCB") == 0)
   oblmercb(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"SOMA") == 0)
   som(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"SOMB") == 0)
   somb(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"HAMMER") == 0)
   hammer(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"ISIN") == 0)
   isin(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"ROBINSON") == 0)
   robinson(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"MOLLWDE") == 0)
   mollwde(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"GOODE") == 0)
   goode(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"ALASKA") == 0)
   alaska(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"IMOLLWDE") == 0)
   imolwde(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"WAGIV") == 0)
   wagiv(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"WAGVII") == 0)
   wagvii(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else if (strcmp(subcmd,"OBLEQ") == 0)
   obleq(&parb,&prjzone,&prjtype,&prjsph,prjparms);
else
   { 
   c_errmsg("This projection is not supported","projprm-proj",&one);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  Write projection parameters to the output labeled table
-----------------------------------------------------------*/
wrlabtab(hosout,prjkey,&prjunits,&prjzone,&prjtype,&prjsph,prjparms);

/*  Output completion message
-----------------------------*/
c_complt ("PROJPRM",hosout,&one);
} 
