/****************************************************************************
NAME:				 SOM 

PURPOSE:   Retrieves parameters for the SPACE OBLIQUE MERCATOR projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Get the semimajor axis and eccentricity squared using the input datum
	Convert the parameters from strings to double precision values.  This
	   done because TAE does not allow the input of double precision 		   parameters
	Convert the values to be placed in PRJPARMS to DMS format and check
	   to be sure they are valid. 
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

#define LANDRAT 0.5201613

void som
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum,			/* projection datum */
    double *prjparms		/* projection parameters */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
double temlon;
double temang;
long status;			/* status of called subroutines */
long fatal = -1;		/* Fatal error condition */
long one = 1;			/* for error messages */
char punits[4];			/* buffer for projection units */
char incang[37];		/* inclination angle */
char lon[37];			/* parameter longitude */
char nminutes[37];		/* number of minutes per revolution */
char option[37];		/* end of path option */
char faleast[37];		/* false east parameter */
char falnorth[37];		/* false north parameter */

/*  Set the zone and projection type.
-------------------------------------*/
*prjzone = 62;
*prjtype = 22;

/*  Retreive the needed parameters.
-----------------------------------*/
val = p_find(parb,"GEOUNITS");
strcpy (punits,SVAL(*val,0));

val = p_find(parb,"INCANG");
strcpy (incang,SVAL(*val,0));

val = p_find(parb,"LONGASC");
strcpy (lon,SVAL(*val,0));

val = p_find(parb,"NMINUTES");
strcpy (nminutes,SVAL(*val,0));

val = p_find(parb,"PATHOPT");
strcpy (option,SVAL(*val,0));

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

val = p_find(parb,"FALEAST");
strcpy (faleast,SVAL(*val,0));

val = p_find(parb,"FALNORTH");
strcpy (falnorth,SVAL(*val,0));

/*  Get the semimajor axis and eccentricity squared for the projection
    parameters using the input datum.
----------------------------------------------------------------------*/
if (getsphereval(prjparms,prjparms + 1,prjdatum) != E_SUCC)
   {
   c_errmsg("Error returned from getsphereval","som-call",&one);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  Convert the parameters from strings to double precision values.  This
    done because TAE does not allow the input of double precision parameters.
-----------------------------------------------------------------------------*/
sscanf(incang,"%lf",&temang);
sscanf(lon,"%lf",&temlon);
sscanf(nminutes,"%lf",(prjparms + 8));
if (!strncmp(option,"START",5))
   *(prjparms + 10) = 0;
else
   *(prjparms + 10) = 1;
/*sscanf(option,"%lf",(prjparms + 10));*/

sscanf(faleast,"%lf",(prjparms + 6));
sscanf(falnorth,"%lf",(prjparms + 7));

/*  Convert the values to be placed in PRJPARMS to DMS format and check
    to be sure they are valid. 
-----------------------------------------------------------------------*/
status = c_degdms(&temang,(prjparms + 3),punits,"CIR");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);

status = c_degdms(&temlon,(prjparms + 4),punits,"LON");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);

/* Set the Landsat Ratio.  This used to be an SOMA parameter but was taken
   out as it was set as a constant and not called by GCTP.  It was kept in the
   parameter array for possible future use.
   ---------------------------------------------------------------------*/
*(prjparms + 9) = LANDRAT;

}
