/****************************************************************************
NAME:				 SOMB 

PURPOSE:   Retrieves parameters for the SPACE OBLIQUE MERCATOR B projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  5.2		6/93	T. Mittan	   CSB      Converted SOM.C to SOMB.C 
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Get the semimajor axis and eccentricity squared using the input datum
	Convert the parameters from strings to double precision values.  This
	   done because TAE does not allow the input of double precision 		   parameters
	Convert the values to be placed in PRJPARMS to DMS format and check
	   to be sure they are valid. 
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

void somb
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum,			/* projection datum */
    double *prjparms		/* projection parameters */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
char one[2];
long fatal = -1;		/* Fatal error condition */
long error = 1;			/* for error messages */
char path[37];			/* end of path option */
/*char satnum[37];	*/		/* ratio */
long satnum;
char faleast[37];		/* false east parameter */
char falnorth[37];		/* false north parameter */

/*  Set the zone and projection type.
-------------------------------------*/
*prjzone = 62;
*prjtype = 22;

/*  Retreive the needed parameters.
-----------------------------------*/
val = p_find(parb,"SATNUM");
satnum = IVAL(*val,0);
*(prjparms + 2) = (float)satnum;

val = p_find(parb,"PATH");
strcpy (path,SVAL(*val,0));

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

val = p_find(parb,"FALEAST");
strcpy (faleast,SVAL(*val,0));

val = p_find(parb,"FALNORTH");
strcpy (falnorth,SVAL(*val,0));

strcpy (one,"1");

/*  Get the semimajor axis and eccentricity squared for the projection
    parameters using the input datum.
----------------------------------------------------------------------*/
if (getsphereval(prjparms,prjparms + 1,prjdatum) != E_SUCC)
   {
   c_errmsg("Error returned from getsphereval","somb-call",&error);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  Convert the parameters from strings to double precision values.  This
    done because TAE does not allow the input of double precision parameters.
-----------------------------------------------------------------------------*/
sscanf(path,"%lf",(prjparms + 3));
sscanf(faleast,"%lf",(prjparms + 6));
sscanf(falnorth,"%lf",(prjparms + 7));
sscanf(one,"%lf",(prjparms + 12));

}
