/****************************************************************************
NAME:					 UTM 

PURPOSE:  Retrieves parameters for the UNIVERSAL TRANSVERSE MERCATOR projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  5.01		4/89	D. Steinwand	   CSB      Calc zone # from longitude
  5.1	       10/92    D. Etrheim         CSB      If latitude is negitive,
						    make the zone negative.
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Convert the parameters from strings to double precision values.  This
	   done because TAE does not allow the input of double precision 		   parameters
	Convert the longitude value to degrees & calculate zone number
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "tm.h"
#include "protos.h"

void utm
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum			/* projection datum */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
struct VARIABLE *latval;	/* pointer to TAE variable within V_BLOCK */
struct VARIABLE *lonval;	/* pointer to TAE variable within V_BLOCK */
struct VARIABLE *valzone;	/* pointer to TAE variable zone within VBLOCK */
double tlat;			/* latitude parameter */
double tlon;			/* longitude parameter */
long status;			/* status of called subroutines */
long fatal = -1;		/* Fatal error condition */
long non_fatal = 1;		/* Fatal error condition */
char punits[4];			/* buffer for projection units */
char lat[37];			/* parameter latitude */
char lon[37];			/* parameter longitude */

/*  Retrieve the parameters needed.
-----------------------------------*/
valzone = p_find(parb,"ZONENUM");
*prjzone = IVAL(*valzone,0);

val = p_find(parb,"GEOUNITS");
strcpy (punits,SVAL(*val,0));

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

latval = p_find(parb,"LAT");
if ((*latval).v_count != 0)
   {
   strcpy (lat,SVAL(*latval,0));
   }

lonval = p_find(parb,"LONG");
if ((*lonval).v_count != 0)
   {
   strcpy (lon,SVAL(*lonval,0));
   }

if ((((*latval).v_count == 0) || ((*lonval).v_count ==0)) && 
    ((*valzone).v_count == 0))
   {
   c_errmsg("ZONE or LAT and LONG have to be specified","projprm-specify",
             &non_fatal);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  If LAT and LONG are specified, convert them to DMS.
------------------------------------------------------*/
if ((*lonval).v_count != 0)
   {
   sscanf(lon,"%lf",&tlon);
   sscanf(lat,"%lf",&tlat);

   status = c_decdeg(&tlon,punits,"LON");
   if (status == E_FAIL)
      c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   status = c_decdeg(&tlat,punits,"LAT");
   if (status == E_FAIL)
      c_errmsg("Fatal error encountered","projprm-fatal",&fatal);

   utmzon(&tlon,prjzone);
   if (tlat < 0.0)
      *prjzone = -(*prjzone);

   /* *prjzone = (long)((tlon + 180) / 6) + 1;
   if (*prjzone > 60) *prjzone -= 60;*/
   } 
*prjtype = 1;
}
