/****************************************************************************
NAME:				 WAGVII

PURPOSE:   Retrieves parameters for the HAMMER projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  5.1		7/92	D. Steinwand 	   CSB	    modified MILLCYL.C to be
						    HAMMER.C
  5.2		6/93	T. Mittan 	   CSB	    modified HAMMER.C to be
						    WAGVII.C
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Get the sphere radius using the input datum
	Convert the parameters from strings to double precision values.  This
	   done because TAE does not allow the input of double precision 		   parameters
	Convert the values to be placed in PRJPARMS to DMS format and check
	   to be sure they are valid. 
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

void wagvii
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum,			/* projection datum */
    double *prjparms		/* projection parameters */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
double temcen;			/* center longitude */
long status;			/* status of called subroutines */
long fatal = -1;		/* Fatal error condition */
long one = 1;			/* for error messages */
char punits[4];			/* buffer for projection units */
char centmer[37];		/* center longitude parameter */
char faleast[37];		/* false east parameter */
char falnorth[37];		/* false north parameter */

/*  Set the zone and projection type.
-------------------------------------*/
*prjzone = 62;
*prjtype = 29;

/*  Retreive the needed parameters.
-----------------------------------*/
val = p_find(parb,"GEOUNITS");
strcpy (punits,SVAL(*val,0));

val = p_find(parb,"CENTMER");
strcpy (centmer,SVAL(*val,0));

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

val = p_find(parb,"FALEAST");
strcpy (faleast,SVAL(*val,0));

val = p_find(parb,"FALNORTH");
strcpy (falnorth,SVAL(*val,0));

/*  Get the sphere radius using the input datum.
------------------------------------------------*/
if (getsphereval(prjparms,prjparms + 1,prjdatum) != E_SUCC)
   {
   c_errmsg("Error returned from getsphereval","wagvii-call",&one);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

/*  Convert the parameters from strings to double precision values.  This
    done because TAE does not allow the input of double precision parameters.
-----------------------------------------------------------------------------*/
sscanf(centmer,"%lf",&temcen);
sscanf(faleast,"%lf",(prjparms + 6));
sscanf(falnorth,"%lf",(prjparms + 7));

/*  Convert the values to be placed in PRJPARMS to DMS format and check
    to be sure they are valid. 
-----------------------------------------------------------------------*/
status = c_degdms(&temcen,(prjparms + 4),punits,"LON");
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
}
