#include "radiom2.h"
#include "util.h"
#include "pixman.h"


void apply_rlut
(
    char hosin[][CMLEN],		/* input images host names	*/
    char hosrlut[],		/* input rlut host name		*/
    char hosout[][CMLEN],		/* output image host name	*/
    long wind[],			/* image window			*/
    long *num_inimages,		/* no. input images		*/
    long *num_bands,		/* no. bands                    */
    long image_band_num[][MAXBND + 1]
)
/*
***************************************************************************

NAME: APPLY_RLUT

PURPOSE: Applies RLUTs to input image and generates output image

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         10-Feb-82    J. Bay
    2         11-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

  Determine number of detectors (4 if band 6, else 16 for tm, 6 for mss).
  Read the set of RLUTs 
  For each line in the image window
     Read image line
     Determine detector number for line
     Apply appropriate RLUT to the line, 4 times if band 6 of tm, 
     else only once
     Write the line to output
  Close image, and RLUT files

***************************************************************************
*/
{
struct	BDDR	bddr;
struct	DDR	ddr;

long    inf_err = 1;
long    zero = 0;
long    one = 1;
char    buf[ERRLEN];                   /* print buffer          */
char    *prog_name = "RADIOM2";        /* program name          */


double	line_inc;
double	max_val[MAXBND];	/* maximum and minimum		*/
double	min_val[MAXBND];	/* values in the output image	*/

long	access, band, 
	iband, ibdet, 
	idet, iline, iter,
	line, 
	master_line, 
	ndetect, 		/* no. detectors                 */
	niter, nseg;
	
FILE * fd_sfin;

long	msg_flag = ONECAT;
long	type = EBYTE;
	
static	long	bandshape[9] = {ELONG, 1, 1, 0, 0, 0, 0, 0, 0};
static	long	detshape[9] = {ELONG, 1, 1, 0, 0, 0, 0, 0, 0};
static	long	filshape[9] = {EWORD, 1, 5, 0, 0, 0, 0, 0, 0};
static	long	lutshape[9] = {EBYTE, 1, 256, 0, 0, 0, 0, 0, 0};

short	filblk[5];
	
static unsigned char *lut_buf;
unsigned char *lut[MAX_DET];

/*
open RLUT file and get no. detectors and other information
----------------------------------------------------------
*/
access = IREAD;
c_stopen (&fd_sfin, hosrlut, &access, &nseg); 
	
c_stget (&fd_sfin, "FILBLK", &zero, &zero, filshape, filblk);
c_stget (&fd_sfin, "NDETECT", &zero, &zero, detshape, &ndetect);
	
if (*num_bands != filblk[0])
   {
   c_eclose (&fdesc_out);
   sprintf (buf, 
"Number of bands in the image: %d, and in the RLUT file: %d",
      *num_bands, filblk[0]);
   c_abort (&block, buf, "radiom2-nbands", prog_name, hosout, &one);
   }

/*
assign storage for lookup table
-------------------------------
*/
if ((lut_buf = (unsigned char *) malloc (ndetect * 256)) == NULL)
   {
   c_eclose (&fdesc_out);
   c_abort (&block, "Error allocating dynamic memory", "radiom2-alloc", 
            prog_name, hosout, &one);
   }
for (idet = 0; idet < ndetect; idet++)
   lut[idet] = &lut_buf[idet * 256];

/*
get image ddr information
-------------------------
*/
c_getddr (hosin[0], &ddr);

if (ddr.master_line == 0) 
   {
   master_line = 1;
   line_inc = 1;
   }
else
   {
   master_line = ddr.master_line;
   line_inc = ddr.line_inc;
   }  

c_inmnmx (&type, num_bands, min_val, max_val);

/*
initialize processing message utility
-------------------------------------
*/
c_inittm ("RADIOM2", "Processed", "line", "bands", " ", &wind[NL], 
         num_bands, &msgtim, &msg_flag);

/*
process one band at a time
--------------------------
*/
for (band = 1; band <= *num_bands; band++)
   {
   c_getbdr (hosin[0], &bddr, &image_band_num[0][band-1]);
   c_stget (&fd_sfin, "BANDNUM", &band, &zero, bandshape, &iband);

   if (iband != bddr.bandno)
      {
      sprintf (buf, 
           "WARNING: Band number in the image: %d, and in the RLUT file: %d\n",
           iband, bddr.bandno);
      c_errmsg (buf, "radiom2-band", &inf_err);
      }
   niter = 1;
   ibdet = ndetect;
/*
   check for TM band 6 processing
   ------------------------------
*/
   if (ndetect  ==  16 && iband == 6) 
      {
      ibdet = 4;
      niter = 4;
      }
/*
   read the set of RLUTS
   ---------------------
*/
   for (idet = 1; idet <= ibdet; idet++)
      {
      if (c_stget (&fd_sfin, "DETRLT", &band, &idet, lutshape,
                   lut[idet-1]) != E_SUCC)
         {
         c_eclose (&fdesc_out);
         sprintf (buf, "Error reading lookup table for detector %d of band %d",
                  idet, band);
         c_abort (&block, buf, "radiom2-lutread", prog_name, hosout, &one);
         }
      min_val[band-1] = min (min_val[band-1], lut[idet-1][0]);
      max_val[band-1] = max (max_val[band-1], lut[idet-1][255]);
      }

/*
   start timed processing message utility
   --------------------------------------
*/
   curent = 1;
   curcat = band;
   c_startm();

/*
   process line by line
   --------------------
*/
   for (line = wind[SL]; line <= wind[SL] + wind[NL] - 1; line++, curent++)
      {
/*    get the detector number
      -----------------------
*/
      idet = (long) ((line-1)*line_inc + master_line - 1) % ibdet; /* mss and others */
      if (ndetect  ==  16) idet = ibdet - 1 - idet;           /* tm data */
/*
      apply RLUT to line
      ------------------
*/
      iline = line - wind[SL] + 1;
      for (iter = 1; iter <= niter; iter++)
         {
         c_eread (fdesc_in, &image_band_num[0][band-1], &iline, inbuf, &one);
      
         c_pxlkup (inbuf, outbuf, &wind[NS], &type, lut[idet], &type);

         c_ewrite (&fdesc_out, &band, &iline, outbuf, &one);
         }
      }  
   c_stoptm ();
   }
/*
write min and max values to ddr
-------------------------------
*/
c_svmnmx (hosout[0], num_bands, min_val, max_val, "YES");

/*
create output image history file
--------------------------------
*/
c_hstout (&block, hosout[0], hosin, num_inimages, " ");

/*
close files and free storage
----------------------------
*/
free (lut_buf);

c_eclose (fdesc_in);
c_eclose (&fdesc_out);

c_stclse (&fd_sfin);
}
