/*
*****************************************************************************

NAME:	RANDSAMP

PURPOSE: Performs image compression by randomly sampling the input image.
         In principle, the statistical nature of the input image is preserved 
         by the compression.

PROGRAM HISTORY
--------------------------------------------------------------------------------
  Date       Programmer                       Changes 
---------  ---------------  ----------------------------------------------------
16-Jun-89  K. Narayanan     Development of fmatrix.c and free_matrix.c
                            Probably also original application development. 
27-Nov-89  K. A. Narayanan  Conversion to 5.0
25-Jan-91  K. Donaldson     Added comments and algorithm description;  in 
                            copysamp.c, allocated more space for BUFFER array 
			    based on the number of duplicate samples;  nsamp is
			    now passed to gensamp() by reference instead of by 
			    value
19-Jul-91  T. Mittan        Conversion to baseline code
24-Jan-92  S. Nelson	    Removed warning messages on compile
23-Sep-92  T. Mittan	    Added extern to curent and curcat to eliminate 
			    warnings on link
 3-Aug-95  D. Zokaites      Put all change histories into randsamp.c

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*****************************************************************************
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct  PARBLK  block;          /* TAE parameter block          */

float   *inbuf;                 /* image i/o buffer             */
float   *outbuf;                /* image i/o buffer             */

struct FDESC *  fdesc[MAXIMG];          /* file descriptors             */
struct FDESC *  fdesc_out;              /* output file descriptor       */
struct GDESC *  gdesc_in;               /* group descriptor             */
long    ipoint[MAXIMG*MAXBND+1]; /* offsets into image i/o	*/
long    min_max;                /* minmax option flag           */
long    msgtim;                 /* message interval             */
long	*samples;		/* samples block coordinates	*/

extern long    curcat;          /* parameters for processing    */
extern long    curent;          /* message utility              */

int main
(void) 
{
double	maxval[MAXBND],		/* output image maximum values		*/
	minval[MAXBND];		/* output image minimum values		*/


static	long	image_band_num[MAXIMG][MAXBND + 1];
                              	/* band numbers in each input image 	*/

long	band,			/* current band number                  */
        fatal = -1,             /* fatal flag                 		*/
	nlout,			/* No. lines in output image		*/
	nsamp,			/* No. sample blocks			*/
	nsout,			/* No. samples/line in output image	*/
	num_image_bands[MAXIMG],/* number of bands in each input image  */
	num_inbands,            /* total number of input image bands	*/
	num_inimages,           /* number of input images           	*/
	obands[1][MAXBND+1],	/* output image bands			*/
	owind[1][4],		/* output image window			*/
	percent,                /* sampling percentage                  */
	sampsize[2], 		/* sample block size			*/
	status,			/* function return flag                 */
	wind[MAXIMG][4],        /* input image windows              	*/
	one = 1;

char	duplicate[4];		/* duplicate samples flag		*/
static	char	hos_in[MAXIMG][CMLEN];
                                /* input images host names          	*/
char	hos_out[1][CMLEN];         /* output image host name           	*/
static	char	las_in[MAXIMG][CMLEN];   
                                /* input images las names           	*/
char	las_out[1][CMLEN];         /* output image las name            	*/

/*
Call GETPAR to get las parameters.
-----------------------------------
*/
if (getpar (las_in, hos_in, las_out, hos_out, &num_inimages, &num_inbands,
    num_image_bands, image_band_num, wind, sampsize, &percent, duplicate) 
     != E_SUCC)
      c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);

/*
Call SELSAMP to select random samples from input image.
--------------------------------------------------------
*/
if (selsamp (wind, sampsize, percent, duplicate, &nsamp, hos_out) 
    != E_SUCC)
      c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);
/*
Process bands sequentially
--------------------------
*/
for (band = 1; band <= num_inbands; band++)
   {
/*
Call FILSPEC to open images.
-----------------------------
*/
   if (filspec (hos_in, hos_out, &num_inimages, num_image_bands, &num_inbands, 
        image_band_num, wind, nsamp, &nlout, &nsout, sampsize, band) != E_SUCC)
	{
	if (c_eclose (&fdesc_out) != E_SUCC)
	c_errmsg("Error closing images","randsamp-close",&one);
        if (c_egclse (&gdesc_in) != E_SUCC)
	c_errmsg("Error closing images","randsamp-close",&one);
        c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);
	}

/*
Call COPYSAMP to build the the output image.
----------------------------------------------
*/
   
   if (copysamp (&band, &num_inbands, nsamp,wind, sampsize, nsout, &nlout, 
    	hos_out) != E_SUCC)
      c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);
   }

/*
Free the output buffer.
-----------------------
*/
free (outbuf);

/*
Update output image history, min/max, and issue completion message
------------------------------------------------------------------
*/
if (min_max)
   {
   for (band = 0; band < num_inbands; band++) 
      obands[0][band] = band+1;
   obands[0][num_inbands] = 0;
   owind[0][SL] = owind[0][SS] = 1; 
   owind[0][NL] = nlout; 
   owind[0][NS] = nsout; 
   if (c_minmax(hos_out, &one, owind, obands, &num_inbands, minval,maxval) 
    != E_SUCC)
      c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);
   }

status = c_hstout (&block, hos_out[0], hos_in, &num_inimages, "");
if (status == E_FAIL)
      c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);
c_complt ("RANDSAMP", las_out, &one);

return EXIT_SUCCESS;
}
