
/*******************************************************************************
NAME:     image.c 
PURPOSE:  Simplify image IO for application developers by providing a data
          structure driven image IO implementation.  Multiple processing modes
          are supported, no global variables are required, full LAS
          functionality is assured.  Application developers may select some of
          these IO routines;  a few applications require trivial revisions to
          these IO routines.

			    SOURCE CODE HISTORY
--------------------------------------------------------------------------------
  Date       Programmer                          Changes 
---------  ---------------  ----------------------------------------------------
 Dec 1992  David Zokaites   Developed imageio application:  functions and data
                            structures for sequential IO.
 Feb 1993  David Zokaites   Enabled random IO by writing openrin(), openrout(),
			    rread(), rwrite(), rimage();  created global vars
			    for easier error messages;  wrote minmax().
 May 1993  David Zokaites   Enabled block IO by writing bread(), bwrite(),
                            bimage();  wrote make_ddr().
 Jul 1993  David Zokaites   Renamed application from imageio to template;
			    eliminated functions getin2() and getout2();  wrote
			    help for template.
 Aug 1993  David Zokaites   Allowed for data conversion upon read and write;
                            wrote convert[srb]().
 May 1994  David Zokaites   Wrote openbin(), openbout(), minmax[srb](),
			    parse_file();  expanded help in template.pdf;
			    removed make.com;  added tae parms iomode,
			    readtype, procmeth, writconv.
 Aug 1994  David Zokaites   Prototyped all las support functions via
			    makep.{c,csh};  changed variable names to keep with
			    las conventions;  put errclean() in a separate file
			    to minimize global vars;  wrote Lintfile to run
			    lint;  eliminated functions real_parm(),
			    string_parm(), integer_parm(), parse_file();
			    renamed global data structures for las images;
			    wrote functions open[sr]out2().
 Nov 1994  David Zokaites   Improved error messages;  renamed ddr_his() to
                            finish_outi2().
 Jan 1995  David Zokaites   Enabled errclean() to process a variable number of
			    arguments;  wrote function errmsg();  (errmsg later
			    became c_lasmsg which later became a support);
			    reduced # of global vars, deleted begin_las();
			    reconstructed program history;  updated prototypes
			    of las support functions.
 Feb 1995  David Zokaites   Updated Makefile format.
 Jun 1995  David Zokaites   Tweaked error messages, called c_lasmsg() instead 
                            of c_errmsg() or errmsg(). 
 Jul 1995  David Zokaites   For fatal errors, errclean() can now be called from 
			    any function;  added status flags to image data
			    structures;  pblock is no longer global;
			    open[sr]out() now just call open[sr]out2();  changed
			    parms to open[sr]out2();  changed indenting per las
			    conventions;  renamed process.c to proces.c;
			    deleted c_lasmsg.c as this function is now a
			    support;  changed data structure and function names;
			    wrote read_bil() and write_bil(). 
 Jul 1995  David Zokaites   Some changes for RANDSEL:  deleted unnecessary
                            functions, changed functions to open images
*******************************************************************************/


#include <string.h>     /* for call to strcmp() */ 
#include <stdlib.h>     /* for call to malloc(), free() */ 
#include "randsel.h"    /* application header file */ 
#include "util.h"
#include "pixman.h"
#include "protos.h"




/*******************************************************************************
NAME:     open_bil_in 
PURPOSE:  open an input image for BIL (sequential) IO and allocate data buffer  
*******************************************************************************/

void open_bil_in 
(
   struct PARBLK *pblock,   /* input:  tae parameter block */ 
   char *parmname,          /* input:  name of tae parm for image.  If NULL, 
                                       image name is already in inimage */ 
   long dtypeap,            /* input:  convert image data to this type upon
			               reading.  pass a 0 if not converting */ 
   struct INIMAGE *inimage  /* output: opened input image */ 
)
{

/* declare variables */ 
   long accessm=IREAD,    /* image access mode, parm to c_eopens() */ 
        bandnum,          /* loop index, band number */ 
        imagenum,         /* loop index, image number */ 
        buffersz,         /* buffer size returned by c_ealloc() */
        maximg=MAXIMG,    /* max number of images allowed */ 
	one=1,            /* value of one */ 
        qflag[MAXIMG][2], /* flag, do image components have a band or
		             window specified */ 
	samewindow,       /* flag, do image components have the same 
			     window specified? */ 
        zero=0;           /* value of 0 */ 

/* parse input image spec */ 
   /* get window, bands, filenames, etc. */ 
      if (parmname == NULL && 
      c_getin2 (inimage->taename[0], inimage->taename,
	       inimage->hostname, &inimage->nmrimg, inimage->window, 
	       inimage->bands, inimage->nbands, &inimage->totalbands, &maximg, 
	       &samewindow, qflag) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");

      if (parmname != NULL && 
      c_getin (pblock, parmname, &zero,
	       inimage->taename, inimage->hostname, &inimage->nmrimg, 
	       inimage->window, inimage->bands, inimage->nbands, 
	       &inimage->totalbands, &maximg, &samewindow, qflag) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");

   /* error check:  different window sizes causes c_egroup() to fail */ 
      if (!samewindow)
	 errclean ("randsel-window", 
	          "Images specified by %s do not have the same window size", 
		  parmname);

   /* set nl, ns */ 
      inimage->nl = inimage->window[0][NL]; 
      inimage->ns = inimage->window[0][NS]; 

/* update last used fields of input images */ 
   for (imagenum=0; imagenum<inimage->nmrimg; imagenum++)
      (void) c_lused (inimage->hostname[imagenum]);

/* open image files */ 
   inimage->iotype = BILIO;             /* BIL (sequential) IO */ 
   inimage->fdarray[inimage->nmrimg]=0; /* null terminate array */ 
   inimage->dtypeap=dtypeap; 

   for (bandnum=imagenum=0; imagenum<inimage->nmrimg;  
        bandnum+=inimage->nbands[imagenum], imagenum++)
      if (c_eopens ((struct FDESC **)inimage->fdarray+imagenum, 
                   inimage->hostname[imagenum],
		   inimage->bands[imagenum], &inimage->window[imagenum][SL],
		   &inimage->window[imagenum][SS],
		   &inimage->window[imagenum][NL],
		   &inimage->window[imagenum][NS], inimage->bufoff+bandnum,
		   &accessm, &inimage->dtypeap, &zero, &zero)  !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");
   inimage->opened = TRUE; 

/* set data type, dtypeap and dtypein */ 
   switch (inimage->dtypeap)
      {
      case EBYTE:  inimage->dsizeap=EBYTESIZE;  break; 
      case EWORD:  inimage->dsizeap=EWORDSIZE;  break; 
      case ELONG:  inimage->dsizeap=ELONGSIZE;  break; 
      case EREAL:  inimage->dsizeap=EREALSIZE;  break; 
      default:  errclean ("randsel-fatal", 
		         "Program error:  unknown image data type");
      }
   /* get datatype of 1st disk image, 
      supporting routines convert other images to data type of first image */ 
      if (c_gettyp (inimage->hostname, &one, &inimage->dtypein) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");

/* allocate memory for image buffer */ 
   if ((inimage->buffer = (unsigned char *) 
   c_ealloc((struct FDESC **)inimage->fdarray, &buffersz)) == NULL)
      errclean ("randsel-memory", 
               "Unable to allocate memory for input image buffer"); 

/* group image files */ 
   if (c_egroup ((struct FDESC **)inimage->fdarray, 
                (struct GDESC **)&inimage->gdesc, inimage->buffer,
                &buffersz) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* initialize first line of image */ 
   read_bil (inimage); 
}





/*******************************************************************************
NAME:     open_bil_out
PURPOSE:  open an output image for BIL (sequential) IO and allocate data 
	  buffer
*******************************************************************************/

void open_bil_out 
(
   struct PARBLK *pblock,   /* input:  tae parameter block */ 
   char *parmname,          /* input:  tae parm for image name */ 
   long dtypeap,            /* input:  date type as processed by the app for
				       the output image, ignored if !writconv */
   long writconv,           /* input:  flag. if TRUE, data will be converted 
				       to odtype upon writing to disk. */ 
   struct INIMAGE  *inimage,  /* input:  input image info, needed for dtypein
                                         and totalbands */ 
   struct OUTIMAGE *outimage  /* output: opened output image */
)
{

/* declare variables */ 
   char *odtype;            /* user value for odtype */
   long accessm=IWRITE,     /* image access mode, parm to c_eopens() */ 
        buffersz,           /* buffer size returned by c_ealloc() */
        one=1,              /* value of one */ 
	opt[2],             /* optional parm to c_eopens(), used for data 
			       type conversion */ 
        zero=0;             /* value of zero */ 
   struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 

/* set dtypeout based on parm odtype */ 
   tparm = p_find (pblock, "odtype");             
   odtype = SVAL (*tparm,0);
   if      ( !strcmp (odtype, "SAME"))  outimage->dtypeout = inimage->dtypein; 
   else if ( !strcmp (odtype, "BYTE"))  outimage->dtypeout = EBYTE; 
   else if ( !strcmp (odtype, "I*2" ))  outimage->dtypeout = EWORD; 
   else if ( !strcmp (odtype, "I*4" ))  outimage->dtypeout = ELONG; 
   else if ( !strcmp (odtype, "R*4" ))  outimage->dtypeout = EREAL; 
   else
      c_lasmsg (TRUE, "randsel-fatal", 
               "Program error:  parm ODTYPE has an invalid value"); 

/* get tae globals:  $delflg, $minmax, $msgtim */ 
   /* delflg */ 
      tparm = p_find (pblock, "$delflg");   
      if ( !strcmp (SVAL(*tparm,0), "YES"))
	 outimage->delflg = TRUE;
      else
	 outimage->delflg = FALSE;
   /* minmax */ 
      tparm = p_find (pblock, "$minmax");   
      if ( !strcmp (SVAL(*tparm,0), "YES"))
	 outimage->minmax = TRUE;
      else 
	 outimage->minmax = FALSE;
   /* msgtim */ 
      tparm = p_find (pblock, "$msgtim");   
      outimage->msgtim = IVAL (*tparm, 0); 

/* get output image name */ 
   if (c_getout (pblock, parmname, (char (*)[256])outimage->taename[0], 
                (char (*)[256])outimage->hostname[0]) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* set data type and size */ 
   /* dtypeap */ 
      if (writconv)
	 outimage->dtypeap = dtypeap; 
      else 
	 outimage->dtypeap = outimage->dtypeout; 
   /* dsizeap */ 
      switch (outimage->dtypeap)
	 {
	 case EBYTE:  outimage->dsizeap=EBYTESIZE;  break; 
	 case EWORD:  outimage->dsizeap=EWORDSIZE;  break; 
	 case ELONG:  outimage->dsizeap=ELONGSIZE;  break; 
	 case EREAL:  outimage->dsizeap=EREALSIZE;  break; 
	 default:  errclean ("randsel-fatal", 
	                    "Program error:  unknown image data type");
	 }

/* set misc variables in data structure */ 
   /* nl and ns were already set in image_size() */ 
   outimage->iotype = BILIO;    /* BIL (sequential) IO */ 
   outimage->fdarray[1] = 0;    /* null terminate array */ 
   outimage->totalbands = inimage->totalbands; 

/* open image file */ 
   if (writconv)
      {
      opt[0] = outimage->dtypeout; 
      opt[1] = 0; 
      }
   else 
      opt[0] = 0; 
   if (c_eopens ((struct FDESC **)outimage->fdarray, outimage->hostname[0], 
	        &outimage->totalbands, &one, &one, &outimage->nl, &outimage->ns,
	        outimage->bufoff, &accessm, &outimage->dtypeap, &zero, opt) 
		!=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");
   outimage->opened = TRUE; 

/* allocate memory for image buffer */ 
   if ((outimage->buffer = (unsigned char *) 
   c_ealloc((struct FDESC **)outimage->fdarray, &buffersz)) == NULL)
      errclean ("randsel-memory", 
               "Unable to allocate memory for output image buffer"); 

/* group image files */ 
   if (c_egroup ((struct FDESC **)outimage->fdarray, 
                (struct GDESC **)&outimage->gdesc, outimage->buffer,
                 &buffersz) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* initialize min max arrays */ 
   if (outimage->minmax && 
   c_inmnmx (&outimage->dtypeout, &outimage->totalbands, outimage->minval, 
	    outimage->maxval) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* initialize first line of image */ 
   write_bil (outimage); 
}





/*******************************************************************************
NAME:     open_bsq_in 
PURPOSE:  open an input image for BSQ (random) IO and allocate data buffers 
*******************************************************************************/

void open_bsq_in
(
   struct PARBLK *pblock, /* input:  tae parameter block */ 
   char *parmname,        /* input:  tae name of image to open */ 
   long dtypeap,          /* input:  convert image data to this type upon
			             reading.  pass a 0 if not converting */ 
   struct INIMAGE *inimage    /* output: opened input image */ 
)
{

/* declare variables */ 
   long accessm=IREAD,    /* image access mode, parm to c_eopenr() */ 
        bandnum,          /* loop index, band number */ 
        imagenum,         /* loop index, image number */ 
        maximg=MAXIMG,    /* max number of images allowed */ 
	one=1,            /* value of one */ 
        qflag[MAXIMG][2], /* flag, do image components have a band or
			     window specified */ 
	samewindow,       /* flag, do image components have the same 
			     window specified? */ 
        tband,            /* loop index, total band number */ 
        zero=0;           /* value of zero */ 

/* parse input image spec */ 
   /* call c_getin() */ 
      if (c_getin (pblock, parmname, &zero, inimage->taename, inimage->hostname,
		  &inimage->nmrimg, inimage->window, inimage->bands, 
		  inimage->nbands, &inimage->totalbands, &maximg, &samewindow, 
		  qflag) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");
   /* error check:  different window sizes */ 
      if (!samewindow)
	 errclean ("randsel-window", 
	          "Images specified by %s do not have the same window size", 
		  parmname);
   /* set nl, ns */ 
      inimage->nl = inimage->window[0][NL]; 
      inimage->ns = inimage->window[0][NS]; 

/* update last used fields of input images */ 
   for (imagenum=0; imagenum<inimage->nmrimg; imagenum++)
      (void) c_lused (inimage->hostname[imagenum]);

/* open image file and set variables */ 
   inimage->iotype = BSQIO;   /* BSQ (random) IO */ 
   inimage->dtypeap=dtypeap; 
   for (imagenum=0; imagenum<inimage->nmrimg; imagenum++)
      if (c_eopenr ((struct FDESC **)inimage->fdarray+imagenum, 
                   inimage->hostname[imagenum],
		   inimage->nbands+imagenum, &inimage->window[imagenum][SL],
		   &inimage->window[imagenum][SS],
		   &inimage->window[imagenum][NL],
		   &inimage->window[imagenum][NS], &accessm, &inimage->dtypeap,
		   &zero, &one) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");
   inimage->opened = TRUE; 

/* set dsizeap and dtypein */ 
   switch (inimage->dtypeap)
      {
      case EBYTE:  inimage->dsizeap=EBYTESIZE;  break; 
      case EWORD:  inimage->dsizeap=EWORDSIZE;  break; 
      case ELONG:  inimage->dsizeap=ELONGSIZE;  break; 
      case EREAL:  inimage->dsizeap=EREALSIZE;  break; 
      default:  errclean ("randsel-fatal", 
                         "Program error:  unknown image data type");
      }
   /* get datatype of 1st disk image, 
      supporting routines convert other images to data type of first image */ 
      if (c_gettyp (inimage->hostname, &one, &inimage->dtypein) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");

/* build luts for later access by read_bsq().  Due to subbanding, the parm 
   eband to c_eread() is relative to the bands in the input -- not output -- 
   image files.  This is accommodated by cblut. */
   for (tband=imagenum=0; imagenum<inimage->nmrimg; imagenum++)
      for (bandnum=0; bandnum<inimage->nbands[imagenum]; bandnum++, tband++)
	 {
	 inimage->cblut[tband] = inimage->bands[imagenum][bandnum];
	 inimage->fdlut[tband] = imagenum; 
	 } 

/* allocate memory for image buffer */ 
   if (( inimage->buffer = (unsigned char *) malloc (inimage->ns * 
   inimage->dsizeap) ) == NULL)
      errclean ("randsel-memory", 
               "Unable to allocate memory for input image buffer"); 
}





/*******************************************************************************
NAME:     open_bsq_out 
PURPOSE:  open an output image for BSQ (random) IO and allocate data buffers,
          does not rely on an input image 
*******************************************************************************/

void open_bsq_out 
(
   struct PARBLK *pblock,   /* input:  tae parameter block */ 
   char *parmname,          /* input:  tae parm for image name.  If NULL, image 
                                       name is already in outimage. */ 
   long dtypeap,            /* input:  date type as processed by the app for
				       the output image, ignored if !writconv */
   long dtypeout,           /* inut:   image data type as output to disk image
                                       = EBYTE, EWORD, ELONG, EREAL */ 
   long writconv,           /* input:  flag. if TRUE, data will be converted 
				       to odtype upon writing to disk. */ 
   long nl,                 /* input:  nl for output image */ 
   long ns,                 /* input:  ns for output image */ 
   long nbands,             /* input:  nbands for output image */ 
   struct OUTIMAGE *outimage /* output: opened output image */
)
{

/* declare variables */ 
   long accessm=IWRITE,     /* image access mode, parm to c_eopenr() */ 
        one=1,              /* value of one */ 
	opt[2];             /* optional parm to c_eopenr(), used for data 
			       type conversion */ 
   struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 

/* get tae globals:  $delflg, $minmax, $msgtim */ 
   /* delflg */ 
      tparm = p_find (pblock, "$delflg");   
      if ( !strcmp (SVAL(*tparm,0), "YES"))
	 outimage->delflg = TRUE;
      else
	 outimage->delflg = FALSE;
   /* minmax */ 
      tparm = p_find (pblock, "$minmax");   
      if ( !strcmp (SVAL(*tparm,0), "YES"))
	 outimage->minmax = TRUE;
      else 
	 outimage->minmax = FALSE;
   /* msgtim */ 
      tparm = p_find (pblock, "$msgtim");   
      outimage->msgtim = IVAL (*tparm, 0); 

/* optionally get output image name */ 
   if (parmname != NULL && 
   c_getout (pblock, parmname, (char (*)[256])outimage->taename[0], 
            (char (*)[256])outimage->hostname[0]) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* set data type and size */
   /* dtypeout */ 
      outimage->dtypeout = dtypeout; 
   /* dtypeap */ 
      if (writconv)
	 outimage->dtypeap = dtypeap; 
      else 
	 outimage->dtypeap = outimage->dtypeout; 
   /* dsizeap */ 
      switch (outimage->dtypeap) 
	 {
	 case EBYTE:  outimage->dsizeap=EBYTESIZE;  break; 
	 case EWORD:  outimage->dsizeap=EWORDSIZE;  break; 
	 case ELONG:  outimage->dsizeap=ELONGSIZE;  break; 
	 case EREAL:  outimage->dsizeap=EREALSIZE;  break; 
	 default:  errclean ("randsel-fatal", 
	                    "Program error:  unknown image data type");
	 }

/* set variables in data structure */ 
   outimage->nl = nl; 
   outimage->ns = ns; 
   outimage->iotype = BSQIO;    /* BSQ (random) IO */ 
   outimage->totalbands = nbands; 

/* open image file */ 
   if (writconv)
      {
      opt[0] = outimage->dtypeout; 
      opt[1] = 0; 
      }
   else 
      opt[0] = 0; 
   if (c_eopenr ((struct FDESC **)outimage->fdarray, outimage->hostname[0],
		&outimage->totalbands, &one, &one, &outimage->nl,
		&outimage->ns, &accessm, &outimage->dtypeap, opt, &one)
		!=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");
   outimage->opened = TRUE; 

/* allocate memory for image buffer */ 
   if ((outimage->buffer = (unsigned char *) 
   malloc (outimage->ns * outimage->dsizeap) ) == NULL)
      errclean ("randsel-memory", 
               "Unable to allocate memory for output image buffer"); 

/* initialize min max arrays */ 
   if (outimage->minmax && 
   c_inmnmx (&outimage->dtypeap, &outimage->totalbands, outimage->minval, 
	    outimage->maxval) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");
}





/*******************************************************************************
NAME:     read_bil 
PURPOSE:  read a line of data for BILIO, and set data pointers  
*******************************************************************************/

void read_bil
(
   struct INIMAGE *inimage   /* read the next line of data for this image */
)
{

/* declare variable */ 
   long band;   /* for loop index, image band */ 

/* get data */ 
   if (c_estep ((struct GDESC **)&inimage->gdesc) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* reset pointers to the data read in */ 
   for (band=0;  band<inimage->totalbands;  band++)
      inimage->linebuf[band] = inimage->buffer + inimage->bufoff[band] * 
			       inimage->dsizeap;
}





/*******************************************************************************
NAME:     write_bil 
PURPOSE:  write a line of data for BILIO, and set data pointers 
*******************************************************************************/

void write_bil 
(
   struct OUTIMAGE *outimage   /* write the next line of data for this image */
)
{

/* declare variable */ 
   long band;   /* for loop index, image band */ 

/* get data */ 
   if (c_estep ((struct GDESC **)&outimage->gdesc) !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* reset pointers to the data read in */ 
   for (band=0;  band<outimage->totalbands;  band++)
      outimage->linebuf[band] = outimage->buffer + outimage->bufoff[band] * 
			        outimage->dsizeap;
}





/*******************************************************************************
NAME:     read_bsq 
PURPOSE:  read data from input image file, using BSQ (random) IO  
*******************************************************************************/

void read_bsq 
(
   long curline,        /* input:  starting line of block of data to read, 
			           0 relative */ 
   long curband,        /* input:  band to read, 0 relative */ 
   struct INIMAGE *inimage  /* input and output: data structure of input image 
			   to read */
)
{

/* declare variables */ 
   long imagenum,       /* image number, index to fdarray[] */ 
	one=1;          /* n image lines to read at a time */ 

/* first determine which image file to access then translate from curband as 
   input to curband relative to input image file */ 
   imagenum = inimage->fdlut[curband]; 
   curband  = inimage->cblut[curband]; 

/* convert from 0 relative to 1 relative */ 
   curline++; 

/* read data from disk image */ 
   if (c_eread ((struct FDESC **)inimage->fdarray+imagenum, &curband, &curline, 
               inimage->buffer, &one) != inimage->ns)
      errclean ("randsel-fatal", "Fatal error encountered");
}





/*******************************************************************************
NAME:     write_bsq 
PURPOSE:  write data to disk image, using BSQ (random) IO  
*******************************************************************************/

void write_bsq 
(
   long curline,           /* input:  starting line of block of data to write, 
			              0 relative */ 
   long curband,           /* input:  band to write, 0 relative */ 
   struct OUTIMAGE *outimage   /* input:  data structure of image to write */
)
{

/* declare var */ 
   long one=1;             /* n lines to write at a time */ 

/* convert from 0 relative to 1 relative */ 
   curline++; 
   curband++; 

/* write data to disk */ 
   if (c_ewrite ((struct FDESC **)outimage->fdarray, &curband, &curline, 
                outimage->buffer, &one) != outimage->ns)
      errclean ("randsel-fatal", "Fatal error encountered");
}





/*******************************************************************************
NAME:     minmax_bil 
PURPOSE:  optionally update minmax for a line of image data for 
	  BIL (sequential) IO
*******************************************************************************/

void minmax_bil
(
   struct OUTIMAGE *outimage  /* input and output:  structure of image data to 
			     update min/max for */ 
)
{

/* declare var */ 
   long band;              /* loop index to image bands */ 

/* return if min/max does not need updated */ 
   if (!outimage->minmax)
      return; 

/* error check */ 
   if (outimage->iotype != BILIO)
      errclean ("randsel-fatal", 
               "Program error:  minmax_bil() was called but iotype!=BILIO"); 

/* update minmax */ 
   for (band=0; band<outimage->totalbands; band++)
      if (c_pxmnmx ((long *)outimage->buffer + outimage->bufoff[band] * 
		   outimage->dsizeap, &outimage->ns, outimage->minval+band, 
		   outimage->maxval+band, &outimage->dtypeap) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");
}





/*******************************************************************************
NAME:     close_inimage 
PURPOSE:  close input image files and free buffers  
*******************************************************************************/

void close_inimage 
(
   struct INIMAGE *inimage   /* input:  close these files */ 
)
{

/* declare variables */
   long imagenum;        /* loop index, image number */ 

/* close image files */ 
   if (inimage->iotype == BILIO)/* close group descriptor */ 
      (void) c_egclse((struct GDESC **)&inimage->gdesc);

   else               /* close individual file descriptors */ 
      for (imagenum=0; imagenum<inimage->nmrimg; imagenum++)
	 (void) c_eclose ((struct FDESC **)inimage->fdarray+imagenum);

/* free buffers */ 
   free (inimage->buffer); 

/* set status flag, image was closed */ 
   inimage->closed = TRUE; 
}





/*******************************************************************************
NAME:     close_outimage 
PURPOSE:  close output image files and free buffers  
*******************************************************************************/

void close_outimage 
(
   struct OUTIMAGE *outimage    /* input:  close these files */ 
)
{

/* close image files */ 
   if (outimage->iotype == BILIO) /* close group descriptor */ 
      (void) c_egclse((struct GDESC **)&outimage->gdesc);

   else               /* close file descriptor */ 
      (void) c_eclose ((struct FDESC **)outimage->fdarray);

/* free buffers */ 
   free (outimage->buffer); 

/* set status flag, image was closed */ 
   outimage->closed = TRUE; 
}






/*******************************************************************************
NAME:     finish_out
PURPOSE:  finalize processing of output image:  close image files, create ddr, 
	  create history file.  This is a custom version of finish_out()
	  modified slightly for RANDSEL. 
*******************************************************************************/


void finish_out 
(
   struct PARBLK *pblock,       /* input:  tae parameter block */ 
   struct OUTIMAGE *outimage    /* input:  specs for output image */ 
)
{

/* declare variables */ 
   char notused[1][CMLEN];  /* used for dynamic tutor sessions, not applicable
                               here, required for c_hstout() */
   long bandnum,            /* for loop index:  band number */ 
        zero=0;             /* value of zero */ 
   struct BDDR bddr;        /* band-dependent record of ddr */ 
   struct DDR ddr;          /* image ddr */ 

/* close image files and free buffers */ 
   close_outimage (outimage); 

/* create ddr */ 
   c_intddr (&ddr);               /* initialize ddr including last used fields*/
   ddr.nl     = outimage->nl;         /* number of lines */ 
   ddr.ns     = outimage->ns;         /* number of samples */  
   ddr.nbands = outimage->totalbands; /* number of bands */ 
   ddr.dtype  = outimage->dtypeout;   /* data type */ 
   ddr.master_line   =1;          /* line relative to master image */
   ddr.master_sample =1;          /* sample relative to master image */
   ddr.line_inc      =1.0;        /* line increment for sampling */
   ddr.sample_inc    =1.0;        /* sample increment for sampling */
   ddr.valid[DDINCV] = VALID;     /* line/sample increments validity flag */
   if (c_getsys(ddr.system)!=E_SUCC || 
   c_putddr(outimage->hostname[0],&ddr)!=E_SUCC) 
      errclean ("randsel-fatal", "Fatal error encountered");

/* create bddr's and update with band number */ 
   c_intbdr (&bddr);          /* initialize bddr */ 
   for (bandnum=1; bandnum<=outimage->totalbands; bandnum++)
      if (bddr.bandno=bandnum, c_putbdr(outimage->hostname[0],&bddr) !=E_SUCC)
	 errclean ("randsel-fatal", "Fatal error encountered");

/* update ddr with min max, must come after writing ddr to disk */ 
   if (outimage->minmax && 
   c_svmnmx (outimage->hostname[0], &outimage->totalbands, outimage->minval, 
	    outimage->maxval, "YES") !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");

/* create history file */ 
   if (c_hstout (pblock, outimage->hostname[0], notused, &zero, "") !=E_SUCC)
      errclean ("randsel-fatal", "Fatal error encountered");
}




