#include "reclass.h"
#include "pixman.h"
#include "protos.h"

/******************************************************************************
NAME:	process_middle

FUNCTION:

PROGRAM HISTORY:
  Version	Date       Author      	   Request
  -------	----	   ------          -------
    1.0       13-Oct-83    L. Urquhart
    2.0       24-Dec-87    N. Deodhar/SAR  New LAS & conversion to C
    3.0       12-Sep-88    B. Allam/SAR    Retrocode
    3.1          Apr-89    D. Hollaren     Added static to compile on PN9050
    7.0          Nov-99    D. Hames        Remove MAXNS from the output
					   buffer "out" and modify to support 
					   images > 2gig.

ALGORITHM (FLOW):

	DO until end of image
	   Read window lines
	   Initialize values
	   Accumulate values
	   Apply weights
	   Place mode in output line
	   Write out output line
	END-DO

******************************************************************************/

void process_middle
(
    long wind[][4],		/* I: Input image window specified            */
    long fcellsiz[],		/* I: Filter cell size specified              */
    long class[],		/* I: Classes to be weighed                   */
    float weight[],		/* I: Weight given to each class              */
    long nore[],		/* I: Classes not to be smoothed              */
    long thresh,		/* I: Smoothing threshold value               */
    long mnmxflag,		/* I: Minmax flag                             */
    double *minval,		/* I/O: Minimum value                         */
    double *maxval		/* I/O: Maximum value                         */
)
{
long	bline,			/* Center line in cell                        */
	ce, nc,			/* Index counter into the filter cell         */
	cee,			/* Index counter into the filter cell         */
	ceflag[MAX_CELL_SIZ * MAX_CELL_SIZ],	/* The filter cell window     */
				/* that contains the original pixel values    */
	cenpix,			/* Pixel value of center of filter cell       */
	cline,			/* Line counter into cell                     */
	clow = -1,		/* Lowest class # with the highest frequency  */
	ctflag[NUM_CLASSES],	/* Logicals that indicate whether a certain   */
				/* class is in the filter cell                */
	dmv,			/* Horizontal center of filter cell           */
	got_one,		/* Flag used to break out of loop             */
	high,			/* Highest frequency                          */
	i,			/* Loop index                                 */
	indx,			/* Used to calculate index into arrays        */
	ink,			/* Pixel counter in the filter cell           */
	j,			/* Loop index                                 */
	k,			/* Used to hold index into an array           */
	l,			/* Used to hold index into an array           */
	loop,			/* Counter for lines to process               */
	lpt = 0,		/* Line pointer in the work buffer            */
	m,			/* Used to hold index into an array           */
	n,			/* Used to hold index into an array           */
	nt,			/* Number of pixels in filter cell            */
	offset = 0,		/* Offset into the filter cell                */
	pointer,		/* Index into the filter cell                 */
	scout,			/* Index into output buffer                   */
	sss,			/* Used to calculate index into arrays        */
	start,			/* Pixel counter in the filter cell           */
	step;			/* Number of lines in cell / 2                */

float	cflag[NUM_CLASSES];	/* Weighted class count                       */

static short *out;	        /* Temporary output buffer                    */

/*
   Allocate memory
   ---------------
*/
if ((out = (short *) malloc(wind[0][NS] * sizeof(short)))== NULL)
     iso_err ("!Error allocating dynamic memory for out", "reclass-alloc"); 

/*
   Initialize local variables
*/
nt    = fcellsiz[NL] * fcellsiz[NS];
step  = fcellsiz[NL] / 2;
cline = fcellsiz[NL] / 2 + 1;
loop  = fcellsiz[NL] + 1;
ink   = start  = fcellsiz[NS] / 2;
scout = dmv    = fcellsiz[NS] / 2 + 1;

for (i = 0; i < NUM_CLASSES; i++)
   ctflag[i] = FALSE;
/*
   Zero out flag arrays, accumulate values
   ---------------------------------------
*/
for (;;)
   {
   for (i = 0; i < NUM_CLASSES; i++)
      cflag[i]  = 0.0;
   for (i = 0; i < MAX_CELL_SIZ * MAX_CELL_SIZ; i++)
      ceflag[i] = 0;

   ce = 0;
   for (j = 0; j < fcellsiz[NS]; j++)
      {
      for (i = 0; i < fcellsiz[NL]; i++)
	 {
	 indx = (i * wind[0][NS]) + j;
	 k = buffer[indx];
	 cflag[k]  += 1.0;
	 ceflag[ce] = k;
	 ce++;
	 }
      }
   pointer = (ce + 1) / 2 + offset - 1;
   ce = 0;
/*
   Apply weights to cflag values
   -----------------------------
*/
   for (;;)
      {
      nc = 0;
      cenpix = ceflag[pointer];
      for (i = 0; i < class_count; i++)
	 {
	 j = class[i];
	 if (j < 0)
	    break;
	 cflag[j] = cflag[j] * weight[i] + 0.5;
	 }
/*
   Determine class of highest frequency
   ------------------------------------
*/
      high = 0;
      for (i = 0; i < nt; i++)
	 {
	 j = ceflag[i];
	 if (!ctflag[j])
	    {
	    ctflag[j] = TRUE;
	    k = (long)cflag[j];
	    if (k > high)
	       {
	       clow = j;
	       high = k;
	       }
	    else if (k == high)
	       {
	       if (j < clow)
		  clow = j;
	       }
	    }
	 }


/*
   Place most frequent value in out
   --------------------------------
*/
      got_one = FALSE;
      for (i = 0; i < class_count; i++)
	 {
	 if (nore[i] == cenpix)
	    {
	    got_one = TRUE;
	    break;
	    }
	 }
      if (thresh >= high || got_one)
	 out[scout - 1] = (short)cenpix;
      else
	 out[scout - 1] = (short)clow;
      scout++;
/*
   Clear out elements in ctflag
   ----------------------------
*/
      for (i = 0; i < nt; i++)
	 {
	 j = ceflag[i];
	 ctflag[j] = FALSE;
	 }
/*
   Check for end of line
   ---------------------
*/
      start++;
      if ((start + dmv) > wind[0][NS])
         break;   /* break out of inner "for (;;)" loop */
      else
	 {
/*
   Decrease values in cflag
   ------------------------
*/
	 for (i = 0; i < class_count; i++)
	    {
	    j = class[i];
	    if (j < 0)
	       break;
	    cflag[j] -= 0.5;
	    if (weight[i] != 0)
	       cflag[j] /= weight[i];
	    }

	 if ((ce + 1) > nt)
	    ce = 0;
	 cee = ce;
	 for (i = 0; i < fcellsiz[NL]; i++)
	    {
	    indx = ceflag[cee + i];
	    cflag[indx] -= 1.0;
	    }
/*
   Accumulate values in cflag for new y-points
   -------------------------------------------
*/
	 sss = start + dmv - 1;
	 for (i = 0; i < fcellsiz[NL]; i++)
	    {
	    indx = (i * wind[0][NS]) + sss;
	    k = buffer[indx];
	    cflag[k]  += 1.0;
	    ceflag[ce] = k;
	    ce++;
	    }
	 pointer += fcellsiz[NL];
	 if ((pointer + 1) > nt)
	    pointer = step + offset;
	 }
      nc++;
   }   /* bottom of inner "for (;;)" loop */
/*
   Read in next set of data
   ------------------------
*/
   start = ink;
   bline = cline;
   if (loop <= wind[0][NL])
      {
      if (c_estep ((struct GDESC **)&ingdesc) != E_SUCC)
        {
         if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
         iso_err ("!Error stepping through image", "reclass-instep");
        }
      c_pxcopy ((unsigned char *)&inbuf[inoff],(unsigned char *)&buffer[lpt],
                 &dtype, &wind[0][NS]);

      lpt += wind[0][NS];
      if ((lpt + 1) > (wind[0][NS] * fcellsiz[NL]))
	 lpt = 0;
      cline++;
      if (cline > fcellsiz[NL])
	 cline = 1;
      }

   l = (bline - 1) * wind[0][NS];
   m = l + wind[0][NS] - ink;
   n = wind[0][NS] - ink;

   c_pxcopy ((unsigned char *)&buffer[l], (unsigned char *)&out[0], &dtype, 
              &ink);
   c_pxcopy ((unsigned char *)&buffer[m], (unsigned char *)&out[n], &dtype, 
              &ink);

   scout = dmv;
/*
   Write out buffer
   ----------------
*/
   c_pxcopy ((unsigned char *)&out[0], (unsigned char *)&outbuf[outoff], &dtype,
              &wind[0][NS]);
   if (mnmxflag)
      c_pxmnmx ((long *)&outbuf[outoff], &wind[0][NS], minval, maxval, &dtype);
   if (c_estep ((struct GDESC **)&outgdesc) != E_SUCC)
     {
      if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
      iso_err ("!Error stepping through image", "reclass-outstep");
     }
   curent++;

   if (loop <= wind[0][NL])
      {
      loop++;
      offset++;
      if (offset > step)
	 offset = -step;
      }
   else
      break;
   }   /* bottom of outer "for (;;)" loop */

/*
   Free allocated memory
   ---------------------
*/
   free (out);

return;
}
