
#include <stdio.h>
#include "statio.h"

#include "las.h"
#include "util.h"
#include "protos.h"

#define NULL_CLASS   -99        /* A null value.  This is set by the PDF    */
#define NUM_CLASSES  256        /* This is retricted by the data type byte  */
#define MAX_CELL_SIZ  15        /* Maximum filter cell size                 */

struct  PARBLK vblock;          /* LAS parameter block                      */

struct DDR      inddr;          /* DDR structure                            */

short   *buffer,        	/* Temporary work buffer                    */
                *inbuf,         /* Input image I/O buffer                   */
                *outbuf;        /* Output image I/O buffer                  */

static long     dtype = EWORD,  /* Data type that we will work with         */
        one = 1,                /* 1 to be passed by reference              */
        three = 3,              /* 3 to be passed by reference              */
        zero = 0;               /* 0 to be passed by reference              */
long    ingdesc,        	/* Input image group descriptor             */
                inoff,          /* Offset into input image I/O buffer       */
                class_count,    /* Number of elements in parameter array    */
                numfiles,       /* no. of files opened                      */
                outgdesc,       /* Output image group descriptor            */
                outoff;         /* Offset into output image I/O buffer      */

extern long curent;

char    out[MAXIMG][CMLEN];     /* names of file opened              */

/******************************************************************************

NAME:	RECLASS

PURPOSE:
	Smooths a single band byte image by applying a mode filter.
	The output will be a byte image but all work is done with
	short data.

PROGRAM HISTORY:
  Version	Date       Author      	   Request
  -------	----	   ------          -------
    1.0       13-Oct-83    L. Urquhart
    2.0       24-Dec-87    N. Deodhar/SAR  New LAS & conversion to C
    3.0       12-Sep-88    B. Allam/SAR    Retrocode
    3.1       17-Dec-90    D. Etrheim/EDC  ADP #5514 Pass doubles to upddr
					   to run on pn9080.  Write TAE name
					   on complt.
    5.0	      23-Sep-92	   T. Mittan/EDC   Modified include file to eliminate
					   warning messages on link. Also
					   used standard parameter names.
    7.0       15-Nov-99	   D. Hames/EDC    Remove reference to MAXNS to support 
					   images > 2gig. Add prototypes. 
 	   
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):
	Do the standard initialization
	Get the user-specified parameters
	Set up image files for I/O
	Allocate temporary work space
	Set up processing line message
	Process the top edge of the image (move some lines as they are)
	Process the main part of the image
	Process the bottom edge of the image (move remaining lines as they are)
	Free the temporary work space
	Fix the output image DDR
	Save the min/max values, history
	Output the completion message
     
******************************************************************************/

void main
(void)
{
static long  bands[MAXIMG][MAXBND + 1];	/* Band numbers                       */
long    nimages,                /* no. of images                              */
        nbands[MAXIMG],         /* no. of bands in each image                 */
	class[NUM_CLASSES],	/* Classes to be weighed                      */
	fcellsiz[4],		/* filter cell size (LAS parameter 'WINDOW')  */
	mnmxflag = FALSE,	/* min/max boolean flag                       */
	msgtim,			/* Processing line message time interval      */
	nore[NUM_CLASSES],	/* Classes not to be smoothed                 */
	stdlun,			/* Standard output                            */
	thresh,			/* Smoothing threshold value                  */
	wind[MAXIMG][4];	/* image window                               */

double	minval,			/* Output image minimum value                 */
	maxval;			/* Output image maximum value                 */
double  y_inc, x_inc;           /* line and sample increments                 */

float	weight[NUM_CLASSES];	/* Weight given to each class                 */

char	minmax[4],		/* Minmax flag buffer                         */
	hosin[MAXIMG][CMLEN],		/* Input image file host name                 */
	hosout[CMLEN],		/* Output image file host name                */
	lasin[MAXIMG][CMLEN],		/* Input image file LAS name                  */
	lasout[CMLEN];		/* Output image file LAS name                 */

numfiles = 0;
/*
   Initiate LAS
   ------------
*/
c_inlas (&vblock, &stdlun);

/*
   Get parameters
   --------------
*/
getpar (lasin, hosin, bands, &nimages, nbands, lasout, hosout,
        class, weight, nore, &thresh, wind, fcellsiz,
        &msgtim, minmax);
/*
   Initialize minimum and maximum values
   -------------------------------------
*/
if  ((strcmp (minmax, "YES") == 0) || (strcmp (minmax, "yes") == 0))
   {
   mnmxflag = TRUE;
   c_inmnmx (&dtype, &one, &minval, &maxval);
   }
/*
   Setup image files for I/O
   -------------------------
*/
setup_files (hosin, hosout, wind, fcellsiz, bands);
/*
   Alocate temporary work space
   ----------------------------
*/
if ((buffer = (short *) malloc (fcellsiz[NL] * wind[0][NS] * sizeof (short))) == 0)
  {
   if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
   iso_err ("!Error allocating temporary work space", "reclass-alloc");
  }
/*
   Set up processing message
   -------------------------
*/
c_inittm ("RECLASS", "Processed", "line", "", "", &wind[0][NL], &zero, &msgtim, 
          &three);
curent = 1;
c_startm ();
/*
   Do estep for first output line
   ------------------------------
*/
if (c_estep ((struct GDESC **)&outgdesc) != E_SUCC)
  {
   if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
   iso_err ("!Error stepping through image", "reclass-outstep");
  }
/*
   Process the image
   -----------------
*/
process_beginning (wind, fcellsiz, mnmxflag, &minval, &maxval);

process_middle (wind, fcellsiz, class, weight, nore, thresh,
                mnmxflag, &minval, &maxval);

process_bottom (wind, fcellsiz, mnmxflag, &minval, &maxval);

c_stoptm ();

/*
   Close the image files
   ---------------------
*/
if (c_egclse ((struct GDESC **)&ingdesc) != E_SUCC)
   iso_err ("?Error closing input image file", "reclass-iclose");
if (c_egclse ((struct GDESC **)&outgdesc) != E_SUCC)
   iso_err ("?Error closing output image file", "reclass-oclose");
/*
   Free the allocated space
   ------------------------
*/
free ((char *) buffer);
free ((char *) inbuf);
free ((char *) outbuf);

/*
Update DDR
----------
*/
c_getddr((char *)hosin, &inddr);
y_inc = inddr.line_inc;
x_inc = inddr.sample_inc;
c_upddr (&vblock, hosin, hosout, &nimages, wind, bands, nbands, &y_inc, &x_inc,
         &one, &zero);

/*
   Handle image history, minmax and completion messages
   ----------------------------------------------------
*/
if (c_svmnmx (hosout, &one, &minval, &maxval, minmax) != E_SUCC)
   iso_err ("?Error writing out minmax values", "reclass-minmax");

if (c_hstout (&vblock, hosout, hosin, &one, " ") != E_SUCC)
   iso_err ("?Error writing out history", "reclass-hst");

c_complt ("RECLASS", out, &numfiles);
}
