#include "reclass.h"
#include "pixman.h"
#include "protos.h"

/******************************************************************************

NAME:	setup_files

FUNCTION:
	Open images for I/O.  Also does some error checking on the input
	image.  Both input and output images must be byte data but all work
	within the program is done with short data.

PROGRAM HISTORY:
  Version	Date       Author      	   Request
  -------	----	   ------          -------
    1.0       13-Oct-83    L. Urquhart
    2.0       24-Dec-87    N. Deodhar/SAR  New LAS & conversion to C
    3.0       12-Sep-88    B. Allam/SAR    Retrocode

ALGORITHM (FLOW):
	Make sure input image is single band and type byte
	Make sure input image window is large enough to accomodate
	    filter cell window
	Open input & output images
	Allocate space for both images
	Create access groups for both images

******************************************************************************/

void setup_files
(
    char hosin[][CMLEN],        /* I: Input image file host name              */
    char hosout[CMLEN],         /* I: Output image file host name             */
    long wind[][4],		/* I: Input image window specified            */
    long fcellsiz[],		/* I: Filter cell size specified              */
    long bands[][MAXBND+1]	/* I: Band number specified                   */
)
{
long	access,			/* Image open access                          */
	inbufsiz,		/* Size of input image I/O buffer             */
	infdesc[2],		/* Input image file descriptors               */
	outbufsiz,		/* Size of output image I/O buffer            */
	outfdesc[2],		/* Output image file descriptors              */
	opt[2],			/* Special option to write image as byte      */
	status;			/* Returned status from functions             */

/*
   Get input image info and check data type
   ----------------------------------------
*/
if (c_getddr ((char *)hosin, &inddr) != E_SUCC)
   iso_err ("!Unable to access input image DDR", "reclass-ddr");

if (inddr.dtype != EBYTE)
   iso_err ("!Input image must be byte", "reclass-inbyte");
/*
   Check image window size with filter cell size
   ---------------------------------------------
*/
if (fcellsiz[NS] > wind[0][NS])
   iso_err("!Number of pixels in window greater than image size", 
           "reclass-numsamp");

if (fcellsiz[NL] > wind[0][NL])
   iso_err("!Number of lines in window greater than image size", 
           "reclass-numbnd");
/*
   Open input image and read in as short data
   ------------------------------------------
*/
access = IREAD;
status = c_eopens ((struct FDESC **)infdesc, hosin[0], bands[0], &wind[0][SL], &
                   wind[0][SS], &wind[0][NL], &wind[0][NS], &inoff, &access, 
		   &dtype, &zero, &zero);
if (status != E_SUCC)
   iso_err("!Unable to open input image", "reclass-inopen");
infdesc[1] = 0;
/*
   Output will be converted from short to byte data
   ------------------------------------------------
*/
access = IWRITE;
opt[0] = EBYTE;
opt[1] = 0;
status = c_eopens ((struct FDESC **)outfdesc, hosout, &one, &one, &one, 
                 &wind[0][NL], &wind[0][NS], &outoff, &access, &dtype, &zero,
		 opt);
if (status != E_SUCC)
   iso_err ("!Unable to open output image", "reclass-outopen");
strcpy(out[numfiles], hosout); 
numfiles = numfiles + 1;
outfdesc[1] = 0;

if ((inbuf = (short *) c_ealloc ((struct FDESC **)infdesc, &inbufsiz)) == NULL)
   iso_err ("!Error allocating dynamic memory for input image", 
            "reclass-alloc");

if ((outbuf = (short *) c_ealloc ((struct FDESC **)outfdesc, &outbufsiz))==NULL)
   iso_err ("!Error allocating dynamic memory for output image", 
            "reclass-alloc");

if (c_egroup ((struct FDESC **)infdesc, (struct GDESC **)&ingdesc, 
                 (unsigned char *)inbuf, &inbufsiz) != E_SUCC)
   iso_err ("!Unable to set up input I/O group", "reclass-grp");

if (c_egroup ((struct FDESC **)outfdesc, (struct GDESC **)&outgdesc,
               (unsigned char *)outbuf, &outbufsiz)!=E_SUCC)
   iso_err ("Unable to set up output I/O group", "reclass-grp");

return;
}
