/*******************************************************************************

NAME:		REGLABEL	

PURPOSE:   	REGLABEL does region labeling in a LAS image.  Each 
		contiguous group of pixels with the same value is 
		assigned a unique identifier.
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
1.0	D.Nichols	2/83	Original Development
2.0	K.Creaser	11/84	Retrocode
3.0	D.Etrheim	9/88	Increase array dimensions
5.0	D.VanderZee	9/90	Re-retrocode (All new C version!)
6.0	S.Nelson	5/92	Updated user guide and error messages.
6.1	D.Eichacker	5/94	Updated open_opt[].
				
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
REGLABEL must be run under TAE.

PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "reglabel.h"

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
int fdesc[3];		/* image file descriptors used for image I/O */
int gdesc;		/* group descriptor used for image I/O */
long in_ptr[1];		/* pointer into "iobuf" for input bands */
long out_ptr[1];   	/* pointer into "iobuf" for output bands */
extern long curent;		/* current processing line */
extern long curcat;		/* current processing pass */


void main
(
  void
)
{
struct PARBLK vblock;			/* TAE parameter block */

long 	bands[1][MAXBND+1];	 	/* bands to be processed - all images*/
long    tmpbands[2];			/* for opening temp image for reading */
long	msgtim;				/* elapsed time between display msg */
long	nbands;				/* number of bands per input image */
long	nmrimg;				/* number of input images */
long	std_out;			/* LUN to standard output */
long	totbnd;				/* total number of bands */
long	window[1][4];			/* window for each input image */
long	msg_frmt = 1;			/* format flag for processing message*/
long	zero = 0;			/* number zero for passing by address */
long	one = 1;			/* number one for passing by address */
long	two = 2;			/* number two for passing by address */
long	sl = 1;				/* start line for output image */
long 	ss = 1;				/* start sample for output image */
long	rd_acc = IREAD;			/* read access flag */
long	wr_acc = IWRITE;		/* write access flag */
long	cflag = NOCOMB;			/* flag for updating ddr */
long	dtype = ELONG;			/* image data type */
long	fatal = -1;			/* fatal error flag */
long	nonfatal = 1;			/* nonfatal error flag */
long	bufsiz;				/* image i/o buffer size */
long	nfile = 1;			/* number of output files */
long 	eightway;			/* eightway connectedness check flag */
long	gross_nreg;			/* gross number of regions */
long	net_nreg;			/* net number regions--after linkage */
long	*regcls;			/* region class numbers */
long	*regcnt;			/* region counts */
long	*reglnk;			/* region linkage pointers */

static	long	open_opt[3]={0,0,0};	/* optional open parameter */

char	hosin[1][CMLEN];		/* host names of input images */
char	hosout[CMLEN];			/* host name of output image */
char	taeout[CMLEN];			/* TAE name of output image */
char	regionlt[CMLEN];		/* region labeled table name */
char	tempimg[CMLEN];			/* temporary image name */
char	openimgs[2][CMLEN];		/* names of open images for cleanup */
char	minmax[4];			/* min max flag */
unsigned char	*iobuf;			/* buffer for image i/o */

double	minval;				/* min values for output image */
double 	maxval;				/* max values for output image */
double  ln_inc = 1;			/* line increment for ddr */
double  smp_inc = 1;			/* sample increment for ddr */
/*****************************************************************************/

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);

/* Get parameters and globals
-----------------------------*/
getpar(&vblock,hosin[0],taeout,hosout,regionlt,&nmrimg,window[0],
	bands[0],&nbands,&totbnd,&eightway,&msgtim,minmax);

/* Initialize processing message utility 
----------------------------------------*/
c_inittm("REGLABEL","Processed","line","pass"," ",&window[0][2],&two,&msgtim,
	&msg_frmt);

/*****************************************************************************/
/* First pass through image data to find preliminary contiguous regions
-----------------------------------------------------------------------*/

/* Create temporary image name
------------------------------*/
strcpy(tempimg,"reglabel");
c_creat_name(tempimg,"img"); 

open_opt[0] = 0;

/* Open input and temp output images and group them for sequential i/o 
----------------------------------------------------------------------*/
if (c_eopens(&fdesc[0],hosin[0],bands[0],&window[0][0],&window[0][1],
	&window[0][2],&window[0][3],in_ptr,&rd_acc,&dtype,&zero,
	open_opt) != E_SUCC)
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);

if (c_eopens(&fdesc[1],tempimg,&nbands,&sl,&ss,&window[0][2],&window[0][3],
	out_ptr,&wr_acc,&dtype,&zero,open_opt) != E_SUCC)
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);
 
/* Allocate memory for image i/o 
--------------------------------*/
if ((iobuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","1.reglabel-alloc",&nonfatal);
   c_clean(&vblock,"reglabel",tempimg,&one);
   }

/* Group the images for i/o
---------------------------*/
if (c_egroup(fdesc,&gdesc,iobuf,&bufsiz) != E_SUCC)
   c_clean(&vblock,"reglabel",tempimg,&one); 


/* Label regions in image (first pass)  
--------------------------------------*/
if (label(iobuf,window[0][2],window[0][3],eightway,&regcls,&regcnt,
	&reglnk,&gross_nreg,&net_nreg) != E_SUCC)
   c_clean(&vblock,"reglabel",tempimg,&one); 


/* Close images in i/o group 
----------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_clean(&vblock,"reglabel",tempimg,&one);

/* Free image i/o buffer 
------------------------*/
free(iobuf);


/*****************************************************************************/
/* Sequence through the labeled image data to link all contiguous regions
-------------------------------------------------------------------------*/

/* Open temp and output images 
------------------------------*/
tmpbands[0] = 1;
tmpbands[1] = 0;
if (c_eopens(&fdesc[0],tempimg,tmpbands,&sl,&ss,&window[0][2],&window[0][3],
	in_ptr,&rd_acc,&dtype,&zero,open_opt) != E_SUCC)
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);

/* Determine data type for output image  
---------------------------------------*/
if (net_nreg <= 255)
   open_opt[0] = EBYTE;
else if (net_nreg <= 32767)
        open_opt[0] = EWORD;
     else
        open_opt[0] = ELONG;

open_opt[1] = 0;

if (c_eopens(&fdesc[1],hosout,&nbands,&sl,&ss,&window[0][2],&window[0][3],
	out_ptr,&wr_acc,&dtype,&zero,open_opt) != E_SUCC)
   c_clean(&vblock,"reglabel",tempimg,&one);

/* Copy image names into array for possible cleanup
---------------------------------------------------*/
strcpy(openimgs[0],tempimg);
strcpy(openimgs[1],hosout);
 
/* Allocate memory for image i/o 
--------------------------------*/
if ((iobuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","2.reglabel-alloc",&nonfatal);
   c_clean(&vblock,"reglabel",openimgs,&two);
   }

/* Group the images for i/o
---------------------------*/
if (c_egroup(fdesc,&gdesc,iobuf,&bufsiz) != E_SUCC)
   c_clean(&vblock,"reglabel",openimgs,&two);

/* Link contiguous flagged regions and write out region description lt 
----------------------------------------------------------------------*/
if (link_region(regionlt,hosin[0],iobuf,window[0][2],window[0][3],&regcls,
	&regcnt,&reglnk,&gross_nreg) != E_SUCC)
   c_clean(&vblock,"reglabel",openimgs,&two);

/* Close images in i/o group 
----------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_clean(&vblock,"reglabel",openimgs,&two);

/* Free image i/o buffer 
------------------------*/
free(iobuf);


/*****************************************************************************/
/* Finish up processing with housekeeping chores 
------------------------------------------------*/
/* Update DDR for output image
------------------------------*/
if (c_upddr(&vblock,hosin[0],hosout,&nmrimg,window[0],bands[0],&nbands,
	&ln_inc,&smp_inc,&totbnd,&cflag) != E_SUCC)
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);

/* if minmax to be tracked, then store them
-------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   minval = 1;
   maxval = net_nreg; 
   if (c_svmnmx(hosout,&totbnd,&minval,&maxval,minmax) != E_SUCC)
      c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);
   }
 
/* Update history file 
----------------------*/
c_hstout(&vblock,hosout,hosin[0],&nmrimg," ");

/* Delete temporary image file
------------------------------*/
c_delfil(tempimg);

/* Output completion message 
----------------------------*/
c_complt("REGLABEL",taeout,&nfile);
}
