/****************************************************************************

NAME:				RELIEF

PURPOSE:  Performs a correction to the input locations of control points to 
	  correct for relief displacement.  Assumption is that no orientation 
	  has been made to original image orientation.  This is not the case 
	  with Level 1b data which has been slightly rotated.  If in this 
  	  case, this correction will be an estimation.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	         CODE     REASON
-------	 ----	------	         ----     -----------------------------
  1.0     3/88  J. Thormodsgard  S&AB     Implement on UNIX/TAE (FORTRAN)
		  & G. Kelly
  5.0	 11/89	D. Steinwand     CSB      LAS 5.0 C conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize LAS processing
	Get the message timer and intialize processing message
	Get the input TPL file name
	Get the output TPL file name
	Get the TM quad number parameter
	Get the satellite height parameter
	Get the inclination angle parameter
	Get the field-of-view parameter
	Get the pitch parameter
	Get the pixel size parameter
	Get the Datum parameter
	Get the Datum units parameter; convert to meters if necessary
	Get the PRINTER options
	Report parameters to user
	Write the table header to the print file
	Open the input TPL file
	Open the output TPL file
	Half the field-of-view parameter & convert others to radians
	Start the message timer
	Loop until EOF
	   Read a control point record
	   Check status; abort if an error occured; break out of loop if EOF
	   Perform the relief displacement correction in the sample direction
	    (refer to the user's guide for the algorithm)
	   If using TM data, the pitch parameter should be zero and therefore
	    there should be no change in the line direction.  If using SPOT
            data, however, the panchromatic is imaged 15 km ahead of the
    	    multispectral.  As the scanlines are imaging topographic relief
	    ahead on SPOT panchromatic, a feature will be imaged in a later
	    scanline than it should.  An adjustment to the line coordinate
     	    is therefore applied here.
	   Report results to the user
	   Write to the output TPL file
	Stop the message timer
	Close all TPL files
	Write the standard completion message and exit
	
ALGORITHM REFERENCES:
	The relief displacement algorithm was based on prior work done
	at the EROS Data Center by J. Thormodsgard, G. Kelly, and L. Nady, 
        and refined with "Evaluation of EDC's Algorithm for Correction of 
        Control Points for Terrian Parallax", by Robert Wolfe, ST Systems 
        Corp (STX), Lanham, Maryland; done under USGS purchase order 
        88-211494., 10/88
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"

#define R	6378206.4	/* Earth radius */

main()		
{
struct PARBLK vblk;		/* TAE parameter block */
struct VARIABLE *var;		/* Pointer to a TAE variable in vblk */
struct TPLHEAD header;		/* Tie point location file header data */
struct TPLDATA tpldata;		/* Tie point location file data record */

struct PTDESC *in_tpl;		/* Input TPL file descriptor */
struct PTDESC *out_tpl;		/* Output TPL file descriptor */

double sathgt;		/* Satellite height in meters */
double h;		/* Earth radius plus satellite height */
double theta;		/* Central earth angle between beginning of scanline 
			   and satellite nadir */
double look;		/* Look-angle of detector where point was imaged */
double cons;		/* fov/2 in radians */
double pitch;		/* Angle of pitch of sensor scanline from nadir, 
			     = 0.53 for SPOT P 
			     = -0.53 for SPOT XS, 
			     = 0. for Landsat  */
double ps;		/* Pixel size */
double datum;		/* Datum in meters */
double relief;		/* Amount of relief displacement in meters */
double d;		/* Distance on earth's surface; nadir to scanline */
double dp;		/* Dist on earth's surface; satellite nadir to point */
double l;		/* Length of line of sight-distance of beginning of 
			   scanline to satellite */
double lp;		/* Length of line of sight-distance of control point 
			   from satellite--does not take into account the 
		           elevation of the point */
double lp2;		/* Length of line of sight-distance of control point 
			   at its elevation from satellite */
double thetap;		/* Central earth angle between control point and 
			   satellite nadir--does not take into account the 
		           elevation of the point */
double rel;		/* Earth radius plus elevation of control point */
double incd;		/* Incidence angle of HRV sensor in degrees (found on 
			   SPOT summary sheets where L makes INCD positive and 
			   R makes INCD negative)   note: INCD-2.065 is the 
			   look-angle of the first CCD detector */
double fov;		/* Angle between first and last detector or sample of
			   scanline */
double thetap2;		/* Central earth angle between control point and 
			   satellite nadir--takes elevation of control point 
			   into account */
double tmp;		/* Temporary variable for computations */
double ssamp;		/* Input image sample coordinate */
double xsamp;		/* Input image sample corrected for relief */
double xline;		/* Input image line corrected for relief */
long term, file, ln_pr;	/* PRINT flags */
long quad;		/* TM quadrant 1-4 */
long ptyp = TPL;	/* Type of tie point file */
long acc;		/* Access flag */
long msgtim;		/* Message timer */
long std_out;		/* Standard output LUN */
long status;		/* Function return status */
long fatal = -1;	/* Fatal flag for errmsg */
long zero = 0;		/* Constant of zero */
long two = 2;		/* Constant of two */
long num_out;		/* Number of output files */
char in[CMLEN];		/* The input Tie Point Location file name */
char out[CMLEN];	/* The output Tie Point Location file name */
char tae_name[2][CMLEN];/* TAE names of output files (TPL & report) */
char name[CMLEN];	/* Name for PRINT parameter */
char filename[CMLEN];	/* Name for PRINT parameter */
char host_name[CMLEN];	/* Name for PRINT parameter */

FILE *fptr;		/* Pointer to output file (report file) */

extern long curent;
extern long curcat;

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the message timer and set up processing message
  ---------------------------------------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);

c_inittm("RELIEF","Processed","point number","","",&two,&two,&msgtim,&two);

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INTL", &zero, in, "tpl", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","relief-fatal",&fatal);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk, "OUTTL", &zero, out, "tpl", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","relief-fatal",&fatal);

/* Get other parameters
  --------------------*/
var = p_find(&vblk, "QUADNUM");
quad = IVAL(*var,0);
var = p_find(&vblk, "SATHT");
sathgt = RVAL(*var,0);
var = p_find(&vblk, "INCANG");
incd = RVAL(*var,0);
var = p_find(&vblk, "FOVIEW");
fov = RVAL(*var,0);
var = p_find(&vblk, "PITCH");
pitch = RVAL(*var,0);
var = p_find(&vblk, "PIXSIZ");
ps = RVAL(*var,0);
var = p_find(&vblk, "DATUM");
datum = RVAL(*var,0);
var = p_find(&vblk, "DATUNIT");
if (strcmp(SVAL(*var,0),"FEET") == 0) datum *= 0.3048;

/* Get the print option.  Make a file name (if neccessary) and open the
   temporary file
  --------------*/
strcpy(host_name, "relief");
status = c_setprt(&vblk, &term, &file, &ln_pr, host_name, filename);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","relief-fatal",&fatal);

acc = IUPDATE;
num_out = 1;
if (file == 1) 
   {
   num_out = 2;
   c_mknam(filename, name, "prt", &acc);
   }
else strcpy(name, " ");
fptr = fopen(host_name, "w");

/* Print operating parameters to report
  ------------------------------------*/
fprintf(fptr,"\nInput point file:   %s\n", in);
fprintf(fptr,"Output point file:  %s\n\n", out);
fprintf(fptr,"TM code (3 if not tm sensor or full quad):      %d\n",quad);
fprintf(fptr,"Satellite height in meters:             %12.2f\n",sathgt);
fprintf(fptr,"Sensor orientation angle in degrees:        %9.3f\n",incd);
fprintf(fptr,"Sensor field of view in degrees:            %9.3f\n",fov);
fprintf(fptr,"Scanline pitch in degrees:                  %9.3f\n",pitch);
fprintf(fptr,"Sensor pixel size in meters:               %9.2f\n",ps);
fprintf(fptr,"Datum in meters:                           %9.2f\n\n",datum);

/* Write table header to printer file
  ----------------------------------*/
fprintf(fptr, "                INPUT               OUTPUT                   REVISED INPUT\n");
fprintf(fptr, "POINT       LINE    SAMPLE      LINE    SAMPLE    ELEV       LINE    SAMPLE\n");
fprintf(fptr, "------   -----------------   -----------------  -------   -----------------\n");

/*  Open the input control file
  ----------------------------*/
acc = IREAD;
if(c_ptopen(&in_tpl, in, &ptyp, &acc, &header) != E_SUCC)
	c_errmsg("Fatal error encountered","relief-fatal",&fatal);

/* Open the new output control file
  --------------------------------*/
acc = IWRITE;
if(c_ptopen(&out_tpl, out, &ptyp, &acc, &header) != E_SUCC)
	c_errmsg("Fatal error encountered","relief-fatal",&fatal);

/* Half the field-of-view of the sensor and convert all angles to radians
  ----------------------------------------------------------------------*/
incd = incd * D2R;
cons = fov / 2.0 * D2R;
pitch = pitch * D2R;
h = R + sathgt;

/*  Read off diskfile the ground control point.
  --------------------------------------------*/
curent = 1;
c_startm();
curent = 0;
for (;;)
   {
   status = c_ptio(&in_tpl, &tpldata);
   if (status == E_EOF) break;
   if (status == E_FAIL) 
	c_errmsg("Fatal error encountered","relief-fatal",&fatal);

   ssamp = tpldata.sea_coord[1];
   if ((quad == 2) || (quad  == 4)) ssamp = ssamp + 2747.0;
   rel = R + tpldata.elevation;
   tmp = (h / R) * sin(incd - cons);
   tmp *= tmp;
   l = h * cos(incd - cons) - R * sqrt(1. - tmp);
   theta = asin((l / R) * sin(incd - cons));
   d = R * theta;
   dp = d + (ssamp - 1) * ps;
   thetap = dp / R;
   lp = sqrt((R * R) + (h * h) - (2 * R * h * cos(thetap)));
   look = asin((R / lp) * sin(thetap));
   tmp =(h / rel) * sin(look);
   tmp *= tmp;
   lp2 = h * cos(look) - rel * sqrt(1. - tmp);
   thetap2 = asin(lp2 / rel) * sin(look);
   relief = (tpldata.elevation - datum) * tan(look + thetap2);
   xsamp = tpldata.sea_coord[1] - (relief / ps);
   relief = tan(pitch) * (tpldata.elevation - datum);

/*  If TM, the pitch = 0 and therefore there is no change to the line location.
    If SPOT, the panchromatic is imaged 15 km ahead of the multispectral.  As 
    the scanlines are imaging topographic relief ahead on SPOT pan, i.e., a 
    feature will be imaged in a later scanline than it should, one must 
    subtract RELIEF from INLINE to correct the SPOT scanline.
  ----------------------------------------------------------*/
   xline = tpldata.sea_coord[0] - relief / ps;

   fprintf(fptr,"%-6s %9.2f %9.2f %9.2f %9.2f %8.2f %9.2f %9.2f\n",
	tpldata.pt_id, tpldata.sea_coord[0], tpldata.sea_coord[1],
	tpldata.ref_coord[0], tpldata.ref_coord[1], tpldata.elevation,
	xline, xsamp);

   tpldata.sea_coord[0] = xline;
   tpldata.sea_coord[1] = xsamp;
   curent++;
   status = c_ptio(&out_tpl, &tpldata);
   if (status != E_SUCC) 
	c_errmsg("Fatal error encountered","relief-fatal",&fatal);
   }

c_stoptm();
c_ptclse(&in_tpl);
c_ptclse(&out_tpl);

/* Write the standard completion message and exit
  ----------------------------------------------*/
strcpy(tae_name[0], out);
strcpy(tae_name[1], name);
c_complt("RELIEF", tae_name, &num_out);
}
