/******************************************************************************

NAME:				NADIRLINE

PURPOSE: Uses a polynomial to find the center point of an image in
	 projection space.  Reads a labeled table file to get the line sample
	 corners of the image data.


PROGRAM HISTORY:
  Version       Date	Author		Request
  -------       ----	------		-------
   1.0		6/95	S. Nelson	Original Development

******************************************************************************/

/* include files, global parameters, and definitions for FDHR2DDR    */

#include <math.h>
#include "las.h"
#include "ltable.h"
#include "labserv.h"
#include "protos.h"

int nadirline
(
    char *inlt,		/* name of the image */
    double *slope,			/* slope of nadir */
    double *y_inter			/* y-intercept of nadir */
)
{
/* file names 
  ----------*/
long nonfatal = 1;		/* non fatal error message */
long mode = 0;			/* flag value for lsstat */

double ul_line;			/* Upper left corner line */
double ul_samp;			/* Upper left corner sample */
double ur_line;			/* Upper right corner line */
double ur_samp;			/* Upper right corner sample */
double ll_line;			/* Lower left corner line */
double ll_samp;			/* Lower left corner sample */
double lr_line;			/* Lower right corner line */
double lr_samp;			/* Lower right corner sample */
double top_center_y;		/* top center line of valid image */
double top_center_x;		/* top center sample of valid image */
double bot_center_y;		/* bottom center line of valid image */
double bot_center_x;		/* bottom center sample of valid image */

char ltname[CMLEN];
   
struct TAB_DEF tab;     /* Tabular structure of labeled table */
struct VECTOR *lineptr;	/* Pointer to line field */
struct VECTOR *sampptr;	/* Pointer to sample field */

/* get labeled table name
  --------------------*/
c_lsmknm(inlt, "lt", ltname);
if (c_lsstat(ltname,&mode) != E_SUCC)
   {
   c_errmsg("Error locating labeled table file","relief2-ltlocate",&nonfatal);
   return(E_FAIL);
   }

/* Open the labeled table file & retrieve the label
  ------------------------------------------------*/
open_tab(&tab, ltname, IREAD);
get_vector(&tab);

/* Determine pointers to fields in the file
  ----------------------------------------*/
lineptr = get_field_ptr(&tab, "LINE");
sampptr = get_field_ptr(&tab, "SAMPLE");

/* Get record and assign line/sample values to the matrix
  ------------------------------------------------------*/
if(get_record(&tab) == E_FAIL)
  {
  c_errmsg("Error reading labeled table file","relief2-ltread",&nonfatal);
  return(E_FAIL);
  }
ul_line = (double)(*lineptr->ptr.i4);
ul_samp = (double)(*sampptr->ptr.i4);

if(get_record(&tab) == E_FAIL)
  {
  c_errmsg("Error reading labeled table file","relief2-ltread",&nonfatal);
  return(E_FAIL);
  }
ur_line = (double)(*lineptr->ptr.i4);
ur_samp = (double)(*sampptr->ptr.i4);

if(get_record(&tab) == E_FAIL)
  {
  c_errmsg("Error reading labeled table file","relief2-ltread",&nonfatal);
  return(E_FAIL);
  }
lr_line = (double)(*lineptr->ptr.i4);
lr_samp = (double)(*sampptr->ptr.i4);

if(get_record(&tab) == E_FAIL)
  {
  c_errmsg("Error reading labeled table file","relief2-ltread",&nonfatal);
  return(E_FAIL);
  }
ll_line = (double)(*lineptr->ptr.i4);
ll_samp = (double)(*sampptr->ptr.i4);

close_tab(&tab);

/* find the top/bot point in the center of the valid image data
  ------------------------------------------------------------*/
top_center_y = (ul_line + ur_line) / 2;
top_center_x = ((ur_samp - ul_samp) / 2) + ul_samp;

bot_center_y = (ll_line + lr_line) / 2;
bot_center_x = ((lr_samp - ll_samp) / 2) + ll_samp;

/* find the slope and y intercept
  ------------------------------*/
*slope = (top_center_y - bot_center_y) / (top_center_x - bot_center_x);
*y_inter = -(*slope) * top_center_x + top_center_y;
*y_inter = -(*slope) * bot_center_x + bot_center_y;

return(E_SUCC);
}
