/************************************************************************

NAME:                          NADIRPIX 

PURPOSE:  This routine finds the approximate nadir pixel of a P-product
	  given the center latitude of the scene and the line number of
	  interest.  Assumes the pass to be descending.

**********************************************************************/
#include <math.h>
#include "las.h"
#include "constants.h"
#include "protos.h"

void nadirpix
(
    double sclat,	/* center latitude			*/
    long linenum, 	/* current line number			*/
    long *nadpix,	/* The center pixel (return)		*/
    long *ns,		/* Number of samples in scene		*/
    long *nl,		/* Number of lines in scene		*/
    long *scwidth,	/* Number of valid samples in scene 	*/
    double *pix	/* pixel size				*/
)
{
double nsamps,  	/* number samples in a P-product 	*/
       nlines,  	/* number of lines in a P-product 	*/
       passtime, 	/* total seconds in a P-product 	*/
       scenewidth, 	/* total sample of image in a P-product */
       re,         	/* radius of the earth in meters 	*/
       pixsize,    	/* pixel size in a P-product 		*/
       incl,		/* Orbital inclination of satellite 	*/
       head,		/* Heading angle of satellite 		*/
       shift,      	/* pixel shift due rotation of earth 	*/
       offset,     	/* padding to fill line  		*/
       slope,      	/* pixel rate of change due rotation of earth */
       intercept;  	/* starting location of nadir at line 1 */

passtime = 25.1669;
nsamps = (double) *ns;
nlines = (double) *nl;
scenewidth = (double) *scwidth;
pixsize = *pix;
re = 6367000.0;
incl = 98.0 * RPD;

/* Calculate heading of the satellite
-----------------------------------*/
head = asin(cos(incl) / cos(sclat));

/* Calculate the image shift from the edges
-----------------------------------------*/
shift = passtime * WE * re * cos(sclat) * cos(head) / pixsize;
offset = (nsamps - (shift + scenewidth)) / 2.0;

/* Calculate the slope and intercept of the center line
-----------------------------------------------------*/
slope = -shift / nlines;
intercept = offset + shift + scenewidth / 2.0 - 1.0;

/* Calculate the center (Nadir) pixel
-----------------------------------*/
*nadpix = (long) (intercept + slope * linenum);
}
