/****************************************************************************

NAME:				RELIEF2

PURPOSE:  Performs a correction to the input locations of control points to 
	  correct for relief displacement in TM imagery.  All corrections are
	  to a datum of zero, that is right on the reference ellipsoid.

VERSION	 DATE	AUTHOR
-------	 ----	------	
  1.0	 8/93	D. Steinwand			Original development
  2.0	 9/93   T. Mittan			Allowed to retrieve
						center LAT from TPL file.

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "constants.h"
#include "util.h"
#include "protos.h"

int main
(void)		
{
struct TPLHEAD header;		/* Tie point location file header data 	*/
struct TPLDATA tpldata;		/* Tie point location file data record 	*/

struct PTDESC *in_tpl;			/* Input TPL file descriptor 		*/
struct PTDESC *out_tpl;			/* Output TPL file descriptor 		*/

double corners[4];		/* the four corners defining the area	*/
double datum;			/* datum				*/
double slope;			/* slope of nadir 			*/
double xsamp;			/* Input image sample corrected for relief */
double y_inter;			/* y-intercept of nadir			*/
long geo = 0;			/* geographic projection		*/
long ptyp = TPL;		/* Type of tie point file 		*/
long acc;			/* Access flag 				*/
long msgtim;			/* Message timer 			*/
long status;			/* Function return status 		*/
long fatal = -1;		/* Fatal flag for errmsg 		*/
long nonfatal = 1;		/* Nonfatal flag for errmsg		*/
long zero = 0;			/* Constant of zero 			*/
long num_out;			/* Number of output files 		*/
long nadir;			/* Nadir pixel 				*/
long prtprm[2];			/* parameters for c_trans		*/
char inlt[CMLEN];		/* Input labeled table file		*/
char fname[CMLEN];		/* parameters for c_trans		*/
long units;			/* projection units			*/
long nsamps;			/* number of samples in scene		*/
long nlines;			/* number of lines in scene		*/
long option;			/* Subcommand option			*/
long scenewidth;		/* number of valid samples in scene	*/
double pixsize;			/* pixel size				*/
double cslat = 0.0;		/* Scene Center Latitude 		*/
char in[CMLEN];			/* The input Tie Point Location file name */
char out[CMLEN];		/* The output Tie Point Location file name */
char outname[2][CMLEN]; 	/* Names of output files (TPL & report) */
char name[CMLEN];		/* Name for PRINT parameter 		*/
char host_name[CMLEN];		/* Name for PRINT parameter 		*/
FILE *fptr;			/* Pointer to output file (report file) */

extern long curent;


/*  Retrieve TAE parameters
-------------------------*/
getpar(in,out,host_name,&msgtim,&nsamps,&nlines,&scenewidth,
       &pixsize,&datum,&fptr,inlt,&option);

/*  Open the input control file
  ----------------------------*/
acc = IREAD;
if(c_ptopen(&in_tpl, in, &ptyp, &acc, &header) != E_SUCC)
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);

/* Open the new output control file
  --------------------------------*/
acc = IWRITE;
if(c_ptopen(&out_tpl, out, &ptyp, &acc, &header) != E_SUCC)
	{
	c_ptclse(&in_tpl);
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
	}

/* Call function to get center lat for non-standard formats.  The corners
   are passed in this way to be consistant with the cornres on the ddr
  ----------------------------------------------------------------------*/
if (option == NONSTAND)
    {
    if (nadirline(inlt, &slope, &y_inter) != E_SUCC)
	{
	c_ptclse(&in_tpl);
	c_ptclse(&out_tpl);
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
	}
    }
else
    {
    /* Check that the projection information in the header is valid
      ------------------------------------------------------------*/
    if (header.proj_valid == INVAL)
	{
	c_ptclse(&in_tpl);
	c_ptclse(&out_tpl);
	c_errmsg("Projection information not available from header",
						"relief2-noproj",&nonfatal);
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
	}

    /*  Calculate the center lat for standard formats
      -----------------------------------------------*/
    corners[0] = (header.corners[0] + header.corners[4]) / 2;
    corners[1] = (header.corners[2] + header.corners[6]) / 2;
    corners[2] = (header.corners[1] + header.corners[3]) / 2;
    corners[3] = (header.corners[5] + header.corners[7]) / 2;

    corners[0] = (corners[0] + corners[1]) / 2;
    corners[1] = (corners[2] + corners[3]) / 2;

    if ((strcmp(header.units,"radians") == 0) || 
					(strcmp(header.units,"RADIANS") == 0))
	units = 0;
    if((strcmp(header.units,"feet") == 0) || (strcmp(header.units,"FEET") == 0))
	units = 1;
    if ((strcmp(header.units,"meters") == 0) || 
					(strcmp(header.units,"METERS") == 0))
	units = 2;
    if ((strcmp(header.units,"seconds") == 0) || 
					(strcmp(header.units,"SECONDS") == 0))
	units = 3;
    if ((strcmp(header.units,"degrees") == 0) || 
					(strcmp(header.units,"DEGREES") == 0))
	units = 4;
    if ((strcmp(header.units,"dms") == 0) || (strcmp(header.units,"DMS") == 0))
	units = 5;

    prtprm[0] = FALSE;
    prtprm[1] = FALSE;

    /* The center lat must be in radians
    ----------------------------------*/
    if ((status = (c_transinit(&header.code,&units,&header.zone,&header.datum,
		header.projprms,&geo,&zero,&header.zone,&header.datum,
		header.projprms,prtprm,fname))) == E_FAIL)
	{
	c_ptclse(&in_tpl);
	c_ptclse(&out_tpl);
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
	}

    if ((status = (c_trans(&header.code,&units,&geo,
		&zero,&corners[1],&corners[0],
		&corners[3], &corners[2]))) != E_SUCC)
	{
	c_ptclse(&in_tpl);
	c_ptclse(&out_tpl);
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
	}

    cslat = corners[2];
    }

/*  Read off disk file the ground control point and process.
  --------------------------------------------------------*/
curent = 1;
c_startm();
curent = 0;
for (;;)
    {
    status = c_ptio(&in_tpl, &tpldata);
    if (status == E_EOF) break;
    if (status == E_FAIL) 
	{
	c_ptclse(&in_tpl);
	c_ptclse(&out_tpl);
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
	}

    /* Calculate the non-standard format scene center pixel for this line
       Note:  This algorithm is set up to truncate for best results.
    --------------------------------------------------------------------*/
    if (option == NONSTAND)
	{
	nadir = (long) (((long)tpldata.sea_coord[0] - y_inter) / slope);
	}

    /* Calculate the MSS-P or TM-P scene center pixel for this line
    --------------------------------------------------------------*/
    else
	nadirpix(cslat,(long)tpldata.sea_coord[0],&nadir,&nsamps,&nlines,
				&scenewidth,&pixsize);

    /* Adjust the pixel location due to relief
    ----------------------------------------*/
    xsamp = terrain_adjust(tpldata.sea_coord[1],nadir,(short)tpldata.elevation,
			  &datum,&pixsize);
   
    /* Report results
    ---------------*/
    fprintf(fptr,"%-6s %9.2f %9.2f %9.2f %9.2f %8.2f %9.2f %9.2f\n",
	tpldata.pt_id, tpldata.sea_coord[0], tpldata.sea_coord[1],
	tpldata.ref_coord[0], tpldata.ref_coord[1], tpldata.elevation,
	tpldata.sea_coord[0], xsamp);

    tpldata.sea_coord[1] = xsamp;
    curent++;
    status = c_ptio(&out_tpl, &tpldata);
    if (status != E_SUCC) 
	{
	c_ptclse(&in_tpl);
	c_ptclse(&out_tpl);
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
	}
    }

c_stoptm();
c_ptclse(&in_tpl);
c_ptclse(&out_tpl);

/* Write the standard completion message and exit
  ----------------------------------------------*/
num_out = 1;
strcpy(outname[0], out);
strcpy(outname[1], name);
c_complt("RELIEF2", outname, &num_out);

return EXIT_SUCCESS;

}
