/****************************************************************************

NAME:				RELIEFX

PURPOSE:  Performs a correction to the input locations of control points to 
	  correct for relief displacement in CCT-X imagery.  All corrections 
          are to a elevation (usually zero).

VERSION	 DATE	AUTHOR
-------	 ----	------	
  1.0	 4/94	D. Steinwand
  6.0	 10/94	T. Mittan			Updated to standards

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "lasmath.h"
#include "protos.h"

#define NADIR_PIX 1610		/* Nadir pixel value			*/

int main
(void)		
{
struct PARBLK vblk;             /* TAE parameter block */
struct TPLHEAD header;		/* Tie point location file header data */
struct TPLDATA tpldata;		/* Tie point location file data record */
struct GEOGRID grid;		/* Geometric mapping grid data */

double bx[4],			/* x output for QR routines		*/
       by[4],			/* y output for QR routines		*/
       v[4],			/* work vector for QR routines		*/
       A[16];			/* input matrix for QR routines		*/
double in_ulx,			/* input upper left x coordinate	*/
       in_urx,			/* input upper right x coordinate	*/
       in_lrx,			/* input lower right x coordinate	*/
       in_llx;			/* input lower left x coordinate	*/
double in_uly,			/* input upper left y coordinate	*/
       in_ury,			/* input upper right y coordinate	*/
       in_lry,			/* input lower right y coordinate	*/
       in_lly;			/* input lower left y coordinate	*/
double out_ulx,			/* output upper left x coordinate	*/
       out_urx,			/* output upper right x coordinate	*/
       out_lrx,			/* output lower right x coordinate	*/
       out_llx;			/* output lower left x coordinate	*/
double out_uly,			/* output upper left y coordinate	*/
       out_ury,			/* output upper right y coordinate	*/
       out_lry,			/* output lower right y coordinate	*/
       out_lly;			/* output lower left y coordinate	*/
double xline,			/* approximate line 			*/
       xsamp,			/* approximate sample 			*/
       new_xsamp;		/* new estimated sample			*/
double new_line, 		/* new output line			*/
       new_samp;		/* new output sample			*/
double elev;			/* elevation				*/
struct PTDESC *in_tpl;			/* Input TPL file descriptor 		*/
struct PTDESC *out_tpl;			/* Output TPL file descriptor 		*/
long term, file, ln_pr;		/* PRINT flags 				*/
long ptyp = TPL;		/* Type of tie point file 		*/
long acc;			/* Access flag 				*/
long status;			/* Function return status 		*/
long fatal = -1;		/* Fatal flag for errmsg 		*/
long ii;			/* Loop counter 			*/
long num_out = 1;		/* Number of output files 		*/
long nadir;			/* Nadir pixel 				*/
char intpl[CMLEN];		/* The input Tie Point Location file name */
char outtpl[CMLEN];		/* The output Tie Point Location file name */
char tae_name[2][CMLEN];	/* TAE names of output files (TPL & report) */
char grid_name[CMLEN];		/* Name of geometric mapping grid file 	*/
char print_name[CMLEN];       	/* Name of the file to be printed       */
char name[CMLEN];             	/* Name of output print file            */

FILE *fptr;			/* Pointer to output file (report file) */

extern long curent;		/* Current tie point number		*/

/* Get the paramaters
-------------------*/
getpar(&vblk, intpl, outtpl, &elev, grid_name, &grid, &term, &file, &ln_pr, 
       print_name, name);

/* Fill output name array
-----------------------*/
strcpy(tae_name[0], outtpl);

/* Print operating parameters to report
-------------------------------------*/
fptr = fopen(print_name, "w");
fprintf(fptr,"\nInput point file:   %s\n", intpl);
fprintf(fptr,"Output point file:  %s\n\n", outtpl);
fprintf(fptr,
 "All points are corrected to the reference ellipsiod + %f meters\n\n",elev);

/* Write table header to printer file
-----------------------------------*/
fprintf(fptr, "                INPUT               OUTPUT                   REVISED INPUT\n");
fprintf(fptr, "POINT       LINE    SAMPLE      LINE    SAMPLE    ELEV       LINE    SAMPLE\n");
fprintf(fptr, "------   -----------------   -----------------  -------   -----------------\n");

/*  Open the input TPL file
-------------------------*/
acc = IREAD;
if(c_ptopen(&in_tpl, intpl, &ptyp, &acc, &header) != E_SUCC)
	c_errmsg("Fatal error encountered","reliefx-fatal",&fatal);

/* Open the new output TPL file
-----------------------------*/
acc = IWRITE;
if(c_ptopen(&out_tpl, outtpl, &ptyp, &acc, &header) != E_SUCC)
	c_errmsg("Fatal error encountered","reliefx-fatal",&fatal);

/*  Read off diskfile the ground control point.
---------------------------------------------*/
nadir = NADIR_PIX;
curent = 1;
c_startm();
curent = 0;
for (;;)
   {
   status = c_ptio(&in_tpl, &tpldata);
   if (status == E_EOF) break;
   if (status == E_FAIL) 
        {
        fclose(fptr);
        strcpy(tae_name[1], print_name);
        num_out++;
        c_clean(&vblk,"RELIEFX",tae_name , &num_out);
        }

   /* Find tie point location in the CCT-x image
   -------------------------------------------*/
   grid_eval(&grid,tpldata.sea_coord[1],tpldata.sea_coord[0],&xsamp,&xline);

   /* Adjust for the effects of terrain
   ----------------------------------*/
   new_xsamp=terrain_adjust(xsamp, nadir, tpldata.elevation, elev);

   /* Calculate mapping back into the space in which control was selected
   --------------------------------------------------------------------*/
   out_ulx = tpldata.sea_coord[1] - 10.0;
   out_uly = tpldata.sea_coord[0] - 10.0;
   out_urx = tpldata.sea_coord[1] + 10.0;
   out_ury = tpldata.sea_coord[0] - 10.0;
   out_llx = tpldata.sea_coord[1] - 10.0;
   out_lly = tpldata.sea_coord[0] + 10.0;
   out_lrx = tpldata.sea_coord[1] + 10.0;
   out_lry = tpldata.sea_coord[0] + 10.0;
   grid_eval(&grid,out_ulx,out_uly,&in_ulx,&in_uly);
   grid_eval(&grid,out_urx,out_ury,&in_urx,&in_ury);
   grid_eval(&grid,out_llx,out_lly,&in_llx,&in_lly);
   grid_eval(&grid,out_lrx,out_lry,&in_lrx,&in_lry);

   /* Calculate a set of first order polynomial coefficients which map input
      space CCT-x coordinates in the neighborhood of the tie point to output
      space (the space the tiepoints were selected in) line,sample coordinates
   -------------------------------------------------------------------------*/
   for(ii = 0; ii < 4; ii++) 
      A[ii] = 1.0;

   A[4]=in_ulx;
   A[5]=in_urx;
   A[6]=in_lrx;
   A[7]=in_llx;

   A[8]=in_uly;
   A[9]=in_ury;
   A[10]=in_lry;
   A[11]=in_lly;

   A[12]=in_ulx*in_uly;
   A[13]=in_urx*in_ury;
   A[14]=in_lrx*in_lry;
   A[15]=in_llx*in_lly;

   bx[0]= out_ulx;
   bx[1]= out_urx;
   bx[2]= out_lrx;
   bx[3]= out_llx;

   by[0]= out_uly;
   by[1]= out_ury;
   by[2]= out_lry;
   by[3]= out_lly;

   c_qrdecomp(A, 4, 4, v, 0);
   c_qrsolve(A, 4, 4, v, bx, 0);
   c_qrsolve(A, 4, 4, v, by, 0);

   new_line = by[0] + by[1]*new_xsamp + by[2]*xline + by[3]*xline*new_xsamp;
   new_samp = bx[0] + bx[1]*new_xsamp + bx[2]*xline + bx[3]*xline*new_xsamp;

   /* Report results
   ---------------*/
   fprintf(fptr,"%-6s %9.2f %9.2f %9.2f %9.2f %8.2f %9.2f %9.2f\n",
	tpldata.pt_id, tpldata.sea_coord[0], tpldata.sea_coord[1],
	tpldata.ref_coord[0], tpldata.ref_coord[1], tpldata.elevation,
	new_line, new_samp);

   tpldata.sea_coord[0] = new_line;
   tpldata.sea_coord[1] = new_samp;
   curent++;

   /* Write results to output TPL file
   ---------------------------------*/
   status = c_ptio(&out_tpl, &tpldata);
   if (status != E_SUCC) 
        {
        fclose(fptr);
        strcpy(tae_name[1], print_name);
        num_out++;
        c_clean(&vblk,"RELIEFX",tae_name , &num_out);
        }
   }

c_stoptm();
c_ptclse(&in_tpl);
c_ptclse(&out_tpl);

/* Write the standard completion message and exit
-----------------------------------------------*/
if (file)
   {
   strcpy(tae_name[1], name);
   num_out++;
   }
c_complt("RELIEFX", tae_name, &num_out);

return EXIT_SUCCESS;

}
