/****************************************************************************
NAME:                           getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

ALGORITHM DESCRIPTION:
        Get the name of the output geometric mapping grid file
        See if an input image is given (to get input image location info)
        Get the input and output space projection information
                Get the name of the projection definition file
                Get the input projection key
                Get the output projection key
                Get the pixel size (2--4 if input info entered manually)
        Get framing information (LSCOORS, COORUNIT, COORS)
        Calculate projection min/max in X and Y if framing is entire input
           image with a window applied
        Get gridding information (NLGRID, NSGRID, TOLVAL)
        Get the rotation angle to be applied to the output space
        Get the print option
        Report the name of the output grid file to the user
        Report the projection parameters for input and output to the user
        Get the Message timer
        Compute coefficients mapping from output image coordinates to output
           space projection coordinates, with angle of rotation applied
        Calculate input location information from tie points, if needed
                Extract tie points
                Extract tie point units
                IF units are geographic,
                   Initialize the projection transformation package
                   Convert the tie points to projection coordinates
                Calculate coefficients mapping input space projection
                  coordinates to input space line, sample coordinates
                  based on the tie point set
        Otherwise, compute coefficients mapping from input space projection
           coordinates to input space image coordinates.  This supports 
           projection spaces which are rotated from the image coordinate
           system
        Return
*****************************************************************************/

#include <stdio.h>
#include "remap.h"
#include "ltable.h"
#include "util.h"        /* c_*nam and c_setprt prototypes */
#include "local_protos.h"

int getpar
(
    struct PARBLK *vblk,    /* TAE variable block */
    struct FRAME *frame,    /* Output framing parameters for GRID subcommand */
    struct PROJ *outp,      /* Output projection parameters */
    struct PROJ *inp,       /* Input projection parameters */
    double *ocoef,          /* Coefficients mapping output l,s to output proj */
    double *icoef,          /* Coeffs mapping input proj space to input l,s */
    char *name,             /* Print file name */
    long *msgtim            /* Message timer */
)
{
    struct VARIABLE *var;         /* pointer to a TAE variable in vblk */
    double scalex;                /* Input space projection pixel size in X */
    double scaley;                /* Input space projection pixel size in Y */
    double angle;                 /* Angle of rotation */
    double ul_corner[2];          /* Upper left corner coordinate */
    double lr_corner[2];          /* Lower right corner coordinate */
    double intie1[2];             /* First tie point in input space */
    double intie2[2];             /* Second tie point in input space */
    double outtie1[2];            /* First tie point in output space */
    double outtie2[2];            /* Second tie point in output space */
    double icorners[8];           /* Projection coordinates--image corners */
    double ocorners[8];           /* Projection coordinates--image corners */
    double newx;                  /* New (temporary) coordinate in X */
    double newy;                  /* New (temporary) coordinate in Y */
    double geo_parms[15];         /* 15 projetion parameters for geographic */
    double wind_proj[8];          /* Projection coordinates of image window */
    double pxmin, pxmax;          /* Projection min & max in X */
    double pymin, pymax;          /* Projection min & max in Y */
    double real_wind[4];          /* Window spec--in double */
    long fatal = -1;              /* fatal error */
    long status;                  /* function status */
    long null_zone = 62;          /* Null zone code */
    long prtprm[2];               /* Print/report flag for projection package */
    long tval_unit_code;          /* Units tie point were entered in */
    long imc[8];                  /* Image coordinates of image corners */
    long omc[8];                  /* Image coordinates of image corners */
    long lscoors[2];              /* Line,sample coordinates of ul coord */
    long nl, ns;                  /* The number of lines & samples in output */
    long unit_code;               /* Unit code of corner coordinates entered */
    long mode;                    /* Framing mode */
    long acc;                     /* access flag for c_getnam */
    long i;                       /* Loop counter */
    long zero = 0;                /* Constant of zero */
    long pass = 1;                /* Pass flag for non-fatal errmsg */
    long term, ln_pr, file;       /* Print option flags */
    long ddr_info;                /* Boolean flag--proj info from ddr used? */
    long out_ddr_info;            /* Boolean flag--proj info from ddr used? */
    long in_wind[1][4];           /* Input image window specification */
    long out_wind[1][4];          /* Output image window specification */
    char tval_units[12];          /* Units of Tie points entered */
    char units[12];               /* Units of corner coordinates entered */
    char filename[CMLEN];         /* Temporary filename */
    char proj_name[CMLEN];        /* Projection parameter file name */
    char host_name[CMLEN];        /* Host name buffer */
    char inkey[10];               /* Input projection key */
    char outkey[10];              /* Output projection key */
    char flag[10];                /* input/output flag                    */

    /* Get the name of the output geometric mapping grid file
       ------------------------------------------------------*/
    acc = IWRITE;
    if(c_getnam(vblk, "OUTGRID", &zero, frame->name, "grid", &acc) != E_SUCC)
        return(E_FAIL);

    /* See if an input image was given
       -------------------------------*/
    ddr_info = FALSE;
    var = p_find(vblk, "INSPACE");
    if ((*var).v_count != 0) ddr_info = TRUE;

    /* See if an output image was given
       -------------------------------*/
    out_ddr_info = FALSE;
    var = p_find(vblk, "OUTSPACE");
    if ((*var).v_count != 0) out_ddr_info = TRUE;


    /* Get input and output projection information.  If input and output images
       are specified, we do not need a projection file
       -----------------------------------------------------------------------*/
    if (!ddr_info || !out_ddr_info)
    {
        acc = IREAD;
        if (c_getnam(vblk, "INPROJ", &zero, proj_name, "proj", &acc) != E_SUCC)
            return(E_FAIL);
    }

    if (!ddr_info)
    {
        var = p_find(vblk, "IPROJKEY");
        if ((*var).v_count != 0)
            strcpy(inkey, SVAL(*var, 0));
        else
        {
            c_errmsg("Source of projection information not fully specified",
                    "remap-proj",&pass);
            return(E_FAIL);
        }
    }

    if (!out_ddr_info)
    {
        var = p_find(vblk, "OPROJKEY");
        if ((*var).v_count != 0)
            strcpy(outkey, SVAL(*var, 0));
        else
        {
            c_errmsg("Source of projection information not fully specified",
                    "remap-proj",&pass);
            return(E_FAIL);
        }
    }

    if (!ddr_info || !out_ddr_info)
    {
        if(get_proj_prm(proj_name, inkey, outkey, ddr_info, out_ddr_info, inp, 
                outp) != E_SUCC)
            return(E_FAIL);
    }

    strcpy(flag,"INSPACE");
    if (ddr_info)
        if(use_ddr_info(vblk, inp, icorners, imc, in_wind,flag,&scaley,
                &scalex)!=E_SUCC) 
            return(E_FAIL);

    strcpy(flag,"OUTSPACE");
    if (out_ddr_info)
        if(use_ddr_info(vblk, outp, ocorners, omc, out_wind,flag,
                &frame->o_dist[0],&frame->o_dist[1]) != E_SUCC) 
            return(E_FAIL);

    ul_corner[0] = ocorners[0];
    ul_corner[1] = ocorners[1];
    ns = omc[5];
    nl = omc[6];

    /* Get the pixel size.  If both the input and output images are specified, 
       we will get the pixel size from the ddr's and PIXSIZ is NULL.  If the 
       input image is specfied and an output image is not, we will get the 
       input pixel size from the DDR and the output pixel size from PIXSIZ.  
       If the output is specfied and the input is not, the output pixel size 
       is gotten from the DDR and the input comes from PIXSIZ.  If neither 
       an input nor an output image is specified both the output and input 
       pixel sizes come from PIXSIZ.  In the case where an output image is
       specified and an output PIXSIZ is also specified the output PIXSIZ
       will be used.
       ----------------------------------------------------------------------*/
    var = p_find(vblk, "PIXSIZ");
    if (!ddr_info && !out_ddr_info)
    {
        if ((*var).v_count != 4)
        {
            c_errmsg("Input & output pixel size must be specified",
                    "remap-pixsiz", &pass);
            return(E_FAIL);
        }
        frame->o_dist[0] = RVAL(*var, 0);
        frame->o_dist[1] = RVAL(*var, 1);
        scaley = RVAL(*var, 2);
        scalex = RVAL(*var, 3);
    }
    else if (ddr_info && !out_ddr_info)
    {
        if ((*var).v_count < 2)
        {
            c_errmsg("Output pixel size must be specified","remap-pixsiz", 
                      &pass);
            return(E_FAIL);
        }
        frame->o_dist[0] = RVAL(*var, 0);
        frame->o_dist[1] = RVAL(*var, 1);
    }
    else if (ddr_info && out_ddr_info)
    {
        if ((*var).v_count >= 2)
        {
            frame->o_dist[0] = RVAL(*var, 0);
            frame->o_dist[1] = RVAL(*var, 1);
        }
    }
    if (!ddr_info && out_ddr_info)
    {
        if ((*var).v_count == 4)
        {
            frame->o_dist[0] = RVAL(*var, 0);
            frame->o_dist[1] = RVAL(*var, 1);
            scaley = RVAL(*var, 2);
            scalex = RVAL(*var, 3);
        }
        else
            if ((*var).v_count == 2)
            {
                scaley = RVAL(*var, 0);
                scalex = RVAL(*var, 1);
            }
            else
            {
                c_errmsg("Input pixel size must be specified","remap-pixsiz", 
                         &pass);
                return(E_FAIL);
            }
    }

    /* Get the number of lines, and number of samples
       -----------------------------------------------*/
    var = p_find(vblk, "NL");
    if ((*var).v_count ==1)
    {
        nl = IVAL(*var,0);
        if (nl < 1)
        {
            c_errmsg("NL must be greater than zero","remap-invalid", &pass);
            return (E_FAIL);
        }
    }
    var = p_find(vblk, "NS");
    if ((*var).v_count ==1)
    {
        ns = IVAL(*var,0);
        if (ns < 1)
        {
            c_errmsg("NS must be greater than zero","remap-invalid", &pass);
            return (E_FAIL);
        }
    }

    var = p_find(vblk, "LSCOORS");
    lscoors[0] = IVAL(*var,0);
    lscoors[1] = IVAL(*var,1);
    var = p_find(vblk, "COORUNIT");
    strcpy(units, SVAL(*var,0));
    if(strcmp(units,"DMS") == 0) unit_code = 5;
    if(strcmp(units,"DEG") == 0) unit_code = 4;
    if(strcmp(units,"RAD") == 0) unit_code = 0;
    if(strcmp(units,"SEC") == 0) unit_code = 3;

    var = p_find(vblk, "COORS");
    if ((*var).v_count == 0)
        /* Framing info is obtained from the input or output image (and window, 
           if applied)
           ------------------------------------------------------------------*/
    {
        if (!ddr_info && !out_ddr_info)
        {
            c_errmsg("Framing coordinates must be specified","remap-frame", 
                      &pass);
            return(E_FAIL);
        }
        if (!out_ddr_info)
        {
            for (i = 4; i--; real_wind[i] = in_wind[0][i]);
            if (c_rotcor(icorners, imc, real_wind, wind_proj) != E_SUCC) 
                return(E_FAIL);
            pxmin = GMIN(wind_proj[1], wind_proj[3]);
            pxmin = GMIN(pxmin, wind_proj[5]);
            pxmin = GMIN(pxmin, wind_proj[7]);
            pxmax = GMAX(wind_proj[1], wind_proj[3]);
            pxmax = GMAX(pxmax, wind_proj[5]);
            pxmax = GMAX(pxmax, wind_proj[7]);
            pymin = GMIN(wind_proj[0], wind_proj[2]);
            pymin = GMIN(pymin, wind_proj[4]);
            pymin = GMIN(pymin, wind_proj[6]);
            pymax = GMAX(wind_proj[0], wind_proj[2]);
            pymax = GMAX(pymax, wind_proj[4]);
            pymax = GMAX(pymax, wind_proj[6]);
            mode = INPUT_XY;
        }
        else
            /* Framming is dertermined from the output image
               ----------------------------------------------*/
        {
            mode = OUTPUT_LS;
        }
    }
           else
               /* Framing is determined from user input
                  -------------------------------------*/
           {
               sscanf(SVAL(*var,0), "%lf", &(ul_corner[0]));
               sscanf(SVAL(*var,1), "%lf", &(ul_corner[1]));
               if ((*var).v_count == 2) mode = OUTPUT_LS;
               else if (strcmp(units, "PRO") == 0) mode = OUTPUT_XY;
               else mode = INPUT;
               if ((mode == INPUT) || (mode == OUTPUT_XY))
               {
                   sscanf(SVAL(*var,2), "%lf", &(lr_corner[0]));
                   sscanf(SVAL(*var,3), "%lf", &(lr_corner[1]));
               }
           }

    /* Get the number of lines and samples desired in each grid cell in the
       output space.  These can have values or can be null.
       ---------------------------------------------------*/
    var = p_find(vblk, "NLGRID");
    if ((*var).v_count > 0)
    {
        frame->ngline = IVAL(*var, 0);
        if (frame->ngline < 1)
        {
            c_lasmsg(FALSE,"remap-invalid","NLGRID must be greater than zero");
            return (E_FAIL);
        }
    }
    else
        frame->ngline = 0;

    var = p_find(vblk, "NSGRID");
    if ((*var).v_count > 0)
    {
        frame->ngsamp = IVAL(*var, 0);
        if (frame->ngsamp < 1)
        {
            c_lasmsg(FALSE,"remap-invalid","NSGRID must be greater than zero");
            return (E_FAIL);
        }
    }
    else
        frame->ngsamp = 0;

    /* Get the tolerance value for grid reduction
       ------------------------------------------*/
    var = p_find(vblk, "TOLVAL");
    frame->tol = RVAL(*var, 0);

    /* Get the rotation angle, if any.
       ------------------------------*/
    var = p_find(vblk, "ROTANG");   
    angle = RVAL(*var, 0);

    /* Get the print option.  Make a file name (if neccessary) and open the
       temporary file
       --------------*/
    strcpy(host_name, "remap");
    status = c_setprt(vblk, &term, &file, &ln_pr, host_name, filename);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","remap-fatal",&fatal);

    acc = IUPDATE;
    if (file == 1) c_mknam(filename, name, "prt", &acc);
    else strcpy(name, " ");
    frame->fptr = fopen(host_name, "w");

    fprintf(frame->fptr,"Geometric Mapping Grid name:  %s\n",  frame->name);

    /* Initialize the Projection Transformation Package (Report projection 
       parameters to the user)
       -----------------------*/
    prtprm[GEO_TERM] = FALSE;
    prtprm[GEO_FILE] = TRUE;
    fclose(frame->fptr);

    if(c_transinit(&inp->proj_code, &inp->units, &inp->zone_code, 
            &inp->datum_code, inp->proj_coef, &outp->proj_code, &outp->units, 
            &outp->zone_code, &outp->datum_code, outp->proj_coef, prtprm, 
            host_name) != E_SUCC)
        return(E_FAIL);

    frame->fptr = fopen(host_name, "a");

    /* Get the message timer
       ---------------------*/
    var = p_find(vblk, "$msgtim");
    *msgtim = IVAL(*var, 0);

    /* Compute coefficients mapping from output image coordinates to output
       space projection coordinates--including the angle of rotation!
       --------------------------------------------------------------*/
    if (out_frame(frame, outp, inp, mode, ul_corner, lr_corner, lscoors, nl, ns,
            unit_code, angle, pxmin, pxmax, pymin, pymax, ocoef) 
            != E_SUCC) return (E_FAIL); 

    /* If input image location information is to be derrived from a set of
       tie points...
       -------------*/
    if (!ddr_info) 
    {
        var = p_find(vblk, "TVAL");
        if ((*var).v_count == 0)
        {
            c_errmsg("Tie point values must be specified","remap-points",&pass);
            return(E_FAIL);
        }
        intie1[0] = RVAL(*var, 0);
        intie1[1] = RVAL(*var, 1);
        outtie1[0] = RVAL(*var, 2);  
        outtie1[1] = RVAL(*var, 3);
        intie2[0] = RVAL(*var, 4);   
        intie2[1] = RVAL(*var, 5);
        outtie2[0] = RVAL(*var, 6);  
        outtie2[1] = RVAL(*var, 7);
        var = p_find(vblk, "TVALUNIT");
        strcpy(tval_units, SVAL(*var,0));
        tval_unit_code = 2;
        if(strcmp(tval_units,"DMS") == 0) tval_unit_code = 5;
        if(strcmp(tval_units,"DEG") == 0) tval_unit_code = 4;
        if(strcmp(tval_units,"RAD") == 0) tval_unit_code = 0;
        if(strcmp(tval_units,"SEC") == 0) tval_unit_code = 3;

        if (tval_unit_code != 2)
        {

            /* Initialize the Projection Transformation Package
               ------------------------------------------------*/
            prtprm[GEO_TERM] = GEO_FALSE;
            prtprm[GEO_FILE] = GEO_FALSE;

            for (i = 0; i < 15; i++)
                geo_parms[i] = 0.0;

            if(c_transinit(&zero, &tval_unit_code, &null_zone, &zero, geo_parms,
                      &inp->proj_code, &inp->units, &inp->zone_code, 
                      &inp->datum_code, inp->proj_coef, prtprm, "\0") != E_SUCC)
            {  
                return(E_FAIL);
            }

            if(c_trans(&zero, &tval_unit_code, &inp->proj_code, &inp->units, 
                      &(outtie1[1]), &(outtie1[0]), &newx, &newy) != E_SUCC)
            {
                return(E_FAIL);
            }

            outtie1[0] = newy;
            outtie1[1] = newx;

            if(c_trans(&zero, &tval_unit_code, &inp->proj_code, &inp->units, 
                      &(outtie2[1]), &(outtie2[0]), &newx, &newy) != E_SUCC)
            {
                return(E_FAIL);
            }
            outtie2[0] = newy;
            outtie2[1] = newx;
        }
        if(twopoint(intie1, intie2, outtie1, outtie2, scalex, scaley, icoef) 
                != E_SUCC) return(E_FAIL);
    }

    else
        /* Compute coefficients mapping from input space projection coordinates
           to input space image coordinates.  Rotated projection spaces are 
           supported.
           ------------------------------------------------------------------*/
        if (in_frame(icorners, imc, icoef) != E_SUCC) return (E_FAIL);

    frame->convert = TRUE;
    if ((inp->datum_code == outp->datum_code) &&
            (inp->proj_code == outp->proj_code) &&
            ((inp->proj_code == 1) || (inp->proj_code == 2)) &&
            (inp->zone_code == outp->zone_code)) frame->convert = FALSE;

    return(E_SUCC);
}

/****************************************************************************
NAME:                           get_proj_prm

PURPOSE:  Retrieves the projection parameters from the projection parameter file

ALGORITHM DESCRIPTION:
        Open the label table file containing the projection parameters
        Get the label
        Determine the position of the fields within the file
        Scan the file for the projection parameters specified by the input
           and output keys  (If the keys are null, the first record in the
           file is assigned to the output projection; the second is the
           input projection)
        Close the table
        Return
*****************************************************************************/
int get_proj_prm
(
    char *proj_name,            /* Projection parameter file name */
    char *inkey,                /* Input projection parameter key */
    char *outkey,               /* Output projection parameter key */
    long in_ddr_info,           /* Boolean flag--proj info from ddr used? */
    long out_ddr_info,          /* Boolean flag--proj info from ddr used? */
    struct PROJ *inp,           /* Input projection parameters */
    struct PROJ *outp           /* Output projection parameters */
)
{
    struct TAB_DEF tab;           /* Tabular structure of label table */
    struct VECTOR *keyptr;        /* Pointer to key field */
    struct VECTOR *codeptr;       /* Pointer to projection code field */
    struct VECTOR *zoneptr;       /* Pointer to zone code field */
    struct VECTOR *unitptr;       /* Pointer to unit of measure field */
    struct VECTOR *sphptr;        /* Pointer to datum (spheroid) code field */
    struct VECTOR *parmptr;       /* Pointer to projection coefficients field */
    long inproj_found = FALSE;    /* Boolean--input projection found? */
    long outproj_found = FALSE;   /* Boolean--output projection found? */
    long status;                  /* Function return status */
    long nonfatal= 1;             /* nonfatal flag */
    long i;                       /* Loop counter */

    /* Open the labeled table file & retrieve the label
       ------------------------------------------------*/
    open_tab(&tab, proj_name, IREAD);
    get_vector(&tab);

    /* Determine pointers to fields in the file
       ----------------------------------------*/
    keyptr = get_field_ptr(&tab, "PROJKEY");
    if (keyptr == NULL) return(E_FAIL);
    codeptr = get_field_ptr(&tab, "PROJTYPE");
    if (codeptr == NULL) return(E_FAIL);
    zoneptr = get_field_ptr(&tab, "PROJZONE");
    if (zoneptr == NULL) return(E_FAIL);
    unitptr = get_field_ptr(&tab, "PROJUNITS");
    if (unitptr == NULL) return(E_FAIL);
    sphptr = get_field_ptr(&tab, "PROJSPH");
    if (sphptr == NULL) return(E_FAIL);
    parmptr = get_field_ptr(&tab, "PROJPARMS");
    if (parmptr == NULL) return(E_FAIL);

    /* Initialize BOOLEAN variables
       ----------------------------*/
    if (in_ddr_info)
        inproj_found = TRUE;
    if (out_ddr_info)
        outproj_found = TRUE;

    /* Loop until key is found or EOF
       -------------------------------*/
    for (;;)
    {
        status = get_record(&tab);
        if (status == E_FAIL) return(E_FAIL);
        if (status == E_EOF) break;
        if (strcmp(keyptr->ptr.c, inkey) == 0)
            if (!inproj_found)
            {
                inp->proj_code = *codeptr->ptr.i4;
                inp->zone_code = *zoneptr->ptr.i4;
                inp->units = *unitptr->ptr.i4;
                inp->datum_code = *sphptr->ptr.i4;
                for (i = 0; i < 15; i++)
                    inp->proj_coef[i] = parmptr->ptr.r8[i];
                inproj_found = TRUE;
            }

        if (strcmp(keyptr->ptr.c, outkey) == 0)
            if (!outproj_found)
            {
                outp->proj_code = *codeptr->ptr.i4;
                outp->zone_code = *zoneptr->ptr.i4;
                outp->units = *unitptr->ptr.i4;
                outp->datum_code = *sphptr->ptr.i4;
                for (i = 0; i < 15; i++)
                    outp->proj_coef[i] = parmptr->ptr.r8[i];
                outproj_found = TRUE;
            }
        if (inproj_found && outproj_found) break;
    }

    /* Close the file & return
       -----------------------*/
    close_tab(&tab);
    if (!inproj_found)
    {
        c_errmsg("Error finding IPROJKEY","remap-fatal",&nonfatal);
        return(E_FAIL);
    }
    if (!outproj_found)
    {
        c_errmsg("Error finding OPROJKEY","remap-fatal",&nonfatal);
        return(E_FAIL);
    }
    return(E_SUCC);
}

/*****************************************************************************
NAME                              use_ddr_info

PURPOSE    Reads projection & framing information from an image DDR

ALGORITHM DESCRIPTION
        Retrieve the name of the input image via c_getin_nocheck()
        Get the DDR information
        If DDR information is valid,
           Pack output projection parameters from the DDR projection information
        Return
*****************************************************************************/
int use_ddr_info
(
    struct PARBLK *vblk,        /* TAE parameter block */
    struct PROJ *pro,           /* Structure containing projection parameters */
    double *corners,            /* Corner projection coordinates */
    long *image,                /* Corner image coordinates */
    long in_wind[][4],          /* Image window specification */
    char *flag,                 /* input/output flag            */
    double *pixy,               /* pixel size in y direction    */
    double *pixx                /* pixel size in x direction    */
)
{
    struct DDR ddr;         /* DDR structure */
    long i;                 /* Loop counters */
    long zero = 0;          /* Constant of zero */
    long one = 1;           /* Constant of one for c_errmsg routine */
    long num_img;           /* Number of images input:  0 or 1 */
    long bands[1][MAXBND+1];/* Band numbers */
    long nbands;            /* Number of bands */
    long totbands;          /* Total number of bands */
    long same;              /* Same window flag */
    long qflag[1][2];       /* Option flags */
    char in_image[1][CMLEN];/* Image file name */
    char tdum[1][CMLEN];    /* Tae name */

    /* Check for projection information and place it in the pro & frame 
       structures if it exists.  
       -----------------------*/
    if(c_getin_nocheck(vblk,flag,&zero,tdum,in_image,&num_img,in_wind,bands,
            &nbands, &totbands,&one,&same,qflag) != E_SUCC) 
    {
        return(E_FAIL);
    }

    if (c_getddr(in_image[0], &ddr) != E_SUCC) 
    {
        return(E_FAIL);
    }
    else
    {
        image[0] = image[1] = 1;
        image[2] = ddr.nl;
        image[3] = 1;
        image[4] = 1;        
        image[5] = ddr.ns;
        image[6] = ddr.nl;
        image[7] = ddr.ns;

        if ((ddr.valid[0] == INVAL) || (ddr.valid[3] == INVAL) || 
            (ddr.valid[4] == INVAL))
        {
            c_errmsg("Projection information not available from DDR",
                    "remap-noproj",&one);
            return(E_FAIL);
            
        }
        else
        {
            pro->proj_code = ddr.proj_code;
            if (ddr.valid[1] == INVAL) pro->zone_code = 62;
            else pro->zone_code = ddr.zone_code;
            if (ddr.valid[2] == INVAL) pro->datum_code = 0;
            else pro->datum_code = ddr.datum_code;

            pro->units = c_string2unit(ddr.proj_units);
            if (pro->units == E_FAIL)
            {
                c_lasmsg(FALSE, "remap-warn", 
                        "Projection units %s are not supported in LAS", 
                         ddr.proj_units);
                c_lasmsg(FALSE, "remap-warn", "Setting units to invalid");
                ddr.valid[4] = INVAL;
            }

            for (i = 0; i < 15; i++)
                pro->proj_coef[i] = ddr.proj_coef[i];
            if (ddr.valid[6] == INVAL)
                for (i = 0; i < 8; i++)
                    corners[i] = 0.0;
            else
            {
                corners[0] = ddr.upleft[0];
                corners[1] = ddr.upleft[1];
                corners[2] = ddr.loleft[0];
                corners[3] = ddr.loleft[1];
                corners[4] = ddr.upright[0];
                corners[5] = ddr.upright[1];
                corners[6] = ddr.loright[0];
                corners[7] = ddr.loright[1];
            }
        }
        *pixx = ddr.pdist_x;
        *pixy = ddr.pdist_y;
    }

    return(E_SUCC);
}
