/****************************************************************************
NAME:                         MAKE_GRID 

PURPOSE:  Creates a Geometric Mapping Grid file mapping from one map
          projection to another.  Grid reduction techniques may be applied and 
          a summary of gridding errors is calcualted and reported to the user.

ALGORITHM DESCRIPTION:
        If a projection conversion is needed
            Setup the projection conversion
        Setup the data structure needed to call ols2ils
        Call the geolib c_makegrid function
        Pack projection related fields into the Geometric Mapping Grid file
        Write the Geometric Mapping Grid file to disk
        Return
*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "util.h"     /* c_inittm prototype */
#include "remap.h"
#include "local_protos.h"

typedef struct
{
    int use_projection;  /* flag that indicates a projection change is involved
                            (1=projection change needed) */
    struct PROJ *outp;   /* output projection info */
    struct PROJ *inp;    /* input projection info */
    double *ocoef;       /* coefficients to move from output line/sample to 
                            projection space */
    double *icoef;       /* coefficients to move from input projection to 
                            input line/sample */
} OLS2ILS_PARAMS;

/* routine to perform output line/sample to input line/sample transformations */
static int ols2ils
(
    void *data,     /* I: pointer to a OLS2ILS_PARAMS structure for the 
                          parameters specific to this gridding application */
    int oline,      /* I: output space line */
    int osamp,      /* I: output space sample */
    double *iline,  /* O: input space line */
    double *isamp   /* O: input space sample */
)
{
    double proj_y, proj_x;  /* projection y/x coordinates */
    OLS2ILS_PARAMS *parm = (OLS2ILS_PARAMS *)data;
    double *ocoef = parm->ocoef;
    double *icoef = parm->icoef;

    /* convert the output line/sample into output projection space */
    proj_y = (ocoef[0] * oline) + (ocoef[1] * osamp) + ocoef[2];
    proj_x = (ocoef[3] * oline) + (ocoef[4] * osamp) + ocoef[5];

    if (parm->use_projection)
    {
        /* a projection change is needed */
        struct PROJ* outp = parm->outp;
        struct PROJ* inp  = parm->inp;
        double outy, outx;

        /* convert from output space projection to the input space projection */
        if(c_trans(&outp->proj_code, &outp->units, &inp->proj_code, 
                &inp->units, &proj_x, &proj_y, &outx, &outy) != E_SUCC)
        {
            return(NO_REMAP);
        }

        /* convert from input space projection to input line/sample */
        *iline = (icoef[0] * outy) + (icoef[1] * outx) + icoef[2];
        *isamp = (icoef[3] * outy) + (icoef[4] * outx) + icoef[5];
    }
    else
    {
        /* no projection change, so convert from projection to input
           line/sample */
        *iline = (icoef[0] * proj_y) + (icoef[1] * proj_x) + icoef[2];
        *isamp = (icoef[3] * proj_y) + (icoef[4] * proj_x) + icoef[5];
    }

    return (E_SUCC);
}

/*---------------------------------------------------------------------*/

int make_grid
(
    struct FRAME *frame,        /* Output framing parameters */
    struct PROJ *outp,          /* Output projection parameters */
    struct PROJ *inp,           /* Input projection parameters */
    double *ocoef,              /* Coefficients mapping ls to output proj */
    double *icoef,              /* Coefficients mapping input proj to ls */
    long msgtim                 /* Message timing parameter */
)
{
    struct GEOGRID grid;        /* Geometric mapping grid */
    long prtprm[2];             /* Print flags for c_transinit */
    long count;                 /* Loop counters */
    OLS2ILS_PARAMS ols2ils_data;/* parameters to ols2ils */

    extern long curent;         /* Current "line" element for processing mess */

    prtprm[GEO_TERM] = FALSE;
    prtprm[GEO_FILE] = FALSE;

    /* setup the projection transformation to use */
    if (frame->convert)
    {
        if(c_transinit(&outp->proj_code, &outp->units, &outp->zone_code,
                &outp->datum_code, outp->proj_coef, &inp->proj_code, 
                &inp->units, &inp->zone_code, &inp->datum_code, inp->proj_coef,
                prtprm, "\0") != E_SUCC) 
        {
            return(NO_REMAP);
        }
    }

    /* setup the data structure to use for the ols2ils call */
    ols2ils_data.use_projection = frame->convert;
    ols2ils_data.outp = outp;
    ols2ils_data.inp  = inp;
    ols2ils_data.ocoef = ocoef;
    ols2ils_data.icoef = icoef;

    /* build the grid */
    if (c_makegrid(frame->window, frame->ngline, frame->ngsamp, ols2ils,
            &ols2ils_data, frame->tol, "REMAP", msgtim, &grid) != E_SUCC)
    {
        c_lasmsg(FALSE,"makegrid-fail","Error returned from c_makegrid");
        return (E_FAIL);
    }

    /* Pack projection related fields in the grid file
       -----------------------------------------------*/
    grid.proj_valid = GEO_VALID;
    grid.code = outp->proj_code;
    grid.zone = outp->zone_code;
    grid.datum = outp->datum_code;
    for (count = 0; count < 8; count++)
        grid.corners[count] = frame->corners[count];
    grid.pdist[0] = frame->o_dist[0];
    grid.pdist[1] = frame->o_dist[1];
    for (count = 0; count < 15; count++)
        grid.projprms[count] = outp->proj_coef[count];

    if (c_unit2string(outp->units, grid.units) != E_SUCC)
    {
        /* c_unit2string displays the error message */
        c_lasmsg(FALSE, "remap-warn", "Set grid.units to empty");
        strcpy(grid.units, " ");
    }

    /* Pack modeling related fields in the grid file
       ---------------------------------------------*/
    strcpy(grid.model, "REPROJECT");
    grid.ncoeff = 0;
    grid.coeffs = NULL;

    /* Pack statistics related fields in the grid file 
       -----------------------------------------------*/
    grid.fiterr_valid = GEO_NOTVALID;
    grid.max_fit_err[0] = 0;
    grid.max_fit_err[1] = 0;
    grid.ave_fit_err[0] = 0;
    grid.ave_fit_err[1] = 0;
    grid.rms_fit_err = 0;
    grid.griderr_valid = GEO_NOTVALID;
    grid.max_grid_err[0] = 0;
    grid.max_grid_err[1] = 0;
    grid.ave_grid_err[0] = 0;
    grid.ave_grid_err[1] = 0;
    grid.rms_grid_err = 0;

    fprintf(frame->fptr,
           "Geometric Mapping Grid file created with %d rows by %d columns\n\n",
           grid.nrows, grid.ncols);

    /* Write the geometric mapping grid to disk
       ----------------------------------------*/
    if(c_putgrd(frame->name, &grid) != E_SUCC) return(E_FAIL);

    fclose(frame->fptr);

    /* Free space 
       -------------*/
    free(grid.out_lines); 
    free(grid.out_samps); 
    free(grid.in_lines);  
    free(grid.in_samps); 

    return(E_SUCC);
}
