/****************************************************************************
NAME:                           out_frame

PURPOSE:  Calculates a set of coefficients mapping from output space
          image coordinates to output space projection coordinates

ALGORITHM DESCRIPTION:
        If frame is to be determined from input image extent:
                Initialize the projection transformation package
                For each corner, put coordinates in temp variables and
                   convert the coordinates to geographic output space
                Find the mininum bounding rectangle
                Proceed as if frame is determined from a rectangular 
                   geographic space
        If frame is to be determined from a rectangular geographic space:
                Initialize the projection transformation package
                Determine output projection space min/maxes (c_mapedg)
                Adjust min/maxes to even pixel increment
        If frame is to be determined from 4 output space projection coords:
                Adjust first coordinate pair to image coordinate (1,1)
        If frame is to be determined from 2 output space projection coords
           & the number of lines and samples in the output space:
                Adjust first coordinate pair to image coordinate (1,1)
                Calculate the lower right corner from the number of lines
                   and samples from the upper left corner (at (1,1))
        Report to the user before rotation is applied:
                The projection min/max in X and in Y
                The pixel size in X and Y
                The number of lines and samples
        Calculate coefficients mapping from output image line/sample to
           output space projection coordinates
        Report the angle of rotation applied
        Calculate coefficients for appling image rotation
        Calculate the number of lines and samples in the rotated image
        Calcualte pivot coordinates--These are at the image centers!
        Apply the rotation, calculating corner projection coordinates in
          the rotated image space
        Report to the user after rotation is applied:
                The upper left image corner projection coordinates
                The lower left image corner projection coordinates
                The upper right image corner projection coordinates
                The lower right image corner projection coordinates
                The pixel size in X and Y
                The number of lines and samples in the rotated output image
        Calculate coefficients mapping output space image coordinates
           in the rotated output space to output space projection coordinates
        Return
*****************************************************************************/

#include <math.h>
#include "remap.h"
#include "local_protos.h"

int out_frame
(
    struct FRAME *frame,        /* Framing parameters */
    struct PROJ *outp,          /* Output projection parameters */
    struct PROJ *inp,           /* Input projection parameters */
    long mode,                  /* Type of framing */
    double *ul,                 /* Upper left corner coordinates */
    double *lr,                 /* Lower right corner coordinates */
    long *ls,                   /* Line/sample coordinates of ul */
    long nl,                    /* Number of lines in output space */
    long ns,                    /* Number of samples in output space */
    long units,                 /* Units code of coordinates entered */
    double angle,               /* Rotatation angle applied to output */
    double pxmin,               /* Maximum & minimum X projection coordinates */
    double pxmax,
    double pymin,               /* Maximum & minimum Y projection coordinates */
    double pymax,
    double *ocoef               /* Coefficients mapping ls to out proj coords */
)
{
    double cc[6];               /* Coeffs mapping non-rotated output space */
    double rot[6];              /* Coeffs appling rotation (pivot at center) */
    double inx, iny;            /* Temporary storage of projection min/max */
    double line,samp;           /* Image coords for rotation calculation */
    double temp_mod;            /* Results from DMOD operation... */
    double piv[4];              /* Pivot coordinates (@ center of image) */
    long nl_rot, ns_rot;        /* Number of lines & samps in rotated image */
    long imc[8];                /* Corner coordinates in image space */
    long prt_flag = TRUE;       /* Flag to print error messages in c_rtcoef */
    long zero = 0;              /* Constant of zero */
    double temp[8];             /* temp lat/long values */
    long status;                /* function status */
    long nonfatal=1;            /* error message status */
    long flg1 = TRUE,           /* Flags that are set if IN_BREAK*/
    flg2 = TRUE,
    flg3 = TRUE,
    flg4 = TRUE;

    /* If geographic corner coordinates were entered, determine the 
       lat/long for each corner and find the min box.  Procced as if 
       frame is to be determined from a rectangular geographic space.
       c_transinit is called in getpar prior to calling out_frame.
       -------------------------------------------------------------*/
    if (mode == INPUT_XY)
    {
        /* UL */
        inx = pxmin;
        iny = pymax;
        status=c_trans(&inp->proj_code, &inp->units, &zero, &units, &inx, &iny, 
                &(temp[1]), &(temp[0]));

        if (status == E_FAIL) 
            return (E_FAIL);
        else if (status == IN_BREAK) 
            flg1 = FALSE;

        /* LR */
        inx = pxmax;
        iny = pymin;
        status=c_trans(&inp->proj_code, &inp->units, &zero, &units, &inx, &iny, 
                &(temp[3]), &(temp[2]));
        if (status == E_FAIL) 
            return (E_FAIL);
        else if (status == IN_BREAK) 
            flg2 = FALSE;

        /* LL */
        inx = pxmin;
        iny = pymin;
        status=c_trans(&inp->proj_code, &inp->units, &zero, &units, &inx, &iny, 
                &(temp[5]), &(temp[4]));

        if (status == E_FAIL) 
            return (E_FAIL);
        else if (status == IN_BREAK) 
            flg3 = FALSE;

        /* UR */
        inx = pxmax;
        iny = pymax;
        status=c_trans(&inp->proj_code, &inp->units, &zero, &units, &inx, &iny, 
                &(temp[7]), &(temp[6]));

        if (status == E_FAIL) 
            return (E_FAIL);
        else if (status == IN_BREAK) 
            flg4 = FALSE;

        /* Find the min/max corners.  Only use
           the lat/long if c_trans was successfull
           Find UL Y
           ---------------------------------------*/
        if (flg1 && flg2)
            ul[0] = GMAX(temp[0],temp[2]); 
        else if (flg1)
            ul[0] = temp[0]; 
        else if (flg2)
            ul[0] = temp[2]; 

        if (flg3)
            ul[0] = GMAX(ul[0],temp[4]); 
        if (flg4)
            ul[0] = GMAX(ul[0],temp[6]); 

        /* Find UR Y
           ------------*/
        if (flg1 && flg2)
            lr[0] = GMIN(temp[0],temp[2]); 
        else if (flg1)
            lr[0] = temp[0]; 
        else if (flg2)
            lr[0] = temp[2]; 

        if (flg3)
            lr[0] = GMIN(lr[0],temp[4]); 
        if (flg4)
            lr[0] = GMIN(lr[0],temp[6]); 

        /* Find UL X
           ------------*/
        if (flg1 && flg2)
            ul[1] = GMIN(temp[1],temp[3]); 
        else if (flg1)
            ul[1] = temp[1]; 
        else if (flg2)
            ul[1] = temp[3]; 

        if (flg3)
            ul[1] = GMIN(ul[1],temp[5]); 
        if (flg4)
            ul[1] = GMIN(ul[1],temp[7]); 

        /* Find LR X
           ------------*/
        if (flg1 && flg2)
            lr[1] = GMAX(temp[1],temp[3]); 
        else if (flg1)
            lr[1] = temp[1]; 
        else if (flg2)
            lr[1] = temp[3]; 

        if (flg3)
            lr[1] = GMAX(lr[1],temp[5]); 
        if (flg4)
            lr[1] = GMAX(lr[1],temp[7]); 

        /* When remapping an image in Goodes (either North America or Eurasia),
           the UL and UR could both be in the break area.  This will check 
           a point in the middle in order to find a max y.  If this fails, 
           the user must break the image into small sections and remap 
           individually. 
           ------------------------------------------------------------------*/

        if (!flg1 && !flg4)
        {
            inx = (pxmax + pxmin)/2;
            iny = pymax;

            status=c_trans(&inp->proj_code,&inp->units,&zero,&units,&inx,&iny, 
                    &(temp[1]), &(temp[0]));
            if (status == E_FAIL) 
                return (E_FAIL);
            else if (status == IN_BREAK) 
            {
                c_errmsg("Error determining framing coordinates","remap-break",
                        &nonfatal);    
                return (E_FAIL); 
            }

            ul[0] = GMAX(ul[0],temp[0]); 
        }

        /* The LL and LR coordinate transformations failed.  The same thing
           as above could be done, but I don't have test data.
           -------------------------------------------------------------------*/
        if (!flg2 && !flg3)
        {
            c_errmsg("Error determining framing coordinates","remap-break",
                    &nonfatal);      
            return (E_FAIL); 
        }

        mode = INPUT;
    }

    if (mode == INPUT)
    {
        /* Determine output projection space minimums and maximums
           -------------------------------------------------------*/
        if(c_mapedg(&(ul[1]), &(lr[1]), &(ul[0]), &(lr[0]), &units, 
                &(outp->proj_code), &(outp->zone_code), outp->proj_coef, 
                &(outp->units), &pxmin, &pxmax, &pymin, &pymax, 
                &outp->datum_code) != E_SUCC) 
            return(E_FAIL);

        /* The UL corner is used as the center of the first line/sample.
           Round the LR corner to the nearest multiple of the pixel size,
           ensuring that the computed min/max values are included.
           --------------------------------------------------------------*/
        temp_mod = pymax - pymin;
        temp_mod = fmod(temp_mod, frame->o_dist[0]);
        if (temp_mod/frame->o_dist[0] > 1e-3)
            pymin -= frame->o_dist[0] - temp_mod;
        else
            pymin += temp_mod;
        temp_mod = pxmax - pxmin;
        temp_mod = fmod(temp_mod, frame->o_dist[1]);
        if (temp_mod/frame->o_dist[1] > 1e-3)
            pxmax += frame->o_dist[1] - temp_mod;
        else
            pxmax -= temp_mod;
    }

    /* If output frame was given in projection coordinates, for two points:
       --------------------------------------------------------------------*/
    else if (mode == OUTPUT_XY)
    {
        pymax = ul[0] + ((ls[0] - 1) * frame->o_dist[0]);
        pxmin = ul[1] - ((ls[1] - 1) * frame->o_dist[1]);
        pymin = lr[0];
        pxmax = lr[1];
    }

    /* If output frame was given in projection coordinates, 1 pt & nl, ns:
       -------------------------------------------------------------------*/
    else if (mode == OUTPUT_LS)
    {
        pymax = ul[0] + ((ls[0] - 1) * frame->o_dist[0]);
        pxmin = ul[1] - ((ls[1] - 1) * frame->o_dist[1]);
        pymin = pymax - ((nl - 1) * frame->o_dist[0]);
        pxmax = pxmin + ((ns - 1) * frame->o_dist[1]);
    }

    /* Report to the user the projection minimums and maximums of the frame
       --------------------------------------------------------------------*/
    fprintf(frame->fptr, "Before rotation is applied:\n");
    fprintf(frame->fptr, "   Min/max Y projection coordinates:  %lf %lf\n"
            , pymin, pymax);
    fprintf(frame->fptr, "   Min/max X projection coordinates:  %lf %lf\n"
            , pxmin, pxmax);

    /* Report to the user the pixel size in line & sample directions
       -------------------------------------------------------------*/
    fprintf(frame->fptr,"   Pixel size in line direction:    %f\n", 
            frame->o_dist[0]);
    fprintf(frame->fptr,"   Pixel size in sample direction:  %f\n",
            frame->o_dist[1]);

    /* Report to the user the number of lines and samples in the output space
       ----------------------------------------------------------------------*/
    frame->window[0] = frame->window[1] = 1;
    frame->window[2] = (long)((pymax - pymin) / frame->o_dist[0] + 1.5);
    frame->window[3] = (long)((pxmax - pxmin) / frame->o_dist[1] + 1.5);
    fprintf(frame->fptr,"   Number of lines in output space:    %d\n",
            frame->window[2]);
    fprintf(frame->fptr,"   Number of samples in output space:  %d\n\n",
            frame->window[3]);

    /* Pack the corner projection coordinates in the frame structure
       -------------------------------------------------------------*/
    frame->corners[1] = frame->corners[3] = pxmin;
    frame->corners[0] = frame->corners[4] = pymax;
    frame->corners[6] = frame->corners[2] = pymin;
    frame->corners[5] = frame->corners[7] = pxmax;

    /* Calculate coefficients mapping from output image line/sample to output
       space projection coordinates
       ----------------------------*/
    imc[0] = imc[1] = imc[3] = imc[4] = 1;
    imc[5] = imc[7] = frame->window[3];
    imc[6] = imc[2] = frame->window[2];

    if(c_rtcoef(frame->corners, imc, cc, &prt_flag) != E_SUCC) return(E_FAIL);

    /* Calculate coefficeints for image rotation 
       -----------------------------------------*/
    fprintf(frame->fptr,"Rotation angle applied:  %lf degrees\n\n", angle);
    if (angle < 0) angle= -angle;
    else angle = 360 - angle;
    if (angle == 360.0) angle = 0.0;
    angle *= PI/180.0;

    rot[0] = cos(angle);
    rot[1] = sin(angle);
    rot[3] = -sin(angle);
    rot[4] = cos(angle);

    nl_rot = (long)((frame->window[2] * fabs(rot[0])) + (frame->window[3] * 
            fabs(rot[1])) + 0.5);
    ns_rot = (long)((frame->window[2] * fabs(rot[1])) + (frame->window[3] * 
            fabs(rot[0])) + 0.5);

    piv[0] = (double)(frame->window[3] + 1) / 2.0;
    piv[1] = (double)(frame->window[2] + 1) / 2.0;
    piv[2] = (double)(ns_rot + 1) / 2.0;
    piv[3] = (double)(nl_rot + 1) / 2.0;

    rot[5] = piv[0] - (piv[2] * rot[0]) - (piv[3] * rot[3]);
    rot[2] = piv[1] - (piv[2] * rot[1]) - (piv[3] * rot[4]);

    /* Apply rotation to calculate corner projection coordinates at rotated 
       image corners
       --------------*/
    frame->window[2]=nl_rot;
    frame->window[3]=ns_rot;
    line = (rot[0] * frame->window[0]) + (rot[1] * frame->window[1]) + rot[2];
    samp = (rot[3] * frame->window[0]) + (rot[4] * frame->window[1]) + rot[5];
    frame->corners[0] = (cc[0] * line) + (cc[1] * samp) + cc[2];
    frame->corners[1] = (cc[3] * line) + (cc[4] * samp) + cc[5];
    line = (rot[0] * (frame->window[0] + frame->window[2] - 1)) + 
        (rot[1] * frame->window[1]) + rot[2];
    samp = (rot[3] * (frame->window[0] + frame->window[2] - 1)) + 
        (rot[4] * frame->window[1]) + rot[5];
    frame->corners[2] = (cc[0] * line) + (cc[1] * samp) + cc[2];
    frame->corners[3] = (cc[3] * line) + (cc[4] * samp) + cc[5];
    line = (rot[0] * frame->window[0]) + 
        (rot[1] * (frame->window[1] + frame->window[3] - 1)) + rot[2];
    samp = (rot[3] * frame->window[0]) + 
        (rot[4] * (frame->window[1] + frame->window[3] - 1)) + rot[5];
    frame->corners[4] = (cc[0] * line) + (cc[1] * samp) + cc[2];
    frame->corners[5] = (cc[3] * line) + (cc[4] * samp) + cc[5];
    line = (rot[0] * (frame->window[0] + frame->window[2] - 1)) + 
        (rot[1] * (frame->window[1] + frame->window[3] - 1)) + rot[2];
    samp = (rot[3] * (frame->window[0] + frame->window[2] - 1)) + 
        (rot[4] * (frame->window[1] + frame->window[3] - 1)) + rot[5];
    frame->corners[6] = (cc[0] * line) + (cc[1] * samp) + cc[2];
    frame->corners[7] = (cc[3] * line) + (cc[4] * samp) + cc[5];

    /* Report to the user the projection minimums and maximums of the frame
       --------------------------------------------------------------------*/
    fprintf(frame->fptr, "After rotation is applied:\n");
    fprintf(frame->fptr, 
            "   Upper left corner projection coordinates:   %lf %lf\n"
            , frame->corners[0], frame->corners[1]);
    fprintf(frame->fptr, 
            "   Lower left corner projection coordinates:   %lf %lf\n"
            , frame->corners[2], frame->corners[3]);
    fprintf(frame->fptr, 
            "   Upper right corner projection coordinates:  %lf %lf\n"
            , frame->corners[4], frame->corners[5]);
    fprintf(frame->fptr, 
            "   Lower right corner projection coordinates:  %lf %lf\n"
            , frame->corners[6], frame->corners[7]);

    /* Report to the user the pixel size in line & sample directions
       -------------------------------------------------------------*/
    frame->o_dist[0] *= (fabs(rot[0]) + fabs(rot[1]));
    frame->o_dist[1] *= (fabs(rot[0]) + fabs(rot[1]));
    fprintf(frame->fptr,"   Pixel size in line direction:    %f\n", 
            frame->o_dist[0]);
    fprintf(frame->fptr,"   Pixel size in sample direction:  %f\n",
            frame->o_dist[1]);

    /* Report to the user the number of lines and samples in the output space
       ----------------------------------------------------------------------*/
    fprintf(frame->fptr,"   Number of lines in output space:    %d\n",
            frame->window[2]);
    fprintf(frame->fptr,"   Number of samples in output space:  %d\n\n",
            frame->window[3]);

    imc[0] = imc[1] = imc[3] = imc[4] = 1;
    imc[5] = imc[7] = frame->window[3];
    imc[6] = imc[2] = frame->window[2];

    if(c_rtcoef(frame->corners, imc, ocoef, &prt_flag) != E_SUCC) 
        return(E_FAIL);

    return(E_SUCC);
}
