/****************************************************************************
NAME:                           REMAP

PURPOSE:  Creates a geometric mapping grid file which specifies an
          image projection change from one of the projections supported
          by the projection transformation package to another.

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  5.0    7/89   D. Steinwand  CSB      Original development
  5.0.1  2/90   D. Steinwand CSB       Fixed gridding error
  5.0.2  5/90   D. Etrheim    CSB      ADP #5531 Changed to do a min box
                                       if an input image is specified
  5.1    6/90   D. Etrheim    CSB      Corrected the handling of the defaults
                                       for IPROJKEY, OPROJKEY, and TVAL
  5.2   11/91   T. Mittan     CSB      Corrected error messages for IPROJKEY
                                       and OPROJKEY
  5.2    2/94   T. Mittan     CSB      Allowed an OUT image to be specified 
                                       to replace  OPROJKEY for projection
                                       information
  5.3    9/94   D. Etrheim    CSB      Modify c_getin to not check for a .img
                                       file.  There are processing flows where
                                       the only the DDR is present.  #P6472
  5.4    4/97   D. Etrheim    CSB      Modified out_frame.c to allow processing
                                       of global data sets where a corner lat/
                                       long is in the "break" area.
  7.0    7/98   J. Janssen    CSB      Corrected process of how we retrieved
                                       projection information in getpar


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
    Initialize LAS processing
    Get parameters from the user
    Create the geometric mapping grid which maps from one projection to
                another
    Call complete & exit

ALGORITHM REFERENCES:
    This version of REMAP is a total re-write from the LAS 4.0 version.
    LAS 5.0 REMAP allows any of the projection transformation package
    projections,  flexable framing parameter options, and an option to
    define the input image if the DDR does not contain valid info.

        1.  PROCHG, a LAS (pre 4.0) local EDC function which changed 
            projections of an image to any of 21 projections.  Written
            by J. Thormodsgard & J. Peterson, EDC, 1984.
        2.  REMAP, the LAS 4.0 projection change routine by A. DasGupta 
            (7/83, CSC) and S. Kondal (4/85, SAR)
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "remap.h"
#include "util.h"    /* c_inlas and c_complt prototypes */
#include "local_protos.h"


int main(void)
{
    struct PARBLK vblk;            /* TAE parameter block */
    struct FRAME frame;            /* Framing & gridding parameters */
    struct PROJ outp;              /* Output space projection parameters */
    struct PROJ inp;               /* Input space projection parameters */
    double ocoef[6];               /* Coeffs mapping l,s to output proj space */
    double icoef[6];               /* Coeffs mapping from input proj to l,s */
    long msgtim;                   /* Message timer variable */
    long num = 1;                  /* Number of output files */
    long pass = 1;                 /* Flag -- non-fatal error message */
    long fatal = -1;               /* Flag -- fatal error message */
    long std_out;                  /* Dummy variable to standard output */
    long status;                   /* make_grid return status */
    char pname[CMLEN];             /* Printer file name */
    char taenam[2][CMLEN];         /* Tae name of output file for complete */

    c_inlas(&vblk, &std_out);
    if (getpar(&vblk,&frame,&outp,&inp,ocoef,icoef,pname,&msgtim) != E_SUCC)
        c_errmsg("Fatal error encountered", "remap-fatal", &fatal);
    status = make_grid(&frame, &outp, &inp, ocoef, icoef, msgtim);
    if(status == NO_REMAP)
    {
        c_errmsg("Unable to re-map to selected output frame", "remap-nomap", 
                 &pass);
        c_errmsg("Fatal error encountered", "remap-fatal", &fatal);
    }
    if(status == E_FAIL) 
        c_errmsg("Fatal error encountered", "remap-fatal", &fatal);
    strcpy( &(taenam[0][0]), frame.name);
    if (strcmp(pname, " ") != 0)
    {
        num++;
        strcpy(&(taenam[1][0]), pname);
    }
    c_complt("REMAP", taenam, &num);
    return (E_SUCC);
}
