/*****************************************************************************
NAME                              TWOPOINT

PURPOSE    Calculates mapping coefficients from a pair of tiepoints

ALGORITHM DESCRIPTION
        Find the pythagorian distance for the input space
        Find the pythagorian distance for the output space
        If the tie points coincide, abort
        Calculate the sine of the angle of rotation
        Calculate the cosine of the angle of rotation
        Calculate mapping coefficients
        Invert coefficients to map opposite direction
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "local_protos.h"

int twopoint
(
    double intie1[2],           /* First input location, line/sample */
    double intie2[2],           /* Second input location, line/sample */
    double outtie1[2],          /* First output location, line/sample */
    double outtie2[2],          /* Second output location, line/sample */
    double scalex,              /* Scale in X (pixel size) */
    double scaley,              /* Scale in Y (pixel size) */
    double icoef[6]             /* Transformation coefficients */
)
{
    double coef[6];                 /* Inverse xform coefficients */
    double sinphi;                  /* rotation angle */
    double cosphi;                  /* rotation angle */
    double dist;                    /* Calculation saver...in_dist * out_dist */
    double in_dist;                 /* Distance between input tiepoints */
    double out_dist;                /* Distance between output tiepoints */
    long pass = 1;                  /* Non-fatal error flag for c_errmsg */

    /* Compute the pythagorian distance of the input and output space
       ---------------------------------------------------------------*/
    in_dist = sqrt(((intie2[0] - intie1[0]) * (intie2[0] - intie1[0])) +
            ((intie2[1] - intie1[1]) * (intie2[1] - intie1[1])));
    out_dist =sqrt(((outtie2[0] - outtie1[0]) * (outtie2[0] - outtie1[0])) +
            ((outtie2[1] - outtie1[1]) * (outtie2[1] - outtie1[1]))); 
    dist = in_dist * out_dist;
    if(dist == 0.0)
    {
        c_errmsg("Tie points coincide","remap-tiepts", &pass);
        return(E_FAIL);
    }

    /* Compute angle and scale factors 
       --------------------------------*/
    sinphi = ((intie2[0] - intie1[0]) * (outtie2[1] - outtie1[1]) +
            (intie2[1] - intie1[1]) * (outtie2[0] - outtie1[0])) / dist;
    cosphi = ((intie2[0] - intie1[0]) * (outtie2[0] - outtie1[0]) -
            (intie2[1] - intie1[1]) * (outtie2[1] - outtie1[1])) / dist;

    /* Calculate mapping coefficients 
       -------------------------------*/
    coef[0] = cosphi * scaley;
    coef[1] = sinphi * scalex;
    coef[2] = outtie1[0] - (intie1[0] * coef[0]) - (intie1[1] * coef[1]);
    coef[3] = sinphi * scaley;
    coef[4] = -(cosphi) * scalex;
    coef[5] = outtie1[1] - (intie1[0] * coef[3]) - (intie1[1] * coef[4]);

    /* Invert coefficients to map from projection space to image space
       ---------------------------------------------------------------*/
    if(c_invert(coef, icoef) != E_SUCC) return(E_FAIL);
    return(E_SUCC);
}
