/*****************************************************************************
NAME:           calc_window_corners

PURPOSE:
The calc_window_corners function calculates the projection coordinates for
a window given the windows line/sample corners and the overall image 
projection window and line/sample corners.

DEVELOPMENT HISTORY:

RETURN VALUE:
The CORNERS structure at window_corner_proj_ptr is returned.

NOTES:
This routine makes the assumption that the window projection coordinates can
be found by using the equation form:
   Line_proj = a0 + a1 * sample_in + a2 * line_in + a3 * sample_in * line_in  
A matrix is packed with the sample/line and projection values of the four 
known corner points and used to solve for the coefficients.  Those 
coefficients are then used to calculate the window projection coordinates.

ALGORITHM REFERENCES:
none

*****************************************************************************/

#include "lasmath.h"
#include "resample.h" /* prototype for this function */

/*
**
** MACROS
**
*/

/* macro to calculate a coordinate given a point and the coefficients for
   an equation of the form: new_proj_coord = coeff0 + coeff1 * sample +
   coeff2 * line + coeff3 * line * sample
-------------------------------------------------------------------------*/
#define find_coord(point,coeff) \
    coeff[0] + coeff[1] * point.x + coeff[2] * point.y + \
    coeff[3] * point.x * point.y


void calc_window_corners 
(
    const struct CORNERS *known_corners_ptr,    /* I: known corner line/sample 
                                                      coordinates           */
    const struct CORNERS *known_corner_proj_ptr,/* I: known corner projection 
                                                       coordinates          */
    const struct CORNERS *window_corners_ptr,   /* I: sub-window line/sample 
                                                      coordinates           */
    struct CORNERS *window_corner_proj_ptr      /* O: sub-window projection 
                                                      coordinates           */
)
{
    double A[16];          /* matrix with the four known corner points
                              coefficients                                  */
    double v[4];           /* decomposed vector                             */
    double line_coeff[4];  /* line coefficients matrix                      */
    double samp_coeff[4];  /* sample coefficients matrix                    */
    long four = 4;         /* the number 4                                  */
    long zero = 0;         /* the number 0                                  */


    /* Pack the A matrix with input lines/samples.  The order of the 
       corner points is upleft, upright, loleft, then loright.
       -------------------------------------------------------------*/
    A[0]  = 1.0;
    A[1]  = 1.0;
    A[2]  = 1.0;
    A[3]  = 1.0;
    /* samples */
    A[4]  = known_corners_ptr->upleft.x;
    A[5]  = known_corners_ptr->upright.x;
    A[6]  = known_corners_ptr->loleft.x;
    A[7]  = known_corners_ptr->loright.x;
    /* lines */
    A[8]  = known_corners_ptr->upleft.y;
    A[9]  = known_corners_ptr->upright.y;
    A[10] = known_corners_ptr->loleft.y;
    A[11] = known_corners_ptr->loright.y;
    /* line*sample terms */
    A[12] = A[4]*A[8];
    A[13] = A[5]*A[9];
    A[14] = A[6]*A[10];
    A[15] = A[7]*A[11];

    /* Pack the sample coefficients vector with the known output samples.
       ------------------------------------------------------------------*/
    samp_coeff[0] = known_corner_proj_ptr->upleft.x;
    samp_coeff[1] = known_corner_proj_ptr->upright.x;
    samp_coeff[2] = known_corner_proj_ptr->loleft.x;
    samp_coeff[3] = known_corner_proj_ptr->loright.x;

    /* Decompose the matrix.
       ---------------------*/
    c_qrdecomp (A, four, four, v, zero);

    /* If the matrix is singular, let's slightly adjust the singular
       element to allow for proper computations.  The only case that has
       been detected to be singular involves a grid with 45-degree rotation.
       And only the last diagonal element has been seen to be zero.
       Since no other cases have been determined, this simple solution is
       being used in lieu of a (possibly more appropriate) SVD approach,
       which would require a much larger effort.

       To do things right in the future, all matrix-related activities (in
       all LAS/ADAPS routines) should use LAPACK.
       ------------------------------------------------------------------*/
    if (A[15] == 0.)
        A[15] = 1e-12;

    /* Solve for the sample coefficients.
       ----------------------------------*/
    c_qrsolve (A, four, four, v, samp_coeff, zero);

    /* Pack the line coefficients vector with the known output lines.
       --------------------------------------------------------------*/
    line_coeff[0] = known_corner_proj_ptr->upleft.y;
    line_coeff[1] = known_corner_proj_ptr->upright.y;
    line_coeff[2] = known_corner_proj_ptr->loleft.y;
    line_coeff[3] = known_corner_proj_ptr->loright.y;

    /* Solve for the line coefficients, reusing the same decomposed matrix.
       --------------------------------------------------------------------*/
    c_qrsolve (A, four, four, v, line_coeff, zero);

    /* Find the window corner projection coordinates.
       ----------------------------------------------*/
    window_corner_proj_ptr->upleft.x = find_coord(window_corners_ptr->upleft,
            samp_coeff);
    window_corner_proj_ptr->upleft.y = find_coord(window_corners_ptr->upleft,
            line_coeff);
    window_corner_proj_ptr->upright.x = find_coord(window_corners_ptr->upright,
            samp_coeff);
    window_corner_proj_ptr->upright.y = find_coord(window_corners_ptr->upright,
            line_coeff);
    window_corner_proj_ptr->loleft.x = find_coord(window_corners_ptr->loleft,
            samp_coeff);
    window_corner_proj_ptr->loleft.y = find_coord(window_corners_ptr->loleft,
            line_coeff);
    window_corner_proj_ptr->loright.x = find_coord(window_corners_ptr->loright,
            samp_coeff);
    window_corner_proj_ptr->loright.y = find_coord(window_corners_ptr->loright,
            line_coeff);
}
