/*****************************************************************************
NAME:           check_buffers

PURPOSE:
  Verify that the available buffer space is enough to handle the given
  scenario and set the output image active line count.

DEVELOPMENT HISTORY:

NOTES:
  none

ALGORITHM:
  Calculate the number of lines that will be active in the output image.
  Since the grid is a reverse grid, figure out how many input lines are
  active.  Then scale it to the output image size.  The number of active
  lines is a "guess" at how many lines can only be partially generated from
  the input data.  This helps to figure out how much memory is required to
  efficiently buffer the output image.  See RESAMPLE design document,
  Section 7.1 for a better explanation.

ALGORITHM REFERENCES:
  RESAMPLE design document, Section 7.1

*****************************************************************************/

#include <string.h>
#include <math.h>
#include "resample.h"


int check_buffers
(
    int iband,                    /* I: current band (indexed)              */
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr,
                                  /* I: pointer to grid band                */
    int in_nl,                    /* I: input image number of lines         */
    int in_ns,                    /* I: input image number of samples       */
    int lines_to_buffer,          /* I: number of input lines to buffer     */
    int min_lines_to_buffer,      /* I: min number of input lines to buffer */
    int *active_lines,            /* O: number of lines active in the output
                                        image                               */
    int *min_active_lines         /* O: minimum number of lines active in
                                        the output image                    */
)
{
    const WINDOW_TYPE *output_window_ptr = get_output_window(iband);
                                  /* ptr to output window geometry          */

    int min_overlap = 0;          /* min/max input line overlap             */
    int max_overlap = 0;
    int max_line, min_line;       /* max/min input line/sample values       */
    int max_sample, min_sample;
    int output_lines[2];          /* output space line/sample pairs         */
    int output_samples[2];

    double dtemp;                 /* temporary storage                      */
    double in_lines;           /* number of input space lines corresponding */
    double min_in_lines;       /* to the active lines in output space       */
    double ul_line, ul_sample;    /* input space line/sample pairs          */
    double ll_line, ll_sample;
    double ur_line, ur_sample;
    double ratio;                 /* output space to input space
                                     dimensioning ratio */


    /* Convert output image UL, LL, and UR corners to input line/samples,
       and set the min/max values.
       ------------------------------------------------------------------*/
    if (ols2ils(grid_band_ptr, output_window_ptr->start_line,
                output_window_ptr->start_sample, &ul_line,
                &ul_sample) != E_SUCC)
    {
        c_lasmsg(FALSE,"resample-ulconv", "Unable to convert UL corner "
                "from output space to input space.");

        /* In break region, so we cannot retrieve the input space
           upper left corner.  We need to make sure we set up enough
           output buffers to handle the entire input image.
           ---------------------------------------------------------*/
        ul_line = 0.;
        ul_sample = 0.;
        min_line = 0.;
        min_sample = 0.;
        max_line = in_nl;
        max_sample = in_ns;
    }
    else if (ols2ils(grid_band_ptr, output_window_ptr->end_line-1,
                output_window_ptr->start_sample, &ll_line,
                &ll_sample) != E_SUCC)
    {
        c_lasmsg(FALSE,"resample-llconv", "Unable to convert LL corner "
                "from output space to input space.");

        /* In break region, so we cannot retrieve the input space
           lower left corner.  We need to make sure we set up enough
           output buffers to handle the entire input image.
           ---------------------------------------------------------*/
        ll_line = ul_line + in_nl;
        ll_sample = ul_sample;
        min_line = 0.;
        min_sample = 0.;
        max_line = in_nl;
        max_sample = in_ns;
    }
    else if (ols2ils(grid_band_ptr, output_window_ptr->start_line,
                output_window_ptr->end_sample-1, &ur_line,
                &ur_sample) != E_SUCC)
    {
        c_lasmsg(FALSE,"resample-urconv", "Unable to convert UR corner "
                "from output space to input space.");

        /* In break region, so we cannot retrieve the input space
           lower left corner.  We need to make sure we set up enough
           output buffers to handle the entire input image.
           ---------------------------------------------------------*/
        ur_line = ul_line;
        ur_sample = ul_sample + in_ns;
    }
    else
    {
        min_line = min(ul_line, ll_line);
        max_line = max(ul_line, ll_line);
        min_sample = min(ul_sample, ll_sample);
        max_sample = max(ul_sample, ll_sample);

        /* Check to see if intermediate points along the left edge will
           alter the min/max line values.
           ------------------------------------------------------------*/
        output_lines[0] = output_window_ptr->start_line;
        output_lines[1] = output_window_ptr->end_line - 1;
        output_samples[0] = output_samples[1] =
            output_window_ptr->start_sample;
        check_point(grid_band_ptr, output_lines, output_samples, &min_line,
                &max_line, &min_sample, &max_sample,
                &min_overlap, &max_overlap);
    }

    /* Determine ratio of output lines to input lines.

       Ratio =      length of output line
       -------------------------------
       length of equivalent input line
       -----------------------------------------------------*/
    ratio = output_window_ptr->num_lines/
        sqrt(((double)max_line - min_line)*((double)max_line - min_line) +
           ((double)max_sample - min_sample)*((double)max_sample - min_sample));

    /* Determine the number of input space lines and samples spanned by
       the top edge of the output window.  If the number of input space
       samples is greater than the number of samples in the input image,
       scale the number of input space lines appropriately.  The number
       of input space lines corresponding to the output space active line
       count is the number of lines spanned by the output window plus the
       number of input lines buffered.
       ------------------------------------------------------------------*/
    dtemp = fabs(ur_sample - ul_sample);
    if (dtemp > in_ns)
        dtemp = in_ns/dtemp;
    else
        dtemp = 1.;
    in_lines = lines_to_buffer + dtemp*fabs(ur_line - ul_line);
    min_in_lines = min_lines_to_buffer + dtemp*fabs(ur_line - ul_line);
    if (in_lines > in_nl)
        in_lines = in_nl;
    if (min_in_lines > in_nl)
        min_in_lines = in_nl;

    /* Convert input space line length to output space.
       (1.2 is a fudge factor to allow for the fact that this
       computation is a rough estimate.)
       ------------------------------------------------------*/
    dtemp = ll_line - ul_line;
    if (fabs(dtemp) > 1.)
        dtemp = (ll_sample - ul_sample)/dtemp;
    else
        dtemp = ll_sample - ul_sample;
    *active_lines = 1.2*in_lines*ratio*sqrt(1. + dtemp*dtemp);
    *min_active_lines = 1.2*min_in_lines*ratio*sqrt(1. + dtemp*dtemp);

    /* Clamp active lines to number of lines in output image.
       ------------------------------------------------------*/
    if (*active_lines > output_window_ptr->num_lines)
        *active_lines = output_window_ptr->num_lines;
    if (*min_active_lines > output_window_ptr->num_lines)
        *min_active_lines = output_window_ptr->num_lines;

    return (E_SUCC);
}
