/*****************************************************************************
NAME:                   check_grid

PURPOSE:   Checks correctness of the geometric mapping grid

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.0           4/89    D. Steinwand       CSB      LAS 5.0 development
  5.01          8/89    D. Steinwand       CSB      Fixed bug with 270 deg rot

ALGORITHM DESCRIPTION:
        Loop on geometric mapping grid columns
           Loop on each line coordinate
                If not increasing,
                  Loop on grid columns
                   Loop on each line coordinate
                        If not decreasing, abort 
        Loop on geometric mapping grid rows
           Loop on each sample coordinate
                If not increasing, 
                  Loop on grid rows
                    Loop on each sample coordinate
                        If not decreasing, abort
        Return (E_SUCC)
*****************************************************************************/
#include <math.h>
#include "resample.h"

#define INCREASING 0
#define DECREASING 1
#define SAMPSAMP 0
#define LINESAMP 1


long check_grid
(
    struct GEOGRID *map   /* I: geometric mapping grid                      */
)
{
    register long i;          /* array index                                */
    register long curcol;     /* column counter                             */
    register long currow;     /* row counter                                */
    register long limit;      /* loop limits                                */
    int line_dir, samp_dir;   /* line, sample "growth" flags                */
    int cor_mode;             /* Correlation mode.  If output samples are more
                                 closely correlated with input samples than
                                 input lines, cor_mode = 0.  Otherwise
                                 cor_mode = 1.  This handles cases where the
                                 grid might define a +/-90 degree rotation. */


    /* Determine the input/output correlation.
       ---------------------------------------*/
    if (fabs(map->in_samps[0] - map->in_samps[1]) >
            fabs(map->in_lines[0] - map->in_lines[1]))
        cor_mode = SAMPSAMP;
    else
        cor_mode = LINESAMP;

    if (cor_mode == SAMPSAMP)
    {
        /* Determine grid line and sample directions.
           ------------------------------------------*/
        if (map->in_lines[0] < map->in_lines[map->ncols])
            line_dir = INCREASING;
        else
            line_dir = DECREASING;
        if (map->in_samps[0] < map->in_samps[1])
            samp_dir = INCREASING;
        else
            samp_dir = DECREASING;

        /* Check mapping grid input lines.  These must be strictly
           increasing or strictly decreasing.  Any wrapping of the grid in
           the line direction is invalid.
           ---------------------------------------------------------------*/
        limit = (map->nrows - 1) * map->ncols;
        for (curcol = 0; curcol < map->ncols; curcol++)
            for (i = curcol; i < limit; i += map->ncols)
                if (((line_dir == INCREASING) &&
                     (map->in_lines[i] > map->in_lines[i+map->ncols])) ||
                    ((line_dir == DECREASING) &&
                     (map->in_lines[i] < map->in_lines[i+map->ncols])))
                {
                    c_lasmsg(FALSE, "resample-wrap",
                            "Wrapping of grid detected at or"
                            " around row %ld, column %ld.",
                            (i/map->ncols+1), (curcol+1));
                    return(GMGLIN);
                }

        /* Check mapping grid input samples.  These must also be
           strictly increasing or strictly decreasing.  Any wrapping
           of the grid in the sample direction is invalid.
           ---------------------------------------------------------*/
        limit = map->ncols - 1;
        for (currow = 0; currow < map->nrows; currow++)
        {
            i = currow*map->ncols;
            for (curcol = 0; curcol < limit; curcol++, i++)
                if (((samp_dir == INCREASING) &&
                            (map->in_samps[i] > map->in_samps[i+1])) ||
                        ((samp_dir == DECREASING) &&
                         (map->in_samps[i] < map->in_samps[i+1])))
                {
                    c_lasmsg(FALSE, "resample-wrap",
                            "Wrapping of grid detected at or"
                            " around row %ld, column %ld",
                            (currow+1), (curcol+1));
                    return(GMGSAM);
                }
        }
    }
    else   /* cor_mode = LINESAMP */
    {
        /* Determine grid line and sample directions.
           ------------------------------------------*/
        if (map->in_lines[0] < map->in_lines[1])
            line_dir = INCREASING;
        else
            line_dir = DECREASING;
        if (map->in_samps[0] < map->in_samps[map->ncols])
            samp_dir = INCREASING;
        else
            samp_dir = DECREASING;

        /* Check mapping grid input samples.  These must be strictly
           increasing or strictly decreasing.  Any wrapping of the grid in
           the line direction is invalid.
           ---------------------------------------------------------------*/
        limit = (map->nrows - 1) * map->ncols;
        for (curcol = 0; curcol < map->ncols; curcol++)
            for (i = curcol; i < limit; i += map->ncols)
                if (((samp_dir == INCREASING) &&
                        (map->in_samps[i] > map->in_samps[i+map->ncols])) ||
                        ((samp_dir == DECREASING) &&
                         (map->in_samps[i] < map->in_samps[i+map->ncols])))
                {
                    c_lasmsg(FALSE, "resample-wrap",
                            "Wrapping of grid detected at or"
                            " around row %ld, column %ld.",
                            (i/map->ncols+1), (curcol+1));
                    return(GMGLIN);
                }

        /* Check mapping grid input lines.  These must also be
           strictly increasing or strictly decreasing.  Any wrapping
           of the grid in the sample direction is invalid.
           ---------------------------------------------------------*/
        limit = map->ncols - 1;
        for (currow = 0; currow < map->nrows; currow++)
        {
            i = currow*map->ncols;
            for (curcol = 0; curcol < limit; curcol++, i++)
                if (((line_dir == INCREASING) &&
                        (map->in_lines[i] > map->in_lines[i+1])) ||
                        ((line_dir == DECREASING) &&
                         (map->in_lines[i] < map->in_lines[i+1])))
                {
                    c_lasmsg(FALSE, "resample-wrap",
                            "Wrapping of grid detected at or"
                            " around row %ld, column %ld",
                            (currow+1), (curcol+1));
                    return(GMGSAM);
                }
        }

    }  /* cor_mode */

    return(E_SUCC);
}
