/*****************************************************************************
NAME                  G_UPDDR

PURPOSE      Updates records 1 and 2 of a DDR file with projection information
             from a geometric mapping grid

PROGRAM HISTORY
PROGRAMMER      DATE        REASON
----------      ----        ------
B. Ailts        Jan. 1988     original development of c_upddr
D. Steinwand    Apr. 1989     Modified C_UPDDR to G_UPDDR to update
                              images which have been geometrically warped
T. Mittan       Jun. 1992     Modified to update DDR corners when
                              window applied.
PROJECT       LAS

ALGORITHM
    Allocate space for all output band DDRs
    Allocate space for an input band DDR
    Get the input image DDR
    Set master line & sample
    Check the projection info contained in the geometric mapping grid
    If valid,
       Update projection information in the output image DDR
    Else
       Set projection fields in output image DDR to invalid
    Copy band information to new DDR
    Write the new information to disk
    Write each band DDR to disk
    Return

ALGORITHM REFERENCES    LAS 5.0 c_upddr by B. Ailts
*****************************************************************************/
#include <stdlib.h>
#include "resample.h"
#include "cproj.h"

int g_upddr
(
    char host_in[][CMLEN],  /*  array of input host names                   */
    char *host_out,         /*  Output host name                            */
    long window[4],         /*  Input or output window (depending on winmod)
                                specification                               */
    long bands[][MAXBND+1], /*  band numbers for each input image           */
    long nbands,            /*  Number of bands in output image             */
    struct GEOGRID *grid,   /*  Geometric mapping grid                      */
    long winmod,            /*  INPUT/OUTPUT window flag                    */ 
    struct DDR *iddr        /*  input ddr                                   */
)
{
    static struct DDR oddr;     /*  output ddr                               */
    struct BDDR *ibdr;          /*  input band dependent ddr records         */
    struct BDDR *obdr;          /*  output band dependent ddr record         */

    long bndcnt;                /*  Loop counter on bands                    */
    long degrees = DEGREE;      /*  unit code for degrees                    */
    long geo = 0;               /*  projection code for geographic           */
    long i;                     /*  loop counter                             */
    long icoords[8];            /*  input image coords                       */
    long imgcnt;                /*  Loop counter on images                   */
    long message = FALSE;       /*  message for rtcoef                       */
    long meters = METER;        /*  unit code for meters                     */
    long prtprm[2];             /*  print parameter                          */
    long units1;                /*  grid units                               */

    double coef[6];             /*  coef for calculating projection units    */
    double max_x, min_x;        /*  temporary max and min x                  */
    double max_y, min_y;        /*  temporary max and min y                  */
    double outx, outy;          /*  transformed xy pair                      */


    /* Allocate space for the input DDRs.
       ----------------------------------*/
    if ((obdr = (struct BDDR *)calloc(nbands,sizeof(struct BDDR))) == NULL)
    {
        c_lasmsg(FALSE,"resample-alloc","Memory allocation error");
        return(E_FAIL);
    }
    if ((ibdr = (struct BDDR *)malloc(sizeof(struct BDDR))) == NULL)
    {
        c_lasmsg(FALSE,"resample-alloc","Memory allocation error");
        return(E_FAIL);
    }

    /* Retreive the output ddr information.  This is done so that the
       information placed in the ddr by eopen_ will not be lost.
       --------------------------------------------------------------*/
    if (c_getddr(host_out,&oddr) != E_SUCC)
    {
        c_lasmsg(FALSE,"resample-ddrget","Error retrieving ddr information.");
        return(E_FAIL);
    }

    /* Write band independent information (including projection information
       contained in the geometric mapping grid.
       --------------------------------------------------------------------*/
    oddr.master_line = grid->out_lines[0];
    oddr.master_sample = grid->out_samps[0];
    oddr.line_inc = oddr.sample_inc = 1.0;

    if (grid->proj_valid == GEO_VALID)
    {
        /* If window specified, adjust corner cordinates.
           ----------------------------------------------*/
        if (winmod == OUTWIND)
        {
            /* If units are not in meters, convert to meters, find window,
               convert back to grid units.  It appears from c_getddr that
               the units are always converted to lower case upon ingest.
               -----------------------------------------------------------*/
            if ((strcmp(grid->units, "meters") != 0) && (grid->code != 0))
            {
                if (strcmp(grid->units,"radians") == 0 )
                    units1 = RADIAN;
                else if (strcmp(grid->units,"feet") == 0 )
                    units1 = FEET;
                else if (strcmp(grid->units,"seconds") == 0 )
                    units1 = SECOND;
                else if (strcmp(grid->units,"degrees") == 0 )
                    units1 = DEGREE;
                else if (strcmp(grid->units,"dms") == 0 )
                    units1 = DMS;
                else
                {
                    c_lasmsg(FALSE, "resample-units", "Unsupported units.");
                    return E_FAIL;
                }

                if (c_transinit(&grid->code, &units1, &grid->zone,
                            &grid->datum, grid->projprms, &grid->code,
                            &meters, &grid->zone, &grid->datum,
                            grid->projprms, prtprm, "\0") != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners.");
                    return(E_FAIL);
                }

                if (c_trans(&grid->code, &units1, &grid->code, &meters,
                            &grid->corners[1], &grid->corners[0], &outx, &outy)
                        != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[1] = outx;
                grid->corners[0] = outy;

                if (c_trans(&grid->code, &units1, &grid->code, &meters,
                            &grid->corners[7], &grid->corners[6], &outx, &outy)
                        != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[7] = outx;
                grid->corners[6] = outy;

                if (c_trans(&grid->code, &units1, &grid->code, &meters,
                            &grid->corners[5], &grid->corners[4], &outx, &outy)
                        != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[5] = outx;
                grid->corners[4] = outy;

                if (c_trans(&grid->code, &units1, &grid->code, &meters,
                            &grid->corners[3], &grid->corners[2], &outx, &outy)
                        != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[3] = outx;
                grid->corners[2] = outy;
            }

            icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
            icoords[2] = icoords[6] = grid->lines - grid->out_lines[0] + 1;
            icoords[5] = icoords[7] = grid->samples - grid->out_samps[0] + 1;

            /* Compute the coefficients that define the transformation between
               line/sample and projection coordinates.
               ---------------------------------------------------------------*/
            if (c_rtcoef(grid->corners, icoords, coef, &message) != E_SUCC)
            {
                c_lasmsg(FALSE, "resample-ddrcal",
                        "Error calculating ddr corners");
                return(E_FAIL);
            }

            /* Compute the min/max xy projection coordinates for the window,
               and copy into the grid structure.
               -------------------------------------------------------------*/
            max_y = coef[0]*window[SL] + coef[1]*window[SS] + coef[2];
            min_x = coef[3]*window[SL] + coef[4]*window[SS] + coef[5];
            min_y = coef[0]*(window[SL] + window[NL] - 1) +
                coef[1]*(window[SS] + window[NS] - 1) + coef[2];
            max_x = coef[3]*(window[SL] + window[NL] - 1) +
                coef[4]*(window[SS] + window[NS] - 1) + coef[5];
            grid->corners[0] = max_y;
            grid->corners[1] = min_x;
            grid->corners[2] = min_y;
            grid->corners[3] = min_x;
            grid->corners[4] = max_y;
            grid->corners[5] = max_x;
            grid->corners[6] = min_y;
            grid->corners[7] = max_x;

            /* If grid units are not in meters, covert corner coordinates to
               grid units.
               -------------------------------------------------------------*/
            if ((strcmp(grid->units, "meters") != 0) && (grid->code != 0))
            {
                if (c_transinit(&grid->code, &meters, &grid->zone,
                            &grid->datum, grid->projprms, &grid->code,
                            &units1, &grid->zone, &grid->datum,
                            grid->projprms, prtprm, "\0") != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }

                if (c_trans(&grid->code, &meters, &grid->code, &units1,
                            &grid->corners[1], &grid->corners[0],
                            &outx, &outy) != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[1] = outx;
                grid->corners[0] = outy;

                if (c_trans(&grid->code, &meters, &grid->code, &units1,
                            &grid->corners[7], &grid->corners[6],
                            &outx, &outy) != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[7] = outx;
                grid->corners[6] = outy;

                if (c_trans(&grid->code, &meters, &grid->code, &units1,
                            &grid->corners[5], &grid->corners[4],
                            &outx, &outy) != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[5] = outx;
                grid->corners[4] = outy;

                if (c_trans(&grid->code, &meters, &grid->code, &units1,
                            &grid->corners[3], &grid->corners[2],
                            &outx, &outy) != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error calculating ddr corners");
                    return(E_FAIL);
                }
                grid->corners[3] = outx;
                grid->corners[2] = outy;
            }
        }
        else if (winmod == INWIND)
        {
            for (i = 0; i < 7; i++)
                if (iddr->valid[i] != VALID)
                {
                    c_lasmsg(FALSE, "resample-ddrcal",
                            "Error retrieving input ddr information");
                    return(E_FAIL);
                }

            /* Convert corner coordinates to degrees.
               --------------------------------------*/
            if (strcmp(iddr->proj_units, "radians") == 0 )
                units1 = RADIAN;
            else if (strcmp(iddr->proj_units, "feet") == 0 )
                units1 = FEET;
            else if (strcmp(iddr->proj_units, "meters") == 0 )
                units1 = METER;
            else if (strcmp(iddr->proj_units, "seconds") == 0 )
                units1 = SECOND;
            else if (strcmp(iddr->proj_units, "degrees") == 0 )
                units1 = DEGREE;
            else if (strcmp(iddr->proj_units,"dms") == 0 )
                units1 = DMS;
            else
            {
                c_lasmsg(FALSE, "resample-units", "Unsupported units.");
                return E_FAIL;
            }
            max_y = iddr->upleft[0] - (window[SL] - 1) * iddr->pdist_y;
            min_x = iddr->upleft[1] + (window[SS] - 1) * iddr->pdist_x;
            min_y = max_y - ((iddr->nl - 1) * iddr->pdist_y);
            max_x = min_x + ((iddr->ns - 1) * iddr->pdist_x);
            if (c_transinit(&iddr->proj_code, &units1, &iddr->zone_code,
                        &iddr->datum_code, iddr->proj_coef, &geo,
                        &degrees, &iddr->zone_code, &iddr->datum_code,
                        iddr->proj_coef, prtprm, "\0") == E_FAIL)
            {
                c_lasmsg(FALSE, "resample-ddrcal",
                        "Error calculating ddr corners");
                return(E_FAIL);
            }

            if (c_trans(&iddr->proj_code, &units1, &geo, &degrees, &min_x,
                        &max_y, &grid->corners[1], &grid->corners[0]) != E_SUCC)
            {
                c_lasmsg(FALSE, "resample-ddrcal",
                        "Error calculating ddr corners");
                return(E_FAIL);
            }

            if (c_trans(&iddr->proj_code, &units1, &geo, &degrees, &max_x,
                        &min_y, &grid->corners[7], &grid->corners[6]) != E_SUCC)
            {
                c_lasmsg(FALSE, "resample-ddrcal",
                        "Error calculating ddr corners");
                return(E_FAIL);
            }

            if (c_trans(&iddr->proj_code, &units1, &geo, &degrees, &min_x,
                        &min_y, &grid->corners[3], &grid->corners[2]) != E_SUCC)
            {
                c_lasmsg(FALSE, "resample-ddrcal",
                        "Error calculating ddr corners");
                return(E_FAIL);
            }

            if (c_trans(&iddr->proj_code, &units1, &geo, &degrees, &max_x,
                        &max_y, &grid->corners[5], &grid->corners[4]) != E_SUCC)
            {
                c_lasmsg(FALSE, "resample-ddrcal",
                        "Error calculating ddr corners");
                return(E_FAIL);
            }

            /* Find the min, max lat/long values.
               ----------------------------------*/
            max_y = GMAX(grid->corners[0],grid->corners[2]);
            max_y = GMAX(max_y, grid->corners[4]);
            max_y = GMAX(max_y, grid->corners[6]);
            max_x = GMAX(grid->corners[1],grid->corners[3]);
            max_x = GMAX(max_x, grid->corners[5]);
            max_x = GMAX(max_x, grid->corners[7]);
            min_y = GMIN(grid->corners[0],grid->corners[2]);
            min_y = GMIN(min_y, grid->corners[4]);
            min_y = GMIN(min_y, grid->corners[6]);
            min_x = GMIN(grid->corners[1],grid->corners[3]);
            min_x = GMIN(min_x, grid->corners[5]);
            min_x = GMIN(min_x, grid->corners[7]);

            /* Compute the corresponding min, max projection coordinates
               that contain the rectangle defined above for the Geographic
               projection.
               -----------------------------------------------------------*/
            if (c_mapedg(&min_x, &max_x, &max_y, &min_y, &degrees,
                        &grid->code, &grid->zone, grid->projprms, &units1,
                        &grid->corners[1], &grid->corners[7],
                        &grid->corners[6], &grid->corners[0],
                        &grid->datum) != E_SUCC)
            {
                c_lasmsg(FALSE, "resample-ddrcal",
                        "Error calculating ddr corners");
                return(E_FAIL);
            }

            grid->corners[2] = grid->corners[6];
            grid->corners[3] = grid->corners[1];
            grid->corners[4] = grid->corners[0];
            grid->corners[5] = grid->corners[7];
        }

        oddr.proj_code = grid->code;
        oddr.zone_code = grid->zone;
        oddr.datum_code = grid->datum;
        oddr.valid[0] = VALID;
        oddr.valid[1] = VALID;
        oddr.valid[2] = VALID;
        oddr.valid[3] = VALID;
        oddr.valid[4] = VALID;
        oddr.valid[5] = VALID;
        oddr.valid[6] = VALID;
        oddr.valid[7] = VALID;
        strcpy(oddr.proj_units, grid->units);
        for (i = 0; i < 15; i++)
            oddr.proj_coef[i] = grid->projprms[i];
        oddr.pdist_y = grid->pdist[0];
        oddr.pdist_x = grid->pdist[1];
        oddr.upleft[0] = grid->corners[0];
        oddr.upleft[1] = grid->corners[1];
        oddr.loleft[0] = grid->corners[2];
        oddr.loleft[1] = grid->corners[3];
        oddr.upright[0] = grid->corners[4];
        oddr.upright[1] = grid->corners[5];
        oddr.loright[0] = grid->corners[6];
        oddr.loright[1] = grid->corners[7];
    }
    else        /* Grid projection not valid. */
    {
        for (i=0; i<8; i++)
            oddr.valid[i] = INVAL;
    }

    /* Update the band-related info for the output DDR.
       ------------------------------------------------*/
    for(bndcnt = 0, imgcnt = 0, i = 0; i < nbands; i++, bndcnt++)
    {
        /* If no more bands in the present image, go to next image.
           --------------------------------------------------------*/
        if (bands[imgcnt][bndcnt] == 0)
        {
            imgcnt++;
            bndcnt = 0;
        }

        /* Retreive the band DDR information for the present band of the
           present image.
           -------------------------------------------------------------*/
        if (c_getbdr(host_in[imgcnt],ibdr,&bands[imgcnt][bndcnt]) != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-ddrget",
                    "Error retrieving ddr information");
            return(E_FAIL);
        }

        /* Copy the band fields of the input DDR to the band fields of
           the output DDR except for minval, maxval, bandno, and validity.
           ---------------------------------------------------------------*/
        obdr[i].bandno = i + 1;
        strcpy(obdr[i].source, ibdr->source);
        strcpy(obdr[i].instrument, ibdr->instrument);
        strcpy(obdr[i].direction, ibdr->direction);
        strcpy(obdr[i].date, ibdr->date);
        strcpy(obdr[i].time, ibdr->time);
    }

    /* Write the band independent information to the output ddr.
       ---------------------------------------------------------*/
    if (c_putddr(host_out, &oddr) != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-ddrwrite", "Error writing to the ddr file.");
        return(E_FAIL);
    }

    /*  Write the band-related info to the output ddr.
        -----------------------------------------------*/
    for (i = 0; i < nbands; i++)
    {
        if (c_putbdr(host_out,&(obdr[i])) != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-ddrwrite",
                    "Error writing to the ddr file");
            return(E_FAIL);
        }
    }

    free(ibdr);
    free(obdr);
    return (E_SUCC);
}
