#ifndef _GEOM_GRID_H_
#define _GEOM_GRID_H_

/* The GRID structure contains a geometric mapping grid for mapping values
   between an input and output image.  The mapping may be defined to be 
   either input to output (FORWARD), or output to input (REVERSE).  A grid 
   will have many grid cells, depending on the grid cell size and the image 
   size.

   The following description applies to a reverse grid since that is the 
   most common type for resampling.

   The output image is divided into a regular grid.  Each grid cell is 
   cell_lines by cell_samps.  The out_lines array holds the line numbers where 
   the grid cell corners lie and the out_samps array holds the sample numbers 
   where the grid cell corners lie.  The out_lines array is num_out_lines by 1 
   and the out_samps array is num_out_samps by 1.  Using both the out_lines
   and out_samps arrays, the entire grid layout can be determined.

   The input space grid is defined by the in_lines and in_samps arrays.  Both
   the in_lines and in_samps arrays are num_out_lines by num_out_samps in
   size.  They hold the input space line and sample number for each grid
   intersection in the output grid.  For flexibily, the size of the in_lines
   and in_samps arrays are specified in num_in_lines and num_in_samps.

   The reverse_coeffs array defines the coefficients used to convert an output
   space (line,sample) into input space (line,sample).  It is grid_rows by 
   grid_cols in size.  Typically, grid_rows is num_out_lines - 1 and grid_cols
   is num_out_samps -1.  However there are exceptions to this, such as landsat
   satellite grids which have grid_rows = num_in_lines/2 (landsat grids are
   forward grids).  The forward_coeffs array defines the coefficients used to
   convert an input space (line,sample) into output space (line,sample).  It
   is optional for a reverse grid (except in the case of landat grids).
   

   EXAMPLE:
      An output image with 60 lines and 120 samps.  In the diagram the grid
      points are marked with the letters A through P.  Assume that each grid
      point has an x and y value associated with it.

      cell_lines = 20
      cell_samps = 40
      num_out_lines = 4
      num_out_samps = 4
      out_lines   = 0 20 40 60
      out_samps   = 0 40 80 120
      num_in_lines = 16
      num_in_samps = 16
      in_lines  = Ay By Cy Dy Ey Fy Gy Hy Iy Jy Ky Ly My Ny Oy Py
      in_samps  = Ax Bx Cx Dx Ex Fx Gx Hx Ix Jx Kx Lx Mx Nx Ox Px
      grid_rows = 3
      grid_cols = 3
      reverse_coeffs = R0 R1 R2 R3 R4 R5 R6 R7 R8
      
      A----B----C----D
      |    |    |    |
      | R0 | R1 | R2 |
      |    |    |    |
      E----F----G----H
      |    |    |    |
      | R3 | R4 | R5 |
      |    |    |    |
      I----J----K----L
      |    |    |    |
      | R6 | R7 | R8 |
      |    |    |    |
      M----N----O----P


   For a forward grid, the above discussion applies by swapping the "in" and 
   "out".
    
Notes: 
- everything is numbered starting at zero.
- the size of "int" is assumed to be 32 bits or greater.

============================================================================*/

/*
**
** DEFINES
**
*/

#define GRIDNAME_SIZE 256
#define UNITS_SIZE 12
#define DATUM_SIZE 16
#define PROJPRMS_SIZE 15
#define CORNERS_SIZE 4
#define COEFS_SIZE 4
#define XY_SIZE 2

/*
**
** TYPE DEFINITIONS
**
*/

typedef struct DBL_XY
    {
    double x;			/* X value                           */
    double y;			/* Y value                           */
    } DBL_XY;

typedef struct DBL_LAT_LONG
{
  double lat;              /* Latitude value                    */
  double lng;              /* Longitude value                   */
} DBL_LAT_LONG;

typedef struct CORNERS
{
  DBL_XY upleft;    /* X/Y value of the upper left corner  */
  DBL_XY upright;   /* X/Y value of the upper right corner */
  DBL_XY loleft;    /* X/Y value of the lower left corner  */
  DBL_XY loright;   /* X/Y value of the lower right corner */
} CORNERS;

typedef struct COEFFICIENTS
{
  double a[COEFS_SIZE];    /* Array of a coefficients           */
  double b[COEFS_SIZE];    /* Array of b coefficients           */
} COEFFICIENTS;

typedef enum
{
    FORWARD_GEOMETRIC_GRID,
    REVERSE_GEOMETRIC_GRID
} GEOM_GRID_TYPE;
/* types of grids defined */


typedef enum 
{
    UNKNOWN_FRAME = -1, 
    GEOBOX_FRAME=1, 
    PROJBOX_FRAME, 
    UL_SIZE_FRAME, 
    MINBOX_FRAME, 
    PATH_ORIENTED_FRAME
} FRAME_TYPE;
/* defines the types of frames allowed */


typedef struct 
{
/* ==== Grid coefficients and other defining info. ====  */
  int band_number;        /* band number                                    */
  GEOM_GRID_TYPE gridtype;/* type of grid this defines                      */

  int cell_lines;         /* lines in each grid cell                        */
  int cell_samps;         /* samples in each grid cell                      */

  double pixsize_line;    /* projection distance per pixel in the line 
                             direction (in meters)                          */
  double pixsize_samp;    /* projection distance per pixel in the sample
                             direction (in meters)                          */
 
  int output_image_lines; /* number of lines in the output image            */
  int output_image_samps; /* number of samples in the output image          */

  int num_in_lines;       /* number of elements in the in_lines array       */
  int num_in_samps;       /* number of elements in the in_samps array       */

  double *in_lines;       /* pointer to the array of num_in_lines input space 
                             lines                                          */
  double *in_samps;       /* pointer to the array of num_in_samps input space 
                             samples                                        */

  int num_out_lines;      /* number of elements in the out_lines array      */
  int num_out_samps;      /* number of elements in the out_samps array      */

  double *out_lines;      /* pointer to the array of num_out_lines output
                             space lines                                    */
  double *out_samps;      /* pointer to the array of num_out_samps output
                             space samples                                  */

  int forward_present;    /* flag to indicate the forward coefficients are
                             included in the grid                           */
  int reverse_present;    /* flag to indicate the reverse coefficients are
                             include in the grid                            */
                             
  int grid_rows;          /* number of rows in the grid                     */
  int grid_cols;          /* number of columns in the grid                  */
  
  COEFFICIENTS *forward_coeffs;
                          /* pointer to the array of grid_rows * grid_cols 
                             coefficients for input space to output space 
                             transformations.                               */

  COEFFICIENTS *reverse_coeffs;
                          /* pointer to the array of grid_rows * grid_cols 
                             coefficients for output space to input space 
                             transformations.                               */

  int degree;             /* Degree of the rough transformation polynomial for
                             a forward grid.  There are (degree+1)^2 data
                             elements in the poly_lines and poly_samps
                             arrays.                                        */

  double *poly_lines;     /* Reverse rough transformation coefficients for
                             the lines.  These are for the grid_cell_poly
                             routine.  There are (degree+1)^2 data elements
                             in this pointer.                               */

  double *poly_samps;     /* Reverse rough transformation coefficients for
                             the samples.  These are for the grid_cell_poly
                             routine.  There are (degree+1)^2 data elements
                             in this pointer.                               */

} gxx_GEOM_GRID_BAND_TYPE;
/* defines the information that may be maintained for a band of a grid */


typedef struct
{
/* ==== General grid info. ==== */
  char gridname[GRIDNAME_SIZE]; /* host file name of the grid file          */

  GEOM_GRID_TYPE gridtype;/* type of grid                                   */

  int nbands;             /* number of bands stored in the grid             */

  int *bandlist;          /* list of valid bands in the grid                */

  int dtype;              /* The data type of all the values to be stored in
                             the grid.  Valid values are
                               BYTE   = unsigned char
                               WORD   = int (16 bit)
                               LONG   = long (32 bit)
                               REAL   = float
                               DOUBLE = double
                               CHAR   = char                                */
        /* ??? dtype is never used and can't be since the data is
               hardwired to be double... ???                                */

  FRAME_TYPE frame_type;  /* Type of framing process to be used.  Valid
                             values are
                               GEOBOX - User specifies UL and LR lat/long
                                 coordinates.
                               PROJBOX - User specifies LR output projection
                                 coordinates.  The user also specifies
                                 another point in output projection space,
                                 along with its coordinating line/sample in
                                 image space.
                               UL_SIZE - User specifies UL output projection
                                 coordinate and the number of lines and
                                 samples in output space.
                               MINBOX - Minbox framing.
                               PATH_ORIENTED - Path-oriented framing.       */

/* ==== Projection parameters and info. ====                                */
  char units[UNITS_SIZE]; /* Projection units.  This value corresponds to the
                             proj_units field in the DDR.                   */

  int code;               /* Projection code for the output space image's DDR.
                             Values for this field are defined in the "proj.h"
                             include file.                                  */

  char datum[DATUM_SIZE]; /* Projection datum code.  It has the same valid
                             range and defaults as the "datum_code" field in
                             the DDR.                                       */

  int spheroid;           /* Projection spheroid code                       */

  int zone;               /* Projection zone code for UTM or State Plane
                             projections.  It has the same valid ranges and
                             defaults as the "zone_code" field in the DDR.  */

  double projprms[PROJPRMS_SIZE];
                          /* Array of 15 projection coefficients as required
                             by the projection transformation package.  Refer
                             to the projection package documentation for a
                             description of each field for a given projection.
                             This field corresponds to the "proj_coef" field
                             in the DDR.                                    */

/* ==== Grid corners. ====                                                  */
  CORNERS corners;        /* Projection coordinates of the resulting output
                             image's four corners.  This field corresponds to
                             the DDR's "upleft", "loleft", "upright", and
                             "loright" fields.                              */

/* ==== Path-oriented info. ====                                            */
  DBL_LAT_LONG center_pt;
                          /* Lat/long of the WRS scene center point.        */

  DBL_XY proj_center_pt;  /* Proj x/y of the WRS scene center point.        */

  double rotate_angle;    /* Rotation angle of the path-oriented product.
                             (degrees)                                      */

  double orient_angle;    /*Frame orientation angle in map coords. (degrees)*/

/* ==== Individual band information. ====                                   */
  gxx_GEOM_GRID_BAND_TYPE *gridbands;
                          /* Array of specific grid band information        */
                             
} gxx_GEOM_GRID_TYPE;

#endif
