/*****************************************************************************
NAME: get_elev

PURPOSE:
Read the specified elevation image

DEVELOPMENT HISTORY:
Taken from Rev 1.6 in ias/gps/gxxLib/gxxHDF/gxx_get_elev.c

RETURN VALUE: 
Type = void *
Value           Description
-----           -----------
pointer to dem image data
NULL            Error reading elevation image

NOTES:          The elevation image is assumed to be in the same format as a 
                regular LAS image.
                This function uses the first band found in the DEM image.  It 
                allocates memory for the image data and returns a pointer to 
                the data.  It also passes back the size (nl, ns) and data type 
                of the data that was read.
                The caller is resposible for freeing this memory.

ALGORITHM REFERENCES:
None

*****************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <malloc.h>
#include "resample.h"


void *get_elev
(
    const char *dem_name,  /* I: name of the dem image file                 */
    int *dem_nl,           /* O: nl in dem image                            */
    int *dem_ns,           /* O: ns in dem image                            */
    OUTPUT_DATA_TYPE *dem_dtype /* O: data type of dem image                */
)
{
    struct FDESC *demfdsc;        /* DEM image file descriptor               */
    long band[MAXBND];            /* band #'s in dem image                   */
    void *dem_buf;                /* buffer of image data                    */
    long acc;                     /* newlas imageio access code              */
    long dtype;                   /* DEM data type                           */
    long nbands=0;                /* # of bands returned by open             */
    long nl[MAXBND];              /* nl of all bands in dem image            */
    long ns[MAXBND];              /* ns of all bands in dem image            */
    long size;                    /* size of data (1 pixel)                  */
    long one = 1;                 /* The number one.                         */
    long opt[2];                  /* option flags                            */
    long sl, ss;                  /* window info: start line, start sample   */


    /* Set the line/sample info.
       -------------------------*/
    sl = 1;
    ss = 1;
    nl[0] = 0;
    ns[0] = 0;

    acc = IREAD;
    opt[0] = 0;
    dtype = 0;

    /* Open the dem image.
       -------------------*/
    if (c_eopenr(&demfdsc, (char *)dem_name, &nbands, &sl, &ss, nl, ns,
                &acc, &dtype, opt, &one) != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-open",
                "Error opening dem image %s.", dem_name);
        return(NULL);
    }

    *dem_nl = nl[0];
    *dem_ns = ns[0];

    /* Make sure the data is 16-bit integers.
       --------------------------------------*/
    if (dtype != EWORD)
    {
        c_eclose(&demfdsc);
        c_lasmsg(FALSE, "resample-dtype",
                "DEM does not contain 16-bit integer data.", dem_name);
        return(NULL);
    }

    *dem_dtype = INT16_OUTPUT;
    size = EWORDSIZE;

    /* Allocate space for the image data.
       ----------------------------------*/
    dem_buf = malloc(*dem_nl * *dem_ns * size);
    if (dem_buf == NULL)
    {
        c_lasmsg(FALSE, "resample-alloc", "Memory allocation error");
        c_eclose(&demfdsc);
        return(NULL);
    }

    /* Read the full dem into the buffer.
       ----------------------------------*/
    band[0] = 1;
    if (c_eread(&demfdsc, &band[0], &one, (unsigned char *)dem_buf,&nl[0]) == 0)
    {
        free(dem_buf);
        c_lasmsg(FALSE, "resample-read",
                "Error reading dem image %s", dem_name);
        c_eclose(&demfdsc);
        return(NULL);
    }

    /* Close the dem.
       --------------*/
    if (c_eclose(&demfdsc) != E_SUCC)
    {
        free(dem_buf);
        c_lasmsg(FALSE, "resample-close",
                "Error closing dem image %s", dem_name);
        return(NULL);
    }

    return(dem_buf);
}
