/*****************************************************************************
NAME                          GET_GEOM_GRID_HEADER

PURPOSE    Load the IAS grid structure using the LAS grid.

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
L.D. Johnson          June 1998         original development

PROJECT       LAS

ALGORITHM
        Return

ALGORITHM REFERENCES
*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "lasmath.h"
#include "resample.h"


int get_geom_grid_headers
(
    const char *grid_filename,       /* I: grid file name                   */
    int *band_ptr,                   /* I: array of band numbers            */
    int nbands,                      /* I: number of bands                  */
    gxx_GEOM_GRID_TYPE *grid         /* O: IAS grid structure               */
)
{
    const struct GEOGRID *las_grid;   /* LAS grid structure                 */
    long ii, iband;                   /* loop counter                       */


    /* Get the geometric mapping grid.
       -------------------------------*/
    las_grid = get_las_grid();

    /* -------------------------------------------------------
       -------------------------------------------------------
       Load the grid structure with information from las_grid.
       -------------------------------------------------------
       -------------------------------------------------------*/

    strcpy(grid->gridname, grid_filename);
    grid->gridtype = REVERSE_GEOMETRIC_GRID;

    /* Allocate grid storage for band-related structure members.
       ---------------------------------------------------------*/
    grid->nbands = nbands;
    if ((grid->bandlist = (int *)malloc(grid->nbands*sizeof(int))) == NULL)
    {
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return E_FAIL;
    }
    if ((grid->gridbands = (gxx_GEOM_GRID_BAND_TYPE *)malloc(grid->nbands*
                    sizeof(gxx_GEOM_GRID_BAND_TYPE))) == NULL)
    {
        free (grid->bandlist);
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return E_FAIL;
    }

    for (iband = 0; iband<grid->nbands; iband++)
    {
        grid->bandlist[iband] = band_ptr[iband];

        grid->gridbands[iband].band_number = grid->bandlist[iband];
        grid->gridbands[iband].gridtype = REVERSE_GEOMETRIC_GRID;
        grid->gridbands[iband].cell_lines =
            (int)(las_grid->out_lines[1] - las_grid->out_lines[0]);
        grid->gridbands[iband].cell_samps = 
            (int)(las_grid->out_samps[1] - las_grid->out_samps[0]);
        grid->gridbands[iband].pixsize_line = las_grid->pdist[0];
        grid->gridbands[iband].pixsize_samp = las_grid->pdist[1];
        grid->gridbands[iband].output_image_lines =
            (int)(las_grid->lines - las_grid->out_lines[0] + 1);
        grid->gridbands[iband].output_image_samps =
            (int)(las_grid->samples - las_grid->out_samps[0] + 1);
        grid->gridbands[iband].num_out_lines = (int)las_grid->nrows;
        grid->gridbands[iband].num_out_samps = (int)las_grid->ncols;
        grid->gridbands[iband].num_in_lines =
            grid->gridbands[iband].num_in_samps =
            (int)(las_grid->nrows*las_grid->ncols);

        grid->gridbands[iband].grid_rows = (int)las_grid->nrows - 1;
        grid->gridbands[iband].grid_cols = (int)las_grid->ncols - 1;

    }

    grid->frame_type = UL_SIZE_FRAME;
    strcpy(grid->units, las_grid->units);
    grid->code = (int)las_grid->code;
    grid->zone = (int)las_grid->zone;
    for (ii=0; ii<15; ii++)
        grid->projprms[ii] = las_grid->projprms[ii];
    grid->corners.upleft.x = las_grid->corners[0];
    grid->corners.upleft.y = las_grid->corners[1];
    grid->corners.loleft.x = las_grid->corners[2];
    grid->corners.loleft.y = las_grid->corners[3];
    grid->corners.upright.x = las_grid->corners[4];
    grid->corners.upright.y = las_grid->corners[5];
    grid->corners.loright.x = las_grid->corners[6];
    grid->corners.loright.y = las_grid->corners[7];

    return (E_SUCC);
}


/*****************************************************************************
NAME                          GET_GEOM_GRID_DATA

PURPOSE    Load the IAS grid structure using the LAS grid.

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
L.D. Johnson          June 1998         original development

PROJECT       LAS

ALGORITHM
        Return

ALGORITHM REFERENCES
*****************************************************************************/

int get_geom_grid_data
(
    int *bands,              /* I: Array of bands to be read from the grid  */
    int nbands,              /* I: Number of bands in the array             */
    gxx_GEOM_GRID_TYPE *grid /* O: IAS grid structure                       */
)
{
    const struct GEOGRID *las_grid; /* LAS grid structure                    */

    gxx_GEOM_GRID_BAND_TYPE *cur_gbnd_ptr;
                                    /* ptr to current grid band structure    */

    int cur_band;                   /* current band in context               */
    int startband = 0;              /* first band to loop on                 */
    int iband, ii, jj, kk;          /* Loop counter                          */
    int ki, kilg;                   /* Array indices                         */
    long four = 4;                  /* The number 4                          */
    long zero = 0;                  /* The number 0                          */
    double aa[16];                  /* QR factorization matrix for computing
                                       coefficients                          */
    double work[4];                 /* work array for QR factorization       */


    /* This routine is set up to fill the IAS grid structure, which expects
       separate grids for each band.  Since the LAS model uses one grid for
       all bands, we can modify the loading to only load the first band data.
       For the remaining bands, the array pointers will be pointed at the
       data in the first band.  Note that this requires that the memory for
       the first band data not be freed until all work is done.
       ----------------------------------------------------------------------*/

    /* Use a local pointer to simplify the code. */
    cur_gbnd_ptr = &grid->gridbands[0];

    if (bands[0] == 0)
    {
        /* Set the first band to loop on (for setting data pointers) to
           be the second band in the list.
           ------------------------------------------------------------*/
        startband = 1;

        /* Read the geometric mapping grid.
           --------------------------------*/
        las_grid = get_las_grid();

        /* Load the grid structure with information from las_grid.
           -------------------------------------------------------*/

        /* Allocate space for arrays.
           --------------------------*/
        if ((cur_gbnd_ptr->in_lines =
                    (double *)malloc(cur_gbnd_ptr->num_in_lines*sizeof(double)))
                == NULL)
        {
            free (grid->bandlist);
            free (grid->gridbands);
            c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
            return E_FAIL;
        }
        if ((cur_gbnd_ptr->in_samps =
                    (double *)malloc(cur_gbnd_ptr->num_in_samps*sizeof(double)))
                == NULL)
        {
            free (cur_gbnd_ptr->in_lines);
            free (grid->bandlist);
            free (grid->gridbands);
            c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
            return E_FAIL;
        }
        if ((cur_gbnd_ptr->out_lines =
                   (double *)malloc(cur_gbnd_ptr->num_out_lines*sizeof(double)))
                == NULL)
        {
            free (cur_gbnd_ptr->in_lines);
            free (cur_gbnd_ptr->in_samps);
            free (grid->bandlist);
            free (grid->gridbands);
            c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
            return E_FAIL;
        }
        if ((grid->gridbands[0].out_samps =
              (double *)malloc(grid->gridbands[0].num_out_samps*sizeof(double)))
                == NULL)
        {
            free (cur_gbnd_ptr->in_lines);
            free (cur_gbnd_ptr->in_samps);
            free (cur_gbnd_ptr->out_lines);
            free (grid->bandlist);
            free (grid->gridbands);
            c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
            return E_FAIL;
        }
        cur_gbnd_ptr->forward_coeffs = (COEFFICIENTS *)NULL;
        if ((cur_gbnd_ptr->reverse_coeffs =
             (COEFFICIENTS *)malloc((cur_gbnd_ptr->num_out_lines - 1)*
             (cur_gbnd_ptr->num_out_samps - 1)*sizeof(COEFFICIENTS)))
            == NULL)
        {
            free (cur_gbnd_ptr->in_lines);
            free (cur_gbnd_ptr->in_samps);
            free (cur_gbnd_ptr->out_lines);
            free (cur_gbnd_ptr->out_samps);
            free (grid->bandlist);
            free (grid->gridbands);
            c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
            return E_FAIL;
        }

        /* Copy the grid lines and samples.
           --------------------------------*/
        for (ii=0; ii<cur_gbnd_ptr->num_in_lines; ii++)
            cur_gbnd_ptr->in_lines[ii] = las_grid->in_lines[ii] - 1;
        for (ii=0; ii<cur_gbnd_ptr->num_in_samps; ii++)
            cur_gbnd_ptr->in_samps[ii] = las_grid->in_samps[ii] - 1;

        kk = 0;
        for (ii=0; ii<cur_gbnd_ptr->num_out_lines; ii++)
            cur_gbnd_ptr->out_lines[ii] = (double)(las_grid->out_lines[ii] -
                    las_grid->out_lines[0]);
        for (ii=0; ii<cur_gbnd_ptr->num_out_samps; ii++)
            cur_gbnd_ptr->out_samps[ii] = (double)(las_grid->out_samps[ii] -
                    las_grid->out_samps[0]);

        /* Build grid coefficients.
           ------------------------*/
        kk = 0;
        for (ii=0; ii<cur_gbnd_ptr->num_out_lines-1; ii++)
            for (jj=0; jj<cur_gbnd_ptr->num_out_samps-1; jj++)
            {
                ki = kk + ii;
                kilg = ki + las_grid->ncols;

                /* Fill the matrix and RHS, QR factor, and solve.
                   ----------------------------------------------*/
                aa[0]  = aa[1]  = aa[2] = aa[3] = 1.0;
                aa[4]  = aa[7]  = cur_gbnd_ptr->out_samps[jj];
                aa[5]  = aa[6]  = cur_gbnd_ptr->out_samps[jj+1];
                aa[8]  = aa[9]  = cur_gbnd_ptr->out_lines[ii];
                aa[10] = aa[11] = cur_gbnd_ptr->out_lines[ii+1];
                aa[12] = aa[4]*aa[8];
                aa[13] = aa[5]*aa[9];
                aa[14] = aa[6]*aa[10];
                aa[15] = aa[7]*aa[11];
                c_qrdecomp (aa, four, four, work, zero);
                cur_gbnd_ptr->reverse_coeffs[kk].a[0] =
                    cur_gbnd_ptr->in_samps[ki];
                cur_gbnd_ptr->reverse_coeffs[kk].a[1] =
                    cur_gbnd_ptr->in_samps[ki+1];
                cur_gbnd_ptr->reverse_coeffs[kk].a[2] =
                    cur_gbnd_ptr->in_samps[kilg+1];
                cur_gbnd_ptr->reverse_coeffs[kk].a[3] =
                    cur_gbnd_ptr->in_samps[kilg];
                cur_gbnd_ptr->reverse_coeffs[kk].b[0] =
                    cur_gbnd_ptr->in_lines[ki];
                cur_gbnd_ptr->reverse_coeffs[kk].b[1] =
                    cur_gbnd_ptr->in_lines[ki+1];
                cur_gbnd_ptr->reverse_coeffs[kk].b[2] =
                    cur_gbnd_ptr->in_lines[kilg+1];
                cur_gbnd_ptr->reverse_coeffs[kk].b[3] =
                    cur_gbnd_ptr->in_lines[kilg];
                c_qrsolve(aa, four, four, work,
                        cur_gbnd_ptr->reverse_coeffs[kk].a, zero);
                c_qrsolve(aa, four, four, work,
                        cur_gbnd_ptr->reverse_coeffs[kk].b, zero);

                kk++;
            }
    }

    /* Set the grid structure pointers to point to the data in the
       first band.
       -----------------------------------------------------------*/
    for (iband=startband; iband<nbands; iband++)
    {
        /* Use a local pointer to simplify the code. */
        cur_band = bands[iband];

        grid->gridbands[cur_band].in_lines = cur_gbnd_ptr->in_lines;
        grid->gridbands[cur_band].in_samps = cur_gbnd_ptr->in_samps;
        grid->gridbands[cur_band].out_lines = cur_gbnd_ptr->out_lines;
        grid->gridbands[cur_band].out_samps = cur_gbnd_ptr->out_samps;
        grid->gridbands[cur_band].forward_coeffs = (COEFFICIENTS *)NULL;
        grid->gridbands[cur_band].reverse_coeffs = cur_gbnd_ptr->reverse_coeffs;
    }

    return (E_SUCC);
}


/*****************************************************************************
NAME                          FREE_GEOM_GRID_DATA

PURPOSE    Free the large members of the LAS grid structure.

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
L.D. Johnson          April 1999        original development

PROJECT       LAS

ALGORITHM
        Return

ALGORITHM REFERENCES
*****************************************************************************/

void free_geom_grid_data
(
    int *bands,              /* I: Array of bands to be read from the grid  */
    int nbands,              /* I: Number of bands in the bands array       */
    gxx_GEOM_GRID_TYPE *grid /* O: IAS grid structure                       */
)
{
    int iband;              /* Loop counter                                 */


    for (iband=0; iband<nbands; iband++)
    {
        free(grid->gridbands[bands[iband]].in_lines);
        free(grid->gridbands[bands[iband]].in_samps);
        free(grid->gridbands[bands[iband]].out_lines);
        free(grid->gridbands[bands[iband]].out_samps);
        free(grid->gridbands[bands[iband]].reverse_coeffs);
    }
}
