/*****************************************************************************
NAME                          GET_SCLAT

PURPOSE   Calculates the scene center latitude (in radians) of an input image.

PROGRAM HISTORY
PROGRAMMER              DATE            REASON
----------              ----            ------
K. Sheffield          Nov. 1993         original development

PROJECT       LAS

ALGORITHM
        Get the input image DDR
        Set master line & sample
        Get the projection info contained in the DDR.
        Calculate the latitude of the center pixel.
        Return

ALGORITHM REFERENCES    LAS 5.0 c_upddr by B. Ailts
*****************************************************************************/
#include <string.h>
#include "resample.h"


int get_sclat
(
    char* host_in, /* I: array of input host names       */
    double *sclat  /* O: returned scene center latitiude */
)
{
    struct DDR ddr;             /*  input ddr                                */
    long i;                     /*  loop counter                             */
    long geo = 0;               /*  GEO projection code (zero)               */
    long zero = 0;              /*  value of zero                            */
    double coef[6];             /*  coef for calculating projection units    */
    double projparms[15];       /*  project parameters                       */
    long message = FALSE;       /*  message for rtcoef                       */
    long icoords[8];            /*  input image coords                       */
    long iunits;                /*  input units                              */
    long zone = 0;              /*  grid zone                                */
    long prtprm[2];             /*  print parameter                          */
    double corners[8];
    double cx, cy;


    /* Set output units to radians.
       -------------------------------*/
    prtprm[0] = prtprm[1] = FALSE;

    for(i=0;i<15;i++)
        projparms[i] = 0.0;


    /* Retrieve the input ddr information.
       --------------------------------------*/
    if (c_getddr(host_in, &ddr) != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-ddrget", "Error retrieving ddr information.");
        return(E_FAIL);
    }

    /* Check validity of corner coordinates.
       -------------------------------------*/
    if (ddr.valid[DDPCV] == INVAL || ddr.valid[DDZCV] == INVAL ||
            ddr.valid[DDDCV] == INVAL || ddr.valid[DDPPV] == INVAL ||
            ddr.valid[DDPUV] == INVAL || ddr.valid[DDCCV] == INVAL )
    {
        c_lasmsg(FALSE, "resample-corner",
                "Invalid corner coordinates for input image.");
        return (E_FAIL);
    }

    /* Get projection units.  The string is converted to lower case
       in c_getddr.
       ------------------------------------------------------------*/
    iunits = c_string2unit(ddr.proj_units);
    if (iunits == E_FAIL)
    {
        c_lasmsg(FALSE, "resample-units", "%s are unsupported units",
                ddr.proj_units);
    } 

    /* Initialize projection transformation process.
       ---------------------------------------------*/
    if (c_transinit(&ddr.proj_code, &iunits, &ddr.zone_code,
            &ddr.datum_code, ddr.proj_coef, &geo, &zero, &zone,
            &zero, projparms, prtprm, "\0") != E_SUCC)
        return (E_FAIL);

    /* Transform corner coordinates to geometric projection.
       -----------------------------------------------------*/
    if (c_trans(&ddr.proj_code, &iunits, &geo, &zero, &ddr.upleft[1],
            &ddr.upleft[0], &corners[1], &corners[0]) != E_SUCC)
        return (E_FAIL);
    if (c_trans(&ddr.proj_code, &iunits, &geo, &zero, &ddr.loleft[1],
            &ddr.loleft[0], &corners[3], &corners[2]) != E_SUCC)
        return (E_FAIL);
    if (c_trans(&ddr.proj_code, &iunits, &geo, &zero, &ddr.upright[1],
            &ddr.upright[0], &corners[5], &corners[4]) != E_SUCC)
        return (E_FAIL);
    if (c_trans(&ddr.proj_code, &iunits, &geo, &zero, &ddr.loright[1],
            &ddr.loright[0], &corners[7], &corners[6]) != E_SUCC)
        return (E_FAIL);

    /* Set input image line,sample values for UL, LL, UR, LR.
       ------------------------------------------------------*/
    icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
    icoords[2] = icoords[6] = ddr.nl;
    icoords[5] = icoords[7] = ddr.ns;

    /* Compute image-to-projection transformation coefficients.
       --------------------------------------------------------*/
    if (c_rtcoef(corners, icoords, coef, &message) != E_SUCC)
        return (E_FAIL);

    /* Compute image center line/sample.
       ---------------------------------*/
    cy = (ddr.nl - 1) * .5 + 1;
    cx = (ddr.ns - 1) * .5 + 1;

    /* Compute center latitude.
       ------------------------*/
    *sclat = coef[0] * cy + coef[1] * cx + coef[2];

    return (E_SUCC);
}
