/******************************************************************************
NAME:           getpar

PURPOSE:        
This module reads the ODL file input parameters required by the resampler.

ROUTINES:
    getpar
    set_input_image_filename
    get_input_image_filename
    get_output_image_filename
    get_grid_filename
    do_terrain_correction
    get_DEM_filename
    get_output_data_type
    get_output_data_size
    do_output_window
    get_output_window
    set_output_window
    do_input_window
    get_input_window
    set_input_window
    set_scaled_output_window
    get_resampling_type
    get_output_range_minimum
    get_output_range_maximum
    get_cubic_convolution_alpha
    get_fill_pixel
    get_product_type
    get_las_grid
    get_sat_num
    get_inproj
    get_outproj
    get_opixsize
    get_ipixsize
    get_oulcoor
    get_iulcoor
    
DEVELOPMENT HISTORY:

Baselined from rev 1.22 of tmresample get_resample_parameters.c


NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

#include <string.h>
#include <stdlib.h>
#include "typlim.h"                  /* data type limit definitions        */
#include "resample.h"
#include "util.h"
#include "debug.h"                   /* ASSERT macro definition            */

/*
**
** VARIABLE DECLARATIONS
**
*/

static int input_image;               /* index to indicate which input image
                                         to use.                            */
static int input_window_flag;         /* flag to indicate the input data
                                         should be windowed.
                                         non-zero = window input data.      */
static int output_window_flag;        /* flag to indicate the output data
                                         should be windowed.
                                         non-zero = window output data.     */
static int satnum;                    /* Landsat satellite number (1-5)     */
static int terrain_flag;              /* flag to indicate terrain correction
                                         should be done.
                                         non-zero = do terrain correction   */
static WINDOW_TYPE input_window;      /* holds the window dimenstions for   */
                                      /* the input space window             */ 
static WINDOW_TYPE output_window[MAXBND]; /* holds the window dimensions
                                             for the selected output window */
static float alpha = -0.5;            /* cubic convolution alpha parameter  */
static float min_max_output[2];       /* minimum and maximum output values
                                         from the resampler                 */
static double fill_pixel;             /* value to use for fill pixels       */
static double ipixsize[2], opixsize[2]; /* Pixel size (line, samp) in relevant
                                           units (input, output spaces)     */
static double icorners[8];            /* Projection coordinates -- input
                                                          image corners     */
static double ocorners[8];            /* Projection coordinates -- output
                                                          image corners     */
static RESAMPLE_TYPE resampling_type; /* resampling type to perform         */
static OUTPUT_DATA_TYPE output_type;  /* output data type                   */
static PRODUCT_TYPE product_type;     /* product type for terrain corr.
                                         product.  Default = NON_STANDARD   */
static char dem_filename[1][CMLEN];   /* elevation model file name          */
static char grid_filename[CMLEN];     /* grid file name                     */
static char input_image_filename[MAXIMG][CMLEN];/* input image file name    */
static char odtype[5] = "BYTE";       /* character string storage for the
                                         output data type read from the ODL
                                         file.  Defaulted to byte.          */
static char output_image_filename[1][CMLEN]; /* output image file name      */
static char product_type_string[CMLEN]; /* character string storage for the
                                           selected product type            */
static char resample_opt[6];          /* resampling opt. Read from ODL file */
static char table_filename[CMLEN];    /* Resampling weight table file name  */
static char corners_filename[CMLEN];  /* Image corners table file name      */
static struct GEOGRID las_grid;       /* LAS grid structure                 */
static struct PROJ inproj;            /* Input projection parameters        */
static struct PROJ outproj;           /* Output projection parameters       */


/*****************************************************************************
NAME:           getpar

PURPOSE:        
This function ingests user input parameters from TAE..  All the necessary
error checking is done on the parameters read.

ALGORITHM REFERENCES:
none

*****************************************************************************/

int getpar
(
    struct PARBLK *vblk,       /* I: TAE parameter block                    */
    char outtae[][CMLEN],      /* O: Output image filename (TAE)            */
    int *ncpu,                 /* O: Number of CPUs (threads) to use        */
    int *image_buf_size,       /* O: Size (in MB) of desired image buffer   */
    int *lines_to_buffer,      /* O: Number of input image lines to buffer  */
    int *buffer_full_output_image, /* O: flag to buffer entire output image */
    int *gridflg,              /* O: Grid flag: 0 = no grid, use proj. params.
                                               1 = use grid                 */
    long bands[][MAXBND+1],    /* O: band array                             */
    int *nbands,               /* O: number of bands                        */
    long *numimg,              /* O: Number of images specified             */
    long img_band[],           /* O: Num. of bands specified for each image */
    long *wind_mode,           /* O: Window mode                            */
    long *eltime,              /* O: TAE global--message timer              */
    char *history_flag,        /* O: TAE global--history flag               */
    char *minmax_flag,         /* O: TAE global--minmax flag                */
    char inhost[][CMLEN]       /* O: Host name of the input image files     */
)
{
    struct VARIABLE *var;        /* Pointer to a parameter in vblk          */
    int maxcpu;                  /* Number of cpus available on system      */

    long access = IREAD;         /* Getnam file access mode                 */
    long demband[1][MAXBND+1];   /* Array of band numbers to process        */
    long dem_band[1];            /* Number of bands specified for the DEM
                                    image                                   */
    long demflg[1][2];           /* Qflag buffer returned by getin          */
    long frame_mode;             /* Framing mode                            */
    long giqflg[1][2];           /* Qflag buffer returned by getin          */
    long giwind[MAXIMG][4];      /* Window buffer returned by getin         */
    long gridchkflg;             /* Grid check flag                         */
    long idtype;                 /* input image data type                   */
    long ii;                     /* Loop counter                            */
    long in_image_corners[8];    /* Image coordinates of image corners      */
    long imglim = MAXIMG;        /* The maximum number of input images      */
    long in_wind[MAXIMG][4];     /* Input image window specification        */
    long length;                 /* String length                           */
    long lscoors[2];             /* Line,sample coordinates of ul coord     */
    long ndembands;              /* Number of bands in DEM image            */
    long nl = 0, ns = 0;         /* Number of lines, samples in output      */
    long one = 1;                /* Constant of one                         */
    long out_def;                /* Default flags for projection parameters */
    long projflag;               /* Projection flag:
                                    0 = projection not needed,
                                    1 = projection info required  */
    long prtprm[2]={FALSE, FALSE}; /* print dest. flags for c_transinit     */
    long same;                   /* Same window flag -- getin               */
    long status;                 /* Function return status                  */
    long tnmrimg;                /* Number of terrain input images          */
    long subcmd_len = 9;         /* Maximum length of subcmd string         */
    long unit_code;              /* Unit code of corner coordinates entered */
    long window[4];              /* window when windowing input or output
                                    data.  Format is start line, start
                                    sample, number of lines, number of
                                    samples.                                */
    long zero = 0;               /* Constant of zero                        */

    double intlim[2];            /* Interpolation limit values              */
    double olr_corner[2];        /* Requested lower right corner coordinate
                                    (output space)       */
    double oul_corner[2];        /* Requested upper left corner coordinate
                                    (output space)       */
    double pxmax = 0, pxmin = 0; /* Max and min X projection coordinate     */
    double pymax = 0, pymin = 0; /* Max and min Y projection coordinate     */
    double real_wind[4];         /* Window spec--in double                  */
    double temp;                 /* temporary storage                       */
    double wind_proj[8];         /* Projection coordinates of image window  */

    char *cthreads;              /* pointer to the IL_COMPUTE_THREADS
                                    environment variable                    */
    char flag[10];               /* Input/output flag                       */
    char intae[MAXIMG][CMLEN];   /* Input image TAE name                    */
    char subcmd[10];             /* Subcommand: GRID or NOGRID              */
    char outkey[CMLEN];          /* Output projection key                   */
    char proj_filename[CMLEN];   /* Projection file name                    */
    char units[12];              /* Units of corner coordinates entered     */


    /* Get input image name.  Ignore windowing entered.
       ------------------------------------------------*/
    if (c_getin(vblk, "IN", &zero, intae, input_image_filename, numimg,
                giwind, bands, img_band, (long *)nbands, &imglim, &same,
                giqflg) != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-getin",
                "Error parsing input image specification for IN.");
        return E_FAIL;
    }

    /* Set the index for the input images to zero to start
       ---------------------------------------------------*/

    input_image = 0;  

    /* Copy the names of the input files into inhost 
       ---------------------------------------------*/
    for(ii = 0 ; ii < *numimg ; ii++)
    {
        strcpy(inhost[ii], input_image_filename[ii]);
    }


    /* Check to see if the input images all have the same window size.
       ---------------------------------------------------------------*/
    if( same == 0)
    {
        c_lasmsg(FALSE, "resample-size", 
                "Input images must be the same size");
        return E_FAIL;
    }

    /* Retrieve TAE globals.
       ---------------------*/
    var = p_find(vblk,"$MSGTIM");
    *eltime = IVAL(*var,0);
    var = p_find(vblk,"$HISTRY");
    strcpy(history_flag, SVAL(*var,0));
    var = p_find(vblk,"$MINMAX");
    strcpy(minmax_flag, SVAL(*var,0));

    /* Load the band information.
       --------------------------*/
    if (*nbands > MAXBND)
    {
        c_lasmsg(FALSE, "resample-nbnd",
                "Number of bands exceeds limit of %d.", MAXBND);
        return E_FAIL;
    }

    /* Get output image name.
       ----------------------*/
    if (c_getout(vblk, "OUT", outtae, output_image_filename) != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-getout",
                "Error parsing output image specification for OUT.");
        return E_FAIL;
    }

    /* Get input terrain image name.  Ignore windowing entered.
       --------------------------------------------------------*/
    if (c_getin(vblk, "DEM", &zero, intae, dem_filename, &tnmrimg, giwind,
                demband, dem_band, &ndembands, &imglim, &same, demflg)
            != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-getin",
                "Error parsing input image specification for DEM.");
        return E_FAIL;
    }

    if (tnmrimg == 0)
        terrain_flag = 0;
    else if (tnmrimg > 1)
    {
        c_lasmsg(FALSE, "resample-tnmrimg",
                "More than one terrain input image specified.");
        return E_FAIL;
    }
    else
    {
        terrain_flag = 1;

        /* Get the product type.
           ---------------------*/
        var = p_find(vblk,"IMGTYPE");
        strcpy(product_type_string, SVAL(*var,0));

        if (strcmp(product_type_string,"TM-P") == 0)
            product_type = TM_P;
        else if (strcmp(product_type_string,"MSS-P") == 0)
            product_type = MSS_P;
        else if (strcmp(product_type_string,"MSS-X") == 0)
            product_type =  MSS_X;
        else if (strcmp(product_type_string,"NONSTAND") == 0)
        {
            product_type = NON_STANDARD;

            /* Get input labeled table file.
               -----------------------------*/
            var = p_find(vblk, "INLT");
            strcpy(corners_filename, SVAL(*var, 0));
        }
        else
        {
            c_lasmsg(FALSE, "resample-imagetype", "Unknown image type.");
            return E_FAIL;
        }

        /* Get the satellite number.
           -------------------------*/
        var = p_find(vblk, "SATNUM");
        satnum = IVAL(*var, 0);
    }

    /* Get the output data type.
       -------------------------*/
    var = p_find(vblk, "ODTYPE");
    strcpy(odtype, SVAL(*var,0));
    if (strcmp(odtype, "SAME") == 0)
    {
        /* Get the data type of the input images.
           --------------------------------------*/
        if (c_gettyp(input_image_filename, numimg, &idtype) != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-dtype", 
                    "The data types of the images are not the same,"
                    "so the data type of %s will be used.",
                    input_image_filename[0]);
            c_gettyp(input_image_filename,&one,&idtype);
        }

        switch (idtype)
        {
            case EBYTE:
                strcpy(odtype, "BYTE");
                break;
            case EWORD:
                strcpy(odtype, "I*2");
                break;
            case ELONG:
                strcpy(odtype, "I*4");
                break;
            case EREAL:
                strcpy(odtype, "R*4");
                break;
        }
    }
    if (strcmp(odtype, "BYTE") == 0)
    {
        output_type = BYTE_OUTPUT;
        min_max_output[0] = MINUTINY;
        min_max_output[1] = MAXUTINY;
    }
    else if (strcmp(odtype, "I*2") == 0)
    {
        output_type = INT16_OUTPUT; 
        min_max_output[0] = MINWORD;
        min_max_output[1] = MAXWORD;
    }
    else if (strcmp(odtype, "I*4") == 0)
    {
        output_type = INT32_OUTPUT;
        min_max_output[0] = MINLONG;
        min_max_output[1] = MAXLONG;
    }
    else if (strcmp(odtype, "R*4") == 0)
    {
        output_type = FLOAT_OUTPUT;
        min_max_output[0] = MINREAL;
        min_max_output[1] = MAXREAL;
    }    
    else
    {
        c_lasmsg(FALSE, "resample-dtype", "Unknown output data type.",
                input_image_filename);
        return E_FAIL;
    }

    /* Get the fill value.  Check against data type range.
       ---------------------------------------------------*/
    var = p_find(vblk, "BACKGRND");
    fill_pixel = RVAL(*var,0);
    if (fill_pixel < min_max_output[0] || fill_pixel > min_max_output[1])
    {
        c_lasmsg(FALSE, "resample-background",
                "BACKGRND out of range.  Set to zero.");
        fill_pixel = 0;
    }

    /* Get the interpolation limits (min/max values).
       Check against data type range.
       ----------------------------------------------*/
    var = p_find(vblk,"INTLIM");
    if ((*var).v_count != 0)
    {
        intlim[0] = RVAL(*var,0);
        intlim[1] = RVAL(*var,1);

        if (intlim[0] > intlim[1])            /* If min > max, swap values. */
        {
            temp = intlim[0];
            intlim[0] = intlim[1];
            intlim[1] = temp;
        }

        if (intlim[0] < min_max_output[0] || intlim[1] > min_max_output[1])
            c_lasmsg(FALSE, "resample-limits", "INTLIM out of range."
                    "  Limits adjusted to range of data type.");
        if (intlim[0] > min_max_output[0] && intlim[0] < min_max_output[1])
            min_max_output[0] = intlim[0];
        if (intlim[1] > min_max_output[0] && intlim[1] < min_max_output[1])
            min_max_output[1] = intlim[1];
    }

    /* Get the resampling method.
       --------------------------*/
    var = p_find(vblk, "RESAMP");
    strcpy(resample_opt, SVAL(*var,0));
    if (strcmp(resample_opt, "CC") == 0)
    {
        resampling_type = CC_RESAMPLING;
        var = p_find(vblk, "PCCALPHA");
        alpha = RVAL(*var,0);
    }
    else if (strcmp(resample_opt, "NN") == 0)
        resampling_type = NN_RESAMPLING;
    else if (strcmp(resample_opt, "BI") == 0)
        resampling_type = BI_RESAMPLING;
    else if (strcmp(resample_opt, "TABLE") == 0)
    {
        resampling_type = TABLE_RESAMPLING;
        if (c_getnam(vblk, "INRWT", &zero, table_filename, "rwt", &access)
                != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-open", "Error opening table file %s.",
                    table_filename);
            return E_FAIL;
        }
    }
    else
    {
        c_lasmsg(FALSE, "resample-option", "Unknown resample option.");
        return E_FAIL;
    }

    /* Get the number of threads.
       --------------------------*/
    maxcpu = GET_NUM_PROCESSORS();   /* get num. of processors */
    var = p_find(vblk, "NTHREADS");
    if(var->v_count == 0)
    {
        if((cthreads = getenv("IL_COMPUTE_THREADS")) == NULL)
            *ncpu = (maxcpu > 1) ? maxcpu/2 : maxcpu;
        else
            sscanf(cthreads,"%d", ncpu);
    }
    else
        *ncpu = IVAL(*var,0);

    if (*ncpu > maxcpu)
    {
        c_lasmsg(FALSE, "resample-ncpu",
                "Number of threads requested greater than"
                " number of available CPUs.  Reducing to %d.", maxcpu);
        *ncpu = maxcpu;
    }
    c_lasmsg(FALSE, "resample-info", "Using %d CPUs", *ncpu);

    /* Get the desired buffer size.
       ----------------------------*/
    var = p_find(vblk, "CACHESZ");
    *image_buf_size = IVAL(*var, 0);

    /* Get the number of input image lines to buffer.
       ----------------------------------------------*/
    var = p_find(vblk, "BUFLINES");
    *lines_to_buffer = IVAL(*var, 0);

    /* Get the flag for full output image buffering.
       ---------------------------------------------*/
    var = p_find(vblk, "FULLBUF");
    if (strcmp(SVAL(*var,0), "YES") == 0)
        *buffer_full_output_image = 1;
    else
        *buffer_full_output_image = 0;

    /*----------------------------------------
      ------------------------------------------
      Get the subcommand-specific parameters.
      ------------------------------------------
      ------------------------------------------*/

    if (c_getstr(vblk, "_SUBCMD", &one, &subcmd_len, subcmd) != 1)
    {
        c_lasmsg(FALSE, "resample-getstr", "Unable to retrieve _SUBCMD.");
        return E_FAIL;
    }

    /* Get the input projection information from the input DDR.
       --------------------------------------------------------*/
    strcpy(flag,"IN");
    if (strcmp(subcmd, "GRID") == 0)
        projflag = 0;       /* no projection information needed */
    else
        projflag = 1;       /* projection information required */
    if (use_ddr_info(vblk, &inproj, projflag, icorners, in_image_corners,
                in_wind, flag, &ipixsize[0], &ipixsize[1]) != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-ddrget",
                "Error retrieving DDR information.");
        return E_FAIL;
    }

    if (strcmp(subcmd, "GRID") == 0)
    {
        /*----------------------------
          ----------------------------
          Grid case.
          ----------------------------
          ----------------------------*/


        *gridflg = 1;

        /* Get the Geometric Mapping Grid file.
           ------------------------------------*/
        if (c_getnam(vblk, "INGRID", &zero, grid_filename, "grid", &access)
                != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-open", "Error opening grid file %s.",
                    grid_filename);
            return E_FAIL;
        }

        /* Get the grid check flag.
           ------------------------*/
        var = p_find(vblk, "GRIDFLG");
        if (strcmp(SVAL(*var,0),"YES") == 0)
            gridchkflg = 1;
        else
            gridchkflg = 0;

        /* Get the window option parameter (IN or OUT).
           --------------------------------------------*/
        var = p_find(vblk, "WINDOPT");
        if (strcmp(SVAL(*var,0), "OUT") == 0)
            *wind_mode = OUTWIND;
        else
            *wind_mode = INWIND;

        /* Get the window spec.
           -----------------------*/
        var = p_find(vblk,"WINDOW");
        if ((*var).v_count > 0)
        {
            window[0] = IVAL(*var, SL) - 1;
            window[1] = IVAL(*var, SS) - 1;
            window[2] = IVAL(*var, NL);
            window[3] = IVAL(*var, NS);
            output_window_flag = 1;
        }
        else
        {
            *wind_mode = NOWIND;
            for (ii=0; ii<4; window[ii++] = 0);
            output_window_flag = 0;
        }

        /* Set up input image windows and output grid windows for the
           windowing method chosen.

           If no windowing was specified (NOWIND), the entire input image is
           used.  The output grid window is driven by the mapping grid
           output lines (map->lines) and output samples (map->samples) as
           entered on the first line of the grid.

           If an input window (INWIND) was given the input image is windowed
           but the entire mapping grid is used.  The size of the output
           image is still determined by the mapping grid in the same manner
           as described above when windowing is not used.  Input windowing
           does, however, allow the user to "adjust" which pixel in the
           input image that RESAMPLE "sees" as (1,1).

           If an output window was specified, the entire input image is
           used.  The image is mapped based on the output window.  In this
           case, the size of the output image is determined by the output
           window.
           -----------------------------------------------------------------*/

        /* Read the geometric mapping grid.
           --------------------------------*/
        if (c_getgrd(grid_filename, &las_grid) != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-grid",
                    "Error reading geometric mapping grid file.");
            return E_FAIL;
        }

        /* Check the mapping grid for correctness if gridchkflg is set.
           ------------------------------------------------------------*/
        if (gridchkflg)
        {
            status = check_grid(&las_grid);
            if (status != E_SUCC)
            {
                if (status == GMGLIN)
                    c_lasmsg(FALSE, "resample-badgridl",
                            "Grid lines wrapped or not strictly increasing.");
                else if (status == GMGSAM)
                    c_lasmsg(FALSE, "resample-badgrids",
                            "Grid samples wrapped or not strictly increasing.");
                return E_FAIL;
            }
        }

        /* Set windowing structures.
           -------------------------*/
        if (output_window_flag)
        {
            /* Make sure values were provided for the window size.
               ---------------------------------------------------*/ 
            if ((window[0] < 0) || (window[1] < 0) || 
                    (window[2] <= 0) || (window[3] <= 0))
            {
                c_lasmsg(FALSE, "resample-wind",
                        "Specified window is not valid.");
                return E_FAIL;
            }

            /* Parse the type of window.
               -------------------------*/
            if (*wind_mode == OUTWIND)
            {
                /* Be sure window entered is within the mapping grid.
                   --------------------------------------------------*/
                if ((window[2] > las_grid.lines) ||
                        (window[3] > las_grid.samples))
                {
                    c_lasmsg(FALSE, "resample-outwind",
                            "Output window falls outside mapping grid.");
                    return E_FAIL;
                }

                /* Initialize the output window data structure relative to
                   the first band specified in the input band list.
                   -------------------------------------------------------*/
                set_output_window(0, window[0], window[1], window[2],
                        window[3]);
                set_input_window(0, 0, in_image_corners[6],
                        in_image_corners[7]);
            }
            else /* wind_mode == INWIND */
            {
                /* Be sure window entered is within the input image.
                   -------------------------------------------------*/
                if ((window[2] > in_image_corners[6]) ||
                        (window[3] > in_image_corners[7]))
                {
                    c_lasmsg(FALSE, "resample-wind",
                            "Specified window is not valid.");
                    return E_FAIL;
                }

                /* Input window: initialize the input window data structure.
                   ---------------------------------------------------------*/
                set_input_window(window[0], window[1], window[2], window[3]);

                /* Set input window flag, and clear output window flag.
                   ----------------------------------------------------*/
                input_window_flag = 1;
                output_window_flag = 0;
                set_output_window(0, 0, 0, las_grid.lines, las_grid.samples);
            }
        }
        else
        {
            set_input_window(0, 0, in_image_corners[6], in_image_corners[7]);
            set_output_window(0, 0, 0, las_grid.lines, las_grid.samples);
        }
    }
    else
    {
        /*----------------------------
          ----------------------------
          No-grid case.
          ----------------------------
          ----------------------------*/

        *gridflg = 0;
        *wind_mode = OUTWIND;

        /* Check to see if there is more than one input image.
           ---------------------------------------------------*/
        if( *numimg > 1)
        {
            c_lasmsg(FALSE, "resample-nmrimg", "Only one input image "
                    "allowed for the -NOGRID subcommand.");
            return(E_FAIL);
        }


        /* Get the name of the file containing projection information.
           -----------------------------------------------------------*/
        if (c_getnam(vblk, "INPROJ", &zero, proj_filename, "proj", &access)
                != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-open",
                    "Error opening projection file %s.", proj_filename);
            return E_FAIL;
        }

        /* Get the units that the coordinates are in.
           ------------------------------------------*/
        var = p_find(vblk, "COORUNIT");
        strcpy(units, SVAL(*var, 0));
        length = (long)strlen(units);
        c_low2up(units, &length);
        unit_code = 0; /* Default for "PRO" */
        if(strcmp(units, "RAD") == 0) unit_code = 0;
        if(strcmp(units, "SEC") == 0) unit_code = 3;
        if(strcmp(units, "DEG") == 0) unit_code = 4;
        if(strcmp(units, "DMS") == 0) unit_code = 5;

        /* Get the output projection coordinates.
           --------------------------------------*/
        var = p_find(vblk, "COORS");
        /* Check number of input values. */
        if (var->v_count != 0 && var->v_count != 2 && var->v_count != 4)
        {
            c_lasmsg(FALSE, "resample-coor",
                    "COORS must contain either 0, 2, or 4 values.");
            return E_FAIL;
        }
        if (var->v_count == 0)
            /* Framing info is obtained from the input image
               (and window, if applied)
               ----------------------------------------------*/
        {
            for (ii = 4; ii--; real_wind[ii] = in_wind[0][ii]);
            if (c_rotcor(icorners, in_image_corners, real_wind, wind_proj)
                    != E_SUCC)
                return(E_FAIL);
            pxmin = GMIN(wind_proj[1], wind_proj[3]);
            pxmin = GMIN(pxmin, wind_proj[5]);
            pxmin = GMIN(pxmin, wind_proj[7]);
            pxmax = GMAX(wind_proj[1], wind_proj[3]);
            pxmax = GMAX(pxmax, wind_proj[5]);
            pxmax = GMAX(pxmax, wind_proj[7]);
            pymin = GMIN(wind_proj[0], wind_proj[2]);
            pymin = GMIN(pymin, wind_proj[4]);
            pymin = GMIN(pymin, wind_proj[6]);
            pymax = GMAX(wind_proj[0], wind_proj[2]);
            pymax = GMAX(pymax, wind_proj[4]);
            pymax = GMAX(pymax, wind_proj[6]);

            frame_mode = INPUT_XY;
        }
        else
            /* Framing is determined from user input
               -------------------------------------*/
        {
            sscanf(SVAL(*var, 0), "%lf", &(oul_corner[0]));
            sscanf(SVAL(*var, 1), "%lf", &(oul_corner[1]));
            if (var->v_count == 2)
                frame_mode = OUTPUT_LS;
            else if (strcmp(units, "PRO") == 0)
                frame_mode = OUTPUT_XY;
            else
                frame_mode = INPUT;
            if ((frame_mode == INPUT) || (frame_mode == OUTPUT_XY))
            {
                sscanf(SVAL(*var, 2), "%lf", &(olr_corner[0]));
                sscanf(SVAL(*var, 3), "%lf", &(olr_corner[1]));

                /* If the UL and LR corner coordinates define a window
                   crossing the 180 degree longitude, change the coordinates
                   to work with c_mapedg in out_frame.
                   ---------------------------------------------------------*/
                if (((olr_corner[0] - oul_corner[0])*
                            (olr_corner[1] - oul_corner[1])) > 0.)
                {
                    switch (unit_code)
                    {
                        case 4:
                            olr_corner[1] += 360.;
                    }
                }
            }
        }

        /* Starting line/sample output coordinates
           ---------------------------------------*/
        var = p_find(vblk, "LSCOORS");
        lscoors[0] = IVAL(*var,0);
        lscoors[1] = IVAL(*var,1);

        if (frame_mode == OUTPUT_LS)
        {
            /* Retrieve the number of lines and samples in the output area
               -----------------------------------------------------------*/
            var = p_find(vblk, "NL");
            if (var->v_count != 1)
            {
                c_lasmsg(FALSE, "resample-nls",
                        "Number of output lines, samples not specified.");
                return E_FAIL;       
            }    
            nl = IVAL(*var,0);
            var = p_find(vblk, "NS");
            if (var->v_count != 1)
            {
                c_lasmsg(FALSE, "resample-nls",
                        "Number of output lines, samples not specified.");
                return E_FAIL;       
            }    
            ns = IVAL(*var,0);
        }


        /* Get the output projection information.
           --------------------------------------*/
        var = p_find(vblk, "OPROJKEY");
        out_def = TRUE;
        if (var->v_count > 0)
        {
            out_def = FALSE;
            strcpy(outkey, SVAL(*var, 0));
        }
        if (get_proj_prm(proj_filename, outkey, out_def, &outproj) != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-proj",
                    "Error reading output projection information.");
            return E_FAIL;
        }

        /* Get the pixel size of the output.
           ---------------------------------*/
        var = p_find(vblk, "PIXSIZ");
        if (var->v_count < 2)
        {
            c_lasmsg(FALSE, "resample-pixsiz",
                    "Output pixel size must be specified.");
            return E_FAIL;
        }
        opixsize[0] = RVAL(*var, 0);
        opixsize[1] = RVAL(*var, 1);

        /* If the frame mode is INPUT_XY, we need to initialize the
           Projection Transformation Package.
           --------------------------------------------------------*/
        if (frame_mode == INPUT_XY)
        {
            if (c_transinit(&inproj.proj_code, &inproj.units,
                        &inproj.zone_code, &inproj.datum_code,
                        inproj.proj_coef, &outproj.proj_code,
                        &outproj.units, &outproj.zone_code,
                        &outproj.datum_code, outproj.proj_coef,
                        prtprm, NULL) != E_SUCC)
                return(E_FAIL);
        }

        /* Compute transformation mapping from output image
           coordinates to output space projection coordinates.
           ---------------------------------------------------*/
        if (out_frame(opixsize, window, ocorners, &outproj, &inproj,
                    frame_mode, oul_corner, olr_corner, lscoors, nl, ns,
                    unit_code, pxmax, pxmin, pymax, pymin) != E_SUCC)
        {
            c_lasmsg(FALSE, "resample-frame",
                    "Error computing the output frame coordinates.");
            return E_FAIL;
        }

        /* Decrement start line and start sample since resampler uses
           zero-based counting for lines and samples.
           ----------------------------------------------------------*/
        window[0]--;
        window[1]--;

        /* Make sure values were provided for the window size.
           ---------------------------------------------------*/
        if ((window[0] < 0) || (window[1] < 0) ||
                (window[2] <= 0) || (window[3] <= 0))
        {
            c_lasmsg(FALSE, "resample-wind",
                    "Specified window is not valid.");
            return E_FAIL;
        }

        /* Initialize the output window data structure relative to
           the first band specified in the input band list.
           -------------------------------------------------------*/
        set_output_window(0, window[0], window[1], window[2], window[3]);
        set_input_window(0, 0, in_image_corners[6], in_image_corners[7]);

    }    /* end of subcommand-specific section */

    return E_SUCC;
}


/*****************************************************************************
NAME:           various accessory functions

PURPOSE:        
These functions provide access to the parameters read. 

RETURN VALUE:
Varies for the function.

NOTES:
The accessor function method of providing parameters was selected since
passing all the parameters up and down a calling sequence is a 
maintenance nightmare.

ALGORITHM REFERENCES:
none

*****************************************************************************/

void set_input_image_filename
(
    int imageno               /* I: Number of input image to get with a call
                                 to get_input_image_filename              */
)
{
    input_image = imageno;    

}

const char *get_input_image_filename(void)
{
    return (input_image_filename[input_image]);
}


const char *get_output_image_filename(void)
{
    return (output_image_filename[0]);
}


const char *get_grid_filename(void)
{
    return (grid_filename);
}


const char *get_table_filename(void)
{
    return (table_filename);
}


const char *get_corners_filename(void)
{
    return (corners_filename);
}


int do_terrain_correction(void)
{
    return (terrain_flag);
}


const char *get_DEM_filename(void)
{
    return (dem_filename[0]);
}


OUTPUT_DATA_TYPE get_output_data_type(void)
{
    return (output_type);
}


int get_output_data_size(void)
{
    if (output_type == BYTE_OUTPUT)
        return(EBYTESIZE);
    else if (output_type == INT16_OUTPUT)
        return(EWORDSIZE);
    else if (output_type == INT32_OUTPUT)
        return(ELONGSIZE);
    else 
    {
        ASSERT(output_type == FLOAT_OUTPUT);
        return(EREALSIZE);
    }
}


int do_output_window(void)
{
    return (output_window_flag);
}


const WINDOW_TYPE *get_output_window
(
    int iband   /* I: indexed band number for the desired the output window */
)
{ 
    /* Verify input band number is legal.
       ----------------------------------*/
    ASSERT((iband >= 0) && (iband < MAXBND));

    return (&output_window[iband]);
}


int do_input_window(void)
{
    return (input_window_flag);
}


const WINDOW_TYPE *get_input_window(void)
{ 
    return (&input_window);
}


void set_output_window
(
    int iband,     /* I: indexed band number for the desired output window  */
    long start_line,   /* I: starting line for the output window        */
    long start_sample, /* I: starting sample for the output window      */
    long num_lines,    /* I: number of lines in the output window       */
    long num_samples   /* I: number of samples in the output window     */
)
{
    /* Verify input band number is legal.
       ----------------------------------*/
    ASSERT((iband >= 0) && (iband < MAXBND));

    output_window[iband].start_line = start_line;
    output_window[iband].start_sample = start_sample;
    output_window[iband].num_lines = num_lines;
    output_window[iband].num_samples = num_samples;
    output_window[iband].end_line = start_line + num_lines;
    output_window[iband].end_sample = start_sample + num_samples;
}


void set_input_window
(
    long start_line,    /* I: starting line for the window             */
    long start_sample,  /* I: starting sample for the window           */
    long num_lines,     /* I: number of lines in the window            */
    long num_samples    /* I: number of samples in the window          */
)
{
    input_window.start_line = start_line;
    input_window.start_sample = start_sample;
    input_window.num_lines = num_lines;
    input_window.num_samples = num_samples;
    input_window.end_line = start_line + num_lines;
    input_window.end_sample = start_sample + num_samples;
}


void set_scaled_output_window
(
    int iband,        /* I: indexed band num. for the desired output window */
    float line_scaling,  /* I: scaling factor for the window line size */
    float sample_scaling /* I: scaling factor for the window samp size */
)
{
    int refband = 0;     /* The reference band for scaling is the first
                            band.                                      */


    /* Verify input band number is legal.
       ----------------------------------*/
    ASSERT((iband >= 0) && (iband < MAXBND));

    output_window[iband].start_line = output_window[refband].start_line * 
        line_scaling;
    output_window[iband].start_sample = output_window[refband].start_sample * 
        sample_scaling;
    output_window[iband].num_lines = output_window[refband].num_lines * 
        line_scaling;
    output_window[iband].num_samples = output_window[refband].num_samples * 
        sample_scaling;
    output_window[iband].end_line = output_window[iband].start_line + 
        output_window[iband].num_lines;
    output_window[iband].end_sample = output_window[iband].start_sample + 
        output_window[iband].num_samples;
}


RESAMPLE_TYPE get_resampling_type(void)
{
    return (resampling_type);
}


float get_output_range_minimum(void)
{
    return (min_max_output[0]);
}


float get_output_range_maximum(void)
{
    return (min_max_output[1]);
}


float get_cubic_convolution_alpha(void)
{
    return (alpha);
}


double get_fill_pixel(void)
{
    return (fill_pixel);
}


PRODUCT_TYPE get_product_type(void)
{
    return (product_type);
}


const struct GEOGRID *get_las_grid(void)
{
    return (&las_grid);
}


int get_sat_num(void)
{
    return (satnum);
}


const struct PROJ *get_inproj(void)
{
    return (&inproj);
}


const struct PROJ *get_outproj(void)
{
    return (&outproj);
}


const double *get_opixsize(void)
{
    return (opixsize);
}


const double *get_ipixsize(void)
{
    return (ipixsize);
}


const double *get_oulcoor(void)
{
    return (ocorners);
}


const double *get_iulcoor(void)
{
    return (icorners);
}
