/****************************************************************************
NAME: ng_upddr

PURPOSE:  This function updates the output image DDR from projection
          information.  It is the "no-grid" version of g_upddr.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  1.0          09/99    L. Johnson        CSB      New development (RESAMPLE)

ALGORITHM DESCRIPTION:

*****************************************************************************/
#include <sys/types.h>
#include <malloc.h>
#include "resample.h"
#include "cproj.h"


int ng_upddr
(
    char inhost[][CMLEN],   /* I: array of input host names                 */
    char *outhost,          /* I: output host name                          */
    long bands[][MAXBND+1], /* I: band numbers for each input image         */
    long nbands             /* I: number of bands in output image           */
)
{
    struct DDR oddr;            /* output DDR                                */
    struct BDDR *ibdr;          /* input band dependent ddr records          */
    struct BDDR *obdr;          /* output band dependent ddr record          */
    const struct PROJ *outproj; /* output projection information             */

    const double *opixsize;     /* output pixel size                         */
    const double *oulcoor;      /* ouput window upper left corner coordinate */

    int ii;                     /* loop counter                              */
    int bndcnt;                 /* band counter                              */
    int imgcnt;                 /* image counter                             */


    /* Allocate space for the band DDRs.
       ---------------------------------*/
    if ((obdr = (struct BDDR *)calloc(nbands, sizeof(struct BDDR))) == NULL)
    {
        c_lasmsg(FALSE,"resample-alloc","Memory allocation error");
        return(E_FAIL);
    }
    if ((ibdr = (struct BDDR *)malloc(sizeof(struct BDDR))) == NULL)
    {
        free(obdr);
        c_lasmsg(FALSE,"resample-alloc","Memory allocation error");
        return(E_FAIL);
    }

    /* Retrieve the output projection information, pixel size, and
       UL coordinate.
       -----------------------------------------------------------*/
    outproj = get_outproj();
    opixsize = get_opixsize();
    oulcoor = get_oulcoor();

    /* Retrieve the output DDR.
       ------------------------*/
    if (c_getddr(outhost, &oddr) != E_SUCC)
    {
        free(ibdr);
        free(obdr);
        c_lasmsg(FALSE,"resample-ddrget","Error retrieving ddr information.");
        return(E_FAIL);
    }

    /* Set the output band-independent DDR information.
       ------------------------------------------------*/
    oddr.master_line = oddr.master_sample = 1;
    oddr.line_inc = oddr.sample_inc = 1.0;
    switch (outproj->units)
    {
        case RADIAN:
            strcpy(oddr.proj_units, "radians");
            break;
        case FEET:
            strcpy(oddr.proj_units, "feet");
            break;
        case METER:
            strcpy(oddr.proj_units, "meters");
            break;
        case SECOND:
            strcpy(oddr.proj_units, "seconds");
            break;
        case DEGREE:
            strcpy(oddr.proj_units, "degrees");
            break;
        case DMS:
            strcpy(oddr.proj_units, "dms");
            break;
        default:
            free(ibdr);
            free(obdr);
            c_lasmsg(FALSE, "resample-units", "Unsupported units.");
            return(E_FAIL);
    }
    oddr.proj_code = outproj->proj_code;
    oddr.zone_code = outproj->zone_code;
    oddr.datum_code = outproj->datum_code;
    for (ii = 0; ii<15; ii++)
        oddr.proj_coef[ii] = outproj->proj_coef[ii];
    oddr.upleft[0] = oulcoor[0];
    oddr.upleft[1] = oulcoor[1];
    oddr.pdist_y = opixsize[0];
    oddr.pdist_x = opixsize[1];
    oddr.loleft[0] = oddr.upleft[0] - (oddr.nl - 1)*oddr.pdist_y;
    oddr.loleft[1] = oddr.upleft[1];
    oddr.upright[0] = oddr.upleft[0];
    oddr.upright[1] = oddr.upleft[1] + (oddr.ns - 1)*oddr.pdist_x;
    oddr.loright[0] = oddr.loleft[0];
    oddr.loright[1] = oddr.upright[1];

    /* Set the VALID flags.
       --------------------*/
    for (ii=0; ii<8; ii++)
        oddr.valid[ii] = VALID;

    /* Set the output band-dependent DDR information.
       ----------------------------------------------*/
    for(bndcnt = 0, imgcnt = 0, ii = 0; ii < nbands; ii++, bndcnt++)
    {
        if (bands[imgcnt][bndcnt] == 0)
        {
            imgcnt++;
            bndcnt = 0;
        }

        /* Retreive the band DDR information for each band of each image.
           --------------------------------------------------------------*/
        if (c_getbdr(inhost[imgcnt],ibdr,&bands[imgcnt][bndcnt]) != E_SUCC)
        {
            free(ibdr);
            free(obdr);
            c_lasmsg(FALSE, "resample-ddrget",
                    "Error retrieving ddr information.");
            return(E_FAIL);
        }

        /* Copy the band fields of the input DDR to the band fields of
           the output DDR except for minval, maxval, bandno, and validity.
           ---------------------------------------------------------------*/
        obdr[ii].bandno = ii + 1;
        strcpy(obdr[ii].source, ibdr->source);
        strcpy(obdr[ii].instrument, ibdr->instrument);
        strcpy(obdr[ii].direction, ibdr->direction);
        strcpy(obdr[ii].date, ibdr->date);
        strcpy(obdr[ii].time, ibdr->time);
    }

    /* Write the band independent information to the output ddr.
       ---------------------------------------------------------*/
    if (c_putddr(outhost, &oddr) != E_SUCC)
    {
        free(ibdr);
        free(obdr);
        c_lasmsg(FALSE, "resample-ddrwrite", "Error writing to the ddr file");
        return(E_FAIL);
    }

    /*  Write each of the bands to the output ddr.
        -------------------------------------------*/
    for (ii = 0; ii < nbands; ii++)
    {
        if (c_putbdr(outhost,&(obdr[ii])) != E_SUCC)
        {
            free(ibdr);
            free(obdr);
            c_lasmsg(FALSE, "resample-ddrwrite",
                    "Error writing to the ddr file");
            return(E_FAIL);
        }
    }

    /* Free the band DDR structures.
       -----------------------------*/
    free(ibdr);
    free(obdr);

    return(E_SUCC);
}
