/*****************************************************************************
NAME: ols2ils

PURPOSE:        
ols2ils finds the pixel in input space that corresponds to a given pixel in 
output space.

DEVELOPMENT HISTORY:


RETURN TYPE:
This routine returns the input space line/sample in the provided pointers.


NOTES:
- This routine assumes the output space pixel passed in lies within the grid
(it is checked by an ASSERT which is only present in debug mode).
- The output pixel passed in is an integer since the resampler only ever 
maps integer output space pixels to input space.  If this routine is ever 
used for a different purpose, the output space line/sample should be changed
to doubles.

ALGORITHM REFERENCES:
None

*****************************************************************************/

/*
**
** INCLUDE FILES 
**
*/

#include "resample.h"
#include "worgen.h"         /* c_trans prototype                            */
#include "debug.h"          /* ASSERT macro                                 */

static struct PROJ *inproj;  /* input projection params                     */
static struct PROJ *outproj; /* output projection params                    */
static const double *ipixsize, *opixsize;
                             /* pixel size array pointer (line,samp)(in,out)*/
static const double *iulcoor, *oulcoor;
                         /* UL corner coordinate pointer (lat,long)(in,out) */


int initialize_ols2ils(void)
{
    long prtprm[2]={FALSE, FALSE};     /* print dest. flags for c_transinit  */


    /* Retrieve projection, pixel, and coordinate information.
       -------------------------------------------------------*/
    inproj = (struct PROJ *)get_inproj();
    outproj = (struct PROJ *)get_outproj();
    opixsize = get_opixsize();
    ipixsize = get_ipixsize();
    oulcoor = get_oulcoor();
    iulcoor = get_iulcoor();

    /* Initialize the projection transformation.
       ----------------------------------------*/
    if (c_transinit(&outproj->proj_code, &outproj->units, &outproj->zone_code,
                &outproj->datum_code, outproj->proj_coef,
                &inproj->proj_code, &inproj->units, &inproj->zone_code,
                &inproj->datum_code, inproj->proj_coef, prtprm, NULL)
            != E_SUCC)
    {
        c_lasmsg(FALSE, "resample-trans",
                "Error initializing the transformation package.");
        return E_FAIL;
    }

    return E_SUCC;
}



int ols2ils
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/
    int output_line,            /* I: output line                           */
    int output_sample,          /* I: output sample                         */
    double *input_line_ptr,     /* O: input line                            */
    double *input_sample_ptr    /* O: input sample                          */
)
{
    int grid_cell_index;      /* index to cell in row for grid coefficients */
    COEFFICIENTS *coeff_ptr;  /* pointer to reverse coefficients in grid    */
    double d_output_sample = (double)output_sample;
    double d_output_line = (double)output_line;


    if (grid_band_ptr)
    {
        /* Verify the specified output space pixel lies in the grid.
           ---------------------------------------------------------*/
        ASSERT((output_line >= 0) && 
                (output_line < grid_band_ptr->output_image_lines));
        ASSERT((output_sample >= 0) &&
                (output_sample < grid_band_ptr->output_image_samps));

        /* Convert the output sample into a grid column.
           ---------------------------------------------*/
        grid_cell_index = output_line/grid_band_ptr->cell_lines * 
            grid_band_ptr->grid_cols + 
            output_sample/grid_band_ptr->cell_samps;

        /* Get the pointer to the grid coefficients.
           -----------------------------------------*/
        coeff_ptr = &grid_band_ptr->reverse_coeffs[grid_cell_index];

        /* Using the grid's reverse coefficients, calculate the input space
           line/sample.
           ----------------------------------------------------------------*/
        *input_sample_ptr = coeff_ptr->a[0] +
            coeff_ptr->a[1]*d_output_sample + 
            coeff_ptr->a[2]*d_output_line + 
            coeff_ptr->a[3]*d_output_sample*d_output_line;
        *input_line_ptr = coeff_ptr->b[0] +
            coeff_ptr->b[1]*d_output_sample + 
            coeff_ptr->b[2]*d_output_line + 
            coeff_ptr->b[3]*d_output_sample*d_output_line;
    }
    else
    {
        /* Compute projection-space coordinate of line/sample from
           upper left corner and pixel size.
           -------------------------------------------------------*/
        d_output_line = oulcoor[0] - d_output_line*opixsize[0];
        d_output_sample = oulcoor[1] + d_output_sample*opixsize[1];

        /* Find the corresponding input space coordinate.
           ----------------------------------------------*/
        if (c_trans(&outproj->proj_code, &outproj->units, &inproj->proj_code,
                    &inproj->units, &d_output_sample, &d_output_line,
                    input_sample_ptr, input_line_ptr) != E_SUCC)
            return (E_FAIL);

        /* Compute the input image line/sample location.
           ---------------------------------------------*/
        *input_line_ptr = (iulcoor[0] - *input_line_ptr)/ipixsize[0];
        *input_sample_ptr = (*input_sample_ptr - iulcoor[1])/ipixsize[1];
    }

    return (E_SUCC);
}
