/*****************************************************************************
FUNCTION:       out_frame

PURPOSE:        Calculates a set of coefficients mapping from output space
                image coordinates to output space projection coordinates

*****************************************************************************/
#include <math.h>
#include "resample.h"


int out_frame
(
    double *pixsize,      /* I: Projection distance in x & y                */
    long *window,         /* O: Output window (SL, SS, NL, NS)              */
    double *corners,      /* O: Output window corner coordinates            */
    struct PROJ *outproj, /* I: Output projection parameters                */
    struct PROJ *inproj,  /* I: Input projection parameters                 */
    long frame_mode,      /* I: Type of framing                             */
    double *ul,           /* I: Upper left corner coordinates               */
    double *lr,           /* I: Lower right corner coordinates              */
    long *ls,             /* I: Line/sample coordinates of ul               */
    long nl,              /* I: Number of lines in output space             */
    long ns,              /* I: Number of samples in output space           */
    long units,           /* I: Units code of coordinates entered           */
    double pxmax,         /* I: Maximum X projection coordinate             */
    double pxmin,         /* I: Minimum X projection coordinate             */
    double pymax,         /* I: Maximum Y projection coordinate             */
    double pymin          /* I: Minimum Y projection coordinate             */
)
{
    double b;               /* Constant in the equation of the line
                               (y = mx + b)                                  */
    double inx, iny;        /* Temporary storage of projection min/max       */
    double temp[8];         /* Temporary lat/long values                     */
    double diagpt[4];       /* lat/long of a point along diagonal between UL
                               corner and LR corner. Used to determine if
                               image wraps over the 180 degree longitude
                               diagpt[0] - output space Y image coordinate
                               diagpt[1] - output space X image coordinate
                               diagpt[2] - output space Y proj. coordinate
                               diagpt[3] - output space X proj. coordinate   */
    double slope;           /* the slope of the diagonal line between the
                               UL and LR corner. Used to step along that
                               diagonal and find a point in the image that
                               is not in the break area (diagpt)             */
    double temp_mod;        /* Results from fmod operation                   */
    double x_dist;          /* Distance between the x min and x max value    */
    double x_step;          /* A distance that is 10% of x_dist - the
                               distance in the x direction to move each time
                               we need to check for a new point on the
                               UL-LR diagonal (diagpt)                       */

    int ul_in_break = FALSE;/* Flags that are set if IN_BREAK                */
    int lr_in_break = FALSE;
    int ll_in_break = FALSE;
    int ur_in_break = FALSE;
    int got_diagpt = FALSE; /* Indicates that a good point - just a point
                               between the UL corner and LR corner - was
                               found that is not in the break area           */
    int mid_pt_flg = TRUE;  /* TRUE for first point to check (true midpoint),
                               when the loop completes first time it will set
                               the diagpt back to the UL corner to start
                               checking points along the diagonal from UL-LR */
    int pos_flg1;           /* Flag to indicate if the difference between the
                               midpoint X component and the UL corner X
                               component is positive. TRUE-> +, FALSE-> - */
    int pos_flg2;           /* Flag to indicate if the difference between the
                               UL corner X component and the midpoint X
                               component is positive. TRUE-> +, FALSE-> - */

    long status;            /* function status                                */
    long zero = 0;          /* Constant of zero                               */

    /* If geographic corner coordinates were entered, determine the
       lat/long for each corner and find the min box.  Proceed as if
       frame is to be determined from a rectangular geographic space.
       c_transinit is called in getpar prior to calling out_frame.
       -------------------------------------------------------------*/
    if (frame_mode == INPUT_XY)
    {
        /* UL */
        inx = pxmin;
        iny = pymax;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                        &inx, &iny, &(temp[1]), &(temp[0]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            ul_in_break = TRUE;             /* status = IN_BREAK */
        }

        /* LR */
        inx = pxmax;
        iny = pymin;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                        &inx, &iny, &(temp[3]), &(temp[2]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            lr_in_break = TRUE;             /* status = IN_BREAK */
        }

        /* LL */
        inx = pxmin;
        iny = pymin;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                        &inx, &iny, &(temp[5]), &(temp[4]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            ll_in_break = TRUE;             /* status = IN_BREAK */
        }

        /* UR */
        inx = pxmax;
        iny = pymax;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                        &inx, &iny, &(temp[7]), &(temp[6]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            ur_in_break = TRUE;             /* status = IN_BREAK */
        }

        /* Start at true mid-point of image to look for a point in the
           image that isn't in the break area to use to determine if the
           image crosses the 180 degree longitude. If the true mid-point
           won't work, then step down the diagonal from the UL corner
           to the LR corner to find one that will work.
           -------------------------------------------------------------*/
        /* Set up the variables needed */
        slope = (pymin - pymax) / (pxmax - pxmin);
        b = pymax - (slope * pxmin);
        x_dist = pxmax - pxmin;
        x_step = 0.1 * x_dist;

        /* Now find a point in the image */
        for(diagpt[0] = pymax - ((pymax - pymin) / 2.0),
            diagpt[1] = pxmax - ((pxmax - pxmin) / 2.0); /* true mid-point */
            (diagpt[0] >= pymin) && (diagpt[1] <= pxmax); /* stop when the
                             point exceeds the lower or right image bounds */
            diagpt[1] = diagpt[1] + x_step, /* move x_step latitudinally to
                                               get longitude of next point */
            diagpt[0] = (slope * diagpt[1]) + b /* corresponding latitude */ )
        {
            if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero,
                            &units, &(diagpt[1]), &(diagpt[0]),
                            &(diagpt[3]), &(diagpt[2]))) != E_SUCC)
            {
                if (status == E_FAIL)
                    return (E_FAIL);
                /* else IN_BREAK; check next point */
            }
            else
            {
                got_diagpt = TRUE;
                break; /* found a point that did not fall in the break area */
            }
            if(mid_pt_flg)
            {
                /* Just got done checking the mid-point, but it fell in
                   the break area. Move the diagpt back to the UL corner
                   to start checking points along the UL-LR diagonal.
                   Set the mid_pt_flg to FALSE since we only want to do
                   this the first time through the loop.
                --------------------------------------------------------*/
                diagpt[0] = pymax;
                diagpt[1] = pxmin;
                mid_pt_flg = FALSE;
            }
        }
        if(!got_diagpt)
        {
            c_lasmsg(FALSE, "resample-break",
                     "Error determining framing coordinates.");
            return(E_FAIL); /* did not find a point along the diagonal, fail */
        }

        /* If the corner coordinates define a window crossing the 180 degree
           longitude, change the coordinates to work with c_mapedg.
           -----------------------------------------------------------------*/
        if(!ul_in_break)
        {
            if((diagpt[3] - temp[1]) > 0.0) /* difference between longitudes of
                                           transformed midpoint and UL corner */
                pos_flg1 = TRUE;
            else
                pos_flg1 = FALSE;
        }
        else if (!ll_in_break)
        {
            if((diagpt[3] - temp[5]) > 0.0) /* difference between longitudes of
                                           transformed midpoint and LL corner */
                pos_flg1 = TRUE;
            else
                pos_flg1 = FALSE;
        }
        else
        {
            /* Both left corner were in the break area, so error */
            c_lasmsg(FALSE, "resample-break",
                     "Error determining framing coordinates.");
            return(E_FAIL);
        }

        if(!lr_in_break)
        {
            if((temp[3] - diagpt[3]) > 0.0) /* difference between logitudes of
                                           transformed LR corner and midpoint */
                pos_flg2 = TRUE;
            else
                pos_flg2 = FALSE;
        }
        else if(!ur_in_break)
        {
            if((temp[7] - diagpt[3]) > 0.0) /* difference between logitudes of
                                           transformed UR corner and midpoint */
                pos_flg2 = TRUE;
            else
                pos_flg2 = FALSE;
        }
        else
        {
            /* Both right corners were in the break area, so error */
            c_lasmsg(FALSE, "resample-break",
                     "Error determining framing coordinates.");
            return(E_FAIL);
        }

        /* The signs of the differences calculated above being different
           indicates the image crosses the 180 degree longitude in output
           space. Subtracting 360 from the X coordinates of the left corners
           will put the corners into the correct perspective to each other
           again (make the left side of the image appear to be west of the
           right side again). */
        if(pos_flg1 != pos_flg2)
        {
            switch(units)
            {
                case 4:
                    temp[1] -= 360; /* UL longitude */
                    temp[5] -= 360; /* LL longitude */
            }
        }

        /* Find the min/max corners.  Only use
           the lat/long if c_trans was successful
           Find UL Y
           --------------------------------------*/
        if (!ul_in_break && !lr_in_break)
            ul[0] = GMAX(temp[0],temp[2]);
        else if (!ul_in_break)
            ul[0] = temp[0];
        else if (!lr_in_break)
            ul[0] = temp[2];

        if (!ll_in_break)
            ul[0] = GMAX(ul[0],temp[4]);
        if (!ur_in_break)
            ul[0] = GMAX(ul[0],temp[6]);

        /* Find LR Y
           ---------*/
        if (!ul_in_break && !lr_in_break)
            lr[0] = GMIN(temp[0],temp[2]);
        else if (!ul_in_break)
            lr[0] = temp[0];
        else if (!lr_in_break)
            lr[0] = temp[2];

        if (!ll_in_break)
            lr[0] = GMIN(lr[0],temp[4]);
        if (!ur_in_break)
            lr[0] = GMIN(lr[0],temp[6]);

        /* Find UL X
           ---------*/
        if (!ul_in_break && !lr_in_break)
            ul[1] = GMIN(temp[1],temp[3]);
        else if (!ul_in_break)
            ul[1] = temp[1];
        else if (!lr_in_break)
            ul[1] = temp[3];

        if (!ll_in_break)
            ul[1] = GMIN(ul[1],temp[5]);
        if (!ur_in_break)
            ul[1] = GMIN(ul[1],temp[7]);

        /* Find LR X
           ---------*/
        if (!ul_in_break && !lr_in_break)
            lr[1] = GMAX(temp[1],temp[3]);
        else if (!ul_in_break)
            lr[1] = temp[1];
        else if (!lr_in_break)
            lr[1] = temp[3];

        if (!ll_in_break)
            lr[1] = GMAX(lr[1],temp[5]);
        if (!ur_in_break)
            lr[1] = GMAX(lr[1],temp[7]);

        /* When remapping an image in Goodes (either North America or
           Eurasia), the UL and UR could both be in the break area.  This
           will check a point in the middle in order to find a max y.  If
           this fails, the user must break the image into small sections and
           resample individually.
           ----------------------------------------------------------------*/

        if (ul_in_break && ur_in_break)
        {
            inx = (pxmax + pxmin)/2;
            iny = pymax;

            if ((status = c_trans(&inproj->proj_code,&inproj->units, &zero,
                            &units, &inx, &iny, &(temp[1]), &(temp[0])))
                    != E_SUCC)
            {
                if (status == E_FAIL)
                    return (E_FAIL);

                /* status = IN_BREAK */
                c_lasmsg(FALSE, "resample-break",
                        "Error determining framing coordinates.");
                return (E_FAIL);
            }

            ul[0] = GMAX(ul[0],temp[0]);
        }

        /* The LL and LR coordinate transformations failed.  The same thing
           as above could be done, but I don't have test data.
           ----------------------------------------------------------------*/
        if (lr_in_break && ll_in_break)
        {
            c_lasmsg(FALSE, "resample-break",
                    "Error determining framing coordinates.");
            return (E_FAIL);
        }

        frame_mode = INPUT;
    }

    if (frame_mode == INPUT)
    {
        /* Determine output projection space Y/X minimums and maximums.
           ------------------------------------------------------------*/
        if (c_mapedg(&(ul[1]), &(lr[1]), &(ul[0]), &(lr[0]), &units,
                    &(outproj->proj_code), &(outproj->zone_code),
                    outproj->proj_coef, &(outproj->units), &pxmin, &pxmax,
                    &pymin, &pymax, &(outproj->datum_code)) != E_SUCC)
            return(E_FAIL);

        /* The UL corner is used as the center of the first line/sample.
           Round the LR corner to the nearest multiple of the pixel size,
           ensuring that the computed min/max values are included.
           --------------------------------------------------------------*/
        temp_mod = pymax - pymin;
        temp_mod = fmod(temp_mod, pixsize[0]);
        if (temp_mod/pixsize[0] > 1e-3)
            pymin -= pixsize[0] - temp_mod;
        else
            pymin += temp_mod;
        temp_mod = pxmax - pxmin;
        temp_mod = fmod(temp_mod, pixsize[1]);
        if (temp_mod/pixsize[1] > 1e-3)
            pxmax += pixsize[1] - temp_mod;
        else
            pxmax -= temp_mod;

    }
    else if (frame_mode == OUTPUT_XY)
    {
        /* If output frame was given in projection coordinates
           (two points).
           ---------------------------------------------------*/
        pymax = ul[0] + ((ls[0] - 1) * pixsize[0]);
        pxmin = ul[1] - ((ls[1] - 1) * pixsize[1]);
        pymin = lr[0];
        pxmax = lr[1];
    }
    else if (frame_mode == OUTPUT_LS)
    {
        /* If output frame was given in projection coordinates
           (1 pt & nl, ns).
           ---------------------------------------------------*/
        pymax = ul[0] + ((ls[0] - 1) * pixsize[0]);
        pxmin = ul[1] - ((ls[1] - 1) * pixsize[1]);
        pymin = pymax - ((nl - 1) * pixsize[0]);
        pxmax = pxmin + ((ns - 1) * pixsize[1]);
    }

    /* Set the output window.  The addition of the 1.5 consists of 0.5
       to handle roundoff and 1 to handle the "zeroth" element.
       ---------------------------------------------------------------*/
    window[0] = window[1] = 1;
    window[2] = (long)((pymax - pymin) / pixsize[0] + 1.5);
    window[3] = (long)((pxmax - pxmin) / pixsize[1] + 1.5);

    /* Pack the corner projection coordinates in the frame structure.
       --------------------------------------------------------------*/
    corners[1] = corners[3] = pxmin;
    corners[0] = corners[4] = pymax;
    corners[6] = corners[2] = pymin;
    corners[5] = corners[7] = pxmax;

    return(E_SUCC);
}
