/*****************************************************************************
NAME:           output_image_buffer

PURPOSE:        
The output_image_buffer hides the details of buffering the output image from
the rest of the resampler.  The goal is to allow more flexibility in the 
memory used by the resampler.

ROUTINES:
    calculate_optimum_buffer
    initialize_output_image_buffer
    get_output_image_buffer
    return_output_image_buffer
    cleanup_output_image_buffer

DEVELOPMENT HISTORY:
  Baselined from revision 1.4 of output_image_buffer.c

NOTES:
- The user must initialize the output image buffer when starting to resample
  a band and cleanup the image buffer after the band is done.
- To protect the sensitive areas of the code from the effects of
  multiprocessing, a critical section is used.  The critical section could be
  split into two different semaphores if desired.  One for the File I/O and
  one for the free buffer manipulation.

Buffer organization:
    The memory to buffer the output image is organized into blocks of output
    lines.  Each block has "granularity" lines in it.  A resampling thread
    requests the buffer for the current block (or set of lines) it is
    currently working on and returns the buffer when it is finished with the
    block.
Image buffering modes:
    The output image may be buffered in two different modes: full image 
    buffering or partial image buffering.  It either selects which mode to 
    use given the memory allowed and the estimated number of active lines
    in the output image, or a command line option may force full image
    buffering.  Full image buffering keeps the entire output image in memory
    at once.  This mode is selected when there is a large rotation between
    the input image and the output image, or when a large number or
    processors are used, or when the command line option forces it.  Partial
    image buffering is selected at all other times.
Buffer allocation:
    The buffer_allocation_ptr points to the buffer allocation array that
    tracks which buffer block is currently being used for each set of lines
    being resampled.  When full image buffering is being used, this array is
    filled to arrange the entire output image in memory sequentially so it
    may be written by a single write call when the output buffer is cleaned
    up at the end of resampling.  When partial image buffering is used, the
    set of buffers available is preallocate by filling in the buffer
    allocation array with the available buffers.
Partial write mode:
    When partial image buffering is being used, it is possible to run out of
    available buffers but still need more to continue processing the portion
    of the input image that is in memory.  This may occur when not enough
    memory was available to buffer estimated set of active lines, or when
    that estimate was not totally correct.  To accomodate this situation,
    there is an extra buffer set aside for each thread to be used when the
    set of buffers is exhausted.  When one of these extra thread buffers are
    returned, the completed portion of the buffer is immediately written to
    disk, even if the lines in it are not completely created.  The sample
    written and sample reached arrays are used to track how much of each line
    has been written when the partial image buffering occurs.  The sample
    written array is valid only for the blocks where partial writes occur.
Next buffer to allocate index:
    The next buffer to allocate index tracks which buffer in the buffer 
    allocation array is the next one to be allocated a buffer.  Once a block
    of output lines has been completely written generated, it is written to
    disk.  The buffer is then reused by entering its index in the buffer
    allocation array at the next buffer to allocate index.  The next buffer
    to allocate index accesses should be protected by a critical section 
    (or semaphore if available) in most cases.
Buffer interleaving:
    When partial image buffering is being used and it is known that not enough 
    memory is available to prevent partial write mode from occurring, a buffer
    interleave factor is calculated.  The goal of the buffer interleave is to 
    spread the partial writes evenly over time to prevent all the threads from
    trying to perform a partial buffer write at the same time.  So, the 
    interleave factor is used to make an entry in the buffer allocation array
    every "interleave" counts that forces a partial write buffer to be used.
Write Queue:
    When this module is compiled with both USE_POSIX_MULTIPROC and 
    USE_WRITE_THREAD defined, a separate thread is used to write the image
    buffers to the HDF file (except for full image buffering).  A queue is
    created for the return_output_image_buffer routine to enter the buffers 
    that need writing.  The write_buffer_thread then does the actual writes 
    of the buffers to the file and returns the buffers to the buffer
    allocation table.  This method helps reduce the amount of time the 
    resampling threads spend waiting for file I/O.
    
ALGORITHM REFERENCES:
None

*****************************************************************************/

/*
**
** INCLUDE FILES
**
*/

#include <stdlib.h>
#include <string.h>
#include "resample.h"
#include "debug.h"                 /* ASSERT macro                        */

/*
**
** MACROS
**
*/

#define LARGE_NUMBER_THREADS_THRESHOLD 6
                                  /* defines the number of threads that are
                                     considered a "large" number for the
                                     optimum buffer determination           */
#define ROTATION_FULL_BUFFER_THRESHOLD 0.75
                                  /* The percent (where 1.0 = 100%) of output
                                     image lines above which the entire output
                                     image will attempt to be buffered      */
#define BYTES_IN_MEGABYTE 1048576 /* defines the number of bytes in a
                                     megabyte of memory                     */

#ifdef USE_WRITE_THREAD

THREAD_RETURN_TYPE write_buffer_thread(void *params_ptr);

#endif


/*****************************************************************************
FUNCTION NAME:  calculate_optimum_buffer

PURPOSE:        
calculate_optimum_buffer uses the number of processors, relative rotation 
between the input and output images, and available memory for output buffering
to select partial/full buffering and the granularity of the output buffer rows.

RETURNS:
The buffer granularity, number of buffers, and full_buffering flag are
returned.

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

static int calculate_optimum_buffer
(
    int *active_lines,  /* I/O: number of lines that should be kept in memory
                                to prevent partial line writes              */
    int min_active_lines,  /* I: minimum number of lines active in
                                 the output image                           */
    int *memory_allowed, /* I/O: mem allowed for output image buffer (in MB)*/
    int buffer_full_image, /* I: flag to indicate full image should be
                                 buffered, ignoring memory limit            */
    OUTPUT_IMAGE_BUFFER_TYPE *buffer_ptr,/*I/O: pointer to output image buffer
                                                information                 */
    int pass                     /* I: iteration pass number for determining
                                       input lines to buffer and output image
                                       buffer size                          */
)
{
    int megabytes_needed;  /* number of megabytes needed for all active lines
                              to remain in memory                           */
    int rows_to_allocate = 0; /* number of buffer rows to allocate          */
    int max_rows;          /* maximum number of buffer rows that can be
                              allocated in the memory allowed               */


    buffer_ptr->full_buffer = 0;  /* default to not using full buffering    */

    buffer_ptr->interleave = 0;   /* default the interleave                 */

    /* Attempt to buffer the full image if there are a large number of 
       processors, the buffer full image flag is set, or the relative
       rotation of the image is large.
       ---------------------------------------------------------------*/
    if ((buffer_ptr->threads >= LARGE_NUMBER_THREADS_THRESHOLD) ||
        (buffer_full_image) || (*active_lines > 
         (buffer_ptr->output_window.num_lines*ROTATION_FULL_BUFFER_THRESHOLD)))
    {
        /* Calculate amount of memory needed to keep the entire output image
           in memory.  The float typecast is used to make sure overflow
           doesn't occur if the image is larger than 2GB.
           -----------------------------------------------------------------*/
        megabytes_needed = (float)buffer_ptr->bytes_per_line*
            buffer_ptr->output_window.num_lines/BYTES_IN_MEGABYTE;

        /* Check if there is enough memory allowed for the output buffer to
           hold the entire output image or if buffer full image flag is set.
           -----------------------------------------------------------------*/
        if ((megabytes_needed <= *memory_allowed) || (buffer_full_image))
        {
            /* Can full buffer.
               ----------------*/
            buffer_ptr->full_buffer = 1;
            buffer_ptr->granularity = SMALL_GRANULARITY;
            rows_to_allocate = (buffer_ptr->output_window.num_lines / 
                    buffer_ptr->granularity) + 1;
        }
        else
        {
            /* Issue a warning that full buffering was not used and that it
               may slow down performance.
               ------------------------------------------------------------*/
            c_lasmsg(FALSE, "resample-buffer", "Full buffering of output "
                    "image not in use.  Performance may be reduced.");
        }
    }

    /* If not doing full buffering, determine if all the lines that need to
       be partially buffered will be able to remain in memory.
       --------------------------------------------------------------------*/
    if (!buffer_ptr->full_buffer)
    {
        /* Calculate the maximum amount of memory needed to keep the active 
           lines in memory.  The float typecast is used to make sure
           overflow doesn't occur if the size is larger than 2GB.  The
           number of threads and "4" are added to allow enough buffers for
           each thread to detect the end of output for the current pass and
           the fact that the buffer may not be exactly aligned on row
           boundaries.
           ----------------------------------------------------------------*/
        megabytes_needed =
            (*active_lines/LARGE_GRANULARITY + buffer_ptr->threads + 4) * 
            LARGE_GRANULARITY * (float)buffer_ptr->bytes_per_line
            / BYTES_IN_MEGABYTE;

        /* If there is sufficient memory, use LARGE_GRANULARITY.  Otherwise
           use SMALL_GRANULARITY.
           ----------------------------------------------------------------*/
        if (megabytes_needed <= *memory_allowed)
            buffer_ptr->granularity = LARGE_GRANULARITY;
        else
            buffer_ptr->granularity = SMALL_GRANULARITY;

        /* Calculate the number of rows to allocate.
           -----------------------------------------*/
        rows_to_allocate = *active_lines/buffer_ptr->granularity + 
            buffer_ptr->threads + 4;
#ifdef USE_WRITE_THREAD

        /* Add some extra buffers if there is a thread doing the writes so 
           some can be in the queue without adversely affecting the active
           set of buffers.
           ---------------------------------------------------------------*/
        rows_to_allocate += 4 * buffer_ptr->threads;
#endif

        /* Calculate the maximum number of rows available.
           -----------------------------------------------*/
        max_rows = (int)(((float)*memory_allowed * BYTES_IN_MEGABYTE) / 
                (buffer_ptr->bytes_per_line * buffer_ptr->granularity));

        /* Limit the rows to the maximum.
           ------------------------------*/
        if (rows_to_allocate > max_rows)
        {
            /* If this is pass 1 (in warp), see if we can at least handle
               the minimum number of active lines.
               ----------------------------------------------------------*/
            if ((min_active_lines < *active_lines) && pass == 1)
            {
                /* Calculate the minimum number of rows to allocate.
                   -------------------------------------------------*/
                rows_to_allocate = min_active_lines/buffer_ptr->granularity +
                    buffer_ptr->threads + 4;

#ifdef USE_WRITE_THREAD

                /* Add some extra buffers if there is a thread doing the
                   writes so some can be in the queue without adversely
                   affecting the active set of buffers.
                   -----------------------------------------------------*/
                rows_to_allocate += 4 * buffer_ptr->threads;
#endif

                if (rows_to_allocate > max_rows)
                {
                    /* Calculate the number of megabytes short.
                       ----------------------------------------*/
                    megabytes_needed = (int)((float)rows_to_allocate *
                           buffer_ptr->bytes_per_line * buffer_ptr->granularity/
                           BYTES_IN_MEGABYTE) - *memory_allowed + 1;
                }
                else
                {
#ifdef USE_WRITE_THREAD
                    max_rows -= 4*buffer_ptr->threads;
#endif
                    /* We have enough memory to work with, so compute the
                       number of active lines we can handle, and pass the
                       value back to warp for pass 2.
                       --------------------------------------------------*/
                    *active_lines = (max_rows - buffer_ptr->threads - 4)*
                        buffer_ptr->granularity;

                    return (E_FAIL);
                }
            }
            else
                megabytes_needed = (int)((float)rows_to_allocate *
                        buffer_ptr->bytes_per_line * buffer_ptr->granularity/
                        BYTES_IN_MEGABYTE) - *memory_allowed + 1;

            /* We need more memory in CACHESZ, so up the value to what is
               needed and inform the user of the change.
               ----------------------------------------------------------*/
            *memory_allowed += megabytes_needed;

            c_lasmsg(FALSE, "resample-cache",
                    "Short %d MB of memory to buffer the active"
                    " lines in the output image.", megabytes_needed);
            c_lasmsg(FALSE, "resample-cache",
                    "Increasing CACHESZ to %d MB.", *memory_allowed);
        }
    }

    /* Calculate the last row index base on granularity.
       -------------------------------------------------*/
    buffer_ptr->last_row_index = (buffer_ptr->output_window.num_lines /
            buffer_ptr->granularity) + 1;

    /* Set the interleave count if it has not been set.
       ------------------------------------------------*/
    if (buffer_ptr->interleave == 0)
        buffer_ptr->interleave = buffer_ptr->last_row_index + 1;    

    /* if the rows_to_allocate is larger than the last row index, that means
       it is more efficient to fully buffer the output image */ 
    if (rows_to_allocate > buffer_ptr->last_row_index)
    {
        buffer_ptr->full_buffer = 1;
        buffer_ptr->granularity = SMALL_GRANULARITY;

        /* Re-calculate the last row index base on new granularity.
           --------------------------------------------------------*/
        buffer_ptr->last_row_index = (buffer_ptr->output_window.num_lines /
                buffer_ptr->granularity) + 1;

        /* Re-calculate the interleave count because of new last_row_index.
           ----------------------------------------------------------------*/
        buffer_ptr->interleave = buffer_ptr->last_row_index + 1;    

        /* It is possible the active_lines should be updated here as well.
           At the time the changes to update the last_row_index and
           interleave were added, it was not deemed necessary since the
           active_lines not being updated will probably only affect some
           extreme cases. If, in the future, problems are found with the
           amount of memory being used, the active_lines value should be
           re-evaluated and considered being updated here so it is not larger
           than necessary. */

        rows_to_allocate = buffer_ptr->last_row_index;

        c_lasmsg(FALSE, "resample-fullbuf", "Full output image buffering was "
                "found to be required to successfully process the image so "
                "it has been forced.");
    }

    buffer_ptr->buffers = rows_to_allocate;

    return E_SUCC;
}


/*****************************************************************************
FUNCTION NAME:  initialize_output_image_buffer

PURPOSE:        
initialize_output_image_buffer does the initial setup of an instance of 
an output image buffer.  All the required memory is allocated and initialized
where applicable.

RETURNS:
A pointer to the output image buffer state structure is returned.  If an error
occurs, an error message is generated and NULL is returned.

NOTES:
The active_lines parameter is a "guess" at how many lines can only be partially
generated from the input data.  This helps to figure out how much memory is 
required to efficiently buffer the output image.

ALGORITHM REFERENCES:
none

*****************************************************************************/

OUTPUT_IMAGE_BUFFER_TYPE *initialize_output_image_buffer
(
    int *active_lines,       /* I/O: number of lines active in output image */
    int min_active_lines,        /* I: minimum number of lines active in
                                       the output image                     */
    int band,                    /* I: current band                         */
    struct FDESC *otfdsc,        /* I: output file descriptor               */
    int threads,                 /* I: number of threads to accomodate      */
    int resample_dir,            /* I: resampling direction                 */
    int *max_output_image_buffer,/* I/O: maximum memory to use for output
                                       buffering                            */
    int buffer_full_output_image,/* I: flag to buffer entire output image,
                                       ignoring max memory size             */
    int pass                     /* I: iteration pass number for determining
                                       input lines to buffer and output image
                                       buffer size                          */
)
{
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr; /* pointer to output image 
                                                    state buffer allocated  */
    int count;           /* loop counter                                    */
    int temp;            /* temporary variable                              */
    int buffer_num;      /* buffer number when allocating buffers           */
    int rounded_lines;   /* number of lines in the output rounded to the 
                            next largest multiple of granularity */
    int interleave;      /* buffer interleave count when assigning buffers
                            to rows                                         */


    /* Allocate memory for the output buffer state.
       --------------------------------------------*/
    output_buffer_ptr = 
        (OUTPUT_IMAGE_BUFFER_TYPE *)malloc(sizeof(OUTPUT_IMAGE_BUFFER_TYPE));
    if (output_buffer_ptr == NULL)
    {
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (NULL);
    }

    /* Initialize members of the output image buffer structure.
       --------------------------------------------------------*/
    output_buffer_ptr->otfdsc = otfdsc;
    output_buffer_ptr->threads = threads;
    output_buffer_ptr->band = band + 1;
    output_buffer_ptr->output_window = *get_output_window(band);
    output_buffer_ptr->data_size = get_output_data_size();
    output_buffer_ptr->resample_dir = resample_dir;
    output_buffer_ptr->bytes_per_line = output_buffer_ptr->data_size *
        output_buffer_ptr->output_window.num_samples;

    output_buffer_ptr->full_buffer_writes = 0;

    if (calculate_optimum_buffer(active_lines, min_active_lines,
                max_output_image_buffer,
                buffer_full_output_image, output_buffer_ptr,
                pass) != E_SUCC)
    {
        free(output_buffer_ptr);
        return (NULL);
    }

    output_buffer_ptr->row_buffer_size = output_buffer_ptr->bytes_per_line *
        output_buffer_ptr->granularity;

    /* Allocate memory for the data buffer, making sure it is in whole
       row buffer increments.
       ---------------------------------------------------------------*/
    output_buffer_ptr->data_buffer_ptr =
        malloc(output_buffer_ptr->row_buffer_size*
                output_buffer_ptr->buffers);
    if (output_buffer_ptr->data_buffer_ptr == NULL)
    {
        free(output_buffer_ptr);
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (NULL);
    }

    /* calculate the number of lines rounded to the next highest multiple
       of granularity.  This is to allow for easily copying granularity sized
       chunks of data in the sample reached and written arrays. */
    rounded_lines = (((output_buffer_ptr->output_window.end_line + 1) / 
                    output_buffer_ptr->granularity) + 1) * 
                    output_buffer_ptr->granularity;
    
    /* Allocate memory for the sample reached array from index 0 through the
       number of lines in the window.
       ---------------------------------------------------------------------*/
    output_buffer_ptr->sample_reached_ptr = (int*)malloc(sizeof(int)*
            rounded_lines);
    if (output_buffer_ptr->sample_reached_ptr == NULL)
    {
        free(output_buffer_ptr->data_buffer_ptr);
        free(output_buffer_ptr);
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (NULL);
    }

    /* Initialize the reached sample array.
       ------------------------------------*/
    temp = output_buffer_ptr->output_window.end_line;
    for (count = 0; count < temp; count++)
    {
        output_buffer_ptr->sample_reached_ptr[count] = 
            output_buffer_ptr->output_window.start_sample; 
    }

    /* Allocate memory for the sample written array.
       ---------------------------------------------*/
    output_buffer_ptr->sample_written_ptr = (int*)malloc(sizeof(int)*
            rounded_lines);
    if (output_buffer_ptr->sample_written_ptr == NULL)
    {
        free(output_buffer_ptr->data_buffer_ptr);
        free(output_buffer_ptr->sample_reached_ptr);
        free(output_buffer_ptr);
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (NULL);
    }

    /* Allocate memory for the buffer allocation array.
       ------------------------------------------------*/
    output_buffer_ptr->buffer_allocation_ptr = (int*)malloc(sizeof(int) * 
            output_buffer_ptr->last_row_index);
    if (output_buffer_ptr->buffer_allocation_ptr == NULL)
    {
        free(output_buffer_ptr->data_buffer_ptr);
        free(output_buffer_ptr->sample_reached_ptr);
        free(output_buffer_ptr->sample_written_ptr);
        free(output_buffer_ptr);
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (NULL);
    }

    /* Initialize the buffer allocations so that the correct buffers are 
       pre-allocated.  The first buffers are reserved so one is available
       for each thread.  These buffers are the buffers that are always
       written out.  The rest of the buffers (assuming they exist) are
       pre-allocated to the first rows to be resampled.  If there are not
       enough buffers to prevent partial line writes, the partial line
       buffers are interleaved between the buffers kept in memory to reduce
       the number of threads attempting to do partial line writes at the
       same time.
       --------------------------------------------------------------------*/

    interleave = output_buffer_ptr->interleave;

    if (output_buffer_ptr->full_buffer)
    {
        for(count = 0; count < output_buffer_ptr->last_row_index; count++)
            output_buffer_ptr->buffer_allocation_ptr[count] = count;

        output_buffer_ptr->next_buffer_to_alloc = count;

    }
    else if (resample_dir > 0)
    {
        /* Resampling from top to bottom so preallocate the buffers from top
           to bottom.
           -----------------------------------------------------------------*/
        for (count=0, buffer_num=threads;
             (buffer_num<output_buffer_ptr->buffers) && 
             (count<output_buffer_ptr->last_row_index); count++, buffer_num++)
        {
            output_buffer_ptr->buffer_allocation_ptr[count] = buffer_num;

            /* Check if time to interleave a partial write buffer with the
               preallocated buffers.
               -----------------------------------------------------------*/
            if (((count % interleave) == 0) && (count != 0))
            {
                /* Interleave the partial write buffer.
                   ------------------------------------*/
                count++;
                if (count < output_buffer_ptr->last_row_index)
                    output_buffer_ptr->buffer_allocation_ptr[count] = 0;
            }
        }

        /* Set the next buffer to allocate.
           --------------------------------*/
        output_buffer_ptr->next_buffer_to_alloc = count;

        /* Fill the rest of the buffers with the not allocated flag.
           ---------------------------------------------------------*/
        for (; count < output_buffer_ptr->last_row_index; count++)
            output_buffer_ptr->buffer_allocation_ptr[count] = -1;
    }
    else
    {
        /* Resampling from bottom to top, so preallocate the buffers from 
           bottom to top, with the buffer numbers for the bottom rows
           having the higher number so write combining will work.
           --------------------------------------------------------------*/
        for (count = output_buffer_ptr->last_row_index - 1, 
                buffer_num = output_buffer_ptr->buffers-1;
                (buffer_num >= threads) && (count >= 0); 
                count--, buffer_num--)
        {
            output_buffer_ptr->buffer_allocation_ptr[count] = buffer_num;

            /* Check if time to interleave a partial write buffer with the
               preallocated buffers.
               -----------------------------------------------------------*/
            if ((count % interleave) == 0)
            {
                /* Interleave the partial write buffer.
                   ------------------------------------*/
                count--;
                if (count >= 0)
                    output_buffer_ptr->buffer_allocation_ptr[count] = 0;
            }
        }

        /* Set the next buffer to allocate.
           --------------------------------*/
        output_buffer_ptr->next_buffer_to_alloc = count;

        /* Fill the rest of the buffers with the not allocated flag.
           ---------------------------------------------------------*/
        for (;count >=0; count--)
            output_buffer_ptr->buffer_allocation_ptr[count] = -1;

    }

    output_buffer_ptr->interleave_count = 1;

    /* Create the mutex's needed for file io and buffer manipulation.
       --------------------------------------------------------------*/
    if (CREATE_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
    {
        free(output_buffer_ptr->buffer_allocation_ptr);
        free(output_buffer_ptr->data_buffer_ptr);
        free(output_buffer_ptr->sample_reached_ptr);
        free(output_buffer_ptr->sample_written_ptr);
        free(output_buffer_ptr);
        return (NULL);
    }

    if (CREATE_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
    {
        DESTROY_MUTEX(&output_buffer_ptr->file_io_mutex);
        free(output_buffer_ptr->buffer_allocation_ptr);
        free(output_buffer_ptr->data_buffer_ptr);
        free(output_buffer_ptr->sample_reached_ptr);
        free(output_buffer_ptr->sample_written_ptr);
        free(output_buffer_ptr);
        return (NULL);
    }

#ifdef USE_WRITE_THREAD    
    if (!output_buffer_ptr->full_buffer)
    {
        /* Init semaphore -- another uglier hack.
           --------------------------------------*/
        if (CREATE_SEMAPHORE(&output_buffer_ptr->data_available_sem,0) != 0)
        {
            DESTROY_MUTEX(&output_buffer_ptr->file_io_mutex);
            DESTROY_MUTEX(&output_buffer_ptr->buffer_mutex);
            free(output_buffer_ptr->buffer_allocation_ptr);
            free(output_buffer_ptr->data_buffer_ptr);
            free(output_buffer_ptr->sample_reached_ptr);
            free(output_buffer_ptr->sample_written_ptr);
            free(output_buffer_ptr);
            return (NULL);
        }

        if (CREATE_MUTEX(&output_buffer_ptr->q_mutex) != 0)
        {
            DESTROY_SEMAPHORE(&output_buffer_ptr->data_available_sem);
            DESTROY_MUTEX(&output_buffer_ptr->file_io_mutex);
            DESTROY_MUTEX(&output_buffer_ptr->buffer_mutex);
            free(output_buffer_ptr->buffer_allocation_ptr);
            free(output_buffer_ptr->data_buffer_ptr);
            free(output_buffer_ptr->sample_reached_ptr);
            free(output_buffer_ptr->sample_written_ptr);
            free(output_buffer_ptr);
            return (NULL);
        }

        /* Initialize the queue indices.
           -----------------------------*/
        output_buffer_ptr->q_insert_index = 0;
        output_buffer_ptr->q_remove_index = 0;

        /* Allocate memory for the write queue, making it slightly larger
           than the number of buffers available so that we don't have to
           worry about queue overflow.
           --------------------------------------------------------------*/
        output_buffer_ptr->q_size = output_buffer_ptr->buffers + 2;
        output_buffer_ptr->q_ptr = (WRITE_QUEUE_BUFFER_TYPE *)
            malloc(output_buffer_ptr->q_size*sizeof(WRITE_QUEUE_BUFFER_TYPE));
        if (output_buffer_ptr->q_ptr == NULL)
        {
            DESTROY_MUTEX(&output_buffer_ptr->q_mutex);
            DESTROY_SEMAPHORE(&output_buffer_ptr->data_available_sem);
            DESTROY_MUTEX(&output_buffer_ptr->file_io_mutex);
            DESTROY_MUTEX(&output_buffer_ptr->buffer_mutex);
            free(output_buffer_ptr->buffer_allocation_ptr);
            free(output_buffer_ptr->data_buffer_ptr);
            free(output_buffer_ptr->sample_reached_ptr);
            free(output_buffer_ptr->sample_written_ptr);
            free(output_buffer_ptr);
            return (NULL);
        }

        /* Create the write thread.
           ------------------------*/
        if (pthread_create(&output_buffer_ptr->write_thread_id, NULL, 
                    write_buffer_thread, (void*)output_buffer_ptr) != 0)
        {
            free(output_buffer_ptr->q_ptr);
            DESTROY_MUTEX(&output_buffer_ptr->q_mutex);
            DESTROY_SEMAPHORE(&output_buffer_ptr->data_available_sem);
            DESTROY_MUTEX(&output_buffer_ptr->file_io_mutex);
            DESTROY_MUTEX(&output_buffer_ptr->buffer_mutex);
            free(output_buffer_ptr->buffer_allocation_ptr);
            free(output_buffer_ptr->data_buffer_ptr);
            free(output_buffer_ptr->sample_reached_ptr);
            free(output_buffer_ptr->sample_written_ptr);
            free(output_buffer_ptr);
            return (NULL);
        }   
    }
#endif

    /* Return the output buffer structure.
       -----------------------------------*/
    return (output_buffer_ptr);  
}


/*****************************************************************************
FUNCTION NAME:  cleanup_output_image_buffer

PURPOSE:        
cleanup_output_image_buffer checks to make sure the entire output image 
requested was generated.  It then frees all memory associated with the 
output image buffer.

RETURNS:
Type = int
Value           Description
-----           -----------
E_FAIL          Error; cleanup not successful
E_SUCC          Successfully cleanup up the output image buffer

NOTES:
After this call, the output image buffer will no longer contain valid data.

ALGORITHM REFERENCES:
none

*****************************************************************************/

int cleanup_output_image_buffer
(
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr,
                           /* I: pointer to output image buffer             */
    int success            /* I: set to non-zero if no errors have occurred */
)
{
    int status = E_SUCC;   /* return status, assume it will succeed         */
    int count;             /* loop counter                                  */
#ifdef USE_WRITE_THREAD
    void *return_val;      /* return value from the write thread            */
#endif
    long one = 1;          /* the number one                                */
    long numlines;


    if (output_buffer_ptr != NULL)
    {
        /* Check and save the output image if no errors detected.
           ------------------------------------------------------*/
        if (success)
        {
            /* Check for possibly not completely generating some output
               lines.
               --------------------------------------------------------*/
            for (count = output_buffer_ptr->output_window.start_line; 
                    count < output_buffer_ptr->output_window.end_line; count++)
            {
                if (output_buffer_ptr->sample_reached_ptr[count] !=
                        output_buffer_ptr->output_window.end_sample)
                {
                    c_lasmsg(FALSE, "resample-incomp",
                            "Resampling error in line %d, last sample reached"
                            " was %d out of %d samples.", count,
                            output_buffer_ptr->sample_reached_ptr[count],
                            output_buffer_ptr->output_window.end_sample);
                    status = E_FAIL;
                }
            }

            /* If buffering the whole image, do the write now.
               -----------------------------------------------*/
            if (output_buffer_ptr->full_buffer)
            {
                numlines = (long)output_buffer_ptr->output_window.num_lines;
                if (c_ewrite(&output_buffer_ptr->otfdsc,
                            (long *)&output_buffer_ptr->band, &one,
                            output_buffer_ptr->data_buffer_ptr, &numlines)
                        == E_FAIL)
                {
                    c_lasmsg(FALSE, "resample-write",
                            "Error writing to output image file.");
                    status = E_FAIL;
                }
            }
        }

#ifdef USE_WRITE_THREAD   
        if (!output_buffer_ptr->full_buffer)
        {
            /* Stop the write thread.  This is done by posting to the 
               data available semaphore and letting the write thread
               detect an empty queue - the signal for it to quit.
               ------------------------------------------------------*/
            if (POST_SEMAPHORE(&output_buffer_ptr->data_available_sem) != 0)
                status = E_FAIL;

#ifdef VERBOSE
            c_lasmsg(FALSE, "resample-info",
                    "Waiting for write thread to stop");
#endif

            /* Wait for the write thread to return before destroying the 
               synchronization semaphores and mutex's.
               ---------------------------------------------------------*/
            if (pthread_join(output_buffer_ptr->write_thread_id, &return_val) 
                    != 0)
            {
                c_lasmsg(FALSE, "resample-thread",
                        "Error waiting for thread termination.");
                status = E_FAIL;
            }

            /* Check if the write thread encountered an error.
               -----------------------------------------------*/
            if ((int)return_val != E_SUCC)
                status = E_FAIL;

#ifdef VERBOSE
            c_lasmsg(FALSE, "resample-info",
                    "There were %d full buffer writes out of %ld rows",
                    output_buffer_ptr->full_buffer_writes,
                    output_buffer_ptr->last_row_index);
#endif        

            if (DESTROY_SEMAPHORE(&output_buffer_ptr->data_available_sem)!= 0)
            {
                c_lasmsg(FALSE, "resample-thread",
                        "Error destroying semaphore.");
                status = E_FAIL;
            }

            /* Destroy the queue mutex.
               ------------------------*/
            if (DESTROY_MUTEX(&output_buffer_ptr->q_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Unable to destroy lock on shared data.");
                status = E_FAIL;
            }

            /* Free queue.
               -----------*/
            free(output_buffer_ptr->q_ptr);
        }
#endif        
        /* Destroy the file io and buffer mutex's.
           ---------------------------------------*/
        if (DESTROY_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
            status = E_FAIL;

        if (DESTROY_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            status = E_FAIL;

        free (output_buffer_ptr->data_buffer_ptr);
        free (output_buffer_ptr->sample_reached_ptr);
        free (output_buffer_ptr->sample_written_ptr);
        free (output_buffer_ptr->buffer_allocation_ptr);
        free (output_buffer_ptr);
    }

    return (status);
}


/*****************************************************************************
FUNCTION NAME:  get_output_image_buffer

PURPOSE:        
get_output_image_buffer returns a buffer to use for creating the resampled 
output.  The buffer is large enough to contain output_buffer_ptr->granularity 
lines of data.

RETURNS:
A pointer to the output image buffer state structure is returned.  If an error
occurs, an error message is generated and NULL is returned.

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

void *get_output_image_buffer
(
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr, /* I: output buffer pointer*/
    int row_index,  /* I: row index (calculated by line number/granularity) */
    int thread_num  /* I: thread number of calling thread                   */
)
{
    int index;   /* calculated index to the buffer                          */


    /* Get the buffer number from the allocation buffer.
       -------------------------------------------------*/
    index = output_buffer_ptr->buffer_allocation_ptr[row_index];

    /* If not full buffering and the buffer number is less than the number
       of threads, get the buffer reserved for this thread.
       -------------------------------------------------------------------*/
    if ((!output_buffer_ptr->full_buffer) &&
            (index < output_buffer_ptr->threads))
    {
        /* Lock the buffer manipulation mutex since the next buffer to
           allocate index and the buffer allocation are being modified and
           only one thread at a time can do so.
           ---------------------------------------------------------------*/
        if (LOCK_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            return(NULL);

        /* Get the index again in case it was changed by another processor.
           ----------------------------------------------------------------*/
        index = output_buffer_ptr->buffer_allocation_ptr[row_index];

        if (index < output_buffer_ptr->threads)
        {
            if (index < 0)
                if ((output_buffer_ptr->resample_dir * (row_index - 
                                output_buffer_ptr->next_buffer_to_alloc)) >= 0)
                    output_buffer_ptr->next_buffer_to_alloc = row_index + 
                        output_buffer_ptr->resample_dir;

            /* Release the buffer manipulation mutex.
               --------------------------------------*/
            if (UNLOCK_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error unlocking access to shared data.");
                return(NULL);
            }

            index = thread_num;

            /* Save the buffer index used for this row.
               ----------------------------------------*/
            output_buffer_ptr->buffer_allocation_ptr[row_index] = (int)index;

            /* Save the number of samples written.
               --------------------------------*/
            memcpy(&output_buffer_ptr->sample_written_ptr[row_index*
                    output_buffer_ptr->granularity],
                    &output_buffer_ptr->sample_reached_ptr[row_index*
                    output_buffer_ptr->granularity],
                    sizeof(int) * output_buffer_ptr->granularity);
        }
        else
        {
            /* Release the buffer manipulation mutex.
               --------------------------------------*/
            if (UNLOCK_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error unlocking access to shared data.");
                return(NULL);
            }
        }
    }

    /* Calculate the index in number of bytes.
       ---------------------------------------*/
    index = output_buffer_ptr->row_buffer_size * index;

    return (&((char *)output_buffer_ptr->data_buffer_ptr)[index]);
}


/*****************************************************************************
FUNCTION NAME:  return_output_image_buffer

PURPOSE:        
return_output_image_buffer accepts a buffer that was allocated by
get_output_image_buffer.  This routine determines whether the buffer should
be written to disk or kept in memory.

RETURNS:
Type = int
Value           Description
-----           -----------
E_FAIL           Error; cleanup not successful
E_SUCC           Successfully cleaned up the output image buffer

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

int return_output_image_buffer
(
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr, /* I: output buf. state ptr*/
    int row_index,  /* I: row index (calculated by line number/GRANULARITY) */
    int row_start,  /* I: starting line number of row                       */
    int row_end     /* I: ending line number of row                         */
)
{
    int buffer_index;        /* index to buffer                             */
    int count;               /* looping variable                            */
    int end_sample;          /* cached value of the end sample number       */
    int full = 1;            /* flag to indicate buffer is full (1=full)    */
#ifndef USE_WRITE_THREAD
    int index;               /* index into buffer                           */
    int next_buffer;         /* cached value of the next buffer index       */
    long numlines;           /* number of lines to write                    */
    long eline;              /* line number to write                        */
#endif
    int *sample_reached_ptr; /* cached pointer to the sample reached index  */


    /* If buffering the entire image, just return since no other action is
       required.
       -------------------------------------------------------------------*/
    if (output_buffer_ptr->full_buffer)
        return (E_SUCC);

    /* Get the buffer associated with the row index.
       ---------------------------------------------*/
    buffer_index = output_buffer_ptr->buffer_allocation_ptr[row_index];

    /* Check to see if the buffer is one reserved for a thread.
       --------------------------------------------------------*/
    if (buffer_index >= output_buffer_ptr->threads)
    {
        /* Not a buffer reserved for a thread, so see if it is completely
           resampled.
           --------------------------------------------------------------*/
        sample_reached_ptr = output_buffer_ptr->sample_reached_ptr;
        end_sample = output_buffer_ptr->output_window.end_sample;
        for (count = row_start; count < row_end; count++)
        {
            if (sample_reached_ptr[count] < end_sample)
            {
                full = 0;
                break;
            }
        }

        /* If buffer is full, write it out and adjust the buffers.
           -------------------------------------------------------*/
        if (full == 1)
        {
#ifdef USE_WRITE_THREAD
            WRITE_QUEUE_BUFFER_TYPE *temp_q_ptr; /* temp ptr to queue entry */
            int local_insert_index; /* local copy of the queue insert index */

            /* Put the buffer number in the buffer queue along with 
               any other data needed to perform a write.
               ----------------------------------------------------*/
            if (LOCK_MUTEX(&output_buffer_ptr->q_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error locking access to shared data.");
                return (E_FAIL);
            }

            /* Get a local copy of the insert index.
               -------------------------------------*/
            local_insert_index = output_buffer_ptr->q_insert_index;

            /* Insert the data into the queue.
               -------------------------------*/
            temp_q_ptr = &output_buffer_ptr->q_ptr[local_insert_index];
            temp_q_ptr->row_index = row_index;
            temp_q_ptr->row_start = row_start;
            temp_q_ptr->row_end = row_end;

            /* Update the local copy of the insert index.
               ------------------------------------------*/
            local_insert_index++;
            if (local_insert_index >= output_buffer_ptr->q_size)
                local_insert_index = 0;

            /* Update the real insert index.
               NOTE:  Only a single write to the real queue insert index
               should be performed after all the buffer data has been
               updated since the write thread doesn't lock the buffer 
               mutex and the write combining could break if the index
               is updated before the data is in the buffer.
               ---------------------------------------------------------*/
            output_buffer_ptr->q_insert_index = local_insert_index;

            /* Release the lock on the write queue mutex.
               ------------------------------------------*/
            if (UNLOCK_MUTEX(&output_buffer_ptr->q_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error unlocking access to shared data.");
                return (E_FAIL);
            }

            /* Signal the write thread that another buffer was added to the
               write queue.
               ------------------------------------------------------------*/
            if (POST_SEMAPHORE(&output_buffer_ptr->data_available_sem) != 0)
            {
                c_lasmsg(FALSE, "resample-thread",
                        "Error attempting to post to an invalid semaphore.");
                return (E_FAIL);
            }

#else
            /* Get the index to the current buffer.
               ------------------------------------*/
            index = buffer_index * output_buffer_ptr->row_buffer_size;

            /* Lock the file io mutex since only one thread at a time can 
               write to the file.
               ----------------------------------------------------------*/
            if (LOCK_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
                return (E_FAIL);

#ifdef VERBOSE
            output_buffer_ptr->full_buffer_writes++;
#endif

            /* Write the buffer.
               -----------------*/
            eline = (long)(row_start -
                    output_buffer_ptr->output_window.start_line) + 1;
            numlines = (long)(row_end - row_start);
            if (c_ewrite(&output_buffer_ptr->otfdsc,
                (long *)&output_buffer_ptr->band, &eline,
                &((unsigned char *)(output_buffer_ptr->data_buffer_ptr))[index],
                &numlines) == E_FAIL)
            {

                if (UNLOCK_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
                {
                    c_lasmsg(FALSE, "resample-mutex",
                            "Error unlocking access to shared data.");
                    return E_FAIL;
                }

                c_lasmsg(FALSE, "resample-write",
                        "Error writing to output image file.");
                return E_FAIL;
            }

            /* Release the file io mutex.
               --------------------------*/
            if (UNLOCK_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error unlocking access to shared data.");
                return E_FAIL;
            }

            /* Lock the buffer manipulation mutex.
               -----------------------------------*/
            if (LOCK_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error locking access to shared data.");
                return E_FAIL;
            }

            /* Update the buffer indices.
               --------------------------*/
            next_buffer = output_buffer_ptr->next_buffer_to_alloc;

            if ((next_buffer >= 0) &&
                    (next_buffer < output_buffer_ptr->last_row_index))
            {
                output_buffer_ptr->buffer_allocation_ptr[next_buffer]
                    = buffer_index;
                next_buffer += output_buffer_ptr->resample_dir;
                output_buffer_ptr->next_buffer_to_alloc = next_buffer;

                /* If interleaving, check if time to interleave.
                   ---------------------------------------------*/
                output_buffer_ptr->interleave_count++;
                if (output_buffer_ptr->interleave_count >= 
                        output_buffer_ptr->interleave)
                {
                    output_buffer_ptr->interleave_count = 1;

                    if ((next_buffer >= 0) &&
                            (next_buffer < output_buffer_ptr->last_row_index))
                    {
                        /* Flag that the next buffer should be thread
                           specific.
                           ------------------------------------------*/
                     output_buffer_ptr->buffer_allocation_ptr[next_buffer] = 0;

                        /* Update the next buffer pointer.
                           -------------------------------*/
                        next_buffer += output_buffer_ptr->resample_dir;
                        output_buffer_ptr->next_buffer_to_alloc = next_buffer;
                    }
                }
            }

            /* Release the buffer manipulation mutex.
               --------------------------------------*/
            if (UNLOCK_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error unlocking access to shared data.");
                return E_FAIL;
            }
#endif
        }
    }
    else
    {
        /*--------------------------------------------
          --------------------------------------------
          We should never have to enter this section,
          so let's just error out.
          --------------------------------------------
          ----------------------------------------------*/
        c_lasmsg(FALSE, "resample-incomp",
                "Partial buffer section.  Shouldn't be here.");
        return (E_FAIL);
    }

    return (E_SUCC);
}


#ifdef USE_WRITE_THREAD

/*****************************************************************************
FUNCTION NAME:  write_buffer_thread

PURPOSE:        
write_buffer_thread is an independent thread to write output image buffers to
the output file as the buffers are filled.  This allows the resampling threads
to continue running at full speed more often.

RETURNS:
Type = void *  (see note)
Value           Description
-----           -----------
E_FAIL           Error; cleanup not successful
E_SUCC           Successfully cleaned up the output image buffer

NOTES:
- Since a thread may only return a type of (void*), the return code is cast
  to a (void*) (which is typedef'ed to THREAD_RETURN_TYPE).  The routine that
  joins to the thread must cast the return value to an (int).
- This routine also includes a mechanism to combine multiple buffer writes
  into a single writes of a larger buffer when possible.  This reduces the 
  number of file I/O requests and helps throughput some.

ALGORITHM REFERENCES:
none

*****************************************************************************/

THREAD_RETURN_TYPE write_buffer_thread
(
    void *params_ptr
)
{
    WRITE_QUEUE_BUFFER_TYPE *temp_q_ptr;  /* ptr to the current queue entry */
    int buffer_index;                     /* index to buffer                */
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr = /* get the output buffer  */
        (OUTPUT_IMAGE_BUFFER_TYPE *)params_ptr;   /* ptr from the parameter */
    int index;                            /* index into buffer              */
    int next_buffer;                      /* cached value of the next buffer
                                             index                          */
    int row_index;                        /* buffer row index               */
    int row_start;                        /* starting line number           */
    int row_end;                          /* ending line number             */
    pthread_t thread_id = pthread_self(); /* thread id for this thread      */
    int policy;                           /* current scheduling policy      */
    struct sched_param schedule_param;    /* scheduling parameters          */
    int q_index;                          /* temporary queue remove index   */
    int no_combine_possible = 0;          /* count of times no write combining
                                             was possible since only one
                                             buffer in queue                */
    int combined_writes = 0;              /* number of times multiple buffer
                                             writes were condensed into a
                                             single write                   */
    long eline;                           /* line number to write           */
    long numlines;                        /* number of lines to write       */


    /* Bump the priority of the write thread so it can keep freeing buffers
       for use by the resampling threads.
       --------------------------------------------------------------------*/
    if (pthread_getschedparam(thread_id, &policy, &schedule_param) !=0)
        return ((THREAD_RETURN_TYPE)E_FAIL);

    /* Set the maximum priority to the maximum for the current scheduling 
       policy.
       ------------------------------------------------------------------*/
    schedule_param.sched_priority = sched_get_priority_max(policy);

    /* Update the priority for this thread.
       ------------------------------------*/
    if (pthread_setschedparam(thread_id, policy, &schedule_param) != 0)
        return ((THREAD_RETURN_TYPE)E_FAIL);

    /* Loop until the conditions for breaking out of the loop are detected.
       --------------------------------------------------------------------*/
    while (1)
    {
        /* Block until something is put into the write queue.
           --------------------------------------------------*/
        if (WAIT_SEMAPHORE(&output_buffer_ptr->data_available_sem) != 0)
            return ((THREAD_RETURN_TYPE)E_FAIL);

        /* If the queue is empty, it is the signal to terminate the thread.
           ----------------------------------------------------------------*/
        if (output_buffer_ptr->q_remove_index ==
                output_buffer_ptr->q_insert_index)
            break;

        /* Get a local copy of the remove index.
           -------------------------------------*/
        q_index = output_buffer_ptr->q_remove_index;

        /* Get the buffer entry.
           ---------------------*/
        temp_q_ptr = &output_buffer_ptr->q_ptr[q_index];
        row_index = temp_q_ptr->row_index;
        row_start = temp_q_ptr->row_start;
        row_end = temp_q_ptr->row_end;

        /* Adjust the local remove index to the next queue location.
           ---------------------------------------------------------*/
        q_index++;
        if (q_index >= output_buffer_ptr->q_size)
            q_index = 0;

        /* Get the buffer associated with the row index.
           ---------------------------------------------*/
        buffer_index = output_buffer_ptr->buffer_allocation_ptr[row_index];

        /* Track when write combining is not possible.
           ----------------------------------------------*/
        if (q_index == output_buffer_ptr->q_insert_index)
            no_combine_possible++;

        /* Check which direction the output is being generated in so the
           write combining can be done properly.
           -------------------------------------------------------------*/
        if (output_buffer_ptr->resample_dir > 0)
        {
            /* Resampling from top to bottom, so set the index to the
               current buffer since the combining will only look for
               following buffers, not preceding buffers.
               ------------------------------------------------------*/
            index = buffer_index * output_buffer_ptr->row_buffer_size;

            /* As long as there is more data in the queue and it is for the 
               next set of lines and it's the next buffer, combine the
               writes.
               ---------------------------------------------------------*/
            while ((q_index != output_buffer_ptr->q_insert_index) &&
                    (output_buffer_ptr->q_ptr[q_index].row_index ==
                     (row_index+1)) &&
                    (output_buffer_ptr->buffer_allocation_ptr[row_index+1] == 
                     (buffer_index + 1)))
            {
                /* Wait on the semaphore so it doesn't get out of sync with
                   the data in the buffer.
                   --------------------------------------------------------*/
                if (WAIT_SEMAPHORE(&output_buffer_ptr->data_available_sem)
                        != 0)
                    return ((THREAD_RETURN_TYPE)E_FAIL);
#ifdef VERBOSE
                c_lasmsg(FALSE, "resample-info", "Trying write combining");
#endif
                /* Set the temporary queue pointer to the next queue entry.
                   --------------------------------------------------------*/
                temp_q_ptr = &output_buffer_ptr->q_ptr[q_index];

                /* Adjust the buffer index to the next one.
                   ----------------------------------------*/
                buffer_index++;

                /* Adjust the local queue remove index.
                   ------------------------------------*/
                q_index++;
                if (q_index >= output_buffer_ptr->q_size)
                    q_index = 0;

                /* Get the new row index and ending row (or line) from the
                   queue.
                   -------------------------------------------------------*/
                row_index = temp_q_ptr->row_index;
                row_end = temp_q_ptr->row_end;

                /* Count the combined writes.
                   --------------------------*/
                combined_writes++;
            }
        }
        else
        {
            /* Check for possible write combining from bottom to top.
               ------------------------------------------------------*/
            /* As long as there is more data in the queue and it is for the
               previous set of lines and it's in the previous buffer,
               combine the writes.
               ------------------------------------------------------------*/
            while ((q_index != output_buffer_ptr->q_insert_index) &&
                    (output_buffer_ptr->q_ptr[q_index].row_index ==
                     (row_index-1)) &&
                    (output_buffer_ptr->buffer_allocation_ptr[row_index-1] == 
                     (buffer_index - 1)))
            {
                /* Wait on the semaphore so it doesn't get out of sync with
                   the data in the buffer.
                   --------------------------------------------------------*/
                if (WAIT_SEMAPHORE(&output_buffer_ptr->data_available_sem)
                        != 0)
                    return ((THREAD_RETURN_TYPE)E_FAIL);
#ifdef VERBOSE
                c_lasmsg(FALSE, "resample-info", "Trying write combining");
#endif
                /* Set the temporary queue pointer to the next queue entry.
                   --------------------------------------------------------*/
                temp_q_ptr = &output_buffer_ptr->q_ptr[q_index];

                /* Adjust the buffer index to the preceding buffer.
                   ------------------------------------------------*/
                buffer_index--;

                /* Adjust the local queue remove index.
                   ------------------------------------*/
                q_index++;
                if (q_index >= output_buffer_ptr->q_size)
                    q_index = 0;

                /* Get the new row index and the starting row (or line) from 
                   the queue.
                   ---------------------------------------------------------*/
                row_index = temp_q_ptr->row_index;
                row_start = temp_q_ptr->row_start;

                /* Count the combined writes.
                   --------------------------*/
                combined_writes++;
            }

            /* Since we're resampling from bottom to top, calculate the
               starting index after the writes have been combined.
               --------------------------------------------------------*/
            index = buffer_index * output_buffer_ptr->row_buffer_size;
        }

        /* Enter a critical section since only one thread at a time can 
           write to the file and adjust the buffer indices.
           ------------------------------------------------------------*/
        if (LOCK_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
            return ((THREAD_RETURN_TYPE)E_FAIL);

#ifdef VERBOSE
        output_buffer_ptr->full_buffer_writes++;
#endif

        /* Write the buffer.
           -----------------*/
        eline = (long)(row_start - output_buffer_ptr->output_window.start_line
                + 1);
        numlines = (long)(row_end - row_start);
        if (c_ewrite(&output_buffer_ptr->otfdsc,
                (long *)&output_buffer_ptr->band, &eline,
                &((unsigned char *)(output_buffer_ptr->data_buffer_ptr))[index],
                &numlines) == E_FAIL)
        {
            if (UNLOCK_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
            {
                c_lasmsg(FALSE, "resample-mutex",
                        "Error unlocking access to shared data.");
                return (THREAD_RETURN_TYPE)E_FAIL;
            }
            c_lasmsg(FALSE, "resample-write",
                    "Error writing to output image file.");
            return (THREAD_RETURN_TYPE)E_FAIL;
        }

        /* Release the file io mutex.
           --------------------------*/
        if (UNLOCK_MUTEX(&output_buffer_ptr->file_io_mutex) != 0)
            return ((THREAD_RETURN_TYPE)E_FAIL);

        /* Lock the buffer manipulation mutex.
           -----------------------------------*/
        if (LOCK_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            return ((THREAD_RETURN_TYPE)E_FAIL);

        /* Return the buffers written for use by the resampling threads.
           -------------------------------------------------------------*/
        while (output_buffer_ptr->q_remove_index != q_index)
        {
            /* Get the row index from the write queue.
               ---------------------------------------*/
            row_index = output_buffer_ptr->q_ptr[output_buffer_ptr->
                q_remove_index].row_index;

            /* Convert the row index into the applicable buffer index.
               -------------------------------------------------------*/
            buffer_index =
                output_buffer_ptr->buffer_allocation_ptr[row_index];

            /* Adjust the queue remove index.
               ------------------------------*/
            output_buffer_ptr->q_remove_index++;
            if (output_buffer_ptr->q_remove_index >=
                    output_buffer_ptr->q_size)
                output_buffer_ptr->q_remove_index = 0;

            /* Update the buffer indices.
               --------------------------*/
            next_buffer = output_buffer_ptr->next_buffer_to_alloc;
            if ((next_buffer >= 0) &&
                    (next_buffer < output_buffer_ptr->last_row_index))
            {
                output_buffer_ptr->buffer_allocation_ptr[next_buffer]
                    = buffer_index;
                next_buffer += output_buffer_ptr->resample_dir;
                output_buffer_ptr->next_buffer_to_alloc = next_buffer;

                /* If interleaving, check if time to interleave.
                   ---------------------------------------------*/
                output_buffer_ptr->interleave_count++;
                if (output_buffer_ptr->interleave_count >= 
                        output_buffer_ptr->interleave)
                {
                    output_buffer_ptr->interleave_count = 1;

                    if ((next_buffer >= 0) &&
                            (next_buffer < output_buffer_ptr->last_row_index))
                    {
                        /* Flag that the next buffer should be thread
                           specific.
                           ------------------------------------------*/
                        output_buffer_ptr->buffer_allocation_ptr[next_buffer]
                            = 0;

                        /* Update the next buffer pointer.
                           -------------------------------*/
                        next_buffer += output_buffer_ptr->resample_dir;
                        output_buffer_ptr->next_buffer_to_alloc = next_buffer;
                    }
                }
            }
        }

        /* Release the buffer manipulation mutex.
           --------------------------------------*/
        if (UNLOCK_MUTEX(&output_buffer_ptr->buffer_mutex) != 0)
            return ((THREAD_RETURN_TYPE)E_FAIL);
    }

#ifdef VERBOSE
    c_lasmsg(FALSE, "resample-info",
            "%d buffers combined for writes", combined_writes);
    c_lasmsg(FALSE, "resample-info",
            "No combination possible %d times", no_combine_possible);
#endif

    return ((THREAD_RETURN_TYPE)E_SUCC);
}
#endif
