/*****************************************************************************
NAME:           read_input_image

PURPOSE:
The read_input_image module provides the routines needed to open, read, and
close an image for input.  The data read is converted to float type for use
by the resampler.

ROUTINES:
    open_input_image
    close_input_image
    read_input_line
    
DEVELOPMENT HISTORY:
  Baselined from revision 1.6 of read_1r_input.c


NOTES:
This module assumes the input file is LAS format.

ALGORITHM REFERENCES:
none

*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "resample.h"
#include "debug.h"                     /* ASSERT macro definition          */


/*****************************************************************************
NAME:           open_input_image

PURPOSE:        
The open_input_image routine opens the HDF file to access the indicated
band number.

RETURN VALUE:
Type = IN_IMAGE_TYPE pointer
Value           Description
-----           -----------
NULL            Error; image not opened
non-NULL        Successfully opened image, file state data returned
The lines and samples in the source image are also returned.

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

IN_IMAGE_TYPE *open_input_image
(
    const char *filename,     /* I: file name of image to open              */
    int band,                 /* I: band of image to open                   */
    int  *in_nl_ptr,          /* O: number of lines in image                */
    int  *in_ns_ptr           /* O: number of samples in image              */
)
{
    int in_size;              /* bytes in one unit of input data            */
    struct FDESC *infdsc;     /* input image file descriptor                */
    long acc;                 /* newlas imageio access code                 */
    long bands_to_read[MAXBND]; /* number of bands to read                  */
    long idtype;              /* data type                                  */
    long nl[MAXBND];          /* number of lines in each band               */
    long ns[MAXBND];          /* number of samples in each band             */
    long one = 1;             /* The number one.                            */
    long opt[2];              /* option flags                               */
    long sl, ss;              /* window info: start line, start sample      */
    IN_IMAGE_TYPE *image_ptr; /* input image data                           */
    const WINDOW_TYPE *window_ptr; /* pointer to input window geometry      */


    /* Verify input parameters are valid.
       ----------------------------------*/
    ASSERT((band > 0) && (band <= MAXBND));

    /* Allocate space for the image state data.
       ----------------------------------------*/
    image_ptr = (IN_IMAGE_TYPE*)malloc(sizeof(IN_IMAGE_TYPE));
    if (image_ptr == NULL)
        return (NULL);

    /* Initialize image state data fields.
       -----------------------------------*/
    image_ptr->fdesc = 0;
    image_ptr->buf_ptr = NULL;
    image_ptr->band = band;

    /* Set the band number to read.
       ----------------------------*/
    bands_to_read[0] = band;
    bands_to_read[1] = 0;

    /* Get the input window, and open input image band.
       ------------------------------------------------*/
    if (do_input_window())
    {
        window_ptr = get_input_window();
        sl = window_ptr->start_line + 1;
        ss = window_ptr->start_sample + 1;
        nl[0] = sl + window_ptr->num_lines - 1;
        ns[0] = ss + window_ptr->num_samples - 1;
    }
    else
    {
        sl = 1;
        ss = 1;
        nl[0] = 0;
        ns[0] = 0;
    }
    /* Reset the starting line and sample to 1 since resample takes care of
       reading the correct input window from the buffer of data retrieved from
       the image. Found input windowing was being duplicated here and in the
       imageio routines causing input windowing to start twice as far into the
       image as the user requested. The fix is to tell c_eopenr we're not
       doing any offset into the image from the upper-left corner and let
       resample do it since it's already implemented here. (CR 6463)
       ----------------------------------------------------------------------*/
    ss = 1;
    sl = 1;
    idtype = EREAL;
    acc = IREAD;
    opt[0] = 0;
    if (c_eopenr(&infdsc,(char *)filename,bands_to_read,&sl,&ss,&nl[0],&ns[0],
                &acc,&idtype,opt,&one) != E_SUCC)
    {
        free(image_ptr);
        c_lasmsg(FALSE, "resample-open", "Error opening input image %s.",
                (char *)filename); 
        return NULL;
    }
    image_ptr->fdesc = infdsc;

    /* Get the input lines and samples.
       --------------------------------*/
    *in_nl_ptr = (int)nl[0];
    *in_ns_ptr = (int)ns[0];
    image_ptr->samples_per_line = (int)ns[0];

    /* Get the input data type.
       ------------------------*/
    image_ptr->in_dtype = FLOAT_IN;
    in_size = EREALSIZE;

    /* Allocate memory for an intermediate buffer to allow the each line of 
       the input data type to be converted to float for the resampler.
       The allocation is done here to prevent the memory allocation
       from taking place for each set of lines read.
       --------------------------------------------------------------------*/
    image_ptr->buf_ptr = malloc (in_size * *in_ns_ptr);
    if (image_ptr->buf_ptr == NULL)
    {
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        close_input_image(image_ptr);
        return (NULL);
    }

    return (image_ptr);
}


/*****************************************************************************
NAME:           close_input_image

PURPOSE:        
The close_input_image routine closes the input file and frees any memory 
allocated.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC           Successfully closed image
E_FAIL           Error; image not closed

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

int close_input_image
(
    IN_IMAGE_TYPE *in_image_ptr  /* I/O: pointer to image to close */
)
{ 
    /* Free buffer memory.
       -------------------*/
    free(in_image_ptr->buf_ptr);
    in_image_ptr->buf_ptr = NULL;

    if (c_eclose(&in_image_ptr->fdesc) != E_SUCC)
    {
        free(in_image_ptr);
        c_lasmsg(FALSE, "resample-close", "Error closing input image.");
        return E_FAIL;
    }

    /* Free image buffer memory.
       -------------------------*/
    free(in_image_ptr);

    return E_SUCC;
}


/*****************************************************************************
NAME:           read_input_lines

PURPOSE:
  The read_input_lines routine reads lines from the input image file
  and returns the data to the calling routine.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC           Successfully read scan
E_FAIL           Error; scan not read

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

int read_input_lines
(
    IN_IMAGE_TYPE *in_image_ptr,           /* I/O: input file state pointer */
    int   start_line,                      /* I: current start line number  */
    int   num_lines,                       /* I: number of lines to read    */
    float *buffer                          /* O: data buffer                */
)
{
    int index;                         /* sample loop counter               */
    int count;                         /* loop counter                      */
    int samps = in_image_ptr->samples_per_line; /* cache the sample per line*/
    int data_index;                    /* index to data buffer              */
    long lineno = (long)start_line;    /* line number to read               */
    long one = 1;                      /* the number one                    */

    float *buf_ptr;                    /* pointer to array of float data    */


    lineno++;                /* Increment for zero-based input line number. */

    /* Read each line, converting it to float.
       ---------------------------------------*/
    for (count = 0; count < num_lines; count++)
    {
        data_index = count * samps;

        /* Read the input image line into the intermediate buffer.
           -------------------------------------------------------*/
        if (c_eread(&in_image_ptr->fdesc, (long *)&in_image_ptr->band,
                    &lineno, (unsigned char *)in_image_ptr->buf_ptr, &one)
                == 0)
        {
            c_lasmsg(FALSE, "resample-read",
                    "Error occurred while reading input image.");
            return E_FAIL;
        }
        lineno++;

        buf_ptr = (float *)in_image_ptr->buf_ptr;

        /* verify the data type is correct in case a type is added */
        ASSERT(in_image_ptr->in_dtype == FLOAT_IN);

        /* Save data to the real image buffer.
           -----------------------------------*/
        for (index = 0; index < samps; index++)
            buffer[data_index+index] = buf_ptr[index];

    }

    return (E_SUCC);
}
