/*****************************************************************************
NAME:           resample

PURPOSE:
The resample module contains the "main" function for the resampler.

DEVELOPMENT HISTORY:
  Baselined from revision 1.46 of the IAS resample.c

NOTES:
  Note that line/sample references are all zero-based rather than one-based.

ALGORITHM REFERENCES:
  None

*****************************************************************************/

#include <stdlib.h>                   /* exit prototype                    */
#include <stdio.h>                    /* sprintf prototype                 */
#include "resample.h"
#include "util.h"


int main() 
{
    struct DDR iddr;                  /*  input ddr                         */
    gxx_GEOM_GRID_TYPE grid;          /* image grid                         */
    int grid_io_band_list[MAXBND];    /* list of bands for grid i/o
                                         operations                         */
    RESAMPLE_TYPE resampling_type;    /* current resampling type            */
    const WINDOW_TYPE *input_window_ptr;/* ptr to input window geometry     */
    const WINDOW_TYPE *output_window_ptr;/* ptr to output window geometry   */
    OUTPUT_DATA_TYPE odtype;          /* selected output type               */
    struct PARBLK vblk;               /* TAE parameter block                */
    struct FDESC *otfdsc;             /* output image file descriptor       */

    int all_bands[MAXBND];            /* Array with all bands to pass to
                                         get_geom_grid_headers              */
    int iband;                        /* indexed band number                */
    int iimg;                         /* indexed image number               */
    int buffer_full_output_image;     /* flag to buffer entire output image */
    int gridflg;                  /* Grid flag: 0 = no grid, use proj. params.
                                     1 = use grid                */
    int input_image_scans_to_buffer;  /* the number of scans to buffer from
                                         the input image during resampling  */ 
    int max_output_image_buffer;      /* the maximum size of the output image
                                         buffer, in MB                      */
    int max_processors_to_use;        /* maximum number of processors to use
                                         for a single gps executable        */
    int num_grid_io_bands = 0;        /* number of bands to operate on for 
                                         grid i/o                           */
    int status = E_SUCC;              /* Return status                      */
    int temp;                         /* temp variable                      */

    long acc;                         /* newlas imageio access code         */
    long bands[MAXIMG][MAXBND+1];     /* band array                         */
    long dtype;                       /* data type                          */
    long eltime;                      /* message timer                      */
    long i;                           /* Loop control variable              */
    long img_bands[MAXIMG];           /* Num. of bands for each image       */
    long las_window[4];               /* LAS output window (SL, SS, NL, NS) */
    long maxlines = LARGE_GRANULARITY;/* maximum number of lines written    */
    int nbands;                       /* number of bands                    */
    long nmrimg;                      /* Number of input images             */
    long one = 1;                     /* The number one.                    */
    long opt[2];                      /* option flags                       */
    long std_out;                     /* standard output logical unit number*/
    long current_band;                /* Keeps track of the current band  
                                         being processed                    */
    long wind_mode;                   /* Window mode (IN, OUT, NOWIND)      */

    double minpix[MAXBND], maxpix[MAXBND];/* Array of min/max pixel values for
                                             all bands                      */

    char history_flag[4];             /* history flag                       */
    char inhost[MAXIMG][CMLEN];       /* host name of input image file      */
    char minmax_flag[8];              /* min/max pixel calculation flag     */
    char outtae[1][CMLEN];            /* output image file name (TAE)       */
    char outhost[CMLEN];              /* output image file name (host)      */


    /* Initialize TAE processing.
       --------------------------*/
    c_inlas(&vblk,&std_out);

    /* Read the resampler control parameters.
       --------------------------------------*/
    if (getpar(&vblk, outtae, &max_processors_to_use,
                &max_output_image_buffer, &input_image_scans_to_buffer,
                &buffer_full_output_image, &gridflg, bands, &nbands,
                &nmrimg, img_bands, &wind_mode, &eltime, history_flag, 
                minmax_flag,inhost) != E_SUCC)
    {
        c_lasmsg(TRUE, "resample-getpar", "Error reading parameters.");
    }

    /* Retrieve the output image filename.
       -----------------------------------*/
    strcpy(outhost, get_output_image_filename());

    odtype = get_output_data_type(); 

    /* Get the resampling type to use and the list of bands to 
       perform the resampling on.
       -------------------------------------------------------*/
    resampling_type = get_resampling_type();

    if (gridflg)
    {
        /* Build an identity array with all the bands to pass to
           get_geom_grid_headers 
           -----------------------------------------------------*/
        i = 0;
        for(iimg = 0 ; iimg < nmrimg ; iimg++)
        {
            for(iband = 0 ; iband < img_bands[iimg] ; iband++)
            {
                all_bands[i] = bands[iimg][iband];
                i++;
            }
        }

        /* Read the grid header information (includes all the information
           in the grid, except for the data which is pointed to by
           pointers).
           --------------------------------------------------------------*/
        if (get_geom_grid_headers(get_grid_filename(), all_bands,
                    nbands, &grid) == E_FAIL)
        {
            c_lasmsg(TRUE, "resample-grid",
                    "Error reading geometric mapping grid file.");
        }


        /* Verify the grid has data for all the bands requested by the user.
           -----------------------------------------------------------------*/
        for(iimg = 0 ; iimg < nmrimg ; iimg++) 
        {
            for (iband = 0; iband < img_bands[iimg]; iband++)
            {
                /* Verify the requested band is in the grid.
                   -----------------------------------------*/
                for (temp = 0; temp < grid.nbands; temp++)
                {
                    if (bands[iimg][iband] == grid.bandlist[temp])
                        break;
                }

                /* If the loop exits with temp equal to grid.nbands, the
                   band is not present in the grid.
                   -----------------------------------------------------*/
                if (temp == grid.nbands)
                {
                    free (grid.bandlist);
                    free (grid.gridbands);
                    c_lasmsg(TRUE, "resample-grid",
                            "Band %d is not present in the grid.", temp);
                }
            }
        }

        if (!do_output_window())
        {
            /* The user did not request output windowing.  Set the output
               window to the output image size from the grid.
               ----------------------------------------------------------*/
            for (iband = 0; iband < grid.nbands; iband++)
                set_output_window(iband, 0, 0,
                        (long)grid.gridbands[iband].output_image_lines,
                        (long)grid.gridbands[iband].output_image_samps);
        }
        else
        {
            /* Output windowing requested.  Scale the output window relative
               to the reference window passed in for the first band in the
               band_list.
               -------------------------------------------------------------*/
            for (iband = 0; iband < grid.nbands; iband++)
            {
                /* Scale the window relative to the reference band 
                   (the first one in the band list).
                   -----------------------------------------------*/
                set_scaled_output_window(iband,
                        grid.gridbands[0].pixsize_line/
                        grid.gridbands[iband].pixsize_line,
                        grid.gridbands[0].pixsize_samp/
                        grid.gridbands[iband].pixsize_samp);

                /* Make sure the window is contained in the output grid.
                   If it is not, generate an error message and exit.
                   -----------------------------------------------------*/
                output_window_ptr = get_output_window(iband);
                if ((output_window_ptr->start_line < 0) ||
                        (output_window_ptr->start_sample < 0) ||
                        (output_window_ptr->end_line-1 >= 
                         grid.gridbands[iband].output_image_lines) ||
                        (output_window_ptr->end_sample-1 >=
                         grid.gridbands[iband].output_image_samps))
                {
                    free (grid.bandlist);
                    free (grid.gridbands);
                    c_lasmsg(TRUE, "resample-outwind",
                            "Output window falls outside mapping grid.");
                }
            }
        }
    }
    else         /* no grid */
    {
        /* Copy first band window info to other bands.
           -------------------------------------------*/
        output_window_ptr = get_output_window(0);
        for (iband = 1; iband < nbands; iband++)
            set_output_window(iband, output_window_ptr->start_line,
                    output_window_ptr->start_sample,
                    output_window_ptr->num_lines,
                    output_window_ptr->num_samples);
    }


    /* Assemble the data needed to open the output image file.
       -------------------------------------------------------*/
    output_window_ptr = get_output_window(0);

    /* Open the output file.
       ---------------------*/
    acc = IWRITE;
    las_window[SL] = (long)1;
    las_window[SS] = (long)1;
    las_window[NL] = (long)output_window_ptr->num_lines;
    las_window[NS] = (long)output_window_ptr->num_samples;
    if (odtype == BYTE_OUTPUT)
        dtype = EBYTE;
    else if (odtype == INT16_OUTPUT)
        dtype = EWORD;
    else if (odtype == INT32_OUTPUT)
        dtype = ELONG;
    else
        dtype = EREAL;
    opt[0] = dtype;
    opt[1] = 0;
    if (buffer_full_output_image)
        maxlines = output_window_ptr->num_lines; /* write entire image    */
    if (c_eopenr(&otfdsc, outhost, (long *)&nbands,
                &las_window[SL], &las_window[SS], &las_window[NL],
                &las_window[NS], &acc, &dtype, opt, &maxlines) != E_SUCC)
    {
        if (gridflg)
        {
            free (grid.bandlist);
            free (grid.gridbands);
        }
        c_lasmsg(TRUE, "resample-create", "Error creating output image %s.",
                outhost); 
    }

    /* Setup the resampler for the output type and window specified in the
       input parameters.
       -------------------------------------------------------------------*/
    setup_resampf1p_output();

    /* If not nearest neighbor resampling, set up the resampling kernel.
       -----------------------------------------------------------------*/
    if (resampling_type != NN_RESAMPLING)
    { 
        if (Kernel_Setup(resampling_type, get_cubic_convolution_alpha())
                != E_SUCC)
        {            
            if (gridflg)
            {
                free (grid.bandlist);
                free (grid.gridbands);
            }
            c_lasmsg(TRUE, "resample-kern",
                    "Error setting up the resampling kernel.");
        }

    } /* not nearest neighbor */ 

    /* Process all the requested bands.
       --------------------------------*/
    current_band = 0;
    for(iimg = 0 ; iimg < nmrimg ; iimg++)
    {
        set_input_image_filename(iimg);
        if( nmrimg > 1)
        {
            c_lasmsg(FALSE, "resample-info",
                    "Resampling image %s",get_input_image_filename());
        }

        for (iband = 0; iband < img_bands[iimg]; iband++)
        {
            c_lasmsg(FALSE, "resample-info",
                    "Resampling band %d", bands[iimg][iband]);

            if (gridflg)
            {
                /* Read the grid data that pertains to this band.  Note that
                   get_geom_grid_data has been modified to only read the data
                   into the first band and set the pointers in the following
                   bands to point to the data in the first band.  This is to
                   improve the efficiency since LAS presently only supports
                   one grid for all bands.
                   ----------------------------------------------------------*/
                grid_io_band_list[0] = current_band;
                num_grid_io_bands = 1;
                status = get_geom_grid_data(grid_io_band_list,
                        num_grid_io_bands, &grid);
                if (status != E_SUCC)
                {
                    c_lasmsg(FALSE, "resample-grid",
                            "Error reading geometric mapping grid file.");
                    break;
                }
            }

            /* Initialize the min/max Z values.
               --------------------------------*/
            minpix[iband] = maxpix[iband] = get_fill_pixel();

            status =warp(&grid, gridflg, current_band, bands[iimg][iband], 
                    resampling_type, otfdsc, &input_image_scans_to_buffer,
                    max_processors_to_use, &max_output_image_buffer,
                    buffer_full_output_image, &minpix[iband], &maxpix[iband]);
            if (status == E_FAIL)
            {
                if (gridflg)
                    /* Free the large data arrays from the grid for this band.
                       -------------------------------------------------------*/
                    free_geom_grid_data(grid_io_band_list,
                            num_grid_io_bands, &grid);
                break;
            }

            if (gridflg && iband == nbands - 1)
            {            
                /* Free the large data arrays from the grid for this band.
                   Note that get_geom_grid_data has been modified to only
                   read the data into the first band and set the pointers
                   in the following bands to point to the data in the
                   first band.  Therefore we should only release the
                   data when we have completed the last loop.
                   LAS presently only supports one grid for all bands.
                   -------------------------------------------------------*/
                free_geom_grid_data(grid_io_band_list,
                        num_grid_io_bands, &grid);
            }

            current_band++;
        } /* for (iband = 0; iband < nbands; iband++) */
    } /*( for(iimg = 0; iimg < nmrimg ; iimg++) */ 


    /* Clean up.
       ---------*/
    Cleanup_Kernel();
    if (do_terrain_correction())
        cleanup_terrain();

    /* Free the grid arrays.
       ---------------------*/
    if (gridflg)
    {
        free (grid.bandlist);
        free (grid.gridbands);
    }

    /* Close the output file.
       ----------------------*/
    c_eclose(&otfdsc);

    /* Exit if resampling failed.
       --------------------------*/
    if (status == E_FAIL)
    {
        c_lasmsg(FALSE, "resample-fatal", "Fatal error encountered.");
        c_clean(&vblk, "resample", &outhost, &one);
    }

    /* Write history information.
       --------------------------*/
    if (c_hstout(&vblk, outhost, inhost, &nmrimg, " ") != E_SUCC)
        c_lasmsg(FALSE,"resample-warn","Nonfatal error encountered");

    /* Update DDR information.
       -----------------------*/
    if (c_getddr(inhost[0], &iddr) != E_SUCC)
        c_lasmsg(FALSE,"resample-warn","Nonfatal error encountered");
    if (gridflg)
    {
        if (wind_mode == OUTWIND)
        {
            las_window[SL] = (long)output_window_ptr->start_line + 1;
            las_window[SS] = (long)output_window_ptr->start_sample + 1;
        }
        else if (wind_mode == INWIND)
        {
            input_window_ptr = get_input_window();
            las_window[SL] = (long)input_window_ptr->start_line;
            las_window[SS] = (long)input_window_ptr->start_sample;
            las_window[NL] = (long)input_window_ptr->num_lines;
            las_window[NS] = (long)input_window_ptr->num_samples;
        }
        if (g_upddr(inhost, outhost, las_window,
                bands, (long)nbands, (struct GEOGRID *)get_las_grid(),
                wind_mode, &iddr) != E_SUCC)
            c_lasmsg(FALSE,"resample-warn","Nonfatal error encountered");
    }
    else
    {
        ng_upddr(inhost, outhost, bands, (long)nbands);
    }

    /* Update min/max info in the DDR
       ------------------------------*/
    if (c_svmnmx(outhost, (long *)&nbands, minpix, maxpix, minmax_flag) !=
            E_SUCC)
        c_lasmsg(FALSE,"resample-warn","Nonfatal error encountered");

    /* Write completion message.
       -------------------------*/
    c_complt("RESAMPLE", outtae, &one);

    return SUCCESS;
}
