#ifndef _RESAMPLE_H_
#define _RESAMPLE_H_

#include "las.h"
#include "geom_grid.h"      /* gxx_GEOM_GRID_BAND_TYPE definition           */
#include "multiproc.h"      /* THREAD_RETURN_TYPE definition                */
#include "geompak.h"        /* GEOGRID structure definition                 */

/*------
--------
 Macros
--------
--------*/
#define INPUT_XY      104         /* input coordinates in x,y (linear)      */
#define OUTPUT_XY     105         /* output coordinates in x,y (linear)     */
#define OUTPUT_LS     106         /* output coor. in line,sample (pixels)   */
#define INPUT         107         /* input coor. in angular units           */
#define LARGE_GRANULARITY 24      /* define the number of lines in a block  */
#define SMALL_GRANULARITY 12      /* for large and small granularity        */
#define NOWIND -1                 /* no windows entered                     */
#define INWIND 0                  /* input window entered                   */
#define OUTWIND 1                 /* output window entered                  */
#define DEFAULT_LINE_OVERLAP 5
                         /* Defines the number of lines to use for overlap
                            when reading input lines into the buffer.  When
                            the next group of lines is read, this is the
                            number of lines that will overlap.              */
#define GMGLIN  32      /* Mapping grid invalid in line direction           */
#define GMGSAM  33      /* Mapping grid invalid in sample direction         */

/* define the MIN and MAX macros if not already defined */
#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif
#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b))
#endif

/*---------------
-----------------
 Type definitions
-----------------
-----------------*/

typedef enum frame_type
{
    unknown_type = -1,
    GEOBOX=1,
    PROJBOX,
    UL_SIZE,
    MINBOX,
    PATH_ORIENTED
} FrameType;


typedef enum
{
    BYTE_IN,
    INT16_IN,
    INT32_IN,
    FLOAT_IN
} INPUT_DATA_TYPE; /* legal input data types */

typedef struct
{
    struct FDESC *fdesc;      /* file descriptor                            */
    INPUT_DATA_TYPE in_dtype; /* type of data stored in image               */
    int band;                 /* band being read                            */
    int samples_per_line;     /* number of samples per line of input        */
    void *buf_ptr;            /* pointer to the buffer for storing the raw
                                 data read from the HDF file.  The storage is
                                 allocated when the file is opened to prevent
                                 the allocation from taking place for each
                                 data set read.                             */
} IN_IMAGE_TYPE; /* This structure contains the state information for reading
                    an input file.                                          */

typedef enum
{
    BYTE_OUTPUT,
    INT16_OUTPUT,
    INT32_OUTPUT,
    FLOAT_OUTPUT,
    NUMBER_OUTPUT_TYPES
} OUTPUT_DATA_TYPE;

typedef struct
{
    int start_line;
    int start_sample;
    int end_line;
    int end_sample;
    int num_lines;
    int num_samples;
} WINDOW_TYPE; /* Holds the parameters that define an image window.  Both the
                  end line/sample and the number of lines/samples are
                  included to allow the user to determine which method of
                  defining the window is more convenient.                   */

#ifdef USE_WRITE_THREAD
typedef struct
{
    int row_index;      /* row index to write              */
    int row_start;      /* starting row (or line) to write */
    int row_end;        /* ending row (or line) to write   */
    
} WRITE_QUEUE_BUFFER_TYPE;
/* queue definition for the write queue */
#endif

/* NOTE: For those not familiar with the "volatile" keyword, the volatile
   keyword is a standard C keyword that tells the compiler the variable may
   change without its knowledge.  This essentially disables some
   optimizations on that variable that are not safe in a multiprocessing
   environment when multiple threads may be changing a variable.
---------------------------------------------------------------------------*/
typedef struct
{
    struct FDESC *otfdsc;       /* output file descriptor                   */
    int band;                   /* band number for this output buffer       */
    int threads;                /* number of threads buffer can accomodate  */
    void *data_buffer_ptr;      /* buffer for resampled data                */
    int data_size;              /* number of bytes per sample in buffer     */
    int bytes_per_line;         /* number of bytes per line in buffer       */
    int row_buffer_size;        /* number of bytes in each row buffer       */
    int last_row_index;         /* last legal row index                     */
    WINDOW_TYPE output_window;  /* pointer to output window geometry        */
    int *sample_reached_ptr;    /* pointer to array that holds
                                   the sample reached in each line          */
    int *sample_written_ptr;    /* pointer to array that holds the sample
                                   written in each line                     */
    int * volatile buffer_allocation_ptr; /* pointer to the array that tracks
                                             the buffer allocations         */
    int buffers;                /* number of buffers allocated for the data */
    volatile int next_buffer_to_alloc;   /* index of next row to allocate a
                                            buffer to                       */
    int resample_dir;           /* direction of resampling                  */
    int full_buffer;            /* full image buffering being used flag
                                   (non-zero=full image buffered)           */
    int granularity;            /* number of lines for buffer granularity   */
    int interleave;             /* interleave between partial write buffers
                                   when insufficient memory to buffer active
                                   output rows                              */
    int interleave_count;       /* running count of when next buffer
                                   interleave should occur                  */
    int full_buffer_writes;     /* count of entire row buffers written.  It is
                                   a measure of whether enough memory is being
                                   used for the output image                */
    MUTEX_TYPE file_io_mutex;   /* mutex that controls access to file i/o   */
    MUTEX_TYPE buffer_mutex;    /* mutex that controls access to buffer
                                   manipulation                             */
#ifdef USE_WRITE_THREAD
    SEM_TYPE data_available_sem; /* semaphore to notify when data is
                                    available in the write queue            */
    MUTEX_TYPE q_mutex;         /* mutex that controls access for inserting
                                   data into the write queue                */
    WRITE_QUEUE_BUFFER_TYPE *q_ptr; /* pointer to the buffer queue          */
    int q_insert_index;         /* index of next queue insert action        */
volatile    int q_remove_index; /* index of next queue remove action        */
    int q_size;                 /* number of entries allocated for the queue*/
    pthread_t write_thread_id;  /* write thread id                          */
#endif
} OUTPUT_IMAGE_BUFFER_TYPE; /* The OUTPUT_IMAGE_BUFFER_TYPE defines the data
                               that is needed to track the state of the output
                               image buffer.                                */

typedef enum
{
    TM_P,
    MSS_P,
    MSS_X,
    NON_STANDARD
} PRODUCT_TYPE; /* Defines the product types understood for terrain
                   correction.                                              */

struct PROJ
{
    long proj_code;       /* Projection code number                         */
    long units;           /* Projection units code number                   */
    long zone_code;       /* Projection zone code (UTM or State Plane)      */
    long datum_code;      /* Projection datum code                          */
    double proj_coef[15]; /* 15 projection coefficients                     */
}; /* Structure holding projection parameters. */

typedef int (*RESAMPLE_FUNCTION_TYPE)(int, double*, int*, double *, double *);
typedef enum
{
    READ_SCAN_FORWARD,
    READ_SCAN_REVERSE
} SCAN_DIR_TYPE; /* Enumerated type to define which direction the raw image
                    scans are being read in.  forward = top of image to
                    bottom.                                                 */

typedef struct
{
    float *image_buffer_ptr;      /* pointer to the memory allocated to hold
                                     the image lines as they are read into
                                     memory                                 */
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr;
                                  /* pointer to the grid band               */
    IN_IMAGE_TYPE *in_image_ptr;  /* pointer to input image state data      */
    int band;                     /* band number held by the scan manager   */
    int lines_in_image;           /* number of lines in the input image     */
    int lines_in_buffer;          /*number of lines held in the image buffer*/
    int samples_per_line;         /* number of samples per line             */
    int start_line;               /* starting line held in the image buffer */
    int kernel_size;              /* number of lines in the kernel.  This is
                                     used to verify that enough lines exist
                                     in the scan buffer for resampling      */
    int last_valid_line;          /* last valid line currently in the scan
                                     buffer that may be used for resampling.
                                     This is a cached value of the last line
                                     in the buffer with the number of gap
                                     lines and the kernel size subtracted out
                                     (gap since the last scan isn't extended
                                     and kernel since the number of lines in
                                     the kernel will be read).              */
    SCAN_DIR_TYPE scan_direction; /* direction image should be read in      */
    int data_valid;               /* non-zero value if data has been read
                                     into the image buffer.                 */
} SCAN_MAN_TYPE; /* Defines the data that is needed to track the state of a
                    scan manager.  Unless otherwise stated, all members apply
                    to the input image.                                     */

typedef struct
{
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr; /* pointer to grid struct */
    int  band;                 /* current band                              */
    int         in_nl;         /* input space number of lines               */
    int         in_ns;         /* input space number of samples             */
    const SCAN_MAN_TYPE *scan_man_ptr; /* pointer to scan manager           */
    int   resample_dir;        /* direction in which resampling should be
                                  performed (1=forward, -1 = reverse)       */
    int last_line_complete_index; /* last line index where all samples have
                                been completed (actually just an estimate due
                                to multiprocessing)                         */
    int returned_last_line_complete_index; /* last line completed by the
                                              last call to resample         */
    double minpix, maxpix;     /* min and max pixel values for band         */
    int last_line_index_changed; /* flag to indicate if the
                                    returned_last_line_complete_index has
                                    been updated yet */
    int thread_error;          /* set to zero by warp and set to thread number
                                  + 1 if an error occurs in a thread.  If
                                  multiple threads encounter errors, the last
                                  thread to encounter an error will be here */
    OUTPUT_IMAGE_BUFFER_TYPE *output_image_buffer_ptr;
                               /* pointer to output buffer structure        */
    OUTPUT_DATA_TYPE output_data_type; /* output data type                  */
    RESAMPLE_FUNCTION_TYPE resample_function; /* type of resampling to apply*/
} RESAMPLE_THREAD_PARAMS;

typedef enum
{
    CC_RESAMPLING,
    /* MTF_RESAMPLING,            not presently supported                   */
    NN_RESAMPLING,
    BI_RESAMPLING,
    TABLE_RESAMPLING,
    NUMBER_RESAMPLING_TYPES
} RESAMPLE_TYPE;

/*---------
-----------
 Prototypes
-----------
-----------*/
int bi_resample
(
    int output_line,          /* I: output space line number to create      */
    double *buffer_ptr,       /* I/O: buffer for resampled data             */
    int *ending_sample_ptr,   /* O: last sample reached in the line         */
    double *minpix,           /* I/O: minimum pixel value for band          */
    double *maxpix            /* I/O: maximum pixel value for band          */
);

int bi_resample_nogrid
(
    int output_line,          /* I: output space line number to create      */
    double *buffer_ptr,       /* I/O: buffer for resampled data             */
    int *ending_sample_ptr,   /* O: last sample reached in the line         */
    double *minpix,           /* I/O: minimum pixel value for band          */
    double *maxpix            /* I/O: maximum pixel value for band          */
);

void calc_window_corners
(
    const struct CORNERS *known_corners_ptr,    /* I: known corner line/sample
                                                      coordinates           */
    const struct CORNERS *known_corner_proj_ptr,/* I: known corner projection
                                                      coordinates           */
    const struct CORNERS *window_corners_ptr,   /* I: sub-window line/sample
                                                      coordinates           */
    struct CORNERS *window_corner_proj_ptr      /* O: sub-window projection
                                                      coordinates           */
);

int cc_resample
(
    int output_line,         /* I: output space line number to create       */
    double *buffer_ptr,      /* I/O: buffer for resampled data              */
    int *ending_sample_ptr,  /* O: last sample reached in the line          */
    double *minpix,          /* I/O: minimum pixel value for band           */
    double *maxpix           /* I/O: maximum pixel value for band           */
);

int cc_resample_nogrid
(
    int output_line,         /* I: output space line number to create       */
    double *buffer_ptr,      /* I/O: buffer for resampled data              */
    int *ending_sample_ptr,  /* O: last sample reached in the line          */
    double *minpix,          /* I/O: minimum pixel value for band           */
    double *maxpix           /* I/O: maximum pixel value for band           */
);

int check_buffers
(
    int band,                     /* I: current band                        */
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr,
                                  /* I: pointer to grid band                */
    int in_nl,                    /* I: input image number of lines         */
    int in_ns,                    /* I: input image number of samples       */
    int lines_to_buffer,          /* I: number of input lines to buffer     */
    int min_lines_to_buffer,      /* I: min number of input lines to buffer */
    int *active_lines,            /* O: number of lines active in the output
                                        image                               */
    int *min_active_lines         /* O: minimum number of lines active in
                                        the output image                    */
);

long check_grid
(
    struct GEOGRID *map       /* I: geometric mapping grid                  */
);

void check_point
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/ 
    const int output_lines[2],   /* I: lines and samples of the window      */
    const int output_samples[2], /* I: corners in output space              */
    int *min_line_ptr,     /* I/O: min line number in transformed window    */
    int *max_line_ptr,     /* I/O: max line number in transformed window    */ 
    int *min_sample_ptr,   /* I/O: min sample number in transformed window  */
    int *max_sample_ptr,   /* I/O: max sample number in transformed window  */
    int *min_overlap,      /* I/O: amount of line overlap for top of
                                                                input space */
    int *max_overlap       /* I/O: amount of line overlap for bottom of
                                                                input space */
);

void Cleanup_Kernel(void);

int cleanup_output_image_buffer
(
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr,
                           /* I: pointer to output image buffer             */
    int success            /* I: set to non-zero if no errors have occurred */
);

void Cleanup_Scan_Manager
(
    SCAN_MAN_TYPE *scan_man_ptr /* I/O: pointer to scan manager to clean up */
);

void cleanup_terrain(void);

int close_input_image
(
    IN_IMAGE_TYPE *in_image_ptr  /* I/O: pointer to image to close */
);

double cubic_convolution
(
    double alpha,   /* I: cubic convolution alpha parameter     */
    double x        /* I: value to perform cubic convolution on */
);

int do_input_window(void);

int do_output_window(void);

int do_terrain_correction(void);

void free_geom_grid_data
(
    int *bands,              /* I: Array of bands to be read from the grid  */
    int nbands,              /* I: Number of bands in the bands array       */
    gxx_GEOM_GRID_TYPE *grid /* O: IAS grid structure                       */
);

int g_upddr
(
    char host_in[][CMLEN],  /*  array of input host names                   */
    char *host_out,         /*  Output host name                            */
    long window[4],         /*  Input or output window (depending on winmod)
                                specification                               */
    long bands[][MAXBND+1], /*  band numbers for each input image           */
    long nbands,            /*  Number of bands in output image             */
    struct GEOGRID *grid,   /*  Geometric mapping grid                      */
    long winmod,            /*  INPUT/OUTPUT window flag                    */
    struct DDR *ddr         /*  output ddr                                  */
);

void gabort
(
    long stflag      /* I: fatal error status flag */
);

const char *get_corners_filename(void);

float get_cubic_convolution_alpha(void);

const char *get_DEM_filename(void);

void *get_elev
(
    const char *dem_name,  /* I: name of the dem image file                 */
    int *dem_nl,           /* O: nl in dem image                  	    */
    int *dem_ns,           /* O: ns in dem image	                    */
    OUTPUT_DATA_TYPE *dem_dtype  /* O: data type of dem image           */
);

double get_fill_pixel(void);

int get_geom_grid_data
(
    int *bands,              /* I: Array of bands to be read from the grid  */
    int nbands,              /* I: Number of bands in the array             */
    gxx_GEOM_GRID_TYPE *grid /* O: IAS grid structure                       */
);

int get_geom_grid_headers
(
    const char *grid_filename,       /* I: grid file name                   */
    int *band_ptr,                   /* I: array of band numbers            */
    int nbands,                      /* I: number of bands                  */
    gxx_GEOM_GRID_TYPE *grid         /* O: IAS grid structure               */
);

const char *get_grid_filename(void);

const struct PROJ  *get_inproj(void);

void set_input_image_filename
(
    int imageno                /* I: Number of input image to get with a 
                                     call to get_input_image_filename       */
);



const char *get_input_image_filename(void);

const WINDOW_TYPE *get_input_window(void);

const double *get_ipixsize(void);

const double *get_iulcoor(void);

double get_kernel_step_size(void);

void get_kernel_table_info
(
    int *subpixel_steps, /* O: number of subpixel steps in table            */
    int *table_entries,  /* O: number of table entries for a whole pixel
                                step (one more than subpixel_steps          */
    int *kernel_width,   /* O: width of kernel (samples across kernel)      */
    int *kernel_height   /* O: height of kernel (lines across kernel)       */
);

const struct GEOGRID *get_las_grid(void);

int get_lines_in_kernel(void);

const double *get_opixsize(void);

const double *get_oulcoor(void);

const struct PROJ  *get_outproj(void);

OUTPUT_DATA_TYPE get_output_data_type(void);

int get_output_data_size(void);/* returns bytes for current OUTPUT_DATA_TYPE*/

void *get_output_image_buffer
(
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr, /* I: output buffer pointer*/
    int row_index,   /* I: row index (calculated by line number/GRANULARITY)*/
    int thread_num   /* I: thread number of calling thread                  */
);

const char *get_output_image_filename(void);

float get_output_range_maximum(void);

float get_output_range_minimum(void);

const WINDOW_TYPE *get_output_window
(
    int band        /* I: band number for the desired the output window     */
);

int getpar
(
    struct PARBLK *vblk,       /* I: TAE parameter block                    */
    char outtae[][CMLEN],      /* O: Output image filename (TAE)            */
    int *ncpu,                 /* O: Number of CPUs (threads) to use        */
    int *image_buf_size,       /* O: Size (in MB) of desired image buffer   */
    int *lines_to_buffer,      /* O: Number of input image lines to buffer  */
    int *buffer_full_output_image, /* O: flag to buffer entire output image */
    int *gridflg,              /* O: Grid flag: 0 = no grid, use proj. params.
                                               1 = use grid                 */
    long las_bands[][MAXBND+1], /* O: LAS band array                        */
    int *nbands,               /* O: number of bands                        */
    long *numimg,              /* O: Number of images specified             */
    long gin_band[],           /* O: Num. of bands specified for each image */  
    long *wind_mode,           /* O: Window mode                            */
    long *eltime,              /* O: TAE global--message timer              */
    char *history_flag,        /* O: TAE global--history flag               */
    char *minmax_flag,          /* O: TAE global--minmax flag                */
    char ihost[][CMLEN]        /* O: Host name of the input image files     */
);

PRODUCT_TYPE get_product_type(void);

int get_proj_prm
(
    char *proj_name,         /* I: Projection parameter file name           */
    char *outkey,            /* I: Output projection parameter key          */
    long out_def,            /* I: Boolean flags--default given by user?    */
    struct PROJ *outp        /* O: Output projection parameters             */
);

const double *Get_Resample_Weight_Table_Ptr(void);

RESAMPLE_TYPE get_resampling_type(void);

const float *Get_Sample_Data
(
    const SCAN_MAN_TYPE *scan_man_ptr, /* I: pointer to scan manager struct */
    int  line,    /* I: line number of the upper left corner of the kernel  */
    int  sample   /* I: sample number of the upper left corner of the kernel*/
);

int get_samples_in_kernel(void);

int get_sat_num(void);

int get_sclat
(
    char* host_in, /* I: array of input host names       */
    double *sclat  /* O: returned scene center latitiude */
);

const char *get_table_filename(void);

float get_terrain_adjustment
(
    double in_line,   /* I: input space line    */
    double in_samp,   /* I: input space sample  */
    int    out_line,  /* I: output space line   */
    int    out_samp   /* I: output space sample */
);

void initialize_bi_resample
(
    const gxx_GEOM_GRID_BAND_TYPE *i_grid_band_ptr,/*I: current band of grid*/
    const SCAN_MAN_TYPE *i_scan_man_ptr, /* I: input image scan manager     */
    int *i_sample_reached_ptr,       /* I: ptr to last sample reached array */
    int i_input_image_samples,       /* I: samples across the input image   */
    int i_output_window_last_sample, /* I: last sample (not included) in
                                           output window                    */
    double i_min_range,              /* I: minimum output range             */
    double i_max_range               /* I: maximum output range             */
);

void initialize_cc_resample
(
    const gxx_GEOM_GRID_BAND_TYPE *i_grid_band_ptr,/*I: current band of grid*/
    const SCAN_MAN_TYPE *i_scan_man_ptr, /* I: input image scan manager     */
    int *i_sample_reached_ptr,       /* I: pointer to the last sample reached
                                           array                            */
    int i_input_image_samples,       /* I: samples across the input image   */
    int i_output_window_last_sample, /* I: last sample (not inc.) in output
                                           window                           */
    double i_min_range,              /* I: minimum output range             */
    double i_max_range               /* I: maximum output range             */
);

void initialize_nn_resample
(
    const gxx_GEOM_GRID_BAND_TYPE *i_grid_band_ptr,/*I: current band of grid*/
    const SCAN_MAN_TYPE *i_scan_man_ptr, /* I: input image scan manager     */
    int *i_sample_reached_ptr,/* I: pointer to the last sample reached array*/
    int i_output_window_last_sample, /* I: last sample (not included) in
                                           output window                    */
    double i_min_range,              /* I: minimum output range             */
    double i_max_range               /* I: maximum output range             */
);

int initialize_ols2ils(void);

OUTPUT_IMAGE_BUFFER_TYPE *initialize_output_image_buffer
(
    int *active_lines,        /* I: number of lines active in output image  */
    int min_active_lines,     /* I: minimum number of lines active in
                                    the output image                        */
    int band,                 /* I: current band                            */
    struct FDESC *otfdsc,     /* I: output file descriptor                  */
    int threads,              /* I: number of threads to accomodate         */
    int resample_dir,         /* I: resampling direction                    */
    int *max_output_image_buffer,/* I/O: maximum memory to use for output
                                       buffering                            */
    int buffer_full_output_image,/* I: flag to buffer entire output image,
                                       ignoring max memory size             */
    int pass                     /* I: iteration pass number for determining
                                       input lines to buffer and output image
                                       buffer size                          */
);

SCAN_MAN_TYPE *Initialize_Scan_Manager
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/
    int iband,                  /* I: indexed band number to read           */
    int band,                   /* I: band number to read                   */
    int  *lines_to_buffer,      /* I/O: number of lines to buffer at a time */
    int  *min_lines_to_buffer,  /* O: minimum number of lines to buffer     */
    int  *samples_per_line_ptr, /* O: number of samples per line            */
    int  *lines_in_image_ptr,   /* O: number of lines in the input image    */
    int  kernel_size            /* I: number of lines in kernel             */
);

void initialize_table_resample
(
    const gxx_GEOM_GRID_BAND_TYPE *i_grid_band_ptr,/*I: current band of grid*/
    const SCAN_MAN_TYPE *i_scan_man_ptr, /* I: input image scan manager     */
    int *i_sample_reached_ptr, /* I: ptr to the last sample reached array   */
    int i_input_image_samples, /* I: samples across the input image         */
    int i_output_window_last_sample, /* I: last sample (not included) in
                                           output window                    */
    double i_min_range,        /* I: minimum output range                   */
    double i_max_range         /* I: maximum output range                   */
);

int initialize_terrain
(
    const char *input_image_name, /* I: input image name                    */
    const char *dem_name,         /* I: DEM name                            */
    PRODUCT_TYPE product_type,    /* I: product type                        */
    int input_image_lines,        /* I: lines in input image                */
    int input_image_samples,      /* I: samples across input image          */
    int output_image_lines,       /* I: lines in output image               */
    int output_image_samples,     /* I: samples across output image         */
    int satnum                    /* I: Landsat satellite number            */
);

int Kernel_Setup
(
    RESAMPLE_TYPE resampling_type, /* I: resampling type                    */
    double cc_alpha                /* I: cubic convolution alpha parameter  */
);

int More_Scans_To_Read
(
    const SCAN_MAN_TYPE *scan_man_ptr /* I: pointer to scan manager struct  */
);

int nadirline
(
    char *inlt,         /* I: name of the corners labeled table file        */
    double *slope,      /* O: slope of nadir                                */
    double *y_inter     /* O: y-intercept of nadir                          */
);

int ng_upddr
(
    char inhost[][CMLEN],   /* I: array of input host names                 */
    char *outhost,          /* I: output host name                          */
    long bands[][MAXBND+1], /* I: band numbers for each input image         */
    long nbands             /* I: number of bands in output image           */
);

int nn_resample
(
    int output_line,        /* I: output space line number to create        */
    double *buffer_ptr,     /* I/O: buffer for resampled data               */
    int *ending_sample_ptr, /* O: last sample reached in the line           */
    double *minpix,         /* I/O: minimum pixel value for band            */
    double *maxpix          /* I/O: maximum pixel value for band            */
);

int nn_resample_nogrid
(
    int output_line,        /* I: output space line number to create        */
    double *buffer_ptr,     /* I/O: buffer for resampled data               */
    int *ending_sample_ptr, /* O: last sample reached in the line           */
    double *minpix,         /* I/O: minimum pixel value for band            */
    double *maxpix          /* I/O: maximum pixel value for band            */
);

int num_bottom_kernel_lines(void);

int num_left_kernel_samples(void);

int num_right_kernel_samples(void);

int num_top_kernel_lines(void);

int ols2ils
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/
    int output_line,            /* I: output line                           */
    int output_sample,          /* I: output sample                         */
    double *input_line_ptr,     /* O: input line                            */
    double *input_sample_ptr    /* O: input sample                          */
);

IN_IMAGE_TYPE *open_input_image
(
    const char *filename,  /* I: file name of image to open                 */
    int band,              /* I: band of image to open                      */
    int  *in_nl_ptr,       /* O: number of lines in image                   */
    int  *in_ns_ptr        /* O: number of samples in image                 */
);

int out_frame
(
    double *pixsize,      /* I: Projection distance in x & y                */
    long *window,         /* O: Output window (SL, SS, NL, NS)              */
    double *corners,      /* O: Output window corner coordinates            */
    struct PROJ *outproj, /* I: Output projection parameters                */
    struct PROJ *inproj,  /* I: Input projection parameters                 */
    long frame_mode,      /* I: Type of framing                             */
    double *ul,           /* I: Upper left corner coordinates               */
    double *lr,           /* I: Lower right corner coordinates              */
    long *ls,             /* I: Line/sample coordinates of ul               */
    long nl,              /* I: Number of lines in output space             */
    long ns,              /* I: Number of samples in output space           */
    long units,           /* I: Units code of coordinates entered           */
    double pxmax,         /* I: Maximum X projection coordinate             */
    double pxmin,         /* I: Minimum X projection coordinate             */
    double pymax,         /* I: Maximum Y projection coordinate             */
    double pymin          /* I: Minimum Y projection coordinate             */
);

int read_input_lines
(
    IN_IMAGE_TYPE *in_image_ptr, /* I/O: input file state pointer */
    int   start_line,            /* I: current start line number  */
    int   num_lines,             /* I: number of lines to read    */
    float *buffer                /* O: data buffer                */
);

int Read_Next_Scan_Set
(
    SCAN_MAN_TYPE *scan_man_ptr /* I/O: pointer to scan manager structure */
);

THREAD_RETURN_TYPE resample_image
(
  void * params   /* I: pointer to input parameter structure.  The
                        parameters are not passed directly since
                        the fork mechanism is limited to passing
                        a maximum of 6 void pointers.                    */
);

int return_output_image_buffer
(
    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr,/*I: output buffer state ptr*/
    int row_index,   /* I: row index (calculated by line number/GRANULARITY)*/
    int row_start,   /* I: starting line number of row                      */
    int row_end      /* I: ending line number of row                        */
);

void set_scaled_output_window
(
    int band,            /* I: band number for the desired output window */
    float line_scaling,  /* I: scaling factor for the window line size   */
    float sample_scaling /* I: scaling factor for the window sample size */
);

void set_input_window
(
    long start_line,   /* I: starting line for the window              */
    long start_sample, /* I: starting sample for the window            */
    long num_lines,    /* I: number of lines in the window             */
    long num_samples   /* I: number of samples in the window           */
);

void set_output_window
(
    int band,          /* I: band number for the desired output window */
    long start_line,   /* I: starting line for the output window       */
    long start_sample, /* I: starting sample for the output window     */
    long num_lines,    /* I: number of lines in the output window      */
    long num_samples   /* I: number of samples in the output window    */
);

void setup_resampf1p_output(void);

int table_resample
(
    int output_line,        /* I: output space line number to create        */
    double *buffer_ptr,     /* I/O: buffer for resampled data               */
    int *ending_sample_ptr, /* O: last sample reached in the line           */
    double *minpix,         /* I/O: minimum pixel value for band            */
    double *maxpix          /* I/O: maximum pixel value for band            */
);

int table_resample_nogrid
(
    int output_line,        /* I: output space line number to create        */
    double *buffer_ptr,     /* I/O: buffer for resampled data               */
    int *ending_sample_ptr, /* O: last sample reached in the line           */
    double *minpix,         /* I/O: minimum pixel value for band            */
    double *maxpix          /* I/O: maximum pixel value for band            */
);

int transform_ow2iw
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/
    const WINDOW_TYPE *output_window_ptr, /* I: pointer to window           */
    int kernel_size,      /* I: number of lines in kernel                   */
    int *min_line_ptr,    /* O: minimum line number in transformed window   */
    int *max_line_ptr,    /* O: maximum line number in transformed window   */
    int *min_sample_ptr,  /* O: minimum sample number in transformed window */
    int *max_sample_ptr,  /* O: maximum sample number in transformed window */
    int *line_overlap     /* O: amount of line overlap to use               */
);

int use_ddr_info
(
    struct PARBLK *vblk, /* I: TAE parameter block                          */
    struct PROJ *pro,    /* O: Structure containing projection parameters   */
    long projflag,       /* I: Projection flag: 0 = projection not needed,
                                                1 = projection info required*/
    double *corners,     /* O: Corner projection coordinates                */
    long *image,         /* O: Corner image coordinates                     */
    long in_wind[][4],   /* I: Image window specification                   */
    char *flag,          /* I: input/output flag                            */
    double *pixy,        /* O: pixel size in y direction                    */
    double *pixx         /* O: pixel size in x direction                    */
);

int warp
(
    const gxx_GEOM_GRID_TYPE *grid_ptr, /* I: pointer to grid               */
    int gridflg,                  /* I: grid flag:
                                              0 = no grid, use proj. params.
                                              1 = use grid                  */
    int iband,                    /* I: current band (indexed)              */
    int band,                     /* I: current band                        */
    RESAMPLE_TYPE resampling_type,/* I: type of resampling                  */
    struct FDESC *otfdsc,         /* I: output file descriptor              */
    int *lines_to_buffer,         /* I: number of input lines to buffer     */
    int  procs_to_use,            /* I: number of processors to use         */
    int  *max_output_image_buffer, /* I/O: max mem in MB to use for buffering
                                           the output image                 */
    int  buffer_full_output_image,/* I: flag to indicate entire output image
                                        should be kept in memory during
                                        resampling                          */
    double *minpix,               /* I/O: minimum pixel value for band      */
    double *maxpix                /* I/O: maximum pixel value for band      */
);

#endif /* _RESAMPLE_H_ */
