/*****************************************************************************
NAME:           resample_image

PURPOSE:
The resample_image module is the driver for performing the different types
of resampling.

DEVELOPMENT HISTORY:
  Baselined from revision 1.32 of resampf1p.c


NOTES:
none

ALGORITHM REFERENCES:
none


*****************************************************************************/
 
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "resample.h"
#include "debug.h"               /* ASSERT macro                            */


/*****************************************************************************
NAME:           convert_data

PURPOSE:                
convert_data converts the intermediate resampling results (currently stored as
doubles) and converts them to the requested output data type.

RETURN VALUE:
The type converted data is returned in the outbuf.

NOTES:
This routine was created to prevent the code having to be copied to the
different specialized resampling routines.  This incurs a small calling 
penalty, but it is small when compared to the actual function.

ALGORITHM REFERENCES:
none

*****************************************************************************/

static void convert_data
(
    const double *mid_output_ptr, /* I: data to convert to the output type  */
    void *outbuf,                 /* O: output data buffer                  */
    int left_sample,          /* I: starting index in mid_output_ptr buffer */
    int right_sample,           /* I: ending index in mid_output_ptr buffer */
    long row_index,               /* I: row index in outbuf                 */
    OUTPUT_DATA_TYPE output_data_type /* I: output data type                */
)
{
    int l;  /* counter */


    /* Convert the intermediate resampling results to the requested 
       output type.
       ------------------------------------------------------------*/
    if (output_data_type == BYTE_OUTPUT)
    {
        unsigned char *char_ptr = (unsigned char *)outbuf;

        /* If buffering output, offset to the correct line and sample.
           -----------------------------------------------------------*/
        char_ptr = &char_ptr[row_index + left_sample];

        /* Convert the data, using unsigned rounding.
           ------------------------------------------*/
        for (l = left_sample; l < right_sample; l++)
            *char_ptr++ = (unsigned char)(*mid_output_ptr++ + 0.5);
    }
    else if (output_data_type == FLOAT_OUTPUT)
    {
        float *float_ptr = (float *)outbuf;

        /* If buffering output, offset to the correct line and sample.
           -----------------------------------------------------------*/
        float_ptr = &float_ptr[row_index + left_sample];

        for (l = left_sample; l < right_sample; l++)
            *float_ptr++ = *mid_output_ptr++;
    }
    else if (output_data_type == INT16_OUTPUT)
    {
        short *int16_ptr = (short *)outbuf;

        /* If buffering output, offset to the correct line and sample.
           -----------------------------------------------------------*/

        int16_ptr = &int16_ptr[row_index + left_sample];

        for (l = left_sample; l < right_sample; l++)
        {
            /* Convert the data, using signed rounding.
               ----------------------------------------*/
            if (*mid_output_ptr >= 0)
                *int16_ptr++ = (short)(*mid_output_ptr++ + 0.5);
            else
                *int16_ptr++ = (short)(*mid_output_ptr++ - 0.5);
        }
    }
    else
    {
        int *int32_ptr = (int *)outbuf;

        /* If buffering output, offset to the correct line and sample.
           -----------------------------------------------------------*/
        int32_ptr = &int32_ptr[row_index + left_sample];

        ASSERT(output_data_type == INT32_OUTPUT);

        for (l = left_sample; l < right_sample; l++)
        {
            /* Convert the data, using signed rounding.
               ----------------------------------------*/
            if (*mid_output_ptr >= 0.0)
                *int32_ptr++ = (int)(*mid_output_ptr++ + 0.5);
            else
                *int32_ptr++ = (int)(*mid_output_ptr++ - 0.5);
        }
    }
}

/*****************************************************************************
NAME:           resample_image

PURPOSE:
resample_image is the driver for all the resampling types.

RETURN VALUE:
The resampled data is returned.

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

THREAD_RETURN_TYPE resample_image
(
    void *params   /* I: pointer to input parameter structure.  The
                         parameters are not passed directly since
                         the fork mechanism is limited to passing
                         a maximum of 6 void pointers.                    */
)
{
    /* Create local copies of the parameters.
       --------------------------------------*/
    RESAMPLE_THREAD_PARAMS *param_ptr = (RESAMPLE_THREAD_PARAMS *)params;
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr = param_ptr->grid_band_ptr;

    OUTPUT_IMAGE_BUFFER_TYPE *output_buffer_ptr = 
        param_ptr->output_image_buffer_ptr;
    RESAMPLE_FUNCTION_TYPE resample_function = param_ptr->resample_function;
                            /* pointer to the current resampling function   */
    const WINDOW_TYPE *output_window_ptr = get_output_window(param_ptr->band); 
                            /* pointer to the output window geometry        */

    void *out_buffer_ptr;   /* Output image buffer                          */

    double *middle_buf_ptr; /* pointer to intermediate buffer allocated
                               to allow resampling to be done in doubles
                               then converted to whichever output type
                               was selected by the user                     */
    double minpix = param_ptr->minpix; /* minimum pixel value for band      */
    double maxpix = param_ptr->maxpix; /* maximum pixel value for band      */

    int changed;            /* flag to indicate something has changed in the 
                               row buffer                                   */
    int current_line;       /* current line being resampled                 */
    int data_was_not_available = 0; /* flag to indicate that input data was
                                       not available to complete a line     */
    int end_sample;         /* last sample created on current line          */
    int ending_line;        /* ending row                                   */
    int granularity = output_buffer_ptr->granularity; /* granularity of the 
                                                         number of rows to
                                                         resample at a time */
    int input_data_used;    /* flag to indicate that input data has been used
                               in the current row                           */
    int last_line_complete_index = param_ptr->last_line_complete_index;
    int last_sample_was_reached = 1;/* flag to indicate the last sample has
                                       been reached for all lines so far    */
    int multi_proc_index;   /* index used to split the resampling task
                               between threads                              */
    int new_last_line_index = 0; /* the last line completed index allows the
                                    resampler to skip over the majority of the
                                    rows that have already been completely
                                    resampled, instead of checking each line
                                    individually.                           */
    int resample_dir = param_ptr->resample_dir;
    int row_index;          /* row index for output buffer                  */
    int *sample_reached_ptr = output_buffer_ptr->sample_reached_ptr; 
                            /* pointer to array of sample numbers reached
                               in resampling                                */
    int starting_line;      /* starting line for the current set of lines   */
    int thread_num;         /* number of current thread                     */


    /* Get the current thread number and check if an error (negative).
       ---------------------------------------------------------------*/
    thread_num = GET_THREAD_NUMBER();
    if (thread_num < 0)
    {
        param_ptr->thread_error = -1;
        RETURN_FROM_THREAD();
    }

    /* Allocate space for the intermediate buffer.
       -------------------------------------------*/
    if (grid_band_ptr)
        middle_buf_ptr = (double *)malloc(sizeof(double)*
                grid_band_ptr->output_image_samps);
    else
        middle_buf_ptr = (double *)malloc(sizeof(double)*
                output_window_ptr->num_samples);
    if (middle_buf_ptr == NULL)
    {
        c_lasmsg(FALSE, "resample-alloc", "Memory allocation error");
        param_ptr->thread_error = thread_num + 1;
        RETURN_FROM_THREAD();
    }

    /*----------------------------------------------------------------------
      ----------------------------------------------------------------------
      Loop through the requested output image window, performing resampling.
      ----------------------------------------------------------------------
      ----------------------------------------------------------------------*/

    for (multi_proc_index = ATOMIC_INCREMENT(); 
            multi_proc_index < output_window_ptr->num_lines; 
            multi_proc_index = ATOMIC_INCREMENT())
    {
        /* Make sure the ATOMIC_INC succeeded (worked when >= 0).
           ------------------------------------------------------*/
        if (multi_proc_index < 0)
        {
            param_ptr->thread_error = thread_num + 1;
            RETURN_FROM_THREAD();
        }

        /* Calculate the starting and ending lines for this pass.
           ------------------------------------------------------*/
        if (resample_dir > 0)
        {
            row_index = multi_proc_index + last_line_complete_index;

            /* Resampling from the top to the bottom of output space, so
               calculate the lines based on the top of the image.
               ---------------------------------------------------------*/
            starting_line = row_index*granularity +
                output_window_ptr->start_line;
            ending_line = starting_line + granularity;

            /* If starting line is past end of requested output window,
               break out of the loop.
               --------------------------------------------------------*/
            if (starting_line >= output_window_ptr->end_line)
                break;

            /* Make sure ending line is limited to the last line of the
               output window.
               --------------------------------------------------------*/
            if (ending_line >= output_window_ptr->end_line)
                ending_line = output_window_ptr->end_line;
        }
        else
        {
            row_index = output_buffer_ptr->last_row_index - multi_proc_index
                - last_line_complete_index - 1;

            /* Resampling from the bottom to the top of output space, so
               calculate the lines based on the bottom of the image.
               ---------------------------------------------------------*/
            starting_line = row_index*granularity +
                output_window_ptr->start_line;
            ending_line = starting_line + granularity;

            /* If starting line is before the first line of requested output
               window, break out of the loop.
               -------------------------------------------------------------*/
            if (starting_line < output_window_ptr->start_line)
                break;

            /* Make sure ending line is limited to the last line of the
               output window.
               --------------------------------------------------------*/
            if (ending_line >= output_window_ptr->end_line)
                ending_line = output_window_ptr->end_line;
        }

        /* Clear the flag that indicates input image data was used for this
           set of lines.
           ----------------------------------------------------------------*/
        input_data_used = 0;

        /* Set the changed flag to indicate no output has changed yet.
           -----------------------------------------------------------*/
        changed = 0;

#ifdef VERBOSE
        c_lasmsg(FALSE, "resample-info",
                "Thread %d working on lines %d to %d of %d", thread_num,
                starting_line, ending_line, output_window_ptr->end_line);
#endif

        /* Get a buffer for the output data.
           ---------------------------------*/
        out_buffer_ptr = get_output_image_buffer(output_buffer_ptr, row_index, 
                thread_num);
        if (out_buffer_ptr == NULL)
        {
            param_ptr->thread_error = thread_num + 1;
            RETURN_FROM_THREAD();
        }

        /* Loop through the current set of lines.
           --------------------------------------*/
        for (current_line = starting_line; current_line < ending_line; 
                current_line++)
        {     
            /* Resample the current line.
               --------------------------*/
            input_data_used |=
                resample_function(current_line, middle_buf_ptr,
                        &end_sample, &minpix, &maxpix);

            /* If the current line wasn't completed, flag that
               data wasn't available to complete a line.
               -----------------------------------------------*/
            if (end_sample < output_window_ptr->end_sample)
                data_was_not_available = 1;

            /* If new resampling done, convert the data to the
               requested output type and save it.
               -----------------------------------------------*/
            if (end_sample != sample_reached_ptr[current_line])
            {
                /* Convert the intermediate data to the final data type
                   for the samples of the output window that have been
                   resampled in this pass.
                   ----------------------------------------------------*/
                convert_data(middle_buf_ptr, out_buffer_ptr,
                        sample_reached_ptr[current_line] -
                        output_window_ptr->start_sample,
                        end_sample-output_window_ptr->start_sample, 
                        (current_line - starting_line) *
                        output_window_ptr->num_samples,
                        param_ptr->output_data_type);

                sample_reached_ptr[current_line] = end_sample;

                /* Remember that data has been changed.
                   ------------------------------------*/
                changed = 1;

                /* If the current line has been completed, update the
                   new last line index to hold the current
                   multiprocessing index (another way to say "set of
                   output lines").
                   --------------------------------------------------*/
                if (last_sample_was_reached)
                {
                    if (end_sample >= output_window_ptr->end_sample)
                        new_last_line_index = multi_proc_index;
                    else
                        last_sample_was_reached = 0;
                }
            }
        } /* end of row loop */

        /* Return the buffer to the buffer pool if something has changed.
           This routine takes care of writing the data when needed.
           --------------------------------------------------------------*/
        if (changed)
        {
            if (return_output_image_buffer(output_buffer_ptr,row_index,
                        starting_line,ending_line) != E_SUCC)
            {
                c_lasmsg(FALSE, "resample-buffer",
                        "Error returning buffer to pool.");
                param_ptr->thread_error = thread_num + 1;
                RETURN_FROM_THREAD();
            }
        }

        /* If sometime in the past, not enough data was available to finish
           a line and no input data was used to complete this set of lines, 
           break out of the loop since more input data needs to be read 
           before continuing.
           ----------------------------------------------------------------*/
        if (data_was_not_available && !input_data_used)
            break;

    } /* end of multiprocessing loop */

    free(middle_buf_ptr);

    /* Calculate the new last line index so it can be used for the next set
       of scans.  Guarantee that the most conservative value from all the
       threads ends up being saved.  This is needed since in some remote
       cases, the last line completed by one thread may be past the last
       line completed by a different thread.
       --------------------------------------------------------------------*/
    new_last_line_index += last_line_complete_index;

    if (ENTER_CRITICAL_SECTION() != 0)
    {
        param_ptr->thread_error = thread_num + 1;
        RETURN_FROM_THREAD();
    }

    if (!param_ptr->last_line_index_changed)
    {
        /* No other thread has updated the last line completed index so just
           update it with this thread's value.
           -----------------------------------------------------------------*/
        param_ptr->returned_last_line_complete_index = new_last_line_index;
        param_ptr->last_line_index_changed = TRUE;
    }
    else
    {
        /* The last line completed index has already been updated by a
           different thread, so check for the more conservative value.
           -----------------------------------------------------------*/
        if (new_last_line_index <
                param_ptr->returned_last_line_complete_index)
            param_ptr->returned_last_line_complete_index =
                new_last_line_index;
    }

    /* Update global min/max pixel values with values computed for this
       thread.
       ----------------------------------------------------------------*/
    if (minpix < param_ptr->minpix)
        param_ptr->minpix = minpix;
    if (maxpix > param_ptr->maxpix)
        param_ptr->maxpix = maxpix;

    if (EXIT_CRITICAL_SECTION() != 0)
    {
        param_ptr->thread_error = thread_num + 1;
        RETURN_FROM_THREAD();
    }

    RETURN_FROM_THREAD();
}
