/*****************************************************************************
NAME:           scanman

PURPOSE:        
The scanman (short for Scan Manager) manages the buffer of scans currently 
residing in memory.  For flexibility, the number of scans to buffer is a 
parameter for initializing the scan manager.  This allows from a few scans
to an entire image to be buffered.  The name is not really accurate anymore 
since this generic version of the resampler manages the input image on
a line basis instead of a scan basis.  This is because the input image is 
not split into scans like the L7 input image.


ROUTINES:
    Initialize_Scan_Manager
    Get_Sample_Data
    Read_Next_Scan_Set
    Cleanup_Scan_Manager
    More_Scans_To_Read

DEVELOPMENT HISTORY:
  Baselined from revision 1.20 of scanman.c

NOTES:
The user must first initialize a scan manager, then read the first set of
lines in by calling the Read_Next_Scan_Set routine before attempting to 
get sample data.  Multiple scan managers may be created to read multiple 
bands, or even different areas of the same band.

ALGORITHM REFERENCES:
None

*****************************************************************************/

/*
**
** INCLUDE FILES 
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "resample.h"
#include "debug.h"                   /* ASSERT macro                       */
#include "ltable.h"                  /* MAX/MIN definitions                */


static int line_overlap; /* number of input lines to overlap from one
                            file read to the next                          */


/*****************************************************************************
FUNCTION NAME:  Initialize_Scan_Manager

PURPOSE:        
Initialize_Scan_Manager does the initial setup of an instance of a scan
manager.

RETURNS:
A pointer to the scan manager state structure is returned.  If an error
occurs, an error message is generated and NULL is returned.

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

SCAN_MAN_TYPE *Initialize_Scan_Manager
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/
    int iband,                  /* I: indexed band number to read           */
    int band,                   /* I: band number to read                   */
    int  *lines_to_buffer,      /* I/O: number of lines to buffer at a time */
    int  *min_lines_to_buffer,  /* O: minimum number of lines to buffer     */
    int  *samples_per_line_ptr, /* O: number of samples per line            */
    int  *lines_in_image_ptr,   /* O: number of lines in the input image    */
    int  kernel_size            /* I: number of lines in kernel             */
)
{
    SCAN_MAN_TYPE *scan_man_ptr; /* pointer to allocated scan manager struct*/
    double left_line, left_samp; /* line/samp of pnt on left of input image */
    double right_line, right_samp;/* line/samp of pt on right of input image*/
    const WINDOW_TYPE *input_window_ptr; /* input window definition         */
    const WINDOW_TYPE *output_window_ptr; /* output window definition       */

    int loop;                    /* loop counter                            */
    int min_input_line, max_input_line, min_input_sample, max_input_sample;
    int output_line, output_sample; /* output space line/sample pair        */
    int status;                  /* return status flag                      */


    /* Verify the band number is legal.
       --------------------------------*/
    ASSERT ((band > 0) && (band <= MAXBND));

    /* Verify other parameters are legal.
       ----------------------------------*/
    ASSERT (kernel_size >= 0);

    /* Allocate memory for the scan manager structure.
       -----------------------------------------------*/
    scan_man_ptr = (SCAN_MAN_TYPE *)malloc(sizeof(SCAN_MAN_TYPE));
    if (scan_man_ptr == NULL)
    {
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (NULL);
    }

    /* Initialize members of the scan manager structure.
       -------------------------------------------------*/
    scan_man_ptr->image_buffer_ptr = NULL;
    scan_man_ptr->grid_band_ptr = grid_band_ptr;
    if (grid_band_ptr)
        scan_man_ptr->band = grid_band_ptr->band_number;
    else
        scan_man_ptr->band = band;
    scan_man_ptr->kernel_size = kernel_size;
    scan_man_ptr->start_line = 0;

    /* Open the input file.
       --------------------*/
    scan_man_ptr->in_image_ptr = open_input_image(get_input_image_filename(), 
            scan_man_ptr->band,
            lines_in_image_ptr, samples_per_line_ptr);
    if (scan_man_ptr->in_image_ptr == NULL)
    {
        free(scan_man_ptr);
        c_lasmsg(FALSE, "resample-open",
                "Error returned from open_input_image for file %s.",
                get_input_image_filename());
        return (NULL);
    }

    scan_man_ptr->samples_per_line = *samples_per_line_ptr;
    scan_man_ptr->lines_in_image = *lines_in_image_ptr;

    /* Ensure that the input window is not outside the grid coverage.
       --------------------------------------------------------------*/

    /* Get the input and output windows.
       ---------------------------------*/
    input_window_ptr = get_input_window();
    output_window_ptr = get_output_window(iband);

    /* Get the input window associated with the output window.
       -------------------------------------------------------*/
    status = transform_ow2iw(grid_band_ptr, output_window_ptr,
            kernel_size, &min_input_line, &max_input_line,
            &min_input_sample, &max_input_sample,
            &line_overlap);
    if (status != E_SUCC || !grid_band_ptr)
    {
        /* Some projections contain undefined regions that
           make it difficult to find min/max input space values.
           Therefore we effectively disable the transform by returning
           the original input window values.
           -----------------------------------------------------------*/
        min_input_line = input_window_ptr->start_line;
        max_input_line = input_window_ptr->end_line - 1;
        min_input_sample = input_window_ptr->start_sample;
        max_input_sample = input_window_ptr->end_sample - 1;
        if (status != E_SUCC)
            line_overlap = DEFAULT_LINE_OVERLAP;
    }

    /* Determine if the file should be read top to bottom or bottom to top.
       This is done by using the first set of in_lines in the grid.  If the 
       left input space line number is smaller than the right input space
       line number, the lines in input space slope upward relative to the 
       lines in output space as you go from left to right.  Therefore, the
       image needs to be read from the top to the bottom (forward).  
       Otherwise the image needs to be read from bottom to top (reverse).
       --------------------------------------------------------------------*/
    if (grid_band_ptr)
    {
        int bottom_left_line, bottom_right_line;

        left_line = grid_band_ptr->in_lines[0]; 
        right_line = grid_band_ptr->in_lines[grid_band_ptr->num_out_samps-1]; 

        /* if the grid was made using a polyfit (and maybe other cases) it is
           possible that the top of the grid will indicate the lines should be
           read in one direction while the bottom will indicate it should be
           read in the other direction.  If that happens, the overlap needs
           to be adjusted to make sure the entire output image can be 
           created. 
           Note: this is not guaranteed to fix the problem in all cases, but
           since this has only been a problem once in several years, this will
           likely be enough.  Also, this hasn't been applied to the non-grid
           case, but again, it is even more unlikely to be a problem there. */
        bottom_left_line = grid_band_ptr->in_lines[
            (grid_band_ptr->num_out_lines -1) * grid_band_ptr->num_out_samps];
        bottom_right_line = grid_band_ptr->in_lines[
            (grid_band_ptr->num_out_lines -1) * grid_band_ptr->num_out_samps
            + grid_band_ptr->num_out_samps - 1];
        if ((left_line - right_line) * (bottom_left_line - bottom_right_line)
                < 0)
        {
            /* if the signs on the line differences is not the same, the 
               top and bottom edges slope in a different direction, so 
               increase the needed overlap by the larger difference */
            int top_diff = abs(left_line - right_line);
            int bottom_diff = abs(bottom_left_line - bottom_right_line);

            if (bottom_diff > top_diff)
                line_overlap += bottom_diff;
            else
                line_overlap += top_diff;

        }
    }
    else
    {
        /* No grid, so it is difficult to ensure that any given point
           has a *defined* output window/input window point pair for
           an arbitrary location.  So let's assume that the middle of
           the output window is a "good" point and pick two adjacent
           samples within an output line for use in determining the scan
           direction.  If this doesn't work, the problem may need to be
           broken into separate runs or we may need to "search" the image
           for usable transformation points.
           --------------------------------------------------------------*/
        output_line = output_window_ptr->num_lines/2;
        output_sample = output_window_ptr->num_samples/2;
        status = ols2ils(grid_band_ptr, output_line, output_sample,
                &left_line, &left_samp);
        if (status == E_SUCC)
        {
            /* Check adjacent output space samples until either we get to
               a different input space sample or we have tried a set number
               of samples.
               ------------------------------------------------------------*/
            right_samp = left_samp;
            loop = 0;
            while (right_samp == left_samp)
            {
                loop++;
                if (loop == 10)
                    break;        /* We've tried enough points, so give up. */
                output_sample += 2;
                status = ols2ils(grid_band_ptr, output_line, output_sample,
                        &right_line, &right_samp);
            }
        }

        if (status != E_SUCC)
        {
            c_lasmsg(FALSE,"resample-warn",
                    "Unable to determine scan direction.  Assuming FORWARD.");
            left_line = 0;
            right_line = 1;                      /* force read_scan_forward */
        }
    }

    /* Set scan_direction.
       -------------------*/
    if (left_line <= right_line)
        scan_man_ptr->scan_direction = READ_SCAN_FORWARD;
    else
        scan_man_ptr->scan_direction = READ_SCAN_REVERSE;

    /* Verify there are enough lines in the buffer.
       --------------------------------------------*/
    *min_lines_to_buffer = 2*line_overlap;
    if (*lines_to_buffer < *min_lines_to_buffer)
    {
        *lines_to_buffer = *min_lines_to_buffer;
        c_lasmsg(FALSE, "resample-buflines",
                "Not enough input lines buffered.  BUFLINES has been "
                "increased to %d (minimum required).", *lines_to_buffer);
    }
    scan_man_ptr->lines_in_buffer = *lines_to_buffer;

    /* Adjust for the size of the kernel.
       ----------------------------------*/
    min_input_line -= num_top_kernel_lines();
    max_input_line += num_bottom_kernel_lines();
    min_input_sample -= num_left_kernel_samples();
    max_input_sample += num_right_kernel_samples();

    /* If doing terrain correction, default the min/max input samples
       to be the extent of the image.  
       NOTE:  The maximum terrain effect could be calculated in the 
       sample direction and it could be subtracted from the min
       sample and added to the max sample, but it wouldn't gain
       much in resampling speed.
       NOTE:  If the satellite has terrain effects in the line
       direction, the min/max input line will need to be
       relaxed to include the max terrain effect in the line
       direction.
       ---------------------------------------------------------------*/
    if (do_terrain_correction())
    {
        min_input_sample = 0;
        max_input_sample = *samples_per_line_ptr;
    }

    /* Clamp the input window to the input image.
       ------------------------------------------*/
    min_input_line = MAX(min_input_line,input_window_ptr->start_line);
    max_input_line = MIN(max_input_line,input_window_ptr->end_line);
    min_input_sample = MAX(min_input_sample,input_window_ptr->start_sample);
    max_input_sample = MIN(max_input_sample,input_window_ptr->end_sample);

    /* Verify there is a window after clamping.
       ----------------------------------------*/
    if ((min_input_line >= max_input_line) || 
            (min_input_sample >= max_input_sample))
    {
        /* The input window lies outside the input image, error.
           -----------------------------------------------------*/
        close_input_image(scan_man_ptr->in_image_ptr);
        free(scan_man_ptr);

        c_lasmsg(FALSE, "resample-outside", "Input window outside image.");
        return (NULL);
    }

    /* Set the input window.
       ---------------------*/
    set_input_window((long)min_input_line, (long)min_input_sample,
            (long)(max_input_line - min_input_line),
            (long)(max_input_sample - min_input_sample));

    /* If doing input windowing, make sure the window limits fall in the 
       input image.
       -----------------------------------------------------------------*/
    if (do_input_window())
    {
        input_window_ptr = get_input_window();
        if ((input_window_ptr->start_line > *lines_in_image_ptr) ||
                (input_window_ptr->end_line > *lines_in_image_ptr) ||
                (input_window_ptr->start_sample > *samples_per_line_ptr) ||
                (input_window_ptr->end_sample > *samples_per_line_ptr))
        {
            /* The input window is outside the image, so abort.
               ------------------------------------------------*/
            close_input_image(scan_man_ptr->in_image_ptr);
            free(scan_man_ptr);
            c_lasmsg(FALSE, "resample-outside",
                    "Input window outside image.");
            return (NULL);
        }
    }

    /* Get the input window (since it has not been gotten for sure yet) 
       so the lines to buffer can be limited to the lines in the input
       window.
       ----------------------------------------------------------------*/
    input_window_ptr = get_input_window();
    if (input_window_ptr->num_lines < *lines_to_buffer)
        *lines_to_buffer = input_window_ptr->num_lines;

    /* Allocate memory for the line buffer.
       ------------------------------------*/
    scan_man_ptr->image_buffer_ptr =
        (float *)malloc(sizeof(float)*(*samples_per_line_ptr)*
                        (*lines_to_buffer));
    if (scan_man_ptr->image_buffer_ptr == NULL)
    {
        close_input_image(scan_man_ptr->in_image_ptr);
        free(scan_man_ptr);
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (NULL);
    }

    /* Flag that the data has not been read yet.
       -----------------------------------------*/
    scan_man_ptr->data_valid = 0;

    return(scan_man_ptr);
}


/*****************************************************************************
FUNCTION NAME:  Cleanup_Scan_Manager

PURPOSE:        
Cleanup_Scan_Manager frees the memory allocated for a scan manager structure.

RETURNS:
  none

NOTES:
- After this call, the scan_man_ptr will no longer contain valid data.

ALGORITHM REFERENCES:
none

*****************************************************************************/

void Cleanup_Scan_Manager
(
    SCAN_MAN_TYPE *scan_man_ptr /* I/O: pointer to scan manager to clean up */
)
{
    if (scan_man_ptr != NULL)
    {
        /* Close the input file.
           ---------------------*/
        close_input_image(scan_man_ptr->in_image_ptr);

        if (scan_man_ptr->image_buffer_ptr != NULL)
            free (scan_man_ptr->image_buffer_ptr);

        free (scan_man_ptr);
    }
}


/*****************************************************************************
FUNCTION NAME:  Read_Next_Scan_Set

PURPOSE:        
Read_Next_Scan_Set reads in the next set of lines that will fit into the
scan buffer.  

RETURNS:
The scans and associated members of the scan manager structure are updated.
The function returns a E_SUCC or E_FAIL indication.

NOTES:
This routine loads the next set of image lines into memory.

ALGORITHM REFERENCES:
none

*****************************************************************************/

int Read_Next_Scan_Set
(
    SCAN_MAN_TYPE *scan_man_ptr /* I/O: pointer to scan manager structure   */
)
{
    int  start_line, end_line;      /* starting and ending lines to be read */
    const WINDOW_TYPE *input_window_ptr; /* pointer to input window size    */


    /* Get the input window size.
       --------------------------*/
    input_window_ptr = get_input_window();

    /* If a range of lines has not been previously read, initialize the
       line location variables.
       ----------------------------------------------------------------*/
    if (!scan_man_ptr->data_valid)
    {
        /* Set the starting scan to match the direction the file must 
           be read in.
           ----------------------------------------------------------*/
        if (scan_man_ptr->scan_direction == READ_SCAN_FORWARD)
            start_line = input_window_ptr->start_line;
        else
            start_line = input_window_ptr->end_line - 
                scan_man_ptr->lines_in_buffer;

        /* Flag that the scan data is now valid.
           -------------------------------------*/
        scan_man_ptr->data_valid = 1;
    }
    else if (scan_man_ptr->scan_direction == READ_SCAN_FORWARD)
    { 
        /* Advance to the next set of lines, overlapping the lines
           with line_overlap scans to allow for the kernel size.
           -------------------------------------------------------*/
        start_line = scan_man_ptr->start_line +
            scan_man_ptr->lines_in_buffer - line_overlap;
    }
    else
    {
        /* Verify the scan direction is legal.
           -----------------------------------*/
        ASSERT(scan_man_ptr->scan_direction == READ_SCAN_REVERSE);

        /* Fall back to the next set of lines, with line_overlap as 
           above.
           --------------------------------------------------------*/
        start_line = scan_man_ptr->start_line -
            scan_man_ptr->lines_in_buffer + line_overlap;
    }

    /* Calculate range of lines to read.
       ---------------------------------*/
    end_line = start_line + scan_man_ptr->lines_in_buffer; 

    /* Make sure the start and end lines are legal.
       --------------------------------------------*/
    if (start_line < input_window_ptr->start_line)
        start_line = input_window_ptr->start_line;

    if (end_line > input_window_ptr->end_line)
        end_line = input_window_ptr->end_line;

    scan_man_ptr->start_line = start_line;

    /* Calculate cached values.
       ------------------------*/
    scan_man_ptr->last_valid_line = end_line - scan_man_ptr->kernel_size + 1;

    /* Log a status message to indicate how far the resampler is on 
       the current band.
       ------------------------------------------------------------*/
    c_lasmsg(FALSE, "resample-info",
            "Resampling input image lines %d through %d of %d",
            start_line+1, end_line+1, input_window_ptr->end_line+1);

    /* Read in the new lines.
       ----------------------*/
    if (read_input_lines(scan_man_ptr->in_image_ptr, start_line, 
                end_line-start_line, scan_man_ptr->image_buffer_ptr) 
            != E_SUCC)
        return (E_FAIL);

    return (E_SUCC);
}


/*****************************************************************************
FUNCTION NAME:  Get_Sample_Data

PURPOSE:        
Get_Sample_Data accepts a line/sample pair and returns
the address in memory where the sample resides.

RETURNS:
The address of the sample located at the line/sample is returned.  If the 
sample is not in memory or the kernel size will take exceed the samples
in memory, a NULL is returned.

NOTES:
The line and sample passed in should represent the upper left corner of the
resampling kernel location.

ALGORITHM REFERENCES:
none

*****************************************************************************/

const float *Get_Sample_Data
(
    const SCAN_MAN_TYPE *scan_man_ptr,
                 /* I: pointer to scan manager structure                    */
    int  line,   /* I: line number of the upper left corner of the kernel   */
    int  sample  /* I: sample number of the upper left corner of the kernel */
)
{
    int  index;  /* index to the line/sample in the extended image          */


    /* Verify the input parameters.
       ----------------------------*/
    ASSERT ((line >= 0) && (line < scan_man_ptr->lines_in_image)); 
    ASSERT ((sample >= 0) && (sample < scan_man_ptr->samples_per_line));

    /* Verify the requested line actually lies in memory.
       --------------------------------------------------*/
    if ((line >= scan_man_ptr->start_line) &&
            (line < scan_man_ptr->last_valid_line))
    {
        /* Calculate the index relative to the beginning of the band.
           ----------------------------------------------------------*/
        index = (line - scan_man_ptr->start_line) *
            scan_man_ptr->samples_per_line + sample;

        /* The index is in memory, so return the address.
           ----------------------------------------------*/
        return (&scan_man_ptr->image_buffer_ptr[index]);
    }

    /* Not in memory, so return NULL to indicate line/sample not available.
       --------------------------------------------------------------------*/
    return (NULL);
}


/*****************************************************************************
FUNCTION NAME:  More_Scans_To_Read

PURPOSE:        
More_Scans_To_Read checks to see if there are more lines to be read from
the input band.

RETURNS:
A non-zero value is returned if there are more lines to read.

NOTES:
This routine allows the details of when an image is done being read to be
isolated to the Scan Manager module.

ALGORITHM REFERENCES:
none

*****************************************************************************/

int More_Scans_To_Read
(
    const SCAN_MAN_TYPE *scan_man_ptr /*I: pointer to scan manager structure*/
)
{
    const WINDOW_TYPE *input_window_ptr; /* pointer to the input window data*/


    /* Get the input window.
       ---------------------*/
    input_window_ptr = get_input_window();

    if (scan_man_ptr->scan_direction == READ_SCAN_FORWARD)
    {
        if (!scan_man_ptr->data_valid || 
                ((scan_man_ptr->start_line + scan_man_ptr->lines_in_buffer) <
                 input_window_ptr->end_line))
        {
            /* Return that there are more scans to read.
               -----------------------------------------*/
            return(TRUE);
        }
        else
            return(FALSE);
    }
    else
    {
        /* Scanning in reverse.
           --------------------*/
        if (!scan_man_ptr->data_valid || 
                (scan_man_ptr->start_line > input_window_ptr->start_line))
        {
            /* Return that there are more scans to read.
               -----------------------------------------*/
            return(TRUE);
        }
        else
            return(FALSE);
    }
}
