/*****************************************************************************
NAME: terrain.c

PURPOSE:        
The terrain module implements the routines to calculate the effects terrain
has on an image.


ROUTINES:
    
    initialize_terrain
    cleanup_terrain
    get_terrain_adjustment
    calculate_MSSX_nadir_pixels         (local)
    calculate_nonstandard_nadir_pixels  (local)
    calculate_TMP_MSSP_nadir_pixels     (local)
    


DEVELOPMENT HISTORY:


NOTES:
- The user must first initialize the module by calling initialize_terrain
  for each new band resampled.
- The static variables located in this module are cached values that should
   not be written to by the get_terrain_adjustment routine.  If they are
   written to by the get_terrain_adjustment routine, it will cause
   unpredictable results when multithreading.  If this caching was not used,
   there would be a performance penalty.

ALGORITHM REFERENCES:
None

*****************************************************************************/

/*
**
** INCLUDE FILES 
**
*/

#include <stdlib.h>
#include <math.h>       /* trig functions             */
#include <stdio.h>      /* sprintf prototype          */
#include "resample.h"

/*
**
** MACRO CONSTANTS
**
*/

#define WEarth 7.292115642e-5 /* angular rotation rate of the earth in
                                 radians/sec                                */
#define RE 6378135.0          /* Radius of the Earth (in meters)            */
#define RE_SQ 4.06806061e13   /* Radius of the Earth squared (in meters
                                 squared)                                   */
#define MSSP_PIXSIZE 57.2     /* pixel sizes for each of the product types  */
#define MSSX_PIXSIZE 57.2     /* (in meters)                                */
#define TMP_PIXSIZE  28.5
#define NONS_PIXSIZE 28.5
#define TMP_INPUT_IMAGE_LINES    5965 /* TM-P product constants             */
#define TMP_INPUT_IMAGE_SAMPLES  6967 /* (pass time is in seconds)          */
#define TMP_PASS_TIME            25.1669
#define MSSP_INPUT_IMAGE_LINES   2983 /* MSS-P product constants            */
#define MSSP_INPUT_IMAGE_SAMPLES 3548 /* (pass time is in seconds)          */
#define MSSP_PASS_TIME           25.1669

/*
**
** TYPE DEFINITIONS
**
*/

typedef struct 
{
    double elevation;    /* in meters                    */
    double inclination;  /* orbit inclination in radians */
} SatInfo;     /* Information that varies by satellite.                     */

typedef struct 
{
    double rse;
    double ae;
    double ba;
} ELEV_CONST;  /* Structure for caching precomputed values for terrain
                  adjustment calculation.                                   */


/*
**
** LOCAL STATIC VARIABLES
**
*/

static SatInfo SatelliteInfo[6] = {
   {920000, 1.731541151 /* inclination in radians, 99.21  degrees */} ,
   {920000, 1.731541151 /* inclination in radians, 99.21  degrees */} ,
   {920000, 1.729917995 /* inclination in radians, 99.117 degrees */} ,
   {705000, 1.714262391 /* inclination in radians, 98.22  degrees */} ,
   {705000, 1.714262391 /* inclination in radians, 98.22  degrees */} ,
   {832000, 1.722639969 /* inclination in radians, 98.7   degrees */} 
};                      /* satellite information for Landsat 1-5            */
/* XXX Warning: Last element contains SPOT orbit numbers XXX */

/* NOTE:  These variables are initialized by the initialize_terrain
          routine and should be variables that are simply READ (and not
          written) by each resampling thread.  Any variables that are
          written by a resampling thread should have a separate copy for
          each thread.
------------------------------------------------------------------------*/

static ELEV_CONST *elevation_constants;
                              /* Array of precompute data that relies upon
                                 elevation.  It is indexed from DEM_min to
                                 DEM_max (inclusive of endpoints), so DEM_min
                                 must be subtracted from the current elevation
                                 before looking up the precomputed value.   */
static short *DEM_elevations; /* Array of DEM elevation values (in meters).
                                 It is DEM_lines x DEM_samples.             */
static int DEM_lines;         /* Lines in the DEM data                      */
static int DEM_samples;       /* samples across the DEM data                */
static int DEM_min;           /* minimum elevation in the DEM (from DDR or
                                 worst case estimate of -8000)              */
static int DEM_max;           /* maximum elevation in the DEM (from DDR or
                                 worst case estimate of 32000)              */
static double pixsize;        /* current input image pixel size             */
static double sat_alt;        /* satellite altitude                         */
static double ec_sat_alt;     /* earth centered satellite altitude (earth
                                 radius + sat altitude)                     */
static int *nadir_samples;    /* Array of nadir samples indexed by the input
                                 image line number.  The max index is
                                 nadir_sample_array_size - 1.               */
static int nadir_sample_array_size; /* number of entries in the nadir samples
                                       array.                               */
static float *scan_angles;    /* Array of precomputed scan angles (in radians)
                                 indexed by the off nadir sample.  The max
                                 index is scan_angle_array_size-1.  Note that
                                 fractional samples are rounded to the nearest
                                 sample.                                    */
static int scan_angle_array_size; /* number of entries in the scan angles
                                     array.                                 */

/*****************************************************************************
FUNCTION NAME:  calculate_MSSX_nadir_pixels

PURPOSE:        
calculate_MSSX_nadir_pixels calculates the approximate nadir pixel for each 
line in the input image for a MSSX product given the number of samples across
the input product.

RETURN VALUE:
Type = void

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

void calculate_MSSX_nadir_pixels
(
    int input_image_samples, /* I: samples across the input image           */
    int lines,               /* I: lines in input image                     */
    int *npix                /* O: pointer to nadir pixel array             */
)
{
    int i;                   /* line loop counter                           */
    int nadir_sample;        /* nadir sample                                */


    /* Set the nadir sample to the center sample of the input image.
       -------------------------------------------------------------*/
    nadir_sample = input_image_samples / 2;

    /* Calculate the nadir pixel for each line of the image.
       -----------------------------------------------------*/
    for(i=0; i < lines; i++) 
        npix[i] = nadir_sample;
}


/*****************************************************************************
FUNCTION NAME:  calculate_nonstandard_nadir_pixels

PURPOSE:        
calculate_nonstandard_nadir_pixels calculates the approximate nadir pixel for 
each line in the input image for a nonstandard product.  This routine calls
nadirline which computes the approximate nadir using the image corners.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          nadir pixels successfully calculated
E_FAIL          Error calculating the nadir pixels

NOTES:
- This routine is necessary for non-standard foreign image data where the 
  fill space and angle is not standard.
- A descending pass is assumed.

ALGORITHM REFERENCES:
none

*****************************************************************************/

int calculate_nonstandard_nadir_pixels
(
    const char *corners_filename,  /* I: name of corners labeled table file */
    int lines,                     /* I: lines in image                     */
    int *npix                      /* O: pointer to nadir pixel array       */
)
{
    double slope;               /* Slope of nadir line                      */
    double y_inter;             /* Y-intercept of nadir line                */
    int i;                      /* line loop counter                        */


    /* Calculate the slope and y-intercept of nadir line.
       --------------------------------------------------*/
    if (nadirline((char *)corners_filename, &slope, &y_inter) != E_SUCC)
        return (E_FAIL);

    /* Calculate the nadir pixel for each line of the image.
       -----------------------------------------------------*/
    for (i = 0; i < lines; i++)
        npix[i] = (int) ((i - y_inter) / slope);

    return(E_SUCC);
}


/*****************************************************************************
FUNCTION NAME:  calculate_TMP_MSSP_nadir_pixels

PURPOSE:        
calculate_TMP_MSSP_nadir_pixels calculates the approximate nadir pixel for 
each line in the input image for a nonstandard product given the scene center
latitude and the line numbers of interest.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC           nadir pixels successfully calculated
E_FAIL           Error calculating the nadir pixels

NOTES:
- The pass is assumed to be descending.

ALGORITHM REFERENCES:
none

*****************************************************************************/

int calculate_TMP_MSSP_nadir_pixels
(
    const char *input_image_name, /* I: input image name                    */
    float pixsize,          /* I: input image space pixel size              */
    double inclination,     /*I: inclination (in radians) of satellite orbit*/
    int lines,              /* I: lines in input image                      */
    int nlines,             /* I: number of lines in the P-product          */
    int nsamps,             /* I: number of samples in the P-product        */
    double passtime,        /* I: total seconds in the P-product            */
    int *npix               /* O: pointer to nadir pixel array              */
) 
{
    double heading;         /* Heading angle of satellite                   */
    double shift;           /* pixel shift due rotation of earth            */
    double slope;           /* pixel rate of change due rotation of earth   */
    double intercept;       /* starting location of nadir at line 1         */
    double sclatcos;        /* cosine of scene center latitude              */
    double SceneCenterLatitude;
                            /* latitude (in radians) of input image center  */
    int i;                  /* line loop counter                            */
    int line;               /* starting line                                */


    /* Get the scene center latitude.
       ------------------------------*/
    if (get_sclat((char *)input_image_name, &SceneCenterLatitude) != E_SUCC)
    {   
        c_lasmsg(FALSE, "resample-sclat",
                "Error calculating scene center latitude for input image.");
        return (E_FAIL);
    }

    /* Calculate the cosine of the scene center latitude.
       --------------------------------------------------*/
    sclatcos = cos(SceneCenterLatitude);

    /* Calculate the satellite heading.
       --------------------------------*/
    heading = asin(cos(inclination) / sclatcos );

    /* Calculate the pixels of shift that occur due to the rotation of the 
       earth over the collection time of the scene.
       -------------------------------------------------------------------*/
    shift = passtime * WEarth * RE * sclatcos  * cos(heading) / pixsize;

    /* Calculate the slope of the line defined by the nadir pixels.
       ------------------------------------------------------------*/
    slope = -shift / nlines;

    /* Calculate the nadir sample of the top line in the image.  The
       original equation from ivgeom was:
       offset = (nsamps - (shift + scenewidth)) / 2.0;
       intercept = offset + shift + scenewidth / 2.0 - 1.0;
       where scenewidth was 6458 for TM-P and 3250 for MSS-P.  However, this
       simplifies to the following equation.  NOTE: I'm not sure why the
       + 1.0 is present.
       ---------------------------------------------------------------------*/
    intercept = (nsamps + shift)/2.0 + 1.0;

    /* Set the starting line.  NOTE: Again, I'm not sure why this is set to
       1 instead of zero.
       --------------------------------------------------------------------*/
    line = 1;

    /* Calculate the nadir pixel for each line of the image.
       -----------------------------------------------------*/
    for(i=0; i < lines; i++) 
    {
        npix[i] = intercept + slope * line;
        line++;
    }

    return (E_SUCC);
}


/*****************************************************************************
FUNCTION NAME:  get_terrain_adjustment

PURPOSE:        
    get_terrain_adjustment corrects a P-products's sample location due to
    elevation given the location of the pixel in input and output space.  The
    output space pixel location is used to find the elevation at that point.
    The input space location is used to find the off nadir viewing angle.  The
    precalculated tables are used to speed up the calculation.

RETURN VALUE:
Type = float
The sample pixel offset due to elevation is returned.


NOTES:
- If the output space pixel location lies outside the DEM, the minimum
  elevation is used.  
- If the input pixel location lies outside the boundaries of the precomputed
  tables, the values are clamped to the endpoints in the tables.

ALGORITHM REFERENCES:
none

*****************************************************************************/

float get_terrain_adjustment
(
    double in_line,   /* I: input space line                                */
    double in_samp,   /* I: input space sample                              */
    int    out_line,  /* I: output space line                               */
    int    out_samp   /* I: output space sample                             */
)
{
    short elev;       /* elevation in meters                                */
    int   nadpix;     /* nadir sample location                              */

    double scanangle; /* scan angle associated with the input pixel location*/
    double dscan;     /* delta scan angle due to elevation                  */
    double dsample;   /* delta sample due to elevation                      */
    double eca;       /* earth centered angle                               */
    double zdd;       /* angle between the scan angle and the earth centered
                         angle to the point in input space                  */
    int index;        /* index into the scan_angles array                   */
    ELEV_CONST *elev_const_ptr; /* pointer to the precalculated values for
                                   the current elevation                    */


    /* Get the elevation associated with this point in output space.
       If the output space line/sample is outside the DEM, use DEM_min.
       ----------------------------------------------------------------*/
    if ((out_line >= DEM_lines) || (out_line < 0) ||
            (out_samp >= DEM_samples) || (out_samp < 0))
        elev = DEM_min;
    else
        elev = DEM_elevations[out_line * DEM_samples + out_samp];

    /* Get the nadir pixel for this line in input space.
       -------------------------------------------------*/

    /* Limit the input line to fall within the defined size.
       -----------------------------------------------------*/
    if (in_line > (nadir_sample_array_size - 1))
        in_line = nadir_sample_array_size - 1;
    else if (in_line < 0)
        in_line = 0;

    /* Get the nadir pixel for this line.
       ----------------------------------*/
    nadpix = nadir_samples[(int)(in_line + 0.5)];

    if (nadpix > in_samp)
    {
        /* negative scan angle
           -------------------*/
        index = (int)(nadpix - in_samp + 0.5);

        /* Calculate the central angle.
           ----------------------------*/
        eca = - index * pixsize / RE;

        /* Limit the index to the precomputed scan angles array size.
           ----------------------------------------------------------*/
        if (index >= scan_angle_array_size)
            index = scan_angle_array_size - 1;

        /* Get the scan angle.
           -------------------*/
        scanangle = - scan_angles[index];
    }
    else
    {
        /* positive scan angle
           -------------------*/
        index = (int)(in_samp - nadpix + 0.5);

        /* Calculate the central angle.
           ----------------------------*/
        eca = index * pixsize / RE;

        /* Limit the index to the precomputed scan angles array size.
           ----------------------------------------------------------*/
        if (index >= scan_angle_array_size)
            index = scan_angle_array_size - 1;

        /* Get the scan angle.
           -------------------*/
        scanangle = scan_angles[index];
    }

    /* Get the index to the precalculated values for the current elevation.
       --------------------------------------------------------------------*/
    if (elev < DEM_min)
        elev = DEM_min;
    else if (elev > DEM_max)
        elev = DEM_max;

    elev_const_ptr = &elevation_constants[elev - DEM_min];

    if(elev < 1000)
    {
        /* Use a linear approximation since the elevation is small.
           --------------------------------------------------------*/
        dscan = (elev_const_ptr->ae * scanangle) / elev_const_ptr->rse;
    }
    else 
    {
        /* Elevation is larger.  Use scanangle squared term approximation.
           ---------------------------------------------------------------*/
        dscan = (elev_const_ptr->ae * scanangle) / (elev_const_ptr->rse + 
                scanangle * scanangle * elev_const_ptr->ba) ;
    }

    /* Calculate the delta sample.
       ---------------------------*/
    zdd = asin(sin(scanangle + dscan) / RE * ec_sat_alt);
    dsample = RE * (zdd - eca - scanangle - dscan) / pixsize;
    return((float) dsample);
}


/*****************************************************************************
FUNCTION NAME:  initialize_terrain

PURPOSE:        
initialize terrain initializes the terrain module for the current band.
Tables of precomputed values are made to speed the actual calculation of the
terrain adjustment.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          terrain module successfully initialized
E_FAIL          Error initializing the terrain module

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

int initialize_terrain
(
    const char *input_image_name, /* I: input image name                    */
    const char *dem_name,         /* I: DEM name                            */
    PRODUCT_TYPE product_type,    /* I: product type                        */
    int input_image_lines,        /* I: lines in input image                */
    int input_image_samples,      /* I: samples across input image          */
    int output_image_lines,       /* I: lines in output image               */
    int output_image_samples,     /* I: samples across output image         */
    int satnum                    /* I: Landsat satellite number            */
)
{
    ELEV_CONST *current_elev_ptr; /* pointer to current elevation constants */
    int elev;               /* current elevation                            */
    int loop;               /* loop counter                                 */
    int max_nadir_pixel;    /* maximum off nadir pixel                      */
    int num_elevs;          /* number of elevations present in DEM          */
    int off_nadir_pixel;    /* off nadir pixel                              */
    int return_value;       /* value returned from called routines          */
    double eca;             /* earth central angle                          */
    double ec_sat_alt_sq;   /* ec_sat_alt squared                           */
    double ec_sat_alt_RE;   /* ec_sat_alt * RE                              */
    double ec_sat_alt_RE_2; /* ec_sat_alt_RE * 2                            */
    double half_ec_sat_alt; /* .5 * (earth radius + sat altitude)           */
    double los;             /* line of sight                                */
    OUTPUT_DATA_TYPE dem_dtype; /* DEM data type                            */


    /* If DEM not loaded, load it.
       ---------------------------*/
    if (DEM_elevations == NULL)
    {
        /* Read the DEM.
           -------------*/
        DEM_elevations =  get_elev(dem_name, &DEM_lines, &DEM_samples,
                &dem_dtype);
        if (DEM_elevations == NULL)
        {
            c_lasmsg(FALSE, "resample-open", "Error reading DEM from %s.",
                    dem_name);
            return (E_FAIL);
        }

        /* Make sure the data is 16-bit integers.
           --------------------------------------*/
        if (dem_dtype != INT16_OUTPUT)
        {
            free(DEM_elevations);
            DEM_elevations = NULL;
            c_lasmsg(FALSE, "resample-dtype",
                    "DEM does not contain 16-bit integer data.", dem_name);
            return(E_FAIL);
        }

        /* Assume the DEM elevation range is worst case.  
           NOTE: if min/max values are present in the DEM DDR they can 
           be used.
           -----------------------------------------------------------*/
        DEM_min = -8000;
        DEM_max = 32000;       
    }

    /* Make sure the DEM size matches the output image size.
       -----------------------------------------------------*/
    if ((DEM_lines != output_image_lines) ||
            (DEM_samples != output_image_samples))
    {
        c_lasmsg(FALSE, "resample-terrain",
                "DEM image size not equal to output image size.");
        free(DEM_elevations);
        DEM_elevations = NULL;
        return(E_FAIL);        
    }

    /* In the future, we may want to also check the upper left corner
       coordinate to make sure it agrees with the output image.
       --------------------------------------------------------------*/

    /* Allocate memory for the nadir pixel array (free the memory first in
       case a new image is being resampled).
       -------------------------------------------------------------------*/
    nadir_sample_array_size = input_image_lines;
    free(nadir_samples);
    nadir_samples = (int*)malloc(nadir_sample_array_size * sizeof(int));
    if (nadir_samples == NULL)
    {
        free(DEM_elevations);
        DEM_elevations = NULL;
        c_lasmsg(FALSE, "resample-alloc",
                "Error allocating memory for nadir samples");
        return (E_FAIL);
    }

    /* Verify the satellite number is valid.
       -------------------------------------*/
    if ((satnum < 1) || (satnum >6))
    {
        free(DEM_elevations);
        DEM_elevations = NULL;
        free(nadir_samples);
        nadir_samples = NULL;
        c_lasmsg(FALSE, "resample-satnum",
                "Satellite number of %d is not valid.", satnum);
        return (E_FAIL);
    }

    /* XXX */
    printf(" === Sattelite number: %d\n", satnum );

    /* Adjust the satellite number to be a zero based index.
       -----------------------------------------------------*/
    satnum--;

    /* Calculate the nadir sample array based on the product type.
       -----------------------------------------------------------*/
    switch (product_type)
    {
        case TM_P:
            return_value = calculate_TMP_MSSP_nadir_pixels(input_image_name, 
                    TMP_PIXSIZE, SatelliteInfo[satnum].inclination,
                    nadir_sample_array_size, TMP_INPUT_IMAGE_LINES,
                    TMP_INPUT_IMAGE_SAMPLES, TMP_PASS_TIME,
                    nadir_samples);
            pixsize = TMP_PIXSIZE;
            break;

        case MSS_P:
            return_value = calculate_TMP_MSSP_nadir_pixels(input_image_name, 
                    MSSP_PIXSIZE, SatelliteInfo[satnum].inclination,
                    nadir_sample_array_size, MSSP_INPUT_IMAGE_LINES,
                    MSSP_INPUT_IMAGE_SAMPLES, MSSP_PASS_TIME,
                    nadir_samples);
            pixsize = MSSP_PIXSIZE;
            break;

        case MSS_X:
            calculate_MSSX_nadir_pixels(input_image_samples, 
                    nadir_sample_array_size, nadir_samples);
            return_value = E_SUCC;          /* fake a success return value */
            pixsize = MSSX_PIXSIZE;
            break;

        case NON_STANDARD:
            return_value =
                calculate_nonstandard_nadir_pixels(get_corners_filename(),
                        nadir_sample_array_size, nadir_samples);
            pixsize = NONS_PIXSIZE;
            break;

        default:
            free(DEM_elevations);
            DEM_elevations = NULL;
            free(nadir_samples);
            nadir_samples = NULL;
            c_lasmsg(FALSE, "resample-terrain", "Unsupported product type.");
            return (E_FAIL);
    }

    if (return_value != E_SUCC)
    {
        free(DEM_elevations);
        DEM_elevations = NULL;
        free(nadir_samples);
        nadir_samples = NULL;
        c_lasmsg(FALSE, "resample-terrain",
                "Unable to calculate the nadir pixels.");
        return (E_FAIL);
    }

    /* Calculate the cached values.
       ----------------------------*/
    sat_alt = SatelliteInfo[satnum].elevation;
    ec_sat_alt = sat_alt + RE;
    half_ec_sat_alt = ec_sat_alt * .5;
    ec_sat_alt_sq = ec_sat_alt * ec_sat_alt;
    ec_sat_alt_RE = ec_sat_alt * RE;
    ec_sat_alt_RE_2 = ec_sat_alt_RE * 2.0;   

    /* Calculate the number of elevations present in the DEM.
       ------------------------------------------------------*/
    num_elevs = DEM_max - DEM_min + 1;

    /* Allocate memory and create the cached table of values that vary by 
       elevation.  Either the table from DEM_min thru DEM_max.  The table is 
       then accessed by subtracting DEM_min from the elevation.
       ---------------------------------------------------------------------*/
    free (elevation_constants);
    elevation_constants = (ELEV_CONST*)malloc(num_elevs * sizeof(ELEV_CONST));
    if (elevation_constants == NULL)
    {
        free(DEM_elevations);
        DEM_elevations = NULL;
        free(nadir_samples);
        nadir_samples = NULL;
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (E_FAIL);
    }

    /* Calculate the table of elevation related constants.
       ---------------------------------------------------*/
    current_elev_ptr = elevation_constants;
    for (elev = DEM_min; elev <= DEM_max; elev++, current_elev_ptr++)
    {
        current_elev_ptr->rse = RE*(sat_alt - elev);
        current_elev_ptr->ae = ec_sat_alt*elev;
        current_elev_ptr->ba = RE*half_ec_sat_alt*(((RE + elev) * 
                    ec_sat_alt)/RE_SQ - 1.0);
    }

    /* Find the maximum nadir pixel number.
       ------------------------------------*/
    max_nadir_pixel = nadir_samples[0];
    for (loop = 1; loop < nadir_sample_array_size; loop++)
    {
        if (nadir_samples[loop] > max_nadir_pixel)
            max_nadir_pixel = nadir_samples[loop];
        if ((input_image_samples - nadir_samples[loop]) > max_nadir_pixel)
            max_nadir_pixel = input_image_samples - nadir_samples[loop];
    }

    /* Add a little extra to the max nadir pixel to allow for it not being
       in the exact image center.
       -------------------------------------------------------------------*/
    max_nadir_pixel += 5;

    /* Set the scan angle array size.
       ------------------------------*/
    scan_angle_array_size = max_nadir_pixel;

    /* Allocate memory and create the table of cached scan angles indexed by 
       off nadir pixel.
       ---------------------------------------------------------------------*/
    free (scan_angles);
    scan_angles = (float *)malloc(scan_angle_array_size * sizeof(float));
    if (scan_angles == NULL)
    {
        free(DEM_elevations);
        DEM_elevations = NULL;
        free(nadir_samples);
        nadir_samples = NULL;
        free(elevation_constants);
        elevation_constants = NULL;
        c_lasmsg(FALSE, "resample-alloc", "Error allocating memory.");
        return (E_FAIL);    
    }

    for (off_nadir_pixel = 0; off_nadir_pixel < scan_angle_array_size; 
            off_nadir_pixel++)
    {
        eca = off_nadir_pixel * pixsize / RE;
        los = sqrt(ec_sat_alt_sq + RE_SQ - ec_sat_alt_RE_2 * cos(eca));
        scan_angles[off_nadir_pixel] = asin(RE * sin(eca)/los);
    }

    return(E_SUCC);
}


/*****************************************************************************
FUNCTION NAME:  cleanup_terrain

PURPOSE:        
cleanup_terrain frees the memory allocated by the terrain correction 
module.

RETURN VALUE:
Type = void

NOTES:
none

ALGORITHM REFERENCES:
none

*****************************************************************************/

void cleanup_terrain(void)
{
    free(elevation_constants);
    free(nadir_samples);
    free(DEM_elevations);
    free(scan_angles);
    elevation_constants = NULL;
    nadir_samples = NULL;
    DEM_elevations = NULL;
    scan_angles = NULL;
}
