/*****************************************************************************
NAME: transform_window

PURPOSE:        
The transform_window module contains the routines to transform windows defined
with line/samples between input and output space.


ROUTINES:
    
    transform_ow2iw

DEVELOPMENT HISTORY:


NOTES:
None

ALGORITHM REFERENCES:
None

*****************************************************************************/

/*
**
** INCLUDE FILES 
**
*/

#include <stdlib.h>
#include <math.h>
#include "debug.h"                   /* ASSERT macro                       */
#include "typlim.h"                  /* MAXWORD, MINWORD definitions       */
#include "resample.h"

/*------
--------
 Macros
--------
--------*/

#define MAX_LENGTH 1000  /* the maximum distance (in image space) allowed
                            between check points when determining min/max
                            line/sample values.                             */

/*****************************************************************************
FUNCTION NAME:  check_point

PURPOSE:
Given two output space line/sample pairs, check along the line connecting them
for new input space max/min values.

RETURNS:
The min/max line and sample of the input space window are returned.
The required line overlap is returned.

NOTES:
None

ALGORITHM REFERENCES:
none

*****************************************************************************/
void check_point
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/ 
    const int output_lines[2],   /* I: lines and samples of the window      */
    const int output_samples[2], /* I: corners in output space              */
    int *min_line_ptr,     /* I/O: min line number in transformed window    */
    int *max_line_ptr,     /* I/O: max line number in transformed window    */ 
    int *min_sample_ptr,   /* I/O: min sample number in transformed window  */
    int *max_sample_ptr,   /* I/O: max sample number in transformed window  */
    int *min_overlap,      /* I/O: amount of line overlap for top of
                                                                input space */
    int *max_overlap       /* I/O: amount of line overlap for bottom of
                                                                input space */
)
{
    int midpt_line, midpt_sample; /* output space midpoint                   */
    int new_out_lines[2];       /* new output line/sample pairs to check     */
    int new_out_samples[2];
    double d_input_line;        /* line and sample in input space            */
    double d_input_sample;
    double dtemp;               /* temporary value                           */


    /* Do we need to check the midpoint for this pair?  How large (in output
       space) is the point separation?
       ---------------------------------------------------------------------*/
    dtemp = sqrt((double)(output_lines[0] - output_lines[1])*
            (double)(output_lines[0] - output_lines[1]) +
            (double)(output_samples[0] - output_samples[1])*
            (double)(output_samples[0] - output_samples[1]));
    if (dtemp < MAX_LENGTH)
        return;

    /* Find the output space midpoint.
       -------------------------------*/
    midpt_line = (output_lines[0] + output_lines[1])/2;
    midpt_sample = (output_samples[0] + output_samples[1])/2;

    /* Convert the output space midpoint to input space.
       -------------------------------------------------*/
    if (ols2ils(grid_band_ptr, midpt_line, midpt_sample,
                &d_input_line, &d_input_sample) != E_SUCC)
    {
        d_input_line = *min_line_ptr;
        d_input_sample = *min_sample_ptr;
    }

    /* Will the midpoint change the min/max line and sample values?
       If so, check the amount of line overlap required.
       ------------------------------------------------------------*/
    if ((int)d_input_line < *min_line_ptr)
    {
        *min_overlap += *min_line_ptr - (int)d_input_line;
        *min_line_ptr = (int)d_input_line;
    }
    if ((int)d_input_line > *max_line_ptr)
    {
        *max_overlap += (int)d_input_line - *max_line_ptr;
        *max_line_ptr = (int)d_input_line;
    }
    if ((int)d_input_sample < *min_sample_ptr)
        *min_sample_ptr = (int)d_input_sample;
    if ((int)d_input_sample > *max_sample_ptr)
        *max_sample_ptr = (int)d_input_sample;

    /* Check the two new line segments created.
       ----------------------------------------*/
    new_out_lines[0] = midpt_line;
    new_out_samples[0] = midpt_sample;

    new_out_lines[1] = output_lines[0];
    new_out_samples[1] = output_samples[0];
    check_point(grid_band_ptr, new_out_lines, new_out_samples, min_line_ptr,
            max_line_ptr, min_sample_ptr, max_sample_ptr,
            min_overlap, max_overlap);

    new_out_lines[1] = output_lines[1];
    new_out_samples[1] = output_samples[1];
    check_point(grid_band_ptr, new_out_lines, new_out_samples, min_line_ptr,
            max_line_ptr, min_sample_ptr, max_sample_ptr,
            min_overlap, max_overlap);
}


/*****************************************************************************
FUNCTION NAME:  transform_ow2iw

PURPOSE:        
transform_ow2iw transforms a window in output space into a square window in 
input space that fully contains the output space window.

RETURNS:
The min/max line and sample of the input space window are returned.
The required line overlap is returned.


NOTES:
None

ALGORITHM REFERENCES:
none

*****************************************************************************/

int transform_ow2iw
(
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr, /* I: pointer to grid band*/ 
    const WINDOW_TYPE *output_window_ptr, /* I: pointer to window           */
    int kernel_size,      /* I: number of lines in kernel                   */
    int *min_line_ptr,    /* O: minimum line number in transformed window   */
    int *max_line_ptr,    /* O: maximum line number in transformed window   */ 
    int *min_sample_ptr,  /* O: minimum sample number in transformed window */
    int *max_sample_ptr,  /* O: maximum sample number in transformed window */
    int *line_overlap     /* O: amount of line overlap to use               */
)
{
    int min_overlap = 0;      /* number of overlap lines required along
                                 top of input image                         */
    int max_overlap = 0;      /* number of overlap lines required along
                                 bottom of input image                      */
    int loop;                 /* loop counter                               */
    int output_lines[4];      /* lines and samples of the window corners in */
    int output_samples[4];    /* output space                               */
    int output_lines_c[4];    /* lines and samples of the window border     */
    int output_samples_c[4];  /* centers in output space                    */
    int temp_line, temp_samp; /* temporary line/sample pair                 */
    double d_input_lines[4];  /* lines and samples in input space           */
    double d_input_samples[4];


    if (grid_band_ptr)
    {
        /* Verify the grid is a reverse grid since this routine is only
           written to handle a reverse grid.
           ------------------------------------------------------------*/
        ASSERT(grid_band_ptr->gridtype == REVERSE_GEOMETRIC_GRID);
    }

    /* Get the output window into a useable structure.
       -----------------------------------------------*/
    output_lines[0] = output_window_ptr->start_line;
    output_lines[1] = output_window_ptr->start_line;
    output_lines[2] = output_window_ptr->end_line - 1;
    output_lines[3] = output_window_ptr->end_line - 1;
    output_samples[0] = output_window_ptr->start_sample;
    output_samples[1] = output_window_ptr->end_sample - 1;
    output_samples[2] = output_window_ptr->start_sample;
    output_samples[3] = output_window_ptr->end_sample - 1;
    output_lines_c[0] = output_window_ptr->start_line;
    output_lines_c[1] = (output_window_ptr->end_line - 1 +
            output_window_ptr->start_line)/2.;
    output_lines_c[2] = output_lines_c[1];
    output_lines_c[3] = output_window_ptr->end_line - 1;
    output_samples_c[0] = (output_window_ptr->end_sample - 1 +
            output_window_ptr->start_sample)/2.;
    output_samples_c[1] = output_window_ptr->start_sample;
    output_samples_c[2] = output_window_ptr->end_sample - 1;
    output_samples_c[3] = output_samples_c[0];

    /* Convert each of the output space corners into an input space pixel.
       -------------------------------------------------------------------*/
    for (loop = 0; loop < 4; loop++)
        if (ols2ils(grid_band_ptr, output_lines[loop], output_samples[loop],
                &d_input_lines[loop], &d_input_samples[loop]) != E_SUCC)
            return E_FAIL;

    /* Find the min/max line and sample from the corners.
       --------------------------------------------------*/
    *min_line_ptr = d_input_lines[0];
    *max_line_ptr = d_input_lines[0];
    *min_sample_ptr = d_input_samples[0];
    *max_sample_ptr = d_input_samples[0];
    for (loop = 1; loop < 4; loop++)
    {
        if (d_input_lines[loop] < *min_line_ptr)
            *min_line_ptr = d_input_lines[loop];
        else if (d_input_lines[loop] > *max_line_ptr)
            *max_line_ptr = d_input_lines[loop];

        if (d_input_samples[loop] < *min_sample_ptr)
            *min_sample_ptr = d_input_samples[loop];
        else if (d_input_samples[loop] > *max_sample_ptr)
            *max_sample_ptr = d_input_samples[loop];
    }

    /* Check along the output space boundaries for input space lines and
       samples that modify the max/min values.
       -----------------------------------------------------------------*/
    /* Top */
    check_point(grid_band_ptr, output_lines, output_samples, min_line_ptr,
            max_line_ptr, min_sample_ptr, max_sample_ptr,
            &min_overlap, &max_overlap);
    /* Bottom */
    check_point(grid_band_ptr, &output_lines[2], &output_samples[2],
            min_line_ptr, max_line_ptr, min_sample_ptr, max_sample_ptr,
            &min_overlap, &max_overlap);
    /* Exchange UR and LL output points. */
    temp_line = output_lines[1];
    temp_samp = output_samples[1];
    output_lines[1] = output_lines[2];
    output_samples[1] = output_samples[2];
    output_lines[2] = temp_line;
    output_samples[2] = temp_samp;
    /* Left */
    check_point(grid_band_ptr, output_lines, output_samples, min_line_ptr,
            max_line_ptr, min_sample_ptr, max_sample_ptr,
            &min_overlap, &max_overlap);
    /* Right */
    check_point(grid_band_ptr, &output_lines[2], &output_samples[2],
            min_line_ptr, max_line_ptr, min_sample_ptr, max_sample_ptr,
            &min_overlap, &max_overlap);

    /* Set the line overlap.
       ---------------------*/
    *line_overlap = DEFAULT_LINE_OVERLAP +
        max(min_overlap, max_overlap);

    /* Verify the line_overlap is large enough to accomodate the kernel size.
       ----------------------------------------------------------------------*/
    if(*line_overlap < (kernel_size - 1))
    {
        /* Need to increase it to one less than the kernel size to
           get enough lines in the buffer to process.
           -------------------------------------------------------*/
        *line_overlap = kernel_size - 1;
    }

    /* Add one to the max values to account for integer truncation.
       ------------------------------------------------------------*/
    *max_line_ptr += 1;
    *max_sample_ptr += 1;

    return E_SUCC;
}
