/*****************************************************************************
NAME                              use_ddr_info

PURPOSE    Reads projection & framing information from an image DDR

ALGORITHM DESCRIPTION
        Retrieve the name of the input image via getin_nocheck()
        Get the DDR information
        If DDR information is valid,
           Pack output projection parameters from the DDR projection
                information
        Return
*****************************************************************************/
#include <string.h>
#include "resample.h"
#include "util.h"


int use_ddr_info
(
    struct PARBLK *vblk, /* I: TAE parameter block                          */
    struct PROJ *pro,    /* O: Structure containing projection parameters   */
    long projflag,       /* I: Projection flag: 0 = projection not needed,
                                                1 = projection info required*/
    double *corners,     /* O: Corner projection coordinates
                                  8 members (UL, LL, UR, LR x X,Y)          */
    long *image,         /* O: Corner image coordinates
                                  8 members (UL, LL, UR, LR x X,Y)          */
    long in_wind[][4],   /* I: Image window specification                   */
    char *flag,          /* I: input/output flag                            */
    double *pixy,        /* O: pixel size in y direction                    */
    double *pixx         /* O: pixel size in x direction                    */
)
{
    struct DDR ddr;           /* DDR structure                              */
    long i;                   /* Loop counters                              */
    long zero = 0;            /* Constant of zero                           */
    long imglim = MAXIMG;
    long num_img;             /* Number of images input:  0 or 1            */
    long bands[MAXIMG][MAXBND+1]; /* Band numbers                           */
    long nbands;              /* Number of bands                            */
    long totbands;            /* Total number of bands                      */
    long same;                /* Same window flag                           */
    long qflag[MAXIMG][2];    /* Option flags                               */
    char in_image[MAXIMG][CMLEN]; /* Image file name                        */
    char tdum[MAXIMG][CMLEN];     /* Tae name                               */


    /* Check for projection information, and place it in the pro 
       structure if it exists.
       ---------------------------------------------------------*/
    if(c_getin_nocheck(vblk, flag, &zero, tdum, in_image, &num_img, in_wind,
                bands, &nbands, &totbands, &imglim, &same, qflag) != E_SUCC)
        return(E_FAIL);
    if (c_getddr(in_image[0], &ddr) != E_SUCC)
        return(E_FAIL);

    image[0] = image[1] = 1;
    image[2] = ddr.nl;
    image[3] = 1;
    image[4] = 1;
    image[5] = ddr.ns;
    image[6] = ddr.nl;
    image[7] = ddr.ns;

    if (projflag && ((ddr.valid[0] == INVAL) || (ddr.valid[3] == INVAL) ||
                (ddr.valid[4] == INVAL)))
    {
        c_lasmsg(FALSE, "resample-noproj",
                "Projection information not available from DDR.");
        return(E_FAIL);
    }
    else
    {
        pro->proj_code = ddr.proj_code;
        if (ddr.valid[1] == INVAL)
            pro->zone_code = 62;
        else
            pro->zone_code = ddr.zone_code;
        if (ddr.valid[2] == INVAL)
            pro->datum_code = 0;
        else
            pro->datum_code = ddr.datum_code;

        pro->units = c_string2unit(ddr.proj_units);
        if (pro->units == E_FAIL)
        {
            c_lasmsg(FALSE, "resample-warn", "%s are unsupported units",
                    ddr.proj_units);
        }

        for (i = 0; i < 15; i++)
            pro->proj_coef[i] = ddr.proj_coef[i];
        if (ddr.valid[6] == INVAL)
            for (i = 0; i < 8; i++)
                corners[i] = 0.0;
        else
        {
            corners[0] = ddr.upleft[0];
            corners[1] = ddr.upleft[1];
            corners[2] = ddr.loleft[0];
            corners[3] = ddr.loleft[1];
            corners[4] = ddr.upright[0];
            corners[5] = ddr.upright[1];
            corners[6] = ddr.loright[0];
            corners[7] = ddr.loright[1];
        }
    }

    *pixx = ddr.pdist_x;
    *pixy = ddr.pdist_y;

    return(E_SUCC);
}
