/*****************************************************************************
NAME:           warp

PURPOSE:
  The warp module is a driver for the resampling process.  It initializes the 
  input scan manager, reads the SLO data, and creates the multiple processor
  threads to perform the resampling.

DEVELOPMENT HISTORY:
  Baselined from revision 1.27 of warp.c (IAS Resampler)

NOTES:
  none

ALGORITHM REFERENCES:
  RESAMPLE design document

*****************************************************************************/

#include <string.h>
#include <math.h>
#include "resample.h"
#include "debug.h"                   /* ASSERT macro                        */

static double min_range,              /* local versions of the minimum and  */
              max_range;              /* maximum output range, adjusted for
                                         the currently selected output type.*/
static OUTPUT_DATA_TYPE output_data_type;
                                      /* local copy of the output data type */

int warp
( 
    const gxx_GEOM_GRID_TYPE *grid_ptr, /* I: pointer to grid               */
    int gridflg,                  /* I: grid flag:
                                              0 = no grid, use proj. params.
                                              1 = use grid                  */
    int iband,                    /* I: current band (indexed)              */
    int band,                     /* I: current band                        */
    RESAMPLE_TYPE resampling_type,/* I: type of resampling                  */
    struct FDESC *otfdsc,         /* I: output file descriptor              */
    int *lines_to_buffer,         /* I: number of input lines to buffer     */
    int  procs_to_use,            /* I: number of processors to use         */
    int  *max_output_image_buffer, /* I/O: max mem in MB to use for buffering
                                           the output image                 */
    int  buffer_full_output_image,/* I: flag to indicate entire output image
                                        should be kept in memory during
                                        resampling                          */
    double *minpix,               /* I/O: minimum pixel value for band      */
    double *maxpix                /* I/O: maximum pixel value for band      */
)
{
    const WINDOW_TYPE *output_window_ptr = get_output_window(iband);
                                  /* ptr to output window geometry          */
    SCAN_MAN_TYPE *scan_man_ptr;  /* pointer to scan manager structure      */
    RESAMPLE_THREAD_PARAMS params;/* parameters to pass to resampling
                                     threads                                */
    const gxx_GEOM_GRID_BAND_TYPE *grid_band_ptr; /* pointer to grid band   */

    int active_lines;             /* number of lines active in the output
                                     image                                  */
    int in_nl;                    /* input image number of lines            */
    int in_ns;                    /* input image number of samples          */
    int min_active_lines;         /* minimum number of lines active in
                                     the output image                       */
    int min_lines_to_buffer;      /* minimum number of input lines to buffer*/
    int num_cpus = GET_NUM_PROCESSORS(); /* number of processors            */
    int pass;                     /* counter for number of passes used
                                     while determining lines to buffer and
                                     cache size                             */
    int rs;                       /* return status of calls                 */
    int t_active_lines;           /* temporary storage for number of lines
                                     active in the output image             */


    if (gridflg)
        grid_band_ptr = &grid_ptr->gridbands[iband];
    else
    {
        grid_band_ptr = NULL;

        /* Initialize the output space to input space conversion routine.
           --------------------------------------------------------------*/
        if (initialize_ols2ils() != E_SUCC)
            return E_FAIL;
    }

    if (procs_to_use <= num_cpus)
        num_cpus = procs_to_use;

    /* Set the parameters to pass to the resampling threads.
       -----------------------------------------------------*/
    params.grid_band_ptr = grid_band_ptr;
    params.band = iband;
    params.thread_error = 0;
    params.output_data_type = output_data_type;
    params.minpix = *minpix;
    params.maxpix = *maxpix;
    params.last_line_index_changed = FALSE;

    /* We may need to make a couple of passes while determining possible
       changes to the number of lines to buffer and the cache size.

       The user *probably* wants to run their job regardless of how many
       input lines are buffered or how much system memory is used.
       Therefore it has been decided that RESAMPLE will override the
       user inputs for BUFLINES and CACHESZ if they are too restrictive
       for the job.  The procedure is as follows:

       Pass 1:
       
       (Initialize_Scan_Manager)
       The minimum number of input lines to buffer (double the
       hysteresis/overlap) is computed.
       If BUFLINES is less than the minimum, BUFLINES is set to the
            minimum.
            
       (check_buffers)
       The number of active output lines is computed for both BUFLINES and
       the minimum number of input lines.

       (initialize_output_image_buffer/calculate_optimum_buffer)
       The amount of memory required to store the active output lines is
       computed.
       The maximum number of output lines that can be stored based on
       CACHESZ is computed.
       If the amount of memory required is less than the amount
            allowed, the storage is allocated and life goes on.
       Else the amount necessary for storing the minimum number of active
            lines is computed.
            If the minimum necessary is still more than allowed,
                 CACHESZ is increased to accomodate the minimum, the
                 storage is allocated, and life goes on.
            Else, a new value for the number of active output lines
                 is computed based on the maximum memory allowed.  The
                 storage is not allocated and the new active line
                 value is returned to warp.

       (warp)
       If no storage has been allocated, check the active lines.
            If the number of active lines has changed, compute a new value
                 for BUFLINES.  Make sure it is at least the minimum.
                 Go to Pass 2.
            Else exit with failure.
       Else go on with the show.

       Pass 2:

       (Initialize_Scan_Manager)
       Reinitialize scan manager with new BUFLINES value.
            
       (check_buffers)
       Recompute active output lines based on new BUFLINES.

       (initialize_output_image_buffer/calculate_optimum_buffer)
       If the amount of memory required is less than the amount
            allowed, the storage is allocated and life goes on.
       Else CACHESZ is increased to accomodate the number of active lines
            (should be a small increase), the storage is allocated, and
            life goes on.

       (warp)
       If no storage has been allocated, exit with failure.
       Else go on with the show.
       -----------------------------------------------------------------*/
    for (pass=1; pass<=2; pass++)
    {
        /* Prepare scan manager for use.
           -----------------------------*/
        scan_man_ptr = Initialize_Scan_Manager(grid_band_ptr, iband, band,
                lines_to_buffer,
                &min_lines_to_buffer, &in_ns,
                &in_nl, get_lines_in_kernel());
        if (scan_man_ptr == NULL)
            return (E_FAIL);

        params.scan_man_ptr = scan_man_ptr;
        params.in_nl = in_nl;
        params.in_ns = in_ns;

        /* Determine which direction resampling should occur in.  Assume
           it is in the same direction the lines are being read.
           -------------------------------------------------------------*/
        if (scan_man_ptr->scan_direction == READ_SCAN_FORWARD)
            params.resample_dir = 1;
        else
            params.resample_dir = -1;

        if (grid_band_ptr)
        {
            /* Invert the scan direction if the image is rotated to more
               than 90 degrees.
               ---------------------------------------------------------*/
            if (grid_band_ptr->in_lines[0] >
                    grid_band_ptr->in_lines[grid_band_ptr->num_out_samps *
                    (grid_band_ptr->num_out_lines - 1)])
                params.resample_dir *= -1;
        }

        /* Calculate the number of lines that will be active in the output
           image.
           ---------------------------------------------------------------*/
        if (check_buffers(iband, grid_band_ptr, in_nl, in_ns,
                    *lines_to_buffer, min_lines_to_buffer,
                    &active_lines, &min_active_lines) != E_SUCC)
        {
            Cleanup_Scan_Manager(scan_man_ptr);
            return(E_FAIL);
        }

        /* Create the output image buffer.
           -------------------------------*/
        t_active_lines = active_lines;
        params.output_image_buffer_ptr =
            initialize_output_image_buffer(&t_active_lines, min_active_lines,
                    iband, otfdsc, num_cpus,
                    params.resample_dir,
                    max_output_image_buffer,
                    buffer_full_output_image, pass);
        if (params.output_image_buffer_ptr == NULL)
        {
            Cleanup_Scan_Manager(scan_man_ptr);
            if ((t_active_lines != active_lines) && pass == 1)
            {
                *lines_to_buffer = (int)((float)*lines_to_buffer*
                        (t_active_lines - min_active_lines)/
                        (active_lines - min_active_lines));
                if (*lines_to_buffer < min_lines_to_buffer)
                    *lines_to_buffer = min_lines_to_buffer;

                c_lasmsg(FALSE, "resample-buflines",
                        "BUFLINES too large for CACHESZ.  Reducing BUFLINES"
                        " to %d.", *lines_to_buffer);
            }
            else
                return (E_FAIL);
        }
        else
            break;
    }

    /* Initialize the last line completed indices to zero so the resampling
       starts at the edge of the image.
       --------------------------------------------------------------------*/
    params.last_line_complete_index = 0;
    params.returned_last_line_complete_index = 0;

    /* Make the terrain table if it is needed.
       ---------------------------------------*/
    if (do_terrain_correction())
    {
        /* Initialize the terrain table.
           -----------------------------*/
        if (initialize_terrain(get_input_image_filename(), get_DEM_filename(),
                    get_product_type(), in_nl, in_ns,
                    grid_band_ptr->output_image_lines,
                    grid_band_ptr->output_image_samps,
                    get_sat_num())
                == E_FAIL)
        {
            cleanup_output_image_buffer(params.output_image_buffer_ptr, 0);
            Cleanup_Scan_Manager(scan_man_ptr);
            return (E_FAIL);
        }
    }

    /* Set up the selected resampling algorithm.
       -----------------------------------------*/
    switch (resampling_type)
    {
        case CC_RESAMPLING:
            /* presently disabled case MTF_RESAMPLING: */
            /* Initialize the cubic convolution resampling module.
               ---------------------------------------------------*/
            initialize_cc_resample(grid_band_ptr, params.scan_man_ptr,
                    params.output_image_buffer_ptr->sample_reached_ptr,
                    params.in_ns, output_window_ptr->end_sample,
                    min_range, max_range);
            if (grid_band_ptr)
                params.resample_function = cc_resample;
            else
                params.resample_function = cc_resample_nogrid;
            break;

        case BI_RESAMPLING:
            /* Initialize the bilinear resampling module.
               ------------------------------------------*/
            initialize_bi_resample(grid_band_ptr, params.scan_man_ptr,
                    params.output_image_buffer_ptr->sample_reached_ptr,
                    params.in_ns, output_window_ptr->end_sample,
                    min_range, max_range);
            if (grid_band_ptr)
                params.resample_function = bi_resample;
            else
                params.resample_function = bi_resample_nogrid;
            break;

        case NN_RESAMPLING:
            /* Initialize the nearest neighbor resampling module.
               --------------------------------------------------*/
            initialize_nn_resample(grid_band_ptr, params.scan_man_ptr,
                    params.output_image_buffer_ptr->sample_reached_ptr,
                    output_window_ptr->end_sample,
                    min_range, max_range);
            if (grid_band_ptr)
                params.resample_function = nn_resample;
            else
                params.resample_function = nn_resample_nogrid;
            break;

        case TABLE_RESAMPLING:
            /* Initialize the generic table resampling module.
               -----------------------------------------------*/
            initialize_table_resample(grid_band_ptr, params.scan_man_ptr,
                    params.output_image_buffer_ptr->sample_reached_ptr,
                    params.in_ns, output_window_ptr->end_sample,
                    min_range, max_range);
            if (grid_band_ptr)
                params.resample_function = table_resample;
            else
                params.resample_function = table_resample_nogrid;
            break;

        default:
            params.thread_error = -1;
    }

    /* Use all the available processors to process the band.
       -----------------------------------------------------*/
    rs = SET_NUM_THREADS_TO_FORK(num_cpus);
    if (rs != 0)
    {
        cleanup_output_image_buffer(params.output_image_buffer_ptr, 0);
        Cleanup_Scan_Manager(scan_man_ptr);
        return (E_FAIL);
    }

    while (More_Scans_To_Read(scan_man_ptr))
    {
        if (Read_Next_Scan_Set(scan_man_ptr) == E_FAIL)
        {
            cleanup_output_image_buffer(params.output_image_buffer_ptr, 0);
            Cleanup_Scan_Manager(scan_man_ptr);
            return (E_FAIL); 
        }

        rs = FORK_THREADS(resample_image, (void *)&params);

        if ((rs != 0) || (params.thread_error != 0))
        {
            if (rs == 0)
            {
                c_lasmsg(FALSE,"resample-thread",
                        "Error encountered in resampling thread %d.",
                        params.thread_error-1);
            }

            cleanup_output_image_buffer(params.output_image_buffer_ptr, 0);
            Cleanup_Scan_Manager(scan_man_ptr);
            KILL_FORKED_THREADS();
            return (E_FAIL);
        }

        /* Update the last line complete index.
           ------------------------------------*/
        params.last_line_complete_index = 
            params.returned_last_line_complete_index;

        /* Unset the last_line_index_changed flag for next set of threads
           --------------------------------------------------------------*/
        params.last_line_index_changed = FALSE;
    }

    /* Close the output image buffer.
       ------------------------------*/
    if (cleanup_output_image_buffer(params.output_image_buffer_ptr, 1)
            == E_FAIL)
    {
        Cleanup_Scan_Manager(scan_man_ptr);
        return (E_FAIL);
    }

    /* Cleanup the threads that were created
       -------------------------------------*/
    KILL_FORKED_THREADS();

    /* Close the scan manager.
       -----------------------*/
    Cleanup_Scan_Manager(scan_man_ptr);

    /* Return the min and max pixel values for this band.
       --------------------------------------------------*/
    *minpix = params.minpix;
    *maxpix = params.maxpix;

    return (E_SUCC);
}

/*****************************************************************************
NAME:           setup_resampf1p_output

PURPOSE:
setup_resampf1p_output configures the resampf1p module to generate the
output image with the requested output data type and dynamic range.

RETURN VALUE:
Nothing is returned by this routine

NOTES:
- This routine uses the parameters read from the ODL parameter file to
  create locally cached copies in the most efficient format for this module.
- This routine does not use the min/max values defined for the types in
  limits.h since those are machine specific.  The min/max values here are
  data dependent and cannot change for different machines.

ALGORITHM REFERENCES:
none

*****************************************************************************/

void setup_resampf1p_output(void)
{
    output_data_type = get_output_data_type();

    min_range = get_output_range_minimum();
    max_range = get_output_range_maximum();

    if (output_data_type == BYTE_OUTPUT)
    {
        /* Byte output, so adjust the min and max accordingly.
           ---------------------------------------------------*/
        if (min_range < 0.0)
            min_range = 0.0;
        else
            min_range = ceil(min_range);
        if (max_range > 255.0)
            max_range = 255.0;
        else
            max_range = floor(max_range);
    }
    else if (output_data_type == INT16_OUTPUT)
    {
        /* 2 byte integer output, so adjust the min and max accordingly.
           -------------------------------------------------------------*/
        if (min_range < -32768.0)
            min_range = -32768.0;
        else
            min_range = ceil(min_range);
        if (max_range > 32767.0)
            max_range = 32767.0;
        else
            max_range = floor(max_range);
    }
    else if (output_data_type == INT32_OUTPUT)
    {
        /* Byte output, so adjust the min and max accordingly.
           ---------------------------------------------------*/
        if (min_range < -2147483648.0)
            min_range = -2147483648.0;
        else
            min_range = ceil(min_range);
        if (max_range > 2147483647.0)
            max_range = 2147483647.0;
        else
            max_range = floor(max_range);
    }
    else
    {
        /* Must be float output.  If it's not, an error should have been
           obtained parsing the input, so just assert here during
           development.  No range manipulations are needed for the float
           case.
           -------------------------------------------------------------*/
        ASSERT (output_data_type == FLOAT_OUTPUT);
    }
}
