/*****************************************************************************
NAME				  ROTATE

PURPOSE   Extracts input image window from full input image specification 

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   J. Bay	   3/84        SAR      Original development
  2.0      R. Wolfe	  12/84	       SAR
  5.0      D. Steinwand    4/89        EDC      LAS 5.0 development
						C conversion
  5.01     D. Steinwand    8/89        EDC      Fixed nl,ns bug

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   ROTATE must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
	Get the name of the input file & it's window specification
        Get output image name and check if it exists
        Get the name of the output mapping grid file and check if it exists
	If multiple windows of different sizes are entered, abort
	Find the number of output lines and samples
	Send the image name (without window spec) back to TAE
	Send the window spec back to TAE
	Send the number of lines in the output space back to TAE
	Send the number of samples in the output space back to TAE
	Write completion message & exit
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"

#define PI	3.14159265359
main()
{
float	rot;				/* Angle of rotation in radians */
long    fatal = -1;			/* Fatal error flag for c_errmsg */
long	zero = 0;			/* constant zero */
long	one = 1;			/* constant one */
long	std_out;			/* standard output (lun) for inlas */
long	num_img;			/* Number of images entered (0 or 1) */
static long bands[MAXIMG][MAXBND+1];	/* Band numbers */
long	nbands;				/* Number of bands */
long	in_wind[MAXIMG][4];		/* Window entered with input image */
long	totbands;			/* Total number of bands entered */
long 	imglim = MAXIMG;		/* Maximum number of images allowed */
long 	same;				/* Window flag */
long	qflag[2];			/* Image option flag */
long 	nl,ns;				/* # of lines and samples in output */
long fail = -1;           /* Flag to c_errmsg -- terminate processing */
static char taenam[MAXIMG][CMLEN];	/* TAE image name of input image */
char 	*strptr[1];			/* Array of pointers to char */
char    outtae[CMLEN];                  /* output image file name */
char    outhost[CMLEN];                  /* output image file name */
static char in_image[MAXIMG][CMLEN];	/* Host image name */
struct  PARBLK  vblk;			/* TAE v-block */
struct  VARIABLE *var,			/* TAE variable value... */
                *p_find();		/* TAE routine */

/* Initialize LAS processing
 --------------------------*/
c_inlas(&vblk, &std_out);

/* Get the name of the input image file & it's window
 ---------------------------------------------------*/
if(c_getin(&vblk,"IN",&zero,taenam,in_image,&num_img,in_wind,bands,&nbands,
            &totbands,&imglim,&same,qflag) != E_SUCC)
	c_errmsg("Fatal error encountered","rotate-fatal",&fatal);
if (same != 1)
	{
	c_errmsg("Input images must be the same size","rotate-size",&one);
	c_errmsg("Fatal error encountered","rotate-fatal",&fatal);
	}

/* Get output image name and check if it exists
  --------------------------------------------*/
if(c_getout(&vblk,"OUT",outtae,outhost) != E_SUCC) 
    c_errmsg("Fatal error encountered","rotate-fatal",&fail);

/* Check if the output mapping grid file exists
 ---------------------------------------------*/
if((c_lsstat("rotate_grid.grid",&zero)) == E_SUCC)
  {
  c_errmsg("Error: rotate.grid;grid already exists", "rotate-exists", &one);
  c_errmsg("Fatal error encountered", "rotate-fatal", &fatal);
  } 

/* Calculate the number of output lines and samples from the rotation
   angle and the number of input lines and samples
  -----------------------------------------------*/
var = p_find(&vblk,"ROTANG");
rot = RVAL(*var,0) * PI / 180.0;
nl = (long)(in_wind[0][2]*fabs(cos(rot))+in_wind[0][3]*fabs(sin(rot))+0.5);
ns = (long)(in_wind[0][2]*fabs(sin(rot))+in_wind[0][3]*fabs(cos(rot))+0.5);

/* Send back to VBLK
  -----------------*/
strptr[0] = &(taenam[0][0]);
if (q_string(&vblk, "NOWIND", 1, strptr, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","rotate-fatal",&fatal);

if (q_intg(&vblk, "IWIN", 4, in_wind, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","rotate-fatal",&fatal);

if (q_intg(&vblk, "NL", 1, &nl, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","rotate-fatal",&fatal);

if (q_intg(&vblk, "NS", 1, &ns, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","rotate-fatal",&fatal);

if (q_out(&vblk) != SUCCESS)
	c_errmsg("Fatal error encountered","rotate-fatal",&fatal);

/* Write completion message and exit program
 ------------------------------------------*/
c_complt("ROTATE", "", &zero);
}
