/*****************************************************************************
NAME				  MAKE_GRID

PURPOSE    Grid the input and output space and write mapping file to disk

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   R. White        4/82        CSC      Original Development
  4.0      S. Kondal       7/84        SAR     
  5.0      D. Steinwand   12/88        EDC      LAS 5.0 development,
						C conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   ROTRNSCL must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION
	Create the grid
	Apply polynomial to calcualate input space coordinates
	Place polynomial coefficients in mapping grid record--
		Note the swap!!!
	Pack misc mapping grid fields not used by ROTRNSCL
	Update projection related fields in the mapping grid, if possible
	Write the geometric mapping grid to disk
	Return
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "local_protos.h"


typedef struct
{
    double aa[3];  /* output space line/sample to input space line coeffs */
    double bb[3];  /* output space line/sample to input space sample coeffs */
} OLS2ILS_PARAMS;
/* structure used to pass parameters to the local ols2ils routine */

static int ols2ils
(
    void *data,     /* I: pointer to a OLS2ILS_PARAMS structure for the 
                          parameters specific to this gridding application */
    int oline,      /* I: output space line */
    int osamp,      /* I: output space sample */
    double *iline,  /* O: input space line */
    double *isamp   /* O: input space sample */
)
{
    double d_oline, d_osamp;
    long degree = 1;
    OLS2ILS_PARAMS *parm = (OLS2ILS_PARAMS *)data;

    d_oline = (double)oline;
    d_osamp = (double)osamp;

    *iline = c_eval(&degree, parm->aa, &d_osamp, &d_oline);
    *isamp = c_eval(&degree, parm->bb, &d_osamp, &d_oline);

    return (E_SUCC);
}


int make_grid
(
    long nlines,		/* Number of lines in the output grid (image) */
    long nsamps,		/* Number of samples in the output grid (image) */
    double coef[6],		/* Transformation coefficients */
    char *name,         /* Grid file name */
    char *in_image,		/* Input image file name */
    long num_img,		/* Number of images input:  0 or 1 */
    long in_wind[4]     /* Image window specification */
)
{
    struct GEOGRID grid;    /* Geometric mapping grid */
    struct DDR ddr;	        /* DDR structure */
    long i;	                /* loop counters */
    long one = 1;           /* constant of one for c_errmsg routine */
    double proj_coord[8];   /* Corner coordinates in projection units */
    double cc[6];           /* First order polynomial for corners calculation */
    double tol = 0.015625;  /* Grid reduction tolerance */
    double aa[3],bb[3];     /* Temporary polynomial buffers for c_eval() */
    double line,samp;       /* Line and sample coord in input of output 
                               corners */
    long prt_flag;          /* Print flag for the c_rtcoef routine */
    long image_coord[8];    /* Corner coordinates in line, sample */
    OLS2ILS_PARAMS ols2ils_data; /* parameters to ols2ils */
    int  grid_window[4];    /* window for constructing the grid */

    /* set the grid window to be the full size specified */
    grid_window[0] = 1;  /* sl */
    grid_window[1] = 1;  /* ss */
    grid_window[2] = nlines;
    grid_window[3] = nsamps;

    /* setup the coefficients */
    aa[0] = ols2ils_data.aa[0] = coef[2];
    aa[1] = ols2ils_data.aa[1] = coef[1];
    aa[2] = ols2ils_data.aa[2] = coef[0];
    bb[0] = ols2ils_data.bb[0] = coef[5];
    bb[1] = ols2ils_data.bb[1] = coef[4];
    bb[2] = ols2ils_data.bb[2] = coef[3];

    /*create the grid, allowing the gridding function to pick the grid density*/
    if (c_makegrid(grid_window, 0, 0, ols2ils, &ols2ils_data, tol, "ROTRNCSL",
            0, &grid) != E_SUCC)
    {
        c_lasmsg(FALSE,"makegrid-fail","Error returned from c_makegrid");
        return (E_FAIL);
    }

    /* Put the polynomial used in the calculations into the geometric mapping
       grid record.  Note:  The polynomial is stored a0 + a1x + a2y !!!!!!!
       ---------------------------------------------------------------------*/
    strcpy(grid.model,"POLYNOMIAL");
    grid.ncoeff = 3;
    grid.coeffs = (double *)malloc(6 * sizeof(double));
    if((grid.coeffs == NULL))
    {
        c_errmsg("Error allocating memory for geometric mapping grid",
                "rotrnscl-alloc",&one);
        return(E_FAIL);
    }
    grid.coeffs[0] = coef[2];
    grid.coeffs[1] = coef[1];
    grid.coeffs[2] = coef[0];
    grid.coeffs[3] = coef[5];
    grid.coeffs[4] = coef[4];
    grid.coeffs[5] = coef[3];

    /* Fill in the gridding error portion of the mapping grid.  Since this is
       a first-order polynomial, gridding errors are not present
       ---------------------------------------------------------*/
    grid.griderr_valid = GEO_VALID;
    grid.max_grid_err[0] = grid.max_grid_err[1] = 0.0;
    grid.ave_grid_err[0] = grid.ave_grid_err[1] = 0.0;
    grid.rms_grid_err = 0.0;
    for (i = 0; i < 32; i++)
        grid.act_grid_err[i] = 0.0;
    grid.grid_tol = 0.0;

    /* Check for projection information and place it in the mapping grid
       if it exists.  The projection corners must be adjusted first for any
       window input with the image and then for the number of lines and samples
       in the output space.
       --------------------*/
    if (num_img == 0) grid.proj_valid = GEO_NOTVALID;
    else
    {
        if (c_getddr(in_image, &ddr) != E_SUCC) 
        {
            c_errmsg("Projection information not available","rotrnscl-noproj",
                     &one);
            c_errmsg("Nonfatal error encountered","rotrnscl-warn",&one);
            grid.proj_valid = GEO_NOTVALID;
        }
        else
        {
            if (ddr.valid[0] == INVAL) grid.proj_valid = GEO_NOTVALID;
            else
            {
                grid.proj_valid = GEO_VALID;
                grid.code = ddr.proj_code;
                if (ddr.valid[1] == INVAL) grid.zone = 62;
                else grid.zone = ddr.zone_code;
                if (ddr.valid[2] == INVAL) grid.datum = 0;
                else grid.datum = ddr.datum_code;

                if ((c_string2unit(ddr.proj_units) == E_FAIL) ||
                    (ddr.valid[4] == INVAL))
                {
                    strcpy(grid.units, " ");
                    c_lasmsg(FALSE, "rotrnscl-warn", 
                            "Projection units %s are invalid", ddr.proj_units); 
                    c_lasmsg(FALSE,"rotrnscl-warn",
                             "Setting projection units to empty");
                }             
                else 
                    strcpy(grid.units, ddr.proj_units);

                if (ddr.valid[3] == INVAL)
                    for (i = 0; i < 15; grid.projprms[i++] = 0.0);
                else 
                    for (i = 0; i < 15; i++)
                        grid.projprms[i] = ddr.proj_coef[i];
                if (ddr.valid[5] == INVAL) grid.pdist[0] = grid.pdist[1] = 0.0;
                else
                {
                    grid.pdist[0] = ddr.pdist_y * (fabs(aa[2]) + fabs(aa[1]));
                    grid.pdist[1] = ddr.pdist_x * (fabs(bb[1]) + fabs(bb[2]));
                }
                if (ddr.valid[6] == INVAL)
                    for (i = 0; i < 8; grid.corners[i++] = 0.0);
                else
                {
                    proj_coord[0] = ddr.upleft[0];
                    proj_coord[1] = ddr.upleft[1];
                    proj_coord[2] = ddr.loleft[0];
                    proj_coord[3] = ddr.loleft[1];
                    proj_coord[4] = ddr.upright[0];
                    proj_coord[5] = ddr.upright[1];
                    proj_coord[6] = ddr.loright[0];
                    proj_coord[7] = ddr.loright[1];
                    image_coord[0] = image_coord[1] = 1;
                    image_coord[2] = ddr.nl;
                    image_coord[3] = 1;
                    image_coord[4] = 1;
                    image_coord[5] = ddr.ns;
                    image_coord[6] = ddr.nl;
                    image_coord[7] = ddr.ns;
                    prt_flag = TRUE;
                    if(c_rtcoef(proj_coord,image_coord,cc,&prt_flag) != E_SUCC) 
                    {
                        c_errmsg("Nonfatal error encountered","rotrnscl-warn",
                                 &one);
                        for (i = 0; i < 8; grid.corners[i++] = 0.0);
                    }
                    else
                    {
                        line = aa[0] + aa[1] * in_wind[1] + aa[2] * in_wind[0];
                        samp = bb[0] + bb[1] * in_wind[1] + bb[2] * in_wind[0];
                        grid.corners[0]=(cc[0] * line) + (cc[1] * samp) + cc[2];
                        grid.corners[1]=(cc[3] * line) + (cc[4] * samp) + cc[5];
                        line = aa[0] + aa[2]*(in_wind[0]+nlines-1) + 
                               aa[1]*in_wind[1];
                        samp=bb[0] + bb[2]*(in_wind[0]+nlines-1) + 
                               bb[1]*in_wind[1];
                        grid.corners[2] = (cc[0] * line) + (cc[1] * samp) + 
                               cc[2];
                        grid.corners[3] = (cc[3] * line) + (cc[4] * samp) + 
                               cc[5];
                        line = aa[0] + aa[2]*in_wind[0] + aa[1]*(in_wind[1]+
                               nsamps-1);
                        samp = bb[0] + bb[2]*in_wind[0] + bb[1]*(in_wind[1]+
                               nsamps-1);
                        grid.corners[4]=(cc[0] * line) + (cc[1] * samp) + cc[2];
                        grid.corners[5]=(cc[3] * line) + (cc[4] * samp) + cc[5];
                        line = aa[0] + aa[2] * (in_wind[0] + nlines - 1) + 
                            aa[1] * (in_wind[1] + nsamps - 1);
                        samp = bb[0] + bb[2] * (in_wind[0] + nlines - 1) +
                            bb[1] * (in_wind[1] + nsamps - 1);
                        grid.corners[6]=(cc[0] * line) + (cc[1] * samp) + cc[2];
                        grid.corners[7]=(cc[3] * line) + (cc[4] * samp) + cc[5];
                    }
                }
            }
        }
    }

    /* Initalize all unused fields in the geometric mapping grid record
       -----------------------------------------------------------------*/
    grid.fiterr_valid = GEO_NOTVALID;

    /* Write to geometric mapping grid file
       -------------------------------------*/
    if (c_putgrd(name, &grid) != E_SUCC) return(E_FAIL);

    /* Free space 
       -------------*/
    free(grid.out_lines); 
    free(grid.out_samps); 
    free(grid.in_lines);  
    free(grid.in_samps); 

    return(E_SUCC);
}
