/*****************************************************************************
NAME				  ROTRNSCL

PURPOSE    Generates a mapping grid for input to geometric transformation
	   routines GEOM and GEOMAP.  These are linear transformations 
	   representing an arbitrary combination of rotation, translation, 
	   and scaling.

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   R. White        4/82        CSC      Original Development
  4.0      S. Kondal       7/84        SAR     
  5.0      D. Steinwand   12/88        EDC      LAS 5.0 development,
						C conversion
  5.1      D. Etrheim     12/92        EDC      Modify to call c_clean if 
                                                unsuccessfull in creating the 
                                                grid
  5.2      D. Etrheim     08/93	       EDC      Leave angle as zero instead of
                                                360.  This helps with some
                                                percision problems.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   ROTRNSCL must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
	Get the name of the input image (It may or may not be NULL)
	Get the output mapping grid file name
	Get the number of output space lines and samples
	Determine the subcommand used
	If subcommand = EXPLICIT,
	   Get the rotation angle, the scale factor in both the line
	    and sample directions, and the pivot point 
	   Adjust the rotation angle so it is positive, counter-clockwise
	   Calculate mapping coefficients
	If subcommand = TWOPTS,
	   Get the first tie point pair
	   Get the second tie point pair
	   Calculate mapping coefficients
	If subcommand = DIRECT,
	   Get the coefficients directly from the user, via TAE
	If subcommnad = INVERT,
	   Get the coefficients 
	   Invert mapping coefficients
	Calculate the mapping grid and write it to disk
	Write completion message and exit
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "local_protos.h"

int main (void)
{
    double 	coef[6];        /* Transformation coefficients */
    double 	incoef[6];      /* Inverse transformation coefficients */
    double 	pivot[4];       /* Pivot point */
    double 	scale[4];       /* Pre- & Post- scales */
    double 	angle;          /* Angle of rotation--counter clockwise */
    double 	intie1[2];      /* First tiepoint--input space */
    double 	intie2[2];      /* Second tiepoint--input space */
    double 	outtie1[2];     /* First tiepoint--output space */
    double 	outtie2[2];     /* Second tiepoint--output space */
    long	nlines;         /* Number of lines in output image */
    long	nsamps;         /* Number of samples in output image */
    long    fatal = -1;     /* Fatal error flag for c_errmsg */
    long	zero = 0;       /* constant zero */
    long	one = 1;        /* constant one */
    long	std_out;        /* standard output (lun) for inlas */
    long    mode;           /* IWRITE flag for getnam() routine */
    long	num_img;        /* Number of images entered (0 or 1) */
    long	bands[1][MAXBND + 1];/* Band numbers */
    long	nbands;         /* Number of bands */
    long	in_wind[1][4];  /* Window entered with input image */
    long	totbands;       /* Total number of bands entered */
    long 	same;           /* Window flag */
    long	qflag[1][2];    /* Image option flag */
    char 	outhost[1][CMLEN];/* output file host name */
    char	tdum[1][CMLEN]; /* TAE image name of input image */
    char 	in_image[1][CMLEN];/* Host image name */
    char 	option[11];     /* Pivot option */
    char 	method[10];     /* SUBCMD value--method of parameter input */
    struct  PARBLK  vblk;   /* TAE v-block */
    struct  VARIABLE *var;  /* TAE variable value... */

    /*--------------------------------------------------------------------------
      Initialize LAS processing
      ------------------------------------------------------------------------*/
    c_inlas(&vblk, &std_out);

    /*--------------------------------------------------------------------------
      Get the name of the input image file.  This may or may not be NULL.  If
      the file is there, it's DDR information will be read and packed into the
      Geometric Mapping Grid file.  If an error occurrs, in finding the file or
      reading the DDR, a warning message will be issued.
      ------------------------------------------------------------------------*/
    if(c_getin(&vblk,"IN",&zero,tdum,in_image,&num_img,in_wind,bands,&nbands,
            &totbands,&one,&same,qflag) != E_SUCC)
        c_errmsg("Nonfatal error encountered","rotrnscl-warn",&one);

    /*--------------------------------------------------------------------------
      Get the name of the output mapping grid file
      ------------------------------------------------------------------------*/
    mode = IWRITE;
    if((c_getnam(&vblk, "OUTGRID", &zero, outhost[0], "grid", &mode)) != E_SUCC) 
        c_errmsg("Fatal error encountered", "rotrnscl-fatal", &fatal);

    /*--------------------------------------------------------------------------
      Get the number of lines and samples in the output space
      ------------------------------------------------------------------------*/
    var = p_find(&vblk, "NL");
    nlines = IVAL(*var, 0);
    if (nlines < 1)
    {
        c_errmsg("NL must be greater than zero","rotrnscl-invalid",&one);
        c_errmsg("Fatal error encountered", "rotrnscl-fatal", &fatal);
    }
    var = p_find(&vblk, "NS");
    nsamps = IVAL(*var, 0);	
    if (nsamps < 1)
    {
        c_errmsg("NS must be greater than zero","rotrnscl-invalid",&one);
        c_errmsg("Fatal error encountered", "rotrnscl-fatal", &fatal);
    }
    /*--------------------------------------------------------------------------
      Get the subcommand (method) 
      ------------------------------------------------------------------------*/
    var = p_find(&vblk, "_SUBCMD");		
    strcpy(method, SVAL(*var, 0));

    /*--------------------------------------------------------------------------
      Get parameters used with the EXPLICIT subcommand
      ------------------------------------------------------------------------*/
    if(strcmp(method, "EXPLICIT") == 0)
    {
        var = p_find(&vblk, "ROTANG");	
        angle = (double)RVAL(*var, 0);
        if (angle < 0) angle = -angle;	/* Make rotang positive, ccwise */
        else if (angle != 0.)
            angle = 360 - angle;
        var = p_find(&vblk, "SCALFACT");	
        scale[0] = (double)RVAL(*var, 0);
        scale[1] = (double)RVAL(*var, 1);
        scale[2] = (double)RVAL(*var, 2);
        scale[3] = (double)RVAL(*var, 3);
        var = p_find(&vblk, "ROTOPT");
        strcpy(option, SVAL(*var, 0));
        if(strcmp(option, "PIVOT") == 0)
        {	
            var = p_find(&vblk, "PIVCOORS");
            pivot[2] = (double)RVAL(*var, 0);
            pivot[3] = (double)RVAL(*var, 1);
            pivot[0] = (double)RVAL(*var, 2);
            pivot[1] = (double)RVAL(*var, 3);
        }
        else
        {
            pivot[0] = pivot[2] = (double)(nsamps + 1) / 2.0;
            pivot[1] = pivot[3] = (double)(nlines + 1) / 2.0;
        }

        /*---------------------------------------------------------------------
          Calculate mapping coefficients from input rotation angle, scale 
          factors, and point of pivot.
          -------------------------------------------------------------------*/
        explicit(angle, scale, pivot, coef);
    }

    /*--------------------------------------------------------------------------
      Get parameters used with the TWOPTS subcommand
      ------------------------------------------------------------------------*/
    else if (strcmp(method, "TWOPTS") == 0)
    {
        var = p_find(&vblk, "TVAL");
        intie1[0] = (double)RVAL(*var, 0);	
        intie1[1] = (double)RVAL(*var, 1);
        outtie1[0] = (double)RVAL(*var, 2);	
        outtie1[1] = (double)RVAL(*var, 3);
        intie2[0] = (double)RVAL(*var, 4);	
        intie2[1] = (double)RVAL(*var, 5);
        outtie2[0] = (double)RVAL(*var, 6);	
        outtie2[1] = (double)RVAL(*var, 7);

        /*---------------------------------------------------------------------
          Calculate mapping coefficients from two tiepoint pairs
          --------------------------------------------------------------------*/
        twopoint(intie1, intie2, outtie1, outtie2, coef);
    }

    /*--------------------------------------------------------------------------
      Get parameters used with the DIRECT subcommand.  The mapping coefficients
      are input directly by the user.
      -----------------------------------------------------------------------*/
    else if (strcmp(method, "DIRECT") == 0)
    {
        var = p_find(&vblk, "TRANCOEF");
        coef[0] = (double)RVAL(*var, 0);
        coef[1] = (double)RVAL(*var, 1);
        coef[2] = (double)RVAL(*var, 2);
        coef[3] = (double)RVAL(*var, 3);
        coef[4] = (double)RVAL(*var, 4);
        coef[5] = (double)RVAL(*var, 5);
    }

    /*--------------------------------------------------------------------------
      Get parameters used with the INVERT subcommand
      -----------------------------------------------------------------------*/
    else
    {
        var = p_find(&vblk, "TRANCOEF");	
        incoef[0] = (double)RVAL(*var, 0);
        incoef[1] = (double)RVAL(*var, 1);
        incoef[2] = (double)RVAL(*var, 2);
        incoef[3] = (double)RVAL(*var, 3);
        incoef[4] = (double)RVAL(*var, 4);
        incoef[5] = (double)RVAL(*var, 5);

        /*--------------------------------------------------------------------
          Invert mapping coefficients as entered by the user
          -------------------------------------------------------------------*/
        if (c_invert(incoef, coef) != E_SUCC)
            c_errmsg("Fatal error encountered", "rotrnscl-fatal", &fatal);
    }

    /*--------------------------------------------------------------------------
      Grid the input and output space and write to disk
      -----------------------------------------------------------------------*/
    if(make_grid(nlines,nsamps,coef,outhost[0],in_image[0],num_img,in_wind[0]) 
            != E_SUCC)
        c_clean(&vblk,"rotrnscl",outhost,&one);

    /*--------------------------------------------------------------------------
      Write completion message and exit program
      -----------------------------------------------------------------------*/
    c_complt("ROTRNSCL", outhost, &one);
    return(0);
}
