/*****************************************************************************
NAME				  TWOPOINT

PURPOSE    Calculates mapping coefficients from a pair of tiepoints

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   R. White        4/82        CSC      Original Development
  4.0      S. Kondal       7/84        SAR     
  5.0      D. Steinwand   10/88        EDC      LAS 5.0 development,
						C conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   ROTRNSCL must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION

	Find the pythagorian distance for the input space
	Find the pythagorian distance for the output space
	If the tie points coincide, abort
	Calculate the sine of the angle of rotation
	Calculate the cosine of the angle of rotation
	Calculate the scale factors
	If the scale factor is out of range, abort
	Calculate mapping coefficients
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "local_protos.h"

void twopoint
(
    double intie1[2],		/* First input location, line/sample */
    double intie2[2],		/* Second input location, line/sample */
    double outtie1[2],		/* First output location, line/sample */
    double outtie2[2],		/* Second output location, line/sample */
    double coef[6]			/* Transformation coefficients */
)
{
    double scale[4];        /* Pre- & Post- scale factors */
    double sinphi;          /* rotation angle */
    double cosphi;          /* rotation angle */
    double in_dist;         /* Distance between input tiepoints */
    double out_dist;        /* Distance between output tiepoints */
    long fatal = -1;        /* Fatal error flag for c_errmsg */
    long pass = 1;          /* Non-fatal error flag for c_errmsg */

    /*--------------------------------------------------------------------------
      Compute the pythagorian distance of the input and output space
      -----------------------------------------------------------------------*/
    in_dist = sqrt(((intie2[0] - intie1[0]) * (intie2[0] - intie1[0])) +
            ((intie2[1] - intie1[1]) * (intie2[1] - intie1[1])));
    out_dist =sqrt(((outtie2[0] - outtie1[0]) * (outtie2[0] - outtie1[0])) +
            ((outtie2[1] - outtie1[1]) * (outtie2[1] - outtie1[1]))); 
    if((in_dist * out_dist) == 0.0)
    {
        c_errmsg("Tie points coincide","rotrnscl-tiepts", &pass);
        c_errmsg("Fatal error encountered","rotrnscl-fatal", &fatal);
    }

    /*--------------------------------------------------------------------------
      Compute angle and scale factors 
      -----------------------------------------------------------------------*/
    sinphi = ((intie2[1] - intie1[1]) * (outtie2[0] - outtie1[0]) - 
            (intie2[0] - intie1[0]) * (outtie2[1] - outtie1[1])) /
        (in_dist * out_dist);
    cosphi = ((intie2[0] - intie1[0]) * (outtie2[0] - outtie1[0]) +
            (intie2[1] - intie1[1]) * (outtie2[1] - outtie1[1])) /
        (in_dist * out_dist);
    scale[0] = scale[1] = 1.0;
    scale[2] = scale[3] = out_dist/in_dist;

    /*--------------------------------------------------------------------------
      Check the range of the scale factor 
      -----------------------------------------------------------------------*/
    if((fabs(scale[2]) < 0.001) || (fabs(scale[2]) > 1000.0))
    {
        c_errmsg("Scale out of range","rotrnscl-scale", &pass);
        c_errmsg("Fatal error encountered","rotrnscl-fatal", &fatal);
    }

    /*--------------------------------------------------------------------------
      Calculate mapping coefficients 
      -----------------------------------------------------------------------*/
    coef[0] = cosphi / (scale[0] * scale[2]);
    coef[1] = -(sinphi) / (scale[0] * scale[3]);
    coef[2] = intie1[0] - (outtie1[0] * coef[0]) - (outtie1[1] * coef[1]);
    coef[3] = sinphi / (scale[1] * scale[2]);
    coef[4] = cosphi / (scale[1] * scale[3]);
    coef[5] = intie1[1] - (outtie1[0] * coef[3]) - (outtie1[1] * coef[4]);

}
