/*****************************************************************************
NAME				rtable 

PURPOSE    Create resampling weight table files for use in the resampling
           process

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   D. Steiwnand    2/89        EDC	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   RTABLE must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
	Get the name of the output resampling weight table file
	Get the dimensions of the resampling kernel
	Determine program mode (which subcommand??)
	If subcommand is SINC
	   Get the Hanning window flag (YES or NO)
	   Generate an inverse point spread function for each dimension for
		sin (PI * x) / (PI * x)
	If subcommand is FILE
	   Get the file name of the first inverse point spread function file
	   Read the file into a program buffer
	   If the second kernel dimension is given
	      Get the name of the second IPSF file & read it
	   If the thrid kernel dimension is given
	      Get the name of the third IPSF file & read it
	Create a resampling weight table
	Write the resampling weight table to disk
	Call complete & exit
*****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

int main
(void)
{
struct RWTAB table;		/* Resampling weight table */
double ipsf1[1025];		/* First inverse point spread function */
double ipsf2[1025];		/* Second inverse point spread function */
double ipsf3[1025];		/* Third inverse point spread function */
long    fatal = -1;		/* Fatal error flag for c_errmsg */
long	zero = 0;		/* constant zero */
long	one = 1;		/* constant one */
long 	two = 2;		/* constant two */
long	std_out;		/* standard output (lun) for inlas */
long    mode;			/* IWRITE flag for getnam() routine */
char 	outhost[CMLEN];		/* output file host name */
char 	infile [CMLEN];		/* Inverse point spread function file name */
char 	window[5];		/* Apply Hanning window? */
char 	method[5];		/* SUBCMD value--SINC or FILE */
struct  PARBLK  vblk;		/* TAE v-block */
struct  VARIABLE *var;		/* TAE variable value... */

/* Initialize LAS processing
 --------------------------*/
c_inlas(&vblk, &std_out);

/* Get the name of the output resampling weight table file
 --------------------------------------------------------*/
mode = IWRITE;
if((c_getnam(&vblk, "OUTRWT", &zero, outhost, "rwt", &mode)) != E_SUCC) 
  c_errmsg("Fatal error encountered", "rtable-fatal", &fatal);

/* Get the dimensions of the resampling kernel
 --------------------------------------------*/
var = p_find(&vblk, "KERNDIM");
table.dim[0] = IVAL(*var, 0);
table.dim[1] = IVAL(*var, 1);
table.dim[2] = IVAL(*var, 2);

/* Get the subcommand
 -------------------*/
var = p_find(&vblk, "_SUBCMD");		
strcpy(method, SVAL(*var, 0));

/* Get parameters used with the SINC subcommand and calculate the inverse
   point spread function for each kernel dimension
 ------------------------------------------------*/
if(strcmp(method, "SINC") == 0)
   {
   var = p_find(&vblk, "WINDFLG");
   strcpy(window, SVAL(*var, 0));
   if (table.dim[0] > 0) sinxx(table.dim[0], window, ipsf1);
   if (table.dim[1] > 0) sinxx(table.dim[1], window, ipsf2);
   if (table.dim[2] > 0) sinxx(table.dim[2], window, ipsf3);
   }

/* Get parameters used with the FILE subcommand, including an inverse
   point spread function file name for each kernel dimension given.  Read
   each file and place in program buffers.
 ----------------------------------------*/
else
   {
   mode = IREAD;
   if((c_getnam(&vblk, "INFILE", &zero, infile, "ipsf", &mode)) != E_SUCC) 
     c_errmsg("Fatal error encountered", "rtable-fatal", &fatal);
   get_ipsf(infile, ipsf1);

   if (table.dim[1] != 0)
     {
     if((c_getnam(&vblk, "INFILE", &one, infile, "ipsf", &mode)) != E_SUCC) 
       c_errmsg("Fatal error encountered", "rtable-fatal", &fatal);
     get_ipsf(infile, ipsf2);
     }

   if (table.dim[2] != 0)
     {
     if((c_getnam(&vblk, "INFILE", &two, infile, "ipsf", &mode)) != E_SUCC) 
       c_errmsg("Fatal error encountered", "rtable-fatal", &fatal);
     get_ipsf(infile, ipsf3);
     }
   }

/* Create the resampling weight table file from the inverse point spread
   functions (ipsf)
  ----------------*/
maketab(ipsf1, table.dim[0], table.table1);
if (table.dim[1] != 0) maketab(ipsf2, table.dim[1], table.table2);
if (table.dim[2] != 0) maketab(ipsf3, table.dim[2], table.table3);

/* Write the resampling weight file to disk
  ----------------------------------------*/
if (c_putrwt(outhost, &table) != E_SUCC)
     c_errmsg("Fatal error encountered", "rtable-fatal", &fatal);

/* Call complete and exit
  ----------------------*/
c_complt("RTABLE", outhost, &one);

return EXIT_SUCCESS;

}
