/****************************************************************************
NAME:   			sinxx

PURPOSE:  Routine to make sin(PI *x)/(PI * x) IPSF (inverse point 
          spread function)  A Hanning window may be applied.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0   12/85   D. Meyer      AB       Ditty in support of restoration contract
  5.0	 2/89	D. Steinwand  CSB      LAS 5.0 development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Convert Hanning window flag to a boolean value
	Set the mid-point of the IPSF to 1 (even if windowing is applied)
	Loop thru the IPSF, starting at the mid-point; working toward the start
	   Determine Hanning window multiplier (1 if no windowing)
	   Multiply the current increment by PI
	   If not out of range, evaluate sin (PI * x) / (PI * x) (sinc function)
	   Apply the windowing factor to the calculated sinc function value and
 		store in the output array on BOTH sides of the IPSF's midpoint
	   Add the IPSF values to the sum accumulator
	Rescale the IPSF by the sum
	Return
*****************************************************************************/
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

void sinxx
(
    long dim,		/* Dimension of table */
    char *window,		/* Apply Hanning window?  YES or NO */
    double *ipsf		/* Array containing the inverse point spread function */
)
{
double pix;		/* Value of PI multiplied by x */
double sinc2;		/* value of sin (pix) / (pix)  */
double sum;		/* Sum of inverse point spread function values */
double x;		/* Current increment at which to evaluate function */
double wnd;		/* Value of window (multiplier); 1 if no window */
long hanning;		/* Flag (TRUE/FALSE);  apply Hanning window?? */
long i;			/* Loop counter */

/* Set boolean flag for windowing (saves computations)
  ---------------------------------------------------*/
hanning = FALSE;
if (strcmp(window, "YES") == 0) hanning = TRUE;

/* Generate sinc function 
  ----------------------*/
sum = ipsf[512] = 1.0;
for (dim--, x = DX, i = 512; i--; x+= DX)
   {
   if (hanning) wnd = (x < (dim/2.0)) ? (0.5 * (1.0+cos(TWOPI * x/dim))) : 0.0;
   else wnd = 1.0;
   pix = PI * x;
   sinc2 = (fabs(x) < 0.000001) ? 1.0 : (sin(pix) / pix);
   ipsf[i] = ipsf[1024 - i] = wnd * sinc2;
   sum += 2.0 * ipsf[i];
   }

/* Rescale
  -------*/
for (i = 0; i < 1025; i++)
   ipsf[i] /= sum;

return;
}
