#include <math.h>
#include "las.h"
#include "protos.h"

void calc_normal
(
    double **save_x,
    double **save_y,
    double **save_z,    /* Buffers containing the cartesian coordinates of the
                           current line, previous line and next line. (I)    */
    long current,       /* Index to current line. (I)			             */
    long next,          /* Index to next line. (I)			                 */
    long prev,          /* Index to previous line. (I)			             */
    long samp,          /* Current sample. (I)				                 */
    double normal[]     /* Average normal of the point of interest (x,y,z) 
			               based on 4 adjacent points. (O)       	         */
)

/*******************************************************************************
NAME:	  CALC_NORMAL 

PURPOSE:  Given a point of interest and 4 adjacent points (expressed as
   	  x, y, z), CALC_NORMAL calculates 4 normal unit vectors, finds
 	  the average of these and returns the resulting vector. 

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90

ALGORITHM:

	Find the difference between the point of interest and each adjacent
	  vector.
	Calculate the cross products of the difference vectors.
	Find the magnitude of each cross product vector.
	Calculate 4 normal unit vectors.
	Find the average of the 4 calculated vectors.
	Find the magnitude of the averaged vector and calculate the avaraged
	  unit vector.
	Return to the calling routine.	

ALGORITHM REFERENCE:

	Input into CALC_NORMAL is a point of interest and 4 adjacent points
	a, b, c, d.
			     .d

			 c.  .p  .a

			     .b

  	Given a point of interest and 2 adjacent points, the normal can
	be calculated using the following equation.

		n = (a - p) X (b - p)
		   -------------------
		   |(a - p) X (b - p)|

	Where:
	    n is the normal vector (expressed as x,y,z).
	    p is the point of interest (expressed as a vector x,y,z).
	    a is the first adjacent point (expressed as a vector x,y,z).
	    b is the second adjacent point (expressed as a vector x,y,z).

*******************************************************************************/
{
    double	cross1[3], /* Cross product of difference vectors AP and BP.    */
        cross2[3],  /* Cross product of difference vectors BP and CP.       */
        cross3[3],  /* Cross product of difference vectors CP and DP.       */
        cross4[3],  /* Cross product of difference vectors DP and AP.       */
        dif_ap[3],  /* Difference between point of interest and the first 
                       adjacent point (A).                                  */
        dif_bp[3],  /* Difference between point of interest and the second 
                       adjacent point (B).                                  */
        dif_cp[3],  /* Difference between point of interest and the third 
                       adjacent point (C).                                  */
        dif_dp[3],  /* Difference between point of interest and the fourth 
                       adjacent point (D).                                  */
        interest,   /* Current point of interest value.                     */
        mag1,       /* Magnitude of the first cross product.                */
        mag2,       /* Magnitude of the second cross product.               */
        mag3,       /* Magnitude of the third cross product.                */
        mag4,       /* Magnitude of the fourth cross product.               */
        magn,       /* Magnitude of the averaged normal vector.             */
        tmpn[4][3]; /* Temporary normal vectors to be averaged.             */

    long	i;      /* Looping variable.				      */


    /* 
       Find the difference between the point of interest vector and each 
       adjacent vector. 
     */

    interest = save_x[current][samp];
    dif_ap[0] = save_x[current][samp+1] - interest; 
    dif_bp[0] = save_x[next][samp] - interest; 
    dif_cp[0] = save_x[current][samp-1] - interest; 
    dif_dp[0] = save_x[prev][samp] - interest; 

    interest = save_y[current][samp];
    dif_ap[1] = save_y[current][samp+1] - interest;
    dif_bp[1] = save_y[next][samp] - interest;
    dif_cp[1] = save_y[current][samp-1] - interest;
    dif_dp[1] = save_y[prev][samp] - interest;

    interest = save_z[current][samp];
    dif_ap[2] = save_z[current][samp+1] - interest;
    dif_bp[2] = save_z[next][samp] - interest;
    dif_cp[2] = save_z[current][samp-1] - interest;
    dif_dp[2] = save_z[prev][samp] - interest;

    /*
       Calculate the cross products of the difference vectors.
     */
    cross1[0] = (dif_ap[1] * dif_bp[2]) - (dif_ap[2] * dif_bp[1]);
    cross1[1] = (dif_ap[2] * dif_bp[0]) - (dif_ap[0] * dif_bp[2]);
    cross1[2] = (dif_ap[0] * dif_bp[1]) - (dif_ap[1] * dif_bp[0]);

    cross2[0] = (dif_bp[1] * dif_cp[2]) - (dif_bp[2] * dif_cp[1]);
    cross2[1] = (dif_bp[2] * dif_cp[0]) - (dif_bp[0] * dif_cp[2]);
    cross2[2] = (dif_bp[0] * dif_cp[1]) - (dif_bp[1] * dif_cp[0]);

    cross3[0] = (dif_cp[1] * dif_dp[2]) - (dif_cp[2] * dif_dp[1]);
    cross3[1] = (dif_cp[2] * dif_dp[0]) - (dif_cp[0] * dif_dp[2]);
    cross3[2] = (dif_cp[0] * dif_dp[1]) - (dif_cp[1] * dif_dp[0]);

    cross4[0] = (dif_dp[1] * dif_ap[2]) - (dif_dp[2] * dif_ap[1]);
    cross4[1] = (dif_dp[2] * dif_ap[0]) - (dif_dp[0] * dif_ap[2]);
    cross4[2] = (dif_dp[0] * dif_ap[1]) - (dif_dp[1] * dif_ap[0]);

    /*
       Find the magnitude of the 4 cross product vectors.
     */
    mag1 = sqrt(cross1[0] * cross1[0] + cross1[1] * cross1[1] + cross1[2] * 
            cross1[2]);
    mag2 = sqrt(cross2[0] * cross2[0] + cross2[1] * cross2[1] + cross2[2] * 
            cross2[2]);
    mag3 = sqrt(cross3[0] * cross3[0] + cross3[1] * cross3[1] + cross3[2] * 
            cross3[2]);
    mag4 = sqrt(cross4[0] * cross4[0] + cross4[1] * cross4[1] + cross4[2] * 
            cross4[2]);

    /*
       Calculate the 4 normal unit vectors and find the average.
     */
    for (i=0; i < 3; i++)
    {
        tmpn[0][i] = cross1[i] / mag1;
        tmpn[1][i] = cross2[i] / mag2;
        tmpn[2][i] = cross3[i] / mag3;
        tmpn[3][i] = cross4[i] / mag4;
    }

    normal[0] = (tmpn[0][0] + tmpn[1][0] + tmpn[2][0] + tmpn[3][0]) / 4.0;
    normal[1] = (tmpn[0][1] + tmpn[1][1] + tmpn[2][1] + tmpn[3][1]) / 4.0;
    normal[2] = (tmpn[0][2] + tmpn[1][2] + tmpn[2][2] + tmpn[3][2]) / 4.0;

    /*
       Unitize the averaged normal vector.
     */
    magn = sqrt(normal[0] * normal[0] + normal[1] * normal[1] + normal[2] * 
            normal[2]);
    normal[0] = normal[0] / magn;
    normal[1] = normal[1] / magn;
    normal[2] = normal[2] / magn;

    return;
}
