#include <math.h>
#include "las.h"
#include "sartopo.h"
#include "protos.h"

void findang
(
    double normal[],    /* Normal of the point of interest (x,y,z). (I)     */
    double *point_x,    /* Point of interest (x,y,z). (I)                   */
    double *point_y,
    double *point_z,
    double *rpoint_lat,	/* Point of interest (radians). (I)                 */
    double *rpoint_lon,
    double position[],	/* Aircraft position (x,y,z). (I)                   */
    double *line,       /* Magnitude of line of sight vector (range vector) 
			               in meters. (O)                                   */
    double *incid,	    /* Incidence angle (in degrees). (O)                */
    double *look,       /* Look angle (in degrees). (O)                     */
    double *azm,        /* Azimuth angle (in degrees). (O)                  */
    long option[]       /* Option indicating the information to be output.(I)
                                1 = Calculate incidence angle
                                2 = Calculate radar look angle
                                3 = Calculate line of sight (range) vector 
                                4 = Calculate azimuth angle		            */
)

/*******************************************************************************
NAME:	  FINDANG 

PURPOSE:  FINDANG calculates various radar viewing angles and/or the line of
	  sight vectors based on the user request.  Radar viewing angles
	  that may be calculated are: incidence angle, radar look angle,
	  and/or azimuth angle. 

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90

ALGORITHM:

	If this is the first time the routine has been called
	    Calculate the magnitude of any vector along the flight path (ie.
	      there is a constant elevation along the flight path).
	    Calculate the sine and cosine of the aircraft/spacecraft latitude.
	Calculate the line of sight vector and its magnitude.
	If the incidence angle is to be calculated
	    Calculate the dot product of the line of sight vector and the
	      normal vector.
	    Calculate the incidence angle and convert it to degrees.
	If the radar look angle is to be calculated
	    Calculate the dot product of the aircraft/spacecraft position
	      vector and the line of sight vector.
	    Calculate the radar look angle and convert it to degrees.
	If the azimuth angle is to be calculated
	    Calculate the the angle of the arc defined by the point of
	      interest and the aircraft/spacecraft position.
	    Calculate the azimuth angle and convert it to degrees. 
	Return to the calling routine.

ALGORITHM REFERENCE:

Line of sight vector = Point interest vector - Aircraft position vector
     or range vector

Incidence angle = 180 -          ((line of sight vector) . (normal vector))
                       arc cosine(----------------------------------------)
				 (       |line of sight vector|		  )
Radar look angle = 
    90 -          ((aircraft position vector) . (line of sight vector))
        arc cosine(---------------------------------------------------) 
                  ( |aircraft position vector| |line of sight vector| )
Where:
  |vector|        = The magnitude of the vector.
  vector . vector = The dot product of the 2 vectors.


cos(AP) = sin(Lat A)sin(Lat P) + cos(Lat A)cos(Lat P)cos(Long P - Long A)

Azimuth angle =	          (sin(Lat P) - cos(AP)sin(Lat A)) 
		arc cosine(----------------------------)
		          (      sin(AP)cos(Lat A)      )

If (sin(Long P - Long A) < 0)  Azimuth angle = 360 - Azimuth angle

Where:
  AP     = The line of sight vector.
  Lat A  = Latitude of the aircraft/spacecraft position.
  Lat P  = Latitude of the point of interest. 
  Long A = Longitude of the aircraft/spacecraft position.
  Long P = Longitude of the point of interest. 

*******************************************************************************/
{
    extern  double
            elev,      /* Elevation (in meters) of flight path (-LINE) or
                          specified point (-POINT).                          */
            rblat,     /* Beginning latitude of flight path (radians).	     */
            rblon;     /* Beginning longitude of flight path (radians).      */

    double 	ap,        /* The angle defined by the point of interest and
                          aircraft/spacecraft position vector.               */
            cos_alat,  /* Cosine of the aircraft/spacecraft position 
                          latitude.                                          */
            cos_ap,    /* Cosine of the point of interest and 
                          aircraft/spacecraft position vector.               */
            cos_azm,   /* Cosine of the azimuth angle.                       */
            dot,       /* Dot product of 2 vectors.                          */
            mag,       /* Magnitude of the vector.                           */
            rad,       /* Radian result.                                     */
            sight[3],  /* Line of sight vector.                              */
            sin_alat;  /* Sine of the aircraft/spacecraft position latitude. */

    long	first = TRUE, /* First time into routine.                        */
            i;            /* Looping index.                                  */

    if (first)
    {
        /*
           The elevation of the aircraft/spacecraft is assumed to be constant, 
           therefore the magnitude needs to be calculated only once.
         */
        mag = elev + EARTH_RAD;
        /*
           The azimuth angle is calculated only for the point mode in which the
           aircraft/spacecraft position is constant.  Therefore, the sine and 
           cosine of the latitude need only be calculated once.
         */
        cos_alat = cos(rblat);
        sin_alat = sin(rblat);
        first = FALSE;
    }
    /*
       Calculate the line of sight vector.
     */
    sight[0] = *point_x - position[0];
    sight[1] = *point_y - position[1];
    sight[2] = *point_z - position[2];

    /*
       Calculate the magnitude of the line of sight vector.
     */
    *line = sqrt((sight[0] * sight[0]) + (sight[1] * sight[1]) + 
            (sight[2] * sight[2]));

    for (i=0; i < 3; i++)
    {
        switch (option[i])
        {
            case 1:
                /*
                   Calculate the incidence angle.
                 */
                dot = (sight[0] * normal[0]) + (sight[1] * normal[1]) + 
                      (sight[2] * normal[2]);
                rad = acos(dot / *line);
                *incid = 180 - (rad * RAD2DEG); 
                break;

            case 2:
                /*
                   Calculate the radar look angle.
                 */
                dot = (position[0] * sight[0]) + (position[1] * sight[1]) + 
                    (position[2] * sight[2]);
                rad = acos(dot / (*line * mag)); 
                *look = 90 - (rad * RAD2DEG);
                break;

            case 4:
                /*
                   Calculate the azimuth angle.
                 */
                cos_ap = (sin_alat * sin(*rpoint_lat)) + 
                    (cos_alat * cos(*rpoint_lat) * cos(*rpoint_lon - rblon));
                ap = acos(cos_ap);
                cos_azm = (sin(*rpoint_lat) - (cos_ap * sin_alat)) / 
                    (sin(ap) * cos_alat);
                *azm = acos(cos_azm);
                *azm = *azm * RAD2DEG;
                if (sin(*rpoint_lon - rblon) < 0.0)
                    *azm = 360.0 - *azm;
                break;
        }
    }
    return;
}
