#include <math.h>
#include "las.h"
#include "sartopo.h"
#include "protos.h"

void findpt
(
    double rplat,     /* Latitude of point of interest (radians). (I)   */
    double rplon,     /* Longitude of point of interest (radians). (I)  */
    double position[] /* Aircraft position. (O)                         */
)

/*******************************************************************************
NAME:	  FINDPT 

PURPOSE:  Given the latitude/longitude of the point of interest, FINDPT
	  calculates the aircraft/spacecraft position closest to the
	  point from which the point can be "seen". 

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90

ALGORITHM:

	If this is the first time into the routine
	    Calculate the sines and cosines of the starting and ending
	      latitudes.
	    Calculate the sine and cosine of the arc whose end points are
	      the starting and ending points of the flight path.
	    Calculate the magnitude of any point along the flight path (this
	      can be done because the elevation is assumed to be constant
	      along the flight path).
	Calculate the cosine of the arc defined by the point of interest and
	  the starting point of the flight path.
	Calculate the cosine of the arc defined by the point of interest and
	  the ending point of the flight path.
	Calculate the earth central angle from the starting vector to the 
	  current aircraft position.
	Calculate the vector which is the current aircraft position.
	Return to the calling routine.

ALGORITHM REFERENCE:

          P
          .
             
         
        
       
      
                      . 
             .        E 
  .          R    
  S

Where:
  P = The point of interest.
  S = The starting point of the aircraft.
  E = The ending point of the aircraft.
  R = The aircraft's position to see the point of interest.

cos PE = sin(Lat P)sin(Lat E) + cos(Lat P)cos(Lat E)cos(Long P - Long E)
cos PS = sin(Lat P)sin(Lat S) + cos(Lat P)cos(Lat S)cos(Long P - Long S)
cos SE = sin(Lat S)sin(Lat E) + cos(Lat S)cos(Lat E)cos(Long E - Long S)

cos PSE = cos(PE) - cos(PS)cos(SE)
          ------------------------
              sin(PS)sin(SE) 

SR = arc tangent (cos(PSE)tan(PS))

SMAG = Earth radius + altitude of S
EMAG = Earth radius + altitude of E
RMAG = Earth radius + altitude of R

do i = 1 to 3
 R(i) = RMAG * (S(i) * cos(SR) + (E(i) - S(i) * cos(SE)) * sin(SR) / sin(SE))
               (----             (----   ----          )                    )
               (SMAG             (EMAG   SMAG          )                    )

*******************************************************************************/
{
    extern double	
        bpos[3],    /* Beginning position of flight path (cartesian 
                       coordinates).                                      */
        elev,       /* Elevation (in meters) of flight path (-LINE) or 
                       specified point (-POINT).                          */
        epos[3],    /* Ending position of flight path (cartesian
                       coordinates).                                      */
        rblat,      /* Beginning latitude of flight path (radians).	      */
        rblon,      /* Beginning longitude of flight path (radians).      */
        relat,      /* Ending latitude of flight path (radians).          */
        relon;      /* Ending longitude of flight path (radians).         */

    static double
        cos_elat,   /* Cosine of ending latitude.                           */
        cos_slat,   /* Cosine of starting latitude.                         */
        cos_se,     /* Cosine of the arc SE (see diagram above).            */
        mag,        /* Magnitude of flight path vectors.                    */
        se,         /* Angle SE (see diagram above).                        */
        sin_elat,   /* Sine of ending latitude.                             */
        sin_se,     /* Sine of the arc SE (see diagram above).              */
        sin_slat;   /* Sine of starting latitude.                           */

    static	long
        first = TRUE;/* First time into the routine.                        */

    double	angle,   /* Earth central angle from starting vector to
                        current aircraft position (angle SR above).	        */
    cos_c,           /* Cosine of earth central angle from starting vector
                        to current aircraft position (angle SR above).      */
    cos_pe,          /* Cosine of the arc PE (see diagram above).           */
    cos_ps,          /* Cosine of the arc PS (see diagram above).           */
    cos_pse,         /* Cosine of the angle PSE (see diagram above).        */
    cos_rplat,       /* Cosine of the latitude of the point of interest.    */
    ps,              /* Angle PS (see diagram above).                       */
    sin_c,           /* Sine of earth central angle from starting vector
                        to current aircraft position (angle SR above).      */
    sin_rplat,       /* Sine of the latitude of the point of interest.      */
    tan_sr;          /* Tangent of the arc SR (see diagram above).          */

    long i;          /* Looping index.                                      */

    /*
       The starting and ending vectors of the flight path do not change, 
       therefore, calculate as much information as possible only once.
     */
    if (first)
    {
        /*
           Calculate the sines and cosines of the starting and ending latitudes.
         */
        sin_elat = sin(relat);
        cos_elat = cos(relat);
        sin_slat = sin(rblat);
        cos_slat = cos(rblat);

        /* 
           Calculate the sine and cosine of the arc whose end points are the 
           starting and ending points of the flight path.
         */
        cos_se = sin_slat * sin_elat + cos(rblat) * cos_elat * cos(relon 
                - rblon);
        se = acos(cos_se);
        sin_se = sin(se);

        /*
           Calculate the magnitude of any point along the flight path (ie. the 
           elevation is assumed to be constant along the flight path).
         */
        mag = elev + EARTH_RAD;

        first = FALSE;
    }

    /*
       Calculate the cosines of the arcs defined by the point of interest and 
       the starting and ending vectors of the flight path.
     */
    sin_rplat = sin(rplat);
    cos_rplat = cos(rplat);
    cos_pe = sin_rplat * sin_elat + cos_rplat * cos_elat * cos(rplon - relon);
    cos_ps = sin_rplat * sin_slat + cos_rplat * cos_slat * cos(rplon - rblon);
    ps = acos(cos_ps);

    /*
       Calculate the earth central angle from the starting vector to the current
       aircraft position.
     */
    cos_pse = (cos_pe - cos_ps * cos_se) / (sin(ps) * sin_se);
    tan_sr = cos_pse * tan(ps);
    angle = (float)(atan(tan_sr)); 
    cos_c = cos(angle);
    sin_c = sin(angle);

    for (i=0; i < 3; i++)
        position[i] = mag * (bpos[i] / mag * cos_c + (epos[i] / mag - bpos[i] 
                / mag * cos_se) * sin_c / sin_se); 

    return;
}
