/******************************************************************************
NAME:		GET_INFO

PURPOSE: 	Given the current input buffer and the user requests, GET_INFO
		calculates the latitude/longitude and the cartesian coordinates
  		for each input sample in the buffer and also calculates the
		appropriate radar viewing angles and/or range vectors and 
		places the values into the output buffer. 

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90

PROJECT:			LAS

ALGORITHM:

	If the projection transformation package has not been initialized
	    Validate the projection units.
	    Initialize the projection transformation package.
	Set the appropriate indices for the current, previous and next lines.
	For each sample in the input buffer
	    Convert the line/sample location to a projection coordinate and 
	      then to a latitude/longitude coordinate.
	    Convert the latitude/longitude/elevation coordinate to an x, y, z
	      coordinate.
	    Calculate the normal vector.
	    If the subcommand is -LINE
	   	Calculate the current aircraft/spacecraft position (x, y, z).
	    Calculate the radar viewing angles and the range vector.
	    Set the values in the output buffer.  If the output data type is
	      not REAL*4, round values to the nearest integer.  Also subtract
	      the user input offset from the range calculation.
	Return to the calling routine.

ALGORITHM REFERENCES:		None
******************************************************************************/
#include "las.h"
#include "protos.h"
#include "geompak.h"
#define	 IMG		1
#define	 OBND		3
#define  RADIANS	0
#define	 NULZONE	62

int get_info
(
    float *ibuf,        /* Input image I/O buffer. (I)*/
    float *obuf,        /* Output image I/O buffer. (O)*/
    long line,          /* Current line number. (I)*/
    double position[],  /* Air craft position (x,y,z). (I)*/
    long window[],      /* Input window. (I)*/
    struct DDR *ddr,    /* Input image's DDR. (I)*/
    double coef[],      /* Coefficients mapping image coordinates to projection
			               coordinates. (I)*/
    long odtype,        /* Output data type. (I)*/
    long option[],      /* Option indicating the information to be output. (I)
				           1 = Calculate incidence angle
				           2 = Calculate radar look angle
				           3 = Calculate range vector
				           4 = Calculate azimuth angle*/
    long offset,        /* Value to subtract from the range vector 
			               lengths. (I)*/
    char *subcmd,       /* Subcommand chosen. (I)*/
    double **latlon_lat,
    double **latlon_lon,/* Buffer to save the lat/lon (in radians) of previous
		 	               lines  (I/O)*/
    double **save_x,
    double **save_y,
    double **save_z     /* Buffer to save the cartesian coordinates of previous
			               lines  (I/O)*/
)
{
    /*
       The following variables have been declared global for image I/O routines.
     */
    extern long offset1;      /* Offsets into "ibuf" for input bands.        */
    extern long offset2[OBND]; 
                              /* Offsets into "obuf" for output bands.       */
    static	double	projparm[15];
                              /* Projection transformation package parameters.*/
    static  long current,     /* Index to current line.                      */
         idx = 0,             /* Index into save buffer.                     */
         next,	              /* Index to next line.                         */
         inunit,	          /* Input unit code for projection coordinates  */
         pjon = FALSE,
                              /* Flag indicating whether projection 
                                 transformation package has been initialized.*/
        prev;                 /* Index to previous line.                     */

    double	azm,              /* Azimuth angle (in degrees).                 */
        incid,                /* Incidence angle (in degrees).    		     */
        look,                 /* Look angle (in degrees).    			     */
        normal[3],            /* Normal of the point of interest (x,y,z).    */
        projx,                /* X projection coordinate.                    */
        projy,                /* Y projection coordiante.                    */
        range;                /* Magnitude of range vector (in meters).      */

    long geo = 0,             /* Output projection system.                   */
        i,k,                  /* Looping indices.                            */
        outproj = RADIANS,
                              /* Output projection units.                    */
        outzone = NULZONE,
                              /* Output zone code.                           */
        outsamp,              /* Output sample number.                       */
        prtprm[2],            /* Print option for the projection 
                                 transformation package.                     */
        status;               /* Status indicator.                           */

    /*
       Initialize the projection transformation package only once.
     */
    if (!pjon)
    {
        inunit = c_string2unit(ddr->proj_units);
        if (inunit == E_FAIL)
        {
            c_lasmsg(FALSE,"sartopo-warn", "Projection units %s are invalid",
                    ddr->proj_units);
            return(E_FAIL);
        }
        prtprm[0] = prtprm[1] = FALSE;
        for (i = 0; i < 15; i++)
            projparm[i] = 0;
        status = c_transinit(&(ddr->proj_code),&inunit,&(ddr->zone_code),
                &(ddr->datum_code),ddr->proj_coef,&geo,&outproj,&outzone,
                &(ddr->datum_code),projparm,prtprm,"");
        if (status != E_SUCC)
            return(E_FAIL);
        pjon = TRUE;

    }
    /*
       Set the appropriate indices for the current line, the previous line 
       and the next line.
     */
    if (line > window[0]+1)
    {
        prev = current;
        current = next;
        next = idx;
    }
    else
    {
        prev = 0;
        current = 1;
        next = 2;
    }

    /*
       Loop for each sample in the current input buffer.  Calculate the
       latitude/ longitude (in radians) for each point as well as the
       cartesian coordinates (x, y, z).
     */
    for (i=window[1], outsamp=0; i <= window[1]+window[3]-1; i++, outsamp++)
    {
        if (i == window[1])
        {
            projy = (coef[0] * line) + (coef[1] * (i - 1)) + coef[2];
            projx = (coef[3] * line) + (coef[4] * (i - 1)) + coef[5];
            status = c_trans(&(ddr->proj_code),&inunit,&geo,&outproj,&projx,
                    &projy,&latlon_lon[idx][i-1],&latlon_lat[idx][i-1]);
            if (status != E_SUCC)
                return(E_FAIL);
            sph2cart(latlon_lat[idx][i-1],latlon_lon[idx][i-1],
                    *(ibuf+(offset1)+outsamp),&save_x[idx][i-1],
                    &save_y[idx][i-1], &save_z[idx][i-1]);

            projy = (coef[0] * line) + (coef[1] * i) + coef[2];
            projx = (coef[3] * line) + (coef[4] * i) + coef[5];
            status = c_trans(&(ddr->proj_code),&inunit,&geo,&outproj,&projx,
                    &projy, &latlon_lon[idx][i],&latlon_lat[idx][i]);
            if (status != E_SUCC)
                return(E_FAIL);
            sph2cart(latlon_lat[idx][i],latlon_lon[idx][i],
                    *(ibuf+(offset1)+outsamp), &save_x[idx][i],&save_y[idx][i],
                    &save_z[idx][i]);
        }
        projy = (coef[0] * line) + (coef[1] * (i + 1)) + coef[2];
        projx = (coef[3] * line) + (coef[4] * (i + 1)) + coef[5];
        status = c_trans(&(ddr->proj_code),&inunit,&geo,&outproj,&projx,&projy,
                &latlon_lon[idx][i+1],&latlon_lat[idx][i+1]);
        if (status != E_SUCC)
            return(E_FAIL);
        if (i == window[1]+window[3]-1)
            sph2cart(latlon_lat[idx][i+1],latlon_lon[idx][i+1],
                    *(ibuf+(offset1)+outsamp),&save_x[idx][i+1],
                    &save_y[idx][i+1], &save_z[idx][i+1]);
        else
            sph2cart(latlon_lat[idx][i+1],latlon_lon[idx][i+1],
                    *(ibuf+(offset1)+outsamp+1),&save_x[idx][i+1],
                    &save_y[idx][i+1], &save_z[idx][i+1]);
        if (line >= window[0]+1)
        {
            /*
               Calculate the normal vector.
             */
            calc_normal(save_x,save_y,save_z,current,next,prev,i,normal);
            /*
               If the subcommand is -LINE, calculate the current 
               aircraft/spacecraft position along the input flight path.
             */
            if (*subcmd == 'L')
                findpt(latlon_lat[current][i],latlon_lon[current][i],position);
            /*
               Calculate the appropriate radar viewing angles and/or range 
               vectors.
             */
            findang(normal,&save_x[current][i],&save_y[current][i],
                    &save_z[current][i],&latlon_lat[current][i],
                    &latlon_lon[current][i], position, &range,&incid,
                    &look,&azm,option);
            /*
               Set the values in the output buffer.  Round values to the 
               nearest integer value if the output values are not REAL*4.
             */
            for (k=0; k < 4; k++)
            {
                if (odtype != EREAL)
                {
                    if ((option[k] == 1) && (incid >= 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(incid + .5);
                    else if ((option[k] == 1) && (incid < 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(incid - .5);
                    else if ((option[k] == 2) && (look >= 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(look + .5);
                    else if ((option[k] == 2) && (look < 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(look - .5);
                    else if ((option[k] == 3) && (range - offset >= 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(range - 
                                offset + .5);
                    else if ((option[k] == 3) && (range - offset < 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(range - 
                                offset - .5);
                    else if ((option[k] == 4) && (azm >= 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(azm + .5);
                    else if ((option[k] == 4) && (azm < 0))
                        *(obuf+(*(offset2+k)+outsamp)) = (float)(azm - .5);
                }
                else
                {
                    if (option[k] == 1) 
                        *(obuf+(*(offset2+k)+outsamp)) = (float)incid;
                    else if (option[k] == 2)
                        *(obuf+(*(offset2+k)+outsamp)) = (float)look;
                    else if (option[k] == 3)
                        *(obuf+(*(offset2+k)+outsamp)) =(float)(range - offset);
                    else if (option[k] == 4)
                        *(obuf+(*(offset2+k)+outsamp)) = (float)azm;
                }
            }
        }
    }

    (idx+1 > 2) ? idx = 0 : idx++;

    return(E_SUCC);
}
