/******************************************************************************
NAME:		POINT

PURPOSE: 	POINT opens the input and output images, calculates the
		transformation coefficients for mapping image coordinates
		to projection coordinates, reads each line of the input
		image, calls GET_INFO to obtain the appropriate information
		and writes the result to the output image.  The DDR is also
		updated after all image data has been processed.

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90

PROJECT:			LAS

ALGORITHM:

	Open the input and output images.
	Allocate memory for the I/O buffers and group the images.
	Find the transformation coefficients mapping image coordinates to
	  projection coordinates.
	Initialize the processing message utility.
	Initialize the min/max arrays if the min/max values are to be 
	  calculated.
	For each line in the input image
	    Write the current line of data to the output image.
	    Read the next line of data from the input image.
	    Calculate the appropriate radar viewing angles and range 
	      vectors for the current line.
	    Calculate min/max values if specified by the user.
	Stop the processing message utility.
	Update the output image's DDR.
	Write min/max values to the DDR.
	Close the input and output images.
	Free allocated buffer space.
	Return to the calling routine.

ALGORITHM REFERENCES:		None
******************************************************************************/
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"
#define	 IMG	1
#define	 OBND	4

void point
(
    struct PARBLK *vblock,  /*  TAE parameter block. (I) */
    char *hosin,            /* Host name of input image. (I) */
    char *hosout,           /* Host name of output image. (I) */
    long window[],          /* Window specification of input image. (I) */
    long band[],            /* Band to be processed. (I) */
    long outband,           /* Number of output bands to create. (I)*/
    long odtype,            /* Data type of output image. (I) */
    long msgtim,            /* Elapsed time between processing messages. (I) */
    char *minmax,           /* Min/max flag. (I) */
    long option[],          /* Option indicating the information to be
                               output.(I)
                               1 = Calculate incidence angle
                               2 = Calculate radar look angle
                               3 = Calculate range vector
                               4 = Calculate azimuth angle */
    long offset,            /* Value to subtract from the range vector 
			                   lengths. (I)	*/
    char *subcmd            /* Subcommand chosen. (I)*/
)
{

    /*
       The following variables have been declared global for image I/O routines.
     */
    extern int 	idesc[IMG+2];
                     /* Input image file descriptor used for image I/O.       */
    extern int 	odesc[2];
                     /* Output image file descriptor used for image I/O.      */
    extern int 	igdesc;	/* Input group descriptor used for image I/O.	      */
    extern int 	ogdesc;	/* Output group descriptor used for image I/O.	      */
    extern long	offset1;
                        /* Offsets into "ibuf" for input bands.   	          */
    extern long	offset2[OBND];    
                        /* Offsets into "obuf" for output bands.	          */
    extern long	curent;	/* Current processing line.                           */
    extern long	curcat;	/* Current processing band.                           */

    extern double bpos[3];/* Beginning position of flight path (cartesian
                               coordinates).                                  */

    struct	DDR	ddr;    /* Input image's DDR.                                 */

    static double minval[OBND],	
                        /* Min pixel value array.                             */
        maxval[OBND];   /* Max pixel value array.                             */

    double coef[6],     /* Coefficients mapping image coordinates to 
                           projection coordinates.                            */
         line_inc = 1.0,/* Line increment.                                    */
         position[3],   /* Aircraft position in cartesian coordinates (x,y,z).*/
         samp_inc = 1.0;/* Sample increment.                                  */

    float *ibuf,        /* Input image I/O buffer.                            */
        *obuf;          /* Ouput image I/O buffer.                            */

    long access=IREAD,  /* Image I/O access type.                     	      */
        bufsiz,         /* Size of image I/O buffer.                  	      */
        curlin,         /* Current line being processed.              	      */
        dtype = EREAL,  /* Data type of output buffer.                        */
        el,             /* Ending line.                                       */
        fatal = -1,     /* Fatal error flag.                                  */
        flag = FALSE,   /* Message flag for c_rtcoef.                         */
        i,              /* Looping index.                                     */
        icoords[8],     /* Image corner line/sample coordinates.              */
        idtype = EREAL,	/* Input image data type.                             */
        mmflag,         /* Min/max flag.                                      */
        msg_flag=ALLCAT,/* Flag for processing message.                       */
        nonfatal = 1,   /* Nonfatal error flag.                               */
        one = 1,        /* One.                                               */
        opt[2],         /* Data type conversion option.                       */
        status,         /* Function return status code.                       */
        upddr_flag,     /* Flag for update DDR.                               */
        zero = 0;       /* Value of zero.                                     */


    static double *latlon_lat[3], 
        *latlon_lon[3],
                        /* Buffer to save the lat/lon (in radians) of 
                           previous lines                                     */
        *save_x[3],
        *save_y[3],
        *save_z[3];
                        /* Buffer to save the cartesian coordinates of 
                           previous lines.                                    */

    /*
       Open the input image.
     */
    status = c_eopens((struct FDESC **)idesc,hosin,band,&window[0],&window[1],
            &window[2],&window[3],&offset1,&access,&idtype,&zero,&zero);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);
    idesc[1] = 0;
    /*
       Open the output image.
     */
    access = IWRITE;	
    opt[0] = odtype;
    opt[1] = 0;
    status = c_eopens((struct FDESC **)odesc,hosout,&outband,&one,&one,
            &window[2], &window[3], offset2,&access,&dtype,&zero,opt);
    if (status != E_SUCC)
    {
        c_eclose((struct FDESC **)odesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }
    odesc[1] = 0;

    /*
       Allocate I/O buffers and group images.
     */
    if ((ibuf = (float*)c_ealloc((struct FDESC **)idesc,&bufsiz)) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","sartopo-alloc",&nonfatal);
        c_eclose((struct FDESC **)odesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }
    if (c_egroup((struct FDESC **)idesc,(struct GDESC **)&igdesc,
            (unsigned char *)ibuf,&bufsiz) != E_SUCC)
    {
        c_eclose((struct FDESC **)odesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }

    if ((obuf = (float*)c_ealloc((struct FDESC **)odesc,&bufsiz)) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","sartopo-alloc",&nonfatal);
        c_eclose((struct FDESC **)odesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }
    if (c_egroup((struct FDESC **)odesc,(struct GDESC **)&ogdesc,
            (unsigned char *)obuf,&bufsiz) != E_SUCC)
    {
        c_eclose((struct FDESC **)odesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }

    /*
       Find the transformation coefficients mapping image coordinates to 
       projection coordinates.
     */
    status = c_getddr(hosin,&ddr);
    if (status != E_SUCC)
    {
        c_egclse((struct GDESC **)&ogdesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }

    /* 
       Allocate memory for latlon and save 
     */
    latlon_lat[0] = malloc((ddr.ns+2) * sizeof(double));
    latlon_lat[1] = malloc((ddr.ns+2) * sizeof(double));
    latlon_lat[2] = malloc((ddr.ns+2) * sizeof(double));

    if ((latlon_lat[0] == NULL)  || (latlon_lat[1] == NULL) ||
            (latlon_lat[2] == NULL))
    {     
        c_errmsg("error allocating memory for latlon_lat","sartopo-alloc",
                  &fatal);
    }

    latlon_lon[0] = malloc((ddr.ns+2) * sizeof(double));
    latlon_lon[1] = malloc((ddr.ns+2) * sizeof(double));
    latlon_lon[2] = malloc((ddr.ns+2) * sizeof(double));

    if ((latlon_lon[0] == NULL)  || (latlon_lon[1] == NULL) ||
            (latlon_lon[2] == NULL))
    {
        c_errmsg("error allocating memory for latlon_lon","sartopo-alloc",  
                 &fatal);
    }

    save_x[0] = malloc((ddr.ns+2) * sizeof(double));
    save_x[1] = malloc((ddr.ns+2) * sizeof(double));
    save_x[2] = malloc((ddr.ns+2) * sizeof(double));

    if ((save_x[0] == NULL)  || (save_x[1] == NULL) ||
            (save_x[2] == NULL))
        c_errmsg("error allocating memory for save_x","sartopo-alloc", &fatal);

    save_y[0] = malloc((ddr.ns+2) * sizeof(double));
    save_y[1] = malloc((ddr.ns+2) * sizeof(double));
    save_y[2] = malloc((ddr.ns+2) * sizeof(double));

    if ((save_y[0] == NULL)  || (save_y[1] == NULL) ||
            (save_y[2] == NULL))
        c_errmsg("error allocating memory for save_y","sartopo-alloc", &fatal);

    save_z[0] = malloc((ddr.ns+2) * sizeof(double));
    save_z[1] = malloc((ddr.ns+2) * sizeof(double));
    save_z[2] = malloc((ddr.ns+2) * sizeof(double));

    if ((save_z[0] == NULL)  || (save_z[1] == NULL) ||
            (save_z[2] == NULL))
        c_errmsg("error allocating memory for save_z","sartopo-alloc", &fatal);


    icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
    icoords[2] = icoords[6] = ddr.nl;
    icoords[5] = icoords[7] = ddr.ns;
    status = c_rtcoef(ddr.upleft,icoords,coef,&flag);
    if (status != E_SUCC)
    {
        c_egclse((struct GDESC **)&ogdesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }

    /*
       Setup for processing message utility.
     */
    c_inittm("SARTOPO","Processed","line","bands"," ",&window[2],&outband,
            &msgtim, &msg_flag);
    curent = 1;    
    c_startm();

    /*
       Initialize min/max arrays.  MMFLAG is set to be used within the 
       processing loop because it is quicker than using strcmp.
     */
    if (strcmp(minmax,"YES") == 0)
    {
        c_inmnmx(&odtype,&outband,minval,maxval);
        mmflag = TRUE;
    }

    /*
       If POINT mode has been selected, copy the user input position into the
       appropriate variable.
     */
    if (*subcmd == 'P')
    {
        position[0] = bpos[0];
        position[1] = bpos[1];
        position[2] = bpos[2];
    }

    /*
       Main processing loop.
     */
    el = window[0] + window[2] - 1;
    for (curlin = window[0]+1; curlin <= el; curlin++,curent++)
    {
        /*
           Write the current line of data to the output image and read the next
           line of data from the input image.
         */ 
        if (c_estep((struct GDESC **)&ogdesc) != E_SUCC)
        {
            c_egclse((struct GDESC **)&ogdesc);
            c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
        }
        if (c_estep((struct GDESC **)&igdesc) != E_SUCC)
        {
            c_egclse((struct GDESC **)&ogdesc);
            c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
        }
        /*
           Calculate the appropriate radar viewing angles and/or range vectors.
         */
        if (curlin == window[0]+1)
        {
            status = get_info(ibuf,obuf,window[0]-1,position,window,&ddr,coef,
                    odtype,option, offset,subcmd,(double **)&latlon_lat,
                    (double **)&latlon_lon, (double **)&save_x,
                    (double **)&save_y, (double **)&save_z);
            if (status != E_SUCC)
            {
                c_egclse((struct GDESC **)&ogdesc);
                c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
            }
            status = get_info(ibuf,obuf,window[0],position,window,&ddr,coef,
                    odtype,option,offset,subcmd,(double **)&latlon_lat,
                    (double **)&latlon_lon,(double **)&save_x,
                    (double **)&save_y,(double **)&save_z);
            if (status != E_SUCC)
            {
                c_egclse((struct GDESC **)&ogdesc);
                c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
            }
            if (c_estep((struct GDESC **)&igdesc) != E_SUCC)
            {
                c_egclse((struct GDESC **)&ogdesc);
                c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
            }
        }
        status = get_info(ibuf,obuf,curlin,position,window,&ddr,coef,odtype,
                option,offset,subcmd,(double **)&latlon_lat,
                (double **)&latlon_lon,(double **)&save_x,
                (double **)&save_y,(double **)&save_z);
        if (status != E_SUCC)
        {
            c_egclse((struct GDESC **)&ogdesc);
            c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
        }
        /*
           Calculate min/max values if necessary.
         */
        if (mmflag)
            for (i=0; i < outband; i++)
                status = c_pxmnmx((long *)obuf + (*(offset2+i)),&window[3],
                        &minval[i],&maxval[i],&odtype); 
        if (curlin == el)
        {
            if (c_estep((struct GDESC **)&ogdesc) != E_SUCC)
            {
                c_egclse((struct GDESC **)&ogdesc);
                c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
            }
            status = get_info(ibuf,obuf,el+1,position,window,&ddr,coef,
                    odtype,option,offset,subcmd,(double **)&latlon_lat,
                    (double **)&latlon_lon,(double **)&save_x,
                    (double **)&save_y,(double **)&save_z);
            if (status != E_SUCC)
            {
                c_egclse((struct GDESC **)&ogdesc);
                c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
            }
            if (mmflag)
                for (i=0; i < outband; i++)
                    status = c_pxmnmx((long *)obuf + (*(offset2+i)),&window[3],
                            &minval[i],&maxval[i],&odtype); 
            if (c_estep((struct GDESC **)&ogdesc) != E_SUCC)
            {
                c_egclse((struct GDESC **)&ogdesc);
                c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
            }
        }
    }
    /*
       Turn off display messages.
     */
    c_stoptm();

    /*
       Update the DDR.
     */
    line_inc = (double)one;
    samp_inc = (double)one;
    upddr_flag = NOCOMB;
    status = c_upddr(vblock,(char (*)[256])hosin,hosout,&one,
            (long (*)[4])window,(long (*)[257])band,&one,&line_inc, 
            &samp_inc,&outband,&upddr_flag);
    if (status != E_SUCC)
    {
        c_egclse((struct GDESC **)&ogdesc);
        c_clean(vblock,"sartopo",(char (*)[256])hosout,&one);
    }

    /*
       Write min/max values to the DDR. 
     */
    if (c_svmnmx(hosout,&outband,minval,maxval,minmax) != E_SUCC)
        c_errmsg("Nonfatal error encountered","sartopo-warn",&one);

    /*
       Close all image files in the group.
     */
    if (c_egclse((struct GDESC **)&igdesc) != E_SUCC)
        c_errmsg("Nonfatal error encountered","sartopo-warn",&one);

    if (c_egclse((struct GDESC **)&ogdesc) != E_SUCC)
        c_errmsg("Nonfatal error encountered","sartopo-warn",&one);

    /*
       Free buffer.
     */
    free(ibuf);
    free(obuf);
    free (latlon_lat[0]);
    free (latlon_lat[1]);
    free (latlon_lat[2]);
    free (latlon_lon[0]);
    free (latlon_lon[1]);
    free (latlon_lon[2]);
    free (save_x[0]);
    free (save_x[1]);
    free (save_x[2]);
    free (save_y[0]);
    free (save_y[1]);
    free (save_y[2]);
    free (save_z[0]);
    free (save_z[1]);
    free (save_z[2]);

    return;
}
