/******************************************************************************
NAME:		SARTOPO

PURPOSE: 	Calculates the incidence angle, radar look angle and/or the 
		range vectors for each pixel of the input elevation 
		data.  Two subcommands are available.  -LINE calculates the
		above information based on a user defined flight path.  -POINT
		calculates the above information based on a stationary point.  
		In addition to the above information, -POINT allows calculation
		of the azimuth angle.

PROGRAM HISTORY:
   Version	Date	Author    Request
   -------	----	------	  -------
     1.0	10/90	L Huewe	  
     2.0	01/00	D Hames	  Modify arrays latlon and save and allocate 
				  using ddr.ns instead of MAXNS.  This was done
				  to support images > 2 gb. Add prototypes. 
				  SRF #893.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   SARTOPO must be run under TAE.

PROJECT:			LAS

ALGORITHM DESCRIPTION:

	Initialize LAS and TAE.
	Get the user specified parameters and TAE globals.
	Convert the input latitude/longitude values to radians and cartesian
	   coordinates (x, y, z).
	Determine the number of output image bands to be created.
	Process the image.
	Update the output image history file.
	Ouput a standard completion message.

ALGORITHM REFERENCES:		none

******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "sartopo.h"
#include "las.h"	
#include "util.h"
#include "geompak.h"
#include "protos.h"
#define	 IMG	1
#define	 IBND	1
#define	 OBND	4

/*
   The following variables have been declared global for image I/O routines.
*/

int  idesc[IMG+2];     /* Input image file descriptor used for image I/O.    */
int  odesc[2];         /* Output image file descriptor used for image I/O.   */
int  igdesc;           /* Input group descriptor used for image I/O.         */
int  ogdesc;           /* Output group descriptor used for image I/O.        */
long offset1;          /* Offsets into IBUF for input bands.                 */
long offset2[OBND];    /* Offsets into OBUF for output bands.                */
extern long curent;    /* Current processing line.                           */
extern long curcat;    /* Current processing band.                           */

double bpos[3],        /* Beginning position of flight path (cartesian 
			              coordinates).                                      */
    elev,              /* Elevation (in meters) of flight path (-LINE) or 
                          specified point (-POINT).                          */
    epos[3],           /* Ending position of flight path (cartesian
                          coordinates).                                      */
    rblat,             /* Beginning latitude of flight path (radians).       */
    rblon,             /* Beginning longitude of flight path (radians).      */
    relat,             /* Ending latitude of flight path (radians).          */
    relon;             /* Ending longitude of flight path (radians).         */

void main
(void)
{
    struct PARBLK vblock; /* TAE parameter block.                           */

    double blat, /* Beginning latitude of flight path (-LINE) or 
                    latitude of specified point (-POINT) (in decimal
                    degrees).                                               */
        blon,    /* Beginning longitude of flight path (-LINE) or
                    longitude of specified point (-POINT) (in decimal
                    degrees.                                                */
        elat,    /* Ending latitude of flight path (in decimal 
                    degrees).		  			                            */
        elon;    /* Ending longitude of flight path (in decimal 
                    degrees.                                                */

    long band[2], /* Band number to be processed.                           */
        i,          /* Looping index.                                       */
        msgtim,     /* Number of seconds between processing messages.       */
        odtype,     /* Data type of the output image.                       */
        offset,     /* Value to subtract from the range vector lengths.     */
        one = 1,    /* Constant one.                                        */
        option[4],	/* Option indicating the information to be output.
                   1 = Calculate incidence angle
                   2 = Calculate radar look angle
                   3 = Calculate range vector
                   4 = Calculate azimuth angle                              */
        outband,    /* Number of output bands to create.                    */
        std_out,    /* LUN to standard output.                              */
        window[4];  /* Window for input image.                              */

    char hosin[CMLEN], /* Host name of input DEM image.                     */
        hosout[CMLEN], /* Host name of output image.                        */
        minmax[4],     /* Min/max flag.                                     */
        subcmd[6],     /* Subcommand chosen.                                */
        taein[CMLEN],  /* TAE name of input DEM image.                      */
        taeout[CMLEN]; /* TAE name of output image.                         */

    /*
       Initialize LAS and TAE.
     */
    c_inlas(&vblock,&std_out);

    /* 
       Get parameters and globals.
     */
    getpar(&vblock,taein,hosin,taeout,hosout,window,band,&odtype,&blat,&blon,
            &elat,&elon,&elev,option,&offset,&msgtim,minmax,subcmd);

    /*
       Convert the input latitude/longitude values to radians and to cartesian
       coordinates.
     */
    rblat = blat * DEG2RAD;
    rblon = blon * DEG2RAD;
    sph2cart(rblat,rblon,elev,&bpos[0],&bpos[1],&bpos[2]);

    if (*subcmd == 'L')
    {
        relat = elat * DEG2RAD;
        relon = elon * DEG2RAD;
        sph2cart(relat,relon,elev,&epos[0],&epos[1],&epos[2]);
    }

    /*
       Process the image.
     */
    for (i=0,outband=0; i < 4; i++)
        if (option[i] != 0)
            outband++;
    point(&vblock,hosin,hosout,window,band,outband,odtype,msgtim,minmax,
            option,offset,subcmd); 

    /*
       Update the history file.
     */
    if (c_hstout(&vblock,hosout,(char (*)[256])hosin,&one," ") != E_SUCC)
        c_errmsg("Nonfatal error encountered","sartopo-warn",&one);

    /*
       Output completion message. 
     */
    c_complt("SARTOPO",(char (*)[256])taeout,&one);
}


/******************************************************************************
NAME:		GETPAR

PURPOSE: 	Retrieve the user specified parameters and TAE globals.	

ALGORITHM DESCRIPTION:

	Determine the subcommand chosen.
	Retrieve and parse the input image specification.
	Output an error message if more than one band was specified.
	Retrieve the output image name.
	Retrieve the output image data type.
	If the subcommand chosen was -LINE
	    Retrieve the beginning and ending latitude and longitude of the
	      aircraft/spacecraft flight path.
	    Convert the degrees, minutes, seconds values to decimal degrees.
	Else
	    Retrieve the latitude and longitude of the stationary 
	      aircraft/spacecraft position. 
	    Convert the degrees, minutes, seconds value to decimal degrees.
	Retrieve the elevation value (in meters).
	Retrieve the offset value.
	Retrieve the information to be output.
	Retrieve the TAE globals $MSGTIM and $MINMAX.
	Return to the calling routine.

ALGORITHM REFERENCES:		none
******************************************************************************/
void getpar
(
    struct PARBLK *vblock,/* Pointer to TAE parameter block. (I/O)	          */
    char taein[CMLEN],	/* TAE file name of input image. (O)           	      */
    char hosin[CMLEN],  /* Host file name of input image. (O)                 */
    char *taeout,       /* TAE file name of output image. (O)		          */
    char *hosout,       /* Host file name of output image. (O)                */
    long window[],      /* Window specifications for input image. (O)         */
    long band[],        /* Band specification for input image. (O)            */
    long *odtype,       /* Output data type. (O)                              */
    double *blat,       /* Beginning latitude of flight path (-LINE) or
                           latitude of specified point (-POINT). (O)          */
    double *blon,       /* Beginning longitude of flight path (-LINE) or
                           longitude of specified point (-POINT). (O)         */
    double *elat,       /* Ending latitude of flight path. (O)                */
    double *elon,       /* Ending longitude of flight path. (O)               */
    double *elev,       /* Elevation (in meters) of flight path (-LINE) or
                           specified point (-POINT). (O)                      */
    long option[],      /* Option indicating the information to be output. (O)
                            1 = Calculate incidence angle
                            2 = Calculate radar look angle
                            3 = Calculate range vector
                            4 = Calculate azimuth angle	                      */
    long *offset,       /* Value to subtract from the vector lengths. (O)     */
    long *msgtim,       /* Elapsed time between processing messages. (O)      */
    char *minmax,       /* Min/max flag. (O)                                  */
    char *subcmd        /* Subcommand chosen. (O)                             */
)
{
    struct VARIABLE	*val;	/* Pointer to a TAE variable within v-block.      */

    long deg,               /* Degrees.	                                     */
        element = 0,        /* Parameter element number.                     */
        fatal = -1,         /* Fatal error flag.                             */
        i,j,                /* Looping variables.                            */
        imglim = 1,         /* Maximum number of images to be processed.	 */
        minute,             /* Minutes.                                      */
        nbands,             /* Number of input bands specified.              */
        nmrimg,             /* Number of input images specified.             */
        nonfatal = 1,       /* Nonfatal error flag.                          */
        qflag[2],           /* Flag indicating if windows and bands were 
                               specified.                                    */
        same,               /* Flag indicating if all images have same size 
                               window.                                       */
        sec,                /* Seconds.                                      */
        status,             /* Function return status code.                  */
        totbnd;             /* Total number of bands specified.              */

    char dtype[5];          /* Data type of output image.                    */

    /*
       Retrieve the subcommand chosen.
     */
    val = p_find(vblock,"_SUBCMD");	
    strcpy(subcmd,SVAL(*val,0));

    /*
       Retrieve and parse input image specification. 
     */
    status = c_getin(vblock,"IN",&element,(char (*)[256])taein,
            (char (*)[256])hosin, &nmrimg,(long (*)[4])window,
            (long (*)[257])band,&nbands,&totbnd, &imglim,&same,
            (long (*)[2])qflag);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);
    if (totbnd > 1)
    {
        c_errmsg("Maximum number of input bands exceeded","sartopo-maxbnd",
                &nonfatal);
        c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);
    }

    /*
       Retrieve output image name.
     */
    if (c_getout(vblock,"OUT",(char (*)[256])taeout,
                (char (*)[256])hosout)==E_FAIL)
    {
        c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);
    }

    /*
       Retrieve output data type.
     */
    val = p_find(vblock,"ODTYPE");	
    strcpy(dtype,SVAL(*val,0));
    if (strcmp(dtype,"BYTE") == 0)
        *odtype = EBYTE;
    else if (strcmp(dtype,"I*2") == 0)
        *odtype = EWORD;
    else if (strcmp(dtype,"I*4") == 0)
        *odtype = ELONG;
    else if (strcmp(dtype,"R*4") == 0)
        *odtype = EREAL;

    /*
       Retrieve the appropriate latitude and longitude values based on the 
       subcommand chosen.  Read the input value as degrees, minutes and seconds,
       convert the value to DMS and then convert the DMS value to total degrees.
     */
    if (*subcmd == 'L')
    {
        val = p_find(vblock,"BEGLAT");
        deg = IVAL(*val,0);
        minute = IVAL(*val,1);
        sec = IVAL(*val,2);
        if (deg > 0)
            *blat = (deg * 1000000) + (minute * 1000) + sec;
        else
            *blat = (deg * 1000000) - (minute * 1000) - sec;
        if ((c_decdeg(blat,"DMS","LAT")) != E_SUCC)
            c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);

        val = p_find(vblock,"BEGLONG");
        deg = IVAL(*val,0);
        minute = IVAL(*val,1);
        sec = IVAL(*val,2);
        if (deg > 0)
            *blon = (deg * 1000000) + (minute * 1000) + sec;
        else
            *blon = (deg * 1000000) - (minute * 1000) - sec;
        if ((c_decdeg(blon,"DMS","LON")) != E_SUCC)
            c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);

        val = p_find(vblock,"ENDLAT");
        deg = IVAL(*val,0);
        minute = IVAL(*val,1);
        sec = IVAL(*val,2);
        if (deg > 0)
            *elat = (deg * 1000000) + (minute * 1000) + sec;
        else
            *elat = (deg * 1000000) - (minute * 1000) - sec;
        if ((c_decdeg(elat,"DMS","LAT")) != E_SUCC)
            c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);

        val = p_find(vblock,"ENDLONG");
        deg = IVAL(*val,0);
        minute = IVAL(*val,1);
        sec = IVAL(*val,2);
        if (deg > 0)
            *elon = (deg * 1000000) + (minute * 1000) + sec;
        else
            *elon = (deg * 1000000) - (minute * 1000) - sec;
        if ((c_decdeg(elon,"DMS","LON")) != E_SUCC)
            c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);
    }
    else
    {
        val = p_find(vblock,"LAT");
        deg = IVAL(*val,0);
        minute = IVAL(*val,1);
        sec = IVAL(*val,2);
        if (deg > 0)
            *blat = (deg * 1000000) + (minute * 1000) + sec;
        else
            *blat = (deg * 1000000) - (minute * 1000) - sec;
        if ((c_decdeg(blat,"DMS","LAT")) != E_SUCC)
            c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);

        val = p_find(vblock,"LONG");
        deg = IVAL(*val,0);
        minute = IVAL(*val,1);
        sec = IVAL(*val,2);
        if (deg > 0)
            *blon = (deg * 1000000) + (minute * 1000) + sec;
        else
            *blon = (deg * 1000000) - (minute * 1000) - sec;
        if ((c_decdeg(blon,"DMS","LON")) != E_SUCC)
            c_errmsg("Fatal error encountered","sartopo-fatal",&fatal);
    }
    /*
       Retrieve the elevation.
     */
    val = p_find(vblock,"ELEV");
    *elev = RVAL(*val,0);

    /*
       Retrieve the offset value.
     */
    val = p_find(vblock,"OFFSET");
    *offset = IVAL(*val,0);

    /*
       Retrieve the information to be output.
     */
    option[0] = option[1] = option[2] = option[3] = 0;
    val = p_find(vblock,"CALCOPT");
    for (i=0, j=0; i < (*val).v_count; i++)
    {
        if (strcmp(SVAL(*val,i),"INCID") == 0)
        {
            option[j] = 1;
            j++;
        }
        else if (strcmp(SVAL(*val,i),"LOOK") == 0)
        {
            option[j] = 2;
            j++;
        }
        else if (strcmp(SVAL(*val,i),"RANGE") == 0)
        {
            option[j] = 3;
            j++;
        }
        else if (strcmp(SVAL(*val,i),"AZM") == 0)
        {
            option[j] = 4;
            j++;
        }
    }
    /*
       Retrieve globals.
     */
    val = p_find(vblock,"$MSGTIM");
    *msgtim = IVAL(*val,0);
    val = p_find(vblock,"$MINMAX");
    strcpy(minmax,SVAL(*val,0));

    return;
}
