#include "scatter.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void accum
(
    long wind[][4],		/* Processing window specification */
    long harray[114][114], 	/* Histogram bin populations */
    float rmin[2],		/* Minimum pixel values */
    float rmax[2],		/* Maximum pixel values */
    long *kmin, 			/* Minimum pixel count */
    long *kmax,			/* Maximum pixel count */
    long msgtim, 		/* Message time interval */
    long dr_count[2]		/* Data range min-max count */
)
/*
**********************************************************************
NAME: ACCUM

FUNCTION: accumulates the scatter plot histogram bin populations.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
     1           Sept-83   R. J. Tighe
     2           Sep-87    D. Etrheim/edc   Prf #4228
     3           May-90    B. Allam	    Conversion to 5.0
     4        16-Nov-90    K. Kunska        Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM:

   Initialize histogram bin populations.
   Compute scale and offset factors.
   Initialize and enable the processing message utility.
   Accumulate histogram bin populations.
   If both pixel values are within specified data ranges then 
      Transform the input array using scale and offset values.
      Use pixel values to set up histogram bin indices.
      Increment histogram bin population.
   End-if
   If the maximum size for the histogram has been exceeded, 
      Print a fatal error message.

*************************************************************************
*/
{

long    zero = 0;
long    two = 2;


static	long	xhdim = 114;	/* Plot dimensions along the X axis */
static	long	yhdim = 114;	/* Plot dimensions along the Y axis */

float	scale[2], 		/* Scaling factor */
	offset[2], 		/* Offset values */
	temp1, 			/* Intermediate result */
	temp2; 			/* Intermediate result */

long	curlin, 		/* Current line */
	ix, iy,			/* Array indices */
	lcount,			/* Loop counter and array index */
	msgflag,		/* Flag for the processing message type */
	nlines, 		/* Number of lines in the input image */
	npixels, 		/* Number of pixels in the input image */
	pix, 			/* Current pixel */
	status,			/* Return status */
	type;			/* Data type */

/*
Initialize histogram bin populations.
--------------------------------------
*/
type = ELONG;
for (lcount = 0; lcount < xhdim; lcount++)
  c_pxfill(harray[lcount], &type, &xhdim, &zero);
nlines = wind[0][NL];
npixels = wind[0][NS];

/*
Compute scale and offset factors.
----------------------------------
*/
if ((rmax[0] - rmin[0])  ==  0.0) 
   scale[0] =(float) ( xhdim - 1);
else
   scale[0] = ((float) (xhdim - 1)) / (rmax[0] - rmin[0]);
offset[0] = 1 - scale[0] * rmin[0];

if ((rmax[1]-rmin[1])  ==  0.0) 
   scale[1] = (float) (yhdim - 1);
else 
   scale[1] = ((float) (yhdim - 1)) / (rmax[1] - rmin[1]);
offset[1] = 1 - scale[1] * rmin[1];

*kmax = 0;
*kmin = 2147483647;

/*
Initialize and enable the processing message utility.
-------------------------------------------------------
*/
msgflag = 0;
c_inittm ("SCATTER", "Processed", "line", "bands", " ", &nlines, &two, 
          &msgtim, &msgflag);
curent = 1;
c_startm();

/*
Accumulate histogram bin populations.
If both pixel values are within specified data ranges then transform 
the input array using scale and offset values.
Use pixel values to set up histogram bin indices.
Increment histogram bin population.
------------------------------------------------------------------------
*/
status = c_estep (&gdesc);
if (status != E_SUCC)
   msg_err ("!Error reading input image", "scatter-read");

for (curlin = 1; curlin <= nlines; curlin++)
   {
   for (pix = 0; pix < npixels; pix++)
      {
      if (dr_count[0]  !=  0) 
         {
         if ((iobuf[ipoint[0]+pix] >= rmin[0])  && 
             (iobuf[ipoint[0]+pix]  <=  rmax[0])) 
            {
            temp1 = (iobuf[ipoint[0]+pix] * scale[0] + offset[0] + 0.5);
            ix = (long)(temp1);
            }
         else
            {
            continue;
            } 
         }
      else
         {
         temp1  = (iobuf[ipoint[0]+pix] * scale[0] + offset[0] + 0.5);
         ix = (long)(temp1);
         } 

      if (dr_count[1]  !=  0) 
         {
         if ((iobuf[ipoint[1]+pix]  >=  rmin[1])  && 
             (iobuf[ipoint[1]+pix]  <=  rmax[1]))  
            {
            temp2 = (iobuf[ipoint[1]+pix] * scale[1] + offset[1] + 0.5);
            iy = (long)(temp2);
            }
         else
            {
            continue;
            } 
         }
      else
         {
         temp2 = (iobuf[ipoint[1]+pix] * scale[1] + offset[1] + 0.5);
         iy = (long)(temp2);
         } 
/*
      if (ix == 98)
        if (iy == 48)
           ix = ix;
*/
/*
If the maximum size for the histogram has been exceeded, print a fatal
error message.
-----------------------------------------------------------------------
*/
      if ((ix > 114) || (iy > 114))
         msg_err ("!Maximum size for histogram has been exceeded", 
                  "scatter-hist");
      harray[ix-1][iy-1] = harray[ix-1][iy-1] + 1;
      *kmax = max(*kmax, harray[ix-1][iy-1]);
      *kmin = min(*kmin, harray[ix-1][iy-1]); 
      }  /* do */
   curent++;
   status = c_estep (&gdesc);
   if (status != E_SUCC)
      msg_err ("!Error reading input image", "scatter-read");
   }  /* do */
c_stoptm();
}
