#include "scatter.h"
#include <string.h>
#include "util.h"
#include "pixman.h"
#include "protos.h"

void getparm
(
    char lasin[][CMLEN],		/* Las input file names */
    char hostin[][CMLEN],	/* Host input file names */
    long window[MAXIMG][4],	/* Processing window */
    char chars[],		/* Plotting characters */
    float smin[2],		/* Minimum pixel values for plotting */
    float smax[2], 		/* Maximum pixel values for plotting */
    char infile[],		/* Input file with plotter characters */
    long *nbrplt, 		/* Number of plotting characters (or bins)*/
    long *msgtim,		/* Message time interval */
    char option[5],		/* Scaling option */
    long *chrfil,		/* Infile option */
    long mins[],		/* Range begin and end */
    long maxs[],
    long dr_count[2], 		/* Data min-max range count */
    long *nimages 		/* Number of images */
)
/*
**********************************************************************
NAME: GETPARM

FUNCTION:  Retrieves the user-supplied parameters.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
     1          Sep-83    R. J. Tighe
     2          Sep-87    D. Etrheim/edc    Prf #4228
     3          May-90    B. Allam	    Conversion to 5.0
     4       16-Nov-90    K. Kunska         Added .dat as default extention for
                                            INFILE and validation of data type

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM:

   Get the input image.
   Validate the data type of the input image(s)
   Validate the number of input image bands.
   Get the file of plotter characters.
   If the character file was not specified, 
      Get the plotting characters parameter.   
   Get the minimum and maximum values for the bin range.
   Get the number of character bins.
   Validate the number of values for the bin ranges and 
   Modify the number of bins accordingly.
   Combine into one array
   If a character file was specified, 
      Call RDFIL to read in the plotter characters.
   Get the data range limits for the both images.
   Find the minimum and maximum values in the input image.
   Check user specified data ranges.
   If the data range limits were not specified, 
      Get the scaling option.
   Get the print option.
   Open the output unit.
   Get global parameter $MSGTIM.

************************************************************************
*/
{

long    zero = 0;
long    one = 1;
char    *prog_name = "SCATTER"; /* program name                 */


char	char1[95], 		/* Plotting characters */
	fileutil_file[CMLEN],	/* File name for fileutil */
	fill_val;		/* Fill value */

double  in_max[2], in_min[2];	/* Max and min values from the input image */

float	dr1[2], 		/* Data range limits for the 1st image (Y) */
	dr2[2];			/* Data range limits for the 2nd image (X) */

long	access,			/* File access type */
	dtype[2],		/* Array for different data types */
	fill_num,		/* Number of elements to fill */
	fill_type,		/* Data type of the array to be filled */
	flag[MAXIMG][2],	/* Window or band option flag */
	i, 			/* Loop counter and array index */
        imglimit = 2,	        /* Maximum number of images */
	max1[94], 		/* Maximum value for the bin range */
	min1[94], 		/* Minimum valuse for the bin range */
	nbrc1,			/* Number of plotter characters */
	nbr_from, 		/* Number of minimum range values */
	nbr_to, 		/* Number of maximum range values */
	omax,			/* Temporary maximum value */
	omin,			/* Temporary minimum value */ 
        same_window,		/* Flag for same window size */
	status,			/* Return status */
	totbnd;			/* Total number of bands */

struct VARIABLE *var, *vc1, *mi1, *ma1, *nb, *mnmx1, *mnmx2, *opt;

dr1[0] = dr1[1] = 0.0;
dr2[0] = dr2[1] = 0.0;
dr_count[0] = dr_count[1] = 0;
/*
Get the input image.
--------------------
*/
status = c_getin (&block, "IN", &zero, lasin, hostin, nimages, window, 
                  bands, nbands, &totbnd, &imglimit, &same_window, flag);
if (status != E_SUCC)
   msg_err("!Error accessing the input file name", "scatter-infile");
if (same_window == 0)
   msg_err ("!Image windows differ in size", "scatter-window");

/*
Validate the data type of the input image(s)
--------------------------------------------
*/
c_gettyp (hostin, &one, &dtype[0]);
if (*nimages > 1)
   {
   c_gettyp (hostin[1], &one, &dtype[1]);
   if (dtype[0] != dtype[1])
      msg_err ("!Input images with different data types are not allowed", 
               "scatter-dtype");
   }

/*
Validate the number of input image bands.
------------------------------------------
*/
if (totbnd <= 1)
  msg_err("!Input image must contain two bands", "scatter-nbnd");

if (*nimages == 1)
  {
  strcpy (lasin[1], lasin[0]);
  bands[1][0] = bands[0][1];
  }

if (nbands[0] > 2)
  nbands[0] = 2;
if (totbnd > 2) 
   {
   msg_err ("?More than two input bands specified", "scatter-totbnd");
   msg_err ("?Only the first two bands will be used", "scatter-totbnd");
   if (nbands[0] == 1)
     if (nbands[1] > 1)
       nbands[1] = 1;
   }  

/*
Get the file of plotter characters.
-------------------------------------
*/
access = IREAD;
status = c_getnam (&block, "INFILE", &zero, infile, "dat", &access);
if (status  !=  E_SUCC)
  msg_err ("!Error accessing input file name", "scatter-infile"); 

/*
If the character file was not specified, get the plotting characters 
parameter. 
---------------------------------------------------------------------
*/
if (!strcmp(infile, ""))
   {
   vc1 = p_find(&block, "PLOTCHR"); 
   if ((*vc1).v_count == 0)
      {
      strcpy(char1, " .,?<>%/;:'\\|$!@#^&*()-_=+`~{}[]");
      strcpy(&char1[strlen(char1)], "asdfghjklqwertyuiopzxcvbnm");
      strcpy(&char1[strlen(char1)], "ASDFGHJKLQWERTYUIOPZXCVBNM1234567890");
      }
   else
      {
      fill_val = ' ';
      fill_num = 94;
      fill_type = EBYTE;
      c_pxfill (char1, &fill_type, &fill_num, &fill_val);
      strcpy(char1, SVAL(*vc1,0));
      }
   nbrc1 = (long) strlen(char1);

/*
Get the minimum and maximum values for the bin range.
------------------------------------------------------
*/
   mi1 = p_find(&block, "MINVAL"); 
   for (i = 0; i < (*mi1).v_count; i++)   
      min1[i] = IVAL(*mi1,i);
   ma1 = p_find(&block, "MAXVAL"); 
   for (i = 0; i < (*ma1).v_count; i++)   
      max1[i] = IVAL(*ma1,i);

/*
Get the number of character bins.
----------------------------------
*/
   nb = p_find(&block, "NBINS"); 
   *nbrplt = IVAL(*nb, 0);

/*
Validate the number of values for the bin ranges and modify the number 
of bins accordingly.
-----------------------------------------------------------------------
*/
   nbr_from = (*mi1).v_count;
   nbr_to = (*ma1).v_count;
   if (nbr_from  !=  nbr_to) 
     msg_err ("!Number of MINVAL and MAXVAL values must be equal", 
              "scatter-equal");
   if (nbr_from != 0) 
      {
      *nbrplt = nbr_from;
      *chrfil = TRUE;
      }
   else
      *chrfil = FALSE;

/*
Combine into one array
----------------------
*/
   if (nbrc1 > *nbrplt) nbrc1 = (*nbrplt);
   for (i = 0; i < nbrc1; i++)
      {
      chars[i] = char1[i];
      mins[i] = min1[i];
      maxs[i] = max1[i];
      }  
   }

/*
If a character file was specified, call RDFIL to read in the plotter
characters.
---------------------------------------------------------------------
*/
else
   {
   rdfil (infile, chars, mins, maxs, nbrplt); 
   *chrfil = TRUE;
   }  

/*
Get the data range limits for the both images.
-----------------------------------------------
*/
mnmx1 = p_find(&block, "RANGE1");
dr_count[0] = (*mnmx1).v_count;
if (dr_count[0] != 0)
   {
   dr1[0] = RVAL(*mnmx1, 0);
   dr1[1] = RVAL(*mnmx1, 1); 
   }
mnmx2 = p_find(&block, "RANGE2");
dr_count[1] = (*mnmx2).v_count;
if (dr_count[1] != 0)
   {
   dr2[0] = RVAL(*mnmx2, 0);
   dr2[1] = RVAL(*mnmx2, 1); 
   }

/*
Find the minimum and maximum values in the input image.
---------------------------------------------------------
*/
if ((nbands[0] == 1) && (nbands[1] >= 1))
   c_minmax (hostin, nimages, window, bands, nbands, in_min, in_max);
else if (nbands[0] == 2)
   c_minmax (hostin, &one, window, bands, &nbands[0], in_min, in_max);

smin[0] = (float)in_min[0]; 
smin[1] = (float)in_min[1];
smax[0] = (float)in_max[0]; 
smax[1] = (float)in_max[1];

/*
Check user specified data ranges.
------------------------------------
*/
for (i = 0; i < 2; i++)
   {
   if (dr_count[i] != 0)
      {
      if (i  ==  0) 
         {
         if ((dr1[0] < smin[i]) || (dr1[1] > smax[i])) 
            msg_err ("!Specified data range limits not within data boundaries", 
                     "scatter-lim");
         else
            {
            smin[i] = dr1[0];
            smax[i] = dr1[1];
            } 
         }
      else
         {
         if ((dr2[0]  <  smin[i])  ||  (dr2[1]  >  smax[i])) 
            msg_err ("!Specified data range limits not within data boundaries",
                     "scatter-lim");
         else
            {
            smin[i] = dr2[0];
            smax[i] = dr2[1];
            }  /* if */
         } /* if */
      } /* if */
   }  /* do */

/*
If the data range limits were not specified, get the scaling option.
----------------------------------------------------------------------
*/
if (((*mnmx1).v_count  ==  0)  &&  ((*mnmx2).v_count  ==  0)) 
   {
   opt = p_find(&block, "SCALOPT");
   strcpy(option, SVAL(*opt,0));
   }
else
   strcpy(option, "IND");

if (!(strncmp(option, "SAME", 4)))
   {
   omin = (long) min(smin[0], smin[1]);
   omax = (long) max(smax[0], smax[1]);
   smin[0] = (float) omin;
   smax[0] = (float) omax;
   smin[1] = (float) omin;
   smax[1] = (float) omax;
   }  

/*
Get the print option.
----------------------
*/
count = 0;
strcpy (fileutil_file, prog_name);
if (p_find (&block, "PRINT")->v_count != 0)
   {
   status = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], 
                      fileutil_file, print_file);
   if (status != E_SUCC)
      msg_err ("!Error occured while retrieving the print file name", 
               "scatter-prtopen");
   }

/*
Open the output unit.
---------------------
*/
if ((print[LP]) || (print[FIL]))
   {
   fp = fopen(fileutil_file, "w");
   strcpy (out[count++], print_file);
   }
else
   fp = NULL;

/*
Get global parameter $MSGTIM.
------------------------------
*/
var = p_find(&block, "$MSGTIM"); 
*msgtim = IVAL(*var,0);
}
