#include "search.h"
#include "util.h"
#include "protos.h"

void divrg
(
    float *a, 		/* one of pair of classes for which divergence is to  */
    float *b,		/* the other class for which divergence is to be      */
    long bands,		/* number of input bands                              */
    float *divergence	/* computed output divergence                         */
)
	
/*
*****************************************************************************

NAME: DIVRG

PURPOSE:  Computes divergence between two classes.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Allocate the dymamic memory and initialize work buffers.
   Compute differences of means for all bands.
   Compute sums and differences using covar matrices and inverses.
   Compute divergence for all bands.
   Free the dynamic memory.

*****************************************************************************
*/

{

float	*delcovar,  		/* differences of covariance matrix elements  */
	*delinvrs = NULL, 		/* differences of  inverse covariance matrix  */
				/* elements                                   */
	delmean[MAXBND],	/* differences of means                       */
	*suminvrs = NULL,		/* sums of inverse covariance matrix elements */
	term;			/* intermediate result for divergence 	      */

long	band,			/* current band number                        */
	buf_size,		/* size of the work arrays                    */
	i, j, k,		/* loop counters and array indexes            */
	index,			/* index for inverses                         */
	total;			/* matrix size                                */

total = bands * (bands + 1) / 2;

/*
Allocate the dymamic memory and initialize work buffers.
---------------------------------------------------------
*/
buf_size = (long) ((ICOUNT + 1) * sizeof (float));
if ((delcovar = (float *) malloc (buf_size)) == NULL ||
    (delinvrs = (float *) malloc (buf_size)) == NULL ||
    (suminvrs = (float *) malloc (buf_size)) == NULL )
    c_abort (&block, "Error allocating dynamic memory", "search_bufalloc",
             "search", out_files, &num_outfiles);

/*
Compute differences of means for all bands.
--------------------------------------------
*/
for (band = 1; band <= bands; band++)
   delmean[band] = a[MEAN+band] - b[MEAN+band];

/*
Compute sums and differences using covar matrices and inverses.
----------------------------------------------------------------
*/
for (i = 1; i <= total; i++)
   {
   index = INVERSE + i;
   delcovar[i] = a[i] - b[i];
   delinvrs[i] = b[index] - a[index];
   suminvrs[i] = a[index] + b[index];
   }

/*
Compute divergence for all bands.
---------------------------------
*/
*divergence = 0.0;
for (band = 1; band <= bands; band++)
   {
   for (j = 1; j <= band; j++)
      {
      k = band * (band - 1) / 2 + j;
      term = delcovar[k] * delinvrs[k] + delmean[band] * delmean[j] 
             * suminvrs[k];
      if (band == j) 
         term = 0.5 * term;
      *divergence = *divergence + term;
      }
   }

/*
Free the dynamic memory.
------------------------
*/
free (delcovar);
free (delinvrs);
free (suminvrs);

}

