#include "search.h"
#include "util.h"
#include "protos.h"

void mergs
(
    float *a, 			/* destination class statistcs to be merged   */
    float *b,			/* class containing statistics to be merged,  */
    long nbands,			/* number of input bands                      */
    long *mer_stat		/* return status                              */
)

/*
*****************************************************************************

NAME: 	MERGS

PURPOSE:  Merges statistics for two classes. It is called only by DIMVC
          when the user has requested the divergence option.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Runs under TAE
   There is a potential loss of numerical precision in the merger of 
   covariance matrices for large values of pt1 and/or pt2.

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Alocate the dynamic memory.
   Compute merged means.
   Compute merged covariance matrix.
   Call SQUARE to store the matrix in square mode (instead symetric).
   Call MATINV to find the inverse of the matrix.
   If matrix inversion ok save matrix inverse.
   Free the dynamic memory.

*****************************************************************************
*/

{

float	pt,			/* sum of populations of class a and b        */
	pt1,			/* population of class a                      */
	pt2,			/* population of class b                      */
	**smt,			/* intermediate result matrix                 */
	temp1, temp2, temp3,	/* intermediate results                       */
	tm[MAXBND];		/* temporary buffer for merged means          */

long	band,			/* current band number                        */
	dyn_rows,		/* number of rows for dynamic allocation      */
	i1, j, j1,		/* loop counters and array indexes            */
	k1, l,
	ret_stat;		/* return status                              */

/*
Alocate the dynamic memory.
---------------------------
*/
dyn_rows = nbands + 1;
if ((smt = fmat2d (dyn_rows, dyn_rows)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "search-bufalloc",
            "search", out_files, &num_outfiles);

*mer_stat = ret_stat = 0;                              
pt1 = a[POP];
pt2 = b[POP];
pt = pt1 + pt2;
l = MEAN;

/*
Compute merged means.
---------------------
*/
for (band = 1; band <= nbands; band++)
   {
   l++;
   tm[band] = (pt1 * a[l] + pt2 * b[l]) / pt;
   }

/*
Compute merged covariance matrix.
---------------------------------
*/
k1 = 1;   
for (band = 1; band <= nbands; band++)
   {
   i1 = band + MEAN;
   for (j = 1; j <= band; j++)
      {
      j1 = j + MEAN;

      temp1 = (pt1 - 1) * a[k1] + pt1 * a[i1] * a[j1];
      temp2 = (pt2 - 1) * b[k1] + pt2 * b[i1] * b[j1];
      temp3 = pt * tm[band] * tm[j];
      a[k1] = (temp1 + temp2 - temp3) / (pt - 1);

      if ((band == j)  &&  (a[k1] < 0.0)) 
         a[k1]  =  0.0;
      k1++;
      }
   }
a[POP] = pt;
l = MEAN;
for (band = 1; band <= nbands; band++)
   {
   l++;
   a[l] = tm[band];
   }

/*
Call SQUARE to store the matrix in square mode (instead symetric).
Call MATINV to find the inverse of the matrix.
If matrix inversion ok save matrix inverse.
--------------------------------------------
*/
square (smt, nbands, a);                                              
matinv (smt, nbands, &ret_stat);                                           
*mer_stat = ret_stat;
if (ret_stat <= 0)
   {
   l = ICOUNT + 1;      
   for (band = 1; band <= nbands; band++)
      {
      for (j = 1; j <= band; j++)
         {
         a[l] = smt[j][band];
         l++;
         }
      }
   }

/*
Free the dynamic memory.
-------------------------
*/
freefmat2d(smt, dyn_rows);
}
