#include "search.h"
#include "util.h"
#include "protos.h"

void process
(
    char taein[][CMLEN],			/* input image tae name               */
    char taeout[],			/* output stat file tae name          */
    long data_type,			/* image data type                    */
    float **classtat,			/* class statistics                   */
    long num_inimg,			/* number of input images             */
    long nbands,				/* total number of input image bands  */
    long num_pixels,			/* number of pixels in the input image*/
    long num_lines, 			/* number of lines in the input image */
    float pts,				/* number of pixels in one field      */
    long bands_per_img[] 		/* number of bands in each image      */
)

/*
*****************************************************************************

NAME: PROCESS

PURPOSE:  Performs an unsupervised training site selection for use in a 
          subsequent classification.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe      
   2.0         25-Mar-85   Y. Sheu		added error msg if processing
						completes with no classes found.
   3.0         05-Jul-89   K. Kunska/STX	conversion to 5.0
   4.0	       09-Sep-89   K. Kunska/STX	fixed output file names in the
                                                header of stat summary
   4.1	       27-Nov-90   D. Akkerman		minor output message fix

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Allocate the dymamic memory for 6*6 window buffer and other work buffers.
   Initialize variables.
   Initialize and enable the processing message utility.
   While not end of image, 
      Read six lines from each band into a data buffer.
      For each of the bands, move a 6*6 block of pixels into zfl array.
      Compute the block mean for each band and decide if it is an eligible 
      training site.
      If the block is an eligible training site, compute the covariance 
      for each band.
      Store class statistics (mean, covariance, population) in classtat array.
      If the number of classes exceeds 50, call DIVMC to merge them using the 
      divergence or call SDMC to merge them using the scaled distance depending 
      on the size of the block and the interclass distance measure selected by 
      the user.
      Compute means for a 3*3 block and decide if it is an eligible training 
      site.
      If the block is an eligible training site, compute the covariance for 
      each band.
      Store class statistics (mean, covariance, population) in classtat array.
      If the number of classes exeeds 50, call SDMC to merge them using the 
      scaled distance.
      Reset the processing flags according to the number of the image 
      lines/strips left to be processed.
   End-while
   If processing completed with no classes found, then don't create output 
   statistics file or printed report. Just stop here and signal error.
   Otherwise, perform the final merging of classes using either the divergence
   or the scaled distance. 
   Write the statistical summary to the printer, file or terminal.
   Call HEADER to write the input parameter summary.
   Compute the number of fields used for the statistics and the total number of 
   fields processed by SEARCH.
   Call PAGEOUT to write the statistics for each class.
Free the dynamic memory.

*****************************************************************************
*/

{
char	outstr[81];			/* output class message string	      */

float	
	av1,				/* intermediate average               */
	avg,				/* average of pixels in a 6*6 bloc    */
	***data,			/* data buffer for input blocks       */
	dv[1226], 			/* pair-wise scaled interclass        */
					/* distances for all classes          */
	rfc,				/* scaling factor for separability    */
					/* one class                          */
	sd[MAXBND+1], 			/* standard deviations for 6*6 block  */
	sdc,				/* computed standard deviation        */
	smns[MAXBND+1],			/* means for one class                */
	**smt, 				/* matrix for intermediate statistics */
	zcrl[MAXBND+1], 		/* correlation matrix elements for    */
					/* one class                          */
	**zfl, 				/* buffer for 6*6 block of pixel data */
	*zlco, 				/* covariance elements for 6*6 block  */
	zmn[MAXBND+1], 			/* mean vectors for 6*6 block         */
	zmv[MAXBND+1];			/* coefficients of variation for      */
      					/* one class                          */
long	band,				/* current band                       */
	buf_size,			/* size of the buffers to be allocated*/
					/* the dynamic memory                 */
	covar_flg,			/* flag for controling the computation*/
					/* of covariance elements             */
	dyn_cols,			/* number of columns for dyn. alloc.  */
	dyn_rows,			/* number of rows for dyn. allocation */
	goto_flg,			/* control flag                       */
	i,				/* loop counter                       */
	idata,				/* loop counter/index into data array */
	ifin,				/* completion indicator               */
	ifl,				/* increment selector flag            */
	incr,				/* line increment factor              */
	init,				/* processing control parameter       */
	ist = 0, ist2,
	j, je, je2, jk,			/* loop counters and array indexes    */
	jrem,				/* number of pixels remainin from the */
					/* last strip                         */
	k, k1, 			/* loop counters and array indexes    */
	ktf,				/* total number of fields processed   */
					/* by search using divergence         */
	l,				/* loop counter                       */
	loop_flg,			/* control flag for looping           */
	lsamp,				/* total number of pixels for a strip */
	m, m1,				/* loop counter and array index       */
	main_flg,			/* main loop flag                     */
	maxp,				/* pointer to the last class accepted */
	msg_flag = 3,			/* flag for the processing message    */
					/* utility                            */
	n1,				/* array index                        */
	nflds,				/* number of fields which produced    */
					/* the set of summary statistics      */
	nl,				/* number of lines actually processed */
	nstrip,				/* total number of strips             */
	read_flg,			/* control flag for reading data array*/
	skip_flg,			/* control flag for skipping code     */
	sktf, 				/* total number of fields processed   */
					/* by search using scaled distance    */
	sl, 				/* strip line counter                 */
	status, 			/* return status                      */
	strip,				/* strip number                       */
	three_flg = 0,			/* flags when a 3x3 window should be  */
 					/* processed                          */
	total,				/* size of the triangular cov matrix  */
	xdim, ydim, zdim;		/* x, y, z dimension for a 3-d buffer */

long    fatal_err = -1;         /* fatal error return status          */
long    zero = 0;
long    one = 1;

/*
Allocate the dymamic memory for 6*6 window buffer and other work buffers.
--------------------------------------------------------------------------
*/
dyn_cols = nbands + 1;
dyn_rows = 37;
buf_size = (long) ((ICOUNT + 1) * sizeof(float));
if ((smt = fmat2d(dyn_rows, dyn_rows)) == NULL)
    c_abort (&block, "Error allocating dynamic memory", "search-bufalloc",
             "search", out_files, &num_outfiles);
if  ((zfl = fmat2d(dyn_rows, dyn_cols)) == NULL)
    c_abort (&block, "Error allocating dynamic memory", "search-bufalloc",
             "search", out_files, &num_outfiles);

if ((zlco = (float *) malloc (buf_size)) == NULL)
    c_abort (&block, "Error allocating dynamic memory", "search-bufalloc",
             "search", out_files, &num_outfiles);
if  ((scv = (float *) malloc (buf_size)) == NULL)
    c_abort (&block, "Error allocating dynamic memory", "search-bufalloc",
             "search", out_files, &num_outfiles);

xdim = nbands + 1;
ydim = 7;
zdim = 406;
if ((data = fmat3d (xdim, ydim, zdim)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "search-bufalloc",
            "search", out_files, &num_outfiles);

/*
Initialize variables.
---------------------
*/
init = -1;                                
ktf = 0;                                  
sktf = 0;                                 
rfc  =  1.0;                              
incr = 1;                               
sl = 1;                                 
nl = (num_lines / 6) * 6;                                           
nstrip = num_pixels / 400;
strip = num_pixels % 400;
if (strip  !=  0) 
   nstrip = nstrip + 1;
strip = 0;
ifin = 0;
jrem = 0;
maxp = 0;
total = nbands * (nbands + 1) / 2;

/* 
Initialize and enable the processing message utility.
------------------------------------------------------
*/
c_inittm ("SEARCH", "Processed", "line", " ", " ", &num_lines, &zero, &msgtim,
           &msg_flag);
curent = 1;
c_startm();

/*
While not end of image, read six lines from each band into a data buffer.
---------------------------------------------------------------------------
*/
main_flg = TRUE;
skip_flg = FALSE;

while (main_flg)
   {
   read_flg = TRUE;
   covar_flg = TRUE;
   if (strip  ==  0) 
      srchio (nbands, num_pixels);

   cscio (&sl, num_pixels, &lsamp, &strip, nstrip, &jrem, &ifin, data, nbands);
   if (ifin == 1) 
      break;

/*
For each of the bands, move a 6*6 block of pixels into zfl array.
------------------------------------------------------------------
*/
   je = 1;                                                       
   while (read_flg)
      {
      covar_flg = TRUE;
      ifl = 0;
      je2 = je + 5;          
      for (i = 1; i <= 6; i++)
         {
         for (band = 1; band <= nbands; band++)
            {
            k1  = (i - 1) * 6 + 1;
            for (l = je; l <= je2; l++)
               {
               zfl[k1][band] = data[band][i][l];   
               k1++;
               }
            }
         }

/*
Compute the block mean for each band and decide if it is an eligible 
training site.
----------------------------------------------------------------------
*/
      for (band = 1; band <= nbands; band++)
         {
         i = band;
         avg = 0.0;                      
         for (j = 1; j <= 36; j++)
            avg = zfl[j][i] + avg;
         avg = avg / 36.0;            
         zmn[i] = avg;    
         avg = 0.0;
         for (j = 1; j <= 36; j++)
            {
            av1 = zfl[j][i] - zmn[i];               
            avg = av1 * av1 + avg;
            }
         j = i * (i - 1) / 2 + i;                
         zlco[j] = avg / 35.0;           
         sdc = (float) sqrt ((double)zlco[j]);                
         if ((sdc - stdevlb) >= 0)
            {
            avg = coefvar * zmn[i];
            if ((avg - stdevub) < 0)
               avg = stdevub;
            if ((sdc-avg) <= 0)
               sd[i]  = sdc;
            else
               {
               covar_flg = FALSE;
               three_flg = TRUE;
               break;
               }
            }
         else
            {
            covar_flg = FALSE;
            three_flg = TRUE;
            break;
            }
         }

/* 
If the block is an eligible training site, compute the covariance for each 
band.
--------------------------------------------------------------------------
*/
      if (covar_flg)
         {
         goto_flg = TRUE;
         k1 = 2;                  
         for (band = 2; band <= nbands; band++)
            {
            for (k = 1; k <= band; k++)
               {
               if ((band - k) != 0)
                  {
                  avg = 0.0;
                  for (i = 1; i <= 36; i++)
                     avg = (zfl[i][k] - zmn[k]) * (zfl[i][band] - zmn[band]) 
                           + avg;
                  zlco[k1] = avg / 35.0;
                  }
               k1++;
               }
            }
/*
Store class statistics (mean, covariance, population) in classtat array.
------------------------------------------------------------------------
*/
         num_class = num_class + 1;             
         for (i = 1; i <= total; i++)
          classtat[num_class][i] = zlco[i];

         for (i= 1; i <= nbands; i++)
            classtat[num_class][i+MEAN] = zmn[i];

         classtat[num_class][POP] = 36.0;                

/* 
If the number of classes exceeds 50, call DIVMC to merge them using the 
divergence or call SDMC to merge them using the scaled distance depending 
on the size of the block and the interclass distance measure selected by 
the user.
---------------------------------------------------------------------------
*/
         if (block_dist == 0) 
            {
            square (smt, nbands, zlco);
            matinv (smt, nbands, &status);    
            if (status  == 1) 
               num_class = num_class - 1;             
            if (status > 0)
               /* goto 399 */
               {
               je = je + incr;
               goto_flg = FALSE;
               }
            else
               {
               ktf++; 
               n1 = ICOUNT + 1;            
               for (i = 1; i <= nbands; i++)
                  {
                  for (j = 1; j <= i; j++)
                     {
                     classtat[num_class][n1] = smt[j][i];
                     n1++;
                     }
                  }
               if ((num_class - 50) >= 0) 
                  {
                  divmc (classtat, &init, dv, pts, rfc, &maxp, nbands);
                  if (init  == 5) 
                     return;
                  else
                     ifl = 1;                       
                  }
               } /* else */
            }
         else
            {
            ktf++; 
            if ((num_class - 50) >= 0)
               sdmc (classtat, &init, dv, pts, rfc, &maxp, nbands);
            }
  
         three_flg = FALSE;
         if (goto_flg)
            je = je + 6;
         } /* if covar_flg */

/*
Compute means for a 3*3 block and decide if it is an eligible training site.
-----------------------------------------------------------------------------
*/
         if (three_flg)
            {
            if (block_dist <= 0)
               /* goto 399 */
               je = je + incr;
            else
               {
               for (m1 =  1; m1 <= 4; m1++)
                  {
                  loop_flg = FALSE;
                  if (m1  ==  1) 
                     ist = 1;           
                  if (m1  ==  2) 
                     ist = 4;     
                  if (m1  ==  3) 
                     ist = 19;       
                  if (m1  ==  4) 
                     ist = 22;       

                  for (band = 1; band <= nbands; band++)
                     {
                     i = band;
                     avg = 0.0;             
                     for (m  = 1; m <= 3; m++)
                        {
                        ist2 = ist + 2;            
                        for (j = ist; j <= ist2; j++)
                           avg = zfl[j][i] + avg;       
                        ist = ist + 6;     
                        }
                     ist = ist - 18;            
                     zmn[i] = avg / 9.0; 
                     avg = 0.0;
                     for (m = 1; m <= 3; m++)
                        {
                        ist2 = ist + 2;
                        for (j = ist; j <= ist2; j++)
                           {
                           av1 = zfl[j][i] - zmn[i]; 
                           avg =  av1 * av1 + avg; 
                           }
                        ist = ist + 6;              
                        }
                     ist = ist - 18;                   
                     k1 = i * (i - 1) / 2 + i;  
                     zlco[k1] = avg / 8.0;                        
                     sdc = (float) sqrt ((double)zlco[k1]);               
                     if ((sdc-stdevlb) >= 0) 
                        {
                        avg = coefvar * zmn[i];
                        if ((avg - stdevub) < 0) 
                           avg = stdevub;
                        if ((sdc - avg) > 0) 
                           {
                           loop_flg = TRUE;
                           break;
                           }
                        }
                     else
                        {
                        loop_flg = TRUE;
                        break;
                        }
                      sd[i] = sdc;
                      }  /* band loop */

/* 
If the block is an eligible training site, compute the covariance for each band.
--------------------------------------------------------------------------------
*/
                     if (!loop_flg)
                        {               
                        k1 = 2;                   
                        for (j = 2; j <= nbands; j++)
                           {
                           for (k = 1; k <= j; k++)
                              {
                              if ((j - k) != 0) 
                                 {
                                 avg = 0.0;
                                 for (m = 1; m <= 3; m++)
                                    {
                                    ist2 = ist + 2;          
                                    for (jk = ist; jk <= ist2; jk++)
                                       avg = (zfl[jk][j] - zmn[j]) * 
                                             (zfl[jk][j] - zmn[k]) + avg;
                                    ist = ist + 6;            
                                    }
                                 ist = ist - 18;           
                                 zlco[k1] = avg / 8.0;
                                 }
                              k1++;
                              }
                           }
         
/*
Store the class statistics (mean, covariance, population) in classtat array.
If the number of classes exeeds 50, call SDMC to merge them using the scaled 
distance.
----------------------------------------------------------------------------
*/
                        sktf++;   
                        num_class = num_class + 1; 
                        ifl = 1;                  
                        for (i = 1; i <= total; i++)
                           classtat[num_class][i] = zlco[i];
     
                        for (i = 1; i <= nbands; i++)
                           classtat[num_class][i+MEAN] = zmn[i];
      
                        classtat[num_class][POP] = 9.0;           

                        if ((num_class - 50) >= 0) 
                           sdmc (classtat, &init, dv, pts, rfc, &maxp, 
                                 nbands);  
                        } /* loop_flg */
                  } /* 391 loop */

                  if (ifl <= 0)
                     je = je + incr;
                  else
                     je = je + 6;
               } /* else block_dist */
            } /* three_flg */

/*
Reset the processing flags according to the number of the image lines/strips
left to be processed.
----------------------------------------------------------------------------
*/
      if (je < 401) 
         {
         skip_flg = FALSE;
         if ((5 - lsamp + je) > 0)
            {
            if (strip != 0)
               {
               jrem = lsamp - je + 1;     
               for (i = 1; i <= 6; i++)
                  {
                  for (band = 1; band <= nbands; band++)
                     {
                     for (idata = 1; idata <= jrem; idata++)
                        {
                        data[band][i][idata] = data[band][i][je];
                        je++;
                        }
                     je = je - jrem;      
                     }
                  }
               /* goto beginning of while main loop */
               skip_flg = TRUE;
               break;
               } /* if strip */
            jrem = 0;
            read_flg = FALSE;
            } /* else */
         } /* je */
      else
         {
         jrem = 0;                 
         read_flg = FALSE;
         skip_flg = FALSE;
         }
      } /* read_flg */

   if (( strip == 0) && (!skip_flg))
      {
      if ((sl-nl) >= 0)
         main_flg = FALSE;
      }
   } /* main_flg */

init = 1;           
c_estep (&gdescin);
curent = num_lines;
c_stoptm();

/*
If processing completed with no classes found, then don't create output 
statistics file or printed report. Just stop here and signal error.
Otherwise, perform the final merging of classes using either the divergence
or the scaled distance. 
---------------------------------------------------------------------------
*/
if (num_class == 0) 
   c_errmsg ("Processing completed with no classes found", "search-noclass",
              &fatal_err); 
if (block_dist == 0)
   {
   divmc (classtat, &init, dv, pts, rfc, &maxp, nbands);
   if (init == 5) 
      return;
   }
else
   sdmc (classtat, &init, dv, pts, rfc, &maxp, nbands);

sprintf(outstr,"SEARCH found %3d class(es)",num_class);
c_errmsg(outstr,"informational",&one);

/*
Write the statistical summary to the printer, file, or terminal.
----------------------------------------------------------------
*/
if (print[LP] || print[FIL] || print[TERM])
   {
   if (print[LP])
      c_errmsg("Statistical summary is being written to the printer",
		"informational",&one);
   else if (print[FIL])
      c_errmsg("Statistical summary is being written to file",
		"informational",&one);
   else 
      c_errmsg("Statistical summary is being written to terminal",
                "informational",&one);
/* 
Call HEADER to write the input parameter summary.
-------------------------------------------------
*/
   header (num_inimg, bands_per_img, taein, taeout);

/*
Compute the number of fields used for the statistics and the total number of 
fields processed by SEARCH.
----------------------------------------------------------------------------
*/
   if (block_dist == 1) 
      ktf = (4*ktf) + sktf; 
   
   for (i = 1; i <= num_class; i++)
      {
      for (j = 1; j <= total; j++)
         scv[j] = classtat[i][j];                

      for (j = 1; j <= nbands; j++)
         smns[j]  = classtat[i][j+MEAN]; 

      nflds =  (long)(classtat[i][POP] / pts);       
      k = 0;                 
      for (j = 1; j <= nbands; j++)
         {
         k = k + j;           
         sd[j] = (float) sqrt ((double) scv[k]);
         }
      k1 = 1;           
      for (j = 1; j <= nbands; j++)
         {
         for (k = 1; k <= j; k++)
            {
            if (sd[j] == 0.0  ||  sd[k] == 0.0) 
               zcrl[k1] = -999.0;
            else
               zcrl[k1] = scv[k1] / (sd[j] * sd[k]);    
            k1 = k1 + 1;
            }
         }

      for (j = 1; j <= nbands; j++)
         {
         if (smns[j] == 0.0) 
            zmv[j] = -999.0;
         else
            zmv[j] = 100.0 * sd[j] / smns[j];       
         }
     
/*
Call PAGEOUT to write the statistics for each class.
-----------------------------------------------------
*/
      pageout (ktf, nflds, nbands, i, smns, sd, zmv, zcrl, data_type);
      } /*num_class loop */
   }   /* if */

/*
Free the dynamic memory.
-------------------------
*/
freefmat3d (data, xdim, ydim);
freefmat2d (smt, dyn_rows);
freefmat2d (zfl, dyn_rows);
free(scv);
free(zlco);
}   
