#include "search.h"
#include "util.h"
#include "protos.h"

void srchstats
(
    char hostout[],		/* output statistics file host name           */
    long nbands,			/* number of input images                     */
    float **classtat		/* final statistics produced by search        */
)

/*
*****************************************************************************

NAME: 	SRCHSTATS

PURPOSE:  Creates a statistics file and saves the statistics of classes found
          by SEARCH. The mean class vectors and covariance matrices are saved
          at both the class and the site nodes in the statistics file.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Allocate the dynamic memory for the covariance buffer.
   Initialize the shape vector for integer output to statio.
   Initialize the shape vector for names output to statio.
   Initialize the shape vector for vector output to statio.
   Open the statistics file.
   For each class:
      Build class name from class index.
      Write the class name, the site name and the number of sites to the 
      stats file.
      Output class mean vectors.
      Output class covariance matrices.
   End-for.
   Close the statistics file and free the dynamic memory.

*****************************************************************************
*/

{

char	class_name[11], 	/* name of a class                            */
	classnm[11],		/* temporary class name                       */
	index[4],		/* class index                                */
	site_name[11];		/* name of a site                             */

double	means[MAXBND], 		/* array holding class and band means         */
	*covar;			/* array holding class covariance elements    */

long	access,			/* file access type                           */
	class,			/* current class                              */
	class_index,		/* class index in stats file                  */
	covarcnt,		/* number of elements in covariance matrix    */
	i, 			/* loop counter and array index               */
	nsites,			/* number of sites                            */
	shapei[9], 		/* statio shape vector for outputting scalars */
	shapen[9], 		/* statio shape vector for outputting class   */
				/* and site name                              */
	shapev[9],		/* statio shape vector for outputting vectors */
	status;			/* return status                              */

long    zero = 0;
long    one = 1;

/*
Allocate the dynamic memory for the covariance buffer.
------------------------------------------------------
*/
if ((covar = (double *) malloc (ICOUNT * sizeof(double))) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "search-bufalloc",
            "search", out_files, &num_outfiles);

strcpy (classnm, "SRCHCLS");
strcpy (site_name, "SRCHSITE1");
nsites = 1;

/*
Initialize the shape vector for integer output to statio.
---------------------------------------------------------
*/
shapei[0] = ELONG;
shapei[1] = 1;
shapei[2] = 1;
for (i = 3; i < 9; i++)
   shapei[i] = 0;

/*
Initialize the shape vector for names output to statio.
---------------------------------------------------------
*/
shapen[0] = EBYTE;
shapen[1] = 1;
shapen[2] = 11;
for (i = 3; i < 9; i++)
   shapen[i] = 0;

/*
Initialize the shape vector for vector output to statio.
----------------------------------------------------------
*/
shapev[0] = EDOUBLE;
shapev[1] = 1;
shapev[2] = 0;
for (i = 3; i < 9; i++)
   shapev[i] = 0;

covarcnt = nbands * (nbands + 1) / 2;
printf ("\nStatistics file is being created\n");

/*
Open the statistics file.
-------------------------
*/
access = IWRITE;
status = c_stopen (&sdesc, hostout, &access, &nclass);
if (status != E_OPNEW)
   c_abort (&block, "Error opening the statistics file", "search-stopen",
            "search", out_files, &num_outfiles);

status = c_stput (&sdesc, "NBANDS", &zero, &zero, shapei, &nbands); 
if (status != E_DTNEW)
   c_abort (&block, "No new data written to the statistics file", "search-stat",
            "search", out_files, &num_outfiles); 
/*
For each class:
Build class name from class index.
Write the class name, the site name and the number of sites to the stats file.
------------------------------------------------------------------------------
*/
for (class = 1; class <= num_class; class++)
   {
   class_index = class;
   sprintf (index, "%2d", class);
   strcpy (&classnm[7], index);
   strcpy (class_name, classnm);
   status = c_stput (&sdesc, "NAME", &class_index, &zero, shapen, class_name);
   if (status != E_DTNEW && status != E_NOCL)
      c_abort (&block, "No new data written to the statistics file", 
               "search-stat", "search", out_files, &num_outfiles); 
   status = c_stput (&sdesc, "NAME", &class_index, &one, shapen, site_name);
   if (status != E_DTNEW && status != E_NOSI)
      c_abort (&block, "No new data written to the statistics file", 
               "search-stat", "search", out_files, &num_outfiles); 
   status = c_stput (&sdesc, "NSITES", &class_index, &zero, shapei, &nsites); 
   if (status != E_DTNEW)
      c_abort (&block, "No new data written to the statistics file", 
               "search-stat", "search", out_files, &num_outfiles); 

   for (i = 1; i <= nbands; i++)
      means[i-1] = classtat[class][i+MEAN];
   for (i = 1; i <= covarcnt; i++)
      covar[i-1] = classtat[class][i];

/*
Output class mean vectors.
---------------------------
*/
   shapev[2] = nbands;
   status = c_stput (&sdesc, "MEAN_VECTOR", &class_index, &zero, shapev, means);
   if (status != E_DTNEW)
      c_abort (&block, "No new data written to the statistics file", 
               "search-stat", "search", out_files, &num_outfiles); 
   status = c_stput (&sdesc, "MEAN_VECTOR", &class_index, &one, shapev, means);
   if (status != E_DTNEW)
      c_abort (&block, "No new data written to the statistics file", 
              "search-stat", "search", out_files, &num_outfiles); 

/*
Output class covariance matrices.
---------------------------------
*/
   shapev[2] = covarcnt;
   status = c_stput (&sdesc, "COVARIANCE_MATRIX", &class_index, &zero, shapev, 
                     covar);
   if (status != E_DTNEW)
      c_abort (&block, "No new data written to the statistics file", 
               "search-stat", "search", out_files, &num_outfiles); 
   status = c_stput (&sdesc, "COVARIANCE_MATRIX", &class_index, &one, shapev, 
                     covar);
   if (status != E_DTNEW)
      c_abort (&block, "No new data written to the statistics file", 
               "search-stat", "search", out_files, &num_outfiles); 
   } 

/*
Close the statistics file and free the dynamic memory.
------------------------------------------------------
*/
c_stclse(&sdesc); 
free (covar);
}	   
