/******************************************************************************
NAME:   		      CONSTANT

PURPOSE    To fix bad rectangular areas within an image by replacing all of
           the pixels within the rectangle with a user supplied constant.

PROGRAM HISTORY:
VERSION  PROGRAMMER   DATE   CODE/CONT  REASON
-------  -----------  -----  ---------  ------
  1.0    E. Harris    01/82  SASC       Original development
  4.0    N. Deodhar   09/87  SAR        Fixed windowing
  5.0    R. McKinney  09/88  EDC        Add UPDDR routine.
                                        Take descriptors out the calling
					sequences.
					Replace newlas.h with las.h.
					Delete tae.h.
					Place 'c_' in front of las subroutines.
					Place subroutines in a separate file.
  5.1    J. Fenno     09/91  EDC        Fixed indentation.
					Added i*4 and real data types.
					Moved repair window error checking to
					function getpar.
					Completely rewrote this function to add
					the ability to update input images
					versus creating a new output image.
					Ran through Saber C.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   CONSTANT must be run under TAE.

PROJECT			LAS

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "imageio.h"
#include "segmrepr.h"
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
------------------------------------------------------------------------------*/
extern int  fdesc[];	        /* image file descriptors used for image I/O  */
extern long curent;		/* current processing line		      */
extern long curcat;		/* current processing band		      */

void constant
(
    struct PARBLK *vblock,	/* I: TAE parameter block                     */
    char hostname[][CMLEN],	/* I: host names of input images	      */
    char *minmax,		/* I: min/max flag                            */
    float pixval[],             /* I: pixel values to be used for const option*/
    long bands[][MAXBND+1],	/* I: bands to be processed for all images    */
    long inwind[][NUM_SPECS],   /* I: input window areas to fix               */
    long msgtim,		/* I: elapsed time between display messages   */
    long nbands[],		/* I: number of bands per input image         */
    long nbrwind,		/* I: number of repair windows                */
    long nmrimg,		/* I: number of input images                  */
    long odtype,                /* I: data type of images to be repaired      */
    long size,                  /* I: number of bytes required for this odtype*/
    long totbnd,		/* I: total number of bands to be processed   */
    long update,                /* I: flag to modify input images             */
    long window[][NUM_SPECS]	/* I: window specification of input images    */
)

{

static double  minval[MAXBND];/* min pixel value array                        */
static double  maxval[MAXBND];/* max pixel value array                        */

long    actual_band;          /* actual band # of current band within image   */
long    current_band;         /* current band of image that's being processed */
long    fatal = -1;           /* fatal error flag                             */
long    i;                    /* just a counter                               */
long	image;                /* loop index counting input images             */
long    line_offset;          /* line offset between repair window and image  */
long    ltside;               /* left side of rectangle                       */
long	msg_flag=ONECAT;      /* flag for processing message                  */
long 	nl;                   /* number of image lines                        */
long	nonfatal = 1;         /* value of one                                 */
long 	ns;                   /* number of image samples                      */
long    one = 1;              /* one                                          */
long    rtside;               /* right side of rectangle                      */
long    sample;               /* sample indicator                             */
long	status;               /* function return status code                  */

unsigned char *thebuf;        /* image input buffer                           */

/* Get the number of lines and samples in the images to be repaired.
   Because all of the input images must have the same size, you can pick
   them up from window[0].
------------------------------------------------------------------------*/
nl = window[0][NL];
ns = window[0][NS];

/* Allocate memory
------------------*/
if ((thebuf = (unsigned char *) malloc(ns * size)) == NULL)
   {
   closefile(nmrimg);
   c_errmsg("Error allocating dynamic memory","segmrepr-alloc",&nonfatal);
   if (update)
      c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   else
      c_clean(vblock,"segmrepr",hostname,&one);
   }

c_inittm("SEGMREPR","Processed","line","bands"," ", &nl, &totbnd, &msgtim,
	 &msg_flag);
curent = 1;
curcat = 1; /* so initial message appears correct */
c_startm();
curcat = 0; /* initialize to 0 so that curcat++ will start us at 1 */

/* For each of the images that needs to be updated
--------------------------------------------------*/
for (image = 0; image < nmrimg; image++)
   {

   /* Repair each of the input bands for the current image
   -------------------------------------------------------*/
   for (current_band = 0; current_band < nbands[image]; current_band++)
      {
      curcat++;
      actual_band = bands[image][current_band];

      /* For each repair window, repair the current band
      --------------------------------------------------*/
      for (i = 0; i < nbrwind; i++)
	 {
	 line_offset = inwind[i][SL];
	 ltside      = inwind[i][SS] - 1;
	 rtside      = ltside + inwind[i][NS];

	 /* For each line, repair all of the samples
	 -------------------------------------------*/
	 for (curent = line_offset; 
	      curent < (line_offset + inwind[i][NL]);
	      curent++)
	    {
	    status=c_eread((struct FDESC **)&fdesc[image],&actual_band,&curent,
                            thebuf,&one);
	    if (status == E_FAIL)
	       {
	       free(thebuf);
	       closefile(nmrimg);
	       if (update)
		  c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	       else
		  c_clean(vblock,"segmrepr",hostname,&one);
	       }
	    switch (odtype)
	       {
	       case EBYTE:
		  for (sample = ltside; sample < rtside; sample++)
		     *(thebuf + sample) = (unsigned char)pixval[i];
		  break;
	       case EWORD:
		  for (sample = ltside; sample < rtside; sample++)
		     *(short *)(thebuf + (sample * size)) = (short)pixval[i];
		  break;
	       case ELONG:
		  for (sample = ltside; sample < rtside; sample++)
		     *(long *)(thebuf + (sample * size)) = (long)pixval[i];
		  break;
	       case EREAL:
		  for (sample = ltside; sample < rtside; sample++)
		     *(float *)(thebuf + (sample * size)) = (float)pixval[i];
		  break;
	       } /* switch (odtype) */

	    /* Write out the lines with fixed windows
	    -----------------------------------------*/
	    status=c_ewrite((struct FDESC **)&fdesc[image],&actual_band,&curent,
                             thebuf,&one);
	    if (status == E_FAIL)
	       {
	       free(thebuf);
	       closefile(nmrimg);
	       if (update)
		  c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	       else
		  c_clean(vblock,"segmrepr",hostname,&one);
	       }
	    } /* for each line of the repair window */
	 } /* for each repair window */
      } /* for each band of the current image */
   } /* for each image */

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Free buffer
--------------*/
free(thebuf);

/* Call terminate to calculate the min/max
   values if necessary and close all files
------------------------------------------*/
terminate(vblock, hostname, minmax, maxval, minval,
	  bands,  nbands,   nmrimg, update, window);

return;
}
