/*******************************************************************************
NAME      		  GETPAR 


PURPOSE        To obtain the necessary parameters for the SEGMREPR function.
 
PROGRAM HISTORY:
PROGRAMMER  DATE       REASON
----------  ---------  ------
R.MCKINNEY  May  1988  converted to LAS 5.0
J.FENNO     Sep  1991  Fixed indentation.
                       Added code to allow i*4 and real data types.
		       Updated the repair window error checking done in this
		       function and moved the error checking that was done in 
		       the other functions here.
		       Updated the code that retrieves the repair window
		       dimensions from the vblock.
		       Ran through Saber-C.
J.FENNO     Mar  1992  Changed OUT so that it does not have a default value, the
		       user is required to enter null to enter update mode.
		       Modified so that combinations of images cannot be used in
		       updated mode (ie. "img1 + img2").
		       PRF Z0744.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	       LAS		                

*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "typlim.h"
#include "imageio.h"
#include "segmrepr.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,     /* I: pointer to TAE block                    */
    char hostname[][CMLEN],    /* O: host file names of the images           */
    char *minmax,              /* O: min/max flag buffer                     */
    char *option,              /* O: replacement option                      */
    char taename[][CMLEN],     /* O: TAE file names of the images            */
    float pixval[],            /* O: pixel values for insertion              */
    long bands[][MAXBND+1],    /* O: bands being repaired for each image     */
    long inwind[][NUM_SPECS],  /* O: repair window specifications            */
    long *msgtim,              /* O: elapsed time between display messages   */
    long *nbrimg,              /* O: number of input images                  */
    long *nbrwind,             /* O: number of windows specified             */
    long *nbands,              /* O: number of bands per image to be fixed   */
    long *odtype,              /* O: data type of input image                */
    long *size,                /* O: number of bytes for this data type      */
    long *totbnd,              /* O: total number of bands to be fixed       */
    long *update,              /* O: flag indicating update mode             */
    long window[][NUM_SPECS]   /* O: window specifications for images        */
)

{
struct VARIABLE  *val;          /* pointer to TAE variable within v-block     */

char   window_name[9];          /* TAE parameter names inwind1 thru inwind10  */

long   element = 0;             /* parameter element number                   */
long   fatal = -1;              /* fatal error flag                           */
long   i, j;                    /* loop control                               */
long   imglim;                  /* max number of images to be processed       */
long   ll_repair;               /* number of last line in the repair window   */
long   ls_repair;               /* number of last sample in the repair window */
long   nl_image;                /* number of lines in the input images        */
long   nl_repair;               /* number of lines in the repair window       */
long   ns_image;                /* number of samples in the input images      */
long   ns_repair;               /* number of samples in the repair window     */
long   one = 1;                 /* address containing the value 1             */
long   sl_repair;               /* starting line of the repair window         */
long   ss_repair;               /* starting sample of the repair window       */
long   nonfatal = 1;            /* nonfatal error flag                        */
long   qflag[MAXIMG][2];        /* flag indicating if subwindows specified    */
long   same;                    /* flag indicating if images have same window */
long   status;                  /* function return status code                */

/* Get the flag for update mode
-------------------------------*/
val = p_find(vblock,"UPDATE");
*update = IVAL(*val,0);

/* Get the image(s) from the tae parameter block
------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock, "IN",  &element, taename, hostname, nbrimg,
		 window, bands, nbands,   totbnd,  &imglim,  &same,  qflag);
if (status != E_SUCC)
   if (*update)
      c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   else
      c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
if (*update && (*nbrimg > 1))
   {
   c_errmsg("Combinations of images are not allowed when updating",
	    "segmrepr-comb", &nonfatal);
   c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   }
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","segmrepr-size",
            &nonfatal);
   if (*update)
      c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   else
      c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
   }

/* Get the data type of the input image and determine the size of its data type
------------------------------------------------------------------------------*/
status = c_gettyp((char (*)[256])hostname[0], nbrimg, odtype);
if (status != E_SUCC)
   if (*update)
      c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   else
      c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
switch(*odtype)
   {
   case EBYTE: *size = 1; break;
   case EWORD: *size = 2; break;
   case ELONG: *size = 4; break;
   case EREAL: *size = 4; break;
   }

/* Get the replacement option
-----------------------------*/
val = p_find(vblock,"REPLOPT");
strcpy(option, SVAL(*val,0));

/* If the replacement option is CONS, get the pixel values for insertion
   and check to make sure they are within the correct range.
-------------------------------------------------------------------------*/
if (strcmp(option,"CONS") == 0)
   {
   val = p_find(vblock,"PIXVAL");
   i = (*val).v_count;
   status = E_SUCC;
   for (j = 0; j < i; j++)
      {
      pixval[j] = RVAL(*val,j); 
      switch (*odtype)
	 {
	 case EBYTE:
	    if ((pixval[j] < MINUTINY) || (pixval[j] > MAXUTINY))
	       status = E_FAIL;
	    break;
	 case EWORD:
	    if ((pixval[j] < MINWORD) || (pixval[j] > MAXWORD))
	       status = E_FAIL;
	    break;
	 case ELONG:
	    if ((pixval[j] < MINLONG) || (pixval[j] > MAXLONG))
	       status = E_FAIL;
	    break;
	 case EREAL:
	    if ((pixval[j] < MINREAL) || (pixval[j] > MAXREAL))
	       status = E_FAIL;
	    break;
	 }
      if (status == E_FAIL)
	 {
	 c_errmsg("PIXVAL is out of range for the input image data type",
		  "segmrepr-range",&nonfatal);
	 if (*update)
	    c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	 else
	    c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
	 }
      }
   }

/* Retrieve the specifications for each of the repair windows.  Once you
   find a window with fewer than 4 values, ignore all of the remaining windows
------------------------------------------------------------------------------*/
*nbrwind = 0;
for (i = 0; i < MAX_INWINDS; i++)
   {
   sprintf(window_name, "inwind%1d", (i + 1));
   val = p_find(vblock, window_name);
   if (val->v_count == NUM_SPECS)
      {
      for (j = 0; j < NUM_SPECS; j++) 
         {
	 inwind[i][j] = IVAL(*val,j);
         if ((inwind[i][j] < 1) && (val->v_count != 0))
             c_lasmsg(fatal, "segmrepr-invalid","Values of %s must be positive",
			window_name);
         }
      (*nbrwind)++;
      }
   else
      {
      for (j = i; j < MAX_INWINDS; j++)
	 inwind[j][SL] = inwind[j][SS] = inwind[j][NL] = inwind[j][NS] = 0;
      break;
      }
   }

/* Check the repair window specifications.  They must be relative to the
   image windows or the images if no windows were given.  (NOTE: the pdf
   insures that the starting line and sample of the repair windows are
   in the range 1:34000)
------------------------------------------------------------------------*/
nl_image = window[0][NL];        /* all images will have same NL and NS */
ns_image = window[0][NS];
for (j = 0; j < *nbrwind; j++)
   {
   sl_repair = inwind[j][SL];
   ss_repair = inwind[j][SS];
   nl_repair = inwind[j][NL];
   ns_repair = inwind[j][NS];
   ll_repair = sl_repair + nl_repair - 1;
   ls_repair = ss_repair + ns_repair - 1;

   if ((sl_repair > nl_image) || (ss_repair > ns_image) ||
       (ll_repair > nl_image) || (ls_repair > ns_image))
      {
      c_errmsg("Repair window extends outside input image","segmrepr-relative",
	       &nonfatal);
      if (*update)
	 c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
      else
	 c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
      }

   /* Check the repair windows to make sure that
      they do not span the entire input image.
   ---------------------------------------------*/
   if ((nl_repair == nl_image) && (strcmp(option, "CONS") != 0))
      {
      if (ns_repair == ns_image)
	 {
	 c_errmsg("Repair window equals entire image","segmrepr-entire",
		  &nonfatal);
	 if (*update)
	    c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	 else
	    c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
	 }
      else if (strcmp(option, "HORI") != 0)
	 {
	 c_errmsg("Repair window spans entire image vertically",
		  "segmrepr-vert", &nonfatal);
	 if (*update)
	    c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	 else
	    c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
	 }
      }
   if ((ns_repair == ns_image)       && 
       (strcmp(option, "CONS") != 0) && (strcmp(option, "VERT") != 0))
      {
      c_errmsg("Repair window spans entire image horizontally",
	       "segmrepr-hori", &nonfatal);
      if (*update)
	 c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
      else
	 c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
      }
   }

/* Retrieve the TAE global variables from the parameter block
   and store them in their respective program variables. 
-------------------------------------------------------------*/
val = p_find(vblock, "$msgtim");
*msgtim = IVAL(*val, 0);

val = p_find(vblock, "$minmax");
strcpy(minmax, SVAL(*val,0));
return;
}
