/*****************************************************************************
NAME:                            TERMINATE

PURPOSE   To update the ddr, close the image files and free the buffer 

PROGRAM HISTORY:
VERSION  PROGRAMMER  DATE   CODE/CONT  REASON
=======  ==========  =====  =========  =========================================
 5.1     J Fenno     08/91  EDC        Creation.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TERMINATE must be run as part of SEGMREPR.

PROJECT:              LAS

ALGORITHM:

   NOTE:
   =====
      Due to the capability of updating input images, this code must ensure
      that min/max values are calculated correctly and that they are valid.

      When NOT using the update mode a new image is created and min/max values
      are not set.  Therefore, if the user requests that they be calculated you
      simply call c_minmax to do the calculations.

      When using update mode, one or more of the images may already have the
      min/max values calculated.  If the user does not request that the values
      be calculated we must insure that none of the images keep their old 
      values.  To do this simply call SVMNMX with the m_flag set to "NO" for
      each image.
      
      When the user requests that the min/max values be calculated in update
      mode we cannot just call c_minmax to calculate the new values.  In this
      case c_minmax simply retrieves the values that exist and these may be
      incorrect.  So in this case we also need to call SVMNMX with the m_flag
      set to "NO" for each input image to "erase" the current values and then
      call c_minmax to recalculate them.  The call to c_minmax will
      automatically update the ddr.


ALGORITHM REFERENCES:
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "imageio.h"
#include "segmrepr.h"
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image i/o routines
------------------------------------------------------------------------------*/
extern int  fdesc[];          /* Image file descriptors used for image I/O    */
extern long curcat;           /* Current processing band                      */
extern long curent;           /* Current processing line                      */

void terminate
(
    struct PARBLK *vblock,       /* I: TAE parameter block                    */
    char hostname[][CMLEN],      /* I: Host names of images                   */
    char *minmax,                /* I: Min max flag                           */
    double maxval[],             /* O: Max pixel value array                  */
    double minval[],             /* O: Min pixel value array                  */
    long bands[MAXIMG][MAXBND+1],/* I: Bands to be processed for all images   */
    long nbands[MAXIMG],         /* I: # of bands per input image             */
    long nmrimg,                 /* I: # of input images                      */
    long update,                 /* I: flag to modify the input images        */
    long window[][NUM_SPECS]     /* I: Window specification of input images   */
)

{
long    image;                   /* loop control                              */
long    nonfatal = 1;            /* Nonfatal error flag                       */
long    status;                  /* Function return status code               */
long    zero = 0;                /* address containing the value of zero      */

if (update)
   for (image = 0; image < nmrimg; image++)
      c_svmnmx(hostname[image], &nbands[image], minval, maxval, "NO");

/* Calculate the new min/max values for each of the images
----------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   status = c_minmax(hostname, &nmrimg, window, bands, nbands, minval, maxval);
   if (status != E_SUCC)
      c_errmsg("Nonfatal error encountered","segmrepr-warn",&nonfatal);
   }

/* Update associated history files
----------------------------------*/
for (image = 0; image < nmrimg; image++)
   if (c_hstout(vblock, hostname[image],(char (*)[256])"", &zero, "") != E_SUCC)
      c_errmsg("Nonfatal error encountered", "segmrepr-warn",&nonfatal);

/* Close all of the files
-------------------------*/
closefile(nmrimg);

return;
}
