/******************************************************************************
NAME:   		      VERTICAL

PURPOSE    To fix bad rectangular areas within an image by averaging the
	   pixels above and below the repair window.

PROGRAM HISTORY:
VERSION  PROGRAMMER   DATE   CODE/CONT  REASON
-------  -----------  -----  ---------  ------
  1.0    E. Harris    01/82  SASC       Original development
  4.0    N. Deodhar   09/87  SAR        Fixed windowing
  5.0    R. McKinney  09/88  EDC        Add UPDDR routine.
                                        Take descriptors out the calling
					sequences.
					Replace newlas.h with las.h.
					Delete tae.h.
					Place 'c_' in front of las subroutines.
					Place subroutines in a separate file.
  5.0    T. Baltzer   11/89  EDC        Add check for case in which the repair
					window spans the entire image in the
					vertical direction.
  5.1    J. Fenno     09/91  EDC        Fixed indentation.
					Added i*4 and real data types.
					Added ability to update input images
					versus creating a new output image.
					Moved repair window error checking to
					function getpar.
					Removed as much duplicate code as 
					possible.
					Ran through Saber C.
  5.2    J. Fenno     11/91  EDC        Fixed index problem with current_band

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   VERTICAL must be run under TAE.

PROJECT			LAS

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "imageio.h"            /* to retrieve constants SL SS NL NS          */
#include "segmrepr.h"           /* to retrieve segmrepr constants             */
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
------------------------------------------------------------------------------*/
extern int  fdesc[];	        /* image file descriptors used for image I/O  */
extern long curent;		/* current processing line		      */
extern long curcat;		/* current processing band		      */

void vertical
(
    struct PARBLK *vblock,	/* I: TAE parameter block		      */
    char hostname[][CMLEN],	/* I: host names of images to be repaired     */
    char *minmax,		/* I: min/max flag                            */
    long bands[][MAXBND+1],	/* I: bands to be processed for all images    */
    long inwind[][NUM_SPECS],   /* I: input window areas to fix               */
    long msgtim,		/* I: elapsed time between display messages   */
    long nbands[],		/* I: number of bands per input image         */
    long nbrwind,		/* I: number of repair windows                */
    long nmrimg,		/* I: number of input images                  */
    long odtype,                /* I: data type of images to be repaired      */
    long size,                  /* I: number of bytes required for this odtype*/
    long totbnd,		/* I: total number of bands to be processed   */
    long update,                /* I: flag to modify input images             */
    long window[][NUM_SPECS]	/* I: window specification of input images    */
)

{

static double  minval[MAXBND];/* min pixel value array                      */
static double  maxval[MAXBND];/* max pixel value array                      */

long    actual_band;          /* actual band # of current band within image   */
long    current_band;         /* current band of image that's being processed */
long    fatal = -1;           /* fatal error flag                             */
long    i;                    /* loop control                                 */
long	image;                /* loop control                                 */
long    ltside;               /* left side of rectangle                       */
long    line;                 /* line to be read                              */
long	msg_flag=ONECAT;      /* flag for processing message                  */
long 	nl;                   /* number of image lines                        */
long	nonfatal = 1;         /* value of one                                 */
long 	ns;                   /* number of image samples                      */
long    one = 1;              /* one                                          */
long    rtside;               /* right side of rectangle                      */
long    sample;               /* sample indicator                             */
long    line_offset;          /* line offset between repair window and image  */
long	status;               /* function return status code                  */

float   top;                  /* top sample indicator                         */
float   bottom;               /* bottom sample indicator                      */
float   lineinc;              /* line increment indicator                     */

unsigned char *thebuf;        /* image input buffer                           */
unsigned char *temp1;         /* image input buffer                           */
unsigned char *temp2;         /* image input buffer                           */

/* Get the number of lines and samples in the images to be repaired.
   Because all of the input images must have the same size, you can pick
   them up from window[0].
------------------------------------------------------------------------*/
nl = window[0][NL];
ns = window[0][NS];

/* Allocate memory
------------------*/
if ((thebuf = (unsigned char *) malloc(ns*size)) == NULL)
   {
   closefile(nmrimg);
   c_errmsg("Error allocating dynamic memory","segmrepr-alloc",&nonfatal);
   if (update)
      c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   else
      c_clean(vblock,"segmrepr",hostname,&one);
   }
if ((temp1 = (unsigned char *) malloc(ns*size)) == NULL)
   {
   free(thebuf);
   closefile(nmrimg);
   c_errmsg("Error allocating dynamic memory","segmrepr-alloc",&nonfatal);
   if (update)
      c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   else
      c_clean(vblock,"segmrepr",hostname,&one);
   }
if ((temp2 = (unsigned char *) malloc(ns*size)) == NULL)
   {
   free(thebuf);
   free(temp1);
   closefile(nmrimg);
   c_errmsg("Error allocating dynamic memory","segmrepr-alloc",&nonfatal);
   if (update)
      c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
   else
      c_clean(vblock,"segmrepr",hostname,&one);
   }

c_inittm("SEGMREPR","Processed","line","bands"," ", &nl, &totbnd, &msgtim,
	 &msg_flag);
curent = 1;
curcat = 1; /* so initial message appears correct */
c_startm();
curcat = 0; /* initialize to 0 so that curcat++ will start us at 1 */

/* For each of the images that needs to be updated
--------------------------------------------------*/
for (image = 0; image < nmrimg; image++)
   {

   /* Repair each of the input bands for the current image
   -------------------------------------------------------*/
   for (current_band = 0; current_band < nbands[image]; current_band++)
      {
      curcat++;
      actual_band = bands[image][current_band];

      /* For each repair window, repair the current band
      --------------------------------------------------*/
      for (i = 0; i < nbrwind; i++)
	 {
	 line_offset = inwind[i][SL];

	 /* Read the line prior to the window
	 ------------------------------------*/
	 if (line_offset > 1)
	    line = line_offset - 1;
	 else
	    line = line_offset + inwind[i][NL];
	 status = c_eread((struct FDESC **)&fdesc[image], &actual_band, &line, 
			  temp1, &one);
	 if (status == E_FAIL)
	    {
	    free(thebuf);
	    free(temp1);
	    free(temp2);
	    closefile(nmrimg);
	    if (update)
	       c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	    else
	       c_clean(vblock,"segmrepr",hostname,&one);
	    }

	 /* Read the line following the window
	 -------------------------------------*/
	 if ((line_offset + inwind[i][NL]) != (nl+1))
	    line = line_offset + inwind[i][NL];
	 else
	    line = line_offset - 1;
	 status = c_eread((struct FDESC **)&fdesc[image], &actual_band, &line,
                           temp2, &one);
	 if (status == E_FAIL)
	    {
	    free(thebuf);
	    free(temp1);
	    free(temp2);
	    closefile(nmrimg);
	    if (update)
	       c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	    else
	       c_clean(vblock,"segmrepr",hostname,&one);
	    }

	 line = 0;
	 for (curent = line_offset; 
	      curent < line_offset + inwind[i][NL];
	      curent++)
	    {
	    status=c_eread((struct FDESC **)&fdesc[image],&actual_band,&curent,
                            thebuf,&one);
	    if (status == E_FAIL)
	       {
	       free(thebuf);
	       free(temp1);
	       free(temp2);
	       closefile(nmrimg);
	       if (update)
		  c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	       else
		  c_clean(vblock,"segmrepr",hostname,&one);
	       }
	    line++; 
	    ltside = inwind[i][SS] - 1;
	    rtside = ltside + inwind[i][NS];
	    switch (odtype)
	       {
	       case EBYTE:
		  for (sample = ltside; sample < rtside; sample++)
		     {
		     top     = *(temp1 + sample);
		     bottom  = *(temp2 + sample);
		     lineinc = (top - bottom) / ((float)inwind[i][NL] + 1); 
		     *(thebuf + sample) = (top - (lineinc * (float)line));
		     }
		  break;
	       case EWORD:
		  for (sample = ltside; sample < rtside; sample++)
		     {
		     top     = *(short *)(temp1 + (sample * size));
		     bottom  = *(short *)(temp2 + (sample * size));
		     lineinc = (top - bottom) / ((float)inwind[i][NL] + 1); 
		     *(short *)(thebuf + (sample * size)) =
			(top - (lineinc * (float)line));
		     }
		  break;
	       case ELONG:
		  for (sample = ltside; sample < rtside; sample++)
		     {
		     top     = *(long *)(temp1 + (sample * size));
		     bottom  = *(long *)(temp2 + (sample * size));
		     lineinc = (top - bottom) / ((float)inwind[i][NL] + 1); 
		     *(long *)(thebuf + (sample * size)) =
			(long)((top - (lineinc * (float)line)));
		     }
		  break;
	       case EREAL:
		  for (sample = ltside; sample < rtside; sample++)
		     {
		     top     = *(float *)(temp1 + (sample * size));
		     bottom  = *(float *)(temp2 + (sample * size));
		     lineinc = (top - bottom) / ((float)inwind[i][NL] + 1); 
		     *(float *)(thebuf + (sample * size)) =
			(top - (lineinc * (float)line));
		     }
		  break;
	       } /* switch (odtype) */

	    /* Write out the lines with fixed windows
	    -----------------------------------------*/
	    status=c_ewrite((struct FDESC **)&fdesc[image],&actual_band,&curent,
                             thebuf,&one);
	    if (status == E_FAIL)
	       {
	       free(thebuf);
	       free(temp1);
	       free(temp2);
	       closefile(nmrimg);
	       if (update)
		  c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
	       else
		  c_clean(vblock,"segmrepr",hostname,&one);
	       }
	    } /* for each line of the repair window */
	 } /* for each repair window */
      } /* for each band of the current image */
   } /* for each image */

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Free buffers
---------------*/
free(thebuf);
free(temp1);
free(temp2);

/* Call terminate to calculate the min/max
   values if necessary and close all files
------------------------------------------*/
terminate(vblock, hostname, minmax, maxval, minval,
	  bands,  nbands,   nmrimg, update, window);

return;
}
