/*******************************************************************************
NAME:		SMOOTHIT	

PURPOSE:    	SMOOTHIT make one pass through image replacing flagged
			 pixel values with neighboring pixel values
		  	 based on weights in weight table. 
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	12/90	New development 
5.1	S.Nelson	 5/92	Illiminated warnings upon compilation.
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/

#include <math.h>
#include "las.h"
#include "protos.h"

/* The following variables have been declared global 
----------------------------------------------------*/
extern long curent;		/* current processing line */
extern long nline;			/* number of lines to process */
extern long ns;			/* number of samples to process */
extern long *curlin;		/* pointer to current line */
extern long *outlin;		/* pointer to output line */

/* Function to make one smoothing pass through the image 
--------------------------------------------------------*/
int smoothit
(
    long fourway,	/* flag indicating whether to enforce strict four-way
			   connectedness criteria for reclamation */
    long *changed,	/* flag indicating when pixel values are changed */
    long *neg_pres,	/* flag indicating presence of negative pixels */
    long *minv,		/* min valued pixel */
    long *maxv		/* max valued pixel */
)

{
long	i;		/* loop counter */
long	j;		/* loop counter */
long	k;		/* loop counter */
long	ln;		/* line being processed */
long 	smp;		/* sample being processed */
long	kern[9];	/* kernel for current pixel */
long	ncl;		/* number of candidate classes for reclamation */ 
long	hicls[9];	/* classes with highest total weight values */
long	cls[9];		/* candidate class values */
long	fourwy[9];	/* flag indicating when candidate class is 
			   four-way connected to the flagged class */ 
double	clswgt[9];	/* cumulative weight values for candidate classes */ 
double sqrt_2 = sqrt(2.0); /* distance factor used in weight calculations */
double	hi = 0.0;		/* hi weight value */
double  eps = .000000000001;   /* for checking equality using doubles */


struct pairweight key; 		/* pairwise weight table structure */


/* Initialize flags for this pass through the image
---------------------------------------------------*/
*neg_pres = FALSE;
*changed = FALSE;

/* Reset current line counter for processing messages 
-----------------------------------------------------*/
curent = 0;

/* Process all lines in image (or window of image)
--------------------------------------------------*/
for (ln = 1; ln <= nline; ln++)
   {
   curent++;
   /* Step to next line in image  
   -----------------------------*/ 
   if (stepline(ln) == E_FAIL)
      return (E_FAIL);

   /* Process all samples in current line 
   --------------------------------------*/
   for (smp = 0; smp < ns; smp++)
      {
      if (curlin[smp] >= 0)
         /* Sample not flagged so transfer directly to output line 
         ----------------------------------------------------------*/
         {
	 outlin[smp] = curlin[smp];
	 if (outlin[smp] > *maxv) *maxv = outlin[smp];
	 if (outlin[smp] < *minv) *minv = outlin[smp];
	 }
      else
  	 /* Sample is flagged with negative value
         ----------------------------------------*/
	 {
   	 /* Retrieve 3x3 kernel for the current flagged pixel
         ----------------------------------------------------*/
	 getkern(ln,smp,kern);
	 
  	 /* Set key flagged class for searching pair-wise weight table
  	 -------------------------------------------------------------*/ 
	 key.fclass = kern[4] * -1;

	 ncl = 0;

	 /* Search through kernel looking for neighboring pixels' 
	    classes that are candidates for replacing flagged pixel
     	 ----------------------------------------------------------*/
	 for (i = 0; i < 9; i++)
	    {
	    if (kern[i] > 0)
	       /* Found a candidate class
	       --------------------------*/
	       {
	       /* Set key neighbor class
               -------------------------*/ 
	       key.nclass = kern[i];

 	       /* Check to see if this class has already occurred in kernel 
               ------------------------------------------------------------*/ 
	       for (j = 0; j < ncl; j++)
		  {
		  if (cls[j] == kern[i])
 		     /* Candidate class duplicated */ 
		     {
		     if ((i % 2) == 0)
			/* Pixel in corner of kernel */
			/* Adjust weight for distance from center of kernel */
			clswgt[j] += getwgt(&key) / sqrt_2;
		     else
			/* Candidate four-way connected */
			{
			/* Accumulate weights for candidate class */
			clswgt[j] += getwgt(&key);
			/* Candidate class is four-way connected */
			fourwy[j] = TRUE;
			}
 		     break;
		     }
	          }
	       if (j == ncl)
		  /* Class has not occurred before in kernel 
		  ------------------------------------------*/
		  {
	          /* Save class value */
		  cls[ncl] = kern[i];
		  /* Set count for class to 1 */
		  if ((i % 2) == 0)
		     /* Pixel in corner of kernel */
		     {
		     /* Adjust weight for distance from center of kernel */
		     clswgt[ncl] = getwgt(&key) / sqrt_2;
		     fourwy[ncl] = FALSE;
		     }
		  else
		     /* Candidate four-way connected */
		     {
		     /* Store weight for candidate class */
		     clswgt[ncl] = getwgt(&key);
		     fourwy[ncl] = TRUE;
		     }

		  /* Increment number of unique candidate classes found */
                  ncl++;
		  }
   	       }  
            }

         if (ncl > 0)
	    /* Candidate class(s) found in kernel 
	    -------------------------------------*/ 
	    {
	    /* Initialize the hi weight value */
            hi = 0.;
	    /* Look for candidate class with the highest weight value 
	    ---------------------------------------------------------*/
	    for (i = 0; i < ncl; i++)
	       {
	       if (clswgt[i] > hi)
		  {
  	          if (fourway) 
		     /* Strict four-way connectedness is to be enforced */
	             {		 
		     if (fourwy[i])
			/* Candidate is four-way connected */
	                hi = clswgt[i];
		     }
	          else 
		     /* Four-way connectedness is not enforced */
	             hi = clswgt[i];
		  }
               }

            if (hi != 0.)
	       /* At least one valid candidate class was found
               -----------------------------------------------*/
	       {
   	       /* See if there is more than one candidate with same hi weight
	       --------------------------------------------------------------*/
               k = 0;
	       for (i = 0; i < ncl; i++)
	          {
		  /* Keep track of classes that have highest weight values */
                  if ((clswgt[i] - hi) > -eps && (clswgt[i] - hi) < eps)
                     {
		     if (fourway)
		        /* Strict four-way connectedness is to be enforced */
			{
		        if (fourwy[i])
			   /* Candidate is four-way connected */
		           hicls[k++] = cls[i];
			}
		     else 
		        /* Four-way connectedness is not enforced */
		        hicls[k++] = cls[i];
   		     }
   	          }

	       /* If multiple classes tied for highest weight, pick 1 randomly*/
  	       if (k > 1) randpick(&k);

	       /* Set ouput pixel to the replacement class value */
  	       outlin[smp] = hicls[k-1];
	       if (outlin[smp] > *maxv) *maxv = outlin[smp];
	       if (outlin[smp] < *minv) *minv = outlin[smp];
	       /* Set changed flag to true */
	       *changed = TRUE;
	       }
            }

         if ((hi == 0.) || (ncl == 0))
	    {
	    /* No valid candidates for replacement were found in kernel 
	    -----------------------------------------------------------*/
	    outlin[smp] = curlin[smp];
	    if (outlin[smp] > *maxv) *maxv = outlin[smp];
	    if (outlin[smp] < *minv) *minv = outlin[smp];
	    *neg_pres = TRUE;
	    }
         }
      }
   }

/* Step one more line to write out las output line 
--------------------------------------------------*/
if (stepline(ln) == E_FAIL)
   return (E_FAIL);
return (E_SUCC);
}
