
/* The following have been declared globally for use in this file
-----------------------------------------------------------------*/

static struct pairweight *wgttbl; 	/* pairwise weight table array */ 
static long	nument;			/* number of entries in wgttbl */
static long	dflt_wgt;		/* default weight value */

/*******************************************************************************
NAME:		READWGTS	

PURPOSE:    	READWGTS reads weights from a pairwise weights table 
			
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	12/90   New development	
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/

#include <stdlib.h>
#include "las.h"
#include "ltable.h"
#include "protos.h"

void readwgts
(
    char *pairwgtlt,		/* pairwise weight values labeled table */
    long dflt			/* user specified default weight */
)

{
int 	s_compar();		/* compare routine for qsort */
int 	compar();		/* compare routine for bsearch */

long	i;			/* loop counter */
long	k;			/* loop counter */
long	access = IREAD;		/* read access */
long	status;			/* status flag value */
long	fatal = -1;		/* fatal error code */
long	nonfatal = 1;		/* nonfatal error code */
long	mxnum;			/* maximum number of entries in arrays */

struct 	TAB_DEF	pairlt;		/* region labeled table */	
struct	VECTOR	*v_fclass;	/* region number vector */		
struct	VECTOR	*v_nclass = NULL;	/* region class vector */		
struct	VECTOR	*v_weight = NULL;	/* region count vector */		

char	msg[ERRLEN+1];		/* error message buffer */
/*****************************************************************************/

/* save default weight for subsequent processing
------------------------------------------------*/
dflt_wgt = dflt;

/* Check whether pairwise weight table was entered by user
----------------------------------------------------------*/
if (*pairwgtlt == NULL)  
   {
   /* No table entered
   -------------------*/
   nument = 0;
   wgttbl = NULL;
   }
else
   {
   /* Read in class pairs and weights from labeled table
   -----------------------------------------------------*/
   open_tab(&pairlt,pairwgtlt,access);
   get_vector(&pairlt);
   if (((v_fclass = get_field_ptr(&pairlt,"FCLASS")) == NULL) ||
      ((v_nclass = get_field_ptr(&pairlt,"CCLASS")) == NULL) ||
      ((v_weight = get_field_ptr(&pairlt,"WEIGHT")) == NULL))
      c_errmsg("Error getting field pointer from pairwise weight labeled table",
		"smooth-getfield",&fatal); 

   /* Allocate initial arrays for reading data from lt
   ---------------------------------------------------*/
   mxnum = 1000;
   if ((wgttbl = (struct pairweight *) malloc((mxnum)*12)) == NULL)  
      c_errmsg("Error allocating dynamic memory","smooth-alloc",&fatal);
   for (i = 0; i < mxnum; i++) 
      wgttbl[i].fclass = wgttbl[i].nclass = wgttbl[i].weight = 0;
   
   /* Read all records in labeled table 
   ------------------------------------*/
   for (i = 0;;)
      {
      status = get_record(&pairlt);
      if (status == E_FAIL)
         c_errmsg("Fatal error encountered","smooth-fatal",&fatal);
      else if (status == E_EOF)
         break;
      else
         {
         i++;
         if (i >= mxnum)
            {
            /* Allocate more memory when needed
            -----------------------------------*/
            mxnum += 1000;
            if ((wgttbl = 
		(struct pairweight *) realloc(wgttbl,(mxnum)*12)) == NULL) 
               c_errmsg("Error allocating dynamic memory",
	      			"smooth-alloc",&fatal);
            for (k = i; k < mxnum; k++) 
               wgttbl[k].fclass = wgttbl[k].nclass = wgttbl[k].weight = 0;
            }
         /* Extract field values for classes and weight
         -----------------------------------------------*/
         get_matrix(v_fclass,I4,(unsigned char *)&(wgttbl[i-1].fclass));
         get_matrix(v_nclass,I4,(unsigned char *)&(wgttbl[i-1].nclass));
         get_matrix(v_weight,I4,(unsigned char *)&(wgttbl[i-1].weight));
         }
      }
 
   /* Close labled table 
   ---------------------*/
   close_tab(&pairlt);

   nument = i;
   sprintf(msg,"Number of classes & weights read from labeled table: %d",i);
   c_errmsg(msg," ",&nonfatal);
   
   /* Sort the class numbers and thresholds
   ----------------------------------------*/
   qsort ((char *)wgttbl, nument, 12, s_compar);
   }

}


/*****************************************************************************/
long getwgt
(	/* Returns weight for pair of classes in key */
    struct pairweight *key		/* Key for searching table */
)
{
struct pairweight *pw_ptr;	/* pointer to entry in table if key found */
int 	compar();		/* compare routine for bsearch */

/* Search table for pair of classes in key
------------------------------------------*/
if (wgttbl == NULL)
   /* No values in table--return default 
   -------------------------------------*/
   return (dflt_wgt);
else 
   pw_ptr = (struct pairweight *) bsearch((char *)key,(char *)wgttbl,
							nument,12,compar);

if (pw_ptr == NULL)
      /* Return user specified default if not found 
      ---------------------------------------------*/
      return (dflt_wgt);
else
   /* For those pairs that occur in table, return weight from table 
   ----------------------------------------------------------------*/ 
   return (pw_ptr->weight);
}
 

/*****************************************************************************/
int s_compar
(		/* compare routine used by qsort */
    struct pairweight *a,
    struct pairweight *b		/* pairwise weight entry pointers */
)

{
char	msg[ERRLEN+1];			/* buffer for error message */
long	nonfatal = 1;			/* nonfatal flag */

if ((a->fclass == b->fclass) && (a->nclass == b->nclass))
   {
   sprintf(msg,
    "Class pair: %d %d occurs more than once in pairwise weight table.", 
    a->fclass,a->nclass);
   c_errmsg(msg,"smooth-weights",&nonfatal);
   c_errmsg("         Weight value used is unpredictable"," ",&nonfatal);
   }
 
/* Return (0) when two entries are equal; (-) when second entry is greater 
   than first; and (+) when first entry is greater than second  
-----------------------------------------------------------------------*/
if (a->fclass == b->fclass)
   return ((int)(a->nclass - b->nclass));
else
   return ((int)(a->fclass - b->fclass));
}


/*****************************************************************************/
int compar
(		/* compare routine used by bsearch */
    struct pairweight *a,
    struct pairweight *b		/* pairwise weight entry pointers */
)

{
 
/* Return (0) when two entries are equal; (-) when second entry is greater 
   than first; and (+) when first entry is greater than second  
-----------------------------------------------------------------------*/
if (a->fclass == b->fclass)
   return ((int)(a->nclass - b->nclass));
else
   return ((int)(a->fclass - b->fclass));
}
