#include "speccomb.h"
#include "util.h"
#include "protos.h"

void combine
(
    char las_pat[],          /* pattern image LAS name              */
    char las_in[][CMLEN],    /* Level II images LAS names           */
    char hos_out[],          /* output image host name              */
    long num_samp,           /* number of samples per line          */
    long *num_lines,         /* number of lines in the pattern image*/
    long num_images,         /* number of classified images         */
    long max_in_gl[],        /* max grey level in Level II images   */
    long min_in_gl[],        /* min grey level in Level II images   */
    long min_pat_gl,         /* min grey level in pattern image     */
    long max_pat_gl,         /* max grey level in pattern image     */
    long grey_level[]       /* class grey level values             */
)
/*
***************************************************************************

NAME: COMBINE

PURPOSE: Combine level I and level II classifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1            1-Sep-83  K. A. Narayanan
   2           29-Oct-84  S. Ross
   3           18-Aug-88  K. A. Narayanan     Conversion to 5.0
   4		8-Sep-92  S. Nelson	      Set variable "space" equal to " "
					      so that printout will be correct
					      on the sg.
   5.1         16-Aug-93  D. Bridges          Added the PRINT parameter

ALGORITHM DESCRIPTION:

   Store the sequence of specified grey level values in a table
   Set min mapped gl for the first image =  
                           max pix value of pattern image + 1 
   Set min out gl = min mapped gl of first image
   Recursively define min mapped gl for all input images using their 
      min and max pixel values, and compute the offsets to be added 
      to these values
   Set max out gl = max pix val of last input image + its offset
   For all grey levels in pattern image
      If any grey level is stored in the table
         Print the grey level value as being split and the new range
      Else
         Print the grey level value unmapped
         Set min out gl = min (grey level, min out gl)
   Print all the Level II images' input and mapped grey levels
   Initialize and start processing message
   For number of lines in pattern image
      Read a line of pattern image
      For number of pixels in the line
         Read pattern image grey level
         Get the Level II image number corresponding to the grey level 
            from table
	  If it is non-zero then
	     Set grey level found flag
	     Read next line of Level II image if necessary
	     Map the next sequential pixel from input to output adding
               the corresponding offset
	     If mapped value > 255 set the value to 255
	  Else
	     Store the pattern image grey level in the output unmapped
      End do
      Write the line to the output
   If any grey level mapping was found to be > 255 or
      if any specified grey level was not found in pattern image
         inform user
   Close all open input and output files
   Update min and max values in the output image DDR

*****************************************************************************
*/
{
long    inf_err = 1;
long    one = 1;


extern  FILE	*fd;            /* fine descriptor for information file  */

double	max_out_gl,             /* max output grey level                 */
	min_out_gl;             /* min output grey level                 */

long	gl_found[NIMG],         /* indicator for gl found                */
	gloff[NIMG],            /* grey level offsets                    */ 
	glmin,                  /* min grey level                        */
	grey, 
	i, in, j, jin, l,
	line,
	msg_flag,               /* flag for message utility              */
	pix[NIMG],              /* no. pixels processed                  */
	samp,
	table[256],             /* grey level table                      */
	val;

char	buf[132],               /* text buffer                           */
	space[100];



/* 
initialize variables
--------------------
*/
strcpy (space," ");
for (i = 0; i < num_images; i++) 
   {
   gl_found[i] = FALSE;
   pix[i] = 0;
   }
/*
store the sequence of specified grey level values in a table
------------------------------------------------------------
*/
for (i = 0; i < 256; i++) table[i] = -1;
for (i = 0; i < num_images; i++) table[grey_level[i]] = i;

/*
define grey level mapping for the images
----------------------------------------
*/
min_out_gl = glmin = max_pat_gl+1;               /*  mapped min gl for 1st image */

for (i = 0; i < num_images; i++)
   {
   gloff[i] = glmin - min_in_gl[i];            /*  gl offset to add */
   glmin = glmin + max_in_gl[i] - min_in_gl[i] + 1;  /*  mapped min gl */
   }  

max_out_gl = max_in_gl[num_images-1] + gloff[num_images-1];

/*
write the heading for the grey level map table
----------------------------------------------
*/

fprintf (fd, "\n\n\n%20sTABLE OF GREY LEVEL MAPPINGS\n\n", space);
fprintf (fd, "INPUT IMAGE%32sINPUT GREY%10sOUTPUT GREY\n", space, space); 
fprintf (fd, "%46sLEVEL%16sLEVEL\n", space, space); 
fprintf (fd, "-----------%32s----------%10s-----------\n", space, space);

/*
print the pattern image grey level mapping
------------------------------------------
*/
min_out_gl = 0;
for (l = min_pat_gl; l <= max_pat_gl; l++)
   {
   in = table[l];
   if (in >= 0) 
      {
      fprintf (fd, "%-40.40s%3s%3d%17sSplit(%3d -%3d)\n", 
               las_pat, space, l, space, min_in_gl[in]+gloff[in], 
               max_in_gl[in]+gloff[in]);
      }
   else
      {
      fprintf (fd, "%-40.40s%3s%3d%17s%3d\n", las_pat, space, l, space, l);

      if (min_out_gl  ==  0) min_out_gl = l;
      }  
   }  

/*
print the image grey level mapping
----------------------------------
*/
for (j = 0; j < num_images; j++)
   {
   fprintf (fd, "%-40.40s%3s%3d - %3d%11s%3d - %3d\n", 
            las_in[j], space, min_in_gl[j], max_in_gl[j], 
            space, min_in_gl[j]+gloff[j], max_in_gl[j]+gloff[j]);
   }  
fclose (fd);

/*
initialize and start processing message
---------------------------------------
*/
msg_flag = GEN2;
c_inittm ("SPECCOMB", "Processed", "line", " ", " ", num_lines, &one, &msgtim, 
          &msg_flag);
curent = 1;
c_startm();
/*
-----------------------------------------------------------------
read the pattern image one line at a time; determine
the grey level in the pattern; get the corresponding input 
image number from table, get the next sequential pixel, map its
grey level and write it to the output; 
-----------------------------------------------------------------

*/
for (line = 0; line < *num_lines; line++, curent++)
   {
/*
   read a line of pattern image and initialize a line for output
   -------------------------------------------------------------
*/
   c_estep (&gdesc_pat);
   c_estep (&gdesc_out);

/*
   get the input pixel corresponding to the pattern grey level
   -----------------------------------------------------------
*/
   for (samp = 0; samp < num_samp; samp++)
      {
      grey = patin_buf[ppoint+samp];      /*  pattern grey level */

      if ((jin = table[grey]) >= 0)        /*  input image number */
         {
/*
         set grey level found flag
         --------------------------
*/
         if (! gl_found[jin]) gl_found[jin] = TRUE;

/*
         read next line of input if needed
         ---------------------------------
*/
         if (pix[jin]  ==  0) c_estep (&gdesc_in[jin]);

/*
        map the pixel from input to output
        ----------------------------------
*/
         val = in_buf[jin][ipoint[jin]+pix[jin]]+gloff[jin];
         pix[jin]++;                     /*  next sequential position */

         if (val > 255)
            {
            c_errmsg ("Some output grey levels exceed 255; set to 255",
                      "speccomb-outgrey", &inf_err);
            val = 255;
            }
         out_buf[opoint+samp] = (unsigned char) val;

         if (pix[jin]  ==  num_samp) pix[jin] = 0;
         
         }
      else
         {
         out_buf[opoint+samp] = (unsigned char) grey;
/*       
         sprintf (buf, 
                  "The pattern image value (%d) was not specified in MASKVAL",
                  grey);
         c_egclse (&gdesc_out);
         c_abort (&block, buf, "speccomb-patval", "speccomb", hos_out, &one);
*/
         }  
      }  
   }
c_estep (&gdesc_out);
c_egclse (&gdesc_out);
c_stoptm ();

/*
send informational messages
----------------------------
*/
for (i = 0; i < num_images; i++)
   if (! gl_found[i]) 
      {
      sprintf (buf, "Specified grey level %d not found in pattern image",
                 grey_level[i]);                  
      c_errmsg (buf, "speccomb-nogrey", &inf_err);
      }

/*
update min and max values in the out image DDR
----------------------------------------------
*/
c_svmnmx (hos_out, &one, &min_out_gl, &max_out_gl,"YES");
}
