#include "specstrt.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void strat
(
    long wind[][4],          /* input image windows          */
    long num_bands,          /* no. bands in the image       */
    long class_num,          /* class number processed       */
    long *num_classes,       /* no. classes                  */
    long out_num_lines[],    /* no. line in output images    */
    long grey_level[8],      /* class grey level values      */
    char tae_out[][CMLEN],   /* output images TAE names      */
    char hos_out[][CMLEN],   /* output images host names     */
    char hos_in[][CMLEN],    /* input images host names      */
    long *num_inimages,      /* no. input images             */
    long type               /* image data type              */
)
/*
**********************************************************************

NAME: STRAT                                 

PURPOSE: Produces a class multispectral image from input multispectral 
          image

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           30-Aug-83  K. A. Narayanan
   2           24-Feb-84  J. Seamone 
   3            5-Dec-88  K. A. Narayanan     Conversion to 5.0

ALGORITHM DESCRIPTION:

   If specified grey level is not found in input image 
      (no. lines in output class image is zero) 
      Inform user and return without further processing
   Otherwise, for number of lines in the pattern image
      Read a line of pattern and input image
      For number of pixels in the line
         Get pattern image grey level
         If pattern gl = specified class gl then
            For number of bands
               Copy the pixel from input to output buffer
               If output buffer is full 
                  Write the line to the output file
   If output buffer (last line ) is partially full then
      Fill the rest of the buffer with zeros
      Write the line to the output file
   Close pattern, input and output images
   Update output class image history
   Report name output image generated
  
*****************************************************************************
*/
{
static	float	fill_val = 0.0;

long	grey_val, 
	k, 
	line, 
	msg_flag,
	nsamp, 
	num_fill, 
	samp,
	size = 0,
	inf_err = 1,
    zero = 0;

char	buf[100];

long	*lip, *lop;
float	*fip, *fop;
short	*sip, *sop;

/*
if specified grey level not found in the pattern image no output image is
generated for that class; return
-------------------------------------------------------------------------
*/
if (out_num_lines[class_num]  ==  0) 
   {
   sprintf (buf, 
      "Specified grey level %d not found in pattern image", 
       grey_level[class_num]);
   c_errmsg (buf, "specstrt-nogrey", &inf_err);
   sprintf (buf, "No output image produced for class %d",class_num+1);
   c_errmsg (buf, "specstrt-noout", &inf_err);

   return;
   }
/*
-----------------------------------------------------------------
read the pattern and input images one line at a time; determine
the grey level in the pattern; if the grey level corresponds to
the current class, store the input pixel at the next sequential
position in the output; write the output line when full.
-----------------------------------------------------------------
*/
nsamp = 0;
switch (type)
   {
   case EBYTE: size = 1; break;
   case EWORD: size = sizeof (short); break;
   case ELONG: size = sizeof (long); break;
   case EREAL: size = sizeof (float); break;
   }

/*
initialize and start processing message
---------------------------------------
*/
msg_flag = ONECAT;
c_inittm ("SPECSTRT", "Processed", "line", "class", " ", 
          &out_num_lines[class_num], num_classes, 
          &msgtim, &msg_flag);
curcat = class_num + 1;
curent = 1;
c_startm();

c_estep (&gdesc_out);
for (line = 0; line  <  wind[0][NL]; line++)
   {
/*
   read a line of pattern and input image
   --------------------------------------
*/
   c_estep (&gdesc_pat);
   c_estep (&gdesc_in);

   for (samp = 0; samp < wind[0][NS]; samp++)
      {
      grey_val = patin_buf[ppoint+samp];      /*  pattern grey level */

      if (grey_val  ==  grey_level[class_num]) 
         {

/*
         copy the pixel from input to output
         -----------------------------------
*/
         for (k = 0; k < num_bands; k++)
            switch (type)
               {
               case EBYTE:
                  out_buf[opoint[k]+nsamp] = in_buf[ipoint[k]+samp];
                  break;
               case EWORD:
                  sop = (short *) &out_buf[(opoint[k]+nsamp) * size];
                  sip = (short *) &in_buf[(ipoint[k]+samp) * size];
                  *sop = *sip;
                  break;
               case ELONG:
                  lop = (long *) &out_buf[(opoint[k]+nsamp) * size];
                  lip = (long *) &in_buf[(ipoint[k]+samp) * size];
                  *lop = *lip;
                  break;
               case EREAL:
                  fop = (float *) &out_buf[(opoint[k]+nsamp) * size];
                  fip = (float *) &in_buf[(ipoint[k]+samp) * size];
                  *fop = *fip;
                  break;
               }
         nsamp++;                            /*  next sequential position */

/*
	 write out the image line when full
	 ----------------------------------
*/
         if (nsamp  ==  wind[0][NS])
            {
            c_estep (&gdesc_out);
            nsamp = 0;
            curent++;
            }  
         }  
      }  
   }  

/*
write out the last line padded with zeros
-----------------------------------------
*/
if (nsamp  ==  0)    
   num_fill = 0;
else
   {
   num_fill = wind[0][NS] - nsamp;
   for (k = 0; k < num_bands; k++)
      switch (type)
         {
         case EREAL:
            c_pxfill (&out_buf[(opoint[k]+nsamp) * size], &type, &num_fill, 
                      &fill_val);
            break;
         default:
            c_pxfill (&out_buf[(opoint[k]+nsamp) * size], &type, &num_fill, 
                      &zero);
         }
   c_estep (&gdesc_out);
   curent++;
   }  

c_stoptm();
if (num_fill != 0)
   {
   sprintf (buf, "Output image  will have %d zeros on the last line", 
            num_fill);
   c_errmsg (buf, "specstrt-zeros", &inf_err);
   }

/*
close all the image files and free dynamic storage
--------------------------------------------------
*/ 
c_egclse (&gdesc_pat);
c_egclse (&gdesc_in);
c_egclse (&gdesc_out);

free (patin_buf);
free (in_buf);
free (out_buf);

/*
update out class image history and report output file name
----------------------------------------------------------
*/
c_hstout (&block, hos_out[class_num], hos_in, num_inimages, " ");

sprintf (buf, "Created output image : %s for Class %d",
         tae_out[class_num], class_num+1);
c_errmsg (buf, "specstrt-outfile", &inf_err);

}
