/******************************************************************************
NAME: 		conic

PURPOSE: 	Fill DDR with Equidistant Conic (A and B) projection 
                information.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      02/00  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: 	LAS

ALGORITHM:
Set projection parameters to valid

If semi major does not exits
    Issue a warning message
    Set projection parameters to invalid
Else
    Set in DDR

If semi minor does not exits
    Issue a warning message
    Set projection parameters to invalid
Else
    Set in DDR

If parallel1 does not exist
    Issue a warning message
    Set projection parameters to invalid
Else
    Convert from degrees to DMS
    Set in DDR

If parallel2 exists
    Convert from degrees to DMS
    Set in DDR
    Set 1 in DDR to reflect conic B

If longitude does not exist
    Issue a warning message
    Set projection parameters to invalid
Else
    Convert from degrees to DMS
    Set in DDR

If latitude does not exist
    Issue a warning message
    Set projection parameters to invalid
Else
    Convert from degrees to DMS
    Set in DDR

If false easting does not exist
    Issue a warning message
    Set projection parameters to invalid
Else
    Set in DDR

If false northing does not exist
    Issue a warning message
    Set projection parameters to invalid
Else
    Set in DDR

If projection units is kilometers
    Then set false easting and false northing to meters

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "spothdf2las.h"
#include "geompak.h"

void conic
(
    struct DDR *ddr, 	    /* I/O: local DDR structure                      */
    char semi_major[],	    /* I: spheroid semi major axis of the image      */
    char semi_minor[],	    /* I: spheroid semi minor axis of the image      */
    char parallel1[],	    /* I: first projection standard parallel         */
    char parallel2[],       /* I: second projection standard parallel        */
    char longitude[],	    /* I: projection longitude of the image          */
    char latitude[],	    /* I: projection latitude of the image           */
    char false_easting[],   /* I: x coordinate of the origin for projection  */
    char false_northing[],  /* I: y coordinate of the origin for projection  */
    int proj_kilometers     /* I: flag indicating if projection units is in
                                  kilometers                                 */
)
{
    double dms;             /* temporary storage of DMS value                */
    double double_val;      /* temporary storage of angle value              */


    /* Set projection parameters to valid
       ------------------------------------*/
    ddr->valid[3] = VALID;


    /* Check projection parameters and set in DDR. If they do not exist then
       set projection parameters to invalid
       ---------------------------------------------------------------------*/
    if (strcmp(semi_major, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-major", 
                "Semi Major Axis does not exist in logfile");
        ddr->valid[3] = INVAL;
    }
    else
        ddr->proj_coef[0] = atof(semi_major);

    if (strcmp(semi_minor, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-minor",
                "Semi Minor Axis does not exist in logfile");
        ddr->valid[3] = INVAL;
    }
    else
        ddr->proj_coef[1] = atof(semi_minor);

    if (strcmp(parallel1, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-paral", 
                "Standard Parallel 1 does not exist in logfile");
        ddr->valid[3] = INVAL;
    } 
    else 
    {
        /* convert degrees to packed DMS */
        double_val = atof(parallel1);
        c_degdms(&double_val,&dms,"DEG","LAT");
        ddr->proj_coef[2] = dms; 
        dms = 0.0;
    }

    /* There is no need to issue a warning message here. If parallel2 does not 
       exist, then the projection is Equidistant Conic B */
    if (strcmp(parallel2, "") != 0)
    {
        double_val = atof(parallel2);
        c_degdms(&double_val,&dms,"DEG","LAT");
        ddr->proj_coef[3] = dms;
        dms = 0.0;
        ddr->proj_coef[8] = 1; /* Add  1 to the DDR to reflect Equid. Conic B */
    }

    if (strcmp(longitude, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-long", 
                "Projection Longitude does not exist in logfile");
        ddr->valid[3] = INVAL;
    }
    else
    {
        double_val = atof(longitude);
        c_degdms(&double_val,&dms,"DEG","LON");
        ddr->proj_coef[4] = dms;
        dms = 0.0;
    }

    if (strcmp(latitude, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-lat", 
                "Projection Latitude does not exist in logfile");
        ddr->valid[3] = INVAL;
    }
    else
    {
        double_val = atof(latitude);
        c_degdms(&double_val,&dms,"DEG","LAT");
        ddr->proj_coef[5] = dms;
    }

    if (strcmp(false_easting, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-east", 
                "False easting does not exist in logfile");
        ddr->valid[3] = INVAL;
    }
    else
        ddr->proj_coef[6] = atof(false_easting);

    if (strcmp(false_northing, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-north", 
                "False northing does not exist in logfile");
        ddr->valid[3] = INVAL;
    }
    else
        ddr->proj_coef[7] = atof(false_northing);

    /* If projection units are kilometers convert to meters
       ------------------------------------------------------*/
    if (proj_kilometers == 1)
    {
        ddr->proj_coef[6] = ddr->proj_coef[6] * 1000;
        ddr->proj_coef[7] = ddr->proj_coef[7] * 1000;
    }
}
