/******************************************************************************
NAME:		gethdfinfo

PURPOSE:        Reads HDF image files using SD interface and VS interface

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      02/00  D. Hames          C    Original developement


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: 	LAS

ALGORITHM:
Begin Loop (loop on each input file)
        Open input file for reading for SD interface
        Initialize the SD interface
        Get SD selected information  (rank, dimensions and data type)
        Copy dimensions to nl and ns
        Copy data type to idtype
        Terminate access to SD interface
        Close HDF files
End Loop

If the number of LAS files equals 1
        Then check all HDF input data types
        If they are not the same
                Then fatal error

Copy the HDF data type(s) to a LAS data type

Begin Loop (loop on each input file)
        Open input file for reading - using VS interface
        Initialize the VS interface
        Get VS selected information  (description of input file)
        Copy the vdata name into an array
        Terminate access to VS interface
        Close the HDF files
End Loop

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "mfhdf.h"
#include "worgen.h" 
#include "spothdf2las.h"
#define  MAXFIELD_SIZE     80         /* maximum length of the field names */

void gethdfinfo
(
    char infile[][CMLEN],   /* I: TAE file name(s) of input HDF images    */
    int numhdf_files,       /* I: total number of input HDF files         */
    int numlas_files,       /* I: total number of output LAS files        */
    long nl[],              /* O: total number of lines                   */
    long ns[],              /* O: total number of samples                 */
    int idtype[],           /* O: LAS data type to be used                */
    char hdf_name[][CMLEN]  /* O: HDF file descriptons needed for DDR     */
)
{
    char sds_name[MAX_NC_NAME];   /* name of the data set		             */
    char fieldname_list[MAXFIELD_SIZE]; /* buffer to retrieve the vdata data */
    char vdata_name[VSNAMELENMAX];/* buffer to retrieve the vdata name       */
    int i;                        /* loop index                              */
    int index;                    /* loop index                              */
    intn status_n;                /* status for functions returning an intn  */
    int32 dim_sizes[3];           /* size of each dimension in the data set  */
    int32 data_type;              /* data type of data in data set           */ 
    int32 file_id;                /* VS file identifier                      */
    int32 in_dtype[NUMFILE];      /* array of data types for the HDF file(s) */
    int32 interlace_mode;         /* interlace mode of the vdata             */
    int32 n_attrs;                /* number of attributes in data set        */
    int32 n_records;              /* number of records in the vdata          */
    int32 rank;                   /* number of dimensions in data set        */
    int32 status_32;              /* status for functions returning an int32 */
    int32 sds_id;                 /* data set identifier                     */
    int32 sd_id;                  /* SD interface identifier                 */
    int32 vdata_size;             /* size of all specified fields            */
    int32 vdata_ref;              /* reference number of the vdata           */
    int32 vdata_id;               /* vdata identifier                        */	


    /* Open the input file(s) for reading and initialize the SD interface
       --------------------------------------------------------------------*/
    for (i=0; i < numhdf_files; i++)
    {
        sd_id = SDstart (infile[i], DFACC_READ);
        if (sd_id == HDF_FAIL)
        {
            c_lasmsg(FATAL,"spothdf2las-SDstart",
                    "Error opening file %s",infile[i]);
        }


        /* Get SD selected image information, (rank, dimension sizes and 
           data type)
           --------------------------------------------------------------*/
        /*select first data set  */
        sds_id = SDselect (sd_id, 0);

        status_n = SDgetinfo(sds_id,sds_name, &rank, dim_sizes, &data_type, 
                &n_attrs);
        if (status_n == HDF_FAIL)
        {
            c_lasmsg(FATAL, "spothdf2las-SDinfo", 
                    "Error reading image information from file %s", infile[i]);
        }
        if (rank == 2)
        {
            nl[i] = dim_sizes[0];
            ns[i] = dim_sizes[1];
        }
        /* Since Spot HDF contains 1 band per image file, rank can only be 2 */
        else 
        {
            c_lasmsg(FATAL, "spothdf2las-rank",
                    "SPOTHDF2LAS cannot handle SD of a rank other than 2");
        }

        in_dtype[i] = data_type; 

        /* Terminate access to the data set and to the SD interface and close 
           the file
           -----------------------------------------------------------------*/
        status_n = SDendaccess (sds_id);
        if (status_n == HDF_FAIL)
        {
            c_lasmsg(NONFATAL, "spothdf2las-warn",
                    "Error terminating access to SD data set in file %s", 
                    infile[i]);
        }

        status_n = SDend (sd_id);
        if (status_n == HDF_FAIL)
        {
            c_lasmsg(NONFATAL,"spothdf2las-SDwarn","Error closing file %s",
                    infile[i]);
        }
    }

    /* If the user requests only one output but inputs more than one input 
       file then all HDF infiles must have the same data type. If not issue  
       a fatal error.
       ----------------------------------------------------------------------*/
    if (numlas_files == 1)
    {
        for (index = 1; index < numhdf_files; index++)
        {
            if (in_dtype[0] != in_dtype[index])
            {             
                c_lasmsg(FATAL, "spothdf2las-dtype",
                        "Input data types must be the same if only one "
                        "output image is specified");
            }

        }

    } 

    /* Select a LAS data type based on SD data type
       -----------------------------------------------*/
    for (i=0; i < numlas_files; i++)
    {
        switch (in_dtype[i])	
        {
            case DFNT_UINT8:
            case DFNT_INT8:
                idtype[i] = EBYTE;
                break;
            case DFNT_UINT16:
            case DFNT_INT16:
                idtype[i] = EWORD;
                break;
            case DFNT_UINT32:
            case DFNT_INT32:
                idtype[i] = ELONG;
                break;
            case DFNT_FLOAT32:
                idtype[i] = EREAL;
                break;
            default: c_lasmsg(FATAL,"spothdf2las-dtype",
                             "HDF contains a data type not supported by LAS");
        }
    }


    /* Open the HDF file(s) for reading and initialize the VS interface
       ------------------------------------------------------------------*/
    for (i=0; i < numhdf_files; i++)
    {
        file_id = Hopen (infile[i], DFACC_READ, 0);
        status_n = Vstart (file_id);
        if (status_n == HDF_FAIL)
        {
            c_lasmsg(FATAL, "spothdf2las-Vstart", "Error opening file %s",
                    infile[i]);
        }

        /* Set vdata_ref to -1 to start the search from the beginning of file
           -----------------------------------------------------------------*/
        vdata_ref = -1;

        do
        {
            /* Use VSgetid to obtain the first vdata by its reference number 
               then attach to the vdata and get its information.  
               ------------------------------------------------------------*/
            vdata_ref = VSgetid (file_id, vdata_ref); 
            /* If the file has yet to be searched and a vdata reference number 
               can't be retrieved, then error */
            if ((vdata_ref == -1) && (strcmp (fieldname_list, "") == 0))
            {
                c_lasmsg(FATAL,"spothdf2las-vdata",
                        "Error retrieving a vdata reference number for file %s",
                        infile[i]);
            }
            /* If the file has been searched and IMAGE INFORMATION has not been 
               found, then error */
            else if ((vdata_ref == -1) && (strcmp (fieldname_list, "") != 0))
            {
                c_lasmsg(FATAL,"spothdf2las-vdata",
                        "Error retrieving IMAGE INFORMATION from file %s",
                        infile[i]);
            }

            /* Attach to the vdata for reading
               ---------------------------------*/
            vdata_id = VSattach (file_id, vdata_ref, "r");
            if (vdata_id == -1)
            {
                c_lasmsg(FATAL,"spothdf2las-vdata",
                        "Error attaching to vdata in file %s",infile[i]);
            }


            /* Obtain vdata information
               --------------------------*/
            status_n = VSinquire (vdata_id, &n_records, &interlace_mode,
                    fieldname_list, &vdata_size, vdata_name);
            if (status_n == HDF_FAIL)
            {
                c_lasmsg(FATAL, "spothdf2las-vdata",
                        "Error reading vdata in file %s", infile[i]);
            }


            /* In order to retrieve the correct vdata_name from the correct 
               vdata_id, check that the fieldname_list = "IMAGE INFORMATION"
               before it's copied to an array.
               -------------------------------------------------------------*/
        }
        while (strcasecmp(fieldname_list,"IMAGE INFORMATION") != 0);

        /* Copy the vdata name into an array
           ----------------------------------*/
        strcpy(hdf_name[i],vdata_name);

        /* Detach from the current vdata
           -------------------------------*/
        status_32 = VSdetach (vdata_id);
        if (status_32 == HDF_FAIL)
        {
            c_lasmsg(NONFATAL,"spothdf2las-warn",
                    "Unable to detach from vdata in file %s", infile[i]);
        }

        /* Terminate access to the vdata and to the VS interface, then close 
           the HDF file
           -------------------------------------------------------------------*/
        status_n = Vend (file_id);
        if (status_n == HDF_FAIL)
        {
            c_lasmsg(NONFATAL,"spothdf2las-warn",
                    "Unable to end access to vdata in file %s", infile[i]);
        }

        status_32 = Hclose (file_id);
        if (status_32 == HDF_FAIL)
        {
            c_lasmsg(NONFATAL,"spothdf2las-VSwarn",
                    "Error closing file %s", infile[i]);
        }
    }
}
