/*****************************************************************************
FUNCTION	getimage

PURPOSE: 	Reads the desired image into an array

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      02/00  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under UNIX and TAE.

PROJECT:         LAS

ALGORITHM:
Open the input file for reading and initialize the SD interface
Select the first data set
Set SDS dimensions
Allocate memory for hdfbuf
Loop on each line of data and write to output image
Free buffer
Terminate access to the data set 
Terminate access to the SD interface 
Close the HDF file

******************************************************************************/
#include <stdio.h>
#include "mfhdf.h"
#include "spothdf2las.h"

extern long curent;                    /* current processing line */
extern long curcat;                    /* current processing band */

int getimage
(
    char infile[],              /* I: input HDF image file name            */
    long nl,                    /* I: total number of lines                */ 
    long ns,                    /* I: total number of samples              */
    struct FDESC *fdesc,        /* I: image file descriptor                */ 
    char hosout[],              /* I: host name of output image            */
    int idtype                  /* I: size of input data type of HDF image */
)
{
    int32 sd_id;            /* SD interface identifier                 */
    int32 sds_id;           /* data set identifier                     */
    int32 start[2];         /* position at which read will start       */
    int32 edges[2];         /* number of data elements to be read      */
    intn status;            /* status for functions returning an intn  */
    unsigned char *hdfbuf;  /* input buffer                            */
    int one = 1;            /* constant one                            */
    long current_line;      /* current line being read in image        */

    /* Open the input file for reading and initialize the SD interface
       ------------------------------------------------------------------*/
    sd_id = SDstart (infile, DFACC_READ);
    if (sd_id == HDF_FAIL)
    {
        c_lasmsg(NONFATAL,"spothdf2las-SDopen", "Error opening file %s", 
                 infile);
        return (E_FAIL);
    }

    /*Select the first data set
      --------------------------*/
    sds_id = SDselect (sd_id, 0);


    /* Set elements of array start to 0, elements of array edges
       to SDS dimensions,and use NULL for the argument stride in SDreaddata
       to read the entire data.
       -------------------------------------------------------------------*/
    start[0] = 0;
    start[1] = 0;
    edges[0] = 1; 
    edges[1] = (int32)ns;

    /* Allocate the input buffer
       --------------------------*/
    if ((hdfbuf = (unsigned char *) malloc(ns * idtype)) == NULL) 
    {
        c_lasmsg(NONFATAL, "spothdf2las-alloc",  
                "Error allocating dynamic memory");
        return (E_FAIL);
    }

    /* Loop on each line of data and read into a data array. Write to 
       output image.
       ------------------------------------------------------------------*/
    for (current_line=1; current_line<=nl; current_line++)  
    {
        status = SDreaddata (sds_id, start, NULL, edges, (VOIDP)hdfbuf);
        if (status == HDF_FAIL)
        {
            c_lasmsg(NONFATAL, "spothdf2las-SDread",
                    "Error reading image data from file %s", infile);
            status = SDendaccess (sds_id);
            if (status == HDF_FAIL)
            {
                c_lasmsg(NONFATAL, "spothdf2las-warn",
                        "Error terminating access to SD data set in file %s", 
                        infile);
            }           
            status = SDend (sd_id);
            if (status == HDF_FAIL)
                c_lasmsg(NONFATAL, "spothdf2las-warn", "Error closing file %s", 
                        infile);
            free(hdfbuf);
            return (E_FAIL);
        }
        status = c_ewrite(&fdesc, &curcat, &current_line,
                hdfbuf, &one);
        if (status == E_FAIL)
        {
            if (c_eclose(&fdesc) != E_SUCC)
            {
                c_lasmsg(NONFATAL,"spothdf2las-warn",
                        "Error closing file outfile %s", hosout);
            }
            free(hdfbuf);
            return (E_FAIL);
        }
        start[0] = current_line; /* start at next line in the image */
        curent ++;
    }

    /* Free the buffer space previously allocated
       --------------------------------------------*/
    free(hdfbuf);

    /* Terminate access to the data set and to the SD interface and close 
       the file
       -----------------------------------------------------------------*/
    status = SDendaccess (sds_id);
    if (status == HDF_FAIL)
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Error terminating access to SD data set in file %s", infile);

    status = SDend (sd_id); 
    if (status == HDF_FAIL)
        c_lasmsg(NONFATAL, "spothdf2las-warn", "Error closing file %s", infile);


    return (E_SUCC);
}

