/*******************************************************************************
NAME:		process_hdf

PURPOSE:        Reads in Scientific Data Sets from an HDF image file(s) and 
                writes it to a LAS image file.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   D.Hames	   02/00	  EDC 		Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    must be run under UNIX and TAE.

PROJECT:	LAS

ALGORITHM:
Allocate the input HDF buffer
Copy the input data type to the output data type
Open the output LAS image file(s) for write access
Initialize the processing message
Begin loop (on each input HDF file)
	Retrieve HDF image using getimage
        Write out LAS image
End loop
Free HDF buffer
Update the DDR and BDDR
Close the output file

******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "spothdf2las.h"
#include "util.h"

/* The following variables must be declared global for image I/O routines*/

long curent;             /* current processing line */
long curcat;             /* current processing band */

int process_hdf
(
    char infile[][CMLEN],   /* I: input image name                        */
    char hosout[],          /* I: host name of output image               */
    long msgtim,            /* I: message time                            */
    int numhdf_files,       /* I: total number of HDF input files         */
    long nl,                /* I: total number of lines                   */
    long ns,                /* I: total number of samples                 */
    int idtype,             /* I: input data type size of HDF image file  */
    char hdf_name[][CMLEN],	/* I: HDF image file descriptions used in DDR */
    char logfile[]          /* I: file containing geographic information  */
)
{
    struct FDESC *fdesc;    /* image file descriptors used for image I/O */ 
    int nbands;		        /* number of bands in the output LAS image   */
    int status;             /* status indicator                          */
    int odtype;             /* output data type of the LAS image         */
    long acc = IWRITE;		/* image file open access                    */
    long flag = ONECAT;		/* display message flag	                     */
    long opt = 0;			/* image file open option                    */
    long sl = 1;			/* image file starting line                  */
    long ss = 1;			/* image file starting sample	             */


    /* The LAS output data type will be the same as the HDF input data type
       ----------------------------------------------------------------------*/
    odtype = idtype;

    /* The total number of output bands equals the total number of HDF files
       ----------------------------------------------------------------------*/
    nbands = numhdf_files;

    /* Open the output image file for write access
       ---------------------------------------------*/
    status = c_eopenr(&fdesc,hosout, &nbands, &sl, &ss, 
            &nl, &ns, &acc, &odtype, &opt, &nl);
    if (status != E_SUCC)
    {
        c_lasmsg(NONFATAL,"spothdf2las-open", "Error opening output image %s",
                hosout);
        return (E_FAIL);
    }

    /* Initialize the processing message
       -----------------------------------*/
    curcat = 1;
    curent = 1;
    c_inittm("SPOTHDF2LAS", "Processed", "line", "bands", " ", &nl, &nbands,
            &msgtim, &flag);
    c_startm();

    /* Loop on each input HDF file
       -----------------------------*/
    for (curcat = 1; curcat <= nbands; curcat++)
    {
        curent = 1;
        status = getimage(infile[curcat-1], nl, ns, fdesc, hosout, idtype);
        if (status != E_SUCC)
        {
            if (c_eclose(&fdesc) != E_SUCC)
            {
                c_lasmsg(NONFATAL,"spothdf2las-warn","Error closing file %s",
                         hosout);
            }
            c_lasmsg(NONFATAL, "spothdf2las-read", 
                    "Error reading input image %s", infile[curcat-1]);
            return (E_FAIL);
        }
    }				

    /* Close the output file
       ------------------------*/
    status = c_eclose(&fdesc);
    if (status != E_SUCC)
        c_lasmsg(NONFATAL, "spothdf2las-warn", "Error closing file %s", hosout);


    /* Update the DDR record file
       -----------------------------*/
    if (E_FAIL == update_ddr(hosout, hdf_name, nbands, logfile))
    {
        c_lasmsg(NONFATAL, "spothdf2las-ddr", "Error updating DDR");
        if (c_eclose(&fdesc) != E_SUCC)
            c_lasmsg(NONFATAL,"spothdf2las-warn", "Error closing file %s",
                    hosout);
        return (E_FAIL);
    }

    c_stoptm();

    return (E_SUCC);
}				
