/******************************************************************************
PROGRAM:	spothdf2las

PURPOSE:	Converts SPOT HDF image file(s) to LAS image file(s).

PROGRAM HISTORY:
 mode Version	Date	Author		Code/Contr	Change Request
  ------	----	------		----------	--------------
  1.0		02/00  D. Hames	           EDC		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    SPOTHDF2LAS must be run under TAE. 

PROJECT:        LAS                             

ALGORITHM:

Get user specified parameters and TAE global.
Read HDF file(s) and get selected information.
Process HDF
    If one output LAS file
        Then process all HDF(s) at once
    Else 
        Begin loop 
            Process one HDF file at a time
        End loop
Create history records
    Begin loop
        Create history file - one for every LAS image
    End loop
Output completion message

******************************************************************************/

#include <stdio.h>
#include "spothdf2las.h"
#include "util.h"

/*
 * The following variables must be declared global for image I/O routines
 */

void main  
(void)
{
    struct PARBLK vblock;/* TAE parameter block 			     */

    char infile[NUMFILE][CMLEN]; /* input HDF image file names in LAS format  */
    char hosout[NUMFILE][CMLEN]; /* output LAS image file names in host form  */
    char taeout[NUMFILE][CMLEN]; /* output LAS image file names in LAS format */
    char hdf_name[NUMFILE][CMLEN];/* HDF image file descriptions 
                                     needed for DDR                           */
    char logfile[CMLEN];         /* file containing geographic information    */
    int idtype[NUMFILE];         /* input HDF file data type(s)               */
    int i;                       /* index used for looping                    */
    int numhdf_files;            /* total number of HDF image files specified */
    int numlas_files;            /* total number of LAS image files specified */
    int zero = 0;                /* zero value                                */
    long msgtim;                 /* # of seconds between processing messages  */
    long nl[NUMFILE];            /* total number of lines                     */
    long ns[NUMFILE];            /* total number of samples                   */

    /* Get user specified parameters and TAE global
       ----------------------------------------------*/
    getpar(&vblock, infile, hosout, taeout, &numhdf_files, &numlas_files, 
            &msgtim,logfile);

    /* Read HDF image file(s) and get selected information
       ----------------------------------------------------*/
    gethdfinfo(infile, numhdf_files, numlas_files, nl, ns, idtype, hdf_name);

    /* Process HDF 
       -------------*/
    /* If one output LAS file, then process all HDF image files at once*/
    if (numlas_files == 1)
    {
        if (E_FAIL == process_hdf(infile, hosout[0], msgtim, numhdf_files, 
                nl[0], ns[0], idtype[0], hdf_name, logfile))
        {
            c_lasmsg(FATAL, "spothdf2las-fatal", "Fatal error encountered");
        }
    } 
    /* If one output LAS file per HDF input file, then process one HDF image 
       file at a time */
    else 
    {
        for (i=0; i < numlas_files; i++)
        {
            if (E_FAIL == process_hdf((char (*)[256])infile[i],
                    hosout[i], msgtim, 1, nl[i], ns[i], idtype[i], 
                    (char (*)[256])hdf_name[i],logfile))
            {
                c_lasmsg(FATAL, "spothdf2las-fatal", "Fatal error encountered");
            }
        }
    }     
    /* Create history record
       -----------------------*/
    for (i=0; i < numlas_files; i++)
    {    
        if (c_hstout(&vblock,hosout[i],(char (*)[256])" ",&zero," ") != E_SUCC)
            c_lasmsg(NONFATAL,"spothdf2las-warn","Nonfatal error encountered");
    }

    /* Output completion message
       ---------------------------*/
    c_complt("SPOTHDF2LAS", taeout, &numlas_files);

} 

