/******************************************************************************
NAME: 		update_ddr

PURPOSE: 	Update the DDR(s) with the information obtained from 
                the logfile. 

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      02/00  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: 	LAS

ALGORITHM:

Open logfile for read access
Search logfile for each keyword

Get the DDR

Check for projection
    If it does not exist
        Then issue a warning message

Set the projection and get the projection parameters
    If they do not exist
        Then issue a warning message

Set datum or spheroid
    If they do not exist
        Then issue a warning message

Set zone to 62

Set projections units
    If it does not exist
        Then issue a warning message
    If projection units is kilometers
        Then convert to meters

Set master line and sample
    If they do not exist
        Then issue a warning message

Set line and sample increments to 1

Set corner coordinates
    If any of them do not exist
        Then issue a warning message
    If projection units is kilometers
        Then convert kilometers to meters

Set pixel distance
    If it does not exist
        Then issue a warning message
    If projection units is kilometers
        Then convert kilometers to meters

Get the BDDR
    Loop on each band

Write to DDR
    Loop on each output LAS file

Write to BDDR
    Loop on each bands

Return E_SUCC or E_FAIL        

PROGRAMMER NOTES:

SPOT Vegetation data products are available in nine projections. They are
Plate Carree, Goode Homolosine, Equatorial Mercator, Equivalent Sinusoidal,
Albers, Mercator, Conic, Lambert and Polar Stereo.  The projections Equatorial 
Mercator, Equivalent Sinusoidal, Lambert and Mercator are not currently 
supported. There are some questions as to which projections in LAS should be  
used for these projections in SPOT. SPOT was asked to provide detailed 
information (ie, what the projection parameters are) on these projections. 
SPOTHDF2LAS will not support them until that information is obtained.

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "spothdf2las.h"
#include "geompak.h"
#include "proj.h"

#define LINELEN 80	    /* Maximum ASCII line length in the logfile      */
#define NOT_READ 0	    /* The log file has not been read and the DDR 
                           structure has not been updated                */
int update_ddr
(
    char hosout[],          /* I: name of the host ddr to update     	     */
    char hdf_name[][CMLEN], /* I: HDF image file descriptions needed for DDR */ 
    int nbands,	            /* I: total number of bands in the LAS image     */
    char logfile[]	        /* I: file containing geographic information     */
)
 
{
    FILE *file_ptr;		      /* file pointer                               */
    struct BDDR bddr[NUMFILE];/* local BDDR structure                       */
    struct BDDR *bptr;        /* pointer to the current BDDR record         */
    struct DDR ddr;           /* local DDR structure                        */

    char datum[LINELEN];      /* datum name retrieved from HDF logfile      */
    char line[BUFSIZ];        /* line scanned in from logfile               */
    char keyword[25];         /* keyword in logfile,maximum length is 25 chars*/
    char *words;              /* temporary storage of the line contents      */
    char false_easting[LINELEN]={'\0'};/* x coord of the origin used for 
                                          projection                         */
    char false_northing[LINELEN]={'\0'};/* ycoord of the origin used for 
                                           projection*/
    char latitude[LINELEN]={'\0'};   /* projection latitude of the image     */
    char longitude[LINELEN]={'\0'};  /* projection longitude of the image    */
    char projection[LINELEN]={'\0'}; /* projection of image                  */
    char projection_units[LINELEN]={'\0'}; /* units meter/kilometer/degrees  */
    char resolution[LINELEN]={'\0'}; /* resolution of image		             */
    char spheroid[LINELEN]={'\0'};   /* spheroid name retrieved from HDF
                                        logfile                              */
    char semi_major[LINELEN]={'\0'}; /* spheroid semi major axis of the image*/
    char semi_minor[LINELEN]={'\0'}; /* spheroid semi minor axis of the image*/
    char sphere_radius[LINELEN]={'\0'};/* sphere radius	                     */
    char scale_factor[LINELEN]={'\0'}; /* projection scale factor            */
    char parallel1[LINELEN]={'\0'};    /* first projection standard parallel */
    char parallel2[LINELEN]={'\0'};    /* second projection standard parallel*/
    char master_line[LINELEN]={'\0'};  /* line relative to master image      */
    char master_sample[LINELEN]={'\0'};/* sample relative to master image    */
    char upper_left_x[LINELEN]={'\0'}; /* image upper left corner (x coord)  */
    char upper_left_y[LINELEN]={'\0'}; /* image upper left corner (y coord)  */
    char upper_right_x[LINELEN]={'\0'};/* image upper right corner (x coord) */
    char upper_right_y[LINELEN]={'\0'};/* image upper right corner (y coord) */
    char lower_right_x[LINELEN]={'\0'};/* image lower right corner (x coord) */
    char lower_right_y[LINELEN]={'\0'};/* image lower right corner (y coord) */
    char lower_left_x[LINELEN]={'\0'}; /* image lower left corner (x coord)  */
    char lower_left_y[LINELEN]={'\0'}; /* image lower left corner (y coord)  */
    int i;                      /* index used for looping                    */
    int proj_code;              /* LAS projection code                       */
    int proj_kilometers = 0;    /* flag indicating projection units 
                                   are kilometers                            */


    /* Open logfile for read access
       ------------------------------*/
    if ((file_ptr = fopen(logfile, "r")) == NULL)
    {
        c_lasmsg(NONFATAL, "spothdf2las-open", "Error opening logfile");
        return (E_FAIL);
    }

    /* Get DDR information from logfile
       ----------------------------------*/
    /* find keyword(s) until end of logfile */
    while (fgets(line,BUFSIZ,file_ptr) != NULL) 
    {
        sscanf(line,"%s", keyword);  /* keywords will always be in uppercase */

        /* Map Projection Information */
        if (strncmp (keyword, "MAP_PROJ_FAMILY",15) == 0)
        {
            strcpy(projection, "");
            words = strtok(&line[15], " \r\n"); 
            while (words != NULL)
            {
                strcat(projection, words);
                words = strtok(NULL, "\r\n");
            }
        }
        else if (strncmp (keyword, "MAP_PROJ_UNIT",13) == 0)
        {
            words = strtok(&line[13], " \r\n");
            strcpy(projection_units, words);
            if(strcmp(projection_units, "KILOMETERS") == 0)
                proj_kilometers = 1;
        }
        else if (strncmp (keyword, "MAP_PROJ_RESOLUTION",19) == 0) 
        {
            words = strtok(&line[19], " \r\n");
            strcpy(resolution, words);
        }

        /* Datum and Spheroid Information */
        else if (strncmp (keyword, "HORIZ_DATUM",11) == 0) 
        {
            strcpy(datum, "");
            words = strtok(&line[11], " \r\n");
            while (words != NULL)
            {
                strcat(datum, words);
                words = strtok(NULL, "\r\n");
            }
        }  
        else if (strncmp (keyword, "SPHEROID_NAME",13) == 0) 
        {
            strcpy(spheroid, "");
            words = strtok(&line[13], " \r\n");
            while (words != NULL)
            {
                strcat(spheroid, words);
                words = strtok(NULL, "\r\n");
            }
        }

        /* Projection Parameters */
        else if (strncmp (keyword, "SPHEROID_SEMI_MAJ_AXIS",22) == 0) 
        {
            words = strtok(&line[22], " \r\n");
            strcpy(semi_major, words);
        }
        else if (strncmp (keyword, "SPHEROID_SEMI_MIN_AXIS",22) == 0) 
        {
            words = strtok(&line[22], " \r\n");
            strcpy(semi_minor, words);
        }
        else if (strncmp (keyword, "PROJ_STD_PARALLEL_1",19) == 0)
        {
            words = strtok(&line[19], " \r\n");
            strcpy(parallel1, words);
        }
        else if (strncmp (keyword, "PROJ_STD_PARALLEL_2",19) == 0)
        {
            words = strtok(&line[19], " \r\n");
            strcpy(parallel2, words);
        }
        else if (strncmp (keyword, "PROJ_LONG_ORIGIN",16) == 0)
        {
            words = strtok(&line[16], " \r\n");
            strcpy(longitude, words);
        }
        else if (strncmp (keyword, "PROJ_LAT_ORIGIN",15) == 0)
        {
            words = strtok(&line[15], " \r\n");
            strcpy(latitude, words);
        }
        else if (strncmp (keyword, "PROJ_X_ORIGIN",13) == 0)
        {
            words = strtok(&line[13], " \r\n");
            strcpy(false_easting, words);
        }
        else if (strncmp (keyword, "PROJ_Y_ORIGIN",13) == 0)
        {
            words = strtok(&line[13], " \r\n");
            strcpy(false_northing, words);
        }
        else if (strncmp (keyword, "PROJ_SPHERE_RADIUS",18) == 0)
        {
            words = strtok(&line[18], " \r\n");
            strcpy(sphere_radius, words);
        }
        else if (strncmp (keyword, "PROJ_SCALE_FACTOR", 17) == 0)
        {
            words = strtok(&line[17], " \r\n");
            strcpy(scale_factor, words);
        }

        /* Line and sample relative to master image */
        else if (strncmp (keyword, "IMAGE_UPPER_LEFT_J",20) == 0)
        {
            words = strtok(&line[20], " \r\n");
            strcpy(master_line, words);
        }
        else if (strncmp (keyword, "IMAGE_UPPER_LEFT_I",20) == 0)
        {
            words = strtok(&line[20], " \r\n");
            strcpy(master_sample, words);
        }

        /* Catrographic Corner Coordinates */
        else if (strncmp (keyword, "CARTO_UPPER_LEFT_X",18) == 0)
        {
            words = strtok(&line[18], " \r\n");
            strcpy(upper_left_x, words);
        }
        else if (strncmp (keyword, "CARTO_UPPER_LEFT_Y",18) == 0)
        {
            words = strtok(&line[18], " \r\n");
            strcpy(upper_left_y, words);
        }
        else if (strncmp (keyword, "CARTO_UPPER_RIGHT_X",19) == 0)
        {
            words = strtok(&line[19], " \r\n");
            strcpy(upper_right_x, words);
        }
        else if (strncmp (keyword, "CARTO_UPPER_RIGHT_Y",19) == 0)
        {
            words = strtok(&line[19], " \r\n");
            strcpy(upper_right_y, words);
        }
        else if (strncmp (keyword, "CARTO_LOWER_RIGHT_X",19) == 0)
        {
            words = strtok(&line[19], " \r\n");
            strcpy(lower_right_x, words);
        }
        else if (strncmp (keyword, "CARTO_LOWER_RIGHT_Y",19) == 0)
        {
            words = strtok(&line[19], " \r\n");
            strcpy(lower_right_y, words);
        }
        else if (strncmp (keyword, "CARTO_LOWER_LEFT_X",18) == 0)
        {
            words = strtok(&line[18], " \r\n");
            strcpy(lower_left_x, words);
        }
        else if (strncmp (keyword, "CARTO_LOWER_LEFT_Y",18) == 0)
        {
            words = strtok(&line[18], " \r\n");
            strcpy(lower_left_y, words);
        }
    }
    fclose(file_ptr);

    /* Get the DDR
       -------------*/
    if(E_FAIL == c_getddr(hosout,&ddr))
    {
        c_lasmsg(NONFATAL,"spothdf2las-fatal","Fatal error encountered");
        return (E_FAIL);
    }

    /* Set the projection and get the projection parameters
       ------------------------------------------------------*/
    if (strcmp(projection, "") == 0)
    {
        c_lasmsg(NONFATAL,"spothdf2las-warn",
                "Projection does not exist in logfile");
        proj_code = GEO;
    }

    /*Plate Carree (or geographic) contains no projection parameters. 
      Set default.*/
    else if (strcasecmp(projection, "unprojected") == 0) /* plate carree */
    {
        proj_code = GEO;
        ddr.valid[0] = VALID;   /* set projection to valid */
        ddr.valid[3] = VALID;   /* set projection parameters to valid */
    }

    else if (strcasecmp(projection, "albers") == 0)
    {
        proj_code = ALBERS;
        albers(&ddr, semi_major, semi_minor, parallel1, parallel2, longitude,
                latitude, false_easting, false_northing, proj_kilometers);
        ddr.valid[0] = VALID;
    }

    else if (strcasecmp(projection, "polarstereo") == 0)
    {
        proj_code = PS;
        polar_stereo(&ddr, semi_major, semi_minor, longitude, latitude, 
                false_easting, false_northing, proj_kilometers);
        ddr.valid[0] = VALID;
    }

    else if (strcasecmp(projection, "conic") == 0)
    {
        proj_code = EQUIDC;
        conic(&ddr, semi_major, semi_minor, parallel1, parallel2, longitude, 
                latitude, false_easting, false_northing, proj_kilometers);
        ddr.valid[0] = VALID;
    }

    /* goode homolosine does not contain projection parameters */
    else if (strcasecmp(projection, "goodehomolosine") == 0)
    {
        proj_code = GOOD;
        ddr.valid[0] = VALID;
    }

    else
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn", 
                "SPOTHDF2LAS does not support projection %s", projection);
        proj_code = GEO;
    }

    ddr.proj_code = proj_code;

    /* Set datum or spheroid
       -----------------------*/
    if (strcasecmp(projection, "unprojected")==0)/* plate carree has no datum */
    {
        ddr.datum_code = 0;
        ddr.valid[2] = VALID;
    }

    else if (proj_code == GOOD) /* goode's uses a spheroid instead of a datum*/
    {
        if (E_FAIL == get_spheroid(&ddr, spheroid, sphere_radius))
        {
            c_lasmsg(FATAL, "spothdf2las-fatal", "Fatal error encountered");
        }
    }
    else
    {
        if (E_FAIL == get_datum(&ddr, datum))
        {
            c_lasmsg(FATAL, "spothdf2las-fatal", "Fatal error encountered");
        }
    }


    /* Set zone 
       ----------*/
    /*since a zone doesn't exist for the supported projections, default to 62*/
    ddr.zone_code = 62;
    ddr.valid[1] = VALID;    /* set zone to valid */

    /* set projection units. If units equals kilometers, change to meters
       --------------------------------------------------------------------*/
    if (strcmp(projection_units, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn", 
                "Projection units do not exist in logfile");
    }

    if (proj_kilometers == 1)
    {
        strcpy(ddr.proj_units,"meters");
        ddr.valid[4] = VALID;   /* set projection units to valid */
    }
    else
    {
        if (c_string2unit(projection_units) == E_FAIL)
        {
            c_lasmsg(NONFATAL, "spothdf2las-warn", 
                    "Projection units %s are not supported in LAS",
                     projection_units);
            c_lasmsg(NONFATAL, "spothdf2las-warn",
                    "Setting projection units to invalid");
            strcpy(ddr.proj_units," ");
            ddr.valid[4] = INVAL;
        }
        else
        {
            strcpy(ddr.proj_units,projection_units);
            ddr.valid[4] = VALID;   /* set projection units to valid */
        }
    }

    /* line and sample relative to master image
       -----------------------------------------*/
    if (strcmp(master_line, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn", 
                "Master line does not exist in logfile");
    }
    else
        ddr.master_line = atol(master_line);


    if (strcmp(master_sample, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn", 
                "Master sample does not exist in logfile");
    }
    else
        ddr.master_sample = atol(master_sample);


    /* line and sample increments
       ----------------------------*/
    ddr.line_inc = 1;
    ddr.sample_inc = 1;
    ddr.valid[7] = VALID;     /* set line and sample increments to valid */

    /* Set corner coordinates to valid
       --------------------------------*/
    ddr.valid[6] = VALID;

    /* Copy corner coordinates. If they do not exist then set corner coordinates
       to invalid.  If projection kilometers flag is 1, convert to meters.
       ----------------------------------------------------------------------*/
    if (strcmp(upper_left_y, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Upper left corner y does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else
        ddr.upleft[0] = atof(upper_left_y);

    if (strcmp(upper_left_x, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Upper left corner x does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else
        ddr.upleft[1] = atof(upper_left_x);

    if (strcmp(upper_right_y, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Upper right corner y does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else
        ddr.upright[0] = atof(upper_right_y);

    if (strcmp(upper_right_x, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Upper right x corner does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else
        ddr.upright[1] = atof(upper_right_x);


    if (strcmp(lower_left_y, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Lower left y corner does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else
        ddr.loleft[0] = atof(lower_left_y);

    if (strcmp(lower_left_x, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Lower left x corner does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else 
        ddr.loleft[1] = atof(lower_left_x);

    if (strcmp(lower_right_y, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Lower right y corner does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else
        ddr.loright[0] = atof(lower_right_y);

    if (strcmp(lower_right_x, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Lower right x corner does not exist in logfile");
        ddr.valid[6] = INVAL;
    }
    else
        ddr.loright[1] = atof(lower_right_x);

    if (proj_kilometers == 1)
    {
        ddr.upleft[0] = ddr.upleft[0] * 1000;
        ddr.upleft[1] = ddr.upleft[1] * 1000;
        ddr.upright[0]= ddr.upright[0] * 1000;
        ddr.upright[1]= ddr.upright[1] * 1000;
        ddr.loleft[0]= ddr.loleft[0] * 1000;
        ddr.loleft[1]= ddr.loleft[1] * 1000;
        ddr.loright[0]= ddr.loright[0] * 1000;
        ddr.loright[1]= ddr.loright[1] * 1000;
    }


    /* Set pixel distance
       -------------------*/
    if (strcmp(resolution, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-warn",
                "Resolution does not exist in logfile");
    }
    else
    {
        ddr.pdist_y = atof(resolution);
        ddr.pdist_x = atof(resolution);
        ddr.valid[5] = VALID;    /* set resolution to valid */ 
    }

    if (proj_kilometers == 1)
    {
        ddr.pdist_y = ddr.pdist_y * 1000;
        ddr.pdist_x = ddr.pdist_x * 1000;
    }

    /* Get the BDDR
       --------------*/
    for (i = 1; i < nbands+1; i++)
    {
        if(E_FAIL == c_getbdr(hosout,&bddr[i-1],&i))
        {
            c_lasmsg(NONFATAL,"spothdf2las-getbdr","Error retrieving the BDR"); 
            return (E_FAIL);
        }
    }

    /* Write to DDR
       --------------*/
    if (E_FAIL == c_putddr(hosout,&ddr))
    {
        c_lasmsg(NONFATAL,"spothdf2las-fatal","Fatal error encountered"); 
        return (E_FAIL);
    }

    /* Write to BDDR
       ---------------*/
    bptr = bddr;
    for (i = 0; i < nbands; i++)
    {
        strcpy(bptr->source, hdf_name[i]);
        if (E_FAIL == c_putbdr(hosout,&bddr[i]))
        {
            c_lasmsg(NONFATAL,"spothdf2las-fatal","Fatal error encountered");
            return (E_FAIL);
        }
        bptr++;
    }

    return(E_SUCC);
}
