#include <stdlib.h>
#include "spread.h"
#include "util.h"
#include "protos.h"

void distance
(
    long target,		/* I: target class for distance search        */
    long radius, 		/* I: radius of search                        */
    long bckgnd,		/* I: background value on output image        */
    long beg_angl, 		/* I: beginning angle for restricted search   */
    long end_angl,		/* I: ending angle for restricted search      */
    long num_inimg,		/* I: total number of images                  */
    long num_img_band[][MAXBND+1],   /* I: band numbers for the input image   */
    long tot_inbands,		/* I: total number of bands                   */
    long bands_per_img[]	/* I: number of bands in each image           */
)

/*
*****************************************************************************

NAME: DISTANCE

PURPOSE: Finds the euclidean distance from each pixel in an image
         to the nearest pixel in a user-specified target class

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0
   4.0 	       20-Jul-89   K. Kunska            fixed a bug in conversion of
                                                the FORTRAN goto

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize local variables.
   Call SETUP to generate search offset and distance tables.
   Calculate dimensions of the work buffer.
   Allocate dynamic memory for the work buffer.
   Initialize work buffer to -1s.
   Initialize and enable the processing message utility.
   Copy initial lines of input image to the work buffer.
   For all lines
      Read a new line into the circular work array.
      If the 'new line' is below the border of the input image, read -1's 
      into the work array
      Perform a 'spiral' (i.e. concentrically increasing) search from each 
      pixel to the nearest pixel in the target class.
      If no match is found, set output pixel value to the background value.
      Determine the starting address for the next sample's search.
      If a match has been found, set the output pixel value to the distance 
      value.
      Determine the starting address for the next sample's search.
   Endfor
   Free the dynamic memory.

*****************************************************************************
*/

{

long	cont_flag,		/* continue flag for processing               */
	cur_band,		/* current band                               */
	dist[862],		/* distances corresponding to x,y values      */
	from_type,		/* data type of the source buffer             */
	i, 			/* loop counter and array index               */
	ifill, 			/* index for initialization                   */
	img,			/* current image                              */
	in_band,		/* input band for reading                     */
	in_line,		/* input line for reading                     */
	isave, 			/* saves the current radius incrememt         */
	isbscr, 		/* starting address for next search           */
	isrch,			/* radius increment counter                   */
	j, m,			/* loop counter and array index               */
	jfill, 			/* index for initialization                   */
	line, 			/* line loop counter                          */
	linit,			/* initial line counter                       */
	match_flag,		/* flag for target matching                   */
	mod_temp,		/* temporary variable for long modulus args.  */
	msg_flag = 1,		/* message type flag                          */
	newadd, 		/* line address in circular work array        */
	newlin, 		/* new line to be processed                   */
	nfin[17], 		/* final indices for radial scans             */
	nstart[17], 		/* starting indices for radial scans          */
	num_lines,		/* number of lines in the input image         */
	num_pixels,		/* number of pixels in the input image        */
	out_band,		/* output band for writing                    */
	out_line,		/* output line for writing                    */
	sample, 		/* pixel counter                              */
	to_type,		/* data type of the destination buffer        */
	trans, 			/* index into the work buffer                 */
	*work[34], 		/* pointers to work rows                      */
	*workbuf,		/* pointer to work buffer area                */
	wrklin, 		/* vertical dimension of work array           */
	wrkpix,			/* horizontal dimension of work array         */
	work_size,		/* size of the working buffer for dyn. alloc. */
	xoff[862], 		/* consecutive x offsets for search           */
	yoff[862];		/* consecutive y offsets for search           */

long    one = 1;
long    fatal_err = -1;                /* fatal error return status */

/*
Initialize local variables.
----------------------------
*/
num_lines = window[0][NL];
num_pixels = window[0][NS];
out_band = 0;
from_type = ELONG;
to_type = EBYTE;

for (i = 0; i < 17; i++)
   {
   nstart[i] = 0;
   nfin[i] = 0;
   }

for (i = 0; i < 862; i++)
   {
   dist[i] = 0;
   xoff[i] = 0;
   yoff[i] = 0;
   } 

/*
Set up search offset and distance tables
----------------------------------------
*/
setup (radius, beg_angl, end_angl, nstart, nfin, xoff, yoff, dist);

/*
Calculate dimensions of the work buffer.
-------------------------------------------
*/
wrklin = 2 * radius + 1;
wrkpix = num_pixels * 2 * radius;

/* 
Allocate dynamic memory for the work buffer.
--------------------------------------------
*/
work_size = (long)(wrkpix * sizeof(long) * 34);
if ((workbuf = (long *) malloc (work_size)) == NULL)
   c_errmsg ("Cannot allocate dynamic memory", "spread-alloc", &fatal_err);
for (i = 0, m = 0; i < 34; i++, m += wrkpix)
   work[i] = &workbuf[m];


/*
Initialize work buffer to -1s.
--------------------------------
*/

for (ifill = 1; ifill <= wrklin; ifill++)
   for (jfill = 1; jfill <= wrkpix; jfill++)
      work[ifill][jfill] = -1;

/* 
Initialize and enable the processing message utility.
-----------------------------------------------------
*/
c_inittm ("SPREAD", "Processed", "line", "band", " ", &num_lines, 
           &tot_inbands, &msgtim, &msg_flag);

curcat = 0;
for (img = 0; img < num_inimg; img++)
   {
   for (cur_band = 0; cur_band < bands_per_img[img]; cur_band++)
      {
      curent = 1;
      curcat++;
      out_band++;
      out_line = 1;
      in_line = 1;
      in_band = num_img_band[img][cur_band];
/*
Copy initial lines of input image to the work buffer.
-----------------------------------------------------
*/
      for (linit = 1; linit <= radius + 1; linit++)
         {
         c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line, 
                   (unsigned char *)inbuf, &one);
         in_line++;
         for (trans = 1; trans <= num_pixels; trans++)
            work[linit + radius][trans + radius] = inbuf[trans-1];
         }

        c_startm();  

/*
Loop on lines
----------------
*/
      for (line = 1; line <= num_lines; line++)
         {
         curent++;
         cont_flag = match_flag = FALSE;

/*
Read a new line into the circular work array
If the 'new line' is below the border of the input image,
read -1's into the work array
-----------------------------------------------------------
*/
         if (line == 1)
            cont_flag = TRUE;
         else
            {
            newlin = line + radius;
            newadd = ((line - 2) % wrklin) + 1;
            if (newlin > num_lines)
               for (ifill = 1; ifill <= wrkpix; ifill++)
                  work[newadd][ifill] = -1;
            else
               {
               c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line, 
                         (unsigned char *)inbuf, &one);
               in_line++;
               for (trans = 1; trans <= num_pixels; trans++)
                  work[newadd][trans + radius] = inbuf[trans-1];
               cont_flag = TRUE;   /*   go to 50; */
               }
            }

/*
Perform a 'spiral' (i.e. concentrically increasing) search
from each pixel to the nearest pixel in the target class.
----------------------------------------------------------
*/
         if (cont_flag == TRUE);
            {
            isbscr = 1;
    
            for (sample = 1; sample <= num_pixels; sample++)
               {
               for (isrch = nstart[isbscr]; isrch <= nfin[radius]; isrch++)
                  {
                  j = sample + radius + xoff[isrch];
                  mod_temp = line + radius - 1 - yoff[isrch];
                  i = (mod_temp % wrklin) + 1;
                  isave = isrch;
                  if (work[i][j] == target)
                     {
                      match_flag = TRUE;
                      break;
                      }
                  }

/*
If no match is found, set output pixel value to the background value.
Determine the starting address for the next sample's search.
--------------------------------------------------------------------
*/
                  if (match_flag == FALSE)
                     {
                     outbuf[sample-1] = bckgnd;
                     if (beg_angl == 0  &&  end_angl == 360) 
                        isbscr = radius;
                     }
/*
If a match has been found, set the output pixel value to the distance value.
Determine the starting address for the next sample's search.
-------------------------------------------------------------------------------
*/
                   else
                      {
                      outbuf[sample-1] = dist[isave];
                      if (dist[isave] == 0  ||  dist[isave] == 1)
                         isbscr = 1;
                      else if (beg_angl == 0  &&  end_angl == 360) 
                         isbscr = dist[isave] - 1;
                      match_flag = FALSE;
                      }
                   }   
                c_ewrite ((struct FDESC **)&fdescout, &out_band, &out_line, 
                           (unsigned char *)outbuf, &one);
                out_line++;
                } /* cont_flag */
             } /* line loop */
          } /* band  loop */
      } /* img loop */      
c_stoptm();  

/*
Free the dynamic memory.
------------------------
*/
free(workbuf);

}

