#include "spread.h"
#include "typlim.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char hos_in[][CMLEN], 	/* O: input image host name                  */
    char hos_out[][CMLEN], 	/* O: output image host name                 */
    long *target,		/* O: target class for distance search       */
    long *radius, 		/* O: radius of search                       */
    long mask_target[],		/* O: pixel target values for mask subcommand*/
    long mask_radius[],		/* O: buffer radius for mask subcommand      */
    long *bckgnd,		/* O: background value on output image       */
    long *beg_angl, 		/* O: beginning angle for restricted search  */
    long *end_angl, 		/* O: ending angle for restricted search     */
    long *mask, 		/* O: mask value                             */
    long quad_num[],		/* O: quadrant numbers                       */
    long *cmdopt,		/* O: subcommand option                      */
    long *dtype, 		/* O: data type                              */
    long *bands_per_img,	/* O: number of bands per image              */
    long (*img_flag)[2],	/* O: specified image/band options           */
    long img_limit,		/* O: maximum number of images possible      */
    long (*num_img_band)[MAXBND+1],/* O: stores band numbers for each image  */
    long *num_inimg,		/* O: number of input images                 */
    long *same_wnd,		/* O: same window size? 1=Y, 0=N             */
    long *tot_inband		/* O: total number of bands                  */
)

/*
*****************************************************************************

NAME: GETPAR

PURPOSE:  Retrives user supplied parameter values and performs appropriat
          validity tests.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize command array and limit array.
   Call INLAS to initialize TAE.
   Get the subcommand and the associated parameters.
   Get the parameters common to both subcommands.
   Get the input image file name.
   Get the output image file name.
   Get the LAS global parameters.
   Compute the ending angle for the restricted search.
   Check the image data type. 
   Print an error message if the image data type is a REAL.
   Check the validity of PIXVAL values.
   Test the dimensions of mask_radius and mask_target.

*****************************************************************************
*/

{
char	command[2][5],		/* array of subcommands                       */
	mnmx[4],		/* minmax option                              */
	subcmd[5],		/* user supplied subcommand name              */
	tae_in[MAXIMG][CMLEN], 	/* input image tae name                       */
	tae_out[MAXIMG][CMLEN];	/* output image tae name                      */

long	found,			/* flag that a subcommand has been found      */
	i, j,			/* loop counter and array index               */
	limit[2][3],		/* data type boundary limits                  */
	par_type,		/* parameter type                             */
	stat,			/* return status value                        */
	std_out;		/* logical unit for standard output           */

struct	VARIABLE   *var;	/* pointer to a variable in the TAE block     */
long    zero = 0;
long    one = 1;
long    four = 4;
long    fifty = 50;
long    fatal_err = -1;                /* fatal error return status */
long    rad_cnt;                       /* number of radius values specified  */
                                       /* by the user for -MASK              */


/* 
Initialize command array and limit array.
------------------------------------------
*/
strcpy (command[0], "DIST");
strcpy (command[1], "MASK");

limit[0][0] = MINUTINY;
limit[0][1] = MINWORD;
limit[0][2] = MINLONG;
limit[1][0] = MAXUTINY;
limit[1][1] = MAXWORD;
limit[1][2] = MAXLONG;

/*
Initialize TAE.
----------------
*/
c_inlas (&block, &std_out); 
par_type = ELONG;

/* 
Get the subcommand and the associated parameters.
-------------------------------------------------
*/
var = p_find (&block, "_SUBCMD");
strcpy (subcmd, SVAL(*var, 0));
found = FALSE;
for (i = 0; found != TRUE; i++)
   {
   if (strcmp(subcmd, command[i]) == 0)
      {
      *cmdopt = i + 1;
      found = TRUE;
      if (*cmdopt == 1)
         {
         c_getnum (&block, "PIXVAL", &one, &par_type, (unsigned char *)target); 
         c_getnum (&block, "SRCHRAD", &one, &par_type,(unsigned char *)radius); 
         c_getnum (&block, "BEGANG", &one, &par_type,(unsigned char *)beg_angl);
         c_getnum (&block, "NDEGREES", &one, &par_type,
                   (unsigned char *) end_angl);
         }
      else if (*cmdopt == 2)
         {
         tar_cnt = c_getnum (&block, "PIXVAL", &fifty, &par_type, 
                             (unsigned char *)mask_target);  
         rad_cnt = c_getnum (&block, "SRCHRAD", &fifty, &par_type, 
                             (unsigned char *)mask_radius);
         qua_cnt = c_getnum (&block, "QUADNUM", &four, &par_type,
                             (unsigned char *)quad_num);
         c_getnum (&block, "MASKVAL", &one, &par_type, (unsigned char *)mask);
         }
      }
   }

/* 
Get the parameters common to both subcommands.
----------------------------------------------
*/
c_getnum (&block, "BACKGRND", &one, &par_type, (unsigned char *)bckgnd);  

/*
Get the input image file name.
------------------------------
*/
stat = c_getin (&block, "IN", &zero, tae_in, hos_in, num_inimg, window, 
                num_img_band, bands_per_img, tot_inband, &img_limit, same_wnd, 
                img_flag);

if (stat != E_SUCC)
   c_errmsg ("Cannot access input file names", "spread-getin",
              &fatal_err);
if (same_wnd == 0)
   c_errmsg ("Image windows differ in size", "spread-window", &fatal_err);

/*
Get the output image file name.
-------------------------------
*/
stat = c_getout (&block, "OUT", tae_out, hos_out);  

if (stat != E_SUCC)
   c_errmsg ("Cannot access output file name", "spread-getin", &fatal_err);

/*
Get the LAS global parameters.
------------------------------
*/
c_getnum (&block, "$MSGTIM", &one, &par_type, (unsigned char *)&msgtim);  

var = p_find(&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;

/*
Compute the ending angle for the restricted search.
---------------------------------------------------
*/
if (*cmdopt == 1)
   *end_angl = *beg_angl + *end_angl;

/*
Check the image data type. 
Print an error message if the image data type is a REAL.
---------------------------------------------------------
*/
for (i = 0; i < *num_inimg; i++)
   {
   c_getddr (hos_in[i], &ddr);  
   *dtype = ddr.dtype;
   if (*dtype == 4) 
      c_errmsg ("Type of the input image must not be REAL", 
                 "spread-dtype", &fatal_err);
   }

/*
Check the validity of PIXVAL values.
--------------------------------------
*/
if (*cmdopt == 1)
   {
   if ((*target < limit[0][*dtype-1]) || (*target > limit[1][*dtype-1]))
      c_errmsg ("Pixval out of range for the input data type", "spread-pxval",
              &fatal_err);
   }
else
   {
   for (j = 0; j < tar_cnt; j++)
      {
      if ((mask_target[j] < limit[0][*dtype-1]) || 
          (mask_target[j] > limit[1][*dtype-1])) 
         c_errmsg ("Pixval out of range for input data type", "spread-pxval",
                   &fatal_err);
      }
   }

/*
Test the dimensions of mask_radius and mask_target.
---------------------------------------------------
*/
if (*cmdopt == 2)
   {
   if (tar_cnt != rad_cnt) 
      {
      if (rad_cnt  ==  1) 
         {
         for (i = 0; i < tar_cnt; i++)
            mask_radius[i] = mask_radius[0];
         }
      else
         c_errmsg ("Number of pixel values and radius values does not match",
                   "spread-match", &fatal_err);
      }      
   }

}

