#include "spread.h"
#include "protos.h"

void runend
(
    long rsize,			/* I: max dimension to runend data array      */
    long line, 		        /* I:current line being accessed during update*/
    long left,			/* I:pointer to the line boundary on left side*/
    long right,			/* I: pointer the line boundary on right side */
    long *end, 			/* O: position in line where code[] ends      */
    long *code,			/* O: codes for output pixel values           */
    long *point,		/* I/O: pointers to next elements in list     */
    long *nextp, 		/* I/O: next pointer                          */
    long nsmp,			/* I: number of samples from the input image  */
    long *root 			/* I/O: starting value in code and end for    */
)
/*
*****************************************************************************

NAME: RUNEND

PURPOSE: Creates an internal array to maintain run-end encoded form of a 
         binary map.  

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize local variables.
   Get first new record and initialize pointer arrays.
   Get the next record.
   Point to it from prior record (this starts the record).
   Set the end, code, and prior values.
   If the right boundary < number of samples 
      get the new record and reset
   Set zeros at end of record.
   Reset prior = null to indicate first pointer from root has 
   not been respecified.
   Loop to repeat processing while either old or new has runs remaining.
      If codes are the same (both 0)
         keep the minimum end value.  
         If end values are the same
            keep the old, and cycle and delete the new.  
         If end values are not the same
            keep the one with the minimum value.
      If codes are the same (both 1)
         keep the maximum end value.  
      If the prior run is null
         then keep the old or new end value, whichever is highest.  
      Otherwise, keep the one with the maximum value.
      If codes are not the same.  
         If the new code value is 1, then keep the new end value and delete 
         the old end value only if it is greater than the new end value.
         If the new code value is 0, then keep the old end value and delete 
         the new end value only if it is not greater than the old end value.

*****************************************************************************
*/

{

long	base, 			/* starting line for the encoding             */
	new, 			/* overlay line containing 1's where the new  */
                                /* buffer zone is required                    */
	newrec, 		/* points to the new current record           */
	next,			/* points to the next record                  */
	null, 			/* null value marking the end of array        */
	old, 			/* image of the line before processing        */
	prior;                  /* points to the prior record                 */

/*
Initialize local variables.
---------------------------
*/
null = 0;
base = line;
old = root[base];
prior = null;

/*
Get first new record and initialize pointer arrays.
---------------------------------------------------
*/
if (left  !=  1) 
   {
   get_rec (rsize, point, nextp, &newrec);
   new = newrec;
   end[newrec] = left - 1;
   code[newrec] = 0;
   prior = newrec;
   } 

/*
Get the next record.
---------------------
*/
get_rec (rsize, point, nextp, &newrec);

/*
Point to it from prior record (this starts the record).
-----------------------------------------------------------
*/
if (prior  ==  null) 
   {
   new = newrec;
   }
else
   {
   point[prior] = newrec;
   } 

/*
Set the end, code, and prior values.
--------------------------------------
*/
end[newrec] = right;
code[newrec] = 1;
prior = newrec;

/*
If the right boundary < number of samples, get the new record and reset
set zeros at end of record.
-----------------------------------------------------------------------
*/
if (right < nsmp) 
   {
   get_rec (rsize, point, nextp, &newrec);
   point[prior] = newrec;
   end[newrec] = nsmp;
   code[newrec] = 0;
   point[newrec] = null;
   }
else
   {

/*
No zeros at end of record.
---------------------------
*/
   point[prior] = null;
   } 

/*
Reset prior = null to indicate first pointer from root has 
not been respecified.
---------------------------------------------------------------
*/
prior = null;

/*
Loop to repeat processing while either old or new has runs remaining.

Rules:
   case i:     If both codes are zero, keep minimum end value.
   case ii:    If one code is zero, and one code is 1, keep the
               run with code = 1.
   case iii:   If both codes are 1, keep the one with the maximum end.
               This may involve replacing a previously kept value.

Supplemental rules:
   1.  In the case of a tie in end values, arbitrarily keep old.
   2.  Cycle the run with the minimum end value.
-----------------------------------------------------------------------
*/

while ((new  !=  null)  &&  (old  !=  null)) 
   {
   if (code[old]  ==  code[new]) 
      {
      if (code[old]  ==  0) 
         {

/*
If codes are the same (both 0) - keep the minimum end value.  
If end values are the same, keep the old, and cycle and delete the new.  
If end values are not the same, keep the one with the minimum value.
---------------------------------------------------------------------------
*/
         if (end[old]  ==  end[new] ) 
            {
            next = point[new];
            delrun (new, point, nextp);
            new = next;
            keep (old, point, root, 
                  &prior, base);
            old = point[old];
            prior = old;
            }
         else
            {
            if (end[old]  <  end[new]) 
               {
               keep (old, point, root, 
                     &prior, base);
               old = point[old];
               }
            else
               {
               keep (new, point, root, 
                     &prior, base);
               new = point[new]; 
               } /* if */
            } /* if */
         }
      else
         {

/*
If codes are the same (both 1) - keep the maximum end value.  
If the prior run is null, then keep the old or new end value, whichever 
is highest.  
Otherwise, keep the one with the maximum value.
--------------------------------------------------------------------------
*/
         if (prior  ==  null) 
            {
            if (end[old]  <  end[new]) 
               {
               root[base] = new;
               next = point[old];
               delrun (old, point, nextp);
               old = next;
               }
            else
               {
               root[base] = old;
               next = point[new];
               delrun (new, point, nextp);
               new = next;
               } 
            }
         else
            {
            if (end[old]  ==  end[new] ) 
               {
               if (old  ==  prior) 
                  {
                  next = point[new];
                  delrun (new, point, nextp);
                  new = next;
                  }
               else
                  {
                  next = point[old];
                  delrun (old, point, nextp);
                  old = next;
                  } 
               }
            else
               {
               if (end[old]  <  end[new]) 
                  {
                  next = point[old];
                  if (new  ==  prior) 
                     delrun (old, point, nextp);        
                  else
                     {
                     end[prior] = end[new];
                     point[prior] = point[new];
                     delrun (new, point, nextp);
                     new = prior;
                     } 
                  old = next;
                  }
               else
                  {
                  next = point[new];
                  if (old  ==  prior) 
                     delrun (new, point, nextp);        
                  else
                     {
                     end[prior] = end[old];
                     point[prior] = point[old];
                     delrun (old, point, nextp);
                     old = prior;
                     } 
                  new = next;
                  } /* if */
               } /* if */
            } /* if */
         } /* if */
      }
   else
      {

/*
Codes are not the same.  
If the new code value is 1, then keep the new end value and delete 
the old end value only if it is greater than the new end value.
If the new code value is 0, then keep the old end value and delete 
the new end value only if it is not greater than the old end value.
--------------------------------------------------------------------
*/
      if (code[new]  ==  1) 
         {
         if (end[old]  <=  end[new]) 
            {
            next = point[old];
            delrun (old, point, nextp);
            old = next;
            keep (new, point, root, 
                  &prior, base);
            }
         else
            {
            keep (new, point, root, 
                 &prior, base);
            new = point[new]; 
            } /* if */
         }
      else
         {
         if (end[new]  <=  end[old]) 
            {
            next = point[new];
            delrun (new, point, nextp);
            new = next;
            keep (old, point, root, 
                  &prior, base);
            }
         else
            {
            keep (old, point, root, 
                  &prior, base);
            old = point[old];
            } /* if */
         } /* if */
      } /* if */
   } /* while */

}
