#include "spread.h"
#include "protos.h"

void setup
(
    long radius, 		/* I: radius of search                        */
    long beg_angl, 		/* I: beginning angle for restricted search   */
    long end_angl, 		/* I: ending angle for restricted search      */
    long *nstart,             	/* O: starting indices for radial scans       */
    long *nfin, 		/* O: final indices for radial scans          */
    long *xoff,			/* O: consecutive x offsets for search        */
    long *yoff,			/* O: consecutive y offsets for search        */
    long *dist 			/* O: distances corresponding to x,y values   */
)

{

/*
*****************************************************************************

NAME: SETUP

PURPOSE: Sets up search offsets and distance tables containig the euclidean
         distances from a target pixel to surrounding pixels.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize local variables
   Loop through concentrically increasing radii
      For each radius increment, 
         compute the x and y offsets for the specified angle.
      Loop through user-specified angular range 
         computing the distances that are appropriate for each x/y offset.
      Check whether the current angle value generates new offsets.
      Otherwize, save the initial offsets.
      Store the computed  offsets and associated distances.

*****************************************************************************
*/

double	d,			/* computed distance in double precision      */
	dradius,		/* current radius in double precision         */
	dzero,			/* zero value in double precision             */
	hdim,			/* horizontal distance                        */
	r, 			/* radius increment in double precision       */
	tempnx,			/* x-offset value squared                     */
	tempny,			/* y-offset value squared                     */
	theta, 			/* current theta increment                    */
	thrad, 			/* theta expressed in radians                 */
	thsav, 			/* initial theta value for radial scans       */
	vdim,			/* verical distance                           */
	x, 			/* x-offset in double precision               */
	y;			/* y-offset in double precision               */

long	i, 		
	ifirst, 		/* looping flag                               */
	ir, 			/* loop counter                               */
	ix, 
	ktheta, 		/* angle loop counter                         */
	n, 
	nd,			/* distance in integer form                   */
	ndeg, 			/* number of degrees for angular search       */
	nx, 			/* x-offset in integer form                   */
	ny, 			/* y-offset in integer form                   */
	xlast, 			/* old x-offset                               */
	xsav, 			/* initial x value for radial scans           */
	ylast,			/* old y-offset                               */
	ysav;			/* initial y value for radial scans           */

/*
Initialize local variables
--------------------------
*/
for (i = 1; i <= radius; i++)
   {
   nstart[i] = 0;
   nfin[i] = 0;
   }

xoff[1] = 0;
yoff[1] = 0;
dist[1] = 0;
dzero = 0.0;
xlast = 0;
ylast = 0;
n = 2;
ix = 1;
ndeg = end_angl - beg_angl;
thsav = 270.0 + (360 - end_angl);

/*
Loop through concentrically increasing radii.
For each radius increment, compute the x and y offsets for the 
specified angle.
------------------------------------------------------------------
*/
for (ir = 1; ir <= radius; ir++)
   {
   dradius = (double) ir;
   theta = thsav;
   hdim = dradius * cos(thsav / 57.29578);
   if (hdim >= dzero)
      xsav = (long)(hdim + 0.5);
   else
      xsav = (long)(hdim - 0.5);

   vdim = dradius * sin(thsav / 57.29578);
   if (vdim >= dzero)
      ysav = (long)(vdim + 0.5);
   else
      ysav = (long)(vdim - 0.5);

   nstart[ir] = ix;
   ifirst = 1;

/*
Loop through user-specified angular range computing the distances
that are appropriate for x/y each offset.
-------------------------------------------------------------------
*/
   for (ktheta = 1; ktheta <= ndeg; ktheta++)
      {
      thrad = theta / 57.29578;
      r = dradius;
      x = r * cos(thrad);
      y = r * sin(thrad);
      if (x >= dzero)
         nx = (long)(x + 0.5);
      else
         nx = (long)(x - 0.5);

      if (y >= dzero)
         ny = (long)(y + 0.5);
      else
         ny = (long)(y - 0.5);

      tempnx = (double) nx * nx;
      tempny = (double) ny * ny;
      d = sqrt(tempnx + tempny);
      if (d >= dzero)
         nd = (long)(d + 0.5);
      else
         nd = (long)(d - 0.5);

/*
Check whether the current theta value generates new offsets
-----------------------------------------------------------
*/
      if ((nx == xlast  &&  ny == ylast)  ||  nd != ir)
         theta = theta + 1.0;

/*
Make sure initial offsets are not overwritten
---------------------------------------------
*/
      else if (nx == xsav  &&  ny == ysav  &&  ifirst == 0) 
         theta = theta + 1.0;
      else 
/*
Store the computed  offsets and associated distances
------------------------------------------------------
*/
         {
         ifirst = 0;
         xoff[n] = nx;
         yoff[n] = ny;
         dist[n] = nd;
         xlast = nx;
         ylast = ny;
         n = n + 1;
         theta = theta + 1.0;
         }
      }

   nfin[ir] = n - 1;
   ix = n;
   }
}
 
