#include <stdlib.h>
#include "spread.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void smask
(
    long bckgnd,		/* I: assigned to background pxels in output  */
    long radius[],		/* I: search radius values                    */
    long target[],		/* I: user-selected target values             */
    long mask,			/* I: assigned to buffer pixels in output     */
    long quads[],		/* I: input quadrants to be searched          */
    long bands_per_img[],	/* I: number of bands for each input image    */
    long num_inimg,		/* I: total number of images                  */
    long tot_inbands,		/* I: total number of bands                   */
    long num_img_bands[][MAXBND+1]/* I: band numbers for the input image      */
)

/*
*****************************************************************************

NAME: 	SMASK

PURPOSE: Creates buffer zones around pixels of given values in an image,
         generating an output image of mask values.  


PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         22-Oct-86   J. Lavergne          prf#- 3033 3587      
   2.0         31-Aug-87   D. Akkerman/EDC      finished
   3.0         29-Mar-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

This code is an LAS version of an algorithm developed by Norman B. Bliss at the
Eros Data Center.

   Set up quadrants to be searched.
   Initialize local variables.
   Allocate the dynamic memory for the work and temporary i/o buffers.
   Initialize and enable the processing message utility.
   Initialize the internal pointers, used by GETREC and DELRUN subroutines 
   to identify the next pointer available.
   The unused portions of the point array are used to store the information 
   on the next pointer.
   For all lines
      Initialize the end array to num_pixels.
      Initialize the code array to 0 (background value).  
      Initialize the point array to zero. 
   Create an array of values showing the limiting pixel value for a
   given line in relation to the target line.
   For each line in the input image
      Calculate the starting and stopping points for the line.
      The first cell in a line which is in the buffer zone is identified by the 
      variable left, and the last cell in the buffer zone is identified by the 
      variable right.  
      For each pixel in the line
        Skip pixels that are not target values.
        Set the left and right pixel limits for the current line.
        Call RUNEND to update the run-end arrays.
        For each line north of the current line (within the radius)
           Set the left and right pixel limits for the current line.
           Call RUNEND to update the run-end arrays.
        For each line south of the current line (within the radius)
           Set the left and right pixel limits for the current line.
           Call RUNEND to update the run-end arrays.
   Create the output image from the run-end array values.
   For pixels coded 1, the mask code is written to the output image.  
   For pixels coded zero, the bckgnd code is written.
   Write the output line to the output image.
   Free the dynamic memory.

*****************************************************************************
*/

{

float	rad;			/* radius value before truncation             */

double	templr,			/* temporary radius increment value           */
	temprad;		/* temporary radius value                     */

long	buf_size,		/* size of the temporary i/o buffers          */
	*code,			/* codes for output pixel values              */
	code_val,		/* output code value                          */
	cont_flag,		/* if TRUE then continue the processing       */
	cur_band,		/* current band                               */
	curlin,			/* current line being processed by las        */
	current,		/* current root line                          */
	dtype,			/* data type                                  */
	*end,			/* position in line where code() ends         */
	i, j, k, l, p, pos,	/* loop indices and position counters         */
 	i4temp,			/* temporary variables holding one pixel      */
	index,			/* index of the target pixel in the target    */
				/* radius arrays                              */
	img,			/* current image                              */
	in_band,		/* input band for reading                     */
        in_line,		/* input line for reading                     */
	left,			/* pointer to line boundary on left side      */
	line,			/* current line being accessed during update  */
	lr,			/* radius length increments                   */
	msg_flag = 1,		/* flag for the processing message utility    */
	nextp,			/* next pointer into run-end arrays           */
	num_lines,		/* number of lines in the input image         */
	num_pixels,		/* number of pixels in the input image        */
	outcode[2],		/* output reference to mask and bckgnd        */
	out_band,		/* output band for writing                    */
	out_line,		/* output line for writing                    */
	pforl[50][81],		/* limiting pixel per line from target line   */
	*point,			/* pointers to next elements in list          */
	quadf[4],		/* quadrant flags                             */
	right,			/* pointer to line boundary on right side     */
	*root,			/* starting value in code and end for         */
				/* each line                                  */
	rsize,			/* max dimension of run-end data arrays       */
	south_flag,		/* if TRUE then process the lines to the      */
                                /* south of the current line                  */
	*tempin,		/* temporary work buffer                      */
        *tempout,		/* temporary work buffer                      */
	work_size,		/* size of the work buffers                   */
	zeroset;		/* flag for full quadrant search              */
	
long    one = 1;
long    fatal_err = -1;                /* fatal error return status */


/*
Set up quadrants to be searched.
---------------------------------
*/
zeroset = FALSE;
for (j = 0; j < 4; j++)
   quadf[j] = FALSE;

for (j = 0; j < qua_cnt; j++)
   if (quads[j]  ==  0) 
      zeroset = TRUE;
   else if (quads[j]  ==  1)  
      quadf[0] = TRUE;
   else if (quads[j]  ==  2)  
      quadf[1] = TRUE;
   else if (quads[j]  ==  3)  
      quadf[2] = TRUE;
   else 
      quadf[3] = TRUE;

/*
Initialize local variables.
---------------------------
*/
dtype = ELONG;
out_band = 0;
rsize = window[0][NL] + 1;
num_lines = window[0][NL];
num_pixels = window[0][NS];

/*
Allocate the dynamic memory for the work and temporary i/o buffers.
-------------------------------------------------------------------
*/
buf_size = (long)((num_pixels + 1) * sizeof(long));
work_size = (long)(rsize * sizeof(long));
if ((code  = (long *) malloc (work_size)) == NULL ||
    (end   = (long *) malloc (work_size)) == NULL ||
    (point = (long *) malloc (work_size)) == NULL ||
    (root  = (long *) malloc (work_size)) == NULL)
   c_errmsg ("Cannot allocate dynamic memory", "spread-alloc", &fatal_err);

if ((tempin  = (long *) malloc (buf_size)) == NULL ||
    (tempout = (long *) malloc (buf_size)) == NULL)
   c_errmsg ("Cannot allocate dynamic memory", "spread-alloc", &fatal_err);

/*
Initialize and enable the processing message utility.
------------------------------------------------------
*/
c_inittm ("SPREAD", "Processed", "line", "band", " ",  &num_lines,
           &tot_inbands, &msgtim, &msg_flag);
curcat = 0;

for (img = 0; img < num_inimg; img++)
   {
   for (cur_band = 0; cur_band < bands_per_img[img]; cur_band++)
      {
      out_band++;
      out_line = 1;
      in_line = 1;
      in_band = num_img_bands[img][cur_band];
      curent = 1;
      curcat++;
      c_startm(); 
/*
Initialize the internal pointers, used by GETREC and DELRUN subroutines 
to identify the next pointer available.
The unused portions of the point array are used to store the information 
on the next pointer.
-------------------------------------------------------------------------
*/
      nextp = num_lines + 1;

      for (i = nextp; i <= (rsize - 2); i++)
         point[i] = i + 1;

/*
For all lines
Initialize the end array to num_pixels.
Initialize the code array to 0 (background value).  
Initialize the point array to zero. 
-----------------------------------------------------
*/
      for (i = 1; i <= num_lines; i++)
         {
         root[i] = i;
         end[i] = num_pixels;
         code[i] = 0;
         point[i] = 0;
         }

/*
Store array of cells in boundary circle, relative to center pixel.  
The circle corresponding to the target pixel is given a radius of 0.56419, 
which results in circle of unit area (1.0).  
Eeach additional line or pixel is incremented by 1.0.  
The radius of the buffer zone is specified by the parameter radius.  
A "circle" of pixels (p,l) is created which have a center falling within 
the circle of radius radius + 0.56419, centered on the target pixel.  
For each target pixel in the input data, the circle is generated, 
and the start and stop pixels in each line are identified.  
The codes identifying the pixels are stored using the runend method, and 
these are used to write the output image file.  
Pixel locations for the "circle" are calculated for one quadrant, and 
mirrored for the other quadrants.

Solve the equation for a circle rad**2 = p**2 + l**2 for p, given l and 
the radius.  
Pforl array holds the limiting pixel for a given line, relative to the target 
line, which is considered line zero.  
Truncation occurs.
------------------------------------------------------------------------------
*/
      for (i = 0; i < tar_cnt; i++)
         {
         for (k = 0; k < radius[i] + 1; k++)
            {
            lr = k;
            rad = (float)(radius[i]) + 0.56419;
            temprad = (double)(rad * rad);
            templr = (double)(lr * lr);
            pforl[i][k] = (long) sqrt(temprad - templr);
            }
         }


/*
For each line of the input image, calculate the start and stop codes.  
The first cell in a line which is in the buffer zone is identified by the 
variable left, and the last cell in the buffer zone is identified by the 
variable right.  
----------------------------------------------------------------------------
*/
      for (curlin = 1; curlin <= num_lines; curlin++)
         {
         curent++;
         c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line,
                  (unsigned char *)inbuf,
                   &one);
         in_line++;
         c_pxcopy ((unsigned char *)inbuf, (unsigned char *)&tempin[1], &dtype,
                    &num_pixels);
/*
Loop on pixels within line, skipping the pixel if it is not a target value.
---------------------------------------------------------------------------
*/
         for (p = 1; p <= num_pixels; p++)
            {
            cont_flag = south_flag = FALSE;
            for (i = 0; i < tar_cnt; i++)
               {
               i4temp = target[i];
               if (tempin[p]  ==  i4temp) 
                  {
                  cont_flag = TRUE;
                  index = i;
                  break;
                  }
               }

/*
Set pixel limits for the current line and update the run-end arrays.
---------------------------------------------------------------------
*/
            if (cont_flag)
               {
               line = curlin;
               if ((quadf[0] || quadf[3]) && !(quadf[1] || quadf[2])) 
                  left = p;
               else
                  left = p - pforl[index][0];
  
               if (left <  1) 
                  left = 1;

               if ((quadf[1] || quadf[2]) && !(quadf[0] || quadf[3])) 
                  right = p;
               else
                  right = p + pforl[index][0];

               if (right > num_pixels) 
                  right = num_pixels;
   
               runend (rsize, line, left, right, end, code, point, 
                       &nextp,  num_pixels, root);

/*
Set pixel limits for lines to the north and update the run-end arrays.
----------------------------------------------------------------------
*/
               for (lr = 1; lr <= radius[index]; lr++)
                  {
                  line = curlin - lr;
                  if (line  <  1) 
                     {
                     south_flag = TRUE;
                     break;
                     }
                  if (quadf[0] && (!(quadf[1]))) 
                     left = p;
                  else
                     left = p - pforl[index][lr];
   
                  if (left < 1) 
                     left = 0;
   
                  if (quadf[1] && (!(quadf[0]))) 
                     right = p;
                  else
                     right = p + pforl[index][lr];

                  if (right > num_pixels) 
                     right = num_pixels;

                  if ((quadf[0]) || (quadf[1]) || (zeroset)) 
                     runend (rsize, line, left, right, end, code, 
                             point, &nextp, num_pixels, root);
                  }
                  south_flag = TRUE;
/*
Set pixel limits for lines to the south and update the run-end arrays.
----------------------------------------------------------------------
*/
               if (south_flag)
                  {
                  for (lr = 1; lr <= radius[index]; lr++)
                     {
                     line = curlin + lr;
                     if (line  >  num_lines) 
                         break;

                     if (quadf[3] && (!(quadf[2]))) 
                        left = p;
                     else
                        left = p - pforl[index][lr];

                     if (left < 1) 
                        left = 1;
 
                     if (quadf[2] && (!(quadf[3]))) 
                        right = p;
                     else
                        right = p + pforl[index][lr];

                     if (right > num_pixels) 
                         right = num_pixels;

                     if ((quadf[2]) || (quadf[3]) || (zeroset)) 
                        runend (rsize, line, left, right, end, code, 
                                point, &nextp, num_pixels, root);
                     }
                  } /* south_flag */
               } /* cont_flag */
            } /* pixel loop */
         } /* line loop */

/* 
Create the ouput image from the runend array values.
For pixels coded 1, the mask code is written to the output image.  
For pixels coded zero, the bckgnd code is written.
Write the output line to the output image.
------------------------------------------------------------------------
*/
      outcode[0] = bckgnd;
      outcode[1] = mask;
      for (l = 1; l <= num_lines; l++)
         {
         current = root[l];
         pos = 1;
         while (current  !=  0) 
            {
	    printf("current == %d\n", current);
            code_val = outcode[code[current]];
            for (p = pos; p <= end[current]; p++)
               tempout[p] = code_val;
            pos = end[current] + 1;
            current = point[current];
            }

         c_pxcopy ((unsigned char *)&tempout[1], (unsigned char *)outbuf,  
                   &dtype, &num_pixels);
         c_ewrite ((struct FDESC **)&fdescout, &out_band, &out_line, 
                   (unsigned char *)outbuf, &one);
         out_line++;
         }
      } /* band loop */
   } /* img loop */
   c_stoptm(); 

/*
Free the dynamic memory.
------------------------
*/
free (code);
free (end);
free (point);
free (root);
free (tempin);
free (tempout);         

}

