#include "stat2tab.h"
#include <math.h>
#include "util.h"
#include "protos.h"

void covar_data
(
    struct PARBLK *vblock,	/* TAE parameter block			*/
    FILE *fd,			/* file descriptor			*/
    struct TAB_DEF *tab,	/* LT file structure			*/
    char outfil[][CMLEN],	/* New LT file name			*/
    long *nfile,		/* number of LT files			*/
    long *cur_class,		/* current class in STAT file		*/
    long *cur_site,		/* current site in STAT file		*/
    long level[],		/* level in STAT file 			*/
    char *classval[TAE_VAL+1],	/* class name for STAT file		*/
    char *siteval[TAE_VAL+1],	/* site name for STAT file		*/
    long nitems,		/* number of items associated with data */
    long *total_covar		/* total number of matrix in STAT file  */
)
/*
**********************************************************************

NAME: COVAR_DATA                            

PURPOSE:  Transfer covariance data from STAT file to the Label Table.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1.0         11/86      K. Gacke     original development
    1.1          1/87      K. Gacke     new LT file format
    2.0          9/87      K. Gacke	fixed an infinite loop when processing
					covariance matrix and the covariance
					matrix does not exist at the image level
    2.1  	 1/90      J. Marmon    v5.0 conversion 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      Assign current level to either (image, class, site, or null)
      If current level not null and there are class & site names then
	 calculate the size of the covariance matrix
	 write covariance LT header info
	 allocate buffers for transfer of data
	 write data from STAT file to LT file
	 get number of points for next class & site
	 get mean vector matrix
	 get covariance matrix

************************************************************************
*/
{
static struct MATRIX in_covar =  {0,0};  /* # of rows, # of colms in matrix   */
static struct MATRIX out_covar = {0,0};  /* # of rows, # of colms in out matrx*/
static struct MATRIX in_mean;	/* # of rows & colms in incomming mean matrix */
static struct MATRIX out_mean;	/* # of rows & colms in out going mean matrix */
static double *covar_buf = NULL;	 /* buffer for covariarnce data       */
static double *mean_buf = NULL;		 /* buffer for mean data	      */
static double *matrix_buf = NULL;	 /* buffer for matrix data	      */
static long size = 0;		/* size for dynamic allocating memory	      */
long nonfatal = 1;

struct  TAB_DEF dummy;		/* blank parameter for get_class_site not used*/
long    shape[9];		/* shape buffer				      */
double	*matrix_ptr;		/* pointer into matrix			      */
double	root;			/* Used for finding the number of bands	      */
long	i;			/* row index into the covariance matrix       */
long	ival;			/* variable to compute STAT index (k)         */
long	j;			/* column index into the covariance matrix    */
long	jval;			/* variable to compute STAT index (k)         */
long	k;			/* index into STAT covariance matrix          */
long	npts;			/* number of points			      */
long	status;			/* return message from STAT calls	      */
char	class_name[STNAME];	/* current STAT class name 		      */
char	site_name[STNAME];	/* current STAT site name		      */
char	*cur_level;		/* current level in STAT file		      */
char	msg[256];		/* message buffer for errors		      */


if ((*cur_class == 0) && (level[IMAGE_LEV]))
   cur_level = "IMAGE";
else if ((*cur_class > 0) && (*cur_site == 0) && (level[CLASS_LEV]))
   cur_level = "CLASS";
else if ((*cur_class > 0) && (*cur_site > 0) && (level[SITE_LEV]))
   cur_level = "SITE";
else
   cur_level = "\0";

if ((*cur_level != NULL) && 
    (get_class_site(vblock,fd,tab,&dummy,outfil,nfile,"NAME",cur_class,cur_site,
		    classval,siteval,class_name,site_name)))
   {
   ++(*total_covar);
   root = sqrt((double) (8.0 * nitems + 1.0));
#ifdef gould
   in_covar.nrow = in_covar.ncol = (long) (((root - 1.0) / 2) + 0.0001);
#else
   in_covar.nrow = in_covar.ncol = (long) ((root - 1.0) / 2);
#endif
   if (out_covar.nrow == 0)		/* Processing first matrix           */
      {
      in_mean.nrow = out_mean.nrow = 1;
      out_mean.ncol = out_covar.nrow = out_covar.ncol = in_covar.nrow;
      covar_header(tab,out_covar.nrow);
      size = in_covar.nrow;
      if ((covar_buf = (double *) calloc(nitems,sizeof(double))) == NULL)
         {
	 strcpy(msg,"Error allocating STAT covariance data buffer");
	 c_errmsg(msg,"stat2tab-dynamic",&nonfatal);
         close_tab(tab);
         c_clean(vblock,"STAT2TAB",outfil,nfile);
	 }
      if ((matrix_buf = (double *) calloc(size*size,sizeof(double))) == NULL)
         {
	 strcpy(msg,"Error allocating Labeled Table covariance matrix");
	 c_errmsg(msg,"stat2tab-dynamic",&nonfatal);
         close_tab(tab);
         c_clean(vblock,"STAT2TAB",outfil,nfile);
	 }
      if ((mean_buf = (double *) calloc(size,sizeof(double))) == NULL)
         {
	 strcpy(msg,"Error allocating buffer for the STAT mean vector");
	 c_errmsg(msg,"stat2tab-dynamic",&nonfatal);
         close_tab(tab);
         c_clean(vblock,"STAT2TAB",outfil,nfile);
	 }
      }
   if (in_covar.nrow != out_covar.nrow)
      {
      sprintf(msg,"%s %d %s %d x %d.  %s %d x %d.\n",
              "Covariance matrix",*total_covar,"in the Labeled Table is",
              in_covar.nrow,in_covar.ncol,"The matrix in the Labeled Table is",
              out_covar.nrow,out_covar.ncol);
      c_errmsg(msg,"stat2tab-write",&nonfatal);
      c_errmsg("Tried to write different matrix sizes into Labeled Table",
              "stat2tab-write",&nonfatal);
      if (size < in_covar.nrow)
         {
	 free(covar_buf);
	 free(matrix_buf);
         free(mean_buf);
         size = in_covar.nrow;
         if ((covar_buf = (double *) calloc(nitems,sizeof(double))) == NULL)
            {
	    strcpy(msg,"Error allocating STAT covariance data buffer");
	    c_errmsg(msg,"stat2tab_dynamic",&nonfatal);
            close_tab(tab);
   	    c_clean(vblock,"STAT2TAB",outfil,nfile);
	    }
         if ((matrix_buf = (double *) calloc(size*size,sizeof(double))) == NULL)
            {
	    strcpy(msg,"Error allocating Labeled Table covariance matrix");
	    c_errmsg(msg,"stat2tab-dynamic",&nonfatal);
            close_tab(tab);
   	    c_clean(vblock,"STAT2TAB",outfil,nfile);
	    }
         if ((mean_buf = (double *) calloc(size,sizeof(double))) == NULL)
            {
	    strcpy(msg,"Error allocating buffer for the STAT mean vector");
	    c_errmsg(msg,"stat2tab-dynamic",&nonfatal);
            close_tab(tab);
   	    c_clean(vblock,"STAT2TAB",outfil,nfile);
	    }
         }
      }
/*
*  Write data into the Labeled Table file.
*/
   put_i4_field(tab,*total_covar,FIELD_SEP1);
   put_string_field(tab,cur_level,FIELD_SEP1,TRUE);
   put_i4_field(tab,in_covar.nrow,FIELD_SEP1);
   put_string_field(tab,class_name,FIELD_SEP1,TRUE);
   put_string_field(tab,site_name,FIELD_SEP1,TRUE);
   status = c_stget(&fd,"NPOINTS",cur_class,cur_site,shape,&npts);
   switch (status)
      {
      case 0:  put_i4_field(tab,npts,FIELD_SEP2);	break;
      default: put_null_field(tab,FIELD_SEP2);		break;
      }
   status = c_stget(&fd,"MEAN_VECTOR",cur_class,cur_site,shape,mean_buf);
   switch (status)
      {
      case E_SUCC:
	in_mean.ncol = in_covar.ncol;
	put_r8_matrix(tab,&in_mean,&out_mean,mean_buf,FIELD_SEP2);
	break;
      default: 
	put_null_matrix(tab,&out_mean,FIELD_SEP2);
	break;
      }
   status = c_stget(&fd,"COVARIANCE_MATRIX",cur_class,cur_site,shape,covar_buf);
   if (status != E_SUCC)
      {
      c_errmsg("Error reading covariance matrix data","stat2tab-fdata",
	       &nonfatal);
      close_tab(tab);
      c_clean(vblock,"STAT2TAB",outfil,nfile);
      }
   for (matrix_ptr=matrix_buf,i=0; i<in_covar.nrow; i++)
      for (j=0; j<in_covar.ncol; j++)
         {
         ival = max(i,j);
    	 jval = min(i,j);
         k = ((ival+1) * ival) / 2 + jval;
         *matrix_ptr++ = *(covar_buf + k);
         }
   put_r8_matrix(tab,&in_covar,&out_covar,matrix_buf,RECORD_SEP);
   }
return;
}
