#include "stats.h"
#include "pixman.h"
#include "typlim.h"

void classtat
(
    char mean[],	/* I: Option for mean                            */
    char covar[],	/* I: Option for covariance                      */
    char hist[],	/* I: Option for histogram                       */
    long icno[],	/* I: Set of class indices                       */
    long ncx		/* I: Number of classes                          */
)

/******************************************************************************

NAME:	classtat

FUNCTION:
	calculate statistics for requested classes from site
	statistics.
PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       15-Mar-82    C. Gurney
    2.0          Mar-1987  N. Deodhar/SAR   New LAS & conversion to C
    3          8-NOV-1989  P. Pease/GSFC    Changed GETPAR.C call to c_minmax
                                            from minmax.
    4.0          Jan 90    B. Ailts	    Dynamically allocated memory to be
					    able to run on PN9050.
    5.0          Jan 90    B. Ailts         Fixed call to c_minmax in getpar
    5.1          Apr 90    B. Ailts         Fixed call to c_stdldf in sitestat
    5.2          Jun 90    B. Ailts         Fixed how isno was passed to called
					    routines
    5.3          Nov 90    W. Xia           Free dynamically allocated memory
    					    modified format for information
                                            message from errmsg
                                            
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS
           
ALGORITHM (FLOW):
	DO for the number of input classes
	   Get the number of sites in this class
	   DO for the number of sites
	      Get the mean vector
	      Accumulate the mean values and number of points
	   END DO
	   Update the class mean vector
	   Update the number of points for the class
	   IF the class has more than one point THEN
	      DO for the number of sites
	         Get the number of points for this site
	         Get the mean vector for this site
	         Get the covariance for this site
	         DO for the number of bands
	            Compute the class covariance
	         End do
	         Get the covariance determinant
	         DO for the number of bands
	            Compute the class correlation matrix
	         End do
	         Write the correlation matrix
	      END DO
	   ELSE
	      Write zeroes to the covariance matrix
	      Write zeroes to the correlation matrix
	   END IF
	   DO for the number of input bands
	      Generate a histogram name based on the current band no.
	      Initialize the class histogram array
	      DO for the number of sites being processed
	         Initialize the site histogram array
	         Retrieve the site histogram
	         Accumulate the class histogram
	      END DO
	      Set the min/max and size of the histogram
	      Output the class histogram
	   END DO
	   DO for any extra bands previously contained in the file
	      Delete old histograms
	   END DO
	END DO

*******************************************************************************/
{
long	omax,			/* Output maximum value                       */
	omin;			/* Output minimum value                       */

double	*ccor,			/* Class correlation                          */
	*ccov,			/* Class covariance                           */
	*scov,			/* Site covariance                            */
	*cmean,			/* Class mean                                 */
        det,                    /* Determinant           	      */
	prod,			/* Temporary variable for multiplication      */
	prod2,			/* Temporary variable for multiplication      */
	*smean,			/* Site mean                                  */
	*tmean;			/* Class mean vector                          */
static double	*cov[MAXBND];	/* Covariance matrix                          */

long	cerr,			/* Return status                              */
	cpts,			/* No. of points for class                    */
	cshape[9],		/* Covariance matrix shape                    */
	herr,			/* Return status                              */
	hshape[9],		/* Histgram shape                             */
	i, n,			/* Loop counter                               */
	ib,			/* Counter for bands                          */
	ic,			/* A class number                             */
	ierr,			/* Return status                              */
	ii,			/* Temporary variable used as array index     */
	ir,			/* Return status                              */
	j,			/* Loop counter                               */
	jj,			/* Temporary valiable used as array index     */
	kk,			/* Temporary valiable used as array index     */
	lb,			/* Loop counter for bands                     */
	nshape[9],		/* Number of sites and points shape           */
	maxh,			/* Max value in class histogram               */
	merr,			/* Return status                              */
	mshape[9],		/* Mean vector shape                          */
	nerr,			/* Return status                              */
	npts,			/* Number of points in polygon                */
	ns,			/* Number of sites in a class                 */
	prevb,			/* The previous band                          */
	prod0,			/* Flag for product equal to zero             */
	shape[9],		/* Stat shape vector                          */
        max_cvsiz,
	three = 3;
static long chist[258],		/* Class histogram                            */
	shist[258],		/* Site histogram                             */
	two58 = 258;		/* A number for passing by reference          */

char	class[STNAME],		/* Class name                                 */
	hname[8],		/* Histogram name                             */
	outbuf[ERRLEN];		/* Error message output buffer                */
long    btype=EBYTE;
long    elenth=ERRLEN;
long    stnam=STNAME;        
long    fatal_err = -1;
long    inf_err = 1;
long    zero=0;

/*
  Allocate dynamic storage
  ------------------------ */
max_cvsiz = (long)((totbnd * (totbnd+1)/2 ) * sizeof(double));
scov = (double *) malloc (max_cvsiz );
ccor = (double *) malloc (max_cvsiz );
ccov = (double *) malloc (max_cvsiz );
cov_matrix = (double *) malloc (totbnd * totbnd * sizeof(double));
if ((scov == NULL) || (ccor == NULL) || (ccov == NULL) || (cov_matrix == NULL))
   {
   c_errmsg("Error allocating dynamic memory","stats-alloc",&inf_err);
   c_errmsg("Fatal error encountered","stats-fatal",&fatal_err);
   }
for (i = 0, n = 0; i < totbnd; i++, n += totbnd)
   cov[i] = &cov_matrix[n];

cmean = (double *) malloc (MAXBND * sizeof(double));
smean = (double *) malloc (MAXBND * sizeof(double));
tmean = (double *) malloc (MAXBND * sizeof(double));
if ((cmean == NULL) || (smean == NULL) || (tmean == NULL))
   {
   c_errmsg("Error allocating dynamic memory","stats-alloc",&inf_err);
   c_errmsg("Fatal error encountered","stats-fatal",&fatal_err);
   }

/* Equivalence the min and max values
   ---------------------------------- */
omin = MAXLONG;
omax = MINLONG;
/*
   Loop through classes.
   --------------------- */
for (i = 0; i < ncx; i++)
   {
   ic = icno[i];
   cpts = 0;
   if (i == 0)
      {
      ierr = c_stget ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &zero, mshape,
		        (unsigned char *)tmean);
      prevb = totbnd;
      if (mshape[2] != totbnd && ierr == E_SUCC)
         prevb = mshape[2];
      }
   nerr = c_stget ((FILE **)&sdesc, "NSITES", &ic, &zero, nshape,
                   (unsigned char *)&ns);
   if (nerr != E_SUCC)
      c_errmsg ("Error getting number of sites", "stats-nsite", &inf_err);

/* Mean and covariance/correlation are needed.
   ------------------------------------------- */
   for (ib = 0; ib < totbnd; ib++)
      cmean[ib] = 0.0;
/*
   Update class means with site means.
   -----------------------------------
*/
   for (j = 1; j <= ns; j++)
      {
      merr = c_stget ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &j, mshape, 
		       (unsigned char *)smean);
      if (merr == E_SUCC)
         {
         nerr = c_stget ((FILE **)&sdesc, "NPOINTS", &ic, &j, nshape, 
		         (unsigned char *)&npts);
         cpts += npts;
         for (ib = 0; ib < totbnd; ib++)
            cmean[ib] += (npts * smean[ib]);
         }
      else
        c_errmsg ("Error getting mean vector", "stats-mean", &inf_err);
      }
/*
   Divide each mean by the number of points in each class.
   -------------------------------------------------------
*/
   for (ib = 0; ib < totbnd; ib++)
	{
         if (cpts > 0 ) cmean[ib] /= cpts;
        }
/*
   Write number points in class and class mean to stats file
   ---------------------------------------------------------
*/
   nerr = c_stput ((FILE **)&sdesc, "NPOINTS", &ic, &zero, nshape,
		   (unsigned char *)&cpts);
   if (nerr != E_DTOLD && nerr != E_DTNEW)
      c_errmsg ("Error writing npoints", "stats-npts", &fatal_err);
   if (mean[0] == 'Y')
      {
      cerr = c_stput ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &zero, mshape,
			(unsigned char *)cmean);
      if (cerr != E_DTOLD && cerr != E_DTNEW)
      c_errmsg ("Error writing mean vector", "stats-mean", &fatal_err);
      }
/*
   Update class covariance with site covariance and means
   Compute class correlation from class covariance
   ------------------------------------------------------ */
   if (cpts > 1)                                               
      {
      for (ib = 0; ib < (totbnd * (totbnd+1)/2); ib++)
         ccov[ib] = 0.0;
      for (j = 1; j <= ns; j++)
         {
         nerr = c_stget ((FILE **)&sdesc, "NPOINTS", &ic, &j, nshape, 
			 (unsigned char *)&npts);
         merr = c_stget ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &j, mshape,
			(unsigned char *)smean);
         cerr = c_stget ((FILE **)&sdesc, "COVARIANCE_MATRIX", &ic, &j, cshape,
			 (unsigned char *)scov);
         if (cerr == E_SUCC && merr == E_SUCC && nerr == E_SUCC)
            {
             c_pxfill((unsigned char *)class, &btype, &stnam, "");
       	     ierr = c_stget ((FILE **)&sdesc,"NAME",&ic,&zero,shape,
			     (unsigned char *)class);
             if (ierr != E_SUCC)
             c_errmsg ("Error getting class names", "stats-name", &inf_err);
             kk = 0;
             for (ib = 0; ib < totbnd; ib++)
               {
               for (lb = 0; lb <= ib; lb++)
                  {
                  prod = (scov[kk] * (npts-1)) / (cpts - 1);
                  prod2 = (smean[lb] - cmean[lb]) * (smean[ib] - cmean[ib]);
                  prod2 = (prod2 * npts) / (cpts-1);
                  ccov[kk] = ccov[kk] + prod + prod2;
                  cov[lb][ib] = cov[ib][lb] = ccov[kk];
                  kk++;
                  }
               }
            if (covar[0] == 'Y')
               {
               cerr = c_stput ((FILE **)&sdesc, "COVARIANCE_MATRIX", &ic, &zero,
                              cshape, (unsigned char *)ccov);
               if (cerr != E_DTNEW && cerr != E_NOSI && cerr != E_DTOLD)
                  c_errmsg ("Error writing data to statistics file",
                           "stats-write", &fatal_err);
               det = stats_determ(cov, totbnd);
               if (det <= 0.00001)
                  {
                  c_errmsg ("Determinant of Covariance matrix <= 0 for",
                           "stats-det", &inf_err);
                  sprintf (outbuf, "class %s", class);
                  c_errmsg (outbuf, "stats-info", &inf_err);
 	          c_pxfill((unsigned char *)outbuf, &btype, &elenth," ");
                  }
               prod0 = FALSE;
               kk = 0;
               for (ib = 0; ib < totbnd; ib++)
                  {
                  ii = ((ib + 1) * (ib + 2) / 2) - 1;
                  for (lb = 0; lb <= ib; lb++)
                     {
                     jj = ((lb + 1) * (lb + 2) / 2) - 1;
                     prod = ccov[ii] * ccov[jj];
                     if (prod <= 0.000001)
                        {
              c_errmsg ("Diagonal element <= 0 in Covariance matrix ",
                                 "stats-diag", &inf_err);
                        sprintf (outbuf, "for class %s ",class);
                        c_errmsg (outbuf, "stats-info", &inf_err);
                        c_errmsg ("Correlation matrix not computed",
				  "stats-info", &inf_err);
	     		c_pxfill((unsigned char *)outbuf,&btype,&elenth," ");                
                        prod0 = TRUE;
                        break;
                        }
                     else
                        ccor[kk] = ccov[kk] / sqrt (prod);
                     kk++;
                     }
                     if (prod0)
                        break;
                  }
               if (!prod0)
                  {
                  cerr = c_stput ((FILE **)&sdesc, "CORRELATION_MATRIX", &ic, 
				  &zero, cshape, (unsigned char *)ccor);
                  if (cerr != E_DTNEW && cerr != E_NOSI && cerr != E_DTOLD)
                     c_errmsg ("Error writing data to statistics file",
                              "stats-write", &fatal_err);
                  }
               }
            }
         else 
           c_errmsg ("Error getting stats", "stats-get", &inf_err);
         }                /* end of for 'j' loop */
      }
   else
      {
/*
   No points for the class... write out zeros
   ------------------------------------------ */
      for (ib = 0; ib < max_cvsiz; ib++)
         {
         ccov[ib] = 0.0;
         ccor[ib] = 0.0;
         }
      cshape[0]=EDOUBLE;
      cshape[1]=1;     
      cshape[2]=(totbnd *(totbnd+1))/2;
      for (i=3; i<9; i++) cshape[i]=0;
      cerr = c_stput ((FILE **)&sdesc, "COVARIANCE_MATRIX", &ic, &zero, cshape,
		       (unsigned char *)ccov);
      if (cerr != E_DTOLD && cerr != E_DTNEW)
        c_errmsg ("Error writing covar matrix", "stats-covar", &fatal_err);
      cerr = c_stput ((FILE **)&sdesc, "CORRELATION_MATRIX", &ic, &zero, cshape,
	              (unsigned char *)ccor);
      if (cerr != E_DTOLD && cerr != E_DTNEW)
        c_errmsg ("Error writing correlation matrix", "stats-corr", &fatal_err);
      }

/* Update class histograms with site histograms.
   --------------------------------------------- */
   if (hist[0] == 'Y')
      {
      herr = c_stget ((FILE **)&sdesc, "HIST001", &ic, &zero, hshape, 
			(unsigned char *)chist);
      for (ib = 1; ib <= totbnd; ib++)
         {
/*
   Convert ib to  character string
   ------------------------------- */
         sprintf (hname, "HIST%03d", ib);
/*
   It will print like this 'HIST001'.
   ---------------------------------- */
         maxh = 0;
         omin = 0;
         omax = 0;
         c_pxfill ((unsigned char *)chist, &three, &two58,
		   (unsigned char *)&zero); 
         for (j = 1; j <= ns; j++)
            {
            c_pxfill ((unsigned char *)shist, &three, &two58,
			(unsigned char *)&zero);
            herr = c_stget ((FILE **)&sdesc, hname, &ic, &j, hshape,
			    (unsigned char *)shist);
            if (herr == E_SUCC)
               {
               if (hshape[2] > maxh)
                  maxh = hshape[2];
               for (ii = 0; ii < 256; ii++)
                  chist[ii] += shist[ii];
               omin += shist[256];
               omax += shist[257];
               }
            else
               {
               c_errmsg ("Error getting histograms", "stats-hist", &inf_err);
               }
            }
         chist[256] = omin;
         chist[257] = omax;
         hshape[2] = maxh;
         herr = c_stput ((FILE **)&sdesc, hname, &ic, &zero, hshape, 
			 (unsigned char *)chist);
         if (herr != E_DTOLD && herr != E_DTNEW)
           c_errmsg ("Error writing histogram", "stats-hist", &fatal_err);
         }
      }
/*
   Delete any extraneous histograms
   -------------------------------- */
   if (totbnd > prevb)
     { 
     for (ib = totbnd + 1; ib <= prevb; ib++)
      {
      sprintf (hname, "HIST%03d", ib);
      ir = c_stdldt ((FILE **)&sdesc, hname, &ic, &zero);
      if (ir != E_SUCC)
        c_errmsg ("Error deleting histogram", "stats-hist", &fatal_err);
      }
     }
 }
free(scov); 
free(ccor); 
free(ccov);
free(cov_matrix);
free (cmean);
free (smean);
free (tmean);
}
