#include "stats.h"
#include "util.h"
#include "pixman.h"
#include "typlim.h"

void sitestat
(
    char mean[],	/* I: Option for mean                            */
    char covar[],	/* I: Option for covariance                      */
    char hist[],	/* I: Option for histogram                       */
    char hosnm[][CMLEN],/* I: Buffer for input host names                */
    long nmrimg,	/* I: Number of images                           */
    long bands_num[][MAXBND + 1],/* I: Band number for each imag         */
    long nbands[],	/* I: Number of bands for each image             */
    long icno[],	/* I: Set of class indices                       */
    long *isno[MAXCLS],	/* I: Set of site indices                        */
    long nsites[],	/* I: Number of sites for a class                */
    long ncx,		/* I: Number of classes to be used               */
    long mdone[],	/* I: Indicates mean written for site            */
    long cdone[],	/* I: Indicates covariance for a site            */
    long hdone[],	/* I: Indicates histogram written for site       */
    char subc[],	/* I: Subcommand to process                      */
    long msgtim,	/* I: Time processing message time interval      */
    long num_samps	/* I: Total number of samples in the input image */
)

/******************************************************************************
 
NAME:	sitestat
 
PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       15-Mar-82    C. Gurney
    2.0          Mar-1987  N. Deodhar/SAR   New LAS & conversion to C
    3.0          Dec-88    B. Allam/STX     Retrocode
    3.1          Apr-89    D. Hollaren      Added static to compile on PN9050
    3.2		 Dec-90    T. Sturman	    Added calls to free dynamic memory
    4.0          DEC-90    W. Xia/stx       fixed calls to c_stput;
    					    get rid off incorrectly used
 					    strncpy             
    					    modified format for information
                                            message from errmsg
    5.0		 Oct-99    D. Hames         Remove MAXNS and replace with 
					    num_samps. Modified to support
				            images > 2gig. Add prototypes. 
					    SRF #893

FUNCTION:
	Calculates requested statistics for sites which do not already
	contain them.

*******************************************************************************/

{
static double	*cov[MAXBND],	/* Covariance matrix                          */
	*scor,			/* Site correlations                          */
	*scov,			/* Site covariances                           */
	smean[MAXBND],		/* Site mean vector                           */
	sum[MAXBND],		/* Site sum of values                         */
	tmean[MAXBND],		/* Mean vector                                */
	det, 			/* Determinant value			      */
        *zmat;			/* Correlation matrix                         */

float	fact;			/* Temporary variable for multiplication      */

long	cshape[9],		/* Covariance vector shape                    */
	dcnt,			/* Site counter                               */
	fact0,			/* Flag indicating multiplication yielded 0   */
	hshape[9],		/* Histogram shape                            */
	i,			/* Loop counter                               */
	ib,			/* Loop counter on bands                      */
	ic,			/* Class number                               */
	ien,			/* Temporary variable                         */
	ierr,			/* Stat I/O return status                     */
	ii,			/* Loop counter on interior points            */
	ij,			/* Loop counter on bands                      */
	is,			/* Site number                                */
	ist,			/* Temporary variable                         */
	j,			/* Loop counter                               */
	jb,			/* Loop counter and array index               */
	jfill,			/* Loop counter                               */
	jj,			/* Loop counter                               */
	k,			/* Number of pixels in a site                 */
	kk, size,		/* Array index                                */
	lmx, dtype, 		/* Line counter                               */
	lnk,			/* Loop counter on lines                      */
	msg_flag,		/* Type of processing line message for cinittm*/
	mshape[9],		/* Mean vector shape                          */
	msize,	         	/* Mask size for passing by reference         */
	n,			/* Loop counter                               */
	mbnd, nb,		/* Loop counter on bands                      */
	ncov,			/* Size of covariance matrix                  */
	nl,			/* Bounding rectangle number of lines         */
	ns,			/* Bounding rectangle number of samples       */
	nshape[9],		/* Number of points in polygon shape          */
	m, nv,			/* Loop counter on vertices                   */
	nverts,			/* Number of vertices                         */
	nxs,			/* Number of points in polygon                */
	prevb,			/* Previous band number                       */
	shape[9],		/* Stat file shape vector                     */
	sl,			/* Bounding rectangle starting line           */
	ss,			/* Bounding rectangle starting sample         */
	three = 3,
	two58 = 258,    	/* Number for passing by reference            */
	vshape[9],		/* Polygon vertices shape                     */
	*xaxis,			/* X co-ords of site vertices                 */
	*xs,			/* x co-ords of intersection of polygon       */
				/* boundary with line                         */
	*xyvert,		/* x + y co-ords of site vertices as returned */
				/* from stats file                            */
	*yaxis;			/* Y co-ords of site vertices                 */
static long zvec[MAXBND],	/* Mean vector                                */
	*shist[MAXBND],		/* Site histograms                            */
	maxh[MAXBND];		/* Site max. value                            */

static  char    class[STNAME],	/* Class name                                 */
	hname[STNAME],		/* Histogram name                             */
	mval  = 1,		/* Mask value for passing by reference        */
	mval0 = 0,		/* Mask value for passing by reference        */
	*mask,			/* Indicates which pixels are inside          */
				/* site polygon on current line               */
	outbuf[ERRLEN],		/* Output message buffer                      */
	site[STNAME];		/* Site name                                  */
long    btype=EBYTE;
long    elenth=ERRLEN;
long    stnam=STNAME;        
long    one=1;
long    fatal_err = -1;
long    inf_err = 1;
long    zero=0;

/* Allocate dynamic storage
  ------------------------ */
size = (long)((totbnd * (totbnd+1)/2) * sizeof(double));
scov = (double *) malloc (size);
scor = (double *) malloc (size);
zmat = (double *) malloc (size);
if ((scov == NULL) || (scor == NULL) || (zmat == NULL))
   {
   c_errmsg("Error allocating dynamic memory","stats-alloc",&inf_err);
   c_errmsg("Fatal error encountered","stats-fatal",&fatal_err);
   }

mask = (char *) malloc(num_samps);	
xaxis = (long *) malloc((MAX_VERTS + 1) * sizeof(long));
xs = (long *) malloc(MAX_VERTS * sizeof(long));
xyvert = (long *) malloc(2 * MAX_VERTS * sizeof(long));
yaxis = (long *) malloc((MAX_VERTS + 1) * sizeof(long));
if ((mask == NULL) || (xaxis == NULL) || (xs == NULL) || 
    (xyvert == NULL) || (yaxis == NULL))
   {
   c_errmsg("Error allocating dynamic memory","stats-alloc",&inf_err);
   c_errmsg("Fatal error encountered","stats-fatal",&fatal_err);
   }

dyn_alloc( cov, shist);
dtype = EREAL;

mshape[0] = cshape[0] = EDOUBLE;
hshape[0] = nshape[0] = ELONG;
mshape[1] = cshape[1] = hshape[1] = nshape[1] = 1;
for (i = 2; i < 9; i++)
   mshape[i] = cshape[i] = 0;
hshape[2] = 258;
nshape[2] = 1;
for (i = 3; i < 9; i++)
   hshape[i] = nshape[i] = 0;

/*
   Limits on size of variables to be written to stats file.
   -------------------------------------------------------- */
ncov = (totbnd * (totbnd + 1)) / 2;
mshape[2] = totbnd;
cshape[2] = ncov;
/*
   Check each site to see if stats are required.
   --------------------------------------------- */
msg_flag = ONECAT;
for (i = 0; i < ncx; i++)
   {
   c_inittm ("STATS", "Processed", "polygon", "class", "", (nsites + i),
            &ncx, &msgtim, &msg_flag);
   curcat = i + 1;
   curent = 1;
   if (i == 0)
      c_startm();
   ic = icno[i];
   dcnt = 0;
   if (i == 0)
      {
      ierr = c_stget ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &zero, shape, 
                      (unsigned char *)tmean);
      if (ierr == E_SUCC && shape[2] != totbnd)
         prevb = shape[2];
      else
         prevb = totbnd;
      }
    c_pxfill((unsigned char *)class,&btype,&stnam,"");
   ierr = c_stget ((FILE **)&sdesc, "NAME", &ic, &zero, shape, 
                    (unsigned char *)class);
   for (j = 0; j < nsites[i]; j++)
      { 
      for (ii=0; ii < totbnd * (totbnd+1) /2; ii++)
         scov[ii] = 0.0;

      curent = j + 1;
      is = isno[j][i];
      if (is != 0)
         {
         c_pxfill((unsigned char *)site, &btype,&stnam,"");
         ierr = c_stget ((FILE **)&sdesc, "NAME", &ic, &is, shape,
                           (unsigned char *)site);
         k = 0;
         for (mbnd=0; mbnd < totbnd; mbnd++)
           c_pxfill ((unsigned char *)shist[mbnd], &three, &two58, 
                      (unsigned char *)&zero);
         for (jj = 0; jj < totbnd; jj++)
            {
            smean[jj] = sum[jj] = 0.0;
            maxh[jj] = 0;
            }
         for (jj = 1; jj < ncov; jj++)
            scov[jj] = 0.0;
/*
   Get site vertices and Find bounding  rectangle of site.
   This is done in the routine VERTS as well -- This was done 
   as a quick fix to remove the array AREA from the code.
   Someday verts and this section should be combined for 
   speed reasons -- BA
   ------------------------------------------------------- */
         ierr = c_stget ((FILE **)&sdesc, "POLYGON", &ic, &is, vshape,
                         (unsigned char *)xyvert);
         nverts = vshape[2];
	 sl = MAXLONG;
	 ss = MAXLONG;
	 nl = MINLONG;
	 ns = MINLONG;
         for (nv = 0,m = 0; nv < nverts; nv++)
            {
            xaxis[nv] = xyvert[m] - master.xoff + 1;
            yaxis[nv] = xyvert[m + 1] - master.yoff + 1;
            m = m+2;
            sl = min(sl,yaxis[nv]);
            ss = min(ss,xaxis[nv]);
            nl = max(nl,yaxis[nv]);
            ns = max(ns,xaxis[nv]);
            }
         nl = nl - sl + 1;
         ns = ns - ss + 1;
/*       
   Set up files for processing
   --------------------------- */
         openfiles (hosnm, nmrimg, bands_num, nbands, &sl, &ss, &nl, &ns);
/*
  For each line in site--find the number of intersections with
   polygon boundary.
   ------------------------------------------------------------ */
         msize = num_samps;
         for (lnk = 1; lnk <= nl; lnk++)
            {
            c_pxfill ((unsigned char *)mask, &one, &msize, 
                      (unsigned char *)&mval0);
            if (c_estep ((struct GDESC **)&gdesc) != E_SUCC)
               c_errmsg ("Fatal error encountered", "stats-fatal", &fatal_err);
            lmx = sl + lnk - 1;
            xsect (xaxis, yaxis, nverts, lmx, xs, &nxs);
/*
   Find interior points in polygon. set mask = 1 for these points
   -------------------------------------------------------------- */
            for (ii = 1; ii < nxs; ii++)
               {
               if (insde (xaxis, yaxis, nverts, xs[ii-1], xs[ii], lmx))
                  {
                  ist = xs[ii-1] - ss + 1;
                  ien = xs[ii] - ss + 1;
                  for (jfill = ist - 1; jfill < ien; jfill++)
                     mask[jfill] = mval;
                  }
               }
/*
   Do statistics for all interior points.
   -------------------------------------- */
            dostats (mask, ns, covar, &k, shist, scov,
                     smean, sum, maxh, subc, lnk);
            }
           c_estep ((struct GDESC **)&gdesc);

/*
   Close the image files
   --------------------- */
         if (c_egclse ((struct GDESC **)&gdesc) != E_SUCC)
            c_errmsg ("Error closing image files","stats-close",&inf_err);

         free (thebuf);
/*
   Write out the number of points in site.
   ---------------------------------------*/
         if (k == 0 && (strcmp (subc, "CUT") == 0))
            {
            sprintf (outbuf, "All pixels cut from Class: %s",class);
            sprintf (&outbuf[strlen(outbuf)],", site: %s",site);
            c_errmsg (outbuf, "stats-cut", &inf_err);       
            strcpy(outbuf," ");       
            t_write ("", T_STDCC);
            }
         ierr = c_stput ((FILE **)&sdesc, "NPOINTS", &ic, &is, nshape,
                         (unsigned char *)&k);
/*
   Write mean vector for this site---if covariance is
   needed,mean will also be needed.
   --------------------------------------------------*/
         if ((mean[0] == 'Y') || (covar[0] == 'Y'))
            {
            ierr = c_stput ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &is, mshape,
                            (unsigned char *)smean);
            mdone[i]++;
            }
/*
   Check determinant of covariance matrix for this site
   Compute correlation from covariance  and write them
   ---------------------------------------------------- */
         if (covar[0] == 'Y')
            {
            fact0 = FALSE;
            if (k-1 > 0)
               for (n = 0; n < ncov; n++)
                  scov[n] = scov[n] / (k-1);
            kk = 0;
            for (ib = 0; ib < totbnd; ib++)
               for (jb = 0; jb <= ib; jb++)
                  {
                  cov[jb][ib] = cov[ib][jb] = scov[kk];
                  kk++;
                  }
            det = stats_determ(cov, totbnd);
            free(arr_matrix);
            if (det <= 0.00001)
               {
               c_errmsg ("Determinant of Covariance matrix <= 0 for",
                        "stats-det", &inf_err);
               sprintf (outbuf, "Class: %s",class);
               sprintf (&outbuf[strlen(outbuf)],",site %s",site);
               c_errmsg (outbuf, "stats-info", &inf_err);        
      	       c_pxfill((unsigned char *)outbuf,&btype,&elenth,"");
               t_write ("", T_STDCC);
               }
            ierr = c_stput ((FILE **)&sdesc, "COVARIANCE_MATRIX", &ic, &is, 
                          cshape, (unsigned char *)scov);
            if (ierr != E_DTOLD && ierr != E_DTNEW && ierr != E_NOSI)
               c_errmsg ("Error writing data to statistics file",
                        "stats-wstat", &fatal_err);
            kk = 0;
            for (ib = 1; ib <= totbnd; ib++)
               {
               ii = (ib * (ib + 1) / 2) - 1;
               for (jb = 1; jb <= ib; jb++)
                  {
                  jj = (jb * (jb + 1) / 2) - 1;
                  fact = scov[ii] * scov[jj];
                  if (fact <= 0.00000001)
                     {                                                      
                     c_errmsg ("Diagonal element <= 0 in Covariance matrix",
                              "stats-diag", &inf_err);
                     sprintf(outbuf, " for class: %s",class);
                     sprintf(&outbuf[strlen(outbuf)],", site: %s",site);
                     c_errmsg (outbuf, "stats-info", &inf_err);
                     c_errmsg("Correlation matrix not computed\n","stats-info",
                               &inf_err);
		     c_pxfill((unsigned char *)outbuf,&btype,&elenth," ");
                     t_write ("", T_STDCC);
                     fact0 = TRUE;
                     break;
                     }
                  else
                     scor[kk] = scov[kk] / sqrt (fact);
                  kk++;
                  }
               if (fact0)
                  break;
               }
            if (!fact0)
               {
               ierr = c_stput ((FILE **)&sdesc, "CORRELATION_MATRIX", &ic, &is,
		                cshape,(unsigned char *)scor);

               if (ierr != E_DTOLD && ierr != E_DTNEW && ierr != E_NOSI)
                  c_errmsg ("Error writing data to statistics file",
                           "stats-wrstat", &fatal_err);
               }
            cdone[i]++;
            }
/*
   Write histograms for this site
   ------------------------------ */
         if (hist[0] == 'Y')
            {
            for (nb = 1; nb <= totbnd; nb++)
               {

               /* nb will be converted to character string like this "HIST001"
                ------------------------------------------------------------ */
               sprintf (hname, "HIST%03d", nb);
               if (scales.sca[nb-1] != 1.0 || scales.off[nb-1] != 0.0)
                  hshape[2] = 258;
               else
                  hshape[2] = maxh[nb-1];
               if (maxh[nb-1] == 0)
                  hshape[2] = 1;
               ierr = c_stput ((FILE **)&sdesc, hname, &ic, &is, hshape,
				(unsigned char *)&shist[nb-1][0]);
               }
            hdone[i]++;
            }
         }
      else
         {

         /* Site went outside of input image. 
            Wipe out all stats for this site 
            --------------------------------- */
         is = dsite[dcnt++][ic];
         ierr = c_stput ((FILE **)&sdesc, "NPOINTS", &ic, &is, nshape, 
                         (unsigned char *)&zero);
         for (mbnd=0; mbnd < totbnd; mbnd++)
           c_pxfill ((unsigned char *)shist[mbnd], &three, &two58, 
                      (unsigned char *)&zero);
         for (jj = 0; jj < totbnd; jj++)
            zvec[jj] = 0.0;
         for (jj = 0; jj < ncov; jj++)
            zmat[jj] = 0.0;
         hshape[2] = 1;
         if (hist[0] == 'Y')
            {
            for (nb = 1; nb <= totbnd; nb++)
               {
               sprintf (hname, "HIST%03d", nb);
               ierr = c_stput ((FILE **)&sdesc, hname, &ic, &is, hshape, 
                               (unsigned char *)&shist[nb-1][0]);
               }
            hdone[i]++;
            }
         if (covar[0] == 'Y')
            {
            ierr = c_stput ((FILE **)&sdesc, "COVARIANCE_MATRIX", &ic, &is, 
                            cshape,(unsigned char *)zmat);
            ierr = c_stput ((FILE **)&sdesc, "CORRELATION_MATRIX", &ic, &is, 
                             cshape,(unsigned char *)zmat);
            cdone[i]++;
            }
         if ((mean[0] == 'Y') || (covar[0] == 'Y'))
            {
            ierr = c_stput ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &is, 
                            mshape,(unsigned char *)zvec);
            mdone[i]++;
            }
         }
/*
   Delete old histograms if necessary
   ---------------------------------- */
      for (ij = totbnd + 1; ij <= prevb; ij++)
         {
         sprintf (hname, "HIST%03d", ij);
         ierr = c_stdldt ((FILE **)&sdesc, hname, &ic, &is);
         }
      }     /* for j loop */
  }     /* for i loop */

   c_stoptm ();

free(scov); free(scor); free(zmat);
free(cov_matrix); free(shist_matrix);
free(mask);	
free(xaxis);
free(xs);
free(xyvert);
free(yaxis);
}
