/* include files, global parameters, and definitions for STATS */

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

#define MAX_CVSIZ     MAXBND * (MAXBND + 1) / 2

#define MAX_VERTS     401
#define PI            3.1415926

#define isspace(c)    (c == ' ' || c == '\t' || c == '\n')

/* The following structs are converted from the
   FORTRAN common blocks originally used by the program */
                                                        
struct 	  	 	/* Values used to scale histograms to 256 bins */
      {       
      float  *sca;	        /* scales                                     */
      float  *off;	        /* offsets                                    */
      } scales;

struct 			/* Used for the -CUT subcommand     */
      {
      float *hival;
      float *loval;
      long  *bands;
      } cut;

struct 			/* Stores master scene coordinates  */
      {
      long yoff;		/* Line offset                                */
      long xoff;		/* Pixel offset                               */
      long imsiz[2];		/* nl, ns                                     */
      } master;

long   dsite[MAXCLS][MAXCLS];	/* Indicates sites outside
					 	image bounds                  */
long   *shist_matrix;
double *cov_matrix;
double *arr_matrix;
long   totbnd;                  /* Total number of bands                      */
      
/* Image I/O globals
   ----------------- */
long   	fdesc[MAXIMG+1];        /* Array of file descritors                   */
long   	gdesc;			/* Group descriptor                           */
long offarray[MAXIMG*MAXBND+1]; /* Array of offsets                           */
long   	sdesc;			/* File descriptor of stats file              */
struct   PARBLK vblock;		/* TAE parameter block                        */
struct  DDR	ddr;            /* ddr                                        */
 
char   	*thebuf;		/* Buffer for image i/o                       */

extern	long	curent;         /* current entity                             */
extern	long	curcat;         /* current category                           */

long    in_type;                /*input data type                             */

/*****************************************************************************

NAME:	STATS

FUNCTION:
	STATS computes mean vectors, covariance matrices and histograms 
	for training sites and classes.  The user can exclude certain 
	grey values from statistics calculations by specifying subcommand 
	-CUT; if subcommand -ALL is specified, then all the data will be 
	used to calculate statistics.  Polygons representing training sites 
	must have been previously selected. Coordinates of the polygons
	are contained in the statistics file.  Up to 99 classes and 99 
	sites can be processed.  The input image may be any data type and 
	can contain up to 256 bands.  No window is allowed for the input 
	image.
                                                                       
PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       15-Mar-82    C. Gurney
    2.0          Mar-1987  N. Deodhar/SAR   New LAS & conversion to C
    3          8-NOV-1989  P. Pease/GSFC    Changed GETPAR.C call to c_minmax
                                            from minmax.
    4.0          Jan 90    B. Ailts	    Dynamically allocated memory to be
					    able to run on PN9050.
    5.0          Jan 90    B. Ailts         Fixed call to c_minmax in getpar
    5.1          Apr 90    B. Ailts         Fixed call to c_stdldf in sitestat
    5.2          Jun 90    B. Ailts         Fixed how isno was passed to called
					    routines
    5.3          Nov 90    W. Xia           Free dynamically allocated memory
                                            immediately after using it.    
    5.4		 Sep 92    D. Etrheim	    Modified global include file
    6.0		 Dec 99	   D. Hames         Modified sitestat to support 
				            images > 2 gig. Add prototypes. 
					    SRF #893

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):
	Do standard initialization
	Initialize arrays
	Get parameters
	Check input stats file for classes
	If list flag is set then
	   list the classes and/or sites
	Get class and site specs and class and site indeces
	Set up site windows
	Compute site stats
	Compute class stats
	Issue completion message

******************************************************************************/
void main
(void)
{
char	hosnm[MAXIMG][CMLEN];	/*  input host names                          */
char	taenm[MAXIMG][CMLEN];	/*  input TAE names                           */
char    hstat[CMLEN];		/*  stat file host name                       */

char	covar[4];		/* calculate covariance flag                  */
char    hist[4];		/* calculate histogram flag                   */
char	list[6];		/* list class option flag                     */
char	mean[4];		/* calculate mean flag                        */

char	runtyp[12];		/* TAE runtype                                */
char	subc[5];		/* sub-command name                           */

static long bands_num[MAXIMG][MAXBND+1];/* Band numbers for each image entered*/
static long nbands[MAXBND];	/* Number of bands for each image             */
long   	nmrimg;	       		/* Number of images entered                   */

long	*cdone,			/* Indicates covariance for a site            */
	cols,			/* number of columns the terminal has         */
	*hdone,			/* Indicates histogram written for site       */
	i, k;			/* loop variable                              */

long   	*icno;	   		/* Class numbers			      */
static long	*isno[MAXCLS];	/* Site number for each class                 */
long    *isno_buf;
long	lines,			/* number of lines the terminal has           */
	*mdone,			/* Indicates mean written for site            */
	msgtim,			/* Processing message time interval           */
	nc,			/* number of classes user requsted            */
	nclass,			/* number of classes in stats file            */
	ncx,			/* number of classes to be processed          */
	*nsites,		/* Number of sites in each class              */
	stdut,	 		/* FORTRAN LUN to standard output             */
	term_type;		/* terminal type                              */
long    fatal_err = -1;         /* fatal error code			      */
long    inf_err = 1;		/* nonfatal error code                        */
long    zero=0;			/* value of 0				      */
long    num_samps;		/* total number of samples in an image	      */

/* Initiate TAE
   ------------ */
c_inlas (&vblock, &stdut);
t_init (&lines, &cols, &term_type);
 
hdone = (long *) malloc(MAXCLS * sizeof(long));
cdone = (long *) malloc(MAXCLS * sizeof(long));
icno = (long *) malloc(MAXCLS * sizeof(long));
mdone = (long *) malloc(MAXCLS * sizeof(long));
nsites = (long *) malloc(MAXCLS * sizeof(long));
cut.hival = (float *) malloc(MAXBND * sizeof(float));
cut.loval = (float *) malloc(MAXBND * sizeof(float));
cut.bands = (long *) malloc(MAXBND * sizeof(long));
scales.sca = (float *) malloc(MAXBND * sizeof(float));
scales.off = (float *) malloc(MAXBND * sizeof(float));
isno_buf = (long *) malloc(MAXCLS * MAXCLS * sizeof(long));

if ((hdone == NULL) ||
    (cdone == NULL) ||
    (icno == NULL) ||
    (mdone == NULL) ||
    (nsites == NULL) ||
    (cut.hival == NULL) ||
    (cut.loval == NULL) ||
    (cut.bands == NULL) ||
    (scales.sca == NULL) ||
    (scales.off == NULL) ||
    (isno_buf == NULL))
   {
   c_errmsg("Error allocating dynamic memory","stats-alloc",&inf_err);
   c_errmsg("Fatal error encountered","stats-fatal",&fatal_err);
   }

/* Get parameters
   -------------- */
getpar (hstat, hosnm, taenm, &nmrimg, bands_num, nbands, 
        mean, covar, hist, &nc, list, subc, runtyp, &msgtim, &num_samps);
                                          
for (i = 0,k = 0;i < MAXCLS; i++,k += MAXCLS)
   isno[i] = &isno_buf[k];
          
/* Initialize arrays
   ----------------- */
for (i=0; i < MAXCLS; i++)
   {
   mdone[i] = 0;
   cdone[i] = 0;
   hdone[i] = 0;
   }                      

/* Open stats file and check the number of classes
   ----------------------------------------------- */
statchk (hstat, nc, &nclass, &ncx);

/* List class and site names in stats file
   --------------------------------------- */
if ((list[0] == 'C') || (list[0] == 'S') || (list[0] == 'B'))
   lnames (nclass, list);

/* Get user class and site specifications and associated stats
   file indices
   ----------------------------------------------------------- */
indices (nc, ncx, icno, isno, nsites);

/* Set up site windows
   ------------------- */
setarea (mean, covar, hist, ncx, icno, isno, nsites, runtyp);

/* Do statistics for sites
   ----------------------- */
sitestat (mean, covar, hist, hosnm, nmrimg, bands_num, nbands,icno, isno, 
          nsites, ncx, mdone, cdone, hdone, subc, msgtim, num_samps);
                                                                      
/* Do statistics for classes                               
   ------------------------- */
classtat (mean, covar, hist, icno, ncx);

/* Close files
   ----------- */
c_stclse ((FILE **)&sdesc);

/*  free dynamic memory
-----------------------*/
free (hdone);
free (cdone);
free (icno);
free (mdone);
free (nsites);
free (cut.hival);
free (cut.loval);
free (cut.bands);
free (scales.sca);
free (scales.off);
free (isno_buf);

c_complt ("STATS",(char (*)[256])"", &zero);
}
