#include "stereo.h"
#include "util.h"
#include "protos.h"

void p3dsub
(
    double amax[],		/* maximum pixel value */
    double amin[], 		/* minimum pixel value */
    long shift,     	/* maximum pixel shift   */
    long modflg,         /* flag indicating whether modulating image exist */
    long wind1[][4],	/* input image window   */
    long num_inbands,    /* total number of input bands to be processed   */
    long *num_outbands  /* total number of output bands to be processed  */
)
                    
/*
********************************************************************

NAME: P3DSUB

PURPOSE:
	p3dsub is the subroutine which contains the algorithm to
	create the stereo pair image. the image is created by 
	shifting pixel values based on either elevation data from
	the modulating image or from intensity data derived from the
	input image if a modulating image is not available.

PROGRAM HISTORY:
  Version   Date       Author                       Request
  -------   ----       ------                       -------
                          
  1.0     11-Nov-84    L. Huewe/CSB
  2.0     26-Mar-86    K. Gacke/CSB                 lasd
  3.0     24-Jan-90    W. Xia                 conversion to 5.0
                 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

- Use c_inittm, startm to initialize and start the processing message utility
- To set the shift factor
- To create the stereo pair complement
- To free dynamic arrays
- To stop processing message utility
- To close all the images

Reference: Batson, Edwards and Eliason, "Synthetic Stereo and Landsat 
	   Pictures", Photogrammetric Engineering and Remote Sensing,
	   Vol 42, No. 10, pp. 1279-1284, 1976.

****************************************************************************
*/

{ 

/*     local variables: */

extern long *opoint;
extern long *ipoint;

static long 
	fatal_err = -1,
	zero = 0;

float       
        delta, 		/* intermediate calculation values   */  
        delfac,      	/* intermediate calculation values   */  
        factor, 	/* shift factor */
        first; 	     	/* intermediate calculation values   */  

float
        lvalue[MAXBND],	   	/* left pixel value			*/  
        rvalue[MAXBND];       	/* right pixel value			*/  
                                                               
long                  
        i,j,m,      		/* loop indices				*/  
        index1, 		/* loop index				*/  
        index2,			/* index variable			*/   
        line, 	       		/* current line being processed		*/
        *pbuf,		        /* intermediate buffer storage		*/  
	stat;

long
        iend[MAXBND],  		/* flag indicating end of processing	*/ 
        outpts[MAXBND];		/* point to output pixels		*/ 

char	*prog_name = "STEREO";	/* program name				*/

/*
initialize and enable the processing message utility
--------------------------------------
*/                                             
c_inittm (prog_name, "Processed", "line", "bands", " ", 
&wind1[0][NL], num_outbands, &msgtim, &zero);            
curent=1;    
c_startm();    

/*
allocate dynamic array 
----------------------
*/
if (( pbuf=(long *) malloc(wind1[0][NS]*sizeof (long))) == NULL)
{
 c_errmsg("error in pbuf","stereo-p3dsub",&fatal_err);
} 

/* 
set the shift factor for each band
-----------------------------------              
*/
      if (shift  ==  0)  shift = 1;
      if(modflg == 1 )    
       {
         factor = (float)(amax[0] - amin[0]) / (float) shift;
         if (factor  == 0)  factor=1.0;
       }
      else
       {
        for (i = 0; i < num_inbands; i++)
   	  {
           if (amax[i] > amax[0]) amax[0]=amax[i];
           if (amin[i] < amin[0]) amin[0]=amin[i];
          }
        factor = (float)(amax[0] - amin[0]) /(float)shift;
        }                                           
        if (factor  ==  0)  factor = 1.0;   

/*
create the stereo pair complement and write it to the output image file
-----------------------------------------------------------------------
*/

for (line = 0; line< wind1[0][NL]; line++, curent++)
{                                       /* start of line loop */
   stat=c_estep (&gdesc);
  
   if (stat != E_SUCC )
      {
      c_errmsg ("Error in estep", "stereo-estep",&fatal_err);
      }
    
    index2=0;      
    for (i = 0; i < num_inbands; i++)
    {	      
     iend[i]=0;
     outpts[i] = 1;                                                
    }
                         
/* for each sample of a line */

   for (i=0; i<num_inbands; i++)
   {                                            /* start of i loop */
       
/*                         
get disparity function for each band 
--------------------------------------
*/
    for(j = 0; j < wind1[0][NS]; j++)
     {                                                          
     if (modflg  ==  1) 
       {                                       
       pbuf[j] = (long) ((iobuf[ipoint[num_inbands]+ j])/factor)+ j+ 1;
       }
     else
       {                                                         
       pbuf[j] = (long) ((iobuf[ipoint[i]+ j] / factor)) + j + 1;      
       }                                                     
     }
 
    for (index1 = 0; index1 < wind1[0][NS]; index1++)
      {                            /* start of index1 loop */

      if (iend[i]  !=  1) 
         {                          /* start of if loop */
         if (index1  ==  0)  lvalue[i] = iobuf[ipoint[i] + index1];
         index2 = pbuf[index1];                          
         rvalue[i] = iobuf[ipoint[i] + index1];                    
         if (index2  <  outpts[i]) 
            {
            lvalue[i] =iobuf[ipoint[i] + index1];
            }                   
         else if (index2 ==  outpts[i])  
            {                        
            lvalue[i] = iobuf[ipoint[i] + index1];
            iobuf[opoint[i] + outpts[i]-1] = iobuf[ipoint[i] + index1];
            outpts[i] ++;
            if (outpts[i]  >  wind1[0][NS]) iend[i] = 1;
            }                                         
         else 
            {
            delta = (float)(index2 - outpts[i]);
            delfac = (rvalue[i] - lvalue[i]) / (delta + 1.0);
            first = lvalue[i]; 
            for (m = 1; (float) (m) <= delta+1; ++m)                    
               {                       
               if (iend[i]  !=  1) 
                  {                                                            
                  iobuf[opoint[i] + outpts[i]-1] = first + (float)(m)*delfac;
                  if (index1  ==  0)  iobuf[opoint[i] + outpts[i] -1] = 0;
		  outpts[i] ++;       
                  if (outpts[i]  >  wind1[0][NS]) iend[i] = 1;
                  } 
                }                    /* end of m loop */         
             }                       /* end of else if */                  
         }                           /* end of if loop */      
      }                              /* end of index1 (pixel) loop  */ 
    }                                /* end of i  (band) loop */
 }                                   /* end of line loop */

stat=c_estep (&gdesc);


if (stat != E_SUCC )
   {
   c_errmsg ("Error in estep", "stereo-estep",&fatal_err);
   }
  
/*
free dynamic array
------------------
*/
free (pbuf);
free(iobuf);

/*                                                 
disable the display message utility
-----------------------------------
*/    
c_stoptm(); 

/*
close image files 
-----------------
*/
c_egclse( &gdesc);
}
