/******************************************************************************

NAME:	PROCESS

FUNCTION:  PROCESS converts a labeled table file to an ascii file

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1         08/89      B. Ailts     Fixed problem with string data on SUN-4
    5.2         08/89      B. Ailts     Fixed with reading a LT file that has
					not subfiles
    5.3         01/92	   T. Mittan    Fixed problem with reading atributes   
					which are only in subfiles.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Initialize variables
Locate the attribute names
Allocate buffers for the maximum matrix of the data types needed
Allocate the output buffers
Allocate the size of the output string to hold all the output columns
Open ascii output file
Read the first record
Set the initial value of save_data -- This value will be used when
 a tie column is specified to know when to read the next record of the
 main labeled table file
Do until EOF is reached
    Read each record of the input labeled table 
    Assign the string pointer to the beginning of the string buffer
    Compare the data of the previous record of the tie column with
     the current record.  If they are different read the next record
     of the main labled table file
    If there is a tie column, save the tie column data to be used
     as a check when the next record of the main labeled table
     file will be read
    Write each column according the the data type of each column
    Write out the record
    Read the next record
End do
Close the output ASCII file

******************************************************************************/

#include <stdio.h>
#include <string.h>
#ifdef unix
#include <sys/file.h>
#else
#include <file.h>
#endif

#include <fcntl.h>
#include "las.h"
#include "ltable.h"
#include "util.h"
#include "tab2asc.h"
#include "protos.h"

void process
(
    struct PARBLK *parb,		/*  Tae parameter block			*/
    struct TAB_DEF *tab,		/* output labeled table descriptor */
    char hosout[][CMLEN],		/*  host output name			*/
    long *outcol,			/* output columns specified */
    long *noutcol,			/* number of output columns specified */
    long *totcol,			/*  total number of columns to write */
    char atrname[][CMLEN],		/*  names of the specified attributes 	*/
    long *tiecol			/* tie column			*/
)

{ /* process */

struct VECTOR *ptr[MAXCOL];	/* pointer to current vector */
struct MATRIX *size;		/* matrix size */

union DATA *data;		/* data to be written to ASCII file	*/
union DATA *save_data;		/* data to be used as a test when a tie
				   column is specified			*/

long fatal = -1;		/*  fatal error code		*/
long flags;			/*  open flags				*/
long gstatus = 0;	/*  status of get_rec calls		*/
long i;				/* counter -- current column */
long j;				/* counter -- current file	*/
long k;				/*  counter -- current attribute value	*/
long nbytes;			/*  no. of bytes			*/
long nonfatal = 1;		/*  nonfatal error code			*/
long one = 1;			/*  value of 1				*/
long pmode = 0755;		/*  mode of open			*/
long files[50]; 		/*  label table files 			*/
long status;			/*  status of function calls		*/
long strsize;			/*  size of output string		*/
long temp;			/*  temporary storage variable		*/
long ntab;  			/*  number of lt tables to be used  	*/

int fd;				/* ascii file descriptor */

short found;			/*  found attribute logical		*/
short max_row = 0;		/*  Max no. of rows of all vectors	*/
short max_col = 0;		/*  Max no. of columns of all vectors	*/
short read_flag = FALSE;	/*  read main file flag			*/

char msg[ERRLEN];		/*  error message buffer		*/
char *strbuf;			/* output string buffer			*/
char *strptr;			/*  output string pointer		*/

/* Initialize variables
-----------------------*/
for (i = 0; i < *totcol; i++)
   ptr[i] = NULL;

/* Locate the attribute names
-----------------------------*/
for (i = 0; i < *noutcol; i++)
   {
   found = FALSE;

   for(j = tab[0].nsubs; (j >= 1) && (found == FALSE); j--)
      {
      ptr[outcol[i]] = (struct VECTOR *)get_field_ptr(&tab[j],atrname[i]);
      if (ptr[outcol[i]] != NULL)
         found = TRUE;
         files[i] = j;
      }

   if (!found)
      {
      ptr[outcol[i]] = (struct VECTOR *)get_field_ptr(&tab[0],atrname[i]);
      if (ptr[outcol[i]] != NULL)
         found = TRUE;
         files[i] = 0;
      }

   if (!found)
      {
      sprintf(msg,"Attribute %s could not be found",atrname[i]);
      c_errmsg(msg,"tab2asc-attribute",&nonfatal);
      c_errmsg("Fatal error encountered","tab2asc-fatal",&fatal);
      }
   }  /*  for i < *noutcol  */

/* sort the labled tables index array
------------------------------------*/
for ( i = 0; i < *noutcol; i++)
   for (j = 0; j < *noutcol - i - 1; j++)
      {
      if (files[j] > files[j+1])
         {
         temp = files[j];
         files[j] = files[j+1];
         files[j+1] = temp;
         }
       }
/* remove any duplicates
----------------------*/
for (ntab = 0,j = 1; j< *noutcol; j++)
    {
    if (files[ntab] == files[j])
      files[j] = 0;
    else
      {
      ntab++;
      files[ntab] = files[j];
      }
    }
      

/*  Allocate buffers for the maximum matrix of the data types needed
--------------------------------------------------------------------*/
for (i = 0; i < *noutcol; i++)
   {
   size = (struct MATRIX *)&ptr[outcol[i]]->lt_size;
   max_row = (short) LASMAX(max_row,size->nrow);
   max_col = (short) LASMAX(max_col,size->ncol);
   }  /*  for i < noutcol  */

/*  Allocate the output buffers
-------------------------------*/
data = (union DATA *) calloc(max_row * sizeof(union DATA),max_col);
save_data = (union DATA *) calloc(max_row * sizeof(union DATA),max_col);

/*  Allocate the size of the output string to hold all the output columns
-------------------------------------------------------------------------*/
strsize = (max_row + max_col) * *noutcol * MAXDIG;
strbuf = (char *) malloc(strsize);

/*  Open ascii output file
-------------------------*/
flags = O_WRONLY | O_TRUNC | O_APPEND | O_CREAT;
if ((fd = open(hosout[0],flags,pmode)) == -1)	/* Open the ASCII file        */
   {
   c_errmsg("Error opening output file","tab2asc-open",&nonfatal);
   close(fd);
   c_clean(parb,"tab2asc",hosout,&one);
   }

/*  Read the first record
-------------------------*/
for (i = 0; i <= tab[0].nsubs; i++)
   {
   gstatus = get_record(&tab[i]);
   if (gstatus == E_FAIL)	/* Read the labeled table file        */
      {
      c_errmsg("Error reading input file","tab2asc-read",&nonfatal);
      close(fd);
      c_clean(parb,"tab2asc",hosout,&one);
      }
   }

/* Set the initial value of save_data -- This value will be used when
   a tie column is specified to know when to read the next record of the
   main labeled table file
------------------------------------------------------------------------*/
if (*tiecol < MAXCOL + 1)
   {
   read_flag = FALSE;
   switch (ptr[outcol[*tiecol]]->dtype)
       {
       case C:     sprintf(&save_data->chr,"%s ",ptr[outcol[*tiecol]]->ptr.c);
                   break;
      case I1:     get_matrix(ptr[outcol[*tiecol]],I1,&save_data->byte);
                   break;
      case I2:     get_matrix(ptr[outcol[*tiecol]],I2,&save_data->i2);
                   break;
      case I4:     get_matrix(ptr[outcol[*tiecol]],I4,&save_data->i4);
                   break;
      case R4:     get_matrix(ptr[outcol[*tiecol]],R4,&save_data->r4);
                   break;
      case R8:     get_matrix(ptr[outcol[*tiecol]],R8,&save_data->r8);
                   break;
      case LTDATE: get_lt_date(ptr[outcol[*tiecol]],&save_data->date);
	           break;
      case LTTIME: get_lt_time(ptr[outcol[*tiecol]],&save_data->time);
                   break;
      } /*  switch  prt[outcol[*tiecol]].dtype  */
   }

/*  read each record of the input labeled table until EOF is reached
--------------------------------------------------------------------*/
do
   {
   /*  Assign the string pointer to the beginning of the string buffer
   ------------------------------------------------------------------*/
   strptr = strbuf;
   for ( i = 0; i < strsize; i++)
      strbuf[i] = ' ';

   /*  Compare the data of the previous record of the tie column with
       the current record.  If they are different read the next record
       of the main labled table file
   -------------------------------------------------------------------*/
   if (*tiecol < MAXCOL + 1)
      tie_comp(ptr,&read_flag,tiecol,data,save_data,outcol);

   if (read_flag)
      {
      gstatus = get_record(&tab[files[0]]);
      if (gstatus == E_FAIL)	/* Read the labeled table file        */
         {
         c_errmsg("Error reading input file","tab2asc-read",&nonfatal);
         close(fd);
         c_clean(parb,"tab2asc",hosout,&one);
         }
      /*  If there is a tie column, save the tie column data to be used
          as a check when the next record of the main labeled table
          file will be read
      ------------------------------------------------------------------*/
      if (*tiecol < MAXCOL + 1)
          {
         switch (ptr[outcol[*tiecol]]->dtype)
             {
             case C:     sprintf(&save_data->chr,"%s ",ptr[outcol[*tiecol]]->ptr.c);
                         break;
            case I1:     get_matrix(ptr[outcol[*tiecol]],I1,&data->byte);
                         break;
            case I2:     get_matrix(ptr[outcol[*tiecol]],I2,&save_data->i2);
                         break;
            case I4:     get_matrix(ptr[outcol[*tiecol]],I4,&save_data->i4);
                         break;
            case R4:     get_matrix(ptr[outcol[*tiecol]],R4,&save_data->r4);
                         break;
            case R8:     get_matrix(ptr[outcol[*tiecol]],R8,&save_data->r8);
                         break;
            case LTDATE: get_lt_date(ptr[outcol[*tiecol]],&save_data->date);
	                 break;
            case LTTIME: get_lt_time(ptr[outcol[*tiecol]],&save_data->time);
                         break;
            } /*  switch  prt[outcol[*tiecol]].dtype  */
         read_flag = FALSE;
         }  /*  if *tiecol < MAXCOL + 1  */
      else
         read_flag = TRUE;
      }

   /* write each column according the the data type of each column
   ---------------------------------------------------------------*/
   for (i = 0; i < *totcol; i++)
      {
      if (ptr[i] != NULL)
         {
         switch (ptr[i]->dtype)
	    {
	    case C: sprintf(strptr,"%s ",ptr[i]->ptr.c);
		    strptr += strlen(strptr);
	            break;
	    case I1: get_matrix(ptr[i],I1,&data->byte);
	   	     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
		        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
			   {
	                   sprintf(strptr,"%hd ",
				   *(&data->byte + (k * j) + k));
			   strptr += strlen(strptr);
			   }
	             break;
	    case I2: get_matrix(ptr[i],I2,&data->i2);
		     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
		        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
			   {
	                   sprintf(strptr,"%hd ",
				   *(&data->i2 + (k * j) + k));
			   strptr += strlen(strptr);
			   }
	             break;
	    case I4: get_matrix(ptr[i],I4,&data->i4);
		     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
		        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
			   {
	                   sprintf(strptr,"%d ",
				   *(&data->i4 + (k * j) + k));
			   strptr += strlen(strptr);
			   }
	             break;
	    case R4: get_matrix(ptr[i],R4,&data->r4);
	   	     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
		        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
			   {
	                   sprintf(strptr,"%f ",
				   *(&data->r4 + (k * j) + k));
			   strptr += strlen(strptr);
			   }
	             break;
	    case R8: get_matrix(ptr[i],R8,&data->r8);
		     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
		        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
			   {
	                   sprintf(strptr,"%f ",
			   *(&data->r8 + (k * j) + k));
			   strptr += strlen(strptr);
			   }
	             break;
	    case LTDATE: get_lt_date(ptr[i],&data->date);
		         sprintf(strptr,"%2d/%2d/%2d ",
				 data->date.month,data->date.day,
			         data->date.year);
			 strptr += strlen(strptr);
	                 break;
	    case LTTIME: get_lt_time(ptr[i],&data->time);
		         sprintf(strptr,"%2d:%2d:%2d ",
			         data->time.hour,data->time.minute,
				 data->time.second);
			 strptr += strlen(strptr);
	                 break;
	    } /*  switch  prt[i].dtype  */
         }  /* if ptr[i]!= NULL  */
      else
	 {
         sprintf(strptr,"0 ");
	 strptr += 2;
	 }
      }

   /*  Since the main labeled table file is read before this write is done
       this check is needed when there is only one labeled table file in
       order that the last record of the LT file is not written out twice
   ----------------------------------------------------------------------*/
   
   if(gstatus != E_EOF)
      {
      /*  Write out the record
      ------------------------*/
      sprintf(strptr,"\n");
      strptr ++;
      nbytes = strptr - strbuf;
      status = write(fd,strbuf,nbytes); /* Write the ASCII file        */
      if (status == -1)	
         {
         c_errmsg("Error writing output file","tab2asc-write",&nonfatal);
         close(fd);
         c_clean(parb,"tab2asc",hosout,&one);
         }

   /*  Read the record
   -------------------*/
   for (i = 1; i <= ntab; i++)
      {
      gstatus = get_record(&tab[files[i]]);
      if (gstatus == E_FAIL)	/* Read the labeled table file        */
         {
         c_errmsg("Error reading input file","tab2asc-read",&nonfatal);
         close(fd);
         c_clean(parb,"tab2asc",hosout,&one);
         }
       if (gstatus == E_EOF)
           break;
      }  /*  for i = 1 to tab.nsubs  */

   if (*tiecol == MAXCOL + 1)
      read_flag = TRUE;
      }  /*  if gstatus != E_EOF  */

   }  /* do while E_SUCC  */
 
while (gstatus == E_SUCC);

/* Free dynamic memory
----------------------*/
free (data);
free (save_data);
free (strbuf);

/*  Close the output ASCII file
-------------------------------*/
close(fd);
return;
} /* process */
