/******************************************************************************

NAME:	ANNOTATION

FUNCTION:
	This function copies data from a LT to an annotation GOF.  It 
        retrieves pointers to the specified LT attribute names (for 
	annotation number X value, Y value, annotation size, annotation
	angle and annotation text).  The header record of the GOF file is 
        written from the label vector of the input attribute LT file.  The
        attribute file may be the same file as the data file, it may be a 
        separate file or it may not exist at all.  For each data record in 
	the LT, the data is copied into the GOF annotation structure and 
	written to the GOF.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   	Get the pointers to the LT structure definitions for the attribute
          (which is optional) and data LT files.
        Make sure the specified attribute names exist in the LT.
        Get pointers into the LT label vector for the specified attribute names.
        Write the appropriate information to the GOF header record.
        Loop until there are no more records to process
	    If an attribute LT exists
		Copy the attribute information to the GOF.
	    Copy the annotation location, angle, size and text to the GOF.
     	Update the number of records in the GOF header.
     	Close the LT files.
 	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

#define current curent		/* Current row for processing message      */

void annotation
(
    int gof_fd,			/* GOF file descriptor (I) 	      */
    struct TAB_DEF *tab,			/* Master LT file structure (I)       */
    char *inattrib[],    	/* Array of attribute names to be 
					   processed (I)                      */
    long msgtim,			/* Number of seconds between messages
					   (I)				      */
    char *xname,         	/* X attribute name (I)               */
    char *yname,         	/* Y attribute name (I)               */
    char *lab_name,      	/* Label attribute name (I)           */
    char *num,           	/* Region, arc, point or annotation 
					   number attribute name (I)          */
    char *size_name,             /* Annotation size attribute name (I) */
    char *angle_name,            /* Annotation angle attribute name (I)*/
    char *annot_name,            /* Annotation string attribute name(I)*/
    char *attr_file,     	/* Attribute file type (I)            */
    char *data_file     	/* Data file type (I)                 */
)
{
extern	struct 	PARBLK	vblock;		/* TAE parameter block                */
extern	long	nout;			/* Number of output files created     */
extern 	long 	current;		/* Current record being processed     */
extern	char 	hostgof[1][CMLEN]; 	/* HOST name of GOF		      */

union   GOF 	antrec;			/* GOF annotation record structure    */
struct 	TAB_DEF *regatt;		/* LT attribute record structure      */
struct 	TAB_DEF *regxy;			/* LT data record structure 	      */
struct 	VECTOR 	*attrib[MAXATT];	/* List of LT attribute pointers      */
struct 	VECTOR 	*label;			/* Pointer to LT attribute LABEL      */
struct 	VECTOR 	*ptr_angle;		/* Pointer to LT attribute ANGLE      */
struct 	VECTOR 	*ptr_annotation;	/* Pointer to LT attribute ANNOT      */
struct 	VECTOR 	*ptr_size;		/* Pointer to LT attribute SIZE       */
struct 	VECTOR 	*ptr_x;			/* Pointer to LT attribute X          */
struct 	VECTOR 	*ptr_y;			/* Pointer to LT attribute Y          */

int addrec = FALSE;			/* GOF I/O flag 		      */
long dummy = -2;			/* Dummy value for processing messages*/
long flag = 2;				/* Type of processing message         */
int gof_type = ANOT;			/* GOF annotation file type 	      */
int length;				/* GOF record length parameter 	      */
long noabort = 1;			/* Report and continue flag for error 
					   messages                           */
long same = FALSE;   			/* Indicates whether the attributes 
					   and data values are in the same 
					   file                		      */
long status;				/* Status indicator		      */

char msg[ERRLEN];			/* Error message buffer 	      */

/*
   Retrieve the LT table structures for the attribute and data files.  The
   attribute LT file is optional.  Note that parameter "tab" is one of these
   table structures.
*/
get_lt_tabdef(tab,attr_file,data_file,&regatt,&regxy,gof_fd);
/*
   Set a flag indicating the attributes and data values are contained in the
   same file.
*/
if (regatt == regxy)
    same = TRUE;

/*
   Ensure the specified attribute names exist in the LT.
*/
check_lt_attrib("ANNOT",regxy,gof_fd,xname,yname,num,size_name,angle_name,
    annot_name);	

/*
   Search the LT label vector for the specified attribute names.
*/
ptr_size = get_field_ptr(regxy,size_name);
ptr_x = get_field_ptr(regxy,xname);
ptr_y = get_field_ptr(regxy,yname);
ptr_angle = get_field_ptr(regxy,angle_name);
ptr_annotation = get_field_ptr(regxy,annot_name);

/*
   Write attribute information to the GOF header record.
*/
put_header(gof_fd,regatt,inattrib,attrib,&label,lab_name,num);
lseek(gof_fd,0,2);
c_inittm("TAB2GOF","Processed","GOF annotation record","","",&dummy,&dummy,
        &msgtim,&flag);
current = 1;
c_startm();

/*
   Copy all annotation records from the LT to the GOF.
*/
for (current=0; ; current++)			
   {					
   if (regatt != NULL)
      {
/*
   Read a record from the attribute LT. 
*/
      status = get_record(regatt);
      if ((status != E_SUCC) && (status != E_EOF)) 
         {
	 c_gofcls(&gof_fd);
         c_clean(&vblock,"tab2gof",hostgof,&nout);
         }
      else if (status == E_EOF)
         break;
/*
   Copy attribute values to the GOF.
*/
      copy_attrib("ANNOT",attrib,label,&antrec,gof_fd,lab_name);
      }
/*
   Copy annotation data information to the GOF
*/
   if (!same)
      {
      status = get_record(regxy); 
      if ((status != E_SUCC) && (status != E_EOF)) 
         {
         c_gofcls(&gof_fd);
         c_clean(&vblock,"tab2gof",hostgof,&nout);
         }
      else if (status == E_EOF)
         break;
      }
   get_matrix(ptr_x,R4,(unsigned char *)&antrec.antrec.x);
   get_matrix(ptr_y,R4,(unsigned char *)&antrec.antrec.y);
   get_matrix(ptr_angle,R4,(unsigned char *)&antrec.antrec.angle);
   get_matrix(ptr_size,I4,(unsigned char *)&antrec.antrec.size);

/*
   Truncate the annotation string if it exceeds the DMS limit.
*/
   if (strlen(ptr_annotation->ptr.c) >= ANTLEN)
      {
      c_stoptm();
      sprintf(msg,"Annotation truncated to %d characters\n",ANTLEN-1);
      c_errmsg(msg,"tab2gof-anotlen",&noabort);
      *(ptr_annotation->ptr.c + ANTLEN - 1) = '\0';
      c_startm();
      }
   strcpy(antrec.antrec.antext,ptr_annotation->ptr.c);

/*
   Write the record to the GOF.
*/
   if (c_gofwr(&gof_fd,&length,&gof_type,(struct DDR *)NULL,&antrec, &addrec) != E_SUCC)
      {
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   }
c_stoptm();

/*
   Update the number of records in the GOF header
*/
update_header(gof_fd,current);

/*
   Close the LT files
*/
if (regatt != NULL)
   close_tab(regatt);
if (!same)
   close_tab(regxy);
return;
}
