/******************************************************************************

NAME:	COPY_ATTRIB

FUNCTION:
	This routine copies the specified attribute values from the LT to 
	the GOF file.  Input parameter ATTRIB is an array of LT attribute 
	pointers whose values are to be copied to the GOF.  The LT attribute 
	pointers are stored in the array such that the index is the same as 
	the GOF header attribute list.  If the attribute data type is some-
	thing other than ASCII, the attribute value is converted to ASCII.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	Assign pointers to labels and attributes based on the type of GOF
	  being processed.
	If a label exists
	    If the label is too long
		Print an informational message and truncate the label.
	    Copy the label into the GOF structure.
	For each possible attribute
	    If the attribute does not exist
		Set the attribute value to null
	    Else
		If the attribute data type is character
		    If the attribute value is too long
			Print an informational message and truncate the value.
		    Copy the attribute value into the GOF structure.
		If the attribute data type is I1 (one byte integer), I2 (two
		  byte integer), I4 (four byte integer), R4 (four byte floating
		  point), or R8 (double precision floating point)
		    Convert the attribute value to a character string and copy
		      it into the GOF structure.
	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

void copy_attrib
(
    char *ftype,		/* GOF file type being processed (I) 	      */
    struct VECTOR *attrib[],	/* Array of LT attribute pointers (I) 	      */
    struct VECTOR *label,		/* LT attribute pointer to LABEL (I) 	      */
    union GOF *gofrec,	/* GOF union for all file types (I) 	      */
    int gof_fd,		/* GOF file descriptor (I) 		      */
    char *lab_name      /* Label attribute name (I)                   */
)
{
extern	struct 	PARBLK	vblock;	/* TAE parameter block                        */
extern	long	nout;		/* Number of output files created	      */
extern	char 	hostgof[1][CMLEN]; /* HOST name of GOF			      */

long 	index;			/* Loop counter 			      */
long 	noabort = 1;		/* Report and continue flag for error messages*/

char 	msg[ERRLEN];		/* Error message buffer 		      */
char 	*ptr_label = NULL;	/* Pointer to label field within GOF union    */
char 	*ptr_attrib = NULL;	/* Pointer to attrb field within GOF union    */

/*
   Determine the GOF type and assign the label and attribute pointers
   to the appropriate field within the GOF structure.
*/
if ((strcmp(ftype,"POLY") == 0) || (strcmp(ftype,"LINE") == 0))
	{
	ptr_label = gofrec->plrec.label;
 	ptr_attrib = gofrec->plrec.attrb[0];
	}
else if (strcmp(ftype,"POINT") == 0)
	{
	ptr_label = gofrec->ptrec.label;
 	ptr_attrib = gofrec->ptrec.attrb[0];
	}
else if (strcmp(ftype,"ANNOT") == 0)
	{
	ptr_label = gofrec->antrec.label;
 	ptr_attrib = gofrec->antrec.attrb[0];
	}
/*
   If the attribute name for the graphics element label exists in the LT, copy 
   it to the GOF.
*/
if (label != NULL)
   {		
   if (label->dtype != C)
      {
      sprintf(msg,"LT attribute %s must have data type of character",
	lab_name);
      c_errmsg(msg,"tab2gof-char",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
/*
   Truncate the label if it exceeds the DMS limit.
*/
   if (strlen(label->ptr.c) >= LABLEN)
      {
      c_stoptm();
      sprintf(msg,"Label truncated to %d characters\n",
              LABLEN-1);
      c_errmsg(msg,"tab2gof-lablen",&noabort);
      *(label->ptr.c + LABLEN - 1) = '\0';
      c_startm();
      }
   strcpy(ptr_label,label->ptr.c);
   }
/*
   Copy the LT attributes into the GOF.  If an element of ATTRIB is NULL,
   it implies a null value is assigned to the GOF attribute.  Otherwise,
   attribute is converted to ASCII and copied to the GOF.
*/
for (index=0; index<MAXATT; index++)
   {				
   if (attrib[index] != NULL)
      {				
      if (attrib[index]->null)
         *(ptr_attrib+(index*ATTVAL)) = '\0';
      else				    
         {
         switch (attrib[index]->dtype)
            {
            case C:
   	      if (strlen(attrib[index]->ptr.c) >= ATTVAL)
	         {
		 c_stoptm();
	         sprintf(msg,
	            "Attribute truncated to %d characters\n", ATTVAL-1);
	         c_errmsg(msg,"tab2gof-attlen",&noabort);
	         *(attrib[index]->ptr.c + ATTVAL - 1) = '\0';
		 c_startm();
	         }
	      strcpy(ptr_attrib+(index*ATTVAL),attrib[index]->ptr.c);
	      break;
            case I1:  
	       sprintf(ptr_attrib+(index*ATTVAL),"%d",*attrib[index]->ptr.i1);
	       break;
	    case I2: 
	       sprintf(ptr_attrib+(index*ATTVAL),"%d",*attrib[index]->ptr.i2);
	       break;
            case I4: 
	       sprintf(ptr_attrib+(index*ATTVAL),"%d",*attrib[index]->ptr.i4);
	       break;
            case R4: 
	       sprintf(ptr_attrib+(index*ATTVAL),"%f",*attrib[index]->ptr.r4);
	       break;
            case R8: 
	       sprintf(ptr_attrib+(index*ATTVAL),"%f",*attrib[index]->ptr.r8);
	       break;
            }
         }
      }
   else
      *(ptr_attrib+(index*ATTVAL)) = '\0';
   }
return;
}
