/******************************************************************************

NAME:	POINT	

FUNCTION:
	This function copies data from a LT to a point GOF.  It retrieves 
 	pointers to the required LT attribute names for the X and Y values.  
	The header record of the GOF is written from the label vector of the 
	input attribute LT file.  For each data record in the LT, the data is 
	copied into the GOF point structure and written to the GOF.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	Get the LT structures for the attribute and data files.
	Make sure the LT contains the specified attributes.
	Get pointers to the X and Y attribute values.
	Write appropriate information to the GOF header record.
	Loop until end of file is reached
	    If an attribute data file exists
		Copy the attribute information to the GOF structure.
	    Read a record from the LT.
	    If end of file was detected
		Break from the loop.
	    Else if the read failed
		Print an error message and abort.
	    Copy the X,Y location into the GOF structure.
	    Write the record to the GOF.
	    If the write fails
		Print an error message and abort.
	Update the number of records in the GOF header record.
	Close the LT files.	
	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

#define current curent		/* Current row for processing message      */

void point
(
    int gof_fd,		/* GOF file descriptor (I) 		      */
    struct TAB_DEF *tab,		/* LT file structure (I) 		      */
    char *inattrib[],    /* Array of attribute names to be processed
                                   (I)                                        */
    long msgtim,		/* Number of seconds between processing 
				   messages (I)				      */
    char *xname,         /* X attribute name (I)                       */
    char *yname,         /* Y attribute name (I)                       */
    char *lab_name,      /* Label attribute name (I)                   */
    char *num,           /* Region, arc, point or annotation number
                                   attribute name (I)                         */
    char *attr_file,     /* Attribute file type (I)                    */
    char *data_file     /* Data file type (I)                         */
)
{
extern	struct 	PARBLK	vblock;	/* TAE parameter block                        */
extern	long	nout;		/* Number of output files created	      */
extern 	long 	current;	/* Current record being processed 	      */
extern	char 	hostgof[1][CMLEN]; /* HOST name of GOF			      */

union   GOF 	ptrec;		/* GOF structure for point data 	      */
struct 	TAB_DEF *regatt;	/* LT attribute file structure 		      */
struct 	TAB_DEF *regxy;		/* LT data file structure 	              */
struct 	VECTOR 	*attrib[MAXATT];/* Array of LT attribute pointers 	      */
struct 	VECTOR 	*label;		/* Pointer to LT attribute LABEL 	      */
struct 	VECTOR 	*ptr_x;		/* Pointer to LT attribute X 	              */
struct 	VECTOR 	*ptr_y;		/* Pointer to LT attribute Y 		      */

	int 	addrec = FALSE;	/* GOF boolean write flag 		      */
	long 	dummy = -2;	/* Dummy value for processing message utility */
	long	flag = 2;	/* Type of processing message to output	      */
	int 	gof_type = PNT;	/* GOF point data type                        */
	int 	length;		/* GOF record length                          */
	long 	same = FALSE;	/* Indicates whether the attributes and data
				   values are in the same file		      */
	long	status;		/* Status indicator			      */

/*
   Retrieve the LT table structures for the attribute and data files.  The
   attribute LT file is optional.  Note that parameter TAB is one of these
   table structures.
*/
get_lt_tabdef(tab,attr_file,data_file,&regatt,&regxy,gof_fd);
/*
   Set a flag indicating the attributes and data are contained in the same
   file.
*/
if (regatt == regxy)
    same = TRUE;

/*
   Make sure the specified attributes exist in the LT.
*/
check_lt_attrib("POINT",regxy,gof_fd,xname,yname,num,"","","");
ptr_x = get_field_ptr(regxy,xname);	
ptr_y = get_field_ptr(regxy,yname);

/*
   Write the appropriate information to the GOF header record.
*/
put_header(gof_fd,regatt,inattrib,attrib,&label,lab_name,num); 
/*
   Seek to the end of the GOF
*/
lseek(gof_fd,0,2);	
c_inittm("TAB2GOF","Processed","GOF point record","","",&dummy,&dummy,&msgtim,
   &flag);
current = 1;
c_startm();
for (current=0; ; current++)
   {
/*
   Copy the attribute information to the GOF.
*/
   if (regatt != NULL)
      {
      status =  get_record(regatt);
      if (status == E_EOF)
         break;
      else if (status != E_SUCC)
         {
         c_gofcls(&gof_fd);
	 c_clean(&vblock,"tab2gof",hostgof,&nout);
         }
      copy_attrib("POINT",attrib,label,&ptrec,gof_fd,lab_name);	
      }
/*
   Read a record from the LT.
*/
   if (!same)
      {
      status = get_record(regxy);
      if (status == E_EOF)
         break;
      else if (status != E_SUCC)
         {
         c_gofcls(&gof_fd);
         c_clean(&vblock,"tab2gof",hostgof,&nout);
         }
      }
/*
   Copy the X,Y location into the GOF structure.
*/
   get_matrix(ptr_x,R4,(unsigned char *)&ptrec.ptrec.x);
   get_matrix(ptr_y,R4,(unsigned char *)&ptrec.ptrec.y);
/*
   Write the record to the GOF.
*/
   if (c_gofwr(&gof_fd,&length,&gof_type, (struct DDR *)NULL, &ptrec, &addrec) != E_SUCC)
      {
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   }
/*
   Update the number of records in the GOF header.
*/
update_header(gof_fd,current);
c_stoptm();
/*
   Close the LT files.
*/
if (regatt != NULL)			
   close_tab(regatt);
if (!same)
   close_tab(regxy);
return;
}
