/******************************************************************************

NAME:	POLY_LINE	

FUNCTION:
	This function copies data from a LT to a polygon or line GOF.  It 
	retrieves pointers to the specified LT attribute names for region
	number or arc number, X value and Y value.  The header record of the 
	GOF is written from the label vector of the input attribute LT file.  
	For each data record in the LT, the data is copied into the GOF polygon
	or line structure and written to the GOF.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	Get the LT structures for the attribute and data files.
	Make sure the LT contains the specified attributes.
	Get pointers to the region or arc number, X value and Y value
	    attributes.
	Write appropriate information to the GOF header record.
	Loop until end of file is reached
	    If an attribute data file exists
		Copy the attribute information to the GOF structure.
   	    Read the X,Y points for the current line or polygon.
	    Write the record to the GOF.
	    If the write fails
		Print an error message and abort.
	Update the number of records in the GOF header record.
	Close the LT files.
	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

#define POLYPT 512		/* Number of points in a polygon for initial
				   allocation */
#define current curent		/* Current row for processing message      */

void poly_line
(
    int gof_fd,		/* GOF file descriptor (I) 		      */
    char *ftype,		/* GOF file type (POLY or LINE) (I) 	      */
    struct TAB_DEF *tab,		/* LT master file structure (I) 	      */
    char *inattrib[],	/* Array of attribute names to be processed
				   (I)  				      */
    long msgtim,		/* Number of seconds between processing 
				   messages (I)				      */
    char *xname,         /* X attribute name (I)                       */
    char *yname,         /* Y attribute name (I)                       */
    char *lab_name,      /* Label attribute name (I)                   */
    char *num,           /* Region, arc, point or annotation number 
				   attribute name (I)    		      */
    char *attr_file,     /* Attribute file type (I)                    */
    char *data_file     /* Data file type (I)                         */
)
{
extern	struct 	PARBLK	vblock;	/* TAE parameter block                        */
extern	long	nout;		/* Number of output files created	      */
extern 	long 	current;	/* Current record being processed 	      */
extern	char 	hostgof[1][CMLEN]; /* HOST name of GOF			      */

union   GOF  	plrec;		/* GOF polygon or line structure 	      */
struct 	TAB_DEF *regatt;	/* Attribute LT file structure 		      */
struct 	TAB_DEF *regxy;		/* Data LT file structure 		      */
struct 	VECTOR 	*attrib[MAXATT];/* Array of LT attribute pointers 	      */
struct 	VECTOR 	*label;		/* Pointer to LT attribute LABEL 	      */
struct 	VECTOR 	*ptr_reg;	/* Pointer to LT attribute REGNUM or ARCNUM   */
struct 	VECTOR 	*ptr_x;		/* Pointer to LT attribute X 		      */
struct 	VECTOR 	*ptr_y;		/* Pointer to LT attribute Y 		      */

	int 	addrec = FALSE;	/* GOF write boolean flag 		      */
	long 	cur_poly;	/* Current polygon (or line) being processed  */
	long 	dummy = -2;	/* Dummy variable for processing msg utility  */
	long	flag = 2;	/* Type of processing message to output	      */
	int 	gof_type = FPOLIN;
				/* GOF polygon or line file type 	      */
	int 	length;		/* GOF record length 			      */
	long 	lt_eof;		/* LT end of file flag 			      */
        long    same = FALSE;   /* Indicates whether the attributes and data
                                   values are in the same file                */
	long	status;		/* Status indicator			      */

long one = 1;
/*
   Retrieve the LT table structures for the attribute and data files.  The
   attribute LT file is optional.  Note that parameter TAB is one of these
   table structures.
*/
get_lt_tabdef(tab,attr_file,data_file,&regatt,&regxy,gof_fd);
if (regatt == regxy)
    same = TRUE;

/*
   Make sure the specified attributes exist in the LT.
*/
check_lt_attrib(ftype,regxy,gof_fd,xname,yname,num,"","","");
ptr_reg = get_field_ptr(regxy,num);
ptr_x = get_field_ptr(regxy,xname);
ptr_y = get_field_ptr(regxy,yname);

/*
   Write the appropriate information to the GOF header record.
*/
put_header(gof_fd,regatt,inattrib,attrib,&label,lab_name,num);
/*
   Seek to end of the GOF.
*/
lseek(gof_fd,0,2);
if (strcmp(ftype,"POLY") == 0)
   c_inittm("TAB2GOF","Processed","GOF polygon record","","",&dummy,&dummy,
           &msgtim,&flag);
else
   c_inittm("TAB2GOF","Processed","GOF line record","","",&dummy,&dummy,
           &msgtim,&flag);
current = 1;
c_startm();

/*
   Read the first record from the data LT file.
*/
lt_eof = get_record(regxy);
for (current = 0,cur_poly = -1; lt_eof == FALSE; current++)
   {
/*
   Copy the attribute information to the GOF.
*/
   if (regatt != NULL)
      {
      if (!same)
         {
         status = get_record(regatt);
         if (status == E_EOF)
            break;
         else if (status != E_SUCC)
            {
            c_gofcls(&gof_fd);
            c_clean(&vblock,"tab2gof",hostgof,&nout);
            }
	 }
      copy_attrib(ftype,attrib,label,&plrec,gof_fd,lab_name);
      }
/*
   Get the X,Y points for the current line or polygon.
*/
    plrec.plrec.x = (float *)malloc(POLYPT * sizeof(float));
    plrec.plrec.y = (float *)malloc(POLYPT * sizeof(float));
    if ((plrec.plrec.x == NULL) || (plrec.plrec.x == NULL))
       {
       c_errmsg ("Error allocating dynamic memory","tab2gof-alloc",&one);
       c_gofcls(&gof_fd);
       c_clean(&vblock,"tab2gof",hostgof,&nout);
       }
   lt_eof = get_xy_points(regxy,ptr_reg,ptr_x,ptr_y,&cur_poly,&plrec.plrec,gof_fd);
/*
   Write the record to the GOF.
*/
   if (c_gofwr(&gof_fd,&length,&gof_type, (struct DDR *)NULL, &plrec, &addrec) != E_SUCC)
      {
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   free(plrec.plrec.x);
   free(plrec.plrec.y);
   }
/*
   Update the number of records in the GOF header.
*/
update_header(gof_fd,current);
c_stoptm();
/*
   Close the LT files.
*/
if (regatt != NULL)
   close_tab(regatt);
if (!same)
   close_tab(regxy);
return;
}
