/******************************************************************************

NAME:	PUT_HEADER	

FUNCTION:
	PUT_HEADER extracts information out of the LT attribute file and 
	copies it to the GOF header record.  If the LT attribute file does 
	not exist,  attribute information will not be written to the GOF.  
	Parameter INATTRIB specifies which attribute names are to be 
	copied.  The function returns an array of LT attribute pointers that 
	are to be copied with parameters ATTRIB and LABEL.  These 
	parameters are used by the function COPY_ATTRIB.

	If an attribute name to be processed already exists in the GOF,
	error checking is done to ensure the attribute in the LT and in 
	the GOF have the same data types.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	Initialize attribute pointers.
	If no attribute LT exists, return to the calling routine.
	Read the GOF header record.
	Based on the type of file being processed, retrieve the attribute
	  for region number, arc number, point number or annotation number.
	Retrieve the label attribute.
	For each attribute in the attribute LT
	    If the attribute is region number, arc number, point number, 
	      annotation number or label
		Get the next attribute (the attribute names above have
                special meaning and are not to be entered in the GOF as GOF
                attributes).
	    If the user specified which attributes are to be processed, make
	      sure they exist in the LT.
	    If the attribute name is too long, truncate it, print an 
	      informational message and continue processing.
	    For the maximum number of attributes or until the attribute is
	      found in the GOF
		If the current GOF attribute is not defined
		    Save the index in case an attribute needs to be added.
		If the current GOF attribute is the same as the LT attribute
		    If the data types of the attributes don't match
			Print an error message and abort. 
	    If the attribute does not currently exist in the GOF
		If the maximum number of attributes would be exceeded by adding
		  the new attribute
		    Print an error message and abort.
		Add the attribute name and data type to the GOF header
		  structure.
	Write the header record to the GOF.
	Abort if the write fails.
	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

void put_header
(
    int gof_fd,			/* GOF file descriptor (I) 	      */
    struct TAB_DEF *regatt,		/* Attribute LT structure (I)         */
    char *inattrib[],		/* Array of attribute names (I)       */
    struct VECTOR *attrib[],		/* Array of LT attribute pointers (O) */
    struct VECTOR **label,		/* Pointer to LT attribute LABEL (O)  */
    char *lab_name,              /* Label attribute name (I)           */
    char *num			/* Region, arc, point or annotation
					   number (I)			      */
)
{
extern	struct 	PARBLK	vblock;		/* TAE parameter block                */
extern	long	nout;			/* Number of output files created     */
extern	char 	hostgof[1][CMLEN]; 	/* HOST name of GOF		      */

struct 	VECTOR 	*regnum;		/* Pointer to LT attribute REGNUM,
					   ARCNUM, PNTNUM or ANNOTNUM         */
struct 	VECTOR 	*vec;			/* Pointer to an LT attribute         */
static 	union 	GOF 	gofrec;		/* GOF union for all file types       */

	int 	addrec = FALSE;		/* GOF write boolean flag             */
	long 	found;			/* Boolean indicating attribute found */
	long 	free;			/* Free GOF attribute index           */
	int 	gof_type = HDR;		/* GOF header record to be written    */
	long 	index;			/* Loop counter 		      */
	int 	length;			/* GOF record length 		      */
	long 	noabort = 1;		/* Report and continue flag for error 
					   messages                           */
	int 	obyte;			/* Number of bytes read from GOF      */
	long 	slen;			/* String length 		      */
	long 	status;			/* Function return status code        */

	char 	msg[ERRLEN];		/* Error message buffer               */

/*
   Initialize attribute pointers to NULL.
*/
for (index=0; index<MAXATT; index++)
   attrib[index] = NULL;
/*
   If no attribute LT exists, nothing else to do.
*/
if (regatt == NULL)
   return;
/*
   Read the GOF header record.
*/
status = c_gofrd(&gof_fd,&obyte,&length,&gof_type,(struct DDR *)NULL,&gofrec);
if (status != E_SUCC)
   {
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }
/*
   Retrieve the appropriate attribute value to identify unique records.
*/
regnum = get_field_ptr(regatt,num);
/*
   Get the value of the label attribute.
*/
*label = get_field_ptr(regatt,lab_name);
for (vec=regatt->vector; vec!=NULL; vec=vec->next)
   {
/*
   Don't process region number/arc number or label again.
*/
   if ((vec == regnum) || (vec == *label))
      continue;
   slen = (long) strlen(vec->label);
   c_low2up(vec->label,&slen);
/*
   If the user specified what attributes are to be processed, make sure
   they exist in the LT.
*/ 
   if (inattrib[0] == NULL)
      found = TRUE;
   else 
      found = FALSE;
   for (index=0; found==FALSE; index++)
      {
      if (inattrib[index] ==  NULL)
         break;
      slen = (long) strlen(inattrib[index]);
      c_low2up(inattrib[index],&slen);
      if (strcmp(inattrib[index],vec->label) == 0)
         found = TRUE;
      }
/*
   If the attribute was not found, try the next one
*/
   if (found == FALSE)
      continue;
/*
   If the attribute name is too long, truncate it and continue processing.
*/
   if (strlen(vec->label) >= ATTNAM)
      {
      sprintf(msg,
              "Attribute \"%s\" truncated to %d characters",
              vec->label,ATTNAM-1);
      c_errmsg(msg,"tab2gof-attnam",&noabort);
      *(vec->label + ATTNAM - 1) = '\0';
      }
   free = -1;
/*
   If the attribute currently exists in the GOF, make sure the data
   type is the same between the LT and GOF.
*/
   for (index=0,found=FALSE; (index<MAXATT) && (found==FALSE); index++)
      {
      if ((gofrec.hdrec.mask[index] == 0) && (free == -1))
         free = index;
      if ((gofrec.hdrec.mask[index] != 0) &&
          (strcmp(gofrec.hdrec.attrb[index],vec->label) == 0))
         {
         found = TRUE;
         attrib[index] = vec;
         sprintf(msg,"Attribute name \"%s\" %s",vec->label,
                 "has different data types between GOF and LT");
         switch (vec->dtype)
            {
            case C:
		if (strcmp(gofrec.hdrec.attyp[index],"CHA") != 0)
	           {
                   c_errmsg(msg,"tab2gof-dtype",&noabort);
   		   c_gofcls(&gof_fd);
   		   c_clean(&vblock,"tab2gof",hostgof,&nout);
	           }
		break;
	    case I1:
	    case I2:
	    case I4: 
		if (strcmp(gofrec.hdrec.attyp[index],"INT") != 0)
	           {
                   c_errmsg(msg,"tab2gof-dtype",&noabort);
   		   c_gofcls(&gof_fd);
   		   c_clean(&vblock,"tab2gof",hostgof,&nout);
	           }
		break;
	    case R4:
	    case R8:
		if (strcmp(gofrec.hdrec.attyp[index],"REA") != 0)
                   {
                   c_errmsg(msg,"tab2gof-dtype",&noabort);
   		   c_gofcls(&gof_fd);
   		   c_clean(&vblock,"tab2gof",hostgof,&nout);
                   }
		break;
            }
         }
      }
/*
   If adding a new attribute ...
*/
   if (found == FALSE)
      {
/*
   Error if the maximum number of attributes has been exceeded
*/
      if (free == -1)
	 {
         c_errmsg("Maximum number of attributes exceeded","tab2gof-maxatt",
	    &noabort);
   	 c_gofcls(&gof_fd);
         c_clean(&vblock,"tab2gof",hostgof,&nout);
	 }
/*
   Add the attribute name and data type to the GOF header structure.
*/
      attrib[free] = vec;
      gofrec.hdrec.mask[free] = strlen(vec->label);
      strcpy(gofrec.hdrec.attrb[free],vec->label);
      switch (vec->dtype)
         {
         case C:  strcpy(gofrec.hdrec.attyp[free],"CHA");	break;
	 case I1:
	 case I2:
	 case I4: strcpy(gofrec.hdrec.attyp[free],"INT");	break;
	 case R4:
	 case R8: strcpy(gofrec.hdrec.attyp[free],"REA");	break;
         }
      }
   }
lseek(gof_fd,0,0);
/*
   Write the header record to the GOF.
*/
if (c_gofwr(&gof_fd,&length,&gof_type, (struct DDR *)NULL, &gofrec, &addrec) != E_SUCC)
   {
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }
return;
}
