/*****************************************************************************
*
*  NAME: GENRST_POL
*
*  FUNCTION: Generate the image for polygon data
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*     3       10/17/91  T. Mittan       CSB     Added DDR information
*                                               to DDR.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "typlim.h"
#include "tab2ras.h"
#include "util.h"
#include "imageio.h"
#include "pixman.h"
#include "protos.h"

void genrst_pol
(
    char tempfile[][CMLEN],     /* I: temp file names */
    long nlines,                /* I: number of lines in output image */
    long nsamps,                /* I: number of samples in output image */
    double *maxval,             /* I: maximum pixel value */
    double *minval,             /* I: minimum pixel value */
    long msgtim,                /* I: time for processing message */
    long *backgrnd,             /* I: back ground value */
    long mosaic,                /* I: mosaic flag */
    double *nw,                 /* I: north west corner */
    double *ne,                 /* I: north east corner */
    double *sw,                 /* I: south west corner */
    double *se,                 /* I: south east corner */
    float *pixsiz               /* I: pixel size */
)

{
    extern struct PARBLK parblk;

    extern int fdesc[2];     /* image file descriptors used for image i/o */
    extern int gdesc;

    extern long curcat;
    extern long curent;
    extern long offset[2];    /* offsets into "thebuf" for output bands    */


    FILE *srtscnfp;                 /* File descriptor for sorted scan file */

    struct DDR ddr;
    struct BDDR bddr;

    char *thebuf;                   /* image buffer */      
    long access = IWRITE;           /* file access */
    long bufsiz;                    /* image i/o buffer size */
    long color;                     /* pixel value */
    long curlin;                    /* current processing line */
    long cursamp;                   /* current sample */
    long dtype = ELONG;             /* data type */
    long i;                         /* loop counter */
    long line;                      /* line number to update */
    long *lthebuf;                  /* image buffer */
    long *savebuf;                  /* saved image buffer */

    long msgflg = GEN1;             /* flag for processing message */
    long nbytes = REC_SIZE;         /* number of bytes to read from file */
    long ncells;                    /* number of image pixels to fill */
    long nonfatal = 1;              /* for nonfatal error messages */
    long numfile = NUMTEMPF;        /* number of temporary files */
    long odtype;                    /* data type of the image */
    long one = 1;                   /* constant of one */
    long sample;                    /* starting sample locaion */
    long status;                    /* function status */
    long odtemp[2];                 /* keys for eopens */
    long zero = 0;                  /* constant of zero */

    double start;                   /* starting sample locaion */

    char *tempptr;                  /* character pointer */
    char temp[REC_SIZE];            /* buffer for file read */

    /* Determine the data type for the las image
       --------------------------------------------*/
    if ((*minval >= MINUTINY) && (*maxval <= MAXUTINY))
        odtype = EBYTE;
    else if ((*minval >= MINWORD) && (*maxval <=  MAXWORD ))
        odtype = EWORD;
    else
        odtype = ELONG;

    /* Open the sorted scan file
       ----------------------------*/
    if ((srtscnfp = fopen(tempfile[SORTSCAN],"r")) == NULL)
    {
        c_errmsg("Error opening sorted scan file","tab2ras-open",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }

    /* Open the output image
       -----------------------*/
    odtemp[0] = odtype;
    odtemp[1] = 0;
    status = c_eopens((struct FDESC **)&fdesc[0],tempfile[OUTIMAGE],&one,&one,
                       &one,&nlines,&nsamps,offset,&access,&dtype,&zero,odtemp);
    if (status != E_SUCC)
        c_clean(&parblk,"tab2ras",tempfile,&numfile);

    fdesc[1] = 0;

    /* Allocate i/o buffer
       ----------------------*/
    thebuf = c_ealloc((struct FDESC **)fdesc,&bufsiz);
    if (thebuf == NULL)
    {
        c_errmsg("Error allocating dynamic memory","tab2ras-alloc",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }
    lthebuf = (long *)thebuf;

    /* Allocate save buffer
       -----------------------*/
    savebuf = (long *)malloc(nsamps * sizeof(long));
    if (savebuf == NULL)
    {
        c_errmsg("Error allocating dynamic memory","tab2ras-alloc",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }

    /* Group the image
       ------------------*/
    status = c_egroup((struct FDESC **)fdesc,(struct GDESC **)&gdesc,
            (unsigned char *)thebuf,&bufsiz);
    if (status != E_SUCC)
        c_clean(&parblk,"tab2ras",tempfile,&numfile);

    /* Initialize the processing message 
       ------------------------------------*/
    cursamp = 0;
    curent = 1;
    c_inittm("TAB2RAS","Processed","line"," "," ",&nlines,&zero,
             &msgtim,&msgflg);
    c_startm();

    /* Locate the las image buffer for the 1st line
       -----------------------------------------------*/
    status = c_estep((struct GDESC **)&gdesc);
    if (status != E_SUCC)
        c_clean(&parblk,"tab2ras",tempfile,&numfile);

    /* Fill the line with the back ground value
       -------------------------------------------*/
    c_pxfill((unsigned char *)(lthebuf + offset[0]),&dtype,&nsamps,
            (unsigned char *)backgrnd);

    /* Get the 1st record from the sorted scan file.  If there 
       are no records, all data must be outside the image.
       ---------------------------------------------------------*/
    tempptr = temp;
    if (fgets(temp,nbytes,srtscnfp) == NULL)
    {
        c_errmsg("No vector data occurs in the output image","tab2ras-empty",
                &nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }
    line = atol(tempptr);
    tempptr += INTLEN;
    sscanf(tempptr,"%lf",&start);
    sample = (long) start;
    tempptr += DBLLEN;
    color = atol(tempptr);

    /* Generate any required preliminary background 
       lines that occur before the first vector data
       ----------------------------------------------------*/
    for (curlin = 1; curlin < line; curlin++)
    {
        /* Fill the next line with the back ground value
           -----------------------------------------------*/
        status = c_estep((struct GDESC **)&gdesc);
        if (status != E_SUCC)
            c_clean(&parblk,"tab2ras",tempfile,&numfile);
        c_pxfill((unsigned char *)(lthebuf + offset[0]),&dtype,&nsamps,
                (unsigned char *)backgrnd);
        curent++;
    }

    curlin = line;

    /* Do for each record of the sorted scan file
       ---------------------------------------------*/
    while (curlin <= nlines)
    {
        if (curlin < line)
        {
            /* Fill the next line with the back ground value.
               Save the last line if requested.
               -----------------------------------------------*/
            if (mosaic == 1)
                c_pxcopy((unsigned char *)lthebuf + offset[0],
                        (unsigned char *)savebuf,&dtype,&nsamps);
            status = c_estep((struct GDESC **)&gdesc);
            if (status != E_SUCC)
                c_clean(&parblk,"tab2ras",tempfile,&numfile);
            c_pxfill((unsigned char *)(lthebuf + offset[0]),&dtype,&nsamps,
                    (unsigned char *)backgrnd);
            curent++;
            curlin++;
            cursamp = 0;
        }
        else
        {
            /* Load the next segment of the current image line.
               --------------------------------------------------------*/
            /*if ((sample > cursamp) && (cursamp < nsamps))*/
            if ((cursamp < nsamps))
            {
                /* Clip at the image edge
                   -------------------------*/
                if ((sample + mosaic) > nsamps)
                    sample = nsamps - mosaic;
                if (cursamp == 0)
                {
                    ncells = 1 + mosaic;
                    cursamp = sample;
                }
                else
                    ncells = (sample + mosaic) - cursamp;
                c_pxfill((unsigned char *)(lthebuf + offset[0] + cursamp - 1),
                        &dtype,&ncells,(unsigned char *)&color);
                cursamp = sample;
            }

            /* Get the next record from the sorted scan file
               ------------------------------------------------*/
            tempptr = temp;
            if (fgets(temp,nbytes,srtscnfp) == NULL)
                break;
            line = atol(tempptr);
            tempptr += INTLEN;
            sscanf(tempptr,"%lf",&start);
            sample = (long) start;
            tempptr += DBLLEN;
            color = atol(tempptr);
        } 
    }

    /* Apply the mosaic option  to the last image line with data 
       ------------------------------------------------------------*/
    if (mosaic == 1)
    {
        for (i = 0; i < nsamps; i++)
            if (lthebuf[offset[0] + i] == *backgrnd)
                lthebuf[offset[0] + i] = savebuf[i]; 
    }

    /* Generate any required background lines that occur 
       after the last vector data
       ----------------------------------------------------*/
    while (curlin <= nlines)
    {
        /* Fill the next line with the back ground value
           -----------------------------------------------*/
        status = c_estep((struct GDESC **)&gdesc);
        if (status != E_SUCC)
            c_clean(&parblk,"tab2ras",tempfile,&numfile);
        c_pxfill((unsigned char *)(lthebuf + offset[0]),&dtype,&nsamps,
                (unsigned char *)backgrnd);
        curlin++;
        curent++;
    }

    /* Close the image  
       ------------------*/
    status = c_egclse((struct GDESC **)&gdesc);
    if (status != E_SUCC)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    c_stoptm();

    /* Check for the back ground value as a minimum or maximum image value
       ---------------------------------------------------------------------*/
    *maxval = max (*maxval, *backgrnd);
    *minval = min (*minval, *backgrnd);

    /* Close and delete the sorted scan file
       ----------------------------------------*/
    fclose (srtscnfp);
    c_delfil (tempfile[SORTSCAN]);

    /* update the ddr
       ---------------*/
    if (c_getbdr(tempfile[OUTIMAGE], &bddr,&one) != E_SUCC)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    if (c_getddr(tempfile[OUTIMAGE], &ddr) != E_SUCC)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    ddr.upleft[0] = nw[0];
    ddr.upleft[1] = nw[1];
    ddr.loleft[0] = sw[0];
    ddr.loleft[1] = sw[1];
    ddr.upright[0] = ne[0];
    ddr.upright[1] = ne[1];
    ddr.loright[0] = se[0];
    ddr.loright[1] = se[1];
    ddr.valid[DDCCV] = 1; /* validate corners */

    ddr.pdist_y = (double) pixsiz[0];
    ddr.pdist_x = (double) pixsiz[1];
    ddr.valid[DDPDV] = 1;  /* validate increment */

    ddr.line_inc = (double) 1;
    ddr.sample_inc = (double) 1;
    ddr.valid[DDINCV] = 1;  /* validate increment */

    strcpy((ddr.proj_units),"meters");
    ddr.valid[DDPUV] = 1;

    if( c_putddr(tempfile[OUTIMAGE], &ddr) == E_FAIL)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    if( c_putbdr(tempfile[OUTIMAGE], &bddr) == E_FAIL)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    /* Free the buffers
       -------------------*/
    free(thebuf);
    free(savebuf);

    return;
}
