/*****************************************************************************
*
*  NAME: GET_CORNERS
*
*  FUNCTION: Read the corner coordinates from the labeled table and calculate
*            the number of lines an samples in the output image if necessary
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*     3       10/17/91  T. Mittan       CSB     Calculate corners correctly
*                                               for DDR
*     4       12/06/91  T. Mittan       CSB     modified corner calculation
*     5       04/28/93  T. Mittan       CSB     Allowed for line weights in
*                                               corner calculation
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include  "protos.h"

void get_corners
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    long *tab_ind,              /* I: tab index */
    struct VECTOR *ptr[],       /* I:  */
    double *corners,            /* I/O: corner coordinates */
    long *nl,                   /* I/O: no. of lines in the output image */
    long *ns,                   /* I/O: no. of samples in the output image */
    float *pixsiz,              /* I: pixel size */
    double *nw,                 /* O: northwest corner */
    double *ne,                 /* O: northeast corner */
    double *sw,                 /* O: southwest corner */
    double *se,                 /* O: southeast corner */
    double *offset,             /* O: offset */
    double *rawynw              /* O: northwest corner */
)

{
    long fatal = -1;                        /* for fatal error messages */
    long status;                            /* function status */


    *offset = 0;
    /* If the corner coordinates were entered, nothing needs to be done
       -------------------------------------------------------------------*/
    if (ptr[CORN1] == NULL)
    {
        nw[0] = corners[0];
        nw[1] = corners[1];
        ne[0] = corners[0];
        ne[1] = corners[1] + (((float)*ns - 1) * pixsiz[1]);
        sw[0] = corners[0] - (((float)*nl - 1) * pixsiz[0]);
        sw[1] = corners[1];
        se[0] = sw[0];
        se[1] = ne[1];
        *rawynw = corners[0];
        return;
    }
    else
    {
        /* Read the corners from the labled table and calculate 
           the number of lines and samples in the image
           ------------------------------------------------------*/
        status = get_record(&tab[tab_ind[CORN1]]);
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
        get_matrix(ptr[CORN1],R8,(unsigned char *)corners);
        if (ptr[CORN2] != NULL)
        {
            if (ptr[CORN4] != NULL)
            {
                get_matrix(ptr[CORN2],R8,(unsigned char *)&corners[1]);
                get_matrix(ptr[CORN3],R8,(unsigned char *)&corners[2]);
                get_matrix(ptr[CORN4],R8,(unsigned char *)&corners[3]);
            }
            else
                get_matrix(ptr[CORN2],R8,(unsigned char *)&corners[2]);
        }
        *rawynw = corners[0];

        /* This will round the corners out to a multiple of the pixel size
           --------------------------------------------------------------*/
        *offset = ((long)((corners[0] +1.5* pixsiz[0])/pixsiz[0])) * pixsiz[0] -
            (corners[0] + .5*pixsiz[0]);

        corners[0] = ((long)((corners[0] +1.5 * pixsiz[0])/ pixsiz[0])) 
                      * pixsiz[0];
        corners[1] = ((long)((corners[1] - 1.5 * pixsiz[1])/ pixsiz[1])) 
                      * pixsiz[1];
        corners[2] = ((long)((corners[2] - (.5  * pixsiz[0]))/
                pixsiz[0])) * pixsiz[0];

        corners[3] = ((long)((corners[3] +  (1.5) 
                * pixsiz[1])/ pixsiz[1])) * pixsiz[1];

        *nl = (long)((corners[0] - corners[2]) / pixsiz[0] + 1);
        *ns = (long)((corners[3] - corners[1]) / pixsiz[1] + 1);
        nw[0] = corners[0];
        nw[1] = corners[1];
        ne[0] = corners[0];
        ne[1] = corners[3];
        sw[0] = corners[2];
        sw[1] = corners[1];
        se[1] = corners[3];
        se[0] = corners[2];
    } 

    return;
}
