/*****************************************************************************
*
*  NAME: GETPAR
*
*  FUNCTION: Get the input parameters
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include "las.h"
#include "tab2ras.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char ltin[][CMLEN],         /* O: labeled tabel file names */
    char *hosout,               /* O: output host image name */
    char *taeout,               /* O: output tae image name */
    char atrname[][CMLEN],      /* O: attribute names */
    long *pixval,               /* O: pixel value */
    long *nl,                   /* O: no. of lines in output images */
    long *ns,                   /* O: no. of samples in output images */
    long *backgrnd,             /* O: background value */
    float *pixsiz,              /* O: pixel size  */
    double *cornval,            /* O: cornes values */
    long *msgtim,               /* O: time for processing message */
    long *listflg,              /* O: flag for error messages */
    long *nfile,                /* O: number of labeled table files */
    char *subcmd,               /* O: subcommand */
    long *mosaic,               /* O: mosaic flag */
    char *datpath               /* O: data path  */
)

{

    extern struct PARBLK parblk;

    struct VARIABLE *var;

    long access = IREAD;            /* file access value */
    long i;                         /* loop counter */
    long len;                       /* string length */
    long status;                    /* function status */
    long fatal = -1;                /* key for fatal error messages */
    long nonfatal = 1;              /* key for nonfatal error messages */

    char flg[4];                    /* list flag */

    /* Get the subcommand 
       ---------------------*/
    var = p_find(&parblk,"_subcmd");
    strcpy(subcmd,SVAL(*var,0));

    /*  Receive parameters naming the labeled table,
        and the attributes that are to be used
        ------------------------------------------------*/
    status = c_getcnt(&parblk,"INLT",nfile);
    for (i = 0; i < *nfile; i++)
    {
        status = c_getnam(&parblk,"INLT",&i,ltin[i],"lt",&access);
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
    }

    /*  Receive the  name for the image that is to be generated
        ----------------------------------------------------------*/
    status = c_getout(&parblk,"OUT",(char (*)[256])taeout,
                     (char (*)[256])hosout);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);

    /*  Initialize the atrname array to be null terminated if no string is
        copied into the array
        ----------------------------------------------------------------------*/
    for (i = 0; i < NUMATR; i++)
        strcpy(atrname[i],"\0");

    /*  Copy the attribute names from the parameter block to the attribute name
        array.  If PIXVAL is specified, the PXVALATR attribute is ignored.
        ---------------------------------------------------------------------*/
    var = p_find(&parblk,"PIXVAL");
    if (var->v_count != 0)
        *pixval = IVAL(*var,0);
    else
    {
        if (strncmp(subcmd,"POLYGON",7) != 0)
        {
            var = p_find(&parblk,"PXVALATR");
            if (var->v_count != 0)
                strcpy(atrname[COLOR],SVAL(*var,0));
            else
            {
                c_errmsg("Either PIXVAL or PXVALATR must be specified",
                        "tab2ras-pixel1",&nonfatal);
                c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
            }
        }
        else
        {
            /* Left and Right region numbers are used to deterimine the
               output pixel value.  Use the prior and color index into 
               the array to make use of read_lt.
               ------------------------------------------------------------*/
            var = p_find(&parblk,"LEFTATR");
            if (var->v_count != 0)
                strcpy(atrname[COLOR],SVAL(*var,0));
            else
            {
                c_errmsg("Either PIXVAL or LEFTATR and RGHTATR must be "
                         "specified", "tab2ras-pixel2",&nonfatal);
                c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
            }
            var = p_find(&parblk,"RGHTATR");
            if (var->v_count != 0)
                strcpy(atrname[PRIOR],SVAL(*var,0));
            else
            {
                c_errmsg("Either PIXVAL or LEFTATR and RGHTATR must be "
                         "specified", "tab2ras-pixel",&nonfatal);
                c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
            }
        }
    }

    /*  PTID and IDCOOR are the values that tie the coordinates to the other 
        values in the labeled tables.  They should be the same.
        -------------------------------------------------------------------*/
    var = p_find(&parblk,"PTIDATR");
    strcpy (atrname[PTID],SVAL(*var,0));
    strcpy (atrname[IDCOOR],SVAL(*var,0));

    /* If INDEXATR is not specified, PTID is used to link the id attribute
       to the other attributes
       -----------------------------------------------------------------------*/
    var = p_find(&parblk,"PTIDATR");
    var = p_find(&parblk,"INDEXATR");
    if (var->v_count != 0)
        strcpy (atrname[INDEX],SVAL(*var,0));
    else
        strcpy (atrname[INDEX],atrname[PTID]);

    var = p_find(&parblk,"COORSATR");
    if (var->v_count != 0)
    {
        strcpy (atrname[COOR1],SVAL(*var,0));
        if (var->v_count > 1)
            strcpy (atrname[COOR2],SVAL(*var,1));
    }

    /* Get the attributes sepcific to -POINT and -LINE
       -------------------------------------------------*/
    if (strncmp(subcmd,"POLYGON",7) != 0)
    {
        var = p_find(&parblk,"PRIORATR");
        strcpy (atrname[PRIOR],SVAL(*var,0));
    }

    /* If CORNVALS is specified, CORNATR is ignored.  
       NL and NS must be specified if CORNVALS is.
       ----------------------------------------------*/
    var = p_find(&parblk,"CORNVALS");
    if (var->v_count != 0)
    {
        for (i = 0; i < var->v_count; i++)
            cornval[i] = (double) RVAL(*var,i);

        var = p_find(&parblk,"NL");
        if (var->v_count != 0)
        {
            *nl = IVAL(*var,0);
            if (*nl <= 0)
            {
                c_errmsg("NL must be greater than zero","tab2ras-nl",&nonfatal);
                c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
            }
        }
        else 
        {
            c_errmsg("NL must be specified","tab2ras-nl",&nonfatal);
            c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
        }
        var = p_find(&parblk,"NS");
        if (var->v_count != 0)
        {
            *ns = IVAL(*var,0);
            if (*ns <= 0)
            {
                c_errmsg("NS must be greater than zero","tab2ras-ns",&nonfatal);
                c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
            }
        }
        else 
        {
            c_errmsg("NS must be specified","tab2ras-ns",&nonfatal);
            c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
        }
    }
    else
    {
        /* The corner attributes may under one, two, or
           four different attributes 
           -----------------------------------------------*/
        var = p_find(&parblk,"CORNATR");
        if (var->v_count != 0)
        {
            strcpy (atrname[CORN1],SVAL(*var,0));
            if (var->v_count == 2)
                strcpy (atrname[CORN2],SVAL(*var,1));
            else if (var->v_count == 4)
            {
                strcpy (atrname[CORN2],SVAL(*var,1));
                strcpy (atrname[CORN3],SVAL(*var,2));
                strcpy (atrname[CORN4],SVAL(*var,3));
            }
        }
        else
        {
            c_errmsg("Either CORNATR or CORNVALS must be specified",
                    "tab2ras-corn",&nonfatal);
            c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
        }
    }

    /* Get the attribute specific to -LINE 
       --------------------------------------*/
    if (strncmp(subcmd,"LINE",4) == 0)
    {
        var = p_find(&parblk,"LNWGTATR");
        if (var->v_count != 0)
            strcpy (atrname[LNWGT],SVAL(*var,0));
    }

    /* Get the attribute specific to -LINE and -POLYGON
       ---------------------------------------------------*/
    if (strncmp(subcmd,"POINT",5) != 0)
    {
        var = p_find(&parblk,"SEQNOATR");
        strcpy (atrname[SEQN],SVAL(*var,0));
    }

    /* Get ddr specific information
       -----------------------------*/
    var = p_find(&parblk,"PRJPMATR");
    if (var->v_count != 0)
    {
        strcpy (atrname[PPARMS],SVAL(*var,0));
    }
    var = p_find(&parblk,"PRJCDATR");
    if (var->v_count != 0)
    {
        strcpy (atrname[PCODE],SVAL(*var,0));
    }
    var = p_find(&parblk,"ZONEATR");
    if (var->v_count != 0)
    {
        strcpy (atrname[PZONE],SVAL(*var,0));
    }
    var = p_find(&parblk,"DCODEATR");
    if (var->v_count != 0)
    {
        strcpy (atrname[PDATUM],SVAL(*var,0));
    }
    var = p_find(&parblk,"UNITSATR");
    if (var->v_count != 0)
    {
        strcpy (atrname[UNITS],SVAL(*var,0));
    }
    var = p_find(&parblk,"VALIDATR");
    if (var->v_count != 0)
    {
        strcpy (atrname[PVALID],SVAL(*var,0));
    }

    /* Make all attributes upper case
       --------------------------------*/
    for (i = 0; i < NUMATR; i++)
    {
        len = (long)strlen(atrname[i]);
        c_low2up(atrname[i],&len);
    }

    /* Get the pixel size and back ground value
       ------------------------------------------*/
    var = p_find(&parblk,"PIXSIZ");
    if (var->v_count != 0)
        for (i = 0; i < var->v_count; i++)
            pixsiz[i] = RVAL(*var,i);
    var = p_find(&parblk,"BACKGRND");
    *backgrnd = IVAL(*var,0);

    /* Get the flag that specifies if error messages
       will be displayed if data is out of bounds 
       -----------------------------------------------*/
    var = p_find(&parblk,"LISTFLG");
    strcpy (flg,SVAL(*var,0));
    if (strncmp(flg,"YES",3) == 0)
        *listflg = TRUE;
    else
        *listflg = FALSE;

    /* Get the parameter specific to -POLYGON
       -----------------------------------------*/
    if (strncmp(subcmd,"POLYGON",7) == 0)
    {
        var = p_find(&parblk,"MOSCFLG");
        strcpy (flg,SVAL(*var,0));
        if (strncmp(flg,"YES",3) == 0)
            *mosaic = 1;
    }

    /* Get message time 
       ------------------*/
    var = p_find(&parblk,"$MSGTIM");
    *msgtim = IVAL(*var,0);

    /* Get data path 
       --------------*/
    datpath = (char *)getenv(DATAPATH);

    return;
}
