/*****************************************************************************
*
*  NAME: POLYSCAN
*
*  FUNCTION: Calculate the line/sample location for the line segment end
*            points of the polygon and write them to the scan file
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*     3.      12/06/91  T. Mittan       CSB     Corrected problem of leaking lines.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include "protos.h"

void polyscan 
(
    FILE *scanfp,               /* I/O: file descriptor for scan file */

    long ptr,                   /* I: pixel value for the right region */ 
    long ptl,                   /* I: pixel value for the left region */ 
    double x1,                  /* I: first x coordinate */
    double y1,                  /* I: first y coordinate */
    double x2,                  /* I: second x coordinate */
    double y2,                  /* I: second y coordinate */
    double *minval,             /* I: minimum pixel value */
    double *maxval,             /* I: maximum pixel value */
    float *pixsiz,              /* I: pixel size */
    long nlines,                /* I: no. of lines in output image */
    long nsamps,                /* I: no. of samples in output image */
    long *listflg,              /* I: flag for printing error messages */
    double xnw,                 /* I: X coord. of north west corner of image */
    double ynw,                 /* I: Y coord. of north west corner of image */
    double offset1,             /* I: offset */
    double rawynw               /* I: Y coord. of north west corner of image */
)
{

    double  dbl1 = 0.1e1          /* double 1.0 */
        , offset = 0.1e-1         /* double 0.01 */
        , xdelta                  /* rate of change in x*/
        , xhi                     /* x coordinate */
        , xi                      /* x intercept line */
        , xlo                     /* x coordinate */
        , xr                      /* pixel location in running line */
        , yhi                     /* x coordinate */
        , yi                      /* y coordinate in running line */
        , ylo;                    /* y coordinate */

    long nonfatal = 1;              /* for nonfatal error messages */
    long one = 1;                   /* constant of one */   
    long ptw;                       /* pixel value to the west */

    long rasthi                    /* x endpoint */
        ,rastlo                    /* x endpoint */
        , scanhi                   /* y endpoint */
        , scanlo                   /* y endpoint */
        , yscani;                  /* y endpoint */

    unsigned char useseg;           /* flag if line is used */ 
    char msg[ERRLEN];               /* buffer for error messages */

    /* Ignore horizontal cases
       --------------------------*/
    if (abs(y2 - y1) <= 0.00000001)
        return;

    /* Ignore near horizontal cases that do not intersect a scan line
       -----------------------------------------------------------------*/
    if (abs(y2 - y1) <  pixsiz[0])
    {
        scanhi = (long)(((rawynw - y1) / pixsiz[0]) + .5);
        scanlo = (long)(((rawynw - y2) / pixsiz[0]) + .5);
        if (scanlo == scanhi)
            return;
    }

    /* Determine the orientation of the line segment
       -----------------------------------------------*/
    if (y1 < y2 )
    {
        /* The line segment rises
           ------------------------*/
        xlo = x1;
        ylo = y1;
        xhi = x2;
        yhi = y2;
        ptw = ptl;
    }
    else 
    {
        /* The line segment falls
           ------------------------*/
        xlo = x2;
        ylo = y2;
        xhi = x1;
        yhi = y1;
        ptw = ptr;
    }

    /* Calculate the range of scan lines to be scanned
       --------------------------------------------------*/
    scanhi = (long) (((ynw - yhi - offset1) / pixsiz[0] ) + dbl1);
    scanlo = (long) (((ynw - ylo - offset1) / pixsiz[0] ) + dbl1);

    /* Calculate the range of pixels to be scanned
       ----------------------------------------------*/
    rasthi = (long) (((xhi - xnw) / pixsiz[1] ) + .5);
    rastlo = (long) (((xlo - xnw) / pixsiz[1] ) + .5);

    /* Validate the range of scan lines to be scanned.  Line segments
       that lie entirely above or below the image are ignored.  Line 
       seqments that line partially above or below the image are included.
       -----------------------------------------------------------------*/
    if (!*listflg)
    {
        if (scanhi > nlines )
            return;
        else if (scanlo < one )
            return;
    }
    else
    {
        if (scanhi > nlines )
        {
            sprintf(msg,"%s %s %d","Data lies below the output image.",
                    "Data would occur at line",scanlo);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
        else if (scanlo < one )
        {
            sprintf(msg,"%s %s %lf","Data lies above the output image.",
             "The y coordinate of the northwest corner should be adjusted to ",
             yhi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
        if (scanhi < one )
        {
            sprintf(msg,"%s %s %lf","Data lies above the output image.",
              "The y coordinate of the northwest corner should be adjusted to ",
              yhi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
        if (scanlo > nlines )
        {
            sprintf(msg,"%s %s %d", "Data lies below the output image.",
                    "Data would occur at line",scanlo);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
    }

    /* Validate the range of pixels to be scanned.  Line segments that line 
       entirely to the left or right of the image are ignored.  Line segments
       that line partially to the left or right of the image are included.
       ----------------------------------------------------------------------*/
    if (!*listflg)
    {
        if (rasthi < one )
            if (rastlo < one )
                return;
            else if (rasthi > nsamps )
                if (rastlo > nsamps )
                    return;
    }
    else
    {
        if (rasthi > nsamps )
        {
            sprintf(msg,"%s %s %d",
                    "Data lies to the right of the output image.",
                    "Data would occur at sample",rasthi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            if (rastlo > nsamps )
            {
                sprintf(msg,"%s %s %d",
                        "Data lies to the right of the output image.",
                        "Data would occur at sample",rasthi);
                c_errmsg(msg,"tab2ras-outside",&nonfatal);
                return;
            }
        }
        else if (rasthi < one )
        {
            sprintf(msg,"%s %s %lf",
            "Data lies to the left of the output image.",
              "The x coordinate of the northwest corner should be adjusted to ",             xhi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            if (rastlo < one )
            {
                sprintf(msg,"%s %s %lf",
                  "Data lies to the left of the output image.",
              "The x coordinate of the northwest corner should be adjusted to ",               xhi);
                c_errmsg(msg,"tab2ras-outside",&nonfatal);
                return;
            }
        }
        if (rastlo > nsamps )
        {
            sprintf(msg,"%s %s %d",
                    "Data lies to the right of the output image.",
                    "Data would occur at sample",rasthi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
        else if (rastlo < one )
        {
            sprintf(msg,"%s %s %lf", 
              "Data lies to the left of the output image.",
              "The x coordinate of the northwest corner should be adjusted to ",
              xhi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
    }

    /* Scan line segments that intersect multiple scan lines.
       Initialize the scan processing for this line segment
       -------------------------------------------------------*/
    useseg = FALSE;

    /* Calculate the initial y intercept for this line segment
       ---------------------------------------------------------*/
    yscani = scanhi;
    yi = (ynw - ((double) yscani * pixsiz[0]) - offset1) + offset;

    /* Calculate the initial x intercept for this line segment (xi)
       -----------------------------------------------------------*/
    if ((yhi - ylo) == 0)
        xdelta = 0.0;
    else
        xdelta = (( xhi - xlo ) / ( yhi - ylo));
    xi = xhi - (xdelta * ( yhi - yi));

    /* Convert xi from coordinate units to pixel units
       --------------------------------------------------*/
    xr = (( xi - xnw ) / pixsiz[1]) + .5;

    /* Do for each scan line intersected by this line segment
       --------------------------------------------------------*/
    do
    {
        if ( yscani <= nlines )
        {
            /* Determine if the scan line segment intersects the image window
               ---------------------------------------------------------------*/
            if ( yscani >= one)
            {
                if (xr >= 1.0)
                {
                    fprintf(scanfp,"%10d %16.7f %10d\n", yscani, xr, ptw);
                    useseg = TRUE;
                }
            }
        }

        yscani++;
        xr -= xdelta;
        yi -= pixsiz[0];
    }
    while ( yscani < scanlo );

    /* Determine if ptw is a minimum or maximum value
       ----------------------------------------------------*/
    if (useseg)
    {
        *maxval = max (*maxval, ptw);
        *minval = min (*minval, ptw);
    }

    return;
}
