/*****************************************************************************
*
*  NAME: SCANI
*
*  FUNCTION: Calculate the line/sample location for the line segment end
*            points and write them to the scan file
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*     3       10/17/91  T. Mittan       CSB     Fixed problems with line weight
*     4       12/06/91  T. Mittan       CSB     Modified error messages.
*     5.0     02/07/92  T. Mittan       CSB     Corrected problems with line 
*                                               placement
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"

void scani 
(
    FILE *scanfp,               /* I: file descriptor for scan file */

    long pixval,                /* I: pixel value for the line */ 
    double x1,                  /* I: first x coordinate */
    double y1,                  /* I: first y coordinate */
    double x2,                  /* I: second x coordinate */
    double y2,                  /* I: second y coordinate */
    long linwt,                 /* I/O: weight of the line */
    double *minval,             /* I: minimum pixel value */
    double *maxval,             /* I: maximum pixel value */
    float *pixsiz,              /* I: pixel size */
    long nlines,                /* I: no. of lines in output image */
    long nsamps,                /* I: no. of samples in output image */
    long *listflg,              /* I: flag for printing error messages */
    double xnw,                 /* I: X coord. of north west corner of image */
    double ynw,                 /* I: Y coord. of north west corner of image */
    long *prior                 /* I: priority of the line */
)
{

    double dbl0 = 0.0             /* double 0.0 */
        , dbl1 = 0.1e1            /* double 1.0 */
        , offset = 0.1e-1         /* double 0.01 */
        , xdelta                  /* rate of change in x*/
        , xhi                     /* x coordinate */
        , xi                      /* x intercept line */
        , xlo                     /* x coordinate */
        , xr                      /* pixel location in running line */
        , yhi                     /* x coordinate */
        , yi                      /* y coordinate in running line */
        , ylo;                    /* y coordinate */

    long i;                        /* loop counter */
    long nonfatal = 1;             /* for nonfatal error messages */
    long one = 1;                  /* constant of one */   
    long eflag = FALSE;            /* even lnwgt flag */

    long rasthi                    /* x endpoint */
        ,rastlo                    /* x endpoint */
        , scanhi                   /* y endpoint */
        , scanlo                   /* y endpoint */
        , xstart                   /* x endpoint */
        , skipflag = FALSE         /* flag to prevent writing data to a file */
        , temp                     /* temp value */
        , xstop                    /* x endpoint */
        , yscani                   /* y endpoint */
        , zero = 0;                /* constant of zero */

    unsigned char useseg;           /* flag if line is used */
    char msg[ERRLEN];               /* buffer for error messages */


    /* Determine the orientation of the line segment
       -----------------------------------------------*/
    if (y1 < y2)
    {
        /* The line segment rises
           ------------------------*/
        ylo = (double) y1;
        yhi = (double) y2;
        xlo = (double) x1;
        xhi = (double) x2;
    }
    else if (y1 > y2)
    {
        /* The line segment falls
           ------------------------*/
        ylo = (double) y2;
        yhi = (double) y1;
        xlo = (double) x2;
        xhi = (double) x1;
    }
    else if (x1 <= x2)
    {
        /* The level line segment runs from left to right
           ------------------------------------------------*/
        xlo = (double) x1;
        ylo = (double) y1;
        xhi = (double) x2;
        yhi = (double) y2;
    }
    else
    {
        /* Level line segment runs from right to left
           --------------------------------------------*/
        xlo = (double) x2;
        ylo = (double) y2;
        xhi = (double) x1;
        yhi = (double) y1;
    }

    /* Calculate line weight
       ----------------------*/
    if (((linwt % 2) == 0) && (linwt != 0))
    {
        linwt = linwt / 2;
        eflag = TRUE;
    }
    else
        if (linwt != 0)
            linwt = (linwt - 1) / 2;

    /* Calculate the slope (xdelta) 
       -----------------------------*/
    if (yhi != ylo)
        xdelta = ((xhi - xlo) / (yhi - ylo));
    else
        xdelta = 0.0;

    /* set start/stop points acording to linwt
       ----------------------------------------*/
    if (eflag == TRUE)
    { 
        /* Calculate the range of scan lines to be scanned
           --------------------------------------------------*/
        if ((xdelta > 9) || (xdelta < -9))
        {
            scanhi = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) - linwt ;
            scanlo = (long) (((ynw - ylo) / pixsiz[0]) + .49 +  dbl1) + linwt  
                      - 1;
        }
        else
            /* Lengthen the bottom of almost vertical lines 
               ---------------------------------------------*/
            if ((xdelta < 1 ) && (xdelta > -1))
            {
                scanhi = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) - linwt;
                scanlo = (long) (((ynw - ylo) / pixsiz[0]) + dbl1) + linwt - 1;
            }
            else
                /* Add offset to prevent turncation errors
                   ----------------------------------------*/
                if (yhi != ylo)
                {
                    scanhi = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) - linwt;
                    scanlo = (long) (((ynw - ylo) / pixsiz[0]) + .49 + dbl1) 
                             + linwt  - 1;
                }
        /* For horizontal lines find starting point and add width to find 
           ending point
           -----------------------------------------------------------------*/
        if (yhi == ylo)
        {
            scanlo = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) + linwt  - 1;
            scanhi = scanlo - linwt - linwt + 1;
        }

        /* Calculate the range of pixels to be scanned
           ----------------------------------------------*/
        rasthi = (long) (((xhi - xnw) / pixsiz[1])  + dbl1) ;
        rastlo = (long) (((xlo - xnw) / pixsiz[1])  + dbl1) ;
        if (rasthi >= rastlo)
        {
            rastlo -= linwt;
            rasthi += linwt -1;
        }
        else
        {
            rasthi -= linwt;
            rastlo += linwt - 1;
        }
    }
    else
    {  
        /* Calculate the range of scan lines to be scanned
           -------------------------------------------------*/
        if ((xdelta > 9) || (xdelta < -9))
        {
            scanhi = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) - linwt;
            scanlo = (long) (((ynw - ylo) / pixsiz[0]) + .49 +  dbl1) + linwt;
        }
        else
            /* Lengthen the bottom of almost vertical lines 
               ---------------------------------------------*/
            if ((xdelta < 1) && (xdelta > -1))
            {
                scanhi = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) - linwt;
                scanlo = (long) (((ynw - ylo) / pixsiz[0]) + dbl1)+ linwt;
            }
            else
                /* Add offset to prevent turncation errors
                   ----------------------------------------*/
                if (yhi != ylo)
                {
                    scanhi = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) - linwt;
                    scanlo = (long) (((ynw - ylo) / pixsiz[0]) + .49 + dbl1)
                             + linwt;
                }
        /* For horizontal lines find starting point and add width to find 
           ending point
           --------------------------------------------------------------*/
        if (yhi == ylo)
        {
            scanhi = (long) (((ynw - yhi) / pixsiz[0]) + dbl1) - linwt;
            scanlo = scanhi + 2*linwt;
        }

        /* Calculate the range of pixels to be scanned
           ----------------------------------------------*/
        rasthi = (long) (((xhi - xnw) / pixsiz[1])  + dbl1) ;
        rastlo = (long) (((xlo - xnw) / pixsiz[1])  + dbl1) ;
        if (rasthi >= rastlo)
        {
            rasthi += linwt;
            rastlo -= linwt;
        }
        else
        {
            rasthi -= linwt;
            rastlo += linwt;
        }
    }

    /* Validate the range of scan lines to be scanned.  Line segments
       that lie entirely above or below the image are ignored.  Line 
       seqments that lie partially above or below the image are included.
       -----------------------------------------------------------------*/
    if (!*listflg)
    {
        if (scanhi > nlines)
            return;
        else if (scanlo < one)
            return;
    }
    else
    {
        if (scanhi > nlines)
        {
            sprintf(msg,"%s %s %d","Data lies below the output image.",
                    "Data would occur at line",scanlo);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
        else if (scanlo < one)
        {
            sprintf(msg,"%s %s %lf","Data lies above the output image.",
              "The y coordinate of the northwest corner should be adjusted to ",
              yhi+ (float)linwt * pixsiz[0]);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
        if (scanhi < one)
        {
            sprintf(msg,"%s %s %lf","Data lies above the output image.",
              "The y coordinate of the northwest corner should be adjusted to ",
               yhi+ (float)linwt*pixsiz[0]);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
        if (scanlo > nlines)
        {
            sprintf(msg,"%s %s %d", "Data lies below the output image.",
                    "Data would occur at line",scanlo);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
    }

    /* Validate the range of pixels to be scanned.  Line segments that lie 
       entirely to the left or right of the image are ignored.  Line segments
       that lie partially to the left or right of the image are included.
       ----------------------------------------------------------------------*/
    if (!*listflg)
    {
        if (rasthi < one)
            if (rastlo < one)
                return;
            else if (rasthi > nsamps)
                if (rastlo > nsamps)
                    return;
    }
    else
    {
        if (rasthi > nsamps)
        {
            sprintf(msg,"%s %s %d",
                    "Data lies to the right of the output image.",
                    "Data would occur at sample",rasthi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            if (rastlo > nsamps )
            {
                sprintf(msg,"%s %s %d",
                        "Data lies to the right of the output image.",
                        "Data would occur at sample",rasthi);
                c_errmsg(msg,"tab2ras-outside",&nonfatal);
                return;
            }
        }
        else if (rasthi < one)
        {
            sprintf(msg,"%s %s %lf",
              "Data lies to the left of the output image.",
              "The x coordinate of the northwest corner should be adjusted to ",
              xhi -(float)linwt*pixsiz[1]);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            if (rastlo < one)
            {
                sprintf(msg,"%s %s %lf",
                   "Data lies to the left of the output image.",
              "The x coordinate of the northwest corner should be adjusted to ",
                xlo -(float)linwt*pixsiz[1]);
                c_errmsg(msg,"tab2ras-outside",&nonfatal);
                return;
            }
        }
        if (rastlo > nsamps)
        {
            sprintf(msg,"%s %s %d",
                    "Data lies to the right of the output image.",
                    "Data would occur at sample",rasthi);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
        else if (rastlo < one)
        {
            sprintf(msg,"%s %s %lf",
              "Data lies to the left of the output image.",
              "The x coordinate of the northwest corner should be adjusted to ",
                xlo- (float)linwt*pixsiz[1]);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
        }
    }

    /* Line segments that intersect only 1 line
       -------------------------------------------*/
    if ((((scanlo - linwt) == (scanhi + linwt)) && (eflag == FALSE))|| 
            ((eflag == TRUE) && ((scanlo - linwt - linwt + 1) == scanhi)))
    {
        /* Determine the start and stop pixels for this scan line
           --------------------------------------------------------*/
        if (rastlo < rasthi)
        {
            xstart = rastlo ;
            xstop  = rasthi; 
        }
        else
        {
            xstart = rasthi;
            xstop  = rastlo;
        }

        /* Adjust out of bounds line segment limits to the image window
           ---------------------------------------------------------------*/
        if (xstart < one)
            xstart = one;
        if (xstop > nsamps)
            xstop = nsamps;

        /* Log this scan line segment to the scan file
           ------------------------------------------------*/
        for (i = scanhi; i <= scanlo; i++)
            fprintf(scanfp,"%10d %10d %10d %10d %11d\n",
                    i,*prior,xstart,xstop,pixval);

        /* Determine if pixval is a minimum or maximum value
           -----------------------------------------------------*/
        *maxval = min (*maxval, pixval);
        *minval = min (*minval, pixval);
        return;
    }

    /* Scan line segments that intersect multiple scan lines.
       Initialize the scan processing for this line segment
       -------------------------------------------------------*/
    useseg = FALSE;

    /* Calculate the initial y intercept for this line segment
       ---------------------------------------------------------*/
    yscani = scanhi;
    yi = (ynw - ((double) (yscani)  * pixsiz[0])) + offset;

    /* Calculate the initial x intercept for this line segment (xi)
       -----------------------------------------------------------*/
    xi = xhi - (xdelta * (yhi - yi));

    /* Convert xi from coordinate units to pixel units
       --------------------------------------------------*/
    xr = ((xi - xnw) / pixsiz[1]) + dbl1;

    /* Do for each scan line intersected by this line segment
       --------------------------------------------------------*/
    do
    {
        if (yscani <= nlines)
        {
            /* adjust for even linwt
               ----------------------*/
            if (eflag == TRUE)
            { 
                /* Determine the scan line segment limits
                   -----------------------------------------*/
                if (xdelta < dbl0)
                {
                    if (xdelta < -7)
                    {
                        xstart = (long)  (xr + linwt * xdelta) - linwt;
                        xstop  = (long)  (xr - linwt * xdelta) + linwt + 1;
                    }
                    else
                        if (xdelta < -2)
                        {
                            xstart = (long) (xr + .75 * linwt * xdelta) - linwt;
                            xstop  = (long) (xr - .25 * linwt * xdelta) + linwt 
                            + 1;
                        }
                        else
                        {
                            if ((long) xr < rasthi)
                                xr = rasthi ;
                            else
                                if  ((long) xr > rastlo) 
                                    xr = rastlo ;
                            xstart = (long) (xr  + xdelta) - linwt;
                            xstop  = (long) (xr  + linwt) - 1 ;
                        }
                    if (((xstart <= rasthi) && (xstop <= rasthi) 
                          && (xdelta < - 7)) || 
                          ((xstart >= rastlo) && (xstop >= rastlo) 
                          && (xdelta < - 7))) 
                    {
                        skipflag = TRUE;
                    }
                    else
                    {
                        if (xstart < rasthi)
                            xstart = rasthi;
                        if (xstop > rastlo)
                            xstop = rastlo;
                        if (xstop < rasthi)
                            xstop = rasthi;
                        if (xstart > rastlo)
                            xstart = rastlo;
                    }
                }
                else
                {
                    if (xdelta > 7)
                    {
                        xstart = (long) (xr  - linwt * xdelta) - linwt;
                        xstop  = (long) (xr  + linwt * xdelta) + linwt - 1;
                    }
                    else
                        if (xdelta > 2)
                        {
                            xstart = (long) (xr - .25 * linwt * xdelta) - linwt;
                            xstop  = (long) (xr + .75 * linwt * xdelta) + linwt 
                                     - 1;
                        }
                        else
                        {
                            if ((long) xr > rasthi)
                                xr =  rasthi ;
                            else
                                if ((long) xr < rastlo)
                                    xr = rastlo ;
                            xstart = (long) (xr) - linwt;
                            xstop  = (long) (xr + xdelta) + linwt - 1;
                        }
                    if (((xstart >= rasthi) && (xstop >= rasthi) && 
                        (xdelta > 7)) || ((xstart <= rastlo) && 
                        (xstop <= rastlo) && (xdelta > 7))) 
                    {
                        skipflag = TRUE;
                    }
                    else
                    {
                        if (xstop > rasthi)
                            xstop = rasthi;
                        if (xstart < rastlo)
                            xstart = rastlo;
                        if (xstart > rasthi)
                            xstart = rasthi; 
                        if (xstop < rastlo)
                            xstop = rastlo;
                    }
                }
            }
            else /* odd line weight */
            { 
                /* Determine the scan line segment limits
                   -----------------------------------------*/
                if (xdelta < dbl0)
                {
                    if (xdelta < -7)
                    {
                        xstart = (long) (xr + (linwt + 1 ) * xdelta) - linwt ;
                        xstop  = (long) (xr - (linwt * xdelta)) + linwt ;
                    }
                    else
                        if (xdelta < -1)
                        {
                            xstart = (long)(xr + (.45 * (linwt + 2.3) * xdelta))
                                     - linwt ;
                            xstop  =(long) (xr - .75 * linwt * xdelta) + linwt ;
                        }
                        else
                        {
                            if (((long) xr < rasthi) && (linwt > 0))
                                xr = rasthi ;
                            else 
                                if  ((long) xr > rastlo && (linwt > 0))
                                    xr = rastlo ;
                            xstart = (long) (xr + xdelta - linwt);
                            xstop  = (long) (xr + linwt);
                        }
                    if (((xstart <= rasthi) && (xstop <= rasthi) 
                            && (xdelta < - 7)) || ((xstart >= rastlo) 
                            && (xstop >= rastlo) && (xdelta < - 7))) 
                    {
                        skipflag = TRUE;
                    }
                    else
                    {
                        if (xstart < rasthi)
                            xstart = rasthi;
                        if (xstop > rastlo)
                            xstop = rastlo;
                        if (xstop < rasthi)
                            xstop = rasthi;
                        if (xstart > rastlo)
                            xstart = rastlo;
                    }
                }
                else
                {
                    if (xdelta > 7)
                    {
                        xstart = (long) (xr - (linwt * xdelta)) - linwt;
                        xstop  = (long) (xr + (linwt + 1) * xdelta) +linwt ;
                    }
                    else
                        if (xdelta > 1)
                        {
                            xstart =(long) (xr - .75 * linwt * xdelta) - linwt ;
                            xstop  =(long) (xr + (.45 * (linwt + 2.3) * xdelta))
                                    +linwt ;
                        }
                        else
                        {
                            if (((long) xr > rasthi) && (linwt > 0))
                                xr =  rasthi ;
                            else
                                if (((long) xr < rastlo) && (linwt > 0))
                                    xr = rastlo ;
                            xstart = (long)  (xr - linwt);
                            xstop  = (long)  (xr + xdelta + linwt);
                        }
                    if (((xstart >= rasthi) && (xstop >= rasthi)
                            && (xdelta > 7)) || ((xstart <= rastlo) && 
                            (xstop <= rastlo) && (xdelta > 7))) 
                    {
                        skipflag = TRUE;
                    }
                    else
                    {
                        if (xstop > rasthi)
                            xstop = rasthi;
                        if (xstart < rastlo)
                            xstart = rastlo;
                        if (xstart > rasthi)
                            xstart = rasthi; 
                        if (xstop < rastlo)
                            xstop = rastlo;
                    }
                }
            }
            /* Determine if the scan line segment intersects the image window
               ---------------------------------------------------------------*/
            if (yscani > zero)
            {
                if (xstart <= nsamps) 
                {
                    if (xstop > zero)
                    {
                        if (xstart < one)
                            xstart = one;
                        if (xstop > nsamps)
                            xstop = nsamps;

                        if (xstart > xstop)
                        {
                            temp = xstart;
                            xstart = xstop;
                            xstop = temp;
                        }
                        if (!skipflag)
                        {
                            fprintf(scanfp,"%10d %10d %10d %10d %11d\n", yscani, 
                                    *prior, xstart, xstop, pixval);
                            useseg = TRUE;
                        }
                    }
                }
            }
            skipflag = FALSE;
            xr -= xdelta;
        }

        yscani++;
        yi -= pixsiz[0];
    }
    while (yscani <= scanlo);

    /* Determine if pixval is a minimum or maximum value
       ----------------------------------------------------*/
    if (useseg)
    {
        *maxval = max (*maxval, pixval);
        *minval = min (*minval, pixval);
    }

    return;
}
