/*****************************************************************************
*
*  NAME: TAB2RAS
*
*  FUNCTION: Convert polygon, line, or point vector data stored in a
*            labeled table to a raster image
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*     3       12/30/92  D. Etrheim      CSB     Remove DATAPATH from system sort
*     4       04/28/93  T. Mittan       CSB     Corrected problem creating DDR,
*                                               and calculating proper NL, NS
*                                               when line weighting is used.
*     5       12/02/99  D. Hames        CSB     Remove 50000 limits from NL and
*                                               NS parameters in tab2ras.pdf.
*                                               Add error checks to getpar to
*                                               verify that NL and NS are 
*                                               positive. Modified to support 
*                                               images > 2gig. Add prototypes.
*                                               SRF #893
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include "typlim.h"
#include "util.h"
#include "protos.h"

int fdesc[2];           /* image file descriptors used for image i/o */
int gdesc;              /* group descriptor used for image i/o       */
long offset[2];         /* offsets into "thebuf" for output bands    */
struct PARBLK parblk;

void main
(void)

{
    static struct TAB_DEF tab[PROJATR];     /* table descriptor */
    static struct VECTOR *ptr[PROJATR];

    double corners[4];                      /* corner coordinates */
    double maxval = MINREAL;                /* maximum value in output image */
    double minval = MAXREAL;                /* minimum value in output image */

    float pixsiz[2];                        /* pixel size */

    long backgrnd;                          /* background value */
    long mosaic = 0;                        /* mosaic flag */
    long msgtim;                            /* time for processing message */
    long nfile;                             /* number of LT files */ 
    long nl;                               /* number of lines in output image */
    long ns;                             /* number of samples in output image */
    long one = 1;                           /* constant of 1 */
    long pixval = 0;                        /* output pixel value */
    long listflg;                         /* flag for printing error messages */
    long std_out;                           /* standard output */
    long tab_ind[PROJATR];                  /* tab index */
    long zero = 0;                          /* constant of 0 */
    long maxweight;                         /* maximum line weight          */

    double nw[2];                           /* north west corner */
    double ne[2];                           /* north east corner */
    double sw[2];                           /* south west corner */
    double se[2];                           /* south east corner */
    double rawynw;                          /* north west corner */
    double offset1;                         /* offset */

    short id_flag[PROJATR];                 /* flag if PTID is used instead of
                                               INDEX */

    char atrname[PROJATR][CMLEN];           /* attribute names */
    char ltin[MAX_SUB][CMLEN];              /* labeled table files */
    char subcmd[8];                         /* subcommand */
    char taeout[CMLEN];                     /* output TAE image name */
    char tempfile[NUMTEMPF][CMLEN];         /* temporary file names */
    char datapath[CMLEN];

    /* Initialize TAE parameter block  
       ----------------------------------*/
    c_inlas(&parblk,&std_out);

    /* Get the parameters
       ---------------------*/ 
    getpar(ltin,tempfile[OUTIMAGE],taeout,atrname,&pixval,&nl,&ns,&backgrnd,
            pixsiz,corners,&msgtim,&listflg,&nfile,subcmd,&mosaic,datapath);

    /* Open the input label table files
       ------------------------------------*/
    ltopen(tab,ltin,atrname,ptr,tab_ind,id_flag,&nfile);


    /* Create the temporary file select file that contains the line id, y/x 
       values, sequence number, priority, color, and line weight
       -----------------------------------------------------------------------*/
    if (strncmp(subcmd,"LINE",4) == 0)
    {
        linesel(tab,tab_ind,ptr,id_flag,&pixval,tempfile[SELECT],&msgtim,&nfile,
                &maxweight);

        /* Sort the select file based on Line id and sequence number
           -----------------------------------------------------------*/ 
        sortsel(tempfile,datapath);

        /* Retrieve the northwest(UL) and southeast(LR) corner of output image
           and calculate nl,ns if needed
           ------------------------------------------------------------------*/
        get_corners(tab,tab_ind,ptr,corners,&nl,&ns,pixsiz,nw,ne,sw,se,&offset1,
                &rawynw);

        /* Scan the sorted select file creating a temporary file that contains 
           the line number, priority, beginning and ending sample location, 
           and color 
           -------------------------------------------------------------------*/
        scnint(tempfile,&minval,&maxval,&msgtim,corners[1],corners[0],
                &listflg,nl,ns,pixsiz);
    }
    else if (strncmp(subcmd,"POINT",4) == 0)
    {
        /* Retrieve the northwest(UL) and southeast(LR) corner of output image
           and calculate nl,ns if needed
           ------------------------------------------------------------------*/
        maxweight = 0;
        get_corners(tab,tab_ind,ptr,corners,&nl,&ns,pixsiz,nw,ne,sw,se,&offset1,
                &rawynw);

        /* Create the temporary file scan file that contains the line number 
           priority, pixel location and color 
           -------------------------------------------------------------------*/
        ptinit(tab,tab_ind,ptr,id_flag,&pixval,tempfile[SCAN],&msgtim,&minval,
                &maxval,pixsiz,nl,ns,&listflg,corners[1],corners[0],&nfile);
    }
    else
    {
        /* Retrieve the northwest(UL) and southeast(LR) corner of output image
           and calculate nl,ns if needed
           -------------------------------------------------------------------*/
        maxweight = 0;
        get_corners(tab,tab_ind,ptr,corners,&nl,&ns,pixsiz,nw,ne,sw,se,&offset1,
                &rawynw);

        polysel(tab,tab_ind,ptr,id_flag,&pixval,tempfile[SELECT],&msgtim,
                &nfile);

        sortsel(tempfile,datapath);

        polyinit(tempfile,&minval,&maxval,&msgtim,corners[1],corners[0],
                &listflg,nl,ns,pixsiz,rawynw,offset1);
    }

    /* Sort the scan output to line, priority, and start sample location
       --------------------------------------------------------------------*/
    sortscan(tempfile,datapath);

    /* Generate the LAS image using the sorted scan output
       ------------------------------------------------------*/
    if (strncmp(subcmd,"POLYGON",7) == 0)
        genrst_pol(tempfile,nl,ns,&maxval,&minval,msgtim,&backgrnd,mosaic,
                nw,ne,sw,se,pixsiz);
    else
        genrst(tempfile,nl,ns,&maxval,&minval,msgtim,&backgrnd,nw,ne,sw,se,
               pixsiz);

    /* Save the min and max values.  Since the values had to be checked to
       determine the data type of the output image, don't bother with the
       $minmax flag.
       ----------------------------------------------------------------------*/
    if (c_svmnmx(tempfile[OUTIMAGE],&one,&minval,&maxval,"YES") != E_SUCC)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    if (nfile == 5)
        get_proj(tab,tab_ind,ptr,tempfile[OUTIMAGE]);

    /*  Update history file 
        -----------------------*/
    if (c_hstout(&parblk,tempfile[OUTIMAGE],(char (*)[256])" ",&zero," ") 
                 != E_SUCC)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    /* Output completion message 
       ----------------------------*/
    c_complt("TAB2RAS",(char (*)[256])taeout,&one);
} 
