/******************************************************************************

NAME:	PROCESS

FUNCTION:  PROCESS processes each record of the input labeled table

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):

Initialize variables
Locate each of the attribute names
Read the first record of the labeled table
Initialize the TPS header structure
Retrieve the data needed for the header of the tie point selection file based
 upon the attribute names specified
Open tie point selection output file
Do
  Read each record of the input labeled table 
  Write each field of the tie point file according the the data type of the 
   field
  Write out the record
  Read the next record
Until EOF is reached
Close the output tie point selection file
******************************************************************************/

#ifdef _IBMR2
#define unix
#endif
#include <stdio.h>
#ifdef unix
#include <sys/file.h>
#else
#include <file.h>
#endif

#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "tab2tie.h"
#include "util.h"
#include "protos.h"

void process
(
    struct PARBLK *parb,		/*  Tae parameter block			*/
    struct TAB_DEF *tab,		/* output labeled table descriptor */
    char hosout[][CMLEN],		/* output host file name		*/
    long *atrcnt,			/* no. of attributes specified		*/
    char atrname[][CMLEN],		/* the atribute names specified		*/
    long *atrcode			/* attribute codes specified		*/
)

{ /* process */

struct VECTOR *ptr[MAXATR];	/* pointer to current vector */
struct TPSHEAD tpshead;		/* TPS header structure			*/
struct TPSDATA tpsdata; 	/* TPS data structure			*/

long acc = IWRITE;		/* Access flag				*/
long fatal = -1;		/* fatal error code			*/
long i;				/* counter -- current column */
long j;				/* counter -- current file	*/
long nonfatal = 1;		/* nonfatal error code			*/
long one = 1;			/* value of 1				*/
struct PTDESC * ptdesc;			/* file descriptor of TPS file		*/
long pttype = TPS;		/* file type				*/
long status;			/* status of function calls		*/

short found;			/*  attribute found logical		*/

char msg[ERRLEN];		/*  errmsg buffer			*/

char tempc[21];			/* temp variable for ptid conversion	*/

/* Initialize variables
-----------------------*/
for (i = 0; i < *atrcnt; i++)
   ptr[i] = NULL;

/* Locate the attribute names in the labeled table vectors
----------------------------------------------------------*/
for (i = 0; i < *atrcnt; i++)
   {
   found = FALSE;
   ptr[i] = (struct VECTOR *)get_field_ptr(&tab[0],atrname[i]);
   if (ptr[i] != NULL)
      found = TRUE;

   for(j = 1; j <= tab[0].nsubs; j++)
      {
      if (!found)
         {
         ptr[i] = (struct VECTOR *)get_field_ptr(&tab[j],atrname[i]);
         if (ptr[i] != NULL)
	    found = TRUE;
         }
      }

   if (!found)
      {
      sprintf(msg,"Attribute %s could not be found",atrname[i]);
      c_errmsg(msg,"tab2tie-attribute",&nonfatal);
      c_errmsg("Fatal error encountered","tab2tie-fatal",&fatal);
      }
   }  /*  for i < *noutcol  */

/*  Read the first record
-------------------------*/
for (i = 0; i <= tab[0].nsubs; i++)
   {
   status = get_record(&tab[i]);
   if (status == E_FAIL)	/* Read the labeled table file        */
      {
      c_errmsg("Error reading input file","tab2tie-read",&nonfatal);
      c_errmsg("Fatal error encountered","tab2tie-fatal",&nonfatal);
      }
   }

/*  Initialize the TPS header structure
---------------------------------------*/
clrtps(&tpshead);

/*  Read the data needed for the header of the tie point selection file
    based on the attribute codes that were set in the subroutine GETPAR
-----------------------------------------------------------------------*/
for ( i = 0; i < *atrcnt; i++)
   {
   switch (atrcode[i])
      {
      case TPSUL:     get_matrix(ptr[i],R8,&tpshead.corners[0]);
		      break;
      case TPSLL:     get_matrix(ptr[i],R8,&tpshead.corners[2]);
		      break;
      case TPSUR:     get_matrix(ptr[i],R8,&tpshead.corners[4]);
		      break;
      case TPSLR:     get_matrix(ptr[i],R8,&tpshead.corners[6]);
		      break;
      case TPSDIST1:  get_matrix(ptr[i],R8,&tpshead.pdist[0]);
		      break;
      case TPSDIST2:  get_matrix(ptr[i],R8,&tpshead.pdist[1]);
		      break;
      case TPSPPARMS: get_matrix(ptr[i],R8,&tpshead.projprms[0]);
		      break;
      case TPSPROJV:  get_matrix(ptr[i],I4,&tpshead.proj_valid);
		      break;
      case TPSCODE:   get_matrix(ptr[i],I4,&tpshead.code);
		      break;
      case TPSZONE:   get_matrix(ptr[i],I4,&tpshead.zone);
		      break;
      case TPSDATUM:  get_matrix(ptr[i],I4,&tpshead.datum);
		      break;
      case TPSMODE:   sprintf(tpshead.mode,"%s",ptr[i]->ptr.c);
		      break;
      case TPSUNITS:  sprintf(tpshead.units,"%s",ptr[i]->ptr.c);
		      break;
      case TPSNAME:   sprintf(tpshead.name,"%s",ptr[i]->ptr.c);
		      break;
      }
   }

/*  Open tie point selection output file
----------------------------------------*/
status =c_ptopen(&ptdesc,hosout[0],&pttype,&acc,&tpshead);
if (status != E_SUCC)
      {
      c_errmsg("Error opening input file","tab2tie-open",&nonfatal);
      status = c_ptclse(&ptdesc);
      c_clean(parb,"tab2tie",hosout,&one);
      }

/*  Initialize each field of the data record
--------------------------------------------*/
tpsdata.coord[0] = tpsdata.coord[1] = tpsdata.image_coord[0] = 
   tpsdata.image_coord[1] = tpsdata.elevation = tpsdata.zoom_factor = 0.0;
tpsdata.chip_size[0] = tpsdata.chip_size[1] = tpsdata.offsets[0] = 
   tpsdata.offsets[1] = 0; 
tpsdata.pt_id[0] = tpsdata.chip_name[0] = (char) NULL;

/*  read each record of the input labeled table until EOF is reached
--------------------------------------------------------------------*/
do
   {

   /* write each column according the the data type of the fields
   --------------------------------------------------------------*/
   for (i = 0; i < *atrcnt; i++)
      {
      if (ptr[i] != NULL)
         {
         switch (atrcode[i])
	    {
            case TPSCOORD1:  get_matrix(ptr[i],R8,&tpsdata.coord[0]);
		             break;
            case TPSCOORD2:  get_matrix(ptr[i],R8,&tpsdata.coord[1]);
		             break;
            case TPSIMGCRD1: get_matrix(ptr[i],R8,&tpsdata.image_coord[0]);
		             break;
            case TPSIMGCRD2: get_matrix(ptr[i],R8,&tpsdata.image_coord[1]);
		             break;
            case TPSELEV:    get_matrix(ptr[i],R8,&tpsdata.elevation);
		             break;
            case TPSSCAL:    get_matrix(ptr[i],R8,&tpsdata.zoom_factor);
		             break;
            case TPSREFSZ1:  get_matrix(ptr[i],I4,&tpsdata.chip_size[0]);
		             break;
            case TPSREFSZ2:  get_matrix(ptr[i],I4,&tpsdata.chip_size[1]);
		             break;
            case TPSSOFF1:   get_matrix(ptr[i],I4,&tpsdata.offsets[0]);
		             break;
            case TPSSOFF2:   get_matrix(ptr[i],I4,&tpsdata.offsets[1]);
		             break;
            case TPSPTID:    if (ptr[i]->dtype == C)
				sprintf(tpsdata.pt_id,"%s",ptr[i]->ptr.c);
			     else if (ptr[i]->dtype == I1)
				{
				sprintf(tempc,"%d",*ptr[i]->ptr.i1);
				sprintf(tpsdata.pt_id,"%s",tempc);
				}
			     else if (ptr[i]->dtype == I2)
				{
				sprintf(tempc,"%d",*ptr[i]->ptr.i2);
				sprintf(tpsdata.pt_id,"%s",tempc);
				}
			     else if (ptr[i]->dtype == I4)
				{
				sprintf(tempc,"%d",*ptr[i]->ptr.i4);
				sprintf(tpsdata.pt_id,"%s",tempc);
				}
			     else if (ptr[i]->dtype == R4)
				{
				sprintf(tempc,"%f",*ptr[i]->ptr.r4);
				sprintf(tpsdata.pt_id,"%s",tempc);
				}
			     else if (ptr[i]->dtype == R8)
				{
				sprintf(tempc,"%f",*ptr[i]->ptr.r8);
				sprintf(tpsdata.pt_id,"%s",tempc);
				}
		             break;
            case TPSSNAME:   sprintf(tpsdata.chip_name,"%s",ptr[i]->ptr.c);
		             break;
	    } /*  switch  prt[i].dtype  */
         }  /* if ptr[i]!= NULL  */
      }

   /*  Write out the record
   ------------------------*/
   status = c_ptio(&ptdesc,&tpsdata); /* Write the tie point selection file   */
   if (status != E_SUCC)	
      {
      c_errmsg("Error writing output file","tab2tie-write",&nonfatal);
      status = c_ptclse(&ptdesc);
      c_clean(parb,"tab2tie",hosout,&one);
      }

   /*  Read the next record
   ------------------------*/
   for (i = 0; i <= tab[0].nsubs; i++)
      {
      status = get_record(&tab[i]);
      if (status == E_FAIL)	/* Read the labeled table file        */
         {
         c_errmsg("Error reading input file","tab2tie-read",&nonfatal);
         status = c_ptclse(&ptdesc);
         c_clean(parb,"tab2tie",hosout,&one);
         }
      }
   }
while (status == E_SUCC);

/*  Close the output tie point selection file
---------------------------------------------*/
status = c_ptclse(&ptdesc);
if (status == E_FAIL)	/* Read the labeled table file        */
    {
    c_errmsg("Error reading input file","tab2tie-read",&nonfatal);
    c_clean(parb,"tab2tie",hosout,&one);
    }

return;
} /* process */
