#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "protos.h"


/******************************************************************************
FUNCTION:       CHECK_WINDOW

PURPOSE:        Checks the validity of the window specification.  If an
                error is detected, the window specification is changed
                to be the intersection of the window specification and
                the actual image.

ALGORITHM REFERENCES    none
*******************************************************************************/
void chk_window
(
    long *inspc,    /* input image file specifications   */
    long *window    /* window area of image to be copied */
)

{
    long nonfatal = 1; /* non-fatal error code              */

    /* If NL or NS is 0 then set them to the total number of lines and samples
       ----------------------------------------------------------------------*/
    if (window[NL] == 0)
        window[NL] = inspc[0];

    if (window[NS] == 0)
        window[NS] = inspc[1];

    /* Validate the number of lines specified
       -----------------------------------------*/
    if (window[SL] + window[NL] - 1 > inspc[0])
    {
        c_errmsg("NL is too large","tapein-window",&nonfatal);
        window[NL] = inspc[0] - window[SL] + 1;
    }

    /* Validate the specified number of samples
       -------------------------------------------*/
    if (window[SS] + window[NS] - 1 > inspc[1])
    {
        c_errmsg("NS is too large","tapein-window",&nonfatal);
        window[NS] = inspc[1] - window[SS] + 1;
    }

    return;
}



/******************************************************************************
FUNCTION:       CHECK_BAND

PURPOSE:        Checks the validity of the band specification.  If an error is
                detected in the band specification, the program continues by
                not copying the band that is in error.  The routine also sets
                up a boolean array indicating which bands are to be copied.
                This array is returned to the calling program.

ALGORITHM REFERENCES    none
*******************************************************************************/
short *check_band
(
    long *inspc,        /* input image file specifications                   */
    long *bands         /* bands to be processed within image                */
)

{
    long  index;            /* loop index                                    */
    long  nonfatal = 1;     /* Non-fatal error code                          */

    short *bflag;           /* flag to indicate bands to copy                */

    char  msg[ERRLEN];      /* buffer space for error message                */


    /* Initialize boolean band array to false
       -----------------------------------------*/
    bflag = (short *) malloc(inspc[2]*4);
    for (index = 0; index < inspc[2]; index++)
        *(bflag + index) = FALSE;

    /* For each band specified, error check to insure the band exists in the
       input image.  If the band exists, the boolean array element for this 
       band is set to TRUE.
       ---------------------------------------------------------------------*/
    for (index = 0; index < inspc[2]; index++) 
    {
        if (bands[index] > inspc[2])
        {
            sprintf(msg,"Band %d does not exist in image file",bands[index]);
            c_errmsg(msg,"tapein-nband",&nonfatal);
            continue;
        }

        if (bands[index] != 0)
            *(bflag + bands[index] - 1) = TRUE;
    }

    return(bflag);
}



/*******************************************************************************
FUNCTION: CHECK_DDR

PURPOSE:  Check the values found in a DDR with the values that are supplied by
          the user.  When appropriate, reset the user supplied values with
          those in the DDR.  If an error is found issue a fatal error.  Only
          return on successful completion.

*******************************************************************************/
void check_ddr
(
    long bands[][MAXBND + 1],   /* Bands to be read from this image file      */
    long *inspc,                /* Image file specs (total lines,samps,bands) */
    long *m2one,                /* Flag placing multiple images in one output */
    long *totbnd,               /* Total number of bands to place in output   */
    struct DDR *ddr             /* Input Data Descriptor Record               */
)

{
    char msg[ERRLEN];               /* String for printing error messages     */

    long fatal    = -1;             /* Fatal error flag                       */
    long i;                         /* Loop control                           */
    long nonfatal =  1;             /* Nonfatal error flag                    */

    /* Check TOTLINES
       -----------------*/
    if (inspc[0] == 0)
        inspc[0] = ddr->nl;
    else if (inspc[0] != ddr->nl)
    {
        sprintf(msg, "Resetting TOTLINES from %1d to %1d", inspc[0], ddr->nl);
        c_errmsg(msg, "tapein-reset", &nonfatal);
        inspc[0] = ddr->nl;
    }

    /* Check TOTSAMPS
       -----------------*/
    if (inspc[1] == 0)
        inspc[1] = ddr->ns;
    else if (inspc[1] != ddr->ns)
    {
        sprintf(msg, "Resetting TOTSAMPS from %1d to %1d", inspc[1], ddr->ns);
        c_errmsg(msg, "tapein-reset", &nonfatal);
        inspc[1] = ddr->ns;
    }

    /* Check TOTBANDS
       -----------------*/
    if (*m2one)
    {
        /* When placing multiple image files in one output image the output 
           image may have already been created and we cannot change the total 
           number of bands in the output image at this time.  If a discrepancy 
           is found, a fatal error must be issued.
           -----------------------------------------------------------------*/
        if (inspc[2] == 0)
        {
            /* If TOTBANDS was null then BANDS had to be specified.  Make sure 
               none of the specified bands is greater than the last band of 
               the image on tape.
               -------------------------------------------------------------*/
            inspc[2] = ddr->nbands;
            for (i = 0; bands[0][i] != 0; i++)
                if (bands[0][i] > inspc[2])
                {
                    sprintf(msg, "User specified band %1d, image contains %1d",
                            bands[0][i], inspc[2]);
                    c_errmsg(msg, "tapein-large", &nonfatal);
                    c_errmsg("Fatal error encountered", "tapein-fatal", &fatal);
                }
        }
        else if (bands[0][0] == 0)
        {
            /* If BANDS was null, TOTBANDS was specified.  Make sure that 
               TOTBANDS is the same as the number of bands indicated in the DDR.
               ---------------------------------------------------------------*/
            if (inspc[2] != ddr->nbands)
            {
                sprintf(msg, "TOTBANDS = %1d, DDR indicates %1d",
                        inspc[2],ddr->nbands);
                c_errmsg(msg, "tapein-totbands", &nonfatal);
                c_errmsg("Fatal error encountered", "tapein-fatal", &fatal);
            }

            for (i = 0; i < inspc[2]; i++)
                bands[0][i] = i + 1;
        }
        else if (inspc[2] > ddr->nbands)
        {
            /* If the value of TOTBANDS is larger than what the image file 
               actually contains reset the value and make sure that none of the 
               band numbers is larger than the largest band in the image file.
               --------------------------------------------------------------*/
            sprintf(msg, "Resetting TOTBANDS from %1d to %1d", inspc[2], 
                    ddr->nbands);
            c_errmsg(msg, "tapein-reset", &nonfatal);
            inspc[2] = ddr->nbands;

            for (i = 0; bands[0][i] != 0; i++)
                if (bands[0][i] > inspc[2])
                {
                    sprintf(msg, "User specified band %1d, image contains %1d",
                            bands[0][i], inspc[2]);
                    c_errmsg(msg, "tapein-large", &nonfatal);
                    c_errmsg("Fatal error encountered", "tapein-fatal", &fatal);
                }
        }
        else if (inspc[2] < ddr->nbands)
        {
            sprintf(msg, "Resetting TOTBANDS from %1d to %1d", inspc[2], 
                    ddr->nbands);
            c_errmsg(msg, "tapein-reset", &nonfatal);
            inspc[2] = ddr->nbands;
        }
    }
    else
    {
        /* When placing each image file in its own output image the total number
           of bands can be changed.  Notify the user of any changes.
           ------------------------------------------------------------------*/
        if ((inspc[2] != 0) && (inspc[2] != ddr->nbands))
        {
            sprintf(msg, "Resetting TOTBANDS from %1d to %1d",inspc[2],
                    ddr->nbands);
            c_errmsg(msg, "tapein-reset", &nonfatal);
        }
        inspc[2] = ddr->nbands;

        if (bands[0][0] == 0)
        {
            /* If BANDS is null then initialize bands
               -----------------------------------------*/
            *totbnd = inspc[2];
            for (i = 0; i < ddr->nbands; i++)
                bands[0][i] = i + 1;
        }
        else
        {
            /* If BANDS was specified, make sure all of
               the specified bands are <= inspc[2]
               -------------------------------------------*/
            for (i = 0; bands[0][i] != 0; i++)
                if (bands[0][i] > inspc[2])
                {
                    *totbnd = i;
                    sprintf(msg,"Band %1d does not exist in image file", 
                            bands[0][i]);
                    c_errmsg(msg, "tapein-nband", &nonfatal);
                    bands[0][i] = 0;
                    break;
                }
        }
    }
}



/******************************************************************************
FUNCTION:       CHECK_TPREAD

PURPOSE:        Checks the status of a tape read.  If the status indicates
                an error, appropriate action is taken.

ALGORITHM REFERENCES    none
*******************************************************************************/
short check_tpread
(
    long status,        /* status code of tpread call                         */
    long *unit,         /* tape unit number                                   */
    unsigned char *buf, /* buffer to read data into                           */
    long *nbyte,        /* number of bytes to read                            */
    long *mode          /* error mode flag                                    */
)

{
    long  one = 1;          /* value of one                                   */
    long  fatal = -1;       /* Fatal error code                               */
    long  nonfatal = 1;     /* Non-fatal error code                           */
    short not_eof = TRUE;   /* flag to indicate EOF, EOV or EOT               */

    /* Operate on the status passed in
       ----------------------------------*/
    switch (status)
    {
        case TPREAD_READ:  /* Read error encountered */ 
            /* Skip to EOF and terminate the process
               ----------------------------------------*/
            c_stoptm();
            skip_file(unit,one);
            c_errmsg("Error reading from tape","tapein-tpread",&fatal);
            break;

        case TAPE_EOF:  /* Unexpected EOF encountered */
            c_errmsg("Unexpected EOF encountered", "tapein-tpeof", &nonfatal);
            not_eof = FALSE;
            break;

        case TAPE_EOV:  /* EOV encountered */
        case TAPE_EOT:  /* EOT encountered */
            /* Mount the next tape and continue process
               -------------------------------------------*/
            close_tape(unit,REWOFFL);
            mount_tape(unit,TRUE);
            c_tpread(unit,buf,nbyte,mode);
            break;
    }

    return(not_eof);
}
