/******************************************************************************
FUNCTION:       COPY_DDR

PURPOSE:        Determines if the DDR exists for the current image on tape.
                If it does, the DDR is used to update the output DDR.

ALGORITHM 
  Read the first record to determine if there is a DDR
  Back space the record just read
  Pull the key from the header record
  If the key is valid then it must be a DDR
      Check to make sure the size of the input file on tape was specified
      since it cannot be pulled from the DDR
      Check to make sure the input data type was specified since it cannot
      be pulled from the DDR
      Allocate memory for the input buffer
  The DDR was found so process it from tape
      Open a temporary ddr file.  This file is used later to update the 
      output DDR.
      Save the temporary DDR name for clean up
      Process each record from the label services DDR file
          Set the character and data buffer lengths to the maximum expected
          for the DDR record.
          Read the label services DDR and BDR records from tape
          Convert the integer portion of the DDR
          Write the DDR information to the temporary DDR file
      Close the label services file
      Free space allocated for the input key
      Read the temporary DDR information so that the number of lines, samples,
      bands and the data type can be updated
      Set the input file size and data type if they were not already set
  Free space allocated for the input key

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "vll.h"
#include "tapeio.h"
#include "sysdef.h"
#include "string.h"
#include "util.h"
#include "labserv.h"
#include "geompak.h"
#include "protos.h"

#define ASCI 2
#define MAX_DDRSIZ 250


void copy_ddr
(
    long bands[][MAXBND + 1],   /* Bands to be read from this image file      */
    long *idtype,               /* Data type of image file on tape            */
    long *inspc,                /* Image file specs (total lines,samps,bands) */
    long *insys,                /* System the image was created on            */
    long *m2one,                /* Flag placing multiple images in one output */
    long *totbnd,               /* Total number of bands to place in output   */
    long *unit,                 /* Tape unit number                           */
    short *found_ddr,           /* Flag indicating DDR was found or not       */
    short *hp_flag,             /* Flag to indicate this is an HP             */
    char *ddrnam,               /* Temporary image file to hold input DDR     */
    long *curfil,               /* curent band file                           */
    char *tddrnam               /* temporary ddr name                         */
)

{
    extern long outcnt;             /* Number of output files opened         */
    extern char out[2][CMLEN];      /* output image files opened             */

    struct DDR ddr;                 /* Input Data Descriptor Record          */
    struct BDDR bddr;               /* Input Data Descriptor Record          */
    struct TAPEDESC *tp;            /* tape drive description                */

    long  bytes_read;               /* actual # of bytes read from tape      */
    long  clen;                     /* Length of the character portion of DDR*/
    long  dlen;                     /* Length of the data portion of DDR     */
    long  fatal    = -1;            /* Fatal error code                      */
    long  fd;                       /* File descriptor to output ddr         */
    long  i;                        /* Temporary variable                    */
    long  mode     = HANDLE;        /* tape I/O does NOT output error msg    */
    long  nbyte;                    /* Maximum size of a DDR record          */
    long  nonfatal = 1;             /* Non-Fatal error code                  */
    long  one      = 1;             /* value of one                          */
    long  status;                   /* function return status code           */
    long  zero     = 0;             /* value of zero                         */

    short not_eof;                  /* flag to check EOF, EOV, or EOT        */

    char *cbuf;                     /* Buffer to read character portion into */
    char dtype[3];                  /* data type of the LS data buffer       */
    char header[HDRL+1];            /* Header information                    */
    char *key;                      /* Key to read from tape                 */
    char key2[100];                 /* Key to read from tape                 */
    char msg[ERRLEN];               /* error message buffer                  */
    char temp_char[100];
    char *ptr;                      /* buffer pointer                        */
    double temp;
    double temp1;
    double temp2;

    unsigned char *dbuf;            /* Buffer to read data portion into      */
    unsigned char *inbuf;           /* Buffer for tape read                  */


    /* Begin by allocating memory buffers.  NOTE:  These buffers are ALLOCATED 
       and not DECLARED to fix a bug on sunm.  By malloc'ing the buffers, the 
       starting address is guaranteed to be at the beginning of a word.  Errors
       result
       otherwise.
       ----------------------------------------------------------------------*/
    tp = (struct TAPEDESC *) *unit;
    nbyte = tp->maxbyte;
    cbuf = (char *)malloc(nbyte);
    if (cbuf == NULL)
    {
        c_errmsg("Error allocating dynamic memory","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }
    dbuf =  (unsigned char *) malloc(nbyte);
    if (dbuf == NULL)
    {
        free(cbuf);
        c_errmsg("Error allocating dynamic memory","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }
    inbuf = (unsigned char *) malloc(nbyte);
    if (inbuf == NULL)
    {
        free(cbuf);
        free(dbuf);
        c_errmsg("Error allocating dynamic memory","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }

    /* Read the first record to determine if there is a DDR
       -------------------------------------------------------*/
    bytes_read = c_tpread(unit,inbuf,&nbyte,&mode);
    not_eof = check_tpread(bytes_read,unit,inbuf,&nbyte,&mode);

    /* Back space the record just read
       ----------------------------------*/
    status = c_tpbsf(unit,&zero,&mode);
    if ((status != TAPE_SUCC) && (status != TAPE_BOT))
    {
        free(cbuf);
        free(dbuf);
        free(inbuf);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }

    /* Pull the key from the header record
       --------------------------------------*/
    strncpy(header,(char *) inbuf,HDRL);
    header[HDRL] = '\0';
    key = squeeze(header+LENL+TYPL,(long) strlen(header+LENL+TYPL));
    strncpy(key2,&header[2],5);
    key2[5] = '\0';

    /* If the key is valid then it must be a DDR
       --------------------------------------------*/
    if ((not_eof) && (strcmp(key,"DDRINT") == 0))
        *found_ddr = TRUE;
    else
        if ((not_eof) && (strcmp(key2,"IMAGE") == 0))
            *found_ddr = ASCI;
        else
        {
            *found_ddr = FALSE;

            /* Make sure that TOTLINES, TOTSAMPS, TOTBANDS, and IDTYPE were
               specified.  If not, error out because a DDR does not exist.
               ---------------------------------------------------------------*/
            if ((inspc[0] == 0) || (inspc[1] == 0) || (inspc[2] == 0) 
                || (*idtype == 0))
            {
                free(cbuf);
                free(dbuf);
                free(inbuf);
                free(key);
                sprintf(msg, 
                        "DDR does not exist.  TOTLINES, TOTSAMPS, TOTBANDS, %s",
                        "and IDTYPE must be specified");
                c_errmsg(msg, "tapein-required", &nonfatal);
                c_errmsg("Fatal error encountered", "tapein-fatal", &fatal);
            }

            /* If BANDS was null then initialize it to read all bands
               ---------------------------------------------------------*/
            if (bands[0][0] == 0)
                for (i = 0; i < inspc[2]; i++)
                    bands[0][i] = i + 1;
        }

    /* The DDR was found so process it from tape
       --------------------------------------------*/
    if (*found_ddr && *found_ddr != ASCI)
    {
        /* Open a temporary ddr file.  This file is
           used later to update the output DDR.
           -------------------------------------------*/
        sprintf(ddrnam,"%s_%d.ddr",tddrnam,*curfil);
        status = c_lsopen(&fd,ddrnam,&one);
        if (status != E_SUCC)
            cleanup(unit);

        /* Save the temporary DDR name for clean up
           -------------------------------------------*/
        if (*m2one != TRUE)
        {
            strcpy(out[outcnt],ddrnam);
            outcnt++;
        }

        /* Process each record from the label services DDR file
           -------------------------------------------------------*/
        for (;;)
        {
            /* Set the character and data buffer lengths
               to the maximum expected for the DDR record.
               ----------------------------------------------*/
            strcpy(key," ");
            clen = MAX_DDRSIZ;
            dlen = MAX_DDRSIZ;

            /* Read the label services DDR and BDR records from tape
               --------------------------------------------------------*/
            status = c_tplsrd(unit,key,&clen,&dlen,cbuf,dbuf,dtype);
            if ((status == TAPE_EOV) || (status == TAPE_EOT))
            {
                /* Mount the next tape and continue process
                   -------------------------------------------*/
                close_tape(unit,REWOFFL);
                mount_tape(unit,TRUE);
                status = c_tplsrd(unit,key,&clen,&dlen,cbuf, dbuf,dtype);
                if (status != TAPE_SUCC)
                    cleanup(unit);
            }
            else if (status == TAPE_EOF)
                break;
            else if (status != TAPE_SUCC)
                cleanup(unit);

            /* Write the DDR information to the temporary DDR file
               ------------------------------------------------------*/
            status = c_lswrit(&fd,key,&clen,&dlen,cbuf,(char *)dbuf,dtype);
            if (status != E_SUCC)
                cleanup(unit);
        }

        /* Close the label services file
           --------------------------------*/
        status = c_lsclos(&fd,ddrnam,&zero);
        if (status != E_SUCC)
            *found_ddr = FALSE;

        /* Read the temporary DDR information so that the number of
           lines, samples, bands and the data type can be updated
           -----------------------------------------------------------*/
        status = getdata(ddrnam,&ddr);
        if (status != E_SUCC)
        {
            free(cbuf);
            free(dbuf);
            free(inbuf);
            free(key);
            c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
        }

        /* Check the DDR values against the user supplied values
           --------------------------------------------------------*/
        check_ddr (bands, inspc, m2one, totbnd, &ddr);

        /* Overwrite the input data type and the input system with the DDR's 
           values
           ----------------------------------------------------------------*/
        *idtype = ddr.dtype;
        i = SETN;       /* set system numerical code from system string */
        c_sysset(ddr.system, insys, &i);
        if (strcmp(ddr.system, HP) == 0)
            *hp_flag = TRUE;
    }

    /* The DDR was found so process it from tape
       --------------------------------------------*/
    if (*found_ddr == ASCI)
    {
        /* Open a temporary ddr file.  This file is
           used later to update the output DDR.
           -------------------------------------------*/
        /*
           sprintf(ddrnam,"t_e_m_p_%d.ddr",*curfil);
         */
        sprintf(ddrnam,"%s_%d.ddr",tddrnam,*curfil);
        status = c_lsopen(&fd,ddrnam,&one);
        if (status != E_SUCC)
            cleanup(unit);

        /* Save the temporary DDR name for clean up
           -------------------------------------------*/
        if (*m2one != TRUE)
        {
            strcpy(out[outcnt],ddrnam);
            outcnt++;
        }

        /* Process each record from the label services DDR file
           -------------------------------------------------------*/
        bytes_read = c_tpread(unit,inbuf,&nbyte,&mode);
        not_eof = check_tpread(bytes_read,unit,inbuf,&nbyte,&mode);

        /* fill ddr
           ---------*/
        ptr = strstr((char *) &inbuf[0],"NL:");
        ptr += 3;
        sscanf(ptr,"%d",&ddr.nl);
        ptr = strstr(ptr,"NS:");
        ptr += 3;
        sscanf(ptr,"%d",&ddr.ns);
        ptr = strstr(ptr,"NB:");
        ptr += 3;
        sscanf(ptr,"%d",&ddr.nbands);
        ptr = strstr(ptr,"DTYPE:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"BYTE") == 0)
            ddr.dtype = 1;
        else
            if (strcmp(temp_char,"INTEGER*2") == 0)
                ddr.dtype = 2;
            else
                if (strcmp(temp_char,"INTEGER*4") == 0)
                    ddr.dtype = 3;
                else
                    if (strcmp(temp_char,"REAL*4") == 0)
                        ddr.dtype = 4;
                    else
                        ddr.dtype = 1;
        ptr = strstr(ptr,"SYSTEM:");
        ptr += 7;
        sscanf(ptr,"%s",ddr.system);
        ptr = strstr(ptr,"CODE:");
        ptr += 6;
        sscanf(ptr,"%d",&ddr.proj_code);
        ptr = strstr(ptr,"Valid:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[0] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[0] = 0;
            else
                ddr.valid[0] = 2;


        ptr = strstr(ptr,"CODE:");
        ptr += 5;
        sscanf(ptr,"%d",&ddr.zone_code);
        ptr = strstr(ptr,"Valid:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[1] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[1] = 0;
            else
                ddr.valid[1] = 2;


        ptr = strstr(ptr,"CODE:");
        ptr += 5;
        sscanf(ptr,"%d",&ddr.datum_code);
        ptr = strstr(ptr,"Valid:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[2] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[2] = 0;
            else
                ddr.valid[2] = 2;

        ptr = strstr(ptr,"Valid:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[3] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[3] = 0;
            else
                ddr.valid[3] = 2;

        ptr = strstr(ptr,"A:");
        ptr += 2;
        /*
           sscanf(ptr,"%lf %lf %lf",&ddr.proj_coef[0],&ddr.proj_coef[1],
           &ddr.proj_coef[2]);
         */
        sscanf(ptr,"%lf %lf %lf",&temp,&temp1,&temp2);
        ddr.proj_coef[0] = temp;
        ddr.proj_coef[1] = temp1;
        ddr.proj_coef[2] = temp2;

        ptr = strstr(ptr,"B:");
        ptr += 2;
        /*
           sscanf(ptr,"%lf %lf %lf",&ddr.proj_coef[3],&ddr.proj_coef[4],                                            &ddr.proj_coef[5]);
         */
        sscanf(ptr,"%lf %lf %lf",&temp,&temp1,&temp2);
        ddr.proj_coef[3] = temp;
        ddr.proj_coef[4] = temp1;
        ddr.proj_coef[5] = temp2;

        ptr = strstr(ptr,"C:");
        ptr += 2;
        /*
           sscanf(ptr,"%lf %lf %lf",&ddr.proj_coef[6],&ddr.proj_coef[7],
           &ddr.proj_coef[8]);
         */
        sscanf(ptr,"%lf %lf %lf",&temp,&temp1,&temp2);
        ddr.proj_coef[6] = temp;
        ddr.proj_coef[7] = temp1;
        ddr.proj_coef[8] = temp2;

        ptr = strstr(ptr,"D:");
        ptr += 2;
        /*
           sscanf(ptr,"%lf %lf %lf",&ddr.proj_coef[9],&ddr.proj_coef[10],
           &ddr.proj_coef[11]);
         */
        sscanf(ptr,"%lf %lf %lf",&temp,&temp1,&temp2);
        ddr.proj_coef[9] = temp;
        ddr.proj_coef[10] = temp1;
        ddr.proj_coef[11] = temp2;

        ptr = strstr(ptr,"E:");
        ptr += 2;
        /*
           sscanf(ptr,"%lf %lf %lf",&ddr.proj_coef[12],&ddr.proj_coef[13],
           &ddr.proj_coef[14]);
         */

        sscanf(ptr,"%lf %lf %lf",&temp,&temp1,&temp2);
        ddr.proj_coef[12] = temp;
        ddr.proj_coef[13] = temp1;
        ddr.proj_coef[14] = temp2;


        ptr = strstr(ptr,"Valid");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[4] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[4] = 0;
            else
                ddr.valid[4] = 2;

        ptr = strstr(ptr,"ULcorner:");
        ptr += 9;
        sscanf(ptr,"%lf %lf",&ddr.upleft[0],&ddr.upleft[1]);
        ptr = strstr(ptr,"URcorner:");
        ptr += 9;
        sscanf(ptr,"%lf %lf",&ddr.upright[0],&ddr.upright[1]);
        ptr = strstr(ptr,"LLcorner:");
        ptr += 9;
        sscanf(ptr,"%lf %lf",&ddr.loleft[0],&ddr.loleft[1]);
        ptr = strstr(ptr,"LRcorner:");
        ptr += 9;
        sscanf(ptr,"%lf %lf",&ddr.loright[0],&ddr.loright[1]);

        ptr = strstr(ptr,"DIST:");
        ptr += 5;
        sscanf(ptr,"%lf %lf",&ddr.pdist_y,&ddr.pdist_x);

        ptr = strstr(ptr,"Valid:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[5] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[5] = 0;
            else
                ddr.valid[5] = 2;


        ptr = strstr(ptr,"UNITS:");
        ptr += 6;
        sscanf(ptr,"%s",ddr.proj_units);
        if (c_string2unit(ddr.proj_units) == E_FAIL)
        {
            c_lasmsg(FALSE, "tapein-warn", 
                    "Projection units %s are invalid",ddr.proj_units);
            c_lasmsg(FALSE, "tapein-warn", "Setting units to invalid");
            ddr.valid[4] = 0;
        } 
        ptr = strstr(ptr,"Valid:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[6] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[6] = 0;
            else
                ddr.valid[6] = 2;



        ptr = strstr(ptr,"INCREMENT:");
        ptr += 10;
        sscanf(ptr,"%lf %lf",&ddr.line_inc,&ddr.sample_inc);
        ptr = strstr(ptr,"Valid:");
        ptr += 6;
        sscanf(ptr,"%s",temp_char);
        if (strcmp(temp_char,"VALID") == 0)
            ddr.valid[7] = 1;
        else
            if (strcmp(temp_char,"INVALID") == 0)
                ddr.valid[7] = 0;
            else
                ddr.valid[7] = 2;

        ptr = strstr(ptr,"COOR:");
        ptr += 5;
        sscanf(ptr,"%d %d",&ddr.master_line,&ddr.master_sample);


        c_putddr(ddrnam, &ddr);

        bytes_read += - (ptr - (char *) inbuf) + 395; 

        /*
           ptr = &inbuf[1447 - 396];
         */

        ptr = strstr(ptr,"\n");
        ptr += 2;

        for (i = 1;;i++)
        {

            bytes_read -= 395;

            /*
               ptr = ptr + 396;
             */

            if (bytes_read <= 395)
            {
                status = c_tpread(unit,inbuf,&nbyte,&mode);
                bytes_read = status;
                ptr = (char *) inbuf;
            }
            else
                status = TAPE_SUCC;

            if ((status == TAPE_EOV) || (status == TAPE_EOT))
            {
                /* Mount the next tape and continue process
                   -------------------------------------------*/
                close_tape(unit,REWOFFL);
                mount_tape(unit,TRUE);
                bytes_read = c_tpread(unit,inbuf,&nbyte,&mode);
                not_eof = check_tpread(bytes_read,unit,inbuf,&nbyte,&mode);
                if (status != TAPE_SUCC)
                    cleanup(unit);
                ptr = (char *) inbuf;
            }
            else if (status == TAPE_EOF)
                break;
            else if (status != TAPE_SUCC)
                cleanup(unit);



            ptr = strstr(ptr,"NO:");
            ptr += 3;
            sscanf(ptr,"%d",&bddr.bandno);
            ptr = strstr(ptr,"MINIMUM:");
            ptr += 8;
            sscanf(ptr,"%lf",&bddr.minval);
            ptr = strstr(ptr,"MAXIMUM:");
            ptr += 8;
            sscanf(ptr,"%lf",&bddr.maxval);
            ptr = strstr(ptr,"Valid:");
            ptr += 6;
            sscanf(ptr,"%s",temp_char);
            if (strcmp(temp_char,"VALID") == 0)
                bddr.valid = 1;
            else
                if (strcmp(temp_char,"INVALID") == 0)
                    bddr.valid = 0;
                else
                    bddr.valid = 2;

            ptr = strstr(ptr,"SOURCE:");
            ptr += 7;
            sscanf(ptr,"%s",bddr.source);
            if (strstr(bddr.source,"SENSOR") != NULL)
                strcpy(bddr.source,"\0");
            ptr = strstr(ptr,"TYPE:");
            ptr += 5;
            sscanf(ptr,"%s",bddr.instrument);
            if (strstr(bddr.instrument,"CAPT.") != NULL)
                strcpy(bddr.instrument,"\0");
            ptr = strstr(ptr,"DIRECTION:");
            ptr += 10;
            sscanf(ptr,"%s",bddr.direction);
            if (strstr(bddr.direction,"DATE:") != NULL)
                strcpy(bddr.direction,"\0");
            ptr = strstr(ptr,"DATE:");
            ptr += 5;
            sscanf(ptr,"%s",bddr.date);
            if (strstr(bddr.date,"TIME:") != NULL)
                strcpy(bddr.date,"\0");
            ptr = strstr(ptr,"TIME:");
            ptr += 4;
            if (sscanf(ptr,":%s ",bddr.time) != 1)
                strcpy(bddr.time,"\0");
            if (strlen(bddr.time) > 10)
                strcpy(bddr.time,"\0");
            ptr += 10;


            /*
               bddr.bandno = i;
             */
            c_putbdr(ddrnam, &bddr);
        }

        /* Read the temporary DDR information so that the number of
           lines, samples, bands and the data type can be updated
           -----------------------------------------------------------*/
        status = getdata(ddrnam,&ddr);
        if (status != E_SUCC)
        {
            free(cbuf);
            free(dbuf);
            free(inbuf);
            free(key);
            c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
        }

        /* Check the DDR values against the user supplied values
           --------------------------------------------------------*/
        check_ddr (bands, inspc, m2one, totbnd, &ddr);

        /* Overwrite the input data type and the input system with the DDR's 
           values
           ------------------------------------------------------------------*/
        *idtype = ddr.dtype;
        i = SETN;           /* set system numerical code from system string */
        c_sysset(ddr.system, insys, &i);
        if (strcmp(ddr.system, HP) == 0)
            *hp_flag = TRUE;

    }


    /* Free space allocated
       -----------------------*/
    free(key);
    free(cbuf);
    free(dbuf);
    free(inbuf);

    return;
}
