/******************************************************************************
FUNCTION:       CP_BIL

PURPOSE:        Reads image data from tape in BIL format, converts it to the
                output data type, and copies (with resampling) to the LAS
                output image.

ALGORITHM 
  Calculate the number of output samples
  Calculate the number of input bytes
  Allocate memory for the input buffer
  If min max values are to be computed
        call inmnmx to initialize the min/max array for the output data type.  
  Start processing message
  Initial step to set up offset into the output image.
  Set for skipping records on tape to the starting line of the window
  If HP flag is set 
        strip the the zero value pixel at the end off
  Initialize the current line in the block
  Process each line
      Skip records on tape to starting line of the window
      Process each band
          Read image line from tape for current band
          Check to see if band is wanted
              Position the input buffer pointer to the starting window and
              block location
              Position the output buffer pointer to the beginning of the
              desired band
              Subsample the image data if specified
              Convert the data from the input system format to the output
              system format.  This includes byte swapping and conversion
              of real numbers to the host system format.
              Compute the min/max values if required
      Write image line for all bands to the output LAS image.  The input
      data is converted to the output data type by IMAGEIO.
      Skip records on tape to the next line for resampling
  Set the number of records to get to the end of the window specified
  If not at EOF skip to the EOF on tape
      Stop the processing message 
      Calculate the number of records to be skipped to get to EOF
      Skip records to the end of the image
          The calculations did not bring the tape to the End Of File.  The
          number of lines in the image on tape is not correct.  Inform the
          user and skip to EOF anyway.
  Free space allocated for the input buffer
  Create/update the min max file for the output image.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "sysdef.h"
#include "pixman.h"
#include "protos.h"

void cp_bil
(
    long *unit,                 /* tape unit number                           */
    long *insys,                /* Input system data if from                  */
    long *outsys,               /* Output system data is written to           */
    char *hosout,               /* output host name                           */
    long *inspc,                /* input file specifications                  */
    long window[],              /* window area of image to be copied          */
    long *totbnd,               /* total number of bands to be copied         */
    long *idtype,               /* input image data type                      */
    long *blksiz,               /* number of line in input block              */
    long *linc,                 /* line increment for subsample               */
    long *sinc,                 /* sample increment for subsample             */
    short *bflag,               /* flag indicating bands to be copied         */
    short *hp_flag,             /* boolean for HP computer system             */
    short *minmax               /* min max flag                               */
)

{
    extern long curfil;             /* current file being processed           */
    extern long nmrfil;             /* total # of files to be processed       */
    extern long curent;             /* output line being processed            */
    extern struct GDESC * gdesc;    /* group descriptor used for image I/O    */
    extern long datasize[];         /* defines number bytes for each data type*/
    extern long offset[];           /* offsets into "thebuf" for output bands */
    extern int EOV_detected;        /* EOV detected flag                      */
    extern unsigned char *thebuf;   /* declare image I/O buffer               */

    long band;                      /* input band number being read           */
    long line;                      /* current input image line               */
    long nrec;                      /* number of records to skip on tape      */
    long curcat;                    /* output band being processed            */
    long curblk;                    /* current line within the input block    */
    long status;                    /* function return status code            */
    long out_ns;                    /* Output number of bytes                 */
    long in_byte;                   /* number of bytes to read from tape      */
    long bytes_read;                /* number bytes read from tape            */
    long one = 1;                   /* value of one                           */
    long mode = 0;                  /* tape I/O error flag                    */
    long zero = 0;                  /* value of zero                          */
    long fatal = -1;                /* Fatal error code                       */
    long nonfatal = 1;              /* Non-fatal error code                   */
    long cvtflg = SYS_SET;          /* Under/over flow flag - let system set  */

    short not_eof = TRUE;           /* flag to test EOF on tape               */

    unsigned char *in_buf;          /* buffer read from image on tape         */
    unsigned char *in_ptr;          /* pointer to input buffer window         */
    unsigned char *out_ptr;         /* pointer to output image buffer band 
                                       loc's                                  */

    double outmax[MAXBND];          /* max pixel values of output image       */
    double outmin[MAXBND];          /* min pixel values of output image       */

    /* Calculate the number of output samples
       -----------------------------------------*/
    out_ns = ((window[NS] - 1) / *sinc) + 1;

    /* Calculate the number of input bytes
       --------------------------------------*/
    in_byte = *blksiz * inspc[1] * datasize[*idtype];

    /* Allocate memory for the input buffer
       ---------------------------------------*/
    in_buf = (unsigned char *) malloc(in_byte);
    if (in_buf == NULL)
    {
        c_errmsg("Error allocating dynamic memory","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }

    /* If min max values are to be computed, call inmnmx to
       initialize the min max array for the output data type.  
       --------------------------------------------------------*/
    if (*minmax)
        c_inmnmx(idtype,&inspc[2],outmin,outmax);

    /* Start processing message
       ---------------------------*/
    curent = 1;
    c_startm();

    /* Initial step to set up offset into the output image.
       -------------------------------------------------------*/
    status = c_estep(&gdesc);
    if (status != E_SUCC)
        cleanup(unit);

    /* Set for skipping records on tape to the starting line of the window
       ----------------------------------------------------------------------*/
    nrec = ((window[SL] - 1) * inspc[2]) / *blksiz;

    /* If HP flag is set strip the the zero value pixel at the end off
       ------------------------------------------------------------------*/
    if (*hp_flag)
        --out_ns;

    /* Initialize the current line in the block: First, find the starting line 
       in the image.  Then subract off the line increment offset (this will be
       added back in later).  Then subtract off one (this will be added back in
       later).  Subtract one to make it zero relative rather than one relative. 
       Finally, add one block size to ensure that the first tpread will take 
       place.
       ----------------------------------------------------------------------*/
    curblk = (( window[SL] - 1 ) * inspc[2] ) + 1;
    curblk -= ((*linc - 1) * inspc[2]);
    curblk -= 2;
    curblk += (*blksiz);

    /* Process each line
       --------------------*/
    for (curent=0, line=0; (line<window[NL])&&(not_eof); line += *linc,curent++)
    {
        /* Skip records on tape to starting line of the window
           ------------------------------------------------------*/
        if (nrec)
            not_eof = skip_record(unit,nrec,FALSE);

        curblk += ((*linc - 1) * inspc[2]);

        /* Process each band
           --------------------*/
        for (curcat = 0, band = 0; (band < inspc[2]) && (not_eof); band++)
        {
            /* Read image line from tape for current band
               ---------------------------------------------*/
            curblk++;

            if (curblk > *blksiz - 1)
            {
                bytes_read = c_tpread(unit,in_buf,&in_byte,&mode);
                not_eof = check_tpread(bytes_read,unit, in_buf,&in_byte,&mode);
                curblk = curblk % *blksiz;
            }

            /* Check to see if band is wanted
               ---------------------------------*/
            if ((*(bflag + band)) && (not_eof))
            {
                /* Position the input buffer pointer to the
                   starting window and block location
                   -------------------------------------------*/
                in_ptr = in_buf + (curblk * inspc[1] * datasize[*idtype]) + 
                    ((window[SS] - 1) * datasize[*idtype]);

                /* Position the output buffer pointer to
                   the beginning of the desired band
                   ----------------------------------------*/
                out_ptr = thebuf + (offset[curcat] * datasize[*idtype]);

                /* Subsample the image data if specified
                   ----------------------------------------*/
                if (*sinc > 1)
                    c_pxsamp(&window[NS],idtype,sinc,in_ptr);

                /* Convert the data from the input system format to the output
                   system format.  This includes byte swapping and conversion
                   of real numbers to the host system format.
                   ----------------------------------------------------------*/
                status = c_pxsys(insys,outsys,in_ptr,idtype,&out_ns,&zero,&zero,
                        &cvtflg);

                /* Copy the converted data to the output buffer
                   -----------------------------------------------*/
                c_pxcopy(in_ptr,out_ptr,idtype,&out_ns);

                /* Compute the min/max values if required
                   -----------------------------------------*/
                if (*minmax)
                    status = c_pxmnmx((long *)out_ptr,&out_ns,&outmin[band],
                            &outmax[band],idtype);
                ++curcat;
            }
        }

        /* Write image line for all bands to the output LAS image.  The
           input data is converted to the output data type by IMAGEIO.
           ---------------------------------------------------------------*/
        status = c_estep(&gdesc);
        if (status != E_SUCC)
            cleanup(unit);

        /* Skip records on tape to the next line for resampling
           -------------------------------------------------------*/
        nrec = ((curblk + ((*linc - 1) * inspc[2]) + 1) / *blksiz) - 1;
        if (nrec < 0) nrec = 0;
    } 

    /* If not at EOF skip to the EOF on tape
       ----------------------------------------*/
    if (not_eof)
    {
        /* Stop the processing message and calculate the
           number of records to be skipped to get to EOF
           ------------------------------------------------*/
        c_stoptm();
        nrec = (((inspc[0] * inspc[2]) - 1) / *blksiz) - 
            ((((window[SL] + (((window[NL] - 1) / *linc) * *linc))
               * inspc[2]) - 1) / *blksiz);

        /* Skip records to the end of the image
           ---------------------------------------*/
        not_eof = skip_record(unit,nrec,FALSE);
        if (not_eof)
        {
            /* The calculations did not bring the tape to the End Of File.  The
               number of lines in the image on tape is not correct.  Inform the
               user and skip to EOF anyway.
               ---------------------------------------------------------------*/
            bytes_read = c_tpread(unit,in_buf,&in_byte,&mode);
            if ((bytes_read != TAPE_EOF) && 
                    (bytes_read != TAPE_EOV) && 
                    (bytes_read != TAPE_EOT))
            {
                c_errmsg("EOF was not detected at completion","tapein-noeof",
                        &nonfatal);
                skip_file(unit,one);
            }

            if ((bytes_read == TAPE_EOV) || (bytes_read == TAPE_EOT))
            {
                if (curfil < nmrfil)
                {
                    close_tape(unit,REWOFFL);
                    mount_tape(unit,FALSE);
                }
                else
                    EOV_detected = 1;
            }
        }
    }

    /* Free space allocated for the input buffer
       --------------------------------------------*/
    free(in_buf);

    /* Update the min max file for the output image
       -----------------------------------------------*/
    if (*minmax)
        c_svmnmx(hosout,totbnd,outmin,outmax,"YES");

    return; 
} 
