/******************************************************************************
FUNCTION:       CP_BSQ

PURPOSE:        Reads image data from tape in BSQ format, converts it to the
                output data type, and copies (with resampling) to the LAS
                output image.

ALGORITHM 
  Calculate the number of bytes in the input buffer
  Allocate memory for the input buffer
  Calculate the number of samples after subsampling
  Allocate memory for the output buffer
  If min max values are to be computed
         call inmnmx to initialize the min max array for the output data type.  
  If HP flag is set
         strip the the zero value pixel at the end off
  Initialize variables
  Process each band
     Current band is to be included in the output image
        Process each line of the output image
           Skip records on tape to the starting line of the window
           Read image line(s) from tape for the current band
           Set input pointer to the starting sample and block location within
           the input buffer
           Subsample the image data if specified
           Convert the data from the input system format to the output system
           format.  This includes byte swapping and conversion of real numbers
           to the host system format.
           Compute the min/max values if required
           Output image line to the LAS image.  The input data is converted
           to the output data type by IMAGEIO.
           Set number of records to skip records on tape to the next line
           for resampling
        else 
                skip all records on tape for the current band

  Skip to the end of the image file on tape
        If an unexpected EOF is encountered, the user is informed
        If an expected EOF is not encountered, the user is informed
        (In both cases the LAS image is saved)
  Free space allocated for the input and output buffers
  Create/update the min max file for the output image.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "sysdef.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

#define outlin    curent
#define outbnd    curcat

void cp_bsq
(
    long *unit,                 /* tape unit number                           */
    long *insys,                /* Input system data is from                  */
    long *outsys,               /* Output system data written to              */
    char *hosout,               /* output host name                           */
    long *inspc,                /* input file specifications                  */
    long *window,               /* window area of image to be copied          */
    long *totbnd,               /* total number of bands to be copied         */
    long *idtype,               /* input image data type                      */
    long *blksiz,               /* number of lines in input block             */
    long *linc,                 /* line increment for subsample               */
    long *sinc,                 /* sample increment for subsample             */
    short *bflag,               /* flag indicating bands to be copied         */
    short *hp_flag,             /* boolean for HP computer system             */
    short *minmax,              /* min max flag                               */
    long *curbnd                /* current band to write to                   */
)
{
    extern long curfil;             /* current file being processed           */
    extern long nmrfil;             /* total # of files to be processed       */
    extern long curent;             /* output line being processed            */
    extern long curcat;             /* output band being processed            */
    extern long datasize[];         /* defines number bytes for each data type*/
    extern int EOV_detected;        /* EOV detected flag                      */
    extern struct FDESC * fdesc[];  /* image file descriptors used for image 
                                       I/O                                    */

    long band;                      /* input band being read                  */
    long line;                      /* current input image line               */
    long nrec;                      /* number of records to skip on tape      */
    long out_ns;                    /* output number of bytes                 */
    long in_byte;                   /* number of bytes to read from tape      */
    long curblk;                    /* current line within the input block    */
    long bytes_read;                /* number bytes read from tape            */
    long one = 1;                   /* value of one                           */
    long mode = 0;                  /* tape I/O error flag                    */
    long zero = 0;                  /* value of zero                          */
    long fatal = -1;                /* Fatal error code                       */
    long nonfatal = 1;              /* Non-fatal error code                   */
    long cvtflg = SYS_SET;          /* Under/over flow flag - let system set  */
    long nxtlin;                    /* next line (absolute)                   */
    long cntrec;                    /* current record (absolute)              */
    long nxtrec;                    /* next record (absolute)                 */
    long actrec;                    /* actual record currently at (absolute)  */

    short not_eof = TRUE;           /* flag to test EOF on tape               */

    char  msg[ERRLEN];              /* message to be outputted                */

    unsigned char *in_buf;          /* buffer read from image on tape         */
    unsigned char *in_ptr;          /* Pointer to the input buffer window     */
    unsigned char *out_buf;         /* buffer written to LAS image            */

    double outmax[MAXBND];          /* max pixel values of output image       */
    double outmin[MAXBND];          /* min pixel values of input image        */

    /* Calculate the number of bytes in the input buffer
       ----------------------------------------------------*/
    in_byte = *blksiz * inspc[1] * datasize[*idtype];

    /* Allocate memory for the input buffer
       ---------------------------------------*/
    in_buf = (unsigned char *) malloc(in_byte);
    if (in_buf == NULL)
    {
        c_errmsg("Error allocating dynamic memory.","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }

    /* Calculate the number of samples after subsampling
       ----------------------------------------------------*/
    out_ns = ((window[NS] - 1) / *sinc) + 1;

    /* Allocate memory for the output buffer
       ----------------------------------------*/
    out_buf = (unsigned char *) malloc(out_ns * datasize[*idtype]);
    if (out_buf == NULL)
    {
        free(in_buf);
        c_errmsg("Error allocating dynamic memory.","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }

    /* If min max values are to be computed, call inmnmx to
       initialize the min max array for the output data type.  
       --------------------------------------------------------*/
    if (*minmax)
        c_inmnmx(idtype,&inspc[2],outmin,outmax);

    /* If HP flag is set strip the the zero value pixel at the end off
       ------------------------------------------------------------------*/
    if (*hp_flag)
        --out_ns;

    /* Initialize variables
       -----------------------*/
    nxtlin = 0;
    nxtrec = 0;
    actrec = 0;

    /* Process each band
       --------------------*/
    for (band = 0, curcat = *curbnd; (band < inspc[2]) && (not_eof); band++)
    {
        /* Current band is to be included in the output image
           -----------------------------------------------------*/
        if (*(bflag + band)) 
        {
            curent = 1;
            c_startm();
            nxtlin = (band * inspc[0]) + window[SL];
            cntrec = nxtrec;
            nxtrec = ((nxtlin - 1) / *blksiz) + 1;
            nrec = nxtrec - cntrec - 1;
            if (nrec < 0)   nrec = 0;

            /* Process each line of the output image
               ----------------------------------------*/
            for (line = 0; ((line < window[NL]) && (not_eof)); line += *linc)
            {
                /* Skip records on tape to the starting line of the window
                   ----------------------------------------------------------*/
                if (nrec)
                    not_eof = skip_record(unit,nrec,FALSE);

                actrec += nrec;

                /* Read image line from tape for the current band
                   -------------------------------------------------*/
                if (actrec < nxtrec)
                {
                    bytes_read = c_tpread(unit,in_buf,&in_byte,&mode);
                    not_eof = check_tpread(bytes_read,unit, in_buf,&in_byte,
                                          &mode);
                    actrec++;
                }

                curblk = (nxtlin - 1) % *blksiz;

                /* Set input pointer to the starting sample and
                   block location within the input buffer
                   -----------------------------------------------*/
                in_ptr = in_buf + (curblk * inspc[1] * datasize[*idtype]) +
                    ((window[SS] - 1) * datasize[*idtype]);

                /* Subsample the image data if specified
                   ----------------------------------------*/
                if (*sinc > 1)
                    c_pxsamp(&window[NS],idtype,sinc,in_ptr);

                /* Convert the data from the input system format to the output 
                   system format.  This includes byte swapping and conversion 
                   of real numbers to the host system format.
                   ----------------------------------------------------------*/
                c_pxsys(insys,outsys,in_ptr,idtype,&out_ns,&zero,&zero,&cvtflg);

                /* Copy the converted data to the output buffer
                   -----------------------------------------------*/
                c_pxcopy(in_ptr,out_buf,idtype,&out_ns);

                /* Compute the min/max values if required
                   -----------------------------------------*/
                if (*minmax)
                    c_pxmnmx((long *)out_buf,&out_ns,&outmin[band],
                             &outmax[band],idtype);

                /* Output image line to the LAS image.  The input data
                   is converted to the output data type by IMAGEIO.
                   ------------------------------------------------------*/
                if (not_eof)
                {
                    if(c_ewrite(&fdesc[0],&curcat,&curent,out_buf,&one)==E_FAIL)
                        cleanup(unit);
                }

                /* Set number of records to skip on tape to get to the next line
                   -----------------------------------------------------------*/
                if ((line + *linc) < window[NL])
                {
                    nxtlin += *linc;
                    cntrec = nxtrec;
                    nxtrec = ((nxtlin - 1) / *blksiz) + 1;
                    nrec = nxtrec - cntrec - 1;
                    if (nrec < 0)  nrec = 0;
                }

                ++curent;
            }

            c_stoptm();

            ++curcat;
        }
        else
        {
            sprintf(msg,"Skipping band # %d on tape.",band+1);
            c_errmsg(msg," ",&nonfatal);
        }
    }

    /* Make sure that one has skipped to the end of the image file on tape.
       ----------------------------------------------------------------------*/
    nxtlin = inspc[2] * inspc[0];
    cntrec = nxtrec;
    nxtrec = ((nxtlin - 1) / *blksiz) + 1;
    nrec = nxtrec - cntrec;
    if (nrec)
        not_eof = skip_record(unit,nrec,FALSE);

    /* Skip to the end of the image file on tape.  If an unexpected EOF is 
       encountered, the user is informed.  If an expected EOF is not 
       encountered, the user is informed.  In both cases the LAS image is saved.
       ----------------------------------------------------------------------*/
    if (not_eof)
    {
        bytes_read = c_tpread(unit,in_buf,&in_byte,&mode);

        if ((bytes_read != TAPE_EOF) && 
                (bytes_read != TAPE_EOV) &&
                (bytes_read != TAPE_EOT))
        {
            c_errmsg("EOF was not detected at completion.","tapein-noeof",
                     &nonfatal);
            skip_file(unit,one);
        }

        if ((bytes_read == TAPE_EOV) || (bytes_read == TAPE_EOT))
            if (curfil < nmrfil)
            {
                close_tape(unit,REWOFFL);
                mount_tape(unit,FALSE);
            }
            else
            {
                EOV_detected = 1;
            }
    }

    /* Free space allocated for the input and output buffers
       --------------------------------------------------------*/
    free(in_buf);
    free(out_buf);

    /* Create/update the min max file for the output image.
       ------------------------------------------------------*/
    if (*minmax)
        c_svmnmx(hosout,totbnd,outmin,outmax,"YES");

    return;
}
