/******************************************************************************
NAME                           GETDATA

PURPOSE      Retrieve records 1 and 2 of a DDR file

PROJECT       LAS

ALGORITHM 
   Create the associated DDR file name from the input host image name.
   Open the DDR file for read access using label services.  
   Read the DDRINT record (first record).
   Retrieve the host system name.
   If the system specified in the DDR is not the same as the host system then:
        Set the convert flag to TRUE.
   Read the DDRDUB record (second one).
   If the convert flag is set to TRUE then:
        Retrieve the system code (integer) for the host system.
        Retrieve the system code (integer) for the DDR specified system.
        Convert the double precision elements of the second DDR record to the
         floating point format of the host system.
        Copy the host system name into the system name element of the DDR
         structure.
   Close the DDR file.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include "las.h"
#include "worgen.h"
#include "labserv.h"
#include "pixman.h"
#include "sysdef.h"
#include "util.h"
#include "protos.h"

int getdata
(
    char *hname,                  /* host image file name                     */
    struct DDR *ddr               /* ddr structure of record 1 and 2          */
)
{
    long    access;                /* file access type                        */
    long    action;                /* file close action flag                  */
    long    clen;                  /* length of char part of record           */
    long    ctemp;                 /* temporary storage for clen              */
    long    dlen;                  /* length of data part of record           */
    long    dtemp;                 /* temporary storage for dlen              */
    long    fd;                    /* file descriptor for opened file         */
    long    mode;                  /* mode for lsstat check                   */
    long    one = 1;
    long    status;                /* function return status code             */
    long    size = DDSIZE;         /* number of doubles in record 2 of DDR    */
    long    lsize = DISIZE;        /* number of doubles in record 2 of DDR    */
    long    flag = SYS_SET;        /* flag that sets the doubles in record 2 
                                      of the DDR to the maximum size of the 
                                      host machine if overflow or underflow 
                                      is detected in the conversion process   */
    long    dtyp = EDOUBLE;        /* data type of the buffer to be converted */
    long    ltyp = ELONG;          /* data type of the buffer to be converted */
    long    convflg = FALSE;       /* working conversion flag                 */
    long    under, over;           /* dummy underflow and overflow flags      */
    long    incode;                /* host system code                        */
    long    syscode;               /* DDR system code                         */
    long    sysflg = SETN;         /* system conversion flag (string to int)  */

    unsigned char *dbuf;           /* pointer to area where data is stuffed   */

    char    dtype[3];              /* type of data portion of record          */
    char    hostddr[CMLEN];        /* host name of DDR file                   */
    char    temp[2];               /* temporary buffer                        */
    char    insys[DDSYLN];         /* host system name                        */
    char    key[16];

    c_lsmknm(hname,"ddr",hostddr); /* make associated DDR file name */

    mode = 0;                      /* make sure DDR file exists on disk*/
    status = c_lsstat(hostddr,&mode);
    if (status != E_SUCC)
    {
        c_errmsg("DDR file does not exist on disk","getddr-exist",&one);
        return(E_FAIL);
    }

    access = 0;                               /* open DDR file for read */
    if (c_lsopen(&fd,hostddr,&access) != E_SUCC)
    {
        c_errmsg("Error returned from lsopen","getddr-call",&one);
        return(E_FAIL);
    }

    clen = DDSTCT * DDSYLN;          /* set up for and read in record 1  */
    ctemp = clen - 1;
    dlen = dtemp = DISIZE * 4;
    dbuf = (unsigned char *) ddr;
    strcpy(key,"DDRINT");
    if (c_lsread(&fd,key,&clen,&dlen,ddr->system,dbuf,dtype) != E_SUCC)
    {
        c_errmsg("Error returned from lsread","getddr-call",&one);
        return(E_FAIL);
    }
    if ( (clen != ctemp) || (dlen != dtemp) )
    {
        c_errmsg("Error reading record 1 of associated DDR file",
                 "getddr-read",&one);
        return(E_FAIL);
    }

    if (c_getsys(insys) != E_SUCC)        /* retrieve the host system name */
    {
        c_errmsg("Error returned from getsys","getddr-call",&one);
        return(E_FAIL);
    }

    if (strcmp(insys,ddr->system) != 0)   /* if the DDR system and the host   */
        convflg = TRUE;                   /* are not the same then set the    */
    /* conversion flag                  */
    if (convflg)                          /* if the conversion flag is set */
    {
        if (c_sysset(insys,&incode,&sysflg)!=E_SUCC)/*retrieve the host system*/
        {
            c_errmsg("Error returned from sysset","getddr-call",&one);
            return(E_FAIL);
        }

        if (c_sysset(ddr->system,&syscode,&sysflg)!=E_SUCC) /*retrieve the DDR*/
        {
            c_errmsg("Error returned from sysset","getddr-call",&one);
            return(E_FAIL);
        }

        if (c_pxsys(&syscode,&incode,dbuf,&ltyp,&lsize,&under,&over,
                &flag) != E_SUCC)               /* perform the conversion   */
        {
            c_errmsg("Error returned from pxsys","getddr-call",&one);
            return(E_FAIL);
        }

        /* Commented out from c_getddr.c, do not want to change the system name
           -------------------------------------------------------------------*/
        /*strcpy(ddr->system,insys);*/          /* copy the host system name */
        /* into the DDR structure    */
    }
    clen = ctemp = 0;                     /* set up for and read in record 2  */
    dlen = dtemp = DDSIZE * 8;            /* no character part for this record*/
    dbuf = (unsigned char *) &(ddr->proj_coef[0]);
    strcpy(key,"DDRDUB");
    if (c_lsread(&fd,key,&clen,&dlen,temp,dbuf,dtype) != E_SUCC)
    {
        c_errmsg("Error returned from lsread","getddr-call",&one);
        return(E_FAIL);
    }
    if ( (clen != ctemp) || (dlen != dtemp) )
    {
        c_errmsg("Error reading record 2 of associated DDR file",
                 "getddr-read",&one);
        return(E_FAIL);
    }

    if (convflg)                          /* if the conversion flag is set */
    {
        /* perform the conversion   */
        if (c_pxsys(&syscode,&incode,dbuf,&dtyp,&size,&under,&over,
                    &flag)!=E_SUCC)
        {
            c_errmsg("Error returned from pxsys","getddr-call",&one);
            return(E_FAIL);
        }
    }

    action = 0;                               /* close associated DDR file */
    if (c_lsclos(&fd,hostddr,&action) != E_SUCC)   
    {
        c_errmsg("Error returned from lsclos","getddr-call",&one);
        return(E_FAIL);
    }

    return (E_SUCC);
}
