#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "protos.h"
#define ASCI 2


/******************************************************************************
FUNCTION:       PROCESS

PURPOSE:        Acts as the control function.  It determines if the tape needs
                to be mounted or reopened and calls the appropriate function.
                It also calls routines to skip the specified number of image
                files, read the DDR file (if it exists), and to skip the
                specified number of header records in the image file.  It
                then opens the LAS image files for either sequential I/O or
                random I/O depending on the bands format of the image on tape.
                A routine is then called to copy the image on tape into the
                LAS image.  If multiple tape images are to be placed in one
                image and image compression is currently on, compression will
                be temporarily disabled.

ALGORITHM REFERENCES    none
*******************************************************************************/
void process
(
    long *access,               /* write access to output image              */
    long bands[][MAXBND+1],     /* bands to be processed within image        */
    long *blksiz,               /* number of lines in block                  */
    long *curbnd,               /* current band to write to                  */
    long *idtype,               /* data type of input image                  */
    long *inhdr,                /* number of header records to skip          */
    long *inspc,                /* input image file specifications           */
    long *insys,                /* Input system data is from                 */
    long *linc,                 /* line increment for subsample              */
    long *m2one,                /* flag for placing multiple images on one   */
    long *msgtim,               /* elapsed time between display msg          */
    long *odtype,               /* data type of output image                 */
    long *outsys,               /* Output system data is to be put to        */
    long *sinc,                 /* sample increment for subsample            */
    long *skip,                 /* number files to skip prior to copy        */
    long *totbnd,               /* total number of bands to be copied        */
    long window[][4],           /* window area of image to be copied         */
    char *bform,                /* input band format (BSQ or BIL)            */
    char *hosout,               /* output host names                         */
    short *ddr_flag,            /* DDR flag                                  */
    short *hp_flag,             /* HP computer system flag                   */
    short *minmax,              /* min max flag                              */
    long *compflg,              /* image compression flag                    */
    char *tddrnam,              /* temporary ddr name                        */
    long nbands[]               /* Array containing number of bands in output*/
)

{
    extern char *drive[1];       /* physical tape drive name                  */
    extern struct GDESC * gdesc; /* group descriptor used for image I/O       */
    extern long curfil;          /* current file being processed              */
    extern long outcnt;          /* Number of output files opened             */
    extern long offset[];        /* offsets into "thebuf" for output bands    */
    extern struct FDESC * fdesc[];/* image file descriptors used for image I/O*/
    extern struct PARBLK vblock; /* TAE parameter block                       */
    extern unsigned char *thebuf;/* declare image I/O buffer                  */
    extern char out[][CMLEN];    /* output image files opened                 */

    double dlinc;                /* Double version of line increment          */
    double dsinc;                /* Double version of sample increment        */

    long  bufsiz;                /* image I/O buffer size                     */
    long  cflag    = NOCOMB;     /* Flag indicating image are not combined    */
    long  conv[3];               /* output conversion array                   */
    long  fatal    = -1;         /* Fatal error code                          */
    long  msgflg;                /* Processing message format flag            */
    long  nonfatal = 1;          /* Non-fatal error code                      */
    long  one      = 1;          /* value of one                              */
    long  outspc[3];             /* output image file specifications          */
    long  status;                /* function return status code               */
    long  temp_ns;               /* temporary number of samples variable      */
    long  unit;                  /* tape unit number                          */
    long  zero     = 0;          /* value of zero                             */

    short *bflag;                /* flag to indicate bands to copy            */
    short found_ddr;             /* Flag indicating that a DDR file was found */
    short not_eof;               /* end of file flag                          */

    char  ddrnam[1][CMLEN];      /* Temp DDR file to update the output DDR    */
    char  msg[ERRLEN];           /* message to be written to terminal         */


    /* Mount the tape the first time
       --------------------------------*/
    if (*drive[0] == '\0')
        mount_tape(&unit,FALSE);  
    else 
        reopen_tape(&unit);

    /* Skip over any image files
       ----------------------------*/
    if (*skip)
        skip_images(&unit,*skip);

    /* Read the DDR file from the tape
       ----------------------------------*/
    copy_ddr(bands, idtype, inspc, insys, m2one, totbnd,
            &unit, &found_ddr, hp_flag, ddrnam[0],&curfil,tddrnam);

    /* Skip over any header records in the image
       --------------------------------------------*/
    if (*inhdr)
    {
        not_eof = skip_record(&unit,*inhdr,FALSE);
        if (!(not_eof))
        {
            c_errmsg("Invalid number of header records","tapein-nheadrec",
                    &nonfatal);
            c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
        }
    }

    /* Compute the number of samples to be taken from the input buffer
       ------------------------------------------------------------------*/
    temp_ns = compute_ns(&unit,idtype,blksiz);
    bflag = check_band(inspc,bands[0]);
    outspc[1] = 0;

    /* Check to determine if the zero at the end of an HP buffer is to be 
       removed
       ------------------------------------------------------------------*/
    if ((*hp_flag) && (*idtype == EBYTE) && (inspc[1]+1 == temp_ns) && 
            (inspc[1] % 2 != 0))
    {
        ++inspc[1];
        --outspc[1];
    }
    else
    {
        *hp_flag = FALSE;
        if (temp_ns != inspc[1])
        {
            sprintf(msg,"Resetting number of samples from %d to %d",
                    inspc[1],temp_ns);
            c_errmsg(msg," ",&nonfatal);
            inspc[1] = temp_ns;
        }
    }

    /* Check the to make sure the input window is valid
       ---------------------------------------------------*/
    chk_window(inspc, window[0]);
    outspc[0]  = (window[0][NL] - 1) / *linc + 1;
    outspc[1] += (window[0][NS] - 1) / *sinc + 1;
    if (*totbnd == 0)
        *totbnd = inspc[2];
    outspc[2] = *totbnd;

    /* If m2one and this isn't the first tape file being processed then let 
       odtype be 0.  This will open the image as the appropriate type.  In all 
       other cases if the output data type was not specified set it to the 
       input data type
       -----------------------------------------------------------------------*/
    if (*m2one && (curfil > 1))
        *odtype = 0;
    else if (*odtype == 0)
        *odtype = *idtype;

    /* Process BIL tape
       -------------------*/
    if (strcmp(bform,"BIL") == 0)
    {
        /* Set the conversion array to convert to the output data type
           when the buffer is written to the output image file.
           --------------------------------------------------------------*/
        conv[0] = *odtype;

        /* If the m2one flag is set and image compression is on, turn off image
           compression.
           -------------------------------------------------------------------*/
        if(*m2one && *compflg == TRUE)
        {
            conv[1] = NO_COMPRES;
            conv[2] = 0;
        }
        else
            conv[1] = 0;


        /* Open the output image file for sequential write access
           ---------------------------------------------------------*/
        status = c_eopens(&fdesc[0],hosout,&outspc[2],&one,&one,&outspc[0],
                &outspc[1],offset,access,idtype,&zero,conv);
        if (status != E_SUCC)
            cleanup(&unit);

        fdesc[1] = 0;

        /* Hold onto the image file name for clean-up
           ---------------------------------------------*/
        strcpy(out[outcnt],hosout);
        outcnt++;

        /* Check to see if a DDR record was found and
           the user wants the DDR record updated
           ---------------------------------------------*/
        if ((*ddr_flag) && ((found_ddr) || (found_ddr == ASCI)))
        {
            /*
               nbands[0] = *totbnd;
               nbands[0] = 1;
             */
            if (nbands[0] == 0)
                nbands[0] = inspc[2];
            dlinc = (double) *linc;
            dsinc = (double) *sinc;
            status = c_upddr(&vblock,ddrnam,hosout,&one,window,bands,nbands,
                    &dlinc, &dsinc,totbnd,&cflag);
            if (status != E_SUCC)
                cleanup(&unit);
        }

        /* Allocate I/O buffer the first time
           -------------------------------------*/
        if ((thebuf = (unsigned char *) c_ealloc(fdesc,&bufsiz)) == NULL)
        {
            c_errmsg("Error allocating dynamic memory","tapein-alloc",
                     &nonfatal);
            c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
        }

        /* Group the file descriptors, so that only one estep is needed
           ---------------------------------------------------------------*/
        status = c_egroup(fdesc,&gdesc,thebuf,&bufsiz);
        if (status != E_SUCC)
            cleanup(&unit);

        /* Initialize the processing message
           ------------------------------------*/
        msgflg = ALLCAT;
        c_inittm("TAPEIN","Processed","line","band(s)","",&outspc[0],&outspc[2],
                msgtim,&msgflg);

        /* Copy the BIL formatted tape to the image file
           ------------------------------------------------*/
        cp_bil(&unit,insys,outsys,hosout,inspc,window[0],totbnd,idtype,blksiz,
                linc,sinc,bflag,hp_flag,minmax);

        /* Close the image file
           -----------------------*/
        c_egclse(&gdesc);
    }
    else if (strcmp(bform,"BSQ") == 0)   /* Band Sequential processing        */
    {
        /* Set the conversion array to convert to the output data
           type when the buffer is written to the output image file.
           ------------------------------------------------------------*/
        conv[0] = *odtype;

        /* If the m2one flag is set and image compression is on, turn off image
           compression.
           ------------------------------------------------------------------*/
        if(*m2one && *compflg == TRUE)
        {
            conv[1] = NO_COMPRES;
            conv[2] = 0;
        }
        else
            conv[1] = 0;

        /* Open the image file for random write access
           ----------------------------------------------*/
        status = c_eopenr(&fdesc[0],hosout,&outspc[2],&one,&one,&outspc[0],
                &outspc[1],access,idtype,conv,&one);
        if (status != E_SUCC)
            cleanup(&unit);

        /* Hold onto the image file name for clean-up
           ---------------------------------------------*/
        strcpy(out[outcnt],hosout);
        outcnt++;

        /* Check to see if a DDR record was found and
           the user wants the DDR record updated
           ---------------------------------------------*/
        if ((*ddr_flag) && ((found_ddr) || (found_ddr == ASCI)))
        {
            /*
               nbands[0] = *totbnd;
               nbands[0] = 1;
             */
            if (nbands[0] == 0)
                nbands[0] = inspc[2];
            dlinc = (double) *linc;
            dsinc = (double) *sinc;
            status = c_upddr(&vblock,ddrnam,hosout,&one,window,bands,nbands,
                    &dlinc, &dsinc,totbnd,&cflag);
            if (status != E_SUCC)
                cleanup(&unit);
        }

        /* Initialize the processing message
           ------------------------------------*/
        msgflg = ONECAT;
        c_inittm("TAPEIN","Processed","line","band","",&outspc[0],&outspc[2],
                msgtim,&msgflg);

        /* Copy the BSQ formatted tape to the image file
           ------------------------------------------------*/
        cp_bsq(&unit,insys,outsys,hosout,inspc,window[0],totbnd,idtype,blksiz,
                linc,sinc,bflag,hp_flag,minmax,curbnd);

        /* Close the image file
           -----------------------*/
        c_eclose(&fdesc[0]);
    }

    /* Delete the temporary file used to hold the input DDR information from 
       tape. The file is then used in UPDDR to update the output DDR information
       -----------------------------------------------------------------------*/
    if (((found_ddr) || (found_ddr == ASCI)))
        c_delfil(ddrnam[0]);

    /* Close the tape without rewinding it
       --------------------------------------*/
    close_tape(&unit,NOREW);

    free(bflag);

    return;
}
