#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "vll.h"
#include "util.h"
#include "protos.h"

/******************************************************************************
FUNCTION:       SKIP_FILE

PURPOSE:        Skips a specified number of physical files.  If the EOV or
                EOT are encountered during the forward skip, the next tape
                is mounted and the skip file command is issued for the
                remaining files that were to be skipped.

ALGORITHM 
  If no files are to be skipped return
  Forward space the tape the specified number of files

ALGORITHM REFERENCES    none
*******************************************************************************/
void skip_file
(
    long *unit,         /* tape unit number                                   */
    long skip           /* number of image files to skip                      */
)

{
    long mode = HANDLE;     /* tape I/O does NOT output error msg            */
    long fatal = -1;        /* Fatal error code                              */
    long nfile;             /* # of files to skip on tape                    */
    long status;            /* function return status code                   */

    /* If no files are to be skipped return
       ---------------------------------------*/
    if (skip <= 0)
        return;

    /* Forward space the tape the specified number of files
       -------------------------------------------------------*/
    nfile = skip;
    status = c_tpfsf(unit,&nfile,&mode);
    switch (status)
    {
        case TAPE_SUCC:
            break;

        case TAPE_EOV:
        case TAPE_EOT:
            close_tape(unit,REWOFFL);
            mount_tape(unit,FALSE);
            nfile = skip - nfile;
            skip_file(unit,nfile);
            break;

        default:
            c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
            break;
    }

    return;
}

/******************************************************************************
FUNCTION:       SKIP_IMAGES

PURPOSE:        Skip image files on tape.  An image file may consist of one
                or two physical tape files.  It may consist of a ddr file,
                and always the image file.

ALGORITHM 
  If no image files are to be skipped return
  Write an informative message about skipping X number of image files
  Allocate memory for tape reads
  Skip over the possible DDR file and the image file
      Determine if the file has a DDR file and skip over it also
      Skip over the image file and/or DDR file
  Free allocated memory

ALGORITHM REFERENCES    none
*******************************************************************************/
void skip_images
(
    long *unit,                 /* tape unit number                           */
    long skip                   /* number of image files to skip              */
)

{
    long bytes_read;                /* actual # of bytes read from tape      */
    long index;                     /* loop index                            */
    long nfile;                     /* # of files to skip on tape            */
    long mode = 0;                  /* tape I/O does NOT output error msg    */
    long fatal = -1;                /* Fatal error code                      */
    long nonfatal = 1;              /* Non-fatal error code                  */
    long nbyte;                     /* # of bytes to be read from tape       */

    short not_eof;                  /* flag to check EOF, EOV, or EOT        */

    char  *key;                     /* Key of the Label services record      */
    char  key2[100];                /* Key of the Label services record      */
    char  msg[ERRLEN];              /* message to be outputted               */
    char  header[HDRL+1];           /* Label services header information     */
    unsigned char  *buf;            /* buffer pointer to read into           */

    struct TAPEDESC *tp;            /* tape drive description                */

    /*
       If no image files are to be skipped return
     */
    if (skip <= 0)
        return;

    /*
       Write an informative message about skipping X number of image files
     */
    sprintf(msg,"Skipping %d image file(s) on tape.",skip);
    c_errmsg(msg," ",&nonfatal);

    /*
       Allocate memory for tape reads
     */
    tp = (struct TAPEDESC *) *unit;
    nbyte = tp->maxbyte;
    buf = (unsigned char *) malloc(nbyte);
    if (buf == NULL)
    {
        c_errmsg("Error allocating dynamic memory","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }

    /*
       Skip over the possible DDR file and the image file
     */
    for (index = 0; index < skip; index++)
    {
        bytes_read = c_tpread(unit, buf,&nbyte,&mode);
        not_eof = check_tpread(bytes_read,unit, buf,&nbyte,&mode);

        /*
           Determine if the file has a DDR file and skip over it also
         */
        strncpy(header,(char *) buf,HDRL);
        header[HDRL] = '\0';
        key = squeeze(header+LENL+TYPL,(long) strlen(header+LENL+TYPL));
        strncpy(key2,&header[2],5);
        key2[5] = '\0';

        if ((not_eof) && ((strcmp(key,"DDRINT") == 0)|| 
                (strcmp(key2,"IMAGE") == 0)))
            nfile = 2;
        else
            nfile = 1;

        /*
           Skip over the image file and/or DDR file
         */
        skip_file(unit,nfile);
    }

    /*
       Free allocated memory
     */
    free(buf);

    return;
}

/******************************************************************************
FUNCTION:       SKIP_RECORD

PURPOSE:        Skips a specified number of records on tape.  The skip is
                actually implemented by reading the specified number of
                records.

ALGORITHM 
  If no records are to be skipped; return
  If display flag is set turn off the processing message
  Forward space the specified number of records
  If the display flag is set restart the processing messages

ALGORITHM REFERENCES    none
*******************************************************************************/
short skip_record
(
    long *unit,         /* tape unit number                                   */
    long skip,          /* number of header records to skip                   */
    int flag            /* flag to turn off display utility                   */
)

{
    long  init_skip;        /* number of records to skip                     */
    long  status;           /* function return status code                   */
    long  mode = 0;         /* tape I/O does NOT output error msg            */
    long  nonfatal = 1;     /* Non-fatal error code                          */
    long fatal = -1;        /* Fatal error code                              */
    short not_eof = TRUE;   /* eof flag                                      */

    /*
       If no records are to be skipped; return
     */
    if (skip <= 0)
        return(not_eof);

    /*
       If display flag is set turn off the processing message
     */
    if (flag)
        c_stoptm();

    /*
       Forward space the specified number of records
     */
    init_skip = skip;
    status = c_tpfsr(unit,&skip,&mode);
    switch (status)
    {
        case TAPE_SUCC:
            not_eof = TRUE;
            break;

        case TAPE_EOF:
            not_eof = FALSE;
            break;

        case TAPE_EOV:                               

        case TAPE_EOT:
            close_tape(unit,REWOFFL);
            mount_tape(unit,TRUE);
            skip = init_skip - skip;
            status = c_tpfsr(unit,&skip,&mode);
            break;

        default: 
            c_errmsg("Error forward skipping records on tape."," ",&nonfatal);
            c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
            break;
    }

    /*
       If the display flag is set restart the processing messages
     */
    if (flag)
        c_startm();

    return(not_eof);
}
