#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "protos.h"

#define MAX_VOL         15

/******************************************************************************
FUNCTION:       MOUNT_TAPE

PURPOSE:        Physically mounts a tape.  The mount message is determined by
                the value of the global variables "comment" and "curvol",
                where "curvol" is an index into array "comment".  After the
                tape is mounted the tape drive name and "curvol" are saved
                in the TAE parameter block.

ALGORITHM 
  If disable display message flag is set stop the processing message
  Mount the current tape volume
  Save the current volume number and the drive the tape was mounted on
  If processing message was stoped, restart it

ALGORITHM REFERENCES    none
*******************************************************************************/

void mount_tape
(
    long *unit,         /* tape unit number                                   */
    int flag            /* flag to disable display messages                   */
)

{
    extern long  curvol;            /* current tape volume being process     */
    extern char comment[][CMLEN];   /* tape mount message to the operator    */
    extern char *drive[1];          /* physical tape drive name              */
    extern struct PARBLK vblock;    /* TAE parameter block                   */

    long skip = 0;                  /* skip zero files on open               */
    long fatal = -1;                /* Fatal error code                      */
    long mode = HANDLE;             /* tape I/O outputs error messages       */
    long access = TREAD;            /* open tape for read access             */
    long rewopt = REWIND;           /* rewind option                         */
    long status;                    /* function return status code           */

    char density[5];                /* tape density in BPI                   */
    char tapeid[CMLEN];             /* tape id                               */

    strcpy(density,"0");
    /*
       If disable display message flag is set stop the processing message
     */
    if (flag)
        c_stoptm();

    /*
       Mount the current tape volume
     */
    curvol++;
    status = c_tpopen(unit,&skip,&mode,&rewopt,density,&access,comment[curvol],
            tapeid); 
    if (status != TAPE_SUCC)
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);

    /*
       Save the current volume number and the drive the tape was mounted on
     */
    drive[0] = comment[curvol];
    status = q_intg(&vblock,"CURVOL",1,(int*)&curvol,P_UPDATE);
    status = q_string(&vblock,"DRIVE",1,drive,P_UPDATE);
    status = q_out(&vblock);

    /*
       If processing message was stoped, restart it
     */
    if (flag)
        c_startm();

    return;
}



/******************************************************************************
FUNCTION:       REOPEN_TAPE

PURPOSE:        Reopens a tape device.  The tape drive name is stored in the
                global "drive".  The drive name is saved in the TAE parameter
                block.

ALGORITHM 
  Reopen the tape drive
  Update the parameter block with the tape drive

ALGORITHM REFERENCES    none
*******************************************************************************/
void reopen_tape
(
    long *unit                  /* tape unit number                           */
)

{
    extern struct PARBLK vblock;    /* TAE parameter block                   */
    extern char *drive[1];          /* string descriptor for tape drive      */
    long access = TREAD;            /* open tape for read access             */
    long rewopt = NOREW;            /* rewind option                         */
    long skip = 0;                  /* skip zero files on open               */
    long mode = 0;                  /* tape I/O outputs error messages       */
    long fatal = -1;                /* Fatal error code                      */
    long status;                    /* function return status code           */

    char density[5];                /* tape density in BPI                   */

    strcpy(density,"0");
    /*
       Reopen the tape drive
     */
    status = c_tpreop(unit,&skip,&mode,&rewopt,density,&access,drive[0]);
    if (status != TAPE_SUCC)
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);

    /*
       Update the parameter block with the tape drive
     */
    status = q_string(&vblock,"DRIVE",1,drive,P_UPDATE);
    status = q_out(&vblock);

    return;
}



/******************************************************************************
FUNCTION:       CLOSE_TAPE

PURPOSE:        Closes the tape device.

ALGORITHM 
  Close the tape

ALGORITHM REFERENCES    none
*******************************************************************************/
void close_tape
(
    long *unit,         /* tape unit number                                  */
    long rewopt         /* rewind option                                     */
)

{
    long one = 1;           /* value of one                                  */
    long mode = NO_HANDLE;  /* tape I/O outputs error messages               */

    /* Close the tape
       -----------------*/
    c_tpclos(unit,&mode,&rewopt,&one);

    return;
}
