/******************************************************************************
FUNCTION:       TAPEIN

PURPOSE:        Reads multiple image files from tape and copies the images 
                into LAS image files on disk.  Each image file may be
                processed with a unique set of parameters.  The image on
                tape may be windowed and/or subsampled when copied to the
                disk file.  If the end of tape (EOT) or the end of volume
                (EOV) is encountered while processing a file from tape, the
                user and the operator are signaled to mount the next tape 
                volume to complete the processing of the image files. 

                The user has the option to copy all available label records
                into the LAS image file.  If the label records exist on tape,
                it is assumed the label records and the image data are in
                two separate tape files.  The data descriptor record is
                updated to represent the generated output image.  However,
                all other label records are copied directly from tape to
                the LAS image file.  This means that some of the other 
                label records may not be valid if the image was windowed,
                subsampled, or converted to a new data type.  It is the
                user's responsibility to update these label records.

        NOTE:   Multiple tape files are processed by the TAPEIN pdf file.
                The executable code processes one tape file per execution run.


PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  -----------------------------------------------
  1.0    01/86  K.Gacke          Original development
  1.2    12/86  K.Gacke          For BYTE images with an odd number of samples
                                 from an HP system, the last pixel (zero valued
                                 pixel) is stripped off.
  5.0    01/89  D.Hollaren       Conversion to LAS-5.0
  6.0    10/90  B.Ailts          Increased the buffer in copy_ddr to MAX_BLKSIZ
                                 in order to run correctly on a sun4 tape drive
  7.0    12/90  T.Baltzer        Replace MAX_BLKSIZ with MAXTAPEWT so that
                                 images won't exceed the maximum
  7.1    11/91  D.Etrheim        Correct problem of allocating space that
                                 occurred on SG when reading one band from a
                                 multiple band image.
                                 ADP #6108  Override TOTLINES, TOTSAMPS,
                                 TOTBANDS with the DDR values. 
  7.2    02/92  J.Fenno          PRF Z0697.  Rewrote user's guide for clarity.
                                 Changed several parameter names and their
                                 default values.
  7.3    04/92  S.Nelson         Changed function call from c_getddr to getdata.
                                 If the current system is different from the
                                 insystem, "getdata" does not change the 
                                 system  name.
  7.4    12/92  K.Sheffield      Turned off image compression when multiple 
                                 images from tape are placed into one disk
                                 image.
  7.5    09/93  D.Etrheim        Changed so the maximum number of bytes to be
                                 read from tape is based on TAPEDESC rather
                                 than MAXTAPEWT
  7.6    08/94  T.Mittan         Modified to read ASCII DDR's correctly.
  7.7    10/94  T.Mittan         Modified to read Mulitple bands to one
                                 image and create a valid DDR if the 
                                 individual DDR's are the same.
  7.8    08/99  T.Beckmann       Added the option to return an EOV flag when
                                 the end of a volume or tape is detected to
                                 support newer tape formats.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT                 LAS

ALGORITHM 
  TAE parameters are read into the program.  In addition to the 
  parameters specified by the user, there are various TAE local variables 
  indicating if the tape needs to be mounted and whether or not the tape 
  is to be dismounted when processing of the current image is finished. 

  TAPEIN copies tape file(s) to LAS images using a unique set of input
  parameters for each file.  TAPEIN initializes the TAE local variables so 
  that the tape is mounted for the first image file, and dismounted after
  the last image file has been copied.  

  Description of tape copy routine (copy one tape file to LAS image):
  On the first invocation of the tape copy routine, the tape is mounted 
  and the physical tape drive name is saved so that the tape may be 
  reopened.  After processing of the image file is complete, the tape 
  is closed, but not dismounted, and control is returned to TAPEIN.  The
  tape copy routine is invoked for each image file that is to be copied
  starting at the current position of the tape.

  Once the tape file is opened, the specified number of files are 
  skipped.  At this point, the tape is at the desired file to be copied 
  into a LAS image.  A check is made to see if a label file exists.  
  If parameter COPYFLG is set to yes, the label file is copied to the 
  label records of the LAS image.  Otherwise the label file is 
  skipped.  The tape is now at the beginning of the image data in either 
  BIL or BSQ format, and of any valid data type (BYTE, INTEGER*2, 
  INTEGER*4, REAL*4).  Each line of image data is read into a buffer.  
  It is then determined whether or not the data is to be written to the 
  LAS image file.  Various checks are made to insure the line of data 
  is to be included in the output image file including:
        1)  Whether the current image line is from a band that is to
            be included in the output image.
        2)  Whether the current image line is within specified window.
        3)  Whether the current image line is to be included within
            the subsampling that was specified.
  If the current image line is to be included, only the samples of the
  image line after subsampling are written to the output image file
  converted to the specified data type.  When the complete window has
  been written to the output image file, any lines remaining in the 
  tape file are skipped and the LAS image file is closed.

  If at any time during processing an end of volume (EOV) or and end of
  tape (EOT) is encountered, the operator is signaled to mount the next
  tape volume so that processing may continue.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "protos.h"

#define MAX_VOL       15

/* Defines the number of bytes for each of the four data types 
--------------------------------------------------------------*/
long datasize[NDTYPES] = {0,1,2,4,4,8};

long  tpin_status = TAPE_SUCC;  /* tapein status (SUCCESS or FAIL)            */

long  curfil;                   /* current file being processed               */
long  curvol;                   /* current tape volume being process          */
long  nmrfil;                   /* total # of files to be processed           */
long  return_EOV;               /* flag to return EOV_detected */
int   EOV_detected = 0;         /* EOV detected flag */
char  comment[MAX_VOL][CMLEN];  /* tape mount message to the operator         */
char  *drive[1];                /* physical tape drive name                   */

long outcnt = 0;                /* Number of opened files                     */
char out[2][CMLEN];             /* Names of opened files for clean_up         */

/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
struct FDESC *  fdesc[2];       /* image file descriptors used for image I/O */
struct GDESC *  gdesc;          /* group descriptor used for image I/O       */
long   offset[MAXBND + 1];      /* offsets into "thebuf" for output bands    */
struct PARBLK vblock;           /* TAE parameter block                       */
unsigned char *thebuf;          /* declare image I/O buffer                  */

int main
(void)
{
    struct VARIABLE *p_find();      /* function to find a TAE variable       */

    static long bands[1][MAXBND+1]; /* bands to be processed within image    */
    static long inspc[3];           /* input file specifications             */
    static long window[1][4];       /* window of input image                 */

    long  insys;                    /* Input system data is from             */
    long  outsys;                   /* Output system data is to be put to    */
    long  inhdr;                    /* number of header records to skip      */
    long  blksiz;                   /* number of lines in a block            */
    long  linc;                     /* line increment for subsample          */
    long  m2one;                    /* flag for placing multiple images in one*/
    long  sinc;                     /* sample increment for subsample        */
    long  skip;                     /* number files to skip prior to copy    */
    long  totbnd;                   /* total number of bands to be copied    */
    long  odtype;                   /* data type of output image             */
    long  idtype;                   /* data type of input image              */
    long  msgtim;                   /* elapsed time between display msg      */
    long  zero = 0;                 /* value of zero                         */
    long  one = 1;                  /* value of one                          */
    long  nonfatal = 1;             /* Non-fatal error code                  */
    long  curbnd;                   /* current band to be written to         */
    long  access;                   /* output image file access              */
    long  compflg;                  /* image compression flag                */
    long  nbands;                   /* number of bands                       */

    short minmax;                   /* min max flag                          */
    short hp_flag;                  /* flag for HP computer system           */
    short ddr_flag;                 /* flag to skip header records on tape   */

    static char bform[4];           /* input band format (BSQ or BIL)        */
    static char hosout[1][CMLEN];   /* Output host name                      */
    static char taeout[1][CMLEN];   /* Output TAE name                       */
    char tddrnam[40];               /* temporary ddr name                    */

    /* Retrieve the input parameters
       --------------------------------*/
    getpar(&access, bands, &blksiz, &curbnd, &idtype, &inhdr, inspc, &insys,
            &linc, &m2one, &msgtim, &odtype, &outsys, &sinc, &skip, &totbnd,
            window, &ddr_flag, &hp_flag, &minmax, bform, hosout[0], taeout[0],
            &compflg, tddrnam, &nbands);

    /* Process the file
       -------------------*/
    process(&access, bands, &blksiz, &curbnd, &idtype, &inhdr, inspc, &insys,
            &linc, &m2one, &msgtim, &odtype, &outsys, &sinc, &skip, &totbnd,
            window, bform, hosout[0], &ddr_flag, &hp_flag, &minmax,&compflg,
            tddrnam,&nbands);


    /***************************************************************************
      The following code is used to write the history of all parameters for
      module TAPEIN.  However the TAE parameter block gets too large, and the 
      program hangs.  Until the TAE bug is fixed, the history contains only
      the parameters of the current tape file.

      p_rdb("tpin.his",0,&vblock_his,P_BYTES,P_ABORT);
      q_string(&vblock_his,"_PROC",1,"TAPEIN",P_ADD);
      q_string(&vblock_his,"_SUBCMD",0,"TAPEIN",P_ADD);
      q_out(&vblock_his);
      c_hstout(&vblock_his,hosout[0],hosout[0],&zero,"TAPEIN");
     **************************************************************************/

    /* return the EOV detected flag if the EOV subcommand was selected
       ------------------------------------------------------------------*/
    if (return_EOV)
    {
        if (q_intg(&vblock,"EOV_DETECTED",1,&EOV_detected,P_UPDATE) != SUCCESS)
        {
            c_errmsg("Error passing EOV_DETECTED to TAE","tpin-tae",&nonfatal);
            c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
        }
        if (q_out(&vblock) != SUCCESS)
        {
            c_errmsg("Error passing variables to TAE","tpin-tae",&nonfatal);
            c_lasmsg(TRUE,"ndfin-fatal","Fatal error encountered");
        }     
    }
    /* Output the history information
       ---------------------------------*/
    c_hstout(&vblock,hosout[0],hosout,&zero,"TAPEIN");

    c_complt("TAPEIN",taeout,&one);

    return EXIT_SUCCESS;

}
