#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "protos.h"

/******************************************************************************

FUNCTION: ascii_ddr

PURPOSE: Creates an ASCII representation of the DDR file and copies it to tape.

ALGORITHM DESCRIPTION:
  If writing the ddr after an image, output an EOF mark.
  Read the temporary DDR file.
  Allocate memory for ascii ddr.
  Convert ddr to ascii representation.
  Write ascii representation to tape.
  If writing the ddr before an image, output an EOF mark.
  Delete the temporary DDR.
  

******************************************************************************/


#define DDRSTRSIZE 1680         /* 21 lines * 80 characters/line              */
#define BDRSTRSIZE 640          /* ~8 lines * 80 characters/line              */

void ascii_ddr
(
    long *unit,		/* tape unit number                  		      */
    char *ddrnam,	        /* input host ddr name		     		      */
    long aft_flag,          /* write ASCII DDR after image flag                   */
    char tname[CMLEN]      /* input image tae name                               */
)


{
   struct DDR ddr;		/* data in records 1 and 2 of DDR 	      */
   struct BDDR bddr;		/* data in the band records 		      */
   struct TAPEDESC *tp;         /* pointer to tape drive descriptor           */


   char *cptr;                  /* temporary pointer to DDR strings           */
   char *ddrstr;                /* ASCII DDR buffer                           */
   char *temp;			/* temporary pointer to image name	      */
   
   
   long fatal = -1;             /* fatal error flag                           */
   long blksize;                /* ascii ddr blocksize                        */
   long	i;			/* loop index				      */
   long length;  		/* length of string                           */
   long mode  = HANDLE;         /* tape I/O outputs error messages   	      */
   long nonfatal = 1;           /* nonfatal error flag                        */
   long one = 1;
   long	status;			/* function return status code		      */
   long	totbnd;			/* total number of bands to be processed      */
   long totlen;  		/* length of ddr buffer                       */
   long zero = 0;
   

   tp = (struct TAPEDESC *) *unit;
   blksize = tp->maxbyte;
   
   /* Output an EOF mark if ddr is placed after an image.
      --------------------------------------------------*/
   if (aft_flag == 1)
     if (c_tpweof(unit,&one,&mode) != TAPE_SUCC)
       c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   
   /* get records 1 and 2 of DDR file
      ----------------------------------*/
   status = c_getddr(ddrnam,&ddr);
   if (status != E_SUCC)
     c_errmsg("Fatal error encountered","dspddr-fatal",&fatal);
   
   totbnd = ddr.nbands;

   /* allocate memory for ASCII DDR buffer
      ------------------------------------*/
   if((ddrstr = (char *) malloc(DDRSTRSIZE + totbnd * BDRSTRSIZE + 1)) == NULL)
   {
      c_errmsg("Error allocating dynamic buffer space","tapeout-malloc",
         &nonfatal);
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   }

   /* Discard the path from the TAE input image name to avoid line
      length problems. Following this, 'temp' will point to the name.
      ---------------------------------------------------------------*/
   temp = strrchr(tname, ']');
   if (!temp)
       temp = tname;
   else
       temp++;

   /* generate string for records 1 and 2
      ----------------------------------*/
   ddr2str(&ddr,ddrstr,&length, temp);

   cptr = ddrstr + length;
   
   /* get and create band dependent strings 
      -----------------------------------------*/
   for (i = 1; i <= totbnd; i++)
   {
      status = c_getbdr(ddrnam,&bddr,&i);
      if (status != E_SUCC)
	c_errmsg("Fatal error encountered","dspddr-fatal",&fatal);
      bdr2str(&bddr,cptr,&length,&i);
      cptr += length;
   }

   totlen = cptr - ddrstr + 1;
   
   /* write out the ascii ddr to tape 
      ------------------------------- */

   if(totlen < blksize )
     write_tape(unit,(unsigned char *) ddrstr,&totlen,&zero);
   else
   {
      cptr = ddrstr;
      while ( totlen > blksize )
      {
	 write_tape(unit,(unsigned char *) cptr,&blksize,&zero);
	 cptr += blksize;
	 totlen -= blksize;
      }
      if( totlen > 0 )
	write_tape(unit,(unsigned char *) cptr,&totlen,&zero);
   }
	 
   
   /* Output an EOF mark if ddr is placed before an image.
      --------------------------------------------------*/
   if (aft_flag == 0)
     if (c_tpweof(unit,&one,&mode) != TAPE_SUCC)
       c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   

   /* delete the temporary DDR 
      ------------------------*/

   c_delfil(ddrnam);

   free(ddrstr);
   
}
