/******************************************************************************
FUNCTION:	CP_BIL

PURPOSE:	Copies a LAS image to tape in BIL format.

ALGORITHM 
  Get the double versions of the line and sample increment values
  Compute output image size
  Error check image size for HP
  If output system is HP and image is BYTE and number of samples is odd
	output number of samples is incremented by one
  Calculate the number of bytes in the output buffer
  Allocate output buffer space
  Copy the ddr information to tape
      Initialize the temporary DDR information
      Create a temporary DDR to be updated before it is written to tape
      Update the temporary DDR before it is written to tape
      Write the temporary DDR to tape
  Initialize the processing message
  Initialize current line, current block and input line
  Start the processing message
  Process the lines from each band
      Skip line increment number of lines
      Process the lines from each band
	  Position the buffer pointer to the beginning of the desired band
	  Subsample the buffer
	  Position the output buffer pointer to beginning of the desired block
	  If the HP flag is still set (the ouput number of samples was
	  incremented to make the output buffer contain an even number
          of samples.)
		Set the last sample to zero and only copy NS - 1 elements.
	  Do any byte swapping or REAL data formatting of the input data to
	  the output system format.
	  Only write the lines to tape when the output block size has been
	  reached.
  Stop the processing message
  Close the image file
  Free allocated buffer memory

ALGORITHM REFERENCES	none
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>		/* for call to malloc()		*/
#include <unistd.h>		/* for call to getpid()		*/
#include <sys/types.h>	/* for pid_t			*/
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "tapeio.h"
#include "sysdef.h"
#include "protos.h"


void cp_bil
(
    long *unit,			/* tape unit number		             */
    long *insys,			/* Input system			             */
    long *outsys,		/* Output system		             */
    char hosin[][CMLEN],		/* input host file names	     	     */
    long *nmrimg,		/* number of input images	             */
    long window[][4],		/* input window (sl,ss,nl,ns)	             */
    long bands[][MAXBND+1],	/* Input bands			             */
    long nbands[],		/* Number of input bands	             */
    long *totbnd,		/* total number of bands to be copied        */
    long *odtype,		/* output image data type            	     */
    long *blksiz,		/* Number of image lines in an output block  */
    long *linc,			/* line increment for subsample      	     */
    long *sinc,			/* sample increment for subsample    	     */
    long *hp_flag,		/* flag for HP computer system       	     */
    long *ad_flag,		/* flag for ASCII DDR                        */
    long *aft_flag,	    	/* ASCII DDR placement flag 1=after 0=before */
    long *msgtim,		/* elapsed time between messages     	     */
    char taein[][CMLEN] 		/* input tae file names 	     	     */
)
{
extern struct GDESC * gdesc;	/* group descriptor used for image I/O       */
extern long  curent;		/* output line being processed       	     */
extern long  offset[];		/* offsets into "thebuf" for input bands     */
extern long  datasize[];	/* data size of the output data		     */
extern short ddr_flag;		/* DDR header flag			     */
extern unsigned char *thebuf;	/* image I/O buffer                          */
extern struct PARBLK vblock;	/* TAE parameter block               	     */

struct DDR ddr;		/* DDR structure for temp DDR			     */
struct BDDR bddr;	/* BDDR structure for temp DDR			     */

long band;		/* loop index for the number of bands		     */
long inlin;		/* current input image line          		     */
long curblk;		/* current output block 			     */
long next_line;		/* next input line to be copied      		     */
long out_byte;		/* # of bytes per output image line  		     */
long out_nl;		/* output number of lines            		     */
long out_ns;		/* output number of samples          		     */
long nelem;		/* number of output element 			     */
long status;		/* function return status code       		     */
long bckrec;		/* back space the specified number of records before */
			/* dismounting the tape.  This is used only for BIL  */
			/* processing to prevent breaking up the lines of a  */
			/* band between tape volumes.			     */
long zero = 0;		/* value of zero                     		     */
long fatal = -1;	/* Fatal error code				     */
long nonfatal = 1;	/* Non-Fatal error code				     */
long sysflg = SETS;	/* Set system string from the system value	     */
long msgflg = ALLCAT;	/* Processing message format flag		     */
long comflg = NOCOMB;	/* Output images are not combined		     */
long cvtflg = SYS_SET;	/* Conversion under/over flow flag - let system set  */

double dlinc;		/* Double version of line increment value	     */
double dsinc;		/* Double version of sample increment value	     */

char ddrnam[CMLEN];	/* Temporary file for update of DDR		     */
unsigned char *out_buf;	/* output data buffer				     */
unsigned char *out_ptr;	/* pointer into the output image buffer blocks       */
unsigned char *in_ptr;	/* pointer into the input image buffer band locations*/

pid_t curpid;		/* current process ID				     */

/* Get the double versions of the line and sample increment values
------------------------------------------------------------------*/
dlinc = (double) *linc;
dsinc = (double) *sinc;

/* Compute output image size
----------------------------*/
out_nl = ((window[0][NL] - 1) / *linc) + 1;
out_ns = ((window[0][NS] - 1) / *sinc) + 1;

/* Error check image size for HP
--------------------------------*/
if ((*hp_flag) && (out_ns > 14999))
    {
    c_errmsg("Maximum image size for HP computer system is 14999 samples",
	     "tapeout-hpmax",&nonfatal);
    c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
    }

/* If output system is HP and image is BYTE and number of samples
   is odd then output number of samples is incremented by one
-----------------------------------------------------------------*/
if ((*hp_flag) && (*odtype == EBYTE) && (out_ns % 2 != 0))
    out_ns++;
else
    *hp_flag = FALSE;

/* Calculate the number of bytes in the output buffer
-----------------------------------------------------*/
out_byte = *blksiz * out_ns * datasize[*odtype];

/* Copy the ddr information to tape
-----------------------------------*/
if (ddr_flag)
    {
    /* Initialize the temporary DDR information
    -------------------------------------------*/
    c_intddr(&ddr);
    c_intbdr(&bddr);
    ddr.nl = out_nl;
    ddr.ns = out_ns;
    ddr.nbands = *totbnd;
    ddr.dtype  = *odtype;
    c_sysset(ddr.system,outsys,&sysflg);

    /* check to see if input and ouput systems are the same
    ------------------------------------------------------*/
    if (*insys != *outsys)
        {
        c_errmsg("Output system and input system must be the same",
        "tapeout-ddr",&fatal);
        c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
        }

    /* Create a temporary DDR to be updated before it is written to tape
    --------------------------------------------------------------------*/
    curpid = getpid();
    sprintf(ddrnam, "temp%d.ddr", (int) curpid);
    c_putddr(ddrnam,&ddr);
    for (band = 1; band <= *totbnd; band++)
	{
	if (*nmrimg == 1)
	    {
	    if (c_getbdr(hosin[0], &bddr, &bands[0][band-1]) != E_SUCC)
		c_lasmsg(TRUE, "tapeout-fatal", "Fatal error encountered");
	    }
	bddr.bandno = band;
	c_putbdr(ddrnam,&bddr);
	}

    /* Update the temporary DDR before it is written to tape
    --------------------------------------------------------*/
    status = c_upddr(&vblock,hosin,ddrnam,nmrimg,window,bands,nbands,&dlinc,
		     &dsinc,totbnd,&comflg);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);

    /* Write the temporary DDR to tape
    ----------------------------------*/
    if(*ad_flag == 0)
      copy_ddr(unit,insys,outsys,ddrnam);
    else if (*aft_flag == 0)
      ascii_ddr(unit,ddrnam, *aft_flag, taein[0]);
    }

/* Allocate output buffer space
-------------------------------*/
out_buf = (unsigned char *) malloc(out_byte);
if (out_buf == NULL)
    {
    c_errmsg("Error allocating dynamic buffer space","tapeout-malloc",
	     &nonfatal);
    c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
    }

/* Initialize the processing message
------------------------------------*/
c_inittm("TAPEOUT","Processed","line","band(s)","",&out_nl,totbnd,msgtim,
	 &msgflg);

/* Initialize current line, current block and input line
--------------------------------------------------------*/
curent = 1;
curblk = 0;
inlin = 0;

/* Start the processing message
-------------------------------*/
c_startm();

/* Process the lines from each band
-----------------------------------*/
for (curent = 0, next_line = 0; curent < out_nl; curent++, next_line += *linc)
    {
    /* Skip line increment number of lines
    --------------------------------------*/
    for (; inlin <= next_line; inlin++)
	{
	status = c_estep(&gdesc);
	if (status != E_SUCC)
	    {
	    bksp_file(unit);
	    c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
	    }
	}

    /* Process the lines from each band
    -----------------------------------*/
    for (band = 0; band < *totbnd; band++)
	{
	/* Position the buffer pointer to the beginning of the desired band
	-------------------------------------------------------------------*/
	in_ptr = thebuf + (offset[band] * datasize[*odtype]);

	/* Subsample the buffer
	-----------------------*/
	if (*sinc > 1)
	    c_pxsamp(&window[0][NS],odtype,sinc,in_ptr);

	/* Position the output buffer pointer to beginning of the desired block
	----------------------------------------------------------------------*/
	out_ptr = out_buf + (curblk * out_ns * datasize[*odtype]);

	/* If the HP flag is still set (the ouput number of samples was
	   incremented to make the output buffer contain an even number
	   of samples.)  Set the last sample to zero and only copy NS - 1
	   elements.
	-----------------------------------------------------------------*/
	if (*hp_flag)
	    {
	    nelem = out_ns - 1;
   	    *(out_ptr + nelem) = 0;
	    }
	else
	    nelem = out_ns;

	/* Do any byte swapping or REAL data formatting
	   of the input data to the output system format
	------------------------------------------------*/
	status = c_pxsys(insys,outsys,in_ptr,odtype,&nelem,&zero,&zero,&cvtflg);

	/* Copy the converted input data to the output buffer
	-----------------------------------------------------*/
	c_pxcopy(in_ptr,out_ptr,odtype,&nelem);

	/* Only write the lines to tape when the
	   output block size has been reached
	----------------------------------------*/
	if (*blksiz == 1)
	    {
	    bckrec = band + 1;
	    status = write_tape(unit,out_buf,&out_byte,&bckrec);
	    if (status == TAPE_EOT)
		band = -1;
	    curblk = 0;
	    }
	else if (curblk == *blksiz - 1)
	    {
	    status = write_tape(unit,out_buf,&out_byte,&zero);
	    curblk = 0;
	    }
	else
	   curblk++;
	}
    }

/* Insure that all data is written
----------------------------------*/
if (curblk != 0)
    status = write_tape(unit,out_buf,&out_byte,&zero);

/* Stop the processing message
------------------------------*/
c_stoptm();

if(ddr_flag && *aft_flag == 1)
  ascii_ddr(unit,ddrnam,*aft_flag, taein[0]);

/* Close the image file
-----------------------*/
status = c_egclse(&gdesc);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered.","tapeout-warn",&nonfatal);

/* Free allocated buffer memory
-------------------------------*/
free(out_buf);

return; 
} 
