/******************************************************************************
FUNCTION:	CP_BSQ

PURPOSE:	Copies a LAS image to tape in BSQ format.

ALGORITHM 

  Set the double value version of line and sample increments
  Calculate the output number of lines and samples
  Error check image size for HP
  If output system is HP and image is BYTE number of samples is odd
	increment output number of samples by one
  Allocate input buffer space
  Calculate the number of bytes for the output buffer
  Copy the DDR file
      Initialize the temporary DDR information
      Create a temporary DDR to be updated before it is written to tape
      Update the temporary DDR before it is written to tape
      Write the DDR to a tape file
  Initialize the processing message
  Process each image
      Process each band
	  Start the processing message
	  Process each line
	      Subsample the buffer
	      Set the output pointer to the desired output block location
	      If the HP flag is still set (the ouput number of samples was
	      incremented to make the output buffer contain an even number
	      of samples.)
		  Set the last sample to zero and only copy NS - 1 elements.
	      Do any byte swapping or REAL data formatting of the input data to
	      the output system format.
	      Only write the data to tape when the block size has been reached
  Stop the processing message
  Free allocated buffer memory

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>     /* for call to malloc()         */
#include <unistd.h>		/* for call to getpid() 	*/
#include <sys/types.h>	/* for definition of pid_t	*/
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "pixman.h"
#include "sysdef.h"
#include "protos.h"


void cp_bsq
(
    long *unit,			/* tape unit number                  	     */
    long *insys,			/* Input system				     */
    long *outsys,			/* Output system			     */
    char hosin[][CMLEN],		/* Input host file names	     	     */
    long *nmrimg,			/* number of input images            	     */
    long window[][4],		/* input window (sl,ss,nl,ns)		     */
    long bands[][MAXBND+1],		/* list of input bands to be copied  	     */
    long nbands[],			/* # bands in each input image               */
    long *totbnd,			/* total number of bands to be copied        */
    long *odtype,			/* output image data type                    */
    long *blksiz,			/* number of image lines in output block     */
    long *linc,			/* line increment for subsample      	     */
    long *sinc,			/* sample increment for subsample            */
    long *hp_flag,			/* ouput computer system flag for HP         */
    long *ad_flag,	        /* flag for ASCII DDR                        */
    long *aft_flag,	        /* ASCII DDR placement flag 1=after 0=before */
    long *msgtim,			/* elapsed time between messages     	     */
    char taein[][CMLEN],		/* Input tae file names 	     	     */
    long qflag[MAXIMG][2]	        /* windowing/banding option flag             */
)
{
extern struct FDESC * fdesc[];	/* image file descriptors used for image I/O */
extern long  curent;		/* output line being processed       	     */
extern long  curcat;		/* output band being processed       	     */
extern long  datasize[];	/* data size of the output data		     */
extern short ddr_flag;		/* DDR header flag			     */
extern struct PARBLK vblock;	/* TAE parameter block                       */

struct DDR ddr;		/* DDR structure for temp DDR		             */
struct BDDR bddr;	/* BDDR structure for temp DDR		             */

long band;		/* loop index for the number of bands	             */
long image;		/* loop index for current input image	             */
long inbnd;		/* current input image band number   	             */
long inlin;		/* current input image line          	             */
long curblk;		/* current line count for blocking	             */
long out_byte;		/* # of bytes per output image line  	             */
long out_nl;		/* output number of lines            	             */
long out_ns;		/* output number of samples          	             */
long nelem;		/* number of output element 		             */
long status;		/* function return status code       	             */
long one = 1;		/* value of one                     	             */
long zero = 0;		/* value of zero                     	             */
long fatal = -1;	/* Fatal error code			             */
long nonfatal = 1;	/* Non-Fatal error code  		             */
long sysflg = SETS;	/* Set system string from system value	             */
long msgflg = ONECAT;	/* Processing message format flag	             */
long comflg = NOCOMB;	/* Output images are not combined	             */
long cvtflg = SYS_SET;	/* Conversion under/over flow flag - let system set  */

double dlinc;		/* Double version of line increment value	     */
double dsinc;		/* Double version of sample increment value	     */

char ddrnam[CMLEN];	/* Temporary DDR file for updates	             */
unsigned char *in_buf;	/* buffer to read input image line   	             */
unsigned char *out_buf;	/* buffer to read input image line   	             */
unsigned char *out_ptr;	/* pointer into output buffer block	             */

pid_t curpid;		/* current process ID				      */

/* Set the double value version of line and sample increments
-------------------------------------------------------------*/
dlinc = (double) *linc;
dsinc = (double) *sinc;

/* Calculate the output number of lines and samples
---------------------------------------------------*/
out_nl = ((window[0][NL] - 1) / *linc) + 1;
out_ns = ((window[0][NS] - 1) / *sinc) + 1;

/* Error check image size for HP
--------------------------------*/
if ((*hp_flag) && (out_ns > 14999))
    {
    c_errmsg("Maximum image size for HP computer system is 14999 samples",
	     "tapeout-hpmax",&nonfatal);
    c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
    }

/* If output system is HP and image is BYTE number of samples
   is odd then increment output number of samples by one
-------------------------------------------------------------*/
if ((*hp_flag) && (*odtype == EBYTE) && (out_ns % 2 != 0))
   out_ns++;
else
   *hp_flag = FALSE;

/* Allocate input buffer space
------------------------------*/
in_buf = (unsigned char *) malloc((window[0][NS] + 1) * datasize[*odtype]);
if (in_buf == NULL)
    {
    c_errmsg("Error allocating dynamic buffer space","tapeout-malloc",
	     &nonfatal);
    c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
    }

/* Calculate the number of bytes for the output buffer
------------------------------------------------------*/
out_byte = *blksiz * out_ns * datasize[*odtype];


/* Copy the DDR file
--------------------*/
if (ddr_flag)
    {
    /* Initialize the temporary DDR information
    -------------------------------------------*/
    c_intddr(&ddr);
    c_intbdr(&bddr);
    ddr.nl = out_nl;
    ddr.ns = out_ns;
    ddr.nbands = *totbnd;
    ddr.dtype  = *odtype;
    c_sysset(ddr.system,outsys,&sysflg);

    /* check to see if input and ouput systems are the same
    ------------------------------------------------------*/
    if (*insys != *outsys)
        {
        c_errmsg("Output system and input system must be the same",
        "tapeout-ddr",&fatal);
        c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
        }

    /* Create a temporary DDR to be updated before it is written to tape
    --------------------------------------------------------------------*/
    curpid = getpid();
    sprintf(ddrnam, "temp%d.ddr", (int) curpid);
    c_putddr(ddrnam,&ddr);
    for (band = 1; band <= *totbnd; band++)
	{
	if ((*nmrimg == 1) && (qflag[0][0] == 0))
	    {

	    /* If we are only dealing with one input image, use the input
	       image's BDDR data in the temporary DDR; this ensures that
	       information such as min/max values are preserved.
	       ----------------------------------------------------------*/
	    if (c_getbdr(hosin[0], &bddr, &bands[0][band-1]) != E_SUCC)
	        c_lasmsg(TRUE, "tapeout-fatal", "Fatal error encountered");
	    }
	bddr.bandno = band;
	c_putbdr(ddrnam,&bddr);
	}

    /* Update the temporary DDR before it is written to tape
    --------------------------------------------------------*/
    status = c_upddr(&vblock,hosin,ddrnam,nmrimg,window,bands,nbands,&dlinc,
		     &dsinc,totbnd,&comflg);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);

    /* Write the DDR to a tape file
    -------------------------------*/
    if(*ad_flag == 0)
      copy_ddr(unit,insys,outsys,ddrnam);
    else if (*aft_flag == 0)
      ascii_ddr(unit,ddrnam,*aft_flag, taein[0]);
    }

/* Allocate output buffer space
-------------------------------*/
out_buf = (unsigned char *) malloc(out_byte);
if (out_buf == NULL)
    {
    c_errmsg("Error allocating dynamic buffer space","tapeout-malloc",
	     &nonfatal);
    c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
    }

/* Initialize the processing message
------------------------------------*/
c_inittm("TAPEOUT","Processed","line","band","",&out_nl,totbnd,msgtim,&msgflg);

/* Initialize the current block
-------------------------------*/
curblk = 0;

/* Process each image
---------------------*/
for (image = 0, curcat = 0; image < *nmrimg; image++)
    {
    /* Process each band
    --------------------*/
    for (inbnd = 0; inbnd < nbands[image]; inbnd++)
	{
	/* Start the processing message
	-------------------------------*/
	++curcat;
	curent = 1;
	c_startm();

	/* Process each line
	--------------------*/
	for (inlin = 1, curent = 0; curent < out_nl; inlin += *linc, curent++)
	    {
	    status = c_eread(&fdesc[image],&bands[image][inbnd],&inlin,in_buf,
			     &one);
	    if (status == E_FAIL)
		{
		bksp_file(unit);
		c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
		}

	    /* Subsample the buffer
	    -----------------------*/
	    if (*sinc > 1)
		    c_pxsamp(&window[0][NS],odtype,sinc,in_buf);

	    /* Set the output pointer to the desired output block location
	    --------------------------------------------------------------*/
	    out_ptr = out_buf + (curblk * nelem * datasize[*odtype]);

	    /* If the HP flag is still set (the ouput number of samples was
	       incremented to make the output buffer contain an even number
	       of samples.)  Set the last sample to zero and only copy NS - 1
	       elements.
	    -----------------------------------------------------------------*/
	    if (*hp_flag)
		{
		nelem = out_ns - 1;
		*(out_ptr + nelem) = 0;
		}
	    else
		nelem = out_ns;

	    /* Do any byte swapping or REAL data formatting
	       of the input data to the output system format
	    ------------------------------------------------*/
	    status = c_pxsys(insys,outsys,in_buf,odtype,&nelem,&zero,&zero,
			     &cvtflg);

	    /* Copy the converted input data to the output buffer
	    -----------------------------------------------------*/
	    c_pxcopy(in_buf,out_ptr,odtype,&nelem);

	    /* Only write the data to tape when the block size has been reached
	    ------------------------------------------------------------------*/
	    if (curblk == *blksiz - 1)
		{
		write_tape(unit,out_buf,&out_byte,&zero);
		curblk = 0;
		}
	    else
		curblk++;
	    }

	/* Stop the processing message
	------------------------------*/
	c_stoptm();
	}

    /* Insure that all the data is written
    --------------------------------------*/
    if (curblk != 0)
	write_tape(unit,out_buf,&out_byte,&zero);


    /* Close the input image
    ------------------------*/
    status = c_eclose(&fdesc[image]);
    if (status != E_SUCC)
	c_errmsg("Nonfatal error encountered","tapeout-warn",&nonfatal);
    }

if(ddr_flag && *aft_flag == 1)
  ascii_ddr(unit,ddrnam,*aft_flag, taein[0]);


/* Free allocated buffer memory
-------------------------------*/
free(in_buf);
free(out_buf);

return;
}
