/*******************************************************************************
NAME:    INITLBL

PURPOSE: Create a project parameter file that will initialize other LAS
	 functions for creating magnetic tape labels.

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Development
  1.1    J.Fenno          03/92  EDC   Added code to use a unique file name when
				       creating the .ppf file that contains the
				       tape label info.  This file name is then
				       passed back to TAE.  PRF Z0810.
  1.2    J.Matzner	  06/92  EDC   Modified purge date to be from 1 to 60
				       months from the current date.  The actual
				       deletion date is the first day of the
				       month following the user stated number of
				       months. 
				       Added code to allow for a consistent four				       character unique code usable for all
				       systems.
  1.3	D.Etrheim         07/94  EDC   Modified to use new support routine
				       c_tplblpr

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                

*******************************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>

#include "las.h"
#include "tapeio.h"
#include "util.h"


int main
(void)
{
struct PARBLK vblock;         /* TAE parameter block                          */
struct VARIABLE *var;         /* Ptr to a variable within TAE variable block  */

char   code[1][CMLEN];        /* Code for the group of the requestor          */
char   description[1][CMLEN]; /* Description of the tape set                  */
char   err_str[CMLEN];        /* String for printing error messages           */
char   info_str[CMLEN];       /* String for printing informational messages   */
char   hostname[1][CMLEN];    /* Host name of the computer                    */
char   name[1][CMLEN];        /* Initials of the requestor                    */
char   parm_file[CMLEN];      /* Name of the parameter file being created     */
char   printer[1][CMLEN];     /* String containing the printer to use         */
char   project[1][CMLEN];     /* Project abbreviation                         */
char   *ptr;                  /* Character ptr for string manipulation        */
char   *str_ptr[1];           /* Pointer to a string for call to q_string     */
char   tmp_str[CMLEN];        /* Temporary string                             */
char   trunc_str[5];          /* Temporary string for truncation of hostname  */
char   words[13][CMLEN];      /* Individual words of the tape description     */

double dbl_val[1];            /* Array of double values sent to c_putparm     */

long   fatal    = -1;         /* Fatal error flag                             */
long   i;                     /* Loop control                                 */
long   j;                     /* Loop control                                 */
long   lng_val[1];            /* Array of long values sent to c_putparm       */
long   nonfatal =  1;         /* Nonfatal error flag                          */
long   std_out;               /* Logical unit number for standard output      */
long   retain;		      /* months to retain after current date, 1-60    */
	                      /* months OK, 0 means permanent, > 60 set to 60 */

FILE   *file_ptr;             /* Temp file which will contain each label      */

/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);

/* Create a unique file name and remove the '.' at the end
----------------------------------------------------------*/
strcpy(parm_file, "tape_labels");
c_creat_name(parm_file, "ppf");

/* Make sure the file parm_file does not exist
----------------------------------------------*/
if ((file_ptr = fopen(parm_file, "r")) != NULL)
   {
   fclose(file_ptr);
   sprintf(err_str, "File %s already exists", tmp_str);
   c_errmsg(err_str, "initlbl-exist", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }

/* Remove the .ppf extension and use just the root name
-------------------------------------------------------*/
ptr = strchr(parm_file, '.');
*ptr = '\0';

/* Retrieve the parameters
--------------------------*/
var = p_find(&vblock, "NAME");
strcpy(name[0], SVAL(*var, 0));
var = p_find(&vblock, "CODE");
strcpy(code[0], SVAL(*var, 0));
var = p_find(&vblock, "PROJECT");
strcpy(project[0], SVAL(*var, 0));
var = p_find(&vblock, "DESCRPT");
strcpy(description[0], SVAL(*var, 0));

/* Convert all spaces in project to underscores.  c_getparm
   and c_putparm do not allow white space in strings.
-----------------------------------------------------------*/
for (i = 0; i < strlen(project[0]); i++)
   if (isspace(project[0][i]))
      project[0][i] = '_';

/* Get the number of retain months
----------------------------------*/
var = p_find(&vblock, "NMONTHS");
retain = IVAL(*var, 0);
lng_val[0] = retain;

/* Retrieve the host name
-------------------------*/
if (gethostname(tmp_str,CMLEN) != E_SUCC)
   {
   c_errmsg("Error retrieving host name", "initlbl-host", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }

/* Get a 4 character hostname
-----------------------------*/
i = (long) strlen(tmp_str);        
c_up2low(tmp_str, &i);

if (strncmp(tmp_str,"eros",4) == 0)
   {
   strncpy(hostname[0],"vax",3);
   hostname[0][3] = tmp_str[4];
   hostname[0][4] = '\0';
   }
else
   {
   /* If the host name is edcxxxx, strip off the edc portion */
   if (strncmp(tmp_str,"edc",3) == 0)
      strcpy(tmp_str,&tmp_str[3]);
   strncpy(trunc_str, tmp_str,4);
   trunc_str[4] = '\0';
   if (strlen(tmp_str) > 4)
	{
   	sprintf(info_str, "Hostname has been truncated to four");
   	c_errmsg(info_str, "informational", &nonfatal);
   	sprintf(info_str, "characters, from %s to %s", tmp_str,trunc_str);
   	c_errmsg(info_str, "informational", &nonfatal);
	}
   strcpy(hostname[0], trunc_str);
   }

/* For consistency convert all fields to lowercase
--------------------------------------------------*/
i = (long) strlen(name[0]);        c_up2low(name[0], &i);
i = (long) strlen(code[0]);        c_up2low(code[0], &i);
i = (long) strlen(project[0]);     c_up2low(project[0], &i);
i = (long) strlen(description[0]); c_up2low(description[0], &i);
i = (long) strlen(hostname[0]);       c_up2low(hostname[0], &i);

/* Retrieve the printer from the configuration file
---------------------------------------------------*/
if (c_tplblpr(printer[0]) != E_SUCC)
   c_errmsg("Fatal error encountered","initlbl-fatal",&fatal);

/* Setup a system command to remove parm_file on fatal errors
-------------------------------------------------------------*/
#ifdef vms
sprintf(err_str, "delete %s.ppf;*", parm_file);
#else
sprintf(err_str, "/bin/rm -f %s.ppf", parm_file);
#endif

/* Place all of the data gathered into the file parm_file
---------------------------------------------------------*/
if (c_putparm(parm_file,"NAME",name,"S",dbl_val,1,lng_val) != E_SUCC)
   {
   system(err_str);
   c_errmsg("Error writing NAME to file", "initlbl-ppf", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }
if (c_putparm(parm_file,"CODE",code,"S",dbl_val,1,lng_val) != E_SUCC)
   {
   system(err_str);
   c_errmsg("Error writing CODE to file", "initlbl-ppf", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }
if (c_putparm(parm_file,"PROJECT",project,"S",dbl_val,1,lng_val) != E_SUCC)
   {
   system(err_str);
   c_errmsg("Error writing PROJECT to file", "initlbl-ppf", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }
if (c_putparm(parm_file,"NMONTHS",project,"I",dbl_val,1,lng_val) != E_SUCC)
   {
   system(err_str);
   c_errmsg("Error writing NMONTHS to file", "initlbl-ppf", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }

if (c_putparm(parm_file,"HOSTNAME",hostname,"S",dbl_val,1,lng_val) != E_SUCC)
   {
   system(err_str);
   c_errmsg("Error writing HOSTNAME to file", "initlbl-ppf", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }
if (c_putparm(parm_file,"PRINTER",printer,"S",dbl_val,1,lng_val) != E_SUCC)
   {
   system(err_str);
   c_errmsg("Error writing PRINTER to file", "initlbl-ppf", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }

/* When writing the description care must be taken to write each word
   separately.  GETPARM and PUTPARM will not work with a string that contains
   white space.  As an example "this is a test" cannot be written as 1 string,
   it must be written as 4 strings.
------------------------------------------------------------------------------*/
ptr = description[0];
while (isspace(*ptr))
   ptr++;
if (*ptr == '\0')
   {
   strcpy(words[0], "_none_");
   i = 1;
   }
else
   {
   i = 0;
   while (*ptr != '\0')
      {
      j = 0;
      while ( !(isspace(*ptr)) && (*ptr != '\0'))
	 words[i][j++] = *(ptr++);
      words[i++][j] = '\0';
      while (isspace(*ptr))
	 ptr++;
      }
   }
if (c_putparm(parm_file,"DESCRPT",words,"S",dbl_val,i,lng_val) != E_SUCC)
   {
   system(err_str);
   c_errmsg("Error writing DESCRPT to file", "initlbl-ppf", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }

/* Output an informational message
----------------------------------*/
sprintf(tmp_str, "%s.ppf file created", parm_file);
c_errmsg("Automatic tape label generation enabled", " ", &nonfatal);
c_errmsg(tmp_str, " ", &nonfatal);

/* Send the name of the parm_file back to TAE
---------------------------------------------*/
str_ptr[0] = parm_file;
if (q_string(&vblock, "PARFILE", 1, str_ptr, P_UPDATE) != SUCCESS)
   {
   system(err_str);
   c_errmsg("Error sending string to TAE", "initlbl-string", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }
if (q_out(&vblock) != SUCCESS)
   {
   system(err_str);
   c_errmsg("Error sending string to TAE", "initlbl-string", &nonfatal);
   c_errmsg("Fatal error encountered", "initlbl-fatal", &fatal);
   }

return EXIT_SUCCESS;
   
}
