/******************************************************************************
FUNCTION:	OPEN_IMAGE

PURPOSE:	Opens the image.  If the output band format is BIL, the LAS
		image is opened for sequential processing (process all bands
		at once).  If the output band format is BSQ, the LAS image
		is opened for random processing (process a single band at a
		time).

ALGORITHM 
  Open image for Sequential processing of bands
      Open each image file
      Allocate I/O buffer the first time
      Group the file descriptors
  Else Open image for Random access
      Open each image file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "protos.h"

void open_image
(
    char hosin[][CMLEN],	/* input host name				      */
    long *nmrimg,		/* number of input images            		      */
    long window[][4],	/* input window specification        		      */
    long bands[][MAXBND+1],	/* list of input bands to be copied  		      */
    long *nbands,		/* # bands in each input image       		      */
    long *odtype,		/* output image data type            		      */
    char *bform		/* output image band format          		      */
)
{
extern struct GDESC * gdesc;	/* group descriptor used for image I/O        */
extern struct FDESC * fdesc[];	/* image file descriptors used for image I/O  */
extern long offset[];	/* offsets into "thebuf" for input bands  */
extern unsigned char *thebuf;	/* image I/O buffer			      */

long band;		/* loop index for counting bands     		      */
long image;		/* loop index for counting images    		      */
long bufsiz;		/* Size of buffer area				      */
long status;		/* function return status code       		      */
long innb;		/* number of bands in input image    		      */
long zero = 0;		/* value of zero                     		      */
long one  = 1;		/* value of one                     		      */
long fatal = -1;	/* Fatal error code				      */
long nonfatal = 1;	/* Non-Fatal error code				      */
long access = IREAD;	/* image I/O access type of READ     		      */

/*
  Open image for Sequential processing of bands
*/
if (strcmp(bform,"BIL") == 0)
    {
    /*
      Open each image file
    */
    for (band = 0, image = 0; image < *nmrimg; image++)
	{
	status = c_eopens(&fdesc[image],hosin[image],bands[image],
			&window[image][0],&window[image][1],&window[image][2],
			&window[image][3],&offset[band],&access,odtype,
			&zero,&zero);
	if (status != E_SUCC)
	    c_errmsg("Fatal error encountered", "tapeout-fatal",&fatal);

	band += nbands[image];
	}

    fdesc[*nmrimg] = 0;

    /*
      Allocate I/O buffer the first time
    */
    if ((thebuf = (unsigned char *) c_ealloc(fdesc,&bufsiz)) == NULL)
	{
	c_errmsg("Error allocating dynamic memory","tapeout-alloc",&nonfatal);
	c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
	}

    /*
      Group the file descriptors
    */
    status = c_egroup(fdesc,&gdesc,thebuf,&bufsiz);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","tapeout-fatal", &fatal);
    }
else if (strcmp(bform,"BSQ") == 0) /* Open for random access		*/
    {
    /*
      Open each image file
    */
    for (image = 0; image < *nmrimg; image++)
	{
	innb = 0;
	status = c_eopenr(&fdesc[image],hosin[image],&innb,&window[image][0],
               		&window[image][1],&window[image][2],&window[image][3],
               		&access,odtype,&zero,&one);
	if (status != E_SUCC)
	    c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
	}
    }

else
    printf("\nINVALID BAND FORMAT = %s",bform);

return;
}
