/*******************************************************************************
FUNCTION: SKIP_FILE

PURPOSE:  Skips physical files on tape.  If the EOT is encountered, the next
	  tape volume is mounted.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         1/86       K. Gacke     initial development
    2		5/90	   K. Gacke	modified tape i/o for sun workstation.
					If tape opened for write access, it is
					for read/write access.
    3.0         1/92       J. Fenno     Removed unnecessary fatal error
					messages.

ALGORITHM 
  Skip the specified number of files 
       check the return status

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "vll.h"
#include "protos.h"

#define MAX_BLKSIZ 32767	/* maximum block size on tape                */

void skip_file
(
    long *unit,		/* tape unit number                  		      */
    long skip		/* number of image files to skip     		      */
)

{
long nfile;		/* # of files to skip on tape        		      */
long status;		/* function return status code       		      */
long fatal = -1;	/* Fatal error code				      */
long nonfatal = 1;	/* Non-fatal error code				      */
long mode = HANDLE;	/* tape I/O does NOT output error msg		      */

/*
  Skip the specified number of files and then check the status
*/
nfile = skip;
status = c_tpfsf(unit,&nfile,&mode);
switch (status)
    {
    case TAPE_SUCC:			/* Success			*/
	break;

    case TAPE_EOV:			/* End of Volume		*/
	if (nfile < skip)
	    c_errmsg("Unexpected EOV encountered"," ",&nonfatal);
	break;

    case TAPE_EOT:			/* End of Tape			*/
	close_tape(unit,REWOFFL,1);
	mount_tape(unit,TWRITE,FALSE);
	nfile = skip - nfile;
	skip_file(unit,nfile);
	break;

    default:
	c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
	break;
    }

return;
}

/******************************************************************************
FUNCTION:	SKIP_IMAGES

PURPOSE:	Skips a specified number of image files.  Each image file may
		consist of one or two physical tape files depending on whether
		or not label files exist.  

ALGORITHM 
  If no images are to be skipped
      return
  Send out an informational message 
  Allocate buffer space for the read
  Process the number of image files to skip
      Read a block from the tape
      Determine if the file has a DDR file and skip over it also
      When a DDR is present need to skip two physical files to equal one
      image file
      Skip the number of specified physical files on tape
  Free the allocated buffer space

ALGORITHM REFERENCES	none
*******************************************************************************/
void skip_images
(
    long *unit,			/* tape unit number                  	      */
    long skip			/* number of image files to skip     	      */
)
{
extern long tpfil;		/* # of files on current tape        	      */
long bytes_read;		/* actual # of bytes read from tape  	      */
long index;			/* loop index                        	      */
long nfile;			/* # of files to skip on tape        	      */
long fatal = -1;		/* Fatal error code			      */
long nonfatal = 1;		/* Non-fatal error code			      */
long mode = HANDLE;		/* tape I/O does NOT output error msg	      */
long nbyte = MAX_BLKSIZ;	/* # of bytes to be read from tape   	      */

short not_eof;			/* flag to check EOF, EOV, or EOT             */
char  *key;			/* Key of the Label services record	      */
char  msg[ERRLEN];		/* message to be outputted                    */
char  header[HDRL];		/* Label services header information	      */
unsigned char	*buf;		/* buffer pointer to read into                */

/* If no images are to be skipped return
----------------------------------------*/
if (skip == 0)
   return;

/* Send out an informational message 
------------------------------------*/
sprintf(msg,"Skipping %d image files on tape",skip);
c_errmsg(msg," ",&nonfatal);

/* Allocate buffer space for the read
-------------------------------------*/
buf = (unsigned char *) malloc(nbyte);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic buffer space","tapeout-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   }

/* Process the number of image files to skip
--------------------------------------------*/
for (index = 0; index < skip; index++)
   {
   /* Read a block from the tape
   -----------------------------*/
   bytes_read = c_tpread(unit,buf,&nbyte,&mode);
   not_eof = check_tpread(bytes_read,unit,buf,nbyte,mode);

   /* Determine if the file has a DDR file and skip over it also
   -------------------------------------------------------------*/
   strncpy(header,(char *) buf,HDRL);
   header[HDRL - 1] = '\0';
   key = squeeze(header+LENL+TYPL,(long) strlen(header+LENL+TYPL));

   /* When a DDR is present need to skip two
   physical files to equal one image file
   -----------------------------------------*/
   if ((not_eof) && (strcmp(key,"DDRINT") == 0))
      nfile = 2;
   else
      nfile = 1;

   /* Skip the number of specified physical files on tape
   ------------------------------------------------------*/

   skip_file(unit,nfile);
   ++tpfil;
   }

/* Free the allocated buffer space
----------------------------------*/
free(buf);
return;
}
