/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "protos.h"

#define MAX_VOL	   15           /* max # of tapes                             */
#define NEXT_VOL   100          /* status to indicate image didnt fit on tape */
/*******************************************************************************
FUNCTION:	MOUNT_TAPE

PURPOSE:	Physically mounts a tape.  The mount message is determined by
		the value of the global variables "comment" and "curvol",
		where "curvol" is an index into array "comment".  After the
		tape is mounted the tape drive name and "curvol" are saved
		in the TAE parameter block.

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  ???    ???              ??/??  ???   Development
  1.0    J.Fenno          02/92  EDC   Modified to print tape labels when at the
				       EROS Data Center and the appropriate file
				       exists in the current working directory.
				       Changed so that tpopen is passed the
				       user-specified tape density instead of
				       always sending 0.
  1.1    J.Fenno          03/92  EDC   Added code so that unique file names are
				       used allowing tapeout to be run multiple
				       times from the same working directory.
				       PRF Z0810.

  1.2   B.Olson           01/98  EDC   Added code to append tapeid to the list
                                       of tapeids stored in MTAPEID.

ALGORITHM REFERENCES	none
*******************************************************************************/

void mount_tape
(
    long *unit,                 /* Tape unit number                           */
    long access,                /* Open tape for specified  access            */
    int flag                  /* Flag to disable display messages           */
)
{
extern struct PARBLK vblock;    /* TAE parameter block                        */
extern char   comment[][CMLEN]; /* Tape mount message to the operator         */
extern char   density[1][CMLEN];/* Tape density in BPI                        */
extern char   *drive[1];        /* Physical tape drive name                   */
extern char   label_file[];     /* File containing tape label info            */
extern int   curvol;           /* Current tape volume being process          */
struct VARIABLE *var;           /* Pointer to a TAE variable                  */

char   tmp_str[CMLEN];          /* Temporary string variable                  */

long   fatal    = -1;           /* Fatal error code                           */
long   i;                       /* Temporary variable                         */
int    j;                       /* Temporary variable                         */
long   mode     = HANDLE;       /* Tape I/O outputs error messages            */
long   nonfatal = 1;            /* Nonfatal error code                        */
long   rewopt   = REWIND;       /* Rewind option                              */
long   skip     = 0;            /* Skip zero files on open                    */
long   status;

char *tapeids[14];		/* List of all tape IDs in tape set           */

/* Variables for tape label generation
--------------------------------------*/
char   date    [1][CMLEN];      /* Date the tape is created                   */
char   ops     [1][CMLEN];      /* Operators initials field of tape label     */
char   sequence[1][CMLEN];      /* Sequence number of this tape               */
char   tapeid  [1][CMLEN];      /* ID of the tape that is mounted             */
char   time    [1][CMLEN];      /* Time the tape is created                   */
double dbl_val[1];              /* Temp array for calls to c_putparm          */
long   lng_val[1];              /* Temp array for calls to c_putparm          */


/* If disable display message flag is set, stop the processing message
----------------------------------------------------------------------*/
if (flag)
   c_stoptm();

/* Mount the current tape volume
--------------------------------*/
curvol++;
status = c_tpopen(unit, &skip, &mode, &rewopt, density[0], &access,
	 comment[curvol], tapeid[0]);
drive[0] = comment[curvol];
if (status != TAPE_SUCC)
   c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);

/* Send the tape drive and current volume back to TAE
-----------------------------------------------------*/
if (q_string(&vblock, "DRIVE", 1, drive, P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error passing string to TAE","tapeout-string",&nonfatal);
   c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   }
if (q_intg(&vblock, "CURVOL", 1, &curvol, P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error passing integer to TAE","tapeout-intg",&nonfatal);
   c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   }
if (q_out(&vblock) != SUCCESS)
   {
   c_errmsg("Error sending variable block to TAE","tapeout-out",&nonfatal);
   c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   }

/* Notify the user of the tape ID mounted by the operator
---------------------------------------------------------*/
i = (long) strlen(tapeid[0]);
if (i < 1)
   {
   sprintf(tmp_str, "Tape %1d: operator did not enter tape ID", curvol + 1);
   c_errmsg(tmp_str, " ", &nonfatal);
   strcpy(tapeid[0], "??????");
   }
else
   {
   c_up2low(tapeid[0], &i);
   sprintf(tmp_str, "Tape %1d: operator mounted %s", curvol + 1, tapeid[0]);
   c_errmsg(tmp_str, " ", &nonfatal);

   /* Ensure tape ID is no longer than six characters */
   if (i > 6)
       {
       tapeid[0][6] = '\0';
       sprintf(tmp_str, "Tape ID truncated to %s", tapeid[0]);
       c_errmsg(tmp_str, "tapeout-longid", &nonfatal);
       }
   

   /* Add the tapeid to the list of tapeids in MTAPEID
      First read in the existing list of tapeids
      ------------------------------------------------ */
   var = p_find(&vblock, "$MTAPEID");
   for (i = 0; i < var->v_count; i++)
     tapeids[i] = SVAL(*var, i);

   /* Add the new tapeid to the end of the list
      ----------------------------------------- */
   tapeids[i] = tapeid[0];
   var->v_count++;

   /* Save the updated list back to MTAPEID
      ------------------------------------- */
   q_string(&vblock, "$MTAPEID", var->v_count, tapeids, P_UPDATE);
   q_out(&vblock);
   }

/* If label_file is a non empty string then output info for tape labels
-----------------------------------------------------------------------*/
if (strlen(label_file) > 0)
   {

   /* Place the labels flag in the vblock
   --------------------------------------*/
   j = TRUE;
   if (q_intg(&vblock, "LABELS", 1, &j, P_UPDATE) != SUCCESS)
      {
      c_errmsg("Error passing integer to TAE","tapeout-intg",&nonfatal);
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
      }
   if (q_out(&vblock) != SUCCESS)
      {
      c_errmsg("Error sending variable block to TAE","tapeout-out",&nonfatal);
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
      }

   /* Write the tape information to label_file.  Write the operators initials
      as ERR, (ie. assume an error will occur), and if the tape is successfully
      completed then this can be changed to SUC.
   ---------------------------------------------------------------------------*/
   c_date_time(date[0], time[0]);
   strcpy(ops[0], "ERR");
   sprintf(sequence[0], "%1d/", curvol + 1);
   if (c_putparm(label_file,"TAPEID",tapeid,"S",dbl_val,1,lng_val) != E_SUCC)
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   if (c_putparm(label_file,"DENSITY",density,"S",dbl_val,1,lng_val)!=E_SUCC)
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   if (c_putparm(label_file,"DRIVE",comment[curvol],"S",dbl_val,1,lng_val)
      != E_SUCC)
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   if (c_putparm(label_file,"OPS",ops,"S",dbl_val,1,lng_val) != E_SUCC)
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   if (c_putparm(label_file,"SEQUENCE",sequence,"S",dbl_val,1,lng_val)!= E_SUCC)
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   if (c_putparm(label_file,"TIME",time,"S",dbl_val,1,lng_val) != E_SUCC)
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   if (c_putparm(label_file,"CREATE",date,"S",dbl_val,1,lng_val) != E_SUCC)
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   }
else
   {
   /* Place the labels flag in the vblock
   --------------------------------------*/
   j = FALSE;
   if (q_intg(&vblock, "LABELS", 1, &j, P_UPDATE) != SUCCESS)
      {
      c_errmsg("Error passing integer to TAE","tapeout-intg",&nonfatal);
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
      }
   if (q_out(&vblock) != SUCCESS)
      {
      c_errmsg("Error sending variable block to TAE","tapeout-out",&nonfatal);
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
      }
   }

/* If processing message was stopped, restart it
-----------------------------------------------*/
if (flag)
   c_startm();

return;
}

/*******************************************************************************
FUNCTION:	REOPEN_TAPE

PURPOSE: 	Reopens a tape device.  The tape drive name is stored in the
		global "drive".  The drive name is saved in the TAE parameter
		block.

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  ???    ???              ???    ???   Development
  1.0    J.Fenno          02/92  EDC   Changed so that tpreop is passed the user
				       specified tape density instead of 0.

ALGORITHM REFERENCES	none
*******************************************************************************/
void reopen_tape
(
    long *unit,                 /* tape unit number                           */
    long access                 /* open tape for read access                  */
)
{
extern char *drive[1];          /* string descriptor for tape drive           */
extern char density[1][CMLEN];  /* tape density in BPI                        */
extern struct PARBLK vblock;	/* TAE parameter block                        */

long rewopt = NOREW;		/* rewind option                              */
long mode = HANDLE;		/* tape I/O outputs error messages            */
long fatal = -1;		/* Fatal error code			      */
long skip = 0;			/* skip zero files on open                    */
long status;			/* function return status code                */


/* Reopen the tape on the specified drive
-----------------------------------------*/
status = c_tpreop(unit,&skip,&mode,&rewopt,density[0],&access,drive[0]);
if (status != TAPE_SUCC)
   c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);

/* Update the parameter block with the tape drive
-------------------------------------------------*/
status = q_string(&vblock,"DRIVE",1,drive,P_UPDATE);
status = q_out(&vblock);

return;
}

/*******************************************************************************
FUNCTION:	WRITE_TAPE

PURPOSE:	Writes a record onto tape and checks the status of a tape 
		write.  If the end of tape is encountered, the next tape
		volume is mounted.  Note that if the multi-volume flag is
		set to "NO" the current image being processed is erased
		and written to the next tape volume.

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  ???    ???              ???    ???   Development
  1.0    J.Fenno          02/92  EDC   Added the capability to produce tape
				       labels when creating tapes at the EROS
				       Data Center.
  2.o  	 T.Mittan	  08/92	 EDC   Modified bksp_file to back space the
				       proper number of files, added call to
				       c_tpweof to write eof marks.

ALGORITHM REFERENCES	none
*******************************************************************************/
int write_tape
(
    long *unit,          /* tape unit number                                   */
    unsigned char *buf,  /* buffer to be written to tape                       */
    long *out_byte,      /* number of bytes to be written                      */
    long *bckrec        /* number of records to back space to get back to the
                           beginning of the bands (BIL only)                  */
)
{
extern char label_file[];/* File containing the tape label info               */
extern char multivol[4];/* flag as to how to split tape vol. 	      	      */
extern long tpfil;	/* # of files on current tape        	      	      */

char msg[ERRLEN];	/* message to be outputted           		      */
char ops[1][CMLEN];     /* Operators initials field of tape label             */

long fatal    = -1;	/* Non-fatal error code				      */
long lng_val[1];        /* Array of long values used for call to c_putparm    */
long mode     = HANDLE;	/* tape I/O outputs error msg.       		      */
long nonfatal =  1;	/* Non-fatal error code				      */
long status;		/* function return status code       		      */
long zero     =  0;     /* Zero						      */
long eofs = 3;
struct TAPEDESC *drive;

double dbl_val[1];      /* Array of double values used for call to c_putparm  */


/* Write the buffer to tape
---------------------------*/
status = c_tpwrit(unit,buf,out_byte,&mode);
if (status == *out_byte)
   return(E_SUCC);

/* Check the return status
--------------------------*/
switch (status)
   {
   case TAPE_EOT:
      if ((strcmp(multivol,"NO") == 0) && (tpfil > 0))
         {
	 /* Stop the processing message
	 ------------------------------*/
	 c_stoptm();
	 sprintf(msg,"EOT encountered.  There are %d images on this tape",
                 tpfil);
	 c_errmsg(msg," ",&nonfatal);

	 drive = (struct TAPEDESC *) *unit;
         if (drive->flags & NOEOT)
            { 
	    c_errmsg("No backspacing after EOT","tapeout-noeot",&nonfatal);
	    /* Close the current tape
	    -------------------------*/
	    close_tape(unit,REWOFFL,1);
            }
         else
         {
	 /* Backspace to the end of the last complete file
	 -------------------------------------------------*/
         bksp_file(unit);

	 /* Write EOT mark 
	 ---------------*/
         if ((status = c_tpweof(unit,&eofs,&mode)) != TAPE_SUCC)
   	 c_errmsg("Nonfatal error encountered", "tapeout-warn", &nonfatal);

	 /* Close the current tape
	 -------------------------*/
	 close_tape(unit,REWOFFL,1);

         }
	 /* If label_file isn't an empty string then print a label for this tape
	 ---------------------------------------------------------------------*/
	 if (strlen(label_file) > 0)
	    {
	    strcpy(ops[0], "SUC");
	    c_putparm(label_file, "OPS", ops, "S", dbl_val, 1, lng_val);
	    print_label(label_file);
	    }

	 /* Mount the next volume
	 ------------------------*/
         mount_tape(unit,TWRITE,FALSE);

	 /* Send a code to the PDF so the file is reprocessed
	 ----------------------------------------------------*/
	 z_exit(NEXT_VOL,"tpout-nextvol");
         }
      else
	 {
	 /* Stop the processing message
	 ------------------------------*/
	 c_stoptm();
	 c_errmsg("EOT encountered.  Image does not fit on one tape",
		  " ", &nonfatal);

	 /* Back space the specified number of records before dismounting the
	    tape.  This is used only for BIL processing to prevent breaking
	    up the lines of a band
	 --------------------------------------------------------------------*/
	 if (*bckrec > 0)
	    {
	    drive = (struct TAPEDESC *) *unit;
            if (drive->flags & NOEOT)
	       c_errmsg("No backspacing after EOT","tapeout-noeot",&nonfatal);
            else
	       c_tpbsr(unit,bckrec,&mode);
	    close_tape(unit,REWOFFL,2);
	    }
	 else
	    close_tape(unit,REWOFFL,2);

	 /* If label_file isn't an empty string then print a label for this tape
	 ---------------------------------------------------------------------*/
	 if (strlen(label_file) > 0)
	    {
	    strcpy(ops[0], "SUC");
	    c_putparm(label_file, "OPS", ops, "S", dbl_val, 1, lng_val);
	    print_label(label_file);
	    }

	 /* Mount the next tape volume
	 -----------------------------*/
	 mount_tape(unit,TWRITE,FALSE);

	 /* Rewrite the record to the next tape
	 --------------------------------------*/
	 if (*bckrec == 0)
	    write_tape(unit,buf,out_byte,&zero);

	 /* restart the processing message
	 ---------------------------------*/
  	 c_startm();
	 }
      break;

   case TPWRIT_SIZE:
      /* Stop the processing message
      ------------------------------*/
      c_stoptm();

      /* Back space to the end of the last file processed to erase it.
      ---------------------------------------------------------------*/
      bksp_file(unit);

      /* Display a fatal error message and terminate
      ----------------------------------------------*/
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
      break;

   case TPWRIT_WRITE:
      /* Stop the processing message
      ------------------------------*/
      c_stoptm();

      /* Back space to the end of the last file processed to erase it.
      ---------------------------------------------------------------*/
      bksp_file(unit);

      /* Display a fatal error message and terminate
      ----------------------------------------------*/
      c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
      break;

   default:
      /* Stop the processing message
      ------------------------------*/
      c_stoptm();

      /* Back space to the end of the last file processed to erase it.
      --------------------------------------------------------------*/
      bksp_file(unit);

      /* Display a fatal error message and terminate
      ----------------------------------------------*/
      c_errmsg("Fatal error encountered", "tapeout-fatal",&fatal);
      break;
   }

return(status);
}

/******************************************************************************
FUNCTION:	CLOSE_TAPE

PURPOSE:	Closes the tape device.

ALGORITHM 
  Close the tape

ALGORITHM REFERENCES	none
*******************************************************************************/
void close_tape
(
    long *unit,         /* tape unit number                                   */
    long rewopt,        /* rewind option                                      */
    long eofs           /* Number of EOF's to write to tape                   */
)
{
long mode = NO_HANDLE;	/* tape I/O outputs error messages   		      */
long nonfatal = 1;	/* Non-fatal error code				      */

/* Close the tape
-----------------*/
if (c_tpclos(unit, &mode, &rewopt, &eofs) != TAPE_SUCC)
   c_errmsg("Nonfatal error encountered", "tapeout-warn", &nonfatal);

return;
}
