/******************************************************************************
FUNCTION:	TAPEOUT

PURPOSE:	Copies a LAS image to a magnetic tape.  The image may be
		windowed, subsampled, or converted to a new data type prior
		to being written to tape.  The user has the option to copy
		the label records of the LAS image to tape as a header file.
		The data descriptor record is updated to represent the
		generated output image; however, all other label records are
		directly copied from the LAS image to tape.


PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         1/86       K. Gacke     initial development
    1.2        12/86       K. Gacke     For BYTE images with an odd number of
					samples being made for an HP system, a
					zero-valued pixel is added to the end of
					each input line so the output record 
					length is even and a limit of 14999
					samples in the window image is applied.
    5.0        01/89       D. Hollaren  Conversion to LAS 5.0
    5.1        05/89       D. Hollaren  Now generates 1600 bpi tapes correctly
    5.2        03/90       B. Davis     changed stoptm to c_stoptm
    5.3        08/90       B. Davis     when encountering EOT upon writing,
					changed the call to close_tape to pass
					2 for eofs to write instead of 0.  
					Tapeio has been changed to check for EOT
					before writing eofs, and EOT is no long-
					er a fatal error for either writing eofs
					or closing a tape.
    5.4        02/92       J. Fenno     Modified so that tape labels are
					automatically generated if the function
					initlbl has been run prior to tapeout.
					The labels are printed to the label
					printer specified in
					$WORCONFIG/initlbl.config
    5.5        03/92       J. Fenno     Added code so that unique file names are
				        used allowing tapeout to be run multiple
				        times from the same working directory.
				        PRF Z0810.
    5.6	       09/92	   T. Mittan 	Added error messages if EOT is not
					supported on a computer system.  Added
					errmesage if outsys is not the same as
					insys and a ddr is being written to 
					tape.
    5.61       09/92       D. Etrheim   Modified to pass the OPS parameter to
                                        c_tapelabel.  Also modified to let
                                        c_tapelabel handle remotely mounted
                                        tapes.
    5.62       10/92       D. Etrheim   Increased the length DESCRPT for label
                                        printing. PRF#P6130.  Return tapeid
                                        to a global PRF$6133.
    5.7        01/93       K. Sheffield Modified so an ascii DDR can be output
                                        to tape. PRF P6135.  Also fixed a 
                                        problem appending images on a tape on
                                        certain tape drives (eg. 3480's)
    5.2        03/93       D. Etrheim	Modified bksp_file.c so the correct
					number of files is sent to c_tpbsf 
					when the ddr_flag is true.
    5.3	       08/94       D. Etrheim	Correct problem with uninitialized
					variable hp_flag
    6.0        04/98       B. Olson     Correct problem with returning
                                        drive id when user aborts

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	On the first invocation of the tape copy routine, the tape is mounted 
	and the physical tape drive name is saved so that the tape may be 
	reopened.  After processing of the image file is complete, the tape 
	is closed (but not dismounted) as long as there is another image file 
	to be copied, and control is returned to TAPEOUT.  When the last image 
	file has been processed, the tape is dismounted.

	Once the tape file is opened, the specified number of image files are 
	skipped.  If parameter DDRFLG is set to "yes", the DDR of the LAS
	image is copied into a file on tape.  The DDR is updated prior to
	being written to tape.  The LAS image is copied to tape in BIL or
	BSQ format, and of any valid data type (BYTE, INTEGER*2, INTEGER*4,
	REAL*4).  After the last image file has been copied to tape, the
	tape is dismounted.

ALGORITHM REFERENCES:	none.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "protos.h"

#define NOVAL          0        /* Parameter value was assigned no value     */
#define MAX_VOL       15        /* Maximum number of tape volumes            */
#define MAX_BLKSIZ 32767        /* Maximum block size on tape                */
#define NEXT_VOL     100        /* Flag indicating image did not fit on the  */
                                /* current tape.                             */
/* Global variable declarations
-------------------------------*/
struct PARBLK vblock;           /* TAE parameter block                       */
char  comment[MAX_VOL][CMLEN];  /* Tape mount message to the operator        */
char  density[1][CMLEN];        /* Tape density entered by the user          */
char  *drive[1];                /* Physical tape drive name                  */
char  label_file[CMLEN];        /* File containing tape label info (no ext)  */
char  multivol[4];              /* Flag as to how to split tape vol.         */
struct FDESC * fdesc[MAXIMG+1]; /* Image file descriptors used for image I/O */
struct GDESC * gdesc;           /* Group descriptor used for image I/O       */
extern long  curcat;            /* Current output band being processed       */
extern long  curent;            /* Current output line being processed       */
int  curvol;                   /* Current tape volume being process         */
long  datasize[NDTYPES] = {0,1,2,4,4,8};/* # of bytes used for each data type*/
long  offset[MAXIMG*MAXBND+1];  /* Offsets into "thebuf" for input bands     */
long  tpfil;                    /* # of files on current tape                */
short ddr_flag;                 /* DDR header flag                           */
unsigned char *thebuf;          /* Image I/O buffer                          */

int main
(void)
{
static long bands[MAXIMG][MAXBND+1];/* list of input bands to be copied      */
static long window[MAXIMG][4];      /* input window specification            */
static long nbands[MAXIMG];         /* # bands in each input image           */

long qflag[MAXIMG][2];		    /* windowing/banding option flags        */
long ad_flag;                       /* flag for ASCII DDR 1=yes, 0=no        */
long aft_flag;                      /* placement flag for ASCII DDR 1=after,
				       0=before                              */
long blksiz;                        /* number of lines in output block       */
long hp_flag = FALSE;               /* flag for HP computer system           */
long insys;                         /* Input system                          */
long linc;                          /* line increment for subsample          */
long msgtim;                        /* elapsed time between display msg      */
long nmrimg;                        /* number of input images                */
long odtype;                        /* data type of output image             */
long outsys;                        /* Output system                         */
long sinc;                          /* sample increment for subsample        */
long skip;                          /* # of files to skip prior to copy      */
long totbnd;                        /* total number of bands to be copied    */
long unit;                          /* unit number of tape device            */
long wrteov_flag;                   /* flag for writing End of Volume        */
long zero = 0;                      /* Value of zero                         */

static char bform[4];               /* band format of output image           */
static char taein[MAXIMG][CMLEN];   /* LAS input file names                  */
static char hosin[MAXIMG][CMLEN];   /* Host input file names                 */
static char out[1][CMLEN];          /* Output files created                  */


/* Get the input parameters
---------------------------*/
getpar(&insys,&outsys,taein,hosin,window,bands,nbands,&totbnd,&odtype,&blksiz,
       &linc,&sinc,&nmrimg,&skip,bform,density[0],&hp_flag,&ad_flag,&aft_flag,
       &wrteov_flag, &msgtim, qflag);

/* Open the input image
-----------------------*/
open_image(hosin,&nmrimg,window,bands,nbands,&odtype,bform);

/* Mount a tape or reopen the drive
-----------------------------------*/
if (strlen(drive[0]) == 0)
   {
   mount_tape(&unit,TWRITE,FALSE);
   if (skip)
       skip_images(&unit,skip);
   }
else
   reopen_tape(&unit,TWRITE);

/* Update the parameter block with the drive the tape was mounted on.
---------------------------------------------------------------------*/
q_string(&vblock,"DRIVE",1,drive,P_UPDATE);
q_out(&vblock);

/* Copy the input data to tape in BIL or BSQ format
---------------------------------------------------*/
if (strcmp(bform,"BIL") == 0)
    cp_bil(&unit,&insys,&outsys,hosin,&nmrimg,window,bands,nbands,&totbnd,
	   &odtype,&blksiz,&linc,&sinc,&hp_flag,&ad_flag,&aft_flag,&msgtim,
	   taein);
else if (strcmp(bform,"BSQ") == 0)
    cp_bsq(&unit,&insys,&outsys,hosin,&nmrimg,window,bands,nbands,&totbnd,
	   &odtype,&blksiz,&linc,&sinc,&hp_flag,&ad_flag,&aft_flag,&msgtim,
	   taein,qflag);

/* Close the tape device
------------------------*/

if(wrteov_flag)
   close_tape(&unit,REWOFFL,2);
else
   close_tape(&unit,NOREW,1);

++tpfil;

/* Update the parameter block with the current tape volume,
   tape file number and the drive the tape was mounted on.
-----------------------------------------------------------*/
q_intg(&vblock,"CURVOL",1,&curvol,P_UPDATE);
q_intg(&vblock,"TPFIL",1,&tpfil,P_UPDATE);
q_string(&vblock,"DRIVE",1,drive,P_UPDATE);
q_string(&vblock,"$TAPEDRV",1,drive,P_UPDATE);
q_out(&vblock);

/* Standard completion message
------------------------------*/
c_complt("TAPEOUT",out,&zero);

return EXIT_SUCCESS;

}
