/*******************************************************************************
NAME      		  GETPAR 

PURPOSE
	GETPAR will retrieve and perform the final editing of the 
	parameters for the TASSELCP module.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      Sept  1988      Written for LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   GETPAR retrieves the required TAE parameters from the parameter block.  The
   input images are parsed so that the window and band specifications are
   known.
******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block            */
    char taein[][CMLEN], 	/* pointer to input TAE file descriptor      */
    char hosin[][CMLEN], 	/* pointer to input host file descriptor     */
    char taeout[][CMLEN],		/* TAE file name of output image             */
    char hosout[][CMLEN],		/* host file name of output image            */
    long *nmrimg,		/* number of input images                    */
    long (*window)[4],		/* window specifications for each input image*/
    long (*bands)[MAXBND+1],	/* band specifications for each input image  */
    long nbands[],		/* number of bands per input image	     */
    long *totbnd,		/* total number of bands to be processed     */
    long *msgtim,		/* elapsed time between display messages     */
    char *minmax,		/* min max flag buffer			     */
    long *term,			/* flag for output to terminal 		     */
    long *fyle,			/* flag for output to file		     */
    long *lnpr,			/* flag for output to line printer	     */
    char *filnm,			/* output filename for FILE option           */
    char *outfil, 		/* pointer to output host file name          */
    char *sub,   		/* type of processing			     */
    long *brght, 		/* generate brightness band		     */
    long *grn,			/* generate greenness band		     */
    long *wt,			/* generate wetness band		     */
    float bright[],		/* brightness coefficients array	     */
    float green[],		/* greenness coefficients array		     */
    float wet[],			/* wetness coefficients array		     */
    float brsoil[],		/* mean bright soil array		     */
    float dksoil[],		/* mean dark soil array			     */
    float grnveg[],		/* mean green vegatation array  	     */
    float dryveg[],		/* mean dry vegatation array		     */
    long *odtype		/* output data type			     */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long    element = 0;		/* element number for getin                  */
long    fatal = -1;		/* fatal error flag			     */
long    i,x;			/* counters				     */
long	maximg;			/* maximum number of images to be processed  */
long    nonfatal = 1;		/* fatal error flag			     */
long	qflag[MAXIMG][2];	/* flag indicating if subwindows specified   */
long	same;			/* flag indicating if images have same window*/
long	status;			/* function return status code               */

char    dtype[5];		/* data type of output image		     */
char    whichbnd[7];		/* which bands to create		     */

maximg = MAXIMG;

/*
  get the subcommand name
*/
  val = p_find(vblock, "_SUBCMD");
  strcpy(sub,SVAL(*val,0));

/*
  get first image specification
*/
element = 0;
status = c_getin( vblock, "IN", &element, taein, hosin, nmrimg, window, 
               bands, nbands, totbnd, &maximg, &same, qflag);
if (status != E_SUCC)
   {
   c_errmsg("Error opening input images","tasselcp-openin",&nonfatal);
   c_errmsg("Fatal error encountered","tasselcp-fatal",&fatal);
   }

if (same != 1)
   {
   c_errmsg("Input images are of different size",
           "tasselcp-images",&nonfatal);
   c_errmsg("Fatal error encountered","tasselcp-fatal",&fatal);
   }

/*
   Retrieve the output file
*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   {
   c_errmsg("Error opening output images","tasselcp-openout",&nonfatal);
   c_errmsg("Fatal error encountered","tasselcp-fatal",&fatal);
   }

/*
   for the APPLY subcommand retrieve the coefficients
*/

if (strcmp(sub,"APPLY") == 0)
{
/* 
  get brightness coefficients
*/
val = p_find(vblock, "BRIGHTCO");
i = (*val).v_count;
for (x=0; x<i; x++)
   bright[x] = RVAL(*val,x);

/*
  get greenness coefficients
*/
val = p_find(vblock, "GREENCO");
i = (*val).v_count;
for (x=0; x<i; x++)
   green[x] = RVAL(*val,x);

/*
  get wetness coefficients
*/
val = p_find(vblock, "WETCO");
i = (*val).v_count;
for (x=0; x<i; x++)
   wet[x] = RVAL(*val,x);

}
/*
   for the CREATE subcommand retrieve the mean values
*/
if (strcmp(sub,"CREATE") == 0)
{
/*
  get mean bright soil values 
*/
val = p_find(vblock, "DRYSOIL");
i = (*val).v_count;
for (x=0; x<i; x++)
   brsoil[x] = RVAL(*val,x);

/*
  get mean dark soil values 
*/
val = p_find(vblock, "WETSOIL");
i = (*val).v_count;
for (x=0; x<i; x++)
   dksoil[x] = RVAL(*val,x);

/*
  get mean green vegatation values 
*/
val = p_find(vblock, "GRNVEG");
i = (*val).v_count;
for (x=0; x<i; x++)
   grnveg[x] = RVAL(*val,x);

/*
  get mean dry vegatation values 
*/
val = p_find(vblock, "DRYVEG");
i = (*val).v_count;
for (x=0; x<i; x++)
   dryveg[x] = RVAL(*val,x);

}
/*
   determine which bands to create
*/
    val = p_find(vblock,"BANDS");
    i = (*val).v_count;
    *brght = *grn = *wt = 0;
    for (x=0;x<i; x++)
      {
      strcpy(whichbnd,SVAL(*val,x)); 
           if (strcmp(whichbnd,"BRIGHT") == 0)
	      *brght = 1;
      else if (strcmp(whichbnd,"GREEN") == 0)
	      *grn = 1;
      else if (strcmp(whichbnd,"WET") == 0)
  	      *wt = 1;
      }

/*
   Retrieve the print option
*/
  strcpy(filnm,"tasselcp");
  status = c_setprt(vblock,term,fyle,lnpr,filnm,outfil);
  if (status != E_SUCC)
     c_errmsg("Fatal error encountered","tasselcp-fatal",&fatal);

/*
   get the output data type
*/
val = p_find(vblock,"ODTYPE");
strcpy(dtype,SVAL(*val,0));
if (strcmp(dtype,"SAME") == 0)
	*odtype = 0;
else if (strcmp(dtype,"BYTE") == 0)
        *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
        *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
        *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
        *odtype = EREAL;
/*
   Retrieve TAE globals
*/

val = p_find(vblock,"$MSGTIM");	
*msgtim = IVAL(*val,0);

val = p_find(vblock,"$MINMAX");
strcpy(minmax,SVAL(*val,0));

return;
}
