/******************************************************************************
NAME      		  TASSEL 

PURPOSE  Generate a tasseled cap image from either MSS or TM images.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.McKinney        Sept 1988     Written for LAS 5.0 
D.Eichacker	  May  1994	added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
    TASSEL must be run under TAE.

PROJECT          	 LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
For each band of input the coefficients are applied and added to
form the Tasseled cap image of brightness, greenness, and wetness.
Finally, associated min/max and history files are created for the output 
image and a completion message is output.

ALGORITHM REFERENCES     none
*******************************************************************************/


#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	/* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND+1];/* offsets into "thebuf" for input bands*/
extern long offset2[MAXBND+1];  /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void tassel
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,			/* output data type			     */
    float *bright,		/* brightness coefficients array             */
    float *green,		/* greenness coefficients array		     */
    float *wet,			/* wetness coefficients array		     */
    long brght,			/* brightness band generation		     */
    long grn,			/* greenness band generation		     */
    long wt			/* wetness band generation		     */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

static	long	opt[3]={0,0,0};	/* optional parameter array for EOPENS	     */
long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sample;			/* sample indicator			     */
long	size = 0;		/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    totout;			/* total number of bands to create	     */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */

float   temp;			/* temporary storage area		     */

unsigned char  *thebuf;	        /* image input buffer                        */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      {
      c_errmsg("Error opening input images","tasselcp-open",&nonfatal);
      c_errmsg("Fatal error encountered","tasselcp-fatal",&fatal);
      }
   band += nbands[image];

   }

/* Open output image
--------------------*/
totout = brght + grn + wt;
access = IWRITE;	
status = c_eopens(&fdesc[nmrimg],hosout[0],&totout,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
      {
      c_errmsg("Error opening output images","tasselcp-open",&nonfatal);
      c_errmsg("Fatal error encountered","tasselcp-fatal",&fatal);
      }
fdesc[nmrimg+1] = 0;

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","tasselcp-alloc",&nonfatal);
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tasselcp-warn",&nonfatal);
   c_clean(vblock,"tasselcp",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   c_errmsg("Error allocating dynamic memory","tasselcp-alloc",&nonfatal);
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tasselcp-warn",&nonfatal);
   c_clean(vblock,"tasselcp",hosout,&one);
   }

/* determine size from odtype
-----------------------------*/
switch (odtype)
  {
  case EBYTE:  size = 1; break;
  case EWORD:  size = 2; break;
  case ELONG:  size = 4; break;
  case EREAL:  size = 4; break;
  }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("TASSELCP","Processed","line","bands"," ",
		&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totout,minval,maxval);

/* Main processing loop
-----------------------*/
/*
  compute tasselcp
*/
for (curlin = 0; curlin < nl; ++curlin)
   {
   if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing images","tasselcp-close",&nonfatal);
   c_clean(vblock,"tasselcp",hosout,&one);
   }

    for (sample = 0; sample < ns; sample++)
      {
      if (brght == 1)
	{
        temp = 0.0;
	switch (odtype)
	  {
	  case 1:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(thebuf + (*(offset1+band)+sample) * size))
	        * bright[band]; 
	  if (temp > 255.0) temp = 255.0;
	  else if (temp < 0.0) temp = 0.0;
	  (*(thebuf + (*(offset2)+sample) * size)) = temp;
	  break;
	  case 2:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(short *)(thebuf + (*(offset1+band)+sample) * size))
	        * bright[band]; 
	  if (temp > 32767.0) temp = 32767.0;
	  else if (temp < -32768.0) temp = -32768.0;
	  (*(short *)(thebuf + (*(offset2)+sample) * size)) = temp;
	  break;
	  case 3:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (float) ((*(long *)(thebuf + (*(offset1+band)+sample) * size)))
	        * bright[band]; 
	  if (temp > 2147483647.0) temp = 2147483647.0;
	  else if (temp < -2147483648.0) temp = -2147483648.0;
	  (*(long *)(thebuf + (*(offset2)+sample) * size)) = (long) temp;
	  break;
	  case 4:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(float *)(thebuf + (*(offset1+band)+sample) * size))
	        * bright[band]; 
	  if (temp > 2147483647.0) temp = 2147483647.0;
	  else if (temp < -2147483648.0) temp = -2147483648.0;
	  (*(float *)(thebuf + (*(offset2)+sample) * size)) = temp;
	  break;
	  }
	}
      if (grn == 1)
	{
        temp = 0.0;
	switch (odtype)
	  {
	  case 1:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(thebuf + (*(offset1+band)+sample) * size))
	        * green[band]; 
	  if (temp > 255.0) temp = 255.0;
	  else if (temp < 0.0) temp = 0.0;
	  (*(thebuf + (*(offset2+brght)+sample) * size)) = temp;
	  break;
	  case 2:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(short *)(thebuf + (*(offset1+band)+sample) * size))
	        * green[band]; 
	  if (temp > 32767.0) temp = 32767.0;
	  else if (temp < -32768.0) temp = -32768.0;
	  (*(short *)(thebuf + (*(offset2+brght)+sample) * size)) = temp;
	  break;
	  case 3:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (float) ((*(long *)(thebuf + (*(offset1+band)+sample) * size)))
	        * green[band]; 
	  if (temp > 2147483647.0) temp = 2147483647.0;
	  else if (temp < -2147483648.0) temp = -2147483648.0;
	  (*(long *)(thebuf + (*(offset2+brght)+sample) * size)) = (long) temp;
	  break;
	  case 4:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(float *)(thebuf + (*(offset1+band)+sample) * size))
	        * green[band]; 
	  if (temp > 2147483647.0) temp = 2147483647.0;
	  else if (temp < -2147483648.0) temp = -2147483648.0;
	  (*(float *)(thebuf + (*(offset2+brght)+sample) * size)) = temp;
	  break;
	  }
	}
      if (wt == 1)
	{
        temp = 0.0;
	switch (odtype)
	  {
	  case 1:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(thebuf + (*(offset1+band)+sample) * size))
	        * wet[band]; 
	  if (temp > 255.0) temp = 255.0;
	  else if (temp < 0.0) temp = 0.0;
	  (*(thebuf + (*(offset2+brght+grn)+sample) * size)) = temp;
	  break;
	  case 2:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(short *)(thebuf + (*(offset1+band)+sample) * size))
	        * wet[band]; 
	  if (temp > 32767.0) temp = 32767.0;
	  else if (temp < -32768.0) temp = -32768.0;
	  (*(short *)(thebuf + (*(offset2+brght+grn)+sample) * size)) = temp;
	  break;
	  case 3:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (float) ((*(long *)(thebuf + (*(offset1+band)+sample) * size)))
	        * wet[band]; 
	  if (temp > 2147483647.0) temp = 2147483647.0;
	  else if (temp < -2147483648.0) temp = -2147483648.0;
	  (*(long *)(thebuf + (*(offset2+brght+grn)+sample) * size)) = (long) temp;
	  break;
	  case 4:
          for (band = 0; band < totbnd; band++)
	  temp +=
	       (*(float *)(thebuf + (*(offset1+band)+sample) * size))
	        * wet[band]; 
	  if (temp > 2147483647.0) temp = 2147483647.0;
	  else if (temp < -2147483648.0) temp = -2147483648.0;
	  (*(float *)(thebuf + (*(offset2+brght+grn)+sample) * size)) = temp;
	  break;
	  }
	}
     }
   curent++;


   if (strcmp(minmax,"YES") == 0)
      {
      if (brght == 1)
      status = c_pxmnmx(thebuf + (*(offset2) * size),&ns, &minval[0],
	        &maxval[0],&odtype);
      if (grn == 1)
      status = c_pxmnmx(thebuf + (*(offset2+brght) * size),&ns,
		&minval[brght],&maxval[brght],&odtype);
      if (wt == 1)
      status = c_pxmnmx(thebuf + (*(offset2+brght+grn) * size),&ns,
	        &minval[brght+grn],&maxval[brght+grn],&odtype);
       }
   }

/*
  final buffer flush
*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing images","tasselcp-close",&nonfatal);
   c_clean(vblock,"tasselcp",hosout,&one);
   }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = COMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,
                 &line_inc, &samp_inc,&totout,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"tasselcp",hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&totout,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Error updating the minmax information","tasselcp-minmax",
		&nonfatal);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing images","tasselcp-close",&nonfatal);

/* Free buffer
--------------*/
free(thebuf);
return;
}
