/*****************************************************************************

     NAME:   GETPAR

     PURPOSE:     GETPAR will retrieve all parameters from TAE and make
                  some necessary calculations and error checking.

     PROGRAM HISTORY:
     VERSION     PROGRAMMER	DATE		REASON
     -------     ----------    	----		------
       5.0       R. McKinney	May 1988	Converted to LAS 5.0
       5.0       R. McKinney    Dec 1988	Updated to close all files
       5.1	 D. Akkerman	Apr 1990	Corrected problems in checking
						min/max values.
       5.2       D.VanderZee    Sep 1990        Fixed input for -RAN to read
						in value for seedval.
       5.3       D. Etrheim     Oct  1991       ADP #5621 Fix for fractional 
					step sizes for real images.  Correct 
					NLEVELS validation check for byte 
					images.
       5.4       D. Etrheim     Sep 1992        Change parameters STDDEV and
						PIXVAL1-6 to STDEV and 
						PIXVAL for standards

     
     COMPUTER HARDWARE AND/OR SOFTARE LIMITATIONS:
	GETPAR must be run under TAE as part of TESTGEN

     PROJECT:			LAS

     LANGUAGE:			 C

     ALGORITHM:

	Receive subcommand
	Receive parameters that are global to all subcommands 
	Receive parameters that are unique to the individual subcommand
	Checks are made for validation as each parameter is accepted.
	Receive the globals.

     TESTING complete on SUN UNIX (unix), PN9050 UNIX (gould), and
        VAX VMS (vms).

***************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "testgen.h"

void getpar
(
  struct PARBLK *vblock,          /* Pointer to TAE param. block */
  long *msgtim,                   /* Time between display messages */
  char *taeout,                   /* TAE output file name */
  char *hosout,                   /* Host output file name */
  long *nl,                       /* Number of lines in output image */
  long *ns,                       /* Number of samples in output image */
  char *minmax,                   /* Min/Max flag buffer */
  long *dflag,                    /* DDR update option */
  long *bands,                    /* Band spec. for image */
  float pixval[],                 /* Pixel values for uniform image */
  float *maxval,                  /* Maximum pixel value */
  float *minval,                  /* Minimum pixel value */
  long *seedval,                  /* Seed value */
  float *meanval,                 /* Mean value */
  float *stddev,                  /* Standard deviation */
  char *weddir,                   /* Direction of the wedge */
  float *ltgrey,                  /* Left of upper grey value */
  float *rbgrey,                  /* Right or bottom grey value */
  long *nlevels,                  /* Number of levels */
  char *method,                   /* Method of use */
  long *odtype                    /* Output data type */
)
{

struct VARIABLE *var;		/* pointer to a TAE variable within vblock    */

long    fatal = -1;		/* fatal error flag                           */
long	i;			/* counter				      */
long    save_vcount = 0;        /* save value for pixel count                 */

char    dflag_str[4];           /* DDR update option string                   */
char    typestr[7];		/* odtype string			      */

unsigned char minbyte = MINUTINY;/* Minimum byte value.		      	      */
unsigned char maxbyte = MAXUTINY;/* Maximum byte value.			      */

short	minword = MINWORD;	/* Minimum word value.			      */
short	maxword = MAXWORD;	/* Maximum word value.			      */

long	minlong = MINLONG;	/* Minimum long value.			      */
long	maxlong = MAXLONG;	/* Maximum long value.			      */

float	minreal = MINREAL;	/* Minimum real value.			      */
float	maxreal = MAXREAL;	/* Maximum real value.			      */


/*
  determine the subcommand chosen
*/

var = p_find(vblock, "_SUBCMD");
strcpy(method, SVAL(*var,0));

/*
   get output image name
*/

if (c_getout(vblock, "out", taeout, hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","testgen-fatal",&fatal);

/*
  get the output data type
*/

var = p_find(vblock, "odtype");
strcpy(typestr,SVAL(*var,0));

if (strcmp(typestr,"BYTE") == 0)
   *odtype = EBYTE;
else if (strcmp(typestr,"I*2") == 0)
   *odtype = EWORD;
else if (strcmp(typestr,"I*4") == 0)
   *odtype = ELONG; 
else if (strcmp(typestr,"R*4") == 0)
   *odtype = EREAL;

/*
  get the number of desired bands
*/

var = p_find(vblock, "nbands");
*bands = IVAL(*var,0);

/*
  get the size of the new image
*/

var = p_find(vblock,"nl");
*nl = IVAL(*var,0);
var = p_find(vblock,"ns");
*ns = IVAL(*var,0);

/*
get the parameters for the uniform subcommand
*/

if (strcmp (method,"UNI") == 0)
 {
   var = p_find(vblock,"pixval");
   for (i=0; i<var->v_count; i++)
        pixval[i] = RVAL(*var,i);
   save_vcount += i;
/*   var = p_find(vblock,"pixval2");
   for (i=0; i<var->v_count; i++)
        pixval[i+50] = RVAL(*var,i);
   save_vcount += i;
   var = p_find(vblock,"pixval3");
   for (i=0; i<var->v_count; i++) 
        pixval[i+100] = RVAL(*var,i);
   save_vcount += i;
   var = p_find(vblock,"pixval4");
   for (i=0; i<var->v_count; i++)
	pixval[i+150] = RVAL(*var,i);
   save_vcount += i;
   var = p_find(vblock,"pixval5");
   for (i=0; i<var->v_count; i++)
        pixval[i+200] = RVAL(*var,i);
   save_vcount += i;
   var = p_find(vblock,"pixval6");
   for (i=0; i<var->v_count; i++)
	pixval[i+250] = RVAL(*var,i);
   save_vcount += i; */
   if (save_vcount > *bands)
       c_errmsg("Number of PIXVAL values does not match NBANDS",
                "testgen-nomatch",&fatal);  
   else
      if (save_vcount < *bands) 
         {
         if (save_vcount == 1)
            for (i = (save_vcount); i < *bands; i++)
               pixval[i] = pixval[0];
          else
             c_errmsg("Number of PIXVAL values does not match NBANDS",
                        "testgen-nomatch",&fatal);
         }

/*
   check for valid ranges
*/

   switch (*odtype)
   {
	case 1:
	for (i=0;i<save_vcount;i++)
	   {
	   if ((pixval[i] < minbyte)||(pixval[i] > maxbyte))
		c_errmsg("Band values are out of the data type range",
			"testgen-values",&fatal);
	   }
        break;
	case 2:
	for (i=0;i<save_vcount;i++)
	   {
	   if ((pixval[i] < minword) || (pixval[i] > maxword))
	 	c_errmsg("Band values are out of the data type range",
			"testgen-values",&fatal);
	   }
	break;
	case 3:
	for (i=0;i<save_vcount;i++)
	   {
	   if ((pixval[i] < minlong) || (pixval[i] > maxlong))
	 	c_errmsg("Band values are out of the data type range",
			"testgen-values",&fatal);
	   }
	break;
	case 4:
	for (i=0;i<save_vcount;i++)
	   {
	   if ((pixval[i] < minreal) || (pixval[i] > maxreal))
		c_errmsg("Band values are out of the data type range",
			"testgen-values",&fatal);
	   }
	break;
     }
   }

/*
  get the parameters for the random subcommand
*/

if (strcmp (method,"RAN") == 0)
   {
	var = p_find(vblock,"maxval");
	*maxval = RVAL(*var,0);
	var = p_find(vblock,"minval");
	*minval = RVAL(*var,0);
	var = p_find(vblock,"seedval");
	*seedval = IVAL(*var,0);

/*
	check for validity of parameters
*/

	if (*maxval < *minval)
	    c_errmsg("Minimum value greater than Maximum value",
		"testgen-minmax",&fatal);
      
	switch (*odtype)
	{
	case 1:
	if ((*maxval < minbyte) || (*maxval > maxbyte))
	    c_errmsg("Maximum value out of data type range",
		"testgen-max",&fatal);
	if ((*minval < minbyte) || (*minval > maxbyte))
	    c_errmsg("Minimum value out of data type range",
		"testgen-min",&fatal);
	if ((*seedval < minbyte) || (*seedval > maxbyte))
	    c_errmsg("Seed value out of data type range",
		"testgen-seed",&fatal);
	break;
	case 2:
	if ((*maxval < minword) || (*maxval > maxword))
	    c_errmsg("Maximum value out of data type range",
		"testgen-max",&fatal);
	if ((*minval < minword) || (*minval > maxword))
	    c_errmsg("Minimum value out of data type range",
		"testgen-min",&fatal);
	if ((*seedval < minword) || (*seedval > maxword))
	    c_errmsg("Seed value out of data type range",
		"testgen-seed",&fatal);
	break;
	case 3:
	if ((*maxval < minlong) || (*maxval > maxlong))
	    c_errmsg("Maximum value out of data type range",
		"testgen-max",&fatal);
	if ((*minval < minlong) || (*minval > maxlong))
	    c_errmsg("Minimum value out of data type range",
		"testgen-min",&fatal);
	if ((*seedval < minlong) || (*seedval > maxlong))
	    c_errmsg("Seed value out of data type range",
		"testgen-seed",&fatal);
	break;
	case 4:
	if ((*maxval < minreal) || (*maxval > maxreal))
	    c_errmsg("Maximum value out of data type range",
		"testgen-max",&fatal);
	if ((*minval < minreal) || (*maxval > maxreal))
	    c_errmsg("Minimum value out of data type range",
	   	"testgen-min",&fatal);
	if ((*seedval < minreal) || (*seedval > maxreal))
	    c_errmsg("Seed value out of data type range",
	  	"testgen-seed",&fatal); 
	break;
	}
	
   } 

/*
   get the parameters for the gaussian subcommand
*/

if (strcmp (method, "GAU") == 0)
   {
	var = p_find(vblock,"meanval");
	*meanval = RVAL(*var,0);
	var = p_find(vblock,"stdev");
	*stddev = RVAL(*var,0);
	var = p_find(vblock,"seedval");
	*seedval = IVAL(*var,0);

/*
	check for the validity of parameters
*/

	switch (*odtype)
	{
	case 1:
	if ((*meanval < minbyte) || (*meanval > maxbyte))
	  c_errmsg("Mean value is out of the data type range",
		"testgen-mean",&fatal);
	if ((*stddev < minbyte) || (*stddev > maxbyte))
	  c_errmsg("Standard Deviation is out of the data type range",
		"testgen-stdev",&fatal);
	break;
	case 2:
	if ((*meanval < minword) || (*meanval > maxword))
	  c_errmsg("Mean value is out of the data type range",
		"testgen-mean",&fatal);
	if ((*stddev < minword) || (*stddev > maxword))
	  c_errmsg("Standard Deviation is out of the data type range",
	   	"testgen-stdev",&fatal);
	break;
	case 3:
	if ((*meanval < minlong) || (*meanval > maxlong))
	  c_errmsg("Mean value is out of the data type range",
		"testgen-mean",&fatal);
	if ((*stddev < minlong) || (*stddev > maxlong))
	  c_errmsg("Standard Deviation is out of the data type range",
	   	"testgen-stdev",&fatal);
	break;
	case 4:
	if ((*meanval < minreal) || (*meanval > maxreal))
	  c_errmsg("Mean value is out of the data type range",
		"testgen-mean",&fatal); 
	if ((*stddev < minreal) || (*stddev > maxreal))
	  c_errmsg("Standard Deviation is out of the data type range",
		"testgen-stdev",&fatal);
	break;
	}
   }

/*
   get the parameters for the wedge subcommand
*/

if (strcmp (method, "WED") == 0)
   {
	var = p_find(vblock,"weddir");
	strcpy(weddir, SVAL(*var,0));
	var = p_find(vblock,"ltgrey");
	*ltgrey = RVAL(*var,0);
	var = p_find(vblock,"rbgrey");
	*rbgrey = RVAL(*var,0);
	var = p_find(vblock,"nlevels");
	*nlevels = IVAL(*var,0);
/*
	check for the validity of parameters
*/

        if (*odtype != 4)
           if (*nlevels > ( abs(*rbgrey - *ltgrey) + 1))
              c_errmsg("NLEVELS are too large","testgen-nlevels",&fatal);

        if (strcmp(weddir,"V") == 0)
           {
            if (*nlevels > *nl)
               c_errmsg("NLEVELS cannot be greater than number of lines",
               "testgen-nl",&fatal);
            } 
        else 
            if (*nlevels > *ns)
               c_errmsg(
              "NLEVELS cannot be greather than number of pixels per line",
                        "testgen-ns",&fatal);
           
	switch (*odtype)
	{
	case 1:
	if ((*ltgrey < minbyte) || (*ltgrey > maxbyte))
	  c_errmsg("Left/top gray level is out of data type range",
		"testgen-ltgrey",&fatal);
	if ((*rbgrey < minbyte) || (*rbgrey > maxbyte))
	  c_errmsg("Right/bottom gray level is out of data type range",
		"testgen-rbgrey",&fatal);
	if (*nlevels > (maxbyte + 1))
	  c_errmsg("Too many gray levels specified",
		"testgen-grey",&fatal);
	break;
	case 2:
	if ((*ltgrey < minword) || (*ltgrey > maxword))
	  c_errmsg("Left/top gray level is out of data type range",
	  	"testgen-ltgrey",&fatal);
	if ((*rbgrey < minword) || (*ltgrey > maxword))
	  c_errmsg("Right/bottom gray level is out of data type range",
		"testgen-rbgrey",&fatal);
	if (*nlevels > 65536)
	  c_errmsg("Too many gray levels specified",
		"testgen-grey",&fatal);
	break;
	case 3:
	if ((*ltgrey < minlong) || (*ltgrey > maxlong))
	  c_errmsg("Left/top gray level is out of data type range",
	  	"testgen-ltgrey",&fatal);
	if ((*rbgrey < minlong) || (*ltgrey > maxlong))
	  c_errmsg("Right/bottom gray level is out of data type range",
		"testgen-rbgrey",&fatal);
	if (*nlevels > 65536)
	  c_errmsg("Too many gray levels specified",
		"testgen-grey",&fatal);
	break;
	case 4:
	if ((*ltgrey < minreal) || (*ltgrey > maxreal))
	  c_errmsg("Left/top gray level is out of data type range",
		"testgen-ltgrey",&fatal);
	if ((*rbgrey < minreal) || (*rbgrey > maxreal))
	  c_errmsg("Right/bottom gray level is out of data type range",
		"testgen-rbgrey",&fatal);
	if (*nlevels > 65536)
	  c_errmsg("Too many gray levels specified",
		"testgen-grey",&fatal);
	break;
	}
   }

/*
  get TAE globals 
*/

var = p_find(vblock, "$MSGTIM");
*msgtim = IVAL(*var, 0);

var = p_find(vblock, "$MINMAX");
strcpy(minmax, SVAL(*var,0));

var = p_find(vblock, "$DDRVFLG");
strcpy(dflag_str, SVAL(*var,0));
 
if (strcmp (dflag_str,"IN") == 0)
   *dflag = INVAL;
else
   *dflag = UNKNOW;
 
}
