/*********************************************************************
NAME: ENGRAVE
 
PURPOSE: Engrave writes tic symbols to the output image as specified
         by the three pixel value buffers--lines,samps,zlevel.  at every
         lines(count),samps(count) coordinate in the output image, the
         symbol brightness value in zlevel(count) is inserted.
 		
PROGRAM HISTORY:
VERSION	DATE       AUTHOR             REQUEST
-------	----       ------             -------
1.0     5/86       D. Steinwand       Prf# 2753,3459
2.0     5/90       W. Xia             Conversion to c
5.0     4/92       T. Mittan          1. Fixed bug to allow multiple bands
                                      2. allowed a grid to be drawn

ALGORITHM DESCRIPTION:

  if input image is null
    use calculated nlines & nsamps; image is byte
  else
    get the nl, ns, and data type from input image through filspec
  call inittm to initialize the processing message
  call startm to start the processing message
  initialize index to one
  if input image is null
    loop on curent = 1 to imgsiz[0]--the number of lines
       initialize the output buffer to bckgnd
       while on this line (curent = lines(index)...
           check valid range & if ok assign the grey value
           contained in zlevel(index) to the sample defined
             in samps(index).
           increment index
       end do
       call ewrite to write line to output image
     end loop
  else (there is an input image...)	
      loop on curent = 1 to imgsiz[0]--the number of lines
         call eread to read input image line
         initialize the output buffer to input buffer
         while on this line (curent = lines(index)...
            check valid range & if ok assign the grey value
            contained in zlevel(index) to the sample defined
              in samps(index).
            increment index
         end do
         call ewrite to write line to output image
       end loop         
  end if
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "ticmark.h"

int engrave
(
    long nullfi,                /* I: Null input file flag */
    long nlines,                /* I: Number of lines in mask image */
    long nsamps,                /* I: Number of samples in mask image */
    long bckgnd,                /* I: Background value for mask image */
    long (*image_band_num)[MAXBND+1], /* I: Band numbers selected for 
                                            each image */
    long num_image_bands[],     /* I: Number of bands in each input image */
    long *num_inimages,         /* I: Number of images */
    long num_inbands,           /* I: Number of bands */
    long lines[],               /* I: Line buffer of ticmarks */
    long samps[],               /* I: Sample buffer of ticmarks */
    long zlevel[],              /* I: Brightness level of ticmarks */
    float *outbuf,              /* O: Image output buffer */
    struct FDESC *fdescin[],    /* I: Input file descriptors */
    struct FDESC **fdescout,    /* I: Output file descriptors */
    long *msgtim,               /* I: Message interval */
    long (*window)[4],          /* I/O: Input image dimensions (SL SS NL NS) */
    long *grid_flg,             /* I: Grid flag */
    char tempfile[][CMLEN]      /* I: Temp file name */
)
{
    long    non_fatal = 1;  /* fatal error return status    */
    long    band, img;      /* loop counters                */
    long	bright;         /* ticmark value                */
    long	index;          /* ticmark counter              */ 
    long	cnt;            /* loop count                   */
    long    line;           /* line number                  */
    long	i;              /* loop counter                 */
    long	samp1;          /* startimg sample for grid	    */
    long	samp2;          /* ending sample for grid       */
    long    iline;          /* image line number            */ 
    long    start;          /* holding value for index      */
    long    work_size;      /* size of input/output buffer  */ 
    long    outband;        /* input/output buffer          */
    long    one = 1;        /* value of one	                */
    char	buffer[80];     /* buffer of lines, and samples	*/
    char    *bufptr;        /* pointer to the buffer        */
    FILE    *fp;            /* file pointer	                */

    /* initialize variables */
    curent = 1;
    curcat = 1;
    outband = 0;                   

    /* if the input image is null, use the number of lines and samples 
       calculated in ticloc.  the image window is set to the full number of
       lines and samples and the window flag is set to false.  a message is 
       set to the standard output informing the user of the number of lines 
       and samples in the output image.  */
    if (nullfi) 
    {
        window[0][SS] = 1;
        window[0][SL] = 1;
        window[0][NS] = nsamps;
        window[0][NL] = nlines;
    }


    /* Allocate dynamic memory for the input, output and temporary work buffers.
       ----------------------------------------------------------------------*/
    work_size = window[0][NS] * (long)sizeof(float);

    if ((outbuf = (float *) malloc (work_size)) == NULL )
    {
        c_errmsg ("Error allocating dynamic memory","ticmark-alloc",&non_fatal);
        for (img = 0; img < *num_inimages; img++)
            c_eclose(&fdescin[img]);
        return(E_FAIL);
    }

    /* initialize the tic-pixel buffer index                          
       ----------------------------------------*/                          
    index=0;          
    start = 0;

    /*adjust lines(index) to be greater than zero...
      --------------------------------------------------- */
    while(lines[index] < 1)
    {
        index = index + 1;
        start = index;
    }  /* do */

    /*initialize the standard processing message
      --------------------------------------------*/
    c_inittm ("TICMARK", "Processed", "line", "band", " ", 
            &window[0][NL], &num_inbands, msgtim, &one);
    c_startm();

    /* process with a null input image....
       first, the output buffer line is initialized to the background value.
       then, pixels to be engraved on this line are given the grey value in
       zlevel(index).  */

    if (nullfi)    
    {                        
        if (*grid_flg)
        {                                 
            fp = fopen(tempfile[1],"r");
            curent = 1;                        
            bufptr = buffer;
            fgets(buffer,80,fp);
            line  = atol(bufptr);
            bufptr += 11;
            samp1 = atol(bufptr);
            bufptr += 14;
            samp2 = atol(bufptr);
            bufptr += 12;
            bright = atol(bufptr);
            for (iline=1; iline <= window[0][NL]; iline++)
            {                        
                for (cnt =0 ; cnt < window[0][NS]; cnt++)   
                    outbuf[cnt] = (float) bckgnd;       
                curent++;
                while (iline == line)
                {
                    if (samp1 >= window[0][NS])
                    {
                        samp1 = 1;
                        samp2 = 0;
                    }
                    else
                        if (samp1 < 0)
                            samp1 = 0;
                        else
                            if (samp2 >= window[0][NS])
                                samp2 = window[0][NS] -1; 
                    for (i = samp1; i <= samp2; i++)
                        outbuf[i] = (float)bright;
                    bufptr = buffer;
                    if (fgets(buffer,255,fp) == NULL)
                    {
                        line = 0;
                        break;
                    }
                    else
                    { 
                        line  = atol(bufptr);
                        bufptr += 11;
                        samp1 = atol(bufptr);
                        bufptr += 14;
                        samp2 = atol(bufptr);
                        bufptr += 12;
                        bright = atol(bufptr);
                    }
                }
                if ((c_ewrite (fdescout, &one, &iline, (void *)outbuf, &one))
                        == E_FAIL)
                {
                    c_eclose(&fdescin[img]);
                    c_eclose(fdescout);
                    return(E_FAIL);
                }
            }
            curcat++;
            rewind(fp);
            fclose(fp);
            c_delfil(tempfile[1]);
        }       /* end of if */ 
        else
            for (iline=1; iline <= window[0][NL]; iline++, curent++)
            {                           
                for (cnt =0 ; cnt < window[0][NS]; cnt++)   
                    outbuf[cnt] = (float) bckgnd;       
                while (iline == lines[index])
                {
                    if ((lines[index] > 0) && (lines[index] <= window[0][NL])&& 
                            (samps[index] > 0)&&(samps[index] <= window[0][NS]))
                        outbuf[samps[index]-1] = (float) zlevel[index];
                    ++index;       
                }  /* do */
                if ((c_ewrite (fdescout, &one, &iline, (void *)outbuf, &one))
                        == E_FAIL)
                {
                    c_eclose(&fdescin[img]);
                    c_eclose(fdescout);
                    return(E_FAIL);
                }
            }             /* iline loop */

        /*process with an input image...(similar to above...)
          ----------------------------------------------------- */
    }                  /* null true */
    else
        if (*grid_flg)
        {                                 
            fp = fopen(tempfile[1],"r");
            for (img = 0; img < *num_inimages; img++)
            {
                for (band = 0; band < num_image_bands[img]; band++)
                {
                    curent = 1;                        
                    outband++;
                    bufptr = buffer;
                    fgets(buffer,80,fp);
                    line  = atol(bufptr);
                    bufptr += 11;
                    samp1 = atol(bufptr);
                    bufptr += 14;
                    samp2 = atol(bufptr);
                    bufptr += 12;
                    bright = atol(bufptr);
                    for (iline=1; iline <= window[0][NL]; iline++)
                    {                        
                        curent++;
                        if ((c_eread (&fdescin[img], &image_band_num[img][band],
                                &iline, (void *)outbuf, &one)) == E_FAIL)
                        {
                            c_eclose(&fdescin[img]);
                            c_eclose(fdescout);
                            return(E_FAIL);
                        }
                        while (iline == line)
                        {
                            if (samp1 >= window[0][NS])
                            {
                                samp1 = 1;
                                samp2 = 0;
                            }
                            else
                                if (samp1 < 0)
                                    samp1 = 0;
                                else
                                    if (samp2 >= window[0][NS])
                                        samp2 = window[0][NS] -1; 
                            for (i = samp1; i <= samp2; i++)
                                outbuf[i] = (float)bright;
                            bufptr = buffer;
                            if (fgets(buffer,255,fp) == NULL)
                            {
                                line = 0;
                                break;
                            }
                            else
                            { 
                                line  = atol(bufptr);
                                bufptr += 11;
                                samp1 = atol(bufptr);
                                bufptr += 14;
                                samp2 = atol(bufptr);
                                bufptr += 12;
                                bright = atol(bufptr);
                            }
                        }
                        if ((c_ewrite(fdescout, &outband, &iline, 
                                (void *)outbuf, &one)) == E_FAIL)
                        {
                            c_eclose(&fdescin[img]);
                            c_eclose(fdescout);
                            return(E_FAIL);
                        }
                    }
                    curcat++;
                    rewind(fp);
                }      /* end of loop for each band */
            }        /* end of loop for img  */ 
            fclose(fp);
            c_delfil(tempfile[1]);
        }       /* end of if */ 
        else
        {                                 
            for (img = 0; img < *num_inimages; img++)
            {
                for (band = 0; band < num_image_bands[img]; band++)
                {
                    curent = 1;                        
                    outband++;
                    index = start;
                    for (iline=1; iline <= window[0][NL]; iline++)
                    {                        
                        curent++;
                        if ((c_eread (&fdescin[img], &image_band_num[img][band],
                                &iline, (void *)outbuf, &one)) == E_FAIL)
                        {
                            c_eclose(&fdescin[img]);
                            c_eclose(fdescout);
                            return(E_FAIL);
                        }
                        while (iline == lines[index])
                        {                        
                            if ((lines[index] <window[0][NL]) 
                                && (samps[index] > 0) 
                                && (samps[index] <window[0][NS]))  
                                outbuf[samps[index]-1] = (float)zlevel[index];       
                            index = index + 1;
                        }  /* end of while loop */               
                        if ((c_ewrite(fdescout, &outband, &iline, 
                                (void *)outbuf, &one)) == E_FAIL)
                        {
                            c_eclose(&fdescin[img]);
                            c_eclose(fdescout);
                            return(E_FAIL);
                        }
                    }
                    curcat++;
                }      /* end of loop for each band */
            }        /* end of loop for img  */ 
        }
    free (outbuf);
    return (E_SUCC);
}
