/****************************************************************************
NAME: GET_PROJ_PRM

PURPOSE: Retrieves the projection parameters from the projection
         parameter file

PROGRAM HISTORY:
VERSION	 DATE   AUTHOR         REASON
-------	 ----	------         -----------------------------
5.0    7/89   D. Steinwand     LAS 5.0 Original development
5.1    4/92   T. Mittan        Took form REMAP and added to TICMARK to allow
                               a projection file to be used instead of a
                               INGCTP file.

ALGORITHM DESCRIPTION:
     Open the label table file containing the projection parameters
     Get the label
     Determine the position of the fields within the file
     Scan the file for the projection parameters specified by the input keys
     Close the table
     Return
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "ltable.h"
#include "ticmark.h"

int get_proj_prm
(
    char *proj_name,        /* I: Projection parameter file name */
    char *inkey,            /* I: Input projection parameter key */
    long *in_def,           /* I: Flag for input image */
    struct PROJ *inp        /* O: Input projection parameters */
)

{
    struct TAB_DEF tab; 	/* Tabular structure of label table */
    struct VECTOR *keyptr;  /* Pointer to key field */
    struct VECTOR *codeptr;	/* Pointer to projection code field */
    struct VECTOR *zoneptr;	/* Pointer to zone code field */
    struct VECTOR *unitptr; /* Pointer to unit of measure field	*/
    struct VECTOR *sphptr;  /* Pointer to datum (spheroid) code field */
    struct VECTOR *parmptr; /* Pointer to projection coefficients field */
    long inproj_found;      /* Boolean--input projection found? */
    long status;            /* Function return status */
    long nonfatal= 1;       /* nonfatal flag */
    long i;                 /* Loop counter */

    /* Open the labeled table file & retrieve the label
       ------------------------------------------------*/
    open_tab(&tab, proj_name, IREAD);
    get_vector(&tab);

    /* Determine pointers to fields in the file
       ----------------------------------------*/
    keyptr = get_field_ptr(&tab, "PROJKEY");
    if (keyptr == NULL) return(E_FAIL);
    codeptr = get_field_ptr(&tab, "PROJTYPE");
    if (codeptr == NULL) return(E_FAIL);
    zoneptr = get_field_ptr(&tab, "PROJZONE");
    if (zoneptr == NULL) return(E_FAIL);
    unitptr = get_field_ptr(&tab, "PROJUNITS");
    if (unitptr == NULL) return(E_FAIL);
    sphptr = get_field_ptr(&tab, "PROJSPH");
    if (sphptr == NULL) return(E_FAIL);
    parmptr = get_field_ptr(&tab, "PROJPARMS");
    if (parmptr == NULL) return(E_FAIL);

    /* Initialize BOOLEAN variables
       ----------------------------*/
    inproj_found = FALSE; 

    /* Loop until key is found or EOF
       -------------------------------*/
    for (;;)
    {
        status = get_record(&tab);
        if (status == E_FAIL) return(E_FAIL);
        if (status == E_EOF) break;
        if((strcmp(keyptr->ptr.c, inkey) == 0) || (*in_def))
        {
            inp->proj_code = *codeptr->ptr.i4;
            inp->zone_code = *zoneptr->ptr.i4;
            inp->units = *unitptr->ptr.i4;
            inp->datum_code = *sphptr->ptr.i4;
            for (i = 0; i < 15; i++)
                inp->proj_coef[i] = parmptr->ptr.r8[i];
            inproj_found = TRUE;
        }
        if (inproj_found ) break;
    }

    /* Close the file & return
       -----------------------*/
    close_tab(&tab);
    if ((!*in_def && !inproj_found))
    {
        c_errmsg("Error finding OPROJKEY","ticmark-getparms",&nonfatal);
        return(E_FAIL);
    }
    return(E_SUCC);
}
