/******************************************************************************
NAME: PICPAK

PURPOSE: Calculate symbol coordinates

        PICPAK packs three arrays with symbol information.  The three arrays
        are:  1.  Line coordinates,
              2.  Sample coordinates,
              3.  Symbol grey-level values.
        Each element of these three arrays combine to describe a pixel in the
        output image:  The pixel in the output image at line_buf[1],
        sample_buf[1] will be assigned the brightness value contained in
        zlevel_buf[1].  Center Tics and Upper Left, Lower Left, Upper Right,
        and Lower Right Corner Tics are the possible symbols.

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ticmark.h"

void picpak
(
    long *center_line,     /* I: Center line coordinate of symbol */
    long *center_sample,   /* I: Center sample coordinate of symbol */
    long *symbol_id,       /* I: Symbol type */
    long *bright,          /* I: Grey-level of symbol in output image */ 
    long symdim[],         /* I: Height, Length, and Thickness of each symbol */
    long *line_buf,        /* O: Buffer of symbol line coordinates */
    long *sample_buf,      /* O: Buffer of symbol sample coordinates */
    long *zlevel_buf,      /* O: Buffer of symbol grey-level values */
    long *index,           /* I/O: Number of pixels in symbols--buffer index */
    long grid_line[],      /* O: Grid line index */
    long grid_samp[]       /* O: Grid samp index */
)
{
    long    startln;        /* symbol starting line                         */
    long    startsm;        /* symbol starting sample                       */
    long    offset;         /* symbol thickness offset                      */
    long    line;           /* symbol center line after thickness
                               offset applied                               */
    long    sample;         /* symbol center samp after thickness 
                               offset applied                               */
    long    cnt,i;          /* loop counters                                */
    long    curlin;         /* current line counter                         */
    long    cursam;         /* current sample counter                       */

    /* Adjust buffer addresses...
       ------------------------ */

    line_buf = &(line_buf[*index]);
    sample_buf = &(sample_buf[*index]);
    zlevel_buf = &(zlevel_buf[*index]);

    switch(*symbol_id)
    {
        /**********

          Pack Center Tics.  First, find the starting line and starting sample
          coordinates.  Note that these are the lower right coordinates and
          processing proceeds in the negative direction--towards the upper
          left corner of the tic.  Next, an offset for tic thickness is
          calculated and applied to line and sample; the center point after
          the tic thickness is calculated.

          Tic pixel coordinates are then calculated in the line direction and
          then in the sample direction.  If the tic thickness is something other
          than one, some pixel coordinates are repeated.  This is not an error--
          checking for the condition would be more costly than appling those
          few points twice.

          Note the location of the center of the tic in the following examples.
          Also, if the tic thickness is an even number, lines are thickened to
          the left of the center line coordinate and below the center sample 
          coordinate:

      (5,5,1)         (5,5,2)         (5,5,3)         (5,5,4)         (5,5,5)

         *               **             ***             ****           *****
         *               **            *****           *****           *****  
       **C**           **C**           **C**           **C**           **C**
         *             *****           *****           *****           *****
         *               **             ***            *****           *****  

         *********/
        case TIC:
        startln = (long)(symdim[0]/2) + *center_line;
        startsm = (long)(symdim[1]/2) + *center_sample;
        offset = (long)(((float)(symdim[2])/2) + 0.5) - 1;
        line = *center_line - offset;
        sample = *center_sample - offset;

        for(cnt = 0; cnt < symdim[2]; cnt++, sample++)
        {
            for(i = 0, curlin = startln; i < symdim[0]; i++, curlin--)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
                grid_samp[cnt] = sample;
            }
        }
        for(cnt = 0; cnt < symdim[2]; cnt++, line++)
        { 
            for(i = 0, cursam = startsm; i < symdim[1]; i++, cursam--)
            {
                *line_buf++ = line;
                *sample_buf++ = cursam;
                *zlevel_buf++ = *bright;
                (*index)++;
                grid_line[cnt] = line;
            }       
        }
        break;


        /**********

        Pack Upper Left Tics.  Processing starts on the "Center" pixel and
        continues outward until the tic is complete.  The "Center" is the
        coordinate point of reference of the tic.

        Tic pixel coordinates are then calculated.  If the tic thickness is 
        something other than one, some pixel coordinates are repeated.  This 
        is not an error--checking for the condition would be more costly than 
        appling those few points twice.

        Note the location of the center of the tic in the following examples.
        Also, note how thickness of the tic is handled.

        (5,5,1)         (5,5,2)         (5,5,3)         (5,5,4)         (5,5,5)

         C****           C****           C****           C****           C****
         *               *****           *****           *****           *****
         *               **              *****           *****           *****
         *               **              ***             *****           *****
         *               **              ***             ****            *****
         *********/
        case ULT:
        sample = *center_sample;
        for (cnt = 0; cnt < symdim[2]; cnt++, sample++)
        {   
            for (i = 0, curlin = *center_line; i < symdim[0]; i++,curlin++)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_samp[cnt] = sample;
        }
        for (cnt = 0; cnt < (symdim[1] - symdim[2]); sample++, cnt++)
        {
           for(i = 0, curlin = *center_line; i < symdim[2]; i++, curlin++)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_line[cnt] = *center_line + cnt;
        }
        break;

        /**********

          Pack Upper Right Tics.  Processing starts on the "Center" pixel and
          continues outward until the tic is complete.  The "Center" is the
          coordinate point of reference of the tic.

          Tic pixel coordinates are then calculated.  If the tic thickness is 
          something other than one, some pixel coordinates are repeated.  This 
          is not an error--checking for the condition would be more costly than 
          appling those few points twice.

          Note the location of the center of the tic in the following examples.
          Also, note how thickness of the tic is handled.

         (5,5,1)         (5,5,2)         (5,5,3)         (5,5,4)         (5,5,5)

         ****C           ****C           ****C           ****C           ****C
             *           *****           *****           *****           *****
             *              **           *****           *****           *****
             *              **             ***           *****           *****
             *              **             ***            ****           *****

        *********/
        case URT:
        sample = *center_sample;
        for (cnt = 0; cnt < symdim[2]; cnt++, sample--)
        {
            for (i = 0, curlin = *center_line; i < symdim[0]; i++,curlin++)
            {
               *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_samp[cnt] = sample;
        }
        for (cnt = 0; cnt < (symdim[1] - symdim[2]); sample--, cnt++)
        {
            for(i = 0, curlin = *center_line; i < symdim[2]; i++, curlin++)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_line[cnt] = *center_line + cnt;
        }
        break;

        /**********

          Pack Lower Left Tics.  Processing starts on the "Center" pixel and
          continues outward until the tic is complete.  The "Center" is the
          coordinate point of reference of the tic.

          Tic pixel coordinates are then calculated.  If the tic thickness is 
          something other than one, some pixel coordinates are repeated.  This 
          is not an error--checking for the condition would be more costly than 
          appling those few points twice.

          Note the location of the center of the tic in the following examples.
          Also, note how thickness of the tic is handled.

         (5,5,1)         (5,5,2)         (5,5,3)         (5,5,4)         (5,5,5)

         *               **              ***             ****            *****
         *               **              ***             *****           *****
         *               **              *****           *****           *****
         *               *****           *****           *****           *****
         C****           C****           C****           C****           C****
        *********/
        case LLT:
        sample = *center_sample;
        for (cnt = 0; cnt < symdim[2]; cnt++, sample++)
        {
        for (i = 0, curlin = *center_line; i < symdim[0]; i++,curlin--)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_samp[cnt] = sample;
        }
        for (cnt = 0; cnt < (symdim[1] - symdim[2]); sample++, cnt++)
        {
            for(i = 0, curlin = *center_line; i < symdim[2]; i++, curlin--)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_line[cnt] = *center_line + cnt;
        }
        break;

        /**********

          Pack Lower Right Tics.  Processing starts on the "Center" pixel and
          continues outward until the tic is complete.  The "Center" is the
          coordinate point of reference of the tic.

          Tic pixel coordinates are then calculated.  If the tic thickness is 
          something other than one, some pixel coordinates are repeated.  This 
          is not an error--checking for the condition would be more costly than 
          appling those few points twice.

          Note the location of the center of the tic in the following examples.
          Also, note how thickness of the tic is handled.

         (5,5,1)         (5,5,2)         (5,5,3)         (5,5,4)         (5,5,5)

              *              **             ***            ****           *****
              *              **             ***           *****           *****
              *              **           *****           *****           *****
              *           *****           *****           *****           *****
          ****C           ****C           ****C           ****C           ****C

        *********/
        case LRT:
        sample = *center_sample;
        for (cnt = 0; cnt < symdim[2]; cnt++, sample--)
        {
            for (i = 0, curlin = *center_line; i < symdim[0]; i++,curlin--)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_samp[cnt] = sample;
        }
        for (cnt = 0; cnt < (symdim[1] - symdim[2]); sample--, cnt++)
        {
            for(i = 0, curlin = *center_line; i < symdim[2]; i++, curlin--)
            {
                *line_buf++ = curlin;
                *sample_buf++ = sample;
                *zlevel_buf++ = *bright;
                (*index)++;
            }
            grid_line[cnt] = *center_line + cnt;
        }
        break;
    }
}
