/*****************************************************************************
NAME: Ticloc
 
PURPOSE: Ticloc calculates line/sample tic locations in output image.
 

PROGRAM HISTORY:
VERSION DATE       AUTHOR              REASON
------- ----       ------              ------
1.0     5/86       D. Steinwand        Prf# 2753,3459
2.0     8/86       D. Steinwand        Prf/adp# 3604
3.0     5/90       W. Xia              Conversion to c
4.0     4/92       T. Mittan           Use a projection file instead of a
                                       GCTP file.
                                       Allow a grid to be formed
5.0     9/92       T. Mittan           Allowed only input image ddr to be
                                       used; so a projection file does not
                                       need to be used if input image is
                                       specified.
5.1     2/93       T. Mittan           Fixed to allow images of GEO
                                       projection code to work correctly.
5.2     3/99       T. Ratliff          Allowed a report to be printed
                                       to terminal,file,or printer
                                       on the location of ticmarks.
******************************************************************************/ 
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "cproj.h"
#include "geompak.h"
#include "ticmark.h"

int ticloc
(
    struct PARBLK *vblk,  /* I: TAE parameter block */
    char *infile,         /* I: File containing user defined symbol locations */
    char mode[],          /* I: Subcommand option */
    double freq,          /* I/O: Frequency of tics--same as lat/long fields */
    double start[2],      /* I/O: Coordinate of first tic */
    long symdim[3],       /* I/O: Symbol/tic dimensions (h,l,w) */
    long bright,          /* I: Symbol brightness value */
    char coform[],        /* I: Format of lat/long coordinate fields */
    long ornull,          /* I: Origin for uniform tics given was null */
    long *lmax,           /* O: Number of lines for null input image */
    long *smax,           /* O: Number of samples for null input image */
    long lines[],         /* I: Lines array for ticmarks */
    long samps[],         /* I: Samps array for ticmarks */
    long zlevel[],        /* I: Intensity array for ticmarks */
    struct DDR *ddr,      /* O: DDR structure */
    char (*hosin)[CMLEN], /* I: Input image host name */
    long *in_def,         /* I: Flag for input image */
    long grid_line[],     /* I: Grid line number array */
    long grid_samp[],     /* I: Grid sample number array */
    long *grid_width,     /* O: Width of grid */
    long usr_line[],      /* I: User specified line number array */
    long *usr_samp,       /* I: User specified sample number array */
    long usr_width[],     /* O: User specified width  */
    long usr_intensity[], /* O: User specified width */
    FILE *fp,             /* I: Pointer to the report file */
    long print[]          /* I: Print array telling where to print report */
)
{
    struct PROJ inp;       /* Input projection parameters                   */
    struct PROJ oup;       /* Input projection parameters                   */
    struct VARIABLE *var;   /* pointer to a TAE variable in vblk            */

    double  ddr_coef[15];   /* DDR prjection coef                           */
    double  incoor[2];      /* input coordinates for gctp                   */
    double  usrcoor[2];     /* user coordinates from infile                 */
    double  icorners[8];    /* Projection coordinates--image corners        */
    double  lorite[2];      /* coordinates of lower right input corner      */  
    double  otcoor[2];      /* output coordinates                           */  
    double  pymin, pymax;   /* minimum & maximum y projection coordinate    */ 
    double  pxmin, pxmax;   /* minimum & maximum x projection coordinate    */
    double  real_wind[4];   /* Window spec--in double                       */
    double  scalex;         /* output pixel size in units defined by measo  */
    double  scaley;         /* output pixel size in units defined by measo  */
    double tmpscalex;
    double tmpscaley;
    double  upleft[2];      /* coordinates of upper left input corner       */
    double  wind_proj[8];   /* Projection coordinates of image window       */

    long    acc;            /* type of acces to file                        */
    long    badtmark;       /* Number of ticmarks outside of image          */
    long    cline;          /* calculated line coordinate                   */
    long    cnt;            /* loop counter                                 */
    long    cnt2;           /* loop counter                                 */  
    long    count=0;        /* tic counter                                  */
    long    csamp;          /* calculated sample coordinate                 */
    long    non_fatal = 1;  /* fatal error return status                    */
    long    four = 4;       /* degrees unit code                            */
    long    geo = 0;        /* geometric system code                        */
    long    i;              /* loop counter                                 */
    long    imc[8];         /* Image coordinates of image corners           */
    long    in_wind[MAXIMG][4]; /* Input image window specification         */
    long    j;              /* loop counter                                 */
    long    nltic;          /* number of tics in line direction             */
    long    nrecs;          /* number of records in user-defined file       */
    long    nstic;          /* number of tics in sample direction           */
    long    one = 1;        /* value of one                                 */
    long    pixflg=FALSE;   /* flag indicating where pixsiz is to come from */
    long    pmode;          /* flag to indicate where coors input from      */
    long    prtprm[2];      /* Print/report flag for projection package     */
    long    status;         /* return status of various functions           */
    long    symbol;         /* symbol/tic id                                */
    long    tmark;          /* Tic mark counter                             */
    long    two = 2;        /* meters unit code                             */
    long    zero = 0;       /* value of zero                                */
    char    inkey[10];      /* Input projection key                         */
    char    temp_coform[4]; /* temporary conversion form                    */
    char    proj_name[CMLEN];     /* Projection parameter file name         */
    FILE *infp;


    /* get the projection key,projection file, and
       the pixel size
       ----------------------------------------------*/
    if (!*in_def)
    {
        status = c_getcnt(vblk,"PIXSIZ",&count);
        if ((status != E_SUCC) && (count != 2))
        {
            c_errmsg("PIXSIZ must be specified","ticmark-specify",&non_fatal);
            return(E_FAIL);
        }
        else
        {
            var = p_find(vblk, "PIXSIZ");
            scalex = RVAL(*var, 1);
            scaley = RVAL(*var, 0);
            pixflg = TRUE;
        }

        var = p_find(vblk, "OPROJKEY");
        if ((*var).v_count != 0)
            strcpy(inkey, SVAL(*var, 0));
        else
        {
            c_errmsg("OPROJKEY must be specified","ticmark-specify",&non_fatal);
            return(E_FAIL);
        }
        acc = IREAD;
        if (c_getnam(vblk, "INPROJ", &zero, proj_name, "proj", &acc) != E_SUCC)
            return(E_FAIL);
        if (strlen(proj_name) <= 0)
        {
            c_errmsg("INPROJ must be specified","ticmark-specify",&non_fatal);
            return(E_FAIL);
        }
        if(get_proj_prm(proj_name, inkey, in_def, &inp) != E_SUCC)
            return(E_FAIL);
    }
    else
    {
        if(use_ddr_info(vblk, &inp, icorners, imc, in_wind,&scaley,&scalex,
                &pixflg) != E_SUCC)
            return(E_FAIL);
    }

    var = p_find(vblk, "COORS");
    if ((*var).v_count == 0)

        /* Framing info is obtained from the input image(and window, if applied)
           ------------------------------------------------------------------*/
    {
        if (!*in_def)
        {
            c_errmsg("Coordinates must be specified","ticmark-coord", &one);
            return(E_FAIL);
        }
        for (i = 4; i--; real_wind[i] = in_wind[0][i]);
        if (c_rotcor(icorners, imc, real_wind, wind_proj) != E_SUCC) 
            return(E_FAIL);
        pxmin = GMIN(wind_proj[1], wind_proj[3]);
        pxmin = GMIN(pxmin, wind_proj[5]);
        pxmin = GMIN(pxmin, wind_proj[7]);
        pxmax = GMAX(wind_proj[1], wind_proj[3]);
        pxmax = GMAX(pxmax, wind_proj[5]);
        pxmax = GMAX(pxmax, wind_proj[7]);
        pymin = GMIN(wind_proj[0], wind_proj[2]);
        pymin = GMIN(pymin, wind_proj[4]);
        pymin = GMIN(pymin, wind_proj[6]);
        pymax = GMAX(wind_proj[0], wind_proj[2]);
        pymax = GMAX(pymax, wind_proj[4]);
        pymax = GMAX(pymax, wind_proj[6]);
        upleft[0] = pymax;
        upleft[1] = pxmin;
        lorite[0] = pymin;
        lorite[1] = pxmax;
        pmode = FALSE;
    }
    else
        /* Framing is determined from user input
           -------------------------------------*/
    {
        sscanf(SVAL(*var,0), "%lf", &(upleft[0]));
        sscanf(SVAL(*var,1), "%lf", &(upleft[1]));
        sscanf(SVAL(*var,2), "%lf", &(lorite[0]));
        sscanf(SVAL(*var,3), "%lf", &(lorite[1]));
        if (strcmp(coform, "PRO") == 0) 
        {
            pymax = upleft[0];
            pymin = lorite[0];
            pxmax = lorite[1];
            pxmin = upleft[1];
        }
        pmode = TRUE;
    }

    /* initialize variables
       ---------------------*/
    if ((inp.units == one) || (inp.units == two))
        oup.units = inp.units;
    else
        if (strcmp(coform,"PRO") == 0)
            oup.units = two;
    oup.zone_code = inp.zone_code;
    oup.datum_code = inp.datum_code;
    for (j = 0; j < 15; j++)
    {
        oup.proj_coef[j] = inp.proj_coef[j];
        ddr_coef[j] = inp.proj_coef[j];
    }
    prtprm[GEO_TERM] = FALSE;
    prtprm[GEO_FILE] = FALSE;

    /* if input coordinates are in meters find the lat/long values
       -------------------------------------------------------------*/
    if ((((inp.units == two) || (inp.units == one)) && (pmode == FALSE)) || 
            ((strcmp(coform,"PRO") == 0) && (pmode == TRUE)))
    {
        status = c_transinit(&inp.proj_code,&oup.units,&inp.zone_code,
                &inp.datum_code, inp.proj_coef,&geo,&four,&oup.zone_code,
                &oup.datum_code,oup.proj_coef, prtprm,"\0"); 
        if (status == E_FAIL)
            return(E_FAIL); 

        /*calculate the upper left and lower right lat/longs.
          ---------------------------------------------------*/
        /* calculate upleft from projection coordinates
           -------------------------------------------- */
        status = c_trans(&inp.proj_code,&oup.units,
                &geo,&four,&upleft[1],
                &upleft[0],&incoor[1],&incoor[0]);
        if (status != E_SUCC)
            return(E_FAIL); 
        upleft[1] = incoor[1];
        upleft[0] = incoor[0];

        /* calculate loright from projection coordinates
           --------------------------------------------------*/
        status = c_trans(&inp.proj_code,&oup.units,
                &geo,&four,&lorite[1],&lorite[0],
                &incoor[1],&incoor[0]);
        if (status != E_SUCC)
            return(E_FAIL); 
        lorite[1] = incoor[1];
        lorite[0] = incoor[0];
    }
    else
        /* Cordinates are in degrees -- find projection coordinates in meters
           -------------------------------------------------------------------*/
    {
        /* convert SEC, MIN, DMS to DEG
           -----------------------------*/
        if (pmode == TRUE)
        {
            c_decdeg(&upleft[0],coform, "LAT");
            c_decdeg(&upleft[1],coform, "LON");
            c_decdeg(&lorite[0],coform, "LAT");
            c_decdeg(&lorite[1],coform, "LON");
        }
        else
            if (inp.units == 0)
            {
                upleft[0] = 180 * upleft[0] / 3.14159265358;
                upleft[1] = 180 * upleft[1] / 3.14159265358;
                lorite[0] = 180 * lorite[0] / 3.14159265358;
                lorite[1] = 180 * lorite[1] / 3.14159265358;
                /*
                   if (pixflg)
                   {
                   scaley = 180 * scaley / 3.14159265358;
                   scalex = 180 * scalex / 3.14159265358;
                   scaley = (pymax - pymin)/((upleft[0] - lorite[0])/scaley);
                   scalex = (pxmax - pxmin)/((lorite[1] - upleft[1])/scalex);
                   }
                 */
            }
            else
                if (inp.units == 3)
                {
                    strcpy(temp_coform,"SEC");
                    c_decdeg(&upleft[0],temp_coform, "LAT");
                    c_decdeg(&upleft[1],temp_coform, "LON");
                    c_decdeg(&lorite[0],temp_coform, "LAT");
                    c_decdeg(&lorite[1],temp_coform, "LON");
                    /*
                       if (pixflg)
                       {
                       c_decdeg(&scaley,temp_coform, "LAT");
                       c_decdeg(&scalex,temp_coform, "LAT");
                       scaley=(pymax - pymin)/((upleft[0] - lorite[0])/scaley);
                       scalex=(pxmax - pxmin)/((lorite[1] - upleft[1])/scalex);
                       }
                     */
                }
                else
                    if (inp.units == 5)
                    {
                        strcpy(temp_coform,"DMS");
                        c_decdeg(&upleft[0],temp_coform, "LAT");
                        c_decdeg(&upleft[1],temp_coform, "LON");
                        c_decdeg(&lorite[0],temp_coform, "LAT");
                        c_decdeg(&lorite[1],temp_coform, "LON");
                        /*
                           if (pixflg)
                           {
                           c_decdeg(&scaley,temp_coform, "LAT");
                           c_decdeg(&scalex,temp_coform, "LAT");
                           scaley = (pymax - pymin)/
                                    ((upleft[0] - lorite[0])/scaley);
                           scalex = (pxmax - pxmin)/
                                    ((lorite[1] - upleft[1])/scalex);
                           }
                         */
                    }
        /* convert DEG to meters
           ----------------------*/
        if (inp.proj_code != 0)
        {
            if (hosin[0][0] == '\0')
            {
                status = c_mapedg(&upleft[1],&lorite[1],&upleft[0],&lorite[0],
                        &four,&inp.proj_code,&inp.zone_code,oup.proj_coef,
                        &two,&pxmin,&pxmax,&pymin,&pymax,&oup.datum_code);
                if (status == E_FAIL)
                    return(E_FAIL);
            }
            else
            {
                if(c_transinit(&geo,&four,&inp.zone_code,&inp.datum_code,
                        inp.proj_coef,&inp.proj_code,&two,&oup.zone_code,
                        &oup.datum_code,oup.proj_coef,prtprm,"\0") != E_SUCC) 
                    return(E_FAIL);

                status = c_trans(&geo,&four,&inp.proj_code,&two,
                        &upleft[1],&upleft[0],&pxmin, &pymax);
                if (status != E_SUCC)
                    return(E_FAIL); 
                status = c_trans(&geo,&four,&inp.proj_code,&two,
                        &lorite[1],&lorite[0],&pxmax, &pymin);
                if (status != E_SUCC)
                    return(E_FAIL); 
            }
        }
    }
    /* adjust the pixel size if it is not meters
       ------------------------------------------*/
    if ((strcmp(coform,"PRO") != 0) && (pixflg))
    {
        c_decdeg (&scaley, coform, "LAT");
        c_decdeg (&scalex, coform, "LON");
        /*
           scaley = (pymax - pymin)/((upleft[0] - lorite[0])/scaley);
           scalex = (pxmax - pxmin)/((lorite[1] - upleft[1])/scalex);
         */
    }

    /* find the number of lines and samples in the image.
       ---------------------------------------------------*/
    if (inp.proj_code != 0)
    {
        *lmax = (long) ((pymax - pymin)/scaley + 1.5);
        *smax = (long) ((pxmax - pxmin)/scalex + 1.5);
    }
    else
    {
        *lmax = (long) (abs(upleft[0] -lorite[0])/scaley +1.5);
        *smax = (long) (abs(-lorite[1] +upleft[1])/scalex +1.5);
    }

    badtmark = 0;
    tmark = 0;

    /* calculate uniform tics at a specified lat/long frequency:
       these coordinates are then converted to line/sample locations.
       ------------------------------------------------------------------ */
    if ((strcmp(mode, "UNIFORM") == 0)  ||  (strcmp(mode, "BOTH") ==0 )) 
        /*
           convert 'freq' and 'start' to decimal degrees (if needed).  also
           initialize 'start' to the lat/long location of the first tic.  if 
           a parameter value was not entered by the user, use the first even
           increment of 'freq' from the upper left corner as the 'start'ing 
           point.  from the starting tic lat/long, calculate how many tics 
           will be placed in each row and place in 'nstic'.  also calculate 
           the number of tic rows and place in 'nltic'.
           --------------------------------------------------------------- */
    {
        if (strcmp(coform ,"PRO") == 0)
        { 
            /* call c_decdeg to check the valid range for freq.  
               it should already be in total degrees.  
               ------------------------------------------------------------ */
            c_decdeg (&freq, "DEG", "LON");
            if (!ornull)
            {   
                if(c_transinit(&inp.proj_code,&two,&inp.zone_code,
                        &inp.datum_code,inp.proj_coef,&geo,&four,
                        &oup.zone_code,&oup.datum_code,
                        oup.proj_coef,prtprm,"\0") != E_SUCC) 
                    return(E_FAIL);
                incoor[1] = start[1];            
                incoor[0] = start[0];
                status = c_trans(&inp.proj_code,&two,&geo,&four,&incoor[1],
                        &incoor[0],&otcoor[1],&otcoor[0]);
                if (status != E_SUCC)
                    return(E_FAIL); 
                start[1] = otcoor[1];
                start[0] = otcoor[0];
            }  
        }
        else
        {
            c_decdeg (&freq, coform, "LON");
            if (!ornull) 
            {
                c_decdeg (&start[0], coform, "LAT");
                c_decdeg (&start[1], coform, "LON");
            }  
        }  

        /* calculate default origin (starting point)
           ---------------------------------------------- */
        if (ornull) 
        {
            if ((long)(upleft[0]/freq) == (upleft[0])/freq) 
                start[0] = upleft[0] - freq;         
            else
            {
                if (upleft[0] > 0)  
                    start[0] = ((long) (upleft[0]/freq) * freq);
                else 
                    start[0] = (long) (upleft[0]/freq) * freq;
            }   

            if (start[0] > 180.0) start[0] = start[0] - 360;

            if ((long) (upleft[1]/freq) == (upleft[1]/freq)) 
                start[1] = upleft[1] + freq;          
            else
            {      
                if (upleft[1] > 0) 
                    start[1] = (long)(upleft[1]/freq) * freq;
                else  
                    start[1] = ((long)(upleft[1]/freq) * freq);
            }          
        }                                               
        /*                                                    
        find the number of tics in the line direction and in the sample 
        direction
        -------------------------------------------------------------- */
    #ifdef gould
        nstic = (long)(fabs(start[1]-lorite[1])/freq + 1.000000001);
        nltic = (long)(fabs(start[0]-lorite[0])/freq + 1.000000001);
    #else
        nstic = (long)(fabs(start[1]-lorite[1])/freq) + 1;
        nltic = (long)(fabs(start[0]-lorite[0])/freq) + 1;
    #endif
        /* calculate tic center line/sample coordinates
           --------------------------------------------- */
        if ((nltic > NLTIC) || (nstic > NSTIC))
        {
            c_errmsg ("Symbol buffer full", "ticmark-ticloc", &non_fatal); 
            return(E_FAIL);
        }
        count = 0;
        symbol = 1;
        if (inp.proj_code == SOM)
            incoor[1] = start[1];  
        else
            incoor[0] = start[0];                   
        for (cnt = 1; cnt <= nltic; cnt++)     
        {
            if (inp.proj_code == SOM)
                incoor[0] = start[0];  
            else
                incoor[1] = start[1];  
            for (cnt2 = 1; cnt2 <=nstic; cnt2++)       
            {
                if (inp.proj_code != 0)
                {
                    if(c_transinit(&geo,&four,&inp.zone_code,&inp.datum_code,
                            inp.proj_coef,&inp.proj_code,&two,&oup.zone_code,
                            &oup.datum_code,oup.proj_coef,prtprm,"\0")!=E_SUCC) 
                        return(E_FAIL);
                    status = c_trans(&geo,&four,&inp.proj_code,&two,
                            &incoor[1],&incoor[0],&otcoor[1], &otcoor[0]);
                    if (status != E_SUCC)
                        return(E_FAIL); 

                    cline = (long)(((pymax - otcoor[0]) / scaley) + 1.5);
                    csamp = (long)(((otcoor[1] - pxmin) / scalex) + 1.5);
                }
                else
                {
                    cline = (long)(((pymax - incoor[0]) / scaley) + 1.5);
                    csamp = (long)(((incoor[1] - pxmin) / scalex) + 1.5);
                }

                if ((cline > 0) && (csamp > 0)) 
                {
                    tmark++;

                    /* Print the report to the user selected output
                       -----------------------------------------------*/
                    if ((print[TERM]) || (print[PFILE]) || (print[LP]))
                    {
                        /* Convert coordinates to get the user selected type
                           ---------------------------------------------------*/
                        if ((strcmp(coform,"DMS") == 0) || 
                                (strcmp(coform,"MIN") == 0) ||
                                (strcmp(coform,"SEC") == 0))
                        {
                            c_degdms(&incoor[0],&usrcoor[0],"DEG","LAT");
                            c_degdms(&incoor[1],&usrcoor[1],"DEG","LON");
                        }
                        else if (strcmp(coform,"DEG") == 0)
                        {
                            usrcoor[0] = incoor[0];
                            usrcoor[1] = incoor[1];
                        }
                        else
                        {
                            usrcoor[0] = otcoor[0];
                            usrcoor[1] = otcoor[1];
                        }
                        badtmark += prtreport(fp,tmark,cline,csamp,coform,
                                mode,usrcoor[0],usrcoor[1],*lmax,*smax,0,
                                print,FALSE);
                    }

                    /* now that we have the center line and sample of the tic, 
                       call picpak to find the surrounding pixels and place 
                       them in the data buffers for line coordinates, sample 
                       coordinates, and brightness value--as long as both the
                       center points are positive.
                       ------------------------------------------------ */
                    picpak (&cline,&csamp,&symbol,&bright,symdim,lines,samps,
                            zlevel,&count,
                            &grid_line[(cnt * NSTIC * WIDTH ) +(cnt2 * WIDTH)],
                            &grid_samp[(cnt * NSTIC * WIDTH ) +(cnt2 * WIDTH)]); 
                }
                /* increment to the next unit of tic spacing (freq) in the
                   sample direction
                   -------------------------------------------------------*/
                if (inp.proj_code == SOM)
                    incoor[0] = incoor[0] + freq;
                else
                    incoor[1] = incoor[1] + freq;
                if (count > MAXCNT) 
                {
                    c_errmsg ("Symbol buffer full", "ticmark-ticloc", 
                              &non_fatal); 
                    return(E_FAIL);
                }
            }
            /* increment to the next unit of tic spacing in the line direction
               -------------------------------------------------------------*/
            if (inp.proj_code == SOM)
                incoor[1] = incoor[1] + freq;
            else
                incoor[0] = incoor[0] - freq;
        } 
        *grid_width = symdim[2];
    }  

    /* calculate user-defined symbol coordinates:
       Each user-given coordinate a lat/long coordinate, symbol id, 
       symbol brightness value, and symbol dimensions.  For each
       symbol coordinate call c_trans to calculate the corresponding 
       location in the output projection.  These coordinates are then 
       converted to line/sample locations.  Results are placed into 
       the 'result' buffer.  symbol id, brightness, and dimensions are 
       also entered for each symbol.  */

    if ((strcmp(mode,"USERDEF") ==0) || (strcmp(mode,"BOTH") == 0))  
    {
        /* get the host name of the user-symbol file
           ------------------------------------------------ */

        if ((infp = fopen(infile,"r")) ==NULL)
        {
            c_errmsg("Error opening file","ticcmark-open",&non_fatal);
            return(E_FAIL);
        }
        count = 0;
        fscanf(infp,"%d", &nrecs);
        /* read the user-defined symbol file and calculate a tic center point 
           for each coordinate.               
           ----------------------------------------------------------------- */
        for (cnt=0; cnt < nrecs; cnt++)  
        {
            fscanf(infp,"%lf %lf %d %d %d %d %d", &incoor[0], &incoor[1], 
                    &symbol, &symdim[0], &symdim[1], &symdim[2], &bright);
            if ((symdim[2] > symdim[0]) || (symdim[2] > symdim[1]))
            {
                fclose(infp);
                c_errmsg ("Invalid ticmark dimension","ticmark-dim",&non_fatal);
                return(E_FAIL);
            }
            usr_intensity[cnt] = bright;

            /* if the units of measure are not projection coordinates, convert 
               them to line & sample to get the tic center point.  */

            if (strcmp(coform, "PRO") != 0) 
            {
                if(c_transinit(&geo,&four,&inp.zone_code,&inp.datum_code,
                        inp.proj_coef,&inp.proj_code,&two,&oup.zone_code,
                        &oup.datum_code,oup.proj_coef,prtprm,"\0") != E_SUCC) 
                    return(E_FAIL);
                c_decdeg (&incoor[0], coform, "LAT");
                c_decdeg (&incoor[1], coform, "LON");
                status = c_trans(&geo,&four,&inp.proj_code,&inp.units, 
                        &incoor[1],&incoor[0],&otcoor[1],&otcoor[0]);
                if (status != E_SUCC)
                { 
                    fclose(infp);
                    return(E_FAIL); 
                }
            }
            else
            {
                otcoor[1] = incoor[1];      
                otcoor[0] = incoor[0];
            } 

            cline = (long)(((pymax - otcoor[0]) / scaley) + 1.5);
            csamp = (long)(((otcoor[1] - pxmin) / scalex) + 1.5);

            /* print to the screen the report on where the point was put
               and if the point is outside the image, place an asterisk
               ------------------------------------------------------------*/
            if ((cline > 0) && (csamp > 0))
            {
                tmark++;
                if ((strcmp(coform,"DMS") == 0) || (strcmp(coform,"MIN") == 0) 
                        || (strcmp(coform,"SEC") == 0))
                {
                    c_degdms(&incoor[0],&usrcoor[0],"DEG","LAT");
                    c_degdms(&incoor[1],&usrcoor[1],"DEG","LON");
                }
                else
                {
                    usrcoor[0] = incoor[0];
                    usrcoor[1] = incoor[1];
                }
                if ((print[TERM]) || (print[PFILE]) || (print[LP]))
                    badtmark += prtreport(fp,tmark,cline,csamp,coform,mode,
                            usrcoor[0], usrcoor[1],*lmax,*smax,0,print,FALSE);

                picpak (&cline, &csamp, &symbol, &bright, symdim, lines, samps,
                        zlevel, &count,&usr_line[cnt * WIDTH],
                        &usr_samp[cnt * WIDTH]);  
            }

            usr_width[cnt] = symdim[2];
            if (count > MAXCNT) 
            {
                fclose(infp);
                c_errmsg ("Symbol buffer full", "ticmark-ticloc", &non_fatal); 
                return(E_FAIL);
            }
        }
        fclose(infp);
    } 

    /* sort the resulting line/sample coordinate values by line number and
       by sample number within each line.  this enables the 'engrave'
       routine to use sequential image i/o.
       -------------------------------------------------------------------- */
    ptsort (&count, lines, samps, zlevel);


    /* This looks like a good place for EMBEDPT stuff if needed
       ---------------------------------------------------------*/

    /* Report on the number of ticmarks that are outside of image
       -------------------------------------------------------------*/
    prtreport(fp,0,0,0,"","",0.0,0.0,0,0,badtmark,print,TRUE);

    /* update the ddr if there is no input image
       --------------------------------------------*/
    if (hosin[0][0] == '\0') 
    {
        /* Initialize the validity flags
           --------------------------------*/
        for (i = 0; i < 8; i++)
            ddr->valid[i] = VALID;

        /* Update the DDR projection values
           -----------------------------------*/
        ddr->master_line   = 1;
        ddr->master_sample = 1;
        ddr->line_inc   = 1;
        ddr->sample_inc = 1;
        ddr->datum_code = 0;

        ddr->zone_code = inp.zone_code;
        ddr->proj_code = inp.proj_code;
        for (i = 0;i < COEFCT;i++)
            ddr->proj_coef[i] = ddr_coef[i];

        if (inp.proj_code != 0)
        {
            if (c_unit2string(inp.units,ddr->proj_units) != E_SUCC)
            {
                c_lasmsg(FALSE,"ticmark-warn",
                        "Input projection unit code %d is invalid", inp.units);
            }
            else
            {
                if (inp.units == RADIAN)
                {
                    scaley= (((upleft[0] - lorite[0])/((pymax - pymin)/scaley))/
                            (180)) * 3.14159265358;
                    scalex= (((upleft[1] - lorite[1])/((pxmax - pxmin)/scalex))/
                            (180)) * 3.14159265358;
                }
                else if (inp.units == FEET)
                {
                    scaley = scaley * 3.2808;
                    scalex = scalex * 3.2808;
                }
                else if (inp.units == SECOND)
                {
                    scaley = ((upleft[0] - lorite[0])/((pymax - pymin)/scaley))
                             *3600;
                    scalex = ((upleft[1] - lorite[1])/((pxmax - pxmin)/scalex))
                             *3600;
                }
                else if (inp.units == DEGREE)
                {
                    scaley = (upleft[0] - lorite[0])/((pymax - pymin)/scaley);
                    scalex = (upleft[1] - lorite[1])/((pxmax - pxmin)/scalex);
                }
                else if (inp.units == DMS)
                {
                    scaley = (upleft[0] - lorite[0])/((pymax - pymin)/scaley);
                    scalex = (upleft[1] - lorite[1])/((pxmax - pxmin)/scalex);
                    strcpy(temp_coform,"DEG");
                    c_degdms(&scaley,&tmpscaley,temp_coform,"LAT");
                    c_degdms(&scalex,&tmpscalex,temp_coform,"LON");
                    scaley = tmpscaley;
                    scalex = tmpscalex;
                }
            }

            /* convert proj_coef to the proper units
               --------------------------------------*/
            if(c_transinit(&inp.proj_code,&two,&oup.zone_code,&oup.datum_code,
                    oup.proj_coef,&inp.proj_code,&inp.units,&oup.zone_code,
                    &oup.datum_code,oup.proj_coef,prtprm,"\0") != E_SUCC) 
                return(E_FAIL);
            status = c_trans(&inp.proj_code,&two,&inp.proj_code,&inp.units,
                    &pxmin,&pymax,&upleft[1],&upleft[0]);
            if (status != E_SUCC)
                return(E_FAIL); 

            status = c_trans(&inp.proj_code,&two,&inp.proj_code,&inp.units,
                    &pxmax,&pymin,&lorite[1],&lorite[0]); 
            if (status != E_SUCC)
                return(E_FAIL); 
        }
        else
        {
            if (inp.units == 0)
            {
                strcpy(ddr->proj_units,"radians");
                upleft[0] = upleft[0]/180  * 3.14159265;
                upleft[1] = upleft[1]/180  * 3.14159265;
                lorite[0] = lorite[0]/180  * 3.14159265;
                lorite[1] = lorite[1]/180  * 3.14159265;
                scaley = scaley/180 * 3.14159265;
                scalex = scalex/180 * 3.14159265;
            }
            else if (inp.units == 1)
            {
                strcpy(ddr->proj_units,"feet");
            }
            else if (inp.units == 2)
                strcpy(ddr->proj_units,"meters");
            else if (inp.units == 3)
            {
                strcpy(ddr->proj_units,"seconds");
                upleft[0] *= 3600;
                upleft[1] *= 3600;
                lorite[0] *= 3600;
                lorite[1] *= 3600;
                scaley *= 3600;
                scalex *= 3600;
            }
            else if (inp.units == 4)
            {
                strcpy(ddr->proj_units,"degrees");
            }
            else if (inp.units == 5)
            {
                strcpy(ddr->proj_units,"dms");
            }
        }

        ddr->pdist_y = scaley;
        ddr->pdist_x = scalex;

        /* Calculate projection coordinates based on full scene center
           --------------------------------------------------------------*/
        ddr->upleft[0] = upleft[0];
        ddr->upleft[1] = upleft[1];
        ddr->upright[0] = upleft[0];
        ddr->upright[1] = lorite[1];
        ddr->loright[0] = lorite[0];
        ddr->loright[1] = lorite[1];
        ddr->loleft[0] = lorite[0];
        ddr->loleft[1] = upleft[1];
    }

    return(E_SUCC);
}
