/*****************************************************************************
NAME: TICMARK
 
PURPOSE: Ticmark places ticmarks or image corner marks at user
         specified lat/longs and at lat/long intervals as defined
         by the user.  The tics are either overlaid onto an existing
         image or onto an image of constant background, creating a
         mask image.  Twenty-one projections are available from gctp.

PROGRAM HISTORY:
VERSION	DATE       AUTHOR             request
-------	----	   ------             -------
1.0     5/86       D. Steinwand       prf# 2753,3459
2.0     8/86       D. Steinwand       prf/adp# 3604
3.0     5/90       W. Xia             conversion to c
3.1     11/90      ??????             Fix bug in ticloc where lat/long needed
                                        to be switched before call to gtpz0
3.2     10/91      B. Ailts           ADP# 6204  Correted to create a valid
                                        ddr when an input image is not entered. 
4.0     4/92       T. Mittan          USE INPROJ file instead of a GCTP file.
                                      Allowed multiple bands.
5.0     9/92       T. Mittan          Allowed INPROJ to be made NULL if a
                                        valid image DDR is provided
5.1     2/93       T. Mittan          Fixed to allow images of GEO projection
                                      code to work correctly.
5.2     6/93       T. Mittan          Modified ticloc so that it works
                                        correctly when the origin is specified.
7.0     7/98       J. Janssen         Increased the number of points allowed
                                        in infile to 75000.
8.0     10/99      T. Ratliff         Modified to allow TICMARK to handle
                                        images > 2gig.
                                      Prototyped the code.

DESCRIPTION/ALGORITHM:
  call getpar to get user defined parameters      
  call ticloc finds line/sample locations of tics in output image
  call filspec to open input/output files, 
  call decdeg converts angles to total degrees
  call engrave to place tics in the output image  
  call c_minmax and c_svmnmx to update the ddr
  call c_hstout to update the output image history records
  call c_complt to issue a standard completion message
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ticmark.h"

void main
(
  void
)
{
struct DDR ddr;			/* ddr structure			*/
struct  PARBLK  block;		/* TAE parameter block     		*/
static
double	amax[MAXBND],		/* maximum pixel value    	    	*/
        amin[MAXBND],		/* minimum pixel value        		*/  
	freq,			/* distance in degrees between tics 	*/
	start[2];		/* starting tic				*/

long    fatal_err = -1;		/* fatal error return status 		*/
static
long	bckgnd,			/* background value			*/
	bright,			/* tic pixel value			*/
	bright1,		/* tic pixel value			*/
	img,			/* image counter			*/
        image_band_num[MAXIMG][MAXBND+1],  /* band numbers for each image */
	in_def,			/* flag if input image is entered	*/
	*lines,			/* ticmark array of lines		*/
	nlines,			/* number of lines			*/
	nsamps,			/* number of samps			*/
	nullfi=FALSE,		/* flag if input image specified	*/
        num_image_bands[MAXIMG], /* number of bands in each input image */
	num_inbands,		/* number of input bands		*/
      	num_inimages,		/* number of input images           	*/
        num_outbands,		/* number of output bands		*/
	ornull=FALSE,		/* starting coordinate flag		*/
	out_type,		/* output image type			*/
	*samps,			/* ticmark array of samples		*/
	symdim[3];		/* ticmark dimensions			*/
long	out_img_bands[1][MAXBND+1],  /* output image band numbers 	*/ 
	*zlevel;		/* ticmark array of intensity values	*/
long    zero = 0;		/* zero					*/
long	one = 1;		/* value of one				*/
long    two = 2;                /* value of two                         */
float   *outbuf;		/* image output buffer 		    	*/
struct FDESC *fdescin[MAXIMG+1];/*  file descriptors  			*/
struct FDESC *fdescout;		/*  file descriptors  			*/
long    min_max;		/* minmax option flag      		*/
long    msgtim;			/* message interval        		*/
long	status;			/* status flag 				*/
float   scale;			/* scale factor for EMBEDPT option	*/
long    window[MAXIMG][4];	/* input image dimensions (SL SS NL NS) */
long	*grid_line;		/* array of grid line values		*/
long	*grid_samp;		/* array of grid sample values		*/
long	*usr_width;		/* width specified by user		*/
long	*usr_line;		/* user specified grid line values	*/
long	*usr_samp;		/* user specified grid sample values	*/
long	*usr_intensity;		/* user specified grid sample values	*/
long	grid_flg;		/* grid flag				*/
long	grid_width;		/* grid width				*/
long	mingrey;		/* minimum grey value			*/
long	maxgrey;		/* maximum grey value			*/
long    print[3];

char    coform[5],		/* type of input			*/
	infile[CMLEN],		/* input file name			*/ 
	mode[10];		/* which subcommand was specified	*/
char    taeout[2][CMLEN];	/* output image tae name      		*/
char    hosout[1][CMLEN];	/* output image host name     		*/
char    hosin[MAXIMG][CMLEN];	/* input image host name      		*/
char    taein[MAXIMG][CMLEN];	/* input image tae name       		*/
char	tempfile[2][CMLEN];	/* array of temporary file names	*/
char    filename[CMLEN];
char    prtfile[CMLEN];
FILE *fp;

outbuf = NULL;

if (getpar(&nullfi, &ornull, &num_inbands, &bckgnd, infile, mode,
           start, &freq, symdim, &bright, coform, &num_inimages,
           num_image_bands, &num_outbands, &out_type, &block, &min_max,
           &msgtim, window, image_band_num, taeout, hosout, taein, hosin,
           &in_def, &grid_flg, &scale, &mingrey,&maxgrey) 
	== E_FAIL)
          c_errmsg("Fatal error encountered","ticmark-fatal",&fatal_err);
bright1 = bright;

/* Get the PRINT parameter values
---------------------------------*/
strcpy(prtfile,"ticmark");
status = c_setprt(&block,&print[TERM],&print[PFILE],&print[LP],prtfile,
                  filename);
if (status != E_SUCC)
  c_errmsg("Fatal error encountered","ticmark-fatal",&fatal_err);

/* Open the print file (if necessary)
-------------------------------------*/
if ((print[PFILE]) || (print[LP]))
  fp = fopen(prtfile,"w");

/* Allocate some space
---------------------*/
if  ((lines = (long *) malloc (MAXCNT * sizeof(long))) == NULL )
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if  ((samps = (long *) malloc (MAXCNT * sizeof(long))) == NULL )
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if  ((zlevel = (long *) malloc (MAXCNT * sizeof(long))) == NULL )
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if  ((usr_width = (long *) calloc (MAXCNT , sizeof(long))) == NULL )
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if((grid_line = (long *)calloc((NLTIC * NSTIC * WIDTH) ,sizeof(long))) == NULL)
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if((grid_samp = (long *)calloc ((NLTIC * NSTIC * WIDTH) ,sizeof(long)))== NULL)
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if  ((usr_line = (long *) calloc ((MAXCNT * WIDTH) , sizeof(long))) == NULL)
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if  ((usr_samp = (long *) calloc ((MAXCNT * WIDTH) , sizeof(long))) == NULL)
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if  ((usr_intensity =(long *) calloc ((MAXCNT * WIDTH) , sizeof(long))) == NULL)
    c_errmsg ("Error allocating dynamic memory", "ticmark-alloc",
             &fatal_err);

if (strcmp(mode,"EMBEDPT")!=0)
if (ticloc (&block, infile, mode, freq, start, symdim, bright1, coform,ornull, 
	&nlines, &nsamps, lines, samps, zlevel, &ddr, hosin, &in_def,grid_line,
	grid_samp, &grid_width, usr_line, usr_samp, usr_width, usr_intensity,
        fp,print)
	== E_FAIL)
          {
          fclose(fp);
          free (usr_width);
          free (grid_line);
          free (grid_samp);
          free (usr_samp);
          free (usr_line);
          free (usr_intensity);
          free (lines);
          free (samps);
          free (zlevel);
          c_errmsg("Fatal error encountered","ticmark-fatal",&fatal_err);
	  }

if (grid_flg)
   if (line_sort(&block,grid_line,grid_samp,grid_width,tempfile,
	     usr_line, usr_samp, usr_width, usr_intensity, &nlines, &nsamps,
	     &bright) 
   	     == E_FAIL)
          {
          fclose(fp);
          free (usr_width);
          free (grid_line);
          free (grid_samp);
          free (usr_samp);
          free (usr_line);
          free (usr_intensity);
          free (lines);
          free (samps);
          free (zlevel);
          c_errmsg("Fatal error encountered","ticmark-fatal",&fatal_err);
          }

if (strcmp(mode,"EMBEDPT") == 0)
   {
   if (embedpt(&block,infile,tempfile,lines,samps,zlevel,usr_line,usr_samp,
	       symdim,scale,mingrey,maxgrey,fp,print,window)
        == E_FAIL)
          {
          fclose(fp);
          free (usr_width);
          free (grid_line);
          free (grid_samp);
          free (usr_samp);
          free (usr_line);
          free (usr_intensity);
          free (lines);
          free (samps);
          free (zlevel);
          c_errmsg("Fatal error encountered","ticmark-fatal",&fatal_err);
          }
   grid_flg = TRUE;
   }

free (usr_width);
free (grid_line);
free (grid_samp);
free (usr_samp);
free (usr_line);
free (usr_intensity);

if (filspec( &num_inimages, num_image_bands, image_band_num, num_outbands,
        out_type, out_img_bands, &nlines, &nsamps, &ddr, &block,fdescin, 
	&fdescout, window, hosout, taein, hosin) == E_FAIL)
          {
          fclose(fp);
          free (lines);
          free (samps);
          free (zlevel);
          c_errmsg("Fatal error encountered","ticmark-fatal",&fatal_err);
          }

if (engrave ( nullfi, nlines,  nsamps, bckgnd, image_band_num, 
	  num_image_bands,&num_inimages, num_inbands, lines, samps, zlevel, 
	  outbuf,fdescin,&fdescout, &msgtim, window,&grid_flg, tempfile)
  	  == E_FAIL)
          {
          fclose(fp);
          free (lines);
          free (samps);
          free (zlevel);
          c_clean(&block,"ticmark",hosout,&one);
          c_errmsg("Fatal error encountered","ticmark-fatal",&fatal_err);
          }
         
c_stoptm();

/* Free the dynamic memory.
------------------------ */
free (lines);
free (samps);
free (zlevel);

/* Close the image files.
----------------------*/
if (!nullfi)
   for (img = 0; img < num_inimages; img++)
      c_eclose (&fdescin[img]);
c_eclose (&fdescout);

/* calculate the minimum and maximum pixel values for the output image
------------------------------------------------------------------- */
if (min_max)    
{
  window[0][SL] = window[0][SS] = 1;      
  c_minmax (hosout, &one, window, out_img_bands, &num_outbands, amin, amax);  
}                          

/* Generate the history file and print the completion message.
------------------------------------------------------------ */                 
if (!nullfi)
   c_hstout (&block, hosout[0], hosin, &num_inimages, " "); 
else
   c_hstout (&block, hosout[0], hosin, &zero, ""); 

if ((print[PFILE]) || (print[LP]))
  fclose(fp);

if (print[PFILE])
  {
  strcpy(taeout[1],filename);
  c_complt("TICMARK",taeout,&two);
  }
else
  c_complt("TICMARK",taeout,&one);
}
