/*****************************************************************************
NAME: USE_DDR_INFO

PURPOSE: Reads projection & framing information from an image DDR

PROGRAM HISTORY:
VERSION  AUTHOR          DATE     REASON
-------  ------          ----     ------
5.0      D. Steinwand    7/89     Original development
5.1      T. Mittan       4/92     Took form REMAP and added to TICMARK to
                                  allow the use of a projection file instead
                                  of an INGCTP file.

ALGORITHM DESCRIPTION:
    Retrieve the name of the input image via getin()
    Get the DDR information
    Return
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "ticmark.h"

int use_ddr_info
(
    struct PARBLK *vblk,        /* I: TAE parameter block */
    struct PROJ *pro,           /* O: Struct containing projection parameters */
    double *corners,            /* O: Corner projection coordinates */
    long *image,                /* O: Corner image coordinates */
    long in_wind[][4],          /* O: Image window specification */
    double *scaley,             /* O: Line pixel value */
    double *scalex,             /* O: Sample pixel value */
    long *pixflg                /* I: Pixsize flag */
)
{
    struct DDR ddr;         /* DDR structure                                */
    long i;                 /* Loop counters                                */
    long imglimit = MAXIMG;
    long zero = 0;          /* Constant of zero                             */
    long one = 1;           /* Constant of one for c_errmsg routine         */
    long num_img;           /* Number of images input:  0 or 1              */
    long bands[1][MAXBND+1];/* Band numbers                                 */
    long nbands;            /* Number of bands                              */
    long totbands;          /* Total number of bands                        */
    long same;              /* Same window flag                             */
    long qflag[MAXIMG][2];  /* Option flags                                 */
    char in_image[MAXIMG][CMLEN];/* Image file name                         */
    char tdum[MAXIMG][CMLEN];/* Tae name                                    */

    /* Check for projection information and place it in the pro structures
       if it exists.  
       -------------*/
    if(c_getin(vblk,"IN",&zero,tdum,in_image,&num_img,in_wind,bands,&nbands,
            &totbands,&imglimit,&same,qflag) != E_SUCC) return(E_FAIL);
            if (c_getddr(in_image[0], &ddr) != E_SUCC) 
            {
                c_errmsg("Projection information not available from DDR",
                        "ticmark-noproj",&one);
                return(E_FAIL);
            }
            else
            {
                image[0] = image[1] = 1;
                image[2] = ddr.nl;
                image[3] = 1;
                image[4] = 1;        
                image[5] = ddr.ns;
                image[6] = ddr.nl;
                image[7] = ddr.ns;

                if ((ddr.valid[0] == INVAL) || (ddr.valid[3] == INVAL) || 
                        (ddr.valid[4] == INVAL) || 
                        (!*pixflg && (ddr.valid[5] == INVAL)))
                {
                    c_errmsg("Projection information not available from DDR",
                            "ticmark-noproj",&one);
                    return(E_FAIL);
                }
                else
                {
                    pro->proj_code = ddr.proj_code;
                    if (ddr.valid[1] == INVAL) pro->zone_code = 62;
                    else pro->zone_code = ddr.zone_code;
                    if (ddr.valid[2] == INVAL) pro->datum_code = 0;
                    else pro->datum_code = ddr.datum_code;
                    pro->units = c_string2unit(ddr.proj_units);
                    if (pro->units == E_FAIL)
                    {
                        c_lasmsg(FALSE,"ticmark-warn",
                                "Projection units %s are invalid",
                                ddr.proj_units);
                    }
                    for (i = 0; i < 15; i++)
                        pro->proj_coef[i] = ddr.proj_coef[i];

                    if (ddr.valid[6] == INVAL)
                        for (i = 0; i < 8; i++)
                            corners[i] = 0.0;
                    else
                    {
                        corners[0] = ddr.upleft[0];
                        corners[1] = ddr.upleft[1];
                        corners[2] = ddr.loleft[0];
                        corners[3] = ddr.loleft[1];
                        corners[4] = ddr.upright[0];
                        corners[5] = ddr.upright[1];
                        corners[6] = ddr.loright[0];
                        corners[7] = ddr.loright[1];
                    }
                }
            }

            if (!*pixflg)
            {
                *scalex = ddr.pdist_x;
                *scaley = ddr.pdist_y;
            }
            return(E_SUCC);
}
