/****************************************************************************
NAME:				merge

PURPOSE:  Merge a two tie point selection files into a merged tie point file 
	  or a tie point location file with or without prompting the user

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		 7/83	D. Fizgerald	   CSC	    Original Development
  4.0		 6/85	S. Kondal	   SAR	    20-498
  5.0		12/88	D. Steinwand	   CSB      LAS 5.0 Development,
						    conversion to C
  5.1           12/91   D. Etrheim         CSB      ADP #6105 Elevation is now
                                                    taken from the reference
                                                    TPS file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Allocate space for reference tie points
	Initialize TAE terminal I/O
	Open the reference tie point selection file
	Open the search tie point selection file
	Read the entire reference file; store in buffer
	If the -PRECORR subcommand was chosen,
	   Pack the MTP file header record
	   Open the MTP file for output
	   Merge tie point records & write to the MTP file
	If the -NOCORR subcommand was chosen,
	   Pack the TPL file header record
	   Open the TPL file for output
	   Merge tie point records & write to the TPL file
	Print status message
	Close files
	Return
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

int merge
(
    char *ref,		/* Reference tie point selection file */
    char *sea,		/* Search tie point selection file */
    char *out,		/* Output tie point file */
    long file_type,		/* Type of output file */
    long confirm		/* Confirm flag:  TRUE or FALSE */
)
{
struct TPSHEAD ref_hdr;		/* Reference tps file header */
struct TPSHEAD sea_hdr;		/* Search tps file header */
struct MTPHEAD mtp_hdr;		/* Merged tie point header record */
struct TPLHEAD loc_hdr;		/* Tie Point Location header record */
struct TPSDATA *ref_data;	/* Search tps data record */
struct PTDESC  *ref_desc;			/* File descriptor for reference tps file */
struct PTDESC  *sea_desc;			/* File descriptor for search tps file */
struct PTDESC  *out_desc;			/* File descriptor for output tie pt file */
long ptyp;			/* Type of tie point data/header record */
long acc;			/* File access flag */
long num_merged = 0;		/* Number of tie point records merged */
long not_merged = 0;		/* Number of tie point records not merged */
long pass = 1;			/* Non-fatal error flag */
long status;			/* Function return status flag */
long count;			/* Loop counter */
int  dum_lines,dum_samps,dum_type; /* Return variables from t_init--not used */
long maxtie;			/* maximum space allocated		*/
char mess[80];			/* Buffer for misc processing messages */

/* Allocate buffer space for reference tie point selection records
  ---------------------------------------------------------------*/
ref_data = (struct TPSDATA *)malloc((sizeof(struct TPSDATA) * MAX_TIE_POINTS));
if (ref_data == NULL) return(NO_SPACE);
maxtie = MAX_TIE_POINTS;

/* Initialize TAE terminal I/O, if user wants to confirm merge
  -----------------------------------------------------------*/
if (confirm)
   if (t_init(&dum_lines, &dum_samps, &dum_type) != SUCCESS) return(E_FAIL);

/* Open the reference tie point selection file and check file mode
  ---------------------------------------------------------------*/
ptyp = TPS;
acc = IREAD;
if (c_ptopen(&ref_desc,ref,&ptyp,&acc,&ref_hdr) != E_SUCC) return(E_FAIL);
if (strcmp(ref_hdr.mode, "REF") == 0)
   c_errmsg("INTS(1) is a reference file"," ",&pass);
else if (strcmp(ref_hdr.mode, "SEA") == 0)
   {
   c_errmsg("INTS(1) is a search file"," ",&pass);
   c_errmsg("The mode of INTS(1) has been set to 'REF' (reference)"," ",&pass);
   }
else
   {
   c_errmsg("INTS(1) is not a search or a reference file"," ",&pass);
   c_errmsg("The mode of INTS(1) has been set to 'REF' (reference)"," ",&pass);
   }

/* Open the search tie point selection file and check file mode
  ------------------------------------------------------------*/
ptyp = TPS;
acc = IREAD;
if (c_ptopen(&sea_desc,sea,&ptyp,&acc,&sea_hdr) != E_SUCC) return(E_FAIL);
if (strcmp(sea_hdr.mode, "SEA") == 0)
   c_errmsg("INTS(2) is a search file"," ",&pass);
else if (strcmp(sea_hdr.mode, "REF") == 0)
   {
   c_errmsg("INTS(2) is a reference file"," ",&pass);
   c_errmsg("The mode of INTS(2) has been set to 'SEA' (search)"," ",&pass);
   }
else
   {
   c_errmsg("INTS(2) is not a search or a reference file"," ",&pass);
   c_errmsg("The mode of INTS(2) has been set to 'SEA' (search)"," ",&pass);
   }

/* Read in the reference tie point file
  ------------------------------------*/
for (count = 0; ; count++)
   {
   if (count + 1 == maxtie)
      {
      maxtie += MAX_TIE_POINTS;
      ref_data = (struct TPSDATA *) realloc(ref_data,
                                 (maxtie * sizeof(struct TPSDATA)));
      }

   status = c_ptio(&ref_desc, &(ref_data[count]));
   if (status == E_EOF) break;
   if (status == E_FAIL) return(E_FAIL);
   }
for (; count < maxtie; count++)
   strcpy(ref_data[count].pt_id, "");

/* Write a Merged Tie Point file if the PRECORR subcommand was chosen
  ------------------------------------------------------------------*/
if (file_type == PRECORR)
   {
   to_mtp_header(&ref_hdr, &sea_hdr, &mtp_hdr);
   ptyp = MTP;
   acc = IWRITE;
   status = c_ptopen(&out_desc,out,&ptyp,&acc,&mtp_hdr);
   if (status != E_SUCC)
	{
	c_ptclse(&ref_desc);
	c_ptclse(&sea_desc);
	c_ptclse(&out_desc);
	return(GEO_CLEAN);
	}
   status = do_merge(sea_desc, out_desc, confirm, &num_merged, &not_merged,
	             ref_data, to_mtp_data, maxtie);
   if (status != E_SUCC)
	{
	c_ptclse(&ref_desc);
	c_ptclse(&sea_desc);
	c_ptclse(&out_desc);
	return(GEO_CLEAN);
	}
   }

/* Write a Tie Point Location file if the NOCORR subcommand was chosen
  -------------------------------------------------------------------*/
else
   {
   to_tpl_header(&ref_hdr, &sea_hdr, &loc_hdr);
   ptyp = TPL;
   acc = IWRITE;
   status = c_ptopen(&out_desc,out,&ptyp,&acc,&loc_hdr);
   if (status != E_SUCC)
	{
	c_ptclse(&ref_desc);
	c_ptclse(&sea_desc);
	c_ptclse(&out_desc);
	return(GEO_CLEAN);
	}
   status = do_merge(sea_desc, out_desc, confirm, &num_merged, &not_merged,
	             ref_data, to_tpl_data,maxtie);
   if (status != E_SUCC)
	{
	c_ptclse(&ref_desc);
	c_ptclse(&sea_desc);
	c_ptclse(&out_desc);
	return(GEO_CLEAN);
	}
   }
sprintf(mess, "TIEMERGE processed %d tie points",(num_merged + not_merged));
c_errmsg(mess," ",&pass);
sprintf(mess, "Of these, %d were successfully merged",num_merged);
c_errmsg(mess," ",&pass);
c_ptclse(&ref_desc);
c_ptclse(&sea_desc);
c_ptclse(&out_desc);
free(ref_data);
return(E_SUCC);
}

/****************************************************************************
NAME:				do_merge

PURPOSE:  Finds a corresponding tp_id in search and reference TPS files
	  and merges the results

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		11/88	D. Steinwand	   CSB      LAS 5.0 Development

ALGORITHM DESCRIPTION:
	For each tie point in the search tie point location file:
	   Get a tie point record
	   Check the I/O status
	   If the user specified confirm mode, 
	      Get confirm action
	   If the point is to be merged,
		Find the corresponding reference record
		If the record exists,
		   Merge records & write to disk
	Return
*****************************************************************************/
int do_merge
(
    struct PTDESC *sea,			/* Pointer to file descriptor; sea TPS */
    struct PTDESC *desc,			/* Pointer to file descriptor; output */
    long confirm,			/* Confirm flag (boolean) */
    long *num,			/* Number of tie points merged */
    long *not,			/* Number of tie points not merged */
    struct TPSDATA ref_data[],	/* Search TPS data records */
    int (*routine)(struct TPSDATA *, struct TPSDATA *, struct PTDESC *),	
                        /* Pointer to merge function */
    long maxtie			/* maximum tie points allocated	*/
)
{
struct TPSDATA sea_data;	/* Reference tps data record */
long merge_it = TRUE;		/* Merge flag */
long count;			/* Loop counter */
long status;			/* Function return status flag */
long index;			/* Index of array containing matching sea tp */

/* Read each record in the search file and match pt_id's in the reference
  ----------------------------------------------------------------------*/
for (count = 0; ; count++)
   {
   status = c_ptio(&sea, &sea_data);
   if (status == E_EOF) return(E_SUCC);
   if (status == E_FAIL) return(E_FAIL);
   if (confirm) merge_it = get_confirm(sea_data.pt_id);
   if (merge_it)
     {
     index = find_ref_rec(sea_data.pt_id, ref_data,maxtie);
     if (index < 0)
	(*not)++;
     else
	{
	(*routine)(&(ref_data[index]), &sea_data, desc);
	(*num)++;
	}
     }
  else
    (*not)++;
  }
}

/****************************************************************************
NAME:				to_tpl_data

PURPOSE:  Merges two TPS data records into a TPL data record

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		11/88	D. Steinwand	   CSB      LAS 5.0 Development

ALGORITHM DESCRIPTION:
	Merge search & reference tie point selection file data records into
	   a tie point location file data record
	Write to disk
	Return
*****************************************************************************/
int to_tpl_data
(
    struct TPSDATA *ref,		/* Reference tie point record */
    struct TPSDATA *search,		/* Search tie point record */
    struct PTDESC  *desc		/* Pointer to file descriptor */
)
{
struct TPLDATA loc;		/* Tie point location data record */

loc.geo_coord[0] = ref->coord[0];
loc.geo_coord[1] = ref->coord[1];
loc.sea_coord[0] = search->image_coord[0];
loc.sea_coord[1] = search->image_coord[1];
loc.ref_coord[0] = ref->image_coord[0];
loc.ref_coord[1] = ref->image_coord[1];
loc.rms_error[0] = loc.rms_error[1] = 0.0;
loc.strength = loc.displacement = 0.0;
loc.nominal[0] = search->image_coord[0];
loc.nominal[1] = search->image_coord[1];
loc.elevation = ref->elevation;
loc.zoom_factor = search->zoom_factor;
loc.ref_size[0] = ref->chip_size[0];
loc.ref_size[1] = ref->chip_size[1];
loc.sea_size[0] = search->chip_size[0];
loc.sea_size[1] = search->chip_size[1];
loc.offsets[0] = search->offsets[0];
loc.offsets[1] = search->offsets[1];
loc.chip_loc[0] = loc.chip_loc[1] = 0;
loc.chip_coord[0] = loc.chip_coord[1] = 0;
loc.req_size[0] = loc.req_size[1] = 0;
loc.location[0] = loc.location[1] = 0;
loc.active = 0;
strcpy(loc.pt_id, ref->pt_id);
strcpy(loc.ref_chip, ref->chip_name);
return(c_ptio(&desc, &loc));
}


/****************************************************************************
NAME:				to_mtp_data

PURPOSE:  Merges two TPS data records into a MTP data record

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		11/88	D. Steinwand	   CSB      LAS 5.0 Development

ALGORITHM DESCRIPTION:
	Merge search & reference tie point selection file data records into
	   a merged tie point file data record
	Write to disk
	Return
*****************************************************************************/
int to_mtp_data
(
    struct TPSDATA *ref,		/* Reference tie point record */
    struct TPSDATA *search,		/* Search tie point record */
    struct PTDESC  *desc		/* Pointer to file descriptor */
)
{
struct MTPDATA merged;		/* Merged tie point record */
merged.geo_coord[0] = ref->coord[0];
merged.geo_coord[1] = ref->coord[1];
merged.nominal[0] = search->image_coord[0];
merged.nominal[1] = search->image_coord[1];
merged.ref_coord[0] = ref->image_coord[0];
merged.ref_coord[1] = ref->image_coord[1];
merged.elevation = ref->elevation;
merged.zoom_factor = search->zoom_factor;
merged.chip_size[0] = ref->chip_size[0];
merged.chip_size[1] = ref->chip_size[1];
merged.offsets[0] = search->offsets[0];
merged.offsets[1] = search->offsets[1];
strcpy(merged.pt_id, ref->pt_id);
strcpy(merged.ref_chip, ref->chip_name);
strcpy(merged.sea_chip, search->chip_name);
return(c_ptio(&desc, &merged));
}

/****************************************************************************
NAME:				to_mtp_header

PURPOSE:  Merges fields from two TPS file headers into a MTP file header

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		11/88	D. Steinwand	   CSB      LAS 5.0 Development

ALGORITHM DESCRIPTION:
	Merge search & reference tie point selection file header records into
	   a merged tie point file header record
	Return
*****************************************************************************/
void to_mtp_header
(
    struct TPSHEAD *ref,		/* Reference tie point header record */
    struct TPSHEAD *search,		/* Search tie point header record */
    struct MTPHEAD *merged		/* Merged tie point header record */
)
{
long count;			/* Loop counter */

for (count = 0; count < 8; count++)
   merged->corners[count] = ref->corners[count];
merged->pdist[0] = ref->pdist[0];
merged->pdist[1] = ref->pdist[1];
for (count = 0; count < 15; count++)
   merged->projprms[count] = ref->projprms[count];
merged->proj_valid = ref->proj_valid;
merged->code = ref->code;
merged->zone = ref->zone;
merged->datum = ref->datum;
strcpy(merged->chip_mode, "NO-CHIPS");
strcpy(merged->units, ref->units);
strcpy(merged->sea_name, search->name);
strcpy(merged->ref_name, ref->name);
return;
}

/****************************************************************************
NAME:				to_tpl_header

PURPOSE:  Merges fields from two TPS file headers into a TPL file header

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		11/88	D. Steinwand	   CSB      LAS 5.0 Development

ALGORITHM DESCRIPTION:
	Merge search & reference tie point selection file header records into
	   a tie point location file header record
	Return
*****************************************************************************/
void to_tpl_header
(
    struct TPSHEAD *ref,		/* Reference tie point header record */
    struct TPSHEAD *search,		/* Search tie point header record */
    struct TPLHEAD *loc		/* Tie point location header record */
)
{
long count;			/* Loop counter */

for (count = 0; count < 8; count++)
   loc->corners[count] = ref->corners[count];
loc->pdist[0] = ref->pdist[0];
loc->pdist[1] = ref->pdist[1];
for (count = 0; count < 15; count++)
   loc->projprms[count] = ref->projprms[count];
loc->edge_thres = 0.0;
loc->min_str = 0.0;
loc->max_dis = 0.0;
loc->proj_valid = ref->proj_valid;
loc->code = ref->code;
loc->zone = ref->zone;
loc->datum = ref->datum;
if ((strcmp(ref->mode, "GEO") == 0) || (strcmp(ref->mode, "PRO") == 0))
   strcpy(loc->ref_mode, "MAP");
else 
   strcpy(loc->ref_mode, "IMAGE");
strcpy(loc->units, ref->units);
strcpy(loc->peak_method, "NO_CORR");
strcpy(loc->corr_type, "MANUAL");
strcpy(loc->sea_name, search->name);
strcpy(loc->ref_name, ref->name);
return;
}

/****************************************************************************
NAME:				find_ref_rec

PURPOSE:  Given a point id, the array index of the reference record with
	  that point id is returned

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		12/88	D. Steinwand	   CSB      LAS 5.0 Development

ALGORITHM DESCRIPTION:
	For each tie point record,
	   Compare point id fields
	   If the fields are the same, return the index of the tie point record
	Return an invalid (negitive) index
*****************************************************************************/
int find_ref_rec
(
    char *pt_id,		/* Tie point identifier field */
    struct TPSDATA reference[],/* Referrence tie point selection data records */
    long maxtie
)
{
long count;		/* Loop counter */

for (count = 0; count < maxtie; count++)
    if (strcmp(pt_id, reference[count].pt_id) == 0) return(count);
return(-9999);
}

/****************************************************************************
NAME:				get_confirm

PURPOSE:  Uses TAE terminal I/O to get flag from user

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		12/88	D. Steinwand	   CSB      LAS 5.0 Development

ALGORITHM DESCRIPTION:
	Prompt the user:  Merge tie point? (Y or N)
	Check the response.  If yes, return (TRUE)
			     If no, return (FALSE)
			     If neither yes or no, do again...
*****************************************************************************/
int get_confirm
(
    char *pt_id		/* Tie point identifier field */
)
{
TEXT mess[80];		/* Message buffer */
FUNINT flag = T_PROMPT;	/* Constant for xt calls */
CODE term;		/* Terminal variable for xt calls */

sprintf(mess, "Do you wish to merge tie point %s ([Y]/N):",pt_id);
t_write(mess, flag);
t_read(mess, &term);
if (strlen(mess) == 0) return(TRUE);
if (strncmp(mess,"y",1) == 0) return(TRUE);
if (strncmp(mess,"Y",1) == 0) return(TRUE);
if (strncmp(mess,"n",1) == 0) return(FALSE);
if (strncmp(mess,"N",1) == 0) return(FALSE);
for (;;)
   {
   t_write("Please enter [Y] or N:", flag);
   t_read(mess, &term);
   if (strlen(mess) == 0) return(TRUE);
   if (strncmp(mess,"y",1) == 0) return(TRUE);
   if (strncmp(mess,"Y",1) == 0) return(TRUE);
   if (strncmp(mess,"n",1) == 0) return(FALSE);
   if (strncmp(mess,"N",1) == 0) return(FALSE);
   }
}
