/****************************************************************************
NAME:				tiemerge

PURPOSE:  Merges selected entries from two input tie point selection files
	  to form either a merged tie point file for use in image correlation 
	  or a tie point location file for use in the modeling process.  In 
	  normal usage, one of the two input files corresponds to a reference
	  coordinate system while the other corresponds to the search 
          coordinate system that is to be transformed.  Examples of this are 
	  image-to-image registration and image-to-map (geographic) 
	  registration.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0		 7/83	D. Fizgerald	   CSC	    Original Development
  4.0		 6/85	S. Kondal	   SAR	    20-498
  5.0		10/88	D. Steinwand	   CSB      LAS 5.0 Development,
						    conversion to C
  5.1		12/91	D. Etrheim	   CSB      ADP #6105 Elevation is now
						    taken from the reference
						    TPS file.	
  5.2		 1/95	T. Mittan	   CSB	    Modified to handle any
						    number of tie points.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize LAS processing
	Get parameters from TAE
	If the user wishes to confirm each merge,
	   Call merge_confirm 
	Else
	   Call merge_batch
	Write the standard completion message & exit

ALGORITHM REFERENCES:  None
*****************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

int main
(void)
{
struct PARBLK vblk;		/* TAE variable block */
long std_out;			/* Standard output */
long stat;			/* Function return status */
long confirm;			/* Confirm flag:  TRUE or FALSE */
long file_type;			/* Processing mode: NOCORR or PRECORR */
long one = 1;			/* Constant of one */
char tps_ref[CMLEN];		/* Reference tie point selection file */
char tps_search[CMLEN];		/* Search tie point selection file */
char outfile[1][CMLEN];		/* Output file name (host) */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get parameters from the user
  ----------------------------*/
stat = getpar(&vblk, tps_ref, tps_search, outfile[0], &confirm, &file_type);
if (stat != E_SUCC) tabort(stat);

/* Merge tie points
  ----------------*/
stat = merge(tps_ref, tps_search, outfile[0], file_type, confirm);
if (stat == GEO_CLEAN) c_clean(&vblk, "TIEMERGE", outfile, &one);
if (stat != E_SUCC) tabort(stat);

/* Write completion message and exit
  ---------------------------------*/
c_complt("TIEMERGE", outfile, &one);

return EXIT_SUCCESS;

}
