/*
 *  ProcessM.c
 */

/*
 *  Implementation file for ProcessM compound widget.
 */

/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or 
 * publicity pertaining to distribution of the software without specific, 
 * written prior permission.  USGS makes no representations about the 
 * suitability of this software for any purpose.  It is provided "as is" 
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS 
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *  Change log:
 *
 *      D. Akkerman, USGS/EROS Data Center, 7/92
 *       - inception.
 *
 */

/*
 *  This module implements a processing message box that an application can
 *  use to provide feedback to the user during long processing operations.
 *  As written, the ProcessM widget is a compound widget encapsulating a
 *  Motif widget tree required for a processing message.  In general, it is
 *  a subclass of the Motif BulletinBoard widget.
 *
 */

/*
 *  Required include files.
 */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/DrawingA.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>

#include "tiepts.h"
#include "ProcessMP.h"
#include "protos.h"

/*
 *  Working constants.
 */

#define CANCEL          "Cancel"
#define PAUSE           "Pause"
#define RESUME          "Resume"

#define EVENT_MASK      (ButtonPressMask | ButtonReleaseMask | PointerMotionMask | KeyPressMask | KeyReleaseMask)

/*
 *  Resource table.
 */

#define res_offset(field) XtOffset(XeProcessMWidget, field)

static XtResource resources[] =
{
        {XeNlabelString, XeCLabelString, XtRString, sizeof(char *), 
         res_offset(processm.label_string), XtRString, PROCESSM_DEFAULT_LABEL},
        {XeNlabelFont, XeCLabelFont, XtRFontStruct, sizeof(XFontStruct *),
         res_offset(processm.label_font), XtRString, "XtDefaultFont"},
        {XeNpixmapWidth, XeCPixmapWidth, XtRInt, sizeof(int),
         res_offset(processm.pixmap_width), XtRImmediate,
         (caddr_t)PROCESSM_DEFAULT_PIXMAP_WIDTH},
        {XeNpixmapHeight, XeCPixmapHeight, XtRInt, sizeof(int),
         res_offset(processm.pixmap_height), XtRImmediate,
         (caddr_t)PROCESSM_DEFAULT_PIXMAP_HEIGHT},
        {XeNpixmap, XeCPixmap, XtRPixmap, sizeof(Pixmap),
         res_offset(processm.pixmap), XtRImmediate, NULL},
        {XeNdrawColors, XeCDrawColors, XtRPointer, sizeof(XtPointer),
         res_offset(processm.input_colors), XtRImmediate, NULL},
        {XeNnumColors, XeCNumColors, XtRInt, sizeof(int),
         res_offset(processm.num_colors), XtRImmediate,
         (caddr_t)PROCESSM_DEFAULT_NUM_COLORS},
        {XeNsetColors, XeCSetColors, XtRBoolean, sizeof(Boolean),
         res_offset(processm.set_colors), XtRBoolean, (Boolean)FALSE},
        {XeNuseCancel, XeCUseCancel, XtRBoolean, sizeof(Boolean),
         res_offset(processm.use_cancel), XtRBoolean, (Boolean)FALSE},
        {XeNusePause, XeCUsePause, XtRBoolean, sizeof(Boolean),
         res_offset(processm.use_pause), XtRBoolean, (Boolean)FALSE},
        {XeNbuttonFont, XeCButtonFont, XtRFontStruct, sizeof(XFontStruct *),
         res_offset(processm.button_font), XtRString, "XtDefaultFont"},
        {XeNupdatePercent, XeCUpdatePercent, XtRInt, sizeof(int),
         res_offset(processm.update_percent), XtRImmediate, 
         (caddr_t)PROCESSM_DEFAULT_UPDATE_PERCENT},
        {XeNbackgroundColor, XeCBackgroundColor, XtRInt, sizeof(int),
         res_offset(processm.background_color), XtRImmediate,
         (caddr_t)PROCESSM_DEFAULT_BACKGROUND_COLOR},
        {XeNusePixmap, XeCUsePixmap, XtRBoolean, sizeof(Boolean),
         res_offset(processm.use_pixmap), XtRBoolean, (Boolean)FALSE},
};

/*
 *  Class methods.
 */

static void Initialize
(
    XeProcessMWidget request,
    XeProcessMWidget new
);
static void Destroy
(
    XeProcessMWidget w
);
static Boolean SetValues
(
    Widget  current,
    Widget  request,
    Widget  new
);

/*
 *  Action functions.
 */


/*
 *  Private callbacks.
 */

static void cancel_callback
(
    Widget w,
    caddr_t client,
    caddr_t call
)
{
    XeProcessMWidget widget = (XeProcessMWidget)XtParent(w);

    widget->processm.cancel_flag = TRUE;
}

static void pause_callback
(
    Widget w,
    caddr_t client,
    caddr_t call
)
{
    XeProcessMWidget widget = (XeProcessMWidget)XtParent(w);

    Arg args[25];

    Position x_pos;

    int argcnt;


    /*
     *  If pause was selected then change the name of the pause button
     *  and loop until resume is pressed.
     */


     if (widget->processm.pause_flag == FALSE)
     { 
       if (widget->processm.use_cancel)
         x_pos = (widget->core.width - widget->processm.cancel_width - GAP - 5);
       else
         x_pos = ((widget->core.width / 2)-(widget->processm.cancel_width / 2));

       argcnt = 0;
       XtSetArg(args[argcnt],XmNlabelString,widget->processm.resume_label);
        argcnt++;
       XtSetArg(args[argcnt], XmNx, x_pos); argcnt++;
       XtSetValues(widget->processm.pauseButton,args,argcnt);
       XFlush(XtDisplay(w));
      
       XtSetSensitive(widget->processm.cancelButton,FALSE);
       XFlush(XtDisplay(w));
       widget->processm.pause_flag = TRUE;
      }
     else
      {
       if (widget->processm.use_cancel)
        x_pos = (widget->core.width - widget->processm.pause_width - GAP - 5);
       else
        x_pos = ((widget->core.width / 2) - (widget->processm.pause_width / 2));

       argcnt = 0;
       XtSetArg(args[argcnt],XmNlabelString,widget->processm.pause_label);
        argcnt++;
       XtSetArg(args[argcnt], XmNx, x_pos); argcnt++;
       XtSetValues(widget->processm.pauseButton,args,argcnt);
       XFlush(XtDisplay(w));

       XtSetSensitive(widget->processm.cancelButton,TRUE);
       XFlush(XtDisplay(w));
       widget->processm.pause_flag = FALSE;
    }
}


/*
 *  Private support functions.
 */

/*
 *
 *  This routine is designed to allow the user to interrupt long
 *  processing operations under X.  It works by polling for a specific
 *  event periodically and forcing the server to act on it (namely the 
 *  selection of a cancel button).  This section of code is from original 
 *  work done by Dan Heller and has been adapted for use in DLG-E.  The 
 *  work is freely distributable -- DA.
 *
 */

static Boolean CheckForInterrupt
(
    XeProcessMWidget processm_widget
)
{
    XEvent event;

    Window cancel_win = XtWindow(processm_widget->processm.cancelButton),
           pause_win  = XtWindow(processm_widget->processm.pauseButton);


    /*
     *  Make sure all our requests get to the server.
     */

    XFlush(XtDisplay(processm_widget));

    /*
     *  Let Motif process all pending exposure events for us.
     */

    XmUpdateDisplay((Widget)processm_widget);

    /*
     *  Check the event queue for pending events.
     */

    while (XCheckMaskEvent(XtDisplay(processm_widget), EVENT_MASK, &event))
    {
       /*
        *  Check for event interest.
        */
       if ((event.xany.window == cancel_win) || 
           (event.xany.window == pause_win))
           XtDispatchEvent(&event); /* It's a cancel/pause button event! */
    }

    return(processm_widget->processm.cancel_flag);
}


/*
 *  Class storage definition.
 */

XeProcessMClassRec XeprocessMClassRec =
{
        { /* core_class variables */
         (WidgetClass) &xmBulletinBoardClassRec, /* ancestor */
         "XeProcessM",                          /* class name */
         sizeof(XeProcessMRec),                 /* widget size */
         NULL,                                  /* class initialize */
         NULL,                                  /* class part init. */
         FALSE,                                 /* class inited */
         Initialize,                            /* initialize */
         NULL,                                  /* initialize hook */
         XtInheritRealize,                      /* realize */
         NULL,                                  /* actions */
         0,                                     /* number of actions */
         resources,                             /* resources */
         XtNumber(resources),                   /* number of resources */
         NULLQUARK,                             /* xrm class */
         TRUE,                                  /* compress motions */
         XtExposeCompressMaximal,               /* compress exposures */
         TRUE,                                  /* compress enter/leave */
         FALSE,                                 /* visibility interest */
         Destroy,                               /* destroy */
         XtInheritResize,                       /* resize */
         XtInheritExpose,                       /* expose */
         SetValues,                             /* set values */
         NULL,                                  /* set values hook */
         XtInheritSetValuesAlmost,              /* set values almost */
         NULL,                                  /* get values hook */
         NULL,                                  /* accept focus */
         XtVersion,                             /* version */
         NULL,                                  /* callback private */
         NULL,                                  /* translation table */
         XtInheritQueryGeometry,                /* query geometry */
         XtInheritDisplayAccelerator,           /* display accelerator */
         NULL,                                  /* extension */
        },
        { /* composite_class variables */
         XtInheritGeometryManager,              /* geometry manager */
         XtInheritChangeManaged,                /* change managed */
         XtInheritInsertChild,                  /* insert child */
         XtInheritDeleteChild,                  /* delete child */
         NULL,                                  /* extension */
        },
        { /* constraint_class fields */
         NULL,                                  /* subresources */
         0,                                     /* number of subresources */
         0,                                     /* record size */
         NULL,                                  /* initialize */
         NULL,                                  /* destroy */
         NULL,                                  /* set values */
         NULL,                                  /* extension */
        },
        { /* manager_class fields */
         XtInheritTranslations,                 /* default translations   */
         NULL,                                  /* syn_resources          */
         0,                                     /* num_syn_resources      */
         NULL,                                  /* syn_cont_resources     */
         0,                                     /* num_syn_cont_resources */
         NULL,                                  /* parent_process         */
         NULL,                                  /* extension              */
        },
        { /* bulletinBoard class fields */
         FALSE,                                 /*always_install_accelerators*/
         NULL,                                  /* geo_matrix_create */
         NULL,/*_XmBulletinBoardFocusMoved,*/           /* focus_moved_proc */
         NULL                                   /* extension */
        },
        { /* processM class fields */
         0,
        },
 }; /* XeprocessMClassRec */

WidgetClass xeProcessMWidgetClass = (WidgetClass) &XeprocessMClassRec;

/*
 *  XeProcessMWidget methods.
 */

/*
 *  Initialize() initializes each instance of a the processing message
 *  widget.
 */

static void Initialize
(
    XeProcessMWidget request,
    XeProcessMWidget new
)
{
    Arg args[512];                      /* Xt resource argument list.           */
     
    XmString xms,                       /* Working Motif strings.               */
         mstring;
                    /* Standard X variable/definitions. */
    Display *display = XtDisplay(new);

    Screen *scr_ptr = XtScreen(new);

    int scr_num = XScreenNumberOfScreen(scr_ptr);

    Window root = RootWindow(display,scr_num);

    unsigned int depth = DefaultDepthOfScreen(scr_ptr);

    Dimension cancel_width,             /* Working width/height variables.      */
          pause_width,
          shadow_width,
          button_height;

    XGCValues values;           /* GC attribute variables.              */

    unsigned long valuemask = 0;

    int argcnt,                 /* Xt resource argument count.          */
        work1,                  /* Working bulletinboard width variables.*/
        work2;

    int i;                              /* Working loop index.                  */


    /*
     *  Initialize working pointers.
     */

    xms = mstring = NULL;

    /*
     *  Check the validity of resources.
     */

    if (new->processm.pixmap_width < 0)
    {
       fprintf(stderr,"ProcessM: Bad pixmap width specified: %d\n",
            new->processm.pixmap_width);

       new->processm.pixmap_width = PROCESSM_DEFAULT_PIXMAP_WIDTH;
    }

    if (new->processm.pixmap_height < 0)
    {
       fprintf(stderr,"ProcessM: Bad pixmap height specified: %d\n",
            new->processm.pixmap_height);

       new->processm.pixmap_height = PROCESSM_DEFAULT_PIXMAP_HEIGHT;
    }

    if (new->processm.num_colors > PROCESSM_MAX_COLORS)
    {
       fprintf(stderr,"ProcessM: Too many colors specified\n");

       new->processm.num_colors = PROCESSM_DEFAULT_NUM_COLORS;
    }

    if (new->processm.num_colors < 0)
    {
       fprintf(stderr,"ProcessM: Negative number of colors specified\n");

       new->processm.num_colors = PROCESSM_DEFAULT_NUM_COLORS;
    }

    if (new->processm.update_percent > 100)
    {
       fprintf(stderr,"ProcessM: Update percentage too large\n");

       new->processm.update_percent = PROCESSM_DEFAULT_UPDATE_PERCENT;
    }

    if (new->processm.update_percent < 1)
    {
       fprintf(stderr,"ProcessM: Update percentage too small\n");

       new->processm.update_percent = PROCESSM_DEFAULT_UPDATE_PERCENT;
    }

    /*
     *  Set initial flags for possible processing.
     */

    new->processm.init_flag = FALSE;

    new->processm.pause_flag = FALSE;

    /*
     *  If colors have been specified then copy them to the internal array.
     */

    if (new->processm.set_colors)
    {
       for (i = 0; i < new->processm.num_colors; i++)
           new->processm.draw_colors[i] = new->processm.input_colors[i];

       new->processm.set_colors = FALSE;
    }

    /*
     *  Create a GC for process message drawing.
     */

    if (new->processm.background_color == PROCESSM_DEFAULT_BACKGROUND_COLOR)
         new->processm.background_color = 
                BlackPixel(display,DefaultScreen(display));

    values.foreground = values.background = new->processm.background_color;
    values.plane_mask = AllPlanes;
    valuemask = (GCForeground | GCBackground | GCPlaneMask);
     
    new->processm.gc = XCreateGC(display,root,valuemask,&values);

    /*
     *  Create a Motif string for the processing message label.
     */

    if (((int)strlen(new->processm.label_string)) > MAX_LABEL_SIZE)
        fprintf(stderr,"ProcessM: Label string too long, being truncated.\n");

    strncpy(new->processm.label_string_store, new->processm.label_string,
            MAX_LABEL_SIZE);
    new->processm.label_string_store[MAX_LABEL_SIZE] = '\0';

    mstring = XmStringCreateLtoR(new->processm.label_string_store,
        XmSTRING_ISO8859_1);

    /*
     *  Calculate the dimensions of the bulletinboard based on the
     *  pixmap size, label and font sizes.
     */

    new->processm.label_font_list = XmFontListCreate(new->processm.label_font,
                                    XmSTRING_ISO8859_1);

    new->processm.button_font_list =XmFontListCreate(new->processm.button_font,
                    XmSTRING_ISO8859_1);

    xms = XmStringCreateLtoR(CANCEL,XmSTRING_ISO8859_1);

    work1 = (GAP + new->processm.pixmap_width + GAP + 
            XmStringWidth(new->processm.label_font_list,mstring) + GAP);

    work2 = (GAP + XmStringWidth(new->processm.button_font_list,xms) + GAP +
        XmStringWidth(new->processm.button_font_list,xms) + GAP);

    new->processm.bulletinboard_width = (work1 > work2) ? work1 : work2;

    XmStringFree(xms);
    xms = NULL;

    /*
     *  Set the width of the ProcessM widget.
     */

    new->core.width = new->processm.bulletinboard_width;

    /*
     *  Create the label widget for the processing message.
     */

    argcnt = 0;
    XtSetArg(args[argcnt], XmNfontList, new->processm.label_font_list); argcnt++;
    XtSetArg(args[argcnt], XmNlabelString, mstring); argcnt++;
    XtSetArg(args[argcnt], XmNx, (GAP + new->processm.pixmap_width + GAP));
    argcnt++;
    XtSetArg(args[argcnt], XmNy, (GAP + (new->processm.pixmap_height / 2) -
      (XmStringHeight(new->processm.label_font_list, mstring) / 2))); argcnt++;
    new->processm.messageLabel = XtCreateWidget("messageLabel",
                      xmLabelWidgetClass,
                      (Widget)new,
                      args,
                      argcnt);
    XtManageChild(new->processm.messageLabel);

    XmStringFree(mstring);
    mstring = NULL;

    /*
     *  Create a drawingarea for displaying processing progress or an icon.
     */

    argcnt = 0;
    XtSetArg(args[argcnt], XmNx, GAP); argcnt++;
    XtSetArg(args[argcnt], XmNy, GAP); argcnt++;
    XtSetArg(args[argcnt], XmNmarginWidth, 0); argcnt++;
    XtSetArg(args[argcnt], XmNmarginHeight, 0); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, new->processm.pixmap_width); argcnt++;
    XtSetArg(args[argcnt], XmNheight, new->processm.pixmap_height); argcnt++;
    new->processm.drawingArea = XtCreateWidget("drawingArea",
            xmDrawingAreaWidgetClass,
            (Widget)new,
            args,
            argcnt);
    XtManageChild(new->processm.drawingArea);

    /*
     *  If no pixmap was specified then create one and assign it to the
     *  drawingArea widget.
     */

    if (!new->processm.use_pixmap)
    {
       new->processm.pixmap = XCreatePixmap(display,root,
        new->processm.pixmap_width,
        new->processm.pixmap_height,depth);

       if (new->processm.pixmap == NULL)
        {
         fprintf(stderr,"ProcessM: Unable to allocate pixmap\n");
         exit(-1);
        }

       XFillRectangle(display,new->processm.pixmap,new->processm.gc,0,0,
        new->processm.pixmap_width,new->processm.pixmap_height);

    }

    argcnt = 0;
    XtSetArg(args[argcnt], XmNbackgroundPixmap, new->processm.pixmap); argcnt++;
    XtSetValues(new->processm.drawingArea,args,argcnt);

    /*
     *  Create a pushbutton to allow a cancel function as required.
     */

    argcnt = 0;
    XtSetArg(args[argcnt], XmNlabelString, (xms=XmStringCreateLtoR(
            CANCEL,XmSTRING_ISO8859_1))); argcnt++;
    XtSetArg(args[argcnt], XmNfontList, new->processm.button_font_list); argcnt++;
    XtSetArg(args[argcnt], XmNfillOnArm, False); argcnt++;
    new->processm.cancelButton = XtCreateWidget("cancelButton",
            xmPushButtonWidgetClass,
            (Widget)new,
            args,
            argcnt);
    XmStringFree(xms);
    xms = NULL;

    XtAddCallback(new->processm.cancelButton,XmNactivateCallback,
               (XtCallbackProc) cancel_callback, NULL);

    /*
     *  Create a pushbutton to allow a pause function if required.
     */

    new->processm.pause_label  = XmStringCreateLtoR(PAUSE,XmSTRING_ISO8859_1);
    new->processm.resume_label = XmStringCreateLtoR(RESUME,XmSTRING_ISO8859_1);
     
    argcnt = 0;
    XtSetArg(args[argcnt], XmNlabelString, new->processm.pause_label); argcnt++;
    XtSetArg(args[argcnt], XmNfontList, new->processm.button_font_list); 
    argcnt++;
    XtSetArg(args[argcnt], XmNfillOnArm, False); argcnt++;
    new->processm.pauseButton = XtCreateWidget("pauseButton",
                    xmPushButtonWidgetClass,
                    (Widget)new,
                    args,
                    argcnt);

    XtAddCallback(new->processm.pauseButton,XmNactivateCallback,
               (XtCallbackProc) pause_callback, NULL);

    /*
     *  Retrieve the dimensions of the pushbuttons.
     */

    argcnt = 0;
    XtSetArg(args[argcnt], XmNwidth, &cancel_width); argcnt++;
    XtSetArg(args[argcnt], XmNshadowThickness, &shadow_width); argcnt++;
    XtSetArg(args[argcnt], XmNheight, &button_height); argcnt++;
    XtGetValues(new->processm.cancelButton, args, argcnt);
    cancel_width += (2 * shadow_width);
    button_height += (2 * shadow_width);
    new->processm.cancel_width  = cancel_width;
    new->processm.button_height = button_height;

    /*
     *  Set the height of the ProcessM widget.
     */

    new->processm.bulletinboard_height =
            (GAP + new->processm.pixmap_height + GAP + button_height + GAP);

    new->core.height= new->processm.bulletinboard_height;

    /*
     *  Set the x/y locations and manage the buttons based on the flags set.
     */

    pause_width = cancel_width; /* Use cancel for both; for looks.      */
    new->processm.pause_width = pause_width;

    argcnt = 0;
    XtSetArg(args[argcnt], XmNy, (new->core.height - button_height - GAP));
        argcnt++;
    XtSetValues(new->processm.cancelButton, args, argcnt);
    XtSetValues(new->processm.pauseButton, args, argcnt);

    if (new->processm.use_cancel && new->processm.use_pause)
    {
       argcnt = 0;
       XtSetArg(args[argcnt], XmNx, GAP); argcnt++;
       XtSetValues(new->processm.cancelButton, args, argcnt);
       XtManageChild(new->processm.cancelButton);

       argcnt = 0;
       XtSetArg(args[argcnt], XmNx, 
        (new->core.width - pause_width - GAP)); argcnt++;
       XtSetValues(new->processm.pauseButton, args, argcnt);
       XtManageChild(new->processm.pauseButton);
    }
    else if (new->processm.use_cancel)
    {
       argcnt = 0;
       XtSetArg(args[argcnt], XmNx, 
        ((new->core.width / 2) - (cancel_width / 2))); argcnt++;
       XtSetValues(new->processm.cancelButton, args, argcnt);
       XtManageChild(new->processm.cancelButton);
    }
    else if (new->processm.use_pause)
    {
       argcnt = 0;
       XtSetArg(args[argcnt], XmNx, 
        ((new->core.width / 2) - (pause_width / 2))); argcnt++;
       XtSetValues(new->processm.pauseButton, args, argcnt);
       XtManageChild(new->processm.pauseButton);
    }
    else /* Neither are used. */
    {
       new->core.height = new->processm.bulletinboard_height = 
                (GAP + new->processm.pixmap_height + GAP);

       argcnt = 0;
       XtSetArg(args[argcnt], XmNx, GAP); argcnt++;
       XtSetValues(new->processm.cancelButton, args, argcnt);

       argcnt = 0;
       XtSetArg(args[argcnt], XmNx, 
        (new->core.width - pause_width - GAP)); argcnt++;
       XtSetValues(new->processm.pauseButton, args, argcnt);

    }
} /* Initialize */

/*
 *  Destroy() cleans up dynamic data structures.
 */
 
static void Destroy
(
    XeProcessMWidget w
)
{
 XtDestroyWidget(w->processm.messageLabel);
 XtDestroyWidget(w->processm.drawingArea);

 XtDestroyWidget(w->processm.cancelButton);

 XtDestroyWidget(w->processm.pauseButton);

 XmFontListFree(w->processm.label_font_list);
 XmFontListFree(w->processm.button_font_list);

 if (!w->processm.use_pixmap)
  {
   XFreePixmap(XtDisplay(w),w->processm.pixmap);
   w->processm.pixmap = NULL;
  }
} /* Destroy */
 
/*
 *  SetValues() handles modifications to resources.
 */

static Boolean SetValues
(
    Widget  current,
    Widget  request,
    Widget  new
)
{
                                /* Standard X variable/definitions.     */
Display *display = XtDisplay(new);
 
Screen *scr_ptr = XtScreen(new);
 
int scr_num = XScreenNumberOfScreen(scr_ptr);
 
Window root = RootWindow(display,scr_num);
 
unsigned int depth = DefaultDepthOfScreen(scr_ptr);

XmString xms,                   /* Working Motif strings.               */
         mstring;

Arg args[25];

Dimension cancel_width,         /* Working width/height variables.      */
          pause_width,
          shadow_width,
          button_height;

Boolean change_flag = FALSE,    /* Working updater flag for pixmaps.    */
        font_change = FALSE,    /* Flag for font change.                */
        label_change= FALSE;    /* Flag for label change.               */

static Dimension string_width,  /* Processing message string size.      */
                 string_height;

int argcnt,                     /* Xt resource argument count.          */
    work1,                      /* Working bulletinboard width variables.*/
    work2;

int i;                          /* Working loop index.                  */


Position new_x,                 /* Working position variables.          */
         new_y;
                                /* Convenience variables.               */

XeProcessMWidget new_widget = (XeProcessMWidget)new;
XeProcessMWidget old_widget = (XeProcessMWidget)current;


/*
 *  Initialize pointers.
 */

xms = mstring = NULL;

/*
 *  Handle a change in the message label string.
 */

 if (strcmp(old_widget->processm.label_string_store,
        new_widget->processm.label_string))
  {
   if (((int)strlen(new_widget->processm.label_string)) > MAX_LABEL_SIZE)
    fprintf(stderr,"ProcessM: Label string too long, being truncated.\n");
 
   strncpy(new_widget->processm.label_string_store, 
                new_widget->processm.label_string, MAX_LABEL_SIZE);
   new_widget->processm.label_string_store[MAX_LABEL_SIZE] = '\0';
 
   mstring = 
     XmStringCreateLtoR(new_widget->processm.label_string, XmSTRING_ISO8859_1);

   label_change = TRUE;
   change_flag = TRUE;
  }

/*
 * Handle a change in the message label font.
 */

 if (old_widget->processm.label_font != new_widget->processm.label_font)
  {
   XtUnmanageChild(new_widget->processm.messageLabel);
   XmFontListFree(new_widget->processm.label_font_list);
   new_widget->processm.label_font_list =
         XmFontListCreate(new_widget->processm.label_font,XmSTRING_ISO8859_1);

   argcnt = 0;
   XtSetArg(args[argcnt], XmNfontList, new_widget->processm.label_font_list);
        argcnt++;
   XtSetValues(new_widget->processm.messageLabel, args, argcnt);
   XtManageChild(new_widget->processm.messageLabel);
   font_change = TRUE;
   change_flag = TRUE;
  }

/*
 *  Update the message label string width if necessary.
 */

 if (change_flag) /* One of above must have changed. */
  {
   if ((label_change) && (font_change))         /* Both changed. */
    {
     string_width = XmStringWidth(new_widget->processm.label_font_list,
                        mstring);
     string_height= XmStringHeight(new_widget->processm.label_font_list,
                        mstring);
    }
   else if (label_change)                       /* Label changed.*/
    {
     string_width = XmStringWidth(new_widget->processm.label_font_list,
                        mstring);
     string_height= XmStringHeight(new_widget->processm.label_font_list,
                        mstring);
    }
   else                                         /* Font changed. */
    {
     xms = XmStringCreateLtoR(new_widget->processm.label_string, 
                XmSTRING_ISO8859_1);

     string_width = XmStringWidth(new_widget->processm.label_font_list, xms);
     string_height= XmStringHeight(new_widget->processm.label_font_list, xms);

     XmStringFree(xms);
    }
  }

/*
 *  Handle a change in the pixmap dimensions.
 */

 if ((old_widget->processm.pixmap_width != new_widget->processm.pixmap_width) ||
    (old_widget->processm.pixmap_height != new_widget->processm.pixmap_height))
  {
   if (new_widget->processm.pixmap_width < 0)
    {
     fprintf(stderr,"ProcessM: Bad pixmap width specified: %d\n",
                new_widget->processm.pixmap_width);
 
     new_widget->processm.pixmap_width = PROCESSM_DEFAULT_PIXMAP_WIDTH;
    }
 
   if (new_widget->processm.pixmap_height < 0)
    {
     fprintf(stderr,"ProcessM: Bad pixmap height specified: %d\n",
                new_widget->processm.pixmap_height);
 
     new_widget->processm.pixmap_height = PROCESSM_DEFAULT_PIXMAP_HEIGHT;
    }

   if (!new_widget->processm.use_pixmap)
    {
     new_widget->processm.pixmap = XCreatePixmap(display,root,
        new_widget->processm.pixmap_width,
        new_widget->processm.pixmap_height,depth);
 
     if (new_widget->processm.pixmap == NULL)
      {
       fprintf(stderr,"ProcessM: Unable to allocate pixmap\n");
       exit(-1);
      }
 
     XFillRectangle(display,new_widget->processm.pixmap,
        new_widget->processm.gc,0,0,new_widget->processm.pixmap_width,
        new_widget->processm.pixmap_height);
    }

   /*
    *  Change the size of the drawingarea to accomodate the new pixmap.
    */

   XtResizeWidget(new_widget->processm.drawingArea,
                (Dimension)new_widget->processm.pixmap_width,
                (Dimension)new_widget->processm.pixmap_height,
                (Dimension)new_widget->core.border_width);

   change_flag = TRUE;
  }

/*
 *  Update the pixmap.
 */

 if ((old_widget->processm.use_pixmap != new_widget->processm.use_pixmap) ||
     (change_flag))
  {
   if ((!old_widget->processm.use_pixmap) && 
       (old_widget->processm.pixmap != new_widget->processm.pixmap) &&
       (old_widget->processm.pixmap != NULL))
     XFreePixmap(display, old_widget->processm.pixmap);

   argcnt = 0;
   XtSetArg(args[argcnt], XmNbackgroundPixmap, new_widget->processm.pixmap);
        argcnt++;
   XtSetValues(new_widget->processm.drawingArea, args, argcnt);
  }

/*
 *  Handle a change in the number of colors.
 */

 if (new_widget->processm.num_colors > PROCESSM_MAX_COLORS)
  {
   fprintf(stderr,"ProcessM: Too many colors specified\n");
 
   new_widget->processm.num_colors = PROCESSM_DEFAULT_NUM_COLORS;
  }
 
 if (new_widget->processm.num_colors < 0)
  {
   fprintf(stderr,"ProcessM: Negative number of colors specified\n");
 
   new_widget->processm.num_colors = PROCESSM_DEFAULT_NUM_COLORS;
  }

 if (new_widget->processm.set_colors)
  {
   for (i = 0; i < new_widget->processm.num_colors; i++)
    new_widget->processm.draw_colors[i] = new_widget->processm.input_colors[i];

   new_widget->processm.set_colors = FALSE;
  }

/*
 *  Handle a change in the status of the cancel button.
 */

 if (old_widget->processm.use_cancel != new_widget->processm.use_cancel)
  {
   XtUnmanageChild(new_widget->processm.cancelButton);
   change_flag = TRUE;
  }

/*
 *  Handle a change in the status of the pause button.
 */

 if (old_widget->processm.use_pause != new_widget->processm.use_pause)
  {
   XtUnmanageChild(new_widget->processm.pauseButton);

   change_flag = TRUE;
  }

/*
 * Handle a change in the button label font.
 */
 
 if (old_widget->processm.button_font != new_widget->processm.button_font)
  {
   XmFontListFree(new_widget->processm.button_font_list);
   new_widget->processm.button_font_list =
         XmFontListCreate(new_widget->processm.button_font,XmSTRING_ISO8859_1);
 
   argcnt = 0;
   XtSetArg(args[argcnt], XmNfontList, new_widget->processm.button_font_list);
        argcnt++;
   XtSetValues(new_widget->processm.cancelButton, args, argcnt);
   XtSetValues(new_widget->processm.pauseButton, args, argcnt);

   change_flag = TRUE;
  }

/*
 *  Handle a change in the update percentage.
 */

 if (new_widget->processm.update_percent > 100)
  {
   fprintf(stderr,"ProcessM: Update percentage too large\n");
 
   new_widget->processm.update_percent = PROCESSM_DEFAULT_UPDATE_PERCENT;
  }
 
 if (new_widget->processm.update_percent < 1)
  {
   fprintf(stderr,"ProcessM: Update percentage too small\n");
 
   new_widget->processm.update_percent = PROCESSM_DEFAULT_UPDATE_PERCENT;
  }

/*
 *  Resize/reposition the internal widgets based on the changes specified.
 */

 if (change_flag)
  {
  /*
   *  Calculate the new height of the processing message widget.
   */

   argcnt = 0;
   XtSetArg(args[argcnt], XmNwidth, &cancel_width); argcnt++;
   XtSetArg(args[argcnt], XmNshadowThickness, &shadow_width); argcnt++;
   XtSetArg(args[argcnt], XmNheight, &button_height); argcnt++;
   XtGetValues(new_widget->processm.cancelButton, args, argcnt);
   cancel_width += (2 * shadow_width);
   button_height += (2 * shadow_width);
   new_widget->processm.pause_width = pause_width =
    new_widget->processm.cancel_width = cancel_width;   /* For looks. */
   new_widget->processm.button_height = button_height;

   new_widget->processm.bulletinboard_height =
        (GAP + new_widget->processm.pixmap_height + GAP + button_height + GAP);

/*
 *  Calculate the new width of the processing message widget.
 */

   xms = XmStringCreateLtoR(RESUME,XmSTRING_ISO8859_1);

   work1 = (GAP + new_widget->processm.pixmap_width + GAP + string_width + GAP);
   work2 = ((3*GAP) + (2*cancel_width));

   new_widget->processm.bulletinboard_width = (work1 > work2) ? work1 : work2;

   XmStringFree(xms);
   xms = NULL;

  /*
   *  Position the cancel and pause buttons.
   */

   new_y = (new_widget->processm.bulletinboard_height - button_height);

   if (new_widget->processm.use_cancel && new_widget->processm.use_pause)
    {
     new_x = GAP;
     XtMoveWidget(new_widget->processm.cancelButton, new_x, new_y);
     XtManageChild(new_widget->processm.cancelButton);

     new_x = (new_widget->processm.bulletinboard_width - pause_width); 
     XtMoveWidget(new_widget->processm.pauseButton, new_x, new_y);
     XtManageChild(new_widget->processm.pauseButton);
    }
   else if (new_widget->processm.use_cancel)
    {
     new_x = ((new_widget->processm.bulletinboard_width / 2) - 
                (cancel_width / 2));
     XtMoveWidget(new_widget->processm.cancelButton, new_x, new_y);
     XtManageChild(new_widget->processm.cancelButton);
    }
   else if (new_widget->processm.use_pause)
    {
     argcnt = 0;
     new_x = ((new_widget->processm.bulletinboard_width / 2) -
                (pause_width / 2));
     XtMoveWidget(new_widget->processm.pauseButton, new_x, new_y);
     XtManageChild(new_widget->processm.pauseButton);
    }
   else /* Neither are used. */
    {
       new_widget->processm.bulletinboard_height = (GAP + 
        new_widget->processm.pixmap_height + GAP);
    }

  /*
   *  Position the message label string.
   */

   XtResizeWidget(new, (Dimension)new_widget->processm.bulletinboard_width,
                (Dimension)new_widget->processm.bulletinboard_height,
                (Dimension)new_widget->core.border_width);

   new_x = (GAP + new_widget->processm.pixmap_width + GAP);
   new_y = (GAP + (new_widget->processm.pixmap_height / 2) - 
                        (string_height / 2));

   XtMoveWidget(new_widget->processm.messageLabel, new_x, new_y);

   if (mstring != NULL)
    {
     argcnt = 0;
     XtSetArg(args[argcnt], XmNlabelString, mstring);
        argcnt++;
     XtSetValues(new_widget->processm.messageLabel, args, argcnt);
 
     XmStringFree(mstring);
     mstring = NULL;
    }

   /*
    *  Resize the parent widget -- this is a bad practice because I assume
    *  that the parent widget is a transient shell or some similar widget.
    *  But this is the only way I found to resize the processing widget 
    *  without having internal widgets overlap occasionally.  When I find
    *  a better way I'll change this -- DA.
    */

   XtResizeWidget(XtParent(new),
                (Dimension)new_widget->processm.bulletinboard_width,
                (Dimension)new_widget->processm.bulletinboard_height,
                (Dimension)new_widget->core.border_width);
   XFlush(display);
  }

 return (change_flag);
} /* SetValues */

/*
 *  Support functions.
 */


/*
 *  Public functions.
 */

/*
 *  This function is used to initialize the ProcessM instance for
 *  message processing calculations/output.
 */

void XeProcessMInit
(
    XeProcessMWidget processm_widget,
    int iterator_start,
    int iterator_end
)
{
/*
 *  Perform basic validation of the input parameters.
 */

 if ((iterator_start < 0) || (iterator_end < 0))
  {
   fprintf(stderr,"ProcessM: iterators cannot be less than zero\n");
   return;
  }
 else if (iterator_end < iterator_start)
  {
   fprintf(stderr,
        "ProcessM: iterator end must be greater than iterator start\n");
   return;
  }
 else if (iterator_start == iterator_end)
  {
   fprintf(stderr,"ProcessM: iterators cannot be equal\n");
   return;
  }

/*
 *  Assign/initialize variables for the processing message.
 */

 processm_widget->processm.start = iterator_start;
 processm_widget->processm.end   = iterator_end;

 processm_widget->processm.old_angle = 90*64;
 processm_widget->processm.current_color = 0;

 processm_widget->processm.old = -1;

 processm_widget->processm.cancel_flag = FALSE;

/*
 *  Clear the working pixmap if required.
 */

 if (!processm_widget->processm.use_pixmap)
  {
   XSetForeground(XtDisplay(processm_widget),processm_widget->processm.gc,
                processm_widget->processm.background_color);

   XFillRectangle(XtDisplay(processm_widget),processm_widget->processm.pixmap,
        processm_widget->processm.gc,0,0,processm_widget->processm.pixmap_width,
        processm_widget->processm.pixmap_height);
  }

/*
 *  Set the initialized flag.
 */

 processm_widget->processm.init_flag = TRUE;
}

/*
 *  This module is used to perform the actual message processing for the
 *  ProcessM widget.  The programmer should place this call in the process
 *  loop so that the widget can calculate updates.
 */

int XeProcessMUpdate
(
    XeProcessMWidget processm_widget,
    int iterator_current
)
{
float pcnt;


/*
 *  Check if processing widget was previously initialized.
 */

 if (processm_widget->processm.init_flag != TRUE)
  {
   fprintf(stderr,"ProcessM: no initialization\n");
   return(-1);
  }

 if ((iterator_current == (processm_widget->processm.end - 1)) ||
     (iterator_current == (processm_widget->processm.end)))
   processm_widget->processm.init_flag = FALSE;

/*
 *  Calculate the percent of the processing completed.
 */

 pcnt = ((float)iterator_current / (float)processm_widget->processm.end);

 if (iterator_current == (processm_widget->processm.end - 1))
  pcnt = 1.0;

 processm_widget->processm.percent = (int)(pcnt * 100.0);

/*
 *  Update the processing message based on the amount of processing done.
 */
 if ((processm_widget->processm.percent % 
        processm_widget->processm.update_percent) == 0)
  {
   if (processm_widget->processm.percent != processm_widget->processm.old)
    {
     if (!processm_widget->processm.use_pixmap)
      {
       if (processm_widget->processm.num_colors > 0)
        {
         XSetForeground(XtDisplay(processm_widget),processm_widget->processm.gc,
          processm_widget->processm.draw_colors[
                processm_widget->processm.current_color]);

         if (processm_widget->processm.percent != 0)
          processm_widget->processm.current_color++;

         if (processm_widget->processm.current_color ==
                processm_widget->processm.num_colors)
          processm_widget->processm.current_color = 0;
        }
       else
        XSetForeground(XtDisplay(processm_widget),processm_widget->processm.gc,
          BlackPixel(XtDisplay(processm_widget),
          DefaultScreen(XtDisplay(processm_widget))));

       processm_widget->processm.new_angle = (short)(90*64-(pcnt * 360.0) * 64);

       XFillArc(XtDisplay(processm_widget),processm_widget->processm.pixmap,
        processm_widget->processm.gc,0,0,processm_widget->processm.pixmap_width,
        processm_widget->processm.pixmap_height,
        processm_widget->processm.old_angle,
        (processm_widget->processm.new_angle - 
                processm_widget->processm.old_angle));

       XCopyArea(XtDisplay(processm_widget),processm_widget->processm.pixmap,
        XtWindow(processm_widget->processm.drawingArea),
        processm_widget->processm.gc,0,0,
        processm_widget->processm.pixmap_width,
        processm_widget->processm.pixmap_height,0,0);

       XFlush(XtDisplay(processm_widget));
       XmUpdateDisplay(processm_widget->processm.drawingArea);

       processm_widget->processm.old_angle = 
                processm_widget->processm.new_angle;
      }

      processm_widget->processm.old = processm_widget->processm.percent;

     /*
      *  Check if an interrupt was requested.
      */

      if (CheckForInterrupt(processm_widget))
       {
        processm_widget->processm.init_flag = FALSE;
        return(-1);
       }

     /*
      *  Pause if requested.
      */

      while (processm_widget->processm.pause_flag)
         {
          CheckForInterrupt(processm_widget);
         };
    }
  }

 return(0);
}

/*
 *  This module is used to calculate the screen x and y coordinates for
 *  the ProcessM widget to allow the programmer to center the parent
 *  widget on the display screen if desired.
 */
 
void XeProcessMCenter
(
    XeProcessMWidget processm_widget,
    int *screen_x,
    int *screen_y
)
{
                                /* Standard X variable/definitions.     */
Display *display = XtDisplay(processm_widget);
 
Screen *scr_ptr = XtScreen(processm_widget);
 
int scr_num = XScreenNumberOfScreen(scr_ptr);
 

/*
 *  Calculate the return values.
 */

 *screen_x = ((DisplayWidth(display,scr_num)/2) -
                ((processm_widget->core.width)/2));
 *screen_y = ((DisplayHeight(display,scr_num)/2) -
                ((processm_widget->core.height)/2));

} 
