#include <ctype.h>   /* isspace prototype */
#include "tiepts.h"
#include "mapsup.h"  /* map_dms2deg prototype */
#include "dlgsup.h"
#include "convert.h" /* degrees2vect prototype */

/**** LLH  
 The record size of the DLG files was modified to 80 characters because the
 format document indicates that the logical record length is 80 bytes for
 distribution (there are 8 spaces of blank fill in bytes 73-80 of each
 record).  Some sources of DLG data separate records with a new line.
#define RECSIZE 72
 LLH ****/
/* J LaVergne - corrected a problem where UTM images worked OK but
   Geographic images didn't - see note below.                           */
/* T Rockvam - added a manual parameter noting whether the files are being
   loaded manually, or automatically using the All button.  All button
   added to load all selected files. Existing error messages are ignored
   in automatic mode                                                    */
/* T Rockvam 3/94 - removed the manual parameter (all dlg file loading is now
   automatic); extra error checking; check attributes to see if we should
   load a line                                                          */


#define RECSIZE 80
#define BUFSIZE (RECSIZE+1)

/* free memory allocated for a given file number in the tie_vectfile list */
static void free_vector_mem
(
    int file_num    /* file number in tie_vector structure to free */
);



int dlg_optread
(
    char *filename
)
{
   FILE         *fd;
   char         buf[BUFSIZE],   /* buffer containing one DLG record     */
                *bufptr;        /* pointer to "buf"                     */
   char         tmp[BUFSIZE]; /* temporary character string           */
   char         cplabel[4][3];  /* control point label                  */
   char         category[21];   /* DLG category                         */
   char         *D;             /* Pointer to exponent 'D' (FORTRAN)    */
   double       file2map[2][2], /* file to map proj parameters          */
                cplat[4],       /* control point latitude               */
                cplon[4],       /* control point longitude              */
                corners[4][2],  /* corner boundaries in vector coords   */
                cpx[4],         /* X corner coordinates of DLG file     */
                cpy[4],         /* Y corner coordinates of DLG file     */
                x,y;            /* temp for converting file to proj coords*/

   int          xin, yin, in=TRUE,
                                /* Flags indicating whether the current
                                   coordinate is within the defined 
                                   bounds                               */
                out;
   int          n, i,   /* loop counter                         */
                defaultID,      /* default ID for this category         */
                fnum,           /* shorthand variable for file index    */
                majatt,         /* major attribute code                 */
                minatt,         /* minor attribute code                 */
                narea,          /* number of areas in the file          */
                nattrib,        /* number of attribute pairs listed     */
                ncat,           /* number of categories in the file     */
                ncpts,          /* number of control points             */
                nline,          /* number of lines in the file          */
                node,           /* node index                           */
                nnode,          /* number of nodes in the file          */
                npts,           /* number of points in the line         */
                prevlines,      /* number of lines previously loaded    */
                check;          /* variable used to check the DLG format*/
   int temp_int;
   unsigned char 
                areacoord,      /* presence of area coord records       */
                area2line,      /* presence of area-to-line records     */
                area2node,      /* presence of area-to-node records     */
                node2area,      /* presence of node-to-area records     */
                node2line,      /* presence of node-to-line records     */
                linecoord;      /* presence of line coordinate lists    */

   if (tie_vector->numfiles >= tie_vector->maxfile)
   {
      struct TIE_VECTFILE *newvect;
      int files = tie_vector->maxfile + VECTOR_FILE_BLOCK_SIZE;
      
      /* max number of files has been reached, so grow the array */
      newvect = realloc(tie_vectfile, files * sizeof(struct TIE_VECTFILE));
      if (newvect == NULL)
      {
         fprintf(stderr,"Unable to allocate more memory for vector files.  "
                 "File %s ignored\n",filename);
         return(E_FAIL);
      }

      tie_vector->maxfile = files;
      tie_vectfile = newvect;
   }

  /*
   *  *** open file
   */
   if ((fd = fopen(filename, "r")) == NULL)
   {
      fprintf(stderr,"Error opening %s.\n",filename);
      return(E_FAIL);
   }
   
   fnum = tie_vector->numfiles;
   tie_vector->type = DLGOPT;
   prevlines = tie_vector->numlines;

  /*
   *  *** initialize vector file structure
   */
   tie_vectfile[fnum].startnode = -1;
   tie_vectfile[fnum].startarea = -1;
   tie_vectfile[fnum].endarea = -1;
   tie_vectfile[fnum].startline = -1;
   tie_vectfile[fnum].endline = -1;
   tie_vectfile[fnum].filename = NULL;
   
   /*
    *  *** Copy the corner coordinates from the previous file loaded
    *      to prevent having to read them from the X widget every time
    */
   if (fnum > 0)
   {
       for (i=0; i < 4; i++)
       {
           tie_vectfile[fnum].corner[i][0] = tie_vectfile[fnum-1].corner[i][0];
           tie_vectfile[fnum].corner[i][1] = tie_vectfile[fnum-1].corner[i][1];
       }
   }
   else
   {
/*     JLaVergne - added following line to fix problem where geographic 
       coords do not work, while UTM coords do.  Problem seems to be that the
       tie_vectfile[fnum].corner array is previously set somewhere, and the
       Lon values are not correct (need to subtract 360 if > 180).  The 
       following lines do the trick for now, simply using the coords from
       the first dlg file */
       tie_vectfile[fnum].corner[0][1] = 
                          map_dms2deg(XmTextGetString(vect_max_lat));
       tie_vectfile[fnum].corner[0][0] = 
                          map_dms2deg(XmTextGetString(vect_min_lon));
       tie_vectfile[fnum].corner[3][1] = 
                          map_dms2deg(XmTextGetString(vect_min_lat));
       tie_vectfile[fnum].corner[3][0] = 
                          map_dms2deg(XmTextGetString(vect_max_lon));

      /*
       *  *** extract bounds from vector panel -- note that the corner 
       *      coordinates array is stored in the order ul, ll, ur, lr.
       */
       tie_vectfile[fnum].corner[1][1] = 
                          map_dms2deg(XmTextGetString(vect_min_lat));
       tie_vectfile[fnum].corner[1][0] = 
                          map_dms2deg(XmTextGetString(vect_min_lon));
       tie_vectfile[fnum].corner[2][1] = 
                          map_dms2deg(XmTextGetString(vect_max_lat));
       tie_vectfile[fnum].corner[2][0] = 
                          map_dms2deg(XmTextGetString(vect_max_lon));
   }

  /*
   *  *** read first 4 records
   */
   for (i = 0; i < 4; i++)
   {
      if (fgets(buf, BUFSIZE, fd) == NULL)
      {
         fclose(fd);
         return(E_FAIL);
      }
   }

  /*
   * *** check to make sure this is a readable record.
   */
   sscanf(buf,"%d",&check);
   if (check != 3)
   {
        fclose(fd);
        return(E_FAIL);
   }
   if ((sscanf(buf+6,  "%d", &tie_vectfile[fnum].inproj) != 1) ||
       (sscanf(buf+12, "%d", &tie_vectfile[fnum].inzone) != 1) ||
       (sscanf(buf+18, "%d", &tie_vectfile[fnum].inunit) != 1) ||
       (sscanf(buf+54, "%d %d", &ncpts, &ncat) != 2))
   {
      fprintf(stderr, "Error parsing record #4.\n");
      fclose(fd);
      return(E_FAIL);
   }

   /* read the datum from the current record at +66 */
   /* byte +66 - +68:
            'b' or 0 = NAD 27 (225)   where 'b' = blank
                   1 = NAD 83 (219)
                   2 = Puerto Rico  (273)
                   3 = Old Hawaiian (254)
                   4 = Local (Astro)  issue a warning and skip the file
       anything else = issue a warning and skip the file
   */
   if ((strlen(buf) < 67) || (sscanf(buf+66,"%3d", &temp_int) != 1))
   {
      /* sscanf failed, so field was blank which defaults to datum 225 */
      tie_vectfile[fnum].indatum = 225;
   }
   else
   {
      /* convert the datum read into a LAS datum */
      switch (temp_int)
      {
        case 0:  tie_vectfile[fnum].indatum = 225; break;
        case 1:  tie_vectfile[fnum].indatum = 219; break;
        case 2:  tie_vectfile[fnum].indatum = 273; break;
        case 3:  tie_vectfile[fnum].indatum = 254; break;
        default:
            fprintf(stderr, "Warning: DLG file %s did not contain a legal "
                    "datum - file skipped", filename);
            fclose(fd);
            return(E_FAIL);
      }
   }

  /*
   *  *** extract project parameters
   */
   for (i = 5; i < 10; i++)
   {
      if (fgets(buf, BUFSIZE, fd) == NULL)
      {
         fclose(fd);
         return(E_FAIL);
      }

      while ((D = strchr(buf, 'D')) != NULL)
         D[0] = 'e';
 
      if (sscanf(buf, "%le %le %le", 
             &tie_vectfile[fnum].inparm[(i-5)*3],
             &tie_vectfile[fnum].inparm[(i-5)*3+1],
             &tie_vectfile[fnum].inparm[(i-5)*3+2]) != 3)
      {
          fprintf(stderr, "Error parsing record #%d.\n",i);
          fclose(fd);
          return(E_FAIL);
      }
   }

  /*
   *  *** extract file to map proj transformation parameters (record #10)
   */
   if (fgets(buf, BUFSIZE, fd) == NULL)
   {
      fclose(fd);
      return(E_FAIL);
   }

   while ((D = strchr(buf, 'D')) != NULL)
      D[0] = 'e';
   if (sscanf(buf, "%lf %lf %lf %lf",  &file2map[0][0], &file2map[0][1],
              &file2map[1][0], &file2map[1][1]) != 4)
   {
      fprintf(stderr, "Error parsing record #10.\n");
      fclose(fd);
      return(E_FAIL);
   }

  /*
   *  *** read control points
   */
   for (i = 0; i < ncpts; i++)
   {
      if (fgets(buf, BUFSIZE, fd) == NULL)
      {
         fclose(fd);
         return(E_FAIL);
      }

      if ((sscanf(buf, "%2s", cplabel[i]) != 1) ||
          (sscanf(buf+5, "%lf %lf %lf %lf", &cplat[i], &cplon[i], 
                  &cpx[i],&cpy[i]) != 4))
      {
         fprintf(stderr, "Error parsing control point record #%d.\n",i+1);
         fclose(fd);
         return(E_FAIL);
      }
   }

  /*
   * Convert the corner coordinates to UTM now that the DLG projection
   * information has been read.
   */
   for (i=0; i < 4; i++)
   {
       corners[i][0] = tie_vectfile[fnum].corner[i][0];
       corners[i][1] = tie_vectfile[fnum].corner[i][1];
   }
   degrees2vect(corners[0],4,fnum);

  /*
   * Determine whether or not any of the points in the file fall within the
   * area of interest.
   */
   out = FALSE;
   if ((cpx[0] > corners[2][0]) && (cpx[0] > corners[3][0]) && (cpx[1] >
       corners[2][0]) && (cpx[1] > corners[3][0]))
       out = TRUE;
   else if ((cpx[2] < corners[0][0]) && (cpx[2] < corners[1][0]) && (cpx[3] >
       corners[0][0]) && (cpx[3] > corners[1][0]))
       out = TRUE;
   else if ((cpy[0] > corners[0][1]) && (cpy[0] > corners[2][1]) && (cpy[3] >
       corners[0][1]) && (cpy[3] > corners[2][1]))
       out = TRUE;
   else if ((cpy[1] < corners[1][1]) && (cpy[1] < corners[3][1]) && (cpy[2] >
       corners[1][1]) && (cpy[2] > corners[3][1]))
       out = TRUE;
   if (out)
   {
       fclose(fd);
       return(DLGREAD_WRONG_AREA);         
   }

  /*
   *  ***  read data category records
   */
   for (i = 0; i < ncat; i++)
   {
      if (fgets(buf, BUFSIZE, fd) == NULL)
      {
         fclose(fd);
         free_vector_mem(fnum);
         return(E_FAIL);
      }

      if ((sscanf(buf, "%20s",  category) != 1) ||
          (sscanf(buf+30, "%d", &nnode) != 1) ||
          (sscanf(buf+37, "%c", &node2area) != 1) ||
          (sscanf(buf+38, "%c", &node2line) != 1) ||
          (sscanf(buf+46, "%d", &narea) != 1) ||
          (sscanf(buf+53, "%c", &area2node) != 1) ||
          (sscanf(buf+54, "%c", &area2line) != 1) ||
          (sscanf(buf+55, "%c", &areacoord) != 1) ||
          (sscanf(buf+62, "%d", &nline) != 1) ||
          (sscanf(buf+71, "%c", &linecoord) != 1))
      {
         fprintf(stderr, "Error parsing data category record #%d.\n",i+1);
         fclose(fd);
         free_vector_mem(fnum);
         return(E_FAIL);
      }
    
     /*
      *  *** determine default ID code
      */
      if ((defaultID = dlg_defaultID(category)) < 0)
         continue;

      while (fgets(buf, BUFSIZE, fd) != NULL)
      {
        /*
         *  *** if area, read area record
         */
         if (buf[0] == 'A')
         {
           /*
            *  *** get number of points and allocate space
            */
            if ((sscanf(buf+36, "%d", &npts) != 1) ||
                (sscanf(buf+48, "%d", &nattrib) != 1))
            {
               fprintf(stderr, "Error parsing data category record #%d.\n",i+1);
               fclose(fd);
               free_vector_mem(fnum);
               return(E_FAIL);
            }
#ifdef SAVEAREA
            /* tjr
            long savearea;
            if (dlg_checkattrib(defaultID,
                 &tie_vector->area[tie_vector->numareas].attrib))
                  savearea = TRUE;
            else
                  savearea = FALSE;
            */
            tie_vector->area[tie_vector->numareas].npts = npts;
            dlg_setattrib(&tie_vector->area[tie_vector->numareas].attrib, 
                          defaultID, defaultID);
            tie_vector->area[tie_vector->numareas].nodes = 
                         (int*)malloc(npts*sizeof(int));
#endif

           /*
            *  *** if present, read area-to-node linkage record
            */
            if (area2node == '1')
            {
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }
            }
   
           /*
            *  *** if present, read area-to-line linkage record
            */
            if (area2line == '1')
            {
               n = 0;
               while (n < npts)
               {
                   if (fgets(buf, BUFSIZE, fd) == NULL)
                   {
                      fclose(fd);
                      free_vector_mem(fnum);
                      return(E_FAIL);
                   }
                   bufptr = buf;
                   while(*bufptr != '\0')
                   {
                       if (!(isspace(*bufptr)))
                       {
                            if ((sscanf(bufptr,"%d",&node) != 1) ||
                                (sscanf(bufptr,"%s",tmp) != 1))
                            {
                               fprintf(stderr, "Error parsing data category "
                                       "record #%d.\n",i+1);
                               fclose(fd);
                               free_vector_mem(fnum);
                               return(E_FAIL);

                            }
#ifdef SAVEAREA
                            /*tjr if (savearea) */
                            tie_vector->area[tie_vector->numareas].nodes[n] = 
                                abs(node);
#endif
                            bufptr += strlen(tmp);
                            n++;
                       }
                       else
                            bufptr++;
                   }
               }

#ifdef SAVEAREA
             /*tjr if (savearea) */
               if (tie_vectfile[fnum].startarea < 0)
               {
                  tie_vectfile[fnum].startarea = 
                  tie_vectfile[fnum].endarea = tie_vector->numareas;
               }
               else
                  tie_vectfile[fnum].endarea++;

               tie_vector->numareas++;
#endif

              /*
               *  *** read to end of line
               */
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }

              /*
               *  *** if present, read area coordinate record
               */
               if (areacoord == '1')
               {
                  if (fgets(buf, BUFSIZE, fd) == NULL)
                  {
                     fclose(fd);
                     free_vector_mem(fnum);
                     return(E_FAIL);
                  }
               }
            }
   
           /*
            *  *** if present, read area coordinate record
            */
            if (areacoord == '1')
            {
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }
            }
         }
   
        /*
         *  *** if node, read node record
         */
         else if (buf[0] == 'N')
         {
            if (sscanf(buf+48, "%d", &nattrib) != 1)
            {
               fprintf(stderr, "Error parsing data category record #%d.\n",i+1);
               fclose(fd);
               free_vector_mem(fnum);
               return(E_FAIL);
            }
#ifdef SAVENODE
            if (sscanf(buf+6, "%lf %lf", 
                       &tie_vector->node[tie_vector->numnodes*2],
                       &tie_vector->node[tie_vector->numnodes*2+1]) != 2)
            {
               fprintf(stderr, "Error parsing data category record #%d.\n",i+1);
               fclose(fd);
               free_vector_mem(fnum);
               return(E_FAIL);
            }

            if (tie_vectfile[fnum].startnode < 0)
            {
               tie_vectfile[fnum].startnode = 
               tie_vectfile[fnum].endnode = tie_vector->numnodes;
            }
            else
               tie_vectfile[fnum].endnode++;

            tie_vector->numnodes++;
#endif

           /*
            *  *** if present, read node-to-area linkage record
            */
            if (node2area == '1')
            {
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }
            }
   
           /*
            *  *** if present, read node-to-line linkage record
            */
            if (node2line == '1')
            {
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }
            }

           /*
            *  *** read line of attributes
            */
            if (nattrib > 0)
            {
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }
            }
         }
   
        /*
         *  *** if line, read line records
         */
         else if (buf[0] == 'L')
         {
            if ((sscanf(buf+42, "%d", &npts) != 1) ||
                (sscanf(buf+48, "%d", &nattrib) != 1))
            {
               fprintf(stderr, "Error parsing data category record #%d.\n",i+1);
               fclose(fd);
               free_vector_mem(fnum);
               return(E_FAIL);
            }

           
           /*
            *  *** allocate x and y for each point
            */
            tie_vector->line[tie_vector->numlines].pts = 
                             (double*)malloc(npts*2*sizeof(double));
            if (tie_vector->line[tie_vector->numlines].pts == NULL)
            {
               fclose(fd);
               free_vector_mem(fnum);
               fprintf(stderr,"Memory allocation error in %s at %d.\n",
                       __FILE__, __LINE__);
               return(E_FAIL);
            }
            tie_vector->line[tie_vector->numlines].npts = npts;

            if (tie_vectfile[fnum].startline < 0)
            {
               tie_vectfile[fnum].startline = 
               tie_vectfile[fnum].endline = tie_vector->numlines;
            }
            else
               tie_vectfile[fnum].endline++;

            tie_vector->numlines++;

           /*
            *  *** if maximum allocated, allocate new block
            */
            if (tie_vector->numlines == tie_vector->maxline)
            {
               struct TIE_LINE *new_ptr;
               
               tie_vector->maxline += LINEBLOCK;
               new_ptr = (struct TIE_LINE*) realloc(tie_vector->line,
                                  tie_vector->maxline*sizeof(struct TIE_LINE));
               if (new_ptr == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  fprintf(stderr,"Memory allocation error in %s at %d.\n",
                          __FILE__, __LINE__);
                  return(E_FAIL);
               }
               tie_vector->line = new_ptr;
            }

           /*
            *  *** read each coordinate pair
            */
            in = FALSE;
            n = 0;
            while (n < npts * 2)
            {
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }
               bufptr = buf;
               while(*bufptr != '\0')
               {
                  if (!(isspace(*bufptr)))
                  {
                     if (sscanf(bufptr,"%lf",
                         &tie_vector->line[tie_vector->numlines-1].pts[n]) != 1)
                     {
                         fprintf(stderr, "Error parsing data category "
                                 "record #%d.\n",i+1);
                         fclose(fd);
                         free_vector_mem(fnum);
                         return(E_FAIL);
                     }

                     if (n % 2)      /* y is found, convert both x & y */
                     {
                        x=tie_vector->line[tie_vector->numlines-1].pts[n-1];
                        y=tie_vector->line[tie_vector->numlines-1].pts[n];
                        tie_vector->line[tie_vector->numlines-1].pts[n-1] =
                                file2map[0][0] * x + file2map[0][1] * y + 
                                file2map[1][0];
                        tie_vector->line[tie_vector->numlines-1].pts[n] =
                                file2map[0][0] * y - file2map[0][1] * x + 
                                file2map[1][1];
                     }
                     if (sscanf(bufptr,"%s",tmp) != 1)
                     {
                         fprintf(stderr, "Error parsing data category "
                                 "record #%d.\n",i+1);
                         fclose(fd);
                         free_vector_mem(fnum);
                         return(E_FAIL);
                     }
                     bufptr += strlen(tmp);
                     n++;
                  }
                  else
                     bufptr++;
               }
            }

            for (n = 0; n < npts; n++)
            {
                xin = yin = FALSE;
                if ((tie_vector->line[tie_vector->numlines-1].pts[n*2] >
                   corners[0][0]) && 
                   (tie_vector->line[tie_vector->numlines-1].pts[n*2] <
                   corners[2][0]))
                    xin = TRUE;
                if ((tie_vector->line[tie_vector->numlines-1].pts[n*2+1] >
                   corners[3][1]) && 
                   (tie_vector->line[tie_vector->numlines-1].pts[n*2+1] <
                   corners[0][1]))
                    yin = TRUE;
                 /*
                  * *** if any coordinate on the line is within the defined
                  * *** bounds, keep the entire line.
                  */
                 if (xin && yin)
                 {
                     in = TRUE;
                     break;
                 }
            }

           /*
            *  *** read attribute pairs (only first is used at this time)
            */
           /*
            *  *** check the enabled attributes to see if we should save
            *      this line
            */
            if (nattrib > 0)
            {
              /*
               * *** read to end-of-line
               */
               if (fgets(buf, BUFSIZE, fd) == NULL)
               {
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }

              /*
               *  *** check the specified attribute vs the enabled attributes 
               */
               if (sscanf(buf, "%d %d", &majatt, &minatt) != 2)
               {
                  fprintf(stderr, "Error parsing data category record #%d.\n",
                          i+1);
                  fclose(fd);
                  free_vector_mem(fnum);
                  return(E_FAIL);
               }

              /*
               *  *** check if we want this attribute
               */
               if (!dlg_checkattrib(majatt*1000 + minatt,
                        &tie_vector->line[tie_vector->numlines-1].attrib))
                  in = FALSE;
            }
            else
            {
              /*
               *  *** check the default attribute vs the enabled attributes 
               */
               if (!dlg_checkattrib(defaultID,
                    &tie_vector->line[tie_vector->numlines-1].attrib))
                  in = FALSE;
            }
            
            /*
             * *** if none of the coordinates of the line fell within the
             * *** the defined bounds, throw the last vector away
             */
             if (!in)
             {
                 free(tie_vector->line[tie_vector->numlines-1].pts);
                 tie_vector->numlines--;
                 tie_vectfile[fnum].endline--;
             }
         } /* end of reading line record */
      } /* end of reading file */
   } /* end of reading category */

   fclose(fd);

  /*
   * only increment number of files if we have read some more lines 
   */
   if (prevlines < tie_vector->numlines)
   {
      /* save the file name since it was a successful read - done here instead
         of at the start of the routine to prevent needing to free the memory
         if the file read fails */
      tie_vectfile[fnum].filename = (char*)malloc(strlen(filename)+1);
      strcpy(tie_vectfile[fnum].filename, filename);
      tie_vector->numfiles++;
      return(E_SUCC);
   }
   else
   {
      fprintf(stderr,"No lines found in %s.\n",filename);
      free_vector_mem(fnum);
      return(DLGREAD_NO_LINES);
   }
}   


/* free memory allocated for a given file number in the tie_vectfile list */
static void free_vector_mem
(
    int file_num    /* file number in tie_vector structure to free */
)
{
    int index;
    
#ifdef SAVEAREA
    /* free the memory allocated for the areas */
    for (index = tie_vectfile[file_num].startarea; 
         index < tie_vectfile[file_num].endarea; index++)
    {
        free (tie_vector->area[index].nodes);
    }
    
    /* reset the numlines to the value before attempting to read this file */
    if (tie_vectfile[file_num].startarea > -1)
        tie_vector->numareas = tie_vectfile[file_num].startarea;
#endif

    /* free the memory allocated for the lines */
    for (index = tie_vectfile[file_num].startline; 
         index < tie_vectfile[file_num].endline; index++)
    {
        free (tie_vector->line[index].pts);
    }

    /* reset the numlines to the value before attempting to read this file */
    if (tie_vectfile[file_num].startline > -1)
        tie_vector->numlines = tie_vectfile[file_num].startline;
        
    free (tie_vectfile[file_num].filename);
}
