#include <string.h>
#include "tiepts.h"
#include "ltable.h"
#include "dlgsup.h"

/*******************************************************************************
Purpose:
    This routines write the current set of lines to a labeled table.
    This data is written to a header file and line file.  The header
    file contains boundary and metadata describing the contents of the
    line file.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
  5.2           02/94   T. Rockvam        CSB       Modified from dcw_writelt
 
PROJECT:  LAS
*******************************************************************************/
void dlg_writelt
(
   char *hdrfile,           /* name of header labeled table file */
   struct TIE_LINE *dlg_line, /* current set of lines */
   int  numlines,           /* number of lines */
   double lllat,            /* lower left latitude */
   double lllon,            /* lower left longitude */
   double urlat,            /* upper right latitude */
   double urlon             /* upper right longitude */
)
{
   struct MATRIX size;      /* matrix structure of labeled table        */
   struct TAB_DEF htab;     /* tabular structure of header lable table*/
   struct TAB_DEF brtab;    /* tabular structure of bounding rect table */
   struct TAB_DEF latab;    /* tabular structure of line attrib table */
   struct TAB_DEF aitab;    /* tabular structure of line priority table */
   struct VECTOR *ptr;      /* pointer to each vector of labeled table  */
   char   brfile[CMLEN],    /* name of bounding rectangle table file*/
          *ext,             /* pointer to file extension            */
          lafile[CMLEN],    /* name of line type labeled table file */
          aifile[CMLEN],    /* name of line priority labeled table file */
          fdescbuf[80];     /* Data buffer                            */
   char htab_subfiles[3][CMLEN]; /* memory for the htab subfiles */ 
   int          attindex[MAXATTRIB],/* line attribute index               */
                linetype,       /* line dash type                         */
                linecolor,      /* line color                             */
                lnum,           /* line sequence number                   */
                seq;            /* sequence number                        */
   int          i, j;

  /*
   *  *** generate names of subfiles
   */
   strcpy(brfile, hdrfile);
   strcpy(lafile, hdrfile);
   strcpy(aifile, hdrfile);
   
   if ((ext=strrchr(brfile, '.')) != NULL)
      strcpy(ext, "_br.lt");
   else
      strcat(brfile, "_br.lt");

   if ((ext=strrchr(lafile, '.')) != NULL)
      strcpy(ext, "_la.lt");
   else
      strcat(lafile, "_la.lt");

   if ((ext=strrchr(aifile, '.')) != NULL)
      strcpy(ext, "_ai.lt");
   else
      strcat(aifile, "_ai.lt");

  /*-------------------------------------------------------------*/
  /* 
   *  ***  Initialize the header records
   */ 
   strcpy(fdescbuf, "Digital Line Graph Header");
   htab.fdesc = fdescbuf;
   htab.ftype = "DLGLT";

/******  Because of TPS projection problem
   htab.nsubs = 3;
 ****************************************/
   htab.nsubs = 0;
   htab.subfile[0] = htab_subfiles[0];
   strcpy(htab.subfile[0],brfile);
   htab.subfile[1] = htab_subfiles[1];
   strcpy(htab.subfile[1],lafile);
   htab.subfile[2] = htab_subfiles[2];
   strcpy(htab.subfile[2],aifile);
   size.nrow = 1;
   size.ncol = 1;

   open_tab(&htab,hdrfile,1);
   put_vector(&htab,"LINENUM",I4,"Line ID",1,1,FIELD_SEP2);
   put_vector(&htab,"X",R8,"Line Points",1,1,FIELD_SEP2);
   put_vector(&htab,"Y",R8,"Line Points",1,1,FIELD_SEP2);
   put_vector(&htab,"SEQ",I4, "Sequence Number",1,1,RECORD_SEP);
   
  /*
   *  *** output each point for each line
   */
   for (i = 0; i < numlines; i++)
   {
     /*
      *  *** if this line is not enabled, continue
      */

      for (j = 0; j < dlg_line[i].npts; j++)
      {
         ptr = htab.vector;
         lnum = i+1;
         put_matrix(ptr,I4,&size,(unsigned char *)&lnum);

         ptr = ptr->next;
         put_matrix(ptr,R8,&size,(unsigned char *)&dlg_line[i].pts[2*j]);

         ptr = ptr->next;
         put_matrix(ptr,R8,&size,(unsigned char *)&dlg_line[i].pts[2*j+1]);

         seq = j+1;
         ptr = ptr->next;
         put_matrix(ptr,I4,&size,(unsigned char *)&seq);
   
         put_record(&htab);
      }
   }

  /*----------------------------------------------------------------*/
  /*
   *  *** output bounding rectangle file
   */
   strcpy(fdescbuf, "Digital Line Graph Bounding Rectangle");
   brtab.ftype = "BOUNDING RECTANGLE";
   brtab.fdesc = fdescbuf;
   brtab.nsubs = 0;
   brtab.subfile[0] = '\0';
   open_tab(&brtab,brfile,IUPDATE);

  /*
   *  *** Open header labeled table file.
   */
   put_vector(&brtab,"YMAX",R8,
              "Upper left Y/latitude coordinate",1,1,FIELD_SEP2);
   put_vector(&brtab,"XMIN",R8,
              "Upper left X/longitude coordinate",1,1,FIELD_SEP2);
   put_vector(&brtab,"YMIN",R8,
              "Lower right Y/latitude coordinate",1,1,FIELD_SEP2);
   put_vector(&brtab,"XMAX",R8,
              "Lower right X/longitude coordinate",1,1,FIELD_SEP2);
   put_vector(&brtab,"NLINES",I4,
              "Number of Lines",1,1,RECORD_SEP);

  /*  
   *  *** Write the header values to the output matrix.
   */
   ptr = brtab.vector;
   put_matrix(ptr,R8,&size,(unsigned char *)&urlat);
 
   ptr = ptr->next;
   put_matrix(ptr,R8,&size,(unsigned char *)&lllon);
 
   ptr = ptr->next;
   put_matrix(ptr,R8,&size,(unsigned char *)&lllat);
 
   ptr = ptr->next;
   put_matrix(ptr,R8,&size,(unsigned char *)&urlon);
 
  /*
   *  *** write the number of enabled lines
   */
   ptr = ptr->next;
   put_matrix(ptr,I4,&size,(unsigned char *)&numlines);
   
  /*  
   *  *** Write the matrix record to the header labeled table file.
   */
   put_record(&brtab);

  /*-------------------------------------------------------------*/
  /*
   *  ***  write the line attribute table
   */
   strcpy(fdescbuf, "Digital Line Graph Line Attributes");
   latab.ftype = "ATTRIBUTES";
   latab.fdesc = fdescbuf;
   latab.nsubs = 0;
   latab.subfile[0] = '\0';
   open_tab(&latab,lafile,1);
 
   /*
   put_vector(&latab,"FEATURE",I4,"Feature",1,1,FIELD_SEP2);
   put_vector(&latab,"ATTINDEX",I4,"Attribute Index",1,1,FIELD_SEP2);
   */
   put_vector(&latab,"MINCODE",I4,"Minimum Code",1,1,FIELD_SEP2);
   put_vector(&latab,"MAXCODE",I4,"Maximum Code",1,1,FIELD_SEP2);
   put_vector(&latab,"COLOR",I4,"Line Color",1,1,FIELD_SEP2);
   put_vector(&latab,"LINESTYL",I4,"Line Style",1,1,FIELD_SEP2);
   put_vector(&latab,"DESCRIPTION",C,"description",1,50,RECORD_SEP);
 
   i = 0;
   j = 0;

  /*
   *  *** output attribute description for each selected line type
   */
   while (dlg_attrib[i].mincode != 0)
   {
      if (!dlg_attrib[i].present)
      {
         i++;
         continue;
      }

      linecolor = dlg_attrib[i].color;
      linetype  = dlg_attrib[i].style;
      size.ncol = 1;

      ptr = latab.vector;
      put_matrix(ptr,I4,&size,(unsigned char *)&dlg_attrib[i].mincode);

      ptr = ptr->next;
      put_matrix(ptr,I4,&size,(unsigned char *)&dlg_attrib[i].maxcode);
    
      ptr = ptr->next;
      put_matrix(ptr,I4,&size,(unsigned char *)&linecolor);

      ptr = ptr->next;
      put_matrix(ptr,I4,&size,(unsigned char *)&linetype);

      size.ncol = 50;
      ptr = ptr->next;
      strcpy(ptr->ptr.c, dlg_attrib[i].desc);

      put_record(&latab);

      attindex[i] = j; 
      j++;      

      i++;
   }
 
  /*-------------------------------------------------------------*/
  /*
   *  ***  write the line attribute index table
   */
   strcpy(fdescbuf, "Digital Line Graph Priorities");
   aitab.ftype = "ATTRIBUTE INDEX";
   aitab.fdesc = fdescbuf;
   aitab.nsubs = 0;
   aitab.subfile[0] = '\0';
   open_tab(&aitab,aifile,1);
   
   put_vector(&aitab,"LINENUM",I4,"Line ID",1,1,FIELD_SEP2);
   put_vector(&aitab,"ATTINDEX",I4,"Attribute Index",1,1,RECORD_SEP);
   size.ncol = 1;
   
  /*
   *  *** output the attribute index for each line
   */
   for (i = 1; i <= numlines; i++)
   {
      ptr = aitab.vector;
      put_matrix(ptr,I4,&size,(unsigned char *)&i);

      ptr = ptr->next;
      put_matrix(ptr,I4,&size,(unsigned char *)&attindex[dlg_line[i-1].attrib]);

      put_record(&aitab);
   }

   
   /*  Close the labeled table file
   --------------------------------*/
   close_tab(&brtab);
   close_tab(&latab);
   close_tab(&aitab);
   close_tab(&htab);
   
}
