#include <math.h>
#include "tiepts.h"
#include "labserv.h"
#include "ltable.h"
#include "mapsup.h"

/****************************************************************************
NAME:                           map_read

PURPOSE:   Reads a labeled table file of map parameters 

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.0           5/89    D. Steinwand       CSB      New development

PROJECT:  LAS

ALGORITHM DESCRIPTION:
        If the file exists, open it.  Otherwise return fail
        Retrieve file label
        Set up pointers to file fields
        Initialize boolean values
        Loop until key (mapid) is found or until EOF
           Get a record
           If found, pack map parameter fields and break out of loop
        Close the file
        Return success
*****************************************************************************/

int map_read
(
    char *mapfile,                      /* Map file name */
    char *mapname,                      /* Map name */
    char *mapid,                        /* Map id */
    struct TIE_MAPPARMS *map_params /* Map registration parameters */
)
{
    struct TAB_DEF tab;         /* Tabular structure of label table */
    struct VECTOR *mapnmptr;    /* Pointer to map name field */
    struct VECTOR *mapidptr;    /* Pointer to map id field */
    struct VECTOR *mapstptr;    /* Pointer to map state or country field */
    struct VECTOR *unitptr;             /* Pointer to units field */
    struct VECTOR *ul_yptr;             /* Pointer to ul corner y field */
    struct VECTOR *ul_xptr;             /* Pointer to ul corner x field */
    struct VECTOR *lr_yptr;             /* Pointer to lr corner y field */
    struct VECTOR *lr_xptr;             /* Pointer to lr corner x field */
    struct VECTOR *ver_yptr;    /* Pointer to verify pt y field */
    struct VECTOR *ver_xptr;    /* Pointer to verify pt x field */
    struct VECTOR *npts_yptr;   /* Pointer to npts in y field */
    struct VECTOR *npts_xptr;   /* Pointer to npts in x field */
    struct VECTOR *scaleptr;    /* Pointer to scale field */
    struct VECTOR *datumptr;    /* Pointer to datum field */
    struct VECTOR *typeptr;             /* Pointer to map type field */
    struct VECTOR *dateptr;             /* Pointer to map date field */
    struct VECTOR *descptr;             /* Pointer to map description field */
    int  no_key;                        /* Boolean--null map name? */
    int  no_id;                 /* Boolean--null map id? */
    int  status;                        /* Function return status */
    long mode = 0;                      /* Assess mode for lsstat */
    int  count = 0;                     /* Count of records read */
    static int  rec_num = 1;    /* Count of records found */

    /* Open the labeled table file & retrieve the label
      ------------------------------------------------*/
    if (c_lsstat(mapfile, &mode) != E_SUCC) return(E_FAIL);
    open_tab(&tab, mapfile, IREAD);
    get_vector(&tab);

    /* Determine pointers to fields in the file
      ----------------------------------------*/
    mapnmptr = get_field_ptr(&tab, "NAME");
    if (mapnmptr == NULL) { close_tab(&tab); return(E_FAIL); }
    mapidptr = get_field_ptr(&tab, "ID");
    if (mapidptr == NULL) { close_tab(&tab); return(E_FAIL); }
    mapstptr = get_field_ptr(&tab, "STATE_COUNTRY");
    if (mapstptr == NULL) { close_tab(&tab); return(E_FAIL); }
    unitptr = get_field_ptr(&tab, "UNITS");
    if (unitptr == NULL) { close_tab(&tab); return(E_FAIL); }
    ul_yptr = get_field_ptr(&tab, "UL_Y");
    if (ul_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
    ul_xptr = get_field_ptr(&tab, "UL_X");
    if (ul_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
    lr_yptr = get_field_ptr(&tab, "LR_Y");
    if (lr_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
    lr_xptr = get_field_ptr(&tab, "LR_X");
    if (lr_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
    ver_yptr = get_field_ptr(&tab, "VER_Y");
    if (ver_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
    ver_xptr = get_field_ptr(&tab, "VER_X");
    if (ver_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
    npts_yptr = get_field_ptr(&tab, "NPTS_Y");
    if (npts_yptr == NULL) { close_tab(&tab); return(E_FAIL); }
    npts_xptr = get_field_ptr(&tab, "NPTS_X");
    if (npts_xptr == NULL) { close_tab(&tab); return(E_FAIL); }
    scaleptr = get_field_ptr(&tab, "SCALE");
    if (scaleptr == NULL) { close_tab(&tab); return(E_FAIL); }
    datumptr = get_field_ptr(&tab, "DATUM");
    if (datumptr == NULL) { close_tab(&tab); return(E_FAIL); }
    typeptr = get_field_ptr(&tab, "TYPE");
    if (typeptr == NULL) { close_tab(&tab); return(E_FAIL); }
    dateptr = get_field_ptr(&tab, "DATE");
    if (dateptr == NULL) { close_tab(&tab); return(E_FAIL); }
    descptr = get_field_ptr(&tab, "DESC");
    if (descptr == NULL) { close_tab(&tab); return(E_FAIL); }

    /* Initialize BOOLEAN variables
      ----------------------------*/
    if (strcmp(mapname, "\0") == 0) no_key = TRUE;
    else 
       {
       no_key = FALSE;
       rec_num = 1;
       }

    if (strcmp(mapid, "\0") == 0) no_id = TRUE;
    else 
       no_id = FALSE;

    /* Loop until key is found or EOF
     -------------------------------*/
    for (;;)
       {
       status = get_record(&tab);
       if (status == E_FAIL) { close_tab(&tab); return(E_FAIL); }
       if (status == E_EOF) { rec_num = 1; close_tab(&tab); return(E_FAIL); }
       count++;
       if(((strcmp(mapnmptr->ptr.c, mapname) == 0) && (strcmp(mapidptr->ptr.c, 
          mapid) == 0)) || ((strcmp(mapnmptr->ptr.c, mapname) == 0) && no_id) || 
          ((no_key) && (count == rec_num)))
            {
            rec_num++;
            strcpy(mapname, mapnmptr->ptr.c);
            strcpy(map_params->mapname, mapnmptr->ptr.c);
            strcpy(map_params->mapid,   mapidptr->ptr.c);
            map_params->units = *unitptr->ptr.i4;
            map_params->ul_y = *ul_yptr->ptr.r8;
            map_params->ul_x = *ul_xptr->ptr.r8;
            map_params->lr_y = *lr_yptr->ptr.r8;
            map_params->lr_x = *lr_xptr->ptr.r8;
            map_params->verify_y = *ver_yptr->ptr.r8;
            map_params->verify_x = *ver_xptr->ptr.r8;
            map_params->npts_y = *npts_yptr->ptr.i4;
            map_params->npts_x = *npts_xptr->ptr.i4;
            map_params->inc_y = -fabs(map_params->ul_y - map_params->lr_y) /
                    (map_params->npts_y - 1);
            map_params->inc_x = fabs(map_params->ul_x - map_params->lr_x) / 
                    (map_params->npts_x - 1);

            map_params->scale = *scaleptr->ptr.i4;
            strcpy(map_params->datum, datumptr->ptr.c);
            strcpy(map_params->maptype, typeptr->ptr.c);

    /******
            if (strcmp(map_params->maptype, "manual") == 0) 
               manual = TRUE;
            else 
               manual = FALSE;
     ******/

            strcpy(map_params->mapdate, dateptr->ptr.c);
            strcpy(map_params->mapdesc, descptr->ptr.c);
            break;
            }
       }

    /* Close the file & return
      -----------------------*/
    close_tab(&tab);
    return(E_SUCC);
}

