/*
 *  This is the parm function source file for the parm panel of the
 *  Tie Point Selection application.  It contains the callbacks, 
 *  action routines, event handlers, and procedure code required 
 *  for the panel.
 *
 *  By Jim Hemmer, USGS/EROS Data Center, 2/92.
 *    - Tie Point Selection, v1.0.
 *  Modified by J LaVergne USGS/EROS Data Center 7/93 
 *    - make the 'Show Maps' button become enabled/disabled based on input mode
 *      (the same way the Register button works).
 *    - fixed typo: changed 'exits' to 'exist', and 'exist' to 'exists'
 *    - added an option to have the elevation values for the tiepoints to be
 *      required, forcing the user to enter a value.  Re-arranged the
 *      parameters panel to accommodate this.
 *    - Changed so that the Load button is reactivated after the Parameters 
 *      panel is closed, if the Select button is not pressed yet.
 *  Modified by T Rockvam, USGS/EROS Data Center, 3/94.
 *    - set flag for datum requirement
 */
#include "tiepts.h"
#include "proj.h"
#include <sys/file.h>
#include <sys/signal.h>
#include "appres.h"
#include "cr.h"
#include "digsup.h"
#include "tpssup.h"
#include "protos.h"


extern TieAppres tie_appres;

/*******************************************************************************
Purpose:
   This routine sets all tiepts parameters to their default values.
    
By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_init(void)
{
    /*
     *  *** initialize zoom factors and window size
     */
    tie_parms.sholab   = TRUE;
    if (tie_appres.fullres_window_size == 0)
    {
        if (tie.mode == IM2MAP)
            tie_parms.fwidth   = tie_parms.fheight = 512;
        else
            tie_parms.fwidth   = tie_parms.fheight = 256;
    }
    else
    {
        tie_parms.fwidth   = tie_parms.fheight = tie_appres.fullres_window_size;
    }
    tie_parms.zsize = tie_appres.zoom_window_size;
    tie_parms.refoff[0] = tie_parms.refoff[1] = tie_appres.refoff;
    tie_parms.srchoff[0] = tie_parms.srchoff[1] = tie_appres.srchoff;

    /*
     *  ***  Initialize the tie point selection filenames 
     */
    strcpy(tie_parms.intps[SEARCH],     "");
    strcpy(tie_parms.intps[REFERENCE],  "");
    strcpy(tie_parms.outtps[SEARCH] ,   "");
    strcpy(tie_parms.outtps[REFERENCE], "");
    strcpy(tie_parms.port, tie_appres.digitizer_port);
    if (tie_appres.elev_required)
        strcpy(tie_parms.epoint, "Required");
    else
        strcpy(tie_parms.epoint, "Not Required");

    /*
     *  *** set vector location
     */
    if (tie.mode == IM2VECT)
    {
        strcpy(tie_parms.cd,   tie_appres.vector_location);
        if (strcmp(tie_appres.vector_source,"WDB") == 0)
            tie_parms.vsource = WDB;
        else if (strcmp(tie_appres.vector_source,"DLG") == 0)
            tie_parms.vsource = DLGOPT;
        else
            tie_parms.vsource = DCW;
    }

    /*
     *  ***  Initialize the zoom parameter structure.
     */
    tie_parms.factor   = tie_appres.zoom_factor;
    strcpy(tie_parms.method,tie_appres.zoom_method);
    tie_parms.alpha = -0.5;
    tie_parms.order = tie_appres.order;

    /*
     *  ***  Initialize spark method
     */
    if (tie.mode == IM2MAP)
    { 
        if (strcmp(tie_appres.im2map_entry_mode,"Digitizer") == 0)
            tie_parms.manual = FALSE;
        else
            tie_parms.manual = TRUE;
        if (tie_appres.datum_required)
            strcpy(tie_parms.datumreq, "Required");
        else
            strcpy(tie_parms.datumreq, "Not Required");

        /* default to entry lat/long */
        tie_parms.flags = PARMS_LATLONG;
    }
}

/*******************************************************************************
Purpose:
    This routine closes the parameter window and sets the main menu 
    parameter option to sensitive.
    
By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to check the status from the tps_open call.
Modified by C. Olsen, USGS/EROS Data Center, 8/98
  - Modified to pop up a warning message if a coordinte type is not selected.
    (SRF #1935)
Modified by C. Olsen, USGS/EROS Data Center, 9/98
  - Modified to set the coordinate type in IM2VECT mode.  (SRF #1958)

Algorithm Description:
*******************************************************************************/
void parm_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
    Arg         args[10];       /* Variable used to set widget resourcees     */
    int         argcnt,
                changed,        /* general purpose modification flag          */
                curorder,       /* current LSQ order                          */
                i,              /* loop counter                               */
                rplane_idx,     /* memory plane of reference image            */
                splane_idx,     /* memory plane of search image               */
                status,         /* status indicator                           */
                tempzfact,      /* temp storage for zoom factor               */
                tempzsize;      /* temp storage for zoom size                 */
    char        *refin,
                *refout,
                *srchin,
                *srchout,
                *port,
                note[256];
    Widget      message,
                oc,
                order,
                vsrc,
                ztype,
                zsize,
                epoint,
                fullsize;
    XmString    xmstr;

    strcpy(note, "");
    message = NULL;

    /*
     * Close the file selector and set the load button on the main panel to 
     * sensitive.
     */

    if ((w == parm_accept) || (w == parm_apply))
    {
        /*
         *  *** set fullres size and redisplay
         */
        XtSetArg(args[0], XmNmenuHistory, &fullsize);
        XtGetValues(parm_fullres_option, args, 1);

        if ((fullsize == parm_fullres_256) && (tie_parms.fwidth != 256))
        {
            if (XtIsRealized(Shell_SrchFull))
                erase_fulloutline(search_image_bulletinboard);
            if (XtIsRealized(Shell_RefFull))
                erase_fulloutline(ref_image_bulletinboard);

            tie_parms.fwidth  = 256;
            tie_parms.fheight = 256;

            if (XtIsRealized(Shell_SrchFull))
                fullres_update(search_image_bulletinboard, 
                        &tie.num_srch_fullres,
                        tie_srch_fullres,
                        tie_srch_fullres[0].window[SL]+256,
                        tie_srch_fullres[0].window[SS]+256);
            if ((tie.mode == IM2IM) && XtIsRealized(Shell_RefFull))
                fullres_update(ref_image_bulletinboard, &tie.num_ref_fullres,
                        tie_ref_fullres, 
                        tie_ref_fullres[0].window[SL]+256,
                        tie_ref_fullres[0].window[SS]+256);
        }
        else if ((fullsize == parm_fullres_512) && (tie_parms.fwidth != 512))
        {
            tie_parms.fwidth  = 512;
            tie_parms.fheight = 512;

            if (XtIsRealized(Shell_SrchFull))
                fullres_update(search_image_bulletinboard, 
                        &tie.num_srch_fullres,
                        tie_srch_fullres,
                        tie_srch_fullres[0].window[SL]+128,
                        tie_srch_fullres[0].window[SS]+128);

            if ((tie.mode == IM2IM) && XtIsRealized(Shell_RefFull))
                fullres_update(ref_image_bulletinboard, &tie.num_ref_fullres,
                        tie_ref_fullres, 
                        tie_ref_fullres[0].window[SL]+128,
                        tie_ref_fullres[0].window[SS]+128);
        }

        /*
         *  *** set elevation point option 
         */
        XtSetArg(args[0], XmNmenuHistory, &epoint);
        XtGetValues(parm_epoint_option, args, 1);

        if (epoint == parm_epoint_rec)
            strcpy(tie_parms.epoint, "Required");
        if (epoint == parm_epoint_norec)
            strcpy(tie_parms.epoint, "Not Required");

        /*
         *  *** set zoom method
         */
        XtSetArg(args[0], XmNmenuHistory, &ztype);
        XtGetValues(parm_ztype_option, args, 1);

        if (ztype == parm_ztype_cc)
            strcpy(tie_parms.method, "CC");
        else if (ztype == parm_ztype_nn)
            strcpy(tie_parms.method, "NN");
        else if (ztype == parm_ztype_bilinear)
            strcpy(tie_parms.method, "BI");

        /*
         *  *** set vector coordinate type and data source
         */
        changed = FALSE;
        if (tie.mode == IM2VECT)
        {
            XtSetArg(args[0], XmNmenuHistory, &oc);
            XtGetValues(parm_oc_option, args, 1);

            /*
             *  *** get the output coordinate units
             */
            if (oc == parm_oc_proj)
            {
                tie_parms.flags = PARMS_PROJ;
            }
            else if (oc == parm_oc_latlong)
            {
                tie_parms.flags = PARMS_LATLONG;
            }

            XtSetArg(args[0], XmNmenuHistory, &vsrc);
            XtGetValues(parm_vsrc_option, args, 1);

            if ((vsrc == parm_vsrc_dlg) && (tie_parms.vsource != DLGOPT))
            {
                tie_parms.vsource = DLGOPT;
                changed = TRUE;
            }
            else if ((vsrc == parm_vsrc_wdb) && (tie_parms.vsource != WDB))
            {
                tie_parms.vsource = WDB;
                tie_vectfile[0].inproj = GEO;
                tie_vectfile[0].indatum = CLARKE;
                tie_vectfile[0].inunit = 4;               /* degrees */
                tie_vectfile[0].inzone =  NULZONE;

                for (i = 0; i < 15; i++)
                    tie_vectfile[0].inparm[i] = 0.0;

                changed = TRUE;
            }
            else if ((vsrc == parm_vsrc_dcw) && (tie_parms.vsource != DCW))
            {
                tie_parms.vsource = DCW;
                tie_vectfile[0].inproj = GEO;
                tie_vectfile[0].indatum = CLARKE;
                tie_vectfile[0].inunit = 4;               /* degrees */
                tie_vectfile[0].inzone =  NULZONE;

                for (i = 0; i < 15; i++)
                    tie_vectfile[0].inparm[i] = 0.0;

                changed = TRUE;
            }

            /*
             *  *** if source changed, destroy existing vector panel
             */
            if (changed) 
                vector_setsource();
        }

        /*
         *  ***  Get the user specified zoom factor, and verify it is valid.
         */
        tempzfact = tie_parms.factor;
        sscanf(XmTextGetString(parm_zfactor),"%d",&tie_parms.factor);

        tie_parms.factor = (tie_parms.factor > 16) ? 2  : tie_parms.factor;
        tie_parms.factor = (tie_parms.factor < 2)  ? 16 : tie_parms.factor;

        /*
         *  ***  set zoom alpha value.
         */
        sscanf(XmTextGetString(parm_alpha),"%f",&tie_parms.alpha);

        /*
         *  *** set zoom window size and redisplay
         */
        XtSetArg(args[0], XmNmenuHistory, &zsize);
        XtGetValues(parm_zsize_option, args, 1);

        tempzsize = tie_parms.zsize;

        if ((zsize == parm_zsize_128) && (tie_parms.zsize != 128))
            tie_parms.zsize = 128;
        else if ((zsize == parm_zsize_64) && (tie_parms.zsize != 64))
            tie_parms.zsize = 64;

        if (XtIsRealized(Shell_Zoom) && ((tempzsize != tie_parms.zsize) || 
           (tempzfact != tie_parms.factor)))
        {
            if ((tie_zoom[REFERENCE].flags & ZOOM_ACTIVE) &&
                    !(tie_zoom[REFERENCE].flags & ZOOM_CLEAR))
            {
                if (tie_zoom[REFERENCE].flags & ZOOM_MAIN)
                    zoom_image(ref_image_bulletinboard, 
                            zoom_refwin_bulletinboard,
                            tie_planes[tie.rview].dspwin);
                else
                    zoom_image(fullref_win_bulletinboard, 
                            zoom_refwin_bulletinboard,
                            tie_ref_fullres[0].dspwin);
            }

            if ((tie_zoom[SEARCH].flags & ZOOM_ACTIVE) &&
                    !(tie_zoom[SEARCH].flags & ZOOM_CLEAR))
            {
                if ((tie.mode == IM2VECT) && (tie_vector->blend_mask != NULL))
                {
                    XFreePixmap(tie.display, tie_vector->blend_mask);
                    tie_vector->blend_mask = NULL;
                }

                if (tie_zoom[SEARCH].flags & ZOOM_MAIN)
                    zoom_image(search_image_bulletinboard, 
                            zoom_srchwin_bulletinboard,
                            tie_planes[tie.sview].dspwin);
                else
                    zoom_image(fullsrch_win_bulletinboard, 
                            zoom_srchwin_bulletinboard,
                            tie_srch_fullres[0].dspwin);
            }
        }

        /*
         *  *** set LSQ fit order
         */
        XtSetArg(args[0], XmNmenuHistory, &order);
        XtGetValues(parm_order_option, args, 1);

        curorder = tie_parms.order;

        if ((order == parm_order_first) && (tie_parms.order != FIRST_ORDER))
            tie_parms.order = FIRST_ORDER;
        else if ((order == parm_order_second) && 
                (tie_parms.order != SECOND_ORDER))
            tie_parms.order = SECOND_ORDER;

        /*
         *  *** if order has change recompute or redisplay as necessary
         */
        if ((curorder != tie_parms.order) && (tie_rt.calc_rt != RT_SEL))
        {
            tie_rt.degree = tie_parms.order;
            tps_Qfit(tie_select.numtie, tie_select.rtie, tie_select.stie,
                    tie_select.disabled, &tie_rt.degree, tie_rt.line_coef,
                    tie_rt.samp_coef);

            if (tie.mode == IM2VECT)
            {
                if (XtIsRealized(Shell_SrchFull))
                    XClearArea(tie.display,XtWindow(fullsrch_win_bulletinboard),
                            0,0,0,0, TRUE);

                if (XtIsRealized(Shell_Zoom))
                    XClearArea(tie.display,XtWindow(zoom_srchwin_bulletinboard),
                            0,0,0,0, TRUE);

            }

            edit_compute();

            if (XtIsRealized(Shell_Edit))
                edit_display(FALSE);
        }

        refin   = XmTextGetString(parm_refin_text);
        srchin  = XmTextGetString(parm_srchin_text);
        refout  = XmTextGetString(parm_refout_text);
        srchout = XmTextGetString(parm_srchout_text);

        /*
         *  ***  Get the user specified zoom factor, verify it is valid and .
         */
        sscanf(XmTextGetString(parm_refoff_horiz),"%d",&tie_parms.refoff[0]);
        sscanf(XmTextGetString(parm_srchoff_horiz),"%d",&tie_parms.srchoff[0]);
        sscanf(XmTextGetString(parm_refoff_vert),"%d",&tie_parms.refoff[1]);
        sscanf(XmTextGetString(parm_srchoff_vert),"%d",&tie_parms.srchoff[1]);

        /*  
         *  *** Image to image specific coordinates
         */
        if (tie.mode == IM2IM)
            if (!(tie_parms.flags & PARMS_LINESAMP) && !(tie_parms.flags & 
                        PARMS_PROJ) && !(tie_parms.flags & PARMS_LATLONG))
            {
                sprintf (note, "Must choose an output coordinate type");  
                tie_message(Shell_Parm, note);
            }
        /*
         *  *** Image to map specific coordinates
         */
        if (tie.mode == IM2MAP)
        {
            XtVaGetValues(parm_spark_option, XmNmenuHistory, &epoint, NULL);
            if (epoint == parm_spark_manual)
            {
                if (!tie_parms.manual && (tie_mapparm.digid != NULL))
                {
                    XtRemoveInput(tie_mapparm.digid);
                    close(tie_mapparm.portfd);
                    tie_mapparm.digid = NULL;
                }

                if (tie.flags & TIE_SELECTING)
                    manual_display(w, client, call_data);

                XtSetSensitive(main_register_button, False);
                XtSetSensitive(main_map_button, False);              
                XtSetSensitive(main_select_button, True); 
                tie_parms.manual = TRUE;
            }
            /*
             *  *** else digitizer mode
             */
            else
            {
                /*
                 *  *** get entry mode
                 */
                port = XmTextGetString(parm_port_text);

                if ((strcmp(port, tie_parms.port) != 0) && 
                        (tie_mapparm.digid != NULL))
                {
                    if ((tie_mapparm.portfd = dig_open(port)) != NULL)
                    {
                        XtRemoveInput(tie_mapparm.digid);
                        tie_mapparm.digid = XtAppAddInput(tie.context, 
                                tie_mapparm.portfd, 
                                (XtPointer)XtInputReadMask,
                                dig_spark, NULL);
                    }
                }
                strcpy(tie_parms.port, XmTextGetString(parm_port_text));

                if (tie_parms.manual && (XtIsRealized(Shell_Manual)))
                    XtUnrealizeWidget(Shell_Manual);

                XtSetSensitive(main_register_button, True);

                if (tie.flags & TIE_SELECTING)
                    XtSetSensitive(main_map_button, True);

                tie_parms.manual = FALSE;
                XtFree(port);
            }

            /*
             *  *** datum required ?
             */
            XtVaGetValues(parm_datum_option, XmNmenuHistory, &epoint, NULL);

            if (epoint == parm_datum_rec)
                strcpy(tie_parms.datumreq, "Required");
            else /* if (epoint == parm_datum_norec) */
                strcpy(tie_parms.datumreq, "Not Required");
        }

        if (tie.mode == IM2VECT)
            strcpy(tie_parms.cd, XmTextGetString(parm_cd_text));

        /* 
         *  *** if TPS filenames were updated
         */
        if (strcmp(tie_parms.intps[REFERENCE],  refin)    ||
                strcmp(tie_parms.intps[SEARCH],     srchin)   ||
                strcmp(tie_parms.outtps[REFERENCE], refout)   ||
                strcmp(tie_parms.outtps[SEARCH],    srchout))
        {
            strcpy(note, "");

            if (strcmp(refin, "") && access(refin, 0)) 
            {
                sprintf(note, "%s does not exist.", refin);
                XmTextSetString(parm_refin_text, tie_parms.intps[REFERENCE]);  
            }
            else if (strcmp(srchin, "") && access(srchin, 0)) 
            {
                sprintf(note, "%s does not exist.", srchin);
                XmTextSetString(parm_srchin_text, tie_parms.intps[SEARCH]);  
            }
            else if (tie_select.ref_desc == NULL)
            {
                if (strcmp(refout, "") && !access(refout, 0)) 
                {
                    sprintf(note, "%s already exists.", refout);
                    XmTextSetString(parm_refout_text, 
                                    tie_parms.outtps[REFERENCE]);  
                }
                else if (strcmp(srchout, "") && !access(srchout, 0)) 
                {
                    sprintf(note, "%s already exists.", srchout);
                    XmTextSetString(parm_srchout_text,tie_parms.outtps[SEARCH]);
                }
            }

            if (strcmp(note, ""))
            {
                tie_message(Shell_Parm, note);
            }
            /* 
             *  *** if nothing selected, set new filenames
             */
            else if (tie_select.seltie == 0)
            {
                if (tie_select.ref_desc != NULL)
                {
                    c_ptclse(&tie_select.ref_desc);
                    unlink(tie_parms.outtps[REFERENCE]);
                }
                if (tie_select.srch_desc != NULL)
                {
                    c_ptclse(&tie_select.srch_desc);
                    unlink(tie_parms.outtps[SEARCH]);
                }

                strcpy(tie_parms.intps[SEARCH],     srchin);
                strcpy(tie_parms.intps[REFERENCE],  refin);
                strcpy(tie_parms.outtps[SEARCH],    srchout);
                strcpy(tie_parms.outtps[REFERENCE], refout);

                /*
                 *  *** if files already open, close and reopen
                 */
                if (tie_select.ref_desc != NULL)
                {
                    if (tie.mode == IM2IM)
                        rplane_idx = tie_composite[REFERENCE].rgbplanes[RED]-1;

                    splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

                    if (tie.mode == IM2VECT)
                        status = tps_open(tie_planes[splane_idx].ddr.proj_units,
                                &tie_planes[splane_idx].ddr);
                    else if (tie.mode == IM2IM)
                        status = tps_open("", &tie_planes[rplane_idx].ddr);
                    else
                        status = tps_open("", &tie_planes[splane_idx].ddr);
                    if (status != E_SUCC)
                        return;
                } 
            }
            /* 
             *  *** else verify loss of work
             */
            else
            {
                argcnt = 0;
                XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); 
                argcnt++;
                XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
                XtSetArg(args[argcnt], XmNmessageString, 
                        (xmstr=XmStringCreateLtoR(
                         "Updating TPS filename will result in loss of existing points!!",
                         XmSTRING_DEFAULT_CHARSET))); argcnt++;

                message = XmCreateMessageDialog(Shell_Parm, 
                        "Message Box",
                        args, argcnt);
                XmStringFree(xmstr);

                XtAddCallback(message,
                        XmNcancelCallback,
                        tie_msgcancel,
                        (caddr_t)0);

                XtAddCallback(message,
                        XmNokCallback,
                        parm_tpsupdate,
                        (caddr_t)w);

                XtUnmanageChild(XmMessageBoxGetChild(message,
                            XmDIALOG_HELP_BUTTON));
            }
        }

        XtFree(refin);
        XtFree(srchin);
        XtFree(refout);
        XtFree(srchout);
    }

    if (((w == parm_accept) || (w == parm_cancel)) && 
            (message == NULL) && (strcmp(note, "") == 0))
    {
        XtUnmanageChild(Parm_bulletinboard);
        XtUnrealizeWidget(Shell_Parm);
        XtSetSensitive(main_parm_button,True);
        /*    if (!(tie.flags & TIE_SELECTING))                   */
        XtSetSensitive(main_load_button, True);

    }
    else if (message != NULL)
    {
        XBell(tie.display, 0);
        XtManageChild(message);
    }
}



/*******************************************************************************
Purpose:
    This activates/deactivates the alpha parameter based on the selected
    zoom method.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_zoom_type
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
    Arg     args[1];          /* Variable used to set widget resourcees     */

    if ((client != NULL) && (strcmp((char*)client, "CC") == 0))
    {
        XtSetArg(args[0], XmNsensitive, True);
        XtSetValues(parm_alpha, args, 1);
        XtSetValues(parm_alpha_label, args, 1);
    }
    else
    {
        XtSetArg(args[0], XmNsensitive, False);
        XtSetValues(parm_alpha, args, 1);
        XtSetValues(parm_alpha_label, args, 1);
    }
}


/*******************************************************************************
Purpose:
    This routine increases or decreases the current zoom factor based on the
    which arrow button was pressed.  Currently there is an arbitrary zoom
    factor limit of 16.  

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_zoom_arrow
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    int  factor;
    char outval[3];

    sscanf(XmTextGetString(parm_zfactor),"%d",&factor);

    /*
     *  Increase or decrease the current zoom factor.  The minimum zoom factor 
     *  is 2 and the maximum is currently 16 (this is an arbitrary limit).
     */
    if (strcmp((char*)client,"UP") == 0)
        factor++;
    else
        factor--;

    factor = (factor > 16) ? 2  : factor;
    factor = (factor < 2)  ? 16 : factor;

    /*
     *  *** Set the zoom factor.
     */
    sprintf(outval,"%d",factor);
    XmTextSetString(parm_zfactor,outval);
}

/*******************************************************************************
Purpose:
    This colors the Cursor Color option to match the current selection.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_cursor_color
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Arg args[1];
    static Widget color_button = NULL;    /* selected attribute color button */

    /*
     *  *** if NULL client, display color requestor
     */
    if (color_button == NULL)
    {
        color_button = w;
        Color_req = Createcolor_req(Shell_Parm, parm_cursor_color);
        XtManageChild(Color_req);
    }
    /*
     *  *** otherwise, set color and remove color requestor
     */
    else
    {
        XtSetArg(args[0],XmNforeground, COLOR(Shell_SrchImg,
                    wk_colors[(int)client]));
        XtSetValues(parm_curcolor_button, args, 1);
        XSetForeground(tie.display, tie_select.gc,
                tie.color_table[RGBBW_IDX + (int)client].pixel);

        XtUnmanageChild(Color_req);
        XtDestroyWidget(Color_req);
        cursor_setcolor((int)client);
        color_button = NULL;
    }
}

/*******************************************************************************
Purpose:
    This routine initializes the parameter structure used later by the
    routine zoom_enlarge.  Checks are made to be sure that enough data exists
    to extract the amount required by the enlargement routine.
                                                                
By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_setup(void)
{
    Arg          args[1];        /* Variable used to set widget resourcees    */
    char         outval[CMLEN];  /* string conversion buffer                  */
    XGCValues gc_values;

    if (!XtIsRealized(Shell_Parm))
        return;

    /*
     *  *** Set Tie point selection filenames
     */
    XmTextSetString(parm_refout_text,  tie_parms.outtps[REFERENCE]);
    XmTextSetString(parm_srchout_text, tie_parms.outtps[SEARCH]);
    XmTextSetString(parm_refin_text,   tie_parms.intps[REFERENCE]);
    XmTextSetString(parm_srchin_text,  tie_parms.intps[SEARCH]);
    if (tie.mode == IM2MAP)
        XmTextSetString(parm_port_text,    tie_parms.port);

    /*
     *  *** Set the text string to reflect the new zoom factor.
     */
    sprintf(outval,"%d",tie_parms.factor);
    XmTextSetString(parm_zfactor,outval);

    /*
     *  *** Set alpha  
     */
    sprintf(outval,"%f",tie_parms.alpha);
    XmTextSetString(parm_alpha, outval);

    /*
     *  *** Set zoom method  
     */
    if (strcmp(tie_parms.method, "NN") == 0)
    {
        XtSetArg(args[0], XmNsensitive, True);
        XtSetValues(parm_alpha, args, 1);
        XtSetValues(parm_alpha_label, args, 1);
        XtSetArg(args[0], XmNmenuHistory, parm_ztype_nn);
    }
    else if (strcmp(tie_parms.method, "BI") == 0)
    {
        XtSetArg(args[0], XmNsensitive, True);
        XtSetValues(parm_alpha, args, 1);
        XtSetValues(parm_alpha_label, args, 1);
        XtSetArg(args[0], XmNmenuHistory, parm_ztype_bilinear);
    }
    else if (strcmp(tie_parms.method, "CC") == 0)
    {
        XtSetArg(args[0], XmNsensitive, True);
        XtSetValues(parm_alpha, args, 1);
        XtSetValues(parm_alpha_label, args, 1);
        XtSetArg(args[0], XmNmenuHistory, parm_ztype_cc);
    }
    XtSetValues(parm_ztype_option, args, 1);

    /*
     *  *** Set LSQ fit option
     */
    if (tie_parms.order == FIRST_ORDER)
        XtSetArg(args[0], XmNmenuHistory, parm_order_first);
    else
        XtSetArg(args[0], XmNmenuHistory, parm_order_second);

    XtSetValues(parm_order_option, args, 1);

    /*
     *  *** Set zoom window size option
     */
    if (tie_parms.zsize == 128)
        XtSetArg(args[0], XmNmenuHistory, parm_zsize_128);
    else
        XtSetArg(args[0], XmNmenuHistory, parm_zsize_64);

    XtSetValues(parm_zsize_option, args, 1);

    /*
     *  *** Set fullres window size
     */
    XtSetArg(args[0], XmNset, False);
    XtSetValues(parm_fullres_256, args, 1);
    XtSetValues(parm_fullres_512, args, 1);
    XtSetArg(args[0], XmNset, True);

    if ((tie_parms.fwidth == 256) && (tie_parms.fheight == 256))
    {
        XtSetArg(args[0], XmNmenuHistory, parm_fullres_256);
        XtSetValues(parm_fullres_option, args, 1);
    }
    else if ((tie_parms.fwidth == 512) && (tie_parms.fheight == 512))
    {
        XtSetArg(args[0], XmNmenuHistory, parm_fullres_512);
        XtSetValues(parm_fullres_option, args, 1);
    }

    if (strcmp(tie_parms.epoint, "Not Required") == 0)
    {
        XtSetArg(args[0], XmNmenuHistory, parm_epoint_norec);
        XtSetValues(parm_epoint_option, args, 1);
    }
    else
    {
        XtSetArg(args[0], XmNmenuHistory, parm_epoint_rec);
        XtSetValues(parm_epoint_option, args, 1);
    }

    /*
     *  *** set spark type radio
     */
    if (tie.mode == IM2MAP)
    {
        if (tie_parms.manual)
        {
            XtSetArg(args[0], XmNmenuHistory, parm_spark_manual);
            XtSetValues(parm_spark_option, args, 1);
            XtSetSensitive(parm_port_label, False);
            XtSetSensitive(parm_port_text,  False);
        }
        else
        {
            XtSetArg(args[0], XmNmenuHistory, parm_spark_digit);
            XtSetValues(parm_spark_option, args, 1);
            XtSetSensitive(parm_port_label, True);
            XtSetSensitive(parm_port_text,  True);
        }

        /*
         *  *** set datum requirement option
         */
        if (strcmp(tie_parms.datumreq, "Not Required") == 0)
        {
            XtSetArg(args[0], XmNmenuHistory, parm_datum_norec);
            XtSetValues(parm_datum_option, args, 1);
        }
        else
        {
            XtSetArg(args[0], XmNmenuHistory, parm_datum_rec);
            XtSetValues(parm_datum_option, args, 1);
        }
    }
    else if (tie.mode == IM2VECT)
    {
        /*
         *  *** set vector source and location
         */
        XmTextSetString(parm_cd_text, tie_parms.cd);
        if (tie_parms.vsource == DLGOPT)
            XtSetArg(args[0],XmNmenuHistory,parm_vsrc_dlg);
        else if (tie_parms.vsource == WDB)
            XtSetArg(args[0],XmNmenuHistory,parm_vsrc_wdb);
        else
            XtSetArg(args[0],XmNmenuHistory,parm_vsrc_dcw);
        XtSetValues(parm_vsrc_option, args, 1);
    }

    /*
     *  *** set cursor color
     *      set up the button foreground here: get color from tie_select.gc
     */
    XGetGCValues(tie.display, tie_select.gc, GCForeground, &gc_values);
    XtSetArg(args[0],XmNforeground, gc_values.foreground);
    XtSetValues(parm_curcolor_button, args, 1);

    /*
     *  ***  set ref and search offsets
     */
    sprintf(outval,"%d",tie_parms.refoff[0]);
    XmTextSetString(parm_refoff_horiz,outval);

    sprintf(outval,"%d",tie_parms.refoff[1]);
    XmTextSetString(parm_refoff_vert,outval);

    sprintf(outval,"%d",tie_parms.srchoff[0]);
    XmTextSetString(parm_srchoff_horiz,outval);

    sprintf(outval,"%d",tie_parms.srchoff[1]);
    XmTextSetString(parm_srchoff_vert,outval);
}

/*******************************************************************************
Purpose:
    This routine is called when the point entry method radio buttons
    are pressed.  If manual entry is selected, the digitizer port
    widgets are not sensitive.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_spark_type
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    if ((client != NULL) && (strcmp((char*)client, "MANUAL") == 0))
    {
        XtSetSensitive(parm_port_text, False);
        XtSetSensitive(parm_port_label, False);
    }
    else
    {
        XtSetSensitive(parm_port_text, True);
        XtSetSensitive(parm_port_label, True);
    }
}

/*******************************************************************************
Purpose:
    This routine checks the tps filename extension and makes the 
    next text item active.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_tps_text
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    char         filename[256];
    char         *ext;

    /*
     *  *** add .tps extension if necessary
     */
    strcpy(filename, XmTextGetString(w));

    if (strcmp(filename, "") !=  0)
        if (((ext = strrchr(filename, '.')) == NULL) ||
                (strcmp(ext, ".tps") != 0))
        {
            strcat(filename, ".tps");
            XmTextSetString(w, filename);
        }

    XmProcessTraversal(Parm_bulletinboard, XmTRAVERSE_NEXT_TAB_GROUP);
}

/*******************************************************************************
Purpose:
    This routine displays a file selector for the selection of existing
    TPS file as input.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void parm_tpsfile
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    if ((int)client == REFERENCE)
    {
        TPS_fileselector = Createtps_fileselector(Shell_Parm,parm_refin_button);
        XtSetSensitive(parm_refin_button, False);
    }
    else if ((int)client == SEARCH)
    {
        TPS_fileselector = Createtps_fileselector(Shell_Parm,
                           parm_srchin_button);
        XtSetSensitive(parm_srchin_button, False);
    }

    XtManageChild(TPS_fileselector);
}

/*******************************************************************************
Purpose:
    This routine handles the update of a TPS file parmater. The names
    of the respective files is extracted from the text widgets and 
    copied into the parameter structure.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Cetner, 4/93
  - Modified to check the status from the tps_open call.

Algorithm Description:
*******************************************************************************/
void parm_tpsupdate
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    int          status;         /* Status indicator.                        */

    /*
     *  *** extract filenames from text widgets
     */
    char *refin   = XmTextGetString(parm_refin_text);
    char *srchin  = XmTextGetString(parm_srchin_text);
    char *refout  = XmTextGetString(parm_refout_text);
    char *srchout = XmTextGetString(parm_srchout_text);

    /*
     *  *** close and remove output tps files
     */
    if (tie_select.ref_desc != NULL)
        c_ptclse(&tie_select.ref_desc);
    if (tie_select.srch_desc != NULL)
        c_ptclse(&tie_select.srch_desc);
    unlink(tie_parms.outtps[REFERENCE]);
    unlink(tie_parms.outtps[SEARCH]);
    tie.flags &= ~TIE_OPEN;

    /*
     *  *** update parameters structure
     */
    strcpy(tie_parms.intps[SEARCH],     srchin);
    strcpy(tie_parms.intps[REFERENCE],  refin);
    strcpy(tie_parms.outtps[SEARCH],    srchout);
    strcpy(tie_parms.outtps[REFERENCE], refout);
    status = tps_open("", &tie_planes[0].ddr);
    if (status != E_SUCC)
        return;

    XtFree(refin);
    XtFree(srchin);
    XtFree(refout);
    XtFree(srchout);
    XtDestroyWidget(w);

    /*
     *  *** if Accept button was pressed, remove parameter panel
     */
    if ((Widget)client == parm_accept)
    {
        XtUnmanageChild(Parm_bulletinboard);
        XtUnrealizeWidget(Shell_Parm);
        XtSetSensitive(main_parm_button,True);
    }
}

/*******************************************************************************
Purpose:
    This routine toggles the flags indicating which coordinate type(s)
    to use as output (i.e. line/sample and/or projection or latitude/longitude).

By C. Olsen, USGS/EROS Data Center, 8/98 (SRF #1935)

Algorithm Description:
*******************************************************************************/
void parm_coord_type
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    /*
     *  *** toggle appropriate coordinate type flag  
     */
    switch (*(char*)client) 
    {
        case 'S':
            tie_parms.flags ^= PARMS_LINESAMP;
            break;
        case 'P':
            tie_parms.flags ^= PARMS_PROJ;
            break;
        case 'L':
            tie_parms.flags ^= PARMS_LATLONG;
            break;
    }
}

/*******************************************************************************
Purpose:
    This is the callback routine for the "Multiple Reference Images" toggle
    button.  It sets Latitude/Longitude as the coordinate type, and sets the
    output coordinate radio box to insensitive.

By C. Olsen, USGS/EROS Data Center, 9/98 (SRF #2207)

Algorithm Description:
*******************************************************************************/
void parm_multiref
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct *)call_data;

    tie_parms.flags ^= PARMS_MULTI;
    tie_parms.flags &= ~(PARMS_PROJ);

    if (acs->set)
    {
        XtVaSetValues(parm_ls_button,XmNset,False,XmNsensitive,False,NULL);
        XtVaSetValues(parm_proj_button,XmNset,False,XmNsensitive,False,NULL);
        XtVaSetValues(parm_latlong_button,XmNset,True,XmNsensitive,False,NULL);
        tie_parms.flags |= PARMS_LATLONG;
        tie_parms.flags &= ~(PARMS_LINESAMP);
    }
    else
    {
        XtVaSetValues(parm_ls_button,XmNset,True,XmNsensitive,True,NULL);
        XtVaSetValues(parm_proj_button,XmNset,False,XmNsensitive,True,NULL);
        XtVaSetValues(parm_latlong_button,XmNset,False,XmNsensitive,True,NULL);
        tie_parms.flags |= PARMS_LINESAMP;
        tie_parms.flags &= ~(PARMS_LATLONG);
    }
}
