#include <math.h>
#include "tiepts.h"
#include "convert.h"
#include "protos.h"

/*******************************************************************************
Purpose:
   This function displays the errors for each of the tie points.
   This function compares the search coordinate with what the reference 
   coordinate says it should be.  It is not a measure of the accuracy 
   of the rough transformation, but is a measure of the relative error 
   of the search image as compared to the reference.
   For each mode, convert rtie to units used by stie.
     For IM2VECT, the search image must have a valid DDR to convert the 
        rtie (srch proj coords) to srch line/sample based on a projection 
        calculation.  If the search image has no valid DDR,  rtie will be
        in degrees, and there is no way to convert that to srch line/sample.
     For IM2IM, stie are in srch image line/sample coords, and rtie are in 
        ref image line/sample coords, so gctp will be used to convert rtie: 
           ref l/s -> ref proj coords -> srch proj coords -> srch l/s
     For IM2MAP, stie are in srch image line/sample coords, and rtie are in 
        degrees lat/long for degrees or proj units, unknown for user-defined
        units.  If degrees or proj, use gctp to convert rtie degrees to 
        srch img proj coords and then to srch img line/samp

   NOTE: Coordinates indexes mean the following:
        stie[0], rtie[0] (x or sample direction)
        stie[1], rtie[1] (y or line direction)

By T. Rockvam, USGS/EROS Data Center, 5/94
*******************************************************************************/
void stats_display
(
    int botflag /* flag to scroll to bottom of list           */
)
{
   Arg          args[2];        /* resource argument variable                 */

   long         i;              /* loop counter                               */
   long         imgwin[8];      /* corners of image window                    */
   int  count,          /* number of items already in list            */
                nvis,           /* number of items visible in list            */
                nactive,        /* number of active points                    */
                rplane_idx,     /* index of the ref image memory plane        */
                splane_idx,     /* index of the srch image memory plane       */
                valid_coefs=TRUE;/* conversion coefs are valid                */
   
   double       refline,refsamp,/* reference coord                            */
                tmpcoord[2],    /* temp coord value for conversion            */
                lerr,serr,      /* line,sample error in coords                */
                avglerr,avgserr,/* average line,sample error in coords        */
                rmsel,rmses,    /* line,sample rmse in coords                 */
                totalrmse,      /* total rmse in line&samp direction          */
                totalrmsepix,   /* total rmse in line&samp direction in pixels*/
                for_coefs[6],   /* coeffs to convert line/samp to proj coords */
                inv_coefs[6];   /* coeffs to convert proj coords to line/samp */

   char         tmpstr[132];    /* temporary string displaying one line       */
   XmString     xmstr;          /* temporary Xm string variable               */

   XtSetArg(args[0], XmNitemCount, &count);
   XtSetArg(args[1], XmNvisibleItemCount, &nvis);
   XtGetValues(stats_list, args, 2);

   rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;
   splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
   avglerr = avgserr = 0.0;
   rmsel = rmses = 0.0;
   totalrmse = 0.0;
   nactive = 0;

   if ((tie_planes[splane_idx].ddr.valid[0] == INVAL) ||
       (tie_planes[splane_idx].ddr.valid[1] == INVAL) ||
       (tie_planes[splane_idx].ddr.valid[2] == INVAL) ||
       (tie_planes[splane_idx].ddr.valid[3] == INVAL) ||
       (tie_planes[splane_idx].ddr.valid[4] == INVAL) ||
       (tie_planes[splane_idx].ddr.valid[5] == INVAL) ||
       (tie_planes[splane_idx].ddr.valid[6] == INVAL)) return;

  /*
   *  *** for IM2VECT get coeffs to convert from srch image projection 
   *      coords to line/sample
   */
   if (tie.mode == IM2VECT)
      {
      imgwin[0] = imgwin[1] = imgwin[3] = imgwin[4] = 1;
      imgwin[2] = imgwin[6] = tie_planes[splane_idx].ddr.nl;
      imgwin[5] = imgwin[7] = tie_planes[splane_idx].ddr.ns;
      i = FALSE;
      if (c_rtcoef(tie_planes[splane_idx].ddr.upleft,imgwin,
             for_coefs,&i) != E_SUCC)
         valid_coefs = FALSE;
      if (c_invert(for_coefs,inv_coefs) != E_SUCC)
         valid_coefs = FALSE;
       }
   else if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
      {
     /* 
      * *** need inverse coefs to convert srch proj coord to srch line/samp 
      */
      imgwin[0] = imgwin[1] = imgwin[3] = imgwin[4] = 1;
      imgwin[2] = imgwin[6] = tie_planes[splane_idx].ddr.nl;
      imgwin[5] = imgwin[7] = tie_planes[splane_idx].ddr.ns;
      i = FALSE;
      if (c_rtcoef(tie_planes[splane_idx].ddr.upleft,imgwin,
             for_coefs,&i) != E_SUCC)
         valid_coefs = FALSE;
      if (c_invert(for_coefs,inv_coefs) != E_SUCC)
         valid_coefs = FALSE;

     /* 
      * *** need forward coefs to convert ref line/samp to proj coord 
      */
      imgwin[0] = imgwin[1] = imgwin[3] = imgwin[4] = 1;
      imgwin[2] = imgwin[6] = tie_planes[rplane_idx].ddr.nl;
      imgwin[5] = imgwin[7] = tie_planes[rplane_idx].ddr.ns;
      i = FALSE;
      if (c_rtcoef(tie_planes[rplane_idx].ddr.upleft,imgwin,
             for_coefs,&i) != E_SUCC)
         valid_coefs = FALSE;
      }
   else if ((tie.mode == IM2MAP) || 
            ((tie.mode == IM2IM) && (tie.flags & TIE_ADRG)))
      {
      if (tie_mapparm.units == MAP_USERDEF)
         valid_coefs = FALSE;
      imgwin[0] = imgwin[1] = imgwin[3] = imgwin[4] = 1;
      imgwin[2] = imgwin[6] = tie_planes[splane_idx].ddr.nl;
      imgwin[5] = imgwin[7] = tie_planes[splane_idx].ddr.ns;
      i = FALSE;
      if (c_rtcoef(tie_planes[splane_idx].ddr.upleft,imgwin,
             for_coefs,&i) != E_SUCC)
         valid_coefs = FALSE;
      if (c_invert(for_coefs,inv_coefs) != E_SUCC)
         valid_coefs = FALSE;
      }

   if (!valid_coefs) return;

  /*
   *  *** for each of the tie points, place errors in scroll list
   */
   for (i=0; i < tie_select.seltie; i++)
      {
      if (tie.mode == IM2VECT)
         {
        /* 
         * *** convert the image projection coords to line/sample coords 
         */
         refline = (inv_coefs[0] * tie_select.rtie[i][1]) + 
                   (inv_coefs[1] * tie_select.rtie[i][0]) + 
                    inv_coefs[2];
         refsamp = (inv_coefs[3] * tie_select.rtie[i][1]) + 
                   (inv_coefs[4] * tie_select.rtie[i][0]) + 
                    inv_coefs[5];
         lerr = (refline-tie_select.stie[i][1]) * 
                tie_planes[splane_idx].ddr.pdist_y;
         serr = (refsamp-tie_select.stie[i][0]) * 
                tie_planes[splane_idx].ddr.pdist_x;
         if (!tie_select.disabled[i])
            {
            nactive++;
            sprintf(tmpstr,"%7s %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f",
               tie_select.pt_id[i],
               refline,refsamp,
               tie_select.stie[i][1],tie_select.stie[i][0],
               lerr,serr,
               sqrt(lerr*lerr + serr*serr));
            avglerr += lerr;
            avgserr += serr;
            rmsel += lerr * lerr;
            rmses += serr * serr;
            }
          else
             {
             sprintf(tmpstr,"DISABLE %9.2f %9.2f %9.2f %9.2f",
                   refline,refsamp,
                   tie_select.stie[i][1],tie_select.stie[i][0]);
             }
         }
      else if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
         {
         /* X: samp, tie_select.rtie[i][0], tie_select.stie[i][0], tmpcoord[0]*/
         /* Y: line, tie_select.rtie[i][1], tie_select.stie[i][1], tmpcoord[1]*/
         /* convert the ref line/samp to proj coord */
         tmpcoord[1] = (for_coefs[0] * tie_select.rtie[i][1]) + 
                       (for_coefs[1] * tie_select.rtie[i][0]) + 
                        for_coefs[2];
         tmpcoord[0] = (for_coefs[3] * tie_select.rtie[i][1]) + 
                       (for_coefs[4] * tie_select.rtie[i][0]) + 
                        for_coefs[5];
         /* convert the ref proj coord to srch proj coord */
         proj2proj(&tie_planes[rplane_idx].ddr,
                   &tie_planes[splane_idx].ddr,tmpcoord,1);
         /* convert the srch proj coord to srch line/samp */
         refline = (inv_coefs[0] * tmpcoord[1]) + 
                   (inv_coefs[1] * tmpcoord[0]) + 
                    inv_coefs[2];
         refsamp = (inv_coefs[3] * tmpcoord[1]) + 
                   (inv_coefs[4] * tmpcoord[0]) + 
                    inv_coefs[5];
         lerr = (refline-tie_select.stie[i][1]) *
                tie_planes[splane_idx].ddr.pdist_y;
         serr = (refsamp-tie_select.stie[i][0]) *
                tie_planes[splane_idx].ddr.pdist_x;
         if (!tie_select.disabled[i])
            {
            nactive++;
            sprintf(tmpstr,"%7s %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f",
               tie_select.pt_id[i],
               refline,refsamp,
               tie_select.stie[i][1],tie_select.stie[i][0],
               lerr,serr,
               sqrt(lerr*lerr + serr*serr));
            avglerr += lerr;
            avgserr += serr;
            rmsel += lerr * lerr;
            rmses += serr * serr;
            }
         else
            {
            sprintf(tmpstr,"DISABLE %9.2f %9.2f %9.2f %9.2f",
                  refline,refsamp,
                  tie_select.stie[i][1],tie_select.stie[i][0]);
            }
         }
      else if ((tie.mode == IM2MAP) || 
               ((tie.mode == IM2IM) && (tie.flags & TIE_ADRG)))
         {
        /* 
         * *** convert the geo coords to srch line/sample coords 
         */
         /* convert the geo coord to srch proj coord */
         tmpcoord[0] = tie_select.rtie[i][0];
         tmpcoord[1] = tie_select.rtie[i][1];
         degrees2image(tmpcoord,1,SEARCH);

         /* convert the srch proj coord to srch line/samp */
         refline = (inv_coefs[0] * tmpcoord[1]) + 
                   (inv_coefs[1] * tmpcoord[0]) + 
                    inv_coefs[2];
         refsamp = (inv_coefs[3] * tmpcoord[1]) + 
                   (inv_coefs[4] * tmpcoord[0]) + 
                    inv_coefs[5];
         lerr = (refline-tie_select.stie[i][1]) * 
                tie_planes[splane_idx].ddr.pdist_y;
         serr = (refsamp-tie_select.stie[i][0]) * 
                tie_planes[splane_idx].ddr.pdist_x;
         if (!tie_select.disabled[i])
            {
            nactive++;
            sprintf(tmpstr,"%7s %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f",
               tie_select.pt_id[i],
               refline,refsamp,
               tie_select.stie[i][1],tie_select.stie[i][0],
               lerr,serr,
               sqrt(lerr*lerr + serr*serr));
            avglerr += lerr;
            avgserr += serr;
            rmsel += lerr * lerr;
            rmses += serr * serr;
            }
         else
            {
            sprintf(tmpstr,"DISABLE %9.2f %9.2f %9.2f %9.2f",
                  refline,refsamp,
                  tie_select.stie[i][1],tie_select.stie[i][0]);
            }
         }

      xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);
      if (i < count)
         XmListReplaceItemsPos(stats_list, &xmstr, 1, i+1); 
      else
         XmListAddItem(stats_list, xmstr, 0); 
      XmStringFree(xmstr);
      }

  /*
   * *** display the total errors
   */
 if (nactive > 0)
   {
   avglerr /= nactive;
   avgserr /= nactive;
   rmsel /= nactive;
   rmses /= nactive;
   totalrmse = rmsel + rmses;
   totalrmsepix = 
      (rmsel /
   (tie_planes[splane_idx].ddr.pdist_y*tie_planes[splane_idx].ddr.pdist_y)) + 
      (rmses/
   (tie_planes[splane_idx].ddr.pdist_x*tie_planes[splane_idx].ddr.pdist_x));
   rmsel = sqrt(rmsel);
   rmses = sqrt(rmses);
   totalrmse = sqrt(totalrmse);
   totalrmsepix = sqrt(totalrmsepix);

   sprintf(tmpstr,"Line   error AVERAGE = %9.2f %s   RMSE = %9.2f %s",
      avglerr,tie_planes[splane_idx].ddr.proj_units,
      rmsel,tie_planes[splane_idx].ddr.proj_units);
   xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);
   if (i < count) XmListReplaceItemsPos(stats_list, &xmstr, 1, i+1); 
   else           XmListAddItem(stats_list, xmstr, 0); 
   XmStringFree(xmstr);
   i++;

   sprintf(tmpstr,"Sample error AVERAGE = %9.2f %s   RMSE = %9.2f %s",
      avgserr,tie_planes[splane_idx].ddr.proj_units,
      rmses,tie_planes[splane_idx].ddr.proj_units);
   xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);
   if (i < count) XmListReplaceItemsPos(stats_list, &xmstr, 1, i+1); 
   else           XmListAddItem(stats_list, xmstr, 0); 
   XmStringFree(xmstr);
   i++;

   sprintf(tmpstr,"Total  error RMSE = %9.2f %s %9.2f pixels",
      totalrmse,tie_planes[splane_idx].ddr.proj_units,
      totalrmsepix);
   xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);
   if (i < count) XmListReplaceItemsPos(stats_list, &xmstr, 1, i+1); 
   else           XmListAddItem(stats_list, xmstr, 0); 
   XmStringFree(xmstr);
   i++;
   }

  /*
   * *** Clear out any remaining lines
   */
   if ((count-i) > 0)
      XmListDeleteItemsPos(stats_list, count-i, i+1); 

  /*
   * *** reset the label, adding the units for the error
   */
   sprintf(tmpstr,
      "               REFERENCE            SEARCH             ERRORS (%s)",
      tie_planes[splane_idx].ddr.proj_units);
   XtVaSetValues(stats_image_label,XmNlabelString,
      (xmstr=XmStringCreateLtoR(tmpstr,XmSTRING_DEFAULT_CHARSET)),NULL);
   XmStringFree(xmstr);

   if (!XtIsRealized(Shell_Stats))
      {
      XtRealizeWidget(Shell_Main);
      XtManageChild(Stats_form);
      }

  /*
   *  *** scroll to bottom of the list
   */
   if (botflag && (tie_select.seltie >= nvis))
      XmListSetPos(stats_list, tie_select.seltie - nvis + 4);
}

/*******************************************************************************
Purpose:
    This routine closes the stats display window.

By T. Rockvam, USGS/EROS Data Center, 5/94
*******************************************************************************/
void stats_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  /*
   *  *** remove stats display
   XtUnmanageChild(Stats_form);
   */
   XtUnrealizeWidget(Shell_Stats);
   XtSetSensitive(main_stats_button, True);
}
