/*
 *  This is the main source file for the Tie Point Selection (tiepts) 
 *  application.
 *
 *  By J. Hemmer/D. Akkerman/L. Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *  Modified by J. LaVergne USGS/EROS Data Center 7/93
 *    - changed 'Tie Points - Mode' panel to 'Tie Points'.
 *    - removed the (Sun) #define main MAIN_ so the command line options
 *      would work on the sun.
 *
 *  Modified by T. Rockvam USGS/EROS Data Center 11/93
 *    - added test to see if user can open the display
 *  Modified by T. Rockvam USGS/EROS Data Center 3/94
 *    - changed error messages to printfs before the display is open
 *  Modified by C. Olsen, USGS/EROS Data Center, 8/98
 *    - set the default coordinate type to line/sample (SRF #1935)
 */
#define _tiepts_h          /* Assign global structures.                    */

#include "tiepts.h"
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>
#include <sys/types.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include "appres.h"
#include "cr.h"
#include "protos.h"

TieAppres tie_appres;

/* tjr - temporary global because of division of vector.c and vector_tmp.c */
Widget        *attrib_form = NULL;

/*
 *  Resource converter variables for the TIE_StringToPixel resource
 *  converter routine.
 */

XtConvertArgRec color_ConvertArgs[] = 
{
    {XtWidgetBaseOffset, (XtPointer)XtOffset(Widget, core.screen),
        sizeof(Screen *)},
    {XtWidgetBaseOffset, (XtPointer)XtOffset(Widget, core.colormap),
        sizeof(Colormap)}
};

static void settraps(void);


/******************************************************************************
 *                                                                            *
 *              Tie Point Selection main application.                         *
 *                                                                            *
 ******************************************************************************/

main
(
   int          argc,
   char         **argv
)
{
   Arg          args[256];      /* X resource arguments.                */
   Status       status;         /* Return status variable.              */
   Visual       *visual;        /* Working visual pointer.              */
   XVisualInfo  visual_template;/* Visual template to match against
                                   server.                              */
   unsigned 
   long         pixelval[256],  /* Array for pixel values.              */
                visual_mask=0;  /* Specifies which elements to match
                                   using the visual template.           */
   XColor       col_cell;       /* Working colorcell.                   */
   XrmOptionDescRec options;    /* Command line options table.          */
   char         errmsg[80];     /* Working error message string.        */
   int          number_visuals; /* Number of server visuals that match
                                   request.                             */
   int  argcnt,         /* Resource argument count.             */
                i,              /* Working loop indices.                */
                offset;         /* Used in scaling the 24-bit LUT.      */

   unsigned long 
                plane_masks[1], /* Working plane mask.                  */
                wk_pixels[1],   /* Return pixel value array.            */
                red_mask,       /* Mask values to reference 24-bit pixels. */
                green_mask,
                blue_mask;

   float        slope;          /* Used in scaling the 24-bit LUT.      */

static String fallback_resources[] = {
#include "TIEpts.ad.h"
NULL
};                              /* Fallback resource specifications.    */

  /*
   *  Initialize the X Toolkit and assign standard globals.
   */
   if (getenv("DISPLAY") == NULL)
      { 

      /* stdio-based error messages have been modified from printf() calls
         to fprintf(stderr) calls.  This adds some additional functionality 
         to the application and ensures that error messages are displayed
         in their proper context (stderr is always unbuffered in SVR4 and
         4.3+BSD; ANSI specifies that it is never _fully_ buffered.) -cje */

      fprintf(stderr, "You must set your DISPLAY variable\n");
      exit_tiepts(0);
      }

   XtToolkitInitialize();
   tie.context = XtCreateApplicationContext();

/*
 *  Set fallback resources.
 */
   XtAppSetFallbackResources(tie.context,fallback_resources);

   tie.display = XtOpenDisplay(tie.context,NULL,BX_APP_NAME,BX_APP_CLASS,
                        &options,0,&argc,argv);
   if (tie.display == NULL)
      {
      XtWarning("Cannot open display");
      exit_tiepts(0);
      }
   tie.screen = DefaultScreen(tie.display);

/*tjr* XSynchronize(tie.display,True); */

  /*
   *  Assign the visual for the application.
   */
   visual = DefaultVisual(tie.display,tie.screen);
   visual_template.screen = tie.screen;

   switch (visual->class)
      {
      case StaticGray:
      case GrayScale:   
          visual_template.class = GrayScale;
          break;

      case StaticColor:
      case PseudoColor: 
          visual_template.class = PseudoColor;
          visual_template.depth = 8;
          break;

      case TrueColor:
      case DirectColor: 
          visual_template.class = DirectColor;
          visual_template.depth = 24;
          break;
      }

   visual_mask = VisualScreenMask;      /* Limit to present screen.     */
   visual_mask |= VisualDepthMask;      /* Limit to standard depth.     */
   visual_mask |= VisualClassMask;      /* Limit to class assigned.     */

   if (visual_template.class == GrayScale)
      {
      fprintf(stderr, "Visual not supported\n");
      exit_tiepts(0);
      } 

   number_visuals = 0;
   tie.visual_info = XGetVisualInfo(tie.display,visual_mask,&visual_template,
                                        &number_visuals);
   if ((number_visuals == 0) || (tie.visual_info == (XVisualInfo *)NULL))
      {
      fprintf(stderr, "Error retrieving visual information\n");
      exit_tiepts(0);
      }

  /*
   *  *** Create the application colormap.
   */
   tie.colormap = XCreateColormap(tie.display,
                        RootWindow(tie.display,tie.screen),
                        tie.visual_info[0].visual,AllocNone);
   if (tie.colormap == DefaultColormap(tie.display,tie.screen))
      {
      fprintf(stderr, "Error creating colormap\n");
      exit_tiepts(0);
      }

  /*
   *  *** Allocate the working colorcells.
   */
   if (tie.visual_info[0].visual->class == PseudoColor)
      {
     /* 
      *  *** Allocate the colorcells.                                   
      */
      status = XAllocColorCells(tie.display,tie.colormap,True,plane_masks,
                        0,pixelval,256);
      if (status == 0)
         {
         fprintf(stderr, "Unable to allocate colorcells\n");
         exit_tiepts(0);
         }

     /* 
      * To minimize color flashing copy the first NUM_WMCOLORS out of
      * the default color map.                                          
      */
      for (i = 0; i < NUM_WMCOLORS; i++)
          tie.color_table[WM_IDX + i].pixel = i;

      XQueryColors(tie.display,DefaultColormap(tie.display,tie.screen),
          &tie.color_table[WM_IDX],NUM_WMCOLORS);

      XStoreColors(tie.display,tie.colormap,
          &tie.color_table[WM_IDX],NUM_WMCOLORS);

     /* 
      *  *** Assign the drawing colors; these are static.
      */
      for (i = 0; i < NUM_CVALS; i++)
         {
         status = XParseColor(tie.display,tie.colormap,wk_colors[i],
                                &tie.color_table[RGBBW_IDX + i]);
         if (status == 0)
            {
            sprintf(errmsg,"Unable to parse %s color",wk_colors[i]);
            fprintf(stderr, "%s\n",errmsg);
            exit_tiepts(0);
            }

         tie.color_table[RGBBW_IDX + i].pixel = pixelval[i + NUM_WMCOLORS];
         XStoreColor(tie.display,tie.colormap,&tie.color_table[RGBBW_IDX + i]);
         }
      
     /* 
      *  *** Assign the working colorcell for PSD/MASK color selection.
      */
      tie.color_table[WORK_IDX].pixel =  
          tie.color_table[RGBBW_IDX + BLACK_COL].pixel;
      XQueryColor(tie.display,tie.colormap,&tie.color_table[WORK_IDX]);
      tie.color_table[WORK_IDX].pixel = pixelval[NUM_WMCOLORS + NUM_CVALS];
      tie.color_workclear = tie.color_table[WORK_IDX];
      XStoreColor(tie.display,tie.colormap,&tie.color_table[WORK_IDX]);

     /* 
      *  *** Assign the color store colorcells.
      */
      for (i = 0; i < NUM_CSTORE; i++)
         tie.color_table[CSTORE_IDX + i].pixel = 
            pixelval[i + NUM_WMCOLORS + NUM_CVALS + NUM_WKCOLOR];

     /* 
      *  ***  Assign the remaining colorcells for image data.           
      */
      for (i = 0; i < NUM_IMGCOLORS; i++)
         tie.color_table[IMG_IDX + i].pixel = pixelval[i + NUM_WMCOLORS + 
                                   NUM_CVALS + NUM_WKCOLOR  + NUM_CSTORE]; 
      }
   else         /* DirectColor */
      {
      XImage *ximage = NULL;
      long *data = NULL;

      /* Assign shift values according to rgb masks returned from
         XGetVisualInfo */
      if (tie.visual_info->red_mask == 0xff)
         tie.red_shift = 0;
      else if (tie.visual_info->red_mask == 0xff0000)
         tie.red_shift = 16;
      else
         tie.red_shift = 8;

      if (tie.visual_info->green_mask == 0xff00)
         tie.green_shift = 8;
      else if (tie.visual_info->green_mask == 0xff0000)
         tie.green_shift = 16;
      else
         tie.green_shift = 0;

      if (tie.visual_info->blue_mask == 0xff0000)
         tie.blue_shift = 16;
      else if (tie.visual_info->blue_mask == 0xff)
         tie.blue_shift = 0;
      else
         tie.blue_shift = 8;

      /* Create an X Image to allow the byte order of the X server to be 
         detected (allocate data since XDestroyImage free's the image data */
      data = (long*)malloc(sizeof(long));
      if (data == NULL)
      {
          XtWarning("unable to allocate memory");
          exit(1);
      }
      /* store a value that can be used to detect byte swapping */
      *data = 1;
      ximage = XCreateImage(tie.display,tie.visual_info->visual,
              tie.visual_info->depth,ZPixmap,0,(char *)data,1,1,8,0);

      /* if the byte order of the display is different than the byte order
         in memory where the program is running, adjust the color shift 
         to accommodate the byte swapping */
      if (((ximage->byte_order == LSBFirst) && (*(char *)data == 0)) ||
          ((ximage->byte_order == MSBFirst) && (*(char *)data == 1)))
      {
          tie.red_shift = 24 - tie.red_shift;
          tie.blue_shift = 24 - tie.blue_shift;
          tie.green_shift = 24 - tie.green_shift;
      }

      /* destroy the temporary image */
      XDestroyImage(ximage);

     /*
      *  ***  Allocate color planes.
      */
      status = XAllocColorPlanes(tie.display,tie.colormap,True,wk_pixels,
                        1,8,8,8,&red_mask,&green_mask,&blue_mask);
      if (status == 0)
         {
         fprintf(stderr, "Unable to allocate color planes\n");
         exit_tiepts(0);
         }

     /* 
      *  *** Assign the pixel values for the color cells.
      */
      for (i = 0; i < 256; i++)
         tie.color_table[i].pixel = (i) + (i << 8) + (i << 16);

     /* 
      *  *** Assign the working colors.
      */
      for (i = 0; i < NUM_COLORS; i++)
        {
        status = XParseColor(tie.display,tie.colormap,wk_colors[i],
                         &tie.color_table[RGBBW_IDX + i]);
        if (status == 0)
           {
           sprintf(errmsg,"Unable to parse %s color",wk_colors[i]);
           fprintf(stderr, "%s\n",errmsg);
           tie.color_table[RGBBW_IDX + i].flags = DoRed | DoGreen | DoBlue;
           tie.color_table[RGBBW_IDX + i].red = 
           tie.color_table[RGBBW_IDX + i].green = 
           tie.color_table[RGBBW_IDX + i].blue = 0;
           }
        }

     /* 
      *  *** Assign the working color cell for PSD/MASK color selection.        
      */
      status = XParseColor(tie.display,tie.colormap,"black",&col_cell);
      if (status == 0 )
         {
         strcpy(errmsg,"Unable to parse black color");
         fprintf(stderr, "%s\n",errmsg);
         col_cell.red = col_cell.green = col_cell.blue = 0;
         col_cell.flags = DoRed | DoGreen | DoBlue;
         }

      tie.color_table[WORK_IDX].red   = tie.color_workclear.red = col_cell.red;
      tie.color_table[WORK_IDX].green = tie.color_workclear.green
          = col_cell.green;
      tie.color_table[WORK_IDX].blue  = tie.color_workclear.blue =col_cell.blue;
      tie.color_table[WORK_IDX].flags = tie.color_workclear.flags
          = col_cell.flags;

      XStoreColor(tie.display,tie.colormap,&tie.color_table[WORK_IDX]);

     /*  
      *  *** Assign a gray scale ramp to the image data colors.         
      */
      slope = (256.0 / (float)NUM_IMGCOLORS);
      offset = (0 - (slope * (float)(NUM_COLORS + NUM_CSTORE + NUM_WMCOLORS 
          + NUM_WKCOLOR)));

      for (i = IMG_IDX; i < (IMG_IDX + NUM_IMGCOLORS); i++)
         {
         tie.color_table[i].red = tie.color_table[i].green = 
              tie.color_table[i].blue = 
              (int)((i * slope) + offset) << 8;
         tie.color_table[i].flags = DoRed | DoGreen | DoBlue;
         }

      XStoreColors(tie.display,tie.colormap,tie.color_table,256);
      }

  /*
   *  Set the String -> Pixel resource converter.
   */
   XtSetTypeConverter(XtRString,XtRPixel,TIE_StringToPixel,color_ConvertArgs,
                        XtNumber(color_ConvertArgs),XtCacheByDisplay,NULL);

  /*
   *  Register the converter.
   */
   XmRegisterConverters();

   if ((tie.font_info = 
       XLoadQueryFont(tie.display,wk_fonts[DEFFONT])) == NULL)
       fprintf(stderr, "Unable to load general font\n");

  /*
   *  Initialize globals with default values.
   */
   tie.logo      = TRUE;
   tie.userplace = FALSE;
   tie.cindex = 0;
   tie.flags  = 0;
   tie.line_x = tie.line_y = -1;
   tie_adjust.color_adjust = (XColor *)NULL;
   tie_adjust.color_linear = (XColor *)NULL;
   tie_adjust.lut_slope  = 45.0;
   tie_adjust.lut_offset =  0.0;
   tie_adjust.full_lut[REFERENCE] = NULL; 
   tie_adjust.full_lut[SEARCH] = NULL; 
   tie_adjust.slope = 2.5;
   tie_adjust.offset = 175; 
   tie_cursor.pointer_pixmap = NULL;
   tie_maplist = NULL;
   tie_mapparm.gotdatums = FALSE;
   tie_parms.flags = PARMS_LINESAMP;
   register_dialog_form = NULL;
   tie_select.rtemp_flag = FALSE;
   for (i=0; i<15; i++)
      tie_cursor.projparm[i] = 0.0;

   /*
    * Set signal traps.
    */
    signal(SIGCLD,  help_reset);
    tie.help_pid = -1;

  /*
   *  *** Do the initial memory plane allocation.
   */
   tie_layout.view_width  = tie_mem[REFERENCE].view_width 
                          = tie_mem[SEARCH].view_width
                          = DEF_VWIDTH;
   tie_layout.mem_height  = tie_mem[REFERENCE].mem_height 
                          = tie_mem[SEARCH].mem_height
                          = DEF_HEIGHT;
   tie_layout.mem_width   = tie_mem[REFERENCE].mem_width 
                          = tie_mem[SEARCH].mem_width
                          = DEF_WIDTH;
   tie_layout.view_height = tie_mem[REFERENCE].view_height 
                          = tie_mem[SEARCH].view_height
                          = DEF_VHEIGHT;

  /*
   *  *** Retrieve the key codes used for single stepping of the cursor.
   */
   tie_cursor.h_key    = XKeysymToKeycode(tie.display,XK_h);
   tie_cursor.j_key    = XKeysymToKeycode(tie.display,XK_j);
   tie_cursor.k_key    = XKeysymToKeycode(tie.display,XK_k);
   tie_cursor.l_key    = XKeysymToKeycode(tie.display,XK_l);
   tie_cursor.left_key = XKeysymToKeycode(tie.display,XK_Left);
   tie_cursor.down_key = XKeysymToKeycode(tie.display,XK_Down);
   tie_cursor.up_key   = XKeysymToKeycode(tie.display,XK_Up);
   tie_cursor.right_key= XKeysymToKeycode(tie.display,XK_Right);
   tie_cursor.ret_key  = XKeysymToKeycode(tie.display,XK_Return);
   XParseColor(tie.display,tie.colormap,"green",&tie_cursor.color);

   tie.current_plane = 0;
   tie.x = tie.y = 0;
   tie.rview = 0;

  /*
   *  *** get application resources 
   *      (need to create a widget to do this, so create the mode shell 
   *       which may be used later or destroyed.)
   */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "Tie Points"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Mode"); argcnt++;
   XtSetArg(args[argcnt], XmNgeometry, "+0-0"); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
   Shell_Mode = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
   XtVaGetApplicationResources(Shell_Mode,&tie_appres,resources,
             XtNumber(resources),NULL);
   if (strcmp(tie_appres.mode,"IM2IM") == 0)
      tie.mode = IM2IM;
   else if (strcmp(tie_appres.mode,"IM2MAP") == 0)
      tie.mode = IM2MAP;
   else if (strcmp(tie_appres.mode,"IM2VECT") == 0)
      tie.mode = IM2VECT;
   else if (strcmp(tie_appres.mode,"ONESRCH") == 0)
      tie.mode = ONESRCH;
   else if (strcmp(tie_appres.mode,"IM2ADRG") == 0)
      tie.mode = IM2ADRG;
   else
      tie.mode = -1;

  /*
   *  *** parse command line arguments
   */
   parseargs(argc, argv);
   settraps();

  /*
   *  *** if mode set in arguments, init mode specific variables
   */
   if (tie.mode != -1)
      {
      XtDestroyWidget(Shell_Mode);
      mode_init(NULL, (caddr_t)tie.mode, NULL);
      }

  /*
   *  *** otherwise, display mode selection panel
   */
   else
      {
     /*
      *  *** Create the mode panel shell.
      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "Tie Points"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "Mode"); argcnt++;
      XtSetArg(args[argcnt], XmNgeometry, "+0-0"); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_Mode = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      */
      Mode_form = Createmode_form(Shell_Mode);
      XtManageChild(Mode_form);
      XtRealizeWidget(Shell_Mode);
      }

  /*
   *  *** initialize registration parameters
   */
   register_init();

  /*
   *  *** Solicit events.
   */
   XtAppMainLoop(tie.context);
}

static void settraps(void)
{
   signal(SIGBUS,  exit_tiepts);
   signal(SIGSEGV, exit_tiepts);
   signal(SIGABRT, exit_tiepts);
   signal(SIGTERM, exit_tiepts);
}

void exit_tiepts
(
   int  sig
)
{
   int i;

   if (tie_mapparm.dtm_cnt > 0)
   {
     for (i = tie_mapparm.dtm_cnt-1; i >= 0; i--) 
       XmStringFree(tie_mapparm.xm_dtm[i]);
   }

   if ((sig) && (sig != SIGTSTP))
       /* if we've recieved a signal, close the tiepts file. */
       {
       if (tie_select.ref_desc != NULL)
           c_ptclse(&tie_select.ref_desc);
       if (tie_select.srch_desc != NULL)
           c_ptclse(&tie_select.srch_desc);
       }

   if (sig != 0)
      {
      switch(sig)
         {

         /* In the case of SEGV and BUS, we want to be able to clean up
            effectively, but still want to have a core dump.  In this case,
            since the signal handler is not re-armed without an additional
            call to signal(), we simply raise the trapped signal again to
            acquire the default behavior. -cje */

         case SIGBUS:
         case SIGSEGV:
            raise(sig); /* cause default behavior (core dump) */
            break;

         case SIGABRT:
            fprintf(stderr, "Aborted\n");
            break;

         case SIGTERM:
            fprintf(stderr, "Terminated\n");
            break;

         case SIGINT:
            fprintf(stderr, "Interrupt\n");
            return;

         case SIGTSTP:
            signal(SIGTSTP, exit_tiepts);
            fprintf(stderr, "Stop\n");
            return;

         default:
            fprintf(stderr, "Unknown signal: %d\n", sig);
         }

      if (sig != SIGTERM)
         fprintf(stderr, "Sorry.....TIEPTS died....\n");

      }

#ifdef DEBUG
   if (Mem_Used())
      Mem_Display(stdout);
#endif

   exit(0);
}
