/****************************************************************************
NAME: TPS_AUTO_RT

PURPOSE: Computes a rough transformation from DDR information.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    ------          ------------
  1.0           6/89    D. Steinwand    Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
        Set flags to turn projection package parameter reporting off
        If an image-to-map registration:
           Check valid flags in the search image DDR
           Convert the output units from a string to numeric code
           Convert the input units from a string to a numeric code
           Initialize output projection coordinates to zero (geographic)
           Initialize the projection transformation package
           Place the (unconverted) projection coordinates into ref array
           Place the image coordinates of the corners into search array
           Convert the ref (projection coords) array to geographics
           Return
        If an image-to-image registration:
           Check valid flags in the search image DDR
           Check valid flags in the reference image DDR
           Find coefficients mapping line/sample to projection (x,y)
                in the reference image
           Invert the coefficients so they map projection coords to image
           Convert the input units from a string to a numeric code
           Convert the output units from a string to a numeric code
           Place the search image corner projection coordinates into temp array
           Check the projection systems of the input and output images.  If
                the are both UTM or State Plane with the same zones, do not
                convert the coordinates (set a flag)...
           If converting, initialize the projection transformation package
           If converting, convert each search image projection coordinate
                to a coordinate in the reference image's projection 
                coordinate system.
           Place the search image corner coordinates in the search array
           Using the coefficients calculated above, compute the corresponding
                reference image location (image coordinates) for each search
                image corner coordinate
           Return
*****************************************************************************/
#include "tiepts.h"
#include "util.h"
#include "proj.h"
#include "convert.h"
#include "tpssup.h"

int tps_auto_rt
(
    struct DDR *ref_ddr,        /* Reference image DDR (not used if I2M) (I) */
    struct DDR *sea_ddr,        /* Search image DDR (I)                      */
    int  mode,                  /* Mode--I2I,I2M,I2V (I)                     */
    char *units,                /* Units of output I2M space (I)             */
    double ref_yx[4][2],        /* Reference space coordinates (O)           */
    double sea_yx[4][2]         /* Search image coordinates (O)              */
)
{
  double inx;                 /* X coords for proj package                 */
  double iny;                 /* Y coordinates for proj package            */
  double temp_cc[6];          /* Array of temporary coefficients           */
  double cc[6];               /* Coefficients mapping proj to l,s          */
  double pc[8];               /* Projection coordinates                    */
  double parms[15];           /* Array of 15 projection parameters         */
  int  i;
  int  j;                     /* Loop counters                             */
  long imc[8];                /* Image coordinates                         */
  long prt_flag;              /* Error flag for c_rtcoef                   */
  long twelve = 12;           /* Constant of 12                            */
  long zero = 0;              /* Constant of 0                             */
  long null_zone = 62;        /* Null zone code for geographic             */
  long in_units;              /* Input unit code                           */
  long out_units;             /* Output unit code                          */
  long prtprm[2];             /* Print codes for c_transinit               */
  int  convert;               /* Boolean -- convert proj coords?           */
  int  status;                /* Function return status                    */
   
  /* Turn all parameter logging from the 
     projection transformation package off
  ----------------------------------------*/
  prtprm[GEO_TERM] = FALSE;
  prtprm[GEO_FILE] = FALSE;

  /* If registration mode is IM2MAP (image to map), get the 
     parameters necessary to use the projection transformation
     package, initialize it, and convert corner projection
     coordinates to geographic coordinates
  ------------------------------------------------------------*/
  if ((mode == IM2MAP) || (mode == IM2VECT) || (tie.flags & TIE_ADRG))
    {
    status = check_ddr(sea_ddr);
    if (status != E_SUCC)
      return(E_FAIL);

    /* Convert output units to unit code
    ------------------------------------*/
    if (c_ckunit(units, &out_units) != E_SUCC)
        return (E_FAIL);
    if (c_ckunit(sea_ddr->proj_units, &in_units) != E_SUCC)
        return (E_FAIL);

    /* Initialize output projection parameter array to zero
    -------------------------------------------------------*/
    for (i = 0; i < 15; i++)
      parms[i] = 0.0;

    /* Initialize the Projection Transformation Package
    ---------------------------------------------------*/
    if ((tie.mode == IM2MAP) || (tie.flags & TIE_ADRG))
      {
      status = c_transinit(&sea_ddr->proj_code, &in_units, &sea_ddr->zone_code, 
                           &sea_ddr->datum_code, sea_ddr->proj_coef, &zero,
                           &out_units, &null_zone, &sea_ddr->datum_code, parms,
                           prtprm, "\0");
      if (status != E_SUCC)
        return(E_FAIL);
      }

    /* Set up output arrays.  The reference system is initially in projection
     coordinates as contained in the DDR (not yet converted to geographics)
    ----------------------------------------------------------------------*/
    ref_yx[0][0] = sea_ddr->upleft[1];
    ref_yx[1][0] = sea_ddr->loleft[1];
    ref_yx[2][0] = sea_ddr->upright[1];
    ref_yx[3][0] = sea_ddr->loright[1];
    ref_yx[0][1] = sea_ddr->upleft[0];
    ref_yx[1][1] = sea_ddr->loleft[0];
    ref_yx[2][1] = sea_ddr->upright[0];
    ref_yx[3][1] = sea_ddr->loright[0];
    sea_yx[0][1] = 1.0;
    sea_yx[1][1] = sea_ddr->nl;
    sea_yx[2][1] = 1.0;
    sea_yx[3][1] = sea_ddr->nl;
    sea_yx[0][0] = 1.0;
    sea_yx[1][0] = 1.0;
    sea_yx[2][0] = sea_ddr->ns;
    sea_yx[3][0] = sea_ddr->ns;

    /* Process each reference coordinate to obtain
       geographic coordinates in the units
       specified by the user
    ----------------------------------------------*/
    if ((tie.mode == IM2MAP) || (tie.flags & TIE_ADRG))
      for (i = 0; i < 4; i++)
        {
        inx = ref_yx[i][0];
        iny = ref_yx[i][1];
        status = c_trans(&sea_ddr->proj_code, &in_units, &zero, &out_units, 
                         &inx, &iny, &(ref_yx[i][0]), &(ref_yx[i][1]));
        if (status != E_SUCC) 
          return(E_FAIL);
        }

    return(E_SUCC);
    }

  /* If registration mode is IM2IM (image to image), find polynomial
     coefficients mapping reference projection coordinates to reference
     image coordinates, convert search image corners (in projection
     coordinates) to the projection coordinate system of the reference
     image, and compute the reference image location of search image
     corner projection coordinates.
  ---------------------------------------------------------------------*/
  if (mode == IM2IM)
    {
    status = check_ddr(sea_ddr);
    if (status != E_SUCC)
      return(E_FAIL);
    status = check_ddr(ref_ddr);
    if (status != E_SUCC)
      return(E_FAIL);

    /* Find coefficients mapping reference projection
       coordinates to reference line, sample coordinates
    ----------------------------------------------------*/
    pc[0] = ref_ddr->upleft[0];
    pc[1] = ref_ddr->upleft[1];
    pc[2] = ref_ddr->loleft[0];
    pc[3] = ref_ddr->loleft[1];
    pc[4] = ref_ddr->upright[0];
    pc[5] = ref_ddr->upright[1];
    pc[6] = ref_ddr->loright[0];
    pc[7] = ref_ddr->loright[1];
    imc[0] = imc[1] = 1;
    imc[2] = ref_ddr->nl;
    imc[3] = 1;
    imc[4] = 1;
    imc[5] = ref_ddr->ns;
    imc[6] = ref_ddr->nl;
    imc[7] = ref_ddr->ns;
    prt_flag = TRUE;

    /* Get the transformation coefficients
    --------------------------------------*/
    status = c_rtcoef(pc,imc,temp_cc,&prt_flag);
    if (status != E_SUCC)
      return(E_FAIL);
    status = c_invert(temp_cc, cc);
    if (status != E_SUCC)
      return(E_FAIL);

    /* Convert the projection corner coordinates in the
       search image ddr to projection coordinates in the
       reference image projection system
    ----------------------------------------------------*/
    c_up2low(sea_ddr->proj_units, &twelve);
    if (strcmp(sea_ddr->proj_units, "radians") == 0) in_units = 0;
    else if (strcmp(sea_ddr->proj_units, "feet") == 0) in_units = 1;
    else if (strcmp(sea_ddr->proj_units, "meters") == 0) in_units = 2;
    else if (strcmp(sea_ddr->proj_units, "seconds") == 0) in_units = 3;
    else if (strcmp(sea_ddr->proj_units, "degrees") == 0) in_units = 4;
    else if (strcmp(sea_ddr->proj_units, "dms") == 0) in_units = 5;
    else return(E_FAIL);

    c_up2low(ref_ddr->proj_units, &twelve);
    if (strcmp(ref_ddr->proj_units, "radians") == 0) out_units = 0;
    else if (strcmp(ref_ddr->proj_units, "feet") == 0) out_units = 1;
    else if (strcmp(ref_ddr->proj_units, "meters") == 0) out_units = 2;
    else if (strcmp(ref_ddr->proj_units, "seconds") == 0) out_units = 3;
    else if (strcmp(ref_ddr->proj_units, "degrees") == 0) out_units = 4;
    else if (strcmp(ref_ddr->proj_units, "dms") == 0) out_units = 5;
    else return(E_FAIL);

    pc[0] = sea_ddr->upleft[0];
    pc[1] = sea_ddr->upleft[1];
    pc[2] = sea_ddr->loleft[0];
    pc[3] = sea_ddr->loleft[1];
    pc[4] = sea_ddr->upright[0];
    pc[5] = sea_ddr->upright[1];
    pc[6] = sea_ddr->loright[0];
    pc[7] = sea_ddr->loright[1];

    convert = TRUE;
    if ((sea_ddr->proj_code == ref_ddr->proj_code) && 
        ((sea_ddr->proj_code == 1) || (sea_ddr->proj_code == 2)) &&
         (sea_ddr->zone_code == ref_ddr->zone_code))
      convert = FALSE;

    /* Initialize the Projection Transformation Package
    ---------------------------------------------------*/
    if (convert)
      {
      status = c_transinit(&sea_ddr->proj_code, &in_units, &sea_ddr->zone_code, 
                           &sea_ddr->datum_code, sea_ddr->proj_coef,
                           &ref_ddr->proj_code, &out_units, &ref_ddr->zone_code,
                           &ref_ddr->datum_code, ref_ddr->proj_coef,
                           prtprm, "\0");
      if (status != E_SUCC)
        return(E_FAIL);
      }

    /* Process each search coordinate 
    ---------------------------------*/
    if (convert)
      for (i = 0, j = 0; i < 4; j+=2, i++)
        {
        iny = pc[j];
        inx = pc[j + 1];
        status = c_trans(&sea_ddr->proj_code, &in_units, &ref_ddr->proj_code,
                         &out_units, &inx, &iny, &(pc[j + 1]), &(pc[j]));
        if (status != E_SUCC)
          return(E_FAIL);
        }

    /* The search image corner coordinates are used as-is
    -----------------------------------------------------*/
    sea_yx[0][1] = 1.0;
    sea_yx[1][1] = sea_ddr->nl;
    sea_yx[2][1] = 1.0;
    sea_yx[3][1] = sea_ddr->nl;
    sea_yx[0][0] = 1.0;
    sea_yx[1][0] = 1.0;
    sea_yx[2][0] = sea_ddr->ns;
    sea_yx[3][0] = sea_ddr->ns;

    /* The reference image coordinates are calculated from the
       re-projected corners of the search image.  In other words,
       where does the search image's corners fall in the reference
       image (in projection space) and what are the coressponding
       image coordinates?  Coordinates are truncated to integers
       to avoid matrice problems in the fit routine.
    --------------------------------------------------------------*/
    ref_yx[0][1] = (int)((cc[0] * pc[0]) + (cc[1] * pc[1]) + cc[2] + 0.5);
    ref_yx[0][0] = (int)((cc[3] * pc[0]) + (cc[4] * pc[1]) + cc[5] + 0.5);
    ref_yx[1][1] = (int)((cc[0] * pc[2]) + (cc[1] * pc[3]) + cc[2] + 0.5);
    ref_yx[1][0] = (int)((cc[3] * pc[2]) + (cc[4] * pc[3]) + cc[5] + 0.5);
    ref_yx[2][1] = (int)((cc[0] * pc[4]) + (cc[1] * pc[5]) + cc[2] + 0.5);
    ref_yx[2][0] = (int)((cc[3] * pc[4]) + (cc[4] * pc[5]) + cc[5] + 0.5);
    ref_yx[3][1] = (int)((cc[0] * pc[6]) + (cc[1] * pc[7]) + cc[2] + 0.5);
    ref_yx[3][0] = (int)((cc[3] * pc[6]) + (cc[4] * pc[7]) + cc[5] + 0.5);
    }

  return(E_SUCC);
}
