/******************************************************************************
NAME: TPS_CPINTPS

PURPOSE: This routine is used to read either reference or search input
         TPS files and write them to the output TPS files.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/99    T.Ratliff       Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
******************************************************************************/
#include "tiepts.h"
#include "convert.h"
#include "tpssup.h"

int tps_cpintps
(
    struct PTDESC **indesc,                /* Input TPS file descriptor    */
    char *hdrmode,                         /* Type of coords in file       */
    char *mesg,                            /* Error message(s) to show     */
    int  imgidx                            /* Image index (REF or SEARCH)  */
)
{
  struct TPSDATA ptdata;                 /* TPS file data record struct  */
  int  id_flg = FALSE;                   /* Flag indicating if the point
                                            id array has been filled     */
  int  id_ok;                            /* Whether point id is valid    */
  int  inv_pts = FALSE;                  /* Flag invalid point(s) found  */
  int  length;                           /* String length                */
  int  srchnum;                          /* Number of search points      */
  int  status;                           /* Return status of functions   */

  /* Reading Reference input file
  -------------------------------*/
  if (imgidx == REFERENCE)
    {
    /* Reading in first point
    -------------------------*/
    status = c_ptio(indesc,(long *)&ptdata);
    while (status != E_EOF)
      {
      /* Allocate memory for the selected tiepoints
      ---------------------------------------------*/
      if (tie_select.numtie == tie_select.numalloc)
        {
        tie_select.numalloc += MAX_TIE_POINTS;
        tie_select.elev    =(double *)(realloc(tie_select.elev,
                            tie_select.numalloc*sizeof(double)));
        tie_select.rtie    =(void *)(realloc(tie_select.rtie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.crtie    =(void *)(realloc(tie_select.crtie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.stie    =(void *)(realloc(tie_select.stie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.lsrtie  =(void *)(realloc(tie_select.lsrtie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.resid   =(void *)(realloc(tie_select.resid,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.pt_id   =(void *)(realloc(tie_select.pt_id,
                            tie_select.numalloc*PTIDLEN*sizeof(char)));
        tie_select.mapname =(void *)(realloc(tie_select.mapname,
                            tie_select.numalloc*80*sizeof(char)));
        tie_select.mapid   =(void *)(realloc(tie_select.mapid,
                            tie_select.numalloc*80*sizeof(char)));
        tie_select.disabled=(unsigned char *)(realloc(tie_select.disabled,
                            tie_select.numalloc*sizeof(unsigned char)));
        }

      /* Write the tiepoint to output file
      ------------------------------------*/
      c_ptio(&tie_select.ref_desc,(long *)&ptdata);

      /* Load the data into appropriate variables
      -------------------------------------------*/
      if ((tie.mode == IM2MAP) || (tie.mode == IM2VECT) ||
          (tie.flags & TIE_ADRG))
        {
        tie_select.rtie[tie_select.numtie][1] = ptdata.coord[0];
        tie_select.rtie[tie_select.numtie][0] = ptdata.coord[1];
        }
      else if (tie.mode == IM2IM)
        {
        if (tie_parms.flags & PARMS_LINESAMP)
          {
          tie_select.rtie[tie_select.numtie][1] = ptdata.image_coord[0];
          tie_select.rtie[tie_select.numtie][0] = ptdata.image_coord[1];
          }
        else if ((tie_parms.flags & PARMS_PROJ) ||
                 (tie_parms.flags & PARMS_LATLONG))
          {
          tie_select.rtie[tie_select.numtie][1] = ptdata.coord[0];
          tie_select.rtie[tie_select.numtie][0] = ptdata.coord[1];
          conv_ls(hdrmode,tie_select.numtie);
          }
        }
      else
        {
        tie_select.rtie[tie_select.numtie][1] = ptdata.image_coord[0];
        tie_select.rtie[tie_select.numtie][0] = ptdata.image_coord[1];
        }
      
      tie_select.elev[tie_select.numtie] = ptdata.elevation;

      if (tie.mode == IM2MAP)
        {
        length = strcspn(ptdata.chip_name,"@");
        if (length == strlen(ptdata.chip_name))
          {
          strcpy(tie_select.mapname[tie_select.numtie],ptdata.chip_name);
          strcpy(tie_select.mapid[tie_select.numtie],"");
          }
        else
          {
          strncpy(tie_select.mapid[tie_select.numtie],ptdata.chip_name,
                  length-1);
          strcpy(tie_select.mapid[tie_select.numtie],
                 &ptdata.chip_name[length]);       
          }
        }

      /* Make sure the input point ids
         are valid
      --------------------------------*/
      id_ok = tps_checkid(ptdata.pt_id);

      /* Set flag if any invalid point ids
         so we only issue 1 message
      ------------------------------------*/
      if (!id_ok)
        inv_pts = TRUE;
      else if (id_ok)
        {
        strcpy(tie_select.pt_id[tie_select.numtie],ptdata.pt_id);
        tie_select.disabled[tie_select.numtie] = FALSE;
        tie_select.numtie++;
        }

      /* Read in the next point
      -------------------------*/
      status = c_ptio(indesc,(long *)&ptdata);
      }

    /* Close the input file and set variables
    -----------------------------------------*/
    c_ptclse(indesc);
    tie_select.seltie = tie_select.numtie;
    id_flg = TRUE;

    /* If any bad point ids, issue message
    --------------------------------------*/
    if (inv_pts)
      {
      length = strlen(mesg);
      mesg += length;
      sprintf(mesg,"Invalid point id contained in TPS file %s\n",
              tie_parms.intps[imgidx]);
      }

    return(tie_select.numtie);
    }
  else
    {
    srchnum = 0;
    /* Reading in first point of
       Search input file
    ----------------------------*/
    status = c_ptio(indesc,(long *)&ptdata);
    while (status != E_EOF)
      {
      /* Allocate memory for the selected tiepoints
      ---------------------------------------------*/
      if (srchnum == tie_select.numalloc)
        {
        tie_select.numalloc += MAX_TIE_POINTS;
        tie_select.elev    =(double *)(realloc(tie_select.elev,
                            tie_select.numalloc*sizeof(double)));
        tie_select.rtie    =(void *)(realloc(tie_select.rtie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.crtie    =(void *)(realloc(tie_select.crtie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.stie    =(void *)(realloc(tie_select.stie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.lsrtie  =(void *)(realloc(tie_select.lsrtie,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.resid   =(void *)(realloc(tie_select.resid,
                            2*tie_select.numalloc*sizeof(double)));
        tie_select.pt_id   =(void *)(realloc(tie_select.pt_id,
                            tie_select.numalloc*PTIDLEN*sizeof(char)));
        tie_select.mapname =(void *)(realloc(tie_select.mapname,
                            tie_select.numalloc*80*sizeof(char)));
        tie_select.mapid   =(void *)(realloc(tie_select.mapid,
                            tie_select.numalloc*80*sizeof(char)));
        tie_select.disabled=(unsigned char *)(realloc(tie_select.disabled,
                            tie_select.numalloc*sizeof(unsigned char)));
        }

      /* Write the tiepoint to output file
      ------------------------------------*/
      c_ptio(&tie_select.srch_desc,(long *)&ptdata);

      /* Load the data into variables
      -------------------------------*/
      tie_select.stie[srchnum][1] = ptdata.image_coord[0];
      tie_select.stie[srchnum][0] = ptdata.image_coord[1];

      /* Make sure the input point ids
         are valid
      --------------------------------*/
      id_ok = tps_checkid(ptdata.pt_id);

      /* Set flag if any invalid point ids
         so we only issue 1 message
      ------------------------------------*/
      if ((!id_ok) && (tie.mode == ONESRCH))
        /* ONESRCH requires all unique point ids
        ----------------------------------------*/
        inv_pts = TRUE;
      else if ((id_ok) && (tie.mode != ONESRCH))
        /* Other modes require srch points to have
           corresponding ref points
        ------------------------------------------*/
        inv_pts = TRUE;

      /* Display point in search image if specified
      ---------------------------------------------*/
      if (!id_flg)
        strcpy(tie_select.pt_id[srchnum],ptdata.pt_id);

      /* Read in the next point
      -------------------------*/
      srchnum++;      
      status = c_ptio(indesc,(long *)&ptdata);
      }

    /* Close the input file and set variables
    -----------------------------------------*/
    c_ptclse(indesc);
    tie_select.seltie = srchnum;

    /* If any bad point ids, issue message
    --------------------------------------*/
    if (inv_pts)
      {
      length = strlen(mesg);
      mesg += length;
      if (tie.mode == ONESRCH)
        sprintf(mesg,"Invalid point id contained in TPS file %s\n",
                tie_parms.intps[imgidx]);
      else
        sprintf(mesg,"Point ids in search TPS file do not\nmatch those in reference TPS file\n");

      }
    }

  return(srchnum);
}
