/******************************************************************************
NAME: TPS_OPEN

PURPOSE: This routine opens both the input and output tie point selection
         files.  Output files are created with the appropriate header
         information and information from the input files is copied to the
         output files.  Points from any input files are displayed in the
         input graphics plane.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/89    L.Huewe         Initial Development
 2.0            2/92    J.Hemmer        Conversion to X release
 3.0            4/93    L.Huewe         Modified to retain DDR info in TPS
                                        header for single search mode.
                                        Modified to return error status if
                                        number of search and reference points
                                        don't match.
 4.0            3/94    T.Rockvam       Put all errors in input TPS files into
                                        one popup message.
 5.0            8/98    C.Olsen         Modified to reallocate memory for the
                                        converted tie points and to change the
                                        header info when coordinate type is
                                        changed.
 6.0            1/99    C.Olsen         Added a call to tps_update when
                                        in IM2IM or IM2VECT modes.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
******************************************************************************/
#include "tiepts.h"
#include "util.h"
#include "convert.h"
#include "protos.h"
#include "tpssup.h"

int tps_open
(
    char *units,               /* Projection units (I)                   */
    struct DDR *ref_ddr        /* Reference image's DDR (I)              */
)
{
  struct TPSHEAD header;     /* TPS file header record structure       */
  struct PTDESC *indesc;     /* Input TPS file descriptor              */
  int  j;                    /* Looping variable                       */
  long length;               /* String length                          */
  int  status;               /* Status indicator                       */
  int  snum = 0;             /* Search point counter                   */
  char filnam[CMLEN];        /* Input file names                       */
  char mesg[1280];           /* Error message text                     */

  mesg[0] = '\0';

  /* Check whether or not the TPS files
     have already been opened
  -------------------------------------*/
  if (tie.flags & TIE_OPEN)
    {
    tie_message(Shell_Main,"Files already open");
    return(E_SUCC);
    }

  /* Initialize variables 
  -----------------------*/
  tie_select.numtie = 0;
  tie_select.seltie = 0;
  header.proj_valid = GEO_NOTVALID;
  for (j=0; j < 8; j++)
    header.corners[j] = 0.0;
  header.pdist[0] = header.pdist[1] = 1.0;
  for (j=0; j < 15; j++)
    header.projprms[j] = 0.0;
  header.zone = header.code = header.datum = 0;
  strcpy(header.units,"");

  /* Determine whether the projection information
     from the reference image's DDR can be used
  -----------------------------------------------*/ 
  if (((tie.mode != ONESRCH) &&
       (strcmp(tie_parms.intps[REFERENCE],"") == 0)) || (tie.mode == ONESRCH)) 
    {
    /* Check the reference DDR valid flags
    --------------------------------------*/
    if (tie.rvalid)
      {
      if (((ref_ddr->proj_code == 1) || (ref_ddr->proj_code == 2)) && 
          (ref_ddr->valid[1] != VALID))
        header.proj_valid = GEO_NOTVALID;
      else
        header.proj_valid = GEO_VALID;

      /* If the information is valid,
         copy it to the TPS file header
      ---------------------------------*/
      if (header.proj_valid == GEO_VALID)
        {
        get_ddr(&header,ref_ddr);
        if (tie_parms.flags & PARMS_MULTI)
          multi_ref(&header);
        else if (tie_parms.flags & PARMS_LATLONG)
          geo_header(&header);
        header.proj_valid = GEO_VALID;
        }
      }
    }

  /* If a reference TPS file is being created
  -------------------------------------------*/
  if (tie.mode != ONESRCH)
    {
    /* Construct the file name to be
       entered in the header record
    --------------------------------*/
    if ((tie.mode != IM2MAP) || (tie.mode == IM2VECT))
      tps_mkname(header.name,filnam,REFERENCE);
    else
      strcpy (filnam,"");

    strcpy(header.name,filnam);
    length = (long)strlen(header.name);
    c_up2low(header.name,&length);

    /* Set the appropriate header fields
       based on registration type
    ------------------------------------*/
    tps_setheader(units,&header); 

    /***********************************************************************
        Set the 15th proj parm element for datum tracking
        IM2MAP mode uses MIXED datums (diff datums for diff maps)
        IM2VECT with DLG uses NAD27
        For IM2MAP this is reset upon exit by tps_checkdatum.
        For IM2VECT it is not yet implemented.

    if (tie.mode == IM2MAP)
      header.projprms[14] = 1.0;        
    else if ((tie.mode == IM2VECT) && (tie_parms.vsource == DLGOPT))
      header.projprms[14] = 2.0;
    ************************************************************************/

    /* If Multiple reference files, clear header
       information
    --------------------------------------------*/
    if (tie_parms.flags & PARMS_MULTI)
      multi_ref(&header);

    /* Opens the output tps file and opens
       the input tps files if necessary
    --------------------------------------*/
    status = tps_fileopen(&indesc,&header,filnam,mesg,REFERENCE);
    if (status != E_SUCC)
      return(E_FAIL);

    /* If the header code and coordinate type
       do not match, generate an error message
    ------------------------------------------*/
    if ((strncmp(header.mode,"GEO",3) == 0) &&
        (!(tie_parms.flags & PARMS_LATLONG) ||
        (tie_parms.flags & PARMS_LINESAMP)))
      {
      /* Header code and output coord type must match
      -----------------------------------------------*/
      tie_message(Shell_Main,
                  "Input header code and Output Coordinate type must match");
      return(E_FAIL);
      }
    else if ((strncmp(header.mode,"PRO",3) == 0) &&
             (!(tie_parms.flags & PARMS_PROJ) ||
             (tie_parms.flags & PARMS_LINESAMP)))
      {
      /* Header code and output coord type must match
      -----------------------------------------------*/
      tie_message(Shell_Main,
                  "Input header code and Output Coordinate type must match");
      return(E_FAIL);
      }
    else if ((strncmp(header.mode,"REF",3) == 0) &&
             !(tie_parms.flags & PARMS_LINESAMP))
      {
      /* Header code and output coord type must match
      -----------------------------------------------*/
      tie_message(Shell_Main,
                  "Input header code and Output Coordinate type must match");
      return(E_FAIL);
      }

    /* Copy records from the input TPS file
       to the output TPS file
    ---------------------------------------*/
    if (strcmp(tie_parms.intps[REFERENCE],"") != 0)
      status = tps_cpintps(&indesc,header.mode,mesg,REFERENCE);
    }

  /* Initialize header record projection information
  --------------------------------------------------*/
  if (tie.mode != ONESRCH)
    {
    header.proj_valid = GEO_NOTVALID;
    for (j=0; j < 8; j++)
      header.corners[j] = 0.0;
    header.pdist[0] = header.pdist[1] = 1.0;
    for (j=0; j < 15; j++)
      header.projprms[j] = 0.0;
    header.zone = header.code = header.datum = 0;
    strcpy(header.units,"");
    }

  /* Construct the file name to be
     entered in the header record
  --------------------------------*/
  tps_mkname(header.name,filnam,SEARCH);
  strcpy(header.name,filnam);
  length = (long)strlen(header.name);
  c_up2low(header.name,&length);
  strcpy(header.mode,"SEA");

  /* Opens output tps files and opens input
     tps files if necessary
  -----------------------------------------*/
  status = tps_fileopen(&indesc,&header,filnam,mesg,SEARCH);
  if (status != E_SUCC)
    return(E_FAIL);

  /* Copy records from the input TPS
     file to the output TPS file
  ----------------------------------*/
  if (strcmp(tie_parms.intps[SEARCH],"") != 0)
    snum = tps_cpintps(&indesc,header.mode,mesg,SEARCH);

  /* If the registration type is image to image
     or image to map, make sure the number of
     input tie points match
  ---------------------------------------------*/
  if (tie.mode != ONESRCH)
    {
    if (tie_select.numtie != snum)
      {
      sprintf(&mesg[strlen(mesg)],"Incompatible reference and search TPS files input");
      tie_message(Shell_Main,mesg);
      tie.flags &= ~TIE_OPEN;
      unlink(tie_parms.outtps[REFERENCE]);
      unlink(tie_parms.outtps[SEARCH]);
      return(E_FAIL);
      }
    }
  else 
    tie_select.numtie = tie_select.seltie;

  if (strlen(mesg) > 3)
    {
    mesg[strlen(mesg)-1] = '\0';
    tie_message(Shell_Main,mesg);
    }

  tie_select.curref = FALSE;
  tie_select.genpt = 1;
  init_rt();

  /* Update output coordinate type
  --------------------------------*/
  if (tie.mode == IM2IM || tie.mode == IM2VECT)
    tps_update();

  tie.flags |= TIE_OPEN;
  return(E_SUCC);
}

/*******************************************************************************
NAME: GET_DDR

PURPOSE: This routine gets the information for the TPS header from the DDR.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            9/98    C.Olsen         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void get_ddr
(
    struct TPSHEAD *hdr,                   /* Header structure           */
    struct DDR *ddr                       /* DDR structure              */
)
{
  long i;                                /* Looping variable           */

  hdr->corners[0] = ddr->upleft[0];
  hdr->corners[1] = ddr->upleft[1];
  hdr->corners[2] = ddr->loleft[0];
  hdr->corners[3] = ddr->loleft[1];
  hdr->corners[4] = ddr->upright[0];
  hdr->corners[5] = ddr->upright[1];
  hdr->corners[6] = ddr->loright[0];
  hdr->corners[7] = ddr->loright[1];
  hdr->pdist[0] = ddr->pdist_y;
  hdr->pdist[1] = ddr->pdist_x;

  for (i = 0; i < 15; i++)
    hdr->projprms[i] = ddr->proj_coef[i];

  hdr->code = ddr->proj_code;
  hdr->zone = ddr->zone_code;
  hdr->datum = ddr->datum_code;
  strcpy (hdr->units, ddr->proj_units);
}

/*******************************************************************************
NAME: GEO_HEADER

PURPOSE: This routine changes the reference TPS header information to reflect
         geographic values.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            9/98    C.Olsen         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void geo_header
(
    struct TPSHEAD *hdr                /* Input header structure         */
)
{
  double img_cnr[8];                  /* Temporary array of corners     */
  long i;                             /* Looping variable               */

  /* Coordinates read from the DDR are in (y,x), TIEPTS
     processes in (x,y), so we will swap the values 
  -----------------------------------------------------*/
  for (i = 0; i < 8; i+=2)
    {
    img_cnr[i] = hdr->corners[i+1];
    img_cnr[i+1] = hdr->corners[i];
    }

  if (tie.mode == IM2VECT)
    proj2degrees(img_cnr, 4, SEARCH);
  else
    proj2degrees(img_cnr, 4, REFERENCE);

  /* Switch x and y values back again
  -----------------------------------*/
  for (i = 0; i < 8; i+=2)
    {
    hdr->corners[i] = img_cnr[i+1];
    hdr->corners[i+1] = img_cnr[i];
    }

  hdr->pdist[0] = hdr->pdist[1] = 0.0;

  for (i = 0; i < 15; i++)
    hdr->projprms[i] = 0.0;
    hdr->projprms[i] = 0.0;

  hdr->code = 0; /* GEOGRAPHIC */
  hdr->zone = NULZONE;
  strcpy(hdr->units, "degrees");
}

/*******************************************************************************
NAME: MULTI_REF

PURPOSE: This routine clears the reference TPS header information to reflect
         multiple reference images in Lat/Long.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            9/98    C.Olsen         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void multi_ref
(
    struct TPSHEAD *hdr                 /* Header structure.            */
)
{
   long         i;                      /* Looping variable.            */

   for (i = 0; i < 8; i++)
      hdr->corners[i] = 0.0;

   hdr->pdist[0] = hdr->pdist[1] = 0.0;

   for (i = 0; i < 15; i++)
      hdr->projprms[i] = 0.0;

   hdr->code = 0;   /* GEOGRAPHIC */
   hdr->zone = NULZONE;
   hdr->datum = 0;
   strcpy(hdr->units, "degrees");
   strcpy(hdr->name, "");   /* No image name */
}
