/*******************************************************************************
NAME: TPS_QFIT

PURPOSE: This routine calculates first-order transformation coefficients from
         a set of points for use as a rough transformation.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/89    D.Steinwand     Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include "tiepts.h"
#include "protos.h"
#include "tpssup.h"

int tps_Qfit
(
    int  npts,                  /* Number of tie point pairs (I)              */
    double rpts[][2],           /* Reference points to fit                    */
    double spts[][2],           /* Search points to fit                       */
    unsigned char *disabled,    /* Flags denoting disabled points             */
    int  *degree,               /* Degree of poly fit                         */
    double *coef_l,             /* Coefficients mapping line coordinates (O)  */
    double *coef_s              /* Coefficients mapping sample coordinates(O) */
)
{
  double*A;                   /* matrix used for LSQ fit                    */
  double *bx;
  double *by;
  double *v;
  int  i;                     /* Loop counters                              */
  int  count;                 /* enabled points index                       */
  int  npts_used;             /* number of enabled points                   */
  int  size;                  /* Number of independent variables            */

  /* Count number of points enabled for fit
  -----------------------------------------*/
  npts_used = 0;
  if (disabled != NULL) 
    {
    for (i = 0; i < npts; i++)
      if (!disabled[i])
        npts_used++;
    }
  else
    npts_used = npts;

  if ((*degree == FIRST_ORDER) || (npts_used < 7))
    {
    size = 3;
    *degree = FIRST_ORDER;
    }
  else
    size = 6;

  A  = (double*)malloc(npts_used*size*sizeof(double));
  if (A == NULL)
    return(E_FAIL);

  bx  = (double*)malloc(npts_used*size*sizeof(double));
  if (bx == NULL)
    return(E_FAIL);

  by  = (double*)malloc(npts_used*size*sizeof(double));
  if (by == NULL)
    return(E_FAIL);

   v  = (double*)malloc(npts_used*size*sizeof(double));
   if (v == NULL)
     return(E_FAIL);

  /* Loop on tie point pairs.  
  --------------------------*/ 
  count = 0;
  for (i = 0; i < npts; i++)
    {
    if ((disabled != NULL) && disabled[i])
      continue;
   
    A[count] = 1.0;
    A[count+npts_used]   = rpts[i][0];
    A[count+2*npts_used] = rpts[i][1];

    if (*degree == SECOND_ORDER)
      {
      A[count+3*npts_used] = rpts[i][0]*rpts[i][0];
      A[count+4*npts_used] = rpts[i][0]*rpts[i][1];
      A[count+5*npts_used] = rpts[i][1]*rpts[i][1];
      }

    bx[count] = spts[i][0];
    by[count] = spts[i][1];
    count++;
    }

  QR(A, npts_used, size, v);
  QR_solve(A, npts_used, size, v, bx, 0);
  QR_solve(A, npts_used, size, v, by, 0);

  for (i = 0; i < size; i++)
    {
    coef_s[i] = bx[i];
    coef_l[i] = by[i];
    }

  free(A);
  free(bx);
  free(by);
  free(v);
   
  return(E_SUCC);
}
