/*******************************************************************************
NAME: TPS_VERIFY

PURPOSE: This routine is used to calculate and print residuals allowing
         the user to decide whether to accept or reject the currently
         selected point.  If specified, the routine will also prompt
         for point IDs of accepted points and/or elevations.  If
         specified, the routine will also allow the user to determine
         which image the next point is to be selected from (the same
         full resolution image or the overview image).

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/89    L.Huewe         Initial Development
 2.0            8/90    L.Huewe         Modified to store rought trans. points
                                        in a temporary ascii file before they
                                        are written to the TPS output file.
 3.0            3/94    T.Rockvam       Removed unused parameters.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include <math.h>
#include "tiepts.h"
#include "protos.h"
#include "tpssup.h"


void tps_verify
(
    int  rt,               /* Whether selecting points for the rough
                              transformation (I)
                               RT_SEL - Selecting RT points.
                               RT_CAL - RT has been calculated using
                                        selected points.
                               RT_DDR - RT has been calculated using
                              points calculated from DDR information     */
    int  *numtie,          /* Total number of tie points selected (I/O)  */
    int  *seltie,          /* Number of tie points selected by the
                              user (I/O)                                 */
    double rtie[][2],      /* Reference tie points (subpixel accuracy -
                              line,sample) (I)                           */
    double stie[][2]       /* Search tie points (subpixel accuracy -
                              line,sample) (I)                           */
)
{
  double calc_line;      /* Calculated line value                      */
  double calc_samp;      /* Calculated sample value                    */
  double labs;           /* Absolute value of line residual            */
  double new_res[2];     /* Residual values after the new point is
                            added to the model                         */
  double new_rms;        /* RMS error after the new point is added to
                            the model                                  */
  double old_res[2];     /* Residual values before the new point is
                            added to the model                         */
  double old_rms;        /* RMS error before the new point is added to
                            the model                                  */
  double sabs;           /* Absolute value of sample residual          */
  int  i;                /* Loop counters                              */

  /* If rough trans calculated and more than one image/map 
  --------------------------------------------------------*/
  if ((rt != RT_SEL) && (tie.mode != ONESRCH))
    {
    /* Save the current coefficients
    --------------------------------*/
    for (i=0; i < 6; i++)
      {
      tie_rt.old_lcoef[i] = tie_rt.line_coef[i];
      tie_rt.old_scoef[i] = tie_rt.samp_coef[i];
      }
    /* Add the new point to the model
    ---------------------------------*/
    tie_rt.degree = tie_parms.order;
    tps_Qfit(*numtie, rtie, stie, tie_select.disabled, 
             &tie_rt.degree, tie_rt.line_coef,
             tie_rt.samp_coef);

    /* Calculate the residuals and RMS error for the
       point based on the old model.
    ------------------------------------------------*/
    calc_samp = c_eval(&tie_rt.degree, tie_rt.old_scoef,
                       &tie_select.rtie[*seltie][0], 
                       &tie_select.rtie[*seltie][1]);
    calc_line = c_eval(&tie_rt.degree, tie_rt.old_lcoef,
                       &tie_select.rtie[*seltie][0], 
                       &tie_select.rtie[*seltie][1]);

    old_res[0] = stie[*seltie][0] - calc_samp;
    old_res[1] = stie[*seltie][1] - calc_line;
    labs = fabs(old_res[1]);
    sabs = fabs(old_res[0]);
    old_rms = sqrt ((labs * labs) + (sabs * sabs));

    /* Calculate the residuals and RMS error for the
       point based on the new model
    ------------------------------------------------*/
    calc_samp = c_eval(&tie_rt.degree, tie_rt.samp_coef,
                       &tie_select.rtie[*seltie][0], 
                       &tie_select.rtie[*seltie][1]);
    calc_line = c_eval(&tie_rt.degree, tie_rt.line_coef,
                       &tie_select.rtie[*seltie][0], 
                       &tie_select.rtie[*seltie][1]);

    new_res[0] = stie[*seltie][0] - calc_samp;
    new_res[1] = stie[*seltie][1] - calc_line;
    labs = fabs(new_res[1]);
    sabs = fabs(new_res[0]);
    new_rms = sqrt ((labs * labs) + (sabs * sabs));
    }

  verify_display(rtie[*seltie][0], rtie[*seltie][1], 
                 stie[*seltie][0], stie[*seltie][1],
                 old_res[0], old_res[1], old_rms,
                 new_res[0], new_res[1], new_rms);
}
