#include <math.h>
#include <dirent.h>
#include <X11/cursorfont.h>
#include "tiepts.h"
#include "dlgsup.h"
#include "convert.h"
#include "manage_cursor.h"
#include "protos.h"

#define NUMCATS 8

/*******************************************************************************
Purpose:
    This routine handles updates to the DLG display based on area.
By T. Rockvam, USGS/EROS Data Center, 2/94

tbeckman:  Modified to work with the new and old file dlg file naming convention
           by checking each file in the dlg directory to see if it is a feature
           we're looking for that covers the correct area.

*******************************************************************************/
void vector_dlgupdate
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
    int     anything = FALSE,
            i,j,
            feature_used[NUMCATS];
    int status;
    char    filename[100];

    static int  feature_index[NUMCATS] = 
       {2,      5,      7,      9,      15,     17,     18,     
        /*19,*/     20};
       /* feature numbers for the following file name lists */
    static const char feature_list[NUMCATS][6] = 
       {"hypso","hydro","cover","bound","pipes","roads","rail",
        /*"mtran",*/"manma"};
       /* old dlg file naming convention information */
    static const char extensions[NUMCATS][4] = 
       {".hp",".hy",".sc",".bd",".mt",".rd",".rr",".ms"};
       /* new dlg file naming convention file extensions */
    DIR *directory;      /* directory structure pointer for searching 
                            directories */
    struct dirent *entry;/* directory entry pointer for searching the 
                            directory */
    int feature;         /* feature counter loop index */

    /* initialize the feature_used array */
    for (i = 0; i < NUMCATS; i++)
       feature_used[i] = FALSE;
    i = 0;
    while (dlg_attrib[i].mincode != 0)
    {
       for (j = 0; j < NUMCATS; j++)
       {
           if ((feature_index[j] == dlg_attrib[i].mincode/10000) &&
               (dlg_attrib[i].present == TRUE))
           {
              anything = TRUE;
              feature_used[j] = TRUE;
              break;
           }
       }
       i++;
    }

    /* check that bounds are entered */
    if ((strcmp(XmTextGetString(vect_max_lat), "") == 0) ||
        (strcmp(XmTextGetString(vect_max_lon), "") == 0) ||
        (strcmp(XmTextGetString(vect_min_lat), "") == 0) ||
        (strcmp(XmTextGetString(vect_min_lon), "") == 0))
    {
        tie_message(Shell_Vector, "Must define Lat / Lon boundaries");
        return;
    }

    /* check that there are some features selected */
    if (!anything)
    {
       tie_message(Shell_Vector, "No Feature requested");
       return;
    }

    /* free all previously read vectors */
    if (tie_vector->numlines > 0)
       vector_free();

    /* check the files in the dlg directory to see if they are files we
       are interested in
       ----------------------------------------------------------------*/
    directory = opendir(tie_parms.cd);
    if (!directory)
    {
        printf ("Error opening directory %s\n",tie_parms.cd);
        return;
    }

    /* Display the watch cursor in both the image window and main panel.  */
    show_wait_cursor();

    /* loop through the directory */
    while ((entry = readdir(directory)) != NULL)
    {
        char *ext_ptr;
        int name_matched;
        
        name_matched = 0;
        
        /* check for new dlg names */
        ext_ptr = strrchr(entry->d_name,'.');
        if (ext_ptr)
        {
            /* check list of file extensions for a file that matches the
               new dlg file naming convention */
            for (feature=0; feature < NUMCATS; feature++)
            {
                /* skip features that aren't used */
                if (!feature_used[feature])
                    continue;
                    
                if (strcmp(ext_ptr,extensions[feature]) == 0)
                {
                    name_matched = 1;
                    break;
                }
            }
        }

        if (!name_matched)
        {
            /* name didn't match new dlg file names, try the old one */
            for (feature=0; feature < NUMCATS; feature++)
            {
                /* skip features that aren't used */
                if (!feature_used[feature])
                    continue;
                    
                if (strstr(entry->d_name,feature_list[feature]) != 0)
                {
                    name_matched = 1;
                    break;
                }
            }
        }

        if (name_matched)
        {
            /* the name matched one of the naming conventions, so 
               try to read it if it hasn't been loaded previously */
            sprintf(filename,"%s/%s",tie_parms.cd,entry->d_name);
            if (!vector_isloaded(filename))
            {
                status = load_dlg_file(filename);
                if (status != E_SUCC)
                {
                    /* just ignore errors loading dlg file since others 
                       will probably load and any warning messages have
                       already been issued. */
                    continue;
                }
            }
            else
               continue;
        }
    }

    closedir(directory);

    /* update the vector display with the new DLG files read */
    update_dlg_display();

    /* Redisplay the default arrow cursor */
    unshow_wait_cursor();

}

/*----------------------------------------------------------------------
ROUTINE: load_dlg_file

PURPOSE:
        Load a single DLG file and take care of issuing an error message only
        when an actual error occurs reading the file.

INPUTS:
        filename - The name of the DLG file to read

OUTPUTS:
        The tie_vector and tie_vectfile global variables are updated with the
        DLG data read.

RETURNS:
        E_SUCC on success
        E_FAIL on error reading file
        DLGREAD_NO_LINES when no lines were read from the file
        DLGREAD_WRONG_AREA when the file does not intersect area requested

----------------------------------------------------------------------*/

int load_dlg_file
(
    char *filename          /* I: name of dlg file to load */
)
{
    int status;

    /* read the dlg file and handle the error conditions */
    status = dlg_optread(filename);
    if (status != E_SUCC)
    {
        /* issue an error if E_FAIL returned */
        if (status == E_FAIL)
        {
            fprintf(stderr,"Error reading %s.\n",filename);
        }
    }
    return (status);
}

/*----------------------------------------------------------------------
ROUTINE: update_dlg_display

PURPOSE:
        This routine updates the DLG window after new DLG files have been
        loaded.  It takes care of converting all the DLG lines to the same
        projection and updating the display.
        
        NOTE: If a valid projection with a datum is available for the search 
        image, all the DLG lines are converted to that projection, otherwise 
        all the lines are converted to the same projection as the first DLG file 
        read.  The search image must be a datum since all DLG files have 
        datums and conversions between datums and non-datums is not allowed.

INPUTS:
        global tie_vector structure and tie_vectfile array

OUTPUTS:
        The tie_vector array is updated with the new projection and the 
        DLG display is updated.

RETURNS:
        E_SUCC on success
        E_FAIL on error 

----------------------------------------------------------------------*/

void update_dlg_display(void)
{
    int line;               /* line index */
    int file;               /* file index */
    int search_index;       /* search image plane index */
    int i;                  /* loop counter */

    /* if the search image projection is invalid or it has a datum less
       than 100 (i.e. simple spheroid), 
       if so do vect2image, otherwise use generic projection
       convertion for between the first dlg file and the current */
    search_index = tie_composite[SEARCH].rgbplanes[RED] - 1;
    if ((!tie_planes[search_index].proj_valid) || 
        (tie_planes[search_index].ddr.datum_code < 100))
        tie_vector->srch_datum_exists = FALSE;
    else
        tie_vector->srch_datum_exists = TRUE;

    /* convert all the files read */
    for (file = 0; file < tie_vector->numfiles; file++)
    {
        if (tie_vector->srch_datum_exists)
        {
            /* step through all the lines for the current file */
            for (line = tie_vectfile[file].startline;
                 line <= tie_vectfile[file].endline; line++)
            { 
                /* convert all points to image proj coords */
                if (vect2image(tie_vector->line[line].pts,
                               tie_vector->line[line].npts, file) != E_SUCC)
                {
                    /* conversion failed so error out since this should
                       never happen */
                    printf ("The previous error was fatal\n");
                    exit(1);
                }
            }
        }
        else
        {
            /* step through all the lines for the current file */
            for (line = tie_vectfile[file].startline;
                 line <= tie_vectfile[file].endline; line++)
            {
                /* convert all projections to match the first vector file */
                if (vect2vect(tie_vector->line[line].pts,
                              tie_vector->line[line].npts, file) != E_SUCC)
                {
                    /* conversion failed so error out since this should
                       never happen */
                    printf ("The previous error was fatal\n");
                    exit(1);
                }
            }
        }

        if (tie_vector->srch_datum_exists)
        {
            /* copy image projection information to the vector file */
            tie_vectfile[file].inproj=tie_planes[search_index].ddr.proj_code;
            tie_vectfile[file].inunit=tie_planes[search_index].units;
            tie_vectfile[file].inzone=tie_planes[search_index].ddr.zone_code;
            tie_vectfile[file].indatum = 
                                    tie_planes[search_index].ddr.datum_code;
            for (i = 0; i < 15; i++)
            {
                tie_vectfile[file].inparm[i] = 
                        tie_planes[search_index].ddr.proj_coef[i];
            }
        }
        else
        {   
            /* copy the projection information from the first vector file to 
               this one */
            tie_vectfile[file].inproj = tie_vectfile[0].inproj;
            tie_vectfile[file].inunit = tie_vectfile[0].inunit;
            tie_vectfile[file].inzone = tie_vectfile[0].inzone;
            tie_vectfile[file].indatum = tie_vectfile[0].indatum;
            for (i = 0; i < 15; i++)
                tie_vectfile[file].inparm[i] = tie_vectfile[0].inparm[i];
    
        } 
    }

    if (tie_vector->numlines > 0)
    {
       /* find the bounds of all the files and set the attributes */
       vector_dlgbounds();
       vector_dlgattribs();

       /* fit the vectors on the screen */
       vector_scaletofit(tie_vector->tmatrix);
       XClearArea(tie.display, XtWindow(ref_image_bulletinboard),0,0,0,0,False);
       vector_redrawall();
    }
    else
       tie_message(Shell_Vector, "No lines found");

}
