#include "topo.h"
#include "protos.h"

void bmaxmin
(
    float matrix[3][3],			/* matrix with pixel under  */
    float *beta,				/* beta angle */
    float coldis, 			/* distance between columns */
    float rowdis, 			/* distance between rows    */
    float aoff, 				/* mask value */
    long method, 			/* calculation method       */
    float sun_x, 				/* sun's ray vector: x-axis */
    float sun_y, 				/* sun's ray vector: y-axis */
    float sun_z				/* sun's ray vector: z-axis */
)

/*
---------------------------------------------------------------------------
NAME: BMAXMIN

FUNCTION:   Calculates the maximum drop, maximum difference, or the 
            minimum difference beta angles from a 3 x 3 matrix and returns 
            a value between zero and 90.

PROGRAM HISTORY:

    Version	date      	author      	request
    -------   	----      	------      	-------
       1     	12/20/83    	D. Steinwand     
       2     	12/23/85  	J. Lavergne     Corrected window problem
       3	7/10/88		B. Allam	Conversion
       4        08/22/90        K. Kunska       Added more detailed 
                                                documentation
       5	11/13/90	D. Akkerman	Fixed problem of zero matrix

ALGORITHM:

   If pixel under study is outside the study area then
      Set (hypot) to sqrt (coldis**2 + rowdis**2)
      Set (value) to pixel under study
      For each of the 8 surrounding matrix elements (i)
         Set (valdn) to the current matrix point
         If valdn <> the mask value then
            Set (slops(i)) to value - valdn
         Else
            Set (slops(i)) to zero
         End-if
      End-do
      If not using the minimum difference method then
         Set (slp) to zero
      Else
         Set (slp) to a maximum value for the data-type used
      End-if
      For each surrounding matrix element (i)
         If using maximum drop & slops(i) > slp then
            Set (slp) to slops(i)
            Set (idir) to i, the loop counter
         Else-If using the maximum diff & abs(slops(i)) > slp then
            Set (slp) to abs(slops(i))
            Set (idir) to i
         Else-If using the minimum diff & abs(slops(i)) > slp then	   
            Set (slp) to abs(slops(i))
            Set (idir) to i
         End-if
      End-do
      If direction of slope points to outside corners of matrix then
         Set (slope) to (slp / hypot) * 100
      Else-If direction of slope points to the middle location on
              either the top or bottom rows then
         Set (slope) to (slp / rowdis) * 100
      Else 
         Set (slope) to (slp / coldis) * 100
      End-if
      Set (slopsqr) to slope**2
      If direction of slope points to the outside corners of matrix then
         Set (slph) to signh(idir) * sqrt (slopsqr / 2)
         Set (slpv) to signv(idir) * abs (slph)
      Else-If direction of slope points to the middle location on
		either the top or bottom rows then
         Set (slph) to signh(idir) * slope
         Set (slpv) to zero
      Else
	 Set (slph) and (slpv) to zero
      End-if
      Set (c1) to sqrt(1 / (slopsqr + 1))
      Set (top) to powerval * (sun_z - sun_x * slpv - sun_y * slph)
      Set (beta) to abs (pi / 2 - acos(top)) * radians-to-degrees
      Set (buffer) to nearest integer of beta
   else
      Set (buffer) to zero
   End-if
---------------------------------------------------------------------------
*/
{
static long loff[8] = {0,0,0,1,1,2,2,2}; /* row index                 */
static long boff[8] = {0,1,2,0,2,0,1,2}; /* column index              */

long 	i, idir = 0; 			/* loop counters and indices  */

float	ftemp, 				/* temporary result as float  */
	hypot, 				/* square root of hypotenus   */
	powerval,			/* value of power function    */
	slope, 				/* current slope value        */
	slp, 				/* value range for slope      */
	slph, 				/* horizontal slope           */
	slpv, 				/* vertical slope             */
	slopsqr, 			/* square of slope value      */
	valdn,				/* other elements from matrix */
	value;				/* center element from matrix */

double  slops[8],			/* slopes                     */
	temp,				/* intermediate results       */
        top;

static short signh[8] = {1, -1, -1, 1, -1, 1, 1, -1};  /* horizontal signs */
static short signv[8] = {-1, -1, -1, -1, 1, 1, 1, 1};  /* vertical signs   */

/*
If pixel under study is outside the study area then calculate the slopes.
-------------------------------------------------------------------------
*/

if (matrix[1][1] != aoff) 
   {
   temp = (coldis * coldis) + (rowdis * rowdis);
   hypot = pow(temp, 0.5);
   value = matrix[1][1];
   for (i = 0; i < 8; i++)
      {
      valdn = matrix[loff[i]][boff[i]];
      if (valdn != aoff) 
         slops[i] = value - valdn;
      else
         slops[i] = 0.0;
      }

/*
If not using the minimum difference method then set (slp) to zero
Else set (slp) to a maximum value for the data-type used.
-----------------------------------------------------------------
*/
   if (method != 4) 
      slp = 0.0;
   else
      slp = 255.0;

/*
For each surrounding matrix element, set the range of values for slope 
according to the calculation method selected by the user.
-----------------------------------------------------------------------
*/
   for (i = 0; i < 8; i++)
      {
      if ((method == 2) && (slops[i] > slp)) 
         {
         slp = slops[i];
         idir = i + 1;
         }
      else if ((method == 3) && (abs(slops[i]) > slp))
         {
         slp = abs(slops[i]);
         idir = i + 1;
         }
      else if ((method == 4) && (abs(slops[i]) < slp)) 
         {
         slp = abs(slops[i]);
         idir = i + 1;
         }  
      }

/*
Compute the slope and the beta angle.
--------------------------------------
*/
   if (((idir == 1) || (idir == 3)) || ((idir == 6) || (idir == 8))) 
      slope = (slp / hypot);
   else if ((idir == 2) || (idir == 7)) 
      slope = (slp / rowdis);
   else
      slope = (slp / coldis);

   slopsqr = slope * slope;
   if (((idir == 1) || (idir == 3)) || ((idir == 6) || (idir == 8))) 
      {
      temp = slopsqr / 2;
      slph = signh[idir-1] * pow(temp, 0.5);
      temp = slph;
      slpv = signv[idir-1] * abs(temp);
      }
   else if ((idir == 2) || (idir == 7))
      {
      slph = signh[idir-1] * slope;
      slpv = 0.0;
      }
   else			/* zero matrix so use sun angle */
      {
      slph = slpv = 0.0;
      }
   temp = 1 / (slopsqr + 1);
   powerval = pow(temp, 0.5);
   top = powerval * (sun_z - sun_x * slpv - sun_y * slph);
   ftemp = acos(top);
   temp = (1.5708 - ftemp) * 57.2958;
   *beta = temp;
   }
else 
   {
   *beta = 0.0;
   }  
}
