#include "topo.h"
#include "protos.h"

void bmean
(
    float matrix[3][3],			/* matrix with center pixel */
    float *beta,				/* beta angle               */
    float aoff, 				/* mask value               */
    float coldis, 			/* distance between columns */
    float rowdis, 			/* distance between rows    */
    float sun_x, 				/* sun's ray vector: x-axis */
    float sun_y, 				/* sun's ray vector: y-axis */
    float sun_z				/* sun's ray vector: z-axis */
)
	
/*
----------------------------------------------------------------------
NAME: BMEAN

FUNCTION:   Calculates the mean beta angle from a 3 x 3 matrix and 
            returns a value between zero and 90.

PROGRAM HISTORY:

     Version	date      	author      	request
     -------   	----      	------      	-------
        1     	12/20/83    	D. Steinwand     
 	2     	12/23/85  	J. Lavergne     Corrected window problem
	3	7/10/88		B. Allam	Conversion
        4       08/22/90        K. Kunska       Added inline and variable
                                                documentation
	5	11/13/90	B. Ailts	Removed absolute value
						function from return
						value
ALGORITHM:

   If the pixel under study (matrix(2,2)) is <> mask value (aoff) then
      For each row in matrix
         For each column in matrix
            If (matrix(row,col)) is equal to aoff, then
               set (matrix(row,col)) to pixel under study 
            End-if
         End-do
      End-do
      Set horizontal slope (slph) to zero
      Set verticle slope (slpv) to zero
      For three slots in the matrix
         Set (slpv) to slpv + appropriate points of the matrix
         Set (slph) to slph + appropriate points of the matrix
      End-do
      Set (slph) to slph / (6 * distance between columns)
      Set (slpv) to slpv / (6 * distance between rows)
      Set (powerval) to sqrt(1 / (slpv**2 + slph**2 + 1))
      Set (top) to powerval * (sun_z - slpv * sun_x - slph * sun_y)
      Set (beta) to nearest int of abs(pi/2 - acos(top) * rad-to-degrees)
   else
      Set (beta) to zero
   End-if
------------------------------------------------------------------------------
*/
{

double  temp1, 				/* temporary result                */
	temp2,				/* temporary result                */
	top;

float	powerval,			/* value returned by power function*/
	slph, 				/* horizontal slope                */
	slpv, 				/* vertical slope                  */
	temp;				/* temporary storage for arc-cosine*/

long	ii, iz;				/* loop counters and array indices */


/*
If the pixel under study (matrix(2,2)) is <> mask value (aoff) then
If (matrix(row,col)) is equal to aoff, then set (matrix(row,col)) to pixel 
under study 
----------------------------------------------------------------------------
*/
if (matrix[1][1] != aoff) 
   {
   for (iz = 0; iz < 3; iz++)
      {
      for (ii = 0; ii < 3; ii++)
         {
         if (matrix[ii][iz] == aoff) matrix[ii][iz] = matrix[1][1];
         }
      }

/* 
Initialize the horizontal and vertical slopes
----------------------------------------------
*/
   slph = 0.0;
   slpv = 0.0;

/*
Compute the horizontal and vertical slopes and the beta angle.
---------------------------------------------------------------
*/
   for (iz = 0; iz < 3; iz++)
      {
      slph = slph + (matrix[0][iz] - matrix[2][iz]);
      slpv = slpv + (matrix[iz][2] - matrix[iz][0]);
      }
   slph = slph / (6.0 * coldis);
   slpv = slpv / (6.0 * rowdis);
   temp2 = 1.0 / ((slph * slph) + (slpv * slpv) + 1);
   powerval = pow (temp2, 0.5);
   top = powerval * (sun_z - slpv * sun_x - slph * sun_y);
   temp = acos (top);
   temp1 = 1.5708 - temp;
   *beta = temp1 * 57.2958;
   }
else
   *beta = 0.0;

}
