#include "topo.h"
#include "util.h"
#include "protos.h"

void getpar
(
    long bands[][MAXBND+1],	/* Band numbers for each input image       */
    long nbands[],		/* Number of bands per input image         */
    long *method, 		/* Method of calculation                   */
    char t1[][CMLEN],		/* Buffer for input TAE names              */
    char t2[][CMLEN],			/* Buffer for output TAE names             */
    char h1[][CMLEN],		/* Buffer for input host names             */
    char h2[][CMLEN],			/* Buffer for output host names            */
    long *nmrimg,		/* Number of input images                  */
    long *totbnd,		/* total no. of bands                      */
    long *msgtim,		/* Time processing message time interval   */
    long *opt,			/* Subcommand option                       */
    long *otype,			/* Output data type                        */
    long *nbrsiz,		/* Number of neighbouring points           */
    float *sampdist,		/* Distance between samples                */
    float *linedist, 		/* Distance between lines                  */
    float *aoff, 			/* Mask value                              */
    float *cparm,			/* Contour interval                        */
    long window[][4],	 	/* Window specifications                   */
    long *min_max,		/* Min/max value                           */
    long *nbc,			/* Neighborhood code                       */
    float *alpha,			/* Sun's elevation angle                   */
    float *gamma			/* Azimuth angle in degrees                */
)
{

/*--------------------------------------------------------------------------
NAME: GETPAR

FUNCTION:   Gets input and output parameters

PROGRAM HISTORY:

  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    4.0       07/20/89     B. Allam         Passed itype
    5.0       08/22/90     K. Kunska        Added more detailed documetation

ALGORITHM:

   Get input image parameters
   Get subcommand name
   Set numerical option code corresponding to the given subcommand
   If subcommand is "slope" or "aspect", 
      Retrieve the appropriate paramters 
      Convert method code to numeric value
      Define neighborhood size code
      Set output data type 
   End-if
   If the subcommand is "beta", 
      Get the beta angle parameters
      Convert method code to numeric value
   End-if
   If the subcommand is "contour", 
      Get contour parameters
   End-if
   Get the output file name
   Get minmax flag and processing message interval

-----------------------------------------------------------------------------
*/

static	long	fatal_err = -1;
static  long    zero = 0;		/* Value of zero			     */

char	mcode[2],		/* Method of calculation                     */
        minmax[4],		/* Min/max option                            */
	option[7], 		/* Option subcommand                         */
        ttype[5];		/* Output data type                          */

long 	imglim,			/* Maximum number of images                  */
	qflag[MAXIMG][2],	/* Window or band option flag                */
	same,			/* Flag indicating if images have same window*/
	status;			/* Return status                             */

struct	VARIABLE *var, *vo, *vc, *vm,
                 *vn, *vs, *vl, *va; 	/* Pointers into paramter block      */

/*
Get input image parameters
---------------------------
*/
imglim = MAXIMG;
status = c_getin (&block, "IN", &zero, t1, h1, nmrimg, window, bands,
                  nbands, totbnd, &imglim, &same, qflag);
if (status != E_SUCC)
   c_errmsg ("Unable to retrieve input image parameter", "topo-in", &fatal_err);

/*
Get subcommand name
---------------------
*/
var = p_find (&block, "_SUBCMD");
strcpy (option, SVAL (*var, 0));

/*
Set numerical option code corresponding to the given subcommand
---------------------------------------------------------------- 
*/
*opt = 0;
if (option[0] == 'S') *opt=1;             
if (option[0] == 'A') *opt=2;             
if (option[0] == 'C') *opt=3;             
if (option[0] == 'B') *opt=4;

/*
If subcommand is "slope" or "aspect", retrieve the appropriate paramters 
-------------------------------------------------------------------------
*/
if (*opt <= 2)  
   {
   vm = p_find (&block, "CALCMETH");
   strcpy (mcode, SVAL(*vm, 0));
   vn = p_find (&block, "NPOINTS");
   *nbrsiz = IVAL(*vn, 0);
   vs = p_find (&block, "SAMPDIST");
   *sampdist = RVAL(*vs, 0);
   vl = p_find (&block, "LINEDIST");
   *linedist = RVAL(*vl, 0);
   va = p_find (&block, "MASKVAL");
   *aoff = RVAL (*va, 0);

/*
Convert method code to numeric value
------------------------------------
*/
   if (mcode[1] == 'E') *method= 1;     
   if (mcode[1] == 'D') *method= 2;     
   if (mcode[1] == 'A') *method= 3;     
   if (mcode[1] == 'I') *method= 4;     

/*
Define neighborhood size code
-----------------------------
*/
   *nbc= 0;
   if (*nbrsiz == 4) *nbc= 1;                
   if (*nbrsiz == 8) *nbc= 2;                

/*
Set output data type 
---------------------
*/
   *otype = EBYTE;

   }

/*
If the subcommand is "beta", get the beta angle parameters
------------------------------------------------------------
*/
else if (*opt == 4)
   {
   var = p_find (&block, "CALCMETH");
   strcpy (mcode, SVAL(*var, 0));

/*
Convert method code to numeric value
-------------------------------------
*/
   if (mcode[1] == 'E') *method= 1;     
   if (mcode[1] == 'D') *method= 2;     
   if (mcode[1] == 'A') *method= 3;     
   if (mcode[1] == 'I') *method= 4;     
 
   var = p_find (&block, "SUNANG");
   *alpha = RVAL (*var, 0);
   var = p_find (&block, "AZIANG");
   *gamma = RVAL (*var, 0);
   vs = p_find (&block, "SAMPDIST");
   *sampdist = RVAL(*vs, 0);
   vl = p_find (&block, "LINEDIST");
   *linedist = RVAL(*vl, 0);
   var = p_find (&block, "MASKVAL");
   *aoff = RVAL (*var, 0);
   var = p_find (&block, "ODTYPE");
   strcpy (ttype, SVAL(*var, 0));
   if (ttype[0] == 'B')
     *otype = EBYTE;
   else
     *otype = EREAL;

   }

/*
If the subcommand is "contour", get contour parameters
--------------------------------------------------------
*/
else
   {
   vc = p_find (&block, "CONTINT");
   *cparm = RVAL(*vc, 0);
   vo = p_find (&block, "ODTYPE");
   strcpy (ttype, SVAL(*vo, 0));
   if (ttype[0] == 'I')
     *otype = EWORD;
   else
     *otype = EBYTE;
   }  /* if */

/*
Get output file name
----------------------
*/
status = c_getout (&block, "OUT", t2, h2); 
if (status != E_SUCC) 
  c_errmsg ("Unable to retrieve output image", "topo-out", &fatal_err);


/*
Get minmax flag and processing message interval
------------------------------------------------
*/
var = p_find (&block, "$MSGTIM");
*msgtim = IVAL (*var, 0);
var = p_find  (&block, "$MINMAX"); 
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y' || minmax[0] == 'y') *min_max = TRUE;
else *min_max = FALSE;

}
