#include "topo.h"
#include "protos.h"

void maxmin
(
    float matrix[3][3],		/* matrix with center as pixel under study */
    short *bufer,			/* holds slope/aspect result               */
    long method,			/* method number                           */
    long opt,			/* option code                             */
    long nbc,			/* neighborhood size code (1-2)            */
    float dist[8],		/* array for 8 directional distances       */
    float angle[9],		/* array for 9 angles used for aspect      */
    float aoff,			/* mask value                              */
    long dtype			/* data type of input image                */
)

/*
-----------------------------------------------------------------------------

NAME:	MAXMIN

FUNCTION:   Calculates the maximum drop, maximum difference, or the minimum 
            difference slope or aspect from a 3 x 3 matrix .

PROGRAM HISTORY:

  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    4.0	      07/19/89     B. Allam         Fixed problem
    5.0       08/22/90     K. Kunska        Added more detailed documentation

ALGORITHM:

   Initialize arrays.
   If pixel under study is not mask value then
      Set (value) to pixel under study
      For each of the 8 (or 4) surrounding matrix elements
         Set (valdn) to the current matrix point
         If valdn is not mask value then
            Set (slops(i)) to value - valdn
         Else
            Set (slops(i)) to zero
         End-if
      End-do
      If all the pixels in the 3x3 matrix are masked, 
         Set the output slope/aspect value to 255
      Else
         Set directional index (dir) to 8
         If using the minimum difference method then
            Set (slp) to a maximum value for the data-type used
         Else         
            Set slope variable(slp) to zero
         End-if
         For each surrounding matrix element
            If using maximum drop & slops(i) > slp then
               Set (slp) to slops(i)
               Set (dir) to i 
            Else-If using the maximum diff & abs(slops(i)) > slp then
               Set (slp) to abs(slops(i))
               Set (dir) to i
            Else-If using the minimum diff & abs(slops(i)) < slp then	   
               Set (slp) to abs(slops(i))
               Set (dir) to i
            End-if
         End-do
         If aspect is to be calculated then
            Get an angle from table at dir directional index
         Else
            If dir is in range 1 - 4 (or 1 - 8) then
               Compute percent slope
            Else
               Set percent slope = 0
            End-if
            Adjust the percent slope to fit in byte length
            Convert the value to integer type
         End-If
      End-if
   Else-If pixel under study is masked, 
      Set (bufer) to 255 as default aspect value
   End-if

-----------------------------------------------------------------------------
*/
{

long	bad, 			/* counts the number of masked values */
	cind[8],		/* column index                       */
	count, 			/* number of elevation changes        */
	dir,			/* directional index                  */
	i,			/* loop counter and array index       */
	rind[8];		/* row index                          */

float	slopct,			/* percent slope                      */
	slops[8],               /* intermediate slope values          */
	slp, 			/* value range for slope              */
	slpmax[4],		/* maximum slope values for each type */
	valdn,			/* value of the surrounding pixels    */
	value;			/* value for the pixel under study    */

/*
Initialize arrays.
-------------------
*/
cind[0] = 0; 
cind[1] = 1; 
cind[2] = 2; 
cind[3] = 1;
cind[4] = 0; 
cind[5] = 2; 
cind[6] = 2; 
cind[7] = 0;           
rind[0] = 1; 
rind[1] = 2; 
rind[2] = 1; 
rind[3] = 0;
rind[4] = 2; 
rind[5] = 2; 
rind[6] = 0; 
rind[7] = 0;              
slpmax[0] = 255.0;
slpmax[1] = 32767.0;
slpmax[2] = 2147483647.0;
slpmax[3] = 1e38;

/*
If pixel under study is not mask value, save it in value.
---------------------------------------------------------
*/
if (matrix[1][1] != aoff) 
   {
   value = matrix[1][1];
   count = 0;
   bad = 0;

/*
For each of the 8 (or 4) surrounding matrix elements, if the
surrounding pixels are not masked, compute the differences between 
the pixel under study and the surrounding pixels and store array 
slops.
If the surrounding pixels are masked, store 0's in the slops array.
Count the number of masked and not masked pixels.
--------------------------------------------------------------------
*/
   for (i = 0; i < 4*nbc; i++)            
      {
      valdn = matrix[cind[i]][rind[i]];
      if (valdn != aoff) 
         {
         slops[i] = value - valdn;
         if (slops[i] != 0.0) 
            count++;
         }
      else 
         {
         bad++;
         if (method == 4) 
            slops[i] = slpmax[dtype];
         else
            slops[i] = 0;
         }  
      }

/*
If all the pixels in the 3x3 matrix are masked, set the output
slope/aspect value to 255.
---------------------------------------------------------------
*/
   if (bad == 4*nbc) 
      *bufer = 255.0;
   else
      {
      dir = 8;                

/*
If using the minimum difference method, set (slp) to a maximum value 
for the data-type used.
If not using the minimum difference method then set slope variable 
(slp) to zero.
---------------------------------------------------------------------
*/
      if (method == 4) 
         slp = slpmax[dtype];
      else
         slp = 0;

/*
For each surrounding matrix element, set the slope value range
and the directional index according to the method selected.
---------------------------------------------------------------
*/
      for (i = 0; i < 4*nbc; i++)                
         {
         if ((method == 2) && (slops[i] > slp)) 
            {
            slp = slops[i];
            dir = i;
            }
         else if ((method == 3) && (abs(slops[i]) > slp))
            {
            slp = abs(slops[i]);
            dir = i;
            }
         else if ((method == 4) && (abs(slops[i]) < slp)) 
            {
            slp = abs(slops[i]);
            dir = i;
            }  
         }

/*
If aspect is to be calculated, get an angle from table at the 
directional index
Otherwise, if the directional index is in range 0 - 3 (or 0 - 7),
compute percent slope
If the directional index is outside range 0 - 3 (or 0 - 7),
set percent slope = 0
-------------------------------------------------------------------
*/
      if (opt == 2)              
         {
         if (count == 0) 
            *bufer = 200.0;
         else
            *bufer = (int)(angle[dir] + 0.5);
         }
      else                          
         {
         if (dir < 4*nbc) 
            slopct = (slp / dist[dir]) * 100.0;     
         else
            slopct= 0.0;
/*
Adjust the percent slope to fit in byte length.
Convert the value to integer type.
------------------------------------------------
*/
         if (slopct >= 254.0) 
            slopct = 254.0;
         *bufer = (int)(slopct + 0.5);
         }  
      }  
   }
else 

/*
If pixel under study is masked, set the outoput slope/aspect value to 255.
--------------------------------------------------------------------------
*/
   *bufer=255;
}
