#include "topo.h"
#include "protos.h"

void mean
(
    float matrix[3][3],	/* 3 x 3 matrix with center as pixel under study */
    short *bufer,		/* holds slope/aspect result upon return         */
    long opt,		/* option code(1-3)                              */
    long nbc,		/* neighborhood size code (1-2)                  */
    float coldis, 	/* column distance                               */
    float rowdis,		/* row distance                                  */
    float aoff		/* mask value                                    */
)

/*
---------------------------------------------------------------------------

NAME:	MEAN

FUNCTION:   Calcualates the mean slope or aspect from a 3 x 3 matrix and 
            returns a value between 0 and 254.

PROGRAM HISTORY:

  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    4.0       09/15/89     B. Allam         Reversed matrix order
    5.0       08/22/90     K. Kunska        Added more detailed documentation

ALGORITHM:

   Initialize arrays
   If the pixel under study (matrix(1,1)) is not = mask value (aoff) then
      For each row in matrix
         For each collumn in matrix
            If (matrix(row,col)) is equal to mask value, then
               Set (tmatrix(row,col)) to the pixel under study
            Else
               Set (tmatrix(row,col)) to matrix(row,col)
            End-if
         End-do
      End-do
      If tmatrix contains any non-masked values then:
         Set horizontal slope (slph) to zero
         Set verticle slope (slpv) to zero
         Extract row and column ranges from table
         Compute differences between east and west values and 
         Sum them up for horizontal slope
         Compute differences between north and south values and
         Sum them up for vertical slope
         Compute average horizontal and vertical slopes
         If aspect is to be calculated then
            If vertical slope is zero then
               Assign constant angles to aspect according to sign of horizontal slope
            Else
               Compute aspect from horizontal and vertical slope
               If vertical slope > 0 then
                  Add 180 to aspect
               Else
                  If horizontal slope > 0, 
                     Add 360 to aspect
                  End-if
               Else-if
            End-if
            Divide aspect by 2 to fit it in byte 
            Store aspect in integer variable
         Else
            Compute percent slope by multilplying 100
            Adjust the value to fit in byte length
            Convert the value to integer
         End-if
      Else-If all values in tmatrix are mask values,
         Set (bufer) to 255 as default aspect value
   Else-If the pixel under study (matrix(1,1)) = mask value(aoff) then
      Set (bufer) to 255 as default aspect value
   End-if
--------------------------------------------------------------------------
*/
{

double  temp;			/* intermediate result                   */

float	slph,			/* horizontal slope                      */
	slpv,			/* vertical slope                        */
	slopct,			/* percent slope value                   */
	aspect = 0.0,	/* percent aspect value                  */
	tmatrx[3][3],		/* matrix with intermediate values       */
	dn[2];                   

long	cind[8],		/* column indices                        */
	count, 			/* counter for non-mask values of matrix */
	end_ind,		/* ending index for matrix processing    */
	i, 			/* loop counter and array index          */
	itab[2][2],		/* matrix indices based on neighborhood  */
				/* size code                             */
	rind[8],		/* row indices                           */
	start_ind;		/* starting index for matrix processing  */

/*
Initialize arrays
-----------------
*/
cind[0] = 0; 
cind[1] = 1; 
cind[2] = 2; 
cind[3] = 1;
cind[4] = 0; 
cind[5] = 2; 
cind[6] = 2; 
cind[7] = 0;
rind[0] = 1; 
rind[1] = 2; 
rind[2] = 1; 
rind[3] = 0;
rind[4] = 2; 
rind[5] = 2; 
rind[6] = 0; 
rind[7] = 0;
itab[0][0] = 1; 
itab[0][1] = 1;
itab[1][0] = 0; 
itab[1][1] = 2;
dn[0] = 2.0; 
dn[1] = 6.0;

/* 
If the pixel under study (matrix(2,2)) is not = mask value (aoff) 
then for each row and column in matrix, if (tmatrix(row,col)) is equal 
to mask value, then set (tmatrix(row,col)) to the pixel under study.
Otherwise, set (tmatrix(row,col)) to matrix(row,col).
-------------------------------------------------------------------
*/
if (matrix[1][1] != aoff) 
   {
   count = 0;
   for (i = 0; i < 4*nbc; i++)
      {
      if (matrix[cind[i]][rind[i]] == aoff) 
         tmatrx[cind[i]][rind[i]] = matrix[1][1];
      else
         {
         tmatrx[cind[i]][rind[i]] = matrix[cind[i]][rind[i]];
         count++;
         }
      }

/* 
If tmatrix contains any non-masked values then:
Set horizontal slope (slph) to zero
Set verticle slope (slpv) to zero
Extract row and column ranges from table
-------------------------------------------
*/
   if (count != 0) 
      {
      slph = 0;
      slpv = 0;
      start_ind = itab[nbc-1][0];
      end_ind = itab[nbc-1][1];

/*
Compute differences between east and west values and sum them up for 
horizontal slope.
Compute differences between north and south values and sum them up for 
vertical slope.
Compute average horizontal and vertical slopes.
-----------------------------------------------------------------------
*/
      for (i = start_ind; i <= end_ind; i++)
         {
         slpv = slpv + (tmatrx[0][i] - tmatrx[2][i]);
         slph = slph + (tmatrx[i][2] - tmatrx[i][0]);
         }
      slph = slph / (dn[nbc-1] * coldis);
      slpv = slpv / (dn[nbc-1] * rowdis);

/* 
If aspect is to be calculated then :
If vertical slope is zero, assign constant angles to aspect according to 
sign of horizontal slope.
----------------------------------------------------------------------------
*/
      if (opt == 2) 
         {
         if (slpv == 0.0) 
            {
            if (slph < 0.0) aspect = 90.0;
            if (slph > 0.0) aspect = 270.0;
            if (slph == 0.0) aspect = 400.0;
            }
         else
/*
If vertical slope is not zero, compute aspect from horizontal and vertical 
slope.
If vertical slope > 0 then add 180 to aspect.
If vertical slope <= 0 but horizontal slope > 0, add 360 to aspect.
---------------------------------------------------------------------------
*/
            {
            aspect = atan (slph / slpv) * 57.2958;          
            if (slpv > 0.0) 
               aspect = aspect + 180.0;
            else
               {
               if (slph > 0.0) 
                  aspect = aspect + 360.0;
               }  
            }  
/*
Divide aspect by 2 to fit it in byte.
Store aspect in integer variable.
--------------------------------------
*/
         aspect = aspect / 2.0;
         *bufer = (int)(aspect + 0.5);
         }
      else
/*
Compute percent slope by multilplying by 100.
Adjust the value to fit in byte length.
Convert the value to integer.
------------------------------------------------
*/
         {
         temp = (slpv * slpv) + (slph * slph);
         temp = sqrt(temp);
         slopct = temp * 100.0;      
         if (slopct >= 254.0) 
             slopct = 254.0;
         *bufer = (int)(slopct + 0.5);
         }  
      }
   else
/*
If all values in tmatrix are mask values, set output value to 255.
------------------------------------------------------------------
*/
      *bufer = 255;
   }

else
/*
If the pixel under study (matrix(1,1)) = mask value(aoff),
set output value to 255.
------------------------------------------------------------
*/
   *bufer=255;
}

