#include "topo.h"
#include "pixman.h"
#include "protos.h"

void process
(
    long np, 				/* number of samples                */
    long nl,				/* number of lines                  */
    long method, 			/* calculation method               */
    float alpha, 				/* sun's elevation angle    */
    float gamma,				/* azimuth angle in degrees */
    long bnd, 				/* band number                      */
    long obnd,				/* output band number               */
    float coldis,				/* distance between columns */
    float rowdis,				/* distance between rows    */
    float aoff, 				/* mask value               */
    long odtype, 			/* image data type                  */
    long img 				/* image number                     */
)
{
/*
-------------------------------------------------------------------------------
NAME: PROCESS

FUNCTION:   Places input image data into a 3 x 3 matrix and calls the 
            appropriate subroutine for computing the beta angles using 
            either the mean method, the maximum drop method, the maximum 
            difference method, or the minimum difference method.  

PROGRAM HISTORY:

     Version	date      	author      	request
     -------   	----      	------      	-------
        1     	12/20/83    	D. Steinwand     
 	2     	12/23/85  	J. Lavergne     corrected window problem
	3	7/10/88		B. Allam	conversion
        4       08/22/90        K. Kunska       fixed dynamic memory allocation
                                                by using a (3 x np) work buffer
                                                instead of (nl x np) work buffer
	5	11/13/90	B. Ailts	updated parameters and error
						messages
ALGORITHM:

   Allocate dynamic memory for work buffers.
   Initialize the variables as needed : 
      Set (psi) to (90 degrees - alpha) * degrees-to-radians
      Set (gam) to gamma * degrees-to-radians
      Set (sun_x) to sin(psi)*sin(gam)
      Set (sun_y) to sin(psi)*cos(gam)
      Set (sun_z) to cos(psi)
   For each image line
      Set (first) to TRUE to flag first image line
      Set (last) to FALSE to flag last image line
      For each pixel in a line
         If processing the last sample, 
            Set (last) to TRUE
         Set counters (ia),(ib),(ic),(id). (These counters help in padding 
                                             the outside edges of the image.)
         If on the first image line, then 
            Pad the 1st line with aoff
            Set (ia) to 2 (This prevents line 1 of the matrix from being filled 
                           with data)
            For the first matrix line
               Set (matrix[line1][x]) to aoff, the mask value
            End-do
         Else-If on the last line of the image then
            Set (ib) to 2
            For the last matrix line
               Set (matrix[line3][x]) to aoff, the mask value
            End-do
         End-if
         For each row of the 3 x 3 matrix
            For each column of the 3 x 3 matrix
               Set (matrix[ix][ii]) to next sample
            End-do
            If on the 1st sample of a line, 
               Pad 1st column with aoff
            If on last sample of line, 
               Pad 3rd column with aoff
         End-do
         If using mean method then
            Call MEAN to compute the mean of the horizontal and veritcal
                      slopes
         Else for all other methods...
            Call MAXMIN to compute maximum difference, minimum difference
                        and maximum drop
         Fit the output value to the output data type.
         Write out the output line.
         Reset input buffer pointers.
         Read the next line into the work buffer.
         Set (first) to FALSE
      End-do
   End-do
   Free work and output buffers

-----------------------------------------------------------------------------
*/

double	gam,				/* azimuth angle in radians     */
	psi;				/* zenith angle (90 - alpha) 	*/

float 	beta,				/* beta angle                   */
	fillval,			/* fill value for workbuff      */
	matrix[3][3],			/* matrix with center pixel     */
	*obuf,				/* output buffer                */
	sun_x, 				/* sun's ray vector: x-axis     */
	sun_y, 				/* sun's ray vector: y-axis     */
	sun_z;				/* sun's ray vector: z-axis     */

long	first, 				/* first pixel flag             */
        icur[3], 			/* pointers to lines            */
	ia, ib, ic, id, 		/* upper and lower limits for   */
					/* looping thru matrix          */
	i, ii, ix, k, 			/* loop counters                */
	index,				/* circular index into workbuf  */
	iinline,			/* current line                 */
	ipos, jpos,			/* position in line             */
	last,				/* last pixel flag              */
	lnum, 				/* line number counter          */
	pnum,				/* pixel number counter         */
	type,				/* data type of work buffer     */
	work_siz;			/* size of work buffer          */

static	float  *workbuf[3], 		/* 2-dimensional work buffer    */
	       *wbuf;			/* pointer to work area         */

static long one = 1;

/*
Allocate dynamic memory for work buffers.
----------------------------------------
*/
work_siz =(long) ( 3 * np * sizeof(float));
if ((wbuf = (float *) malloc(work_siz)) == NULL)
   msg_err ("!Error: cannot allocate dynamic memory", "topo-alloc");

if ((obuf = (float *) malloc(np * sizeof(float))) == NULL)
   msg_err ("!Error: cannot allocate dynamic memory", "topo-alloc");

for (i = 0, k = 0; i < 3; i++, k += np)
   workbuf[i] = &wbuf[k];

/*
Initialize the variables as needed.
-----------------------------------
*/
psi = 1.5708 - 0.017453 * alpha;
gam = gamma * 0.017453;
sun_x = sin(psi) * sin(gam);
sun_y = sin(psi) * cos(gam);
sun_z = cos(psi);
for (i = 0; i < 3; i++)
   for (k = 0; k < 3; k++)
      matrix[i][k] = 0.0;

/*
Copy input image into temporary buffer
--------------------------------------
*/
type = EREAL;
fillval = 0.0;
c_pxfill (workbuf[0], &type, &np, &fillval);
for (iinline = 1; iinline <= 2; iinline++) 
   {
   c_eread (&fdesc[img], &bnd, &iinline, workbuf[iinline], &one);
   curent++;
   }

index = -1;
for (i = 0; i < 3; i++)
   icur[i] = i;

/*
For all lines:
Set the first and last line flags.
-----------------------------------
*/
for (lnum = 1; lnum <= nl; lnum++)
   {
   first = TRUE;
   last = FALSE;

/*
For all samples:
Set upper and lower loop limits.
If on the first image line, pad the 1st row in matrix with mask value.
If on the last image line, pad the 3rd row in matrix with mask value.
----------------------------------------------------------------------
*/
   for (pnum = 1; pnum <= np; pnum++)
      {
      if (pnum == np) last = TRUE;
      ia = 1;
      ib = 3;
      ic = 1;
      if (first) ic = 2;
      id = 3;
      if (last) id = 2;
      if (lnum == 1) 
         {
         ia = 2;
         for (ii = 0; ii < 3; ii++)
            matrix[0][ii] = aoff;
         }
      else if (lnum == nl) 
         {
         ib = 2;
         for (ii = 0; ii < 3; ii++)
            matrix[2][ii] = aoff;
         }  

/*
Fill the 3x3 matrix with pixels from the work buffer.
If on the 1st sample of a line, pad 1st column with the mask value.
If on the last sample of line, pad 3rd column with the mask value.
-------------------------------------------------------------------
*/
      for (ix = ia; ix <= ib; ix++)
         {
         jpos = (pnum - 1);
         for (ii = ic; ii <= id; ii++)
            {
            ipos= jpos + ii - 2;
            matrix[ix-1][ii-1] = workbuf[icur[ix-1]][ipos];
            }
         if (first) matrix[ix-1][0] = aoff;
         if (last) matrix[ix-1][2] = aoff;
         }

/*
If using mean method, call MEAN to compute the mean of the horizontal 
and veritcal slopes.
Otherwise, call MAXMIN to compute maximum difference, minimum difference
and maximum drop.
-------------------------------------------------------------------------
*/
      if (method == 1) 
         bmean (matrix, &beta, aoff, coldis, rowdis, sun_x, sun_y, sun_z);
      else
         bmaxmin (matrix, &beta, coldis, rowdis, aoff, method, sun_x, sun_y, 
                  sun_z);

/*
Fit the output value to the output data type.
----------------------------------------------
*/
      if ( odtype  ==  4 ) 
         obuf[pnum-1] = beta;
      else
         {
         beta += 0.5;
         if (beta < 0)
            obuf[pnum - 1] = 0;
         else
            obuf[pnum-1] = (float)(int) beta;
         }
      first = FALSE;
      }

/*
Write out the output line.
--------------------------
*/
   c_ewrite (&fdescout, &obnd, &lnum, obuf, &one);

/*
Reset input buffer pointers.
Read the next line into the work buffer.
------------------------------------------
*/
   for (i = 0; i < 3; i++)
      icur[i] = (icur[i] + 1) % 3;
   if (lnum <= nl-2)
      {
      index = (index + 1) % 3;
      c_eread (&fdesc[img], &bnd, &iinline, workbuf[index], &one);
      iinline++;
      curent++;
      }
   }

/*
Free work and output buffers
-----------------------------
*/
free (wbuf);
free (obuf);
}
