#include "topo.h"
#include "pixman.h"
#include "protos.h"

void sacomp
(
    long method,				/* method no. (1-4)            */
    long opt,				/* option code(1-3)            */
    long np,				/* no. of pixels per line      */
    long nl,				/* no. of lines in input image */
    long nbc,				/* neighborhood size code(1-2) */
    float coldis,				/* column distance             */
    float rowdis,    			/* row distance                */
    float aoff,				/* mask value                  */
    long img,       			/* current image index         */
    long bnd,       			/* input band number           */
    long obnd,				/* output band number          */
    long itype				/* input image type 	       */
)

/*
---------------------------------------------------------------------------

NAME:	SACOMP

FUNCTION:  Places input image data into a 3 x 3 matrix and calls the 
           appropriate subroutine for computing slope or aspect using
           either the mean method, the maximum drop method, the maximum 
           difference method, or the minimum difference method.

PROGRAM HISTORY:

  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    3.1       04/  /89     D. Hollaren      Added static to compile on PN9050
    4.0       07/20/89     B. Allam         Fixed bugs
    5.0       08/22/90     K. Kunska        Fixed dynamic memory allocation
                                            by using a (3 x np) work buffer
                                            instead of (nl x np) work buffer
ALGORITHM:

   Allocate dynamic memory for work buffers.
   If option is slope and method is other than mean method then
      Generate distance table
   End-if
   If option is aspect and method is other than mean method then
      Generate angle table
   End-if
   Copy input image lines into work buffer.
   For each image line
      Set row range parameter for matrix
      Fill matrix with the mask value for lines outside the image 
      For each pixel in a line
         Set column range for 3 x 3 matrix
         Fill matrix with the mask value for pixels outside the image 
         Build 3x3 matrix from 3 image lines if possible
         If using mean method then
            Call MEAN to compute slope or aspect by the mean method
         Else for all other methods...
            Call MAXMIN to compute slope or aspect by maximum drop, maximum 
                         or mimimum difference method
         End-if
         Write out the output line.
      End-do
      Reset line pointers for the work buffer 
   End-do
   Free dynamic memory
---------------------------------------------------------------------------
*/
{
extern struct FDESC * fdescout;

static long one = 1;		/* value of 1                          */

double	temp;			/* intermediate result                 */

float	ang,			/* arc-tangent of (coldis - rowdis)     */
	angle[9],        	/* angle matrix                         */
	matrix[3][3],    	/* processing matrix                    */
	dist[8],        	/* distance matrix                      */
	*outbuf, 		/* image output buffer                  */
	*wbuf,			/* pointer to work area                 */
	*workbuf[3];		/* 2-dimensional work buffer            */

long	curline[3], 		/* pointers to lines                    */
	dtype, 			/* data type                            */
	fillval,		/* fill value for array initialization  */
	ia, ib, ic, id,   	/* upper and lower limits for           */
				/* looping thru matrix                  */
	i, ii, ix, j, k, 	/* loop counters and array indices      */
	index,			/* circular index into work buffer      */
	iinline,		/* current input line                   */
	ipos, jpos,		/* positions in line                    */
	lnum, 			/* line number counter                  */
	pnum, 			/* pixel number counter                 */
	work_siz;		/* size of work buffer                  */

short	ival;        		/* slope or aspect result               */

/*
Allocate dynamic memory for work buffers.
----------------------------------------
*/
work_siz = (long) (3 * np * sizeof(float));
if ((wbuf = (float *) malloc(work_siz)) == NULL)
   msg_err ("!Error: cannot allocate dynamic memory", "topo-alloc");

if((outbuf = (float *) malloc(np * sizeof(float))) == NULL)
   msg_err("!Error: cannot allcate dynamic memory", "topo-alloc");

for (i = 0, k = 0; i < 3; i++, k += np)
   workbuf[i] = &wbuf[k];

/*
If option is slope and method is other than mean method then generate 
distance table.
-----------------------------------------------------------------------
*/
if ((opt == 1)  &&  (method >= 2)) 
   {
   dist[0]= rowdis;                
   dist[1]= coldis;                
   dist[2]= rowdis;                
   dist[3]= coldis;                
   temp = (coldis * coldis + rowdis * rowdis);
   dist[4]= sqrt(temp);  
   dist[5]= dist[4];
   dist[6]= dist[4];
   dist[7]= dist[4];
   }  

/* 
If option is aspect and method is other than mean method then generate 
angle table
-----------------------------------------------------------------------
*/
if ((opt == 2)  &&  (method >= 2))
   {
   temp = (coldis / rowdis);
   ang = atan (temp) * 57.2958;         
   angle[0] =  0.0;                
   angle[1] = 90.0;                
   angle[2] = 180.0;                
   angle[3] = 270.0;                
   angle[4] = ang;
   angle[5] = 180.0 - ang;
   angle[6] = 180.0 + ang;
   angle[7] = 360.0 - ang;
   angle[8] = 400.0;                           
   for (k = 0; k < 9; k++)
      angle[k] = angle[k] / 2.0;      
   }  

/*
Copy input image lines into work buffer.
-----------------------------------------
*/
dtype = EREAL;
fillval = 0.0;
c_pxfill (workbuf[0], &dtype, &np, &fillval);
for (iinline = 1; iinline <= 2; iinline++) 
   {
   c_eread (&fdesc[img], &bnd, &iinline, workbuf[iinline], &one);
   curent++;
   }
index = -1;
for (i = 0; i < 3; i++)
   curline[i] = i;

/*
Process lines.
--------------
*/
for (lnum = 1; lnum <= nl; lnum++) 
   {
/*
Initialize matrix.
------------------
*/
   for (i = 0; i < 3; i++)
      for (j = 0; j < 3; j++)
         matrix[i][j] = 0.0;

/*
Set row number range ia & ib 
Fill matrix with the mask value for lines outside the image 
--------------------------------------------------------------
*/
   ia = 0;
   ib = 2;
   if (lnum == 1) 
      {
      ia = 1;
      for (i = 0; i < 3; i++)
         matrix[0][i] = aoff;		
      }  
   if (lnum == nl) 
      {
      ib = 1;
      for (j = 0; j < 3; j++)
         matrix[2][j] = aoff;		
      }  

/*
Process each pixel
Set column number range ic & id
Fill matrix with the mask value for pixels outside the image 
----------------------------------------------------------------
*/
   for (pnum = 1; pnum <= np; pnum++)
      {
      ic = 0;
      id = 2;
      if (pnum == 1) 
         {
         ic = 1;
         for (j = 0; j < 3; j++)
            matrix[j][0] = aoff;			
         }  
      if (pnum == np) 
         {
         id = 1;
         for (j = 0; j < 3; j++)
            matrix[j][2] = aoff;			
         }  

/*
Build 3x3 matrix from 3 image lines if possible
------------------------------------------------
*/
      for (ix = ia; ix <= ib; ix++)
         {
         jpos = (pnum - 1);
         for (ii = ic; ii <= id; ii++)
            {
            ipos = jpos + ii - 1;
            matrix[ix][ii] = workbuf[curline[ix]][ipos];
            }
         }

/*
If using mean method, call MEAN to compute slope or aspect by the 
mean method.
Otherwise, call MAXMIN to compute slope or aspect by maximum drop, 
maximum or mimimum difference method.
-----------------------------------------------------------------
*/
      if (method == 1) 		
         mean (matrix, &ival, opt, nbc, coldis, rowdis, aoff);
      else                      
         maxmin (matrix, &ival, method, opt, nbc, dist, angle, aoff, itype);

      outbuf[pnum-1] = (float)ival;
      }

/*
Write out the output line.
--------------------------
*/
   c_ewrite (&fdescout, &obnd, &lnum, outbuf, &one);

/*
Reset line pointers for the work buffer
------------------------------------------
*/
   for (i = 0; i < 3; i++)
      curline[i] = (curline[i] + 1) % 3;
   if (lnum <= nl-2)
      {
      index = (index + 1) % 3;
      c_eread (&fdesc[img], &bnd, &iinline, workbuf[index], &one);
      curent++;
      iinline++;
      }
   }

/*
Free dynamic memory
--------------------
*/
free (wbuf);
free (outbuf);
}
