/****************************************************************************
NAME:				get_tp

PURPOSE:  Gets a tie point coordinate from the user

PROGRAM	HISTORY:
VERSION	 DATE	AUTHOR	CODE/CONT	REASON
-------	 ----	------	---------	---------------------------------
 5.0	  --	  --	   --		Original development
 5.1	 8/92	S. Nelsn   CSB		Added assignments to correct error
					in compiler.

ALGORITHM DESCRIPTION:
	Increment the point (prompt) counter
	Do unit statisfactory input has been given:
	   Print prompt
	   Get the input back from the user
	   If done, return EOF
	   For the appropriate units:
		Unpack the user input into Point_id, Y coord, X coord
		Check the valid range of Y coord & X coord
		If valid, break out of loop
		Prompt user to re-enter invalid input
	Return SUCCESS
*****************************************************************************/
#include <string.h>
#include "trancoord.h"

int get_tp
(
  struct PRO *pro,            /* Input and output projection parameters */
  struct TPSDATA *data,       /* Tie Point Selection data record(user input) */
  char *infile                /* Input data file for tie point coordinates */
)
{
  char mess[80];		/* Buffer for user messages and input */
  long term;		/* Terminal flag for t_io */
  long x_ok, y_ok, id_ok;	/* Boolean flags */
  long deg1, min1;	/* Degrees & minutes part of first DMS coordinate */
  long deg2, min2;	/* Degrees & minutes part of second DMS coordinate */
  float tempdeg1, tempdeg2;
  float tempmin1, tempmin2;
  float sec1, sec2;	/* Seconds parts of DMS coordatates */
  static long pt_num = 0;	/* Prompt number (counter) */
  static long first = 1;
  static FILE *fp;

  /* Increment coordinate counter
    ----------------------------*/
  pt_num++;
  if (strlen(infile))
  {
    if (first)
    {
      fp = fopen(infile,"r");
      if (fp == NULL)
      {
        if(strchr(infile,';') || strchr(infile,'[') || strchr(infile,']'))
        {
         c_lasmsg(TRUE,"trancoord-open",                                
                  "Error opening input file %s. INFILE should"
                  " use host naming convention",infile);
        }
        else
        {
          c_lasmsg(TRUE, "trancoord-open",
                   "Error opening input file %s", infile);
        }
      }
      first=0;
    }
  }
  /* Do until satisfactory user input is given
     -----------------------------------------*/
  for (;;)
  {
    if (strlen(infile) > 0)
    {
      if (fgets(mess,80,fp) == NULL)
      {
        fclose(fp);
        return(E_EOF);
      }
    }
    else
    {

      /* Prompt user
         -----------*/
#ifdef vms
      t_write(" ",T_STDCC);
#endif
      sprintf(mess," %d.  ",pt_num);
      t_write(mess,T_NULL);

      /* Read user input
         ---------------*/
      t_read(mess, &term);
   
      /* Check for end of entry
        ----------------------*/
      if (strlen(mess) == 0) return(E_EOF);
    }

    /* For the given units, unpack the fields and check validity.
       Break out of loop if satisfactory input is given
       -------------------------------------------------*/
    x_ok = y_ok = id_ok = FALSE;
    if (pro->in_units == 0)  /* Radians */
    {
      sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                                     &(data->coord[1])); 
      if ((data->coord[0] >= NEG_HALF_PI) && (data->coord[0] <= HALF_PI))
        y_ok = TRUE;
      if ((data->coord[1] >= NEG_PI) && (data->coord[1] <= PI))
        x_ok = TRUE;
      if (x_ok && y_ok)
      {
        if (c_ckptid(data->pt_id)) 
          id_ok = TRUE;
      }
      else 
        id_ok = TRUE;
    }
    if (pro->in_units == 1)  /* Feet */
    {
      sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                                     &(data->coord[1])); 
      if (c_ckptid(data->pt_id)) 
        id_ok = TRUE;
      x_ok = y_ok = TRUE;
    }
    if (pro->in_units == 2)  /* Meters */
    {
      sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                                     &(data->coord[1])); 
      if (c_ckptid(data->pt_id)) 
        id_ok = TRUE;
      x_ok = y_ok = TRUE;
    }
    if (pro->in_units == 3)  /* Seconds */
    {
      sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                                     &(data->coord[1])); 
      if ((data->coord[0] >= -324000.0) && (data->coord[0] <= 324000.0))
        y_ok = TRUE;
      if ((data->coord[1] >= -648000.0) && (data->coord[1] <= 648000.0))
        x_ok = TRUE;
      if (x_ok && y_ok)
      {
        if (c_ckptid(data->pt_id)) 
          id_ok = TRUE;
      }
      else
        id_ok = TRUE;
    }
    if (pro->in_units == 4)  /* Degrees */
    {
      sscanf(mess, "%s %lf %lf", data->pt_id, &(data->coord[0]),
                                   &(data->coord[1])); 
      if ((data->coord[0] >= -90.0) && (data->coord[0] <= 90.0))
        y_ok = TRUE;
      if((data->coord[1] >= -180.0) && (data->coord[1] <= 180.0))
        x_ok = TRUE;
      if (x_ok && y_ok)
      {
        if (c_ckptid(data->pt_id)) 
          id_ok = TRUE;
      }
      else 
        id_ok = TRUE;
    }
    if (pro->in_units == 5)  /* Packed DMS */
    {
      int deg1_neg, deg2_neg;
      int fields_parsed;
      
      x_ok = y_ok = TRUE;
      deg1_neg = deg2_neg = FALSE;
      
      fields_parsed = sscanf(mess, "%s %d %d %f %d %d %f", data->pt_id, &deg1, 
                             &min1, &sec1, &deg2, &min2, &sec2);
      
      /* cover the cases where the lat/long may be negative, but deg1 or deg2
         may have been entered as "-0" */
      if ((deg1 == 0) || (deg2 == 0))
      {
         char *token;
         
         /* tokenize the input buffer, looking for negative signs before the
            degree inputs.  Don't bother looking for NULL since the sscanf
            would have detected an incorrect number of parameters */
         if (fields_parsed >= 2)
         {
             /* get rid of point id */
             token = strtok(mess," ");
             /* get deg1 */
             token = strtok(NULL," ");
             if (deg1 == 0)
             {
                if (token[0] == '-')
                    deg1_neg = TRUE;
             }
             if ((deg2 == 0) && (fields_parsed >= 5))
             {
                 /* waste the next two parameters */
                 token = strtok(NULL," ");
                 token = strtok(NULL," ");

                 /* get deg2 */
                 token = strtok(NULL," ");
                 if (token[0] == '-')
                    deg2_neg = TRUE;
             }
         }
      }
      
      if ((deg1 > 90) || (deg1 < -90)) 
        y_ok = FALSE;
      if ((deg2 > 180) || (deg2 < -180)) 
        x_ok = FALSE;
      if ((min1 > 60) || (min2 > 60)) 
        y_ok = FALSE;
      if ((sec1 > 60.0) || (sec2 > 60.0)) 
        x_ok = FALSE;

      /* The following assignments were added due to compiler error that
         would not multiply and add integers with reals accurately */

      tempdeg1=(float)deg1;
      tempdeg2=(float)deg2;
      tempmin1=(float)min1;
      tempmin2=(float)min2;
      if ((deg1 < 0.0) || deg1_neg)
        data->coord[0] = ((tempdeg1 * -1000000.0) + (tempmin1 *
					 1000.0)+sec1) * -1.0;
      else
        data->coord[0] = (tempdeg1 * 1000000.0) + (tempmin1 * 1000.0) + sec1;

      if ((deg2 < 0.0) || deg2_neg)
        data->coord[1] = ((tempdeg2*-1000000.0)+(tempmin2*1000.0)+sec2)*-1.0;
      else
        data->coord[1] = (tempdeg2 * 1000000.0) + (tempmin2 * 1000.0) + sec2;

      if (data->coord[0] > 90000000.00) 
        y_ok = FALSE;
      if (data->coord[0] < -90000000.00) 
        y_ok = FALSE;
      if (data->coord[1] > 180000000.00) 
        x_ok = FALSE;
      if (data->coord[1] < -180000000.00) 
        x_ok = FALSE;
      if (x_ok && y_ok)
      {
        if (c_ckptid(data->pt_id)) 
          id_ok = TRUE;
      }
      else id_ok = TRUE;
    }
    if (x_ok && y_ok && id_ok) 
      break;
    if (!x_ok) 
      t_write(" Illegal X/Longitude value entered; re-enter!", T_STDCC);
    if (!y_ok) 
      t_write(" Illegal Y/Latitude value entered; re-enter!", T_STDCC);
    if (!id_ok) 
      t_write(" Point id is not unique; re-enter!", T_STDCC);
  }
  data->elevation = 0.0;
  data->zoom_factor = 0.0;
  data->chip_size[0] = data->chip_size[1] = 0;
  data->offsets[0] = data->offsets[1] = 0;
  strcpy(data->chip_name,"No chip");

  return(E_SUCC);
}
