/****************************************************************************
NAME:                           getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.0           3/89    D. Steinwand       CSB      New development
  5.1           6/90    D. Etrheim         CSB      Trap status of c_setprt
  7.0           7/98    J. Janssen         CSB      Corrected process of how
                                                    we retrieve projection
                                                    information.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
        Check the runtype.  Manual input cannot be given in batch mode.
        Get the subcommand used
        Get the name of the input tie point selection file
        Get the name of the output tie point selection file
        Get the parameters used with the POLY subcommand
           Get COOROPT (tells whether to xform USR or image coordinates)
           Get the input grid file name
           If the grid file name is null, read coefficients from TRNSCOEF
           else, read from the grid file
           Get the OFFSETS
        Get parameters common to PROJ & GRID subcommands
           Get the name of the projection parameter file
           Get the input & output projection parameter keys
           Read the projection parameters from the file
        Get parameters for the GRID subcommand
           Get the pixel size in y & x
           Get the upper left coordinate
           Get the line, sample coordinate of the upper left coordinate
           Get the number of lines in the output image (optional)
           Get the number of samples in the output image (optional)
           Get the units of the corner coordinates
           Get the lower left coordinate
           Determine framing mode
        Get the print parameter option
        Return
*****************************************************************************/
#include <string.h>
#include "trancoord.h"
#include "ltable.h"
#include "util.h"


int get_proj_prm
(
    char *proj_name,        /* Projection parameter file name */
    char *inkey,            /* Input projection parameter key */
    char *outkey,           /* Output projection parameter key */
    long ddr_info,          /* Boolean flag--output proj info from ddr used? */
    long tps_info,          /* Boolean flag--input proj info from tps file? */
    struct PRO *pro         /* Input & output projection parameters */
);

int use_ddr_info
(
    struct PARBLK *vblk,        /* TAE parameter block */
    struct PRO *pro,            /* Structure containing projection parameters */
    struct FRAME *frame         /* Structure containing framing parameters */
);


int getpar
(
    struct PARBLK *vblk,        /* TAE variable block */
    long *option,               /* Contains flags for subcommand chosen */
    char *tps_name,             /* Input tie point selection file name */
    char *out_tps,              /* Output tie point selection file name */
    struct PRO *pro,            /* Parameters for the PROJ & GRID subcommands */
    struct FRAME *frame,        /* Output framing params for GRID subcommand */
    struct FIT *poly,           /* Poly coeffs & misc data for POLY subcmd */
    struct PRINT *print,        /* Print/report parameters */
    long *msgtim,               /* Message timer */
    float *projtol,             /* Projection coordinate tolerance */
    char *infile                /* Input file containing tie point coordinates*/
)
{
    struct VARIABLE *var;           /* pointer to a TAE variable in vblk */
    struct GEOGRID grid;            /* Geometric mapping grid */
    long acc;                       /* access flag for c_getnam */
    long i;                         /* Loop counter */
    long zero = 0;                  /* Constant of zero */
    long pass = 1;                  /* Pass flag for non-fatal errmsg */
    long term, ln_pr, file;         /* Print option flags */
    long ddr_info;                  /* Boolean flag--proj info from ddr used? */
    long tps_info = FALSE;          /* Boolean flag--proj info from tps file? */
    char filename[CMLEN];           /* Temporary filename */
    char grid_name[CMLEN];          /* Geometric mapping grid file name */
    char proj_name[CMLEN];          /* Projection parameter file name */
    char inkey[10];                 /* Input projection key */
    char outkey[10];                /* Output projection key */


    /* Check the runtype.  If runtype is batch, input cannot be given manually
       -----------------------------------------------------------------------*/
    /*
       var = p_find(vblk, "$RUNTYPE");
       if (strcmp(SVAL(*var, 0), "INTERACTIVE") != 0) 
     */

    /* Get the subcommand used
       -----------------------*/
    var = p_find(vblk, "_SUBCMD");
    if (strcmp(SVAL(*var, 0), "PROJ") == 0) *option = PROJ_ONLY;
    if (strcmp(SVAL(*var, 0), "GRID") == 0) *option = PROJ_GRID;
    if (strcmp(SVAL(*var, 0), "POLY") == 0) *option = POLYNOMIAL;

    /* Get the input tie point selection file name
       ------------------------------------------*/
    acc = IREAD;
    if (c_getnam(vblk, "INTS", &zero, tps_name, "tps", &acc) != E_SUCC)
        return(E_FAIL);

    /* Get the output tie point selection file name
       -------------------------------------------*/
    acc = IWRITE;
    if(c_getnam(vblk, "OUTTS", &zero, out_tps, "tps", &acc) != E_SUCC)
        return(E_FAIL);

    /* Get parameters used with the POLY subcommand
       --------------------------------------------*/
    if (*option == POLYNOMIAL)
    {
        var = p_find(vblk, "COOROPT");
        if (strcmp(SVAL(*var, 0), "XY") == 0) poly->field_mode = COORS;
        if (strcmp(SVAL(*var, 0), "IMAGE") == 0) poly->field_mode = IMAGE_COORS;

        acc = IREAD;
        if(c_getnam(vblk, "INGRID", &zero, grid_name, "grid", &acc) != E_SUCC)
            return(E_FAIL);
        if(strcmp(grid_name, "") == 0)
        {
            for (i = 0; i < 15; i++)
                poly->a[i] = poly->b[i] = 0.0;
            var = p_find(vblk, "LINECOEF");
            for (i = 0; i < (*var).v_count; i++)
                poly->b[i] = RVAL(*var,i);
            var = p_find(vblk, "SAMPCOEF");
            for (i = 0; i < (*var).v_count; i++)
                poly->a[i] = RVAL(*var,i);
            poly->ncoef = 3;
            if ((poly->a[3] != 0.0) || (poly->a[4] != 0.0) 
                    || (poly->a[5] != 0.0) || (poly->b[3] != 0.0) 
                    || (poly->b[4] != 0.0) || (poly->b[5] != 0.0))
                poly->ncoef = 6;
            if ((poly->a[6] != 0.0) || (poly->a[7] != 0.0) 
                    || (poly->a[8] != 0.0) || (poly->a[9] != 0.0) 
                    || (poly->b[6] != 0.0) || (poly->b[7] != 0.0) 
                    || (poly->b[8] != 0.0) || (poly->b[9] != 0.0)) 
                poly->ncoef = 10;
            if ((poly->a[10] != 0.0) || (poly->a[11] != 0.0) 
                    || (poly->a[12] != 0.0) || (poly->a[13] != 0.0) 
                    || (poly->a[14] != 0.0) || (poly->b[10]!=0.0)
                    || (poly->b[11] != 0.0) || (poly->b[12] != 0.0) 
                    || (poly->b[13]!=0.0) || (poly->b[14] != 0.0)) 
                poly->ncoef = 15;
        }
        else
        {
            for(i = 0; i < 15; i++)
                poly->a[i] = poly->b[i] = 0.0;
            if(c_getgrd(grid_name, &grid) != E_SUCC) return(E_FAIL);
            poly->ncoef = grid.ncoeff;
            for (i = 0; i < poly->ncoef; i++)
            {
                poly->b[i] = grid.coeffs[i];
                poly->a[i] = grid.coeffs[i + grid.ncoeff];
            }
        }
        var = p_find(vblk, "OFFSETS");
        poly->offset[0] = RVAL(*var, 0);
        poly->offset[1] = RVAL(*var, 1);
        poly->offset[2] = RVAL(*var, 2);
        poly->offset[3] = RVAL(*var, 3);
    }

    /* Get parameters common to the PROJ & GRID subcommands 
       ----------------------------------------------------*/
    else  
    {
        ddr_info = FALSE;
        pro->in_proj = pro->out_proj = NULL_PROJ;
        var = p_find(vblk, "IN");
        if ((*var).v_count != 0) ddr_info = TRUE;

        var = p_find(vblk, "INPROJ");
        if ((*var).v_count != 0)
        {
            acc = IREAD;
            if (c_getnam(vblk,"INPROJ",&zero,proj_name,"proj",&acc) != E_SUCC)
                return(E_FAIL);
            var = p_find(vblk, "IPROJKEY");
            if ((*var).v_count != 0) 
                strcpy(inkey, SVAL(*var, 0));
            else
                /* if INPROJKEY was not specified and INTS was not specified, 
                   error 
                   ----------------------------------------------------------*/ 
                if (strcmp(tps_name, "") == 0)
                {
                    c_errmsg("Source of projection information not fully "
                             "specified", "trancoord-proj", &pass);
                    return(E_FAIL);
                }
            /* else input proj info will come from the input tps file
               --------------------------------------------------------*/
                else
                    tps_info = TRUE;
            if (!ddr_info)
            {
                var = p_find(vblk, "OPROJKEY");
                if ((*var).v_count != 0) 
                    strcpy(outkey, SVAL(*var, 0));
                else
                {
                    c_errmsg("Source of projection information not fully "
                             "specified", "trancoord-proj", &pass);
                    return(E_FAIL);
                }         
            }
            if ((!ddr_info) || (!tps_info))
                if (get_proj_prm(proj_name,inkey,outkey,ddr_info,
                       tps_info,pro) != E_SUCC)
                    return(E_FAIL);
        }
        else
            if ((!ddr_info) || (strcmp(tps_name, "") == 0))
            {
                c_errmsg("Source of projection information not fully specified",
                        "trancoord-proj", &pass);
                return(E_FAIL);
            }

        if (ddr_info)
        {
            if(use_ddr_info(vblk, pro, frame) != E_SUCC) 
                return(E_FAIL);
        }
        else
            frame->ddr_info = FALSE;
    }
    if (*option == PROJ_GRID)
    {
        var = p_find(vblk, "INFILE");
        if (var->v_count == 0)
            *infile = '\0';
        else
            strcpy(infile, SVAL(*var,0));

        /* Get the projection coordinate tolerance
           ------------------------------------------*/
        var = p_find(vblk, "$PROJTOL");
        *projtol = RVAL(*var, 0);
    }
    /* Get parameters needed for the GRID subcommand
       ---------------------------------------------*/
    if ((*option == PROJ_GRID) && (!ddr_info))
    {
        var = p_find(vblk, "PIXSIZ");
        if ((!ddr_info) && ((*var).v_count == 0))
        {
            c_errmsg("Pixel size must be specified","trancoord-pixsiz",&pass);
            return(E_FAIL);
        }
        frame->p_dist[0] = RVAL(*var, 0);
        frame->p_dist[1] = RVAL(*var, 1);
        var = p_find(vblk, "LSCOORS");
        frame->lscoors[0] = IVAL(*var,0);
        frame->lscoors[1] = IVAL(*var,1);
        var = p_find(vblk, "NL");
        frame->nl = IVAL(*var,0);
        var = p_find(vblk, "NS");
        frame->ns = IVAL(*var,0);
        var = p_find(vblk, "COORUNIT");
        strcpy(frame->units, SVAL(*var,0));
        if(strcmp(frame->units,"DMS") == 0) frame->unit_code = 5;
        if(strcmp(frame->units,"DEG") == 0) frame->unit_code = 4;
        if(strcmp(frame->units,"RAD") == 0) frame->unit_code = 0;
        if(strcmp(frame->units,"SEC") == 0) frame->unit_code = 3;
        var = p_find(vblk, "COORS");
        if ((!ddr_info) && ((*var).v_count == 0))
        {
            c_errmsg("Framing coordinates must be specified","trancoord-frame",
                    &pass);
            return(E_FAIL);
        }
        sscanf(SVAL(*var,0), "%lf", &(frame->ul_corner[0]));
        sscanf(SVAL(*var,1), "%lf", &(frame->ul_corner[1]));
        if ((*var).v_count == 2) frame->mode = OUTPUT_LS;
        else if (strcmp(frame->units, "PRO") == 0) frame->mode = OUTPUT_XY;
        else frame->mode = INPUT;
        if ((frame->mode == INPUT) || (frame->mode == OUTPUT_XY))
        {
            sscanf(SVAL(*var,2), "%lf", &(frame->lr_corner[0]));
            sscanf(SVAL(*var,3), "%lf", &(frame->lr_corner[1]));
        }
    }
    else
    {
        frame->mode = OUTPUT_XY;
        frame->lscoors[0] = 1;
        frame->lscoors[1] = 1;
    }

    /* Get the print option.  Make a file name (if neccessary) and open the
       temporary file
       --------------*/
    strcpy(print->host_name, "trancoord");
    if (c_setprt(vblk, &term, &file, &ln_pr, print->host_name, 
                 filename) != E_SUCC)
        return(E_FAIL);
    acc = IUPDATE;
    if (file == 1) c_mknam(filename, print->name, "prt", &acc);
    else strcpy(print->name, " ");
    print->fptr = fopen(print->host_name, "w");

    /* Get the message timer
       ---------------------*/
    var = p_find(vblk, "$msgtim");
    *msgtim = IVAL(*var, 0);

    return(E_SUCC);
}


/****************************************************************************
NAME:                           get_proj_prm

PURPOSE:  Retrieves the projection parameters from the projection parameter file

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  5.0    2/89   D. Steinwand  CSB      LAS 5.0 Original development

ALGORITHM DESCRIPTION:
        Open the label table file containing the projection parameters
        Get the label
        Determine the position of the fields within the file
        Scan the file for the projection parameters specified by the input
           and output keys  (If the keys are null, the first record in the
           file is assigned to the output projection; the second is the
           input projection)
        Close the table
        Return
*****************************************************************************/
int get_proj_prm
(
    char *proj_name,        /* Projection parameter file name */
    char *inkey,            /* Input projection parameter key */
    char *outkey,           /* Output projection parameter key */
    long ddr_info,          /* Boolean flag--output proj info from ddr used? */
    long tps_info,          /* Boolean flag--input proj info from tps file? */
    struct PRO *pro         /* Input & output projection parameters */
)
{
    struct TAB_DEF tab;           /* Tabular structure of label table */
    struct VECTOR *keyptr;        /* Pointer to key field */
    struct VECTOR *codeptr;       /* Pointer to projection code field */
    struct VECTOR *zoneptr;       /* Pointer to zone code field */
    struct VECTOR *unitptr;       /* Pointer to unit of measure field */
    struct VECTOR *sphptr;        /* Pointer to datum (spheroid) code field */
    struct VECTOR *parmptr;       /* Pointer to projection coefficients field */
    long inproj_found = FALSE;    /* Boolean--input projection found? */
    long outproj_found = FALSE;   /* Boolean--output projection found? */
    long status;                  /* Function return status */
    long nonfatal = 1;            /* nonfatal flag */
    long i;                       /* Loop counter */

    /* Open the labeled table file & retrieve the label
       ------------------------------------------------*/
    open_tab(&tab, proj_name, IREAD);
    get_vector(&tab);

    /* Determine pointers to fields in the file
       ----------------------------------------*/
    keyptr = get_field_ptr(&tab, "PROJKEY");
    if (keyptr == NULL) return(E_FAIL);
    codeptr = get_field_ptr(&tab, "PROJTYPE");
    if (codeptr == NULL) return(E_FAIL);
    zoneptr = get_field_ptr(&tab, "PROJZONE");
    if (zoneptr == NULL) return(E_FAIL);
    unitptr = get_field_ptr(&tab, "PROJUNITS");
    if (unitptr == NULL) return(E_FAIL);
    sphptr = get_field_ptr(&tab, "PROJSPH");
    if (sphptr == NULL) return(E_FAIL);
    parmptr = get_field_ptr(&tab, "PROJPARMS");
    if (parmptr == NULL) return(E_FAIL);

    /* Initialize BOOLEAN variables
       ----------------------------*/
    if (tps_info)
        inproj_found = TRUE;
    if (ddr_info)
        outproj_found = TRUE;

    /* Loop until key is found or EOF
       -------------------------------*/
    for (;;)
    {
        status = get_record(&tab);
        if (status == E_FAIL) return(E_FAIL);
        if (status == E_EOF) break;
        if (strcmp(keyptr->ptr.c, inkey) == 0)
            if(!inproj_found) 
            {
                pro->in_proj = *codeptr->ptr.i4;
                pro->in_zone = *zoneptr->ptr.i4;
                pro->in_units = *unitptr->ptr.i4;
                pro->in_datum = *sphptr->ptr.i4;
                for (i = 0; i < 15; i++)
                    pro->in_parms[i] = parmptr->ptr.r8[i];
                inproj_found = TRUE;
            }

        if (strcmp(keyptr->ptr.c, outkey) == 0)  
            if(!outproj_found)
            {
                pro->out_proj = *codeptr->ptr.i4;
                pro->out_zone = *zoneptr->ptr.i4;
                pro->out_units = *unitptr->ptr.i4;
                pro->out_datum = *sphptr->ptr.i4;
                for (i = 0; i < 15; i++)
                    pro->out_parms[i] = parmptr->ptr.r8[i];
                outproj_found = TRUE;
            }
        if (inproj_found && outproj_found) break;
    }

    /* Close the file & return
       -----------------------*/
    close_tab(&tab);
    if (!inproj_found)
    {
        c_errmsg("Error finding IPROJKEY","trancoord-fatal",&nonfatal);
        return(E_FAIL);
    }
    if (!outproj_found)
    {
        c_errmsg("Error finding OPROJKEY","trancoord-fatal",&nonfatal);
        return(E_FAIL);
    }
    return(E_SUCC);
}


/*****************************************************************************
NAME                              use_ddr_info

PURPOSE    Reads projection & framing information from an image DDR

PROGRAM HISTORY
VERSION    PROGRAMMER      DATE     CODE/CONT   REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    3/89        EDC      Original development

ALGORITHM DESCRIPTION
        Retrieve the name of the input image via c_getin_nocheck()
        Get the DDR information
        If DDR information is valid,
           Pack output projection parameters from the DDR projection information
*****************************************************************************/
int use_ddr_info
(
    struct PARBLK *vblk,        /* TAE parameter block */
    struct PRO *pro,            /* Structure containing projection parameters */
    struct FRAME *frame         /* Structure containing framing parameters */
)
{
    struct DDR ddr;         /* DDR structure */
    long i;                 /* Loop counters */
    long zero = 0;          /* Constant of zero */
    long one = 1;           /* Constant of one for c_errmsg routine */
    long num_img;           /* Number of images input:  0 or 1 */
    long in_wind[4];        /* Image window specification */
    long bands[MAXBND+1];   /* Band numbers */
    long nbands;            /* Number of bands */
    long totbands;          /* Total number of bands */
    long same;              /* Same window flag */
    long qflag[2];          /* Option flags */
    char in_image[CMLEN];   /* Image file name */
    char tdum[CMLEN];       /* Tae name */

    /* Check for projection information and place it in the pro & frame 
       structures if it exists.  
       -----------------------*/
    if(c_getin_nocheck(vblk,"IN",&zero,(char (*)[256])tdum,
                (char (*)[256])in_image, &num_img,(long (*)[4])in_wind,
                (long (*)[257])bands, &nbands,&totbands,&one,&same,
                (long (*)[2])qflag) != E_SUCC) 
        return(E_FAIL);

    if (c_getddr(in_image, &ddr) != E_SUCC) 
    {
        c_errmsg("Projection information not available from DDR",
                "trancoord-noproj",&one);
        return(E_FAIL);
    }
    else
    {
        if ((ddr.valid[0] == INVAL) || (ddr.valid[3] == INVAL) || 
                (ddr.valid[4] == INVAL))
        {
            c_errmsg("Projection information not available from DDR",
                    "trancoord-noproj",&one);
            return(E_FAIL);
        }
        else
        {
            pro->out_proj = ddr.proj_code;
            if (ddr.valid[1] == INVAL) pro->out_zone = 62;
            else pro->out_zone = ddr.zone_code;
            if (ddr.valid[2] == INVAL) pro->out_datum = 0;
            else pro->out_datum = ddr.datum_code;

            pro->out_units = c_string2unit(ddr.proj_units);
            if (pro->out_units == E_FAIL)
            {
                c_lasmsg(FALSE, "trancoord-warn", 
                        "Projection units %s are invalid", ddr.proj_units);
            }

            for (i = 0; i < 15; i++)
                pro->out_parms[i] = ddr.proj_coef[i];
            if(ddr.valid[5] == INVAL) frame->p_dist[0] = frame->p_dist[1] = 0.0;
            else
            {
                frame->p_dist[0] = ddr.pdist_y;
                frame->p_dist[1] = ddr.pdist_x;
            }
            if (ddr.valid[6] == INVAL)
            {
                frame->ul_corner[0] = frame->ul_corner[1] = 0.0;
                frame->lr_corner[0] = frame->lr_corner[1] = 0.0;
            }
            else
            {
                frame->ul_corner[0] = ddr.upleft[0];
                frame->ul_corner[1] = ddr.upleft[1];
                frame->ur_corner[0] = ddr.upright[0];
                frame->ur_corner[1] = ddr.upright[1];
                frame->ll_corner[0] = ddr.loleft[0];
                frame->ll_corner[1] = ddr.loleft[1];
                frame->lr_corner[0] = ddr.loright[0];
                frame->lr_corner[1] = ddr.loright[1];

                frame->nl = ddr.nl;
                frame->ns = ddr.ns;
                frame->ddr_info = TRUE;
            }
        }
    }

    return(E_SUCC);
}
