/****************************************************************************
NAME:                           map_grid

PURPOSE:  Transforms coordinates with the LAS projection transformation package
          and grids to line, sample image coordinates

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  5.0    2/89   D. Steinwand  CSB      LAS 5.0 Original development
  5.01   7/89   D. Steinwand  CSB      Fixed bug in read of units from header
  5.02  11/89   D. Steinwand  CSB      Added information to output report
        02/00   L. Johnson    CSB      Corrected computation of coordinate
                                       min/max values rounded to multiple of
                                       pixel size.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
        Determine the coordinate input mode (are files present?)
        Open the input Tie Point Selection file, if present
           Read the projection data contained in the header record, if valid
           If projection data wasn't given from INPROJ & header projection
            info isn't valid, abort
        If input projection data isn't available, abort
        Initialize the LAS Projection Transformation Package
        Report to the user the units of the input coordinates
        Initialize data for the header of the output TPS file
        Report the units of the output coordinates to the user
        If an input TPS file is not given, initialize user prompting
        Process all points (until EOF or user is done entering)
           Get tie point
           Call the LAS Projection Transformation Package to convert the
            coordinate
           Write to output file, if one is being used
           Write to the user specified PRINT destination
        Return
*****************************************************************************/
#include <string.h>
#include <math.h>
#include "trancoord.h"
#include "util.h"

int map_grid
(
    struct PRO *pro,                 /* Input & output projection parameters */
    struct FRAME *frame,             /* Framing parameters */
    struct PRINT *print,             /* PRINT destination parameters */
    char *in,                        /* Input Tie Point Selection file name */
    char *out,                       /* Output Tie Point Selection file name */
    long msgtim,                     /* Message timer */
    float *projtol,                  /* Projection coordinate tolerance */
    char *infile                     /* Input file for tie point coordinates */
)
{
    extern long curent;
    extern long curcat;
    struct TPSHEAD header;      /* Tie Point Selection file header data */
    struct TPSDATA tpsdata;     /* Tie Point Selection file data records */
    double inx, iny;            /* X & Y coordinates input to transformation */
    double pxmax, pxmin;        /* Maximum & minimum X projection coordinates */
    double pymax, pymin;        /* Maximum & minimum Y projection coordinates */
    double uly, ulx, lry, lrx;  /* upr left, lwr right window coordinates   */ 
    long num_out_lines;         /* Number of lines in output space */
    long num_out_samps;         /* Number of samples in output space */
    long i;                     /* Loop counter */
    long in_file;               /* Boolean flag--does input TPS file exist */
    long out_file;              /* Boolean flag--does output TPS file exist */
    long status;                /* Function return status */
    long prtprm[2];             /* Print flags for c_transinit */
    int *in_tps;                /* File descriptor for input TPS */
    int *out_tps;               /* File descriptor for output TPS */
    long ptyp = TPS;            /* Type of tie point file (TPS) */
    long acc;                   /* Access mode */
    long pass = 1;              /* Flag for non-fatal errmsg()  */
    long two = 2;               /* Constant of two */

    void init_user_input1
    (
        struct PRO *pro        /* Input & output projection parameters */
    );

    void report1
    (
        struct PRINT *print,          /* Print parameters */
        struct TPSDATA *data          /* Tie point selection data records */
    );

    int check_coors
    (
        long units,                   /* Unit number coordinates are in */
        double ul_cor[2],             /* Upper left coordinates */
        double lr_cor[2]              /* Lower right coordinates */
    );

    /* Determine coordinate input mode
       -------------------------------*/
    in_file = TRUE;
    if (strcmp(in, "") == 0) in_file = FALSE;
    out_file = TRUE;
    if (strcmp(out, "") == 0) out_file = FALSE;

    if (in_file)
        fprintf(print->fptr, "Input tie point selection file:   %s\n", in);

    if (out_file)
        fprintf(print->fptr, "Output tie point selection file:  %s\n", out);

    /* Open the input Tie Point Selection file
       ---------------------------------------*/
    if(in_file)
    {
        c_inittm("TRANCOORD","Processed","coordinate number","","",&two,&two,
                &msgtim,&two);
        acc = IREAD;
        if(c_ptopen((struct PTDESC **)&in_tps, in, &ptyp, &acc, 
                    (long *)&header) != E_SUCC) 
            return(E_FAIL);

        /* Read input projection information from header, if available
           -----------------------------------------------------------*/
        if (header.proj_valid == GEO_VALID)
        {
            pro->in_proj = header.code;
            pro->in_zone = header.zone;
            pro->in_datum = header.datum;
            for (i = 0; i < 15; i++)
                pro->in_parms[i] = header.projprms[i];
            pro->in_units = c_string2unit(header.units);
            if (pro->in_units == E_FAIL)
            {
                c_lasmsg(FALSE, "trancoord-warn",
                        "Projection units %s are invalid", header.units);
            }
        }

        else if (pro->in_proj == NULL_PROJ)
        {
            c_errmsg("Input projection information not found",
                    "trancoord-inproj", &pass);
            return(E_FAIL);
        }
    }
    else if (pro->in_proj == NULL_PROJ)
    {
        c_errmsg("Input projection information not found","trancoord-inproj", 
                 &pass);
        return(E_FAIL);
    }

    header.pdist[0] = frame->p_dist[0];
    header.pdist[1] = frame->p_dist[1];

    /* Determine output projection space minimums and maximums
       -------------------------------------------------------*/
    if (frame->mode == INPUT)  /* Lat/long corners in input given as frame */
    {
        fprintf(print->fptr, "\nFramming Coordinates:\n");
        fprintf(print->fptr, "  Upper left Latitude, Longitude:   %f %f\n", 
                frame->ul_corner[0], frame->ul_corner[1]);
        fprintf(print->fptr, "  Lower right Latitude, Longitude:  %f %f\n\n",
                frame->lr_corner[0], frame->lr_corner[1]);

        status=check_coors(frame->unit_code,frame->ul_corner,frame->lr_corner);
        if (status != E_SUCC)
        {
            c_lasmsg(FALSE,"trancoord-invalid", "Framing coordinates are "
                     "outside limits of the specified units");
            return(E_FAIL);
        }

        ulx = frame->ul_corner[1];
        lrx = frame->lr_corner[1];
        uly = frame->ul_corner[0];
        lry = frame->lr_corner[0];

        if(c_calc_bound_ls_coords(&ulx, &lrx, &uly, &lry, frame->unit_code,
                    pro->out_proj, pro->out_zone, pro->out_parms,
                    pro->out_units, pro->out_datum, header.pdist[0],
                    header.pdist[1], &pxmin, &pxmax, &pymin, &pymax)
                != E_SUCC)
            return(E_FAIL);
    }
    /* Output proj coords given for two points */
    else if (frame->mode == OUTPUT_XY) 
    {
        pymax = frame->ul_corner[0] + ((frame->lscoors[0] - 1) 
                * frame->p_dist[0]);
        pxmin = frame->ul_corner[1] - ((frame->lscoors[1] - 1) 
                * frame->p_dist[1]);
        pymin = frame->lr_corner[0];
        pxmax = frame->lr_corner[1];
    }
    /* Output proj coord given (1 pt) w/nl, ns */
    else if (frame->mode == OUTPUT_LS) 
    {
        pymax = frame->ul_corner[0] + ((frame->lscoors[0] - 1) 
                * frame->p_dist[0]);
        pxmin = frame->ul_corner[1] - ((frame->lscoors[1] - 1) 
                * frame->p_dist[1]);
        pymin = pymax - ((frame->nl - 1) * frame->p_dist[0]);
        pxmax = pxmin + ((frame->ns - 1) * frame->p_dist[1]);
    }

    /* Report to the user the projection minimums and maximums of the frame
       --------------------------------------------------------------------*/
    fprintf(print->fptr, 
            "Minimum & maximum Y projection coordinates:  %lf %lf\n",
            pymin, pymax);
    fprintf(print->fptr, 
            "Minimum & maximum X projection coordinates:  %lf %lf\n\n",
            pxmin, pxmax);

    /* Report to the user the pixel size in line & sample directions
       -------------------------------------------------------------*/
    fprintf(print->fptr,
            "Pixel size in line direction:    %f\n", header.pdist[0]);
    fprintf(print->fptr,
            "Pixel size in sample direction:  %f\n\n", header.pdist[1]);

    /* Report to the user the number of lines and samples in the output space
       ----------------------------------------------------------------------*/
    num_out_lines = (long)((pymax - pymin) / header.pdist[0] + 1.5);
    num_out_samps = (long)((pxmax - pxmin) / header.pdist[1] + 1.5);
    fprintf(print->fptr,
            "Number of lines in output space:    %d\n",num_out_lines);
    fprintf(print->fptr,
            "Number of samples in output space:  %d\n\n",num_out_samps);

    /* Pack the corner projection coordinates in the file header, if used.
       -------------------------------------------------------------------*/
    header.corners[0] = pymax;
    header.corners[1] = pxmin;
    header.corners[2] = pymin;
    header.corners[3] = pxmin;
    header.corners[4] = pymax;
    header.corners[5] = pxmax;
    header.corners[6] = pymin;      
    header.corners[7] = pxmax;

    if (frame->ddr_info)
    {
        if (abs(frame->ul_corner[1] - frame->ur_corner[1]) > *projtol)
            c_errmsg("Image coords appear to be rotated","trancoord-warn",
                     &pass);
    }

    /* Inform the user of the units of measure of the coordinates
       ----------------------------------------------------------*/
    if (pro->in_units == 0)
        fprintf(print->fptr, "Unit of measure, input coordinates:  RADIANS\n");
    if (pro->in_units == 1)
        fprintf(print->fptr, "Unit of measure, input coordinates:  FEET\n");
    if (pro->in_units == 2)
        fprintf(print->fptr, "Unit of measure, input coordinates:  METERS\n");
    if (pro->in_units == 3)
        fprintf(print->fptr, "Unit of measure, input coordinates:  SECONDS\n");
    if (pro->in_units == 4)
        fprintf(print->fptr, "Unit of measure, input coordinates:  DEGREES\n");
    if (pro->in_units == 5)
        fprintf(print->fptr, "Unit of measure, input coordinates:  DMS\n");

    /* Initialize header information for open of output Tie Point Selection file
       -----------------------------------------------------------------------*/
    for (i = 0; i < 15; i++)
        header.projprms[i] = pro->out_parms[i];
    header.proj_valid = GEO_VALID;
    header.code = pro->out_proj;
    header.zone = pro->out_zone;
    header.datum = pro->out_datum;
    strcpy(header.mode, "REF");

    if (c_unit2string(pro->out_units, header.units) != E_SUCC)
    {
        /* c_unit2string displays the error message */
        c_lasmsg(FALSE, "trancoord-warn", "Setting header.units to empty");
        strcpy(header.units, " ");
    }

    fprintf(print->fptr,
            "Unit of measure, output coordinates:  %s\n",header.units);

    /* If an input TPS file is not given, initialize user prompting
       ------------------------------------------------------------*/
    if (!in_file) init_user_input1(pro);

    /* Open output Tie Point Selection file
       ------------------------------------*/
    if (out_file)
    {
        acc = IWRITE;
        if(c_ptopen((struct PTDESC **)&out_tps, out, &ptyp, &acc, 
                    (long*)&header) != E_SUCC) 
            return(E_FAIL);
    }

    /* Process all points in the file (Loop until EOF)
       -----------------------------------------------*/
    if (in_file)
    {
        curent = 1;
        c_startm();
        curent = 0;
    }

    /* Initialize the Projection Transformation Package
       ------------------------------------------------*/
    prtprm[GEO_TERM] = TRUE;
    if (in_file) prtprm[GEO_TERM] = FALSE;
    prtprm[GEO_FILE] = TRUE;
    fclose(print->fptr);

    if(c_transinit(&pro->in_proj, &pro->in_units, &pro->in_zone, &pro->in_datum,
                pro->in_parms, &pro->out_proj, &pro->out_units, &pro->out_zone,
                &pro->out_datum, pro->out_parms, prtprm, 
                print->host_name) != E_SUCC)
        return(E_FAIL);

    print->fptr = fopen(print->host_name, "a");

    for (;;)
    {
        if (in_file)
        {
            status = c_ptio((struct PTDESC **)&in_tps, (long *)&tpsdata);
            if (status == E_EOF) break;
            if (status == E_FAIL) return(E_FAIL);
            curent++;
        }
        else
        {
            status = get_tp(pro, &tpsdata, infile);
            if (status == E_EOF) break;
        }

        inx= tpsdata.coord[1];
        iny = tpsdata.coord[0];
        if ((status = c_trans(&pro->in_proj, &pro->in_units, &pro->out_proj,
                        &pro->out_units, &inx, &iny,
                        &tpsdata.coord[1], &tpsdata.coord[0])) != E_SUCC)
        {
            if (status == IN_BREAK)
            {
                c_errmsg("Specified point is in the break area","Informational"
                        ,&pass);
                tpsdata.coord[1] = 0;
                tpsdata.coord[0] = 0;
            }
            else
                return(E_FAIL);
        }


        /* Convert to line, sample coordinates
           -----------------------------------*/
        tpsdata.image_coord[0]=(pymax - tpsdata.coord[0]) / header.pdist[0] + 1;
        tpsdata.image_coord[1]=(tpsdata.coord[1] - pxmin) / header.pdist[1] + 1;

        /* Write to output file (if chosen) and report
           -------------------------------------------*/
        if (status != IN_BREAK)
        {
            if (out_file)
                if (c_ptio((struct PTDESC **)&out_tps, 
                           (long *)&tpsdata) != E_SUCC) 
                    return(E_FAIL);
            report1(print, &tpsdata);
        }
    }

    if (in_file) c_stoptm();
    return(E_SUCC);
}

/****************************************************************************
NAME:                           init_user_input

PURPOSE:  Initializes user input of coordinates (replaces TPS file read)

ALGORITHM DESCRIPTION:
        Print instructions to user
        Print a prompt appropriate to the units of the coordinates
        Return
*****************************************************************************/
void init_user_input1
(
    struct PRO *pro              /* Input & output projection parameters */
)
{

    /* Give user initial instructions for coordinate entry
       ---------------------------------------------------*/
    t_write(
          "Separate all entries with spaces.  Type <CR> on blank line to exit.",
           T_STDCC);

    /* Prompt for coordinates, based on units of coordinates expected
       --------------------------------------------------------------*/
    if (pro->in_units == 0)  /* Radians */
    {
        t_write("Coordinates should be entered in RADIANS",T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude  Longitude",T_STDCC);
        t_write("----------------------------------",T_STDCC);
    }
    if (pro->in_units == 1)  /* Feet */
    {
        t_write("Coordinates should be entered in FEET",T_DOUBLE);
        t_write("Enter:  Pt_id  Projection_Y  Projection_X",T_STDCC);
        t_write("-----------------------------------------",T_STDCC);
    }
    if (pro->in_units == 2)  /* Meters */
    {
        t_write("Coordinates should be entered in METERS",T_DOUBLE);
        t_write("Enter:  Pt_id  Projection_Y  Projection_X",T_STDCC);
        t_write("-----------------------------------------",T_STDCC);
    }
    if (pro->in_units == 3)  /* Seconds */
    {
        t_write("Coordinates should be entered in SECONDS",T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude  Longitude",T_STDCC);
        t_write("----------------------------------",T_STDCC);
    }
    if (pro->in_units == 4)  /* Degrees */
    {
        t_write("Coordinates should be entered in DEGREES",T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude  Longitude",T_STDCC);
        t_write("----------------------------------",T_STDCC);
    }
    if (pro->in_units == 5)  /* Packed DMS (SDDDMMMSSS.SS) */
    {
        t_write("Coordinates should be entered in degrees, minutes, seconds",
                T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude(DD MM SS.SS)  Longitude(DDD MM SS.SS)",
                T_STDCC);
        t_write("-------------------------------------------------------------",
                T_STDCC);
    }
}

/****************************************************************************
NAME:                           report

PURPOSE:  Writes to output (report) temporary file to be used by FILEUTIL

ALGORITHM DESCRIPTION:
        If first time through, print headers of report
        Print point id, input coordinates (Y, X), output coordinates (Y, X)
        Return
*****************************************************************************/
void report1
(
    struct PRINT *print,               /* Print parameters */
    struct TPSDATA *data               /* Tie point selection data records */
)
{
    static long first_time = 0;     /* Flag for first time through routine */
    static long count = 45;         /* Counter for new page */

    /* Print report headers first time through
       ---------------------------------------*/
    if (first_time == 0) 
    {
        first_time = 1;
        fprintf(print->fptr, 
                "\nPoint Id           Output Coordinates (y,x)    "
                "Image Coordinates (l,s)\n");
        fprintf(print->fptr, 
                "--------------- ------------------------------- "
                "---------------------\n");
    }

    /* Print point id, input coordinates (Y, X), output coordinates (Y, X)
       -------------------------------------------------------------------*/
    fprintf(print->fptr, "%-15s %15.3f %15.3f %10.3f %10.3f\n", data->pt_id,
            data->coord[0], data->coord[1], data->image_coord[0], 
            data->image_coord[1]);

    count++;
    if (count > 58) 
    {
        count = 12;
        first_time = 0;
        fprintf(print->fptr, "\f\n");
    }
}

/*****************************************************************************
NAME: CHECK_COORS

PURPOSE: This function checks the framing coordinates to see if they are
         within the limits of the specified units.

PROGRAM HISTORY:
Version   Date    Author            Request
-------   -----   ------            -------
1.0       08/99   T.Ratliff         Initial Development

******************************************************************************/
int check_coors
(
    long units,                   /* Unit number coordinates are in */
    double ul_cor[2],             /* Upper left coordinates */
    double lr_cor[2]              /* Lower right coordinates */
)
{
    /* Degrees as input
       -------------------*/
    if (units == 4)
    {
        if ((ul_cor[0] < -90.0) || (ul_cor[0] > 90.0) ||
                (ul_cor[1] < -180.0) || (ul_cor[1] > 180.0) ||
                (lr_cor[0] < -90.0) || (lr_cor[0] > 90.0) ||
                (lr_cor[1] < -180.0) || (lr_cor[1] > 180.0))
            return(E_FAIL);
    }
    /* Seconds as input
       -------------------*/
    else if (units == 3)
    {
        if ((ul_cor[0] < -324000.0) || (ul_cor[0] > 324000.0) ||
                (ul_cor[1] < -648000.0) || (ul_cor[1] > 648000.0) ||
                (lr_cor[0] < -324000.0) || (lr_cor[0] > 324000.0) ||
                (lr_cor[1] < -648000.0) || (lr_cor[1] > 648000.0))
            return(E_FAIL);
    }
    /* Radians as input
       -------------------*/
    else if (units == 0)
    {
        if ((ul_cor[0] < -2578.31) || (ul_cor[0] > 2578.31) ||
                (ul_cor[1] < -5156.62) || (ul_cor[1] > 5156.62) ||
                (lr_cor[0] < -2578.31) || (lr_cor[0] > 2578.31) ||
                (lr_cor[1] < -5156.62) || (lr_cor[1] > 5156.62))
            return(E_FAIL);
    }

    /* If here, all coordinates checked out
       ---------------------------------------*/
    return(E_SUCC);
}
