/****************************************************************************
NAME:				map_poly

PURPOSE:  Transforms coordinates with the LAS polyjection transformation package

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 2/89	D. Steinwand  CSB      LAS 5.0 Original development
  5.1    8/90   D. Etrheim    CSB      ADP #5879 Initialize the header and data
				       fields for the output TPS file for -POLY

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize min/max values
	Calculate the degree of the polynomial
	Determine the coordinate input mode
	Open the input TPS file, if used; otherwise initialize user input
	Initialize output TPS file header information
	Open the output TPS file, if used
	Process all points
	   Get the input coordinates (from file or user)
	   Apply offsets to input coordinates
	   Apply the polynomial to the input coordinates to get output coords
	   Write to file (if used)
	   Report to the user
	Report the minimum & maximum X & Y values to the user
	Return
*****************************************************************************/
#include <string.h>
#include "trancoord.h"

int map_poly
(
  struct FIT *poly,                 /* Polynomial fit parameters */
  struct PRINT *print,              /* Print parameters */
  char *in,                         /* Input TPS file name (optional) */
  char *out,                        /* Output TPS file name (optional) */
  long msgtim                       /* Message timer */
)
{
extern long curent;
extern long curcat;
struct TPSHEAD header;		/* TPS header records */
struct TPSDATA tpsdata;		/* TPS data record */
double c_eval();		/* Function to evaluate a polynomial */
double inx, iny;		/* Input X & Y coordinates */
double min_x, max_x;		/* Minimum & maximum X coordinates */
double min_y, max_y;		/* Minimum & maximum Y coordinates */
long in_file;			/* Boolean flag--input TPS file used? */
long out_file;			/* Boolean flag--output TPS file used? */
long status;			/* Function return status */
int *in_tps;			/* File descriptor for input TPS file */
int *out_tps;			/* File descriptor for output TPS file */
long ptyp = TPS;		/* Type of TPS file used (for I/O) */
long degree;			/* Degree of polynomial */
long acc;			/* File access mode */
long two=2;			/* Constant of two */

void init_user_input2
(
  void
);

int get_tp_from_user
(
  double *x,            /* User entered X coordinates */
  double *y,            /* User entered Y coordinates */
  char *pt_id           /* User entered point id */
);

void report2
(
  struct PRINT *print,          /* Print parameters */
  double x,                     /* Input X coordinates */
  double y,                     /* Input Y coordinates */
  struct TPSDATA *data,         /* Tie point data records */
  struct FIT *poly              /* Polynomial fit parameters */
);

/* Initialize min/max values
  -------------------------*/
min_x = min_y = 1000000000000.0;
max_x = max_y = -1000000000000.0;

/* Calculate degree of polynomial used
  -----------------------------------*/
if (poly->ncoef == 3) degree = 1;
if (poly->ncoef == 6) degree = 2;
if (poly->ncoef == 10) degree = 3;
if (poly->ncoef == 15) degree = 4;

/* Determine coordinate input mode
  -------------------------------*/
in_file = TRUE;
if (strcmp(in, "") == 0) in_file = FALSE;
out_file = TRUE;
if (strcmp(out, "") == 0) out_file = FALSE;

/* Initialize header information for open of output Tie Point Selection file
  -------------------------------------------------------------------------*/
if (out_file)
   clrtps(&header);

/* Open the input Tie Point Selection file
  ---------------------------------------*/
if(in_file)
   {
   c_inittm("TRANCOORD","Processed","coordinate number","","",&two,&two,
	&msgtim, &two);
   acc = IREAD;
   if(c_ptopen(&in_tps, in, &ptyp, &acc, &header) != E_SUCC) return(E_FAIL);
   }
else
   {
   if (poly->field_mode == IMAGE_COORS) strcpy(header.mode,"REF");
   if (poly->field_mode == COORS) strcpy(header.mode,"USR");
   init_user_input2();
   }

/* Open output Tie Point Selection file
  ------------------------------------*/
if (out_file)
   {
   acc = IWRITE;
   if(c_ptopen(&out_tps, out, &ptyp, &acc, &header) != E_SUCC) return(E_FAIL);
   }

/* Clear out some fields in the data record if there is an 
   output tps file. 
   -----------------------------------------------------------*/
if (out_file)
   {
   tpsdata.coord[0] = tpsdata.coord[1] = 0.0;
   tpsdata.image_coord[0] = tpsdata.image_coord[1] = 0.0;
   tpsdata.elevation = tpsdata.zoom_factor =  0.0;
   tpsdata.chip_size[0] = tpsdata.chip_size[1] = 0;
   tpsdata.offsets[0] = tpsdata.offsets[1] = 0;
   strcpy(tpsdata.chip_name,"No chip");
   }

/* Process all points in the file (Loop until EOF)
  -----------------------------------------------*/
if (in_file)
   {
   curent = 1;
   c_startm();
   curent = 0;
   }

for (;;)
   {

/* Get input coordinates
  ---------------------*/
   if (in_file)
	{
   	status = c_ptio(&in_tps, &tpsdata);
   	if (status == E_EOF) break;
   	if (status == E_FAIL) return(E_FAIL);
   	curent++;
   	if (poly->field_mode == COORS)
	   {
   	   inx = tpsdata.coord[1];
   	   iny = tpsdata.coord[0];
	   }
	else
	   {
   	   inx = tpsdata.image_coord[1];
   	   iny = tpsdata.image_coord[0];
	   }
  	}
   else
 	{
	status = get_tp_from_user(tpsdata.pt_id, &inx, &iny);
	if (status == E_EOF) break;
	}

/* Apply offsets to input coordinates
  ----------------------------------*/
	inx += poly->offset[1];
	iny += poly->offset[0];

/* Map coordinates & report results
  --------------------------------*/
   if (poly->field_mode == COORS)
	{
   	tpsdata.coord[1]=c_eval(&degree, poly->a, &inx, &iny) + poly->offset[3];
   	tpsdata.coord[0]=c_eval(&degree, poly->b, &inx, &iny) + poly->offset[2];
   	report2(print, (inx - poly->offset[1]), (iny - poly->offset[0]), 
          &tpsdata, poly);
	if (tpsdata.coord[0] > max_y) max_y = tpsdata.coord[0];
	if (tpsdata.coord[1] > max_x) max_x = tpsdata.coord[1];
	if (tpsdata.coord[0] < min_y) min_y = tpsdata.coord[0];
	if (tpsdata.coord[1] < min_x) min_x = tpsdata.coord[1];
	}
   else
	{
   	tpsdata.image_coord[1] = c_eval(&degree, poly->a, &inx, &iny) + 
           poly->offset[3];
   	tpsdata.image_coord[0] = c_eval(&degree, poly->b, &inx, &iny) + 
           poly->offset[2];
   	report2(print, (inx - poly->offset[1]), (iny - poly->offset[0]), 
          &tpsdata, poly);
	if (tpsdata.image_coord[0] > max_y) max_y = tpsdata.image_coord[0];
	if (tpsdata.image_coord[1] > max_x) max_x = tpsdata.image_coord[1];
	if (tpsdata.image_coord[0] < min_y) min_y = tpsdata.image_coord[0];
	if (tpsdata.image_coord[1] < min_x) min_x = tpsdata.image_coord[1];
	}

   if (out_file)
   	if (c_ptio(&out_tps, &tpsdata) != E_SUCC) return(E_FAIL);
   }

c_stoptm();

/* Report the minimum & maximum projection coordinates to the user
  ---------------------------------------------------------------*/
fprintf(print->fptr, "\nMinimum X coordinate:  %lf\n", min_x);
fprintf(print->fptr, "Minimum Y coordinate:  %lf\n", min_y);
fprintf(print->fptr, "Maximum X coordinate:  %lf\n", max_x);
fprintf(print->fptr, "Maximum Y coordinate:  %lf\n", max_y);

return(E_SUCC);
}

/****************************************************************************
NAME:				init_user_input

PURPOSE:  Initializes user input of coordinates (replaces TPS file read)

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 2/89	D. Steinwand  CSB      LAS 5.0 Original development

ALGORITHM DESCRIPTION:
	Print instructions to user
	Print prompting header info
	Return
*****************************************************************************/
void init_user_input2
(
  void
)
{
t_write(
    "Separate all entries with spaces.  Type <CR> on blank line to exit.",
    T_STDCC);
t_write("Enter:  Pt_id  Y-Coordinate X-Coordinate",T_STDCC);
t_write("----------------------------------------",T_STDCC);
}

/****************************************************************************
NAME:				get_tp_from_user

PURPOSE:  Gets a tie point coordinate from the user

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 2/89	D. Steinwand  CSB      LAS 5.0 Original development

ALGORITHM DESCRIPTION:
	Increment the point (prompt) counter
	Do until satisfactory input has been given:
	   Print prompt
	   Get the input string
	   Parse the input string
	   If done, return EOF
	   Unpack into point_id, Y coord, X coord
	   If valid, break out of loop
	   Tell user to re-enter input
	Return SUCCESS
*****************************************************************************/
int get_tp_from_user
(
  double *x,            /* User entered X coordinates */
  double *y,            /* User entered X & Y coordinates */
  char *pt_id           /* User entered point id */
)
{
char mess[80];		/* Message buffer (also holds user input) */
long term;		/* Terminal code for TAE t_ calls */
static long pt_num = 0;	/* Point (prompt) number */
long status;		/* Function return status flag */

pt_num++;

for (;;)
   {
   sprintf(mess," %d.  ",pt_num);
   t_write(mess,T_NULL);
   t_read(mess, &term);
 
   if (strlen(mess) == 0) return(E_EOF);

   status = sscanf(mess, "%s %lf %lf", pt_id, y, x);
   if (status == 3)
	{
	if (c_ckptid(pt_id)) break;
	else t_write("Point id is not unique; re-enter!",T_STDCC);
	}
   else t_write("Illegal value entered; re-enter!",T_STDCC);
   }
return(E_SUCC);
}

/****************************************************************************
NAME:				report

PURPOSE:  Writes to output (report) temporary file to be used by FILEUTIL

ALGORITHM DESCRIPTION:
	If first time through, print headers (coefs, offsets) of report
	Print point id, input coordinates (Y, X), output coordinates (Y, X)
	Return
*****************************************************************************/
void report2
(
  struct PRINT *print,          /* Print parameters */
  double x,                     /* Input X coordinates */
  double y,                     /* Input Y coordinates */
  struct TPSDATA *data,         /* Tie point data records */
  struct FIT *poly              /* Polynomial fit parameters */
)
{
static long first_time = 0;	/* Flag for first time through routine */
long i;				/* Loop counter */
char term[15][20];		/* Character constants:  terms of polynomial */

/* Print report headers first time through
  ---------------------------------------*/
if (first_time == 0) 
   {
   first_time = 1;

/* Initialize the "terms of polynomial" contsant
  ---------------------------------------------*/
   strcpy(&(term[0][0]), "Constant");
   strcpy(&(term[1][0]), "X");
   strcpy(&(term[2][0]), "Y");
   strcpy(&(term[3][0]), "X**2");
   strcpy(&(term[4][0]), "XY");
   strcpy(&(term[5][0]), "Y**2");
   strcpy(&(term[6][0]), "X**3");
   strcpy(&(term[7][0]), "(X**2)Y");
   strcpy(&(term[8][0]), "X(Y**2)");
   strcpy(&(term[9][0]), "Y**3");
   strcpy(&(term[10][0]), "X**4");
   strcpy(&(term[11][0]), "(X**3)Y");
   strcpy(&(term[12][0]), "(X**2)(Y**2)");
   strcpy(&(term[13][0]), "X(Y**3)");
   strcpy(&(term[14][0]), "Y**4");

/* Print polynomial coefficients
  -----------------------------*/
   fprintf(print->fptr,"\n");
   fprintf(print->fptr, 
     "First polynomial mapping line or Y coordinates:\n");
   for (i = 0; i < poly->ncoef; i++)
      fprintf(print->fptr, "      a%d:  %20.12e  (%s)\n",i, poly->b[i],
			&(term[i][0]));
   fprintf(print->fptr, 
    "\nSecond polynomial mapping sample or X coordinates:\n");
   for (i = 0; i < poly->ncoef; i++)
      fprintf(print->fptr, "      b%d:  %20.12e  (%s)\n",i,poly->a[i],
                        &(term[i][0]));
   fprintf(print->fptr,"\n");

/* Report the offsets applied
  --------------------------*/
   fprintf(print->fptr,"Offset applied to input lines:     %f\n",
               poly->offset[0]);
   fprintf(print->fptr,"Offset applied to input samples:   %f\n",
               poly->offset[1]);
   fprintf(print->fptr,"Offset applied to output lines:    %f\n",
               poly->offset[2]);
   fprintf(print->fptr,"Offset applied to output samples:  %f\n\n",
               poly->offset[3]);

/* Print table header
  ------------------*/
   if (poly->field_mode == IMAGE_COORS) fprintf(print->fptr, 
"\n Point Id      Input Coordinates(L,S)           Output Coordinates(L,S)\n");
   else fprintf(print->fptr, 
"\n Point Id      Input Coordinates(Y,X)           Output Coordinates(Y,X)\n");
   fprintf(print->fptr, 
"---------- ------------------------------- -------------------------------\n");
   }

/* Print point id, input coordinates (Y, X), output coordinates (Y, X)
  -------------------------------------------------------------------*/
if (poly->field_mode == COORS) fprintf(print->fptr, 
   "%-10s %15.3f %15.3f %15.3f %15.3f\n", data->pt_id, y, x, 
   data->coord[0], data->coord[1]);
else fprintf(print->fptr, "%-10s %15.3f %15.3f %15.3f %15.3f\n", 
   data->pt_id, y, x, data->image_coord[0], data->image_coord[1]);
}
