/****************************************************************************
NAME:                           map_proj

PURPOSE:  Transforms coordinates with the LAS projection transformation package

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  5.0    2/89   D. Steinwand  CSB      LAS 5.0 Original development
  5.01   7/89   D. Steinwand  CSB      Fixed bug appling units from header
  5.02   8/92   S. Nelson     CSB      Set header.name to "" and
                                       tpsdata.image_coord to 0.0 to clean up
                                       tiepoint file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
        Determine the coordinate input mode (are files present?)
        Open the input Tie Point Selection file, if present
           Read the projection data contained in the header record, if valid
           If projection data wasn't given from INPROJ & header projection
            info isn't valid, abort
        If input projection data isn't available, abort
        Initialize the LAS Projection Transformation Package
        Report to the user the units of the input coordinates
        Initialize data for the header of the output TPS file
        Report the units of the output coordinates to the user
        If an input TPS file is not given, initialize user prompting
        Process all points (until EOF or user is done entering)
           Get tie point
           Call the LAS Projection Transformation Package to convert the
            coordinate
           Write to output file, if one is being used
           Write to the user specified PRINT destination
        Report minimum/maximum projection coordinates to the user
        Return
*****************************************************************************/
#include <string.h>
#include "trancoord.h"
#include "util.h"

int map_proj
(
    struct PRO *pro,                 /* Input & output projection parameters */
    struct PRINT *print,             /* PRINT destination parameters */
    char *in,                        /* Input Tie Point Selection file name */
    char *out,                       /* Output Tie Point Selection file name */
    long msgtim                      /* Message timer */
)
{
    extern long curent;
    extern long curcat;
    struct TPSHEAD header;       /* Tie Point Selection file header data */
    struct TPSDATA tpsdata;      /* Tie Point Selection file data records */
    double inx, iny;             /* X & Y coordinates input to transformation */
    double min_x, max_x;         /* Minimum & maximum X coordinates */
    double min_y, max_y;         /* Minimum & maximum Y coordinates */
    long i;                      /* Loop counter */
    long in_file;                /* Boolean flag--does input TPS file exist */
    long out_file;               /* Boolean flag--does output TPS file exist */
    long status;                 /* Function return status */
    long prtprm[2];              /* Print flags for c_transinit */
    int *in_tps;                 /* File descriptor for input TPS */
    int *out_tps;                /* File descriptor for output TPS */
    long ptyp = TPS;             /* Type of tie point file (TPS) */
    long acc;                    /* Access mode */
    long pass = 1;               /* Flag for non-fatal errmsg()  */
    long two = 2;                /* Constant of two */
    char empty[1];               /* Empty string to send to get_tp() */

    void init_user_input3
    (
        struct PRO *pro               /* Input & output projection parameters */
    );

    void report3
    (
        struct PRINT *print,          /* Print parameters */
        double x,                     /* Input x values */
        double y,                     /* Input y values */
        struct TPSDATA *data          /* Tie point data record */
    );

    /* Initialize min/max values
       -------------------------*/
    min_x = min_y = 1000000000000.0;
    max_x = max_y = -1000000000000.0;
    empty[0] = '\0';

    /* Determine coordinate input mode
       -------------------------------*/
    in_file = TRUE;
    if (strcmp(in, "") == 0) in_file = FALSE;
    out_file = TRUE;
    if (strcmp(out, "") == 0) out_file = FALSE;

    /* Open the input Tie Point Selection file
       ---------------------------------------*/
    if(in_file)
    {
        c_inittm("TRANCOORD","Processed","coordinate number","","",&two,&two,
                &msgtim,&two);
        acc = IREAD;
        if(c_ptopen((struct PTDESC **)&in_tps, in, &ptyp, &acc, 
                    (long *)&header) != E_SUCC) 
            return(E_FAIL);

        /* Read input projection information from header, if available
           -----------------------------------------------------------*/
        if (header.proj_valid == GEO_VALID)
        {
            pro->in_proj = header.code;
            pro->in_zone = header.zone;
            pro->in_datum = header.datum;
            for (i = 0; i < 15; i++)
                pro->in_parms[i] = header.projprms[i];

            pro->in_units = c_string2unit(header.units);
            if (pro->in_units == E_FAIL)
            {
                c_lasmsg(FALSE, "trancoord-warn",
                        "Projection units %s are invalid", header.units);
            }
        }
        else if (pro->in_proj == NULL_PROJ)
        {
            c_errmsg("Input projection information not found",
                    "trancoord-inproj", &pass);
            return(E_FAIL);
        }
    }
    else if (pro->in_proj == NULL_PROJ)
    {
        c_errmsg("Input projection information not found",
                 "trancoord-inproj", &pass);
        return(E_FAIL);
    }

    /* Initialize the Projection Transformation Package
       ------------------------------------------------*/
    prtprm[GEO_TERM] = TRUE;
    if (in_file) prtprm[GEO_TERM] = FALSE;
    prtprm[GEO_FILE] = TRUE;
    fclose(print->fptr);

    if(c_transinit(&pro->in_proj, &pro->in_units, &pro->in_zone, &pro->in_datum,
                pro->in_parms, &pro->out_proj, &pro->out_units, &pro->out_zone,
                &pro->out_datum, pro->out_parms, prtprm, 
                print->host_name) != E_SUCC)
        return(E_FAIL);

    print->fptr = fopen(print->host_name, "a");

    /* Inform the user of the units of measure of the coordinates
       ----------------------------------------------------------*/
    if (pro->in_units == 0)
        fprintf(print->fptr, "Unit of measure, input coordinates:  RADIANS\n");
    if (pro->in_units == 1)
        fprintf(print->fptr, "Unit of measure, input coordinates:  FEET\n");
    if (pro->in_units == 2)
        fprintf(print->fptr, "Unit of measure, input coordinates:  METERS\n");
    if (pro->in_units == 3)
        fprintf(print->fptr, "Unit of measure, input coordinates:  SECONDS\n");
    if (pro->in_units == 4)
        fprintf(print->fptr, "Unit of measure, input coordinates:  DEGREES\n");
    if (pro->in_units == 5)
        fprintf(print->fptr, "Unit of measure, input coordinates:  DMS\n");

    /* Initialize header information for open of output Tie Point Selection file
       -------------------------------------------------------------------------*/
    header.pdist[0] = 0.0;
    header.pdist[1] = 0.0;
    for (i = 0; i < 8; i++)
        header.corners[i] = 0.0;
    for (i = 0; i < 15; i++)
        header.projprms[i] = pro->out_parms[i];
    header.proj_valid = GEO_VALID;
    header.code = pro->out_proj;
    header.zone = pro->out_zone;
    header.datum = pro->out_datum;
    strcpy(header.name,"");
    if (pro->out_proj != 0)
        strcpy(header.mode, "PRO");
    else
        strcpy(header.mode, "GEO");

    if (c_unit2string (pro->out_units, header.units) != E_SUCC)
    {
        /* c_unit2string displays the error message */
        c_lasmsg(FALSE,"ddr-warn", "Setting header.units to empty");
        strcpy (header.units, " ");
    }

    fprintf(print->fptr,"Unit of measure, output coordinates:  %s\n",
            header.units);

    /* If an input TPS file is not given, initialize user prompting
       ------------------------------------------------------------*/
    if (!in_file) init_user_input3(pro);

    /* Open output Tie Point Selection file
       ------------------------------------*/
    if (out_file)
    {
        acc = IWRITE;
        if(c_ptopen((struct PTDESC **)&out_tps, out, &ptyp, &acc,
                    (long*)&header) != E_SUCC) 
            return(E_FAIL);
    }

    /* Process all points in the file (Loop until EOF)
       -----------------------------------------------*/
    if (in_file)
    {
        curent = 1;
        c_startm();
        curent = 0;
    }
    for (;;)
    {
        if (in_file)
        {
            status = c_ptio((struct PTDESC **)&in_tps, (long *)&tpsdata);
            if (status == E_EOF) break;
            if (status == E_FAIL) return(E_FAIL);
            curent++;
        }
        else
        {
            status = get_tp(pro, &tpsdata, empty);
            if (status == E_EOF) break;
        }

        inx= tpsdata.coord[1];
        iny = tpsdata.coord[0];
        if ((status = c_trans(&pro->in_proj, &pro->in_units, &pro->out_proj, 
                        &pro->out_units, &inx, &iny, 
                        &tpsdata.coord[1], &tpsdata.coord[0])) != E_SUCC)
        {
            if (status == IN_BREAK)
            {
                c_errmsg("Specified point is in the break area","Informational"
                        ,&pass);
                tpsdata.coord[1] = 0;
                tpsdata.coord[0] = 0;
            }
            else
                return(E_FAIL);
        }
        tpsdata.image_coord[0] = 0.0;
        tpsdata.image_coord[1] = 0.0;

        if (status != IN_BREAK)
        {
            if (out_file)
                if (c_ptio((struct PTDESC **)&out_tps, 
                           (long *)&tpsdata) != E_SUCC) 
                    return(E_FAIL);
            report3(print, inx, iny, &tpsdata);
            if (tpsdata.coord[0] > max_y) max_y = tpsdata.coord[0];
            if (tpsdata.coord[1] > max_x) max_x = tpsdata.coord[1];
            if (tpsdata.coord[0] < min_y) min_y = tpsdata.coord[0];
            if (tpsdata.coord[1] < min_x) min_x = tpsdata.coord[1];
        }
    }

    c_stoptm();

    /* Report the minimum & maximum projection coordinates to the user
       ---------------------------------------------------------------*/
    fprintf(print->fptr,"\nMinimum X coordinate:  %lf\n",min_x);
    fprintf(print->fptr,"Minimum Y coordinate:  %lf\n",min_y);
    fprintf(print->fptr,"Maximum X coordinate:  %lf\n",max_x);
    fprintf(print->fptr,"Maximum Y coordinate:  %lf\n",max_y);

    return(E_SUCC);
}

/****************************************************************************
NAME:                           init_user_input

PURPOSE:  Initializes user input of coordinates (replaces TPS file read)

ALGORITHM DESCRIPTION:
        Print instructions to user
        Print a prompt appropriate to the units of the coordinates
        Return
*****************************************************************************/
void init_user_input3
(
    struct PRO *pro             /* Input & output projection parameters */
)
{

    /* Give user initial instructions for coordinate entry
       ---------------------------------------------------*/
    t_write(
          "Separate all entries with spaces.  Type <CR> on blank line to exit.",
          T_STDCC);

    /* Prompt for coordinates, based on units of coordinates expected
       --------------------------------------------------------------*/
    if (pro->in_units == 0)  /* Radians */
    {
        t_write("Coordinates should be entered in RADIANS",T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude  Longitude",T_STDCC);
        t_write("----------------------------------",T_STDCC);
    }
    if (pro->in_units == 1)  /* Feet */
    {
        t_write("Coordinates should be entered in FEET",T_DOUBLE);
        t_write("Enter:  Pt_id  Projection_Y  Projection_X",T_STDCC);
        t_write("-----------------------------------------",T_STDCC);
    }
    if (pro->in_units == 2)  /* Meters */
    {
        t_write("Coordinates should be entered in METERS",T_DOUBLE);
        t_write("Enter:  Pt_id  Projection_Y  Projection_X",T_STDCC);
        t_write("-----------------------------------------",T_STDCC);
    }
    if (pro->in_units == 3)  /* Seconds */
    {
        t_write("Coordinates should be entered in SECONDS",T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude  Longitude",T_STDCC);
        t_write("----------------------------------",T_STDCC);
    }
    if (pro->in_units == 4)  /* Degrees */
    {
        t_write("Coordinates should be entered in DEGREES",T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude  Longitude",T_STDCC);
        t_write("----------------------------------",T_STDCC);
    }
    if (pro->in_units == 5)  /* Packed DMS (SDDDMMMSSS.SS) */
    {
        t_write("Coordinates should be entered in degrees, minutes, seconds",
                T_DOUBLE);
        t_write("Enter:  Pt_id  Latitude(DD MM SS.SS)  Longitude(DDD MM SS.SS)",
                T_STDCC);
        t_write("-------------------------------------------------------------",
                T_STDCC);
    }
}

/****************************************************************************
NAME:                           report

PURPOSE:  Writes to output (report) temporary file to be used by FILEUTIL

ALGORITHM DESCRIPTION:
        If first time through, print headers of report
        Print point id, input coordinates (Y, X), output coordinates (Y, X)
        Return
*****************************************************************************/
void report3
(
    struct PRINT *print,          /* Print parameters */
    double x,                     /* Input x values */
    double y,                     /* Input y values */
    struct TPSDATA *data          /* Tie point data record */
)
{
    static long first_time = 0;     /* Flag for first time through */

    /* Print report headers first time through
       ---------------------------------------*/
    if (first_time == 0) 
    {
        first_time = 1;
        fprintf(print->fptr, 
                "\n Point Id         Input Coordinates              "
                "Output Coordinates\n");
        fprintf(print->fptr, 
                "---------- ------------------------------- "
                "-------------------------------\n");
    }

    /* Print point id, input coordinates (Y, X), output coordinates (Y, X)
       -------------------------------------------------------------------*/
    fprintf(print->fptr, "%-10s %15.3f %15.3f %15.3f %15.3f\n", data->pt_id, 
            y, x, data->coord[0], data->coord[1]);
}
