/****************************************************************************
NAME:				TRANCOORD

PURPOSE:  Converts coordinates from one coordinate system to another, 
          including to/from any of the LAS Projection Transformation
          Package projections and via polynomial transformation.  The
          user also may grid the projection spaces to line, sample space
          using the appropriate framing parameters.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 3/89	D. Steinwand  CSB      LAS 5.0 Original development
  5.1    6/90   D. Etrheim    CSB      Trap status of c_setprt
  5.2    8/90	D. Etrheim    CSB      ADP #5786 Initialize the header and data
				       fields of the output TPS file for -POLY
  5.3	 8/92	S. Nelson     CSB      Made adjustments to correct problems
				       encountered when rebuilding geolib.
  7.0    7/98   J. Janssen    CSB      Corrected process of how we retrieve
				       projection information in getpar 


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize LAS processing
	Get all parameters (including parameters read from disk files)
	If PROJ subcommand is chosen, convert map projection coordinates
	If GRID subcommand is chosen, convert coordinates & grid to line,sample
	If POLY subcommand is chosen, convert coordinates w/polynomial
	Setup information for the completion message
	Write the standard completion message & exit

ALGORITHM REFERENCES:
	This version of TRANCOORD is entirely new for version 5.0. Capabilities
	are similar to LAS 4.0 TRANCOORD (by M. Nishihama & K. Narayanan) and
        to newLAS INTCAM (by J. Peterson, J. Thormodsgard, B. Ailts, and D.
        Steinwand).  Other references include:

        Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
        Survey Proffessional Paper 1395 (Supersedes USGS Bulletin 1532), United
        State Government Printing Office, Washington D.C., 1987. 

        "Software Documentation for GCTP General Cartographic Transformation
        Package", U.S. Geological Survey National Mapping Division, May 1982.

        Clarie, Charles N, "State Plane Coordinates by Automatic Data 
        Processing", U.S. Department of Commerce, Environmental Science 
        Services Admin., Coast and Geodetic Survey, Publication 62-4, 1973.
*****************************************************************************/
#include <string.h>
#include "trancoord.h"

void main
(
  void
)		
{
struct PARBLK vblk;	/* TAE parameter block */
struct PRO pro;		/* Input and output projection parameters */
struct FRAME frame;	/* Framing parameters for the GRID subcommand */
struct FIT poly;	/* Polynomial coefficients for the POLY subcommand */
struct PRINT prt;	/* PRINT destination parameters */
long option;		/* Flags for subcommand chosen */
long msgtim;		/* Message timer */
long std_out;		/* Standard output LUN */
long status;		/* Function return status */
long fatal = -1;	/* Fatal flag for errmsg */
long num_out;		/* Number of output images generated */
char tps_name[CMLEN];	/* The input Tie Point Selection file name */
char out_tps[CMLEN];	/* The output Tie Point Selection file name */
char tae_name[2][CMLEN];/* TAE names of output files (TPS & report) */
char infile[CMLEN];	/* Input file */
float projtol;          /* projection coordinate tolerance                  */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get all program parameters (TAE & those derrived from files)
  ------------------------------------------------------------*/
status=getpar(&vblk,&option,tps_name,out_tps,&pro,&frame,&poly,&prt,
              &msgtim, &projtol, infile);
if(status!=E_SUCC)c_errmsg("Fatal error encountered","trancoord-fatal",&fatal);

/* Transform tie points, based on the subcommand (option) chosen
  -------------------------------------------------------------*/
if(option==PROJ_ONLY)status=map_proj(&pro,&prt,tps_name,out_tps,msgtim);
if(option==PROJ_GRID)status=map_grid(&pro,&frame,&prt,tps_name,
                                     out_tps,msgtim,&projtol,infile);
if(option== POLYNOMIAL)status=map_poly(&poly,&prt,tps_name,out_tps,msgtim);
if(status!=E_SUCC)c_errmsg("Fatal error encountered","trancoord-fatal",&fatal);

/* Set up file names for completion message
  ----------------------------------------*/
num_out = 0;
if(strcmp(out_tps,"") != 0) 
   {
   strcpy(tae_name[num_out], out_tps);
   num_out++;
   }
if(strcmp(prt.name," ") != 0) 
   {
   strcpy(tae_name[num_out], prt.name);
   num_out++;
   }

/* Write the standard completion message and exit
  ----------------------------------------------*/
c_complt("TRANCOORD", tae_name, &num_out);
}
