/*******************************************************************************
NAME      		   GETPAR


PURPOSE   To retrieve and edit the parameters.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TRANSLAS must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   GETPAR retrieves the required TAE parameters from the parameter block.  The
   input images are parsed so that the window and band specifications are
   known.  Error checking is done on the input image(s) to insure that they 
   all have equal window sizes.  
******************************************************************************/


#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,	/* I: Parameter block	             */
    char subcmd[],		/* O: Subcommand chosen	      	     */
    long *msgtim,		/* O: Number of seconds between
					   processing messages       */
    char minmax[],		/* O: Min/max flag		     */
    long *dflag,                /* O: DDR update option		     */
    long *nl,			/* O: Number of lines in input image */
    long *ns,			/* O: Number of samples in input image*/
    long *nbands,		/* O: Number of bands in input image */
    long *nheader,		/* O: Number of header bytes	     */
    long *idtype,		/* O: Input data type		     */
    long *odtype,		/* O: Output data type		     */
    char taein[][CMLEN], 	/* O: Image file TAE nam             */
    char hosin[][CMLEN], 	/* O: Image file HOST name    	     */
    char taeout[][CMLEN],	/* O: TAE file name of output image  */
    char hosout[][CMLEN],	/* O: Host file name of output image */
    long *nfiles,		/* O: Number of transfer files       */
    long window[][4], 	        /* O: Window specification	     */
    long bands[][MAXBND+1],     /* O: Band numbers specified	     */
    char ioname[][CMLEN],	/* O: Output host file name(s)       */
    long *totbnd,		/* O: Total number of bands specified*/
    long *nmrout,		/* O: Number of output files         */
    char *bform,		/* O: Band format option	     */
    long *nmrimg,		/* O: number of input images	     */
    long *inbands		/* O: Number of bands in transfer file*/
)
{

struct	VARIABLE        *in;		/* Pointer to TAE variable within the
					   parameter block		     */

static  struct  PRS_LST p_list[MAXIMG]; /* Image list returned from imgprs */

	long		element;	/* element for getin call	     */
	long 		fatal = -1;	/* fatal error flag		     */
	long		i;		/* Loop index			     */
	long		imglimit;	/* Number of images to expect input  */
	long		nonfatal = 1;	/* nonfatal error flag		     */
	long		qflag[MAXIMG][2];/* Were window and bands allowed to
					   default?			     */
	long		same;	        /* Signals whether multiple images
					   have the same window		     */
	long		status;		/* Status indicator		     */
	long		std_out;	/* Standard output		     */
	char		dtype[5];	/* Data type buffer                  */
	char 	        dflag_str[3];   /* DDR update option string          */
	char 		inparm[CMLEN];	/* temporary name		     */
        char 		dir[CMLEN];	/* directory name		     */
        char 		dir1[CMLEN];	/* directory name		     */
        char 		ext[CMLEN];	/* extension name		     */
        char 		root[CMLEN];	/* root name			     */


/* LAS initialization
------------------------------------------------*/
	c_inlas(vblock, &std_out);

/* Determine the subcommand chosen
------------------------------------------------*/
	in = p_find(vblock, "_SUBCMD");
	strcpy(subcmd,SVAL(*in,0));

/* Get the TAE globals needed
------------------------------------------------*/
	in = p_find(vblock,"$MSGTIM");
	*msgtim = IVAL(*in,0);
	in = p_find(vblock,"$MINMAX");
	strcpy(minmax,SVAL(*in,0));

/* Get the band format option
-------------------------------*/
	in = p_find(vblock, "BFORM");
	strcpy(bform,SVAL(*in,0));

/* Get the number of lines, samples and bands in the input image
------------------------------------------------------------------*/
	if (strcmp(subcmd, "IN") == 0)
	    {
            in = p_find(vblock, "$DDRVFLG");
            strcpy(dflag_str, SVAL(*in,0));
 
            if (strcmp (dflag_str,"IN") == 0)
                *dflag = INVAL;
            else
                *dflag = UNKNOW;
	    in = p_find(vblock, "NL");
	    *nl=IVAL(*in,0);
	    in = p_find(vblock, "NS");
	    *ns=IVAL(*in,0);
	    in = p_find(vblock, "NBANDS");
	    *inbands=IVAL(*in,0);
	    in = p_find(vblock, "NHEADER");
	    *nheader=IVAL(*in,0);

/* Retrieve input data type
--------------------------------------------------*/
            in = p_find(vblock, "IDTYPE");
            strcpy(dtype,SVAL(*in,0));
            if (strcmp(dtype,"BYTE") == 0)
               *idtype = EBYTE;
            else if (strcmp(dtype,"I*2") == 0)
               *idtype = EWORD;
            else if (strcmp(dtype,"I*4") == 0)
               *idtype = ELONG;
            else if (strcmp(dtype,"R*4") == 0)
               *idtype = EREAL;

/* Retrieve output data type
--------------------------------------------------*/
	    in = p_find(vblock, "ODTYPE");
            strcpy(dtype,SVAL(*in,0));
            if (strcmp(dtype,"BYTE") == 0)
               *odtype = EBYTE;
            else if (strcmp(dtype,"I*2") == 0)
               *odtype = EWORD;
            else if (strcmp(dtype,"I*4") == 0)
               *odtype = ELONG;
            else if (strcmp(dtype,"R*4") == 0)
               *odtype = EREAL;
	    else if (strcmp(dtype, "SAME") == 0)
	       *odtype = *idtype;

/* Get the name(s) of the input file(s)	and make sure the correct number of
   input names was specified
----------------------------------------------------------------------------*/
            in = p_find(vblock, "INFILE");
            *nfiles = in->v_count;

            for (i=0; i < *nfiles; i++)
               {
               strcpy(inparm,SVAL(*in,i));
               if( E_FAIL == c_imgprs(inparm,p_list,nmrimg))
                  {
                  c_errmsg("Input image parse was unsuccessful",
                           "translas-parse", &nonfatal);
                  c_errmsg("Fatal error encountered","translas-fatal",&fatal);
                  }
               if (strchr(p_list[0].name, TAE_DIR_BEG))
                  c_cmhost(p_list[0].name,ioname[i]);
               else if (strchr(p_list[0].name,HOST_DIR_BEG))
                  {
                  strcpy(ioname[i],p_list[0].name);
                  }
               else
                  c_cmhost(p_list[0].name, ioname[i]);

               window[i][SL] = (long)p_list[0].window[SL];
               window[i][SS] = (long)p_list[0].window[SS];
               window[i][NL] = (long)p_list[0].window[NL];
               window[i][NS] = (long)p_list[0].window[NS];
               }
 
            status = wndchk(window,nl,ns,nfiles);
            if (status != E_SUCC)
                c_errmsg("Fatal error encountered","translas-fatal",&fatal);
            status = bndchk(bands[0],inbands,totbnd,p_list[0]);
            if (status != E_SUCC)
                c_errmsg("Fatal error encountered","translas-fatal",&fatal);
            if ((*nfiles != 1) && (*nfiles != *totbnd))
                c_errmsg("Error specifying number of TRANSFER files",
                                                "translas-names",&fatal);

/* Get the name of the output file(s)
-----------------------------------------------------------------*/
	    in = p_find(vblock, "OUT");
            *nmrout = in->v_count;
            if ((*nmrout !=1) && (*nmrout != *totbnd))
               {
               c_errmsg("Number of output images must equal the number of bands",
                        "translas-out",&nonfatal);
               c_errmsg("Fatal error encountered","translas-fatal",&fatal);
               }
	    status = c_getout (vblock,"OUT",taeout,hosout);
	    if (status != E_SUCC)
		c_errmsg("Fatal error encountered","translas-fatal",&fatal);
	    }
	    		/* End of subcommand -- IN */
	else
	    {

/* Get the name of the input file
-------------------------------------------------*/
	    element = 0;
	    imglimit = MAXIMG;
	    status = c_getin (vblock, "IN", &element, taein, hosin, nmrimg,
			window, bands, nbands, totbnd, &imglimit, &same, qflag);
	    if (status != E_SUCC)
		c_errmsg("Fatal error encountered","translas-fatal",&fatal);
	    if (same == 0)
		{
		c_errmsg("Input images do not have the same window size",
						"translas-size",&nonfatal);
		c_errmsg("Fatal error encountered","translas-fatal",&fatal);
		}
/* Get the data type of the input image
----------------------------------------*/
	    if (c_gettyp(hosin,nmrimg,odtype)== E_FAIL)
		c_errmsg("Fatal error encountered","translas-fatal",&fatal);

/* Get the name(s) of the output file(s) and make sure the correct number
   of output names was specified
-------------------------------------------------------------------------*/
	    in = p_find(vblock, "OUTFILE");
	    *nfiles = in->v_count;

	    if ((*nfiles != 1) && (*nfiles != *totbnd))
	       c_errmsg("Error specifying number of TRANSFER files",
					      "translas-names",&fatal);	
	    for (i = 0; i < *nfiles; i++)
		{
                /* Format the directory name properly
                -----------------------------------*/
		strcpy(inparm,SVAL(*in,i));
                c_splitname (inparm, dir1, root, ext);
                if (strcmp(dir1,"") == 0)
                   {
#ifndef sun
                   getwd(dir);
#else
                   getcwd(dir,CMLEN);
#endif
		   strcat(dir,"/\0");
                   }
                else
                if (dir1[0] != '/')
                   {
#ifndef sun
                   getwd(dir);
#else
                   getcwd(dir,CMLEN);
#endif
		   strcat(dir,"/\0");
                   }
                else
                   dir[0] = '\0';
              

                if (strcmp(ext,"") == 0)
                   sprintf(ioname[i],"%s%s%s",dir,dir1,root); 
                else
                   sprintf(ioname[i],"%s%s%s.%s",dir,dir1,root,ext); 
		}
            *nmrout = *nfiles;
	    }

	/* If separate files are used for each band, process as BSQ
	----------------------------------------------------------*/
	if (*nfiles != 1)
	   strcpy(bform, "BSQ"); 

	return(E_SUCC);

} /* End of getpar */
