/******************************************************************************

PROGRAM:	TRANSLAS

PURPOSE:	Conversion between TRANSLAS (pure data) and LAS NLIO files.
		TRANSLAS has been updated allowing it to be used with
		the TMFLOPPY procedure. TMFLOPPY is used for the generation
		of TM floppy disk products.  The update included the following
		enhancements:
		     1) Window and band options are now allowed as part
	        	of the input image name for TRANSLAS-OUT
		     2) Images of all data types (byte, integer*2, integer*4,
		        real*4) are presently allowed as input. 

PROGRAM HISTORY:
 mode Version   Date        Author           Code/Contr   Change Request
  -------       ----        ------           ----------   --------------
   1.0		6/85        T. Butzer		EDC	  New Development
		11/87	    L. Huewe		EDC	  Updated to
							  standards

   5.0          05/88  	    R. McKinney		EDC	  Convert to 5.0
   5.0		12/88	    R. McKinney		EDC	  Updated to close
							  all files
   5.1          05/89       B.Ailts             EDC       Took out blocked 
							  reads + opened a 
							  VMS file with var-
							  iable record type
							  in order to be 
							  compatible 
							  with VMSIN
   5.2          08/89       B.Davis             EDC       used rms io for 
							  writing vms
							  output files,
							  incorporated an
							  ncp read routine
							  for reading vms
							  files.
   5.3          09/89       B.Davis             EDC       oops, added same
							  rms io call for 
							  multi-band con-
							  dition also.
   5.4          05/90       B.Ailts             EDC	  Added size of data
							  type to bufsiz
							  calculation
            	08/91       T.Mittan            EDC       Ability to skip 
							  header
            	11/91       T.Mittan            EDC       Checked return status
							  of vms open call 
   5.5		04/92	    S.Nelson		EDC	  Added functions to
							  process BIL format.
							  Updated err messages.
							  Extended limit of
							  bands to 256.
   5.6		08/93	    S.Nelson		EDC	  Allowed multiple
							  images to be entered.
   5.7          05/94       D.Eichacker       	EDC       Modified opt to opt[3]
   5.8      	08/94       D. Etrheim          EDC       Change 777 to 666 on 
							  file open
   5.9		12/94	    T. Mittan		EDC	  Modified to use host
							  name on ouput
   7.1		03/99	    J. Janssen          EDC       Modified to accept 
							  standard LAS window & 
   							  band specifications.
							  Modified to allow LAS 
 							  syntax or host syntax 
							  for INFILE names and 
							  to allow more than 1 
							  output file to be 
							  specified. 
   7.2		06/99	    D. Hames		EDC	  Modified ODTYPE by 
							  adding parameter 
							  IDTYPE.This eliminates
							  the assumption that
							  the input image is the
							  same data type as the
							  output image.
   7.3		11/99	    D. Hames		EDC       Remove 50000 limit 
                                                          from translas.pdf. Add
							  the file name to the
							  error message that
                                                          occurs when a TRANSFER
							  file does not exist.
							  Add prototypes. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    TRANSLAS must be run under TAE. 

PROJECT	       	LAS		                

******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "desc.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
int  fdesc[MAXIMG+1];		/* image file descriptors used for image I/O */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

void main
(void)
{
struct	PARBLK		vblock;		/* Parameter block		      */

        char            bform[4];	/* Band format option                 */
	char		hosout[MAXBND][CMLEN];	
					/* Buffer storage for HOST name       */
static	char 		hosin[MAXIMG][CMLEN];
					/* Input file name(s)		      */
	char		minmax[4];	/* Min/max flag			      */
static	char 		ioname[MAXBND][CMLEN];
					/* Output host file name(s)	      */
	char 		subcmd[4];	/* Subcommand chosen		      */
	char		taein[MAXIMG][CMLEN];
					/* TAE names of input images	      */
	char		taeout[MAXBND][CMLEN];
       					/* Buffer storage for TAE name	      */

static	long	        bands[MAXIMG][MAXBND+1];/* Band numbers specified     */
	long		dflag;	        /* delete flag 			      */
	long		i;		/* loop counter variable	      */
	long		inbands;	/* Number of bands in Transfer file   */
	long		idtype;		/* Data type of INFILE		      */
	long		odtype;		/* Data type of the output image      */
	long		msgtim;		/* Number of seconds between processing
					   messages			      */
	long    	nbands[MAXIMG];	/* Number of bands in input image     */
	long    	nheader;  	/* Number of header bytes in input 
					   image     			      */
	long 		nl;		/* Number of lines in input image     */
	long		nfiles;		/* Number of transfer files	      */
	long		nmrimg;		/* Number of input images	      */
	long		nmrout;		/* Number of output files created     */
	long		ns;		/* Number of samples in input image   */
	long		one = 1;	/* Constant one			      */
	long            totbnd;   	/* total number of bands              */
	long		window[MAXIMG][4]; /* Window specification	      */
	long		zero = 0;	/* zero value			      */

/* Get user specified parameters and TAE globals
-----------------------------------------------------------------------------*/

	getpar (&vblock,subcmd,&msgtim,minmax,&dflag,&nl,&ns,nbands,&nheader,
	        &idtype,&odtype,taein,hosin,taeout,hosout,&nfiles,window,bands,
		ioname,&totbnd,&nmrout,bform,&nmrimg,&inbands);

/* Conversion from DAL file to LAS image file
-----------------------------------------------------------------------------*/
	if (strcmp(subcmd, "IN") == 0)
	    {

	    /* Call function to process from BSQ or separate files
	     ----------------------------------------------------*/
	    if (strcmp(bform, "BSQ") == 0)
	    	indal(&vblock,hosout,nfiles,nl,ns,inbands,(long *)window,
                      bands[0],nheader,nmrout,ioname,idtype,odtype,msgtim,
		      minmax,dflag,totbnd);

	    /* Call function to process from BIL format
	     -----------------------------------------*/
	    else			 
	    	inbil(&vblock,hosout,ns,inbands,(long *)window,bands[0],nheader,
		      nmrout,ioname,idtype,odtype,msgtim,minmax,dflag,totbnd);
	

/* If output file is a LAS image, create history record
-----------------------------------------------------------------------------*/
            for (i = 0; i < nmrout; i++)
	       if (c_hstout(&vblock,hosout[i],hosout,&zero," ") != E_SUCC)
	          c_errmsg("Nonfatal error encountered","translas-warn",&one);

/* Output completion message for DAL to LAS conversion
-------------------------------------------------------*/
            c_complt("TRANSLAS", taeout, &nmrout);
	    }

/* Conversion from LAS image to DAL files.
----------------------------------------------------------------------------*/
         else
	    {

	    /* Call function to process BIL file or separate files
 	     -----------------------------------------------------*/
	    if (strcmp(bform, "BSQ") == 0)
	    	outdal(&vblock,hosin,nfiles,window,bands,nbands, odtype,
						ioname,msgtim,nmrimg,totbnd);
	    /* Call function to process BSQ file
	     ----------------------------------*/
	    else
	    	outbil(&vblock,hosin,window,bands,nbands, odtype, ioname,
						msgtim,nmrimg,totbnd);

/* Output completion message for LAS to DAL cnversion
-------------------------------------------------------*/
            c_complt("TRANSLAS", ioname, &nmrout);
	    }
	 free(taeout);
	 free(hosout);

}  /* End of main */
