#include "trig.h"
#include "util.h"
#include "protos.h"

/*********************************************************************
NAME                  CALCTRG

FUNCTION NAME         TRIG

PURPOSE:              Opens data files, groups them, performs the actual 
                      calculation of the requested trig function, (arcsin, 
                      arccos, or arctan), using the pixels from the user 
                      provided input image or images, and closes the files.

PROGRAM HISTORY

VERSION      PROGRAMMER         DATE    CODE/CONT       REASON
-------      ----------         ----    ---------       ------
1.0          B. DEALY	        9/84	SAR		Original development
2.0	     B. Kovalick	5/86	SAR		Added timed message
3.0	     M. Mack		5/89	GSFC/636	Conversion to 5.0
                                                        and combination of
							three related fns.
							(ARCSIN, ARCCOS, AND
							ARCTAN), into one,
							TRIG. Similar routine
							in each of these
							earlier codes was
							called COMCALC.
4.0	     F. Irani   	7/90	STX		fix inmnmx call
							removed redeclaration
							of buf.
5.0	     M.Mack             9/90	GSFC		Changed timed report 
							message and calculation
							of minmax. Also changed
                                                        the requirement that
							two images had to
                                                        have the same start
							sample and line - 4.1
							had precluded this in
							execution, (even if
							user's guide had
							appeared to allow it),
							but it did not make 
                                                        sense to have such a
							restriction.
 5.1         D. Etrheim		2/91	EDC		Allocate space dynamicly
							General cleanup.
 6.1	     D. Eichacker	5/94	EDC		Added opt[].
  
FUNCTIONS CALLED:

 - WRITTEN FOR THIS FUNCTION:
   NONE

 - LAS LIBRARY FUNCTIONS:
   c_abort     -
   c_clean     -
   c_eclose    -
   c_egclse    -
   c_egroup    -
   c_eopens    -
   c_errmsg    -
   c_estep     -
   c_inmnmx    -
   c_inittm    -
   c_minmax    -
   c_startm    -
   c_stoptm    -
   c_svmnmx    -
   c_upddr     -

 - STANDARD C FUNCTIONS:
   asin        -
   acos        -
   atan        -
   c_ealloc    -
   free        -
   sizeof      -
   sprintf     -
   strcmp      -
   strcpy      -

ALGORITHM:

After values are initialized, the files are initialized and opened 
for image i/o.
The ddr for the output image is derived using the input images and
input information.
Space for the data is obtained and the images are grouped.
For each line,
    for each pixel,
        for each band of an image, depending upon the desired sub-function, 
        the program will calculate the trig value for the pixel value 
        using either one or two input images, (again, as requested by 
        the user). The calculated output value for each pixel is then
        placed into the output file.
The files are closed. If desired, the minimum-maximum value of the output 
image is calculated. The dynamically allocated space is freed.
The number of invalid input values, (those outside the range of -1 to +1), 
that were discovered are printed for each band.
***********************************************************************/


void calctrg
(

    long *func,                   /* indicates which function is being 
                                      accessed:
                                      1=arcsin; 2=arccos; 3=arctan     */
    char hosname[][CMLEN], /* host name of first input image */
    char hosname2[][CMLEN],/* host name of second input image*/
    char hosin[][CMLEN],   /* host names of input images, (needed
                                     in one array for upddr routine) */
    char outhost[][CMLEN], /* Output host file descriptor */
    long window[][4],
    long window2[][4],
    long totwin[][4],    /* total number of windows */
    long *nibs1,          /* number of bands in input 
                                         image 1 */
    long *nibs2,          /* number of bands in input 
                                         image 2 */
    long *nibso,          /* number of bands in output 
                                         image  */
    long (*bands1)[MAXBND + 1],
    long (*bands2)[MAXBND + 1],
    long *totbnd1,                /* Total number of bands 
					  in 1st input */
    long *totbndo,                /* Total number of bands 
					  in output */
    long *nim1,                  /* number of input images in first 
                                    "image" */
    long *nim2, 	          /* number of input images in second 
                                    "image" */
    char *mnmx,                  /* Indicator of desire for 
                                         minimum-maximum value 
                                         calculation */
    float *atzero,                /* value user wishes used if zero
                                     occurs in calculation:
                                       in arcsin, when both numerator pixel
                                         from input file one and
                                         denominator pixel from input
                                         file two, (or even just
                                         denominator pixel from input
                                         file two by itself), are (is)
                                         zero                
                                       in arccos, when both numerator pixel
                                         from input file one and
                                         denominator pixel from input
                                         file two, (or even just
                                         denominator pixel from input
                                         file two by itself), are (is)
                                         zero                
                                       in arctan, if both numerator pixel
                                         from first input file and 
                                         denominator pixel from second
                                         input file are zero */
    long *noimin                /* Indicates whether one or two
                                       images were input:
                                       1 means one image was input
                                       2 means 2 images were input */
)


/*  ------ End of argument list - beginning of local code -----  */

 {
extern  struct FDESC * fdesc1[MAXIMG + 1];        /* file descriptors input 1*/
extern  struct FDESC * fdesc2[MAXIMG + 1];        /* file descriptors input 2*/
extern  struct FDESC * fdesc[2*MAXIMG + 1];       /* file descriptors-all    */

/*  Local Variables:                                       */

  long acc1 = IREAD;       /* access mode for 1st input 
                                          data set, (read only)*/
  long acc2 = IREAD;       /* access mode for 2nd input 
                                          data set, (read only)*/
  long acc3 = IWRITE;      /* access mode for output 
                                          data set, (write) */
  long      band;  		  /* index to loop through all bands  */

  long      bandso[1][MAXBND+1];
                                  /* Band specification for output 
                                          image*/

  long      bnd;                  /* Used in printout of errors 
					  by band */
  long      bufsiz;               /* size of image I/O buffer  */
  long      dtypei1 = EREAL;      /* Indicates desire to take first 
                                         input in data of all types but
                                         convert it to real */
  long      dtypei2 = EREAL;      /* Indicates desire to take second  
                                         input in data of all types but
					 convert it to real  */
  long      dtypeo = EREAL;       /* Indicates output data type 
					 will be real */
  long      image;	    	  /* Index to loop through all images */
  long      j;	         	  /* Index to calculation loop     */
  double    line_inc;		  /* Line increment needed for updating 
					 the DDR  */
  long      msg_flg = 0;          /* Message flag indicating desire for
                                         BSQ output of messages */
  long      nm1 = 0;              /* number of input images in first 
                                    "image" (set to nim1 but not
				    a pointer; used in comparisons)*/
  long      nm2 = 0;              /* number of input images in second 
                                    "image" (set to nim2 but not 
				    a pointer; used in comparisons)*/

  long      nmrimg;               /* Total number of input images   */
                                
  long non_fatal=1;       /* Non-fatal error indicator */ 
  long *num_invalid;
			         /* Storage for no. of incorrect 
                                         values by band*/
  long      noline;              /* Holds no. of lines for this 
					 window */
  long      nosamp;              /* Holds no. samples for this
					 window */
  long one = 1;        /* Used in routine calls requiring 
					 a long 1 */
  float     *ptin1;              /* Image I/O first input 
					 buffer pointer */
  float     *ptin2;              /* Image I/O second input 
					 buffer pointer */
  float     *ptout;              /* Image I/O output 
					 buffer pointer */
  float     quotient;            /* Temp. storage for quotient 
                                         of two input arguments */
  float     rt;                  /* Storage for real argument of
                                         trig fns. - 1 input case */
  float     rt2;                 /* Storage for 2nd real argument of
                                         trig fns. - 2 input case */
  double    samp_inc;	         /* Samp increment needed for updating 
					 the DDR  */
  long      sizei1;              /* Size of first input image pixel  */
  long      sizei2;              /* Size of second input image pixel */
  long      sizeo;               /* Size of output image pixel  */
  char      stmes[65];           /* Actual message printed for
					 invalid values found
					 in input  */
  char      str[9];              /* Holds number of invalid values 
					 found in input */
  char strnb[14];		 /* Used with strng, str, and stmes
				    to give error message */
  static char strng[45]="values outside the valid range of -1 to +1: ";
				 /* Informational error message */
  long      upddr_flag;		 /* Combination flag indicator 
					 for up DDR    */
  float	    value;               /* Used for temporary storage of
					 calculated trig value */

  static long opt[3]={0,0,0};	 /* Optional parameter array for EOPENS	*/

  long wino[1][4];                  /* Window for output image  */
  long zero = 0; 	         /* Used in routine calls   */
  long    fatal_err = -1;        /* Used in errmsg calls    */


/*                   
   The following variables are used in image I/O routines
*/

  long      curlin;              /* current line number for display 
				         message */

  double    *minval;      /* Minimum pixel value array  */

  double    *maxval;      /* Maximum pixel value array  */

/*
/////////////////////////////////////////////////////////////////////
                Start of executable code for calctrg
/////////////////////////////////////////////////////////////////////

-------------------------------------------------------------------
		Initialize values
------------------------------------------------------------------
*/
offset1 = (long *) malloc ((MAXIMG * MAXBND + 1) * sizeof(long));
offset2 = (long *) malloc ((MAXIMG * MAXBND + 1) * sizeof(long));
if ((offset1 == NULL) || (offset2 == NULL))
   {
   c_errmsg("Error allocating dynamic memory","trig-alloc",
         &non_fatal);
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   }
offset3 = (long *) malloc ((MAXBND + 1) * sizeof(long));
if (offset3 == NULL)
   {
   c_errmsg("Error allocating dynamic memory","trig-alloc",
         &non_fatal);
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   }
minval = (double *) malloc (MAXBND * sizeof(double));
maxval = (double *) malloc (MAXBND * sizeof(double));
if ((minval == NULL) || (maxval == NULL))
   {
   c_errmsg("Error allocating dynamic memory","trig-alloc",
         &non_fatal);
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   }
num_invalid = (long *) malloc ((MAXBND + 1) * sizeof(long));
if (num_invalid == NULL)
   {
   c_errmsg("Error allocating dynamic memory","trig-alloc",
         &non_fatal);
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   }

sizei1 = sizeof(float);
sizei2 = sizeof(float);
sizeo  = sizeof(float);

/* --------------------------------------------------------------------
       set up the files for input with imageio calls
--------------------------------------------------------------------*/
nm1 = *nim1;

for (band = 0, image = 0; image < nm1; image++)
   {
   totwin[image][SL] = window[image][SL];
   totwin[image][SS] = window[image][SS];
   totwin[image][NL] = window[image][NL];
   totwin[image][NS] = window[image][NS];
 
/* *** Check if offset is by image or by band for one image  *** */

    if(c_eopens(&fdesc1[image], hosname[image], bands1[image],  
		&window[0][SL], &window[0][SS], &window[0][NL],
             	&window[0][NS], &offset1[band],  &acc1, &dtypei1, 
		&zero, opt) != E_SUCC)
         {
         c_abort(&vblock, "First image could not be opened", 
                "trig-open1", "trig", outhost, &zero);
         }

    fdesc[image]=fdesc1[image];
    band += nibs1[image];
     }
    nibso[0] = band;
    nibso[1]= 0;
   if(*noimin == TWOIN)
   {

     nm2 = *nim2;
     for (band = 0, image = 0; image < nm2; image++)
     {
      totwin[image + nm1][SL] = window2[image][SL];
      totwin[image + nm1][SS] = window2[image][SS];
      totwin[image + nm1][NL] = window2[image][NL];
      totwin[image + nm1][NS] = window2[image][NS];
      if(c_eopens(&fdesc2[image], hosname2[image], 
		    bands2[image], &window2[0][SL], &window2[0][SS], 
		    &window2[0][NL], &window2[0][NS], &offset2[band], 
                    &acc2, &dtypei2, &zero, opt) != E_SUCC)
       {
       c_abort(&vblock, "Second image could not be opened", 
                "trig-open2", "trig", outhost, &zero);
       }  

      fdesc[image+nm1] = fdesc2[image];
      band += nibs2[image];
     }

   }

/*----------------------------------------------------------------- 
      			set up output image
------------------------------------------------------------------*/

    *totbndo = *totbnd1;

    if(c_eopens(&fdesco, outhost[0], totbndo, &one, 
             &one, &window[0][NL], &window[0][NS], offset3, 
             &acc3, &dtypeo, &zero, opt) != E_SUCC)
       {
       c_abort(&vblock, 
		"Output image could not be opened", 
                "trig-openo", "trig", outhost, &one);
        }

    fdesc[nm1+nm2] = fdesco;
    fdesc[1+nm1+nm2] = 0;

/* ---------------------------------------------------------------
  		Update the DDR for the output image
------------------------------------------------------------------*/
    line_inc = (double) one;
    samp_inc = (double) one;
    upddr_flag = NOCOMB;
    if(*noimin == TWOIN) upddr_flag = COMB;
    nmrimg = nm1 + nm2;
    
    if (c_upddr(&vblock, hosin, outhost[0], &nmrimg, totwin,
                   bands1, nibs1, &line_inc, &samp_inc,
		   totbndo, &upddr_flag) != E_SUCC)
       c_clean(&vblock,"trig",outhost,&one);

    nosamp = window[0][NS];
/*------------------------------------------------------------------
       Get space for the image data
-------------------------------------------------------------------*/
   
   if ((buf = (unsigned char * )c_ealloc(fdesc, &bufsiz)) == NULL)
       c_abort(&vblock, "Error allocating dynamic memory", 
                "trig-alloc", "trig", outhost, &one);

/*--------------------------------------------------------------------
                               Group images                          
--------------------------------------------------------------------*/

  if(c_egroup(fdesc,&gdesc,buf,&bufsiz) != E_SUCC)
    {
     if(c_eclose(&fdesc[0]) != E_SUCC)
       c_errmsg("Error closing image files", "trig-close", &non_fatal);
  
    c_errmsg("Error grouping image files", "trig-group", &non_fatal);
    c_clean(&vblock,"trig",outhost,&one);
    } 

c_inittm("TRIG","Processed", "line", "band(s)", " ", &window[0][NL], 
	  totbndo, &msgtim, &msg_flg);

curent = 1;
curcat = 1;
c_startm();

/* Initialize min/max arrays
   ------------------------- */


   if (strcmp(mnmx,"YES") == 0)
     {

      if(c_inmnmx(&dtypeo, totbndo, minval, maxval) != E_SUCC)
        {
         c_abort(&vblock, 
          "Minimum and maximum values were not initialized", 
				"trig-nomnmx", "trig", outhost, &one);
         }
      }


for(j=0; j<MAXBND; j++)
  num_invalid[j]=0;

/*--------------------------------------------------------------------
                          Line by Line
---------------------------------------------------------------------*/
noline = window[0][NL];
for (curlin=0; curlin < noline; curlin++)
 {

   if (c_estep (&gdesc) != E_SUCC)
     {
     if(c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing image files", "trig-close", &non_fatal);
 
     c_abort(&vblock, "Error reading image files", 
	   "trig-noread", "trig", outhost, &one);
     }

/*
-----------------------------------------------------------
		Pixel by Pixel
-----------------------------------------------------------
*/
  for( j=0; j<nosamp;j++)
    {

/*--------------------------------------------------------------------
                            Band by band                            
---------------------------------------------------------------------*/ 

    for (band=0; band < *totbndo; band++)
     {

/*
       *** interleaving for each line in these images ***

         *** do the transformation for the output. Load       ***
         *** inputs into temporary arrays. ***

*/
      if (*noimin  ==  ONEIN)
                                         /* one input file used */
        {
             ptin1 = (float *) &buf[(offset1[band] + j)*sizei1];
             rt = *ptin1;
             ptout = (float *) &buf[(offset3[band] + j)*sizeo];
             
             switch (*func)
             {

             case ARCSIN:

              {
              if (rt < -1.0)
                 {
                  rt = -1.0;
                  num_invalid[band] += 1;
                 }
               else if (rt > 1.0)
                 {
                  rt = 1.0;
                  num_invalid[band] += 1;
                 }
   		  value=asin(rt);
		  *ptout = value;

	      break;

              }

             case ARCCOS:

              {
              if(rt < -1.0)
                {
                 rt = -1.0;
                 num_invalid[band] += 1;
                }
               else if (rt > 1.0)
                {
                 rt = 1.0;
                 num_invalid[band] += 1;
                }

                value=acos(rt);
        	*ptout = value;

               break;

              }

             case ARCTAN:

             {
	     value = atan (rt);
             *ptout = value;
             break;
             }
        }
       }
      else
                                         /* two input files used */
       {
             ptin1 = (float *)&buf[(offset1[band] + j)*sizei1];
             rt = *ptin1;
             ptin2 = (float *)&buf[(offset2[band] + j)*sizei2];
             rt2 = *ptin2;
             ptout = (float *)&buf[(offset3[band] + j)*sizeo];

            switch (*func)
            {
             case ARCSIN:

              {
              if ( ( (rt2 == 0)  &&  (rt == 0) ) || (rt2 == 0) )

                {
                  value = *atzero;
		  *ptout = value;
                }
              else
                {
                 quotient = rt/rt2;
                 if (quotient < -1.0)
                   {
                     quotient = -1.0;
                     num_invalid[band] += 1;
                    }
                 else if (quotient > 1.0)
                    {
                     quotient = 1.0;
                     num_invalid[band] += 1;
                    }
                  value=asin(quotient);
		  *ptout = value;
                 }
              break;

              } 

           case ARCCOS:

           {
            if ( ( (rt2 == 0)  &&  (rt == 0) ) || (rt2 == 0) )
              {
                 value = *atzero;
		 *ptout = value;
              }
            else
              {
                quotient = rt/rt2;
                if(quotient < -1.0)
                  {
                   quotient = -1.0;
                   num_invalid[band] += 1;
                  }
                 else if (quotient > 1.0)
                  {
                   quotient = 1.0;
                   num_invalid[band] += 1;
                  }
                  value=acos(quotient);
		  *ptout = value;
            
              }
             break; 
           }
           case ARCTAN:

           {
            if ( ( (rt2 == 0)  &&  (rt == 0) ) || (rt2 == 0) ) 
              {
                value = *atzero;
	        *ptout = value;

              }
            else
              {
                value= atan2(rt,rt2);
		*ptout = value;
              } 
            break; 
           }

          }
        }
      }


      }
                    

   curent++;

   } 

/* ------------------------------------------------------
	Call estep once more to remove leftover garbage
-------------------------------------------------------*/

 if (c_estep (&gdesc) != E_SUCC)
   {
   if(c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing image files", "trig-close", &non_fatal);

    c_errmsg("Error reading image files", "trig-noread", &non_fatal);
    c_clean(&vblock,"trig",outhost,&one);
   }

c_stoptm();

/*-----------------------------------------------------------
			Close the files
----------------------------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
    c_errmsg("Error closing image files", "trig-close", &non_fatal);

/* ------------------------------------------------------
			Free the buffer space
-------------------------------------------------------*/
free(buf);
free(offset1);
free(offset2);


/*--------------------------------------------------------
		Create the associated min-max file
-------------------------------------------------------*/
if(strcmp(mnmx,"YES") == 0)
 {

   for (band = 0; band< *totbndo;++band)
      {
          bandso[0][band] = band+1;
      }
          wino[0][SS] = wino[0][SL] = 1;
          wino[0][NS] = window[0][NS];
          wino[0][NL] = window[0][NL];

    if(c_minmax(outhost, &one, wino, bandso, totbndo, 
		minval, maxval) != E_SUCC)
        c_errmsg("Error calculating MIN/MAX values",
                "trig-minmax",&non_fatal);

   if(c_svmnmx(outhost[0],totbndo,minval,maxval,mnmx) != E_SUCC)
	c_errmsg("MIN/MAX values were not saved",
			"trig-save",&non_fatal);
    }

/*
----------------------------------------------------------
		Print out number of invalid values found 
		(number outside of valid range)
----------------------------------------------------------
*/

 for(band=0;band<*totbndo;band++)
  {
    str[8]='\0';

    if(num_invalid[band] != 0)
	{
        bnd=band+1;
        sprintf(strnb,"For band %2d,",bnd);
	sprintf(str,"%d",num_invalid[band]);
        strcpy(&stmes[0],strnb);
        strcpy(&stmes[12],strng);
        strcpy(&stmes[56],str);
	c_errmsg(stmes, "trig-information", &non_fatal);
        }
   }
return;
}/* End of calctrg.c */
