#include "trig.h"
#include "util.h"
#include "protos.h"

/*****************************************************************
NAME			GETPAR

FUNCTION                TRIG

PURPOSE   To get the necessary parameters and perform the preliminary
          edit checking.

PROGRAM HISTORY
PROGRAMMER	DATE	     CODE/CONT	REASON
----------	----         ---------	------

(For orig. separate programs in LAS):
B. DEALY	Sept. 18 '84 SAR	ver. 1 implementation
B. KOVALICK	May    1 '86 SAR     	added timed message

M. MACK		May   11 '89 GSFC/636	converted to 5.0 stds. and
                                        combined arcsin, arccos, 
                                        and arctan into one pkg (TRIG).
F. Irani	July  11 '90 STX	Added flag check to c_getout.
M. Mack         Sept  19 '90 GSFC       Changed names of msg. added by
					Irani to be consistent with
					others.
D. Etrheim      2/91         EDC        Allocate space dynamicly and make vars.
 					static.  General cleanup.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	GETPAR must be run under TAE.
PROJECT		LAS 5.0

LANGUAGE	C

FUNCTIONS CALLED

 - WRITTEN FOR THIS FUNCTION:
   NONE

 - LAS LIBRARY FUNCTIONS:
   c_errmsg    -
   c_getin     -
   c_getnum    -
   c_getout    -
   p_find      -

 - STANDARD C FUNCTIONS:
   ealloc
   strcat
   strcmp
   strcpy

ALGORITHM (FLOW)
	Getpar first determines the subcommand used.
	Secondly, the value of NOIMIN is set depending upon
            the chosen subcommand.
	Next, (depending upon NOIMIN), one or two input
            images are received.
 	Edit checking is performed for windowing, etc.
	The output image is retrieved.
	The globals are checked and control returns to the driver.
***************************************************************** */


void getpar
(


/* Arguments to  function    */
    long *func,                  /* Indicates which trig function
                                     was requested: 1=arcsin, 2=arccos,
                                         3=arctan                 */
    char hosname[][CMLEN],
    char hosname2[][CMLEN],
    char las_names[][CMLEN],
    char outfile[][CMLEN],       /* Pointer to Output image LAS TAE file
                                         descriptor */
    char outhost[][CMLEN],       /* Pointer to Output host file 
                                         descriptor */
    long window[][4],      /* Image window for first 
                                      input "image" */
    long window2[][4],     /* Image window for second 
                                      input "image" */
    long *nibs1,                 /* number of bands(images) in input 
                                         image 1 */
    long *nibs2,                 /* number of bands(images) in input 
                                         image 2 */
    long *nibso,                 /* number of bands in output image  */
    long (*bands1)[MAXBND + 1],  /* Band specification for first image */
    long (*bands2)[MAXBND + 1],  /* Band specification for second image */
    long *totbnd1,                /* Total number of bands to be 
					  processed in input 1    */
    long *totbnd2,                /* Total number of bands to be 
					  processed in input 2    */
    long *nmrimg1,                /* number of images defined for 
					 first input   */
    long *nmrimg2,                /* number of images defined for 
					 second input  */
    char *mnmx,                  /* Min-max indicator */
    float *atzero,                /*  value user wishes used if zero
                                      occurs in calculation:
                                      in arcsin, when both numerator pixel
                                         from input file one and
                                         denominator pixel from input
                                         file two, (or even just
                                         denominator pixel from input
                                         file two by itself), are (is)
                                         zero                
                                      in arccos, when both numerator pixel
                                         from input file one and
                                         denominator pixel from input
                                         file two, (or even just
                                         denominator pixel from input
                                         file two by itself), are (is)
                                         zero                
                                      in arctan, if both numerator pixel
                                         from first input file and 
                                         denominator pixel from second
                                         input file are zero */
    long *noimin                /* Indicates whether one (noimin=1) 
                                         or two (noimin=2)
                                         images were input  */
)

{

/* Local variables   */

  static long flag[MAXIMG][2] = {0,0};
                                  /* Used in GETIN call      */
  long flag1 = 0;          /* status value for function */
  long imglimit= MAXIMG;   /* image band limit - maximum 
                                     no. of images this routine 
                                     can handle  */
  char   method[11];              /* alphanumeric description of desired
                                     trig function and input file desire:
                                     ARCSIN-ONE; ARCSIN-TWO; 
                                     ARCCOS-ONE; ARCCOS-TWO;
				     ARCTAN-ONE; ARCTAN-TWO           */
  long    non_fatal=1;            /* Non-fatal error indicator */
  long one = 1;            /* Long word for 1 used in 
					 GETIN call   */
  long   same_window;             /* flag indicating if images 
                                     have same window */
  struct VARIABLE *var;           /* pointer to a variable in 
					 TAE block */
  long zero = 0;           /* long word for 0 used in 
					 GETIN call */
  long    fatal_err = -1;                /* Used in errmsg calls    */


/*
/////////////////////////////////////////////////////////////////////
           start of executable code for getpar.c 
////////////////////////////////////////////////////////////////////


  --------------------------------------------------------------
               Get input parameters
            (in particular, subcommand)
---------------------------------------------------------------*/

var = p_find  (&vblock, "_SUBCMD"); 
strcpy(method, SVAL(*var,0));

if (strcmp(method,"ARCSIN_ONE") == 0)
  {
   *func = ARCSIN;
   *noimin=ONEIN;
  }          

if (strcmp(method,"ARCSIN_TWO") == 0)
  {
   *func = ARCSIN;
   *noimin=TWOIN;
  }

if (strcmp(method,"ARCCOS_ONE") == 0)
  {
   *func = ARCCOS;
   *noimin=ONEIN;
  }

if (strcmp(method,"ARCCOS_TWO") == 0)
  {
   *func = ARCCOS;
   *noimin=TWOIN;
  }

if (strcmp(method,"ARCTAN_ONE") == 0)
  {
   *func = ARCTAN;
   *noimin=ONEIN;
  }

if (strcmp(method,"ARCTAN_TWO") == 0)
  {
   *func = ARCTAN;
   *noimin=TWOIN;
  }

if (*noimin  ==  TWOIN) 
   {
   flag1 = c_getin (&vblock, "IN", &one, las_names[1], hosname2,
                    nmrimg2, window2, bands2, nibs2,
                    totbnd2, &imglimit, 
		    &same_window,flag);
   if(flag1 != E_SUCC) 
        c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
    
   flag1 = c_getin (&vblock, "IN", &zero, las_names, hosname,
                    nmrimg1, window, bands1, nibs1, 
		    totbnd1, &imglimit,
                     &same_window, flag);
   if(flag1 != E_SUCC)
        c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);

   var = p_find(&vblock,"PIXVAL");
   *atzero = RVAL(*var,0);
   }
else
   {
   flag1 = c_getin (&vblock, "IN", &zero, las_names, hosname,
                    nmrimg1, window, bands1, nibs1,  
		    totbnd1, &imglimit,
                    &same_window, flag);
   if(flag1 != E_SUCC) 
        c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   } 

/*--------------------------------------------------------------
  check that inputs have same number of bands
--------------------------------------------------------------*/

if (*noimin == TWOIN  &&  *nibs1 != *nibs2)
 {
  c_errmsg ("Both input images must have the same number of bands", 
              "trig-bands",&non_fatal);
  c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
  }
if (c_getout (&vblock, "OUT", outfile, outhost) != E_SUCC)
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
 
*nibso = *nibs1;

/*  --- Note nl and ns are defined by default in DDR structure */

/*-------------------------------------------------------------
                       2nd image window check
--------------------------------------------------------------*/

if (*noimin == TWOIN)
 
   {
     if (window[0][NS]  !=  window2[0][NS] ) 
       {
       c_errmsg ("Input images have a different number of samples", 
                      "trig-window1", &non_fatal);
       c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
       } 
     if (window[0][NL]  !=  window2[0][NL] ) 
       {
       c_errmsg ("Input images have a different number of lines", 
                      "trig-window2", &non_fatal);
       c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
       }
 }
  
/*------------------------------------------------------------
              -- get elapsed time between display messages
-------------------------------------------------------------*/


var = p_find(&vblock,"$MSGTIM");
msgtim = IVAL(*var,0);

/*-------------------------------------------------------------
     get min max flag
---------------------------------------------------------------*/
var = p_find  (&vblock, "$MINMAX"); 
strcpy(mnmx,SVAL(*var,0));

return; 
} /* End of getpar.c    */
