/* include files, global parameters, and definitions for TRIG  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct	PARBLK	vblock;                /* TAE parameter block     */

struct FDESC *fdesco;                        /* file descriptor output  */
struct GDESC *gdesc;                         /* group descriptor        */
long	*offset1,  /* offsets into image i/o  */
	*offset2;  /* for input bands         */
long    *offset3;               /* offset into image i/o   */
					       /* for output bands        */
long 	msgtim;                        /* message interval        */
 
unsigned char	*buf;                  /* image i/o buffer        */

/* parameters for processing message utility, (used by inittm) */
extern	long	curent;                        /* current processing line */
extern	long	curcat;                        /* current processing band */

#define ONEIN   1
#define TWOIN   2

#define ARCSIN  1
#define ARCCOS  2
#define ARCTAN  3

/*******************************************************************
NAME			TRIG

FUNCTION                TRIG

PURPOSE:	TRIG performs various trigonometric functions on one
  		or two input images and creates a single output image. 
		The functions are ARCSIN, ARCCOS, and ARCTAN.

PROGRAM HISTORY

VERSION		PROGRAMMER	DATE	CODE/CONT	REASON
-------		----------	----	---------	------
 1.0		B. Dealy	9/84    SAR		original development
 2.0            B. Kovalick     5/86    SAR             added timed message
 3.0		M. Mack		5/89	GSFC 636	Conversion to 5.0
						        and combination of
							three related fns.
							(ARCSIN, ARCCOS, AND
  							ARCTAN) into one, TRIG.
 4.0            M. Mack         9/90    GSFC            Method of combining
							input host file names
							into one file was
							changed.
 5.0		D. Etrheim	2/91	EDC		Allocate space dynamicly
							and make vars. static.
							General cleanup.
 5.1		D. Etrheim	9/92	EDC		Modifed global include 
							file

FUNCTIONS CALLED:

 - WRITTEN FOR THIS FUNCTION:
   getpar      -    This routine elicits the parameters supplied by the user
                    through the TAE invocation of the function.
   calctrg     -    This routine does the actual calculation of the trig
                    function for each pixel of the image(s).

 - LAS LIBRARY FUNCTIONS:
   c_complt     -
   c_errmsg     -
   c_hstout     -
   c_inlas      -

 - STANDARD C FUNCTIONS:
   strcpy       -

ALGORITHM:

This function initially gathers the parameters supplied by the user in
his LAS/TAE invocation of TRIG. Because some routines require input host
names in one array, they are moved to one array.
It then calls CALCTRG to do the actual file opening, calculations, and
file closing.
Finally a completion message is sent to the user.

**************************************************************************/

int     fdesc1[MAXIMG + 1];            /* file descriptors input 1*/
int     fdesc2[MAXIMG + 1];            /* file descriptors input 2*/
int     fdesc[2*MAXIMG + 1];           /* file descriptors-all    */

int main
(void) /* trig */

{

float   atzero;              /* value to be used when numerator and
                                denominator are both zero */
long    func;                /* indicates which function is being accessed:
                                1=arcsin; 2=arccos; 3=arctan */
long    k;                   /* Used as index for several loops  */
static char las_names[MAXIMG][CMLEN];
			     /* LAS [TAE] names of Input image(s)*/
long    nmrimg1;	     /* number of input images in first 
                                input "image" */
long    nmrimg2;	     /* number of input images in second 
                                input "image" (if one is used)*/
long    noimin;              /* indicates if there are one (noimin=1)
                                or two (noimin=2)
                                input images available  */
long    non_fatal=1;         /* Non-fatal error indicator */
long	one  = 1;       /* integer value for 1 - in complt,
				specifies that only one output 
				file name is supplied  */
long    totbnd1;             /* Total number of bands in 1st input */
long    totbnd2;             /* Total number of bands in 2nd input */
long    totbndo;             /* Total number of bands in output */
long    totwin[MAXIMG+1][4]; /* total number of windows  */
long    std_out;             /* logical unit # for standard output */
long	window[MAXIMG][4];   /* window for first image   */
long    window2[MAXIMG][4];  /* window for second image  */
long    fatal_err = -1;                /* Used in errmsg calls    */

/*
  file info
*/

long bands1[MAXIMG][MAXBND + 1];
			     /* Band specification for first 
                                     input image*/
static long bands2[MAXIMG][MAXBND + 1];
			     /* Band specification for second 
                                     input image*/
static char hosname[MAXIMG][CMLEN]; 
                             /* host name of first input file  */
static char hosname2[MAXIMG][CMLEN];
                             /* host name of second input file,
                                     (if any)  */ 
static char hosin[MAXIMG][CMLEN];	
                             /* host names of both (or one) image
			             input (depending on noimin)   */
char        mnmx[8];         /* Shows desire for min-max values 
			             for input file(s)and 
                                     output file */
long        *nibs1;
       			     /* number of bands/image in input 1 */
long        *nibs2;
       			     /* number of bands/image in input 2 */
long        *nibso;
       			     /* number of bands/image in output  */
char        outfile[1][CMLEN]; 
                             /* LAS name of output file   */
char        outhost[1][CMLEN];  
                             /* host name of output file  */

/*
////////////////////////////////////////////////////////////////////
       Start of executable code for main program Trig             
////////////////////////////////////////////////////////////////////

---------------------------------------------------------------
      Ininitialize LAS and TAE
---------------------------------------------------------------
*/

c_inlas(&vblock,&std_out);

/* Allocate space */
nibs1 = (long *) malloc(MAXBND * sizeof(long));
if (nibs1 == NULL)
   {
   c_errmsg("Error allocating dynamic memory","trig-alloc",
         &non_fatal);
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   }
nibs2 = (long *) malloc(MAXBND * sizeof(long));
if (nibs2 == NULL)
   {
   c_errmsg("Error allocating dynamic memory","trig-alloc",
         &non_fatal);
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   }
nibso = (long *) malloc(MAXBND * sizeof(long));
if (nibso == NULL)
   {
   c_errmsg("Error allocating dynamic memory","trig-alloc",
         &non_fatal);
   c_errmsg("Fatal error encountered","trig-fatal",&fatal_err);
   }

/*
--------------------------------------------------------------
      Get TAE input parameters   
--------------------------------------------------------------
*/

getpar (&func, hosname, hosname2, las_names, outfile, 
	       outhost, window, window2, nibs1, nibs2, nibso, bands1, 
               bands2, &totbnd1, &totbnd2, &nmrimg1, &nmrimg2, 
               mnmx, &atzero, &noimin);

/*

--------------------------------------------------------------
      Copy host file names for later use in upddr in calctrg
--------------------------------------------------------------
*/

for (k=0; k< nmrimg1; k++)
  {
  strcpy(hosin[k], hosname[k]);
  }
if(noimin > 1)
  {
  for (k=0; k< nmrimg2; k++)
    {
    strcpy(hosin[k+nmrimg1], hosname2[k]);
    }
  }

/*
-----------------------------------------------------------------
      Set up to report processing status, and do the job
-----------------------------------------------------------------
*/
calctrg (&func,  hosname, hosname2, hosin, 
	     outhost, window, window2, totwin, nibs1, 
	     nibs2, nibso, bands1, bands2, &totbnd1,
	     &totbndo, &nmrimg1, &nmrimg2, 
             mnmx, &atzero, &noimin);

/*
----------------------------------------------------------------
       That's it for this image. Update history.
----------------------------------------------------------------
*/

if(c_hstout (&vblock, outhost[0], hosin, &noimin, " ") != E_SUCC)
           c_errmsg("Nonfatal error encountered","trig-warn",&non_fatal); 
/*
---------------------------------------------------------------
        Send completion message to user
---------------------------------------------------------------
*/

if (func == ARCSIN)
 {
    c_complt ("TRIG-ARCSIN", outfile, &one); 
 }                 
else if (func == ARCCOS)
 {
    c_complt ("TRIG-ARCCOS", outfile, &one); 
 } 
else if (func == ARCTAN)
 {
    c_complt ("TRIG-ARCTAN", outfile, &one); 
 }

return EXIT_SUCCESS;

} /* End of Trig.c     */
