
#include "unary.h"    
#include "pixman.h"
#include "util.h"
#include "protos.h"

void runopt
(
    char fopt[],         /* math function option */
    long wind[][4],	           /* input image window   */
    float base,           /* log/exp base */
    float exsc,           /* scale for exp input images */
    float scoff[],	/* scale and offset */
    long scflg,                     /* scale/no scale flag */
    float pscale,         /* scale for sine and cosine  */
    float phase,          /* offset for sine and cosine */
    long clipped[],                    /* inclip message flag  */
    long num_inbands,    /* total number of input bands to be processed   */
    long *num_outbands   /* total number of output bands to be processed  */
)
                
/*
********************************************************************

NAME: RUNOPT
              
PURPOSE: runopt is called to perform the user-specified opereation
         on one input image 

PROGRAM HISTORY:
  Version   Date       Author                       Request
  -------   ----       ------                       -------
                          
  1.0     05-Mar-84    S. Ross/SAR            scn#20-188
  4.0     26-Mar-86    K. Gacke                 lasd
  5.0     01-Feb-90    W. Xia, STX              conversion to 5.0
  6.0     09-May-90    F. Irani, STX            Add NOT subcmd
  7.0     10-Oct-90    W. Xia,  STX             fix bugs
               
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

Call c_inittm to initialize and enable the processing message utility
To allocate dynamic array 
To perform the arithmetic operations 
Free dynamic array
Call c_stoptm to disable the display message utility
Call c_egclse to close image files 
 
*************************************************************************
*/

{ 
/*
Local variables:
----------------						*/
long   ibnd,                    /* current band index           */
       line,                    /* process line loop index      */
       rtype = EREAL,		/* data type for real data px calls	*/
       stat,
       px;                      /* loop index */      

long    inf_err = 1;                   /* info error return status  */
long    fatal_err = -1;                /* fatal error return status */
long    zero = 0;

float	div,		   	/* intermediate value         */
	*tbuf = NULL,     	/* temporary image i/o buffer */
	tval;               	/* intermediate value         */

char   
  	outbuf[ERRLEN];      	/* message buffer             */


/*                                                         
Initialize and enable the processing message utility        
--------------------------------------
*/                                             
c_inittm ("UNARY", "Processed", "line", "bands", " ", 
&wind[0][NL], num_outbands, &msgtim, &zero);            
curent = 1;    

/*
Handle allocation for NOT subcommand as a special case.
-------------------------------------------------------			*/
if (strcmp(fopt, "NOT") != 0)
{
	if (( tbuf = (float *) malloc(wind[0][NS] * sizeof (float))) == NULL)
		c_errmsg("Error allocating dynamic memory", 
                         "unary-alloc", &fatal_err);
}

/*
Start timed processing message
------------------------------		*/
c_startm();    

for (line = 0; line< wind[0][NL]; line++, curent++)
   {                                       /* start of line loop */
   stat = c_estep (&gdesc);
  
   if (stat != E_SUCC )
      {
      c_errmsg ("Fatal error encountered", "unary-fatal",&fatal_err);
      }
 
   for (ibnd = 0; ibnd< num_inbands; ibnd++)
      {                                       /* start of band loop */

      /*
      Loop to process sine option
      ---------------------------					*/
      if (strcmp(fopt,"SIN") == 0)
        {
        for (px = 0; px < wind[0][NS]; px++)
          {
          tbuf[px] = sin(pscale*iobuf[ipoint[ibnd]+ px] +phase);
          }
        } 
      /*
      Loop to process cosine option
      -----------------------------					*/
      else if (strcmp(fopt,"COS") == 0)
         {
         for(px = 0; px < wind[0][NS]; px++)
            {
            tbuf[px] = cos(pscale * iobuf[ipoint[ibnd] + px] + phase);
            }
         }
      /*
      Loop to process NOT (one's complement) option.
      ----------------------------------------------		*/
      else if (strcmp(fopt, "NOT") == 0)
      {
	switch (out_type)
	{
		case EBYTE:
			stat = 	c_pxcomp( &biobuf[ipoint[ibnd]],
				          &biobuf[opoint[ibnd]],
				  	  &wind[0][NS], &out_type);
			break;

		case EWORD:
			stat = 	c_pxcomp( &siobuf[ipoint[ibnd]], 
					  &siobuf[opoint[ibnd]], 
				  	  &wind[0][NS], &out_type);
			break;

		case ELONG:
			stat = 	c_pxcomp( &liobuf[ipoint[ibnd]], 
					  &liobuf[opoint[ibnd]], 
				  	  &wind[0][NS], &out_type);
			break;

		case EREAL:
			stat = 	c_pxcomp( &riobuf[ipoint[ibnd]], 
					  &riobuf[opoint[ibnd]], 
				  	  &wind[0][NS], &out_type);
			break;

	}	/* Switch  */
   
	/*
	Check c_pxcomp status.
	----------------------					*/
	if ( stat != E_SUCC )
		c_errmsg ("Error finding one's complement",
			  "Unary-estep",&fatal_err);
      }

      /*
      Loop to process exponentiation option
      -------------------------------------			*/
      else if (strcmp(fopt,"EXP") == 0)
      {
	for(px = 0; px < wind[0][NS]; px++)
	{             
		tval = exsc * iobuf[ipoint[ibnd]+ px] * log(base);  
		if (tval < -88.74 || tval > 88.03)
		 {  
		 if (!clipped[ibnd])
		   {
		   sprintf(outbuf,"\nSome pixels for band %3d",ibnd+1);
		   c_errmsg(outbuf,"unary-runopt",&inf_err);
		   c_errmsg(
		   "have not been exponentiated to prevent overflow.",
		   "unary-runopt",&inf_err);     
		   clipped[ibnd] = TRUE;
	 	   }
		   tbuf[px] = exsc * iobuf[ipoint[ibnd]+ px];	
                 }             
                else
                 {  
		 tbuf[px] = (float) pow(base,(exsc*iobuf[ipoint[ibnd]+ px]));
                 }         
        }	/* for */

      }	 /* if EXP */

      /*
      Call to process scale option
      ----------------------------					*/
      else if (strcmp(fopt,"SCA") == 0)
	c_pxline (&iobuf[ipoint[ibnd]], &iobuf[opoint[ibnd]], &wind[0][NS], 
		  &rtype, &scoff[0], &scoff[1]);    
      else
        {
  	/*
	Calls to process logarithm option
	--------------------------------- */
	for (px = 0; px < wind[0][NS]; px++)
		tbuf[px] = log10(iobuf[ipoint[ibnd]+ px]);

	if (base != 10.0)
	   {
		/*
		Set divisor for log computation.
		--------------------------------		*/         
		div = 1./log10(base);    
		c_pxline (tbuf, tbuf, &wind[0][NS], &rtype, &div,
			  &zero);
           }
        }

      /*
      Post option scale and offset
      ----------------------------				*/
      if (scflg)             
        {
        if (strcmp(fopt,"SCA") != 0)
           {
           c_pxline (tbuf, &iobuf[opoint[ibnd]], &wind[0][NS], 
	    	     &rtype, &scoff[0], &scoff[1]);
           }
        }   
      if (strcmp(fopt, "NOT") != 0 && strcmp(fopt,"SCA") !=0) 
        {
        c_pxcopy (tbuf, &iobuf[opoint[ibnd]], &rtype, &wind[0][NS]);
        }
      }                                /* end of band loop */
   }                                  /* end of each line */

stat = c_estep (&gdesc);

if (stat != E_SUCC )
	c_errmsg ("Fatal error encountered", "unary-fatal",&fatal_err);

if (strcmp(fopt, "NOT") == 0)
   {    
	switch (out_type)
	{
		case EBYTE:
			free((unsigned char *) biobuf);
			break;
		case EWORD:
			free((short *) siobuf);
			break;
		case ELONG:
			free((long *) liobuf);
			break;
		case EREAL:
		        free((float *) riobuf);
			break;
	}	/* Switch  */
   }     
else 
   {
	free((float *) iobuf);
	free(tbuf);
   }
/*                                                 
Disable the display message utility
-----------------------------------			*/    
c_stoptm(); 

/*
Close image files 
-----------------					*/
c_egclse( &gdesc);

}
