/****************************************************************************

NAME:				VERIFY

PURPOSE:   Computes the error in a verification point due to relief 
	   displacement.  Calculates means, std. deviations and RMSE of 
	   verification points.

	   Uses polynomial fit applied to image for registration to map 
	   verification point locations back to input image space.  With 
	   input image space location, the amount of relief displacement in 
	   pixels is computed and the input location corrected and mapped 
	   back to output space.  A difference between the original and 
	   recomputed output space represents the amount of relief 
	   displacement in the imagery.


PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	         CODE     REASON
-------	 ----	------	         ----     -----------------------------
  1.0     3/88  J. Thormodsgard  S&AB     Implement on UNIX/TAE (FORTRAN)
                 & G. Kelly
  5.0	 11/89	D. Steinwand     CSB      LAS 5.0/IMAP C conversion
  5.1	  2/90	D. Steinwand     CSB      Changed plot file slightly
  5.2	  4/90	D. Steinwand     S&AB     Mod to use only active points
  5.3	  5/93	K. Sheffield     CSB      Mod to edit points based on 
                                          residual errors and reformat
                                          the plot file.
  5.4     1/94  D. Etrheim       CSB      Modify to keep OUTTL if no 
                                          points are deleted. (PRF# 6444) 
  5.5     1/95  T. Mittan        CSB      Modified to handle any number of
                                          tie points.
  5.6     6/00  A. Shimitz       CSB      Fix to properly handle any 
                                          number of tie points

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initalize LAS processing
	Get the subcommand used.  If only residuals are to be calculated,
	   set SIMPLE TRUE.  Otherwise SIMPLE is FALSE, and relief corrections
	   are applied to the verification points
	Get the name of the tie point location file
        If applying relief corrections:
	   Get the name of the geometric mapping grid which has inverse coefs
	   Get the name of the geometric mapping grid which has forward coefs
	   Get the TM quad number
	   Get the satellite height in meters
	   Get the sensor orientation angle
	   Get the sensor field of view
	   Get the scanline pitch
	   Get the reference datum
	Get the pixel size in input and output space
	Set up the PRINT file options
	Report parameters to the user via the print file
	Write report header to print file
	Open the verification point (TPL) file
	If appling relief corrections:
	   Get the inverse polynomial coefficients
	   Get the forward polynomial coefficients
	   Set up misc constants
	Loop until EOF in verification file:
	   Read a verification record
	   Calculate the residual
	   Calculate the mean error along lines
	   Calculate the mean error along samples
	   Calculate the RMS error
	   If appling relief corrections:
		Map the verification point back to input space
	 	Compute the amount of relief displacement
		Correct the location for relief displacement
		Adjust scanline, if necessary
	 	Map the corrected location back to output space
		Subtract the calc location from the observed location
		Compute the amount of error due to relief in output space
                If editing is desired:
                   Write out the verification record if the error is less than
                   or equal to the maximum desired error
		Report to user
	   Else 
                If editing is desired:
                   Write out the verification record if the error is less than
                   or equal to the maximum desired error
                Report residuals to the user
	Calculate residuals
	Report to user
	Close the verification file(s)
	Issue the standard completion message and exit
	
ALGORITHM REFERENCES:
	The relief displacement algorithm was based on prior work done
	at the EROS Data Center by J. Thormodsgard, G. Kelly, and L. Nady, 
        and refined with "Evaluation of EDC's Algorithm for Correction of 
        Control Points for Terrian Parallax", by Robert Wolfe, ST Systems 
        Corp (STX), Lanham, Maryland; done under USGS purchase order 
        88-211494., 10/88
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"

#define R	6378206.4		/* Earth radius */

#define KEEP_PT 0
#define CUT_PT 1

char outbuf[1024];
char *obptr;
static char * printbuf
(
    char *buf,
    FILE *p1,
    FILE *p2
);

typedef struct                         /* structure for output plot data */
{
   char pt_id[20];
   double line, samp;
   double rl, rs;
} PLOTPT;


main()		
{
   struct PARBLK vblk;		/* TAE parameter block */
   struct VARIABLE *var;	/* Pointer to a TAE variable in vblk */
   struct TPLHEAD header;	/* Tie point location file header data */
   struct TPLDATA tpldata;	/* Tie point location file data record */
   struct GEOGRID grid;		/* Geometric mapping grid data records */
 
   PLOTPT *pltbuf;              /* Corrplot data records */
   PLOTPT *pltptr;              /* Pointer to data records */
   
   
   double sathgt;		/* Satellite height in meters */
   double h;		        /* Earth radius plus satellite height */
   double theta;		/* Central earth angle between beginning of 
				   scanline and satellite nadir */
   double look;		        /* Look-angle of detector where point was 
				   imaged */
   double cons;		        /* fov/2 in radians */
   double pitch;		/* Angle of pitch of sensor scanline from nadir,
				   = 0.53 for SPOT P 
				   = -0.53 for SPOT XS, 
				   = 0. for Landsat  */
   double psin;		        /* Pixel size in input space */
   double psout;		/* Pixel size in output space */
   double datum;		/* Datum in meters */
   double relief;		/* Amount of relief displacement in meters */
   double d;		        /* Distance on earth's surface; nadir to 
				   scanline */
   double dp;		        /* Dist on earth's surface; satellite nadir to 
				   point */
   double l;		        /* Length of line of sight-distance of 
				   beginning of scanline to satellite */
   double lp;		        /* Length of line of sight-distance of control 
				   point from satellite--does not take into 
				   account the elevation of the point */
   double lp2;		        /* Length of line of sight-distance of control 
				   point at its elevation from satellite */
   double thetap;		/* Central earth angle between control point 
				   and satellite nadir--does not take into 
				   account the elevation of the point */
   double rel;		        /* Earth radius plus elevation of control 
				   point */
   double incd;		        /* Incidence angle of HRV sensor in degrees 
				   (found on SPOT summary sheets where L makes 
				   INCD positive and R makes INCD negative)   
				   note: INCD-2.065 is the look-angle of the 
				   first CCD detector */
   double fov;		        /* Angle between first and last detector or 
				   sample of scanline */
   double thetap2;		/* Central earth angle between control point 
				   and satellite nadir--takes elevation of 
				   control point into account */
   double tdisp;		/* Pythagorean distance error in 
				   verification point */
   double rmssq;		/* RMSE of the TDISP */
   double rmsem;		/* RMSE in meters in output space */
   double ale;		        /* Mean error of the verification points in 
				   line */
   double ase;		        /* Mean error of the verification points in 
				   sample */
   double rmsel;		/* RMSE of line errors in the verification 
				   points */
   double rmses;		/* RMSE of sample errors in the verification 
				   points */
   double inv_a[MAX_COEF];	/* Inverse polynomial coefficients mapping X */
   double inv_b[MAX_COEF];	/* Inverse polynomial coefficients mapping Y */
   double for_a[MAX_COEF];	/* Forward polynomial coefficients mapping X */
   double for_b[MAX_COEF];	/* Forward polynomial coefficients mapping y */
   double *srel;		/* Error due to relief displacement in output */
   double *lrel;		/* Error due to relief displacement in output */
   double *dline;		/* Error in verification point in line dir */
   double *dsamp;		/* Error in verification point in sample dir */
   double tmp,tmp1,tmp2;	/* Temporary variables for computations */
   double xsamp, xline;	        /* Verification points mapped back to input 
				   space */
   double oline, osamp;	        /* Relief corrected verify points in output 
				   space */
   
   double cp_rmssq;	        /* RMSE of the TDISP for cut points */
   double cp_ale;		/* Mean error of the verification points in 
				   line for cut points */
   double cp_ase;		/* Mean error of the verification points in 
				   sample for cut points */
   double cp_rmsel;	        /* RMSE of line errors in the verification 
				   points for cut points */
   double cp_rmses;	        /* RMSE of sample errors in the verification 
				   points */
   
   double tmp_rmssq;	        /* temporary value holder rmssq/cp_rmssq */
   double tmp_rmsem;	        /* temporary value holder rmsem/cp_rmsem */
   double tmp_ale;		/* temporary value holder ale/cp_ale */
   double tmp_ase;		/* temporary value holder ase/cp_ase */
   double tmp_rmsel;	        /* temporary value holder rmsel/cp_rmsel */
   double tmp_rmses;	        /* temporary value holder rmses/cp_rmses */
   
   double maxres;               /* maximum allowable residual */
   
   struct PTDESC *in_tpl;  /* Input TPL file descriptor */
   struct PTDESC *out_tpl; /* Output TPL file descriptor */
   
   long npts;   		/* Total number of verification points */
   long cp_npts;   		/* Total number of verification points cut */
   long k_npts;   		/* Total number of verification points kept */
   long inv_ord;		/* Order of inverse polynomial */
   long for_ord;		/* Order of forward polynomial */
   long term, file, ln_pr;	/* PRINT flags */
   long quad;		        /* TM quadrant 1-4 */
   long ptyp = TPL;	        /* Type of tie point file */
   long acc;		        /* Access flag */
   long std_out;		/* Standard output LUN */
   long status;		        /* Function return status */
   long fatal = -1;	        /* Fatal flag for errmsg */
   long zero = 0;		/* Constant of zero */
   long one = 1;		/* Constant of one */
   long two = 2;		/* Constant of two */
   long num_out;		/* Number of output files */
   long simple;		        /* Boolean--perform a "simple" verification or 
				   not */
   long cutflag;		/* Boolean--cut out bad points or not */
   long i,j;		        /* Counters */
   long size;		        /* Size of buffer to allocate */
   long msgtim;		        /* Message timer */
   long plotflg = FALSE;	/* Flag if plot file is to be created */ 
   long *cutflgs;               /* buffer determining whether or not to cut a 
				   point */
   long maxtie;			/* Maximum tie points		*/
   long pmaxtie;		/* maximum tie points allocated for pltfile */
   
   
   char in[CMLEN];		/* The input Tie Point Location file name */
   char out[CMLEN];	        /* The output Tie Point Location file name */
   char in2[CMLEN];	        /* The input inverse coefficient file name */
   char in3[CMLEN];	        /* The input forward coefficient file name */
   char out_names[4][CMLEN];    /* TAE names of files */
   char name[CMLEN];	        /* Name for PRINT parameter */
   char filename[CMLEN];	/* Name for PRINT parameter */
   char host_name[CMLEN];	/* Name for PRINT parameter */
   char plotnm[CMLEN];	        /* Name for PRINT parameter */
   char cutfile[CMLEN];	        /* Name for OUTFILE parameter */
   
   FILE *fptr = NULL;		/* Pointer to output file (report file) */
   FILE *pptr = NULL;		/* Pointer to plot file */
   FILE *eptr = NULL;		/* Pointer to error output file */
   
   extern long curent;
   extern long curcat;
   
   /* Initialize LAS processing
      -------------------------*/
   c_inlas(&vblk, &std_out);
   
   /* Get the message timer & set up processing message
      -------------------------------------------------*/
   var = p_find(&vblk, "$msgtim");
   msgtim = IVAL(*var, 0);
   
   c_inittm("VERIFY","Processed","active point number","","",&two,&two,
	    &msgtim,&two);

   strcpy(plotnm,"");
   strcpy(cutfile,"");
   strcpy(out,"");
   

/* Extract parameters from TAE 
   -------------------------------------------------------------------------*/
   
   /* Get the subcommand used
      -----------------------*/
   var = p_find(&vblk, "_SUBCMD");
   if (strcmp(SVAL(*var,0),"NOREL") == 0) simple = TRUE;
   else simple = FALSE;
   
   /* Get the tie point location file name
      ------------------------------------*/
   acc = IREAD;
   if (c_getnam(&vblk, "INTL", &zero, in, "tpl", &acc) != E_SUCC)
     c_errmsg("Fatal error encountered","verify-fatal",&fatal);
   
   /* Get the plot file name
      ------------------------------------*/
   acc = IWRITE;
   if (c_getnam(&vblk, "OUTPLOT", &zero, plotnm, "plt", &acc) != E_SUCC)
     c_errmsg("Fatal error encountered","verify-fatal",&fatal);
   
   /*  Open the verification (TPL) file
       ---------------------------------*/
   acc = IREAD;
   if(c_ptopen(&in_tpl, in, &ptyp, &acc, &header) != E_SUCC)
     c_errmsg("Fatal error encountered","verify-fatal",&fatal);
   
   if (!simple)
   {
      
      /* Get the geometric mapping grid containing the inverse coefficients
	 ------------------------------------------------------------------*/
      acc = IREAD;
      if (c_getnam(&vblk, "INGRID", &zero, in2, "grid", &acc) != E_SUCC)
	     c_errmsg("Fatal error encountered","verify-fatal",&fatal);
      
      /* Get the geometric mapping grid containing the forward coefficients
	 ------------------------------------------------------------------*/
      acc = IREAD;
      if (c_getnam(&vblk, "INGRID", &one, in3, "grid", &acc) != E_SUCC)
	     c_errmsg("Fatal error encountered","verify-fatal",&fatal);
      
      /* Get other parameters
	 --------------------*/
      var = p_find(&vblk, "QUADNUM");
      quad = IVAL(*var,0);
      var = p_find(&vblk, "SATHT");
      sathgt = RVAL(*var,0);
      var = p_find(&vblk, "INCANG");
      incd = RVAL(*var,0);
      var = p_find(&vblk, "FOVIEW");
      fov = RVAL(*var,0);
      var = p_find(&vblk, "PITCH");
      pitch = RVAL(*var,0);
      var = p_find(&vblk, "DATUM");
      datum = RVAL(*var,0);
      var = p_find(&vblk, "DATUNIT");
      if (strcmp(SVAL(*var,0),"FEET") == 0) datum *= 0.3048;
      var = p_find(&vblk, "PIXSIZ");
      psin = RVAL(*var,0);
      if ((*var).v_count == 1) 
      {
         psout = header.pdist[0];
         if (header.proj_valid == GEO_NOTVALID)
         {
            c_errmsg("Output pixel size must be specified","verify-pixsiz",
                      &fatal);
         }
      }
      else psout = RVAL(*var,1);
   }
   else
   {
      var = p_find(&vblk, "PIXSIZ");
      if ((*var).v_count == 0)
      {
         psout = header.pdist[0];
         if (header.proj_valid == GEO_NOTVALID)
         {
            c_errmsg("Output pixel size must be specified","verify-pixsiz",
                     &fatal);
         }
      }
      else psout = RVAL(*var,0);
   }
   if (strcmp(header.units, "") == 0) strcpy(header.units,"meters");
   
   /* Get the print option.  Make a file name (if neccessary) and open the
      temporary file
      --------------*/
   strcpy(host_name, "verify");
   status = c_setprt(&vblk, &term, &file, &ln_pr, host_name, filename);
   if (status != E_SUCC)
     c_errmsg("Fatal error encountered","verify-fatal",&fatal);
   
   acc = IUPDATE;
   num_out = 0;
   if (file == 1) 
   {
      c_mknam(filename, name, "prt", &acc);
      num_out = 1;
   }
   else strcpy(name, " ");
   fptr = fopen(host_name, "w");

   var = p_find(&vblk, "MAXRES");
   if (var->v_count == 1) 
   {
      maxres = RVAL(*var,0);
      cutflag = TRUE;
      /* Get the tie point location file name
	 ------------------------------------*/
      acc = IWRITE;
      if (c_getnam(&vblk, "OUTTL", &zero, out, "tpl", &acc) != E_SUCC)
         c_errmsg("Fatal error encountered","verify-fatal",&fatal);

      if(strlen(out) == 0)
      {
         c_errmsg("OUTTL must be specified","verify-getnam",&fatal);
      }
      
      if (c_getnam(&vblk, "OUTFILE", &zero, cutfile, "prt", &acc) != E_SUCC)
         c_errmsg("Fatal error encountered","verify-fatal",&fatal);

      if(strcmp(name,cutfile) == 0)
      {
         c_errmsg("OUTFILE and PRINT cannot be the same","verify-chknam",
                  &fatal);
      }
      
   
      /*  Open the cut file verification (TPL) file
	  ---------------------------------*/
      acc = IWRITE;
      if(c_ptopen(&out_tpl, out, &ptyp, &acc, &header) != E_SUCC)
         c_errmsg("Fatal error encountered","verify-fatal",&fatal);
   }
   
   else cutflag = FALSE;
   
/* --------------------------------------------------------------------------*/


   /* Allocate buffers
      ----------------*/
   maxtie = MAX_TIE_POINTS;
   size = MAX_TIE_POINTS * sizeof(double);
   srel = (double *)malloc(size);
   lrel = (double *)malloc(size);
   dline = (double *)malloc(size);
   dsamp = (double *)malloc(size);

   /* allocate cut flags buffer and initialize to KEEP */

   cutflgs = (long *)calloc(maxtie , sizeof(long));

   if((srel == NULL) || (lrel == NULL) || (dline == NULL) || (dsamp == NULL) ||
      (cutflgs == NULL) )
   {
      c_errmsg("Error allocating dynamic memory","verify-alloc",&fatal);
   }
   
   /* Open the plot file
      ------------------*/
   if (strlen(plotnm) != 0)
   {
      pmaxtie = MAX_TIE_POINTS;
      pptr = fopen(plotnm, "w");
      plotflg = TRUE;
      pltbuf = (PLOTPT *) malloc(pmaxtie*sizeof(PLOTPT));
      if((pltbuf == NULL) )
      {
         c_errmsg("Error allocating dynamic memory","verify-alloc",&fatal);
      }
      pltptr = pltbuf;
   }

   if(strlen(cutfile) != 0)
     eptr = fopen(cutfile,"w");
   else
     eptr = NULL;
   
   
   /* OPEN the input verification file and input coefficient files.  The input 
      coefficient files contain the coefficients of the first or second order 
      fit used to rectify the image which has been verified.  This module will 
      use the coefficients to map the verification points back to input space, 
      and then calculate the amount of relief displacement.  The input location 
      of the verification point is then corrected for relief displacement and 
      then, using the forward coefficients, mapped back to output space.  The 
      output location is subtracted from the original verification point to 
      estimate the amount of error in the point which is due to relief.
      -----------------------------------------------------------------*/

   obptr = outbuf;

   if (!simple)
   {
      sprintf(obptr,
          "Relief displacement correction applied to verification points.\n");
      obptr += strlen(obptr);
      
      sprintf(obptr, "Relief displacement remains in the output image.\n\n");
      obptr += strlen(obptr);
   }
   sprintf(obptr,"Input verification file:        %s\n", in);

   obptr = printbuf(outbuf,fptr,eptr);
   
   if (!simple)
   {
      sprintf(obptr,"Input inverse coefficient file: %s\n", in2);
      obptr += strlen(obptr);
      
      sprintf(obptr,"Input forward coefficient file: %s\n\n", in3);
      obptr += strlen(obptr);
      sprintf(obptr,"TM code (3 if not tm sensor or full quad):      %d\n",
	      quad);
      obptr += strlen(obptr);
      sprintf(obptr,"Satellite height in meters:             %12.2f\n",sathgt);
      obptr += strlen(obptr);
      sprintf(obptr,"Sensor orientation angle in degrees:        %9.3f\n",incd);
      obptr += strlen(obptr);
      sprintf(obptr,"Sensor field of view in degrees:            %9.3f\n",fov);
      obptr += strlen(obptr);
      sprintf(obptr,"Scanline pitch in degrees:                  %9.3f\n",
	      pitch);
      obptr += strlen(obptr);
      sprintf(obptr,"Datum in meters:                           %9.2f\n",datum);
      obptr += strlen(obptr);
      obptr = printbuf(outbuf,fptr,eptr);

   }
   if (!simple) 
   {
      sprintf(obptr, "Input image pixel size in meters:          %9.2f" ,psin);
      obptr += strlen(obptr);
   }
   
   sprintf(obptr,"\nOutput image pixel size in %s:         %9.2f\n\n", 
	   header.units, psout);
   obptr += strlen(obptr);

   obptr = printbuf(outbuf,fptr,eptr);
   
   /* Write table header to printer file
      ----------------------------------*/
   if (simple)
   {
      sprintf(obptr, "         Reference Image   Registered Image           Errors (%s)\n", header.units);
      obptr += strlen(obptr);
      sprintf(obptr, "Pt Id     Line   Sample     Line   Sample       Line      Sample     Total\n");
      obptr += strlen(obptr);
      sprintf(obptr, "------   ---------------   ---------------     -----------------------------\n");
      obptr += strlen(obptr);
   }
   else
   {
      sprintf(obptr, "                                 Map-derrived     Registered Image     Errors (%s)             Relief Disp Removed\n", header.units);
      obptr += strlen(obptr);
      sprintf(obptr, "Pt Id    Latitude  Longitude    Line    Sample     Line   Sample    Line  Sample   Total    Elev       Line   Sample\n");
      obptr += strlen(obptr);
      sprintf(obptr, "------  --------------------   ----------------   --------------   ----------------------  -------   -----------------\n");
   }
   obptr = printbuf(outbuf,fptr,eptr);
   
   /* Open the geometric mapping grid containing the inverse polynomial 
      coefficients and read them
      --------------------------*/
   if (!simple)
   {
      if (c_getgrd(in2, &grid) != E_SUCC)
         c_errmsg("Fatal error encountered","verify-fatal",&fatal);
      if (grid.ncoeff == 3) inv_ord = 1;
      if (grid.ncoeff == 6) inv_ord = 2;
      if (grid.ncoeff == 10) inv_ord = 3;
      if (grid.ncoeff == 15) inv_ord = 4;
      for (i = 0; i < grid.ncoeff; i++)
      {
         inv_b[i] = grid.coeffs[i];
         inv_a[i] = grid.coeffs[i + grid.ncoeff];
      }
      
      /* Open the geometric mapping grid containing the forward polynomial 
	 coefficients and read them
	 --------------------------*/
      if (c_getgrd(in3, &grid) != E_SUCC)
         c_errmsg("Fatal error encountered","verify-fatal",&fatal);
      if (grid.ncoeff == 3) for_ord = 1;
      if (grid.ncoeff == 6) for_ord = 2;
      if (grid.ncoeff == 10) for_ord = 3;
      if (grid.ncoeff == 15) for_ord = 4;
      for (i = 0; i < grid.ncoeff; i++)
      {
         for_b[i] = grid.coeffs[i];
         for_a[i] = grid.coeffs[i + grid.ncoeff];
      }
      
      /* Half the field-of-view of the sensor and convert all angles to radians
	----------------------------------------------------------------------*/
      incd = incd * D2R;
      cons = (fov / 2.0) * D2R;
      pitch = pitch * D2R;
      h = R + sathgt;
   }
   
   /* Read off diskfile the verification control point.  The verification files 
      contains the map-selected location and the error vector to the actual 
      location in the registered image.  We convert the location to the actual 
      location in the registered image to compute how much error is due to 
      relief.
      ------------------------------------------------------------------------*/

   tdisp = 0.0;
   ale = 0.0;
   ase = 0.0;
   rmsel = 0.0;
   rmses = 0.0;
   rmssq = 0.0;
   npts = 0;
   j = 0;
   
   cp_ale = 0.0;
   cp_ase = 0.0;
   cp_rmsel = 0.0;
   cp_rmses = 0.0;
   cp_rmssq = 0.0;
   cp_npts = 0;
   k_npts = 0;

   curent = 1;
   c_startm();
   curent = 0;
   for(;;)
   {
      status = c_ptio(&in_tpl, &tpldata);
      if (status == E_EOF) break;
      if (status == E_FAIL) 
         c_errmsg("Fatal error encountered","verify-fatal",&fatal);
      
      obptr = outbuf;
      
      if ((tpldata.active == 0) || (tpldata.active == 1) || 
	  (tpldata.active > 9))
      {
         if (j + 1 == maxtie)
         {

            /* Allocate buffers
            ----------------*/
            maxtie += MAX_TIE_POINTS;
            size = maxtie * (long)sizeof(double);
            srel = (double *)realloc(srel,size);
            lrel = (double *)realloc(lrel,size);
            dline = (double *)realloc(dline,size);
            dsamp = (double *)realloc(dsamp,size);
   
            /* allocate cut flags buffer and initialize to KEEP */
   
            cutflgs = (long *)realloc(cutflgs, maxtie * sizeof(long));
   
            if((srel == NULL) || (lrel == NULL) || (dline == NULL) || 
               (dsamp == NULL) || (cutflgs == NULL) )
            {
               c_errmsg("Error allocating dynamic memory","verify-alloc",
                        &fatal);
            }
            for (i = j; i < maxtie; i++)
               cutflgs[i] = KEEP_PT;

         }

	 curent++;
	 dline[j] = (tpldata.ref_coord[0] - tpldata.sea_coord[0]) * psout;
	 dsamp[j] = (tpldata.ref_coord[1] - tpldata.sea_coord[1]) * psout;
	 tmp1 = dline[j] * dline[j];
	 tmp2 = dsamp[j] * dsamp[j];
	 tdisp = sqrt(tmp1 + tmp2);
	 npts++;
	 
	 tmp_ale = dline[j];
	 tmp_ase = dsamp[j];
	 tmp_rmssq = tmp1 + tmp2;
	 tmp_rmsel = tmp1;
	 tmp_rmses = tmp2;
	 
	 if (!simple)
	 {
	    
	    /* Map each verification point back to input space
	       -----------------------------------------------*/
	    xline = c_eval(&inv_ord, inv_b, &(tpldata.sea_coord[1]), 
			   &(tpldata.sea_coord[0]));
	    xsamp = c_eval(&inv_ord, inv_a, &(tpldata.sea_coord[1]), 
			   &(tpldata.sea_coord[0]));
	    
	    /* Compute the amount of relief displacement at the input location 
	       of the verification point and correct the location for relief 
	       displacement.
	       ---------------------------------------------------------------*/

	    if ((quad == 2) || (quad  == 4)) xsamp = xsamp + 2747.0;
	    rel = R + tpldata.elevation;
	    tmp = (h / R) * sin(incd - cons);
	    tmp *= tmp;
	    l = h * cos(incd - cons) - R * sqrt(1. - tmp);
	    theta = asin((l / R) * sin(incd - cons));
	    d = R * theta;
	    dp = d + (xsamp - 1) * psin;
	    thetap = dp / R;
	    lp = sqrt((R * R) + (h * h) - (2 * R * h * cos(thetap)));
	    look = asin((R / lp) * sin(thetap));
	    tmp =(h / rel) * sin(look);
	    tmp *= tmp;
	    lp2 = h * cos(look) - rel * sqrt(1. - tmp);
	    thetap2 = asin(lp2 / rel) * sin(look);
	    relief = (tpldata.elevation - datum) * tan(look + thetap2);
	    xsamp = xsamp - (relief / psin);
	    relief = tan(pitch) * (tpldata.elevation - datum);
	    
	    /* If TM, the pitch = 0 and therefore there is no change to the 
	       line location.  If SPOT, the panchromatic is imaged 15 km ahead 
	       of the multispectral.  As the scanlines are imaging topographic
	       relief ahead on SPOT pan, i.e., a feature will be imaged in a 
	       later scanline than it should, one must subtract
	       RELIEF from INLINE to correct the SPOT scanline.
	       ------------------------------------------------*/
	    xline = xline - (relief / psin);
	    if ((quad == 2) || (quad  == 4)) xsamp = xsamp - 2747.0;
	    
	    /* Map the corrected verification point which is in input space 
	       back to output space.
	       -----------------------------------------------------------*/
	    oline = c_eval(&for_ord, for_b, &xsamp, &xline);
	    osamp = c_eval(&for_ord, for_a, &xsamp, &xline);
	    
	    /* OLINE,OSAMP at this point is the verification point corrected for
	       relief--the coordinates are in output image space.  Assumptions:
	       a verification point is selected on a map and converted to image 
	       coordinates assuming no relief.  Therefore a comparision between 
	       the selected location and the location corrected for relief is 
	       the amount of relief displacement in output space.
	       -------------------------------------------------------*/
	    srel[j] = (osamp - tpldata.sea_coord[1]) * psout;
	    lrel[j] = (oline - tpldata.sea_coord[0]) * psout;
	    osamp = dsamp[j] - srel[j];
	    oline = dline[j] - lrel[j];
	    
	    sprintf(obptr, "%-6s %10.5f %10.5f %8.1f %8.1f %8.1f %8.1f %7.1f %7.1f %8.1f %8.2f %9.2f %9.2f\n",
		    tpldata.pt_id, tpldata.geo_coord[0], tpldata.geo_coord[1], 
		    tpldata.ref_coord[0], tpldata.ref_coord[1], 
		    tpldata.sea_coord[0], tpldata.sea_coord[1], dline[j], 
		    dsamp[j], tdisp, tpldata.elevation, oline, osamp);

	 }
	 else
	 {
	    sprintf(obptr,"%-6s %8.1f %8.1f %8.1f %8.1f %10.2f %10.2f %10.2f\n",
		    tpldata.pt_id, tpldata.ref_coord[0], tpldata.ref_coord[1],
		    tpldata.sea_coord[0], tpldata.sea_coord[1], dline[j], 
		    dsamp[j], tdisp);
	    osamp = dsamp[j];
	    oline = dline[j];
	 }

	 /* convert the output residuals from meters to pixels */

	 osamp /= psout;
	 oline /= psout;
	 
	 /* if the TPL file is to be edited, calculate the residual magnitude
	    and accumulate sums depending on whether or not the point is cut
	    or not
	    ----------------------------------------------------------------- */
	 if(cutflag)
	 {

	    tdisp = sqrt(osamp * osamp + oline * oline);

	    /* over the limit so mark it as rejected */

	    if(tdisp > maxres)
	    {
	       cutflgs[j] = CUT_PT;
	       obptr = printbuf(outbuf,NULL,eptr);
	       cp_npts++;
	       cp_ale += tmp_ale; 
	       cp_ase += tmp_ase;
	       cp_rmssq += tmp_rmssq;
	       cp_rmsel += tmp_rmsel;
	       cp_rmses += tmp_rmses;
	       /* 5:  Reject--diagonal displacement from nominal location 
		              exceeds the maximum specified by the user
		   */

	       tpldata.active=5;
	       
	    }
	    else
	    {
	       obptr = printbuf(outbuf,fptr,NULL);

	       ale += tmp_ale; 
	       ase += tmp_ase;
	       rmssq += tmp_rmssq;
	       rmsel += tmp_rmsel;
	       rmses += tmp_rmses;
	       k_npts++;
	    }

	    /* write it out to the new TPL file */

	    status = c_ptio(&out_tpl, &tpldata);
	    if (status == E_FAIL) 
	      c_errmsg("Fatal error encountered","verify-fatal",&fatal);
	 }
	 else
	 {
	    obptr = printbuf(outbuf,fptr,NULL);
	    ale += tmp_ale; 
	    ase += tmp_ase;
	    rmssq += tmp_rmssq;
	    rmsel += tmp_rmsel;
	    rmses += tmp_rmses;
	    k_npts++;
	 }
	 
	    
	 /* accumulate points that will be output to the plot file 
	    ------------------------------------------------------ */

	 if (plotflg && cutflgs[j] == KEEP_PT)
	 {
        if (k_npts == pmaxtie)
        {
           pmaxtie += MAX_TIE_POINTS;
           pltbuf = (PLOTPT *) realloc(pltbuf,pmaxtie*sizeof(PLOTPT));
           if(pltbuf == NULL)
              {
                 c_errmsg("Error allocating dynamic memory","verify-alloc",
                          &fatal);
              }

           pltptr = &pltbuf[k_npts - 1];
        }
	    strcpy(pltptr->pt_id,tpldata.pt_id);
	    pltptr->line = tpldata.sea_coord[0];
	    pltptr->samp = tpldata.sea_coord[1];
	    pltptr->rl = oline;
	    pltptr->rs = osamp;
	    pltptr++;
	    
	 }
	 j++;
      }
      else if (cutflag)
      {
	 /* write out the point to the output TPL file even if it is marked as
	    rejected.  Someone might want them.
	    ------------------------------------------------------------------*/
         status = c_ptio(&out_tpl, &tpldata);
         if (status == E_FAIL) 
         c_errmsg("Fatal error encountered","verify-fatal",&fatal);
      }
      
	 
   }

   /* if any points were kept, calculate rms values */

   if(k_npts)
   {
      ale = ale / k_npts;
      ase = ase / k_npts;
      rmssq = sqrt(rmssq / k_npts);
      rmses = sqrt(rmses / k_npts);
      rmsel = sqrt(rmsel / k_npts);
      rmsem = rmssq / psout;
      
      fprintf(fptr, "\nLine error     average = %f %s  RMSE = %f %s\n", ale, 
	      header.units, rmsel, header.units);
      fprintf(fptr, "Sample error   average = %f %s  RMSE = %f %s\n", ase, 
	      header.units, rmses, header.units);
      fprintf(fptr, "RMSE of total error = %f %s\n", rmssq, header.units);
      fprintf(fptr, "RMSE of total error in pixels = %f\n", rmsem);

      tmp_rmsem = rmsem;
   }

   /* generate RMS values for the cut pixels */

   if(cutflag && eptr != NULL && cp_npts)
   {
      ale = cp_ale / cp_npts;
      ase = cp_ase / cp_npts;
      rmssq = sqrt(cp_rmssq / cp_npts);
      rmses = sqrt(cp_rmses / cp_npts);
      rmsel = sqrt(cp_rmsel / cp_npts);
      rmsem = rmssq / psout;
   
      fprintf(eptr, "\nLine error     average = %f %s  RMSE = %f %s\n", ale, 
	      header.units, rmsel, header.units);
      fprintf(eptr, "Sample error   average = %f %s  RMSE = %f %s\n", ase, 
	      header.units, rmses, header.units);
      fprintf(eptr, "RMSE of total error = %f %s\n", rmssq, header.units);
      fprintf(eptr, "RMSE of total error in pixels = %f\n", rmsem);
   }
   
      
   /* calculate the RMS values after relief displacement has been removed */
   
   if (!simple)
   {
      cp_rmssq = cp_rmses = cp_rmsel = 0.0;
      
      for (rmssq=0.0, rmses=0.0, rmsel=0.0, j = 0; j < npts; j++)
      {
         tmp1 = dline[j] - lrel[j];
         tmp1 *= tmp1;
         tmp2 = dsamp[j] - srel[j];
         tmp2 *= tmp2;
         if(cutflgs[j] == KEEP_PT)
         {
            rmssq += tmp1 + tmp2;
            rmses += tmp2;
            rmsel += tmp1;
         }
         else
         {
            cp_rmssq += tmp1 + tmp2;
            cp_rmses += tmp2;
            cp_rmsel += tmp1;
         }
      }

      if(k_npts)
      {
	 
         rmssq = sqrt(rmssq / k_npts);
         rmses = sqrt(rmses / k_npts);
         rmsel = sqrt(rmsel / k_npts);
         rmsem = rmssq / psout;
      
         fprintf(fptr,
                 "\nAfter removal of relief from line and sample error:\n");
         fprintf(fptr,"Line error     RMSE = %f %s\n", rmsel, header.units);
         fprintf(fptr,"Sample error   RMSE = %f %s\n", rmses, header.units);
         fprintf(fptr,"RMSE of total error = %f %s\n", rmssq, header.units);
         fprintf(fptr,"RMSE of total error in pixels = %f\n", rmsem);
         tmp_rmsem = rmsem;
      }
      
      if(cutflag && eptr != NULL && cp_npts )
      {
         rmssq = sqrt(cp_rmssq / cp_npts);
         rmses = sqrt(cp_rmses / cp_npts);
         rmsel = sqrt(cp_rmsel / cp_npts);
         rmsem = rmssq / psout;
      
         fprintf(eptr,
                 "\nAfter removal of relief from line and sample error:\n");
         fprintf(eptr,"Line error     RMSE = %f %s\n", rmsel, header.units);
         fprintf(eptr,"Sample error   RMSE = %f %s\n", rmses, header.units);
         fprintf(eptr,"RMSE of total error = %f %s\n", rmssq, header.units);
         fprintf(eptr,"RMSE of total error in pixels = %f\n", rmsem);
      }
   }
   
   /* generate plot file */

   if (plotflg)
   {
      fprintf(pptr,"%d\n",k_npts);
      for(j=0,pltptr=pltbuf;j<k_npts;j++,pltptr++)
         fprintf(pptr,"%s %f %f %f %f 0\n",pltptr->pt_id,pltptr->samp, 
      pltptr->line, pltptr->rs, pltptr->rl);
      fprintf(pptr,"%f\n",tmp_rmsem);

      fclose(pptr);
   }

   c_ptclse(&in_tpl);
   if(cutflag)
      c_ptclse(&out_tpl);
   c_stoptm();
   fclose(fptr);
   
   if (eptr)
      fclose(eptr);
   
   /* Write the standard completion message and exit
      ----------------------------------------------*/
   if (file == 1)
      strcpy(out_names[0], name);

   /* if any points were kept, add plot file to output list, otherwise, delete
      it 
      ----------------------------------------------------------------------- */

   if (plotflg)
   {
      if(k_npts)
      {
         strcpy(out_names[num_out], plotnm);
         num_out++;
      }
      else
         c_delfil(plotnm);
   }

   if (cutflag)
   {
      if(cp_npts)
      {
         if (eptr)
         {
            strcpy(out_names[num_out], cutfile);
            num_out++;
         }
      }
      else
      {
      /* delete cutfile if no points were cut */
         if (eptr)
            c_delfil(cutfile);
      }

      strcpy(out_names[num_out], out);
      num_out++;
   }

   c_complt("VERIFY", out_names, &num_out);
}

/* printbuf - a routine to output a buffer to two different output streams */
static char * printbuf
(
    char *buf,
    FILE *p1,
    FILE *p2
)
{
   if(p1)
      fputs(buf,p1);
   if(p2)
      fputs(buf,p2);

   return (buf);
}
