/**************************************************************************
NAME:				VERIFY2

PURPOSE:  Calculates and reports residuals and creates plot files for a set 
          of verify pts.

VERSION  DATE	AUTHOR
-------  ----   ------
  1.0    5/94   D. Steinwand/N. Holzapfel	Derived from VERIFY
  2.0	 7/94	T. Mittan			Updated to standards	
  3.0    1/95   T. Mittan        		Modified to handle any number of
                                          	tie points.
  3.1    4/95   D. Etrheim			Corrected problem of pixel size
						being hard-coded to 60m in
						some instances.
  4.0    12/99  D. Hames			Remove MAXNL and MAXNS and 
						replace with a defined number of
						lines and samples. Modified to
						support images > 2 gig.
						Add prototypes. SRF #893
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "verify2.h"
#include "util.h"
#define TERM 0
#define PFILE 1
#define LP 2
#define MAX_LINES 100000	/* MAXNL x 2 */
#define MAX_SAMPS 100000	/* MAXNS x 2 */

void main
(void)
{
struct PARBLK vblk;		/* TAE parameter block 			*/
struct TPLHEAD header;		/* Tie point location file header data 	*/
struct TPLDATA tpldata;		/* Tie point location file data record 	*/
struct POINT *pointref;		/* Reference image point array 	*/
struct POINT *pointref1;	/* Reference image point array 	*/
struct POINT *pointsrch;	/* Search image point array 	*/
struct POINT *pointsrch1;	/* Search image point array 	*/

double psout = 0.0;	/* Pixel size in output space 			*/
double pixacc[7];	/* User input pixel accuracy desired 		*/
double corners[2];	/* holds projection corrdinates			*/
int *in_tpl;		/* TPL file descriptor 				*/
long ptyp = TPL;	/* Type of tie point file 			*/
long acc;		/* Access flag 					*/
long npts;		/* Total number of verification points 		*/
long std_out;		/* Standard output LUN 				*/
long status;		/* Function return status 			*/
long fatal = -1;	/* Fatal flag for errmsg 			*/
long linereg;		/* Number of line regions in image 		*/
long sampreg;		/* Number of sample regions in image 		*/
long lines = 0;		/* Total number of lines in the image 		*/
long maxtie;		/* maximum number of tie points allocated	*/
long samps = 0;		/* Total number of samples in the image 	*/
long sl;		/* Starting line				*/
long ss;		/* Starting sample				*/
long accnum;		/* Number of user specified accuracy levels 	*/
long j;			/* Counter 					*/
char intpl[CMLEN];	/* The input Tie Point Location file name 	*/
long log_flag;		/* the flag for the 1st output file		*/
long plot_flag;		/* the plot flag 				*/
char logfile[CMLEN];	/* The image statistics logfile 		*/
char plotname[CMLEN];	/* The plot file 				*/
char prtfile[CMLEN];    /* print file name                      	*/
char filename[CMLEN];	/* print file name				*/
char outfile[3][CMLEN];	/* output file name array for complete message	*/
long print[3];          /* print option                         	*/
long non_fatal = 1;	/* nonfatal error flag				*/


/* Initialize LAS processing
--------------------------*/
c_inlas(&vblk, &std_out);

/* Get the parameters
-------------------*/
getpar(&vblk,intpl,&psout,&linereg,&sampreg,&sl,&ss,&lines,&samps,&accnum,
       pixacc, &log_flag,logfile,&plot_flag,plotname,prtfile,filename,print);

/* OPEN the input verification (tpl) file
---------------------------------------*/
acc = IREAD;
if(c_ptopen((struct PTDESC **)&in_tpl,intpl,&ptyp,&acc,(long *)&header)!=E_SUCC)
   c_errmsg("Fatal error encountered","verify2-fatal",&fatal);

/* Define units
-------------*/
if (strcmp(header.units, "") == 0)
    strcpy(header.units,"meters");

/* Get pixel size
---------------*/
if (psout == 0.0)
   {
   if (header.proj_valid == TRUE)
      {
      psout = header.pdist[0];
      }
   else
     {
     c_errmsg("Pixel size not specified","verify2-pixsiz",&non_fatal);
     c_errmsg("Fatal error encountered","verify2-fatal",&fatal);
     }
   }
  

/* Get number of lines and samples
--------------------------------*/
if (lines == 0)
   {
   if (linereg != 1)
      {
      if (header.proj_valid != TRUE)
         {
         c_errmsg("Tie point header projection information invalid",
                  "verify2-header",&non_fatal);
         c_errmsg("Fatal error encountered","verify2-fatal",&fatal);
         }
      else
         {
         corners[1] = ((header.corners[2] + header.corners[6])/2.0);
         corners[0] = ((header.corners[0] + header.corners[4])/2.0);
         lines = (long) fabs((corners[1] - corners[0])/ psout);
         }
      }
   else
      lines = MAX_LINES; /* if the corner coordinates do not exist then total
			    lines cannot be calculated. A region must be defined
			    using lines. May need to increase MAX_LINES in the
			    future */ 
   }
if (samps == 0)
   {
   if (sampreg != 1)
      {
      if (header.proj_valid != TRUE)
         {
         c_errmsg("Tie point header projection information invalid",
                  "verify2-header",&non_fatal);
         c_errmsg("Fatal error encountered","verify2-fatal",&fatal);
         }
      else
         {
         corners[1] = ((header.corners[5] + header.corners[7])/2.0);
         corners[0] = ((header.corners[1] + header.corners[3])/2.0);
         samps = (long) fabs((corners[1] - corners[0])/ psout);
         }
      }
   else
      samps = MAX_SAMPS; /* if the corner coordinates do not exist then total
                            samples cannot be calculated. A region must be
               		    defined using samples. May need to increase 
			    MAX_SAMPS in the future */
   }

/* Malloc pointref and pointsrch arrays
-------------------------------------*/
pointref =(struct POINT *)malloc((sizeof(struct POINT)*MAX_TIE_POINTS));
pointsrch =(struct POINT *)malloc((sizeof(struct POINT)*MAX_TIE_POINTS));

/* Read each active verification point from the TPL file
------------------------------------------------------*/
maxtie = MAX_TIE_POINTS;
pointref1 = pointref;
pointsrch1 = pointsrch;
for (npts=0;;)
  {
  status = c_ptio((struct PTDESC **)&in_tpl, (long *)&tpldata);
  if(status==E_EOF) 
     break;
  if(status==E_FAIL)
     c_errmsg("Fatal error encountered","verify2-fatal",&fatal);
  if ((tpldata.active == 0) || (tpldata.active == 1) || (tpldata.active > 9))
     {
     /* if need more space, realloc
     ----------------------------*/
     if (npts + 1 == maxtie)
        {
        maxtie += MAX_TIE_POINTS;
        pointref1 = (struct POINT *) realloc(pointref1,
                                  (maxtie * sizeof(struct POINT)));
        pointref = pointref1;
        pointsrch1 = (struct POINT *) realloc(pointsrch1,
                                  (maxtie * sizeof(struct POINT)));
        pointsrch = pointsrch1;
        }
     pointref[npts].x = tpldata.ref_coord[0];
     pointref[npts].y = tpldata.ref_coord[1];
     strncpy(pointref[npts].ptnum, tpldata.pt_id, 21);
     pointsrch[npts].x = tpldata.sea_coord[0];
     pointsrch[npts].y = tpldata.sea_coord[1];
     strncpy(pointsrch[npts].ptnum, pointref[npts].ptnum, 21);
     ++npts;
     }
  }

c_ptclse((struct PTDESC **)&in_tpl);

/* Call function to calculate regions and region statistics
---------------------------------------------------------*/
report(intpl, logfile, prtfile, plotname, npts, psout, linereg, sampreg, 
       pixacc, pointref, pointsrch, header.units, sl,ss,lines, samps, accnum,
       log_flag, plot_flag);

/* Write the standard completion message and exit
-----------------------------------------------*/
j = 0;
if ((plot_flag == TRUE) && ( linereg == 1) && (sampreg == 1 ))
   strcpy(outfile[j++], plotname);
if (log_flag == TRUE)
   strcpy(outfile[j++], logfile);
if (print[PFILE])
   strcpy(outfile[j++],filename);

c_complt("VERIFY2", outfile, &j);
}
