/*****************************************************************************

NAME: daub4.c

PURPOSE: Apply either the forward or inverse wavelet transform to a one-
         dimensional array using the Daubechies 4 coefficients.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Taken from "Numerical Recipes in C".  See WAVEMERGE design document.

RETURN VALUES:
     WAVE_SUCCESS              successful completion
     WAVE_FAILURE              an error occurred during operation

REFERENCES:
     Numerical Recipes in C.
*****************************************************************************/

#include <stdlib.h>
#include "wavelet.h"


int daub4
(
    float aa[],            /* I/O: One-dimensional array of data to be
                                   transformed                              */
    int na,                /* I: Size of array
                                 (not counting mirror extension)            */
    int type               /* I: Flag denoting type of transform:
                                 FORWARD or REVERSE                         */
)
{
  float *wksp;             /* temporary workspace                           */

  int ii, jj;              /* loop counters                                 */
  int nh;                  /* half the size of the array                    */


  /* If the array has less than 4 elements, it is not worth applying the
     transform.  This is mostly used in image compression where the
     transform is applied recursively until the smooth array is less than
     4x4.
     --------------------------------------------------------------------*/
  if (na < 4)
      return WAVE_SUCCESS;

  /* The transform consists of two subarrays (smooth and detail) that are
     each half the size of the non-transformed array.
                            mirror              mirror
                           |-----|   na        |-----|
                           |-------------------|
          original:         x x x x x x x x x x x x x
          transformed:        s   s   s   s   s
                              d   d   d   d   d

     The transform works for any value of na >= 4.
     --------------------------------------------------------------------*/
  nh = na/2;

  /* Allocate workspace.
     -------------------*/
  if ((wksp = (float *)malloc(2*nh*sizeof(float))) == NULL)
      return WAVE_FAILURE;
  
  if (type == FORWARD)
  {
      /* Apply the wavelet transform.
         ----------------------------*/
      for (ii=0, jj=1; jj<na; jj+=2, ii++)
      {
          wksp[ii] = C0*aa[jj] + C1*aa[jj+1] + C2*aa[jj+2] + C3*aa[jj+3];
          wksp[ii+nh] = C3*aa[jj] - C2*aa[jj+1] + C1*aa[jj+2] - C0*aa[jj+3];
      }
      for (ii=0; ii<2*nh; ii++)
          aa[ii] = wksp[ii];
  }
  else
  {
      /* Apply the inverse wavelet transform.
         ------------------------------------*/
      for (ii=0, jj=0; ii<nh-1; ii++)
      {
          wksp[jj++] = C2*aa[ii] + C1*aa[ii+nh] + C0*aa[ii+1] +
                       C3*aa[ii+nh+1];
          wksp[jj++] = C3*aa[ii] - C0*aa[ii+nh] + C1*aa[ii+1] -
                       C2*aa[ii+nh+1];
      }
      for (ii=0; ii<na-3; ii++)
          aa[ii] = wksp[ii];
  }
  
  free(wksp);

  return WAVE_SUCCESS;
}
