/*****************************************************************************

NAME: dwt2.c

PURPOSE: Apply the wavelet transform to a two-dimensional array using the
         wavelet of choice.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Taken from "Numerical Recipes in C".  See WAVEMERGE design document.

RETURN VALUE:
     pointer to generated transform array

REFERENCES:
     Numerical Recipes in C.
*****************************************************************************/

#include <stdlib.h>
#include "wavelet.h"


float *dwt2                      /* Returns pointer to generated transform
                                    array.  Calling routine must ensure that
                                    memory is freed.                        */
(
    float aa[],                  /* I: Array to be transformed              */
    int nrows,                   /* I: Number of rows in array              */
    int ncols,                   /* I: Number of columns in array           */
    struct transform func        /* I: Function to be used for 1D transform */
)
{
  int lof;                /* length of the wavelet filter                   */
  int ntrows, ntcols;     /* number of rows and columns in transform array  */
  int offset, toffset;    /* indexing offsets for array and transform array */
  int irow, icol;         /* row and column loop counters                   */
  
  float *ta;              /* transformed array                              */
  float *temp_buf;        /* temporary storage for bottom boundary rows     */
  float *wksp;            /* temporary storage for 1D array                 */


  /* Get the length of the wavelet.
     ------------------------------*/
  lof = func.lof;
  
  /* Determine transform array size.  In general, the input array size
     includes a mirror boundary surrounding the actual data.  The mirror
     boundary is of size lof - 1.  The transform array will be lof - 1
     smaller than the original array.

     For one-dimensional arrays, the transform takes place in only one
     dimension, so the mirror boundary only exists in the transformable
     dimension.
   ----------------------------------------------------------------------*/
  if (nrows == 1)
      ntrows = 1;
  else if (nrows < lof)
      return NULL;
  else
      ntrows = nrows - lof + 1;
  if (ncols == 1)
      ntcols = 1;
  else if (ncols < lof)
      return NULL;
  else
      ntcols = ncols - lof + 1;

  /* Allocate storage for the transform array as well as a temporary
     array for the bottom boundary rows.
  ------------------------------------------------------------------*/
  if ((ta = (float *)malloc(ntrows*ntcols*sizeof(float))) == NULL)
      return NULL;
  if ((temp_buf = (float *)malloc((lof - 1)*ntcols*sizeof(float))) == NULL)
  {
      free(ta);
      return NULL;
  }
  if (ncols > nrows)
      wksp = (float *)malloc(ncols*sizeof(float));
  else
      wksp = (float *)malloc(nrows*sizeof(float));
  if (wksp == NULL)
  {
      free(ta);
      free(temp_buf);
      return NULL;
  }
  
  /* Convolve matrix with filters.
   -------------------------------*/

  /* If more than one column, convolve rows.
   -----------------------------------------*/
  if (ncols > 1)
  {
      /* Transform all rows except bottom boundary rows (done below).
         ------------------------------------------------------------*/
      for (irow=0, offset=0, toffset=0; irow<ntrows;
           irow++, offset+=ncols, toffset+=ntcols)
      {
          /* Copy row to workspace.
           ------------------------*/
          for (icol=0; icol<ncols; icol++)
              wksp[icol] = aa[offset+icol];

          /* Transform row.
             --------------*/
          if ((*(func.dwt1))(wksp, ntcols, FORWARD) == WAVE_FAILURE)
          {
              free(wksp);
              free(temp_buf);
              free(ta);
              return NULL;
          }

          /* Copy transformed row into transform array.
             ------------------------------------------*/
          for (icol=0; icol<ntcols; icol++)
              ta[toffset+icol] = wksp[icol];
      }

      /* Transform bottom boundary rows.
         -------------------------------*/
      for (irow=0, offset=ntrows*ncols, toffset=0; irow<lof-1;
           irow++, offset+=ncols, toffset+=ntcols)
      {
          /* Copy row to workspace.
           ------------------------*/
          for (icol=0; icol<ncols; icol++)
              wksp[icol] = aa[offset+icol];

          /* Transform row.
             --------------*/
          if ((*(func.dwt1))(wksp, ntcols, FORWARD) == WAVE_FAILURE)
          {
              free(wksp);
              free(temp_buf);
              free(ta);
              return NULL;
          }

          /* Copy transformed row back into temporary array.
             -----------------------------------------------*/
          for (icol=0; icol<ntcols; icol++)
              temp_buf[toffset+icol] = wksp[icol];
      }
  } /* end row transforms */
  else
  {
      /* Only one column, so copy original array to transform array and
         temporary array.
         --------------------------------------------------------------*/
      for (irow=0; irow<ntrows; irow++)
          ta[irow] = aa[irow];
      for (irow=0; irow<lof-1; irow++)
          temp_buf[irow] = aa[irow+ntrows];
  }
  
  /* If more than one row, convolve columns.
   -----------------------------------------*/

  if (nrows > 1)
  {
      /* Transform all columns.
         ----------------------*/
      for (icol=0; icol<ntcols; icol++)
      {
          /* Copy column to workspace from transform array and
             tempororary array.
           ---------------------------------------------------*/
          for (irow=0, toffset=icol; irow<ntrows; irow++, toffset+=ntcols)
              wksp[irow] = ta[toffset];
          for (irow=0, toffset=icol; irow<lof-1; irow++, toffset+=ntcols)
              wksp[ntrows+irow] = temp_buf[toffset];

          /* Transform column.
             -----------------*/
          if ((*(func.dwt1))(wksp, ntrows, FORWARD) == WAVE_FAILURE)
          {
              free(wksp);
              free(temp_buf);
              free(ta);
              return NULL;
          }

          /* Copy transformed column into transform array.
             ---------------------------------------------*/
          for (irow=0, toffset=icol; irow<ntrows; irow++, toffset+=ntcols)
              ta[toffset] = wksp[irow];
      }
  }

  /* Free temporary storage.
     -----------------------*/
  free(wksp);
  free(temp_buf);

  /* Return transform array.
     -----------------------*/
  return ta;
}
